package com.crunii.micro.intf.httpdemo.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;


import com.crunii.micro.intf.httpdemo.service.AccessTokenService;

import com.dcqc.uc.oauth.sdk.model.Result;
import com.dcqc.uc.oauth.sdk.util.JwtHelper;
import com.dcqc.uc.oauth.sdk.util.SM2ToolUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * @author lmlkyc
 * @date 2023-10-06 17:37
 */
@Service
public class AccessTokenServiceImpl implements AccessTokenService {
	@Autowired
	private JwtHelper jwtHelper;

	@Value("${uc.api.url}")
	private String ucApiUrl;

	@Value("${uc.api.appId}")
	private String appId;

	@Value("${uc.api.secret}")
	private String appSecret;


	@Override
	public Result getAccessToken() {
		try {
		// 将appSecret使用SM2加密
		String appSecretEncode = SM2ToolUtil.sm2Encode(jwtHelper.getPublicKey(), appSecret);
		JSONObject jsonObject = new JSONObject()
				.fluentPut("data", new JSONObject()
						.fluentPut("appId", appId)
						.fluentPut("appSecret", appSecretEncode)
				).fluentPut("requestId", StrUtil.uuid());
		String resp = HttpUtil.post(ucApiUrl+"/ykz/access_token", jsonObject.toJSONString());
		cn.hutool.json.JSONObject entries = JSONUtil.parseObj(resp);
		boolean success = entries.getBool("success");
		String message = entries.getStr("message");
		if (success) {
			cn.hutool.json.JSONObject data = entries.getJSONObject("data");
			String accessToken = data.getStr("accessToken");
			return new Result(accessToken,0,message);
		} else {
			return new Result(40045,message);
		}
	}catch (Exception e) {
			e.printStackTrace();
			return new Result(500,e.toString());
		}
	}
}