/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved. <a>http://www.crunii.com</a>
 */
package com.crunii.micro.intf.httpdemo.service.impl;

import com.crunii.micro.common.constants.LogConstants;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.http.HttpClientHolder;
import com.crunii.micro.common.spring.rest.RestBuilder;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.common.utils.NanoIdUtil;
import com.crunii.micro.intf.httpdemo.service.HttpLogService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Map;

/**
 * @author 田平 create 2020年1月6日下午1:56:54
 */
@Service
public class HttpLogServiceImpl implements HttpLogService {

	@Value("${core.addr:http://81.68.133.219:9345/service-sysmgr/TestController/testLog}")
	private String testAddress;

	/**
	 * 注意使用记录日志的RestTemplate对应的bean名称
	 * 具体参见
	 *
	 * @see com.crunii.micro.common.config.RestTemplateConfiguration
	 */
	@Resource(name = "logRestTemplate")
	private RestTemplate restTemplate;

	@Override
	public Result<String> httpLogDemo() {
		//设置日志管理信息
		Map<String, Object> busiInfo = HttpClientHolder.getBusiInfoAndClean();
		busiInfo.put(LogConstants.LOGIN_ID, 9998);
		busiInfo.put(LogConstants.LOGIN_CODE, "tianping");
		busiInfo.put(LogConstants.WORKER_NAME, "田平");
		//有时候一个业务方法会反复调用多次外围的http接口，可以这种多次调用的tradeNo一样，方便查询
		busiInfo.put(LogConstants.TRADE_NO, NanoIdUtil.randomLowerNanoId());
		busiInfo.put(LogConstants.BUSI_TYPE, "Test");
		busiInfo.put(LogConstants.BUSI_NO, "Abc123");
		//通过tradeNo|busiType|busiNo 到t_busi_log查询http调用日志
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON);
		//设置接收格式，也可以为xml MediaType.APPLICATION_XML
		headers.setAccept(Arrays.asList(MediaType.APPLICATION_JSON));
		HttpEntity<Void> requestEntity = new HttpEntity<>(headers);
		ParameterizedTypeReference<Result<String>> type = new ParameterizedTypeReference<Result<String>>() {};
		ResponseEntity<Result<String>> exchange = restTemplate.exchange(testAddress, HttpMethod.GET, requestEntity, type);
		return exchange.getBody();
	}

	public static void main(String[] args) {
		String url = "http://localhost:12000/intf/IntfHttpDemoController/httpLogDemo";
		RestTemplate restTemplate = RestBuilder.buildRestTemplate();
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON);
		headers.setAccept(Arrays.asList(MediaType.APPLICATION_JSON));
		HttpEntity<Void> requestEntity = new HttpEntity<>(headers);
		ParameterizedTypeReference<Result<String>> type = new ParameterizedTypeReference<Result<String>>() {};
		ResponseEntity<Result<String>> exchange = restTemplate.exchange(url, HttpMethod.GET, requestEntity, type);
		System.out.println(JsonUtil.getPrettyJsonString(exchange.getBody()));
	}
}
