package com.crunii.micro.intf.httpdemo.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.YkzOrganization;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.domain.repo.main.YcqOrganizationRepo;
import com.crunii.micro.intf.httpdemo.service.AccessTokenService;
import com.crunii.micro.intf.httpdemo.service.YkzOrganizationService;
import com.dcqc.uc.oauth.sdk.enums.ResultCodeEnum;
import com.dcqc.uc.oauth.sdk.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author lmlkyc
 * @date 2023-10-06 16:03
 */
@Service
@Slf4j
public class YkzOrganizationServiceImpl implements YkzOrganizationService {

	@Autowired
	private AccessTokenService accessTokenService;

	@Autowired
	private YcqOrganizationRepo ycqOrganizationRepo;

	@Autowired
	private TsmOrgRepo tsmOrgRepo;

	@Value("${uc.api.url}")
	private String ucApiUrl;

	@Value("${uc.api.defaultOrganizationCode}")
	private String defaultOrganizationCode;

	@Override public Result<List<YkzOrganization>> saveAllYkzOrganizations(String parentOrganizationCode) {
		List<YkzOrganization> allOrganizations = new ArrayList<>();
		int pageNumber = 1;
		int pageSize = 100;
		//每次获取全量
		ycqOrganizationRepo.deleteAll();
		//修改tsm_org 愉快政为state=0
		tsmOrgRepo.updateStateForDataSourcesYkz();
		Result result = fetchOrganizationsRecursively(parentOrganizationCode, allOrganizations, pageNumber, pageSize);
		if (result.getStatus().equals(ResultCodeEnum.SUCCESS.getCode())){
			return SendTsmOrg((List<YkzOrganization>)result.getData());
		}
		return result;
	}

	private Result fetchOrganizationsRecursively(String parentOrganizationCode,List<YkzOrganization> allOrganizations,int pageNumber, int pageSize) {
//		allOrganizations.clear();
		while (true) {
		String apiUrl = ucApiUrl + "/ykz/org/pageByParentOrganizationCode";
		String organizationCode = parentOrganizationCode != null ? parentOrganizationCode : defaultOrganizationCode;
			JSONObject requestBody = new JSONObject()
					.fluentPut("data", new JSONObject()
							.fluentPut("pageNumber", pageNumber)
							.fluentPut("organizationCode", organizationCode)
							.fluentPut("pageSize", pageSize)
					)
					.fluentPut("requestId", StrUtil.uuid());

			Result accessTokens = accessTokenService.getAccessToken();
			log.info("accessTokens:"+accessTokens);
			if (accessTokens.getStatus() != 0) {
				return accessTokens;
			}
			String accessToken = (String) accessTokens.getData();

			try {
				String resp = HttpRequest.post(apiUrl)
						.header("Content-Type", "application/json")
						.header("Authorization", "Bearer " + accessToken)
						.body(requestBody.toJSONString())
						.timeout(5000)
						.execute()
						.body();
				cn.hutool.json.JSONObject jsonResponse = JSONUtil.parseObj(resp);
				log.info("JSONObject参数:"+jsonResponse);
				if (jsonResponse.getBool("success")) {
					cn.hutool.json.JSONObject data = jsonResponse.getJSONObject("data");
					JSONArray organizationList = data.getJSONArray("list");

					for (int i = 0; i < organizationList.size(); i++) {
						cn.hutool.json.JSONObject orgData = organizationList.getJSONObject(i);
						YkzOrganization organization = new YkzOrganization();
						organization.setName(orgData.getStr("name"));
						organization.setOrganizationCode(orgData.getStr("organizationCode"));
						organization.setCreateTime(orgData.getLong("createTime"));
						organization.setCreditCode(orgData.getStr("creditCode"));
						organization.setDisplayOrder(orgData.getLong("displayOrder"));
						organization.setGovAddress(orgData.getStr("govAddress"));
						organization.setGovBusinessStripCodes(orgData.getStr("govBusinessStripCodes"));
						organization.setGovDivisionCode(orgData.getStr("govDivisionCode"));
						organization.setGovInstitutionLevelCode(orgData.getStr("govInstitutionLevelCode"));
						organization.setGovShortName(orgData.getStr("govShortName"));
						organization.setOrganizationId(orgData.getLong("id"));
						organization.setIsDeleted(orgData.getInt("isDeleted"));
						organization.setIsEnable(orgData.getInt("isEnable"));
						organization.setOrgType(orgData.getStr("orgType"));
						organization.setParentId(orgData.getLong("parentId"));
						organization.setParentOrganizationCode(orgData.getStr("parentOrganizationCode"));
						organization.setPrincipal(orgData.getStr("principal"));
						organization.setRemark(orgData.getStr("remark"));
						organization.setUpdateTime(orgData.getLong("updateTime"));

						allOrganizations.add(organization);

						String childOrganizationCode = orgData.getStr("organizationCode");
						log.info("执行第一页递归:"+childOrganizationCode);
						Result<List<YkzOrganization>> result = fetchOrganizationsRecursively(childOrganizationCode, allOrganizations, 1, pageSize);
						if (result.getStatus() != 200) {
							return result; // 如果递归调用失败，返回失败结果
						}
					}
					int totalPage = data.getInt("totalPage");
					log.info("totalPage:"+totalPage);
					log.info("total:"+data.getInt("total"));
					if (pageNumber >= totalPage) {
						break; // 没有下一页了，退出循环
					}else {
						pageNumber++;
						log.info("执行第n页递归:"+pageNumber);
						return fetchOrganizationsRecursively(organizationCode, allOrganizations, pageNumber, pageSize);
					}
				} else {
					String errorMessage = jsonResponse.getStr("message");
					return new Result<>(ResultCodeEnum.FAIL, ResultCodeEnum.FAIL.getCode(), errorMessage);
				}
			} catch (Exception e) {
				e.printStackTrace();
				return new Result<>(500, e.toString());
			}
		}
		List<YkzOrganization> ykzOrganizations = ycqOrganizationRepo.saveAll(allOrganizations);
		log.info("YkzOrganization数据集合:"+ykzOrganizations);

		return new Result<>(ykzOrganizations,ResultCodeEnum.SUCCESS);
	}

	private Result SendTsmOrg(List<YkzOrganization> ykzOrganizations) {
		List<TsmOrg> tsmOrgList = new ArrayList<>();

		for (YkzOrganization ykzOrganization : ykzOrganizations) {
			TsmOrg tsmOrg = new TsmOrg();
			//orgid
//			tsmOrg.setOrgId();
			//组织编码-政务钉钉组织机构code
			tsmOrg.setOrgCode(ykzOrganization.getOrganizationCode());
			//组织名称-机构简称
			tsmOrg.setOrgName(ykzOrganization.getGovShortName());
			//组织全称-机构全称
			tsmOrg.setOrgFullName(ykzOrganization.getName());
			//组织描述-备注
			tsmOrg.setOrgDesc(ykzOrganization.getRemark());
			//父组织ID,根组织id为0-机构父级id
			tsmOrg.setParentOrgId(ykzOrganization.getParentId());
			//父组织编码-父组织机构code
			tsmOrg.setParentOrgCode(ykzOrganization.getParentOrganizationCode());
			//创建时间
			tsmOrg.setCreateTime(new Date());
			//更新时间
//			tsmOrg.setUpdateTime(new Date());
			//排序-同级排序字段
			Long longValue = ykzOrganization.getDisplayOrder();
			if (longValue >= Integer.MIN_VALUE && longValue <= Integer.MAX_VALUE) {
				Integer intValue = longValue.intValue();
				tsmOrg.setSortId(intValue.longValue());
			}
			//0无效，1有效-是否启用
			if (ykzOrganization.getIsDeleted() != null) {
				if (ykzOrganization.getIsDeleted() == 0) {
					tsmOrg.setState(ykzOrganization.getIsEnable());
				} else {
					tsmOrg.setState(0);
				}
			}
				//组织管理员
//			tsmOrg.setManagePeople();
				//组织等级-机构/单位级别
				tsmOrg.setOrgLevel(ykzOrganization.getGovInstitutionLevelCode());
				//数据来源
				tsmOrg.setDataSources("愉快政");
				//所属区域-行政区划Code
				tsmOrg.setRegion(ykzOrganization.getGovDivisionCode());
				//所属行业类别
//			tsmOrg.setIndustryCategory();
				//组织类别树
//			tsmOrg.setOrgCategoryTree();
				//所属行业类别(团组织)
//			tsmOrg.setOcylIndustryCategory();
				//组织类别(团组织)
//			tsmOrg.setOcylOrgCategory();
				//是否乡镇团委(团组织)
//			tsmOrg.setOcylIsXztw();
				//联系电话(团组织)
//			tsmOrg.setTelephone();
				tsmOrgList.add(tsmOrg);
			}
		List<TsmOrg> tsmOrgs = tsmOrgRepo.saveAll(tsmOrgList);
		log.info("tsmOrg筛选前数量:"+tsmOrgs.size());
		Map<String, TsmOrg> orgCodeToTsmOrg = tsmOrgs.stream().collect(Collectors.toMap(TsmOrg::getOrgCode, org -> org));
		tsmOrgs.forEach(org -> {
			String parentOrgCode = org.getParentOrgCode();
			if (orgCodeToTsmOrg.containsKey(parentOrgCode)){
				org.setParentOrgId(orgCodeToTsmOrg.get(parentOrgCode).getOrgId());
			}
		});

		List<TsmOrg> tsmOrgs1 = tsmOrgRepo.saveAll(tsmOrgs);
		log.info("tsmOrg筛选后数量:"+tsmOrgs1.size());
		return new Result<>(tsmOrgs1,ResultCodeEnum.SUCCESS);
	}
}
