package com.crunii.common.excel.convert;

import cn.hutool.core.annotation.AnnotationUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.converters.Converter;
import com.alibaba.excel.enums.CellDataTypeEnum;
import com.alibaba.excel.metadata.GlobalConfiguration;
import com.alibaba.excel.metadata.data.WriteCellData;
import com.alibaba.excel.metadata.property.ExcelContentProperty;
import org.springframework.format.annotation.DateTimeFormat;

import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @Description: excel导出时间格式转换
 *（根据DateTimeFormat注解设置的pattern，默认yyyy-MM-dd HH:mm:ss）
 *
 * @Author: shanbin
 * @Date: 2023/5/10 14:47
 */
public class ExcelDateConverter implements Converter<Date> {

    private static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    @Override
    public Class<?> supportJavaTypeKey() {
        return Converter.super.supportJavaTypeKey();
    }

    @Override
    public CellDataTypeEnum supportExcelTypeKey() {
        return Converter.super.supportExcelTypeKey();
    }

    @Override
    public WriteCellData<?> convertToExcelData(Date value, ExcelContentProperty contentProperty, GlobalConfiguration globalConfiguration) throws Exception {
        DateTimeFormat anno = getAnnotation(contentProperty.getField());
        String pattern = anno.pattern();
        if(ObjectUtil.isEmpty(anno.pattern())){
            pattern = YYYY_MM_DD_HH_MM_SS;
        }
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        String dateValue = sdf.format(value);
        return new WriteCellData<>(dateValue);
    }

    private DateTimeFormat getAnnotation(Field field) {
        return AnnotationUtil.getAnnotation(field, DateTimeFormat.class);
    }
}
