package com.crunii.micro.service.appsecond.api.bo;

import cn.hutool.core.lang.RegexPool;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;


import javax.validation.constraints.*;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author denggl
 * date 2023/12/11
 */
@ApiModel(value = "app团队入住青创空间实体")
@Data
public class AppCqzdTeamStayBo {

    @ApiModelProperty(value = "主键")
    private Long id ;

    @ApiModelProperty(value = "营业执照(0:没有、1:有)")
    @NotNull(message = "请填写是否有营业执照")
    private Integer businessLicense ;
    /** 申请人姓名 */
    @ApiModelProperty(value = "申请人姓名")
    @NotBlank(message="申请人姓名必填")
    @Size(max=10)
    private String applicantName ;
    /** 性别(男、女) */
    @ApiModelProperty(value = "性别(男、女)")
    @NotBlank(message = "性别必填")
    private String sex ;
    /** 民族 */
    @ApiModelProperty(value = "民族")
    @NotBlank(message = "民族必填")
    private String nation ;
    /** 身份证号 */
    @ApiModelProperty(value = "身份证号")
    @Pattern(regexp="^$|" + RegexPool.CITIZEN_ID,message="身份证号格式错误")
    @NotBlank(message = "身份证号必填")
    private String idCode;

    @ApiModelProperty(value = "邮箱 ")
    @Pattern(regexp = "^$|" + RegexPool.EMAIL, message = "邮箱格式错误")
    private String mailbox ;
    /** 年龄 */
  /*  @NotNull(message = "年龄必填")
    @ApiModelProperty(value = "年龄")
    private Integer age;*/
    /** 学历 */
    @ApiModelProperty(value = "学历")
    @NotBlank(message = "学历必填")
    private String formalSchooling ;
    /** 政治面貌 */
    @ApiModelProperty(value = "政治面貌")
    @NotBlank(message = "政治面貌必填")
    private String politicalStatus ;
    /** 电话 */
    @ApiModelProperty(value = "电话")
    @Pattern(regexp = "^(13[0-9]|14[579]|15[0-3,5-9]|16[6]|17[0135678]|18[0-9]|19[89])\\d{8}$",message = "手机号码有误！")
    @NotBlank(message = "电话必填")
    private String phone ;
    /** 行业领域 */
    @ApiModelProperty(value = "行业领域")
    @NotEmpty(message = "行业领域必填")
    private List<String> industryFields ;
    /** 企业名称 */
    @ApiModelProperty(value = "企业名称")
    private String companyName ;
    /** 统一社会信用代码 */
    @ApiModelProperty(value = "统一社会信用代码")
    private String socialCode ;
    /** 创业经历 */
    @ApiModelProperty(value = "创业经历")
    @Size(max = 500,message = "创业经历限制输入500个字符!")
    private String entrepreneurialExperience ;
    /** 团队简介 */
    @ApiModelProperty(value = "团队简介")
    @Size(max = 500,message = "团队简介限制输入500个字符!")
    private String teamIntroduction ;
    /** 工位需求(0:否、1:是) */
    @NotNull(message = "工位需求必填")
    @ApiModelProperty(value = "工位需求(0:否、1:是)")
    private Integer workNeed ;
    /** 办公室需求(0:否、1:是) */
    @ApiModelProperty(value = "办公室需求(0:否、1:是)")
    @NotNull(message = "办公室需求必填")
    private Integer officeNeed ;

    /** 入住青春空间id */
    @ApiModelProperty(value = "入住青春空间id")
    @NotNull(message = "入住青春空间必填")
    private Long spaceId ;


}
