package com.crunii.micro.service.appsecond.api.dto.youthsapce;

import com.alibaba.excel.annotation.ExcelProperty;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serializable;
import javax.validation.constraints.NotBlank;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;

/**
 * @Author thw
 * @date 2023/12/14
 * @description 团队导入导出DTO
 */
@Data
@NoArgsConstructor
@ApiModel(value = "团队导入导出DTO")
public class SpacePassTeamCUExportDTO implements Serializable {

  private static final Long serialVersionUID = 2L;

  /**
   * 营业执照(0:没有、1:有)
   */
  @NotBlank(message = "营业执照不能为空", groups = { AddGroup.class, EditGroup.class })
  @ExcelProperty(value = "营业执照名称(没有、有)")
  private String businessLicenseName;

  /**
   * 企业名称
   */
  @ExcelProperty(value = "企业名称")
  @ApiModelProperty(value = "企业名称")
  private String companyName;

  /**
   * 统一社会信用代码
   */
  @ExcelProperty(value = "统一社会信用代码")
  @ApiModelProperty(value = "统一社会信用代码")
  private String socialCode;

  /**
   * 申请人姓名
   */
  @ExcelProperty(value = "申请人姓名")
  @NotBlank(message = "姓名不能为空", groups = { AddGroup.class, EditGroup.class })
  @Length(message = "姓名字符长度不能超过10", max = 10 )
  @ApiModelProperty(value = "申请人姓名", required = true)
  private String applicantName;

  /**
   * 性别(男、女)
   */
  @NotBlank(message = "性别不能为空", groups = { AddGroup.class, EditGroup.class })
  @ExcelProperty(value = "性别(男、女)")
  private String sexName;

  /**
   * 民族
   */
  @NotBlank(message = "民族不能为空", groups = { AddGroup.class, EditGroup.class })
  @ExcelProperty(value = "民族")
  private String nationName;

  /**
   * 身份证号
   */
  @ExcelProperty(value = "身份证号")
  @NotBlank(message = "身份证号不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "身份证号", required = true)
  private String idCode;

  /**
   * 学历
   */
  @NotBlank(message = "学历不能为空", groups = { AddGroup.class, EditGroup.class })
  @ExcelProperty(value = "学历")
  private String formalSchoolingName;

  /**
   * 政治面貌
   */
  @NotBlank(message = "政治面貌不能为空", groups = { AddGroup.class, EditGroup.class })
  @ExcelProperty(value = "政治面貌")
  private String politicalStatusName;

  /**
   * 联系人电话
   */
  @ExcelProperty(value = "联系人电话")
  @NotBlank(message = "联系人电话不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "联系人电话", required = true)
  private String phone;

  /**
   * 行业领域
   */
  @ExcelProperty(value = "行业领域")
  @NotBlank(message = "行业领域不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "行业领域", required = true)
  private String industryField;

  /**
   * 电子邮箱
   */
  @ExcelProperty(value = "电子邮箱")
  @ApiModelProperty(value = "电子邮箱")
  private String mailbox;

  /**
   * 创业经历
   */
  @ExcelProperty(value = "创业经历")
  @ApiModelProperty(value = "创业经历")
  private String entrepreneurialExperience;

  /**
   * 团队简介
   */
  @ExcelProperty(value = "团队简介")
  @ApiModelProperty(value = "团队简介")
  private String teamIntroduction;

  /**
   * 工位需求(0:否、1:是)
   */
  @NotBlank(message = "工位需求不能为空", groups = { AddGroup.class, EditGroup.class })
  @ExcelProperty(value = "工位需求(否、是)")
  private String workNeedName;

  /**
   * 办公室需求(0:否、1:是)
   */
  @NotBlank(message = "办公室需求不能为空", groups = { AddGroup.class, EditGroup.class })
  @ExcelProperty(value = "办公室需求(否、是)")
  private String officeNeedName;

}
