package com.crunii.micro.service.appsecond.api.dto.youthsapce;

import com.alibaba.excel.annotation.ExcelProperty;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serializable;
import java.util.List;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

/**
 * @Author thw
 * @date 2023/12/13
 * @description
 */
@Data
@ApiModel(value = "已入住空间团队新增DTO")
public class SpacePassTeamCUReqDTO implements Serializable {

  private static final Long serialVersionUID = 2L;

  /**
   * 团队id
   */
  @NotBlank(message = "团队id不能为空", groups = { EditGroup.class })
  @ApiModelProperty(value = "团队id")
  private String id;

  /**
   * 空间id
   */
  @NotBlank(message = "空间id不能为空", groups = { AddGroup.class, EditGroup.class  })
  @ApiModelProperty(value = "空间id", required = true)
  private String spaceId;

  /**
   * 营业执照(0:无、1:有)
   */
  @NotNull(message = "营业执照不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "营业执照(0:无、1:有)" , required = true)
  private Integer businessLicense;

  /**
   * 企业名称
   */
  @ExcelProperty(value = "企业名称")
  @ApiModelProperty(value = "企业名称")
  private String companyName;

  /**
   * 统一社会信用代码
   */
  @ExcelProperty(value = "统一社会信用代码")
  @ApiModelProperty(value = "统一社会信用代码")
  private String socialCode;

  /**
   * 申请人姓名
   */
  @NotBlank(message = "姓名不能为空", groups = { AddGroup.class, EditGroup.class })
  @Length(message = "姓名字符长度不能超过10", max = 10 )
  @ApiModelProperty(value = "申请人姓名", required = true)
  private String applicantName;

  /**
   * 性别(男、女)
   */
  @NotBlank(message = "性别不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "性别(男、女)", required = true)
  private String sex;

  /**
   * 民族
   */
  @NotBlank(message = "民族不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "民族", required = true)
  private String nation;

  /**
   * 身份证号
   */
  @NotBlank(message = "身份证号不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "身份证号", required = true)
  private String idCode;

  /**
   * 学历
   */
  @NotBlank(message = "学历不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "学历", required = true)
  private String formalSchooling;

  /**
   * 政治面貌
   */
  @NotBlank(message = "政治面貌不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "政治面貌", required = true)
  private String politicalStatus;

  /**
   * 联系人电话
   */
  @NotBlank(message = "联系人电话不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "联系人电话", required = true)
  private String phone;

  /**
   * 行业领域
   */
  @NotNull(message = "行业领域不能为空", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "行业领域", required = true)
  private List<String> industryField;

  /**
   * 电子邮箱
   */
  @ApiModelProperty(value = "电子邮箱")
  private String mailbox;

  /**
   * 创业经历
   */
  @ApiModelProperty(value = "创业经历")
  private String entrepreneurialExperience;

  /**
   * 团队简介
   */
  @ApiModelProperty(value = "团队简介")
  private String teamIntroduction;

  /**
   * 工位需求(0:否、1:是)
   */
//  @NotNull(message = "工位需求(0:否、1:是)", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "工位需求(0:否、1:是)")
  private Integer workNeed;


  /**
   * 办公室需求(0:否、1:是)
   */
//  @NotNull(message = "办公室需求(0:否、1:是)", groups = { AddGroup.class, EditGroup.class })
  @ApiModelProperty(value = "办公室需求(0:否、1:是)")
  private Integer officeNeed;


}
