package com.crunii.micro.service.appsecond.api.dto.youthsapce;

import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serializable;
import java.util.List;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import lombok.Data;

/**
 * @Author thw
 * @date 2023/12/12
 * @description 青创空间发布请求DTO
 */
@Data
@ApiModel(value = "青创空间发布请求DTO")
public class YouthSpacCReqDTO implements Serializable {

  private static final Long serialVersionUID = 2L;

  /** 空间id */
  @NotBlank(message = "空间id不能为空", groups = {EditGroup.class })
  @ApiModelProperty(value = "空间id")
  private String id;

  /** 青创空间名称 */
  @NotBlank(message = "青创空间名称不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "青创空间名称", required = true)
  private String spaceName;

  /** 所在区域 */
  @NotBlank(message = "所在区域不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "所在区域code", required = true)
  private String location;
  @ApiModelProperty(name = "所在区域名称")
  private String locationName ;

  /** 联系人 */
  @NotBlank(message = "联系人不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "联系人", required = true)
  private String contactsName;

  /** 联系电话 */
  @NotBlank(message = "联系电话不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "联系电话", required = true)
  private String phone;

  /** 共享配置*/
  @NotNull(message = "共享配置不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "共享配置")
  private List<String> sharingSettings;

  /** 侧重领域 */
  @NotNull(message = "侧重领域不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "侧重领域")
  private List<String> focusOnAreas;

  /** 联系邮箱 */
  @ApiModelProperty(value = "联系邮箱")
  private String contactMail;

  /** 优惠政策标志(0:无、1:有) */
  @NotNull(message = "优惠政策标志不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "优惠政策标志(0:无、1:有)" , required = true)
  private Integer preferentialPolicySign;
  /** 优惠政策 */
  @ApiModelProperty(value = "优惠政策")
  private String preferentialPolicy;

  /** 入驻要求标志(0:无、1:有) */
  @NotNull(message = "入驻要求标志不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "入驻要求标志(0:无、1:有)" , required = true)
  private Integer settledRequiresSign;
  /** 入驻要求 */
  @ApiModelProperty(value = "入驻要求")
  private String settledRequires;

  /** 青创空间层级标签(no:无、city:市级、country:国家) */
  @NotBlank(message = "青创空间层级标签不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "青创空间层级标签(no:无、city:市级、country:国家)")
  private String hierarchyLabel;

  /** 层级附件(pdf) */
  @ApiModelProperty(value = "层级附件(pdf)")
  private List<YouthSpaceFileDTO> hierarchyFile;

  /** 详细地址 */
//  @NotBlank(message = "详细地址不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "详细地址" , required = true)
  private String fullAddress;

  /** 经度 */
//  @NotBlank(message = "经度不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "经度", required = true)
  private String longitude;

  /** 维度 */
//  @NotBlank(message = "维度不能为空", groups = {AddGroup.class ,EditGroup.class })
  @ApiModelProperty(value = "维度", required = true)
  private String dimension;

  /** 总工位数 */
  @ApiModelProperty(value = "总工位数")
  private Integer totalWorkers;

  /** 可申请工位数 */
  @ApiModelProperty(value = "可申请工位数")
  private Integer applyWorkers;

  /** 工位单价 */
  @ApiModelProperty(value = "工位单价")
  private YouthSpacePriceReqDTO workerPrice;

  /** 总面积数 */
  @ApiModelProperty(value = "总面积数")
  private String totalArea;

  /** 可申请面积数 */
  @ApiModelProperty(value = "可申请面积数")
  private String applyArea;

  /** 面积单价 */
  @ApiModelProperty(value = "面积单价")
  private YouthSpacePriceReqDTO areaPrice;

  /** 简介 */
  @ApiModelProperty(value = "简介")
  private String introduction;

  /** 宣传展示 */
  @ApiModelProperty(value = "图片文件")
  private List<YouthSpaceFileDTO> imageFiles;
  @ApiModelProperty(value = "视频文件")
  private List<YouthSpaceFileDTO> videoFiles;

}
