package com.crunii.micro.domain.repo.main;

import com.crunii.micro.domain.common.BaseRepo;
import com.crunii.micro.domain.entity.main.AppContentManage;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.util.List;

/**
 * 【请填写功能名称】Mapper接口
 *
 * @author liuchuan
 * @date 2023-12-06
 */
@Repository
public interface AppContentManageRepo extends BaseRepo<AppContentManage, Long> {
    
    /**
     * 根据内容id更新收藏数
     * @author leixb
     * @since 2023-12-09 15:09
     * @param count 加减
     * @param contentManageId 主键id
     */
    @Transactional(rollbackOn = Exception.class)
    @Modifying
    @Query(value = "update app_content_manage set collect_count = CASE WHEN collect_count + ?1 <= 0 THEN 0 ELSE collect_count + ?1 END  where content_manage_id = ?2", nativeQuery = true)
    void updateCollectCountById(Integer count, Long contentManageId);
    
    /**
     * 根据内容id更新点赞数
     * @author leixb
     * @since 2023-12-09 15:09
     * @param count 加减
     * @param contentManageId 主键id
     */
    @Transactional(rollbackOn = Exception.class)
    @Modifying
    @Query(value = "update app_content_manage set like_count = CASE WHEN like_count + ?1 <= 0 THEN 0 ELSE like_count + ?1 END  where content_manage_id = ?2", nativeQuery = true)
    void updateLikeCountById(Integer count, Long contentManageId);
    
    
    /**
     * 根据内容id更新收藏数
     * @author leixb
     * @since 2023-12-09 15:09
     * @param count 加减
     * @param contentManageIds 主键id集合
     */
    @Transactional(rollbackOn = Exception.class)
    @Modifying
    @Query(value = "update app_content_manage set collect_count = CASE WHEN collect_count + ?1 <= 0 THEN 0 ELSE collect_count + ?1 END  where content_manage_id in ?2", nativeQuery = true)
    void updateCollectCountByIds(Integer count, List<Long> contentManageIds);

}
