package com.crunii.micro.domain.repo.main;

import com.crunii.micro.domain.common.BaseRepo;
import com.crunii.micro.domain.entity.main.AppUserCollect;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.util.List;

/**
 * app收藏Mapper接口
 *
 * @author liuchuan
 * @date 2023-12-06
 */
@Repository
public interface AppUserCollectRepo extends BaseRepo<AppUserCollect, Long> {
    
    /**
     * 根据内容ID和用户ID查询收藏对象列表
     * @param contentId 内容ID
     * @param userId    用户ID
     * @return 收藏对象列表
     */
    List<AppUserCollect> findByContentIdAndUserId(Long contentId, Long userId);
    
    
    /**
     * 根据用户ID查询浏览记录数量
     * @param loginId   用户ID
     * @return 浏览记录数量
     */
    @Query(value = "select count(*) from app_user_collect a join app_content_manage b on a.content_id = b.content_manage_id " +
        "where user_id = ? and a.del_flag = 0 and b.state = 1", nativeQuery = true)
    Integer countByUserId(Long loginId);
    
    /**
     * 根据用户ID和内容id取消收藏
     * @param userId    用户ID
     * @param contentIds 内容ID列表
     * @return 删除数量
     */
    @Transactional(rollbackOn = Exception.class)
    @Modifying
    @Query(value = "update AppUserCollect set delFlag = 1 where userId = ?1 and contentId in ?2")
    Integer updateDelFlagByUserIdAndContentIds(Long userId, List<Long> contentIds);
}
