package com.crunii.micro.domain.repo.main;

import com.crunii.micro.domain.entity.main.AppUserView;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import com.crunii.micro.domain.common.BaseRepo;

import javax.transaction.Transactional;
import java.util.List;

/**
 * app浏览记录Mapper接口
 *
 * @author liuchuan
 * @date 2023-12-06
 */
@Repository
public interface AppUserViewRepo extends BaseRepo<AppUserView, Long> {
    
    /**
     * 根据内容ID和用户ID查询浏览记录
     * @param contentId 内容ID
     * @param userId    用户ID
     * @return 点赞对象列表
     */
    List<AppUserView> findByContentIdAndUserId(Long contentId, Long userId);
    
    /**
     * 根据用户ID清空浏览记录
     * @param userId    用户ID
     * @return 删除数量
     */
    @Transactional(rollbackOn = Exception.class)
    @Modifying
    @Query(value = "update AppUserView set delFlag = 1 where userId = ?1")
    Integer updateByUserId(Long userId);
    
    /**
     * 根据用户ID查询浏览记录数量
     * @param loginId   用户ID
     * @return 浏览记录数量
     */
    @Query(value = "select count(*) from app_user_view a join app_content_manage b on a.content_id = b.content_manage_id " +
        "where user_id = ? and a.del_flag = 0 and b.state = 1", nativeQuery = true)
    Integer countByUserId(Long loginId);
}
