package com.crunii.micro.service.appsecond.controller;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ContextCreateDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ContextManageDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ToTopDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.YouthAuditDto;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.AuditFlowVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.ContextManageDetailVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.ContextManageVo;
import com.crunii.micro.service.appsecond.service.IAppContentManageService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * 【请填写功能名称】Controller
 * 前端访问路由地址为:/app/manage
 * @author liuchuan
 * @date 2023-12-06
 */
@Validated
@Api(value = "【青年热点】控制器", tags = {"【青年热点】管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/AppContentManageController")
public class AppContentManageController {

    private final IAppContentManageService iAppContentManageService;

    @ApiOperation("内容新增/修改-草稿")
    @PostMapping("/createContextDraft")
    public Result<String> createContextDraft(@RequestBody ContextCreateDto dto, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.createContextDraft(dto,worker));
    }

    @ApiOperation("内容新增-发布")
    @PostMapping("/createContext")
    public Result<String> createContext(@Validated @RequestBody ContextCreateDto dto, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.createContext(dto,worker));
    }

    @ApiOperation("分页查询内容管理界面")
    @PostMapping("/pageContextManage")
    public Result<PageRsp<ContextManageVo>> pageContextManage(@RequestBody PageReq<ContextManageDto> req, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.pageContextManage(req,worker));
    }

    @ApiOperation("查询详情")
    @PostMapping("/findContextManageDetail")
    public Result<ContextManageDetailVo> findContextManageDetail(@RequestParam("contentManageId") String contentManageId, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.findContextManageDetail(Long.parseLong(contentManageId), worker));
    }

    @ApiOperation("批量删除（更改为下架）")
    @PostMapping("/batchDelContext")
    public Result<String> batchDelContext(@RequestBody List<Long> contentManageIds, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.delContext(contentManageIds,worker));
    }

    @ApiOperation("批量删除")
    @PostMapping("/batchRealDelContext")
    public Result<String> batchRealDelContext(@RequestBody List<Long> contentManageIds, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.delRealContext(contentManageIds,worker));
    }

    @ApiOperation("批量发布（提交审核）")
    @PostMapping("/batchReleaseContext")
    public Result<String> batchReleaseContext(@RequestBody List<Long> contentManageIds, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.updateReleaseContext(contentManageIds,worker));
    }

    @ApiOperation("es同步接口")
    @PostMapping("/synEsData")
    public Result<String> synEsData(@CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.synEsData(worker));
    }

    @ApiOperation("置顶")
    @PostMapping("/toTop")
    public Result<String> toTop(@RequestBody ToTopDto toTopDto, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.updateTop(toTopDto,worker));
    }

    @ApiOperation("审核")
    @PostMapping("/submitAudit")
    public Result<String> submitAudit(@Validated @RequestBody YouthAuditDto dto, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.updateAuditState(dto,worker));
    }

    @ApiOperation("查询审核流程")
    @PostMapping("/queryAuditFlow")
    public Result<List<AuditFlowVo>> queryAuditFlow(@RequestParam("contentManageId") Long contentManageId, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppContentManageService.queryAuditFlow(contentManageId,worker));
    }
}
