package com.crunii.micro.service.appsecond.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.bo.AppCqzdTeamStayBo;
import com.crunii.micro.service.appsecond.api.dto.AppCqzdSpaceContentDto;
import com.crunii.micro.service.appsecond.api.vo.*;
import com.crunii.micro.service.appsecond.service.AppCqzdSpaceContentService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.*;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author denggl
 * date 2023/12/11
 */

@Validated
@Api(value = "青创空间app",tags = { "青创空间app端相关" })
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/AppCqzdSpaceContentController")
public class AppCqzdSpaceContentController {

    private final AppCqzdSpaceContentService appCqzdSpaceContentService;

    @ApiOperation("中间介绍信息")
    @GetMapping("/introduction")
    public Result<AppCqzdSpaceIntroductionVo>  introduction( ){
        return Result.ofSuccess(appCqzdSpaceContentService.introduction());
    }
    @ApiOperation("app空间列表查询")
    @PostMapping("/pageAppCqzdSpaceContent")
    public Result<PageRsp<AppCqzdSpaceContentVo>>  pageAppCqzdSpaceContent(@RequestBody PageReq<AppCqzdSpaceContentDto> req, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(appCqzdSpaceContentService.pageAppCqzdSpaceContent(req,worker));
    }
    @ApiOperation("空间详情")
    @GetMapping("/cqzdSpaceContentdetails")
    public Result<AppCqzdSpaceContentVo>  cqzdSpaceContentdetails(@ApiParam("spaceId 青创空间id")
                                                                      @NotNull(message = "青创空间spaceId不能为空")
                                                                      @RequestParam("spaceId")Long spaceId,@CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(appCqzdSpaceContentService.cqzdSpaceContentdetails(spaceId,worker));
    }
    @ApiOperation("返回填充数据")
    @GetMapping("/getBaseInfo")
    public Result< AppCqzdTeamStayVo> getBaseInfo(@CurrWorker @ApiIgnore Worker worker ){
         return Result.ofSuccess(appCqzdSpaceContentService.getBaseInfo(null,worker));
    }

    @ApiOperation("我的申请详情")
    @GetMapping("/getTeamStay")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "teamId", value = "团队申请空间id", required = true)
     })
    public Result< AppCqzdTeamStayVo> getTeamStay(Long  teamId, @CurrWorker @ApiIgnore Worker worker ){

        return Result.ofSuccess(appCqzdSpaceContentService.getTeamStay(teamId,worker));
    }

    @ApiOperation("保存申请")
    @PostMapping("/saveApplycCqzdSpace")
    public  Result<String>  saveApplycCqzdSpace(@RequestBody @Validated
                                                  AppCqzdTeamStayBo cqzdTeamStayBo,@CurrWorker @ApiIgnore Worker worker){

        appCqzdSpaceContentService.saveApplycCqzdSpace(cqzdTeamStayBo,worker);
        return Result.ofSuccess();
    }



    @ApiOperation("编辑申请")
    @PostMapping("/updateApplycCqzdSpace")
    public  Result<String> updateApplycCqzdSpace(@RequestBody @Validated
                                                    AppCqzdTeamStayBo cqzdTeamStayBo,@CurrWorker @ApiIgnore Worker worker){

         appCqzdSpaceContentService.updateApplycCqzdSpace(cqzdTeamStayBo,worker);
        return Result.ofSuccess();
    }


    @ApiOperation("撤销申请")
    @GetMapping("/revoke")
    public Result<String>    revoke( @ApiParam("teamId 入驻id")
                              @NotNull(message = "入驻id不能为空")
                              @RequestParam("teamId")Long teamId,@CurrWorker @ApiIgnore Worker worker ){

        appCqzdSpaceContentService.revoke(teamId,worker);
        return Result.ofSuccess();
    }
    @ApiOperation("申请列表")
    @GetMapping("/applyList")
    public  Result<AppSpaceTeamApplyVo> applyList(@CurrWorker @ApiIgnore Worker worker ){
        return  Result.ofSuccess(appCqzdSpaceContentService.applyList(worker));
    }

    @ApiOperation("申请列表-申请进度")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "teamId", value = "团队申请id", required = true),
            @ApiImplicitParam(name = "spaceId", value = "空间id", required = true),
    })
    @GetMapping("/schedule")
    public Result<List<QczdTeamStayschedule>> schedule(Long teamId, Long spaceId) {
        return  Result.ofSuccess(appCqzdSpaceContentService.schedule(teamId,spaceId));
    }
}
