package com.crunii.micro.service.appsecond.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.service.appsecond.api.bo.AppDiscoverPositionBo;
import com.crunii.micro.service.appsecond.api.bo.AppQueryPositionBo;
import com.crunii.micro.service.appsecond.api.vo.AppDiscoverListVo;
import com.crunii.micro.service.appsecond.api.vo.AppDiscoverPositionVo;
import com.crunii.micro.service.appsecond.service.IAppDiscoverPositionService;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSort;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author lyx
 * @date 2023/12/7
 */
@Validated
@Api(value = "青创阵地控制器", tags = {"青创阵地管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/AppDiscoverPositionController")
@ApiSort(1)
public class AppDiscoverPositionController {
	private final IAppDiscoverPositionService iAppDiscoverPositionService;

	/**
	 * 查询【请填写功能名称】列表
	 */
	@ApiOperation("查询阵地列表（app端）")
	@PostMapping("/appList")
	@ApiOperationSupport(order = 10)
	public Result<List<AppDiscoverListVo>> appList(@RequestBody AppQueryPositionBo bo) {
		return iAppDiscoverPositionService.queryAppList(bo);
	}
	/**
	 * 查询发现阵地主列表
	 */
	@ApiOperation("查询发现阵地列表")
	@PostMapping("/list")
	@ApiOperationSupport(order = 20)
	public Result<PageRsp<AppDiscoverPositionVo>> list(@RequestBody PageReq<AppDiscoverPositionBo> bo) {
		return iAppDiscoverPositionService.queryPageList(bo);
	}

	/**
	 * 导出发现阵地主列表
	 */
	@ApiOperation("导出阵地列表")
	@PostMapping("/export")
	public void export(AppDiscoverPositionBo bo, HttpServletResponse response) {
		List<AppDiscoverPositionVo> list = iAppDiscoverPositionService.queryList(bo);
		//ExcelUtil.exportExcel(list, "发现阵地主", AppDiscoverPositionVo.class, response);
	}

	/**
	 * 获取发现阵地主详细信息
	 */
	@ApiOperation("获取发现阵地详细信息")
	@GetMapping("/detail")
	@ApiOperationSupport(order = 30)
	public Result<AppDiscoverPositionVo> getInfo(
															 @RequestParam Long id) {
		return Result.ofSuccess(iAppDiscoverPositionService.queryById(id));
	}

	/**
	 * 新增发现阵地主
	 */
	@ApiOperation("新增阵地")
	@PostMapping("/add")
	@ApiOperationSupport(order = 40)
	public Result add(@Validated(AddGroup.class) @RequestBody AppDiscoverPositionBo bo) {
		return Result.ofSuccess(iAppDiscoverPositionService.insertByBo(bo) ? 1 : 0);
	}

	/**
	 * 修改发现阵地主
	 */
	@ApiOperation("修改阵地")
	@PostMapping("/edit")
	@ApiOperationSupport(order = 50)
	public Result edit(@Validated(EditGroup.class) @RequestBody AppDiscoverPositionBo bo) {
		return Result.ofSuccess(iAppDiscoverPositionService.updateByBo(bo) ? 1 : 0);
	}

	/**
	 * 删除发现阵地主
	 */
	@ApiOperation("删除阵地")
	@PostMapping("/remove")
	@ApiOperationSupport(order = 60)
	public Result remove(
						 @RequestParam  Long id) {
		return Result.ofSuccess(iAppDiscoverPositionService.deleteWithValidByIds(id, true) ? 1 : 0);
	}
}
