package com.crunii.micro.service.appsecond.controller;

import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.appsecond.api.bo.AppHelpCenterTypeBo;
import com.crunii.micro.service.appsecond.api.vo.AppHelpHotCenterVo;
import com.crunii.micro.service.appsecond.utils.EasyExcelUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.crunii.micro.service.appsecond.api.vo.AppHelpCenterVo;
import com.crunii.micro.service.appsecond.api.bo.AppHelpCenterBo;
import com.crunii.micro.service.appsecond.service.IAppHelpCenterService;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.servlet.http.HttpServletResponse;

/**
 * app帮助中心Controller
 * 前端访问路由地址为:/app/center
 *
 * @author liuchuan
 * @date 2023-12-06
 */
@Validated
@Api(value = "app帮助中心控制器", tags = { "app帮助中心管理" })
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/appHelpCenterController")
public class AppHelpCenterController {

	private final IAppHelpCenterService iAppHelpCenterService;

	/**
	 * 查询app帮助中心列表
	 */
	@ApiOperation("查询app帮助中心列表")
	@PostMapping("/list")
	public Result<PageRsp<AppHelpCenterVo>> list(@RequestBody PageReq<AppHelpCenterBo> bo) {
		return iAppHelpCenterService.queryPageList(bo);
	}

	/**
	 * 查询app帮助中心列表
	 */
	@ApiOperation("查询app热门问题")
	@PostMapping("/appHotList")
	public Result<List<AppHelpHotCenterVo>> appHotList() {
		return iAppHelpCenterService.queryAppHotList();
	}

	/**
	 * 导出app帮助中心列表
	 */
	@ApiOperation("导出app帮助中心列表")
	@PostMapping("/export")
	public void export(AppHelpCenterBo bo, HttpServletResponse response) {
		List<AppHelpCenterVo> list = iAppHelpCenterService.queryList(bo);
		//ExcelUtil.exportExcel(list, "app帮助中心", AppHelpCenterVo.class, response);
	}

	/**
	 * 获取app帮助中心详细信息
	 */
	@ApiOperation("获取app帮助中心详细信息")
	@PostMapping("/get")
	public Result<AppHelpCenterVo> getInfo(@ApiParam("主键")
										   @NotNull(message = "主键不能为空")
										   @RequestParam("id") Long id) {
		return Result.ofSuccess(iAppHelpCenterService.queryById(id));
	}

	/**
	 * 新增app帮助中心
	 */
	@ApiOperation("新增app帮助中心")
	@PostMapping("/save")
	public Result add(@RequestBody AppHelpCenterBo bo) {
		return Result.ofSuccess(iAppHelpCenterService.insertByBo(bo) ? 1 : 0);
	}

	/**
	 * 修改app帮助中心
	 */
	@ApiOperation("修改app帮助中心")
	@PostMapping("/edit")
	public Result edit(@RequestBody AppHelpCenterBo bo) {
		return Result.ofSuccess(iAppHelpCenterService.updateByBo(bo) ? 1 : 0);
	}

	/**
	 * 删除app帮助中心
	 */
	@ApiOperation("删除app帮助中心")
	@PostMapping("/remove")
	public Result remove(@ApiParam("主键串")
						 @RequestParam("id") Long id) {
		return Result.ofSuccess(iAppHelpCenterService.deleteById(id) ? 1 : 0);
	}

	/**
	 * app根据输入问题搜索问题
	 */
	@ApiOperation("app帮助中心搜索")
	@PostMapping("/appGet")
	public Result<List<AppHelpHotCenterVo>> appGet(@RequestParam("question") String question) {
		return iAppHelpCenterService.appGet(question);
	}

	/**
	 * 点击问题查询答案
	 */
	@ApiOperation("点击问题查询答案")
	@PostMapping("/appGetById")
	public Result<AppHelpHotCenterVo> appGetById(@RequestParam("id") Long id) {
		AppHelpHotCenterVo vo = iAppHelpCenterService.doGetById(id);
		return Result.ofSuccess(vo);
	}

	/**
	 * 点击问题查询答案
	 */
	@ApiOperation("根据type查询")
	@PostMapping("/appGetByType")
	public Result<AppHelpHotCenterVo> appGetByType(@RequestBody AppHelpCenterTypeBo appHelpCenterTypeBo) {
		AppHelpHotCenterVo vo = iAppHelpCenterService.doGetByType(appHelpCenterTypeBo);
		return Result.ofSuccess(vo);
	}

	@ApiOperation(value = "模板下载")
	@PostMapping(value = "/downloadExcelTemp")
	public Object downloadModel() throws IOException {
		return iAppHelpCenterService.downloadModel();
	}


	@ApiOperation("校验excel")
	@PostMapping("/verifyExcel")
	public Result doVerifyExcel(MultipartFile file) {
		if (!EasyExcelUtils.isExcelFile(file)) {
			return new Result<>("文件类型出错,请上传正确的文件!",Boolean.FALSE);
		}
		if (file.getSize() > 5 * 1024 * 1024) {
			return new Result<>("文件大小超过5MB限制！", Boolean.FALSE);
		}

		return iAppHelpCenterService.doVerifyExcel(file);
	}

	@ApiOperation("导入")
	@PostMapping("/batchImport")
	public Result doImportDate(@RequestParam("dataKey") String dataKey) {
		return iAppHelpCenterService.doImportDate(dataKey);
	}
}
