package com.crunii.micro.service.appsecond.controller;


import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.crunii.micro.service.appsecond.api.vo.AppHelpHistoryVo;
import com.crunii.micro.service.appsecond.api.bo.AppHelpHistoryBo;
import com.crunii.micro.service.appsecond.service.IAppHelpHistoryService;

import java.util.List;
import java.util.Arrays;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.servlet.http.HttpServletResponse;

/**
 * 帮助中心搜索Controller
 * 前端访问路由地址为:/app/history
 * @author liuchuan
 * @date 2023-12-06
 */
@Validated
@Api(value = "帮助中心搜索控制器", tags = {"帮助中心搜索管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/appHelpHistoryController")
public class AppHelpHistoryController {

    private final IAppHelpHistoryService iAppHelpHistoryService;

    /**
     * 查询帮助中心搜索列表
     */
    @ApiOperation("查询帮助中心搜索列表")
    @PostMapping("/list")
    public Result<List<AppHelpHistoryVo>> list(@CurrWorker Worker worker) {
        return iAppHelpHistoryService.queryAppList(worker);
    }

    /**
     * 导出帮助中心搜索列表
     */
    @ApiOperation("导出帮助中心搜索列表")
    @PostMapping("/export")
    public void export(AppHelpHistoryBo bo, HttpServletResponse response) {
        List<AppHelpHistoryVo> list = iAppHelpHistoryService.queryList(bo);
        //ExcelUtil.exportExcel(list, "帮助中心搜索", AppHelpHistoryVo.class, response);
    }

    /**
     * 获取帮助中心搜索详细信息
     */
    @ApiOperation("获取帮助中心搜索详细信息")
    @GetMapping("/{id}")
    public Result<AppHelpHistoryVo> getInfo(@ApiParam("主键")
                                     @NotNull(message = "主键不能为空")
                                     @PathVariable("id") Long id) {
        return Result.ofSuccess(iAppHelpHistoryService.queryById(id));
    }

    /**
     * 新增帮助中心搜索
     */
    @ApiOperation("新增帮助中心搜索")
    @PostMapping("/add")
    public Result add(@Validated(AddGroup.class) @RequestBody AppHelpHistoryBo bo) {
        return Result.ofSuccess(iAppHelpHistoryService.insertByBo(bo) ? 1 : 0);
    }

    /**
     * 修改帮助中心搜索
     */
    @ApiOperation("修改帮助中心搜索")
    @PostMapping("/edit")
    public Result edit(@Validated(EditGroup.class) @RequestBody AppHelpHistoryBo bo) {
        return Result.ofSuccess(iAppHelpHistoryService.updateByBo(bo) ? 1 : 0);
    }

    /**
     * 删除帮助中心搜索
     */
    @ApiOperation("删除帮助中心搜索")
    @PostMapping("/remove")
    public Result remove(@ApiParam("主键串")
                          @NotEmpty(message = "主键不能为空")
                             @RequestParam("ids")  Long[] ids) {
        return Result.ofSuccess(iAppHelpHistoryService.deleteWithValidByIds(Arrays.asList(ids), true) ? 1 : 0);
    }
}
