package com.crunii.micro.service.appsecond.controller;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.*;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.*;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.crunii.micro.service.appsecond.service.IAppMediumService;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;


/**
 * 【请填写功能名称】Controller
 * 前端访问路由地址为:/app/medium
 * @author liuchuan
 * @date 2023-12-06
 */
@Validated
@Api(value = "媒体账号控制器", tags = {"【媒体账号】管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/AppMediumController")
public class AppMediumController {

    private final IAppMediumService iAppMediumService;

    @ApiOperation("媒体渠道新增/修改")
    @PostMapping("/createMedium")
    public Result<String> createMedium(@RequestBody @Validated MediumCreateDto dto, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppMediumService.createMedium(dto,worker));
    }

    @ApiOperation("分页查询媒体渠道界面")
    @PostMapping("/pageMedium")
    public Result<PageRsp<MediumVo>> pageMedium(@RequestBody PageReq<MediumDto> req, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppMediumService.pageMedium(req,worker));
    }

    @ApiOperation("媒体渠道关注")
    @PostMapping("/attentionMedium")
    public Result<String> attentionMedium(@RequestBody AttentionMediumDto dto){
        return Result.ofSuccess(iAppMediumService.attentionMedium(dto));
    }

    @ApiOperation("媒体渠道批量启用/停用")
    @PostMapping("/batchEnableMedium")
    public Result<String> batchEnableMedium(@Validated @RequestBody EnableMediumDto dto, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppMediumService.updateEnableMedium(dto,worker));
    }

    @ApiOperation("删除媒体渠道")
    @PostMapping("/delMedium")
    public Result<String> delMedium(@RequestParam("mediumId") Long mediumId, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppMediumService.delMedium(mediumId,worker));
    }

    @ApiOperation("媒体矩阵热度-分页列表")
    @PostMapping("/pageMediumHot")
    public Result<PageRsp<MediumHotVo>> pageMediumHot(@RequestBody PageReq<MediumHotDto> req, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppMediumService.pageMediumHot(req,worker));
    }

    @ApiOperation("媒体矩阵热度-媒体id查询分页列表")
    @PostMapping("/pageMediumContentHot")
    public Result<PageRsp<MediumContentHotVo>> pageMediumContentHot(@Validated @RequestBody PageReq<MediumContentHotDto> req, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppMediumService.pageMediumContentHot(req,worker));
    }

    @ApiOperation("媒体矩阵导出-导出")
    @PostMapping("/exportExcel")
    public Result<String> exportExcel(@RequestBody MediumHotExcelDto dto, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppMediumService.exportExcel(dto,worker));
    }

    @ApiOperation("媒体矩阵查询树状图")
    @PostMapping("/findMediumTree")
    public Result<MediumOrgTree> findMediumTree( @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppMediumService.findMediumTree(worker));
    }

    @ApiOperation("查询组织树")
    @PostMapping("/findAuthTree")
    public Result<List<OrgAuthTreeVo>> findAuthTree(@RequestBody OrgAuthTreeDto dto,@CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(iAppMediumService.findAuthTree(dto,worker));
    }

}
