package com.crunii.micro.service.appsecond.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.dto.appmultimodules.AppMultiModulesDto;
import com.crunii.micro.service.appsecond.api.vo.appmultimodules.AppUserViewVo;
import com.crunii.micro.service.appsecond.service.AppMultiModulesService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;


/**
 * youth-cq.
 * 热搜/热榜/点赞/收藏/关注/浏览记录
 * @author leixb
 * @since 2023-12-07 19:32
 **/
@Validated
@Api(tags = "热搜/热榜/点赞/收藏/关注/浏览记录")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/multiModules")
public class AppMultiModulesController {
    
    private final AppMultiModulesService appMultiModulesService;
    
    @ApiOperation("点赞/收藏/关注及取消")
    @PostMapping("/doAction")
    public Result<Void> doAction(@CurrWorker @ApiIgnore Worker worker, @Validated @RequestBody AppMultiModulesDto req) {
        return Result.ofSuccessMsg(appMultiModulesService.doAction(worker, req));
    }
    
    /*@ApiOperation("查询热搜榜/热度榜")
    @PostMapping("/findHot")
    public Result<AppMultiModulesHotResVo> findHot(@CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(appMultiModulesService.findHot(worker));
    }*/
    
    @ApiOperation("新增浏览记录")
    @PostMapping("/createView")
    public Result<Void> createView(@CurrWorker @ApiIgnore Worker worker, @ApiParam("内容id") Long contentId) {
        appMultiModulesService.createView(worker, contentId);
        return Result.ofSuccessMsg("操作成功！");
    }
    
    @ApiOperation("查询浏览记录列表")
    @PostMapping("/pageView")
    public Result<PageRsp<AppUserViewVo>> pageView(@CurrWorker @ApiIgnore Worker worker, @RequestBody PageReq<Void> req) {
        return Result.ofSuccess(appMultiModulesService.pageView(worker, req));
    }
    
    @ApiOperation("一键清空浏览记录列表")
    @PostMapping("/doClearView")
    public Result<Void> doClearView(@CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccessMsg(appMultiModulesService.doClearView(worker));
    }
    
    @ApiOperation("查询数量")
    @PostMapping("/findNum")
    public Result<Integer> findNum(@CurrWorker @ApiIgnore Worker worker, @RequestParam("type") Integer type) {
        return Result.ofSuccess(appMultiModulesService.findNum(worker, type));
    }
    
    @ApiOperation("查询我的收藏列表")
    @PostMapping("/pageCollect")
    public Result<PageRsp<AppUserViewVo>> pageCollect(@CurrWorker @ApiIgnore Worker worker, @RequestBody PageReq<Void> req) {
        return Result.ofSuccess(appMultiModulesService.pageCollect(worker, req));
    }
    
    @ApiOperation("批量取消收藏")
    @PostMapping("/doCancelCollect")
    public Result<Void> doCancelCollect(@CurrWorker @ApiIgnore Worker worker, @RequestBody List<String> ids) {
        return Result.ofSuccessMsg(appMultiModulesService.doCancelCollect(worker, ids));
    }
    
}
