package com.crunii.micro.service.appsecond.controller;


import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.crunii.micro.service.appsecond.api.vo.AppPersonalHonorVo;
import com.crunii.micro.service.appsecond.api.bo.AppPersonalHonorBo;
import com.crunii.micro.service.appsecond.service.IAppPersonalHonorService;

import java.util.List;
import java.util.Arrays;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.servlet.http.HttpServletResponse;

/**
 * 个人荣誉信息Controller
 * 前端访问路由地址为:/app/honor
 * @author liuchuan
 * @date 2023-12-06
 */
@Validated
@Api(value = "个人荣誉信息控制器", tags = {"个人荣誉信息管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/app_personal_honor")
public class AppPersonalHonorController {

    private final IAppPersonalHonorService iAppPersonalHonorService;

    /**
     * 查询个人荣誉信息列表
     */
    @ApiOperation("查询个人荣誉信息列表")
    @GetMapping("/list")
    public Result<PageRsp<AppPersonalHonorVo>> list(PageReq<AppPersonalHonorBo> bo) {
        return iAppPersonalHonorService.queryPageList(bo);
    }

    /**
     * 导出个人荣誉信息列表
     */
    @ApiOperation("导出个人荣誉信息列表")
    @PostMapping("/export")
    public void export(AppPersonalHonorBo bo, HttpServletResponse response) {
        List<AppPersonalHonorVo> list = iAppPersonalHonorService.queryList(bo);
        //ExcelUtil.exportExcel(list, "个人荣誉信息", AppPersonalHonorVo.class, response);
    }

    /**
     * 获取个人荣誉信息详细信息
     */
    @ApiOperation("获取个人荣誉信息详细信息")
    @GetMapping("/{ID}")
    public Result<AppPersonalHonorVo> getInfo(@ApiParam("主键")
                                     @NotNull(message = "主键不能为空")
                                     @PathVariable("ID") Long ID) {
        return Result.ofSuccess(iAppPersonalHonorService.queryById(ID));
    }

    /**
     * 新增个人荣誉信息
     */
    @ApiOperation("新增个人荣誉信息")
    @PostMapping("/add")
    public Result add(@Validated(AddGroup.class) @RequestBody AppPersonalHonorBo bo) {
        return Result.ofSuccess(iAppPersonalHonorService.insertByBo(bo) ? 1 : 0);
    }

    /**
     * 修改个人荣誉信息
     */
    @ApiOperation("修改个人荣誉信息")
    @PostMapping("/edit")
    public Result edit(@Validated(EditGroup.class) @RequestBody AppPersonalHonorBo bo) {
        return Result.ofSuccess(iAppPersonalHonorService.updateByBo(bo) ? 1 : 0);
    }

    /**
     * 删除个人荣誉信息
     */
    @ApiOperation("删除个人荣誉信息")
    @PostMapping("/remove")
    public Result remove(@ApiParam("主键串")
                          @NotEmpty(message = "主键不能为空")
                             @RequestParam("ids")  Long[] IDs) {
        return Result.ofSuccess(iAppPersonalHonorService.deleteWithValidByIds(Arrays.asList(IDs), true) ? 1 : 0);
    }
}
