package com.crunii.micro.service.appsecond.controller;


import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.appsecond.api.dto.AppPersonalInfoDto;
import com.crunii.micro.service.sysmgr.api.domain.dto.OrgNode;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.crunii.micro.service.appsecond.api.vo.AppPersonalInfoVo;
import com.crunii.micro.service.appsecond.service.IAppPersonalInfoService;
import springfox.documentation.annotations.ApiIgnore;

import java.text.ParseException;
import java.util.List;

/**
 * 个人信息Controller
 * 前端访问路由地址为:/app/info
 * @author liuchuan
 * @date 2023-12-06
 */
@Validated
@Api(value = "个人信息", tags = {"个人信息"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/AppPersonalInfoController")
public class AppPersonalInfoController {

    private final IAppPersonalInfoService iAppPersonalInfoService;

    @ApiOperation("查询个人信息")
    @GetMapping("/getInfo")
    public Result<AppPersonalInfoVo> getInfo(@CurrWorker @ApiIgnore Worker worker) throws ParseException {
//        CurrWorkerUtil.mockCurrWorker();
//        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        return iAppPersonalInfoService.getInfo(worker);
    }


    /**
     * 修改个人信息
     */
    @ApiOperation("修改个人信息")
    @PostMapping("/edit")
    public Result edit(@RequestBody AppPersonalInfoDto dto) throws ParseException {
        return iAppPersonalInfoService.updateInfo(dto);
    }


    @ApiOperation(value = "通过父组织Id获取子组织(无数据权限学校版)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "parentOrgId", value = "父组织id", required = true)
    })
    @RequestMapping(path = "/getOrgChildrenByParent", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<OrgNode>> getOrgChildrenByParentOrgId(@RequestParam("parentOrgId") Long parentOrgId) {
        Result<List<OrgNode>> result = new Result<>();
        result.setData(iAppPersonalInfoService.getOrgChildrenByParentOrgId(parentOrgId));
        return result;
    }
}
