package com.crunii.micro.service.appsecond.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.feign.client.dto.HotEsPageDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.YouthPublicAuditDto;
import com.crunii.micro.service.appsecond.api.dto.UserViewDto;
import com.crunii.micro.service.appsecond.api.vo.appmultimodules.AppMultiModulesHotResVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.ContextAppDetailVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.YouthContextVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.YouthHotPublicDetailVo;
import com.crunii.micro.service.appsecond.service.AppMultiModulesService;
import com.crunii.micro.service.appsecond.service.AppYouthHotService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.view.RedirectView;
import springfox.documentation.annotations.ApiIgnore;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wangchen create 2023/12/8 15:42
 */
@Api(value = "【app首页青年热点】控制器", tags = {"【app首页青年热点】管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/AppYouthHotController")
public class AppYouthHotController {

    private final AppYouthHotService appYouthHotService;
    private final AppMultiModulesService appMultiModulesService;

    @ApiOperation("app首页-分页查询青年热点")
    @PostMapping("/pageYouthHot")
    public Result<PageRsp<YouthContextVo>> pageYouthHot(@RequestBody PageReq<HotEsPageDto> req, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(appYouthHotService.pageYouthHot(req,worker));
    }

    @ApiOperation("app首页-查询详情")
    @PostMapping("/getYouthHotDetail")
    public Result<ContextAppDetailVo> getYouthHotDetail(@RequestParam("contentManageId") String contentManageId, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(appYouthHotService.getYouthHotDetail(Long.parseLong(contentManageId),worker));
    }

    @ApiOperation("app首页-查询搜索历史")
    @PostMapping("/queryHotSearch")
    public Result<List<String>> queryHotSearch(Integer searchType,@CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(appYouthHotService.queryHotSearch(searchType,worker));
    }

    @ApiOperation("app首页-清除搜索历史")
    @PostMapping("/clearSearch")
    public Result<String> clearSearch(@CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(appYouthHotService.delSearch(worker));
    }
    
    @ApiOperation("查询热搜榜/热度榜")
    @PostMapping("/findHot")
    public Result<AppMultiModulesHotResVo> findHot(@CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(appMultiModulesService.findHot(worker));
    }

    @ApiOperation("app首页-浏览记录新增")
    @PostMapping("/createUserView")
    public Result<String> createUserView(@Validated @RequestBody UserViewDto dto, @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(appYouthHotService.createUserView(dto,worker));
    }

    @ApiOperation("查询app应用热搜")
    @GetMapping("/findAppHot")
    public Result<List<String>> findAppHot(@CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(appMultiModulesService.findAppHot(worker));
    }

    @ApiOperation("公共查询审核详情")
    @PostMapping("/queryYouthHot")
    public Result<YouthHotPublicDetailVo> queryYouthHot(@RequestParam("contentManageId") Long contentManageId) {
        return Result.ofSuccess(appYouthHotService.queryYouthHot(contentManageId));
    }

    @ApiOperation("公共提交审核")
    @PostMapping("/submitPublicAudit")
    public Result<String> submitPublicAudit(@RequestBody YouthPublicAuditDto dto) {
        return Result.ofSuccess(appYouthHotService.updatePublicAudit(dto));
    }
    private static Map<String,String> redirect = new HashMap<>();

    static {
        redirect.put("aa","http://121.229.175.82:63221/#/mine/news/detail/judge?contentManageId=1748231807211933696&id=3o+5GlenENdZuf92plWVTg==&fileType=2&isImportNews=0");
    }

    @ApiOperation("重定向")
    @GetMapping("/redirect/{id}")
    public RedirectView redirect(@PathVariable("id") String id) {
        String s = redirect.get(id);
        if(!StringUtils.hasText(s)){
            throw new BusinessException("链接失效");
        }
        RedirectView redirectView = new RedirectView(s);
        redirectView.setExposeModelAttributes(false);
        return redirectView;
    }
}
