package com.crunii.micro.service.appsecond.controller.youthspace;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.ExportSpaceTeamTemplateDTO;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.SpacePassTeamCUReqDTO;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.SpacePassTeamPageReqDTO;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.TeamBatchImportDTO;
import com.crunii.micro.service.appsecond.api.vo.youthspace.SpacePassTeamPageVO;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.BusinessStateEnum;
import com.crunii.micro.service.appsecond.service.SpacePassTeamService;
import com.crunii.micro.service.appsecond.utils.EasyExcelUtils;
import io.swagger.annotations.*;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import lombok.RequiredArgsConstructor;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

/**
 * @Author thw
 * @date 2023/12/13
 * @description 青创空间的团队管理
 */
@Validated
@Api(value = "青创空间的团队管理", tags = {"青创空间的团队管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/SpacePassTeamController")
public class SpacePassTeamController {

  /**
   * excel导入文件模板地址
   */
  public static final String EXCEL_DEMO_PATH_NO_BUSINESS = "static/团队新增没有营业执照模板.xlsx";
  public static final String EXCEL_DEMO_PATH_YES_BUSINESS = "static/团队新增有营业执照模板.xlsx";

  private final SpacePassTeamService spacePassTeamService;

  @ApiOperation("查询具体的青创空间信息通过id")
  @GetMapping("/oneSpaceInfo")
  public Result saveSpace(@ApiParam("主键")
  @NotBlank(message = "空间主键不能为空")  String spaceId) {
    return spacePassTeamService.oneSpaceInfo(Long.valueOf(spaceId));
  }

  @ApiOperation("已入住团队的分页")
  @PostMapping("/pageTeamInfo")
  public Result<PageRsp<SpacePassTeamPageVO>> pageTeamInfo(@Validated(AddGroup.class) @RequestBody PageReq<SpacePassTeamPageReqDTO> bo) {
    return Result.ofSuccess(spacePassTeamService.pageTeamInfo(bo));
  }

  @ApiOperation("新增团队")
  @PostMapping("/saveTeam")
  public Result saveTeam(@Validated(AddGroup.class) @RequestBody SpacePassTeamCUReqDTO bo) {
    return spacePassTeamService.saveTeam(bo);
  }

  @ApiOperation("编辑团队")
  @PostMapping("/updateTeam")
  public Result updateTeam(@Validated(EditGroup.class) @RequestBody SpacePassTeamCUReqDTO bo) {
    return spacePassTeamService.updateTeam(bo);
  }


/*  @ApiOperation(value = "批量导入新增团队")
  @RequestMapping(path = "/saveBatchImportExcelTeam", method = RequestMethod.POST)
  @ApiImplicitParams({
      @ApiImplicitParam(name = "file", value = "文件", required = true, dataType = "MultipartFile") ,
      @ApiImplicitParam(name = "businessLicense", value = "营业执照(0:无、1:有)", required = true, dataType = "Integer")
  } )
  @ResponseBody
  public Result saveBatchImportExcelTeam(@RequestParam("file") MultipartFile file,@NotNull(message = "营业执照不能为空") @RequestParam("businessLicense")Integer businessLicense)
      throws IOException {
    ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
    HttpServletResponse response = attributes.getResponse();
    return spacePassTeamService.saveBatchImportExcelTeam(response,file,businessLicense);
  }*/


  @ApiOperation(value = "验证批量导入新增团队")
  @RequestMapping(path = "/doVerifyImportTeamExcel", method = RequestMethod.POST)
  @ApiImplicitParams({
      @ApiImplicitParam(name = "file", value = "文件", required = true, dataType = "MultipartFile") ,
      @ApiImplicitParam(name = "businessLicense", value = "营业执照(0:无、1:有)", required = true, dataType = "Integer") ,
      @ApiImplicitParam(name = "spaceId", value = "空间id", required = true, dataType = "String")
  } )
  @ResponseBody
  public Result doVerifyImportTeamExcel(@RequestParam("file") MultipartFile file,@NotNull(message = "营业执照不能为空") @RequestParam("businessLicense")Integer businessLicense ,@NotBlank(message = "空间id不允许为空") @RequestParam("spaceId") String spaceId) {
    ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
    HttpServletResponse response = attributes.getResponse();
    return spacePassTeamService.doVerifyImportTeamExcel(response,file,businessLicense,Long.valueOf(spaceId));
  }


  @ApiOperation("批量导入新增团队")
  @PostMapping("/teamBatchImport")
  public Result saveTeamBatchImport(@Validated(EditGroup.class) @RequestBody TeamBatchImportDTO dto) {
    if (ObjectUtil.isEmpty(dto)) {
      throw new BusinessException("参数对象为空.");
    }
    if (ObjectUtil.isEmpty(dto.getDataKey()) || ObjectUtil.isEmpty(dto.getSpaceId())) {
      throw new BusinessException("必传参数为空.");
    }
    return spacePassTeamService.saveTeamBatchImport(dto);
  }




/*  @ApiOperation(value = "excel新增团队模板下载")
  @RequestMapping(path = "/exportSpaceTeamTemplate", method = RequestMethod.POST)
  @ResponseBody
    public void exportSpaceTeamTemplate(@Validated(EditGroup.class) @RequestBody ExportSpaceTeamTemplateDTO dto ) {
    ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
    HttpServletResponse response = attributes.getResponse();
    if (ObjectUtil.isEmpty(dto)) {
      throw new BusinessException("参数对象为空.");
    }
    if (ObjectUtil.isEmpty(dto.getBusinessLicense())) {
      throw new BusinessException("营业执照不能为空.");
    }
    spacePassTeamService.exportSpaceTeamTemplate(response,dto.getBusinessLicense());
  }*/

  @ApiOperation(value = "excel新增团队模板下载")
  @PostMapping("/exportSpaceTeamTemplate")
  public Object downloadExcelTemp(@Validated(EditGroup.class) @RequestBody ExportSpaceTeamTemplateDTO dto ) {
    ResponseEntity<InputStreamResource> response = null;
    try {
      if (ObjectUtil.isEmpty(dto) || ObjectUtil.isEmpty(dto.getBusinessLicense())){
        throw new BusinessException("参数对象必传");
      }
      if (BusinessStateEnum.NO_BUSINESS.getNum().equals(dto.getBusinessLicense())) {
        response = EasyExcelUtils.downloadExcelDemo(EXCEL_DEMO_PATH_NO_BUSINESS, "团队新增没有营业执照模板");
      }else {
        response = EasyExcelUtils.downloadExcelDemo(EXCEL_DEMO_PATH_YES_BUSINESS, "团队新增有营业执照模板");
      }
    } catch (Exception e) {
      throw new BusinessException("新增团队模板下载报错！");
    }
    return response;
  }


  @ApiOperation(value = "已入住团队的批量导出")
  @RequestMapping(path = "/batchExportTeam", method = RequestMethod.POST)
  @ResponseBody
  public void batchExportTeam(HttpServletResponse response ,@Validated(AddGroup.class) @RequestBody SpacePassTeamPageReqDTO bo){
    spacePassTeamService.batchExportTeam(response,bo);
  }

  @ApiOperation("更改团队状态(孵化结束)")
  @GetMapping("/updateTeamState")
  public Result updateTeamState(@ApiParam("主键")
  @NotBlank(message = "团队主键不能为空") String id) {
    return spacePassTeamService.updateTeamState(Long.valueOf(id));
  }

}
