package com.crunii.micro.service.appsecond.controller.youthspace;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.EnumUtils;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.YouthSpacAuditReqDTO;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.YouthSpacCReqDTO;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.YouthSpacePageReqDTO;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.YouthSpaceStateReqDTO;
import com.crunii.micro.service.appsecond.api.vo.youthspace.YouthSpacePageVO;
import com.crunii.micro.service.appsecond.enums.PriceEnums;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums;
import com.crunii.micro.service.appsecond.enums.YouthSpaceEnums;
import com.crunii.micro.service.appsecond.service.YouthSpaceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import java.util.List;
import java.util.Map;
import javax.validation.constraints.NotBlank;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @Author thw
 * @date 2023/12/12
 * @description 青创空间管理
 */
@Validated
@Api(value = "青创空间管理", tags = {"青创空间管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/YouthSpaceController")
public class YouthSpaceController {

  private final YouthSpaceService youthSpaceService;

  @ApiOperation("用到的枚举 StateEnum(青创空间下架与公示 )、HierarchyLabelEnum(青创空间层级-级别标志)、EvaluateStateEnum(青创空间层级审核状态)、FileTypeEnum(文件类型枚举) 、"
      + "CommEnum(枚举(无、有))、DemandEnum(工位需求枚举 )、PriceTypeEnum(工位单价类型枚举)、TeamStateEnum(团队入住状态 )、SteamAuditStateEnum(团队审核状态 )、"
      + "BusinessStateEnum(营业模板枚举状态)、SexEnum(性别枚举)")
  @GetMapping("/enumList")
  public Result<List<Map<Object, String>>> enumList(@RequestParam("enumName") String enumName) {

    List<Map<Object, String>> utilMap = null;

    switch (enumName) {
      case "StateEnum":
        utilMap = EnumUtils.EnumToMap(YouthSpaceEnums.StateEnum.class);
        break;
      case "HierarchyLabelEnum":
        utilMap = EnumUtils.EnumToMap(YouthSpaceEnums.HierarchyLabelEnum.class,"getName");
        break;
      case "EvaluateStateEnum":
        utilMap = EnumUtils.EnumToMap(YouthSpaceEnums.EvaluateStateEnum.class,"getNum");
        break;
      case "FileTypeEnum":
        utilMap = EnumUtils.EnumToMap(YouthSpaceEnums.FileTypeEnum.class,"getNum");
        break;
      case "CommEnum":
        utilMap = EnumUtils.EnumToMap(YouthSpaceEnums.CommEnum.class,"getNum");
        break;
      case "DemandEnum":
        utilMap = EnumUtils.EnumToMap(StayTeamEnums.DemandEnum.class,"getNum");
        break;
      case "TeamStateEnum":
        utilMap = EnumUtils.EnumToMap(StayTeamEnums.TeamStateEnum.class,"getNum");
        break;
      case "SteamAuditStateEnum":
        utilMap = EnumUtils.EnumToMap(StayTeamEnums.SteamAuditStateEnum.class,"getNum");
        break;
      case "BusinessStateEnum":
        utilMap = EnumUtils.EnumToMap(StayTeamEnums.BusinessStateEnum.class,"getNum");
        break;
      case "SexEnum":
        utilMap = EnumUtils.EnumToMap(StayTeamEnums.SexEnum.class,"getNum");
        break;
      case "PriceTypeEnum":
        utilMap = EnumUtils.EnumToMap(PriceEnums.PriceTypeEnum.class,"getNum");
        break;
      default:
        utilMap = EnumUtils.EnumToMap(YouthSpaceEnums.CommEnum.class,"getNum");
    }
    return Result.ofSuccess(utilMap);
  }

  @ApiOperation("青创空间分页")
  @PostMapping("/pageSpaceInfo")
  public Result<PageRsp<YouthSpacePageVO>> pageSpaceInfo(@Validated(AddGroup.class) @RequestBody PageReq<YouthSpacePageReqDTO> bo) {
    return Result.ofSuccess(youthSpaceService.pageSpaceInfo(bo));
  }

  @ApiOperation("发布青创空间")
  @PostMapping("/saveSpace")
  public Result saveSpace(@Validated(AddGroup.class) @RequestBody YouthSpacCReqDTO bo) {
    return youthSpaceService.saveSpace(bo);
  }

  @ApiOperation("编辑青创空间")
  @PostMapping("/updateSpace")
  public Result updateSpace(@Validated(EditGroup.class) @RequestBody YouthSpacCReqDTO bo) {
    return youthSpaceService.updateSpace(bo);
  }


  @ApiOperation("修改青创空间层级审核状态")
  @PostMapping("/updateEvaluateState")
  public Result updateEvaluateState(@Validated(EditGroup.class) @RequestBody YouthSpacAuditReqDTO bo) {
    return youthSpaceService.updateEvaluateState(bo);
  }

  @ApiOperation("修改空间数据的是发布与下架")
  @PostMapping("/updateSpaceState")
  public Result updateSpaceState(@Validated(EditGroup.class) @RequestBody YouthSpaceStateReqDTO bo) {
    return youthSpaceService.updateSpaceState(bo);
  }


  @ApiOperation("删除指定青创空间通过id")
  @GetMapping("/deleteSpace")
  public Result deleteSpaceById(@ApiParam("主键")
  @NotBlank(message = "主键不能为空") String id) {
    return youthSpaceService.deleteSpaceById(Long.valueOf(id));
  }


}
