package com.crunii.micro.service.appsecond.dao;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.bo.WorkerAppReqVO;
import com.crunii.micro.service.appsecond.api.vo.AppAllRspVO;
import com.crunii.micro.service.appsecond.api.vo.WorkerAppRspVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author gongsongtao
 * @date 2023/12/23 10:23
 */
@Repository
public class AppSubjectDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNPJdbcTemplate;


    /**
     * 查询我的应用
     * @param worker
     * @return
     */
    public List<WorkerAppRspVO> getMyApp(Worker worker) {
        String sql = "select a.* " +
                " from ycq_app_config_url a " +
                " left join tsm_worker_app b on a.app_id=b.app_id" +
                " inner join (select b.app_id from tsm_worker_role a left join tsm_role_app b on a.role_id=b.role_id" +
                " where login_id=? group by b.app_id) c on a.app_id=c.app_id" +
                " where b.del_flag=0 and b.worker_id=? and a.state=1 order by b.CREATE_TIME desc";
        return jdbcTemplate.queryForBeans(WorkerAppRspVO.class,sql,worker.getLoginId(),worker.getLoginId());
    }

    /**
     * 查询所有app
     * @param worker
     * @return
     */
    public List<AppAllRspVO> getAllApp(WorkerAppReqVO workerAppReqVO, Worker worker) {
        StringBuilder sql = new StringBuilder("select a.*,a.subject as subjectType," +
                " case when (select count(*) from  tsm_worker_app where app_id=a.app_id and worker_id=? and DEL_FLAG=0)>0 then true else false end as whetherAdd " +
                " from ycq_app_config_url a " +
                " inner join ( select b.app_id from tsm_worker_role a left join tsm_role_app b on a.role_id=b.role_id" +
                " where login_id=? group by b.app_id) b on a.app_id=b.app_id" +
                " where a.state=1");
        List<Object> param = new ArrayList<>();
        param.add(worker.getLoginId());
        param.add(worker.getLoginId());
        if (ObjectUtil.isNotEmpty(workerAppReqVO) && StringUtils.isNotEmpty(workerAppReqVO.getAppName())){
            sql.append(" and a.app_name like concat('%',?,'%')");
            param.add(workerAppReqVO.getAppName());
        }
        sql.append(" order by a.sort ");
        return jdbcTemplate.queryForBeans(AppAllRspVO.class,sql.toString(),param.toArray());
    }
}
