package com.crunii.micro.service.appsecond.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.PicInfo;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.AppContentManage;
import com.crunii.micro.domain.entity.main.AppMedium;
import com.crunii.micro.feign.client.dto.YouthHotEsVo;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ContextManageDto;
import com.crunii.micro.service.appsecond.api.dto.YouthHotLabelVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.*;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/12/7 15:35
 */
@Repository
public class AppContentManageDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNPJdbcTemplate;

    public AppMedium findMediumByWorkerId(int loginId) {
        String sql = "SELECT * FROM APP_MEDIUM WHERE associated_user_id = ? and medium_state = 1 AND del_flag = 0";
        return jdbcTemplate.queryForBean(AppMedium.class, sql, loginId);
    }

    public PageRsp<ContextManageVo> pageContextManage(PageReq<ContextManageDto> req, List<Long> mediumIds, boolean superAdmin, String labelId) {
        StringBuilder sb = new StringBuilder("SELECT a.content_manage_id,a.file_type,d.worker_name,a.state,a.content_title,a.like_count,a.collect_count,a.read_count,a.label_name,c.id_name,a.app_sort,a.release_time,a.hot_list,a.hot_search FROM APP_CONTENT_MANAGE a  LEFT JOIN APP_MEDIUM c ON a.medium_id = c.medium_id LEFT JOIN TSM_WORKER d ON c.associated_user_id = d.login_id WHERE a.del_flag = 0 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM APP_CONTENT_MANAGE a  LEFT JOIN APP_MEDIUM c ON a.medium_id = c.medium_id WHERE a.del_flag = 0 ");
        Map<String, Object> params = new HashMap<>(6);
        ContextManageDto query = req.getQuery();
        if (!superAdmin) {
            sb.append(" AND  a.medium_id in (:ids) ");
            sbCount.append(" AND  a.medium_id in(:ids) ");
            params.put("ids", mediumIds);
        }

        if (query.isImportantNews()) {
            sb.append(" and a.label_name = :labelId");
            sbCount.append(" and a.label_name = :labelId");
        } else {
            sb.append(" and a.label_name <> :labelId");
            sbCount.append(" and a.label_name <> :labelId");
        }
        params.put("labelId", labelId);

        if (StringUtils.hasText(query.getIdName())) {
            sb.append(" and c.id_name  like concat('%',:idName,'%')");
            sbCount.append(" and c.id_name  like concat('%',:idName,'%')");
            params.put("idName", query.getIdName());
        }
        if (Objects.nonNull(query.getContentTitle())) {
            sb.append(" and a.content_title like concat('%',:contentTitle,'%') ");
            sbCount.append(" and a.content_title like concat('%',:contentTitle,'%') ");
            params.put("contentTitle", query.getContentTitle());
        }
        if (Objects.nonNull(query.getState())) {
            sb.append(" and a.state = :state  ");
            sbCount.append(" and a.state = :state ");
            params.put("state", query.getState());
        }

        if (Objects.nonNull(query.getHotSearch())) {
            sb.append(" and a.hot_search = :hotSearch  ");
            sbCount.append(" and a.hot_search = :hotSearch ");
            params.put("hotSearch", query.getHotSearch());
        }

        if (Objects.nonNull(query.getHotList())) {
            sb.append(" and a.hot_list = :hotList  ");
            sbCount.append(" and a.hot_list = :hotList ");
            params.put("hotList", query.getHotList());
        }

        if (Objects.nonNull(query.getStartTime())) {
            sb.append(" and a.release_time >= :startTime  ");
            sbCount.append(" and a.release_time >= :startTime  ");
            params.put("startTime", query.getStartTime());
        }

        if (Objects.nonNull(query.getEndTime())) {
            sb.append(" and a.release_time <= :endTime  ");
            sbCount.append(" and a.release_time <= :endTime  ");
            params.put("endTime", query.getEndTime());
        }


        List<ContextManageVo> values;
        int total = baseNPJdbcTemplate.queryForInt(sbCount.toString(), params);
        String dbType = baseNPJdbcTemplate.getDbType();
        if (total > 0) {
            //添加orderBy
            if (query.isImportantNews()) {
                sb.append(" ORDER BY a.app_sort,a.release_time IS NULL ,a.release_time DESC,a.update_time DESC ");
            } else {
                sb.append(" ORDER BY a.release_time is null, a.release_time desc ");
            }
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = baseNPJdbcTemplate.queryForBeans(ContextManageVo.class, sb.toString(), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<ContextManageVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }


    public ContextManageDetailVo findContextManageDetail(Long contentManageId) {
        String sql = "SELECT b.file_id,a.file_type ,b.pic_key,a.content_title,b.article_context,a.label_name,a.state FROM APP_CONTENT_MANAGE a LEFT JOIN APP_CONTENT_DETAIL b ON a.content_manage_id =b.content_manage_id WHERE a.del_flag = 0 AND a.content_manage_id = ? ";
        return jdbcTemplate.queryForBean(ContextManageDetailVo.class, sql, contentManageId);
    }

    public void offShelfContext(List<Long> contentManageIds, Worker worker) {
        String sql = "UPDATE APP_CONTENT_MANAGE SET state = -1,app_sort = 2,hot_list=0,hot_search=0,list_ordinal_value=null,search_ordinal_value=null,update_by= :updateId,update_time=:updateTime,off_count=off_count+1 WHERE content_manage_id in(:ids)";
        Map<String, Object> params = new HashMap<>(6);
        params.put("ids", contentManageIds);
        params.put("updateId", worker.getLoginId());
        params.put("updateTime", new Date());
        baseNPJdbcTemplate.update(sql, params);
    }

    public void delContext(List<Long> contentManageIds, Worker worker) {
        String sql = "UPDATE APP_CONTENT_MANAGE SET del_flag = 1,app_sort = 2,release_time = null,hot_list=0,hot_search=0,list_ordinal_value=null,search_ordinal_value=null,update_by= :updateId,update_time=:updateTime WHERE content_manage_id in(:ids)";
        Map<String, Object> params = new HashMap<>(6);
        params.put("ids", contentManageIds);
        params.put("updateId", worker.getLoginId());
        params.put("updateTime", new Date());
        baseNPJdbcTemplate.update(sql, params);
    }

    public YouthHotEsVo findPushEsData(Long mediumId) {
        String sql = "SELECT medium_id,id_name,medium_head_pic FROM APP_MEDIUM WHERE medium_id = ?";
        return jdbcTemplate.queryForBean(YouthHotEsVo.class, sql, mediumId);
    }

    public List<YouthHotEsVo> findAllContext() {
        String sql = "SELECT a.content_manage_id,a.file_type,a.content_title,b.article_context,c.medium_id,c.id_name,c.medium_head_pic FROM APP_CONTENT_MANAGE a LEFT JOIN APP_CONTENT_DETAIL b ON a.content_manage_id =b.content_manage_id  LEFT JOIN APP_MEDIUM c ON a.medium_id =c.medium_id \n" +
                "WHERE a.state =1 AND a.del_flag = 0";
        return jdbcTemplate.queryForBeans(YouthHotEsVo.class, sql);
    }

    public List<YouthHotLabelVo> findLabelList(Set<Long> labelIds) {
        String sql = "SELECT label_id,label_name FROM TSM_LABEL_MANAGEMENT WHERE label_id in(:ids)";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", labelIds);
        return baseNPJdbcTemplate.queryForBeans(YouthHotLabelVo.class, sql, params);
    }

    public Integer findMaxApp() {
        String sql = "SELECT MAX(app_sort)+1 FROM APP_CONTENT_MANAGE WHERE state = 1";
        return jdbcTemplate.queryForRawObject(Integer.class, sql);
    }

    public AppMedium findMediumInfo(Long contentManageId) {
        String sql = "SELECT id_name,org_id,medium_head_pic FROM APP_MEDIUM WHERE medium_id = (SELECT medium_id FROM APP_CONTENT_MANAGE a  WHERE content_manage_id = ?)";
        return jdbcTemplate.queryForBean(AppMedium.class, sql, contentManageId);
    }

    public List<PicInfo> findPicInfoByFileKey(List<String> fileKey) {
        String sql = "SELECT object_key AS picUrl,width,height FROM YCQ_OBS_FILE WHERE object_key in(:ids)";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", fileKey);
        return baseNPJdbcTemplate.queryForBeans(PicInfo.class, sql, params);
    }

    public int findStateIsPush(List<Long> contentManageIds) {
        String sql = "SELECT count(1) FROM APP_CONTENT_MANAGE WHERE content_manage_id in(:ids)";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", contentManageIds);
        return baseNPJdbcTemplate.queryForInt(sql, params);
    }

    public Long getImportantNewsLabel() {
        String sql = "SELECT label_id FROM TSM_LABEL_MANAGEMENT WHERE label_name  = '要闻'";
        return jdbcTemplate.queryForRawObject(Long.class, sql);
    }

    public AppContentManage findSecond() {
        String sql = "SELECT * FROM APP_CONTENT_MANAGE WHERE del_flag = 0 AND app_sort = 1 ORDER BY release_time DESC LIMIT 1,1";
        return jdbcTemplate.queryForBean(AppContentManage.class, sql);
    }

    public List<IdValueVo> findPicFileKey(List<Long> collect) {
        String sql = "SELECT content_manage_id AS id,pic_key AS value FROM APP_CONTENT_DETAIL WHERE content_manage_id in(:ids) and pic_key is not null";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", collect);
        return baseNPJdbcTemplate.queryForBeans(IdValueVo.class, sql, params);
    }

    public List<AuditFlowVo> queryAuditFlow(Long contentManageId) {
        String sql = "SELECT tw.worker_name as operator ,a.create_time ,to2.org_name ,a.result_type ,a.result_str ,a.reason,(CASE WHEN result_type IS NOT NULL THEN 0 ELSE 1 end) AS isSubmitUser  FROM YCQ_ACTIVE_FLOW_NODE a LEFT JOIN TSM_WORKER tw ON a.create_id =tw.login_id LEFT JOIN TSM_ORG to2 ON tw.org_id = to2.org_id  WHERE biz_id =? AND is_restart = 0 ORDER BY create_time ";
        return jdbcTemplate.queryForBeans(AuditFlowVo.class, sql, contentManageId);
    }

    public void delAuditFlow(List<Long> contentManageIds) {
        String sql = "UPDATE YCQ_ACTIVE_FLOW_NODE SET is_restart = 1 WHERE biz_id in(:ids)";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", contentManageIds);
        baseNPJdbcTemplate.update(sql, params);
    }

    public List<String> findAuditOrgUser(Long orgId, Integer auditRoleId) {
        String sql = "SELECT a.login_id  FROM TSM_WORKER_ROLE a LEFT JOIN TSM_WORKER tw ON a.login_id =tw.login_id  WHERE a.role_id = ? AND tw.state =1 AND tw.org_id = ?";
        return jdbcTemplate.queryForRawObjects(String.class, sql, auditRoleId, orgId);
    }

    public List<String> findMediumOrgIds(List<Long> contentManageIds) {
        String sql = "SELECT am.org_id FROM APP_CONTENT_MANAGE a LEFT JOIN APP_MEDIUM am ON a.medium_id =am.medium_id  WHERE a.content_manage_id in(:ids) GROUP BY am.org_id";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", contentManageIds);
        return baseNPJdbcTemplate.queryForRawObjects(String.class, sql, params);
    }

    public List<Long> findMediums(String orgId) {
        String sql = "SELECT medium_id FROM APP_MEDIUM WHERE org_id =? AND del_flag = 0";
        return jdbcTemplate.queryForRawObjects(Long.class, sql, orgId);
    }

    public OrgAuthTreeVo findOrgName(Long contentManageId) {
        String sql = " SELECT c.org_id,c.org_name FROM APP_CONTENT_MANAGE a LEFT JOIN APP_MEDIUM b ON a.medium_id = b.medium_id LEFT JOIN TSM_ORG c ON b.org_id = c.org_id WHERE content_manage_id = ?";
        return jdbcTemplate.queryForBean(OrgAuthTreeVo.class,sql,contentManageId);
    }

    public List<String> findUserName(List<String> auditOrgUser) {
        String sql = "SELECT worker_name FROM TSM_WORKER WHERE login_id in(:ids)";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", auditOrgUser);
        return baseNPJdbcTemplate.queryForRawObjects(String.class,sql,params);
    }
}
