package com.crunii.micro.service.appsecond.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.AppContentManage;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ContextManageDto;
import com.crunii.micro.service.appsecond.api.dto.YouthHotLabelVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.ContextManageVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.HandleHistoryVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.IdNumberVo;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/12/8 10:36
 */
@Repository
public class AppContentSortDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNPJdbcTemplate;

    public int findHotSearchCount(boolean isHotSearch,Long contentManageId) {
        String sql = "SELECT count(1) FROM APP_CONTENT_MANAGE where del_flag = 0 %s and content_manage_id <> ?";
        String format;
        if(isHotSearch){
            format = String.format(sql, " AND hot_search = 1 ");
        }else {
            format = String.format(sql,"AND hot_list = 1");
        }
        return jdbcTemplate.queryForInt(format,contentManageId);
    }

    public List<IdNumberVo> findAllOrdinalValueBySearch(boolean isHotSearch,Long contentManageId) {
        String sql = "SELECT content_manage_id as id,%s as number FROM APP_CONTENT_MANAGE WHERE %s AND del_flag = 0 and content_manage_id<> ?";
        String format;
        if(isHotSearch){
            format = String.format(sql, "search_ordinal_value"," hot_search = 1 ");
        }else {
            format = String.format(sql,"list_ordinal_value"," hot_list = 1");
        }

        return jdbcTemplate.queryForBeans(IdNumberVo.class,format,contentManageId);
    }

    public PageRsp<HandleHistoryVo> pageHandleHistory(PageReq<Object> req) {
        StringBuilder sb = new StringBuilder("SELECT a.content_manage_id,b.content_title ,a.before_ordinal_value,a.after_ordinal_value,(SELECT worker_name FROM TSM_WORKER WHERE login_id = a.create_by)  AS workerName,a.create_time FROM APP_CONTENT_RECORD a LEFT JOIN APP_CONTENT_MANAGE b ON a.content_manage_id =b.content_manage_id LEFT JOIN APP_MEDIUM c ON b.medium_id =c.medium_id WHERE  b.del_flag = 0");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM APP_CONTENT_RECORD a LEFT JOIN APP_CONTENT_MANAGE b ON a.content_manage_id =b.content_manage_id LEFT JOIN APP_MEDIUM c ON b.medium_id =c.medium_id WHERE b.del_flag = 0 ");
        List<Object> params = new ArrayList<>();

        List<HandleHistoryVo> values;
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //添加orderBy
            sb.append(" ORDER BY a.create_time desc ");
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(HandleHistoryVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<HandleHistoryVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public PageRsp<ContextManageVo> pageContextManage(PageReq<ContextManageDto> req) {
        String sql = "SELECT  ROWNUM+(:pageNumber*:pageSize) AS appSort,* FROM (%s)";
        StringBuilder sb = new StringBuilder("SELECT a.content_manage_id,a.file_type,a.state,a.content_title,a.like_count,a.collect_count,a.read_count,a.label_name,c.id_name,a.release_time,a.hot_list,a.hot_search FROM APP_CONTENT_MANAGE a  LEFT JOIN APP_MEDIUM c ON a.medium_id = c.medium_id WHERE a.del_flag = 0 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM APP_CONTENT_MANAGE a  LEFT JOIN APP_MEDIUM c ON a.medium_id = c.medium_id WHERE a.del_flag = 0 ");
        Map<String,Object> params = new HashMap<>(6);
        Map<String,Object> paramsCount = new HashMap<>(6);
        ContextManageDto query = req.getQuery();

        params.put("pageNumber",req.getPageNo()-1);
        params.put("pageSize",req.getPageSize());
        if(StringUtils.hasText(query.getIdName())){
            sb.append(" and c.id_name  like concat('%',:idName,'%')");
            sbCount.append(" and c.id_name  like concat('%',:idName,'%')");
            params.put("idName",query.getIdName());
            paramsCount.put("idName",query.getIdName());
        }
        if(Objects.nonNull(query.getContentTitle())){
            sb.append(" and a.content_title like concat('%',:contentTitle,'%') ");
            sbCount.append(" and a.content_title like concat('%',:contentTitle,'%') ");
            params.put("contentTitle",query.getContentTitle());
            paramsCount.put("contentTitle",query.getContentTitle());
        }
        if(Objects.nonNull(query.getState())){
            sb.append(" and a.state = :state  ");
            sbCount.append(" and a.state = :state ");
            params.put("state",query.getState());
            paramsCount.put("state",query.getState());
        }

        if(Objects.nonNull(query.getHotSearch())){
            sb.append(" and a.hot_search = :hotSearch  ");
            sbCount.append(" and a.hot_search = :hotSearch ");
            params.put("hotSearch",query.getHotSearch());
            paramsCount.put("hotSearch",query.getHotSearch());
        }

        if(Objects.nonNull(query.getHotList())){
            sb.append(" and a.hot_list = :hotList  ");
            sbCount.append(" and a.hot_list = :hotList ");
            params.put("hotList",query.getHotList());
            paramsCount.put("hotList",query.getHotList());
        }


        List<ContextManageVo> values;
        int total = baseNPJdbcTemplate.queryForInt(sbCount.toString() , paramsCount );
        String dbType = baseNPJdbcTemplate.getDbType();
        if (total > 0) {
            //添加orderBy
            sb.append(" ORDER BY a.app_sort ASC ,a.update_time DESC ");
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = baseNPJdbcTemplate.queryForBeans(ContextManageVo.class, String.format(sql,sb), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<ContextManageVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public List<YouthHotLabelVo> findLabelList(Set<Long> labelIds) {
        String sql = "SELECT label_id,label_name FROM TSM_LABEL_MANAGEMENT WHERE label_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",labelIds);
        return baseNPJdbcTemplate.queryForBeans(YouthHotLabelVo.class,sql,params);
    }

    public AppContentManage findByAppSort(Integer appSort) {
        String sql = "SELECT * FROM APP_CONTENT_MANAGE WHERE app_sort = ? and state = 1";
        return jdbcTemplate.queryForBean(AppContentManage.class,sql,appSort);
    }
}
