package com.crunii.micro.service.appsecond.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.appsecond.api.bo.AppDiscoverPositionBo;
import com.crunii.micro.service.appsecond.api.bo.AppQueryPositionBo;
import com.crunii.micro.service.appsecond.api.vo.AppDiscoverListVo;
import com.crunii.micro.service.appsecond.api.vo.AppDiscoverPositionVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author lyx
 * @date 2023/12/7
 */
@Repository
public class AppDiscoverPositionDao {
	@Resource(name = "mainJT")
	protected BaseJdbcTemplate jdbcTemplate;

	@Resource(name = "mainNPJT")
	protected BaseNPJdbcTemplate NPJT;
	private static final String YOUTH_SPACE = "youthSpace"; // 青创空间

	private static final String APP_QUERY_DISCOVER_SQL="  select*from(  select  a.id,a.position_type_code,a.address,a.phone,a.position_name,a.district_name,a.district_code,a.longitude,a.latitude,b.file_id, b.file_url,b.file_type,file_name from app_discover_position a  "
			+ "   left join app_discover_position_annex b on a.id = b.business_id  and  "
			+ "      b.id = (select min(c.id) from  app_discover_position_annex c  where a.id = c.business_id  and c.video_flag=0  and c.del_flag = 0) where a.del_flag = 0 and a.longitude is not null and a.latitude is not null ";
	private static final String APP_QUERY_DISCOVER_YOUTH_SPACE_SQL=" select *from( select  a.id,'youthSpace' position_type_code,a.full_address address,a.phone,  space_name position_name,location_name district_name,location district_code,longitude,dimension latitude, "
			+ " b.file_id, b.file_url,file_suffix file_type,file_name from qczd_space_content a left join qczd_relation_file b on a.id = b.service_id and  "
			+ " b.id = (select min(c.id) from  qczd_relation_file c  where a.id = c.service_id  and c.del_flag = 0) where a.del_flag = 0 and longitude is not null and dimension is not null) a where 1=1 ";

	public List<AppDiscoverListVo> queryAppList(AppQueryPositionBo bo,double maxLong,double minLong,double maxLat,double minLat) {
		StringBuilder sb;
		List<Object> list = new ArrayList<>();
		if(YOUTH_SPACE.equals(bo.getPositionTypeCode())){
			sb = new StringBuilder(APP_QUERY_DISCOVER_YOUTH_SPACE_SQL);
		}else{
			sb = new StringBuilder(APP_QUERY_DISCOVER_SQL);
			sb.append(" union all ");
			sb.append(APP_QUERY_DISCOVER_YOUTH_SPACE_SQL);
			sb.append(")a where 1=1 ");

			if (Objects.nonNull(bo.getDiscoverType())){
				//发现类型，0阵地 TODO 2023年12月7日14:27:24 赛事活动暂时不做
				sb.append(" and a.discover_type = ? ");
			}
			if(StringUtils.isNotEmpty(bo.getPositionTypeCode())){
				//阵地类型，传code
				sb.append(" and a.position_type_code = ? ");
				list.add(bo.getPositionTypeCode());
			}
		}
		if (Objects.nonNull(bo.getDistrictCode())){
			//行政区。传id
			sb.append(" and district_code = ? ");
			list.add(bo.getDistrictCode());
		}
		if(StringUtils.isNotEmpty(bo.getName())){
			//阵地类型，传code
			sb.append(" and a.position_name like concat('%',?,'%')  ");
			list.add(bo.getName());
		}
		if (maxLong!=0.0&&minLong!=0.0&& maxLat!=0.0&&minLat!=0.0){
			sb.append(" and longitude between  ? and ? and latitude between  ? and ? ");
			list.add(BigDecimal.valueOf(minLong));
			list.add(BigDecimal.valueOf(maxLong));
			list.add(BigDecimal.valueOf(minLat));
			list.add(BigDecimal.valueOf(maxLat));
		}else {
			if(Objects.nonNull(bo.getLeftLongitude()) && Objects.nonNull(bo.getRightLongitude())
			&& Objects.nonNull(bo.getRightLatitude()) && Objects.nonNull(bo.getLeftLatitude())
			&& Objects.isNull(bo.getDistrictCode()) && StringUtils.isEmpty(bo.getPositionTypeCode())
			&& StringUtils.isEmpty(bo.getName())){
				sb.append(" and longitude between  ? and ? and latitude between  ? and ? ");
				list.add(bo.getLeftLongitude());
				list.add(bo.getRightLongitude());
				list.add(bo.getRightLatitude());
				list.add(bo.getLeftLatitude());
			}

		}
		return jdbcTemplate.queryForBeans(AppDiscoverListVo.class,sb.toString(),list.toArray());


	}

	public PageRsp<AppDiscoverPositionVo> pageList(PageReq<AppDiscoverPositionBo> req) {
		StringBuilder sb = new StringBuilder("select * from APP_DISCOVER_POSITION where del_flag=0  ");
		StringBuilder sbCount = new StringBuilder(" select count(*) from APP_DISCOVER_POSITION where del_flag=0   ");
		List<Object> params = new ArrayList<>();
		List<AppDiscoverPositionVo> values;

		if (ObjectUtil.isNotEmpty(req.getQuery().getDistrictCode())){
			sb.append(" and district_code= ? ");
			sbCount.append(" and district_code= ?  ");
			params.add(req.getQuery().getDistrictCode());
		}
		if (ObjectUtil.isNotEmpty(req.getQuery().getPositionName())){
			sb.append(" and position_name  LIKE concat('%',?,'%')");
			sbCount.append(" and position_name  LIKE concat('%',?,'%')");
			params.add(req.getQuery().getPositionName());
		}
		if (ObjectUtil.isNotEmpty(req.getQuery().getPositionTypeCode())){
			sb.append(" and position_type_code= ? ");
			sbCount.append(" and position_type_code= ?  ");
			params.add(req.getQuery().getPositionTypeCode());
		}
		int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
		String dbType = jdbcTemplate.getDbType();
		if (total > 0) {
			//添加orderBy
			sb.append(" ORDER BY update_time desc ");
			SqlUtil.appendLimit(dbType, params, sb, req);
			values = jdbcTemplate.queryForBeans(AppDiscoverPositionVo.class, sb.toString(), params.toArray());
		} else {
			values = new ArrayList<>(0);
		}
		PageRsp<AppDiscoverPositionVo> page = new PageRsp<>();
		page.setPageNo(req.getPageNo());
		page.setPageSize(req.getPageSize());
		page.setTotalNum(total);
		page.setResult(values);
		// 为手机计算是否由下一页
		page.next();
		return page;
	}

}
