package com.crunii.micro.service.appsecond.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.bo.AppHelpCenterBo;
import com.crunii.micro.service.appsecond.api.dto.LabelDto;
import com.crunii.micro.service.appsecond.api.dto.LabelUpload;
import com.crunii.micro.service.appsecond.api.vo.AppDownLoadFileVo;
import com.crunii.micro.service.appsecond.api.vo.AppHelpCenterVo;
import com.crunii.micro.service.appsecond.api.vo.AppHelpHotCenterVo;
import com.crunii.micro.service.appsecond.api.vo.ImagesVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author lyx
 * @date 2023/12/9
 */
@Repository
public class AppDownRecordDao {
	@Resource(name = "mainJT")
	protected BaseJdbcTemplate JT;

	@Resource(name = "mainNPJT")
	protected BaseNPJdbcTemplate baseNPJdbcTemplate;


	private static final String pageLabelValueBaseSql = "SELECT a.upload_id AS uploadId,a.file_title AS fileTitle,b.file_key AS fileKey FROM APP_DOWNLOAD_RECORD d  LEFT JOIN YCQ_UPLOAD_INFO a on a.upload_id = d.file_id  LEFT JOIN YCQ_FILE_RECORD B ON a.upload_id=b.upload_id  WHERE 1=1 AND b.is_pic=1 AND b.sort=0 AND a.audit_status=1";


	private static final String pageLabelValueCountSql = "SELECT count(*) FROM APP_DOWNLOAD_RECORD d  LEFT JOIN YCQ_UPLOAD_INFO a on a.upload_id = d.file_id LEFT JOIN YCQ_FILE_RECORD B ON a.upload_id=b.upload_id  WHERE 1=1 AND b.is_pic=1 AND b.sort=0 AND a.audit_status=1";

	public PageRsp<ImagesVo> queryList(PageReq<LabelDto> req, Worker worker) {
		StringBuilder sb = new StringBuilder(pageLabelValueBaseSql);
		StringBuilder sbCount = new StringBuilder(pageLabelValueCountSql);
		List<Object> params = new ArrayList<>();
		if (Objects.nonNull(worker)) {
			sb.append(" and d.create_id = ? and d.type = 0");
			sbCount.append(" and d.create_id = ? and d.type = 0");
			params.add(worker.getLoginId());
		}else{
			throw new BusinessException("未登录！");
		}
		if (Objects.nonNull(req.getQuery()) && StringUtils.isNotEmpty(req.getQuery().getName())){
			sb.append(" and a.file_title like concat('%',?,'%') ");
			sbCount.append(" and a.file_title like concat('%',?,'%') ");
			params.add(req.getQuery().getName());
		}
		List<ImagesVo> values;
		//添加逻辑删除判断
		//sbCount.append(" and a.state =1 ");
		int total = JT.queryForInt(sbCount.toString(), params.toArray());
		String dbType = JT.getDbType();
		if (total > 0) {
			//添加逻辑删除判断
			//sb.append(" and a.state =1 ");
			//添加orderBy
			sb.append(" ORDER BY d.create_time desc ");
			SqlUtil.appendLimit(dbType, params, sb, req);
			values = JT.queryForBeans(ImagesVo.class, sb.toString(), params.toArray());
		} else {
			values = new ArrayList<>(0);
		}
		PageRsp<ImagesVo> page = new PageRsp<>();
		page.setPageNo(req.getPageNo());
		page.setPageSize(req.getPageSize());
		page.setTotalNum(total);
		page.setResult(values);
		// 为手机计算是否由下一页
		page.next();
		return page;
	}



	public List<LabelUpload> findLabelIdAndName(List<Long> uploadIds) {
		String sql = "SELECT file_id uploadId,label_id,label_name FROM YCQ_FILE_LABEL_RELATION WHERE file_id in(:ids)";
		Map<String, Object> params = new HashMap<>(2);
		params.put("ids", uploadIds);
		return baseNPJdbcTemplate.queryForBeans(LabelUpload.class, sql, params);
	}

	public List<ImagesVo> queryFileInfo(List<String> picList) {
		String sql = "SELECT object_key as fileKey, width,height FROM YCQ_OBS_FILE WHERE object_key in(:ids)";
		Map<String, Object> params = new HashMap<>(2);
		params.put("ids", picList);
		return baseNPJdbcTemplate.queryForBeans(ImagesVo.class, sql, params);
	}

	public PageRsp<AppDownLoadFileVo> queryFileList(PageReq<LabelDto> req, Worker worker) {
		String sql = "select a.id,a.file_id,a.path,a.file_url,b.file_name,b.type file_type from APP_DOWNLOAD_RECORD a "
				+ "left join YCQ_OBS_FILE b on a.file_id = b.id where a.type = 1  and a.del_flag = 0";
		String countSql = "select count(*) from APP_DOWNLOAD_RECORD a "
				+ "left join YCQ_OBS_FILE b on a.file_id = b.id where a.type = 1  and a.del_flag = 0";
		StringBuilder sb = new StringBuilder(sql);
		StringBuilder sbCount = new StringBuilder(countSql);
		List<Object> params = new ArrayList<>();
		if (Objects.nonNull(worker)) {
			sb.append(" and a.create_id = ? ");
			sbCount.append(" and a.create_id = ? ");
			params.add(worker.getLoginId());
		}else{
			throw new BusinessException("未登录！");
		}
		if (Objects.nonNull(req.getQuery()) && StringUtils.isNotEmpty(req.getQuery().getName())){
			sb.append(" and b.file_name like concat('%',?,'%') ");
			sbCount.append(" and b.file_name like concat('%',?,'%') ");
			params.add(req.getQuery().getName());
		}
		int total = JT.queryForInt(sbCount.toString(), params.toArray());
		String dbType = JT.getDbType();
		List<AppDownLoadFileVo> list;
		if (total > 0) {
			SqlUtil.appendLimit(dbType, params, sb, req);
			list = JT.queryForBeans(AppDownLoadFileVo.class, sb.toString(), params.toArray());
		}else{
			list = new ArrayList<>();
		}
		PageRsp<AppDownLoadFileVo> page = new PageRsp<>();
		page.setPageNo(req.getPageNo());
		page.setPageSize(req.getPageSize());
		page.setTotalNum(total);
		page.setResult(list);
		// 为手机计算是否由下一页
		page.next();
		return page;

	}
}
