package com.crunii.micro.service.appsecond.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.AppHelpCenter;
import com.crunii.micro.service.appsecond.api.bo.AppHelpCenterBo;
import com.crunii.micro.service.appsecond.api.vo.AppHelpCenterVo;
import com.crunii.micro.service.appsecond.api.vo.AppHelpHotCenterVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author lyx
 * @date 2023/12/9
 */
@Repository
public class AppHelpCenterDao {
	@Resource(name = "mainJT")
	protected BaseJdbcTemplate jdbcTemplate;

	@Resource(name = "mainNPJT")
	protected BaseNPJdbcTemplate NPJT;

	private final static String QUERY_PAGE_LIST_SQL = "select *from app_help_center a where del_flag = 0 ";
	private final static String QUERY_PAGE_LIST_COUNT_SQL = "select count(*)from app_help_center a  where del_flag = 0 ";

	public PageRsp<AppHelpCenterVo> queryPageList(PageReq<AppHelpCenterBo> bo) {
		StringBuilder sb = new StringBuilder(QUERY_PAGE_LIST_SQL);
		StringBuilder sbCount = new StringBuilder(QUERY_PAGE_LIST_COUNT_SQL);
		List<Object> params = new ArrayList<>();
		if (Objects.nonNull(bo.getQuery()) && StringUtils.isNotEmpty(bo.getQuery().getQuestion())){
			sb.append(" and (a.question like concat('%',?,'%') or a.keywords  like concat('%',?,'%')) ");
			sbCount.append(" and (a.question like concat('%',?,'%') or a.keywords  like concat('%',?,'%')) ");
			params.add(bo.getQuery().getQuestion());
			params.add(bo.getQuery().getQuestion());
		}
		sb.append(" order by a.create_time desc ");

		int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
		List<AppHelpCenterVo> result;
		String dbType = jdbcTemplate.getDbType();
		if (total > 0) {
			SqlUtil.appendLimit(dbType, params, sb, bo);
			result = jdbcTemplate.queryForBeans(AppHelpCenterVo.class, sb.toString(), params.toArray());
		} else {
			result = new ArrayList<>(0);
		}
		PageRsp<AppHelpCenterVo> page = new PageRsp<>();
		page.setPageNo(bo.getPageNo());
		page.setPageSize(bo.getPageSize());
		page.setTotalNum(total);
		page.setResult(result);
		page.next();
		return page;
	}

	public AppHelpCenterVo queryById(Long id) {
		StringBuilder sb = new StringBuilder(QUERY_PAGE_LIST_SQL);
		sb.append(" and a.id = ? ");
		AppHelpCenterVo result = jdbcTemplate.queryForBean(AppHelpCenterVo.class, sb.toString(), id);
		return result;
	}

	public List<AppHelpHotCenterVo> findAllOrOrderByClickCountDescLimit() {
		String sql = "select id,question from  app_help_center where (help_type = 3 or help_type = 4)  and del_flag = 0 order by click_count desc ";
		return jdbcTemplate.queryForBeans(AppHelpHotCenterVo.class,sql);
	}

	public List<AppHelpHotCenterVo> findAllByQuestionLike(String question) {
		String sql = "select id,question from  app_help_center where  keywords like concat('%',?,'%') or question like concat('%',?,'%')   and help_type != 0 and help_type != 1  and help_type != 2 and help_type != 5 and del_flag = 0  order by click_count desc";

		return jdbcTemplate.queryForBeans(AppHelpHotCenterVo.class,sql,question,question);
	}
}
