package com.crunii.micro.service.appsecond.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.EnableMediumDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.MediumContentHotDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.MediumDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.MediumHotDto;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.*;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/12/4 16:38
 */
@Repository
public class AppMediumDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNPJdbcTemplate;

    public PageRsp<MediumVo> pageMedium(PageReq<MediumDto> req, String orgId,boolean superAdmin) {
        StringBuilder sb = new StringBuilder("SELECT a.medium_id,a.associated_user_id,b.org_id,b.org_name,a.medium_head_pic,c.worker_name AS associatedUserName,c.mobile AS associatedUserPhone,a.medium_state,a.attention,a.id_name FROM APP_MEDIUM a LEFT JOIN TSM_ORG b ON a.org_id =b.org_id LEFT JOIN TSM_WORKER c ON a.associated_user_id = c.login_id WHERE  del_flag = 0 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM APP_MEDIUM a LEFT JOIN TSM_ORG b ON a.org_id =b.org_id LEFT JOIN TSM_WORKER c ON a.associated_user_id = c.login_id WHERE del_flag = 0 ");
        List<Object> params = new ArrayList<>();
        MediumDto query = req.getQuery();
        if(!superAdmin){
            sb.append(" and ? in(SELECT org_id FROM TSM_ORG START WITH org_id= a.org_id CONNECT BY PRIOR parent_org_id=org_id)");
            sbCount.append(" and ? in(SELECT org_id FROM TSM_ORG START WITH org_id= a.org_id CONNECT BY PRIOR parent_org_id=org_id)");
            params.add(Long.parseLong(orgId));
        }
        if (StringUtils.hasText(query.getIdName())) {
            sb.append(" and a.id_name  like concat('%',?,'%')");
            sbCount.append(" and a.id_name  like concat('%',?,'%')");
            params.add(query.getIdName());
        }
        if (Objects.nonNull(query.getMediumState())) {
            sb.append(" and a.medium_state = ?");
            sbCount.append(" and a.medium_state = ?");
            params.add(query.getMediumState());
        }

        List<MediumVo> values;
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //添加orderBy
            sb.append(" ORDER BY a.create_time desc ");
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(MediumVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<MediumVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public Integer findIsOtherMedium(Integer associatedUserId) {
        String sql = "SELECT 1 FROM APP_MEDIUM WHERE associated_user_id = ? AND del_flag = 0";
        return jdbcTemplate.queryForRawObject(Integer.class, sql, associatedUserId);
    }

    public Integer findIsCreateMedium(Long orgId, String idName) {
        String sql = "SELECT 1 FROM APP_MEDIUM WHERE org_id = ? AND id_name = ? AND del_flag = 0";
        return jdbcTemplate.queryForRawObject(Integer.class, sql, orgId, idName);
    }

    public void updateEnableMedium(EnableMediumDto dto, Worker worker) {
        String sql = "UPDATE APP_MEDIUM SET medium_state = :mediumState,update_by= :updateId,update_time=:updateTime WHERE medium_id in(:ids)";
        Map<String, Object> params = new HashMap<>(6);
        params.put("mediumState", dto.getMediumState());
        params.put("updateId", worker.getLoginId());
        params.put("updateTime", new Date());
        params.put("ids", dto.getMediumIds());
        baseNPJdbcTemplate.update(sql, params);
    }

    public PageRsp<MediumHotVo> pageMediumHot(PageReq<MediumHotDto> req, Long orgId) {
        StringBuilder sb = new StringBuilder("SELECT a.medium_id,b.org_name,a.id_name,a.attention,(SELECT count(1) FROM APP_CONTENT_MANAGE WHERE state = 1 AND del_flag = 0 AND medium_id = a.medium_id) AS contextNumber  FROM APP_MEDIUM a LEFT JOIN TSM_ORG b ON a.org_id =b.org_id  WHERE ? in (SELECT org_id FROM TSM_ORG START WITH org_id= a.org_id CONNECT BY PRIOR parent_org_id=org_id)  AND a.del_flag = 0  ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1)  FROM APP_MEDIUM a LEFT JOIN TSM_ORG b ON a.org_id =b.org_id  WHERE ? in (SELECT org_id FROM TSM_ORG START WITH org_id= a.org_id CONNECT BY PRIOR parent_org_id=org_id) AND a.del_flag = 0 ");
        List<Object> params = new ArrayList<>();
        params.add(orgId);

        List<MediumHotVo> values;
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //添加orderBy
            sb.append(" ORDER BY a.create_time  ");
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(MediumHotVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<MediumHotVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public PageRsp<MediumContentHotVo> pageMediumContentHot(PageReq<MediumContentHotDto> req) {
        StringBuilder sb = new StringBuilder("SELECT c.org_name ,b.id_name ,a.content_title ,a.like_count ,a.collect_count ,a.read_count ,a.release_time  FROM APP_CONTENT_MANAGE a LEFT JOIN APP_MEDIUM b LEFT JOIN TSM_ORG c ON b.org_id =c.org_id  ON a.medium_id =b.medium_id WHERE a.state = 1 AND a.del_flag = 0 AND a.medium_id = ? ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM APP_CONTENT_MANAGE a LEFT JOIN APP_MEDIUM b LEFT JOIN TSM_ORG c ON b.org_id =c.org_id  ON a.medium_id =b.medium_id WHERE a.state = 1 AND a.del_flag = 0 AND a.medium_id = ?");
        List<Object> params = new ArrayList<>();
        params.add(req.getQuery().getMediumId());

        List<MediumContentHotVo> values;
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //添加orderBy
            sb.append(" ORDER BY a.create_time  ");
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(MediumContentHotVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<MediumContentHotVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public Integer validatedWorkerFindOrg(String workerOrg, Long findOrg) {
        String sql = "SELECT 1 FROM TSM_ORG  WHERE org_id = ? START WITH org_id= ? CONNECT BY PRIOR parent_org_id=org_id ";
        return jdbcTemplate.queryForRawObject(Integer.class, sql, workerOrg, findOrg);
    }

    public List<MediumHotVo> findOrgInfo(Long id) {
        String sql = "SELECT a.medium_id,b.org_name,a.id_name,a.attention,(SELECT count(1) FROM APP_CONTENT_MANAGE WHERE state = 1 AND del_flag = 0 AND medium_id = a.medium_id) AS contextNumber  FROM APP_MEDIUM a LEFT JOIN TSM_ORG b ON a.org_id =b.org_id  WHERE ? in (SELECT org_id FROM TSM_ORG START WITH org_id= a.org_id CONNECT BY PRIOR parent_org_id=org_id)  AND a.del_flag = 0   ORDER BY a.create_time  ";
        return jdbcTemplate.queryForBeans(MediumHotVo.class, sql, id);
    }

    public List<MediumContentHotVo> findMediumContent(Long mediumId) {
        String sql = "SELECT c.org_name ,b.id_name ,a.content_title ,a.like_count ,a.collect_count ,a.read_count ,a.release_time  FROM APP_CONTENT_MANAGE a LEFT JOIN APP_MEDIUM b LEFT JOIN TSM_ORG c ON b.org_id =c.org_id  ON a.medium_id =b.medium_id WHERE a.state = 1 AND a.del_flag = 0 AND a.medium_id = ?";
        return jdbcTemplate.queryForBeans(MediumContentHotVo.class,sql,mediumId);
    }

    public MediumOrgTree findTopOrg() {
        String sql = "SELECT org_id,org_name,(SELECT sum(attention)FROM APP_MEDIUM WHERE del_flag = 0) AS attention FROM TSM_ORG WHERE parent_org_id = 0 AND org_type = '1'";
        return jdbcTemplate.queryForBean(MediumOrgTree.class,sql);
    }

    public List<MediumOrgTree> findAllCityAttention() {
        String sql = "SELECT levelTwoOrg AS orgId,SUM(attention) AS attention,b.org_name FROM (SELECT (SELECT org_id FROM TSM_ORG WHERE parent_org_id = 1674703692521037822 START WITH org_id= a.org_id CONNECT BY PRIOR parent_org_id=org_id) AS levelTwoOrg,* FROM APP_MEDIUM a  WHERE del_flag = 0) temp LEFT JOIN TSM_ORG b ON temp.levelTwoOrg = b.org_id  WHERE b.org_id IS NOT NULL GROUP BY levelTwoOrg,b.org_name ";
        return jdbcTemplate.queryForBeans(MediumOrgTree.class,sql);
    }

    public Integer findMyCityAttention(Long orgId) {
        String sql = "SELECT SUM(attention) FROM APP_MEDIUM a LEFT JOIN TSM_ORG b ON a.org_id =b.org_id  WHERE a.del_flag = 0 AND ? in (SELECT org_id FROM TSM_ORG WHERE parent_org_id = 1674703692521037822 START WITH org_id= a.org_id CONNECT BY PRIOR parent_org_id=org_id)";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,orgId);
    }

    public MediumOrgTree findIsCityOrg(Long orgId) {
        String sql = "SELECT org_id,org_name FROM TSM_ORG WHERE parent_org_id = 1674703692521037822 START WITH org_id= ? CONNECT BY PRIOR parent_org_id=org_id";
        return jdbcTemplate.queryForBean(MediumOrgTree.class,sql,orgId);
    }

    public void updateAttention(Integer attention,Long mediumId) {
        String sql = "UPDATE APP_MEDIUM SET attention = ? WHERE medium_id = ?";
        jdbcTemplate.update(sql,attention,mediumId);
    }

    public List<AuditOrgUser> findAuditOrgUser(List<Long> orgIds,Integer auditRoleId) {
        String sql = "SELECT tw.worker_name as auditUserName,tw.mobile as auditUserPhone,tw.org_id  FROM TSM_WORKER_ROLE a LEFT JOIN TSM_WORKER tw ON a.login_id =tw.login_id  WHERE a.role_id = :roleId AND tw.state =1 AND tw.org_id in(:ids)";
        Map<String,Object> params = new HashMap<>(4);
        params.put("roleId",auditRoleId);
        params.put("ids",orgIds);
        return baseNPJdbcTemplate.queryForBeans(AuditOrgUser.class,sql,params);
    }

    public OrgAuthTreeVo findCqOrg(Long orgId) {
        String sql = "SELECT org_id ,org_name FROM TSM_ORG ";
        List<Object> params = new ArrayList<>();
        if(Objects.isNull(orgId)){
            sql += " WHERE org_id = 1674703692521037822";
        }else {
            sql += " WHERE org_id = ? and parent_org_id = 1674703692521037822 ";
            params.add(orgId);
        }
        return jdbcTemplate.queryForBean(OrgAuthTreeVo.class,sql,params.toArray());
    }

    public List<OrgAuthTreeVo> findCqListOrgIds(Long orgId) {
        String sql = "SELECT org_id ,org_name,1 AS isLeaf  FROM TSM_ORG WHERE parent_org_id  = ?";
        return jdbcTemplate.queryForBeans(OrgAuthTreeVo.class,sql,orgId);
    }
}
