package com.crunii.micro.service.appsecond.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.appsecond.api.vo.appmultimodules.AppMultiModulesHotVo;
import com.crunii.micro.service.appsecond.api.vo.appmultimodules.AppUserViewVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * youth-cq.
 * 热搜/热榜/点赞/收藏/关注/浏览记录
 * @author leixb
 * @since 2023-12-07 19:25
 **/
@Repository
public class AppMultiModulesDao {
    
    @Resource(name = "mainJT")
    private BaseJdbcTemplate jdbcTemplate;
    
    @Resource(name = "mainNPJT")
    private BaseNPJdbcTemplate npJdbcTemplate;
    
    public List<AppMultiModulesHotVo> findHot() {
        String sql = " SELECT a.content_manage_id,a.file_type,a.content_title,a.medium_id,a.hot_list,a.hot_search, \n"
            + "a.list_ordinal_value,a.search_ordinal_value,b.id_name,b.medium_head_pic \n"
            + "FROM APP_CONTENT_MANAGE a LEFT JOIN APP_MEDIUM b ON a.medium_id =b.medium_id \n"
            + "WHERE a.state = 1 AND a.del_flag = 0 AND (hot_list = 1 OR hot_search = 1) ";
        return jdbcTemplate.queryForBeans(AppMultiModulesHotVo.class, sql);
    }
    
    public PageRsp<AppUserViewVo> pageView(PageReq<Void> req, Long loginId) {
        String sql =
            " SELECT a.content_id, a.user_id, b.content_manage_id, b.file_type, b.content_title, b.medium_id, \n"
                + " b.like_count, b.medium_id, c.id_name, c.medium_head_pic, d.pic_key, d.file_id, \n"
                + " CASE WHEN COALESCE(auf.del_flag,1) = 1 THEN 0 ELSE 1 END interest,\n"
                + " CASE WHEN COALESCE(aul.del_flag,1) = 1 THEN 0 ELSE 1 END concurrent_like,\n"
                + " CASE WHEN CAST(COALESCE(a.update_time, a.create_time) AS DATE) = CURRENT_DATE THEN 1 \n"
                + " WHEN CAST(COALESCE(a.update_time, a.create_time) AS DATE) = CURRENT_DATE - INTERVAL '1' DAY THEN 2 \n"
                + " ELSE 3 END AS view_type FROM app_user_view a \n"
                + " LEFT JOIN app_content_manage b ON a.content_id = b.content_manage_id \n"
                + " LEFT JOIN app_medium c ON b.medium_id = c.medium_id \n"
                + " LEFT JOIN app_content_detail d ON b.content_manage_id = d.content_manage_id\n"
                + " LEFT JOIN app_user_follow auf ON a.user_id = auf.user_id AND b.medium_id = auf.account_id\n"
                + " LEFT JOIN app_user_like aul ON a.user_id = aul.user_id AND a.content_id = aul.content_id\n"
                + " WHERE a.user_id = ? AND a.del_flag = 0 AND b.state = 1 \n"
                + " AND CAST(COALESCE(a.update_time, a.create_time) AS DATE) >=  CURRENT_DATE - INTERVAL '7' DAY \n"
                + " ORDER BY view_type, COALESCE(a.update_time, a.create_time) DESC  ";
        String sqlCount =
            " SELECT count(*) FROM app_user_view a LEFT JOIN app_content_manage b ON a.content_id = b.content_manage_id \n"
                + " LEFT JOIN app_medium c ON b.medium_id = c.medium_id \n"
                + " LEFT JOIN app_content_detail d ON b.content_manage_id = d.content_manage_id\n"
                + " LEFT JOIN app_user_follow auf ON a.user_id = auf.user_id AND b.medium_id = auf.account_id\n"
                + " LEFT JOIN app_user_like aul ON a.user_id = aul.user_id AND a.content_id = aul.content_id\n"
                + " WHERE a.user_id = ? AND a.del_flag = 0 AND b.state = 1 \n"
                + " AND CAST(COALESCE(a.update_time, a.create_time) AS DATE) >=  CURRENT_DATE - INTERVAL '7' DAY  ";
        List<Object> params =new ArrayList<>();
        params.add(loginId);
        return jdbcTemplate.queryPageBase(req, sql, sqlCount, AppUserViewVo.class, params);
    }
    
    public PageRsp<AppUserViewVo> pageCollect(PageReq<Void> req, long loginId) {
        /*String sql =
            " SELECT a.content_id, a.user_id, b.content_manage_id, b.file_type, b.state, b.content_title, b.like_count, \n"
                + "b.collect_count, b.read_count, b.label_name, c.id_name, b.app_sort, b.release_time, b.hot_search \n"
                + "FROM app_user_collect a \n"
                + "LEFT JOIN app_content_manage b ON a.content_id = b.content_manage_id \n"
                + "LEFT JOIN app_medium c ON b.medium_id = c.medium_id \n"
                + "WHERE a.user_id = ? AND a.del_flag = 0 AND b.state = 1 \n"
                + "ORDER BY COALESCE(a.update_time, a.create_time) DESC  ";
        String sqlCount = " SELECT count(*) FROM app_user_collect a \n"
            + "LEFT JOIN app_content_manage b ON a.content_id = b.content_manage_id \n"
            + "LEFT JOIN app_medium c ON b.medium_id = c.medium_id \n"
            + "WHERE a.user_id = ? AND a.del_flag = 0 AND b.state = 1 ";*/
        String sql =
            " SELECT a.content_id, a.user_id, b.content_manage_id, b.file_type, b.content_title, b.medium_id, \n"
                + " b.like_count, b.medium_id, c.id_name, c.medium_head_pic, d.pic_key, d.file_id, \n"
                + " CASE WHEN COALESCE(auf.del_flag,1) = 1 THEN 0 ELSE 1 END interest,\n"
                + " CASE WHEN COALESCE(aul.del_flag,1) = 1 THEN 0 ELSE 1 END concurrent_like \n"
                + " FROM app_user_collect a \n"
                + " LEFT JOIN app_content_manage b ON a.content_id = b.content_manage_id \n"
                + " LEFT JOIN app_medium c ON b.medium_id = c.medium_id \n"
                + " LEFT JOIN app_content_detail d ON b.content_manage_id = d.content_manage_id\n"
                + " LEFT JOIN app_user_follow auf ON a.user_id = auf.user_id AND b.medium_id = auf.account_id\n"
                + " LEFT JOIN app_user_like aul ON a.user_id = aul.user_id AND a.content_id = aul.content_id\n"
                + " WHERE a.user_id = ? AND a.del_flag = 0 AND b.state = 1 \n"
                + " ORDER BY COALESCE(a.update_time, a.create_time) DESC  ";
        String sqlCount =
            " SELECT count(*) FROM app_user_collect a LEFT JOIN app_content_manage b ON a.content_id = b.content_manage_id \n"
                + " LEFT JOIN app_medium c ON b.medium_id = c.medium_id \n"
                + " LEFT JOIN app_content_detail d ON b.content_manage_id = d.content_manage_id\n"
                + " LEFT JOIN app_user_follow auf ON a.user_id = auf.user_id AND b.medium_id = auf.account_id\n"
                + " LEFT JOIN app_user_like aul ON a.user_id = aul.user_id AND a.content_id = aul.content_id\n"
                + " WHERE a.user_id = ? AND a.del_flag = 0 AND b.state = 1 \n";
        List<Object> params =new ArrayList<>();
        params.add(loginId);
        return jdbcTemplate.queryPageBase(req, sql, sqlCount, AppUserViewVo.class, params);
    }
}
