package com.crunii.micro.service.appsecond.jdbc;

import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqAppConfigUrl;
import com.crunii.micro.service.appsecond.api.bo.AppQueryPositionBo;
import com.crunii.micro.service.appsecond.api.dto.AppPersonalBasicInfoDto;
import com.crunii.micro.service.appsecond.api.dto.AppPersonalEduInfoDto;
import com.crunii.micro.service.appsecond.api.dto.AppPersonalHonorInfoDto;
import com.crunii.micro.service.appsecond.api.dto.AppPersonalInfoDto;
import com.crunii.micro.service.appsecond.api.vo.*;
import com.crunii.micro.service.sysmgr.api.domain.dto.OrgNode;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @description: 个人信息
 * @author: limy
 * @date: 2023/12/8 17:13
 **/
@Repository
public class AppPersonalInfoDao {
	@Resource(name = "mainJT")
	protected BaseJdbcTemplate jdbcTemplate;

	@Resource(name = "mainNPJT")
	protected BaseNPJdbcTemplate NPJT;

	private static final String Base_SQL="select *from YCQ_QM_USER_INFO a left join APP_PERSONAL_HONOR b on a.id=b.info_id where a.deleted = 0 and b.deleted=0";
	public List<AppPersonalBasicInfoDto> queryInfo(Long loginId) {
		StringBuilder sb = new StringBuilder(Base_SQL);
		List<Object> list = new ArrayList<>();
		if (Objects.nonNull(loginId)){
			sb.append(" and a.login_id = ? ");
			list.add(loginId);
		}
		return jdbcTemplate.queryForBeans(AppPersonalBasicInfoDto.class,sb.toString(),list.toArray());
	}

	public AppPersonalOrgInfoVo queryEchoPersonalInformation(Integer workerId){
		//String sql ="select a.*, b.league_name as leagueCodeText from TSM_WORKER a left join YCQ_LEAGUE b on a.org_id = b.league_code where a.state = 1 and a.login_id = ? ";
		//String sql ="select a.*, b.league_name as orgIdText from TSM_WORKER a left join YCQ_LEAGUE b on a.org_id = b.league_code where a.state = 1 and a.login_id = ? ";
		String sql="select a.*, b.org_name as orgIdText,b.org_id from TSM_WORKER a left join TSM_ORG b on a.org_id = b.org_id where a.state = 1 and a.login_id = ? ";
		return jdbcTemplate.queryForBean(AppPersonalOrgInfoVo.class,sql,workerId);
	}

	public List<StudentInfoByUserVo> searchStudentInfoByUser(Long userId) {
		String sql = " select student_name,identity_card_no,yysbu.id,yys.created_time,yys.last_updated_time,yys.id as student_id,yys.current_school_id,yys.current_school_name,  " +
				"yys.student_num,yysbu.in_flag,yys.current_class_id,yys.current_class_name,yys.current_grade_id,yys.current_grade_name,yys.political_status,yys.political_status_type,yys.sex  " +
				"from YCQ_YJS_STUDENT yys " +
				"left join YCQ_YJS_STUDENT_BINDING_USER yysbu on yys.id = yysbu.student_id " +
				"where yys.del_flag = 0 and yysbu.del_flag = 0 and yysbu.user_id = ?  order by yysbu.id ";
		return jdbcTemplate.queryForBeans(StudentInfoByUserVo.class, sql, userId);
	}

	/**
	 * @description: 青马的信息
	 * @author: limy
	 * @date: 2023/12/12 11:00
	 * @param: [loginId]
	 * @return: CheckUserInfoVo
	 **/
	public QmUserInfo query(Long loginId) {
		String sql = " SELECT sex,political_outlook as politic_countenance,education,\t\n" +
				"area_id as area,card_id as id_num,work_unit,create_time,update_time as last_updated_time from YCQ_QM_USER_INFO a WHERE a.deleted = 0  AND a.worker_id = ? ";
		return jdbcTemplate.queryForBean(QmUserInfo.class, sql, loginId);
	}

	/**
	 * @description: 根据学生id查询
	 * @author: limy
	 * @date: 2023/12/12 16:21
	 * @param: [id]
	 * @return: com.crunii.micro.service.appsecond.api.vo.StudentInfoByUserVo
	 **/
	public StudentInfoByUserVo selectOneStudentById(Long id) {
		String sql ="select * from YCQ_YJS_STUDENT a  where a.del_flag = 0 and a.id = ? ";
		return jdbcTemplate.queryForBean(StudentInfoByUserVo.class,sql,id);
	}

	public void updateOneStudentById(AppPersonalEduInfoDto eduInfoDto, Worker worker) {
		String sql = "update YCQ_YJS_STUDENT set current_school_id = ?, current_school_name = ?,current_grade_id = ?, current_grade_name = ?,current_class_id=?,current_class_name=?," +
				"last_updated_id=?,last_updated_time=?,last_updated_name=? where id = ? ";
		jdbcTemplate.update(sql, eduInfoDto.getCurrentSchoolId(),eduInfoDto.getCurrentSchoolName() ,
				eduInfoDto.getCurrentGradeId(),eduInfoDto.getCurrentGradeName(),
				eduInfoDto.getCurrentClassId(),eduInfoDto.getCurrentClassName(),
				BigInteger.valueOf(worker.getLoginId()),new Date(),worker.getWorkerName(),eduInfoDto.getEduId() );
	}

	public List<String> findRolewithApp(Worker worker) {
		String sql = " select a.app_code from ycq_app_config_url a left join TSM_ROLE_APP b on a.app_id = b.app_id " +
				" where a.state = 1 and b.role_id in (select role_id from TSM_WORKER_ROLE where login_id = ?) order by sort asc ";
		return jdbcTemplate.queryForRawObjects(String.class, sql,worker.getLoginId());
	}

	public List<OrgNode> getOrgChildrenByParentOrgId(Long parentOrgId) {
		String sql="SELECT distinct(a.org_id),a.*,case when (SELECT COUNT(*) FROM tsm_org b WHERE b.parent_org_id=a.org_id)>0 then FALSE ELSE TRUE end leaf "+
				" from tsm_org a " +
				" WHERE a.parent_org_id = ? ";
		return jdbcTemplate.queryForBeans(OrgNode.class, sql, parentOrgId);
	}
	//查询青马的荣誉
	public List<AppPersonalHonorVo> findWorkerHonor(Integer workerId) {
		String sql = "SELECT acquisition_time as honor_time,honor_desc FROM YCQ_QM_USER_HONOR WHERE honor_type = 1 and deleted = 0 AND worker_id = ? order by honor_time desc";
		return jdbcTemplate.queryForBeans(AppPersonalHonorVo.class, sql, workerId);
	}

	//青马的专长
	public String findExperts(Integer workerId) {
		String sql = "SELECT honor_desc FROM YCQ_QM_USER_HONOR WHERE honor_type = 2 and deleted = 0 AND worker_id = ? order by create_time desc limit 1";
		return jdbcTemplate.queryForRawObject(String.class, sql, workerId);
	}

	//逻辑删除青马的荣誉专长
	public void updateQmHonor(Integer workerId,Integer honorType) {
		String sql = "update YCQ_QM_USER_HONOR set update_time=?,updater_id=?,deleted=? where worker_id = ? and honor_type= ?  ";
		jdbcTemplate.update(sql,new Date(),workerId,1,workerId ,honorType);
	}
   //新增青马的荣誉专长
	public void createQmHonor(Long id,Integer workerId, Date honorTime, String honorDesc,Integer honorType) {
		String sql = "insert into YCQ_QM_USER_HONOR(honor_id,worker_id,acquisition_time,honor_desc,creator_id,create_time,updater_id,update_time,honor_type) values(?,?,?,?,?,?,?,?,?)";
		jdbcTemplate.update(sql,id,workerId,honorTime,honorDesc,workerId,new Date(),workerId,new Date(),honorType);
	}

	//通过workerId查询青马用户
	public Integer findInfo(Integer workerId) {
		String sql = "SELECT worker_id FROM YCQ_QM_USER_INFO WHERE deleted=0 and worker_id=?";
		return jdbcTemplate.queryForRawObject(Integer.class, sql, workerId);
	}

}
