package com.crunii.micro.service.appsecond.jdbc;

import cn.hutool.core.date.DateTime;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.PicInfo;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.AppMedium;
import com.crunii.micro.feign.client.dto.HotEsPageDto;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.ContextAppDetailVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.YouthContextVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.YouthHotPublicDetailVo;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/12/8 15:52
 */
@Repository
public class AppYouthHotDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNPJdbcTemplate;

    public PageRsp<YouthContextVo> pageYouthHot(PageReq<HotEsPageDto> req,String importantNewsLabel) {
        StringBuilder sb = new StringBuilder("SELECT a.content_manage_id,a.file_type,a.content_title,a.medium_id,b.id_name,b.medium_head_pic FROM APP_CONTENT_MANAGE a LEFT JOIN APP_MEDIUM b ON a.medium_id =b.medium_id WHERE a.state = 1 AND a.del_flag = 0 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM APP_CONTENT_MANAGE a LEFT JOIN APP_MEDIUM b ON a.medium_id =b.medium_id WHERE a.state = 1 AND a.del_flag = 0 ");
        Map<String,Object> params = new HashMap<>(4);

        if(StringUtils.hasText(req.getQuery().getLabelId())){
            sb.append(" and a.label_name like concat('%',:labelName,'%')");
            sbCount.append(" and a.label_name like concat('%',:labelName,'%')");
            params.put("labelName",req.getQuery().getLabelId());
        }else {
            sb.append(" and a.label_name <> :labelName");
            sbCount.append(" and a.label_name <> :labelName ");
            params.put("labelName",importantNewsLabel);
        }

        if(Objects.nonNull(req.getQuery().getContentManageId())){
            sb.append(" and a.content_manage_id = :contentManageId");
            sbCount.append(" and a.content_manage_id = :contentManageId");
            params.put("contentManageId",req.getQuery().getContentManageId());
        }

        List<YouthContextVo> values;
        int total = baseNPJdbcTemplate.queryForInt(sbCount.toString(), params);
        String dbType = baseNPJdbcTemplate.getDbType();
        if (total > 0) {
            //添加orderBy
            sb.append(" ORDER BY a.release_time DESC ");
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = baseNPJdbcTemplate.queryForBeans(YouthContextVo.class, sb.toString(), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<YouthContextVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public PicInfo findPicInfoByFileKey(String fileKey) {
        String sql = "SELECT object_key AS picUrl,width,height FROM YCQ_OBS_FILE WHERE object_key = ?";
        return jdbcTemplate.queryForBean(PicInfo.class,sql,fileKey);
    }

    public List<String> queryHotSearch(Integer integer,Integer searchType) {
        String sql = "SELECT search_context FROM APP_WORKER_SEARCH WHERE deleted = 0 AND worker_id = ? and search_type = ? ORDER BY create_time DESC LIMIT 8 ";
        return jdbcTemplate.queryForRawObjects(String.class,sql,integer,searchType);
    }

    public void delSearch(Integer integer) {
        String sql = "UPDATE APP_WORKER_SEARCH SET deleted = 1 WHERE worker_id = ? and search_type = 1";
        jdbcTemplate.update(sql,integer);
    }

    public Integer findLikeCount(Long contentManageId) {
        String sql = "SELECT count(1) FROM app_user_like WHERE content_id = ? AND del_flag = 0";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,contentManageId);
    }

    public List<Long> findWorkerLike(Integer loginId,List<Long> ids) {
        String sql = "SELECT content_id FROM app_user_like WHERE user_id = :userId and content_id in(:ids) AND del_flag = 0";
        Map<String,Object> params = new HashMap<>(4);
        params.put("userId",loginId);
        params.put("ids",ids);
        return  baseNPJdbcTemplate.queryForRawObjects(Long.class,sql,params);
    }

    public ContextAppDetailVo findContextManageDetail(Long contentManageId) {
        String sql = "SELECT c.medium_id,c.id_name,c.medium_head_pic,b.file_id,b.pic_key,a.content_title,b.article_context,a.label_name,(CASE WHEN  a.state = 2 THEN 1 ELSE 0 END) AS canAuth FROM APP_CONTENT_MANAGE a LEFT JOIN APP_CONTENT_DETAIL b ON a.content_manage_id =b.content_manage_id LEFT JOIN APP_MEDIUM c ON c.medium_id =a.medium_id WHERE a.del_flag = 0  AND a.content_manage_id = ? ";
        return jdbcTemplate.queryForBean(ContextAppDetailVo.class,sql,contentManageId);
    }

    public Integer findOne(Long aLong) {
        String sql = "SELECT read_count FROM APP_CONTENT_MANAGE WHERE content_manage_id = ?";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,aLong);
    }

    public void updateReadCount(Long id, int max) {
        String sql = "UPDATE APP_CONTENT_MANAGE SET read_count = ? where content_manage_id = ?";
        jdbcTemplate.update(sql,max,id);
    }

    public List<Long> findWorkerFollow(Integer loginId,List<Long> mediumId) {
        String sql = "SELECT account_id FROM app_user_follow WHERE user_id = :userId AND account_id in(:ids)  AND del_flag = 0";
        Map<String,Object> params = new HashMap<>(4);
        params.put("userId",loginId);
        params.put("ids",mediumId);
        return baseNPJdbcTemplate.queryForRawObjects(Long.class,sql,params);
    }

    public Long findMediumId(Long contextId) {
        String sql = "SELECT medium_id FROM APP_CONTENT_MANAGE WHERE content_manage_id = ?";
        return jdbcTemplate.queryForRawObject(Long.class,sql,contextId);
    }

    public Integer findCollectCount(Long contentManageId) {
        String sql = "SELECT collect_count FROM APP_CONTENT_MANAGE WHERE content_manage_id = ?";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,contentManageId);
    }

    public Integer findIsCollect(Long contentManageId,Integer loginId) {
        String sql = "SELECT 1 FROM app_user_collect WHERE del_flag = 0 AND user_id = ? AND content_id = ?";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,loginId,contentManageId);
    }


    public Float findMaxHeight(List<String> list) {
        String sql = "SELECT MIN(ROUND( convert(float,width)/height,2))  FROM YCQ_OBS_FILE  WHERE object_key in(:keys) AND height <> 0 AND height IS NOT  NULL  ";
        Map<String,Object> params = new HashMap<>(2);
        params.put("keys",list);
        return baseNPJdbcTemplate.queryForRawObject(Float.class,sql,params);
    }

    public List<String> queryApplyHotSearch(Integer searchType) {
        String sql = "select search_context from APP_WORKER_SEARCH where search_type=? group by search_context order by count(1) desc limit 5 ";
        return jdbcTemplate.queryForRawObjects(String.class,sql,searchType);
    }

    public Long getImportantNewsLabel() {
        String sql = "SELECT label_id FROM TSM_LABEL_MANAGEMENT WHERE label_name  = '要闻'";
        return jdbcTemplate.queryForRawObject(Long.class,sql);
    }

    public List<YouthContextVo> findImportTop(String importantNewsLabel) {
        String sql = "SELECT a.content_manage_id,a.file_type,a.content_title,a.medium_id,b.id_name,b.medium_head_pic,TRUE AS isImportantNews FROM APP_CONTENT_MANAGE a LEFT JOIN APP_MEDIUM b ON a.medium_id =b.medium_id WHERE a.state = 1 AND a.del_flag = 0 AND a.label_name = ? AND a.app_sort = 1 ORDER BY a.release_time DESC  LIMIT 2";
        return jdbcTemplate.queryForBeans(YouthContextVo.class,sql,importantNewsLabel);
    }

    public YouthHotPublicDetailVo queryYouthHot(Long contentManageId,String importantNewsLabel) {
        String sql = "SELECT a.content_manage_id,a.content_title,a.file_type,b.article_context,(CASE WHEN label_name = ? THEN 1 ELSE 0 END ) AS isImportantNews,(CASE WHEN state = 2 THEN 1 ELSE 0 end) AS canAuth FROM APP_CONTENT_MANAGE a LEFT JOIN APP_CONTENT_DETAIL b ON a.content_manage_id =b.content_manage_id WHERE a.content_manage_id = ?  AND a.del_flag = 0";
        return jdbcTemplate.queryForBean(YouthHotPublicDetailVo.class,sql,importantNewsLabel,contentManageId);
    }

    public Worker findWorkerInfo(Integer workerId) {
        String sql = "SELECT tw.login_id ,tw.worker_name ,to2.org_id ,to2.org_name FROM TSM_WORKER tw LEFT JOIN TSM_ORG to2 ON tw.org_id =to2.org_id  WHERE login_id =?\n";
        return jdbcTemplate.queryForBean(Worker.class,sql,workerId);
    }
}
