package com.crunii.micro.service.appsecond.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.SpaceApplyTeamReqDTO;
import com.crunii.micro.service.appsecond.api.vo.youthspace.SpaceApplyTeamPageVO;
import com.crunii.micro.service.appsecond.enums.YouthSpaceEnums.CommEnum;
import com.google.common.base.Joiner;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import org.springframework.stereotype.Repository;

/**
 * @Author thw
 * @date 2023/12/15
 * @description
 */
@Repository
public class SpaceApplyTeamDao {

  @Resource(name = "mainJT")
  protected BaseJdbcTemplate jdbcTemplate;

  @Resource(name = "mainNPJT")
  protected BaseNPJdbcTemplate baseNPJdbcTemplate;

  // 空间申请页面的列表数据只有来自app申请的数据，不展示web页面直接新增的数据
  private static final String PAGE_BASE_SQL = "select a.*, b.space_id, h.space_name  from qczd_team_stay a "
      + "inner join qczd_space_team b on a.id = b.team_id and b.del_flag = 0 and a.del_flag = 0 "
      + "inner join qczd_space_content h on b.space_id = h.id and h.del_flag = 0 "
      + " where a.del_flag = 0 and a.create_point = 1 ";

  private static final String PAGE_COUNT_SQL = "select count(*)  from qczd_team_stay a "
      + "inner join qczd_space_team b on a.id = b.team_id and b.del_flag = 0 and a.del_flag = 0 "
      + "inner join qczd_space_content h on b.space_id = h.id and h.del_flag = 0 "
      + " where a.del_flag = 0 and a.create_point = 1 ";
  @InjectProperty
  public PageRsp<SpaceApplyTeamPageVO> pageApplyTeam(PageReq<SpaceApplyTeamReqDTO> req,
      Worker currWorker,Boolean flag) {
    StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
    StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
    // query对象
    SpaceApplyTeamReqDTO query = req.getQuery();
    List<Object> params = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(query)) {
      if (flag){
        // 权限:只能查看向本空间申请的团队信息
        if (ObjectUtil.isNotEmpty(currWorker.getLoginId())) {
          sb.append(" and h.create_id = ?  ");
          sbCount.append(" and h.create_id = ?  ");
          params.add(currWorker.getLoginId());
        }
      }

      if (ObjectUtil.isNotEmpty(query.getId())) {
        Set<Long> idset = query.getId().stream().map(e -> Long.valueOf(e))
            .collect(Collectors.toSet());
        String idStr = Joiner.on(",").join(idset);
        sb.append(" and a.id in (" + idStr + ") ");
        sbCount.append(" and a.id in (" + idStr + ") ");
      }
      if (ObjectUtil.isNotEmpty(query.getApplicantName())) {
        sb.append(" and a.applicant_name like CONCAT('%',?,'%')  ");
        sbCount.append(" and a.applicant_name like CONCAT('%',?,'%')  ");
        params.add(query.getApplicantName());
      }
      if (ObjectUtil.isNotEmpty(query.getPhone())) {
        sb.append(" and a.phone like CONCAT('%',?,'%') ");
        sbCount.append(" and a.phone like CONCAT('%',?,'%')  ");
        params.add(query.getPhone());
      }

      if (ObjectUtil.isNotEmpty(query.getSpaceName())) {
        sb.append(" and h.space_name like CONCAT('%',?,'%') ");
        sbCount.append(" and h.space_name like CONCAT('%',?,'%')  ");
        params.add(query.getSpaceName());
      }


      if (ObjectUtil.isNotEmpty(query.getAuditState())) {
        sb.append(" and a.audit_state = ?  ");
        sbCount.append(" and a.audit_state = ? ");
        params.add(query.getAuditState());
      }

/*      if (ObjectUtil.isNotEmpty(query.getStayState())) {
        List<Integer> needList = query.getStayState();
        if (needList.containsAll(Arrays.asList(CommEnum.ZERO_COM.getNum(),CommEnum.ONE_COM.getNum()))){
          sb.append(" and a.work_need = ? and a.office_need = ? ");
          sbCount.append(" and a.work_need = ? and a.office_need = ? ");
          params.add(CommEnum.ONE_COM.getNum());
          params.add(CommEnum.ONE_COM.getNum());
        }else if (needList.contains(CommEnum.ZERO_COM.getNum())){
          sb.append(" and a.work_need = ? ");
          sbCount.append(" and a.work_need = ? ");
          params.add(CommEnum.ONE_COM.getNum());
        }else if (needList.contains(CommEnum.ONE_COM.getNum())){
          sb.append(" and a.office_need = ? ");
          sbCount.append(" and a.office_need = ? ");
          params.add(CommEnum.ONE_COM.getNum());
        }
      }*/
      if (ObjectUtil.isNotEmpty(query.getStayState())) {
        if (CommEnum.ZERO_COM.getNum().equals(query.getStayState())){
          sb.append(" and a.work_need = ? ");
          sbCount.append(" and a.work_need = ? ");
          params.add(CommEnum.ONE_COM.getNum());
        }else if (CommEnum.ONE_COM.getNum().equals(query.getStayState())){
          sb.append(" and a.office_need = ? ");
          sbCount.append(" and a.office_need = ? ");
          params.add(CommEnum.ONE_COM.getNum());
        }
      }



      if (ObjectUtil.isNotEmpty(query.getBusinessLicense())) {
        sb.append(" and a.business_license = ? ");
        sbCount.append(" and a.business_license = ? ");
        params.add(Integer.valueOf(query.getBusinessLicense()));
      }

      if (ObjectUtil.isNotEmpty(query.getStartTime())) {
        sb.append(" and a.apply_time >= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        sbCount.append(" and a.apply_time >= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        params.add(query.getStartTime() + " 00:00:00");
      }
      if (ObjectUtil.isNotEmpty(query.getEndTime())) {
        sb.append(" and a.apply_time <= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        sbCount.append(" and a.apply_time <= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        params.add(query.getEndTime() + " 23:59:59");
      }

    }

    sb.append(" order by created_time desc");

    int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
    List<SpaceApplyTeamPageVO> values;
    String dbType = jdbcTemplate.getDbType();
    if (total > 0) {
      //根据数据库类型设置limit
      SqlUtil.appendLimit(dbType, params, sb, req);
      values = jdbcTemplate.queryForBeans(SpaceApplyTeamPageVO.class, sb.toString(), params.toArray());
    } else {
      values = new ArrayList<>(0);
    }
    PageRsp<SpaceApplyTeamPageVO> page = new PageRsp<>();
    page.setPageNo(req.getPageNo());
    page.setPageSize(req.getPageSize());
    page.setTotalNum(total);
    page.setResult(values);
    // 为手机计算是否由下一页
    page.next();

    return page;
  }
}
