package com.crunii.micro.service.appsecond.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.QczdSpaceTeam;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.SpacePassTeamPageReqDTO;
import com.crunii.micro.service.appsecond.api.vo.youthspace.SpacePassTeamPageVO;
import com.google.common.base.Joiner;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import org.springframework.stereotype.Repository;

/**
 * @Author thw
 * @date 2023/12/13
 * @description 入住空间的团队Dao
 */
@Repository
public class SpacePassTeamDao {

  @Resource(name = "mainJT")
  protected BaseJdbcTemplate jdbcTemplate;

  @Resource(name = "mainNPJT")
  protected BaseNPJdbcTemplate baseNPJdbcTemplate;

  private static final String PAGE_BASE_SQL = "select a.*,b.space_id from  qczd_team_stay a   "
      + " left join qczd_space_team b on a.id = b.team_id and b.del_flag = 0 "
      + " where a.audit_state = 2 and a.del_flag = 0  ";

  private static final String PAGE_COUNT_SQL = "select count(*) from  qczd_team_stay a "
      + "left join qczd_space_team b on a.id = b.team_id and b.del_flag = 0 "
      + "where a.audit_state = 2 and a.del_flag = 0  ";

  @InjectProperty
  public PageRsp<SpacePassTeamPageVO> pageTeamInfo(PageReq<SpacePassTeamPageReqDTO> req) {
    StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
    StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
    // query对象
    SpacePassTeamPageReqDTO query = req.getQuery();
    List<Object> params = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(query)) {
      // 必传参数spaceId
      if (ObjectUtil.isNotEmpty(query.getSpaceId())) {
        sb.append(" and b.space_id = ? ");
        sbCount.append(" and b.space_id = ? ");
        params.add(Long.valueOf(query.getSpaceId()));
      }
      if (ObjectUtil.isNotEmpty(query.getId())) {
        Set<Long> idset = query.getId().stream().map(e -> Long.valueOf(e))
            .collect(Collectors.toSet());
        String idStr = Joiner.on(",").join(idset);
        sb.append(" and a.id in (" + idStr + ") ");
        sbCount.append(" and a.id in (" + idStr + ") ");
      }
      if (ObjectUtil.isNotEmpty(query.getApplicantName())) {
        sb.append(" and a.applicant_name like CONCAT('%',?,'%')  ");
        sbCount.append(" and a.applicant_name like CONCAT('%',?,'%')  ");
        params.add(query.getApplicantName());
      }
      if (ObjectUtil.isNotEmpty(query.getPhone())) {
        sb.append(" and a.phone like CONCAT('%',?,'%')  ");
        sbCount.append(" and a.phone like CONCAT('%',?,'%')  ");
        params.add(query.getPhone());
      }
      if (ObjectUtil.isNotEmpty(query.getCompanyName())) {
        sb.append(" and a.company_name like CONCAT('%',?,'%')  ");
        sbCount.append(" and a.company_name like CONCAT('%',?,'%')  ");
        params.add(query.getCompanyName());
      }

      if (ObjectUtil.isNotEmpty(query.getSocialCode())) {
        sb.append(" and a.social_code like CONCAT('%',?,'%')  ");
        sbCount.append(" and a.social_code like CONCAT('%',?,'%')  ");
        params.add(query.getSocialCode());
      }

      if (ObjectUtil.isNotEmpty(query.getPhone())) {
        sb.append(" and a.phone like CONCAT('%',?,'%') ");
        sbCount.append(" and a.phone like CONCAT('%',?,'%')  ");
        params.add(query.getPhone());
      }

      if (ObjectUtil.isNotEmpty(query.getStartTime())) {
        sb.append(" and a.settled_time >= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        sbCount.append(" and a.settled_time >= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        params.add(query.getStartTime() + " 00:00:00");
      }
      if (ObjectUtil.isNotEmpty(query.getEndTime())) {
        sb.append(" and a.settled_time <= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        sbCount.append(" and a.settled_time <= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        params.add(query.getEndTime() + " 23:59:59");
      }

    }
    sb.append(" order by created_time desc");
    int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
    List<SpacePassTeamPageVO> values;
    String dbType = jdbcTemplate.getDbType();
    if (total > 0) {
      //根据数据库类型设置limit
      SqlUtil.appendLimit(dbType, params, sb, req);
      values = jdbcTemplate
          .queryForBeans(SpacePassTeamPageVO.class, sb.toString(), params.toArray());
    } else {
      values = new ArrayList<>(0);
    }
    PageRsp<SpacePassTeamPageVO> page = new PageRsp<>();
    page.setPageNo(req.getPageNo());
    page.setPageSize(req.getPageSize());
    page.setTotalNum(total);
    page.setResult(values);
    // 为手机计算是否由下一页
    page.next();

    return page;
  }


  /** 查询审核通过的团队信息 = 已入住空间的团队 */
  public List<QczdSpaceTeam> findBySpaceIdAndAuditState(Long spaceId) {
    StringBuilder sb = new StringBuilder("select * from  qczd_team_stay a  "
        + " left join qczd_space_team b on a.id = b.team_id and b.del_flag = 0 and a.del_flag = 0 "
        + " where a.audit_state = 2 and a.team_state = 0");
    // query对象
    List<Object> params = new ArrayList<>();
    // 必传参数spaceId
    if (ObjectUtil.isNotEmpty(spaceId)) {
      sb.append(" and b.space_id = ? ");
      params.add(spaceId);
    }
    return jdbcTemplate.queryForBeans(QczdSpaceTeam.class,sb.toString(),params.toArray());
  }
}
