package com.crunii.micro.service.appsecond.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.YouthSpacePageReqDTO;
import com.crunii.micro.service.appsecond.api.vo.youthspace.YouthSpacePageVO;
import com.google.common.base.Joiner;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import org.springframework.stereotype.Repository;

/**
 * @Author thw
 * @date 2023/12/12
 * @description 青创空间管理
 */
@Repository
public class YouthSpaceJdbc {

  @Resource(name = "mainJT")
  protected BaseJdbcTemplate jdbcTemplate;

  @Resource(name = "mainNPJT")
  protected BaseNPJdbcTemplate baseNPJdbcTemplate;

  private static final String PAGE_BASE_SQL = "select * from  qczd_space_content a where del_flag = 0  ";

  private static final String PAGE_COUNT_SQL = "select count(*) from qczd_space_content a where del_flag = 0  ";

  @InjectProperty
  public PageRsp<YouthSpacePageVO> pageSpaceInfo(PageReq<YouthSpacePageReqDTO> req,Boolean flag,
      Worker currWorker) {

    StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
    StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
    // query对象
    YouthSpacePageReqDTO query = req.getQuery();
    List<Object> params = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(query)) {
      // 权限控制
      if (flag) {
        sb.append(" and a.create_id = ? ");
        sbCount.append(" and a.create_id = ?");
        params.add(currWorker.getLoginId());
      }

      if (ObjectUtil.isNotEmpty(query.getId())) {
        Set<Long> idset = query.getId().stream().map(e -> Long.valueOf(e))
            .collect(Collectors.toSet());
        String idStr = Joiner.on(",").join(idset);
        sb.append(" and a.id in (" + idStr + ") ");
        sbCount.append(" and a.id in (" + idStr + ") ");
      }
      if (ObjectUtil.isNotEmpty(query.getSpaceName())) {
        sb.append(" and a.space_name like CONCAT('%',?,'%') ");
        sbCount.append(" and a.space_name like CONCAT('%',?,'%') ");
        params.add(query.getSpaceName());
      }
      if (ObjectUtil.isNotEmpty(query.getContactsName())) {
        sb.append(" and a.contacts_name like CONCAT('%',?,'%')  ");
        sbCount.append(" and a.contacts_name like CONCAT('%',?,'%')  ");
        params.add(query.getContactsName());
      }
      if (ObjectUtil.isNotEmpty(query.getPhone())) {
        sb.append(" and a.phone like CONCAT('%',?,'%')  ");
        sbCount.append(" and a.phone like CONCAT('%',?,'%')  ");
        params.add(query.getPhone());
      }
      if (ObjectUtil.isNotEmpty(query.getFullAddress())) {
        sb.append(" and a.full_address like CONCAT('%',?,'%')  ");
        sbCount.append(" and a.full_address like CONCAT('%',?,'%')  ");
        params.add(query.getFullAddress());
      }

      if (ObjectUtil.isNotEmpty(query.getEvaluateState())) {
        sb.append(" and a.evaluate_state = ? ");
        sbCount.append(" and a.evaluate_state = ? ");
        params.add(Integer.valueOf(query.getEvaluateState()));
      }

      if (ObjectUtil.isNotEmpty(query.getCreatedBy())) {
        sb.append(" and a.created_by like CONCAT('%',?,'%')  ");
        sbCount.append(" and a.created_by like CONCAT('%',?,'%') ");
        params.add(query.getCreatedBy());
      }

      if (ObjectUtil.isNotEmpty(query.getStartTime())) {
        sb.append(" and a.updated_time >= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        sbCount.append(" and a.updated_time >= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        params.add(query.getStartTime() + " 00:00:00");
      }
      if (ObjectUtil.isNotEmpty(query.getEndTime())) {
        sb.append(" and a.updated_time <= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        sbCount.append(" and a.updated_time <= TO_TIMESTAMP( ? , 'YYYY-MM-DD HH24:MI:SS' ) ");
        params.add(query.getEndTime() + " 23:59:59");
      }

    }
    sb.append(" order by created_time desc");
    int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
    List<YouthSpacePageVO> values;
    String dbType = jdbcTemplate.getDbType();
    if (total > 0) {
      //根据数据库类型设置limit
      SqlUtil.appendLimit(dbType, params, sb, req);
      values = jdbcTemplate
          .queryForBeans(YouthSpacePageVO.class, sb.toString(), params.toArray());
    } else {
      values = new ArrayList<>(0);
    }
    PageRsp<YouthSpacePageVO> page = new PageRsp<>();
    page.setPageNo(req.getPageNo());
    page.setPageSize(req.getPageSize());
    page.setTotalNum(total);
    page.setResult(values);
    // 为手机计算是否由下一页
    page.next();

    return page;
  }

}
