package com.crunii.micro.service.appsecond.readListener;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.metadata.CellData;
import com.alibaba.excel.util.ListUtils;
import com.alibaba.fastjson.JSON;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.BeanUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.QczdTeamStay;
import com.crunii.micro.domain.entity.main.QczdTeamStayTemp;
import com.crunii.micro.domain.entity.main.TsmPropertyValue;
import com.crunii.micro.service.appsecond.api.dto.data.ExcelMistakeData;
import com.crunii.micro.service.appsecond.api.dto.data.MistakeDataInfo;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.SpacePassTeamCUExportDTO;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.SpacePassTeamCUReqDTO;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.BusinessStateEnum;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.DemandEnum;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.SexEnum;
import com.crunii.micro.service.appsecond.enums.YouthSpaceEnums.CommEnum;
import com.crunii.micro.service.appsecond.service.impl.SpacePassTeamServiceImpl;
import com.google.common.base.Joiner;
import java.util.*;
import java.util.Map.Entry;
import java.util.stream.Collectors;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;

/**
 * @Author thw
 * @date 2023/12/14
 * @description
 */
@Slf4j
public class ImportTeamReadListenerTwo extends AnalysisEventListener<SpacePassTeamCUExportDTO> {

  /**
   * 每隔5条存储数据库，实际使用中可以100条，然后清理list ，方便内存回收
   */
  private static final int BATCH_COUNT = 600;
  /**
   * 缓存的数据(业务规定：单个文件数据量不允许超过500条。)
   */
  private List<QczdTeamStay> cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
  private List<QczdTeamStayTemp> cachedDataTempList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);

  private SpacePassTeamServiceImpl service;
  private Worker worker;

  /** 存放相关静态值 */
  private HashMap<String, List<TsmPropertyValue>> tsmProValueMap;

  /**
   * 表头属性名
   */
  private List<String> includeFieldList;

  // 导入的字段name和表头值
  private LinkedHashMap<String, String> headeValueMap;
  private String dataKey;

  /**
   * 错误的数据
   */
  List<MistakeDataInfo> errContentList;

  private Integer countNum = 0;
  private Integer headNum = 0;

  // 是否读取到数据的标志
  private boolean hasData = false;
//  private boolean errTableHeadFlag = true;


  public ImportTeamReadListenerTwo(SpacePassTeamServiceImpl service, List<String> includeFieldList,
      Worker worker, List<MistakeDataInfo> errContentList,HashMap<String, List<TsmPropertyValue>> tsmProValueMap,
      LinkedHashMap<String, String> headeValueMap,String dataKey) {
    this.service = service;
    this.includeFieldList = includeFieldList;
    this.worker = worker;
    this.errContentList = errContentList;
    this.tsmProValueMap = tsmProValueMap;
    this.headeValueMap = headeValueMap;
    this.dataKey = dataKey;
  }

  @Override
  public void invokeHead(Map<Integer, CellData> headMap, AnalysisContext context) {

    List<String> tableHeadContent = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(headeValueMap)){
      // 使用迭代器遍历 Map
      Iterator<Map.Entry<String, String>> iterator = headeValueMap.entrySet().iterator();
      while (iterator.hasNext()) {
        Map.Entry<String, String> entry = iterator.next();
        if (includeFieldList.contains(entry.getKey())) {
          // 添加表头值
          tableHeadContent.add(entry.getValue());
        }else {
          // 使用迭代器的 remove 方法删除元素
          iterator.remove();
        }
      }
    }
      // headRowNumber(2)时，easyexcel会认为1、2两行都是表头，所以这里取消比较
    ++headNum;
    if (ObjectUtil.equals(headNum, 2)){
      for (Integer key : headMap.keySet()) {
        CellData data = headMap.get(key);
        // 由于只存在一行表头
        if (ObjectUtil.isNotEmpty(tableHeadContent)){
          if (!tableHeadContent.contains(data.getStringValue())) {
            throw new BusinessException("导入模板错误!");
          }
        }
      }
    }
  }


  @Override
  public void invoke(SpacePassTeamCUExportDTO dto, AnalysisContext context) {

    if (cachedDataList.size() > 500){
      throw new BusinessException("单个文件数据量不允许超过500条.");
    }
    log.info("解析到第" + countNum++ + "条数据:{}", JSON.toJSONString(dto));
    // 保存每一行的错误数据信息
    MistakeDataInfo rowMistakeData = new MistakeDataInfo();

    LinkedHashMap<String, ExcelMistakeData> rowMap = new LinkedHashMap<>();
    if (ObjectUtil.isNotEmpty(headeValueMap)){
//      if (errTableHeadFlag){
        // 第一次创建默认第一行的列数并带表头
        rowMap.put("countNum",service.createRightCellContent("行数",countNum.toString(),null));
        for (Entry<String, String> entry : headeValueMap.entrySet()) {
          // 通过属性名与错误对象对应
          // 反射获取值
          Object fieldValue = BeanUtil.getFieldValue(dto, entry.getKey());
          ExcelMistakeData cellMistakaConten = null;
          if (ObjectUtil.isNotEmpty(fieldValue)){
            cellMistakaConten = service.createRightCellContent(entry.getValue(), fieldValue.toString(),
                null);
          }else {
            cellMistakaConten = service.createRightCellContent(entry.getValue(), null,
                null);
          }
          if (ObjectUtil.isNotEmpty(cellMistakaConten)){
            rowMap.put(entry.getKey(),cellMistakaConten);
          }
        }
/*
        rowMap.put("tableConten",service.createRightCellContent("表格本身数据重复","",null));
        rowMap.put("tableAndDatabase",service.createRightCellContent("表格和数据库数据重复","",null));
        rowMap.put("tableAge",service.createRightCellContent("申请年龄不满足要求","",null));
        errTableHeadFlag = false;
      }else {
        // 后续创建默认其他行的的列数不带表头
        rowMap.put("countNum",service.createRightCellContent("",countNum.toString(),null));
        for (Entry<String, String> entry : headeValueMap.entrySet()) {
          // 通过属性名与错误对象对应
          ExcelMistakeData cellMistakaConten = service.createRightCellContent("", "",
              null);
          rowMap.put(entry.getKey(),cellMistakaConten);
        }

        rowMap.put("tableConten",service.createRightCellContent("","",null));
        rowMap.put("tableAndDatabase",service.createRightCellContent("","",null));
        rowMap.put("tableAge",service.createRightCellContent("","",null));
      }*/
    }


    // 参数校验
    checkExportDateContent(dto, countNum ,rowMap);

    // 校验数据是否重复（表格本身数据重复）
    if (ObjectUtil.isNotEmpty(cachedDataList)) {
      Iterator<QczdTeamStay> teamStayIterator = cachedDataList.iterator();
      while (teamStayIterator.hasNext()) {
        QczdTeamStay qczdTeamStay = teamStayIterator.next();
        if (ObjectUtil.isNotEmpty(dto.getApplicantName()) && ObjectUtil.isNotEmpty(dto.getPhone())
            && dto.getApplicantName().equals(qczdTeamStay.getApplicantName()) && dto.getPhone()
            .equals(qczdTeamStay.getPhone())) {

//          rowMap.put("tableConten",service.createMistakeCellContent("表格本身数据重复","","数据参数校验失败: {" +
//              "申请人姓名与联系人电话:[" + dto.getApplicantName() + "-" + dto.getPhone()
//              + "]在表中重复! }"));

//          service.doWithMistakeErrMessage(rowMap,"tableConten",dto.getApplicantName() + "-" + dto.getPhone(),"数据参数校验失败: {" +
//              "申请人姓名与联系人电话:[" + dto.getApplicantName() + "-" + dto.getPhone()
//              + "]在表中重复! }");
          service.doWithMistakeErrMessage(rowMap, "applicantName", dto.getApplicantName(),
              "表格内容中姓名重复!");
          service.doWithMistakeErrMessage(rowMap, "phone", dto.getPhone(),
              "表格内容中联系人电话重复!");

        }
      }
    }

    // 校验数据是否重复（表格和数据库数据重复）
    List<QczdTeamStay> existTeams = service.qczdTeamStayRepo.findByApplicantNameAndPhone(
        dto.getApplicantName(), dto.getPhone());
    if (ObjectUtil.isNotEmpty(existTeams)) {
//      rowMap.put("tableAndDatabase",service.createMistakeCellContent("表格和数据库数据重复","","数据参数校验失败: {" + "申请人姓名与联系人电话:[" + dto.getApplicantName() + "-" + dto.getPhone()
//          + "]在数据库中已存在!"));

//      service.doWithMistakeErrMessage(rowMap,"tableAndDatabase",dto.getApplicantName() + "-" + dto.getPhone(),"数据参数校验失败: {" + "申请人姓名与联系人电话:[" + dto.getApplicantName() + "-" + dto.getPhone()
//          + "]在数据库中已存在!");
      service.doWithMistakeErrMessage(rowMap, "applicantName", dto.getApplicantName(),
          "表格和数据库中姓名重复!");
      service.doWithMistakeErrMessage(rowMap, "phone", dto.getPhone(),
          "表格和数据库中联系人电话重复!");
    }


    // 最终获取每一行的数据内容
    List<ExcelMistakeData> cellMistakeData = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(rowMap)){
      for (Entry<String, ExcelMistakeData> entry : rowMap.entrySet()) {
        ExcelMistakeData excelMistakeData = entry.getValue();
        if (ObjectUtil.isNotEmpty(excelMistakeData)) {
          cellMistakeData.add(excelMistakeData);
        }
      }
    }
    rowMistakeData.setMistakeDataInfo(cellMistakeData);
    // 保存每一行的错误数据
    errContentList.add(rowMistakeData);


    QczdTeamStay qczdTeamStay = new QczdTeamStay();
    BeanUtils.copyProperties(dto, qczdTeamStay);
    // 特殊属性处理
    BusinessStateEnum businessStateEnumDesc = BusinessStateEnum.getBusinessStateEnumDesc(
        dto.getBusinessLicenseName());
    if (ObjectUtil.isNotEmpty(businessStateEnumDesc)){
      qczdTeamStay.setBusinessLicense(businessStateEnumDesc.getNum());
    }
    SexEnum sexEnumDesc = SexEnum.getSexEnumDesc(dto.getSexName());
    if (ObjectUtil.isNotEmpty(sexEnumDesc)){
      qczdTeamStay.setSex(sexEnumDesc.getNum().toString());
    }
    DemandEnum workerNeedName = DemandEnum.getDemandEnumDesc(dto.getWorkNeedName());
    if (ObjectUtil.isNotEmpty(workerNeedName)){
      qczdTeamStay.setWorkNeed(workerNeedName.getNum());
    }
    DemandEnum officeNeedName = DemandEnum.getDemandEnumDesc(dto.getOfficeNeedName());
    if (ObjectUtil.isNotEmpty(officeNeedName)){
      qczdTeamStay.setOfficeNeed(officeNeedName.getNum());
    }


    // 相关静态值
    // 民族
    List<TsmPropertyValue> Nationas = tsmProValueMap.get(service.propCode[0]);
    // 民族静态值回显
    if (ObjectUtil.isNotEmpty(Nationas)){
      for (TsmPropertyValue Nationa : Nationas) {
        if (Objects.equals(Nationa.getValueName(), dto.getNationName())){
          qczdTeamStay.setNation(Nationa.getValueCode());
          break;
        }
      }
    }

    // 学历
    List<TsmPropertyValue> SchoolingNames = tsmProValueMap.get(service.propCode[1]);
    // 民族静态值回显
    if (ObjectUtil.isNotEmpty(SchoolingNames)){
      for (TsmPropertyValue School : SchoolingNames) {
        if (Objects.equals(School.getValueName(), dto.getFormalSchoolingName())){
          qczdTeamStay.setFormalSchooling(School.getValueCode());
          break;
        }
      }
    }

    // 政治面貌
    List<TsmPropertyValue> PoliticalStatusNames = tsmProValueMap.get(service.propCode[2]);
    // 政治面貌静态值回显
    if (ObjectUtil.isNotEmpty(PoliticalStatusNames)){
      for (TsmPropertyValue status : PoliticalStatusNames) {
        if (Objects.equals(status.getValueName(), dto.getPoliticalStatusName())){
          qczdTeamStay.setPoliticalStatus(status.getValueCode());
          break;
        }
      }
    }
    // 行业领域
    List<TsmPropertyValue> IndustryFields = tsmProValueMap.get(service.propCode[3]);
    // 政治面貌静态值回显
    List<String> fieldCodeList = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(dto.getIndustryField()) && ObjectUtil.isNotEmpty(IndustryFields)){
      String[] IndustryFieldArr = dto.getIndustryField().split(",");
      if (ObjectUtil.isNotEmpty(IndustryFieldArr) ){
        for (String field : IndustryFieldArr) {
          for (TsmPropertyValue industryField : IndustryFields) {
            if (ObjectUtil.equals(field,industryField.getValueName())){
              fieldCodeList.add(industryField.getValueCode());
            }
          }
        }
      }
      String fieldCodeStr = Joiner.on(",").join(fieldCodeList);
      qczdTeamStay.setIndustryField(fieldCodeStr);
    }

    // 设置新增时团队实例必要属性
    service.createTeamNeedSetMethod(dto.getIdCode(), qczdTeamStay, worker,rowMap,Boolean.FALSE);

    cachedDataList.add(qczdTeamStay);
    QczdTeamStayTemp qczdTeamStayTemp = new QczdTeamStayTemp();
    qczdTeamStayTemp.setDataKey(dataKey);
    BeanUtil.copyPropertiesIgnoreNull(qczdTeamStay,qczdTeamStayTemp);
    cachedDataTempList.add(qczdTeamStayTemp);
    // 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
    if (cachedDataList.size() >= BATCH_COUNT) {
      boolean flag = saveTeamData();
      if (flag){
        // 存储完成清理 list
        cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
        cachedDataTempList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
      }
    }
    // 读取到数据，将标志设置为 true
    hasData = true;
  }

  /**
   * 参数校验
   */
  public void checkExportDateContent(SpacePassTeamCUExportDTO bo, Integer countNum,Map<String, ExcelMistakeData> rowMap) {

    if (ObjectUtil.isEmpty(BusinessStateEnum.getBusinessStateEnumDesc(bo.getBusinessLicenseName()))) {
      service.doWithMistakeErrMessage(rowMap,"businessLicenseName",bo.getBusinessLicenseName(),"营业执照必填!");
    }

    if (ObjectUtil.isNotEmpty(bo.getBusinessLicenseName()) && BusinessStateEnum.YES_BUSINESS.getDesc()
        .equals(bo.getBusinessLicenseName())) {
      if (ObjectUtil.isEmpty(bo.getCompanyName()) || bo.getCompanyName().length() > 50) {
        service.doWithMistakeErrMessage(rowMap,"companyName",bo.getCompanyName(),"有营业执照时,企业名称必填且字符长度不能超过50字符.");
      }
    }

    if (ObjectUtil.isEmpty(DemandEnum.getDemandEnumDesc(bo.getWorkNeedName()))) {
      service.doWithMistakeErrMessage(rowMap,"workNeedName",bo.getWorkNeedName(),"工位需求必填!");
    }

    if (ObjectUtil.isEmpty(DemandEnum.getDemandEnumDesc(bo.getOfficeNeedName()))) {
      service.doWithMistakeErrMessage(rowMap,"officeNeedName",bo.getOfficeNeedName(),"办公室需求必填!");
    }

    if (ObjectUtil.isEmpty(SexEnum.getSexEnumDesc(bo.getSexName()))) {
      service.doWithMistakeErrMessage(rowMap,"sexName",bo.getSexName(),"性别必填!");
    }

    List<String> NationNames = tsmProValueMap.get(service.propCode[0]).stream()
        .map(TsmPropertyValue::getValueName).collect(
            Collectors.toList());
    if (ObjectUtil.isEmpty(bo.getNationName()) || !NationNames.contains(bo.getNationName())) {
      service.doWithMistakeErrMessage(rowMap,"nationName",bo.getNationName(),"民族必填且必须满足包含在民族的静态值名称列表中!");
    }

    List<String> SchoolingNames = tsmProValueMap.get(service.propCode[1]).stream()
        .map(TsmPropertyValue::getValueName).collect(
            Collectors.toList());
    if (ObjectUtil.isEmpty(bo.getFormalSchoolingName()) || !SchoolingNames.contains(bo.getFormalSchoolingName()) ) {
      service.doWithMistakeErrMessage(rowMap,"formalSchoolingName",bo.getFormalSchoolingName(),"学历必填且必须满足包含在学历的静态值名称列表中!");
    }

    List<String> PoliticalStatusNames = tsmProValueMap.get(service.propCode[2]).stream()
        .map(TsmPropertyValue::getValueName).collect(
            Collectors.toList());
    if (ObjectUtil.isEmpty(bo.getPoliticalStatusName()) || !PoliticalStatusNames.contains(bo.getPoliticalStatusName())) {
      service.doWithMistakeErrMessage(rowMap,"politicalStatusName",bo.getPoliticalStatusName(),"政治面貌必填且必须满足包含在政治面貌的静态值名称列表中!");
    }

    List<String> IndustryFields = tsmProValueMap.get(service.propCode[3]).stream()
        .map(TsmPropertyValue::getValueName).collect(
            Collectors.toList());
    if (ObjectUtil.isEmpty(bo.getIndustryField()) || !IndustryFields.containsAll(Arrays.asList(bo.getIndustryField().split(",")))) {
      service.doWithMistakeErrMessage(rowMap,"industryField",bo.getIndustryField(),"行业领域必填且必须满足包含在行业领域的静态值名称列表中[多个以英文逗号分隔]!");
    }

    // 共同参数校验
    SpacePassTeamCUReqDTO spacePassTeamCUReqDTO = new SpacePassTeamCUReqDTO();
    BeanUtil.copyPropertiesIgnoreNull(bo, spacePassTeamCUReqDTO);
    List<ExcelMistakeData> mistakeData = new ArrayList<>();
    service.checkParamsData(spacePassTeamCUReqDTO, mistakeData, rowMap);

  }



  /**
   * 所有数据解析完成了 都会来调用
   * @param context
   */
  @Override
  public void doAfterAllAnalysed(AnalysisContext context) {
    if (!hasData) {
      // 没有读取到数据的处理逻辑
      throw new BusinessException("导入excel为空文件!");
    }
    // 这里也要保存数据，确保最后遗留的数据也存储到数据库
    saveTeamData();
    log.info("所有数据解析完成！");
  }

  /**
   * 加上存储数据库
   */
  private boolean saveTeamData() {

    // 判断是否存在错误数据
    List<ExcelMistakeData> errDataList = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(errContentList)) {
      for (MistakeDataInfo mistakeDataInfo : errContentList) {
        if (ObjectUtil.isNotEmpty(mistakeDataInfo)) {
          errDataList.addAll(mistakeDataInfo.getMistakeDataInfo());
        }
      }
    }

    if (ObjectUtil.isNotEmpty(errDataList)) {
      List<ExcelMistakeData> errList = errDataList.stream().filter(e -> Integer.valueOf(0).equals(e.getState()))
          .collect(Collectors.toList());
      // 没有错误数据才能存储到数据库中
      if (ObjectUtil.isEmpty(errList)) {
        log.info("团队{}条数据，开始存储数据库！", cachedDataTempList.size());
        service.qczdTeamStayTempRepo.saveAll(cachedDataTempList);
        log.info("存储数据库成功！");
        return true;
      }
    }
    return false;
  }
}
