package com.crunii.micro.service.appsecond.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.appsecond.api.dto.appmultimodules.AppMultiModulesDto;
import com.crunii.micro.service.appsecond.api.vo.appmultimodules.AppMultiModulesHotResVo;
import com.crunii.micro.service.appsecond.api.vo.appmultimodules.AppUserViewVo;

import java.util.List;

/**
 * youth-cq.
 * 热搜/热榜/点赞/收藏/关注/浏览记录
 * @author leixb
 * @since 2023-12-07 19:29
 **/
public interface AppMultiModulesService {
    
    /**
     * 点赞/收藏/关注及取消
     * @author leixb
     * @since 2023-12-09 16:45
     * @param worker worker
     * @param req req
     * @return String
     */
    String doAction(Worker worker, AppMultiModulesDto req);
    
    /**
     * 查询热搜榜/热度榜
     * @author leixb
     * @since 2023-12-09 17:24
     * @param worker worker
     * @return AppMultiModulesHotResVo
     */
    AppMultiModulesHotResVo findHot(Worker worker);
    
    /**
     * 新增浏览记录
     * @author leixb
     * @since 2023-12-12 15:52
     * @param worker worker
     * @param contentId contentId
     */
    void createView(Worker worker, Long contentId);
    
    /**
     * 浏览记录分页查询
     * @author leixb
     * @since 2023-12-13 16:32
     * @param worker worker
     * @param req req
     * @return PageRsp<AppUserViewVo>
     */
    PageRsp<AppUserViewVo> pageView(Worker worker, PageReq<Void> req);
    
    /**
     * 一键清空浏览记录列表
     * @author leixb
     * @since 2023-12-13 17:36
     * @param worker worker
     * @return String
     */
    String doClearView(Worker worker);
    
    /**
     * 查询浏览记录数/收藏数
     * @author leixb
     * @since 2023-12-13 19:44
     * @param worker worker
     * @param type type
     * @return String
     */
    Integer findNum(Worker worker, Integer type);
    
    /**
     * 收藏列表分页查询
     * @param worker worker
     * @param req req
     * @return PageRsp<AppUserViewVo>
     */
    PageRsp<AppUserViewVo> pageCollect(Worker worker, PageReq<Void> req);
    
    /**
     * 取消收藏
     * @param worker worker
     * @param ids ids
     * @return String
     */
    String doCancelCollect(Worker worker, List<String> ids);

    /**
     * app-应用-热搜
     * @param worker
     * @return
     */
    List<String> findAppHot(Worker worker);
}
