package com.crunii.micro.service.appsecond.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.feign.client.dto.HotEsPageDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.YouthPublicAuditDto;
import com.crunii.micro.service.appsecond.api.dto.UserViewDto;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.ContextAppDetailVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.YouthContextVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.YouthHotPublicDetailVo;

import java.util.List;

/**
 * @author wangchen create 2023/12/8 15:51
 */
public interface AppYouthHotService {

    /**
     * app首页分页查询
     * @param req 请求参数
     * @param worker 当前用户
     * @return data
     */
    PageRsp<YouthContextVo> pageYouthHot(PageReq<HotEsPageDto> req, Worker worker);

    /**
     * 查询内容详情
     * @param contentManageId 主键id
     * @param worker worker
     * @return data
     */
    ContextAppDetailVo getYouthHotDetail(Long contentManageId, Worker worker);

    /**
     * 查询当前用户搜索历史
     * @param searchType 搜索类型，m默认青年热点
     * @param worker 当前worker
     * @return data
     */
    List<String> queryHotSearch(Integer searchType,Worker worker);

    /**
     * 清空记录
     * @param worker 当前用户
     * @return str
     */
    String delSearch(Worker worker);

    /**
     * 刷新本地缓存
     */
    void flushCash();

    /**
     * 添加app浏览记录
     * @param dto 请求参数
     * @param worker 当前用户
     * @return str
     */
    String createUserView(UserViewDto dto, Worker worker);
    
    /**
     * 处理参数
     * @author leixb
     * @since 2023-12-23 16:38
     * @param result result
     * @param loginId loginId
     */
    void handleParams(List<YouthContextVo> result, Integer loginId);

    /**
     * 查询公共类文章详情
     * @param contentManageId 文章id
     * @return data
     */
    YouthHotPublicDetailVo queryYouthHot(Long contentManageId);

    /**
     * 提交审核
     * @param dto 参数
     * @return str
     */
    String updatePublicAudit(YouthPublicAuditDto dto);
}
