package com.crunii.micro.service.appsecond.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqObsFile;
import com.crunii.micro.domain.repo.main.YcqObsFileRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.ds.uid.util.DateUtils;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.feign.client.dto.DownloadFileVo;
import com.crunii.micro.service.appsecond.api.dto.AppCertificateRespDTO;
import com.crunii.micro.service.appsecond.api.vo.CertificateVO;
import com.crunii.micro.service.appsecond.dao.AppCertificateDao;
import com.crunii.micro.service.appsecond.service.IAppCertificateService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author: zwx
 * @CreateTime: 2023-12-07  10:56:57
 * @Description: TODO
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class AppCertificateServiceImpl implements IAppCertificateService {

    @Autowired
    AppCertificateDao appCertificateDao;

    @Resource
    private YcqObsFileRepo fileRepo;

    @Resource
    ObsService obsService;

    @Override
    public Result getmyCertificate() {
        
        Worker currWorker = CurrWorkerUtil.getCurrWorker();

        List<CertificateVO> certificateVOList = new ArrayList<>();

        // 获取青马聘书
        List<AppCertificateRespDTO> ps = appCertificateDao.getLetterOfAppointment(currWorker);
        if (CollectionUtils.isNotEmpty(ps)) {
            for (AppCertificateRespDTO p : ps) {
                CertificateVO certificateVO = new CertificateVO();
                String fileKey = p.getFileKey();
                YcqObsFile ycqObsFile = fileRepo.findByObjectKeyAndIsPicAndState(fileKey,1,1);
                if (ycqObsFile!=null) {
                    Result<DownloadFileVo> downloadFileVoResult = obsService.doDownloadFile(ycqObsFile.getId());
                    certificateVO.setIssuingUnit(p.getUnit());
                    certificateVO.setType("导师聘书");
                    certificateVO.setImage("data:image/"+ycqObsFile.getType()+";base64,"+downloadFileVoResult.getData().getFile());
                    certificateVO.setReleaseTime(p.getReleaseTime().format(DateTimeFormatter.ofPattern(DateUtils.DAY_PATTERN)));
                    certificateVOList.add(certificateVO);
                }
            }
        }

        // 获取青马结业证书
        List<AppCertificateRespDTO> jy =  appCertificateDao.getCertificateOfCompletion(currWorker);
        if (CollectionUtils.isNotEmpty(jy)) {
            for (AppCertificateRespDTO p : jy) {
                CertificateVO certificateVO = new CertificateVO();
                String fileKey = p.getFileKey();
                YcqObsFile ycqObsFile = fileRepo.findByObjectKeyAndIsPicAndState(fileKey,1,1);
                if (ycqObsFile!=null) {
                    Result<DownloadFileVo> downloadFileVoResult = obsService.doDownloadFile(ycqObsFile.getId());
                    certificateVO.setIssuingUnit(p.getUnit());
                    certificateVO.setType("结业证书");
                    certificateVO.setImage("data:image/"+ycqObsFile.getType()+";base64,"+downloadFileVoResult.getData().getFile());
                    certificateVO.setReleaseTime(p.getReleaseTime().format(DateTimeFormatter.ofPattern(DateUtils.DAY_PATTERN)));
                    certificateVOList.add(certificateVO);
                }
            }
        }

        // 志愿山城志愿证 tozwx  发放单位:重庆市青年志愿者协会
        List<CertificateVO> result = certificateVOList.stream().sorted(Comparator.comparing(CertificateVO::getReleaseTime).reversed()).collect(Collectors.toList());
        return Result.ofSuccess(result);
    }
}
