package com.crunii.micro.service.appsecond.service.impl;
import java.util.Date;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import cn.hutool.core.util.URLUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.enums.TaskTypeEnums;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.AESUtil;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.PicInfo;
import com.crunii.micro.domain.dto.TsmRoleDto;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.message.SendMessageTaskReqDTO;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.AppContentDetailRepo;
import com.crunii.micro.domain.repo.main.YcqActiveFlowNodeRepo;
import com.crunii.micro.domain.repo.main.YcqMessageTemplateRepo;
import com.crunii.micro.feign.client.ElsticService;
import com.crunii.micro.feign.client.MessageService;
import com.crunii.micro.feign.client.dto.YouthHotEsVo;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ContextCreateDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ContextManageDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ToTopDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.YouthAuditDto;
import com.crunii.micro.service.appsecond.api.dto.YouthHotLabelVo;
import com.crunii.micro.service.appsecond.api.enums.YouthHotAuditEnum;
import com.crunii.micro.service.appsecond.api.enums.YouthHotEnum;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.*;
import com.crunii.micro.service.appsecond.config.YouthHotConfigProperty;
import com.crunii.micro.service.appsecond.jdbc.AppContentManageDao;
import com.crunii.micro.service.appsecond.utils.CharacterLengthUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.crunii.micro.domain.repo.main.AppContentManageRepo;
import com.crunii.micro.service.appsecond.service.IAppContentManageService;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import static com.crunii.micro.service.appsecond.api.constants.AppSecondConstants.*;

/**
 * 【请填写功能名称】Service业务层处理
 *
 * @author liuchuan
 * @date 2023-12-06
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class AppContentManageServiceImpl implements IAppContentManageService {

    private final AppContentManageRepo baseRepo;
    private final AppContentDetailRepo appContentDetailRepo;
    private final AppContentManageDao appContentManageDao;
    private final ElsticService elsticService;
    private final YcqActiveFlowNodeRepo flowNodeRepo;
    private final MessageService messageService;
    private final YouthHotConfigProperty configProperty;
    private final YcqMessageTemplateRepo ycqMessageTemplateRepo;

    @Value("${upload.url}")
    private String urlPrefix;

    @Override
    public String createContextDraft(ContextCreateDto dto, Worker worker) {
        validatedFileType(dto.getFileType(),dto.getFileKeys());
        // 查询当前用户绑定的媒体账号id
        Date now = new Date();
        int loginId = worker.getLoginId();
        AppMedium medium = findMediumIdByWorkerId(worker.getLoginId());
        if(Objects.isNull(medium)){
            throw new BusinessException("该账号未绑定媒体");
        }
        if(!StringUtils.hasText(worker.getOrgId())){
            throw new BusinessException("该账号无组织信息");
        }
        if(CharacterLengthUtil.isExceedLength(dto.getContentTitle(), MAX_TITLE_SIZE)){
            throw new BusinessException("文章标题不能超过"+MAX_TITLE_SIZE+"个字符");
        }
        AppContentManage appContentManage;
        if(Objects.nonNull(dto.getContentManageId())){
            appContentManage = baseRepo.getOne(dto.getContentManageId());
            // 如果有id为空 则抛出异常
            if(Objects.isNull(appContentManage)){
                throw new BusinessException("该内容已被删除");
            }
        }else {
            appContentManage = new AppContentManage();
            appContentManage.setCreateBy(loginId);
            appContentManage.setCreateTime(now);
        }
        BeanUtils.copyProperties(dto,appContentManage);
        appContentManage.setMediumId(medium.getMediumId());
        String labelName = "";
        if(!CollectionUtils.isEmpty(dto.getLabelName())){
            if(MAX_LABEL_COUNT <dto.getLabelName().size()){
                throw new BusinessException("标签最大为"+MAX_LABEL_COUNT+"个");
            }
            dto.setLabelName(dto.getLabelName().stream().filter(StringUtils::hasText).collect(Collectors.toList()));
            labelName = String.join(",", dto.getLabelName());
        }
        appContentManage.setLabelName(labelName);
        Integer state = appContentManage.getState();
        appContentManage.setState(Objects.nonNull(state)?state:0);
        appContentManage.setAppSort(2);
        appContentManage.setUpdateBy(loginId);
        appContentManage.setUpdateTime(now);
        AppContentManage appContentManage1 = baseRepo.saveAndFlush(appContentManage);

        AppContentDetail appContentDetail = appContentDetailRepo.findByContentManageId(appContentManage1.getContentManageId());
        if(Objects.isNull(appContentDetail)){
            appContentDetail = new AppContentDetail();
            appContentDetail.setCreateBy(loginId);
            appContentDetail.setCreateTime(now);
        }

        appContentDetail.setContentManageId(appContentManage1.getContentManageId());
        if(!CollectionUtils.isEmpty(dto.getFileKeys())){
            // 处理filekey带url
            List<String> collect = dto.getFileKeys().stream().map(PicInfo::getPicUrl).collect(Collectors.toList());
            collect.replaceAll(x->x.replaceAll(urlPrefix,""));
            appContentDetail.setPicKey(String.join(",",collect));
        }
        appContentDetail.setArticleContext(dto.getArticleContext());
        appContentDetail.setFileId(dto.getFileId());
        appContentDetail.setUpdateBy(loginId);
        appContentDetail.setUpdateTime(now);
        appContentDetailRepo.saveAndFlush(appContentDetail);
        return "操作成功";
    }


    private void validatedFileType(Integer fileType, List<PicInfo> fileKeys) {
        int maxNumber = YouthHotEnum.getMaxNumber(fileType);
        if(maxNumber <0){
            throw new BusinessException("文件类型不存在");
        }
        if(!CollectionUtils.isEmpty(fileKeys) && maxNumber <fileKeys.size()){
            throw new BusinessException("超过上传最大");
        }
    }

    @Override
    public String createContext(ContextCreateDto dto, Worker worker) {
        // 查询当前用户绑定的媒体账号id
        validatedFileType(dto.getFileType(),dto.getFileKeys());
        Date now = new Date();
        int loginId = worker.getLoginId();
        AppMedium medium = findMediumIdByWorkerId(worker.getLoginId());
        if(Objects.isNull(medium)){
            throw new BusinessException("该账号未绑定媒体");
        }
        if(!StringUtils.hasText(worker.getOrgId())){
            throw new BusinessException("该账号无组织信息");
        }
        if(CharacterLengthUtil.isExceedLength(dto.getContentTitle(), MAX_TITLE_SIZE)){
            throw new BusinessException("文章标题不能超过"+MAX_TITLE_SIZE+"个字符");
        }
        // 校验视频类型 必穿视频
        if(videoType.equals(dto.getFileType())){
            if(!StringUtils.hasText(dto.getFileId())){
                throw new BusinessException("请上传视频");
            }
        }
        AppContentManage appContentManage;
        if(Objects.nonNull(dto.getContentManageId())){
            appContentManage = baseRepo.getOne(dto.getContentManageId());
            // 如果有id为空 则抛出异常
            if(Objects.isNull(appContentManage)){
                throw new BusinessException("该内容已被删除");
            }
            // 查看审核状态是否可修改
            validatedIsModify(appContentManage.getState());
        }else {
            appContentManage = new AppContentManage();
            appContentManage.setCreateBy(loginId);
            appContentManage.setCreateTime(now);
        }
        List<String> labelName = dto.getLabelName();
        if(CollectionUtils.isEmpty(labelName)){
            labelName = new ArrayList<>();
        }
        BeanUtils.copyProperties(dto,appContentManage);
        appContentManage.setMediumId(medium.getMediumId());
        if(MAX_LABEL_COUNT <labelName.size()){
            throw new BusinessException("标签最大为"+MAX_LABEL_COUNT+"个");
        }
        String labelNames = "";
        if(!CollectionUtils.isEmpty(labelName)){
            dto.setLabelName(dto.getLabelName().stream().filter(StringUtils::hasText).collect(Collectors.toList()));
            labelNames = String.join(",", dto.getLabelName());
        }
        appContentManage.setLabelName(labelNames);
        appContentManage.setAppSort(2);
        appContentManage.setUpdateBy(loginId);
        appContentManage.setUpdateTime(now);
        // 校验下架次数
        validatedOffCount(appContentManage);
        // 如果改文章是已下架再发布，直接通过，无需审核
        boolean isOff = validatedIsOff(appContentManage.getOffCount());
        if(isOff){
            appContentManage.setState(1);
        }else {
            appContentManage.setState(2);
        }
        AppContentManage appContentManage1 = baseRepo.saveAndFlush(appContentManage);

        AppContentDetail appContentDetail = appContentDetailRepo.findByContentManageId(appContentManage1.getContentManageId());
        if(Objects.isNull(appContentDetail)){
            appContentDetail = new AppContentDetail();
            appContentDetail.setCreateBy(loginId);
            appContentDetail.setCreateTime(now);
        }

        appContentDetail.setContentManageId(appContentManage1.getContentManageId());
        if(CollectionUtils.isEmpty(dto.getFileKeys())){
           throw new BusinessException("封面图不能为空");
        }
        // 处理filekey带url
        List<String> collect = dto.getFileKeys().stream().map(PicInfo::getPicUrl).collect(Collectors.toList());
        collect.replaceAll(x->x.replaceAll(urlPrefix,""));
        appContentDetail.setPicKey(String.join(",",collect));

        appContentDetail.setFileId(dto.getFileId());
        appContentDetail.setUpdateBy(loginId);
        appContentDetail.setArticleContext(dto.getArticleContext());
        appContentDetail.setUpdateTime(now);
        appContentDetailRepo.saveAndFlush(appContentDetail);
        if(isOff){
            sendEs(appContentManage,appContentDetail,medium);
            return "操作成功";
        }
        // 提交审核
        List<Long> auditIds = new ArrayList<>();
        auditIds.add(appContentManage1.getContentManageId());
        submitAudit(worker,auditIds);
        // 推送消息
        Long importantNewsLabel = appContentManageDao.getImportantNewsLabel();
        int isImportNews = 0;
        if(!CollectionUtils.isEmpty(dto.getLabelName())){
            if(dto.getLabelName().size()==1 && dto.getLabelName().get(0).equals(importantNewsLabel+"")){
                isImportNews = 1;
            }
        }
        sendMessage(medium.getOrgId(),Long.parseLong(worker.getLoginId()+""),appContentManage,isImportNews);
        return "操作成功";
    }

    private boolean validatedIsOff(Integer offCount) {
        if(Objects.nonNull(offCount)){
            return 0 < offCount;
        }
        return false;
    }

    private void validatedOffCount(AppContentManage appContentManage) {
        Integer offCount = appContentManage.getOffCount();
        if(Objects.nonNull(offCount)){
            Integer draftState = 0;
            if(!draftState.equals(appContentManage.getState())){
                if(YOUTH_HOT_OFF_COUNT<offCount){
                    throw new BusinessException("修改次数已达最大，不可发布");
                }
            }
        }
    }


    private void sendMessage(Long orgId,Long loginId,AppContentManage appContentManage,int isImportNews) {
        List<String> auditOrgUser = appContentManageDao.findAuditOrgUser(orgId ,configProperty.getAuditRoleId());
        if(CollectionUtils.isEmpty(auditOrgUser)){
            return;
        }
        String contextTitle = appContentManage.getContentTitle();

        if(contextTitle.length()>25){
            contextTitle = contextTitle.substring(0, 25).concat("...");
        }
        YcqMessageTemplate msgTempLate = ycqMessageTemplateRepo.findBySystemNameAndSceneNameAndTitleAndDelFlag(YOUTH_HOT, SCENE_NAME_MSG, SCENE_NAME_MSG, (byte) 0);
        YcqMessageTemplate sortTempLate = ycqMessageTemplateRepo.findBySystemNameAndSceneNameAndTitleAndDelFlag(YOUTH_HOT, SCENE_NAME_SORT, SCENE_NAME_SORT, (byte) 0);
        if (Objects.nonNull(msgTempLate)) {
            sendByMsgTempLate(loginId,msgTempLate,auditOrgUser,contextTitle,isImportNews,appContentManage.getContentManageId().toString(),appContentManage.getFileType());
        }else {
            log.info("消息模板未配置，无法发送消息--》{}",contextTitle);
        }
        if(Objects.nonNull(sortTempLate)){
            sendBySortTempLate(loginId,sortTempLate,auditOrgUser,contextTitle,appContentManage,isImportNews);
        }else {
            log.info("消息模板未配置，无法发送短信--》{}",contextTitle);
        }
    }

    private void sendBySortTempLate(Long loginId, YcqMessageTemplate sortTempLate, List<String> auditOrgUser, String contextTitle,AppContentManage appContentManage,int isImportNews) {
        SendMessageTaskReqDTO dto = new SendMessageTaskReqDTO();
        dto.setScopeType(4);
        dto.setScopeData(String.join(",",auditOrgUser));
        dto.setSendWay(sortTempLate.getPushType());
        dto.setMessageType(sortTempLate.getPushEndpoint());
        dto.setMessageContent(sortTempLate.getMessageTemplate());
        dto.setDataSource(sortTempLate.getSystemName());
        dto.setMessageClassify(sortTempLate.getTitle());
        dto.setTemplateId(sortTempLate.getTemplateId());
        dto.setSendLoginId(loginId);

        Map<String,Object> fillData = new HashMap<>(6);
        fillData.put("title",contextTitle);

        // 参数拼接
        String userId = AESUtil.encrypt(auditOrgUser.get(0));
        fillData.put("address",String.format(configProperty.getJudgeUrl(), appContentManage.getContentManageId().toString(), userId,appContentManage.getFileType(),isImportNews));
        dto.setFillData(JsonUtil.getJsonString(fillData));

        Result result = messageService.saveMessageTask(dto);
        if(!result.isSuccess()){
            log.info("青年热点推送消息失败------》{}",result.getMsg());
        }
    }

    /**
     * 推送消息
     */
    private void sendByMsgTempLate(Long loginId, YcqMessageTemplate msgTempLate, List<String> auditOrgUser, String contextTitle, int isImportNews, String contentManageId, Integer fileType) {
        SendMessageTaskReqDTO dto = new SendMessageTaskReqDTO();
        dto.setScopeType(4);
        dto.setScopeData(String.join(",",auditOrgUser));
        dto.setSendWay(msgTempLate.getPushType());
        dto.setMessageType(msgTempLate.getPushEndpoint());
        dto.setMessageContent(String.format(msgTempLate.getMessageTemplate(), contextTitle));
        dto.setDataSource(msgTempLate.getSystemName());
        dto.setMessageClassify(msgTempLate.getTitle());
        dto.setTemplateId(-1L);
        dto.setSendLoginId(loginId);

        Map<String,Object> pathJson = new HashMap<>(6);
        pathJson.put("type","youthHot");
        pathJson.put("isImportNews",isImportNews);
        pathJson.put("id",contentManageId);
        pathJson.put("fileType",fileType);
        dto.setPathAndJson(JsonUtil.getJsonString(pathJson));

        Result result = messageService.saveMessageTask(dto);
        if(!result.isSuccess()){
            log.info("青年热点推送消息失败------》{}",result.getMsg());
        }
    }


    /**
     * 提交审核
     */
    private void submitAudit(Worker worker, List<Long> contentManageId) {
        if(CollectionUtils.isEmpty(contentManageId)){
            return;
        }
        List<YcqFlowNode> flowNodes = new ArrayList<>();
        for (Long l : contentManageId) {
            YcqFlowNode ycqFlowNode = new YcqFlowNode();
            ycqFlowNode.setOrgId(Long.parseLong(worker.getOrgId()));
            ycqFlowNode.setOrgName(worker.getOrgId());
            ycqFlowNode.setCreateTime(new Date());
            ycqFlowNode.setCreateBy(worker.getWorkerName());
            ycqFlowNode.setCreateId(Long.parseLong(worker.getLoginId()+""));
            ycqFlowNode.setBizId(l);
            ycqFlowNode.setBizType(TaskTypeEnums.HOT.getCode());
            ycqFlowNode.setApproveType(1);
            flowNodes.add(ycqFlowNode);
        }
        flowNodeRepo.saveAllAndFlush(flowNodes);
    }

    /**
     * 验证改状态是否可修改
     */
    private void validatedIsModify(Integer state) {
        boolean contains = canModifyState.contains(state);
        if(!contains){
            throw new BusinessException("该状态无法修改");
        }
    }

    /**
     * 推送数据到es
     */
    private void sendEs(AppContentManage appContentManage, AppContentDetail appContentDetail, AppMedium medium) {
        YouthHotEsVo youthHotEsVo = new YouthHotEsVo();
        youthHotEsVo.setContentManageId(appContentManage.getContentManageId());
        youthHotEsVo.setFileType(appContentManage.getFileType());
        youthHotEsVo.setContentTitle(appContentManage.getContentTitle());
        youthHotEsVo.setArticleContext(appContentDetail.getArticleContext());
        youthHotEsVo.setMediumId(medium.getMediumId());
        youthHotEsVo.setIdName(medium.getIdName());
        youthHotEsVo.setMediumHeadPic(medium.getMediumHeadPic());
        elsticService.saveYouthHot(youthHotEsVo);
    }

    @Override
    @InjectProperty
    public PageRsp<ContextManageVo> pageContextManage(PageReq<ContextManageDto> req, Worker worker) {
        // 查询当前组织下的媒体id
        boolean superAdmin = isSuperAdmin(worker.getLoginId());
        List<Long> mediumIds = findMediums(superAdmin, worker);
        if(CollectionUtils.isEmpty(mediumIds)){
            return new PageRsp<>();
        }
        // 查询要闻标签id
        String labelId = getImportantNewsLabel();
        PageRsp<ContextManageVo> contextManageVoPageRsp = appContentManageDao.pageContextManage(req, mediumIds,superAdmin,labelId);
        List<ContextManageVo> result = contextManageVoPageRsp.getResult();
        if(CollectionUtils.isEmpty(result)){
            return contextManageVoPageRsp;
        }
        Set<Long> labelIds = new HashSet<>();
        // 处理标签
        for (ContextManageVo contextManageVo : result) {
            if(StringUtils.hasText(contextManageVo.getLabelName())){
                Arrays.stream(contextManageVo.getLabelName().split(",")).forEach(y->labelIds.add(Long.parseLong(y)));
            }
        }
        Map<Long,String> labelMap = new HashMap<>(16);
        if(!CollectionUtils.isEmpty(labelIds)){
            List<YouthHotLabelVo> list = appContentManageDao.findLabelList(labelIds);
            labelMap = list.stream().collect(Collectors.toMap(YouthHotLabelVo::getLabelId, YouthHotLabelVo::getLabelName));
        }

        List<Long> collect = result.stream().map(ContextManageVo::getContentManageId).collect(Collectors.toList());
        Map<Long,String> coverMap = new HashMap<>(16);
        if(!CollectionUtils.isEmpty(collect)){
            List<IdValueVo> list = appContentManageDao.findPicFileKey(collect);
            if(!CollectionUtils.isEmpty(list)){
                coverMap = list.stream().collect(Collectors.toMap(IdValueVo::getId, IdValueVo::getValue));
            }
        }

        boolean b = validatedIsAuditUser(worker);
        for (ContextManageVo contextManageVo : result) {
            // 有审核权限 
            if(superAdmin || b){
                contextManageVo.setCanAuth(1);
            }
            StringJoiner newLabelName = new StringJoiner(",");
            if(StringUtils.hasText(contextManageVo.getLabelName())){
                String[] split = contextManageVo.getLabelName().split(",");
                for (String s : split) {
                    String s1 = labelMap.get(Long.parseLong(s));
                    newLabelName.add(s1);
                }
                contextManageVo.setLabelName(newLabelName.toString());
            }
            String s = coverMap.get(contextManageVo.getContentManageId());
            if(StringUtils.hasText(s)){
                s = s.split(",")[0];
                if(!s.contains("http")){
                    s = urlPrefix + s;
                }
            }
            contextManageVo.setCoverPic(s);
        }
        return contextManageVoPageRsp;
    }

    private List<Long> findMediums(boolean superAdmin,Worker worker) {
        List<Long> mediumIds = new ArrayList<>();
        boolean authUser = false;
        if(!superAdmin){
            authUser = validatedIsAuditUser(worker);
        }
        // 如果不是审核员查询媒体账号
        if(authUser){
            return appContentManageDao.findMediums(worker.getOrgId());
        }
        AppMedium mediumByWorkerId = appContentManageDao.findMediumByWorkerId(worker.getLoginId());
        if (Objects.isNull(mediumByWorkerId)) {
            return mediumIds;
        }
        mediumIds.add(mediumByWorkerId.getMediumId());
        return mediumIds;
    }

    private boolean validatedIsAuditUser(Worker worker) {
        Integer auditRoleId = configProperty.getAuditRoleId();
        List<Integer> collect = worker.getRoles().stream().map(TsmRoleDto::getRoleId).collect(Collectors.toList());
        return collect.contains(auditRoleId);
    }

    private String getImportantNewsLabel() {
        return appContentManageDao.getImportantNewsLabel()+"";
    }

    /**
     * 判断是否超级管理员
     */
    private boolean isSuperAdmin(int loginId) {
        List<Integer> superLoginIds = CommonConstants.SUPER_LOGIN_IDS;
        return superLoginIds.contains(loginId);
    }


    @Override
    public ContextManageDetailVo findContextManageDetail(Long contentManageId, Worker worker) {
        ContextManageDetailVo contextManageDetail = appContentManageDao.findContextManageDetail(contentManageId);
        if(Objects.nonNull(contextManageDetail)){
            List<PicInfo> urls = new ArrayList<>();
            List<String> fileKeys = new ArrayList<>();
            String picKey = contextManageDetail.getPicKey();
            if(StringUtils.hasText(picKey)){
                fileKeys = Arrays.stream(picKey.split(",")).collect(Collectors.toList());
                List<PicInfo> picInfoByFileKey = appContentManageDao.findPicInfoByFileKey(fileKeys);
                Map<String, PicInfo> picInfoMap = new HashMap<>(2);
                if(!CollectionUtils.isEmpty(picInfoByFileKey)){
                    picInfoMap = picInfoByFileKey.stream().collect(Collectors.toMap(PicInfo::getPicUrl, Function.identity()));
                }
                for (String s : picKey.split(",")) {
                    if(s.contains("http")){
                        PicInfo picInfo = getDefaultPic(s);
                        urls.add(picInfo);
                    }else {
                        PicInfo picInfo = picInfoMap.get(s);
                        if(Objects.isNull(picInfo)){
                            picInfo = getDefaultPic(s);
                        }
                        picInfo.setPicUrl(urlPrefix+s);
                        urls.add(picInfo);
                    }
                }
            }
            // 处理标签
            handleLabel(contextManageDetail);
            // c处理权限
            boolean b = validatedIsAuditUser(worker);
            if(CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId()) || b){
                contextManageDetail.setCanAuth(1);
            }
            contextManageDetail.setFileKeys(fileKeys);
            contextManageDetail.setUrls(urls);
            // 查询媒体信息
            AppMedium appMedium  =  appContentManageDao.findMediumInfo(contentManageId);
            contextManageDetail.setIdName(appMedium.getIdName());
            contextManageDetail.setMediumHeadPic(appMedium.getMediumHeadPic());

        }
        return contextManageDetail;
    }

    private void handleLabel(ContextManageDetailVo contextManageDetail) {
        String labelId = contextManageDetail.getLabelName();
        if(StringUtils.hasText(labelId)){
            String[] split = labelId.split(",");
            Set<Long> labelIds = new HashSet<>();
            for (String s : split) {
                labelIds.add(Long.parseLong(s));
            }
            List<YouthHotLabelVo> labelList = appContentManageDao.findLabelList(labelIds);
            String labelNames = labelList.stream().map(YouthHotLabelVo::getLabelName).collect(Collectors.joining(","));
            contextManageDetail.setLabelNameTrans(labelNames);
        }
    }

    private PicInfo getDefaultPic(String s) {
        PicInfo picInfo = new PicInfo();
        picInfo.setPicUrl(s);
        picInfo.setWidth(WATER_FALL_DEFAULT_WIDTH);
        picInfo.setHeight(WATER_FALL_DEFAULT_HEIGHT);
        return picInfo;
    }

    @Override
    public String delContext(List<Long> contentManageIds,Worker worker) {
        appContentManageDao.offShelfContext(contentManageIds, worker);
       /* // 重置流程
        appContentManageDao.delAuditFlow(contentManageIds);*/
        contentManageIds.forEach(x-> elsticService.delYouthHot(x.toString()));
        return "删除成功";
    }

    @Override
    public String updateReleaseContext(List<Long> contentManageIds, Worker worker) {
        List<String> mediumOrgIds = appContentManageDao.findMediumOrgIds(contentManageIds);
        if(mediumOrgIds.size()!=1){
            throw new BusinessException("无法审核");
        }
        long orgId = Long.parseLong(mediumOrgIds.get(0));
        List<AppContentManage> list = new ArrayList<>();
        // 是否要闻解析
        List<AppContentManage> labels = new ArrayList<>();
        // 需要审核的id
        List<Long> auditIds = new ArrayList<>();
        // 需要推送es的数据
        List<YouthHotEsVo> youthHotEsVos = new ArrayList<>();
        for (Long contentManageId : contentManageIds) {
            AppContentManage one = baseRepo.getOne(contentManageId);
            if(Objects.isNull(one)){
               continue;
            }
            validatedOffCount(one);
            boolean isOff = validatedIsOff(one.getOffCount());
            String s = validateParams(one);
            one.setUpdateBy(worker.getLoginId());
            one.setAppSort(2);
            one.setUpdateTime(new Date());
            // 查看之前是否审核过
            if(isOff){
                one.setState(1);
                // es参数构建
                YouthHotEsVo vo =  appContentManageDao.findPushEsData(one.getMediumId());
                vo.setContentManageId(contentManageId);
                vo.setFileType(one.getFileType());
                vo.setContentTitle(one.getContentTitle());
                vo.setArticleContext(s);
                youthHotEsVos.add(vo);
            }else {
                one.setState(2);
                labels.add(one);
                auditIds.add(contentManageId);
            }
            list.add(one);
        }
        baseRepo.saveAllAndFlush(list);
        Long importantNewsLabel = appContentManageDao.getImportantNewsLabel();
        youthHotEsVos.forEach(elsticService::saveYouthHot);
        labels.forEach(x->{
            String labelId = x.getLabelName();
            int isImportNews = 0;
            if(StringUtils.hasText(labelId)){
                String[] split = labelId.split(",");
                if(split.length==1){
                    if(split[0].equals(importantNewsLabel+"")){
                        isImportNews = 1;
                    }
                }
            }
            sendMessage(orgId,Long.parseLong(worker.getLoginId()+""),x,isImportNews);
        });
        submitAudit(worker,auditIds);
        return "发布成功";
    }

    private Long validatedIsAudit(List<Long> contentManageIds, Worker worker) {
      /*  boolean contains = CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId());
        // 校验谁否有审核角色
        Integer auditRoleId = configProperty.getAuditRoleId();
        List<Integer> roleIds = worker.getRoles().stream().map(TsmRoleDto::getRoleId).collect(Collectors.toList());
        if(!contains && !roleIds.contains(auditRoleId)){
            throw new BusinessException("未配置审核角色");
        }*/
        List<String> mediumOrgIds = appContentManageDao.findMediumOrgIds(contentManageIds);
        if(mediumOrgIds.size()!=1){
            throw new BusinessException("无法审核");
        }
        return Long.parseLong(mediumOrgIds.get(0));
    }

    @Override
    public String synEsData(Worker worker) {
        List<YouthHotEsVo> list = appContentManageDao.findAllContext();
        log.info("开始青年热点es数据同步----共{}条数据",list.size());
        for (YouthHotEsVo youthHotEsVo : list) {
            elsticService.saveYouthHot(youthHotEsVo);
        }
        log.info("青年热点es数据同步完成");
        return "更新成功";
    }

    @Override
    public String delRealContext(List<Long> contentManageIds, Worker worker) {
        if(CollectionUtils.isEmpty(contentManageIds)){
            throw new BusinessException("删除的id不能为空");
        }
        // 验证是否上级 上架不允许删除
        validatedIsPush(contentManageIds);
        appContentManageDao.delContext(contentManageIds, worker);
        contentManageIds.forEach(x-> elsticService.delYouthHot(x.toString()));
        return "删除成功";
    }

    @Override
    public String updateTop(ToTopDto toTopDto, Worker worker) {
        Integer appSortUp = 1;
        AppContentManage one = baseRepo.getOne(toTopDto.getContentManageId());
        if(Objects.isNull(one)){
            throw new BusinessException("已删除");
        }
        List<AppContentManage> list = new ArrayList<>();
        if(appSortUp.equals(toTopDto.getAppSort())){
            // 查询之前有2条的话，最后一条数据
            one.setAppSort(1);
            AppContentManage appContentManage = appContentManageDao.findSecond();
            if(Objects.nonNull(appContentManage)){
                appContentManage.setAppSort(2);
                list.add(appContentManage);
            }
        }else {
            one.setAppSort(2);
        }
        list.add(one);
        baseRepo.saveAllAndFlush(list);
        return "更新成功";
    }

    @Override
    public String updateAuditState(YouthAuditDto dto, Worker worker) {
        List<Long> contentManageIds = new ArrayList<>();
        contentManageIds.add(dto.getContentManageId());
        validatedIsAudit(contentManageIds,worker);
        // 校验当前用户权限
        AppContentManage one = baseRepo.getOne(dto.getContentManageId());
        validatedUserAudit(one,worker);
        YcqFlowNode ycqFlowNode = new YcqFlowNode();
        if(dto.getAuditPass()){
            one.setState(1);
            one.setReleaseTime(new Date());
            AppContentDetail byContentManageId = appContentDetailRepo.findByContentManageId(one.getContentManageId());
            AppMedium mediumInfo = appContentManageDao.findMediumInfo(one.getContentManageId());
            ycqFlowNode.setResultType(1);
            // 推送es
            sendEs(one,byContentManageId,mediumInfo);
        }else {
            ycqFlowNode.setResultType(3);
            one.setState(3);
        }
        one.setUpdateBy(worker.getLoginId());
        one.setUpdateTime(new Date());
        baseRepo.saveAndFlush(one);

        ycqFlowNode.setBizId(one.getContentManageId());
        ycqFlowNode.setReason(dto.getReason());
        pushFlow(worker,ycqFlowNode);
        YcqMessageTemplate template = ycqMessageTemplateRepo.findBySystemNameAndSceneNameAndTitleAndDelFlag(YOUTH_HOT, FINISH_AUDIT_MSG, FINISH_AUDIT_MSG, (byte) 0);
        if(Objects.nonNull(template)){
            sendAuthMsg(one,worker.getLoginId(),template,dto.getAuditPass());
        }else {
            log.info("站内信通知模板未配置");
        }
        return "更新成功";
    }

    private void sendAuthMsg(AppContentManage one ,Integer loginId,YcqMessageTemplate template,Boolean auditPass) {
        String contextTitle = one.getContentTitle();
        SendMessageTaskReqDTO dto = new SendMessageTaskReqDTO();
        if(contextTitle.length()>25){
            contextTitle = contextTitle.substring(0, 25).concat("...");
        }
        dto.setScopeType(4);
        dto.setScopeData(one.getCreateBy()+"");
        dto.setSendWay(template.getPushType());
        dto.setMessageType(template.getPushEndpoint());
        dto.setMessageContent(String.format(template.getMessageTemplate(), contextTitle, YouthHotAuditEnum.getTrans(auditPass)));
        dto.setDataSource(template.getSystemName());
        dto.setMessageClassify(template.getTitle());
        dto.setTemplateId(-1L);
        dto.setSendLoginId(loginId.longValue());
        // 查询要闻
        int isImportNews = 0;
        Long importantNewsLabel = appContentManageDao.getImportantNewsLabel();
        String labelName = one.getLabelName();
        if(StringUtils.hasText(labelName)){
            if(labelName.equals(importantNewsLabel+"")){
                isImportNews = 1;
            }
        }
        Map<String,Object> pathJson = new HashMap<>(6);
        pathJson.put("type","youthHot");
        pathJson.put("isImportNews",isImportNews);
        pathJson.put("id",one.getContentManageId()+"");
        pathJson.put("fileType",one.getFileType());
        dto.setPathAndJson(JsonUtil.getJsonString(pathJson));

        Result result = messageService.saveMessageTask(dto);
        if(!result.isSuccess()){
            log.info("青年热点推送审核消息失败------》{}",result.getMsg());
        }
    }

    @Override
    public List<AuditFlowVo> queryAuditFlow(Long contentManageId, Worker worker) {
        List<AuditFlowVo> auditFlowVos = appContentManageDao.queryAuditFlow(contentManageId);
        if(CollectionUtils.isEmpty(auditFlowVos)){
            return auditFlowVos;
        }
        AuditFlowVo auditFlowVo = auditFlowVos.get(auditFlowVos.size() - 1);
        if(auditFlowVo.getIsSubmitUser() == 1){
            // 查询当前组织审核人
            AuditFlowVo auditFlowVo1 = new AuditFlowVo();
            Integer uploadRoleId = configProperty.getAuditRoleId();
            OrgAuthTreeVo vo = appContentManageDao.findOrgName(contentManageId);
            List<String> auditOrgUser = appContentManageDao.findAuditOrgUser(vo.getOrgId(), uploadRoleId);
            if(!CollectionUtils.isEmpty(auditOrgUser)){
                auditOrgUser = appContentManageDao.findUserName(auditOrgUser);
                auditFlowVo1.setOperator(String.join(",", auditOrgUser));
            }
            auditFlowVo1.setIsSubmitUser(2);
            auditFlowVo1.setOrgName(vo.getOrgName());
            auditFlowVos.add(auditFlowVo1);
        }
        return auditFlowVos;
    }

    private void pushFlow(Worker worker,YcqFlowNode ycqFlowNode) {
        ycqFlowNode.setOrgId(Long.parseLong(worker.getOrgId()));
        ycqFlowNode.setOrgName(worker.getOrgName());
        ycqFlowNode.setCreateTime(new Date());
        ycqFlowNode.setCreateBy(worker.getWorkerName());
        ycqFlowNode.setCreateId(Long.parseLong(worker.getLoginId()+""));
        ycqFlowNode.setResultStr(ycqFlowNode.getResultType()==1?"审核通过":"审核不通过");
        ycqFlowNode.setBizType(TaskTypeEnums.HOT.getCode());
        ycqFlowNode.setApproveType(1);
        flowNodeRepo.saveAndFlush(ycqFlowNode);
    }

    /**
     * 验证审核权限
     */
    private void validatedUserAudit(AppContentManage one, Worker worker) {
        // 查询当前用户审核状态是否是已审核状态
        if(Objects.isNull(one)){
            throw new BusinessException("当前文章不存在");
        }
        Integer waitState = 2;
        // 查询当前文章是否待审核状态
        if(!waitState.equals(one.getState())){
            throw new BusinessException("当前文章已审核");
        }
    }

    private void validatedIsPush(List<Long> contentManageIds) {
        List<Long> collect = contentManageIds.stream().distinct().collect(Collectors.toList());
        int count =  appContentManageDao.findStateIsPush(collect);
        if(count != collect.size()){
            throw new BusinessException("不能删除已上架文章");
        }
    }

    /**
     * 参数校验
     */
    private String validateParams(AppContentManage one) {
        if(!StringUtils.hasText(one.getContentTitle())){
            throw new BusinessException("内容标题不能为空");
        }
        AppContentDetail byContentManageId = appContentDetailRepo.findByContentManageId(one.getContentManageId());
        if(!StringUtils.hasText(byContentManageId.getPicKey())){
            throw new BusinessException("封面图不能为空");
        }
        if(!StringUtils.hasText(byContentManageId.getArticleContext())){
            throw new BusinessException("文章内容不能为空");
        }
        if(videoType.equals(one.getFileType())){
            if(!StringUtils.hasText(byContentManageId.getFileId())){
                throw new BusinessException("请上传视频");
            }
        }
        return byContentManageId.getArticleContext();
    }

    private AppMedium findMediumIdByWorkerId(int loginId) {
        AppMedium medium = appContentManageDao.findMediumByWorkerId(loginId);
        if(Objects.isNull(medium)){
            throw new BusinessException("该账号未绑定媒体或已停用");
        }
        return medium;
    }

    /**
     * 查询用户角色 false 上传员  true审核员 null 无角色
     */
    private Boolean findIsAuditRole(Worker worker){
        // 超级管理员默认给超管角色
        if(isSuperAdmin(worker.getLoginId())){
            return true;
        }
        Integer uploadRoleId = 11;
        Integer auditRoleId = 22;
        List<Integer> roleIds = worker.getRoles().stream().map(TsmRoleDto::getRoleId).collect(Collectors.toList());
        if(roleIds.contains(uploadRoleId)){
            return false;
        }
        if(roleIds.contains(auditRoleId)){
            return true;
        }
        throw new BusinessException("无权限");
    };
}
