package com.crunii.micro.service.appsecond.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.RandomUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.BeanUtil;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.message.PushMsgTemplateReqBo;
import com.crunii.micro.domain.entity.main.QczdIndustryField;
import com.crunii.micro.domain.entity.main.QczdSpaceContent;
import com.crunii.micro.domain.entity.main.QczdSpaceTeam;
import com.crunii.micro.domain.entity.main.QczdTeamStay;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.MessageCommonService;
import com.crunii.micro.service.appsecond.api.bo.AppCqzdTeamStayBo;
import com.crunii.micro.service.appsecond.api.dto.AppCqzdSpaceContentDto;
import com.crunii.micro.service.appsecond.api.vo.*;
import com.crunii.micro.service.appsecond.enums.SecondPushMessageEnum;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums;
import com.crunii.micro.service.appsecond.enums.YouthSpaceEnums;
import com.crunii.micro.service.appsecond.jdbc.AppCqzdSpaceContentDao;
import com.crunii.micro.service.appsecond.service.AppCqzdSpaceContentService;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

/**
 * @author denggl
 * date 2023/12/11
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class AppCqzdSpaceContentServiceImpl implements AppCqzdSpaceContentService {

    private  final QczdFocusOnAreasRepo qczdFocusOnAreasRepo ;
    private final QczdIndustryFieldRepo  qczdIndustryFieldRepo;
    private  final QczdPriceObjectRepo qczdPriceObjectRepo;
    private  final QczdSharingSettingsRepo qczdSharingSettingsRepo ;
    private  final QczdSpaceContentRepo qczdSpaceContentRepo;
    private  final QczdTeamStayRepo qczdTeamStayRepo;
    private  final  QczdSpaceTeamRepo qczdSpaceTeamRepo;
     private  final  AppCqzdSpaceContentDao appCqzdSpaceContentDao;
     private final QczdRelationFileRepo qczdRelationFileRepo;

     private final MessageCommonService messageCommonService;
     private static final String SETTLE_INFO="已入驻";
    private final String TeamCodePrefix = "QCKJ";

    @Override
    public AppCqzdSpaceIntroductionVo introduction() {
        AppCqzdSpaceIntroductionVo appCqzdSpaceIntroductionVo=new AppCqzdSpaceIntroductionVo();
        appCqzdSpaceIntroductionVo.setReportNumber(appCqzdSpaceContentDao.reportNumber());//待入驻
        appCqzdSpaceIntroductionVo.setSettleNumber(appCqzdSpaceContentDao.settleNumber());//已入驻
        List<AppCqzdTeamStayVo> qczdTeamStayLimitFive = appCqzdSpaceContentDao.findQczdTeamStayLimitFive();//最新5条
        List<String> collect = qczdTeamStayLimitFive.stream().map(itme -> {
            String info = "";
            if ((StayTeamEnums.DemandEnum.NO_NEED.getNum()).equals(itme.getBusinessLicense())) {
                info = itme.getApplicantName();
            } else {
                info = itme.getCompanyName();
            }
            return info + SETTLE_INFO+itme.getSpaceName();
        }).collect(Collectors.toList());
        appCqzdSpaceIntroductionVo.setBroadcast(collect);
        return appCqzdSpaceIntroductionVo;
    }


    @Override
    @InjectProperty
    public PageRsp<AppCqzdSpaceContentVo> pageAppCqzdSpaceContent(PageReq<AppCqzdSpaceContentDto> req, Worker worker) {
        PageRsp<AppCqzdSpaceContentVo> pageRsp=  appCqzdSpaceContentDao.pageAppCqzdSpaceContent(req,worker);
        if (CollectionUtil.isEmpty(pageRsp.getResult())){
            return pageRsp;
        }
        List<Long> collect = pageRsp.getResult().stream().map(item -> item.getId()).collect(Collectors.toList());
        Map<Long, List<QczdRelationFileVo>> file = getSpaceFile(collect).stream().collect(Collectors.groupingBy(QczdRelationFileVo::getServiceId));
        Map<Long, List<FocusOnAreaVo>> focus = getSpaceFocusOnAreas(collect).stream().collect(Collectors.groupingBy(FocusOnAreaVo::getSpaceId));
        pageRsp.getResult().forEach(item->{
            if (CollectionUtil.isNotEmpty(file)) {
                item.setFileVoList(file.get(item.getId()));
            }
            if (CollectionUtil.isNotEmpty(focus)) {
                item.setFocusOnAreas(focus.get(item.getId()));
            }
          //  item.setSharingSettings(getSharingSettings(item.getId()));
        });
        return pageRsp;
    }

    /**
     * 空间文件
     * @param serviceId
     * @return
     */
    private List<QczdRelationFileVo> getSpaceFile(List<Long> serviceId){
        // 1 图片,2 视频
        List<Integer> list=new ArrayList<>();
        list.add(YouthSpaceEnums.FileTypeEnum.ONCE_TYPE.getNum());
        list.add(YouthSpaceEnums.FileTypeEnum.SECOND_TYPE.getNum());
        List<QczdRelationFileVo> fileVoList =qczdRelationFileRepo.findByServiceIdInAndFileTypeIn(serviceId,list).stream().map(file->{
            QczdRelationFileVo qczdRelationFileVo=new QczdRelationFileVo();
            BeanUtil.copyProperties(file,qczdRelationFileVo);
            return qczdRelationFileVo;
        }).collect(Collectors.toList());
        return fileVoList;
    }

    private List<SaringSettingsVo> getSharingSettings(List<Long> serviceId){
        List<SaringSettingsVo> collect = qczdSharingSettingsRepo.findBySpaceIdIn(serviceId).stream().map(item ->
                {
                    SaringSettingsVo saringSettingsVo = new SaringSettingsVo();
                    saringSettingsVo.setSharingSetting(  item.getConfigName());
                    saringSettingsVo.setSpaceId(item.getSpaceId());
                    return saringSettingsVo;
                }
              ).collect(Collectors.toList());
        return collect;
    }


    private List<FocusOnAreaVo>  getSpaceFocusOnAreas(List<Long> serviceId){
        List<FocusOnAreaVo> collect = qczdFocusOnAreasRepo.findBySpaceIdIn(serviceId).stream().map(item ->{
                    FocusOnAreaVo focusOnAreaVo = new FocusOnAreaVo();
                    focusOnAreaVo.setFocusOnArea(item.getFieldName());
                    focusOnAreaVo.setSpaceId(item.getSpaceId());
                    return focusOnAreaVo;
                }
                ).collect(Collectors.toList());
        return collect;
    }


    private List<IndustryFieldVo> getTeamIndustryField(Long teamId){
        List<IndustryFieldVo> allByTeamId = qczdIndustryFieldRepo.findByTeamId(teamId).stream().map(item->
                {
                    IndustryFieldVo industryFieldVo = new IndustryFieldVo();
                    industryFieldVo.setIndustryField(item.getIndustryField());
                    return industryFieldVo;
                })
              .collect(Collectors.toList());
        return allByTeamId;
    }
    @InjectProperty
    @Override
    public AppCqzdSpaceContentVo cqzdSpaceContentdetails(Long spaceId, Worker worker) {
        AppCqzdSpaceContentVo cqzdSpaceContent =appCqzdSpaceContentDao.findQczdSpaceContent(spaceId,worker);
        if (cqzdSpaceContent==null){
            return null;
        }
        AppCqzdSpaceContentVo appCqzdSpaceContentVo=new AppCqzdSpaceContentVo();
        BeanUtil.copyProperties(cqzdSpaceContent,appCqzdSpaceContentVo);
        //空间对应图片文件
        List<Long> list=new ArrayList<>();
        list.add(cqzdSpaceContent.getId());
        appCqzdSpaceContentVo.setFileVoList(getSpaceFile(list));
        appCqzdSpaceContentVo.setFocusOnAreas(getSpaceFocusOnAreas(list));
        appCqzdSpaceContentVo.setSharingSettings(getSharingSettings(list));
        return appCqzdSpaceContentVo;
    }


    /** app空间详情分享 */
    @InjectProperty
    @Override
    public AppCqzdSpaceContentVo spaceContentDetails(Long spaceId) {
        AppCqzdSpaceContentVo cqzdSpaceContent =appCqzdSpaceContentDao.findOneSpaceDetails(spaceId);
        if (ObjectUtil.isEmpty(cqzdSpaceContent)){
            return null;
        }
        AppCqzdSpaceContentVo appCqzdSpaceContentVo=new AppCqzdSpaceContentVo();
        BeanUtil.copyProperties(cqzdSpaceContent,appCqzdSpaceContentVo);
        // 处理空间团队申请的  res.audit_state,res.team_id ,当前用户只能申请同一个空间一次
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isNotEmpty(currWorker)){
            List<QczdSpaceTeam> spaceTeams = qczdSpaceTeamRepo.findBySpaceId(spaceId);
            if (ObjectUtil.isNotEmpty(spaceTeams)){
                // 获取空间对应团队id
                List<Long> teamIds = spaceTeams.stream().filter(item -> item.getTeamId() != null)
                    .map(QczdSpaceTeam::getTeamId).collect(
                        Collectors.toList());
                List<QczdTeamStay> teamStays = qczdTeamStayRepo.findAllById(teamIds);
                if (ObjectUtil.isNotEmpty(teamStays)){
                    // 通过创建时间倒序
                    List<QczdTeamStay> existTeams = teamStays.stream().filter(
                            item -> ObjectUtil.equals(item.getCreateId(), currWorker.getLoginId()) && ObjectUtil.isNotEmpty(item.getCreatedTime())).sorted((dt1, dt2) -> dt2.getCreatedTime().compareTo(dt1.getCreatedTime()))
                        .collect(Collectors.toList());
                    if (ObjectUtil.isNotEmpty(existTeams)){
                        QczdTeamStay qczdTeamStay = existTeams.get(0);
                        appCqzdSpaceContentVo.setAuditState(qczdTeamStay.getAuditState());
                        appCqzdSpaceContentVo.setTeamId(qczdTeamStay.getId());
                    }
                }
            }
        }


        //空间对应图片文件
        List<Long> list=new ArrayList<>();
        list.add(cqzdSpaceContent.getId());
        appCqzdSpaceContentVo.setFileVoList(getSpaceFile(list));
        appCqzdSpaceContentVo.setFocusOnAreas(getSpaceFocusOnAreas(list));
        appCqzdSpaceContentVo.setSharingSettings(getSharingSettings(list));
        return appCqzdSpaceContentVo;
    }

    private void checkQczdTeamStay(AppCqzdTeamStayBo cqzdTeamStayBo){


        //选择营业执照校验
        if (StayTeamEnums.DemandEnum.YES_NEED.getNum().equals(cqzdTeamStayBo.getBusinessLicense())){
                if (StringUtils.isBlank(cqzdTeamStayBo.getCompanyName())){
                    throw new BusinessException("企业名称必填!");
                }else {
                       if ( cqzdTeamStayBo.getCompanyName().length()>50){
                           throw new BusinessException("企业名称输入不超过50个字符!");
                    }
              }
        }
    }


    @Override
    public void saveApplycCqzdSpace(AppCqzdTeamStayBo cqzdTeamStayBo, Worker worker) {
        QczdSpaceContent spaceContent = qczdSpaceContentRepo.findById(cqzdTeamStayBo.getSpaceId())
            .orElseThrow(() -> new BusinessException("青创空间不存在！"));
        checkQczdTeamStay(cqzdTeamStayBo);;

        QczdTeamStay appCqzdTeamStay=new QczdTeamStay();
        BeanUtil.copyProperties(cqzdTeamStayBo,appCqzdTeamStay);
        appCqzdTeamStay.setCreateId(worker.getLoginId());
        appCqzdTeamStay.setCreatedTime(LocalDateTime.now());
        appCqzdTeamStay.setCreatedBy(worker.getWorkerName());
        appCqzdTeamStay.setApplyTime(appCqzdTeamStay.getCreatedTime());
        int age=0;
        try {
              age = IdcardUtil.getAgeByIdCard(cqzdTeamStayBo.getIdCode());
        } catch (Exception e) {
            throw new BusinessException("身份证号异常!");
        }
        appCqzdTeamStay.setAge(age);
        appCqzdTeamStay.setCreatePoint(1);
        appCqzdTeamStay.setDelFlag(0);
        Snowflake snowflake = new Snowflake();
        appCqzdTeamStay.setId(snowflake.nextId());
        appCqzdTeamStay.setCreateId(worker.getLoginId());
        appCqzdTeamStay.setAuditState(StayTeamEnums.SteamAuditStateEnum.STAY_AUDIT.getNum());//待审核
        if (age<18){
            appCqzdTeamStay.setAuditState(StayTeamEnums.SteamAuditStateEnum.NO_PASS_AUDIT.getNum());//默认不通过
            appCqzdTeamStay.setReasonDetail("年龄小于18岁，不符合申请条件。");
            appCqzdTeamStay.setAuditTime(LocalDateTime.now());
            appCqzdTeamStay.setAuditWorkerName("系统");
            appCqzdTeamStay.setAuditReason("Others"); //其他原因

        }
        String crateTiem = DateUtil.format(new Date(), DateUtil.defaultDT1Pattern);
        appCqzdTeamStay.setTeamCode(TeamCodePrefix+crateTiem+generateRandomCode(4)); //团队编码
        QczdSpaceTeam qczdSpaceTeam=new QczdSpaceTeam();
        qczdSpaceTeam.setSpaceId(cqzdTeamStayBo.getSpaceId());
        qczdSpaceTeam.setTeamId(appCqzdTeamStay.getId());
        qczdSpaceTeam.setDelFlag(0);
        qczdTeamStayRepo.saveAndFlush(appCqzdTeamStay);
        qczdSpaceTeamRepo.saveAndFlush(qczdSpaceTeam); //团队关联空间
        saveQczdIndustryField(appCqzdTeamStay.getId(),cqzdTeamStayBo.getIndustryFields());

        // 申请成功后，发送消息给空间的创建人
        if (ObjectUtil.equals(appCqzdTeamStay.getAuditState(),StayTeamEnums.SteamAuditStateEnum.STAY_AUDIT.getNum())){
            PushMsgTemplateReqBo pushMsgTemplateReqBo = new PushMsgTemplateReqBo();
            pushMsgTemplateReqBo.setSystemName(SecondPushMessageEnum.TEAM_STAY_APPLY.getSystemName());
            pushMsgTemplateReqBo.setSceneName(SecondPushMessageEnum.TEAM_STAY_APPLY.getSceneName());
            pushMsgTemplateReqBo.setTitle(SecondPushMessageEnum.TEAM_STAY_APPLY.getTitle());
            HashMap<String, Object> map = new HashMap<>();
            pushMsgTemplateReqBo.setMap(map);
            messageCommonService.pushMsg(pushMsgTemplateReqBo, Long.valueOf(spaceContent.getCreateId()));
        }

    }

    /**
     * 随机生成num位编码(数字+英文字母)
     */
    private static String generateRandomCode(int num) {
        return RandomUtil.randomString("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", num);
    }

    private void saveQczdIndustryField(Long iteamId , List<String> list){
        List<QczdIndustryField> collect = list.stream().map(item -> {
            QczdIndustryField qczdIndustryField = new QczdIndustryField();
            qczdIndustryField.setDelFlag(0);
            qczdIndustryField.setIndustryField(item);
            qczdIndustryField.setTeamId(iteamId);
            return qczdIndustryField;
        }).collect(Collectors.toList());

        qczdIndustryFieldRepo. saveAllAndFlush(collect);
    }

    @Override
    public void revoke(Long id, Worker worker) {
        QczdTeamStay cqzdTeamStay = qczdTeamStayRepo.findById(id).orElseThrow(() -> new BusinessException("数据异常！"));
        cqzdTeamStay.setAuditState( StayTeamEnums.SteamAuditStateEnum.CANCEL_AUDIT.getNum());
        cqzdTeamStay.setUpdateId(worker.getLoginId());
        cqzdTeamStay.setUpdatedBy(worker.getWorkerName());
        cqzdTeamStay.setUpdatedTime(LocalDateTime.now());
        qczdTeamStayRepo.saveAndFlush(cqzdTeamStay);
    }

    @Override
    public AppSpaceTeamApplyVo applyList(Worker worker) {
        List<AppCqzdTeamStayVo> appCqzdTeamStayVos = appCqzdSpaceContentDao.applyList(worker);
        if (CollectionUtil.isEmpty(appCqzdTeamStayVos)){
            return null;
        }
        //未审核
        List<AppCqzdTeamStayVo> wait = appCqzdTeamStayVos.stream().filter(item ->
                StayTeamEnums.SteamAuditStateEnum.STAY_AUDIT.getNum().equals(item.getAuditState())
        ).collect(Collectors.toList());

        //审核结束
        List<AppCqzdTeamStayVo> apply = appCqzdTeamStayVos.stream().filter(item ->
                 StayTeamEnums.SteamAuditStateEnum.PASS_AUDIT.getNum().equals( item.getAuditState())||
                         StayTeamEnums.SteamAuditStateEnum.NO_PASS_AUDIT.getNum().equals(item.getAuditState())
        ).collect(Collectors.toList());
        AppSpaceTeamApplyVo appSpaceTeamApplyVo=new AppSpaceTeamApplyVo();
        appSpaceTeamApplyVo.setWaitApply(wait);
        appSpaceTeamApplyVo.setApply(apply);
        return appSpaceTeamApplyVo;
    }

    @Override
    @InjectProperty
    public AppCqzdTeamStayVo getBaseInfo(Integer businessLicense, Worker worker) {
       List<AppCqzdTeamStayVo>   appCqzdTeamStayVos= appCqzdSpaceContentDao.getBaseInfo(businessLicense,worker);
         if (CollectionUtil.isEmpty(appCqzdTeamStayVos)){
             return null;
         }
        //返回最新的数据
        AppCqzdTeamStayVo appCqzdTeamStayVo = appCqzdTeamStayVos.get(0);
        appCqzdTeamStayVo.setIndustryFields(getTeamIndustryField(appCqzdTeamStayVo.getId()));
        return appCqzdTeamStayVo;
    }

    @Override
    public void updateApplycCqzdSpace(AppCqzdTeamStayBo cqzdTeamStayBo, Worker worker) {
        QczdTeamStay qczdTeamStay = qczdTeamStayRepo.findById(cqzdTeamStayBo.getId()).orElseThrow(() -> new BusinessException("数据异常！"));
        if (!StayTeamEnums.SteamAuditStateEnum.STAY_AUDIT.getNum().equals(qczdTeamStay.getAuditState())){
            throw  new BusinessException("非待审核状态！");
        }else {
            int age=0;
            try {
                age = IdcardUtil.getAgeByIdCard(cqzdTeamStayBo.getIdCode());
            } catch (Exception e) {
                throw new BusinessException("身份证号异常!");
            }
            qczdIndustryFieldRepo.deleteByTeamId(qczdTeamStay.getId());
            saveQczdIndustryField(qczdTeamStay.getId(), cqzdTeamStayBo.getIndustryFields());
            //保存创建信息
            BeanUtils.copyProperties(cqzdTeamStayBo,qczdTeamStay,new String[]{"createdBy","createTime,createId"});
            qczdTeamStay.setAuditState(StayTeamEnums.SteamAuditStateEnum.STAY_AUDIT.getNum());//待审核
            if (age<18){
                qczdTeamStay.setAuditState(StayTeamEnums.SteamAuditStateEnum.NO_PASS_AUDIT.getNum());//默认不通过
                qczdTeamStay.setReasonDetail("年龄小于18岁，不符合申请条件。");
                qczdTeamStay.setAuditTime(LocalDateTime.now());
                qczdTeamStay.setAuditWorkerName("系统");
                qczdTeamStay.setAuditReason("Others"); //其他原因

            }
            qczdTeamStay.setUpdateId(worker.getLoginId());
            qczdTeamStay.setCreatePoint(1);
            qczdTeamStay.setAge(IdcardUtil.getAgeByIdCard(cqzdTeamStayBo.getIdCode()));
            qczdTeamStay.setUpdatedBy(worker.getWorkerName());
            qczdTeamStay.setUpdatedTime(LocalDateTime.now());
            qczdTeamStayRepo.saveAndFlush(qczdTeamStay);
        }
    }

    @Override
    @InjectProperty
    public AppCqzdTeamStayVo getTeamStay(Long teamId, Worker worker) {
        QczdTeamStay qczdTeamStay = qczdTeamStayRepo.findById(teamId).orElseThrow(() -> new BusinessException("数据异常！"));
        AppCqzdTeamStayVo appCqzdTeamStay=new AppCqzdTeamStayVo();
        BeanUtil.copyProperties(qczdTeamStay,appCqzdTeamStay);
        appCqzdTeamStay.setIndustryFields(getTeamIndustryField(qczdTeamStay.getId()));
        return appCqzdTeamStay;
    }

    /**
     * 申请进度
     * @param teamId
     * @param spaceId
     */
    @Override
    @InjectProperty
    public  List<QczdTeamStayschedule>  schedule(Long teamId, Long spaceId) {
        QczdTeamStay qczdTeamStay = qczdTeamStayRepo.findById(teamId).orElseThrow(() -> new BusinessException("数据异常"));
        List<QczdTeamStayschedule> qczdTeamStayschedule=new ArrayList<>();
        //申请人
        QczdTeamStayschedule qczdTeamStayschedule1=new QczdTeamStayschedule();
        qczdTeamStayschedule1.setApplicantName (qczdTeamStay.getApplicantName());
        qczdTeamStayschedule1.setApplyTime(Date.from(qczdTeamStay.getApplyTime().atZone(ZoneId.systemDefault()).toInstant()));
        qczdTeamStayschedule.add(qczdTeamStayschedule1);
        //是否审核
        if (  !StayTeamEnums.SteamAuditStateEnum.STAY_AUDIT.getNum().equals(qczdTeamStay.getAuditState()) && !StayTeamEnums.SteamAuditStateEnum.CANCEL_AUDIT.getNum().equals(qczdTeamStay.getAuditState())){
            QczdTeamStayschedule qczdTeamStayschedule2=new QczdTeamStayschedule();
            qczdTeamStayschedule2.setAuditWorkerName(qczdTeamStay.getAuditWorkerName());
            qczdTeamStayschedule2.setAuditState(qczdTeamStay.getAuditState());
            if (qczdTeamStay.getAuditTime()!=null) {
                qczdTeamStayschedule2.setAuditTime(Date.from(qczdTeamStay.getAuditTime().atZone(ZoneId.systemDefault()).toInstant()));
            }
            qczdTeamStayschedule2.setReson(qczdTeamStay.getAuditReason());
            if (StringUtils.isNotBlank(qczdTeamStay.getReasonDetail())) {
                qczdTeamStayschedule2.setAuditReason(qczdTeamStay.getReasonDetail());
                qczdTeamStayschedule2.setReson(null);
            }
            qczdTeamStayschedule.add(qczdTeamStayschedule2);
        }
        //是否孵化
        if (qczdTeamStay.getOperateId()!=null){
            QczdTeamStayschedule qczdTeamStayschedule3=new QczdTeamStayschedule();
            qczdTeamStayschedule3.setOperateName(qczdTeamStay.getOperateName());
            qczdTeamStayschedule3.setOperateTime(Date.from(qczdTeamStay.getOperateTime().atZone(ZoneId.systemDefault()).toInstant()));
            //qczdTeamStayschedule3.setTeamState();
            qczdTeamStayschedule.add(qczdTeamStayschedule3);
        }

          return qczdTeamStayschedule;

    }
}
