package com.crunii.micro.service.appsecond.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.AppDiscoverPosition;
import com.crunii.micro.domain.entity.main.AppDiscoverPositionAnnex;
import com.crunii.micro.domain.repo.main.AppDiscoverPositionAnnexRepo;
import com.crunii.micro.domain.repo.main.AppDiscoverPositionRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.appsecond.api.bo.AppDiscoverPositionAnnexBo;
import com.crunii.micro.service.appsecond.api.bo.AppDiscoverPositionBo;
import com.crunii.micro.service.appsecond.api.bo.AppQueryPositionBo;
import com.crunii.micro.service.appsecond.api.vo.AppDiscoverListVo;
import com.crunii.micro.service.appsecond.api.vo.AppDiscoverPositionAnnexVo;
import com.crunii.micro.service.appsecond.api.vo.AppDiscoverPositionVo;
import com.crunii.micro.service.appsecond.converter.AppDiscoverPositionAnnexConverter;
import com.crunii.micro.service.appsecond.converter.AppDiscoverPositionConverter;
import com.crunii.micro.service.appsecond.jdbc.AppDiscoverPositionDao;
import com.crunii.micro.service.appsecond.service.IAppDiscoverPositionService;
import com.crunii.micro.service.appsecond.utils.PositionUtil;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

/**
 * @author lyx
 * @date 2023/12/7
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class AppDiscoverPositionServiceImpl implements IAppDiscoverPositionService {

	private final AppDiscoverPositionDao appDiscoverPositionDao;
	private final AppDiscoverPositionRepo baseRepo;
	private final AppDiscoverPositionConverter baseConverter;

	private final AppDiscoverPositionAnnexRepo annexRepo;
	private final AppDiscoverPositionAnnexConverter annexConverter;

	private final AppDiscoverPositionDao discoverPositionDao;


	@Override
	public Result<List<AppDiscoverListVo>> queryAppList(AppQueryPositionBo bo) {
		if(Objects.isNull(bo.getLatitude()) || Objects.isNull(bo.getLongitude())){
			throw new BusinessException("经纬度必传！");
		}
		double maxLong = 0.0;
		double minLong =0.0;
		double maxLat =0.0;
		double minLat =0.0;
		if (Objects.nonNull(bo.getDistance())){
//			//计算最大经度
//			maxLong = PositionUtil.calculateNewLongitude(bo.getLongitude().doubleValue(), bo.getDistance());
//			//计算最小经度
//			minLong = PositionUtil.calculateNewLongitude(bo.getLongitude().doubleValue(), 0-bo.getDistance());
//			//计算最大纬度
//			maxLat = PositionUtil.calculateNewLatitude(bo.getLatitude().doubleValue(), bo.getDistance());
//			//计算最小纬度
//			minLat = PositionUtil.calculateNewLatitude(bo.getLatitude().doubleValue(), 0-bo.getDistance());


			//先计算查询点的经纬度范围
			double r = PositionUtil.EARTH_RADIUS;//地球半径千米
			int dis = bo.getDistance();//米/距离
			double dlng =  2*Math.asin(Math.sin(dis/(2*r))/Math.cos(bo.getLatitude().doubleValue()*Math.PI/180));
			dlng = dlng*180/Math.PI;//角度转为弧度
			double dlat = dis/r;
			dlat = dlat*180/Math.PI;
			minLat =bo.getLatitude().doubleValue()-dlat;
			maxLat = bo.getLatitude().doubleValue()+dlat;
			minLong = bo.getLongitude().doubleValue() -dlng;
			maxLong = bo.getLongitude().doubleValue() + dlng;
		}


		List<AppDiscoverListVo> list = appDiscoverPositionDao.queryAppList(bo,maxLong,minLong,maxLat,minLat);
		for (AppDiscoverListVo vo:list) {
			Integer distance = PositionUtil.getDistance(bo.getLongitude().doubleValue(), bo.getLatitude().doubleValue(), vo.getLongitude().doubleValue(), vo.getLatitude().doubleValue());
			vo.setDistance(distance);
		}
		if (Objects.nonNull(bo.getDistance())){
			list = list.stream().filter(o -> o.getDistance().compareTo(bo.getDistance()) != 1).collect(Collectors.toList());
		}

		list.sort(Comparator.comparing(AppDiscoverListVo::getDistance));
		return new Result<>(list);
	}

	/**
	 * 查询发现阵地主
	 */
	@Override
	public AppDiscoverPositionVo queryById(Long id){
		AppDiscoverPosition appDiscoverPosition = baseRepo.getOne(id);
		AppDiscoverPositionVo vo =baseConverter.entityToVo(appDiscoverPosition);

		List<AppDiscoverPositionAnnex> files = annexRepo.findAppDiscoverPositionAnnexByBusinessIdAndVideoFlag(vo.getId(),0);
		if (CollectionUtil.isNotEmpty(files)){
			List<AppDiscoverPositionAnnexVo> fileVoList = annexConverter.entityListToVoList(files);
			vo.setFiles(fileVoList);
		}
		List<AppDiscoverPositionAnnex> videoList = annexRepo.findAppDiscoverPositionAnnexByBusinessIdAndVideoFlag(vo.getId(),1);
		if (CollectionUtil.isNotEmpty(videoList)){
			List<AppDiscoverPositionAnnexVo> videoVoList = annexConverter.entityListToVoList(videoList);
			vo.setFileVideoList(videoVoList);
		}


		return vo;
	}

	/**
	 * 查询发现阵地主列表
	 */
	@Override
	public Result<PageRsp<AppDiscoverPositionVo>> queryPageList(PageReq<AppDiscoverPositionBo> bo) {


		return Result.ofSuccess(discoverPositionDao.pageList(bo));
	}

	/**
	 * 查询发现阵地主列表
	 */
	@Override
	public List<AppDiscoverPositionVo> queryList(AppDiscoverPositionBo bo) {
		return null;
	}

	/**
	 * 新增发现阵地主
	 */
	@Override
	public Boolean insertByBo(AppDiscoverPositionBo bo) {
		AppDiscoverPosition add = baseConverter.boToEntity(bo);
		add.setDiscoverType(0);
		validEntityBeforeSave(add);
		baseRepo.save(add);
		//保存图片
		saveFile(bo.getFiles(), add.getId(), 0);
		//保存视频
		saveFile(bo.getFileVideoList(), add.getId(), 1);



		return true;
	}

	/**
	 * 修改发现阵地主
	 */
	@Override
	public Boolean updateByBo(AppDiscoverPositionBo bo) {
		AppDiscoverPosition update = baseConverter.boToEntity(bo);
		AppDiscoverPosition discoverPosition = baseRepo.getOne(bo.getId());
//		BeanUtil.copyProperties(update, discoverPosition, CopyOptions.create().setIgnoreNullValue(true));
		// 为null的也要更新
		BeanUtils.copyProperties(bo,discoverPosition,"id","files","fileVideoList");
		validEntityBeforeSave(update);
		baseRepo.saveAndFlush(discoverPosition);
		//删除附件
		annexRepo.deleteByBusinessIds(discoverPosition.getId());
		//附件不为空
		saveFile(bo.getFiles(), update.getId(), 0);

		saveFile(bo.getFileVideoList(), update.getId(), 1);


		return true;
	}

	/**
	 * @description: 保存附近
	 * @param businessId 业务id
	 * @param videoFlag  是否视频
	 * @return:
	 * @author liuchuan
	 * @date: 2023/12/19 14:20
	*/
	private void saveFile(List<AppDiscoverPositionAnnexBo> boList,Long businessId,Integer videoFlag){
		if (CollectionUtil.isNotEmpty(boList)){
			List<AppDiscoverPositionAnnex> annexList = Lists.newArrayList();
			boList.stream().forEach(fileBo->{
				AppDiscoverPositionAnnex annex = annexConverter.boToEntity(fileBo);
				annex.setBusinessId(businessId);
				annex.setDelFlag(0);
				annex.setVideoFlag(videoFlag);
				annexList.add(annex);
			});
			annexRepo.saveAllAndFlush(annexList);
		}
	}

	/**
	 * 保存前的数据校验
	 */
	private void validEntityBeforeSave(AppDiscoverPosition entity){
		Worker worker = CurrWorkerUtil.getCurrWorker();
		if (ObjectUtil.isEmpty(entity.getId())){
//		entity.setCreateBy(String.valueOf(worker.getLoginId()));
			entity.setCreateTime(new Date());
		}
		entity.setUpdateTime(new Date());
		entity.setDelFlag(0);
	}

	/**
	 * 批量删除发现阵地主
	 */
	@Override
	public Boolean deleteWithValidByIds(Long id, Boolean isValid) {
		if(isValid){

		}
		baseRepo.deleteById(id);
		return true;
	}

}
