package com.crunii.micro.service.appsecond.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.AppDownloadRecord;
import com.crunii.micro.domain.repo.main.AppDownloadRecordRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.appsecond.api.dto.AppDownloadRecordSaveDTO;
import com.crunii.micro.service.appsecond.api.dto.LabelDto;
import com.crunii.micro.service.appsecond.api.dto.LabelUpload;
import com.crunii.micro.service.appsecond.api.vo.AppDownLoadFileVo;
import com.crunii.micro.service.appsecond.api.vo.ImagesVo;
import com.crunii.micro.service.appsecond.jdbc.AppDownRecordDao;
import com.crunii.micro.service.appsecond.service.AppDownloadRecordService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author lyx
 * @date 2023/12/8
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class AppDownloadRecordServiceImpl implements AppDownloadRecordService {
	private final AppDownloadRecordRepo recordRepo;

	private final AppDownRecordDao jdbc;
	@Value("${upload.url}")
	private String URL_PREFIX;

	@Override
	public Result<Boolean> saveDownloadRecord(Worker worker, AppDownloadRecordSaveDTO dto) {
		log.info("保存下载记录入参日志-------------------------------------------"+dto.toString());
		if (Objects.isNull(worker) || Objects.isNull(dto)
				|| Objects.isNull(dto.getType()) ||  Objects.isNull(dto.getFileId())) {
			throw new BusinessException("参数异常！");
		}
		if (1 == dto.getType()) {
			if (StringUtils.isEmpty(dto.getPath())) {
				throw new BusinessException("文件下载必传下载后的手机路径！");
			}
			if (StringUtils.isEmpty(dto.getFileUrl())) {
				throw new BusinessException("文件下载必传fileUrl！");

			}
		}
		AppDownloadRecord record = recordRepo
				.findByFileIdAndTypeAndCreateIdAndDelFlagOrderByCreateTimeDesc(dto.getFileId(),dto.getType(), Long.valueOf(worker.getLoginId()),0);
		if (Objects.isNull(record)) {
			Snowflake snowflake = new Snowflake();
			record = new AppDownloadRecord();
			record.setPath(dto.getPath());
			record.setFileUrl(dto.getFileUrl());
			record.setId(snowflake.nextId());
			record.setFileId(dto.getFileId());
			record.setType(dto.getType());
			record.setCreateBy(worker.getWorkerName());
			record.setCreateId(Long.valueOf(worker.getLoginId()));
			record.setCreateTime(LocalDateTime.now());
		} else {
			record.setCreateTime(LocalDateTime.now());
			record.setPath(dto.getPath());
		}
		recordRepo.save(record);
		return new Result<>(true);
	}

	@Override
	public Result<List<AppDownloadRecord>> queryAppList(Worker worker, Integer type) {
		List<AppDownloadRecord> list = recordRepo
				.findAllByTypeAndCreateIdAndDelFlagOrderByCreateTimeDesc(type, Long.valueOf(worker.getLoginId()), 0);
		return new Result<>(list);
	}

	@Override
	public Result<PageRsp<ImagesVo>> queryList(PageReq<LabelDto> req) {
		Result<PageRsp<ImagesVo>> result = new Result<>();
		Worker worker = CurrWorkerUtil.getCurrWorker();
		PageRsp<ImagesVo> images = jdbc.queryList(req, worker);

		List<ImagesVo> imagesVos = images.getResult();
		if (CollectionUtils.isNotEmpty(imagesVos)) {
			// 查询标签
			List<Long> uploadIds = imagesVos.stream().map(ImagesVo::getUploadId).collect(Collectors.toList());
			List<LabelUpload> list = jdbc.findLabelIdAndName(uploadIds);
			Map<Long, List<LabelUpload>> labelCollect = new HashMap<>(16);
			if (CollectionUtils.isNotEmpty(list)) {
				labelCollect = list.stream().collect(Collectors.groupingBy(LabelUpload::getUploadId));
			}
			List<String> picList = imagesVos.stream().map(ImagesVo::getFileKey).collect(Collectors.toList());
			List<ImagesVo> picInfo = jdbc.queryFileInfo(picList);
			Map<String, ImagesVo> collect = picInfo.stream().collect(Collectors.toMap(ImagesVo::getFileKey, Function.identity()));
			for (ImagesVo vo : imagesVos) {
				// 查询该id对应标签设置
				List<LabelUpload> list1 = labelCollect.get(vo.getUploadId());
				List<String> labels = new ArrayList<>();
				for (LabelUpload labelUpload : list1) {
					String labelName = labelUpload.getLabelName();
					labels.add(labelName);
				}
				vo.setLabels(labels);
				vo.setUrl(URL_PREFIX + vo.getFileKey());
				// 查询图片长高
				ImagesVo imagesVo = collect.get(vo.getFileKey());
				if (Objects.nonNull(imagesVo)) {
					vo.setWidth(imagesVo.getWidth());
					vo.setHeight(imagesVo.getHeight());
					vo.setFileName(imagesVo.getFileName());
				}
			}
		}
		result.setData(images);
		return result;
	}

	@Override
	public Result<PageRsp<AppDownLoadFileVo>> queryFileList(PageReq<LabelDto> req) {
		Worker worker = CurrWorkerUtil.getCurrWorker();

		PageRsp<AppDownLoadFileVo> images = jdbc.queryFileList(req, worker);
		return new Result<>(images);
	}
}
