package com.crunii.micro.service.appsecond.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Snowflake;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.enums.AuditStatusEnums;
import com.crunii.micro.common.enums.ResultEnums;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.*;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.appsecond.api.bo.AppHelpAnnexBo;
import com.crunii.micro.service.appsecond.api.bo.AppHelpCenterTypeBo;
import com.crunii.micro.service.appsecond.api.dto.data.ExcelDataInfo;
import com.crunii.micro.service.appsecond.api.dto.data.ExcelMistakeData;
import com.crunii.micro.service.appsecond.api.dto.data.ExcelHelpCenterData;
import com.crunii.micro.service.appsecond.api.dto.data.MistakeDataInfo;
import com.crunii.micro.service.appsecond.api.vo.AppHelpAnnexVo;
import com.crunii.micro.service.appsecond.api.vo.AppHelpHotCenterVo;
import com.crunii.micro.service.appsecond.converter.AppHelpAnnexConverter;
import com.crunii.micro.service.appsecond.jdbc.AppHelpCenterDao;
import com.crunii.micro.service.appsecond.utils.EasyExcelUtils;
import com.crunii.micro.service.appsecond.utils.ExcelHelpCenterVerify;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import com.crunii.micro.service.appsecond.api.bo.AppHelpCenterBo;
import com.crunii.micro.service.appsecond.api.vo.AppHelpCenterVo;
import com.crunii.micro.service.appsecond.converter.AppHelpCenterConverter;
import com.crunii.micro.service.appsecond.service.IAppHelpCenterService;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * app帮助中心Service业务层处理
 *
 * @author liuchuan
 * @date 2023-12-06
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class AppHelpCenterServiceImpl implements IAppHelpCenterService {

	private final AppHelpCenterRepo baseRepo;
	private final AppHelpCenterConverter baseConverter;
	private final AppHelpAnnexConverter annexConverter;
	private final AppHelpAnnexRepo annexRepo;
	private final AppHelpCenterDao baseJdbc;
	private final AppHelpCenterTempRepo appHelpCenterTempRepo;
	private final ExcelHelpCenterVerify excelStuVerify;
	private final AppHelpHistoryRepo appHelpHistoryRepo;
	private final AppNotHelpCenterRepo appNotHelpCenterRepo;

	public static final int EXCEL_MAX_RAW = 500;
	public static final String EXCEL_DEMO_PATH = "static/帮助中心导入模板.xlsx";


	/**
	 * 查询app帮助中心
	 */
	@Override
	public AppHelpCenterVo queryById(Long id) {
		AppHelpCenterVo vo = baseJdbc.queryById(id);
		List<AppHelpAnnex> annexes = annexRepo.findByBusinessIdAndDelFlag(id, 0);
		ArrayList<AppHelpAnnexVo> annexVos = new ArrayList<>();
		//附件设置进去
		if (CollectionUtils.isNotEmpty(annexes)) {
			for (AppHelpAnnex annex : annexes) {
				AppHelpAnnexVo annexVo = annexConverter.entityToVo(annex);
				annexVos.add(annexVo);
			}
			vo.setAnnexs(annexVos);
		}
		return vo;
	}

	/**
	 * 查询app帮助中心列表
	 */
	@Override
	public Result<PageRsp<AppHelpCenterVo>> queryPageList(PageReq<AppHelpCenterBo> bo) {
		PageRsp<AppHelpCenterVo> page = baseJdbc.queryPageList(bo);
		List<AppHelpCenterVo> result = page.getResult();
		List<Long> list = result.stream().map(AppHelpCenterVo::getId).collect(Collectors.toList());
		List<AppHelpAnnex> annexs = annexRepo.findAllByBusinessIdInAndDelFlag(list,0);
		for (AppHelpCenterVo appHelpCenterVo : result) {
			ArrayList<AppHelpAnnexVo> annexVoes = new ArrayList<>();
			//附件设置进去
			if (CollectionUtils.isNotEmpty(annexs)) {
				for (AppHelpAnnex annex : annexs) {
					if (appHelpCenterVo.getId().equals(annex.getBusinessId())){
						AppHelpAnnexVo annexVo = annexConverter.entityToVo(annex);
						annexVoes.add(annexVo);
					}
				}
				if (CollectionUtil.isNotEmpty(annexVoes)){
					appHelpCenterVo.setAnnexs(annexVoes);
				}
			}
		}
		return new Result<>(page);
	}

	/**
	 * 查询app帮助中心列表
	 */
	@Override
	public List<AppHelpCenterVo> queryList(AppHelpCenterBo bo) {
		return null;
	}

	/**
	 * 新增app帮助中心
	 */
	@Override
	public Boolean insertByBo(AppHelpCenterBo bo) {
		Worker worker = CurrWorkerUtil.getCurrWorker();
		validata(bo);
		Snowflake snowflake = new Snowflake();

		List<AppHelpAnnexBo> annexs = bo.getAnnexs();
		ArrayList<AppHelpAnnex> helpAnnexes = new ArrayList<>();
		Long appHelpCenterId = snowflake.nextId();
		//修改编辑将原本附件失效
		if (Objects.nonNull(bo.getId())) {
			appHelpCenterId = bo.getId();
			//修改id一通过baseConverter  写入
			List<AppHelpAnnex> flag = annexRepo.findByBusinessIdAndDelFlag(bo.getId(), 0);
			flag.stream().forEach(e -> e.setDelFlag(1));
			helpAnnexes.addAll(flag);
			AppHelpCenter appHelpCenter = baseRepo.findByIdAndDelFlag(bo.getId(), 0);
			BeanUtils.copyProperties(bo, appHelpCenter);
			appHelpCenter.setUpdateBy(worker.getWorkerName());
			appHelpCenter.setUpdateTime(LocalDateTime.now());
			baseRepo.save(appHelpCenter);

		} else {
			//新增将id写入
			if (3!=bo.getHelpType()&&4!=bo.getHelpType()){
				AppHelpCenter entity = baseRepo.findByHelpTypeAndDelFlag(bo.getHelpType(), 0);
				if (Objects.nonNull(entity)){
					throw new BusinessException("此类型已存在，不允许新增！");
				}
			}
			AppHelpCenter add = baseConverter.boToEntity(bo);
			add.setDelFlag(0);
			add.setId(appHelpCenterId);
			add.setCreateBy(worker.getWorkerName());
			add.setCreateTime(LocalDateTime.now());
			add.setClickCount(0L);
			add.setNotHelpCount(0L);
			baseRepo.save(add);
		}
		//创建新的附件
		if (CollectionUtils.isNotEmpty(annexs)) {
			for (AppHelpAnnexBo annex : annexs) {
				AppHelpAnnex helpAnnex = annexConverter.boToEntity(annex);
				helpAnnex.setId(snowflake.nextId());
				helpAnnex.setBusinessId(appHelpCenterId);
				helpAnnexes.add(helpAnnex);
			}
		}
		if (CollectionUtils.isNotEmpty(helpAnnexes)) {
			annexRepo.saveAll(helpAnnexes);
		}

		return true;
	}

	/**
	 * 数据校验
	 *
	 * @param bo
	 */
	private void validata(AppHelpCenterBo bo) {
		if (Objects.isNull(bo.getHelpType())) {
			throw new BusinessException("类型必选！");
		}
		if (StringUtils.isEmpty(bo.getQuestion())) {
			throw new BusinessException("问题必填！");
		}
		if (StringUtils.isEmpty(bo.getAnswer())) {
			throw new BusinessException("答案必填！");
		}
		if (StringUtils.isEmpty(bo.getKeywords())) {
			throw new BusinessException("关键词必填！");
		}
	}

	/**
	 * 修改app帮助中心
	 */
	@Override
	public Boolean updateByBo(AppHelpCenterBo bo) {
		AppHelpCenter update = baseConverter.boToEntity(bo);
		validEntityBeforeSave(update);
		return null;
	}

	/**
	 * 保存前的数据校验
	 */
	private void validEntityBeforeSave(AppHelpCenter entity) {
		//TODO 做一些数据校验,如唯一约束
	}

	/**
	 * 批量删除app帮助中心
	 */
	@Override
	public Boolean deleteById(Long id) {
		AppHelpCenter center = baseRepo.findById(id).get();
		center.setDelFlag(1);
		List<AppHelpAnnex> annexes = annexRepo.findByBusinessIdAndDelFlag(id, 0);
		//附件设置进去
		if (CollectionUtils.isNotEmpty(annexes)) {
			for (AppHelpAnnex annex : annexes) {
				annex.setDelFlag(1);
			}
		}
		baseRepo.saveAndFlush(center);
		annexRepo.saveAllAndFlush(annexes);
		return true;
	}

	@Override
	public Result<List<AppHelpHotCenterVo>> queryAppHotList() {
		List<AppHelpHotCenterVo> limit = baseJdbc.findAllOrOrderByClickCountDescLimit();
		return new Result<>(limit);
	}

	@Override
	public Result<List<AppHelpHotCenterVo>> appGet(String question) {
		if (StringUtils.isEmpty(question)){
			throw new BusinessException("请输入您想搜索的问题");
		}
		Worker worker = CurrWorkerUtil.getCurrWorker();
		List<AppHelpHotCenterVo> list = baseJdbc.findAllByQuestionLike(question);
		if (CollectionUtils.isEmpty(list)){
			//存入问题库中
			AppNotHelpCenter center = new AppNotHelpCenter();
			center.setQuestion(question);
			center.setDelFlag(0);
			center.setCreateBy(worker.getWorkerName());
			center.setCreateTime(LocalDateTime.now());
			appNotHelpCenterRepo.saveAndFlush(center);
			return Result.ofFail("未搜索到类似问题，已记录到问题库！");
		}
		return new Result<>(list);
	}

	@Override
	public AppHelpHotCenterVo doGetById(Long id) {
		Worker worker = CurrWorkerUtil.getCurrWorker();
		AppHelpCenter entity = baseRepo.findByIdAndDelFlag(id, 0);
		entity.setClickCount(entity.getClickCount() + 1);
		baseRepo.saveAndFlush(entity);
		List<AppHelpAnnex> annexs = annexRepo.findByBusinessIdAndDelFlag(entity.getId(), 0);
		List<AppHelpAnnexVo> annexVos = new ArrayList<>();
		AppHelpHistory history = appHelpHistoryRepo.findAllByCreatedIdAndSearchHistoryAndDelFlag(Long.valueOf(worker.getLoginId()),entity.getQuestion(),0);
		if (Objects.isNull(history)){
			history = new AppHelpHistory();
			history.setSearchHistory(entity.getQuestion());
			history.setDelFlag(0);
			history.setCreatedId(Long.valueOf(worker.getLoginId()));
			history.setCreatedBy(worker.getWorkerName());
			history.setCreatedTime(LocalDateTime.now());
		}else{
			history.setCreatedTime(LocalDateTime.now());
		}
		appHelpHistoryRepo.saveAndFlush(history);
		AppHelpHotCenterVo vo = new AppHelpHotCenterVo();
		if (CollectionUtil.isNotEmpty(annexs)){
			for (AppHelpAnnex annex : annexs) {
				AppHelpAnnexVo annexVo = annexConverter.entityToVo(annex);
				annexVos.add(annexVo);
			}
			if (CollectionUtils.isNotEmpty(annexVos)) {
				vo.setAnnexVos(annexVos);
			}
		}
		vo.setId(entity.getId());
		vo.setAnswer(entity.getAnswer());
		vo.setUrl(entity.getUrl());
		return vo;
	}

	@Override
	public Object downloadModel() {
		ResponseEntity<InputStreamResource> response = null;

		try {
			response = EasyExcelUtils.downloadExcelDemo(EXCEL_DEMO_PATH, "帮助中心导入模板");

		} catch (Exception e) {
			e.printStackTrace();
		}
		return response;
	}

	@Override
	public Result doVerifyExcel(MultipartFile file) {
		//ExcelUtil.checkModel(file,ExcelHelpCenterData.class,0);
		EasyExcelUtils.validateExcelHeaders(EXCEL_DEMO_PATH,file);
		ExcelHelpCenterData excelHelpCenterData = new ExcelHelpCenterData();
		ExcelDataInfo info = new ExcelDataInfo();
		List<ExcelHelpCenterData> excelHelpCenterDataList;
		List<MistakeDataInfo> lists = new ArrayList<>();
		List<ExcelMistakeData> dataList;
		try {
			excelHelpCenterDataList = ExcelUtil.readSingleExcel(file, excelHelpCenterData, 1);
		} catch (Exception e) {
			e.printStackTrace();
			throw new BusinessException("导入失败,请使用导入模板!");
		}
		if (CollectionUtil.isEmpty(excelHelpCenterDataList)) {
			throw new BusinessException("解析异常,查看数据是否规范");
		} else if (excelHelpCenterDataList.size() > EXCEL_MAX_RAW) {
			throw new BusinessException("超出最大导入条数");
		}
		// 开始校验参数
		for (ExcelHelpCenterData stuData : excelHelpCenterDataList) {
			dataList = excelStuVerify.data(stuData, String.valueOf(excelHelpCenterDataList.indexOf(stuData) + 2));
			if (CollectionUtil.isNotEmpty(dataList)) {
				MistakeDataInfo dataInfo = new MistakeDataInfo();
				dataInfo.setMistakeDataInfo(dataList);
				lists.add(dataInfo);
			}
		}
		// 校验通过保存临时表
		if (CollectionUtil.isEmpty(lists)) {
			List<AppHelpCenterTemp> tempList = new ArrayList<>();
			String dataKey = "app" + UUIDUtil.getUUID();
			excelHelpCenterDataList.forEach(e -> {
				AppHelpCenterTemp activityPeopleAuditTemp = new AppHelpCenterTemp();
				BeanUtil.copyPropertiesIgnoreNull(e, activityPeopleAuditTemp);
				activityPeopleAuditTemp.setDelFlag(0);
				if (e.getQuestion() != null) {
					activityPeopleAuditTemp.setQuestion(e.getQuestion());
				}
				if (e.getAnswer() != null) {
					activityPeopleAuditTemp.setAnswer(e.getAnswer());
				}
				if (e.getKeywords() != null) {
					activityPeopleAuditTemp.setKeywords(e.getKeywords());
				}
				if (e.getUrl() != null) {
					activityPeopleAuditTemp.setUrl(e.getUrl());
				}

				activityPeopleAuditTemp.setDataKey(dataKey);
				tempList.add(activityPeopleAuditTemp);
				info.setDataKey(dataKey);
			});
			appHelpCenterTempRepo.saveAll(tempList);
		}
		// 返回错误信息
		info.setLists(lists);
		return Result.ofSuccess(info);
	}

	@Override
	public Result doImportDate(String dataKey) {
		Worker currWorker = CurrWorkerUtil.getCurrWorker();
		List<AppHelpCenterTemp> tempData = appHelpCenterTempRepo.findAllByDataKey(dataKey);
		if (CollectionUtil.isEmpty(tempData)) {
			throw new BusinessException("导入失败,没有可导入数据！");
		}
		List<AppHelpCenter> peopleAudits = new ArrayList<>();
		tempData.forEach(x -> {
			AppHelpCenter ycqActivityPeopleAudit = new AppHelpCenter();
			BeanUtil.copyProperties(x, ycqActivityPeopleAudit);
			ycqActivityPeopleAudit.setNotHelpCount(0L);
			ycqActivityPeopleAudit.setClickCount(0L);
			ycqActivityPeopleAudit.setHelpType(3);
			ycqActivityPeopleAudit.setCreateBy(currWorker.getWorkerName());
			ycqActivityPeopleAudit.setCreateTime(LocalDateTime.now());
			peopleAudits.add(ycqActivityPeopleAudit);
		});
		baseRepo.saveAll(peopleAudits);
		// 导入成功后删除临时表数据
		appHelpCenterTempRepo.deleteAll(tempData);
		Result<String> result = new Result<>();
		result.setSuccess(true);
		result.setData(String.valueOf(tempData.size()));
		result.setMsg("导入成功");
		return result;
	}

	@Override
	public AppHelpHotCenterVo doGetByType(AppHelpCenterTypeBo appHelpCenterTypeBo) {
		if (Objects.isNull(appHelpCenterTypeBo) || Objects.isNull(appHelpCenterTypeBo.getHelpType())){
			throw new BusinessException("类型必传！");
		}
		if(appHelpCenterTypeBo.getHelpType()==2 && Objects.isNull(appHelpCenterTypeBo.getId())){
			throw new BusinessException("无帮助时数据id必传！");
		}
		AppHelpCenter appHelpCenter = baseRepo.findByHelpTypeAndDelFlag(appHelpCenterTypeBo.getHelpType(), 0);
		if(Objects.isNull(appHelpCenter)){
			return null;
		}
		AppHelpHotCenterVo vo = new AppHelpHotCenterVo();
		vo.setAnswer(appHelpCenter.getAnswer());
		if (appHelpCenterTypeBo.getHelpType()==2){
			AppHelpCenter appHelpCenter1 = baseRepo.findByIdAndDelFlag(appHelpCenterTypeBo.getId(), 0);
			appHelpCenter1.setNotHelpCount(appHelpCenter1.getNotHelpCount()+1);
			baseRepo.saveAndFlush(appHelpCenter1);
		}
		return vo;
	}
}
