package com.crunii.micro.service.appsecond.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.core.cache.TsmPropertyCache;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.AppPersonalHonor;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.repo.main.AppPersonalHonorRepo;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.appsecond.api.dto.AppPersonalBasicInfoDto;
import com.crunii.micro.service.appsecond.api.dto.AppPersonalHonorInfoDto;
import com.crunii.micro.service.appsecond.api.dto.AppPersonalInfoDto;
import com.crunii.micro.service.appsecond.api.vo.*;
import com.crunii.micro.service.appsecond.enums.AuthAppEnum;
import com.crunii.micro.service.appsecond.enums.SamePersonalFieldEnum;
import com.crunii.micro.service.appsecond.jdbc.AppPersonalInfoDao;
import com.crunii.micro.service.appsecond.service.IAppVolunteerService;
import com.crunii.micro.service.appsecond.utils.BeanCopyUtil;
import com.crunii.micro.service.appsecond.utils.Sm2Util;
import com.crunii.micro.service.sysmgr.api.domain.dto.OrgNode;
import jodd.util.StringUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.crunii.micro.service.appsecond.api.bo.AppPersonalInfoBo;
import com.crunii.micro.domain.entity.main.AppPersonalInfo;
import com.crunii.micro.domain.repo.main.AppPersonalInfoRepo;
import com.crunii.micro.service.appsecond.converter.AppPersonalInfoConverter;
import com.crunii.micro.service.appsecond.service.IAppPersonalInfoService;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 个人基础信息Service业务层处理
 *
 * @author liuchuan
 * @date 2023-12-06
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class AppPersonalInfoServiceImpl implements IAppPersonalInfoService {

    private final AppPersonalInfoConverter baseConverter;
    @Resource
    private  AppPersonalInfoRepo appPersonalInfoRepo;
    @Resource
    private AppPersonalHonorRepo appPersonalHonorRepo;
    @Resource
    private BeanCopyUtil beanCopyUtil;
    @Resource
    private TsmWorkerRepo tsmWorkerRepo;
    @Resource
    private AppPersonalInfoDao appPersonalInfoDao;
    @Resource
    private TsmPropertyCache tsmPropertyCache;
    @Resource
    private TsmOrgRepo tsmOrgRepo;
    @Value("${upload.url}")
    private String urlPrefix;
    @Resource
    private IAppVolunteerService iAppVolunteerService;

//    @Resource
//    private AppHomeConfigService appHomeConfigService;

    private final static String WORKER_NOT_EXISTS = "获取当前登录用户失败";

    /**
     * 青马工程SM2加密：privateKey
     */
    public static final String PRIVATE_KEY = "MIGTAgEAMBMGByqGSM49AgEGCCqBHM9VAYItBHkwdwIBAQQgs2S6qSZzbHdcjLOfNxY2P0CeqjItMc3IF6UjhgtEJNygCgYIKoEcz1UBgi2hRANCAASGFlOEnOP9p4AFFA0FopjAeX3HhDqaJE2yQJcdwsqleXH+XN1WTa4ae3+m3H6rf8RYRLMnwRf6K417i7E3yaa0";

    /**
     * 青马工程SM2 web加密：privateKey
     */
    public static final String PRIVATE_WEB_KEY = "184c211702d2bc05d8e5a1729c4295df7d96309397174d56f46ca04372ccc0b5";

    @Override
    public Result updateInfo(AppPersonalInfoDto dto) throws ParseException {
        //CurrWorkerUtil.mockCurrWorker();
        if (org.apache.commons.lang3.StringUtils.isNotBlank(dto.getWorkUnit()) && dto.getWorkUnit().length() > 50) {
            throw new RuntimeException("工作单位不允许超过50个字符");
        }
        if (ObjectUtil.isNotEmpty(dto.getOrgInfoDto())&&org.apache.commons.lang3.StringUtils.isNotBlank(dto.getOrgInfoDto().getPosition()) && dto.getOrgInfoDto().getPosition().length() > 50) {
            throw new RuntimeException("职务不允许超过50个字符");
        }
        Worker worker = getWorker();
        AppPersonalInfo personalInfo;
        if(ObjectUtil.isEmpty(dto.getId())){
            //新增 补充信息
            personalInfo = appPersonalInfoRepo.findByLoginId(Long.valueOf(worker.getLoginId()));
            if(ObjectUtil.isNotEmpty(personalInfo)){
                personalInfo.setUpdatedBy(Long.valueOf(worker.getLoginId()));
                personalInfo.setUpdatedTime(new Date());
            }else {
                personalInfo = new AppPersonalInfo();
                Snowflake snowflake = new Snowflake();
                personalInfo.setId(snowflake.nextId());
                personalInfo.setLoginId(Long.valueOf(worker.getLoginId()));
                personalInfo.setCreatedBy(Long.valueOf(worker.getLoginId()));
                personalInfo.setCreatedTime(new Date());
            }

        }else{
            personalInfo = appPersonalInfoRepo.findByInfoId(dto.getId());
            personalInfo.setUpdatedBy(Long.valueOf(worker.getLoginId()));
            personalInfo.setUpdatedTime(new Date());
        }
        personalInfo.setVolunteerOrg(dto.getWorkUnit());
        personalInfo.setVolunteerOrg(dto.getVolunteerOrg());
        personalInfo.setExpertise(dto.getExpertise());
        AppPersonalBasicInfoDto basicInfoDto = dto.getBasicInfoDto();
        if(ObjectUtil.isNotEmpty(basicInfoDto)){
            BeanUtils.copyProperties(basicInfoDto,personalInfo);
        }

        //修改少先队
        if(ObjectUtil.isNotEmpty(dto.getEduInfoDto())){
            // 校验学生是否存在
            StudentInfoByUserVo studentInfo = appPersonalInfoDao.selectOneStudentById(dto.getEduInfoDto().getEduId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            appPersonalInfoDao.updateOneStudentById(dto.getEduInfoDto(),worker);
        }

        //拿到当前用户，修改信息
        TsmWorker user = tsmWorkerRepo.findByLoginIdAndState(worker.getLoginId(),CommonConstants.STATE_VALID);
        //修改组织信息
        if(ObjectUtil.isNotEmpty(dto.getOrgInfoDto())){
            user.setRegistrationSystem(ObjectUtil.isEmpty(dto.getOrgInfoDto().getRegistrationSystem())?null:Integer.valueOf(dto.getOrgInfoDto().getRegistrationSystem()));
            if(StringUtils.hasText(dto.getOrgInfoDto().getLeagueCode())){
                //20240119 orgId不修改tsm_worker表
//                user.setOrgId(dto.getOrgInfoDto().getLeagueCode());
//                user.setLeagueCode(tsmOrgRepo.findCodeById(dto.getOrgInfoDto().getLeagueCode()));
                personalInfo.setOrgId(Long.valueOf(dto.getOrgInfoDto().getLeagueCode()));
            }
            user.setIdentityCategory(ObjectUtil.isEmpty(dto.getOrgInfoDto().getIdentityCategory())?null:Integer.valueOf(dto.getOrgInfoDto().getIdentityCategory()));
            if (StringUtil.isNotBlank(dto.getOrgInfoDto().getPosition())){
                user.setPosition(dto.getOrgInfoDto().getPosition());
            }
        }
        //用户表里的姓名，性别，头像也修改
        if(ObjectUtil.isNotEmpty(basicInfoDto)){
            if(StringUtil.isNotBlank(dto.getBasicInfoDto().getWorkerName())){
                user.setWorkerName(dto.getBasicInfoDto().getWorkerName());
            }
            if(ObjectUtil.isNotEmpty(dto.getBasicInfoDto().getSex())){
                user.setSex(dto.getBasicInfoDto().getSex());
            }
            if (StringUtil.isNotBlank(dto.getBasicInfoDto().getHeadPortraitPicKey())) {
                if(dto.getBasicInfoDto().getHeadPortraitPicKey().startsWith(urlPrefix)){
                    String replace = dto.getBasicInfoDto().getHeadPortraitPicKey().replace(urlPrefix,"");
                    user.setHeadPortraitPicKey(replace);
                }else{
                    user.setHeadPortraitPicKey(dto.getBasicInfoDto().getHeadPortraitPicKey());
                }
            }else {
                user.setHeadPortraitPicKey(null);
            }
        }

        tsmWorkerRepo.save(user);
        AppPersonalInfo save = appPersonalInfoRepo.save(personalInfo);
        //荣誉信息
        if(ObjectUtil.isNotEmpty(dto.getHonorInfoDtos())){
            //获取角色权限
            List<String> appCodes = appPersonalInfoDao.findRolewithApp(worker);
            //青马
            Integer info = appPersonalInfoDao.findInfo(worker.getLoginId());
            if(appCodes.contains(AuthAppEnum.TYPE_QM.getCode())&&null!=info){
                //逻辑删除青马荣誉
                appPersonalInfoDao.updateQmHonor(worker.getLoginId(),1);
                //新增
                for (AppPersonalHonorInfoDto honorInfoDto : dto.getHonorInfoDtos()) {
                    Snowflake snow = new Snowflake();
                    long id = snow.nextId();
                    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
                    appPersonalInfoDao.createQmHonor(id,worker.getLoginId(),simpleDateFormat.parse(honorInfoDto.getHonorTime()),honorInfoDto.getHonorDesc(),1);
                }
            }else{
                //逻辑删除原来的
                appPersonalHonorRepo.deleteHonors(personalInfo.getId());
                //新增荣誉信息
                ArrayList<AppPersonalHonor> honors = new ArrayList<>();
                for (AppPersonalHonorInfoDto honorInfoDto : dto.getHonorInfoDtos()) {
                    AppPersonalHonor honor = new AppPersonalHonor();
                    honor.setInfoId(save.getId());
                    honor.setHonorDesc(honorInfoDto.getHonorDesc());
                    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
                    honor.setHonorTime(simpleDateFormat.parse(honorInfoDto.getHonorTime()));
                    honors.add(honor);
                }
                appPersonalHonorRepo.saveAll(honors);
            }
        }
        //专长
        if(ObjectUtil.isNotEmpty(dto.getExpertise())){
            //获取角色权限
            List<String> appCodes = appPersonalInfoDao.findRolewithApp(worker);
            //青马
            if(appCodes.contains(AuthAppEnum.TYPE_QM.getCode())){
                //逻辑删除青马专长
                appPersonalInfoDao.updateQmHonor(worker.getLoginId(),2);
                //新增
                    Snowflake snow = new Snowflake();
                    long id = snow.nextId();
                    appPersonalInfoDao.createQmHonor(id,worker.getLoginId(),new Date(),dto.getExpertise(),2);

            }
        }
        return Result.ofSuccess();
    }

    /**
     * 查询个人信息
     */
    @Override
    @InjectProperty
    public Result<AppPersonalInfoVo> getInfo(Worker worker) throws ParseException {
        //获取角色权限
        List<String> appCodes = appPersonalInfoDao.findRolewithApp(worker);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //相同字段
        ArrayList<AppPersonalSameFieldVo> sameFieldVos = new ArrayList<>();
        //没有授权应用 仅查询基本信息
        AppPersonalInfoVo personalInfoVo = new AppPersonalInfoVo();
        //信息表
        AppPersonalInfo personalInfo = appPersonalInfoRepo.findByLoginId(Long.valueOf(worker.getLoginId()));
            //基础信息
            AppPersonalBasicInfoVo basicInfoVo =new AppPersonalBasicInfoVo();
            if(ObjectUtil.isNotEmpty(personalInfo)){
                //设置id
                personalInfoVo.setId(personalInfo.getId());
                BeanUtils.copyProperties(personalInfo,basicInfoVo);
                //相同政治面貌
                if(ObjectUtil.isNotEmpty(personalInfo.getPoliticCountenance())){
                    AppPersonalSameFieldVo sameFieldVo1 = new AppPersonalSameFieldVo();
                    sameFieldVo1.setAppCode(AuthAppEnum.TYPE_MQ.getCode());
                    sameFieldVo1.setUpdateTime(ObjectUtil.isEmpty(personalInfo.getUpdatedTime())?personalInfo.getCreatedTime():personalInfo.getUpdatedTime());
                    sameFieldVo1.setFieldCode(SamePersonalFieldEnum.TYPE_ZZMM.getCode());
                    sameFieldVo1.setFieldValue(personalInfo.getPoliticCountenance());
                    sameFieldVos.add(sameFieldVo1);
                }
                if(ObjectUtil.isNotEmpty(personalInfo.getWorkUnit())){
                    //相同工作单位
                    AppPersonalSameFieldVo sameFieldVo2 = new AppPersonalSameFieldVo();
                    sameFieldVo2.setAppCode(AuthAppEnum.TYPE_MQ.getCode());
                    sameFieldVo2.setUpdateTime(ObjectUtil.isEmpty(personalInfo.getUpdatedTime())?personalInfo.getCreatedTime():personalInfo.getUpdatedTime());
                    sameFieldVo2.setFieldCode(SamePersonalFieldEnum.TYPE_GZDW.getCode());
                    sameFieldVo2.setFieldValue(personalInfo.getWorkUnit());
                    sameFieldVos.add(sameFieldVo2);
                }
                if(ObjectUtil.isNotEmpty(personalInfo.getEducation())){
                    //相同学历
                    AppPersonalSameFieldVo sameFieldVo3 = new AppPersonalSameFieldVo();
                    sameFieldVo3.setAppCode(AuthAppEnum.TYPE_MQ.getCode());
                    sameFieldVo3.setUpdateTime(ObjectUtil.isEmpty(personalInfo.getUpdatedTime())?personalInfo.getCreatedTime():personalInfo.getUpdatedTime());
                    sameFieldVo3.setFieldCode(SamePersonalFieldEnum.TYPE_XL.getCode());
                    sameFieldVo3.setFieldValue(personalInfo.getEducation());
                    sameFieldVos.add(sameFieldVo3);
                }
                if(ObjectUtil.isNotEmpty(personalInfo.getIdNum())){
                    //相同身份证
                    AppPersonalSameFieldVo sameFieldVo4 = new AppPersonalSameFieldVo();
                    sameFieldVo4.setAppCode(AuthAppEnum.TYPE_MQ.getCode());
                    sameFieldVo4.setUpdateTime(ObjectUtil.isEmpty(personalInfo.getUpdatedTime())?personalInfo.getCreatedTime():personalInfo.getUpdatedTime());
                    sameFieldVo4.setFieldCode(SamePersonalFieldEnum.FIELD_SFZH.getCode());
                    sameFieldVo4.setFieldValue(personalInfo.getIdNum());
                    sameFieldVos.add(sameFieldVo4);
                }

                //荣誉信息
                List<AppPersonalHonor> appPersonalHonors = appPersonalHonorRepo.findByInfoId(personalInfo.getId());
                List<AppPersonalHonorVo> honorVos = beanCopyUtil.copyList(appPersonalHonors, AppPersonalHonorVo.class);
                personalInfoVo.setPersonalHonorVos(honorVos);
            }
                //拿到当前用户
                TsmWorker user = tsmWorkerRepo.findByLoginIdAndState(worker.getLoginId(), CommonConstants.STATE_VALID);
                //BeanUtils.copyProperties(worker,basicInfoVo);
                basicInfoVo.setWorkerName(user.getWorkerName());
                basicInfoVo.setSex(user.getSex());
                basicInfoVo.setMobile(user.getMobile());
                basicInfoVo.setHeadPortraitPicKey(ObjectUtil.isEmpty(user.getHeadPortraitPicKey())?null:urlPrefix+user.getHeadPortraitPicKey());




        //专长
        personalInfoVo.setExpertise(ObjectUtil.isEmpty(personalInfo)?null:personalInfo.getExpertise());
        //组织信息
        AppPersonalOrgInfoVo orgInfoVo = appPersonalInfoDao.queryEchoPersonalInformation(worker.getLoginId());

        if (null != orgInfoVo) {
                if(ObjectUtil.isNotEmpty(personalInfo)&&ObjectUtil.isNotEmpty(personalInfo.getOrgId())){
                    orgInfoVo.setOrgId(personalInfo.getOrgId().toString());
                    TsmOrg tsmOrg = tsmOrgRepo.findOrgById(personalInfo.getOrgId().toString());
                    orgInfoVo.setOrgIdText(ObjectUtil.isEmpty(tsmOrg)?null:tsmOrg.getOrgName());
                    if (null != tsmOrg.getRegistrationSystemType()) {
                        orgInfoVo.setRegistrationSystem(tsmOrg.getRegistrationSystemType());
                        if (tsmOrg.getRegistrationSystemType().equals(1)) {
                            orgInfoVo.setIdentityCategoryText(tsmPropertyCache.get("areaIdentityType", String.valueOf(orgInfoVo.getIdentityCategory())));
                        }
                        if (tsmOrg.getRegistrationSystemType().equals(3)) {
                            orgInfoVo.setIdentityCategoryText(tsmPropertyCache.get("cityIdentityType", String.valueOf(orgInfoVo.getIdentityCategory())));
                        }
                        if (tsmOrg.getRegistrationSystemType().equals(4)) {
                            orgInfoVo.setIdentityCategoryText(tsmPropertyCache.get("schoolIdentityType", String.valueOf(orgInfoVo.getIdentityCategory())));
                        }
                         }
                }else {
                    orgInfoVo.setOrgId(orgInfoVo.getOrgId());
                    if (null != orgInfoVo.getRegistrationSystem()) {
                        if (orgInfoVo.getRegistrationSystem().equals(1)) {
                            orgInfoVo.setIdentityCategoryText(tsmPropertyCache.get("areaIdentityType", String.valueOf(orgInfoVo.getIdentityCategory())));
                        }
                        if (orgInfoVo.getRegistrationSystem().equals(3)) {
                            orgInfoVo.setIdentityCategoryText(tsmPropertyCache.get("cityIdentityType", String.valueOf(orgInfoVo.getIdentityCategory())));
                        }
                        if (orgInfoVo.getRegistrationSystem().equals(4)) {
                            orgInfoVo.setIdentityCategoryText(tsmPropertyCache.get("schoolIdentityType", String.valueOf(orgInfoVo.getIdentityCategory())));
                        }
                    }
                }

        }
        personalInfoVo.setAppPersonalOrgInfoVo(orgInfoVo);
        //少先队
        if(appCodes.contains(AuthAppEnum.TYPE_SXD.getCode())){
            List<StudentInfoByUserVo> studentInfoByUserVos = appPersonalInfoDao.searchStudentInfoByUser(Long.valueOf(worker.getLoginId()));
            if(!CollectionUtils.isEmpty(studentInfoByUserVos)){
                StudentInfoByUserVo studentInfoByUserVo = studentInfoByUserVos.get(0);
                //相同政治面貌
//                AppPersonalSameFieldVo sameFieldVo1 = new AppPersonalSameFieldVo();
//                sameFieldVo1.setAppCode(AuthAppEnum.TYPE_SXD.getCode());
//                sameFieldVo1.setUpdateTime(sdf.parse(studentInfoByUserVo.getLastUpdatedTime()));
//                sameFieldVo1.setFieldCode(SamePersonalFieldEnum.TYPE_ZZMM.getCode());
//                sameFieldVo1.setFieldValue(studentInfoByUserVo.getPoliticalStatus());
//                sameFieldVos.add(sameFieldVo1);
                //相同身份证
                if(ObjectUtil.isNotEmpty(studentInfoByUserVo.getIdentityCardNo())){
                    AppPersonalSameFieldVo sameFieldVo2 = new AppPersonalSameFieldVo();
                    sameFieldVo2.setAppCode(AuthAppEnum.TYPE_SXD.getCode());
                    sameFieldVo2.setUpdateTime(ObjectUtil.isEmpty(studentInfoByUserVo.getLastUpdatedTime())?sdf.parse(studentInfoByUserVo.getCreatedTime()):sdf.parse(studentInfoByUserVo.getLastUpdatedTime()));
                    sameFieldVo2.setFieldCode(SamePersonalFieldEnum.FIELD_SFZH.getCode());
                    sameFieldVo2.setFieldValue(studentInfoByUserVo.getIdentityCardNo());
                    sameFieldVos.add(sameFieldVo2);
                }
                AppPersonalEduInfoVo eduInfoVo = beanCopyUtil.copyObject(studentInfoByUserVo, AppPersonalEduInfoVo.class);
                eduInfoVo.setEduId(studentInfoByUserVo.getStudentId());
                personalInfoVo.setAppPersonalEduInfoVo(eduInfoVo);
            }
        }
        //青马
        if(appCodes.contains(AuthAppEnum.TYPE_QM.getCode())){
            QmUserInfo qmUserInfo = appPersonalInfoDao.query(Long.valueOf(worker.getLoginId()));
            if(ObjectUtil.isNotEmpty(qmUserInfo)){
                //身份证解密
                //qmUserInfo.setIdNum(Sm2Util.decrypt(PRIVATE_KEY, false, qmUserInfo.getIdNum()));
                //相同政治面貌
                if(ObjectUtil.isNotEmpty(qmUserInfo.getPoliticCountenance())){
                    AppPersonalSameFieldVo sameFieldVo1 = new AppPersonalSameFieldVo();
                    sameFieldVo1.setAppCode(AuthAppEnum.TYPE_QM.getCode());
                    sameFieldVo1.setUpdateTime(ObjectUtil.isEmpty(qmUserInfo.getLastUpdatedTime())?sdf.parse(qmUserInfo.getCreateTime()):sdf.parse(qmUserInfo.getLastUpdatedTime()));
                    sameFieldVo1.setFieldCode(SamePersonalFieldEnum.TYPE_ZZMM.getCode());
                    sameFieldVo1.setFieldValue(qmUserInfo.getPoliticCountenance());
                    sameFieldVos.add(sameFieldVo1);
                }
                if(ObjectUtil.isNotEmpty(qmUserInfo.getWorkUnit())){
                    //相同工作单位
                    AppPersonalSameFieldVo sameFieldVo2 = new AppPersonalSameFieldVo();
                    sameFieldVo2.setAppCode(AuthAppEnum.TYPE_QM.getCode());
                    sameFieldVo2.setUpdateTime(ObjectUtil.isEmpty(qmUserInfo.getLastUpdatedTime())?sdf.parse(qmUserInfo.getCreateTime()):sdf.parse(qmUserInfo.getLastUpdatedTime()));
                    sameFieldVo2.setFieldCode(SamePersonalFieldEnum.TYPE_GZDW.getCode());
                    sameFieldVo2.setFieldValue(qmUserInfo.getWorkUnit());
                    sameFieldVos.add(sameFieldVo2);
                }

                //相同学历
                if(ObjectUtil.isNotEmpty(qmUserInfo.getEducation())){
                    AppPersonalSameFieldVo sameFieldVo3 = new AppPersonalSameFieldVo();
                    sameFieldVo3.setAppCode(AuthAppEnum.TYPE_QM.getCode());
                    sameFieldVo3.setUpdateTime(ObjectUtil.isEmpty(qmUserInfo.getLastUpdatedTime())?sdf.parse(qmUserInfo.getCreateTime()):sdf.parse(qmUserInfo.getLastUpdatedTime()));
                    sameFieldVo3.setFieldCode(SamePersonalFieldEnum.TYPE_XL.getCode());
                    sameFieldVo3.setFieldValue(qmUserInfo.getEducation());
                    sameFieldVos.add(sameFieldVo3);
                }
                //相同身份证
                if(ObjectUtil.isNotEmpty(qmUserInfo.getIdNum())){
                    AppPersonalSameFieldVo sameFieldVo4 = new AppPersonalSameFieldVo();
                    sameFieldVo4.setAppCode(AuthAppEnum.TYPE_QM.getCode());
                    sameFieldVo4.setUpdateTime(ObjectUtil.isEmpty(qmUserInfo.getLastUpdatedTime())?sdf.parse(qmUserInfo.getCreateTime()):sdf.parse(qmUserInfo.getLastUpdatedTime()));
                    sameFieldVo4.setFieldCode(SamePersonalFieldEnum.FIELD_SFZH.getCode());
                    sameFieldVo4.setFieldValue(decryptUserCardId(qmUserInfo.getIdNum(),false));
                    sameFieldVos.add(sameFieldVo4);
                }
                //荣誉和专长取青马的
                List<AppPersonalHonorVo> honorVos = appPersonalInfoDao.findWorkerHonor(worker.getLoginId());
                String experts = appPersonalInfoDao.findExperts(worker.getLoginId());
                personalInfoVo.setPersonalHonorVos(honorVos);
                personalInfoVo.setExpertise(experts);

            }
        }
        //志愿山城
        if(appCodes.contains(AuthAppEnum.TYPE_ZYSC.getCode())){
            Result result = iAppVolunteerService.getPersonalInformation();
            PersonalInformationVO data = (PersonalInformationVO) result.getData();
            if(ObjectUtil.isNotEmpty(data)){
                //设置志愿组织
                List<String> list = data.getJoinOrg();
                String collect = CollectionUtils.isEmpty(list)?null:list.stream().map(String::valueOf).collect(Collectors.joining(","));
                personalInfoVo.setVolunteerOrg(collect);
                //相同工作单位
                if(ObjectUtil.isNotEmpty(data.getCollegeName())){
                    AppPersonalSameFieldVo sameFieldVo1 = new AppPersonalSameFieldVo();
                    sameFieldVo1.setAppCode(AuthAppEnum.TYPE_ZYSC.getCode());
                    sameFieldVo1.setUpdateTime(ObjectUtil.isEmpty(data.getUpdateTime())?sdf.parse(data.getCreateTime()):sdf.parse(data.getUpdateTime()));
                    sameFieldVo1.setFieldCode(SamePersonalFieldEnum.TYPE_GZDW.getCode());
                    sameFieldVo1.setFieldValue(data.getCollegeName());
                    sameFieldVos.add(sameFieldVo1);
                }
                if(ObjectUtil.isNotEmpty(data.getCertNoSearch())){
                    //相同身份证
                    AppPersonalSameFieldVo sameFieldVo2 = new AppPersonalSameFieldVo();
                    sameFieldVo2.setAppCode(AuthAppEnum.TYPE_ZYSC.getCode());
                    sameFieldVo2.setUpdateTime(ObjectUtil.isEmpty(data.getUpdateTime())?sdf.parse(data.getCreateTime()):sdf.parse(data.getUpdateTime()));
                    sameFieldVo2.setFieldCode(SamePersonalFieldEnum.FIELD_SFZH.getCode());
                    sameFieldVo2.setFieldValue(data.getCertNoSearch());
                    sameFieldVos.add(sameFieldVo2);
                }
            }
        }

        Map<String, List<AppPersonalSameFieldVo>> listMap = sameFieldVos.stream().collect(Collectors.groupingBy(c -> c.getFieldCode()));
        if(ObjectUtil.isNotEmpty(listMap)){
            for (Map.Entry<String, List<AppPersonalSameFieldVo>> entry : listMap.entrySet()) {
                List<AppPersonalSameFieldVo> value = entry.getValue();
                value.sort((t1, t2) -> t2.getUpdateTime().compareTo(t1.getUpdateTime()));
                //身份证
                if(entry.getKey().equals(SamePersonalFieldEnum.FIELD_SFZH.getCode())&&ObjectUtil.isNotEmpty(value)){
                    basicInfoVo.setIdNum(DesensitizedUtil.idCardNum(value.get(0).getFieldValue(),4,4));
                }
                //政治面貌
                if(entry.getKey().equals(SamePersonalFieldEnum.TYPE_ZZMM.getCode())&&ObjectUtil.isNotEmpty(value)){
                    //青马
                    if(value.get(0).getAppCode().equals(AuthAppEnum.TYPE_QM.getCode())){
                        basicInfoVo.setPoliticCountenance(value.get(0).getFieldValue());
                        basicInfoVo.setPoliticCountenanceDesc(tsmPropertyCache.get("qm-politicalOutlook", value.get(0).getFieldValue()));
                    }else {
                        basicInfoVo.setPoliticCountenance(value.get(0).getFieldValue());
                    }

                }
                //工作单位
                if(entry.getKey().equals(SamePersonalFieldEnum.TYPE_GZDW.getCode())&&ObjectUtil.isNotEmpty(value)){
                    personalInfoVo.setWorkUnit(value.get(0).getFieldValue());
                }
                //学历
                if(entry.getKey().equals(SamePersonalFieldEnum.TYPE_XL.getCode())&&ObjectUtil.isNotEmpty(value)){
                    //青马
                    if(value.get(0).getAppCode().equals(AuthAppEnum.TYPE_QM.getCode())){
                        basicInfoVo.setEducation(value.get(0).getFieldValue());
                        basicInfoVo.setEducationDesc(tsmPropertyCache.get("qm-education", value.get(0).getFieldValue()));
                    }else {
                        basicInfoVo.setEducation(value.get(0).getFieldValue());
                    }
                }
            }
        }

//        if(ObjectUtil.isNotEmpty(basicInfoVo.getHeadPortraitPicKey())&&!basicInfoVo.getHeadPortraitPicKey().startsWith(urlPrefix)){
//            basicInfoVo.setHeadPortraitPicKey(urlPrefix+basicInfoVo.getHeadPortraitPicKey());
//        }
        basicInfoVo.setMobile(DesensitizedUtil.idCardNum(basicInfoVo.getMobile(),3,4));
        personalInfoVo.setAppPersonalBasicInfoVo(basicInfoVo);
        return Result.ofSuccess(personalInfoVo);
    }

    /**
     * 获取子组织
     */
    @Override
    public List<OrgNode> getOrgChildrenByParentOrgId(Long parentOrgId) {
        return appPersonalInfoDao.getOrgChildrenByParentOrgId(parentOrgId);
    }

    /**
     * @description: 获取用户
     * @author: limy
     * @date: 2023/12/12 19:31
     * @param: []
     * @return: com.crunii.micro.domain.dto.Worker
     **/
    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        return worker;
    }


    /**
     * 解密身份证号
     * @param userCardId 身份证号 加密后
     * @param isDesensitization 是否脱敏
     * @return str
     */
    public static String decryptUserCardId(String userCardId,boolean isDesensitization){
        if (ObjectUtil.isNotEmpty(userCardId)) {
            userCardId = Sm2Util.decrypt(PRIVATE_KEY, false, userCardId);
            userCardId = Sm2Util.decrypt(PRIVATE_WEB_KEY, true, userCardId);
            if(isDesensitization){
                userCardId = DesensitizedUtil.idCardNum(userCardId, 1, 2);
            }
        }
        return userCardId;
    }


}
