package com.crunii.micro.service.appsecond.service.impl;

import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;
import com.crunii.micro.service.appsecond.api.dto.AddFaceDTO;
import com.crunii.micro.service.appsecond.api.vo.QueryAccountVO;
import com.crunii.micro.service.appsecond.service.AppSetUserService;
import jodd.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * @author: LG
 * @date: 2023/12/14 15:08
 */
@Service
public class AppSetUserServiceImpl implements AppSetUserService {


    @Resource
    private TsmWorkerRepo tsmWorkerRepo;

    @Override
    public Result<String> updateFace(Worker worker, AddFaceDTO dto) {
        if (dto.getWhetherFace().equals(CommonConstants.STATE_INVALID)){
            return deleteFace(worker);
        }else if (dto.getWhetherFace().equals(CommonConstants.STATE_VALID)){
            if (StringUtil.isBlank(dto.getUserIdKey())){
                throw new BusinessException("请传入公钥");
            }
            Map<String, Object> props = new HashMap<>(2);
            props.put("mobile", worker.getMobile());
            TsmWorker curr = tsmWorkerRepo.getUniqueByProps(props);
            curr.setFaceKey(dto.getUserIdKey());
            tsmWorkerRepo.save(curr);
            return Result.ofSuccess("添加成功");
        }else {
            throw new BusinessException("请传入正确入参");
        }


    }

    @Override
    public Result<String> deleteFace(Worker worker) {
        Map<String, Object> props = new HashMap<>(2);
        props.put("mobile", worker.getMobile());
        TsmWorker curr = tsmWorkerRepo.getUniqueByProps(props);
        curr.setFaceKey(null);
        tsmWorkerRepo.save(curr);
        return Result.ofSuccess("删除成功");
    }

    @Override
    public Result<QueryAccountVO> queryAccount(Worker worker) {
        Map<String, Object> props = new HashMap<>(2);
        props.put("mobile", worker.getMobile());
        TsmWorker curr = tsmWorkerRepo.getUniqueByProps(props);
        QueryAccountVO queryAccountVO = new QueryAccountVO();
        if (StringUtil.isBlank(curr.getFaceKey())){
            queryAccountVO.setWhetherFace(false);
        }else{
            queryAccountVO.setWhetherFace(true);
        }
        return Result.ofSuccess(queryAccountVO);
    }
}
