package com.crunii.micro.service.appsecond.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.AppWorkerSearch;
import com.crunii.micro.domain.entity.main.TsmWorkerApp;
import com.crunii.micro.domain.entity.main.YcqAppConfigUrl;
import com.crunii.micro.domain.repo.main.AppWorkerSearchRepo;
import com.crunii.micro.domain.repo.main.TsmWorkerAppRepo;
import com.crunii.micro.domain.repo.main.YcqAppConfigUrlRepo;
import com.crunii.micro.service.appsecond.api.bo.WorkerAppReqVO;
import com.crunii.micro.service.appsecond.api.vo.AppAllRspVO;
import com.crunii.micro.service.appsecond.api.vo.WorkerAppRspVO;
import com.crunii.micro.service.appsecond.constants.ErrorMsgConstants;
import com.crunii.micro.service.appsecond.dao.AppSubjectDao;
import com.crunii.micro.service.appsecond.enums.AppSearchTypeEnum;
import com.crunii.micro.service.appsecond.service.AppSubjectService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author gongsongtao
 * @date 2023/12/13 9:49
 */
@Service
public class AppSubjectServiceImpl implements AppSubjectService {
    @Resource
    private TsmWorkerAppRepo tsmWorkerAppRepo;
    @Resource
    private YcqAppConfigUrlRepo ycqAppConfigUrlRepo;
    @Resource
    private AppSubjectDao appSubjectDao;
    @Value("${upload.url}")
    private String urlPrefix;
    @Resource
    private AppWorkerSearchRepo appWorkerSearchRepo;

    /**
     * 查询我的应用
     * @param worker
     * @return
     */
    @Override
    public List<WorkerAppRspVO> getMyApp(Worker worker) {
        List<WorkerAppRspVO> myApp = appSubjectDao.getMyApp(worker);
        myApp.forEach(e -> e.setLogoUrl(urlPrefix + e.getLogoUrl()));
        return myApp;
    }

    /**
     * 删除我的应用
     * @param id
     * @param worker
     * @return
     */
    @Override
    public Boolean deleteMyApp(Long id, Worker worker) {
        List<TsmWorkerApp> myApps = tsmWorkerAppRepo.findByWorkerIdAndAppId(worker.getLoginId(), id);
        if (myApps.size()==0){
            throw new BusinessException(ErrorMsgConstants.APP_MY_APP_DELETED);
        }
        List<Long> ids = myApps.stream().map(TsmWorkerApp::getId).collect(Collectors.toList());
        tsmWorkerAppRepo.deleteMyApp(ids);
        return true;
    }

    /**
     * 新增我的应用
     * @param workerAppReqVO
     * @param worker
     * @return
     */
    @Override
    public Boolean createMyApp(WorkerAppReqVO workerAppReqVO, Worker worker) {
        if (ObjectUtil.isEmpty(workerAppReqVO) || workerAppReqVO.getAppId()==null){
            throw new BusinessException(ErrorMsgConstants.REQUEST_ERROR);
        }
        //app是否存在
        YcqAppConfigUrl app = ycqAppConfigUrlRepo.getOne(workerAppReqVO.getAppId());
        if (ObjectUtil.isEmpty(app)|| app.getState()==0){
            throw new BusinessException(ErrorMsgConstants.APP_NOT_EXIST);
        }
        //app是否有效
        if (ObjectUtil.isNotEmpty(app) && ObjectUtil.isNotEmpty(app.getState()) && app.getState()==0){
            throw new BusinessException(ErrorMsgConstants.APP_NOT_VALID);
        }
        //该应用是否已添加
        List<TsmWorkerApp> myApps = tsmWorkerAppRepo.findByWorkerIdAndAppId(worker.getLoginId(), workerAppReqVO.getAppId());
        if (myApps.size()>0){
            throw new BusinessException(ErrorMsgConstants.APP_MY_APP_IS_EXIST);
        }
        //新增我的应用
        TsmWorkerApp tsmWorkerApp=new TsmWorkerApp();
        tsmWorkerApp.setAppId(workerAppReqVO.getAppId());
        tsmWorkerApp.setWorkerId(worker.getLoginId());
        tsmWorkerApp.setCreateBy(worker.getWorkerName());
        tsmWorkerApp.setCreateId((long)worker.getLoginId());
        tsmWorkerApp.setDelFlag(0);
        tsmWorkerApp.setCreateTime(new Date());
        tsmWorkerAppRepo.saveAndFlush(tsmWorkerApp);
        return true;
    }

    /**
     * app-专题-查询所有应用
     * @param workerAppReqVO
     * @return
     */
    @Override
    public List<AppAllRspVO> doFindAllApp(WorkerAppReqVO workerAppReqVO, Worker worker) {
        List<AppAllRspVO> allApp = appSubjectDao.getAllApp(workerAppReqVO, worker);
        allApp.forEach(e -> e.setLogoUrl(urlPrefix + e.getLogoUrl()));
        //记录应用搜索历史
        if (ObjectUtil.isNotEmpty(workerAppReqVO) && StringUtils.isNotEmpty(workerAppReqVO.getAppName())) {
            AppWorkerSearch appWorkerSearch = new AppWorkerSearch();
            appWorkerSearch.setWorkerId(worker.getLoginId());
            appWorkerSearch.setSearchContext(workerAppReqVO.getAppName());
            appWorkerSearch.setCreateTime(new Date());
            appWorkerSearch.setCreatorId(worker.getLoginId());
            appWorkerSearch.setDeleted(0);
            appWorkerSearch.setSearchType(AppSearchTypeEnum.SYSTEM.getCode());
            appWorkerSearchRepo.saveAndFlush(appWorkerSearch);
        }
        return allApp;
    }
}
