package com.crunii.micro.service.appsecond.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.appsecond.api.dto.MobilDTO;
import com.crunii.micro.service.appsecond.api.vo.PersonalInformationVO;
import com.crunii.micro.service.appsecond.api.vo.VolunteerActivityVO;
import com.crunii.micro.service.appsecond.api.vo.VolunteerDetailsVO;
import com.crunii.micro.service.appsecond.service.IAppVolunteerService;
import com.crunii.micro.service.appsecond.utils.Sm2Util;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Author: zwx
 * @CreateTime: 2023-12-08  16:28:52
 * @Description: 我的志愿服务
 */
@Service
@Slf4j
public class AppVolunteerServiceImpl implements IAppVolunteerService {

    @Value("${volunteer.testUrl}")
//    private String volunteerTestUrl="http://221.229.99.73:62221";
    private String volunteerTestUrl;

    @Value("${volunteer.productUrl}")
//    private String volunteerProductUrl="http://10.169.25.7:8222";
    private String volunteerProductUrl;

    @Value("${spring.profiles.active}")
    private String env;

    @Resource(name = "logRestTemplate")
    RestTemplate restTemplate;

    private final static String PUBLIC_KEY="04c079acf860e53fa8bf2947e9d0793700e0bef8436a590c8f706a65bdacc5c56bbe597183f4d7f0b1f1831a20238216d9103c17206f81bfc73984f60095f3ace7";

    @Override
    public Result getPersonalInformation() {

        Worker currWorker = getWorker();
        //调用志愿山城个人信息接口
        String encryptPhone = Sm2Util.encrypt(PUBLIC_KEY,Boolean.FALSE,currWorker.getMobile());
        MobilDTO mobilDTO = new MobilDTO();
        mobilDTO.setMobile(encryptPhone);
        mobilDTO.setActivityId(null);
        log.info("青春重庆请求参数:"+JSONUtil.toJsonStr(mobilDTO));
        // header根据实际情况设置，没有就空着
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<?> requestEntity = new HttpEntity<>(mobilDTO, headers);
        headers.add("Content-Type", "application/json");
        log.info("当前环境:"+env);
        String url = "h3cTest".equals(env) ? volunteerProductUrl : volunteerTestUrl;
        ParameterizedTypeReference<Result<PersonalInformationVO>> responseBodyType = new ParameterizedTypeReference<Result<PersonalInformationVO>>() {};
        Result<PersonalInformationVO> result = restTemplate.exchange(url + "/zyzForeign/volunteerInfo", HttpMethod.POST, requestEntity, responseBodyType).getBody();
        log.info("志愿山城响应参数:"+JSONUtil.toJsonStr(result));
        return Optional.ofNullable(result)
                .map(r -> r.isSuccess()?Result.ofSuccess(r.getData()):Result.ofFail(r.getMsg()))
                .orElse(Result.ofFail("获取数据失败,请联系管理员"));
    }

    @Override
    public Result getVolunteerActivity() {

        Worker currWorker = getWorker();
        String encrypt = Sm2Util.encrypt(PUBLIC_KEY,Boolean.FALSE,currWorker.getMobile());
        MobilDTO mobilDTO = new MobilDTO();
        mobilDTO.setMobile(encrypt);
        mobilDTO.setActivityId(null);
        log.info("青春重庆请求参数:"+JSONUtil.toJsonStr(mobilDTO));
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<?> requestEntity = new HttpEntity<>(mobilDTO, headers);
        headers.add("Content-Type", "application/json");
        log.info("当前环境:"+env);
        String url = "h3cTest".equals(env) ? volunteerProductUrl : volunteerTestUrl;
        ParameterizedTypeReference<Result<List<VolunteerActivityVO>>> responseBodyType = new ParameterizedTypeReference<Result<List<VolunteerActivityVO>>>() {};
        Result<List<VolunteerActivityVO>> result = restTemplate.exchange(url + "/zyzForeign/activityInfo", HttpMethod.POST, requestEntity, responseBodyType).getBody();
        log.info("志愿山城响应参数:"+JSONUtil.toJsonStr(result));
        return Optional.ofNullable(result)
                .map(r -> r.isSuccess()?Result.ofSuccess(r.getData()):Result.ofFail(r.getMsg()))
                .orElse(Result.ofFail("获取数据失败,请联系管理员"));
    }

    @Override
    public Result getVolunteerDetails(Integer activityId) {

        Worker currWorker = getWorker();
        String encrypt = Sm2Util.encrypt(PUBLIC_KEY,Boolean.FALSE,currWorker.getMobile());
        MobilDTO mobilDTO = new MobilDTO();
        mobilDTO.setMobile(encrypt);
        mobilDTO.setActivityId(activityId);
        log.info("青春重庆请求参数:"+JSONUtil.toJsonStr(mobilDTO));
        // header根据实际情况设置，没有就空着
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<?> requestEntity = new HttpEntity<>(mobilDTO, headers);
        headers.add("Content-Type", "application/json");
        log.info("当前环境:"+env);
        String url = "h3cTest".equals(env) ? volunteerProductUrl : volunteerTestUrl;
        ParameterizedTypeReference<Result<VolunteerDetailsVO>> responseBodyType = new ParameterizedTypeReference<Result<VolunteerDetailsVO>>() {};
        Result<VolunteerDetailsVO> result = restTemplate.exchange(url + "/zyzForeign/activityDetail", HttpMethod.POST, requestEntity, responseBodyType).getBody();
        log.info("志愿山城响应参数:"+JSONUtil.toJsonStr(result));
        return Optional.ofNullable(result)
                .map(r -> r.isSuccess()?Result.ofSuccess(r.getData()):Result.ofFail(r.getMsg()))
                .orElse(Result.ofFail("获取数据失败,请联系管理员"));
    }
    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException("获取当前登录用户失败");
        }
        return worker;
    }

}
