package com.crunii.micro.service.appsecond.service.impl;

import java.util.*;

import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.nacos.common.utils.MD5Utils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.AESUtil;
import com.crunii.micro.common.utils.Base64Util;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.common.utils.RSAUtil;
import com.crunii.micro.domain.dto.PicInfo;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.AppContentDetailRepo;
import com.crunii.micro.domain.repo.main.AppContentManageRepo;
import com.crunii.micro.domain.repo.main.AppWorkerSearchRepo;
import com.crunii.micro.feign.client.ElsticService;
import com.crunii.micro.feign.client.dto.HotEsPageDto;
import com.crunii.micro.feign.client.dto.YouthHotEsVo;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.YouthAuditDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.YouthPublicAuditDto;
import com.crunii.micro.service.appsecond.api.dto.UserViewDto;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.ContextAppDetailVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.YouthContextVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.YouthHotPublicDetailVo;
import com.crunii.micro.service.appsecond.jdbc.AppYouthHotDao;
import com.crunii.micro.service.appsecond.service.AppMultiModulesService;
import com.crunii.micro.service.appsecond.service.AppYouthHotService;
import com.crunii.micro.service.appsecond.service.IAppContentManageService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static com.crunii.micro.service.appsecond.api.constants.AppSecondConstants.*;

/**
 * @author wangchen create 2023/12/8 15:51
 */
@RequiredArgsConstructor
@Service
public class AppYouthHotServiceImpl implements AppYouthHotService {

    private final AppYouthHotDao appYouthHotDao;
    private final AppContentDetailRepo appContentDetailRepo;
    private final AppWorkerSearchRepo baseRepo;
    private final ElsticService elsticService;
    private final AppMultiModulesService appMultiModulesService;
    private final AppContentManageRepo manageRepo;
    private final IAppContentManageService manageService;

    private final Map<Long, AtomicInteger> mediumReadMap = new HashMap<>(16);

    @Value("${upload.url}")
    private String urlPrefix;

    @Override
    public PageRsp<YouthContextVo> pageYouthHot(PageReq<HotEsPageDto> req, Worker worker) {
        Integer loginId = validatedIsTourist(worker);
        HotEsPageDto query = req.getQuery();
        Date date = new Date();
        PageRsp<YouthContextVo> rsp;
        // 带参数查询，走es接口
        if (StringUtils.hasText(query.getKeyWord()) || Objects.nonNull(query.getMediumId())) {
            rsp = getAndChangeEsData(req);
            if (Objects.nonNull(loginId) && StringUtils.hasText(query.getKeyWord())) {
                // 保存用户搜索记录
                AppWorkerSearch appWorkerSearch = baseRepo.findByWorkerIdAndSearchTypeAndSearchContextAndDeleted(loginId, 1, query.getKeyWord(), 0);
                if (Objects.isNull(appWorkerSearch)) {
                    appWorkerSearch = new AppWorkerSearch();
                }
                appWorkerSearch.setSearchType(1);
                appWorkerSearch.setWorkerId(loginId);
                appWorkerSearch.setSearchContext(query.getKeyWord());
                appWorkerSearch.setCreatorId(loginId);
                appWorkerSearch.setCreateTime(date);
                baseRepo.saveAndFlush(appWorkerSearch);
            }
        } else {
            String importantNewsLabel = getImportantNewsLabel();
            List<YouthContextVo> list = new ArrayList<>();
            // 标签为空
            if(req.getPageNo() == 1 && !StringUtils.hasText(req.getQuery().getLabelId())){
                if(StringUtils.hasText(importantNewsLabel)){
                    list = appYouthHotDao.findImportTop(importantNewsLabel);
                }
            }

            req.setPageSize(req.getPageSize()-list.size());
            rsp = appYouthHotDao.pageYouthHot(req,importantNewsLabel);
            if(!CollectionUtils.isEmpty(list)){
                rsp.setTotalNum(rsp.getTotalNum()+list.size());
                List<YouthContextVo> result = rsp.getResult();
                list.addAll(result);
                rsp.setResult(list);
            }
        }
        List<YouthContextVo> result = rsp.getResult();
        if (CollectionUtils.isEmpty(result)) {
            return rsp;
        }
        handleParams(result, loginId);
        return rsp;
    }

    private String getImportantNewsLabel() {
        return appYouthHotDao.getImportantNewsLabel()+"";
    }
    /**
     * es 转换
     */
    private PageRsp<YouthContextVo> getAndChangeEsData(PageReq<HotEsPageDto> req) {
        PageRsp<YouthHotEsVo> data = elsticService.pageSearchYouthHot(req).getData();
        PageRsp<YouthContextVo> rsp = new PageRsp<>();
        List<YouthContextVo> list = new ArrayList<>();
        data.getResult().forEach(x -> {
            YouthContextVo vo = new YouthContextVo();
            BeanUtils.copyProperties(x, vo);
            list.add(vo);
        });
        rsp.setTotalNum(data.getTotalNum());
        rsp.setPageSize(data.getPageSize());
        rsp.setPageNo(data.getPageNo());
        rsp.setResult(list);
        return rsp;
    }

    @Override
    public ContextAppDetailVo getYouthHotDetail(Long contentManageId, Worker worker) {
        ContextAppDetailVo contextAppDetailVo = appYouthHotDao.findContextManageDetail(contentManageId);
        if (Objects.isNull(contextAppDetailVo)) {
            throw new BusinessException("该文章已被删除");
        }
        List<String> urls = new ArrayList<>();
        String picKey = contextAppDetailVo.getPicKey();
        if (StringUtils.hasText(picKey)) {
            String[] split = picKey.split(",");
            List<String> list = Arrays.asList(split);
            contextAppDetailVo.setDivideNumber(appYouthHotDao.findMaxHeight(list));
            // 如是视频 url则无需展示
            if (!StringUtils.hasText(contextAppDetailVo.getFileId())) {
                for (String s : split) {
                    if (s.contains("http")) {
                        urls.add(s);
                    }
                    urls.add(urlPrefix + s);
                }
            }
        }
        contextAppDetailVo.setUrls(urls);
        contextAppDetailVo.setCollectCount(appYouthHotDao.findCollectCount(contentManageId));
        contextAppDetailVo.setLikeCount(appYouthHotDao.findLikeCount(contentManageId));
        // 新增浏览记录 防止游客浏览报错
        fillResult(worker, contentManageId, contextAppDetailVo);
        return contextAppDetailVo;
    }

    /**
     * 填充返回参数
     */
    private void fillResult(Worker worker, Long contentManageId, ContextAppDetailVo contextAppDetailVo) {
        if (Objects.nonNull(worker)) {
            int loginId = worker.getLoginId();
            // 查询是否关注点赞
            List<Long> contextList = new ArrayList<>();
            contextList.add(contentManageId);
            List<Long> mediumList = new ArrayList<>();
            mediumList.add(appYouthHotDao.findMediumId(contentManageId));
            // 点赞
            List<Long> id = appYouthHotDao.findWorkerLike(loginId, contextList);
            if (!CollectionUtils.isEmpty(id)) {
                contextAppDetailVo.setConcurrentLike(1);
            }
            // 关注
            List<Long> mediumIds = appYouthHotDao.findWorkerFollow(loginId, mediumList);
            if (!CollectionUtils.isEmpty(mediumIds)) {
                contextAppDetailVo.setInterest(1);
            }
            // 收藏
            Integer courrentCollect = appYouthHotDao.findIsCollect(contentManageId, loginId);
            if (Objects.nonNull(courrentCollect)) {
                contextAppDetailVo.setConcurrentCollect(1);
            }
        }
    }

    @Override
    public List<String> queryHotSearch(Integer searchType, Worker worker) {
        Integer integer = validatedIsTourist(worker);
        if (Objects.isNull(searchType)) {
            searchType = 1;
        }
        if (Objects.isNull(integer)) {
            return new ArrayList<>();
        }
        return appYouthHotDao.queryHotSearch(integer, searchType);
    }

    @Override
    public String delSearch(Worker worker) {
        Integer integer = validatedIsTourist(worker);
        if (Objects.nonNull(integer)) {
            appYouthHotDao.delSearch(integer);
        }
        return "删除成功";
    }

    @Override
    public void flushCash() {
        if (CollectionUtils.isEmpty(mediumReadMap)) {
            return;
        }
        Map<Long, AtomicInteger> copyMap;
        copyMap = new HashMap<>(mediumReadMap.size());
        copyMap.putAll(mediumReadMap);
        mediumReadMap.clear();
        Set<Long> set = copyMap.keySet();
        for (Long aLong : set) {
            Integer readCount = appYouthHotDao.findOne(aLong);
            if (Objects.nonNull(readCount)) {
                int i = readCount + copyMap.get(aLong).get();
                int max = Math.max(i, 0);
                appYouthHotDao.updateReadCount(aLong, max);
            }
        }

    }

    @Override
    public String createUserView(UserViewDto dto, Worker worker) {
        // 更新浏览数
        long contentManageId = Long.parseLong(dto.getContentManageId());
        AtomicInteger atomicInteger = mediumReadMap.get(contentManageId);
        if (Objects.isNull(atomicInteger)) {
            atomicInteger = new AtomicInteger(1);
            mediumReadMap.put(contentManageId, atomicInteger);
        }
        atomicInteger.incrementAndGet();
        // 如果不是游客登录
        if (Objects.nonNull(worker)) {
            appMultiModulesService.createView(worker, contentManageId);
        }
        return "创建成功";
    }

    /**
     * 处理分页后参数
     */
    @Override
    public void handleParams(List<YouthContextVo> result, Integer loginId) {
        List<Long> collect = result.stream().map(YouthContextVo::getMediumId).distinct().collect(Collectors.toList());
        List<Long> contextList = result.stream().map(YouthContextVo::getContentManageId).distinct().collect(Collectors.toList());
        Map<Long, Integer> likeMap = new HashMap<>(16);
        Map<Long, Integer> collectMap = new HashMap<>(16);
        if (Objects.nonNull(loginId)) {
            // 点赞
            List<Long> id = appYouthHotDao.findWorkerLike(loginId, contextList);
            id.forEach(x -> likeMap.put(x, 1));
            // 关注
            List<Long> mediumIds = appYouthHotDao.findWorkerFollow(loginId, collect);
            mediumIds.forEach(x -> collectMap.put(x, 1));
        }
        for (YouthContextVo youthHotEsVo : result) {
            AppContentDetail one = appContentDetailRepo.findByContentManageId(youthHotEsVo.getContentManageId());
            if (Objects.nonNull(one)) {
                // 点赞数
                youthHotEsVo.setLikeCount(appYouthHotDao.findLikeCount(one.getContentManageId()));
                // 是否点赞
                Integer concurrentLike = likeMap.get(one.getContentManageId());
                if (Objects.isNull(concurrentLike)) {
                    concurrentLike = 0;
                }
                youthHotEsVo.setConcurrentLike(concurrentLike);
                // 是否关注
                Integer interest = collectMap.get(youthHotEsVo.getMediumId());
                if (Objects.isNull(interest)) {
                    interest = 0;
                }
                youthHotEsVo.setInterest(interest);
                // url处理 查询封面图（取第一个）
                String[] split = one.getPicKey().split(",");
                if (split.length == 0) {
                    continue;
                }
                PicInfo picInfo;
                // 如果是视频，无需拼接url
                if (StringUtils.hasText(one.getFileId())) {
                    picInfo = new PicInfo();
                    String picUrl;
                    if (one.getPicKey().contains("http")) {
                        picUrl = one.getPicKey();
                    } else {
                        picUrl = urlPrefix + one.getPicKey();
                        picInfo = appYouthHotDao.findPicInfoByFileKey(one.getPicKey());
                    }
                    if (Objects.isNull(picInfo)) {
                        picInfo = new PicInfo();
                        picInfo.setWidth(WATER_FALL_DEFAULT_WIDTH);
                        picInfo.setHeight(WATER_FALL_DEFAULT_HEIGHT);
                    }
                    picInfo.setPicUrl(picUrl);
                } else {
                    picInfo = appYouthHotDao.findPicInfoByFileKey(split[0]);
                    if (Objects.nonNull(picInfo)) {
                        picInfo.setPicUrl(urlPrefix + picInfo.getPicUrl());
                    } else {
                        picInfo = new PicInfo();
                        picInfo.setPicUrl(urlPrefix + picInfo.getPicUrl());
                        picInfo.setWidth(WATER_FALL_DEFAULT_WIDTH);
                        picInfo.setHeight(WATER_FALL_DEFAULT_HEIGHT);
                    }
                }
                youthHotEsVo.setUrls(picInfo);
            }

        }
    }

    @Override
    public YouthHotPublicDetailVo queryYouthHot(Long contentManageId) {
        // 查询要闻标签
        String importantNewsLabel = getImportantNewsLabel();
        YouthHotPublicDetailVo youthHotPublicDetailVo = appYouthHotDao.queryYouthHot(contentManageId,importantNewsLabel);
        if(Objects.isNull(youthHotPublicDetailVo)){
            throw new BusinessException("该文章已被删除！");
        }
        if(!StringUtils.hasText(importantNewsLabel)){
            youthHotPublicDetailVo.setIsImportantNews(0);
        }
        return youthHotPublicDetailVo;
    }

    @Override
    public String updatePublicAudit(YouthPublicAuditDto dto) {
        // 解密
        Integer workerId = getWorkerId(dto.getId());
        AppContentManage one = manageRepo.getOne(dto.getContentManageId());
        if(Objects.isNull(one)){
            throw new BusinessException("该文章已被删除");
        }
        YouthAuditDto youthAuditDto = new YouthAuditDto();
        BeanUtils.copyProperties(dto,youthAuditDto);
        Worker worker = appYouthHotDao.findWorkerInfo(workerId);
        manageService.updateAuditState(youthAuditDto,worker);
        return "success";
    }




    @SuppressWarnings("all")
    private Integer getWorkerId(String id) {
        try {
             return Integer.parseInt(AESUtil.decrypt(id));
        } catch (Exception e) {
            throw new BusinessException("无法校验身份");
        }
    }

    /**
     * 验证是否为游客
     */
    private Integer validatedIsTourist(Worker worker) {
        if (Objects.nonNull(worker)) {
            return worker.getLoginId();
        }
        return null;
    }
}
