package com.crunii.micro.service.appsecond.service.impl;
import java.util.*;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.AppContentManage;
import com.crunii.micro.domain.entity.main.AppContentRecord;
import com.crunii.micro.domain.repo.main.AppContentManageRepo;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ContextAppSortDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ContextManageDto;
import com.crunii.micro.service.appsecond.api.bo.youthHotPoint.ContextSortTypeDto;
import com.crunii.micro.service.appsecond.api.dto.YouthHotLabelVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.ContextManageVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.HandleHistoryVo;
import com.crunii.micro.service.appsecond.api.vo.youthHotPoint.IdNumberVo;
import com.crunii.micro.service.appsecond.jdbc.AppContentSortDao;
import com.crunii.micro.service.appsecond.service.IAppContentSortService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import com.crunii.micro.domain.repo.main.AppContentRecordRepo;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.stream.Collectors;

import static com.crunii.micro.service.appsecond.api.constants.AppSecondConstants.MAX_HOT_SEARCH;


/**
 * 【请填写功能名称】Service业务层处理
 *
 * @author liuchuan
 * @date 2023-12-06
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class IAppContentSortServiceImpl implements IAppContentSortService {

    private final AppContentRecordRepo baseRepo;
    private final AppContentManageRepo appContentManageRepo;
    private final AppContentSortDao appContentSortDao;

    @Override
    public String updateAppSort(ContextAppSortDto dto, Worker worker) {
        AppContentManage one = getAppContentManage(dto.getContentManageId());
        Date now = new Date();
        int loginId = worker.getLoginId();
        if(one.getAppSort().equals(dto.getAppSort())){
            return "更新成功";
        }
        List<AppContentManage> list = new ArrayList<>();
        // 之前序值先存入记录
        AppContentRecord appContentRecord = new AppContentRecord();
        appContentRecord.setBeforeOrdinalValue(one.getAppSort());
        one.setAppSort(dto.getAppSort());
        one.setUpdateBy(loginId);
        one.setUpdateTime(now);
        list.add(one);

        // 查询这个app之前的序值
        AppContentManage appContentManage = appContentSortDao.findByAppSort(dto.getAppSort());
        if(Objects.nonNull(appContentManage)){
            appContentManage.setAppSort(appContentRecord.getBeforeOrdinalValue());
            list.add(appContentManage);
        }
        appContentManageRepo.saveAllAndFlush(list);

        appContentRecord.setContentManageId(one.getContentManageId());
        appContentRecord.setAfterOrdinalValue(dto.getAppSort());
        appContentRecord.setCreateBy(loginId);
        appContentRecord.setCreateTime(now);
        baseRepo.saveAndFlush(appContentRecord);
        return "更新成功";
    }


    @Override
    public String updateWhetherHot(ContextSortTypeDto dto, Worker worker) {
        AppContentManage appContentManage = getAppContentManage(dto.getContentManageId());
        if(Objects.isNull(dto.getHotSearch()) && Objects.isNull(dto.getHotList())){
            throw new BusinessException("热搜/热榜不能为空");
        }
        // 不是上架状态
        Integer passState = 1;
        if(!passState.equals(appContentManage.getState())){
            throw new BusinessException("该文章未上架！");
        }
        Integer state = 0;
        Date now = new Date();
        // 判断是搜索还是热榜
        Integer hotSearch = dto.getHotSearch();
        List<AppContentManage> list;
        // 热搜
        if(Objects.nonNull(hotSearch)){
            list = handleHotSearch(appContentManage,dto,state,now,worker.getLoginId());
        }else {
            list = handleHotList(appContentManage,dto,state,now,worker.getLoginId());
        }
        appContentManage.setUpdateTime(now);
        appContentManage.setUpdateBy(worker.getLoginId());
        list.add(appContentManage);
        appContentManageRepo.saveAllAndFlush(list);
        return "更新成功";
    }

    /**
     * 处理热榜
     */
    private List<AppContentManage> handleHotList(AppContentManage appContentManage, ContextSortTypeDto dto, Integer state, Date now, int loginId) {
        // 下架
        if(state.equals(dto.getHotList())){
            appContentManage.setHotList(0);
            appContentManage.setListOrdinalValue(null);
        }else {
            // 查询热搜/热榜 目前有多少个
            int count = appContentSortDao.findHotSearchCount(false,appContentManage.getContentManageId());
            if(MAX_HOT_SEARCH <= count){
                throw new BusinessException("热榜已达到最大"+MAX_HOT_SEARCH+"个");
            }
            appContentManage.setHotList(dto.getHotList());
            if(Objects.isNull(dto.getOrdinalValue())){
                throw new BusinessException("上热榜 需传入序值");
            }
            appContentManage.setListOrdinalValue(dto.getOrdinalValue());
           return adjustSort(appContentManage.getContentManageId(),dto.getOrdinalValue(),false,now,loginId);
        }
        return new ArrayList<>();
    }

    /**
     * 处理热搜
     */
    private List<AppContentManage> handleHotSearch(AppContentManage appContentManage, ContextSortTypeDto dto,Integer state,Date now,Integer loginId) {
        // 下架
        if(state.equals(dto.getHotSearch())){
            appContentManage.setHotSearch(0);
            appContentManage.setSearchOrdinalValue(null);
        }else {
            // 查询热搜/热榜 目前有多少个
            int count = appContentSortDao.findHotSearchCount(true,appContentManage.getContentManageId());
            if(MAX_HOT_SEARCH <= count){
                throw new BusinessException("热搜已达到最大"+MAX_HOT_SEARCH+"个");
            }
            appContentManage.setHotSearch(dto.getHotSearch());
            if(Objects.isNull(dto.getOrdinalValue())){
                throw new BusinessException("上热搜 需传入序值");
            }
            appContentManage.setSearchOrdinalValue(dto.getOrdinalValue());
           return adjustSort(appContentManage.getContentManageId(),dto.getOrdinalValue(),true,now,loginId);
        }
        return new ArrayList<>();
    }

    @Override
    public PageRsp<HandleHistoryVo> pageHandleHistory(PageReq<Object> req, Worker worker) {
        if(!StringUtils.hasText(worker.getOrgId())){
            return new PageRsp<>();
        }
        return appContentSortDao.pageHandleHistory(req);
    }

    @Override
    @InjectProperty
    public PageRsp<ContextManageVo> pageContextManage(PageReq<ContextManageDto> req, Worker worker) {
        PageRsp<ContextManageVo> contextManageVoPageRsp = appContentSortDao.pageContextManage(req);
        List<ContextManageVo> result = contextManageVoPageRsp.getResult();
        if(CollectionUtils.isEmpty(result)){
            return contextManageVoPageRsp;
        }
        Set<Long> labelIds = new HashSet<>();
        // 处理标签
        result.forEach(x-> {
            if(StringUtils.hasText(x.getLabelName())){
                Arrays.stream(x.getLabelName().split(",")).forEach(y->labelIds.add(Long.parseLong(y)));
            }
        });
        Map<Long,String> labelMap = new HashMap<>(16);
        if(!CollectionUtils.isEmpty(labelIds)){
            List<YouthHotLabelVo> list = appContentSortDao.findLabelList(labelIds);
            labelMap = list.stream().collect(Collectors.toMap(YouthHotLabelVo::getLabelId, YouthHotLabelVo::getLabelName));
        }
        for (ContextManageVo contextManageVo : result) {
            StringJoiner newLabelName = new StringJoiner(",");
            if(StringUtils.hasText(contextManageVo.getLabelName())){
                String[] split = contextManageVo.getLabelName().split(",");
                for (String s : split) {
                    String s1 = labelMap.get(Long.parseLong(s));
                    newLabelName.add(s1);
                }
            }
            contextManageVo.setLabelName(newLabelName.toString());
        }
        return contextManageVoPageRsp;
    }

    /**
     * 调整排序
     */
    private List<AppContentManage> adjustSort(Long contentManageId,Integer ordinalValue,boolean isHotSearch,Date now,Integer loginId) {
        List<AppContentManage> list = new ArrayList<>();
        // 查询所有id+序值进行处理
        List<IdNumberVo> idNumber = appContentSortDao.findAllOrdinalValueBySearch(isHotSearch,contentManageId);
        if(CollectionUtils.isEmpty(idNumber)){
            return list;
        }
        List<Integer> ordinalValues = idNumber.stream().map(IdNumberVo::getNumber).collect(Collectors.toList());
        if(!ordinalValues.contains(ordinalValue)){
            return list;
        }
        IdNumberVo idNumberVo = new IdNumberVo();
        idNumberVo.setNumber(ordinalValue);
        idNumber.add(0,idNumberVo);
        // 重排序
        List<IdNumberVo> newList = idNumber.stream().sorted(Comparator.comparingInt(IdNumberVo::getNumber)).collect(Collectors.toList());

        for (int i = 0; i < newList.size(); i++) {
            if(newList.get(i).getNumber() != i+1){
                newList.get(i).setNumber(i+1);
            }
        }
        for (IdNumberVo idNumberVo1 : newList) {
            if(Objects.isNull(idNumberVo1.getId())){
                continue;
            }
            AppContentManage one = appContentManageRepo.getOne(idNumberVo1.getId());
            if(isHotSearch){
                if(!idNumberVo1.getNumber().equals(one.getSearchOrdinalValue())){
                    one.setUpdateBy(loginId);
                    one.setUpdateTime(now);
                    one.setSearchOrdinalValue(idNumberVo1.getNumber());
                    list.add(one);
                }
            }else {
                if(!idNumberVo1.getNumber().equals(one.getListOrdinalValue())){
                    one.setUpdateBy(loginId);
                    one.setUpdateTime(now);
                    one.setListOrdinalValue(idNumberVo1.getNumber());
                    list.add(one);
                }
            }
        }
        return list;

    }

    /**
     * 获取主表数据
     */
    private AppContentManage getAppContentManage(Long contentManageId) {
        AppContentManage one = appContentManageRepo.getOne(contentManageId);
        if(Objects.isNull(one)){
            throw new BusinessException("该内容已被删除");
        }
        return one;
    }
}
