package com.crunii.micro.service.appsecond.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.message.PushMsgTemplateReqBo;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.QczdIndustryFieldRepo;
import com.crunii.micro.domain.repo.main.QczdSpaceContentRepo;
import com.crunii.micro.domain.repo.main.QczdSpaceTeamRepo;
import com.crunii.micro.domain.repo.main.QczdTeamStayRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.MessageCommonService;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.ApplyTeamAuditStateUDTO;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.SpaceApplyTeamReqDTO;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.StaticValueDTO;
import com.crunii.micro.service.appsecond.api.vo.youthspace.SpaceApplyTeamPageVO;
import com.crunii.micro.service.appsecond.enums.SecondPushMessageEnum;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.BusinessStateEnum;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.DemandEnum;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.SexEnum;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.SteamAuditStateEnum;
import com.crunii.micro.service.appsecond.enums.YouthSpaceRoleEnum;
import com.crunii.micro.service.appsecond.jdbc.SpaceApplyTeamDao;
import com.crunii.micro.service.appsecond.service.SpaceApplyTeamService;
import com.google.common.base.Joiner;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @Author thw
 * @date 2023/12/15
 * @description 青创团队管理
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class SpaceApplyTeamServiceImpl implements SpaceApplyTeamService {

  private final SpaceApplyTeamDao spaceApplyTeamDao;

  public final YouthSpaceServiceImpl youthSpaceService;

  private final SpacePassTeamServiceImpl spacePassTeamService;

  public final QczdIndustryFieldRepo qczdIndustryFieldRepo;

  public final QczdTeamStayRepo qczdTeamStayRepo;

  public final QczdSpaceTeamRepo qczdSpaceTeamRepo;

  private final QczdSpaceContentRepo qczdSpaceContentRepo;

  private final MessageCommonService messageCommonService;


  /** 青创空间-团队审核原因:其他 */
  private final String valueCode = "Others";

  @Override
  public PageRsp<SpaceApplyTeamPageVO> pageApplyTeam(PageReq<SpaceApplyTeamReqDTO> req) {

    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    Boolean flag = Boolean.FALSE;
    // 查看权限（青创空间管理员：对青年的申请信息进行查看并审核,只能查看加入自己空间的申请团队信息）
    Map<String, YouthSpaceRoleEnum> workerPermissonMap = youthSpaceService.getCurrentWorkerIncludeRole(
        currWorker);
    if (ObjectUtil.isEmpty(workerPermissonMap)){
      throw new BusinessException("没有对应空间的查看权限!");
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))){
      // 不做任何处理
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      flag = Boolean.TRUE;
    }
    PageRsp<SpaceApplyTeamPageVO> pageRsp =  spaceApplyTeamDao.pageApplyTeam(req,currWorker,flag);
    // 分页返回数据包装
    packagePageData(pageRsp.getResult());
    return pageRsp;
  }

  /** 分页返回数据包装 */
  private void packagePageData(List<SpaceApplyTeamPageVO> result) {
    if (ObjectUtil.isNotEmpty(result)){
      for (SpaceApplyTeamPageVO pageVO : result) {
        // 营业执照(0:没有、1:有)
        BusinessStateEnum businessStateEnum = BusinessStateEnum.getBusinessStateEnum(pageVO.getBusinessLicense());
        if (ObjectUtil.isNotEmpty(businessStateEnum)) {
          pageVO.setBusinessLicenseName(businessStateEnum.getDesc());
        }
        // 性别
        if (ObjectUtil.isNotEmpty(pageVO.getSex())) {
          SexEnum sexEnum = SexEnum.MAN.getSexEnum(Integer.valueOf(pageVO.getSex()));
          if (ObjectUtil.isNotEmpty(sexEnum)){
            pageVO.setSexName(sexEnum.getDesc());
          }
        }

        // 行业领域
        List<QczdIndustryField> industryFields = qczdIndustryFieldRepo.findByTeamId(
            Long.valueOf(pageVO.getId()));
        List<String> fieldList = industryFields.stream().map(QczdIndustryField::getIndustryField).distinct()
            .collect(Collectors.toList());
        List<StaticValueDTO> qckhIndustryField = youthSpaceService.staticPackData(fieldList,
            "qckhIndustryField");
        List<String> fieldStr = qckhIndustryField.stream().map(StaticValueDTO::getValueName).distinct()
            .collect(Collectors.toList());
        String fieldNameStr = Joiner.on(",").join(fieldStr);
        pageVO.setIndustryFieldName(fieldNameStr);
        pageVO.setIndustryFieldList(qckhIndustryField);

        //工位需求 与 办公室需求
        DemandEnum workerDemand = DemandEnum.getDemandEnum(pageVO.getWorkNeed());
        if (ObjectUtil.isNotEmpty(workerDemand)) {
          pageVO.setWorkNeedName(workerDemand.getDesc());
        }
        DemandEnum officeDemand = DemandEnum.getDemandEnum(pageVO.getOfficeNeed());
        if (ObjectUtil.isNotEmpty(officeDemand)) {
          pageVO.setOfficeNeedName(officeDemand.getDesc());
        }

        // 审核状态
        SteamAuditStateEnum auditStateEnum = SteamAuditStateEnum.getSteamAuditStateEnum(
            pageVO.getAuditState());
        if (ObjectUtil.isNotEmpty(auditStateEnum)) {
          pageVO.setAuditStateName(auditStateEnum.getDesc());
        }

        // 处理审核详情
        if (ObjectUtil.isNotEmpty(pageVO.getAuditReasonName()) && ObjectUtil.isNotEmpty(pageVO.getReasonDetail())){
          pageVO.setAuditReasonName(pageVO.getAuditReasonName() + "[" + pageVO.getReasonDetail() + "]");
        }

        // 时间处理
        // 统一处理时间
        pageVO.setApplyTime(youthSpaceService.LocalDateTimeSub(pageVO.getApplyTime(),youthSpaceService.timeSubNum));
        pageVO.setAuditTime(youthSpaceService.LocalDateTimeSub(pageVO.getAuditTime(),youthSpaceService.timeSubNum));

      }
    }

  }

  @Override
  public Result updateBatchAuditState(ApplyTeamAuditStateUDTO bo) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    if (ObjectUtil.isEmpty(bo)){
      throw new BusinessException("参数对象必传!");
    }

    if (ObjectUtil.isEmpty(bo.getId()) || ObjectUtil.isEmpty(bo.getAuditState())){
      throw new BusinessException("存在必传参数为空!");
    }

    if (ObjectUtil.isNotEmpty(bo.getAuditState()) && SteamAuditStateEnum.NO_PASS_AUDIT.getNum().equals(bo.getAuditState())){
      if (ObjectUtil.isEmpty(bo.getAuditReason())){
        throw new BusinessException("审核结果选择不通过时,原因为必须项!");
      }
    }

/*    // 青创空间-团队审核原因:其他
    TsmPropertyValue tsmPropertyValue = spacePassTeamService.findAuditOtherReasonDesc(
        valueCode);
    // 审核原因为**其他**时(最终看静态值中存的是什么)，文本框才能有数据且不允许超过200字符
    if (Objects.equals(tsmPropertyValue.getValueCode(),bo.getAuditReason())) {
      if (ObjectUtil.isEmpty(bo.getReasonDetail()) || bo.getReasonDetail().length() > 200) {
        throw new BusinessException("原因选择【其他】时,文本域中必填且限制输入200个字符!");
      }
    }*/

    List<Long> ids = bo.getId().stream().map(e -> Long.valueOf(e)).collect(Collectors.toList());
    List<QczdTeamStay> allById = qczdTeamStayRepo.findAllById(ids);


    if (ObjectUtil.isNotEmpty(allById)) {
      List<QczdSpaceTeam> qczdSpaceTeams = new ArrayList<>();
      // 查看用户的权限
      Map<String, YouthSpaceRoleEnum> workerPermissionMap = youthSpaceService.getCurrentWorkerIncludeRole(
          currWorker);
      // 青创空间管理员：发布本空间的展示信息。
      if (ObjectUtil.isEmpty(workerPermissionMap)) {
        throw new BusinessException("没有权限");
      } else if (ObjectUtil.isNotEmpty(workerPermissionMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))) {
        // 不做任何处理
      } else if (ObjectUtil.isNotEmpty(workerPermissionMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))) {

        qczdSpaceTeams = qczdSpaceTeamRepo.findAllByTeamIdIn(ids);
        if (ObjectUtil.isEmpty(qczdSpaceTeams)) {
          throw new BusinessException("通过团队id未找到关联的空间数据");
        }

        List<Long> spaceIds = qczdSpaceTeams.stream().map(QczdSpaceTeam::getSpaceId).distinct()
            .collect(Collectors.toList());
        List<QczdSpaceContent> qczdSpaceContents = qczdSpaceContentRepo.findAllById(spaceIds);
        if (ObjectUtil.isEmpty(qczdSpaceContents)) {
          throw new BusinessException("通过空间id未找到空间数据");
        }

        // 每一个需要审核的团队都需要满足（青创空间管理员是该此空间的创建人)
        StringBuilder stringBuilder = new StringBuilder("");
        for (QczdSpaceContent qczdSpaceContent : qczdSpaceContents) {
          for (QczdSpaceTeam qczdSpaceTeam : qczdSpaceTeams) {
            if (ObjectUtil.equals(qczdSpaceContent.getId(), qczdSpaceTeam.getSpaceId())) {
              if (ObjectUtil.notEqual(qczdSpaceContent.getCreateId(), currWorker.getLoginId())) {
                stringBuilder.append("团队id为:" + qczdSpaceTeam.getTeamId() + "所属的空间id为:"
                    + qczdSpaceTeam.getSpaceId() + "的管理员不是当前"
                    + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc());
                stringBuilder.append(",");
              }
            }
          }

        }
        if (ObjectUtil.isNotEmpty(stringBuilder.toString())) {
          throw new BusinessException(stringBuilder.substring(0, stringBuilder.length() - 1));
        }

      }

      // 保存需要发送消息的用户
      for (QczdTeamStay qczdTeamStay : allById) {
        // 审核记录
        qczdTeamStay.setAuditState(bo.getAuditState());
        if (SteamAuditStateEnum.PASS_AUDIT.getNum().equals(bo.getAuditState())){
          // 设置入住时间和团队状态
          qczdTeamStay.setTeamState(0);
          qczdTeamStay.setSettledTime(LocalDateTime.now());
        }
        if (ObjectUtil.isNotEmpty(bo.getAuditReason())) {
          qczdTeamStay.setAuditReason(bo.getAuditReason());
        }
        if (ObjectUtil.isNotEmpty(bo.getReasonDetail())) {
          qczdTeamStay.setReasonDetail(bo.getReasonDetail());
        }
        // 申请人年龄若小于18岁，系统直接判定审核结果为不通过，审核原因为“年龄小于18岁，不符合申请条件
        if (qczdTeamStay.getAge() < 18) {
          qczdTeamStay.setAuditState(SteamAuditStateEnum.NO_PASS_AUDIT.getNum());
          // 查询青创空间-团队审核原因:其他
          TsmPropertyValue auditOtherReasonDesc = spacePassTeamService.findAuditOtherReasonDesc(
              valueCode);
          qczdTeamStay.setAuditReason(auditOtherReasonDesc.getValueCode());
          qczdTeamStay.setReasonDetail(spacePassTeamService.auditReason);
        }

        qczdTeamStay.setAuditTime(LocalDateTime.now());
        qczdTeamStay.setAuditWorkerId(currWorker.getLoginId());
        qczdTeamStay.setAuditWorkerName(currWorker.getWorkerName());

        // 审核状态发生改变则向发起申请得团队创建人发送消息
        PushMsgTemplateReqBo pushMsgTemplateReqBo = new PushMsgTemplateReqBo();
        pushMsgTemplateReqBo.setSystemName(SecondPushMessageEnum.TEAM_STAY_CHECK.getSystemName());
        pushMsgTemplateReqBo.setSceneName(SecondPushMessageEnum.TEAM_STAY_CHECK.getSceneName());
        pushMsgTemplateReqBo.setTitle(SecondPushMessageEnum.TEAM_STAY_CHECK.getTitle());
        Map<String, Object> map = new LinkedHashMap<>();
        // 获取空间名称
        qczdSpaceTeams = qczdSpaceTeamRepo.findAllByTeamIdIn(ids);
        for (int i = 0; i < qczdSpaceTeams.size(); i++) {
          if (ObjectUtil.equals(qczdSpaceTeams.get(i).getTeamId(), qczdTeamStay.getId())){
            QczdSpaceContent one = qczdSpaceContentRepo.getOne(qczdSpaceTeams.get(i).getSpaceId());
            if (ObjectUtil.isNotEmpty(one)){
              map.put("space", one.getSpaceName());
            }else {
              throw new BusinessException("未通过空间id:" + qczdSpaceTeams.get(i).getSpaceId() + "找到空间信息!");
            }
          }
        }
        SteamAuditStateEnum auditStateEnum = SteamAuditStateEnum.getSteamAuditStateEnum(
            qczdTeamStay.getAuditState());
        if (ObjectUtil.isNotEmpty(auditStateEnum)){
          map.put("state", auditStateEnum.getDesc());
        }else {
          throw new BusinessException("获取审核状态失败!");
        }
        pushMsgTemplateReqBo.setMap(map);
        messageCommonService.pushMsg(pushMsgTemplateReqBo, Long.valueOf(qczdTeamStay.getCreateId()));
      }
      qczdTeamStayRepo.saveAllAndFlush(allById);

    }else {
      return Result.ofFail("根据主键未找到相关信息");
    }
    return Result.ofSuccessMsg("审核成功!");
  }

  @Override
  public void exportApplyTeam(HttpServletResponse response, SpaceApplyTeamReqDTO bo) {
    PageReq<SpaceApplyTeamReqDTO> req = new PageReq<>();
    req.setQuery(bo);
    req.setPageSize(10000000);

    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    Boolean flag = Boolean.FALSE;
    // 查看权限（青创空间管理员：对青年的申请信息进行查看并审核,只能查看加入自己空间的申请团队信息）
    Map<String, YouthSpaceRoleEnum> workerPermissonMap = youthSpaceService.getCurrentWorkerIncludeRole(
        currWorker);
    if (ObjectUtil.isEmpty(workerPermissonMap)){
      throw new BusinessException("没有对应空间的查看权限!");
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))){
      // 不做任何处理
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      flag = Boolean.TRUE;
    }
    PageRsp<SpaceApplyTeamPageVO> pageRsp =  spaceApplyTeamDao.pageApplyTeam(req,currWorker,flag);
    // 分页返回数据包装
    packagePageData(pageRsp.getResult());
    try {
      ExcelUtil.writeSingleExcel("团队信息导出", "团队信息",
          pageRsp.getResult(), SpaceApplyTeamPageVO.class);
    }catch (Exception e){
      throw new BusinessException("团队信息导出失败!");
    }

  }
}
