package com.crunii.micro.service.appsecond.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.PhoneUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.BeanUtil;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.appsecond.api.dto.data.ExcelDataInfo;
import com.crunii.micro.service.appsecond.api.dto.data.ExcelMistakeData;
import com.crunii.micro.service.appsecond.api.dto.data.MistakeDataInfo;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.*;
import com.crunii.micro.service.appsecond.api.vo.youthspace.SpaceContentOneVO;
import com.crunii.micro.service.appsecond.api.vo.youthspace.SpacePassTeamPageVO;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.*;
import com.crunii.micro.service.appsecond.enums.YouthSpaceRoleEnum;
import com.crunii.micro.service.appsecond.jdbc.SpacePassTeamDao;
import com.crunii.micro.service.appsecond.readListener.ImportTeamReadListener;
import com.crunii.micro.service.appsecond.readListener.ImportTeamReadListenerTwo;
import com.crunii.micro.service.appsecond.service.SpacePassTeamService;
import com.google.common.base.Joiner;
import java.io.IOException;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.SecureRandom;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

/**
 * @Author thw
 * @date 2023/12/13
 * @description 入住空间的团队管理
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class SpacePassTeamServiceImpl implements SpacePassTeamService {

  private final SpacePassTeamDao spacePassTeamDao;

  public final QczdTeamStayRepo qczdTeamStayRepo;

  public final QczdTeamStayTempRepo qczdTeamStayTempRepo;

  public final QczdSpaceContentRepo qczdSpaceContentRepo;

  private final QczdSpaceTeamRepo qczdSpaceTeamRepo;

  public final QczdIndustryFieldRepo qczdIndustryFieldRepo;

  public final YouthSpaceServiceImpl youthSpaceService;

  private final TsmPropertyValueRepo tsmPropertyValueRepo;

  private final String TeamCodePrefix = "QCKJ";
  private final int TeamCodeRandomNum = 4;

  /**
   * 青创空间-团队审核原因:其他
   */
  public final String valueCode = "Others";

  /**
   * 民族、学历、政治面貌、行业领域
   */
  public final String[] propCode = {"leagueNationality", "sqkjFormalSchooling",
      "qckjPoliticalStatus", "qckhIndustryField"};
  public final String auditReason = "年龄小于18岁，不符合申请条件。";
  public final String auditReasonTwo = "年龄小于18岁，不符合入驻团队条件。";


  @Override
  public Result oneSpaceInfo(Long spaceId) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    Optional<QczdSpaceContent>  contentOptional = qczdSpaceContentRepo.findById(spaceId);
    if (!contentOptional.isPresent()) {
      throw new BusinessException("根据空间id:" + spaceId + "未找到青创空间信息");
    }
    // 查看用户的权限
    Map<String, YouthSpaceRoleEnum> workerPermissionMap = youthSpaceService.getCurrentWorkerIncludeRole(
        currWorker);
    if (ObjectUtil.isEmpty(workerPermissionMap)){
      throw new BusinessException("当前用户没有查看此空间信息的权限");
    }else if (ObjectUtil.isNotEmpty(workerPermissionMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))){
      // 不做任何限制（2、青年创新创业服务中心管理员可查看到所有青创空间的数据）
    }else if (ObjectUtil.isNotEmpty(workerPermissionMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){

       QczdSpaceContent spaceContent = contentOptional.get();
     if (ObjectUtil.notEqual(currWorker.getLoginId(),spaceContent.getCreateId())){
        throw new BusinessException(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "只能查看自己创建的空间信息");
      }
    }

    if (ObjectUtil.isNotEmpty(contentOptional)){
      QczdSpaceContent spaceContent = contentOptional.get();
      SpaceContentOneVO spaceContentOneVO = new SpaceContentOneVO();
      BeanUtils.copyProperties(spaceContent, spaceContentOneVO);
      if (ObjectUtil.isNotEmpty(spaceContent.getTotalArea())){
        spaceContentOneVO.setTotalArea(spaceContent.getTotalArea().toString());
      }
      if (ObjectUtil.isNotEmpty(spaceContent.getApplyArea())){
        spaceContentOneVO.setApplyArea(spaceContent.getApplyArea().toString());
      }
      // 处理时间
      String localDateTime = DateUtils.formatLocalDateTime(spaceContent.getCreatedTime());
      spaceContentOneVO.setCreatedTime(localDateTime);

      // 已入住的团队必须满足审核通过
      List<QczdSpaceTeam> spaceTeams = spacePassTeamDao.findBySpaceIdAndAuditState(spaceId);

      spaceContentOneVO.setPassTeamNum(spaceTeams.size());
      return Result.ofSuccessWithMsg(spaceContentOneVO, "查询空间信息成功!");
    }else {
      return Result.ofFail( "查询空间信息失败!");
    }

  }

  @Override
  public PageRsp<SpacePassTeamPageVO> pageTeamInfo(PageReq<SpacePassTeamPageReqDTO> bo) {
    if (ObjectUtil.isEmpty(bo) || ObjectUtil.isEmpty(bo.getQuery())) {
      throw new BusinessException("参数对象必传!");
    }
    if (ObjectUtil.isEmpty(bo.getQuery().getSpaceId())) {
      throw new BusinessException("空间id必传!");
    }
    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    // 查看用户的权限
    Map<String, YouthSpaceRoleEnum> workerPermissonMap = youthSpaceService.getCurrentWorkerIncludeRole(
        currWorker);
    if (ObjectUtil.isEmpty(workerPermissonMap)){
      throw new BusinessException("当前用户没有查看此空间下已入驻团队信息的权限");
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))){
      // 不做任何限制（2、青年创新创业服务中心管理员可查看到所有青创空间的数据）
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      Optional<QczdSpaceContent> contentOptional = qczdSpaceContentRepo.findById(
          Long.valueOf(bo.getQuery().getSpaceId()));
      if (contentOptional.isPresent()){
        QczdSpaceContent spaceContent = contentOptional.get();
        if (ObjectUtil.notEqual(currWorker.getLoginId(),spaceContent.getCreateId())){
          throw new BusinessException(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "只能查看自己创建空间的已入驻团队信息");
        }
      }else {
        throw new BusinessException("通过空间id:" + bo.getQuery().getSpaceId() + "未找到空间已入驻团队信息");
      }
    }


    PageRsp<SpacePassTeamPageVO> pageRsp = spacePassTeamDao.pageTeamInfo(bo);

    // 返回值包装
    packageContent(pageRsp.getResult());
    return pageRsp;
  }

  /**
   * 分页参数包装
   */
  public void packageContent(List<SpacePassTeamPageVO> result) {
    if (ObjectUtil.isNotEmpty(result)) {
      for (SpacePassTeamPageVO pageVO : result) {
        // 营业执照(0:没有、1:有)
        BusinessStateEnum businessStateEnum = BusinessStateEnum.getBusinessStateEnum(
            pageVO.getBusinessLicense());
        if (ObjectUtil.isNotEmpty(businessStateEnum)) {
          pageVO.setBusinessLicenseName(businessStateEnum.getDesc());
        }

        // 性别
        if (ObjectUtil.isNotEmpty(pageVO.getSex())) {
          SexEnum sexEnum = SexEnum.MAN.getSexEnum(Integer.valueOf(pageVO.getSex()));
          if (ObjectUtil.isNotEmpty(sexEnum)) {
            pageVO.setSexName(sexEnum.getDesc());
          }
        }

        // 行业领域
        List<QczdIndustryField> industryFields = qczdIndustryFieldRepo.findByTeamId(
            Long.valueOf(pageVO.getId()));
        List<String> industryFieldList = industryFields.stream()
            .map(QczdIndustryField::getIndustryField).distinct()
            .collect(Collectors.toList());
        List<StaticValueDTO> qckhIndustryFields = youthSpaceService.staticPackData(
            industryFieldList,
            "qckhIndustryField");
        pageVO.setIndustryFieldList(qckhIndustryFields);
        List<String> str = qckhIndustryFields.stream().map(StaticValueDTO::getValueName).distinct()
            .collect(Collectors.toList());
        String fieldNameStr = Joiner.on(",").join(str);
        pageVO.setIndustryFieldName(fieldNameStr);
        //工位需求 与 办公室需求
        DemandEnum workerDemand = DemandEnum.getDemandEnum(pageVO.getWorkNeed());
        if (ObjectUtil.isNotEmpty(workerDemand)) {
          pageVO.setWorkNeedName(workerDemand.getDesc());
        }
        DemandEnum officeDemand = DemandEnum.getDemandEnum(pageVO.getOfficeNeed());
        if (ObjectUtil.isNotEmpty(officeDemand)) {
          pageVO.setOfficeNeedName(officeDemand.getDesc());
        }

        // 团队状态
        TeamStateEnum teamStateEnum = TeamStateEnum.getTeamStateEnum(pageVO.getTeamState());
        if (ObjectUtil.isNotEmpty(teamStateEnum)) {
          pageVO.setTeamStateName(teamStateEnum.getDesc());
        }

        // 审核状态
        SteamAuditStateEnum steamAuditStateEnum = SteamAuditStateEnum.getSteamAuditStateEnum(
            pageVO.getAuditState());
        if (ObjectUtil.isNotEmpty(steamAuditStateEnum)){
          pageVO.setAuditStateName(steamAuditStateEnum.getDesc());
        }

        // 处理审核详情
        if (ObjectUtil.isNotEmpty(pageVO.getAuditReasonName()) && ObjectUtil.isNotEmpty(pageVO.getReasonDetail())){
          pageVO.setAuditReasonName(pageVO.getAuditReasonName() + "[" + pageVO.getReasonDetail() + "]");
        }

        // 统一处理时间
        pageVO.setApplyTime(youthSpaceService.LocalDateTimeSub(pageVO.getApplyTime(),
            youthSpaceService.timeSubNum));
        pageVO.setSettledTime(youthSpaceService.LocalDateTimeSub(pageVO.getSettledTime(),
            youthSpaceService.timeSubNum));
        pageVO.setAuditTime(youthSpaceService.LocalDateTimeSub(pageVO.getAuditTime(),
            youthSpaceService.timeSubNum));
      }

    }
  }


  @Override
  public Result saveTeam(SpacePassTeamCUReqDTO bo) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    // 查看用户的权限
    doSaveTeamPermissionCheck(currWorker, Long.valueOf(bo.getSpaceId()),
        "通过空间id:" + bo.getSpaceId() + "未找到具体的空间信息");
    // 参数校验
    checkParams(bo);
    // 校验数据是否重复（表格和数据库数据重复）
    List<QczdTeamStay> existTeams = qczdTeamStayRepo.findByApplicantNameAndPhone(
        bo.getApplicantName(), bo.getPhone());
    if (ObjectUtil.isNotEmpty(existTeams)) {
      throw new BusinessException("表格和数据库数据重复数据参数校验失败: {" + "申请人姓名与联系人电话:[" + bo.getApplicantName() + "-" + bo.getPhone()
          + "]在数据库中已存在!");
    }

    QczdTeamStay qczdTeamStay = new QczdTeamStay();
    BeanUtils.copyProperties(bo, qczdTeamStay);
    createTeamNeedSetMethod(bo.getIdCode(), qczdTeamStay, currWorker,null,Boolean.TRUE);
    qczdTeamStayRepo.save(qczdTeamStay);
    // 创建与空间的关联
    createSpaceRelationTeam(Long.valueOf(bo.getSpaceId()), qczdTeamStay.getId(), currWorker,
        Boolean.TRUE);

    // 行业领域
    List<QczdIndustryField> industryFields = new ArrayList<>();
    // 创建团队和行业领域的关联关系
    createIndustryFieldContent(industryFields, bo.getIndustryField(), qczdTeamStay, Boolean.TRUE);
    qczdIndustryFieldRepo.saveAll(industryFields);
    qczdTeamStayRepo.save(qczdTeamStay);

    return Result.ofSuccessMsg("新增团队成功!");
  }

  /***
   * @Description: 创建团队与空间的关联
   * @Author: thw
   * @Date: 2023/12/18 12:47
   * @Params:
   * @param spaceId     空间id
   * @param teamId      团队id
   * @param currWorker  当前用户
   * @param flag        新增：true、修改：false
   * @return: void
   **/
  private void createSpaceRelationTeam(Long spaceId, Long teamId, Worker currWorker, Boolean flag) {
    if (!flag) {
      qczdSpaceTeamRepo.updateDelFlagBySpaceId(currWorker.getLoginId(), currWorker.getWorkerName(),
          spaceId);
    }
    QczdSpaceContent spaceContent = qczdSpaceContentRepo.findById(Long.valueOf(spaceId))
        .orElseThrow(() -> new BusinessException("通过空间id未找到青创空间相关信息"));
    QczdSpaceTeam qczdSpaceTeam = new QczdSpaceTeam();
    qczdSpaceTeam.setSpaceId(spaceId);
    qczdSpaceTeam.setTeamId(teamId);
    qczdSpaceTeam.setDelFlag(0);
    qczdSpaceTeam.setUpdateId(currWorker.getLoginId());
    qczdSpaceTeam.setUpdateBy(currWorker.getWorkerName());
    qczdSpaceTeam.setUpdateTime(LocalDateTime.now());
    qczdSpaceTeamRepo.save(qczdSpaceTeam);
  }

  /**
   * 新建团队需要的公共设置
   */
  public void createTeamNeedSetMethod(String idCode, QczdTeamStay qczdTeamStay, Worker currWorker,Map<String, ExcelMistakeData> rowMap,Boolean flag) {
    // 手动设置主键
    Snowflake snowflake = new Snowflake();
    qczdTeamStay.setId(snowflake.nextId());
    qczdTeamStay.setCreateId(currWorker.getLoginId());
    qczdTeamStay.setCreatedBy(currWorker.getWorkerName());
    qczdTeamStay.setDelFlag(0);
    qczdTeamStay.setCreatePoint(0);
    qczdTeamStay.setCreatedTime(LocalDateTime.now());
    // 设置默认审核状态和团队入住状态
    qczdTeamStay.setTeamState(TeamStateEnum.CHECK_IN.getNum());
//    qczdTeamStay.setAuditState(SteamAuditStateEnum.STAY_AUDIT.getNum());
    qczdTeamStay.setAuditState(SteamAuditStateEnum.PASS_AUDIT.getNum());

    // 1、这种在后台新增的团队，申请时间=入驻时间=新增提交时间 2023-12-15需求确认的
    // 2、孵化结束是团队入驻到期，点击孵化结束按钮，更改入驻状态为【已结束孵化】
    qczdTeamStay.setApplyTime(qczdTeamStay.getCreatedTime());
    qczdTeamStay.setSettledTime(qczdTeamStay.getCreatedTime());
    // 生成团队编号
    // 获取已经存在的团队编码
    List<String> unionTeamCode = getUnionTeamCode();
    qczdTeamStay.setTeamCode(getTeamCode(unionTeamCode, TeamCodePrefix, TeamCodeRandomNum));
    // 通过身份证号计算年龄
    int ageNum = 0;
    if (ObjectUtil.isNotEmpty(idCode)){
      try{
        ageNum = IdcardUtil.getAgeByIdCard(idCode, new Date());
      }catch (Exception e){
        if (flag){
          throw new BusinessException("身份证号必填且格式必须正确");
        }else {
          doWithMistakeErrMessage(rowMap, "idCode", idCode, "身份证号必填且格式必须正确!");
        }
      }
      qczdTeamStay.setAge(ageNum);
    }
    // 新增/导入的团队的数据直接再此页面列表展示，不走啥子空间申请列表去审核 2023-12-23
    // 申请人年龄若小于18岁，系统直接判定审核结果为不通过，审核原因为“年龄小于18岁，不符合申请条件
    if (ageNum < 18) {
      qczdTeamStay.setAuditState(SteamAuditStateEnum.NO_PASS_AUDIT.getNum());
      // 查询青创空间-团队审核原因:其他
      TsmPropertyValue auditOtherReasonDesc = findAuditOtherReasonDesc(valueCode);
      qczdTeamStay.setAuditReason(auditOtherReasonDesc.getValueCode());
      qczdTeamStay.setReasonDetail(auditReasonTwo);
      if (flag){
        throw new BusinessException(auditReasonTwo);
      }else {
        doWithMistakeErrMessage(rowMap, "idCode", idCode, "根据身份证号得出的年龄小于18岁!");
      }
    }
    qczdTeamStay.setAuditTime(LocalDateTime.now());
    qczdTeamStay.setAuditWorkerId(currWorker.getLoginId());
    qczdTeamStay.setAuditWorkerName(currWorker.getWorkerName());
  }

  /**
   * 青创空间-团队审核原因:其他
   */
  public TsmPropertyValue findAuditOtherReasonDesc(String valueCode) {
    HashMap<String, Object> props = new HashMap<>();
    props.put("state", 1);
    props.put("valueCode", valueCode);
    List<TsmPropertyValue> tsmPropertyValues = tsmPropertyValueRepo.findByProps(props);
    if (ObjectUtil.isEmpty(tsmPropertyValues) || tsmPropertyValues.size() > 1) {
      throw new BusinessException("静态值中数据有误!");
    }
    return tsmPropertyValues.get(0);
  }

  @Override
  public Result updateTeam(SpacePassTeamCUReqDTO bo) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    // 查看用户的权限
    doSaveTeamPermissionCheck(currWorker, Long.valueOf(bo.getSpaceId()),
        "通过空间id:" + bo.getSpaceId() + "未找到具体的空间信息");
    // 参数校验
    checkParams(bo);
    Optional<QczdTeamStay> teamStayOptional = qczdTeamStayRepo.findById(Long.valueOf(bo.getId()));
    if (!teamStayOptional.isPresent()) {
      throw new BusinessException("通过团队id:" + bo.getId() + "未找到团队信息。");
    }
    // 校验数据是否重复（表格和数据库数据重复）
    List<Long> ids = new ArrayList<>();
    ids.add(Long.valueOf(bo.getId()));
    List<QczdTeamStay> existTeams = qczdTeamStayRepo.findByApplicantNameAndPhoneAndIdNotIn(
        bo.getApplicantName(), bo.getPhone(),ids);
    if (ObjectUtil.isNotEmpty(existTeams)) {
      throw new BusinessException("表格和数据库数据重复","","数据参数校验失败: {" + "申请人姓名与联系人电话:[" + bo.getApplicantName() + "-" + bo.getPhone()
          + "]在数据库中已存在!");
    }

    QczdTeamStay teamStay = teamStayOptional.get();
    BeanUtils.copyProperties(bo, teamStay);
    teamStay.setUpdateId(currWorker.getLoginId());
    teamStay.setUpdatedBy(currWorker.getWorkerName());
    teamStay.setUpdatedTime(LocalDateTime.now());
    teamStay.setDelFlag(0);
    // 设置默认审核状态和团队入住状态
    teamStay.setTeamState(TeamStateEnum.CHECK_IN.getNum());
    teamStay.setAuditState(SteamAuditStateEnum.STAY_AUDIT.getNum());
    // 通过身份证号计算年龄
    int ageNum = 0;
    if (ObjectUtil.isNotEmpty(bo.getIdCode())){
      try{
        ageNum = IdcardUtil.getAgeByIdCard(bo.getIdCode(), new Date());
      }catch (Exception e){
        throw new BusinessException("身份证号必填且格式必须正确");
      }
      teamStay.setAge(ageNum);
    }
    // 申请人年龄若小于18岁，系统直接判定审核结果为不通过，审核原因为“年龄小于18岁，不符合申请条件
    if (ageNum < 18) {
      teamStay.setAuditState(SteamAuditStateEnum.NO_PASS_AUDIT.getNum());
      // 查询青创空间-团队审核原因:其他
      TsmPropertyValue auditOtherReasonDesc = findAuditOtherReasonDesc(valueCode);
      teamStay.setAuditReason(auditOtherReasonDesc.getValueCode());
      teamStay.setReasonDetail(auditReasonTwo);
      teamStay.setAuditTime(LocalDateTime.now());
      teamStay.setAuditWorkerId(currWorker.getLoginId());
      teamStay.setAuditWorkerName(currWorker.getWorkerName());
    }
    qczdTeamStayRepo.save(teamStay);
    // 创建与空间的关联
    createSpaceRelationTeam(Long.valueOf(bo.getSpaceId()), teamStay.getId(), currWorker,
        Boolean.FALSE);
    // 行业领域
    List<QczdIndustryField> industryFields = new ArrayList<>();
    createIndustryFieldContent(industryFields, bo.getIndustryField(), teamStay, Boolean.FALSE);
    qczdIndustryFieldRepo.saveAll(industryFields);
    qczdTeamStayRepo.save(teamStay);
    return Result.ofSuccessMsg("修改团队信息成功!");
  }

  /***
   * @Description: 生成团队编号
   * @Author: thw
   * @Date: 2023/12/14 10:09
   * @Params:
   * @param codePrefix
   * @return: java.lang.String
   **/
  public String getTeamCode(List<String> unionTeamCode, String codePrefix, int num) {
    String finalStr = "";
    do {
      String second = DateUtils.formatLocalDateTime(LocalDateTime.now(),
          DateUtils.DEFAULT_DATE_TIME_FORMAT2);
      finalStr = codePrefix + second + generateRandomCode(num);
    } while (unionTeamCode.contains(finalStr));
    unionTeamCode.add(finalStr);
    return finalStr;
  }

  /**
   * 获取存在的团队编码
   */
  private List<String> getUnionTeamCode() {
    // 查询团队的所有code
    List<QczdTeamStay> teamStays = qczdTeamStayRepo.findAll();
    if (ObjectUtil.isNotEmpty(teamStays)) {
      return teamStays.stream().map(QczdTeamStay::getTeamCode)
          .collect(Collectors.toList());
    }
    return Lists.newArrayList();
  }


  /**
   * 随机生成num位编码(数字+英文字母)
   */
  private static String generateRandomCode(int num) {
    String CHARACTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
    StringBuilder randomCode = new StringBuilder();
    Random random = new SecureRandom();
    for (int i = 0; i < num; i++) {
      int randomIndex = random.nextInt(CHARACTERS.length());
      char randomChar = CHARACTERS.charAt(randomIndex);
      randomCode.append(randomChar);
    }
    return randomCode.toString();
  }

  /***
   * @Description: 创建团队和行业领域的关联关系
   * @Author: thw
   * @Date: 2023/12/14 9:52
   * @Params:
   * @param industryFields 存放需要生成的行业领域对象集合
   * @param fields 行业领域集合
   * @param qczdTeamStay  团队实例
   * @param flag  true:新增；flag:修改
   * @return: void
   **/
  public void createIndustryFieldContent(List<QczdIndustryField> industryFields,
      List<String> fields, QczdTeamStay qczdTeamStay, Boolean flag) {
    if (ObjectUtil.isNotEmpty(fields)) {
      if (!flag) {
        qczdIndustryFieldRepo.deleteByTeamId(qczdTeamStay.getId());
      }
      // 设置行业领域字符串
      String fieldStr = fields.stream().distinct().collect(Collectors.joining(","));
      qczdTeamStay.setIndustryField(fieldStr);
      QczdIndustryField industryField;
      for (String fieldName : fields) {
        industryField = new QczdIndustryField();
        industryField.setTeamId(qczdTeamStay.getId());
        industryField.setIndustryField(fieldName);
        industryField.setDelFlag(0);
        // 将对象添加到集合
        industryFields.add(industryField);
      }
    }
  }

  /**
   * 新增、修改 参数校验
   */
  public void checkParams(SpacePassTeamCUReqDTO bo) {
    if (ObjectUtil.isEmpty(bo)) {
      throw new BusinessException("参数对象必传!");
    }
    if (ObjectUtil.isEmpty(bo.getSpaceId())) {
      throw new BusinessException("所属空间id必须.");
    }
    if (ObjectUtil.isEmpty(bo.getBusinessLicense())) {
      throw new BusinessException("有营业执照必填.");
    }
    if (ObjectUtil.isNotEmpty(bo.getBusinessLicense()) && BusinessStateEnum.YES_BUSINESS.getNum()
        .equals(bo.getBusinessLicense())) {
      if (ObjectUtil.isEmpty(bo.getCompanyName()) || bo.getCompanyName().length() > 50) {
        throw new BusinessException("有营业执照时,企业名称必填且字符长度不能超过50字符.");
      }
    }
    if (ObjectUtil.isNotEmpty(bo.getBusinessLicense()) && BusinessStateEnum.NO_BUSINESS.getNum()
        .equals(bo.getBusinessLicense())) {
      if (ObjectUtil.isEmpty(bo.getWorkNeed())) {
        throw new BusinessException("工位需求域必填");
      }
      if (ObjectUtil.isEmpty(bo.getOfficeNeed())) {
        throw new BusinessException("办公室需求域必填");
      }
    }
    if (ObjectUtil.isEmpty(SexEnum.getSexEnum(Integer.valueOf(bo.getSex())))) {
      throw new BusinessException("性别必填且格式为[0|1]");
    }

    if (ObjectUtil.isEmpty(bo.getNation())) {
      throw new BusinessException("民族必填");
    }

    if (ObjectUtil.isEmpty(bo.getFormalSchooling())) {
      throw new BusinessException("学历必填");
    }

    if (ObjectUtil.isEmpty(bo.getPoliticalStatus())) {
      throw new BusinessException("政治面貌必填");
    }

    if (ObjectUtil.isEmpty(bo.getIndustryField())) {
      throw new BusinessException("行业领域必填");
    }

    List<ExcelMistakeData> mistakeData = new ArrayList<>();
    // 共同参数检测
    checkParamsData(bo, mistakeData, null);
    if (ObjectUtil.isNotEmpty(mistakeData)) {
      throw new BusinessException(mistakeData.get(0).getErrText());
    }




  }


  /**
   * 共同参数校验
   */
  public static List<ExcelMistakeData> checkParamsData(SpacePassTeamCUReqDTO bo,
      List<ExcelMistakeData> mistakeData, Map<String, ExcelMistakeData> rowMap) {
    if (ObjectUtil.isEmpty(bo.getApplicantName()) || bo.getApplicantName().length() > 10) {
      mistakeData.add(
          createMistakeCellContent("姓名", bo.getApplicantName(), "姓名必填且不能超过10字符"));
      if (ObjectUtil.isNotEmpty(rowMap)) {
        doWithMistakeErrMessage(rowMap, "applicantName", bo.getApplicantName(),
            "姓名必填且不能超过10字符!");
      }
    }
    if (ObjectUtil.isEmpty(bo.getIdCode()) || !isIdCode(bo.getIdCode())) {
      mistakeData.add(
          createMistakeCellContent("身份证号", bo.getIdCode(), "身份证号必填且格式必须正确"));
      if (ObjectUtil.isNotEmpty(rowMap)) {
        doWithMistakeErrMessage(rowMap, "idCode", bo.getIdCode(), "身份证号必填且格式必须正确!");
      }
    }
    if (ObjectUtil.isNotEmpty(bo.getMailbox()) && !isMailbox(bo.getMailbox())) {
      mistakeData.add(createMistakeCellContent("邮箱", bo.getMailbox(), "邮箱格式不正确"));
      if (ObjectUtil.isNotEmpty(rowMap)) {
        doWithMistakeErrMessage(rowMap, "mailbox", bo.getMailbox(), "邮箱格式不正确!");
      }
    }
    if (ObjectUtil.isNotEmpty(bo.getEntrepreneurialExperience())
        && bo.getEntrepreneurialExperience().length() > 500) {
      mistakeData.add(createMistakeCellContent("创业经历", bo.getEntrepreneurialExperience(),
          "创业经历不能超过500字符"));
      if (ObjectUtil.isNotEmpty(rowMap)) {
        doWithMistakeErrMessage(rowMap, "entrepreneurialExperience",
            bo.getEntrepreneurialExperience(), "创业经历不能超过500字符!");
      }
    }
    if (ObjectUtil.isNotEmpty(bo.getTeamIntroduction())
        && bo.getTeamIntroduction().length() > 500) {
      mistakeData.add(createMistakeCellContent("团队简介", bo.getTeamIntroduction(),
          "团队简介不能超过500字符"));
      if (ObjectUtil.isNotEmpty(rowMap)) {
        doWithMistakeErrMessage(rowMap, "teamIntroduction", bo.getTeamIntroduction(),
            "团队简介不能超过500字符!");
      }
    }
    if (ObjectUtil.isEmpty(bo.getPhone()) || !PhoneUtil.isMobile(bo.getPhone())) {
      mistakeData.add(
          createMistakeCellContent("联系人电话", bo.getPhone(), "联系人电话必填且格式必须正确!"));
      if (ObjectUtil.isNotEmpty(rowMap)) {
        doWithMistakeErrMessage(rowMap, "phone", bo.getPhone(),
            "联系人电话必填且格式必须正确!");
      }
    }

    return mistakeData;
  }


  /** 验证是否为手机号 */
  public static boolean isMobile(String phone){
    String REGEX = "[1][3456789][0-9]{9}";
    return phone.matches(REGEX);
  }


  /** 验证是否为身份证号 */
  public static Boolean isIdCode(String idCode){
    return idCode.matches("(^\\d{15}$)|(^\\d{18}$)|(^\\d{17}(\\d|X|x)$)");
  }

  /** 验证是否为邮箱 */
  public static Boolean isMailbox(String email){
    return email.matches("^\\w+([-+.]\\w+)*@\\w+([-.]\\w+)*\\.\\w+([-.]\\w+)*$");
  }


  /***
   * @Description: 创建错误信息单元格信息
   * @Author: thw
   * @Date: 2023/12/20 11:15
   * @Params:
   * @param headName      对应的表头名称
   * @param value            单元格值
   * @param errMessage            错误信息
   * @return: void
   **/
  public static ExcelMistakeData createMistakeCellContent(String headName, String value,
      String errMessage) {
    ExcelMistakeData excelMistakeData = new ExcelMistakeData();
    excelMistakeData.setName(headName);
    excelMistakeData.setValue(value);
    excelMistakeData.setErrText(errMessage);
    excelMistakeData.setFlag(false);
    excelMistakeData.setState(0);
    return excelMistakeData;
  }


  /***
   * @Description: 创建正确信息单元格信息
   * @Author: thw
   * @Date: 2023/12/20 17:06
   * @Params:
   * @param headName
   * @param value
   * @param errMessage
   * @return: com.crunii.micro.service.appsecond.api.dto.data.ExcelMistakeData
   **/
  public static ExcelMistakeData createRightCellContent(String headName, String value,
      String rightMessage) {
    ExcelMistakeData excelMistakeData = new ExcelMistakeData();
    excelMistakeData.setName(headName);
    excelMistakeData.setValue(value);
    excelMistakeData.setErrText(rightMessage);
    excelMistakeData.setFlag(true);
    excelMistakeData.setState(1);
    return excelMistakeData;
  }

  @Override
  public Result saveBatchImportExcelTeam(HttpServletResponse response, MultipartFile file,
      Integer businessLicense) throws IOException {
   /* Worker currWorker = CurrWorkerUtil.getCurrWorker();

    if (file == null || file.isEmpty() || ObjectUtil.isEmpty(file.getOriginalFilename())) {
      return Result.ofFail("文件为空！");
    }
    String originalFilename = file.getOriginalFilename();
    if (!(ExcelUtil.isExcel2003(originalFilename) || ExcelUtil.isExcel2007(originalFilename))) {
      return Result.ofFail("文件类型出错,请上传xls或xlsx后缀文件！");
    }
    // 出错的导入数据
    List<SpacePassTeamCUExportDTO> errContentList = new ArrayList<>();
    // 导入的字段
    List<String> includeFieldList = getExcelTemplateFieldList(businessLicense);
    try {

      EasyExcel.read(file.getInputStream(), SpacePassTeamCUExportDTO.class,
          new ImportTeamReadListener(this, includeFieldList, currWorker, errContentList));
    } catch (Exception e) {
      log.error("导入团队信息失败", e);
      if (errContentList.size() > 0) {
        String fileName =
            "导入团队信息失败内容导出" + System.currentTimeMillis() + ExcelTypeEnum.XLS.getValue();
        String sheetName = "团队信息失败内容导出";
        // 文件名称
        fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name());
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename=" + fileName);
        // 不符合原因
        includeFieldList.add("errMessage");
        ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream()).build();
        excelWriter.write(errContentList,
            EasyExcel.writerSheet(sheetName).head(SpacePassTeamCUExportDTO.class)
                .includeColumnFiledNames(includeFieldList).build());
        excelWriter.finish();
      }
      return Result.ofFail("导入团队信息失败");
    }
    return Result.ofSuccessMsg("导入新增成功!");
    */
    return null;
  }


  @Override
  public Result doVerifyImportTeamExcel(HttpServletResponse response, MultipartFile file,
      Integer businessLicense,Long spaceId) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    // 查看用户的权限
    doSaveTeamPermissionCheck(currWorker, Long.valueOf(spaceId),
        "通过空间id:" + spaceId + "未找到具体的空间信息");

    if (file == null || file.isEmpty() || ObjectUtil.isEmpty(file.getOriginalFilename())) {
      return Result.ofFail("文件为空！");
    }
    String originalFilename = file.getOriginalFilename();
    if (!(ExcelUtil.isExcel2003(originalFilename) || ExcelUtil.isExcel2007(originalFilename))) {
      return Result.ofFail("文件类型出错,请上传xls或xlsx后缀文件！");
    }

    if (file.getSize() > 5 * 1024 * 1024) {
      return new Result<>("文件大小超过5MB限制！", Boolean.FALSE);
    }

    // 最终返回数据格式
    ExcelDataInfo excelDataInfo = new ExcelDataInfo();

      HashMap<String, List<TsmPropertyValue>> tsmProValueMap = new HashMap<>();
      // 民族
      List<TsmPropertyValue> nations = tsmPropertyValueRepo.findAllByPropCodeAndState(propCode[0],
          1);
      tsmProValueMap.put(propCode[0], nations);
      // 学历
      List<TsmPropertyValue> formalSchoolings = tsmPropertyValueRepo.findAllByPropCodeAndState(
          propCode[1],
          1);
      tsmProValueMap.put(propCode[1], formalSchoolings);
      // 政治面貌
      List<TsmPropertyValue> politicalStatus = tsmPropertyValueRepo.findAllByPropCodeAndState(
          propCode[2],
          1);
      tsmProValueMap.put(propCode[2], politicalStatus);
      // 行业领域
      List<TsmPropertyValue> industryFields = tsmPropertyValueRepo.findAllByPropCodeAndState(
          propCode[3],
          1);
      tsmProValueMap.put(propCode[3], industryFields);

      // 出错的导入数据
      List<MistakeDataInfo> errContentList = new ArrayList<>();
      // 导入的字段名称
      List<String> includeFieldList = getExcelTemplateFieldList(businessLicense);

      // 没有错误的缓存key
      Snowflake snowflake = new Snowflake();
      String dataKey = "team" + snowflake.nextId();

      // 导入的字段name和表头值
    LinkedHashMap<String, String> headeValueMap = getHeaderContent(SpacePassTeamCUExportDTO.class);

      // 导入excel
      try {
        EasyExcel.read(file.getInputStream(), SpacePassTeamCUExportDTO.class,
            new ImportTeamReadListenerTwo(this, includeFieldList, currWorker, errContentList,
                tsmProValueMap, headeValueMap, dataKey)).headRowNumber(2).ignoreEmptyRow(true).sheet().doRead();
      }catch (Exception e){
        log.error("批量导入新增团队信息失败", e);
        throw new BusinessException("批量导入新增团队信息失败:" + e.getMessage());
      }


      // 判断是否存在错误数据
      List<ExcelMistakeData> errDataList = new ArrayList<>();
      if (ObjectUtil.isNotEmpty(errContentList)) {
        for (MistakeDataInfo mistakeDataInfo : errContentList) {
          if (ObjectUtil.isNotEmpty(mistakeDataInfo)) {
            errDataList.addAll(mistakeDataInfo.getMistakeDataInfo());

          }
        }
      }

      if (ObjectUtil.isNotEmpty(errDataList)) {
        List<ExcelMistakeData> errList = errDataList.stream().filter(e -> Integer.valueOf(0).equals(e.getState()))
            .collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(errList)) {
          excelDataInfo.setLists(errContentList);
        } else {
          excelDataInfo.setDataKey(dataKey);
        }
      }
      return Result.ofSuccess(excelDataInfo);
  }

  @Override
  public Result saveTeamBatchImport(TeamBatchImportDTO dto) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    // 查看用户的权限
    doSaveTeamPermissionCheck(currWorker, Long.valueOf(dto.getSpaceId()),
        "通过空间id:" + dto.getSpaceId() + "未找到具体的空间信息");
    // 获取临时表数据
    List<QczdTeamStayTemp> teamStayTemps = qczdTeamStayTempRepo.findAllByDataKey(dto.getDataKey());
    if (ObjectUtil.isEmpty(teamStayTemps)){
      throw new BusinessException("导入失败,没有可导入数据!");
    }

    List<QczdTeamStay> qczdTeamStays = new ArrayList<>();
    // 团队与行业领域的关联表
    List<QczdIndustryField> industryFieldList = new ArrayList<>();

    for (QczdTeamStayTemp teamStayTemp : teamStayTemps) {
      QczdTeamStay qczdTeamStay = new QczdTeamStay();
      BeanUtil.copyPropertiesIgnoreNull(teamStayTemp,qczdTeamStay);
      qczdTeamStays.add(qczdTeamStay);

      // 新增空间与团队关联表
      QczdSpaceTeam qczdSpaceTeam = new QczdSpaceTeam();
      createSpaceRelationTeam(Long.valueOf(dto.getSpaceId()), qczdTeamStay.getId(), currWorker,Boolean.TRUE);

      // 将临时表中的行业领域新增到关联表中
      if (ObjectUtil.isNotEmpty(teamStayTemp.getIndustryField())) {
        String[] industryFields = teamStayTemp.getIndustryField().split(",");
        if (ObjectUtil.isNotEmpty(industryFields)){
          for (String industryField : industryFields) {
            QczdIndustryField fieldEntity = new QczdIndustryField();
            fieldEntity.setTeamId(teamStayTemp.getId());
            fieldEntity.setIndustryField(industryField);
            fieldEntity.setDelFlag(0);
            industryFieldList.add(fieldEntity);
          }
        }
      }
    }
    // 删除临时表数据
    qczdTeamStayTempRepo.deleteAllByDataKey(dto.getDataKey());
    // 新增内容
    qczdTeamStayRepo.saveAll(qczdTeamStays);
    qczdIndustryFieldRepo.saveAll(industryFieldList);
    return Result.ofSuccessWithMsg(String.valueOf(qczdTeamStays.size()),"批量新增团队导入成功!");
  }

  /***
   * @Description: 新增团队时的权限校验
   * @Author: thw
   * @Date: 2023/12/23 15:38
   * @Params:
   * @param currWorker    当前用户
   * @param spaceId       空间id
   * @param message       提示信息
   * @return: void
   **/
  private void doSaveTeamPermissionCheck(Worker currWorker, Long spaceId, String message) {
    Map<String, YouthSpaceRoleEnum> workerPermissonMap = youthSpaceService.getCurrentWorkerIncludeRole(
        currWorker);
    // 目前最大权限都可以操作
    if (ObjectUtil.isEmpty(workerPermissonMap)){
      throw new BusinessException("没有权限操作空间下的团队");
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))){
      // 不做处理
      return;
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))) {
      Optional<QczdSpaceContent> contentOptional = qczdSpaceContentRepo.findById(
          spaceId);
      if (contentOptional.isPresent()) {
        QczdSpaceContent spaceContent = contentOptional.get();
        if (ObjectUtil.notEqual(currWorker.getLoginId(), spaceContent.getCreateId())) {
          throw new BusinessException("该"+
              YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "不是此空间的创建人，不能操作该空间团队的内容。");
        }
      } else {
        throw new BusinessException(message);
      }
    } else {
//      throw new BusinessException("当前用户不是" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "没有操作权限！");
      throw new BusinessException("没有权限操作空间下的团队");
    }
  }

  /***
   * @Description: 获取动态的表头Field集合
   * @Author: thw
   * @Date: 2023/12/20 15:45
   * @Params:
   * @param obj                 实例对象
   * @param includeFieldList    动态导入包含的实例属性字段
   * @return: java.util.List<java.lang.reflect.Field>
   * */
  public static List<Field> getHeadFieldList(Class<?> obj, List<String> includeFieldList) {
    List<Field> fieldList = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(obj) && ObjectUtil.isNotEmpty(includeFieldList)) {
      fieldList = new ArrayList<>();
      Field[] fields = obj.getDeclaredFields();
      for (Field field : fields) {
        String fieldName = field.getName();
        if (includeFieldList.contains(fieldName)) {
          fieldList.add(field);
        }
      }
    }
    return fieldList;
  }


  /***
   * @Description: 获取对象属性值
   * @Author: thw
   * @Date: 2023/12/20 15:59
   * @Params:
   * @param obj       实例对象
   * @param field     对象属性
   * @return: java.lang.Object
   **/
  public static Object getFieldValue(Class<?> obj, Field field) {
    if (ObjectUtil.isNotEmpty(obj) && ObjectUtil.isNotEmpty(field)) {
      Object value;
      try {
        field.setAccessible(true);
        value = field.get(obj);
      } catch (IllegalAccessException e) {
        throw new BusinessException("获取对象值失败!");
      }
      return value;
    } else {
      return null;
    }
  }

  /**
   * 根据Excel表头动态设置对象的属性
   *
   * @param clazz   对象的Class
   * @param headers Excel表头
   * @return 对象属性与Excel表头的映射关系
   */
  public static <T> LinkedHashMap<String, String> getHeaderContent(Class<T> clazz) {
    LinkedHashMap<String, String> headMap = new LinkedHashMap<>();
    Field[] fields = clazz.getDeclaredFields();
    for (Field field : fields) {
      ExcelProperty excelProperty = field.getAnnotation(ExcelProperty.class);
      if (excelProperty != null) {
        headMap.put(field.getName(), excelProperty.value()[0]);
      }
    }
    return headMap;
  }

  /***
   * @Description: 获取需要导出的字段列表
   * @Author: thw
   * @Date: 2023/12/15 11:28
   * @Params:
   * @param businessLicense  营业执照(0:无、1:有)
   * @return: java.util.List<java.lang.String>
   **/
  private List<String> getExcelTemplateFieldList(Integer businessLicense) {
    // 获取需要导出的字段列表
    List<String> includeFieldList = getFiledList(SpacePassTeamCUExportDTO.class);
    if (ObjectUtil.isNotEmpty(businessLicense) && BusinessStateEnum.NO_BUSINESS.getNum()
        .equals(businessLicense)) {
      includeFieldList.remove("companyName");
      includeFieldList.remove("socialCode");
    } else {
      includeFieldList.remove("entrepreneurialExperience");
      includeFieldList.remove("teamIntroduction");
    }
    return includeFieldList;
  }

  /***
   * @Description: 导入时单元格填充错误信息
   * @Author: thw
   * @Date: 2023/12/21 9:49
   * @Params:
   * @param rowMap        行数据映射
   * @param mapKey        map的key（属性名）
   * @param errMessage    错误信息
   * @return: void
   **/
  public static void doWithMistakeErrMessage(Map<String, ExcelMistakeData> rowMap, String mapKey,
      String value, String errMessage) {
    if (ObjectUtil.isNotEmpty(rowMap)) {
      if (rowMap.containsKey(mapKey)) {
        ExcelMistakeData excelMistakeData = rowMap.get(mapKey);
        excelMistakeData.setValue(value);
        excelMistakeData.setFlag(false);
        excelMistakeData.setState(0);
        excelMistakeData.setErrText(errMessage);
      }
    }
  }

  /**
   * 获取导入导出需要的属性名
   */
  public List<String> getFiledList(Class<?> obj) {
    List<String> fieldNames = new ArrayList<>();
    Field[] fields = obj.getDeclaredFields();
    if (ObjectUtil.isNotEmpty(fields)) {
      for (Field field : fields) {
        boolean exist = field.isAnnotationPresent(ExcelProperty.class);
        if (exist) {
          fieldNames.add(field.getName());
        }
      }
      return fieldNames;
    }
    return fieldNames;
  }

  @Override
  public void batchExportTeam(HttpServletResponse response, SpacePassTeamPageReqDTO bo) {
    if (ObjectUtil.isEmpty(bo) || ObjectUtil.isEmpty(bo)) {
      throw new BusinessException("参数对象必传!");
    }
    if (ObjectUtil.isEmpty(bo.getSpaceId())) {
      throw new BusinessException("空间id必传!");
    }

    PageReq<SpacePassTeamPageReqDTO> req = new PageReq<>();
    req.setQuery(bo);
    req.setPageSize(10000000);
    PageRsp<SpacePassTeamPageVO> pageRsp = spacePassTeamDao.pageTeamInfo(req);
    // 返回值包装
    packageContent(pageRsp.getResult());

    Optional<QczdSpaceContent> contentOptional = qczdSpaceContentRepo.findById(
        Long.valueOf(bo.getSpaceId()));

    String spaceName = "";
    if (contentOptional.isPresent()) {
      spaceName = contentOptional.get().getSpaceName();
    }

    try {
      ExcelUtil.writeSingleExcel(spaceName + "已入住团队信息导出", spaceName + "已入住团队信息",
          pageRsp.getResult(), SpacePassTeamPageVO.class);
    } catch (Exception e) {
      log.error(spaceName + "导出团队信息失败", e);
      e.printStackTrace();
      throw new BusinessException(spaceName + "已入住团队信息导出失败!");
    }
  }

  @Override
  public void exportSpaceTeamTemplate(HttpServletResponse response, Integer businessLicense) {
    try {
      // 导入的字段
      List<String> includeFieldList = getExcelTemplateFieldList(businessLicense);
      String fixedContent = "营业执照模板";
      String fileName = "";
      String sheetName = "";
      if (BusinessStateEnum.NO_BUSINESS.getNum().equals(businessLicense)) {
        fileName = BusinessStateEnum.NO_BUSINESS.getDesc() + fixedContent;
        sheetName = BusinessStateEnum.NO_BUSINESS.getDesc() + fixedContent;
      } else if (BusinessStateEnum.YES_BUSINESS.getNum().equals(businessLicense)) {
        fileName = BusinessStateEnum.YES_BUSINESS.getDesc() + fixedContent;
        sheetName = BusinessStateEnum.YES_BUSINESS.getDesc() + fixedContent;
      } else {
        throw new BusinessException("参数传递有误!应为数值[0|1]");
      }
      fileName = "团队新增" + sheetName + System.currentTimeMillis() + ExcelTypeEnum.XLS.getValue();
      sheetName = "团队新增" + sheetName;
      // 文件名称
      fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name());
      response.setContentType("application/vnd.ms-excel;charset=utf-8");
      response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename=" + fileName);
      ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream()).build();
      excelWriter.write(Lists.newArrayList(),
          EasyExcel.writerSheet(sheetName).head(SpacePassTeamCUExportDTO.class)
              .includeColumnFiledNames(includeFieldList).build());
      // 关闭ExcelWriter，释放内存
      excelWriter.finish();
    } catch (Exception e) {
      throw new BusinessException("团队新增模板导出失败!");
    }
  }

  /***
   * @Description: 更改团队状态(孵化结束)
   * @Author: thw
   * @Date: 2023/12/19 15:49
   * @Params:
   * @param teamId
   * @return: com.crunii.micro.common.dto.Result
   **/
  @Override
  public Result updateTeamState(Long teamId) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();

    Optional<QczdTeamStay> teamStayOptional = qczdTeamStayRepo.findById(teamId);
    if (!teamStayOptional.isPresent()) {
      throw new BusinessException("未通过团队id:" + teamId + "查询到相关信息");
    }

    // 查看用户的权限
    Map<String, YouthSpaceRoleEnum> workerPermissionMap = youthSpaceService.getCurrentWorkerIncludeRole(
        currWorker);
    // 青创空间管理员：发布本空间的展示信息。
    if (ObjectUtil.isEmpty(workerPermissionMap)){
      throw new BusinessException("没有修改此空间下团队的孵化结束权限.");
    }else if (ObjectUtil.isNotEmpty(workerPermissionMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))){
      // 不处理
    }else if(ObjectUtil.isNotEmpty(workerPermissionMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      List<QczdSpaceTeam> qczdSpaceTeams = qczdSpaceTeamRepo.findByTeamId(teamId);
      if (ObjectUtil.isEmpty(qczdSpaceTeams)){
        throw new BusinessException("通过团队id:" + teamId + "未找到与团队关联的空间信息");
      }
      List<Long> spaceIds = qczdSpaceTeams.stream().filter(e -> e.getSpaceId() != null).map(QczdSpaceTeam::getSpaceId).distinct()
          .collect(Collectors.toList());
      List<QczdSpaceContent> spaceContents = qczdSpaceContentRepo.findAllById(spaceIds);
      if (ObjectUtil.isEmpty(spaceContents)){
        throw new BusinessException("通过空间id:" + spaceIds + "未找到具体的空间信息");
      }

      Boolean flag = Boolean.FALSE;
      // 通过空间id查询空间创建者id(目前空间与团队是一对多，某个一团队只能属于一个空间下面)
      outerLoop:for (QczdSpaceContent spaceContent : spaceContents) {
        for (QczdSpaceTeam qczdSpaceTeam : qczdSpaceTeams) {
          if (ObjectUtil.equals(spaceContent.getId(),qczdSpaceTeam.getSpaceId())){
            if (ObjectUtil.equals(currWorker.getLoginId(),spaceContent.getCreateId())){
              flag = Boolean.TRUE;
              break outerLoop;
            }
          }
        }

      }
      if (!flag){
        throw new BusinessException("该" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "不是该空间的创建者，没有操作空间下团队的权限。");
      }
    }


    QczdTeamStay qczdTeamStay = teamStayOptional.get();
    // 2、孵化结束是团队入驻到期，点击孵化结束按钮，更改入驻状态为【已结束孵化】
    qczdTeamStay.setTeamState(1);
    qczdTeamStay.setOperateId((long) currWorker.getLoginId());
    qczdTeamStay.setOperateName(currWorker.getWorkerName());
    qczdTeamStay.setOperateTime(LocalDateTime.now());
    qczdTeamStayRepo.save(qczdTeamStay);
    return Result.ofSuccessMsg("孵化结束操作成功!");
  }


}
