package com.crunii.micro.service.appsecond.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.PhoneUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.TsmRoleDto;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.service.CurrWorkerService;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.appsecond.api.dto.youthsapce.*;
import com.crunii.micro.service.appsecond.api.vo.youthspace.YouthSpacePageVO;
import com.crunii.micro.service.appsecond.enums.StayTeamEnums.TeamStateEnum;
import com.crunii.micro.service.appsecond.enums.YouthSpaceEnums.*;
import com.crunii.micro.service.appsecond.enums.YouthSpaceRoleEnum;
import com.crunii.micro.service.appsecond.jdbc.SpacePassTeamDao;
import com.crunii.micro.service.appsecond.jdbc.YouthSpaceJdbc;
import com.crunii.micro.service.appsecond.service.YouthSpaceService;
import com.google.common.base.Joiner;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.ConcurrentMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.aspectj.weaver.ast.Var;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

/**
 * @Author thw
 * @date 2023/12/12
 * @description 青创空间管理
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class YouthSpaceServiceImpl implements YouthSpaceService {

  private final QczdSpaceContentRepo qczdSpaceContentRepo;

  private final QczdSharingSettingsRepo qczdSharingSettingsRepo;

  private final QczdFocusOnAreasRepo qczdFocusOnAreasRepo;

  private final QczdRelationFileRepo qczdRelationFileRepo;

  private final QczdPriceObjectRepo qczdPriceObjectRepo;

  private final QczdSpaceTeamRepo qczdSpaceTeamRepo;

  private final TsmPropertyValueRepo tsmPropertyValueRepo;

  private final YouthSpaceJdbc youthSpaceJdbc;

  private final SpacePassTeamDao spacePassTeamDao;

  private final TsmRoleRepo tsmRoleRepo;

  private final TsmWorkerRoleRepo tsmWorkerRoleRepo;

  private final TsmWorkerOrgAllRepo tsmWorkerOrgAllRepo;


  /** lacalDateTime字符串时间截取统一长度 */
  public final int timeSubNum = 19;
  @Override
  public PageRsp<YouthSpacePageVO> pageSpaceInfo(PageReq<YouthSpacePageReqDTO> req) {
    if (ObjectUtil.isEmpty(req) || ObjectUtil.isEmpty(req.getQuery())) {
      throw new BusinessException("参数对象必传!");
    }

    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    Boolean flag = Boolean.FALSE;
    // 查看用户的权限
    Map<String, YouthSpaceRoleEnum> workerPermissonMap = getCurrentWorkerIncludeRole(
        currWorker);
    if (ObjectUtil.isEmpty(workerPermissonMap)){
      throw new BusinessException("当前用户没有查看青创空间的权限");
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))){
      // 不做任何限制（2、青年创新创业服务中心管理员可查看到所有青创空间的数据）
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      // 1、各青创空间管理员看到自己青创空间展示数据（及本空间下的团队数据）、自己青创空间申请数据；
      flag = Boolean.TRUE;
    }
    PageRsp<YouthSpacePageVO> pageRsp = youthSpaceJdbc.pageSpaceInfo(req,flag,currWorker);
    // 封装数据
    pageData(pageRsp.getResult());
    return pageRsp;
  }

  /**
   * 分页数据封装
   */
  public void pageData(List<YouthSpacePageVO> result) {
    if (ObjectUtil.isNotEmpty(result)) {
      for (YouthSpacePageVO pageVO : result) {

        // 已入住团队
        List<QczdSpaceTeam> spaceTeams = spacePassTeamDao.findBySpaceIdAndAuditState(
            Long.valueOf(pageVO.getId()));
        if (ObjectUtil.isNotEmpty(spaceTeams)) {
          // 需要保证关联表数据状态一致修改
          pageVO.setPassTeamNum(spaceTeams.size());
        }

        // 层级审核状态
        EvaluateStateEnum evaluateStateEnum = EvaluateStateEnum.getEvaluateStateEnum(
            pageVO.getEvaluateState());
        if (ObjectUtil.isNotEmpty(evaluateStateEnum)) {
          pageVO.setEvaluateStateName(evaluateStateEnum.getDesc());
        }

        // 青创空间层级标签
        HierarchyLabelEnum labelEnum = HierarchyLabelEnum.getHierarchyLabel(
            pageVO.getHierarchyLabel());
        if (ObjectUtil.isNotEmpty(labelEnum)) {
          pageVO.setHierarchyLabelName(labelEnum.getDesc());
        }

        // 设置状态
        StateEnum stateEnum = StateEnum.getStateEnum(pageVO.getState());
        if (ObjectUtil.isNotEmpty(stateEnum)) {
          pageVO.setStateName(stateEnum.getDesc());
        }

        // 优惠政策标志
        CommEnum policyEnum = CommEnum.getCommEnum(pageVO.getPreferentialPolicySign());
        if (ObjectUtil.isNotEmpty(policyEnum)) {
          pageVO.setPreferentialPolicyName(policyEnum.getDesc());
        }

        // 入驻要求标志
        CommEnum requireEnum = CommEnum.getCommEnum(pageVO.getSettledRequiresSign());
        if (ObjectUtil.isNotEmpty(requireEnum)) {
          pageVO.setSettledRequiresName(requireEnum.getDesc());
        }

        // 共享配置封装
        List<QczdSharingSettings> settingsList = qczdSharingSettingsRepo.findBySpaceId(
            Long.valueOf(pageVO.getId()));
        List<String> stringSet = settingsList.stream().map(QczdSharingSettings::getConfigName).distinct()
            .collect(Collectors.toList());
        List<StaticValueDTO> sharingSettings = staticPackData(stringSet,
            "qckjsharingSettings");
        pageVO.setSharingSettings(sharingSettings);

        // 侧重领域封装
        List<QczdFocusOnAreas> focusList = qczdFocusOnAreasRepo.findBySpaceId(
            Long.valueOf(pageVO.getId()));
        List<String> fieldNameSet = focusList.stream().map(QczdFocusOnAreas::getFieldName).distinct()
            .collect(Collectors.toList());
        List<StaticValueDTO> focusOnAreas = staticPackData(fieldNameSet,
            "qckjFocusOnAreas");
        pageVO.setFocusOnAreas(focusOnAreas);
        // 单价封装
        List<QczdPriceObject> priceObjects = qczdPriceObjectRepo.findBySpaceId(
            Long.valueOf(pageVO.getId()));
        if (ObjectUtil.isNotEmpty(priceObjects)) {
          // 每个空间id对应的单价对象最多不过两个
          if (priceObjects.size() > 2) {
            throw new BusinessException("空间对应单价数据出错!出错空间id为:" + pageVO.getId());
          }
          // 根据单价指向(0:工位单价、1:面积单价)分组
          ConcurrentMap<Integer, List<QczdPriceObject>> concurrentMap = priceObjects.stream()
              .collect(Collectors.groupingByConcurrent(QczdPriceObject::getPointType));
          // 工位单价
          List<QczdPriceObject> workPriceObjects = concurrentMap.get(CommEnum.ZERO_COM.getNum());
          if (ObjectUtil.isNotEmpty(workPriceObjects) && workPriceObjects.size() == 1) {
            QczdPriceObject workPriceObject = workPriceObjects.get(0);
            // 将单价对象转换为DTO
            QczdPriceObjectDTO priceObjectDTO = covertPriceObjectDTO(workPriceObject);
            pageVO.setWorkerUnitPrice(priceObjectDTO);
          }

          // 面积单价
          List<QczdPriceObject> areaPriceObjects = concurrentMap.get(CommEnum.ONE_COM.getNum());
          if (ObjectUtil.isNotEmpty(areaPriceObjects) && areaPriceObjects.size() == 1) {
            QczdPriceObject areaPriceObject = areaPriceObjects.get(0);
            QczdPriceObjectDTO priceObjectDTO = covertPriceObjectDTO(areaPriceObject);
            pageVO.setAreaUnitPrice(priceObjectDTO);
          }

        }
        // 宣传展示以及pdf封装
        List<QczdRelationFile> files = qczdRelationFileRepo.findByServiceIdAndFileTypeIn(
            Long.valueOf(pageVO.getId()),
            Arrays.asList(1, 2, 3));
       if (ObjectUtil.isNotEmpty(files)){
         List<YouthSpaceFileDTO> imageDTOs = new ArrayList<>();
         List<YouthSpaceFileDTO> videoDTOs = new ArrayList<>();
         List<YouthSpaceFileDTO> pdfDTOs = new ArrayList<>();
         for (QczdRelationFile file : files) {
           // 图片
           if (FileTypeEnum.ONCE_TYPE.getNum().equals(file.getFileType())){
             // 文件转DTO
             imageDTOs.add(FileCovetFileDTO(file));
           }
           if (FileTypeEnum.SECOND_TYPE.getNum().equals(file.getFileType())){
             // 视频转DTO
             videoDTOs.add(FileCovetFileDTO(file));
           }
           if (FileTypeEnum.THIRD_TYPE.getNum().equals(file.getFileType())){
             // pdf转DTO
             pdfDTOs.add(FileCovetFileDTO(file));
           }
         }
         pageVO.setImageFileIds(imageDTOs);
         pageVO.setVideoFileIds(videoDTOs);
         pageVO.setHierarchyFile(pdfDTOs);
       }

        // 统一处理时间
        pageVO.setUpdatedTime(LocalDateTimeSub(pageVO.getUpdatedTime(),timeSubNum));
      }
    }

  }

  /***
   * @Description:
   * @Author: thw
   * @Date: 2023/12/19 16:24
   * @Params:
   * @param time         lacalDateTime时间截取
   * @param subNumEnd    截取的末尾，不包含
   * @return: java.lang.String
   **/
  public String LocalDateTimeSub(String time,int subNumEnd){
    if (ObjectUtil.isNotEmpty(time)){
      if (subNumEnd < 0 || time.length() < subNumEnd){
        throw new BusinessException("截取失败!");
      }
      return time.substring(0,subNumEnd);
    }else {
      return time;
    }

  }

  /***
   * 文件实例转文件DTO
   **/
  private static YouthSpaceFileDTO FileCovetFileDTO(QczdRelationFile file) {
    YouthSpaceFileDTO fileDTO = new YouthSpaceFileDTO();
    fileDTO.setFileName(file.getFileName());
    fileDTO.setFileUrl(file.getFileUrl());
    fileDTO.setFileType(file.getFileSuffix());
    fileDTO.setFileId(file.getFileId());
    return fileDTO;
  }

  /***
   * @Description:
   * @Author: thw
   * @Date: 2023/12/18 14:42
   * @Params:
   * @param list     表中存储的静态值code集合
   * @param propCode  静态值PropCode
   * @return: java.util.List<com.crunii.micro.service.appsecond.api.dto.youthsapce.StaticValueDTO>
   **/
  public List<StaticValueDTO> staticPackData(List<String> list, String propCode) {
    List<TsmPropertyValue> settings = tsmPropertyValueRepo.findAllByPropCodeAndState(
        propCode, 1);
    List<StaticValueDTO> staticValueDTOS = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(list) && ObjectUtil.isNotEmpty(settings)) {
      Map<String, List<TsmPropertyValue>> stringListMap = settings.stream()
          .collect(Collectors.groupingBy(TsmPropertyValue::getValueCode));
      if (ObjectUtil.isNotEmpty(stringListMap)) {
        StaticValueDTO staticValueDTO;
        for (int i = 0; i < list.size(); i++) {
          staticValueDTO = new StaticValueDTO();
          List<TsmPropertyValue> tsmPropertyValues = stringListMap.get(list.get(i));
          if (ObjectUtil.isNotEmpty(tsmPropertyValues)){
            // 静态值编码唯一
            TsmPropertyValue tsmPropertyValue = tsmPropertyValues.get(0);
            staticValueDTO.setValueCode(tsmPropertyValue.getValueCode());
            staticValueDTO.setValueName(tsmPropertyValue.getValueName());
            staticValueDTOS.add(staticValueDTO);
          }
        }
      }
    }
    return staticValueDTOS;
  }


  @Override
  public Result saveSpace(YouthSpacCReqDTO bo) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();

    // 查看用户的权限
    Map<String, YouthSpaceRoleEnum> workerPermissonMap = getCurrentWorkerIncludeRole(
        currWorker);
    // 青创空间管理员：发布本空间的展示信息。(现在变成了两个角色都可以发布)
    if (ObjectUtil.isEmpty(workerPermissonMap)){
      throw new BusinessException("没有发布空间权限");
    }

/*    else if((ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode())))){
      // 不处理
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      throw new BusinessException(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "才有发布空间的权限.");
    }*/

    // 参数校验
    checkParameterLimit(bo);
    QczdSpaceContent qczdSpaceContent = new QczdSpaceContent();
    List<String> ignoreProperties = new ArrayList<>();
    // 忽略属性（新增、修改一致),并为特定属性赋值
    ignorePropertiesList(ignoreProperties, bo, qczdSpaceContent, currWorker,Boolean.TRUE);
    QczdSpaceContent spaceContentSave = qczdSpaceContentRepo.save(qczdSpaceContent);
    // 针对关联数据的处理
    // 设置区域或工位单价
    HashMap<Integer, YouthSpacePriceReqDTO> dtoHashMap = new HashMap<>();
    YouthSpacePriceReqDTO workerPrice = bo.getWorkerPrice();
    if (ObjectUtil.isNotEmpty(workerPrice) && ObjectUtil.isNotEmpty(bo.getTotalWorkers())){
      dtoHashMap.put(0, workerPrice);
    }else {
      dtoHashMap.put(0, null);
    }
    YouthSpacePriceReqDTO areaPrice = bo.getAreaPrice();
    if (ObjectUtil.isNotEmpty(areaPrice) && ObjectUtil.isNotEmpty(bo.getTotalArea())){
      dtoHashMap.put(1, areaPrice);
    }else {
      dtoHashMap.put(1, null);
    }
    doWorkAndAreaPrice(dtoHashMap, spaceContentSave.getId(), Boolean.TRUE);
    // 共享配置
    doSharingSettings(bo.getSharingSettings(), Long.valueOf(spaceContentSave.getId()),
        Boolean.TRUE);
    // 侧重领域
    doFocusOnAreas(bo.getFocusOnAreas(), Long.valueOf(spaceContentSave.getId()), Boolean.TRUE);
    // 空间层级关联文件和宣传展示中上传的图片与视频
    doFileRelationInsert(bo, Long.valueOf(spaceContentSave.getId()), currWorker, Boolean.TRUE);
    return Result.ofSuccessMsg("空间发布成功!");
  }

  /***
   * @Description:
   * @Author: thw
   * @Date: 2023/12/16 16:37
   * @Params:
   * @param dtoHashMap  区域或工位的单价对象
   * @param spaceId  空间id
   * @param flag  是否为修改(ture:新增;false:修改)
   * @return: void
   **/
  private void doWorkAndAreaPrice(HashMap<Integer, YouthSpacePriceReqDTO> dtoHashMap, Long spaceId,
      Boolean flag) {
    if (!flag) {
      qczdPriceObjectRepo.deleteBySpaceId(spaceId);
    }
    if (ObjectUtil.isNotEmpty(dtoHashMap)) {
      List<QczdPriceObject> priceObjects = new ArrayList<>();
      for (int i = 0; i < dtoHashMap.size(); i++) {
        YouthSpacePriceReqDTO priceDTO = dtoHashMap.get(i);
        if (ObjectUtil.isNotEmpty(priceDTO)) {
          QczdPriceObject qczdPriceObject = new QczdPriceObject();
          qczdPriceObject.setSpaceId(spaceId);
          // 单价指向(0:工位单价、1:面积单价)
          qczdPriceObject.setPointType(i);
          qczdPriceObject.setPriceType(priceDTO.getPriceType());
          qczdPriceObject.setIncludeState(priceDTO.getIncludeState());
          qczdPriceObject.setPriceMax(new BigDecimal(priceDTO.getPriceMax()));
          qczdPriceObject.setPriceMin(new BigDecimal(priceDTO.getPriceMin()));
          String includeContent = Joiner.on(",").join(priceDTO.getIncludeContent());
          qczdPriceObject.setIncludeContent(includeContent);
          qczdPriceObject.setDelFlag(0);
          priceObjects.add(qczdPriceObject);
        }
      }
      qczdPriceObjectRepo.saveAll(priceObjects);
    }
  }


  @Override
  public Result updateSpace(YouthSpacCReqDTO bo) {

    Worker currWorker = CurrWorkerUtil.getCurrWorker();

    if (ObjectUtil.isEmpty(bo.getId())) {
      throw new BusinessException("修改时id必传!");
    }

    Optional<QczdSpaceContent> spaceContentOption = qczdSpaceContentRepo.findById(
        Long.valueOf(bo.getId()));
    if (!spaceContentOption.isPresent()) {
      throw new BusinessException("通过空间id" +bo.getId()+ "为查询到相关数据!");
    }
    QczdSpaceContent spaceContent = spaceContentOption.get();
    // 查看用户的权限
    Map<String, YouthSpaceRoleEnum> workerPermissonMap = getCurrentWorkerIncludeRole(
        currWorker);
    // 青创空间管理员：发布本空间的展示信息。
/*    if (ObjectUtil.isEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      throw new BusinessException("只有" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "才有修改已发布空间的权限.");
    }

    if (ObjectUtil.notEqual(spaceContent.getCreateId(),currWorker.getLoginId())){
      throw new BusinessException("此" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "不是该空间的创建人,没有修改此空间权限!");
    }*/

    // 2023-12-26 现在两个角色都可以操作 最新
    if (ObjectUtil.isEmpty(workerPermissonMap)){
      throw new BusinessException("没有修改此空间权限.");
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))){
      // 不处理
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      if (ObjectUtil.notEqual(spaceContent.getCreateId(),currWorker.getLoginId())){
        throw new BusinessException("此" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "不是该空间的创建人,没有修改此空间权限!");
      }
    }

//    // 审核通过的数据不允许修改
//    if (ObjectUtil.equals(spaceContent.getEvaluateState(), EvaluateStateEnum.PASS_AUDIT.getNum())){
//      throw new BusinessException("审核通过的数据不允许修改!");
//    }

    // 参数校验
    checkParameterLimit(bo);

    List<String> ignoreProperties = new ArrayList<>();
    // 忽略属性（新增、修改一致),并为特定属性赋值
    ignorePropertiesList(ignoreProperties, bo, spaceContent, currWorker,Boolean.FALSE);
    QczdSpaceContent updateSpace = qczdSpaceContentRepo.save(spaceContent);
    // 针对关联数据的处理
    // 设置区域或工位单价
    HashMap<Integer, YouthSpacePriceReqDTO> dtoHashMap = new HashMap<>();
    YouthSpacePriceReqDTO workerPrice = bo.getWorkerPrice();
    if (ObjectUtil.isNotEmpty(workerPrice) && ObjectUtil.isNotEmpty(bo.getTotalWorkers())){
      dtoHashMap.put(0, workerPrice);
    }else {
      dtoHashMap.put(0, null);
    }
    YouthSpacePriceReqDTO areaPrice = bo.getAreaPrice();
    if (ObjectUtil.isNotEmpty(areaPrice) && ObjectUtil.isNotEmpty(bo.getTotalArea())){
      dtoHashMap.put(1, areaPrice);
    }else {
      dtoHashMap.put(1, null);
    }
    doWorkAndAreaPrice(dtoHashMap, updateSpace.getId(), Boolean.FALSE);
    // 共享配置
    doSharingSettings(bo.getSharingSettings(), Long.valueOf(updateSpace.getId()), Boolean.FALSE);
    // 侧重领域
    doFocusOnAreas(bo.getFocusOnAreas(), Long.valueOf(updateSpace.getId()), Boolean.FALSE);
    // 空间层级关联文件和宣传展示中上传的图片与视频
    doFileRelationInsert(bo, Long.valueOf(updateSpace.getId()), currWorker, Boolean.FALSE);
    return Result.ofSuccessMsg("空间编辑成功!");

  }

  @Override
  public Result updateEvaluateState(YouthSpacAuditReqDTO bo) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();

    if (ObjectUtil.isEmpty(bo)) {
      throw new BusinessException("参数对象必传");
    }
    if (ObjectUtil.isEmpty(bo.getId()) || ObjectUtil.isEmpty(bo.getEvaluateState())) {
      throw new BusinessException("必传参数为空!");
    }
    Optional<QczdSpaceContent> contentOptional = qczdSpaceContentRepo.findById(
        Long.valueOf(bo.getId()));
    if (!contentOptional.isPresent()) {
      throw new BusinessException("通过空间id:" + bo.getId() + "为查询到相关数据!");
    }

    // 查看用户的权限
    Map<String, YouthSpaceRoleEnum> workerPermissonMap = getCurrentWorkerIncludeRole(
        currWorker);
    // 团市委管理员：查看各青创空间发布的空间信息，审核级别为市级、国家级的空间信息；
    if (ObjectUtil.isEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))) {
      throw new BusinessException(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getDesc() + "才有审核权限！");
    }

    QczdSpaceContent spaceContent = contentOptional.get();
    if (!(HierarchyLabelEnum.CITY_LABEL.getName().equals(spaceContent.getHierarchyLabel())
        || HierarchyLabelEnum.COUNTRY_LABEL.getName().equals(spaceContent.getHierarchyLabel()))) {
      throw new BusinessException("“青创空间层级”选择【市级】或【国家级】时，空间数据才需要审核!");
    }

    spaceContent.setEvaluateState(bo.getEvaluateState());
    spaceContent.setUpdatedBy(currWorker.getWorkerName());
    spaceContent.setUpdateId(currWorker.getLoginId());
    spaceContent.setUpdatedTime(LocalDateTime.now());
    if (ObjectUtil.isNotEmpty(bo.getEvaluateRemarks())) {
      spaceContent.setEvaluateRemarks(bo.getEvaluateRemarks());
    }
    qczdSpaceContentRepo.save(spaceContent);
    return Result.ofSuccessMsg("层级审核成功!");
  }

  @Override
  public Result updateSpaceState(YouthSpaceStateReqDTO bo) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();

    if (ObjectUtil.isEmpty(bo)) {
      throw new BusinessException("参数对象必传");
    }
    if (ObjectUtil.isEmpty(bo.getId()) || ObjectUtil.isEmpty(bo.getState())) {
      throw new BusinessException("必传参数为空!");
    }
    Optional<QczdSpaceContent> contentOptional = qczdSpaceContentRepo.findById(
        Long.valueOf(bo.getId()));
    if (!contentOptional.isPresent()) {
      throw new BusinessException("通过空间id:" + bo.getId() + "为查询到相关数据!");
    }
    QczdSpaceContent spaceContent = contentOptional.get();
    // 查看用户的权限
    Map<String, YouthSpaceRoleEnum> workerPermissonMap = getCurrentWorkerIncludeRole(
        currWorker);
/*    // 青创空间管理员：发布本空间的展示信息。
    if (ObjectUtil.isEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      throw new BusinessException("只有" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "才有此空间发布与下架的权限.");
    }

    if (ObjectUtil.notEqual(spaceContent.getCreateId(),currWorker.getLoginId())){
      throw new BusinessException("此" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "不是该空间的创建人,没有发布与下架权限!");
    }*/

    if (ObjectUtil.isEmpty(workerPermissonMap)){
      throw new BusinessException("无此空间发布与下架的权限.");
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))){
      // 不处理
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      if (ObjectUtil.notEqual(spaceContent.getCreateId(),currWorker.getLoginId())){
        throw new BusinessException("此" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "不是该空间的创建人,没有发布与下架权限!");
      }
    }

    spaceContent.setState(bo.getState());
    spaceContent.setUpdatedBy(currWorker.getWorkerName());
    spaceContent.setUpdateId(currWorker.getLoginId());
    spaceContent.setUpdatedTime(LocalDateTime.now());
    qczdSpaceContentRepo.save(spaceContent);
    return Result.ofSuccessMsg("状态修改成功!");
  }

  @Override
  public Result deleteSpaceById(Long id) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();

    Optional<QczdSpaceContent> contentOptional = qczdSpaceContentRepo.findById(
        Long.valueOf(id));
    if (!contentOptional.isPresent()) {
      throw new BusinessException("通过空间id:" + id + "未查询到相关数据!");
    }

    QczdSpaceContent spaceContent = contentOptional.get();
    // 查看用户的权限
    Map<String, YouthSpaceRoleEnum> workerPermissonMap = getCurrentWorkerIncludeRole(
        currWorker);
/*    // 青创空间管理员：发布本空间的展示信息。
    if (ObjectUtil.isEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      return Result.ofFail("只有" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "才有删除空间的权限.");
    }

    if (ObjectUtil.notEqual(spaceContent.getCreateId(),currWorker.getLoginId())){
      return Result.ofFail("此" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "不是该空间的创建人,没有删除此空间的权限!");
    }*/

    if (ObjectUtil.isEmpty(workerPermissonMap)){
      throw new BusinessException("没有删除此空间的权限.");
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode()))){
      // 不处理
    }else if (ObjectUtil.isNotEmpty(workerPermissonMap.get(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode()))){
      if (ObjectUtil.notEqual(spaceContent.getCreateId(),currWorker.getLoginId())){
        throw new BusinessException("此" + YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc() + "不是该空间的创建人,没有删除此空间的权限!");
      }
    }

    // 逻辑删除
    qczdSpaceContentRepo.updateDelFlagById(currWorker.getLoginId(), currWorker.getWorkerName(), id);
    // 关联删除 入住团队关联表
    qczdSpaceTeamRepo.updateDelFlagBySpaceId(currWorker.getLoginId(), currWorker.getWorkerName(),
        id);
    // 侧重领域
    qczdFocusOnAreasRepo.deleteBySpaceId(id);
    // 共享配置
    qczdSharingSettingsRepo.deleteBySpaceId(id);
    // 单价对象
    qczdPriceObjectRepo.deleteBySpaceId(id);
    return Result.ofSuccessMsg("删除成功!");
  }

  /***
   * @Description: 将单价对象转换为单价DTO对象
   * @Author: thw
   * @Date: 2023/12/12 16:26
   * @Params:
   * @param priceObject 青创空间单价对象
   * @return: QczdPriceObjectDTO
   **/
  private static QczdPriceObjectDTO covertPriceObjectDTO(QczdPriceObject priceObject) {
    QczdPriceObjectDTO priceObjectDTO = new QczdPriceObjectDTO();
    priceObjectDTO.setPriceType(priceObject.getPriceType());
    priceObjectDTO.setPriceMin(priceObject.getPriceMin().toString());
    priceObjectDTO.setPriceMax(priceObject.getPriceMax().toString());
    priceObjectDTO.setIncludeState(priceObject.getIncludeState());
    priceObjectDTO.setIncludeContent(priceObject.getIncludeContent());
    if (ObjectUtil.isNotEmpty(priceObject.getIncludeContent())) {
      String[] split = priceObject.getIncludeContent().split(",");
      priceObjectDTO.setIncludeContentList(Arrays.asList(split));
    }
    return priceObjectDTO;
  }

  /***
   * @Description: 空间层级关联文件和宣传展示中上传的图片与视频
   * @Author: thw
   * @Date: 2023/12/13 11:23
   * @Params:
   * @param bo                  空间新增DTo
   * @param spaceId             空间id
   * @param currWorker          当前用户
   * @return: void
   **/
  private void doFileRelationInsert(YouthSpacCReqDTO bo, Long spaceId,
      Worker currWorker, Boolean flag) {
    if (!flag) {
      // 将之前的关联文件（逻辑删除）
      qczdRelationFileRepo.updateByServiceId(spaceId);
    }
    List<QczdRelationFile> files = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(bo.getHierarchyFile())) {
      files.addAll(saveSpaceFile(bo.getHierarchyFile(), spaceId, FileTypeEnum.THIRD_TYPE.getNum(),
          currWorker));
    }
    if (ObjectUtil.isNotEmpty(bo.getImageFiles())) {
      files.addAll(saveSpaceFile(bo.getImageFiles(), spaceId, FileTypeEnum.ONCE_TYPE.getNum(),
          currWorker));
    }
    if (ObjectUtil.isNotEmpty(bo.getVideoFiles())){
     files.addAll(saveSpaceFile(bo.getVideoFiles(), spaceId,FileTypeEnum.SECOND_TYPE.getNum(),
            currWorker));
    }
    qczdRelationFileRepo.saveAll(files);
  }

  /****
   * @Description: 忽略属性（新增、修改一致),并为特定属性赋值
   * @Author: thw
   * @Date: 2023/12/13 11:21
   * @Params:
   * @param ignoreProperties  忽略属性集合
   * @param bo                新增对象
   * @param qczdSpaceContent  空间对象
   * @param flag              是否为新增true、修改false
   * @return: void
   **/
  private static void ignorePropertiesList(List<String> ignoreProperties, YouthSpacCReqDTO bo,
      QczdSpaceContent qczdSpaceContent, Worker currWorker,Boolean flag) {
    ignoreProperties.add("id");
    ignoreProperties.add("sharingSettings");
    ignoreProperties.add("focusOnAreas");
    ignoreProperties.add("hierarchyFile");
    ignoreProperties.add("imageFiles");
    ignoreProperties.add("videoFiles");
    ignoreProperties.add("longitude");
    ignoreProperties.add("dimension");
    ignoreProperties.add("totalArea");
    ignoreProperties.add("applyArea");
    BeanUtils.copyProperties(bo, qczdSpaceContent);
//    qczdSpaceContent.setHierarchyFileId(Long.valueOf(bo.getHierarchyFile()));
    if (ObjectUtil.isNotEmpty(bo.getLongitude())){
      qczdSpaceContent.setLongitude(new BigDecimal(bo.getLongitude()));
    }else {
      qczdSpaceContent.setLongitude(null);
    }

    if(ObjectUtil.isNotEmpty(bo.getDimension())){
      qczdSpaceContent.setDimension(new BigDecimal(bo.getDimension()));
    }else {
      qczdSpaceContent.setDimension(null);
    }

    if (ObjectUtil.isNotEmpty(bo.getTotalArea())){
      qczdSpaceContent.setTotalArea(new BigDecimal(bo.getTotalArea()));
    }else {
      qczdSpaceContent.setTotalArea(null);
    }

    if (ObjectUtil.isNotEmpty(bo.getApplyArea())){
      qczdSpaceContent.setApplyArea(new BigDecimal(bo.getApplyArea()));
    }else {
      qczdSpaceContent.setApplyArea(null);
    }


    // 发布青创空间时，级别选择[市级]或[国家级]时，需要进行审核，数据为待审核状态；
    // 级别选择[无]时，不需要进行审核，数据为已公布状态
    if (HierarchyLabelEnum.NO_LABEL.getName().equals(bo.getHierarchyLabel())) {
      qczdSpaceContent.setEvaluateState(EvaluateStateEnum.PASS_AUDIT.getNum());
      qczdSpaceContent.setState(StateEnum.UP_STATE.getCode());
    } else {
      qczdSpaceContent.setEvaluateState(EvaluateStateEnum.STAY_AUDIT.getNum());
      qczdSpaceContent.setState(StateEnum.DOWN_STATE.getCode());
    }
    // 设置默认值
    if (flag){
      qczdSpaceContent.setCreateId(currWorker.getLoginId());
      qczdSpaceContent.setCreatedBy(currWorker.getWorkerName());
      qczdSpaceContent.setCreatedTime(LocalDateTime.now());
      qczdSpaceContent.setCreatePoint(0);
    }
    qczdSpaceContent.setUpdateId(currWorker.getLoginId());
    qczdSpaceContent.setUpdatedBy(currWorker.getWorkerName());
    qczdSpaceContent.setUpdatedTime(LocalDateTime.now());
    qczdSpaceContent.setDelFlag(0);
  }

  /***
   * @Description: 空间的侧重领域关联
   * @Author: thw
   * @Date: 2023/12/13 11:08
   * @Params:
   * @param focusList 侧重领域value集合
   * @param spaceId   空间id
   * @param flag    · 是否为修改(ture:新增;false:修改)
   * @return: void
   **/
  private void doFocusOnAreas(Collection<String> focusList, Long spaceId, Boolean flag) {
    if (!flag) {
      qczdFocusOnAreasRepo.deleteBySpaceId(spaceId);
    }
    if (ObjectUtil.isNotEmpty(focusList)) {
      List<QczdFocusOnAreas> focus = new ArrayList<>();
      QczdFocusOnAreas qczdFocusOnAreas;
      for (String focusOnArea : focusList) {
        qczdFocusOnAreas = new QczdFocusOnAreas();
        qczdFocusOnAreas.setSpaceId(spaceId);
        qczdFocusOnAreas.setDelFlag(0);
        qczdFocusOnAreas.setFieldName(focusOnArea);
        focus.add(qczdFocusOnAreas);
      }
      qczdFocusOnAreasRepo.saveAll(focus);
    }
  }

  /***
   * @Description: 空间的共享配置关联
   * @Author: thw
   * @Date: 2023/12/13 10:58
   * @Params:
   * @param sharingsettings 共享配置value集合
   * @param spaceId         空间id
   * @param flag            是否为修改(ture:新增;false:修改)
   * @return: void
   **/
  private void doSharingSettings(Collection<String> sharingsettings, Long spaceId, Boolean flag) {
    if (!flag) {
      qczdSharingSettingsRepo.deleteBySpaceId(spaceId);
    }
    if (ObjectUtil.isNotEmpty(sharingsettings)) {
      List<QczdSharingSettings> settings = new ArrayList<>();
      QczdSharingSettings sharingSetting;
      for (String setting : sharingsettings) {
        sharingSetting = new QczdSharingSettings();
        sharingSetting.setSpaceId(spaceId);
        sharingSetting.setDelFlag(0);
        sharingSetting.setConfigName(setting);
        settings.add(sharingSetting);
      }
      qczdSharingSettingsRepo.saveAll(settings);
    }
  }

  /**
   * 新增参数校验
   */
  public void checkParameterLimit(YouthSpacCReqDTO bo) {
    if (ObjectUtil.isEmpty(bo)) {
      throw new BusinessException("参数对象必传!");
    }
    if (ObjectUtil.isEmpty(bo.getSpaceName()) || bo.getSpaceName().length() > 30) {
      throw new BusinessException("青创空间名称必填且不允许超过30个字符.");
    }
    if (ObjectUtil.isEmpty(bo.getContactsName()) || bo.getContactsName().length() > 10) {
      throw new BusinessException("联系人必填且不允许超过10个字符.");
    }
    if (ObjectUtil.isEmpty(bo.getPhone()) || !(PhoneUtil.isMobile(bo.getPhone()) || PhoneUtil.isTel(bo.getPhone()))) {
      throw new BusinessException("座机号|手机号码必填且要符合格式.");
    }
    if (ObjectUtil.isNotEmpty(bo.getContactMail()) && !validateEmail(bo.getContactMail())) {
      throw new BusinessException("邮箱格式不正确.");
    }
    if (ObjectUtil.isNotEmpty(bo.getPreferentialPolicySign()) && CommEnum.ONE_COM.getNum()
        .equals(bo.getPreferentialPolicySign())
        && (ObjectUtil.isEmpty(bo.getPreferentialPolicy())
        || bo.getPreferentialPolicy().length() > 200)) {
      throw new BusinessException("优惠政策不超过200个字符.");
    }
    if (ObjectUtil.isNotEmpty(bo.getSettledRequiresSign()) && CommEnum.ONE_COM.getNum()
        .equals(bo.getSettledRequiresSign())
        && (ObjectUtil.isEmpty(bo.getSettledRequires())
        || bo.getSettledRequires().length() > 200)) {
      throw new BusinessException("入驻要求不超过200个字符.");
    }
    if (ObjectUtil.isNotEmpty(bo.getHierarchyLabel()) && !HierarchyLabelEnum.NO_LABEL.getName()
        .equals(bo.getHierarchyLabel())
        && ObjectUtil.isEmpty(bo.getHierarchyFile())) {
      throw new BusinessException("层级标签为:无,佐证材料为必填.");
    }
    // 总工位相关判断
    if (ObjectUtil.isNotEmpty(bo.getTotalWorkers()) && (ObjectUtil.isEmpty(bo.getApplyWorkers())
        || ObjectUtil.isEmpty(bo.getWorkerPrice()))) {
      throw new BusinessException("总工位数填写时,可申请工位数与工位单价必填.");
    }
    if (ObjectUtil.isNotEmpty(bo.getTotalWorkers()) && (bo.getTotalWorkers() < 0
        || bo.getApplyWorkers() < 0 || bo.getTotalWorkers() < bo.getApplyWorkers())) {
      throw new BusinessException(
          "总工位数与可申请工位限制输入非负数且可申请工位数不可大于总工位数.");
    }
    // 总面积数相关判断
    if (ObjectUtil.isNotEmpty(bo.getTotalArea()) && (ObjectUtil.isEmpty(bo.getApplyArea())
        || ObjectUtil.isEmpty(bo.getAreaPrice()))) {
      throw new BusinessException("总面积数填写时,可申请面积数与面积单价必填.");
    }
    if (ObjectUtil.isNotEmpty(bo.getTotalArea()) && (Double.valueOf(bo.getTotalArea()) < 0
        || Double.valueOf(bo.getApplyArea()) < 0
        || Double.valueOf(bo.getTotalArea()) < Double.valueOf(bo.getApplyArea()))) {
      throw new BusinessException(
          "总面积数与可申请面积数限制输入非负数且申请面积数不可大于总面积数");
    }
    if (ObjectUtil.isEmpty(bo.getIntroduction()) || bo.getIntroduction().length() > 1000) {
      throw new BusinessException("简介必填且不超过1000个字符.");
    }


  }

  /**
   * 校验是否为邮箱
   */
  public static boolean validateEmail(String email) {
    String EMAIL_PATTERN = "^\\w+([-+.]\\w+)*@\\w+([-.]\\w+)*\\.\\w+([-.]\\w+)*$";
    Pattern pattern = Pattern.compile(EMAIL_PATTERN);
    Matcher matcher = pattern.matcher(email);
    return matcher.matches();
  }

  /***
   * @Description:
   * @Author: thw
   * @Date: 2023/12/13 10:42
   * @Params:
   * @param fileDTOS    文件对象集合
   * @param serviceId   业务表id
   * @param type        文件类型
   * @param currWorker  当前用户
   * @return: java.util.List<com.crunii.micro.domain.entity.main.QczdRelationFile>
   **/
  public List<QczdRelationFile> saveSpaceFile(List<YouthSpaceFileDTO> fileDTOS, Long serviceId,
      Integer type, Worker currWorker) {
    if (ObjectUtil.isNotEmpty(fileDTOS)) {
      List<QczdRelationFile> files = new ArrayList<>();
      QczdRelationFile relationFile;
      for (YouthSpaceFileDTO fileDTO : fileDTOS) {
        relationFile = new QczdRelationFile();
        relationFile.setFileId(fileDTO.getFileId());
        relationFile.setFileName(fileDTO.getFileName());
        relationFile.setFileSuffix(fileDTO.getFileType());
        relationFile.setFileType(type);
        relationFile.setFileUrl(fileDTO.getFileUrl());
        relationFile.setCreateId(currWorker.getLoginId());
        relationFile.setCreatedBy(currWorker.getWorkerName());
        relationFile.setCreatedTime(LocalDateTime.now());
        relationFile.setDelFlag(0);
        // 设置业务id
        relationFile.setServiceId(serviceId);
        files.add(relationFile);
      }
      return files;
    }
    return Lists.newArrayList();
  }


  /***
   * @Description: 判断当前用户是否包含指定的角色
   * @Author: thw
   * @Date: 2023/12/23 10:35
   * @Params:
   * @return: java.lang.String
   **/
  public Map<String, YouthSpaceRoleEnum> getCurrentWorkerIncludeRole(Worker worker){
    // 将所有角色遍历，得到需要的角色
    Map<String, YouthSpaceRoleEnum> youthSpaceRoleEnumHashMap = new HashMap<>();
    if (ObjectUtil.isNotEmpty(worker)){
      List<TsmRole> tsmRoles = null;
      // 查询用户关联的角色id
      List<TsmWorkerRole> workerRoles = tsmWorkerRoleRepo.findByLoginId(Long.valueOf(worker.getLoginId()));
      if (ObjectUtil.isNotEmpty(workerRoles)){
        List<Integer> roleIds = workerRoles.stream().distinct().map(TsmWorkerRole::getRoleId)
            .collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(roleIds)){
          tsmRoles = tsmRoleRepo.findAllById(roleIds);
        }
      }
      if (ObjectUtil.isNotEmpty(tsmRoles)){
        for (TsmRole role : tsmRoles) {
          if (YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getDesc().equals(role.getRoleName())) {
            youthSpaceRoleEnumHashMap.put(YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER.getCode(),YouthSpaceRoleEnum.YOUTH_SERVICE_CENTRE_MANAGER);
          }
          if (YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getDesc().equals(role.getRoleName())) {
            youthSpaceRoleEnumHashMap.put(YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER.getCode(),YouthSpaceRoleEnum.YOUTH_SPACE_MANAGER);
          }
        }
      }
    }
    return youthSpaceRoleEnumHashMap;
  }


  /***
   * @Description: 得到当前用户的所有权限组织
   * @Author: thw
   * @Date: 2023/12/25 0:40
   * @Params:
   * @param worker
   * @return: java.util.List<java.lang.Long>
   **/
  public List<Long> getOrgList(Worker worker){
    List<Long> allOrgId = tsmWorkerOrgAllRepo.findAllOrgIdByWorkerId(Long.valueOf(worker.getLoginId()));
    allOrgId.add(Long.valueOf(worker.getOrgId()));
    return allOrgId;
  }

}
