package com.crunii.micro.service.appsecond.utils;

import com.crunii.micro.common.exception.BusinessException;
import org.apache.poi.ss.usermodel.*;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import static cn.hutool.core.util.ClassLoaderUtil.getClassLoader;

/**
 * @author lmlksy
 * @date 2023/11/24 14:29
 */
public class EasyExcelUtils {
    public static final String EXCEL_FILE_XLSX = ".xlsx";
    public static final String EXCEL_FILE_XLS = ".xls";

    public static Boolean isExcelFile(MultipartFile multipartFile) {
        return multipartFile != null && !multipartFile.isEmpty() && (
                multipartFile.getOriginalFilename().indexOf(EXCEL_FILE_XLSX) > 0
                        || multipartFile.getOriginalFilename().indexOf(EXCEL_FILE_XLS) > 0);
    }

    public static ResponseEntity<InputStreamResource> downloadExcelDemo(String filePath, String newName)
            throws IOException {
        InputStream is = new ClassPathResource(filePath).getInputStream();
        HttpHeaders headers = new HttpHeaders();
        headers.add("Cache-Control", "no-cache, no-store, must-revalidate");
        headers.add("Content-Disposition",
                "attachment; filename=" + URLEncoder.encode(newName,"UTF-8") + ".xlsx");
        headers.add("Pragma", "no-cache");
        headers.add("Expires", "0");
        return ResponseEntity.ok().headers(headers).contentType(MediaType.parseMediaType("application/octet-stream"))
                .body(new InputStreamResource(is));
    }

    public static boolean validateExcelHeaders(String templateFile, MultipartFile file) {
        InputStream resourceAsStream = getClassLoader().getResourceAsStream(templateFile);

        List<String> templateHeaders = readHeadersFromInputStream(resourceAsStream);
        List<String> uploadedHeaders = null;
        try {
            uploadedHeaders = readHeadersFromInputStream(file.getInputStream());
        } catch (IOException e) {
            throw new BusinessException("文件处理时出错!");
        }
        return compareHeaders(templateHeaders, uploadedHeaders);
    }

    private static List<String> readHeadersFromInputStream(InputStream inputStream) {
        List<String> headers = new ArrayList<>();
        try {
            Workbook workbook = WorkbookFactory.create(inputStream);
            Sheet sheet = workbook.getSheetAt(0);
            Row headerRow = sheet.getRow(0);
            for (Cell cell : headerRow) {
                headers.add(cell.getStringCellValue());
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("校验失败,请使用导入模板!");
        }
        return headers;
    }

    private static boolean compareHeaders(List<String> templateHeaders, List<String> uploadedHeaders) {
        boolean equals = templateHeaders.equals(uploadedHeaders);
        if (!equals) {
            throw new BusinessException("上传的文件列标题与下载模板的列标题不匹配!");
        }
        return true;
    }
}
