package com.crunii.micro.service.app.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.sys.PicInfoVo;
import com.crunii.micro.service.app.dto.HeadlineArticleVo;
import com.crunii.micro.service.app.dto.MatchActivityVo;
import com.crunii.micro.service.app.dto.PageDto;
import com.crunii.micro.service.app.dto.StudyPlacardVo;
import com.crunii.micro.service.app.service.HomeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

import static com.crunii.micro.service.app.constants.AppConstants.CHILDREN_PALACE;
import static com.crunii.micro.service.app.constants.AppConstants.YOUTH_BIG_LEARNING;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/25 19:40
 */
@Api(tags = "app首页相关")
@RestController
@Slf4j
@RequestMapping("/auth/HomeController")
public class HomeController {

    @Autowired
    private HomeService homeService;

    @ApiOperation(value = "青年大学习课程")
    @PostMapping(path = "/bigLearnList")
    public Result<List<StudyPlacardVo>> bigLearnList() {
        return homeService.queryHotPlacard(YOUTH_BIG_LEARNING);
    }

    @ApiOperation(value = "少年宫热门培训")
    @PostMapping(path = "/juvenilePalaceList")
    public Result<List<StudyPlacardVo>> juvenilePalaceList() {
        return homeService.queryHotPlacard(CHILDREN_PALACE);
    }

    @ApiOperation(value = "志愿活动总人数")
    @PostMapping(path = "/activityNum")
    public Result<Object> activityNum() {
        return homeService.activityNum();
    }

    @ApiOperation(value = "志愿活动列表")
    @PostMapping(path = "/activityList")
    public Result<Object> activityList(@RequestBody @Validated PageDto pageDto) {
        return homeService.activityList(pageDto);
    }

    @ApiOperation(value = "图绘青春-热度排行")
    @PostMapping(path = "/hotRankingList")
    public Result<PageRsp<PicInfoVo>> hotRankingList(@RequestBody PageReq<String> req) {
        return homeService.hotRankingList(req);
    }

    // 2023-07-07 baoxy添加接口
    @ApiOperation(value = "赛事活动")
    @PostMapping(path = "matchActivity")
    public Result<MatchActivityVo> matchActivity() {
        return homeService.matchActivity();
    }

    @ApiOperation(value = "青春头条top")
    @PostMapping(path = "/headlineArticleTop")
    public Result<List<HeadlineArticleVo>> headlineArticleTop() {
        return Result.ofSuccess(homeService.headlineArticleTop());
    }
}
