package com.crunii.micro.service.app.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.app.dto.*;
import com.crunii.micro.service.app.service.HeadlineArticleService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author wangchen create 2023/8/14 9:23
 */
@Api(tags = "青春头条")
@RestController
@Slf4j
@RequestMapping("/public/HeadlineArticleController")
public class PublicHeadlineArticleController {

    private final HeadlineArticleService headlineArticleService;

    public PublicHeadlineArticleController(HeadlineArticleService headlineArticleService) {
        this.headlineArticleService = headlineArticleService;
    }

    @ApiOperation("分页查询文章")
    @PostMapping("/pageArticle")
    public Result<PageRsp<HeadlineArticleVo>> pageArticle(@RequestBody PageReq<HeadlineArticleReqDto> pageReq){
        return Result.ofSuccess(headlineArticleService.pageArticle(pageReq));
    }

    @ApiOperation("查询文章详情")
    @GetMapping("/getArticleDetail")
    public Result<ArticleDetailVo> getArticleDetail(@RequestParam("headlineId") @NotNull(message = "文章id不能为空") Long headlineId){
        return Result.ofSuccess(headlineArticleService.getArticleDetail(headlineId));
    }

    @ApiOperation("点赞文章")
    @GetMapping("/modifyArticleLike")
    public Result<String> modifyArticleLike(@RequestParam("headlineId") @NotNull(message = "文章id不能为空") Long headlineId, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(headlineArticleService.modifyArticleLike(headlineId,worker));
    }

    @ApiOperation("当前文章基本信息")
    @GetMapping("/getArticleCount")
    public Result<ArticleCountVo> getArticleCount(@RequestParam("headlineId") @NotNull(message = "文章id不能为空") Long headlineId, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(headlineArticleService.getArticleCount(headlineId,worker));
    }
    
    @ApiOperation("查询标签")
    @PostMapping("/findLabel")
    public Result<List<HeadlineArticleLabelVo>> findLabel(){
        return Result.ofSuccess(headlineArticleService.findLabel());
    }
}
