package com.crunii.micro.service.app.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.app.dto.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/16 15:28
 */
@Slf4j
@Repository
public class FileJdbc {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNPJdbcTemplate;

    private static final String pageValueBaseSql = "SELECT a.upload_id AS uploadId,a.file_title AS fileTitle,b.file_key AS fileKey FROM YCQ_UPLOAD_INFO a LEFT JOIN YCQ_FILE_RECORD B ON a.upload_id=b.upload_id WHERE 1=1 AND b.is_pic=1 AND b.sort=0 AND a.audit_status = 1";

    private static final String pageLabelValueBaseSql = "SELECT a.upload_id AS uploadId,a.file_title AS fileTitle,b.file_key AS fileKey FROM YCQ_UPLOAD_INFO a LEFT JOIN YCQ_FILE_RECORD B ON a.upload_id=b.upload_id LEFT JOIN YCQ_FILE_LABEL_RELATION C ON a.upload_id=c.file_id WHERE 1=1 AND b.is_pic=1 AND b.sort=0 AND a.audit_status=1";

    private static final String pageValueCountSql = "SELECT count(*) FROM YCQ_UPLOAD_INFO a LEFT JOIN YCQ_FILE_RECORD B ON a.upload_id=b.upload_id WHERE 1=1 AND b.is_pic=1 AND b.sort=0 AND a.audit_status = 1";

    private static final String pageLabelValueCountSql = "SELECT count(*) FROM YCQ_UPLOAD_INFO a LEFT JOIN YCQ_FILE_RECORD B ON a.upload_id=b.upload_id LEFT JOIN YCQ_FILE_LABEL_RELATION C ON a.upload_id=c.file_id WHERE 1=1 AND b.is_pic=1 AND b.sort=0 AND a.audit_status=1";

    public PageRsp<ImagesVo> queryList(PageReq<LabelDto> req) {
        StringBuilder sb = new StringBuilder(pageValueBaseSql);
        StringBuilder sbCount = new StringBuilder(pageValueCountSql);
        List<Object> params = new ArrayList<>();

        if(StringUtils.hasText(req.getQuery().getSearchText())){
            sb.append(" and a.file_title like concat('%',?,'%') ");
            sbCount.append(" and a.file_title like concat('%',?,'%') ");
            params.add(req.getQuery().getSearchText());
        }

        List<ImagesVo> values;
        //添加逻辑删除判断
        sbCount.append(" and a.state =1 ");
        int total = JT.queryForInt(sbCount.toString(),params.toArray());
        String dbType = JT.getDbType();
        if (total > 0) {
            //添加逻辑删除判断
            sb.append(" and a.state =1 ");
            //添加orderBy
            sb.append(" ORDER BY a.push_time desc ");
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = JT.queryForBeans(ImagesVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<ImagesVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public PageRsp<ImagesVo> queryForLabelList(PageReq<LabelDto> req) {
        StringBuilder sb = new StringBuilder(pageLabelValueBaseSql);
        StringBuilder sbCount = new StringBuilder(pageLabelValueCountSql);
        List<Object> params = new ArrayList<>();
        if (Objects.nonNull(req.getQuery().getLabelId())) {
            sb.append(" and c.label_id = ?");
            sbCount.append(" and c.label_id = ?");
            params.add(req.getQuery().getLabelId());
        }
        if(StringUtils.hasText(req.getQuery().getSearchText())){
            sb.append(" and a.file_title like concat('%',?,'%') ");
            sbCount.append(" and a.file_title like concat('%',?,'%') ");
            params.add(req.getQuery().getSearchText());
        }
        List<ImagesVo> values;
        //添加逻辑删除判断
        sbCount.append(" and a.state =1 ");
        int total = JT.queryForInt(sbCount.toString(), params.toArray());
        String dbType = JT.getDbType();
        if (total > 0) {
            //添加逻辑删除判断
            sb.append(" and a.state =1 ");
            //添加orderBy
            sb.append(" ORDER BY a.push_time desc ");
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = JT.queryForBeans(ImagesVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<ImagesVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    /**
     * 通过文件ID查询标签
     *
     * @param fileId
     * @return
     */
    public List<String> queryLabelByFileId(Long fileId) {
        String sql = "SELECT label_name FROM YCQ_FILE_LABEL_RELATION WHERE file_id = ?";
        return JT.queryForRawObjects(String.class, sql, fileId);
    }

    public List<ImageInfoDto> getImageInfo(Long uploadId) {
        String sql = "SELECT a.upload_id AS uploadId,a.file_title AS fileTitle,a.upload_worker_name AS NAME,b.file_key AS fileKey,a.\n" +
                "DESCRIBE,a.number AS number,a.required_integral AS requiredIntegral ,a.is_album AS isAlbum FROM YCQ_UPLOAD_INFO a LEFT JOIN YCQ_FILE_RECORD B ON a.upload_id=b.upload_id WHERE 1=1 AND b.is_pic=1 AND a.audit_status=1 AND a.state=1 AND a.upload_id=? ORDER BY b.sort";
        return JT.queryForBeans(ImageInfoDto.class, sql, uploadId);
    }

    public List<AppHotSearchVo> queryHotSearch() {
        String sql = "SELECT upload_id AS uploadId,file_title AS fileTitle FROM YCQ_UPLOAD_INFO WHERE state = 1 AND audit_status = 1 ORDER BY number DESC LIMIT 10";
        return JT.queryForBeans(AppHotSearchVo.class, sql);
    }

    public List<LabelUpload> findLabelIdAndName(List<Long> uploadIds) {
        String sql = "SELECT file_id uploadId,label_id,label_name FROM YCQ_FILE_LABEL_RELATION WHERE file_id in(:ids)";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", uploadIds);
        return baseNPJdbcTemplate.queryForBeans(LabelUpload.class, sql, params);
    }

    public List<ImagesVo> queryFileInfo(List<String> picList) {
        String sql = "SELECT object_key as fileKey, width,height FROM YCQ_OBS_FILE WHERE object_key in(:ids)";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", picList);
        return baseNPJdbcTemplate.queryForBeans(ImagesVo.class, sql, params);
    }
}
