package com.crunii.micro.service.app.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.app.dto.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wangchen create 2023/8/14 9:24
 */
@Repository
public class HeadlineArticleDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNPJdbcTemplate;

    public PageRsp<HeadlineArticleVo> pageArticle(PageReq<HeadlineArticleReqDto> pageReq) {
        HeadlineArticleReqDto query = pageReq.getQuery();
        StringBuilder sb = new StringBuilder(" SELECT  \n"
            + " a.headline_id,a.headline_title,a.cover_pic,a.article_source,a.release_time,a.article_abstract,a.is_top,a.update_time,b.column_two_id label_id,b.column_two_name label_name\n"
            + " FROM YCQ_HEADLINE_ARTICLE a \n"
            + " LEFT JOIN YCQ_HEADLINE_ARTICLE_COLUMN b ON a.headline_id = b.headline_id AND b.deleted = '0' \n"
            + " WHERE a.state = 1 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(*) FROM YCQ_HEADLINE_ARTICLE a \n"
                + " LEFT JOIN YCQ_HEADLINE_ARTICLE_COLUMN b ON a.headline_id = b.headline_id AND b.deleted = '0' \n"
                + " WHERE a.state = 1 ");
        List<Object> params = new ArrayList<>();
        if (StringUtils.isNotEmpty(query.getLabelName())) {
            sb.append(" AND b.column_two_name = ? ");
            sbCount.append(" AND b.column_two_name = ? ");
            params.add(query.getLabelName());
        }
        sb.append(" ORDER BY a.is_top DESC,COALESCE(a.update_time,a.release_time) DESC ");
        return jdbcTemplate.queryPageBase(pageReq, sb.toString(), sbCount.toString(), HeadlineArticleVo.class, params);
    }

    public Integer count(){
        String sql = "SELECT count(1) FROM  YCQ_HEADLINE_ARTICLE WHERE state = 1";
        return jdbcTemplate.queryForInt(sql);
    }
    public Boolean queryWorkerIsLike(int loginId,Long headlineId) {
        String sql = "SELECT count(1) FROM YCQ_HEADLINE_ARTICLE_WORKER WHERE worker_id = ? and headline_id = ?";
        return jdbcTemplate.queryForRawObject(Boolean.class,sql,loginId,headlineId);
    }

    public ArticleCountVo findArticleCount(Long headlineId) {
        String sql = "SELECT likes_count,reading_quantity FROM YCQ_HEADLINE_ARTICLE  WHERE headline_id = ? AND state = 1";
        return jdbcTemplate.queryForBean(ArticleCountVo.class,sql,headlineId);
    }

    public int pinningArticle(Long id) {
        String sql = "UPDATE YCQ_HEADLINE_ARTICLE  SET UPDATE_TIME = ?,is_top = 1 WHERE headline_id = ?";
        return jdbcTemplate.update(sql, new Date(), id);
    }

    public int cancelPinningArticle(Long id) {
        String sql = "UPDATE YCQ_HEADLINE_ARTICLE  SET is_top = 0 WHERE headline_id = ?";
        return jdbcTemplate.update(sql, id);
    }

    public List<HeadlineArticleVo> sortList() {
        String sql = "SELECT headline_id,headline_title,cover_pic,article_source,release_time,article_abstract,is_top,update_time FROM  YCQ_HEADLINE_ARTICLE WHERE state = 1";
        List<HeadlineArticleVo> headlineArticleVos = jdbcTemplate.queryForBeans(HeadlineArticleVo.class, sql);
        // 排序
        return headlineArticleVos.stream().sorted(Comparator.comparing(HeadlineArticleVo::getIsTop).reversed()
                        .thenComparing(item -> Objects.equals("1", item.getIsTop()) ? item.getUpdateTime() : item.getReleaseTime(), Comparator.reverseOrder()))
                .collect(Collectors.toList());
    }
    
    /**
     * 查询标签
     * @author leixb
     * @since 2024-01-19 09:28
     * @return List<HeadlineArticleLabelVo>
     */
    public List<HeadlineArticleLabelVo> findLabel() {
        String sql = " SELECT DISTINCT column_two_id label_id, column_two_name label_name FROM YCQ_HEADLINE_ARTICLE_COLUMN WHERE deleted = '0'";
        return jdbcTemplate.queryForBeans(HeadlineArticleLabelVo.class, sql);
    }
    
}
