package com.crunii.micro.service.app.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.sys.FileDetailDto;
import com.crunii.micro.domain.dto.sys.PicInfoVo;
import com.crunii.micro.service.app.dto.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/25 19:58
 */
@Slf4j
@Repository
public class HomeJdbc {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public List<StudyPlacardVo> queryHotPlacard(String sysCode){
        String sql = "SELECT placard_id AS placardId , placard_url AS placardUrl ,placard_link AS placardLink FROM YCQ_STUDY_PLACARD WHERE state =1 and sys_code = ? ORDER BY sort LIMIT 6";
        return JT.queryForBeans(StudyPlacardVo.class,sql,sysCode);
    }

    public Integer activityNum(){
        String sql ="SELECT sum(service_num) FROM YCQ_CQZYSC_SERVICE_TIME WHERE  date_time = (SELECT date_time FROM YCQ_CQZYSC_SERVICE_TIME GROUP BY date_time ORDER BY date_time DESC LIMIT 1)";
        return JT.queryForRawObject(Integer.class,sql);
    }

    public MatchActivityVo matchActivity(int type, int showDue) {
        String sql = "select file_key, article_title,article_id from ycq_article_detail where article_type = ? and show_due = ? " +
                "and state = ? and article_state = ? order by publish_time desc limit 1";
        List<Object> param = new ArrayList<>();
        param.add(type);
        param.add(showDue);
        param.add(CommonConstants.STATE_VALID);
        param.add(CommonConstants.STATE_VALID);
        return JT.queryForBean(MatchActivityVo.class, sql, param.toArray());
    }

    public MatchActivityVo matchActivityByLastTime() {
        String sql = "select file_key, article_title,article_id from ycq_article_detail where state = ? and article_state = ? " +
                "order by publish_time desc limit 1";
        List<Object> param = new ArrayList<>();
        param.add(CommonConstants.STATE_VALID);
        param.add(CommonConstants.STATE_VALID);
        return JT.queryForBean(MatchActivityVo.class, sql, param.toArray());
    }

    public AppAuthVo queryAppAuth(){
        String sql ="SELECT app_code ,app_url ,must_auth FROM YCQ_APP_CONFIG_URL WHERE state =1 AND app_code = 'CQZYSC'";
        return JT.queryForBean(AppAuthVo.class,sql);
    }

    public List<HeadlineArticleVo> findHeadlineArticleTop() {
        String sql = "SELECT headline_id,headline_title,cover_pic,article_source,release_time,article_abstract FROM  YCQ_HEADLINE_ARTICLE where is_top = 1 order by push_time desc limit 3";
        return JT.queryForBeans(HeadlineArticleVo.class,sql);
    }

    public List<PicInfoVo> queryHotSearch() {
        String sql = "SELECT upload_id,required_integral,push_time,describe,is_album,file_title,upload_worker_name FROM YCQ_UPLOAD_INFO WHERE state = 1 and audit_status = 1 AND number <> 0 ORDER BY number DESC LIMIT 12";
        return JT.queryForBeans(PicInfoVo.class,sql);
    }

    public List<LabelUpload> findLabelIdAndName(List<Long> uploadIds) {
        String sql = "SELECT file_id uploadId,label_id,label_name FROM YCQ_FILE_LABEL_RELATION WHERE file_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        return baseNpJdbcTemplate.queryForBeans(LabelUpload.class,sql,params);
    }

    public List<FileDetailDto> findPicDetail(List<Long> uploadIds) {
        String sql = "SELECT upload_id,file_name,file_record_id,file_type,pic_width,pic_height,is_pic,file_size,file_key FROM YCQ_FILE_RECORD WHERE upload_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        return baseNpJdbcTemplate.queryForBeans(FileDetailDto.class,sql,params);
    }

    public List<HeadlineArticleVo> findHeadlineArticleTop(List<String> keyWord, int i) {
        StringBuilder sb = new StringBuilder("SELECT headline_id,headline_title,cover_pic,article_source,release_time,article_abstract FROM  YCQ_HEADLINE_ARTICLE where 1=1");
        List<Object> param = new ArrayList<>();
        if(CollectionUtil.isNotEmpty(keyWord)){
            StringJoiner sj = new StringJoiner("or");
            sb.append(" and ( ");
            for (String s : keyWord) {
                sj.add(" headline_title like concat('%',?,'%') ");
                param.add(s);
            }
            sb.append(sj);
            sb.append(")");
        }
        sb.append(" order by push_time desc ");
        sb.append( "limit ?");
        param.add(i);
        return JT.queryForBeans(HeadlineArticleVo.class,sb.toString(),param.toArray());
    }

    public List<HeadlineArticleVo> findHeadlineArticleIgnoreTop(int limit) {
        String sql = "SELECT headline_id,headline_title,cover_pic,article_source,release_time,article_abstract FROM  YCQ_HEADLINE_ARTICLE where 1=1 order by push_time desc limit ?";
        return JT.queryForBeans(HeadlineArticleVo.class,sql,limit);
    }
}
