package com.crunii.micro.service.app.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.sys.PicInfoVo;
import com.crunii.micro.service.app.dto.*;
import com.crunii.micro.service.app.jdbc.FileJdbc;
import com.crunii.micro.service.app.service.FileService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/16 14:40
 */
@Service
@Slf4j
public class FileServiceImpl implements FileService {

    @Value("${upload.url}")
    private String URL_PREFIX;

    @Resource
    private FileJdbc fileJdbc;

    private final RedisTemplate redisTemplate;

    public FileServiceImpl(RedisTemplate redisTemplate) {
        this.redisTemplate = redisTemplate;
    }

    @Override
    public Result<PageRsp<ImagesVo>> queryList(PageReq<LabelDto> req) {
        PageRsp<ImagesVo> images;
        Result<PageRsp<ImagesVo>> result = new Result<>();
        if (ObjectUtil.isEmpty(req.getQuery().getLabelId())) {
            // 全部
            images = fileJdbc.queryList(req);
        } else {
            // 按标签
            images = fileJdbc.queryForLabelList(req);
        }
        List<ImagesVo> imagesVos = images.getResult();
        if (CollectionUtils.isNotEmpty(imagesVos)) {
            // 查询标签
            List<Long> uploadIds = imagesVos.stream().map(ImagesVo::getUploadId).collect(Collectors.toList());
            List<LabelUpload> list = fileJdbc.findLabelIdAndName(uploadIds);
            Map<Long, List<LabelUpload>> labelCollect = new HashMap<>(16);
            if (CollectionUtils.isNotEmpty(list)) {
                labelCollect = list.stream().collect(Collectors.groupingBy(LabelUpload::getUploadId));
            }
            List<String> picList = imagesVos.stream().map(ImagesVo::getFileKey).collect(Collectors.toList());
            List<ImagesVo> picInfo = fileJdbc.queryFileInfo(picList);
            Map<String, ImagesVo> collect = picInfo.stream().collect(Collectors.toMap(ImagesVo::getFileKey, Function.identity()));
            for (ImagesVo vo : imagesVos) {
                // 查询该id对应标签设置
                List<LabelUpload> list1 = labelCollect.get(vo.getUploadId());
                List<String> labels = new ArrayList<>();
                for (LabelUpload labelUpload : list1) {
                    String labelName = labelUpload.getLabelName();
                    labels.add(labelName);
                }
                vo.setLabels(labels);
                vo.setUrl(URL_PREFIX + vo.getFileKey());
                // 查询图片长高
                ImagesVo imagesVo = collect.get(vo.getFileKey());
                if(Objects.nonNull(imagesVo)){
                    vo.setWidth(imagesVo.getWidth());
                    vo.setHeight(imagesVo.getHeight());
                    vo.setFileName(imagesVo.getFileName());
                }
            }
        }
        result.setData(images);
        return result;
    }

    @Override
    public Result<ImageInfoVo> queryImageInfo(Long uploadId) {
        ImageInfoVo imageInfoVo = new ImageInfoVo();
        List<ImageInfoDto> imageInfo = fileJdbc.getImageInfo(uploadId);
        List<String> urlList = new ArrayList<>();
        if (!CollectionUtil.isEmpty(imageInfo)) {
            BeanUtils.copyProperties(imageInfo.get(0), imageInfoVo);
            imageInfo.forEach(x -> {
                urlList.add(URL_PREFIX + x.getFileKey());
            });
        }
        imageInfoVo.setLabels(getLabels(imageInfoVo.getUploadId()));
        imageInfoVo.setPhotosUrl(urlList);
        return Result.ofSuccess(imageInfoVo);
    }

    @Override
    public Result<List<AppHotSearchVo>> queryHostSearch() {
        int k = 0;
        List<AppHotSearchVo> voList = new ArrayList<>();
        List<PicInfoVo> list = (List<PicInfoVo>) redisTemplate.opsForValue().get(RedisKeys.HOT_FILE_SEARCH);
        if (!CollectionUtil.isEmpty(list)) {
            for (PicInfoVo x : list) {
                if (k == 10) {
                    break;
                }
                AppHotSearchVo vo = new AppHotSearchVo();
                vo.setUploadId(x.getUploadId());
                vo.setFileTitle(x.getFileTitle());
                voList.add(vo);
                k++;
            }
        } else {
            voList = fileJdbc.queryHotSearch();
        }
        return Result.ofSuccess(voList);
    }

    public List<String> getLabels(Long fileId) {
        return fileJdbc.queryLabelByFileId(fileId);
    }
}
