package com.crunii.micro.service.app.service.impl;
import java.util.*;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqHeadlineArticle;
import com.crunii.micro.domain.entity.main.YcqHeadlineArticleWorker;
import com.crunii.micro.domain.repo.main.YcqHeadlineArticleRepo;
import com.crunii.micro.domain.repo.main.YcqHeadlineArticleWorkerRepo;
import com.crunii.micro.service.app.config.ParseHeadLineDetailPicProperty;
import com.crunii.micro.service.app.dto.*;
import com.crunii.micro.service.app.enums.HeadlineLabelEnum;
import com.crunii.micro.service.app.jdbc.HeadlineArticleDao;
import com.crunii.micro.service.app.service.HeadlineArticleService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Collectors;

/**
 * @author wangchen create 2023/8/14 9:24
 */
@Slf4j
@Service
public class HeadlineArticleServiceImpl implements HeadlineArticleService {


    public static final String NEW_URL_PREFIX = "https://youth.cq.cqyl.org.cn:11021/qcttpich";
    public static final String REPLACE_URL_PREFIX = "http://www.cqyl.org.cn";
    public static final String REPLACE_URL_PREFIX2 = "http://cqyl.org.cn";
    @Resource
    private YcqHeadlineArticleWorkerRepo ycqHeadlineArticleWorkerRepo;
    @Resource
    private YcqHeadlineArticleRepo ycqHeadlineArticleRepo;


    @Autowired
    private ParseHeadLineDetailPicProperty property;

    private final HeadlineArticleDao headlineArticleDao;

    public HeadlineArticleServiceImpl(HeadlineArticleDao headlineArticleDao) {
        this.headlineArticleDao = headlineArticleDao;
    }

    /*@Override
    public PageRsp<HeadlineArticleVo> pageArticle(PageReq<Object> pageReq) {
        int pageNo = pageReq.getPageNo() == 0 ? 1 : pageReq.getPageNo();
        int pageSize = pageReq.getPageSize();
        Integer count = headlineArticleDao.count();
        PageRsp<HeadlineArticleVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        List<HeadlineArticleVo> list;
        if (count == 0){
            page.setTotalNum(0);
            list = Collections.emptyList();
        }else {
            // 查询所有 内润分页
            List<HeadlineArticleVo> totalList = headlineArticleDao.sortList();
            int size = totalList.size();
            page.setTotalNum(size);
            int index = pageNo * pageSize;
            int start = index - pageSize;
            int end = index - 1;
            if (size <= start){
                list = Collections.emptyList();
            }else{
                if (start + 1 == size){
                    list = Collections.singletonList(totalList.get(start));
                } else if (end + 1 > size) {
                    end = size;
                    list = totalList.subList(start, end);
                }else {
                    list = totalList.subList(start, end + 1);
                }
            }
        }
        page.setResult(list);
        page.next();
        return page;
    }*/
    
    @Override
    public PageRsp<HeadlineArticleVo> pageArticle(PageReq<HeadlineArticleReqDto> pageReq) {
        return headlineArticleDao.pageArticle(pageReq);
    }

    @Override
    public ArticleDetailVo getArticleDetail(Long headlineId) {
        YcqHeadlineArticle one = ycqHeadlineArticleRepo.getOne(headlineId);
        if(Objects.isNull(one)){
            throw new BusinessException("文章不存在");
        }
        ArticleDetailVo articleDetail = new ArticleDetailVo();
        BeanUtils.copyProperties(one,articleDetail);
        // 更新浏览量
        Long readingQuantity = one.getReadingQuantity();
        AtomicLong atomicLong = new AtomicLong(readingQuantity);
        one.setReadingQuantity(atomicLong.incrementAndGet());
        ycqHeadlineArticleRepo.saveAndFlush(one);
        // 修改响应图片地址
        String headlineArticleContext = articleDetail.getHeadlineArticleContext();
        if (StringUtils.isNotBlank(headlineArticleContext)){
            String replace = headlineArticleContext.replace(REPLACE_URL_PREFIX, NEW_URL_PREFIX).replace(REPLACE_URL_PREFIX2, NEW_URL_PREFIX);
            articleDetail.setHeadlineArticleContext(replace);
            List<String> list = property.getConvert();
            list.forEach(item -> {
                String[] split = item.split(",");
                String articleSource = split[0];
                String oldPrefix = split[1];
                String newPrefix = split[2];
                if (Objects.equals(one.getArticleSource(), articleSource)){
                    String newUrl = articleDetail.getHeadlineArticleContext().replace(oldPrefix, newPrefix);
                    articleDetail.setHeadlineArticleContext(newUrl);
                }
            });
        }
        return articleDetail;
    }

    @Override
    public String modifyArticleLike(Long headlineId, Worker worker) {
        YcqHeadlineArticleWorker byWorkerIdAndAndHeadlineId = ycqHeadlineArticleWorkerRepo.findByWorkerIdAndAndHeadlineId(worker.getLoginId(), headlineId);
        if(Objects.isNull(byWorkerIdAndAndHeadlineId)){
            byWorkerIdAndAndHeadlineId = new YcqHeadlineArticleWorker();
            byWorkerIdAndAndHeadlineId.setHeadlineId(headlineId);
            byWorkerIdAndAndHeadlineId.setWorkerId(worker.getLoginId());
            byWorkerIdAndAndHeadlineId.setCreateTime(new Date());
            ycqHeadlineArticleWorkerRepo.saveAndFlush(byWorkerIdAndAndHeadlineId);
            modifyArticleLikeCount(headlineId,true);
        }else {
            ycqHeadlineArticleWorkerRepo.delete(byWorkerIdAndAndHeadlineId);
            ycqHeadlineArticleWorkerRepo.flush();
            modifyArticleLikeCount(headlineId,false);
        }
        return "更新成功";
    }

    @Override
    public ArticleCountVo getArticleCount(Long headlineId, Worker worker) {
        // 分享的链接如果当前浏览器未登录过，会没有token，此时worker为null
        Boolean aBoolean = false;
        if (!Objects.isNull(worker) ) {
            aBoolean = headlineArticleDao.queryWorkerIsLike(worker.getLoginId(), headlineId);
        }
        ArticleCountVo articleCountVo = headlineArticleDao.findArticleCount(headlineId);
        articleCountVo.setLike(aBoolean);
        return articleCountVo;
    }

    @Override
    public boolean pinningArticle(Long id){
        headlineArticleDao.pinningArticle(id);
        return true;
    }

    @Override
    public boolean cancelPinningArticle(Long headlineId) {
        headlineArticleDao.cancelPinningArticle(headlineId);
        return true;
    }

    /**
     * 修改文章喜欢
     */
    private void modifyArticleLikeCount(Long headlineId, boolean b) {
        YcqHeadlineArticle one = ycqHeadlineArticleRepo.getOne(headlineId);
        if(Objects.isNull(one)){
            return;
        }
        AtomicLong atomicLong = new AtomicLong(one.getLikesCount());
        // true 增加
        if(b){
            one.setLikesCount(atomicLong.incrementAndGet());
        }else {
            one.setLikesCount(atomicLong.decrementAndGet());
        }
        ycqHeadlineArticleRepo.saveAndFlush(one);
    }
    
    /**
     * 查询标签
     * @author leixb
     * @since 2024-01-19 09:28
     * @return List<HeadlineArticleLabelVo>
     */
    @Override
    public List<HeadlineArticleLabelVo> findLabel(){
        List<HeadlineArticleLabelVo> resultList = new ArrayList<>();
        for (HeadlineLabelEnum enumItem : HeadlineLabelEnum.values()) {
            HeadlineArticleLabelVo vo = new HeadlineArticleLabelVo();
            vo.setLabelCode(enumItem.getCode());
            vo.setLabelName(enumItem.getName());
            resultList.add(vo);
        }
        return resultList;
    }
}
