package com.crunii.micro.service.app.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONObject;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.core.cache.TsmPropertyCache;
import com.crunii.micro.domain.dto.sys.FileDetailDto;
import com.crunii.micro.domain.dto.sys.LabelCpDto;
import com.crunii.micro.domain.dto.sys.PicInfoVo;
import com.crunii.micro.domain.entity.main.TsmPropertyValue;
import com.crunii.micro.domain.repo.main.TsmPropertyValueRepo;
import com.crunii.micro.service.app.constants.AppConstants;
import com.crunii.micro.service.app.constants.FileConstants;
import com.crunii.micro.service.app.dto.*;
import com.crunii.micro.service.app.jdbc.FileUploadDao;
import com.crunii.micro.service.app.jdbc.HomeJdbc;
import com.crunii.micro.service.app.service.AiconsumeCacheService;
import com.crunii.micro.service.app.service.HomeService;
import com.crunii.micro.service.app.util.ListPageUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/25 19:48
 */
@Service
@Slf4j
public class HomeServiceImpl implements HomeService {

    @Resource
    private HomeJdbc jdbc;

    @Resource
    private RedisTemplate<String, List<PicInfoVo>> redisTemplate;

    @Resource
    private AiconsumeCacheService aiconsumeCacheService;


    @Value("${upload.url}")
    private String urlPrefix;

    @Value("${activity.zyscUrl}")
    private String zyscUrl;

    @Resource
    private TsmPropertyValueRepo tsmPropertyValueRepo;

    final String DISPLACE_BEFORE_URL="photo-displace-before-url";
    final String PHOTO_DISPLACE_URL="photo-displace-url";


    @Override
    public Result<List<StudyPlacardVo>> queryHotPlacard(String sysCode) {
        List<StudyPlacardVo> vo = jdbc.queryHotPlacard(sysCode);
        //被替换域名
        List<String> urls = tsmPropertyValueRepo.findAllByPropCodeAndState(DISPLACE_BEFORE_URL, 1).stream().map(TsmPropertyValue::getValueCode).collect(Collectors.toList());
        //替换域名
        List<String> afterUrl = tsmPropertyValueRepo.findAllByPropCodeAndState(PHOTO_DISPLACE_URL, 1).stream().map(TsmPropertyValue::getValueCode).collect(Collectors.toList());
        //http://resstatic.sngcloud.cn/files/636506732981490039/22b5509b4c164362b9a48d90aa0d7584.jpg
        for (StudyPlacardVo studyPlacardVo : vo) {
            for (String url : urls) {
                if (StringUtils.isNotEmpty(studyPlacardVo.getPlacardUrl()) && studyPlacardVo.getPlacardUrl().contains(url) && afterUrl.size()>0){
                    String replaceUrl = afterUrl.get(0);
                    String replace = studyPlacardVo.getPlacardUrl().replace(url, replaceUrl);
                    studyPlacardVo.setPlacardUrl(replace);
                    break;
                }
            }
        }
        return Result.ofSuccess(vo);
    }

    @Override
    public Result<Object> activityNum() {
        Integer num = jdbc.activityNum();
        if (num == null) {
            num = 0;
        }
        return Result.ofSuccess(num);
    }

    @Override
    public Result<Object> activityList(PageDto pageDto) {
        if (ObjectUtil.isEmpty(pageDto)) {
            throw new BusinessException("分页参数不能为空");
        }
        String json = JsonUtil.getJsonString(pageDto);
        String url = zyscUrl;
        List<ActivityVo> voList = new ArrayList<>();
        try {
            String result = HttpRequest.post(url)
                    .body(json)
                    .execute().body();
            JSONObject jsonObject = new JSONObject(result);
            String data = jsonObject.get("data").toString();
            JSONObject jsonObject1 = new JSONObject(data);
            String records = jsonObject1.get("records").toString();
            voList = JsonUtil.jsonToList(records, ActivityVo.class);
        } catch (Exception e) {
            log.info("调用志愿山城活动出错了,{}",e.getMessage());
            return Result.ofSuccess(voList);
        }
        AppAuthVo appAuthVo = jdbc.queryAppAuth();
        if (ObjectUtil.isEmpty(appAuthVo)) {
            log.info("志愿山城app授权为空");
            throw new BusinessException("查询失败,请联系管理员");
        }
        if (CollectionUtil.isEmpty(voList)){
            log.info("暂无志愿山城活动");
            return Result.ofSuccess(voList);
        }
        for (ActivityVo vo : voList) {
            vo.setActUrl(appAuthVo.getAppUrl()+vo.getActUrl());
            if (vo.getBegTime().after(new Date())) {
                vo.setActivityTime("活动未开始");
            } else {
                Date expireDate = new Date(vo.getEndTime().getTime());
                Date currentDate = new Date();
                long diffInMillies = expireDate.getTime() - currentDate.getTime();
                long diffInDays = TimeUnit.DAYS.convert(diffInMillies, TimeUnit.MILLISECONDS);
                long diffInHours = TimeUnit.HOURS.convert(diffInMillies - TimeUnit.DAYS.toMillis(diffInDays), TimeUnit.MILLISECONDS);
                String a = "离招募结束 %d天%d小时";
                vo.setActivityTime(String.format(a, diffInDays, diffInHours));
            }
        }
        ActivityListVo listVo = new ActivityListVo();
        listVo.setAppAuthVo(appAuthVo);
        listVo.setActivityVoList(voList);
        return Result.ofSuccess(listVo);
    }

    @Override
    public Result<PageRsp<PicInfoVo>> hotRankingList(PageReq<String> req) {
        List<PicInfoVo> list = redisTemplate.opsForValue().get(RedisKeys.HOT_FILE_SEARCH);
        if (CollectionUtil.isEmpty(list)) {
            list = queryHotSearch();
        }
        List<PicInfoVo> voList = ListPageUtils.page(list, req.getPageNo(), req.getPageSize());
        PageRsp<PicInfoVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(list.size());
        page.setResult(voList);
        // 为手机计算是否由下一页
        page.next();
        return Result.ofSuccess(page);
    }

    @Override
    public Result<MatchActivityVo> matchActivity() {
        // 先通过缓存取值,有值直接返回,缓存时间为30分钟
        MatchActivityVo redisVo = aiconsumeCacheService.getMatchActivity(AppConstants.REDIS_MATCH_ACTIVITY);
        if (Objects.nonNull(redisVo)) {
            return Result.ofSuccess(redisVo);
        }
        // 2023-08-16修改逻辑:数据类型是报名中的发布时间最新, 如果不存在这个数据，显示赛事结果发布时间最新的数据，如果过期了还是按发布时间最新展示
        // app的首页赛事活动优先查询预告中的赛事文章信息, 文章类别报名中的值为1,同时是需要展示不过期
        MatchActivityVo vo1 = jdbc.matchActivity(1, 0);
        if (Objects.nonNull(vo1)) {
            vo1.setFileKey(urlPrefix + vo1.getFileKey());
            aiconsumeCacheService.putMatchActivity(AppConstants.REDIS_MATCH_ACTIVITY, vo1);
            return Result.ofSuccess(vo1);
        }
        // 若文章类别报告中没有值,则搜索文章类别赛事结果,值为4,同时是需要展示不过期
        MatchActivityVo vo2 = jdbc.matchActivity(4, 0);
        if (Objects.nonNull(vo2)) {
            vo2.setFileKey(urlPrefix + vo2.getFileKey());
            aiconsumeCacheService.putMatchActivity(AppConstants.REDIS_MATCH_ACTIVITY, vo2);
            return Result.ofSuccess(vo2);
        }
        // 若上述两个情况都没有数据,则按发布时间最新展示
        MatchActivityVo vo3 = jdbc.matchActivityByLastTime();
        if (Objects.nonNull(vo3)) {
            vo3.setFileKey(urlPrefix + vo3.getFileKey());
            aiconsumeCacheService.putMatchActivity(AppConstants.REDIS_MATCH_ACTIVITY, vo3);
            return Result.ofSuccess(vo3);
        }
        return null;
    }

    @Override
    public List<HeadlineArticleVo>  headlineArticleTop() {
     /*   // 本地关键字查询 替换为推送置顶 （暂时保留）
        Map<String, String> headlineArticleKeyWord = tsmPropertyCache.getAllValueByCode("headlineArticleKeyWord");
        List<String> keyWord = new ArrayList<>();
        if(!CollectionUtil.isEmpty(headlineArticleKeyWord)){
            keyWord.addAll(headlineArticleKeyWord.keySet());
        }
        List<HeadlineArticleVo> headlineArticleTop = jdbc.findHeadlineArticleTop(keyWord, AppConstants.HEADLINE_ARTICLE_LIMIT);
        // 如果关键字不为空
        if(headlineArticleTop.size()==0 && CollectionUtil.isNotEmpty(keyWord)){
            headlineArticleTop = jdbc.findHeadlineArticleIgnoreTop(AppConstants.HEADLINE_ARTICLE_LIMIT);
        }*/
        List<HeadlineArticleVo> headlineArticleTop = jdbc.findHeadlineArticleTop();
        return headlineArticleTop;
    }

    public List<PicInfoVo> queryHotSearch() {
        List<PicInfoVo> hotSearchVos = jdbc.queryHotSearch();
        if (CollectionUtil.isEmpty(hotSearchVos)) {
            return new ArrayList<>();
        }
        List<Long> uploadIds = hotSearchVos.stream().map(PicInfoVo::getUploadId).collect(Collectors.toList());
        // 查询标签
        List<LabelUpload> list = jdbc.findLabelIdAndName(uploadIds);
        Map<Long, List<LabelUpload>> labelCollect = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(list)) {
            labelCollect = list.stream().collect(Collectors.groupingBy(LabelUpload::getUploadId));
        }
        // 查询图片内容详细
        List<FileDetailDto> fileDetailDtoList = jdbc.findPicDetail(uploadIds);
        fileDetailDtoList.forEach(x -> x.setFileUrl(urlPrefix + x.getFileKey()));
        Map<Long, List<FileDetailDto>> picCollect = fileDetailDtoList.stream().collect(Collectors.groupingBy(FileDetailDto::getUploadId));
        // 填充
        for (PicInfoVo hotSearchVo : hotSearchVos) {
            Long uploadId = hotSearchVo.getUploadId();
            // 查询该id对应标签设置
            List<LabelUpload> list1 = labelCollect.get(hotSearchVo.getUploadId());
            List<LabelCpDto> labelCpDtoList = new ArrayList<>();
            for (LabelUpload labelUpload : list1) {
                LabelCpDto labelDto = new LabelCpDto();
                BeanUtils.copyProperties(labelUpload, labelDto);
                labelCpDtoList.add(labelDto);
            }
            hotSearchVo.setLabels(labelCpDtoList);
            // 图片详情
            hotSearchVo.setFileDetailDtoList(picCollect.get(uploadId));
        }
        redisTemplate.opsForValue().set(RedisKeys.HOT_FILE_SEARCH, hotSearchVos, FileConstants.HOT_SEARCH_TIME, TimeUnit.MINUTES);
        return hotSearchVos;
    }
}
