/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.elstic.config;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.json.jackson.JacksonJsonpMapper;
import co.elastic.clients.transport.ElasticsearchTransport;
import co.elastic.clients.transport.TransportUtils;
import co.elastic.clients.transport.rest_client.RestClientTransport;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.elasticsearch.client.RestClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;

import javax.net.ssl.SSLContext;
import java.io.*;
import java.util.Objects;

/**
 * @author shenhs create 2023/6/10 16:54
 */
@Getter
@Setter
@Configuration
@Slf4j
public class ElsticClientConfig implements Serializable {

	private static final long serialVersionUID = 1L;

	@Value("${elstic.host}")
	private String host;

	@Value("${elstic.port}")
	private int port;

	@Value("${elstic.username:elstic}")
	private String userName;

	@Value("${elstic.password:elstic}")
	private String passWord;



	@Bean
	public ElasticsearchClient getElasticsearchClient(){
		// 生产走https，测试走http
		String env = System.getProperty("spring.profiles.active");
		if(!"h3cTest".equals(env)){
			log.info("------------------http测试-------------------");
			RestClient httpClient = RestClient.builder(
					new HttpHost(host, port)
			).build();
			ElasticsearchTransport transport = new RestClientTransport(httpClient, new JacksonJsonpMapper());
			ElasticsearchClient esClient = new ElasticsearchClient(transport);
			return esClient;
		}
		log.info("------------------http生产-------------------");
		BasicCredentialsProvider credentialsProvider = new BasicCredentialsProvider();
		credentialsProvider.setCredentials(AuthScope.ANY,
				new UsernamePasswordCredentials(userName, passWord));
		RestClient httpClient = RestClient.builder(
				new HttpHost(host, port,"http")
		).build();
		ElasticsearchTransport transport = new RestClientTransport(httpClient, new JacksonJsonpMapper());
		ElasticsearchClient esClient = new ElasticsearchClient(transport);
		return esClient;
	}

	// es走https请求 ，保留
/*	public ElasticsearchClient getElasticsearchClient(){
		BasicCredentialsProvider credentialsProvider = new BasicCredentialsProvider();
		credentialsProvider.setCredentials(AuthScope.ANY,
				new UsernamePasswordCredentials(userName, passWord));
		RestClient httpClient = RestClient.builder(
				new HttpHost(host, port,"https")
		).setHttpClientConfigCallback(httpClientBuilder -> httpClientBuilder
				.setDefaultCredentialsProvider(credentialsProvider).setSSLContext(getContext()).setSSLHostnameVerifier(new NoopHostnameVerifier())).build();
		ElasticsearchTransport transport = new RestClientTransport(httpClient, new JacksonJsonpMapper());
		ElasticsearchClient esClient = new ElasticsearchClient(transport);
		return esClient;
	}*/



	public SSLContext getContext() {
		SSLContext sslContext = null;
		InputStream is = null;
		try {
			Resource resource = new ClassPathResource("http_ca.crt");
			is = resource.getInputStream();
			sslContext = TransportUtils
					.sslContextFromHttpCaCrt(is);

		} catch (Exception e) {
			e.printStackTrace();
		}finally {
			if(Objects.nonNull(is)){
				try {
					is.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return sslContext;
	}

}
