/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.elstic.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.feign.client.dto.ElsticFileDto;
import com.crunii.micro.feign.client.dto.FileElsticPageDto;
import com.crunii.micro.service.elstic.service.FileEsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.List;

/**
 * @author shenhs create 2023/6/10 18:20
 */
@Api(tags = "图绘青春ES管理")
@RestController
@RequestMapping("/auth/fileESController")
public class  FileESController {

	@Autowired
	private FileEsService fileEsService;

	@PostMapping("/create")
	@ApiOperation("创建")
	public Result<?> create() throws IOException {
		fileEsService.create();
		return Result.ofSuccess(null);
	}

	@PostMapping("/save")
	@ApiOperation("保存数据到ES")
	public Result<?> save2ES(@RequestBody ElsticFileDto dto) throws IOException {
		fileEsService.save(dto);
		return Result.ofSuccess(null);
	}

	@GetMapping("/search")
	@ApiOperation("搜索ES中的数据")
	public Result<List<ElsticFileDto>> query(String searchText,Integer mode) throws IOException {
		// mode搜索模式 0搜索全部、1搜索标题、2仅搜索标签
		// from-size
		List<ElsticFileDto> elsticFileDtos = fileEsService.searchFileInfo(searchText);
		return Result.ofSuccess(elsticFileDtos);
	}

	@PostMapping("/pageSearch")
	@ApiOperation("分页搜索ES中的数据")
	public Result<PageRsp<List<ElsticFileDto>>> query(@RequestBody PageReq<FileElsticPageDto> dtoPageReq) throws IOException {
		// mode搜索模式 0搜索全部、1搜索标题、2仅搜索标签
		// from-size
		PageRsp<List<ElsticFileDto>> elsticFileDtos = fileEsService.searchFileInfo(dtoPageReq);
		return Result.ofSuccess(elsticFileDtos);
	}

	@PostMapping("/batchSave")
	@ApiOperation("批量保存ES中的数据")
	public Result<?> save(@RequestBody List<ElsticFileDto> dtos) throws IOException {
		fileEsService.save(dtos);
		return Result.ofSuccess(null);
	}

	@GetMapping("/del")
	@ApiOperation("删除ES中的数据")
	public Result<?> del(String id) throws IOException {
		fileEsService.del(id);
		return Result.ofSuccess(null);
	}

	@PostMapping("/batchUpdate")
	@ApiOperation("批量修改ES中的数据")
	public Result<?> batchUpdate(@RequestBody List<ElsticFileDto> dtos) throws IOException {
		fileEsService.batchUpdate(dtos);
		return Result.ofSuccess(null);
	}

	@GetMapping("/delIndex")
	@ApiOperation("批量修改ES中的数据")
	public Result<?> delIndex(String indexName) throws IOException {
		fileEsService.delIndex(indexName);
		return Result.ofSuccess("删除成功");
	}

}
