package com.crunii.micro.service.elstic.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.feign.client.dto.HotEsPageDto;
import com.crunii.micro.feign.client.dto.YouthHotEsVo;
import com.crunii.micro.service.elstic.service.YouthHotEsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.List;

/**
 * @author wangchen create 2023/12/5 15:05
 */
@Api(tags = "青年热点-ES")
@RestController
@RequestMapping("/auth/YouthHotEsController")
public class YouthHotEsController {

    private final YouthHotEsService youthHotEsService;

    public YouthHotEsController(YouthHotEsService youthHotEsService) {
        this.youthHotEsService = youthHotEsService;
    }

    @PostMapping("/createYouthHotIndex")
    @ApiOperation("创建")
    public Result<String> createYouthHotIndex() throws IOException {
        return Result.ofSuccess(youthHotEsService.create());
    }

    @PostMapping("/saveYouthHot")
    @ApiOperation("新增")
    public Result<String> saveYouthHot(@RequestBody YouthHotEsVo dto) throws IOException {
        return Result.ofSuccess(youthHotEsService.save(dto));
    }

    @PostMapping("/pageSearchYouthHot")
    @ApiOperation("分页搜索ES中的数据")
    public Result<PageRsp<List<YouthHotEsVo>>> pageSearchYouthHot(@RequestBody PageReq<HotEsPageDto> dtoPageReq) throws IOException {
        PageRsp<List<YouthHotEsVo>> elsticFileDtos = youthHotEsService.searchFileInfo(dtoPageReq);
        return Result.ofSuccess(elsticFileDtos);
    }

    @GetMapping("/delYouthHot")
    @ApiOperation("删除ES中的数据")
    public Result<?> delYouthHot(String id) throws IOException {
        youthHotEsService.del(id);
        return Result.ofSuccess(null);
    }

}
