/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.elstic.service;

import co.elastic.clients.elasticsearch._types.query_dsl.BoolQuery;
import co.elastic.clients.elasticsearch.core.IndexResponse;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.feign.client.dto.ElsticFileDto;
import com.crunii.micro.service.elstic.dto.ESOperate;

import java.io.IOException;
import java.util.List;

/**
 * @author shenhs create 2023/6/10 16:57
 */
public interface ElsticService {

	String INS_ES_TIME_FIELD = "insEsTime";

	/**
	 * 往ES写入数据
	 * @param indexName 索引名称
	 * @Param idField ID字段
	 * @param obj 写入的对象
	 */
	IndexResponse index(String indexName, String idField, Object obj) throws IOException;

	/**
	 * 批量往ES写入数据
	 * @param indexName
	 * @param obj
	 */
	IndexResponse batchIndex(String indexName, String idFiled, Object... obj) throws IOException;

	/**
	 * 删除某条数据
	 * @param indexName
	 * @param operate
	 */
	void del(String indexName, ESOperate operate);

	/**
	 * 多重必须匹配的条件
	 * @param indexName
	 * @param operates
	 */
	<T> List<T> mustQuery(String indexName, List<ESOperate> operates, Class<T> clazz) throws IOException;

	/**
	 * 多重必须匹配的条件
	 * @param indexName
	 * @param operates
	 */
	<T> List<T> shouldQuery(String indexName,List<ESOperate> operates,Class<T> clazz)throws IOException;

	/**
	 *
	 * @param indexName
	 * @param clazz
	 * @param builder
	 * @return
	 */
	<T> List<T> query(String indexName, Class<T> clazz, BoolQuery.Builder builder) throws IOException;

	public <T> List<T> query(String indexName, Class<T> clazz, BoolQuery.Builder builder, PageReq pageReq) throws IOException;

	Long count(String indexName, BoolQuery.Builder builder) throws IOException;
	/**
	 * 根据ID删除数据
	 * @param indexName 索引名称
	 * @param id 数据ID
	 * @throws IOException
	 */
	void del(String indexName,String id) throws IOException;

	<T> PageRsp<List<T>> defMatchQuery(String indexName, List<ESOperate> operates, Class<T> clazz, PageReq pageReq) throws IOException;

	<T> PageRsp<List<T>> termMatchQuery(String indexName, List<ESOperate> operates, Class<T> clazz, PageReq pageReq) throws IOException;

	/**
	 * 批量修改
	 * @param fileIndex index
	 * @param dtos data
	 */
	void batchUpdate(String fileIndex, List<ElsticFileDto> dtos) throws IOException;

    void delIndex(String indexName);

    boolean matchIndex(String youth_hot_index) throws IOException;
}
