package com.crunii.micro.service.elstic.service.impl;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch._types.mapping.Property;
import co.elastic.clients.elasticsearch._types.query_dsl.BoolQuery;
import co.elastic.clients.elasticsearch._types.query_dsl.Query;
import co.elastic.clients.elasticsearch._types.query_dsl.TermQuery;
import co.elastic.clients.elasticsearch._types.query_dsl.WildcardQuery;
import co.elastic.clients.elasticsearch.indices.CreateIndexRequest;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.dto.AppMessageCenterDTO;
import com.crunii.micro.feign.client.dto.MessageSendSelect;
import com.crunii.micro.feign.client.dto.MessageSendSelectVO;
import com.crunii.micro.service.elstic.config.IndexConfig;
import com.crunii.micro.service.elstic.dto.ESOperate;
import com.crunii.micro.service.elstic.service.AppMessageService;
import com.crunii.micro.service.elstic.service.ElsticService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @program: microservice
 * @description: 消息中心impl
 * @author: xiang xin
 * @create: 2023-12-16 15:26
 */
@Service
public class AppMessageServiceImpl implements AppMessageService {
    @Autowired
    ElsticService elsticService;
    @Autowired
    IndexConfig indexConfig;
    @Autowired
    ElasticsearchClient esClient;
    private String ID_FIELD = "appMessageId";

    private static final String FIELD_CONTENT_TITLE = "dataTitle";
    private static final String FIELD_CONTENT = "messageContent";
    private static final String LOGINID = "loginId";

    @Override
    public String create() throws IOException {
        Property keywordProperty = Property.of(p -> p.keyword(k -> k));
        Property textKeywordProperty = Property.of(l -> l.text(t -> t.fields("keyword", f -> f.keyword(k -> k)).analyzer("ik_max_word")));
        CreateIndexRequest.Builder mappings = new CreateIndexRequest.Builder().index(indexConfig.getAPP_MESSAGE_INDEX())
                .mappings(m ->
                        m.properties("insEsTime", k -> k.date(d -> d))
                                .properties("id", textKeywordProperty)
                                .properties("sendId", textKeywordProperty)
                                .properties("sendWay", textKeywordProperty)
                                .properties("messageType", textKeywordProperty)
                                .properties("receiver", textKeywordProperty)
                                .properties("messageContent", keywordProperty)
                                .properties("isRead", textKeywordProperty)
                                .properties("dataSource", textKeywordProperty)
                                .properties("dataTitle", textKeywordProperty)
                                .properties("notifyObject", textKeywordProperty)
                                .properties("pushTerminal", textKeywordProperty)
                                .properties("pushTime", textKeywordProperty)
                                .properties("userName", textKeywordProperty)
                                .properties("mobilePhone", textKeywordProperty)
                                .properties("accountNumber", textKeywordProperty)
                                .properties("messageClassify", textKeywordProperty)
                                .properties("loginId", keywordProperty)
                                .properties("sendLoginId", keywordProperty)
                                .properties("messageSendTaskId", keywordProperty)
                                .properties("pathAndJson", keywordProperty)
                );
        esClient.indices().create(m -> mappings);
        return "创建成功";
    }

    @Override
    public String save(MessageSendSelect dto) throws IOException {
        elsticService.index(indexConfig.getAPP_MESSAGE_INDEX(), ID_FIELD, dto);
        return "保存成功";
    }

    @Override
    public List<MessageSendSelectVO> searchFileInfo(AppMessageCenterDTO dto) throws IOException {
        CurrWorkerUtil.mockCurrWorker();
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        //模糊
        List<ESOperate> operateShould = new ArrayList<>();
        if (StringUtils.isNotBlank(dto.getData())) {
            ESOperate contextIdOperate1 = new ESOperate(FIELD_CONTENT_TITLE, dto.getData());
            operateShould.add(contextIdOperate1);
            ESOperate contextIdOperate2 = new ESOperate(FIELD_CONTENT, dto.getData());
            operateShould.add(contextIdOperate2);
        }
        List<Query> queryList = getQueryList(operateShould);
//        ESOperate contextIdOperate = new ESOperate(LOGINID, Long.valueOf(currWorker.getLoginId()));
//        List<ESOperate> operateMust = new ArrayList<>();
//        operateMust.add(contextIdOperate);
//        List<Query> termQuery = getQueryList(operateMust);
        BoolQuery.Builder builder = new BoolQuery.Builder().should(queryList);
        Long loginId = Long.valueOf(currWorker.getLoginId());
        List<JSONObject> query = elsticService.query(indexConfig.getAPP_MESSAGE_INDEX(), JSONObject.class, builder);
        List<MessageSendSelectVO> collect = query.stream().map(item -> JSONUtil.toBean(item, MessageSendSelectVO.class)).collect(Collectors.toList());
        if (collect.size() > 0) {
            Iterator<MessageSendSelectVO> iterator = collect.iterator();
            while (iterator.hasNext()) {
                MessageSendSelectVO messageSendSelect = iterator.next();
                if (!messageSendSelect.getLoginId().equals(loginId)) {
                    iterator.remove();
                }
            }
        }
        return collect;
    }

    private List<Query> getQueryList(List<ESOperate> operates) {
        //模糊
        List<Query> queryList = operates.stream().map(operate ->
                WildcardQuery.of(f ->
                        f.field(operate.getField() + ".keyword").value("*" + operate.getValue().toString() + "*")
                )._toQuery()
        ).collect(Collectors.toList());
        return queryList;
    }

    public List<Query> getTermQuery(List<ESOperate> operates) {
        //精准
        List<Query> collect = operates.stream().map(operate ->
                TermQuery.of(f ->
                        f.field(operate.getField() + ".keyword").value(operate.getValue().toString()))._toQuery()
        ).collect(Collectors.toList());
        return collect;
    }

    @Override
    public void del(String id) throws IOException {
        elsticService.del(indexConfig.getAPP_MESSAGE_INDEX(), id);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
 /*       boolean b = elsticService.matchIndex(indexConfig.getAPP_MESSAGE_INDEX());
        if (!b) {
            this.create();
        }*/
    }
}
