/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.elstic.service.impl;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch._types.query_dsl.*;
import co.elastic.clients.elasticsearch.core.*;
import co.elastic.clients.elasticsearch.core.bulk.BulkOperation;
import co.elastic.clients.elasticsearch.core.bulk.BulkOperationBuilders;
import co.elastic.clients.elasticsearch.core.bulk.IndexOperation;
import co.elastic.clients.elasticsearch.core.search.Hit;
import co.elastic.clients.elasticsearch.indices.DeleteIndexRequest;
import co.elastic.clients.elasticsearch.indices.ExistsRequest;
import co.elastic.clients.elasticsearch.indices.GetIndexRequest;
import co.elastic.clients.transport.endpoints.BooleanResponse;
import co.elastic.clients.util.ObjectBuilder;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.feign.client.dto.ElsticFileDto;
import com.crunii.micro.service.elstic.dto.ESOperate;
import com.crunii.micro.service.elstic.service.ElsticService;
import org.elasticsearch.client.RequestOptions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author shenhs create 2023/6/10 17:07
 */
@Service
public class ElsticServiceImpl implements ElsticService {

	@Autowired ElasticsearchClient esClient;


	@Override public IndexResponse index(String indexName,String idFiled, Object obj) throws IOException {
		JSONObject entries = JSONUtil.parseObj(obj);
		JSONObject entries1 = entries.putOnce(INS_ES_TIME_FIELD, getESTime());
		IndexResponse index = esClient.index(i -> i.index(indexName).id(entries1.get(idFiled,String.class))
				.document(entries1));

		return index;
	}

	@Override public IndexResponse batchIndex(String indexName,String idFiled, Object... objs) throws IOException {
		List<BulkOperation> list = new ArrayList<>();
		for (Object obj: objs){
			JSONObject entries = JSONUtil.parseObj(obj);
			entries.putOnce(INS_ES_TIME_FIELD, getESTime());
			BulkOperation index = BulkOperationBuilders.index(
					fn -> (ObjectBuilder<IndexOperation>) fn.document(entries).index(indexName).id(entries.get(idFiled, String.class)));
			list.add(index);
		}
		esClient.bulk(fn-> fn.operations(list));

//		objs.forEach(item->{
//			JSONObject entries = JSONUtil.parseObj(item);
//			entries.putOnce(INS_ES_TIME_FIELD, getESTime());
//			op.create(cr->cr.index(indexName).id(entries.get(idFiled,String.class)).document(entries))
//		})
//		BulkOperation.of(fn->fn.index(new IndexOperation.Builder<>().))
//
//		BulkRequest.Builder builder = new BulkRequest.Builder();
//		BulkRequest.Builder index = builder.index(indexName);
//		index.operations();
//		esClient.bulk()
		return null;
		/*
		BulkOperation.of(fn->fn.index())
		BulkRequest.of(fn->fn.index(indexName).operations())

		objs.stream().forEach(obj -> {
			JSONObject entries = JSONUtil.parseObj(obj);
			entries.putOnce(INS_ES_TIME_FIELD, getESTime());
			IndexRequest indexRequest = new IndexRequest();
			indexRequest.id(entries.get(idFiled,String.class));
			IndexRequest source = indexRequest.source(entries.toString(), XContentType.JSON);
			bulkRequestBuilder.add(source);
		});*/

	}

	@Override public void del(String indexName, ESOperate operate) {

	}

	@Override public <T> List<T> mustQuery(String indexName, List<ESOperate> operates, Class<T> clazz) throws IOException {
		List<Query> queryList = getQueryList(operates);
		BoolQuery.Builder must = new BoolQuery.Builder().must(queryList);
		return query(indexName,clazz,must);
	}

	@Override public <T> List<T> shouldQuery(String indexName, List<ESOperate> operates,Class<T> clazz) throws IOException {
		List<Query> queryList = getQueryList(operates);
		BoolQuery.Builder should = new BoolQuery.Builder().should(queryList);
		return query(indexName, clazz, should);
	}


	@Override
	public <T> List<T> query(String indexName, Class<T> clazz, BoolQuery.Builder builder) throws IOException {
		SearchResponse<T> search = esClient.search(s -> s
				.index(indexName)
				.query(q -> q.bool(b -> builder)
				),clazz);
		List<Hit<T>> hits = search.hits().hits();
		return hits.stream().map(item->item.source()).collect(Collectors.toList());
	}

	@Override
	public <T> List<T> query(String indexName, Class<T> clazz, BoolQuery.Builder builder,PageReq pageReq) throws IOException {
		SearchResponse<JSONObject> search = esClient.search(s -> s
				.from(pageReq.getMyStart())
				.size(pageReq.getPageSize())
				.index(indexName)
				.query(q -> q.bool(b -> builder)
				),JSONObject.class);
		List<Hit<JSONObject>> hits = search.hits().hits();
		return hits.stream().map(item->item.source().toBean(clazz)).collect(Collectors.toList());
	}

	@Override
	public Long count(String indexName, BoolQuery.Builder builder) throws IOException {
		CountResponse count = esClient.count(c -> c.index(indexName).query(q -> q.bool(b -> builder)));
		return count.count();
	}


	public void del(String indexName,String id) throws IOException {
		DeleteResponse delete = esClient.delete(fn -> fn.index(indexName).id(id));
	}

	private String getESTime(){
		return DateUtil.format(new Date(), DatePattern.UTC_WITH_ZONE_OFFSET_PATTERN);
	}

	private List<Query> getQueryList(List<ESOperate> operates){
		List<Query> queryList = operates.stream().map(operate ->
				/*MatchQuery.of(f ->
						f.field(operate.getField()).query(operate.getValue().toString())
				)._toQuery() */
				/*FuzzyQuery.of(f->
								f.field(operate.getField()).value(operate.getValue().toString())
						)._toQuery() */
				WildcardQuery.of(f->
						f.field(operate.getField()+".keyword").value("*"+operate.getValue().toString()+"*")
				)._toQuery()
		).collect(Collectors.toList());
		return queryList;
	}


	public <T> PageRsp<List<T>> defMatchQuery(String indexName, List<ESOperate> operates, Class<T> clazz, PageReq pageReq) throws IOException {
		List<Query> queryList = getQueryList(operates);
		BoolQuery.Builder should = new BoolQuery.Builder().should(queryList);
		BoolQuery.Builder should2 = new BoolQuery.Builder().should(queryList);
		Long count = count(indexName, should);
		List<T> query = query(indexName, clazz, should2, pageReq);
		PageRsp pageRsp = new PageRsp(pageReq.getPageNo(), pageReq.getPageSize(), count.intValue(), query);
		return pageRsp;
	}

	@Override
	public void delIndex(String indexName) {
		try {
			esClient.indices().delete(req->req.index(indexName));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public boolean matchIndex(String index) throws IOException {
		ExistsRequest of = ExistsRequest.of(fn -> fn.index(index));
		BooleanResponse exists = esClient.indices().exists(of);
		return exists.value();
	}

	public <T> PageRsp<List<T>> termMatchQuery(String indexName, List<ESOperate> operates, Class<T> clazz, PageReq pageReq) throws IOException {
		List<Query> termQuery = getTermQuery(operates);
		BoolQuery.Builder should = new BoolQuery.Builder().should(termQuery);
		BoolQuery.Builder should2 = new BoolQuery.Builder().should(termQuery);
		Long count = count(indexName, should);
		List<T> query = query(indexName, clazz, should2, pageReq);
		return new PageRsp(pageReq.getPageNo(), pageReq.getPageSize(), count.intValue(), query);
	}

	@Override
	public void batchUpdate(String fileIndex, List<ElsticFileDto> dtos) throws IOException {
		List<BulkOperation> operations = new ArrayList<>();
		for (ElsticFileDto elsticFileDto : dtos) {
			String uploadId = elsticFileDto.getUploadId().toString();
			BooleanResponse exists = esClient.exists(fn -> fn.index(fileIndex).id(uploadId));
			if(exists.value()){
				JSONObject jsonObject = JSONUtil.parseObj(elsticFileDto);
				jsonObject.putOnce(INS_ES_TIME_FIELD,getESTime());
				BulkOperation index = BulkOperationBuilders.index(fn -> (ObjectBuilder<IndexOperation>) fn.document(jsonObject).index(fileIndex).id(elsticFileDto.getUploadId().toString()));
				operations.add(index);
			}
		}
		BulkRequest of = BulkRequest.of(fn -> fn.operations(operations));
		esClient.bulk(of);
	}

	/**
	 * 精确查询
	 * @param operates
	 * @return
	 */
	public List<Query> getTermQuery(List<ESOperate> operates){

		List<Query> collect = operates.stream().map(operate ->
				TermQuery.of(f ->
						f.field(operate.getField()+".keyword").value(operate.getValue().toString()))._toQuery()
		).collect(Collectors.toList());
		return collect;
	}
}
