/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.elstic.service.impl;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch._types.mapping.Property;
import co.elastic.clients.elasticsearch.indices.CreateIndexRequest;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.feign.client.dto.ElsticFileDto;
import com.crunii.micro.feign.client.dto.FileElsticPageDto;
import com.crunii.micro.service.elstic.config.IndexConfig;
import com.crunii.micro.service.elstic.dto.ESOperate;
import com.crunii.micro.service.elstic.service.ElsticService;
import com.crunii.micro.service.elstic.service.FileEsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author shenhs create 2023/6/10 18:35
 */
@Service
public class FileEsServiceImpl implements FileEsService {

	@Autowired ElsticService elsticService;
	@Autowired IndexConfig indexConfig;
	@Autowired ElasticsearchClient esClient;

	private String ID_FIELD = "uploadId";

	private static final String FIELD_FILE_NAME = "fileTitle";

	private static final String FIELD_FILE_LABEL_NAME = "labels.labelName";

	private static final int ALL_SEARCH = 0;
	private static final int NAME_SEARCH = 1;
	private static final int LABEL_SEARCH = 2;

	private static final int DEF_MATCH = 0;
	private static final int TERM_MATCH = 1;

	@Override public void create() throws IOException {

		Property keywordProperty = Property.of(p -> p.keyword(k -> k));
		Property textKeywordProperty = Property.of(l -> l.text(t -> t.fields("keyword", f -> f.keyword(k -> k)).analyzer("ik_max_word")));

		CreateIndexRequest.Builder mappings = new CreateIndexRequest.Builder().index(indexConfig.getFILE_MGR_INDEX())
				.mappings(m ->
						m.properties("insEsTime", k -> k.date(d->d))
						.properties("uploadId",keywordProperty)
						.properties("isAlbum",keywordProperty)
						.properties("fileTitle",textKeywordProperty)
						.properties("requiredIntegral",keywordProperty)
						.properties("uploadWorkerName",textKeywordProperty)
						.properties("pushTime",keywordProperty)
						.properties("describe",textKeywordProperty)
						// labels
						.properties("labels.labelId",keywordProperty)
						.properties("labels.labelName",textKeywordProperty)
						// fileDetailDtoList
						.properties("fileDetailDtoList.uploadId",keywordProperty)
						.properties("fileDetailDtoList.fileRecordId",keywordProperty)
						.properties("fileDetailDtoList.fileName",textKeywordProperty)
						.properties("fileDetailDtoList.fileType",keywordProperty)
						.properties("fileDetailDtoList.fileKey",keywordProperty)
						.properties("fileDetailDtoList.picWidth",keywordProperty)
						.properties("fileDetailDtoList.picHeight",keywordProperty)
						.properties("fileDetailDtoList.isPic",keywordProperty)
						.properties("fileDetailDtoList.fileUrl",keywordProperty)
						.properties("fileDetailDtoList.fileSize",keywordProperty)
				);
		esClient.indices().create(m-> mappings);
	}

	public void save(ElsticFileDto dto) throws IOException {
		elsticService.index(indexConfig.getFILE_MGR_INDEX(),ID_FIELD,dto);
	}

	@Override
	public void save(List<ElsticFileDto> dtos) throws IOException {
		elsticService.batchIndex(indexConfig.getFILE_MGR_INDEX(),ID_FIELD, dtos.toArray());
	}

	public List<ElsticFileDto> searchFileInfo(String searchText) throws IOException {
		List<ESOperate> operates = new ArrayList<>();
		operates.add(new ESOperate(FIELD_FILE_NAME,searchText));
		operates.add(new ESOperate(FIELD_FILE_LABEL_NAME,searchText));
		List<JSONObject> jsonObjects = elsticService.shouldQuery(indexConfig.getFILE_MGR_INDEX(), operates, JSONObject.class);
		List<ElsticFileDto> collect = jsonObjects.stream().map(item -> JSONUtil.toBean(item, ElsticFileDto.class)).collect(Collectors.toList());
		return collect;
	}

	@Override public PageRsp<List<ElsticFileDto>> searchFileInfo(PageReq<FileElsticPageDto> dtoPageReq) throws IOException {
		FileElsticPageDto query = dtoPageReq.getQuery();
		String searchText = query.getSearchText();
		int searchMode = query.getSearchMode();
		int matchMode = query.getMatchMode();
		ESOperate fileOperate = new ESOperate(FIELD_FILE_NAME, searchText);
		ESOperate labelOperate = new ESOperate(FIELD_FILE_LABEL_NAME, searchText);
		List<ESOperate> operates = new ArrayList<>();
		if(searchMode==ALL_SEARCH){
			operates.add(fileOperate);
			operates.add(labelOperate);
		}else if(searchMode==NAME_SEARCH){
			operates.add(fileOperate);
		}else {
			operates.add(labelOperate);
		}

		if(matchMode == DEF_MATCH){
			// 模糊匹配
			PageRsp<List<ElsticFileDto>> listPageRsp = elsticService.defMatchQuery(indexConfig.getFILE_MGR_INDEX(), operates, ElsticFileDto.class,
					dtoPageReq);
			return listPageRsp;
		}else{
			// 精确匹配
			PageRsp<List<ElsticFileDto>> listPageRsp = elsticService.termMatchQuery(indexConfig.getFILE_MGR_INDEX(), operates, ElsticFileDto.class,
					dtoPageReq);
			return listPageRsp;
		}
	}

	@Override public void del(String id) throws IOException{
		elsticService.del(indexConfig.getFILE_MGR_INDEX(),id);
	}

	@Override
	public void batchUpdate(List<ElsticFileDto> dtos) throws IOException {
		elsticService.batchUpdate(indexConfig.getFILE_MGR_INDEX(),dtos);
	}

	@Override
	public void delIndex(String indexName) {
		elsticService.delIndex(indexName);
	}

}
