/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.elstic.service.impl;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch.indices.CreateIndexRequest;
import com.crunii.micro.feign.client.dto.LabelDto;
import com.crunii.micro.service.elstic.config.IndexConfig;
import com.crunii.micro.service.elstic.dto.ESOperate;
import com.crunii.micro.service.elstic.service.ElsticService;
import com.crunii.micro.service.elstic.service.LabelEsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author shenhs create 2023/6/12 15:14
 */
@Service
public class LabelEsServiceImpl implements LabelEsService {
	@Autowired ElsticService elsticService;
	@Autowired IndexConfig indexConfig;
	@Autowired ElasticsearchClient esClient;
	private String ID_FIELD = "labelId";

	private static final String LABEL_NAME = "labelName";

	@Override public void create() throws IOException {
		CreateIndexRequest.Builder mappings = new CreateIndexRequest.Builder().index(indexConfig.getLABEL_MGR_INDEX())
				.mappings(m ->
						m.properties("insEsTime", k -> k.date(d->d))
						.properties("labelId",k->k.keyword(w->w))
						.properties("labelName",l -> l.text(t -> t.fields("keyword", f -> f.keyword(k -> k)).analyzer("ik_max_word")))
				);
		esClient.indices().create(m-> mappings);
	}

	public void save(LabelDto dto) throws IOException {
		elsticService.index(indexConfig.getLABEL_MGR_INDEX(),ID_FIELD,dto);
	}

	@Override
	public void save(List<LabelDto> dtos) throws IOException {
		elsticService.batchIndex(indexConfig.getLABEL_MGR_INDEX(),ID_FIELD, dtos.toArray());
	}

	public List<LabelDto> searchFileInfo(String searchText) throws IOException {
		List<ESOperate> operates = new ArrayList<>();
		operates.add(new ESOperate(LABEL_NAME,searchText));
		List<JSONObject> jsonObjects = elsticService.mustQuery(indexConfig.getLABEL_MGR_INDEX(), operates, JSONObject.class);
		List<LabelDto> collect = jsonObjects.stream().map(item -> JSONUtil.toBean(item, LabelDto.class)).collect(Collectors.toList());
		return collect;
	}

	@Override public void del(String id) throws IOException{
		elsticService.del(indexConfig.getLABEL_MGR_INDEX(),id);
	}

}
