package com.crunii.micro.service.elstic.service.impl;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch._types.mapping.Property;
import co.elastic.clients.elasticsearch.core.UpdateRequest;
import co.elastic.clients.elasticsearch.indices.CreateIndexRequest;
import co.elastic.clients.transport.endpoints.BooleanResponse;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.feign.client.dto.YouthHotEsVo;
import com.crunii.micro.service.elstic.config.IndexConfig;
import com.crunii.micro.service.elstic.dto.ESOperate;
import com.crunii.micro.feign.client.dto.HotEsPageDto;
import com.crunii.micro.service.elstic.service.ElsticService;
import com.crunii.micro.service.elstic.service.YouthHotEsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author wangchen create 2023/12/5 15:06
 */
@Service
public class YouthHotEsServiceImpl implements YouthHotEsService {

    @Resource
    ElsticService elsticService;
    @Resource IndexConfig indexConfig;
    @Resource
    ElasticsearchClient esClient;
    private String ID_FIELD = "contentManageId";

    private static final String FIELD_CONTENT_TITLE= "contentTitle";
    private static final String FIELD_CONTENT = "articleContext";
    private static final String FIELD_CONTENT_ID = "mediumId";
    /* k -> k.date(d->d)*/
    @Override
    public String create() throws IOException {
        Property keywordProperty = Property.of(p -> p.keyword(k -> k));
        Property textKeywordProperty = Property.of(l -> l.text(t -> t.fields("keyword", f -> f.keyword(k -> k.ignoreAbove(256))).analyzer("ik_max_word")));

        CreateIndexRequest.Builder mappings = new CreateIndexRequest.Builder().index(indexConfig.getYOUTH_HOT_INDEX())
                .mappings(m ->
                        m.properties("insEsTime",k->k.date(d->d))
                                .properties("contentManageId",keywordProperty)
                                .properties("contentTitle",textKeywordProperty)
                                .properties("articleContext",textKeywordProperty)
                                .properties("fileType",keywordProperty)
                                .properties("idName",keywordProperty)
                                .properties("mediumHeadPic",keywordProperty)
                                .properties("mediumId",textKeywordProperty)
                );
        esClient.indices().create(m-> mappings);
        return "创建成功";
    }

    @Override
    public String save(YouthHotEsVo dto) throws IOException {
        elsticService.index(indexConfig.getYOUTH_HOT_INDEX(),ID_FIELD,dto);
        return "保存成功";
    }

    @Override
    public PageRsp<List<YouthHotEsVo>> searchFileInfo(PageReq<HotEsPageDto> dtoPageReq) throws IOException {
        HotEsPageDto query = dtoPageReq.getQuery();
        String searchText = query.getKeyWord();
        List<ESOperate> operates = new ArrayList<>();
        Long  mediumId = query.getMediumId();
        if(Objects.nonNull(mediumId)){
            ESOperate contextIdOperate = new ESOperate(FIELD_CONTENT_ID, mediumId);
            operates.add(contextIdOperate);
            return  elsticService.termMatchQuery(indexConfig.getYOUTH_HOT_INDEX(), operates, YouthHotEsVo.class,dtoPageReq);
        }else {
            ESOperate contextTitleOperate = new ESOperate(FIELD_CONTENT_TITLE, searchText);
            ESOperate contextOperate = new ESOperate(FIELD_CONTENT, searchText);
            operates.add(contextTitleOperate);
            operates.add(contextOperate);
            return elsticService.defMatchQuery(indexConfig.getYOUTH_HOT_INDEX(),operates,YouthHotEsVo.class,dtoPageReq);
        }
    }

    @Override
    public void del(String id) throws IOException {
        elsticService.del(indexConfig.getYOUTH_HOT_INDEX(),id);
    }
}
