package com.crunii.micro.service.filemanager.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.feign.client.dto.ElsticFileDto;
import com.crunii.micro.feign.client.dto.FileElsticPageDto;
import com.crunii.micro.service.filemanager.dto.*;
import com.crunii.micro.service.filemanager.service.FileUploadService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

/**
 * @author wangchen create 2023/6/16 14:19
 */
@Api(tags = "文件中心模块2.0")
@RestController
@RequestMapping("/auth/FileUploadController")
public class FileUploadController {

    private final FileUploadService fileUploadService;

    public FileUploadController(FileUploadService fileUploadService) {
        this.fileUploadService = fileUploadService;
    }

    @ApiOperation(value = "文件中心-搜索")
    @ApiImplicitParam(name = "keyword",value = "关键字",required = true)
    @PostMapping("/keywordSearch")
    public Result<PageRsp<ElsticFileDto>> keywordSearch(@RequestBody PageReq<FileElsticPageDto> keyword){
        return fileUploadService.keywordSearch(keyword);
    }

    @ApiOperation(value = "热门搜索 && 最近上传")
    @GetMapping(path = "/hotSearchDisplay")
    public Result<FileExhibition> queryFileExhibition(){
        return Result.ofSuccess(fileUploadService.queryFileExhibition());
    }

    @ApiOperation(value = "热门搜索刷新缓存")
    @PostMapping("/flushCash")
    public Result<String> flushCash(){
        fileUploadService.queryHotSearch();
        return Result.ofSuccess("刷新成功");
    }

    @ApiOperation(value = "es同步数据")
    @PostMapping("/synEsFile")
    public Result<String> synEsFile(String code){
        fileUploadService.synEsFile(code);
        return Result.ofSuccess("同步成功");
    }
}
