package com.crunii.micro.service.filemanager.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.filemanager.dto.*;
import com.crunii.micro.service.filemanager.service.MyFileCenterService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * @author wangchen create 2023/6/17 15:35
 */
@Api(tags = "我的文件")
@RestController
@RequestMapping("/auth/MyFileCenterController")
public class MyFileCenterController {

    private final MyFileCenterService myFileCenterService;

    public MyFileCenterController(MyFileCenterService myFileCenterService) {
        this.myFileCenterService = myFileCenterService;
    }

    @ApiOperation(value = "上传文件")
    @PostMapping("/uploadFile")
    public Result<String> uploadFile(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @Validated UploadInfoDto uploadInfoDto){
        return Result.ofSuccess(myFileCenterService.uploadFile(worker,uploadInfoDto));
    }

    @ApiOperation(value = "我的-文件删除")
    @PostMapping("/deleteFile")
    public Result<String> deleteFile(@CurrWorker  @ApiIgnore Worker worker,@RequestBody FileDelDto fileDelDto){
        return Result.ofSuccess(myFileCenterService.deleteFile(worker.getLoginId(),fileDelDto));
    }

    @ApiOperation(value = "分页查询文件信息")
    @PostMapping("/pageMyFile")
    public Result<PageRsp<MyFileVo>> pageMyFile(@CurrWorker  @ApiIgnore Worker worker, @RequestBody PageReq<PageMyFileDto> fileDelDto){
        return Result.ofSuccess(myFileCenterService.pageMyFile(worker.getLoginId(),fileDelDto));
    }

    @ApiOperation(value = "修改文件信息")
    @PostMapping("/updateMyFile")
    public Result<String> updateMyFile(@CurrWorker  @ApiIgnore Worker worker, @RequestBody ModifyInfoDto modifyInfoDto){
        return Result.ofSuccess(myFileCenterService.updateMyFile(worker,modifyInfoDto));
    }

    @GetMapping("/auditRetrial")
    @ApiOperation(value = "审核重审")
    public Result<String> auditRetrial(@CurrWorker  @ApiIgnore Worker worker,Long uploadId){
        return Result.ofSuccess(myFileCenterService.auditDismissal(worker,uploadId));
    }

    @GetMapping("/queryProcess")
    @ApiOperation(value = "查询流程")
    public Result<List<AuditProcessVo>> queryProcess(@CurrWorker  @ApiIgnore Worker worker, Long uploadId){
        return Result.ofSuccess(myFileCenterService.queryProcess(worker,uploadId));
    }

}
