package com.crunii.micro.service.filemanager.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.sys.FileDetailDto;
import com.crunii.micro.service.filemanager.dto.AuditProcessVo;
import com.crunii.micro.service.filemanager.dto.LabelUpload;
import com.crunii.micro.service.filemanager.dto.MyFileDto;
import com.crunii.micro.service.filemanager.dto.MyFileVo;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/6/19 9:03
 */
@Repository
public class FileAuditDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;
    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNPJdbcTemplate;

    public PageRsp<MyFileVo> pageAuditFile(PageReq<MyFileDto> fileDelDto, boolean isSysRole,String orgId) {

        String roleAudit = "A.deparment_audit_status";
        if(isSysRole){
            roleAudit = "A.sys_audit_status";
        }
        String format = String.format("SELECT A.upload_id ,B.push_time,B.describe,B.is_album ,B.file_title ,%s AS auditStatus,B.required_integral ,B.upload_worker_name  FROM YCQ_FILE_AUDIT_ROLE A LEFT JOIN YCQ_UPLOAD_INFO B ON A.upload_id =B.upload_id where B.state  = 1", roleAudit);
        StringBuilder sb = new StringBuilder(format);
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_FILE_AUDIT_ROLE A LEFT JOIN YCQ_UPLOAD_INFO B ON A.upload_id =B.upload_id where B.state  = 1 ");
        MyFileDto query = fileDelDto.getQuery();
        Map<String,Object> params = new HashMap<>(10);
        if(isSysRole){
            sb.append(" and A.sys_audit_status is not null");
            sbCount.append(" and A.sys_audit_status is not null");
        }else {
            sb.append(" and A.deparment_audit_status is not null and :orgId in(SELECT org_id FROM TSM_ORG START WITH org_id= B.org_id CONNECT BY PRIOR parent_org_id=org_id)");
            sbCount.append(" and A.deparment_audit_status is not null and :orgId in(SELECT org_id FROM TSM_ORG START WITH org_id= B.org_id CONNECT BY PRIOR parent_org_id=org_id)");
            params.put("orgId",orgId);
        }
        if(Objects.nonNull(query.getAuditStatus())){
            if(isSysRole){
                sb.append(" and A.sys_audit_status = :status");
                sbCount.append(" and A.sys_audit_status = :status");
            }else {
                sb.append(" and A.deparment_audit_status = :status");
                sbCount.append(" and A.deparment_audit_status = :status");
            }
            params.put("status",query.getAuditStatus());
        }
        if(StringUtils.hasText(query.getName())){
            sb.append(" AND ( B.file_title LIKE concat('%',:fileTitle,'%')  OR A.upload_id in(SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_name LIKE concat('%',:labelName,'%')) )");
            sbCount.append(" AND ( B.file_title LIKE concat('%',:fileTitle,'%')  OR A.upload_id in(SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_name LIKE concat('%',:labelName,'%')) )");
            params.put("fileTitle",query.getName());
            params.put("labelName",query.getName());
        }

        int total = baseNPJdbcTemplate.queryForInt(sbCount.toString(), params);
        List<MyFileVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by B.update_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, fileDelDto);
            values = baseNPJdbcTemplate.queryForBeans(MyFileVo.class, sb.toString(), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<MyFileVo> page = new PageRsp<>();
        page.setPageNo(fileDelDto.getPageNo());
        page.setPageSize(fileDelDto.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }
    public List<LabelUpload> findLabelIdAndName(List<Long> uploadIds) {
        String sql = "SELECT file_id uploadId,label_id,label_name FROM YCQ_FILE_LABEL_RELATION WHERE file_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        return baseNPJdbcTemplate.queryForBeans(LabelUpload.class,sql,params);
    }

    public List<FileDetailDto> findPicDetail(List<Long> uploadIds) {
        String sql = "SELECT upload_id,file_name,file_record_id,file_type,pic_width,pic_height,is_pic,file_size,file_key FROM YCQ_FILE_RECORD WHERE upload_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        return baseNPJdbcTemplate.queryForBeans(FileDetailDto.class,sql,params);
    }

    public List<AuditProcessVo> queryProcess(Long uploadId) {
        String sql = "SELECT (SELECT worker_name FROM TSM_WORKER WHERE login_id = worker_id) workerName,role_id,status,reason_label,reason,create_time FROM YCQ_FILE_AUDIT_RECORD WHERE upload_id = ? ORDER BY create_time";
        return jdbcTemplate.queryForBeans(AuditProcessVo.class,sql,uploadId);
    }

    public String findRoleName(Integer roleId) {
        String sql = "SELECT role_name FROM TSM_ROLE WHERE role_id = ?";
        return jdbcTemplate.queryForRawObject(String.class,sql,roleId);
    }
}
