package com.crunii.micro.service.filemanager.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wangchen create 2023/8/8 15:20
 */
@Repository
public class FileOperationDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public List<Long> findFileIdsByLabelId(Long labelId) {
        String sql = "SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_id = ?";
        return jdbcTemplate.queryForRawObjects(Long.class,sql,labelId);
    }

    public List<Long> filterUploadIds(List<Long> uploadIds) {
        String sql = "SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE file_id IN (:ids) GROUP BY file_id HAVING  count(1) =1";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        return baseNpJdbcTemplate.queryForRawObjects(Long.class,sql,params);
    }

    public void delUploadInfoByUploadIds(List<Long> uploadIds) {
        String sql = "DELETE FROM YCQ_UPLOAD_INFO WHERE upload_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public List<String> findFileKeyByUploadIds(List<Long> uploadIds) {
        String sql = "SELECT file_key  FROM  YCQ_FILE_RECORD WHERE upload_id IN(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        return baseNpJdbcTemplate.queryForRawObjects(String.class,sql,params);
    }

    public void delFilRecordByUploadIds(List<Long> uploadIds) {
        String sql = "DELETE FROM YCQ_FILE_RECORD WHERE upload_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void delFileRecordByUploadIds(List<Long> uploadIds) {
        String sql = "DELETE FROM YCQ_FILE_LABEL_RELATION WHERE file_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void delFileAuditRoleByUploadIds(List<Long> uploadIds) {
        String sql = "DELETE FROM YCQ_FILE_AUDIT_ROLE WHERE upload_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void delFileAuditRecordByUploadIds(List<Long> uploadIds) {
        String sql = "DELETE FROM YCQ_FILE_AUDIT_RECORD WHERE upload_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void delFileRecordByUploadIdsAndLabelId(List<Long> uploadIds, Long labelId) {
        String sql = "DELETE FROM YCQ_FILE_LABEL_RELATION WHERE file_id in(:ids) and label_id = :labelId";
        Map<String,Object> params = new HashMap<>(4);
        params.put("ids",uploadIds);
        params.put("labelId",labelId);
        baseNpJdbcTemplate.update(sql,params);
    }
}
