package com.crunii.micro.service.filemanager.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.sys.FileDetailDto;
import com.crunii.micro.domain.dto.sys.PicInfoVo;
import com.crunii.micro.feign.client.dto.ElsticFileDto;
import com.crunii.micro.service.filemanager.dto.LabelUpload;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wangchen create 2023/6/16 14:37
 */
@Repository
public class FileUploadDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public List<PicInfoVo> queryHotSearch() {
        String sql = "SELECT upload_id,required_integral,push_time,describe,is_album,file_title,upload_worker_name FROM YCQ_UPLOAD_INFO WHERE state = 1 and audit_status = 1 AND number <> 0 ORDER BY number DESC LIMIT 12";
        return jdbcTemplate.queryForBeans(PicInfoVo.class,sql);
    }

    public List<LabelUpload> findLabelIdAndName(List<Long> uploadIds) {
        String sql = "SELECT file_id uploadId,label_id,label_name FROM YCQ_FILE_LABEL_RELATION WHERE file_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        return baseNpJdbcTemplate.queryForBeans(LabelUpload.class,sql,params);
    }

    public List<FileDetailDto> findPicDetail(List<Long> uploadIds) {
        String sql = "SELECT upload_id,file_name,file_record_id,file_type,pic_width,pic_height,is_pic,file_size,file_key FROM YCQ_FILE_RECORD WHERE upload_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        return baseNpJdbcTemplate.queryForBeans(FileDetailDto.class,sql,params);
    }

    public List<PicInfoVo> paddingLatelyUpload(List<Long> uploadIds) {
        StringBuilder sb = new StringBuilder("SELECT upload_id,required_integral,is_album,push_time,describe,file_title,upload_worker_name FROM YCQ_UPLOAD_INFO WHERE state=1 and audit_status = 1");
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        if(CollectionUtil.isNotEmpty(uploadIds)){
            sb.append(" and upload_id not in(:ids)");
            params.put("ids",uploadIds);
        }
        sb.append(" ORDER BY update_time DESC LIMIT 12");
        return baseNpJdbcTemplate.queryForBeans(PicInfoVo.class,sb.toString(),params);
    }

    public List<ElsticFileDto> findAllFiles() {
        String sql = "SELECT upload_id,is_album,file_title,required_integral,upload_worker_name,push_time,describe FROM YCQ_UPLOAD_INFO WHERE audit_status = 1 and state = 1";
        return jdbcTemplate.queryForBeans(ElsticFileDto.class,sql);
    }

    public List<ElsticFileDto> findElsticFiles(List<Long> uploadList) {
        String sql = "SELECT upload_id,is_album,file_title,required_integral,upload_worker_name,push_time,describe FROM YCQ_UPLOAD_INFO WHERE upload_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadList);
        return baseNpJdbcTemplate.queryForBeans(ElsticFileDto.class,sql,params);
    }
}
