package com.crunii.micro.service.filemanager.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.sys.FileDetailDto;
import com.crunii.micro.service.filemanager.dto.*;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/6/17 16:35
 */
@Repository
public class MyFileCenterDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public List<String> findUploadIdByLoginId(int loginId, List<Long> uploadId) {
        String sql = "SELECT upload_id FROM YCQ_UPLOAD_INFO WHERE upload_id in(:ids) AND upload_worker_id in(:loginId)";
        Map<String,Object> params = new HashMap<>(3);
        params.put("ids",uploadId);
        params.put("loginId",loginId);
        return baseNpJdbcTemplate.queryForRawObjects(String.class,sql,params);
    }

    public void updateUploadInfoByUploadId(List<Long> uploadId) {
        String sql = "UPDATE YCQ_UPLOAD_INFO SET state = 0 WHERE upload_id IN (:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadId);
        baseNpJdbcTemplate.update(sql,params);
    }

    public List<String> findLabelByUploadIds(List<Long> uploadId) {
        String sql = "SELECT label_id FROM YCQ_FILE_LABEL_RELATION WHERE file_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadId);
        return baseNpJdbcTemplate.queryForRawObjects(String.class,sql,params);
    }

    public PageRsp<MyFileVo> pageMyFile(int loginId,PageReq<PageMyFileDto> req) {
        StringBuilder sb = new StringBuilder("SELECT upload_id,is_album,audit_status,push_time,describe,required_integral,upload_worker_name,file_title FROM YCQ_UPLOAD_INFO WHERE upload_worker_id = ? and state = 1");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_UPLOAD_INFO WHERE upload_worker_id = ? and state = 1");
        List<Object> params = new ArrayList<>();
        PageMyFileDto query = req.getQuery();
        params.add(loginId);
        List<Integer> auditStatus = query.getAuditStatus();
        if(CollectionUtil.isNotEmpty(auditStatus)){
            StringJoiner sj = new StringJoiner(" or ");
            for (Integer status : auditStatus) {
                sj.add(" audit_status = ? ");
                params.add(status);
            }
            sb.append(" and (").append(sj).append(")");
            sbCount.append(" and (").append(sj).append(")");
        }
        if(StringUtils.hasText(query.getName())){
            montageSql(query,sb,sbCount,params);
        }

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<MyFileVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            sb.append(" order by update_time desc");
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(MyFileVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<MyFileVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    /**
     * 拼接sql处理
     */
    private void montageSql(PageMyFileDto query, StringBuilder sb, StringBuilder sbCount, List<Object> params) {
        if(query.getSearchMode() == 0){
            if(query.getMatchMode() == 0){
                sb.append(" AND (file_title LIKE concat('%',?,'%')  OR upload_id in(SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_name LIKE concat('%',?,'%')) )");
                sbCount.append(" AND (file_title LIKE concat('%',?,'%')  OR upload_id in(SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_name LIKE concat('%',?,'%')) )");
            }else {
                sb.append(" AND (file_title = ?  OR upload_id in(SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_name =? ))");
                sbCount.append(" AND （file_title = ?  OR upload_id in(SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_name = ? ))");
            }
            params.add(query.getName());
            params.add(query.getName());
        }

        if(query.getSearchMode() == 1){
            if(query.getMatchMode() == 0){
                sb.append(" AND file_title LIKE concat('%',?,'%')");
                sbCount.append(" AND file_title LIKE concat('%',?,'%')");
            }else {
                sb.append(" AND file_title = ?");
                sbCount.append(" AND file_title = ? ");
            }
            params.add(query.getName());
        }

        if(query.getSearchMode() == 2){
            if(query.getMatchMode() == 0){
                sb.append(" AND upload_id in(SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_name LIKE concat('%',?,'%') )");
                sbCount.append(" AND upload_id in(SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_name LIKE concat('%',?,'%') )");
            }else {
                sb.append(" AND upload_id in(SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_name =? )");
                sbCount.append(" AND upload_id in(SELECT file_id FROM YCQ_FILE_LABEL_RELATION WHERE label_name =? ) ");
            }
            params.add(query.getName());
        }
    }


    public List<LabelUpload> findLabelIdAndName(List<Long> uploadIds) {
        String sql = "SELECT file_id uploadId,label_id,label_name FROM YCQ_FILE_LABEL_RELATION WHERE file_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        return baseNpJdbcTemplate.queryForBeans(LabelUpload.class,sql,params);
    }

    public List<FileDetailDto> findPicDetail(List<Long> uploadIds) {
        String sql = "SELECT upload_id,file_record_id,file_name,file_type,pic_width,pic_height,is_pic,file_size,file_key FROM YCQ_FILE_RECORD WHERE upload_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",uploadIds);
        return baseNpJdbcTemplate.queryForBeans(FileDetailDto.class,sql,params);
    }

    public List<Long> findFileIdByUploadId(Long uploadId) {
        String sql = "SELECT file_record_id FROM YCQ_FILE_RECORD WHERE upload_id = ?";
        return jdbcTemplate.queryForRawObjects(Long.class,sql,uploadId);
    }

    public List<String> findKeys(List<Long> needDelIds) {
        String sql = "SELECT file_key FROM YCQ_FILE_RECORD WHERE file_record_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",needDelIds);
        return baseNpJdbcTemplate.queryForRawObjects(String.class,sql,params);
    }

    public void delFileByFileIds(List<Long> needDelIds) {
        String sql = "DELETE FROM YCQ_FILE_RECORD WHERE file_record_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",needDelIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public List<AuditProcessVo> queryProcess(Long uploadId) {
        String sql = "SELECT (SELECT worker_name FROM TSM_WORKER WHERE login_id = worker_id) workerName,role_id,status,reason_label,reason,create_time FROM YCQ_FILE_AUDIT_RECORD WHERE upload_id = ? ORDER BY create_time";
        return jdbcTemplate.queryForBeans(AuditProcessVo.class,sql,uploadId);
    }

    public String findRoleName(Integer roleId) {
        String sql = "SELECT role_name FROM TSM_ROLE WHERE role_id = ?";
        return jdbcTemplate.queryForRawObject(String.class,sql,roleId);
    }
}
