package com.crunii.micro.service.filemanager.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqUploadInfo;
import com.crunii.micro.service.filemanager.dto.*;
import com.crunii.micro.service.filemanager.statemachine.FileAuditEvent;

import java.util.List;

/**
 * @author shenhs create 2023/6/17 14:46
 */
public interface FileAuditService {

	/**
	 * 分页查询
	 * @param worker worker
	 * @param fileDelDto 分页参数
	 * @return data
	 */
	PageRsp<MyFileVo> pageAuditFile(Worker worker, PageReq<MyFileDto> fileDelDto);

	/**
	 * 审核通过
	 * @param worker worker
	 * @param pasDto req
	 * @return str
	 */
	String auditPass(Worker worker, AuditPasDto pasDto);

	/**
	 * 审核拒绝
	 * @param worker worker
	 * @param regDto req
	 * @return str
	 */
	String auditReg(Worker worker, AuditRegDto regDto);

	/**
	 * 查询审核流程
	 * @param worker worker
 	 * @param uploadId 上传id
	 * @return data
	 */
	List<AuditProcessVo> queryProcess(Worker worker, Long uploadId);

	/**
	 * 发送状态转换事件
	 * synchronized修饰保证这个方法是线程安全的
	 * @param changeEvent 改变事件
	 * @param uploadInfo 实体类
	 * @param worker worker
	 * @return true发送时间success
	 */
	boolean sendAuditEvent(FileAuditEvent changeEvent, YcqUploadInfo uploadInfo, Worker worker);

	/**
	 * 发送状态转换事件 新增同意备注/拒绝原因
	 * synchronized修饰保证这个方法是线程安全的
	 * @param changeEvent 改变事件
	 * @param uploadInfo 实体类
	 * @param worker worker
	 * @param reasonLabel 拒绝原因（静态值）“1，2，3”
	 * @param reason 拒绝原因
	 * @return true发送时间success
	 */
	boolean sendAuditEvent(FileAuditEvent changeEvent, YcqUploadInfo uploadInfo, Worker worker,String reasonLabel,String reason);

}
