package com.crunii.micro.service.filemanager.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.filemanager.dto.*;

import java.util.List;

/**
 * @author wangchen create 2023/6/17 16:22
 */
public interface MyFileCenterService {

    /**
     * 删除文件
     * @param loginId 登录人id
     * @param fileDelDto 上传ids
     * @return str
     */
    String deleteFile(int loginId, FileDelDto fileDelDto);

    /**
     * 保存已上传文件信息
     * @param worker worker
     * @param uploadInfoDto 文件信息
     * @return str
     */
    String uploadFile(Worker worker, UploadInfoDto uploadInfoDto);

    /**
     * 分页查询
     * @param loginId 当前登录人id
     * @param fileDelDto 查询条件
     * @return data
     */
    PageRsp<MyFileVo> pageMyFile(int loginId, PageReq<PageMyFileDto> fileDelDto);

    /**
     * 修改文件
     * @param worker worker
     * @param modifyInfoDto 修改内容
     * @return str
     */
    String updateMyFile(Worker worker, ModifyInfoDto modifyInfoDto);

    /**
     * 发起重审
     * @param worker worker
     * @param uploadId 上传id
     * @return str
     */
    String auditDismissal(Worker worker, Long uploadId);

    /**
     * 查询审核流程
     * @param worker worker
     * @param uploadId 上传id
     * @return data
     */
    List<AuditProcessVo> queryProcess(Worker worker, Long uploadId);
}
