package com.crunii.micro.service.filemanager.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.core.cache.TsmPropertyCache;
import com.crunii.micro.domain.dto.TsmRoleDto;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.sys.FileDetailDto;
import com.crunii.micro.domain.dto.sys.LabelCpDto;
import com.crunii.micro.domain.entity.main.TsmRole;
import com.crunii.micro.domain.entity.main.YcqUploadInfo;
import com.crunii.micro.domain.repo.main.YcqUploadInfoRepo;
import com.crunii.micro.feign.client.ElsticService;
import com.crunii.micro.feign.client.dto.ElsticFileDto;
import com.crunii.micro.service.filemanager.constants.FileConstants;
import com.crunii.micro.service.filemanager.dto.*;
import com.crunii.micro.service.filemanager.enums.RoleNameEnum;
import com.crunii.micro.service.filemanager.jdbc.FileAuditDao;
import com.crunii.micro.service.filemanager.service.FileAuditService;
import com.crunii.micro.service.filemanager.statemachine.FileAuditEvent;
import com.crunii.micro.service.filemanager.statemachine.FileEventListener;
import com.crunii.micro.service.filemanager.statemachine.FileStatus;
import jdk.nashorn.internal.ir.Labels;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author shenhs create 2023/6/17 14:46
 */
@Slf4j
@Service
public class FileAuditServiceImpl implements FileAuditService {

	@Resource
	private StateMachine<FileStatus, FileAuditEvent> fileStateMachine;
	@Resource
	private StateMachinePersister<FileStatus, FileAuditEvent, String> stateMachineMemPersister;
	@Autowired
	private YcqUploadInfoRepo ycqUploadInfoRepo;
	@Autowired
	private FileAuditDao fileAuditDao;
	@Resource
	private TsmPropertyCache tsmPropertyCache;
	@Autowired
	private ElsticService elsticService;
	@Value("${upload.url}")
	private String urlPrefix;

	@Override
	@InjectProperty
	public PageRsp<MyFileVo> pageAuditFile(Worker worker, PageReq<MyFileDto> fileDelDto) {
		boolean isSysRole = validatedRoleAndIsSysRole(worker);
		PageRsp<MyFileVo> rsp = fileAuditDao.pageAuditFile(fileDelDto,isSysRole,worker.getOrgId());
		List<MyFileVo> result = rsp.getResult();
		if(CollectionUtil.isEmpty(result)){
			return rsp;
		}
		List<Long> uploadIds = result.stream().map(MyFileVo::getUploadId).collect(Collectors.toList());
		List<LabelUpload> list = fileAuditDao.findLabelIdAndName(uploadIds);
		Map<Long, List<LabelUpload>> labelCollect = new HashMap<>(16);
		if(CollectionUtils.isNotEmpty(list)){
			labelCollect = list.stream().collect(Collectors.groupingBy(LabelUpload::getUploadId));
		}
		// 查询图片内容详细
		List<FileDetailDto> fileDetailDtoList = fileAuditDao.findPicDetail(uploadIds);
		fileDetailDtoList.forEach(x->x.setFileUrl(urlPrefix+x.getFileKey()));
		Map<Long, List<FileDetailDto>> picCollect = fileDetailDtoList.stream().collect(Collectors.groupingBy(FileDetailDto::getUploadId));
		// 填充
		for (MyFileVo myFileVo : result) {
			Long uploadId = myFileVo.getUploadId();
			// 查询该id对应标签设置
			List<LabelUpload> list1 = labelCollect.get(uploadId);
			List<LabelCpDto> Labels = new ArrayList<>();
			if(!CollectionUtil.isEmpty(list1)){
				for (LabelUpload labelUpload : list1) {
					LabelCpDto labelDto = new LabelCpDto();
					BeanUtils.copyProperties(labelUpload,labelDto);
					Labels.add(labelDto);
				}
			}
			myFileVo.setLabels(Labels);
			// 图片详情
			myFileVo.setFileDetailDtoList(picCollect.get(uploadId));
		}
		return rsp;
	}

	@Override
	public String auditPass(Worker worker, AuditPasDto pasDto) {
		Date now = new Date();
		boolean b = validatedRoleAndIsSysRole(worker);
		YcqUploadInfo one = ycqUploadInfoRepo.getOne(pasDto.getUploadId());
		if(Objects.isNull(one)){
			throw new BusinessException("无此id");
		}
		// true系统审核 false部门审核
		if(b){
			 sendAuditEvent(FileAuditEvent.SYSTEM_AUDIT_PAAS,one,worker,"",pasDto.getReason());
			// 系统审核后推送数据到es
			pushDataToEs(pasDto.getUploadId());
			// 保存审核通过时间
			one.setPushTime(now);
			ycqUploadInfoRepo.saveAndFlush(one);
		}else {
			sendAuditEvent(FileAuditEvent.DEPARTMENT_AUDIT_PAAS,one,worker,"",pasDto.getReason());
		}
		return "审核成功";
	}

	@Override
	public String auditReg(Worker worker, AuditRegDto regDto) {
		boolean b = validatedRoleAndIsSysRole(worker);
		YcqUploadInfo one = ycqUploadInfoRepo.getOne(regDto.getUploadId());
		if(Objects.isNull(one)){
			throw new BusinessException("无此id");
		}
		// true系统审核 false部门审核
		if(b){
			sendAuditEvent(FileAuditEvent.SYSTEM_AUDIT_REJECT,one,worker,regDto.getReasonLabel(),"");
		}else {
			sendAuditEvent(FileAuditEvent.DEPARTMENT_AUDIT_REJECT,one,worker,regDto.getReasonLabel(),"");
		}
		return "审核成功";
	}

	@Override
	public List<AuditProcessVo> queryProcess(Worker worker, Long uploadId) {
		validatedRoleAndIsSysRole(worker);
		if(Objects.isNull(uploadId)){
			throw new BusinessException("上传id不能为空");
		}
		List<AuditProcessVo> auditProcessVos = fileAuditDao.queryProcess(uploadId);
		for (AuditProcessVo auditProcessVo : auditProcessVos) {
			// 翻译拒绝审核原因
			String reasonLabel = auditProcessVo.getReasonLabel();
			if(StringUtils.hasText(reasonLabel)){
				String[] split = reasonLabel.split(",");
				StringJoiner sj = new StringJoiner(",");
				for (String s : split) {
					sj.add(tsmPropertyCache.get(FileConstants.REJECT_REASON_CODE,s));
				}
				auditProcessVo.setReasonLabel(sj.toString());
			}
			if(Objects.nonNull(auditProcessVo.getStatus())){
				String name;
				if(auditProcessVo.getStatus() == -1 || auditProcessVo.getStatus() == 0){
					name = RoleNameEnum.getName(auditProcessVo.getRoleId());
					name += tsmPropertyCache.get("fileStatus",auditProcessVo.getStatus()+"");
				}else {
					name = tsmPropertyCache.get("fileStatus",auditProcessVo.getStatus()+"");
				}
				auditProcessVo.setStatusTrans(name);
			}
			// 如果没有名字查角色
			if(!StringUtils.hasText(auditProcessVo.getWorkerName())){
				String roleName = fileAuditDao.findRoleName(auditProcessVo.getRoleId());
				auditProcessVo.setWorkerName(roleName);
			}
		}
		return auditProcessVos;
	}

	@Override
	public synchronized boolean sendAuditEvent(FileAuditEvent changeEvent, YcqUploadInfo uploadInfo, Worker worker) {
		boolean result = false;
		try {
			//启动状态机
			fileStateMachine.start();
			//尝试恢复状态机状态
			stateMachineMemPersister.restore(fileStateMachine, String.valueOf(uploadInfo.getUploadId()));
			Message message = MessageBuilder.withPayload(changeEvent).setHeader(FileEventListener.MESSAGE_KEY, uploadInfo).setHeader(
					FileEventListener.WORKER_KEY,worker).build();
			result = fileStateMachine.sendEvent(message);
			//持久化状态机状态
			stateMachineMemPersister.persist(fileStateMachine, String.valueOf(uploadInfo.getUploadId()));
		} catch (Exception e) {
			log.error("操作失败:{}", e);
		} finally {
			fileStateMachine.stop();
		}
		return result;
	}

	@Override
	public synchronized boolean sendAuditEvent(FileAuditEvent changeEvent, YcqUploadInfo uploadInfo, Worker worker,String reasonLabel,String reason) {
		boolean result = false;
		try {
			//启动状态机
			fileStateMachine.start();
			//尝试恢复状态机状态
			stateMachineMemPersister.restore(fileStateMachine, String.valueOf(uploadInfo.getUploadId()));
			Message message = MessageBuilder.withPayload(changeEvent).setHeader(FileEventListener.MESSAGE_KEY, uploadInfo).setHeader(
					FileEventListener.WORKER_KEY,worker).setHeader(FileEventListener.REASON_LABEL_KEY,reasonLabel).setHeader(FileEventListener.REASON_KEY,reason).build();
			result = fileStateMachine.sendEvent(message);
			//持久化状态机状态
			stateMachineMemPersister.persist(fileStateMachine, String.valueOf(uploadInfo.getUploadId()));
		} catch (Exception e) {
			log.error("操作失败:{}", e);
		} finally {
			fileStateMachine.stop();
		}
		return result;
	}

	/**
	 * 验证+判断角色
	 * @param worker worker
	 * @return true系统审核 false部门审核
	 */
	private boolean validatedRoleAndIsSysRole(Worker worker) {
		List<TsmRoleDto> roles = worker.getRoles();
		List<Integer> roleIds = roles.stream().map(TsmRoleDto::getRoleId).collect(Collectors.toList());
		// 判断是否部门审核人
		if(roleIds.contains(FileConstants.DEPARTMENT_AUDIT_PERSON)){
			return false;
		}
		// 判断是否系统审核
		if(roleIds.contains(FileConstants.SYS_AUDIT_PERSON)){
			return true;
		}
		if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())){
			return true;
		}
		throw new BusinessException("权限不足！");
	}

	/**
	 * 推送数据到es
	 * @param uploadId
	 */
	private void pushDataToEs(Long uploadId) {
		YcqUploadInfo one = ycqUploadInfoRepo.getOne(uploadId);
		ElsticFileDto elsticFileDto = new ElsticFileDto();
		elsticFileDto.setUploadId(one.getUploadId());
		elsticFileDto.setIsAlbum(one.getIsAlbum());
		elsticFileDto.setFileTitle(one.getFileTitle());
		elsticFileDto.setRequiredIntegral(one.getRequiredIntegral());
		elsticFileDto.setUploadWorkerName(one.getUploadWorkerName());
		elsticFileDto.setPushTime(one.getUploadTime());
		elsticFileDto.setDescribe(one.getDescribe());
		// 查询标签
		List<Long> uploadIds = new ArrayList<>();
		uploadIds.add(uploadId);
		List<LabelUpload> labelIdAndName = fileAuditDao.findLabelIdAndName(uploadIds);
		List<LabelCpDto> labels = new ArrayList<>();
		for (LabelUpload labelUpload : labelIdAndName) {
			LabelCpDto labelCpDto = new LabelCpDto();
			labelCpDto.setLabelId(labelUpload.getLabelId());
			labelCpDto.setLabelName(labelUpload.getLabelName());
			labels.add(labelCpDto);
		}
		elsticFileDto.setLabels(labels);
		// 查询图片详情
		List<FileDetailDto> picDetail = fileAuditDao.findPicDetail(uploadIds);
		elsticFileDto.setFileDetailDtoList(picDetail);
		elsticService.saveFile(elsticFileDto);
	}

}
