package com.crunii.micro.service.filemanager.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.domain.dto.sys.LabelCpDto;
import com.crunii.micro.feign.client.ElsticService;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.feign.client.dto.ElsticFileDto;
import com.crunii.micro.service.filemanager.jdbc.FileOperationDao;
import com.crunii.micro.service.filemanager.service.FileOperationService;
import com.crunii.micro.service.filemanager.service.FileUploadService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Iterator;
import java.util.List;

/**
 * @author wangchen create 2023/8/8 15:20
 */
@Service
public class FileOperationServiceImpl implements FileOperationService {

    @Resource
    private ElsticService elsticService;

    @Resource
    private ObsService obsService;

    private final FileUploadService fileUploadService;

    private final FileOperationDao fileOperationDao;

    public FileOperationServiceImpl(FileOperationDao fileOperationDao,FileUploadService fileUploadService) {
        this.fileOperationDao = fileOperationDao;
        this.fileUploadService = fileUploadService;
    }

    @Override
    public String delPicByLabelId(Long labelId) {
        List<Long> uploadIds = fileOperationDao.findFileIdsByLabelId(labelId);
        if(CollectionUtil.isEmpty(uploadIds)){
            return "删除成功";
        }
        // 查询当前文件是否只有这一个标签
        List<Long> onlyOneUploadIds = fileOperationDao.filterUploadIds(uploadIds);
        // 只有1个标签删除关联关系
        if(CollectionUtil.isNotEmpty(onlyOneUploadIds)){
            // 排除uploadId字段中只有一个标签的信息
            uploadIds.removeIf(onlyOneUploadIds::contains);
            // 如果只有这一个标签则删除
            List<String> fileKeys = delAssociationTable(onlyOneUploadIds);
            // 调用fegin删除es中文件信息
            for (Long uploadId : onlyOneUploadIds) {
                elsticService.delFile(String.valueOf(uploadId));
            }
            // 调用fegin删除obs中文件信息
            obsService.deleteListFileKey(fileKeys);
        }
        // 删除标签关联关系
        if(CollectionUtil.isNotEmpty(uploadIds)){
            List<ElsticFileDto> elsticFileDtos = fileUploadService.queryFileDetail(uploadIds);
            for (ElsticFileDto elsticFileDto : elsticFileDtos) {
                List<LabelCpDto> labels = elsticFileDto.getLabels();
                labels.removeIf(next -> next.getLabelId().equals(labelId+""));
            }
            elsticService.batchUpdate(elsticFileDtos);
           fileOperationDao.delFileRecordByUploadIdsAndLabelId(uploadIds,labelId);
        }
        return "删除成功";
    }

    /**
     * 删除关联表 返回filekey，调接口删除
     */
    private List<String> delAssociationTable(List<Long> uploadIds) {
        // 1 删除上传表
        fileOperationDao.delUploadInfoByUploadIds(uploadIds);
        // 2 删除文件信息表
        List<String> fileKeys = fileOperationDao.findFileKeyByUploadIds(uploadIds);
        fileOperationDao.delFilRecordByUploadIds(uploadIds);
        // 3 删除文件标签关联表
        fileOperationDao.delFileRecordByUploadIds(uploadIds);
        // 4 删除文件审核相关日志表
        fileOperationDao.delFileAuditRoleByUploadIds(uploadIds);
        fileOperationDao.delFileAuditRecordByUploadIds(uploadIds);
        return fileKeys;
    }
}
