package com.crunii.micro.service.filemanager.service.impl;
import cn.hutool.json.JSONUtil;
import com.google.common.collect.Lists;
import java.util.Date;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.sys.FileDetailDto;
import com.crunii.micro.domain.dto.sys.LabelCpDto;
import com.crunii.micro.domain.dto.sys.PicInfoVo;
import com.crunii.micro.feign.client.ElsticService;
import com.crunii.micro.feign.client.dto.ElsticFileDto;
import com.crunii.micro.feign.client.dto.FileElsticPageDto;
import com.crunii.micro.service.filemanager.constants.FileConstants;
import com.crunii.micro.service.filemanager.dto.*;
import com.crunii.micro.service.filemanager.jdbc.FileUploadDao;
import com.crunii.micro.service.filemanager.service.FileUploadService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author wangchen create 2023/6/16 14:37
 */
@Service
public class FileUploadServiceImpl implements FileUploadService {

    @Value("${upload.url}")
    private String urlPrefix;
    @Resource
    private ElsticService elsticService;

    @Resource
    private RedisTemplate<String, List<PicInfoVo>> redisTemplate;

    private final FileUploadDao fileUploadDao;

    public FileUploadServiceImpl(FileUploadDao fileUploadDao) {
        this.fileUploadDao = fileUploadDao;
    }

    @Override
    public FileExhibition queryFileExhibition() {
        FileExhibition fileExhibition = new FileExhibition();
        // 查询热门搜索
        List<PicInfoVo> list = redisTemplate.opsForValue().get(RedisKeys.HOT_FILE_SEARCH);
        if(CollectionUtil.isEmpty(list)){
            list = queryHotSearch();
        }
        fileExhibition.setHotSearchVos(list);
        List<PicInfoVo> latestUploadVos = paddingLatelyUpload(list);
        fileExhibition.setLatestUploadVos(latestUploadVos);
        return fileExhibition;
    }

    /**
     * 填充最近上传
     * @param list 最近上传数据
     * @return data
     */
    private List<PicInfoVo> paddingLatelyUpload(List<PicInfoVo> list) {
        List<Long> uploadIds = new ArrayList<>();
        if(CollectionUtil.isNotEmpty(list)){
            uploadIds = list.stream().map(PicInfoVo::getUploadId).collect(Collectors.toList());
        }
        List<PicInfoVo> latelyUploadVos = fileUploadDao.paddingLatelyUpload(uploadIds);
        if(CollectionUtil.isEmpty(latelyUploadVos)){
            return new ArrayList<>();
        }
        List<Long> uploadList = latelyUploadVos.stream().map(PicInfoVo::getUploadId).collect(Collectors.toList());
        // 查询标签
        List<LabelUpload> labelUploads = fileUploadDao.findLabelIdAndName(uploadList);
        Map<Long, List<LabelUpload>> labelCollect = new HashMap<>(16);
        if(CollectionUtils.isNotEmpty(labelUploads)){
            labelCollect = labelUploads.stream().collect(Collectors.groupingBy(LabelUpload::getUploadId));
        }
        // 查询图片内容详细
        List<FileDetailDto> fileDetailDtoList = fileUploadDao.findPicDetail(uploadList);
        fileDetailDtoList.forEach(x->x.setFileUrl(urlPrefix+x.getFileKey()));
        Map<Long, List<FileDetailDto>> picCollect = fileDetailDtoList.stream().collect(Collectors.groupingBy(FileDetailDto::getUploadId));
        // 填充
        for (PicInfoVo latelyUploadVo : latelyUploadVos) {
            Long uploadId = latelyUploadVo.getUploadId();
            // 查询该id对应标签设置
            List<LabelUpload> list1 = labelCollect.get(latelyUploadVo.getUploadId());
            List<LabelCpDto> labelCpDtoList = new ArrayList<>();
            // 异常情况 无标签
            if(!CollectionUtil.isEmpty(list1)){
                for (LabelUpload labelUpload : list1) {
                    LabelCpDto labelDto = new LabelCpDto();
                    BeanUtils.copyProperties(labelUpload,labelDto);
                    labelCpDtoList.add(labelDto);
                }
            }
            latelyUploadVo.setLabels(labelCpDtoList);
            // 图片详情
            latelyUploadVo.setFileDetailDtoList(picCollect.get(uploadId));
        }
        return latelyUploadVos;
    }


    @Override
    public List<PicInfoVo> queryHotSearch() {
        List<PicInfoVo> hotSearchVos = fileUploadDao.queryHotSearch();
        if(CollectionUtil.isEmpty(hotSearchVos)){
            return new ArrayList<>();
        }
        List<Long> uploadIds = hotSearchVos.stream().map(PicInfoVo::getUploadId).collect(Collectors.toList());
        // 查询标签
        List<LabelUpload> list = fileUploadDao.findLabelIdAndName(uploadIds);
        Map<Long, List<LabelUpload>> labelCollect = new HashMap<>(16);
        if(CollectionUtils.isNotEmpty(list)){
            labelCollect = list.stream().collect(Collectors.groupingBy(LabelUpload::getUploadId));
        }
        // 查询图片内容详细
        List<FileDetailDto> fileDetailDtoList = fileUploadDao.findPicDetail(uploadIds);
        fileDetailDtoList.forEach(x->x.setFileUrl(urlPrefix+x.getFileKey()));
        Map<Long, List<FileDetailDto>> picCollect = fileDetailDtoList.stream().collect(Collectors.groupingBy(FileDetailDto::getUploadId));
        // 填充
        for (PicInfoVo hotSearchVo : hotSearchVos) {
            Long uploadId = hotSearchVo.getUploadId();
            // 查询该id对应标签设置
            List<LabelUpload> list1 = labelCollect.get(hotSearchVo.getUploadId());
            List<LabelCpDto> labelCpDtoList = new ArrayList<>();
            for (LabelUpload labelUpload : list1) {
                LabelCpDto labelDto = new LabelCpDto();
                BeanUtils.copyProperties(labelUpload,labelDto);
                labelCpDtoList.add(labelDto);
            }
            hotSearchVo.setLabels(labelCpDtoList);
            // 图片详情
            hotSearchVo.setFileDetailDtoList(picCollect.get(uploadId));
        }
        redisTemplate.opsForValue().set(RedisKeys.HOT_FILE_SEARCH,hotSearchVos, FileConstants.HOT_SEARCH_TIME, TimeUnit.MINUTES);
        return hotSearchVos;
    }

    @Override
    public List<ElsticFileDto> queryFileDetail(List<Long> uploadList) {
        // 查询标签
        List<LabelUpload> labelUploads = fileUploadDao.findLabelIdAndName(uploadList);
        Map<Long, List<LabelUpload>> labelCollect = new HashMap<>(16);
        if(CollectionUtils.isNotEmpty(labelUploads)){
            labelCollect = labelUploads.stream().collect(Collectors.groupingBy(LabelUpload::getUploadId));
        }
        List<ElsticFileDto> ElsticFileDtos = fileUploadDao.findElsticFiles(uploadList);
        // 查询图片内容详细
        List<FileDetailDto> fileDetailDtoList = fileUploadDao.findPicDetail(uploadList);
        fileDetailDtoList.forEach(x->x.setFileUrl(urlPrefix+x.getFileKey()));
        Map<Long, List<FileDetailDto>> picCollect = fileDetailDtoList.stream().collect(Collectors.groupingBy(FileDetailDto::getUploadId));
        // 填充
        for (ElsticFileDto elsticFileDto : ElsticFileDtos) {
            Long uploadId = elsticFileDto.getUploadId();
            // 查询该id对应标签设置
            List<LabelUpload> list1 = labelCollect.get(elsticFileDto.getUploadId());
            List<LabelCpDto> labelCpDtoList = new ArrayList<>();
            // 异常情况 无标签
            if(!CollectionUtil.isEmpty(list1)){
                for (LabelUpload labelUpload : list1) {
                    LabelCpDto labelDto = new LabelCpDto();
                    BeanUtils.copyProperties(labelUpload,labelDto);
                    labelCpDtoList.add(labelDto);
                }
            }
            elsticFileDto.setLabels(labelCpDtoList);
            // 图片详情
            elsticFileDto.setFileDetailDtoList(picCollect.get(uploadId));
        }
        return ElsticFileDtos;
    }

    @Override
    public Result<PageRsp<ElsticFileDto>> keywordSearch(PageReq<FileElsticPageDto> keyword) {
        // 查询
        Result<PageRsp<ElsticFileDto>> query = elsticService.query(keyword);
        if(!query.isSuccess()){
            return Result.ofSuccess(new PageRsp<>());
        }
        PageRsp<ElsticFileDto> data = query.getData();
        List<ElsticFileDto> result = data.getResult();
        if(CollectionUtil.isEmpty(result)){
            return query;
        }
        for (ElsticFileDto elsticFileDto : result) {
            List<FileDetailDto> fileDetailDtoList = elsticFileDto.getFileDetailDtoList();
            for (FileDetailDto fileDetailDto : fileDetailDtoList) {
                fileDetailDto.setFileUrl(urlPrefix+fileDetailDto.getFileKey());
            }
        }
        return query;
    }

    @Override
    public void synEsFile(String code) {
        if(!"gongqingtuanfile".equals(code)){
            return;
        }
        List<ElsticFileDto> fileDto = fileUploadDao.findAllFiles();
        if(CollectionUtils.isEmpty(fileDto)){
            return;
        }
        List<Long> uploadIds = fileDto.stream().map(ElsticFileDto::getUploadId).collect(Collectors.toList());
        // 查询标签
        List<LabelUpload> list = fileUploadDao.findLabelIdAndName(uploadIds);
        Map<Long, List<LabelUpload>> labelCollect = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(list)) {
            labelCollect = list.stream().collect(Collectors.groupingBy(LabelUpload::getUploadId));
        }
        // 查询数据
        List<FileDetailDto> fileDetailDtoList = fileUploadDao.findPicDetail(uploadIds);
        fileDetailDtoList.forEach(x -> x.setFileUrl(urlPrefix + x.getFileKey()));
        Map<Long, List<FileDetailDto>> collect = fileDetailDtoList.stream().collect(Collectors.groupingBy(FileDetailDto::getUploadId));
        for (ElsticFileDto elsticFileDto : fileDto) {
            List<LabelUpload> labelUploads = labelCollect.get(elsticFileDto.getUploadId());
            List<LabelCpDto> labelCpDtoList = new ArrayList<>();
            for (LabelUpload labelUpload : labelUploads) {
                LabelCpDto labelCpDto = new LabelCpDto();
                labelCpDto.setLabelId(labelUpload.getLabelId());
                labelCpDto.setLabelName(labelUpload.getLabelName());
                labelCpDtoList.add(labelCpDto);
            }
            elsticFileDto.setLabels(labelCpDtoList);
            elsticFileDto.setFileDetailDtoList(collect.get(elsticFileDto.getUploadId()));
        }
        elsticService.batchSaveFile(fileDto);
        System.out.println("sADas");
    }
}
