package com.crunii.micro.service.filemanager.service.impl;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.domain.dto.TsmRoleDto;
import com.crunii.micro.domain.dto.obs.ObsServiceCodeDto;
import com.google.common.collect.Lists;
import java.util.Date;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.StopWatch;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.core.cache.TsmPropertyCache;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.label.UseCountDto;
import com.crunii.micro.domain.dto.sys.FileDetailDto;
import com.crunii.micro.domain.dto.sys.LabelCpDto;
import com.crunii.micro.domain.entity.main.TsmRole;
import com.crunii.micro.domain.entity.main.YcqFileLabelRelation;
import com.crunii.micro.domain.entity.main.YcqFileRecord;
import com.crunii.micro.domain.entity.main.YcqUploadInfo;
import com.crunii.micro.domain.repo.main.YcqFileLabelRelationRepo;
import com.crunii.micro.domain.repo.main.YcqFileRecordRepo;
import com.crunii.micro.domain.repo.main.YcqUploadInfoRepo;
import com.crunii.micro.feign.client.ElsticService;
import com.crunii.micro.feign.client.LabelService;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.feign.client.dto.ElsticFileDto;
import com.crunii.micro.feign.client.dto.LabelDto;
import com.crunii.micro.service.filemanager.constants.FileConstants;
import com.crunii.micro.service.filemanager.dto.*;
import com.crunii.micro.service.filemanager.enums.RoleNameEnum;
import com.crunii.micro.service.filemanager.jdbc.MyFileCenterDao;
import com.crunii.micro.service.filemanager.service.FileAuditService;
import com.crunii.micro.service.filemanager.service.MyFileCenterService;
import com.crunii.micro.service.filemanager.statemachine.FileAuditEvent;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wangchen create 2023/6/17 16:22
 */
@Service
public class MyFileCenterServiceImpl implements MyFileCenterService {

    @Resource
    private YcqUploadInfoRepo ycqUploadInfoRepo;
    @Resource
    private YcqFileRecordRepo ycqFileRecordRepo;
    @Resource
    private YcqFileLabelRelationRepo ycqFileLabelRelationRepo;
    @Resource
    private LabelService labelService;
    @Resource
    private ObsService obsService;
    @Resource
    private TsmPropertyCache tsmPropertyCache;
    @Resource
    private ElsticService elsticService;

    @Value("${upload.url}")
    private String urlPrefix;


    private final MyFileCenterDao myFileCenterDao;
    private final FileAuditService fileAuditService;

    public MyFileCenterServiceImpl(MyFileCenterDao myFileCenterDao,FileAuditService fileAuditService) {
        this.myFileCenterDao = myFileCenterDao;
        this.fileAuditService = fileAuditService;
    }

    @Override
    public String uploadFile(Worker worker, UploadInfoDto uploadInfoDto) {
        boolean b = checkUpLoadRole(worker);
        YcqUploadInfo ycqUploadInfo = svUploadInfo(uploadInfoDto, worker);
        Long uploadId = ycqUploadInfo.getUploadId();
        List<FileDetailDto> fileDetailDtoList = uploadInfoDto.getFileDetailDtoList();
        List<ObsServiceCodeDto> obsServiceCodeDtos = new ArrayList<>();
        for (int i = 0; i < fileDetailDtoList.size(); i++) {
            FileDetailDto fileDetailDto = fileDetailDtoList.get(i);
            YcqFileRecord fileRecord = new YcqFileRecord();
            BeanUtils.copyProperties(fileDetailDto, fileRecord);
            fileRecord.setUploadId(uploadId);
            fileRecord.setFileKey(fileRecord.getFileKey());
            fileRecord.setSort(i);
            YcqFileRecord ycqFileRecord = ycqFileRecordRepo.saveAndFlush(fileRecord);
            // 标记obsFile的业务编码
            ObsServiceCodeDto obsServiceCodeDto = new ObsServiceCodeDto();
            obsServiceCodeDto.setFileId(fileDetailDto.getFileId());
            obsServiceCodeDto.setServiceId(ycqFileRecord.getFileRecordId());
            obsServiceCodeDto.setServiceCode(CommonConstants.OBS_FILE_PIC_YOUTH_CODE);
            obsServiceCodeDtos.add(obsServiceCodeDto);
        }
        obsService.updateObsServiceCode(obsServiceCodeDtos);
        // 保存标签
        List<YcqFileLabelRelation> ycqFileLabelRelations = new ArrayList<>();
        List<LabelDto> labels = uploadInfoDto.getLabels();
        for (LabelDto label : labels) {
            YcqFileLabelRelation ycqFileLabelRelation = new YcqFileLabelRelation();
            ycqFileLabelRelation.setFileId(uploadId);
            ycqFileLabelRelation.setLabelId(label.getLabelId());
            ycqFileLabelRelation.setLabelName(label.getLabelName());
            ycqFileLabelRelations.add(ycqFileLabelRelation);
        }
        ycqFileLabelRelationRepo.saveAllAndFlush(ycqFileLabelRelations);
        // 保存标签引用次数
        saveLabelCount(labels);
        // 更改审核状态
        fileAuditService.sendAuditEvent(FileAuditEvent.UPLOAD,ycqUploadInfo,worker);
        // 如果是系统管理员，直接展示
        if(b){
            fileAuditService.sendAuditEvent(FileAuditEvent.SYSTEM_AUDIT_PAAS,ycqUploadInfo,worker);
            Date date = new Date();
            ycqUploadInfo.setUploadTime(date);
            ycqUploadInfo.setPushTime(date);
            // 推送到es
            ElsticFileDto elsticFileDto = getElsticFileDto(ycqUploadInfo,labels,fileDetailDtoList);
            elsticService.saveFile(elsticFileDto);
            ycqUploadInfoRepo.saveAndFlush(ycqUploadInfo);
        }
        return "上传成功";
    }

    /**
     * 封装es请求参数
     */
    private ElsticFileDto getElsticFileDto(YcqUploadInfo ycqUploadInfo,List<LabelDto> labels,List<FileDetailDto> fileDetailDtoList) {
        ElsticFileDto elsticFileDto = new ElsticFileDto();
        elsticFileDto.setUploadId(ycqUploadInfo.getUploadId());
        elsticFileDto.setIsAlbum(ycqUploadInfo.getIsAlbum());
        elsticFileDto.setFileTitle(ycqUploadInfo.getFileTitle());
        elsticFileDto.setRequiredIntegral(ycqUploadInfo.getRequiredIntegral());
        elsticFileDto.setUploadWorkerName(ycqUploadInfo.getUploadWorkerName());
        elsticFileDto.setPushTime(ycqUploadInfo.getPushTime());
        elsticFileDto.setDescribe(ycqUploadInfo.getDescribe());
        List<LabelCpDto> labelCpDtos = new ArrayList<>();
        for (LabelDto label : labels) {
            LabelCpDto labelCpDto = new LabelCpDto();
            labelCpDto.setLabelId(label.getLabelId());
            labelCpDto.setLabelName(label.getLabelName());
            labelCpDtos.add(labelCpDto);
        }
        elsticFileDto.setLabels(labelCpDtos);
        elsticFileDto.setFileDetailDtoList(fileDetailDtoList);
        return elsticFileDto;
    }

    @Override
    @InjectProperty
    public PageRsp<MyFileVo> pageMyFile(int loginId, PageReq<PageMyFileDto> fileDelDto) {
        PageRsp<MyFileVo> pageMyFile = myFileCenterDao.pageMyFile(loginId, fileDelDto);
        List<MyFileVo> result = pageMyFile.getResult();
        if (CollectionUtil.isEmpty(result)) {
            return pageMyFile;
        }
        List<Long> uploadIds = result.stream().map(MyFileVo::getUploadId).collect(Collectors.toList());
        Map<Long, List<LabelUpload>> labels = findLabelByUploadIds(uploadIds);
        Map<Long, List<FileDetailDto>> picCollect = findDataByUploadIds(uploadIds);
        for (MyFileVo myFileVo : result) {
            myFileVo.setFileDetailDtoList(picCollect.get(myFileVo.getUploadId()));
            List<LabelCpDto> labelList = new ArrayList<>();
            List<LabelUpload> labelUploads = labels.get(myFileVo.getUploadId());
            // 没用标签  异常
            if(!CollectionUtil.isEmpty(labelUploads)){
                for (LabelUpload labelUpload : labelUploads) {
                    LabelCpDto labelCpDto = new LabelCpDto();
                    BeanUtils.copyProperties(labelUpload, labelCpDto);
                    labelList.add(labelCpDto);
                }
            }
            myFileVo.setLabels(labelList);
        }
        return pageMyFile;
    }

    @Override
    public String updateMyFile(Worker worker, ModifyInfoDto modifyInfoDto) {
        Date now = new Date();
        Long uploadId = modifyInfoDto.getUploadId();
        List<Long> list = new ArrayList<>();
        list.add(uploadId);
        validatedIsMyUpload(worker.getLoginId(), list);
        // 删除&&保存标签与上传id的关联关系
        List<LabelDto> labels = modifyInfoDto.getLabels();
        saveAndDelLabelInfo(labels, uploadId);
        // 文件处理
        List<FileDetailDto> fileDetailDtoList = modifyInfoDto.getFileDetailDtoList();
        saveAndDelPicInfo(fileDetailDtoList, uploadId);

        YcqUploadInfo one = ycqUploadInfoRepo.getOne(uploadId);
        one.setIsAlbum(modifyInfoDto.getIsAlbum());
        one.setImgNum(modifyInfoDto.getImgNum());
        one.setDescribe(modifyInfoDto.getDescribe());
        one.setFileTitle(modifyInfoDto.getFileTitle());
        one.setUpdateTime(now);
        one.setUpdateWorkerId(worker.getLoginId());
        // 如果此文件是已上架，则发起重审
        if(one.getAuditStatus().equals(1)){
            boolean b = checkUpLoadRole(worker);
            // 如果是系统管理员，无需审核
            if(b){
                fileAuditService.sendAuditEvent(FileAuditEvent.SYSTEM_AUDIT_PAAS,one,worker);
                Date date = new Date();
                one.setPushTime(date);
                // 推送到es
                ElsticFileDto elsticFileDto = getElsticFileDto(one,labels,fileDetailDtoList);
                elsticService.saveFile(elsticFileDto);
                ycqUploadInfoRepo.saveAndFlush(one);
            }else {
                fileAuditService.sendAuditEvent(FileAuditEvent.RE_PROCESS,one,worker);
                // 修改审核通过时间为空
                one.setPushTime(null);
                // 删除es数据
                elsticService.delFile(one.getUploadId().toString());
            }
        }
        ycqUploadInfoRepo.saveAndFlush(one);
        return "更新成功";
    }

    @Override
    public String auditDismissal(Worker worker, Long uploadId) {
        if(Objects.isNull(uploadId)){
            throw new BusinessException("上传id不能为空");
        }
        YcqUploadInfo one = ycqUploadInfoRepo.getOne(uploadId);
        fileAuditService.sendAuditEvent(FileAuditEvent.RE_AUDIT,one,worker);
        return "重审成功";
    }

    @Override
    public List<AuditProcessVo> queryProcess(Worker worker, Long uploadId) {
        List<Long> uploadIds = new ArrayList<>();
        uploadIds.add(uploadId);
        validatedIsMyUpload(worker.getLoginId(),uploadIds);
        List<AuditProcessVo> auditProcessVos = myFileCenterDao.queryProcess(uploadId);
        // 翻译拒绝审核原因
        for (AuditProcessVo auditProcessVo : auditProcessVos) {
            // 翻译拒绝审核原因
            String reasonLabel = auditProcessVo.getReasonLabel();
            if(StringUtils.hasText(reasonLabel)){
                String[] split = reasonLabel.split(",");
                StringJoiner sj = new StringJoiner(",");
                for (String s : split) {
                    sj.add(tsmPropertyCache.get(FileConstants.REJECT_REASON_CODE,s));
                }
                auditProcessVo.setReasonLabel(sj.toString());
            }
            // 如果没有名字查角色
            if(!StringUtils.hasText(auditProcessVo.getWorkerName())){
               String roleName = myFileCenterDao.findRoleName(auditProcessVo.getRoleId());
                auditProcessVo.setWorkerName(roleName);
            }
            if(Objects.nonNull(auditProcessVo.getStatus())){
                String name;
                if(auditProcessVo.getStatus() == -1 || auditProcessVo.getStatus() == 0){
                    name = RoleNameEnum.getName(auditProcessVo.getRoleId());
                    name += tsmPropertyCache.get("fileStatus",auditProcessVo.getStatus()+"");
                }else {
                    name = tsmPropertyCache.get("fileStatus",auditProcessVo.getStatus()+"");
                }
                auditProcessVo.setStatusTrans(name);
            }
        }
        return auditProcessVos;
    }

    /**
     * 文件新增删除处理
     *
     * @param fileDetailDtoList 文件信息
     * @param uploadId          上传id
     */
    private void saveAndDelPicInfo(List<FileDetailDto> fileDetailDtoList, Long uploadId) {
        // 查询不为空的文件id
        List<Long> hasIds = fileDetailDtoList.stream().map(FileDetailDto::getFileRecordId).filter(Objects::nonNull).collect(Collectors.toList());
        if(CollectionUtils.isNotEmpty(hasIds)){
            List<Long> fileIds = myFileCenterDao.findFileIdByUploadId(uploadId);
            List<Long> needDelIds = fileIds.stream().filter(x -> !hasIds.contains(x)).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(needDelIds)) {
                List<String> keys = myFileCenterDao.findKeys(needDelIds);
                obsService.deleteListFileKey(keys);
            }
            // 删除之前图片
            myFileCenterDao.delFileByFileIds(fileIds);
        }
        List<ObsServiceCodeDto> obsServiceCodeDtos = new ArrayList<>();
        for (int i = 0; i < fileDetailDtoList.size(); i++) {
            FileDetailDto fileDetailDto = fileDetailDtoList.get(i);
            YcqFileRecord ycqFileRecord = new YcqFileRecord();
            ycqFileRecord.setFileRecordId(fileDetailDto.getFileRecordId());
            ycqFileRecord.setUploadId(uploadId);
            ycqFileRecord.setFileName(fileDetailDto.getFileName());
            ycqFileRecord.setFileType(fileDetailDto.getFileType());
            ycqFileRecord.setFileKey(fileDetailDto.getFileKey());
            ycqFileRecord.setPicWidth(fileDetailDto.getPicWidth());
            ycqFileRecord.setPicHeight(fileDetailDto.getPicHeight());
            ycqFileRecord.setIsPic(fileDetailDto.getIsPic());
            ycqFileRecord.setFileSize(fileDetailDto.getFileSize());
            ycqFileRecord.setSort(i);
            YcqFileRecord ycqFileRecord1 = ycqFileRecordRepo.saveAndFlush(ycqFileRecord);
            ObsServiceCodeDto obsServiceCodeDto = new ObsServiceCodeDto();
            obsServiceCodeDto.setFileId(fileDetailDto.getFileId());
            obsServiceCodeDto.setServiceId(ycqFileRecord1.getFileRecordId());
            obsServiceCodeDto.setServiceCode(CommonConstants.OBS_FILE_PIC_YOUTH_CODE);
            obsServiceCodeDtos.add(obsServiceCodeDto);
        }
        obsService.updateObsServiceCode(obsServiceCodeDtos);
    }


    /**
     * 删除之前标签以及标签引用，保存现在标签以及引用
     *
     * @param labels   标签
     * @param uploadId 上传id
     */
    private void saveAndDelLabelInfo(List<LabelDto> labels, Long uploadId) {
        List<Long> longList = new ArrayList<>();
        longList.add(uploadId);
        List<UseCountDto> useCountDtos = new ArrayList<>();
        List<String> labelByUploadIds = myFileCenterDao.findLabelByUploadIds(longList);
        ycqFileLabelRelationRepo.deleteAllByFileId(uploadId);
        // 删除引用次数
        for (String labelByUploadId : labelByUploadIds) {
            UseCountDto useCountDto = new UseCountDto();
            useCountDto.setMark("0");
            useCountDto.setLabelId(labelByUploadId);
            useCountDtos.add(useCountDto);
        }
        List<YcqFileLabelRelation> list = new ArrayList<>();
        for (LabelDto label : labels) {
            YcqFileLabelRelation ycqFileLabelRelation = new YcqFileLabelRelation();
            ycqFileLabelRelation.setFileId(uploadId);
            ycqFileLabelRelation.setLabelId(label.getLabelId());
            ycqFileLabelRelation.setLabelName(label.getLabelName());
            list.add(ycqFileLabelRelation);
            UseCountDto useCountDto = new UseCountDto();
            useCountDto.setMark("1");
            useCountDto.setLabelId(label.getLabelId());
            useCountDtos.add(useCountDto);
        }
        if(!CollectionUtil.isEmpty(useCountDtos)){
            labelService.labelUseCount(useCountDtos);
        }
        // 保存
        ycqFileLabelRelationRepo.saveAll(list);
        ycqFileLabelRelationRepo.flush();
    }

    private Map<Long, List<FileDetailDto>> findDataByUploadIds(List<Long> uploadIds) {
        List<FileDetailDto> fileDetailDtoList = myFileCenterDao.findPicDetail(uploadIds);
        fileDetailDtoList.forEach(x -> x.setFileUrl(urlPrefix + x.getFileKey()));
        return fileDetailDtoList.stream().collect(Collectors.groupingBy(FileDetailDto::getUploadId));
    }

    /**
     * 通过id查询标签
     */
    private Map<Long, List<LabelUpload>> findLabelByUploadIds(List<Long> uploadIds) {
        // 查询标签
        List<LabelUpload> list = myFileCenterDao.findLabelIdAndName(uploadIds);
        Map<Long, List<LabelUpload>> labelCollect = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(list)) {
            labelCollect = list.stream().collect(Collectors.groupingBy(LabelUpload::getUploadId));
        }
        return labelCollect;
    }

    @Override
    public String deleteFile(int loginId, FileDelDto fileDelDto) {
        List<Long> uploadId = fileDelDto.getUploadId();
        validatedIsMyUpload(loginId, uploadId);
        // 下面的文件信息
        myFileCenterDao.updateUploadInfoByUploadId(uploadId);
        // 标签信息
        List<String> labels = myFileCenterDao.findLabelByUploadIds(uploadId);
        List<UseCountDto> useCountList = new ArrayList<>();
        for (String label : labels) {
            UseCountDto useCountDto = new UseCountDto();
            useCountDto.setLabelId(label);
            useCountDto.setMark("0");
            useCountList.add(useCountDto);
        }
        labelService.labelUseCount(useCountList);
        for (Long aLong : uploadId) {
            elsticService.delFile(aLong+"");
        }
        return "删除成功";
    }

    /**
     * 校验是否本人上传文件
     */
    private void validatedIsMyUpload(int loginId, List<Long> uploadId) {
        if (CollectionUtil.isEmpty(uploadId)) {
            throw new BusinessException("上传id不能为空");
        }
        List<String> uploadIds = myFileCenterDao.findUploadIdByLoginId(loginId, uploadId);
        // 查出来不一致证明 有数据不是本人上传
        if (uploadIds.size() < uploadId.size()) {
            throw new BusinessException("不可操作非本人操作");
        }
    }

    /**
     * 保存标签引用次数
     */
    private void saveLabelCount(List<LabelDto> labels) {
        List<UseCountDto> list = new ArrayList<>();
        for (LabelDto label : labels) {
            UseCountDto useCountDto = new UseCountDto();
            useCountDto.setMark("1");
            useCountDto.setLabelId(label.getLabelId());
            list.add(useCountDto);
        }
        labelService.labelUseCount(list);
    }

    /**
     * 封装实体并保存
     */
    private YcqUploadInfo svUploadInfo(UploadInfoDto uploadInfoDto, Worker worker) {
        Date now = new Date();
        YcqUploadInfo ycqUploadInfo = new YcqUploadInfo();
        ycqUploadInfo.setUploadWorkerId(worker.getLoginId());
        // 下载所需积分未定
        ycqUploadInfo.setRequiredIntegral(0);
        ycqUploadInfo.setOrgId(worker.getOrgId());
        ycqUploadInfo.setNumber(0);
        ycqUploadInfo.setIsAlbum(uploadInfoDto.getIsAlbum());
        ycqUploadInfo.setImgNum(uploadInfoDto.getImgNum());
        ycqUploadInfo.setUploadTime(now);
        ycqUploadInfo.setAuditStatus(-1);
        ycqUploadInfo.setUpdateTime(now);
        ycqUploadInfo.setUpdateWorkerId(worker.getLoginId());
        ycqUploadInfo.setDescribe(uploadInfoDto.getDescribe());
        ycqUploadInfo.setState(1);
        ycqUploadInfo.setUploadWorkerName(worker.getWorkerName());
        ycqUploadInfo.setFileTitle(uploadInfoDto.getFileTitle());
        return ycqUploadInfoRepo.saveAndFlush(ycqUploadInfo);
    }

    /**  todo 超管权限（暂时设置）
     * 检查该用户是否有上传角色 && 是否系统审核员
     */
    private boolean checkUpLoadRole(Worker worker) {
        if(CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())){
            return true;
        }
        // 判断角色是否上传员
        List<TsmRoleDto> roles = worker.getRoles();
        List<Integer> roleIds = roles.stream().map(TsmRoleDto::getRoleId).collect(Collectors.toList());
        Integer authRoleId = FileConstants.UPLOAD_PERSON;
        if (!roleIds.contains(authRoleId) && !roleIds.contains(FileConstants.SYS_AUDIT_PERSON)) {
            throw new BusinessException("当前用户无上传权限！");
        }
        return roleIds.contains(FileConstants.SYS_AUDIT_PERSON);
    }
}
