/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.filemanager.statemachine;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqFileAuditRecord;
import com.crunii.micro.domain.entity.main.YcqFileAuditRole;
import com.crunii.micro.domain.entity.main.YcqUploadInfo;
import com.crunii.micro.domain.repo.main.YcqFileAuditRecordRepo;
import com.crunii.micro.domain.repo.main.YcqFileAuditRoleRepo;
import com.crunii.micro.domain.repo.main.YcqUploadInfoRepo;
import com.crunii.micro.service.filemanager.constants.FileConstants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.WithStateMachine;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @author shenhs create 2023/6/17 14:31
 */
@Component("fileEventListener")
@WithStateMachine(name = "fileStateMachine")
@Slf4j
public class FileEventListener {

	public static final String MESSAGE_KEY = "upload_header";
	public static final String WORKER_KEY = "worker";
	public static final String REASON_LABEL_KEY = "reasonLabel";
	public static final String REASON_KEY = "reason";

	@Autowired
	private YcqUploadInfoRepo ycqUploadInfoRepo;

	@Autowired
	private YcqFileAuditRoleRepo roleRepo;

	@Autowired
	private YcqFileAuditRecordRepo recordRepo;

	/**
	 * 待审核
	 * @param message
	 */
	@StatesOnTransition(source = FileStatus.UPLOAD_SUCCESS, target = FileStatus.WAIT_DEPARTMENT_AUDIT)
	public void saveUploadTransition(Message<FileAuditEvent> message){
		Worker worker = (Worker)message.getHeaders().get(WORKER_KEY);
		YcqUploadInfo uploadInfo = (YcqUploadInfo)message.getHeaders().get(MESSAGE_KEY);
		log.info("待审核"+uploadInfo.getUploadId());
		uploadInfo.setAuditStatus(FileStatus.WAIT_DEPARTMENT_AUDIT.getState());
		ycqUploadInfoRepo.save(uploadInfo);
		YcqFileAuditRole ycqFileAuditRole = new YcqFileAuditRole();
		ycqFileAuditRole.setUploadId(uploadInfo.getUploadId());
		ycqFileAuditRole.setUploadWorkerId(Integer.toUnsignedLong(worker.getLoginId()));
		ycqFileAuditRole.setUploadWorkerOrg(worker.getOrgId());
		// 设置状态为待审核
		ycqFileAuditRole.setDepartmentAuditStatus(AuditWorkerStatus.WAIT_AUDIT.getState());
		roleRepo.save(ycqFileAuditRole);

		// 状态变更为上传完成
		Integer nextRoleId = getNextRoleId(FileConstants.UPLOAD_PERSON);
		saveWaitAuditRecord(uploadInfo, worker, FileConstants.UPLOAD_PERSON, AuditRecordStatus.UPLOAD, nextRoleId, AuditRecordStatus.WAIT_AUDIT,null,null);

		roleRepo.flush();
	}

	/**
	 * 重新走流程
	 * @param message
	 */
	@StatesOnTransition(source = FileStatus.PASS, target = FileStatus.WAIT_DEPARTMENT_AUDIT)
	public void saveReProcess(Message<FileAuditEvent> message){
		Worker worker = (Worker)message.getHeaders().get(WORKER_KEY);
		YcqUploadInfo uploadInfo = (YcqUploadInfo)message.getHeaders().get(MESSAGE_KEY);
		log.info("待审核"+uploadInfo.getUploadId());
		uploadInfo.setAuditStatus(FileStatus.WAIT_DEPARTMENT_AUDIT.getState());
		ycqUploadInfoRepo.save(uploadInfo);
		YcqFileAuditRole ycqFileAuditRole = new YcqFileAuditRole();
		ycqFileAuditRole.setUploadId(uploadInfo.getUploadId());
		ycqFileAuditRole.setUploadWorkerId(Integer.toUnsignedLong(worker.getLoginId()));
		ycqFileAuditRole.setUploadWorkerOrg(worker.getOrgId());
		// 设置状态为待审核
		ycqFileAuditRole.setDepartmentAuditStatus(AuditWorkerStatus.WAIT_AUDIT.getState());
		roleRepo.save(ycqFileAuditRole);

		// 状态变更为上传完成
		Integer nextRoleId = getNextRoleId(FileConstants.UPLOAD_PERSON);
		saveWaitAuditRecord(uploadInfo, worker, FileConstants.UPLOAD_PERSON, AuditRecordStatus.RE_PROCESS, nextRoleId, AuditRecordStatus.WAIT_AUDIT,null,null);

		roleRepo.flush();
	}

	/**
	 * 部门审核通过
	 * @param message
	 */
	@StatesOnTransition(source = FileStatus.WAIT_DEPARTMENT_AUDIT, target = FileStatus.WAIT_SYSTEM_AUDIT)
	public void saveDepPass(Message<FileAuditEvent> message){
		// 从状态机里面获取参数
		Worker worker = (Worker)message.getHeaders().get(WORKER_KEY);
		YcqUploadInfo uploadInfo = (YcqUploadInfo)message.getHeaders().get(MESSAGE_KEY);
		String reason = (String)message.getHeaders().get(REASON_KEY);
		String reasonLabel = (String)message.getHeaders().get(REASON_LABEL_KEY);

		log.info("部门审核通过"+uploadInfo.getUploadId());
		uploadInfo.setAuditStatus(FileStatus.WAIT_SYSTEM_AUDIT.getState());
		ycqUploadInfoRepo.save(uploadInfo);
		YcqFileAuditRole auditRole = roleRepo.getOne(uploadInfo.getUploadId());
		changeAuditRoleDepart(auditRole,AuditWorkerStatus.PASS,worker);
		changeAuditRoleSys(auditRole,AuditWorkerStatus.WAIT_AUDIT,null);
		roleRepo.save(auditRole);

		// 状态变更为上传完成
		Integer nextRoleId = getNextRoleId(FileConstants.DEPARTMENT_AUDIT_PERSON);
		saveWaitAuditRecord(uploadInfo, worker, FileConstants.DEPARTMENT_AUDIT_PERSON, AuditRecordStatus.PASS, nextRoleId, AuditRecordStatus.WAIT_AUDIT,reason,reasonLabel);

		roleRepo.flush();
	}

	/**
	 * 系统审核通过
	 * @param message
	 */
	@StatesOnTransition(source = FileStatus.WAIT_SYSTEM_AUDIT, target = FileStatus.PASS)
	public void saveSysPass(Message<FileAuditEvent> message){
		Worker worker = (Worker)message.getHeaders().get(WORKER_KEY);
		YcqUploadInfo uploadInfo = (YcqUploadInfo)message.getHeaders().get(MESSAGE_KEY);
		String reason = (String)message.getHeaders().get(REASON_KEY);
		String reasonLabel = (String)message.getHeaders().get(REASON_LABEL_KEY);

		log.info("系统审核通过"+uploadInfo.getUploadId());
		YcqFileAuditRole auditRole = roleRepo.getOne(uploadInfo.getUploadId());
		changeAuditRoleSys(auditRole,AuditWorkerStatus.PASS,worker);
		uploadInfo.setAuditStatus(FileStatus.PASS.getState());
		ycqUploadInfoRepo.save(uploadInfo);
		roleRepo.save(auditRole);

		// 状态修改
		saveWaitAuditRecord(uploadInfo, worker, FileConstants.SYS_AUDIT_PERSON, AuditRecordStatus.PASS, null, null,reason,reasonLabel);

		roleRepo.flush();
	}

	/**
	 * 系统审核通过(系统审核员上传文件直接pass)
	 * @param message
	 */
	@StatesOnTransition(source = FileStatus.WAIT_DEPARTMENT_AUDIT, target = FileStatus.PASS)
	public void saveSysUpload(Message<FileAuditEvent> message){
		Worker worker = (Worker)message.getHeaders().get(WORKER_KEY);
		YcqUploadInfo uploadInfo = (YcqUploadInfo)message.getHeaders().get(MESSAGE_KEY);
		String reason = (String)message.getHeaders().get(REASON_KEY);
		String reasonLabel = (String)message.getHeaders().get(REASON_LABEL_KEY);

		log.info("系统审核员上传"+uploadInfo.getUploadId());
		YcqFileAuditRole auditRole = roleRepo.getOne(uploadInfo.getUploadId());
		// 系统审核员直接上传，部门审核以及管理员审核直接通过
		auditRole.setSysAuditStatus(AuditWorkerStatus.PASS.getState());
		auditRole.setSysWorkerId(Integer.toUnsignedLong(worker.getLoginId()));
		auditRole.setDepartmentAuditStatus(AuditWorkerStatus.PASS.getState());
		auditRole.setDepartmentWorkerId(Integer.toUnsignedLong(worker.getLoginId()));
		uploadInfo.setAuditStatus(FileStatus.PASS.getState());
		ycqUploadInfoRepo.save(uploadInfo);
		roleRepo.save(auditRole);

		// 状态修改
		saveWaitAuditRecord(uploadInfo, worker, FileConstants.SYS_AUDIT_PERSON, AuditRecordStatus.PASS, null, null,reason,reasonLabel);

		roleRepo.flush();
	}

	/**
	 * 部门审核驳回
	 * @param message
	 */
	@StatesOnTransition(source = FileStatus.WAIT_DEPARTMENT_AUDIT, target = FileStatus.REJECT)
	public void saveDepartReject(Message<FileAuditEvent> message){
		Worker worker = (Worker)message.getHeaders().get(WORKER_KEY);
		YcqUploadInfo uploadInfo = (YcqUploadInfo)message.getHeaders().get(MESSAGE_KEY);
		String reason = (String)message.getHeaders().get(REASON_KEY);
		String reasonLabel = (String)message.getHeaders().get(REASON_LABEL_KEY);

		log.info("部门审核驳回"+uploadInfo.getUploadId());
		YcqFileAuditRole auditRole = roleRepo.getOne(uploadInfo.getUploadId());
		changeAuditRoleDepart(auditRole,AuditWorkerStatus.REJECT,worker);
		roleRepo.save(auditRole);

		uploadInfo.setAuditStatus(FileStatus.REJECT.getState());
		ycqUploadInfoRepo.save(uploadInfo);

		// 状态修改
		saveWaitAuditRecord(uploadInfo, worker, FileConstants.DEPARTMENT_AUDIT_PERSON, AuditRecordStatus.REJECT, null, null,reason,reasonLabel);

		ycqUploadInfoRepo.flush();
	}

	/**
	 * 系统审核驳回
	 * @param message
	 */
	@StatesOnTransition(source = FileStatus.WAIT_SYSTEM_AUDIT, target = FileStatus.REJECT)
	public void saveSystemReject(Message<FileAuditEvent> message){
		Worker worker = (Worker)message.getHeaders().get(WORKER_KEY);
		YcqUploadInfo uploadInfo = (YcqUploadInfo)message.getHeaders().get(MESSAGE_KEY);
		String reason = (String)message.getHeaders().get(REASON_KEY);
		String reasonLabel = (String)message.getHeaders().get(REASON_LABEL_KEY);
		log.info("系统审核驳回"+uploadInfo.getUploadId());

		YcqFileAuditRole auditRole = roleRepo.getOne(uploadInfo.getUploadId());
		auditRole.setSysWorkerId(Integer.toUnsignedLong(worker.getLoginId()));
		auditRole.setSysAuditStatus(AuditWorkerStatus.REJECT.getState());
		changeAuditRoleSys(auditRole,AuditWorkerStatus.REJECT,worker);
		roleRepo.save(auditRole);

		uploadInfo.setAuditStatus(FileStatus.REJECT.getState());
		ycqUploadInfoRepo.save(uploadInfo);

		// 状态修改
		saveWaitAuditRecord(uploadInfo, worker, FileConstants.SYS_AUDIT_PERSON, AuditRecordStatus.REJECT, null, null,reason,reasonLabel);

		ycqUploadInfoRepo.flush();
	}

	/**
	 * 发起重审
	 */
	@StatesOnTransition(source = FileStatus.REJECT, target = FileStatus.WAIT_DEPARTMENT_AUDIT)
	public void saveReReject(Message<FileAuditEvent> message){
		Worker worker = (Worker)message.getHeaders().get(WORKER_KEY);
		YcqUploadInfo uploadInfo = (YcqUploadInfo)message.getHeaders().get(MESSAGE_KEY);
		log.info("系统审核驳回"+uploadInfo.getUploadId());

		YcqFileAuditRole auditRole = roleRepo.getOne(uploadInfo.getUploadId());
		changeAuditRoleSys(auditRole,null,null);
		changeAuditRoleDepart(auditRole,AuditWorkerStatus.WAIT_AUDIT,null);
		roleRepo.save(auditRole);

		uploadInfo.setAuditStatus(FileStatus.WAIT_DEPARTMENT_AUDIT.getState());
		ycqUploadInfoRepo.save(uploadInfo);
		// 状态修改
		Integer nextRoleId = getNextRoleId(FileConstants.UPLOAD_PERSON);
		saveWaitAuditRecord(uploadInfo, worker, FileConstants.UPLOAD_PERSON, AuditRecordStatus.RE_AUDIT, nextRoleId, AuditRecordStatus.WAIT_AUDIT,null,null);

		ycqUploadInfoRepo.flush();
	}




	private void changeAuditRoleDepart(YcqFileAuditRole auditRole,AuditWorkerStatus status,Worker worker){
		if(status!=null){
			auditRole.setDepartmentAuditStatus(status.getState());
		}else{
			auditRole.setDepartmentAuditStatus(null);
		}
		if(worker!=null){
			auditRole.setDepartmentWorkerId(Integer.toUnsignedLong(worker.getLoginId()));
		}else{
			auditRole.setDepartmentWorkerId(null);
		}
	}

	private void changeAuditRoleSys(YcqFileAuditRole auditRole,AuditWorkerStatus status,Worker worker){
		if(status!=null) {
			auditRole.setSysAuditStatus(status.getState());
		}else{
			auditRole.setSysAuditStatus(null);
		}
		if(worker!=null) {
			auditRole.setSysWorkerId(Integer.toUnsignedLong(worker.getLoginId()));
		}else {
			auditRole.setSysWorkerId(null);
		}
	}

	private Integer getNextRoleId(Integer roleId){
		int i = FileConstants.SORT_ROLES.indexOf(roleId);
		Integer nextRoleId = FileConstants.SORT_ROLES.get(i + 1);
		return nextRoleId;
	}

	private void saveWaitAuditRecord(YcqUploadInfo uploadInfo,Worker worker,Integer currRoleId,AuditRecordStatus status,Integer nextRoleId,AuditRecordStatus nextStatus,String reason,String reasonLabel){
		Date date = new Date();
		// 时间+1秒
		DateTime dateTime = DateUtil.offsetSecond(date, 1);
		List<YcqFileAuditRecord> recordList = new ArrayList<>();
		// 查询是否有待审核的记录未修改
		Map<String,Object> props = new HashMap<>();
		props.put("uploadId",uploadInfo.getUploadId());
		props.put("status",AuditRecordStatus.WAIT_AUDIT.getState());
		props.put("roleId",currRoleId);
		List<YcqFileAuditRecord> byProps = recordRepo.findByProps(props);
		YcqFileAuditRecord record = null;
		if(byProps.isEmpty()){
			record = new YcqFileAuditRecord();
			record.setUploadId(uploadInfo.getUploadId());
			record.setRoleId(currRoleId);
		}else{
			record = byProps.get(0);
		}
		record.setWorkerName(worker.getWorkerName());
		record.setWorkerId(worker.getLoginId());
		record.setStatus(status.getState());
		record.setCreateTime(date);
		record.setReason(reason);
		record.setReasonLabel(reasonLabel);
		recordList.add(record);

		if(nextRoleId!=null){
			// 状态变更为上传完成
			YcqFileAuditRecord record2 = new YcqFileAuditRecord();
			record2.setWorkerId(null);
			record2.setUploadId(uploadInfo.getUploadId());
			record2.setRoleId(nextRoleId);
			record2.setStatus(nextStatus.getState());
			record2.setCreateTime(dateTime);
			recordList.add(record2);
		}
		recordRepo.saveAll(recordList);
	}

}
