/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.filemanager.statemachine;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineConfigurerAdapter;
import org.springframework.statemachine.config.builders.StateMachineConfigurationConfigurer;
import org.springframework.statemachine.config.builders.StateMachineStateConfigurer;
import org.springframework.statemachine.config.builders.StateMachineTransitionConfigurer;
import org.springframework.statemachine.listener.StateMachineListener;
import org.springframework.statemachine.listener.StateMachineListenerAdapter;
import org.springframework.statemachine.state.State;

import java.util.EnumSet;

/**
 * @author shenhs create 2023/6/17 11:32
 */
@Configuration
@EnableStateMachine(name = "fileStateMachine")
public class FileStateMachineConfig extends StateMachineConfigurerAdapter<FileStatus, FileAuditEvent> {


	@Override
	public void configure(StateMachineConfigurationConfigurer<FileStatus, FileAuditEvent> config)
			throws Exception {
		config
				.withConfiguration()
				.autoStartup(true);
//				.listener(listener());
	}

	@Override
	public void configure(StateMachineStateConfigurer<FileStatus, FileAuditEvent> states)
			throws Exception {
		states
				.withStates()
				.initial(FileStatus.UPLOAD_SUCCESS)
				.states(EnumSet.allOf(FileStatus.class));
	}


	/**
	 * 配置状态转换事件关系
	 *
	 * @param transitions
	 * @throws Exception
	 */
	public void configure(StateMachineTransitionConfigurer<FileStatus, FileAuditEvent> transitions) throws Exception {
		transitions
				//上传:上传完成-> 待部门审核
				.withExternal().source(FileStatus.UPLOAD_SUCCESS).target(FileStatus.WAIT_DEPARTMENT_AUDIT).event(FileAuditEvent.UPLOAD)
				.and()
				// 部门审核通过事件:待部门审核-> 待系统审核员审核
				.withExternal().source(FileStatus.WAIT_DEPARTMENT_AUDIT).target(FileStatus.WAIT_SYSTEM_AUDIT).event(FileAuditEvent.DEPARTMENT_AUDIT_PAAS)
				.and()
				// 部门审核驳回事件:待部门审核-> 驳回
				.withExternal().source(FileStatus.WAIT_DEPARTMENT_AUDIT).target(FileStatus.REJECT).event(FileAuditEvent.DEPARTMENT_AUDIT_REJECT).
				and()
				// 系统审核通过事件:待系统审核员审核-> 通过
				.withExternal().source(FileStatus.WAIT_SYSTEM_AUDIT).target(FileStatus.PASS).event(FileAuditEvent.SYSTEM_AUDIT_PAAS).
				and()
				// 系统审核驳回事件:待系统审核员审核-> 驳回
				.withExternal().source(FileStatus.WAIT_SYSTEM_AUDIT).target(FileStatus.REJECT).event(FileAuditEvent.SYSTEM_AUDIT_REJECT).
				and()
				// 发起重审事件:驳回-> 系统审核
				.withExternal().source(FileStatus.REJECT).target(FileStatus.WAIT_DEPARTMENT_AUDIT).event(FileAuditEvent.RE_AUDIT).
				and()
				// 系统审核员上传 ->通过
				.withExternal().source(FileStatus.WAIT_DEPARTMENT_AUDIT).target(FileStatus.PASS).event(FileAuditEvent.SYSTEM_AUDIT_PAAS).
				and()
				// 修改--》待部门审核
				.withExternal().source(FileStatus.PASS).target(FileStatus.WAIT_DEPARTMENT_AUDIT).event(FileAuditEvent.RE_PROCESS);
	}


	/*
	@Bean
	public StateMachineListener<FileStatus, FileAuditEvent> listener() {
		return new StateMachineListenerAdapter<FileStatus, FileAuditEvent>() {
			@Override
			public void stateChanged(State<FileStatus, FileAuditEvent> from, State<FileStatus, FileAuditEvent> to) {
				System.out.println("State change to " + to.getId());
			}
		};
	}
 */

}
