package com.crunii.micro.service.label.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.label.dto.category.dto.*;
import com.crunii.micro.service.label.dto.category.vo.CategorySearchVo;
import com.crunii.micro.service.label.dto.category.vo.SearchAsynchronousTreeVo;
import com.crunii.micro.service.label.dto.category.vo.SearchSynchronousTreeVo;
import com.crunii.micro.service.label.service.CategoryManagementService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author baoxy
 * @date 2023-05-30
 */
@Api(tags = "类别管理")
@RestController
@RequestMapping("/auth/CategoryManagementController")
public class CategoryManagementController {

    @Resource
    private CategoryManagementService categoryManagementService;

    @ApiOperation(value = "类别管理分页查询")
    @RequestMapping(path = "/pageSearch", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<CategorySearchVo>> pageSearch(@RequestBody PageReq<Object> req) {
        return categoryManagementService.pageSearch(req);
    }

    @ApiOperation(value = "查看类别树(异步查询)")
    @RequestMapping(path = "/searchAsynchronousTree", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<SearchAsynchronousTreeVo>> searchAsynchronousTree(@RequestBody SearchAsynchronousTreeDto dto) {
        return categoryManagementService.searchAsynchronousTree(dto);
    }

    @ApiOperation(value = "查看类别树(同步查询)")
    @RequestMapping(path = "/searchSynchronousTree", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<SearchSynchronousTreeVo>> searchSynchronousTree() {
        return categoryManagementService.searchSynchronousTree();
    }

    @ApiOperation(value = "添加分类")
    @RequestMapping(path = "/addCategory", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> addCategory(@RequestBody AddCategoryDto dto, @CurrWorker  @ApiIgnore Worker worker) {
        return categoryManagementService.addCategory(dto, worker);
    }

    @ApiOperation(value = "新增下级")
    @RequestMapping(path = "/addBelow", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> addBelow(@RequestBody AddBelowDto dto, @CurrWorker  @ApiIgnore Worker worker) {
        return categoryManagementService.addBelow(dto, worker);
    }

    @ApiOperation(value = "转移标签")
    @RequestMapping(path = "/moveLabel", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> moveLabel(@RequestBody MoveLabelDto dto, @CurrWorker  @ApiIgnore Worker worker) {
        return categoryManagementService.moveLabel(dto, worker);
    }

    @ApiOperation(value = "编辑类别")
    @RequestMapping(path = "/updateCategory", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateCategory(@RequestBody UpdateCategoryDto dto, @CurrWorker  @ApiIgnore Worker worker) {
        return categoryManagementService.updateCategory(dto, worker);
    }

    @ApiOperation(value = "删除确认")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "categoryId", value = "类别id", required = true, dataType = "String")
    )
    @RequestMapping(path = "/deleteConfirm", method = RequestMethod.GET)
    @ResponseBody
    public Result<Boolean> deleteConfirm(String categoryId) {
        return categoryManagementService.deleteConfirm(categoryId);
    }

    @ApiOperation(value = "删除类别")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "categoryId", value = "类别id", required = true, dataType = "String")
    )
    @RequestMapping(path = "/deleteCategory", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> deleteCategory(String categoryId, @CurrWorker  @ApiIgnore Worker worker) {
        return categoryManagementService.deleteCategory(categoryId, worker);
    }
}
