package com.crunii.micro.service.label.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.feign.client.dto.LabelDto;
import com.crunii.micro.service.label.dto.label.dto.*;
import com.crunii.micro.service.label.dto.label.vo.LabelPageSearchVo;
import com.crunii.micro.service.label.dto.label.vo.LabelRecentlyCountVo;
import com.crunii.micro.service.label.service.LabelManagementService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author baoxy
 * @date 2023-05-30
 */
@Api(tags = "标签管理")
@RestController
@RequestMapping("/auth/LabelManagementController")
public class LabelManagementController {

    @Resource
    private LabelManagementService labelManagementService;

    @ApiOperation(value = "标签管理分页查询")
    @RequestMapping(path = "/pageSearch", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<LabelPageSearchVo>> pageSearch(@RequestBody PageReq<LabelPageSearchDto> req) {
        return labelManagementService.pageSearch(req);
    }

    @ApiOperation(value = "添加标签")
    @RequestMapping(path = "/addLabel", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> addLabel(@RequestBody AddLabelDto dto, @CurrWorker  @ApiIgnore Worker worker) {
        return labelManagementService.addLabel(dto, worker);
    }

    @ApiOperation(value = "编辑标签")
    @RequestMapping(path = "/updateLabel", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateLabel(@RequestBody UpdateLabelDto dto, @CurrWorker  @ApiIgnore Worker worker) {
        return labelManagementService.updateLabel(dto, worker);
    }

    @ApiOperation(value = "批量删除标签")
    @RequestMapping(path = "/deleteLabel", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> deleteLabel(@RequestBody DeleteLabelDto dto, @CurrWorker  @ApiIgnore Worker worker) {
        return labelManagementService.deleteLabel(dto, worker);
    }

    @ApiOperation(value = "热门标签")
    @RequestMapping(path = "/hotLabel", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<LabelDto>> hotLabel() {
        return labelManagementService.hotLabel();
    }

    @ApiOperation(value = "标签管理分页查询-业务类型为图绘青春")
    @RequestMapping(path = "/pageSearchByDrawYouth", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<LabelPageSearchVo>> pageSearchByDrawYouth(@RequestBody PageReq<LabelPageSearchDto> req) {
        return labelManagementService.pageSearchByDrawYouth(req);
    }

    @ApiOperation(value = "内容标签-展示最近时间的标签(数量自定义)")
    @PostMapping("/labelRecentlyCount")
    public Result<List<LabelRecentlyCountVo>> labelRecentlyCount(@RequestBody LabelRecentlyCountDto dto) {
        return labelManagementService.labelRecentlyCount(dto);
    }
}
