package com.crunii.micro.service.label.jdbc;

import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.entity.main.TsmCategoryManagement;
import com.crunii.micro.feign.client.dto.LabelDto;
import com.crunii.micro.service.label.constants.LabelConstants;
import com.crunii.micro.service.label.dto.category.vo.TotalVo;
import com.crunii.micro.service.label.dto.label.dto.DeleteLabelDto;
import com.crunii.micro.service.label.dto.label.dto.LabelPageSearchDto;
import com.crunii.micro.service.label.dto.label.dto.LabelRecentlyCountDto;
import com.crunii.micro.service.label.dto.label.vo.LabelPageSearchVo;
import com.crunii.micro.service.label.dto.label.vo.LabelRecentlyCountVo;
import com.crunii.micro.service.label.dto.label.vo.TotalUseCountVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author baoxy
 * @date 2023-05-30
 */
@Repository
public class LabelManagementDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public static final String PAGE_SEARCH_SQL = " select a.label_id, a.category_id, a.label_name, a.label_desc, a.create_time, " +
            " a.use_count,a.service_type from (select label_id, category_id, label_name, label_desc, create_time :: DATE" +
            " as create_time, use_count, service_type from tsm_label_management where state = ? ";

    public static final String PAGE_SEARCH_COUNT_SQL = "select count(*) from (select category_id from tsm_label_management where state = ?";

    public static final String PAGE_SEARCH_INNER_JOIN_SQL = "INNER JOIN (select category_id from tsm_category_management " +
            " START WITH category_id = ? CONNECT BY PRIOR category_id  = category_parent_id) b ON a.category_id = b.category_id";

    public static final String PAGE_SEARCH_DRAW_YOUTH_SQL = "select label_id, category_id, label_name, label_desc, create_time :: DATE" +
            " as create_time, use_count, service_type from tsm_label_management where state = ? and service_type = ? ";

    public static final String PAGE_SEARCH_COUNT_DRAW_YOUTH_SQL = "select count(*) from tsm_label_management where state = ? " +
            "and service_type = ? ";

    public PageRsp<LabelPageSearchVo> pageSearch(PageReq<LabelPageSearchDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_SEARCH_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_SEARCH_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        param.add(CommonConstants.STATE_VALID);
        if (StringUtils.isNotBlank(req.getQuery().getLabelName())) {
            sb.append(" and label_name like ? ");
            sbCount.append(" and label_name like ? ");
            param.add("%" + req.getQuery().getLabelName() + "%");
        }
        if (Objects.nonNull(req.getQuery().getReleaseStartTime())) {
            sb.append(" and create_time :: DATE >= ? ");
            sbCount.append(" and create_time :: DATE >= ? ");
            param.add(req.getQuery().getReleaseStartTime());
        }
        if (Objects.nonNull(req.getQuery().getReleaseEndTime())) {
            sb.append(" and create_time :: DATE <= ? ");
            sbCount.append(" and create_time :: DATE <= ? ");
            param.add(req.getQuery().getReleaseEndTime());
        }
        if (StringUtils.isNotBlank(req.getQuery().getServiceType())) {
            sb.append(" and service_type = ? ");
            sbCount.append(" and service_type = ? ");
            param.add(req.getQuery().getServiceType());
        }
        sb.append(") a ");
        sbCount.append(") a ");
        if (StringUtils.isNotBlank(req.getQuery().getCategoryId())) {
            sb.append(PAGE_SEARCH_INNER_JOIN_SQL);
            sbCount.append(PAGE_SEARCH_INNER_JOIN_SQL);
            param.add(req.getQuery().getCategoryId());
        }
        int total = jt.queryForInt(sbCount.toString(), param.toArray());
        List<LabelPageSearchVo> labelPageSearchVoList;
        String dbType = jt.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sb, req);
            labelPageSearchVoList = jt.queryForBeans(LabelPageSearchVo.class, sb.toString(), param.toArray());
        } else {
            labelPageSearchVoList = new ArrayList<>();
        }
        PageRsp<LabelPageSearchVo> pageRsp = new PageRsp<>();
        pageRsp.setPageNo(req.getPageNo());
        pageRsp.setPageSize(req.getPageSize());
        pageRsp.setTotalNum(total);
        pageRsp.setResult(labelPageSearchVoList);
        return pageRsp;
    }

    public PageRsp<LabelPageSearchVo> pageSearchByDrawYouth(PageReq<LabelPageSearchDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_SEARCH_DRAW_YOUTH_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_SEARCH_COUNT_DRAW_YOUTH_SQL);
        List<Object> param = new ArrayList<>();
        param.add(CommonConstants.STATE_VALID);
        param.add(LabelConstants.DRAW_YOUTH_SERVICE_TYPE);
        if (StringUtils.isNotBlank(req.getQuery().getLabelName())) {
            sb.append(" and label_name like ? ");
            sbCount.append(" and label_name like ? ");
            param.add("%" + req.getQuery().getLabelName() + "%");
        }
        int total = jt.queryForInt(sbCount.toString(), param.toArray());
        List<LabelPageSearchVo> labelPageSearchVoList;
        String dbType = jt.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sb, req);
            labelPageSearchVoList = jt.queryForBeans(LabelPageSearchVo.class, sb.toString(), param.toArray());
        } else {
            labelPageSearchVoList = new ArrayList<>();
        }
        PageRsp<LabelPageSearchVo> pageRsp = new PageRsp<>();
        pageRsp.setPageNo(req.getPageNo());
        pageRsp.setPageSize(req.getPageSize());
        pageRsp.setTotalNum(total);
        pageRsp.setResult(labelPageSearchVoList);
        return pageRsp;
    }

    public TsmCategoryManagement linkCategoryName(Long categoryId) {
        String sql = "select category_id, category_name, level,category_parent_id from tsm_category_management" +
                " where state = ? and category_id = ?";
        return jt.queryForBean(TsmCategoryManagement.class, sql, CommonConstants.STATE_VALID, categoryId);
    }

    public void deleteLabel(DeleteLabelDto dto, String loginId) {
        StringBuilder sql = new StringBuilder("update tsm_label_management set state = ?, last_worker = ?, update_time = ?" +
                " where label_id in (");
        List<Object> param = new ArrayList<>();
        param.add(CommonConstants.STATE_INVALID);
        param.add(loginId);
        param.add(new Date());
        dto.getLabelIdList().forEach(e -> {
            sql.append("?,");
            param.add(Long.valueOf(e));
        });
        sql.deleteCharAt(sql.length() - 1).append(")");
        jt.update(sql.toString(), param.toArray());
    }

    public List<LabelRecentlyCountVo> labelRecentlyCount(LabelRecentlyCountDto dto) {
        String sql = "select label_id, label_name from tsm_label_management where service_type = ? and state = ? ";
        if(dto.isImportantNews()){
            sql += "and label_name = '要闻' ";
        }else {
            sql += "and label_name <> '要闻' ";
        }
        sql += "order by update_time desc limit 0,?";
        return jt.queryForBeans(LabelRecentlyCountVo.class, sql, dto.getServiceType(), CommonConstants.STATE_VALID, dto.getCount());
    }

    public List<LabelRecentlyCountVo> labelPublicRecentlyCount(LabelRecentlyCountDto dto) {
        String sql = "select label_id, label_name from tsm_label_management where service_type = ? and state = ? order by " +
                "case when label_name = '要闻' then '0' else update_time end limit 0,?";
        return jt.queryForBeans(LabelRecentlyCountVo.class, sql, dto.getServiceType(), CommonConstants.STATE_VALID, dto.getCount());
    }


    public TotalVo selectCountLabelByCategoryId(Long categoryId) {
        String sql = "select count(1) as total from tsm_label_management where category_id = ? and state = ?";
        return jt.queryForBean(TotalVo.class, sql, categoryId, CommonConstants.STATE_VALID);
    }

    public TotalVo checkRepeatLabelName(String labelName) {
        String sql = "select count(1) as total from tsm_label_management where label_name = ? and state = ? ";
        return jt.queryForBean(TotalVo.class, sql, labelName, CommonConstants.STATE_VALID);
    }

    public void updateUseCount(String labelId, Integer useCount) {
        String sql = "update tsm_label_management set use_count = ? where label_id = ?";
        jt.update(sql, useCount, labelId);
    }

    public List<TotalUseCountVo> fileLabelCount() {
        String sql = "select total, file.label_id, label_name from (select count(1) as total, label_id from " +
                "ycq_file_label_relation group by label_id) file INNER JOIN (select label_id, label_name from " +
                "tsm_label_management where state = ?) label ON file.label_id = label.label_id";
        return jt.queryForBeans(TotalUseCountVo.class, sql, CommonConstants.STATE_VALID);
    }

    public void deleteStateZero() {
        String sql = "delete from tsm_label_management where state = ?";
        jt.update(sql, CommonConstants.STATE_INVALID);
    }

    public List<LabelDto> hotLabel() {
        String sql = "select label_id, label_name from tsm_label_management where use_count != ? and service_type = ? " +
                "order by use_count desc limit 10";
        return jt.queryForBeans(LabelDto.class, sql, LabelConstants.ZERO_INTEGER, LabelConstants.DRAW_YOUTH_SERVICE_TYPE);
    }
}
