package com.crunii.micro.service.label.service;

import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.feign.client.dto.LabelDto;
import com.crunii.micro.service.label.dto.category.vo.SearchSynchronousTreeVo;
import org.redisson.api.RMapCache;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @author baoxy
 * @date 2023-06-02
 */
@Service
public class AiconsumeCacheService implements InitializingBean {

    @Resource
    private RedissonClient redissonClient;

    private RMapCache<String, List<SearchSynchronousTreeVo>> synchronousTreeCache = null;
    private RMapCache<String, Integer> labelUseCountCache = null;
    // 热门标签的48小时缓存
    private RMapCache<String, List<LabelDto>> fortyEightHotLabelCache = null;
    // 热门标签的长期缓存(不设置多少时间删除)
    private RMapCache<String, List<LabelDto>> longTimeHotLabelCache = null;

    @Override
    public void afterPropertiesSet() {
        synchronousTreeCache = redissonClient.getMapCache(RedisKeys.LABEL_SYNCHRONOUS_TREE);
        labelUseCountCache = redissonClient.getMapCache(RedisKeys.LABEL_USE_COUNT);
        fortyEightHotLabelCache = redissonClient.getMapCache(RedisKeys.HOT_LABEL_FORTY_EIGHT);
        longTimeHotLabelCache = redissonClient.getMapCache(RedisKeys.HOT_LABEL_LONG_TIME);
    }

    public void putSynchronousTree(String key, List<SearchSynchronousTreeVo> searchSynchronousTreeVos) {
        synchronousTreeCache.put(key, searchSynchronousTreeVos, 30, TimeUnit.MINUTES);
    }

    public List<SearchSynchronousTreeVo> getSynchronousTree(String key) {
        return synchronousTreeCache.get(key);
    }

    public void removeSynchronousTree(String key) {
        synchronousTreeCache.remove(key);
    }

    public Integer getLabelUseCount(String key) {
        return labelUseCountCache.get(key);
    }

    public void putLabelUseCount(String key, Integer useCount) {
        labelUseCountCache.put(key, useCount);
    }

    public Map<String, Integer> labelUseCountAllMap() {
        return labelUseCountCache.readAllMap();
    }

    public Set<String> labelUseCountKey() {
        return labelUseCountCache.keySet();
    }

    public void putFortyEightHotLabel(String key, List<LabelDto> labelDtoList) {
        fortyEightHotLabelCache.put(key, labelDtoList, 48, TimeUnit.HOURS);
    }

    public List<LabelDto> getFortyEightHotLabel(String key) {
        return fortyEightHotLabelCache.get(key);
    }

    public void removeFortyEightHotLabel(String key) {
        fortyEightHotLabelCache.remove(key);
    }

    public void putLongTimeHotLabel(String key, List<LabelDto> labelDtoList) {
        longTimeHotLabelCache.put(key, labelDtoList);
    }

    public List<LabelDto> getLongTimeHotLabel(String key) {
        return longTimeHotLabelCache.get(key);
    }

    public void removeLongTimeHotLabel(String key) {
        longTimeHotLabelCache.remove(key);
    }
}
