package com.crunii.micro.service.label.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.label.dto.category.dto.*;
import com.crunii.micro.service.label.dto.category.vo.CategorySearchVo;
import com.crunii.micro.service.label.dto.category.vo.SearchAsynchronousTreeVo;
import com.crunii.micro.service.label.dto.category.vo.SearchSynchronousTreeVo;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

/**
 * @author baoxy
 * @date 2023-05-30
 */
public interface CategoryManagementService {

    /**
     * 类别管理分页查询
     *
     * @param req 实体类
     * @return 返回PageRsp<CategorySearchVo>
     */
    Result<PageRsp<CategorySearchVo>> pageSearch(PageReq<Object> req);

    /**
     * 添加分类
     *
     * @param dto    实体类
     * @param worker 工号对象
     * @return 返回Object
     */
    Result<Object> addCategory(AddCategoryDto dto, Worker worker);

    /**
     * 新增下级
     *
     * @param dto    实体类
     * @param worker 工号对象
     * @return 返回object
     */
    Result<Object> addBelow(@RequestBody AddBelowDto dto, Worker worker);

    /**
     * 查看类别树(异步查询)
     *
     * @param dto 实体类
     * @return 返回List<SearchBelowVo>
     */
    Result<List<SearchAsynchronousTreeVo>> searchAsynchronousTree(SearchAsynchronousTreeDto dto);

    /**
     * 查看类别树(异步查询)
     *
     * @return 返回List<SearchBelowVo>
     */
    Result<List<SearchSynchronousTreeVo>> searchSynchronousTree();

    /**
     * 转移标签
     *
     * @param dto    实体类
     * @param worker 工号对象
     * @return 返回object
     */
    Result<Object> moveLabel(MoveLabelDto dto, Worker worker);

    /**
     * 编辑类别
     *
     * @param dto    实体类
     * @param worker 工号对象
     * @return 返回Object
     */
    Result<Object> updateCategory(UpdateCategoryDto dto, Worker worker);

    /**
     * 删除确认
     *
     * @param categoryId 类别id
     * @return 返回boolean类型
     */
    Result<Boolean> deleteConfirm(String categoryId);

    /**
     * 删除类别
     *
     * @param categoryId 类别id
     * @param worker     工号对象
     * @return 返回object
     */
    Result<Object> deleteCategory(String categoryId, Worker worker);
}
