package com.crunii.micro.service.label.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.feign.client.dto.LabelDto;
import com.crunii.micro.service.label.dto.label.dto.*;
import com.crunii.micro.service.label.dto.label.vo.LabelPageSearchVo;
import com.crunii.micro.service.label.dto.label.vo.LabelRecentlyCountVo;

import java.util.List;

/**
 * @author baoxy
 * @date 2023-05-30
 */
public interface LabelManagementService {

    /**
     * 标签管理分页查询
     *
     * @param req 分页实体类
     * @return 返回分页数据
     */
    Result<PageRsp<LabelPageSearchVo>> pageSearch(PageReq<LabelPageSearchDto> req);

    /**
     * 添加标签
     *
     * @param dto    实体类
     * @param worker 工号对象
     * @return 返回object
     */
    Result<Object> addLabel(AddLabelDto dto, Worker worker);

    /**
     * 编辑标签
     *
     * @param dto    实体类
     * @param worker 工号对象
     * @return 返回object
     */
    Result<Object> updateLabel(UpdateLabelDto dto, Worker worker);

    /**
     * 批量删除标签
     *
     * @param dto    实体类
     * @param worker 工号对象
     * @return 返回object
     */
    Result<Object> deleteLabel(DeleteLabelDto dto, Worker worker);

    /**
     * 热门标签
     *
     * @return 返回list类型参数
     */
    Result<List<LabelDto>> hotLabel();

    /**
     * 标签管理分页查询
     *
     * @param req 分页实体类
     * @return 返回分页数据
     */
    Result<PageRsp<LabelPageSearchVo>> pageSearchByDrawYouth(PageReq<LabelPageSearchDto> req);

    /**
     * 内容标签-展示最近时间的标签(数量自定义)
     *
     * @param dto 实体类
     * @return LabelRecentlyCountVo
     */
    Result<List<LabelRecentlyCountVo>> labelRecentlyCount(LabelRecentlyCountDto dto);

    /**
     * 内容标签-展示最近时间的标签(数量自定义)
     *
     * @param dto 实体类
     * @return LabelRecentlyCountVo
     */
    Result<List<LabelRecentlyCountVo>> labelPublicRecentlyCount(LabelRecentlyCountDto dto);
}
