package com.crunii.micro.service.label.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.label.UseCountDto;
import com.crunii.micro.domain.entity.main.TsmLabelManagement;
import com.crunii.micro.domain.repo.main.TsmLabelManagementRepo;
import com.crunii.micro.feign.client.ElsticService;
import com.crunii.micro.feign.client.dto.LabelDto;
import com.crunii.micro.service.label.constants.LabelConstants;
import com.crunii.micro.service.label.dto.label.vo.TotalUseCountVo;
import com.crunii.micro.service.label.jdbc.LabelManagementDao;
import com.crunii.micro.service.label.service.AiconsumeCacheService;
import com.crunii.micro.service.label.service.LabelInternalService;
import com.crunii.micro.service.label.utils.MapSortUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author baoxy
 * @date 2023-06-07
 */
@Service
@Slf4j
public class LabelInternalServiceImpl implements LabelInternalService {

    @Resource
    private AiconsumeCacheService aiconsumeCacheService;

    @Resource
    private LabelManagementDao labelManagementDao;

    @Resource
    private TsmLabelManagementRepo tsmLabelManagementRepo;

    @Resource
    private ElsticService elsticService;

    @Override
    public Result<Object> labelUseCount(List<UseCountDto> useCountList) {
        useCountList.forEach(e -> {
            // 这里标识为1,表示标签被引用,redis里面次数数据加1, 标识为0, 表示标签被取消引用,redis里面次数数据减去1
            Integer redisUseCount = aiconsumeCacheService.getLabelUseCount(e.getLabelId());
            // 先要判断该标签id对应的userCount是否有值
            if (Objects.nonNull(redisUseCount)) {
                // 执行到这一步说明有值,那么进行标识判断
                if (LabelConstants.ONE_STRING.equals(e.getMark())) {
                    // 标识为1,说明该标签被引用,因此次数加1
                    redisUseCount += 1;
                    aiconsumeCacheService.putLabelUseCount(e.getLabelId(), redisUseCount);
                }
                if (LabelConstants.ZERO_STRING.equals(e.getMark())) {
                    // 标识为0,说明该标签被取消引用,因此次数减1
                    // 先要判断该标签对应的次数是否为0,如果为0,不能再减1,数据不能显示为负数
                    if (!LabelConstants.ZERO_INTEGER.equals(redisUseCount)) {
                        redisUseCount -= 1;
                        aiconsumeCacheService.putLabelUseCount(e.getLabelId(), redisUseCount);
                    }
                }
            } else {
                // 执行到这一步,表示该标签对应使用次数useCount为null,说明未赋值数字
                if (LabelConstants.ONE_STRING.equals(e.getMark())) {
                    // 标识为1,说明该标签被引用,因此将使用次数设置为1
                    redisUseCount = 1;
                    aiconsumeCacheService.putLabelUseCount(e.getLabelId(), redisUseCount);
                }
                // 如果标识为0,表示标签被取消引用,次数减1,这里不需要在写入代码,因为使用次数本身为null
            }
        });
        // 2023-06-13扩充；给热门标签功能添加缓存
        hotLabel();
        return Result.ofSuccess("调用成功！");
    }

    @Override
    public Result<Object> assembleRedisUseCount() {
        List<TotalUseCountVo> totalUseCountVoList = labelManagementDao.fileLabelCount();
        log.info("文件关联标签的数据:" + totalUseCountVoList);
        totalUseCountVoList.forEach(e -> aiconsumeCacheService.putLabelUseCount(e.getLabelId(), e.getTotal()));
        return Result.ofSuccess("调用成功！");
    }

    @Override
    public Result<Object> reassemblyLabelEsData() {
        List<TsmLabelManagement> tsmLabelManagementList = tsmLabelManagementRepo.findAll();
        List<LabelDto> labelDtoList = new ArrayList<>();
        tsmLabelManagementList.forEach(e -> {
            LabelDto labelDto = new LabelDto();
            labelDto.setLabelId(e.getLabelId().toString());
            labelDto.setLabelName(e.getLabelName());
            labelDtoList.add(labelDto);
        });
        elsticService.batchSaveLabel(labelDtoList);
        return Result.ofSuccess("调用成功！");
    }

    public void hotLabel() {
        // 获取标签使用数量map类型全部数据
        Map<String, Integer> redisLabelCountMap = aiconsumeCacheService.labelUseCountAllMap();
        // 先进行倒序排序
        Map<String, Integer> sortDescMap = MapSortUtils.mapValueSortDesc(redisLabelCountMap);
        // 存入redis的热门标签列表
        List<LabelDto> labelDtoList = new ArrayList<>();
        // 从map中获取前10条数据
        for (String key : sortDescMap.keySet()) {
            // 列表有10条数据后就不需要再存入数据
            if (labelDtoList.size() < 10) {
                LabelDto labelDto = new LabelDto();
                labelDto.setLabelId(key);
                // 通过labelId查询labelName
                TsmLabelManagement labelName = tsmLabelManagementRepo.getOne(Long.valueOf(key));
                if (Objects.isNull(labelName)) {
                    // 若数据为null,说明有错误数据,因此跳过此次循环
                    continue;
                }
                labelDto.setLabelName(tsmLabelManagementRepo.getOne(Long.valueOf(key)).getLabelName());
                labelDtoList.add(labelDto);
            }
        }
        aiconsumeCacheService.removeFortyEightHotLabel(LabelConstants.REDIS_HOT_LABEL_FORTY_EIGHT);
        aiconsumeCacheService.putFortyEightHotLabel(LabelConstants.REDIS_HOT_LABEL_FORTY_EIGHT, labelDtoList);
        aiconsumeCacheService.removeLongTimeHotLabel(LabelConstants.REDIS_HOT_LABEL_LONG_TIME);
        aiconsumeCacheService.putLongTimeHotLabel(LabelConstants.REDIS_HOT_LABEL_LONG_TIME, labelDtoList);
    }
}
