package com.crunii.micro.service.league.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.league.dto.*;
import com.crunii.micro.service.league.service.LeagueService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * @author wangchen create 2023/5/31 17:27
 */
@Api(tags = "团委组织相关")
@RestController
@RequestMapping("/auth/LeagueController")
public class LeagueController {

    private final LeagueService leagueService;

    public LeagueController(LeagueService leagueService) {
        this.leagueService = leagueService;
    }

    @GetMapping("/getOrgChildren")
    @ApiImplicitParam(name = "leagueCode", value = "组织编码，为空查询顶层")
    @ApiOperation("左侧组织-查询组织")
    public Result<List<YcqLeagueTreeVo>> getOrgChildren(String leagueCode) {
        return Result.ofSuccess(leagueService.getOrgChildren(leagueCode));
    }

    @GetMapping("/getPublicOrgChildren")
    @ApiImplicitParam(name = "leagueCode", value = "组织编码，为空查询顶层")
    @ApiOperation("公共接口-查询组织")
    public Result<List<YcqLeagueTreeVo>> getPublicOrgChildren(String leagueCode, @ApiIgnore @CurrWorker Worker worker) {
        return Result.ofSuccess(leagueService.getPublicOrgChildren(leagueCode, worker));
    }

    @PostMapping("/pageLeague")
    @ApiOperation("分页查询组织")
    public Result<PageRsp<YcqLeagueTreeVo>> pageLeague(@RequestBody PageReq<PageLeague> req) {
        return Result.ofSuccess(leagueService.pageLeague(req));
    }

    @GetMapping("/getLmAgeBigDataEcharts")
    @ApiImplicitParam(name = "leagueCode", value = "组织编码")
    @ApiOperation("团员大数据-获取年龄分布渲染")
    public Result<List<AgeDataVo>> getLmAgeBigDataEcharts(String leagueCode) {
        return Result.ofSuccess(leagueService.getLmAgeBigDataEcharts(leagueCode));
    }

    @GetMapping("/getLmNationalityBigDataEcharts")
    @ApiImplicitParam(name = "leagueCode", value = "组织编码")
    @ApiOperation("团员大数据-获取民族分布渲染")
    public Result<NationalityDataVo> getLmNationalityBigDataEcharts(String leagueCode) {
        return Result.ofSuccess(leagueService.getLmNationalityBigDataEcharts(leagueCode));
    }

    @PostMapping("/getOrgTabulation")
    @ApiOperation("组织列表-查询组织列表")
    public Result<PageRsp<YcqLeagueVo>> getOrgTabulation(@Validated @RequestBody PageReq<YcqLeagueDto> req) {
        return Result.ofSuccess(leagueService.getOrgTabulation(req));
    }


    @GetMapping("/getOrgDetail")
    @ApiImplicitParam(name = "leagueCode", value = "组织编码")
    @ApiOperation("组织列表-获取组织详情")
    public Result<LeagueOrgDetailVo> getOrgDetail(String leagueCode) {
        return Result.ofSuccess(leagueService.getOrgDetail(leagueCode));
    }

    @GetMapping("/getSuperiorOrgDetail")
    @ApiImplicitParam(name = "leagueCode", value = "组织编码")
    @ApiOperation("组织列表-获取组织详情(非叶子节点)")
    public Result<LeagueSuperOrgDetailVo> getSuperiorOrgDetail(String leagueCode) {
        return Result.ofSuccess(leagueService.getSuperiorOrgDetail(leagueCode));
    }

    @PostMapping("/getLmAgeBigData")
    @ApiOperation("团员大数据-获取年龄分布")
    public Result<PageRsp<AgeDataVo>> getLmAgeBigData(@RequestBody PageReq<PageLeagueAgeDto> req) {
        return Result.ofSuccess(leagueService.getLmAgeBigData(req));
    }

    @PostMapping("/getLmNationalityBigData")
    @ApiOperation("团员大数据-获取民族分布")
    public Result<PageRsp<MemberNationality>> getLmNationalityBigData(@RequestBody PageReq<PageNationalityDto> req) {
        return Result.ofSuccess(leagueService.getLmNationalityBigData(req));
    }

    @GetMapping("/getLmNationalityCount")
    @ApiImplicitParam(name = "leagueCode", value = "组织编码")
    @ApiOperation("团员大数据-获取民族分布")
    public Result<NationalityDataVo> getLmNationalityCount(String leagueCode) {
        return Result.ofSuccess(leagueService.getLmNationalityCount(leagueCode));
    }

    @GetMapping("/getLmEduBigData")
    @ApiImplicitParam(name = "leagueCode", value = "组织编码")
    @ApiOperation("团员大数据-获取团内学历")
    public Result<LeagueEduVo> getLmEduBigData(String leagueCode) {
        return Result.ofSuccess(leagueService.getLmEduBigData(leagueCode));
    }


}
