package com.crunii.micro.service.league.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.league.dto.*;
import com.crunii.micro.service.league.service.LeagueSynService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author wangchen create 2023/7/10 9:30
 */
@Api(tags = "团委组织同步接口")
@RestController
@RequestMapping("/auth/LeagueSynController")
public class LeagueSynController {

    private final LeagueSynService leagueSynService;

    public LeagueSynController(LeagueSynService leagueSynService) {
        this.leagueSynService = leagueSynService;
    }

    @GetMapping("/getNotSynData")
    @ApiOperation("获取数据")
    public Result<List<LeagueIdTypeVo>> getNotSynData() {
        return Result.ofSuccess(leagueSynService.getNotSynData());
    }

    @PostMapping("/synLeagueData")
    @ApiOperation("同步团委组织结构表")
    public Result<String> synLeagueData(@RequestBody YcqLeagueC ycqLeagueC) {
        return Result.ofSuccess(leagueSynService.saveSynLeagueData(ycqLeagueC, 0));
    }

    @PostMapping("/synLeagueDataType")
    @ApiOperation("同步团委组织结构表类型2")
    public Result<String> synLeagueDataType(@RequestBody YcqLeagueC ycqLeagueC) {
        return Result.ofSuccess(leagueSynService.saveSynLeagueData(ycqLeagueC, 1));
    }

    @PostMapping("/synLeagueLeader")
    @ApiOperation("同步团委组织结领导表")
    public Result<String> synLeagueLeader(@RequestBody List<LeagueLeaderC> leagueLeaderC) {
        return Result.ofSuccess(leagueSynService.synLeagueLeader(leagueLeaderC));
    }

    @PostMapping("/synLeagueLeaderCount")
    @ApiOperation("同步团委组织结领导数量表")
    public Result<String> synLeagueLeaderCount(@RequestBody List<YcqLeagueLeaderCountC> leagueLeaderC) {
        return Result.ofSuccess(leagueSynService.synLeagueLeaderCount(leagueLeaderC));
    }

    @GetMapping("/leaderCompleteNotice")
    @ApiOperation("组织结领导表完成后调用")
    public Result<String> leaderCompleteNotice() {
        return Result.ofSuccess(leagueSynService.leaderCompleteNotice());
    }

    @PostMapping("/synLeagueAgeData")
    @ApiOperation("同步团委组织年龄表")
    public Result<String> synLeagueAgeData(@RequestBody List<YcqLeagueAgeC> ycqLeagueAgeC) {
        return Result.ofSuccess(leagueSynService.synLeagueAgeData(ycqLeagueAgeC));
    }

    @PostMapping("/synLeagueEduData")
    @ApiOperation("同步团委组织学历表")
    public Result<String> synLeagueEduData(@RequestBody List<YcqLeagueEduC> ycqLeagueEduC) {
        return Result.ofSuccess(leagueSynService.synLeagueEduData(ycqLeagueEduC));
    }

    @PostMapping("/synLeagueNationData")
    @ApiOperation("同步民族数量表")
    public Result<String> synLeagueNationData(@RequestBody List<YcqLeagueNationC> ycqLeagueNation) {
        return Result.ofSuccess(leagueSynService.synLeagueNationData(ycqLeagueNation));
    }

    @PostMapping("/synLeagueDetail")
    @ApiOperation("同步团委组织详情表")
    public Result<String> synLeagueDetail(@RequestBody List<YcqLeagueDetailC> ycqLeagueDetailC) {
        return Result.ofSuccess(leagueSynService.synLeagueDetail(ycqLeagueDetailC));
    }

    @PostMapping("/synUserHonor")
    @ApiOperation("同步团委团员荣誉表")
    public Result<String> synUserHonor(@RequestBody List<UserHonorC> userHonor) {
        return Result.ofSuccess(leagueSynService.synUserHonor(userHonor));
    }

    @GetMapping("/getLeagueIdAndTypeCount")
    @ApiOperation("查询组织id和类型次数 ps:提供爬虫使用")
    public Result<Integer> getLeagueIdAndTypeCount(String secret) {
        return Result.ofSuccess(leagueSynService.getLeagueIdAndTypeCount(secret));
    }

    @GetMapping("/getLeagueIdAndType")
    @ApiOperation("查询组织id和类型 ps:提供爬虫使用")
    public Result<List<LeagueIdTypeVo>> getLeagueIdAndType(String secret, Integer limit, Integer pageNum) {
        return Result.ofSuccess(leagueSynService.getLeagueIdAndType(secret, limit, pageNum));
    }

    @GetMapping("/saveLeagueCountDate")
    @ApiOperation("同步数据（组织数量情况）")
    public Result<String> saveLeagueCountDate() {
        return Result.ofSuccess(leagueSynService.saveLeagueCountDate());
    }

}
