package com.crunii.micro.service.league.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.league.dto.*;
import com.crunii.micro.service.league.service.MemberService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/1 19:28
 */
@Api(tags = "团员相关")
@RestController
@RequestMapping("/auth/MemberController")
public class MemberController {

    @Resource
    private MemberService memberService;

    @ApiOperation(value = "保存团员信息")
    @PostMapping("/saveMemberData")
    public Result<String> saveMemberData(@RequestBody List<YcqMemberC> list) {
        return Result.ofSuccess(memberService.saveMemberData(list));
    }

    @ApiOperation(value = "保存超龄离团青年信息")
    @PostMapping("/saveYouthData")
    public Result<String> saveYouthData(@RequestBody List<YcqOverageYouthLeaveC> list) {
        return Result.ofSuccess(memberService.saveOverageYouthLeaveData(list));
    }

    @ApiOperation(value = "团员列表-获取组织数量")
    @PostMapping("/getOrgMember")
    public Result<PageRsp<OrgMemberVo>> getOrgMember(@RequestBody PageReq<OrgMemberDto> req) {
        return Result.ofSuccess(memberService.getOrgMember(req));
    }

    @ApiOperation(value = "团员列表-获取团员数量")
    @GetMapping("/getLmNumber")
    public Result<Integer> getLmNumber(String leagueCode) {
        return Result.ofSuccess(memberService.getLmNumber(leagueCode));
    }

    @ApiOperation(value = "团员列表-获取组织下团员信息")
    @PostMapping("/getOrgLeagueMember")
    public Result<PageRsp<MemberVo>> getOrgLeagueMember(@Validated @RequestBody PageReq<MemberDto> req) {
        return Result.ofSuccess(memberService.getOrgLeagueMember(req));
    }

    @ApiOperation(value = "组织干部列表-分页获取组织下干部信息")
    @PostMapping("/pageLeader")
    public Result<Object> pageLeader(@Validated @RequestBody PageReq<PageLeaderDto> req) {
        return Result.ofSuccess(memberService.pageLeader(req));
    }

    @ApiOperation(value = "组织干部列表-获取干部详情")
    @ApiImplicitParam(name = "leaderId", value = "主键id")
    @GetMapping("/getLeaderDetail")
    public Result<Object> getLeaderDetail(Long leaderId, Integer queryType) {
        return Result.ofSuccess(memberService.getLeaderDetail(leaderId, queryType));
    }

    @ApiOperation(value = "团员公共-查询用户获得的荣誉")
    @ApiImplicitParam(name = "userId", value = "用户id")
    @GetMapping("/getHonor")
    public Result<List<String>> getHonor(String userId) {
        return Result.ofSuccess(memberService.getHonor(userId));
    }
}
