package com.crunii.micro.service.league.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.YcqLeagueCount;
import com.crunii.micro.service.league.dto.LeagueIdTypeVo;
import com.crunii.micro.service.league.dto.NameValue;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/7/10 9:31
 */
@Repository
public class LeagueSynDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate npJdbcTemplate;


    public List<LeagueIdTypeVo> getNotSynData() {
        String sql = "SELECT league_code queryLeagueId,league_type queryLeagueType FROM YCQ_LEAGUE WHERE league_code NOT in(SELECT DISTINCT(league_id) FROM YCQ_LEAGUE_DETAIL)";
        return jdbcTemplate.queryForBeans(LeagueIdTypeVo.class,sql);
    }

    public List<String> findLeagueLeaderCount(List<String> collect) {
        String sql = "SELECT league_code FROM YCQ_LEAGUE_LEADER_COUNT WHERE league_code IN (:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",collect);
        return npJdbcTemplate.queryForRawObjects(String.class,sql,params);
    }

    public List<String> findLeagueCodes() {
        String sql = "SELECT league_code FROM YCQ_LEAGUE_EDU GROUP BY league_code";
        return jdbcTemplate.queryForRawObjects(String.class,sql);
    }

    public List<LeagueIdTypeVo> getLeagueIdAndType(Integer limit, Integer pageNum) {
        String sql = "SELECT league_code AS queryLeagueId,league_type AS queryLeagueType FROM YCQ_LEAGUE order by league_id";
        List<Object> params = new ArrayList<>();
        if(Objects.nonNull(limit)&&Objects.nonNull(pageNum)){
            sql = sql+" limit ? offset ?";
            params.add(limit);
            params.add((pageNum-1)*limit);
        }
        return jdbcTemplate.queryForBeans(LeagueIdTypeVo.class,sql,params.toArray());
    }

    public Integer getLeagueIdAndTypeCount() {
        String sql = "SELECT count(1) FROM YCQ_LEAGUE";
        return jdbcTemplate.queryForInt(sql);
    }

    public void updateLeaderOrgDepartment() {
        String sql = "UPDATE YCQ_LEAGUE_LEADER SET org_department = (CASE org_department WHEN '001BZCY' THEN '班子成员' WHEN '006BGS' THEN '办公室'\n" +
                "WHEN '011ZZB' THEN '组织部' WHEN '006BGS' THEN '办公室'\n" +
                "WHEN '016JGDW' THEN '机关党委' WHEN '021XCB' THEN '宣传部'\n" +
                "WHEN '026QNFZB' THEN '青年发展部' WHEN '031JCZZJSB' THEN '基层组织建设部'\n" +
                "WHEN '036XX' THEN '学校部' WHEN '041SNB' THEN '少年部'\n" +
                "WHEN '046TZB' THEN '统战部' WHEN '051SNB' THEN '维护青少年权益部'\n" +
                "WHEN '056TZB' THEN '社会联络部' WHEN '061SNB' THEN '国际联络部' ELSE '其他' END) WHERE org_department in('001BZCY','006BGS','011ZZB','006BGS','016JGDW','021XCB','026QNFZB','031JCZZJSB','036XX','041SNB','046TZB','051SNB','056TZB','061SNB')\n";
        jdbcTemplate.update(sql);
    }

    public List<Long> findNeedUpdateIds() {
        String sql = "SELECT leader_id FROM YCQ_LEAGUE_LEADER WHERE org_department IS NULL";
        return jdbcTemplate.queryForRawObjects(Long.class,sql);
    }

    public void updateLeaderOrgDepartmentByIds(List<Long> ids) {
        String sql = "UPDATE YCQ_LEAGUE_LEADER SET org_department = '其他' WHERE leader_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",ids);
        npJdbcTemplate.update(sql,params);
    }

    public List<String> findLeagueCodesByParent(String leagueOrgTopCode) {
        String sql = "SELECT league_id FROM YCQ_LEAGUE_DETAIL WHERE parent_league_id = ?";
        return jdbcTemplate.queryForRawObjects(String.class,sql,leagueOrgTopCode);
    }

    public List<String> findNextLeagueIds(String leagueCode) {
        String sql = "SELECT league_id FROM YCQ_LEAGUE_DETAIL START WITH league_id= ? CONNECT BY PRIOR league_id=parent_league_id";
        return jdbcTemplate.queryForRawObjects(String.class,sql,leagueCode);
    }

    public List<NameValue> findAllCount(List<String> leagueIds) {
        String sql = "SELECT league_type_name as name ,count(1) AS num FROM YCQ_LEAGUE_DETAIL WHERE league_id in(:ids) GROUP by league_type_name";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",leagueIds);
        return npJdbcTemplate.queryForBeans(NameValue.class,sql,params);
    }

    public Integer findCount(List<String> leagueIds) {
        String sql = "SELECT count(1) FROM YCQ_MEMBER WHERE league_code in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",leagueIds);
        return npJdbcTemplate.queryForInt(sql,params);
    }

    public List<String> findAreaLeagueIds() {
        String sql = "SELECT league_code FROM YCQ_REPORT_AREA ";
        return jdbcTemplate.queryForRawObjects(String.class,sql);
    }

    public List<NameValue> findSuperAllCount() {
        String sql = "SELECT league_type_name as name ,count(1) AS num FROM YCQ_LEAGUE_DETAIL GROUP BY league_type_name ";
        return jdbcTemplate.queryForBeans(NameValue.class,sql);
    }

    public Integer findLeagueCount() {
        String sql = "SELECT count(1)  FROM YCQ_LEAGUE_DETAIL";
        return jdbcTemplate.queryForRawObject(Integer.class,sql);
    }

    public Integer findSuperCount() {
        String sql = "SELECT count(1)  FROM YCQ_MEMBER ";
        return jdbcTemplate.queryForRawObject(Integer.class,sql);
    }
}
