package com.crunii.micro.service.league.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.league.constants.LeagueConstants;
import com.crunii.micro.service.league.dto.*;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/2 14:38
 */
@Repository
public class MemberDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate baseJdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate npJdbcTemplate;

    public PageRsp<OrgMemberVo> getOrgMember(PageReq<OrgMemberDto> req) {
        StringBuilder sb = new StringBuilder("SELECT A.league_code,B.league_name,count(1) as lmNum,B.is_leaf FROM YCQ_MEMBER A LEFT JOIN YCQ_LEAGUE B ON A.league_code =B.league_code WHERE A.league_code IN(SELECT league_code FROM YCQ_LEAGUE WHERE parent_league_code = ?) ");
        OrgMemberDto query = req.getQuery();
        List<Object> params = new ArrayList<>();
        params.add(StringUtils.hasText(query.getLeagueCode())?query.getLeagueCode(): LeagueConstants.LEAGUE_ORG_TOP_CODE);
        sb.append(" GROUP BY A.league_code,B.league_name,b.is_leaf");

        int total = baseJdbcTemplate.queryForInt("SELECT count(DISTINCT(league_code)) FROM YCQ_MEMBER WHERE league_code in(SELECT league_code FROM YCQ_LEAGUE WHERE parent_league_code = ?)", params.toArray());
        List<OrgMemberVo> orgMemberVos;
        String dbType = baseJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            sb.append(" order by B.league_name");
            orgMemberVos = baseJdbcTemplate.queryForBeans(OrgMemberVo.class, sb.toString(), params.toArray());
        } else {
            orgMemberVos = new ArrayList<>(0);
        }
        PageRsp<OrgMemberVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(orgMemberVos);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public PageRsp<MemberVo> getSuperOrgLeagueMember(PageReq<MemberDto> req) {
        StringBuilder sb = new StringBuilder("SELECT * FROM (select *,row_number() over (partition by phone_number order by phone_number desc)num from YCQ_OVERAGE_YOUTH_LEAVE)  WHERE num = 1 AND league_code = ?");
        MemberDto query = req.getQuery();
        List<Object> params = new ArrayList<>();
        params.add(query.getLeagueCode());
        int total = baseJdbcTemplate.queryForInt("SELECT count（1） FROM (select *,row_number() over (partition by phone_number order by phone_number desc)num from YCQ_OVERAGE_YOUTH_LEAVE)  WHERE num = 1 AND league_code = ?", params.toArray());
        List<MemberVo> memberVos;
        String dbType = baseJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            sb.append(" order by id");
            memberVos = baseJdbcTemplate.queryForBeans(MemberVo.class, sb.toString(), params.toArray());
        } else {
            memberVos = new ArrayList<>(0);
        }
        PageRsp<MemberVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(memberVos);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public PageRsp<MemberVo> getOrgLeagueMember(PageReq<MemberDto> req) {
        StringBuilder sb = new StringBuilder("SELECT id,name,user_id,gender,identity_card_mark,position,political,nationality,phone_number,e_mail,member_number,league_branch,league_entry_time FROM YCQ_MEMBER WHERE  league_code = ?");
        MemberDto query = req.getQuery();
        List<Object> params = new ArrayList<>();
        params.add(query.getLeagueCode());
        int total = baseJdbcTemplate.queryForInt("SELECT count(1) FROM YCQ_MEMBER WHERE league_code = ?", params.toArray());
        List<MemberVo> memberVos;
        String dbType = baseJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            sb.append(" order by id");
            memberVos = baseJdbcTemplate.queryForBeans(MemberVo.class, sb.toString(), params.toArray());
        } else {
            memberVos = new ArrayList<>(0);
        }
        PageRsp<MemberVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(memberVos);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public Integer getLmNumber(String leagueCode,Integer type) {
        String sql = "SELECT league_person FROM YCQ_LEAGUE_COUNT WHERE league_code = ?";
        if(type == 1){
            sql = "SELECT count(1) FROM YCQ_MEMBER WHERE league_code in(SELECT league_id FROM YCQ_LEAGUE_DETAIL START WITH league_id= ? CONNECT BY PRIOR league_id=parent_league_id)";
        }
        return baseJdbcTemplate.queryForRawObject(Integer.class,sql,leagueCode);
    }

    public PageRsp<PageLeaderVo> pageLeader(PageReq<PageLeaderDto> req) {
        StringBuilder sb = new StringBuilder("SELECT leader_id, user_id,current_duty,name,mobile,take_current_duty_date,cadre_property,on_duty_status,check_status FROM YCQ_LEAGUE_LEADER WHERE league_code = ? AND linshi IS NULL ");
        StringBuilder sbCount = new StringBuilder("SELECT COUNT(1) FROM YCQ_LEAGUE_LEADER WHERE league_code = ? AND linshi IS NULL ");
        PageLeaderDto query = req.getQuery();
        List<Object> params = new ArrayList<>();
        params.add(query.getLeagueCode());

        if(Objects.nonNull(query.getGender())){
            sb.append(" and gender = ?");
            sbCount.append(" and gender = ?");
            params.add(query.getGender());
        }

        if(StringUtils.hasText(query.getNameOrPhone())){
            sb.append(" and (name LIKE concat('%',?,'%') OR mobile LIKE concat('%',?,'%'))");
            sbCount.append(" and (name LIKE concat('%',?,'%') OR mobile LIKE concat('%',?,'%'))");
            params.add(query.getNameOrPhone());
            params.add(query.getNameOrPhone());
        }

        int total = baseJdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<PageLeaderVo> memberVos;
        String dbType = baseJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            sb.append(" order by leader_id");
            memberVos = baseJdbcTemplate.queryForBeans(PageLeaderVo.class, sb.toString(), params.toArray());
        } else {
            memberVos = new ArrayList<>(0);
        }
        PageRsp<PageLeaderVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(memberVos);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public PageRsp<OtherLeaderVo> pageOtherLeader(PageReq<PageLeaderDto> req) {
        StringBuilder sb = new StringBuilder("SELECT leader_id,user_id,name,mobile,position_id,identity_card_mark FROM YCQ_LEAGUE_LEADER WHERE league_code = ? AND linshi IS NULL ");
        StringBuilder sbCount = new StringBuilder("SELECT COUNT(1) FROM YCQ_LEAGUE_LEADER WHERE league_code = ? AND linshi IS NULL ");
        PageLeaderDto query = req.getQuery();
        List<Object> params = new ArrayList<>();
        params.add(query.getLeagueCode());

        if(Objects.nonNull(query.getGender())){
            sb.append(" and gender = ?");
            sbCount.append(" and gender = ?");
            params.add(query.getGender());
        }

        if(StringUtils.hasText(query.getNameOrPhone())){
            sb.append(" and (name LIKE concat('%',?,'%') OR mobile = ?)");
            sbCount.append(" and (name LIKE concat('%',?,'%') OR mobile = ?)");
            params.add(query.getNameOrPhone());
            params.add(query.getNameOrPhone());
        }

        int total = baseJdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<OtherLeaderVo> memberVos;
        String dbType = baseJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            sb.append(" order by leader_id");
            memberVos = baseJdbcTemplate.queryForBeans(OtherLeaderVo.class, sb.toString(), params.toArray());
        } else {
            memberVos = new ArrayList<>(0);
        }
        PageRsp<OtherLeaderVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(memberVos);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }
}
