package com.crunii.micro.service.league.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.league.constants.LeagueConstants;
import com.crunii.micro.service.league.dto.*;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/6/1 19:34
 */
@Repository
public class YcqLeagueDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate npJdbcTemplate;


    public List<YcqLeagueCountDto> findIsHaveChidren(List<String> collect) {
        String sql = "SELECT COUNT(1) as sum,parent_league_code as league_code FROM  YOUTH_CQ.YCQ_LEAGUE WHERE parent_league_code in(:ids) GROUP BY parent_league_code ";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",collect);
        return npJdbcTemplate.queryForBeans(YcqLeagueCountDto.class,sql,params);
    }

    public List<YcqLeagueTreeVo> getOrgChildren(String leagueCode) {
        String sql = "SELECT league_id,league_code,query_type,league_name  FROM YCQ_LEAGUE WHERE 1=1 ";
        List<Object> params = new ArrayList<>();
        if(StringUtils.hasText(leagueCode)){
            sql += " and parent_league_code = ?";
        }else {
            leagueCode = LeagueConstants.LEAGUE_ORG_TOP_CODE;
            sql += " and (league_code = ? or league_code = ?)" ;
            params.add(LeagueConstants.LEAGUE_ORG_TOP_CODE_2);
        }
        sql += " ORDER BY league_name";
        params.add(leagueCode);
        return jdbcTemplate.queryForBeans(YcqLeagueTreeVo.class,sql,params.toArray());
    }

    public PageRsp<YcqLeagueVo> getOrgTabulation(PageReq<YcqLeagueDto> req) {
        StringBuilder sb = new StringBuilder("SELECT league_detail_id ,is_country,league_id,league_name,league_type_id,league_type_name,contact_telephone,industry_name FROM YCQ_LEAGUE_DETAIL WHERE 1=1 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_LEAGUE_DETAIL  WHERE 1=1 ");
        YcqLeagueDto query = req.getQuery();
        List<Object> params = new ArrayList<>();
        if(StringUtils.hasText(query.getLeagueName())){
            sb.append(" and league_name like concat('%',?,'%')");
            sbCount.append(" and league_name like concat('%',?,'%')");
            params.add(query.getLeagueName());
        }
        if(StringUtils.hasText(query.getLeagueType())){
            sb.append(" and league_type_id = ?");
            sbCount.append(" and league_type_id = ?");
            params.add(query.getLeagueType());
        }

        if (query.isLeaf()) {
            // 为最下级组织则查询上级组织信息
            sb.append(" AND parent_league_id = (SELECT parent_league_id FROM YCQ_LEAGUE_DETAIL WHERE league_id = ?)");
            sbCount.append(" AND parent_league_id = (SELECT parent_league_id FROM YCQ_LEAGUE_DETAIL WHERE league_id = ?) ");
        }else {
            sb.append(" AND parent_league_id = ?");
            sbCount.append(" AND parent_league_id = ?");
        }
        params.add(query.getLeagueCode());

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqLeagueVo> ycqLeagueVos;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            sb.append(" ORDER BY league_name");
            ycqLeagueVos = jdbcTemplate.queryForBeans(YcqLeagueVo.class, sb.toString(), params.toArray());
        } else {
            ycqLeagueVos = new ArrayList<>(0);
        }
        PageRsp<YcqLeagueVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(ycqLeagueVos);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }



    public  PageRsp<AgeDataVo> getLmAgeBigData(PageReq<PageLeagueAgeDto> req) {
        StringBuilder sb = new StringBuilder("SELECT age,male_lm,female_lm,male_lm + female_lm AS lmCount FROM YCQ_LEAGUE_AGE WHERE league_code = ?");
        List<Object> params = new ArrayList<>();
        params.add(req.getQuery().getLeagueCode());

        int total = jdbcTemplate.queryForInt("SELECT count(1) FROM YCQ_LEAGUE_AGE WHERE league_code = ? ", params.toArray());
        List<AgeDataVo> ycqLeagueVos;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            sb.append(" order by age");
            ycqLeagueVos = jdbcTemplate.queryForBeans(AgeDataVo.class, sb.toString(), params.toArray());
        } else {
            ycqLeagueVos = new ArrayList<>(0);
        }
        PageRsp<AgeDataVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(ycqLeagueVos);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<LeagueEduTabulation> getLmEduBigData(String leagueCode) {
        String sql = "SELECT edu_level,lm_count FROM  YCQ_LEAGUE_EDU WHERE league_code = ? order by sort";
        return jdbcTemplate.queryForBeans(LeagueEduTabulation.class,sql,leagueCode);
    }

    public PageRsp<MemberNationality> getLmNationalityBigData(PageReq<PageNationalityDto> req) {
        StringBuilder sb = new StringBuilder("SELECT nation_id,member_num FROM YCQ_LEAGUE_NATION WHERE league_id= ?");
        List<Object> params = new ArrayList<>();
        params.add(req.getQuery().getLeagueCode());

        int total = jdbcTemplate.queryForInt("SELECT count(1) FROM YCQ_LEAGUE_NATION WHERE league_id= ? ", params.toArray());
        List<MemberNationality> ycqLeagueVos;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            sb.append(" order by nation_id");
            ycqLeagueVos = jdbcTemplate.queryForBeans(MemberNationality.class, sb.toString(), params.toArray());
        } else {
            ycqLeagueVos = new ArrayList<>(0);
        }
        PageRsp<MemberNationality> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(ycqLeagueVos);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public LeagueOrgDetailVo getOrgDetail(String leagueCode) {
        String sql = "SELECT * FROM YCQ_LEAGUE_DETAIL WHERE league_id = ?";
        return jdbcTemplate.queryForBean(LeagueOrgDetailVo.class,sql,leagueCode);
    }



    public LeagueSuperOrgDetailVo getSuperiorOrgDetail(String leagueCode) {
        String sql = "SELECT * FROM YCQ_LEAGUE_DETAIL WHERE league_id = ?";
        return jdbcTemplate.queryForBean(LeagueSuperOrgDetailVo.class,sql,leagueCode);
    }


    public PageRsp<YcqLeagueTreeVo> pageLeague(PageReq<PageLeague> req) {
        StringBuilder sb = new StringBuilder("SELECT league_id,parent_league_code,league_code,league_name  FROM YCQ_LEAGUE WHERE 1=1");
        StringBuilder sbCount = new StringBuilder("SELECT count(1)  FROM YCQ_LEAGUE WHERE 1=1");
        PageLeague query = req.getQuery();
        List<Object> params = new ArrayList<>();
        if(StringUtils.hasText(query.getLeagueName())){
            sb.append(" and league_name like concat('%',?,'%')");
            sbCount.append(" and league_name like concat('%',?,'%')");
            params.add(query.getLeagueName());
        }

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqLeagueTreeVo> ycqLeagueVos;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            sb.append(" order by league_id");
            ycqLeagueVos = jdbcTemplate.queryForBeans(YcqLeagueTreeVo.class, sb.toString(), params.toArray());
        } else {
            ycqLeagueVos = new ArrayList<>(0);
        }
        PageRsp<YcqLeagueTreeVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(ycqLeagueVos);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public NationalityDataVo findNationalityCount(String leagueCode) {
        String sql = "SELECT (SELECT count(1) FROM YCQ_LEAGUE_NATION WHERE league_id = ?) AS lmSumCount,(SELECT count(1) FROM YCQ_LEAGUE_NATION WHERE league_id = ? AND nation_id = '01HZ') AS hanLmCount,(SELECT count(1) FROM YCQ_LEAGUE_NATION WHERE league_id = ? AND nation_id <> '01HZ') AS minorityLmCount";
        List<Object> params = new ArrayList<>();
        params.add(leagueCode);
        params.add(leagueCode);
        params.add(leagueCode);
        return jdbcTemplate.queryForBean(NationalityDataVo.class,sql,params.toArray());
    }

    public List<AgeDataVo> getLmAgeBigDataEcharts(String leagueCode) {
        String sql ="SELECT age,male_lm,female_lm,male_lm + female_lm AS lmCount FROM YCQ_LEAGUE_AGE WHERE league_code = ?";
        return jdbcTemplate.queryForBeans(AgeDataVo.class,sql,leagueCode);
    }


    public List<MemberNationality> getLmNationalityBigDataEcharts(String leagueCode) {
        String sql = "SELECT nation_id,member_num FROM YCQ_LEAGUE_NATION WHERE league_id= ?";
        return jdbcTemplate.queryForBeans(MemberNationality.class,sql,leagueCode);
    }

    public List<YcqLeagueTreeVo> findParentLeagueCode(String leagueCode) {
        String sql = "SELECT league_id,parent_league_code,league_code,league_name  FROM YCQ_LEAGUE WHERE league_code in(SELECT league_code FROM YCQ_LEAGUE START WITH league_code= ? CONNECT BY PRIOR parent_league_code=league_code) AND  league_code <> ?";
        return jdbcTemplate.queryForBeans(YcqLeagueTreeVo.class,sql,leagueCode,leagueCode);
    }

    public List<YcqLeagueTreeVo> getPublicOrgChildren(String leagueCode,String orgId) {
        String sql = "SELECT league_id,league_code,league_name  FROM YCQ_LEAGUE WHERE 1=1 ";
        List<Object> params = new ArrayList<>();
        if(StringUtils.hasText(leagueCode)){
            sql += " and parent_league_code = ?";
        }else {
            leagueCode = orgId;
            sql += " and league_code = ?" ;
        }
        sql += " ORDER BY league_name";
        params.add(leagueCode);
        return jdbcTemplate.queryForBeans(YcqLeagueTreeVo.class,sql,params.toArray());
    }

}
