package com.crunii.micro.service.league.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.league.dto.*;

import java.util.List;

/**
 * @author wangchen create 2023/5/31 17:31
 */
public interface LeagueService {





    /**
     * 查询下组织
     * @param leagueCode 组织编码
     * @return 该组织下组织
     */
    List<YcqLeagueTreeVo> getOrgChildren(String leagueCode);

    /**
     * 获取组织列表，ps:组织是最下级组织时查询父类组织
     * @param req 分页参数
     * @return 分页数据
     */
    PageRsp<YcqLeagueVo> getOrgTabulation(PageReq<YcqLeagueDto> req);



    /**
     * 获取组织年龄数量
     * @param req 分页参数
     * @return data
     */
    PageRsp<AgeDataVo> getLmAgeBigData(PageReq<PageLeagueAgeDto> req);

    /**
     * 获取组织学历
     * @param leagueCode 组织编码
     * @return data
     */
    LeagueEduVo getLmEduBigData(String leagueCode);

    /**
     * 获取组织民族
     * @param req 请求参数
     * @return data
     */
    PageRsp<MemberNationality> getLmNationalityBigData(PageReq<PageNationalityDto> req);

    /**
     * 获取团委组织详细
     * @param leagueCode 组织编码
     * @return data
     */
    LeagueOrgDetailVo getOrgDetail(String leagueCode);

    /**
     * 查询组织详情（非字节点）
     * @param leagueCode 组织code
     * @return data
     */
    LeagueSuperOrgDetailVo getSuperiorOrgDetail(String leagueCode);


    /**
     * 分页查询组织
     * @param req 请求
     * @return data
     */
    PageRsp<YcqLeagueTreeVo> pageLeague(PageReq<PageLeague> req);

    /**
     * 获取民族分布团员数量
     * @param leagueCode 组织编码
     * @return data
     */
    NationalityDataVo getLmNationalityCount(String leagueCode);


    /**
     * 获取组织年龄数量
     * @param leagueCode 组织编码
     * @return data
     */
    List<AgeDataVo> getLmAgeBigDataEcharts(String leagueCode);


    /**
     * 获取组织民族
     * @param leagueCode 组织编码
     * @return data
     */
    NationalityDataVo getLmNationalityBigDataEcharts(String leagueCode);

    /**
     * 公共查询组织
     * @param leagueCode 组织编码
     * @param worker 当前登录人id
     * @return data
     */
    List<YcqLeagueTreeVo> getPublicOrgChildren(String leagueCode, Worker worker);
}
