package com.crunii.micro.service.league.service.impl;

import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.league.constants.LeagueConstants;
import com.crunii.micro.service.league.dto.*;
import com.crunii.micro.service.league.enums.EduLevelSortEnum;
import com.crunii.micro.service.league.jdbc.LeagueSynDao;
import com.crunii.micro.service.league.service.LeagueSynService;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

/**
 * @author wangchen create 2023/7/10 9:31
 */
@Slf4j
@Service
public class LeagueSynServiceImpl implements LeagueSynService {

    @Resource
    private YcqLeagueRepo ycqLeagueRepo;

    @Resource
    private YcqLeagueAgeRepo ycqLeagueAgeRepo;

    @Resource
    private YcqLeagueEduRepo ycqLeagueEduRepo;

    @Resource
    private YcqLeagueDetailRepo ycqLeagueDetailRepo;

    @Resource
    private YcqLeagueNationRepo ycqLeagueNationRepo;

    @Resource
    private YcqLeagueLeaderRepo ycqLeagueLeaderRepo;

    @Resource
    private YcqLeagueCountRepo ycqLeagueCountRepo;

    @Resource
    private YcqLeagueUserHonorRepo ycqLeagueUserHonorRepo;

    @Resource
    private YcqLeagueLeaderCountRepo ycqLeagueLeaderCountRepo;
    
    private final LeagueSynDao leagueSynDao;

    public LeagueSynServiceImpl(LeagueSynDao leagueSynDao) {
        this.leagueSynDao = leagueSynDao;
    }

    @Override
    public List<LeagueIdTypeVo> getNotSynData() {
        return leagueSynDao.getNotSynData();
    }

    @Override
    public String synLeagueLeader(List<LeagueLeaderC> leagueLeaderC) {
        List<YcqLeagueLeader> ycqLeagueLeaders = new ArrayList<>();
        for (LeagueLeaderC leaderC : leagueLeaderC) {
            YcqLeagueLeader ycqLeagueLeader = new YcqLeagueLeader();
            BeanUtils.copyProperties(leaderC,ycqLeagueLeader);
            if(StringUtils.hasText(ycqLeagueLeader.getJoinVolunteerTime()) || StringUtils.hasText(ycqLeagueLeader.getBecomeEolunteerTime())){
                ycqLeagueLeader.setVolunteer(1);
            }else {
                ycqLeagueLeader.setVolunteer(0);
            }
            ycqLeagueLeader.setLinshi(2);
            ycqLeagueLeaders.add(ycqLeagueLeader);
        }
        ycqLeagueLeaderRepo.saveAllAndFlush(ycqLeagueLeaders);
        return "保存成功";
    }

    @Override
    public String synLeagueLeaderCount(List<YcqLeagueLeaderCountC> leagueLeaderC) {
        List<YcqLeagueLeaderCount> leagueLeaderCounts = new ArrayList<>();
        List<String> collect = leagueLeaderC.stream().map(YcqLeagueLeaderCountC::getLeagueCode).collect(Collectors.toList());
        List<String> leagueCodes = leagueSynDao.findLeagueLeaderCount(collect);
        for (YcqLeagueLeaderCountC ycqLeagueLeaderCountC : leagueLeaderC) {
            if(leagueCodes.contains(ycqLeagueLeaderCountC.getLeagueCode())){
                continue;
            }
            YcqLeagueLeaderCount ycqLeagueLeaderCount = new YcqLeagueLeaderCount();
            BeanUtils.copyProperties(ycqLeagueLeaderCountC,ycqLeagueLeaderCount);
            leagueLeaderCounts.add(ycqLeagueLeaderCount);
        }
        ycqLeagueLeaderCountRepo.saveAllAndFlush(leagueLeaderCounts);
        return "保存成功";
    }

    @Override
    public List<LeagueIdTypeVo> getLeagueIdAndType(String secret, Integer limit, Integer pageNum) {
        if(LeagueConstants.LEAGUE_SECRET.equals(secret)){
            return leagueSynDao.getLeagueIdAndType(limit,pageNum);
        }
        return null;
    }

    @Override
    public Integer getLeagueIdAndTypeCount(String secret) {
        if(LeagueConstants.LEAGUE_SECRET.equals(secret)){
            return leagueSynDao.getLeagueIdAndTypeCount();
        }
        return null;
    }

    @Override
    public String leaderCompleteNotice() {
        leagueSynDao.updateLeaderOrgDepartment();
        List<Long> ids = leagueSynDao.findNeedUpdateIds();
        if(!CollectionUtils.isEmpty(ids)){
            leagueSynDao.updateLeaderOrgDepartmentByIds(ids);
        }
        return "保存成功";
    }

    @Override
    public String saveLeagueCountDate() {
        CompletableFuture.runAsync(()->{
            List<String> leagueIds = leagueSynDao.findLeagueCodesByParent(LeagueConstants.LEAGUE_ORG_TOP_CODE);
            saveSuperLeagueCode(LeagueConstants.LEAGUE_ORG_TOP_CODE);
            List<String> set = leagueSynDao.findAreaLeagueIds();
            Set<String> leagueCodes = new HashSet<>();
            leagueCodes.addAll(leagueIds);
            leagueCodes.addAll(set);
            for (String leagueCode : leagueCodes) {
                log.info("同步组织开始------------组织编码{}>",leagueCode);
                List<String> leagueId = leagueSynDao.findNextLeagueIds(leagueCode);

                YcqLeagueCount ycqLeagueCount = ycqLeagueCountRepo.findByLeagueCode(leagueCode);
                if(Objects.isNull(ycqLeagueCount)){
                    ycqLeagueCount = new YcqLeagueCount();
                }
                List<NameValue> nameValue = leagueSynDao.findAllCount(leagueId);
                for (NameValue value : nameValue) {
                    setData(value,ycqLeagueCount);
                }
                ycqLeagueCount.setLeagueCount(leagueId.size());
                ycqLeagueCount.setLeagueCode(leagueCode);
                // 查询团员数量
                Integer personNum = leagueSynDao.findCount(leagueId);
                ycqLeagueCount.setLeaguePerson(personNum);
                ycqLeagueCountRepo.saveAndFlush(ycqLeagueCount);
                leagueIds.clear();
                log.info("同步完成组织------------组织编码{}>",leagueCode);
            }
            log.info("-------------数据同步完成------------------");
        });
        return "保存成功";
    }

    @Override
    public String synUserHonor(List<UserHonorC> userHonor) {
        List<YcqLeagueUserHonor> list = new ArrayList<>();
        for (UserHonorC userHonorC : userHonor) {
            YcqLeagueUserHonor ycqLeagueUserHonor = new YcqLeagueUserHonor();
            BeanUtils.copyProperties(userHonorC,ycqLeagueUserHonor);
            list.add(ycqLeagueUserHonor);
        }
        ycqLeagueUserHonorRepo.saveAllAndFlush(list);
        return "保存成功";
    }

    /**
     * 顶层组织处理
     */
    private void saveSuperLeagueCode(String leagueOrgTopCode) {
        YcqLeagueCount ycqLeagueCount = ycqLeagueCountRepo.findByLeagueCode(leagueOrgTopCode);
        if(Objects.isNull(ycqLeagueCount)){
            ycqLeagueCount = new YcqLeagueCount();
        }
        List<NameValue> nameValue = leagueSynDao.findSuperAllCount();
        for (NameValue value : nameValue) {
            setData(value,ycqLeagueCount);
        }
        Integer count = leagueSynDao.findLeagueCount();
        ycqLeagueCount.setLeagueCount(count);
        ycqLeagueCount.setLeagueCode(leagueOrgTopCode);
        // 查询团员数量
        Integer personNum = leagueSynDao.findSuperCount();
        ycqLeagueCount.setLeaguePerson(personNum);
        ycqLeagueCountRepo.saveAndFlush(ycqLeagueCount);
    }

    /**
     * 设置数量
     */
    private void setData(NameValue value, YcqLeagueCount ycqLeagueCount) {
        switch (value.getName()){
            case LeagueConstants.LEADING_ORGANS:
                ycqLeagueCount.setLeadingOrgans(value.getNum());
                return;
            case LeagueConstants.COMMITTEE:
                ycqLeagueCount.setCommittee(value.getNum());
                return;
            case LeagueConstants.UNION_MEMBERS:
                ycqLeagueCount.setUnionMembers(value.getNum());
                return;
            case LeagueConstants.BRANCH:
                ycqLeagueCount.setBranch(value.getNum());
                return;
            case LeagueConstants.GRADUATE_ORG:
                ycqLeagueCount.setGraduateOrg(value.getNum());
                return;
            case LeagueConstants.WAIT_BRANCH:
                ycqLeagueCount.setWaitBranch(value.getNum());
                return;
            case LeagueConstants.GEN_BRANCH:
                ycqLeagueCount.setGenBranch(value.getNum());
                return;
        }
    }


    @Override
    public String synLeagueDetail(List<YcqLeagueDetailC> ycqLeagueDetailC) {
        List<YcqLeagueDetail> list = new ArrayList<>();
        for (YcqLeagueDetailC leagueDetailC : ycqLeagueDetailC) {
            YcqLeagueDetail ycqLeagueDetail = new YcqLeagueDetail();
            BeanUtils.copyProperties(leagueDetailC,ycqLeagueDetail);
            list.add(ycqLeagueDetail);
        }
        ycqLeagueDetailRepo.saveAllAndFlush(list);
        return "success";
    }

    @Override
    public String saveSynLeagueData(YcqLeagueC ycqLeagueC,Integer type) {
        YcqLeague byLeagueCode = ycqLeagueRepo.findByLeagueCode(ycqLeagueC.getLeagueCode());
        if(Objects.isNull(byLeagueCode)){
            byLeagueCode = new YcqLeague();
        }
        BeanUtils.copyProperties(ycqLeagueC, byLeagueCode);
        byLeagueCode.setType(type);
        ycqLeagueRepo.saveAndFlush(byLeagueCode);
        return "success";
    }

    @Override
    public String synLeagueAgeData(List<YcqLeagueAgeC> ycqLeagueAgeC) {
        List<YcqLeagueAge> ycqLeagueList = new ArrayList<>();
        // 查询数据库有的实体
        for (YcqLeagueAgeC leagueAgeC : ycqLeagueAgeC) {
            String leagueName = leagueAgeC.getLeagueName();
            String leagueCode = leagueAgeC.getLeagueCode();
            List<LeagueAge> ages = leagueAgeC.getAges();
            for (LeagueAge age : ages) {
                YcqLeagueAge ycqLeague = new YcqLeagueAge();
                ycqLeague.setAge(age.getAge());
                ycqLeague.setFemaleLm(age.getWoman());
                ycqLeague.setMaleLm(age.getMan());
                ycqLeague.setLeagueCode(leagueCode);
                ycqLeague.setLeagueName(leagueName);
                ycqLeagueList.add(ycqLeague);
            }
        }
        ycqLeagueAgeRepo.saveAllAndFlush(ycqLeagueList);
        return "success";
    }

    @Override
    public String synLeagueNationData(List<YcqLeagueNationC> ycqLeagueNation) {
        List<YcqLeagueNation> list = new ArrayList<>();
        for (YcqLeagueNationC ycqLeagueNationC : ycqLeagueNation) {
            String leagueId = ycqLeagueNationC.getLeagueId();
            String leagueName = ycqLeagueNationC.getLeagueName();
            List<YcqLeagueNationList> nationList = ycqLeagueNationC.getNationList();
            if(CollectionUtils.isEmpty(nationList)){
                continue;
            }
            for (YcqLeagueNationList ycqLeagueNationList : nationList) {
                YcqLeagueNation ycqLeagueNation1 = new YcqLeagueNation();
                ycqLeagueNation1.setLeagueName(leagueName);
                ycqLeagueNation1.setNationId(ycqLeagueNationList.getNationId());
                ycqLeagueNation1.setMemberNum(ycqLeagueNationList.getMemberNum());
                ycqLeagueNation1.setLeagueId(leagueId);
                list.add(ycqLeagueNation1);
            }
        }
        ycqLeagueNationRepo.saveAllAndFlush(list);
        return "success";
    }

    @Override
    public String synLeagueEduData(List<YcqLeagueEduC> ycqLeagueEduC) {
        List<YcqLeagueEdu> ycqLeagueList = new ArrayList<>();
        // 查询数据库有的实体
        List<String> codes = leagueSynDao.findLeagueCodes();
        for (YcqLeagueEduC leagueAgeC : ycqLeagueEduC) {
            String leagueName = leagueAgeC.getLeagueName();
            String leagueCode = leagueAgeC.getLeagueCode();
            if(codes.contains(leagueCode)){
                continue;
            }
            codes.add(leagueCode);
            List<LeagueEdu> edus = leagueAgeC.getEducationList();
            for (LeagueEdu leagueEdu : edus) {
                YcqLeagueEdu ycqLeagueEdu = new YcqLeagueEdu();
                ycqLeagueEdu.setLeagueCode(leagueCode);
                ycqLeagueEdu.setLeagueName(leagueName);
                ycqLeagueEdu.setEduLevel(leagueEdu.getEduId());
                ycqLeagueEdu.setImCount(leagueEdu.getMemberNum());
                ycqLeagueEdu.setSort(EduLevelSortEnum.getLevelSort(leagueEdu.getEduId()));
                ycqLeagueList.add(ycqLeagueEdu);
            }
        }
        ycqLeagueEduRepo.saveAllAndFlush(ycqLeagueList);
        return "success";
    }
}
