package com.crunii.micro.service.league.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.stream.CollectorUtil;
import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.core.cache.TsmPropertyCache;
import com.crunii.micro.domain.entity.main.YcqLeagueLeader;
import com.crunii.micro.domain.entity.main.YcqLeagueUserHonor;
import com.crunii.micro.domain.entity.main.YcqMember;
import com.crunii.micro.domain.entity.main.YcqOverageYouthLeave;
import com.crunii.micro.domain.repo.main.YcqLeagueLeaderRepo;
import com.crunii.micro.domain.repo.main.YcqLeagueUserHonorRepo;
import com.crunii.micro.domain.repo.main.YcqMemberRepo;
import com.crunii.micro.domain.repo.main.YcqOverageYouthLeaveRepo;
import com.crunii.micro.service.league.constants.LeagueConstants;
import com.crunii.micro.service.league.dto.*;
import com.crunii.micro.service.league.enums.JobTransEnum;
import com.crunii.micro.service.league.jdbc.MemberDao;
import com.crunii.micro.service.league.service.MemberService;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/1 19:30
 */
@Service
public class MemberServiceImpl implements MemberService {

    @Resource
    private YcqMemberRepo ycqMemberRepo;

    @Resource
    private YcqOverageYouthLeaveRepo leaveRepo;

    @Resource
    private YcqLeagueLeaderRepo ycqLeagueLeaderRepo;

    @Resource
    private MemberDao memberDao;

    @Resource
    private YcqLeagueUserHonorRepo ycqLeagueUserHonorRepo;

    @Resource
    private TsmPropertyCache tsmPropertyCache;

    @Override
    public String saveMemberData(List<YcqMemberC> list) {
        List<YcqMember> memberList = new ArrayList<>();
        list.forEach(x -> {
            YcqMember ycqMember = new YcqMember();
            BeanUtils.copyProperties(x, ycqMember);
            memberList.add(ycqMember);
        });
        // 保存
        ycqMemberRepo.saveAllAndFlush(memberList);
        return "success";

        //List<String> collect = list.stream().map(YcqMemberC::getUserId).collect(Collectors.toList());
        //List<String> hasId = memberDao.findNotInData(collect);
        //for (YcqMemberC ycqMemberC : list) {
        //    if(hasId.contains(ycqMemberC.getUserId())){
        //        continue;
        //    }
        //    YcqMember ycqMember = new YcqMember();
        //    BeanUtils.copyProperties(ycqMemberC, ycqMember);
        //    memberList.add(ycqMember);
        //}
    }

    @Override
    public String saveOverageYouthLeaveData(List<YcqOverageYouthLeaveC> list) {
        List<YcqOverageYouthLeave> youthList = new ArrayList<>();
        list.forEach(x -> {
            YcqOverageYouthLeave youthLeave = new YcqOverageYouthLeave();
            BeanUtils.copyProperties(x, youthLeave);
            youthList.add(youthLeave);
        });
        // 保存
        leaveRepo.saveAllAndFlush(youthList);
        return "success";
    }

    @Override
    public PageRsp<OrgMemberVo> getOrgMember(PageReq<OrgMemberDto> req) {
        return memberDao.getOrgMember(req);
    }

    @Override
    @InjectProperty
    public PageRsp<MemberVo> getOrgLeagueMember(PageReq<MemberDto> req) {
        // 分开查询，放页面展示变动
        MemberDto query = req.getQuery();
        if (LeagueConstants.SUPER_MEMBER_LEAGUE.equals(query.getQueryType())) {
            // 超龄团员查询
            PageRsp<MemberVo> superOrgLeagueMember = memberDao.getSuperOrgLeagueMember(req);
            handleSuperOrgLeague(superOrgLeagueMember);
            return superOrgLeagueMember;
        }
        // 团员查询
        return memberDao.getOrgLeagueMember(req);
    }

    /**
     * 超龄团员处理
     */
    private void handleSuperOrgLeague(PageRsp<MemberVo> superOrgLeagueMember) {
        List<MemberVo> result = superOrgLeagueMember.getResult();
        if(CollectionUtils.isEmpty(result)){
            return;
        }
        for (MemberVo memberVo : result) {
            String occupation = memberVo.getOccupation();
            if(StringUtils.hasText(occupation)){
                String[] split = occupation.split(":");
                if(split.length == 2){
                    String nameTrans = JobTransEnum.getNameTrans(split[0]);
                    memberVo.setJob(nameTrans);
                    String value = tsmPropertyCache.get("youngStudents", split[1]);
                    memberVo.setJobLearWorker(value);
                }
            }
        }
    }

    @Override
    public Integer getLmNumber(String leagueCode) {
        if (!StringUtils.hasText(leagueCode)) {
            throw new BusinessException("组织编码不能为空");
        }
        Integer lmNumber = memberDao.getLmNumber(leagueCode,2);
        if(Objects.isNull(lmNumber)){
            lmNumber = memberDao.getLmNumber(leagueCode,1);
        }
        return Objects.isNull(lmNumber)?0:lmNumber;
    }


    @Override
    public Object pageLeader(PageReq<PageLeaderDto> req) {
        return handleQueryType(req);
    }

    /**
     * 处理数据
     */
    private Object handleQueryType(PageReq<PageLeaderDto> req) {
        Integer queryType = req.getQuery().getQueryType();
        if(Objects.isNull(queryType)){
            queryType = 1;
        }
        if(queryType == 1){
            PageRsp<PageLeaderVo> rsp = memberDao.pageLeader(req);
            List<PageLeaderVo> result = rsp.getResult();
            if(CollectionUtils.isEmpty(result)){
                return rsp;
            }
            for (PageLeaderVo pageLeaderVo : result) {
                if(StringUtils.hasText(pageLeaderVo.getCurrentDuty())){
                    String value = tsmPropertyCache.get("leaguePosition", pageLeaderVo.getCurrentDuty());
                    pageLeaderVo.setCurrentDutyTrans(value);
                }
                if(StringUtils.hasText(pageLeaderVo.getCadreProperty())){
                    String value = tsmPropertyCache.get("natures", pageLeaderVo.getCadreProperty());
                    pageLeaderVo.setCadrePropertyTrans(value);
                }
                if(StringUtils.hasText(pageLeaderVo.getOnDutyStatus())){
                    String value = tsmPropertyCache.get("onDutyStatuss", pageLeaderVo.getOnDutyStatus());
                    pageLeaderVo.setOnDutyStatusTrans(value);
                }
            }
            return rsp;
        }else {
            PageRsp<OtherLeaderVo> otherLeaderVos = memberDao.pageOtherLeader(req);
            List<OtherLeaderVo> result = otherLeaderVos.getResult();
            if(CollectionUtils.isEmpty(result)){
                return otherLeaderVos;
            }
            for (OtherLeaderVo otherLeaderVo : result) {
                if(StringUtils.hasText(otherLeaderVo.getPositionId())){
                    String value = tsmPropertyCache.get("LeaguePositionsTwo", otherLeaderVo.getPositionId());
                    otherLeaderVo.setPositionIdTrans(value);
                }
                if(StringUtils.hasText(otherLeaderVo.getIdentityCardMark())){
                    String value = tsmPropertyCache.get("identityCardMarkCode", otherLeaderVo.getIdentityCardMark());
                    otherLeaderVo.setIdentityCardMarkTrans(value);
                }
            }
            return otherLeaderVos;
        }
    }

    @Override
    public Object getLeaderDetail(Long leaderId,Integer queryType) {
        if(Objects.isNull(leaderId)){
            throw new BusinessException("id不能为空");
        }
        YcqLeagueLeader one = ycqLeagueLeaderRepo.getOne(leaderId);
        if(Objects.isNull(one)){
            return null;
        }
        return chooseAndReturn(one,queryType);
    }

    @Override
    public List<String> getHonor(String userId) {
        List<String> honorList = new ArrayList<>();
        if(!StringUtils.hasText(userId)){
            return honorList;
        }
        List<YcqLeagueUserHonor> userHonors = ycqLeagueUserHonorRepo.findByUserId(userId);
        if(CollectionUtils.isEmpty(userHonors)){
            return honorList;
        }
        for (YcqLeagueUserHonor userHonor : userHonors) {
            String honorName = userHonor.getHonorName();
            String honorStruct = userHonor.getHonorStruct();
            if(StringUtils.hasText(honorName)){
                honorList.add(honorName);
            }
            if(StringUtils.hasText(honorStruct)){
                HonorVo honorVo = JSONUtil.toBean(honorStruct, HonorVo.class);
                honorList.add(honorVo.getTime()+"担任"+honorVo.getLeagueName()+honorVo.getPosition());
            }
        }
        return honorList;
    }

    private Object chooseAndReturn(YcqLeagueLeader one, Integer queryType) {
        if(Objects.isNull(queryType)){
            queryType = 1;
        }
        switch (queryType){
            case 0:
                return handleQueryTypeZero(one);
            case 1:
                LeaderDetailVo leaderDetailVo = new LeaderDetailVo();
                BasicLeaderVo basicLeaderVo = handleBasicLeader(one);
                LeaderWorkerVo leaderWorkerVo =  handleLeaderWorker(one);
                leaderDetailVo.setLeaderWorker(leaderWorkerVo);
                leaderDetailVo.setBasicLeader(basicLeaderVo);
                return leaderDetailVo;
            case 2:
                return handleQueryTypeTwo(one);
        }
        return null;
    }

    /**
     * 处理queryType类型为0的数据
     */
    private LeaderDetailZeroVo handleQueryTypeZero(YcqLeagueLeader one) {
        LeaderDetailZeroVo leaderDetailVo = new LeaderDetailZeroVo();
        BasicLeaderZeroVo basicLeaderZeroVo = new BasicLeaderZeroVo();
        BeanUtils.copyProperties(one,basicLeaderZeroVo);
        leaderDetailVo.setBasicLeaderZero(basicLeaderZeroVo);
        return leaderDetailVo;
    }

    /**
     * 处理queryType类型为2的数据
     */
    private LeaderDetailTwoVo handleQueryTypeTwo(YcqLeagueLeader one) {
        LeaderDetailTwoVo leaderDetailVo = new LeaderDetailTwoVo();
        BasicLeaderTwoVo basicLeaderTwoVo = new BasicLeaderTwoVo();
        BeanUtils.copyProperties(one,basicLeaderTwoVo);
        leaderDetailVo.setBasicLeaderTwo(basicLeaderTwoVo);
        return leaderDetailVo;
    }

    /**
     * 封装worker信息
     */
    private LeaderWorkerVo handleLeaderWorker(YcqLeagueLeader one) {
        LeaderWorkerVo leaderWorkerVo = new LeaderWorkerVo();
        BeanUtils.copyProperties(one,leaderWorkerVo);
        // 静态值翻译
        if(StringUtils.hasText(leaderWorkerVo.getCurrentDuty())){
            String value = tsmPropertyCache.get("leaguePosition", leaderWorkerVo.getCurrentDuty());
            leaderWorkerVo.setCurrentDutyTrans(value);
        }
        if(StringUtils.hasText(leaderWorkerVo.getCurrentDutyLevel())){
            String value = tsmPropertyCache.get("dutyLevels", leaderWorkerVo.getCurrentDutyLevel());
            leaderWorkerVo.setCurrentDutyLevelTrans(value);
        }
        if(StringUtils.hasText(leaderWorkerVo.getOtherOrgDuty())){
            String value = tsmPropertyCache.get("leaguePosition", leaderWorkerVo.getOtherOrgDuty());
            leaderWorkerVo.setOtherOrgDutyTrans(value);
        }
        if(StringUtils.hasText(leaderWorkerVo.getEstablishment())){
            String value = tsmPropertyCache.get("establishmentTypes", leaderWorkerVo.getEstablishment());
            leaderWorkerVo.setEstablishmentTrans(value);
        }
        if(StringUtils.hasText(leaderWorkerVo.getGenerationType())){
            String value = tsmPropertyCache.get("generationTypes", leaderWorkerVo.getGenerationType());
            leaderWorkerVo.setGenerationTypeTrans(value);
        }
        if(StringUtils.hasText(leaderWorkerVo.getBeginDuty())){
            String value = tsmPropertyCache.get("leaguePosition", leaderWorkerVo.getBeginDuty());
            leaderWorkerVo.setBeginDutyTrans(value);
        }
        if(StringUtils.hasText(leaderWorkerVo.getOnDutyStatus())){
            String value = tsmPropertyCache.get("onDutyStatuss", leaderWorkerVo.getOnDutyStatus());
            leaderWorkerVo.setOnDutyStatusTrans(value);
        }
        if(StringUtils.hasText(leaderWorkerVo.getCadreProperty())){
            String value = tsmPropertyCache.get("natures", leaderWorkerVo.getCadreProperty());
            leaderWorkerVo.setCadrePropertyTrans(value);
        }

        if(StringUtils.hasText(leaderWorkerVo.getBeginDutyLevel())){
            String value = tsmPropertyCache.get("dutyLevels", leaderWorkerVo.getBeginDutyLevel());
            leaderWorkerVo.setBeginDutyLevelTrans(value);
        }
        if(StringUtils.hasText(leaderWorkerVo.getSampCcpDuty())){
            String value = tsmPropertyCache.get("leaguePosition", leaderWorkerVo.getSampCcpDuty());
            leaderWorkerVo.setSampCcpDutyTrans(value);
        }
        if(StringUtils.hasText(leaderWorkerVo.getCurrentLeagueDuty())){
            String value = tsmPropertyCache.get("leaguePosition", leaderWorkerVo.getCurrentLeagueDuty());
            leaderWorkerVo.setCurrentLeagueDutyTrans(value);
        }

        if(StringUtils.hasText(leaderWorkerVo.getSuperiorLeagueDuty())){
            String value = tsmPropertyCache.get("leaguePosition", leaderWorkerVo.getSuperiorLeagueDuty());
            leaderWorkerVo.setSuperiorLeagueDutyTrans(value);
        }

        return leaderWorkerVo;
    }

    /**
     * 封装基本信息
     */
    private BasicLeaderVo handleBasicLeader(YcqLeagueLeader one) {
        BasicLeaderVo basicLeaderVo = new BasicLeaderVo();
        BeanUtils.copyProperties(one,basicLeaderVo);
        // 处理年龄
        if(StringUtils.hasText(basicLeaderVo.getBirthDay())){
            DateTime yyyyMMdd = DateUtil.parse("19841228", "yyyyMMdd");
            String format = DateUtil.format(yyyyMMdd, "yyyy-MM-dd");
            int age = DateUtil.age(yyyyMMdd, new Date());
            basicLeaderVo.setBirthDay(format);
            basicLeaderVo.setAge(age);
        }
        // 静态值处理
        if(StringUtils.hasText(basicLeaderVo.getNationalId())){
            String leagueNationality = tsmPropertyCache.get("leagueNationality", basicLeaderVo.getNationalId());
            basicLeaderVo.setNationalIdTrans(leagueNationality);
        }
        if(StringUtils.hasText(basicLeaderVo.getOnTheJobTrainingEd())){
            String leagueNationality = tsmPropertyCache.get("leagueEduLevel", basicLeaderVo.getOnTheJobTrainingEd());
            basicLeaderVo.setOnTheJobTrainingEdTrans(leagueNationality);
        }
        if(StringUtils.hasText(basicLeaderVo.getMaxFullTimeEducationDegree())){
            String leagueNationality = tsmPropertyCache.get("leagueNationality", basicLeaderVo.getMaxFullTimeEducationDegree());
            basicLeaderVo.setMaxFullTimeEducationDegreeTrans(leagueNationality);
        }
        if(StringUtils.hasText(basicLeaderVo.getAcademicDegrees())){
            String value = tsmPropertyCache.get("academicDegrees", basicLeaderVo.getAcademicDegrees());
            basicLeaderVo.setAcademicDegreeTrans(value);
        }
        if(StringUtils.hasText(basicLeaderVo.getPoliticalId())){
            String value = tsmPropertyCache.get("politicals", basicLeaderVo.getPoliticalId());
            basicLeaderVo.setPoliticalIdTrans(value);
        }
        return basicLeaderVo;
    }

}
