package com.crunii.micro.service.match.api.bo;

import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Data
@EqualsAndHashCode
@ApiModel("web-青春赛事-新增-请求参数")
public class FinancingBo {

    /*----------------------------------------赛事信息----------------------------------------*/
    @ApiModelProperty(value = "赛事名称")
    private String matchName;

    @ApiModelProperty(value = "赛事分类")
    private String matchType;

    @ApiModelProperty(value = "赛项名称")
    private String matchItem;

    @ApiModelProperty(value = "组别名称")
    private String matchGroup;

    @ApiModelProperty(value = "参赛者姓名")
    private String participantName;

    @ApiModelProperty(value = "参赛者手机号")
    private String participantMobil;

    @ApiModelProperty(value = "获奖名称")
    private String prizeName;

    @ApiModelProperty(value = "编码")
    private String code;

    /*----------------------------------------项目信息----------------------------------------*/
    @ApiModelProperty(value = "项目启动时间")
    private Date startTime;

    @ApiModelProperty(value = "项目所在地")
    private String address;

    @ApiModelProperty(value = "项目简介")
    private String itemDesc;

    @ApiModelProperty(value = "项目展示(逗号隔开)")
    private String showImages;

    @ApiModelProperty(value = "单位名称")
    private String companyName;

    @ApiModelProperty(value = "单位职务")
    private String position;

    @ApiModelProperty(value = "注册地址")
    private String registerAddress;

    @ApiModelProperty(value = "注册时间")
    private Date regiserTime;

    @ApiModelProperty(value = "注册资本")
    private BigDecimal regiserCapital;

    @ApiModelProperty(value = "法人")
    private String legalRepresentative;

    @ApiModelProperty(value = "所属行业")
    private String industry;

    @ApiModelProperty(value = "企业负债金额")
    private BigDecimal enterpriseDebtAmount;

    @ApiModelProperty(value = "上一年营收")
    private BigDecimal previousYearRevenue;

    @ApiModelProperty(value = "上一年税收")
    private BigDecimal previousYearTaxRevenue;

    @ApiModelProperty(value = "员工数")
    private Integer employeeCount;

    @ApiModelProperty(value = "项目logo")
    private Long logo;

    @ApiModelProperty(value = "项目名称")
    private String name;

    @ApiModelProperty(value = "团队成员数")
    private Integer teamTotal;

    /*----------------------------------------融资信息----------------------------------------*/
    @ApiModelProperty(value = "一句话介绍")
    private String introduce;

    @ApiModelProperty(value = "期望融资")
    private BigDecimal expectedAmount;

    @ApiModelProperty(value = "出让股权(%)")
    private BigDecimal transferEquity;

    @ApiModelProperty(value = "官方网址")
    private String officialWebsite;

    @ApiModelProperty(value = "融资历史")
    private List<History> historyList;

    @ApiModelProperty(value = "核心成员")
    private List<Member> memberList;

    @Data
    @EqualsAndHashCode
    @ApiModel("融资历史")
    public static class History {

        @ApiModelProperty(value = "融资轮次")
        private String financingRounds;

        @ApiModelProperty(value = "融资时间")
        private String financingTime;

        @ApiModelProperty(value = "融资金额(万元)")
        private BigDecimal amount;

        @ApiModelProperty(value = "对外公布融资金额(0否1是)")
        private Long publicityAmount;

        @ApiModelProperty(value = "投资方")
        @Length(max = 255, message = "投资方长度不能超过255", groups = {AddGroup.class, EditGroup.class})
        private String investor;

        @ApiModelProperty(value = "对外公布投资方(0否1是)")
        private Long publicityInvestor;

        @ApiModelProperty(value = "排序")
        private Long sortNum;
    }

    @Data
    @EqualsAndHashCode
    @ApiModel("核心成员")
    public static class Member {

        @ApiModelProperty(value = "姓名")
        @Length(max = 255, message = "姓名长度不能超过255", groups = {AddGroup.class, EditGroup.class})
        private String name;

        @ApiModelProperty(value = "职务")
        @Length(max = 255, message = "职务长度不能超过255", groups = {AddGroup.class, EditGroup.class})
        private String position;

        @ApiModelProperty(value = "个人经历")
        @Length(max = 1024, message = "个人经历长度不能超过1024", groups = {AddGroup.class, EditGroup.class})
        private String personalExperience;

        @ApiModelProperty(value = "头像")
        private Long headSculpture;

        @ApiModelProperty(value = "排序")
        private Long sortNum;
    }
}