package com.crunii.micro.service.match.api.bo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * @author denggl
 * date 2023/11/16
 */
@Data
@EqualsAndHashCode
@ApiModel("承办方赛事场次业务对象")
public class MthContractorRoundInstanceBo {

    @ApiModelProperty("id")
    private Long id;

    /**
     * 比赛id
     */
    @ApiModelProperty(value = "比赛id", required = true)
    @NotNull(message = "比赛id必填")
    private Long matchId;



    /**
     * 场次名称
     */
    @ApiModelProperty(value = "场次名称", required = true)
    @NotBlank(message = "场次名称必填")
    private String roundName;

    /**
     * 比赛方式（线上 线下）
     */
    @ApiModelProperty(value = "比赛方式（线上online 线下offline）", required = true)
    @NotBlank(message = "比赛方式必填")
    private String playMode;

    /**
     * 评审方式(线上 线下)
     */
    @ApiModelProperty(value = "评审方式(线上online 线下offline)", required = true)
    @NotBlank(message = "评审方式必填")
    private String auditMode;

    /**
     * 评分方式(10分制、百分制、ABC、通过不通过))
     */
    @ApiModelProperty(value = "评分方式(10分制、百分制、ABC、通过不通过))", required = true)
    //@NotBlank(message = "评分方式必填")
    private String socreMode;

    /**
     * 统计分方式(平均分、去掉最高最低))
     */
    @ApiModelProperty(value = "统计分方式(平均分 avg、去掉最高最低 excludeMaxMin))", required = true)
  //  @NotBlank(message = "统计分方式必填")
    private String scoreStatistics;

    /**
     * 晋级方式(人数、百分比)
     */
    @ApiModelProperty(value = "晋级方式(人数、百分比)", required = true)
    @NotBlank(message = "晋级方式必填")
    private String promotionMode;

    /**
     * 晋级数量
     */
    @ApiModelProperty(value = "晋级数量", required = true)
    private Integer promptionNum;

    /**
     * 报名开始时间
     */
    @ApiModelProperty(value = "报名开始时间", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    //@NotNull(message = "报名开始时间必填")
    private Date registerationStart;

    /**
     * 报名结束时间
     */
    @ApiModelProperty(value = "报名结束时间", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
  //  @NotNull(message = "报名结束时间必填")
    private Date registerationEnd;

    /**
     * 作品提交方式（单独上传、不需上传）
     */
    @ApiModelProperty(value = "作品提交方式（单独上传、不需上传）", required = true)
    @NotBlank(message = "作品提交方式必填")
    private String entrySubmitMode;

    /**
     * 作品截止时间
     */
    @ApiModelProperty(value = "作品截止时间", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
   // @NotNull(message = "作品截止时间必填")  //上传作品时候需要校验
    private Date entryDeadline;

    /**
     * 场次开始时间  不需要必传
     */
    @ApiModelProperty(value = "场次开始时间", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date roundStart;

    /**
     * 场次结束时间 不需要必传
     */
    @ApiModelProperty(value = "场次结束时间", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date roundEnd;

    /**
     * 比赛等级（市、片区、区县、校区）
     */
    @ApiModelProperty(value = "比赛等级（市、片区、区县、校区）", required = true)
    @NotBlank(message = "比赛等级必填")
    private String matchLevel;

    /**
     * 场次等级初赛复赛(1234数字递增)
     */
    @ApiModelProperty(value = "场次等级初赛复赛(1234数字递增)", required = true)
    private Integer roundLevel;

    /**
     * 场次序号
     */
    @ApiModelProperty(value = "场次序号", required = true)
    private Integer roundSort;

    /**
     * 区域id
     */
    @ApiModelProperty(value = "区域id", required = true)
    private Long areaId;

    /**
     * 区域名称
     */
    @NotBlank(message = "区域名称必填")
    @ApiModelProperty(value = "区域名称", required = true)
    private String areaName;

   //不需必传
    private List<MthRoundPrizeBo> prizeBoList;

    private  List<MthRoundAreaBo> areaBoList;


    /**
     * 是否公示
     */
    @ApiModelProperty(value = "是否公示", required = true)
    @NotNull(message = "是否公示必填")
    private Integer publicityFlag;

    /**
     * 公示天数
     */
    @ApiModelProperty(value = "公示天数")
    private Integer publicityDays;

    /**
     * 名次设置类型(组别、赛项、 比赛)
     */
    @ApiModelProperty(value = "名次设置类型(组别、赛项、 比赛)", required = true)
    private String prizeType;

    /**
     * 评审标准（按赛项、按比赛）
     */
    @ApiModelProperty(value = "评审标准（按赛项、按比赛）")
    private String reviewerStandar;

    /**
     * 评分标准
     */
    @ApiModelProperty(value = "评分标准", required = true)
    private  List<MthRoundReviewStandarBo> reviewStandarBoList;

    /**
     * 签到信息
     */
    @ApiModelProperty(value = "签到信息")
    private SignInfoVo signInInfoVo;

    /**
     * 签退信息
     */
    @ApiModelProperty(value = "签退信息")
    private SignInfoVo signOutInfoVo;




}
