package com.crunii.micro.service.match.api.bo;

import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.service.match.api.FileDto;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * 赛事信息业务对象 mth_match_info
 *
 * @author mth
 * @date 2023-09-19
 */

@Data
@EqualsAndHashCode
@ApiModel("赛事信息业务对象")
public class MthMatchInfoBo {

    @ApiModelProperty("id")
    private Long id;

    /**
     * 比赛名称;比赛名称
     */
    @ApiModelProperty(value = "比赛名称", required = false)
    @NotBlank(message = "比赛名称不能为空", groups = {  EditGroup.class })
    private String matchName;

    /**
     * 分类code;类别
     */
    @ApiModelProperty(value = "分类code", required = false)
    private Long categoryId;

    /**
     * 分类名称
     */
    @ApiModelProperty(value = "分类名称", required = false)
    @NotBlank(message = "分类名称不能为空", groups = {EditGroup.class })
    private String categoryName;


    /**
     * 标签
     */
    @ApiModelProperty(value = "标签列表")
    private List<MthMatchLabelBo> labelList;

    /**
     * banner
     */
    @ApiModelProperty(value = "banner", required = false)
    private Long bannerId;

    @ApiModelProperty(value = "赛项")
    private List<MthMatchItemBo> matchItemBoList;

    /**
     * 报名开始时间
     */
    @ApiModelProperty(value = "报名开始时间", required = false)
    @NotNull(message = "报名开始时间不能为空", groups = {EditGroup.class })
    private Date registerationStart;

    /**
     * 报名结束时间
     */
    @ApiModelProperty(value = "报名结束时间", required = false)
    @NotNull(message = "报名结束时间不能为空", groups = {EditGroup.class })
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date registerationEnd;

    /**
     * 报名审核标志
     *
     * git test
     */
    @ApiModelProperty(value = "报名审核标志", required = false)
    @NotNull(message = "报名审核标志不能为空", groups = {EditGroup.class })
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Integer registerationAuditFlag;

    /**
     * 发布组织id
     */
    @ApiModelProperty(value = "发布组织id", required = false)
    private Long publishOrgId;

    /**
     * 发布组织
     */
    @ApiModelProperty(value = "发布组织", required = false)
    private String publishOrgName;

    /**
     * 发布人
     */
    @ApiModelProperty(value = "发布人", required = false)
    private Long publisherId;

    /**
     * 发布人
     */
    @ApiModelProperty(value = "发布人", required = false)
    private String publisherName;

    /**
     * 发布时间
     */
    @ApiModelProperty(value = "发布时间", required = false)
    private Date publishTime;

    /**
     * 状态
     */
    @ApiModelProperty(value = "状态", required = false)
    private String status;

    /**
     * 类型
     */
    @ApiModelProperty(value = "类型")
    private String matchType;

    @ApiModelProperty(value = "主办方")
    private List<MthMatchSponsorBo> sponsorBoList;

    @ApiModelProperty(value = "承办方")
    private List<MthMatchSponsorBo> organizerBoList;

    @ApiModelProperty(value = "图片")
    private FileDto fileDto;


    @ApiModelProperty("统计类别")
    private String statisticsType;

    @ApiModelProperty("报名方式（统一报名、承办单位确定）")
    private String registerationType;

    @ApiModelProperty("年龄限制方式（按比赛、按组别）")
    private String ageLimit;

    @ApiModelProperty("开始年龄")
    private String ageBegin;

    @ApiModelProperty("结束年龄")
    private String ageEnd;

    @ApiModelProperty("比赛等级(逗号分割)（直接创建场次模板)）")
    private String matchLevel;

    @ApiModelProperty(value = "是否青创赛事(1是0否)")
    private Integer qcFlag;

    @ApiModelProperty(value = "其他主办方")
    @Length(max = 64,message = "其他主办方限制输入64个字符!", groups = {AddGroup.class, EditGroup.class })
    private String otherSponsor;

    @ApiModelProperty(value = "其他承办方")
    @Length(max = 64,message = "其他承办方限制输入64个字符!", groups = {AddGroup.class, EditGroup.class })
    private String otherOrganizer;
}
