package com.crunii.micro.service.match.api.bo;

import com.crunii.micro.common.utils.ValidatorUtils;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.Range;

import javax.persistence.Column;
import javax.validation.constraints.Max;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * APP赛事活动报名作品-入参
 *
 * @author mth
 * @date 2023-09-19
 */

@Data
@EqualsAndHashCode
@ApiModel("APP赛事活动报名作品-入参")
public class MthRegisterAddAppBo {

    /**
     * 报名表
     */
    @ApiModelProperty(value = "报名表", required = true)
    private Register register;

    /**
     * 作品表-随报名上传
     */
    @ApiModelProperty(value = "作品表-随报名上传")
    private Work work;

    /**
     * 融资需求
     */
    @ApiModelProperty(value = "融资需求")
    private Financing financing;

    @Data
    @EqualsAndHashCode
    @ApiModel("报名表")
    public static class Register {
        @ApiModelProperty("报名id")
        @NotNull(message = "报名id不能为空", groups = { EditGroup.class })
        private Long id;

        /**
         * 赛事活动id
         */
        @ApiModelProperty(value = "赛事活动id", required = true)
        @NotBlank(message = "赛事活动id不能为空", groups = { AddGroup.class, EditGroup.class })
        private Long matchId;

        /**
         * 赛事活动名称
         */
        @ApiModelProperty(value = "赛事活动名称", required = true)
        @NotBlank(message = "赛事活动名称不能为空", groups = { AddGroup.class, EditGroup.class })
        private String matchName;

        /**
         * 赛项id
         */
        @ApiModelProperty(value = "赛项id", required = true)
        @NotBlank(message = "赛项id不能为空", groups = { AddGroup.class, EditGroup.class })
        private Long itemId;

        /**
         * 赛项名称
         */
        @ApiModelProperty(value = "赛项名称", required = true)
        @NotBlank(message = "赛项名称不能为空", groups = { AddGroup.class, EditGroup.class })
        private String itemName;

        /**
         * 组别id
         */
        @ApiModelProperty(value = "组别id", required = true)
        @NotBlank(message = "组别id不能为空", groups = { AddGroup.class, EditGroup.class })
        private Long groupId;

        /**
         * 组别名称
         */
        @ApiModelProperty(value = "组别名称", required = true)
        @NotBlank(message = "组别名称不能为空", groups = { AddGroup.class, EditGroup.class })
        private String groupName;

        /**
         * 区域id
         */
        @ApiModelProperty(value = "区域id", required = true)
        @NotBlank(message = "区域id不能为空", groups = { AddGroup.class, EditGroup.class })
        private Long areaId;

        /**
         * 区域名称
         */
        @ApiModelProperty(value = "区域名称", required = true)
        @NotBlank(message = "区域名称不能为空", groups = { AddGroup.class, EditGroup.class })
        private String areaName;

        /**
         * 场次id
         */
        @ApiModelProperty("场次id")
        @NotBlank(message = "场次id不能为空", groups = { AddGroup.class, EditGroup.class })
        private Long roundId;

        /**
         * 配置id
         */
        @ApiModelProperty("配置id")
        @NotBlank(message = "配置id不能为空", groups = { AddGroup.class, EditGroup.class })
        private Long registerConfigId;

        /**
         * 用户id
         */
        @ApiModelProperty(value = "用户id", required = true)
        private String userId;

        /**
         * 用户名称
         */
        @ApiModelProperty(value = "用户名称", required = true)
        private String userName;

        /**
         * 联系电话
         */
        @ApiModelProperty(value = "联系电话", required = true)
        private String telephone;

        /**
         * 报名参数详情
         */
        @ApiModelProperty(value = "报名参数详情", required = true)
        @NotEmpty(message = "报名参数详情不能为空", groups = { AddGroup.class, EditGroup.class })
        private List<Detail> detailList;
    }


    @Data
    @EqualsAndHashCode
    @ApiModel("作品表")
    public static class Work {

        @ApiModelProperty("作品id")
        @NotNull(message = "作品id不能为空", groups = { EditGroup.class })
        private Long id;

        /**
         * 报名id
         */
        @ApiModelProperty("报名id")
        private Long registerId;

        /**
         * 配置id
         */
        @ApiModelProperty("配置id")
        @NotBlank(message = "配置id不能为空", groups = { AddGroup.class, EditGroup.class })
        private Long registerConfigId;

        /**
         * 作品参数详情
         */
        @ApiModelProperty(value = "作品参数详情", required = true)
        @NotEmpty(message = "作品参数详情不能为空", groups = { AddGroup.class, EditGroup.class })
        private List<Detail> detailList;
    }

    @Data
    @EqualsAndHashCode
    @ApiModel("详情")
    public static class Detail {

        @ApiModelProperty(value = "报名详情id")
        @NotNull(message = "报名详情id不能为空", groups = {EditGroup.class})
        private Long id;

        /**
         * 报名配置详情id
         */
        @ApiModelProperty(value = "报名配置详情id", required = true)
        @NotNull(message = "报名配置详情id不能为空", groups = {AddGroup.class, EditGroup.class})
        private Long configDetailId;

        /**
         * 配置id
         */
        @ApiModelProperty(value = "配置id", required = true)
        @NotNull(message = "配置id不能为空", groups = {AddGroup.class, EditGroup.class})
        private Long configId;

        /**
         * 配置中的key
         */
        @ApiModelProperty(value = "配置中的key", required = true)
        @NotBlank(message = "配置中的key不能为空", groups = {AddGroup.class, EditGroup.class})
        private String key;

        /**
         * 用户提交数据
         */
        @ApiModelProperty(value = "用户提交数据", required = true)
        @NotBlank(message = "用户提交数据不能为空", groups = {AddGroup.class, EditGroup.class})
        private String value;

        /**
         * 类型(报名信息、作品信息)
         */
        @ApiModelProperty(value = "类型(报名信息、作品信息)", required = true)
        @NotBlank(message = "类型(报名信息、作品信息)不能为空", groups = {AddGroup.class, EditGroup.class})
        private String type;

    }


    @Data
    @EqualsAndHashCode
    @ApiModel("青创赛事融资需求")
    public static class Financing {
        /**
         * 一句话介绍
         */
        @ApiModelProperty(value = "一句话介绍")
        @Length(max = 1024, message = "一句话介绍长度不能超过1024", groups = { AddGroup.class, EditGroup.class })
        private String introduce;
        /**
         * 期望融资金额(万)
         */
        @ApiModelProperty(value = "期望融资金额(万)")
        private BigDecimal expectedAmount;
        /**
         * 出让股权(%)
         */
        @ApiModelProperty(value = "出让股权(%)")
        @Range(min = 0, max = 100, message = "出让股权范围在0-100之间", groups = { AddGroup.class, EditGroup.class })
        private BigDecimal transferEquity;
        /**
         * 官方网址
         */
        @ApiModelProperty(value = "官方网址")
        @Length(max = 255, message = "官方网址长度不能超过255", groups = { AddGroup.class, EditGroup.class })
        private String officialWebsite;
        /**
         * 融资历史
         */
        @ApiModelProperty(value = "融资历史")
        private List<History> historyList;
        /**
         * 核心成员
         */
        @ApiModelProperty(value = "核心成员")
        private List<Member> memberList;

        @Data
        @EqualsAndHashCode
        @ApiModel("融资历史")
        public static class History {
            /**
             * 融资轮次
             */
            @ApiModelProperty(value = "融资轮次")
            private String financingRounds;
            /**
             * 融资时间
             */
            @ApiModelProperty(value = "融资时间")
            private String financingTime;
            /**
             * 融资金额(万元)
             */
            @ApiModelProperty(value = "融资金额(万元)")
            private BigDecimal amount;
            /**
             * 对外公布融资金额(0否1是)
             */
            @ApiModelProperty(value = "对外公布融资金额(0否1是)")
            private Long publicityAmount;
            /**
             * 投资方
             */
            @ApiModelProperty(value = "投资方")
            @Length(max = 255, message = "投资方长度不能超过255", groups = { AddGroup.class, EditGroup.class })
            private String investor;
            /**
             * 对外公布投资方(0否1是)
             */
            @ApiModelProperty(value = "对外公布投资方(0否1是)")
            private Long publicityInvestor;
            /**
             * 排序
             */
            @ApiModelProperty(value = "排序")
            private Long sortNum;
        }

        @Data
        @EqualsAndHashCode
        @ApiModel("核心成员")
        public static class Member {
            /**
             * 姓名
             */
            @ApiModelProperty(value = "姓名")
            @Length(max = 255, message = "姓名长度不能超过255", groups = { AddGroup.class, EditGroup.class })
            private String name;
            /**
             * 职务
             */
            @ApiModelProperty(value = "职务")
            @Length(max = 255, message = "职务长度不能超过255", groups = { AddGroup.class, EditGroup.class })
            private String position;
            /**
             * 个人经历
             */
            @ApiModelProperty(value = "个人经历")
            @Length(max = 1024, message = "个人经历长度不能超过1024", groups = { AddGroup.class, EditGroup.class })
            private String personalExperience;
            /**
             * 头像
             */
            @ApiModelProperty(value = "头像")
            private Long headSculpture;
            /**
             * 排序
             */
            @ApiModelProperty(value = "排序")
            private Long sortNum;
        }
    }

}
