package com.crunii.micro.service.match.api.bo;

import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import javax.validation.constraints.*;

import java.util.Date;

/**
 * 赛事-场次配置;业务对象 mth_round
 *
 * @author mth
 * @date 2023-09-19
 */

@Data
@EqualsAndHashCode
@ApiModel("赛事场次配置业务对象")
public class MthRoundBo {

    @ApiModelProperty("id")
    private Long id;

    /**
     * 比赛id
     */
    @ApiModelProperty(value = "比赛id", required = true)
    @NotBlank(message = "比赛id不能为空", groups = { AddGroup.class, EditGroup.class })
    private String matchId;

    /**
     * 场次名称
     */
    @ApiModelProperty(value = "场次名称", required = true)
    @NotBlank(message = "场次名称不能为空", groups = { AddGroup.class, EditGroup.class })
    private String roundName;

    /**
     * 比赛方式（线上 线下）
     */
    @ApiModelProperty(value = "比赛方式（线上 线下）", required = true)
    @NotBlank(message = "比赛方式（线上 线下）不能为空", groups = { AddGroup.class, EditGroup.class })
    private String playMode;

    /**
     * 评审方式(线上 线下)
     */
    @ApiModelProperty(value = "评审方式(线上 线下)", required = true)
    @NotBlank(message = "评审方式(线上 线下)不能为空", groups = { AddGroup.class, EditGroup.class })
    private String auditMode;

    /**
     * 评分方式(10分制、百分制、ABC、通过不通过))
     */
    @ApiModelProperty(value = "评分方式(10分制、百分制、ABC、通过不通过))", required = true)
    @NotBlank(message = "评分方式(10分制、百分制、ABC、通过不通过))不能为空", groups = { AddGroup.class, EditGroup.class })
    private String socreMode;

    /**
     * 统计分方式(平均分、去掉最高最低))
     */
    @ApiModelProperty(value = "统计分方式(平均分、去掉最高最低))", required = true)
    @NotBlank(message = "统计分方式(平均分、去掉最高最低))不能为空", groups = { AddGroup.class, EditGroup.class })
    private String scoreStatistics;

    /**
     * 晋级方式(人数、百分比)
     */
    @ApiModelProperty(value = "晋级方式(人数、百分比)", required = true)
    @NotBlank(message = "晋级方式(人数、百分比)不能为空", groups = { AddGroup.class, EditGroup.class })
    private String promotionMode;

    /**
     * 晋级数量
     */
    @ApiModelProperty(value = "晋级数量", required = true)
    @NotNull(message = "晋级数量不能为空", groups = { AddGroup.class, EditGroup.class })
    private Integer promptionNum;

    /**
     * 报名开始时间
     */
    @ApiModelProperty(value = "报名开始时间", required = true)
    @NotNull(message = "报名开始时间不能为空", groups = { AddGroup.class, EditGroup.class })
    private Date registerationStart;

    /**
     * 报名结束时间
     */
    @ApiModelProperty(value = "报名结束时间", required = true)
    @NotNull(message = "报名结束时间不能为空", groups = { AddGroup.class, EditGroup.class })
    private Date registerationEnd;

    /**
     * 作品提交方式（单独上传、不需上传）
     */
    @ApiModelProperty(value = "作品提交方式（单独上传、不需上传）", required = true)
    @NotBlank(message = "作品提交方式（单独上传、不需上传）不能为空", groups = { AddGroup.class, EditGroup.class })
    private String entrySubmitMode;

    /**
     * 作品截止时间
     */
    @ApiModelProperty(value = "作品截止时间", required = true)
    @NotNull(message = "作品截止时间不能为空", groups = { AddGroup.class, EditGroup.class })
    private Date entryDeadline;

    /**
     * 场次开始时间
     */
    @ApiModelProperty(value = "场次开始时间", required = true)
    @NotBlank(message = "场次开始时间不能为空", groups = { AddGroup.class, EditGroup.class })
    private String roundStart;

    /**
     * 场次结束时间
     */
    @ApiModelProperty(value = "场次结束时间", required = true)
    @NotBlank(message = "场次结束时间不能为空", groups = { AddGroup.class, EditGroup.class })
    private String roundEnd;

    /**
     * 场次序号
     */
    @ApiModelProperty(value = "场次序号", required = true)
    @NotBlank(message = "场次序号不能为空", groups = { AddGroup.class, EditGroup.class })
    private String roundSort;

    /**
     * 比赛等级（市、片区、区县、校区）
     */
    @ApiModelProperty(value = "比赛等级（市、片区、区县、校区）", required = true)
    @NotBlank(message = "比赛等级（市、片区、区县、校区）不能为空", groups = { AddGroup.class, EditGroup.class })
    private String matchLevel;

    /**
     * 场次等级初赛复赛(1234数字递增)
     */
    @ApiModelProperty(value = "场次等级初赛复赛(1234数字递增)", required = true)
    @NotNull(message = "场次等级初赛复赛(1234数字递增)不能为空", groups = { AddGroup.class, EditGroup.class })
    private Integer roundLevel;



}
