package com.crunii.micro.service.match.api.vo;

import com.alibaba.excel.annotation.ExcelIgnoreUnannotated;
import com.crunii.micro.common.serializer.DateTimeTwoSerializer;
import com.crunii.micro.service.match.api.bo.ObsFileInfoBo;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * APP赛事报名配置查询-返回参数
 *
 * @author mth
 * @date 2023-09-19
 */
@Data
@ApiModel("APP查询报名表配置-返回参数")
@ExcelIgnoreUnannotated
public class MthMyMatchListDetailAppVo implements Serializable {
    private static final long serialVersionUID = 1L;

    @ApiModelProperty("当前场次报名信息")
    private Register register;

    @ApiModelProperty("作品表")
    private List<Work> workList;

    @ApiModelProperty("场次奖项")
    private List<RoundPrize> roundPrizeList;

    @ApiModelProperty("页面展示按钮")
    private ShowButton showButton;

    @ApiModelProperty("签退/签到 信息")
    private SignInformation signInformation;

    @ApiModelProperty("签到记录")
    private List<Map<String,Object>> signRecord;

    @ApiModelProperty("评委点评信息")
    private List<Reviewer> reviewer;

    @Data
    @ApiModel("页面展示按钮")
    public static class ShowButton {
        /**
         * 是否展示撤销报名按钮
         */
        @ApiModelProperty("是否展示撤销报名按钮")
        private Boolean showCancelBtn;
        /**
         * 是否展示信息更新按钮
         */
        @ApiModelProperty("是否展示信息更新按钮")
        private Boolean showUpdateBtn;
        /**
         * 是否展示上传作品按钮
         */
        @ApiModelProperty("是否展示上传作品按钮")
        private Boolean showUploadWorkBtn;

        /**
         * 是否收藏
         */
        @ApiModelProperty("是否收藏")
        private Boolean whetherCollect;

        @ApiModelProperty(value = "是否需要签到")
        private Boolean whetherSignIn=false;

        @ApiModelProperty(value = "是否需要签退")
        private Boolean whetherSignOut=false;
    }

    @Data
    @ApiModel("当前场次报名信息")
    public static class Register {
        /**
         * id
         */
        @ApiModelProperty("id")
        private Long id;
        /**
         * 赛事id
         */
        @ApiModelProperty("赛事id")
        private Long matchId;

        @ApiModelProperty("赛事名称")
        private String matchName;
        /**
         * bannerId
         */
        @ApiModelProperty(value = "bannerId")
        private Long bannerId;
        /**
         * banner
         */
        @ApiModelProperty(value = "banner")
        private ObsFileInfoBo banner;

        @ApiModelProperty("赛事场次ID")
        private Long roundInstanceId;

        /**
         * 类别（个人信息、作品）
         */
        @ApiModelProperty("类别（个人信息、作品）")
        private String registerType;
        /**
         * 名称
         */
        @ApiModelProperty("名称")
        private String name;
        /**
         * 描述
         */
        @ApiModelProperty("描述")
        private String configDesc;
        /**
         * 图片
         */
        @ApiModelProperty("图片")
        private Long picId;
        /**
         * pic图片、file附件回显值
         */
        @ApiModelProperty("pic图片、file附件回显值")
        private ObsFileInfoBo picFile;
        /**
         * 赛项id
         */
        @ApiModelProperty("赛项id")
        private Long itemId;
        /**
         * 赛项名称
         */
        @ApiModelProperty("赛项名称")
        private String itemName;
        /**
         * 组别id
         */
        @ApiModelProperty("组别id")
        private Long groupId;
        /**
         * 组别名称
         */
        @ApiModelProperty("组别名称")
        private String groupName;
        /**
         * 配置详情
         */
        @ApiModelProperty("配置详情")
        private Map<String, List<Detail>> detailList;
        /**
         * 赛项选项列表
         */
        @ApiModelProperty("赛项选项列表")
        List<MatchBaseItemVo> itemList;
        /**
         * 报名时间
         */
        @ApiModelProperty(value = "报名时间")
        private Date createTime;
        /**
         * 报名开始时间
         */
        @ApiModelProperty(value = "报名开始时间")
        @JsonSerialize(using = DateTimeTwoSerializer.class)
        private Date registerationStart;
        /**
         * 报名结束时间
         */
        @ApiModelProperty(value = "报名结束时间")
        @JsonSerialize(using = DateTimeTwoSerializer.class)
        private Date registerationEnd;
        /**
         * 作品提交方式
         */
        @ApiModelProperty(value = "作品提交方式")
        private String entrySubmitMode;
        /**
         * 作品截止时间
         */
        @ApiModelProperty(value = "作品截止时间")
        private Date entryDeadline;
        /**
         * 审核结果
         */
        @ApiModelProperty(value = "审核结果")
        private String reviewResult;
        /**
         * 审核失败原因
         */
        @ApiModelProperty(value = "审核失败原因")
        private String reviewReason;
        /**
         * 作品id, 用于判断当前场次是否已上传作品, 已上传展示作品更新按钮, 未上传展示作品上传按钮
         */
        @ApiModelProperty("作品id")
        private Long workId;
        /**
         * 比赛场次扭转状态
         */
        @ApiModelProperty(value = "比赛场次扭转状态")
        private Integer matchState;

        /**
         * 证书领取方式
         */
        @ApiModelProperty(value = "证书领取方式（SELF 自取,DELIVERY 邮寄）")
        private String receiveType;


    }

    @Data
    @ApiModel("作品表")
    public static class Work {
        /**
         * id
         */
        @ApiModelProperty("id")
        private Long id;
        /**
         * 赛事id
         */
        @ApiModelProperty("赛事id")
        private Long matchId;
        /**
         * 类别（个人信息、作品）
         */
        @ApiModelProperty("类别（个人信息、作品）")
        private String registerType;
        /**
         * 名称
         */
        @ApiModelProperty("名称")
        private String name;
        /**
         * 描述
         */
        @ApiModelProperty("描述")
        private String configDesc;
        /**
         * 图片
         */
        @ApiModelProperty("图片")
        private Long picId;
        /**
         * pic图片、file附件回显值
         */
        @ApiModelProperty("pic图片、file附件回显值")
        private ObsFileInfoBo picFile;
        /**
         * 作品编码
         */
        @ApiModelProperty("作品编码")
        private String workCode;
        /**
         * 作品名称
         */
        @ApiModelProperty("作品名称")
        private String workName;
        /**
         * 比赛范围+场次名称
         */
        @ApiModelProperty("比赛范围+场次名称")
        private String roundRangeName;
        /**
         * 配置详情
         */
        @ApiModelProperty("配置详情")
        private Map<String, List<Detail>> detailList;
        /**
         * 作品上传时间
         */
        @ApiModelProperty(value = "作品上传时间")
        private Date createTime;
        /**
         * 作品更新时间
         */
        @ApiModelProperty(value = "作品更新时间")
        private Date updateTime;
    }

    @Data
    @ApiModel("配置详情")
    public static class Detail {
        /**
         * id
         */
        @ApiModelProperty("id")
        private Long id;
        /**
         * 报名id或作品id
         */
        @ApiModelProperty("报名id或作品id")
        private Long bizId;
        /**
         * 报名配置详情id
         */
        @ApiModelProperty("报名配置详情id")
        private Long configDetailId;
        /**
         * 配置id
         */
        @ApiModelProperty("配置id")
        private Long configId;
        /**
         * 分组名称
         */
        @ApiModelProperty("分组名称")
        private String groupName;
        /**
         * 排序
         */
        @ApiModelProperty("排序")
        private String sort;
        /**
         * 显示名称
         */
        @ApiModelProperty("显示名称")
        private String showName;

        /**
         * 用户提交数据
         */
        @ApiModelProperty("用户提交数据")
        private String value;
        /**
         * pic图片、file附件回显值
         */
        @ApiModelProperty("pic图片、file附件回显值")
        private List<ObsFileInfoBo> fileValues;
        /**
         * 是否必填
         */
        @ApiModelProperty("是否必填")
        private String requiredFlag;
        /**
         * 类型(文本、单选、多选、附件)
         */
        @ApiModelProperty("类型(文本、单选、多选、附件)")
        private String type;
        /**
         * 关联id类型(数据字典、业务数据id)
         */
        @ApiModelProperty("refType(数据字典、业务数据id)")
        private String refType;
        /**
         * 关联id(数据字典、业务数据id)
         */
        @ApiModelProperty("关联id(数据字典、业务数据id)")
        private String refId;
        /**
         * key
         */
        @ApiModelProperty("key")
        private String key;
    }


    @Data
    @ApiModel("场次奖项")
    public static class RoundPrize {
        /**
         * 场次全名称
         */
        @ApiModelProperty(value = "场次全名称")
        private String roundFullName;
        /**
         * 结果（晋级淘汰）
         */
        @ApiModelProperty(value = "结果（晋级淘汰）")
        private String result;
        /**
         * 奖项
         */
        @ApiModelProperty(value = "奖项")
        private String prizeId;
        /**
         * 奖项
         */
        @ApiModelProperty(value = "奖项")
        private String prizeName;
        /**
         * 其他奖项
         */
        @ApiModelProperty(value = "其他奖项")
        private String otherPrizeId;
        /**
         * 其他奖项
         */
        @ApiModelProperty(value = "其他奖项")
        private String otherPrizeName;

        /**
         * 是否邮寄
         */
        @ApiModelProperty(value = "是否邮寄")
        private String deliveryFlagName;

        /**
         * 是否开启填写邮寄地址按钮
         */
        @ApiModelProperty(value = "是否开启填写邮寄地址按钮")
        private Boolean whetherDelivery;

    }
    @Data
    @ApiModel("签到信息")
    public static class SignInformation {

        @ApiModelProperty(value = "签到类型")
        private Integer signType;

        @ApiModelProperty(value = "经度")
        private String longitude;

        @ApiModelProperty(value = "纬度")
        private String latitude;

        @ApiModelProperty(value = "签到/签退 开始时间")
        @JsonSerialize(using = DateTimeTwoSerializer.class)
        private Date signStartTime;

        @ApiModelProperty(value = "签到/签退 结束时间")
        @JsonSerialize(using = DateTimeTwoSerializer.class)
        private Date signEndTime;

        @ApiModelProperty(value = "签到/签退 地点名称")
        private String signSiteName;

        @ApiModelProperty(value = "签到/签退 详细地址")
        private String signAddress;
    }

    @Data
    @ApiModel("评委点评信息")
    public static class Reviewer {

        @ApiModelProperty(value = "评语表ID")
        private Long id;

        /**
         * 评语
         */
        @ApiModelProperty(value = "评语")
        private String comments;
        /**
         * 评审员id
         */
        @ApiModelProperty(value = "评审员id")
        private Long reviewerId;
        /**
         * 评委
         */
        @ApiModelProperty(value = "评委")
        private String reviewerName;
        /**
         * 评委签名
         */
        @ApiModelProperty(value = "评委签名（fileId）")
        private String reviewerSign;

        /**
         * 评委签名url
         */
        @ApiModelProperty(value = "评委签名")
        private String reviewerSignUrl;
    }
}
