package com.crunii.micro.service.match.api.vo;

import com.crunii.micro.common.serializer.BigDecimalSerializer;
import com.crunii.micro.service.match.api.bo.ObsFileInfoBo;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.persistence.Column;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


/**
 * app-我的项目详情
 *
 * @author ruoyi
 * @date 2023-12-18
 */
@Data
@ApiModel("app-我的项目详情-返回参数")
public class MthMyProjectApplyDetailsVo implements Serializable {
    private static final long serialVersionUID = 1L;

    private Long id;

    @ApiModelProperty(value = "审核信息")
    private MthMyProjectApplyAppVo reviewInfo;

    @ApiModelProperty(value = "报名表信息")
    private RegisterInfo registerInfo;

    @ApiModelProperty(value = "融资信息")
    private FinancingInfo financingInfo;

    /**
     * 报名表信息
     */
    @Data
    @ApiModel("报名表信息")
    public static class RegisterInfo {

        @ApiModelProperty(value = "项目赛项")
        private String itemName;

        @ApiModelProperty(value = "项目组别")
        private String groupName;

        @ApiModelProperty(value = "项目名称")
        private String name;

        @ApiModelProperty(value = "项目logo")
        private Long logo;
        @ApiModelProperty(value = "项目logoFiles")
        private ObsFileInfoBo logoFile;

        @ApiModelProperty(value = "项目启动时间")
        private Date startTime;

        @ApiModelProperty(value = "项目所在地")
        private String address;

        @ApiModelProperty(value = "项目简介")
        private String itemDesc;

        @ApiModelProperty(value = "项目展示")
        private String showImages;

        @ApiModelProperty(value = "项目展示FileList")
        private List<ObsFileInfoBo> showImagesFileList;

        @ApiModelProperty(value = "单位名称")
        private String companyName;

        @ApiModelProperty(value = "单位职务")
        private String position;

        @ApiModelProperty(value = "注册地址")
        private String registerAddress;

        @ApiModelProperty(value = "注册时间")
        private Date regiserTime;

        @ApiModelProperty(value = "注册资本")
        @JsonSerialize(using = BigDecimalSerializer.class)
        private BigDecimal regiserCapital;

        @ApiModelProperty(value = "法人")
        private String legalRepresentative;

        @ApiModelProperty(value = "所属行业")
        private String industry;

        @ApiModelProperty(value = "企业负债金额")
        @JsonSerialize(using = BigDecimalSerializer.class)
        private BigDecimal enterpriseDebtAmount;

        @ApiModelProperty(value = "上一年营收")
        @JsonSerialize(using = BigDecimalSerializer.class)
        private BigDecimal previousYearRevenue;

        @ApiModelProperty(value = "上一年税收")
        @JsonSerialize(using = BigDecimalSerializer.class)
        private BigDecimal previousYearTaxRevenue;

        @ApiModelProperty(value = "员工数")
        private Integer employeeCount;

        @ApiModelProperty(value = "团队成员人数")
        private Integer teamMembers;
    }

    /**
     * 融资信息
     */
    @Data
    @ApiModel("融资信息")
    public static class FinancingInfo {

        @ApiModelProperty(value = "一句话介绍")
        private String introduce;

        @ApiModelProperty(value = "期望融资")
        @JsonSerialize(using = BigDecimalSerializer.class)
        private BigDecimal expectedAmount;

        @ApiModelProperty(value = "官方网址")
        private String officialWebsite;

        @ApiModelProperty(value = "出让股权(%)")
        private BigDecimal transferEquity;

        @ApiModelProperty(value = "融资历史")
        private List<FinancingHistory> financingHistory;

        @ApiModelProperty(value = "团队核心成员")
        private List<TeamMembers> allMembers;

        @Data
        @ApiModel("融资历史")
        public static class FinancingHistory {

            private Long id;

            @ApiModelProperty(value = "融资轮次")
            private String financingRounds;

            @ApiModelProperty(value = "融资时间")
            private String financingTime;

            @ApiModelProperty(value = "融资金额")
            @JsonSerialize(using = BigDecimalSerializer.class)
            private BigDecimal amount;

            @ApiModelProperty(value = "投资方")
            private String investor;
        }

        @Data
        @ApiModel("团队核心成员")
        public static class TeamMembers {

            private Long id;

            @ApiModelProperty(value = "头像")
            private Long headSculpture;

            @ApiModelProperty(value = "头像file对象")
            private ObsFileInfoBo headSculptureFlie;

            @ApiModelProperty(value = "姓名")
            private String memberName;

            @ApiModelProperty(value = "职务")
            private String position;

            @ApiModelProperty(value = "个人经历")
            private String personalExperience;
        }
    }

}
