package com.crunii.micro.service.match.api.vo;

import com.crunii.micro.common.serializer.*;
import com.crunii.micro.service.match.api.bo.ObsFileInfoBo;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


/**
 * 青创赛事项目融资申请-详情返回参数
 *
 * @author ruoyi
 * @date 2023-12-18
 */
@Data
@ApiModel("青创赛事项目融资申请视图对象")
public class MthProjectFinancingDetailsVo extends MthProjectFinancingBaselVo{
    /*----------------------------------------报名信息(青创赛事报名信息)----------------------------------------*/

    @ApiModelProperty(value = "项目展示")
    private List<ObsFileInfoBo> showImageList;

    /*----------------------------------------融资信息----------------------------------------*/
    @ApiModelProperty(value = "融资历史")
    private List<FinancingHistory> financingHistory;

    @ApiModelProperty(value = "团队核心成员")
    private List<TeamMembers> teamMembers;

    @Data
    @ApiModel("评审信息")
    public static class ReviewList {

        private Long id;

        @ApiModelProperty(value = "分数")
        private String score;

        @ApiModelProperty(value = "评委名称")
        private String reviewerName;

        @ApiModelProperty(value = "评审时间")
        @JsonSerialize(using = DateTimeSerializer.class)
        private Date reviewDate;
    }

    @Data
    @ApiModel("融资历史")
    public static class FinancingHistory {

        private Long id;

        @ApiModelProperty(value = "融资轮次")
        private String financingRounds;

        @ApiModelProperty(value = "融资时间")
        private String financingTime;

        @ApiModelProperty(value = "融资金额")
        @JsonSerialize(using = BigDecimalSerializer.class)
        private BigDecimal amount;

        @ApiModelProperty(value = "投资方")
        private String investor;
    }

    @Data
    @ApiModel("团队核心成员")
    public static class TeamMembers {

        private Long id;

        @ApiModelProperty(value = "头像")
        private Long headSculpture;

        @ApiModelProperty(value = "头像file对象")
        private ObsFileInfoBo headSculptureFlie;

        @ApiModelProperty(value = "姓名")
        private String name;

        @ApiModelProperty(value = "职务")
        private String position;

        @ApiModelProperty(value = "个人经历")
        private String personalExperience;
    }

    @Data
    @ApiModel("参加过的场次信息")
    public static class RoundList {

        @ApiModelProperty(value = "场次id")
        private Long instanceId;

        @ApiModelProperty(value = "场次名称")
        private String roundName;

        @ApiModelProperty(value = "分数")
        private Integer score;

        @ApiModelProperty(value = "评审结果")
        private String endReviewResult;

        @ApiModelProperty(value = "是否晋级")
        private String whetherPromotion;

        @ApiModelProperty(value = "晋级时间")
        private Date promotionDate;

        @ApiModelProperty(value = "获奖名次")
        private String prize;
    }
}
