package com.crunii.micro.service.match.api.vo;

import com.alibaba.excel.annotation.ExcelIgnoreUnannotated;
import com.crunii.micro.service.match.api.bo.ObsFileInfoBo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;


/**
 * APP赛事报名配置查询-返回参数
 *
 * @author mth
 * @date 2023-09-19
 */
@Data
@ApiModel("APP查询报名表配置-返回参数")
@ExcelIgnoreUnannotated
public class MthRegisterConfigAppVo implements Serializable {
    private static final long serialVersionUID = 1L;

    @ApiModelProperty("报名表")
    private Register register;

    @ApiModelProperty("作品表")
    private Work work;

    @ApiModelProperty(value = "融资需求")
    private Financing financing;

    @Data
    @ApiModel("报名表")
    public static class Register {
        /**
         * id
         */
        @ApiModelProperty("id")
        private Long id;
        /**
         * registerId
         */
        @ApiModelProperty("registerId")
        private Long registerId;
        /**
         * 赛事id
         */
        @ApiModelProperty("赛事id")
        private Long matchId;
        /**
         * 类别（个人信息、作品）
         */
        @ApiModelProperty("类别（个人信息、作品）")
        private String registerType;
        /**
         * 名称
         */
        @ApiModelProperty("名称")
        private String name;
        /**
         * 赛事名称
         */
        @ApiModelProperty("赛事名称")
        private String matchName;
        /**
         * 描述
         */
        @ApiModelProperty("描述")
        private String configDesc;
        /**
         * 图片
         */
        @ApiModelProperty("图片")
        private Long picId;
        /**
         * pic图片、file附件回显值
         */
        @ApiModelProperty("pic图片、file附件回显值")
        private ObsFileInfoBo picFile;
        /**
         * 赛项id
         */
        @ApiModelProperty("赛项id")
        private Long itemId;
        /**
         * 赛项名称
         */
        @ApiModelProperty("赛项名称")
        private String itemName;
        /**
         * 组别id
         */
        @ApiModelProperty("组别id")
        private Long groupId;
        /**
         * 组别名称
         */
        @ApiModelProperty("组别名称")
        private String groupName;
        /**
         * 配置详情
         */
        @ApiModelProperty("配置详情")
        private Map<String, List<Detail>> detailList;
        /**
         * 赛项选项列表
         */
        @ApiModelProperty("赛项选项列表")
        List<MatchBaseItemVo> itemList;

        /**
         * 限制类型
         */
        @ApiModelProperty("年龄限制方式（group:按组别,unify_age:t统一）")
        private String ageLimit;

        /**
         * 结束年龄
         */
        @ApiModelProperty("结束年龄")
        private String ageEnd;

        /**
         * 开始年龄
         */
        @ApiModelProperty("开始年龄")
        private String ageBegin;
    }

    @Data
    @ApiModel("作品表")
    public static class Work {
        /**
         * id
         */
        @ApiModelProperty("id")
        private Long id;
        /**
         * workId
         */
        @ApiModelProperty("workId")
        private Long workId;
        /**
         * 赛事id
         */
        @ApiModelProperty("赛事id")
        private Long matchId;
        /**
         * 类别（个人信息、作品）
         */
        @ApiModelProperty("类别（个人信息、作品）")
        private String registerType;
        /**
         * 名称
         */
        @ApiModelProperty("名称")
        private String name;
        /**
         * 描述
         */
        @ApiModelProperty("描述")
        private String configDesc;
        /**
         * 图片
         */
        @ApiModelProperty("图片")
        private Long picId;
        /**
         * pic图片、file附件回显值
         */
        @ApiModelProperty("pic图片、file附件回显值")
        private ObsFileInfoBo picFile;
        /**
         * 作品编码
         */
        @ApiModelProperty("作品编码")
        private String workCode;
        /**
         * 作品名称
         */
        @ApiModelProperty("作品名称")
        private String workName;
        /**
         * 配置详情
         */
        @ApiModelProperty("配置详情")
        private Map<String, List<Detail>> detailList;
    }

    @Data
    @ApiModel("配置详情")
    public static class Detail {
        /**
         * id
         */
        @ApiModelProperty("id")
        private Long id;
        /**
         * detailId
         */
        @ApiModelProperty("detailId")
        private Long detailId;
        /**
         * 报名信息id
         */
        @ApiModelProperty("报名信息id")
        private Long registerInfoId;
        /**
         * 配置id
         */
        @ApiModelProperty("配置id")
        private Long registerConfigId;
        /**
         * 分组名称
         */
        @ApiModelProperty("分组名称")
        private String groupName;
        /**
         * 排序
         */
        @ApiModelProperty("排序")
        private String sort;
        /**
         * 显示名称
         */
        @ApiModelProperty("显示名称")
        private String showName;

        /**
         * 用户提交数据
         */
        @ApiModelProperty("用户提交数据")
        private String value;
        /**
         * pic图片、file附件回显值
         */
        @ApiModelProperty("pic图片、file附件回显值")
        private List<ObsFileInfoBo> fileValues;
        /**
         * 是否必填
         */
        @ApiModelProperty("是否必填")
        private String requiredFlag;
        /**
         * 类型(文本、单选、多选、附件)
         */
        @ApiModelProperty("类型(text文本、single单选、checkbox多选、pic图片、file附件)")
        private String configType;
        /**
         * 关联id类型(数据字典、业务数据id)
         */
        @ApiModelProperty("refType(数据字典、业务数据id)")
        private String refType;
        /**
         * 关联id(数据字典、业务数据id)
         */
        @ApiModelProperty("关联id(数据字典、业务数据id)")
        private String refId;
        /**
         * key
         */
        @ApiModelProperty("key")
        private String key;
        /**
         * 提示
         */
        @ApiModelProperty("提示")
        private String hint;
        /**
         * 最小文件大小
         */
        @ApiModelProperty("最小文件大小")
        private String minFileSize;
        /**
         * 最大文件大小
         */
        @ApiModelProperty("最大文件大小")
        private String maxFileSize;
        /**
         * 数量限制
         */
        @ApiModelProperty("数量限制")
        private String limitNum;


        @ApiModelProperty("单选, 多选项值列表")
        List<MthFormConfigDataVo> configDataList;
    }



    @Data
    @EqualsAndHashCode
    @ApiModel("青创赛事融资需求")
    public static class Financing {
        /**
         * 一句话介绍
         */
        @ApiModelProperty(value = "一句话介绍")
        private String introduce;
        /**
         * 期望融资金额(万)
         */
        @ApiModelProperty(value = "期望融资金额(万)")
        private BigDecimal expectedAmount;
        /**
         * 出让股权(%)
         */
        @ApiModelProperty(value = "出让股权(%)")
        private BigDecimal transferEquity;
        /**
         * 官方网址
         */
        @ApiModelProperty(value = "官方网址")
        private String officialWebsite;
        /**
         * 融资历史
         */
        @ApiModelProperty(value = "融资历史")
        private List<History> historyList;
        /**
         * 核心成员
         */
        @ApiModelProperty(value = "核心成员")
        private List<Member> memberList;

        @Data
        @EqualsAndHashCode
        @ApiModel("融资历史")
        public static class History {
            /**
             * 融资轮次
             */
            @ApiModelProperty(value = "融资轮次")
            private String financingRounds;
            /**
             * 融资时间
             */
            @ApiModelProperty(value = "融资时间")
            private String financingTime;
            /**
             * 融资金额(万元)
             */
            @ApiModelProperty(value = "融资金额(万元)")
            private BigDecimal amount;
            /**
             * 对外公布融资金额(0否1是)
             */
            @ApiModelProperty(value = "对外公布融资金额(0否1是)")
            private Long publicityAmount;
            /**
             * 投资方
             */
            @ApiModelProperty(value = "投资方")
            private String investor;
            /**
             * 对外公布投资方(0否1是)
             */
            @ApiModelProperty(value = "对外公布投资方(0否1是)")
            private Long publicityInvestor;
            /**
             * 排序
             */
            @ApiModelProperty(value = "排序")
            private Long sortNum;
        }

        @Data
        @EqualsAndHashCode
        @ApiModel("核心成员")
        public static class Member {
            /**
             * 姓名
             */
            @ApiModelProperty(value = "姓名")
            private String name;
            /**
             * 职务
             */
            @ApiModelProperty(value = "职务")
            private String position;
            /**
             * 个人经历
             */
            @ApiModelProperty(value = "个人经历")
            private String personalExperience;
            /**
             * 头像
             */
            @ApiModelProperty(value = "头像")
            private Long headSculpture;
            /**
             * 头像
             */
            @ApiModelProperty(value = "头像")
            private ObsFileInfoBo headSculptureFile;
            /**
             * 排序
             */
            @ApiModelProperty(value = "排序")
            private Long sortNum;
        }
    }
}
