package com.crunii.micro.domain.entity.main;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Where;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import java.util.Date;


@Entity
@Table(name = "mth_match_sign_information")
@NamedQuery(name = "MthMatchSignInformation.findAll", query = "SELECT t FROM MthMatchSignInformation t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = { "id" })
@GenericGenerator(name = "idGenerator", strategy = "com.crunii.micro.core.distributed.DistributedSnowFlakeId")
@ApiModel(description = "赛事签到表")
@Where(clause=" del_flag = 0 ")
public class MthMatchSignInformation extends BaseEntity{

    @Id
    @GeneratedValue(generator = "idGenerator")
    @Column(name = "id")
    @ApiModelProperty(value = "id")
    private Long id ;

    /**
     * 赛事场次ID
     */
    @ApiModelProperty(name = "赛事场次ID")
    @Column(name ="instance_id")
    private Long instanceId ;

    /**
     * 签到开始时间
     */
    @ApiModelProperty(value ="开始时间" )
    @Column(name = "sign_start_time")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date signStartTime;

    /**
     * 签到结束时间
     */
    @ApiModelProperty(value ="结束时间" )
    @Column(name = "sign_end_time")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date signEndTime;

    /**
     * 签到方式(1扫码,2定位,3扫码+定位)
     */
    @ApiModelProperty(value ="签到方式(1扫码,2定位,3扫码+定位)" )
    @Column(name = "sign_type")
    private Integer signType;

    /**
     * 经度
     */
    @ApiModelProperty(value = "经度")
    @Column(name = "sign_longitude")
    private String signLongitude;

    /**
     * 纬度
     */
    @ApiModelProperty(value = "纬度")
    @Column(name = "sign_latitude")
    private String signLatitude;

    /**
     * 二维码
     */
    @ApiModelProperty(value ="二维码" )
    @Column(name = "sign_qr_id")
    private String signQrId;

    /**
     * 地点名称
     */
    @ApiModelProperty(value ="地点名称" )
    @Column(name = "sign_site_name")
    private String signSiteName;

    /**
     * 详细地址
     */
    @ApiModelProperty(value ="详细地址" )
    @Column(name = "sign_address")
    private String signAddress;

    /**
     * (1签到，-1签退)
     */
    @ApiModelProperty(value ="(1签到,-1签退)" )
    @Column(name = "information_type")
    private Integer informationType;
}