package com.crunii.micro.domain.entity.main;


import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Where;
import org.hibernate.validator.constraints.Length;

import javax.persistence.*;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 青创赛事项目对象 mth_project
 *
 * @author ruoyi
 * @date 2023-12-18
 */
@Entity
@Table(name = "mth_project")
@NamedQuery(name = "MthProject.findAll", query = "SELECT t FROM MthProject t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = {"id"})
@GenericGenerator(name = "idGenerator", strategy = "com.crunii.micro.core.distributed.DistributedSnowFlakeId")
@ApiModel(description = "青创赛事项目")
@Where(clause = " del_flag = 0 ")
public class MthProject extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /**
     * ID
     */
    @Id
    @GeneratedValue(generator = "idGenerator")
    @Column(name = "id")
    @ApiModelProperty(value = "ID")
    private Long id;
    /**
     * 赛事报名id
     */
    @Column(name = "register_id")
    @ApiModelProperty(value = "赛事报名id")
    private Long registerId;
    /**
     * 项目名称
     */
    @Column(name = "name")
    @ApiModelProperty(value = "项目名称")
    @Length(max = 255, message = "项目名称长度不能超过255", groups = { AddGroup.class, EditGroup.class })
    private String name;
    /**
     * 项目logo
     */
    @Column(name = "logo")
    @ApiModelProperty(value = "项目logo")
    private Long logo;
    /**
     * 项目赛项
     */
    @Column(name = "item_name")
    @ApiModelProperty(value = "项目赛项")
    @Length(max = 255, message = "项目赛项长度不能超过255", groups = { AddGroup.class, EditGroup.class })
    private String itemName;
    /**
     * 项目组别
     */
    @Column(name = "group_name")
    @ApiModelProperty(value = "项目组别")
    @Length(max = 255, message = "项目组别长度不能超过255", groups = { AddGroup.class, EditGroup.class })
    private String groupName;
    /**
     * 项目启动时间
     */
    @Column(name = "start_time")
    @ApiModelProperty(value = "项目启动时间")
    private Date startTime;
    /**
     * 项目所在地
     */
    @Column(name = "address")
    @ApiModelProperty(value = "项目所在地")
    @Length(max = 255, message = "项目赛项长度不能超过255", groups = { AddGroup.class, EditGroup.class })
    private String address;
    /**
     * 项目简介
     */
    @Column(name = "item_desc")
    @ApiModelProperty(value = "项目简介")
    @Length(max = 1024, message = "项目简介长度不能超过1024", groups = { AddGroup.class, EditGroup.class })
    private String itemDesc;
    /**
     * 项目展示(逗号分隔)
     */
    @Column(name = "show_images")
    @ApiModelProperty(value = "项目展示(逗号分隔)")
    private String showImages;
    /**
     * 单位名称
     */
    @Column(name = "company_name")
    @ApiModelProperty(value = "单位名称")
    @Length(max = 255, message = "单位名称长度不能超过255", groups = { AddGroup.class, EditGroup.class })
    private String companyName;
    /**
     * 单位职务
     */
    @Column(name = "position")
    @ApiModelProperty(value = "单位职务")
    @Length(max = 255, message = "单位职务长度不能超过255", groups = { AddGroup.class, EditGroup.class })
    private String position;
    /**
     * 注册地址
     */
    @Column(name = "register_address")
    @ApiModelProperty(value = "注册地址")
    @Length(max = 255, message = "注册地址长度不能超过255", groups = { AddGroup.class, EditGroup.class })
    private String registerAddress;
    /**
     * 注册时间
     */
    @Column(name = "regiser_time")
    @ApiModelProperty(value = "注册时间")
    private Date regiserTime;
    /**
     * 注册资本(万)
     */
    @Column(name = "regiser_capital")
    @ApiModelProperty(value = "注册资本(万)")
    private BigDecimal regiserCapital;
    /**
     * 注册资本(货币)
     */
    @Column(name = "regiser_capital_currency")
    @ApiModelProperty(value = "注册资本(货币)")
    private String regiserCapitalCurrency;
    /**
     * 法定代表人
     */
    @Column(name = "legal_representative")
    @ApiModelProperty(value = "法定代表人")
    @Length(max = 255, message = "法定代表人长度不能超过255", groups = { AddGroup.class, EditGroup.class })
    private String legalRepresentative;
    /**
     * 所属行业(逗号分隔)
     */
    @Column(name = "industry")
    @ApiModelProperty(value = "所属行业(逗号分隔)")
    private String industry;
    /**
     * 企业负债金额
     */
    @Column(name = "enterprise_debt_amount")
    @ApiModelProperty(value = "企业负债金额")
    private BigDecimal enterpriseDebtAmount;
    /**
     * 上一年营收
     */
    @Column(name = "previous_year_revenue")
    @ApiModelProperty(value = "上一年营收")
    private BigDecimal previousYearRevenue;
    /**
     * 上一年税收
     */
    @Column(name = "previous_year_tax_revenue")
    @ApiModelProperty(value = "上一年税收")
    private BigDecimal previousYearTaxRevenue;
    /**
     * 员工数
     */
    @Column(name = "employee_count")
    @ApiModelProperty(value = "员工数")
    private Integer employeeCount;
    /**
     * 是否有融资需求
     */
    @Column(name = "financing_need")
    @ApiModelProperty(value = "是否有融资需求")
    private String financingNeed;

    /**
     * 团队成员人数
     */
    @Column(name = "team_members")
    @ApiModelProperty(value = "团队成员人数")
    private Integer teamMembers;

    /**
     * 赛事名称
     */
    @Column(name = "match_name")
    @ApiModelProperty(value = "赛事名称")
    private String matchName;

    /**
     * 赛事分类
     */
    @Column(name = "match_type")
    @ApiModelProperty(value = "赛事分类")
    private String matchType;

    /**
     * 赛项名称
     */
    @Column(name = "match_item")
    @ApiModelProperty(value = "赛项名称")
    private String matchItem;

    /**
     * 组别名称
     */
    @Column(name = "match_group")
    @ApiModelProperty(value = "组别名称")
    private String matchGroup;

    /**
     * 参赛者姓名
     */
    @Column(name = "participant_name")
    @ApiModelProperty(value = "参赛者姓名")
    private String participantName;

    /**
     * 参赛者手机号
     */
    @Column(name = "participant_mobile")
    @ApiModelProperty(value = "参赛者手机号")
    private String participantMobil;
}
