package com.crunii.micro.domain.entity.main;


import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Where;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import java.util.Date;

/**
 * 赛事-区域-场次实例（实例与配置的关系）;对象 mth_round_instance
 *
 * @author mth
 * @date 2023-09-19
 */
@Entity
@Table(name = "mth_round_instance")
@NamedQuery(name = "MthRoundInstance.findAll", query = "SELECT t FROM MthRoundInstance t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = { "id" })
@GenericGenerator(name = "idGenerator", strategy = "com.crunii.micro.core.distributed.DistributedSnowFlakeId")
@ApiModel(description = "赛事-区域-场次实例（实例与配置的关系）;")
@Where(clause=" del_flag = 0 ")
public class MthRoundInstance  extends BaseEntity{

private static final long serialVersionUID=1L;

    /**
     * id
     */
    @Id
    @GeneratedValue(generator = "idGenerator")
    @Column(name = "id")
    @ApiModelProperty(value = "id")
    private Long id;
    /**
     * 比赛id
     */
    @Column(name = "match_id")
    @ApiModelProperty(value = "比赛id")
    private Long matchId;

    /**
     * 场次名称
     */
    @Column(name = "round_name")
    @ApiModelProperty(value = "场次名称")
    private String roundName;
    /**
     * 比赛方式（线上 线下）
     */
    @Column(name = "play_mode")
    @ApiModelProperty(value = "比赛方式（线上 线下）")
    private String playMode;
    /**
     * 评审方式(线上 线下)
     */
    @Column(name = "audit_mode")
    @ApiModelProperty(value = "评审方式(线上 线下)")
    private String auditMode;
    /**
     * 评分方式(10分制、百分制、ABC、通过不通过))
     */
    @Column(name = "socre_mode")
    @ApiModelProperty(value = "评分方式(10分制、百分制、ABC、通过不通过))")
    private String socreMode;
    /**
     * 统计分方式(平均分、去掉最高最低))
     */
    @Column(name = "score_statistics")
    @ApiModelProperty(value = "统计分方式(平均分、去掉最高最低))")
    private String scoreStatistics;
    /**
     * 晋级方式(人数、百分比)
     */
    @Column(name = "promotion_mode")
    @ApiModelProperty(value = "晋级方式(人数、百分比)")
    private String promotionMode;
    /**
     * 晋级数量
     */
    @Column(name = "promption_num")
    @ApiModelProperty(value = "晋级数量")
    private Integer promptionNum;
    /**
     * 报名开始时间
     */
    @Column(name = "registeration_start")
    @ApiModelProperty(value = "报名开始时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date registerationStart;
    /**
     * 报名结束时间
     */
    @Column(name = "registeration_end")
    @ApiModelProperty(value = "报名结束时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date registerationEnd;
    /**
     * 作品提交方式（单独上传、不需上传）
     */
    @Column(name = "entry_submit_mode")
    @ApiModelProperty(value = "作品提交方式（单独上传、不需上传）")
    private String entrySubmitMode;
    /**
     * 作品截止时间
     */
    @Column(name = "entry_deadline")
    @ApiModelProperty(value = "作品截止时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date entryDeadline;
    /**
     * 场次开始时间
     */
    @Column(name = "round_start")
    @ApiModelProperty(value = "场次开始时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date roundStart;
    /**
     * 场次结束时间
     */
    @Column(name = "round_end")
    @ApiModelProperty(value = "场次结束时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date roundEnd;
    /**
     * 比赛等级（市级、片区级、区县/校赛级）
     */
    @Column(name = "match_level")
    @ApiModelProperty(value = "比赛等级（市级、片区级、区县/校赛级）")
    private String matchLevel;
    /**
     * 场次等级初赛复赛(1234数字递增)
     */
    @Column(name = "round_level")
    @ApiModelProperty(value = "场次等级初赛复赛(1234数字递增)")
    private Integer roundLevel;
    /**
     * 场次序号
     */
    @Column(name = "round_sort")
    @ApiModelProperty(value = "场次序号")
    private Integer roundSort;
    /**
     * 区域id
     */
    @Column(name = "area_id")
    @ApiModelProperty(value = "区域id")
    private Long areaId;
    /**
     * 区域名称
     */
    @Column(name = "area_name")
    @ApiModelProperty(value = "区域名称")
    private String areaName;

    /**
     * 场次状态
     */
    @Column(name = "round_state")
    @ApiModelProperty(value = "场次状态")
    private Integer roundState;

    /**
     * 比赛场次扭转状态
     */
    @Column(name = "match_state")
    @ApiModelProperty(value = "比赛场次扭转状态")
    private Integer matchState;


    /**
     * 场次类型
     */
    @Column(name = "round_type")
    private String roundType;

    /**
     * 是否公示
     */
    @Column(name = "publicity_flag")
    private Integer publicityFlag;

    /**
     * 公示天数
     */
    @Column(name = "publicity_days")
    private Integer publicityDays;

    /**
     * 名次设置类型(组别、赛项、 比赛)
     */
    @Column(name = "prize_type")
    private String prizeType;

    /**
     * 评审标准（按赛项、按比赛）
     */
    @Column(name = "reviewer_standar")
    private String reviewerStandar;

    /**
     * 场次结束时间
     */
    @Column(name = "publicity_deadline")
    @ApiModelProperty(value = "公示截止时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date publicityDeadline;


    /**
     * 领奖截止时间
     */
    @ApiModelProperty(value ="领奖截止时间" )
    @Column(name = "receive_deadline")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date receiveDeadline;

    /**
     * 领奖地点
     */
    @ApiModelProperty(value ="领奖地点" )
    @Column(name = "receive_address")
    private String receiveAddress;

    /**
     * 领奖方式
     */
    @ApiModelProperty(value ="领奖方式  SELF 自取 DELIVERY 邮寄" )
    @Column(name = "receive_type")
    private String receiveType;

}
