package com.crunii.micro.domain.repo.main;

import com.crunii.micro.domain.common.BaseRepo;
import com.crunii.micro.domain.entity.main.MthRoundInstance;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;
import org.springframework.validation.annotation.ValidationAnnotationUtils;

import java.util.Date;
import java.util.List;

/**
 * 赛事-区域-场次实例（实例与配置的关系）;Mapper接口
 *
 * @author mth
 * @date 2023-09-19
 */
@Repository
public interface MthRoundInstanceRepo extends BaseRepo<MthRoundInstance, Long> {
    List<MthRoundInstance> findByMatchIdAndRoundType(Long matchId,String roundType);
    List<MthRoundInstance> findByMatchIdAndRoundTypeAndAreaId(Long matchId, String roundType, Long areaId);
    List<MthRoundInstance> findByMatchIdAndRoundTypeAndAreaIdAndRoundNameLikeAndIdIsNot(Long matchId, String roundType, Long areaId,String roundName,Long id);

    List<MthRoundInstance> findByMatchIdAndRoundTypeAndMatchLevel(Long matchId, String roundType, String matchLevel);
    List<MthRoundInstance> findByMatchIdAndRoundTypeAndMatchLevelAndRoundSort(Long matchId, String roundType, String matchLevel, Integer roundSort);
    MthRoundInstance findByIdAndRoundType(Long id, String roundType);

    List<MthRoundInstance> findByMatchIdAndMatchLevelAndRoundNameLikeAndIdIsNot(Long matchId, String matchLevel, String roundName,Long id);

    List<MthRoundInstance> findByMatchIdAndMatchLevelAndRoundSortAndIdIsNot(Long matchId, String matchLevel, Integer roundSort,Long id);
    List<MthRoundInstance> findByMatchIdAndAreaIdAndRoundSortAndIdIsNot(Long matchId, Long areaId, Integer roundSort,Long id);
    List<MthRoundInstance>  findByMatchStateAndRoundTypeAndRegisterationEndBefore(Integer matchState,String roundType,Date date);

    @Query(value = "select * from mth_round_instance where del_flag=0 and round_type='instance' and match_state =0 and " +
            "DATE_FORMAT(registeration_end,'yyyy-MM-dd HH:mm:ss')<CURRENT_TIMESTAMP() ",nativeQuery = true)
    List<MthRoundInstance> queryByDate(String date);

    @Query(value = "update mth_round_instance set match_state =?1,update_time=?2 where id =?3",nativeQuery = true)
    void upadteMatchStateById(Integer matchState, Date updateTime,Long roundId);


    @Query(value = "update mth_round_instance set match_state =?1,update_time=?2 where id  in( ?3 ) ",nativeQuery = true)
    void upadteMatchStateByIdList(Integer matchState, Date updateTime,List<Long> idList);


    @Query(value = "update mth_round_instance set registeration_start=?2,registeration_end=?3 where match_id=?1",nativeQuery = true)
    void updateRegisterTimeByMatchId(Long matchId,Date startDate,Date endDate);

    @Query(value = "select * from mth_round_instance where match_id = ?1 and  round_sort = ?2 and round_type= ?3 and  del_flag= ?4 " ,nativeQuery = true)
    List<MthRoundInstance> findByMatchIdAndRoundSortAndRoundTypeAndDelFlag(Long matchId, Integer roundSort,String roundType,Integer delFlag);

    @Query(value = "select * from mth_round_instance where match_id = ?1 and  del_flag= 0 order by  create_time desc limit 1 " ,nativeQuery = true)
    MthRoundInstance findNewestInstance(Long matchId);

    /**
     * @description: 查询片区级场次总数量
     * @param
     * @return:
     * @author liuchuan
     * @date: 2023/12/7 17:53
    */
    @Query(value = "select count(id) from mth_round_instance where match_id= ?1 " +
            "and round_type='instance' and round_level=2 ",nativeQuery = true)
    Integer queryDistrictNum(Long matchId);

    /**
     * @description: 查询片区级已完成场次数量
     * @param
     * @return:
     * @author liuchuan
     * @date: 2023/12/7 17:54
    */
    @Query(value = "select count(id) from mth_round_instance where match_id=?1 \n" +
            "and round_type='instance' and round_level=2\n" +
            "and match_state='4' and del_flag=0  " ,nativeQuery = true)
    Integer queryDistrictFinishNum(Long matchId);


    /**
     * @description: 统计片区级的上一级（区县级）场次总量
     * @param null
     * @return:
     * @author liuchuan
     * @date: 2023/12/7 17:58
    */
    @Query(value = "select count(mri.id) from mth_round_area mra \n" +
            "left join mth_round_instance mri on mra.id=mri.area_id\n" +
            "where    round_type='instance' and mri.del_flag=0  \n" +
            "and mra.parent_id=?2 " +
            "and mri.match_id= ?1 " ,nativeQuery = true)
    Integer querycityCountryNum(Long matchId,Long areaId);

    /**
     * @description: 计片区级的上一级（区县级）场次已完成总量
     * @param null
     * @return: 
     * @author liuchuan
     * @date: 2023/12/7 18:00
    */
    @Query(value = "select count(mri.id) from mth_round_area mra \n" +
            "left join mth_round_instance mri on mra.id=mri.area_id\n" +
            "where    round_type='instance'  and mri.del_flag=0 \n" +
            "and mra.parent_id=?2 " +
            "and mri.match_id= ?1 and match_state='4' " ,nativeQuery = true)
    Integer querycityCountryFinishNum(Long matchId,Long areaId);

    @Query(value = "select count(id) from mth_round_instance where match_id=?1 and match_state!='4' and round_state='0' and del_flag=0 ",nativeQuery = true)
    Integer queryMatchNotFinishNum(Long matchId);

    @Query(value = "  select count(id) from mth_round_instance where match_id=?1 and round_type='instance' and match_level='county_school' and del_flag=0;",
    nativeQuery = true)
    Integer queryCountryLevelTotal(Long matchId);

    @Query(value = "  select count(id) from mth_round_instance where match_id=?1 and round_type='instance' and match_level='county_school' and del_flag =0 and match_state='4';\n",nativeQuery = true)
    Integer queryCountryLevelFinishNum(Long matchId);

    @Query(value = "select * from mth_round_instance where del_flag=0 and round_type='instance' and round_state =0 and match_id=?1 and del_flag=0 ",nativeQuery = true)
    List<MthRoundInstance> findByMatchIdAndMatchState(Long matchId);

}
