package com.crunii.micro.domain.repo.main;

import com.crunii.micro.domain.common.BaseRepo;
import com.crunii.micro.domain.entity.main.MthRoundScore;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 赛事-场次-比赛评分;Mapper接口
 *
 * @author mth
 * @date 2023-09-19
 */
@Repository
public interface MthRoundScoreRepo extends BaseRepo<MthRoundScore, Long> {

    List<MthRoundScore> findByRoundListId(Long roundListId);

    /**
     * @description: 查询除开当前页面外的其他晋级数量
     * @param roundId 场次id
     * @param idList 当前页面id
     * @return: 晋级数量
     * @author liuchuan
     * @date: 2023/11/28 14:11
    */
    @Query(value = "select count(s.id) from mth_round_list l left join mth_round_score s on l.id=s.round_list_id" +
            " where s.RESULT='1' and l.round_id= ?1" +
            " and s.id not in( ?2 )",nativeQuery = true)
    Integer promotionNum(Long roundId,List<Long> idList);


    /**
     * @description: 查询获奖人员名单
     * @param
     * @return: 
     * @author liuchuan
     * @date: 2023/12/11 16:27
    */
    @Query(value = "          select l.user_id,s.prize_name,s.other_prize_name from mth_round_list l left join mth_round_score s on l.id=s.round_list_id where \n" +
            "          l.round_id=?1 " +
            "          and ((s.prize_id is not null and s.prize_id!='') or (s.other_prize_id is not null and s.other_prize_id!=''))",nativeQuery = true)
    List<Object[]> getAwardsList(Long roundId);


    /**
     * @description: 查询晋级人员id
     * @param
     * @return: 
     * @author liuchuan
     * @date: 2023/12/11 16:27
    */
    @Query(value = "  select l.user_id from mth_round_list l left join mth_round_score s on l.id=s.round_list_id where \n" +
            "          l.round_id=?1 " +
            "          and s.RESULT='1'",nativeQuery = true)
    List<Long> getPromptionList(Long roundId);



    @Query(value = "select l.register_id,s.result,s.prize_name,s.other_prize_name from mth_round_list l left join mth_round_score s on l.id = s.round_list_id\n" +
            " where  l.round_id=?1 ",nativeQuery = true)
    List<Object[]> getRoundPromotionInfo(Long roundId);

    /**
     * 查询最新获奖记录
     * @param registerId
     * @return
     */
    @Query(value = " select b.* from mth_round_list a  " +
            " left join mth_round_score b on a.id = b.round_list_id " +
            " left join mth_register c on a.register_id=c.id " +
            " left join mth_match_info d on c.match_id=d.id " +
            " where a.register_id=?1 and d.status='FINISHED' " +
            " and ((b.prize_name is not null and b.prize_name!='') or (b.other_prize_name is not null and b.other_prize_name!=''))  " +
            " order by b.create_time desc limit 1",nativeQuery = true)
    List<MthRoundScore> getAllPrizeName(Long registerId);




}
