package com.crunii.micro.service.match.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.MatchApproveBo;
import com.crunii.micro.service.match.api.bo.MatchApprovePageReqBo;
import com.crunii.micro.service.match.api.bo.MatchApproveReqBo;
import com.crunii.micro.service.match.api.vo.AppDetailVo;
import com.crunii.micro.service.match.api.vo.ApproveRecordVo;
import com.crunii.micro.service.match.api.vo.MatchApprovePageReqVo;
import com.crunii.micro.service.match.service.MatchApproveService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;


/**
 * @author gongsongtao
 * @date 2024/1/17 16:49
 */
@Api(tags = "赛事发布审核")
@RestController
@RequiredArgsConstructor
@RequestMapping("/auth/MatchApproveController")
public class MatchApproveController {

    private final MatchApproveService matchApproveService;

    @ApiOperation("提交审核")
    @PostMapping("/doSubmitApprove")
    private Result<Boolean> doSubmitApprove(@RequestBody MatchApproveReqBo matchApproveReqBo,
                                          @CurrWorker @ApiIgnore Worker worker){
        matchApproveService.doSubmitAudit(matchApproveReqBo,worker);
        return Result.ofSuccess(true);
    }

    @ApiOperation("审核")
    @PostMapping("/doApprove")
    private Result<Boolean> doApprove(@RequestBody MatchApproveBo matchApproveReqBo,
                                          @CurrWorker @ApiIgnore Worker worker){
        matchApproveService.doApprove(matchApproveReqBo,worker);
        return Result.ofSuccess(true);
    }

    @ApiOperation("查询赛事审核待办")
    @PostMapping("/pageApprove")
    private Result<PageRsp<MatchApprovePageReqVo>> pageApprove(@RequestBody PageReq<MatchApprovePageReqBo> pageReqBo,
                                                               @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(matchApproveService.pageApprove(pageReqBo,worker));
    }

    @ApiOperation("查询赛事审核记录")
    @GetMapping("/getApproveRecord")
    private Result<List<ApproveRecordVo>> getApproveRecord(@RequestParam("matchId") Long matchId){
        return Result.ofSuccess(matchApproveService.getApproveRecord(matchId));
    }

    @ApiOperation("app-查看待审核详情")
    @GetMapping("/getApproveDetail")
    private Result<AppDetailVo> getApproveDetail(@ApiParam("赛事id") @RequestParam("matchId") Long matchId,
                                                 @ApiParam("待办id") @RequestParam("nodeId") Long nodeId,
                                                 @CurrWorker @ApiIgnore Worker worker){
        return Result.ofSuccess(matchApproveService.getApproveDetail(matchId,nodeId,worker));
    }
}
