package com.crunii.micro.service.match.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.MatchRoundTemplateInfoVo;
import com.crunii.micro.service.match.api.vo.MthMatchInfoVo;
import com.crunii.micro.service.match.api.vo.MthRoundInstanceVo;
import com.crunii.micro.service.match.service.IMthRoundInstanceService;
import com.crunii.micro.service.match.service.MatchContractorManageService;
import com.crunii.micro.web.mvc.CurrWorker;

import io.swagger.annotations.*;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author denggl
 * date 2023/11/13
 */
@Api(tags = "赛事承办管理")
@RestController
@RequiredArgsConstructor
@RequestMapping("/auth/MatchContractorController")
public class MatchContractorManageController {
    private final MatchContractorManageService matchContractorManageService;
    private final IMthRoundInstanceService iMthRoundInstanceService;

    @ApiOperation("分页查询承办人的赛事")
    @PostMapping("/pageMatchContractor")
    public Result<PageRsp<MthMatchInfoVo>> pageMatchContractor(@RequestBody  PageReq<MatchPublishListBo> bo, @CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(matchContractorManageService.pageMatchContractor(bo, worker));

    }



    @ApiOperation("查看完善场次信息界面数据")
    @GetMapping("/previewInstance")
    public Result<List<MatchRoundTemplateInfoVo>> matchProcess(@ApiParam("主键")
                                                               @NotNull(message = "主键不能为空")
                                                               @RequestParam("matchId") Long matchId, @CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(matchContractorManageService.previewInstance(matchId,worker));
    }





    @ApiOperation("获取比赛下的所有场次信息")
    @GetMapping("/allRound")
    public Result<List<MthRoundInstanceVo>> getAllRoundInfo(@ApiParam("比赛id")
                                                            @NotNull(message = "主键不能为空")
                                                            @RequestParam("matchId") Long matchId,
                                                            @ApiParam("区域id")
                                                            @NotNull(message = "主键不能为空")
                                                            @RequestParam("areaId") Long areaId,
                                                            @CurrWorker @ApiIgnore Worker worker) {

        return Result.ofSuccess(matchContractorManageService.getAllRoundInfoByMatchId(matchId,areaId, worker));
    }
    @ApiOperation("承办人完善场次信息-编辑")
    @PostMapping("/updateMatchContractorInfo")
    public Result<String> updateMatchContractorInfo(@RequestBody  @Validated MthContractorRoundInstanceBo mthRoundReviewStandarBo, @CurrWorker @ApiIgnore Worker worker) {
        MthRoundInstanceBo mthRoundInstanceBo=new MthRoundInstanceBo();
        BeanUtils.copyProperties(mthRoundReviewStandarBo,mthRoundInstanceBo);
        iMthRoundInstanceService.updateMatchContractorInstance(mthRoundInstanceBo, worker);
        return Result.ofSuccess();
    }

    @ApiOperation("承办人完善场次信息-新增")
    @PostMapping("/saveMatchContractorInfo")
    public Result<String> saveMatchContractorInfo(@RequestBody  @Validated  MthContractorRoundInstanceBo mthRoundReviewStandarBo, @CurrWorker @ApiIgnore Worker worker) {
        MthRoundInstanceBo mthRoundInstanceBo=new MthRoundInstanceBo();
        BeanUtils.copyProperties(mthRoundReviewStandarBo,mthRoundInstanceBo);
        iMthRoundInstanceService.saveMatchContractorInstance(mthRoundInstanceBo, worker);
        return Result.ofSuccess();
    }


    @ApiOperation("承办人开启/关闭场次")
    @GetMapping("/updateRoundState")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roundId", value = "场次id", required = true),
            @ApiImplicitParam(name = "state", value = "0开启，1关闭", required = true)
    })
    public Result<String> updateRoundState(Long roundId, Integer state, @CurrWorker @ApiIgnore Worker worker) {
        iMthRoundInstanceService.updateRoundState(roundId, state, worker);
        return Result.ofSuccess();
    }
    @ApiOperation("承办人删除场次")
    @GetMapping("/delContractorRoundInstance")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roundId", value = "场次id", required = true)
    })
    public Result<String> delContractorRoundInstance(Long roundId,@CurrWorker @ApiIgnore Worker worker){
        matchContractorManageService.delContractorRoundInstance(roundId,worker);
        return Result.ofSuccess();

    }
    @ApiOperation("延长报名时间")
    @PostMapping("/updaterRgisterationEnd")
    public Result<String> updaterRgisterationEnd(@RequestBody  @Validated MathContractorExtendTimeBo bo,@CurrWorker @ApiIgnore Worker worker) {
        matchContractorManageService.updaterRgisterationEnd(bo,worker);
        return Result.ofSuccess();
    }

    @ApiOperation("延长作品上传时间")
    @PostMapping("/updateEntryDeadline")
    public Result<String>   updateEntryDeadline(@RequestBody  @Validated MathContractorExtendTimeBo bo,@CurrWorker @ApiIgnore Worker worker){
        matchContractorManageService.updateEntryDeadline(bo,worker);
        return Result.ofSuccess();
    }
    @ApiOperation("修改评审指标")
   @PostMapping("/updateIMthRoundReviewStandar")
    public Result<String> updateIMthRoundReviewStandar(@RequestBody List<MthRoundReviewStandarBo> list,@CurrWorker @ApiIgnore Worker worker){
        matchContractorManageService.updateIMthRoundReviewStandar(list,worker);
        return Result.ofSuccess();
    }
    /**
     * 获取场次详情信息
     */
    @ApiOperation("获取新增场场次基本信息")
    @GetMapping("/roundBaseInfo")
    public Result<MthRoundInstanceVo> roundBaseInfo(@ApiParam("主键")
                                              @NotNull(message = "主键不能为空")
                                                  @RequestParam("matchId") Long matchId,@CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(matchContractorManageService.roundBaseInfo(matchId,worker));
    }


}
