package com.crunii.micro.service.match.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.match.dto.ArticleDetailVo;
import com.crunii.micro.service.match.dto.MatchCarouselVo;
import com.crunii.micro.service.match.dto.MatchSearchDto;
import com.crunii.micro.service.match.service.MatchEventService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * @author baoxy
 * @date 2023-06-26
 */
@Api(tags = "赛事活动管理")
@RestController
@RequestMapping("/auth/MatchEventController")
public class MatchEventController {

    @Resource
    private MatchEventService matchEventService;

    @ApiOperation(value = "赛事活动页面分页查询(web端)")
    @RequestMapping(path = "/matchSearchWeb", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> matchSearchWeb(@Valid @RequestBody PageReq<MatchSearchDto> req) {
        return matchEventService.matchSearchWeb(req);
    }

    @ApiOperation(value = "赛事活动页面分页查询(app端)")
    @RequestMapping(path = "/matchSearchApp", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> matchSearchApp(@Valid @RequestBody PageReq<MatchSearchDto> req) {
        return matchEventService.matchSearchApp(req);
    }

    @ApiOperation(value = "赛事活动页面轮播(web端)")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "type", value = "查看类型(0:预告,1:进行中,2:赛事结果,历史赛事不传值不调用接口)", dataType = "String")
    )
    @RequestMapping(path = "/matchCarouselWeb", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<MatchCarouselVo>> matchCarouselWeb(String type) {
        return matchEventService.matchCarouselWeb(type);
    }

    @ApiOperation(value = "赛事活动页面轮播(app端)")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "type", value = "查看类型(0:预告,1:进行中,2:赛事结果,历史赛事类别不调用接口)", dataType = "String")
    )
    @RequestMapping(path = "/matchCarouselApp", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<MatchCarouselVo>> matchCarouselApp(String type) {
        return matchEventService.matchCarouselApp(type);
    }

    @ApiOperation(value = "赛事活动文章详情页")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "articleId", value = "文章id", required = true, dataType = "String")
    )
    @RequestMapping(path = "/articleDetail", method = RequestMethod.GET)
    @ResponseBody
    public Result<ArticleDetailVo> articleDetail(String articleId) {
        return matchEventService.articleDetail(articleId);
    }
}
