package com.crunii.micro.service.match.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqArticleDetail;
import com.crunii.micro.service.match.dto.*;
import com.crunii.micro.service.match.service.MatchManageService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * @author wangchen create 2023/6/27 9:32
 */
@Api(tags = "赛事管理-管理端")
@RestController
@RequestMapping("/auth/MatchManageController")
public class MatchManageController {

    private final MatchManageService matchManageService;

    public MatchManageController(MatchManageService matchManageService) {
        this.matchManageService = matchManageService;
    }

    @ApiOperation("创建文章 update公示")
    @PostMapping("/createArticleDetail")
    public Result<String> createArticleDetail(@Validated @RequestBody ArticleDetailDto articleDetailDto,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(matchManageService.createArticleDetail(articleDetailDto,worker));
    }

    @ApiOperation("关联比赛")
    @PostMapping("/relateMatch")
    public Result relateMatch(@Validated @RequestBody RelateMatchDto relateMatchDto){
        matchManageService.relateMatch(relateMatchDto);
        return Result.ofSuccess();
    }

    @ApiOperation("取消关联比赛")
    @PostMapping("/cancelRelateMatch")
    public Result cancelRelateMatch(@Validated @RequestBody RelateMatchDto relateMatchDto){
        matchManageService.cancelRelateMatch(relateMatchDto);
        return Result.ofSuccess();
    }

    @ApiOperation("分页查询文章")
    @PostMapping("/pageArticle")
    public Result<PageRsp<PageArticleVo>> pageArticle(@RequestBody PageReq<PageArticleDto> req, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(matchManageService.pageArticle(req,worker));
    }

    @ApiOperation("根据条件查询文章")
    @PostMapping("/listArticle")
    public Result<List<PageArticleVo>> listArticle(@RequestBody PageArticleDto req){
        return Result.ofSuccess(matchManageService.listArticle(req));
    }

    @ApiOperation("根据关联赛事id查询文章")
    @PostMapping("/findByMatchId")
    public Result<List<YcqArticleDetail>> findByMatchId(@RequestParam Long matchId){
        return Result.ofSuccess(matchManageService.findByMatchId(matchId));
    }

    @ApiOperation("更新文章状态")
    @PostMapping("/updateArticleState")
    public Result<String> updateArticleState(@Validated @RequestBody ArticleStateDto articleState ,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(matchManageService.updateArticleState(articleState,worker));
    }

    @ApiOperation("删除文章")
    @GetMapping("/delArticle")
    public Result<String> delArticle(Long articleId,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(matchManageService.delArticle(articleId,worker));
    }

    @ApiOperation("获取文章基本信息")
    @GetMapping("/getBasicArticle")
    public Result<BasicArticleVo> getBasicArticle(Long articleId){
        return Result.ofSuccess(matchManageService.getBasicArticle(articleId));
    }

    @ApiOperation("获取文章内容")
    @GetMapping("/getArticleContext")
    public Result<String> getArticleContext(Long articleId){
        return Result.ofSuccess(matchManageService.getArticleContext(articleId));
    }

    @ApiOperation("修改文章")
    @PostMapping("/updateArticle")
    public Result<String> updateArticle(@Validated @RequestBody ArticleUpdateDto articleUpdate,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(matchManageService.updateArticle(articleUpdate,worker));
    }

    @ApiOperation("轮播是否展示")
    @PostMapping("/updateCarousel")
    public Result<String> updateCarousel(@Validated @RequestBody CarouselShowDto carousel,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(matchManageService.updateCarousel(carousel,worker));
    }
}
