package com.crunii.micro.service.match.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.service.match.api.bo.MatchOrganizerAccountReqBo;
import com.crunii.micro.service.match.api.bo.MatchOrganizerAccountUpdateBo;
import com.crunii.micro.service.match.api.bo.OrganizationReqBo;
import com.crunii.micro.service.match.api.vo.MatchOrganizerAccountPageVO;
import com.crunii.micro.service.match.api.vo.TsmWorInfoVO;
import com.crunii.micro.service.match.service.MatchOrganizerAccountService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @Author thw
 * @date 2023/11/16
 * @description
 */
@Validated
@Api(tags = "赛事发布管理-生成承办方相关功能")
@RestController
@RequestMapping("/auth/MatchOrganizerAccountController")
public class MatchOrganizerAccountController {

  @Autowired
  private MatchOrganizerAccountService matchOrganizerAccountService;

  /**
   * 查询承办方账号信息内容
   */
  @ApiOperation("查询承办方账号信息内容")
  @PostMapping("/pageOrganizerAccountInfo")
  public Result<PageRsp<MatchOrganizerAccountPageVO>> pageOrganizerAccountInfo(@Validated(AddGroup.class) @RequestBody PageReq<MatchOrganizerAccountReqBo> bo) {
    return Result.ofSuccess(matchOrganizerAccountService.pageOrganizerAccountInfo(bo));
  }


  @ApiOperation("生成承办方临时账号")
  @PostMapping("/saveTempOrganizerAccount")
  public Result saveTempOrganizerAccount(@Validated(AddGroup.class) @RequestBody MatchOrganizerAccountReqBo bo) {
    return Result.ofSuccess(matchOrganizerAccountService.saveTempOrganizerAccount(bo));
  }


  @ApiOperation("修改承办方账号信息")
  @PostMapping("/updateOrganizerAccount")
  public Result updateOrganizerAccount(@Validated(EditGroup.class) @RequestBody List<MatchOrganizerAccountUpdateBo> bo) {
    return Result.ofSuccess(matchOrganizerAccountService.updateOrganizerAccount(bo));
  }

  @ApiOperation("通过比赛id查询所有承办方临时账号并设置临时账号不可用。")
  @GetMapping("/updateOrganizerAccountState")
  public Result updateOrganizerAccountState(@RequestParam("matchId") @NotNull(message = "比赛id不能为空") Long matchId) {
    return Result.ofSuccess(matchOrganizerAccountService.updateOrganizerAccountState(matchId));
  }

  @ApiOperation(value = "导出承办方临时账号")
  @RequestMapping(path = "/exportOrganizerAccount", method = RequestMethod.POST)
  @ResponseBody
  public void exportOrganizerAccount(HttpServletResponse response ,@Validated(EditGroup.class) @RequestBody MatchOrganizerAccountReqBo bo){
    matchOrganizerAccountService.exportOrganizerAccount(response,bo);
  }

  /**
   * 通过组织id查询用户信息
   */
  @ApiOperation("通过组织id查询用户信息")
  @PostMapping("/queryWorkerInfo")
  public Result<List<TsmWorInfoVO>> queryWorkerInfo(@Validated(EditGroup.class) @RequestBody  OrganizationReqBo bo) {
    return Result.ofSuccess(matchOrganizerAccountService.queryWorkerInfo(bo));
  }


  /**
   * 查询所有拥有承办方角色的用户
   */
//  @ApiOperation("查询所有拥有承办方角色的用户")
//  @PostMapping("/queryOrganizerAccount")
//  public Result<List<MthMatchContractor>> queryOrganizerAccount(@RequestBody MatchOrganizerAccountBo bo) {
//    return Result.ofSuccess(matchOrganizerAccountService.queryOrganizerAccount(bo));
//  }
}
