package com.crunii.micro.service.match.controller;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.enums.EnumUtils;
import com.crunii.micro.service.match.enums.RoundEnums;
import com.crunii.micro.service.match.service.IMthMatchInfoService;
import com.crunii.micro.service.match.service.IMthRegisterConfigService;
import com.crunii.micro.service.match.service.IMthRoundInstanceService;
import com.crunii.micro.service.sysmgr.api.domain.dto.OrgSimpleVO;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author xujie
 * @date 2023-09-22
 */
@Validated
@Api(value = "比赛发布管理控制器", tags = {"比赛发布管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/MatchPublishManageController")
public class MatchPublishManageController {
    private final IMthMatchInfoService iMthMatchInfoService;
    private final IMthRoundInstanceService iMthRoundInstanceService;
    private final IMthRegisterConfigService iMthRegisterConfigService;
//    private final OrgClient orgClient;


    @ApiOperation("用到的枚举LineEnum、ScoreMode、PromotionMode、EntrySubmitMode,AreaLevel")
    @GetMapping("/enumList")
    public Result<List<Map<Object, String>>> enumList(@RequestParam("enumName") String enumName) {

        List<Map<Object, String>> utilMap = null;

        switch (enumName) {
            case "LineEnum":
                utilMap = EnumUtils.EnumToMap(RoundEnums.LineEnum.class);
                break;

            case "ScoreMode":
                utilMap = EnumUtils.EnumToMap(RoundEnums.ScoreMode.class);
                break;
            case "PromotionMode":
                utilMap = EnumUtils.EnumToMap(RoundEnums.PromotionMode.class);
                break;
            case "EntrySubmitMode":
                utilMap = EnumUtils.EnumToMap(RoundEnums.EntrySubmitMode.class);
                break;
            case "AreaLevel":
                utilMap = EnumUtils.EnumToMap(RoundEnums.AreaLevel.class);
                break;
            default:
                utilMap = EnumUtils.EnumToMap(RoundEnums.LineEnum.class);
        }

        return Result.ofSuccess(utilMap);
    }

    /**
     * 新增赛事信息主;
     */
    @ApiOperation("新增赛事信息")
    @PostMapping("/add")
    public Result<Long> add(@Validated(AddGroup.class) @RequestBody MthMatchInfoBo bo) {
        return Result.ofSuccess(iMthMatchInfoService.insertByBo(bo));
    }

    @ApiOperation("查询创建市级比赛标志")
    @PostMapping("/queryCityLevelFlag")
    public Result<Boolean> queryCityLevelFlag() {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        boolean flag = StrUtil.equals(currWorker.getOrgId(), "1674703692521037822") || iMthMatchInfoService.queryCityLevelFlag(Long.valueOf(currWorker.getLoginId()));
        return Result.ofSuccess(flag);
    }

    /**
     * 修改赛事;
     */
    @ApiOperation("修改赛事信息")
    @PostMapping("/edit")
    public Result edit(@Validated(EditGroup.class) @RequestBody MthMatchInfoBo bo) {
        return Result.ofSuccess(iMthMatchInfoService.updateByBo(bo) ? 1 : 0);
    }

    /**
     * 赛事-保存草稿;
     */
    @ApiOperation("赛事-保存草稿")
    @PostMapping("/saveDraft")
    public Result<Long> saveDraft(@RequestBody MthMatchInfoBo bo) {
        return Result.ofSuccess(iMthMatchInfoService.saveDraft(bo));
    }

    /**
     * 新增赛事-区域-场次实例（实例与配置的关系）;
     */
    @ApiOperation("新增场次")
    @PostMapping("/roundAdd")
    public Result roundAdd(@Validated(AddGroup.class) @RequestBody MthRoundInstanceBo bo) {
        return Result.ofSuccess(iMthRoundInstanceService.insertByBo(bo));
    }

    /**
     * 场次-保存草稿
     */
    @ApiOperation("场次-保存草稿")
    @PostMapping("/saveRoundDraft")
    public Result saveRoundDraft(@RequestBody MthRoundInstanceBo bo) {
        return Result.ofSuccess(iMthRoundInstanceService.saveRoundDraft(bo));
    }

    /**
     * 获取场次详情信息
     */
    @ApiOperation("获取场次详情信息")
    @GetMapping("/roundDetail")
    public Result<MthRoundInstanceVo> getInfo(@ApiParam("主键")
                                              @NotNull(message = "主键不能为空")
                                              @RequestParam("roundId") Long roundId) {
        return Result.ofSuccess(iMthRoundInstanceService.getMthRoundInstanceById(roundId));
    }

    /**
     * 获取区县/校赛级场次配置的区域
     */
    @ApiOperation("获取区县/校赛级场次配置的区域")
    @GetMapping("/getCountySchoolRoundConfigArea")
    public Result<List<MthRoundAreaSimpleVo>> getCountySchoolRoundConfigArea(@ApiParam("主键")
                                                                             @NotNull(message = "主键不能为空")
                                                                             @RequestParam("matchId") Long matchId) {
        List<MthRoundAreaVo> roundAreaVos = iMthRoundInstanceService.getFirstRoundAreaByMatchIdAndMatchLevel(matchId, RoundEnums.AreaLevel.COUNTY_SCHOOL.getCode());
        List<MthRoundAreaSimpleVo> countyChildren = Lists.newArrayList();
        List<MthRoundAreaSimpleVo> universityChildren = Lists.newArrayList();
        for (MthRoundAreaVo item : roundAreaVos) {
            MthRoundAreaSimpleVo vo = new MthRoundAreaSimpleVo();
            vo.setAreaId(item.getId().toString());
            vo.setAreaCode(item.getAreaCode());
            vo.setAreaName(item.getAreaName());
            if (StrUtil.equals(item.getAreaType(), "3")) {
                vo.setAreaType("3");
                countyChildren.add(vo);
            }
            if (StrUtil.equals(item.getAreaType(), "4")) {
                vo.setAreaType("4");
                universityChildren.add(vo);
            }
        }
        MthRoundAreaSimpleVo county = new MthRoundAreaSimpleVo();
        county.setAreaName("重庆市区县");
        county.setChildren(countyChildren);

        MthRoundAreaSimpleVo university = new MthRoundAreaSimpleVo();
        university.setAreaName("重庆市高校");
        university.setChildren(universityChildren);
        Result<List<MthRoundAreaSimpleVo>> result = new Result<>();
        result.setData(Lists.newArrayList(county, university));
        return result;
    }

    @ApiOperation(value = "获取重庆市区县以及高校组织")
    @RequestMapping(path = "/getCountyAndUniversityOrg", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<MthRoundAreaSimpleVo>> getCountyAndUniversityOrg() {
        // 500000重庆市, 12141重庆高校
        List<Long> orgIds = Lists.newArrayList(500000L, 12141L);
        List<OrgSimpleVO> childrenList = iMthRoundInstanceService.getChildrenByParentOrgIds(orgIds);
        Map<Long, List<MthRoundAreaSimpleVo>> childrenMap = childrenList.stream()
                .map(item -> {
                    MthRoundAreaSimpleVo vo = new MthRoundAreaSimpleVo();
                    vo.setAreaId(item.getOrgId().toString());
                    vo.setAreaCode(item.getOrgCode());
                    vo.setAreaName(item.getOrgName());
                    vo.setParentId(item.getParentOrgId());
                    return vo;
                }).collect(Collectors.groupingBy(MthRoundAreaSimpleVo::getParentId));
        MthRoundAreaSimpleVo county = new MthRoundAreaSimpleVo();
        county.setAreaName("重庆市区县");
        county.setChildren(childrenMap.get(500000L));
        county.getChildren().forEach(item -> item.setAreaType("3"));
        MthRoundAreaSimpleVo university = new MthRoundAreaSimpleVo();
        university.setAreaName("重庆市高校");
        university.setChildren(childrenMap.get(12141L));
        university.getChildren().forEach(item -> item.setAreaType("4"));
        Result<List<MthRoundAreaSimpleVo>> result = new Result<>();
        result.setData(Lists.newArrayList(county, university));
        return result;
    }

    /**
     * 获取场次详情信息
     */
    @ApiOperation("获取比赛下的所有场次信息")
    @GetMapping("/allRound")
    public Result<List<MthRoundInstanceVo>> getAllRoundInfo(@ApiParam("比赛id")
                                                            @NotNull(message = "主键不能为空")
                                                            @RequestParam("matchId") Long matchId) {
        return Result.ofSuccess(iMthRoundInstanceService.getAllRoundInfoByMatchId(matchId));
    }

    /**
     * 获取比赛区域下的所有场次信息
     */
    @ApiOperation("获取比赛区域下的所有场次信息")
    @GetMapping("/areaAllRound")
    public Result<List<MthRoundInstanceVo>> getAreaAllRoundInfo(@ApiParam("比赛id")
                                                                @NotNull(message = "主键不能为空")
                                                                @RequestParam("matchId") Long matchId, @RequestParam("areaId") Long areaId) {
        return Result.ofSuccess(iMthRoundInstanceService.getAllRoundInfoByMatchIdAndAreaId(matchId, areaId));
    }

    /**
     * 获取比赛范围信息
     */
    @ApiOperation("获取比赛范围信息")
    @GetMapping("/queryArea")
    public Result<List<MthMatchAreaVo>> queryArea(@ApiParam("比赛id")
                                                  @NotNull(message = "主键不能为空")
                                                  @RequestParam("matchId") Long matchId) {
        return Result.ofSuccess(iMthRoundInstanceService.queryArea(matchId));
    }

    /**
     * 修改赛事-区域-场次实例（实例与配置的关系）;
     */
    @ApiOperation("修改场次")
    @PostMapping("/roundEdit")
    public Result roundEdit(@Validated(EditGroup.class) @RequestBody MthRoundInstanceBo bo) {
        return Result.ofSuccess(iMthRoundInstanceService.updateByBo(bo));
    }

    /**
     * 删除场次
     */
    @ApiOperation("删除场次")
    @PostMapping("/roundRemove")
    public Result remove(@RequestBody @ApiParam("主键串")
                         @NotEmpty(message = "主键不能为空")
                         Long[] ids) {
        return Result.ofSuccess(iMthRoundInstanceService.deleteWithValidByIds(Arrays.asList(ids), true));
    }


    @ApiOperation("系统配置表")
    @GetMapping("/formConfig")
    public Result<List<MthFormConfigVo>> fromConfigList() {
        return Result.ofSuccess(iMthRegisterConfigService.queryFormConfig());
    }

    @ApiOperation("系统配置表new")
    @GetMapping("/formConfigNew")
    public Result<Map<String, List<MthFormConfigVo>>> formConfigNew() {
        List<MthFormConfigVo> voList = iMthRegisterConfigService.queryFormConfig();
        Map<String, List<MthFormConfigVo>> group = StreamUtils.groupByKey(voList, MthFormConfigVo::getGroupName);
        return Result.ofSuccess(group);
    }

    @ApiOperation("查询系统配置详细信息 如性别")
    @GetMapping("/formConfigData/{configId}")
    public Result<List<MthFormConfigDataVo>> fromConfigDataList(@ApiParam("主键")
                                                                @NotNull(message = "主键不能为空")
                                                                @PathVariable("configId") Long configId) {
        return Result.ofSuccess(iMthRegisterConfigService.queryFormConfigData(Lists.newArrayList(configId)));
    }

    @ApiOperation("增加赛事配置表")
    @PostMapping("/matchRegisterConfig")
    public Result matchRegisterConfig(@Validated(EditGroup.class) @RequestBody MthRegisterConfigBo bo) {
        return Result.ofSuccess(iMthRegisterConfigService.createMatchConfig(bo));
    }


    @ApiOperation("修改赛事配置表")
    @PostMapping("/matchRegisterConfigEdit")
    public Result matchRegisterConfigEdit(@Validated(EditGroup.class) @RequestBody MthRegisterConfigBo bo) {
        return Result.ofSuccess(iMthRegisterConfigService.updateMatchConfit(bo));
    }

    @ApiOperation("配置App端赛事-新增")
    @PostMapping("/appMatchConfig")
    public Result appMatchConfig(@Validated(EditGroup.class) @RequestBody MthMatchConfigBo bo) {
        return Result.ofSuccess(iMthMatchInfoService.insertAppMatchConfig(bo));
    }


    @ApiOperation("配置App端赛事-查询")
    @GetMapping("/appMatchConfig/query")
    public Result<MthMatchConfigVo> getAppMatchConfig(@RequestParam("matchId") Long matchId) {
        return Result.ofSuccess(iMthMatchInfoService.getAppMatchConfig(matchId));
    }


    @ApiOperation("赛事提交")
    @PostMapping("/matchSubmit")
    public Result appMatchConfigSubmit(@Validated(EditGroup.class) @RequestBody MthMatchSubmitBo bo) {
        return Result.ofSuccess(iMthMatchInfoService.matchSubmit(bo));
    }

    /**
     * 比赛发布管理列表
     */
    @ApiOperation("查询赛事发布列表")
    @PostMapping("/publish/list")
    public Result<PageRsp<MthMatchInfoVo>> publishList(@RequestBody PageReq<MatchPublishListBo> bo) {
        return Result.ofSuccess(iMthMatchInfoService.publishList(bo));
    }

    @ApiOperation("获取赛事状态列表")
    @GetMapping("/match/status")
    public Result<List<MatchStatusVo>> matchStatus() {
        return Result.ofSuccess(iMthMatchInfoService.matchStatus());
    }

    @ApiOperation("发布赛事")
    @PostMapping("/release")
    public Result<Boolean> publishMatch(@ApiParam("主键串")
                                        @NotEmpty(message = "主键不能为空")
                                        @RequestBody Long[] ids) {
        return Result.ofSuccess(iMthMatchInfoService.publishMatch(Arrays.asList(ids)));
    }

    @ApiOperation("下架赛事")
    @PostMapping("/delist")
    public Result<Boolean> soldOutMatch(@ApiParam("主键串")
                                        @NotNull(message = "主键不能为空")
                                        @RequestBody Long[] ids) {

        return Result.ofSuccess(iMthMatchInfoService.soldOutMatch(Arrays.asList(ids)));
    }

    /**
     * 删除赛事;
     */
    @ApiOperation("删除赛事信息")
    @PostMapping("/remove")
    public Result removeMatch(@ApiParam("主键串")
                              @NotEmpty(message = "主键不能为空")
                              @RequestBody Long[] ids) {
        return Result.ofSuccess(iMthMatchInfoService.deleteWithValidByIds(Arrays.asList(ids), true) ? 1 : 0);
    }

    @ApiOperation("预览比赛-基本信息")
    @GetMapping("/baseinfo")
    public Result<MatchBaseInfoVo> matchBaseInfo(@ApiParam("主键")
                                                 @NotNull(message = "主键不能为空")
                                                 @RequestParam("matchId") Long matchId) {
        return Result.ofSuccess(iMthMatchInfoService.matchBaseInfo(matchId));
    }

    @ApiOperation("预览比赛场次流程")
    @GetMapping("/process")
    public Result<List<MatchRoundTemplateInfoVo>> matchProcess(@ApiParam("主键")
                                                               @NotNull(message = "主键不能为空")
                                                               @RequestParam("matchId") Long matchId) {
        return Result.ofSuccess(iMthMatchInfoService.matchProcess(matchId));
    }

    @ApiOperation("预览比赛报名表列表")
    @GetMapping("/application/form")
    public Result<List<MatchRegisterFormVo>> matchApplicationForm(@ApiParam("主键")
                                                                  @NotNull(message = "主键不能为空")
                                                                  @RequestParam("matchId") Long matchId) {
        return Result.ofSuccess(iMthMatchInfoService.matchApplicationForm(matchId));
    }

    @ApiOperation("预览比赛报名表详情")
    @GetMapping("/application/form/detail")
    public Result<MthRegisterConfigVo> ApplicationFormDetail(@ApiParam("主键")
                                                             @NotNull(message = "主键不能为空")
                                                             @RequestParam("id") Long id) {
        return Result.ofSuccess(iMthRegisterConfigService.queryConfigById(id));
    }
    @ApiOperation("获取赛项组别信息")
    @GetMapping("/queryItemList")
    public Result<List<MatchBaseItemVo> > queryItemList(@ApiParam("比赛id")
                                                            @NotNull(message = "主键不能为空")
                                                            @RequestParam("matchId") Long matchId) {
        return Result.ofSuccess(iMthMatchInfoService.queryItemList(matchId));
    }

    @ApiOperation("复制赛事")
    @PostMapping("/copyMatch")
    public Result<Long> copyMatch(@ApiParam("比赛id")
                                                        @NotNull(message = "主键不能为空")
                                                        @RequestParam("matchId") Long matchId) {
        return Result.ofSuccess(iMthMatchInfoService.copyMatch(matchId));
    }
}
