package com.crunii.micro.service.match.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.service.match.api.bo.MatchReviewerPageReqBo;
import com.crunii.micro.service.match.api.bo.MatchReviewerReqBo;
import com.crunii.micro.service.match.api.bo.MatchReviewerUpBo;
import com.crunii.micro.service.match.api.vo.MatchReviewerPageVO;
import com.crunii.micro.service.match.service.MatchReviewerManageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @Author thw
 * @date 2023/11/20
 * @description 生成评审员账号相关内容
 */
@Validated
@Api(value = "生成评审员账号相关内容", tags = {"生成评审员账号相关内容"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/MatchReviewerManageController")
public class MatchReviewerManageController {

  @Autowired
  private MatchReviewerManageService matchReviewerManageService;


  /**
   * 查询评审员账号账号信息内容
   */
  @ApiOperation("查询评审员账号账号信息内容")
  @PostMapping("/pageReviewerAccounts")
  public Result<PageRsp<MatchReviewerPageVO>> pageReviewerAccounts(@Validated(AddGroup.class) @RequestBody PageReq<MatchReviewerPageReqBo> bo) {
    return Result.ofSuccess(matchReviewerManageService.pageReviewerAccounts(bo));
  }


  @ApiOperation("生成评审员账号")
  @PostMapping("/saveReviewerAccount")
  public Result saveReviewerAccount(@Validated(AddGroup.class) @RequestBody MatchReviewerReqBo bo) {
    return Result.ofSuccess(matchReviewerManageService.saveReviewerAccount(bo));
  }

  @ApiOperation("修改评审员账号")
  @PostMapping("/updateReviewerAccount")
  public Result updateReviewerAccount(@Validated(EditGroup.class) @RequestBody List<MatchReviewerUpBo> bo) {
    return Result.ofSuccess(matchReviewerManageService.updateReviewerAccount(bo));
  }

  @ApiOperation(value = "导出评审员账号")
  @RequestMapping(path = "/exportReviewerAccount", method = RequestMethod.POST)
  @ResponseBody
  public void exportReviewerAccount(HttpServletResponse response ,@Validated(EditGroup.class) @RequestBody MatchReviewerPageReqBo bo){
    matchReviewerManageService.exportReviewerAccount(response,bo);
  }




}
