package com.crunii.micro.service.match.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.match.api.bo.MthAreaBo;
import com.crunii.micro.service.match.api.vo.MthAreaVo;
import com.crunii.micro.service.match.service.IMthAreaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 赛事-场次-比赛区域等级;Controller
 *
 * @author mth
 * @date 2023-09-19
 */

@Validated
@Api(value = "比赛区域等级控制器", tags = {"比赛区域等级控制器"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/MthAreaController")
public class MthAreaController {

    private final IMthAreaService iMthAreaService;

    /**
     * 查询赛事-场次-比赛区域等级;列表
     */
    @ApiOperation("比赛区域等级列表")
    @PostMapping("/pageList")
    public Result<PageRsp<MthAreaVo>> pageList(@RequestBody PageReq<MthAreaBo> bo) {
        return iMthAreaService.queryPageList(bo);
    }

    /**
     * 查询赛事-场次-比赛区域等级;列表
     */
    @ApiOperation("查询比赛区域等级列表")
    @PostMapping("/list")
    public Result<List<MthAreaVo>> list(MthAreaBo bo, HttpServletResponse response) {
        return Result.ofSuccess(iMthAreaService.queryList(bo));
    }

    /**
     * 获取赛事-场次-比赛区域等级;详细信息
     */
    @ApiOperation("获取比赛区域等级详细信息")
    @GetMapping("/{id}")
    public Result<MthAreaVo> getInfo(@ApiParam("主键")
                                          @NotNull(message = "主键不能为空")
                                          @PathVariable("id") Long id) {
        return Result.ofSuccess(iMthAreaService.queryById(id));
    }

}
