package com.crunii.micro.service.match.controller;


import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.pinyin.PinyinUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.domain.entity.main.TsmPropertyValue;
import com.crunii.micro.feign.client.SysService;
import com.crunii.micro.feign.client.dto.ValueC;
import com.crunii.micro.feign.client.dto.ValueU;
import com.crunii.micro.service.match.api.bo.TsmPropertyValueBo;
import com.crunii.micro.service.match.service.IMthMatchInfoService;
import com.crunii.micro.service.match.service.IMthRegisterConfigService;
import com.crunii.micro.service.match.service.IMthRoundInstanceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotEmpty;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

/**
 * 赛事信息主;Controller
 * 前端访问路由地址为:/match/info
 * @author mth
 * @date 2023-09-19
 */
@Validated
@Api(value = "赛事信息控制器", tags = {"赛事信息管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/MthMatchInfoController")
public class MthMatchInfoController {

    private final IMthMatchInfoService iMthMatchInfoService;
    private final IMthRoundInstanceService iMthRoundInstanceService;
    private final IMthRegisterConfigService iMthRegisterConfigService;

    private final SysService sysService;



//
//    /**
//     * 比赛管理列表
//     */
//    @ApiOperation("查询比赛管理列表")
//    @GetMapping("/match/manager/list")
//    public Result<PageRsp<mthMatchManagerVo>> mthMatchManagerList(PageReq<MthMatchManagerReqBo> bo) {
//        return Result.ofSuccess(iMthMatchInfoService.mthMatchManagerList(bo));
//    }
//
//    /**
//     * 比赛管理列表查看
//     */
//    @ApiOperation("比赛管理列表查看")
//    @GetMapping("/match_manager/detail")
//    public Result<List<mthMatchManagerDetailVo>> mthMatchManagerDetailList(MthMatchManagerReqBo bo) {
//        return Result.ofSuccess(iMthMatchInfoService.queryMatchManagerDetailList(bo));
//    }



    @ApiOperation("新增比赛分类")
    @PostMapping("/add_classification")
    public Result addClassification(@Validated(AddGroup.class) @RequestBody TsmPropertyValueBo bo){
        TsmPropertyValue value = new TsmPropertyValue();
        value.setPropCode("matchClassification");
        value.setValueCode(PinyinUtil.getPinyin(bo.getValueName(), StrUtil.EMPTY));
        if(bo.getValueName().length() > 10){
            throw new BusinessException("比赛分类名称不能超过10个字符");
        }
        value.setValueName(bo.getValueName());
        value.setState(1);
        Result<List<TsmPropertyValue>> items = sysService.getValuesByCode("matchClassification");
        value.setSortId(items.getData().size() + 1);
        return sysService.createValue(value);
    }

    @ApiOperation("修改比赛分类")
    @PostMapping("/edit_classification")
    public Result editClassification(@Validated(AddGroup.class) @RequestBody TsmPropertyValueBo bo) {
        Result<List<TsmPropertyValue>> result = sysService.getValuesByCode("matchClassification");
        Optional<TsmPropertyValue> valueOp = getTsmPropertyValue(bo, result);
        TsmPropertyValue value = valueOp.get();
        value.setValueName(bo.getValueName());
        value.setValueCode(PinyinUtil.getPinyin(bo.getValueName(), StrUtil.EMPTY));
        return sysService.updateValue(value);
    }

    @ApiOperation("删除比赛分类")
    @PostMapping("/remove_classification")
    public Result removeClassification(@Validated(AddGroup.class) @RequestBody TsmPropertyValueBo bo) {
        iMthMatchInfoService.validCategoryIdExistsUse(bo.getValueId());
        return sysService.delValue("matchClassification", bo.getValueId().intValue());
    }

    /**
     * 比赛分类列表
     */
    @ApiOperation("比赛分类列表查看")
    @GetMapping("/match_classification/list")
    public Result<List<TsmPropertyValue>> matchClassificationDetailList() {
        return sysService.getValuesByCode("matchClassification");
    }

    @ApiOperation("新增比赛标签")
    @PostMapping("/add_label")
    public Result addLabel(@Validated(AddGroup.class) @RequestBody TsmPropertyValueBo bo) {
        TsmPropertyValue value = new TsmPropertyValue();
        value.setPropCode("matchLabel");
        value.setValueCode(PinyinUtil.getPinyin(bo.getValueName(), StrUtil.EMPTY));
        if(bo.getValueName().length() > 30){
            throw new BusinessException("比赛标签名称不能超过30个字符");
        }
        value.setValueName(bo.getValueName());
        value.setState(1);
        Result<List<TsmPropertyValue>> items = sysService.getValuesByCode("matchLabel");
        value.setSortId(items.getData().size() + 1);
        return sysService.createValue(value);
    }

    @ApiOperation("修改比赛标签")
    @PostMapping("/edit_label")
    public Result editLabel(@Validated(AddGroup.class) @RequestBody TsmPropertyValueBo bo) {
        Result<List<TsmPropertyValue>> result = sysService.getValuesByCode("matchLabel");
        Optional<TsmPropertyValue> valueOp = getTsmPropertyValue(bo, result);
        TsmPropertyValue value = valueOp.get();
        value.setValueName(bo.getValueName());
        value.setValueCode(PinyinUtil.getPinyin(bo.getValueName(), StrUtil.EMPTY));
        return sysService.updateValue(value);
    }

    @ApiOperation("删除比赛标签")
    @PostMapping("/remove_label")
    public Result removeLabel(@Validated(AddGroup.class) @RequestBody TsmPropertyValueBo bo) {
        iMthMatchInfoService.validLabelUse(bo.getValueId());
        return sysService.delValue("matchLabel", bo.getValueId().intValue());
    }

    /**
     * 比赛标签列表
     */
    @ApiOperation("比赛标签列表查看")
    @GetMapping("/match_label/list")
    public Result<List<TsmPropertyValue>> matchLabelDetailList() {
        return sysService.getValuesByCode("matchLabel");
    }




    @ApiOperation("新增获奖名次")
    @PostMapping("/add_rank")
    public Result addRank(@Validated(AddGroup.class) @RequestBody TsmPropertyValueBo bo) {
        TsmPropertyValue value = new TsmPropertyValue();
        value.setPropCode("awardRank");
        value.setValueCode(PinyinUtil.getPinyin(bo.getValueName(), StrUtil.EMPTY));
        value.setValueName(bo.getValueName());
        value.setState(1);
        Result<List<TsmPropertyValue>> items = sysService.getValuesByCode("awardRank");
        value.setSortId(items.getData().size() + 1);
        return sysService.createValue(value);
    }

    @ApiOperation("修改获奖名次")
    @PostMapping("/edit_rank")
    public Result editRank(@Validated(AddGroup.class) @RequestBody TsmPropertyValueBo bo) {
        Result<List<TsmPropertyValue>> result = sysService.getValuesByCode("awardRank");
        Optional<TsmPropertyValue> valueOp = getTsmPropertyValue(bo, result);
        TsmPropertyValue value = valueOp.get();
        value.setValueName(bo.getValueName());
        value.setValueCode(PinyinUtil.getPinyin(bo.getValueName(), StrUtil.EMPTY));
        return sysService.updateValue(value);
    }

    private static Optional<TsmPropertyValue> getTsmPropertyValue(TsmPropertyValueBo bo, Result<List<TsmPropertyValue>> result) {
        Optional<TsmPropertyValue> valueOp = result.getData().stream()
                .filter(item -> ObjectUtil.equal(item.getValueId(), bo.getValueId().intValue())).findFirst();
        if (!valueOp.isPresent()) {
            throw new BusinessException("数据不存在");
        }
        return valueOp;
    }

    @ApiOperation("删除获奖名次")
    @PostMapping("/remove_rank")
    public Result removeRank(@Validated(AddGroup.class) @RequestBody TsmPropertyValueBo bo) {
        iMthMatchInfoService.validPrizeUse(bo.getValueId());
        return sysService.delValue("awardRank", bo.getValueId().intValue());
    }

    /**
     * 比赛标签列表
     */
    @ApiOperation("获奖名次列表查看")
    @GetMapping("/award_rank/list")
    public Result<List<TsmPropertyValue>> awardRankDetailList() {
        return sysService.getValuesByCode("awardRank");
    }
    @ApiOperation(value = "创建审核原因")
    @RequestMapping(path = "/crateCompetitionReview", method = RequestMethod.POST)
    Result<String> crateCompetitionReview(@RequestBody @ApiParam ValueC valueC){
        iMthMatchInfoService.crateCompetitionReview(valueC);
        return  Result.ofSuccess();
    }

    @ApiOperation(value = "修改审核原因")
    @RequestMapping(path = "/updateCompetitionReview", method = RequestMethod.POST)
    Result<String> updateCompetitionReview(@RequestBody @ApiParam ValueU valueU){
        iMthMatchInfoService.updateCompetitionReview(valueU);
        return Result.ofSuccess();
    }


}
