package com.crunii.micro.service.match.controller;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.MthMatchGroup;
import com.crunii.micro.domain.entity.main.MthMatchItem;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.dto.RoundColumVo;
import com.crunii.micro.service.match.service.IMthMatchInfoService;
import com.crunii.micro.service.match.service.IMthRoundInstanceService;
import com.crunii.micro.service.match.service.IMthRoundScoreService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Map;

/**
 * 比赛管理 Controller
 * 前端访问路由地址为:/match/mth_match_manager
 * @author luox
 * @date 2023-09-23
 */
@Validated
@Api(value = "比赛管理控制器", tags = {"比赛管理管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/MthMatchManagerController")
public class MthMatchManagerController {

    private final IMthMatchInfoService iMthMatchInfoService;

    private final IMthRoundScoreService iMthRoundScoreService;

    private final IMthRoundInstanceService iMthRoundInstanceService;
    /**
     * 比赛管理列表
     */
    @ApiOperation("比赛管理-列表")
    @PostMapping("/match_manager/list")
    public Result<PageRsp<mthMatchManagerVo>> mthMatchManagerList(@RequestBody PageReq<MthMatchManagerReqBo> bo) {
        return Result.ofSuccess(iMthMatchInfoService.mthMatchManagerList(bo));
    }

    /**
     * 比赛管理列表查看(列表查看按钮)
     */
    @ApiOperation("比赛管理-列表查看")
    @PostMapping("/match_manager/detail")
    public Result<List<mthMatchManagerDetailVo>> mthMatchManagerDetailList(@RequestBody MthMatchManagerReqBo bo) {
        return Result.ofSuccess(iMthMatchInfoService.queryMatchManagerDetailList(bo));
    }

    /**
     * 比赛管理列表查看报名信息-报名审核详情
     */
    @ApiOperation("比赛管理-报名审核详情")
    @PostMapping("/match_manager/registerInfo/detail")
    public Result<MthMyMatchListDetailAppVo> mthMatchRegisterDetail(@RequestBody MthMatchManagerRegisterReqBo bo) {
        return Result.ofSuccess(iMthMatchInfoService.queryMatchRegisterDetail(bo));
    }

    /**
     * 比赛管理-查看比赛过程信息
     */
    @ApiOperation("比赛管理-查看比赛过程信息")
    @PostMapping("/match_manager/processInfo")
    public Result<List<MthMatchProcessVo>> matchProcessInfo(@RequestBody MthMatchProcessReqBo bo) {
        return Result.ofSuccess(iMthMatchInfoService.queryProcessInfo(bo));
    }

    /**
     * 比赛管理-查看报名时间节点信息
     */
    @ApiOperation("比赛管理-查看报名时间节点信息")
    @PostMapping("/match_manager/registration_processInfo")
    public Result<MthRegistrationTimeNodeVo> registrationTimeNodeInfo(@RequestBody MthRegistrationTimeNodeReqBo req) {
        return Result.ofSuccess(iMthMatchInfoService.queryRegistrationProcessInfo(req));
    }

    /**
     * 比赛管理列表查看报名信息-报名审核
     */
    @ApiOperation("比赛管理-报名审核")
    @PostMapping("/match_manager/registerInfo/examine")
    public Result  mthMatchRegisterExamine(@RequestBody MthMatchManagerRegisterReqBo bo) {
        return Result.ofSuccess(iMthMatchInfoService.matchRegisterExamine(bo));
    }

    /**
     * 比赛管理-查看比赛信息-报名中
     */
    @ApiOperation("比赛管理-查看比赛信息-报名中")
    @PostMapping("/match_manager/registerInfo")
    public Result<PageRsp<MthMatchManagerRegisterVo>> queryMatchRegisterList(@RequestBody PageReq<MthMatchManagerRegisterReqBo> req) {
        return Result.ofSuccess(iMthMatchInfoService.queryMatchRegisterList(req));
    }

    /**
     * 比赛管理-查看比赛信息-报名中-导出
     * @param req
     * @return
     */
    @ApiOperation("比赛管理-查看比赛信息-报名中-导出")
    @PostMapping("/match_manager/exportRegisterInfo")
    public void exportRegisterInfo(@RequestBody PageReq<MthMatchManagerRegisterReqBo> req) {
        iMthMatchInfoService.exportRegisterList(req);
    }

    /**
     * 比赛管理-查看比赛信息-待评审
     */
    @ApiOperation("比赛管理-查看比赛信息-待评审")
    @PostMapping(path={"/match_manager/reviewInfo"})
    public Result<PageRsp<MthMatchManagerReviewVo>> queryPageReviewList(@RequestBody PageReq<MthMatchManagerReviewReqBo> req) {
        return Result.ofSuccess(iMthMatchInfoService.queryMatchReviewList(req));
    }

    /**
     * 比赛管理-查看比赛信息-待评审-导出
     * @param req
     * @return
     */
    @ApiOperation("比赛管理-查看比赛信息-待评审-导出")
    @PostMapping("/match_manager/exportReviewList")
    public void exportReviewList(@RequestBody PageReq<MthMatchManagerReviewReqBo> req) {
        iMthMatchInfoService.exportReviewList(req);
    }

    /**
     * 比赛管理-查看比赛信息-待设置晋级
     */
    @ApiOperation("比赛管理-查看比赛信息-待设置晋级")
    @PostMapping("/match_manager/queryMatchPromotedList")
    public Result<PageRsp<MthMatchManagerPromotedVo>> queryMatchPromotedList(@RequestBody PageReq<MthMatchManagerPromotedReqBo> req) {
        return Result.ofSuccess(iMthMatchInfoService.queryMatchPromotedList(req));
    }

    /**
     * 比赛管理-查看比赛信息-待设置晋级-导出
     * @param req
     * @return
     */
    @ApiOperation("比赛管理-查看比赛信息-待设置晋级-导出")
    @PostMapping("/match_manager/exportPromotedList")
    public void exportPromotedList(@RequestBody PageReq<MthMatchManagerPromotedReqBo> req) {
        iMthMatchInfoService.exportPromotedList(req);
    }

    /**
     * 比赛管理-查看比赛信息-待公示及以后
     */
    @ApiOperation("比赛管理-查看比赛信息-待公示及以后")
    @PostMapping("/match_manager/queryMatchPublicityList")
    public Result<PageRsp<MthMatchManagerPublicityVo>> queryMatchPublicityList(@RequestBody PageReq<MthMatchManagerPublicityReqBo> req) {
        return Result.ofSuccess(iMthMatchInfoService.queryMatchPublicityList(req));
    }

    /**
     * 比赛管理-查看比赛信息-待公示及以后-导出
     * @param req
     * @return
     */
    @ApiOperation("比赛管理-查看比赛信息-待公示及以后-导出")
    @PostMapping("/match_manager/exportPublicityList")
    public void exportPublicityList(@RequestBody PageReq<MthMatchManagerPublicityReqBo> req) {
        iMthMatchInfoService.exportPublicityList(req);
    }

    /**
     * 比赛管理-查看比赛信息-场次比赛信息
     */
    @ApiOperation("比赛管理-查看比赛信息-场次比赛信息")
    @PostMapping("/match_manager/queryMatchContestantList")
    public Result<PageRsp<MthMatchManagerContestantVo>> queryMatchContestantList(@RequestBody PageReq<MthMatchManagerContestantReqBo> req) {
        return Result.ofSuccess(iMthMatchInfoService.queryMatchContestantList(req));
    }

    /**
     * 比赛管理-查看比赛信息-场次比赛信息-导出
     * @param req
     * @return
     */
    @ApiOperation("比赛管理-查看比赛信息-场次比赛信息-导出")
    @PostMapping("/match_manager/exportContestantList")
    public void exportContestantList(@RequestBody PageReq<MthMatchManagerContestantReqBo> req) {
        iMthMatchInfoService.exportContestantList(req);
    }

    /**
     * 比赛管理-查看比赛信息-详情
     */
    @ApiOperation("比赛管理-查看比赛信息-详情")
    @PostMapping("/match_manager/reviewInfo/detail")
    public Result<MthMyMatchListDetailAppVo> mthMatchReviewDetail(@RequestBody MthMatchManagerReviewReqBo bo) {
        return Result.ofSuccess(iMthMatchInfoService.queryMatchReviewDetail(bo));
    }

    /**
     * 比赛管理-作品晋级排名设置
     */
    @ApiOperation("比赛管理-作品晋级排名设置(update)")
    @PostMapping("/match_manager/workPromotion")
    public Result<PageRsp<MthWorkPromotionVo>> workPromotionManagerList(@RequestBody PageReq<MthWorkPromiseReqBo> bo) {
        return Result.ofSuccess(iMthMatchInfoService.queryWorkPromotionManagerList(bo));
    }

    @ApiOperation("比赛管理-查询赛项")
    @GetMapping("/match_manager/itemList")
    public Result<List<MthMatchItem>> itemList(@RequestParam("matchId")Long matchId ) {
        return Result.ofSuccess(iMthMatchInfoService.getItemListByMatchId(matchId));
    }

    @ApiOperation("比赛管理-查询赛项下的组别")
    @GetMapping("/match_manager/groupList")
    public Result<List<MthMatchGroup>> groupList(@RequestParam("itemId")Long itemId ) {
        return Result.ofSuccess(iMthMatchInfoService.getGroupListByItemId(itemId));
    }

    /**
     *  场次晋级排名列表保存、提交
     */
    @ApiOperation("场次晋级排名列表保存、提交。（待公示 公示结束状态点进去的页面点击必须是提交 saveOrSign=2）(update)")
    @PostMapping("/match_manager/roundScoreRankingSaveList")
    public Result getRoundScoreRankingSaveList(@ApiIgnore @CurrWorker Worker worker, @RequestBody @ApiParam MthRoundScoreRankingSaveBo bo) {
        return Result.ofSuccess(iMthRoundScoreService.getRoundScoreRankingSaveList(worker,bo)? 1 : 0);
    }



    /**
     *  比赛管理-公示结束
     */
    @ApiOperation("比赛管理-公示结束(add)")
    @PostMapping("/match_manager/publicityFinish")
    public Result matchRoundPublicityFinish(@RequestBody MthMatchManagerReqBo req) {
        return Result.ofSuccess(iMthMatchInfoService.matchRoundPulicityFinish(req)? 1 : 0);
    }

    /**
     *  比赛管理-结束比赛
     */
    @ApiOperation("比赛管理-结束比赛")
    @PostMapping("/match_manager/finish")
    public Result matchRoundfinish(@RequestBody MthMatchManagerReqBo req) {
        return Result.ofSuccess(iMthMatchInfoService.matchRoundFinish(req)? 1 : 0);
    }


    /**
     *  比赛管理-发送获奖通知
     */
    @ApiOperation("比赛管理-发送获奖通知")
    @PostMapping("/match_manager/receiveMsg")
    public Result receiveMsg(@RequestBody ReceiveInfoBo bo) {
         return Result.ofSuccess(iMthRoundInstanceService.receiveMsg(bo));
    }



    /**
     * 比赛管理-查看获奖名单
     */
    @ApiOperation("比赛管理-查看获奖名单")
    @PostMapping("/match_manager/AwardList")
    public Result<List<MthAwardInfoVo>> AwardList(@RequestBody MthMatchManagerReqBo bo) {
        return Result.ofSuccess(iMthRoundScoreService.queryAwardList(bo));
    }


    /**
     * 比赛管理 查看收获地址
     */
    @ApiOperation("比赛管理-查看收获地址 add ")
    @PostMapping("/match_manager/deliveryInfo")
    public Result<PageRsp<MthRoundDeliveryVo>> deliveryInfo(@RequestBody PageReq<MthDeliveryBo> req) {
        return Result.ofSuccess(iMthMatchInfoService.getDeliveryInfo(req));
    }

    /**
     * 比赛管理 查看收获地址-导出
     */
    @ApiOperation("比赛管理-查看收获地址-导出")
    @PostMapping("/match_manager/exportDelivery")
    public void exportDelivery(@RequestBody PageReq<MthDeliveryBo> req) {
        iMthMatchInfoService.exportDelivery(req);
    }

    /**
     * 比赛管理 查看收获地址-是否邮寄
     */
    @ApiOperation("比赛管理-查看收获地址-是否邮寄")
    @PostMapping("/match_manager/updateDeliveryFlag")
    public Result updateDeliveryFlag(@RequestBody MthRoundDeliveryBo req) {
        return Result.ofSuccess(iMthMatchInfoService.updateDlieryFlag(req));
    }

//    /**
//     * 导出赛事信息主;列表
//     */
//    @ApiOperation("导出赛事信息列表")
//    @PostMapping("/export")
//    public void export(MthMatchInfoBo bo, HttpServletResponse response) {
//        List<MthMatchInfoVo> list = iMthMatchInfoService.queryList(bo);
//        //ExcelUtil.exportExcel(list, "赛事信息主;", MthMatchInfoVo.class, response);
//    }

    /**
     * 获取赛事信息主;详细信息
     */
    @ApiOperation("获取赛事信息详细信息")
    @GetMapping("/{id}")
    public Result<MthMatchInfoVo> getInfo(@ApiParam("主键")
                                     @NotNull(message = "主键不能为空")
                                     @PathVariable("id") Long id) {
        return Result.ofSuccess(iMthMatchInfoService.queryById(id));
    }

    /**
     * 获取场次信息主;详细信息
     */
    @ApiOperation("获取场次信息详细信息")
    @GetMapping("/match_manager/roundInfo")
    public Result<MthRoundInstanceVo> getRoundInfo(@ApiParam("主键")
                                          @NotNull(message = "主键不能为空")
                                          @RequestParam("roundId") Long roundId) {
        return Result.ofSuccess(iMthMatchInfoService.queryRoundInfoById(roundId));
    }

    @ApiOperation("场次信息动态表头信息")
    @PostMapping("/roundColum")
    public   Result<RoundColumVo> roundColum (@RequestBody  MthMatchManagerContestantReqBo  req){
        return Result.ofSuccess(iMthMatchInfoService.roundColum(req));

    }
    @ApiOperation("赛程管理报名中动态表头信息")
    @PostMapping("/registerInfoColum")
    public   Result<RoundColumVo> registerInfoColum (@RequestBody  MthMatchManagerRegisterReqBo  req){
        return Result.ofSuccess(iMthMatchInfoService.registerInfoColum(req));
    }

}
