package com.crunii.micro.service.match.controller;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.service.IMthProjectFinancingService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 青创赛事项目融资申请Controller
 * 前端访问路由地址为:/system/financing
 * @author ruoyi
 * @date 2023-12-18
 */
@Validated
@Api(value = "青创赛事项目融资申请控制器", tags = {"青创赛事项目融资申请管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/mthProjectFinancing")
public class MthProjectFinancingController {

    private final IMthProjectFinancingService iMthProjectFinancingService;

    /**
     * 查询青创赛事项目融资申请列表
     */
    @ApiOperation("查询青创赛事项目融资申请列表")
    @PostMapping("/pageList")
    public Result<PageRsp<MthProjectFinancingPageVo>> list(@RequestBody PageReq<PageProjectFinancingBo> bo) {
        return Result.ofSuccess(iMthProjectFinancingService.queryPageList(bo));
    }

    /**
     * 青创赛事项目融资申请列表-审核
     */
    @ApiOperation("青创赛事项目融资申请列表-审核")
    @PostMapping("/doAudit")
    public Result<Boolean> doAudit(@RequestBody ProjectFinancingAuditBo bo,@CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(iMthProjectFinancingService.doAudit(bo,worker));
    }

    /**
     * 青创赛事项目融资申请列表-修改公示状态
     */
    @ApiOperation("青创赛事项目融资申请列表-修改公示状态")
    @PostMapping("/doPublicity")
    public Result<Boolean> doPublicity(@RequestBody ProjectFinancingAuditBo bo) {
        return Result.ofSuccess(iMthProjectFinancingService.doPublicity(bo));
    }

    /**
     * 获取青创赛事项目融资申请详细信息
     */
    @ApiOperation("获取青创赛事项目融资申请详细信息")
    @GetMapping("/{id}")
    public Result<MthProjectFinancingDetailsVo> getInfo(@ApiParam("主键")
                                     @NotNull(message = "主键不能为空")
                                     @PathVariable("id") Long id) {
        return Result.ofSuccess(iMthProjectFinancingService.queryById(id));
    }

    /**
     * app-查询融资项目库
     */
    @ApiOperation("查询融资项目库")
    @PostMapping("/pageProject")
    public Result<PageRsp<MthProjectAppVo>> pageProject(@RequestBody PageReq<MthProjectAppBo> bo) {
        return Result.ofSuccess(iMthProjectFinancingService.pageProject(bo));
    }

    /**
     * app-查询融资详情
     */
    @ApiOperation("app-查询融资详情")
    @GetMapping("/getProjectDetails/{id}")
    public Result<MthProjectDetailsVo> getProjectDetails(@ApiParam("主键")
                                                                  @NotNull(message = "主键不能为空")
                                                                  @PathVariable("id") Long id) {
        return Result.ofSuccess(iMthProjectFinancingService.getProjectDetails(id));
    }


    /**
     * app-查询我的项目
     */
    @ApiOperation("查询我的项目")
    @PostMapping("/pageMyProjectApply")
    public Result<PageRsp<MthMyProjectApplyAppVo>> pageMyProjectApply(@RequestBody PageReq<MthMyProjectApplyAppBo> bo) {
        return Result.ofSuccess(iMthProjectFinancingService.pageMyProjectApply(bo));
    }

    /**
     * app-查询融资详情
     */
    @ApiOperation("app-查询我的项目详情")
    @GetMapping("/getMyProjectApplyDetails/{id}")
    public Result<MthMyProjectApplyDetailsVo> getMyProjectApplyDetails(@ApiParam("主键")
                                                         @NotNull(message = "主键不能为空")
                                                         @PathVariable("id") Long id) {
        return Result.ofSuccess(iMthProjectFinancingService.getMyProjectApplyDetails(id));
    }

    /**
     * web-新增融资申请
     */
    @ApiOperation("web-新增融资申请")
    @PostMapping("/createProject")
    public Result<Boolean> createProject(@RequestBody FinancingBo bo,@CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(iMthProjectFinancingService.createProject(bo,worker));
    }

    /**
     * 青创赛事-填写联系人信息
     */
    @ApiOperation("青创赛事-填写联系人信息")
    @PostMapping("/createFinancingContact")
    public Result<Boolean> createFinancingContact(@RequestBody ContactBo bo,@CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(iMthProjectFinancingService.createFinancingContact(bo,worker));
    }

    /**
     * app-青创赛事-填写联系人信息
     */
    @ApiOperation("app-青创赛事-查询联系人信息")
    @PostMapping("/getMyProjectContact")
    public Result<PageRsp<ContactVo>> getMyProjectContact(@RequestBody PageReq<ContactBo> req,@CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(iMthProjectFinancingService.getMyProjectContact(req,worker));
    }
}
