package com.crunii.micro.service.match.controller;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.MthReviewRoundBo;
import com.crunii.micro.service.match.api.bo.MthRoundRankingDetailBo;
import com.crunii.micro.service.match.api.bo.MthRoundRankingListBo;
import com.crunii.micro.service.match.api.bo.MthRoundScoreReviewBo;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.service.IMthRoundInstanceService;
import com.crunii.micro.service.match.service.IMthRoundListService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Map;

/**
 * 场次名单Controller
 * 前端访问路由地址为:/match/list
 * @author mth
 * @date 2023-09-19
 */
@Validated
@Api(value = "场次名单控制器", tags = {"场次名单管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/MthRoundListController")
public class MthRoundListController {

    private final IMthRoundListService iMthRoundListService;
    private final IMthRoundInstanceService iMthRoundInstanceService;


    /**
     * 评审场次查询;
     */
    @ApiOperation("评审场次查询")
    @PostMapping("/getReviewRound")
    public Result<List<MthReviewRoundVo>> getReviewRound(@ApiIgnore @CurrWorker Worker worker, @RequestBody MthReviewRoundBo bo) {
        return Result.ofSuccess(iMthRoundListService.getReviewRound(worker,bo));
    }

    @ApiOperation("场次奖项查询 (update)")
    @GetMapping("/getRoundPrize")
    public Result<List<MthRoundPrizeVo>> getRoundPrize(@NotNull(message = "场次id不能为空") @RequestParam("roundId")Long roundId) {
        return Result.ofSuccess(iMthRoundInstanceService.queryRoundPrizeById(roundId));
    }


    @ApiOperation("场次奖项（下拉框查询） (add)")
    @GetMapping("/getRoundPrizeForSelect")
    public Result<Map<Long,List<MthRoundPrizeVo>>> getRoundPrizeForSelect(@NotNull(message = "场次id不能为空") @RequestParam("roundId")Long roundId) {
        return Result.ofSuccess(iMthRoundInstanceService.getRoundPrizeByMode(roundId));
    }

    @ApiOperation("晋级人员查询 (add)")
    @GetMapping("/getRoundPromotion")
    public Result<List<MthRoundPrizeVo>> getRoundPromotion(@NotNull(message = "场次id不能为空") @RequestParam("roundId")Long roundId) {
        return Result.ofSuccess(iMthRoundInstanceService.queryRoundPromoyion(roundId));
    }





    /**
     * 查询设置排名列表(去评审);
     *      根据场次查询所有作品列表
     * 一期默认一个人评审员评审全部作品
     */
    @ApiOperation("查询设置排名列表，去评审")
    @PostMapping("/getRankingListByRound")
    public Result<PageRsp<MthRoundRankingListVo>> getRankingListByRound(@ApiIgnore @CurrWorker Worker worker, @RequestBody @ApiParam PageReq<MthRoundRankingListBo>  bo) {
        return Result.ofSuccess(iMthRoundListService.getRankingListByRound(worker,bo,false));
    }

    /**
     * 查询评审详情;
     */
    @ApiOperation("查询评审详情")
    @PostMapping("/getReviewDetail")
    public Result<MthReviewDetailVo> getReviewDetail(@ApiIgnore @CurrWorker Worker worker, @RequestBody @ApiParam MthRoundRankingDetailBo  bo) {
        return Result.ofSuccess(iMthRoundListService.getReviewDetail(worker,bo));
    }

    /**
     *  保存评审结果
     */
    @ApiOperation("保存评审结果")
    @PostMapping("/saveReviewResult")
    public Result saveReviewResult(@ApiIgnore @CurrWorker Worker worker, @RequestBody @ApiParam MthRoundScoreReviewBo bo) {
        return Result.ofSuccess(iMthRoundListService.
                 saveReviewResult(worker,bo) ? 1 : 0);
    }

    /**|
     * 导出当前筛选列表所有内容
     */
    @ApiOperation("列表导出")
    @PostMapping("/exportRankingListByRound")
    public void exportRankingListByRound( @ApiIgnore @CurrWorker Worker worker, @RequestBody @ApiParam MthRoundRankingListBo  bo) {
        iMthRoundListService.exportRankingListByRound(worker,bo);
    }


    /**
     *编辑查看已评分信息
     */
    @ApiOperation("编辑时查看已评分信息")
    @PostMapping("/getMthRoundScoreinfo")
     public  Result <MthRoundScoreinfoVo>  getMthRoundScoreinfo(@RequestBody @ApiParam  MthRoundRankingDetailBo bo,@ApiIgnore @CurrWorker Worker worker ){
        return  Result.ofSuccess(iMthRoundListService.getMthRoundScoreinfo(bo,worker));
    }

    /**
     *
     */
    @ApiOperation("修改评审结果")
    @PostMapping("/updateReviewResult")
    public Result updateReviewResult(@ApiIgnore @CurrWorker Worker worker, @RequestBody @ApiParam MthRoundScoreReviewBo bo) {
        return Result.ofSuccess(iMthRoundListService.
                updateReviewResult(worker,bo) ? 1 : 0);
    }


}
