package com.crunii.micro.service.match.controller;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.MthRoundScoreBo;
import com.crunii.micro.service.match.api.bo.MthRoundScoreRankingBo;
import com.crunii.micro.service.match.api.bo.MthRoundScoreRankingSaveBo;
import com.crunii.micro.service.match.api.vo.MthRoundScoreRankingVo;
import com.crunii.micro.service.match.api.vo.MthRoundScoreVo;
import com.crunii.micro.service.match.service.IMthRoundScoreService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Arrays;
import java.util.List;

/**
 * 比赛评分Controller
 * 前端访问路由地址为:/match/score
 * @author mth
 * @date 2023-09-19
 */
@Validated
@Api(value = "比赛评分控制器", tags = {"比赛评分管理"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/MthRoundScoreController")
public class MthRoundScoreController {

    private final IMthRoundScoreService iMthRoundScoreService;

    /**
     *  场次晋级排名列表查询
     *      二期建议评审和总分值及奖项拆分
     */
    @ApiOperation("场次晋级排名列表查询")
    @PostMapping("/getRoundScoreRankingList")
    public Result<PageRsp<MthRoundScoreRankingVo>> getRoundScoreRankingList(@ApiIgnore @CurrWorker Worker worker, @RequestBody @ApiParam PageReq<MthRoundScoreRankingBo> bo) {
        return Result.ofSuccess(iMthRoundScoreService.getRoundScoreRankingList(worker,bo));
    }

    /**
     *  场次晋级排名列表保存、提交
     */
    @ApiOperation("场次晋级排名列表保存、提交")
    @PostMapping("/roundScoreRankingSaveList")
    public Result getRoundScoreRankingSaveList(@ApiIgnore @CurrWorker Worker worker, @RequestBody @ApiParam MthRoundScoreRankingSaveBo bo) {
        return Result.ofSuccess(iMthRoundScoreService.getRoundScoreRankingSaveList(worker,bo)? 1 : 0);
    }
}
