package com.crunii.micro.service.match.controller.app;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.validate.QueryGroup;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.service.IMthMatchInfoService;
import com.crunii.micro.service.match.service.IMthRegisterService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.util.List;

/**
 * APP比赛活动控制器Controller
 * 前端访问路由地址为:/auth/app/match
 *
 * @author mth
 * @date 2023-09-19
 */
@Validated
@Api(value = "APP比赛活动控制器", tags = {"APP比赛活动"})
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/MthMatchInfoAppController")
public class MthMatchInfoAppController {

    private final IMthMatchInfoService iMthMatchInfoService;

    private final IMthRegisterService registerService;

    /**
     * APP赛事活动-查询赛事列表
     */
    @ApiOperation("APP赛事活动-查询赛事列表")
    @PostMapping("/list")
    public Result<PageRsp<MthMatchInfoAppVo>> list(@RequestBody PageReq<MthMatchInfoAppBo> bo) {
        return iMthMatchInfoService.queryPageListForApp(bo);
    }

    /**
     * APP赛事活动-查询赛事列表-查看详情
     */
    @ApiOperation("APP赛事活动-查询赛事列表-查看详情")
    @PostMapping("/getListDetails")
    public Result<MthMatchInfoAppVo> doListDetails(@ApiIgnore @CurrWorker Worker worker,@RequestBody MthMatchInfoAppBo bo) {
        return iMthMatchInfoService.doListDetails(worker,bo);
    }


    /**
     * APP赛事活动-查询报名范围
     */
    @ApiOperation("APP赛事活动-查询报名范围")
    @GetMapping("/queryRegisterArea")
    public Result<List<MthRegisterAreaAppVo>> queryRegisterArea(@ApiParam("比赛matchId")
                                                                @NotNull(message = "比赛matchId不能为空")
                                                                @RequestParam("matchId") Long matchId) {
        return Result.ofSuccess(registerService.queryRegisterAreaListForApp(matchId));
    }


    /**
     * APP赛事活动-查询报名表配置
     */
    @ApiOperation("APP赛事活动-查询报名表配置")
    @GetMapping("/queryRegisterConfig")
    public Result<MthRegisterConfigAppVo> queryRegisterConfig(@ApiParam("比赛id")
                                                              @NotNull(message = "比赛id不能为空") Long matchId,
                                                              @ApiParam("场次实例id")
                                                              @NotNull(message = "场次实例id不能为空") Long roundInstanceId,
                                                              @ApiParam("报名id") Long registerId) {
        return Result.ofSuccess(registerService.queryRegisterConfigForApp(matchId, roundInstanceId, registerId));
    }

    /**
     * APP赛事活动-查询作品表配置
     */
    @ApiOperation("APP赛事活动-查询作品表配置")
    @GetMapping("/queryWorkTableConfig")
    public Result<MthRegisterConfigAppVo.Work> queryWorkTableConfig(@ApiParam("比赛id")
                                                                    @NotNull(message = "比赛id不能为空") Long matchId,
                                                                    @ApiParam("报名id") Long registerId) {
        return Result.ofSuccess(registerService.queryWorkTableConfig(matchId, registerId));
    }

    /**
     * APP赛事活动-报名
     */
    @ApiOperation("APP赛事活动-报名")
    @PostMapping("/register")
    public Result register(@RequestBody MthRegisterAddAppBo bo) {
        return Result.ofSuccess(registerService.registerForApp(bo));
    }
    /**
     * APP赛事活动-信息更新
     */
    @ApiOperation("APP赛事活动-信息更新")
    @PostMapping("/registerUpdate")
    public Result registerUpdate(@RequestBody MthRegisterAddAppBo bo) {
        return Result.ofSuccess(registerService.registerUpdateForApp(bo));
    }

    /**
     * APP赛事活动-作品上传
     */
    @ApiOperation("APP赛事活动-作品上传")
    @PostMapping("/uploadWork")
    public Result uploadWork(@RequestBody MthRegisterAddAppBo.Work bo) {
        return Result.ofSuccess(registerService.uploadWorkForApp(bo));
    }

    /**
     * APP赛事活动-我的赛事列表
     */
    @ApiOperation("APP赛事活动-我的赛事列表")
    @PostMapping("/myMatchList")
    public Result<PageRsp<MthMyMatchListAppVo>> myMatchList(@RequestBody @Validated(QueryGroup.class) PageReq<MthMyMatchListAppBo> bo) {
        return iMthMatchInfoService.myMatchListForApp(bo);
    }

    /**
     * APP赛事活动-我的赛事列表-详情
     */
    @ApiOperation("APP赛事活动-我的赛事列表-详情")
    @GetMapping("/myMatchListDetail")
    public Result<MthMyMatchListDetailAppVo> myMatchListDetail(@ApiParam("比赛id")
                                                              @NotNull(message = "比赛id不能为空") Long matchId,
                                                              @ApiParam("报名id") Long registerId) {
        return Result.ofSuccess(registerService.myMatchListDetail(matchId, registerId));
    }

    /**
     * APP赛事活动-撤销报名
     */
    @ApiOperation("APP赛事活动-撤销报名")
    @PostMapping("/cancelRegister")
    public Result cancelRegister(@RequestBody @Validated(QueryGroup.class) MthCancelRegisterAppBo bo) {
        return Result.ofSuccess(registerService.cancelRegister(bo));
    }

    /**
     * APP赛事活动-收藏/取消收藏 赛事
     */
    @ApiOperation("APP赛事活动-赛事详情-收藏赛事")
    @PostMapping("/doCollectMatch")
    public Result doCollectMatch(@ApiIgnore @CurrWorker Worker worker, @RequestBody MatchCollectReqVo matchCollectReqVo) {
        return registerService.doCollectMatch(worker,matchCollectReqVo);
    }

    /**
     * APP赛事活动-查看已收藏赛事
     */
    @ApiOperation("APP赛事活动-查看已收藏赛事")
    @PostMapping("/pageCollectedMatch")
    public Result<PageRsp<MthMyMatchListAppVo>> pageCollectedMatch(@ApiIgnore @CurrWorker Worker worker, @RequestBody PageReq<MatchMyCollectBo> pageReq) {
        return registerService.pageCollectedMatch(worker,pageReq);
    }

    /**
     * APP赛事活动-赛事详情-查看通知公告
     */
    @ApiOperation("APP赛事活动-赛事详情-查看通知公告")
    @GetMapping("/queryMatchArticleByMatchId")
    public Result<List<MatchArticleRspVo>> queryMatchArticleByMatchId(@ApiParam("比赛id")@NotNull(message = "赛事id不能为空") Long matchId) {
        return Result.ofSuccess(registerService.queryMatchArticleByMatchId(matchId));
    }

    /**
     * APP赛事活动-赛事详情-下载签名图片
     */
    @ApiOperation("APP赛事活动-赛事详情-下载签名图片")
    @PostMapping("/doDownSignPicture")
    public Result<String> getSignPicture(@RequestBody MatchDownPictureReqVO matchDownPictureReqVO) throws IOException {
        return Result.ofSuccess(registerService.doDownSignPicture(matchDownPictureReqVO));
    }

    /**
     * APP青呼我为 banner图
     */
    @ApiOperation("APP青呼我为 banner图")
    @GetMapping("/queryBanner")
    public Result<List<MatchBannerVo>> queryBanner() {
        return Result.ofSuccess(iMthMatchInfoService.queryBanner());
    }
}
