package com.crunii.micro.service.match.controller.app;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.SignInfoVo;
import com.crunii.micro.service.match.api.vo.sign.LocationDataVO;
import com.crunii.micro.service.match.api.vo.sign.MthSignInReqVo;
import com.crunii.micro.service.match.service.MthMatchSignInService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author gongsongtao
 * @date 2023/11/21 18:46
 */
@Validated
@Api(tags = "赛事-App签到管理")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/SignInController")
public class MthMatchSignInController {

    @Resource
    private MthMatchSignInService mthMatchSignInService;

    @ApiOperation("APP签到-扫码提交")
    @PostMapping("/doSignIn")
    public Result<Boolean> doSignIn(@ApiIgnore @CurrWorker Worker worker, @RequestBody MthSignInReqVo mthSignInReqVo) {
        return mthMatchSignInService.doSignIn(worker, mthSignInReqVo);
    }

    @ApiOperation("APP签到-位置提交")
    @PostMapping("/doSiteSignIn")
    public Result<Boolean> doSiteSignIn(@ApiIgnore @CurrWorker Worker worker, @RequestBody MthSignInReqVo mthSignInReqVo) {
        return mthMatchSignInService.doSiteSignIn(worker, mthSignInReqVo);
    }

    @ApiOperation("APP赛事-签到-生成签到二维码")
    @PostMapping("/createSignQrCode")
    public void createSignQrCode(HttpServletResponse httpServletResponse,@RequestBody MthSignInReqVo mthSignInReqVo) {
        mthMatchSignInService.createSignQrCode(httpServletResponse,mthSignInReqVo);
    }

    @ApiOperation("web赛事-场次管理-保存签到信息")
    @PostMapping("/saveSignInformation")
    public Result<Boolean> saveSignInformation(@RequestBody SignInfoVo signInfoVo) {
        return Result.ofSuccess(mthMatchSignInService.saveSignInformation(signInfoVo));
    }

    @ApiOperation("web赛事-场次管理-查询签到信息")
    @GetMapping("/getSignInformation")
    public Result<List<SignInfoVo>> getSignInformation(@ApiParam("场次ID") @NotNull(message = "场次ID不能为空") Long instanceId) {
        return Result.ofSuccess(mthMatchSignInService.getSignInformation(instanceId));
    }

    @ApiOperation("计算经纬度")
    @PostMapping("/calculateDistance")
    public Result<LocationDataVO> doCalculateDistance(@RequestBody @Validated MthSignInReqVo mthSignInReqVo){
        return mthMatchSignInService.doCalculateDistance(mthSignInReqVo);
    }
}
