package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.enums.TaskTypeEnums;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.MatchApprovePageReqBo;
import com.crunii.micro.service.match.api.vo.MatchApprovePageReqVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author gongsongtao
 * @date 2024/1/18 20:18
 */
@Repository
public class MatchApproveDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;


    public PageRsp<MatchApprovePageReqVo> pageApprove(PageReq<MatchApprovePageReqBo> pageReqBo, Worker worker) {
        StringBuilder sql = new StringBuilder("select a.id,b.id as matchId,b.match_name,b.category_name,b.status,match_level,d.org_name as publishOrgName,b.qc_flag," +
                " c.worker_name as launchApproveWorkerName,a.launch_approve_time as launchApproveTime,a.result_type as resultType,b.other_organizer,b.other_sponsor," +
                " case when a.result_type=0 then false else true end as whetherApprove," +
                " case when a.result_type=1 then '审核通过' when a.result_type=3 then '审核不通过' when a.result_type=0 then '待审核' else null end resultTypeName" +
                " from YCQ_ACTIVE_FLOW_NODE a " +
                " left join mth_match_info b on a.biz_id=b.id" +
                " left join tsm_worker c on a.launch_approve_worker=c.login_id" +
                " left join tsm_org d on b.publish_org_id=d.org_id" +
                " where biz_type=:bizType " +
                " and(a.org_id=:orgId or a.login_id=:loginId or (a.department=:department and a.login_id is null))");

        StringBuilder countSql = new StringBuilder(" select count(*) from YCQ_ACTIVE_FLOW_NODE a  " +
                " left join mth_match_info b on a.biz_id=b.id " +
                " left join tsm_worker c on a.launch_approve_worker=c.login_id " +
                " where a.biz_type=:bizType " +
                " and(a.org_id=:orgId or a.login_id=:loginId or (a.department=:department and a.login_id is null))"
        );
        Map<String,Object> params = new HashMap<>();
        MatchApprovePageReqBo query = pageReqBo.getQuery();
        params.put("bizType",TaskTypeEnums.MATCH.getCode());
        params.put("orgId",worker.getOrgId());
        params.put("loginId",worker.getLoginId());
        params.put("department",worker.getDepartment());
        if (ObjectUtil.isNotEmpty(query)) {
            //赛事名称
            if (StringUtils.isNotEmpty(query.getMatchName())) {
                sql.append(" and b.match_name like concat('%',:matchName,'%')");
                countSql.append(" and b.match_name like concat('%',:matchName,'%')");
                params.put("matchName",query.getMatchName());
            }
            //审核状态
            if (query.getState() != null) {
                sql.append(" and a.result_type=:resultType");
                countSql.append(" and a.result_type=:resultType");
                params.put("resultType",query.getState());
            }
            //承办方
            //主办方
            if (query.getMatchIds().size() > 0) {
                sql.append(" and b.id in (:ids)");
                countSql.append(" and b.id in (:ids)");
                params.put("ids",query.getMatchIds());
            }
            // 创建组织
            if (query.getCreateOrg() != null) {
                sql.append(" and b.publish_org_id=:pushOrgId");
                countSql.append(" and b.publish_org_id=:pushOrgId");
                params.put("pushOrgId",query.getCreateOrg());
            }
            //提审人
            if (StringUtils.isNotEmpty(query.getLaunchApproveWorker())) {
                sql.append(" and c.worker_name like concat('%',:workerName,'%')");
                countSql.append(" and c.worker_name like concat('%',:workerName,'%')");
                params.put("workerName",query.getLaunchApproveWorker());
            }
            //比赛等级
            if (StringUtils.isNotEmpty(query.getMatchLevel())) {
                sql.append(" b.match_level like concat('%',:matchLevel,'%')");
                countSql.append(" b.match_level like concat('%',:matchLevel,'%')");
                params.put("matchLevel",query.getMatchLevel());
            }
        }
        sql.append(" order by a.create_time desc ");
        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<MatchApprovePageReqVo> matchSearchVoList;
        String dyType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dyType, params, sql, pageReqBo);
            matchSearchVoList = baseNpJdbcTemplate.queryForBeans(MatchApprovePageReqVo.class, sql.toString(), params);
        } else {
            matchSearchVoList = new ArrayList<>();
        }
        PageRsp<MatchApprovePageReqVo> rsp = new PageRsp<>();
        rsp.setResult(matchSearchVoList);
        rsp.setTotalNum(total);
        rsp.setPageNo(pageReqBo.getPageNo());
        rsp.setPageSize(pageReqBo.getPageSize());
        return rsp;
    }
}
