package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.MthRoundInstance;
import com.crunii.micro.service.match.api.bo.MatchPublishListBo;
import com.crunii.micro.service.match.api.vo.MatchRoundTemplateInfoVo;
import com.crunii.micro.service.match.api.vo.MthMatchInfoVo;
import com.crunii.micro.service.match.api.vo.MthRoundInstanceVo;
import com.crunii.micro.service.match.enums.RoundEnums;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author denggl
 * date 2023/11/13
 */
@Repository
public class MatchContractorManageDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;
    private static final String   ROUND_BASE_SQL=" SELECT  i.match_level,i.area_name,i.area_id  FROM mth_match_contractor c  right JOIN  mth_round_instance  i ON   c.match_id=i.match_id and c.area_id =i.area_id    where  i.del_flag=0 and  i.round_type='instance' and i.match_id=? and  c.contractor_id=? and c.del_flag=0";
    private static final String   PAGE_CONTRACTOR_MATCH_SQL=" SELECT  c.area_name,c.area_id, i.* FROM mth_match_contractor c  right JOIN  mth_match_info  i ON  c.match_id=i.id where  i.del_flag=0 and  c.contractor_id= ? and c.del_flag=0   ";
    private static final String PAGE_COUNT_CONTRACTOR_MATCH_SQL = " SELECT count(*) FROM mth_match_contractor c  right JOIN  mth_match_info  i ON  c.match_id=i.id where  i.del_flag=0  and c.contractor_id= ?  and c.del_flag=0  ";
    private static final String  FIND_BY_MATCHID_AND_ROUNDTYPE_SQL=" SELECT  i.* FROM mth_match_contractor c  right JOIN  mth_round_instance  i ON   c.match_id=i.match_id and c.area_id =i.area_id    where  i.del_flag=0 and  i.round_type='instance' and i.match_id=?  and c.del_flag=0 and   c.contractor_id=?  and c.area_id=?  ";
    private static final String previewInstance_sql=" \n" +
            " select sign_in_start_time,sign_in_end_time,sign_in_type, sign_in_longitude, sign_in_latitude, sign_in_qr_id,\n" +
            "    sign_out_start_time,sign_out_end_time,sign_out_type, sign_out_longitude, sign_out_latitude, sign_out_qr_id ," +
            "   publicity_flag as  publicity , publicity_days as publicityDay,prize_type ,reviewer_standar,entry_deadline ,round_start,round_end, mth_round_instance.id,round_name    as name,round_sort as roundSort,case match_level \n" +
            "                                   when 'city' then '市区级' \n" +
            "                                   when 'district' then '片区级' \n" +
            "                                   when 'county' then '区县级' \n" +
            "                                   when 'school' then '校区级' \n" +
            "                                   else '未知' end         as level, \n" +
            "                       case play_mode \n" +
            "                           when 'online' then '线上比赛' \n" +
            "                           when 'offline' then '线下比赛' \n" +
            "                           else '未知' end                              as playMode, \n" +
            "                       case audit_mode \n" +
            "                           when 'online' then '线上评审' \n" +
            "                           when 'offline' then '线下评审' \n" +
            "                           else '未知' end                              as auditMode, \n" +
            "                       case promotion_mode \n" +
            "                           when 'percent' then '百分比' \n" +
            "                           when 'num' then '人数' \n" +
            "                           else '其他' end                              as promotionMode, \n" +
            "                       promption_num                                    as promptionNum, \n" +
            "                       case entry_submit_mode \n" +
            "                           when 'follow_register' then '随报名上传' \n" +
            "                           when 'needless' then '不需要上传' \n" +
            "                           when 'alone' then '单独上传' \n" +
            "                           else '不需要上传' end                        as entrySubmitMode, \n" +
            "                       concat(to_char(registeration_start, 'YYYY-MM-DD'), '至', \n" +
            "                              to_char(registeration_end, 'YYYY-MM-DD')) as registerationTime, \n" +
            "                       concat(to_char(round_start, 'YYYY-MM-DD'), '至', \n" +
            "                              to_char(round_end, 'YYYY-MM-DD'))         as roundTime \n" +
            "                from mth_round_instance LEFT JOIN mth_match_contractor ON mth_match_contractor.match_id =mth_round_instance.match_id AND    mth_match_contractor.area_id =mth_round_instance.area_id \n" +
            "                WHERE mth_match_contractor. contractor_id=:contractorId  and  mth_round_instance. del_flag = 0  and mth_round_instance.match_id = :matchId and round_type = :type ";



    public PageRsp<MthMatchInfoVo> pageMatchContractor(PageReq<MatchPublishListBo> bo, Worker worker) {
        StringBuilder sql = new StringBuilder(PAGE_CONTRACTOR_MATCH_SQL);
        StringBuilder sqlCount = new StringBuilder(PAGE_COUNT_CONTRACTOR_MATCH_SQL);
        List<Object> params = new ArrayList<>();
        MatchPublishListBo query = bo.getQuery();
        //承办人id
        params.add(worker.getLoginId());
        if (query != null) {
            if (StrUtil.isNotEmpty(query.getMatchName())) {
                sql.append(" and i.match_name like ?");
                sqlCount.append(" and i.match_name like ?");
                params.add("%" + query.getMatchName() + "%");
            }
        }
        sql.append(" order by i.update_time desc ");
        int total = jt.queryForInt(sqlCount.toString(), params.toArray());
        List<MthMatchInfoVo> matchInfos;
        String dyType = jt.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dyType, params, sql, bo);
            matchInfos = jt.queryForBeans(MthMatchInfoVo.class, sql.toString(), params.toArray());
        } else {
            matchInfos = new ArrayList<>();
        }
        PageRsp<MthMatchInfoVo> rsp = new PageRsp<>();
        rsp.setResult(matchInfos);
        rsp.setTotalNum(total);
        rsp.setPageNo(bo.getPageNo());
        rsp.setPageSize(bo.getPageSize());
        return rsp;
    }
    public List<MatchRoundTemplateInfoVo>  previewInstance(Long id,Worker worker) {
        Map<String, Object> map = new HashMap<>();
        map.put("contractorId", worker.getLoginId());
        map.put("matchId", id);
        map.put("type", RoundEnums.RoundType.INSTANCE.getCode());
        return baseNpJdbcTemplate.queryForBeans(MatchRoundTemplateInfoVo.class, previewInstance_sql, map);
    }


    public List<MthRoundInstance> findByMatchIdAndRoundType(Long matchId,Long areaId, Worker worker) {
           return jt.queryForBeans(MthRoundInstance.class, FIND_BY_MATCHID_AND_ROUNDTYPE_SQL,matchId,worker.getLoginId(),areaId);
    }

    public List<MthRoundInstanceVo> roundBaseInfo(Long matchId, Worker worker) {

        return jt.queryForBeans(MthRoundInstanceVo.class,ROUND_BASE_SQL,matchId,worker.getLoginId());

    }
}
