package com.crunii.micro.service.match.jdbc;

import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.match.constants.MatchConstants;
import com.crunii.micro.service.match.dto.ArticleDetailVo;
import com.crunii.micro.service.match.dto.MatchCarouselVo;
import com.crunii.micro.service.match.dto.MatchSearchDto;
import com.crunii.micro.service.match.dto.MatchSearchVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @author baoxy
 * @date 2023-06-26
 */
@Repository
public class MatchEventDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public static final String MATCH_SEARCH_SQL = "select article_id, article_title,article_type, article_state," +
            " to_char(publish_time, 'yyyy-MM-dd') as publish_time,file_key,show_due from ycq_article_detail" +
            " where state = ? ";

    public static final String MATCH_SEARCH_COUNT_SQL = "select count(*) from ycq_article_detail where state = ? ";

    public static final String CAROUSEL_SQL = "select a.carousel_id, a.article_id, a.file_key, a.pic_width, a.pic_height from " +
            " ycq_match_carousel a, ycq_article_detail b where a.state = ? and b.state = ? and b.article_state = ? and b.carousel = ? " +
            " and start_time <= ? and end_time >= ? and web_pic = ? ";

    public PageRsp<MatchSearchVo> matchSearch(PageReq<MatchSearchDto> req) {
        StringBuilder sql = new StringBuilder(MATCH_SEARCH_SQL);
        StringBuilder countSql = new StringBuilder(MATCH_SEARCH_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        param.add(CommonConstants.STATE_VALID);
        if (StringUtils.isNotBlank(req.getQuery().getKeywords())) {
            sql.append(" and article_title like ? ");
            countSql.append(" and article_title like ? ");
            param.add("%" + req.getQuery().getKeywords() + "%");
        } else {
            dealArticleTypeAndShowDue(sql, countSql, param, req.getQuery().getType());
        }
        sql.append(" order by publish_time desc, article_type desc ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        List<MatchSearchVo> matchSearchVoList;
        String dyType = jt.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dyType, param, sql, req);
            matchSearchVoList = jt.queryForBeans(MatchSearchVo.class, sql.toString(), param.toArray());
        } else {
            matchSearchVoList = new ArrayList<>();
        }
        PageRsp<MatchSearchVo> rsp = new PageRsp<>();
        rsp.setResult(matchSearchVoList);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    public List<MatchCarouselVo> matchCarousel(String type, int isWebPic) {
        StringBuilder sql = new StringBuilder(CAROUSEL_SQL);
        List<Object> param = new ArrayList<>();
        param.add(CommonConstants.STATE_VALID);
        param.add(CommonConstants.STATE_VALID);
        param.add(CommonConstants.STATE_VALID);
        param.add(CommonConstants.STATE_VALID);
        param.add(LocalDateTime.now());
        param.add(LocalDateTime.now());
        param.add(isWebPic);
        sql.append(" and a.article_id = b.article_id ");
        if (StringUtils.isNotBlank(type)) {
            dealArticleTypeAndShowDue(sql, new StringBuilder(), param, type);
        }
        sql.append(" order by b.publish_time desc limit 10 ");
        return jt.queryForBeans(MatchCarouselVo.class, sql.toString(), param.toArray());
    }

    public ArticleDetailVo articleDetail(String articleId) {
        String sql = "select article_id, article_title, article_content, TO_CHAR(publish_time,'yyyy-MM-dd hh24:mi') as publish_time," +
                " org_name from ycq_article_detail where article_id = ? ";
        return jt.queryForBean(ArticleDetailVo.class, sql, articleId);
    }

    public void dealArticleTypeAndShowDue(StringBuilder sql, StringBuilder countSql, List<Object> param, String type) {
        if (StringUtils.isBlank(type)) {
            sql.append(" and article_state = ? ");
            countSql.append(" and article_state = ? ");
            param.add(CommonConstants.STATE_VALID);
        }
        if (MatchConstants.TYPE_HERALD.equals(type)) {
            sql.append(" and article_state = ? and article_type = ? and show_due = ? ");
            countSql.append(" and article_state = ? and article_type = ? and show_due = ? ");
            param.add(CommonConstants.STATE_VALID);
            param.add(type);
            param.add(MatchConstants.NOT_SHOW_DUE);
        }
        if (MatchConstants.TYPE_MATCH_ONGOING.equals(type)) {
            // 如果前端传值为1:赛事进行中.而赛事进行中又有三个类别 1:报名中 2:报名结束 3:赛事报道
            sql.append(" and article_state = ? and article_type in (?,?,?) and show_due = ? ");
            countSql.append(" and article_state = ? and article_type in (?,?,?) and show_due = ? ");
            param.add(CommonConstants.STATE_VALID);
            param.addAll(MatchConstants.MATCH_ONGOING_TYPE_LIST);
            param.add(MatchConstants.NOT_SHOW_DUE);
        }
        if (MatchConstants.TYPE_MATCH_RESULT.equals(type)) {
            sql.append(" and article_state = ? and article_type = ? and show_due = ? ");
            countSql.append(" and article_state = ? and article_type = ? and show_due = ? ");
            param.add(CommonConstants.STATE_VALID);
            param.add(MatchConstants.STATIC_MATCH_RESULT);
            param.add(MatchConstants.NOT_SHOW_DUE);
        }
        if (MatchConstants.TYPE_MATCH_HISTORY.equals(type)) {
            sql.append(" and show_due = ? ");
            countSql.append(" and show_due = ? ");
            param.add(MatchConstants.SHOW_DUE);
        }
    }
}
