package com.crunii.micro.service.match.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.match.dto.AnnexDto;
import com.crunii.micro.service.match.dto.PageArticleDto;
import com.crunii.micro.service.match.dto.PageArticleVo;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/6/27 9:33
 */
@Repository
public class MatchManageDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public PageRsp<PageArticleVo> pageArticle(PageReq<PageArticleDto> req) {
        StringBuilder sb = new StringBuilder("SELECT article_id,org_id,article_title,article_type,carousel,article_state,org_name,login_name,show_due,publish_time,match_id FROM YCQ_ARTICLE_DETAIL WHERE state = 1");
        StringBuilder sbCount = new StringBuilder("SELECT count(1)  FROM YCQ_ARTICLE_DETAIL WHERE state = 1");
        List<Object> params = new ArrayList<>();
        PageArticleDto query = req.getQuery();
        if(StringUtils.hasText(query.getArticleTitle())){
            sb.append(" and article_title like concat('%',?,'%')");
            sbCount.append(" and article_title like concat('%',?,'%')");
            params.add(query.getArticleTitle());
        }
        if(Objects.nonNull(query.getArticleType())){
            sb.append(" and article_type = ?");
            sbCount.append(" and article_type = ?");
            params.add(query.getArticleType());
        }

        if(Objects.nonNull(query.getCarousel())){
            sb.append(" and carousel = ?");
            sbCount.append(" and carousel = ?");
            params.add(query.getCarousel());
        }

        if(Objects.nonNull(query.getArticleState())){
            sb.append(" and article_state = ?");
            sbCount.append(" and article_state = ?");
            params.add(query.getArticleState());
        }

        if(Objects.nonNull(query.getOrgId())){
            sb.append(" and org_id = ?");
            sbCount.append(" and org_id = ?");
            params.add(query.getOrgId());
        }

        if(Objects.nonNull(query.getShowDue())){
            sb.append(" and show_due = ?");
            sbCount.append(" and show_due = ?");
            params.add(query.getShowDue());
        }

        if(Objects.nonNull(query.getStartPublishTime())){
            sb.append(" and publish_time >= ?");
            sbCount.append(" and publish_time >= ?");
            params.add(query.getStartPublishTime());
        }

        if(Objects.nonNull(query.getEndPublishTime())){
            sb.append(" and publish_time <= ?");
            sbCount.append(" and publish_time <= ?");
            params.add(query.getEndPublishTime());
        }

        if(Objects.nonNull(query.getMatchId())){
            sb.append(" and match_id = ?");
            sbCount.append(" and match_id = ?");
            params.add(query.getMatchId());
        }

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<PageArticleVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            sb.append(" order by create_time desc");
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(PageArticleVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<PageArticleVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<PageArticleVo> listArticle(PageArticleDto query) {
        StringBuilder sb = new StringBuilder("SELECT article_id,org_id,article_title,article_type,carousel,article_state,org_name,login_name,show_due,publish_time,match_id FROM YCQ_ARTICLE_DETAIL WHERE state = 1");
        List<Object> params = new ArrayList<>();
        if (StringUtils.hasText(query.getArticleTitle())) {
            sb.append(" and article_title like concat('%',?,'%')");
            params.add(query.getArticleTitle());
        }
        if (Objects.nonNull(query.getArticleType())) {
            sb.append(" and article_type = ?");
            params.add(query.getArticleType());
        }

        if (Objects.nonNull(query.getCarousel())) {
            sb.append(" and carousel = ?");
            params.add(query.getCarousel());
        }

        if (Objects.nonNull(query.getArticleState())) {
            sb.append(" and article_state = ?");
            params.add(query.getArticleState());
        }

        if (Objects.nonNull(query.getOrgId())) {
            sb.append(" and org_id = ?");
            params.add(query.getOrgId());
        }

        if (Objects.nonNull(query.getShowDue())) {
            sb.append(" and show_due = ?");
            params.add(query.getShowDue());
        }

        if (Objects.nonNull(query.getStartPublishTime())) {
            sb.append(" and publish_time >= ?");
            params.add(query.getStartPublishTime());
        }

        if (Objects.nonNull(query.getEndPublishTime())) {
            sb.append(" and publish_time <= ?");
            params.add(query.getEndPublishTime());
        }

        if (Objects.nonNull(query.getMatchId())) {
            sb.append(" and match_id = ?");
            params.add(query.getMatchId());
        }
        sb.append(" order by create_time desc");
        List<PageArticleVo> list = jdbcTemplate.queryForBeans(PageArticleVo.class, sb.toString(), params.toArray());
        return list;
    }

    public List<String> findOrgIdsByOrgId(String orgId) {
        String sql = "SELECT org_id FROM TSM_ORG START WITH org_id= ? CONNECT BY PRIOR parent_org_id=org_id";
        return jdbcTemplate.queryForRawObjects(String.class,sql,orgId);
    }

    public String findWorkerNameByLoginId(Integer loginId) {
        String sql = "SELECT worker_name FROM TSM_WORKER WHERE login_id = ?";
        return jdbcTemplate.queryForRawObject(String.class,sql,loginId);
    }

    public List<AnnexDto> findAnnex(Long articleId) {
        String sql = "SELECT file_name,file_key,file_type,file_size FROM YCQ_MATCH_ANNEX WHERE article_id = ?";
        return jdbcTemplate.queryForBeans(AnnexDto.class,sql,articleId);
    }

    public void delAnnexByArticleId(Long articleId) {
        String sql = "DELETE FROM YCQ_MATCH_ANNEX WHERE article_id = ?";
        jdbcTemplate.update(sql,articleId);
    }

    public Integer findCarouseCount(List<Integer> articleIds) {
        String sql = "SELECT count(1) FROM YCQ_ARTICLE_DETAIL WHERE have_carousel = 1 AND carousel = 1 AND state = 1 AND article_state = 1 AND article_type in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",articleIds);
        return baseNpJdbcTemplate.queryForRawObject(Integer.class,sql,params);
    }
}
