package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.MthMatchContractor;
import com.crunii.micro.service.match.api.bo.MatchOrganizerAccountBo;
import com.crunii.micro.service.match.api.bo.MatchOrganizerAccountReqBo;
import com.crunii.micro.service.match.api.vo.MatchOrganizerAccountPageVO;
import com.crunii.micro.service.match.api.vo.MthMatchContractorVO;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import javax.annotation.Resource;
import org.springframework.stereotype.Repository;

/**
 * @Author thw
 * @date 2023/11/16
 * @description
 */
@Repository
public class MatchOrganizerAccountDao {

  @Resource(name = "mainJT")
  protected BaseJdbcTemplate jdbcTemplate;

  @Resource(name = "mainNPJT")
  protected BaseNPJdbcTemplate baseNpJdbcTemplate;

  private static final String PAGE_BASE_SQL = "select  a.match_id , a.area_id , a.area_name , "
      + "LISTAGG ( CASE WHEN a.limit_type = 'long_term' THEN CONCAT(a.org_id, '-', a.org_name, '-' ,a.contractor_id , '-' , a.contractor_name ) ELSE NULL END , ',')  "
      + "WITHIN GROUP (ORDER BY  a.contractor_id) contractorIdList , "
      + "LISTAGG ( CASE WHEN a.limit_type = 'temp' THEN CONCAT(a.contractor_id , '-' , a.contractor_name ) ELSE NULL END , ',') "
      + "WITHIN GROUP (ORDER BY  a.contractor_id) temp_id "
      + "from  mth_match_contractor a where a.del_flag = 0 ";

//  private static final String PAGE_COUNT_SQL = "select count(*) from  mth_match_contractor a where a.del_flag = 0 ";


  public PageRsp<MatchOrganizerAccountPageVO> pageOrganizerAccountInfo(
      PageReq<MatchOrganizerAccountReqBo> req, Worker currWorker) {
    StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
    StringBuilder sbCount = new StringBuilder();
    // query对象
    MatchOrganizerAccountReqBo query = req.getQuery();

    List<Object> params = new ArrayList<>();
      if (Objects.nonNull(query.getMatchId())) {
          sb.append(" and a.match_id = ? ");
//            sbCount.append(" and a.match_id = ? ");
          params.add(query.getMatchId());
      }

      sb.append(" group by a.match_id , a.area_id , a.area_name  ");
      sbCount.append( " select count(*) from (" + sb + ") a" );
    int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
    List<MatchOrganizerAccountPageVO> result;
    String dbType = jdbcTemplate.getDbType();
    if (total > 0) {
      //根据数据库类型设置limit
      SqlUtil.appendLimit(dbType, params, sb, req);

      result = jdbcTemplate.queryForBeans(MatchOrganizerAccountPageVO.class, sb.toString(), params.toArray());
    } else {
      result = new ArrayList<>(0);
    }
    PageRsp<MatchOrganizerAccountPageVO> page = new PageRsp<>();
    page.setPageNo(req.getPageNo());
    page.setPageSize(req.getPageSize());
    page.setTotalNum(total);
    page.setResult(result);
    // 为手机计算是否由下一页
    page.next();

    return page;
  }

  public List<MthMatchContractorVO> findLimitType(Long matchId) {
    StringBuffer sql = new StringBuffer("select a.* , b.match_level , w.login_pwd as contractorIdPwd , w.mobile from mth_match_contractor a   "
        + "left join mth_round_instance b on a.area_id = b.area_id and b.round_type = 'instance'  and b.round_sort = 1 and b.del_flag = 0  "
        + "left join TSM_WORKER w on a.contractor_id = w.login_id "
        + "where a.del_flag = 0 and a.limit_type = 'temp' ");
    ArrayList<Object> params = new ArrayList<>();
    if (ObjectUtil.isEmpty(matchId)){
      throw new BusinessException("比赛id必传!");
    }
    sql.append(" and a.match_id = ? ");
    params.add(matchId);
    return jdbcTemplate.queryForBeans(MthMatchContractorVO.class,sql.toString(),params.toArray());
  }

  public List<MthMatchContractor> queryOrganizerAccount(MatchOrganizerAccountBo bo) {
    StringBuffer sql = new StringBuffer(" select *  from mth_match_contractor where del_flag = 0  ");
    ArrayList<Object> params = new ArrayList<>();

    if (ObjectUtil.isNotEmpty(bo.getMatchId())){
      sql.append(" and match_id = ? ");
      params.add(bo.getMatchId());
    }
    if (ObjectUtil.isNotEmpty(bo.getAreaId())){
      sql.append(" and area_id = ? ");
      params.add(bo.getAreaId());
    }
    if (ObjectUtil.isNotEmpty(bo.getOrgId())){
      sql.append(" and org_id = ? ");
      params.add(bo.getOrgId());
    }
    if (ObjectUtil.isNotEmpty(bo.getLimitType())){
      sql.append(" and limit_type = ? ");
      params.add(bo.getLimitType());
    }

    return jdbcTemplate.queryForBeans(MthMatchContractor.class,sql.toString(),params.toArray());
  }
}
