package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.match.api.bo.MatchReviewerPageReqBo;
import com.crunii.micro.service.match.api.vo.MatchReviewerPageVO;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.Resource;
import org.springframework.stereotype.Repository;

/**
 * @Author thw
 * @date 2023/11/20
 * @description 评审员dao
 */
@Repository
public class MatchReviewerManageDao {

  @Resource(name = "mainJT")
  protected BaseJdbcTemplate jdbcTemplate;

  @Resource(name = "mainNPJT")
  protected BaseNPJdbcTemplate baseNpJdbcTemplate;

  private static final String PAGE_BASE_SQL = "select a.* , w.mobile , w.login_pwd reviewerPwd from mth_round_reviewer a \n"
      + "left join TSM_WORKER w on a.reviewer_id = w.login_id \n"
      + "where a.del_flag = 0  ";

  private static final String PAGE_COUNT_SQL = "select count(*) from mth_round_reviewer a \n"
      + "left join TSM_WORKER w on a.reviewer_id = w.login_id \n"
      + "where a.del_flag = 0  ";

  public PageRsp<MatchReviewerPageVO> pageReviewerAccounts(PageReq<MatchReviewerPageReqBo> req) {
    StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
    StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
    // query对象
    MatchReviewerPageReqBo query = req.getQuery();
    List<Object> params = new ArrayList<>();
    if (ObjectUtil.isNotEmpty(query)) {
      if (ObjectUtil.isNotEmpty(query.getId())) {
        sb.append(" and a.id = ? ");
        sbCount.append(" and a.id = ? ");
        params.add(query.getId());
      }
      if (ObjectUtil.isNotEmpty(query.getRoundId())) {
        sb.append(" and a.round_id = ? ");
        sbCount.append(" and a.round_id = ? ");
        params.add(query.getRoundId());
      }
      if (ObjectUtil.isNotEmpty(query.getReviewerId())) {
        sb.append(" and a.reviewer_id = ? ");
        sbCount.append(" and a.reviewer_id = ? ");
        params.add(query.getReviewerId());
      }
      if (ObjectUtil.isNotEmpty(query.getItemId())) {
        sb.append(" and a.item_id = ? ");
        sbCount.append(" and a.item_id = ? ");
        params.add(query.getItemId());
      }
      if (ObjectUtil.isNotEmpty(query.getItemName())) {
        sb.append(" and a.item_name = ? ");
        sbCount.append(" and a.item_name = ? ");
        params.add(query.getItemName());
      }
    }

      int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
      List<MatchReviewerPageVO> values;
      String dbType = jdbcTemplate.getDbType();
      if (total > 0) {
        //根据数据库类型设置limit
        sb.append(" order by create_time desc");
        SqlUtil.appendLimit(dbType, params, sb, req);
        values = jdbcTemplate
            .queryForBeans(MatchReviewerPageVO.class, sb.toString(), params.toArray());
      } else {
        values = new ArrayList<>(0);
      }
      PageRsp<MatchReviewerPageVO> page = new PageRsp<>();
      page.setPageNo(req.getPageNo());
      page.setPageSize(req.getPageSize());
      page.setTotalNum(total);
      page.setResult(values);
      // 为手机计算是否由下一页
      page.next();

      return page;
    }

}
