package com.crunii.micro.service.match.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.MthMatchInfo;
import com.crunii.micro.domain.entity.main.MthRoundInstance;
import com.crunii.micro.service.match.api.vo.RegisterPendingCheckVo;
import java.util.*;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

/**
 * @author wangchen create 2023/7/4 14:49
 */
@Repository
@Slf4j
public class MatchTaskDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public List<Long> findArticleIdsFromCarousel(Date now) {
        String sql = "SELECT DISTINCT(a.article_id) FROM YCQ_MATCH_CAROUSEL a LEFT JOIN YCQ_ARTICLE_DETAIL b ON a.article_id =b.article_id WHERE a.end_time < ? AND (b.carousel = 1 OR b.carousel = 2) AND b.state = 1 AND b.have_carousel = 1 ";
        return jdbcTemplate.queryForRawObjects(Long.class,sql,now);
    }

    public List<Long> findArticleIds(Date now) {
        String sql = "SELECT article_id FROM YCQ_ARTICLE_DETAIL WHERE (carousel = 1 OR carousel = 2) AND ? > end_show_time AND state = 1 AND have_carousel = 1";
        return jdbcTemplate.queryForRawObjects(Long.class, sql, now);
    }

    public void updateArticleCarousel(List<Long> allArticleList) {
        String sql = "UPDATE YCQ_ARTICLE_DETAIL SET carousel = 3 WHERE article_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",allArticleList);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void updateArticleCarouselAndArticleState(List<Long> articleIds) {
        String sql = "UPDATE YCQ_ARTICLE_DETAIL SET carousel = 3 ,article_state = 3 WHERE article_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",articleIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public List<Long> scanEnroll(Date now) {
        String sql = "SELECT article_id FROM YCQ_ARTICLE_DETAIL WHERE article_type = 1 AND state =1 AND end_enroll_time < ?";
        return jdbcTemplate.queryForRawObjects(Long.class, sql, now);
    }

    public void updateArticleType(List<Long> articleIds) {
        String sql = "UPDATE YCQ_ARTICLE_DETAIL SET article_type = 2 WHERE article_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",articleIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void updateAndShowDue(List<Long> showDueArticleIds) {
        String sql = "UPDATE YCQ_ARTICLE_DETAIL SET show_due = 1 ,article_state = 3 WHERE article_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",showDueArticleIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public List<Long> findShowDueArticleIds(Date now) {
        String sql = "SELECT article_id FROM YCQ_ARTICLE_DETAIL WHERE state = 1 AND ?>end_show_time AND show_due=0";
        return jdbcTemplate.queryForRawObjects(Long.class,sql,now);
    }

    public List<MthRoundInstance> findMthRoundInstance(Date now) {
        String sql = "select * from mth_round_instance where del_flag=0 and round_type='instance' and round_sort=1 and match_state is Null and registeration_start<=now() and registeration_end>now() ";//and CURRENT_DATE()>=registeration_start

        return jdbcTemplate.queryForBeans(MthRoundInstance.class,sql);
    }

    /**
     * @description: 查询需要流转转台为报名中的场次
     * @param
     * @return:
     * @author liuchuan
     * @date: 2023/12/5 10:27
    */
    public void  turnRoundMathState(Long matchId,String matchLevel) {
        String sql = "update mth_round_instance set match_state='0' where \n" +
                "del_flag=0 and round_type='instance' \n" +
                "and round_sort=1 and round_state='0' and match_state is Null and registeration_start<=now()\n" +
                " and registeration_end>now() and match_id= ?   and match_level= ? ";

        Integer num = jdbcTemplate.update(sql,matchId,matchLevel);
        if (num>0){
                log.info("matchId:{},matchLevel:{},updateNum:{}",matchId,matchLevel,num);
            log.info(sql);

        }
    }


    public List<MthMatchInfo> findMthByReleased() {
        String sql = "select * from mth_match_info where del_flag=0 and (status='RELEASED' or status='GOING' OR status='REGISTER') ";

        return jdbcTemplate.queryForBeans(MthMatchInfo.class,sql);
    }

    public List<MthRoundInstance> findEndMthRoundInstance(Date date) {
        String sql = "select * from mth_round_instance where del_flag=0 and round_type='instance' and match_state =0 and  registeration_end<=now() ";

        return jdbcTemplate.queryForBeans(MthRoundInstance.class,sql);
    }

    public  void updatePublicityFinish(){
        //将公示中的场次流转为公示结束
        String sql="update mth_round_instance set match_state='28' where round_type ='instance' and del_flag=0 and match_state='25' and publicity_deadline<now()";
        Integer num = jdbcTemplate.update(sql);
        if (num>0){
            log.info("将公示中场次状态流转为公示结束：{}",num);
        }
    }
    public RegisterPendingCheckVo checkRegisterPendingCheck(Long roundId) {
        String sql = "SELECT\n" +
                "\ts.round_id,\n" +
                "\tcount( CASE WHEN s.review_result = 'PENDING' THEN 1 END ) AS pending_num \n" +
                "FROM\n" +
                "\t(\n" +
                "\tSELECT\n" +
                "\t\tmrl.round_id AS round_id,\n" +
                "\t\tmrl.id AS round_list_id,\n" +
                "\t\tmr.id AS register_id,\n" +
                "\t\tmr.review_result AS review_result \n" +
                "\tFROM\n" +
                "\t\tmth_round_list mrl\n" +
                "\t\tLEFT JOIN mth_register mr ON mr.id = mrl.register_id \n" +
                "\tWHERE\n" +
                "\t\tmrl.round_id =? \n" +
                "\t) s GROUP BY s.round_id ";
        List<Object> params = new ArrayList<>();
        params.add(roundId);
        return jdbcTemplate.queryForBean(RegisterPendingCheckVo.class,sql,params.toArray());
    }

    public void updateRoundInstanceState(MthRoundInstance dto) {
        String sql = " update mth_round_instance set match_state =:state where id =:id ";
        Map<String,Object> params = new HashMap<>(2);
        params.put("state",dto.getMatchState());
//        params.put("date",new Date());
        params.put("id",dto.getId());
        log.info(" updateRoundInstanceState id:{},state:{}",dto.getId(),dto.getMatchState());
        baseNpJdbcTemplate.update(sql,params);
    }

    public List<MthRoundInstance> queryMatchRegisterRoundInstance(List<Long> matchIds) {
        // 根据赛事id查询场次序号为1的场次实例
        String sql = "\n" +
                "SELECT * FROM \n" +
                "\t(SELECT * FROM mth_round_instance \n" +
                "\t\tWHERE\n" +
                "\t\tdel_flag = 0\n" +
                "\t\tAND round_sort = 1\n" +
                "\t\tAND round_type = 'instance'\n" +
                "\t\tAND match_id IN (:matchIds)\n" +
                "\t) mri\n" +
                "\tINNER JOIN (\n" +
                "\t\tSELECT\n" +
                "\t\t\tmri.match_id,\n" +
                "\t\t\tIF(MIN(ml.sort)=3,'city',IF(MIN(ml.sort)=2,'district','county_school')) match_level\n" +
                "\t\tFROM\n" +
                "\t\t\tmth_round_instance mri\n" +
                "\t\t\tLEFT JOIN (SELECT 'city' AS match_level, 3 AS sort\n" +
                "\t\t\t\t\t\tUNION ALL SELECT 'district' AS match_level, 2 AS sort\n" +
                "\t\t\t\t\t\tUNION ALL SELECT 'county_school' AS match_level, 1 AS sort\n" +
                "\t\t\t\t) ml ON mri.match_level = ml.match_level\n" +
                "\t\tWHERE\n" +
                "\t\t\tmri.del_flag = 0\n" +
                "\t\t\tAND mri.round_sort = 1\n" +
                "\t\t\tAND mri.round_type = 'instance'\n" +
                "\t\t\tAND mri.match_id IN (:matchIds)\n" +
                "\t\t\tAND mri.match_level IS NOT NULL\n" +
                "\t\t\tGROUP BY mri.match_id\n" +
                "\t) mris \n" +
                "\tON mri.match_id = mris.match_id AND mri.match_level = mris.match_level";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        params.put("matchIds", matchIds);
        sb.append(whereSql);
        List<MthRoundInstance> result = baseNpJdbcTemplate.queryForBeans(MthRoundInstance.class, sb.toString(), params);
        return result;
    }
}
