package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.match.api.bo.MthAreaBo;
import com.crunii.micro.service.match.api.vo.MthAreaVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

@Repository
public class MthAreaDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    private static final String PAGE_BASE_SQL = "select * from mth_area a where del_flag = 0 ";
    private static final String PAGE_COUNT_SQL = "select count(*) from mth_area a where del_flag = 0 ";


    public PageRsp<MthAreaVo> queryPageList(PageReq<MthAreaBo> req) {
        StringBuilder sql = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        Map<String,Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        if (Objects.nonNull(req.getQuery())){

            if (Objects.nonNull(req.getQuery().getAreaName())) {
                whereSql.append(" and a.area_name like concat('%', :AreaName ,'%') ");
                params.put("AreaName", req.getQuery().getAreaName());
            }
            if (Objects.nonNull(req.getQuery().getAreaType())) {
                whereSql.append(" and a.area_type = :areaType ");
                params.put("areaType", req.getQuery().getAreaType());
            }
            if (Objects.nonNull(req.getQuery().getId())) {
                whereSql.append(" and a.id = :id ");
                params.put("id", req.getQuery().getId());
            }
        }
        countSql.append(whereSql);
        sql.append(whereSql);
        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<MthAreaVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(MthAreaVo.class, sql.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthAreaVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<MthAreaVo> queryList(MthAreaBo req){
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        Map<String,Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        if (StrUtil.isNotBlank(req.getAreaName())) {
            whereSql.append(" and a.area_name like concat('%', :AreaName ,'%') ");
            params.put("AreaName", req.getAreaName());
        }
        if (Objects.nonNull(req.getAreaType())) {
            whereSql.append(" and a.area_type = :areaType ");
            params.put("areaType", req.getAreaType());
        }
        if (Objects.nonNull(req.getId())) {
            whereSql.append(" and a.id = :id ");
            params.put("id", req.getId());
        }
        sb.append(whereSql);
        List<MthAreaVo> result = baseNpJdbcTemplate.queryForBeans(MthAreaVo.class, sb.toString(), params);
        return result;
    }

}
