package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.MthMatchInfo;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.api.vo.sign.MthSignRspVo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author xujie
 * @date 2023-09-20
 */
@Repository
public class MthMatchInfoDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    private static final String PAGE_BASE_SQL = "select * from mth_match_info a where del_flag = 0 ";
    private static final String PAGE_COUNT_SQL = "select count(*) from mth_match_info a where del_flag = 0 ";
    private static final String DELETE_SQL = "update mth_match_info set del_flag=1, update_id=:updateId, update_by=:updateBy, update_time=:updateTime where id in (:ids)";

    private static final String EDIT_STATUS_SQL = "update mth_match_info set status =:status where del_flag = 0 and id in(:ids)";

    public PageRsp<MthMatchInfoVo> publishList(PageReq<MatchPublishListBo> bo) {
        String sqlStr = "select mmi.*, if(mmc.match_id is not null, 1, 0) gen_flag, tw.org_id from mth_match_info mmi " +
                "left join (select match_id from mth_match_contractor group by match_id) mmc on mmi.id = mmc.match_id " +
                "left join tsm_worker tw on mmi.create_id = tw.login_id " +
                "where mmi.del_flag = 0 ";
        String sqlCountStr = "select count(*) from mth_match_info mmi where mmi.del_flag = 0 ";
        StringBuilder sql = new StringBuilder(sqlStr);
        StringBuilder sqlCount = new StringBuilder(sqlCountStr);
        Map<String, Object> params = new HashMap<>(16);
        MatchPublishListBo query = bo.getQuery();
        if (query != null) {
            if (StrUtil.isNotEmpty(query.getMatchName())) {
                sql.append(" and mmi.match_name like concat('%', :matchName ,'%') ");
                sqlCount.append(" and mmi.match_name like concat('%', :matchName ,'%') ");
                params.put("matchName", query.getMatchName());
            }
            if (ObjectUtil.isNotNull(query.getStatus())) {
                sql.append(" and mmi.status = :status");
                sqlCount.append(" and mmi.status = :status");
                params.put("status", query.getStatus());
            }
            if (ObjectUtil.isNotEmpty(query.getStatusList())) {
                sql.append(" and mmi.status in (:statusList)");
                sqlCount.append(" and mmi.status in (:statusList)");
                params.put("statusList", query.getStatusList());
            }
            if (StrUtil.isNotEmpty(query.getPublisherName())) {
                sql.append(" and mmi.publisher_name like concat('%', :publisherName ,'%') ");
                sqlCount.append(" and mmi.publisher_name like concat('%', :publisherName ,'%') ");
                params.put("publisherName", query.getPublisherName());
            }
            if (ObjectUtil.isNotNull(query.getPublishOrgId())) {
                sql.append(" and mmi.publish_org_id = :publishOrgId");
                sqlCount.append(" and mmi.publish_org_id = :publishOrgId");
                params.put("publishOrgId", query.getPublishOrgId());
            }
            if (StrUtil.isNotBlank(query.getMatchLevel())) {
                sql.append(" and mmi.match_level like concat('%', :matchLevel ,'%') ");
                sqlCount.append(" and mmi.match_level like concat('%', :matchLevel ,'%') ");
                params.put("matchLevel", query.getMatchLevel());
            }
            if (ObjectUtil.isNotNull(query.getSponsorId())) {
                sql.append(" and exists(select * from mth_match_sponsor where del_flag = 0 and type = 1 and match_id = mmi.id and sponsor_id = :sponsorId) ");
                sqlCount.append(" and exists(select * from mth_match_sponsor where del_flag = 0 and type = 1 and match_id = mmi.id and sponsor_id = :sponsorId) ");
                params.put("sponsorId", query.getSponsorId());
            }
            if (ObjectUtil.isNotNull(query.getOrganizerId())) {
                sql.append(" and exists(select * from mth_match_sponsor where del_flag = 0 and type = 2 and match_id = mmi.id and sponsor_id = :organizerId) ");
                sqlCount.append(" and exists(select * from mth_match_sponsor where del_flag = 0 and type = 2 and match_id = mmi.id and sponsor_id = :organizerId) ");
                params.put("organizerId", query.getOrganizerId());
            }
            if (ObjectUtil.isNotNull(query.getCreateOrgId())) {
                sql.append(" and mmi.create_org_id = :createOrgId");
                sqlCount.append(" and mmi.create_org_id = :createOrgId");
                params.put("createOrgId", query.getCreateOrgId());
            }
            if (StrUtil.isNotBlank(query.getCreateDeptName())) {
                sql.append(" and mmi.create_dept_name = :createDeptName");
                sqlCount.append(" and mmi.create_dept_name = :createDeptName");
                params.put("createDeptName", query.getCreateDeptName());
            }
        }
        sql.append(" order by mmi.update_time desc ");
        int total = baseNpJdbcTemplate.queryForInt(sqlCount.toString(), params);
        List<MthMatchInfoVo> matchInfos;
        String dyType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dyType, params, sql, bo);
            matchInfos = baseNpJdbcTemplate.queryForBeans(MthMatchInfoVo.class, sql.toString(), params);
        } else {
            matchInfos = new ArrayList<>();
        }
        PageRsp<MthMatchInfoVo> rsp = new PageRsp<>();
        rsp.setResult(matchInfos);
        rsp.setTotalNum(total);
        rsp.setPageNo(bo.getPageNo());
        rsp.setPageSize(bo.getPageSize());
        return rsp;
    }


    public PageRsp<MthMatchInfoAppVo> queryPageListForApp(PageReq<MthMatchInfoAppBo> req) {
        StringBuilder sql = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        if (ObjectUtil.isNotNull(req.getQuery().getId())) {
            whereSql.append(" and a.id = :id ");
            params.put("id", req.getQuery().getId());
        }
        if (StrUtil.isNotBlank(req.getQuery().getMatchName())) {
            whereSql.append(" and a.match_name like concat('%', :matchName ,'%') ");
            params.put("matchName", req.getQuery().getMatchName());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getStatusList())) {
            whereSql.append(" and a.status in (:statusList)");
            params.put("statusList", req.getQuery().getStatusList());
        }
        if (ObjectUtil.isNotNull(req.getQuery().getCategoryId())) {
            whereSql.append(" and a.category_id = :categoryId");
            params.put("categoryId", req.getQuery().getCategoryId());
        }
        if (ObjectUtil.isNotNull(req.getQuery().getQcFlag())) {
            whereSql.append(" and a.qc_flag = :qcFlag");
            params.put("qcFlag", req.getQuery().getQcFlag());
        }
        countSql.append(whereSql);
        sql.append(whereSql);
        sql.append(" order by a.publish_time desc");
        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<MthMatchInfoAppVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(MthMatchInfoAppVo.class, sql.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthMatchInfoAppVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }


    public PageRsp<MthMyMatchListAppVo> myMatchListForApp(PageReq<MthMyMatchListAppBo> req) {
        StringBuilder sql = new StringBuilder("select mr.id register_id, mr.review_result, mmi.* from mth_register mr left join mth_match_info mmi on mr.match_id = mmi.id where mr.del_flag = 0 and mmi.del_flag = 0");
        StringBuilder countSql = new StringBuilder("select count(*) from mth_register mr left join mth_match_info mmi on mr.match_id = mmi.id where mr.del_flag = 0 and mmi.del_flag = 0");
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and mr.user_id = :userId ");
        params.put("userId", req.getQuery().getUserId());
        if (ObjectUtil.isNotEmpty(req.getQuery().getStatusList())) {
            whereSql.append(" and mmi.status in (:statusList)");
            params.put("statusList", req.getQuery().getStatusList());
        }
        if(req.getQuery().isQcFlag()){
            whereSql.append(" and mmi.qc_flag =1 ");
        }

        countSql.append(whereSql);
        sql.append(whereSql);
        sql.append(" order by mr.create_time desc");
        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<MthMyMatchListAppVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(MthMyMatchListAppVo.class, sql.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthMyMatchListAppVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }


    public PageRsp<mthMatchManagerVo> queryMthMatchList(PageReq<MthMatchManagerReqBo> req) {
        StringBuilder sql = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        if (StrUtil.isNotBlank(req.getQuery().getMthMatchName())) {
            whereSql.append(" and a.match_name like concat('%', :matchName ,'%') ");
            params.put("matchName", req.getQuery().getMthMatchName());
        }
//        if (Objects.nonNull(req.getQuery().getId())) {
//            whereSql.append(" and a.id = :id ");
//            params.put("id", req.getQuery().getId());
//        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getStatusList())) {
            whereSql.append(" and a.status in (:statusList)");
            params.put("statusList", req.getQuery().getStatusList());
        }
        countSql.append(whereSql);
        sql.append(whereSql);
        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<mthMatchManagerVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(mthMatchManagerVo.class, sql.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<mthMatchManagerVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<MthMatchInfoVo> queryList(MthMatchInfoBo req) {
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        if (StrUtil.isNotBlank(req.getMatchName())) {
            whereSql.append(" and a.match_name like concat('%', :matchName ,'%') ");
            params.put("matchName", req.getMatchName());
        }
        if (Objects.nonNull(req.getId())) {
            whereSql.append(" and a.id = :id ");
            params.put("id", req.getId());
        }
        sb.append(whereSql);
        List<MthMatchInfoVo> result = baseNpJdbcTemplate.queryForBeans(MthMatchInfoVo.class, sb.toString(), params);
        return result;
    }


    public void deleteByIds(Collection<Long> ids, Integer updateId, String updateBy) {
        Map<String, Object> params = new HashMap<>(4);
        params.put("ids", ids);
        params.put("updateId", updateId);
        params.put("updateBy", updateBy);
        params.put("updateTime", new Date());
        baseNpJdbcTemplate.update(DELETE_SQL, params);
    }

    public List<MthMatchInfo> getMatchInfoByIds(Collection<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptyList();
        } else {
            Map<String, Object> map = new HashMap<>();
            map.put("ids", ids);
            String sql = "SELECT * FROM mth_match_info WHERE id in(:ids)";
            return baseNpJdbcTemplate.queryForBeans(MthMatchInfo.class, sql, map);
        }
    }

    public Integer validateCategoryExists(Long categoryId) {
        Map<String, Object> map = new HashMap<>();
        map.put("categoryId", categoryId);
        String sql = "SELECT count(*) FROM mth_match_info WHERE del_flag = 0 and category_id = :categoryId";
        return baseNpJdbcTemplate.queryForInt(sql, map);
    }

    public Integer validateLabelExists(Long labelId) {
        Map<String, Object> map = new HashMap<>();
        map.put("labelId", labelId);
        String sql = "SELECT count(*) FROM mth_match_label mml inner join mth_match_info mmi on mml.match_id = mmi.id WHERE mmi.del_flag = 0 and mml.label_id = :labelId";
        return baseNpJdbcTemplate.queryForInt(sql, map);
    }

    public Integer validatePrizeExists(Long prizeId) {
        Map<String, Object> map = new HashMap<>();
        map.put("prizeId", String.valueOf(prizeId));
        String sql = "SELECT count(*) FROM mth_round_prize mrp inner join mth_round_instance mri on mrp.round_id = mri.id inner join mth_match_info mmi on mri.match_id = mmi.id WHERE mmi.del_flag = 0 and mri.del_flag = 0 and mrp.prize_code = :prizeId";
        return baseNpJdbcTemplate.queryForInt(sql, map);

    }


    public Integer updateStatus(List<Long> ids, String status) {
        if (CollectionUtils.isEmpty(ids) || ObjectUtil.isNull(status)) {
            throw new BusinessException("系统内部错误");
        } else {
            Map<String, Object> map = new HashMap<>();
            map.put("ids", ids);
            map.put("status", status);
            int update = baseNpJdbcTemplate.update(EDIT_STATUS_SQL, map);
            return update;
        }
    }

    public Integer publishMatch(List<Long> ids, String status, Worker currWorker, OrgInfoBo orgInfoBo) {
        String sql = "update mth_match_info set status =:status,publish_org_id = :orgId,publish_org_name=:orgName," +
                "publisher_id = :publishId,publisher_name = :publishName,publish_time = :publishTime where del_flag = 0 and id in(:ids)";

        if (CollectionUtils.isEmpty(ids) || ObjectUtil.isNull(status)) {
            throw new BusinessException("系统内部错误");
        } else {
            Map<String, Object> map = new HashMap<>();
            map.put("ids", ids);
            map.put("status", status);
            map.put("orgId", ObjectUtil.isEmpty(orgInfoBo.getOrgId())?null:orgInfoBo.getOrgId());
            map.put("orgName", StrUtil.isBlank(orgInfoBo.getOrgName()) ? null : orgInfoBo.getOrgName());
            map.put("publishId", currWorker.getLoginId());
            map.put("publishName", currWorker.getWorkerName());
            map.put("publishTime", new Date());
            int update = baseNpJdbcTemplate.update(sql, map);
            return update;
        }
    }
    public List<MthSignRspVo> getWhetherSign(Long id){
        String sql="select msi.instance_id as registerId," +
                " msi.*," +
                " (case when msi.sign_end_time is not null and msi.sign_start_time is not null and msi.sign_start_time<=now() and msi.sign_end_time>=now() and information_type=1 then true else false end) whetherSignIn," +
                " (case when msi.sign_start_time is not null and msi.sign_end_time is not null and msi.sign_start_time<=now() and msi.sign_end_time>=now() and information_type=-1 then true else false end) whetherSignOut" +
                " from mth_match_sign_information msi" +
                " where msi.del_flag=0" +
                " and msi.instance_id =:id";
        Map<String, Object> params = new HashMap<>();
        params.put("id",id);
       return baseNpJdbcTemplate.queryForBeans(MthSignRspVo.class, sql, params);
    }

    public List<MatchBannerVo> queryBanner(){
        String sql ="SELECT mmi.id AS matchId, mmi.banner_id AS fileId\n" +
                "FROM mth_match_info mmi LEFT JOIN \n" +
                "(SELECT 'REGISTER' AS status, 1 AS sortNum\n" +
                "UNION ALL SELECT 'RELEASED' AS status, 2 AS sortNum\n" +
                "UNION ALL SELECT 'GOING' AS status, 3 AS sortNum\n" +
                "UNION ALL SELECT 'FINISHED' AS status, 4 AS sortNum) s ON mmi.status = s.status\n" +
                "WHERE mmi.status in ('REGISTER','RELEASED','GOING','FINISHED') and mmi.banner_id IS NOT NULL AND mmi.del_flag = 0\n" +
                "ORDER BY s.sortNum ASC, mmi.publish_time DESC LIMIT 0,5";
        return baseNpJdbcTemplate.queryForBeans(MatchBannerVo.class,sql,null);
    }

    /**
     * 	团重庆市委以及下级组织可以创建市级比赛
     * @param workerId
     * @return
     */
    public boolean queryCityLevelFlag(Long workerId) {
        String sql = "select count(*) from (select * from tsm_org where org_code = 'QCCQ3537136584' or parent_org_code = 'QCCQ3537136584') org " +
                "inner join tsm_worker_org_all twoa on org.org_id=twoa.org_id where twoa.worker_id = :workerId ";
        Map<String, Object> params = new HashMap<>();
        params.put("workerId", workerId);
        int count = baseNpJdbcTemplate.queryForInt(sql, params);
        return count > 0 ? true : false;
    }
}
