package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqArticleDetail;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.enums.MatchInfoStatusEnums;
import com.crunii.micro.service.match.enums.RoundEnums;
import org.assertj.core.util.Lists;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author luox
 * @date 2023-09-21
 */
@Repository
public class MthMatchManageDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    private static  MthMatchInfoDao mthMatchInfoDao ;
    private static final String PAGE_BASE_SQL = "select * from mth_match_info a where del_flag = 0 ";
    private static final String PAGE_COUNT_SQL = "select count(*) from mth_match_info a where del_flag = 0 ";
    private static final String DELETE_SQL = "update mth_match_info set del_flag=1, update_id=?, update_by=?, update_time=? where id in ";

    private static final String STATISTIC_COUNT_SQL="SELECT\n" +
            "\tcount(*)\n" +
            "FROM\n" +
            "\t(\n" +
            "\tSELECT\n" +
            "\t\ts.*,\n" +
            "\tCASE\n" +
            "\t\t\tWHEN s.doing_num = 0  THEN\n" +
            "\t\t\t1 \n" +
            "\t\t\tWHEN s.total_last > 0 and s.total_round-s.total_last>=0 THEN\n" +
            "\t\t\t2 \n" +
            "\t\t\tWHEN s.total_round > 0 AND s.total_last = 0 THEN\n" +
            "\t\t\t3 \n" +
            "\t\t\tEND  AS match_status \n" +
            "\t\tFROM\n" +
            "\t\t\t(\n" +
            "\t\t\tSELECT\n" +
            "\t\t\t\tt.match_id,\n" +
            "\t\t\t\tt.match_name,\n" +
            "\t\t\t\tt.registeration_start,\n" +
            "\t\t\t\tt.registeration_end,\n" +
            "\t\t\t\tcount( round_id) AS total_round,\n" +
            "\t\t\t\tcount( CASE WHEN t.match_state is not null THEN 1 END ) AS doing_num,\n" +
            "\t\t\t\tcount( CASE WHEN t.match_state is null THEN 1 END ) AS total_enroll,\n" +
            "\t\t\t\tcount( CASE WHEN t.match_state = '4' THEN 1 END ) AS total_end,\n" +
            "\t\t\t\tcount( CASE WHEN t.match_state is not null and t.match_state != '4' THEN 1 END ) AS total_last \n" +
            "\t\t\tFROM\n" +
            "\t\t\t\t(\n" +
            "\t\t\t\tSELECT\n" +
            "\t\t\t\t\tmmi.id AS match_id,\n" +
            "\t\t\t\t\tmmi.match_name AS match_name,\n" +
            "\t\t\t\t\tmmi.STATUS AS STATUS,\n" +
            "\t\t\t\t\tmmi.registeration_start AS registeration_start,\n" +
            "\t\t\t\t\tmmi.registeration_end AS registeration_end,\n" +
            "\t\t\t\t\tmmi.create_by AS create_by,\n" +
            "\t\t\t\t\tmmi.create_time AS create_time,\n" +
            "\t\t\t\t\tmmi.update_by AS update_by,\n" +
            "\t\t\t\t\tmmi.update_time AS update_time,\n" +
            "\t\t\t\t\tmmi.del_flag AS del_flag,\n" +
            "\t\t\t\t\tmri.id AS round_id,\n" +
            "\t\t\t\t\tmri.round_name AS round_name,\n" +
            "\t\t\t\t\tmri.round_state AS round_state,\n" +
            "\t\t\t\t\tmri.match_state AS match_state \n" +
            "\t\t\t\tFROM\n" +
            "\t\t\t\t\tmth_match_info mmi left join mth_match_contractor mmc on mmi.id=mmc.match_id" +
            "\t\t\t\t\tLEFT JOIN mth_round_instance mri ON mri.match_id = mmi.id and mri.area_id =mmc.area_id \n" +
            "\t\t\t\t\tWHERE mmc.contractor_id=:userId AND mri.del_flag = 0  and mri.round_type ='instance'\n" +
            "\t\t\t\t) t \n" +
            "\t\t\tWHERE\n" +
            "\t\t\t\tt.del_flag = 0 \n" +
            "\t\t\tGROUP BY\n" +
            "\t\t\t\tt.match_id,\n" +
            "\t\t\t\tt.match_name,\n" +
            "\t\t\t\tt.registeration_start,\n" +
            "\t\t\t\tt.registeration_end \n" +
            "\t\t\t) s \n" +
            "\t\t) st \n" +
            "\tWHERE\n" +
            "\t1 = 1 ";


    private static final String BASE_STATISTIC_SQL="SELECT\n" +
            "\tst.* \n" +
            "FROM\n" +
            "\t(\n" +
            "\tSELECT\n" +
            "\t\ts.*,\n" +
            "\t\ts.total_round - s.total_end AS total_last,\n" +
            "\tCASE\n" +
            "\t\t\t\n" +
            "\t\t\tWHEN s.doing_num = 0 THEN\n" +
            "\t\t\t1 \n" +
            "\t\t\tWHEN s.last_num > 0 \n" +
            "\t\t\tAND s.total_round - s.last_num >= 0 THEN\n" +
            "\t\t\t\t2 \n" +
            "\t\t\t\tWHEN s.total_round > 0 \n" +
            "\t\t\t\tAND last_num = 0 THEN\n" +
            "\t\t\t\t\t3 \n" +
            "\t\t\t\t\tEND AS match_status \n" +
            "\t\t\tFROM\n" +
            "\t\t\t\t(\n" +
            "\t\t\t\tSELECT\n" +
            "\t\t\t\t\tt.match_id,\n" +
            "\t\t\t\t\tt.match_name,\n" +
            "\t\t\t\t\tt.registeration_start,\n" +
            "\t\t\t\t\tt.registeration_end,\n" +
            "\t\t\t\t\tt.publish_time,\n" +
            "\t\t\t\t\tcount( round_id ) AS total_round,\n" +
            "\t\t\t\t\tcount( CASE WHEN t.match_state IS NOT NULL THEN 1 END ) AS doing_num,\n" +
            "\t\t\t\t\tcount( CASE WHEN t.match_state IS NULL THEN 1 END ) AS total_enroll,\n" +
            "\t\t\t\t\tcount( CASE WHEN t.match_state = '4' THEN 1 END ) AS total_end,\n" +
            "\t\t\t\t\tcount( CASE WHEN t.match_state IS NOT NULL AND t.match_state != '4' THEN 1 END ) AS last_num \n" +
            "\t\t\t\tFROM\n" +
            "\t\t\t\t\t(\n" +
            "\t\t\t\t\tSELECT\n" +
            "\t\t\t\t\t\tmmi.id AS match_id,\n" +
            "\t\t\t\t\t\tmmi.match_name AS match_name,\n" +
            "\t\t\t\t\t\tmmi.STATUS AS STATUS,\n" +
            "\t\t\t\t\t\tmmi.registeration_start AS registeration_start,\n" +
            "\t\t\t\t\t\tmmi.registeration_end AS registeration_end,\n" +
            "\t\t\t\t\t\tmmi.create_by AS create_by,\n" +
            "\t\t\t\t\t\tmmi.create_time AS create_time,\n" +
            "\t\t\t\t\t\tmmi.update_by AS update_by,\n" +
            "\t\t\t\t\t\tmmi.update_time AS update_time,\n" +
            "\t\t\t\t\t\tmmi.del_flag AS del_flag,\n" +
            "\t\t\t\t\t\tmri.id AS round_id,\n" +
            "\t\t\t\t\t\tmri.round_name AS round_name,\n" +
            "\t\t\t\t\t\tmmi.publish_time AS publish_time,\n" +
            "\t\t\t\t\t\tmri.round_state AS round_state,\n" +
            "\t\t\t\t\t\tmri.match_state AS match_state \n" +
            "\t\t\t\t\tFROM\n" +
            "\t\t\t\t\t\tmth_match_info mmi left join mth_match_contractor mmc on mmi.id=mmc.match_id" +
            "\t\t\t\t\t\tLEFT JOIN mth_round_instance mri ON mri.match_id = mmi.id and mri.area_id =mmc.area_id \n" +
            "\t\t\t\t\t\t where  mri.del_flag = 0  and mmc.del_flag=0 and mmc.contractor_id=:userId " +
            "\t\t\t\t\t\tAND mri.round_type = 'instance' \n" +
            "\t\t\t\t\t) t \n" +
            "\t\t\t\tWHERE\n" +
            "\t\t\t\t\tt.del_flag = 0 \n" +
            "\t\t\t\tGROUP BY\n" +
            "\t\t\t\t\tt.match_id,\n" +
            "\t\t\t\t\tt.match_name,\n" +
            "\t\t\t\t\tt.registeration_start,\n" +
            "\t\t\t\t\tt.registeration_end,\n" +
            "\t\t\t\t\tt.publish_time \n" +
            "\t\t\t\t) s \n" +
            "\t\t\t) st \n" +
            "\tWHERE\n" +
            "\t1 = 1";

    private static  final String DETAIL_STATISTIC_SQL ="SELECT\n" +
            "\tst.* \n" +
            "FROM\n" +
            "\t(\n" +
            "\tSELECT\n" +
            "\t\ts.match_id,\n" +
            "\t\ts.match_name,\n" +
            "\t\ts.round_id,\n" +
            "\t\ts.ROUND_NAME,\n" +
            "\t\ts.round_session,\n" +
            "\t\ts.LEVEL,\n" +
            "\t\ts.match_state,\n" +
            "\t\ts.round_state,\n" +
            "\t\ts.play_mode AS play_mode,\n" +
            "\t\ts.audit_mode AS audit_mode,\n" +
            "\t\ts.registeration_audit_flag,s.publicity_deadline,s.receive_address,s.receive_deadline,s.receive_type,s.prize_type,\n" +
            "\t\tcount( register_id ) AS enroll_num,\n" +
            "\t\tcount( CASE WHEN s.review_result = 'PENDING' THEN 1 END ) AS total_reviewer,\n" +
            "\t\tcount( s.work_id ) AS works_num,\n" +
            "\t\tcount( CASE WHEN s.workReviewStatus = 1 THEN 1 END ) AS work_Review_num,\n" +
            "\t\tcount( CASE WHEN s.awardsStatus = 1 THEN 1 END ) AS Awards_num \n" +
            "\tFROM\n" +
            "\t\t(\n" +
            "\t\tSELECT distinct \n" +
            "\t\t\tmri.match_id AS match_id,\n" +
            "\t\t\tmmi.match_name AS match_name,\n" +
            "\t\t\tmri.id AS round_id,\n" +
            "\t\t\tmri.round_name AS round_name,\n" +
            "\t\t\tmri.match_state AS match_state,\n" +
            "\t\t\tmri.round_state AS round_state,mri.publicity_deadline as publicity_deadline,mri.receive_address,mri.receive_deadline,mri.receive_type,mri.prize_type,\n" +
            "\t\t\tmri.play_mode AS play_mode,\n" +
            "\t\t\tmri.audit_mode AS audit_mode,\n" +
            "\t\t\tmmi.registeration_audit_flag AS registeration_audit_flag,\n" +
            "\t\t\tconcat( mri.round_level, '-', mri.round_sort ) AS round_session,\n" +
            "\t\t\tconcat(\n" +
            "\t\t\tCASE\n" +
            "\t\t\t\t\tmri.match_level \n" +
            "\t\t\t\t\tWHEN 'city' THEN\n" +
            "\t\t\t\t\t'市区级' \n" +
            "\t\t\t\t\tWHEN 'district' THEN\n" +
            "\t\t\t\t\t'片区级' \n" +
            "\t\t\t\t\tWHEN 'county' THEN\n" +
            "\t\t\t\t\t'区县级' \n" +
            "\t\t\t\t\tWHEN 'school' THEN\n" +
            "\t\t\t\t\t'校区级' ELSE '未知' \n" +
            "\t\t\t\tEND,\n" +
            "\t\t\t\t' ',\n" +
            "\t\t\tCASE\n" +
            "\t\t\t\t\tmri.round_level \n" +
            "\t\t\t\t\tWHEN 1 THEN\n" +
            "\t\t\t\t\t'初赛' \n" +
            "\t\t\t\t\tWHEN 2 THEN\n" +
            "\t\t\t\t\t'复赛' \n" +
            "\t\t\t\t\tWHEN 3 THEN\n" +
            "\t\t\t\t\t'决赛' ELSE '未知' \n" +
            "\t\t\t\tEND \n" +
            "\t\t\t\t) AS LEVEL,\n" +
            "\t\t\t\tconcat(\n" +
            "\t\t\t\t\tto_char ( mri.registeration_start, 'YYYY-MM-DD' ),\n" +
            "\t\t\t\t\t'至',\n" +
            "\t\t\t\tto_char ( mri.registeration_end, 'YYYY-MM-DD' )) AS registerationTime,\n" +
            "\t\t\t\tconcat(\n" +
            "\t\t\t\t\tto_char ( mri.round_start, 'YYYY-MM-DD' ),\n" +
            "\t\t\t\t\t'至',\n" +
            "\t\t\t\tto_char ( mri.round_end, 'YYYY-MM-DD' )) AS roundTime,\n" +
            "\t\t\t\tmrl.register_id AS register_id,\n" +
            "\t\t\t\tmrl.work_id AS work_id,\n" +
            "\t\t\t\tmr.review_result  as review_result,\n" +
            "\t\t\t\tCOALESCE ( mrs.STATUS, 0 ) AS workReviewStatus,\n" +
            "\t\t\t\t\t\t\t\tcase when (mrs.prize_id is not null and mrs.prize_id !='') or (mrs.other_prize_id  is not null and mrs.other_prize_id!='') then 1 ELSE 0  END as awardsStatus  \n" +
            "\t\t\tFROM\n" +
            "\t\t\t\tmth_round_instance mri\n" +
            "\t\t\t\tLEFT JOIN mth_match_info mmi ON mmi.id = mri.match_id     \n" +
            "\t\t\t\tLEFT JOIN mth_round_list mrl ON mrl.round_id = mri.id\n" +
            "\t\t\t\tLEFT JOIN mth_round_score mrs ON mrs.round_list_id = mrl.id \n" +
            "\t\t\t\tleft join mth_register mr on mr.id =mrl.register_id AND mr.review_result='APPROVED'" +
            " left join mth_match_contractor mmc on mmc.area_id = mri.area_id\n " +
            "\t\t\tWHERE\n" +
            "\t\t\t\tmri.del_flag = 0 \n" +
            "\t\t\t\tAND mri.round_type = 'instance' and mmc.contractor_id= :userId \n" +
            "\t\t\t) s \n" +
            "\t\tGROUP BY\n" +
            "\t\t\ts.match_id,\n" +
            "\t\t\ts.match_name,\n" +
            "\t\t\ts.round_id,\n" +
            "\t\t\ts.ROUND_NAME,\n" +
            "\t\t\ts.round_session,\n" +
            "\t\t\ts.LEVEL,\n" +
            "\t\t\ts.match_state,\n" +
            "\t\t\ts.round_state,\n" +
            "\t\t\ts.play_mode,\n" +
            "\t\t\ts.audit_mode,\n" +
            "\t\t\ts.registeration_audit_flag,s.publicity_deadline,s.receive_address,s.receive_deadline,s.receive_type,s.prize_type \n" +
            "\t\t) st \n" +
            "\tWHERE\n" +
            "\t\t1 = 1 \n" +
            "\t\tAND match_id =:matchId ";

    private static  final String QUERY_MATCHREVIE_COUNT_SQL="select count(*)\n" +
            "from mth_register mr\n" +
            "inner join mth_round_list mrl on mrl.register_id = mr.id\n" +
            "inner join mth_round_instance mri on mri.id = mrl.round_id\n" +
            "inner join mth_match_info mmi on mmi.id = mr.match_id\n" +
            "inner join mth_round_score mrs on mrs.round_list_id =mrl.id\n" +
            "inner join mth_round_qualify_log mrql on mrql.score_id =mrs.id\n" +
            "where 1=1 ";

    public PageRsp<mthMatchManagerVo> queryStatisticPageList(PageReq<MthMatchManagerReqBo> req) {
        StringBuilder sql = new StringBuilder(BASE_STATISTIC_SQL);
        StringBuilder countSql = new StringBuilder(STATISTIC_COUNT_SQL);
        Worker worker = CurrWorkerUtil.getCurrWorker();
//        List<Object> params = new ArrayList<>();
        Map<String,Object> paramMaps = new HashMap<>();
        paramMaps.put("userId",worker.getLoginId());
        StringBuilder whereSql = new StringBuilder();
        if(StrUtil.isNotEmpty(req.getQuery().getMthMatchName())){
            sql.append(" and st.match_name like concat('%', :matchName ,'%') ");
            countSql.append(" and st.match_name like concat('%', :matchName ,'%')");
            paramMaps.put("matchName",req.getQuery().getMthMatchName());
//            params.add("%"+req.getQuery().getMthMatchName()+"%");
        }
        if(ObjectUtil.isNotEmpty(req.getQuery().getState())){
            if (0!=req.getQuery().getState()){
                sql.append(" and st.match_status = :matchStatus");
                countSql.append(" and st.match_status = :matchStatus");
//                params.add(req.getQuery().getState());
                paramMaps.put("matchStatus",req.getQuery().getState());
            }

        }
        sql.append(" order by st.publish_time desc ");
        int total = baseNpJdbcTemplate.queryForInt(countSql.append(whereSql).toString(), paramMaps);
        List<mthMatchManagerVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sql.append(whereSql);
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, paramMaps, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(mthMatchManagerVo.class, sql.toString(), paramMaps);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<mthMatchManagerVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public List<mthMatchManagerDetailVo> queryMatchManagerDetailList(MthMatchManagerReqBo req) {
        StringBuilder sql = new StringBuilder(DETAIL_STATISTIC_SQL);
        Map<String, Object> map = new HashMap<>(16);
        map.put("matchId", req.getMthMatchId());

            map.put("userId",CurrWorkerUtil.getCurrWorker().getLoginId());
//            map.put("userId",10069);
        if(ObjectUtil.isNotEmpty(req.getMatchRoundState())){
            if (req.getMatchRoundState()!=null&&
                    req.getMatchRoundState().size()>0){
                sql.append(" and st.match_state in (:matchStatus) ");
                map.put("matchStatus", req.getMatchRoundState());
            }
        }
        if(ObjectUtil.isNotEmpty(req.getMthRoundName())){
            sql.append(" and st.round_name like concat('%', :roundName ,'%') ");
            map.put("roundName", req.getMthRoundName());
        }
        sql.append(" ORDER BY round_session ");
        List<mthMatchManagerDetailVo> result;
        result = baseNpJdbcTemplate.queryForBeans(mthMatchManagerDetailVo.class, sql.toString(), map);

        result.stream().forEach(item->{
            //如果状态是公示中 返回最后一条公示的关联id
            if (ObjectUtil.isNotNull(item.getMatchState())&&item.getMatchState().equals("25")){
                String publicitySql="select article_id from ycq_article_detail where round_id=:roundId and article_type=4 order by create_time desc LIMIT 1";
                Map<String, Object> publicityMap = new HashMap<>(16);
                publicityMap.put("roundId",item.getRoundId());
                Long articleId = baseNpJdbcTemplate.queryForObject(publicitySql,publicityMap,Long.class);
                item.setArticleId(articleId);
            }
            //如果状态是报名中或待评审 查询待审核人数
            if (ObjectUtil.isNotNull(item.getMatchState())&&(
                    item.getMatchState().equals("0")||item.getMatchState().equals("1")
                    )){
                Map<String, Object> pengdingMap = new HashMap<>(16);
                pengdingMap.put("roundId",item.getRoundId());
                String pendingSql=" select count(r.id) from mth_round_list l left join mth_register r on l.register_id=r.id " +
                        " where l.round_id=:roundId and r.review_result='PENDING'";
                Integer pendingCount = baseNpJdbcTemplate.queryForInt(pendingSql,pengdingMap);
                item.setTotalReviewer(Long.valueOf(pendingCount.toString()));
            }
        });



        return result;
    }

    public List<mthMatchManagerDetailVo> queryList(MthMatchManagerReqBo req) {
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        List<Object> params = new ArrayList<>();
        List<mthMatchManagerDetailVo> result = jt.queryForBeans(mthMatchManagerDetailVo.class, sb.toString(), params.toArray());
        return result;
    }


    public void deleteByIds(Collection<Long> ids, Integer updateId, String updateBy) {
        String inIds = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ") "));
        StringBuilder sb = new StringBuilder(DELETE_SQL).append(inIds);
        jt.update(sb.toString(), updateId, updateBy, new Date());
    }


    public PageRsp<MthMatchManagerRegisterVo> queryMatchRegisterList(PageReq<MthMatchManagerRegisterReqBo> req, boolean exportFlag) {
        String sql = "SELECT\n" +
                "\tmr.CODE AS register_code,\n" +
                "\tmr.id AS register_id,\n" +
                "\tmrl.id AS round_list_id,\n" +
                "\tmr.user_id AS user_id,\n" +
                "\tmr.match_id AS match_id,\n" +
                "\tmmi.match_name AS match_name,\n" +
                "\tmrl.round_id AS round_id,\n" +
                "\tmri.round_name AS round_name,\n" +
                "\tmr.item_name AS item_name,\n" +
                "\tmr.group_name as group_name, \n" +
                "\tmr.user_name AS user_name,\n" +
                "\tmr.area_id AS area_id,\n" +
                "\tmr.area_name AS area_name,\n" +
                "\tmr.telephone AS telephone,\n" +
                "\tmr.school_name AS school_name,\n" +
                "\tmr.company_name AS company_name,\n" +
                "\tmr.idcared AS id_cared,\n" +
                "\tmr.create_time AS register_time,\n" +
                "\tmr.review_result AS review_result,\n" +
                "\tmgr.review_time AS review_time\n" +
                "FROM\n" +
                "\tmth_register mr\n" +
                "\tINNER JOIN mth_round_list mrl ON mrl.register_id = mr.id\n" +
                "\tINNER JOIN mth_round_instance mri ON mri.id = mrl.round_id AND mri.round_type = 'instance'\n" +
                "\tINNER JOIN mth_match_info mmi ON mmi.id = mr.match_id   \n" +
                "\tLEFT JOIN (select register_id,max(create_time) review_time from mth_register_review group by register_id) mgr ON mgr.register_id = mr.id\n" +
                "WHERE\n" +
                "\t1 = 1";
        String count = "select count(*)\n" +
                "from mth_register mr\n" +
                "inner join mth_round_list mrl on mrl.register_id = mr.id\n" +
                "inner join mth_round_instance mri on mri.id = mrl.round_id\n" +
                "inner join mth_match_info mmi on mmi.id = mr.match_id     \n" +
                "where 1=1 ";
        StringBuilder sb = new StringBuilder(sql);
        StringBuilder countSql = new StringBuilder(count);

        Map<String, Object> params = new HashMap<>(16);
        List<Object> countParams = new ArrayList<>();

        StringBuilder whereSql = new StringBuilder();
        if (ObjectUtil.isNotEmpty(req.getQuery().getMthMatchId())){
            whereSql.append(" and mr.match_id = ? ");
            countSql.append(" and mr.match_id = ? ");
            countParams.add(req.getQuery().getMthMatchId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getMthRoundId())){
            whereSql.append(" and mrl.round_id = ? ");
            countSql.append(" and mrl.round_id = ? ");
            countParams.add(req.getQuery().getMthRoundId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getRegisterCode())){
            whereSql.append(" and mr.code = ? ");
            countSql.append(" and mr.code = ? ");
            countParams.add(req.getQuery().getRegisterCode());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getUserName())){
            whereSql.append(" and mr.user_name like ? ");
            countSql.append(" and mr.user_name like ? ");
            countParams.add("%"+req.getQuery().getUserName()+"%");
        }
        /**参赛者学校*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getSchoolName())){
            whereSql.append(" and  mr.school_name like ? ");
            countSql.append(" and  mr.school_name like ? ");
            countParams.add("%"+req.getQuery().getSchoolName()+"%");
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getTelephone())){
            whereSql.append(" and mr.telephone = ? ");
            countSql.append(" and mr.telephone = ? ");
            countParams.add(req.getQuery().getTelephone());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getAreaName())){
            whereSql.append(" and mr.area_name like ? ");
            countSql.append(" and mr.area_name like ? ");
            countParams.add("%"+req.getQuery().getAreaName()+"%");
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewResult())){
            whereSql.append(" and mr.review_result = ? ");
            countSql.append(" and mr.review_result = ? ");
            countParams.add(req.getQuery().getReviewResult());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getItemId())){
            whereSql.append(" and mr.item_id = ? ");
            countSql.append(" and mr.item_id = ? ");
            countParams.add(req.getQuery().getItemId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getGroupId())){
            whereSql.append(" and mr.group_id = ? ");
            countSql.append(" and mr.group_id = ? ");
            countParams.add(req.getQuery().getGroupId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getRegisterStartTime())&&
                ObjectUtil.isNotEmpty(req.getQuery().getRegisterEndTime())){
            whereSql.append(" and mr.create_time >= ? and mr.create_time<= ? ");
            countSql.append(" and mr.create_time >= ? and mr.create_time<= ? ");
            countParams.add(req.getQuery().getRegisterStartTime());
            countParams.add(req.getQuery().getRegisterEndTime());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewStartTime())&&
                ObjectUtil.isNotEmpty(req.getQuery().getReviewEndTime())){
            whereSql.append(" and mr.update_time >= ? and mr.update_time<= ? ");
            countSql.append(" and mr.update_time >= ? and mr.update_time<= ? ");
            countParams.add(req.getQuery().getReviewStartTime());
            countParams.add(req.getQuery().getReviewEndTime());
        }
        sb.append(whereSql);

        int total = jt.queryForInt(countSql.toString(), countParams.toArray());
        List<MthMatchManagerRegisterVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit  导出不设置分页
            if (!exportFlag){
                SqlUtil.appendLimit(dbType, countParams, sb, req);
            }
            result = jt.queryForBeans(MthMatchManagerRegisterVo.class, sb.toString(), countParams.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthMatchManagerRegisterVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public PageRsp<MthMatchManagerReviewVo> queryMatchReviewList(PageReq<MthMatchManagerReviewReqBo> req, boolean exportFlag) {
        String sql = "SELECT\n" +
                "\tmr.id,\n" +
                "\tmrl.id AS round_list_id,\n" +
                "\tmr.id AS register_id,\n" +
                "\tmr.user_id AS user_id,\n" +
                "\tmr.match_id AS match_id,\n" +
                "\tmmi.match_name AS match_name,\n" +
                "\tmrl.round_id AS round_id,\n" +
                "\tmr.create_time AS register_time,\n" +
                "\tmr.review_result AS review_result,\n" +
                "\tmri.round_name AS round_name,\n" +
                "\tmr.user_name AS user_name,\n" +
                "\tmr.area_id AS area_id,\n" +
                "\tmr.area_name AS area_name,\n" +
                "\tmr.CODE AS register_code,\n" +
                "\tmr.telephone AS telephone,\n" +
                "\tmr.idcared AS id_cared,\n" +
                "\tmr.item_Name AS item_name,\n" +
                "\tmr.group_name as group_name,\n" +
                "\tmr.school_name AS school_name,\n" +
                "\tmr.company_name AS company_name,\n" +

                "\tmw.work_code AS work_code,\n" +
                "\tmw.work_name AS work_name,\n" +
                "\tmw.create_time AS workUploadTime,\n" +
                "\tif(mrs.STATUS==1,'已评审','待评审') AS review_status,\n" +
                "\tmrs.score AS score,\n" +
                "\tmrs.review_time AS review_time\n" +

                "\tFROM\n" +
                "\t\tmth_register mr\n" +
                "\t\tINNER JOIN mth_round_list mrl ON mrl.register_id = mr.id\n" +
                "\t\tINNER JOIN mth_round_instance mri ON mri.id = mrl.round_id AND mri.round_type = 'instance'\n" +
                "\t\tINNER JOIN mth_match_info mmi ON mmi.id = mr.match_id  \n" +
                "\t\tINNER JOIN mth_round_score mrs ON mrs.round_list_id = mrl.id\n" +
                "\t\tLEFT JOIN mth_work mw ON mw.id = mrl.work_id \n" +
                "WHERE\n" +
                "\t1 = 1";

        String couSql ="select count(*)\n" +
                "from mth_register mr\n" +
                "         inner join mth_round_list mrl on mrl.register_id = mr.id\n" +
                "         inner join mth_round_instance mri on mri.id = mrl.round_id and mri.round_type='instance'\n" +
                "         inner join mth_match_info mmi on mmi.id = mr.match_id\n" +
                "         inner join mth_round_score mrs on mrs.round_list_id =mrl.id\n" +
                "         left join mth_work mw on mw.id =mrl.work_id\n" +
                "where 1=1 ";
        StringBuilder sb = new StringBuilder(sql);
        StringBuilder countSql = new StringBuilder(couSql);

        Map<String, Object> params = new HashMap<>(16);
        List<Object> countParams = new ArrayList<>();

        StringBuilder whereSql = new StringBuilder();
        if (ObjectUtil.isNotEmpty(req.getQuery().getMthMatchId())){
            whereSql.append(" and mr.match_id = ? ");
            countSql.append(" and mr.match_id = ? ");
            countParams.add(req.getQuery().getMthMatchId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getMthRoundId())){
            whereSql.append(" and mrl.round_id = ? ");
            countSql.append(" and mrl.round_id = ? ");
            countParams.add(req.getQuery().getMthRoundId());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getAreaName())){
            whereSql.append(" and mr.area_name like ? ");
            countSql.append(" and mr.area_name like ? ");
            countParams.add("%"+req.getQuery().getAreaName()+"%");
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getWorkCode())){
            whereSql.append(" and  mw.work_code like ? ");
            countSql.append(" and  mw.work_code like ? ");
            countParams.add("%"+req.getQuery().getWorkCode()+"%");
        }
        /**参赛者编码*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getRegisterCode())){
            whereSql.append(" and  mr.code like ? ");
            countSql.append(" and  mr.code like ? ");
            countParams.add("%"+req.getQuery().getRegisterCode()+"%");
        }
        /**参赛者姓名*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getUserName())){
            whereSql.append(" and  mr.user_name like ? ");
            countSql.append(" and  mr.user_name like ? ");
            countParams.add("%"+req.getQuery().getUserName()+"%");
        }
        /**参赛者学校*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getSchoolName())){
            whereSql.append(" and  mr.school_name like ? ");
            countSql.append(" and  mr.school_name like ? ");
            countParams.add("%"+req.getQuery().getSchoolName()+"%");
        }
        /**参赛者手机号*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getTelephone())){
            whereSql.append(" and  mr.telephone = ? ");
            countSql.append(" and  mr.telephone = ? ");
            countParams.add(req.getQuery().getTelephone());
        }
        /**报名审核状态*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewResult())){
            whereSql.append(" and  mr.review_result = ? ");
            countSql.append(" and  mr.review_result = ? ");
            countParams.add(req.getQuery().getReviewResult());
        }
        /**得分评审状态*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewState())){
            whereSql.append(" and mrs.STATUS = ? ");
            countSql.append(" and mrs.STATUS = ? ");
            countParams.add(req.getQuery().getReviewState());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getItemId())){
            whereSql.append(" and mr.item_id = ? ");
            countSql.append(" and mr.item_id = ? ");
            countParams.add(req.getQuery().getItemId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getGroupId())){
            whereSql.append(" and mr.group_id = ? ");
            countSql.append(" and mr.group_id = ? ");
            countParams.add(req.getQuery().getGroupId());
        }
        //是否晋级
        if (ObjectUtil.isNotEmpty(req.getQuery().getPromotionFlag())){
            if (req.getQuery().getPromotionFlag().equals(1)){
                whereSql.append(" and mrs.result = 1 ");
                countSql.append(" and mrs.result = 1 ");
            }else {
                whereSql.append(" and (mrs.result is null or mrs.result <> 1)");
                countSql.append(" and (mrs.result is null or mrs.result <> 1)");
            }
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getPrizeFlag())){
            if (req.getQuery().getPrizeFlag().equals(1)){
                //获奖
                whereSql.append(" and (length(mrs.prize_id) > 0 or length(mrs.other_prize_id) > 0) ");
                countSql.append(" and (length(mrs.prize_id) > 0 or length(mrs.other_prize_id) > 0) ");

            }else {
                //没获奖
                whereSql.append(" and ((mrs.prize_id is null or mrs.prize_id = '') and (mrs.other_prize_id is null or mrs.other_prize_id = '')) ");
                countSql.append(" and ((mrs.prize_id is null or mrs.prize_id = '') and (mrs.other_prize_id is null or mrs.other_prize_id = '')) ");
            }
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getRegisterStartTime())&&
                ObjectUtil.isNotEmpty(req.getQuery().getRegisterEndTime())){
            whereSql.append(" and mr.create_time >= ? and mr.create_time<= ? ");
            countSql.append(" and mr.create_time >= ? and mr.create_time<= ? ");
            countParams.add(req.getQuery().getRegisterStartTime());
            countParams.add(req.getQuery().getRegisterEndTime());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewStartTime())&&
                ObjectUtil.isNotEmpty(req.getQuery().getReviewEndTime())){
            whereSql.append(" and mrs.review_time >= ? and mrs.review_time<= ? ");
            countSql.append(" and mrs.review_time >= ? and mrs.review_time<= ? ");
            countParams.add(req.getQuery().getReviewStartTime());
            countParams.add(req.getQuery().getReviewEndTime());
        }
        whereSql.append(" order by mrs.sort ");
        int total = jt.queryForInt(countSql.toString(), countParams.toArray());
        List<MthMatchManagerReviewVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(whereSql);
            //根据数据库类型设置limit
            if (!exportFlag){
                SqlUtil.appendLimit(dbType, countParams, sb, req);
            }
            result = jt.queryForBeans(MthMatchManagerReviewVo.class, sb.toString(), countParams.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthMatchManagerReviewVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }


    public PageRsp<MthMatchManagerPromotedVo> queryMatchPromotedList(PageReq<MthMatchManagerPromotedReqBo> req, boolean exportFlag) {
        String sql = "SELECT\n" +
                "\tmr.id,\n" +
                "\tmrl.id AS round_list_id,\n" +
                "\tmr.id AS register_id,\n" +
                "\tmr.user_id AS user_id,\n" +
                "\tmr.match_id AS match_id,\n" +
                "\tmmi.match_name AS match_name,\n" +
                "\tmrl.round_id AS round_id,\n" +
                "\tmri.round_name AS round_name,\n" +
                "\tmr.user_name AS user_name,\n" +
                "\tmr.area_id AS area_id,\n" +
                "\tmr.area_name AS area_name,\n" +
                "\tmr.CODE AS register_code,\n" +
                "\tmr.telephone AS telephone,\n" +
                "\tmr.idcared AS id_cared,\n" +
                "\tmr.item_Name AS item_name,\n" +
                "\tmr.group_name as group_name,\n" +
                "\tmr.school_name AS school_name,\n" +
                "\tmr.company_name AS company_name,\n" +

                "\tif(mms.id is null, '未签到', '已签到') AS sign_status,\n" +
                "\tmms.sign_time AS sign_time,\n" +
                "\tmw.work_code AS work_code,\n" +
                "\tmw.work_name AS work_name,\n" +
                "\tmrs.score AS score\n" +

                "\tFROM\n" +
                "\t\tmth_register mr\n" +
                "\t\tINNER JOIN mth_round_list mrl ON mrl.register_id = mr.id\n" +
                "\t\tINNER JOIN mth_round_instance mri ON mri.id = mrl.round_id AND mri.round_type = 'instance'\n" +
                "\t\tINNER JOIN mth_match_info mmi ON mmi.id = mr.match_id  \n" +
                "\t\tINNER JOIN mth_round_score mrs ON mrs.round_list_id = mrl.id\n" +
                "\t\tLEFT JOIN mth_work mw ON mw.id = mrl.work_id \n" +
                "\t\tLEFT JOIN mth_match_sign mms on mrl.id = mms.mth_round_list_id and mms.sign_handle = 1 " +
                "WHERE\n" +
                "\t1 = 1";

        String couSql ="select count(*)\n" +
                "from mth_register mr\n" +
                "         inner join mth_round_list mrl on mrl.register_id = mr.id\n" +
                "         inner join mth_round_instance mri on mri.id = mrl.round_id and mri.round_type='instance'\n" +
                "         inner join mth_match_info mmi on mmi.id = mr.match_id\n" +
                "         inner join mth_round_score mrs on mrs.round_list_id =mrl.id\n" +
                "         left join mth_work mw on mw.id =mrl.work_id\n" +
                "where 1=1 ";
        StringBuilder sb = new StringBuilder(sql);
        StringBuilder countSql = new StringBuilder(couSql);

        Map<String, Object> params = new HashMap<>(16);
        List<Object> countParams = new ArrayList<>();

        StringBuilder whereSql = new StringBuilder();
        if (ObjectUtil.isNotEmpty(req.getQuery().getMthMatchId())){
            whereSql.append(" and mr.match_id = ? ");
            countSql.append(" and mr.match_id = ? ");
            countParams.add(req.getQuery().getMthMatchId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getMthRoundId())){
            whereSql.append(" and mrl.round_id = ? ");
            countSql.append(" and mrl.round_id = ? ");
            countParams.add(req.getQuery().getMthRoundId());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getAreaName())){
            whereSql.append(" and mr.area_name like ? ");
            countSql.append(" and mr.area_name like ? ");
            countParams.add("%"+req.getQuery().getAreaName()+"%");
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getWorkCode())){
            whereSql.append(" and  mw.work_code like ? ");
            countSql.append(" and  mw.work_code like ? ");
            countParams.add("%"+req.getQuery().getWorkCode()+"%");
        }
        /**参赛者编码*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getRegisterCode())){
            whereSql.append(" and  mr.code like ? ");
            countSql.append(" and  mr.code like ? ");
            countParams.add("%"+req.getQuery().getRegisterCode()+"%");
        }
        /**参赛者姓名*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getUserName())){
            whereSql.append(" and  mr.user_name like ? ");
            countSql.append(" and  mr.user_name like ? ");
            countParams.add("%"+req.getQuery().getUserName()+"%");
        }
        /**参赛者学校*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getSchoolName())){
            whereSql.append(" and  mr.school_name like ? ");
            countSql.append(" and  mr.school_name like ? ");
            countParams.add("%"+req.getQuery().getSchoolName()+"%");
        }
        /**参赛者手机号*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getTelephone())){
            whereSql.append(" and  mr.telephone = ? ");
            countSql.append(" and  mr.telephone = ? ");
            countParams.add(req.getQuery().getTelephone());
        }
        /**报名审核状态*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewResult())){
            whereSql.append(" and  mr.review_result = ? ");
            countSql.append(" and  mr.review_result = ? ");
            countParams.add(req.getQuery().getReviewResult());
        }
        /**得分评审状态*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewState())){
            whereSql.append(" and mrs.STATUS = ? ");
            countSql.append(" and mrs.STATUS = ? ");
            countParams.add(req.getQuery().getReviewState());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getItemId())){
            whereSql.append(" and mr.item_id = ? ");
            countSql.append(" and mr.item_id = ? ");
            countParams.add(req.getQuery().getItemId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getGroupId())){
            whereSql.append(" and mr.group_id = ? ");
            countSql.append(" and mr.group_id = ? ");
            countParams.add(req.getQuery().getGroupId());
        }
        //是否晋级
        if (ObjectUtil.isNotEmpty(req.getQuery().getPromotionFlag())){
            if (req.getQuery().getPromotionFlag().equals(1)){
                whereSql.append(" and mrs.result = 1 ");
                countSql.append(" and mrs.result = 1 ");
            }else {
                whereSql.append(" and (mrs.result is null or mrs.result <> 1)");
                countSql.append(" and (mrs.result is null or mrs.result <> 1)");
            }

        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getPrizeFlag())){
            if (req.getQuery().getPrizeFlag().equals(1)){
                //获奖
                whereSql.append(" and (length(mrs.prize_id) > 0 or length(mrs.other_prize_id) > 0) ");
                countSql.append(" and (length(mrs.prize_id) > 0 or length(mrs.other_prize_id) > 0) ");

            }else {
                //没获奖
                whereSql.append(" and ((mrs.prize_id is null or mrs.prize_id = '') and (mrs.other_prize_id is null or mrs.other_prize_id = '')) ");
                countSql.append(" and ((mrs.prize_id is null or mrs.prize_id = '') and (mrs.other_prize_id is null or mrs.other_prize_id = '')) ");
            }
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getRegisterStartTime())&&
                ObjectUtil.isNotEmpty(req.getQuery().getRegisterEndTime())){
            whereSql.append(" and mr.create_time >= ? and mr.create_time<= ? ");
            countSql.append(" and mr.create_time >= ? and mr.create_time<= ? ");
            countParams.add(req.getQuery().getRegisterStartTime());
            countParams.add(req.getQuery().getRegisterEndTime());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewStartTime())&&
                ObjectUtil.isNotEmpty(req.getQuery().getReviewEndTime())){
            whereSql.append(" and mrs.review_time >= ? and mrs.review_time<= ? ");
            countSql.append(" and mrs.review_time >= ? and mrs.review_time<= ? ");
            countParams.add(req.getQuery().getReviewStartTime());
            countParams.add(req.getQuery().getReviewEndTime());
        }
        whereSql.append(" order by mrs.sort ");
        int total = jt.queryForInt(countSql.toString(), countParams.toArray());
        List<MthMatchManagerPromotedVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(whereSql);
            //根据数据库类型设置limit
            if (!exportFlag){
                SqlUtil.appendLimit(dbType, countParams, sb, req);
            }
            result = jt.queryForBeans(MthMatchManagerPromotedVo.class, sb.toString(), countParams.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthMatchManagerPromotedVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }


    public PageRsp<MthMatchManagerPublicityVo> queryMatchPublicityList(PageReq<MthMatchManagerPublicityReqBo> req, boolean exportFlag) {
        String sql = "SELECT\n" +
                "\tmr.id,\n" +
                "\tmrl.id AS round_list_id,\n" +
                "\tmr.id AS register_id,\n" +
                "\tmr.user_id AS user_id,\n" +
                "\tmr.match_id AS match_id,\n" +
                "\tmmi.match_name AS match_name,\n" +
                "\tmrl.round_id AS round_id,\n" +
                "\tmri.round_name AS round_name,\n" +
                "\tmr.user_name AS user_name,\n" +
                "\tmr.area_id AS area_id,\n" +
                "\tmr.area_name AS area_name,\n" +
                "\tmr.CODE AS register_code,\n" +
                "\tmr.telephone AS telephone,\n" +
                "\tmr.idcared AS id_cared,\n" +
                "\tmr.item_Name AS item_name,\n" +
                "\tmr.group_name as group_name,\n" +
                "\tmr.school_name AS school_name,\n" +
                "\tmr.company_name AS company_name,\n" +

                "\tmr.create_time AS register_time,\n" +
                "\tmr.review_result AS review_result,\n" +
                "\tmgr.review_time AS review_time,\n" +

                "\tif(mms.id is null, '未签到', '已签到') AS sign_status,\n" +
                "\tmms.sign_time AS sign_time,\n" +
                "\tif(mmsout.id is null, '未签退', '已签退') AS sign_out_status,\n" +
                "\tmmsout.sign_time AS sign_out_time,\n" +
                "\tmw.work_code AS work_code,\n" +
                "\tmw.work_name AS work_name,\n" +
                "\tmrs.score AS score,\n" +

                "CASE\n" +
                "\t\tWHEN nullif( mrs.prize_name, '' ) IS NOT NULL \n" +
                "\t\tAND nullif( mrs.other_prize_name, '' ) IS NOT NULL THEN\n" +
                "\t\t\tmrs.prize_name || '、' || mrs.other_prize_name ELSE COALESCE (\n" +
                "\t\t\t\tnullif( mrs.prize_name, '' ),\n" +
                "\t\t\tnullif( mrs.other_prize_name, '' )) \n" +
                "\t\tEND AS prize_name,\n" +
                "\tif(mrs.result = 1,'是','否') AS result\n" +

                "\tFROM\n" +
                "\t\tmth_register mr\n" +
                "\t\tINNER JOIN mth_round_list mrl ON mrl.register_id = mr.id\n" +
                "\t\tINNER JOIN mth_round_instance mri ON mri.id = mrl.round_id AND mri.round_type = 'instance'\n" +
                "\t\tINNER JOIN mth_match_info mmi ON mmi.id = mr.match_id  \n" +
                "\t\tINNER JOIN mth_round_score mrs ON mrs.round_list_id = mrl.id\n" +
                "\t\tLEFT JOIN mth_work mw ON mw.id = mrl.work_id \n" +
                "\t\tLEFT JOIN (select register_id,max(create_time) review_time from mth_register_review group by register_id) mgr ON mgr.register_id = mr.id\n" +
               /* "\t\tLEFT JOIN mth_match_sign mms on mrl.id = mms.mth_round_list_id and mms.sign_handle = 1 " +
                " LEFT JOIN mth_match_sign mmsout on mrl.id = mmsout.mth_round_list_id and mmsout.sign_handle = -1  " +*/
                //签到表 mth_round_list_id存成了场次id ，通过create_by 和场次id关联查询
                " LEFT JOIN mth_match_sign mms on mr.create_by= mms.create_by and   mms.sign_handle = 1 and mms.mth_round_list_id=mri.id\n" +
                " LEFT JOIN mth_match_sign mmsout on mr.create_by= mmsout.create_by and   mmsout.sign_handle = -1 and mms.mth_round_list_id=mri.id\n"+
                "WHERE\n" +
                "\t1 = 1";

        String couSql ="select count(*)\n" +
                "from mth_register mr\n" +
                "         inner join mth_round_list mrl on mrl.register_id = mr.id\n" +
                "         inner join mth_round_instance mri on mri.id = mrl.round_id and mri.round_type='instance'\n" +
                "         inner join mth_match_info mmi on mmi.id = mr.match_id\n" +
                "         inner join mth_round_score mrs on mrs.round_list_id =mrl.id\n" +
                "         left join mth_work mw on mw.id =mrl.work_id\n" +
                "where 1=1 ";
        StringBuilder sb = new StringBuilder(sql);
        StringBuilder countSql = new StringBuilder(couSql);

        Map<String, Object> params = new HashMap<>(16);
        List<Object> countParams = new ArrayList<>();

        StringBuilder whereSql = new StringBuilder();
        if (ObjectUtil.isNotEmpty(req.getQuery().getMthMatchId())){
            whereSql.append(" and mr.match_id = ? ");
            countSql.append(" and mr.match_id = ? ");
            countParams.add(req.getQuery().getMthMatchId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getMthRoundId())){
            whereSql.append(" and mrl.round_id = ? ");
            countSql.append(" and mrl.round_id = ? ");
            countParams.add(req.getQuery().getMthRoundId());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getAreaName())){
            whereSql.append(" and mr.area_name like ? ");
            countSql.append(" and mr.area_name like ? ");
            countParams.add("%"+req.getQuery().getAreaName()+"%");
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getWorkCode())){
            whereSql.append(" and  mw.work_code like ? ");
            countSql.append(" and  mw.work_code like ? ");
            countParams.add("%"+req.getQuery().getWorkCode()+"%");
        }
        /**参赛者编码*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getRegisterCode())){
            whereSql.append(" and  mr.code like ? ");
            countSql.append(" and  mr.code like ? ");
            countParams.add("%"+req.getQuery().getRegisterCode()+"%");
        }
        /**参赛者姓名*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getUserName())){
            whereSql.append(" and  mr.user_name like ? ");
            countSql.append(" and  mr.user_name like ? ");
            countParams.add("%"+req.getQuery().getUserName()+"%");
        }
        /**参赛者学校*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getSchoolName())){
            whereSql.append(" and  mr.school_name like ? ");
            countSql.append(" and  mr.school_name like ? ");
            countParams.add("%"+req.getQuery().getSchoolName()+"%");
        }
        /**参赛者手机号*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getTelephone())){
            whereSql.append(" and  mr.telephone = ? ");
            countSql.append(" and  mr.telephone = ? ");
            countParams.add(req.getQuery().getTelephone());
        }
        /**报名审核状态*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewResult())){
            whereSql.append(" and  mr.review_result = ? ");
            countSql.append(" and  mr.review_result = ? ");
            countParams.add(req.getQuery().getReviewResult());
        }
        /**得分评审状态*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewState())){
            whereSql.append(" and mrs.STATUS = ? ");
            countSql.append(" and mrs.STATUS = ? ");
            countParams.add(req.getQuery().getReviewState());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getItemId())){
            whereSql.append(" and mr.item_id = ? ");
            countSql.append(" and mr.item_id = ? ");
            countParams.add(req.getQuery().getItemId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getGroupId())){
            whereSql.append(" and mr.group_id = ? ");
            countSql.append(" and mr.group_id = ? ");
            countParams.add(req.getQuery().getGroupId());
        }
        //是否晋级
        if (ObjectUtil.isNotEmpty(req.getQuery().getPromotionFlag())){
            if (req.getQuery().getPromotionFlag().equals(1)){
                whereSql.append(" and mrs.result = 1 ");
                countSql.append(" and mrs.result = 1 ");
            }else {
                whereSql.append(" and (mrs.result is null or mrs.result <> 1)");
                countSql.append(" and (mrs.result is null or mrs.result <> 1)");
            }
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getPrizeFlag())){
            if (req.getQuery().getPrizeFlag().equals(1)){
                //获奖
                whereSql.append(" and (length(mrs.prize_id) > 0 or length(mrs.other_prize_id) > 0) ");
                countSql.append(" and (length(mrs.prize_id) > 0 or length(mrs.other_prize_id) > 0) ");

            }else {
                //没获奖
                whereSql.append(" and ((mrs.prize_id is null or mrs.prize_id = '') and (mrs.other_prize_id is null or mrs.other_prize_id = '')) ");
                countSql.append(" and ((mrs.prize_id is null or mrs.prize_id = '') and (mrs.other_prize_id is null or mrs.other_prize_id = '')) ");
            }
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getRegisterStartTime())&&
                ObjectUtil.isNotEmpty(req.getQuery().getRegisterEndTime())){
            whereSql.append(" and mr.create_time >= ? and mr.create_time<= ? ");
            countSql.append(" and mr.create_time >= ? and mr.create_time<= ? ");
            countParams.add(req.getQuery().getRegisterStartTime());
            countParams.add(req.getQuery().getRegisterEndTime());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewStartTime())&&
                ObjectUtil.isNotEmpty(req.getQuery().getReviewEndTime())){
            whereSql.append(" and mrs.review_time >= ? and mrs.review_time<= ? ");
            countSql.append(" and mrs.review_time >= ? and mrs.review_time<= ? ");
            countParams.add(req.getQuery().getReviewStartTime());
            countParams.add(req.getQuery().getReviewEndTime());
        }
        whereSql.append(" order by mrs.prize_name desc nulls first  ,mrs.score desc  nulls last ");
        int total = jt.queryForInt(countSql.toString(), countParams.toArray());
        List<MthMatchManagerPublicityVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(whereSql);
            //根据数据库类型设置limit
            if (!exportFlag){
                SqlUtil.appendLimit(dbType, countParams, sb, req);
            }
            result = jt.queryForBeans(MthMatchManagerPublicityVo.class, sb.toString(), countParams.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthMatchManagerPublicityVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }


    public PageRsp<MthMatchManagerContestantVo> queryMatchContestantList(PageReq<MthMatchManagerContestantReqBo> req, boolean exportFlag) {
        String sql = "SELECT\n" +
                "\tmr.id,\n" +
                "\tmrl.id AS round_list_id,\n" +
                "\tmr.id AS register_id,\n" +
                "\tmr.user_id AS user_id,\n" +
                "\tmr.match_id AS match_id,\n" +
                "\tmmi.match_name AS match_name,\n" +
                "\tmrl.round_id AS round_id,\n" +
                "\tmri.round_name AS round_name,\n" +
                "\tmr.user_name AS user_name,\n" +
                "\tmr.area_id AS area_id,\n" +
                "\tmr.area_name AS area_name,\n" +
                "\tmr.CODE AS register_code,\n" +
                "\tmr.telephone AS telephone,\n" +
                "\tmr.idcared AS id_cared,\n" +
                "\tmr.item_Name AS item_name,\n" +
                "\tmr.group_name as group_name,\n" +
                "\tmr.school_name AS school_name,\n" +
                "\tmr.company_name AS company_name,\n" +

                "\tmr.create_time AS register_time,\n" +
                "\tmrs.score AS score,\n" +
                "CASE\n" +
                "\t\tWHEN nullif( mrs.prize_name, '' ) IS NOT NULL \n" +
                "\t\tAND nullif( mrs.other_prize_name, '' ) IS NOT NULL THEN\n" +
                "\t\t\tmrs.prize_name || '、' || mrs.other_prize_name ELSE COALESCE (\n" +
                "\t\t\t\tnullif( mrs.prize_name, '' ),\n" +
                "\t\t\tnullif( mrs.other_prize_name, '' )) \n" +
                "\t\tEND AS prize_name,\n" +
                "\tif(mrs.result==1,'是','否') AS result\n" +

                "\tFROM\n" +
                "\t\tmth_register mr\n" +
                "\t\tINNER JOIN mth_round_list mrl ON mrl.register_id = mr.id\n" +
                "\t\tINNER JOIN mth_round_instance mri ON mri.id = mrl.round_id AND mri.round_type = 'instance'\n" +
                "\t\tINNER JOIN mth_match_info mmi ON mmi.id = mr.match_id  \n" +
                "\t\tINNER JOIN mth_round_score mrs ON mrs.round_list_id = mrl.id\n" +
                "\t\tLEFT JOIN mth_work mw ON mw.id = mrl.work_id \n" +
                "WHERE\n" +
                "\t1 = 1 and mr.review_result !='CANCEL'";

        String couSql ="select count(*)\n" +
                "from mth_register mr\n" +
                "         inner join mth_round_list mrl on mrl.register_id = mr.id\n" +
                "         inner join mth_round_instance mri on mri.id = mrl.round_id and mri.round_type='instance'\n" +
                "         inner join mth_match_info mmi on mmi.id = mr.match_id\n" +
                "         inner join mth_round_score mrs on mrs.round_list_id =mrl.id\n" +
                "         left join mth_work mw on mw.id =mrl.work_id\n" +
                "where 1=1 and  mr.review_result !='CANCEL'";
        StringBuilder sb = new StringBuilder(sql);
        StringBuilder countSql = new StringBuilder(couSql);

        Map<String, Object> params = new HashMap<>(16);
        List<Object> countParams = new ArrayList<>();

        StringBuilder whereSql = new StringBuilder();
        if (ObjectUtil.isNotEmpty(req.getQuery().getMthMatchId())){
            whereSql.append(" and mr.match_id = ? ");
            countSql.append(" and mr.match_id = ? ");
            countParams.add(req.getQuery().getMthMatchId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getMthRoundId())){
            whereSql.append(" and mrl.round_id = ? ");
            countSql.append(" and mrl.round_id = ? ");
            countParams.add(req.getQuery().getMthRoundId());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getAreaName())){
            whereSql.append(" and mr.area_name like ? ");
            countSql.append(" and mr.area_name like ? ");
            countParams.add("%"+req.getQuery().getAreaName()+"%");
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getWorkCode())){
            whereSql.append(" and  mw.work_code like ? ");
            countSql.append(" and  mw.work_code like ? ");
            countParams.add("%"+req.getQuery().getWorkCode()+"%");
        }
        /**参赛者编码*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getRegisterCode())){
            whereSql.append(" and  mr.code like ? ");
            countSql.append(" and  mr.code like ? ");
            countParams.add("%"+req.getQuery().getRegisterCode()+"%");
        }
        /**参赛者姓名*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getUserName())){
            whereSql.append(" and  mr.user_name like ? ");
            countSql.append(" and  mr.user_name like ? ");
            countParams.add("%"+req.getQuery().getUserName()+"%");
        }
        /**参赛者学校*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getSchoolName())){
            whereSql.append(" and  mr.school_name like ? ");
            countSql.append(" and  mr.school_name like ? ");
            countParams.add("%"+req.getQuery().getSchoolName()+"%");
        }
        /**参赛者手机号*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getTelephone())){
            whereSql.append(" and  mr.telephone = ? ");
            countSql.append(" and  mr.telephone = ? ");
            countParams.add(req.getQuery().getTelephone());
        }
        /**报名审核状态*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewResult())){
            whereSql.append(" and  mr.review_result = ? ");
            countSql.append(" and  mr.review_result = ? ");
            countParams.add(req.getQuery().getReviewResult());
        }
        /**得分评审状态*/
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewState())){
            whereSql.append(" and mrs.STATUS = ? ");
            countSql.append(" and mrs.STATUS = ? ");
            countParams.add(req.getQuery().getReviewState());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getItemId())){
            whereSql.append(" and mr.item_id = ? ");
            countSql.append(" and mr.item_id = ? ");
            countParams.add(req.getQuery().getItemId());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getGroupId())){
            whereSql.append(" and mr.group_id = ? ");
            countSql.append(" and mr.group_id = ? ");
            countParams.add(req.getQuery().getGroupId());
        }
        //是否晋级
        if (ObjectUtil.isNotEmpty(req.getQuery().getPromotionFlag())){
            if (req.getQuery().getPromotionFlag().equals(1)){
                whereSql.append(" and mrs.result = 1 ");
                countSql.append(" and mrs.result = 1 ");
            }else {
                whereSql.append(" and (mrs.result is null or mrs.result <> 1)");
                countSql.append(" and (mrs.result is null or mrs.result <> 1)");
            }

        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getPrizeFlag())){
            if (req.getQuery().getPrizeFlag().equals(1)){
                //获奖
                whereSql.append(" and (length(mrs.prize_id) > 0 or length(mrs.other_prize_id) > 0) ");
                countSql.append(" and (length(mrs.prize_id) > 0 or length(mrs.other_prize_id) > 0) ");

            }else {
                //没获奖
                whereSql.append(" and ((mrs.prize_id is null or mrs.prize_id = '') and (mrs.other_prize_id is null or mrs.other_prize_id = '')) ");
                countSql.append(" and ((mrs.prize_id is null or mrs.prize_id = '') and (mrs.other_prize_id is null or mrs.other_prize_id = '')) ");
            }
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getRegisterStartTime())&&
                ObjectUtil.isNotEmpty(req.getQuery().getRegisterEndTime())){
            whereSql.append(" and mr.create_time >= ? and mr.create_time<= ? ");
            countSql.append(" and mr.create_time >= ? and mr.create_time<= ? ");
            countParams.add(req.getQuery().getRegisterStartTime());
            countParams.add(req.getQuery().getRegisterEndTime());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getReviewStartTime())&&
                ObjectUtil.isNotEmpty(req.getQuery().getReviewEndTime())){
            whereSql.append(" and mrs.review_time >= ? and mrs.review_time<= ? ");
            countSql.append(" and mrs.review_time >= ? and mrs.review_time<= ? ");
            countParams.add(req.getQuery().getReviewStartTime());
            countParams.add(req.getQuery().getReviewEndTime());
        }
        whereSql.append(" order by mrs.sort ");
        int total = jt.queryForInt(countSql.toString(), countParams.toArray());
        List<MthMatchManagerContestantVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(whereSql);
            //根据数据库类型设置limit
            if (!exportFlag){
                SqlUtil.appendLimit(dbType, countParams, sb, req);
            }
            result = jt.queryForBeans(MthMatchManagerContestantVo.class, sb.toString(), countParams.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthMatchManagerContestantVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public List<MthMatchManagerContestantVo.ScoreDetail> findScoreDetailByRoundIds(List<Long> roundListId) {
        // 根据赛事id查询报名配置表详情
        String sql = "select tw.worker_name as reviewer_name, mrsi.round_list_id, mrsi.score, mrsi.comments, mrsi.reviewer_id, mrsi.reviewer_sign  " +
                "from mth_round_scoreinfo mrsi left join tsm_worker tw on mrsi.reviewer_id = tw.login_id " +
                "where mrsi.del_flag = 0";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and mrsi.round_list_id in (:roundListId)");
        params.put("roundListId", roundListId);
        sb.append(whereSql);
        List<MthMatchManagerContestantVo.ScoreDetail> result = baseNpJdbcTemplate.queryForBeans(MthMatchManagerContestantVo.ScoreDetail.class, sb.toString(), params);
        return result;
    }

    public PageRsp<MthRoundDeliveryVo> getDeliveryInfo(PageReq<MthDeliveryBo> req, boolean exportFlag){
        String sql="select mrs.id as roundScoreId ,mr.match_name,mri.round_name,mr.user_name,mr.school_name,mr.company_name,concat(mrs.prize_name,ifnull(mrs.other_prize_name,'')) as prize_name,\n" +
                "  a.consignee_address ,a.postalCode,a.consignee_phone ,a.consignee_name,ifnull(mrs.delivery_flag,0) as delivery_flag from mth_round_score mrs\n" +
                "                left join mth_round_list mrl on mrs.round_list_id=mrl.id \n" +
                "             left join mth_register mr on mrl.register_id=mr.id \n" +
                "             left join mth_round_instance mri on mrl.round_id=mri.id\n" +
                "             left join tsm_worker_delivery_address a on mr.user_id=a.worker_id  \n" +
                "                where mrl.round_id= :roundId  and mrs.del_flag=0 and mrs.del_flag=0 \n" +
                "                and ((mrs.prize_id is not null and mrs.prize_id!='') or (mrs.other_prize_id is not null and mrs.other_prize_id!='')) ";
        String countSql ="select count(mrs.id) from mth_round_score mrs\n" +
                "                left join mth_round_list mrl on mrs.round_list_id=mrl.id \n" +
                "            \tleft join mth_register mr on mrl.register_id=mr.id \n" +
                "            \tleft join mth_round_instance mri on mrl.round_id=mri.id\n" +
                "            \tleft join tsm_worker_delivery_address a on mr.user_id=a.worker_id  \n" +
                "                where mrl.round_id= :roundId  and mrs.del_flag=0 and mrs.del_flag=0 \n" +
                "                and ((mrs.prize_id is not null and mrs.prize_id!='') or (mrs.other_prize_id is not null and mrs.other_prize_id!='')) ";

        StringBuilder sb = new StringBuilder(sql);
        StringBuilder countSqlSb = new StringBuilder(countSql);

        Map<String, Object> paramsMap = new HashMap<>(16);
        paramsMap.put("roundId",req.getQuery().getRoundId());
        if (ObjectUtil.isNotEmpty(req.getQuery().getUserName())){
            sb.append(" and mr.user_name like  concat('%', :userName ,'%') ");
            countSqlSb.append(" and mr.user_name like  concat('%', :userName ,'%')");
            paramsMap.put("userName",req.getQuery().getUserName());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getSchoolName())){
            sb.append(" and mr.school_name like  concat('%', :schoolName ,'%') ");
            countSqlSb.append(" and mr.school_name like  concat('%', :schoolName ,'%')");
            paramsMap.put("schoolName",req.getQuery().getSchoolName());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getDeliveryFlag())){
            sb.append( " and mrs.delivery_flag = :deliveryFlag");
            countSqlSb.append(" and mrs.delivery_flag = :deliveryFlag" );
            paramsMap.put("deliveryFlag",req.getQuery().getDeliveryFlag());
        }
        int total = baseNpJdbcTemplate.queryForInt(countSqlSb.toString(),paramsMap );
        List<MthRoundDeliveryVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by mrs.sort ");
            //根据数据库类型设置limit
            if (!exportFlag){
                SqlUtil.appendLimit(dbType, paramsMap, sb, req);
            }
            result = baseNpJdbcTemplate.queryForBeans(MthRoundDeliveryVo.class, sb.toString(), paramsMap);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthRoundDeliveryVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;

    }

    /**
     *
     * 设置晋级排名列表
     * */
    public PageRsp<MthWorkPromotionVo> queryWorkPromotionManagerList(PageReq<MthWorkPromiseReqBo> req) {
        String sql = "select \n" +
                "mrs.id,\n" +
                "mrs.round_list_id ,\n" +
                "mrs.sort ,\n" +
                "mrs.score ,\n" +
                "mrs.result ,\n" +
                "mrs.old_sort ,\n" +
                "mrs.old_score ,\n" +
                "mrs.prize_id ,\n" +
                "mrs.prize_name ,\n" +
                "mrs.other_prize_id ,\n" +
                "mrs.other_prize_name ,\n" +
                "mrs.comments ,\n" +
                "mrs.reviewer_id ,\n" +
                "mrs.reviewer_name ,\n" +
                "mrs.reviewer_sign ,\n" +
                "mrs.del_flag ,\n" +
                "mrs.create_id ,\n" +
                "mrs.create_by ,\n" +
                "mrs.create_time ,\n" +
                "mrs.update_by ,\n" +
                "mrs.update_id ,\n" +
                "mrs.update_time ,\n" +
                "mrs.status ,\n" +
                "mrs.review_time,\n" +
                "mw.work_code,\n" +
                "mw.work_name, \n" +
                "mr.idCared,\n" +
                "mr.school_name, mr.item_name,mr.group_name,mr.item_id,mr.group_id,\n" +
                "mrl.user_name, \n" +
                "mrl.round_id \n" +
                "from mth_round_score mrs\n" +
                "left join mth_round_list mrl on mrl.id =mrs.round_list_id\n" +
                "inner join mth_register mr on mr.id=mrl.register_id and mr.review_result in ('APPROVED','NOREVIEW')\n" +
                "left join mth_work mw on mw.id=mrl.work_id\n" +
                "where  mrl.round_id =? ";

        String count = "select  count(mrs.id) " +
                "from mth_round_score mrs\n" +
                "left join mth_round_list mrl on mrl.id =mrs.round_list_id\n" +
                "inner join mth_register mr on mr.id=mrl.register_id and mr.review_result in ('APPROVED','NOREVIEW')\n" +
                "left join mth_work mw on mw.id=mrl.work_id\n" +
                "where  mrl.round_id =? ";

        StringBuilder sb = new StringBuilder(sql);
        StringBuilder countSql = new StringBuilder(count);
        List<Object> countParams = new ArrayList<>();
        countParams.add(req.getQuery().getRoundId());
        if (ObjectUtil.isNotNull(req.getQuery().getItemId())){
            sb.append(" and mr.item_id=? ");
            countSql.append(" and mr.item_id=? ");
            countParams.add(req.getQuery().getItemId());
        }
        if (ObjectUtil.isNotNull(req.getQuery().getGroupId())){
            sb.append(" and mr.group_id = ? ");
            countSql.append(" and mr.group_id = ? ");
            countParams.add(req.getQuery().getGroupId());
        }

        if (ObjectUtil.isNotNull(req.getQuery().getUserName())){
            sb.append( " and mr.user_name like ? ");
            countSql.append("and mr.user_name like ? ");
            countParams.add("%"+req.getQuery().getUserName()+"%");

        }
        sb.append(" order by  mrs.sort asc NULLS LAST ,mw.create_time ");
        int total = jt.queryForInt(countSql.toString(), countParams.toArray());

        List<MthWorkPromotionVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, countParams, sb, req);
            result = jt.queryForBeans(MthWorkPromotionVo.class, sb.toString(), countParams.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthWorkPromotionVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }


    /**
     * 比赛过程记录查询
     * */
    public List<MthMatchProcessVo> queryProcessInfo(MthMatchProcessReqBo bo) {
        String sql = "select * from mth_match_process a where del_flag = 0 and user_id=:user_id ";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> map = new HashMap<>(16);
        map.put("match_id", bo.getMthMatchId());
        map.put("user_id", bo.getRegisterId());
        sb.append(" order by id ");
        List<MthMatchProcessVo> result;
        result = baseNpJdbcTemplate.queryForBeans(MthMatchProcessVo.class, sb.toString(), map);
        return result;
    }


        public List<MthMyMatchListDetailAppVo.Detail> queryRegisterDetailForApp(List<Long> bizIds) {
            // 根据赛事id查询报名配置表详情
            String sql = " select if(mfc.key is null,mfc.show_name,mfc.key)  AS KEY,mrd.VALUE ,mfc.show_name ,mrcd.sort_num AS sort,mfc.config_type AS type,mrd.biz_id  ,mrcd.group_name from mth_register_detail mrd " +
                    "left join mth_register_config_detail mrcd on mrd.config_detail_id = mrcd.id " +
                    "left join mth_form_config mfc on mrcd.register_config_id = mfc.id " +
                    "where mrd.del_flag = 0 and mrcd.del_flag = 0 and mfc.del_flag = 0";
            StringBuilder sb = new StringBuilder(sql);
            Map<String, Object> params = new HashMap<>(16);
            StringBuilder whereSql = new StringBuilder();
            whereSql.append(" and mrd.biz_id in (:bizIds)");
            params.put("bizIds", bizIds);
            sb.append(whereSql);
            sb.append(" order by sort_num");
            List<MthMyMatchListDetailAppVo.Detail> result = baseNpJdbcTemplate.queryForBeans(MthMyMatchListDetailAppVo.Detail.class, sb.toString(), params);
            return result;
        }


//查询赛事报名配置
    public MatchRegisterFormVo getregisterInfos(Long matchId) {
        String sql = "select id as registerId,config_type as configType,name from mth_register_config where del_flag = 0 and match_id = :matchId AND config_type='register'";
        Map<String, Object> map = new HashMap<>();
        map.put("matchId", matchId);
        return baseNpJdbcTemplate.queryForBean(MatchRegisterFormVo.class,sql,map);
    }

    public    List<MthMyMatchListDetailAppVo.Detail>  findConfigInfoBy(Long registerId) {


        String sql="SELECT  d.group_name AS groupName," +
                " d.sort_num AS sort,c.show_name AS showName," +
                " c.required_flag AS requiredFlag," +
                " c.config_type AS type, " +
                "  c.ref_type AS refType," +
                " c.ref_id AS refId," +
               // "  C.key AS key," +
                "IF(C.KEY IS NULL,c.show_name,C.KEY) AS key,"+
                "c.dynamic_flag as dynamicFlag ," +
                "c.min_file_size as minFileSize," +
                "c.max_file_size as maxFileSize ," +
                "c.limit_num as limit_num ," +
                "  C.id as registerConfigId," +
                " d.register_info_id as registerInfoId " +
                "  FROM YOUTH_CQ.mth_register_config_detail d ,YOUTH_CQ.mth_form_config  c WHERE d.register_config_id =c.id and d.del_flag=0 and c.del_flag=0" +
                "   AND d.register_info_id = ? order by d.sort_num asc";

        return   jt.queryForBeans(MthMyMatchListDetailAppVo.Detail.class,sql,registerId);
    }
}
