package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.ContactBo;
import com.crunii.micro.service.match.api.bo.MthMyProjectApplyAppBo;
import com.crunii.micro.service.match.api.bo.MthProjectAppBo;
import com.crunii.micro.service.match.api.bo.PageProjectFinancingBo;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.enums.FinancingEnums;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author gongsongtao
 * @date 2023/12/18 16:19
 */
@Repository
public class MthProjectFinancingDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;
    String PAGE_BASE_SQL="select a.id," +
            " case when a.source=1 or a.source is null then mmi.match_name else b.match_name end as matchName," +
            " case when a.source=1 or a.source is null then mmi.category_name else b.match_type end as categoryName," +
            " case when a.source=1 or a.source is null then d.item_name else b.match_item end as itemName," +
            " case when a.source=1 or a.source is null then e.group_name else b.match_group end as groupName," +
            " case when a.source=1 or a.source is null then f.worker_name else b.participant_name end as workerName," +
            " case when a.source=1 or a.source is null then f.mobile else b.participant_mobile end as mobile," +
            " case when a.source=2 then a.prize_name else null end as prizeName," +
            " a.expected_amount,a.transfer_equity/100 as transferEquity,a.create_time,a.review_status," +
            " case when a.review_status=0 then '待审核' when a.review_status=1 then '已通过' when a.review_status=2 then '不通过' when a.review_status=3 then '已撤销' else null end as reviewStatusName," +
            " a.publicity_status,case when a.publicity_status=1 then '已公示' else '未公示' end as publicityStatusName,a.code,c.match_id,c.id as registerId,a.source " +
            " from mth_project_financing a   " +
            " left join mth_project b on a.project_id=b.id   " +
            " left join mth_register c on b.register_id=c.id  " +
            " left join mth_match_info mmi on c.match_id=mmi.id  " +
            " left join mth_match_item d on c.item_id=d.id  " +
            " left join mth_match_group e on c.group_id=e.id  " +
            " left join tsm_worker f on c.user_id=f.login_id  " +
            " where a.del_flag=0 and b.del_flag=0 " +
            " and (b.register_id is null or (c.del_flag=0 and mmi.del_flag=0 and d.del_flag=0 and c.review_result in ('APPROVED','NOREVIEW')))";
    String PAGE_COUNT_SQL="select count(*) from mth_project_financing a" +
            " left join mth_project b on a.project_id=b.id" +
            " left join mth_register c on b.register_id=c.id" +
            " left join mth_match_info mmi on c.match_id=mmi.id" +
            " left join mth_match_item d on c.item_id=d.id" +
            " left join mth_match_group e on c.group_id=e.id" +
            " left join tsm_worker f on c.user_id=f.login_id" +
            " where a.del_flag=0 and b.del_flag=0 " +
            " and (b.register_id is null or (c.del_flag=0 and mmi.del_flag=0 and d.del_flag=0 and c.review_result in ('APPROVED','NOREVIEW')))";

    public PageRsp<MthProjectFinancingPageVo> queryPageList(PageReq<PageProjectFinancingBo> req) {
        StringBuilder sql = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        Map<String, Object> params = new HashMap<>();
        PageProjectFinancingBo query = req.getQuery();
        if (ObjectUtil.isNotEmpty(query)){
            //关键字,匹配 赛事名称、比赛分类、赛项、组别、参赛者姓名、参赛者手机号
            if (StringUtils.isNotEmpty(query.getPivotalWord())){
                sql.append(" and (mmi.match_name like concat('%',:pivotalWord,'%') or mmi.category_name like concat('%',:pivotalWord,'%') or d.item_name like concat('%',:pivotalWord,'%')" +
                        " or e.group_name like concat('%',:pivotalWord,'%') or f.worker_name like concat('%',:pivotalWord,'%') or f.mobile like concat('%',:pivotalWord,'%'))");
                countSql.append(" and (mmi.match_name like concat('%',:pivotalWord,'%') or mmi.category_name like concat('%',:pivotalWord,'%') or d.item_name like concat('%',:pivotalWord,'%')" +
                        " or e.group_name like concat('%',:pivotalWord,'%') or f.worker_name like concat('%',:pivotalWord,'%') or f.mobile like concat('%',:pivotalWord,'%'))");
                params.put("pivotalWord",query.getPivotalWord());
            }
            //公示状态
            if (query.getPublicityStatus()!=null){
                sql.append(" and a.publicity_status=:publicityStatus");
                countSql.append(" and a.publicity_status=:publicityStatus");
                params.put("publicityStatus",query.getPublicityStatus());
            }
            //审核状态
            if (query.getReviewStatus()!=null){
                sql.append(" and a.review_status=:reviewStatus");
                countSql.append(" and a.review_status=:reviewStatus");
                params.put("reviewStatus",query.getReviewStatus());
            }
            //申请时间
            if (StringUtils.isNotEmpty(query.getCreateStartTime()) && StringUtils.isNotEmpty(query.getCreateEndTime())){
                sql.append(" and a.create_time>=:startTime and a.create_time<=:endTime");
                countSql.append(" and a.create_time>=:startTime and a.create_time<=:endTime");
                params.put("startTime",query.getCreateStartTime());
                params.put("endTime",query.getCreateEndTime());
            }
        }
        sql.append(" order by a.create_time desc ");

        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<MthProjectFinancingPageVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(MthProjectFinancingPageVo.class, sql.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthProjectFinancingPageVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    /**
     * 查询融资详情
     * @param id
     * @return
     */
    public MthProjectFinancingBaselVo queryById(Long id){
        String sql="select b.*," +
                " case when a.source=1 or a.source is null then mmi.match_name else b.match_name end as matchName," +
                " case when a.source=1 or a.source is null then mmi.category_name else b.match_type end as categoryName," +
                " case when a.source=1 or a.source is null then d.item_name else b.match_item end as projectItemName," +
                " case when a.source=1 or a.source is null then e.group_name else b.match_group end as projectGroupName," +
                " case when a.source=1 or a.source is null then f.worker_name else b.participant_name end as workerName," +
                " case when a.source=1 or a.source is null then f.mobile else b.participant_mobile end as mobile," +
                " case when a.source=2 then a.prize_name else null end as prizeName," +
                " a.id as financingId,a.*,b.team_members as teamTotal,mmi.registeration_end,mmi.registeration_start," +
                " a.expected_amount,a.transfer_equity,a.create_time,a.review_status," +
                " case when a.review_status=0 then '待审核' when a.review_status=1 then '已通过' when a.review_status=2 then '不通过' when a.review_status=3 then '已撤销' else null end as ProjectReviewStatus" +
                " from mth_project_financing a" +
                " left join mth_project b on a.project_id=b.id" +
                " left join mth_register c on b.register_id=c.id" +
                " left join mth_match_info mmi on c.match_id=mmi.id" +
                " left join mth_match_item d on c.item_id=d.id" +
                " left join mth_match_group e on c.group_id=e.id" +
                " left join tsm_worker f on c.user_id=f.login_id" +
                " where a.del_flag=0 and b.del_flag=0 " +
                " and (b.register_id is null or (c.del_flag=0 and mmi.del_flag=0 and d.del_flag=0 and c.review_result in ('APPROVED','NOREVIEW')))" +
                " and a.id=:id";
                Map<String,Object>map=new HashMap<>();
                map.put("id",id);
        return baseNpJdbcTemplate.queryForBean(MthProjectFinancingBaselVo.class,sql,map);
    }

    /**
     * 根据报名ID查询评审信息
     * @param roundListId
     * @return
     */
    public List<MthProjectFinancingDetailsVo.ReviewList> getReviewList(Long roundListId){
            String sql="select a.id,a.create_time as reviewDate,b.worker_name as reviewerName,a.score  " +
                    " from mth_round_scoreinfo a " +
                    " left join tsm_worker b on a.reviewer_id=b.login_id" +
                    " where a.del_flag=0 and a.round_list_id=:id";
        Map<String,Object>map=new HashMap<>();
        map.put("id",roundListId);
        return baseNpJdbcTemplate.queryForBeans(MthProjectFinancingDetailsVo.ReviewList.class,sql,map);
    }

    /**
     * 根据赛事ID查询所有场次
     * @param matchId
     * @return
     */
    public List<MthProjectFinancingDetailsVo.RoundList> getAllRoundByMatchId(Long matchId){
        String sql="select c.id as instanceId,c.round_name,(case when b.sort is null then b.old_score else b.score end) as score," +
                " (case when b.RESULT=1 then '是' else '否' end)as whetherPromotion,a.create_time as promotionDate,b.prize_name as  prize" +
                " from mth_round_list a " +
                " left join mth_round_score b on a.id=b.round_list_id" +
                " left join mth_round_instance c on a.round_id=c.id" +
                " where a.del_flag=0 and c.del_flag=0 and b.del_flag=0  and c.round_state=0 and a.register_id=:id";
        Map<String,Object>map=new HashMap<>();
        map.put("id",matchId);
        return baseNpJdbcTemplate.queryForBeans(MthProjectFinancingDetailsVo.RoundList.class,sql,map);
    }

    /**
     * app-查询融资申请记录
     * @param req
     * @return
     */
    public PageRsp<MthProjectAppVo> pageProject(PageReq<MthProjectAppBo> req) {
        String BASE_SQL="select  a.id,b.name,b.logo,b.address,b.industry,b.start_time,a.expected_amount,a.official_website,a.introduce,d.group_name as groupName,c.id as registerId" +
                " from mth_project_financing a  " +
                " left join mth_project b on b.id=a.project_id " +
                " left join mth_register c on b.register_id=c.id " +
                " left join mth_match_group d on c.group_id=d.id " +
                " where a.del_flag=0 and (b.register_id is null or c.del_flag=0) and a.review_status=1 and a.publicity_status=1";

        String count="select count(*)" +
                " from mth_project_financing a  " +
                " left join mth_project b on b.id=a.project_id " +
                " where a.del_flag=0 and b.del_flag=0 and a.review_status=1 and a.publicity_status=1";
        StringBuilder sql = new StringBuilder(BASE_SQL);
        StringBuilder countSql = new StringBuilder(count);
        Map<String, Object> params = new HashMap<>();
        MthProjectAppBo query = req.getQuery();
        if (ObjectUtil.isNotEmpty(query)){
            //所在地区
            if (StringUtils.isNotEmpty(query.getAddress())){
                sql.append(" and b.address=:address");
                countSql.append(" and b.address=:address");
                params.put("address",query.getAddress());
            }
            //所属行业
            if (StringUtils.isNotEmpty(query.getIndustry())){
                sql.append(" and b.industry like concat ('%',:industry,'%')");
                countSql.append(" and b.industry like concat ('%',:industry,'%')");
                params.put("industry",query.getIndustry());
            }
            //关键字匹配
            if (StringUtils.isNotEmpty(query.getPivotalWord())){
                sql.append(" and (b.name like concat ('%',:pivotalWord,'%') or b.name like concat ('%',:pivotalWord,'%'))");
                countSql.append(" and (b.name like concat ('%',:pivotalWord,'%') or b.name like concat ('%',:pivotalWord,'%'))");
                params.put("pivotalWord",query.getPivotalWord());
            }
        }
        sql.append(" order by a.create_time desc ");

        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<MthProjectAppVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(MthProjectAppVo.class, sql.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthProjectAppVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }


    /**
     * app-查询我的项目
     * @param req
     * @return
     */
    public PageRsp<MthMyProjectApplyAppVo> pageMyProjectApply(PageReq<MthMyProjectApplyAppBo> req) {
        String BASE_SQL = "select  a.id,a.code,b.name,a.review_status,a.publicity_status,a.user_id,c.user_name,a.create_time, " +
                " a.reviewer_id,a.reviewer_name,a.review_time,a.review_reason " +
                " from mth_project_financing a  " +
                " left join mth_project b on b.id=a.project_id " +
                " left join mth_register c on b.register_id=c.id " +
                " where a.del_flag=0 and b.del_flag=0 " +
                " and (b.register_id is null or (c.del_flag=0 and c.review_result in ('APPROVED','NOREVIEW')))";
        String BASE_COUNT_SQL = "select count(*) " +
                " from mth_project_financing a  " +
                " left join mth_project b on b.id=a.project_id " +
                " left join mth_register c on b.register_id=c.id " +
                " where a.del_flag=0 and b.del_flag=0 " +
                " and (b.register_id is null or (c.del_flag=0 and c.review_result in ('APPROVED','NOREVIEW')))";
        StringBuilder sql = new StringBuilder(BASE_SQL);
        StringBuilder countSql = new StringBuilder(BASE_COUNT_SQL);
        StringBuilder whereSql = new StringBuilder();
        Map<String, Object> params = Maps.newHashMap();
        MthMyProjectApplyAppBo query = req.getQuery();

        whereSql.append(" and a.user_id = :userId");
        params.put("userId", query.getUserId());
        if (ObjectUtil.isNotEmpty(query)) {
            //公示状态
            if (ObjectUtil.isNotEmpty(query.getPublicityStatus()) && 1==query.getPublicityStatus()){
                whereSql.append(" and a.publicity_status=:publicityStatus");
                params.put("publicityStatus", query.getPublicityStatus());
            }else {
                //待展示数据为[待审核,审核不通过,审核通过未展示]
                whereSql.append(" and (a.review_status in (0,2) or (a.review_status=1 and a.publicity_status=0))");
            }
        }
        countSql.append(whereSql);
        sql.append(whereSql);
        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<MthMyProjectApplyAppVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(MthMyProjectApplyAppVo.class, sql.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthMyProjectApplyAppVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }


    /**
     * app-查询我的项目详情
     * @param id
     * @return
     */
    public MthMyProjectApplyAppVo getProjectApplyDetail(Long id) {
        String sql = "select  a.id,a.code,b.name,a.review_status,a.publicity_status,a.user_id,c.user_name,a.create_time, " +
                " a.reviewer_id,a.reviewer_name,a.review_time,a.review_reason " +
                " from mth_project_financing a  " +
                " left join mth_project b on b.id=a.project_id " +
                " left join mth_register c on b.register_id=c.id " +
                " where a.del_flag=0 and b.del_flag=0 and (b.register_id is null or c.del_flag=0) and a.id = :id";
        Map<String, Object> map = new HashMap<>();
        map.put("id", id);
        return baseNpJdbcTemplate.queryForBean(MthMyProjectApplyAppVo.class, sql, map);
    }

    /**
     * 查询联系人信息
     * @param req
     * @return
     */
    public PageRsp<ContactVo> getContact(PageReq<ContactBo> req, Worker worker) {
        String baseSql = "select c.name as projectName,a.* from mth_project_financing_contact a" +
                " left join mth_project_financing b on a.financing_id=b.id" +
                " left join mth_project c on b.project_id=c.id" +
                " where a.del_flag=0 and b.del_flag=0 ";


        String baseCountSql = "select count(*) from mth_project_financing_contact a" +
                " left join mth_project_financing b on a.financing_id=b.id" +
                " left join mth_project c on b.project_id=c.id" +
                " where a.del_flag=0 and b.del_flag=0 ";
        StringBuilder sql = new StringBuilder(baseSql);
        StringBuilder countSql = new StringBuilder(baseCountSql);
        StringBuilder whereSql = new StringBuilder();
        Map<String, Object> params = Maps.newHashMap();
        ContactBo query = req.getQuery();
        if (ObjectUtil.isNotEmpty(query) && query.getFinancingId()!=null ) {
            whereSql.append(" and b.id=:id");
            params.put("id",query.getFinancingId());
        }else {
            whereSql.append(" and b.user_id=:userId");
            params.put("userId",worker.getLoginId());
        }
        countSql.append(whereSql);
        sql.append(whereSql);
        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<ContactVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(ContactVo.class, sql.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<ContactVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }
}
