package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.MthRegister;
import com.crunii.micro.domain.entity.main.MthRegisterReview;
import com.crunii.micro.service.match.api.bo.MatchMyCollectBo;
import com.crunii.micro.service.match.api.bo.MthMatchManagerRegisterReqBo;
import com.crunii.micro.service.match.api.bo.MthRegisterBo;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.enums.MatchEnums;
import com.crunii.micro.service.match.enums.RoundListStatusEnums;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author xujie
 * @date 2023-09-20
 */
@Repository
public class MthRegisterDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    private static final String PAGE_BASE_SQL = "select * from mth_register a where del_flag = 0 ";
    private static final String PAGE_COUNT_SQL = "select count(*) from mth_register a where del_flag = 0 ";
    private static final String DELETE_SQL = "update mth_register set del_flag=1, update_id=:updateId, update_by=:updateBy, update_time=:updateTime where id in (:ids)";

    private static final String EDIT_STATUS_SQL = "update mth_register set status =:status where del_flag = 0 and id in(:ids)";

    public PageRsp<MthRegisterVo> queryPageList(PageReq<MthRegisterBo> req) {
        StringBuilder sql = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        if (StrUtil.isNotBlank(req.getQuery().getMatchName())) {
            whereSql.append(" and a.match_name like concat('%', :matchName ,'%') ");
            params.put("matchName", req.getQuery().getMatchName());
        }
        if (Objects.nonNull(req.getQuery().getId())) {
            whereSql.append(" and a.id = :id ");
            params.put("id", req.getQuery().getId());
        }
        countSql.append(whereSql);
        sql.append(whereSql);
        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<MthRegisterVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(MthRegisterVo.class, sql.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthRegisterVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<MthRegisterVo> queryList(MthRegisterBo req) {
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        if (StrUtil.isNotBlank(req.getMatchName())) {
            whereSql.append(" and a.match_name like concat('%', :matchName ,'%') ");
            params.put("matchName", req.getMatchName());
        }
        if (Objects.nonNull(req.getId())) {
            whereSql.append(" and a.id = :id ");
            params.put("id", req.getId());
        }
        sb.append(whereSql);
        List<MthRegisterVo> result = baseNpJdbcTemplate.queryForBeans(MthRegisterVo.class, sb.toString(), params);
        return result;
    }


    public List<MthRegisterAreaAppVo> queryRegisterAreaListForApp(Long matchId) {
        // 根据赛事id查询场次序号为1的场次实例
        String sql = "select match_id, id, area_id, area_name from mth_round_instance " +
                " where del_flag = 0 and round_sort = 1 and round_type = 'instance' and match_id = :matchId" +
                " and match_level in (select match_level from mth_round_instance where del_flag = 0 " +
                " and round_sort = 1 " +
                " and round_type = 'instance'" +
                " and match_id=:matchId" +
                " and match_level is not null " +
                " order by case when match_level='city' then 0 when match_level='district' then 1 else 3 end desc limit 1) ";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        params.put("matchId", matchId);
        sb.append(whereSql);
        List<MthRegisterAreaAppVo> result = baseNpJdbcTemplate.queryForBeans(MthRegisterAreaAppVo.class, sb.toString(), params);
        return result;
    }

    public MthRegisterConfigAppVo.Register queryRegisterConfigForApp(Long matchId) {
        // 根据赛事id查询报名配置表
        String sql = "select mrc.*, mmi.match_name from mth_register_config mrc left join mth_match_info mmi on mrc.match_id = mmi.id" +
                " where mrc.del_flag = 0 and mmi.del_flag = 0";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and mrc.match_id = :matchId");
        params.put("matchId", matchId);
        whereSql.append(" and mrc.config_type = :registerType");
        params.put("registerType", MatchEnums.ConfigType.REGISTER.getCode());
        sb.append(whereSql);
        List<MthRegisterConfigAppVo.Register> result = baseNpJdbcTemplate.queryForBeans(MthRegisterConfigAppVo.Register.class, sb.toString(), params);
        if (ObjectUtil.isNotEmpty(result)) {
            return result.get(0);
        }
        return null;
    }

    public MthRegisterConfigAppVo.Work queryWorkConfigForApp(Long matchId) {
        // 根据赛事id查询报名配置表
        String sql = "select * from mth_register_config where del_flag = 0";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and match_id = :matchId");
        params.put("matchId", matchId);
        whereSql.append(" and config_type = :configType");
        params.put("configType", MatchEnums.ConfigType.WORKS.getCode());
        sb.append(whereSql);
        List<MthRegisterConfigAppVo.Work> result = baseNpJdbcTemplate.queryForBeans(MthRegisterConfigAppVo.Work.class, sb.toString(), params);
        if (ObjectUtil.isNotEmpty(result)) {
            return result.get(0);
        }
        return null;
    }

    public List<MthRegisterConfigAppVo.Detail> queryRegisterConfigDetailForApp(Long registerInfoId) {
        // 根据赛事id查询报名配置表详情
        String sql = "select mrcd.*, mfc.show_name, mfc.required_flag, mfc.config_type, mfc.ref_id, mfc.key, mfc.hint,mfc.min_file_size,max_file_size,limit_num " +
                "from mth_register_config_detail mrcd left join mth_form_config mfc on mrcd.register_config_id = mfc.id " +
                "where mrcd.del_flag = 0 and mfc.del_flag = 0";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and mrcd.register_info_id = :registerInfoId");
        params.put("registerInfoId", registerInfoId);
        sb.append(whereSql);
        sb.append(" order by sort_num");
        List<MthRegisterConfigAppVo.Detail> result = baseNpJdbcTemplate.queryForBeans(MthRegisterConfigAppVo.Detail.class, sb.toString(), params);
        return result;
    }

    public MthMyMatchListDetailAppVo.Register queryRegisterForApp(Long registerId) {
        // 根据赛事id查询报名配置表
        String sql = "select mr.*, mri.id round_instance_id, mri.registeration_start, " +
                "mri.registeration_end, mri.entry_submit_mode, mri.entry_deadline, " +
                "mrc.config_desc, mrc.pic_id, mrl.work_id, mri.match_state,mri.receive_type " +
                " from mth_register mr " +
                " left join mth_round_list mrl on mr.id = mrl.register_id " +
                " left join mth_round_instance mri on mrl.round_id = mri.id " +
                " left join mth_register_config mrc on mr.register_config_id = mrc.id " +
                " where mr.del_flag = 0 and mrl.del_flag = 0 and mri.del_flag = 0";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and mr.id = :registerId ");
        params.put("registerId", registerId);
        sb.append(whereSql);
        sb.append(" order by mrl.create_time desc");
        List<MthMyMatchListDetailAppVo.Register> result = baseNpJdbcTemplate.queryForBeans(MthMyMatchListDetailAppVo.Register.class, sb.toString(), params);
        if (ObjectUtil.isNotEmpty(result)) {
            return result.get(0);
        }
        return null;
    }

    public MthRegisterReview queryRegisterReview(Long registerId) {
        // 查询审核原因
        String sql = "select mrr.* from mth_register mr " +
                "left join mth_register_review mrr on mr.id = mrr.register_id " +
                "where mr.del_flag = 0 and mrr.del_flag = 0";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and mr.id = :registerId");
        params.put("registerId", registerId);
        sb.append(whereSql);
        sb.append(" order by mrr.create_time desc");
        List<MthRegisterReview> result = baseNpJdbcTemplate.queryForBeans(MthRegisterReview.class, sb.toString(), params);
        if (ObjectUtil.isNotEmpty(result)) {
            return result.get(0);
        }
        return null;
    }

    public List<MthMyMatchListDetailAppVo.Work> queryWorkForApp(Long registerId) {
        // 根据赛事id查询报名配置表
        String sql = "select mw.*, mri.area_name || '-' || mri.round_name as round_range_name, " +
                "mrc.config_desc, mrc.pic_id from mth_work mw " +
                "left join mth_round_list mrl on mw.id = mrl.work_id " +
                "left join mth_round_instance mri on mrl.round_id = mri.id " +
                "left join mth_register_config mrc on mw.register_config_id = mrc.id " +
                "where mw.del_flag = 0 and mrl.del_flag = 0 and mri.del_flag = 0 and mw.id is not null";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and mrl.register_id in (:registerId)");
        params.put("registerId", registerId);
        sb.append(whereSql);
        List<MthMyMatchListDetailAppVo.Work> result = baseNpJdbcTemplate.queryForBeans(MthMyMatchListDetailAppVo.Work.class, sb.toString(), params);
        return result;
    }

    public List<MthMyMatchListDetailAppVo.Detail> queryRegisterDetailForApp(List<Long> bizIds) {
        // 根据赛事id查询报名配置表详情
        String sql = "select *,mrcd.group_name from mth_register_detail mrd " +
                "left join mth_register_config_detail mrcd on mrd.config_detail_id = mrcd.id " +
                "left join mth_form_config mfc on mrcd.register_config_id = mfc.id " +
                "where mrd.del_flag = 0 and mrcd.del_flag = 0 and mfc.del_flag = 0";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and mrd.biz_id in (:bizIds)");
        params.put("bizIds", bizIds);
        sb.append(whereSql);
        sb.append(" order by sort_num");
        List<MthMyMatchListDetailAppVo.Detail> result = baseNpJdbcTemplate.queryForBeans(MthMyMatchListDetailAppVo.Detail.class, sb.toString(), params);
        return result;
    }

    public List<MthWorkVo> queryRegisterWorkForApp(Long registerId) {
        // 根据报名id查询作品id
        String sql = "select mrl.work_id as id, mw.work_code, mw.work_name, mrl.id round_list_id " +
                "from mth_round_list mrl left join mth_work mw on mrl.work_id = mw.id where mrl.del_flag = 0";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and mrl.register_id = :registerId ");
        params.put("registerId", registerId);
        sb.append(whereSql);
        sb.append(" order by mrl.create_time desc");
        List<MthWorkVo> result = baseNpJdbcTemplate.queryForBeans(MthWorkVo.class, sb.toString(), params);
        return result;
    }


    public List<MthMyMatchListDetailAppVo.RoundPrize> queryRoundScoreForApp(Long registerId) {
        // 根据赛事id查询报名配置表
        String sql = "select mrs.*, mmi.match_name || '-' || mri.area_name || '-' || mri.round_name as round_full_name," +
                " case when mrs.delivery_flag=1 then '已发出' else '未发出' end deliveryFlagName," +
                " case when mri.receive_type='DELIVERY' and mrs.prize_msg_flag=1 and (length(mrs.prize_id)>0) then true else false end whetherDelivery "+
                "from mth_round_score mrs " +
                "left join mth_round_list mrl on mrs.round_list_id = mrl.id " +
                "left join mth_round_instance mri on mrl.round_id = mri.id " +
                "left join mth_match_info mmi on mmi.id = mri.match_id " +
                "where mrs.del_flag = 0 and mrl.del_flag = 0 and mri.del_flag = 0 " +
                "and mri.match_state = 4";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and mrl.register_id in (:registerId)");
        params.put("registerId", registerId);
        sb.append(whereSql);
        sb.append(" order by mrl.create_time desc");
        List<MthMyMatchListDetailAppVo.RoundPrize> result = baseNpJdbcTemplate.queryForBeans(MthMyMatchListDetailAppVo.RoundPrize.class, sb.toString(), params);
        return result;
    }

    public List<MthRegister> queryMyRegisterList(Long userId, Long matchId, Long itemId, Long groupId) {
        // 根据赛事id查询报名配置表详情
        String sql = "select * from mth_register a where del_flag = 0 ";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder whereSql = new StringBuilder();
        whereSql.append(" and user_id = :userId");
        params.put("userId", userId);
        whereSql.append(" and match_id = :matchId");
        params.put("matchId", matchId);
//        if (Objects.nonNull(itemId)) {
//            whereSql.append(" and item_id = :itemId");
//            params.put("itemId", itemId);
//        }
//        if (Objects.nonNull(groupId)) {
//            whereSql.append(" and group_id = :groupId");
//            params.put("groupId", groupId);
//        }
        sb.append(whereSql);
        sb.append(" and review_result != 'CANCEL' ");
        List<MthRegister> result = baseNpJdbcTemplate.queryForBeans(MthRegister.class, sb.toString(), params);
        return result;
    }

    public void deleteByIds(Collection<Long> ids, Integer updateId, String updateBy) {
        Map<String, Object> params = new HashMap<>(4);
        params.put("ids", ids);
        params.put("updateId", updateId);
        params.put("updateBy", updateBy);
        params.put("updateTime", new Date());
        baseNpJdbcTemplate.update(DELETE_SQL, params);
    }

    public Integer updateStatus(List<Long> ids, String status) {
        if (CollectionUtils.isEmpty(ids) || ObjectUtil.isNull(status)) {
            throw new BusinessException("系统内部错误");
        } else {
            Map<String, Object> map = new HashMap<>();
            map.put("ids", ids);
            map.put("status", status);
            int update = baseNpJdbcTemplate.update(EDIT_STATUS_SQL, map);
            return update;
        }
    }

    /**
     * 查询已收藏赛事
     * @param worker
     * @param req
     * @return
     */
    public PageRsp<MthMyMatchListAppVo> pageCollectedMatch(Worker worker, PageReq<MatchMyCollectBo> req) {
        StringBuilder sql = new StringBuilder("select  mmi.* " +
                " from mth_match_collected mmc" +
                " left join mth_match_info mmi on mmc.match_info_id = mmi.id " +
                " where mmc.del_flag = 0 and mmi.del_flag = 0 and mmc.user_id=:userId");
        StringBuilder countSql = new StringBuilder("select count(*) " +
                " from mth_match_collected mmc " +
                " left join mth_match_info mmi on mmc.match_info_id = mmi.id " +
                " where mmc.del_flag = 0 and mmi.del_flag = 0 and mmc.user_id=:userId");
        Map<String, Object> params = new HashMap<>(16);
        params.put("userId", worker.getLoginId());

        if(req.getQuery().isQcFlag()){
            sql.append(" and mmi.qc_flag = 1 ");
            countSql.append(" and mmi.qc_flag = 1 ");
        }

        sql.append(" order by mmc.create_time desc");
        int total = baseNpJdbcTemplate.queryForInt(countSql.toString(), params);
        List<MthMyMatchListAppVo> result;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sql, req);
            result = baseNpJdbcTemplate.queryForBeans(MthMyMatchListAppVo.class, sql.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthMyMatchListAppVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    /**
     * 查询赛事通知
     * @param id 赛事ID
     * @return
     */
    public List<MatchArticleRspVo>queryMatchArticleByMatchId (Long id){
        String sql="select article_id,article_title,file_key from YCQ_ARTICLE_DETAIL where match_id=:id and state!=0 order by create_time desc";
        Map<String, Object> params = new HashMap<>();
        params.put("id",id);
        return baseNpJdbcTemplate.queryForBeans(MatchArticleRspVo.class,sql,params);
    }

    /**
     * 根据用户查询已报名场次
     * @param matchId 赛事id
     * @return
     */
    public List<MthRegister>findRegisterByUserId (Integer userId,Long matchId,String state){
        String sql="select * from mth_register  where del_flag=0 and match_id=:matchId and user_id=:userId and review_result !=:state ";
        Map<String, Object> params = new HashMap<>();
        params.put("matchId",matchId);
        params.put("userId",userId);
        params.put("state",state);
        return baseNpJdbcTemplate.queryForBeans(MthRegister.class,sql,params);
    }

    /**
     * 查询当前用户参加过的场次信息
     */
    public List<MthInstanceVo> findByMatchIdAndUserId(Long matchId,Integer userId){
        String sql="select a.area_name,c.* from mth_register a " +
                " left join mth_round_list b on a.id=b.register_id" +
                " left join mth_round_instance c on b.round_id=c.id" +
                " where a.del_flag=0 and b.del_flag=0 and c.del_flag=0 and a.match_id=:matchId and a.user_id=:userId " +
                " and c.round_state=0 and a.review_result in ('APPROVED','NOREVIEW')";
        Map<String, Object> params = new HashMap<>();
        params.put("matchId",matchId);
        params.put("userId",userId);
        return baseNpJdbcTemplate.queryForBeans(MthInstanceVo.class, sql, params);
    }

    /**
     * 查询该比赛记录评委点评信息
     */
    public List<MthMyMatchListDetailAppVo.Reviewer> findReviewer(Long registerId){
        String sql=" select a.id,b.comments,c.login_id as reviewerId,c.worker_name as reviewerName," +
                " c.signature_file_id as reviewerSign,c.signature_url as reviewerSignUrl " +
                " from mth_round_list a" +
                " left join mth_round_scoreinfo b on a.id=b.round_list_id " +
                " left join tsm_worker c on b.reviewer_id=c.login_id" +
                " where a.del_flag=0 and b.id is not null and a.register_id=:registerId and b.comments is not null";
        Map<String, Object> params = new HashMap<>();
        params.put("registerId",registerId);
        return baseNpJdbcTemplate.queryForBeans(MthMyMatchListDetailAppVo.Reviewer.class, sql, params);
    }
}
