package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.match.api.bo.MthMatchManagerReqBo;
import com.crunii.micro.service.match.api.bo.MthRegistrationTimeNodeReqBo;
import com.crunii.micro.service.match.api.bo.RegistrationTimeBo;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.enums.RoundEnums;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author xujie
 * @date 2023-09-23
 */
@Repository
public class MthRoundInsInstanceDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;


    public RegistrationTimeBo getRegistrationTime(Long id) {
        String sql = "SELECT MIN(registeration_start) AS registrationStartTime, MAX(registeration_end) AS registrationEndTime FROM mth_round_instance " +
                "WHERE round_sort = ( SELECT MIN(round_sort) FROM mth_round_instance where match_id = :id and del_flag = 0) and match_id = :id and del_flag = 0";
        Map<String, Object> map = new HashMap<>();
        map.put("id",id);
        return baseNpJdbcTemplate.queryForBean(RegistrationTimeBo.class,sql,map);
    }

    public List<MatchRoundTemplateInfoVo> getRoundTemplateInfos(Long id) {
        String sql = "select id,round_name      as name,round_sort as roundSort,match_level as level,case match_level\n" +
                "                   when 'city' then '市区级'\n" +
                "                   when 'district' then '片区级'\n" +
                "                   when 'county_school' then '区县/校赛级'\n" +
                "                   else '未知' end         as levelName,\n" +
                "       case play_mode\n" +
                "           when 'online' then '线上比赛'\n" +
                "           when 'offline' then '线下比赛'\n" +
                "           else '未知' end                              as playMode,\n" +
                "       case audit_mode\n" +
                "           when 'online' then '线上评审'\n" +
                "           when 'offline' then '线下评审'\n" +
                "           else '未知' end                              as auditMode,\n" +
                "       case promotion_mode\n" +
                "           when 'percent' then '百分比'\n" +
                "           when 'num' then '人数'\n" +
                "           else '其他' end                              as promotionMode,\n" +
                "       promption_num                                    as promptionNum,\n" +
                "       case entry_submit_mode\n" +
                "           when 'follow_register' then '随报名上传'\n" +
                "           when 'needless' then '不需要上传'\n" +
                "           when 'alone' then '单独上传'\n" +
                "           else '不需要上传' end                        as entrySubmitMode,\n" +
                "       concat(to_char(registeration_start, 'YYYY-MM-DD'), '至',\n" +
                "              to_char(registeration_end, 'YYYY-MM-DD')) as registerationTime,\n" +
                "       concat(to_char(round_start, 'YYYY-MM-DD'), '至',\n" +
                "              to_char(round_end, 'YYYY-MM-DD'))         as roundTime\n" +
                "from mth_round_instance\n" +
                "where del_flag = 0 and match_id = :matchId and round_type = :type";
        Map<String, Object> map = new HashMap<>();
        map.put("matchId", id);
        map.put("type", RoundEnums.RoundType.TEMPLATE.getCode());
        return baseNpJdbcTemplate.queryForBeans(MatchRoundTemplateInfoVo.class, sql, map);
    }

    public List<MatchRegisterFormVo> getregisterInfos(Long matchId) {
        String sql = "select id as registerId,config_type as configType,name from mth_register_config where del_flag = 0 and match_id = :matchId";
        Map<String, Object> map = new HashMap<>();
        map.put("matchId", matchId);
        return baseNpJdbcTemplate.queryForBeans(MatchRegisterFormVo.class,sql,map);
    }

    public MthRoundVo findMatchRound(MthMatchManagerReqBo req) {
        String sql = "select mri.* \n" +
                "from mth_round_instance mri\n" +
                "inner join mth_match_info mmi on mmi.id =mri.match_id\n" +
                "where mri.id=:round_id and mmi.id =:match_id";
        Map<String, Object> map = new HashMap<>();
        map.put("round_id", req.getMthRoundId());
        map.put("match_id", req.getMthMatchId());
        MthRoundVo matchBaseInfoVo = baseNpJdbcTemplate.queryForBean(MthRoundVo.class, sql, map);
        return matchBaseInfoVo;
    }

    public void finishMatchRound(MthRoundVo mthRoundInstanceVo) {
        String sql="update mth_round_instance set match_State=:matchState where id =:roundId and del_flag = 0";
        Map<String, Object> params = new HashMap<>();
        params.put("matchState", mthRoundInstanceVo.getMatchState());
        params.put("roundId", mthRoundInstanceVo.getId());
        baseNpJdbcTemplate.update(sql, params);
    }

    public MthRegistrationTimeNodeVo queryRegistrationProcessInfo(MthRegistrationTimeNodeReqBo req) {
        String sql = "select mr.user_id as user_id,\n" +
                "mri.match_id as match_id,mri.id as round_id,\n" +
                "mri.registeration_start as registeration_start, \n" +
                "mri.registeration_end as registeration_end,\n" +
                "mr.update_time as update_time,mr.create_time as register_time\n" +
                "from mth_round_instance mri\n" +
                "inner join mth_round_list mrl on mrl.round_id =mri.id\n" +
                "inner join mth_register mr on mr.id = mrl.register_id\n" +
                "where mri.match_id =:match_id  and mri.id=:round_id and mr.user_id =:user_id ";
        Map<String, Object> map = new HashMap<>();
        map.put("round_id", req.getRoundId());
        map.put("match_id", req.getMthMatchId());
        map.put("user_id", req.getUserId());
        MthRegistrationTimeNodeVo result = new MthRegistrationTimeNodeVo();
        List<MthRegistrationTimeNodeVo> matchBaseInfoVo = baseNpJdbcTemplate.queryForBeans(MthRegistrationTimeNodeVo.class, sql, map);
        if (ObjectUtil.isNotEmpty(matchBaseInfoVo)){
            result=matchBaseInfoVo.get(0);
        }
        return result;
    }
}
