package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.MthRoundInstance;
import com.crunii.micro.domain.entity.main.MthRoundReviewConf;
import com.crunii.micro.service.match.api.bo.MthReviewRoundBo;
import com.crunii.micro.service.match.api.vo.MthMatchAreaVo;
import com.crunii.micro.service.match.api.vo.MthContractorVo;
import com.crunii.micro.service.match.api.vo.MthReviewRoundVo;
import com.crunii.micro.service.match.api.vo.MthRoundInstanceVo;
import java.util.*;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

/**
* @description: TODO
* @author 叶明臻
* @date 2023/9/23 9:13
* @version 1.0
*/
@Slf4j
@Repository
public class MthRoundInstanceDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;
    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public List<MthReviewRoundVo> reviewRound(Long id, MthReviewRoundBo bo,MthRoundReviewConf mthRoundReviewConf) {
        String sql =  "select mi.match_name,mi.id as matchId,mr.round_state,mr.create_time,mr.match_state,mr.round_name,mr.id,count(mrg.id) as totalWorkNum,count(mrl.id)-count(mrs.id) as waitWorkNum " +
                "from mth_round_instance mr " +
                "left join mth_round_reviewer mrr on mrr.round_id=mr.id " +
                "left join mth_match_info mi on mr.match_id=mi.id " +
                "left join mth_round_list mrl on mrl.round_id=mr.id " +
                "left join mth_register mrg on mrg.id= mrl.register_id "+
                "left join mth_round_scoreinfo  mrs on mrs.round_list_id=mrl.id and mrs.status=1  and mrs.reviewer_id =?" +
                "where mrr.reviewer_id=? and mr.del_flag=0 and mr.round_state=0 " +
                "and (mr.match_state=2 or mr.match_state=1) and mr.audit_mode='online' " +
                "and (mrg.review_result='APPROVED' or mrg.review_result='NOREVIEW' ) ";

        StringBuilder sb = new StringBuilder(sql);
        List<Object> params = new ArrayList<>();
        params.add(id);
        params.add(id);
        //是否按赛项
        if(mthRoundReviewConf.getMatchItemFlag()==1){
            sb.append(" AND mrg.item_id =mrr.item_id");

        }
        if (StrUtil.isNotBlank(bo.getMatchName())) {
            sb.append(" and mi.match_name like ? ");
            params.add("%"+bo.getMatchName()+"%");
        }
        if (StrUtil.isNotBlank(bo.getRoundName())) {
            sb.append(" and mr.round_name like ? ");
            params.add("%"+bo.getRoundName()+"%");
        }
        sb.append(" group by mi.match_name,mi.id , mr.create_time,mr.round_state,mr.match_state,mr.round_name,mr.id  order by create_time desc");
        return jt.queryForBeans(MthReviewRoundVo.class, sb.toString(), params.toArray());
    }

    public List<String> findByAreaId(Long areaId,Long matchId) {
        String sql = "select area_name from mth_round_instance where area_id =  " + areaId + "and match_id = " + matchId;
        return jt.queryForRawObjects(String.class,sql);
    }

    public List<MthMatchAreaVo> queryTemplateArea(Long matchId) {
        String sql = "select mri.match_level, mra.id as area_id, mra.area_name, areaCount.totalCount, areaCount.nextLevelCount from mth_round_instance mri left join mth_round_area mra on mri.id = mra.round_id " +
                "left join (select ri.area_id, sum(IF(ri.round_sort = 1 AND (mr.review_result = 'APPROVED' OR mr.review_result = 'NOREVIEW'), 1, 0)) AS totalCount, sum(IF(ri.round_sort = arms.maxRoundSort AND mrs.RESULT = 1, 1, 0)) AS nextLevelCount from mth_round_instance ri " +
                "left join mth_round_list rl on ri.id = rl.round_id left join mth_register mr on rl.register_id = mr.id left join mth_round_score mrs on rl.id = mrs.round_list_id " +
                "left join (select area_id, max(round_sort) maxRoundSort from mth_round_instance where match_id = :matchId and del_flag=0 and round_type='instance' group by area_id) " +
                "arms ON ri.area_id = arms.area_id where ri.match_id = :matchId and ri.del_flag=0 and ri.round_type='instance' and mrs.status=1 group by ri.area_id) areaCount on areaCount.area_id = mra.id " +
                "where mra.del_flag = 0 and mri.del_flag=0 and mri.round_type='template' and mri.round_sort = 1 and mri.match_id = :matchId";
        Map<String, Object> params = new HashMap<>(16);
        params.put("matchId", matchId);
        List<MthMatchAreaVo> result = baseNpJdbcTemplate.queryForBeans(MthMatchAreaVo.class, sql, params);
        return result;
    }

    public MthRoundReviewConf getMthRoundReviewConf(int longid ){
               String sql=" SELECT mrvc.* FROM mth_round_review_conf mrvc LEFT JOIN mth_round_reviewer mrr ON mrvc.round_id =mrr.round_id WHERE  mrr.reviewer_id=? " ;
               return jt.queryForBean(MthRoundReviewConf.class, sql,longid);
    }

    public List<MthRoundInstanceVo> queryByEntryDeadline() {
        String sql = "select mri.*, mmi.match_name from mth_round_instance mri " +
                "left join mth_match_info mmi on mri.match_id = mmi.id " +
                "where mmi.del_flag = 0 and mri.del_flag=0 and mri.round_type='instance' and mri.entry_submit_mode = 'alone' AND mri.entry_deadline IS NOT NULL " +
                "AND (UNIX_TIMESTAMP(mri.entry_deadline) - UNIX_TIMESTAMP(SYSDATE())) BETWEEN 1 AND (60 * 60 * 24 * 1)";
        List<MthRoundInstanceVo> result = baseNpJdbcTemplate.queryForBeans(MthRoundInstanceVo.class, sql, null);
        return result;
    }

    public List<MthRoundInstanceVo> queryByRoundStart() {
        String sql = "select mri.*, mmi.match_name, mmsi.sign_address from mth_round_instance mri " +
                "left join mth_match_info mmi on mri.match_id = mmi.id " +
                "left join mth_match_sign_information mmsi on mri.id = mmsi.instance_id and mmsi.information_type = 1 " +
                "where mmi.del_flag = 0 and mri.del_flag=0 and mri.round_type='instance' and mri.round_start is not null " +
                "AND (UNIX_TIMESTAMP(mri.round_start) - UNIX_TIMESTAMP(SYSDATE())) BETWEEN 1 AND (60 * 60 * 24 * 1)";
        List<MthRoundInstanceVo> result = baseNpJdbcTemplate.queryForBeans(MthRoundInstanceVo.class, sql, null);
        return result;
    }

    public List<MthContractorVo> queryByRegisterationEnd() {
        String innerSql = "\n" +
                "SELECT mri.* FROM \n" +
                "\t(SELECT * FROM mth_round_instance \n" +
                "\t\tWHERE\n" +
                "\t\tdel_flag = 0\n" +
                "\t\tAND round_sort = 1\n" +
                "\t\tAND round_type = 'instance'\n" +
                "\t) mri\n" +
                "\tINNER JOIN (\n" +
                "\t\tSELECT\n" +
                "\t\t\tmri.match_id,\n" +
                "\t\t\tIF(MIN(ml.sort)=3,'city',IF(MIN(ml.sort)=2,'district','county_school')) match_level\n" +
                "\t\tFROM\n" +
                "\t\t\tmth_round_instance mri\n" +
                "\t\t\tLEFT JOIN (SELECT 'city' AS match_level, 3 AS sort\n" +
                "\t\t\t\t\t\tUNION ALL SELECT 'district' AS match_level, 2 AS sort\n" +
                "\t\t\t\t\t\tUNION ALL SELECT 'county_school' AS match_level, 1 AS sort\n" +
                "\t\t\t\t) ml ON mri.match_level = ml.match_level\n" +
                "\t\tWHERE\n" +
                "\t\t\tmri.del_flag = 0\n" +
                "\t\t\tAND mri.round_sort = 1\n" +
                "\t\t\tAND mri.round_type = 'instance'\n" +
                "\t\t\tAND mri.match_level IS NOT NULL\n" +
                "\t\t\tGROUP BY mri.match_id\n" +
                "\t) mris \n" +
                "\tON mri.match_id = mris.match_id AND mri.match_level = mris.match_level";
        String sql = "select mmc.match_id,mmi.match_name,mmc.contractor_id,mmc.limit_type from ("+innerSql+") mri " +
                "left join mth_match_info mmi on mri.match_id = mmi.id " +
                "inner join mth_match_contractor mmc on mmc.match_id = mmi.id and mmc.area_id = mri.area_id " +
                "where mmi.del_flag = 0 and mri.del_flag=0 and mri.round_type='instance' and mri.registeration_end IS NOT NULL " +
                "AND (UNIX_TIMESTAMP(mri.registeration_end) - UNIX_TIMESTAMP(SYSDATE())) BETWEEN (60 * 60 * 24 * 1.1) AND (60 * 60 * 24 * 2)";
        List<MthContractorVo> result = baseNpJdbcTemplate.queryForBeans(MthContractorVo.class, sql, null);
        return result;
    }

    public List<MthContractorVo> queryByRegisterationEndAfter() {
        String innerSql = "\n" +
                "SELECT mri.* FROM \n" +
                "\t(SELECT * FROM mth_round_instance \n" +
                "\t\tWHERE\n" +
                "\t\tdel_flag = 0\n" +
                "\t\tAND round_sort = 1\n" +
                "\t\tAND round_type = 'instance'\n" +
                "\t) mri\n" +
                "\tINNER JOIN (\n" +
                "\t\tSELECT\n" +
                "\t\t\tmri.match_id,\n" +
                "\t\t\tIF(MIN(ml.sort)=3,'city',IF(MIN(ml.sort)=2,'district','county_school')) match_level\n" +
                "\t\tFROM\n" +
                "\t\t\tmth_round_instance mri\n" +
                "\t\t\tLEFT JOIN (SELECT 'city' AS match_level, 3 AS sort\n" +
                "\t\t\t\t\t\tUNION ALL SELECT 'district' AS match_level, 2 AS sort\n" +
                "\t\t\t\t\t\tUNION ALL SELECT 'county_school' AS match_level, 1 AS sort\n" +
                "\t\t\t\t) ml ON mri.match_level = ml.match_level\n" +
                "\t\tWHERE\n" +
                "\t\t\tmri.del_flag = 0\n" +
                "\t\t\tAND mri.round_sort = 1\n" +
                "\t\t\tAND mri.round_type = 'instance'\n" +
                "\t\t\tAND mri.match_level IS NOT NULL\n" +
                "\t\t\tGROUP BY mri.match_id\n" +
                "\t) mris \n" +
                "\tON mri.match_id = mris.match_id AND mri.match_level = mris.match_level";
        String sql = "select mmc.match_id,mmi.match_name,mmc.contractor_id,mmc.limit_type from ("+innerSql+") mri " +
                "left join mth_match_info mmi on mri.match_id = mmi.id " +
                "inner join mth_match_contractor mmc on mmc.match_id = mmi.id and mmc.area_id = mri.area_id " +
                "where mmi.del_flag = 0 and mri.del_flag=0 and mri.round_type='instance' and mri.registeration_end IS NOT NULL " +
                "AND (UNIX_TIMESTAMP(SYSDATE()) - UNIX_TIMESTAMP(mri.registeration_end)) BETWEEN 1 AND (60 * 60 * 24 * 1)";
        List<MthContractorVo> result = baseNpJdbcTemplate.queryForBeans(MthContractorVo.class, sql, null);
        return result;
    }

    public List<MthContractorVo> queryByEntryDeadlineAfter() {
        String sql = "select mmc.match_id,mmi.match_name,mmc.contractor_id,mmc.limit_type from mth_round_instance mri " +
                "left join mth_match_info mmi on mri.match_id = mmi.id " +
                "inner join mth_match_contractor mmc on mmc.match_id = mmi.id and mmc.area_id = mri.area_id " +
                "where mmi.del_flag = 0 and mri.del_flag=0 and mri.round_type='instance' and mri.entry_submit_mode = 'alone' AND mri.entry_deadline IS NOT NULL " +
                "AND (UNIX_TIMESTAMP(SYSDATE()) - UNIX_TIMESTAMP(mri.entry_deadline)) BETWEEN 1 AND (60 * 60 * 24 * 1)";
        List<MthContractorVo> result = baseNpJdbcTemplate.queryForBeans(MthContractorVo.class, sql, null);
        return result;
    }

    public List<MthRoundInstance> findByRoundTypeAndMatchStateAndPublicityDeadLine() {
        String sql = "select * from mth_round_instance where round_type ='instance' and del_flag=0 and match_state='25' and publicity_deadline < now()";
        return jt.queryForBeans(MthRoundInstance.class,sql);
    }


    public void updateMatchStateByIdIn(String roundIdStr) {
        //将公示中的场次流转为公示结束
        String sql="update mth_round_instance set match_state='28' where id in (" + roundIdStr + " )";
        Integer num = jt.update(sql);
        if (num > 0){
            log.info("将公示中场次状态流转为公示结束：{}",num);
        }
    }

    public static void main(String[] args) {
        Date end = DateUtil.parse("2023-12-20 12:00:00");
        long l = end.getTime() - System.currentTimeMillis();
        System.out.println(l);
    }
}
