package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.nacos.common.utils.StringUtils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.MthRoundList;
import com.crunii.micro.domain.entity.main.MthRoundReviewConf;
import com.crunii.micro.service.match.api.bo.MthRoundRankingListBo;
import com.crunii.micro.service.match.api.bo.OrderByBo;
import com.crunii.micro.service.match.api.vo.MatchGroupRoundNameVo;
import com.crunii.micro.service.match.api.vo.MthRoundRankingListVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author baoxy
 * @date 2023-06-26
 */
@Repository
public class MthRoundListDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public static final String MATCH_SEARCH_SQL = "select mrl.id as roundListId ,mrg.item_id,mrg.item_name,mrg.group_id,mrg.group_name,  mw.id, mw.work_code,mw.work_name,mw.create_time as workSignTime,mmi.match_name,mri.round_name,mri.socre_mode,COALESCE(mrs.status, 0) as  workReviewStatus,mrs.score,mrs.create_time as   review_time ,mri.id as roundId ,mmi.id as matchId " +
            "from mth_round_list mrl " +
            "left join mth_register mrg on mrg.id= mrl.register_id "+
            "left join mth_work mw on mrl.work_id = mw.id " +
            "left join mth_round_instance mri on mri.id = mrl.round_id " +
            "left join mth_match_info mmi on mmi.id=mri.match_id " +
            "left join mth_round_scoreinfo mrs on mrs.round_list_id = mrl.id and   mrs.reviewer_id= ?" +
             "LEFT JOIN  mth_round_reviewer mrr ON mri.id=mrr.round_id "+
            "where  mrr.reviewer_id=? and  mrl.round_id= ?  and mrl.del_flag=0 " +
            "and (mrg.review_result='APPROVED' or mrg.review_result='NOREVIEW' ) ";
    private static final String PAGE_COUNT_SQL =  "select count(mrl.id) " +
            "from mth_round_list mrl " +
            "left join mth_register mrg on mrg.id= mrl.register_id "+
            "left join mth_work mw on mrl.work_id = mw.id " +
            "left join mth_round_instance mri on mri.id = mrl.round_id " +
            "left join mth_match_info mmi on mmi.id=mri.match_id " +
            "left join mth_round_scoreinfo mrs on mrs.round_list_id =mrl.id  and mrs.reviewer_id= ? " +
            "LEFT JOIN  mth_round_reviewer mrr ON mri.id=mrr.round_id "+
            "where mrr.reviewer_id=? and mrl.round_id= ?  and mrl.del_flag=0 "+
            "and (mrg.review_result='APPROVED' or mrg.review_result='NOREVIEW' )";

    private static final String DELETE_SQL = "update mth_round_list set del_flag=1, update_id=:updateId, update_by=:updateBy, update_time=:updateTime where id in (:ids)";



    public PageRsp<MthRoundRankingListVo>       pageRankingListByRound(PageReq<MthRoundRankingListBo> req, Worker worker, MthRoundReviewConf mthRoundReviewConf, boolean export) {
        StringBuilder sb = new StringBuilder(MATCH_SEARCH_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        params.add(worker.getLoginId());
        params.add(worker.getLoginId());
        params.add(req.getQuery().getRoundId());
        //是否按赛项
        if(mthRoundReviewConf.getMatchItemFlag()==1){
            sb.append(" AND mrg.item_id =mrr.item_id");
            sbCount.append(" AND mrg.item_id =mrr.item_id ");
        }
        if (StrUtil.isNotBlank(req.getQuery().getWorkCode())) {
            sb.append(" and mw.work_code like ? ");
            sbCount.append(" and mw.work_code like ? ");
            params.add("%"+req.getQuery().getWorkCode()+"%");
        }
        if (StrUtil.isNotBlank(req.getQuery().getMatchName())) {
            sb.append(" and mmi.match_name like ? ");
            sbCount.append(" and mmi.match_name like ? ");
            params.add("%"+req.getQuery().getMatchName()+"%");
        }
        if (StrUtil.isNotBlank(req.getQuery().getRoundName())) {
            sb.append(" and mri.round_name like ? ");
            sbCount.append(" and mri.round_name like ? ");
            params.add("%"+req.getQuery().getRoundName()+"%");
        }
        if (ObjectUtil.isNotNull(req.getQuery().getWorkStatus())) {
            sb.append(" and mrs.status = ? ");
            sbCount.append(" and mrs.status = ? ");
            params.add(req.getQuery().getWorkStatus());
        }
        if (Objects.nonNull(req.getQuery().getWorkSignTimeEnd())) {
            sb.append("and mw.create_time <=  to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append("and mw.create_time <=  to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(req.getQuery().getWorkSignTimeEnd());
        }
        if (Objects.nonNull(req.getQuery().getWorkSignTimeStart())) {
            sb.append("and mw.create_time >=  to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append("and mw.create_time >=  to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(req.getQuery().getWorkSignTimeStart());
        }
        if (Objects.nonNull(req.getQuery().getReviewTimeStart())) {
            sb.append("and mrs.create_time >=  to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append("and mrs.create_time >=  to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(req.getQuery().getReviewTimeStart() );
        }
        if (Objects.nonNull(req.getQuery().getReviewTimeEnd())) {
            sb.append("and mrs.create_time <=  to_date(?,'yyyy-mm-dd hh24:mi:ss')  ");
            sbCount.append("and mrs.create_time <=  to_date(?,'yyyy-mm-dd hh24:mi:ss')  ");
            params.add(req.getQuery().getReviewTimeEnd() );
        }
        if (Objects.nonNull(req.getQuery().getItemId())){
            sb.append(" and mrg.item_id=? ");
            sbCount.append(" and mrg.item_id=? ");
            params.add(req.getQuery().getItemId());
        }
        if (Objects.nonNull(req.getQuery().getGroupId())){
            sb.append(" and mrg.group_id=? ");
            sbCount.append(" and mrg.group_id=? ");
            params.add(req.getQuery().getGroupId());
        }

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<MthRoundRankingListVo> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {

          //排序
           if (!CollectionUtil.isEmpty(req.getQuery().getOrderByBo())){
               List<OrderByBo> orderByBo = req.getQuery().getOrderByBo();
               String collect = orderByBo.stream().map(data -> {
                   String oder="";
                   if (data.getOrderBy()!=null) {
                       if (data.getOrderBy()==1) {
                           oder= " mrg.item_id ";
                       }
                       if (data.getOrderBy()==2) {
                           oder= " mrg.group_id ";
                       }
                   }
                   return oder;
               }).collect(Collectors.joining(","));
               if (StringUtils.isNoneBlank(collect)) {
                   sb.append(" ORDER BY" + collect);
               }
           }
           //不导出分页
           if (!export) {
               //根据数据库类型设置limit
               SqlUtil.appendLimit(dbType, params, sb, req);
           }
            result = jdbcTemplate.queryForBeans(MthRoundRankingListVo.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthRoundRankingListVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        if (!export) {
            page.setPageSize(req.getPageSize());
            page.setTotalNum(total);
            // 为手机计算是否由下一页
            page.next();
        }
        page.setResult(result);

        return page;
    }

    public void deleteByIds(Collection<Long> ids, Integer updateId, String updateBy) {
        Map<String, Object> params = new HashMap<>(4);
        params.put("ids", ids);
        params.put("updateId", updateId);
        params.put("updateBy", updateBy);
        params.put("updateTime", new Date());
        baseNpJdbcTemplate.update(DELETE_SQL, params);
    }

    public List<MthRoundList> queryPromotionList(Long roundId) {
        // 查询晋级名单
        String sql = "SELECT\n" +
                "\tmrl.* \n" +
                "FROM\n" +
                "\tmth_round_score mrs\n" +
                "\tINNER JOIN mth_round_list mrl ON mrl.id = mrs.round_list_id \n" +
                "WHERE\n" +
                "\tmrs.result = 1 \n" +
                "\tAND mrl.round_id =:roundId";
        StringBuilder sb = new StringBuilder(sql);
        Map<String, Object> params = new HashMap<>(16);
        params.put("roundId", roundId);
        List<MthRoundList> result = baseNpJdbcTemplate.queryForBeans(MthRoundList.class, sb.toString(), params);
        return result;
    }

    public List<MthRoundList> queryRoundListByRegisterId(Long registerId) {
        String sql = "select * from mth_round_list mrl where mrl.del_flag = 0 and mrl.register_id = :registerId order by mrl.create_time desc";
        Map<String, Object> params = new HashMap<>(16);
        params.put("registerId", registerId);
        List<MthRoundList> result = baseNpJdbcTemplate.queryForBeans(MthRoundList.class, sql, params);
        return result;
    }

    public List<MthRoundList> queryNotUploadedWorkUser(List<Long> roundIds) {
        String sql = "select * from mth_round_list mrl " +
                "left join mth_register mr on mrl.register_id = mr.id " +
                "where mrl.del_flag = 0 and mr.del_flag = 0 and mr.review_result in ('APPROVED','NOREVIEW') " +
                "and mrl.work_id is null " +
                "and mrl.round_id in (:roundIds) order by mrl.create_time desc";
        Map<String, Object> params = new HashMap<>(16);
        params.put("roundIds", roundIds);
        List<MthRoundList> result = baseNpJdbcTemplate.queryForBeans(MthRoundList.class, sql, params);
        return result;
    }

    public List<MthRoundList> queryRoundUser(List<Long> roundIds) {
        String sql = "select * from mth_round_list mrl " +
                "left join mth_register mr on mrl.register_id = mr.id " +
                "where mrl.del_flag = 0 and mr.del_flag = 0 and mr.review_result in ('APPROVED','NOREVIEW') " +
                "and mrl.round_id in (:roundIds) order by mrl.create_time desc";
        Map<String, Object> params = new HashMap<>(16);
        params.put("roundIds", roundIds);
        List<MthRoundList> result = baseNpJdbcTemplate.queryForBeans(MthRoundList.class, sql, params);
        return result;
    }
}
