package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.MthRoundRankingListBo;
import com.crunii.micro.service.match.api.vo.MthRoundRankingListVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author baoxy
 * @date 2023-06-26
 */
@Repository
public class MthRoundReviewStandarDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    public static final String MATCH_SEARCH_SQL = "select mw.*,mw.create_time as workSignTime,mmi.match_name,mr.round_name,mrs.status,mrs.score,mrs.reviewTime " +
            "from mth_round_list mrl " +
            "left join mth_work mw on mrl.work_id = mw.id " +
            "left join mth_round mr on mr.id = mrl.round_id " +
            "left join mth_match_info mmi on mmi.id=mr.match_id " +
            "left join mth_round_score mrs on mrs.id = mrl.id" +
            "where mrl.round_id= ? and  mw.del_flag=0 and mrl.del_flag=0";
    private static final String PAGE_COUNT_SQL =  "select count(mw.*)" +
            "from mth_round_list mrl " +
            "left join mth_work mw on mrl.work_id = mw.id " +
            "left join mth_round mr on mr.id = mrl.round_id " +
            "left join mth_match_info mmi on mmi.id=mr.match_id " +
            "left join mth_round_score mrs on mrs.id = mrl.id" +
            "where mrl.round_id= ? and  mw.del_flag=0 and mrl.del_flag=0";


    public PageRsp<MthRoundRankingListVo> pageRankingListByRound(PageReq<MthRoundRankingListBo> req, Worker worker) {
        StringBuilder sb = new StringBuilder(MATCH_SEARCH_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        if (StrUtil.isNotBlank(req.getQuery().getWorkCode())) {
            sb.append(" and mw.work_code = ? ");
            sbCount.append(" and mw.work_code = ? ");
            params.add(req.getQuery().getWorkCode());
        }
        if (StrUtil.isNotBlank(req.getQuery().getMatchName())) {
            sb.append(" and mmi.match_name = ? ");
            sbCount.append(" and mmi.match_name = ? ");
            params.add(req.getQuery().getMatchName());
        }
        if (ObjectUtil.isNotNull(req.getQuery().getWorkStatus())) {
            sb.append(" and mrs.status = ? ");
            sbCount.append(" and mrs.status = ? ");
            params.add(req.getQuery().getWorkStatus());
        }
        if (Objects.nonNull(req.getQuery().getWorkSignTimeEnd())) {
            sb.append("and mw.workSignTime between  to_date(?,'yyyy-mm-dd hh24:mi:ss') and to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" a.workSignTime between  to_date(?,'yyyy-mm-dd hh24:mi:ss') and to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(req.getQuery().getWorkSignTimeEnd() + " 00:00:00");
        }
        if (Objects.nonNull(req.getQuery().getWorkSignTimeStart())) {
            sb.append("and mw.workSignTime between  to_date(?,'yyyy-mm-dd hh24:mi:ss') and to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" mw.workSignTime between  to_date(?,'yyyy-mm-dd hh24:mi:ss') and to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(req.getQuery().getWorkSignTimeStart() + " 23:59:59");
        }
        if (Objects.nonNull(req.getQuery().getReviewTimeStart())) {
            sb.append("and mw.workSignTime between  to_date(?,'yyyy-mm-dd hh24:mi:ss') and to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" mw.workSignTime between  to_date(?,'yyyy-mm-dd hh24:mi:ss') and to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(req.getQuery().getReviewTimeStart() + " 23:59:59");
        }
        if (Objects.nonNull(req.getQuery().getReviewTimeEnd())) {
            sb.append("and mw.workSignTime between  to_date(?,'yyyy-mm-dd hh24:mi:ss') and to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" mw.workSignTime between  to_date(?,'yyyy-mm-dd hh24:mi:ss') and to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(req.getQuery().getReviewTimeEnd() + " 23:59:59");
        }

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<MthRoundRankingListVo> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(MthRoundRankingListVo.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthRoundRankingListVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

}
