package com.crunii.micro.service.match.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.MthRoundList;
import com.crunii.micro.domain.entity.main.MthRoundScore;
import com.crunii.micro.service.match.api.bo.MthMatchManagerReqBo;
import com.crunii.micro.service.match.api.bo.MthRoundScoreRankingBo;
import com.crunii.micro.service.match.api.vo.*;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author baoxy
 * @date 2023-06-26
 */
@Repository
public class MthRoundScoreDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    //排序字段score 可能是通过不通过、ABCDE
    public static final String MATCH_SEARCH_SQL = "select mrs.*,mw.work_code,mw.work_name from mth_round_score mrs left join mth_work mw on mw.round_list_id=mrs.round_list_id where mrs.round_list_id in(:ids) and mrs.del_flag=0 and mw.del_flag=0 order by mrs.score,mw.create_time";
    public static final String MATCH_SEARCH_COUNT_SQL = "select count(*) from mth_round_score mrs left join mth_work mw on mw.round_list_id=mrs.round_list_id where mrs.round_list_id in(:ids) and mrs.del_flag=0 and mw.del_flag=0 order by mrs.score,mw.create_time";

    public PageRsp<MthRoundScoreRankingVo> getRoundScoreRankingList(List<Long> roundListIds, PageReq<MthRoundScoreRankingBo> bo) {
        StringBuilder sb = new StringBuilder(MATCH_SEARCH_SQL);
        StringBuilder sbCount = new StringBuilder(MATCH_SEARCH_COUNT_SQL);
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", roundListIds);

        int total = baseNpJdbcTemplate.queryForInt(sbCount.toString(), params);
        String dbType = baseNpJdbcTemplate.getDbType();
        List<MthRoundScoreRankingVo> result;
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, bo);

            result = baseNpJdbcTemplate.queryForBeans(MthRoundScoreRankingVo.class, sb.toString(), params);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<MthRoundScoreRankingVo> page = new PageRsp<>();
        page.setPageNo(bo.getPageNo());
        page.setPageSize(bo.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<MthRoundScore> getRankingSortListByRoundIds(String roundId) {
        String sql="select mrs.* from mth_round_score mrs " +
                "left join mth_round_list mrl on mrs.round_list_id=mrl.id " +
                "where mrl.round_id=? and mrs.status=1 and mrs.del_flag=0 and mrs.del_flag=0";
        List<Object> params = new ArrayList<>();
        params.add(roundId);
        List<MthRoundScore> mthRoundScores = jdbcTemplate.queryForBeans(MthRoundScore.class, sql, params.toArray());
        return mthRoundScores;
    }


    public List<Long> getReceviePrizeList(Long roundId) {
        String sql=" select mrl.* from mth_round_score mrs\n" +
                "                left join mth_round_list mrl on mrs.round_list_id=mrl.id \n" +
                "                where mrl.round_id=:roundId  and mrs.del_flag=0 and mrs.del_flag=0 \n" +
                "                and ((mrs.prize_id is not null and mrs.prize_id!='') or (mrs.other_prize_id is not null and mrs.other_prize_id!=''))";
        Map<String, Object> paramMap = new HashMap<>(2);
        paramMap.put("roundId",roundId);
        List<MthRoundList> roundList = baseNpJdbcTemplate.queryForBeans(MthRoundList.class, sql, paramMap);
        return roundList.stream().map(MthRoundList::getUserId).collect(Collectors.toList());
    }

    public void updatePrizeMsgFlag(Long roundId){
        try {
            String sql = "update mth_round_score set prize_msg_flag=1 where round_list_id in(\n" +
                    "select id from mth_round_list where round_id=:roundId" +
                    ")and ((prize_id is not null and prize_id!='') or (other_prize_id is not null and other_prize_id!='')) ";
            Map<String, Object> paramMap = new HashMap<>(2);
            paramMap.put("roundId",roundId);
            baseNpJdbcTemplate.update(sql,paramMap);
        } catch (DataAccessException e) {
            e.printStackTrace();
        }
    }


    public List<MthRoundScore> getRankingSortListByRoundId(String roundId) {
        String sql="select mrs.* from mth_round_score mrs " +
                "left join mth_round_list mrl on mrs.round_list_id=mrl.id " +
                "where mrl.round_id=? and mrs.status=1 and mrs.del_flag=0 and mrs.del_flag=0 and RESULT='1' ";
        List<Object> params = new ArrayList<>();
        params.add(roundId);
        List<MthRoundScore> mthRoundScores = jdbcTemplate.queryForBeans(MthRoundScore.class, sql, params.toArray());
        return mthRoundScores;
    }

    /***
     * 查看获奖名单
     */
    public List<MthAwardInfoVo> queryAwardList(MthMatchManagerReqBo bo) {
        String sql="SELECT\n" +
                "\tmrl.user_id,mr.item_name,mr.group_name,\n" +
                "\tmrl.user_name,\n" +
                "\tmrs.score,\n" +
                "\tmrs.sort,\n" +
                "\tmrs.result,\n" +
                "\tmrs.prize_id,\n" +
                "\tmw.work_code,\n" +
                "\twork_name,\n" +
                "\tmri.area_id,\n" +
                "\tmri.area_name,\n" +
                "CASE\n" +
                "\t\tWHEN nullif( mrs.prize_name, '' ) IS NOT NULL \n" +
                "\t\tAND nullif( mrs.other_prize_name, '' ) IS NOT NULL THEN\n" +
                "\t\t\tmrs.prize_name || '、' || mrs.other_prize_name ELSE COALESCE (\n" +
                "\t\t\t\tnullif( mrs.prize_name, '' ),\n" +
                "\t\t\tnullif( mrs.other_prize_name, '' )) \n" +
                "\t\tEND AS prize_name\n" +
                "\tFROM\n" +
                "\t\tmth_round_score mrs\n" +
                "\t\tINNER JOIN mth_round_list mrl ON mrl.id = mrs.round_list_id\n" +
                "\t\tINNER JOIN mth_round_instance mri ON mri.id = mrl.round_id\n" +
                "\t\tLEFT JOIN mth_work mw ON mw.id = mrl.work_id left join mth_register mr on mrl.register_id=mr.id \n" +
                "\tWHERE\n" +
                 "((mrs.prize_name is not null and mrs.prize_name!='')\n" +
                "   or (mrs.other_prize_name is not null and mrs.other_prize_name!=''))\n" +
                "\n" +
                "\t\tAND mri.id =? "+
                "ORDER BY\n" +
                "\tmrs.score";
        List<Object> params = new ArrayList<>();
        params.add(bo.getMthRoundId());
        List<MthAwardInfoVo> mthRoundScores = jdbcTemplate.queryForBeans(MthAwardInfoVo.class, sql, params.toArray());
        return mthRoundScores;
    }

    public List<MthRoundList> findByProps(Long roundId) {
        String sql="select mrl.* \n" +
                "from mth_round_list mrl \n" +
                "         left join  mth_register mrg on mrl.register_id =mrg.id \n" +
                "         where mrl.round_id=? and mrl.del_flag=0 and (mrg.review_result='APPROVED' or mrg.review_result='NOREVIEW')";
        List<Object> params = new ArrayList<>();
        params.add(roundId);
        List<MthRoundList> mthRoundScores = jdbcTemplate.queryForBeans(MthRoundList.class, sql, params.toArray());
        return mthRoundScores;

    }

    /**
     * @description: 统计场次上除当前页面的外的奖项设置数量
     * @param roundId 场次id
     * @return: 
     * @author liuchuan
     * @date: 2023/11/29 10:05
    */
    public Map<Long,Integer> prizeCountByRoundId(Long roundId,List<Long> idList){


        String sql = " select s.prize_id as prizeId,s.other_prize_id as otherPrizeId,count(s.id) as countNum  from mth_round_list l left join mth_round_score s on l.id=s.round_list_id " +
                " where ((S.prize_id is not null and s.prize_id !='') or (s.other_prize_id is not null or s.other_prize_id!=''))" +
                " and l.round_id= :roundId " +
                " and s.id not in ( :ids )  and( (s.prize_id is not null and s.prize_id!='') or (s.prize_id is not null and s.prize_id!=''))" +
                "  group by s.prize_id,s.other_prize_id order by countNum desc ";

        Map<String, Object> paramMap = new HashMap<>(2);
        paramMap.put("roundId",roundId);
        paramMap.put("ids",idList);

        List<MthRoundPrizeNumVo> numList = baseNpJdbcTemplate.queryForBeans(MthRoundPrizeNumVo.class,sql,paramMap);
        HashMap<Long,Integer> map = new HashMap<Long,Integer>();
        if (CollectionUtil.isNotEmpty(numList)){
            numList.stream().forEach(vo->{
                //保存奖项
                if (ObjectUtil.isNotEmpty(vo.getPrizeId())){
                    if (map.containsKey(vo.getPrizeId())){
                        map.put(Long.valueOf(vo.getPrizeId()),map.get(Long.valueOf(vo.getPrizeId()))+vo.getCountNum());
                    }else {
                        map.put(Long.valueOf(vo.getPrizeId()),vo.getCountNum());
                    }
                }
                //保存其他奖项
                if (ObjectUtil.isNotEmpty(vo.getOtherPrizeId())){
                    if (map.containsKey(vo.getOtherPrizeId())){
                        map.put(Long.valueOf(vo.getOtherPrizeId()),map.get(Long.valueOf(vo.getOtherPrizeId()))+vo.getCountNum());
                    }else {
                        map.put(Long.valueOf(vo.getOtherPrizeId()),vo.getCountNum());
                    }
                }

            });
        }
        return map;

    }


    public List<MthRoundReviewStandarVo> getMthRoundScoreinfoByStandar(Long id) {
        String sql=" SELECT mrrs.* ,msd.socre AS socre FROM mth_round_score_detail msd LEFT JOIN mth_round_review_standar mrrs " +
                "  ON msd.standar_id=mrrs.id  WHERE  msd.score_id=?";
        return  jdbcTemplate.queryForBeans(MthRoundReviewStandarVo.class,sql,id);
    }
}
