package com.crunii.micro.service.match.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.MthMatchGroup;
import com.crunii.micro.domain.entity.main.MthMatchItem;
import com.crunii.micro.feign.client.dto.ValueC;
import com.crunii.micro.feign.client.dto.ValueU;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.dto.RoundColumVo;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 赛事信息主;Service接口
 *
 * @author mth
 * @date 2023-09-19
 */
public interface IMthMatchInfoService {

    /**
     * 查询赛事信息主;
     */
    MthMatchInfoVo queryById(Long id);

    /**
     * 查询赛事信息主;列表
     */
    Result<PageRsp<MthMatchInfoVo>> queryPageList(PageReq<MthMatchInfoBo> bo);

    /**
     * APP查询赛事列表
     */
    Result<PageRsp<MthMatchInfoAppVo>> queryPageListForApp(PageReq<MthMatchInfoAppBo> bo);

    /**
     * APP查询我的赛事列表
     * @param bo
     * @return
     */
    Result<PageRsp<MthMyMatchListAppVo>> myMatchListForApp(PageReq<MthMyMatchListAppBo> bo);

    /**
     * 查询赛事信息主;列表
     */
    List<MthMatchInfoVo> queryList(MthMatchInfoBo bo);

    /**
     * 修改赛事信息主;
     */
    Long insertByBo(MthMatchInfoBo bo);

    /**
     * 修改赛事信息主;
     */
    Boolean updateByBo(MthMatchInfoBo bo);

    /**
     * 赛事-保存草稿
     */
    Long saveDraft(MthMatchInfoBo bo);

    /**
     * 校验并批量删除赛事信息
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);

    PageRsp<MthMatchInfoVo> publishList(PageReq<MatchPublishListBo> bo);

    /**
     * 比赛管理列表页
     */
    PageRsp<mthMatchManagerVo> mthMatchManagerList(PageReq<MthMatchManagerReqBo> bo);

    /**
     * 比赛管理列表  比赛详情查看
     */
    List<mthMatchManagerDetailVo> queryMatchManagerDetailList(MthMatchManagerReqBo req);

    List<mthMatchManagerDetailVo> mthMatchManagerDetailList(MthMatchManagerReqBo bo);

    /**
     *发布赛事
     * @param ids
     * @return
     */
    Boolean publishMatch(List<Long> ids);

    /**
     * 下架赛事
     * @param asList
     * @return
     */
    Boolean soldOutMatch(List<Long> asList);

    /**
     * 查询赛项组别
     * @param matchId
     * @return
     */
    List<MatchBaseItemVo> queryItemList(Long matchId);

    /**
     * 预览比赛-基本信息
     * @param id
     * @return
     */
    MatchBaseInfoVo matchBaseInfo(Long id);

    List<MatchRoundTemplateInfoVo> matchProcess(Long matchId);

    List<MatchRegisterFormVo> matchApplicationForm(Long matchId);

    List<MatchStatusVo> matchStatus();

    /**
     * @description: 插入app端比赛配置
     * @param bo
     * @return:
     * @author liuchuan
     * @date: 2023/9/27 15:05
    */
    boolean insertAppMatchConfig(MthMatchConfigBo bo);

    /**
     * @description: 查询app配置
     * @param matchId 比赛id
     * @return:
     * @author liuchuan
     * @date: 2023/10/6 10:04
    */
    MthMatchConfigVo getAppMatchConfig(Long matchId);

    /**
     * @description: 插入app端比赛配置而且提交
     * @param bo
     * @return:
     * @author liuchuan
     * @date: 2023/9/27 20:00
    */
    boolean matchSubmit(MthMatchSubmitBo bo);

    MthMyMatchListDetailAppVo queryMatchRegisterDetail(MthMatchManagerRegisterReqBo bo);

    MthMyMatchListDetailAppVo queryMatchReviewDetail(MthMatchManagerReviewReqBo bo);

    Boolean matchRegisterExamine(MthMatchManagerRegisterReqBo bo);

    PageRsp<MthMatchManagerRegisterVo> queryMatchRegisterList(PageReq<MthMatchManagerRegisterReqBo> req);

    void exportRegisterList(PageReq<MthMatchManagerRegisterReqBo> req);

    PageRsp<MthMatchManagerReviewVo>  queryMatchReviewList(PageReq<MthMatchManagerReviewReqBo> req);

    void exportReviewList(PageReq<MthMatchManagerReviewReqBo> req);

    PageRsp<MthMatchManagerPromotedVo>  queryMatchPromotedList(PageReq<MthMatchManagerPromotedReqBo> req);

    void exportPromotedList(PageReq<MthMatchManagerPromotedReqBo> req);

    PageRsp<MthMatchManagerPublicityVo>  queryMatchPublicityList(PageReq<MthMatchManagerPublicityReqBo> req);

    void exportPublicityList(PageReq<MthMatchManagerPublicityReqBo> req);

    PageRsp<MthMatchManagerContestantVo>  queryMatchContestantList(PageReq<MthMatchManagerContestantReqBo> req);

    void exportContestantList(PageReq<MthMatchManagerContestantReqBo> req);

    PageRsp<MthRoundDeliveryVo> getDeliveryInfo(PageReq<MthDeliveryBo> req);

    void exportDelivery(PageReq<MthDeliveryBo> req);

    boolean updateDlieryFlag(MthRoundDeliveryBo bo);

    PageRsp<MthWorkPromotionVo> queryWorkPromotionManagerList(PageReq<MthWorkPromiseReqBo> bo);

    List<MthMatchProcessVo> queryProcessInfo(MthMatchProcessReqBo bo);

     void validCategoryIdExistsUse(Long categoryId);

     void validLabelUse(Long labelId);

     void validPrizeUse(Long prizeId);

    boolean matchRoundFinish(MthMatchManagerReqBo req);

    /**
     * @description: 公示结束
     * @param req
     * @return:
     * @author liuchuan
     * @date: 2023/11/22 14:31
    */
    boolean matchRoundPulicityFinish(MthMatchManagerReqBo req);

    MthRegistrationTimeNodeVo queryRegistrationProcessInfo(MthRegistrationTimeNodeReqBo req);

    MthRoundInstanceVo queryRoundInfoById(Long id);

    /**
     * @description: 查询比赛下的赛项
     * @param matchId 比赛id
     * @return:
     * @author liuchuan
     * @date: 2023/11/23 16:00
    */
    List<MthMatchItem> getItemListByMatchId(Long matchId);

    /**
     * @description: 查询赛项下的组别
     * @param itemId 赛项id
     * @return:
     * @author liuchuan
     * @date: 2023/11/23 16:00
    */
    List<MthMatchGroup> getGroupListByItemId(Long itemId);

    /**
     * APP赛事活动-查询赛事列表-查看详情
     * @param bo
     * @return
     */
    Result<MthMatchInfoAppVo> doListDetails(Worker worker,MthMatchInfoAppBo bo);


    /**
     * @description: 青呼我为 banner图
     * @param
     * @return:
     * @author liuchuan
     * @date: 2023/12/19 17:50
    */
    List<MatchBannerVo> queryBanner();

    /**
     * 团重庆市委机关团委组织以及下级组织可以创建市级比赛
     * @param workerId
     * @return
     */
    boolean queryCityLevelFlag(Long workerId);
    /**
     *报名审核原因新增
     */
    void crateCompetitionReview(ValueC valueC);

    /**
     * 报名审核原因修改
     */
    void updateCompetitionReview(ValueU valueU);

    /**
     * 场次参赛着列表表头信息
     * @return
     */
    RoundColumVo roundColum(MthMatchManagerContestantReqBo mthMatchManagerContestantReqBo);

    RoundColumVo registerInfoColum(MthMatchManagerRegisterReqBo req);

    Long copyMatch(Long matchId);
}
