package com.crunii.micro.service.match.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.MthRegister;
import com.crunii.micro.service.match.api.bo.MatchMyCollectBo;
import com.crunii.micro.service.match.api.bo.MthRegisterAddAppBo;
import com.crunii.micro.service.match.api.bo.MthRegisterBo;
import com.crunii.micro.service.match.api.bo.MthCancelRegisterAppBo;
import com.crunii.micro.service.match.api.vo.*;

import java.io.IOException;
import java.util.Collection;
import java.util.List;

/**
 * 赛事-赛项-组别-区域报名信息;Service接口
 *
 * @author mth
 * @date 2023-09-19
 */
public interface IMthRegisterService {

    /**
     * 查询赛事-赛项-组别-区域报名信息;
     */
    MthRegisterVo queryById(Long id);

    /**
     * 查询赛事-赛项-组别-区域报名信息;列表
     */
    Result<PageRsp<MthRegisterVo>> queryPageList(PageReq<MthRegisterBo> bo);

    /**
     * 查询赛事-赛项-组别-区域报名信息;列表
     */
    List<MthRegisterVo> queryList(MthRegisterBo bo);


    /**
     * APP查询报名范围
     * @param matchId
     * @return
     */
    List<MthRegisterAreaAppVo> queryRegisterAreaListForApp(Long matchId);

    /**
     * APP查询报名表配置
     * @param matchId
     * @param roundInstanceId
     * @param registerId
     * @return
     */
    MthRegisterConfigAppVo queryRegisterConfigForApp(Long matchId, Long roundInstanceId, Long registerId);

    /**
     * APP查询作品表配置
     * @param matchId
     * @param registerId
     * @return
     */
    MthRegisterConfigAppVo.Work queryWorkTableConfig(Long matchId, Long registerId);

    /**
     * APP我的赛事-详情
     * @param matchId
     * @param registerId
     * @return
     */
    MthMyMatchListDetailAppVo myMatchListDetail(Long matchId, Long registerId);


    /**
     * 修改赛事-赛项-组别-区域报名信息;
     */
    Boolean insertByBo(MthRegisterBo bo);

    /**
     * APP赛事活动报名
     * @param bo
     * @return
     */
    MthRegister registerForApp(MthRegisterAddAppBo bo);

    /**
     * APP赛事活动报名-信息更新
     * @param bo
     * @return
     */
    Boolean registerUpdateForApp(MthRegisterAddAppBo bo);

    /**
     * APP赛事活动作品-单独上传
     * @param bo
     * @return
     */
    Boolean uploadWorkForApp(MthRegisterAddAppBo.Work bo);

    /**
     * APP我的赛事-撤销报名
     * @param bo
     * @return
     */
    Boolean cancelRegister(MthCancelRegisterAppBo bo);

    /**
     * 修改赛事-赛项-组别-区域报名信息;
     */
    Boolean updateByBo(MthRegisterBo bo);

    Boolean updateExamineBo(MthRegisterBo bo);

    /**
     * 校验并批量删除赛事-赛项-组别-区域报名信息;信息
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);

    /**
     * 收藏或取消收藏赛事
     * @param matchCollectReqVo
     * @return
     */
    Result doCollectMatch(Worker worker,MatchCollectReqVo matchCollectReqVo);

    /**
     * 分页查询已收藏赛事
     * @param worker
     * @param pageReq
     * @return
     */
    Result<PageRsp<MthMyMatchListAppVo>> pageCollectedMatch(Worker worker, PageReq<MatchMyCollectBo> pageReq);

    /**
     * APP赛事活动-赛事详情-查看通知公告
     */
    List<MatchArticleRspVo> queryMatchArticleByMatchId(Long matchId);

    /**
     * 下载签名图片
     * @return
     */
    String doDownSignPicture(MatchDownPictureReqVO matchDownPictureReqVO) throws IOException;

}
