package com.crunii.micro.service.match.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.match.api.bo.MatchStatusInfoBo;
import com.crunii.micro.service.match.api.bo.MthRoundInstanceBo;
import com.crunii.micro.service.match.api.bo.ReceiveInfoBo;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.sysmgr.api.domain.dto.OrgSimpleVO;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 赛事-区域-场次实例（实例与配置的关系）;Service接口
 *
 * @author mth
 * @date 2023-09-19
 */
public interface IMthRoundInstanceService {

    /**
     * 查询赛事-区域-场次实例（实例与配置的关系）;
     */
    MthRoundInstanceVo queryById(Long id);

    /**
     * @param id
     * @description:
     * @return:
     * @author liuchuan
     * @date: 2023/9/27 16:20
     */
    MthRoundInstanceVo getMthRoundInstanceById(Long id);

    List<MthRoundAreaVo> getFirstRoundAreaByMatchIdAndMatchLevel(Long matchId, String matchLevel);

    /**
     * 通过父组织Ids获取子组织
     *
     * @param parentOrgIds
     * @return
     */
    List<OrgSimpleVO> getChildrenByParentOrgIds(List<Long> parentOrgIds);

    List<MthRoundInstanceVo> getAllRoundInfoByMatchId(Long matchId);

    List<MthRoundInstanceVo> getAllRoundInfoByMatchIdAndAreaId(Long matchId, Long areaId);

    List<MthMatchAreaVo> queryArea(Long matchId);

    /**
     * 查询赛事-区域-场次实例（实例与配置的关系）;列表
     */
    Result<PageRsp<MthRoundInstanceVo>> queryPageList(PageReq<MthRoundInstanceBo> bo);

    /**
     * 查询赛事-区域-场次实例（实例与配置的关系）;列表
     */
    List<MthRoundInstanceVo> queryList(MthRoundInstanceBo bo);

    /**
     * 修改赛事-区域-场次实例（实例与配置的关系）;
     */
    Long insertByBo(MthRoundInstanceBo bo);

    /**
     * 更新赛事同等级场次序号-自动排序
     *
     * @param matchId
     * @param matchLevel
     */
    void updateRoundSort(Long matchId, String matchLevel);

    /**
     * 场次-保存草稿
     */
    Long saveRoundDraft(MthRoundInstanceBo bo);

    /**
     * 修改赛事-区域-场次实例（实例与配置的关系）;
     */
    Boolean updateByBo(MthRoundInstanceBo bo);

    /**
     * 校验并批量删除赛事-区域-场次实例（实例与配置的关系）;信息
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);

    void createRoundInstance(List<MatchStatusInfoBo> mathchIdList);

    /**
     * @param roundId 场次id
     * @description: 查询场次奖项
     * @return:
     * @author liuchuan
     * @date: 2023/9/26 19:41
     */
    List<MthRoundPrizeVo> queryRoundPrizeById(Long roundId);
    
    
    /**
     * @description:  根据组别、赛项、比赛,返回场次下的奖项设置
     * @param null
     * @return: 
     * @author liuchuan
     * @date: 2023/11/28 15:57
    */
    Map<Long,List<MthRoundPrizeVo>> getRoundPrizeByMode(Long roundId);


    /**
     * 查询晋级人员  兼容之前 不改变返回结构
     *
     * @param roundId
     * @return
     */
    List<MthRoundPrizeVo> queryRoundPromoyion(Long roundId);


    /**
     * 承办方修改场次
     *
     * @param bo
     * @param worker
     */
    void updateMatchContractorInstance(MthRoundInstanceBo bo, Worker worker);

    /**
     * 承办方新增场次
     *
     * @param bo
     * @param worker
     */
    void saveMatchContractorInstance(MthRoundInstanceBo bo, Worker worker);

    /**
     * @param roundId 场次id
     * @description: 去公示
     * @return:
     * @author liuchuan
     * @date: 2023/11/22 14:12
     */
    void doPulicity(Long roundId);

    /**
     * @param bo
     * @description: 发送领奖通知
     * @return:
     * @author liuchuan
     * @date: 2023/11/23 14:36
     */
    boolean receiveMsg(ReceiveInfoBo bo);

    /**
     * 承办方开启场次
     * @param roundId
     * @param state
     * @param worker
     */
    void updateRoundState(Long roundId, Integer state, Worker worker);
}
