package com.crunii.micro.service.match.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.MthRoundList;
import com.crunii.micro.domain.entity.main.MthRoundScore;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 赛事-赛项-组别-区域-场次-名单;Service接口
 *
 * @author mth
 * @date 2023-09-19
 */
public interface IMthRoundListService {

    /**
     * 查询赛事-赛项-组别-区域-场次-名单;
     */
    MthRoundListVo queryById(Long id);

    /**
     * 查询赛事-赛项-组别-区域-场次-名单;列表
     */
    Result<PageRsp<MthRoundListVo>> queryPageList(PageReq<MthRoundListBo> bo);

    /**
     * 查询赛事-赛项-组别-区域-场次-名单;列表
     */
    List<MthRoundListVo> queryList(MthRoundListBo bo);

    /**
     * 根据报名id查询当前场次名单
     * @param registerId
     * @return
     */
    MthRoundList queryCurrRoundList(Long registerId);

    /**
     * 修改赛事-赛项-组别-区域-场次-名单;
     */
    Boolean insertByBo(MthRoundListBo bo);

    /**
     * 修改赛事-赛项-组别-区域-场次-名单;
     */
    Boolean updateByBo(MthRoundListBo bo);

    /**
     * 校验并批量删除赛事-赛项-组别-区域-场次-名单;信息
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);
    /**
     * 评审场次查询;
     */
    List<MthReviewRoundVo> getReviewRound(Worker worker, MthReviewRoundBo bo);
    /**
     * 查询设置排名列表(去评审);
     */
    /**
     *
     * @param worker
     * @param bo
     * @param export  是否导出
     * @return
     */
    PageRsp<MthRoundRankingListVo> getRankingListByRound(Worker worker, PageReq<MthRoundRankingListBo> bo,boolean export);
    /**
     * 查询评审详情
     */
    MthReviewDetailVo getReviewDetail(Worker worker, MthRoundRankingDetailBo bo);
    /**
     *  保存评审结果
     */
    boolean saveReviewResult(Worker worker, MthRoundScoreReviewBo bo);

    /**
    * @description: 新增场次评审员  默认创建场次的用户作为评审员
    */
    boolean saveRoundReviewer(Worker worker,Long roundId);

    void roundListRank(String scoreMode, List<MthRoundScore> list );

    void exportRankingListByRound(Worker worker, MthRoundRankingListBo bo);

    MthRoundScoreinfoVo getMthRoundScoreinfo(MthRoundRankingDetailBo bo, Worker worker);

    boolean updateReviewResult(Worker worker, MthRoundScoreReviewBo bo);
}
