package com.crunii.micro.service.match.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqArticleDetail;
import com.crunii.micro.service.match.dto.*;

import java.util.List;

/**
 * @author wangchen create 2023/6/27 9:33
 */
public interface MatchManageService {
    /**
     * 创建文章数据
     * @param articleDetailDto 文章参数
     * @param worker worker
     * @return str
     */
    String createArticleDetail(ArticleDetailDto articleDetailDto, Worker worker);

    void relateMatch(RelateMatchDto relateMatchDto);

    void cancelRelateMatch(RelateMatchDto relateMatchDto);

    /**
     * 分页查询文章
     * @param req 请求参数
     * @param worker worker
     * @return 分页数据
     */
    PageRsp<PageArticleVo> pageArticle(PageReq<PageArticleDto> req, Worker worker);

    List<PageArticleVo> listArticle(PageArticleDto req);
    /**
     * 查询文章
     * @param matchId
     * @return 分页数据
     */
    List<YcqArticleDetail> findByMatchId(Long matchId);

    /**
     * 更新文章状态
     * @param articleState 文章状态参数
     * @param worker 当前登录人
     * @return str
     */
    String updateArticleState(ArticleStateDto articleState, Worker worker);

    /**
     * 删除文章
     * @param articleId 文章id
     * @param worker worker
     * @return str
     */
    String delArticle(Long articleId,Worker worker);

    /**
     * 查询文章基本信息
     * @param articleId 文章id
     * @return data
     */
    BasicArticleVo getBasicArticle(Long articleId);

    /**
     * 获取文章内容
     * @param articleId 文章id
     * @return context
     */
    String getArticleContext(Long articleId);

    /**
     * 更新文章
     * @param articleUpdate 文章信息
     * @param worker worker
     * @return str
     */
    String updateArticle(ArticleUpdateDto articleUpdate,Worker worker);

    /**
     * 更新轮播暂时状态
     * @param carousel 轮播状态
     * @param worker worker
     * @return str
     */
    String updateCarousel(CarouselShowDto carousel,Worker worker);
}
