package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.service.match.api.bo.ObsFileInfoBo;
import com.crunii.micro.service.match.jdbc.ObsGetFileDao;
import com.crunii.micro.service.match.service.IObsFilService;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.function.BiConsumer;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author xujie
 * @date 2023-09-22
 */
@RequiredArgsConstructor
@Service
public class IObsFileServiceImpl implements IObsFilService {
    @Value("${upload.url}")
    private String urlPrefix;
    private final ObsGetFileDao fileDao;

    @Override
    public List<ObsFileInfoBo> getObsFileByIds(List<Long> bannerIds) {
        if(CollectionUtil.isEmpty(bannerIds)){
            return null;
        }
        List<ObsFileInfoBo> bos = fileDao.getFileInfoByIds(bannerIds);
        if (CollectionUtil.isEmpty(bos)){
            return null;
        }
        for (ObsFileInfoBo bo : bos) {
            bo.setObjectKey(urlPrefix+bo.getObjectKey());
        }
        return bos;
    }

    @Override
    public <T> void setObsFileByIds(List<T> list, Function<T, Long> getFileIdFun, BiConsumer<T, String> setFileUrlFun) {
        setObsFileByIds(list, getFileIdFun, setFileUrlFun, null);
    }

    @Override
    public <T> void setObsFileByIds(List<T> list, Function<T, Long> getFileIdFun, BiConsumer<T, String> setFileUrlFun, BiConsumer<T, String> setFileNameFun) {
        if(CollectionUtil.isEmpty(list)){
            return;
        }
        List<Long> idList = StreamUtils.toList(list, getFileIdFun);
        List<ObsFileInfoBo> bos = fileDao.getFileInfoByIds(idList);
        if (CollectionUtil.isEmpty(bos)){
            return;
        }
        Map<Long, ObsFileInfoBo> fileMap = StreamUtils.toIdentityMap(bos, ObsFileInfoBo::getId);
        for (T item : list) {
            ObsFileInfoBo obsFileInfoBo = fileMap.get(getFileIdFun.apply(item));
            if(obsFileInfoBo != null) {
                String objectKey = urlPrefix + obsFileInfoBo.getObjectKey();
                setFileUrlFun.accept(item, objectKey);
                if (setFileNameFun != null) {
                    setFileNameFun.accept(item, obsFileInfoBo.getFileName());
                }
            }
        }
    }

    @Override
    public <T> void setObsFileBoByIds(List<T> list, Function<T, Long> getFileIdFun, BiConsumer<T, ObsFileInfoBo> setFileBoFun) {
        if(CollectionUtil.isEmpty(list)){
            return;
        }
        List<Long> idList = StreamUtils.toList(list, getFileIdFun);
        if(CollectionUtil.isEmpty(idList)){
            return;
        }
        List<ObsFileInfoBo> bos = fileDao.getFileInfoByIds(idList);
        if (CollectionUtil.isEmpty(bos)){
            return;
        }
        Map<Long, ObsFileInfoBo> fileMap = StreamUtils.toIdentityMap(bos, ObsFileInfoBo::getId);
        for (T item : list) {
            ObsFileInfoBo newObsFileInfoBo = new ObsFileInfoBo();
            ObsFileInfoBo obsFileInfoBo = fileMap.get(getFileIdFun.apply(item));
            if(obsFileInfoBo != null) {
                String objectKey = urlPrefix + obsFileInfoBo.getObjectKey();
                newObsFileInfoBo.setObjectKey(objectKey);
                newObsFileInfoBo.setId(obsFileInfoBo.getId());
                newObsFileInfoBo.setFileName(obsFileInfoBo.getFileName());
                setFileBoFun.accept(item, newObsFileInfoBo);
            }
        }
    }

    @Override
    public <T> void setObsFileBoById(T t, Function<T, Long> getFileIdFun, BiConsumer<T, ObsFileInfoBo> setFileBoFun) {
        if(t == null || getFileIdFun.apply(t) == null){
            return;
        }
        Long fileId = getFileIdFun.apply(t);
        List<ObsFileInfoBo> bos = fileDao.getFileInfoByIds(Lists.newArrayList(fileId));
        if (CollectionUtil.isEmpty(bos)){
            return;
        }
        ObsFileInfoBo obsFileInfoBo = bos.get(0);
        if(obsFileInfoBo != null) {
            String objectKey = urlPrefix + obsFileInfoBo.getObjectKey();
            obsFileInfoBo.setObjectKey(objectKey);
            setFileBoFun.accept(t, obsFileInfoBo);
        }
    }

    @Override
    public <T> void setObsFileBoListById(T t, Function<T, String> getFileIdFun, BiConsumer<T, List<ObsFileInfoBo>> setFileBoFun) {
        if(t == null || getFileIdFun.apply(t) == null){
            return;
        }
        String fileId = getFileIdFun.apply(t);
        List<Long> fileIds = StrUtil.split(fileId, StrUtil.COMMA).stream().map(Long::valueOf).collect(Collectors.toList());
        List<ObsFileInfoBo> bos = fileDao.getFileInfoByIds(fileIds);
        if (CollectionUtil.isEmpty(bos)){
            return;
        }
        for (ObsFileInfoBo obsFileInfoBo : bos) {
            if(obsFileInfoBo != null) {
                String objectKey = urlPrefix + obsFileInfoBo.getObjectKey();
                obsFileInfoBo.setObjectKey(objectKey);
            }
        }
        setFileBoFun.accept(t, bos);
    }

    @Override
    public <T> void setObsFileBoListByIds(List<T> list, Function<T, String> getFileIdFun, BiConsumer<T, List<ObsFileInfoBo>> setFileBoFun) {
        for (T item : list) {
            setObsFileBoListById(item, getFileIdFun, setFileBoFun);
        }
    }
}
