package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.enums.TaskTypeEnums;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.match.api.FileDto;
import com.crunii.micro.service.match.api.bo.MatchApproveBo;
import com.crunii.micro.service.match.api.bo.MatchApprovePageReqBo;
import com.crunii.micro.service.match.api.bo.MatchApproveReqBo;
import com.crunii.micro.service.match.api.vo.AppDetailVo;
import com.crunii.micro.service.match.api.vo.ApproveRecordVo;
import com.crunii.micro.service.match.api.vo.MatchApprovePageReqVo;
import com.crunii.micro.service.match.api.vo.MthMatchAreaVo;
import com.crunii.micro.service.match.converter.FileConverter;
import com.crunii.micro.service.match.enums.MatchApproveEnums;
import com.crunii.micro.service.match.enums.MatchInfoStatusEnums;
import com.crunii.micro.service.match.enums.RoundEnums;
import com.crunii.micro.service.match.jdbc.MatchApproveDao;
import com.crunii.micro.service.match.jdbc.MthRoundInstanceDao;
import com.crunii.micro.service.match.service.MatchApproveService;
import com.crunii.micro.service.match.service.PushBusinessMsgService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

import static com.crunii.micro.service.match.constants.MatchMessageConstants.*;

/**
 * @author gongsongtao
 * @date 2024/1/17 17:17
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class MatchApproveServiceImpl implements MatchApproveService {

    private static final String orgCode = "QCCQ3537136584";//团重庆市委
    private static final String departmentCode = "书记班子";//所属部门：书记班子
    private static final String propertyCode = "workerDepartment";//所属部门
    private static final String departmentYouthMan = "青年创新创业服务中心";//所属部门:重庆市青年创新创业服务中心
    private static final String roleName = "赛事审核管理员";//赛事审核管理员

    private final YcqActiveFlowNodeRepo ycqActiveFlowNodeRepo;
    private final MthMatchSponsorRepo mthMatchSponsorRepo;
    private final TsmPropertyValueRepo tsmPropertyValueRepo;
    private final MthMatchAuditNodeRepo mthMatchAuditNodeRepo;
    private final MthMatchInfoRepo mthMatchInfoRepo;
    private final TsmOrgRepo tsmOrgRepo;
    private final TsmWorkerRepo tsmWorkerRepo;
    private final MatchApproveDao matchApproveDao;
    private final TsmWorkerRoleRepo tsmWorkerRoleRepo;
    private final TsmRoleRepo tsmRoleRepo;
    private final MthMatchGroupRepo matchGroupRepo;
    private final MthMatchItemRepo matchItemRepo;
    private final MthRoundInstanceDao roundInstanceDao;
    private final MthMatchLabelRepo mthMatchLabelRepo;
    private final PushBusinessMsgService pushBusinessMsgService;
    private final MthRoundInstanceRepo mthRoundInstanceRepo;
    private final YcqObsFileRepo fileRepo;
    private final FileConverter fileConverter;
    @Value("${upload.url}")
    private String urlPrefix;

    /**
     * 提交审核
     *
     * @param matchApproveReqBo
     * @param worker
     */
    public void doSubmitAudit(MatchApproveReqBo matchApproveReqBo, Worker worker) {
        StringBuilder organizingName=new StringBuilder();
        //赛事
        MthMatchInfo matchInfo = mthMatchInfoRepo.getOne(matchApproveReqBo.getMatchId());
        //团重庆市委
        TsmOrg tsmOrg = tsmOrgRepo.findByOrgCode(orgCode);
        if (ObjectUtil.isEmpty(tsmOrg)) {
            throw new BusinessException(MATCH_APPROVE_CITY_NOT_EXISTS);
        }
        //校验部门以及赛事
        validDepartment(matchInfo, matchApproveReqBo, worker,tsmOrg);
        //节点信息
        //过滤脏数据
        List<MatchApproveReqBo.NodeInformation> nodeInformationList = matchApproveReqBo.getNodeInformation().stream().filter(s -> StringUtils.isNotEmpty(s.getDepartment())).collect(Collectors.toList());
        //审核流程信息
        YcqFlowNode ycqFlowNode = new YcqFlowNode();
        ycqFlowNode.setBizId(matchApproveReqBo.getMatchId());
        ycqFlowNode.setBizType(TaskTypeEnums.MATCH.getCode());
        ycqFlowNode.setApproveType(3);
        ycqFlowNode.setCreateTime(new Date());
        ycqFlowNode.setCreateId((long) worker.getLoginId());
        ycqFlowNode.setCreateBy(worker.getWorkerName());
        ycqFlowNode.setResultType(0);
        ycqFlowNode.setLaunchApproveTime(new Date());
        ycqFlowNode.setLaunchApproveWorker(worker.getLoginId());
        //提交人是否属于市团委
        if (tsmOrg.getOrgId().toString().equals(worker.getOrgId())) {
            //是否包含至少一个审核部门
            if (nodeInformationList.size() == 0) {
                throw new BusinessException(MATCH_APPROVE_DEPARTMENT_NULL);
            }
            //第一个节点
            MatchApproveReqBo.NodeInformation nodeInformation = nodeInformationList.get(0);
            //判断一个审核部门是否是当前登录人所属部门
            if (!worker.getDepartment().equals(nodeInformation.getDepartment())) {
                throw new BusinessException(MATCH_APPROVE_DEPARTMENT_EQUAL);
            }
            organizingName.append(tsmOrg.getOrgName()).append("-").append(worker.getDepartment());
            //写入流程节点信息
            saveNode(matchApproveReqBo, nodeInformationList);
            //审核部门
            ycqFlowNode.setDepartment(nodeInformation.getDepartment());
            //删除第一个节点
            mthMatchAuditNodeRepo.deleteNodeBySort(matchApproveReqBo.getMatchId(), 0);
            //写入审核流程信息
            ycqActiveFlowNodeRepo.saveAndFlush(ycqFlowNode);
            //赛事状态设置为审核中
            matchInfo.setStatus(MatchInfoStatusEnums.APPROVING.getStatus());
            matchInfo.setLaunchApproveWorker(worker.getLoginId());
            matchInfo.setLaunchOrgId(Long.valueOf(worker.getOrgId()));
            mthMatchInfoRepo.saveAndFlush(matchInfo);
            //发送待审核消息
            List<Long> loginIds = tsmWorkerRoleRepo.findByRoleNameAndDepartment(roleName, nodeInformation.getDepartment()).stream().map(s->Long.valueOf(s.getLoginId())).collect(Collectors.toList());
            sendMsg(loginIds,matchInfo.getMatchName(),organizingName.toString(),worker.getWorkerName(),matchInfo.getId(),ycqFlowNode.getId());
            return;
        }
        organizingName.append(worker.getOrgName());
        //主办方是否包含重庆市团委
        List<String> orgIds = mthMatchSponsorRepo.findByMatchId(matchApproveReqBo.getMatchId()).stream().filter(s->s.getType()==1)
                .map(s -> s.getSponsorId().toString()).collect(Collectors.toList());
        if (orgIds.contains(tsmOrg.getOrgId().toString())) {
            //是否包含节点信息
            if (nodeInformationList.size() == 0) {
                throw new BusinessException(MATCH_APPROVE_NODE_NULL);
            }
            //写入流程节点信息
            saveNode(matchApproveReqBo, nodeInformationList);
            //先由本组织审核
            ycqFlowNode.setApproveType(1);
            ycqFlowNode.setOrgId(Long.parseLong(worker.getOrgId()));
        } else {
            //是否是青创赛事
            if (matchInfo.getQcFlag() == 1) {
                //是否包含节点信息
                if (nodeInformationList.size() == 0) {
                    throw new BusinessException(MATCH_APPROVE_NODE_NULL);
                }
                MatchApproveReqBo.NodeInformation nodeInformation = nodeInformationList.get(0);
                //第一个节点
                //判断第一个审核部门是否是重庆市青年创新创业服务中心
                if (!departmentYouthMan.equals(nodeInformation.getDepartment())) {
                    throw new BusinessException(MATCH_APPROVE_QC_MATCH);
                }
                ycqFlowNode.setApproveType(1);//修改审核类型为组织审核
                ycqFlowNode.setOrgId(Long.valueOf(worker.getOrgId()));
                ycqFlowNode.setDepartment(departmentYouthMan);
                //写入流程节点信息
                saveNode(matchApproveReqBo, nodeInformationList);
            } else {
                //非青创赛事，由本组织审核
                ycqFlowNode.setApproveType(1);
                ycqFlowNode.setOrgId(Long.parseLong(worker.getOrgId()));
                ycqFlowNode.setOrgName(worker.getOrgName());
            }
        }
        //写入审核流程信息
        ycqActiveFlowNodeRepo.saveAndFlush(ycqFlowNode);
        //赛事状态设置为审核中
        matchInfo.setStatus(MatchInfoStatusEnums.APPROVING.getStatus());
        matchInfo.setLaunchApproveWorker(worker.getLoginId());
        matchInfo.setLaunchApproveTime(new Date());
        matchInfo.setLaunchOrgId(Long.valueOf(worker.getOrgId()));
        mthMatchInfoRepo.saveAndFlush(matchInfo);
        //发送待审核消息
        List<Long> loginIds = tsmWorkerRoleRepo.findByRoleNameAndOrg(roleName, worker.getOrgId()).stream().map(s -> Long.valueOf(s.getLoginId())).collect(Collectors.toList());
        sendMsg(loginIds,matchInfo.getMatchName(),organizingName.toString(),worker.getWorkerName(),matchInfo.getId(),ycqFlowNode.getId());
    }

    /**
     * 发送待审核消息
     * @param userIds 接收用户
     * @param matchName 赛事名称
     * @param organizingName 组织
     * @param workerName 用户名称
     */
    private void sendMsg(List<Long> userIds, String matchName, String organizingName, String workerName,Long matchId,Long nodeId) {
        Map<String,String> map1=new HashMap<>();
        map1.put("matchId",matchId.toString());
        map1.put("nodeId",nodeId.toString());
        String json = JsonUtil.getJsonString(map1);
        pushBusinessMsgService.pushMatchToApproveMsg(userIds,matchName,organizingName,workerName,json);
    }

    /**
     * 发送审核结果消息
     * @param userIds 接收用户
     * @param matchName 赛事名称
     * @param organizingName 组织
     * @param workerName 用户名称
     * @param stateName 状态名称
     */
    private void sendResultMsg(Long userIds, String matchName, String organizingName, String workerName,String stateName,Long matchId,Long nodeId) {
        Map<String,String> map1=new HashMap<>();
        map1.put("matchId",matchId.toString());
        map1.put("nodeId",nodeId.toString());
        String json = JsonUtil.getJsonString(map1);
        pushBusinessMsgService.pushMatchResultMsg(userIds,matchName,organizingName,workerName,stateName,json);
    }

    /**
     * 审核
     *
     * @param matchApproveReqBo
     * @param worker
     */
    @Override
    public void doApprove(MatchApproveBo matchApproveReqBo, Worker worker) {
        StringBuilder organizingName=new StringBuilder();
        //剩余节点信息
        List<MthMatchAuditNode> nodeList = mthMatchAuditNodeRepo.findNode(matchApproveReqBo.getBizId());
        //审核
        YcqFlowNode ycqFlowNode = ycqActiveFlowNodeRepo.getOne(matchApproveReqBo.getId());
        //待办是否已审核
        validApprove(ycqFlowNode);
        //是否选择审核意见
        if (matchApproveReqBo.getResultType() == null) {
            throw new BusinessException(MATCH_APPROVE_RESULT_TYPE_NULL);
        }
        ycqFlowNode.setResultStr(matchApproveReqBo.getResultType()==1?"审核通过":"审核不通过");
        ycqFlowNode.setResultType(matchApproveReqBo.getResultType());
        ycqFlowNode.setReason(matchApproveReqBo.getReason());
        ycqFlowNode.setLoginId(worker.getLoginId());
        ycqActiveFlowNodeRepo.saveAndFlush(ycqFlowNode);
        //团重庆市委
        TsmOrg tsmOrg = tsmOrgRepo.findByOrgCode(orgCode);
        if (ycqFlowNode.getApproveType()==2 ||ycqFlowNode.getApproveType()==3 ){
            organizingName.append(ObjectUtil.isNotEmpty(tsmOrg)&&StringUtils.isNotEmpty(tsmOrg.getOrgName())?tsmOrg.getOrgName():"团重庆市委");
        }else {
            organizingName.append(worker.getOrgName());
        }
        MthMatchInfo matchInfo = mthMatchInfoRepo.getOne(matchApproveReqBo.getBizId());
        //是否通过
        if (matchApproveReqBo.getResultType() == 1) {
            //是否还有审核环节
            if (nodeList.size() > 0) {
                MthMatchAuditNode mthMatchAuditNode = nodeList.get(0);
                YcqFlowNode newNode = new YcqFlowNode();
                newNode.setApproveType(mthMatchAuditNode.getType() == 1 ? 2 : 3);//审核方式
                newNode.setBizId(matchApproveReqBo.getBizId());
                newNode.setBizType(TaskTypeEnums.MATCH.getCode());
                newNode.setCreateId((long) worker.getLoginId());
                newNode.setCreateBy(worker.getWorkerName());
                newNode.setCreateTime(new Date());
                newNode.setLoginId(mthMatchAuditNode.getWorkerId());
                newNode.setDepartment(mthMatchAuditNode.getDepartment());
                newNode.setResultType(0);
                newNode.setLaunchApproveTime(ycqFlowNode.getLaunchApproveTime());
                newNode.setLaunchApproveWorker(ycqFlowNode.getLaunchApproveWorker());
                ycqActiveFlowNodeRepo.saveAndFlush(newNode);
                //删除该节点
                mthMatchAuditNode.setDelFlag(1);
                mthMatchAuditNodeRepo.saveAndFlush(mthMatchAuditNode);

                //发送待审核消息
                List<Long> users=new ArrayList<>();
                switch (mthMatchAuditNode.getType()){
                    case 1:
                        users.add((long)mthMatchAuditNode.getWorkerId());
                    case 2:
                        List<Long> loginIds = tsmWorkerRoleRepo.findByRoleNameAndDepartment(roleName,mthMatchAuditNode.getDepartment()).stream().map(s->Long.valueOf(s.getLoginId())).collect(Collectors.toList());
                        users.addAll(loginIds);
                }
                sendMsg(users,matchInfo.getMatchName(),organizingName.toString(),worker.getWorkerName(),matchInfo.getId(),newNode.getId());
            } else {
                //审核流程已走完
                //将赛事状态修改为待发布
                if (ObjectUtil.isNotEmpty(matchInfo)) {
                    matchInfo.setStatus(MatchInfoStatusEnums.TO_BE_RELEASED.getStatus());
                    mthMatchInfoRepo.saveAndFlush(matchInfo);
                    //发送审核结果通知
                    sendResultMsg((long)matchInfo.getLaunchApproveWorker(),matchInfo.getMatchName(),organizingName.toString(),worker.getWorkerName(),"通过",matchInfo.getId(),matchApproveReqBo.getId());
                }
            }
        } else {
            //审核不通过
            //1.将赛事状态设置为待审核
            if (ObjectUtil.isNotEmpty(matchInfo)) {
                matchInfo.setStatus(MatchInfoStatusEnums.NO_PASS.getStatus());
                mthMatchInfoRepo.saveAndFlush(matchInfo);
            }
            //2.将已有审核流程设置为已重新启动
            List<YcqFlowNode> nodes = ycqActiveFlowNodeRepo.findByBizIdAndState(matchApproveReqBo.getBizId());
            nodes.forEach(s -> {
                s.setIsRestart(1);
            });
            ycqActiveFlowNodeRepo.saveAllAndFlush(nodes);
            //3.删除剩余未审核节点
            nodeList.forEach(node -> node.setDelFlag(1));
            mthMatchAuditNodeRepo.saveAllAndFlush(nodeList);
            //发送审核结果通知
            sendResultMsg((long)matchInfo.getLaunchApproveWorker(),matchInfo.getMatchName(),organizingName.toString(),worker.getWorkerName(),"不通过",matchInfo.getId(),matchApproveReqBo.getId());
        }
    }

    /**
     * 查询赛事待办
     *
     * @param pageReqBo
     * @param worker
     * @return
     */
    @Override
    public PageRsp<MatchApprovePageReqVo> pageApprove(PageReq<MatchApprovePageReqBo> pageReqBo, Worker worker) {
        //当前登陆人是否拥有赛事审核管理员角色
        List<Integer> roleIds = tsmWorkerRoleRepo.findByLoginId((long) worker.getLoginId()).stream().map(TsmWorkerRole::getRoleId).collect(Collectors.toList());
        TsmRole role = tsmRoleRepo.findByRoleName(roleName);
        if (!roleIds.contains(role.getRoleId())) {
            return new PageRsp<>();
        }
        //主办方
        MatchApprovePageReqBo query = pageReqBo.getQuery();
        List<Long>matchIds=new ArrayList<>();
        if (ObjectUtil.isNotEmpty(query) && query.getSponsorId()!=null){
            List<MthMatchSponsor> sponsorList = mthMatchSponsorRepo.findBySponsorId(query.getSponsorId());
            List<Long> ids = sponsorList.stream().map(MthMatchSponsor::getMatchId).collect(Collectors.toList());
            matchIds.addAll(ids);
        }
        //承办方
        if (ObjectUtil.isNotEmpty(query) && query.getOrganizerId()!=null){
            List<MthMatchSponsor> sponsorList = mthMatchSponsorRepo.findBySponsorId(query.getOrganizerId());
            List<Long> ids = sponsorList.stream().map(MthMatchSponsor::getMatchId).collect(Collectors.toList());
            matchIds.addAll(ids);
        }
        query.setMatchIds(matchIds);
        PageRsp<MatchApprovePageReqVo> rsp = matchApproveDao.pageApprove(pageReqBo, worker);
        //处理赛事等级
        for (MatchApprovePageReqVo vo : rsp.getResult()) {
            //赛事等级
            if (StrUtil.isNotBlank(vo.getMatchLevel())) {
                String[] arr = vo.getMatchLevel().split(",");
                String matchLevel = Arrays.stream(arr)
                        .map(item -> {
                            RoundEnums.AreaLevel areaByCode = RoundEnums.AreaLevel.getAreaByCode(item);
                            return areaByCode == null ? "" : areaByCode.getDesc();
                        }).collect(Collectors.joining(","));
                vo.setMatchLevelName(matchLevel);
            }
            StringBuilder undertakeNames= new StringBuilder();//承办方
            StringBuilder sponsorNames= new StringBuilder();//主办方
            //承办方和主办方
            List<MthMatchSponsor> list = mthMatchSponsorRepo.findByMatchId(vo.getMatchId());
            List<String> undertakeNameList = list.stream().filter(s -> s.getType() == 2).map(MthMatchSponsor::getSponsorName).collect(Collectors.toList());
            List<String> sponsorNameList = list.stream().filter(s -> s.getType() == 1).map(MthMatchSponsor::getSponsorName).collect(Collectors.toList());
            undertakeNameList.forEach(s->undertakeNames.append(undertakeNames.length()==0?s:","+s));
            sponsorNameList.forEach(s->sponsorNames.append(sponsorNames.length()==0?s:","+s));
            //拼接其他承办方
            if (StringUtils.isNotEmpty(vo.getOtherOrganizer())){
                undertakeNames.append(undertakeNames.length()==0?vo.getOtherOrganizer():","+vo.getOtherOrganizer());
            }
            //拼接其他主办方
            if (StringUtils.isNotEmpty(vo.getOtherOrganizer())){
                sponsorNames.append(sponsorNames.length()==0?vo.getOtherSponsor():","+vo.getOtherSponsor());
            }
            vo.setMatchSponsorName(sponsorNames.toString());
            vo.setMatchUndertakeName(undertakeNames.toString());
        }
        return rsp;
    }

    /**
     * 查询审核记录
     *
     * @param matchId
     * @return
     */
    @Override
    public List<ApproveRecordVo> getApproveRecord(Long matchId) {
        List<ApproveRecordVo> recordVoList = new ArrayList<>();
        MthMatchInfo matchInfo = mthMatchInfoRepo.getOne(matchId);
        if (ObjectUtil.isEmpty(matchInfo)|| (MatchInfoStatusEnums.TO_APPROVE.getStatus().equals(matchInfo.getStatus())||MatchInfoStatusEnums.DRAFT.getStatus().equals(matchInfo.getStatus()))) {
            return recordVoList;
        }
        //团重庆市委
        String name="重庆市团委";
        TsmOrg cityOrg = tsmOrgRepo.findByOrgCode(orgCode);
        if (ObjectUtil.isNotEmpty(cityOrg)&&StringUtils.isNotEmpty(cityOrg.getOrgName())){
            name=cityOrg.getOrgName();
        }
        //提审信息
        ApproveRecordVo launchRecord = new ApproveRecordVo();
        launchRecord.setId(matchInfo.getId());
        launchRecord.setNodeName(MatchApproveEnums.LAUNCH.getName());
        launchRecord.setNode(MatchApproveEnums.LAUNCH.getCode());
        launchRecord.setApproveTime(matchInfo.getLaunchApproveTime());
        StringBuilder dealInformation = new StringBuilder(name);
        //是否有发布组织
        if (matchInfo.getLaunchOrgId()!=null && matchInfo.getLaunchApproveWorker()!=null) {
            TsmOrg tsmOrg = tsmOrgRepo.getOne(matchInfo.getLaunchOrgId());
            TsmWorker tsmWorker = tsmWorkerRepo.getOne(matchInfo.getLaunchApproveWorker());
            if (orgCode.equals(tsmOrg.getOrgCode())) {
                dealInformation.append("-").append(tsmWorker.getDepartment() == null ? "" : tsmWorker.getDepartment());
            } else {
                dealInformation.append(spiltInformation("", tsmOrg));
            }
            dealInformation.append("-").append(tsmWorker.getWorkerName());
            launchRecord.setDealInformation(dealInformation.toString());
            recordVoList.add(launchRecord);
            //审核记录
            List<YcqFlowNode> nodes = ycqActiveFlowNodeRepo.findByBizId(matchId);
            for (YcqFlowNode node : nodes) {
                StringBuilder approveInformation = new StringBuilder(name);
                TsmWorker approveWorker;
                if (node.getResultType()!=0){
                    approveWorker = tsmWorkerRepo.getOne(node.getLoginId());
                }else {
                    approveWorker=new TsmWorker();
                }
                //是否是部门审核或者人审核
                if (node.getApproveType() == 3 || node.getApproveType() == 2) {
                    //是否已审核
                    if (node.getResultType()==0){
                        approveInformation.append("-").append(node.getDepartment());
                    } else {
                        approveInformation.append("-").append(approveWorker.getDepartment() == null ? "" : approveWorker.getDepartment());
                    }
                } else {
                    //是否属于团组织
                    if (ObjectUtil.isNotEmpty(tsmOrg) && orgCode.equals(tsmOrg.getOrgCode())) {
                       approveInformation.append("-").append(approveWorker.getDepartment() == null ? "" : approveWorker.getDepartment());
                    } else {
                        approveInformation.append("-").append(spiltInformation("", tsmOrg));
                    }
                }
                if (node.getResultType()!=0){
                    approveInformation.append("-").append(approveWorker.getWorkerName());
                }
                //拼接处理人
                ApproveRecordVo approveRecordVo = new ApproveRecordVo();
                approveRecordVo.setId(node.getId());
                approveRecordVo.setNode(MatchApproveEnums.APPROVE.getCode());
                approveRecordVo.setNodeName(MatchApproveEnums.APPROVE.getName());
                approveRecordVo.setReason(node.getReason());
                approveRecordVo.setResultType(node.getResultType());
                approveRecordVo.setResultTypeName(node.getResultStr()==null?"待审核":node.getResultStr());
                approveRecordVo.setDealInformation(approveInformation.toString());
                approveRecordVo.setApproveTime(node.getCreateTime());
                recordVoList.add(approveRecordVo);
            }
        }
        return recordVoList;
    }

    /**
     * app-查看待审核详情
     * @param matchId 赛事id
     * @param nodeId 待办id
     * @return
     */
    @Override
    public AppDetailVo getApproveDetail(Long matchId,Long nodeId, Worker worker) {
        AppDetailVo appDetailVo=new AppDetailVo();
        if (matchId==null || nodeId==null){
            throw new BusinessException(REQUEST_ERROR);
        }
        //待办
        YcqFlowNode ycqFlowNode = ycqActiveFlowNodeRepo.getOne(nodeId);
        appDetailVo.setWhetherApprove(ycqFlowNode.getResultType()!=0);
        //赛事是否还存在
        MthMatchInfo matchInfo = mthMatchInfoRepo.getOne(matchId);
        if (ObjectUtil.isEmpty(matchInfo)){
            throw new BusinessException(MATCH_NOT_EXIST);
        }
        BeanUtil.copyProperties(matchInfo,appDetailVo);
        appDetailVo.setId(ycqFlowNode.getId());
        appDetailVo.setMatchId(matchInfo.getId());
        appDetailVo.setRegisterationAuditFlagName(matchInfo.getRegisterationAuditFlag()!=null && matchInfo.getRegisterationAuditFlag()==1?"是":"否");
        //查询实例ID
        List<MthRoundInstance> MthRoundInstances = mthRoundInstanceRepo.findByMatchIdAndRoundType(matchInfo.getId(), RoundEnums.RoundType.TEMPLATE.getCode());
        if (MthRoundInstances.size()>0){
            MthRoundInstance mthRoundInstance = MthRoundInstances.get(0);
            appDetailVo.setRoundInstanceId(ObjectUtil.isNotEmpty(mthRoundInstance)?mthRoundInstance.getId():0);
        }
        //赛事赛项与组别
        List<MthMatchItem> itemList = matchItemRepo.findByMatchId(matchId);
        List<MthMatchGroup> groups = matchGroupRepo.findByMatchId(matchId);
        List<AppDetailVo.ItemList> groupItemList = new ArrayList<>();
        for (MthMatchItem mthMatchItem : itemList) {
            AppDetailVo.ItemList item=new AppDetailVo.ItemList();
            item.setItemName(mthMatchItem.getItemName());
            item.setId(mthMatchItem.getId());
            //当前赛项是否存在组别
            List<MthMatchGroup> collect = groups.stream().filter(s -> Objects.equals(item.getId(), s.getItemId())).collect(Collectors.toList());
            List<AppDetailVo.GroupList> groupList=new ArrayList<>();
            for (MthMatchGroup mthMatchGroup : collect) {
                AppDetailVo.GroupList group=new AppDetailVo.GroupList();
                group.setId(mthMatchGroup.getId());
                group.setName(mthMatchGroup.getGroupName());
                groupList.add(group);
            }
            item.setGroupList(groupList);
            groupItemList.add(item);
        }
        appDetailVo.setGroupItemList(groupItemList);
        //赛事标签
        StringBuilder label=new StringBuilder();
        List<MthMatchLabel> labelList = mthMatchLabelRepo.findByMatchId(matchId);
        labelList.forEach(s->label.append(label.length()==0?s.getLabelName():","+s.getLabelName()));
        appDetailVo.setMatchLabel(label.toString());
        //比赛范围
        List<MthMatchAreaVo> areaVoList = roundInstanceDao.queryTemplateArea(matchId);
        // 根据区域分组
        Map<String, List<MthMatchAreaVo>> matchLevelMap = StreamUtils.groupByKey(areaVoList, MthMatchAreaVo::getMatchLevel);
        List<MthMatchAreaVo> areaGroupList = new ArrayList<>();
        for (Map.Entry<String, List<MthMatchAreaVo>> entry : matchLevelMap.entrySet()) {
            MthMatchAreaVo group = new MthMatchAreaVo();
            group.setAreaName(RoundEnums.AreaLevel.getAreaByCode(entry.getKey()).getDesc());
            group.setChildren(entry.getValue());
            areaGroupList.add(group);
        }
        appDetailVo.setMatchScope(areaGroupList);
        //承办方和主办方
        StringBuilder matchSponsorName=new StringBuilder();
        StringBuilder matchUndertakeName=new StringBuilder();
        List<MthMatchSponsor> list = mthMatchSponsorRepo.findByMatchId(matchId);
        List<String> undertakeNameList = list.stream().filter(s -> s.getType() == 2).map(MthMatchSponsor::getSponsorName).collect(Collectors.toList());
        List<String> sponsorNameList = list.stream().filter(s -> s.getType() ==1).map(MthMatchSponsor::getSponsorName).collect(Collectors.toList());
        undertakeNameList.forEach(s->matchUndertakeName.append(matchUndertakeName.length()==0?s:","+s));
        sponsorNameList.forEach(s->matchSponsorName.append(matchSponsorName.length()==0?s:","+s));

        appDetailVo.setMatchSponsorName(matchSponsorName.toString());
        appDetailVo.setMatchUndertakeName(matchUndertakeName.toString());
        appDetailVo.setOtherMatchSponsorName(matchInfo.getOtherSponsor());
        appDetailVo.setOtherMatchUndertakeName(matchInfo.getOtherOrganizer());
        //封面图片
        if (ObjectUtil.isNotNull(matchInfo.getBannerId())) {
            YcqObsFile file = fileRepo.findByIdAndState(matchInfo.getBannerId(), 1);
            if (ObjectUtil.isNotNull(file)) {
                FileDto fileDto = fileConverter.ObsFileToFileDto(file);
                fileDto.setFileUrl(urlPrefix + fileDto.getFileKey());
                appDetailVo.setFileDto(fileDto);
            }
        }
        return appDetailVo;
    }

    /**
     * 校验待办
     * @param ycqFlowNode
     */
    private void validApprove(YcqFlowNode ycqFlowNode) {
        //待办是否还存在
        if (ObjectUtil.isEmpty(ycqFlowNode) || ycqFlowNode.getIsRestart()==1){
            throw new BusinessException(MATCH_TASK_NOT_EXISTS);
        }
        //待办是否已处理
        if (ycqFlowNode.getResultType()!=0 && (1 == ycqFlowNode.getResultType() || 2 == ycqFlowNode.getResultType() || 3 == ycqFlowNode.getResultType())) {
            throw new BusinessException(MATCH_APPROVE_DONE);
        }
    }

    /**
     * 处理审核信息
     *
     * @param tsmOrg
     */
    private String spiltInformation(String dealChain, TsmOrg tsmOrg) {
        if (dealChain == null) {
            dealChain = "";
        }
        dealChain = tsmOrg.getOrgName() + dealChain;
        TsmOrg parentOrg = tsmOrgRepo.getOne(tsmOrg.getParentOrgId());
        if (ObjectUtil.isEmpty(parentOrg) || orgCode.equals(parentOrg.getOrgCode())) {
            return dealChain;
        }
        return spiltInformation(dealChain, parentOrg);
    }

    /**
     * 写入流程节点信息
     *
     * @param matchApproveReqBo
     * @param nodeInformationList
     */
    private void saveNode(MatchApproveReqBo matchApproveReqBo, List<MatchApproveReqBo.NodeInformation> nodeInformationList) {
        //写入节点信息
        for (int i = 0; i < nodeInformationList.size(); i++) {
            //写入节点信息
            MatchApproveReqBo.NodeInformation nodeInformation1 = nodeInformationList.get(i);
            MthMatchAuditNode mthMatchAuditNode = new MthMatchAuditNode();
            mthMatchAuditNode.setMatchId(matchApproveReqBo.getMatchId());
            mthMatchAuditNode.setDepartment(nodeInformation1.getDepartment());
            mthMatchAuditNode.setWorkerId(nodeInformation1.getAuditWorkerId());
            mthMatchAuditNode.setType(nodeInformation1.getAuditWorkerId() != null ? 1 : 2);
            mthMatchAuditNode.setSort(i);
            mthMatchAuditNodeRepo.saveAndFlush(mthMatchAuditNode);
        }
    }

    /**
     * 校验部门以及赛事
     *
     * @param matchApproveReqBo
     * @param worker
     */
    private void validDepartment(MthMatchInfo matchInfo, MatchApproveReqBo matchApproveReqBo, Worker worker,TsmOrg tsmOrg) {
        //判断赛事是否存在
        if (ObjectUtil.isEmpty(matchInfo)) {
            throw new BusinessException(MATCH_NOT_EXIST);
        }
        //当前状态是否能够发起审核流程(待审核和审核不通过可以发起)
        if (!(MatchInfoStatusEnums.TO_APPROVE.getStatus().equals(matchInfo.getStatus()) || MatchInfoStatusEnums.NO_PASS.getStatus().equals(matchInfo.getStatus()))) {
            throw new BusinessException(MATCH_APPROVE_STATE_NOT_ALLOW);
        }
        //校验当前登录用户是否设置了部门信息
        if (worker.getOrgId()!=null && tsmOrg.getOrgId().toString().equals(worker.getOrgId()) && StringUtils.isEmpty(worker.getDepartment())) {
            throw new BusinessException(MATCH_APPROVE_WORKER_DEPARTMENT_NULL);
        }
        //判断当前登陆人是否属于团市委且所属部门是否存在
        List<String> codes = tsmPropertyValueRepo.findByPropCode(propertyCode).stream().map(TsmPropertyValue::getValueCode).collect(Collectors.toList());
        if (StringUtils.isNotEmpty(worker.getDepartment()) && !codes.contains(worker.getDepartment())) {
            throw new BusinessException(MATCH_APPROVE_DEPARTMENT_ERROR);
        }
        List<MatchApproveReqBo.NodeInformation> nodeInformation = matchApproveReqBo.getNodeInformation();
        //所选择的审核部门是否都存在
        List<MatchApproveReqBo.NodeInformation> collect = nodeInformation.stream().filter(s -> !codes.contains(s.getDepartment()) && StringUtils.isNotEmpty(s.getDepartment())).collect(Collectors.toList());
        if (collect.size() > 0) {
            throw new BusinessException(MATCH_APPROVE_DEPARTMENT_NEWEST);
        }
        //部门是否包含书记班子
        List<MatchApproveReqBo.NodeInformation> secretary = nodeInformation.stream().filter(s -> StringUtils.isNotEmpty(s.getDepartment()) && departmentCode.contains(s.getDepartment()) && s.getAuditWorkerId() == null).collect(Collectors.toList());
        if (secretary.size() > 0) {
            throw new BusinessException(MATCH_APPROVE_WORKER_NULL);
        }
    }
}
