package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.converter.MthRoundAreaConverter;
import com.crunii.micro.service.match.converter.MthRoundInstanceConverter;
import com.crunii.micro.service.match.converter.MthRoundPrizeConverter;
import com.crunii.micro.service.match.converter.MthRoundReviewConverter;
import com.crunii.micro.service.match.enums.RoundEnums;
import com.crunii.micro.service.match.jdbc.MatchContractorManageDao;
import com.crunii.micro.service.match.jdbc.MthRoundInsInstanceDao;
import com.crunii.micro.service.match.service.IMthRoundInstanceService;
import com.crunii.micro.service.match.service.IMthRoundReviewStandarService;
import com.crunii.micro.service.match.service.MatchContractorManageService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @author denggl
 * date 2023/11/15
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class MatchContractorManageServiceImpl implements MatchContractorManageService {

    private final MthRoundInstanceRepo baseRepo;
    private final MthRoundInstanceConverter baseConverter;
    private final MthRoundReviewConverter mthRoundReviewConverter;
    private final MatchContractorManageDao matchContractorManageDao;
    private final MthRoundPrizeConverter roundPrizeConverter;
    private final IMthRoundReviewStandarService iMthRoundReviewStandarService;
    private final MthRoundAreaRepo roundAreaRepo;
    private final MthRoundPrizeRepo roundPrizeRepo;
    private final MthRoundAreaConverter roundAreaConverter;
    private final IMthRoundInstanceService iMthRoundInstanceService;
    private final MthMatchInfoRepo mthMatchInfoRepo;
    @Autowired
    private MthMatchContractorRepo mthMatchContractorRepo;
    private final MthRoundReviewStandarRepo mthRoundReviewStandarRepo;
    private final MthRoundReviewerRepo mthRoundReviewerRepo;
    private final   MthRoundInsInstanceDao mthRoundInsInstanceDao;
    private final MthMatchSignInformationRepo mthMatchSignInformationRepo;
    private final MthMatchSponsorRepo mthMatchSponsorRepo;

    @Override
    public PageRsp<MthMatchInfoVo> pageMatchContractor(PageReq<MatchPublishListBo> bo, Worker worker) {
        PageRsp<MthMatchInfoVo> rsp = matchContractorManageDao.pageMatchContractor(bo, worker);
        if(ObjectUtil.isNotEmpty(rsp.getResult())) {
            //主办方
            List<Long> matchIds = StreamUtils.toList(rsp.getResult(), MthMatchInfoVo::getId);
            List<MthMatchSponsor> matchSponsorList = mthMatchSponsorRepo.findByMatchIdIn(matchIds);
            Map<Long, Map<Integer, List<MthMatchSponsor>>> matchTypeSponsorListMap = StreamUtils.groupBy2Key(matchSponsorList, MthMatchSponsor::getMatchId, MthMatchSponsor::getType);
            for (MthMatchInfoVo vo : rsp.getResult()) {
                if (StrUtil.isNotBlank(vo.getMatchLevel())) {
                    String[] arr = vo.getMatchLevel().split(",");
                    String matchLevel = Arrays.stream(arr)
                            .map(item -> {
                                RoundEnums.AreaLevel areaByCode = RoundEnums.AreaLevel.getAreaByCode(item);
                                return areaByCode == null ? "" : areaByCode.getDesc();
                            }).collect(Collectors.joining(","));
                    vo.setMatchLevel(matchLevel);
                }
                Map<Integer, List<MthMatchSponsor>> typeSponsorListMap = matchTypeSponsorListMap.get(vo.getId());
                if(ObjectUtil.isNotEmpty(typeSponsorListMap)) {
                    List<MthMatchSponsor> sponsors = typeSponsorListMap.get(1);
                    if(ObjectUtil.isNotEmpty(sponsors)) {
                        String sponsorsName = sponsors.stream().map(MthMatchSponsor::getSponsorName).collect(Collectors.joining(StrUtil.COMMA));
                        vo.setSponsor(sponsorsName);
                    }
                    List<MthMatchSponsor> organizers = typeSponsorListMap.get(2);
                    if(ObjectUtil.isNotEmpty(organizers)) {
                        String organizersName = organizers.stream().map(MthMatchSponsor::getSponsorName).collect(Collectors.joining(StrUtil.COMMA));
                        vo.setOrganizer(organizersName);
                    }
                }
            }
        }
        return rsp;
    }


    @Override
    public List<MthRoundReviewStandarVo> listMthRoundReviewStandar(Long roundId) {

        return null;

    }

    @Override
    public void  updateRoundState(Long roundId, Integer state,Worker worker) {

        MthRoundInstance mthRoundInstance = baseRepo.findById(roundId).orElse(null);
        if (mthRoundInstance==null){
            throw  new BusinessException("场次不存在！");
        }
        if (state==0){
            //开启判断否完善比赛信息
             if (!checkMhRoundInstance(mthRoundInstance)){
                  new BusinessException("请先完善场次信息！");
              }
        }
        mthRoundInstance.setRoundState(state);
        mthRoundInstance.setUpdateTime(new Date());
        mthRoundInstance.setUpdateId((long)worker.getLoginId());
        mthRoundInstance.setUpdateBy(worker.getWorkerName());
        baseRepo.save(mthRoundInstance);
    }

    @Override
    public void delContractorRoundInstance(Long roundId, Worker worker) {
        MthRoundInstance mthRoundInstance = baseRepo.findById(roundId).orElse(null);
        if (mthRoundInstance==null){
            throw  new BusinessException("场次不存在！");
        }
        if (mthRoundInstance.getRoundState()!=null) {
            if (mthRoundInstance.getRoundState() == 0) {
                throw new BusinessException("场次已开启,不能删除！");
            }
        }
        MthMatchInfo mthMatchInfo = mthMatchInfoRepo.findById(mthRoundInstance.getMatchId()).orElse(new MthMatchInfo());
        //比赛创建人和场次创建人相同代表是主办方创建
        if (mthMatchInfo.getCreateId().equals(mthRoundInstance.getCreateId())){
            throw  new BusinessException("主办方创建的场次无权删除！");
        }
        mthRoundInstance.setDelFlag(1);
        mthRoundInstance.setUpdateTime(new Date());
        mthRoundInstance.setUpdateId((long)worker.getLoginId());
        mthRoundInstance.setUpdateBy(worker.getWorkerName());
        baseRepo.saveAndFlush(mthRoundInstance);
        //排序
        List<MthRoundInstance> byMatchIdAndRoundTypeAndAreaId = baseRepo.findByMatchIdAndRoundTypeAndAreaId(mthMatchInfo.getId(), RoundEnums.RoundType.INSTANCE.getCode(), mthRoundInstance.getAreaId());
        updateRoundSort(byMatchIdAndRoundTypeAndAreaId);
      }

    private void updateRoundSort(List<MthRoundInstance> levelRoundList){
        if(levelRoundList.size() > 1) {
            // 场次序号从2开始
            AtomicInteger roundSort = new AtomicInteger(2);
            // 修改场次序号不等于1的自动排序
            levelRoundList.stream().filter(item -> item.getRoundSort() != 1).sorted(Comparator.comparing(MthRoundInstance::getRoundSort))
                    .forEach(item -> item.setRoundSort(roundSort.getAndIncrement()));
            baseRepo.saveAllAndFlush(levelRoundList);
        }
    }
      //报名时间
    @Override
    public void updaterRgisterationEnd(MathContractorExtendTimeBo bo, Worker worker) {
        MthRoundInstance mthRoundInstance = baseRepo.findById(bo.getRoundId()).orElse(null);
        if (mthRoundInstance==null){
            throw  new BusinessException("场次不存在！");
        }
        checkRoundInstance(mthRoundInstance.getRoundState());

       if(checkHour(mthRoundInstance.getRegisterationEnd(),bo.getExtendTime())){
           throw  new BusinessException("报名时间结束前24小时内禁止修改！");

       }

       if (checkYear(mthRoundInstance.getRegisterationEnd(),bo.getExtendTime())){
           throw  new BusinessException("延长时间不能超过3年！");

        }
        mthRoundInstance.setRegisterationEnd(bo.getExtendTime());
        baseRepo.saveAndFlush(mthRoundInstance);

    }

    private void  checkRoundInstance(Integer state){
        if ( state!=0){
            log.info("场次未开启,修改失败");
            throw new BusinessException("场次未开启,修改失败！");
        }
   }
    @Override
    public void updateEntryDeadline(MathContractorExtendTimeBo bo, Worker worker) {
        MthRoundInstance mthRoundInstance = baseRepo.findById(bo.getRoundId()).orElse(null);
        if (mthRoundInstance==null){
            throw  new BusinessException("场次不存在！");
        }
       // checkRoundInstance(mthRoundInstance.getRoundState());
        if(checkHour(mthRoundInstance.getEntryDeadline(),bo.getExtendTime())){
            throw new BusinessException("截至时间结束前24小时内禁止修改");
        }
        if (checkYear(mthRoundInstance.getEntryDeadline(),bo.getExtendTime())){
            throw   new BusinessException("延长时间不能超过3年！");

        }
        mthRoundInstance.setEntryDeadline(bo.getExtendTime());
        baseRepo.saveAndFlush(mthRoundInstance);

    }



    @Override
    public void updateIMthRoundReviewStandar(List<MthRoundReviewStandarBo> list, Worker worker) {
        MthRoundReviewStandarBo mthRoundReviewStandarBo=list.get(0);
        MthRoundInstance mthRoundInstance = baseRepo.findById(mthRoundReviewStandarBo.getRoundId()).orElse(null);
        mthRoundInstance.setReviewerStandar(mthRoundReviewStandarBo.getReviewerStandar());
        baseRepo.saveAndFlush(mthRoundInstance);
        iMthRoundReviewStandarService.saveBatchIMthRoundReviewStandar(list,mthRoundInstance.getSocreMode(),mthRoundInstance.getReviewerStandar());
    }
    private  String getMatchLevel(String level){
        if (StringUtils.isBlank(level)) {
             return null;
        }
        String[] split = level.split(",");
        List<RoundEnums.AreaLevel> matchLevel = Arrays.stream(split).map(item -> RoundEnums.AreaLevel.getAreaByCode(item)).sorted(Comparator.comparing(RoundEnums.AreaLevel::getLevel)).collect(Collectors.toList());
        String Levelcode = matchLevel.get(0).getCode();
        return Levelcode;
    }
    @Override
    public List<MthRoundInstanceVo> getAllRoundInfoByMatchId(Long matchId,Long  areaId, Worker worker) {
        MthMatchInfo mthMatchInfo = mthMatchInfoRepo.findById(matchId).orElse(new MthMatchInfo());
        String matchLevel = getMatchLevel(mthMatchInfo.getMatchLevel());
        List<MthRoundInstanceVo> voList = Lists.newArrayList();
        //查询比赛下的所有场次
        List<MthRoundInstance> roundInstanceList=  matchContractorManageDao.findByMatchIdAndRoundType(matchId,areaId, worker);
        if (CollectionUtil.isNotEmpty(roundInstanceList)) {
            MthRoundInstanceVo vo = new MthRoundInstanceVo();
            List<Long> roundIdList = roundInstanceList.stream().map(MthRoundInstance::getId).collect(Collectors.toList());
            //查询所有场次对应的区域
            List<MthRoundArea> roundAreaList = roundAreaRepo.findByRoundIdIn(roundIdList);
            Map<Long, List<MthRoundArea>> roundAreaMap = new HashMap<Long, List<MthRoundArea>>();
            if (CollectionUtil.isNotEmpty(roundAreaList)) {
                roundAreaMap = roundAreaList.stream().collect(Collectors.groupingBy(MthRoundArea::getRoundId));
            }
            //查询奖项
            List<MthRoundPrize> roundPrizeList = roundPrizeRepo.findByRoundIdIn(roundIdList);
            Map<Long, List<MthRoundPrize>> roundPrizeMap = new HashMap<Long, List<MthRoundPrize>>();
            if (CollectionUtil.isNotEmpty(roundPrizeList)) {
                roundPrizeMap = roundPrizeList.stream().collect(Collectors.groupingBy(MthRoundPrize::getRoundId));
            }
            //查询所有场次对应的评分标准
            List<MthRoundReviewStandar> mthRoundReviewStandarList = mthRoundReviewStandarRepo.findByRoundIdIn(roundIdList);
            Map<Long, List<MthRoundReviewStandar>>   reviewStandarMap=new HashMap<>();

            if(!CollectionUtil.isEmpty(mthRoundReviewStandarList)){
                reviewStandarMap= mthRoundReviewStandarList.stream().collect(Collectors.groupingBy(MthRoundReviewStandar::getRoundId));
            }

            //查询评审员
            List<MthRoundReviewer>  mthRoundReviewerList = mthRoundReviewerRepo.findByRoundIdIn(roundIdList);
            Map<Long,List<MthRoundReviewer>> mthRoundReviewerMap=new HashMap<>();
            if ( !CollectionUtil.isEmpty(mthRoundReviewerList)){
                mthRoundReviewerMap=mthRoundReviewerList.stream().collect(Collectors.groupingBy(MthRoundReviewer::getRoundId));
            }
            for (MthRoundInstance roundInstance : roundInstanceList) {
                SignInfoVo signInInfoVo=new SignInfoVo();
                SignInfoVo signOutInfoVo=new SignInfoVo();

                vo = baseConverter.entityToVo(roundInstance);
                //赛事最低等级
                vo.setLowestLevel(matchLevel);
                if (roundAreaMap.containsKey(roundInstance.getId())) {
                    vo.setAreavoList(roundAreaConverter.entityListToVoList(roundAreaMap.get(roundInstance.getId())));
                }
                if (roundPrizeMap.containsKey(roundInstance.getId())) {
                    vo.setPrizeVoList(roundPrizeConverter.entitryListToVoList(roundPrizeMap.get(roundInstance.getId())));
                }
                //评分标准
                if (reviewStandarMap.containsKey(roundInstance.getId())) {
                    vo.setMthRoundReviewStandarBoList(mthRoundReviewConverter.entitryListToVoList(reviewStandarMap.get(roundInstance.getId())));
                }
                //查询签到签退信息
                MthMatchSignInformation signIn = mthMatchSignInformationRepo.findByInstanceIdAndHandle(roundInstance.getId(), 1);
                MthMatchSignInformation signOut = mthMatchSignInformationRepo.findByInstanceIdAndHandle(roundInstance.getId(), -1);
                //是否设置评审员
                if (mthRoundReviewerMap.containsKey(roundInstance.getId())){
                      vo.setReviewer(0);
                }else {
                    vo.setReviewer(1);
                }
                //签到信息
                if (ObjectUtil.isNotEmpty(signIn)){
                    BeanUtil.copyProperties(signIn,signInInfoVo);
                    vo.setSignInInfoVo(signInInfoVo);

                }
                //签退信息
                if (ObjectUtil.isNotEmpty(signOut)){
                    BeanUtil.copyProperties(signOut,signOutInfoVo);
                    vo.setSignOutInfoVo(signOutInfoVo);
                }

                voList.add(vo);

            }



        }
        return voList;
    }

    @Override
    public List<MatchRoundTemplateInfoVo> previewInstance(Long matchId, Worker worker) {
        List<MatchRoundTemplateInfoVo> list = matchContractorManageDao.previewInstance(matchId, worker);
       if (CollectionUtil.isEmpty(list)){
           return list;
       }
        List<Long> roundIdList = list.stream().map(MatchRoundTemplateInfoVo::getId).collect(Collectors.toList());
        //查询奖项
        List<MthRoundPrize> roundPrizeList = roundPrizeRepo.findByRoundIdIn(roundIdList);
        Map<Long, List<MthRoundPrize>> roundPrizeMap = new HashMap<Long, List<MthRoundPrize>>();
        if (CollectionUtil.isNotEmpty(roundPrizeList)) {
            roundPrizeMap = roundPrizeList.stream().collect(Collectors.groupingBy(MthRoundPrize::getRoundId));
        }
        //查询所有场次对应的评分标准
        List<MthRoundReviewStandar> mthRoundReviewStandarList = mthRoundReviewStandarRepo.findByRoundIdIn(roundIdList);
        Map<Long, List<MthRoundReviewStandar>>   reviewStandarMap=new HashMap<>();

        if(!CollectionUtil.isEmpty(mthRoundReviewStandarList)){
            reviewStandarMap= mthRoundReviewStandarList.stream().collect(Collectors.groupingBy(MthRoundReviewStandar::getRoundId));
        }
        for (MatchRoundTemplateInfoVo matchRoundTemplateInfoVo : list) {
            //签到信息
            SignInfoVo signInfoVo=new SignInfoVo();
            BeanUtils.copyProperties(matchRoundTemplateInfoVo,signInfoVo);
            matchRoundTemplateInfoVo.setSignInfoVo(signInfoVo);
            //评分标准
            if (reviewStandarMap.containsKey(matchRoundTemplateInfoVo.getId())) {
                matchRoundTemplateInfoVo.setMthRoundReviewStandarBoList(mthRoundReviewConverter.entitryListToVoList(reviewStandarMap.get(matchRoundTemplateInfoVo.getId())));
            }
            //名次信息
            if (roundPrizeMap.containsKey(matchRoundTemplateInfoVo.getId())) {
                matchRoundTemplateInfoVo.setPrizeVoList(roundPrizeConverter.entitryListToVoList(roundPrizeMap.get(matchRoundTemplateInfoVo.getId())));
            }
        }

        return list;
    }

    @Override
    public MthRoundInstanceVo roundBaseInfo(Long matchId, Worker worker) {
        List<MthRoundInstanceVo> mthRoundInstanceVos = matchContractorManageDao.roundBaseInfo(matchId, worker);
        if (CollectionUtils.isEmpty(mthRoundInstanceVos)){
            throw  new BusinessException("场次信息获失败！");
        }

        return mthRoundInstanceVos.get(0);
    }


    private boolean  checkHour(Date roundTime,Date extendTime){
        if (roundTime==null){
            return true;
        }
        //报名时间与当前时间差  报名时间24小时内
        long diffHour=(roundTime.getTime()-new Date().getTime())/(60 * 60 * 1000);
        // 计算小时差
        if (diffHour<24){
            return true;
        }
        return false;

    }
    private boolean   checkYear(Date roundTime,Date extendTime){
        //设置的时间原先报名时时间差 三年内
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMdd");
        String startDateStr = simpleDateFormat.format(roundTime);
        String endDateStr = simpleDateFormat.format(extendTime);
        long diffYear=  (Long.valueOf(endDateStr) -Long.valueOf (startDateStr)) / 10000;
        if (diffYear>=3){
            return true;
        }
        return false;
    }
    /**
     * 校验场次信息是否完善
     * @param mthRoundInstance
     * @return
     */
   private boolean checkMhRoundInstance(MthRoundInstance mthRoundInstance){
       MthContractorRoundInstanceBo mthContractorRoundInstanceBo = new MthContractorRoundInstanceBo();
       BeanUtils.copyProperties(mthRoundInstance,mthContractorRoundInstanceBo);
       ValidatorFactory vf = Validation.buildDefaultValidatorFactory();
       Validator validator = vf.getValidator();
       Set<ConstraintViolation<MthContractorRoundInstanceBo>> set = validator.validate(mthContractorRoundInstanceBo);
       //set 为空表示场次信息填写完毕
       if (CollectionUtil.isEmpty(set)){
           return true;
       }
        return false;
   }

    public static void main(String[] args) {
        String ss="city";
        String[] split = ss.split(",");
        List<RoundEnums.AreaLevel> matchLevel =Arrays.stream(split).map(item -> RoundEnums.AreaLevel.getAreaByCode(item)). sorted(Comparator.comparing(RoundEnums.AreaLevel::getLevel)).collect(Collectors.toList());
        String Levelcode = matchLevel.get(0).getCode();
        System.out.println(Levelcode);
    }

}
