package com.crunii.micro.service.match.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.entity.main.YcqMatchAnnex;
import com.crunii.micro.domain.repo.main.YcqMatchAnnexRepo;
import com.crunii.micro.service.match.constants.MatchConstants;
import com.crunii.micro.service.match.dto.*;
import com.crunii.micro.service.match.jdbc.MatchEventDao;
import com.crunii.micro.service.match.service.MatchEventService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author baoxy
 * @date 2023-06-26
 */
@Service
public class MatchEventServiceImpl implements MatchEventService {

    @Value("${upload.url}")
    private String urlPrefix;

    @Resource
    private MatchEventDao matchEventDao;

    @Resource
    private YcqMatchAnnexRepo ycqMatchAnnexRepo;

    @Override
    public Result<Object> matchSearchWeb(PageReq<MatchSearchDto> req) {
        PageRsp<MatchSearchVo> rsp = pageSetFileKey(req);
        return Result.ofSuccess(rsp);
    }

    @Override
    public Result<Object> matchSearchApp(PageReq<MatchSearchDto> req) {
        PageRsp<MatchSearchVo> rsp = pageSetFileKey(req);
        return Result.ofSuccess(rsp);
    }

    @Override
    public Result<List<MatchCarouselVo>> matchCarouselWeb(String type) {
        List<MatchCarouselVo> matchCarouselVoList = matchEventDao.matchCarousel(type, MatchConstants.WEB_PIC);
        matchCarouselVoList.forEach(e -> e.setFileKey(urlPrefix + e.getFileKey()));
        return Result.ofSuccess(matchCarouselVoList);
    }

    @Override
    public Result<List<MatchCarouselVo>> matchCarouselApp(String type) {
        List<MatchCarouselVo> matchCarouselVoList = matchEventDao.matchCarousel(type, MatchConstants.NOT_WEB_PIC);
        matchCarouselVoList.forEach(e -> e.setFileKey(urlPrefix + e.getFileKey()));
        return Result.ofSuccess(matchCarouselVoList);
    }

    @Override
    public Result<ArticleDetailVo> articleDetail(String articleId) {
        ArticleDetailVo articleDetailVo = matchEventDao.articleDetail(articleId);
        if (Objects.isNull(articleDetailVo)) {
            return Result.ofSuccess(null);
        }
        // 查询文章对应的附件列表
        List<YcqMatchAnnex> findAllByArticleId = ycqMatchAnnexRepo.findAllByArticleId(articleDetailVo.getArticleId());
        List<ArticleDetailAnnexVo> fileList = new ArrayList<>();
        findAllByArticleId.forEach(e -> {
            ArticleDetailAnnexVo articleDetailAnnexVo = new ArticleDetailAnnexVo();
            articleDetailAnnexVo.setFileId(String.valueOf(e.getAnnexId()));
            articleDetailAnnexVo.setFileName(e.getFileName() + "." + e.getFileType());
            // 数据库里存储的是文件后面key,需要拼接前缀
            articleDetailAnnexVo.setFileKey(urlPrefix + e.getFileKey());
            fileList.add(articleDetailAnnexVo);
        });
        articleDetailVo.setFileList(fileList);
        return Result.ofSuccess(articleDetailVo);
    }

    public PageRsp<MatchSearchVo> pageSetFileKey(PageReq<MatchSearchDto> req) {
        PageRsp<MatchSearchVo> rsp = matchEventDao.matchSearch(req);
        List<MatchSearchVo> matchSearchVoList = rsp.getResult();
        matchSearchVoList.forEach(e -> e.setFileKey(urlPrefix + e.getFileKey()));
        rsp.setResult(matchSearchVoList);
        return rsp;
    }
}
