package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.obs.ObsServiceCodeDto;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.service.match.constants.MatchConstants;
import com.crunii.micro.service.match.dto.*;
import com.crunii.micro.service.match.jdbc.MatchManageDao;
import com.crunii.micro.service.match.service.IMthRoundInstanceService;
import com.crunii.micro.service.match.service.MatchManageService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wangchen create 2023/6/27 9:33
 */
@Service
public class MatchManageServiceImpl implements MatchManageService {

    @Resource
    private YcqArticleDetailRepo ycqArticleDetailRepo;
    @Resource
    private YcqMatchAnnexRepo ycqMatchAnnexRepo;
    @Resource
    private YcqMatchCarouselRepo ycqMatchCarouselRepo;
    @Resource
    private YcqMatchCoverRepo ycqMatchCoverRepo;
    @Value("${upload.url}")
    private String urlPrefix;
    @Resource
    private ObsService obsService;

    private final MatchManageDao matchManageDao;

    @Resource
    private MthMatchInfoRepo matchInfoRepo;
    @Resource
    private IMthRoundInstanceService iMthRoundInstanceService;

    public MatchManageServiceImpl(MatchManageDao matchManageDao) {
        this.matchManageDao = matchManageDao;
    }

    @Override
    public String createArticleDetail(ArticleDetailDto articleDetailDto, Worker worker) {
        Date date = new Date();
        String fileKey = "";
        CoverDto cover = articleDetailDto.getCover();
        if(Objects.nonNull(cover)){
            fileKey = cover.getFileKey();
        }
        //validatedTimeRight(articleDetailDto.getStartShowTime(),articleDetailDto.getEndShowTime(),articleDetailDto.getStartEnrollTime(),articleDetailDto.getEndEnrollTime());
        Long articleId = saveArticle(articleDetailDto, worker,date,fileKey);
        // 标记obsFile的业务编码
        List<ObsServiceCodeDto> obsServiceCodeDtos = new ArrayList<>();
        // 保存封面图片
        saveCover(cover,articleId,obsServiceCodeDtos);
        int carousel = articleDetailDto.getHaveCarousel();
        // 保存轮播图
        if(carousel == 1){
            saveCarousel(date,articleDetailDto.getCarouselCreateInfo(),articleId,obsServiceCodeDtos);
        }
        // 保存附件
        saveAnnex(articleDetailDto.getAnnexCreateList(),articleId,obsServiceCodeDtos);
        obsService.updateObsServiceCode(obsServiceCodeDtos);
        return "保存成功";
    }

    @Override
    public void relateMatch(RelateMatchDto relateMatchDto){
        Optional<YcqArticleDetail> detailOptional = ycqArticleDetailRepo.findById(relateMatchDto.getArticleId());
        if (detailOptional.isPresent()) {
            YcqArticleDetail ycqArticleDetail = detailOptional.get();
            ycqArticleDetail.setMatchId(relateMatchDto.getMatchId());
            ycqArticleDetailRepo.saveAndFlush(ycqArticleDetail);
        }
    }

    public void cancelRelateMatch(RelateMatchDto relateMatchDto){
        Optional<YcqArticleDetail> detailOptional = ycqArticleDetailRepo.findById(relateMatchDto.getArticleId());
        if (detailOptional.isPresent()) {
            YcqArticleDetail ycqArticleDetail = detailOptional.get();
            ycqArticleDetail.setMatchId(null);
            ycqArticleDetailRepo.saveAndFlush(ycqArticleDetail);
        }
    }

    /**
     * 保存封面图片
     */
    private void saveCover(CoverDto cover,Long articleId,List<ObsServiceCodeDto> obsServiceCodeDtos) {
        if(Objects.isNull(cover)){
            return;
        }
        YcqMatchCover matchCover = new YcqMatchCover();
        BeanUtils.copyProperties(cover,matchCover);
        matchCover.setArticleId(articleId);
        matchCover.setFileKey(matchCover.getFileKey());
        matchCover.setState(1);
        YcqMatchCover ycqMatchCover = ycqMatchCoverRepo.saveAndFlush(matchCover);
        ObsServiceCodeDto obsServiceCodeDto = new ObsServiceCodeDto();
        obsServiceCodeDto.setFileId(cover.getFileId());
        obsServiceCodeDto.setServiceId(ycqMatchCover.getCoverId());
        obsServiceCodeDto.setServiceCode(CommonConstants.OBS_FILE_MATCH_CODE);
        obsServiceCodeDtos.add(obsServiceCodeDto);
    }

    @Override
    @InjectProperty
    public PageRsp<PageArticleVo> pageArticle(PageReq<PageArticleDto> req, Worker worker) {
        PageRsp<PageArticleVo> pageRsp = matchManageDao.pageArticle(req);
        List<PageArticleVo> result = pageRsp.getResult();
        if(CollectionUtils.isEmpty(result)){
            return pageRsp;
        }
        List<Long> matchIdList = StreamUtils.toList(result, PageArticleVo::getMatchId);
        List<MthMatchInfo> matchInfos = matchInfoRepo.findByIdIn(matchIdList);
        Map<Long, MthMatchInfo> matchInfoMap = StreamUtils.toIdentityMap(matchInfos, MthMatchInfo::getId);
        // 查询当前文章的组织以及上级组织id
        for (PageArticleVo pageArticleVo : result) {
            List<String> orgIds = matchManageDao.findOrgIdsByOrgId(pageArticleVo.getOrgId());
            // 设置当前人是否可操作
            if(orgIds.contains(worker.getOrgId()) || CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())){
                pageArticleVo.setAllowModify(1);
            }else {
                pageArticleVo.setAllowModify(0);
            }
            MthMatchInfo mthMatchInfo = matchInfoMap.get(pageArticleVo.getMatchId());
            if(mthMatchInfo != null){
                pageArticleVo.setMatchName(mthMatchInfo.getMatchName());
                pageArticleVo.setMatchStatus(mthMatchInfo.getStatus());
            }
        }
        return pageRsp;
    }

    @Override
    public List<PageArticleVo> listArticle(PageArticleDto req) {
        return matchManageDao.listArticle(req);
    }

    @Override
    public List<YcqArticleDetail> findByMatchId(Long matchId) {
        return ycqArticleDetailRepo.findByMatchId(matchId);
    }

    @Override
    public String updateArticleState(ArticleStateDto articleStateDto, Worker worker) {
        YcqArticleDetail one = ycqArticleDetailRepo.getOne(articleStateDto.getArticleId());
        if(Objects.isNull(one)){
            throw new BusinessException("该文章不存在！");
        }
        // 验证是否有权限修改
        validatedIsModify(worker.getOrgId(), one.getOrgId(),worker.getLoginId());
        Integer articleState = articleStateDto.getArticleState();
        updateArticleState(articleState,one,worker);
        return "更新成功";
    }

    @Override
    public String delArticle(Long articleId,Worker worker) {
        if(Objects.isNull(articleId)){
            throw new BusinessException("文章id不能为空");
        }
        YcqArticleDetail one = ycqArticleDetailRepo.getOne(articleId);
        if(Objects.nonNull(one)){
            validatedIsModify(worker.getOrgId(),one.getOrgId(),worker.getLoginId());
            one.setState(0);
            ycqArticleDetailRepo.saveAndFlush(one);
        }
        return "删除成功";
    }

    @Override
    @InjectProperty
    public BasicArticleVo getBasicArticle(Long articleId) {
        if(Objects.isNull(articleId)){
            throw new BusinessException("文章id不能为空");
        }
        YcqArticleDetail one = ycqArticleDetailRepo.getOne(articleId);
        if(Objects.isNull(one)){
            throw new BusinessException("该文章文章不存在");
        }
        return paddingBasicArticle(one);
    }

    @Override
    public String getArticleContext(Long articleId) {
        if(Objects.isNull(articleId)){
            throw new BusinessException("文章id不能为空");
        }
        YcqArticleDetail one = ycqArticleDetailRepo.getOne(articleId);
        if(Objects.isNull(one)){
            throw new BusinessException("该文章文章不存在");
        }
        return one.getArticleContent();
    }

    @Override
    public String updateArticle(ArticleUpdateDto articleUpdate,Worker worker) {
        Long articleId = articleUpdate.getArticleId();
        YcqArticleDetail one = ycqArticleDetailRepo.getOne(articleId);
        if(Objects.isNull(one)){
            throw new BusinessException("该文章不存在");
        }
        //validatedTimeRight(articleUpdate.getStartShowTime(),articleUpdate.getEndShowTime(),articleUpdate.getStartEnrollTime(),articleUpdate.getEndEnrollTime());
        handleAssociationInfo(articleUpdate,one,worker);
        return "更新成功";
    }

    /**
     * 验证时间是否满足需求
     */
    private void validatedTimeRight(Date startShowTime, Date endShowTime, Date startEnrollTime, Date endEnrollTime) {
        // 不为空 证明为报名中
        if(Objects.nonNull(startEnrollTime)){
            boolean in = !DateUtil.isIn(startEnrollTime, startShowTime, endShowTime);
            boolean in2 = !DateUtil.isIn(endEnrollTime, startShowTime, endShowTime);
            if(in || in2){
                throw new BusinessException("报名时间应在展示周期范围内！");
            }
        }else {
            // 展示周期不可超过一年
            DateTime dateTime = DateUtil.offsetMonth(startShowTime, 12);
            int compare = DateUtil.compare(dateTime, endShowTime);
            if(compare != 1){
                throw new BusinessException("展示周期不可超过一年");
            }
        }
    }

    @Override
    public String updateCarousel(CarouselShowDto carousel,Worker worker) {
        YcqArticleDetail one = ycqArticleDetailRepo.getOne(carousel.getArticleId());
        if(Objects.isNull(one)){
            throw new BusinessException("文章不存在");
        }
        if(carousel.getCarousel().equals(1) && one.getArticleState() == 1){
            validatedCarouselOverTen(one.getArticleType());
        }
        one.setUpdateTime(new Date());
        one.setUpdateWorker(worker.getLoginId());
        one.setCarousel(carousel.getCarousel());
        ycqArticleDetailRepo.saveAndFlush(one);
        return "更新成功";
    }

    private void handleAssociationInfo(ArticleUpdateDto articleUpdate,YcqArticleDetail one,Worker worker) {
        List<ObsServiceCodeDto> obsServiceCodeDtos = new ArrayList<>();
        Date now = new Date();
        Long articleId = articleUpdate.getArticleId();
        List<String> fileKey = new ArrayList<>();
        // 处理轮播图
        YcqMatchCarousel webPicAndState = ycqMatchCarouselRepo.findByArticleIdAndWebPic(articleId, 1);
        YcqMatchCarousel appPicAndState = ycqMatchCarouselRepo.findByArticleIdAndWebPic(articleId, 0);
        if(articleUpdate.getHaveCarousel() == 1){
            CarouselInfoDto carouselCreateInfo = articleUpdate.getCarouselCreateInfo();
            // 之前轮播为空则新增，不为空则修改
            if(Objects.nonNull(webPicAndState)){
                DateTime dateTime = DateUtil.offsetDay(now, carouselCreateInfo.getShowDay());
                CarouselDto webCarousel = carouselCreateInfo.getWebCarousel();
                CarouselDto appCarousel = carouselCreateInfo.getAppCarouselDto();
                webCarousel.setFileKey(webCarousel.getFileKey());
                // 图片不相等则删除
                if(!webCarousel.getFileKey().equals(webPicAndState.getFileKey())){
                    fileKey.add(webPicAndState.getFileKey());
                }
                if(!appCarousel.getFileKey().equals(appPicAndState.getFileKey())){
                    fileKey.add(appPicAndState.getFileKey());
                }
                BeanUtils.copyProperties(webCarousel,webPicAndState);
                webPicAndState.setShowDay(carouselCreateInfo.getShowDay());
                webPicAndState.setStartTime(now);
                webPicAndState.setEndTime(dateTime);
                BeanUtils.copyProperties(appCarousel,appPicAndState);
                appPicAndState.setShowDay(carouselCreateInfo.getShowDay());
                appPicAndState.setStartTime(now);
                appPicAndState.setEndTime(dateTime);
                // 标记obsFile的业务编码
                YcqMatchCarousel ycqMatchCarousel = ycqMatchCarouselRepo.saveAndFlush(webPicAndState);
                ObsServiceCodeDto obsServiceCodeDto = new ObsServiceCodeDto();
                obsServiceCodeDto.setFileId(webCarousel.getFileId());
                obsServiceCodeDto.setServiceId(ycqMatchCarousel.getCarouselId());
                obsServiceCodeDto.setServiceCode(CommonConstants.OBS_FILE_MATCH_CODE);
                obsServiceCodeDtos.add(obsServiceCodeDto);
                YcqMatchCarousel appMatchCarousel = ycqMatchCarouselRepo.saveAndFlush(webPicAndState);
                ObsServiceCodeDto obsServiceCode = new ObsServiceCodeDto();
                obsServiceCode.setFileId(appCarousel.getFileId());
                obsServiceCode.setServiceId(appMatchCarousel.getCarouselId());
                obsServiceCode.setServiceCode(CommonConstants.OBS_FILE_MATCH_CODE);
                obsServiceCodeDtos.add(obsServiceCode);
            }else {
                saveCarousel(now,carouselCreateInfo,articleId,obsServiceCodeDtos);
            }
        }else {
            // 删除
            if(Objects.nonNull(webPicAndState)){
                ycqMatchCarouselRepo.delete(webPicAndState);
            }
            if(Objects.nonNull(appPicAndState)){
                ycqMatchCarouselRepo.delete(appPicAndState);
            }
        }
        // 处理封面
        CoverDto cover = articleUpdate.getCover();
        cover.setFileKey(cover.getFileKey());
        YcqMatchCover byArticleIdAndState = ycqMatchCoverRepo.findByArticleIdAndState(articleId, 1);
        if(!cover.getFileKey().equals(byArticleIdAndState.getFileKey())){
            fileKey.add(byArticleIdAndState.getFileKey());
        }
        BeanUtils.copyProperties(cover,byArticleIdAndState);
        YcqMatchCover ycqMatchCover = ycqMatchCoverRepo.saveAndFlush(byArticleIdAndState);
        ObsServiceCodeDto obsServiceCode = new ObsServiceCodeDto();
        obsServiceCode.setFileId(cover.getFileId());
        obsServiceCode.setServiceId(ycqMatchCover.getCoverId());
        obsServiceCode.setServiceCode(CommonConstants.OBS_FILE_MATCH_CODE);
        obsServiceCodeDtos.add(obsServiceCode);

        // 处理附件
        List<YcqMatchAnnex> allByArticleId = ycqMatchAnnexRepo.findAllByArticleId(articleId);
        List<AnnexDto> annexCreateList = articleUpdate.getAnnexCreateList();
        if(CollectionUtils.isEmpty(allByArticleId)){
            saveAnnex(annexCreateList,articleId,obsServiceCodeDtos);
        }else {
            // 不为null则修改 为null则删除
            if(!CollectionUtils.isEmpty(annexCreateList)){
                annexCreateList.forEach(x->x.setFileKey(x.getFileKey()));
                List<String> collect = annexCreateList.stream().map(AnnexDto::getFileKey).collect(Collectors.toList());
                for (YcqMatchAnnex ycqMatchAnnex : allByArticleId) {
                    if(!collect.contains(ycqMatchAnnex.getFileKey())){
                        fileKey.add(ycqMatchAnnex.getFileKey());
                    }
                }
                // 删除附件
                matchManageDao.delAnnexByArticleId(articleId);
                // 保存附件
                saveAnnex(annexCreateList,articleId,obsServiceCodeDtos);
            }else {
               matchManageDao.delAnnexByArticleId(articleId);
            }
        }
        // 处理主表
        BeanUtils.copyProperties(articleUpdate,one);
        // 设置了轮播图修改轮播展示状态
        if(articleUpdate.getHaveCarousel() == 1){
            one.setCarousel(1);
        }else{
            one.setCarousel(0);
        }
        one.setShowDue(0);
        one.setFileKey(cover.getFileKey());
        one.setCarousel(articleUpdate.getHaveCarousel());
        one.setLoginName(matchManageDao.findWorkerNameByLoginId(one.getLoginId()));
        one.setArticleState(0);
        one.setUpdateWorker(worker.getLoginId());
        one.setUpdateTime(now);
        one.setPublishTime(null);
        ycqArticleDetailRepo.saveAndFlush(one);
        if(!CollectionUtils.isEmpty(fileKey)){
            obsService.deleteListFileKey(fileKey);
        }
        obsService.updateObsServiceCode(obsServiceCodeDtos);
    }

    /**
     * 填充文章基本信息
     */
    private BasicArticleVo paddingBasicArticle(YcqArticleDetail one) {
        BasicArticleVo basicArticleVo = new BasicArticleVo();
        BeanUtils.copyProperties(one,basicArticleVo);
        if(basicArticleVo.getHaveCarousel() == 1){
            // 设置web轮播图
            CarouselInfoDto carouselInfoDto = new CarouselInfoDto();
            YcqMatchCarousel webMatchCarousel = ycqMatchCarouselRepo.findByArticleIdAndWebPic(one.getArticleId(), 1);
            carouselInfoDto.setShowDay(webMatchCarousel.getShowDay());
            CarouselDto webCarouse = new CarouselDto();
            BeanUtils.copyProperties(webMatchCarousel,webCarouse);
            webCarouse.setFileUrl(urlPrefix+webCarouse.getFileKey());
            carouselInfoDto.setWebCarousel(webCarouse);
            // 设置app轮播图
            YcqMatchCarousel appMatchCarousel = ycqMatchCarouselRepo.findByArticleIdAndWebPic(one.getArticleId(), 0);
            CarouselDto appCarouse = new CarouselDto();
            BeanUtils.copyProperties(appMatchCarousel,appCarouse);
            appCarouse.setFileUrl(urlPrefix+appCarouse.getFileKey());
            carouselInfoDto.setAppCarouselDto(appCarouse);
            basicArticleVo.setCarouselInfo(carouselInfoDto);
        }
        // 设置封面图片
        YcqMatchCover byArticleIdAndState = ycqMatchCoverRepo.findByArticleIdAndState(basicArticleVo.getArticleId(), 1);
        CoverDto coverDto = new CoverDto();
        BeanUtils.copyProperties(byArticleIdAndState,coverDto);
        coverDto.setFileUrl(urlPrefix+byArticleIdAndState.getFileKey());
        basicArticleVo.setCover(coverDto);
        // 设置附件
        List<AnnexDto> annexList = matchManageDao.findAnnex(one.getArticleId());
        for (AnnexDto annexDto : annexList) {
            annexDto.setFileUrl(urlPrefix+annexDto.getFileKey());
        }
        basicArticleVo.setAnnexCreateList(annexList);
        return basicArticleVo;
    }

    /**
     * 修改文章状态
     */
    private void updateArticleState(Integer articleState, YcqArticleDetail one,Worker worker) {
        Date now = new Date();
        one.setUpdateTime(now);
        one.setArticleState(articleState);
        one.setUpdateWorker(worker.getLoginId());
        if(MatchConstants.ARTICLE_WITHDRAWN.equals(articleState)){
            one.setPublishTime(null);
            ycqArticleDetailRepo.saveAndFlush(one);
            return;
        }
        // 发布
        if(MatchConstants.ARTICLE_PUBLISHED.equals(articleState)){
            int haveCarousel = one.getHaveCarousel();
            int carousel = one.getCarousel();
            if(haveCarousel==1 && carousel==1){
                validatedCarouselOverTen(one.getArticleType());
            }
            one.setPublishTime(now);
            ycqArticleDetailRepo.saveAndFlush(one);
            //如果类型是赛事结果而且有场次id则是发布公示，需要更新场次状态
            if (one.getArticleType()==4&& ObjectUtil.isNotNull(one.getRoundId())){
                try {
                    iMthRoundInstanceService.doPulicity(one.getRoundId());
                } catch (Exception e) {

                    e.printStackTrace();
                    throw new BusinessException("场次状态变更失败！");

                }
            }

            return;
        }
        throw new BusinessException("当前状态有误！");
    }

    /**
     * 分快块验证轮播图是否超过10个
     */
    private void validatedCarouselOverTen(Integer articleType) {
        List<Integer> articleTypeList = new ArrayList<>();
        List<String> matchOngoingTypeList = MatchConstants.MATCH_ONGOING_TYPE_LIST;
        if(matchOngoingTypeList.contains(articleType+"")){
            for (String s : matchOngoingTypeList) {
                articleTypeList.add(Integer.parseInt(s));
            }
        }else {
            articleTypeList.add(articleType);
        }
        Integer count = matchManageDao.findCarouseCount(articleTypeList);
        if(Objects.nonNull(count) && count>=10){
            throw new BusinessException("轮播数量已达最大！");
        }
    }

    /**
     * 验证是否有权限修改
     */
    private void validatedIsModify(String orgId,String articleOrgId,Integer loginId) {
        List<String> orgIds = matchManageDao.findOrgIdsByOrgId(articleOrgId);
        if(!orgIds.contains(orgId) && !CommonConstants.SUPER_LOGIN_IDS.contains(loginId)){
            throw new BusinessException("当前用户不可操作该文章！");
        }
    }

    /**
     * 保存附件
     */
    private void saveAnnex(List<AnnexDto> annexCreateList, Long articleId,List<ObsServiceCodeDto> obsServiceCodeDtos) {
        if(CollectionUtils.isEmpty(annexCreateList)){
            return;
        }
        for (AnnexDto annexCreateDto : annexCreateList) {
            YcqMatchAnnex ycqMatchAnnex = new YcqMatchAnnex();
            BeanUtils.copyProperties(annexCreateDto,ycqMatchAnnex);
            ycqMatchAnnex.setState(1);
            ycqMatchAnnex.setArticleId(articleId);
            ycqMatchAnnex.setFileKey(ycqMatchAnnex.getFileKey());
            YcqMatchAnnex ycqMatchAnnex1 = ycqMatchAnnexRepo.saveAndFlush(ycqMatchAnnex);
            ObsServiceCodeDto obsServiceCodeDto = new ObsServiceCodeDto();
            obsServiceCodeDto.setFileId(annexCreateDto.getFileId());
            obsServiceCodeDto.setServiceId(ycqMatchAnnex1.getAnnexId());
            obsServiceCodeDto.setServiceCode(CommonConstants.OBS_FILE_MATCH_CODE);
            obsServiceCodeDtos.add(obsServiceCodeDto);
        }
    }

    /**
     * 保存轮播图
     */
    private void saveCarousel(Date startTime, CarouselInfoDto carouselCreateInfo,Long articleId,List<ObsServiceCodeDto> obsServiceCodeDtos) {
        if(Objects.isNull(carouselCreateInfo)){
            throw new BusinessException("轮播不能为空");
        }
        if(Objects.isNull(carouselCreateInfo.getShowDay())){
            throw new BusinessException("轮播天数不能为空");
        }
        DateTime endTime = DateUtil.offsetDay(startTime, carouselCreateInfo.getShowDay());
        YcqMatchCarousel web = new YcqMatchCarousel();
        CarouselDto webCarousel = carouselCreateInfo.getWebCarousel();
        if(!StringUtils.hasText(webCarousel.getFileKey())){
            throw new BusinessException("web fileKey不能为空");
        }
        BeanUtils.copyProperties(webCarousel,web);
        web.setArticleId(articleId);
        web.setStartTime(startTime);
        web.setEndTime(endTime);
        web.setShowDay(carouselCreateInfo.getShowDay());
        web.setState(1);
        web.setWebPic(1);
        web.setFileKey(web.getFileKey());
        YcqMatchCarousel ycqMatchCarousel = ycqMatchCarouselRepo.saveAndFlush(web);
        ObsServiceCodeDto obsServiceCodeDto = new ObsServiceCodeDto();
        obsServiceCodeDto.setFileId(webCarousel.getFileId());
        obsServiceCodeDto.setServiceId(ycqMatchCarousel.getCarouselId());
        obsServiceCodeDto.setServiceCode(CommonConstants.OBS_FILE_MATCH_CODE);
        obsServiceCodeDtos.add(obsServiceCodeDto);


        YcqMatchCarousel app = new YcqMatchCarousel();
        CarouselDto appCarousel = carouselCreateInfo.getAppCarouselDto();
        if(!StringUtils.hasText(appCarousel.getFileKey())){
            throw new BusinessException("app fileKey不能为空");
        }
        BeanUtils.copyProperties(appCarousel,app);
        app.setArticleId(articleId);
        app.setStartTime(startTime);
        app.setEndTime(endTime);
        app.setShowDay(carouselCreateInfo.getShowDay());
        app.setState(1);
        app.setWebPic(0);
        app.setFileKey(web.getFileKey());
        YcqMatchCarousel appMatchCarousel = ycqMatchCarouselRepo.saveAndFlush(app);
        ObsServiceCodeDto obsServiceCode = new ObsServiceCodeDto();
        obsServiceCode.setFileId(appCarousel.getFileId());
        obsServiceCode.setServiceId(appMatchCarousel.getCarouselId());
        obsServiceCode.setServiceCode(CommonConstants.OBS_FILE_MATCH_CODE);
        obsServiceCodeDtos.add(obsServiceCode);
    }


    /**
     * 保存文章内容（返回文章id）
     */
    private Long saveArticle(ArticleDetailDto articleDetailDto,Worker worker,Date date, String fileKey){
        YcqArticleDetail articleDetail = new YcqArticleDetail();
        BeanUtils.copyProperties(articleDetailDto,articleDetail);
        articleDetail.setArticleState(0);
        articleDetail.setCreateTime(date);
        articleDetail.setLoginName(matchManageDao.findWorkerNameByLoginId(articleDetailDto.getLoginId()));
        articleDetail.setCreateWorker(worker.getLoginId());
        articleDetail.setUpdateTime(date);
        articleDetail.setUpdateWorker(worker.getLoginId());
        articleDetail.setState(1);
        articleDetail.setCarousel(articleDetailDto.getHaveCarousel());
        articleDetail.setFileKey(fileKey);
        YcqArticleDetail ycqArticleDetail = ycqArticleDetailRepo.saveAndFlush(articleDetail);
        return ycqArticleDetail.getArticleId();
    }
}
