package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.AESUtil;
import com.crunii.micro.common.utils.RSAUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.platform.components.Locker;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.MatchOrganizerAccountPageVO;
import com.crunii.micro.service.match.api.vo.MthMatchContractorVO;
import com.crunii.micro.service.match.api.vo.OrganizerAccountInfoVo;
import com.crunii.micro.service.match.api.vo.TsmWorInfoVO;
import com.crunii.micro.service.match.enums.ContractorEnums.LimitType;
import com.crunii.micro.service.match.enums.MatchInfoStatusEnums;
import com.crunii.micro.service.match.enums.MatchInsertRoleEnum.RoleType;
import com.crunii.micro.service.match.enums.RoundEnums.AreaLevel;
import com.crunii.micro.service.match.enums.RoundEnums.RoundType;
import com.crunii.micro.service.match.jdbc.MatchOrganizerAccountDao;
import com.crunii.micro.service.match.jdbc.TsmWorkerDao;
import com.crunii.micro.service.match.service.MatchOrganizerAccountService;
import com.crunii.micro.service.match.service.PushBusinessMsgService;
import com.google.common.collect.Maps;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.Transactional;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

/**
 * @Author thw
 * @date 2023/11/16
 * @description 赛事发布管理-生成承办方相关功能
 */
@Slf4j
@Service(value = "MatchOrganizerAccountServiceImpl")
public class MatchOrganizerAccountServiceImpl implements MatchOrganizerAccountService {

  @Autowired
  private MatchOrganizerAccountDao matchOrganizerAccountDao;

  @Autowired
  private MthRoundInstanceRepo mthRoundInstanceRepo;

  @Autowired
  private MthMatchInfoRepo mthMatchInfoRepo;

  @Autowired
  private MthMatchContractorRepo mthMatchContractorRepo;

  @Autowired
  private TsmWorkerRepo tsmWorkerRepo;

  @Autowired
  private TsmRoleRepo tsmRoleRepo;

  @Autowired
  private TsmWorkerRoleRepo tsmWorkerRoleRepo;

  @Autowired
  private TsmWorkerDao tsmWorkerDao;

  @Autowired
  private MthRoundAreaRepo mthRoundAreaRepo;

  @Autowired
  private MatchReviewerManageServiceImpl matchReviewerManageService;

  @Autowired
  private Locker locker;

  @Autowired
  private TsmOrgRepo tsmOrgRepo;

  @Autowired
  private PushBusinessMsgService pushBusinessMsgService;

  //生成承办方临时账号锁参数
  private final static String MATCH_ORGANIZER = "LOCK:MATCH:ORGANIZER";


  private String[] limitTypeValue = {LimitType.LONG_TERM.getCode(), LimitType.TEMP.getCode()};

  private String roleName = RoleType.ORGANIZER_ROLE.getName();

  @Override
  public PageRsp<MatchOrganizerAccountPageVO> pageOrganizerAccountInfo(
      PageReq<MatchOrganizerAccountReqBo> bo) {

    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    PageRsp<MatchOrganizerAccountPageVO> info = matchOrganizerAccountDao
        .pageOrganizerAccountInfo(bo, currWorker);

    if (CollectionUtils.isNotEmpty(info.getResult())) {
      info.getResult().stream().forEach(e -> {
        // 处理系统账号授权承办方内容
        if (StringUtils.isNotEmpty(e.getContractorIdList())) {
          String[] entities = e.getContractorIdList().split(",");
          List<OrganizerAccountInfoVo> list = new ArrayList();
          for (String entity : entities) {
            OrganizerAccountInfoVo organizerAccountInfoVo = new OrganizerAccountInfoVo();
            String[] value = entity.split("-");
            // 设置组织
            organizerAccountInfoVo.setOrgId(Long.valueOf(value[0]));
            organizerAccountInfoVo.setOrgName(value[1]);
            // 设置组织人员
            organizerAccountInfoVo.setContractorId(Long.valueOf(value[2]));
            organizerAccountInfoVo.setContractorName(value[3]);
            organizerAccountInfoVo.setLimitType(limitTypeValue[0]);
            list.add(organizerAccountInfoVo);
          }
          // 通过组织id分组
          Map<Long, List<OrganizerAccountInfoVo>> mapGroupInfo = list.stream()
              .collect(Collectors.groupingBy(OrganizerAccountInfoVo::getOrgId));
//          if (mapGroupInfo.size() > 1) {
//            throw new BusinessException("当前组织只能唯一!");
//          }
          // 封装成前端需要的格式
//          ArrayList<OrganizerAccountVO> accountVOS = new ArrayList<>();
//          OrganizerAccountVO organizerAccountVO = new OrganizerAccountVO();
          // 类型变动后 2023-12-03
          OrganizerAccountContentBo accountVO = new OrganizerAccountContentBo();
          List<String> OrgIds = new ArrayList<>();
          List<String> orgName = new ArrayList<>();
          List<Integer> workerIds = new ArrayList<>();
          for (Map.Entry<Long, List<OrganizerAccountInfoVo>> entry : mapGroupInfo.entrySet()) {
            // 设置组织
//            organizerAccountVO.setOrgId(entry.getKey().toString());
            OrgIds.add(entry.getKey().toString());

            if (ObjectUtil.isNotEmpty(entry.getValue())){
              // 设置组织名称
              orgName.add(entry.getValue().get(0).getOrgName());
              List<Integer> longList = entry.getValue().stream()
                  .map(OrganizerAccountInfoVo::getContractorId).map(Math::toIntExact).collect(Collectors.toList());
              // 设置用户id
//            organizerAccountVO.setContractorId(longList);
//            accountVOS.add(organizerAccountVO);
              workerIds.addAll(longList);
            }
          }
//          e.setAccountList(accountVOS);
          accountVO.setOrgId(OrgIds);
          accountVO.setContractorId(workerIds);
          accountVO.setOrgName(orgName);
          e.setAccount(accountVO);
        }

        // 处理临时账号
        if (StringUtils.isNotBlank(e.getTempId())) {
          String[] temp = e.getTempId().split(",");
          if (temp.length > 1) {
            throw new BusinessException("每个范围生成的临时账号唯一!");
          }
          for (String t : temp) {
            String[] value = t.split("-");
            Map<String, Object> props = new HashMap<>();
            props.put("loginId", Long.valueOf(value[0]));
            List<TsmWorker> tsmWorkers = tsmWorkerRepo.findByProps(props);
            if (CollectionUtils.isNotEmpty(tsmWorkers)) {
              if (tsmWorkers.size() > 1) {
                throw new BusinessException("通过一个账号查出多条记录!");
              }
              // 默认手机号为账号
              String mobileAccount = tsmWorkers.get(0).getMobile();
              String loginPwd = tsmWorkers.get(0).getLoginPwd();
              // 临时账号
              e.setContractorId(RSAUtil.encrypt(mobileAccount));
              // 密码解密后加密
              e.setContractorIdPwd(RSAUtil.encrypt(AESUtil.decrypt(loginPwd)));
            }
          }
        }

        // 关键内容置空
        e.setContractorIdList(null);
        e.setTempId(null);
      });
    }
    return info;
  }

  @Override
  @Transactional(rollbackOn = Exception.class)
  public Result saveTempOrganizerAccount(MatchOrganizerAccountReqBo bo) {

    Worker currWorker = CurrWorkerUtil.getCurrWorker();

    if (ObjectUtil.isEmpty(bo)) {
      throw new BusinessException("参数对象必传!");
    }
    if (ObjectUtil.isEmpty(bo.getMatchId())) {
      throw new BusinessException("比赛id不能为空!");
    }

    locker.lock(MATCH_ORGANIZER, bo.getMatchId().toString());

    try {
//      log.info("{} lockMatchId: {}", MATCH_ORGANIZER, bo.getMatchId().toString());
      //通过比赛id查询比赛是否已发布（比赛发布后才能操作生成承办方账号）
      Map<String, Object> props = Maps.newHashMap();
      props.put("id", bo.getMatchId());
      props.put("status", MatchInfoStatusEnums.RELEASED.getStatus());
      props.put("delFlag", 0);
      List<MthMatchInfo> matchInfoRepoByProps = mthMatchInfoRepo.findByProps(props);
      if (CollectionUtils.isEmpty(matchInfoRepoByProps)) {
        throw new BusinessException("已发布比赛才可生成承办方账号!");
      }
      if (matchInfoRepoByProps.size() > 1) {
        throw new BusinessException("比赛id重复,数据异常!");
      }

      MthMatchInfo mthMatchInfo = matchInfoRepoByProps.get(0);

      // 判断该比赛是否已存在承办方账号
      List<MthMatchContractor> matchContractors = mthMatchContractorRepo.findByMatchIdAndLimitType(bo.getMatchId(), limitTypeValue[1]);
      if (CollectionUtils.isNotEmpty(matchContractors)) {
        throw new BusinessException("该比赛已经生成了承办方临时账号，不允许重复生成!");
      }else {
        // 通过比赛id查询范围
        List<MthRoundInstance> instance = mthRoundInstanceRepo
            .findByMatchIdAndRoundSortAndRoundTypeAndDelFlag(bo.getMatchId(), 1,
                RoundType.INSTANCE.getCode(), 0);
        if (CollectionUtils.isNotEmpty(instance)) {
          // 创建并生成临时账号
          List<TsmWorker> workerList = doAddTsmWorkers(instance.size(), "承办方临时账号");
          // 查询指定角色.角色唯一
          TsmRole role = getTsmRoles(roleName);
          // 将临时账号与角色关联
          Set<Integer> workerIds = workerList.stream().map(TsmWorker::getLoginId)
              .collect(Collectors.toSet());
          insertWorkerRole(workerIds, role.getRoleId());
          // 创建并生成承办方信息
          createMatchContractor(currWorker, instance, workerList);

          // 生产承办方临时账号时发送管理端消息
          String matchName = "";
          if (ObjectUtil.isNotEmpty(mthMatchInfo)){
            matchName = mthMatchInfo.getMatchName();
          }
          List<Long> workerIdList = workerList.stream().map(item -> Long.valueOf(item.getLoginId()))
              .collect(Collectors.toList());
          log.info("完善比赛信息通知:{} ",matchName);
          pushBusinessMsgService.pushImproveMatchInfoMsg(workerIdList,matchName);

        }
      }

    }finally {
      locker.unlock();
    }
    return Result.ofSuccess("生成承办方临时账号成功!");
  }


  @Override
  public Result updateOrganizerAccount(List<MatchOrganizerAccountUpdateBo> bo) {

    Worker currWorker = CurrWorkerUtil.getCurrWorker();

    if (ObjectUtil.isNotEmpty(bo)) {
      for (MatchOrganizerAccountUpdateBo accountUpdateBo : bo) {
        if (ObjectUtil.isEmpty(accountUpdateBo.getMatchId()) || ObjectUtil.isEmpty(accountUpdateBo.getAreaId())){
          throw new BusinessException("比赛id与区域id必传！");
        }
        HashMap<String, Object> props = new HashMap<>();
        props.put("matchId", accountUpdateBo.getMatchId());
        props.put("areaId", accountUpdateBo.getAreaId());
        props.put("limitType", limitTypeValue[0]);
        props.put("delFlag", 0);
        // 查询表中是否已经存在关联
        List<MthMatchContractor> queryContent = mthMatchContractorRepo.findByProps(props);
        if (CollectionUtils.isNotEmpty(queryContent)) {
          // 逻辑删除承办方信息表中以前关联的内容
          mthMatchContractorRepo.updateDelFlagByMatchIdAndAreaIdAndLimitType(Long.valueOf(accountUpdateBo.getMatchId()),
              Long.valueOf(accountUpdateBo.getAreaId()), limitTypeValue[0]);

          List<Integer> workerIds = queryContent.stream().map(e -> e.getContractorId().intValue())
              .collect(Collectors.toList());
          // 查询指定角色.角色唯一
          TsmRole role = getTsmRoles(roleName);
          // 删除添加的角色
          tsmWorkerRoleRepo.deleteAllByRoleIdAndLoginIdIn(role.getRoleId(), workerIds);
        }

        // account为空说明不需要更改
        OrganizerAccountContentBo account = accountUpdateBo.getAccount();
        if (ObjectUtil.isNotEmpty(account)){
//          for (OrganizerAccountVO account : accounts) {
            if (ObjectUtil.isNotEmpty(account.getOrgId()) && ObjectUtil.isNotEmpty(account.getContractorId())) {

              // 通过AreaId查询区域名称
              List<MthRoundArea> areaContent = mthRoundAreaRepo.findByIdIn(Arrays.asList(Long.valueOf(accountUpdateBo.getAreaId())));
              if (CollectionUtils.isEmpty(areaContent)) {
                throw new BusinessException("通过区域id未找到对应的区域!");
              }
              if (areaContent.size() > 1) {
                throw new BusinessException("区域id对应区域名称不唯一.");
              }
              String areaName = areaContent.get(0).getAreaName();

              // 通过具体的workerId查询相关组织
//              List<Integer> workerIds = accountUpdateBo.getAccount().getContractorId().stream()
//                  .map(e -> Integer.valueOf(Math.toIntExact(e))).collect(Collectors.toList());
              List<Integer> workerIds = account.getContractorId().stream()
                  .map(Math::toIntExact).collect(Collectors.toList());
              List<TsmWorInfoVO> tsmWorkers = tsmWorkerDao.findByWorker(workerIds, null);
              // 创建承办方信息，此时关联的是系统已经存在的用户账号
              createContractorAndLongTerm(currWorker, accountUpdateBo, areaName, tsmWorkers);
              // 向指定账号添加 承办方角色
              Set<Integer> workIds = tsmWorkers.stream().map(TsmWorInfoVO::getLoginId)
                  .collect(Collectors.toSet());
              // 查询指定角色.角色唯一
              TsmRole role = getTsmRoles(roleName);
              insertWorkerRole(workIds, role.getRoleId());
            }
          }
        }
//      }
      return Result.ofSuccess();

    } else {
      throw new BusinessException("参数对象必传!");
    }

  }


  @Override
  public Result updateOrganizerAccountState(Long matchId) {

//    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    HashMap<String, Object> props = new HashMap<>();
    props.put("matchId", matchId);
    props.put("limitType", limitTypeValue[1]);
    props.put("delFlag", 0);

    Set<Long> ids = new HashSet<>();
    // 查询比赛id关联的所有承办方临时账号
    List<MthMatchContractor> queryContent = mthMatchContractorRepo.findByProps(props);
    if (ObjectUtil.isNotEmpty(queryContent)) {
      ids = queryContent.stream().map(MthMatchContractor::getContractorId)
          .collect(Collectors.toSet());
    }
    // 查询比赛id关联的所有评审员临时账号
    List<Long> reviewerAccountList = matchReviewerManageService.getReviewerAccount(matchId);
    if (ObjectUtil.isNotEmpty(reviewerAccountList)){
      ids.addAll(reviewerAccountList);
    }
    if (ObjectUtil.isNotEmpty(ids)){
      // 修改用户状态
      tsmWorkerDao.updateStateByLoginIn(ids);
      return Result.ofSuccess("账号禁用成功!");
    }else {
      return Result.ofSuccess("比赛未关联任何临时账号!");
    }

  }

  @Override
  public void exportOrganizerAccount(HttpServletResponse response, MatchOrganizerAccountReqBo bo) {
//    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    if (ObjectUtil.isEmpty(bo) || ObjectUtil.isEmpty(bo.getMatchId())) {
      throw new BusinessException("参数必传!");
    }

    try {
      String fileName = "临时承办方账号导出" + System.currentTimeMillis() + ExcelTypeEnum.XLS.getValue();
      String sheetName = "临时承办方账号";
      // 文件名称
      fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name());
      response.setContentType("application/vnd.ms-excel;charset=utf-8");
//      response.setHeader(HttpHeaders.CONTENT_DISPOSITION,
//          "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);
      response.setHeader(HttpHeaders.CONTENT_DISPOSITION,
          "attachment;filename=" + fileName);

      if (ObjectUtil.isNotEmpty(bo.getTemplateFlag()) && bo.getTemplateFlag()
          .equals(Boolean.TRUE)) {
        ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream()).build();
        excelWriter.write(new ArrayList(),
            EasyExcel.writerSheet(sheetName).head(MthMatchContractorVO.class).build());
        excelWriter.finish();
      } else {
        // 查询比赛id关联的所有临时账号
        List<MthMatchContractorVO> queryList = matchOrganizerAccountDao
            .findLimitType(bo.getMatchId());
        // 转换
        if (ObjectUtil.isNotEmpty(queryList)) {
          for (MthMatchContractorVO mthMatchContractorVO : queryList) {
            // 防止字段matchLevel为空的情况
            AreaLevel areaByCode = AreaLevel.getAreaByCode(mthMatchContractorVO.getMatchLevel());
            if (ObjectUtil.isNotEmpty(areaByCode)) {
              mthMatchContractorVO.setMatchLevelName(areaByCode.getDesc());
            }
            // 密码解密
            mthMatchContractorVO
                .setContractorIdPwd(AESUtil.decrypt(mthMatchContractorVO.getContractorIdPwd()));
          }
        }
//        ExcelUtil.writeSingleExcel(fileName,sheetName,queryList,MthMatchContractorVO.class);
        ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream()).build();
        excelWriter.write(queryList,
            EasyExcel.writerSheet(sheetName).head(MthMatchContractorVO.class).build());
        excelWriter.finish();
      }
    } catch (Exception e) {
      throw new BusinessException("导出失败!");
    }

  }

  @Override
  public List<TsmWorInfoVO> queryWorkerInfo(OrganizationReqBo bo) {
    if (ObjectUtil.isEmpty(bo) || ObjectUtil.isEmpty(bo.getOrgId())) {
      throw new BusinessException("必传参数为空!");
    }
    List<Long> orgIds = bo.getOrgId().stream().map(e -> Long.valueOf(e))
        .collect(Collectors.toList());
    List<TsmWorInfoVO> workers = tsmWorkerDao.findByWorker(null, orgIds);
    return workers;
  }

  @Override
  public List<MthMatchContractor> queryOrganizerAccount(MatchOrganizerAccountBo bo) {
    if (ObjectUtil.isEmpty(bo)) {
      throw new BusinessException("请求体必传!");
    }
    return matchOrganizerAccountDao.queryOrganizerAccount(bo);
  }

  /**
   * 设置导出错误响应
   */
  private void errorWrite(HttpServletResponse response, Exception e) {
    log.error(e.getMessage(), e);
    response.setContentType("application/json");
    response.setCharacterEncoding("utf-8");
    response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR.value());
  }


  /**
   * 生成临时账号
   */
  public List<TsmWorker> doAddTsmWorkers(int size, String accountName) {
    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    // 查询已经存在的手机号
//    List<TsmWorker> workers = tsmWorkerRepo.findAll();
    List<String> workers = tsmWorkerRepo.findAllMobile();
    Set<String> workMobiles = new HashSet();
    if (ObjectUtil.isNotEmpty(workers)){
//      List<String> wokerMobileList = workers.stream().map(TsmWorker::getMobile)
//          .collect(Collectors.toList());
      workMobiles.addAll(workers);
    }
    List<TsmWorker> tsmWorkers = new ArrayList<>();
    for (int i = 0; i < size; i++) {
      TsmWorker tsmWorker = new TsmWorker();
      // 生成密码
      int sixNum = RandomNumberGenerator();
      tsmWorker.setLoginPwd(AESUtil.encrypt(String.valueOf(sixNum)));
      tsmWorker.setWorkerName(accountName);
      tsmWorker.setLoginNum(0);
      // 获取唯一电话号码
      String onlyMobile = getOnlyMobile(workMobiles, i);
      // 叠加
      workMobiles.add(onlyMobile);
      tsmWorker.setMobile(onlyMobile);
      tsmWorker.setState(1);
      tsmWorker.setWhetherTemporary(1);
      tsmWorker.setSource("1");
      tsmWorker.setWorkerType(2);
      tsmWorker.setCreateTime(new Date());
      // 默认设置创建人的组织
      tsmWorker.setOrgId(currWorker.getOrgId());
      tsmWorkers.add(tsmWorker);
    }
    return tsmWorkerRepo.saveAllAndFlush(tsmWorkers);
  }

  /***
   * @Description: 获取唯一电话号码
   * @Author: thw
   * @Date: 2023/12/4 11:25
   * @Params:
   * @param workMobiles  用户电话列表
   * @param num           循环变量
   * @return: java.lang.String
   **/
  public String getOnlyMobile(Collection<String> workMobiles,int num){
    String account = "";
    do {
      Snowflake snowflake = new Snowflake();
      long phoneAccount = snowflake.nextId() + num;
      String mobile = Long.valueOf(phoneAccount).toString().substring(8);
      account = mobile;
      num++;
    }while (workMobiles.contains(account));
    return account;
  }


  /**
   * 向指定账号添加 角色
   */
  public void insertWorkerRole(Collection<Integer> workerIdList, Integer roleId) {
    if (CollectionUtils.isNotEmpty(workerIdList)) {
      List<TsmWorkerRole> tsmWorkerRoles = new ArrayList<>();
      for (Integer loginId : workerIdList) {
        TsmWorkerRole workerRole = new TsmWorkerRole();
        workerRole.setLoginId(loginId);
        workerRole.setRoleId(roleId);
        workerRole.setSource(1);
        // 由于存在相同的用户id和角色id会违反唯一性约束
        HashMap<String, Object> props = new HashMap<>();
        props.put("loginId", loginId);
        props.put("roleId", roleId);
        List<TsmWorkerRole> roles = tsmWorkerRoleRepo.findByProps(props);
        if (ObjectUtil.isEmpty(roles)) {
          tsmWorkerRoles.add(workerRole);
        }

        // 默认赋值公共权限
        TsmWorkerRole workerRole2 = new TsmWorkerRole();
        workerRole2.setLoginId(loginId);
        workerRole2.setRoleId(CommonConstants.COMMON_ROLE_ID);
        workerRole2.setSource(1);
        // 由于存在相同的用户id和角色id会违反唯一性约束
        HashMap<String, Object> props2 = new HashMap<>();
        props2.put("loginId", loginId);
        props2.put("roleId", CommonConstants.COMMON_ROLE_ID);
        List<TsmWorkerRole> roles2 = tsmWorkerRoleRepo.findByProps(props2);
        if (ObjectUtil.isEmpty(roles2)) {
          tsmWorkerRoles.add(workerRole2);
        }

      }
      tsmWorkerRoleRepo.saveAllAndFlush(tsmWorkerRoles);
    }

  }


  /**
   * 查询指定角色,角色唯一
   */
  public TsmRole getTsmRoles(String roleName) {
    Map<String, Object> props = new HashMap<>();
    props.put("roleName", roleName);
    props.put("state", 1);
    List<TsmRole> roles = tsmRoleRepo.findByProps(props);
    if (CollectionUtils.isEmpty(roles)) {
      throw new BusinessException("该角色未找到!");
    }
    if (roles.size() > 1) {
      throw new BusinessException("角色重复!");
    }
    return roles.get(0);
  }


  /**
   * 临时账号创建承办方对象
   */
  private List<MthMatchContractor> createMatchContractor(Worker currWorker, List<MthRoundInstance> instance, List<TsmWorker> workerList) {
    List<MthMatchContractor> list = new ArrayList<>();
    for (int i = 0; i < instance.size(); i++) {
      // 承办方对象
      MthMatchContractor entity = new MthMatchContractor();
      MthRoundInstance roundInstance = instance.get(i);
      TsmWorker workerGe = workerList.get(i);

      entity.setMatchId(roundInstance.getMatchId());
      entity.setAreaId(roundInstance.getAreaId());
      entity.setAreaName(roundInstance.getAreaName());
      // 系统默认生成的账号无组织
      entity.setContractorId((long) workerGe.getLoginId());
      entity.setContractorName(workerGe.getWorkerName());
      entity.setLimitType(limitTypeValue[1]);
      // 默认组织为创建人组织
      entity.setOrgId(Long.valueOf(workerGe.getOrgId()));
/*      TsmOrg tsmOrg = tsmOrgRepo.findOrgById(currWorker.getOrgId());
      if (ObjectUtil.isNotEmpty(tsmOrg)){
        entity.setOrgName(tsmOrg.getOrgName());
      }*/
      entity.setOrgName(currWorker.getOrgName());
      entity.setDelFlag(0);
      entity.setCreateId((long) currWorker.getLoginId());
      entity.setCreateBy(currWorker.getWorkerName());
      entity.setCreateTime(new Date());
      list.add(entity);
    }
    List<MthMatchContractor> mthMatchContractors = mthMatchContractorRepo.saveAllAndFlush(list);
    return mthMatchContractors;
  }


  /**
   * 创建承办方信息，此时关联的是系统已经存在的用户账号
   */
  private List<MthMatchContractor> createContractorAndLongTerm(Worker currWorker,
      MatchOrganizerAccountUpdateBo accountUpdateBo, String areaName,
      List<TsmWorInfoVO> tsmWorkers) {
    if (CollectionUtils.isEmpty(tsmWorkers)) {
      throw new BusinessException("通过用户id未找到相关信息");
    }
    ArrayList<MthMatchContractor> entities = new ArrayList<>();
    for (TsmWorInfoVO tsmWorker : tsmWorkers) {
      // 添加修改的内容
      MthMatchContractor entity = new MthMatchContractor();
      entity.setMatchId(Long.valueOf(accountUpdateBo.getMatchId()));
      entity.setAreaId(Long.valueOf(accountUpdateBo.getAreaId()));
      entity.setAreaName(areaName);
      entity.setOrgId(Long.valueOf(tsmWorker.getOrgId()));
      entity.setOrgName(tsmWorker.getOrgName());
      entity.setContractorId((long) tsmWorker.getLoginId());
      entity.setContractorName(tsmWorker.getWorkerName());
      entity.setLimitType(limitTypeValue[0]);
      entity.setDelFlag(0);
      entity.setCreateId((long) currWorker.getLoginId());
      entity.setCreateBy(currWorker.getWorkerName());
      entity.setCreateTime(new Date());
      entities.add(entity);
    }
    List<MthMatchContractor> contractorList = mthMatchContractorRepo.saveAll(entities);
    return contractorList;
  }


  /**
   * 六位随机数
   */
  public static int RandomNumberGenerator() {
    Set<Integer> generatedNumbers = new HashSet<>();
    Random random = new Random();
    int randomNumber;
    do {
      randomNumber = random.nextInt(900000) + 100000; // 生成6位随机数
    } while (generatedNumbers.contains(randomNumber));
    generatedNumbers.add(randomNumber);
    return randomNumber;
  }

}
