package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.domain.entity.main.MthMatchContractor;
import com.crunii.micro.domain.entity.main.MthMatchInfo;
import com.crunii.micro.domain.entity.main.MthRoundInstance;
import com.crunii.micro.domain.entity.main.MthRoundList;
import com.crunii.micro.domain.repo.main.MthMatchContractorRepo;
import com.crunii.micro.domain.repo.main.MthMatchInfoRepo;
import com.crunii.micro.domain.repo.main.MthRoundInstanceRepo;
import com.crunii.micro.service.match.api.vo.MthContractorVo;
import com.crunii.micro.service.match.api.vo.MthRoundInstanceVo;
import com.crunii.micro.service.match.api.vo.RegisterPendingCheckVo;
import com.crunii.micro.service.match.enums.MatchInfoStatusEnums;
import com.crunii.micro.service.match.enums.RoundEnums;
import com.crunii.micro.service.match.jdbc.MatchTaskDao;
import com.crunii.micro.service.match.jdbc.MthRoundInstanceDao;
import com.crunii.micro.service.match.jdbc.MthRoundListDao;
import com.crunii.micro.service.match.service.MatchTaskService;
import com.crunii.micro.service.match.service.PushBusinessMsgService;
import com.google.common.base.Joiner;
import java.util.*;
import java.util.Map.Entry;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RSetCache;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;


/**
 * @author wangchen create 2023/7/4 14:49
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class MatchTaskServiceImpl implements MatchTaskService {

    private final MatchTaskDao matchTaskDao;

    private final MthMatchInfoRepo matchInfoRepo;
    private  final MthRoundInstanceRepo baseRepo;
    private  final MthRoundInstanceDao roundInstanceDao;
    private  final PushBusinessMsgService pushBusinessMsgService;

    private  final MthRoundListDao mthRoundListDao;

    private final MthMatchContractorRepo mthMatchContractorRepo;

    private final RedissonClient redisson;

//    public MatchTaskServiceImpl(MatchTaskDao matchTaskDao, MthRoundInstanceRepo baseRepo) {
//        this.matchTaskDao = matchTaskDao;
//        this.baseRepo = baseRepo;
//    }

    @Override
    public void updateAndScanCarousel() {
        Date now = new Date();
        // 轮播表查看已到期文章id 更新轮播状态为展示到期
        List<Long> ids =  matchTaskDao.findArticleIdsFromCarousel(now);
        if(!CollectionUtils.isEmpty(ids)){
            matchTaskDao.updateArticleCarousel(ids);
        }
        // 查看文章表轮播展示到期的id 更新轮播状态为展示到期 && 状态为已结束
        List<Long> articleIds = matchTaskDao.findArticleIds(now);
        if(!CollectionUtils.isEmpty(articleIds)){
            matchTaskDao.updateArticleCarouselAndArticleState(articleIds);
        }
    }

    @Override
    public void updateAndScanEnroll() {
        Date now = new Date();
        // 扫描报名中 报名结束的文章id
        List<Long> articleIds = matchTaskDao.scanEnroll(now);
        // 更新状态为报名结束
        if(CollectionUtils.isEmpty(articleIds)){
            return;
        }
        matchTaskDao.updateArticleType(articleIds);
    }

    @Override
    public void updateAndShowDue() {
        List<Long> showDueArticleIds = matchTaskDao.findShowDueArticleIds(new Date());
        if(CollectionUtils.isEmpty(showDueArticleIds)){
            return;
        }
        matchTaskDao.updateAndShowDue(showDueArticleIds);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void matchStatusTransfer() {
        // 查询已发布的赛事
        List<MthMatchInfo> releaseMatchList = matchInfoRepo.findByStatus(MatchInfoStatusEnums.RELEASED.getStatus());
        if (ObjectUtil.isNotEmpty(releaseMatchList)) {
            List<Long> releaseMatchIds = StreamUtils.toList(releaseMatchList, MthMatchInfo::getId);
            // 查询比赛报名场次实例
            List<MthRoundInstance> roundInstanceList = matchTaskDao.queryMatchRegisterRoundInstance(releaseMatchIds);
            Map<Long, List<MthRoundInstance>> matchRoundListMap = StreamUtils.groupByKey(roundInstanceList, MthRoundInstance::getMatchId);
            List<Long> matchIds = releaseMatchIds.stream().filter(matchId -> {
                List<MthRoundInstance> list = matchRoundListMap.get(matchId);
                if(list == null){
                    return false;
                }
                // 判断比赛场次是否存在已到报名时间 and 判断比赛场次是否开启
                return list.stream().anyMatch(round -> DateUtil.after(new Date(), round.getRegisterationStart()) && round.getRoundState() != null && Objects.equals(round.getRoundState(), 0));
            }).collect(Collectors.toList());
            if (ObjectUtil.isNotEmpty(matchIds)) {
                // 已发布赛事修改为报名中
                matchInfoRepo.updateRegisterStatus(matchIds);
            }
        }

        // 查询报名中的赛事
        List<MthMatchInfo> registerMatchList = matchInfoRepo.findByStatus(MatchInfoStatusEnums.REGISTER.getStatus());
        if (ObjectUtil.isNotEmpty(registerMatchList)) {
            List<Long> registerMatchIds = StreamUtils.toList(registerMatchList, MthMatchInfo::getId);
            // 查询比赛报名场次实例
            List<MthRoundInstance> registerRoundInstanceList = matchTaskDao.queryMatchRegisterRoundInstance(registerMatchIds);
            Map<Long, List<MthRoundInstance>> registerMatchRoundListMap = StreamUtils.groupByKey(registerRoundInstanceList, MthRoundInstance::getMatchId);
            List<Long> matchIds = registerMatchIds.stream().filter(matchId -> {
                List<MthRoundInstance> list = registerMatchRoundListMap.get(matchId);
                if(list == null){
                    return false;
                }
                // 判断比赛报名时间是否结束
                return list.stream().allMatch(round -> DateUtil.after(new Date(), round.getRegisterationEnd()));
            }).collect(Collectors.toList());
            if (ObjectUtil.isNotEmpty(matchIds)) {
                // 报名中赛事修改为进行中
                matchInfoRepo.updateGoingStatus(matchIds);
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void roundStatusTransfer() {
        //比赛报名状态
        updateMatchStateToRegister();
//        List<MthRoundInstance> mthRoundInstanceList = matchTaskDao.findMthRoundInstance(new Date());
//        if (ObjectUtil.isNotEmpty(mthRoundInstanceList)){
//            for (MthRoundInstance item :mthRoundInstanceList){
//                    //更新比赛-报名中状态
//                    item.setMatchState(0);
//                baseRepo.upadteMatchStateById(0,new Date(),item.getId());
//                log.info("upadteMatchStateById ,{} ,matchState:{}",item.getId(),0);
//
//            }
//
//        }
        //比赛报名结束状态
        List<MthRoundInstance> mthRoundInstanceEndList = matchTaskDao.findEndMthRoundInstance(new Date());

        if (ObjectUtil.isNotEmpty(mthRoundInstanceEndList)){
            for (MthRoundInstance item :mthRoundInstanceEndList){
                RegisterPendingCheckVo registerPendingCheck= matchTaskDao.checkRegisterPendingCheck(item.getId());
                if (ObjectUtil.isNotEmpty(registerPendingCheck)&&
                        registerPendingCheck.getPendingNum()==0L ){
                    //没有需要报名审核的
                    //更新比赛->报名结束->待评审状态
                    if (RoundEnums.LineEnum.ONLINE.getCode().equals(item.getAuditMode())){
                        baseRepo.upadteMatchStateById(1,new Date(),item.getId());

                        log.info("upadteMatchStateById ,{} ,matchState:{}",item.getId(),1);
                    }else {
                        item.setMatchState(2);
                        baseRepo.upadteMatchStateById(2,new Date(),item.getId());

                        log.info("upadteMatchStateById ,{} ,matchState:{}",item.getId(),2);


                    }

                }


            }
        }
    }

    /**
     * @description: 更新场次状态为报名中
     * 1、查询比赛状态为发布中的比赛id
     * 2、查询最低等级的第一个场次是否到达报名时间
     * @param
     * @return:
     * @author liuchuan
     * @date: 2023/12/5 9:52
    */
    private void updateMatchStateToRegister(){
        //发布中比赛
        List<MthMatchInfo> mthMatchInfoList = matchTaskDao.findMthByReleased();
        mthMatchInfoList.stream().forEach(mthMatchInfo -> {
            //
            String matchLevel="";
            if (ObjectUtil.isNotEmpty(mthMatchInfo.getMatchLevel())){

                if (mthMatchInfo.getMatchLevel().contains(RoundEnums.AreaLevel.CITY.getCode())){
                    matchLevel = RoundEnums.AreaLevel.CITY.getCode();
                }
                if (mthMatchInfo.getMatchLevel().contains(RoundEnums.AreaLevel.DISTRICT.getCode())){
                    matchLevel = RoundEnums.AreaLevel.DISTRICT.getCode();
                }

                if (mthMatchInfo.getMatchLevel().contains(RoundEnums.AreaLevel.COUNTY_SCHOOL.getCode())){
                    matchLevel = RoundEnums.AreaLevel.COUNTY_SCHOOL.getCode();
                }


            }else {
                //为空 二期默认是市级
                matchLevel = RoundEnums.AreaLevel.CITY.getCode();
            }
            matchTaskDao.turnRoundMathState(mthMatchInfo.getId(),matchLevel);

        });


    }

    /**
     * @description: 将公示到期的是场次状态从公示中流转为公示结束
     * @param
     * @return:
     * @author liuchuan
     * @date: 2023/11/27 14:51
    */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updatePublicityFinish() {
        // 查询出需要修改为公式结束的场次id
        List<MthRoundInstance> mthRoundInstances = roundInstanceDao.findByRoundTypeAndMatchStateAndPublicityDeadLine();
        if (ObjectUtil.isNotEmpty(mthRoundInstances)){
            List<Long> roundIds = StreamUtils.toList(mthRoundInstances, MthRoundInstance::getId);
            if (ObjectUtil.isNotEmpty(roundIds)){
                String roundIdStr = Joiner.on(",").join(roundIds);
                //将公示中的场次流转为公示结束
                roundInstanceDao.updateMatchStateByIdIn(roundIdStr);
            }

            // 根据比赛的id分组
            Map<Long, List<MthRoundInstance>> mapByMatchId = mthRoundInstances.stream()
                .collect(Collectors.groupingBy(MthRoundInstance::getMatchId));
            if (ObjectUtil.isNotEmpty(mapByMatchId)){
                String matchName = "";
                for (Entry<Long, List<MthRoundInstance>> entry : mapByMatchId.entrySet()) {
                    Optional<MthMatchInfo> mthMatchInfo = matchInfoRepo.findById(entry.getKey());
                    if (mthMatchInfo.isPresent()){
                        matchName = mthMatchInfo.get().getMatchName();
                    }else {
                        throw new BusinessException("根据比赛id:"+entry.getKey()+ "未找到比赛信息!");
                    }
                    // 根据场次中的区域id找到对应的承办方
                    List<Long> areaIds = entry.getValue().stream().filter(item -> ObjectUtil.isNotEmpty(item.getAreaId())).map(MthRoundInstance::getAreaId)
                        .collect(Collectors.toList());
                    List<MthMatchContractor> contractors = mthMatchContractorRepo.findByDelFlagAndAreaIdIn(
                        0,areaIds);
                    if (ObjectUtil.isNotEmpty(contractors)){
                        List<Long> loginIds = StreamUtils.toList(contractors,
                            MthMatchContractor::getContractorId);
                        // 公示结束通知
                        log.info("公示结束通知:{} ",matchName);
                        pushBusinessMsgService.pushPublicFinish(loginIds,matchName);
                    }
                }
            }
        }
//        matchTaskDao.updatePublicityFinish();
    }

    @Override
    public void pushWorkNotUploadedMsg() {
        List<MthRoundInstanceVo> roundInstanceList = roundInstanceDao.queryByEntryDeadline();
        if(ObjectUtil.isNotEmpty(roundInstanceList)) {
            List<Long> roundIdList = StreamUtils.toList(roundInstanceList, MthRoundInstanceVo::getId);
            List<MthRoundList> roundLists = mthRoundListDao.queryNotUploadedWorkUser(roundIdList);
            RSetCache<String> setCache = redisson.getSetCache("MATCH_PUSH_WORK_NOT_UPLOADED_MSG");
            roundLists = roundLists.stream().filter(item -> !CollectionUtil.contains(setCache, item.getRoundId() + "_" + item.getUserId())).collect(Collectors.toList());
            if (ObjectUtil.isEmpty(roundLists)) {
                return;
            }
            Map<Long, List<MthRoundList>> roundListsMap = StreamUtils.groupByKey(roundLists, MthRoundList::getRoundId);

            for (MthRoundInstanceVo roundInstance : roundInstanceList) {
                List<MthRoundList> mthRoundLists = roundListsMap.get(roundInstance.getId());
                if (ObjectUtil.isNotEmpty(mthRoundLists)) {
                    List<Long> userIdList = StreamUtils.toList(mthRoundLists, MthRoundList::getUserId);
                    pushBusinessMsgService.pushWorkNotUploadedMsg(userIdList, roundInstance.getMatchName(), roundInstance.getEntryDeadline());
                    for (MthRoundList item : mthRoundLists) {
                        setCache.add(item.getRoundId() + "_" + item.getUserId(), 2, TimeUnit.DAYS);
                    }
                }
            }
        }
    }


    @Override
    public void pushPlayMsg() {
        List<MthRoundInstanceVo> roundInstanceList = roundInstanceDao.queryByRoundStart();
        if(ObjectUtil.isNotEmpty(roundInstanceList)) {
            List<Long> roundIdList = StreamUtils.toList(roundInstanceList, MthRoundInstanceVo::getId);
            List<MthRoundList> roundLists = mthRoundListDao.queryRoundUser(roundIdList);
            RSetCache<String> setCache = redisson.getSetCache("MATCH_PUSH_PLAY_MSG");
            roundLists = roundLists.stream().filter(item -> !CollectionUtil.contains(setCache, item.getRoundId() + "_" + item.getUserId())).collect(Collectors.toList());
            if (ObjectUtil.isEmpty(roundLists)) {
                return;
            }
            Map<Long, List<MthRoundList>> roundListsMap = StreamUtils.groupByKey(roundLists, MthRoundList::getRoundId);

            for (MthRoundInstanceVo roundInstance : roundInstanceList) {
                List<MthRoundList> mthRoundLists = roundListsMap.get(roundInstance.getId());
                if (ObjectUtil.isNotEmpty(mthRoundLists)) {
                    List<Long> userIdList = StreamUtils.toList(mthRoundLists, MthRoundList::getUserId);
                    pushBusinessMsgService.pushPlayMsg(userIdList, roundInstance.getMatchName() + roundInstance.getRoundName(), roundInstance.getRoundStart(), roundInstance.getSignAddress());
                    for (MthRoundList item : mthRoundLists) {
                        setCache.add(item.getRoundId() + "_" + item.getUserId(), 2, TimeUnit.DAYS);
                    }
                }
            }
        }
    }

    @Override
    public void pushRegisterDeadlineMsg() {
        List<MthContractorVo> contractorVos = roundInstanceDao.queryByRegisterationEnd();
        if(ObjectUtil.isNotEmpty(contractorVos)) {
            RSetCache<String> setCache = redisson.getSetCache("MATCH_PUSH_REGISTER_DEADLINE_MSG");
            contractorVos = contractorVos.stream().filter(item -> !CollectionUtil.contains(setCache, item.getMatchId() + "_" + item.getContractorId())).collect(Collectors.toList());
            if (ObjectUtil.isEmpty(contractorVos)) {
                return;
            }
            Map<String, List<MthContractorVo>> matchContractor = StreamUtils.groupByKey(contractorVos, MthContractorVo::getMatchName);
            for (Entry<String, List<MthContractorVo>> entry : matchContractor.entrySet()) {
                List<Long> contractorIds = StreamUtils.toList(entry.getValue(), MthContractorVo::getContractorId);
                pushBusinessMsgService.pushRegisterDeadlineMsg(contractorIds, entry.getKey());
                for (MthContractorVo item : entry.getValue()) {
                    setCache.add(item.getMatchId() + "_" + item.getContractorId(), 2, TimeUnit.DAYS);
                }
            }
        }
    }

    @Override
    public void pushRegisterAuditMsg() {
        List<MthContractorVo> contractorVos = roundInstanceDao.queryByRegisterationEndAfter();
        if(ObjectUtil.isNotEmpty(contractorVos)) {
            RSetCache<String> setCache = redisson.getSetCache("MATCH_PUSH_REGISTER_AUDIT_MSG");
            contractorVos = contractorVos.stream().filter(item -> !CollectionUtil.contains(setCache, item.getMatchId() + "_" + item.getContractorId())).collect(Collectors.toList());
            if (ObjectUtil.isEmpty(contractorVos)) {
                return;
            }
            Map<String, List<MthContractorVo>> matchContractor = StreamUtils.groupByKey(contractorVos, MthContractorVo::getMatchName);
            for (Entry<String, List<MthContractorVo>> entry : matchContractor.entrySet()) {
                List<Long> contractorIds = StreamUtils.toList(entry.getValue(), MthContractorVo::getContractorId);
                pushBusinessMsgService.pushRegisterAuditMsg(contractorIds, entry.getKey());
                for (MthContractorVo item : entry.getValue()) {
                    setCache.add(item.getMatchId() + "_" + item.getContractorId(), 2, TimeUnit.DAYS);
                }
            }
        }
    }

    @Override
    public void pushWorkReviewMsg() {
        List<MthContractorVo> contractorVos = roundInstanceDao.queryByEntryDeadlineAfter();
        if(ObjectUtil.isNotEmpty(contractorVos)) {
            RSetCache<String> setCache = redisson.getSetCache("MATCH_PUSH_WORK_REVIEW_MSG");
            contractorVos = contractorVos.stream().filter(item -> !CollectionUtil.contains(setCache, item.getMatchId() + "_" + item.getContractorId())).collect(Collectors.toList());
            if (ObjectUtil.isEmpty(contractorVos)) {
                return;
            }
            Map<String, List<MthContractorVo>> matchContractor = StreamUtils.groupByKey(contractorVos, MthContractorVo::getMatchName);
            for (Entry<String, List<MthContractorVo>> entry : matchContractor.entrySet()) {
                List<Long> contractorIds = StreamUtils.toList(entry.getValue(), MthContractorVo::getContractorId);
                pushBusinessMsgService.pushWorkReviewMsg(contractorIds, entry.getKey());
                for (MthContractorVo item : entry.getValue()) {
                    setCache.add(item.getMatchId() + "_" + item.getContractorId(), 2, TimeUnit.DAYS);
                }
            }
        }
    }
}
