package com.crunii.micro.service.match.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.entity.main.MthArea;
import com.crunii.micro.domain.repo.main.MthAreaRepo;
import com.crunii.micro.service.match.api.bo.MthAreaBo;
import com.crunii.micro.service.match.api.vo.MthAreaVo;
import com.crunii.micro.service.match.converter.MthAreaConverter;
import com.crunii.micro.service.match.jdbc.MthAreaDao;
import com.crunii.micro.service.match.service.IMthAreaService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 赛事-场次-比赛区域等级;Service业务层处理
 *
 * @author mth
 * @date 2023-09-19
 */

@RequiredArgsConstructor
@Service
public class MthAreaServiceImpl implements IMthAreaService {

    private final MthAreaRepo baseRepo;

    private final MthAreaConverter baseConverter;

    private final MthAreaDao areaDao;

    /**
     * 查询赛事-场次-比赛区域等级;
     */
    @Override
    public MthAreaVo queryById(Long id){
        MthArea byId = baseRepo.getById(id);
        if (byId == null) {
            throw new BusinessException("无效数据");
        }
        return baseConverter.entityToVo(baseRepo.getById(id));
    }

    /**
     * 查询赛事-场次-比赛区域等级;列表
     */
    @Override
    public Result<PageRsp<MthAreaVo>> queryPageList(PageReq<MthAreaBo> bo) {
        return Result.ofSuccess(areaDao.queryPageList(bo));
    }

    /**
     * 查询赛事-场次-比赛区域等级;列表
     */
    @Override
    public List<MthAreaVo> queryList(MthAreaBo bo) {
        return areaDao.queryList(bo);
    }

}
