package com.crunii.micro.service.match.service.impl;

import static com.crunii.micro.service.match.constants.MatchMessageConstants.MATCH_NOT_EXIST;
import static com.crunii.micro.service.match.constants.MatchMessageConstants.REQUEST_ERROR;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.nacos.common.utils.StringUtils;
import com.crunii.micro.common.dto.NoModelWriteData;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.PropertyService;
import com.crunii.micro.feign.client.dto.ValueC;
import com.crunii.micro.feign.client.dto.ValueU;
import com.crunii.micro.service.match.api.FileDto;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.converter.*;
import com.crunii.micro.service.match.dto.MatchTableInfoVo;
import com.crunii.micro.service.match.dto.RoundColumVo;
import com.crunii.micro.service.match.enums.MatchEnums;
import com.crunii.micro.service.match.enums.MatchInfoStatusEnums;
import com.crunii.micro.service.match.enums.RegisterAuditStatusEnums;
import com.crunii.micro.service.match.enums.RoundEnums;
import com.crunii.micro.service.match.jdbc.*;
import com.crunii.micro.service.match.service.*;
import com.google.common.collect.Maps;
import java.io.IOException;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


/**
 * 赛事信息主;Service业务层处理
 *
 * @author mth
 * @date 2023-09-19
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class MthMatchInfoServiceImpl implements IMthMatchInfoService {
    private static final String orgCode = "QCCQ3537136584";//团重庆市委

    private final MthMatchInfoRepo baseRepo;
    private final MthMatchInfoConverter baseConverter;
    private final MthRoundInstanceConverter mthRoundConverter;

    private final MthRoundScoreConverter scoreConverter;

    private final MthMatchInfoDao matchManageDao;
    private final MthMatchItemRepo matchItemRepo;
    private final MthMatchGroupRepo matchGroupRepo;
    private final MthRoundInstanceRepo roundInstanceRepo;
    private final MthRoundListRepo roundListRepo;
    private final IMthRoundInstanceService iMthRoundInstanceService;
    private final MthRoundScoreRepo roundScoreRepo;
    private final MatchOrganizerAccountService matchOrganizerAccountService;

    private final MthMatchContractorRepo mthMatchContractorRepo;
    @Lazy
    @Resource
    private IMthRoundScoreService iMthRoundScoreService;
    private final MthMatchProcessRepo mthMatchProcessRepo;
    private final MthRoundInstanceRepo mthRoundInstanceRepo;
    private final MthRegisterConfigRepo registerConfigRepo;
    private final MthRegisterConfigDetailRepo configDetailRepo;

    private final MthRegisterReviewRepo registerReviewRepo;
    /**
     * 不使用构造函数注入, 解决循环依赖
     */
    @Resource
    private IMthRegisterService registerService;


    private final MthMatchItemDao itemDao;

    private final MthMatchGroupDao groupDao;

    private final MthMatchLabelDao labelDao;

    private final IObsFilService obsFilService;

    private final MthRoundInsInstanceDao roundInsInstanceDao;
    /**
     * 赛事管理列表页面相关业务处理专用Dao
     */
    private final MthMatchManageDao mthMatchManageDao;

    private final MthRoundScoreinfoRepo scoreinfoRepo;

    private final OrgDao orgDao;

    private final MthRegisterDao mthRegisterDao;

    private final MthMatchSponsorConverter matchSponsorConverter;
    private final MthMatchSponsorRepo mthMatchSponsorRepo;

    private final MthMatchLabelConverter labelConverter;
    private final MthMatchLabelRepo labelRepo;

    private final MthRoundAreaRepo roundAreaRepo;

    @Value("${upload.url}")
    private String urlPrefix;

    private final YcqObsFileRepo fileRepo;

    private final FileConverter fileConverter;

    private final MthMatchCollectedRepo mthMatchCollectedRepo;

    private final PushBusinessMsgService pushBusinessMsgService;

    private final MthMatchInfoRepo mthMatchInfoRepo;
    @Resource
    private PropertyService propertyService;
    @Resource
    private TsmPropertyValueRepo tsmPropertyValueRepo;

    private final MthMatchSignInformationRepo  mthMatchSignInformationRepo;

    private final MthRoundReviewStandarRepo standarRepo;
    private final MthFormConfigRepo formConfigRepo;
    private final MthFormConfigDataRepo formConfigDataRepo;
    private final TsmOrgRepo tsmOrgRepo;
    private final MthRoundPrizeRepo roundPrizeRepo;

    // 图片,文件,视频,音频
    private static final List<String> FILE_TYPES = com.google.common.collect.Lists.newArrayList("pic", "file", "video", "audio");
    /**
     * 查询赛事信息列表
     *
     * @param bo
     * @return
     */
    @Override
    public PageRsp<MthMatchInfoVo> publishList(PageReq<MatchPublishListBo> bo) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        PageRsp<MthMatchInfoVo> rsp = matchManageDao.publishList(bo);
        TsmOrg tsmOrg = tsmOrgRepo.findByOrgCode(orgCode);
        if(ObjectUtil.isNotEmpty(rsp.getResult())) {
            //主办方
            List<Long> matchIds = StreamUtils.toList(rsp.getResult(), MthMatchInfoVo::getId);
            List<MthMatchSponsor> matchSponsorList = mthMatchSponsorRepo.findByMatchIdIn(matchIds);
            Map<Long, Map<Integer, List<MthMatchSponsor>>> matchTypeSponsorListMap = StreamUtils.groupBy2Key(matchSponsorList, MthMatchSponsor::getMatchId, MthMatchSponsor::getType);

            for (MthMatchInfoVo vo : rsp.getResult()) {
                if (StrUtil.isNotBlank(vo.getMatchLevel())) {
                    String[] arr = vo.getMatchLevel().split(",");
                    String matchLevel = Arrays.stream(arr)
                            .map(item -> {
                                RoundEnums.AreaLevel areaByCode = RoundEnums.AreaLevel.getAreaByCode(item);
                                return areaByCode == null ? "" : areaByCode.getDesc();
                            }).collect(Collectors.joining(","));
                    vo.setMatchLevel(matchLevel);
                }
                vo.setEditFlag(currWorker != null && Objects.equals(currWorker.getOrgId(), vo.getOrgId()) ? 1 : 0);
                Map<Integer, List<MthMatchSponsor>> typeSponsorListMap = matchTypeSponsorListMap.get(vo.getId());
                if(ObjectUtil.isNotEmpty(typeSponsorListMap)) {
                    List<MthMatchSponsor> sponsors = typeSponsorListMap.get(1);
                    if(ObjectUtil.isNotEmpty(sponsors)) {
                        String sponsorsName = sponsors.stream().map(MthMatchSponsor::getSponsorName).collect(Collectors.joining(StrUtil.COMMA));
                        vo.setSponsor(sponsorsName);
                    }
                    List<MthMatchSponsor> organizers = typeSponsorListMap.get(2);
                    if(ObjectUtil.isNotEmpty(organizers)) {
                        String organizersName = organizers.stream().map(MthMatchSponsor::getSponsorName).collect(Collectors.joining(StrUtil.COMMA));
                        vo.setOrganizer(organizersName);
                    }
                }
                //主办方是否包含团市委
                List<Long> sponsorIds = matchSponsorList.stream().filter(s -> s.getType() == 1&&s.getMatchId().equals(vo.getId())).map(MthMatchSponsor::getSponsorId).collect(Collectors.toList());
                if (ObjectUtil.isNotEmpty(tsmOrg)){
                    vo.setWhetherSponsor(sponsorIds.contains(tsmOrg.getOrgId())?1:0);
                }
            }
        }
        return rsp;
    }

    @Override
    public PageRsp<mthMatchManagerVo> mthMatchManagerList(PageReq<MthMatchManagerReqBo> bo) {
        return mthMatchManageDao.queryStatisticPageList(bo);
    }

    @Override
    public List<mthMatchManagerDetailVo> queryMatchManagerDetailList(MthMatchManagerReqBo req) {
        return mthMatchManageDao.queryMatchManagerDetailList(req);
    }

    @Override
    public List<mthMatchManagerDetailVo> mthMatchManagerDetailList(MthMatchManagerReqBo bo) {
        return mthMatchManageDao.queryList(bo);
    }

    @Override
    public Boolean publishMatch(List<Long> ids) {
        List<MthMatchInfo> list = matchManageDao.getMatchInfoByIds(ids);
        List<MatchStatusInfoBo> matchStatusInfoBos = baseConverter.entityToStatusBo(list);
        if (CollectionUtil.isEmpty(list)) {
            throw new BusinessException("未查询到赛事");
        }
        List<String> statusList = list.stream().map(MthMatchInfo::getStatus).collect(Collectors.toList());
        for (String status : statusList) {
            if (!MatchInfoStatusEnums.publishableList().contains(status)) {
                throw new BusinessException("请操作可发布赛事");
            }
        }
        for (Long matchId : ids) {
            checkDistrictMatchLevel(matchId);
        }

        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        OrgInfoBo orgInfoByCode = null;
        try {
            assert currWorker != null;
            orgInfoByCode = orgDao.getOrgInfoByCode(currWorker.getOrgId());
        } catch (Exception e) {

        }
        Integer i = matchManageDao.publishMatch(ids, MatchInfoStatusEnums.RELEASED.getStatus(), currWorker, orgInfoByCode);
        boolean b = i > 0;
        if (b) {
            //发布后 根据模板生成实例
            iMthRoundInstanceService.createRoundInstance(matchStatusInfoBos);
        }
        return true;
    }

    private void checkDistrictMatchLevel(Long matchId) {
        MthMatchInfo matchInfo = baseRepo.getById(matchId);
        if (matchInfo.getMatchLevel().contains(RoundEnums.AreaLevel.DISTRICT.getCode())) {
            // 校验片区级比赛存在未分配完或已删除的区县/校赛级区域
            List<MthRoundAreaVo> districtAreaList = iMthRoundInstanceService.getFirstRoundAreaByMatchIdAndMatchLevel(matchId, RoundEnums.AreaLevel.DISTRICT.getCode());
            List<Long> idList = StreamUtils.toList(districtAreaList, MthRoundAreaVo::getId);
            List<MthRoundArea> childrenList = roundAreaRepo.findByParentIdIn(idList);
            List<MthRoundAreaVo> countySchoolAreaList = iMthRoundInstanceService.getFirstRoundAreaByMatchIdAndMatchLevel(matchId, RoundEnums.AreaLevel.COUNTY_SCHOOL.getCode());
            if (childrenList.size() != countySchoolAreaList.size()) {
                throw new BusinessException("片区级比赛存在未分配完或已删除的区县/校赛级区域!");
            }
        }
    }

    /**
     * @param ids 模板id
     * @description: 根据模板生成实例
     * @return:
     * @author liuchuan
     * @date: 2023/9/25 9:29
     */


    @Override
    public Boolean soldOutMatch(List<Long> ids) {
        List<MthMatchInfo> list = matchManageDao.getMatchInfoByIds(ids);
        if (CollectionUtil.isEmpty(list)) {
            throw new BusinessException("未查询到赛事");
        }
        List<String> statusList = list.stream().map(MthMatchInfo::getStatus).collect(Collectors.toList());
        for (String status : statusList) {
            if (!MatchInfoStatusEnums.soldOutAbleList().contains(status)) {
                throw new BusinessException("请操作可下架赛事");
            }
        }
        return matchManageDao.updateStatus(ids, MatchInfoStatusEnums.SOLD_OUT.getStatus()) > 0;
    }

    @Override
    public List<MatchBaseItemVo> queryItemList(Long matchId) {
        List<MatchBaseItemVo> itemVos = itemDao.getItemList(matchId);
        if (CollectionUtil.isNotEmpty(itemVos)) {
            List<MatchBaseGroupVo> groups = groupDao.getGroups(matchId);
            if (CollectionUtil.isNotEmpty(groups)) {
                Map<Long, List<MatchBaseGroupVo>> map = groups.stream().collect(Collectors.groupingBy(MatchBaseGroupVo::getItemId));
                for (MatchBaseItemVo itemVo : itemVos) {
                    if (map.containsKey(itemVo.getId())) {
                        itemVo.setGroupList(map.get(itemVo.getId()));
                    }
                }
            }
        }
        return itemVos;
    }

    @Override
    public MatchBaseInfoVo matchBaseInfo(Long id) {
        MthMatchInfo matchInfo = baseRepo.getById(id);
        MatchBaseInfoVo matchBaseInfoVo = BeanUtil.copyProperties(matchInfo, MatchBaseInfoVo.class);
        matchBaseInfoVo.setRegistrationStartTime(matchInfo.getRegisterationStart());
        matchBaseInfoVo.setRegistrationEndTime(matchInfo.getRegisterationEnd());
        List<MatchBaseItemVo> itemVos = queryItemList(id);
        matchBaseInfoVo.setItemLists(itemVos);
        List<ObsFileInfoBo> obsFileByIds = obsFilService.getObsFileByIds(Arrays.asList(matchBaseInfoVo.getBannerId()));

        List<String> labels = labelDao.getLabelName(id);
        matchBaseInfoVo.setLabelName(labels);
//        RegistrationTimeBo registrationTimeBo = roundInsInstanceDao.getRegistrationTime(id);
//        if (ObjectUtil.isNotNull(registrationTimeBo)) {
//            if (ObjectUtil.isNotNull(registrationTimeBo.getRegistrationStartTime())) {
//                matchBaseInfoVo.setRegistrationStartTime(registrationTimeBo.getRegistrationStartTime());
//            }
//            if (ObjectUtil.isNotNull(registrationTimeBo.getRegistrationEndTime())) {
//                matchBaseInfoVo.setRegistrationEndTime(registrationTimeBo.getRegistrationEndTime());
//            }
//
//        }
        //图片
        if (ObjectUtil.isNotNull(matchBaseInfoVo.getBannerId())) {
            YcqObsFile file = fileRepo.findByIdAndState(matchBaseInfoVo.getBannerId(), 1);
            if (ObjectUtil.isNotNull(file)) {
                FileDto fileDto = fileConverter.ObsFileToFileDto(file);
                fileDto.setFileUrl(urlPrefix + fileDto.getFileKey());
                matchBaseInfoVo.setFileDto(fileDto);
            }
        }
        //主办方
        List<MthMatchSponsor> matchSponsorList = mthMatchSponsorRepo.findByMatchId(matchBaseInfoVo.getId());
        if (CollectionUtil.isNotEmpty(matchSponsorList)) {
            Map<Integer, List<MthMatchSponsor>> typeMap = StreamUtils.groupByKey(matchSponsorList, MthMatchSponsor::getType);
            //主办方
            matchBaseInfoVo.setSponsorBoList(matchSponsorConverter.entityListToVoList(typeMap.get(1)));
            //承办方
            matchBaseInfoVo.setOrganizerBoList(matchSponsorConverter.entityListToVoList(typeMap.get(2)));
        }
        return matchBaseInfoVo;
    }

    @Override
    public List<MatchRoundTemplateInfoVo> matchProcess(Long matchId) {
        List<MatchRoundTemplateInfoVo> list = roundInsInstanceDao.getRoundTemplateInfos(matchId);
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>();
        }

        MthMatchInfo matchInfo = baseRepo.getById(matchId);
        Map<String, List<MthRoundAreaVo>> matchLevelMap = Maps.newHashMap();
        for (RoundEnums.AreaLevel value : RoundEnums.AreaLevel.values()) {
            if (matchInfo.getMatchLevel().contains(value.getCode())) {
                List<MthRoundAreaVo> areaVoList = iMthRoundInstanceService.getFirstRoundAreaByMatchIdAndMatchLevel(matchId, value.getCode());
                matchLevelMap.put(value.getCode(), areaVoList);
            }
        }
        for (MatchRoundTemplateInfoVo roundVo : list) {
            if (matchLevelMap.containsKey(roundVo.getLevel())) {
                roundVo.setOrgInfos(matchLevelMap.get(roundVo.getLevel()));
            }
        }
        return list;
    }


    @Override
    public Result<PageRsp<MthMyMatchListAppVo>> myMatchListForApp(PageReq<MthMyMatchListAppBo> bo) {
        if (ObjectUtil.isNull(bo.getQuery())) {
            bo.setQuery(new MthMyMatchListAppBo());
        }
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        bo.getQuery().setUserId(Integer.valueOf(currWorker.getLoginId()).longValue());
        bo.getQuery().setStatusList(MatchInfoStatusEnums.queryAppList());
        PageRsp<MthMyMatchListAppVo> result = matchManageDao.myMatchListForApp(bo);
        // 设置文件url地址
       // obsFilService.setObsFileBoByIds(result.getResult(), MthMyMatchListAppVo::getBannerId, MthMyMatchListAppVo::setBanner);
        if (ObjectUtil.isNotEmpty(result.getResult())) {
            for (MthMyMatchListAppVo item : result.getResult()) {
                // 获取赛事状态
                item.setStatusDesc(MatchInfoStatusEnums.getAppStatusDesc(item.getStatus()));
                // 审核结果描述
                item.setReviewResultDesc(RegisterAuditStatusEnums.getDesc(item.getReviewResult()));
            }
        }
        return Result.ofSuccess(result);
    }

    @Override
    public List<MatchRegisterFormVo> matchApplicationForm(Long matchId) {
        List<MatchRegisterFormVo> list = roundInsInstanceDao.getregisterInfos(matchId);
        return list;
    }

    @Override
    public List<MatchStatusVo> matchStatus() {
        MatchInfoStatusEnums[] values = MatchInfoStatusEnums.values();
        List<MatchStatusVo> list = new ArrayList<>();
        for (MatchInfoStatusEnums value : values) {
            MatchStatusVo statusVo = new MatchStatusVo();
            statusVo.setStatus(value.getStatus());
            statusVo.setStatusDesc(value.getDesc());
            list.add(statusVo);
        }
        return list;
    }


    @Override
    public boolean insertAppMatchConfig(MthMatchConfigBo bo) {
        MthMatchInfo matchInfo = baseRepo.getOne(bo.getMatchId());
        matchInfo.setConfigUrl(bo.getConfigUrl());
        matchInfo.setRegisterUrl(bo.getRegisterUrl());
        baseRepo.save(matchInfo);
        return true;
    }

    @Override
    public MthMatchConfigVo getAppMatchConfig(Long matchId) {
        MthMatchInfo matchInfo = baseRepo.getOne(matchId);
        MthMatchConfigVo vo = new MthMatchConfigVo();
        if (ObjectUtil.isNotNull(matchInfo)) {
            vo.setConfigUrl(matchInfo.getConfigUrl());
            vo.setMatchId(matchId);
            vo.setRegisterUrl(matchInfo.getRegisterUrl());
        }
        return vo;
    }


    @Override
    public boolean matchSubmit(MthMatchSubmitBo bo) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        MthMatchInfo matchInfo = baseRepo.getOne(bo.getMatchId());
        //设置发布组织
        if (worker!=null && StringUtils.isNotEmpty(worker.getOrgId())){
            try {
                matchInfo.setPublishOrgId(Long.valueOf(worker.getOrgId()));
                matchInfo.setPublishOrgName(worker.getOrgName());
            }catch (Exception e){
                log.info("设置发布组织");
            }
        }
        // 验证比赛必填字段
        if (ObjectUtil.isNull(matchInfo.getCategoryName())) {
            throw new BusinessException("比赛分类为空");
        }
        if (ObjectUtil.isNull(matchInfo.getBannerId())) {
            throw new BusinessException("比赛封面图片为空");
        }
        if (ObjectUtil.isNull(matchInfo.getRegisterationStart()) || ObjectUtil.isNull(matchInfo.getRegisterationEnd())) {
            throw new BusinessException("比赛报名时间为空");
        }
        if (ObjectUtil.isNull(matchInfo.getRegisterationAuditFlag())) {
            throw new BusinessException("比赛报名审核为空");
        }
        if (StrUtil.isBlank(matchInfo.getConfigUrl())) {
            throw new BusinessException("app端链接地址为空");

        }
        // 验证场次必填字段
        List<MthRoundInstance> roundInstanceList = roundInstanceRepo.findByMatchIdAndRoundType(bo.getMatchId(),
                RoundEnums.RoundType.TEMPLATE.getCode());
        if (CollectionUtil.isEmpty(roundInstanceList)) {
            throw new BusinessException("请配置场次信息");
        }
        roundInstanceList.stream().forEach(roundInstance -> {
            if (ObjectUtil.isNull(roundInstance.getMatchLevel())){
                throw new BusinessException("请选择场次等级");
            }
            if (ObjectUtil.isNull(roundInstance.getRoundName())){
                throw new BusinessException("场次名称为空");
            }

            if (ObjectUtil.isNotNull(roundInstance.getEntrySubmitMode())){
                if(RoundEnums.EntrySubmitMode.ALONE.getCode().equals(roundInstance.getEntrySubmitMode())
                || RoundEnums.EntrySubmitMode.FOLLOWREGISTER.getCode().equals(roundInstance.getEntrySubmitMode())){
                    //需要有作品表
                    List<MthRegisterConfig> list = registerConfigRepo.findByMatchIdAndConfigType(matchInfo.getId(),
                            MatchEnums.ConfigType.WORKS.getCode());
                    if (CollectionUtil.isEmpty(list)){
                        throw new BusinessException("请填写作品表");

                    }
                }
            }

//            if (ObjectUtil.isNull(roundInstance.getRoundStart())||ObjectUtil.isNull(roundInstance.getRoundEnd())){
//                throw new BusinessException("场次时间为空");
//            }
            // 查询比赛等级第一场场次区域
            List<MthRoundAreaVo> roundAreaList = iMthRoundInstanceService.getFirstRoundAreaByMatchIdAndMatchLevel(matchInfo.getId(), roundInstance.getMatchLevel());
            if (CollectionUtil.isEmpty(roundAreaList)){
                throw new BusinessException("请选择场次区域");
            }
        });

        checkDistrictMatchLevel(matchInfo.getId());
        matchInfo.setStatus(MatchInfoStatusEnums.TO_APPROVE.getStatus());
        baseRepo.saveAndFlush(matchInfo);

        return true;
    }

    @Override
    public MthMyMatchListDetailAppVo queryMatchRegisterDetail(MthMatchManagerRegisterReqBo bo) {
        return registerService.myMatchListDetail(bo.getMthMatchId(), bo.getRegisterId());
    }

    @Override
    public MthMyMatchListDetailAppVo queryMatchReviewDetail(MthMatchManagerReviewReqBo bo) {
        return registerService.myMatchListDetail(bo.getMthMatchId(), bo.getRegisterId());
    }

    @Override
    public Boolean matchRegisterExamine(MthMatchManagerRegisterReqBo reqBo) {
        MthRegisterBo bo = new MthRegisterBo();
        bo.setId(reqBo.getRegisterId());
        bo.setReviewResult(reqBo.getReviewResult());
        Boolean result = registerService.updateExamineBo(bo);
        MthRegisterReview registerReview = new MthRegisterReview();
        registerReview.setReason(reqBo.getReviewReason());
        registerReview.setReviewResult(reqBo.getReviewResult());
        registerReview.setRegisterId(reqBo.getRegisterId());
        registerReview.setReviewerId(Long.valueOf(CurrWorkerUtil.getCurrWorker().getLoginId() + ""));
        registerReview.setReviewerName(CurrWorkerUtil.getCurrWorker().getWorkerName());
        registerReview.setReviewerTime(new Date());
        registerReviewRepo.saveAndFlush(registerReview);

        // 推送消息
        MthRegisterVo registerVo = registerService.queryById(reqBo.getRegisterId());
        pushBusinessMsgService.pushRegisterAuditMsg(registerVo.getUserId(), reqBo.getReviewResult(), registerVo.getMatchName());

        if (result) {
            //查询比赛id
            MthRegisterVo vo = registerService.queryById(reqBo.getRegisterId());
            List<MthMatchProcess> mthMatchProcesses = new LinkedList<>();
            MthMatchProcess matchProcessResult = new MthMatchProcess();
            matchProcessResult.setType("AUDIT");
            matchProcessResult.setBizId(registerReview.getId());
            matchProcessResult.setUserId(vo.getId());
            matchProcessResult.setShowTitle("报名审核结果：" + RegisterAuditStatusEnums.getDesc(reqBo.getReviewResult()));
            matchProcessResult.setMatchId(vo.getMatchId());
            mthMatchProcesses.add(matchProcessResult);

            MthMatchProcess matchProcessTime = new MthMatchProcess();
            matchProcessTime.setType("AUDIT");
            matchProcessTime.setBizId(registerReview.getId());
            matchProcessTime.setUserId(vo.getId());
            matchProcessTime.setShowTitle("报名审核时间：" + DateUtil.format(new Date()));
            matchProcessTime.setMatchId(vo.getMatchId());
            mthMatchProcesses.add(matchProcessTime);
            mthMatchProcessRepo.saveAllAndFlush(mthMatchProcesses);
        }
        return result;
    }


    @Override
    public PageRsp<MthMatchManagerRegisterVo> queryMatchRegisterList(PageReq<MthMatchManagerRegisterReqBo> req) {
        PageRsp<MthMatchManagerRegisterVo> pageRsp = mthMatchManageDao.queryMatchRegisterList(req, false);
        if(CollectionUtil.isEmpty(pageRsp.getResult())){
            return pageRsp;
        }
        boolean reviewFlag = reviewFlag(req.getQuery().getRegisterFlag());
        Map<Long, List<MthMyMatchListDetailAppVo.Detail>> collect=new HashMap<>();
        //报名中查询报名表配置
        if (reviewFlag) {
            List<Long> registerId = pageRsp.getResult().stream().map(item -> item.getRegisterId()).collect(Collectors.toList());
            collect= getRegisterDetail( registerId);
        }
        for (MthMatchManagerRegisterVo item : pageRsp.getResult()) {
            if (RegisterAuditStatusEnums.NOREVIEW.getCode().equals(item.getReviewResult())){
                //当类型为无审核时  直接返回空字符串
                item.setReviewResult("");
            }
            if (reviewFlag) {
                item.setDetails(collect.get(item.getRegisterId()));
            }
            if (ObjectUtil.isNotEmpty(item.getReviewResult())){
                item.setReviewResult(RegisterAuditStatusEnums.getDesc(item.getReviewResult()));
            }

            item.setTelephone(DesensitizedUtil.mobilePhone(item.getTelephone()));
        }
        return pageRsp;
    }

    @Override
    public void exportRegisterList(PageReq<MthMatchManagerRegisterReqBo> req) {
        List<MthMatchManagerRegisterVo> list = mthMatchManageDao.queryMatchRegisterList(req, true).getResult();

        List<Map<String, Object>> data=new ArrayList<>();
        boolean reviewFlag = reviewFlag(req.getQuery().getRegisterFlag());
        for (MthMatchManagerRegisterVo item : list) {

            if (reviewFlag) {
                Map<String, Object> roundIndfo = new HashMap<>();
                roundIndfo.put("matchName", item.getMatchName());
                roundIndfo.put("areaName", item.getAreaName());
                roundIndfo.put("itemName", item.getItemName());
                roundIndfo.put("groupName", item.getGroupName());
                roundIndfo.put("userName", item.getUserName());
                roundIndfo.put("telephone", item.getTelephone());
                roundIndfo.put("roundName", item.getRoundName());
                roundIndfo.put("registerTime", DateUtil.format(item.getRegisterTime()));
                roundIndfo.put("reviewResult", RegisterAuditStatusEnums.getDesc(item.getReviewResult()));
                roundIndfo.put("reviewTime", DateUtil.format(item.getRegisterTime()));
                roundIndfo.putAll(getDetail(item.getRegisterId()));
                data.add(roundIndfo);
            }else {
                item.setReviewResult(RegisterAuditStatusEnums.getDesc(item.getReviewResult()));
                item.setTelephone(DesensitizedUtil.mobilePhone(item.getTelephone()));
            }
        }
        //赛程管理报名中导出
        if (reviewFlag) {
            MthMatchManagerRegisterReqBo mthMatchManagerRegisterReqBo = new MthMatchManagerRegisterReqBo();
            mthMatchManagerRegisterReqBo.setMthMatchId(req.getQuery().getMthMatchId());
            mthMatchManagerRegisterReqBo.setMthRoundId(req.getQuery().getMthRoundId());
            RoundColumVo roundColumVo = registerInfoColum(mthMatchManagerRegisterReqBo);
            try {
                setNoModelWriteData(roundColumVo, "审核列表", data);
            } catch (Exception e) {
                e.printStackTrace();
                log.error("导出审核列表错误:", e);

            }
        }else {
              try {
            ExcelUtil.writeSingleCommonExcel("审核列表", null, list, MthMatchManagerRegisterVo.class);
        } catch (IOException e) {
            e.printStackTrace();
            log.error("导出审核列表错误:", e);
        }
        }

    }


    Map<Long, List<MthMyMatchListDetailAppVo.Detail>>   getRegisterDetail(List<Long> registerId){
        List<MthMyMatchListDetailAppVo.Detail> maps = mthMatchManageDao.queryRegisterDetailForApp(registerId);
        setDetailFileValue(maps);
        Map<Long, List<MthMyMatchListDetailAppVo.Detail>>  collect = maps.stream().collect(Collectors.groupingBy(MthMyMatchListDetailAppVo.Detail::getBizId));
        return collect;
    }


    @Override
    public PageRsp<MthMatchManagerReviewVo> queryMatchReviewList(PageReq<MthMatchManagerReviewReqBo> req) {
        PageRsp<MthMatchManagerReviewVo> pageRsp = mthMatchManageDao.queryMatchReviewList(req, false);
       if(CollectionUtil.isEmpty(pageRsp.getResult())){
           return pageRsp;
        }
        boolean reviewFlag = reviewFlag(req.getQuery().getRegisterFlag());
        Map<Long, List<MthMyMatchListDetailAppVo.Detail>> collect=new HashMap<>();
        //报名中查询报名表配置
        if (reviewFlag) {
            List<Long> registerId = pageRsp.getResult().stream().map(item -> item.getRegisterId()).collect(Collectors.toList());
            collect= getRegisterDetail(registerId);
        }
        for (MthMatchManagerReviewVo item : pageRsp.getResult()) {
            item.setTelephone(DesensitizedUtil.mobilePhone(item.getTelephone()));
            item.setReviewResult(RegisterAuditStatusEnums.getDesc(item.getReviewResult()));
            if (reviewFlag) {
                item.setDetails(collect.get(item.getRegisterId()));
            }
        }
        return pageRsp;
    }

    @Override
    public void exportReviewList(PageReq<MthMatchManagerReviewReqBo> req) {
        List<MthMatchManagerReviewVo> list = mthMatchManageDao.queryMatchReviewList(req, true).getResult();
        List<Map<String, Object>> data=new ArrayList<>();
        boolean reviewFlag = reviewFlag(req.getQuery().getRegisterFlag());

        for (MthMatchManagerReviewVo item : list) {

            if (reviewFlag) {
                Map<String, Object> roundIndfo = new HashMap<>();
                roundIndfo.put("matchName", item.getMatchName());
                roundIndfo.put("areaName", item.getAreaName());
                roundIndfo.put("itemName", item.getItemName());
                roundIndfo.put("groupName", item.getGroupName());
                roundIndfo.put("roundName", item.getRoundName());
                roundIndfo.put("userName", item.getUserName());
                roundIndfo.put("telephone", item.getTelephone());
                roundIndfo.put("registerTime", DateUtil.format(item.getRegisterTime()));
                roundIndfo.put("reviewResult", RegisterAuditStatusEnums.getDesc(item.getReviewResult()));
                roundIndfo.put("reviewTime", DateUtil.format(item.getRegisterTime()));
                roundIndfo.putAll(getDetail(item.getRegisterId()));
                data.add(roundIndfo);
            }else{
                item.setReviewResult(RegisterAuditStatusEnums.getDesc(item.getReviewResult()));
                item.setTelephone(DesensitizedUtil.mobilePhone(item.getTelephone()));
            }
        }
        if (reviewFlag) {
            MthMatchManagerRegisterReqBo mthMatchManagerRegisterReqBo = new MthMatchManagerRegisterReqBo();
            mthMatchManagerRegisterReqBo.setMthMatchId(req.getQuery().getMthMatchId());
            mthMatchManagerRegisterReqBo.setMthRoundId(req.getQuery().getMthRoundId());
            RoundColumVo roundColumVo = registerInfoColum(mthMatchManagerRegisterReqBo);
            try {
                setNoModelWriteData(roundColumVo, "参赛者列表", data);
            } catch (Exception e) {
                e.printStackTrace();
                log.error("导出参赛者列表错误:", e);

            }
        }else {
                 try {
            ExcelUtil.writeSingleCommonExcel("参赛者列表", null, list, MthMatchManagerReviewVo.class);
        } catch (IOException e) {
            e.printStackTrace();
            log.error("导出参赛者列表错误:", e);
        }
        }


    }

    @Override
    public PageRsp<MthMatchManagerPromotedVo> queryMatchPromotedList(PageReq<MthMatchManagerPromotedReqBo> req) {
        PageRsp<MthMatchManagerPromotedVo> pageRsp = mthMatchManageDao.queryMatchPromotedList(req, false);
        for (MthMatchManagerPromotedVo item : pageRsp.getResult()) {
            item.setTelephone(DesensitizedUtil.mobilePhone(item.getTelephone()));
        }
        return pageRsp;
    }

    @Override
    public void exportPromotedList(PageReq<MthMatchManagerPromotedReqBo> req) {
        List<MthMatchManagerPromotedVo> list = mthMatchManageDao.queryMatchPromotedList(req, true).getResult();
        for (MthMatchManagerPromotedVo item : list) {
            item.setTelephone(DesensitizedUtil.mobilePhone(item.getTelephone()));
        }
        try {
            ExcelUtil.writeSingleCommonExcel("参赛者列表", null, list, MthMatchManagerPromotedVo.class);
        } catch (IOException e) {
            e.printStackTrace();
            log.error("导出参赛者列表错误:", e);
        }
    }

    @Override
    public PageRsp<MthMatchManagerPublicityVo> queryMatchPublicityList(PageReq<MthMatchManagerPublicityReqBo> req) {
        PageRsp<MthMatchManagerPublicityVo> pageRsp = mthMatchManageDao.queryMatchPublicityList(req, false);

       MthMatchSignInformation signIn = mthMatchSignInformationRepo.findByInstanceIdAndHandle(req.getQuery().getMthRoundId(),1);
        MthMatchSignInformation signOut = mthMatchSignInformationRepo.findByInstanceIdAndHandle(req.getQuery().getMthRoundId(),-1);
        MthMatchInfo matchInfo = baseRepo.getById(req.getQuery().getMthMatchId());
        for (MthMatchManagerPublicityVo item : pageRsp.getResult()) {
            item.setReviewResult(RegisterAuditStatusEnums.getDesc(item.getReviewResult()));
            item.setTelephone(DesensitizedUtil.mobilePhone(item.getTelephone()));
            if (matchInfo.getRegisterationAuditFlag()!=null){
                if (matchInfo.getRegisterationAuditFlag()==1){
                    item.setReviewFlag(1);
                }else {
                    item.setReviewFlag(0);
                }
            }
            //是否展示签到
            if (signIn==null){
                item.setSignFlag(0);
            } else {
                item.setSignFlag(1);
            }
            //是否展示签退
            if (signOut==null){
                item.setSignOutFlag(0);
            } else {
                item.setSignOutFlag(1);
            }
        }
        return pageRsp;
    }

    @Override
    public void exportPublicityList(PageReq<MthMatchManagerPublicityReqBo> req) {
        List<MthMatchManagerPublicityVo> list = mthMatchManageDao.queryMatchPublicityList(req, true).getResult();
        MthMatchSignInformation signIn = mthMatchSignInformationRepo.findByInstanceIdAndHandle(req.getQuery().getMthRoundId(),1);
        MthMatchSignInformation signOut = mthMatchSignInformationRepo.findByInstanceIdAndHandle(req.getQuery().getMthRoundId(),-1);
        MthMatchInfo matchInfo = baseRepo.getById(req.getQuery().getMthMatchId());
        Set<String> ignoreParam=new HashSet<>();
        for (MthMatchManagerPublicityVo item : list) {
            item.setReviewResult(RegisterAuditStatusEnums.getDesc(item.getReviewResult()));
            item.setTelephone(DesensitizedUtil.mobilePhone(item.getTelephone()));

        }
        //忽略导出的字段
        if (matchInfo.getRegisterationAuditFlag()==null || matchInfo.getRegisterationAuditFlag()!=1){
            ignoreParam.add("reviewResult");
            ignoreParam.add("reviewTime");
        }

        if (signIn==null){
            ignoreParam.add("signStatus");
            ignoreParam.add("signTime");
        }
        if (signOut==null){
            ignoreParam.add("signOutStatus");
            ignoreParam.add("signOutTime");
        }
        try {
            ExcelUtil.export("参赛者列表","参赛者列表",list, MthMatchManagerPublicityVo.class,ignoreParam);
         //   ExcelUtil.writeSingleCommonExcel("参赛者列表", null, list, MthMatchManagerPublicityVo.class);
        } catch (IOException e) {
            e.printStackTrace();
            log.error("导出参赛者列表错误:", e);
        }
    }

    @Override
    public PageRsp<MthMatchManagerContestantVo> queryMatchContestantList(PageReq<MthMatchManagerContestantReqBo> req) {
        PageRsp<MthMatchManagerContestantVo> pageRsp = mthMatchManageDao.queryMatchContestantList(req, false);
        if (CollectionUtil.isNotEmpty(pageRsp.getResult())) {
            List<Long> roundListId = StreamUtils.toList(pageRsp.getResult(), MthMatchManagerContestantVo::getRoundListId);
            List<MthMatchManagerContestantVo.ScoreDetail> roundScoreList = mthMatchManageDao.findScoreDetailByRoundIds(roundListId);
            Map<Long, List<MthMatchManagerContestantVo.ScoreDetail>> roundListMap = StreamUtils.groupByKey(roundScoreList, MthMatchManagerContestantVo.ScoreDetail::getRoundListId);
            List<Long> collect1 = pageRsp.getResult().stream().map(item -> item.getRegisterId()).collect(Collectors.toList());
            List<MthMyMatchListDetailAppVo.Detail> maps = mthMatchManageDao.queryRegisterDetailForApp(collect1);
            setDetailFileValue(maps);
            Map<Long, List<MthMyMatchListDetailAppVo.Detail>> collect = maps.stream().collect(Collectors.groupingBy(MthMyMatchListDetailAppVo.Detail::getBizId));

           for (MthMatchManagerContestantVo item : pageRsp.getResult()) {
                item.setScoreDetailList(roundListMap.get(item.getRoundListId()));
                item.setTelephone(DesensitizedUtil.mobilePhone(item.getTelephone()));
                //报名配置
                item.setDetails(collect.get(item.getRegisterId()));
            }

        }
        return pageRsp;
    }

    private void setNoModelWriteData(RoundColumVo roundColumVo,String fileName, List<Map<String, Object>> data){
        List<MatchTableInfoVo> table = roundColumVo.getTable();
        List<MthMyMatchListDetailAppVo.Detail> details = roundColumVo.getMap();
        int size = table.size();
        int size1 = details.size();
        int len=  size1+size;
        //表头字段
        String[] dataStrMap=new String[len];
        //表头字段名
        String[] headMap=new String[len];

      NoModelWriteData noModelWriteData = new NoModelWriteData();
        for (int i = 0; i < size; i++) {
            dataStrMap[i]=table.get(i).getTableK();
            headMap[i]=table.get(i).getTableName();
        }
        for (int i = 0; i < size1; i++) {
            if (StringUtils.isNotBlank(details.get(i).getSort())&&StringUtils.isNotBlank(details.get(i).getShowName())) {
                headMap[size+i] = details.get(i).getShowName();
                dataStrMap[size+i] = details.get(i).getShowName();
            }
        }
        noModelWriteData.setFileName(fileName);
        noModelWriteData.setHeadMap(headMap);
        noModelWriteData.setDataStrMap(dataStrMap);
        noModelWriteData.setDataList(data);
        ExcelUtil.noModleWrite(noModelWriteData);


    }

   private Map<String,Object> getDetail(Long id){
        List<Long> list=new ArrayList<>();
        list.add(id);
        List<MthMyMatchListDetailAppVo.Detail> details = mthMatchManageDao.queryRegisterDetailForApp(list);
        setDetailFileValue(details);
        Map<String,Object> map=new HashMap();
        for (MthMyMatchListDetailAppVo.Detail detail : details) {
            Object object;
            //封装map数据
            if (CollectionUtil.isNotEmpty(detail.getFileValues())){
                object = detail.getFileValues().stream().map(file -> file.getObjectKey()).collect(Collectors.joining(","));
            }else {
                object=detail.getValue();
            }
            map.put(detail.getShowName(),object);
        }
        return map;
    }

    @Override
    public void exportContestantList(PageReq<MthMatchManagerContestantReqBo> req) {
        List<MthMatchManagerContestantVo> list = mthMatchManageDao.queryMatchContestantList(req, true).getResult();
        List<Map<String, Object>> data=new ArrayList<>();
        for (MthMatchManagerContestantVo item : list) {
            item.setTelephone(DesensitizedUtil.mobilePhone(item.getTelephone()));
            Map<String, Object> roundIndfo=new HashMap<>();
            roundIndfo.put("matchName",item.getMatchName());
            roundIndfo.put("areaName",item.getAreaName());
            roundIndfo.put("itemName",item.getItemName());
            roundIndfo.put("groupName",item.getGroupName());
            roundIndfo.put("userName",item.getUserName());
            roundIndfo.put("telephone",item.getTelephone());
            roundIndfo.put("roundName",item.getRoundName());
            roundIndfo.put("registerTime",DateUtil.format(item.getRegisterTime()));
            roundIndfo.put("score",item.getScore());
            roundIndfo.put("result",item.getResult());
            roundIndfo.put("prizeName",item.getPrizeName());
            roundIndfo.putAll(getDetail(item.getRegisterId()));
            data.add(roundIndfo);
        }
        RoundColumVo roundColumVo = roundColum(req.getQuery());
        try {
            setNoModelWriteData(roundColumVo, "参赛者列表",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("导出参赛者列表错误:", e);
        }
       /* try {
            ExcelUtil.writeSingleCommonExcel("参赛者列表", null, list, MthMatchManagerContestantVo.class);
        } catch (IOException e) {
            e.printStackTrace();
            log.error("导出参赛者列表错误:", e);
        }*/
    }

    @Override
    public PageRsp<MthRoundDeliveryVo> getDeliveryInfo(PageReq<MthDeliveryBo> req) {
        return mthMatchManageDao.getDeliveryInfo(req, false);
    }

    @Override
    public void exportDelivery(PageReq<MthDeliveryBo> req) {
        List<MthRoundDeliveryVo> list = mthMatchManageDao.getDeliveryInfo(req, true).getResult();
        try {
            for (MthRoundDeliveryVo item : list) {
                item.setDeliveryFlagName(Objects.equals(item.getDeliveryFlag(), 1) ? "是" : "否" );
            }
            ExcelUtil.writeSingleCommonExcel("收货地址地址", null, list, MthRoundDeliveryVo.class);
        } catch (IOException e) {
            e.printStackTrace();
            log.error("导出收获地址错误:", e);
        }
    }

    @Override
    public boolean updateDlieryFlag(MthRoundDeliveryBo bo) {
        MthRoundScore mthRoundScore = roundScoreRepo.getOne(bo.getRoundScoreId());
        mthRoundScore.setDeliveryFlag(bo.getDeliveryFlag());
        roundScoreRepo.saveAndFlush(mthRoundScore);
        return true;
    }

    @Override
    public PageRsp<MthWorkPromotionVo> queryWorkPromotionManagerList(PageReq<MthWorkPromiseReqBo> bo) {
        PageRsp<MthWorkPromotionVo> page = mthMatchManageDao.queryWorkPromotionManagerList(bo);
//        List<MthWorkPromotionVo> mthWorkPromotionVos = page.getResult();
//        List<MthWorkPromotionVo> result = new LinkedList<>();
//        /**筛选评审结果是否已有晋级排名设置，若sort均为空，则代表评审结果均未设置晋级排名
//         * 若sort至少有一条记录不为空，则代表评审结果已设置过晋级排名 */
//        List<MthWorkPromotionVo> temp = mthWorkPromotionVos.stream()
//                .filter(vo -> {
//                    return ObjectUtil.isNotEmpty(vo.getSort());
//                }).collect(Collectors.toList());
//
//        /***当第一次查询时，如未设置晋级排名，需根据得分自动计算排名后返回前端展示
//         * 第N次查询时，如已有晋级排名设置，则根据数据实际排名展示 */
//        if (ObjectUtil.isNotEmpty(temp)) {
//            result = mthWorkPromotionVos.stream()
//                    .filter(vo -> {
//                        return ObjectUtil.isNotEmpty(vo.getSort());
//                    })
//                    .sorted(Comparator.comparing(v -> v.getSort())).collect(Collectors.toList());
//        } else {
//            /**查询该场次的晋级排名规则*/
//            MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.getById(bo.getQuery().getRoundId());
//            if (mthRoundInstance.getSocreMode().equals("grad")) {
//                mthWorkPromotionVos.stream().forEach(v -> {
//                    if (ObjectUtil.isNotEmpty(v.getScore())) {
//                        if (v.getScore().equals("A")) {
//                            v.setSort(1);
//                        } else if (v.getScore().equals("B")) {
//                            v.setSort(2);
//                        } else if (v.getScore().equals("C")) {
//                            v.setSort(3);
//                        } else if (v.getScore().equals("D")) {
//                            v.setSort(4);
//                        } else {
//                            v.setSort(5);
//                        }
//                    }
//                });
//            } else if (mthRoundInstance.getSocreMode().equals("pass")) {
//                mthWorkPromotionVos.stream().forEach(v -> {
//                    if (ObjectUtil.isNotEmpty(v.getScore())) {
//                        if (v.getScore().equals("通过")) {
//                            v.setSort(1);
//                        } else {
//                            v.setSort(2);
//                        }
//                    }
//                });
//            } else if (mthRoundInstance.getSocreMode().equals("ten_point") ||
//                    mthRoundInstance.getSocreMode().equals("hundred_point")) {
//                /**获取得分分布情况*/
//                /**方式2*/
////                mthWorkPromotionVos=mthWorkPromotionVos.stream().
////                        sorted(Comparator.comparing(MthWorkPromotionVo::getScore,Comparator.comparing(item->{
////                            if (ObjectUtil.isEmpty(item)){
////                                item="0";
////                            }
////                            return Integer.valueOf(item);
////                        })).reversed())
////                        .collect(Collectors.toList());
//                /**方式1*/
//                mthWorkPromotionVos.forEach(v -> {
//                    if (ObjectUtil.isNotNull(v.getScore())) {
//                        v.setTempScore(Double.parseDouble(v.getScore()));
//                    } else {
//                        v.setTempScore(0.00);
//                    }
//                });
//                mthWorkPromotionVos = mthWorkPromotionVos.stream().
//                        sorted(Comparator.comparing(MthWorkPromotionVo::getTempScore).
//                                reversed()).collect(Collectors.toList());
//                int index = 0;
//                double lastScore = -1;
//                /**
//                 * 出现并列排名后名次不间断
//                 */
//                for (MthWorkPromotionVo item : mthWorkPromotionVos) {
//                    if (ObjectUtil.isNotNull(item.getScore())) {
//                        if (Double.compare(lastScore, Double.parseDouble(item.getScore())) != 0) {
//                            lastScore = Double.parseDouble(item.getScore());
//                            index++;
//                        }
//                        item.setSort(index);
//                    }
//                }
//            }
//            result = mthWorkPromotionVos.stream()
//                    .filter(vo -> {
//                        return ObjectUtil.isNotEmpty(vo.getSort());
//                    })
//                    .sorted(Comparator.comparing(v -> v.getSort())).collect(Collectors.toList());
//        }
//        page.setResult(result);
        return page;
    }

    /**
     * 查询赛事过程;
     */
    @Override
    public List<MthMatchProcessVo> queryProcessInfo(MthMatchProcessReqBo bo) {
        return mthMatchManageDao.queryProcessInfo(bo);
    }

    /**
     * 查询赛事信息主;
     */
    @Override
    public MthMatchInfoVo queryById(Long id) {
        return baseConverter.entityToVo(baseRepo.getById(id));
    }

    /**
     * 查询赛事信息主;列表
     */
    @Override
    public Result<PageRsp<MthMatchInfoVo>> queryPageList(PageReq<MthMatchInfoBo> bo) {
        return null;
    }

    @Override
    public Result<PageRsp<MthMatchInfoAppVo>> queryPageListForApp(PageReq<MthMatchInfoAppBo> bo) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        bo.getQuery().setStatusList(MatchInfoStatusEnums.queryAppList());
        PageRsp<MthMatchInfoAppVo> result = matchManageDao.queryPageListForApp(bo);
        // 设置文件url地址
        obsFilService.setObsFileBoByIds(result.getResult(), MthMatchInfoAppVo::getBannerId, MthMatchInfoAppVo::setBanner);
        if (ObjectUtil.isNotEmpty(result.getResult())) {
            for (MthMatchInfoAppVo item : result.getResult()) {
                //是否收藏
                List<MthMatchCollected> matchs = mthMatchCollectedRepo.findByMatchId(item.getId(), (long) worker.getLoginId());
                item.setWhetherCollect(matchs.size() > 0);
                //查询报名记录
                List<MthRegister> mthRegisters = mthRegisterDao.findRegisterByUserId(worker.getLoginId(),item.getId(),RegisterAuditStatusEnums.CANCEL.getCode());
                item.setWhetherApply(mthRegisters.size()>0);
                // 获取赛事状态
                item.setStatusDesc(MatchInfoStatusEnums.getAppStatusDesc(item.getStatus()));

            }
        }
        return Result.ofSuccess(result);
    }

    /**
     * 查询赛事信息主;列表
     */
    @Override
    public List<MthMatchInfoVo> queryList(MthMatchInfoBo bo) {
        return matchManageDao.queryList(bo);
    }

    /**
     * 新增赛事信息主;
     */
    @Override
    public Long insertByBo(MthMatchInfoBo bo) {
        MthMatchInfo add = baseConverter.boToEntity(bo);
        if (ObjectUtil.isNotNull(bo.getFileDto()) && ObjectUtil.isNotNull(bo.getFileDto().getFileId())) {
            add.setBannerId(bo.getFileDto().getFileId());
        }
        //类型为实例
        validEntityBeforeSave(add);
        add.setStatus(MatchInfoStatusEnums.DRAFT.getStatus());
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        add.setCreateOrgId(Long.valueOf(currWorker.getOrgId()));
        add.setCreateOrgName(currWorker.getOrgName());
        add.setCreateDeptName(currWorker.getDepartment());
        baseRepo.save(add);

        //保存赛项 赛别
        if (CollectionUtil.isNotEmpty(bo.getMatchItemBoList())) {
            saveMatchItemAndGroup(add.getId(), bo.getMatchItemBoList());
        }
        if (CollectionUtil.isNotEmpty(bo.getSponsorBoList())) {
            saveSponsor(add.getId(), bo.getSponsorBoList(), 1);
        }
        if (CollectionUtil.isNotEmpty(bo.getOrganizerBoList())) {
            saveSponsor(add.getId(), bo.getOrganizerBoList(), 2);
        }
        if (CollectionUtil.isNotEmpty(bo.getLabelList())) {
            saveMatchLabel(add.getId(), bo.getLabelList());
        }
        // 自动创建比赛等级第一场场次
        createLevelFirstRound(add.getId(), add.getMatchLevel());
        return add.getId();
    }

    private void createLevelFirstRound(Long matchId, String matchLevel) {
        List<String> matchLevelList = StrUtil.split(matchLevel, StrUtil.COMMA);
        for (RoundEnums.AreaLevel value : RoundEnums.AreaLevel.values()) {
            if (matchLevelList.contains(value.getCode())) {
                List<MthRoundInstance> levelRoundList = roundInstanceRepo.findByMatchIdAndRoundTypeAndMatchLevel(matchId, RoundEnums.RoundType.TEMPLATE.getCode(), value.getCode());
                if (CollectionUtil.isEmpty(levelRoundList)) {
                    MthRoundInstanceBo instanceBo = new MthRoundInstanceBo();
                    instanceBo.setMatchId(matchId);
                    instanceBo.setMatchLevel(value.getCode());
                    instanceBo.setRoundName("初赛");
                    instanceBo.setRoundSort(1);
                    // 自动创建比赛等级第一场场次
                    iMthRoundInstanceService.insertByBo(instanceBo);
                }
            } else {
                // 删除不存在的赛事等级场次
                List<MthRoundInstance> levelRoundList = roundInstanceRepo.findByMatchIdAndRoundTypeAndMatchLevel(matchId, RoundEnums.RoundType.TEMPLATE.getCode(), value.getCode());
                if (CollectionUtil.isNotEmpty(levelRoundList)) {
                    List<Long> roundIdList = StreamUtils.toList(levelRoundList, MthRoundInstance::getId);
                    iMthRoundInstanceService.deleteWithValidByIds(roundIdList, true);
                }
            }
        }
    }

    private boolean saveMatchLabel(Long matchId, List<MthMatchLabelBo> labelBoList) {
        List<MthMatchLabel> list = Lists.newArrayList();
        labelBoList.stream().forEach(labelBo -> {
            MthMatchLabel label = labelConverter.boToEntity(labelBo);
            label.setMatchId(matchId);
            list.add(label);

        });
        if (CollectionUtil.isNotEmpty(list)) {
            labelRepo.saveAllAndFlush(list);
        }
        return true;
    }

    private boolean saveSponsor(Long matchId, List<MthMatchSponsorBo> boList, Integer type) {
        List<MthMatchSponsor> list = Lists.newArrayList();
        boList.stream().forEach(mthMatchSponsorBo -> {
            MthMatchSponsor sponsor = matchSponsorConverter.boToEntity(mthMatchSponsorBo);
            sponsor.setMatchId(matchId);
            sponsor.setType(type);
            list.add(sponsor);
        });
        if (CollectionUtil.isNotEmpty(list)) {
            mthMatchSponsorRepo.saveAll(list);
        }
        return true;
    }

    /**
     * @param bo
     * @description: 保存赛项及组别
     * @return:
     * @author liuchuan
     * @date: 2023/9/22 14:16
     */
    private boolean saveMatchItemAndGroup(Long matchId, List<MthMatchItemBo> bo) {
        bo.stream().forEach(itemBo -> {
            MthMatchItem item = new MthMatchItem();
            item.setMatchId(matchId);
            item.setItemName(itemBo.getItemName());
            item.setDelFlag(0);
            if (Objects.nonNull(itemBo.getId())) {
                item.setId(itemBo.getId());
            } else {
                item.setId(IdUtil.getSnowflakeNextId());
            }
            matchItemRepo.save(item);
            if (CollectionUtil.isNotEmpty(itemBo.getGroupBoList())) {
                itemBo.getGroupBoList().stream().filter(group -> StrUtil.isNotBlank(group.getGroupName())).forEach(mthMatchGroupBo -> {
                    MthMatchGroup matchGroup = new MthMatchGroup();
                    matchGroup.setItemId(item.getId());
                    matchGroup.setMatchId(matchId);
                    matchGroup.setGroupName(mthMatchGroupBo.getGroupName());
                    matchGroup.setAgeBegin(mthMatchGroupBo.getAgeBegin());
                    matchGroup.setAgeEnd(mthMatchGroupBo.getAgeEnd());
                    matchGroup.setDelFlag(0);
                    if (Objects.nonNull(mthMatchGroupBo.getId())) {
                        matchGroup.setId(mthMatchGroupBo.getId());
                    } else {
                        matchGroup.setId(IdUtil.getSnowflakeNextId());
                    }
                    matchGroupRepo.save(matchGroup);
                });


            }
        });
        return true;
    }

    /**
     * 修改赛事信息主;
     */
    @Override
    public Boolean updateByBo(MthMatchInfoBo bo) {
        MthMatchInfo update = baseConverter.boToEntity(bo);
        if (ObjectUtil.isNotNull(bo.getFileDto()) && ObjectUtil.isNotNull(bo.getFileDto().getFileId())) {
            update.setBannerId(bo.getFileDto().getFileId());
        }
        validEntityBeforeSave(update);
        //删除赛项组别数据
        matchItemRepo.deleteMthMatchItemsByMatchId(update.getId());
        matchGroupRepo.deleteMthMatchGroupsByMatchId(update.getId());
        //删除主办方
        mthMatchSponsorRepo.deleteByMatchId(update.getId());
        ///标签
        labelRepo.deleteByMatchId(update.getId());
        MthMatchInfo matchInfo = baseRepo.getOne(bo.getId());
        //判断是否修改比赛报名时间
        if (!Objects.equals(update.getRegisterationStart(),matchInfo.getRegisterationStart()) ||
                !Objects.equals(update.getRegisterationEnd(),matchInfo.getRegisterationEnd())) {
            //需要更新场次中的报名时间
            roundInstanceRepo.updateRegisterTimeByMatchId(matchInfo.getId(), update.getRegisterationStart(),
                    update.getRegisterationEnd());
        }
        BeanUtil.copyProperties(update, matchInfo, CopyOptions.create().setIgnoreNullValue(true));
        baseRepo.saveAndFlush(matchInfo);
        //保存赛项 赛别
        if (CollectionUtil.isNotEmpty(bo.getMatchItemBoList())) {
            saveMatchItemAndGroup(update.getId(), bo.getMatchItemBoList());
        }

        if (CollectionUtil.isNotEmpty(bo.getSponsorBoList())) {
            saveSponsor(update.getId(), bo.getSponsorBoList(), 1);
        }

        if (CollectionUtil.isNotEmpty(bo.getOrganizerBoList())) {
            saveSponsor(update.getId(), bo.getOrganizerBoList(), 2);
        }

        if (CollectionUtil.isNotEmpty(bo.getLabelList())) {
            saveMatchLabel(update.getId(), bo.getLabelList());
        }
        // 自动创建比赛等级第一场场次
        createLevelFirstRound(update.getId(), update.getMatchLevel());
        return true;
    }

    @Override
    public Long saveDraft(MthMatchInfoBo bo) {
        if (Objects.isNull(bo.getId())) {
            return insertByBo(bo);
        }
        updateByBo(bo);
        return bo.getId();
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(MthMatchInfo entity) {
        //名称是否重复
        List<MthMatchInfo> list = baseRepo.findByMatchNameLikeAndIdIsNot(entity.getMatchName(), ObjectUtil.isNull(entity.getId()) ? 0L : entity.getId());
        if (CollectionUtil.isNotEmpty(list)) {
            throw new BusinessException("比赛名称重复");
        }
    }

    /**
     * 批量删除赛事信息主;
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if (isValid) {
            List<MthMatchInfo> list = matchManageDao.getMatchInfoByIds(ids);
            if (CollectionUtil.isEmpty(list)) {
                throw new BusinessException("输入错误参数");
            }
            List<String> statusList = list.stream().map(MthMatchInfo::getStatus).collect(Collectors.toList());
            for (String status : statusList) {
                if (!MatchInfoStatusEnums.deletableList().contains(status)) {
                    throw new BusinessException("存在不可删除赛事");
                }
            }
        }

        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        matchManageDao.deleteByIds(ids, currWorker.getLoginId(), currWorker.getWorkerName());
        return true;
    }

    public void validCategoryIdExistsUse(Long categoryId) {
        if (matchManageDao.validateCategoryExists(categoryId) > 0) {
            throw new BusinessException("已被赛事占用，不可修改和删除");
        }
    }

    public void validLabelUse(Long labelId) {
        if (matchManageDao.validateLabelExists(labelId) > 0) {
            throw new BusinessException("已被赛事占用，不可修改和删除");
        }
    }

    public void validPrizeUse(Long prizeId) {
        if (matchManageDao.validatePrizeExists(prizeId) > 0) {
            throw new BusinessException("已被赛事占用，不可修改和删除");
        }
    }


    /**
     * @param
     * @description: 结束比赛
     * 1、将比赛状态设置为结束
     * 2、将场次名单状态设置为END
     * 3、初始化下一场次数据
     * 将晋级名单插入下一场次名单中
     * 设置一下场场次状态 线上评审设置为作品待评审1    线下评审设置为 待设置晋级2
     * @return:
     * @author liuchuan
     * @date: 2023/11/22 14:45
     */
    @Override
    public boolean matchRoundFinish(MthMatchManagerReqBo req) {
        MthRoundVo mthRoundInstanceVo = roundInsInstanceDao.findMatchRound(req);
        if (ObjectUtil.isNotEmpty(req.getMatchRoundState().get(0))) {
            mthRoundInstanceVo.setMatchState(req.getMatchRoundState().get(0).toString());
        }
        mthRoundInstanceVo.setUpdateTime(new Date());
        // 更新round_list
//        List<MthRoundList> mthRoundListList=roundListRepo.findRoundListByRoundId(req.getMthRoundId());
//        for (MthRoundList item:mthRoundListList){
//            item.setStatus(RoundListStatusEnums.END.getCode());
//            item.setUpdateTime(new Date());
//        }
//        roundListRepo.saveAllAndFlush(mthRoundListList);
        roundListRepo.updateRoundListStatus(req.getMthRoundId());
        roundInsInstanceDao.finishMatchRound(mthRoundInstanceVo);

        try {
            //发送获奖消息、晋级消息
            MthMatchInfo matchInfo = baseRepo.getOne(mthRoundInstanceVo.getMatchId());
            sendAwardsMsg(matchInfo.getMatchName(),mthRoundInstanceVo);
            sendPromotionMsg(matchInfo.getMatchName(),mthRoundInstanceVo);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("消息发送失败:",e);
        }

        //场次流转
        roundTurn(req.getMthRoundId());
        return true;
    }

    /**
     * @description: 比赛结束 发送获奖通知及晋级通知
     * @param
     * @return:
     * @author liuchuan
     * @date: 2023/12/11 15:46
    */
    private void sendAwardsMsg(String matchName ,MthRoundVo vo){
        // TODO: 2023/12/11
        List<Object[]> awarsVoList = roundScoreRepo.getAwardsList(vo.getId());
       List<PushAwardsBo> pushAwardsBoList = Lists.newArrayList();
      if (CollectionUtil.isNotEmpty(awarsVoList)){
          for (Object[] obj :awarsVoList){
              BigInteger bigInteger = (BigInteger)obj[0];
              PushAwardsBo bo = new PushAwardsBo();
              bo.setUserId(Long.valueOf((bigInteger.toString())));
              String awards="";
              String prizeName = (String) obj[1];
              String otherPrizeName =(String) obj[2];
              if (ObjectUtil.isNotEmpty(prizeName)){
                  awards = prizeName;
              }
              if (ObjectUtil.isNotEmpty(otherPrizeName)){
                  awards = awards +" " +otherPrizeName;
              }
              bo.setAward(awards);
              bo.setCompetition(matchName);
              pushAwardsBoList.add(bo);
          }
      }


       if (CollectionUtil.isNotEmpty(pushAwardsBoList)){
           pushBusinessMsgService.pushAwardsMsg(pushAwardsBoList);
       }
    }

    private void sendPromotionMsg(String matchName ,MthRoundVo vo){
        log.info("发送晋级消息:{} ",matchName);
        List<Long> userIdList  = roundScoreRepo.getPromptionList(vo.getId());
        if (CollectionUtil.isNotEmpty(userIdList)){
            log.info("发送晋级消息:{} ",matchName);
            pushBusinessMsgService.pushPromotedMsg(userIdList,matchName);
        }


    }
    /**
     * @param roundId
     * @description: 场次流转
     * 初始化下一场次数据
     * 将晋级名单插入下一场次名单中
     * 设置一下场场次状态 线上评审设置为作品待评审1    线下评审设置为 待设置晋级2
     * @return:
     * @author liuchuan
     * @date: 2023/11/22 17:19
     */
    public void roundTurn(Long roundId) {
        /** 流转场次处理 */
        MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.findById(roundId).orElseThrow(() -> new BusinessException("系统错误，未查询到该场次"));
        MthRoundInstance nextRoundInstance = iMthRoundScoreService.queryNextRoundIns(mthRoundInstance);

        /**nextRoundInstance!=null 还有后续比赛需要初始化数据 */
        if (ObjectUtil.isNotNull(nextRoundInstance)&&ObjectUtil.isNotNull(nextRoundInstance.getId())) {
            if (ObjectUtil.isEmpty(nextRoundInstance.getRoundState())||nextRoundInstance.getRoundState()==1){
                //下一场次 未开启
                throw  new BusinessException("下一场次("+nextRoundInstance.getRoundName()+")未开启，请开启下一场次");
            }
//            /**关联新的mth_round_List记录*/
            iMthRoundScoreService.dealNextRoundInfo(mthRoundInstance, nextRoundInstance);

            updateNextRoundState(mthRoundInstance, nextRoundInstance);

        }else {

            //判断是否所有场次都结束  
            if (mthRoundInstanceRepo.queryMatchNotFinishNum(mthRoundInstance.getMatchId())==0){

                //结束比赛
                matchManageDao.updateStatus(Arrays.asList(mthRoundInstance.getMatchId()), MatchInfoStatusEnums.FINISHED.getStatus());
                // 禁用相关账号
                try {
                    matchOrganizerAccountService.updateOrganizerAccountState(mthRoundInstance.getMatchId());
                } catch (Exception e) {
                    log.info("禁用账号报错:",e);
                    e.printStackTrace();
                }
            }
        }

    }

    /**
     * @param
     * @description: 更新下一场次状态
     * 1、场次是同等级 进行状态更改
     * 2、是不同等级 需要判断上一等级是否全都已结束
     * @return:
     * @author liuchuan
     * @date: 2023/12/7 17:18
     */
    private void updateNextRoundState(MthRoundInstance mthRoundInstance, MthRoundInstance nextRoundInstance) {
        if (!mthRoundInstance.getRoundLevel().equals(nextRoundInstance.getRoundLevel())) {
            //下一级市区级而且当前时片区级 判断上一级是否只剩当前场次未完成
            if (nextRoundInstance.getRoundLevel().equals(RoundEnums.AreaLevel.CITY.getLevel())&&
                    mthRoundInstance.getRoundLevel().equals(RoundEnums.AreaLevel.DISTRICT.getLevel())) {
                //查询片区级已结束数量
                Integer finishNum = mthRoundInstanceRepo.queryDistrictFinishNum(mthRoundInstance.getMatchId());
                //查询片区级总量
                Integer totalNum = mthRoundInstanceRepo.queryDistrictNum(mthRoundInstance.getMatchId());
                if (!totalNum.equals(finishNum)) {
                    //加上当前场次还不相等 则片区没有完成
                    log.info("加上当前场次还不相等 则片区没有完成.matchId:{},roundId:{} ", mthRoundInstance.getMatchId(),
                            mthRoundInstance.getId());
                    return;
                }
            }
            //下一级市区级而且当前时区县级 判断上一级是否只剩当前场次未完成
            if (nextRoundInstance.getRoundLevel().equals(RoundEnums.AreaLevel.CITY.getLevel())&&
                    mthRoundInstance.getRoundLevel().equals(RoundEnums.AreaLevel.COUNTY_SCHOOL.getLevel())) {
                //查询片区级已结束数量
                Integer finishNum = mthRoundInstanceRepo.queryCountryLevelFinishNum(mthRoundInstance.getMatchId());
                //查询片区级总量
                Integer totalNum = mthRoundInstanceRepo.queryCountryLevelTotal(mthRoundInstance.getMatchId());
                if (!totalNum.equals(finishNum)) {
                    //加上当前场次还不相等 则片区没有完成
                    log.info("加上当前场次还不相等 则片区没有完成.matchId:{},roundId:{} ", mthRoundInstance.getMatchId(),
                            mthRoundInstance.getId());
                    return;
                }
            }
            //是片区级 查询上一等级（区县级）是否都完成
            if (nextRoundInstance.getRoundLevel().equals(RoundEnums.AreaLevel.DISTRICT.getLevel())) {
                //区县级已结束总量
                Integer finishNum = mthRoundInstanceRepo.querycityCountryFinishNum(mthRoundInstance.getMatchId(),nextRoundInstance.getAreaId());
                Integer totalNum = mthRoundInstanceRepo.querycityCountryNum(mthRoundInstance.getMatchId(),nextRoundInstance.getAreaId());

                if (!totalNum.equals(finishNum)) {
                    //加上当前场次还不相等 则片区没有完成
                    log.info("加上当前场次还不相等 则区县级没有完成.matchId:{},roundId:{} ", mthRoundInstance.getMatchId(),
                            mthRoundInstance.getId());
                    return;
                }
            }

        }

        /**处理下一场次mth_Round_Instance的记录状态*/
        //是场次升级 需要判断上一级都晋级后，再设置场次状态
        if ("online".equals(nextRoundInstance.getAuditMode())) {
            nextRoundInstance.setMatchState(1);
        } else {//线下赛事流转第二场次 状态设置为2 待设置晋级
            nextRoundInstance.setMatchState(2);
        }
        nextRoundInstance.setUpdateTime(new Date());
        mthRoundInstanceRepo.saveAndFlush(nextRoundInstance);

    }

    /**
     * @param req
     * @description: 结束公示
     * 1、判断是否到公示截止时间
     * 2、将场次状态设置为待结束
     * @return:
     * @author liuchuan
     * @date: 2023/11/22 14:31
     */
    @Override
    public boolean matchRoundPulicityFinish(MthMatchManagerReqBo req) {
        try {
            MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.findById(req.getMthRoundId()).orElseThrow(() -> new BusinessException("系统错误，未查询到该场次"));

            if (ObjectUtil.isNotNull(mthRoundInstance.getPublicityDeadline()) &&
                    mthRoundInstance.getPublicityDeadline().after(new Date())) {
                throw new BusinessException("公示截止日期未到，不能结束公示");

            }
            mthRoundInstance.setUpdateTime(new Date());
            mthRoundInstance.setMatchState(3);

            mthRoundInstanceRepo.saveAndFlush(mthRoundInstance);

            // 向承办方发送消息
            // 通过areaId查询场次相关的承办方
            if (ObjectUtil.isNotEmpty(mthRoundInstance.getAreaId())){
                List<MthMatchContractor> contractors = mthMatchContractorRepo.findByAreaIdAndDelFlag(mthRoundInstance.getAreaId(),0);
                String matchName = "";
                if (ObjectUtil.isNotEmpty(contractors)){
                    List<Long> contractorIds = contractors.stream().filter(item -> ObjectUtil.isNotEmpty(item.getContractorId())).map(MthMatchContractor::getContractorId)
                        .collect(Collectors.toList());
                    // 查询比赛信息
                    MthMatchInfo mthMatchInfo  = mthMatchInfoRepo.findById(mthRoundInstance.getMatchId()).orElseThrow(() -> new BusinessException("根据比赛id:"+mthRoundInstance.getMatchId()+ "未找到比赛信息!"));
                    if (ObjectUtil.isNotEmpty(mthMatchInfo.getMatchName())){
                        matchName = mthMatchInfo.getMatchName();
                    }
                    log.info("比赛待结束通知:{} ",matchName);
                    pushBusinessMsgService.pushMatchFinish(contractorIds,matchName);
                }
            }


        } catch (BusinessException e) {
            log.error("公示结束错误:", e);
            e.printStackTrace();
            throw new BusinessException("公示结束错误");
        }

        return true;
    }

    @Override
    public MthRegistrationTimeNodeVo queryRegistrationProcessInfo(MthRegistrationTimeNodeReqBo req) {
        return roundInsInstanceDao.queryRegistrationProcessInfo(req);
    }

    @Override
    public MthRoundInstanceVo queryRoundInfoById(Long id) {
        MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.getById(id);
        return mthRoundConverter.entityToVo(mthRoundInstance);
    }


    @Override
    public List<MthMatchItem> getItemListByMatchId(Long matchId) {

        return matchItemRepo.findByMatchId(matchId);
    }

    @Override
    public List<MthMatchGroup> getGroupListByItemId(Long itemId) {

        return matchGroupRepo.findByItemId(itemId);
    }

    /**
     * APP赛事活动-查询赛事列表-查看详情
     * @param bo
     * @return
     */
    @Override
    public Result<MthMatchInfoAppVo> doListDetails(Worker worker,MthMatchInfoAppBo bo) {
        if (ObjectUtil.isEmpty(bo)|| bo.getId()==null){
            throw new BusinessException(REQUEST_ERROR);
        }
        MthMatchInfoAppVo mthMatchInfoAppVo=new MthMatchInfoAppVo();
        MthMatchInfo matchInfo = mthMatchInfoRepo.getOne(bo.getId());
        if(ObjectUtil.isEmpty(matchInfo)){
            throw new BusinessException(MATCH_NOT_EXIST);
        }
        BeanUtil.copyProperties(matchInfo,mthMatchInfoAppVo);
        //是否收藏
        List<MthMatchCollected> matchs = mthMatchCollectedRepo.findByMatchId(matchInfo.getId(), (long) worker.getLoginId());
        mthMatchInfoAppVo.setWhetherCollect(matchs.size() > 0);
        //查询报名记录
        List<MthRegister> mthRegisters = mthRegisterDao.findRegisterByUserId(worker.getLoginId(),matchInfo.getId(),RegisterAuditStatusEnums.CANCEL.getCode());
        mthMatchInfoAppVo.setWhetherApply(mthRegisters.size()>0);
        // 获取赛事状态
        mthMatchInfoAppVo.setStatusDesc(MatchInfoStatusEnums.getAppStatusDesc(matchInfo.getStatus()));
        return Result.ofSuccess(mthMatchInfoAppVo);
    }

    @Override

    public List<MatchBannerVo> queryBanner() {
        List<MatchBannerVo> bannerVoList = matchManageDao.queryBanner();
        if (CollectionUtil.isNotEmpty(bannerVoList)){
            bannerVoList.stream().forEach(bannerVo->{
                YcqObsFile file = fileRepo.findByIdAndState(bannerVo.getFileId(), 1);
                if (ObjectUtil.isNotNull(file)) {
                    FileDto fileDto = fileConverter.ObsFileToFileDto(file);
                    fileDto.setFileUrl(urlPrefix + fileDto.getFileKey());
                    BeanUtil.copyProperties(fileDto,bannerVo);
                }
            });
        }

        return bannerVoList;
    }


    @Override
    public boolean queryCityLevelFlag(Long workerId) {
        return matchManageDao.queryCityLevelFlag(workerId);
    }

    @Override
    public void crateCompetitionReview(ValueC valueC) {
        TsmPropertyValue tsmPropertyValue = tsmPropertyValueRepo.findTsmPropertyValueByValueNameAndPropCodeAndState(valueC.getValueName(), valueC.getPropCode(), 1);
        if(Objects.nonNull(tsmPropertyValue)){
            throw new BusinessException("审核原因已存在");
        }
        propertyService.createValue(valueC);
    }

    @Override
    public void updateCompetitionReview(ValueU valueU) {
        TsmPropertyValue value = tsmPropertyValueRepo.getOne(valueU.getValueId());
        if (value==null){
            throw new BusinessException("数据异常！");

        }
        // 如果修改静态值，则判断当前是否存在值
            TsmPropertyValue tsmPropertyValue = tsmPropertyValueRepo.findTsmPropertyValueByValueNameAndPropCodeAndState(valueU.getValueName(), value.getPropCode(),1);
            if(Objects.nonNull(tsmPropertyValue)) {
                if (tsmPropertyValue.getValueId()!=value.getValueId()) {
                    throw new BusinessException("审核原因已存在");
                }
            }
        propertyService.updateValue(valueU);
    }




   private List<MatchTableInfoVo> getRoundTablefoVo(boolean group){
       List<MatchTableInfoVo> list=new ArrayList<>();
       list.add(new MatchTableInfoVo("matchName","赛事名称"));
       list.add(new MatchTableInfoVo("areaName","比赛范围"));
       list.add(new MatchTableInfoVo("itemName","赛项"));
       if (group){
           list.add(new MatchTableInfoVo("groupName","组别"));
       }
       list.add(new MatchTableInfoVo("roundName","场次名称"));

       list.add(new MatchTableInfoVo("userName","参赛者姓名"));
       list.add(new MatchTableInfoVo("telephone","参赛者手机号"));
       list.add(new MatchTableInfoVo("registerTime","报名时间"));
       list.add(new MatchTableInfoVo("score","评审结果"));
       list.add(new MatchTableInfoVo("result","是否晋级"));
       list.add(new MatchTableInfoVo("prizeName","获奖名次"));
       return list;
   }



    @Override
    public RoundColumVo roundColum(MthMatchManagerContestantReqBo mthMatchManagerContestantReqBo) {
        //固定字段 是否有组别
        RoundColumVo roundColum=new  RoundColumVo();
        List<MatchBaseGroupVo> groups = groupDao.getGroups(mthMatchManagerContestantReqBo.getMthMatchId());
        boolean groupFlag=false;
        if (CollectionUtil.isNotEmpty(groups)){
            //存在赛项组别
            groupFlag=true;
        }

        //Map<String,String> table= setMatchInfo(groupFlag);
        List<MatchTableInfoVo> matchInfoVo = getRoundTablefoVo(groupFlag);
        List<String> collect = matchInfoVo.stream().map(item -> item.getTableName()).collect(Collectors.toList());
        roundColum.setTable(matchInfoVo);
        MatchRegisterFormVo matchRegisterFormVos = mthMatchManageDao.getregisterInfos(mthMatchManagerContestantReqBo.getMthMatchId());
        if (matchRegisterFormVos!=null) {
            //去重报名表字段
            List<MthMyMatchListDetailAppVo.Detail>configList = mthMatchManageDao.findConfigInfoBy(matchRegisterFormVos.getRegisterId()).stream().filter(item->
                !collect.contains( item.getShowName())
            ).collect(Collectors.toList());
            roundColum.setMap(configList);
        }




        return roundColum;
    }
    private void setDetailFileValue(List<MthMyMatchListDetailAppVo.Detail> detailList) {
        if (detailList == null) {
            return;
        }
        // 查询文件类型信息
        List<MthMyMatchListDetailAppVo.Detail> fileDetailList = detailList.stream()
                .filter(item -> StrUtil.isNotBlank(item.getValue()))
                .filter(item -> FILE_TYPES.contains(item.getType()))
                .collect(Collectors.toList());
        // 设置文件路径
        obsFilService.setObsFileBoListByIds(fileDetailList, item -> item.getValue(), MthMyMatchListDetailAppVo.Detail::setFileValues);
    }





    private List<MatchTableInfoVo> getRegisterInfoColumInfo(boolean review){
        List<MatchTableInfoVo> list=new ArrayList<>();
        list.add(new MatchTableInfoVo("matchName","赛事名称"));
        list.add(new MatchTableInfoVo("areaName","比赛范围"));
        list.add(new MatchTableInfoVo("itemName","赛项"));
        list.add(new MatchTableInfoVo("groupName","组别"));
        list.add(new MatchTableInfoVo("roundName","场次名称"));
        list.add(new MatchTableInfoVo("userName","参赛者姓名"));
        list.add(new MatchTableInfoVo("telephone","参赛者手机号"));
        list.add(new MatchTableInfoVo("registerTime","报名时间"));
        if (review) {
            list.add(new MatchTableInfoVo("reviewResult", "审核状态"));
            list.add(new MatchTableInfoVo("reviewTime", "审核时间"));
        }
        return list;
    }

    private  boolean reviewFlag(Integer registerFlag){
        boolean reviewFlag=false;
           if(registerFlag!=null){
            if (registerFlag==0){
                reviewFlag=true;
            }
        }
        return reviewFlag;
    }


    @Override
    public RoundColumVo registerInfoColum(MthMatchManagerRegisterReqBo req) {

        RoundColumVo roundColumVo=new RoundColumVo();
        //是否需要审核
        boolean flag=false;
        MthMatchInfo matchInfo = mthMatchInfoRepo.getOne(req.getMthMatchId());
       if (matchInfo.getRegisterationAuditFlag()==1){
           flag=true;
       }
        List<MatchTableInfoVo> registerInfoColumInfo = getRegisterInfoColumInfo(flag);
        roundColumVo.setTable(registerInfoColumInfo);
        List<String> collect = registerInfoColumInfo.stream().map(item -> item.getTableName()).collect(Collectors.toList());
        MatchRegisterFormVo matchRegisterFormVos = mthMatchManageDao.getregisterInfos(req.getMthMatchId());
        if (matchRegisterFormVos!=null) {
            List<MthMyMatchListDetailAppVo.Detail>configList = mthMatchManageDao.findConfigInfoBy(matchRegisterFormVos.getRegisterId()).stream().filter(
                    item->!collect.contains(item.getShowName())
            ).collect(Collectors.toList());
            roundColumVo.setMap(configList);
        }
        return roundColumVo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long copyMatch(Long matchId) {
        MthMatchInfo matchInfo = baseRepo.getOne(matchId);
        List<MthMatchItem> itemList = matchItemRepo.findByMatchId(matchId);
        List<MthMatchGroup> groupList = matchGroupRepo.findByMatchId(matchId);
        Map<Long, List<MthMatchGroup>> itemGroupListMap = StreamUtils.groupByKey(groupList, MthMatchGroup::getItemId);
        List<MthMatchLabel> labelList = labelRepo.findByMatchId(matchId);
        List<MthMatchSponsor> sponsorList = mthMatchSponsorRepo.findByMatchId(matchId);
        MthMatchInfo newMatch = new MthMatchInfo();
        BeanUtil.copyProperties(matchInfo, newMatch, "id");
        setDefaultProperty(newMatch);
        newMatch.setStatus(MatchInfoStatusEnums.DRAFT.getStatus());
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        newMatch.setCreateOrgId(Long.valueOf(currWorker.getOrgId()));
        newMatch.setCreateOrgName(currWorker.getOrgName());
        newMatch.setCreateDeptId(null);
        newMatch.setCreateDeptName(currWorker.getDepartment());
        newMatch.setPublishOrgId(null);
        newMatch.setPublishOrgName(null);
        newMatch.setPublisherId(null);
        newMatch.setPublisherName(null);
        newMatch.setPublishTime(null);
        newMatch.setMatchName(newMatch.getMatchName() + "-复制-" + IdUtil.objectId());
        baseRepo.saveAndFlush(newMatch);
        Long newMatchId = newMatch.getId();

        Map<Long, Long> itemIdMap = Maps.newHashMap();
        Map<Long, Long> groupIdMap = Maps.newHashMap();
        for (MthMatchItem item : itemList) {
            MthMatchItem newItem = BeanUtil.copyProperties(item, MthMatchItem.class);
            newItem.setId(IdUtil.getSnowflakeNextId());
            newItem.setMatchId(newMatchId);
            setDefaultProperty(newItem);
            matchItemRepo.saveAndFlush(newItem);
            itemIdMap.put(item.getId(), newItem.getId());
            List<MthMatchGroup> groups = itemGroupListMap.get(item.getId());
            if(ObjectUtil.isNotEmpty(groups)) {
                for (MthMatchGroup group : groups) {
                    MthMatchGroup newGroup = BeanUtil.copyProperties(group, MthMatchGroup.class);
                    newGroup.setId(IdUtil.getSnowflakeNextId());
                    newGroup.setItemId(newItem.getId());
                    newGroup.setMatchId(newMatchId);
                    setDefaultProperty(newGroup);
                    matchGroupRepo.saveAndFlush(newGroup);
                    groupIdMap.put(group.getId(), newGroup.getId());
                }
            }
        }

        for (MthMatchLabel label : labelList) {
            MthMatchLabel newLabel = BeanUtil.copyProperties(label, MthMatchLabel.class);
            newLabel.setId(null);
            newLabel.setMatchId(newMatchId);
            setDefaultProperty(newLabel);
            labelRepo.saveAndFlush(newLabel);
        }

        for (MthMatchSponsor sponsor : sponsorList) {
            MthMatchSponsor newSponsor = BeanUtil.copyProperties(sponsor, MthMatchSponsor.class);
            newSponsor.setId(null);
            newSponsor.setMatchId(newMatchId);
            setDefaultProperty(newSponsor);
            mthMatchSponsorRepo.saveAndFlush(newSponsor);
        }
        //单选复选多选
        List<String> optionalTypes = com.google.common.collect.Lists.newArrayList("single","checkbox","select");
        List<MthRegisterConfig> registerConfigs = registerConfigRepo.findByMatchId(matchId);

        for (MthRegisterConfig registerConfig : registerConfigs) {

            MthRegisterConfig newRegisterConfig = BeanUtil.copyProperties(registerConfig, MthRegisterConfig.class);
            newRegisterConfig.setId(null);
            setDefaultProperty(newRegisterConfig);
            newRegisterConfig.setMatchId(newMatchId);
            registerConfigRepo.saveAndFlush(newRegisterConfig);
            Map<Long, Long> formConfigIdMap = Maps.newHashMap();
            List<MthRegisterConfigDetail> details = configDetailRepo.findByRegisterInfoId(registerConfig.getId());

            if(ObjectUtil.isNotEmpty(details)) {
                List<Long> configIds = StreamUtils.toList(details, MthRegisterConfigDetail::getRegisterConfigId);
                List<MthFormConfig> formConfigs = formConfigRepo.findAllById(configIds);

                for (MthFormConfig formConfig : formConfigs) {
                    MthFormConfig newFormConfig = BeanUtil.copyProperties(formConfig, MthFormConfig.class);
                    newFormConfig.setId(null);
                    setDefaultProperty(newFormConfig);
                    formConfigRepo.saveAndFlush(newFormConfig);
                    formConfigIdMap.put(formConfig.getId(), newFormConfig.getId());
                    if (optionalTypes.contains(formConfig.getConfigType()) && !StrUtil.equals(formConfig.getRefType(), "dict")) {
                        List<MthFormConfigData> configDatas = formConfigDataRepo.findByConfigId(formConfig.getId());
                        for (MthFormConfigData configData : configDatas) {
                            MthFormConfigData newConfigData = BeanUtil.copyProperties(configData, MthFormConfigData.class);
                            newConfigData.setId(null);
                            setDefaultProperty(newConfigData);
                            newConfigData.setConfigId(newFormConfig.getId());
                            formConfigDataRepo.saveAndFlush(newConfigData);
                        }
                    }
                }
                for (MthRegisterConfigDetail detail : details) {
                    MthRegisterConfigDetail newDetail = BeanUtil.copyProperties(detail, MthRegisterConfigDetail.class);
                    newDetail.setId(null);
                    setDefaultProperty(newDetail);
                    newDetail.setRegisterInfoId(newRegisterConfig.getId());
                    newDetail.setRegisterConfigId(formConfigIdMap.get(detail.getRegisterConfigId()));
                    configDetailRepo.saveAndFlush(newDetail);
                }
            }

        }

        List<MthRoundInstance> roundTemplates = mthRoundInstanceRepo.findByMatchIdAndRoundType(matchId, RoundEnums.RoundType.TEMPLATE.getCode());
        for (MthRoundInstance roundTemplate : roundTemplates) {
            MthRoundInstance newRoundTemplate = BeanUtil.copyProperties(roundTemplate, MthRoundInstance.class);
            newRoundTemplate.setId(null);
            newRoundTemplate.setMatchId(newMatchId);
            setDefaultProperty(newRoundTemplate);
            mthRoundInstanceRepo.saveAndFlush(newRoundTemplate);
            List<MthRoundReviewStandar> standars = standarRepo.findAllByRoundId(roundTemplate.getId());
            for (MthRoundReviewStandar standar : standars) {
                MthRoundReviewStandar newStandar = BeanUtil.copyProperties(standar, MthRoundReviewStandar.class);
                newStandar.setId(null);
                newStandar.setRoundId(newRoundTemplate.getId());
                setDefaultProperty(newStandar);
                newStandar.setItemId(itemIdMap.get(standar.getItemId()));
                standarRepo.saveAndFlush(newStandar);
            }
            List<MthRoundArea> roundAreas = roundAreaRepo.findByRoundId(roundTemplate.getId());
            for (MthRoundArea roundArea : roundAreas) {
                MthRoundArea newRoundArea = BeanUtil.copyProperties(roundArea, MthRoundArea.class);
                newRoundArea.setId(null);
                newRoundArea.setRoundId(newRoundTemplate.getId());
                newRoundArea.setParentId(null);
                setDefaultProperty(newRoundArea);
                roundAreaRepo.saveAndFlush(newRoundArea);
            }
            List<MthRoundPrize> prizes = roundPrizeRepo.findByRoundId(roundTemplate.getId());
            for (MthRoundPrize prize : prizes) {
                MthRoundPrize newPrize = BeanUtil.copyProperties(prize, MthRoundPrize.class);
                newPrize.setId(null);
                newPrize.setRoundId(newRoundTemplate.getId());
                setDefaultProperty(newPrize);
                newPrize.setItemId(itemIdMap.get(prize.getItemId()));
                newPrize.setGroupId(groupIdMap.get(prize.getGroupId()));
                roundPrizeRepo.saveAndFlush(newPrize);
            }
        }
        return newMatchId;
    }

    private static void setDefaultProperty(BaseEntity entity) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        entity.setCreateId(Integer.valueOf(currWorker.getLoginId()).longValue());
        entity.setCreateBy(currWorker.getWorkerName());
        entity.setCreateTime(new Date());
        entity.setUpdateId(Integer.valueOf(currWorker.getLoginId()).longValue());
        entity.setUpdateBy(currWorker.getWorkerName());
        entity.setUpdateTime(new Date());
    }
}
