package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.common.utils.NumberUtil;
import com.crunii.micro.common.utils.ValidatorUtils;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.converter.MthProjectFinancingConverter;
import com.crunii.micro.service.match.enums.FinancingEnums;
import com.crunii.micro.service.match.enums.SourceTypeEnums;
import com.crunii.micro.service.match.jdbc.MthProjectFinancingDao;
import com.crunii.micro.service.match.service.IMthProjectFinancingService;
import com.crunii.micro.service.match.service.IObsFilService;
import com.crunii.micro.service.match.service.PushBusinessMsgService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static com.crunii.micro.service.match.constants.MatchMessageConstants.*;

/**
 * 青创赛事项目融资申请Service业务层处理
 *
 * @author ruoyi
 * @date 2023-12-18
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class MthProjectFinancingServiceImpl implements IMthProjectFinancingService {
    private static String format="yyyyMMddHHmmss";
    private final MthProjectFinancingRepo mthProjectFinancingRepo;
    private final MthProjectFinancingConverter baseConverter;
    private final MthProjectFinancingDao mthProjectFinancingDao;
    private final MthProjectFinancingHistoryRepo historyRepo;
    private final MthProjectFinancingMemberRepo memberRepo;
    private final IObsFilService obsFilService;
    private final MthProjectRepo mthProjectRepo;
    private final MthRoundScoreRepo mthRoundScoreRepo;
    private final PushBusinessMsgService pushBusinessMsgService;
    private final TsmWorkerRepo tsmWorkerRepo;
    private final MthProjectFinancingContactRepo contactRepo;

    /**
     * 查询青创赛事项目融资申请
     */
    @Override
    public MthProjectFinancingDetailsVo queryById(Long id){
        if (id==null){
            throw new BusinessException(REQUEST_ERROR);
        }
        MthProjectFinancing mthProjectFinancing = mthProjectFinancingRepo.getById(id);
        if (ObjectUtil.isEmpty(mthProjectFinancing) && mthProjectFinancing.getDelFlag()==1){
            throw new BusinessException(PROJECT_FINANCING_NOT_EXISTS);
        }
        MthProjectFinancingDetailsVo detailsVo=new MthProjectFinancingDetailsVo();
        MthProjectFinancingBaselVo baselVo = mthProjectFinancingDao.queryById(id);
        if (baselVo==null){
            return new MthProjectFinancingDetailsVo();
        }
        baselVo.setTransferEquity(baselVo.getTransferEquity()==null? BigDecimal.valueOf(0.00) :baselVo.getTransferEquity().divide(BigDecimal.valueOf(100)));
        BeanUtil.copyProperties(baselVo,detailsVo);
        if (ObjectUtil.isEmpty(detailsVo) || detailsVo.getFinancingId()==null){
            return null;
        }
        //处理项目展示
       if (StringUtils.isNotEmpty(detailsVo.getShowImages())){
           obsFilService.setObsFileBoListById(detailsVo, MthProjectFinancingDetailsVo::getShowImages, MthProjectFinancingDetailsVo::setShowImageList);
       }
       //项目logo
        if (detailsVo.getLogo()!=null){
            obsFilService.setObsFileBoById(detailsVo, MthProjectFinancingDetailsVo::getLogo, MthProjectFinancingDetailsVo::setLogoFile);
        }
        //查询融资历史
        List<MthProjectFinancingHistory> historyList = historyRepo.findByFinancingId(detailsVo.getFinancingId());
        List<MthProjectFinancingDetailsVo.FinancingHistory> collect = historyList.stream().map(l -> {
            MthProjectFinancingDetailsVo.FinancingHistory financingHistory = new MthProjectFinancingDetailsVo.FinancingHistory();
            financingHistory.setId(l.getId());
            financingHistory.setFinancingTime(l.getFinancingTime());
            financingHistory.setFinancingRounds(l.getFinancingRounds());
            financingHistory.setAmount(l.getAmount());
            financingHistory.setInvestor(l.getInvestor());
            return financingHistory;
        }).collect(Collectors.toList());
        detailsVo.setFinancingHistory(collect);
        //查询核心成员
        List<MthProjectFinancingMember> members = memberRepo.findByFinancingId(detailsVo.getFinancingId());
        List<MthProjectFinancingDetailsVo.TeamMembers> membersList = members.stream().map(meber -> {
            MthProjectFinancingDetailsVo.TeamMembers teamMembers = new MthProjectFinancingDetailsVo.TeamMembers();
            BeanUtil.copyProperties(meber,teamMembers);
            return teamMembers;
        }).collect(Collectors.toList());
        //成员头像
        obsFilService.setObsFileBoByIds(membersList, MthProjectFinancingDetailsVo.TeamMembers::getHeadSculpture, MthProjectFinancingDetailsVo.TeamMembers::setHeadSculptureFlie);
        detailsVo.setTeamMembers(membersList);
        return detailsVo;
    }

    /**
     * 查询青创赛事项目融资申请列表
     */
    @Override
    public PageRsp<MthProjectFinancingPageVo> queryPageList(PageReq<PageProjectFinancingBo> bo) {
        PageRsp<MthProjectFinancingPageVo> result = mthProjectFinancingDao.queryPageList(bo);
        for (MthProjectFinancingPageVo mthProjectFinancingPageVo : result.getResult()) {
            if (SourceTypeEnums.SUBMIT.getType().equals(mthProjectFinancingPageVo.getSource())){
                StringBuilder prizeName=new StringBuilder();
                List<MthRoundScore> allPrizeName = mthRoundScoreRepo.getAllPrizeName(mthProjectFinancingPageVo.getRegisterId());
                for (MthRoundScore mthRoundScore : allPrizeName) {
                    //奖项
                    if (StringUtils.isNotEmpty(mthRoundScore.getPrizeName())){
                        prizeName.append(prizeName.length()>0?","+mthRoundScore.getPrizeName():mthRoundScore.getPrizeName());
                    }
                    //其他奖项
                    if (StringUtils.isNotEmpty(mthRoundScore.getOtherPrizeName())){
                        prizeName.append(prizeName.length()>0?","+mthRoundScore.getOtherPrizeName():mthRoundScore.getOtherPrizeName());
                    }
                }
                mthProjectFinancingPageVo.setPrizeName(prizeName.toString());
            }
        }
        return result;
    }

    /**
     * 青创赛事项目融资申请列表-审核
     * @param bo
     * @return
     */
    @Override
    public Boolean doAudit(ProjectFinancingAuditBo bo,Worker worker) {
        if (ObjectUtil.isEmpty(bo) || bo.getId()==null || bo.getReviewStatus()==null){
            throw new BusinessException(REQUEST_ERROR);
        }
        //校验状态是否正常
        if (!(bo.getReviewStatus()==1 || bo.getReviewStatus()==2)){
            throw new BusinessException(PROJECT_FINANCING_STATUS_ERROR);
        }
        //审核不通过时，原因必填
        if (bo.getReviewStatus()==2 && StringUtils.isEmpty(bo.getReviewReason())){
            throw new BusinessException(PROJECT_FINANCING_AUDIT_REASON);
        }
        //审核原因不能超过200字符
        if (StringUtils.isNotEmpty(bo.getReviewReason()) && bo.getReviewReason().length()>200){
            throw new BusinessException(PROJECT_FINANCING_AUDIT_REASON_OUT);
        }
        //查询融资申请是否存在
        MthProjectFinancing mthProjectFinancing = mthProjectFinancingRepo.getOne(bo.getId());
        if (ObjectUtil.isEmpty(mthProjectFinancing) || 1==mthProjectFinancing.getDelFlag()){
            throw new BusinessException(PROJECT_FINANCING_NOT_EXISTS);
        }
        //判断是否需要审核
        if (!(FinancingEnums.ReviewStatus.PENDING.getCode().equals(mthProjectFinancing.getReviewStatus()))){
            throw new BusinessException(PROJECT_FINANCING_UNNEEDED_AUDIT);
        }
        //是否已撤销
        if (FinancingEnums.ReviewStatus.CANCEL.getCode().equals(mthProjectFinancing.getReviewStatus())){
            throw new BusinessException(PROJECT_FINANCING_REVOCATION);
        }
        String stateName="不通过";
        //审核通过时,将公示状态改为已公示
        if (bo.getReviewStatus()==1){
            stateName="已通过";
            mthProjectFinancing.setPublicityStatus(1);
        }
        mthProjectFinancing.setReviewStatus(bo.getReviewStatus());
        mthProjectFinancing.setReviewReason(bo.getReviewReason());
        mthProjectFinancing.setReviewTime(new Date());
        mthProjectFinancing.setReviewerId((long)worker.getLoginId());
        mthProjectFinancing.setReviewerName(worker.getWorkerName());
        mthProjectFinancingRepo.saveAndFlush(mthProjectFinancing);
        //发送审核消息
        pushBusinessMsgService.pushResultMsg(mthProjectFinancing.getUserId(),stateName);
        return true;
    }

    /**
     * 青创赛事项目融资申请列表-修改公示状态
     * @param bo
     * @return
     */
    @Override
    public Boolean doPublicity(ProjectFinancingAuditBo bo) {
        if (ObjectUtil.isEmpty(bo) || bo.getId()==null || bo.getPublicityStatus()==null){
            throw new BusinessException(REQUEST_ERROR);
        }
        //查询融资申请
        MthProjectFinancing mthProjectFinancing = mthProjectFinancingRepo.getOne(bo.getId());
        if (ObjectUtil.isEmpty(mthProjectFinancing) || 1==mthProjectFinancing.getDelFlag()){
            throw new BusinessException(PROJECT_FINANCING_NOT_EXISTS);
        }
        //融资申请是否通过
        if (mthProjectFinancing.getReviewStatus()==null|| mthProjectFinancing.getReviewStatus()!=1){
            throw new BusinessException(PROJECT_FINANCING_NOT_PASS);
        }
        mthProjectFinancing.setPublicityStatus(bo.getPublicityStatus());
        mthProjectFinancingRepo.saveAndFlush(mthProjectFinancing);
        return true;
    }

    /**
     * app-融资项目-列表
     * @param bo
     * @return
     */
    @Override
    public PageRsp<MthProjectAppVo> pageProject(PageReq<MthProjectAppBo> bo) {
        PageRsp<MthProjectAppVo> mthProjectAppVoPageRsp = mthProjectFinancingDao.pageProject(bo);
        List<MthProjectAppVo> result = mthProjectAppVoPageRsp.getResult();
        obsFilService.setObsFileBoByIds(result, MthProjectAppVo::getLogo, MthProjectAppVo::setLogoFile);
        for (MthProjectAppVo mthProjectAppVo : result) {
            StringBuilder allPrizeName=new StringBuilder();
            List<MthRoundScore> prizeName = mthRoundScoreRepo.getAllPrizeName(mthProjectAppVo.getRegisterId());
            prizeName.forEach(one->allPrizeName.append(allPrizeName.length()>0 && StringUtils.isNotEmpty(one.getPrizeName())?","+one.getPrizeName():StringUtils.isNotEmpty(one.getPrizeName())?one.getPrizeName():""));
            mthProjectAppVo.setPrizeName(allPrizeName.toString());
            }
        return mthProjectAppVoPageRsp;
    }

    /**
     * app-查询融资项目库-详情
     * @param id
     * @return
     */
    @Override
    public MthProjectDetailsVo getProjectDetails(Long id) {
        MthProjectDetailsVo mthProjectDetailsVo=new MthProjectDetailsVo();
        MthProjectFinancing mthProjectFinancing = mthProjectFinancingRepo.getOne(id);
        if (ObjectUtil.isEmpty(mthProjectFinancing) ||mthProjectFinancing.getDelFlag()==1){
            throw new BusinessException(PROJECT_FINANCING_NOT_EXISTS);
        }
        BeanUtil.copyProperties(mthProjectFinancing,mthProjectDetailsVo);
        //项目信息
        MthProject mthProject = mthProjectRepo.getOne(mthProjectFinancing.getProjectId());
        if (ObjectUtil.isEmpty(mthProject) ||mthProject.getDelFlag()==1){
            throw new BusinessException(PROJECT_PROJECT_NOT_EXISTS);
        }
        BeanUtil.copyProperties(mthProject,mthProjectDetailsVo,"id");
        //项目logo
        if (mthProjectDetailsVo.getLogo()!=null){
            obsFilService.setObsFileBoById(mthProjectDetailsVo, MthProjectDetailsVo::getLogo, MthProjectDetailsVo::setLogoFile);
        }
        //查询融资历史
        List<MthProjectFinancingHistory> historyList = historyRepo.findByFinancingId(mthProjectFinancing.getId());
        List<MthProjectDetailsVo.FinancingHistory> collect = historyList.stream().map(l -> {
            MthProjectDetailsVo.FinancingHistory financingHistory = new MthProjectDetailsVo.FinancingHistory();
            financingHistory.setId(l.getId());
            financingHistory.setFinancingTime(l.getFinancingTime());
            financingHistory.setFinancingRounds(l.getFinancingRounds());
            String format=null;
            if (l.getAmount()!=null){
                DecimalFormat decimalFormat2 = new DecimalFormat("0.00####");
               format = decimalFormat2.format(l.getAmount());
            }
            financingHistory.setAmount(l.getPublicityAmount()==1?format:"****");
            financingHistory.setInvestor(l.getPublicityInvestor()==1?l.getInvestor():"****");
            return financingHistory;
        }).collect(Collectors.toList());
        mthProjectDetailsVo.setFinancingHistory(collect);
        //查询核心成员
        List<MthProjectFinancingMember> members = memberRepo.findByFinancingId(mthProjectFinancing.getId());
        List<MthProjectDetailsVo.TeamMembers> membersList = members.stream().map(meber -> {
            MthProjectDetailsVo.TeamMembers teamMembers = new MthProjectDetailsVo.TeamMembers();
            BeanUtil.copyProperties(meber,teamMembers);
            teamMembers.setMemberName(meber.getName());
            return teamMembers;
        }).collect(Collectors.toList());
        obsFilService.setObsFileBoByIds(membersList, MthProjectDetailsVo.TeamMembers::getHeadSculpture, MthProjectDetailsVo.TeamMembers::setHeadSculptureFlie);
        mthProjectDetailsVo.setAllMembers(membersList);
        return mthProjectDetailsVo;
    }

    @Override
    public PageRsp<MthMyProjectApplyAppVo> pageMyProjectApply(PageReq<MthMyProjectApplyAppBo> bo) {
        int loginId = CurrWorkerUtil.getCurrWorker().getLoginId();
        if(bo.getQuery() == null){
            bo.setQuery(new MthMyProjectApplyAppBo());
        }
        bo.getQuery().setUserId(Long.valueOf(loginId));
        PageRsp<MthMyProjectApplyAppVo> page = mthProjectFinancingDao.pageMyProjectApply(bo);
        if(ObjectUtil.isNotEmpty(page.getResult())){
            for (MthMyProjectApplyAppVo item : page.getResult()) {
                item.setReviewStatusName(FinancingEnums.ReviewStatus.getDesc(item.getReviewStatus()));
                item.setPublicityStatusName(FinancingEnums.PublicityStatus.getDesc(item.getPublicityStatus()));
            }
        }
        return page;
    }


    /**
     * app-查询融资项目库-详情
     * @param id
     * @return
     */
    @Override
    public MthMyProjectApplyDetailsVo getMyProjectApplyDetails(Long id) {
        MthMyProjectApplyDetailsVo detailVo = new MthMyProjectApplyDetailsVo();
        MthMyProjectApplyDetailsVo.RegisterInfo registerInfo = new MthMyProjectApplyDetailsVo.RegisterInfo();
        MthMyProjectApplyDetailsVo.FinancingInfo financingInfo = new MthMyProjectApplyDetailsVo.FinancingInfo();

        MthMyProjectApplyAppVo projectApplyDetail = mthProjectFinancingDao.getProjectApplyDetail(id);
        projectApplyDetail.setReviewStatusName(FinancingEnums.ReviewStatus.getDesc(projectApplyDetail.getReviewStatus()));
        projectApplyDetail.setPublicityStatusName(FinancingEnums.PublicityStatus.getDesc(projectApplyDetail.getPublicityStatus()));
        detailVo.setReviewInfo(projectApplyDetail);

        MthProjectFinancing mthProjectFinancing = mthProjectFinancingRepo.getOne(id);
        if (ObjectUtil.isEmpty(mthProjectFinancing) ||mthProjectFinancing.getDelFlag()==1){
            throw new BusinessException(PROJECT_FINANCING_NOT_EXISTS);
        }
        BeanUtil.copyProperties(mthProjectFinancing, financingInfo);
        //项目信息
        MthProject mthProject = mthProjectRepo.getOne(mthProjectFinancing.getProjectId());
        if (ObjectUtil.isEmpty(mthProject) ||mthProject.getDelFlag()==1){
            throw new BusinessException(PROJECT_PROJECT_NOT_EXISTS);
        }
        BeanUtil.copyProperties(mthProject, registerInfo, "id");
        obsFilService.setObsFileBoById(registerInfo, MthMyProjectApplyDetailsVo.RegisterInfo::getLogo, MthMyProjectApplyDetailsVo.RegisterInfo::setLogoFile);
        obsFilService.setObsFileBoListById(registerInfo, MthMyProjectApplyDetailsVo.RegisterInfo::getShowImages, MthMyProjectApplyDetailsVo.RegisterInfo::setShowImagesFileList);
        detailVo.setRegisterInfo(registerInfo);
        //查询融资历史
        List<MthProjectFinancingHistory> historyList = historyRepo.findByFinancingId(mthProjectFinancing.getId());
        List<MthMyProjectApplyDetailsVo.FinancingInfo.FinancingHistory> financingHistoryList = historyList.stream().map(l -> {
            MthMyProjectApplyDetailsVo.FinancingInfo.FinancingHistory financingHistory = new MthMyProjectApplyDetailsVo.FinancingInfo.FinancingHistory();
            financingHistory.setId(l.getId());
            financingHistory.setFinancingTime(l.getFinancingTime());
            financingHistory.setFinancingRounds(l.getFinancingRounds());
            financingHistory.setAmount(l.getAmount());
            financingHistory.setInvestor(l.getInvestor());
            return financingHistory;
        }).collect(Collectors.toList());
        financingInfo.setFinancingHistory(financingHistoryList);
        //查询核心成员
        List<MthProjectFinancingMember> members = memberRepo.findByFinancingId(mthProjectFinancing.getId());
        List<MthMyProjectApplyDetailsVo.FinancingInfo.TeamMembers> membersList = members.stream().map(meber -> {
            MthMyProjectApplyDetailsVo.FinancingInfo.TeamMembers teamMembers = new MthMyProjectApplyDetailsVo.FinancingInfo.TeamMembers();
            BeanUtil.copyProperties(meber,teamMembers);
            teamMembers.setMemberName(meber.getName());
            return teamMembers;
        }).collect(Collectors.toList());
        obsFilService.setObsFileBoByIds(membersList, MthMyProjectApplyDetailsVo.FinancingInfo.TeamMembers::getHeadSculpture, MthMyProjectApplyDetailsVo.FinancingInfo.TeamMembers::setHeadSculptureFlie);
        financingInfo.setAllMembers(membersList);
        detailVo.setFinancingInfo(financingInfo);
        return detailVo;
    }

    /**
     * web-新增融资申请
     */
    @Override
    public Boolean createProject(FinancingBo financingBo, Worker worker) {
        // 校验字段要求
        validationThrowException(financingBo);
        //根据手机号查询用户
        TsmWorker tsmWorker=findWorker(financingBo.getParticipantMobil());
        //新增/修改 融资项目
        MthProject mthProject = new MthProject();
        BeanUtil.copyProperties(financingBo,mthProject,"id");
        mthProject.setTeamMembers(financingBo.getTeamTotal());
        mthProject.setGroupName(financingBo.getMatchGroup());
        mthProject.setItemName(financingBo.getMatchItem());
        mthProjectRepo.saveAndFlush(mthProject);
        // 新增 融资申请
        MthProjectFinancing financing=new MthProjectFinancing();
        BeanUtil.copyProperties(financingBo,financing);
        financing.setProjectId(mthProject.getId());
        financing.setUserId((long)tsmWorker.getLoginId());
        financing.setCode(StringUtils.isNotEmpty(financingBo.getCode()) ? financingBo.getCode() : "QCSS" + DateUtil.format(new Date(), format) + RandomUtil.randomStringUpper(4));
        financing.setReviewStatus(FinancingEnums.ReviewStatus.PENDING.getCode());
        financing.setPublicityStatus(FinancingEnums.PublicityStatus.UNPUBLISHED.getCode());
        financing.setSource(SourceTypeEnums.SUBMIT.getType());
        mthProjectFinancingRepo.saveAndFlush(financing);
        historyRepo.deleteByFinancingId(financing.getId());
        memberRepo.deleteByFinancingId(financing.getId());
        //融资历史
        if (ObjectUtil.isNotEmpty(financingBo.getHistoryList())) {
            financingBo.getHistoryList().forEach(MthProjectFinancingServiceImpl::validationThrowException);
            List<FinancingBo.History> historyListBo = financingBo.getHistoryList();
            List<MthProjectFinancingHistory> historyList = historyListBo.stream().map(item -> {
                MthProjectFinancingHistory history = new MthProjectFinancingHistory();
                BeanUtil.copyProperties(item, history);
                history.setFinancingId(financing.getId());
                return history;
            }).collect(Collectors.toList());
            historyRepo.saveAllAndFlush(historyList);
        }
        //团队成员
        if (ObjectUtil.isNotEmpty(financingBo.getMemberList())) {
            financingBo.getMemberList().forEach(MthProjectFinancingServiceImpl::validationThrowException);
            List<FinancingBo.Member> memberListBo = financingBo.getMemberList();
            List<MthProjectFinancingMember> memberList = memberListBo.stream().map(item -> {
                MthProjectFinancingMember member = new MthProjectFinancingMember();
                BeanUtil.copyProperties(item, member);
                member.setFinancingId(financing.getId());
                return member;
            }).collect(Collectors.toList());
            memberRepo.saveAllAndFlush(memberList);
        }
        return true;
    }

    /**
     * app-青创赛事-填写联系人信息
     * @param bo
     * @param worker
     * @return
     */
    @Override
    public Boolean createFinancingContact(ContactBo bo, Worker worker) {
        validInformation(bo);
        MthProjectFinancingContact mthProjectFinancingContact=new MthProjectFinancingContact();
        BeanUtil.copyProperties(bo,mthProjectFinancingContact);
        contactRepo.saveAndFlush(mthProjectFinancingContact);
        return true;
    }

    /**
     * 获取项目联系人信息
     * @param worker
     * @return
     */
    @Override
    public PageRsp<ContactVo> getMyProjectContact(PageReq<ContactBo> req,Worker worker) {
       return  mthProjectFinancingDao.getContact(req,worker);
    }

    /**
     * 校验填写信息
     * @param bo
     */
    private void validInformation(ContactBo bo) {
        if (ObjectUtil.isEmpty(bo)|| bo.getFinancingId()==null){
            throw new BusinessException(REQUEST_ERROR);
        }
        //检验必填项是否已填
        if (StringUtils.isEmpty(bo.getMobile())||StringUtils.isEmpty(bo.getPostName()) || StringUtils.isEmpty(bo.getUserName())){
            throw new BusinessException(PROJECT_FINANCING_INFORMATION_NULL);
        }
        //校验手机号格式
        if (!NumberUtil.isMobile(bo.getMobile())){
            throw new BusinessException(PROJECT_FINANCING_MOBILE_ERROR);
        }
    }

    /**
     * 根据手机号查询用户
     * @param participantMobil
     * @return
     */
    private TsmWorker findWorker(String participantMobil) {
        if (StringUtils.isEmpty(participantMobil)){
            throw new BusinessException(PROJECT_FINANCING_JOIN_MOBILE_NULL);
        }
        List<TsmWorker> tsmWorker = tsmWorkerRepo.findAllByMobile(participantMobil);
        if (tsmWorker==null || tsmWorker.size()==0){
            throw new BusinessException(PROJECT_FINANCING_JOIN_USER_NOT_EXISTS);
        }
        if (tsmWorker.size()>1){
            throw new BusinessException(PROJECT_FINANCING_USER_MORE);
        }
        return tsmWorker.get(0);
    }

    private static void validationThrowException(Object bean) {
        ValidatorUtils.validateMsgException(bean, AddGroup.class);
    }
}
