package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.BeanUtil;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.feign.client.SysService;
import com.crunii.micro.service.match.api.FileDto;
import com.crunii.micro.service.match.api.bo.MthRegisterConfigBo;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.converter.*;
import com.crunii.micro.service.match.service.IMthRegisterConfigService;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 赛事-报名信息;Service业务层处理
 *
 * @author mth
 * @date 2023-09-19
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class MthRegisterConfigServiceImpl implements IMthRegisterConfigService {

    private final MthRegisterConfigRepo baseRepo;
    private final MthRegisterConfigConverter baseConverter;

    private final MthRegisterConfigDetailConverter registerConfigDetailConverter;
    private final MthRegisterConfigDetailRepo registerConfigDetailRepo;

    private final MthFormConfigRepo formConfigRepo;
    private final MthFormConfigConverter formConfigConverter;

    private final MthFormConfigDataRepo formConfigDataRepo;
    private final MthFormConfigDataConverter formConfigDataConverter;

    @Value("${upload.url}")
    private String urlPrefix;

    private final YcqObsFileRepo fileRepo;

    private final FileConverter fileConverter;

    private final SysService sysService;

    /**
     * 查询赛事-报名信息;
     */
    @Override
    public MthRegisterConfigVo queryById(Long id){
        return baseConverter.entityToVo(baseRepo.getById(id));
    }

    /**
     * 查询赛事-报名信息;列表
     */
    @Override
    public Result<PageRsp<MthRegisterConfigVo>> queryPageList(PageReq<MthRegisterConfigBo> bo) {
        return null;
    }

    /**
     * 查询赛事-报名信息;列表
     */
    @Override
    public List<MthRegisterConfigVo> queryList(MthRegisterConfigBo bo) {
        return null;
    }

    /**
     * 新增赛事-报名信息;
     */
    @Override
    public Boolean insertByBo(MthRegisterConfigBo bo) {
        MthRegisterConfig add = baseConverter.boToEntity(bo);
        validEntityBeforeSave(add);
        baseRepo.save(add);
        return true;
    }

    /**
     * 修改赛事-报名信息;
     */
    @Override
    public Boolean updateByBo(MthRegisterConfigBo bo) {
        MthRegisterConfig update = baseConverter.boToEntity(bo);
        validEntityBeforeSave(update);
        return null;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(MthRegisterConfig entity){

    }

    /**
     * 批量删除赛事-报名信息;
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){

        }
        return null;
    }

    /**
     * @description: 创建报名配置表
     * @param bo
     * @return: 
     * @author liuchuan
     * @date: 2023/9/25 17:05
    */
    @Override
    public boolean createMatchConfig(MthRegisterConfigBo bo) {
        MthRegisterConfig add = baseConverter.boToEntity(bo);
        if (ObjectUtil.isNotNull(bo.getFileDto())&&ObjectUtil.isNotNull(bo.getFileDto().getFileId())){
            add.setPicId(bo.getFileDto().getFileId());
        }
        validEntityBeforeSave(add);
        baseRepo.save(add);
        List<MthRegisterConfigDetail> configDetailList= Lists.newArrayList();
        if (CollectionUtil.isNotEmpty(bo.getConfigDetailList())){
            bo.getConfigDetailList().stream().forEach(detailBo->{
                //插入动态配置
                MthFormConfig formConfig = new MthFormConfig();
                formConfig.setDynamicFlag(1);
                formConfig.setHint(detailBo.getHint());
                formConfig.setRequiredFlag(detailBo.getRequiredFlag());
                formConfig.setShowName(detailBo.getShowName());
                formConfig.setConfigType(detailBo.getConfigType());
                formConfig.setMaxFileSize(detailBo.getMaxFileSize());
                formConfig.setMinFileSize(detailBo.getMinFileSize());
                formConfig.setLimitNum(detailBo.getLimitNum());
                formConfig.setRefType(detailBo.getRefType());
                formConfig.setRefId(detailBo.getRefId());
                formConfig.setKey(detailBo.getKey());
                formConfigRepo.save(formConfig);
                if (CollectionUtil.isNotEmpty(detailBo.getValueList())){
                    //插入动态配置的选项数据
                    List<MthFormConfigData> formConfigDataList= Lists.newArrayList();
                    detailBo.getValueList().stream().forEach(value->{
                        MthFormConfigData formConfigData = new MthFormConfigData();
                        formConfigData.setConfigId(formConfig.getId());
                        formConfigData.setConfigValue(value.getName());
                        formConfigDataList.add(formConfigData);
                    });
                    formConfigDataRepo.saveAll(formConfigDataList);
                }
                detailBo.setRegisterConfigId(formConfig.getId());
                MthRegisterConfigDetail detail = registerConfigDetailConverter.boToEntity(detailBo);
                detail.setRegisterInfoId(add.getId());
                configDetailList.add(detail);
            });
        }
        if (CollectionUtil.isNotEmpty(configDetailList)){
            registerConfigDetailRepo.saveAllAndFlush(configDetailList);
        }


        return true;
    }

    /**
     * @description: 修改配置信息
     * @param bo
     * @return:
     * @author liuchuan
     * @date: 2023/9/25 17:15
    */
    @Override
    public boolean updateMatchConfit(MthRegisterConfigBo bo) {
        MthRegisterConfig registerConfig = baseRepo.getOne(bo.getId());
        if (ObjectUtil.isNotNull(bo.getFileDto())&&ObjectUtil.isNotNull(bo.getFileDto().getFileId())){
            registerConfig.setPicId(bo.getFileDto().getFileId());
        }
        BeanUtil.copyPropertiesIgnoreNull(bo,registerConfig);
        baseRepo.saveAndFlush(registerConfig);
        //删除之前的
        registerConfigDetailRepo.deleteByRegisterInfoId(bo.getId());
        List<MthRegisterConfigDetail> configDetailList= Lists.newArrayList();
        if (CollectionUtil.isNotEmpty(bo.getConfigDetailList())){
            bo.getConfigDetailList().stream().forEach(detailBo->{
                //插入动态配置
                MthFormConfig formConfig = new MthFormConfig();
                formConfig.setDynamicFlag(1);
                formConfig.setHint(detailBo.getHint());
                formConfig.setRequiredFlag(detailBo.getRequiredFlag());
                formConfig.setShowName(detailBo.getShowName());
                formConfig.setConfigType(detailBo.getConfigType());
                formConfig.setMaxFileSize(detailBo.getMaxFileSize());
                formConfig.setMinFileSize(detailBo.getMinFileSize());
                formConfig.setLimitNum(detailBo.getLimitNum());
                formConfig.setRefType(detailBo.getRefType());
                formConfig.setRefId(detailBo.getRefId());
                formConfig.setKey(detailBo.getKey());
                formConfigRepo.save(formConfig);
                if (CollectionUtil.isNotEmpty(detailBo.getValueList())){
                    //插入动态配置的选项数据
                    List<MthFormConfigData> formConfigDataList=Lists.newArrayList();
                    detailBo.getValueList().stream().forEach(value->{
                        MthFormConfigData formConfigData = new MthFormConfigData();
                        formConfigData.setConfigId(formConfig.getId());
                        formConfigData.setConfigValue(value.getName());
                        formConfigDataList.add(formConfigData);
                    });
                    formConfigDataRepo.saveAll(formConfigDataList);
                }
                detailBo.setRegisterConfigId(formConfig.getId());
                MthRegisterConfigDetail detail = registerConfigDetailConverter.boToEntity(detailBo);
                detail.setRegisterInfoId(bo.getId());
                configDetailList.add(detail);
            });
        }
        if (CollectionUtil.isNotEmpty(configDetailList)){
            registerConfigDetailRepo.saveAllAndFlush(configDetailList);
        }

        return true;
    }


    @Override
    public MthRegisterConfigVo queryConfigById(Long configId) {
        MthRegisterConfig registerConfig = baseRepo.getOne(configId);
        MthRegisterConfigVo vo = new MthRegisterConfigVo();
        BeanUtil.copyProperties(registerConfig,vo);
        List<MthRegisterConfigDetailVo> voList =Lists.newArrayList();
        if (ObjectUtil.isNotNull(registerConfig)){

            List<Map<String,Object>> configList = registerConfigDetailRepo.findConfigInfoBy(configId);
            if (CollectionUtil.isNotEmpty(configList)){
                configList.stream().forEach(map->{

                    MthRegisterConfigDetailVo detailVo = cn.hutool.core.bean.BeanUtil.mapToBean(map, MthRegisterConfigDetailVo.class,true, CopyOptions.create().ignoreCase());

                            if ("single".equals(detailVo.getConfigType())
                                    ||"checkbox".equals(detailVo.getConfigType())
                                    ||"radio".equals(detailVo.getConfigType())
                                    ||"select".equals(detailVo.getConfigType())) {
                                List<MthFormConfigData> dataList = formConfigDataRepo.findByConfigIdAndDelFlag(detailVo.getRegisterConfigId(), 0);
                                List<MthFormConfigValueVo> valueVoList = Lists.newArrayList();
                                dataList.stream().forEach(data -> {
                                    MthFormConfigValueVo valueVo = new MthFormConfigValueVo();
                                    valueVo.setName(data.getConfigValue());
                                    valueVoList.add(valueVo);
                                });
                                detailVo.setValueList(valueVoList);
                            }
                    voList.add(detailVo);


                });
            }
            vo.setFormList(voList);
        }
        //图片
        if (ObjectUtil.isNotNull(registerConfig.getPicId())){
            YcqObsFile file = fileRepo.findByIdAndState(registerConfig.getPicId(), 1);
            if (ObjectUtil.isNotNull(file)){
                FileDto fileDto = fileConverter.ObsFileToFileDto(file);
                fileDto.setFileUrl(urlPrefix+fileDto.getFileKey());
                vo.setFileDto(fileDto);
            }
        }
        return vo;
    }



    @Override
    public List<MthFormConfigVo> queryFormConfig() {
        List<MthFormConfig> list =  formConfigRepo.findNotDynamic();
        List<MthFormConfigVo> voList = formConfigConverter.entityListToVoList(list);
        voList.stream().forEach(mthFormConfigVo -> {
            //单选复选多选 将值给前端
            List<String> optionalTypes = com.google.common.collect.Lists.newArrayList("single","checkbox","select");
            if (optionalTypes.contains(mthFormConfigVo.getConfigType())){
                if (StrUtil.equals(mthFormConfigVo.getRefType(), "dict")) {
                    // 数据字典静态值
                    Result<List<TsmPropertyValue>> dataListResult = sysService.getValuesByCode(mthFormConfigVo.getRefId());
                    if (dataListResult.isSuccess() && ObjectUtil.isNotEmpty(dataListResult.getData())) {
                        List<MthFormConfigValueVo> valueVoList = dataListResult.getData().stream()
                                .map(data -> new MthFormConfigValueVo(data.getValueName()))
                                .collect(Collectors.toList());
                        mthFormConfigVo.setValueList(valueVoList);
                    }
                } else {
                    List<MthFormConfigData> dataList = formConfigDataRepo.findByConfigIdAndDelFlag(mthFormConfigVo.getId(), 0);
                    List<MthFormConfigValueVo> valueVoList = Lists.newArrayList();
                    dataList.stream().forEach(data -> {
                        MthFormConfigValueVo valueVo = new MthFormConfigValueVo();
                        valueVo.setName(data.getConfigValue());
                        valueVoList.add(valueVo);
                    });
                    if (CollectionUtil.isNotEmpty(valueVoList)) {
                        mthFormConfigVo.setValueList(valueVoList);
                    }
                }
            }
        });
        return voList;
    }


    @Override
    public List<MthFormConfigDataVo> queryFormConfigData(List<Long> configIdList) {
//        List<MthFormConfigData> dataList = formConfigDataRepo.findByConfigIdInAndDelFlag(configIdList,0);
        List<MthFormConfig> configList = formConfigRepo.findAllById(configIdList);
        List<MthFormConfigDataVo> dataList = Lists.newArrayList();
        for (MthFormConfig mthFormConfig : configList) {
            if (StrUtil.equals(mthFormConfig.getRefType(), "dict")) {
                // 数据字典静态值
                Result<List<TsmPropertyValue>> dataListResult = sysService.getValuesByCode(mthFormConfig.getRefId());
                if (dataListResult.isSuccess() && ObjectUtil.isNotEmpty(dataListResult.getData())) {
                    List<MthFormConfigDataVo> valueVoList = dataListResult.getData().stream()
                            .map(data -> {
                                MthFormConfigDataVo vo = new MthFormConfigDataVo();
                                vo.setConfigId(mthFormConfig.getId());
                                vo.setConfigValue(data.getValueName());
                                return vo;
                            }).collect(Collectors.toList());
                    dataList.addAll(valueVoList);
                }
            }
        }

        List<MthFormConfigData> confgDataList = formConfigDataRepo.findByConfigIdIn(configIdList);
        List<MthFormConfigDataVo> configDataVoList = formConfigDataConverter.entityListToVoList(confgDataList);
        dataList.addAll(configDataVoList);
        return dataList;
    }
}
