package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Validator;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.pinyin.PinyinUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.BeanUtil;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.common.utils.ValidatorUtils;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.sys.FileBase64;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.feign.client.dto.DownloadFileVo;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.match.api.bo.MatchMyCollectBo;
import com.crunii.micro.service.match.api.bo.MthCancelRegisterAppBo;
import com.crunii.micro.service.match.api.bo.MthRegisterAddAppBo;
import com.crunii.micro.service.match.api.bo.MthRegisterBo;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.api.vo.sign.MthSignRspVo;
import com.crunii.micro.service.match.converter.MthRegisterConverter;
import com.crunii.micro.service.match.converter.MthRegisterDetailConverter;
import com.crunii.micro.service.match.enums.*;
import com.crunii.micro.service.match.jdbc.MthMatchGroupDao;
import com.crunii.micro.service.match.jdbc.MthMatchInfoDao;
import com.crunii.micro.service.match.jdbc.MthMatchItemDao;
import com.crunii.micro.service.match.jdbc.MthRegisterDao;
import com.crunii.micro.service.match.service.*;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.RequiredArgsConstructor;
import net.coobird.thumbnailator.Thumbnails;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import sun.font.FontDesignMetrics;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;
import java.awt.image.BufferedImage;
import java.io.*;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.*;
import java.util.stream.Collectors;

import static com.crunii.micro.common.constants.FontConstants.PingFang_SC;
import static com.crunii.micro.service.match.constants.MatchConstants.ROLE_NAME;
import static com.crunii.micro.service.match.constants.MatchMessageConstants.*;

/**
 * 赛事-赛项-组别-区域报名信息;Service业务层处理
 *
 * @author mth
 * @date 2023-09-19
 */
@RequiredArgsConstructor
@Service
public class MthRegisterServiceImpl implements IMthRegisterService {

    private static final String PNG_BASE64="data:image/png;base64,";
    private final MthRegisterRepo baseRepo;
    private final MthRegisterConverter baseConverter;

    private final MthRegisterDetailRepo detailRepo;
    private final MthRegisterDetailConverter detailConverter;

    private final MthWorkRepo workRepo;
    private final MthRoundListRepo roundListRepo;

    private final MthRoundScoreRepo roundScoreRepo;

    /**
     * 不使用构造函数注入, 解决循环依赖
     */
    @Resource
    private IMthRoundListService roundListService;

    private final IMthRegisterConfigService configService;
    private final MthRoundInstanceRepo roundInstanceRepo;

    private final MthMatchInfoRepo matchInfoRepo;

    /**
     * 不使用构造函数注入, 解决循环依赖
     */
    @Resource
    private IMthMatchInfoService matchInfoService;

    private final MthRegisterDao registerDao;

    private final MthMatchItemRepo itemRepo;
    private final MthMatchGroupRepo groupRepo;

    private final MthMatchItemDao itemDao;

    private final MthMatchGroupDao groupDao;

    private final IObsFilService obsFilService;
    private final MthMatchProcessRepo processRepo;
    // 图片,文件,视频,音频
    private static final List<String> FILE_TYPES = Lists.newArrayList("pic", "file", "video", "audio");
    private final MthMatchCollectedRepo mthMatchCollectedRepo;
    private final MthMatchInfoDao matchManageDao;
    private final MthMatchSignRepo mthMatchSignRepo;
    private final MthMatchSignInformationRepo mthMatchSignInformationRepo;
    private final ObsService obsService;

    private final PushBusinessMsgService pushBusinessMsgService;

    private final MthProjectRepo projectRepo;
    private final MthProjectFinancingRepo financingRepo;
    private final MthProjectFinancingHistoryRepo financingHistoryRepo;
    private final MthProjectFinancingMemberRepo financingMemberRepo;
    private final TsmWorkerRoleRepo tsmWorkerRoleRepo;

    /**
     * 查询赛事-赛项-组别-区域报名信息;
     */
    @Override
    public MthRegisterVo queryById(Long id) {
        return baseConverter.entityToVo(baseRepo.getById(id));
    }

    /**
     * 查询赛事-赛项-组别-区域报名信息;列表
     */
    @Override
    public Result<PageRsp<MthRegisterVo>> queryPageList(PageReq<MthRegisterBo> bo) {
        return Result.ofSuccess(registerDao.queryPageList(bo));
    }

    /**
     * 查询赛事-赛项-组别-区域报名信息;列表
     */
    @Override
    public List<MthRegisterVo> queryList(MthRegisterBo bo) {
        return registerDao.queryList(bo);
    }

    @Override
    public List<MthRegisterAreaAppVo> queryRegisterAreaListForApp(Long matchId) {
        List<MthRegisterAreaAppVo> list = registerDao.queryRegisterAreaListForApp(matchId);
        return list;
    }

    @Override
    public MthRegisterConfigAppVo queryRegisterConfigForApp(Long matchId, Long roundInstanceId, Long registerId) {
        MthRegisterConfigAppVo result = new MthRegisterConfigAppVo();
        MthRoundInstance roundInstance = roundInstanceRepo.getById(roundInstanceId);
        String entrySubmitMode = roundInstance.getEntrySubmitMode();

        // 查询报名配置
        MthRegisterConfigAppVo.Register registerConfig = registerDao.queryRegisterConfigForApp(matchId);
        // 设置文件路径
        obsFilService.setObsFileBoById(registerConfig, MthRegisterConfigAppVo.Register::getPicId, MthRegisterConfigAppVo.Register::setPicFile);
        List<MthRegisterConfigAppVo.Detail> detailList = registerDao.queryRegisterConfigDetailForApp(registerConfig.getId());
        List<MatchBaseItemVo> itemList = matchInfoService.queryItemList(matchId);
        registerConfig.setItemList(itemList);
        if (ObjectUtil.isNotNull(registerId)) {
            MthRegister register = baseRepo.getById(registerId);
            // 回显赛项和组别填写值
            registerConfig.setRegisterId(register.getId());
            registerConfig.setItemId(register.getItemId());
            registerConfig.setGroupId(register.getGroupId());
            if (ObjectUtil.isNotNull(register.getItemId())) {
                MthMatchItem item = itemRepo.getById(register.getItemId());
                if (item != null) {
                    registerConfig.setItemName(item.getItemName());
                }
            }
            if (ObjectUtil.isNotNull(register.getGroupId())) {
                MthMatchGroup group = groupRepo.getById(register.getGroupId());
                if (group != null) {
                    registerConfig.setGroupName(group.getGroupName());
                }
            }
            // 回显报名表动态填写值
            setDetailValue(registerId, detailList);
        }
        // 回显动态表单选项值列表
        setDetailConfigDataList(detailList);
        // 根据组别分组
        Map<String, List<MthRegisterConfigAppVo.Detail>> detailMapList = StreamUtils.groupByKey(detailList, MthRegisterConfigAppVo.Detail::getGroupName);
        registerConfig.setDetailList(detailMapList);
        //查询年龄限制
        MthMatchInfo matchInfo = matchInfoRepo.getOne(matchId);
        if (ObjectUtil.isNotEmpty(matchInfo)){
            registerConfig.setAgeBegin(matchInfo.getAgeBegin());
            registerConfig.setAgeEnd(matchInfo.getAgeEnd());
            registerConfig.setAgeLimit(matchInfo.getAgeLimit());
        }
        // 报名配置
        result.setRegister(registerConfig);

        if (StrUtil.equals(RoundEnums.EntrySubmitMode.FOLLOWREGISTER.getCode(), entrySubmitMode)) {
            // 查询作品配置
            MthRegisterConfigAppVo.Work workConfig = queryWorkTableConfig(matchId, registerId);
            // 作品配置
            result.setWork(workConfig);
        }

        if (Objects.equals(matchInfo.getQcFlag(), 1) && ObjectUtil.isNotNull(registerId)) {
            // 青创赛事
            MthProject project = projectRepo.findByRegisterId(registerId);
            if(project != null) {
                MthProjectFinancing financing = financingRepo.findByProjectId(project.getId());
                if(financing != null) {
                    MthRegisterConfigAppVo.Financing financingVo = new MthRegisterConfigAppVo.Financing();
                    cn.hutool.core.bean.BeanUtil.copyProperties(financing, financingVo);
                    //查询融资历史
                    List<MthProjectFinancingHistory> historyList = financingHistoryRepo.findByFinancingId(financing.getId());
                    List<MthRegisterConfigAppVo.Financing.History> financingHistoryList = historyList.stream().map(l -> {
                        MthRegisterConfigAppVo.Financing.History financingHistory = new MthRegisterConfigAppVo.Financing.History();
                        financingHistory.setFinancingTime(l.getFinancingTime());
                        financingHistory.setFinancingRounds(l.getFinancingRounds());
                        financingHistory.setAmount(l.getAmount());
                        financingHistory.setInvestor(l.getInvestor());
                        return financingHistory;
                    }).collect(Collectors.toList());
                    financingVo.setHistoryList(financingHistoryList);
                    //查询核心成员
                    List<MthProjectFinancingMember> members = financingMemberRepo.findByFinancingId(financing.getId());
                    List<MthRegisterConfigAppVo.Financing.Member> membersList = members.stream().map(meber -> {
                        MthRegisterConfigAppVo.Financing.Member teamMembers = new MthRegisterConfigAppVo.Financing.Member();
                        cn.hutool.core.bean.BeanUtil.copyProperties(meber,teamMembers);
                        return teamMembers;
                    }).collect(Collectors.toList());
                    obsFilService.setObsFileBoByIds(membersList, MthRegisterConfigAppVo.Financing.Member::getHeadSculpture, MthRegisterConfigAppVo.Financing.Member::setHeadSculptureFile);
                    financingVo.setMemberList(membersList);
                    result.setFinancing(financingVo);
                }
            }
        }

        return result;
    }

    @Override
    public MthRegisterConfigAppVo.Work queryWorkTableConfig(Long matchId, Long registerId) {
        // 查询作品配置
        MthRegisterConfigAppVo.Work workConfig = registerDao.queryWorkConfigForApp(matchId);
        if(ObjectUtil.isNotNull(workConfig)) {
            // 设置文件路
            obsFilService.setObsFileBoById(workConfig, MthRegisterConfigAppVo.Work::getPicId, MthRegisterConfigAppVo.Work::setPicFile);
            List<MthRegisterConfigAppVo.Detail> workDetailList = registerDao.queryRegisterConfigDetailForApp(workConfig.getId());
            if (ObjectUtil.isNotNull(registerId)) {
                List<MthWorkVo> mthWorkVos = registerDao.queryRegisterWorkForApp(registerId);
                if (ObjectUtil.isNotEmpty(mthWorkVos)) {
                    MthWorkVo work = mthWorkVos.get(0);
                    if(ObjectUtil.isNotNull(work.getId())) {
                        // 回显作品名称填写值
                        workConfig.setWorkName(work.getWorkName());
                        workConfig.setWorkId(work.getId());
                        // 回显作品表动态填写值
                        setDetailValue(work.getId(), workDetailList);
                    }
                }
            }
            // 回显动态表单选项值列表
            setDetailConfigDataList(workDetailList);
            // 根据组别分组
            Map<String, List<MthRegisterConfigAppVo.Detail>> detailMapList = StreamUtils.groupByKey(workDetailList, MthRegisterConfigAppVo.Detail::getGroupName);
            // 作品配置
            workConfig.setDetailList(detailMapList);
        }
        return workConfig;
    }

    @Override
    public MthMyMatchListDetailAppVo myMatchListDetail(Long matchId, Long registerId) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        MthMyMatchListDetailAppVo result = new MthMyMatchListDetailAppVo();
        MthMatchInfo matchInfo = matchInfoRepo.getById(matchId);
        // 查询报名信息
        MthMyMatchListDetailAppVo.Register registerVo = registerDao.queryRegisterForApp(registerId);
        if (ObjectUtil.isNotEmpty(registerVo) && Objects.equals(registerVo.getReviewResult(), RegisterAuditStatusEnums.REJECT.getCode())) {
            MthRegisterReview review = registerDao.queryRegisterReview(registerId);
            if (review != null) {
                registerVo.setReviewReason(review.getReason());
            }
        }
        registerVo.setBannerId(matchInfo.getBannerId());
        // 设置文件路
        obsFilService.setObsFileBoById(registerVo, MthMyMatchListDetailAppVo.Register::getBannerId, MthMyMatchListDetailAppVo.Register::setBanner);
        List<MthMyMatchListDetailAppVo.Detail> detailList = registerDao.queryRegisterDetailForApp(Lists.newArrayList(registerVo.getId()));
        // 根据组别分组
        Map<String, List<MthMyMatchListDetailAppVo.Detail>> detailMapList = StreamUtils.groupByKey(detailList, MthMyMatchListDetailAppVo.Detail::getGroupName);
        registerVo.setDetailList(detailMapList);
        registerVo.setMatchName(matchInfo.getMatchName());
        List<MatchBaseItemVo> itemList = matchInfoService.queryItemList(matchId);
        registerVo.setItemList(itemList);
        // 设置附件值
        setDetailFileValue(detailList);
        // 报名信息
        result.setRegister(registerVo);
        // 查询作品信息
        List<MthMyMatchListDetailAppVo.Work> workList = registerDao.queryWorkForApp(registerId);
        if (ObjectUtil.isNotEmpty(workList)) {
            // 设置文件路
            obsFilService.setObsFileBoByIds(workList, MthMyMatchListDetailAppVo.Work::getPicId, MthMyMatchListDetailAppVo.Work::setPicFile);
            List<Long> workIds = StreamUtils.toList(workList, MthMyMatchListDetailAppVo.Work::getId);
            List<MthMyMatchListDetailAppVo.Detail> workDetailList = registerDao.queryRegisterDetailForApp(workIds);
            Map<Long, List<MthMyMatchListDetailAppVo.Detail>> workDetailMapList = StreamUtils.groupByKey(workDetailList, MthMyMatchListDetailAppVo.Detail::getBizId);
            for (MthMyMatchListDetailAppVo.Work work : workList) {
                List<MthMyMatchListDetailAppVo.Detail> details = workDetailMapList.get(work.getId());
                // 设置附件值
                setDetailFileValue(details);
                // 根据组别分组
                Map<String, List<MthMyMatchListDetailAppVo.Detail>> groupDetailMapList = StreamUtils.groupByKey(details, MthMyMatchListDetailAppVo.Detail::getGroupName);
                work.setDetailList(groupDetailMapList);
            }
            // 作品信息
            result.setWorkList(workList);
        }
        // 查询晋级和获奖并且场次结束的场次信息
        List<MthMyMatchListDetailAppVo.RoundPrize> roundPrizeList = registerDao.queryRoundScoreForApp(registerId);
        for (MthMyMatchListDetailAppVo.RoundPrize roundPrize : roundPrizeList) {
            if (StrUtil.isNotBlank(roundPrize.getOtherPrizeName())) {
                if (StrUtil.isNotBlank(roundPrize.getPrizeName())) {
                    roundPrize.setPrizeName(roundPrize.getPrizeName() + "、" + roundPrize.getOtherPrizeName());
                } else {
                    roundPrize.setPrizeName(roundPrize.getOtherPrizeName());
                }
            }
        }
        result.setRoundPrizeList(roundPrizeList);

        List<String> showUpdateBtnStatus = Lists.newArrayList(RegisterAuditStatusEnums.PENDING.getCode(),
                RegisterAuditStatusEnums.REJECT.getCode(), RegisterAuditStatusEnums.NOREVIEW.getCode());

        MthMyMatchListDetailAppVo.ShowButton showButton = new MthMyMatchListDetailAppVo.ShowButton();
        if (Objects.equals(registerVo.getMatchState(), 0)
                && showUpdateBtnStatus.contains(registerVo.getReviewResult())
                && (registerVo.getRegisterationStart() != null && System.currentTimeMillis() > registerVo.getRegisterationStart().getTime())
                && (registerVo.getRegisterationEnd() != null && System.currentTimeMillis() < registerVo.getRegisterationEnd().getTime())) {
            showButton.setShowUpdateBtn(true);
            showButton.setShowCancelBtn(true);
        } else {
            showButton.setShowUpdateBtn(false);
            showButton.setShowCancelBtn(false);
        }
        //判断作品上传按钮,单独上传且报名已通过时展示
        MthRegister register = baseRepo.getOne(registerId);
        if ((Objects.equals(registerVo.getMatchState(), 0) || Objects.equals(registerVo.getMatchState(), 1))
                && StrUtil.equals(registerVo.getEntrySubmitMode(), RoundEnums.EntrySubmitMode.ALONE.getCode())
                && (registerVo.getEntryDeadline() != null && System.currentTimeMillis() < registerVo.getEntryDeadline().getTime())
                && (RegisterAuditStatusEnums.APPROVED.getCode().equals(register.getReviewResult()) || RegisterAuditStatusEnums.NOREVIEW.getCode().equals(register.getReviewResult()))) {
            showButton.setShowUploadWorkBtn(true);
        } else {
            showButton.setShowUploadWorkBtn(false);
        }
        //报名未审核、审核未通过,将签到签退按钮设置为false
        if (!(RegisterAuditStatusEnums.APPROVED.getCode().equals(register.getReviewResult()) || RegisterAuditStatusEnums.NOREVIEW.getCode().equals(register.getReviewResult()))){
            showButton.setWhetherSignIn(false);
            showButton.setWhetherSignOut(false);
        }else {
            //查询最新场次设置的签到信息
            MthRoundInstance newestInstance = roundInstanceRepo.findNewestInstance(matchId);
            List<MthSignRspVo> signInformation=new ArrayList<>();
            if (ObjectUtil.isNotEmpty(newestInstance)){
                signInformation.addAll(matchManageDao.getWhetherSign(newestInstance.getId()));
            }
            //是否需要签到
            for (MthSignRspVo mthSignRspVo : signInformation) {
                MthMyMatchListDetailAppVo.SignInformation signInformations=new MthMyMatchListDetailAppVo.SignInformation();
                //是否需要签到
                if (mthSignRspVo.getWhetherSignIn()){
                    showButton.setWhetherSignIn(mthSignRspVo.getWhetherSignIn());
                    BeanUtil.copyProperties(mthSignRspVo,signInformations);
                    signInformations.setLatitude(mthSignRspVo.getSignLatitude());
                    signInformations.setLongitude(mthSignRspVo.getSignLongitude());
                    result.setSignInformation(signInformations);
                }
                //是否需要签退
                if (mthSignRspVo.getWhetherSignOut()){
                    showButton.setWhetherSignOut(mthSignRspVo.getWhetherSignOut());
                    BeanUtil.copyProperties(mthSignRspVo,signInformations);
                    signInformations.setLatitude(mthSignRspVo.getSignLatitude());
                    signInformations.setLongitude(mthSignRspVo.getSignLongitude());
                    result.setSignInformation(signInformations);
                }
            }
        }
        //是否收藏
        List<MthMatchCollected> matchs = mthMatchCollectedRepo.findByMatchId(matchId,(long)worker.getLoginId());
        showButton.setWhetherCollect(matchs.size()>0);
        result.setShowButton(showButton);
        //查询所有场次以及签到信息
        List<Map<String,Object>>signRecords=new ArrayList<>();
        List<MthInstanceVo> instances = registerDao.findByMatchIdAndUserId(matchId, worker.getLoginId());
        for (MthInstanceVo instance : instances) {
            Map<String,Object> signRecord=new LinkedHashMap<>();
            signRecord.put("场次",instance.getAreaName()+instance.getRoundName());
            //比赛时间
            StringBuilder roundTime=new StringBuilder();
            SimpleDateFormat dateTime = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            if (ObjectUtil.isNotEmpty(instance.getRoundStart())){
                String createFormat = dateTime.format(instance.getRoundStart());
                roundTime.append(createFormat);
            }
            if (ObjectUtil.isNotEmpty(instance.getRoundEnd())){
                String createFormat = dateTime.format(instance.getRoundEnd());
                roundTime.append("~").append(createFormat);
            }
            if (StringUtils.isNotEmpty(roundTime)){
                signRecord.put("比赛时间",roundTime.toString());
            }
            //签到记录,线下比赛时，查询签到记录
            if (StringUtils.isNotEmpty(instance.getPlayMode()) && "offline".equals(instance.getPlayMode())) {
                List<MthMatchSign> signs = mthMatchSignRepo.findByWorkerIdAndInstanceId(worker.getLoginId(), instance.getId());
                List<MthMatchSignInformation> signInformations = mthMatchSignInformationRepo.findByInstanceId(instance.getId());
                //签到签退信息
                for (MthMatchSignInformation information : signInformations) {
                    List<MthMatchSign> signOutList = signs.stream().filter(x -> x.getSignHandle() == -1).collect(Collectors.toList());
                    List<MthMatchSign> signInList = signs.stream().filter(x -> x.getSignHandle() == 1).collect(Collectors.toList());
                    MthMyMatchListDetailAppVo.ShowButton showButton1 = result.getShowButton();
                    //存在签到信息，将签到按钮设置为false
                    if (signInList.size() > 0) {
                        showButton1.setWhetherSignIn(false);
                    }
                    //存在签退信息，将签退按钮设置为false
                    if (signOutList.size() > 0) {
                        showButton1.setWhetherSignOut(false);
                    }
                    result.setShowButton(showButton1);
                    //签到信息
                    if (1 == information.getInformationType()) {
                        String startTime = dateTime.format(information.getSignStartTime());
                        String endTime = dateTime.format(information.getSignEndTime());
                        signRecord.put("签到地点", StringUtils.isEmpty(information.getSignAddress()) ? information.getSignSiteName() : information.getSignSiteName() + information.getSignAddress());
                        signRecord.put("签到时间", startTime + "~" + endTime);
                        signRecord.put("签到状态", signInList.size() > 0 ? "已签到" : "未签到");
                    }
                    //签退信息
                    if (-1 == information.getInformationType()) {
                        String startTime = dateTime.format(information.getSignStartTime());
                        String endTime = dateTime.format(information.getSignEndTime());
                        signRecord.put("签退地点", StringUtils.isEmpty(information.getSignAddress()) ? information.getSignSiteName() : information.getSignSiteName() + information.getSignAddress());
                        signRecord.put("签退时间", startTime + "~" + endTime);
                        signRecord.put("签退状态", signInList.size() > 0 ? "已签退" : "未签退");
                    }
                }
            }
            signRecords.add(signRecord);
        }
        if (instances.size()>0){
            result.setSignRecord(signRecords);
        }
        //查询评委点评信息
        List<MthMyMatchListDetailAppVo.Reviewer> reviewer = registerDao.findReviewer(registerId);
        result.setReviewer(reviewer);
        return result;
    }

    /**
     * 回显动态表单选项值列表
     * @param detailList
     */
    private void setDetailConfigDataList(List<MthRegisterConfigAppVo.Detail> detailList) {
        // 查询配置选项值
        List<Long> configIds = StreamUtils.toList(detailList, MthRegisterConfigAppVo.Detail::getRegisterConfigId);
        List<MthFormConfigDataVo> configDataVoList = configService.queryFormConfigData(configIds);
        Map<Long, List<MthFormConfigDataVo>> configDataMapList = StreamUtils.groupByKey(configDataVoList, MthFormConfigDataVo::getConfigId);
        for (MthRegisterConfigAppVo.Detail detail : detailList) {
            List<MthFormConfigDataVo> configDataList = configDataMapList.get(detail.getRegisterConfigId());
            if (ObjectUtil.isNotEmpty(configDataList)) {
                // 配置选项值
                detail.setConfigDataList(configDataList);
            }
        }
    }

    /**
     * 回显动态表单填写值
     * @param bizId
     * @param detailList
     */
    private void setDetailValue(Long bizId, List<MthRegisterConfigAppVo.Detail> detailList) {
        // 查询填写值
        Map<String, Object> props = Maps.newHashMap();
        props.put("bizId", bizId);
        props.put("delFlag", 0);
        List<MthRegisterDetail> detailValList = detailRepo.findByProps(props);
        Map<Long, MthRegisterDetail> detailValMap = StreamUtils.toIdentityMap(detailValList, MthRegisterDetail::getConfigDetailId);
        for (MthRegisterConfigAppVo.Detail detail : detailList) {
            MthRegisterDetail detailVal = detailValMap.get(detail.getId());
            if (ObjectUtil.isNotNull(detailVal)) {
                // 设置填写值
                detail.setDetailId(detailVal.getId());
                detail.setValue(detailVal.getValue());
            }
        }
        // 查询文件类型信息
        List<MthRegisterConfigAppVo.Detail> fileDetailList = detailList.stream()
                .filter(item -> StrUtil.isNotBlank(item.getValue()))
                .filter(item -> FILE_TYPES.contains(item.getConfigType()))
                .collect(Collectors.toList());
        // 设置文件路径
        obsFilService.setObsFileBoListByIds(fileDetailList, item -> item.getValue(), MthRegisterConfigAppVo.Detail::setFileValues);
    }
    private void setDetailFileValue(List<MthMyMatchListDetailAppVo.Detail> detailList) {
        if (detailList == null) {
            return;
        }
        // 查询文件类型信息
        List<MthMyMatchListDetailAppVo.Detail> fileDetailList = detailList.stream()
                .filter(item -> StrUtil.isNotBlank(item.getValue()))
                .filter(item -> FILE_TYPES.contains(item.getType()))
                .collect(Collectors.toList());
        // 设置文件路径
        obsFilService.setObsFileBoListByIds(fileDetailList, item -> item.getValue(), MthMyMatchListDetailAppVo.Detail::setFileValues);
    }


    /**
     * 新增赛事-赛项-组别-区域报名信息;
     */
    @Override
    public Boolean insertByBo(MthRegisterBo bo) {
        MthRegister add = baseConverter.boToEntity(bo);
        validEntityBeforeSave(add);
        baseRepo.save(add);
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public MthRegister registerForApp(MthRegisterAddAppBo bo) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        long userId = Integer.valueOf(currWorker.getLoginId()).longValue();
        List<MthRegister> exists = registerDao.queryMyRegisterList(userId, bo.getRegister().getMatchId(), bo.getRegister().getItemId(), bo.getRegister().getGroupId());
        if (ObjectUtil.isNotEmpty(exists)) {
            throw new BusinessException("赛事已报名, 不能重复报名!");
        }
        MthRoundInstance roundInstance = roundInstanceRepo.getById(bo.getRegister().getRoundId());
        Date registerationEnd = roundInstance.getRegisterationEnd();
        // 不需要审核, 判断是否到报名截止日
        if (registerationEnd == null || registerationEnd.before(new Date())) {
            throw new BusinessException("报名失败! 报名已结束");
        }
        MthMatchInfo matchInfo = matchInfoRepo.getById(bo.getRegister().getMatchId());
        if (matchInfo == null) {
            throw new BusinessException("报名失败! 赛事不存在");
        }

        if (ObjectUtil.isNotNull(bo.getRegister().getGroupId())) {
            //校验身份证号
            Optional<MthRegisterAddAppBo.Detail> idcared = bo.getRegister().getDetailList().stream()
                    .filter(item -> StrUtil.equals(item.getKey(), "idcared") && StrUtil.isNotBlank(item.getValue())).findFirst();
            if (idcared.isPresent() && !Validator.isCitizenId(idcared.get().getValue())) {
                throw new BusinessException(MATCH_SIGN_UP_IDCAR_FORMAT_ERROR);
            }
            MthMatchGroup mthMatchGroup = groupRepo.getOne(bo.getRegister().getGroupId());
            //如果设置了年龄限制,就校验年龄是否满足要求
            if (mthMatchGroup != null && StringUtils.isNotEmpty(mthMatchGroup.getAgeBegin()) && StringUtils.isNotEmpty(mthMatchGroup.getAgeEnd())) {
                //校验年龄是否满足要求
                if (idcared.isPresent() && StringUtils.isNotEmpty(matchInfo.getAgeLimit())) {
                    String idCar = idcared.get().getValue();
                    Calendar startTime = Calendar.getInstance();
                    Calendar endTime = Calendar.getInstance();
                    Calendar birthDateTime = Calendar.getInstance();
                    //根据身份证号计算出生年月
                    int year = Integer.parseInt(idCar.substring(6, 10));
                    int moth = Integer.parseInt(idCar.substring(11, 12));
                    int day = Integer.parseInt(idCar.substring(13, 14));
                    //格式化时间
                    birthDateTime.set(year, moth, day, 0, 0, 0);
                    Date time = birthDateTime.getTime();
                    //统一年龄限制
                    if (MatchEnums.AgeLimit.UNIFY_AGE.getCode().equals(matchInfo.getAgeLimit())) {
                        startTime.add(Calendar.YEAR, -Integer.parseInt(matchInfo.getAgeBegin()));
                        endTime.add(Calendar.YEAR, -Integer.parseInt(matchInfo.getAgeEnd()));
                    }
                    //组别限制
                    if (MatchEnums.AgeLimit.GROUP.getCode().equals(matchInfo.getAgeLimit())) {
                        startTime.add(Calendar.YEAR, -Integer.parseInt(mthMatchGroup.getAgeBegin()));
                        endTime.add(Calendar.YEAR, -Integer.parseInt(mthMatchGroup.getAgeEnd()));
                    }
                    Date ageStartTime = startTime.getTime();
                    Date ageEndTime = endTime.getTime();
                    if (ageStartTime.before(time) || ageEndTime.after(time)) {
                        throw new BusinessException(MATCH_SIGN_UP_AGE_OUT);
                    }
                }
            }
        }

        // 校验赛事活动是否报名中状态
        if (!StrUtil.equals(matchInfo.getStatus(), MatchInfoStatusEnums.REGISTER.getStatus())) {
            throw new BusinessException("赛事非报名中状态!");
        }
        // 新增报名信息
        MthRegisterAddAppBo.Register registerBo = bo.getRegister();
        MthRegister register = baseConverter.appBoToEntity(registerBo);
        // 设置参赛者编号
        register.setCode(getRegisterCode(matchInfo.getCategoryName()));
        // 配置id
        register.setRegisterConfigId(registerBo.getRegisterConfigId());
        // 设置当前报名用户信息
        register.setUserId(Integer.valueOf(currWorker.getLoginId()).longValue());
        // 设置参赛者姓名与手机
        setUserNameAndMobile(currWorker, registerBo, register);
        Integer registerationAuditFlag = matchInfo.getRegisterationAuditFlag();
        if (ObjectUtil.equals(registerationAuditFlag, 0)) {
            // 不需要审核
            register.setReviewResult(RegisterAuditStatusEnums.NOREVIEW.getCode());
        } else {
            // 需要报名审核
            register.setReviewResult(RegisterAuditStatusEnums.PENDING.getCode());
        }
        baseRepo.saveAndFlush(register);
        List<MthRegisterDetail> detailList = detailConverter.appBoToEntity(registerBo.getDetailList());
        for (MthRegisterDetail detail : detailList) {
            detail.setBizId(register.getId());
        }
        MthRoundList roundList = new MthRoundList();
        detailRepo.saveAllAndFlush(detailList);
        if (ObjectUtil.isNotNull(bo.getWork())) {
            // 随报名上传作品信息
            Long workId = uploadWork(bo.getWork(), matchInfo.getCategoryName());
            roundList.setWorkId(workId);
        }
        roundList.setRegisterId(register.getId());
        roundList.setUserId(register.getUserId());
        roundList.setUserName(register.getUserName());
        roundList.setRoundId(bo.getRegister().getRoundId());
        roundList.setStatus(RoundListStatusEnums.RUNNING.getCode());
        // 新增场次名单
        roundListRepo.saveAndFlush(roundList);
        MthRoundScore roundScore = new MthRoundScore();
        roundScore.setRoundListId(roundList.getId());
        // 新增场次比赛记录
        roundScoreRepo.saveAndFlush(roundScore);
        if (Objects.equals(matchInfo.getQcFlag(), 1)) {
            // 保存青创赛事融资申请
            saveProjectFinancing(bo, userId, register.getId(), true);
        }
        // 新增过程记录
        MthMatchProcess matchProcessTime =new MthMatchProcess();
        matchProcessTime.setType("1");
        matchProcessTime.setBizId(register.getId());
        matchProcessTime.setMatchId(roundInstance.getMatchId());
        matchProcessTime.setUserId(roundList.getRegisterId());
        matchProcessTime.setShowTitle("报名时间：" + DateUtil.format(new Date(), DatePattern.NORM_DATETIME_FORMATTER));
        processRepo.saveAndFlush(matchProcessTime);
        // 推送消息
        pushBusinessMsgService.pushRegisteredMsg(userId, matchInfo.getMatchName());
        return register;
    }

    /**
     * 保存青创赛事融资申请
     * @param bo
     * @param userId
     * @param regiserId
     */
    private void saveProjectFinancing(MthRegisterAddAppBo bo, long userId, Long regiserId, boolean addFlag) {
        // 是否融资需求
        String q_financing_need = getDetailValue(bo, "q_financing_need");
        if (StrUtil.isNotBlank(q_financing_need)) {
            // 青创项目
            String q_name = getDetailValue(bo, "q_name");
            String q_logo = getDetailValue(bo, "q_logo");
            String q_start_time = getDetailValue(bo, "q_start_time");
            String q_address = getDetailValue(bo, "q_address");
            String q_item_desc = getDetailValue(bo, "q_item_desc");
            String q_show_images = getDetailValue(bo, "q_show_images");
            String q_company_name = getDetailValue(bo, "q_company_name");
            String q_position = getDetailValue(bo, "q_position");
            String q_register_address = getDetailValue(bo, "q_register_address");
            String q_regiser_time = getDetailValue(bo, "q_regiser_time");
            String q_regiser_capital = getDetailValue(bo, "q_regiser_capital");
            String q_legal_representative = getDetailValue(bo, "q_legal_representative");
            String q_industry = getDetailValue(bo, "q_industry");
            String q_enterprise_debt_amount = getDetailValue(bo, "q_enterprise_debt_amount");
            String q_previous_year_revenue = getDetailValue(bo, "q_previous_year_revenue");
            String q_previous_year_tax_revenue = getDetailValue(bo, "q_previous_year_tax_revenue");
            String q_employee_count = getDetailValue(bo, "q_employee_count");
            String q_team_members = getDetailValue(bo, "q_team_members");

            MthProject project = projectRepo.findByRegisterId(regiserId);
            if (project == null) {
                project = new MthProject();
            }
            project.setFinancingNeed(q_financing_need);
            project.setRegisterId(regiserId);
            project.setName(q_name);
            if(StrUtil.isNotBlank(q_logo)) {
                project.setLogo(Long.valueOf(q_logo));
            }
            project.setItemName(bo.getRegister().getItemName());
            project.setGroupName(bo.getRegister().getGroupName());
            if(StrUtil.isNotBlank(q_start_time)) {
                project.setStartTime(DateUtil.parse(q_start_time));
            }
            project.setAddress(q_address);
            project.setItemDesc(q_item_desc);
            project.setShowImages(q_show_images);
            project.setCompanyName(q_company_name);
            project.setPosition(q_position);
            project.setRegisterAddress(q_register_address);
            if(StrUtil.isNotBlank(q_regiser_time)) {
                project.setRegiserTime(DateUtil.parse(q_regiser_time));
            }
            if(StrUtil.isNotBlank(q_regiser_capital)) {
                if (NumberUtil.isNumber(q_regiser_capital)) {
                    project.setRegiserCapital(new BigDecimal(q_regiser_capital));
                } else {
                    throw new BusinessException("注册资本-类型错误");
                }
            }
            project.setLegalRepresentative(q_legal_representative);
            project.setIndustry(q_industry);
            if(StrUtil.isNotBlank(q_enterprise_debt_amount)) {
                if (NumberUtil.isNumber(q_enterprise_debt_amount)) {
                    project.setEnterpriseDebtAmount(new BigDecimal(q_enterprise_debt_amount));
                } else {
                    throw new BusinessException("企业负债金额-类型错误");
                }
            }
            if(StrUtil.isNotBlank(q_previous_year_revenue)) {
                if (NumberUtil.isNumber(q_previous_year_revenue)) {
                    project.setPreviousYearRevenue(new BigDecimal(q_previous_year_revenue));
                } else {
                    throw new BusinessException("上一年营收-类型错误");
                }
            }
            if(StrUtil.isNotBlank(q_previous_year_tax_revenue)) {
                if (NumberUtil.isNumber(q_previous_year_tax_revenue)) {
                    project.setPreviousYearTaxRevenue(new BigDecimal(q_previous_year_tax_revenue));
                } else {
                    throw new BusinessException("上一年税收-类型错误");
                }
            }
            if(StrUtil.isNotBlank(q_employee_count)) {
                if (NumberUtil.isInteger(q_employee_count)) {
                    project.setEmployeeCount(Integer.valueOf(q_employee_count));
                } else {
                    throw new BusinessException("员工数-类型错误");
                }
            }
            if(StrUtil.isNotBlank(q_team_members)) {
                if (NumberUtil.isInteger(q_team_members)) {
                    project.setTeamMembers(Integer.valueOf(q_team_members));
                } else {
                    throw new BusinessException("团队成员人数-类型错误");
                }
            }
            validationThrowException(project);

            projectRepo.saveAndFlush(project);
            Long projectId = project.getId();
            if (StrUtil.equals(q_financing_need, "是") && bo.getFinancing() != null) {
                // 有融资需求并且是新增
                validationThrowException(bo.getFinancing());
                MthProjectFinancing financing = financingRepo.findByProjectId(projectId);
                MthRegisterAddAppBo.Financing financingBo = bo.getFinancing();
                if (financing == null) {
                    financing = new MthProjectFinancing();
                    financing.setProjectId(projectId);
                    financing.setUserId(userId);
                    financing.setCode("QCSS"+DateUtil.format(new Date(), DatePattern.PURE_DATETIME_FORMAT) + RandomUtil.randomStringUpper(4));
                    financing.setReviewStatus(FinancingEnums.ReviewStatus.PENDING.getCode());
                    financing.setPublicityStatus(FinancingEnums.PublicityStatus.UNPUBLISHED.getCode());
                } else {
                    // 审核状态是不通过, 则再次提交融资申请
                    if(FinancingEnums.ReviewStatus.REJECT.getCode().equals(financing.getReviewStatus())) {
                        financing.setReviewStatus(FinancingEnums.ReviewStatus.PENDING.getCode());
                    }
                }
                BeanUtil.copyPropertiesIgnoreNull(financingBo, financing);
                financingRepo.saveAndFlush(financing);
                Long financingId = financing.getId();
                financingHistoryRepo.deleteByFinancingId(financingId);
                financingMemberRepo.deleteByFinancingId(financingId);

                if (ObjectUtil.isNotEmpty(financingBo.getHistoryList())) {
                    financingBo.getHistoryList().forEach(MthRegisterServiceImpl::validationThrowException);
                    List<MthRegisterAddAppBo.Financing.History> historyListBo = financingBo.getHistoryList();
                    List<MthProjectFinancingHistory> historyList = historyListBo.stream().map(item -> {
                        MthProjectFinancingHistory history = new MthProjectFinancingHistory();
                        BeanUtil.copyPropertiesIgnoreNull(item, history);
                        history.setFinancingId(financingId);
                        return history;
                    }).collect(Collectors.toList());
                    financingHistoryRepo.saveAllAndFlush(historyList);
                }

                if (ObjectUtil.isNotEmpty(financingBo.getMemberList())) {
                    financingBo.getMemberList().forEach(MthRegisterServiceImpl::validationThrowException);
                    List<MthRegisterAddAppBo.Financing.Member> memberListBo = financingBo.getMemberList();
                    List<MthProjectFinancingMember> memberList = memberListBo.stream().map(item -> {
                        MthProjectFinancingMember member = new MthProjectFinancingMember();
                        BeanUtil.copyPropertiesIgnoreNull(item, member);
                        member.setFinancingId(financingId);
                        return member;
                    }).collect(Collectors.toList());
                    financingMemberRepo.saveAllAndFlush(memberList);
                }
                //新增青创赛事成功，给管理员发送待审核消息
                List<TsmWorkerRole> workes = tsmWorkerRoleRepo.findByRoleName(ROLE_NAME);
                List<Long> loginIds = workes.stream().map(work->Long.valueOf(work.getLoginId())).collect(Collectors.toList());
                if (loginIds.size()>0){
                    pushBusinessMsgService.pushCheckMsg(loginIds);
                }
            }
        }
    }

    private static void validationThrowException(Object bean) {
        ValidatorUtils.validateMsgException(bean, AddGroup.class);
    }

    private static String getDetailValue(MthRegisterAddAppBo bo, String name) {
        Optional<MthRegisterAddAppBo.Detail> optional = bo.getRegister().getDetailList().stream()
                .filter(item -> StrUtil.equals(item.getKey(), name) && StrUtil.isNotBlank(item.getValue())).findFirst();
        if (optional.isPresent()) {
            return optional.get().getValue();
        }
        return null;
    }

    private static void setUserNameAndMobile(Worker currWorker, MthRegisterAddAppBo.Register registerBo, MthRegister register) {
        Optional<MthRegisterAddAppBo.Detail> userName = registerBo.getDetailList().stream()
                .filter(item -> StrUtil.equals(item.getKey(), "name") && StrUtil.isNotBlank(item.getValue())).findFirst();
        if (userName.isPresent()) {
            if (userName.get().getValue().length() > 64) {
                throw new BusinessException("参赛者姓名长度不能超过64位!");
            }
            register.setUserName(userName.get().getValue());
        } else {
            register.setUserName(currWorker.getWorkerName());
            //throw new BusinessException("参赛者姓名不能为空!");
        }
        Optional<MthRegisterAddAppBo.Detail> phone = registerBo.getDetailList().stream()
                .filter(item -> StrUtil.equals(item.getKey(), "phone") && StrUtil.isNotBlank(item.getValue())).findFirst();
        if (phone.isPresent()) {
            if (!Validator.isMobile(phone.get().getValue())) {
                throw new BusinessException("参赛者手机号码格式不正确!");
            }
            register.setTelephone(phone.get().getValue());
        } else {
            register.setTelephone(currWorker.getMobile());
            //throw new BusinessException("参赛者手机号不能为空!");
        }
        Optional<MthRegisterAddAppBo.Detail> schoolName = registerBo.getDetailList().stream()
                .filter(item -> StrUtil.equals(item.getKey(), "schoolName") && StrUtil.isNotBlank(item.getValue())).findFirst();
        if (schoolName.isPresent()) {
            if (schoolName.get().getValue().length() > 64) {
                throw new BusinessException("学校名称长度不能超过64位!");
            }
            register.setSchoolName(schoolName.get().getValue());
        }
        Optional<MthRegisterAddAppBo.Detail> idcared = registerBo.getDetailList().stream()
                .filter(item -> StrUtil.equals(item.getKey(), "idcared") && StrUtil.isNotBlank(item.getValue())).findFirst();
        if (idcared.isPresent()) {
            if (!Validator.isCitizenId(idcared.get().getValue())) {
                throw new BusinessException("身份证号码格式不正确!");
            }
            register.setIdcared(idcared.get().getValue());
        }
        Optional<MthRegisterAddAppBo.Detail> companyName = registerBo.getDetailList().stream()
                .filter(item -> StrUtil.equals(item.getKey(), "companyName") && StrUtil.isNotBlank(item.getValue())).findFirst();
        if (companyName.isPresent()) {
            if (companyName.get().getValue().length() > 64) {
                throw new BusinessException("单位名称长度不能超过64位!");
            }
            register.setCompanyName(companyName.get().getValue());
        }
    }

    /**
     * 报名编号
     * @param categoryName
     * @return
     */
    private static String getRegisterCode(String categoryName) {
        StringBuilder sb = new StringBuilder();
        sb.append(PinyinUtil.getFirstLetter(categoryName, StrUtil.EMPTY).toUpperCase())
                .append(DateUtil.format(new Date(), DatePattern.SIMPLE_MONTH_FORMATTER))
                .append("CSZ").append(RandomUtil.randomNumbers(6));
        return sb.toString();
    }

    /**
     * 作品编号
     * @param categoryName
     * @return
     */
    private static String getWorkCode(String categoryName) {
        StringBuilder sb = new StringBuilder();
        sb.append(PinyinUtil.getFirstLetter(categoryName, StrUtil.EMPTY).toUpperCase())
                .append(DateUtil.format(new Date(), DatePattern.SIMPLE_MONTH_FORMATTER))
                .append("ZP").append(RandomUtil.randomNumbers(6));
        return sb.toString();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean registerUpdateForApp(MthRegisterAddAppBo bo) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        long userId = Integer.valueOf(currWorker.getLoginId()).longValue();
        MthRoundList roundList = roundListService.queryCurrRoundList(bo.getRegister().getId());
        MthRoundInstance roundInstance = roundInstanceRepo.getById(roundList.getRoundId());
        Date registerationEnd = roundInstance.getRegisterationEnd();
        // 不需要审核, 判断是否到报名截止日
        if (registerationEnd == null || registerationEnd.before(new Date())) {
            throw new BusinessException("更新报名信息失败! 报名时间已结束");
        }
        // 更新报名信息
        MthRegisterAddAppBo.Register registerBo = bo.getRegister();
        MthRegister register = baseConverter.appBoToEntity(registerBo);
        // 设置当前报名用户信息
        register.setUserId(Integer.valueOf(currWorker.getLoginId()).longValue());
        // 设置参赛者姓名与手机
        setUserNameAndMobile(currWorker, registerBo, register);
        MthRegister registerDb = baseRepo.getById(bo.getRegister().getId());
        // JPA持久化状态更新
        BeanUtil.copyPropertiesIgnoreNull(register, registerDb);
        if (StrUtil.equals(registerDb.getReviewResult(), RegisterAuditStatusEnums.REJECT.getCode())) {
            // 如果是审批拒绝状态, 重新提交待审核
            registerDb.setReviewResult(RegisterAuditStatusEnums.PENDING.getCode());
        }
        List<MthRegisterDetail> detailList = detailConverter.appBoToEntity(registerBo.getDetailList());
        for (MthRegisterDetail detail : detailList) {
            detail.setBizId(register.getId());
            detail.setDelFlag(0);
            detail.setCreateId(Integer.valueOf(currWorker.getLoginId()).longValue());
            detail.setCreateBy(currWorker.getWorkerName());
            detail.setCreateTime(new Date());
        }
        // id存在修改, 不存在新增
        detailRepo.saveAllAndFlush(detailList);
        MthMatchInfo matchInfo = matchInfoRepo.getById(bo.getRegister().getMatchId());
        if (ObjectUtil.isNotNull(bo.getWork())) {
            // 随报名上传作品信息
            uploadWork(bo.getWork(), matchInfo.getCategoryName());
        }

        if (Objects.equals(matchInfo.getQcFlag(), 1)) {
            // 保存青创赛事融资申请
            saveProjectFinancing(bo, userId, register.getId(), false);
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean uploadWorkForApp(MthRegisterAddAppBo.Work workBo) {
        if (ObjectUtil.isNull(workBo.getRegisterId())) {
            throw new BusinessException("上传作品信息失败, 报名id不能为空!");
        }
        // 查询当前的场次名单
        MthRoundList roundList = roundListService.queryCurrRoundList(workBo.getRegisterId());
        if (ObjectUtil.isNull(roundList)) {
            throw new BusinessException("上传作品信息失败, 场次名单id不存在!");
        }
        MthRoundInstance roundInstance = roundInstanceRepo.getById(roundList.getRoundId());
        Date entryDeadline = roundInstance.getEntryDeadline();
        // 不需要审核, 判断是否到报名截止日
        if (entryDeadline == null || entryDeadline.before(new Date())) {
            throw new BusinessException("上传失败! 作品上传时间已结束");
        }
        MthMatchInfo matchInfo = matchInfoRepo.getById(roundInstance.getMatchId());
        Long workId = uploadWork(workBo, matchInfo.getCategoryName());
        roundList.setWorkId(workId);
        // 修改场次名单
        roundListRepo.saveAndFlush(roundList);
        return true;
    }



    /**
     * 上传报名信息
     *
     * @param workBo
     */
    private Long uploadWork(MthRegisterAddAppBo.Work workBo, String categoryName) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        // 随报名上传作品信息
        MthWork work = new MthWork();
        if (workBo.getId() != null) {
            work = workRepo.getById(workBo.getId());
        } else {
            // 生成作品编码
            work.setWorkCode(getWorkCode(categoryName));
            // 配置id
            work.setRegisterConfigId(workBo.getRegisterConfigId());
        }
        Optional<MthRegisterAddAppBo.Detail> workName = workBo.getDetailList().stream()
                .filter(item -> StrUtil.equals(item.getKey(), "workName") && StrUtil.isNotBlank(item.getValue())).findFirst();
        if (workName.isPresent()) {
            if (workName.get().getValue().length() > 64) {
                throw new BusinessException("作品名称长度不能超过64位!");
            }
            work.setWorkName(workName.get().getValue());
        } else {
            // throw new BusinessException("作品名称不能为空!");
        }

        workRepo.saveAndFlush(work);
        // id存在修改, 不存在新增
        List<MthRegisterDetail> detailList = detailConverter.appBoToEntity(workBo.getDetailList());
        for (MthRegisterDetail detail : detailList) {
            detail.setBizId(work.getId());
            detail.setDelFlag(0);
            detail.setCreateId(Integer.valueOf(currWorker.getLoginId()).longValue());
            detail.setCreateBy(currWorker.getWorkerName());
            detail.setCreateTime(new Date());
        }
        detailRepo.saveAllAndFlush(detailList);
        return work.getId();
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean cancelRegister(MthCancelRegisterAppBo bo) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        MthRegister register = baseRepo.getById(bo.getRegisterId());
        if (ObjectUtil.notEqual(register.getUserId().intValue(), currWorker.getLoginId())) {
            // 非本人撤销报名
            throw new BusinessException("撤销报名失败! 非本人操作");
        }
        MthMatchInfo matchInfo = matchInfoRepo.getById(register.getMatchId());
        Integer registerationAuditFlag = matchInfo.getRegisterationAuditFlag();
        if (ObjectUtil.equals(registerationAuditFlag, 0)) {
            MthRoundList roundList = roundListService.queryCurrRoundList(register.getId());
            Long roundId = roundList.getRoundId();
            MthRoundInstance roundInstance = roundInstanceRepo.getById(roundId);
            Date registerationEnd = roundInstance.getRegisterationEnd();
            // 不需要审核, 判断是否到报名截止日
            if (registerationEnd == null || registerationEnd.before(new Date())) {
                throw new BusinessException("撤销报名失败! 报名已结束");
            }
        } else {
            // 需要审核, 判断是否待审核
            if (ObjectUtil.notEqual(register.getReviewResult(), RegisterAuditStatusEnums.PENDING.getCode())) {
                throw new BusinessException("撤销报名失败! 非待审核状态");
            }
        }
        register.setReviewResult(RegisterAuditStatusEnums.CANCEL.getCode());
        baseRepo.saveAndFlush(register);
        MthRoundList roundList = roundListService.queryCurrRoundList(bo.getRegisterId());
        roundList.setStatus(RoundListStatusEnums.END.getCode());
        roundListRepo.saveAndFlush(roundList);

        if(Objects.equals(matchInfo.getQcFlag(), 1)){
            // 青创赛事
            MthProject project = projectRepo.findByRegisterId(register.getId());
            MthProjectFinancing financing = financingRepo.findByProjectId(project.getId());
            if (financing != null) {
                financing.setReviewStatus(FinancingEnums.ReviewStatus.CANCEL.getCode());
                // 撤销融资申请
                financingRepo.saveAndFlush(financing);
            }
        }

        return true;
    }

    /**
     * 修改赛事-赛项-组别-区域报名信息;
     */
    @Override
    public Boolean updateByBo(MthRegisterBo bo) {
        MthRegister update = baseConverter.boToEntity(bo);
        validEntityBeforeSave(update);
        baseRepo.save(update);
        return true;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(MthRegister entity) {

    }

    /**
     * 批量删除赛事-赛项-组别-区域报名信息;
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if (isValid) {

        }
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        registerDao.deleteByIds(ids, currWorker.getLoginId(), currWorker.getWorkerName());
        return true;
    }

    /**
     * 修改赛事-赛项-组别-区域报名信息;
     */
    @Override
    public Boolean updateExamineBo(MthRegisterBo bo) {
        MthRegister register = baseRepo.getById(bo.getId());
        register.setReviewResult(bo.getReviewResult());
        register.setUpdateTime(new Date());
        validEntityBeforeSave(register);
        baseRepo.saveAndFlush(register);
        return true;
    }

    /**
     * 收藏或取消收藏赛事
     * @param matchCollectReqVo
     * @return
     */
    @Override
    public Result doCollectMatch(Worker worker,MatchCollectReqVo matchCollectReqVo) {
        if (ObjectUtil.isNotEmpty(matchCollectReqVo) && matchCollectReqVo.getMatchId()!=null){
            MthMatchInfo one = matchInfoRepo.getOne(matchCollectReqVo.getMatchId());
            //赛事是否存在
            if (ObjectUtil.isEmpty(one)){
                throw new BusinessException(MATCH_NOT_EXIST);
            }
            String message=null;
            List<MthMatchCollected> mthMatchCollecteds=mthMatchCollectedRepo.findByMatchId(matchCollectReqVo.getMatchId(),(long)worker.getLoginId());
            if (matchCollectReqVo.getType()!=null && matchCollectReqVo.getType().equals(1) && mthMatchCollecteds.size()==0){
                //收藏
                MthMatchCollected mthMatchCollected=new MthMatchCollected();
                mthMatchCollected.setMatchInfoId(matchCollectReqVo.getMatchId());
                mthMatchCollected.setUserId((long)worker.getLoginId());
                mthMatchCollected.setCreateId((long)worker.getLoginId());
                mthMatchCollected.setCreateBy(worker.getWorkerName());
                mthMatchCollected.setCreateTime(new Date());
                message=MATCH_COLLECT_SUCCESS;
                mthMatchCollectedRepo.save(mthMatchCollected);
            }
            //取消收藏
            if (matchCollectReqVo.getType()!=null && matchCollectReqVo.getType().equals(0) && mthMatchCollecteds.size()>0){
                for (MthMatchCollected mthMatchCollected : mthMatchCollecteds) {
                    mthMatchCollected.setDelFlag(1);
                    mthMatchCollected.setUpdateId((long)worker.getLoginId());
                    mthMatchCollected.setUpdateBy(worker.getWorkerName());
                    mthMatchCollected.setUpdateTime(new Date());
                }
                mthMatchCollectedRepo.saveAllAndFlush(mthMatchCollecteds);
                message=MATCH_COLLECT_CANCEL_FAILED;
            }
            return Result.ofSuccessMsg(message);
        }
        return Result.ofFail(REQUEST_ERROR);
    }

    /**
     * 分页查询已收藏赛事
     * @param worker
     * @param pageReq
     * @return
     */
    @Override
    public Result<PageRsp<MthMyMatchListAppVo>> pageCollectedMatch(Worker worker, PageReq<MatchMyCollectBo> pageReq) {
        PageRsp<MthMyMatchListAppVo> matchs = registerDao.pageCollectedMatch(worker, pageReq);
        List<MthMyMatchListAppVo> matchListAppVos = matchs.getResult();
        //设置文件url地址
        obsFilService.setObsFileBoByIds(matchListAppVos, MthMyMatchListAppVo::getBannerId, MthMyMatchListAppVo::setBanner);
        if (ObjectUtil.isNotEmpty(matchListAppVos)) {
            List<Long> matchIds = CollUtil.map(matchListAppVos, MthMyMatchListAppVo::getId, true);
            for (MthMyMatchListAppVo item : matchListAppVos) {
                // 获取赛事状态
                item.setStatusDesc(MatchInfoStatusEnums.getAppStatusDesc(item.getStatus()));
            }
        }
        return Result.ofSuccess(matchs);
    }

    /**
     * APP赛事活动-赛事详情-查看通知公告
     */
    @Override
    public List<MatchArticleRspVo> queryMatchArticleByMatchId(Long matchId) {
        MthMatchInfo matchInfo = matchInfoRepo.getOne(matchId);
        if (ObjectUtil.isEmpty(matchInfo)){
            throw new BusinessException(MATCH_NOT_EXIST);
        }
        List<MatchArticleRspVo> matchArticleRspVos = registerDao.queryMatchArticleByMatchId(matchId);
        matchArticleRspVos.forEach(e->e.setBannerId(matchInfo.getBannerId()));
        obsFilService.setObsFileBoByIds(matchArticleRspVos, MatchArticleRspVo::getBannerId, MatchArticleRspVo::setFileInfoBo);
        return matchArticleRspVos;
    }

    /**
     * 赛事app-下载评语图片
     * @return 评语加签名合并后的图片(base64格式)
     */
    @Override
    public String doDownSignPicture(MatchDownPictureReqVO reqVO) throws IOException {
        if (StringUtils.isEmpty(reqVO.getSignFile()) || StringUtils.isEmpty(reqVO.getComments())) {
            throw new BusinessException(MATCH_SIGN_UNFINISHED);
        }
        Result<DownloadFileVo> downloadFileVoResult = obsService.doDownloadFile(Long.valueOf(reqVO.getSignFile()));
        if (!downloadFileVoResult.isSuccess() || null == downloadFileVoResult.getData().getFile()) {
            throw new BusinessException(MATCH_FILE_DOWNLOAD_ERROR);
        }
        String file = downloadFileVoResult.getData().getFile();
        //将评语写入一张新建的图片
        //评语相关参数
        int maxWidth = reqVO.getWidth();
        int fontSize = reqVO.getFontSize();
        //拿到单行字体高度
        Font fontOne = new Font(PingFang_SC, Font.BOLD, fontSize);
        FontDesignMetrics metricsOne = FontDesignMetrics.getMetrics(fontOne);
        int commitHeight = metricsOne.getHeight();
        //拿到总宽度计算行数
        int commitWidth = metricsOne.stringWidth(reqVO.getComments());
        int lineTotal = commitWidth / maxWidth + (commitWidth % maxWidth == 0 ? 0 : 1) + 3;//预留三行高度放签名
        //评语框总高度
        int totalHeight = lineTotal * commitHeight;
        //用画笔将背景刷成白色
        BufferedImage textImage = new BufferedImage(maxWidth, totalHeight, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = (Graphics2D) textImage.getGraphics();
        g2.setBackground(Color.WHITE);//设置背景色
        g2.clearRect(0, 0, maxWidth, totalHeight);//通过使用当前绘图表面的背景色进行填充来清除指定的矩形。
        g2.setPaint(Color.BLACK);//设置画笔,设置Paint属性
        g2.drawOval(0, 50, 0, 0);
        FontRenderContext context = g2.getFontRenderContext();
        // 文本抗锯齿
        g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        //消除边缘抗锯齿
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        int x = 10;//边距10px
        int y;
        String text = reqVO.getComments();
        JLabel label = new JLabel(text);
        //设置字体相关参数
        Font font = new Font(PingFang_SC, Font.BOLD, fontSize);
        LineMetrics lineMetrics = font.getLineMetrics(text, context);
        y = (int) lineMetrics.getHeight();
        label.setFont(font);
        FontMetrics metrics = label.getFontMetrics(label.getFont());
        int textH = metrics.getHeight();
        int textW = metrics.stringWidth(label.getText()); //字符串的宽
        String tempText = text;
        //将评语分行写入
        while (textW > maxWidth) {
            int n = textW / maxWidth;
            int subPos = tempText.length() / n;
            String drawText = tempText.substring(0, subPos);
            int subTxtW = metrics.stringWidth(drawText);
            while (subTxtW > maxWidth) {
                subPos--;
                drawText = tempText.substring(0, subPos);
                subTxtW = metrics.stringWidth(drawText);
            }
            g2.drawString(drawText, x, y);
            y += textH;
            textW = textW - subTxtW;
            tempText = tempText.substring(subPos);
        }
        //写入最后一行评语
        g2.drawString(tempText, x, y);
        g2.dispose();

        //缩小签名图片
        try {
            // 解密
            Base64.Decoder decoder = Base64.getDecoder();
            // 去掉base64前缀 data:image/jpeg;base64,
            file = file.substring(file.indexOf(",", 1) + 1);
            byte[] bytes1 = decoder.decode(file);
            // 处理数据
            for (int i = 0; i < bytes1.length; ++i) {
                if (bytes1[i] < 0) {
                    bytes1[i] += 256;
                }
            }

            ByteArrayInputStream bais = new ByteArrayInputStream(bytes1);
            BufferedImage read = ImageIO.read(bais);
            int newHeight = commitHeight*3;
            int newWidth=maxWidth * newHeight/totalHeight;
            //将签名图片生成略缩图
            BufferedImage newImage = shrinkImage(read, newWidth, newHeight);
            int shrinkWidth=textImage.getWidth()-newImage.getWidth()-10;//边距10px
            int shrinkHeight=textImage.getHeight()-newImage.getHeight();
            //签名写入评语右下角
            Graphics2D graphics = textImage.createGraphics();
            g2.setBackground(Color.WHITE);//设置背景色
            g2.clearRect(0, 0,shrinkWidth,shrinkHeight);
            graphics.drawImage(newImage,shrinkWidth,shrinkHeight,null);
            graphics.dispose();
            //将图片转base64并上传服务器，返给前端URL
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ImageIO.write(textImage, "png", baos);
            baos.flush();
            byte[] imageInByte = baos.toByteArray();
            baos.close();
            String base64String = Base64.getEncoder().encodeToString(imageInByte);
            FileBase64 fileBase64 = new FileBase64();
            fileBase64.setImgBase64(PNG_BASE64 + base64String);
            fileBase64.setFilePath("/");
            Result<FileVo> fileVoResult = obsService.appFileUpload(fileBase64);
        if (!fileVoResult.isSuccess()){
            throw new BusinessException(MATCH_FILE_UPLOAD_ERROR);
        }
        return fileVoResult.getData().getFileUrl();
    } catch (IOException e) {
        e.printStackTrace();
    }
        return null;
    }

    /**
     * 缩小图片
     * @param bufferedImage 图片
     * @param targetWidth 缩小后的宽度
     * @param targetHeight 缩小后的高度
     * @return
     * @throws IOException
     */
    public BufferedImage shrinkImage(BufferedImage bufferedImage,int targetWidth, int targetHeight) throws IOException {
        BufferedImage image = Thumbnails.of(bufferedImage)
                .size(targetWidth, targetHeight)
                .outputFormat("JPEG")
                .outputQuality(1)
                .asBufferedImage();
        return image;
    }
}
