package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.common.utils.StreamUtils;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.converter.MthRoundAreaConverter;
import com.crunii.micro.service.match.converter.MthRoundInstanceConverter;
import com.crunii.micro.service.match.converter.MthRoundPrizeConverter;
import com.crunii.micro.service.match.converter.MthRoundReviewConverter;
import com.crunii.micro.service.match.enums.MatchEnums;
import com.crunii.micro.service.match.enums.MatchInfoStatusEnums;
import com.crunii.micro.service.match.enums.RoundEnums;
import com.crunii.micro.service.match.jdbc.MthMatchGroupDao;
import com.crunii.micro.service.match.jdbc.MthMatchItemDao;
import com.crunii.micro.service.match.jdbc.MthRoundInstanceDao;
import com.crunii.micro.service.match.jdbc.MthRoundScoreDao;
import com.crunii.micro.service.match.service.*;
import com.crunii.micro.service.sysmgr.api.domain.dto.OrgSimpleVO;
import com.google.common.collect.Maps;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.assertj.core.util.Lists;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static com.crunii.micro.service.match.constants.MatchMessageConstants.*;

/**
 * 赛事-区域-场次实例（实例与配置的关系）;Service业务层处理
 *
 * @author mth
 * @date 2023-09-19
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class MthRoundInstanceServiceImpl implements IMthRoundInstanceService {

    private final MthRoundInstanceRepo baseRepo;
    private final MthRoundInstanceConverter baseConverter;
    private final MthRoundPrizeConverter roundPrizeConverter;
    private final MthRoundPrizeRepo roundPrizeRepo;
    private final MthRoundAreaConverter roundAreaConverter;
    private final MthRoundAreaRepo roundAreaRepo;
    private final MthMatchInfoRepo mthMatchInfoRepo;
    private final MthRoundListRepo mthRoundListRepo;
    private final MthRoundReviewStandarRepo mthRoundReviewStandarRepo;
    private final MthRoundReviewConverter mthRoundReviewConverter;
    private final IMthRoundReviewStandarService  iMthRoundReviewStandarService;
    private final MthRoundReviewerRepo mthRoundReviewerRepo;
    private final PushBusinessMsgService pushBusinessMsgService;
    private final MthRoundScoreDao roundScoreDao;
    @Resource
    @Lazy
    private IMthRoundListService iMthRoundListService;

    private final MthRoundInstanceDao roundInstanceDao;

    private final MthRegisterConfigRepo registerConfigRepo;
    private final MthMatchGroupDao mthMatchGroupDao;
    private  final MthMatchItemDao mthMatchItemDao;
    private final  MthMatchSignInService mthMatchSignInService;
    private final MthMatchSignInformationRepo mthMatchSignInformationRepo;


    @Resource
    private TsmOrgRepo tsmOrgRepo;

    /**
     * 查询赛事-区域-场次实例（实例与配置的关系）;
     */
    @Override
    public MthRoundInstanceVo queryById(Long id) {
        return baseConverter.entityToVo(baseRepo.getById(id));
    }

    @Override
    public MthRoundInstanceVo getMthRoundInstanceById(Long id) {
        MthRoundInstance roundInstance = baseRepo.getOne(id);
        MthRoundInstanceVo vo = baseConverter.entityToVo(roundInstance);
        // 名次信息
        List<MthRoundPrize> prizeList = roundPrizeRepo.findByRoundIdAndDelFlag(id, 0);
        if (CollectionUtil.isNotEmpty(prizeList)) {
            vo.setPrizeVoList(roundPrizeConverter.entitryListToVoList(prizeList));
        }
        // 查询场次对应的评分标准
        List<MthRoundReviewStandar> mthRoundReviewStandarList = mthRoundReviewStandarRepo.findAllByRoundId(id);
        vo.setMthRoundReviewStandarBoList(mthRoundReviewConverter.entitryListToVoList(mthRoundReviewStandarList));

        // 查询当前场次所属比赛等级的第一场区域配置
        List<MthRoundAreaVo> areaList = this.getFirstRoundAreaByMatchIdAndMatchLevel(roundInstance.getMatchId(), roundInstance.getMatchLevel());
        if (CollectionUtil.isNotEmpty(areaList)) {
            vo.setAreavoList(areaList);
            if (RoundEnums.AreaLevel.DISTRICT.getCode().equals(roundInstance.getMatchLevel())) {
                // 查询片区级下选择的区县/校赛级
                List<Long> idList = StreamUtils.toList(areaList, MthRoundAreaVo::getId);
                List<MthRoundArea> childrenList = roundAreaRepo.findByParentIdIn(idList);
                if (CollectionUtil.isNotEmpty(childrenList)) {
                    Map<Long, List<MthRoundArea>> childrenMap = StreamUtils.groupByKey(childrenList, MthRoundArea::getParentId);
                    for (MthRoundAreaVo roundAreaVo : areaList) {
                        List<MthRoundArea> children = childrenMap.get(roundAreaVo.getId());
                        if (CollectionUtil.isNotEmpty(children)) {
                            List<Long> childrenIds = StreamUtils.toList(children, MthRoundArea::getId);
                            roundAreaVo.setChildrenIds(childrenIds);
                        }
                    }
                }
            }
        }

        //查询签到签退信息
        MthMatchSignInformation signIn = mthMatchSignInformationRepo.findByInstanceIdAndHandle(roundInstance.getId(), 1);
        MthMatchSignInformation signOut = mthMatchSignInformationRepo.findByInstanceIdAndHandle(roundInstance.getId(), -1);
        SignInfoVo signInInfoVo = new SignInfoVo();
        SignInfoVo signOutInfoVo = new SignInfoVo();
        //签到信息
        if (ObjectUtil.isNotEmpty(signIn)){
            BeanUtil.copyProperties(signIn,signInInfoVo);
            vo.setSignInInfoVo(signInInfoVo);

        }
        //签退信息
        if (ObjectUtil.isNotEmpty(signOut)){
            BeanUtil.copyProperties(signOut,signOutInfoVo);
            vo.setSignOutInfoVo(signOutInfoVo);
        }

        return vo;
    }

    @Override
    public List<MthRoundAreaVo> getFirstRoundAreaByMatchIdAndMatchLevel(Long matchId, String matchLevel) {
        // 获取比赛等级的第一场次模板
        List<MthRoundInstance> roundList = baseRepo.findByMatchIdAndRoundTypeAndMatchLevelAndRoundSort(matchId,
                RoundEnums.RoundType.TEMPLATE.getCode(), matchLevel, 1);
        if (CollectionUtil.isNotEmpty(roundList)) {
            Long roundId = roundList.get(0).getId();
            // 获取场次模板配置区域
            List<MthRoundArea> roundAreaList = roundAreaRepo.findByRoundId(roundId);
            if (CollectionUtil.isNotEmpty(roundAreaList)) {
                return roundAreaConverter.entityListToVoList(roundAreaList);
            }
        }
        return Collections.emptyList();
    }

    @Override
    public List<OrgSimpleVO> getChildrenByParentOrgIds(List<Long> parentOrgIds) {
        List<TsmOrg> childrenList = tsmOrgRepo.findByParentOrgIdInAndState(parentOrgIds, 1);
        List<OrgSimpleVO> childrenVoList = childrenList.stream().map(item -> {
            OrgSimpleVO vo = new OrgSimpleVO();
            vo.setOrgId(item.getOrgId());
            vo.setOrgCode(item.getOrgCode());
            vo.setOrgName(item.getOrgName());
            vo.setParentOrgId(item.getParentOrgId());
            return vo;
        }).collect(Collectors.toList());
        return childrenVoList;
    }

    @Override
    public List<MthRoundInstanceVo> getAllRoundInfoByMatchId(Long matchId) {
        List<MthRoundInstanceVo> voList = Lists.newArrayList();
        Map<String, Object> props = Maps.newHashMap();
        props.put("matchId", matchId);
        props.put("delFlag", 0);
        props.put("roundType", "template");
        //查询比赛下的所有场次
        List<MthRoundInstance> roundInstanceList = baseRepo.findByMatchIdAndRoundType(matchId, RoundEnums.RoundType.TEMPLATE.getCode());
        if (CollectionUtil.isNotEmpty(roundInstanceList)) {
            MthRoundInstanceVo vo = new MthRoundInstanceVo();
            List<Long> roundIdList = roundInstanceList.stream().map(MthRoundInstance::getId).collect(Collectors.toList());
            //查询所有场次对应的区域
            List<MthRoundArea> roundAreaList = roundAreaRepo.findByRoundIdIn(roundIdList);
            Map<Long, List<MthRoundArea>> roundAreaMap = new HashMap<Long, List<MthRoundArea>>();
            if (CollectionUtil.isNotEmpty(roundAreaList)) {
                roundAreaMap = roundAreaList.stream().collect(Collectors.groupingBy(MthRoundArea::getRoundId));

            }
            //查询奖项
            List<MthRoundPrize> roundPrizeList = roundPrizeRepo.findByRoundIdIn(roundIdList);
            Map<Long, List<MthRoundPrize>> roundPrizeMap = new HashMap<Long, List<MthRoundPrize>>();
            if (CollectionUtil.isNotEmpty(roundPrizeList)) {
                roundPrizeMap = roundPrizeList.stream().collect(Collectors.groupingBy(MthRoundPrize::getRoundId));
            }

            for (MthRoundInstance roundInstance : roundInstanceList) {
                vo = baseConverter.entityToVo(roundInstance);
                if (roundAreaMap.containsKey(roundInstance.getId())) {
                    vo.setAreavoList(roundAreaConverter.entityListToVoList(roundAreaMap.get(roundInstance.getId())));
                }
                if (roundPrizeMap.containsKey(roundInstance.getId())) {
                    vo.setPrizeVoList(roundPrizeConverter.entitryListToVoList(roundPrizeMap.get(roundInstance.getId())));
                }
                voList.add(vo);

            }


        }

        return voList;
    }


    @Override
    public List<MthRoundInstanceVo> getAllRoundInfoByMatchIdAndAreaId(Long matchId, Long areaId) {
        MthMatchInfo matchInfo = mthMatchInfoRepo.getById(matchId);
        List<MthRoundInstanceVo> voList = Lists.newArrayList();
        List<MthRoundInstance> roundInstanceList;
        if (MatchInfoStatusEnums.templateList().contains(matchInfo.getStatus())) {
            MthRoundArea roundArea = roundAreaRepo.getById(areaId);
            MthRoundInstance firstRound = baseRepo.getById(roundArea.getRoundId());
            String matchLevel = firstRound.getMatchLevel();
            // 查询模板-所有比赛下的所有场次
            roundInstanceList = baseRepo.findByMatchIdAndRoundTypeAndMatchLevel(matchId, RoundEnums.RoundType.TEMPLATE.getCode(), matchLevel);
            for (MthRoundInstance roundInstance : roundInstanceList) {
                roundInstance.setAreaName(roundArea.getAreaName());
            }
        } else {
            // 查询实例-所有比赛下的所有场次
            roundInstanceList = baseRepo.findByMatchIdAndRoundTypeAndAreaId(matchId, RoundEnums.RoundType.INSTANCE.getCode(), areaId);
        }

        if (CollectionUtil.isNotEmpty(roundInstanceList)) {
            List<Long> roundIdList = roundInstanceList.stream().map(MthRoundInstance::getId).collect(Collectors.toList());
            //查询奖项
            List<MthRoundPrize> roundPrizeList = roundPrizeRepo.findByRoundIdIn(roundIdList);
            Map<Long, List<MthRoundPrize>> roundPrizeMap = Collections.emptyMap();
            if (CollectionUtil.isNotEmpty(roundPrizeList)) {
                roundPrizeMap = roundPrizeList.stream().collect(Collectors.groupingBy(MthRoundPrize::getRoundId));
            }

            //查询所有场次对应的评分标准
            List<MthRoundReviewStandar> mthRoundReviewStandarList = mthRoundReviewStandarRepo.findByRoundIdIn(roundIdList);
            Map<Long, List<MthRoundReviewStandar>> reviewStandarMap = new HashMap<>();

            if (!CollectionUtil.isEmpty(mthRoundReviewStandarList)) {
                reviewStandarMap = mthRoundReviewStandarList.stream().collect(Collectors.groupingBy(MthRoundReviewStandar::getRoundId));
            }

            //查询评审员
            List<MthRoundReviewer> mthRoundReviewerList = mthRoundReviewerRepo.findByRoundIdIn(roundIdList);
            Map<Long, List<MthRoundReviewer>> mthRoundReviewerMap = new HashMap<>();
            if (!CollectionUtil.isEmpty(mthRoundReviewerList)) {
                mthRoundReviewerMap = mthRoundReviewerList.stream().collect(Collectors.groupingBy(MthRoundReviewer::getRoundId));
            }
            for (MthRoundInstance roundInstance : roundInstanceList) {
                MthRoundInstanceVo vo = baseConverter.entityToVo(roundInstance);
                if (roundPrizeMap.containsKey(roundInstance.getId())) {
                    vo.setPrizeVoList(roundPrizeConverter.entitryListToVoList(roundPrizeMap.get(roundInstance.getId())));
                }
                //评分标准
                if (reviewStandarMap.containsKey(roundInstance.getId())) {
                    vo.setMthRoundReviewStandarBoList(mthRoundReviewConverter.entitryListToVoList(reviewStandarMap.get(roundInstance.getId())));
                }
                //是否设置评审员
                if (mthRoundReviewerMap.containsKey(roundInstance.getId())) {
                    vo.setReviewer(0);
                } else {
                    vo.setReviewer(1);
                }
                SignInfoVo signInInfoVo = new SignInfoVo();
                SignInfoVo signOutInfoVo = new SignInfoVo();
                //查询签到签退信息
                MthMatchSignInformation signIn = mthMatchSignInformationRepo.findByInstanceIdAndHandle(roundInstance.getId(), 1);
                MthMatchSignInformation signOut = mthMatchSignInformationRepo.findByInstanceIdAndHandle(roundInstance.getId(), -1);
                //是否设置评审员
                if (mthRoundReviewerMap.containsKey(roundInstance.getId())){
                    vo.setReviewer(0);
                }else {
                    vo.setReviewer(1);
                }
                //签到信息
                if (ObjectUtil.isNotEmpty(signIn)){
                    BeanUtil.copyProperties(signIn,signInInfoVo);
                    vo.setSignInInfoVo(signInInfoVo);

                }
                //签退信息
                if (ObjectUtil.isNotEmpty(signOut)){
                    BeanUtil.copyProperties(signOut,signOutInfoVo);
                    vo.setSignOutInfoVo(signOutInfoVo);
                }
                voList.add(vo);
            }
        }

        return voList;
    }

    @Override
    public List<MthMatchAreaVo> queryArea(Long matchId) {
        // 查询赛事区域以及参与人数和晋级人数
        List<MthMatchAreaVo> areaVoList = roundInstanceDao.queryTemplateArea(matchId);
        // 根据区域分组
        Map<String, List<MthMatchAreaVo>> matchLevelMap = StreamUtils.groupByKey(areaVoList, MthMatchAreaVo::getMatchLevel);
        List<MthMatchAreaVo> areaGroupList = new ArrayList<>();
        for (Map.Entry<String, List<MthMatchAreaVo>> entry : matchLevelMap.entrySet()) {
            MthMatchAreaVo group = new MthMatchAreaVo();
            group.setAreaName(RoundEnums.AreaLevel.getAreaByCode(entry.getKey()).getDesc());
            group.setChildren(entry.getValue());
            areaGroupList.add(group);
        }
        return areaGroupList;
    }

    /**
     * 查询赛事-区域-场次实例（实例与配置的关系）;列表
     */
    @Override
    public Result<PageRsp<MthRoundInstanceVo>> queryPageList(PageReq<MthRoundInstanceBo> bo) {
        return null;
    }

    /**
     * 查询赛事-区域-场次实例（实例与配置的关系）;列表
     */
    @Override
    public List<MthRoundInstanceVo> queryList(MthRoundInstanceBo bo) {
        return null;
    }

    /**
     * 新增赛事-区域-场次实例（实例与配置的关系）;
     */
    @Override
    public Long insertByBo(MthRoundInstanceBo bo) {
        MthRoundInstance add = baseConverter.boToEntity(bo);
        validEntityBeforeSave(add);

        //校验签到签退信息
        SignInfoVo signInInfoVo = bo.getSignInInfoVo();
        SignInfoVo signOutInfoVo = bo.getSignOutInfoVo();
        if ("offline".equals(bo.getPlayMode())){
            validSignInformation(signInInfoVo);
            validSignInformation(signOutInfoVo);
        }

        add.setRoundType(RoundEnums.RoundType.TEMPLATE.getCode());
        if (add.getRegisterationEnd()==null||add.getReviewerStandar()==null) {
            //将比赛的报名开始结束时间赋给场次
            MthMatchInfo matchInfo = mthMatchInfoRepo.getOne(add.getMatchId());
            add.setRegisterationStart(matchInfo.getRegisterationStart());
            add.setRegisterationEnd(matchInfo.getRegisterationEnd());
        }
        if (StrUtil.isNotBlank(add.getMatchLevel())) {
            RoundEnums.AreaLevel areaLevel = RoundEnums.AreaLevel.getAreaByCode(add.getMatchLevel());
            add.setRoundLevel(areaLevel.getLevel());
        }
        baseRepo.save(add);
        saveRoundInfo(add.getId(), bo);
        // 保存评审标准
        svaeIMthRoundReviewStandar(bo.getId(), bo);

        //保存签到与签退信息
        if (ObjectUtil.isNotEmpty(signInInfoVo)){
            signInInfoVo.setInformationType(1);
            mthMatchSignInService.saveSignInformation(signInInfoVo);
        }
        if (ObjectUtil.isNotEmpty(signOutInfoVo)){
            signOutInfoVo.setInformationType(-1);
            mthMatchSignInService.saveSignInformation(signOutInfoVo);
        }

        // 更新赛事同等级场次序号-自动排序
        updateRoundSort(bo.getMatchId(), bo.getMatchLevel());
        return add.getId();
    }

    @Override
    public void updateRoundSort(Long matchId, String matchLevel) {
        List<MthRoundInstance> levelRoundList = baseRepo.findByMatchIdAndRoundTypeAndMatchLevel(matchId, RoundEnums.RoundType.TEMPLATE.getCode(), matchLevel);
        updateRoundSort(levelRoundList);
    }
    private void updateRoundSort(List<MthRoundInstance> levelRoundList){
        if(levelRoundList.size() > 1) {
            // 场次序号从2开始
            AtomicInteger roundSort = new AtomicInteger(2);
            // 修改场次序号不等于1的自动排序
            levelRoundList.stream().filter(item -> item.getRoundSort() == null || item.getRoundSort() != 1).sorted(Comparator.comparing(MthRoundInstance::getRoundSort))
                    .forEach(item -> item.setRoundSort(roundSort.getAndIncrement()));
            baseRepo.saveAll(levelRoundList);
        }
    }
    private void updateRoundInstanceSort(Long matchId,Long areaId){
        List<MthRoundInstance> areaRoundList = baseRepo.findByMatchIdAndRoundTypeAndAreaId(matchId, RoundEnums.RoundType.INSTANCE.getCode(), areaId);
        updateRoundSort(areaRoundList);
    }

    @Override
    public Long saveRoundDraft(MthRoundInstanceBo bo) {
        if (Objects.isNull(bo.getId())) {
            return insertByBo(bo);
        }
        updateByBo(bo);
        return bo.getId();
    }

    private void saveRoundInfo(Long roundId, MthRoundInstanceBo bo) {
        // 场次序号为1才能新增或修改区域
        if (Objects.equals(bo.getRoundSort(), 1) && CollectionUtil.isNotEmpty(bo.getAreaBoList())) {
            if (Objects.equals(bo.getMatchLevel(), RoundEnums.AreaLevel.COUNTY_SCHOOL.getCode())) {
                List<Long> areaIdList = StreamUtils.toList(bo.getAreaBoList(), MthRoundAreaBo::getAreaId);
                List<MthRoundArea> roundAreaList = roundAreaRepo.findByRoundIdAndAreaIdIn(roundId, areaIdList);
                Map<Long, MthRoundArea> roundAreaMap = StreamUtils.toIdentityMap(roundAreaList, MthRoundArea::getAreaId);
                for (MthRoundAreaBo roundAreaBo : bo.getAreaBoList()) {
                    MthRoundArea roundArea = roundAreaMap.get(roundAreaBo.getAreaId());
                    if (roundArea != null) {
                        // 区域id存在, 删除前将原parentId数据保留
                        roundAreaBo.setParentId(roundArea.getParentId());
                    }
                }
            }
            // 删除区域
            roundAreaRepo.deleteByRoundId(bo.getId());
            for (MthRoundAreaBo areaBo : bo.getAreaBoList()) {
                MthRoundArea area = roundAreaConverter.boToEntity(areaBo);
                area.setId(null);
                area.setRoundId(roundId);
                roundAreaRepo.save(area);
                if (ObjectUtil.isNotEmpty(areaBo.getChildrenIds())) {
                    Long parentId = area.getId();
                    List<MthRoundArea> children = roundAreaRepo.findByIdIn(areaBo.getChildrenIds());
                    children.forEach(item -> item.setParentId(parentId));
                    roundAreaRepo.saveAll(children);
                }
            }
        } else {
            // 删除区域
            roundAreaRepo.deleteByRoundId(bo.getId());
        }
     /*   if (CollectionUtil.isNotEmpty(bo.getPrizeBoList())) {
            for (MthRoundPrizeBo prizeBo : bo.getPrizeBoList()) {
                MthRoundPrize prize = roundPrizeConverter.boToEntity(prizeBo);
                prize.setRoundId(roundId);
                roundPrizeRepo.save(prize);
            }
        }*/
        savePrizeBo(roundId,bo);
    }

    /**
     * 保存名次配置
     */
    private void savePrizeBo(Long roundId, MthRoundInstanceBo bo){
        if (CollectionUtil.isNotEmpty(bo.getPrizeBoList())) {
            List<MthRoundPrizeBo> prizeBoList=bo.getPrizeBoList();
            if(!RoundEnums.PrizeType.MATCH.getCode().equals(bo.getPrizeType())){
                //赛项
                List<MatchBaseItemVo> itemList = mthMatchItemDao.getItemList(bo.getMatchId());
                Map<Long, String> collect = itemList.stream().collect(Collectors.toMap(MatchBaseItemVo::getId, MatchBaseItemVo::getItemName));
                if (RoundEnums.PrizeType.ITEM.getCode().equals(bo.getPrizeType())){
                    prizeBoList.forEach(prize->{
                        prize.setItemName(collect.get(prize.getItemId()));
                    });

                }else {
                    //组别
                    List<MatchBaseGroupVo> groups = mthMatchGroupDao.getGroups(bo.getMatchId());
                    Map<Long, String> group = groups.stream().collect(Collectors.toMap(MatchBaseGroupVo::getId, MatchBaseGroupVo::getGroupName));
                    prizeBoList.forEach(prize->{
                        prize.setItemName(collect.get(prize.getItemId()));
                        prize.setGroupName(group.get(prize.getGroupId()));

                    });
                }
            }
            List<MthRoundPrize> save=prizeBoList.stream().map(item->{
                MthRoundPrize prize = roundPrizeConverter.boToEntity(item);
                prize.setRoundId(roundId);
                return prize;
            }).collect(Collectors.toList());
                roundPrizeRepo.saveAllAndFlush(save);

        }
    }

    /**
     * 修改赛事-区域-场次实例（实例与配置的关系）;
     */
    @Override
    public Boolean updateByBo(MthRoundInstanceBo bo) {
        MthRoundInstance update = baseConverter.boToEntity(bo);
        if (update.getRegisterationEnd()==null||update.getReviewerStandar()==null) {
            //将比赛的报名开始结束时间赋给场次
            MthMatchInfo matchInfo = mthMatchInfoRepo.getOne(update.getMatchId());
            update.setRegisterationStart(matchInfo.getRegisterationStart());
            update.setRegisterationEnd(matchInfo.getRegisterationEnd());
        }
        if (StrUtil.isNotBlank(update.getMatchLevel())) {
            RoundEnums.AreaLevel areaLevel = RoundEnums.AreaLevel.getAreaByCode(update.getMatchLevel());
            update.setRoundLevel(areaLevel.getLevel());
        }
        validEntityBeforeSave(update);

        //校验签到签退信息
        SignInfoVo signInInfoVo = bo.getSignInInfoVo();
        SignInfoVo signOutInfoVo = bo.getSignOutInfoVo();
        if ("offline".equals(bo.getPlayMode())){
            validSignInformation(signInInfoVo);
            validSignInformation(signOutInfoVo);
        }

        MthRoundInstance mthRoundInstance = baseRepo.getOne(bo.getId());
        BeanUtil.copyProperties(update, mthRoundInstance, CopyOptions.create().setIgnoreNullValue(true));
        baseRepo.saveAndFlush(mthRoundInstance);
        //删除奖项
        roundPrizeRepo.deleteMthRoundPrizesByRoundId(bo.getId());

        // 保存区域
        saveRoundInfo(bo.getId(), bo);
        // 保存评审标准
        svaeIMthRoundReviewStandar(bo.getId(), bo);
        //保存签到与签退信息
        if (ObjectUtil.isNotEmpty(signInInfoVo)){
            signInInfoVo.setInformationType(1);
            signInInfoVo.setInstanceId(mthRoundInstance.getId());
            mthMatchSignInService.saveSignInformation(signInInfoVo);
        }
        if (ObjectUtil.isNotEmpty(signOutInfoVo)){
            signOutInfoVo.setInformationType(-1);
            signInInfoVo.setInstanceId(mthRoundInstance.getId());
            mthMatchSignInService.saveSignInformation(signOutInfoVo);
        }
        // 更新赛事同等级场次序号-自动排序
        updateRoundSort(bo.getMatchId(), bo.getMatchLevel());

        return true;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(MthRoundInstance entity) {
        //验证名称是否重复
        if (StrUtil.isNotBlank(entity.getRoundName())) {
            List<MthRoundInstance> list = baseRepo.findByMatchIdAndMatchLevelAndRoundNameLikeAndIdIsNot(entity.getMatchId(),
                    entity.getMatchLevel(), entity.getRoundName(), ObjectUtil.isNull(entity.getId()) ? 0L : entity.getId());
            if (CollectionUtil.isNotEmpty(list)) {
                throw new BusinessException("场次名称重复");
            }
        }
        //验证场次序号是否重复
        if (ObjectUtil.isNotNull(entity.getRoundSort())) {
            if (entity.getRoundSort() < 1) {
                throw new BusinessException("场次序号不能小于1");
            }
            List<MthRoundInstance> sortList = baseRepo.findByMatchIdAndMatchLevelAndRoundSortAndIdIsNot(entity.getMatchId(),
                    entity.getMatchLevel(), entity.getRoundSort(), ObjectUtil.isNull(entity.getId()) ? 0L : entity.getId());
            if (CollectionUtil.isNotEmpty(sortList)) {
                throw new BusinessException("场次序号重复");
            }
        }
    }

    /**
     * 获取赛事最低比赛等级
     * @param level
     * @return
     */
    private  String getMatchLevel(String level){
        if (org.apache.commons.lang3.StringUtils.isBlank(level)) {
            return null;
        }
        String[] split = level.split(",");
        List<RoundEnums.AreaLevel> matchLevel = Arrays.stream(split).map(item -> RoundEnums.AreaLevel.getAreaByCode(item)).sorted(Comparator.comparing(RoundEnums.AreaLevel::getLevel)).collect(Collectors.toList());
        String Levelcode = matchLevel.get(0).getCode();
        return Levelcode;
    }

    /**
     * 保存前的数据校验
     */
    private void validContractorEntityBeforeSave(MthRoundInstance entity,boolean sortFlag) {
        if (ObjectUtil.isNull(entity.getRoundName())) {
            throw new BusinessException("场次名称为空");
        }
        //验证名称是否重复
        if (StrUtil.isNotBlank(entity.getRoundName())) {
            List<MthRoundInstance> list = baseRepo.findByMatchIdAndRoundTypeAndAreaIdAndRoundNameLikeAndIdIsNot(entity.getMatchId(),RoundEnums.RoundType.INSTANCE.getCode(),
                    entity.getAreaId(), entity.getRoundName(), ObjectUtil.isNull(entity.getId()) ? 0L : entity.getId());
            if (CollectionUtil.isNotEmpty(list)) {
                throw new BusinessException("场次名称重复");
            }
        }

        MthMatchInfo mthMatchInfo = mthMatchInfoRepo.findById(entity.getMatchId()).orElseThrow(() -> new BusinessException("系统错误，未查询到该赛事"));
         if (ObjectUtil.isNotNull(entity.getRegisterationEnd())&&ObjectUtil.isNotNull(entity.getRegisterationStart())) {
             if (mthMatchInfo.getRegisterationStart().compareTo(entity.getRegisterationStart()) > 0 || mthMatchInfo.getRegisterationEnd().compareTo(entity.getRegisterationEnd()) < 0) {
                 throw new BusinessException("报名时间不在比赛报名时间范围内！");
             }
         }
        //新增不校验序号
        if (sortFlag) {
            if (ObjectUtil.isNull(entity.getRoundSort())) {
                throw new BusinessException("场次序号不能为空");
            }
            if (entity.getRoundSort() < 1) {
                throw new BusinessException("场次序号不能小于1");
            }
            List<MthRoundInstance> sortList = baseRepo.findByMatchIdAndAreaIdAndRoundSortAndIdIsNot(entity.getMatchId(),
                    entity.getAreaId(), entity.getRoundSort(), ObjectUtil.isNull(entity.getId()) ? 0L : entity.getId());
            if (CollectionUtil.isNotEmpty(sortList)) {
                throw new BusinessException("场次序号重复");
            }
        }
        if (ObjectUtil.isNotNull(entity.getPublicityFlag())){
            if (entity.getPublicityFlag()==1){
                if (ObjectUtil.isNull(entity.getPublicityDays())){
                    throw new BusinessException("未设置公式天数");
                }
            }
        }else {
            throw new BusinessException("是否公示不能为空！");
        }


        if (StringUtils.isBlank(entity.getPlayMode())) {
            throw new BusinessException("比赛方式为空");
        }
        if (StringUtils.isBlank(entity.getAuditMode())) {
            throw new BusinessException("评审方式为空");
        }
        if (RoundEnums.LineEnum.ONLINE.getCode().equals(entity.getAuditMode())) {
            if (ObjectUtil.isNull(entity.getSocreMode())) {
                throw new BusinessException("评分方式为空");
            } else {
                if ((RoundEnums.ScoreMode.TEN_POINT.getCode().equals(entity.getSocreMode()) ||
                        RoundEnums.ScoreMode.HUNDRED.getCode().equals(entity.getSocreMode()))
                        && ObjectUtil.isNull(entity.getScoreStatistics())) {
                    throw new BusinessException("评分统计方式为空");
                }

            }
            if (ObjectUtil.isNull(entity.getPromotionMode())) {
                throw new BusinessException("请选择晋级方式");
            }
        }



        if (ObjectUtil.isNull(entity.getEntrySubmitMode())) {
            throw new BusinessException("请选择上传作品方式");
        } else {
            if ((RoundEnums.EntrySubmitMode.ALONE.getCode().equals(entity.getEntrySubmitMode()))) {
                if (ObjectUtil.isNull(entity.getEntryDeadline())) {
                    throw new BusinessException("上传作品截止时间为空");
                }
                //需要有作品表
                List<MthRegisterConfig> list = registerConfigRepo.findByMatchIdAndConfigType(entity.getMatchId(),
                        MatchEnums.ConfigType.WORKS.getCode());
                if (CollectionUtil.isEmpty(list)) {
                    throw new BusinessException("请填写作品表");

                }
            }
            if (RoundEnums.EntrySubmitMode.FOLLOWREGISTER.getCode().equals(entity.getEntrySubmitMode())) {
                //需要有作品表
                List<MthRegisterConfig> list = registerConfigRepo.findByMatchIdAndConfigType(entity.getMatchId(),
                        MatchEnums.ConfigType.WORKS.getCode());
                if (CollectionUtil.isEmpty(list)) {
                    throw new BusinessException("请填写作品表");

                }
            }
        }

    }

    /**
     * 批量删除赛事-区域-场次实例（实例与配置的关系）;
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if (isValid) {

        }
        List<MthRoundInstance> list = baseRepo.findAllById(ids);
        list.stream().forEach(instance -> {
            instance.setDelFlag(1);
        });
        baseRepo.saveAllAndFlush(list);
        return true;
    }

    @Override
    public void createRoundInstance(List<MatchStatusInfoBo> mathchIdList) {
        for (MatchStatusInfoBo match : mathchIdList) {
            if (StrUtil.equals(match.getStatus(), MatchInfoStatusEnums.TO_BE_RELEASED.getStatus())) {
                List<MthRoundInstance> roundTemplateList = baseRepo.findByMatchIdAndRoundType(match.getId(), RoundEnums.RoundType.TEMPLATE.getCode());
                //查询场次区域
                if (CollectionUtil.isNotEmpty(roundTemplateList)) {
                    Map<String, List<MthRoundInstance>> matchLevelTemplate = StreamUtils.groupByKey(roundTemplateList, MthRoundInstance::getMatchLevel);
                    for (Map.Entry<String, List<MthRoundInstance>> entry : matchLevelTemplate.entrySet()) {
                        String matchLevel = entry.getKey();
                        // 查询比赛等级第一场场次区域
                        List<MthRoundAreaVo> firstRoundAreaList = this.getFirstRoundAreaByMatchIdAndMatchLevel(match.getId(), matchLevel);
                        for (MthRoundInstance roundTemplate : entry.getValue()) {
                            // 查询奖项
                            List<MthRoundPrize> roundPrizeList = roundPrizeRepo.findByRoundIdAndDelFlag(roundTemplate.getId(), 0);
                            // 查询评审标准
                            List<MthRoundReviewStandar> reviewStandarList = mthRoundReviewStandarRepo.findAllByRoundId(roundTemplate.getId());
                            // 查询签到签退
                            List<MthMatchSignInformation> signInfoList = mthMatchSignInformationRepo.findByInstanceId(roundTemplate.getId());
                            if (CollectionUtil.isNotEmpty(firstRoundAreaList)) {
                                firstRoundAreaList.forEach(area -> {
                                    MthRoundInstance instance = new MthRoundInstance();
                                    BeanUtil.copyProperties(roundTemplate, instance, "id", "createBy", "createId", "createTime");
                                    instance.setRoundType(RoundEnums.RoundType.INSTANCE.getCode());
                                    instance.setAreaId(area.getId());
                                    instance.setAreaName(area.getAreaName());
                                    instance.setRoundState(1);
                                    //插入实例
                                    baseRepo.saveAndFlush(instance);
                                    log.info("模板:{};插入实例{}", roundTemplate.getId(), instance);
                                  /*  try {
                                        //实例插入评审员
                                        iMthRoundListService.saveRoundReviewer(CurrWorkerUtil.getCurrWorker(), instance.getId());
                                        log.info("实例增加评审员:{},{}", CurrWorkerUtil.getCurrWorker().getLoginId(), instance.getId());
                                    } catch (Exception e) {
                                        log.error("增加评审员出错：{}", e.getMessage());
                                        e.printStackTrace();
                                    }*/
                                    //实例插入奖项
                                    if (CollectionUtil.isNotEmpty(roundPrizeList)) {
                                        roundPrizeList.forEach(roundPrizeTemplate -> {
                                            MthRoundPrize roundPrize = new MthRoundPrize();
                                            BeanUtil.copyProperties(roundPrizeTemplate, roundPrize, "id",
                                                    "createBy", "createId", "createTime", "updateId", "updateBy", "updateTime");
                                            roundPrize.setRoundId(instance.getId());
                                            roundPrizeRepo.saveAndFlush(roundPrize);
                                        });
                                    }
                                    if (CollectionUtil.isNotEmpty(reviewStandarList)) {
                                        reviewStandarList.forEach(item -> {
                                            MthRoundReviewStandar standar = new MthRoundReviewStandar();
                                            BeanUtil.copyProperties(item, standar, "id",
                                                    "createBy", "createId", "createTime", "updateId", "updateBy", "updateTime");
                                            standar.setRoundId(instance.getId());
                                            mthRoundReviewStandarRepo.saveAndFlush(standar);
                                        });
                                    }
                                    if (CollectionUtil.isNotEmpty(signInfoList)) {
                                        signInfoList.forEach(item -> {
                                            MthMatchSignInformation signInfo = new MthMatchSignInformation();
                                            BeanUtil.copyProperties(item, signInfo, "id",
                                                    "createBy", "createId", "createTime", "updateId", "updateBy", "updateTime");
                                            signInfo.setInstanceId(instance.getId());
                                            mthMatchSignInformationRepo.saveAndFlush(signInfo);
                                        });
                                    }
                                });
                            }
                        }
                    }
                }
            }
        }
    }


    @Override
    public List<MthRoundPrizeVo> queryRoundPrizeById(Long roundId) {
        List<MthRoundPrize> prizeList = roundPrizeRepo.findByRoundIdAndDelFlagOrderByItemIdAscGroupIdAsc(roundId, 0);
        List<MthRoundPrizeVo> voList = roundPrizeConverter.entitryListToVoList(prizeList);
        //将是百分比的转为数字
        Integer totalCount = mthRoundListRepo.queryRoundListCount(roundId);

        if (CollectionUtil.isNotEmpty(voList)) {
            voList.stream().filter(vo -> {
                        return vo.getPrizeMode().equals(RoundEnums.PromotionMode.PERCENT.getCode());
                    })
                    .forEach(
                            vo -> {
                                log.info("totalCount:{},num:{}", totalCount, vo.getPrizeNum());
                                Double count = Math.ceil((Double.valueOf(totalCount.toString()) * vo.getPrizeNum() / 100D));
                                vo.setPrizeNum(count.intValue());
                            }
                    );


        }


        return voList;
    }


    /**
     * @description: 按照奖项设置方式返回奖项
     * @param
     * @return: 
     * @author liuchuan
     * @date: 2023/11/28 16:13
    */
    @Override
    public Map<Long, List<MthRoundPrizeVo>> getRoundPrizeByMode(Long roundId) {
        Map<Long, List<MthRoundPrizeVo>> map = new HashMap<Long,List<MthRoundPrizeVo>>();
        MthRoundInstance mthRoundInstance =baseRepo.getOne(roundId);
        List<MthRoundPrize> prizeList = roundPrizeRepo.findByRoundIdAndDelFlagOrderByItemIdAscGroupIdAsc(roundId, 0);
        if (CollectionUtil.isNotEmpty(prizeList)&&ObjectUtil.isNotNull(mthRoundInstance)){
            List<MthRoundPrizeVo> list = roundPrizeConverter.entitryListToVoList(prizeList);
            if (ObjectUtil.isNull(mthRoundInstance.getPrizeType())||"match".equals(mthRoundInstance.getPrizeType())){
                map.put(mthRoundInstance.getMatchId(),list);
            }else if ("item".equals(mthRoundInstance.getPrizeType())){
                map = list.stream().collect(Collectors.groupingBy(MthRoundPrizeVo::getItemId));
            }else if ("group".equals(mthRoundInstance.getPrizeType())){
                map = list.stream().collect(Collectors.groupingBy(MthRoundPrizeVo::getGroupId));
            }
        }
        return map;
    }

    @Override
    public List<MthRoundPrizeVo> queryRoundPromoyion(Long roundId) {
        List<MthRoundPrizeVo> voList = Lists.newArrayList();
        //增加晋级人员
        MthRoundPrizeVo promption = new MthRoundPrizeVo();
        MthRoundInstance roundInstance = baseRepo.getOne(roundId);
        promption.setPrizeName("晋级人员");
        if (RoundEnums.PromotionMode.PERCENT.getCode().equals(roundInstance.getPromotionMode())) {
            Integer totalCount = mthRoundListRepo.queryRoundListCount(roundId);
            Double count = Math.ceil((Double.valueOf(totalCount.toString()) * roundInstance.getPromptionNum() / 100D));
            promption.setPrizeNum(count.intValue());
        } else {
            promption.setPrizeNum(roundInstance.getPromptionNum());
        }
        voList.add(promption);
        return voList;
    }

    /**
     * 承办方编辑场次
     *
     * @param bo
     * @param worker
     */
    @Override
    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    public void updateMatchContractorInstance(MthRoundInstanceBo bo, Worker worker) {
        MthRoundInstance mthRoundInstance = baseRepo.getOne(bo.getId());
        if (mthRoundInstance.getRoundState()!=null) {
            if (mthRoundInstance.getRoundState() == 0) {
                throw new BusinessException("场次已开启,不能修改！");
            }
        }
        MthRoundInstance update = baseConverter.boToEntity(bo);
        //校验签到签退信息
        if ("offline".equals(bo.getPlayMode())){
            validSignInformation(bo.getSignInInfoVo());
            validSignInformation(bo.getSignOutInfoVo());
        }
        if (StrUtil.isNotBlank(update.getMatchLevel())) {
            RoundEnums.AreaLevel areaLevel = RoundEnums.AreaLevel.getAreaByCode(update.getMatchLevel());
            update.setRoundLevel(areaLevel.getLevel());
        }
        validContractorEntityBeforeSave(update,true);
        BeanUtil.copyProperties(update, mthRoundInstance, CopyOptions.create().setIgnoreNullValue(true));
        mthRoundInstance.setAreaId(mthRoundInstance.getAreaId());
        //保存评分标准
        svaeIMthRoundReviewStandar(bo.getId(), bo);
        //删除名次配置
        roundPrizeRepo.deleteMthRoundPrizesByRoundId(bo.getId());
        //保存名次
        savePrizeBo(update.getId(),bo);
        //保存签到与签退信息
        if ("offline".equals(bo.getPlayMode()) && ObjectUtil.isNotEmpty(bo.getSignInInfoVo())){
            bo.getSignInInfoVo().setInformationType(1);
            mthMatchSignInService.saveSignInformation(bo.getSignInInfoVo());
        }
        if ("offline".equals(bo.getPlayMode()) && ObjectUtil.isNotEmpty(bo.getSignOutInfoVo())){
            bo.getSignOutInfoVo().setInformationType(-1);
            mthMatchSignInService.saveSignInformation(bo.getSignOutInfoVo());
        }
        baseRepo.saveAndFlush(mthRoundInstance);
        updateRoundInstanceSort(bo.getMatchId(),mthRoundInstance.getAreaId());

    }

    /**
     * 、
     * 承办方新增场次
     *
     * @param bo
     * @param worker
     */
    @Override
    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    public void saveMatchContractorInstance(MthRoundInstanceBo bo, Worker worker) {
        MthRoundInstance save = baseConverter.boToEntity(bo);
        SignInfoVo signInInfoVo = bo.getSignInInfoVo();
        SignInfoVo signOutInfoVo = bo.getSignOutInfoVo();
        if (StrUtil.isNotBlank(save.getMatchLevel())) {
            RoundEnums.AreaLevel areaLevel = RoundEnums.AreaLevel.getAreaByCode(save.getMatchLevel());
            save.setRoundLevel(areaLevel.getLevel());
        }
        validContractorEntityBeforeSave(save,false);
        //线下比赛时校验签到签退信息
        if ("offline".equals(bo.getPlayMode())){
            validSignInformation(signInInfoVo);
            validSignInformation(signOutInfoVo);
        }


        save.setRoundType(RoundEnums.RoundType.INSTANCE.getCode());
        List<MthRoundInstance> areaRoundList = baseRepo.findByMatchIdAndRoundTypeAndAreaId(save.getMatchId(), RoundEnums.RoundType.INSTANCE.getCode(),save.getAreaId() );
        save.setRoundSort(areaRoundList.size()+1);
        MthRoundInstance mthRoundInstance = baseRepo.saveAndFlush(save);

        //保存评分标准
        svaeIMthRoundReviewStandar(mthRoundInstance.getId(), bo);
        //保存名次配置
        savePrizeBo(mthRoundInstance.getId(),bo);
        areaRoundList.add(mthRoundInstance);
        //排序
        updateRoundSort(areaRoundList);
        //保存签到与签退信息
        if (ObjectUtil.isNotEmpty(signInInfoVo)){
            signInInfoVo.setInstanceId(mthRoundInstance.getId());
            signInInfoVo.setInformationType(1);
            mthMatchSignInService.saveSignInformation(signInInfoVo);
        }
        if (ObjectUtil.isNotEmpty(signOutInfoVo)){
            signOutInfoVo.setInstanceId(mthRoundInstance.getId());
            signOutInfoVo.setInformationType(-1);
            mthMatchSignInService.saveSignInformation(signOutInfoVo);
        }
    }


    private void roundPrizePercentHandle(Long roundId, List<MthRoundPrizeVo> list) {
        Integer totalCount = mthRoundListRepo.queryRoundListCount(roundId);
        list.stream().forEach(vo -> {

            Double count = Math.ceil((Double.valueOf(totalCount.toString()) * vo.getPrizeNum() / 100D));
            vo.setPrizeNum(Integer.valueOf(count.toString()));
        });
    }

    /**
     * 校验签到信息
     *
     * @param signInfoVo
     */
    private void validSignInformation(SignInfoVo signInfoVo) {
        if (signInfoVo != null) {
            //校验签到时间是否完善
            if ((ObjectUtil.isNotEmpty(signInfoVo.getSignStartTime()) && ObjectUtil.isEmpty(signInfoVo.getSignEndTime()))
                    || (ObjectUtil.isNotEmpty(signInfoVo.getSignEndTime()) && ObjectUtil.isEmpty(signInfoVo.getSignStartTime()))) {
                throw new BusinessException(signInfoVo.getInformationType() == 1 ? MATCH_SIGN_IN_TIME_INCOMPLETE : MATCH_SIGN_OUT_TIME_INCOMPLETE);
            }
            //校验签到地点是否完善
            if (signInfoVo.getSignType()!=1 && ((ObjectUtil.isNotEmpty(signInfoVo.getSignLatitude()) && ObjectUtil.isEmpty(signInfoVo.getSignLongitude())) ||
                    (ObjectUtil.isNotEmpty(signInfoVo.getSignLongitude()) && ObjectUtil.isEmpty(signInfoVo.getSignLatitude())))) {
                throw new BusinessException(signInfoVo.getInformationType() == 1 ? MATCH_SIGN_IN_SITE_INCOMPLETE : MATCH_SIGN_OUT_SITE_INCOMPLETE);
            }
            //有签到地点就必须有签到时间
            if ((ObjectUtil.isNotEmpty(signInfoVo.getSignLatitude()) || ObjectUtil.isNotEmpty(signInfoVo.getSignLongitude()))
                    && (ObjectUtil.isEmpty(signInfoVo.getSignStartTime()) || ObjectUtil.isEmpty(signInfoVo.getSignEndTime()))) {
                throw new BusinessException(signInfoVo.getInformationType() == 1 ? MATCH_SIGN_IN_TIME_INCOMPLETE : MATCH_SIGN_OUT_TIME_INCOMPLETE);
            }
            //非扫码的情况下,有签到时间就必须有签到地点
            if (signInfoVo.getSignType()!=1 && ((ObjectUtil.isNotEmpty(signInfoVo.getSignStartTime()) || ObjectUtil.isNotEmpty(signInfoVo.getSignEndTime()))
                    && (StringUtils.isEmpty(signInfoVo.getSignLatitude()) || StringUtils.isEmpty(signInfoVo.getSignLongitude())))) {
                throw new BusinessException(signInfoVo.getInformationType() == 1 ? MATCH_SIGN_IN_SITE_INCOMPLETE : MATCH_SIGN_OUT_SITE_INCOMPLETE);
            }
            //开始时间不能早于上一次设置的开始时间
            if (signInfoVo.getId() != null) {
                MthMatchSignInformation information = mthMatchSignInformationRepo.getOne(signInfoVo.getId());
                if (information != null && ObjectUtil.isNotEmpty(signInfoVo.getSignStartTime())
                        && ObjectUtil.isNotEmpty(information.getSignStartTime())
                        && signInfoVo.getSignStartTime().before(information.getSignStartTime())
                        && !signInfoVo.getSignStartTime().equals(information.getSignStartTime())) {
                    throw new BusinessException(signInfoVo.getInformationType() == 1 ? MATCH_SIGN_IN_DATE_ONLY_AFTER : MATCH_SIGN_OUT_DATE_ONLY_AFTER);
                }
            }
        }
    }


    /**
     * @param roundId
     * @description: 去公示
     * 1、修改场次状态为公示中
     * 2、更新公示截止时间
     * @return:
     * @author liuchuan
     * @date: 2023/11/22 14:13
     */

    @Override
    public void doPulicity(Long roundId) {
        MthRoundInstance mthRoundInstance = baseRepo.findById(roundId).orElseThrow(() -> new BusinessException("场次id错误"));
        //设置状态为公示中
        mthRoundInstance.setMatchState(25);
        //设置公示截止日期
        mthRoundInstance.setPublicityDeadline(DateUtil.addDay(new Date(), mthRoundInstance.getPublicityDays()));
        baseRepo.saveAndFlush(mthRoundInstance);
    }


    /**
     * @param bo
     * @description: 发送领奖通知
     * 1、将领奖时间、地点、方式保存数据库
     * 2、查询领奖人员、组装消息，调用消息发送
     * @return:
     * @author liuchuan
     * @date: 2023/11/23 14:36
     */
    @Override
    public boolean receiveMsg(ReceiveInfoBo bo) {
        MthRoundInstance mthRoundInstance = baseRepo.getOne(bo.getRoundId());
        mthRoundInstance.setReceiveType(bo.getReceiveType());
        mthRoundInstance.setReceiveAddress(bo.getReceiveAddress());
        mthRoundInstance.setReceiveDeadline(bo.getReceiveDeadline());
        baseRepo.saveAndFlush(mthRoundInstance);
        // TODO: 2023/11/23 liuchuan  查询获奖人员 发送消息
        pushMsg(bo);

        return true;
    }

    /**
     * 承办方开启场次
     * @param roundId
     * @param state
     * @param worker
     */
    @Override
    public void updateRoundState(Long roundId, Integer state, Worker worker) {
        MthRoundInstance mthRoundInstance = baseRepo.findById(roundId).orElse(null);
        if (mthRoundInstance==null){
            throw  new BusinessException("场次不存在！");
        }
        if (state==0){
            //开启判断否完善比赛信息
            try {
                validContractorEntityBeforeSave(mthRoundInstance,false);
            } catch (Exception e) {
                throw  new BusinessException("场次信息未完善！");
            }
        }
        mthRoundInstance.setRoundState(state);
        mthRoundInstance.setUpdateTime(new Date());
        mthRoundInstance.setUpdateId((long)worker.getLoginId());
        mthRoundInstance.setUpdateBy(worker.getWorkerName());
        baseRepo.save(mthRoundInstance);
    }

    private void pushMsg(ReceiveInfoBo bo){
        //查询场次获奖人员
        List<Long> idList = roundScoreDao.getReceviePrizeList(bo.getRoundId());

        roundScoreDao.updatePrizeMsgFlag(bo.getRoundId());
        //查询比赛名称
        MthRoundInstance mthRoundInstance = baseRepo.getOne(bo.getRoundId());
        MthMatchInfo matchInfo = mthMatchInfoRepo.getOne(mthRoundInstance.getMatchId());
        pushBusinessMsgService.pushReceiveMsg(idList, bo.getReceiveType(), matchInfo.getMatchName(), bo.getReceiveDeadline(), bo.getReceiveAddress());


    }

    /**
     * 保存评分表
     *
     * @param roundId
     * @param bo
     */
    public void svaeIMthRoundReviewStandar(Long roundId, MthRoundInstanceBo bo) {
        if (!CollectionUtil.isEmpty(bo.getReviewStandarBoList())) {
            List<MthRoundReviewStandarBo> reviewStandarBoList = bo.getReviewStandarBoList().stream().map(
                    item -> {
                        item.setRoundId(roundId);
                        return item;
                    }).collect(Collectors.toList());
            iMthRoundReviewStandarService.saveBatchIMthRoundReviewStandar(reviewStandarBoList,bo.getSocreMode(),bo.getReviewerStandar());

        }

    }
}
