package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.BeanUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.*;
import com.crunii.micro.service.match.converter.MthRoundListConverter;
import com.crunii.micro.service.match.converter.MthRoundReviewConverter;
import com.crunii.micro.service.match.enums.RoundEnums;
import com.crunii.micro.service.match.jdbc.MthRoundInstanceDao;
import com.crunii.micro.service.match.jdbc.MthRoundListDao;
import com.crunii.micro.service.match.jdbc.MthRoundScoreDao;
import com.crunii.micro.service.match.service.IMthMatchInfoService;
import com.crunii.micro.service.match.service.IMthRegisterService;
import com.crunii.micro.service.match.service.IMthRoundListService;
import com.crunii.micro.service.match.service.PushBusinessMsgService;
import com.google.common.collect.Maps;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.NumberFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 赛事-赛项-组别-区域-场次-名单;Service业务层处理
 *
 * @author mth
 * @date 2023-09-19
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class MthRoundListServiceImpl implements IMthRoundListService {

    private final MthRoundListRepo baseRepo;
    private final MthRoundInstanceRepo mthRoundInstanceRepo;
    private final MthRoundInstanceDao mthRoundInstanceDao;
    private final MthRoundScoreRepo mthRoundScoreRepo;
    private final MthRoundScoreDao mthRoundScoreDao;
    private final MthRoundScoreDetailRepo mthRoundScoreDetailRepo;
    private final MthRoundListConverter baseConverter;
    private final MthRoundListDao mthRoundListDao;
    private final MthRoundReviewStandarRepo mthRoundReviewStandarRepo;
    private final MthWorkRepo mthWorkRepo;
    private final MthRoundReviewerRepo mthRoundReviewerRepo;
    private final MthRoundReviewConverter mthRoundReviewConverter;

    private final MthMatchContractorRepo mthMatchContractorRepo;
    private final MthMatchInfoRepo mthMatchInfoRepo;
    private final PushBusinessMsgService pushBusinessMsgService;

    @Resource
    IMthRegisterService registerService;
    @Resource
    private IMthMatchInfoService iMthMatchInfoService;
    private final MthRoundPrizeRepo mthRoundPrizeRepo;
    private final MthMatchProcessRepo processRepo;
    private final TsmWorkerRepo tsmWorkerRepo;
    private final MthRoundScoreinfoRepo mthRoundScoreinfoRepo;
    /**
     * 查询赛事-赛项-组别-区域-场次-名单;
     */
    @Override
    public MthRoundListVo queryById(Long id){
        return baseConverter.entityToVo(baseRepo.getById(id));
    }

    /**
     * 查询赛事-赛项-组别-区域-场次-名单;列表
     */
    @Override
    public Result<PageRsp<MthRoundListVo>> queryPageList(PageReq<MthRoundListBo> bo) {
        return null;
    }

    /**
     * 查询赛事-赛项-组别-区域-场次-名单;列表
     */
    @Override
    public List<MthRoundListVo> queryList(MthRoundListBo bo) {
        return null;
    }

    @Override
    public MthRoundList queryCurrRoundList(Long registerId) {
        List<MthRoundList> mthRoundLists = mthRoundListDao.queryRoundListByRegisterId(registerId);
        if (ObjectUtil.isNotEmpty(mthRoundLists)) {
            return mthRoundLists.get(0);
        }
        return null;
    }

    /**
     * 新增赛事-赛项-组别-区域-场次-名单;
     */
    @Override
    public Boolean insertByBo(MthRoundListBo bo) {
        MthRoundList add = baseConverter.boToEntity(bo);
        validEntityBeforeSave(add);
        baseRepo.save(add);
        return true;
    }

    /**
     * 修改赛事-赛项-组别-区域-场次-名单;
     */
    @Override
    public Boolean updateByBo(MthRoundListBo bo) {
        MthRoundList update = baseConverter.boToEntity(bo);
        validEntityBeforeSave(update);
        return null;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(MthRoundList entity){

    }

    /**
     * 批量删除赛事-赛项-组别-区域-场次-名单;
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){

        }
        return null;
    }

    /**
     * @description: 查询当前审核员拥有的场次状态为待评审的场次列表
     */
    @Override
    public List<MthReviewRoundVo> getReviewRound(Worker worker, MthReviewRoundBo bo) {
        MthRoundReviewConf mthRoundReviewConf = mthRoundInstanceDao.getMthRoundReviewConf(worker.getLoginId());
       if(mthRoundReviewConf==null){
           log.info("评审员数据异常{}",worker.getLoginId());
           return null;
       }
        List<MthReviewRoundVo> mthReviewRoundVos = mthRoundInstanceDao.reviewRound((long) worker.getLoginId(),bo,mthRoundReviewConf);
        //

        return mthReviewRoundVos;
    }
    /**
     * 查询设置排名列表(去评审);
     */
    @Override
    public PageRsp<MthRoundRankingListVo> getRankingListByRound(Worker worker, PageReq<MthRoundRankingListBo> bo,boolean export) {
        MthRoundReviewConf mthRoundReviewConf = mthRoundInstanceDao.getMthRoundReviewConf(worker.getLoginId());
        if (mthRoundReviewConf==null){
            log.info("评审员数据异常{}",worker.getLoginId());
            return null;
        }
        PageRsp<MthRoundRankingListVo> mthRoundRankingListVoPageRsp = mthRoundListDao.pageRankingListByRound(bo, worker,mthRoundReviewConf, export);
        //查询场次上传作品时间校验是否可以评审
        Long roundId = bo.getQuery().getRoundId();
        MthRoundInstance roundInstance = mthRoundInstanceRepo.getOne(roundId);
        Date date = new Date();
        Date entryDeadline = roundInstance.getEntryDeadline();
        mthRoundRankingListVoPageRsp.getResult().forEach(item->{
            //导出翻译作品评审状态
            if (export){
               if (item.getWorkReviewStatus()==0){
                   item.setWorkReviewStatusCode("待评审");
               }else  if (item.getWorkReviewStatus()==1){
                   item.setWorkReviewStatusCode("已评审");
               }else {
                   item.setWorkReviewStatusCode("作品截止时间未到且作品未上传");
               }
            }

            if (ObjectUtil.isNull(item.getId()) && (ObjectUtil.isNotNull(entryDeadline) && date.before(entryDeadline) )){
                //作品为空 且在截止事前之前 评审按钮置灰
                item.setNoWorkReviewFlag(0);
            }else{
                item.setNoWorkReviewFlag(1);
            }
        });
        return mthRoundRankingListVoPageRsp;
    }
    /**
     * 查询评审详情
     */
    @Override
    public MthReviewDetailVo getReviewDetail(Worker worker, MthRoundRankingDetailBo bo) {
        MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.getById(bo.getRoundId());
        MatchBaseInfoVo matchBaseInfoVo = null;
        try {
            matchBaseInfoVo = iMthMatchInfoService.matchBaseInfo(bo.getMatchId());
        } catch (Exception e) {
            e.printStackTrace();
            log.error("查询赛事信息出错{}",e.getMessage());
        }
        Long roundListId = bo.getRoundListId();
        MthRoundList mthRoundList = baseRepo.findById(roundListId).get();
        MthReviewDetailVo mthReviewDetailVo = new MthReviewDetailVo();
        mthReviewDetailVo.setSocreMode(mthRoundInstance.getSocreMode());
        mthReviewDetailVo.setReviewerStandar(mthRoundInstance.getReviewerStandar());
        mthReviewDetailVo.setId(bo.getWorkId());
        //添加比赛信息
        mthReviewDetailVo.setMthMatchInfoDetailVo(matchBaseInfoVo);
        //添加作品信息
        MthMyMatchListDetailAppVo mthMyMatchListDetailAppVo = null;
        try {
            mthMyMatchListDetailAppVo = registerService.myMatchListDetail(bo.getMatchId(), mthRoundList.getRegisterId());
        } catch (Exception e) {
            e.printStackTrace();
            log.error("查询作品信息出错{}",e.getMessage());
        }
        //查询评分标准表信息
        List<MthRoundReviewStandar> mthRoundReviewStandarList;

        if (RoundEnums.ReviewerStandar.ITEM.getCode().equals(mthRoundInstance.getReviewerStandar())) {
                //查询赛项评分你标准
                String itemId = registerService.queryById(mthRoundList.getRegisterId()).getItemId();
                mthRoundReviewStandarList=mthRoundReviewStandarRepo.findAllByRoundIdAndItemId(mthRoundInstance.getId(),Long.valueOf(itemId));
        }else {
            mthRoundReviewStandarList= mthRoundReviewStandarRepo.findAllByRoundId(mthRoundInstance.getId());
        }
        mthReviewDetailVo.setRoundReviewStandarVoList(mthRoundReviewConverter.entitryListToVoList(mthRoundReviewStandarList));

        mthReviewDetailVo.setMthMyMatchListDetailAppVo(mthMyMatchListDetailAppVo);
        mthReviewDetailVo.setRoundListId(roundListId);
//        mthReviewDetailVo.setCreateTime(mthWork.getCreateTime());
//        mthReviewDetailVo.setUpdateTime(mthWork.getUpdateTime());
        return mthReviewDetailVo;
    }
    /**
     *这个方法的四舍五入也是不算真正的四舍五入，只要后面一位有值就会往前进一位，
     * @return

     */
    public static String formatDouble(double d){
        NumberFormat nf = NumberFormat.getNumberInstance();
        //保留两位小数
        nf.setMaximumFractionDigits(2);
        return nf.format(d);
    }

    /**
     * 计算总分
     * @param mthRoundInstance
     * @param mthRoundScoreinfo
     * @return
     */

    private MthRoundScore saveMthRoundScore( MthRoundInstance mthRoundInstance , MthRoundScoreinfo mthRoundScoreinfo,MthRoundScore mthRoundScore,Worker worker ){
        //记录原分数
        String oldScore=mthRoundScore.getScore();
        if (mthRoundInstance.getSocreMode().equals(RoundEnums.ScoreMode.HUNDRED.getCode()) || RoundEnums.ScoreMode.TEN_POINT.getCode().equals(mthRoundInstance.getSocreMode())){
            MthRoundReviewConf mthRoundReviewConf = mthRoundInstanceDao.getMthRoundReviewConf(worker.getLoginId());
            if (mthRoundReviewConf==null){
                throw new BusinessException("数据异常！");
            }
            //查询场次评审员
            List<MthRoundReviewer> mthRoundReviewerList = mthRoundReviewerRepo.findByRoundId(mthRoundInstance.getId());
            //所有评审员评分信息
            List<MthRoundScoreinfo> mthRoundScoreinfoList = mthRoundScoreinfoRepo.findByRoundListId(mthRoundScoreinfo.getRoundListId());
            //是否区分赛项
             if(mthRoundReviewConf.getMatchItemFlag()==1){
                 MthRoundReviewer byRoundIdAndReviewerId = mthRoundReviewerRepo.findByRoundIdAndReviewerId(mthRoundInstance.getId(),(long) worker.getLoginId());
               //同赛项评审员
                 mthRoundReviewerList=  mthRoundReviewerList.stream().filter(item->item.getItemId().equals(byRoundIdAndReviewerId.getItemId())).collect(Collectors.toList());
             //    List<Long> listSeviewer = mthRoundReviewerList.stream().map(itm -> itm.getReviewerId()).collect(Collectors.toList());
               //  mthRoundScoreinfoList = mthRoundScoreinfoList.stream().filter(item -> listSeviewer.contains(item.getReviewerId())).collect(Collectors.toList());
             }
            //所有评审员已经评完
            if (mthRoundReviewerList.size()==mthRoundScoreinfoList.size()){
                List<Double> collect = mthRoundScoreinfoList.stream().map(data -> Double.valueOf(data.getScore())).collect(Collectors.toList());
                double sorce=0;
                //根据统计方式计算总分  平均分计算
                if (mthRoundInstance.getScoreStatistics().equals("avg")){
                  double  sumsorce =  collect.stream().map(e->new BigDecimal(String.valueOf(e))).reduce(BigDecimal::add).get().doubleValue();
                    sorce= sumsorce/collect.size();

                }
                //  去掉最高分最低分
                if (mthRoundInstance.getScoreStatistics().equals("excludeMaxMin")){
                    HashSet<Double>   set=new HashSet<>(collect);
                    //如果所有评分员分数相同或者评审员小于等于4人则按照平均分计算
                    if (set.size()==collect.size()|| mthRoundReviewerList.size()<=4){
                        double  sumsorce =  collect.stream().map(e->new BigDecimal(String.valueOf(e))).reduce(BigDecimal::add).get().doubleValue();
                        sorce= sumsorce/collect.size();
                    }else {

                        //最高分
                        double max = collect.stream().mapToDouble(Double::valueOf).max().getAsDouble();
                        //最低分
                        double min = collect.stream().mapToDouble(Double::valueOf).min().getAsDouble();
                        //去掉一个最高和最低 计算平均分
                        double sum = collect.stream().map(e->new BigDecimal(String.valueOf(e))).reduce(BigDecimal::add).get().doubleValue();
                        sorce= (sum - max - min) / (collect.size() - 2);
                    }
                }
                mthRoundScore.setScore(String.valueOf(formatDouble(sorce)));
            }else {
                return null;
            }
        }else {
            //非十分制和百分制直接存入评分表
            mthRoundScore.setScore(mthRoundScoreinfo.getScore());
            
        }
         //   mthRoundScore.setResult("2");
            mthRoundScore.setReviewTime(new Date());
            mthRoundScore.setStatus(1);
            if (StringUtils.isBlank(oldScore)){
                mthRoundScore.setOldScore(mthRoundScore.getScore());
            }else {
                mthRoundScore.setOldScore(oldScore);
            }
          //  mthRoundScore.setReviewerId((long) worker.getLoginId());
         //   mthRoundScore.setReviewerName(worker.getWorkerName());
       //     mthRoundScore.setReviewerSign(worker.getSignatureFileId());//评委签名
            mthRoundScore.setRoundListId(mthRoundScoreinfo.getRoundListId());
        return mthRoundScore;
    }


    /**
     *  保存评审结果
     *  2023 /11/29 新增评分明细表
     */

    @SneakyThrows
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveReviewResult(Worker worker, MthRoundScoreReviewBo bo) {
        MthRoundList mthRoundList = baseRepo.findById(bo.getRoundListId()).orElseThrow(()->new Exception("未查询到该场次信息"));
        MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.findById(Long.valueOf(mthRoundList.getRoundId())).orElseThrow(()->new Exception("未查询到该比赛信息"));
        if (mthRoundInstance.getMatchState()!=null){
            if (mthRoundInstance.getMatchState()!=1){
                throw new BusinessException("场次已评审结束！");
            }
        }
        AtomicReference<Double> totalScore= new AtomicReference<>(0.00);
        List<MthRoundScore> roundScores = mthRoundScoreRepo.findByRoundListId(bo.getRoundListId());
        if (CollUtil.isEmpty(roundScores)){
            throw new BusinessException("内部数据错误，场次名分表为插入！");
        }

        // 评分明细记录每位评委的评分信息
        MthRoundScoreinfo mthRoundScoreinfo = mthRoundScoreinfoRepo.findByRoundListIdAndReviewerId(mthRoundList.getId(),(long) worker.getLoginId());
        //确保每位评审员只存在一条评分信息
        if (mthRoundScoreinfo==null){
            mthRoundScoreinfo=new MthRoundScoreinfo();
        }
        //排名信息和总分
        MthRoundScore mthRoundScore = roundScores.get(0);

        String socreMode = mthRoundInstance.getSocreMode();
        validSave(bo, socreMode);

        //根据场次id查询是否有指标，有则需要计算后存入
        Map<String,Object> props = new HashMap<>();
        props.put("roundId",mthRoundList.getRoundId());
        props.put("delFlag",0);
        List<MthRoundReviewStandar> byProps = mthRoundReviewStandarRepo.findByProps(props);
        List<MthRoundScoreDetail> mthRoundScoreDetails=new LinkedList<>();
        if ( CollectionUtil.isNotEmpty(byProps)){
            List<MthRoundScoreDetailBo> mthRoundScoreDetailBos = bo.getMthRoundScoreDetailBos();
            List<Long> standarId = mthRoundScoreDetailBos.stream().map(MthRoundScoreDetailBo::getStandarId).collect(Collectors.toList());
           // List<MthRoundReviewStandar> standarRepoAllById = mthRoundReviewStandarRepo.findAllById(standarId);
            List<MthRoundReviewStandar> standarRepoAllById=byProps.stream().filter(item->
                standarId.contains(item.getId())
            ).collect(Collectors.toList());


            //权重  要么全部有要么全部为空
            String weight = standarRepoAllById.get(0).getWeight();
            Map<Long, MthRoundReviewStandar> standarMap = standarRepoAllById.stream().collect(Collectors.toMap(MthRoundReviewStandar::getId, s -> s));
//            Assert.notEmpty(mthRoundScoreDetailBos,"评分指标不能为空");

           //不需要计算权重
            if (StrUtil.isEmpty(weight)){
                mthRoundScoreDetailBos.forEach(s->{
                    totalScore.set(totalScore.get() + Double.parseDouble(s.getSocre()));
                });
            }else {
                mthRoundScoreDetailBos.forEach(s->{
                    MthRoundReviewStandar mthRoundReviewStandar = standarMap.get(s.getStandarId());
                    double weightScore = Math.ceil(Double.parseDouble(s.getSocre()) * getWeight(mthRoundReviewStandar.getWeight()));
                    totalScore.set(weightScore +totalScore.get());

                    MthRoundScoreDetail mthRoundScoreDetail = new MthRoundScoreDetail();
                   // mthRoundScoreDetail.setSocre(String.valueOf(weightScore));
                    mthRoundScoreDetail.setSocre(s.getSocre());

                    mthRoundScoreDetail.setStandarId(s.getStandarId());
//                    mthRoundScoreDetail.setScoreId();
                    mthRoundScoreDetails.add(mthRoundScoreDetail);
                });
            }
           // mthRoundScoreinfo.setScore(String.valueOf(Math.ceil(totalScore.get())));
            if (socreMode.equals(RoundEnums.ScoreMode.HUNDRED.getCode())) {
                if (totalScore.get() >100 || totalScore.get()<0) {
                    throw new BusinessException("分数不符合百分制！");
                }

            } else {
                if (totalScore.get() >10 || totalScore.get()<0) {
                    throw new BusinessException("分数不符合十分制！");
                }
            }

            mthRoundScoreinfo.setScore(bo.getScore());
          //  mthRoundScoreinfo.setOldScore(String.valueOf(Math.ceil(totalScore.get())));
        }else {//直接打分情况，保存即可
            mthRoundScoreinfo.setScore(bo.getScore());
          //  mthRoundScore.setOldScore(bo.getScore());
        }
        mthRoundScoreinfo.setCreateTime(new Date());
        mthRoundScoreinfo.setComments(bo.getComment());
        mthRoundScoreinfo.setReviewerId((long) worker.getLoginId());
        mthRoundScoreinfo.setReviewerName(worker.getWorkerName());
        mthRoundScoreinfo.setReviewerSign(worker.getSignatureFileId());//评委签名
        mthRoundScoreinfo.setRoundListId(bo.getRoundListId());
        mthRoundScoreinfo.setStatus(1);
        MthRoundScoreinfo save = mthRoundScoreinfoRepo.save(mthRoundScoreinfo);
        //计算所有评审员总分
        MthRoundScore resultScore = saveMthRoundScore(mthRoundInstance, mthRoundScoreinfo, mthRoundScore,worker);
        if (CollectionUtil.isNotEmpty(mthRoundScoreDetails)){
            mthRoundScoreDetails.forEach(item->{
                item.setScoreId(save.getId());
                item.setDelFlag(0);
            });
            mthRoundScoreDetailRepo.saveAll(mthRoundScoreDetails);
        }
        // 新增过程记录
        saveProcessRecord(mthRoundInstance,mthRoundList.getRegisterId(), save, worker);
         //最后一个评审员评审结束后计算排名
          if (resultScore==null){
              return true;
          }
          mthRoundScoreRepo.save(mthRoundScore);
       //记录过程 结果
        saveMthRoundScoreProcessRecord(mthRoundInstance,mthRoundList.getRegisterId(),mthRoundScore) ;
        //todo ymz 判断如果是最后一条作品，需扭转场次状态为待设置晋级名单  ，且设置排名，和默认晋级奖项等
        //查询名单数量
        List<MthRoundList> byProps2 = mthRoundScoreDao.findByProps(mthRoundList.getRoundId());

        //查询评分表数量  一个场次名单对应一个评分， 数量相等则表示评审最后一条 (过滤本次评分)
        List<MthRoundScore> byProps1 = mthRoundScoreDao.getRankingSortListByRoundIds(mthRoundList.getRoundId().toString()).stream().filter(item->
           !item.getId().equals(mthRoundScore.getId())
        ).collect(Collectors.toList());
         byProps1.add(mthRoundScore);
        if (byProps1.size()==byProps2.size()){
            byProps1 = getMthRoundScores(socreMode, byProps1);

            //跟新场次状态
            mthRoundInstance.setMatchState(2);
            mthRoundInstanceRepo.save(mthRoundInstance);

            // 向承办方发送消息
            // 通过areaId查询场次相关的承办方
            if (ObjectUtil.isNotEmpty(mthRoundInstance.getAreaId())){
                List<MthMatchContractor> contractors = mthMatchContractorRepo.findByAreaIdAndDelFlag(mthRoundInstance.getAreaId(),0);
                String matchName = "";
                if (ObjectUtil.isNotEmpty(contractors)){
                    List<Long> contractorIds = contractors.stream().filter(item -> ObjectUtil.isNotEmpty(item.getContractorId())).map(MthMatchContractor::getContractorId)
                        .collect(Collectors.toList());
                    // 查询比赛信息
                    MthMatchInfo mthMatchInfo  = mthMatchInfoRepo.findById(mthRoundInstance.getMatchId()).orElseThrow(() -> new BusinessException("根据比赛id:"+mthRoundInstance.getMatchId()+ "未找到比赛信息!"));
                    if (ObjectUtil.isNotEmpty(mthMatchInfo.getMatchName())){
                        matchName = mthMatchInfo.getMatchName();
                    }
                    log.info("待设置排名晋级通知:{} ",matchName);
                    pushBusinessMsgService.pushProMotionRandingMsg(contractorIds,matchName);
                }
            }


            //查询场次名次配置 设置默认排名、是否晋级(奖项  确定不了)
            String promotionMode = mthRoundInstance.getPromotionMode();
            int num=0;
            if (promotionMode.equals(RoundEnums.PromotionMode.NUM.getCode())){
                num = mthRoundInstance.getPromptionNum();
            }
            if (promotionMode.equals(RoundEnums.PromotionMode.PERCENT.getCode())){//百分比
                num = (int)Math.ceil( byProps1.size() * mthRoundInstance.getPromptionNum() / 100.00);
            }
            AtomicInteger j= new AtomicInteger(1);
            int finalNum = num;
            byProps1.forEach(item->{
               if (j.get() <= finalNum){
                    item.setResult("1");
                }else {
                   item.setResult("2");
               }
                //记录原排名
                if ( item.getSort()!=null){
                    item.setOldSort(item.getSort());
                }else {
                    item.setOldSort(j.get());
                }
                item.setSort(j.get());
                j.getAndIncrement();
            });

            mthRoundScoreRepo.saveAll(byProps1);

        }
        return true;
    }

    private double getWeight(String weight) {
        return Double.valueOf(weight)*0.01;
    }

    public List<MthRoundScore> getMthRoundScores(String socreMode, List<MthRoundScore> byProps1) {
        //根据评分排序
        if (socreMode.equals(RoundEnums.ScoreMode.HUNDRED.getCode()) || RoundEnums.ScoreMode.TEN_POINT.getCode().equals(socreMode)){
            byProps1 = byProps1.stream().sorted(Comparator.comparing(MthRoundScore::getScore,Comparator.comparing(Double::valueOf)).reversed().thenComparing(BaseEntity::getUpdateTime)).collect(Collectors.toList());
        }
        if (RoundEnums.ScoreMode.GRAD.getCode().equals(socreMode)){
            Map<String, Integer> sortMap = Maps.newHashMap();
            sortMap.put("A", 4);
            sortMap.put("B", 3);
            sortMap.put("C", 2);
            sortMap.put("D", 1);
            sortMap.put("E", 1);
            //根据评分排序
            byProps1 = byProps1.stream().sorted(Comparator.comparing(MthRoundScore::getScore, Comparator.comparing(item -> sortMap.get(item))).reversed().thenComparing(BaseEntity::getUpdateTime)).collect(Collectors.toList());
        }
        if (RoundEnums.ScoreMode.PASS.getCode().equals(socreMode)){
            Map<String, Integer> sortMap = Maps.newHashMap();
            sortMap.put("通过", 2);
            sortMap.put("不通过", 1);
            //根据评分排序
            byProps1 = byProps1.stream().sorted(Comparator.comparing(MthRoundScore::getScore, Comparator.comparing(item -> sortMap.get(item))).reversed().thenComparing(BaseEntity::getUpdateTime)).collect(Collectors.toList());
        }
        return byProps1;
    }

    private void saveProcessRecord(MthRoundInstance mthRoundInstance,Long userId, MthRoundScoreinfo save,Worker worker) {
        LinkedList<MthMatchProcess> mthMatchProcesses = new LinkedList<>();
        MthMatchProcess matchProcessResult =new MthMatchProcess();
        matchProcessResult.setType("REVIEW");
        matchProcessResult.setBizId(save.getId());
        matchProcessResult.setUserId(userId);
        matchProcessResult.setMatchId(mthRoundInstance.getMatchId());
        //获取用户名称
        TsmWorker wr = tsmWorkerRepo.findById(worker.getLoginId()).orElse(null);
        String name="";
        if (wr!=null){
            name=wr.getWorkerName();
        }else {
            name=worker.getWorkerName();
        }

        matchProcessResult.setShowTitle(mthRoundInstance.getRoundName()+"评审结果：" + save.getScore() +"  评委: "+name);
        mthMatchProcesses.add(matchProcessResult);
        MthMatchProcess matchProcessTime =new MthMatchProcess();
        matchProcessTime.setType("REVIEW");
        matchProcessTime.setBizId(save.getId());
        matchProcessTime.setMatchId(mthRoundInstance.getMatchId());
        matchProcessTime.setUserId(userId);
        matchProcessTime.setShowTitle("评审时间："+ DateUtil.format(new Date(),"yyyy-MM-dd HH:mm:ss") );
        mthMatchProcesses.add(matchProcessTime);
        processRepo.saveAll(mthMatchProcesses);
    }



    private void saveMthRoundScoreProcessRecord(MthRoundInstance mthRoundInstance,Long userId,MthRoundScore save) {
        processRepo.deleteByBizId(save.getId());
        LinkedList<MthMatchProcess> mthMatchProcesses = new LinkedList<>();
        Long bizId=save.getId();
        MthMatchProcess matchProcessResult =new MthMatchProcess();
        matchProcessResult.setType("REVIEW");
        matchProcessResult.setBizId(save.getId());
        matchProcessResult.setUserId(userId);
        matchProcessResult.setMatchId(mthRoundInstance.getMatchId());
        matchProcessResult.setShowTitle(mthRoundInstance.getRoundName()+"最终评审结果：" + save.getScore());
       mthMatchProcesses.add(matchProcessResult);
        MthMatchProcess matchProcessTime =new MthMatchProcess();
        matchProcessTime.setType("REVIEW");
        matchProcessTime.setBizId(bizId);
        matchProcessTime.setMatchId(mthRoundInstance.getMatchId());
        matchProcessTime.setUserId(userId);
        matchProcessTime.setShowTitle("评审时间："+ DateUtil.format(new Date(),"yyyy-MM-dd HH:mm:ss") );
        mthMatchProcesses.add(matchProcessTime);
        processRepo.saveAll(mthMatchProcesses);
    }


    /**
     * @description: 对评分进行排名
     * @param
     * @return: 
     * @author liuchuan
     * @date: 2023/11/23 9:13
    */
    @Override
    public void roundListRank(String scoreMode, List<MthRoundScore> list) {
        list =  getMthRoundScores(scoreMode,list);
        AtomicInteger rank= new AtomicInteger(1);
        list.stream().forEach(mthRoundScore -> {
            mthRoundScore.setSort(rank.get());
            rank.getAndIncrement();

        });
        mthRoundScoreRepo.saveAllAndFlush(list);
    }


    @Override
    public void exportRankingListByRound(Worker worker, MthRoundRankingListBo bo) {
        PageReq<MthRoundRankingListBo> req=new PageReq();
        req.setQuery(bo);
        List<MthRoundRankingListVo> rankingListByRound = getRankingListByRound(worker,req,true).getResult();
        try {
            ExcelUtil.writeSingleExcel("评审作品列表","作品列表",rankingListByRound,MthRoundRankingListVo.class);
        } catch (IOException e) {
            e.printStackTrace();
            throw new  BusinessException("评审作品列表导出失败");
        }
    }

    @Override
    public MthRoundScoreinfoVo getMthRoundScoreinfo(MthRoundRankingDetailBo bo, Worker worker) {
        MthRoundReviewConf mthRoundReviewConf = mthRoundInstanceDao.getMthRoundReviewConf(worker.getLoginId());
        if (mthRoundReviewConf==null){
            log.info("评审员数据异常{}",worker.getLoginId());
            return null;
        }
        if (mthRoundReviewConf.getMatchItemFlag()==1){

        }
        //查询评分信息
        MthRoundScoreinfo mthRoundScoreinfo = mthRoundScoreinfoRepo.findByRoundListIdAndReviewerId(bo.getRoundListId(), (long) worker.getLoginId());//
        //如果设置评分指标查询指标详情
       List<MthRoundReviewStandarVo>   list=  mthRoundScoreDao.getMthRoundScoreinfoByStandar(mthRoundScoreinfo.getId());
        MthRoundScoreinfoVo mthRoundScoreinfoVo=new MthRoundScoreinfoVo();
        BeanUtil.copyProperties(mthRoundScoreinfo,mthRoundScoreinfoVo);
        mthRoundScoreinfoVo.setComment(mthRoundScoreinfo.getComments());
        mthRoundScoreinfoVo.setReviewStandarVoList(list);
        return mthRoundScoreinfoVo;
    }
    @SneakyThrows
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateReviewResult(Worker worker, MthRoundScoreReviewBo bo) {
        MthRoundList mthRoundList = baseRepo.findById(bo.getRoundListId()).orElseThrow(()->new Exception("未查询到该场次信息"));
        MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.findById(Long.valueOf(mthRoundList.getRoundId())).orElseThrow(()->new Exception("未查询到该比赛信息"));
        if (mthRoundInstance.getMatchState()!=null){
            if (mthRoundInstance.getMatchState()!=1){
                throw new BusinessException("场次已评审结束！");
            }
        }
        MthRoundScoreinfo mthRoundScoreinfo = mthRoundScoreinfoRepo.findByRoundListIdAndReviewerId(bo.getRoundListId(), Long.valueOf( worker.getLoginId()));
        mthRoundScoreDetailRepo.deleteByScoreId(mthRoundScoreinfo.getId());
        mthRoundScoreinfoRepo.deleteById(mthRoundScoreinfo.getId());
        processRepo.deleteByBizId(mthRoundScoreinfo.getId());
        saveReviewResult(worker,bo);
       return true;
    }




    @Override
    public boolean saveRoundReviewer(Worker worker, Long roundId) {
        MthRoundReviewer mthRoundReviewer = new MthRoundReviewer();
        mthRoundReviewer.setReviewerId((long) worker.getLoginId());
        mthRoundReviewer.setRoundId(roundId);
        try {
            MthRoundReviewer save = mthRoundReviewerRepo.save(mthRoundReviewer);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("添加场次评审员失败！");
        }
        return true;
    }

    private void validSave(MthRoundScoreReviewBo bo, String socreMode) {
        Long roundListId = bo.getRoundListId();
      /*  boolean present = mthRoundScoreRepo.findById(roundListId).isPresent();
        if (present){
            throw new BusinessException("请勿重复评审该作品");
        }*/
        if (RoundEnums.ScoreMode.TEN_POINT.getCode().equals(socreMode)){
            Double integer = Double.valueOf(bo.getScore());
            if (integer>10 || integer<0){
                throw new BusinessException("分数有误请重新输入");
            }
        }
        if (RoundEnums.ScoreMode.HUNDRED.getCode().equals(socreMode)){
            Double integer = Double.valueOf(bo.getScore());
            if (integer>100 || integer<0){
                throw new BusinessException("分数有误请重新输入");
            }
        }
    }


}
