package com.crunii.micro.service.match.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.entity.main.MthRoundReviewStandar;
import com.crunii.micro.domain.repo.main.MthRoundReviewStandarRepo;
import com.crunii.micro.service.match.api.bo.MthRoundReviewStandarBo;
import com.crunii.micro.service.match.api.vo.MthRoundReviewStandarVo;
import com.crunii.micro.service.match.converter.MthRoundReviewConverter;
import com.crunii.micro.service.match.enums.RoundEnums;
import com.crunii.micro.service.match.service.IMthRoundReviewStandarService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 赛事-场次-评审指标配置;Service业务层处理
 *
 * @author mth
 * @date 2023-09-19
 */
@RequiredArgsConstructor
@Service
public class MthRoundReviewStandarServiceImpl implements IMthRoundReviewStandarService {

    private final MthRoundReviewStandarRepo baseRepo;
    private final MthRoundReviewConverter baseConverter;

    @Override
    public void saveBatchIMthRoundReviewStandar(List<MthRoundReviewStandarBo> list, String socreMode,String reviewerStandar) {
        //评分制需要校验数据
        //只能百分制和十分制能设置评分标准
        if (!socreMode.equals(RoundEnums.ScoreMode.HUNDRED.getCode())&&!socreMode.equals(RoundEnums.ScoreMode.TEN_POINT.getCode())){
            throw  new BusinessException("该场次不能设置评分标准");
        }
        //分赛项校验
        if (RoundEnums.ReviewerStandar.ITEM.getCode().equals(reviewerStandar)){
            Map<Long, List<MthRoundReviewStandarBo>> map = list.stream().collect(Collectors.groupingBy(MthRoundReviewStandarBo::getItemId));
            map.forEach((k,v)->{
                validEntityBeforeSave(v,socreMode,reviewerStandar);
            });

        }else {
            validEntityBeforeSave(list,socreMode,reviewerStandar);

        }
        List<MthRoundReviewStandar> save = list.stream().map(item -> {
            MthRoundReviewStandar mthRoundReviewStandar = baseConverter.boToEntity(item);
            return mthRoundReviewStandar;
        }).collect(Collectors.toList());
        baseRepo.deleteMthRoundReviewStandarsByRoundId(list.get(0).getRoundId());
        baseRepo.saveAll(save);
    }

    /**
     * 查询赛事-场次-评审指标配置;
     */
    @Override
    public MthRoundReviewStandarVo queryById(Long id){
        return baseConverter.entityToVo(baseRepo.getById(id));
    }

    /**
     * 查询赛事-场次-评审指标配置;列表
     */
    @Override
    public Result<PageRsp<MthRoundReviewStandarVo>> queryPageList(PageReq<MthRoundReviewStandarBo> bo) {
        return null;
    }

    /**
     * 查询赛事-场次-评审指标配置;列表
     */
    @Override
    public List<MthRoundReviewStandarVo> queryList(MthRoundReviewStandarBo bo) {
        List<MthRoundReviewStandar> allByRoundId = baseRepo.findAllByRoundId(bo.getRoundId());
        List<MthRoundReviewStandarVo> collect = allByRoundId.stream().map(item -> {
            MthRoundReviewStandarVo mthRoundReviewStandarVo = baseConverter.entityToVo(item);
            return mthRoundReviewStandarVo;
        }).collect(Collectors.toList());
        return collect;
    }

    /**
     * 新增赛事-场次-评审指标配置;
     */
    @Override
    public Boolean insertByBo(MthRoundReviewStandarBo bo) {
        MthRoundReviewStandar add = baseConverter.boToEntity(bo);
       // validEntityBeforeSave(add);
        baseRepo.save(add);
        return true;
    }

    /**
     * 修改赛事-场次-评审指标配置;
     */
    @Override
    public Boolean updateByBo(MthRoundReviewStandarBo bo) {
        MthRoundReviewStandar update = baseConverter.boToEntity(bo);
       // validEntityBeforeSave(update);
        return null;
    }
    private double checkWeight(String weight){
        return Double.valueOf(weight)*0.01;
    }
    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(List<MthRoundReviewStandarBo > list,String socreMode,String reviewerStandar){

            //过滤处权重不空的数据
            List<MthRoundReviewStandarBo> collect = list.stream().filter(item -> StringUtils.isNotBlank(item.getWeight())).collect(Collectors.toList());

            if (!CollectionUtils.isEmpty(collect)) {
                if (collect.size() != list.size()) {
                    throw new BusinessException("存在指标未设置权重！");
                }else {
                    Integer wet = collect.stream().mapToInt(MthRoundReviewStandarBo->Integer.valueOf(MthRoundReviewStandarBo.getWeight())).sum();
                    if (wet!=100){
                        if (RoundEnums.ReviewerStandar.ITEM.getCode().equals(reviewerStandar)) {
                            throw new BusinessException("每个赛项权重之和必须为100%！");
                        }else {
                            throw new BusinessException("权重之和必须为100%！");
                        }
                    }

                }

            }

        int count = 0;
            for (MthRoundReviewStandarBo mthRoundReviewStandarBo : list) {
                if (StringUtils.isBlank(mthRoundReviewStandarBo.getStandar())){
                    throw new BusinessException("评审指标必填！");
                }
                if (mthRoundReviewStandarBo.getStandar().length()>200){
                    throw new BusinessException("评审指标长度不能超过200！");

                }
                if (!StringUtils.isEmpty((mthRoundReviewStandarBo.getWeight()))) {
                    double wt = checkWeight(mthRoundReviewStandarBo.getWeight());
                    count += wt * Double.valueOf(mthRoundReviewStandarBo.getScore());
                } else {
                    count += Double.valueOf(mthRoundReviewStandarBo.getScore());
                }
            }

            if (socreMode.equals(RoundEnums.ScoreMode.HUNDRED.getCode())) {

                if (count != 100) {
                    throw new BusinessException("分数之和不符合百分制！");
                }

            } else {
                if (count != 10) {
                    throw new BusinessException("分数之和不符合十分制！");
                }
            }
    }

    /**
     * 批量删除赛事-场次-评审指标配置;
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return null;
    }
}
