package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.match.api.bo.*;
import com.crunii.micro.service.match.api.vo.MthAwardInfoVo;
import com.crunii.micro.service.match.api.vo.MthRoundScoreRankingVo;
import com.crunii.micro.service.match.api.vo.MthRoundScoreVo;
import com.crunii.micro.service.match.converter.MthRoundScoreConverter;
import com.crunii.micro.service.match.enums.RoundEnums;
import com.crunii.micro.service.match.enums.RoundListStatusEnums;
import com.crunii.micro.service.match.jdbc.MthRoundListDao;
import com.crunii.micro.service.match.jdbc.MthRoundScoreDao;
import com.crunii.micro.service.match.service.IMthMatchProcessesService;
import com.crunii.micro.service.match.service.IMthRoundListService;
import com.crunii.micro.service.match.service.IMthRoundScoreService;
import com.crunii.micro.service.match.service.PushBusinessMsgService;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

/**
 * 赛事-场次-比赛评分;Service业务层处理
 *
 * @author mth
 * @date 2023-09-19
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class MthRoundScoreServiceImpl implements IMthRoundScoreService {

    private final MthRoundScoreRepo baseRepo;
    private final MthRoundScoreConverter baseConverter;
    private final MthRoundScoreDao mthRoundScoreDao;
    private final MthRoundInstanceRepo mthRoundInstanceRepo;
    private final MthRoundListRepo mthRoundListRepo;

    private final MthMatchProcessRepo mthMatchProcessRepo;
    private final MthRoundScoreRepo mthRoundScoreRepo;
    private final MthRoundPrizeRepo mthRoundPrizeRepo;
    private final MthRegisterDetailRepo mthRegisterDetailRepo;

    private final MthRoundListDao mthRoundListDao;

    private final MthRoundAreaRepo mthRoundAreaRepo;

    private final IMthMatchProcessesService iMthMatchProcessesService;

    private final MthMatchContractorRepo mthMatchContractorRepo;
    private final MthMatchInfoRepo mthMatchInfoRepo;
    private final PushBusinessMsgService pushBusinessMsgService;

    @Lazy
    @Resource
    private IMthRoundListService iMthRoundListService;

    /**
     * 查询赛事-场次-比赛评分;
     */
    @Override
    public MthRoundScoreVo queryById(Long id) {
        return baseConverter.entityToVo(baseRepo.getById(id));
    }

    /**
     * 查询赛事-场次-比赛评分;列表
     */
    @Override
    public Result<PageRsp<MthRoundScoreVo>> queryPageList(PageReq<MthRoundScoreBo> bo) {
        return null;
    }

    /**
     * 查询赛事-场次-比赛评分;列表
     */
    @Override
    public List<MthRoundScoreVo> queryList(MthRoundScoreBo bo) {
        return null;
    }

    /**
     * 新增赛事-场次-比赛评分;
     */
    @Override
    public Boolean insertByBo(MthRoundScoreBo bo) {
        MthRoundScore add = baseConverter.boToEntity(bo);
        validEntityBeforeSave(add);
        baseRepo.save(add);
        return true;
    }

    /**
     * 修改赛事-场次-比赛评分;
     */
    @Override
    public Boolean updateByBo(MthRoundScoreBo bo) {
        MthRoundScore update = baseConverter.boToEntity(bo);
        validEntityBeforeSave(update);
        return null;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(MthRoundScore entity) {

    }

    /**
     * 批量删除赛事-场次-比赛评分;
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if (isValid) {

        }
        return null;
    }

    /**
     * 场次晋级排名列表查询
     * 分页查询，根据分数排序分数相同根据姓名序号排序
     */
    @SneakyThrows
    @Override
    public PageRsp<MthRoundScoreRankingVo> getRoundScoreRankingList(Worker worker, PageReq<MthRoundScoreRankingBo> bo) {
        Long roundId = bo.getQuery().getRoundId();
        mthRoundInstanceRepo.findById(roundId).orElseThrow(() -> new BusinessException("未查询到该场次"));
        Map<String, Object> props = new HashMap<>();
        props.put("roundId", roundId);
        props.put("delFlag", 0);
        List<Long> roundListIds = mthRoundListRepo.findByProps(props).stream().map(MthRoundList::getId).collect(Collectors.toList());
        PageRsp<MthRoundScoreRankingVo> mthRoundScoreRankingVoPageRsp = mthRoundScoreDao.getRoundScoreRankingList(roundListIds, bo);
        List<Long> registerIdsByPage = mthRoundScoreRankingVoPageRsp.getResult().stream().map(MthRoundScoreRankingVo::getRegisterId).collect(Collectors.toList());
        List<MthRegisterDetail> registerDetailsVo = mthRegisterDetailRepo.findAllById(registerIdsByPage);
        Map<Long, MthRegisterDetail> collect = registerDetailsVo.stream().collect(Collectors.toMap(MthRegisterDetail::getId, item -> item));
        mthRoundScoreRankingVoPageRsp.getResult().forEach(item -> {
            //封装参数者姓名、身份证字段
//            item.setContestantName(collect.get(item.getRegisterId()));
        });
        return mthRoundScoreRankingVoPageRsp;
    }

    /**
     * 场次晋级排名列表保存、提交
     * 提交需要校验选择晋级数量，名次不能超过场次设定
     */
    @SneakyThrows
    @Override
    public boolean getRoundScoreRankingSaveList(Worker worker, MthRoundScoreRankingSaveBo bo) {
        List<MthRoundScoreRankingSaveListBo> mthRoundScoreRankingSaveListBos = bo.getMthRoundScoreRankingSaveListBos();
        mthRoundScoreRankingSaveListBos.forEach(v -> {
            if (ObjectUtil.isNull(v.getResult())) {
                throw new BusinessException("晋级状态不能为空！");
            }
        });
        Long mthRoundId = mthRoundScoreRankingSaveListBos.get(0).getRoundId();
        List<Long> mathRoundScoreIds = mthRoundScoreRankingSaveListBos.stream().map(MthRoundScoreRankingSaveListBo::getId).collect(Collectors.toList());
        //查询id不在列表的数据
//       List<MthRoundScoreRankingSaveListBo> mthRoundScoreRankingListInDB=mthRoundScoreDao.selectScoreListNoInIds(mathRoundScoreIds,mthRoundId);
        //校验名次数量设置
        this.checkRankingSettings(mthRoundScoreRankingSaveListBos, mthRoundId);
        List<MthMatchProcess> mthMatchProcesses = new LinkedList<>();
        List<MthRoundScore> mthRoundScores = new LinkedList<>();
        mthRoundScoreRankingSaveListBos.forEach(item -> {
            MthRoundScore mthRoundScore = baseRepo.getById(item.getId());
            mthRoundScore.setScore(item.getScore());
            mthRoundScore.setSort(item.getSort());
            mthRoundScore.setPrizeId(item.getPrizeId());
            mthRoundScore.setPrizeName(item.getPrizeName());
            mthRoundScore.setResult(item.getResult());
            mthRoundScore.setOtherPrizeId(item.getOtherPrizeId());
            mthRoundScore.setOtherPrizeName(item.getOtherPrizeName());
            mthRoundScore.setUpdateTime(new Date());
            mthRoundScores.add(mthRoundScore);
            /**设置评审晋级日志*/
//            this.delReviewAndPromotionLogs(mthMatchProcesses, mthRoundScore, item, worker);
        });
        baseRepo.saveAllAndFlush(mthRoundScores);
        /**查询下一场次是否存在*/

        //场景名单结束状态在结束场次时更新
//        if(ObjectUtil.isNull(nextRoundInstance.getId())){
//            /**更新round_list*/
//            List<MthRoundList> mthRoundListList=roundListRepo.findRoundListByRoundId(mthRoundId);
//            for (MthRoundList item:mthRoundListList){
//                item.setStatus(RoundListStatusEnums.END.getCode());
//                item.setUpdateTime(new Date());
//            }
//            roundListRepo.saveAllAndFlush(mthRoundListList);
//        }
        // 2023/11/21 liuchuan 重新进行排名
        List<MthRoundScore> mthRoundScoreList = mthRoundScoreDao.getRankingSortListByRoundIds(mthRoundId.toString());
        MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.getOne(mthRoundId);

        /**写入评审晋级日志*/
        iMthRoundListService.roundListRank(mthRoundInstance.getSocreMode(), mthRoundScoreList);
        mthMatchProcessRepo.saveAllAndFlush(mthMatchProcesses);
        /**提交操作 扭转场次状态*/
        if (2 == bo.getSaveOrSign()) {
            rankSubmit(mthRoundId);

            try {
                log.info("保存晋级及获奖记录，场次id：{}",mthRoundInstance.getId());
                iMthMatchProcessesService.saveAwardsAndPromotionLogs(mthRoundInstance);
            } catch (Exception e) {
                log.info("晋级及获奖记录保存失败:",e);
                e.printStackTrace();
            }
        }
        return true;
    }

    /**
     * @param roundId
     * @description: 提交排名
     * 判断当初场次状态：
     * 是待设置排名晋级2
     * 判断是否需要公示
     * 是->将场次状态设置为待公示20
     * 否->将场次状态设置为待结束3
     * 公示结束28:
     * 把场次状态修改为待公示20
     * 是待公示20:
     * 不处理
     * @return:
     * @author liuchuan
     * @date: 2023/11/21 14:55
     */
    public void rankSubmit(Long roundId) {
        MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.findByIdAndRoundType(roundId, RoundEnums.RoundType.INSTANCE.getCode());
        if (ObjectUtil.isEmpty(mthRoundInstance) && ObjectUtil.isNull(mthRoundInstance.getId())) {
            throw new BusinessException("系统错误，未查询到该场次");
        }
        log.info("roundState: "+mthRoundInstance.getRoundState());
        if (mthRoundInstance.getMatchState().equals(2)) {

            // 获取发送消息的信息
            // 通过areaId查询场次相关的承办方
            List<Long> loginIds = getMessagePushUser(mthRoundInstance.getAreaId());
            String matchName = getMatchName(mthRoundInstance.getMatchId());

            //状态为待设置排名
            if (mthRoundInstance.getPublicityFlag().equals(1)) {
                //需要公示
                mthRoundInstance.setMatchState(20);

                log.info("待公示通知:{} ",matchName);
                pushBusinessMsgService.pushStayPassMsg(loginIds,matchName);

            } else {
                mthRoundInstance.setMatchState(3);

                log.info("比赛待结束通知:{} ",matchName);
                pushBusinessMsgService.pushMatchFinish(loginIds,matchName);
            }

        } else if (mthRoundInstance.getMatchState().equals(28)) {
            //状态为公示结束   把场次状态修改为待公示20
            mthRoundInstance.setMatchState(20);
        }

        /*  原代码
        MthRoundInstance nextRoundInstance =this.queryNextRoundIns(mthRoundInstance);

        //nextRoundInstance=null 没有后续比赛-->结束比赛
        if (ObjectUtil.isNull(nextRoundInstance.getId())){
            mthRoundInstance.setMatchState(3);
            mthRoundInstance.setUpdateTime(new Date());
            mthRoundInstanceRepo.saveAndFlush(mthRoundInstance);
        }else {
            //关联新的mth_round_List记录
            this.dealNextRoundInfo(mthRoundInstance,nextRoundInstance);
//                处理下一场次mth_Round_Instance的记录状态
//                if ("online".equals(nextRoundInstance.getAuditMode())){
//                    nextRoundInstance.setMatchState(1);
//                }else {//线下赛事流转第二场次 状态设置为2 待设置晋级
//                    nextRoundInstance.setMatchState(2);
//                }
            nextRoundInstance.setUpdateTime(new Date());
            mthRoundInstanceRepo.saveAndFlush(nextRoundInstance);
            //当前场次状态变成4
             //* 10.16修正:当前场次状态变成3，需手动点击结束比赛，开起下一场次
            mthRoundInstance.setUpdateTime(new Date());
            mthRoundInstance.setMatchState(3);
            mthRoundInstanceRepo.saveAndFlush(mthRoundInstance);
        }
        */
        mthRoundInstance.setUpdateTime(new Date());
        mthRoundInstanceRepo.saveAndFlush(mthRoundInstance);


    }

    /**
     * 设置评审晋级日志
     */
    private void delReviewAndPromotionLogs(List<MthMatchProcess> mthMatchProcesses, MthRoundScore mthRoundScore, MthRoundScoreRankingSaveListBo item, Worker worker) {

        MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.findById(item.getRoundId()).orElseThrow(() -> new BusinessException("系统错误，未查询到该场次"));
        MthRoundList mthRoundList = mthRoundListRepo.getById(item.getMthRoundListId());
        MthMatchProcess matchProcessTitle = new MthMatchProcess();
        matchProcessTitle.setType("1");
        matchProcessTitle.setBizId(mthRoundInstance.getMatchId());
        matchProcessTitle.setUserId(mthRoundList.getRegisterId());
        if (mthRoundInstance.getSocreMode().equals(RoundEnums.ScoreMode.HUNDRED) ||
                mthRoundInstance.getSocreMode().equals(RoundEnums.ScoreMode.TEN_POINT)) {
            matchProcessTitle.setShowTitle(mthRoundInstance.getRoundName() + "评审结果: " + item.getScore() + "分");
        } else {
            matchProcessTitle.setShowTitle(mthRoundInstance.getRoundName() + "评审结果: " + item.getScore());
        }
        mthMatchProcesses.add(matchProcessTitle);

        MthMatchProcess matchProcessTime = new MthMatchProcess();
        matchProcessTime.setType("1");
        matchProcessTime.setBizId(mthRoundInstance.getMatchId());
        matchProcessTime.setUserId(mthRoundList.getRegisterId());
        matchProcessTitle.setShowTitle("评审时间：" + DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
        mthMatchProcesses.add(matchProcessTitle);

        MthMatchProcess matchProcessResultMark = new MthMatchProcess();
        if (mthRoundScore.getResult().equals(1)) {//是否晋级
            matchProcessResultMark.setType("1");
            matchProcessResultMark.setBizId(mthRoundInstance.getMatchId());
            matchProcessResultMark.setUserId(mthRoundList.getRegisterId());
            matchProcessResultMark.setShowTitle("是否晋级到下一场次：是 ");
            mthMatchProcesses.add(matchProcessResultMark);

            MthMatchProcess matchProcessResultTime = new MthMatchProcess();
            matchProcessResultTime.setType("1");
            matchProcessResultTime.setBizId(mthRoundInstance.getMatchId());
            matchProcessResultTime.setUserId(mthRoundList.getRegisterId());
            matchProcessResultTime.setShowTitle("晋级日期：" + DateUtil.format(new Date(), "yyyy-MM-dd"));
            mthMatchProcesses.add(matchProcessResultTime);

            MthMatchProcess matchProcessResult = new MthMatchProcess();
            matchProcessResult.setType("1");
            matchProcessResult.setBizId(mthRoundInstance.getMatchId());
            matchProcessResult.setUserId(mthRoundList.getRegisterId());
            //获奖名次：
            matchProcessResult.setShowTitle("获奖名次：" + mthRoundScore.getSort());
            mthMatchProcesses.add(matchProcessResult);


        } else {
            matchProcessResultMark.setType("1");
            matchProcessResultMark.setBizId(mthRoundInstance.getMatchId());
            matchProcessResultMark.setUserId(mthRoundList.getRegisterId());
            matchProcessResultMark.setShowTitle("是否晋级到下一场次：否 ");
            mthMatchProcesses.add(matchProcessResultMark);

            MthMatchProcess matchProcessResult = new MthMatchProcess();
            matchProcessResult.setType("1");
            matchProcessResult.setBizId(mthRoundInstance.getMatchId());
            matchProcessResult.setUserId(mthRoundList.getRegisterId());
            //获奖名次：
            matchProcessResult.setShowTitle("获奖名次：淘汰");
            mthMatchProcesses.add(matchProcessResult);
        }


    }


    /**
     * 校验名次设置
     */
    private void checkRankingSettings(List<MthRoundScoreRankingSaveListBo> bos, Long mthRoundId) {
        MthRoundInstance mthRoundInstance = mthRoundInstanceRepo.getById(mthRoundId);
        /**校验得分*/
        if (mthRoundInstance.getAuditMode().equals(RoundEnums.LineEnum.ONLINE.getCode())){
            if (mthRoundInstance.getSocreMode().equals("ten_point")) {
                bos.stream().forEach(item -> {
                    if (Double.parseDouble(item.getScore()) > 10) {
                        throw new BusinessException("得分不可超过10分!");
                    }
                });
            } else if (mthRoundInstance.getSocreMode().equals("hundred_point")) {
                bos.stream().forEach(item -> {
                    if (Double.parseDouble(item.getScore()) > 100) {
                        throw new BusinessException("得分不可超过100分!");
                    }
                });
            } else if (mthRoundInstance.getSocreMode().equals("pass")) {//pass
                bos.stream().forEach(item -> {
                    if ((!item.getScore().equals("通过")) && (!item.getScore().equals("不通过"))) {
                        throw new BusinessException("评分结果只能为通过或者不通过!");
                    }
                });
            } else if (mthRoundInstance.getSocreMode().equals("grad")) {
                bos.stream().forEach(item -> {
                    if ((!item.getScore().equals("A")) && (!item.getScore().equals("B")) &&
                            (!item.getScore().equals("C")) && (!item.getScore().equals("D")) &&
                            (!item.getScore().equals("C")) && (!item.getScore().equals("E"))) {
                        throw new BusinessException("评分结果只能为A、B、C、D、E!");
                    }
                });
            }
        }

        /**校验晋级人数*/
        checkPromotionNum(bos, mthRoundInstance);
        /**奖项校验*/
        checkPrizeNum(bos, mthRoundInstance);
        /*
            1、查询除页面数据外的所有数据库已设置奖项的数量 存放map中。 map<prizeId,num>
            2、计算页面传过的奖项数量 map<prizeId,num>
            3、查询比赛下所有奖项数量（忽略赛项、组别） 循环判断设置的数量是否超过
            4、
         */
//        List<MthRoundPrize> mthRoundPrizes= mthRoundPrizeRepo.findByRoundIdAndDelFlag(mthRoundId,0);
//        if (ObjectUtil.isNotEmpty(mthRoundPrizes)){
//            for (MthRoundPrize mthRoundPrize: mthRoundPrizes){
//                Long setPrizeNum = bos.stream().filter(item->{ return ObjectUtil.isNotNull(item.getPrizeName());})
//                        .filter(item->item.getPrizeName().equals(mthRoundPrize.getPrizeName())).count();
//                if (setPrizeNum>mthRoundPrize.getPrizeNum()){
//                    throw new BusinessException("该场次 "+mthRoundPrize.getPrizeName() +" 只能设置 " +mthRoundPrize.getPrizeNum()+" 个!");
//                }
//            }
//        }
    }



    public void checkPrizeNum(List<MthRoundScoreRankingSaveListBo> bos, MthRoundInstance mthRoundInstance) {
        
        //场次总人数
        HashMap<Long,Integer> countMap = totalCount(mthRoundInstance);

        //页面id
        List<Long> idList = bos.stream().map(MthRoundScoreRankingSaveListBo::getId).collect(Collectors.toList());
        //查询数据库中奖项数量
        Map<Long, Integer> map = mthRoundScoreDao.prizeCountByRoundId(mthRoundInstance.getId(), idList);
        //统计页面传过来的奖项数量
        Map<String, Long> prizeMap = bos.stream().filter(bo -> {
            return ObjectUtil.isNotEmpty(bo.getPrizeId());
        }).
                collect(Collectors.groupingBy(MthRoundScoreRankingSaveListBo::getPrizeId, Collectors.counting()));
        Map<String, Long> otherPrizeMap = bos.stream().filter(bo -> {
            return ObjectUtil.isNotEmpty(bo.getOtherPrizeId());
        }).
                collect(Collectors.groupingBy(MthRoundScoreRankingSaveListBo::getOtherPrizeId, Collectors.counting()));
        //数据库场次设置
        List<MthRoundPrize> roundPrizeList = mthRoundPrizeRepo.findByRoundIdAndDelFlag(mthRoundInstance.getId(), 0);
        if (CollectionUtil.isNotEmpty(roundPrizeList)) {
            roundPrizeList.stream().forEach(mthRoundPrize -> {
                Integer totalCount=0;
                if (ObjectUtil.isNull(mthRoundInstance.getPrizeType())||RoundEnums.PrizeType.MATCH.getCode().equals(mthRoundInstance.getPrizeType())){
                    totalCount = countMap.get(mthRoundInstance.getId());
                }
                //根据赛项
                if (RoundEnums.PrizeType.ITEM.getCode().equals(mthRoundInstance.getPrizeType())){
                    totalCount = countMap.get(mthRoundPrize.getItemId());
                }
                //根据组别
                if (RoundEnums.PrizeType.GROUP.getCode().equals(mthRoundInstance.getPrizeType())){
                    totalCount = countMap.get(mthRoundPrize.getGroupId());

                }
                Double setedNum = 0d;
                Double prizeNum=null;
                if (mthRoundPrize.getPrizeMode().equals(RoundEnums.PromotionMode.PERCENT.getCode())){
                     prizeNum = Math.ceil((Double.valueOf(totalCount.toString()) * mthRoundPrize.getPrizeNum() / 100D));
                }else {
                    prizeNum = Double.parseDouble(mthRoundPrize.getPrizeNum().toString());
                }
                if (ObjectUtil.isEmpty(mthRoundPrize.getPrizeNum())){
                    prizeNum = null;
                }
                if (map.containsKey(mthRoundPrize.getId())) {
                    setedNum = setedNum + map.get(mthRoundPrize.getId());
                }
                if (prizeMap.containsKey(mthRoundPrize.getId().toString())) {
                    log.info("prizeMap.get(mthRoundInstance.getId().toString()):{}",prizeMap.get(mthRoundPrize.getId().toString()));
                    setedNum = setedNum + prizeMap.get(mthRoundPrize.getId().toString()).intValue();
                }
                if (otherPrizeMap.containsKey(mthRoundPrize.getId().toString())) {
                    setedNum = setedNum + otherPrizeMap.get(mthRoundPrize.getId().toString()).intValue();
                }

                if (ObjectUtil.isNotEmpty(prizeNum)&&setedNum > prizeNum) {
                    String msg = "该场次,";
                    if ("item".equals(mthRoundInstance.getPrizeType())) {
                        msg = msg + mthRoundPrize.getItemName();
                    }
                    if ("group".equals(mthRoundInstance.getPrizeType())) {
                        msg = msg + mthRoundPrize.getItemName() + "," + mthRoundPrize.getGroupName() + ",";
                    }
                    throw new BusinessException(msg + mthRoundPrize.getPrizeName() + " 只能设置 " + prizeNum + " 个!");
                }
            });
        }
        log.info("map {}", map.size());
    }

    
    /**
     * @description: 根据获奖类型查询对应的总人数
     * @param
     * @return: 
     * @author liuchuan
     * @date: 2023/12/11 19:13
    */
    private HashMap<Long,Integer> totalCount(MthRoundInstance mthRoundInstance){
        HashMap<Long,Integer> map = new HashMap<Long, Integer>();
        //根据比赛
        if (ObjectUtil.isNull(mthRoundInstance.getPrizeType())||RoundEnums.PrizeType.MATCH.getCode().equals(mthRoundInstance.getPrizeType())){
            Integer totalCount = mthRoundListRepo.queryRoundListCount(mthRoundInstance.getId());
            map.put(mthRoundInstance.getId(),totalCount);
        }
        //根据赛项
        if (RoundEnums.PrizeType.ITEM.getCode().equals(mthRoundInstance.getPrizeType())){
            List<Object[]> countVoList = mthRoundListRepo.getCoutByItemId(mthRoundInstance.getId());
            for (Object[] objects:countVoList){
                BigInteger a = (BigInteger)objects[0];
                BigInteger b =(BigInteger) objects[1];
                map.put(Long.valueOf(a.toString()),Integer.valueOf(b.toString()));
            }

        }
        //根据组别
        if (RoundEnums.PrizeType.GROUP.getCode().equals(mthRoundInstance.getPrizeType())){
            List<Object[]> countVoList = mthRoundListRepo.getCountByGroupId(mthRoundInstance.getId());
            for (Object[] objects:countVoList){
                BigInteger a = (BigInteger)objects[0];
                BigInteger b =(BigInteger) objects[1];
                map.put(Long.valueOf(a.toString()),Integer.valueOf(b.toString()));
            }
        }
        return map;
    }
    /**
     * @param
     * @description: 校验晋级人数设置是否超过场次设置人数
     * @return:
     * @author liuchuan
     * @date: 2023/11/28 14:20
     */
    private void checkPromotionNum(List<MthRoundScoreRankingSaveListBo> bos, MthRoundInstance mthRoundInstance) {
        Long promotionNum = bos.stream().filter(item -> item.getResult().equals("1")).count();
        Integer roundPromotionNum = null;
        if (RoundEnums.PromotionMode.PERCENT.getCode().equals(mthRoundInstance.getPromotionMode())) {
            Integer totalCount = mthRoundListRepo.queryRoundListCount(mthRoundInstance.getId());
            Double count = Math.ceil((Double.valueOf(totalCount.toString()) * mthRoundInstance.getPromptionNum() / 100D));
            roundPromotionNum = count.intValue();
        } else {
            roundPromotionNum = mthRoundInstance.getPromptionNum();
        }
        //查询数据库中非当前分页数据 加上当页前端传的数据进行判断

        //当前页面id
        List<Long> idList = bos.stream().map(MthRoundScoreRankingSaveListBo::getId).collect(Collectors.toList());
        //数据库中晋级数量
        Integer dbCount = baseRepo.promotionNum(mthRoundInstance.getId(), idList);
        if ((dbCount + promotionNum.intValue()) > roundPromotionNum) {
            log.error("晋级人数设置错误。场次中晋级人数:{},页面中晋级人数：{},数据库中晋级人数:{}", roundPromotionNum, promotionNum, dbCount);
            throw new BusinessException(mthRoundInstance.getRoundName() + " 最大晋级人数为 : " + roundPromotionNum);
        }
    }

    /**
     * 处理下一场次相关信息mth_round_list、mth_round_score
     */
    @Override
    public void dealNextRoundInfo(MthRoundInstance mthRoundInstance, MthRoundInstance nextRoundInstance) {
        List<MthRoundList> mthRoundLists = new LinkedList<>();
        List<MthRoundScore> mthRoundScores = new LinkedList<>();

        List<MthRoundList> mthRoundList = mthRoundListDao.queryPromotionList(mthRoundInstance.getId());
        if (ObjectUtil.isNotEmpty(mthRoundList)) {
            mthRoundList.forEach(item -> {
                MthRoundList mrl = new MthRoundList();
                mrl.setRegisterId(item.getRegisterId());
                mrl.setRoundId(nextRoundInstance.getId());
                mrl.setUserId(item.getUserId());
                mrl.setUserName(item.getUserName());
                mrl.setDelFlag(0);
                mrl.setCreateId(item.getCreateId());
                mrl.setCreateBy(item.getCreateBy());
                mrl.setCreateTime(new Date());
                mrl.setUpdateId(item.getUpdateId());
                mrl.setUpdateBy(item.getUpdateBy());
                mrl.setUpdateTime(new Date());
                mrl.setStatus("RUNNING");
                //如果下一场次是不需要上传作品则将上一个场次作品带入
//                if (nextRoundInstance.getEntrySubmitMode().equals(RoundEnums.EntrySubmitMode.NEEDLESS.getCode())){
//                    mrl.setWorkId(item.getWorkId());
//                }
                mthRoundLists.add(mrl);
                //将上一场的状态设置为end
                item.setStatus(RoundListStatusEnums.END.getCode());
            });
            //将上一场的状态设置为end
            mthRoundListRepo.saveAllAndFlush(mthRoundList);
            mthRoundListRepo.saveAllAndFlush(mthRoundLists);
        }

//        List<MthRoundList> nextRoundLists = mthRoundListRepo.findRoundListByRoundId(nextRoundInstance.getId());
        //上面会将之前结束的场次名单查询出来  直接使用本场次名单
        List<MthRoundList> nextRoundLists = mthRoundLists;
        List<MthRoundScore> previousSceneRoundScores = mthRoundScoreDao.getRankingSortListByRoundId(mthRoundInstance.getId().toString());
        if (ObjectUtil.isNotEmpty(nextRoundLists)) {
            nextRoundLists.forEach(item -> {
                MthRoundScore mthRoundScore = new MthRoundScore();
                mthRoundScore.setRoundListId(item.getId());
                mthRoundScore.setStatus(0);
                if (ObjectUtil.isNotEmpty(previousSceneRoundScores) && previousSceneRoundScores.size() > 0) {
                    mthRoundScore.setReviewerId(previousSceneRoundScores.get(0).getReviewerId());
                    mthRoundScore.setReviewerName(previousSceneRoundScores.get(0).getReviewerName());
                }
                /**根据评分方式给与默认值*/
                if (!"online".equals(nextRoundInstance.getAuditMode())) {
                    if (nextRoundInstance.getSocreMode().equals("grad")) {
                        mthRoundScore.setScore("E");
                    } else if (nextRoundInstance.getSocreMode().equals("pass")) {
                        mthRoundScore.setScore("通过");
                    } else {
                        mthRoundScore.setScore("0");
                    }
                }
                mthRoundScores.add(mthRoundScore);
            });
            mthRoundScoreRepo.saveAllAndFlush(mthRoundScores);
        }
    }

    @Override
    public MthRoundInstance queryNextRoundIns(MthRoundInstance mthRoundInstance) {
        MthRoundInstance nextRoundInstance = new MthRoundInstance();
        /**先查找同level下，是否还有*/
        log.info("{},{} 寻找下一场次.",mthRoundInstance.getId(),mthRoundInstance.getRoundName());
        List<MthRoundInstance> allRoundInsList = mthRoundInstanceRepo.
                findByMatchIdAndRoundType(mthRoundInstance.getMatchId(),
                        RoundEnums.RoundType.INSTANCE.getCode());
        for (MthRoundInstance item : allRoundInsList) {
            //场次等级相同 区域相同 有下一场
            if (item.getRoundLevel().equals(mthRoundInstance.getRoundLevel())&&mthRoundInstance.getAreaId().equals(item.getAreaId())) {
                if (item.getRoundSort().equals(mthRoundInstance.getRoundSort() + 1)) {
                    log.info("{},{} 寻找下一场次.在同一等级找到下一场：{}，{}.",mthRoundInstance.getId(),mthRoundInstance.getRoundName(),
                            item.getId(),item.getRoundName());
                    return item;
                }
            }
        }
        //当前等级为区县级 而且没有找到后续场次 则继续找片区级 片区级需要片区的区域id是区县的上级
        if (mthRoundInstance.getRoundLevel().equals(RoundEnums.AreaLevel.COUNTY_SCHOOL.getLevel())) {
            MthRoundArea roundArea = mthRoundAreaRepo.getOne(mthRoundInstance.getAreaId());
            if (ObjectUtil.isNotEmpty(roundArea)) {
                List<MthRoundInstance> districtRoundList = allRoundInsList.stream()
                        .filter(round -> round.getRoundLevel().equals(RoundEnums.AreaLevel.DISTRICT.getLevel()))
                        .filter(round -> round.getAreaId().equals(roundArea.getParentId()))
                        .collect(Collectors.toList());
                if (ObjectUtil.isNotEmpty(districtRoundList)){
                    nextRoundInstance = districtRoundList.get(0);
                    log.info("{},{} 寻找下一场次.向片区晋级.：{}，{}.",mthRoundInstance.getId(),mthRoundInstance.getRoundName(),
                            nextRoundInstance.getId(),nextRoundInstance.getRoundName());
                    return nextRoundInstance;
                }
            }

        }
        //
        if (mthRoundInstance.getRoundLevel() < RoundEnums.AreaLevel.CITY.getLevel()) {
            //加一级进行查找
            for (MthRoundInstance item : allRoundInsList) {
                if (item.getRoundLevel().equals(mthRoundInstance.getRoundLevel() + 1)) {
                    if (item.getRoundSort().equals(1)) {
                        nextRoundInstance = item;
                        log.info("{},{} 寻找下一场次.直接等级+1寻找下一场.：{}，{}.",mthRoundInstance.getId(),mthRoundInstance.getRoundName(),
                                nextRoundInstance.getId(),nextRoundInstance.getRoundName());
                        return nextRoundInstance;
                    }
                }
            }
            //没找到 加两级
            for (MthRoundInstance item : allRoundInsList) {
                if (item.getRoundLevel().equals(mthRoundInstance.getRoundLevel() + 2)) {
                    if (item.getRoundSort().equals(1)) {
                        nextRoundInstance = item;
                        log.info("{},{} 寻找下一场次.直接等级+2寻找下一场.：{}，{}.",mthRoundInstance.getId(),mthRoundInstance.getRoundName(),
                                nextRoundInstance.getId(),nextRoundInstance.getRoundName());
                        return nextRoundInstance;
                    }
                }
            }
        }else {
            log.info("{},{} 寻找下一场次.没有找到后续场次",mthRoundInstance.getId(),mthRoundInstance.getRoundName());
        }
        return nextRoundInstance;
    }


    /**
     * 查看获奖名单
     */
    @Override
    public List<MthAwardInfoVo> queryAwardList(MthMatchManagerReqBo bo) {
        List<MthAwardInfoVo> result = mthRoundScoreDao.queryAwardList(bo);
        return result;
    }


    /** 获取短信推送人 */
    public List<Long> getMessagePushUser(Long areaId){
        List<Long> contractorIds = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(areaId)) {
            List<MthMatchContractor> contractors = mthMatchContractorRepo.findByAreaIdAndDelFlag(areaId, 0);
            if (ObjectUtil.isNotEmpty(contractors)) {
                contractorIds = contractors.stream().filter(item -> ObjectUtil.isNotEmpty(item.getContractorId()))
                    .map(MthMatchContractor::getContractorId).collect(Collectors.toList());
            }
        }
        return contractorIds;
    }


    /** 获取比赛名称 */
    public String getMatchName(Long matchId){
        // 查询比赛信息
        String matchName = "";
        if (ObjectUtil.isNotEmpty(matchId)){
            MthMatchInfo mthMatchInfo  = mthMatchInfoRepo.findById(matchId).orElseThrow(() -> new BusinessException("根据比赛id:"+ matchId + "未找到比赛信息!"));
            if (ObjectUtil.isNotEmpty(mthMatchInfo.getMatchName())){
                matchName = mthMatchInfo.getMatchName();
            }
        }
        return matchName;
    }


    public static void main(String[] args) {
        String a ="23.0";
        Double al = Double.valueOf(a);
        System.out.println(al);
        Long count = 5L;
        Integer per = 10;

        Double d = (Double.valueOf(count.toString()) * Double.parseDouble(per.toString())) / 100;

        System.out.println(d);
        System.out.println(Math.ceil(d));
        System.out.println((5 * 10) / 100);
        Double bePromotedNum = Math.ceil((5 * 10) / 100);

        System.out.println(bePromotedNum);
        List<MthRoundScore> byProps1 = new LinkedList<>();

        MthRoundScore mthRoundScore = new MthRoundScore();
        mthRoundScore.setScore(String.valueOf(110));
        mthRoundScore.setSort(110);
        byProps1.add(mthRoundScore);

        List<MthRoundScore> sort = CollectionUtil.sort(byProps1, Comparator.comparing(MthRoundScore::getScore));
        System.out.println(sort);
    }
}
