package com.crunii.micro.service.match.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.service.match.api.bo.PushAwardsBo;
import com.crunii.micro.service.match.api.bo.PushMsgTemplateBo;
import com.crunii.micro.service.match.constants.PushMsgConstants;
import com.crunii.micro.service.match.enums.RegisterAuditStatusEnums;
import com.crunii.micro.service.match.service.PushBusinessMsgService;
import com.crunii.micro.service.match.service.PushMsgService;
import com.google.common.collect.Maps;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 赛事业务消息推送
 *
 * @author xiangjunming
 * @date 2023/12/9
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class PushBusinessMsgServiceImpl implements PushBusinessMsgService {

    private final PushMsgService pushMsgService;

    @Override
    public void pushRegisteredMsg(Long userId, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.SCENE_NAME_REGISTERED);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.TITLE_REGISTERED);
        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushMsg(msgTemplateBo, userId, params);
    }

    @Override
    public void pushRegisterAuditMsg(Long userId, String reviewResult, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        if (Objects.equals(RegisterAuditStatusEnums.APPROVED.getCode(), reviewResult)) {
            msgTemplateBo.setSceneName(PushMsgConstants.SCENE_NAME_REGISTER_AUDIT_APPROVE);
            msgTemplateBo.setTitle(PushMsgConstants.TITLE_REGISTER_AUDIT_APPROVE);
        } else if (Objects.equals(RegisterAuditStatusEnums.REJECT.getCode(), reviewResult)) {
            msgTemplateBo.setSceneName(PushMsgConstants.SCENE_NAME_REGISTER_AUDIT_REJECT);
            msgTemplateBo.setTitle(PushMsgConstants.TITLE_REGISTER_AUDIT_REJECT);
        } else {
            return;
        }
        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushMsg(msgTemplateBo, userId, params);
    }

    @Override
    public void pushWorkNotUploadedMsg(List<Long> userIds, String competition, Date deadline) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.SCENE_NAME_WORK_NOT_UPLOADED);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.TITLE_WORK_NOT_UPLOADED);
        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        params.put("deadline", DateUtil.format(deadline, DateUtil.defaultDTPattern));
        pushMsgService.pushSameMsgs(msgTemplateBo, userIds, params);
    }

    @Override
    public void pushPlayMsg(List<Long> userIds, String competition, Date time, String place) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.SCENE_NAME_PLAY);
        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        params.put("time", DateUtil.format(time, DateUtil.defaultDTPattern));
        //设置TITLE
        if (StrUtil.isNotBlank(place)) {
            msgTemplateBo.setTitle(PushMsgConstants.TITLE_PLAY_LOCATION);
            params.put("place", place);
        } else {
            msgTemplateBo.setTitle(PushMsgConstants.TITLE_PLAY);
        }

        pushMsgService.pushSameMsgs(msgTemplateBo, userIds, params);
    }

    @Override
    public void pushAwardsMsg(List<PushAwardsBo> list) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.SCENE_NAME_AWARDS);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.TITLE_AWARDS);
        List<Long> userIds = Lists.newArrayList();
        List<Map<String, Object>> params = Lists.newArrayList();
        for (PushAwardsBo item : list) {
            userIds.add(item.getUserId());
            Map<String, Object> param = new HashMap<>(16);
            param.put("competition", item.getCompetition());
            param.put("award", item.getAward());
            params.add(param);
        }
        pushMsgService.pushMsgs(msgTemplateBo, userIds, params);
    }

    @Override
    public void pushPromotedMsg(List<Long> userIds, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.SCENE_NAME_PROMOTED);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.TITLE_PROMOTED);
        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushSameMsgs(msgTemplateBo, userIds, params);
    }

    @Override
    public void pushReceiveMsg(List<Long> userIds, String receiveType, String competition, Date time, String place) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.SCENE_NAME_RECEVIE);
        //设置TITLE
        if ("DELIVERY".equals(receiveType)) {
            msgTemplateBo.setTitle(PushMsgConstants.TITLE_RECEVIE_DELIVERY);
        } else {
            msgTemplateBo.setTitle(PushMsgConstants.TITLE_RECEVIE_SELF);
        }
        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        params.put("time", DateUtil.format(time, DateUtil.defaultDPattern));
        params.put("place", place);
        pushMsgService.pushSameMsgs(msgTemplateBo, userIds, params);
    }

    /**
     * 发送待审核通知消息
     */
    @Override
    public void pushCheckMsg(List<Long> userId) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.APPLY_CHECK_REGISTERED);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.APPLY_CHECK_REGISTERED_TITLE);
        Map<String, Object> params = new HashMap<>(16);
        pushMsgService.pushSameMsgs(msgTemplateBo, userId,params);
    }

    /**
     * 发送审核结果通知消息
     * @param userId
     */
    @Override
    public void pushResultMsg(Long userId,String stateName) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.APPLY_RESULT_REGISTERED);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.APPLY_RESULT_REGISTERED_TITLE);
        Map<String, Object> params = new HashMap<>(16);
        params.put("stateName",stateName);
        pushMsgService.pushMsg(msgTemplateBo, userId, params);
    }


    /**
     * 完善比赛信息通知
     */
    @Override
    public void pushImproveMatchInfoMsg(List<Long> userIds, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.IMPROVE_MATCH_INFO_SCENE_NAME);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.IMPROVE_MATCH_INFO_TITLE);
        //设置跳转
        Map<String, Object> pathJson = Maps.newHashMap();
        pathJson.put("type", "match-organizerList");
        pathJson.put("bizId", "");
        msgTemplateBo.setPathAndJson(JSONUtil.toJsonStr(pathJson));

        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushSameMsgs(msgTemplateBo, userIds, params);
    }

    @Override
    public void pushRegisterDeadlineMsg(List<Long> contractorIds, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.REGISTER_DEADLINE_SCENE_NAME);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.REGISTER_DEADLINE_TITLE);
        Map<String, Object> pathJson = Maps.newHashMap();
        pathJson.put("type", "match-processlist");
        pathJson.put("bizId", "");
        msgTemplateBo.setPathAndJson(JSONUtil.toJsonStr(pathJson));
        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushSameMsgs(msgTemplateBo, contractorIds, params);
    }

    @Override
    public void pushRegisterAuditMsg(List<Long> contractorIds, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.REGISTER_AUDIT_SCENE_NAME);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.REGISTER_AUDIT_TITLE);
        Map<String, Object> pathJson = Maps.newHashMap();
        pathJson.put("type", "match-processlist");
        pathJson.put("bizId", "");
        msgTemplateBo.setPathAndJson(JSONUtil.toJsonStr(pathJson));
        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushSameMsgs(msgTemplateBo, contractorIds, params);
    }

    @Override
    public void pushWorkReviewMsg(List<Long> contractorIds, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.WORK_REVIEW_SCENE_NAME);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.WORK_REVIEW_TITLE);
        Map<String, Object> pathJson = Maps.newHashMap();
        pathJson.put("type", "match-processlist");
        pathJson.put("bizId", "");
        msgTemplateBo.setPathAndJson(JSONUtil.toJsonStr(pathJson));
        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushSameMsgs(msgTemplateBo, contractorIds, params);
    }

    /**
     * 待设置排名晋级通知消息
     */
    @Override
    public void pushProMotionRandingMsg(List<Long> userIds, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.PROMOTION_RANKING_SCENE_NAME);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.PROMOTION_RANKING_REGISTERED_TITLE);
        //设置跳转
        Map<String, Object> pathJson = Maps.newHashMap();
        pathJson.put("type", "match-processlist");
        pathJson.put("bizId", "");
        msgTemplateBo.setPathAndJson(JSONUtil.toJsonStr(pathJson));

        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushSameMsgs(msgTemplateBo, userIds, params);
    }


    /**
     * 待公示通知
     */
    @Override
    public void pushStayPassMsg(List<Long> userIds, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.STAY_PASS_SCENE_NAME);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.STAY_PASS_TITLE);
        //设置跳转
        Map<String, Object> pathJson = Maps.newHashMap();
        pathJson.put("type", "match-processlist");
        pathJson.put("bizId", "");
        msgTemplateBo.setPathAndJson(JSONUtil.toJsonStr(pathJson));

        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushSameMsgs(msgTemplateBo, userIds, params);
    }


    /**
     * 公示结束通知
     */
    @Override
    public void pushPublicFinish(List<Long> userIds, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.PUBLIC_FINISH_SCENE_NAME);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.PUBLIC_FINISH_TITLE);
        //设置跳转
        Map<String, Object> pathJson = Maps.newHashMap();
        pathJson.put("type", "match-processlist");
        pathJson.put("bizId", "");
        msgTemplateBo.setPathAndJson(JSONUtil.toJsonStr(pathJson));

        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushSameMsgs(msgTemplateBo, userIds, params);
    }


    /**
     * 比赛待结束通知
     */
    @Override
    public void pushMatchFinish(List<Long> userIds, String competition) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.MATCH_FINISH_SCENE_NAME);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.MATCH_FINISH_TITLE);
        //设置跳转
        Map<String, Object> pathJson = Maps.newHashMap();
        pathJson.put("type", "match-processlist");
        pathJson.put("bizId", "");
        msgTemplateBo.setPathAndJson(JSONUtil.toJsonStr(pathJson));

        Map<String, Object> params = new HashMap<>(16);
        params.put("competition", competition);
        pushMsgService.pushSameMsgs(msgTemplateBo, userIds, params);
    }

    /**
     * 发送审核结果通知消息
     * @param userId
     */
    @Override
    public void pushMatchToApproveMsg(List<Long> userId,String matchName,String organizingName,String userName,String pathAndJson) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.MATCH_PUBLISH_TO_APPROVE);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.MATCH_PUBLISH_TO_APPROVE_TITLE);
        //设置额外参数
        msgTemplateBo.setPathAndJson(pathAndJson);
        Map<String, Object> params = new LinkedHashMap<>(16);
        params.put("organizingName",organizingName);
        params.put("userName",userName);
        params.put("matchName",matchName);
        pushMsgService.pushSameMsgs(msgTemplateBo, userId, params);
    }

    /**
     * 发送审核结果通知消息
     * @param userId
     */
    @Override
    public void pushMatchResultMsg(Long userId,String matchName,String organizingName,String userName,String stateName,String pathAndJson) {
        PushMsgTemplateBo msgTemplateBo = new PushMsgTemplateBo();
        //设置场景
        msgTemplateBo.setSceneName(PushMsgConstants.MATCH_PUBLISH_RESULT);
        //设置TITLE
        msgTemplateBo.setTitle(PushMsgConstants.MATCH_PUBLISH_RESULT_TITLE);
        //设置额外参数
        msgTemplateBo.setPathAndJson(pathAndJson);
        Map<String, Object> params = new LinkedHashMap<>(16);
        params.put("matchName",matchName);
        params.put("organizingName",organizingName);
        params.put("userName",userName);
        params.put("stateName",stateName);
        pushMsgService.pushMsg(msgTemplateBo, userId, params);
    }


}
