/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.obs.config.obs;

import com.crunii.micro.common.pool.PoolConfig;
import com.obs.services.ObsClient;
import org.apache.commons.pool2.impl.GenericObjectPool;
import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.InitializingBean;

/**
 * @author shenhs create 2023/5/25 11:38
 */
public class ObsClientPoolFactory implements FactoryBean<GenericObjectPool<ObsClient>>, DisposableBean, InitializingBean {
	private ObsClientFactory obsClientFactory;

	private GenericObjectPool<ObsClient> pool;

	private GenericObjectPoolConfig<ObsClient> config = null;

	/**
	 * @see DisposableBean#destroy()
	 */
	@Override
	public void destroy() throws Exception {
		if (pool != null) {
			pool.close();
		}

	}

	@Override public GenericObjectPool<ObsClient> getObject() throws Exception {
		return pool;
	}

	@Override public Class<?> getObjectType() {
		return GenericObjectPool.class;
	}

	/**
	 * @see FactoryBean#isSingleton()
	 */
	@Override
	public boolean isSingleton() {
		return false;
	}

	@Override public void afterPropertiesSet() throws Exception {
		if (config == null) {
			config = new GenericObjectPoolConfig<ObsClient>();
			config.setMaxTotal(PoolConfig.DEFAULT_MAX_TOTAL);
			config.setMaxIdle(PoolConfig.DEFAULT_MAX_IDLE);
			config.setMinIdle(PoolConfig.DEFAULT_MIN_IDLE);
			config.setMaxWaitMillis(PoolConfig.DEFAULT_MAX_WAIT_MILLIS);
			config.setMinEvictableIdleTimeMillis(PoolConfig.DEFAULT_MIN_EVICTABLE_IDLE_TIME_MILLIS);
			config.setSoftMinEvictableIdleTimeMillis(PoolConfig.DEFAULT_SOFT_MIN_EVICTABLE_IDLE_TIME_MILLIS);
			config.setTimeBetweenEvictionRunsMillis(PoolConfig.DEFAULT_TIME_BETWEEN_EVICTION_RUNS_MILLIS);
			config.setTestOnBorrow(PoolConfig.DEFAULT_TEST_ON_BORROW);
			config.setTestOnReturn(PoolConfig.DEFAULT_TEST_ON_RETURN);
			config.setTestOnCreate(PoolConfig.DEFAULT_TEST_ON_CREATE);
		}
		pool = new GenericObjectPool<ObsClient>(obsClientFactory, config);
	}

	public void setObsClientFactory(ObsClientFactory obsClientFactory){
		this.obsClientFactory = obsClientFactory;
	}

	/**
	 * @param config the config to set
	 */
	public void setConfig(GenericObjectPoolConfig<ObsClient> config) {
		this.config = config;
	}

	/**
	 * @param pool the pool to set
	 */
	public void setPool(GenericObjectPool<ObsClient> pool) {
		this.pool = pool;
	}


}
