package com.crunii.micro.service.obs.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.obs.ObsServiceCodeDto;
import com.crunii.micro.domain.dto.sys.FileBase64;
import com.crunii.micro.feign.client.dto.DownloadFileVo;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.obs.service.ObsObjectService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/1 14:44
 */
@Slf4j
@Api(tags = "文件操作")
@RestController
@RequestMapping("/auth/FileController")
public class FileController {

    @Resource
    private ObsObjectService obsObjectService;


    @ApiOperation("上传文件取消操作")
    @RequestMapping(value = "/operation", method = RequestMethod.POST)
    public void fileOperation(@RequestBody List<String> fileIds) {
        obsObjectService.updateFileOperation(fileIds);
    }

    @ApiOperation("Web上传文件")
    @RequestMapping(value = "/fileUpload", method = RequestMethod.POST)
    public Result<FileVo> fileUpload(@RequestBody MultipartFile file) throws Exception {
        if (ObjectUtil.isEmpty(file)) {
            return Result.ofFail("文件不能为空");
        }
        return obsObjectService.fileUpload(file, file.getOriginalFilename());
    }

    @ApiOperation("feign上传文件")
    @RequestMapping(value = "/feignFileUpload", method = RequestMethod.POST)
    public Result<FileVo> feignFileUpload(@RequestParam("file") MultipartFile file) throws Exception {
        if (ObjectUtil.isEmpty(file)) {
            return Result.ofFail("文件不能为空");
        }
        return obsObjectService.fileUpload(file, file.getOriginalFilename());
    }


    @ApiOperation("App上传文件")
    @RequestMapping(value = "/appFileUpload", method = RequestMethod.POST)
    public Result<FileVo> appFileUpload(@RequestBody FileBase64 fileBase64) throws Exception {
        if (ObjectUtil.isEmpty(fileBase64)) {
            return Result.ofFail("文件不能为空");
        }
        return obsObjectService.appFileUpload(fileBase64);
    }

    @ApiOperation("单文件下载")
    @RequestMapping(value = "/download", method = RequestMethod.GET)
    public Result<String> download(HttpServletRequest request, HttpServletResponse response, Long fileRecordId) throws Exception {
        // 0成功 1失败
        int i = obsObjectService.fileDownload(request, response, fileRecordId);
        if (i == 0) {
            return Result.ofSuccess("下载成功");
        }
        return Result.ofFail("下载失败");
    }

    @ApiOperation("附件下载")
    @RequestMapping(value = "/annexDownload", method = RequestMethod.GET)
    public void annexDownload(HttpServletRequest request, HttpServletResponse response, Long annexId) throws Exception {
        obsObjectService.annexDownload(request, response, annexId);
    }

    @ApiOperation("app下载量统计")
    @RequestMapping(value = "/appDownload", method = RequestMethod.GET)
    public Result<String> appDownload(@RequestParam("uploadId") Long uploadId) {
        return obsObjectService.appDownload(uploadId);
    }

    @ApiOperation("删除文件")
    @RequestMapping(value = "/delete", method = RequestMethod.GET)
    public Result<String> delete(Long uploadId) throws Exception {
        if (uploadId == null) {
            return Result.ofFail("上传uploadId不能为空");
        }
        return obsObjectService.deleteObject(uploadId);
    }

    @ApiOperation("通过fileKey删除文件")
    @RequestMapping(value = "/deleteListFileKey", method = RequestMethod.POST)
    public Result<String> deleteListFileKey(@RequestBody List<String> fileKey) {
        if (CollectionUtil.isEmpty(fileKey)) {
            return Result.ofFail("文件fileKey不能为空");
        }
        return obsObjectService.deleteObjectFileKey(fileKey);
    }


    @ApiOperation("判断对象是否存在")
    @RequestMapping(value = "/exits", method = RequestMethod.GET)
    public Result<String> exits(@RequestParam String objectKey) throws Exception {
        return obsObjectService.exists(objectKey);
    }

    @ApiOperation("获取redis的value")
    @RequestMapping(value = "/getKey", method = RequestMethod.GET)
    public Result<String> getKey(@RequestParam String key) {
        return obsObjectService.getKey(key);
    }

    @ApiOperation("标记obsFile的业务编码")
    @RequestMapping(value = "/updateObsServiceCode", method = RequestMethod.POST)
    public void updateObsServiceCode(@RequestBody List<ObsServiceCodeDto> obsServiceCodeDtos) {
        obsObjectService.updateObsServiceCode(obsServiceCodeDtos);
    }

    @ApiOperation("附件下载")
    @RequestMapping(value = "/downloadFlie", method = RequestMethod.GET)
    public Result<String> downloadFlie(HttpServletRequest request, HttpServletResponse response, Long fileId) throws Exception {
        // 0成功 1失败
        int i = obsObjectService.downloadFlie(request, response, fileId);
        if (i == 0) {
            return Result.ofSuccess("下载成功");
        }
        return Result.ofFail("下载失败");
    }


    @ApiOperation("文件下载")
    @RequestMapping(value = "/doDownloadFile", method = RequestMethod.GET)
    public Result<DownloadFileVo> doDownloadFile(@RequestParam("fileId") Long fileId) throws Exception {
       return Result.ofSuccess(obsObjectService.doDownloadFile(fileId));
    }

    @ApiOperation("单文件下载")
    @RequestMapping(value = "/downloadSetName", method = RequestMethod.GET)
    public Result<String> downloadSetName(HttpServletRequest request, HttpServletResponse response, Long fileRecordId,String fileName) throws Exception {
        // 0成功 1失败
        int i = obsObjectService.downloadSetName(request, response, fileRecordId,fileName);
        if (i == 0) {
            return Result.ofSuccess("下载成功");
        }
        return Result.ofFail("下载失败");
    }
}
