package com.crunii.micro.service.obs.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.obs.service.ObsBucketService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/1 15:25
 */
@Slf4j
@Api(tags = "OBS桶操作")
@RestController
@RequestMapping("/ObsBucketController/auth")
public class ObsBucketController {

    private final ObsBucketService obsBucketService;

    public ObsBucketController(ObsBucketService obsBucketService) {
        this.obsBucketService = obsBucketService;
    }

    @ApiOperation(value = "创建桶")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bucketName", value = "桶的名称", required = true)
    })
    @GetMapping(path = "/createBucket")
    public Result<String> createBucket(@RequestParam("bucketName") String bucketName) throws Exception {
        obsBucketService.createBucket(bucketName);
        return Result.ofSuccessMsg("创建桶成功");
    }

    @ApiOperation(value = "删除桶")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bucketName", value = "桶的名称", required = true)
    })
    @GetMapping(path = "/deleteBucket")
    public Result<String> deleteBucket(@RequestParam("bucketName") String bucketName) throws Exception {
        obsBucketService.deleteBucket(bucketName);
        return Result.ofSuccessMsg("删除桶成功");
    }

    @ApiOperation(value = "判断桶是否存在")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bucketName", value = "桶的名称", required = true)
    })
    @GetMapping(path = "/exists")
    public Result<Boolean> exists(@RequestParam("bucketName") String bucketName) throws Exception {
        return Result.ofSuccess(obsBucketService.exists(bucketName));
    }
}
