package com.crunii.micro.service.obs.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.obs.dto.*;
import com.crunii.micro.service.obs.service.VideoService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/10/11 18:57
 */
@Api(tags = "流媒体相关")
@RestController
@RequestMapping("/VideoController")
public class VideoController {

    @Resource
    private VideoService videoService;

    @ApiOperation("Web上传流媒体文件")
    @RequestMapping(value = "/auth/videoFileUpload", method = RequestMethod.POST)
    public Result<UploadVideoVo> videoFileUpload(@CurrWorker @ApiIgnore Worker worker,@RequestParam("file") MultipartFile file,@RequestParam("serverType") Integer serverType) throws Exception {
        return videoService.videoFileUpload(worker,file,serverType);
    }

    //@ApiOperation("文件上传完成通知转码")
    //@PostMapping("/auth/uploadFileNotification")
    //public Result<String> uploadFileNotification(@CurrWorker @ApiIgnore Worker worker,@RequestBody @Validated NoticeDto dto) {
    //    return Result.ofSuccess(videoService.uploadFileNotification(worker,dto));
    //}

    @ApiOperation("通过类型获取文件地址")
    @PostMapping("/auth/getFileUrl")
    public Result<String> getFileUrl(@CurrWorker @ApiIgnore Worker worker,@RequestBody @Validated FileDto dto) {
        return videoService.getFileUrl(worker,dto);
    }
    

    @ApiOperation("获取文件播放地址")
    @PostMapping("/playAddress")
    public Result<VideoPlatVo> playAddress(@CurrWorker @ApiIgnore Worker worker,@Validated @RequestBody VideoReqDto dto) {
        return videoService.queryRealTimeVideoAddress(worker,dto);
    }

    @ApiOperation("转码完成通知接口")
    @PostMapping("/streaming/updateCourseVideoStatus")
    public Result<String> courseVideoStatus(@Validated @RequestBody VideoDto videoDto) {
        return videoService.updateFileStatus(videoDto);
    }

    @ApiOperation(value = "获取视频文件下载地址")
    @PostMapping(path = "/auth/downloadVideo")
    public Result<VideoPlatVo> downloadVideo(@Validated @RequestBody VideoReqDto dto) {
        return videoService.downloadVideo(dto);
    }
}
