package com.crunii.micro.service.obs.core.client;

import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;

/**
 * @author wangchen create 2023/9/5 16:44
 */
public interface FileClient {

    /**
     * 获得客户端编号
     *
     * @return 客户端编号
     */
    String getObsName();

    /**
     * 上传文件
     * @param bucketName 桶
     * @param fileKey 文件
     * @param fileKey 文件key
     * @return success/false
     * @throws Exception
     */
    String uploadFile(String bucketName, InputStream fileInput, String fileKey) throws IOException;

    /**
     * 上传文件
     * @param bucketName 桶名
     * @param file 文件
     * @param fileKey 文件key
     * @return str
     */
    String uploadMultipartFileByPart(String bucketName, MultipartFile file, String fileKey);

    /**
     * 删除文件
     * @param bucket 桶
     * @param fileKey 文件id
     * @throws Exception 删除文件时，抛出 Exception 异常
     */
    void delete(String bucket,String fileKey) throws Exception;

    /**
     * 删除文件
     * @param bucket 桶
     * @throws Exception 删除文件时，抛出 Exception 异常
     */

    void createBucket(String bucket) throws Exception;

    /**
     * 获取文件刘
     * @param bucketName 桶
     * @param fileKey 文件key
     * @return 输入流
     * @throws Exception Exception
     */
    InputStream getObject(String bucketName, String fileKey) throws Exception;

    /**
     * 文件是否存在
     * @param bucketName
     * @param fileKey
     * @return
     */
    boolean doesObjectExist(String bucketName, String fileKey);

    void deleteBucket(String bucketName);

    boolean headBucket(String bucketName);

    String getUrl(String bucketName,String fileKey);
}
