package com.crunii.micro.service.obs.service;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.obs.ObsServiceCodeDto;
import com.crunii.micro.domain.dto.sys.FileBase64;
import com.crunii.micro.domain.entity.main.YcqObsFile;
import com.crunii.micro.feign.client.dto.DownloadFileVo;
import com.crunii.micro.feign.client.dto.FileVo;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author shenhs create 2023/5/25 17:31
 */
public interface ObsObjectService {
	/**
	 * 上传文件时取消上传,先标记,然后定时清除,防止资源浪费
	 * @param fileId obs文件主键id
	 */
	void updateFileOperation(List<String> fileId);

	/**
	 * 判断对象是否存在
	 * @param objectName 文件key
	 * @return str
	 * @throws Exception Exception
	 */
	Result<String> exists(String objectName) throws Exception;


	/**
	 * 上传文件
	 * @param uploadFile 文件
	 * @param fileName 文件名称带路径
	 * @return file信息
	 * @throws Exception Exception
	 */
	Result<FileVo> fileUpload(MultipartFile uploadFile, String fileName) throws Exception;

	/**
	 * app上传文件 base64
	 * @param fileBase64 base64图片
	 * @return data
	 * @throws Exception Exception
	 */
	Result<FileVo> appFileUpload(FileBase64 fileBase64) throws Exception;



	/**
	 * 下载文件
 	 * @param request 请求
	 * @param response 返回
	 * @param fileRecordId  文件详细表id
	 * @return 下载量
	 * @throws Exception Exception
	 */
	int fileDownload(HttpServletRequest request, HttpServletResponse response,Long fileRecordId) throws Exception;

	/**
	 * 附件下载
	 * @param request 请求
	 * @param response 响应
	 * @param annexId 附件id
	 * @throws Exception Exception
	 */
	void annexDownload(HttpServletRequest request, HttpServletResponse response,Long annexId) throws Exception;


	/**
	 * app下载量统计
	 * @param uploadId 上传文件信息表id
	 * @return str
	 */

	Result<String> appDownload(Long uploadId);

 	/**
	 * 删除(YcqUploadInfo)表文件
	 * @param uploadId 上传文件信息表id
	 * @return str
	 * @throws Exception Exception
	 */
	Result<String> deleteObject(Long uploadId) throws Exception;

	/**
	 * 删除(YCQ_OBS_FILE)表文件
	 * @param ycqObsFile 文件信息
	 * @throws Exception Exception
	 */
	void deleteObject2(YcqObsFile ycqObsFile) throws Exception;

	/**
	 * 通过fileKeu删除文件
	 * @param fileKey 文件keys
	 * @return str
	 */
	Result<String> deleteObjectFileKey(List<String> fileKey);

	/**
	 * 获取key
	 * @param key key
	 * @return str
	 */
	Result<String> getKey(String key);

	/**
	 * 标记obsFile的业务编码
	 * @param obsServiceCodeDtos 标记上传的文件类型
	 */
	void updateObsServiceCode(List<ObsServiceCodeDto> obsServiceCodeDtos);

	int downloadFlie(HttpServletRequest request, HttpServletResponse response, Long fileId);

	/**
	 * 文件下载
	 * @param fileId 文件ID
	 * @return DownloadFileVo
	 */
	DownloadFileVo doDownloadFile(Long fileId);

	/**
	 * 下载文件
	 * @param request 请求
	 * @param response 返回
	 * @param fileRecordId  文件详细表id
	 * @param fileName  文件名
	 * @return 下载量
	 * @throws Exception Exception
	 */
	int downloadSetName(HttpServletRequest request, HttpServletResponse response,Long fileRecordId,String fileName) throws Exception;
}
