package com.crunii.micro.service.obs.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.domain.entity.main.YcqFileRecord;
import com.crunii.micro.domain.entity.main.YcqObsFile;
import com.crunii.micro.domain.repo.main.YcqFileRecordRepo;
import com.crunii.micro.domain.repo.main.YcqObsFileRepo;
import com.crunii.micro.ds.uid.util.DateUtils;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.obs.dto.FileUploadStatus;
import com.crunii.micro.service.obs.service.FileStoreService;
import com.crunii.micro.service.obs.util.RedisUtils;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.Duration;
import java.time.Instant;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.*;
import java.util.concurrent.TimeUnit;

import static com.crunii.micro.service.obs.constants.ObsConstants.IS_PIC1;

@Service
public class FileStoreServiceImpl implements FileStoreService {

    //@Value("${upload.path}")
    private String uploadPath;

    @Resource
    private YcqObsFileRepo fileRepo;

    @Resource
    private YcqFileRecordRepo recordRepo;

    @Resource
    private RedisUtils redisUtils;

    private static final String EXIST = "exists!";

    @Override
    public Result<FileVo> uploadFile(MultipartFile uploadFile, String fileName){
        FileVo fileVo = new FileVo();
        String date = DateUtils.formatDate(new Date(), "yyyyMMdd");
        final String objectName = date + "/";
        YcqObsFile obsFile = new YcqObsFile();
        long size = uploadFile.getSize();
        obsFile.setFileSize(String.valueOf(size));
        fileVo.setFileSize(String.valueOf(size));
        fileVo.setFileName(fileName.substring(0, fileName.lastIndexOf(".")));
        // 获取文件信息
        try {
            InputStream inputStream = uploadFile.getInputStream();
            // 解析到文件后缀
            int index = fileName.lastIndexOf(".");
            String suffix;
            if (index == -1 || (suffix = fileName.substring(index + 1)).isEmpty()) {
                return Result.ofFail("文件后缀不能为空");
            }
            // 浏览器识别的图片类型
            Set<String> allowSuffix = new HashSet<>(Arrays.asList("jpg", "jpeg", "png", "gif", "bmp", "webp", "svg", "tiff", "ico"));
            if (allowSuffix.contains(suffix.toLowerCase())) {
                InputStream inPut = uploadFile.getInputStream();
                // 尝试IO文件，判断文件的合法性
                BufferedImage bufferedImage = ImageIO.read(inPut);
                // 除以1024可以得到以kb为单位的文件大小
                obsFile.setIsPic(IS_PIC1);
                obsFile.setWidth(bufferedImage.getWidth());
                obsFile.setHeight(bufferedImage.getHeight());

                fileVo.setIsPic(IS_PIC1);
                fileVo.setPicWidth(bufferedImage.getWidth());
                fileVo.setPicHeight(bufferedImage.getHeight());
            }
            // 判断文件夹对象是否存在
            if (Objects.equals(EXIST, this.exist(fileName).getData())){
                return Result.ofFail("服务器繁忙，请稍后再试");
            }
            // 获取文件信息
            String uuidName = UUIDUtil.getUUID() + "." + suffix;
            String objectKey = objectName + uuidName;
            String filePath = uploadPath  + objectKey;
            Path path = Paths.get(filePath);
            Instant start = Instant.now();
            long totalBytesRead = 0;
            FileOutputStream outputStream = new FileOutputStream(filePath);
            byte[] buffer = new byte[2048];
            int bytesRead;
            FileUploadStatus fileUploadStatus = new FileUploadStatus();
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
                totalBytesRead += bytesRead;
                Instant current = Instant.now();
                Duration duration = Duration.between(start, current);
                double transferPercentage = totalBytesRead / size;
                double seconds = duration.toMillis() / 1000.0;
                double uploadRate = totalBytesRead / 1024 / seconds; // 上传速率，M字节/秒
                fileUploadStatus.setPct(String.valueOf(transferPercentage));
                fileUploadStatus.setAvgSpeed(uploadRate);
                redisUtils.setEx(objectKey, JSONObject.toJSONString(fileUploadStatus), 2, TimeUnit.HOURS);
            }
            // 保存数据库
            obsFile.setBucketName(null);
            obsFile.setObjectKey(objectKey);
            obsFile.setCreateTime(new Date());
            obsFile.setFileName(fileName);
            obsFile.setType(suffix.toLowerCase());
            fileRepo.saveAndFlush(obsFile);
            // 响应实体
            fileVo.setFileId(String.valueOf(obsFile.getId()));
            fileVo.setFileUrl(filePath);
            fileVo.setFileType(suffix.toLowerCase());
            fileVo.setFileKey(objectKey);
            return Result.ofSuccess(fileVo);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("文件上传失败");
        }
    }

    @Override
    public int downloadFile(HttpServletRequest request, HttpServletResponse response, Long fileRecordId) throws IOException {
        YcqFileRecord fileRecord = recordRepo.findByFileRecordId(fileRecordId);
        if (ObjectUtil.isEmpty(fileRecord)) {
            return 1;
        }
        // 对象key
        String objectKey = fileRecord.getFileKey();
        if (ObjectUtil.isEmpty(objectKey)) {
            return 1;
        }
        String objectNewKey = objectKey.replaceFirst("%2F", "/");
        Path filePath = Paths.get(uploadPath, objectNewKey);
        if (!Files.exists(filePath)) {
            return 1;
        }
        long size = Files.size(filePath);
        BufferedOutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
        try (InputStream inputStream = Files.newInputStream(filePath)) {
            byte[] buffer = new byte[1024];
            int totalBytesRead = 0;
            int bytesRead;
            Instant start = Instant.now();
            FileUploadStatus fileUploadStatus = new FileUploadStatus();
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
                totalBytesRead += bytesRead;
                double transferPercentage = totalBytesRead / size;

                Instant current = Instant.now();
                Duration duration = Duration.between(start, current);
                double seconds = duration.toMillis() / 1000.0;
                double downloadRate = totalBytesRead / 1024/ seconds;

                fileUploadStatus.setPct(String.valueOf(transferPercentage));
                fileUploadStatus.setAvgSpeed(downloadRate);
                redisUtils.setEx(objectNewKey, JSONObject.toJSONString(fileUploadStatus), 2, TimeUnit.HOURS);
            }
        } catch (IOException e) {
            e.printStackTrace();
            return 1;
        }
        response.addHeader("Content-Disposition", "attachment;filename=" + objectNewKey);
        outputStream.flush();
        return 0;
    }


    @Override
    public Result<String> deleteFile(List<String> fileNameList) throws IOException {
        for (String fileName : fileNameList) {
            Path filePath = Paths.get(uploadPath + fileName);
            if (Files.exists(filePath)) {
                Files.delete(filePath);
            }
        }
        return Result.ofSuccess("删除成功");
    }

    @Override
    public Result<String> exist(String fileName) throws IOException {
        Path filePath = Paths.get(uploadPath + fileName);
        return Result.ofSuccess(Files.exists(filePath) ? "exists!" : "does not exist!");
    }

    @Override
    public Result<FileUploadStatus> getFileUploadPlan(String fileName){
        String s = redisUtils.get(fileName);
        if (StringUtils.isNotBlank(s)){
            FileUploadStatus fileUploadStatus = JSONObject.parseObject(s, FileUploadStatus.class);
            return Result.ofSuccess(fileUploadStatus);
        }
        return Result.ofFail("does not exist!");
    }
}

