package com.crunii.micro.service.obs.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.img.ImgUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.domain.dto.obs.ObsServiceCodeDto;
import com.crunii.micro.domain.dto.sys.FileBase64;
import com.crunii.micro.domain.entity.main.YcqFileRecord;
import com.crunii.micro.domain.entity.main.YcqMatchAnnex;
import com.crunii.micro.domain.entity.main.YcqObsFile;
import com.crunii.micro.domain.entity.main.YcqUploadInfo;
import com.crunii.micro.domain.repo.main.YcqFileRecordRepo;
import com.crunii.micro.domain.repo.main.YcqMatchAnnexRepo;
import com.crunii.micro.domain.repo.main.YcqObsFileRepo;
import com.crunii.micro.domain.repo.main.YcqUploadInfoRepo;
import com.crunii.micro.ds.uid.util.DateUtils;
import com.crunii.micro.feign.client.dto.DownloadFileVo;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.obs.core.client.FileClient;
import com.crunii.micro.service.obs.jdbc.ObsJdbc;
import com.crunii.micro.service.obs.service.FileConfigService;
import com.crunii.micro.service.obs.service.ObsObjectService;
import com.crunii.micro.service.obs.util.RedisUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.util.IOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundGeoOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.TimeUnit;

import static com.crunii.micro.service.obs.constants.ObsConstants.*;
import static java.lang.Boolean.TRUE;

/**
 * @author shenhs create 2023/5/25 17:31
 */
@Service
@Slf4j
public class ObsObjectServiceImpl implements ObsObjectService {


    @Resource
    private FileConfigService fileConfigService;

    /**
     * 桶
     */
    @Value("${obs.s3.bucketName}")
    private String bucketName;

    @Resource
    private YcqObsFileRepo fileRepo;

    @Resource
    private YcqFileRecordRepo recordRepo;

    @Resource
    private YcqMatchAnnexRepo annexRepo;

    @Resource
    private YcqUploadInfoRepo uploadInfoRepo;

    @Resource
    private ObsJdbc obsJdbc;

    @Autowired
    private RedisUtils redisUtils;

    @Override
    public void updateFileOperation(List<String> fileIds) {
        if (CollectionUtil.isEmpty(fileIds)) {
            return;
        }
        for (String fileId : fileIds) {
            YcqObsFile file = fileRepo.findByIdAndState(Long.valueOf(fileId), STATE_1);
            if (ObjectUtil.isEmpty(file)) {
                continue;
            }
            file.setState(STATE_0);
            fileRepo.saveAndFlush(file);
        }
    }

    @Override
    public Result<String> exists(String objectName) throws Exception {
        FileClient fileClient = fileConfigService.getFileClient();
        return Result.ofSuccess(fileClient.doesObjectExist(bucketName, objectName) ? "exists!" : "does not exist!");
    }


    @Override
    public Result<FileVo> fileUpload(MultipartFile uploadFile, String fileName) {
        FileVo fileVo = new FileVo();
        String date = DateUtils.formatDate(new Date(), "yyyyMMdd");
        final String objectName = date + "/";
        FileClient fileClient = fileConfigService.getFileClient();
        YcqObsFile obsFile = new YcqObsFile();
        long size = uploadFile.getSize() / 1024;
        obsFile.setFileSize(String.valueOf(size));
        fileVo.setFileSize(String.valueOf(size));
        fileVo.setFileName(fileName.substring(0, fileName.lastIndexOf(".")));
        // 获取文件信息
        try {
            InputStream inputStream = uploadFile.getInputStream();
            // 解析到文件后缀
            int index = fileName.lastIndexOf(".");
            String suffix;
            if (index == -1 || (suffix = fileName.substring(index + 1)).isEmpty()) {
                return Result.ofFail("文件后缀不能为空");
            }
            // 浏览器识别的图片类型
            Set<String> allowSuffix = new HashSet<>(Arrays.asList("jpg", "jpeg", "png", "gif", "bmp", "webp", "svg", "tiff", "ico"));
            if (allowSuffix.contains(suffix.toLowerCase())) {
                InputStream inPut = uploadFile.getInputStream();
                // 尝试IO文件，判断文件的合法性
                BufferedImage bufferedImage = ImageIO.read(inPut);
                // 除以1024可以得到以kb为单位的文件大小
                obsFile.setIsPic(IS_PIC1);
                obsFile.setWidth(bufferedImage.getWidth());
                obsFile.setHeight(bufferedImage.getHeight());

                fileVo.setIsPic(IS_PIC1);
                fileVo.setPicWidth(bufferedImage.getWidth());
                fileVo.setPicHeight(bufferedImage.getHeight());
            }

            // 获取文件信息
            String uuidName = UUIDUtil.getUUID() + "." + suffix;
            String objectKey = objectName + uuidName;
            String url = fileClient.uploadFile(bucketName, inputStream, objectKey);;
            // 屏蔽多线程上传，有问题
            /*if (size > MAX_SINGLE_UPLOAD) {
                url = fileClient.uploadMultipartFileByPart(bucketName, uploadFile, objectKey);
            } else {

            }*/
            // 保存数据库
            obsFile.setBucketName(bucketName);
            obsFile.setObjectKey(objectKey);
            obsFile.setCreateTime(new Date());
            obsFile.setFileName(fileName);
            obsFile.setType(suffix.toLowerCase());
            fileRepo.saveAndFlush(obsFile);
            // 响应实体
            fileVo.setFileId(String.valueOf(obsFile.getId()));
            fileVo.setFileUrl(url);
            fileVo.setFileType(suffix.toLowerCase());
            fileVo.setFileKey(objectKey);
            return Result.ofSuccess(fileVo);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("文件上传失败");
        }
    }

    @Override
    public Result<FileVo> appFileUpload(FileBase64 fileBase64) {
        String[] split = fileBase64.getImgBase64().split(StrUtil.COMMA);
        BufferedImage bufferedImage = ImgUtil.toImage(split[1]);
        ByteArrayInputStream jpg = ImgUtil.toStream(bufferedImage, "jpg");
        // 将Base64字符串解码为字节数组获取大小
        byte[] decodedBytes = Base64.getDecoder().decode(split[1]);
        String date = DateUtils.formatDate(new Date(), "yyyyMMdd");
        final String objectName = date + "/";
        // 文件名
        String fileName = UUIDUtil.getUUID() + ".jpg";
        FileClient fileClient = fileConfigService.getFileClient();
        try {
            InputStream inPut = ImgUtil.toStream(bufferedImage, "jpg");
            // 尝试IO文件，判断文件的合法性
            BufferedImage image = ImageIO.read(inPut);
            // 除以1024可以得到以kb为单位的文件大小
            String type = "jpg";
            long size = decodedBytes.length / 1024;
            int width = image.getWidth();
            int height = image.getHeight();
            // 获取文件信息
            String objectKey = objectName + fileName;
            String url = fileClient.uploadFile(bucketName, jpg, objectKey);
            // 保存数据库
            YcqObsFile obsFile = new YcqObsFile();
            obsFile.setBucketName(bucketName);
            obsFile.setObjectKey(objectKey);
            obsFile.setCreateTime(new Date());
            obsFile.setFileName(fileName);
            obsFile.setType(type);
            obsFile.setWidth(width);
            obsFile.setHeight(height);
            obsFile.setFileSize(String.valueOf(size));
            obsFile.setIsPic(IS_PIC1);
            fileRepo.saveAndFlush(obsFile);
            // 响应实体
            FileVo fileVo = new FileVo();
            fileVo.setFileId(String.valueOf(obsFile.getId()));
            fileVo.setFileUrl(url);
            fileVo.setFileType(type);
            fileVo.setFileSize(String.valueOf(size));
            fileVo.setPicWidth(width);
            fileVo.setPicHeight(height);
            fileVo.setFileKey(objectKey);
            fileVo.setIsPic(IS_PIC1);
            return Result.ofSuccess(fileVo);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("文件上传失败");
        }
    }

    @Override
    public int fileDownload(HttpServletRequest request, HttpServletResponse response, Long fileRecordId) {
        FileClient fileClient = fileConfigService.getFileClient();
        // 通过fileRecordId查找 objectKey
        YcqFileRecord fileRecord = recordRepo.findByFileRecordId(fileRecordId);
        if (ObjectUtil.isEmpty(fileRecord)) {
            return 1;
        }
        // 对象key
        String objectKey = fileRecord.getFileKey();
        if (ObjectUtil.isEmpty(objectKey)) {
            return 1;
        }
        try {
            String objectNewKey = objectKey.replaceFirst("%2F", "/");
            InputStream input = fileClient.getObject(bucketName, objectNewKey);
            //缓冲文件输出流
            BufferedOutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
            // 为防止 文件名出现乱码
            final String userAgent = request.getHeader("USER-AGENT");
            // IE浏览器
            if (userAgent.contains(MSIE)) {
                objectNewKey = URLEncoder.encode(objectNewKey, "UTF-8");
            } else {
                // google,火狐浏览器
                if (userAgent.contains(MOZILLA)) {
                    objectNewKey = new String(objectNewKey.getBytes(), "ISO8859-1");
                } else {
                    // 其他浏览器
                    objectNewKey = URLEncoder.encode(objectNewKey, "UTF-8");
                }
            }
            //设置让浏览器弹出下载提示框，而不是直接在浏览器中打开
            response.addHeader("Content-Disposition", "attachment;filename=" + objectNewKey);
            IOUtils.copy(input, outputStream);
            outputStream.flush();
            input.close();
            // 下载量处理
            NumOfDownloads(fileRecord.getUploadId());
        } catch (Exception e) {
            log.error("文件下载失败：{}", e.getMessage());
            return 1;
        }
        return 0;
    }

    @Override
    public void annexDownload(HttpServletRequest request, HttpServletResponse response, Long annexId) {
        FileClient fileClient = fileConfigService.getFileClient();
        // 通过fileRecordId查找 objectKey
        YcqMatchAnnex annex = annexRepo.findByAnnexIdAndState(annexId, STATE_1);
        if (ObjectUtil.isEmpty(annex)) {
            throw new BusinessException("下载失败");
        }
        // 对象key
        String objectKey = annex.getFileKey();
        if (ObjectUtil.isEmpty(objectKey)) {
            throw new BusinessException("下载失败");
        }
        try {
            String fileName = objectKey.replaceFirst("%2F", "/");
            InputStream input = fileClient.getObject(bucketName, fileName);
            //缓冲文件输出流
            BufferedOutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
            fileName = URLEncoder.encode(annex.getFileName(), "UTF-8") + fileName.substring(fileName.lastIndexOf("."));
            response.setContentType("application/x-download;charset=utf-8");
            response.setCharacterEncoding("utf-8");
            //设置让浏览器弹出下载提示框，而不是直接在浏览器中打开
            response.addHeader("Content-Disposition", "attachment;filename=" + fileName);
            IOUtils.copy(input, outputStream);
            outputStream.flush();
            outputStream.close();
            input.close();
        } catch (Exception e) {
            log.error("文件下载失败：{}", e.getMessage());
            throw new BusinessException("下载失败");
        }
    }

    @Override
    public Result<String> appDownload(Long uploadId) {
        // 判断uploadId是否存在
        YcqUploadInfo info = uploadInfoRepo.findByUploadIdAndState(uploadId, STATE_1);
        if (ObjectUtil.isNotEmpty(info)) {
            NumOfDownloads(uploadId);
        }
        return Result.ofSuccess("ok");
    }

    @Override
    public Result<String> deleteObject(Long uploadId) throws Exception {
        FileClient fileClient = fileConfigService.getFileClient();
        // 判断数据库文件是否存在
        YcqUploadInfo info = uploadInfoRepo.findByUploadIdAndState(uploadId, STATE_0);
        if (Objects.nonNull(info)) {
            info.setState(STATE_DEL);
            // 通过上传id查询fileKey
            List<String> fileKey = obsJdbc.queryFileKey(uploadId);
            if (CollectionUtil.isEmpty(fileKey)) {
                return Result.ofFail("删除失败,文件不存在");
            }
            for (String objectKey : fileKey) {
                // 判断OBS文件对象是否存在
                boolean exist = fileClient.doesObjectExist(bucketName, objectKey);
                if (!exist) {
                    continue;
                }
                fileClient.delete(bucketName, objectKey);
            }
            uploadInfoRepo.saveAndFlush(info);
        }
        return Result.ofSuccess("删除成功");
    }

    @Override
    public void deleteObject2(YcqObsFile ycqObsFile) throws Exception {
        FileClient fileClient = fileConfigService.getFileClient();
        YcqObsFile file = new YcqObsFile();
        BeanUtils.copyProperties(ycqObsFile, file);
        // 删除华为云obs对象
        if (fileClient.doesObjectExist(bucketName, file.getObjectKey())) {
            fileClient.delete(bucketName, file.getObjectKey());
        }
        // 设置删除成功状态
        file.setState(STATE_DEL);
        fileRepo.saveAndFlush(file);
    }

    @Override
    public Result<String> deleteObjectFileKey(List<String> fileKey) {
        FileClient fileClient;
        try {
            fileClient = fileConfigService.getFileClient();
            for (String objectKey : fileKey) {
                // 判断OBS文件对象是否存在
                String objectNewKey = objectKey.replaceFirst("%2F", "/");
                boolean exist = fileClient.doesObjectExist(bucketName, objectNewKey);
                if (!exist) {
                    continue;
                }
                fileClient.delete(bucketName, objectNewKey);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.info("删除失败,{}", e.getMessage());
        }
        return Result.ofSuccess("删除成功");
    }

    @Override
    public Result<String> getKey(String key) {
        return Result.ofSuccess(redisUtils.get(key));
    }

    @Override
    public void updateObsServiceCode(List<ObsServiceCodeDto> obsServiceCodeDtos) {
        for (ObsServiceCodeDto dto : obsServiceCodeDtos) {
            Optional<YcqObsFile> byId = fileRepo.findById(dto.getFileId());
            if (byId.isPresent()) {
                YcqObsFile ycqObsFile = byId.get();
                if (!dto.getServiceCode().equals(ycqObsFile.getServiceCode()) || !dto.getServiceId().equals(ycqObsFile.getServiceId())) {
                    ycqObsFile.setServiceId(dto.getServiceId());
                    ycqObsFile.setServiceCode(dto.getServiceCode());
                    fileRepo.saveAndFlush(ycqObsFile);
                }
            }

        }
//        obsJdbc.updateObsServiceCode(obsServiceCodeDtos);
        //判断【】和list个数是否相同
//        int size = obsServiceCodeDtos.size();
//        int length = ints.length;
//        if (size != length) {
//            throw new BusinessException("图片提交异常");
//        }
    }

    /**
     * 下载量处理
     */
    public void NumOfDownloads(Long uploadId) {
        StringRedisTemplate redisTemplate = redisUtils.getRedisTemplate();
        // 拼接前缀
        String key = PREFIX + uploadId;
        ValueOperations<String, String> forValue = redisTemplate.opsForValue();
        if (!TRUE.equals(redisTemplate.hasKey(key))) {
            //设置初值
            forValue.set(key, "0");
            BoundGeoOperations<String, String> ops = redisTemplate.boundGeoOps(key);
            //设置过期
            ops.expire(1, TimeUnit.DAYS);
        }
        forValue.increment(key, 1L);
    }

    @Override
    public int downloadFlie(HttpServletRequest request, HttpServletResponse response, Long fileId) {
        FileClient fileClient = fileConfigService.getFileClient();
        // 通过fileId查找 objectKey
        YcqObsFile obsFile = fileRepo.getOne(fileId);
        if (ObjectUtil.isEmpty(obsFile)) {
            return 1;
        }
        // 对象key
        String objectKey = obsFile.getObjectKey();
        if (ObjectUtil.isEmpty(objectKey)) {
            return 1;
        }
        try {
            String objectNewKey = objectKey.replaceFirst("%2F", "/");
            InputStream input = fileClient.getObject(bucketName, objectNewKey);
            //缓冲文件输出流
            BufferedOutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
            // 为防止 文件名出现乱码
            final String userAgent = request.getHeader("USER-AGENT");
            // IE浏览器
            if (userAgent.contains(MSIE)) {
                objectNewKey = URLEncoder.encode(objectNewKey, "UTF-8");
            } else {
                // google,火狐浏览器
                if (userAgent.contains(MOZILLA)) {
                    objectNewKey = new String(objectNewKey.getBytes(), "ISO8859-1");
                } else {
                    // 其他浏览器
                    objectNewKey = URLEncoder.encode(objectNewKey, "UTF-8");
                }
            }
            //设置让浏览器弹出下载提示框，而不是直接在浏览器中打开
            response.addHeader("Content-Disposition", "attachment;filename=" + objectNewKey);
            IOUtils.copy(input, outputStream);
            outputStream.flush();
            input.close();
            // 下载量处理
            NumOfDownloads(obsFile.getId());
        } catch (Exception e) {
            log.error("文件下载失败：{}", e.getMessage());
            return 1;
        }
        return 0;
    }

    /**
     * 文件下载，返回流数据
     *
     * @param fileId
     * @return
     */
    @Override
    public DownloadFileVo doDownloadFile(Long fileId) {
        DownloadFileVo downloadFileVo=new DownloadFileVo();
        FileClient fileClient = fileConfigService.getFileClient();
        // 通过fileId查找 objectKey
        YcqObsFile obsFile = fileRepo.getOne(fileId);
        if (ObjectUtil.isEmpty(obsFile)) {
            return downloadFileVo;
        }
        // 对象key
        String objectKey = obsFile.getObjectKey();
        if (ObjectUtil.isEmpty(objectKey)) {
            return downloadFileVo;
        }
        downloadFileVo.setFileName(obsFile.getFileName());
        downloadFileVo.setFileSize(obsFile.getFileSize());
        downloadFileVo.setFileType(obsFile.getType());
        String objectNewKey = objectKey.replaceFirst("%2F", "/");
        try {
            InputStream input = fileClient.getObject(bucketName, objectNewKey);
            byte[] bytes = IOUtils.toByteArray(input);
            String base64= Base64.getEncoder().encodeToString(bytes);
            downloadFileVo.setFile(base64);
        } catch (Exception e) {
            log.info("下载文件异常:fileId={}",fileId);
            e.printStackTrace();
        }
        return downloadFileVo;
    }

    @Override
    public int downloadSetName(HttpServletRequest request, HttpServletResponse response, Long fileRecordId,String fileName) {
        FileClient fileClient = fileConfigService.getFileClient();
        // 通过fileRecordId查找 objectKey
        YcqObsFile fileRecord = fileRepo.getOne(fileRecordId);
        if (ObjectUtil.isEmpty(fileRecord)) {
            return 1;
        }
        // 对象key
        String objectKey = fileRecord.getObjectKey();
        if (ObjectUtil.isEmpty(objectKey)) {
            return 1;
        }
        try {
            String objectNewKey = objectKey.replaceFirst("%2F", "/");
            InputStream input = fileClient.getObject(bucketName, objectNewKey);
            //缓冲文件输出流
            BufferedOutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
            // 为防止 文件名出现乱码
            final String userAgent = request.getHeader("USER-AGENT");
            // IE浏览器
            if (userAgent.contains(MSIE)) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                // google,火狐浏览器
                if (userAgent.contains(MOZILLA)) {
                    fileName = new String(fileName.getBytes(), "ISO8859-1");
                } else {
                    // 其他浏览器
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                }
            }
            //设置让浏览器弹出下载提示框，而不是直接在浏览器中打开
            response.addHeader("Content-Disposition", "attachment;filename=" + fileName);
            IOUtils.copy(input, outputStream);
            outputStream.flush();
            input.close();
            // 下载量处理
//            NumOfDownloads(fileRecord.getUploadId());
        } catch (Exception e) {
            log.error("文件下载失败：{}", e.getMessage());
            return 1;
        }
        return 0;
    }
}
