/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.obs.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.img.ImgUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.domain.dto.obs.ObsServiceCodeDto;
import com.crunii.micro.domain.dto.sys.FileBase64;
import com.crunii.micro.domain.entity.main.YcqFileRecord;
import com.crunii.micro.domain.entity.main.YcqMatchAnnex;
import com.crunii.micro.domain.entity.main.YcqObsFile;
import com.crunii.micro.domain.entity.main.YcqUploadInfo;
import com.crunii.micro.domain.repo.main.YcqFileRecordRepo;
import com.crunii.micro.domain.repo.main.YcqMatchAnnexRepo;
import com.crunii.micro.domain.repo.main.YcqObsFileRepo;
import com.crunii.micro.domain.repo.main.YcqUploadInfoRepo;
import com.crunii.micro.ds.uid.util.DateUtils;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.obs.config.obs.ObsClientPoolFactory;
import com.crunii.micro.service.obs.dto.FileUploadStatus;
import com.crunii.micro.service.obs.jdbc.ObsJdbc;
import com.crunii.micro.service.obs.service.ObsObjectService2;
import com.crunii.micro.service.obs.util.MFileUtil;
import com.crunii.micro.service.obs.util.RedisUtils;
import com.obs.services.ObsClient;
import com.obs.services.exception.ObsException;
import com.obs.services.model.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.pool2.impl.GenericObjectPool;
import org.apache.poi.util.IOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundGeoOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import static com.crunii.micro.service.obs.constants.ObsConstants.*;
import static java.lang.Boolean.TRUE;

/**
 * @author shenhs create 2023/5/25 17:31
 */
@Service
@Slf4j
@Deprecated
public class ObsObjectServiceImpl2 implements ObsObjectService2 {
    /**
     * 桶
     */
    @Value("${obs.bucketName}")
    private String bucketName;

    @Resource
    private YcqObsFileRepo fileRepo;

    @Resource
    private YcqFileRecordRepo recordRepo;

    @Resource
    private YcqMatchAnnexRepo annexRepo;

    @Resource
    private YcqUploadInfoRepo uploadInfoRepo;

    @Resource
    private ObsJdbc obsJdbc;

    private final ObsClientPoolFactory obsClientPoolFactory;

    private GenericObjectPool<ObsClient> pool;

    public ObsObjectServiceImpl2(ObsClientPoolFactory obsClientPoolFactory) {
        this.obsClientPoolFactory = obsClientPoolFactory;
    }

    @Autowired
    private RedisUtils redisUtils;

    @Override
    public void afterPropertiesSet() throws Exception {
        pool = obsClientPoolFactory.getObject();
    }

    @Override
    public void updateFileOperation(List<String> fileIds) {
        if (CollectionUtil.isEmpty(fileIds)){
            return;
        }
        for (String fileId : fileIds) {
            YcqObsFile file = fileRepo.findByIdAndState(Long.valueOf(fileId), STATE_1);
            if (ObjectUtil.isEmpty(file)) {
                continue;
            }
            file.setState(STATE_0);
            fileRepo.saveAndFlush(file);
        }
    }

    @Override
    public Result<String> exists(String objectName) throws Exception {
        ObsClient obsClient = pool.borrowObject();
        return Result.ofSuccess(obsClient.doesObjectExist(bucketName, objectName) ? "exists!" : "does not exist!");
    }

    @Override
    public PutObjectResult putObject(InputStream inputStream) throws Exception {
        return putObject(IdUtil.fastUUID(), inputStream);
    }

    @Override
    public PutObjectResult putObject(String fileKey, InputStream inputStream) throws Exception {
        String date = DateUtils.formatDate(new Date(), "yyyyMMdd");
        final String keySuffixWithSlash = date + "/";
        ObsClient obsClient = pool.borrowObject();
        try {
            return obsClient.putObject(bucketName, keySuffixWithSlash + fileKey, inputStream);
        } catch (Exception e) {
            throw e;
        } finally {
            pool.returnObject(obsClient);
        }

    }

    @Override
    public Result<FileVo> fileUpload(MultipartFile uploadFile, String fileName) throws Exception {
        FileVo fileVo = new FileVo();
        String date = DateUtils.formatDate(new Date(), "yyyyMMdd");
        final String objectName = date + "/";
        ObsClient obsClient = pool.borrowObject();
        YcqObsFile obsFile = new YcqObsFile();
        long size = uploadFile.getSize() / 1024;
        obsFile.setFileSize(String.valueOf(size));
        fileVo.setFileSize(String.valueOf(size));
        fileVo.setFileName(fileName.substring(0, fileName.lastIndexOf(".")));
        // 获取文件信息
        try {
            InputStream inputStream = uploadFile.getInputStream();
            // 解析到文件后缀
            int index = fileName.lastIndexOf(".");
            String suffix;
            if (index == -1 || (suffix = fileName.substring(index + 1)).isEmpty()) {
                return Result.ofFail("文件后缀不能为空");
            }
            // 浏览器识别的图片类型
            Set<String> allowSuffix = new HashSet<>(Arrays.asList("jpg", "jpeg", "png", "gif", "bmp", "webp", "svg", "tiff", "ico"));
            if (allowSuffix.contains(suffix.toLowerCase())) {
                InputStream inPut = uploadFile.getInputStream();
                // 尝试IO文件，判断文件的合法性
                BufferedImage bufferedImage = ImageIO.read(inPut);
                // 除以1024可以得到以kb为单位的文件大小
                obsFile.setIsPic(IS_PIC1);
                obsFile.setWidth(bufferedImage.getWidth());
                obsFile.setHeight(bufferedImage.getHeight());

                fileVo.setIsPic(IS_PIC1);
                fileVo.setPicWidth(bufferedImage.getWidth());
                fileVo.setPicHeight(bufferedImage.getHeight());
            }
            // 判断文件夹对象是否存在
            boolean exist = obsClient.doesObjectExist(bucketName, objectName);
            if (!exist) {
                // 创建文件夹对象
                obsClient.putObject(bucketName, objectName, new ByteArrayInputStream(new byte[0]));
                log.info("创建文件夹成功,文件名称[{}]", objectName);
            }
            // 获取文件信息
            long available = inputStream.available();
            String uuidName = UUIDUtil.getUUID() + "." + suffix;
            String objectKey = objectName + uuidName;
            PutObjectRequest request = new PutObjectRequest(bucketName, objectKey, inputStream);
            ObjectMetadata objectMetadata = new ObjectMetadata();
            objectMetadata.setContentLength(available);
            request.setMetadata(objectMetadata);
            // 设置公共读
            request.setAcl(AccessControlList.REST_CANNED_PUBLIC_READ);
            PutObjectResult result = obsClient.putObject(request);
            // 保存数据库
            obsFile.setBucketName(result.getBucketName());
            obsFile.setObjectKey(result.getObjectKey());
            //obsFile.setObjectUrl(result.getObjectUrl());
            obsFile.setCreateTime(new Date());
            obsFile.setFileName(fileName);
            obsFile.setType(suffix.toLowerCase());
            fileRepo.saveAndFlush(obsFile);
            // 响应实体
            fileVo.setFileId(String.valueOf(obsFile.getId()));
            fileVo.setFileUrl(result.getObjectUrl());
            fileVo.setFileType(suffix.toLowerCase());
            fileVo.setFileKey(objectKey);
            return Result.ofSuccess(fileVo);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("文件上传失败");
        } finally {
            pool.returnObject(obsClient);
        }
    }

    @Override
    public Result<FileVo> appFileUpload(FileBase64 fileBase64) throws Exception {
        String[] split = fileBase64.getImgBase64().split(StrUtil.COMMA);
        BufferedImage bufferedImage = ImgUtil.toImage(split[1]);
        ByteArrayInputStream jpg = ImgUtil.toStream(bufferedImage, "jpg");
        // 将Base64字符串解码为字节数组获取大小
        byte[] decodedBytes = Base64.getDecoder().decode(split[1]);
        String date = DateUtils.formatDate(new Date(), "yyyyMMdd");
        final String objectName = date + "/";
        // 文件名
        String fileName = UUIDUtil.getUUID() + ".jpg";
        ObsClient obsClient = pool.borrowObject();
        try {
            // 判断文件夹对象是否存在
            boolean exist = obsClient.doesObjectExist(bucketName, objectName);
            if (!exist) {
                // 创建文件夹对象
                obsClient.putObject(bucketName, objectName, new ByteArrayInputStream(new byte[0]));
                log.info("创建文件夹成功,文件名称[{}]", objectName);
            }
            InputStream inPut = ImgUtil.toStream(bufferedImage, "jpg");

            // 尝试IO文件，判断文件的合法性
            BufferedImage image = ImageIO.read(inPut);
            // 除以1024可以得到以kb为单位的文件大小
            String type = "jpg";
            long size = decodedBytes.length / 1024;
            int width = image.getWidth();
            int height = image.getHeight();
            // 获取文件信息
            long available = jpg.available();
            String objectKey = objectName + fileName;
            PutObjectRequest request = new PutObjectRequest(bucketName, objectKey, jpg);
            ObjectMetadata objectMetadata = new ObjectMetadata();
            objectMetadata.setContentLength(available);
            request.setMetadata(objectMetadata);
            // 设置公共读
            request.setAcl(AccessControlList.REST_CANNED_PUBLIC_READ);
            PutObjectResult result = obsClient.putObject(request);

            // 保存数据库
            YcqObsFile obsFile = new YcqObsFile();
            obsFile.setBucketName(result.getBucketName());
            obsFile.setObjectKey(result.getObjectKey());
            //obsFile.setObjectUrl(result.getObjectUrl());
            obsFile.setCreateTime(new Date());
            obsFile.setFileName(fileName);
            obsFile.setType(type);
            obsFile.setWidth(width);
            obsFile.setHeight(height);
            obsFile.setFileSize(String.valueOf(size));
            obsFile.setIsPic(IS_PIC1);
            fileRepo.saveAndFlush(obsFile);
            // 响应实体
            FileVo fileVo = new FileVo();
            fileVo.setFileId(String.valueOf(obsFile.getId()));
            fileVo.setFileUrl(result.getObjectUrl());
            fileVo.setFileType(type);
            fileVo.setFileSize(String.valueOf(size));
            fileVo.setPicWidth(width);
            fileVo.setPicHeight(height);
            fileVo.setFileKey(objectKey);
            fileVo.setIsPic(IS_PIC1);
            return Result.ofSuccess(fileVo);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("文件上传失败");
        } finally {
            pool.returnObject(obsClient);
        }
    }

    @Override
    public Result<FileUploadStatus> getFileUploadPlan(String objectName) throws Exception {
        ObsClient obsClient = pool.borrowObject();
        FileUploadStatus fileUploadStatus = new FileUploadStatus();
        try {
            GetObjectRequest request = new GetObjectRequest(bucketName, objectName);
            request.setProgressListener(status -> {
                // 获取上传平均速率
                fileUploadStatus.setAvgSpeed(status.getAverageSpeed());
                // 获取上传进度百分比
                fileUploadStatus.setPct(String.valueOf(status.getTransferPercentage()));
            });
            // 每下载1MB数据反馈下载进度
            request.setProgressInterval(1024 * 1024L);
            ObsObject obsObject = obsClient.getObject(request);
            // 读取对象内容
            InputStream input = obsObject.getObjectContent();
            byte[] b = new byte[1024];
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            int len;
            while ((len = input.read(b)) != -1) {
                //将每一次的数据写入缓冲区
                byteArrayOutputStream.write(b, 0, len);
            }
            byteArrayOutputStream.close();
            input.close();
            return Result.ofSuccess(fileUploadStatus);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            pool.returnObject(obsClient);
        }
        return null;
    }

    @Override
    public Result<FileUploadStatus> getFileDownPlan(String objectName) throws Exception {
        ObsClient obsClient = pool.borrowObject();
        FileUploadStatus fileUploadStatus = new FileUploadStatus();
        GetObjectRequest request = new GetObjectRequest(bucketName, objectName);
        request.setProgressListener(status -> {
            // 获取下载平均速率
            fileUploadStatus.setAvgSpeed(status.getAverageSpeed());
            // 获取下载进度百分比
            fileUploadStatus.setPct(String.valueOf(status.getTransferPercentage()));
        });
        // 每下载1MB数据反馈下载进度
        request.setProgressInterval(1024 * 1024L);
        ObsObject obsObject = obsClient.getObject(request);
        // 读取对象内容
        InputStream input = obsObject.getObjectContent();
        byte[] b = new byte[1024];
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        int len;
        while ((len = input.read(b)) != -1) {
            bos.write(b, 0, len);
        }
        bos.close();
        input.close();
        return Result.ofSuccess(fileUploadStatus);
    }

    @Override
    public PutObjectResult uploadFile(File file) throws Exception {
        ObsClient obsClient = pool.borrowObject();
        try {
            //转为File
            String filename = file.getName();
            //后缀
            String prefix = filename.substring(filename.lastIndexOf(".") + 1);
            //customer_YYYYMMDDHHMM
            String uuid = UUID.randomUUID().toString().replace("-", "");
            //文件全路径
            String fileName = uuid + "." + prefix;
            return obsClient.putObject(bucketName, fileName.substring(0, fileName.lastIndexOf("/")) + "/" + fileName, file);
        } finally {
            pool.returnObject(obsClient);
        }
    }

    @Override
    public int fileDownload(HttpServletRequest request, HttpServletResponse response, Long fileRecordId) throws Exception {
        ObsClient obsClient = pool.borrowObject();
        // 通过fileRecordId查找 objectKey
        YcqFileRecord fileRecord = recordRepo.findByFileRecordId(fileRecordId);
        if (ObjectUtil.isEmpty(fileRecord)) {
            return 1;
        }
        // 对象key
        String objectKey = fileRecord.getFileKey();
        if (ObjectUtil.isEmpty(objectKey)) {
            return 1;
        }
        try {
            String objectNewKey = objectKey.replaceFirst("%2F", "/");
            ObsObject obsObject = obsClient.getObject(bucketName, objectNewKey);
            InputStream input = obsObject.getObjectContent();
            //缓冲文件输出流
            BufferedOutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
            // 为防止 文件名出现乱码
            final String userAgent = request.getHeader("USER-AGENT");
            // IE浏览器
            if (userAgent.contains(MSIE)) {
                objectNewKey = URLEncoder.encode(objectNewKey, "UTF-8");
            } else {
                // google,火狐浏览器
                if (userAgent.contains(MOZILLA)) {
                    objectNewKey = new String(objectNewKey.getBytes(), "ISO8859-1");
                } else {
                    // 其他浏览器
                    objectNewKey = URLEncoder.encode(objectNewKey, "UTF-8");
                }
            }
            //设置让浏览器弹出下载提示框，而不是直接在浏览器中打开
            response.addHeader("Content-Disposition", "attachment;filename=" + objectNewKey);
            IOUtils.copy(input, outputStream);
            outputStream.flush();
            input.close();
            // 下载量处理
            NumOfDownloads(fileRecord.getUploadId());
        } catch (Exception e) {
            log.error("文件下载失败：{}", e.getMessage());
            return 1;
        } finally {
            pool.returnObject(obsClient);
        }
        return 0;
    }

    @Override
    public void annexDownload(HttpServletRequest request, HttpServletResponse response, Long annexId) throws Exception {
        ObsClient obsClient = pool.borrowObject();
        // 通过fileRecordId查找 objectKey
        YcqMatchAnnex annex = annexRepo.findByAnnexIdAndState(annexId, STATE_1);
        if (ObjectUtil.isEmpty(annex)) {
            throw new BusinessException("下载失败");
        }
        // 对象key
        String objectKey = annex.getFileKey();
        if (ObjectUtil.isEmpty(objectKey)) {
            throw new BusinessException("下载失败");
        }
        try {
            String fileName = objectKey.replaceFirst("%2F", "/");
            ObsObject obsObject = obsClient.getObject(bucketName, fileName);
            InputStream input = obsObject.getObjectContent();
            //缓冲文件输出流
            BufferedOutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
            fileName = URLEncoder.encode(annex.getFileName(), "UTF-8") + fileName.substring(fileName.lastIndexOf("."));
            response.setContentType("application/x-download;charset=utf-8");
            response.setCharacterEncoding("utf-8");
            //设置让浏览器弹出下载提示框，而不是直接在浏览器中打开
            response.addHeader("Content-Disposition", "attachment;filename=" + fileName);
            IOUtils.copy(input, outputStream);
            outputStream.flush();
            outputStream.close();
            input.close();
        } catch (Exception e) {
            log.error("文件下载失败：{}", e.getMessage());
            throw new BusinessException("下载失败");
        } finally {
            pool.returnObject(obsClient);
        }
    }

    @Override
    public Result<String> appDownload(Long uploadId) {
        // 判断uploadId是否存在
        YcqUploadInfo info = uploadInfoRepo.findByUploadIdAndState(uploadId, STATE_1);
        if (ObjectUtil.isNotEmpty(info)) {
            NumOfDownloads(uploadId);
        }
        return Result.ofSuccess("ok");
    }

    @Override
    public Result<String> deleteObject(Long uploadId) throws Exception {
        ObsClient obsClient = pool.borrowObject();
        try {
            // 判断数据库文件是否存在
            YcqUploadInfo info = uploadInfoRepo.findByUploadIdAndState(uploadId, STATE_0);
            if (ObjectUtil.isEmpty(info)) {
                return Result.ofFail("删除失败,文件不存在");
            } else {
                info.setState(STATE_DEL);
                // 通过上传id查询fileKey
                List<String> fileKey = obsJdbc.queryFileKey(uploadId);
                if (CollectionUtil.isEmpty(fileKey)) {
                    return Result.ofFail("删除失败,文件不存在");
                }
                for (String objectKey : fileKey) {
                    // 判断OBS文件对象是否存在
                    String objectNewKey = objectKey.replaceFirst("%2F", "/");
                    boolean exist = obsClient.doesObjectExist(bucketName, objectNewKey);
                    if (!exist) {
                        return Result.ofFail("删除失败,文件不存在");
                    }
                    obsClient.deleteObject(bucketName, objectNewKey);
                }
                uploadInfoRepo.saveAndFlush(info);
                return Result.ofSuccess("删除成功");
            }
        } catch (Exception e) {
            throw e;
        } finally {
            pool.returnObject(obsClient);
        }

    }

    @Override
    public void deleteObject2(YcqObsFile ycqObsFile) throws Exception {
        ObsClient obsClient = pool.borrowObject();
        YcqObsFile file = new YcqObsFile();
        BeanUtils.copyProperties(ycqObsFile,file);
        try {
            // 删除华为云obs对象
            obsClient.deleteObject(bucketName, file.getObjectKey());
            // 设置删除成功状态
            file.setState(STATE_DEL);
            fileRepo.saveAndFlush(file);
        } catch (Exception e) {
            throw e;
        } finally {
            pool.returnObject(obsClient);
        }
    }

    @Override
    public Result<String> deleteObjectFileKey(List<String> fileKey) {
        ObsClient obsClient = null;
        try {
            obsClient = pool.borrowObject();
            for (String objectKey : fileKey) {
                // 判断OBS文件对象是否存在
                String objectNewKey = objectKey.replaceFirst("%2F", "/");
                boolean exist = obsClient.doesObjectExist(bucketName, objectNewKey);
                if (!exist) {
                    return Result.ofFail("删除失败,文件不存在");
                }
                obsClient.deleteObject(bucketName, objectNewKey);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.info("删除失败,{}", e.getMessage());
        } finally {
            pool.returnObject(obsClient);
        }
        return Result.ofSuccess("删除成功");
    }

    @Override
    public Result<Map<String, Object>> bigFileUpload(MultipartFile uploadFile, String fileName) throws Exception {
        String date = DateUtils.formatDate(new Date(), "yyyyMMdd");
        final String objectName = date + "/";
        ObsClient obsClient = pool.borrowObject();
        try {
            // 判断文件夹对象是否存在
            boolean exist = obsClient.doesObjectExist(bucketName, objectName);
            if (!exist) {
                // 创建文件夹对象
                obsClient.putObject(bucketName, objectName, new ByteArrayInputStream(new byte[0]));
                log.info("创建文件夹成功,文件名称[{}]", objectName);
            }
            // 初始化线程池
            // 设定自己想要的线程数
            ExecutorService executorService = Executors.newFixedThreadPool(5);
            File largeFile = MFileUtil.MultipartFileToFile(uploadFile);

            // 初始化分段上传任务
            InitiateMultipartUploadRequest request = new InitiateMultipartUploadRequest(bucketName, objectName + fileName);
            InitiateMultipartUploadResult result = obsClient.initiateMultipartUpload(request);
            final String uploadId = result.getUploadId();
            // 每段上传100MB
            long partSize = 100 * 1024 * 1024L;
            assert largeFile != null;
            long fileSize = largeFile.length();

            // 计算需要上传的段数
            long partCount = fileSize % partSize == 0 ? fileSize / partSize : fileSize / partSize + 1;

            final List<PartEtag> partEtags = Collections.synchronizedList(new ArrayList<PartEtag>());

            // 执行并发上传段
            for (int i = 0; i < partCount; i++) {
                // 分段在文件中的起始位置
                final long offset = i * partSize;
                // 分段大小
                final long currPartSize = (i + 1 == partCount) ? fileSize - offset : partSize;
                // 分段号
                final int partNumber = i + 1;
                executorService.execute(new Runnable() {
                    @Override
                    public void run() {
                        UploadPartRequest uploadPartRequest = new UploadPartRequest();
                        uploadPartRequest.setBucketName(bucketName);
                        uploadPartRequest.setObjectKey(objectName + fileName);
                        uploadPartRequest.setUploadId(uploadId);
                        uploadPartRequest.setFile(largeFile);
                        uploadPartRequest.setPartSize(currPartSize);
                        uploadPartRequest.setOffset(offset);
                        uploadPartRequest.setPartNumber(partNumber);

                        UploadPartResult uploadPartResult;
                        try {
                            uploadPartResult = obsClient.uploadPart(uploadPartRequest);
                            System.out.println("Part#" + partNumber + " done\n");
                            partEtags.add(new PartEtag(uploadPartResult.getEtag(), uploadPartResult.getPartNumber()));
                        } catch (ObsException e) {
                            e.printStackTrace();
                        }
                    }
                });
            }
            // 等待上传完成
            executorService.shutdown();
            while (!executorService.isTerminated()) {
                try {
                    executorService.awaitTermination(120, TimeUnit.SECONDS);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
            log.info("合并段");
            // 合并段
            CompleteMultipartUploadRequest completeMultipartUploadRequest = new CompleteMultipartUploadRequest(bucketName, objectName + fileName, uploadId, partEtags);
            CompleteMultipartUploadResult result1 = obsClient.completeMultipartUpload(completeMultipartUploadRequest);
            // 保存数据库
            YcqObsFile file = new YcqObsFile();
            file.setBucketName(result1.getBucketName());
            file.setObjectKey(result1.getObjectKey());
            //file.setObjectUrl(result1.getObjectUrl());
            file.setCreateTime(new Date());
            file.setFileName(fileName);
            fileRepo.saveAndFlush(file);
            Map<String, Object> map = new HashMap<>(2);
            map.put("fileId", file.getId());
            map.put("fileKey", result1.getObjectKey());
            map.put("fileUrl", result1.getObjectUrl());
            return Result.ofSuccess(map);
        } finally {
            pool.returnObject(obsClient);
        }
    }

    @Override
    public Result<String> getKey(String key) {
        return Result.ofSuccess(redisUtils.get(key));
    }

    @Override
    public void updateObsServiceCode(List<ObsServiceCodeDto> obsServiceCodeDtos) {
        for (ObsServiceCodeDto dto : obsServiceCodeDtos) {
            Optional<YcqObsFile> byId = fileRepo.findById(dto.getFileId());
            if (byId.isPresent()) {
                YcqObsFile ycqObsFile = byId.get();
                if (!dto.getServiceCode().equals(ycqObsFile.getServiceCode()) || !dto.getServiceId().equals(ycqObsFile.getServiceId())) {
                    ycqObsFile.setServiceId(dto.getServiceId());
                    ycqObsFile.setServiceCode(dto.getServiceCode());
                    fileRepo.saveAndFlush(ycqObsFile);
                }
            }

        }
//        obsJdbc.updateObsServiceCode(obsServiceCodeDtos);
        //判断【】和list个数是否相同
//        int size = obsServiceCodeDtos.size();
//        int length = ints.length;
//        if (size != length) {
//            throw new BusinessException("图片提交异常");
//        }
    }

    /**
     * 下载量处理
     */
    public void NumOfDownloads(Long uploadId) {
        StringRedisTemplate redisTemplate = redisUtils.getRedisTemplate();
        // 拼接前缀
        String key = PREFIX + uploadId;
        ValueOperations<String, String> forValue = redisTemplate.opsForValue();
        if (!TRUE.equals(redisTemplate.hasKey(key))) {
            //设置初值
            forValue.set(key, "0");
            BoundGeoOperations<String, String> ops = redisTemplate.boundGeoOps(key);
            //设置过期
            ops.expire(1, TimeUnit.DAYS);
        }
        forValue.increment(key, 1L);
    }

    /**
     * 下载量
     * @param uploadId
     */
    //public void NumOfDownloads(Long uploadId) {
    //    // 拼接前缀
    //    String key = PREFIX + uploadId;
    //    if (redisUtils.get(key) == null) {
    //        //设置7天过期
    //        redisUtils.setEx(key, "1", 1L, TimeUnit.DAYS);
    //    } else {
    //        int count = Integer.parseInt(redisUtils.get(key));
    //        redisUtils.setEx(key, String.valueOf(count + 1), 1L, TimeUnit.DAYS);
    //    }
    //}

    /**
     * 防止中文乱码
     *
     * @param userAgent
     * @return
     * @throws UnsupportedEncodingException
     */
    public String objectKeyEncoder(String userAgent) throws UnsupportedEncodingException {
        String objectKey = userAgent;
        // IE浏览器
        if (userAgent.contains(MSIE)) {
            objectKey = URLEncoder.encode(objectKey, "UTF-8");
        } else {
            // google,火狐浏览器
            if (userAgent.contains(MOZILLA)) {
                objectKey = new String(objectKey.getBytes(), "ISO8859-1");
            } else {
                // 其他浏览器
                objectKey = URLEncoder.encode(objectKey, "UTF-8");
            }
        }
        return objectKey;
    }
}
