package com.crunii.micro.service.obs.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.domain.entity.main.YcqObsFile;
import com.crunii.micro.domain.entity.main.YcqUploadInfo;
import com.crunii.micro.domain.repo.main.YcqObsFileRepo;
import com.crunii.micro.domain.repo.main.YcqUploadInfoRepo;
import com.crunii.micro.ds.uid.util.DateUtils;
import com.crunii.micro.service.obs.core.client.FileClient;
import com.crunii.micro.service.obs.jdbc.ObsJdbc;
import com.crunii.micro.service.obs.service.FileConfigService;
import com.crunii.micro.service.obs.service.ObsObjectService;
import com.crunii.micro.service.obs.service.ObsTaskService;
import com.crunii.micro.service.obs.util.ImageUtil;
import com.crunii.micro.service.obs.util.RedisUtils;
import com.google.common.base.Stopwatch;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.InputStream;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static com.crunii.micro.service.obs.constants.ObsConstants.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/9 15:46
 */
@Slf4j
@Service
public class ObsTaskServiceImpl implements ObsTaskService {

    @Resource
    private ObsJdbc obsJdbc;

    @Resource
    private ObsObjectService obsObjectService;

    @Resource
    private YcqUploadInfoRepo uploadInfoRepo;

    @Autowired
    private RedisUtils redisUtils;

    @Resource
    private YcqObsFileRepo fileRepo;

    @Resource
    private YcqObsFileRepo ycqObsFileRepo;

    @Resource
    private FileConfigService fileConfigService;

    /**
     * 桶
     */
    @Value("${obs.s3.bucketName}")
    private String bucketName;

    @Override
    public void deleteObs() {
        // 查询待删除的对象
        List<Long> list = obsJdbc.queryDeleteObsKeys();
        if (CollectionUtils.isEmpty(list)) {
            log.info("暂无待删除的obs对象");
            return;
        }
        list.forEach(x -> {
            try {
                obsObjectService.deleteObject(x);
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    @Override
    public void deleteFileOperation() {
        // 查询待删除的对象
        List<YcqObsFile> files = fileRepo.findAllByState(STATE_0);
        if (CollectionUtils.isEmpty(files)) {
            log.info("暂无（取消上传、修改、删除的文件）待删除的obs对象");
            return;
        }
        for (YcqObsFile file : files) {
            try {
                obsObjectService.deleteObject2(file);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        log.info("共删除{}个文件",files.size());
    }

    //@Override
    //public void saveIncrement() {
    //    log.info("统计下载量开始");
    //    Stopwatch st = Stopwatch.createStarted();
    //    List<YcqFile> fileList = ycqFileRepo.findByState(STATE_1);
    //    for (YcqFile file : fileList) {
    //        String num = redisTemplate.opsForValue().get(file.getFileId());
    //        if (num == null) {
    //            continue;
    //        }
    //        Integer a = Integer.valueOf(file.getNumber());
    //        Integer b = Integer.valueOf(num);
    //        String number = String.valueOf((a + b));
    //        file.setNumber(number);
    //        ycqFileRepo.saveAndFlush(file);
    //        // 更新完后删除key
    //        redisTemplate.delete(file.getFileId());
    //    }
    //    st.elapsed();
    //    log.info("更新下载量结束,花费了:{}", st);
    //}

    @Override
    public void saveIncrement() {
        log.info("统计下载量开始");
        Stopwatch st = Stopwatch.createStarted();
        Set<String> listKey = redisUtils.getListKey(PREFIX);
        if (!listKey.isEmpty()) {
            for (String k : listKey) {
                Integer num = Integer.valueOf(redisUtils.get(k));
                if (ObjectUtil.isEmpty(num) || num == 0) {
                    continue;
                }
                Long uploadId = Long.valueOf(k.split(":")[1]);
                YcqUploadInfo file = uploadInfoRepo.findByUploadIdAndState(uploadId, STATE_1);
                if (ObjectUtil.isNotEmpty(file)) {
                    Integer oldNum = file.getImgNum();
                    Integer number = oldNum + num;
                    file.setNumber(number);
                    uploadInfoRepo.saveAndFlush(file);
                    // 将key 重新set 0
                    redisUtils.setDay(k, "0", 1);
                }
            }
        }
        st.elapsed();
        log.info("更新下载量结束,花费了:{}", st);
    }

    @Override
    public void saveCompression() {
        List<YcqObsFile> imageList= ycqObsFileRepo.findAllByStateAndIsPicLimitTen();
        if (imageList.size()==0) {
            return;
        }
        FileClient fileClient = fileConfigService.getFileClient();
        try {
            for (YcqObsFile ycqObsFile : imageList) {
                if (!fileClient.doesObjectExist(bucketName,ycqObsFile.getObjectKey())) {
                    ycqObsFile.setCompressStatus(-2);
                    continue;
                }
                InputStream imageStream = fileClient.getObject(bucketName, ycqObsFile.getObjectKey());
                String type = ycqObsFile.getType();
                String date = DateUtils.formatDate(new Date(), "yyyyMMdd");
                final String objectName = date + "/";
                InputStream inputStreamCompress = ImageUtil.compressNew(imageStream, 100, type);
                if (inputStreamCompress==null) {
                    ycqObsFile.setCompressStatus(-1);
                    continue;
                }
                String uuidName = UUIDUtil.getUUID() + "." + type;
                String objectKey = objectName + uuidName;
                fileClient.uploadFile(bucketName,inputStreamCompress, objectKey);
                ycqObsFile.setCompressStatus(1);
                ycqObsFile.setCompressObjectKey(objectKey);
            }
            ycqObsFileRepo.saveAll(imageList);
            log.info("已压缩处理图片:["+imageList.stream().map(e -> e.getId().toString()).collect(Collectors.joining(","))+"]");
        } catch (Exception e) {
            log.error("压缩失败,失败信息："+e);
        }

    }
}
