package com.crunii.micro.service.obs.service.impl;

import com.crunii.micro.service.obs.constants.ObsConstants;
import com.crunii.micro.service.obs.dto.DownLoadZipDto;
import com.crunii.micro.service.obs.dto.ZipFileDto;
import com.crunii.micro.service.obs.dto.ZipFileVo;
import com.crunii.micro.service.obs.jdbc.ZipFileJdbc;
import com.crunii.micro.service.obs.service.ZipFileService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.InputStream;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import static com.crunii.micro.service.obs.constants.ObsConstants.*;

/**
 * @author baoxy
 * @date 2023-06-19
 */
@Service
@Slf4j
public class ZipFileServiceImpl implements ZipFileService {

    @Resource
    private ZipFileJdbc zipFileJdbc;

    @Resource
    private ObsObjectServiceImpl obsObjectServiceImpl;

    @Value("${upload.url}")
    private String urlPrefix;

    @Override
    public int downloadZip(HttpServletRequest request, HttpServletResponse response, DownLoadZipDto dto) {
        String env = System.getProperty("spring.profiles.active");
        if("h3cTest".equals(env)){
            urlPrefix = S3InnerUrl;
        }
        // 储存url和名称
        List<ZipFileDto> fileUrlList = new ArrayList<>();
        if (StringUtils.isBlank(dto.getCode())) {
            // 如果编码为空，代表调用接口的是图绘青春文件中心功能,就调用这段dao层方法
            // 获取图片key列表
            List<ZipFileVo> getFileUrlList = zipFileJdbc.getFileUrlList(dto.getUploadIdList());
            getFileUrlList.forEach(e -> {
                ZipFileDto zipFileDto = new ZipFileDto();
                // 这里的url需要用nacos上的url前缀和图片key组合
                zipFileDto.setUrl(urlPrefix + e.getFileKey());
                // 将图片key可标识为图片名称
                zipFileDto.setName(e.getFileKey());
                fileUrlList.add(zipFileDto);
            });
        } else {
            // 若有值,则是图绘青春文件中心以外的值在调用接口,使用查询ycq_obs_file表的方法
            List<ZipFileVo> getObsFileList = zipFileJdbc.getObsFileList(dto.getUploadIdList());
            getObsFileList.forEach(e -> {
                ZipFileDto zipFileDto = new ZipFileDto();
                // 这里的url需要用nacos上的url前缀和图片key组合
                String newFileKey = e.getFileKey();
                zipFileDto.setUrl(urlPrefix + newFileKey);
                // 将图片key可标识为图片名称
                zipFileDto.setName(newFileKey);
                fileUrlList.add(zipFileDto);
            });
        }

        if (fileUrlList.size() == 0) {
            return 2;
        }
        String fileName = "已下载图片.zip";
        // 为防止 文件名出现乱码
        String agent = request.getHeader("User-Agent");
        // IE浏览器
        try {
            if (agent.contains(MSIE)) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                // google,火狐浏览器
                if (agent.contains(MOZILLA)) {
                    fileName = new String(fileName.getBytes(), "ISO8859-1");
                } else {
                    // 其他浏览器
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                }
            }
        } catch (Exception e) {
            log.error("文件下载失败:" + e.getMessage());
            return 1;
        }
        response.setContentType("APPLICATION/OCTET-STREAM");
        response.addHeader("Content-Disposition", "attachment;filename=" + fileName);
        zipFiles(fileUrlList, response);
        // 下载次数+1
        dto.getUploadIdList().forEach(e -> obsObjectServiceImpl.NumOfDownloads(Long.valueOf(e)));
        return 0;
    }

    public static void zipFiles(List<ZipFileDto> srcFiles, HttpServletResponse response) {
        try {
            if (srcFiles.size() != 0) {
                // 判断压缩后的文件存在不，不存在则创建
                InputStream fileInputStream = null;
                // 实例化 FileOutputStream 对象
//                FileOutputStream fileOutputStream = new FileOutputStream(zipFile);
                // 实例化 ZipOutputStream 对象
                ZipOutputStream zipOutputStream = new ZipOutputStream(new BufferedOutputStream(response.getOutputStream()));
                // 创建 ZipEntry 对象
                ZipEntry zipEntry;
                // 遍历源文件数组
                //第三方公司要求每个图片的命名方式需要和他们一致所以我使用的普通for循环，如没要求可用增强for循环，后缀名可截取图片的地址。
                for (ZipFileDto srcFile : srcFiles) {
                    // 将源文件数组中的当前文件读入 FileInputStream 流中
                    URL uri = new URL(srcFile.getUrl());
                    fileInputStream = uri.openStream();
                    // 实例化 ZipEntry 对象，源文件数组中的当前文件
                    zipEntry = new ZipEntry(srcFile.getName());
                    zipOutputStream.putNextEntry(zipEntry);
                    // 该变量记录每次真正读的字节个数
                    int len;
                    // 定义每次读取的字节数组
                    byte[] buffer = new byte[1024];
                    while ((len = fileInputStream.read(buffer)) > 0) {
                        zipOutputStream.write(buffer, 0, len);
                    }
                }
                //关闭流
                zipOutputStream.closeEntry();
                zipOutputStream.close();
                assert fileInputStream != null;
                fileInputStream.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
