package com.crunii.micro.service.obs.task;

import com.crunii.micro.service.obs.service.ObsTaskService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/9 15:29
 */
@EnableScheduling
@Component
@Slf4j
public class ObsTask {

    @Resource
    private ObsTaskService service;

    /**
     * 定时删除华为云obs对象
     */
    @Scheduled(cron = "0 0 1 * * ?")
    public void delObs() {
        log.info("开始处理页面已删除的obs对象");
        service.deleteObs();
    }

    /**
     * 定时删除（取消上传、修改、删除的文件）
     */
    @Scheduled(cron = "0 0 2 * * ?")
    public void delFileOperation() {
        log.info("开始处理取消上传、修改、删除的文件的obs对象");
        service.deleteFileOperation();
    }

    /**
     * 文件下载量同步数据库5分钟一次
     */
    @Scheduled(cron = "0 0/5 * * * ?")
    public void increment() {
        service.saveIncrement();
    }


    /**
     * 图片压缩 每10秒执行一次
     */
    @Scheduled(cron = "9,19,29,39,49,59 * * * * ?")
    public void imageCompressionTask() {
        service.saveCompression();
    }
}
