package com.crunii.micro.service.obs.util;

import cn.hutool.json.JSONObject;
import com.amazonaws.ClientConfiguration;
import com.amazonaws.HttpMethod;
import com.amazonaws.Protocol;
import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.auth.BasicAWSCredentials;
import com.amazonaws.regions.Region;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.AmazonS3Client;
import com.amazonaws.services.s3.S3ClientOptions;
import com.amazonaws.services.s3.model.GeneratePresignedUrlRequest;
import com.amazonaws.services.s3.model.ResponseHeaderOverrides;
import com.crunii.micro.service.obs.core.client.s3.S3FileClientConfig;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 文件上传工具类，主要用户孟富东流媒体服务对接
 * */

public class UploadFileUtil {
	// 获取文件上传路径
	public static final String FILE_STORAGE_URL = "/smart/media/getFileStorageUrl";
	// 文件上传完成通知
	public static final String UPLOAD_FILENOTIFICATION = "/smart/media/uploadFileNotification";
	// 获取文件播放地址
	public static final String FILE_URL = "/smart/media/getFileUrl";


	@SuppressWarnings("finally")
	public static String uploadFile(String actionUrl, MultipartFile file) {
		DataOutputStream ds = null;
		InputStream inputStream = null;
		InputStreamReader inputStreamReader = null;
		BufferedReader reader = null; 
		StringBuffer resultBuffer = new StringBuffer();
		String tempLine = null;
		try {
			// 统一资源
			URL url = new URL(actionUrl);
			// 连接类的父类，抽象类
			URLConnection urlConnection = url.openConnection();
			// http的连接类
			HttpURLConnection httpURLConnection = (HttpURLConnection) urlConnection;
			// 设置是否从httpUrlConnection读入，默认情况下是true;
			httpURLConnection.setDoInput(true);
			// 设置是否向httpUrlConnection输出
			httpURLConnection.setDoOutput(true);
			// Post 请求不能使用缓存
			httpURLConnection.setUseCaches(false);
			// 设定请求的方法，默认是GET
			httpURLConnection.setRequestMethod("PUT");
			// 设置字符编码连接参数
			httpURLConnection.setRequestProperty("Connection", "Keep-Alive");
			// 设置字符编码
			httpURLConnection.setRequestProperty("Charset", "UTF-8");
			// 设置请求内容类型
			httpURLConnection.setRequestProperty("Content-Type", "multipart/form-data;");
			// 设置DataOutputStream
			ds = new DataOutputStream(httpURLConnection.getOutputStream());
//			String filename = uploadFile.substring(uploadFile.lastIndexOf("//") + 1);
			InputStream fStream = file.getInputStream();
			int bufferSize = 1024;
			byte[] buffer = new byte[bufferSize];
			int length = -1;
			while ((length = fStream.read(buffer)) != -1) {
				ds.write(buffer, 0, length);
			}
			fStream.close();
			ds.flush();
			if (httpURLConnection.getResponseCode() >= 300) {
				throw new Exception(
						"HTTP Request is not success, Response code is " + httpURLConnection.getResponseCode() + "\n"
								+ "HTTP body=" + httpURLConnection.getContent());
			}
			if (httpURLConnection.getResponseCode() == HttpURLConnection.HTTP_OK) {
				inputStream = httpURLConnection.getInputStream();
				inputStreamReader = new InputStreamReader(inputStream);
				reader = new BufferedReader(inputStreamReader);
				tempLine = null;
				resultBuffer = new StringBuffer();
				while ((tempLine = reader.readLine()) != null) {
					resultBuffer.append(tempLine);
					resultBuffer.append("\n");
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (ds != null) {
				try {
					ds.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if (inputStreamReader != null) {
				try {
					inputStreamReader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if (inputStream != null) {
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			return resultBuffer.toString();
		}
	}

	/**
	 * 文件上传url预授权
	 *
	 * @param s3FileClientConfig
	 * @param objectKey
	 * @return
	 */
	public static Map<String, Object> signUrl(S3FileClientConfig s3FileClientConfig, String objectKey, int expTime) {
		Map<String, Object> resulMap = new HashMap<String, Object>();
		String fileUrl = "";
		int status = 0;
		String msg = "成功";
		try {
			if (StringUtils.isBlank(s3FileClientConfig.getAccessKey())) {
				resulMap.put("status", -1);
				resulMap.put("msg", "accessKey不能为空");
				return resulMap;
			}
			if (StringUtils.isBlank(s3FileClientConfig.getAccessSecret())) {
				resulMap.put("status", -2);
				resulMap.put("msg", "secretKey不能为空");
				return resulMap;
			}
			if (StringUtils.isBlank(s3FileClientConfig.getEndpoint())) {
				resulMap.put("status", -3);
				resulMap.put("msg", "上传地址不能为空");
				return resulMap;
			}
			if (StringUtils.isBlank(s3FileClientConfig.getBucketName())) {
				resulMap.put("status", -4);
				resulMap.put("msg", "bucketName不能为空");
				return resulMap;
			}
			if (StringUtils.isBlank(objectKey)) {
				resulMap.put("status", -5);
				resulMap.put("msg", "objectKey不能为空");
				return resulMap;
			}
			if (expTime <= 0) {//有效期
				resulMap.put("status", -6);
				resulMap.put("msg", "url有效期无效");
				return resulMap;
			}
			AWSCredentials awsCredentials = new BasicAWSCredentials(s3FileClientConfig.getAccessKey(), s3FileClientConfig.getAccessSecret());
			ClientConfiguration clientConfig = new ClientConfiguration();
			clientConfig.setProtocol(Protocol.HTTP);
			AmazonS3 s3 = new AmazonS3Client(awsCredentials, clientConfig);
			s3.setEndpoint(s3FileClientConfig.getEndpoint());
			if (s3.doesBucketExistV2(s3FileClientConfig.getBucketName())) {

				// 设置过期时间为1小时
				Date expiration = new Date();
				long expTimeMillis = expiration.getTime();
//				expTimeMillis += 1000 * 60 * 60;
				expTimeMillis += 1000 * 60 * expTime;//设置时间（分钟）
				expiration.setTime(expTimeMillis);

				//生成预签名URL
				GeneratePresignedUrlRequest generatePresignedUrlRequest = new GeneratePresignedUrlRequest(s3FileClientConfig.getBucketName(), objectKey)
						.withMethod(HttpMethod.PUT)
						.withExpiration(expiration);
				URL url = s3.generatePresignedUrl(generatePresignedUrlRequest);

				if (null != url) {
					fileUrl = url.toString();
				}
//                System.out.println("生成的签名的URL: " + fileUrl);
				resulMap.put("fileUrl", fileUrl);
			} else {
				status = 1001;
				msg = "桶不存在";
			}
			resulMap.put("status", status);
			resulMap.put("msg", msg);
		} catch (Exception e) {
			//调用成功传输，但Amazon S3无法处理
			//它返回了一个错误响应。
			e.printStackTrace();
			status = 1002;
			msg = "失败";
			resulMap.put("status", status);
			resulMap.put("msg", msg);
//			log.error("定时任务获取预上传url失败:", e);
		}
		return resulMap;
	}

	/**
	 * 文件下载url
	 *
	 * @param s3FileClientConfig
	 * @param objectKey
	 * @return
	 */
	public static Map<String, Object> downloadUrl(S3FileClientConfig s3FileClientConfig, String objectKey, int expTime) {
		Map<String, Object> resulMap = new HashMap<String, Object>();
		String fileUrl = "";
		int status = 0;
		String msg = "成功";
		try {
			if (StringUtils.isBlank(s3FileClientConfig.getAccessKey())) {
				resulMap.put("status", -1);
				resulMap.put("msg", "accessKey不能为空");
				return resulMap;
			}
			if (StringUtils.isBlank(s3FileClientConfig.getAccessSecret())) {
				resulMap.put("status", -2);
				resulMap.put("msg", "secretKey不能为空");
				return resulMap;
			}
			if (StringUtils.isBlank(s3FileClientConfig.getEndpoint())) {
				resulMap.put("status", -3);
				resulMap.put("msg", "上传地址不能为空");
				return resulMap;
			}
			if (StringUtils.isBlank(s3FileClientConfig.getBucketName())) {
				resulMap.put("status", -4);
				resulMap.put("msg", "bucketName不能为空");
				return resulMap;
			}
			if (StringUtils.isBlank(objectKey)) {
				resulMap.put("status", -5);
				resulMap.put("msg", "objectKey不能为空");
				return resulMap;
			}
			if (expTime <= 0) {//有效期
				resulMap.put("status", -6);
				resulMap.put("msg", "url有效期无效");
				return resulMap;
			}

			ClientConfiguration config = new ClientConfiguration().withProtocol(Protocol.HTTP);
			config.setSignerOverride("S3SignerType");
			AmazonS3 s3 = new AmazonS3Client(new BasicAWSCredentials(s3FileClientConfig.getAccessKey(), s3FileClientConfig.getAccessSecret()), config);
			S3ClientOptions options = new S3ClientOptions();
			options.setPathStyleAccess(true);
			// 设置Region
			String regionName = "cn-north-1";
			Region region = Region.getRegion(Regions.fromName(regionName));
			s3.setS3ClientOptions(options);
			s3.setRegion(region);
			// ECS IP Address
//            s3.setEndpoint("61.147.2.202:7480");

           /* AWSCredentials awsCredentials = new BasicAWSCredentials(accessKey, secretKey);
            ClientConfiguration clientConfig = new ClientConfiguration();
            clientConfig.setProtocol(Protocol.HTTP);
            AmazonS3 s3 = new AmazonS3Client(awsCredentials, clientConfig);*/
			s3.setEndpoint(s3FileClientConfig.getEndpoint());
			if (s3.doesBucketExistV2(s3FileClientConfig.getBucketName())) {
				if (s3.doesObjectExist(s3FileClientConfig.getBucketName(), objectKey)) {
					Date expiration = new Date();
					long expTimeMillis = expiration.getTime();
					expTimeMillis += 1000 * 60 * expTime;//调试设置时间5分钟
					expiration.setTime(expTimeMillis);

					GeneratePresignedUrlRequest generatePresignedUrlRequest = new GeneratePresignedUrlRequest(s3FileClientConfig.getBucketName(), objectKey)
							.withMethod(HttpMethod.GET)
							.withResponseHeaders(new ResponseHeaderOverrides().withContentDisposition("attachment"))
							.withExpiration(expiration);
					URL url = s3.generatePresignedUrl(generatePresignedUrlRequest);
					if (null != url) {
						fileUrl = url.toString();
					}
					System.out.println("生成的签名的URL: " + fileUrl);
					resulMap.put("fileUrl", fileUrl);
				} else {
					status = 1;
					msg = "文件不存在";
				}
			} else {
				status = 1001;
				msg = "桶不存在";
			}
			resulMap.put("status", status);
			resulMap.put("msg", msg);
		} catch (Exception e) {
			msg = "失败";
			status = 1003;
			resulMap.put("status", status);
			resulMap.put("msg", msg);
			e.printStackTrace();
//			logger.error("文件下载url失败:", e);
		}
		return resulMap;
	}


	public static void main(String[] args) {
		// Map<String, Object> map = signUrl("6E0UGC3FIC4IKEF5DYU6", "E0lGnO8bMlnZtUpUyAmkWFB58D85bPsrL7rltPAU", "221.178.46.50:8082","zggczyqntcqswy-obs-qccq01", "5229022E85/2020/09/17/20200917135055-20200917221111", 60 * 24 * 7 );

		String returnStr = "{'code':'0','msg':'OK','pictureUrl':'http://102.2.12.224:18080/sjzl/78229307D3D6F222_720p.mp4?token=fajfaggfajfgjadfgadfdaj'}";

		JSONObject json=new JSONObject(returnStr);

		System.out.println(json);
	}
}
