package com.crunii.micro.service.oneaffair.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.AppElseStudentArchivesVo;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.AppStudentArchivesReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.AppStudentArchivesVo;
import com.crunii.micro.service.oneaffair.service.AppArchiveService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import java.util.List;

@Slf4j
@Api(tags = "APP端档案API接口")
@RestController
@RequestMapping("/yjs/app/archives/auth")
public class AppArchiveController {

    @Resource
    private AppArchiveService appArchiveService;

    @ApiOperation(value = "政治进步三部曲查询档案信息")
    @RequestMapping(path = "/searchStudentArchives", method = RequestMethod.POST)
    public Result<AppStudentArchivesVo> searchStudentArchives(@RequestBody AppStudentArchivesReq req){
        log.info("AppArchiveController.searchStudentArchives.param：{}", req);
        try {
            return Result.ofSuccess(appArchiveService.searchStudentArchives(req));
        } catch (BusinessException e) {
            log.error("AppArchiveController.searchStudentArchives.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("AppArchiveController.searchStudentArchives.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询志愿服务、赛事活动、学习档案、其他档案信息")
    @RequestMapping(path = "/searchElseStudentArchives", method = RequestMethod.POST)
    public Result<List<AppElseStudentArchivesVo>> searchOtherStudentArchives(@RequestBody AppStudentArchivesReq req){
        log.info("AppArchiveController.searchOtherStudentArchives.param：{}", req);
        try {
            return Result.ofSuccess(appArchiveService.searchOtherStudentArchives(req));
        } catch (BusinessException e) {
            log.error("AppArchiveController.searchOtherStudentArchives.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("AppArchiveController.searchOtherStudentArchives.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


}
