package com.crunii.micro.service.oneaffair.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveChangeReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveSelectReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveVo;
import com.crunii.micro.service.oneaffair.dto.vo.honor.HonorReq;
import com.crunii.micro.service.oneaffair.dto.vo.honor.HonorVo;
import com.crunii.micro.service.oneaffair.dto.vo.honor.MthMatchVo;
import com.crunii.micro.service.oneaffair.dto.vo.student.StudentInfoVo;
import com.crunii.micro.service.oneaffair.dto.vo.student.StudentReq;
import com.crunii.micro.service.oneaffair.manager.StudentManager;
import com.crunii.micro.service.oneaffair.service.ArchiveService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author : sunzefeng
 * @date : 2023-9-19
 */
@Slf4j
@Api(tags = "档案API接口")
@RestController
@RequestMapping("/yjs/archive/auth")
public class ArchiveController {

    @Resource
    private StudentManager studentManager;

    @Resource
    private ArchiveService archiveService;

    @ApiOperation(value = "展示学生基本信息")
    @RequestMapping(path = "/showStudentInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<StudentInfoVo>> pageSearchStudentInfo(@RequestBody PageReq<StudentReq> req) {
        try {
            // 学生信息详情页面大部分满足，积分，综合得分还没有。
            return Result.ofSuccess(studentManager.pageSearchStudentInfo(req));
        } catch (BusinessException e) {
            log.error("StudentController.pageSearchStudentInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.pageSearchStudentInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "展示学生档案信息")
    @RequestMapping(path = "/showStudentArchivePage", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<ArchiveVo>> showStudentArchivePage(@RequestBody PageReq<ArchiveSelectReq> req) {
        try {
            return Result.ofSuccess(archiveService.pageSearchArchiveInfo(req));
        } catch (BusinessException e) {
            log.error("StudentController.pageSearchStudentInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.pageSearchStudentInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "展示学生档案信息")
    @RequestMapping(path = "/showStudentArchive", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<ArchiveVo>> showStudentArchive(@RequestBody ArchiveSelectReq req) {
        try {
            List<ArchiveVo> list = archiveService.getList(req);
            return Result.ofSuccess(list);
        } catch (BusinessException e) {
            log.error("StudentController.pageSearchStudentInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.pageSearchStudentInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "插入档案信息")
    @RequestMapping(path = "/createArchive", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createArchive(@RequestBody ArchiveChangeReq req,@ApiIgnore @CurrWorker Worker worker) {
        try {
            archiveService.insert(req,worker);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("MedalController.createMedalInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.createMedalInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除档案信息")
    @RequestMapping(path = "/deleteArchive", method = RequestMethod.DELETE)
    @ResponseBody
    public Result<Boolean> deleteArchiveInfo(String id,@ApiIgnore @CurrWorker Worker worker) {
        try {
            if (archiveService.delete(id,worker)) {
                return Result.ofSuccess(true);
            } else {
                return Result.ofFail("删除数据失败");
            }
        } catch (BusinessException e) {
            log.error("MedalController.createMedalInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.createMedalInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "更新档案信息")
    @RequestMapping(path = "/updateArchive", method = RequestMethod.PUT)
    @ResponseBody
    public Result<Boolean> updateArchive(@RequestBody ArchiveChangeReq archiveChangeReq,@ApiIgnore @CurrWorker Worker worker) {
        try {
            if (archiveService.update(archiveChangeReq,worker)) {
                return Result.ofSuccess(true);
            } else {
                return Result.ofFail("更新数据失败");
            }
        } catch (BusinessException e) {
            log.error("MedalController.createMedalInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.createMedalInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "我的荣誉")
    @RequestMapping(path = "/pageSearchMyHonor", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<HonorVo>> pageSearchMyHonor(@RequestBody PageReq<HonorReq> req) {
        log.info("GoodStudentController.pageSearchMyHonor.param：{}", req.toString());
        try {
            Date endCreatedTime = req.getQuery().getEndCreatedTime();
            endCreatedTime = null != endCreatedTime ? DateUtil.addSecondDate(DateUtil.addMinuteDate(DateUtil.addHour(endCreatedTime, 23), 59), 59) : endCreatedTime;
            req.getQuery().setEndCreatedTime(endCreatedTime);
            return Result.ofSuccess(archiveService.pageSearchMyHonor(req));
        } catch (BusinessException e) {
            log.error("GoodStudentController.pageSearchMyHonor.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.pageSearchMyHonor.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "我参与的活动")
    @RequestMapping(path = "/pageSearchMyAcceptMthMatchInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<HonorVo>> pageSearchMyAcceptMthMatchInfo(@RequestBody PageReq<HonorReq> req) {
        log.info("GoodStudentController.pageSearchMyAcceptActivity.param：{}", req.toString());
        try {
            Date endCreatedTime = req.getQuery().getEndCreatedTime();
            endCreatedTime = null != endCreatedTime ? DateUtil.addSecondDate(DateUtil.addMinuteDate(DateUtil.addHour(endCreatedTime, 23), 59), 59) : endCreatedTime;
            req.getQuery().setEndCreatedTime(endCreatedTime);
            return Result.ofSuccess(archiveService.pageSearchMyAcceptMthMatchInfo(req));
        } catch (BusinessException e) {
            log.error("GoodStudentController.pageSearchMyAcceptMthMatchInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.pageSearchMyAcceptMthMatchInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "活动详情")
    @RequestMapping(path = "/getMthMatchInfo", method = RequestMethod.GET)
    @ResponseBody
    public Result<MthMatchVo> getMthMatchInfo(@RequestParam("eventId") Long eventId, @RequestParam("studentId") Long studentId) {
        log.info("GoodStudentController.getMthMatchInfo.param：{}", eventId);
        try {
            return Result.ofSuccess(archiveService.getMthMatchInfo(eventId, studentId));
        } catch (BusinessException e) {
            log.error("GoodStudentController.pageSearchMyAcceptMthMatchInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.pageSearchMyAcceptMthMatchInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }





}
