package com.crunii.micro.service.oneaffair.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.vo.evaluate.*;
import com.crunii.micro.service.oneaffair.service.EvaluateIndexService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

@Slf4j
@Api(tags = "指标API接口")
@RestController
@RequestMapping("/yjs/evaluateIndex/auth")
public class EvaluateIndexController {

    @Resource
    private EvaluateIndexService evaluateIndexService;

    @ApiOperation(value = "下拉查询体系对应的指标信息")
    @RequestMapping(path = "/searchIndexListInfo", method = RequestMethod.POST)
    public Result<List<YcqYjsEvaluateIndexVo>> searchIndexListInfo(@RequestBody SystemIndexReq req) {
        log.info("IndexController.searchIndexListInfo.param：{}", req);
        try {
            return Result.ofSuccess(evaluateIndexService.searchIndexListInfo(req));
        } catch (BusinessException e) {
            log.error("IndexController.searchIndexListInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("IndexController.searchIndexListInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "分页查询指标信息")
    @RequestMapping(path = "/pageSearchIndexInfo", method = RequestMethod.POST)
    public Result<PageRsp<YcqYjsEvaluateIndexVo>> pageSearchIndexInfo(@RequestBody PageReq<EvaluateIndexReq> req) {
        log.info("IndexController.pageSearchIndexInfo.param：{}", req);
        try {
            return Result.ofSuccess(evaluateIndexService.pageSearchIndexInfo(req));
        } catch (BusinessException e) {
            log.error("IndexController.pageSearchIndexInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("IndexController.pageSearchIndexInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "新建指标信息")
    @RequestMapping(path = "/createIndexInfo", method = RequestMethod.POST)
    public Result<Boolean> createIndexInfo(@Validated @RequestBody EvaluateIndexCreateReq createReq) {
        log.info("IndexController.createIndexInfo.param：{}", createReq);
        try {
            evaluateIndexService.createIndexInfo(createReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("IndexController.createIndexInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("IndexController.createIndexInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改指标信息")
    @RequestMapping(path = "/updateIndexInfo", method = RequestMethod.POST)
    public Result<Boolean> updateIndexInfo(@Validated @RequestBody EvaluateIndexUpdateReq updateReq) {
        log.info("IndexController.updateIndexInfo.param：{}", updateReq);
        try {
            evaluateIndexService.updateIndexInfo(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("IndexController.updateIndexInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("IndexController.updateIndexInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("删除数据")
    @RequestMapping(path = "/batchDelete",method = RequestMethod.GET)
    public Result<Boolean> batchDelete(@RequestParam("id")  BigInteger id) {
        log.info("IndexController.batchDelete.param：{}",id);
        try {
            evaluateIndexService.batchDelete(id);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("IndexController.deleteIndexInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("IndexController.deleteIndexInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
