package com.crunii.micro.service.oneaffair.controller;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.vo.evaluate.*;
import com.crunii.micro.service.oneaffair.service.EvaluateSystemService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

@Slf4j
@Api(tags = "体系API接口")
@RestController
@RequestMapping("/yjs/evaluateSystem/auth")
public class EvaluateSystemController {

    @Resource
    private EvaluateSystemService evaluateSystemService;

    @ApiOperation(value = "查询业务信息")
    @RequestMapping(path = "/searchAppConfig", method = RequestMethod.POST)
    @ApiImplicitParam(name="appName",value = "appName" ,required = false)
    public Result<List<AppConfigVo>> searchAppConfig(String appName) {
        log.info("EvaluateSystemController.searchAppConfig.param：{}", appName);
        try {
            return Result.ofSuccess(evaluateSystemService.searchAppConfig(appName));
        } catch (BusinessException e) {
            log.error("EvaluateSystemController.searchAppConfig.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("EvaluateSystemController.searchAppConfig.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询机构信息")
    @RequestMapping(path = "/searchTsmOrg", method = RequestMethod.GET)
    public Result<List<TsmOrgVo>> searchTsmOrg(@RequestParam("parentOrgId") BigInteger parentOrgId) {
        log.info("EvaluateSystemController.searchTsmOrg.param：{}", parentOrgId);
        try {
            return Result.ofSuccess(evaluateSystemService.searchTsmOrg(parentOrgId));
        } catch (BusinessException e) {
            log.error("EvaluateSystemController.searchTsmOrg.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("EvaluateSystemController.searchTsmOrg.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询上级组织")
    @RequestMapping(path = "/getParentTsmOrg", method = RequestMethod.GET)
    public Result<TsmOrgVo> getParentTsmOrg(@RequestParam("orgId") BigInteger orgId) {
        log.info("EvaluateSystemController.searchTsmOrg.param：{}", orgId);
        try {
            List<TsmOrgVo> parentTsmOrg = evaluateSystemService.getParentTsmOrg(orgId);
            if (!CollectionUtils.isEmpty(parentTsmOrg)){
                return Result.ofSuccess(parentTsmOrg.get(0));
            }else {
                return Result.ofSuccess(null);
            }
        } catch (BusinessException e) {
            log.error("EvaluateSystemController.searchTsmOrg.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("EvaluateSystemController.searchTsmOrg.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "分页查询体系信息")
    @RequestMapping(path = "/pageSearchSystemInfo", method = RequestMethod.POST)
    public Result<PageRsp<EvaluateSystemVo>> pageSearchSystemInfo(@RequestBody PageReq<EvaluateSystemReq> req) {
        log.info("EvaluateSystemController.pageSearchSystemInfo.param：{}", req);
        try {
            return Result.ofSuccess(evaluateSystemService.pageSearchSystemInfo(req));
        } catch (BusinessException e) {
            log.error("EvaluateSystemController.pageSearchSystemInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("EvaluateSystemController.pageSearchSystemInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "新建体系信息")
    @RequestMapping(path = "/createSystemInfo", method = RequestMethod.POST)
    public Result<Boolean> createSystemInfo(@Validated @RequestBody EvaluateSystemCreateReq createReq) {
        log.info("EvaluateSystemController.createSystemInfo.param：{}", createReq);
        try {
            evaluateSystemService.createSystemInfo(createReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("EvaluateSystemController.createSystemInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("EvaluateSystemController.createSystemInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改体系信息")
    @RequestMapping(path = "/updateSystemInfo", method = RequestMethod.POST)
    public Result<Boolean> updateSystemInfo(@Validated @RequestBody EvaluateSystemUpdateReq updateReq) {
        log.info("EvaluateSystemController.updateSystemInfo.param：{}", updateReq);
        try {
            evaluateSystemService.updateSystemInfo(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("EvaluateSystemController.updateSystemInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("EvaluateSystemController.updateSystemInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除体系信息")
    @RequestMapping(path = "/deleteSystemInfo", method = RequestMethod.GET)
    public Result<Boolean> deleteSystemInfo(@RequestParam("id") BigInteger id) {
        log.info("EvaluateSystemController.deleteSystemInfo.param：{}",id);
        try {
            evaluateSystemService.deleteSystemInfo(id);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("EvaluateSystemController.deleteSystemInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("EvaluateSystemController.deleteSystemInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "查询机构信息(含学校信息)")
    @RequestMapping(path = "/searchTsmOrgAndSchool", method = RequestMethod.GET)
    public Result<TsmOrgVoChildrenVo> searchTsmOrgAndSchool(@RequestParam("orgId") BigInteger orgId) {
        log.info("EvaluateSystemController.searchTsmOrg.param：{}", orgId);
        try {
            return Result.ofSuccess(evaluateSystemService.searchTsmOrgAndSchool(orgId));
        } catch (BusinessException e) {
            log.error("EvaluateSystemController.searchTsmOrg.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("EvaluateSystemController.searchTsmOrg.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }
}
