package com.crunii.micro.service.oneaffair.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.service.oneaffair.dto.vo.goodstudent.*;
import com.crunii.micro.service.oneaffair.manager.GoodStudentManager;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Slf4j
@Api(tags = "优秀学生API接口")
@RestController
@RequestMapping("/yjs/goodStudent/auth")
public class GoodStudentController {

    @Resource
    private GoodStudentManager goodStudentManager;

    @ApiOperation(value = "新建优秀学生信息")
    @RequestMapping(path = "/createGoodStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createGoodStudent(@Validated @RequestBody GoodStudentCreateReq createReq) {
        log.info("GoodStudentController.createGoodStudent.param：{}", createReq.toString());
        try {
            goodStudentManager.createGoodStudent(createReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("GoodStudentController.createGoodStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.createGoodStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改优秀学生信息")
    @RequestMapping(path = "/updateGoodStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateGoodStudent(@Validated @RequestBody GoodStudentUpdateReq updateReq) {
        log.info("GoodStudentController.updateGoodStudent.param：{}", updateReq.toString());
        try {
            goodStudentManager.updateGoodStudent(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("GoodStudentController.updateGoodStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.updateGoodStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "批量修改优秀学生的展出状态")
    @RequestMapping(path = "/updateGoodStudentShowFlag", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateGoodStudentShowFlag(@Validated @RequestBody GoodStudentUpdateShowFlagReq updateReq) {
        log.info("GoodStudentController.updateGoodStudentShowFlag.param：{}", updateReq.toString());
        try {
            goodStudentManager.updateGoodStudentShowFlag(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("GoodStudentController.updateGoodStudentShowFlag.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.updateGoodStudentShowFlag.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除优秀学生信息")
    @RequestMapping(path = "/deleteGoodStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> deleteGoodStudent(@Validated @RequestBody GoodStudentDeleteReq deleteReq) {
        log.info("GoodStudentController.deleteGoodStudent.param：{}", deleteReq.toString());
        try {
            goodStudentManager.deleteGoodStudent(deleteReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("GoodStudentController.deleteGoodStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.deleteGoodStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "分页查询优秀学生信息")
    @RequestMapping(path = "/pageSearchGoodStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<GoodStudentVo>> pageSearchGoodStudent(@RequestBody PageReq<GoodStudentReq> req) {
        log.info("GoodStudentController.pageSearchGoodStudent.param：{}", req.toString());
        try {
            return Result.ofSuccess(goodStudentManager.pageSearchGoodStudent(req));
        } catch (BusinessException e) {
            log.error("GoodStudentController.pageSearchGoodStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.pageSearchGoodStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("优秀学生信息excel导出")
    @RequestMapping(value = "/exportGoodStudentExcel", method = {RequestMethod.POST})
    @ResponseBody
    public Result<Boolean> exportGoodStudentExcel(HttpServletResponse response, @RequestBody GoodStudentReq req) {
        log.info("GoodStudentController.exportGoodStudentExcel.param：{}", req.toString());
        try {
            goodStudentManager.exportGoodStudentExcel(response, req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("GoodStudentController.exportGoodStudentExcel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.exportGoodStudentExcel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "App端荣誉墙(分页查询优秀学生信息)")
    @RequestMapping(path = "/pageSearchHonorWallGoodStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<GoodStudentVo>> pageSearchHonorWallGoodStudent(@RequestBody PageReq<GoodStudentReq> req) {
        log.info("GoodStudentController.pageSearchHonorWallGoodStudent.param：{}", req.toString());
        try {
            Date endCreatedTime = req.getQuery().getEndCreatedTime();
            endCreatedTime = null != endCreatedTime ? DateUtil.addSecondDate(DateUtil.addMinuteDate(DateUtil.addHour(endCreatedTime, 23), 59), 59) : endCreatedTime;
            req.getQuery().setEndCreatedTime(endCreatedTime);
            req.getQuery().setShowFlag(1);
            return Result.ofSuccess(goodStudentManager.pageSearchGoodStudent(req));
        } catch (BusinessException e) {
            log.error("GoodStudentController.pageSearchHonorWallGoodStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.pageSearchHonorWallGoodStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询荣誉称号列表)")
    @RequestMapping(path = "/searchGoodStudentHonoraryTitleTypeList", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<Map<String, String>>> searchGoodStudentHonoraryTitleTypeList(@RequestParam(value = "studentNum") String studentNum) {
        log.info("GoodStudentController.searchGoodStudentList.param：{}", studentNum);
        try {
            return Result.ofSuccess(goodStudentManager.searchGoodStudentHonoraryTitleTypeList(studentNum));
        } catch (BusinessException e) {
            log.error("GoodStudentController.pageSearchHonorWallGoodStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentController.pageSearchHonorWallGoodStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }



}
