package com.crunii.micro.service.oneaffair.controller;


import com.crunii.framework.api.encrypt.annotation.EncryptRequestBody;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.vo.idengtitychoose.*;
import com.crunii.micro.service.oneaffair.service.IdentityChooseService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

@Slf4j
@Api(tags = "身份选择接口")
@RestController
@RequestMapping("/yjs/identityChoose/auth")
public class IdentityChooseController {

    @Resource
    private IdentityChooseService identityChooseService;

    @ApiOperation(value = "根据身份证号查询学生信息")
    @RequestMapping(path = "/searchStudentInfoByIdentityCardNo", method = RequestMethod.POST)
    public Result<StudentInfoByIdentityCardNoVo> searchStudentInfoByIdentityCardNo(@EncryptRequestBody @RequestBody IdentityCardNoVo identityCardNo) {
        log.info("IdentityChooseController.searchStudentInfoByIdentityCardNo.param：{}", identityCardNo);
        try {
            return Result.ofSuccess(identityChooseService.searchStudentInfoByIdentityCardNo(identityCardNo));
        } catch (BusinessException e) {
            log.error("IdentityChooseController.searchStudentInfoByIdentityCardNo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("IdentityChooseController.searchStudentInfoByIdentityCardNo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "查询当前账号绑定的学生信息")
    @RequestMapping(path = "/searchStudentInfoByUser", method = RequestMethod.GET)
    public Result<List<StudentInfoByUserVo>> searchStudentInfoByUser(@RequestParam("userId") BigInteger userId) {
        log.info("IdentityChooseController.searchStudentInfoByUser.param：{}", userId);
        try {
            return Result.ofSuccess(identityChooseService.searchStudentInfoByUser(userId));
        } catch (BusinessException e) {
            log.error("IdentityChooseController.searchStudentInfoByUser.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("IdentityChooseController.searchStudentInfoByUser.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "绑定学生")
    @RequestMapping(path = "/encryptionRequired/bindStudent", method = RequestMethod.POST)
    public Result<StudentInfoByUserVo> bindStudent(@Validated @EncryptRequestBody BindStudentReq bindStudentReq) {
        log.info("IdentityChooseController.bindStudent.param：{}", bindStudentReq);
        try {
            return Result.ofSuccess(identityChooseService.bindStudent(bindStudentReq));
        } catch (BusinessException e) {
            log.error("IdentityChooseController.bindStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("IdentityChooseController.bindStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除绑定学生")
    @RequestMapping(path = "/deleteBindStudent", method = RequestMethod.GET)
    public Result<Boolean> deleteBindStudentById(@RequestParam("id") BigInteger id) {
        log.info("IdentityChooseController.deleteBindStudentById.param：{}", id.toString());
        try {
            identityChooseService.deleteBindStudentById(id);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("IdentityChooseController.deleteBindStudentById.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("IdentityChooseController.deleteBindStudentById.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "切换绑定学生")
    @RequestMapping(path = "/cutBindStudent", method = RequestMethod.POST)
    public Result<Boolean> cutBindStudent(@RequestBody CutBindStudentVo cutBindStudentVo) {
        log.info("IdentityChooseController.cutBindStudent.param：{}", cutBindStudentVo);
        try {
            identityChooseService.cutBindStudent(cutBindStudentVo);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("IdentityChooseController.cutBindStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("IdentityChooseController.cutBindStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
