package com.crunii.micro.service.oneaffair.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.annotation.Desensitization;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordReq;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordVo;
import com.crunii.micro.service.oneaffair.manager.ImportRecordManager;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;
import java.util.List;

@Slf4j
@Api(tags = "批量导入记录API接口")
@RestController
@RequestMapping("/yjs/importRecord/auth")
public class ImportRecordController {

    @Resource
    private ImportRecordManager importRecordManager;

    @ApiOperation(value = "分页查询批量导入记录信息")
    @RequestMapping(path = "/pageSearchImportRecord", method = RequestMethod.POST)
    @Desensitization
    @ResponseBody
    public Result<PageRsp<ImportRecordVo>> pageSearchImportRecord(@RequestBody PageReq<ImportRecordReq> req) {
        log.info("ImportRecordController.pageSearchImportRecord.param：{}", req.toString());
        try {
            return Result.ofSuccess(importRecordManager.pageSearchImportRecord(req));
        } catch (BusinessException e) {
            log.error("ImportRecordController.pageSearchImportRecord.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordController.pageSearchImportRecord.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "列表查询批量导入记录信息")
    @RequestMapping(path = "/selectImportRecordList", method = RequestMethod.POST)
    @Desensitization
    @ResponseBody
    public Result<List<ImportRecordVo>> selectImportRecordList(@RequestBody ImportRecordReq req) {
        log.info("ImportRecordController.selectImportRecordList.param：{}", req.toString());
        try {
            return Result.ofSuccess(importRecordManager.selectImportRecordList(req));
        } catch (BusinessException e) {
            log.error("ImportRecordController.selectImportRecordList.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordController.selectImportRecordList.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("批量导入的结果excel导出")
    @RequestMapping(value = "/exportImportResultExcel", method = {RequestMethod.GET})
    @ResponseBody
    public Result<Boolean> exportImportResultExcel(HttpServletResponse response, @RequestParam("importRecordId") BigInteger importRecordId) {
        log.info("ImportRecordController.exportImportResultExcel.param：{}", importRecordId.toString());
        try {
            importRecordManager.exportImportResultExcel(response, importRecordId);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("ImportRecordController.exportImportResultExcel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordController.exportImportResultExcel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("重试")
    @RequestMapping(value = "/reExecuteTask", method = {RequestMethod.GET})
    @ResponseBody
    public Result<Boolean> reExecuteTask(@RequestParam("importRecordId") BigInteger importRecordId) {
        log.info("ImportRecordController.reExecuteTask.param：{}", importRecordId);
        try {
            importRecordManager.reExecuteTask(importRecordId);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("ImportRecordController.reExecuteTask.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordController.reExecuteTask.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
