package com.crunii.micro.service.oneaffair.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.TsmPropertyValue;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsMedal;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveVo;
import com.crunii.micro.service.oneaffair.dto.vo.medal.*;
import com.crunii.micro.service.oneaffair.service.MedalActivityService;
import com.crunii.micro.service.oneaffair.service.MedalRuleService;
import com.crunii.micro.service.oneaffair.service.MedalService;
import com.crunii.micro.service.oneaffair.service.MedalUnifyService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * @author : yingtiansheng
 * @date : 2023-9-20
 */
@Slf4j
@Api(tags = "奖章API接口")
@RestController
@RequestMapping("/yjs/medal")
public class MedalController {

    @Resource
    private MedalService medalService;

    @Resource
    private MedalActivityService medalActivityService;

    @Resource
    private MedalRuleService medalRuleService;

    @Resource
    private MedalUnifyService medalUnifyService;

    @ApiOperation(value = "分页查询奖章信息")
    @RequestMapping(path = "/auth/pageSearchMedalInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<MedalInfoVo>> pageSearchMedalInfo(@RequestBody PageReq<MedalReq> req, @ApiIgnore @CurrWorker Worker worker) {
        try {
            return Result.ofSuccess(medalService.pageSearchMedalInfo(req, worker));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "分页查询统一奖章信息")
    @RequestMapping(path = "/auth/pageSearchMedalUnifyInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<MedalUnifyInfoVo>> pageSearchMedalUnifyInfo(@RequestBody PageReq<MedalReq> req) {
        try {
            return Result.ofSuccess(medalUnifyService.pageSearchMedalUnifyInfo(req));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询奖章信息列表")
    @RequestMapping(path = "/auth/listSearchMedalInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<YcqYjsMedal>> listSearchMedalInfo(@RequestBody MedalReq req) {
        try {
            return Result.ofSuccess(medalService.selectList(req));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据id查询奖章信息")
    @RequestMapping(path = "/auth/searchMedalInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<YcqYjsMedal> searchMedalInfo(@Validated @RequestBody @ApiParam StudentActivityReq studentActivityReq) {
        try {
            return Result.ofSuccess(medalService.selectMedalLight(studentActivityReq));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新建奖章信息")
    @RequestMapping(path = "/auth/createMedalInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createMedalInfo(@Validated @RequestBody @ApiParam MedalCreateReq createReq, @ApiIgnore @CurrWorker Worker worker) {
        log.info("MedalController.createMedalInfo.param：{}", createReq.toString());
        try {
            //return medalService.createMedalInfo(createReq, worker);
            return medalService.createMedalInfoMulti(createReq, worker);
        } catch (BusinessException e) {
            log.error("MedalController.createMedalInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.createMedalInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新建全国统一奖章信息")
    @RequestMapping(path = "/auth/createMedalUnifyInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createMedalUnifyInfo(@Validated @RequestBody @ApiParam MedalUnifyCreateReq createReq,@ApiIgnore @CurrWorker Worker worker) {
        log.info("MedalController.createMedalUnifyInfo.param：{}", createReq.toString());
        try {
            return medalUnifyService.createMedalUnifyInfo(createReq,worker);
        } catch (BusinessException e) {
            log.error("MedalController.createMedalUnifyInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.createMedalUnifyInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改奖章信息")
    @RequestMapping(path = "/auth/updateMedalInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateMedalInfo(@Validated @RequestBody @ApiParam MedalUpdateReq updateReq,@ApiIgnore @CurrWorker Worker worker) {
        log.info("MedalController.updateMedalInfo.param：{}", updateReq.toString());
        try {
            return medalService.updateMedalInfo(updateReq,worker);
            //medalService.updateMedalInfoMulti(updateReq,worker);
            //return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("MedalController.updateMedalInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.updateMedalInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改全国统一奖章信息")
    @RequestMapping(path = "/auth/updateMedalUnifyInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateMedalUnifyInfo(@Validated @RequestBody @ApiParam MedalUnifyUpdateReq updateReq,@ApiIgnore @CurrWorker Worker worker) {
        log.info("MedalController.updateMedalUnifyInfo.param：{}", updateReq.toString());
        try {
            return medalUnifyService.updateMedalUnifyInfo(updateReq,worker);
        } catch (BusinessException e) {
            log.error("MedalController.updateMedalUnifyInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.updateMedalUnifyInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "切换展示状态")
    @RequestMapping(path = "/auth/changeUnifyDisplayStatus", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> changeUnifyDisplayStatus(@Validated @RequestBody @ApiParam MedalDisplayStatusReq displayStatusReq,@ApiIgnore @CurrWorker Worker worker) {
        log.info("MedalController.changeUnifyDisplayStatus.param：{}", displayStatusReq.toString());
        try {
            medalUnifyService.changeDisplayStatus(displayStatusReq,worker);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("MedalController.changeDisplayStatus.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.changeDisplayStatus.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("删除统一奖章")
    @GetMapping("/auth/delMedalUnifyInfo")
    public Result<?> delMedalUnifyInfo(BigInteger id){
        if (id == null) {
            log.error("MedalController.delMedalUnifyInfo.error. id is null");
            return Result.ofFail("系统内部异常，请联系管理员！");
        } else {
            medalUnifyService.deleteMedalUnifyInfo(id);
            return Result.ofSuccess(true);
        }
    }

    @ApiOperation("删除")
    @GetMapping("/auth/delMedalInfo")
    public Result<?> delMedalInfo(BigInteger id){
        if (id == null) {
            log.error("MedalController.delMedalInfo.error. id is null");
            return Result.ofFail("系统内部异常，请联系管理员！");
        } else {
            medalService.deleteMedalInfo(id);
            return Result.ofSuccess(true);
        }
    }

    @ApiOperation(value = "切换展示状态")
    @RequestMapping(path = "/auth/changeDisplayStatus", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> changeDisplayStatus(@Validated @RequestBody @ApiParam MedalDisplayStatusReq displayStatusReq,@ApiIgnore @CurrWorker Worker worker) {
        log.info("MedalController.changeDisplayStatus.param：{}", displayStatusReq.toString());
        try {
            medalService.changeDisplayStatus(displayStatusReq,worker);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("MedalController.changeDisplayStatus.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.changeDisplayStatus.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询app接入配置")
    @RequestMapping(path = "/auth/findAppConfigUrl", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<MedalAppConfigVo>> findAppConfigUrl(@ApiIgnore @CurrWorker Worker worker) {
        try {
            return Result.ofSuccess(medalService.queryAppConfigList(worker));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询用户关联业务配置")
    @RequestMapping(path = "/auth/selectUserMedalApp", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<MedalAppInfoVo>> selectUserMedalApp(@ApiIgnore @CurrWorker Worker worker) {
        try {
            return Result.ofSuccess(medalService.selectUserMedalApp(worker));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "批量新建用户选择业务信息")
    @RequestMapping(path = "/auth/batchCreateMedalApp", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> batchCreateMedalApp(@Validated @RequestBody @ApiParam List<MedalAppCreateReq> medalAppCreateReqList, @ApiIgnore @CurrWorker Worker worker) {
        try {
            medalService.batchCreateMedalApp(medalAppCreateReqList,worker);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("MedalController.batchCreateMedalApp.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.batchCreateMedalApp.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("删除用户关联业务")
    @GetMapping("/auth/deleteMedalApp")
    public Result<?> deleteMedalApp(BigInteger id){
        if (id == null) {
            log.error("MedalController.deleteMedalApp.error. id is null");
            return Result.ofFail("系统内部异常，请联系管理员！");
        } else {
            medalService.deleteMedalApp(id);
            return Result.ofSuccess(true);
        }
    }

    @ApiOperation(value = "查询登录用户可选年级")
    @RequestMapping(path = "/auth/selectMedalOrgGrade", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<MedalOrgGradeInfoVo>> selectMedalOrgGrade(@ApiIgnore @CurrWorker Worker worker) {
        try {
            return Result.ofSuccess(medalService.selectMedalOrgGrade(worker));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询奖章类型下的奖章标识")
    @RequestMapping(path = "/auth/selectMedalMark", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<TsmPropertyValue>> selectMedalMark(@Validated @RequestBody @ApiParam  MedalReq medalReq) {
        try {
            return Result.ofSuccess(medalService.selectMedalMarkByType(medalReq.getMedalType()));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "批量新建奖章活动关联信息")
    @RequestMapping(path = "/auth/batchCreateMedalActivity", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> batchCreateMedalActivity(@Validated @RequestBody @ApiParam MedalActivityCreateReq medalActivityCreateReq, @ApiIgnore @CurrWorker Worker worker) {
        log.info("MedalController.batchCreateMedalActivity.param：{}", medalActivityCreateReq.toString());
        try {
            medalActivityService.batchCreateMedalActivity(medalActivityCreateReq,worker);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("MedalController.batchCreateMedalActivity.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.batchCreateMedalActivity.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("删除奖章活动关联信息")
    @RequestMapping(path = "/auth/deleteMedalActivityInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<?> deleteMedalActivityInfo(@Validated @RequestBody @ApiParam MedalActivityRemoveReq medalActivityRemoveReq){
        log.info("MedalController.deleteMedalInfo.param：{}", medalActivityRemoveReq.toString());
        try {
            medalActivityService.deleteMedalActivityInfo(medalActivityRemoveReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("MedalController.deleteMedalInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.deleteMedalInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("更新奖章活动状态信息")
    @RequestMapping(path = "/auth/updateMedalActivityStatus", method = RequestMethod.POST)
    @ResponseBody
    public Result<?> updateMedalActivityStatus(@Validated @RequestBody @ApiParam MedalActivityStatusReq medalActivityStatusReq){
        log.info("MedalController.updateMedalActivityStatus.param：{}", medalActivityStatusReq.toString());
        try {
            medalActivityService.updateMedalActivityStatus(medalActivityStatusReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("MedalController.updateMedalActivityStatus.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.updateMedalActivityStatus.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "分页查询不与该奖章关联的活动信息")
    @RequestMapping(path = "/auth/pageSearchMedalNoActivityInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<MedalActivityInfoVo>> pageSearchMedalNoActivityInfo(@RequestBody PageReq<ActivityReq> req) {
        try {
            return Result.ofSuccess(medalService.pageSearchMedalNoActivityInfo(req));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "分页查询与该奖章关联的活动信息")
    @RequestMapping(path = "/auth/pageSearchMedalActivityInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<MedalActivityInfoVo>> pageSearchMedalActivityInfo(@RequestBody PageReq<ActivityReq> req) {
        try {
            return Result.ofSuccess(medalService.pageSearchMedalActivityInfo(req));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询奖章下活动信息列表")
    @RequestMapping(path = "/auth/listSearchMedalActivityInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<MedalActivityInfoVo>> listSearchMedalActivityInfo(@RequestBody ActivityReq req) {
        try {
            return Result.ofSuccess(medalService.selectActivityList(req.getMedalId()));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询奖章下学生参加的活动信息列表")
    @RequestMapping(path = "/auth/selectStudentActivityList", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<ArchiveVo>> selectStudentActivityList(@RequestBody StudentActivityReq req) {
        try {
            return Result.ofSuccess(medalService.selectStudentActivityList(req));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询统一奖章使用情况列表")
    @RequestMapping(path = "/auth/searchMedalUnifySchoolInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<MedalUnifySchoolInfoVo>> searchMedalUnifySchoolInfo(@RequestBody MedalSchoolReq medalSchoolReq) {
        try {
            return Result.ofSuccess(medalUnifyService.searchMedalUnifySchoolInfo(medalSchoolReq));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("根据统一编号查询关联奖章信息")
    @GetMapping("/auth/searchMedalInfoByUnifyMedalNo")
    public Result<?> searchMedalInfoByUnifyMedalNo(String medalNo){
        try {
            if (medalNo == null) {
                log.error("MedalController.searchMedalInfoByUnifyMedalNo.error. id is null");
                return Result.ofFail("系统内部异常，请联系管理员！");
            } else {
                return Result.ofSuccess(medalUnifyService.searchMedalInfoByUnifyMedalNo(medalNo));
            }
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询可添加统一奖章信息列表")
    @RequestMapping(path = "/auth/searchAvailableMedalUnifyInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<MedalUnifyInfoVo>> searchAvailableMedalUnifyInfo(@RequestBody MedalReq req) {
        try {
            return Result.ofSuccess(medalService.searchAvailableMedalUnifyInfo(req));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("根据统一编号查询全国统一奖章数据")
    @GetMapping("/auth/searchUnifyMedalInfo")
    public Result<?> searchUnifyMedalInfo(String medalNo){
        try {
            if (medalNo == null) {
                log.error("MedalController.searchUnifyMedalInfo.error. id is null");
                return Result.ofFail("系统内部异常，请联系管理员！");
            } else {
                return Result.ofSuccess(medalService.searcMedalUnifyInfoByMedalNo(medalNo));
            }
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "批量新建争章规则信息")
    @RequestMapping(path = "/auth/batchCreateMedalRule", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> batchCreateMedalRule(@Validated @RequestBody @ApiParam MedalRuleCreateReq medalRuleCreateReq,@ApiIgnore @CurrWorker Worker worker) {
        log.info("MedalController.batchCreateMedalRule.param：{}", medalRuleCreateReq.toString());
        try {
            return medalRuleService.batchCreateMedalRule(medalRuleCreateReq,worker);
        } catch (BusinessException e) {
            log.error("MedalController.batchCreateMedalRule.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.batchCreateMedalRule.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改争章规则信息")
    @RequestMapping(path = "/auth/updateMedalRule", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateMedalRule(@Validated @RequestBody @ApiParam MedalRuleUpdateReq medalRuleUpdateReq,@ApiIgnore @CurrWorker Worker worker) {
        log.info("MedalController.updateMedalRule.param：{}", medalRuleUpdateReq.toString());
        try {
            return medalRuleService.updateMedalRule(medalRuleUpdateReq,worker);
        } catch (BusinessException e) {
            log.error("MedalController.updateMedalRule.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("MedalController.updateMedalRule.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询可配置争章规则奖章列表")
    @RequestMapping(path = "/auth/searchNoRuleMedal", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<MedalBaseInfoVo>> searchNoRuleMedal(@ApiIgnore @CurrWorker Worker worker) {
        try {
            return Result.ofSuccess(medalService.searchNoRuleMedal(worker));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据奖章编号查询可配置争章规则奖章列表")
    @RequestMapping(path = "/auth/searchMedalGradeByMedalNo", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<MedalGradeInfoVo>> searchMedalGradeByMedalNo(@Validated @RequestBody @ApiParam MedalNoReq medalNoReq, @ApiIgnore @CurrWorker Worker worker) {
        try {
            if (medalNoReq == null || StringUtils.isEmpty(medalNoReq.getMedalNo())) {
                log.error("MedalController.searchMedalGradeByMedalNo.error. id is null");
                return Result.ofFail("参数异常");
            } else {
                return Result.ofSuccess(medalService.searchMedalGradeByMedalNo(medalNoReq,worker));
            }
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据奖章id查询可配置规则年级")
    @RequestMapping(path = "/auth/selectGradeInfoByMedalId", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<TsmOrg>> selectGradeInfoByMedalId(BigInteger id) {
        try {
            return Result.ofSuccess(medalService.selectGradeInfoByMedalId(id));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("删除争章规则")
    @GetMapping("/auth/delMedalRule")
    public Result<?> delMedalRule(BigInteger id){
        if (id == null) {
            log.error("MedalController.delMedalRule.error. id is null");
            return Result.ofFail("系统内部异常，请联系管理员！");
        } else {
            medalRuleService.delMedalRule(id);
            return Result.ofSuccess(true);
        }
    }

    @ApiOperation(value = "分页查询争章规则信息")
    @RequestMapping(path = "/auth/pageSearchMedalRuleInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<MedalRuleInfoVo>> pageSearchMedalRuleInfo(@RequestBody PageReq<MedalReq> req, @ApiIgnore @CurrWorker Worker worker) {
        try {
            return Result.ofSuccess(medalRuleService.pageSearchMedalRuleInfo(req, worker));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据奖章id查询规则信息")
    @RequestMapping(path = "/auth/selectMedalRuleInfo", method = RequestMethod.GET)
    @ResponseBody
    public Result<MedalRuleCreateReq> selectMedalRuleInfo(BigInteger medalId, String ownerGrade) {
        try {
            return Result.ofSuccess(medalRuleService.selectMedalRuleInfo(medalId));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据奖章标识分组查询奖章信息列表")
    @RequestMapping(path = "/auth/selectListGroupByMedalMark", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<MedalInfoMarkVo>> selectListGroupByMedalMark(@RequestBody MedalReq req) {
        try {
            return Result.ofSuccess(medalService.selectListGroupByMedalMark(req));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据学生信息分组查询奖章信息列表")
    @RequestMapping(path = "/auth/selectListGroupByStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<MedalInfoTypeVo>> selectListGroupByStudent(@RequestBody MedalStudentReq req) {
        try {
            return Result.ofSuccess(medalService.selectListGroupByMedalType(req));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "获取登录用户学校权限列表")
    @RequestMapping(path = "/auth/findSchoolInfo", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<TsmOrg>> selectSchoolByUserId(@ApiIgnore @CurrWorker Worker worker) {
        try {
            return Result.ofSuccess(medalService.selectSchoolByUserId(worker));
        } catch (BusinessException e) {
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }
}
