package com.crunii.micro.service.oneaffair.controller;

import com.crunii.framework.api.encrypt.annotation.EncryptRequestBody;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.annotation.Desensitization;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentHonorary;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentPosition;
import com.crunii.micro.service.oneaffair.dto.vo.student.*;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryBatchCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryEditReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.StudentPositionBatchCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.StudentPositionEditReq;
import com.crunii.micro.service.oneaffair.manager.StudentManager;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;
import java.util.List;
import java.util.Map;

/**
 * @author : dinglian
 * @date : 2023-9-19
 */
@Slf4j
@Api(tags = "学生API接口")
@RestController
@RequestMapping("/yjs/student/auth")
public class StudentController {

    @Resource
    private StudentManager studentManager;

    @ApiOperation(value = "分页查询学生信息")
    @RequestMapping(path = "/pageSearchStudentInfo", method = RequestMethod.POST)
    @Desensitization
    @ResponseBody
    public Result<PageRsp<StudentInfoVo>> pageSearchStudentInfo(@RequestBody PageReq<StudentReq> req) {
        log.info("StudentController.pageSearchStudentInfo.param：{}", req.toString());
        try {
            return Result.ofSuccess(studentManager.pageSearchStudentInfo(req));
        } catch (BusinessException e) {
            log.error("StudentController.pageSearchStudentInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.pageSearchStudentInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询学生信息详情")
    @RequestMapping(path = "/selectStudentInfo", method = RequestMethod.GET)
    @ResponseBody
    public Result<StudentInfoVo> selectStudentInfo(@RequestParam("studentId") BigInteger studentId) {
        log.info("StudentController.selectStudentInfo.param：{}", studentId);
        try {
            return Result.ofSuccess(studentManager.selectStudentInfo(studentId));
        } catch (BusinessException e) {
            log.error("StudentController.selectStudentInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.selectStudentInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据身份证查询少先队员的学生信息详情")
    @RequestMapping(path = "/selectStudentByIdentityCardNo", method = RequestMethod.GET)
    @ResponseBody
    public Result<StudentDetailVo> selectStudentByIdentityCardNo(@RequestParam("identityCardNo") String identityCardNo) {
        log.info("StudentController.selectStudentByIdentityCardNo.param：{}", identityCardNo);
        try {
            return studentManager.selectStudentByIdentityCardNo(identityCardNo);
        } catch (BusinessException e) {
            log.error("StudentController.selectStudentByIdentityCardNo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.selectStudentByIdentityCardNo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据身份证和姓名查询少先队员的学生信息详情")
    @RequestMapping(path = "/selectStudentByIdentityCardNoAndStudentName", method = RequestMethod.GET)
    @ResponseBody
    public Result<StudentDetailVo> selectStudentByIdentityCardNoAndStudentName(@RequestParam("identityCardNo") String identityCardNo, @RequestParam("studentName") String studentName) {
        log.info("StudentController.selectStudentByIdentityCardNoAndStudentName.param：identityCardNo:{},studentName:{}", identityCardNo, studentName);
        try {
            return studentManager.selectStudentByIdentityCardNoAndStudentName(identityCardNo, studentName);
        } catch (BusinessException e) {
            log.error("StudentController.selectStudentByIdentityCardNoAndStudentName.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.selectStudentByIdentityCardNoAndStudentName.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新建学生信息")
    @RequestMapping(path = "/encryptionRequired/createStudentInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createStudentInfo(@Validated @EncryptRequestBody StudentCreateReq createReq) {
        log.info("StudentController.createStudentInfo.param：{}", createReq.toString());
        try {
            studentManager.createStudentInfo(createReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.createStudentInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.createStudentInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改学生信息")
    @RequestMapping(path = "/encryptionRequired/updateStudentInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateStudentInfo(@Validated @EncryptRequestBody StudentUpdateReq updateReq) {
        log.info("StudentController.updateStudentInfo.param：{}", updateReq.toString());
        try {
            studentManager.updateStudentInfo(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.updateStudentInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.updateStudentInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "设置学生入队时间")
    @RequestMapping(path = "/enqueueStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> enqueueStudent(@Validated @RequestBody StudentEnqueueReq enqueueReq) {
        log.info("StudentController.enqueueStudent.param：{}", enqueueReq.toString());
        try {
            studentManager.enqueueStudent(enqueueReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.enqueueStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.enqueueStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "学生退出少先队")
    @RequestMapping(path = "/quitEnqueueByStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> quitEnqueueByStudent(@Validated @RequestBody StudentBatchUpdateReq updateReq) {
        log.info("StudentController.quitEnqueueByStudent.param：{}", updateReq.toString());
        try {
            studentManager.quitEnqueueByStudent(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.quitEnqueueByStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.quitEnqueueByStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "导师评分")
    @RequestMapping(path = "/teacherScore", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> teacherScore(@Validated @RequestBody TeacherScoreReq updateReq) {
        log.info("StudentController.teacherScore.param：{}", updateReq.toString());
        try {
            studentManager.teacherScore(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.teacherScore.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.teacherScore.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "推优入团")
    @RequestMapping(path = "/promoteBestToLeague", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> promoteBestToLeague(@Validated @RequestBody PromoteBestToLeagueReq updateReq) {
        log.info("StudentController.promoteBestToLeague.param：{}", updateReq.toString());
        try {
            studentManager.promoteBestToLeague(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.promoteBestToLeague.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.promoteBestToLeague.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "学生退出共青团")
    @RequestMapping(path = "/quitLeagueByStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> quitLeagueByStudent(@Validated @RequestBody StudentBatchUpdateReq updateReq) {
        log.info("StudentController.quitLeagueByStudent.param：{}", updateReq.toString());
        try {
            studentManager.quitLeagueByStudent(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.quitLeagueByStudent.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.quitLeagueByStudent.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除学生信息")
    @RequestMapping(path = "/deleteStudentInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> deleteStudentInfo(@Validated @RequestBody StudentInfoDeleteReq deleteReq) {
        log.info("StudentController.deleteStudentInfo.param：{}", deleteReq.toString());
        try {
            studentManager.deleteStudentInfo(deleteReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.deleteStudentInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.deleteStudentInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("学生信息excel导出")
    @RequestMapping(value = "/exportStudentExcel", method = {RequestMethod.POST})
    @ResponseBody
    public Result<Boolean> exportStudentExcel(HttpServletResponse response, @RequestBody StudentReq req) {
        log.info("StudentController.exportStudentExcel.param：{}", req.toString());
        try {
            studentManager.exportStudentExcel(response, req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.exportStudentExcel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.exportStudentExcel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "推优入团分析报告")
    @RequestMapping(path = "/promoteBestToLeagueReport", method = RequestMethod.GET)
    @ResponseBody
    public Result<Map<String, String>> promoteBestToLeagueReport(@RequestParam("studentId") Long studentId, @RequestParam("studentNum") String studentNum) {
        log.info("StudentController.promoteBestToLeague.param：{}", studentId.toString());
        try {
            Map<String, String> promoteBestToLeagueReportResp = studentManager.promoteBestToLeagueReport(studentId, studentNum);
            return Result.ofSuccess(promoteBestToLeagueReportResp);
        } catch (BusinessException e) {
            log.error("StudentController.promoteBestToLeague.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.promoteBestToLeague.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "Excel模板导入批量新增队前学生信息")
    @RequestMapping(path = "/importStudentInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> importStudentInfo(HttpServletResponse response, @ApiParam(value = "导入文件", required = true) @RequestParam(value = "file") MultipartFile file) {
        log.info("StudentController.importStudentInfo.param：{}", file.getName());
        try {
            Boolean flag = studentManager.importStudentInfo(response, file);
            if (flag) {
                return Result.ofSuccess(flag);
            } else {
                return Result.ofFail("Excel模板导入批量新增队前学生信息");
            }
        } catch (BusinessException e) {
            log.error("StudentController.importStudentInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.importStudentInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "Excel模板导入批量新增入队学生信息")
    @RequestMapping(path = "/importStudentInfoEnqueue", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> importStudentInfoEnqueue(HttpServletResponse response, @ApiParam(value = "导入文件", required = true) @RequestParam(value = "file") MultipartFile file) {
        log.info("StudentController.importStudentInfoEnqueue.param：{}", file.getName());
        try {
            Boolean flag = studentManager.importStudentInfoTwo(response, file);
            if (flag) {
                return Result.ofSuccess(flag);
            } else {
                return Result.ofFail("Excel模板导入批量新增入队学生信息");
            }
        } catch (BusinessException e) {
            log.error("StudentController.importStudentInfoEnqueue.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.importStudentInfoEnqueue.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "Excel模板导入批量新增推优入团学生信息")
    @RequestMapping(path = "/importStudentInfoLeague", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> importStudentInfoLeague(HttpServletResponse response, @ApiParam(value = "导入文件", required = true) @RequestParam(value = "file") MultipartFile file) {
        log.info("StudentController.importStudentInfoLeague.param：{}", file.getName());
        try {
            Boolean flag = studentManager.importStudentInfoThree(response, file);
            if (flag) {
                return Result.ofSuccess(flag);
            } else {
                return Result.ofFail("Excel模板导入批量新增推优入团学生信息");
            }
        } catch (BusinessException e) {
            log.error("StudentController.importStudentInfoLeague.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.importStudentInfoLeague.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "批量设置学生荣誉称号")
    @RequestMapping(path = "/batchCreateStudentHonorary", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> batchCreateStudentHonorary(@Validated @RequestBody StudentHonoraryBatchCreateReq updateReq) {
        log.info("StudentController.batchCreateStudentHonorary.param：{}", updateReq.toString());
        try {
            studentManager.batchCreateStudentHonorary(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.batchCreateStudentHonorary.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.batchCreateStudentHonorary.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "编辑学生荣誉称号")
    @RequestMapping(path = "/editStudentsHonorary", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> editStudentsHonorary(@Validated @RequestBody StudentHonoraryEditReq updateReq) {
        log.info("StudentController.editStudentsHonorary.param：{}", updateReq.toString());
        try {
            studentManager.editStudentsHonorary(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.editStudentsHonorary.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.editStudentsHonorary.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询指定学生的学生荣誉称号列表")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "studentId", value = "学生ID", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/selectStudentsHonoraryList", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<YcqYjsStudentHonorary>> selectStudentsHonoraryList(@RequestParam("studentId") BigInteger studentId) {
        log.info("StudentController.selectStudentsHonoraryList.param：{}", studentId);
        try {
            return Result.ofSuccess(studentManager.selectStudentsHonoraryList(studentId));
        } catch (BusinessException e) {
            log.error("StudentController.selectStudentsHonoraryList.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.selectStudentsHonoraryList.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "批量设置学生岗位评级")
    @RequestMapping(path = "/batchCreateStudentPosition", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> batchCreateStudentPosition(@Validated @RequestBody StudentPositionBatchCreateReq updateReq) {
        log.info("StudentController.batchCreateStudentPosition.param：{}", updateReq.toString());
        try {
            studentManager.batchCreateStudentPosition(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.batchCreateStudentPosition.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.batchCreateStudentPosition.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "编辑学生岗位评级")
    @RequestMapping(path = "/editStudentPosition", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> editStudentPosition(@Validated @RequestBody StudentPositionEditReq updateReq) {
        log.info("StudentController.editStudentPosition.param：{}", updateReq.toString());
        try {
            studentManager.editStudentPosition(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentController.editStudentPosition.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.editStudentPosition.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询指定学生的学生岗位评级列表")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "studentId", value = "学生ID", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/selectStudentsPositionList", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<YcqYjsStudentPosition>> selectStudentsPositionList(@RequestParam("studentId") BigInteger studentId) {
        log.info("StudentController.selectStudentsPositionList.param：{}", studentId);
        try {
            return Result.ofSuccess(studentManager.selectStudentsPositionList(studentId));
        } catch (BusinessException e) {
            log.error("StudentController.selectStudentsPositionList.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentController.selectStudentsPositionList.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
