package com.crunii.micro.service.oneaffair.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentCourse;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentCourseUploadVideo;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentSubCourse;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentTest;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourse.*;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourseUploadVideo.StudentCourseUploadVideoCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourseUploadVideo.StudentCourseUploadVideoReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentSubCourse.StudentSubCourseCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentSubCourse.StudentSubCourseInfoVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentSubCourse.StudentSubCourseReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentSubCourse.StudentSubCourseViewProgressReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.*;
import com.crunii.micro.service.oneaffair.manager.StudentCourseManager;
import com.sun.org.apache.xpath.internal.operations.Bool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

@Slf4j
@Api(tags = "学生课程API接口")
@RestController
@RequestMapping("/yjs/student/course/auth")
public class StudentCourseController {

    @Resource
    private StudentCourseManager studentCourseManager;

    @ApiOperation(value = "列表查询学生课程信息")
    @RequestMapping(path = "/selectStudentCourseInfoList", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<StudentCourseInfoVo>> selectStudentCourseInfoList(@RequestBody StudentCourseReq req) {
        log.info("StudentCourseController.selectStudentCourseInfoList.param：{}", req.toString());
        try {
            List<StudentCourseInfoVo> studentCourseInfoVos = studentCourseManager.selectStudentCourseInfoList(req);
            return Result.ofSuccess(studentCourseInfoVos);
        } catch (BusinessException e) {
            log.error("StudentCourseController.selectStudentCourseInfoList.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.selectStudentCourseInfoList.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "列表查询学生参与的课程信息")
    @RequestMapping(path = "/selectStudentCourseListInStudy", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<StudentCourseInfoVo>> selectStudentCourseListInStudy(@RequestBody StudentCourseReq req) {
        log.info("StudentCourseController.selectStudentCourseListInStudy.param：{}", req.toString());
        try {
            List<StudentCourseInfoVo> studentCourseInfoVos = studentCourseManager.selectStudentCourseListInStudy(req);
            return Result.ofSuccess(studentCourseInfoVos);
        } catch (BusinessException e) {
            log.error("StudentCourseController.selectStudentCourseListInStudy.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.selectStudentCourseListInStudy.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "统计学生学习课程情况")
    @RequestMapping(path = "/sumStudentCourse", method = RequestMethod.POST)
    @ResponseBody
    public Result<StudentCourseSumVo> sumStudentCourse(@RequestBody StudentCourseReq req) {
        log.info("StudentCourseController.sumStudentCourse.param：{}", req.toString());
        try {
            return Result.ofSuccess(studentCourseManager.sumStudentCourse(req));
        } catch (BusinessException e) {
            log.error("StudentCourseController.sumStudentCourse.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.sumStudentCourse.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新建学生课程")
    @RequestMapping(path = "/createStudentCourse", method = RequestMethod.POST)
    @ResponseBody
    public Result<YcqYjsStudentCourse> createStudentCourse(@Validated @RequestBody StudentCourseCreateReq createReq) {
        log.info("StudentCourseController.createStudentCourse.param：{}", createReq.toString());
        try {
            return Result.ofSuccess(studentCourseManager.createStudentCourse(createReq));
        } catch (BusinessException e) {
            log.error("StudentCourseController.createStudentCourse.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.createStudentCourse.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "列表查询学生课程上传视频")
    @RequestMapping(path = "/selectStudentCourseUploadVideoList", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<YcqYjsStudentCourseUploadVideo>> selectStudentCourseUploadVideoList(@RequestBody StudentCourseUploadVideoReq req) {
        log.info("StudentCourseController.selectStudentCourseUploadVideoList.param：{}", req.toString());
        try {
            return Result.ofSuccess(studentCourseManager.selectStudentCourseUploadVideoList(req));
        } catch (BusinessException e) {
            log.error("StudentCourseController.selectStudentCourseUploadVideoList.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.selectStudentCourseUploadVideoList.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "获取视频唯一标识")
    @RequestMapping(path = "/getFileNumber", method = RequestMethod.POST)
    @ResponseBody
    public Result<String> getFileNumber() {
        log.info("StudentCourseController.getFileNumber");
        try {
            return Result.ofSuccess(studentCourseManager.getFileNumber());
        } catch (BusinessException e) {
            log.error("StudentCourseController.getFileNumber.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.getFileNumber.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "学生课程上传视频")
    @RequestMapping(path = "/studentCourseUploadVideo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> studentCourseUploadVideo(@Validated @RequestBody StudentCourseUploadVideoCreateReq req) {
        log.info("StudentCourseController.studentCourseUploadVideo.param：{}", req.toString());
        try {
            studentCourseManager.studentCourseUploadVideo(req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentCourseController.studentCourseUploadVideo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.studentCourseUploadVideo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除学生课程的上传视频")
    @RequestMapping(path = "/deleteStudentCourseVideo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> deleteStudentCourseVideo(@Validated @RequestBody StudentCourseVideoDeleteReq req) {
        log.info("StudentCourseController.deleteStudentCourseVideo.param：{}", req.toString());
        try {
            studentCourseManager.deleteStudentCourseVideo(req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentCourseController.deleteStudentCourseVideo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.deleteStudentCourseVideo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "学生课程视频观看进度")
    @RequestMapping(path = "/studentCourseVideoViewProgress", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> studentCourseVideoViewProgress(@Validated @RequestBody StudentCourseVideoViewProgressReq req) {
        log.info("StudentCourseController.studentCourseVideoViewProgress.param：{}", req.toString());
        try {
            studentCourseManager.studentCourseVideoViewProgress(req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentCourseController.studentCourseVideoViewProgress.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.studentCourseVideoViewProgress.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据学生课程ID列表查询学生子课程信息")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "studentCourseId", value = "学生课程ID", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/selectStudentSubCourseInfoListByStudentCourseId", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<StudentSubCourseInfoVo>> selectStudentSubCourseInfoListByStudentCourseId(@RequestParam("studentCourseId") BigInteger studentCourseId) {
        log.info("StudentCourseController.selectStudentSubCourseInfoListByStudentCourseId.param：{}", studentCourseId);
        try {
            List<StudentSubCourseInfoVo> studentSubCourseInfoVos = studentCourseManager.selectStudentSubCourseInfoListByStudentCourseId(studentCourseId);
            return Result.ofSuccess(studentSubCourseInfoVos);
        } catch (BusinessException e) {
            log.error("StudentCourseController.selectStudentSubCourseInfoListByStudentCourseId.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.selectStudentSubCourseInfoListByStudentCourseId.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "学生子课程视频观看进度")
    @RequestMapping(path = "/studentSubCourseVideoViewProgress", method = RequestMethod.POST)
    @ResponseBody
    public Result<BigInteger> studentSubCourseVideoViewProgress(@Validated @RequestBody StudentSubCourseViewProgressReq req) {
        log.info("StudentCourseController.studentSubCourseVideoViewProgress.param：{}", req.toString());
        try {
            return Result.ofSuccess(studentCourseManager.studentSubCourseVideoViewProgress(req));
        } catch (BusinessException e) {
            log.error("StudentCourseController.studentSubCourseVideoViewProgress.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.studentSubCourseVideoViewProgress.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新建学生子课程")
    @RequestMapping(path = "/createStudentSubCourse", method = RequestMethod.POST)
    @ResponseBody
    public Result<YcqYjsStudentSubCourse> createStudentSubCourse(@Validated @RequestBody StudentSubCourseCreateReq createReq) {
        log.info("StudentCourseController.createStudentSubCourse.param：{}", createReq.toString());
        try {
            return Result.ofSuccess(studentCourseManager.createStudentSubCourse(createReq));
        } catch (BusinessException e) {
            log.error("StudentCourseController.createStudentSubCourse.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.createStudentSubCourse.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "减少学生答卷次数-新增式")
    @RequestMapping(path = "/minusResidueTestCount", method = RequestMethod.POST)
    @ResponseBody
    public Result<YcqYjsStudentCourse> minusResidueTestCount(@Validated @RequestBody MinusResidueTestCountReq createReq) {
        log.info("StudentCourseController.minusResidueTestCount.param：{}", createReq.toString());
        try {
            return Result.ofSuccess(studentCourseManager.minusResidueTestCount(createReq));
        } catch (BusinessException e) {
            log.error("StudentCourseController.minusResidueTestCount.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.minusResidueTestCount.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "学生交卷-新增式")
    @RequestMapping(path = "/submitStudentTest", method = RequestMethod.POST)
    @ResponseBody
    public Result<YcqYjsStudentTest> submitStudentTest(@Validated @RequestBody StudentTestCreateReq createReq) {
        log.info("StudentCourseController.submitStudentTest.param：{}", createReq.toString());
        try {
            return Result.ofSuccess(studentCourseManager.submitStudentTest(createReq));
        } catch (BusinessException e) {
            log.error("StudentCourseController.submitStudentTest.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.submitStudentTest.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "初始化答卷并扣减考试次数-更新式")
    @RequestMapping(path = "/initStudentTest", method = RequestMethod.POST)
    @ResponseBody
    public Result<InitStudentTestVo> initStudentTest(@Validated @RequestBody InitStudentTestReq createReq) {
        log.info("StudentCourseController.initStudentTest.param：{}", createReq.toString());
        try {
            return Result.ofSuccess(studentCourseManager.initStudentTest(createReq));
        } catch (BusinessException e) {
            log.error("StudentCourseController.initStudentTest.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.initStudentTest.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "学生交卷-更新式")
    @RequestMapping(path = "/submitStudentTestByUpdate", method = RequestMethod.POST)
    @ResponseBody
    public Result<YcqYjsStudentTest> submitStudentTestByUpdate(@Validated @RequestBody StudentTestUpdateReq createReq) {
        log.info("StudentCourseController.submitStudentTestByUpdate.param：{}", createReq.toString());
        try {
            return Result.ofSuccess(studentCourseManager.submitStudentTestByUpdate(createReq));
        } catch (BusinessException e) {
            log.error("StudentCourseController.submitStudentTestByUpdate.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.submitStudentTestByUpdate.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询学生答卷详情")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "studentTestId", value = "学生答卷ID", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/selectStudentTestInfoById", method = RequestMethod.GET)
    @ResponseBody
    public Result<StudentTestInfoVo> queryById(@RequestParam("studentTestId") BigInteger studentTestId) {
        log.info("StudentCourseController.selectStudentTestInfoById.param：{}", studentTestId);
        try {
            return Result.ofSuccess(studentCourseManager.selectStudentTestInfo(studentTestId));
        } catch (BusinessException e) {
            log.error("StudentCourseController.selectStudentTestInfoById.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.selectStudentTestInfoById.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "分页查询学生答卷")
    @RequestMapping(path = "/pageSearchStudentTest", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<YcqYjsStudentTest>> pageSearchStudentTest(@RequestBody PageReq<StudentTestReq> req) {
        log.info("StudentCourseController.pageSearchStudentTest.param：{}", req.toString());
        try {
            return Result.ofSuccess(studentCourseManager.pageSearchStudentTest(req));
        } catch (BusinessException e) {
            log.error("StudentCourseController.pageSearchStudentTest.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseController.pageSearchStudentTest.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
