package com.crunii.micro.service.oneaffair.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsMedal;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.*;
import com.crunii.micro.service.oneaffair.manager.StudentMedalManager;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

@Slf4j
@Api(tags = "学生奖章API接口")
@RestController
@RequestMapping("/yjs/student/medal/auth")
public class StudentMedalController {

    @Resource
    private StudentMedalManager studentMedalManager;

    @ApiOperation(value = "查询指定学生参与的奖章信息列表")
    @RequestMapping(path = "/selectJoinMedalInfoList", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<StudentMedalInfo>> selectJoinMedalInfoList(@Validated @RequestBody StudentMedalInfoReq req) {
        log.info("StudentMedalController.selectJoinMedalInfoList.param：{}", req.toString());
        try {
            return Result.ofSuccess(studentMedalManager.selectJoinMedalInfoList(req));
        } catch (BusinessException e) {
            log.error("StudentMedalController.selectJoinMedalInfoList.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalController.selectJoinMedalInfoList.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据年级学期奖章类型分组查询指定学生参与的奖章信息列表")
    @RequestMapping(path = "/selectJoinMedalInfoListGroupByGradeAndMedalType", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<StudentMedalGroupByGradeAndMedalTypeInfo>> selectJoinMedalInfoListGroupByGradeAndMedalType(@Validated @RequestBody StudentMedalInfoReq req) {
        log.info("StudentMedalController.selectJoinMedalInfoListGroupByGradeAndMedalType.param：{}", req.toString());
        try {
            return Result.ofSuccess(studentMedalManager.selectJoinMedalInfoListGroupByGradeAndMedalType(req));
        } catch (BusinessException e) {
            log.error("StudentMedalController.selectJoinMedalInfoListGroupByGradeAndMedalType.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalController.selectJoinMedalInfoListGroupByGradeAndMedalType.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据年级学期分组查询指定学生参与的奖章信息列表")
    @RequestMapping(path = "/selectJoinMedalInfoListGroupByGrade", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<StudentMedalGroupByGradeInfo>> selectJoinMedalInfoListGroupByGrade(@Validated @RequestBody StudentMedalInfoReq req) {
        log.info("StudentMedalController.selectJoinMedalInfoListGroupByGrade.param：{}", req.toString());
        try {
            return Result.ofSuccess(studentMedalManager.selectJoinMedalInfoListGroupByGrade(req));
        } catch (BusinessException e) {
            log.error("StudentMedalController.selectJoinMedalInfoListGroupByGrade.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalController.selectJoinMedalInfoListGroupByGrade.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询奖章详情")
    @RequestMapping(path = "/selectStudentMedalDetail", method = RequestMethod.POST)
    @ResponseBody
    public Result<StudentMedalDetailVo> selectStudentMedalDetail(@Validated @RequestBody StudentMedalDetailReq req) {
        log.info("StudentMedalController.selectStudentMedalDetail.param：{}", req.toString());
        try {
            return Result.ofSuccess(studentMedalManager.selectStudentMedalDetail(req));
        } catch (BusinessException e) {
            log.error("StudentMedalController.selectStudentMedalDetail.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalController.selectStudentMedalDetail.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询指定学生待点亮奖章信息列表")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "studentId", value = "学生ID", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/selectWaitLightMedalInfoList", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<YcqYjsMedal>> selectWaitLightMedalInfoList(@RequestParam("studentId") BigInteger studentId) {
        log.info("StudentMedalController.selectWaitLightMedalInfoList.param：{}", studentId);
        try {
            return Result.ofSuccess(studentMedalManager.selectWaitLightMedalInfoList(studentId));
        } catch (BusinessException e) {
            log.error("StudentMedalController.selectWaitLightMedalInfoList.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalController.selectWaitLightMedalInfoList.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据奖章类型分组查询指定学生待点亮奖章信息列表")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "studentId", value = "学生ID", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/selectWaitLightMedalInfoListGroupByMedalType", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<MedalGroupByMedalTypeInfo>> selectWaitLightMedalInfoListGroupByMedalType(@RequestParam("studentId") BigInteger studentId) {
        log.info("StudentMedalController.selectWaitLightMedalInfoListGroupByMedalType.param：{}", studentId);
        try {
            return Result.ofSuccess(studentMedalManager.selectWaitLightMedalInfoListGroupByMedalType(studentId));
        } catch (BusinessException e) {
            log.error("StudentMedalController.selectWaitLightMedalInfoListGroupByMedalType.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalController.selectWaitLightMedalInfoListGroupByMedalType.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新增学生奖章")
    @RequestMapping(path = "/createStudentMedalInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createStudentMedalInfo(@Validated @RequestBody StudentMedalCreateReq req) {
        log.info("StudentMedalController.createStudentMedalInfo.param：{}", req.toString());
        try {
            studentMedalManager.createStudentMedalInfo(req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentMedalController.createStudentMedalInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalController.createStudentMedalInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改学生奖章")
    @RequestMapping(path = "/updateStudentMedalInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateStudentMedalInfo(@Validated @RequestBody StudentMedalDetailUpdateVo req) {
        log.info("StudentMedalController.updateStudentMedalInfo.param：{}", req.toString());
        try {
            studentMedalManager.updateStudentMedalInfo(req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentMedalController.updateStudentMedalInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalController.updateStudentMedalInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改学生奖章的点亮状态")
    @RequestMapping(path = "/updateStudentMedalLightStatus", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateStudentMedalLightStatus(@Validated @RequestBody StudentMedalLightStatusUpdateVo req) {
        log.info("StudentMedalController.updateStudentMedalLightStatus.param：{}", req.toString());
        try {
            studentMedalManager.updateStudentMedalLightStatus(req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("StudentMedalController.updateStudentMedalLightStatus.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalController.updateStudentMedalLightStatus.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


}
