package com.crunii.micro.service.oneaffair.dto.oneaffair;

import javax.persistence.*;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;

import java.io.Serializable;

import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.validator.constraints.Length;
import org.springframework.format.annotation.DateTimeFormat;

/**
* 学生档案信息
* @TableName YCQ_YJS_ARCHIVE
*/
@Entity
@Table(name = "YCQ_YJS_ARCHIVE")
@NamedQuery(name = "YcqYjsArchive.findAll", query = "SELECT t FROM YcqYjsArchive t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = {"id"})
@ApiModel(description = "学生档案信息表")
public class YcqYjsArchive implements Serializable {

    @Id
    @ApiModelProperty(name = "id", notes = "主键")
    @Column(name = "id")
    private String id;
    /**
    * 学生ID
    */
    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("学生ID")
    @Length(max= 32,message="编码长度不能超过32")
    @Column(name = "student_id")
    private String studentId;
    /**
    * 事项名称
    */
    @Size(max= 1000,message="编码长度不能超过1000")
    @ApiModelProperty("事项名称")
    @Length(max= 1000,message="编码长度不能超过1000")
    @Column(name = "name")
    private String name;
    /**
    * 参与时间
    */
    @ApiModelProperty("参与时间")
    @Column(name = "participate_time")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date participateTime;
    /**
    * 事项图片URL
    */
    @Size(max= 1000,message="编码长度不能超过1000")
    @ApiModelProperty("事项图片URL")
    @Length(max= 1000,message="编码长度不能超过1000")
    @Column(name = "img")
    private String img;
    /**
    * 应用代码
    */
    @Size(max= 50,message="编码长度不能超过50")
    @ApiModelProperty("应用代码")
    @Length(max= 50,message="编码长度不能超过50")
    @Column(name = "app_code")
    private String appCode;
    /**
    * 1、志愿服务2、赛事活动3、学习档案4、政治进步入队5、政治进步入团6、政治进步入党7、其他
    */
    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("1、志愿服务2、赛事活动3、学习档案4、政治进步入队5、政治进步入团6、政治进步入党7、其他")
    @Length(max= 32,message="编码长度不能超过32")
    @Column(name = "activity_type")
    private String activityType;
    /**
    * 删除标志
    */
    @ApiModelProperty("删除标志")
    @Column(name = "del_flag")
    private Integer delFlag;
    /**
    * 创建时间
    */
    @ApiModelProperty("创建时间")
    @Column(name = "create_time")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createTime;
    /**
    * 
    */
    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("创建人ID")
    @Length(max= 32,message="编码长度不能超过32")
    @Column(name = "created_id")
    private String createdId;

    @ApiModelProperty("最后更新时间")
    @Column(name = "last_updated_time")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastUpdatedTime;

    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("最后更新人ID")
    @Length(max= 32,message="编码长度不能超过32")
    @Column(name = "last_updated_id")
    private String lastUpdatedId;
    /**
    * 
    */
    @Size(max= 1000,message="编码长度不能超过1000")
    @ApiModelProperty("详情")
    @Length(max= 1000,message="编码长度不能超过1000")
    @Column(name = "detail")
    private String detail;
    /**
     *
     */
    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("活动id、服务id、奖章id等标识")
    @Length(max= 32,message="编码长度不能超过32")
    @Column(name = "event_id")
    private String eventId;

    @ApiModelProperty("事件类型")
    @Column(name = "event_type")
    private Integer eventType;

    /**
    * 事项描述
    */
    @ApiModelProperty("事项描述")
    @Column(name = "description")
    private String description;
    /**
     *政治面貌
     */
    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("政治面貌：1-群众、2-少先队员、-共青团员、4-预备党员、5-共产党员")
    @Length(max= 32,message="编码长度不能超过32")
    @Column(name = "political_type")
    private String politicalType;
    /**
     * 活动分类
     */
    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("活动分类（暂时供赛事活动分类档案使用)")
    @Length(max= 32,message="编码长度不能超过32")
    @Column(name = "extension_type")
    private String extensionType;

    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("用户id")
    @Length(max= 32,message="编码长度不能超过32")
    @Column(name = "user_id")
    private String userId;

    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("用户名称")
    @Length(max= 32,message="编码长度不能超过32")
    @Column(name = "user_name")
    private String userName;

    @Size(max= 32,message="编码长度不能超过32")
    @ApiModelProperty("班级学期")
    @Length(max= 32,message="编码长度不能超过32")
    @Column(name = "class_name")
    private String className;

    @ApiModelProperty("详情话题")
    @Column(name = "detail_topic")
    private String detailTopic;

    @ApiModelProperty("详情介绍")
    @Column(name = "detail_introduction")
    private String detailIntroduction;

    @ApiModelProperty("详情图片")
    @Column(name = "detail_img")
    private String detailImg;

    @ApiModelProperty("拓展json")
    @Column(name = "extension_json")
    private String extensionJson;
}
