package com.crunii.micro.service.oneaffair.dto.oneaffair;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

@Entity
@Table(name = "YCQ_YJS_GOOD_STUDENT")
@NamedQuery(name = "YcqYjsGoodStudent.findAll", query = "SELECT t FROM YcqYjsGoodStudent t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = { "id" })
@ApiModel(description = "优秀学生表")
public class YcqYjsGoodStudent implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @ApiModelProperty(name = "id",notes = "主键")
    @Column(name = "id")
    private BigInteger id;

    @Size(max= 32,message="学生编号不能超过32")
    @Column(name = "student_num")
    @ApiModelProperty(name = "studentNum",notes = "学生编号")
    private String studentNum;

    @Size(max= 64,message="学生名称不能超过64")
    @Column(name = "student_name")
    @ApiModelProperty(name = "studentName",notes = "学生名称")
    private String studentName;

    @Column(name = "sex")
    @ApiModelProperty(name = "sex",notes = "性别: 0-女，1-男")
    private Integer sex;

    @Size(max= 500,message="学生照片URL不能超过500")
    @Column(name = "student_photo_url")
    @ApiModelProperty(name = "studentPhotoUrl",notes = "学生照片URL")
    private String studentPhotoUrl;

    @Size(max= 64,message="学年类型不能超过64")
    @Column(name = "school_year_type")
    @ApiModelProperty(name = "schoolYearType",notes = "学年类型")
    private String schoolYearType;

    @Size(max= 64,message="学年不能超过64")
    @Column(name = "school_year")
    @ApiModelProperty(name = "schoolYear",notes = "学年")
    private String schoolYear;

    @Size(max= 64,message="政治面貌不能超过64")
    @Column(name = "political_status")
    @ApiModelProperty(name = "politicalStatus",notes = "政治面貌")
    private String politicalStatus;

    @Size(max= 64,message="政治面貌类型不能超过64")
    @Column(name = "political_status_type")
    @ApiModelProperty(name = "politicalStatusType",notes = "政治面貌类型：群众/少先队/团队/党员群众")
    private String politicalStatusType;

    @Size(max= 128,message="优秀称号不能超过128")
    @Column(name = "honorary_title")
    @ApiModelProperty(name = "honoraryTitle",notes = "优秀称号")
    private String honoraryTitle;

    @Size(max= 64,message="优秀称号类型不能超过64")
    @Column(name = "honorary_title_type")
    @ApiModelProperty(name = "honoraryTitleType",notes = "优秀称号类型")
    private String honoraryTitleType;

    @Column(name = "school_id")
    @ApiModelProperty(name = "schoolId",notes = "学校ID")
    private BigInteger schoolId;

    @Size(max= 128,message="学校名称不能超过128")
    @Column(name = "school_name")
    @ApiModelProperty(name = "schoolName",notes = "学校名称")
    private String schoolName;

    @Column(name = "grade_id")
    @ApiModelProperty(name = "gradeId",notes = "年级ID")
    private BigInteger gradeId;

    @Size(max= 128,message="年级名称不能超过128")
    @Column(name = "grade_name")
    @ApiModelProperty(name = "gradeName",notes = "年级名称")
    private String gradeName;

    @Column(name = "class_id")
    @ApiModelProperty(name = "classId",notes = "班级ID")
    private BigInteger classId;

    @Size(max= 128,message="班级名称不能超过128")
    @Column(name = "class_name")
    @ApiModelProperty(name = "className",notes = "班级名称")
    private String className;

    @Size(max= 900,message="简介不能超过900")
    @Column(name = "brief_introduction")
    @ApiModelProperty(name = "briefIntroduction",notes = "简介")
    private String briefIntroduction;

    @Column(name = "show_flag")
    @ApiModelProperty(name = "showFlag",notes = "是否展示：0-否；1-是")
    private Integer showFlag;

    @Column(name = "del_flag")
    @ApiModelProperty(name = "delFlag",notes = "是否删除：0-未删除；1-已删除")
    private Integer delFlag;

    @Column(name = "created_time")
    @ApiModelProperty(name = "createdTime",notes = "创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createdTime;

    @Column(name = "created_id")
    @ApiModelProperty(name = "createdId",notes = "创建人ID")
    private BigInteger createdId;

    @Column(name = "last_updated_time")
    @ApiModelProperty(name = "lastUpdatedTime",notes = "最后更新时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastUpdatedTime;

    @Column(name = "last_updated_id")
    @ApiModelProperty(name = "lastUpdatedId",notes = "最后更新人ID")
    private BigInteger lastUpdatedId;

    @Size(max= 128,message="创建人姓名不能超过128")
    @Column(name = "created_name")
    @ApiModelProperty(name = "createdName", notes = "创建人姓名")
    private String createdName;

    @Size(max= 128,message="最后更新人名称不能超过128")
    @Column(name = "last_updated_name")
    @ApiModelProperty(name = "lastUpdatedName", notes = "最后更新人名称")
    private String lastUpdatedName;

}