package com.crunii.micro.service.oneaffair.dto.oneaffair;

import com.crunii.micro.service.oneaffair.annotation.FiledDesensitization;
import com.crunii.micro.service.oneaffair.enums.MaskingTypeEnum;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.validator.constraints.Length;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

@Entity
@Table(name = "YCQ_YJS_STUDENT")
@NamedQuery(name = "YcqYjsStudent.findAll", query = "SELECT t FROM YcqYjsStudent t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = { "id" })
@ApiModel(description = "学生表")
public class YcqYjsStudent implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @ApiModelProperty(name = "id",notes = "主键")
    @Column(name = "id")
    private BigInteger id;

    @Size(max= 32,message="学生编号不能超过32")
    @Column(name = "student_num")
    @ApiModelProperty(name = "studentNum",notes = "学生编号")
    private String studentNum;

    @Size(max= 64,message="学生名称不能超过64")
    @Column(name = "student_name")
    @ApiModelProperty(name = "studentName",notes = "学生名称")
    private String studentName;

    @Column(name = "sex")
    @ApiModelProperty(name = "sex",notes = "性别: 0-女，1-男")
    private Integer sex;

    @Size(max= 18,message="身份证号不能超过18")
    @Column(name = "identity_card_no")
    @ApiModelProperty(name = "identityCardNo",notes = "身份证号")
    @FiledDesensitization(type = MaskingTypeEnum.ID_CARD)
    private String identityCardNo;

    @Size(max= 11,message="手机号不能超过11")
    @Column(name = "mobile")
    @ApiModelProperty(name = "mobile", notes = "手机号")
    @FiledDesensitization(type = MaskingTypeEnum.PHONE)
    private String mobile;

    @Size(max= 128,message="中队名称不能超过128")
    @Column(name = "middle_team_name")
    @ApiModelProperty(name = "middleTeamName",notes = "中队名称")
    private String middleTeamName;

    @Size(max= 64,message="政治面貌类型不能超过64")
    @Column(name = "political_status_type")
    @ApiModelProperty(name = "politicalStatusType",notes = "政治面貌类型：群众/少先队/团队/党员群众")
    private String politicalStatusType;

    @Size(max= 64,message="政治面貌名称不能超过64")
    @Column(name = "political_status")
    @ApiModelProperty(name = "politicalStatus",notes = "政治面貌")
    private String politicalStatus;

    @Size(max= 64,message="学期类型不能超过64")
    @Column(name = "semester_type")
    @ApiModelProperty(name = "semesterType",notes = "学期类型")
    private String semesterType;

    @Size(max= 64,message="学期名称不能超过64")
    @Column(name = "semester_name")
    @ApiModelProperty(name = "semesterName",notes = "学期名称")
    private String semesterName;

    @Column(name = "youth_committee_org_id")
    @ApiModelProperty(name = "youthCommitteeOrgId",notes = "团委组织ID")
    private BigInteger youthCommitteeOrgId;

    @Size(max= 255,message="团委组织名称不能超过255")
    @Column(name = "youth_committee_org_name")
    @ApiModelProperty(name = "youthCommitteeOrgName",notes = "团委组织名称")
    private String youthCommitteeOrgName;

    @Column(name = "current_school_id")
    @ApiModelProperty(name = "currentSchoolId",notes = "当前学校ID")
    private BigInteger currentSchoolId;

    @Size(max= 128,message="学校名称不能超过128")
    @Column(name = "current_school_name")
    @ApiModelProperty(name = "currentSchoolName",notes = "当前学校名称")
    private String currentSchoolName;

    @Column(name = "current_grade_id")
    @ApiModelProperty(name = "currentGradeId",notes = "当前年级ID")
    private BigInteger currentGradeId;

    @Size(max= 128,message="年级名称不能超过128")
    @Column(name = "current_grade_name")
    @ApiModelProperty(name = "currentGradeName",notes = "当前年级名称")
    private String currentGradeName;

    @Column(name = "current_class_id")
    @ApiModelProperty(name = "currentClassId",notes = "当前班级ID")
    private BigInteger currentClassId;

    @Size(max= 128,message="班级名称不能超过128")
    @Column(name = "current_class_name")
    @ApiModelProperty(name = "currentClassName",notes = "当前班级名称")
    private String currentClassName;

    @Column(name = "current_class_director_id")
    @ApiModelProperty(name = "currentClassDirectorId",notes = "当前班主任ID")
    private BigInteger currentClassDirectorId;

    @Size(max= 64,message="班主任名称不能超过64")
    @Column(name = "current_class_director_name")
    @ApiModelProperty(name = "currentClassDirectorName",notes = "当前班主任名称")
    private String currentClassDirectorName;

    @Column(name = "tutor_teacher_id")
    @ApiModelProperty(name = "tutorTeacherId",notes = "辅导老师ID")
    private BigInteger tutorTeacherId;

    @Size(max= 64,message="辅导老师名称不能超过64")
    @Column(name = "tutor_teacher_name")
    @ApiModelProperty(name = "tutorTeacherName",notes = "辅导老师名称")
    private String tutorTeacherName;

    @Column(name = "if_join_youth_Pioneers")
    @ApiModelProperty(name = "ifJoinYouthPioneers",notes = "是否入队: 0-否；1-是")
    private Integer ifJoinYouthPioneers;

    @Column(name = "join_young_pioneers_time")
    @ApiModelProperty(name = "joinYoungPioneersTime",notes = "入队时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date joinYoungPioneersTime;

    @Column(name = "teacher_score")
    @ApiModelProperty(name = "teacherScore",notes = "老师评分")
    private Integer teacherScore;

    @Column(name = "score_time")
    @ApiModelProperty(name = "scoreTime",notes = "评分时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date scoreTime;

    @Column(name = "if_join_youth_league")
    @ApiModelProperty(name = "ifJoinYouthLeague",notes = "是否入团: 0-否；1-是")
    private Integer ifJoinYouthLeague;

    @Column(name = "join_youth_league_time")
    @ApiModelProperty(name = "joinYouthLeagueTime",notes = "入团时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date joinYouthLeagueTime;

    @Column(name = "if_join_communist_party")
    @ApiModelProperty(name = "ifJoinCommunistParty",notes = "是否入党: 0-否；1-是")
    private Integer ifJoinCommunistParty;

    @Column(name = "join_communist_party_time")
    @ApiModelProperty(name = "joinCommunistPartyTime",notes = "入党时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date joinCommunistPartyTime;

    @Column(name = "del_flag")
    @ApiModelProperty(name = "delFlag",notes = "是否删除：0-未删除；1-已删除")
    private Integer delFlag;

    @Column(name = "created_time")
    @ApiModelProperty(name = "createdTime",notes = "创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createdTime;

    @Column(name = "created_id")
    @ApiModelProperty(name = "createdId",notes = "创建人ID")
    private BigInteger createdId;

    @Size(max= 128,message="创建人名称不能超过128")
    @Column(name = "created_name")
    @ApiModelProperty(name = "createdName",notes = "创建人名称")
    private String createdName;

    @Column(name = "last_updated_time")
    @ApiModelProperty(name = "lastUpdatedTime",notes = "最后更新时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastUpdatedTime;

    @Column(name = "last_updated_id")
    @ApiModelProperty(name = "lastUpdatedId",notes = "最后更新人ID")
    private BigInteger lastUpdatedId;

    @Size(max= 128,message="最后更新人名称不能超过128")
    @Column(name = "last_updated_name")
    @ApiModelProperty(name = "lastUpdatedName",notes = "最后更新人名称")
    private String lastUpdatedName;

}