
package com.crunii.micro.service.oneaffair.dto.oneaffair;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

@Entity
@Table(name = "YCQ_YJS_STUDENT_HONORARY")
@NamedQuery(name = "YcqYjsStudentHonorary.findAll", query = "SELECT t FROM YcqYjsStudentHonorary t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = {"id"})
@ApiModel(description = "学生荣誉表")
public class YcqYjsStudentHonorary implements Serializable {

    private static final long serialVersionUID = 1L;


    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @ApiModelProperty(name = "id", notes = "主键")
    @Column(name = "id")
    private BigInteger id;

    @Column(name = "student_id")
    @ApiModelProperty(name = "studentId", notes = "学生ID")
    private BigInteger studentId;

    @Size(max= 128,message="荣誉称号类型不能超过128")
    @Column(name = "honorary_title_type")
    @ApiModelProperty(name = "honoraryTitleType", notes = "荣誉称号类型")
    private String honoraryTitleType;

    @Size(max= 128,message="荣誉称号名称不能超过128")
    @Column(name = "honorary_title")
    @ApiModelProperty(name = "honoraryTitle", notes = "荣誉称号名称")
    private String honoraryTitle;

    @Size(max= 255,message="荣誉称号全称不能超过255")
    @Column(name = "honorary_full_title")
    @ApiModelProperty(name = "honoraryFullTitle", notes = "荣誉称号全称")
    private String honoraryFullTitle;

    @Size(max= 128,message="荣誉称号级别类型不能超过128")
    @Column(name = "honorary_level_type")
    @ApiModelProperty(name = "honoraryLevelType", notes = "荣誉称号级别类型：1-全国、2-市级、3-区级、4-校级")
    private String honoraryLevelType;

    @Size(max= 128,message="荣誉称号级别名称不能超过128")
    @Column(name = "honorary_level")
    @ApiModelProperty(name = "honoraryLevel", notes = "荣誉称号级别名称")
    private String honoraryLevel;

    @Column(name = "honorary_time")
    @ApiModelProperty(name = "honoraryTime", notes = "获取荣誉称号时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date honoraryTime;

    @Size(max= 64,message="学期名称不能超过64")
    @Column(name = "semester_name")
    @ApiModelProperty(name = "semesterName", notes = "学期名称")
    private String semesterName;

    @Size(max= 64,message="学期类型不能超过64")
    @Column(name = "semester_type")
    @ApiModelProperty(name = "semesterType", notes = "学期类型")
    private String semesterType;

    @Column(name = "school_id")
    @ApiModelProperty(name = "schoolId", notes = "学校ID")
    private BigInteger schoolId;

    @Size(max= 255,message="学校名称不能超过255")
    @Column(name = "school_name")
    @ApiModelProperty(name = "schoolName", notes = "学校名称")
    private String schoolName;

    @Column(name = "grade_id")
    @ApiModelProperty(name = "gradeId", notes = "年级ID")
    private BigInteger gradeId;

    @Size(max= 255,message="年级名称不能超过255")
    @Column(name = "grade_name")
    @ApiModelProperty(name = "gradeName", notes = "年级名称")
    private String gradeName;

    @Column(name = "class_id")
    @ApiModelProperty(name = "classId", notes = "班级ID")
    private BigInteger classId;

    @Size(max= 255,message="班级名称不能超过255")
    @Column(name = "class_name")
    @ApiModelProperty(name = "className", notes = "班级名称")
    private String className;

    @Column(name = "del_flag")
    @ApiModelProperty(name = "delFlag", notes = "是否删除：0-未删除；1-已删除")
    private Integer delFlag;

    @Column(name = "created_time")
    @ApiModelProperty(name = "createdTime", notes = "创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createdTime;

    @Column(name = "created_id")
    @ApiModelProperty(name = "createdId", notes = "创建人ID")
    private BigInteger createdId;

    @Size(max= 255,message="创建人名称不能超过255")
    @Column(name = "created_name")
    @ApiModelProperty(name = "createdName", notes = "创建人名称")
    private String createdName;

    @Column(name = "last_updated_time")
    @ApiModelProperty(name = "lastUpdatedTime", notes = "最后更新时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastUpdatedTime;

    @Size(max= 255,message="最后更新名称不能超过255")
    @Column(name = "last_updated_name")
    @ApiModelProperty(name = "lastUpdatedName", notes = "最后更新名称")
    private String lastUpdatedName;

    @Column(name = "last_updated_id")
    @ApiModelProperty(name = "lastUpdatedId", notes = "最后更新人ID")
    private BigInteger lastUpdatedId;

}