package com.crunii.micro.service.oneaffair.dto.oneaffair;

import com.crunii.micro.service.oneaffair.annotation.FiledDesensitization;
import com.crunii.micro.service.oneaffair.enums.MaskingTypeEnum;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

@Entity
@Table(name = "YCQ_YJS_TEACHER")
@NamedQuery(name = "YcqYjsTeacher.findAll", query = "SELECT t FROM YcqYjsTeacher t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = {"id"})
@ApiModel(description = "教师表")
public class YcqYjsTeacher implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @ApiModelProperty(name = "id", notes = "主键")
    @Column(name = "id")
    private BigInteger id;

    @Size(max= 32,message="教师编号不能超过32")
    @Column(name = "teacher_num")
    @ApiModelProperty(name = "teacherNum", notes = "教师编号")
    private String teacherNum;

    @Size(max= 32,message="教师工号不能超过32")
    @Column(name = "login_code")
    @ApiModelProperty(name = "loginCode", notes = "教师工号")
    private String loginCode;

    @Column(name = "role_id")
    @ApiModelProperty(name = "roleId", notes = "角色id", required = true)
    private Integer roleId;

    @Size(max= 64,message="角色名称不能超过64")
    @Column(name = "role_name")
    @ApiModelProperty(name = "roleId", notes = "角色名称", required = true)
    private String roleName;

    @Column(name = "user_id")
    @ApiModelProperty(name = "userId", notes = "用户ID")
    private BigInteger userId;

    @Size(max= 32,message="教师名称不能超过32")
    @Column(name = "teacher_name")
    @ApiModelProperty(name = "teacherName", notes = "教师名称")
    private String teacherName;

    @Column(name = "sex")
    @ApiModelProperty(name = "sex", notes = "性别: 0-女，1-男")
    private Integer sex;

    @Size(max= 18,message="身份证号不能超过18位")
    @Column(name = "identity_card_no")
    @FiledDesensitization(type = MaskingTypeEnum.ID_CARD)
    @ApiModelProperty(name = "identityCardNo", notes = "身份证号")
    private String identityCardNo;

    @Size(max= 11,message="手机号不能超过11位")
    @Column(name = "mobile")
    @FiledDesensitization(type = MaskingTypeEnum.PHONE)
    @ApiModelProperty(name = "mobile", notes = "手机号")
    private String mobile;

    @Size(max= 64,message="少先队职务类型不能超过64")
    @Column(name = "young_pioneer_position_type")
    @ApiModelProperty(name = "youngPioneerPositionType", notes = "少先队职务类型")
    private String youngPioneerPositionType;

    @Size(max= 64,message="少先队职务名称不能超过64")
    @Column(name = "young_pioneer_position")
    @ApiModelProperty(name = "youngPioneerPosition", notes = "少先队职务名称")
    private String youngPioneerPosition;

    @Size(max= 64,message="教师职务类型不能超过64")
    @Column(name = "teacher_position_type")
    @ApiModelProperty(name = "teacherPositionType", notes = "教师职务类型")
    private String teacherPositionType;

    @Size(max= 64,message="政治面貌类型不能超过64")
    @Column(name = "political_status_type")
    @ApiModelProperty(name = "politicalStatusType", notes = "政治面貌类型：群众/少先队/团队/党员群众")
    private String politicalStatusType;

    @Size(max= 64,message="政治面貌不能超过64")
    @Column(name = "political_status")
    @ApiModelProperty(name = "politicalStatus", notes = "政治面貌")
    private String politicalStatus;

    @Size(max= 64,message="教师职务不能超过64")
    @Column(name = "teacher_position")
    @ApiModelProperty(name = "teacherPosition", notes = "教师职务")
    private String teacherPosition;

    @Column(name = "youth_committee_org_id")
    @ApiModelProperty(name = "youthCommitteeOrgId",notes = "团委组织ID")
    private BigInteger youthCommitteeOrgId;

    @Size(max= 255,message="团委组织名称不能超过255")
    @Column(name = "youth_committee_org_name")
    @ApiModelProperty(name = "youthCommitteeOrgName",notes = "团委组织名称")
    private String youthCommitteeOrgName;

    @Column(name = "school_id")
    @ApiModelProperty(name = "schoolId", notes = "学校ID")
    private BigInteger schoolId;

    @Size(max= 128,message="学校名称不能超过128")
    @Column(name = "school_name")
    @ApiModelProperty(name = "schoolName", notes = "学校名称")
    private String schoolName;

    @Column(name = "del_flag")
    @ApiModelProperty(name = "delFlag", notes = "是否删除：0-未删除；1-已删除")
    private Integer delFlag;

    @Column(name = "created_time")
    @ApiModelProperty(name = "createdTime", notes = "创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createdTime;

    @Column(name = "created_id")
    @ApiModelProperty(name = "createdId", notes = "创建人ID")
    private BigInteger createdId;

    @Size(max= 128,message="创建人姓名不能超过128")
    @Column(name = "created_name")
    @ApiModelProperty(name = "createdName", notes = "创建人姓名")
    private String createdName;

    @Column(name = "last_updated_time")
    @ApiModelProperty(name = "lastUpdatedTime", notes = "最后更新时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastUpdatedTime;

    @Column(name = "last_updated_id")
    @ApiModelProperty(name = "lastUpdatedId", notes = "最后更新人ID")
    private BigInteger lastUpdatedId;

    @Size(max= 128,message="最后更新人名称不能超过128")
    @Column(name = "last_updated_name")
    @ApiModelProperty(name = "lastUpdatedName", notes = "最后更新人名称")
    private String lastUpdatedName;

}