package com.crunii.micro.service.oneaffair.dto.vo.goodstudent;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

@ApiModel(description = "优秀学生新增对象")
@ToString
@Data
public class GoodStudentCreateReq implements Serializable {

    private static final long serialVersionUID = 1L;

    @Size(max= 32,message="学生编号不能超过32")
    @ApiModelProperty(name = "studentNum", notes = "学生编号")
    private String studentNum;

    @Size(max= 64,message="学生名称不能超过64")
    @NotBlank(message = "学生名称必填")
    @ApiModelProperty(name = "studentName", notes = "学生名称", required = true)
    private String studentName;

    @NotNull(message = "性别必选")
    @ApiModelProperty(name = "sex", notes = "性别: 0-女，1-男", required = true)
    private Integer sex;

    @Size(max= 500,message="学生照片URL不能超过500")
    @ApiModelProperty(name = "studentPhotoUrl", notes = "学生照片URL")
    private String studentPhotoUrl;

    @Size(max= 64,message="学年类型不能超过64")
    @NotBlank(message = "学年必选")
    @ApiModelProperty(name = "schoolYearType", notes = "学年类型", required = true)
    private String schoolYearType;

    @Size(max= 64,message="学年不能超过64")
    @NotBlank(message = "学年必选")
    @ApiModelProperty(name = "schoolYear", notes = "学年", required = true)
    private String schoolYear;

    @Size(max= 64,message="政治面貌不能超过64")
    @NotBlank(message = "政治面貌必选")
    @ApiModelProperty(name = "politicalStatus", notes = "政治面貌", required = true)
    private String politicalStatus;

    @Size(max= 64,message="政治面貌类型不能超过64")
    @NotBlank(message = "政治面貌必选")
    @ApiModelProperty(name = "politicalStatusType", notes = "政治面貌类型：群众/少先队/团队/党员群众", required = true)
    private String politicalStatusType;

    @Size(max= 128,message="优秀称号不能超过128")
    @NotBlank(message = "优秀称号必选")
    @ApiModelProperty(name = "honoraryTitle", notes = "优秀称号", required = true)
    private String honoraryTitle;

    @Size(max= 64,message="优秀称号类型不能超过64")
    @NotBlank(message = "优秀称号必选")
    @ApiModelProperty(name = "honoraryTitleType", notes = "优秀称号类型", required = true)
    private String honoraryTitleType;

    @NotNull(message = "学校必选")
    @ApiModelProperty(name = "schoolId", notes = "学校ID", required = true)
    private BigInteger schoolId;

    @Size(max= 128,message="学校名称不能超过128")
    @NotBlank(message = "学校必选")
    @ApiModelProperty(name = "schoolName", notes = "学校名称", required = true)
    private String schoolName;

    @NotNull(message = "年级必选")
    @ApiModelProperty(name = "gradeId", notes = "年级ID", required = true)
    private BigInteger gradeId;

    @Size(max= 128,message="年级名称不能超过128")
    @NotBlank(message = "年级必选")
    @ApiModelProperty(name = "gradeName", notes = "年级名称", required = true)
    private String gradeName;

    @NotNull(message = "班级必选")
    @ApiModelProperty(name = "classId", notes = "班级ID", required = true)
    private BigInteger classId;

    @Size(max= 128,message="班级名称不能超过128")
    @NotBlank(message = "班级必选")
    @ApiModelProperty(name = "className", notes = "班级名称", required = true)
    private String className;

    @Size(max= 900,message="简介不能超过900")
    @ApiModelProperty(name = "briefIntroduction", notes = "简介")
    private String briefIntroduction;

    @ApiModelProperty(name = "showFlag", notes = "是否展示：0-否；1-是")
    private Integer showFlag;

    @ApiModelProperty(name = "delFlag", notes = "是否删除：0-未删除；1-已删除")
    private Integer delFlag;

    @ApiModelProperty(name = "createdTime", notes = "创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createdTime;

    @ApiModelProperty(name = "createdId", notes = "创建人ID")
    private BigInteger createdId;

    @Size(max= 128,message="创建人姓名不能超过128")
    @ApiModelProperty(name = "createdName", notes = "创建人姓名")
    private String createdName;

}
