package com.crunii.micro.service.oneaffair.dto.vo.student;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

@ApiModel(value = "StudentUpdateReq", description = "学生更新对象")
@ToString
@Data
public class StudentUpdateReq implements Serializable {

    private static final long serialVersionUID = 1L;

    @NotNull(message = "学生必选")
    @ApiModelProperty(name = "id", notes = "主键")
    private BigInteger id;

    @Size(max= 64,message="学生名称不能超过64")
    @NotBlank(message = "学生名称必填")
    @ApiModelProperty(name = "studentName", notes = "学生名称")
    private String studentName;

    @ApiModelProperty(name = "mobile", notes = "手机号")
    private String mobile;

    @NotBlank(message = "身份证号必填")
    @ApiModelProperty(name = "identityCardNo", notes = "身份证号")
    private String identityCardNo;

    @NotNull(message = "学生性别必选")
    @ApiModelProperty(name = "sex", notes = "性别: 0-女，1-男")
    private Integer sex;

    @NotNull(message = "学校必选")
    @ApiModelProperty(name = "currentSchoolId", notes = "当前学校ID")
    private BigInteger currentSchoolId;

    @Size(max= 128,message="学校名称不能超过128")
    @NotBlank(message = "学校必选")
    @ApiModelProperty(name = "currentSchoolName", notes = "当前学校名称")
    private String currentSchoolName;

    @NotNull(message = "年级必选")
    @ApiModelProperty(name = "currentGradeId", notes = "当前年级ID")
    private BigInteger currentGradeId;

    @Size(max= 128,message="年级名称不能超过128")
    @NotBlank(message = "年级必选")
    @ApiModelProperty(name = "currentGradeName", notes = "当前年级名称")
    private String currentGradeName;

    @NotNull(message = "班级必选")
    @ApiModelProperty(name = "currentClassId", notes = "当前班级ID")
    private BigInteger currentClassId;

    @Size(max= 128,message="班级名称不能超过128")
    @NotBlank(message = "班级必选")
    @ApiModelProperty(name = "currentClassName", notes = "当前班级名称")
    private String currentClassName;

    @Size(max= 128,message="中队名称不能超过128")
    @ApiModelProperty(name = "middleTeamName",notes = "中队名称")
    private String middleTeamName;

    @ApiModelProperty(name = "currentClassDirectorId", notes = "当前班主任ID")
    private BigInteger currentClassDirectorId;

    @Size(max= 64,message="班主任名称不能超过64")
    @ApiModelProperty(name = "currentClassDirectorName", notes = "当前班主任名称")
    private String currentClassDirectorName;

    @ApiModelProperty(name = "tutorTeacherId",notes = "辅导老师ID")
    private BigInteger tutorTeacherId;

    @Size(max= 64,message="辅导老师名称不能超过64")
    @ApiModelProperty(name = "tutorTeacherName",notes = "辅导老师名称")
    private String tutorTeacherName;

    @Size(max= 64,message="学期类型不能超过64")
    @NotNull(message = "学期必选")
    @ApiModelProperty(name = "semesterType", notes = "学期类型")
    private String semesterType;

    @Size(max= 64,message="学期名称不能超过64")
    @NotNull(message = "学期必选")
    @ApiModelProperty(name = "semesterName", notes = "学期名称")
    private String semesterName;

    @ApiModelProperty(name = "youthCommitteeOrgId", notes = "团委组织ID")
    private BigInteger youthCommitteeOrgId;

    @Size(max= 255,message="团委组织名称不能超过255")
    @ApiModelProperty(name = "youthCommitteeOrgName", notes = "团委组织名称")
    private String youthCommitteeOrgName;

    @ApiModelProperty(name = "lastUpdatedTime", notes = "最后更新时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastUpdatedTime;

    @ApiModelProperty(name = "lastUpdatedId", notes = "最后更新人ID")
    private BigInteger lastUpdatedId;

    @Size(max= 128,message="创建人名称不能超过128")
    @ApiModelProperty(name = "lastUpdatedName",notes = "最后更新人名称")
    private String lastUpdatedName;

}
