package com.crunii.micro.service.oneaffair.dto.vo.studentTest;

import com.crunii.micro.service.oneaffair.dto.vo.studentQuestion.StudentQuestionCreateReq;
import com.fasterxml.jackson.annotation.JsonFormat;
import dm.jdbc.util.StringUtil;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;
import java.util.List;

@ApiModel(value = "StudentTestCreateReq", description = "学生答卷新增对象")
@ToString
@Data
public class StudentTestCreateReq implements Serializable {

    private static final long serialVersionUID = 1L;

    @NotNull(message = "学生不能为空")
    @ApiModelProperty(name = "studentId", notes = "学生ID")
    private BigInteger studentId;

    @NotNull(message = "学生课程不能为空")
    @ApiModelProperty(name = "studentCourseId", notes = "学生课程ID", required = true)
    private BigInteger studentCourseId;

    @NotNull(message = "试卷不能为空")
    @ApiModelProperty(name = "paperId", notes = "试卷ID", required = true)
    private BigInteger paperId;

    @Size(max= 255,message="试卷名称不能超过255")
    @ApiModelProperty(name = "paperName", notes = "试卷名称")
    private String paperName;

    public void setPaperName(String paperName){
        this.paperName = StringUtil.isEmpty(paperName) ? "练习题" : paperName;
    }

    @ApiModelProperty(name = "ifShowScore", notes = "是否显示分数：0-否 1是")
    private Integer ifShowScore;

    @ApiModelProperty(name = "ifExam", notes = "是否是考试：0-否 1-是")
    private Integer ifExam;

    @ApiModelProperty(name = "testScore", notes = "试卷总分", required = true)
    private Integer testScore = 0;

    public void setTestScore(Integer testScore){
        this.testScore = testScore == null ? 0 : testScore;
    }

    @ApiModelProperty(name = "passScore", notes = "合格分", required = true)
    private Integer passScore = 0;

    public void setPassScore(Integer passScore){
        this.passScore = passScore == null ? 0 : passScore;
    }

    @ApiModelProperty(name = "answerDuration", notes = "答题时长：秒", required = true)
    private Integer answerDuration;

    public void setAnswerDuration(Integer answerDuration){
        this.answerDuration = answerDuration == null ? 0 : answerDuration;
    }

    @NotNull(message = "题目总数不能为空")
    @ApiModelProperty(name = "questionCount", notes = "题目总数", required = true)
    private Integer questionCount;

    @NotNull(message = "学生答题不能为空")
    @Size(min = 1, message = "学生答题不能为空")
    @ApiModelProperty(name = "studentQuestionCreateReqList", notes = "学生答题列表", required = true)
    private List<StudentQuestionCreateReq> studentQuestionCreateReqList;

    @ApiModelProperty(name = "singleScore", notes = "单选题分数", required = true)
    private Integer singleScore = 0;

    public void setSingleScore(Integer singleScore){
        this.singleScore = singleScore == null ? 0 : singleScore;
    }

    @ApiModelProperty(name = "multiScore", notes = "多选题分数", required = true)
    private Integer multiScore = 0;

    public void setMultiScore(Integer multiScore){
        this.multiScore = multiScore == null ? 0 : multiScore;
    }

    @ApiModelProperty(name = "judgeScore", notes = "判断题分数", required = true)
    private Integer judgeScore = 0;

    public void setJudgeScore(Integer judgeScore){
        this.judgeScore = judgeScore == null ? 0 : judgeScore;
    }

    @ApiModelProperty(name = "allowTimes", notes = "可考次数", required = true)
    private Integer allowTimes;

    @ApiModelProperty(name = "answerScore", notes = "答题得分")
    private Integer answerScore;

    @ApiModelProperty(name = "ifPass", notes = "是否合格：0-不合格；1-合格")
    private Integer ifPass;

    @ApiModelProperty(name = "ifAnswer", notes = "是否提交答案：0-否；1-是")
    private Integer ifAnswer;

    @ApiModelProperty(name = "rightQuestionCount", notes = "答对题目数")
    private Integer rightQuestionCount;

    @ApiModelProperty(name = "errorQuestionCount", notes = "答错题目数")
    private Integer errorQuestionCount;

    @ApiModelProperty(name = "delFlag", notes = "是否删除：0-未删除；1-已删除")
    private Integer delFlag;

    @ApiModelProperty(name = "createdTime", notes = "创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createdTime;

    @ApiModelProperty(name = "createdId", notes = "创建人ID")
    private BigInteger createdId;

    @Size(max= 255,message="创建人名称不能超过255")
    @ApiModelProperty(name = "createdName", notes = "创建人名称")
    private String createdName;

}
