package com.crunii.micro.service.oneaffair.dto.vo.studentTest;

import com.crunii.micro.service.oneaffair.dto.vo.studentQuestion.StudentQuestionCreateReq;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;
import java.util.List;

@ApiModel(value = "StudentTestUpdateReq", description = "学生答卷修改对象")
@ToString
@Data
public class StudentTestUpdateReq implements Serializable {

    private static final long serialVersionUID = 1L;

    @NotNull(message = "学生答卷不能为空")
    @ApiModelProperty(name = "id", notes = "学生答卷ID")
    private BigInteger id;

    @NotNull(message = "学生不能为空")
    @ApiModelProperty(name = "studentId", notes = "学生ID")
    private BigInteger studentId;

    @ApiModelProperty(name = "ifShowScore", notes = "是否显示分数：0-否 1是")
    private Integer ifShowScore;

    @ApiModelProperty(name = "ifExam", notes = "是否是考试：0-否 1-是")
    private Integer ifExam;

    @ApiModelProperty(name = "testScore", notes = "试卷总分", required = true)
    private Integer testScore = 0;

    public void setTestScore(Integer testScore) {
        this.testScore = testScore == null ? 0 : testScore;
    }

    @ApiModelProperty(name = "passScore", notes = "合格分", required = true)
    private Integer passScore = 0;

    public void setPassScore(Integer passScore) {
        this.passScore = passScore == null ? 0 : passScore;
    }

    @ApiModelProperty(name = "answerDuration", notes = "答题时长：秒", required = true)
    private Integer answerDuration;

    public void setAnswerDuration(Integer answerDuration) {
        this.answerDuration = answerDuration == null ? 0 : answerDuration;
    }

    @NotNull(message = "题目总数不能为空")
    @ApiModelProperty(name = "questionCount", notes = "题目总数", required = true)
    private Integer questionCount;

    @NotNull(message = "学生答题不能为空")
    @Size(min = 1, message = "学生答题不能为空")
    @ApiModelProperty(name = "studentQuestionCreateReqList", notes = "学生答题列表", required = true)
    private List<StudentQuestionCreateReq> studentQuestionCreateReqList;

    @ApiModelProperty(name = "singleScore", notes = "单选题分数", required = true)
    private Integer singleScore = 0;

    public void setSingleScore(Integer singleScore) {
        this.singleScore = singleScore == null ? 0 : singleScore;
    }

    @ApiModelProperty(name = "multiScore", notes = "多选题分数", required = true)
    private Integer multiScore = 0;

    public void setMultiScore(Integer multiScore) {
        this.multiScore = multiScore == null ? 0 : multiScore;
    }

    @ApiModelProperty(name = "judgeScore", notes = "判断题分数", required = true)
    private Integer judgeScore = 0;

    public void setJudgeScore(Integer judgeScore) {
        this.judgeScore = judgeScore == null ? 0 : judgeScore;
    }

    @ApiModelProperty(name = "answerScore", notes = "答题得分")
    private Integer answerScore;

    @ApiModelProperty(name = "ifPass", notes = "是否合格：0-不合格；1-合格")
    private Integer ifPass;

    @ApiModelProperty(name = "ifAnswer", notes = "是否提交答案：0-否；1-是")
    private Integer ifAnswer;

    @ApiModelProperty(name = "rightQuestionCount", notes = "答对题目数")
    private Integer rightQuestionCount;

    @ApiModelProperty(name = "errorQuestionCount", notes = "答错题目数")
    private Integer errorQuestionCount;

    @ApiModelProperty(name = "lastUpdatedTime", notes = "最后更新时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastUpdatedTime;

    @Size(max= 255,message="最后更新名称不能超过255")
    @ApiModelProperty(name = "lastUpdatedName", notes = "最后更新名称")
    private String lastUpdatedName;

    @ApiModelProperty(name = "lastUpdatedId", notes = "最后更新人ID")
    private BigInteger lastUpdatedId;


}
