package com.crunii.micro.service.oneaffair.dto.vo.teacher;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;
import java.util.List;

@ApiModel(value = "TeacherInfoUpdateReq", description = "教师编辑对象")
@ToString
@Data
public class TeacherInfoUpdateReq implements Serializable {

    private static final long serialVersionUID = 1L;

    @NotNull(message = "教师必选")
    @ApiModelProperty(name = "id", notes = "主键", required = true)
    private BigInteger id;

    @Size(max= 32,message="教师工号不能超过32")
    @ApiModelProperty(name = "loginCode", notes = "教师工号", required = true)
    private String loginCode;

    @ApiModelProperty(name = "userId", notes = "用户ID")
    private BigInteger userId;

    @NotNull(message = "教师角色必选")
    @ApiModelProperty(name = "roleId", notes = "角色id", required = true)
    private Integer roleId;

    @Size(max= 64,message="角色名称不能超过64")
    @NotNull(message = "教师角色必选")
    @ApiModelProperty(name = "roleId", notes = "角色名称", required = true)
    private String roleName;

    @Size(max= 32,message="教师名称不能超过32")
    @NotBlank(message = "教师名称必填")
    @ApiModelProperty(name = "teacherName", notes = "教师名称", required = true)
    private String teacherName;

    @NotNull(message = "性别必选")
    @ApiModelProperty(name = "sex", notes = "性别: 0-女，1-男", required = true)
    private Integer sex;

    @NotBlank(message = "身份证号必填")
    @ApiModelProperty(name = "identityCardNo", notes = "身份证号", required = true)
    private String identityCardNo;

    @NotBlank(message = "手机号必填")
    @ApiModelProperty(name = "mobile", notes = "手机号", required = true)
    private String mobile;

    @Size(max= 64,message="教师职务不能超过64")
    @NotBlank(message = "教师职务必选")
    @ApiModelProperty(name = "teacherPosition", notes = "教师职务", required = true)
    private String teacherPosition;

    @Size(max= 64,message="教师职务类型不能超过64")
    @NotBlank(message = "教师职务必选")
    @ApiModelProperty(name = "teacherPositionType", notes = "教师职务类型", required = true)
    private String teacherPositionType;

    @Size(max= 64,message="政治面貌类型不能超过64")
    @NotBlank(message = "政治面貌必选")
    @ApiModelProperty(name = "politicalStatusType", notes = "政治面貌类型", required = true)
    private String politicalStatusType;

    @Size(max= 64,message="政治面貌不能超过64")
    @NotBlank(message = "政治面貌必选")
    @ApiModelProperty(name = "politicalStatus", notes = "政治面貌：群众/少先队/团队/党员群众", required = true)
    private String politicalStatus;

    @Size(max= 64,message="少先队职务类型不能超过64")
    @ApiModelProperty(name = "youngPioneerPositionType", notes = "少先队职务类型")
    private String youngPioneerPositionType;

    @Size(max= 64,message="少先队职务名称不能超过64")
    @ApiModelProperty(name = "youngPioneerPosition", notes = "少先队职务名称")
    private String youngPioneerPosition;

    @ApiModelProperty(name = "youthCommitteeOrgId", notes = "团委组织ID")
    private BigInteger youthCommitteeOrgId;

    @Size(max= 255,message="团委组织名称不能超过255")
    @ApiModelProperty(name = "youthCommitteeOrgName", notes = "团委组织名称")
    private String youthCommitteeOrgName;

    @NotNull(message = "schoolId")
    @ApiModelProperty(name = "schoolId", notes = "学校ID", required = true)
    private BigInteger schoolId;

    @Size(max= 128,message="学校名称不能超过128")
    @NotNull(message = "所属单位必选")
    @ApiModelProperty(name = "schoolName", notes = "学校名称", required = true)
    private String schoolName;

    @NotNull(message = "负责年级/班级必选")
    @Size(min = 1, message = "负责年级/班级必选")
    @ApiModelProperty(name = "gradeInfoList", notes = "负责年级/班级树状图", required = true)
    private List<GradeInfo> gradeInfoList;

    @ApiModelProperty(name = "lastUpdatedTime", notes = "最后更新时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastUpdatedTime;

    @ApiModelProperty(name = "lastUpdatedId", notes = "最后更新人ID")
    private BigInteger lastUpdatedId;

    @Size(max= 128,message="最后更新人名称不能超过128")
    @ApiModelProperty(name = "lastUpdatedName", notes = "最后更新人名称")
    private String lastUpdatedName;

}
