package com.crunii.micro.service.oneaffair.dto.vo.teacher;

import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsTeacher;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsTeacherBindingClass;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import org.springframework.util.CollectionUtils;

import java.io.Serializable;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@ApiModel(value = "TeacherInfoVo", description = "教师查询返回对象")
@ToString
@Data
public class TeacherInfoVo extends YcqYjsTeacher implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(name = "teacherBindingClassList", notes = "负责班级列表")
    private List<YcqYjsTeacherBindingClass> teacherBindingClassList;

    @ApiModelProperty(name = "gradeInfoList", notes = "负责年级/班级树状图")
    private List<GradeInfo> gradeInfoList;

    @ApiModelProperty(name = "gradeClassStrList", notes = "负责班级列表")
    private List<String> gradeClassStrList;

    public void setTeacherBindingClassList(List<YcqYjsTeacherBindingClass> teacherBindingClassList) {
        this.teacherBindingClassList = teacherBindingClassList;
        // 处理负责班级展示列表
        if (!CollectionUtils.isEmpty(teacherBindingClassList)) {
            // 根据年级和班级排序
            teacherBindingClassList = teacherBindingClassList.stream().sorted(Comparator.comparing(YcqYjsTeacherBindingClass::getGradeId).thenComparing(YcqYjsTeacherBindingClass::getClassId)).collect(Collectors.toList());
            // 根据年级分组
            Map<BigInteger, List<YcqYjsTeacherBindingClass>> map = teacherBindingClassList.stream().collect(Collectors.groupingBy(YcqYjsTeacherBindingClass::getGradeId));
            List<GradeInfo> gradeInfoList = new ArrayList<>();
            List<String> gradeClassStrList = new ArrayList<>();
            for (Map.Entry<BigInteger, List<YcqYjsTeacherBindingClass>> entry : map.entrySet()) {
                List<YcqYjsTeacherBindingClass> classList = entry.getValue();
                if (!CollectionUtils.isEmpty(classList)) {
                    StringBuilder gradeClassStr = new StringBuilder();
                    GradeInfo gradeInfo = BeanCopyUtil.convert(classList.get(0), GradeInfo.class);
                    gradeClassStr.append(gradeInfo.getGradeName());
                    gradeClassStr.append("：");
                    if (gradeInfo.getIfAll() != null && gradeInfo.getIfAll().intValue() == 1) {
                        gradeClassStr.append("全部");
                    }
                    List<ClassInfo> classInfoList = new ArrayList<>();
                    for (int i = 0; i < classList.size(); i++) {
                        YcqYjsTeacherBindingClass item = classList.get(i);
                        if (gradeInfo.getIfAll() == null || gradeInfo.getIfAll().intValue() != 1) {
                            if (i != 0) {
                                gradeClassStr.append("/");
                            }
                            gradeClassStr.append(item.getClassName());
                        }
                        classInfoList.add(BeanCopyUtil.convert(item, ClassInfo.class));
                    }
                    gradeInfo.setClassInfoList(classInfoList);
                    gradeInfoList.add(gradeInfo);
                    gradeClassStrList.add(gradeClassStr.toString());
                }
            }
            if (!CollectionUtils.isEmpty(gradeInfoList)) {
                this.gradeInfoList = gradeInfoList;
            }
            if(!CollectionUtils.isEmpty(gradeClassStrList)){
                this.gradeClassStrList = gradeClassStrList;
            }
        }
    }

}
