package com.crunii.micro.service.oneaffair.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveVo;
import com.crunii.micro.service.oneaffair.dto.vo.medal.ActivityReq;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalActivityInfoVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * 奖章活动相关表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsActivityJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String PAGE_SQL = "SELECT a.id, a.id AS match_code, a.match_name, category_name, a.publish_org_name AS org_name, a.registeration_start, a.registeration_audit_flag, b.id as medal_activity_id, b.status " +
            "FROM mth_match_info a " +
            "LEFT JOIN ycq_yjs_medal_activity b ON a.id = b.activity_id AND b.del_flag = 0 AND b.medal_id = ? " +
            "WHERE a.del_flag = 0 and a.id IN (SELECT DISTINCT id FROM mth_match_info) " ;

    private static final String PAGE_COUNT_SQL = "select count(*) " +
            "FROM mth_match_info a " +
            "LEFT JOIN ycq_yjs_medal_activity b ON a.id = b.activity_id AND b.del_flag = 0 AND b.medal_id = ? " +
            "WHERE a.del_flag = 0 and a.id IN (SELECT DISTINCT id FROM mth_match_info) ";

    /*private static final String MATCH_ACTIVITY_STUDENT_SQL = "SELECT id, id AS match_code, match_name, category_name, publish_org_name AS org_name, registeration_start, status, registeration_audit_flag " +
            "FROM mth_match_info WHERE del_flag = 0 " +
            "AND id IN ( " +
            "SELECT a.activity_id FROM YCQ_YJS_MEDAL_ACTIVITY a " +
            "INNER JOIN YCQ_YJS_ARCHIVE b ON a.activity_id = b.event_id AND b.activity_type = '2'  AND b.del_flag = 0 AND b.student_id = ? " +
            "WHERE a.del_flag = 0 AND a.app_code = 'matchManagement' AND a.medal_id = ? ) ";*/

    private static final String ACTIVITY_STUDENT_SQL = "SELECT * FROM YCQ_YJS_ARCHIVE " +
            "WHERE del_flag = 0 AND student_id = ? " +
            "AND ((activity_type = 2 AND event_id IN (SELECT activity_id FROM YCQ_YJS_MEDAL_ACTIVITY WHERE del_flag = 0 AND medal_id = ? AND app_code = 'matchManagement' )) )"; //后续加or增加更多活动类型

    /**
     * 赛事活动相关(未与奖章关联)
     */
    public PageRsp<MedalActivityInfoVo> matchManagementPageSearchNo(PageReq<ActivityReq> req){
        log.info("YcqYjsMedalJdbcDao.matchManagementPageSearchNo.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        sql.append("AND b.id IS NULL ");
        countSql.append("AND b.id IS NULL ");
        param.add(req.getQuery().getMedalId());
        // 比赛名称
        if (StringUtils.isNotBlank(req.getQuery().getMatchName())) {
            sql.append(" and a.match_name like ? ");
            countSql.append(" and a.match_name like ? ");
            param.add("%" + req.getQuery().getMatchName() + "%");
        }
        sql.append(" order by a.create_time desc ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<MedalActivityInfoVo> result;
        // 校验是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(MedalActivityInfoVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<MedalActivityInfoVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    /**
     * 赛事活动相关(已与奖章关联)
     */
    public PageRsp<MedalActivityInfoVo> matchManagementPageSearch(PageReq<ActivityReq> req){
        log.info("YcqYjsMedalJdbcDao.matchManagementPageSearch.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        sql.append("AND b.id IS NOT NULL ");
        countSql.append("AND b.id IS NOT NULL ");
        param.add(req.getQuery().getMedalId());
        // 比赛名称
        if (StringUtils.isNotBlank(req.getQuery().getMatchName())) {
            sql.append(" and a.match_name like ? ");
            countSql.append(" and a.match_name like ? ");
            param.add("%" + req.getQuery().getMatchName() + "%");
        }
        sql.append(" order by a.create_time desc ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<MedalActivityInfoVo> result;
        // 校验是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(MedalActivityInfoVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<MedalActivityInfoVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    public List<MedalActivityInfoVo> selectList(BigInteger medalId){
        // 暂时只查赛事活动
        log.info("YcqYjsActivityJdbcDao.selectList.param：{}", medalId);
        StringBuilder sql = new StringBuilder(PAGE_SQL);
        sql.append("AND b.id IS NOT NULL ");
        List<Object> param = new ArrayList<>();
        param.add(medalId);
        sql.append(" order by a.create_time desc ");
        return jt.queryForBeans(MedalActivityInfoVo.class, sql.toString(), param.toArray());
    }

    public List<ArchiveVo> selectStudentActivityList(String studentId, BigInteger medalId){
        // 暂时只查赛事活动
        StringBuilder sql = new StringBuilder(ACTIVITY_STUDENT_SQL);
        List<Object> param = new ArrayList<>();
        param.add(studentId);
        param.add(medalId);
        sql.append(" order by create_time desc ");
        return jt.queryForBeans(ArchiveVo.class, sql.toString(), param.toArray());
    }
}