package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsArchive;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.AppElseStudentArchivesVo;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.AppStudentArchivesReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveSelectReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveVo;
import com.crunii.micro.service.oneaffair.dto.vo.honor.MthMatchVo;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import com.google.common.base.Joiner;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 档案表(YCQ_YJS_ARCHIVE)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsArchiveJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String ID_SQL = "select * from ycq_yjs_archive where del_flag = 0 and id = ?";

    private static final String SELECT_SQL = "select * from ycq_yjs_archive where del_flag = 0";

    private static final String PAGE_COUNT_SQL = "select count(*) from ycq_yjs_archive where del_flag = 0";

    public ArchiveVo selectOneById(String id) {
        log.info("YcqYjsStudentJdbcDao.selectOneById.param：{}", id);
        if (id == null) {
            return null;
        }
        return jt.queryForBean(ArchiveVo.class, ID_SQL, id);
    }

    public List<ArchiveVo> selectList(ArchiveSelectReq req) {
        StringBuilder sql = new StringBuilder(SELECT_SQL);
        List<Object> param = new ArrayList<>();

        // 主键ID
        if (StringUtils.isNotBlank(req.getId())) {
            sql.append(" and id = ? ");
            param.add(req.getId());
        }
        // 学生ID
        if (StringUtils.isNotBlank(req.getStudentId())) {
            sql.append(" and student_id = ? ");
            param.add(req.getStudentId());
        }
        // 学生ID列表
        if (CollectionUtil.isNotEmpty(req.getStudentIdList())) {
            String itemSql = " and student_id in " + CommonUtil.fromStrListToStr1(req.getStudentIdList());
            sql.append(itemSql);
        }
        // 姓名
        if (StringUtils.isNotBlank(req.getName())) {
            sql.append(" and name like ? ");
            param.add("%" + req.getName() + "%");
        }
        // 应用代码
        if (StringUtils.isNotBlank(req.getAppCode())) {
            sql.append(" and app_code = ? ");
            param.add(req.getAppCode());
        }
        // 活动类型
        if (StringUtils.isNotBlank(req.getActivityType())) {
            sql.append(" and activity_type = ? ");
            param.add(req.getActivityType());
        }
        // 创建时间
        if (req.getCreateTime() != null) {
            sql.append(" and created_time = ? ");
            param.add(req.getCreateTime());
        }
        // 描述
        if (StringUtils.isNotBlank(req.getDescription())) {
            sql.append(" and description like ? ");
            param.add("%" + req.getDescription() + "%");
        }
        // 事项id
        if (StringUtils.isNotBlank(req.getDetail())) {
            sql.append(" and detail = ? ");
            param.add(req.getDetail());
        }
        if (StringUtils.isNotBlank(req.getEventId())) {
            sql.append(" and event_id = ? ");
            param.add(req.getEventId());
        }
        if (CollectionUtil.isNotEmpty(req.getEventIdList())) {
            String itemSql = " and event_id in " + CommonUtil.fromStrListToStr1(req.getEventIdList());
            sql.append(itemSql);
        }
        if (req.getEventType() != null) {
            sql.append(" and event_type = ? ");
            param.add(req.getEventType());
        }
        // 类型
        if (StringUtils.isNotBlank(req.getPoliticalType())) {
            sql.append(" and political_type = ? ");
            param.add(req.getPoliticalType());
        }
        if (StringUtils.isNotBlank(req.getExtensionType())) {
            sql.append(" and extension_type = ? ");
            param.add(req.getExtensionType());
        }
        if (StringUtils.isNotBlank(req.getDetailTopic())) {
            sql.append(" and detail_topic = ? ");
            param.add(req.getDetailTopic());
        }if (StringUtils.isNotBlank(req.getDetailImg())) {
            sql.append(" and detail_img = ? ");
            param.add(req.getDetailImg());
        }if (StringUtils.isNotBlank(req.getDetailIntroduction())) {
            sql.append(" and detail_introduction = ? ");
            param.add(req.getDetailIntroduction());
        }if (StringUtils.isNotBlank(req.getExtensionJson())) {
            sql.append(" and extension_json = ? ");
            param.add(req.getExtensionJson());
        }
        if (StringUtils.isNotBlank(req.getUserId())) {
            sql.append(" and user_id = ? ");
            param.add(req.getUserId());
        }
        if (StringUtils.isNotBlank(req.getClassName())) {
            sql.append(" and class_name = ? ");
            param.add(req.getClassName());
        }
        if (StringUtils.isNotBlank(req.getUserName())) {
            sql.append(" and user_name = ? ");
            param.add(req.getUserName());
        }
        if (req.getParticipateStartTime() != null) {
            sql.append(" and participate_time >= ? ");
            param.add(req.getParticipateStartTime());
        }
        if (req.getParticipateEndTime() != null) {
            sql.append(" and participate_time <= ? ");
            param.add(req.getParticipateEndTime());
        }
        sql.append(" order by participate_time desc");
        return jt.queryForBeans(ArchiveVo.class, sql.toString(), param.toArray());
    }


    //分页查询
    public PageRsp<ArchiveVo> pageSearchStudentInfo(PageReq<ArchiveSelectReq> req) {
        StringBuilder sql = new StringBuilder(SELECT_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req.getQuery() != null) {
            // ID
            if (StringUtils.isNotBlank(req.getQuery().getId())) {
                sql.append(" and id = ? ");
                countSql.append(" and id = ? ");
                param.add(req.getQuery().getId());
            }

            // 学生ID
            if (StringUtils.isNotBlank(req.getQuery().getStudentId())) {
                sql.append(" and student_id = ? ");
                countSql.append(" and student_id = ? ");
                param.add(req.getQuery().getStudentId());
            }

            // 姓名
            if (StringUtils.isNotBlank(req.getQuery().getName())) {
                sql.append(" and name like ? ");
                countSql.append(" and name like ? ");
                param.add("%" + req.getQuery().getName() + "%");
            }

            // 应用代码
            if (StringUtils.isNotBlank(req.getQuery().getAppCode())) {
                sql.append(" and app_code = ? ");
                countSql.append(" and app_code = ? ");
                param.add(req.getQuery().getAppCode());
            }

            // 活动类型
            if (StringUtils.isNotBlank(req.getQuery().getActivityType())) {
                sql.append(" and activity_type = ? ");
                countSql.append(" and activity_type = ? ");
                param.add(req.getQuery().getActivityType());
            }
            // 描述
            if (StringUtils.isNotBlank(req.getQuery().getDescription())) {
                sql.append(" and description like ? ");
                countSql.append(" and description like ? ");
                param.add("%" + req.getQuery().getDescription() + "%");
            }

            // 事项ID
            if (StringUtils.isNotBlank(req.getQuery().getEventId())) {
                sql.append(" and event_id = ? ");
                countSql.append(" and event_id = ? ");
                param.add(req.getQuery().getEventId());
            }

            if (StringUtils.isNotBlank(req.getQuery().getExtensionType())) {
                sql.append(" and extension_type = ? ");
                countSql.append(" and extension_type = ? ");
                param.add(req.getQuery().getExtensionType());
            }
            if (StringUtils.isNotBlank(req.getQuery().getDetailTopic())) {
                sql.append(" and detail_topic = ? ");
                countSql.append(" and detail_topic = ? ");
                param.add(req.getQuery().getDetailTopic());
            }if (StringUtils.isNotBlank(req.getQuery().getDetailImg())) {
                sql.append(" and detail_img = ? ");
                countSql.append(" and detail_img = ? ");
                param.add(req.getQuery().getDetailImg());
            }if (StringUtils.isNotBlank(req.getQuery().getDetailIntroduction())) {
                sql.append(" and detail_introduction = ? ");
                countSql.append(" and detail_introduction = ? ");
                param.add(req.getQuery().getDetailIntroduction());
            }if (StringUtils.isNotBlank(req.getQuery().getExtensionJson())) {
                sql.append(" and extension_json = ? ");
                countSql.append(" and extension_json = ? ");
                param.add(req.getQuery().getExtensionJson());
            }

            if (StringUtils.isNotBlank(req.getQuery().getUserId())) {
                sql.append(" and user_id = ? ");
                countSql.append(" and user_id = ? ");
                param.add(req.getQuery().getUserId());
            }
            if (StringUtils.isNotBlank(req.getQuery().getClassName())) {
                sql.append(" and class_name = ? ");
                countSql.append(" and class_name = ? ");
                param.add(req.getQuery().getClassName());
            }
            if (StringUtils.isNotBlank(req.getQuery().getUserName())) {
                sql.append(" and user_name = ? ");
                countSql.append(" and user_name = ? ");
                param.add(req.getQuery().getUserName());
            }

            // 类型
            if (StringUtils.isNotBlank(req.getQuery().getPoliticalType())) {
                sql.append(" and political_type = ? ");
                countSql.append(" and political_type = ? ");
                param.add(req.getQuery().getPoliticalType());
            }
            if (req.getQuery().getParticipateStartTime() != null) {
                sql.append(" and participate_time >= ? ");
                countSql.append(" and participate_time >= ? ");
                param.add(req.getQuery().getParticipateStartTime());
            }
            if (req.getQuery().getParticipateEndTime() != null) {
                sql.append(" and participate_time <= ? ");
                countSql.append(" and participate_time <= ? ");
                param.add(req.getQuery().getParticipateEndTime());
            }
        }

            sql.append(" order by participate_time desc");

            int total = jt.queryForInt(countSql.toString(), param.toArray());
            String dbType = jt.getDbType();
            List<ArchiveVo> result;
            if (total > 0) {
                SqlUtil.appendLimit(dbType, param, sql, req);
                result = jt.queryForBeans(ArchiveVo.class, sql.toString(), param.toArray());
            } else {
                result = new ArrayList<>();
            }
            PageRsp<ArchiveVo> rsp = new PageRsp<>();
            rsp.setResult(result);
            rsp.setTotalNum(total);
            rsp.setPageNo(req.getPageNo());
            rsp.setPageSize(req.getPageSize());
            return rsp;
        }


    public Integer insertArchive(YcqYjsArchive obj) {
        String sql = "INSERT INTO ycq_yjs_archive (id,name, participate_time, img, app_code, ACTIVITY_TYPE, del_flag, create_time, description,student_id,detail,political_type,last_updated_id,last_updated_time,created_id,extension_type,event_id,user_id,user_name,class_name,detail_topic,detail_img,detail_introduction,extension_json) " +
                "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
        int update = jt.update(sql,  obj.getId(),obj.getName(), obj.getParticipateTime(), obj.getImg(), obj.getAppCode(),
                obj.getActivityType(), obj.getDelFlag(), obj.getCreateTime(), obj.getDescription(),obj.getStudentId(),obj.getDetail(),obj.getPoliticalType(),obj.getLastUpdatedId(),obj.getLastUpdatedTime(),obj.getCreatedId(),obj.getExtensionType(),obj.getEventId(),obj.getUserId(),obj.getUserName(),obj.getClassName(),obj.getDetailTopic(),obj.getDetailImg(),obj.getDetailIntroduction(),obj.getExtensionJson());
        return update;
    }

    public boolean updateArchive(YcqYjsArchive archive) {
        StringBuffer sql = new StringBuffer("UPDATE ycq_yjs_archive SET");
        List<Object> params = new ArrayList<>();

        if (archive != null) {
            if (archive.getStudentId() != null) {
                sql.append(" student_id = ?,");
                params.add(archive.getStudentId());
            }
            if (archive.getName() != null) {
                sql.append(" name = ?,");
                params.add(archive.getName());
            }
            if (archive.getParticipateTime() != null) {
                sql.append(" participate_time = ?,");
                params.add(archive.getParticipateTime());
            }
            if (archive.getImg() != null) {
                sql.append(" img = ?,");
                params.add(archive.getImg());
            }
            if (archive.getAppCode() != null) {
                sql.append(" app_code = ?,");
                params.add(archive.getAppCode());
            }
            if (archive.getActivityType() != null) {
                sql.append(" activity_type = ?,");
                params.add(archive.getActivityType());
            }
            if (archive.getLastUpdatedTime() != null) {
                sql.append(" last_updated_time = ?,");
                params.add(archive.getLastUpdatedTime());
            }
            if (archive.getLastUpdatedId() != null) {
                sql.append(" last_updated_id = ?,");
                params.add(archive.getLastUpdatedId());
            }
            if (archive.getDetail() != null) {
                sql.append(" detail = ?,");
                params.add(archive.getDetail());
            }
            if (archive.getEventId() != null) {
                sql.append(" event_id = ?,");
                params.add(archive.getEventId());
            }
            if (archive.getExtensionType() != null) {
                sql.append(" extension_type = ?,");
                params.add(archive.getExtensionType());
            }
            if (archive.getUserId() != null) {
                sql.append(" user_id = ?,");
                params.add(archive.getUserId());
            }
            if (archive.getDetailTopic() != null) {
                sql.append(" detail_topic = ?,");
                params.add(archive.getDetailTopic());
            }
            if (archive.getDetailImg() != null) {
                sql.append(" detail_img = ?,");
                params.add(archive.getDetailImg());
            }if (archive.getDetailIntroduction() != null) {
                sql.append(" detail_introduction = ?,");
                params.add(archive.getDetailIntroduction());
            }if (archive.getExtensionJson() != null) {
                sql.append(" extension_json = ?,");
                params.add(archive.getExtensionJson());
            }
            if (archive.getClassName() != null) {
                sql.append(" class_name = ?,");
                params.add(archive.getClassName());
            }
            if (archive.getUserName() != null) {
                sql.append(" user_name = ?,");
                params.add(archive.getUserName());
            }
            if (archive.getDescription() != null) {
                sql.append(" description = ?,");
                params.add(archive.getDescription());
            }
            if (archive.getPoliticalType() != null) {
                sql.append(" political_type = ?,");
                params.add(archive.getPoliticalType());
            }
            if (archive.getDelFlag() != null) {
                sql.append(" del_flag = ?,");
                params.add(archive.getDelFlag());
            }
        }
        // 删除最后的逗号
        sql.deleteCharAt(sql.length() - 1);
        sql.append(" WHERE id = ?");
        params.add(archive.getId());

        int rowsAffected = jt.update(sql.toString(), params.toArray());

        if (rowsAffected > 0) {
            System.out.println("属性更新成功！");
            return true;
        } else {
            System.out.println("属性更新失败！");
            return false;
        }
    }

    public void deleteById(String id) {
        String sql = "update ycq_yjs_archive set del_flag = 1 where id = ?";
        jt.update(sql,id);
    }

    public MthMatchVo getMthMatchInfo(Long eventId) {
        String QUERY_SQL ="SELECT * FROM mth_match_info where id = ? AND del_flag = '0'";
        return jt.queryForBean (MthMatchVo.class, QUERY_SQL, new Object[]{eventId});
    }


    public List<YcqYjsArchive> selectByReq(AppStudentArchivesReq req) {
        StringBuilder sql = new StringBuilder("select  student_id,name,participate_time,description,img,app_code,activity_type,detail,political_type,extension_type,event_id" +
                "  from ycq_yjs_archive where del_flag = 0  ");
        List<Object> param = new ArrayList<>();

        // 学生ID
        if (req.getStudentId() != null) {
            sql.append(" and student_id = ? ");
            param.add(req.getStudentId());
        }

        // 活动类型
        if (StringUtils.isNotBlank(req.getActivityType())) {
            sql.append(" and activity_type = ? ");
            param.add(req.getActivityType());
        }

        // 类型
        if (StringUtils.isNotBlank(req.getPoliticalType())) {
            sql.append(" and political_type = ? ");
            param.add(req.getPoliticalType());
        }
        if (req.getParticipateStartTime() != null) {
            sql.append(" and participate_time >= ? ");
            param.add(req.getParticipateStartTime());
        }
        if (req.getParticipateEndTime() != null) {
            sql.append(" and participate_time <= ? ");
            param.add(req.getParticipateEndTime());
        }
        sql.append(" order by participate_time desc");
        return jt.queryForBeans(YcqYjsArchive.class, sql.toString(), param.toArray());
    }

    public List<AppElseStudentArchivesVo> selectListByReq(AppStudentArchivesReq req) {
        StringBuilder sql = new StringBuilder("select  *  from ycq_yjs_archive where del_flag = 0  ");
        List<Object> param = new ArrayList<>();

        // 学生ID
        if (Objects.isNull(req.getStudentId())) {
            sql.append(" and student_id = ? ");
            param.add(req.getStudentId());
        }

        // 活动类型
        if (StringUtils.isNotBlank(req.getActivityType())) {
            sql.append(" and activity_type = ? ");
            param.add(req.getActivityType());
        }

        // 类型
        if (StringUtils.isNotBlank(req.getPoliticalType())) {
            sql.append(" and political_type = ? ");
            param.add(req.getPoliticalType());
        }
        if (!CollectionUtil.isEmpty(req.getExtensionTypes())) {
            String extensionTypes = Joiner.on(",").join(req.getExtensionTypes());
            sql.append(" and extension_type in ("+ extensionTypes +") ");

        }
        if (req.getParticipateStartTime() != null) {
            sql.append(" and participate_time >= ? ");
            param.add(req.getParticipateStartTime());
        }
        if (req.getParticipateEndTime() != null) {
            sql.append(" and participate_time <= ? ");
            param.add(req.getParticipateEndTime());
        }
        sql.append(" order by participate_time desc");
        return jt.queryForBeans(AppElseStudentArchivesVo.class, sql.toString(), param.toArray());
    }
}