package com.crunii.micro.service.oneaffair.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsEvaluateIndex;
import com.crunii.micro.service.oneaffair.dto.vo.evaluate.EvaluateIndexReq;
import com.crunii.micro.service.oneaffair.dto.vo.evaluate.EvaluateIndexVo;
import com.crunii.micro.service.oneaffair.dto.vo.evaluate.YcqYjsEvaluateIndexVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Repository
public class YcqYjsEvaluateIndexDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;


    public PageRsp<YcqYjsEvaluateIndexVo> pageSearchIndexInfo(PageReq<EvaluateIndexReq> req) {
        StringBuilder sql = new StringBuilder("select id from ycq_yjs_evaluate_index where del_flag = 0 and parent_id is null and system_id = ? ");
        StringBuilder countSql = new StringBuilder("select count(1) from ycq_yjs_evaluate_index where del_flag = 0 and parent_id is null  and system_id = ? ");
        List<Object> param = new ArrayList<>();
        param.add(req.getQuery().getSystemId());
        int total = jt.queryForInt(countSql.toString(),param.toArray());
        String dbType = jt.getDbType();
        List<YcqYjsEvaluateIndexVo> result = new ArrayList<>();
        // 是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            List<BigInteger> ids = new ArrayList<>();
            List<Map<String, Object>> maps = jt.queryForList(sql.toString(), param.toArray());
            for(Map<String, Object> map : maps){
                Object id = map.get("id");
                ids.add(new BigInteger(id.toString()));
            }
            //封装sql
            StringBuilder allSql =createQueryIndexSql(ids);
            if(StringUtils.isNotBlank(req.getQuery().getName())){
                allSql.append(" and name like '%"+req.getQuery().getName()+"%' ");
            }
            allSql.append( " order by sort asc " );
            List<YcqYjsEvaluateIndexVo> all = jt.queryForBeans(YcqYjsEvaluateIndexVo.class,allSql.toString());
            List<YcqYjsEvaluateIndexVo> first = all.stream().filter(e -> e.getParentId() == null).collect(Collectors.toList());
            if(!CollectionUtils.isEmpty(first)){
                for(YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo : first){
                    List<YcqYjsEvaluateIndexVo> children  =getChildren(all, ycqYjsEvaluateIndexVo);
                    ycqYjsEvaluateIndexVo.setChildren(CollectionUtils.isEmpty(children) ? null :children);
                }
                result = first;
            }
        }
        PageRsp<YcqYjsEvaluateIndexVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }


    private  List<YcqYjsEvaluateIndexVo> getChildren(List<YcqYjsEvaluateIndexVo> all, YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo) {
        List<YcqYjsEvaluateIndexVo> result = new ArrayList<>();

        for (YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo2 : all) {
             //如果子节点是父节点的子节点,则存入子节点
            if (ycqYjsEvaluateIndexVo2.getParentId() != null && ycqYjsEvaluateIndexVo2.getParentId().compareTo(ycqYjsEvaluateIndexVo.getId()) == 0) {
                result.add(ycqYjsEvaluateIndexVo2);
            }
        }
        for(YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo2 : result){
            List<YcqYjsEvaluateIndexVo> children = getChildren(all, ycqYjsEvaluateIndexVo2);
            ycqYjsEvaluateIndexVo2.setChildren(CollectionUtils.isEmpty(children) ? null :children);
        }
        return result;
    }

    private StringBuilder createQueryIndexSql(List<BigInteger> ids){
        StringBuilder allSql = new StringBuilder("WITH subordinates(id, system_id, parent_id, name, judgment_method, calculation_method, index_type, index_level, score, grade, is_satisfy, show_type, sort, system_rule,create_worker,update_worker,create_time,update_time,del_flag,index_description) " );
        allSql.append("AS ( SELECT id, system_id,parent_id,name,judgment_method,calculation_method,index_type,index_level,score,grade,is_satisfy,show_type,sort,system_rule,create_worker,update_worker,create_time,update_time,del_flag,index_description ");
        allSql.append(" FROM YCQ_YJS_EVALUATE_INDEX  WHERE id in( "+ StringUtils.join(ids, ",") +" )  " );
        allSql.append(" UNION ALL SELECT e.id,e.system_id,e.parent_id,e.name,e.judgment_method,e.calculation_method,");
        allSql.append("e.index_type,e.index_level, e.score,e.grade,e.is_satisfy,e.show_type,e.sort, e.system_rule,e.create_worker,e.update_worker,e.create_time,e.update_time,e.del_flag,e.index_description FROM YCQ_YJS_EVALUATE_INDEX e INNER JOIN subordinates s   ON e.parent_id = s.id   ");
        allSql.append(" where e.del_flag = 0 ) SELECT * FROM subordinates where 1=1 ");
        return allSql;
    }
    public List<YcqYjsEvaluateIndex> queryIndexByIds(List<BigInteger> ids) {
        StringBuilder allSql = createQueryIndexSql(ids);
        return jt.queryForBeans(YcqYjsEvaluateIndex.class,allSql.toString());
    }
}
