package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsGoodStudent;
import com.crunii.micro.service.oneaffair.dto.vo.goodstudent.GoodStudentReq;
import com.crunii.micro.service.oneaffair.dto.vo.goodstudent.GoodStudentVo;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 优秀学生表(YCQ_YJS_GOOD_STUDENT)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsGoodStudentJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String ID_SQL = "select * from ycq_yjs_good_student where del_flag = 0 and id = ?";

    private static final String BASE_SQL = "select * from ycq_yjs_good_student where del_flag = 0";

    private static final String COUNT_SQL = "select count(*) from ycq_yjs_good_student where del_flag = 0";

    /**
     * 分页查询优秀学生数据
     *
     * @param req
     * @return
     */
    public PageRsp<GoodStudentVo> pageSearchGoodStudent(PageReq<GoodStudentReq> req) {
        log.info("YcqYjsGoodStudentJdbcDao.pageSearchGoodStudent.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        StringBuilder countSql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();

        if (req.getQuery() != null) {
            // 学生编号
            if (StringUtils.isNotBlank(req.getQuery().getStudentNum())) {
                sql.append(" and student_num like ? ");
                countSql.append(" and student_num like ? ");
                param.add("%" + req.getQuery().getStudentNum() + "%");
            }
            // 学生名称
            if (StringUtils.isNotBlank(req.getQuery().getStudentName())) {
                sql.append(" and student_name like ? ");
                countSql.append(" and student_name like ? ");
                param.add("%" + req.getQuery().getStudentName() + "%");
            }
            // 学校名称
            if (StringUtils.isNotBlank(req.getQuery().getSchoolName())) {
                sql.append(" and school_name like ? ");
                countSql.append(" and school_name like ? ");
                param.add("%" + req.getQuery().getSchoolName() + "%");
            }
            // 性别
            if (req.getQuery().getSex() != null) {
                sql.append(" and sex = ? ");
                countSql.append(" and sex = ? ");
                param.add(req.getQuery().getSex());
            }
            // 学校ID
            if (req.getQuery().getSchoolId() != null) {
                sql.append(" and school_id = ? ");
                countSql.append(" and school_id = ? ");
                param.add(req.getQuery().getSchoolId());
            }
            // 年级ID
            if (req.getQuery().getGradeId() != null) {
                sql.append(" and grade_id = ? ");
                countSql.append(" and grade_id = ? ");
                param.add(req.getQuery().getGradeId());
            }
            // 班级ID
            if (req.getQuery().getClassId() != null) {
                sql.append(" and class_id = ? ");
                countSql.append(" and class_id = ? ");
                param.add(req.getQuery().getClassId());
            }
            // 政治面貌
            if (req.getQuery().getPoliticalStatusType() != null) {
                sql.append(" and political_status_type = ? ");
                countSql.append(" and political_status_type = ? ");
                param.add(req.getQuery().getPoliticalStatusType());
            }
            // 学年类型
            if (req.getQuery().getSchoolYearType() != null) {
                sql.append(" and school_year_type = ? ");
                countSql.append(" and school_year_type = ? ");
                param.add(req.getQuery().getSchoolYearType());
            }
            // 优秀称号类型
            if (req.getQuery().getHonoraryTitleType() != null) {
                sql.append(" and honorary_title_type = ? ");
                countSql.append(" and honorary_title_type = ? ");
                param.add(req.getQuery().getHonoraryTitleType());
            }
            // 创建时间
            if (req.getQuery().getStartCreatedTime() != null) {
                sql.append(" and created_time >= ? ");
                countSql.append(" and created_time >= ? ");
                param.add(req.getQuery().getStartCreatedTime());
            }
            if (req.getQuery().getEndCreatedTime() != null) {
                sql.append(" and  ?  >= created_time ");
                countSql.append(" and  ?  >= created_time ");
                param.add(req.getQuery().getEndCreatedTime());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getQuery().getIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 学校ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getSchoolIdList())) {
                String itemSql = " and school_id in " + CommonUtil.fromListToStr(req.getQuery().getSchoolIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 年级ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getGradeIdList())) {
                String itemSql = " and grade_id in " + CommonUtil.fromListToStr(req.getQuery().getGradeIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 班级ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getClassIdList())) {
                String itemSql = " and class_id in " + CommonUtil.fromListToStr(req.getQuery().getClassIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 是否展示
            if (req.getQuery().getShowFlag() != null) {
                sql.append(" and show_flag = ? ");
                countSql.append(" and show_flag = ? ");
                param.add(req.getQuery().getShowFlag());
            }
        }
        sql.append(" order by show_flag desc,created_time desc ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<GoodStudentVo> result;
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(GoodStudentVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<GoodStudentVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    /**
     * 列表查询优秀学生数据
     *
     * @param req
     * @return
     */
    public List<YcqYjsGoodStudent> selectList(GoodStudentReq req) {
        log.info("YcqYjsGoodStudentJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生编号
            if (StringUtils.isNotBlank(req.getStudentNum())) {
                sql.append(" and student_num like ? ");
                param.add("%" + req.getStudentNum() + "%");
            }
            // 学生名称
            if (StringUtils.isNotBlank(req.getStudentName())) {
                sql.append(" and student_name like ? ");
                param.add("%" + req.getStudentName() + "%");
            }
            // 学校名称
            if (StringUtils.isNotBlank(req.getSchoolName())) {
                sql.append(" and school_name like ? ");
                param.add("%" + req.getSchoolName() + "%");
            }
            // 学校ID
            if (req.getSchoolId() != null) {
                sql.append(" and school_id = ? ");
                param.add(req.getSchoolId());
            }
            // 年级ID
            if (req.getGradeId() != null) {
                sql.append(" and grade_id = ? ");
                param.add(req.getGradeId());
            }
            // 班级ID
            if (req.getClassId() != null) {
                sql.append(" and class_id = ? ");
                param.add(req.getClassId());
            }
            // 政治面貌
            if (req.getPoliticalStatusType() != null) {
                sql.append(" and political_status_type = ? ");
                param.add(req.getPoliticalStatusType());
            }
            // 学年类型
            if (req.getSchoolYearType() != null) {
                sql.append(" and school_year_type = ? ");
                param.add(req.getSchoolYearType());
            }
            // 优秀称号类型
            if (req.getHonoraryTitleType() != null) {
                sql.append(" and honorary_title_type = ? ");
                param.add(req.getHonoraryTitleType());
            }
            // 创建时间
            if (req.getStartCreatedTime() != null) {
                sql.append(" and created_time >= ? ");
                param.add(req.getStartCreatedTime());
            }
            if (req.getEndCreatedTime() != null) {
                sql.append(" and  ?  >= created_time ");
                param.add(req.getEndCreatedTime());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                sql.append(" and id in " + CommonUtil.fromListToStr(req.getIdList()));
            }
            // 学校ID列表
            if (CollectionUtil.isNotEmpty(req.getSchoolIdList())) {
                sql.append(" and school_id in " + CommonUtil.fromListToStr(req.getSchoolIdList()));
            }
            // 年级ID列表
            if (CollectionUtil.isNotEmpty(req.getGradeIdList())) {
                sql.append(" and grade_id in " + CommonUtil.fromListToStr(req.getGradeIdList()));
            }
            // 班级ID列表
            if (CollectionUtil.isNotEmpty(req.getClassIdList())) {
                sql.append(" and class_id in " + CommonUtil.fromListToStr(req.getClassIdList()));
            }
        }

        sql.append(" order by created_time desc ");
        return jt.queryForBeans(YcqYjsGoodStudent.class, sql.toString(), param.toArray());
    }

}
