package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsMedalActivity;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentCourse;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalActivityInfoReq;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 学生表(YCQ_YJS_MEDAL_ACTIVITY)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsMedalActivityJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String BASE_SQL = "select * from YCQ_YJS_MEDAL_ACTIVITY where del_flag = 0";

    public void deleteMedalInfo(BigInteger medalId, List<BigInteger> activityIds) {
        // 判断参数正常
        if (medalId != null && !CollectionUtils.isEmpty(activityIds)) {
            // 构建动态的IN子句
            String inClause = activityIds.stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining(", ", "(", ")")); // 生成"(1, 2, 3)"格式的子句
            String sql = "UPDATE ycq_yjs_medal_activity SET del_flag = 1, last_updated_time = ? WHERE medal_id = ? AND activity_id IN " + inClause;
            jt.update(sql, new Timestamp(new Date().getTime()), medalId);
        } else {
            log.info("无需删除关联数据");
        }
    }

    public void updateMedalActivityStatus(BigInteger medalId, BigInteger activityId, String status) {
        // 判断参数正常
        if (medalId != null && activityId != null && StringUtils.isNotEmpty(status)) {
            String sql = "UPDATE ycq_yjs_medal_activity SET status = ?, last_updated_time = ? WHERE medal_id = ? AND activity_id = ? ";
            jt.update(sql, status, new Timestamp(new Date().getTime()), medalId, activityId);
        } else {
            log.info("无更新关联活动状态数据");
        }
    }

    public List<YcqYjsMedalActivity> selectList(MedalActivityInfoReq req) {
        log.info("YcqYjsMedalActivityJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 奖章id
            if (req.getMedalId() != null) {
                sql.append(" and medal_id = ? ");
                param.add(req.getMedalId());
            }
            // 活动id
            if (req.getActivityId() != null) {
                sql.append(" and activity_id = ? ");
                param.add(req.getActivityId());
            }
            // 业务编码
            if (req.getAppCode() != null) {
                sql.append(" and app_code = ? ");
                param.add(req.getAppCode());
            }
            // 状态 0:已报名，1:已参与，2:已获奖
            if (StringUtils.isNotBlank(req.getStatus())) {
                sql.append(" and status = ? ");
                param.add(req.getStatus());
            }
        }
        sql.append(" order by id");
        return jt.queryForBeans(YcqYjsMedalActivity.class, sql.toString(), param.toArray());
    }

}