package com.crunii.micro.service.oneaffair.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsMedal;
import com.crunii.micro.service.oneaffair.dto.vo.medal.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 学生表(YCQ_YJS_MEDAL)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsMedalJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String ID_SQL = "select * from ycq_yjs_medal where id = ?";

    private static final String LIST_SQL = "select * from ycq_yjs_medal where del_flag = 0 ";

    private static final String STUDENT_LIST_SQL = "select a.*,IFNULL(b.if_light,0) AS if_light,IFNULL(b.light_progress,0) AS light_progress,b.id as student_medal_id from ycq_yjs_medal a " +
            "left join ycq_yjs_student_medal b " +
            "on a.id = b.medal_id and b.student_id = ? and b.del_flag = 0 where a.del_flag = 0 ";

    private static final String STUDENT_MEDAL_SQL = "select a.*,IFNULL(b.if_light,0) AS if_light,b.light_time from ycq_yjs_medal a left join ycq_yjs_student_medal b on a.id = b.medal_id and b.student_id = ? and b.del_flag = 0 where a.del_flag = 0 and a.id = ? ";

    private static final String PAGE_SQL = "SELECT a.*, subquery.medal_activity " +
            "FROM ycq_yjs_medal a " +
            "LEFT JOIN ( " +
            "    SELECT b.medal_id, LISTAGG(c.match_name, ',') WITHIN GROUP (ORDER BY c.match_name) AS medal_activity " +
            "    FROM ycq_yjs_medal_activity b " +
            "    LEFT JOIN mth_match_info c ON b.app_code = 'matchManagement' AND b.activity_id = c.id AND c.del_flag = 0 " +
            "    WHERE b.del_flag = 0 " +
            "    GROUP BY b.medal_id " +
            ") AS subquery ON a.id = subquery.medal_id " +
            "WHERE a.del_flag = 0 ";

    private static final String PAGE_COUNT_SQL = "select count(*) from ycq_yjs_medal where del_flag = 0 ";

    public MedalInfoVo selectOneById(BigInteger id){
        log.info("YcqYjsMedalJdbcDao.selectOneById.param：{}", id);
        if(id == null){
            return null;
        }
        return jt.queryForBean(MedalInfoVo.class, ID_SQL, id);
    }

    public List<YcqYjsMedal> selectList(MedalReq req){
        log.info("YcqYjsMedalJdbcDao.selectList.param：{}", req);
        StringBuilder sql = new StringBuilder(LIST_SQL);
        List<Object> param = new ArrayList<>();

        // 奖章名称
        if (StringUtils.isNotBlank(req.getMedalName())) {
            sql.append(" and medal_name like ? ");
            param.add("%" + req.getMedalName() + "%");
        }
        // 奖章类型
        if (StringUtils.isNotBlank(req.getMedalType())) {
            sql.append(" and medal_type = ? ");
            param.add(req.getMedalType());
        }
        // 奖章标识
        if (StringUtils.isNotBlank(req.getMedalMark())) {
            sql.append(" and medal_mark = ? ");
            param.add(req.getMedalMark());
        }
        // 年级
        if (StringUtils.isNotBlank(req.getOwnerGrade())) {
            sql.append(" and FIND_IN_SET(?, owner_grade) > 0 ");
            param.add(req.getOwnerGrade());
        }
        // 学期
        if (StringUtils.isNotBlank(req.getOwnerSemester())) {
            sql.append(" and owner_semester = ? ");
            param.add(req.getOwnerSemester());
        }
        // 学校code
        if (StringUtils.isNotBlank(req.getOrgCode())) {
            sql.append(" and org_id = ? ");
            param.add(req.getOrgCode());
        }
        // 学校id
        if (StringUtils.isNotBlank(req.getOrgId())) {
            sql.append(" and org_id = ? ");
            param.add(req.getOrgId());
        }
        // 展示状态 0:展示 1:不展示
        if (req.getDisplayStatus() != null) {
            sql.append(" and display_status = ? ");
            param.add(req.getDisplayStatus());
        }
        if (!CollectionUtils.isEmpty(req.getIds())) {
            // 构建动态的IN子句
            String inClause = req.getIds().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining(", ", "(", ")")); // 生成"(1, 2, 3)"格式的子句
            sql.append(" and id IN  ").append(inClause);
        }
        // 奖章类型 多选
        if (!CollectionUtils.isEmpty(req.getMedalTypes())) {
            // 构建动态的IN子句
            String inClause = req.getMedalTypes().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining("', '", "('", "')")); // 生成"(1, 2, 3)"格式的子句
            sql.append(" and medal_type IN  ").append(inClause);
        }
        // 所属年级 多选
        if (!CollectionUtils.isEmpty(req.getOwnerGrades())) {
            // 构建动态的IN子句
            /*String inClause = req.getOwnerGrades().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining("', '", "(", ")")); // 生成"(1, 2, 3)"格式的子句
            sql.append(" and ").append(inClause);*/
            sql.append("AND (");
            for (int i = 0; i < req.getOwnerGrades().size(); i++) {
                sql.append("FIND_IN_SET(?, owner_grade) > 0");
                // 如果不是最后一个元素，则加上OR连接符
                if (i < req.getOwnerGrades().size() - 1) {
                    sql.append(" OR ");
                }
                param.add(req.getOwnerGrades().get(i));
            }
            sql.append(")");
        }
        // 所属学期 多选
        if (!CollectionUtils.isEmpty(req.getOwnerSemesters())) {
            // 构建动态的IN子句
            String inClause = req.getOwnerSemesters().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining("', '", "('", "')")); // 生成"(1, 2, 3)"格式的子句
            sql.append(" and owner_semester IN  ").append(inClause);
        }
        if (!CollectionUtils.isEmpty(req.getUnIds())) {
            // 构建动态的IN子句
            String inClause = req.getUnIds().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining(", ", "(", ")")); // 生成"(1, 2, 3)"格式的子句
            sql.append(" and id not IN  ").append(inClause);
        }
        sql.append(" order by medal_sort asc ");
        return jt.queryForBeans(YcqYjsMedal.class, sql.toString(), param.toArray());
    }

    public List<MedalStudentVo> selectStudentMedalList(MedalReq req){
        log.info("YcqYjsMedalJdbcDao.selectStudentMedalList.param：{}", req);
        StringBuilder sql = new StringBuilder(STUDENT_LIST_SQL);
        List<Object> param = new ArrayList<>();
        param.add(req.getStudentId());

        // 奖章名称
        if (StringUtils.isNotBlank(req.getMedalName())) {
            sql.append(" and a.medal_name like ? ");
            param.add("%" + req.getMedalName() + "%");
        }
        // 奖章类型
        if (StringUtils.isNotBlank(req.getMedalType())) {
            sql.append(" and a.medal_type = ? ");
            param.add(req.getMedalType());
        }
        // 奖章标识
        if (StringUtils.isNotBlank(req.getMedalMark())) {
            sql.append(" and a.medal_mark = ? ");
            param.add(req.getMedalMark());
        }
        // 年级
        if (StringUtils.isNotBlank(req.getOwnerGrade())) {
            sql.append(" and a.owner_grade = ? ");
            param.add(req.getOwnerGrade());
        }
        // 学期
        if (StringUtils.isNotBlank(req.getOwnerSemester())) {
            sql.append(" and a.owner_semester = ? ");
            param.add(req.getOwnerSemester());
        }
        // 学校code
        if (StringUtils.isNotBlank(req.getOrgId())) {
            sql.append(" and a.org_id = ? ");
            param.add(req.getOrgId());
        }
        // 展示状态 0:展示 1:不展示
        if (req.getDisplayStatus() != null) {
            sql.append(" and a.display_status = ? ");
            param.add(req.getDisplayStatus());
        }
        if (!CollectionUtils.isEmpty(req.getIds())) {
            // 构建动态的IN子句
            String inClause = req.getIds().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining(", ", "(", ")")); // 生成"(1, 2, 3)"格式的子句
            sql.append(" and a.id IN  ").append(inClause);
        }
        if (!CollectionUtils.isEmpty(req.getUnIds())) {
            // 构建动态的IN子句
            String inClause = req.getUnIds().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining(", ", "(", ")")); // 生成"(1, 2, 3)"格式的子句
            sql.append(" and a.id not IN  ").append(inClause);
        }
        // 所属学期 多选
        if (!CollectionUtils.isEmpty(req.getOwnerSemesters())) {
            // 构建动态的IN子句
            String inClause = req.getOwnerSemesters().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining("', '", "('", "')")); // 生成"(1, 2, 3)"格式的子句
            sql.append(" and a.owner_semester IN  ").append(inClause);
        }
        sql.append(" order by a.medal_sort asc ");
        return jt.queryForBeans(MedalStudentVo.class, sql.toString(), param.toArray());
    }

    public MedalStudentLightVo selectMedalLight(StudentActivityReq studentActivityReq) {
        log.info("YcqYjsMedalJdbcDao.selectMedalLight.param：{}", studentActivityReq.toString());
        StringBuilder sql = new StringBuilder(STUDENT_MEDAL_SQL);
        List<Object> param = new ArrayList<>();
        param.add(studentActivityReq.getStudentId());
        param.add(studentActivityReq.getMedalId());
        return jt.queryForBean(MedalStudentLightVo.class, sql.toString(), param.toArray());
    }

    public PageRsp<MedalInfoVo> pageSearchMedalInfo(PageReq<MedalReq> req){
        log.info("YcqYjsMedalJdbcDao.pageSearchMedalInfo.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        sql.append(" and a.owner_grade IS NOT NULL ");
        countSql.append(" and owner_grade IS NOT NULL ");
        List<Object> param = new ArrayList<>();
        // 奖章编号
        if (StringUtils.isNotBlank(req.getQuery().getMedalNo())) {
            sql.append(" and a.medal_no = ? ");
            countSql.append(" and medal_no = ? ");
            param.add(req.getQuery().getMedalNo());
        }
        // 奖章名称
        if (StringUtils.isNotBlank(req.getQuery().getMedalName())) {
            sql.append(" and a.medal_name like ? ");
            countSql.append(" and medal_name like ? ");
            param.add("%" + req.getQuery().getMedalName() + "%");
        }
        // 奖章类型
        if (StringUtils.isNotBlank(req.getQuery().getMedalType())) {
            sql.append(" and a.medal_type = ? ");
            countSql.append(" and medal_type = ? ");
            param.add(req.getQuery().getMedalType());
        }
        // 奖章标识
        if (StringUtils.isNotBlank(req.getQuery().getMedalMark())) {
            sql.append(" and a.medal_mark = ? ");
            countSql.append(" and medal_mark = ? ");
            param.add(req.getQuery().getMedalMark());
        }
        // 展示状态 0:展示 1:不展示
        if (req.getQuery().getDisplayStatus() != null ) {
            sql.append(" and a.display_status = ? ");
            countSql.append(" and display_status = ? ");
            param.add(req.getQuery().getDisplayStatus());
        }
        // 所属学校
        if (StringUtils.isNotBlank(req.getQuery().getOrgId())) {
            sql.append(" and a.org_id = ? ");
            countSql.append(" and org_id = ? ");
            param.add(req.getQuery().getOrgId());
        }
        // 所属年级
        if (StringUtils.isNotBlank(req.getQuery().getOwnerGrade())) {
            sql.append(" and FIND_IN_SET(?, a.owner_grade) > 0 ");
            countSql.append(" and FIND_IN_SET(?, owner_grade) > 0  ");
            param.add(req.getQuery().getOwnerGrade());
        }
        // 所属班级
        if (StringUtils.isNotBlank(req.getQuery().getOwnerClass())) {
            sql.append(" and a.owner_class = ? ");
            countSql.append(" and owner_class = ? ");
            param.add(req.getQuery().getOwnerClass());
        }
        // 创建时间 开始时间
        if (req.getQuery().getStartCreatedTime() != null) {
            sql.append(" and CAST(a.created_time AS DATE) >=  ? ");
            countSql.append(" and CAST(created_time AS DATE) >= ? ");
            param.add(req.getQuery().getStartCreatedTime());
        }
        // 创建时间 结束时间
        if (req.getQuery().getEndCreatedTime() != null) {
            sql.append(" and CAST(a.created_time AS DATE) <= ?  ");
            countSql.append(" and CAST(created_time AS DATE) <= ?  ");
            param.add(req.getQuery().getEndCreatedTime());
        }
        // 登录用户学校组织权限
        if (!CollectionUtils.isEmpty(req.getQuery().getOrgIds())) {
            // 构建动态的IN子句
            String inClause = req.getQuery().getOrgIds().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining("', '", "('", "')")); // 生成"('1', '2', '3')"格式的子句
            sql.append(" and a.org_id IN  ").append(inClause);
            countSql.append(" and org_id IN  ").append(inClause);
        }
        sql.append(" order by a.display_status asc, a.created_time desc ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<MedalInfoVo> result;
        // 是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(MedalInfoVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<MedalInfoVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    public void deleteMedalInfo(BigInteger id) {
        String sql = "update ycq_yjs_medal set del_flag = 1, last_updated_time = ? "
                + " where id = ? ";
        jt.update(sql, new Timestamp(new Date().getTime()), id);
    }

    public void changeDisplayStatus(MedalDisplayStatusReq displayStatusReq) {
        String sql = "update ycq_yjs_medal set display_status = ?, last_updated_time = ?, last_updated_id = ? "
                + " where id = ? ";
        jt.update(sql, displayStatusReq.getDisplayStatus(), new Timestamp(new Date().getTime()), displayStatusReq.getLastUpdatedId(), displayStatusReq.getId());
    }
}