package com.crunii.micro.service.oneaffair.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalReq;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalRuleInfoVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 争章规则表(YCQ_YJS_MEDAL_RULE)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsMedalRuleJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String PAGE_SQL = "SELECT m.id , medal_no AS medal_no,m.medal_name AS medal_name,m.medal_pic_key AS medal_pic_key, " +
            " m.medal_type_name AS medal_type_name,m.introduction AS introduction,m.org_id AS org_id,m.org_name AS org_name,r.* " +
            " FROM ycq_yjs_medal m LEFT JOIN " +
            "(SELECT medal_id,owner_grade,owner_grade_name,LISTAGG(rule_content, '|||') AS rule_content,LISTAGG(rule_type, '|||') AS rule_type,MAX(need_voucher) AS need_voucher,MAX(created_time) AS created_time,MAX(created_name) AS created_name,MAX(id) AS rule_id FROM YCQ_YJS_MEDAL_RULE " +
            "WHERE del_flag = 0 " +
            "GROUP BY medal_id,owner_grade,owner_grade_name) r ON m.id = r.medal_id WHERE m.del_flag = 0 AND r.rule_type is not null ";

    private static final String PAGE_COUNT_SQL = "SELECT count(1) " +
            " FROM ycq_yjs_medal m LEFT JOIN " +
            "(SELECT medal_id,owner_grade,owner_grade_name,LISTAGG(rule_type, '|||') AS rule_type,MAX(created_time) AS created_time FROM YCQ_YJS_MEDAL_RULE " +
            "WHERE del_flag = 0 " +
            "GROUP BY medal_id,owner_grade,owner_grade_name) r ON m.id = r.medal_id WHERE m.del_flag = 0 AND r.rule_type is not null ";

    public PageRsp<MedalRuleInfoVo> pageSearchMedalRuleInfo(PageReq<MedalReq> req){
        log.info("YcqYjsMedalRuleJdbcDao.pageSearchMedalRuleInfo.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        // 奖章编号
        if (StringUtils.isNotBlank(req.getQuery().getMedalNo())) {
            sql.append(" and m.medal_no = ? ");
            countSql.append(" and m.medal_no = ? ");
            param.add(req.getQuery().getMedalNo());
        }
        // 奖章名称
        if (StringUtils.isNotBlank(req.getQuery().getMedalName())) {
            sql.append(" and m.medal_name like ? ");
            countSql.append(" and m.medal_name like ? ");
            param.add("%" + req.getQuery().getMedalName() + "%");
        }
        // 奖章类型
        if (StringUtils.isNotBlank(req.getQuery().getMedalType())) {
            sql.append(" and m.medal_type = ? ");
            countSql.append(" and m.medal_type = ? ");
            param.add(req.getQuery().getMedalType());
        }
        // 奖章标识
        if (StringUtils.isNotBlank(req.getQuery().getMedalMark())) {
            sql.append(" and m.medal_mark = ? ");
            countSql.append(" and m.medal_mark = ? ");
            param.add(req.getQuery().getMedalMark());
        }
        // 登录用户学校组织权限
        if (!CollectionUtils.isEmpty(req.getQuery().getOrgIds())) {
            // 构建动态的IN子句
            String inClause = req.getQuery().getOrgIds().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining("', '", "('", "')")); // 生成"('1', '2', '3')"格式的子句
            sql.append(" and m.org_id IN  ").append(inClause);
            countSql.append(" and m.org_id IN  ").append(inClause);
        }
        // 创建时间 开始时间
        if (req.getQuery().getStartCreatedTime() != null) {
            sql.append(" and CAST(r.created_time AS DATE) >=  ? ");
            countSql.append(" and CAST(r.created_time AS DATE) >= ? ");
            param.add(req.getQuery().getStartCreatedTime());
        }
        // 创建时间 结束时间
        if (req.getQuery().getEndCreatedTime() != null) {
            sql.append(" and CAST(r.created_time AS DATE) <= ?  ");
            countSql.append(" and CAST(r.created_time AS DATE) <= ?  ");
            param.add(req.getQuery().getEndCreatedTime());
        }
        sql.append(" ORDER BY r.created_time DESC ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<MedalRuleInfoVo> result;
        // 是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(MedalRuleInfoVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<MedalRuleInfoVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    public void deleteMedalRule(BigInteger medalId) {
        // 判断参数正常
        if (medalId != null) {
            String sql = "update ycq_yjs_medal_rule set del_flag = 1, last_updated_time = ? "
                    + " where medal_id = ? ";
            jt.update(sql, new Timestamp(new Date().getTime()), medalId);
        } else {
            log.info("无需删除关联数据");
        }
    }

}