package com.crunii.micro.service.oneaffair.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsMedal;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalDisplayStatusReq;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalInfoVo;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalReq;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalUnifyInfoVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 学生表(YCQ_YJS_MEDAL_UNIFY)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsMedalUnifyJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String LIST_SQL = "select * from ycq_yjs_medal_unify where del_flag = 0 ";


    private static final String PAGE_COUNT_SQL = "select count(*) from ycq_yjs_medal_unify where del_flag = 0 ";

    public PageRsp<MedalUnifyInfoVo> pageSearchMedalUnifyInfo(PageReq<MedalReq> req){
        log.info("YcqYjsMedalUnifyJdbcDao.pageSearchMedalUnifyInfo.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(LIST_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        // 奖章编号
        if (StringUtils.isNotBlank(req.getQuery().getMedalNo())) {
            sql.append(" and medal_no = ? ");
            countSql.append(" and medal_no = ? ");
            param.add(req.getQuery().getMedalNo());
        }
        // 奖章名称
        if (StringUtils.isNotBlank(req.getQuery().getMedalName())) {
            sql.append(" and medal_name like ? ");
            countSql.append(" and medal_name like ? ");
            param.add("%" + req.getQuery().getMedalName() + "%");
        }
        // 奖章类型
        if (StringUtils.isNotBlank(req.getQuery().getMedalType())) {
            sql.append(" and medal_type = ? ");
            countSql.append(" and medal_type = ? ");
            param.add(req.getQuery().getMedalType());
        }
        // 奖章标识
        if (StringUtils.isNotBlank(req.getQuery().getMedalMark())) {
            sql.append(" and medal_mark = ? ");
            countSql.append(" and medal_mark = ? ");
            param.add(req.getQuery().getMedalMark());
        }
        // 展示状态
        if (req.getQuery().getDisplayStatus() != null ) {
            sql.append(" and display_status = ? ");
            countSql.append(" and display_status = ? ");
            param.add(req.getQuery().getDisplayStatus());
        }
        // 创建时间 开始时间
        if (req.getQuery().getStartCreatedTime() != null) {
            sql.append(" and CAST(created_time AS DATE) >=  ? ");
            countSql.append(" and CAST(created_time AS DATE) >= ? ");
            param.add(req.getQuery().getStartCreatedTime());
        }
        // 创建时间 结束时间
        if (req.getQuery().getEndCreatedTime() != null) {
            sql.append(" and CAST(created_time AS DATE) <= ?  ");
            countSql.append(" and CAST(created_time AS DATE) <= ?  ");
            param.add(req.getQuery().getEndCreatedTime());
        }
        sql.append(" order by display_status asc, created_time desc ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<MedalUnifyInfoVo> result;
        // 是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(MedalUnifyInfoVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<MedalUnifyInfoVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    public void deleteMedalUnifyInfo(BigInteger id) {
        String sql = "update ycq_yjs_medal_unify set del_flag = 1, last_updated_time = ? "
                + " where id = ? ";
        jt.update(sql, new Timestamp(new Date().getTime()), id);
    }

    public void changeDisplayStatus(MedalDisplayStatusReq displayStatusReq) {
        String sql = "update ycq_yjs_medal_unify set display_status = ?, last_updated_time = ?, last_updated_id = ? "
                + " where id = ? ";
        jt.update(sql, displayStatusReq.getDisplayStatus(), new Timestamp(new Date().getTime()), displayStatusReq.getLastUpdatedId(), displayStatusReq.getId());
    }

}