package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentBindingUser;
import com.crunii.micro.service.oneaffair.dto.vo.idengtitychoose.StudentInfoByIdentityCardNoVo;
import com.crunii.micro.service.oneaffair.dto.vo.idengtitychoose.StudentInfoByUserVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentBindingUser.StudentBindingUserReq;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生与用户的绑定表(YCQ_YJS_STUDENT_BINDING_USER)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentBindingUserJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String ID_SQL = "select * from ycq_yjs_student_binding_user where del_flag = 0 and id = ?";

    private static final String SEECT_SQL = "select * from ycq_yjs_student_binding_user where del_flag = 0";

    public YcqYjsStudentBindingUser selectOneById(BigInteger id) {
        log.info("YcqYjsStudentBindingUserJdbcDao.selectOneById.param：{}", id);
        if (id == null) {
            return null;
        }
        return jt.queryForBean(YcqYjsStudentBindingUser.class, ID_SQL, id);
    }

    public List<YcqYjsStudentBindingUser> selectList(StudentBindingUserReq req) {
        log.info("YcqYjsStudentBindingUserJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(SEECT_SQL);
        List<Object> param = new ArrayList<>();
        // 主键
        if (req.getId() != null) {
            sql.append(" and id = ? ");
            param.add(req.getId());
        }
        // id列表
        if (CollectionUtil.isNotEmpty(req.getIdList())) {
            sql.append(" and id in ? ");
            param.add(fromListToStr(req.getIdList()));
        }
        // 学生ID
        if (req.getStudentId() != null) {
            sql.append(" and student_id = ? ");
            param.add(req.getStudentId());
        }
        // 学生ID列表
        if (CollectionUtil.isNotEmpty(req.getStudentIdList())) {
            sql.append(" and student_id in ? ");
            param.add(fromListToStr(req.getStudentIdList()));
        }
        // 用户ID
        if (req.getUserId() != null) {
            sql.append(" and user_id = ? ");
            param.add(req.getUserId());
        }
        // 用户ID列表
        if (CollectionUtil.isNotEmpty(req.getUserIdList())) {
            sql.append(" and user_id in ? ");
            param.add(fromListToStr(req.getUserIdList()));
        }
        // 用户类型：0-学生；1-家长
        if (req.getUserType() != null) {
            sql.append(" and user_type = ? ");
            param.add(req.getUserType());
        }
        if(req.getInFlag() != null){
            sql.append(" and in_flag = ? ");
            param.add(req.getInFlag());
        }

        sql.append(" order by id ");

        return jt.queryForBeans(YcqYjsStudentBindingUser.class, sql.toString(), param.toArray());
    }

    public String fromListToStr(List<BigInteger> list) {
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.toString().replace("[", "(").replace("]", ")").replace(" ", "").trim();
    }

    public List<StudentInfoByUserVo> searchStudentInfoByUser(BigInteger userId) {
        String sql = " select student_name,identity_card_no,yysbu.id, yys.id as student_id,yys.current_school_id,yys.current_school_name,  " +
                "yys.student_num,yysbu.in_flag,yys.current_class_name,yys.current_grade_name,yys.political_status,yys.political_status_type,yys.sex  " +
                "from YCQ_YJS_STUDENT yys " +
                "left join YCQ_YJS_STUDENT_BINDING_USER yysbu on yys.id = yysbu.student_id " +
                "where yys.del_flag = 0 and yysbu.del_flag = 0 and yysbu.user_id = ?  order by yysbu.id ";
        return jt.queryForBeans(StudentInfoByUserVo.class, sql, userId);
    }

    public Integer bindStudent(YcqYjsStudentBindingUser ycqYjsStudentBindingUser) {
        String sql = "insert ycq_yjs_student_binding_user(user_id,student_id,user_type,created_time,created_id,del_flag) values(?,?,?,?,?,?)";
        int update = jt.update(sql,ycqYjsStudentBindingUser.getUserId(),ycqYjsStudentBindingUser.getStudentId(),ycqYjsStudentBindingUser.getUserType(),ycqYjsStudentBindingUser.getCreatedTime(),ycqYjsStudentBindingUser.getCreatedId(),ycqYjsStudentBindingUser.getDelFlag());
        return update;
    }

    public void deleteBindStudentById(BigInteger id) {
        String sql = "update ycq_yjs_student_binding_user set del_flag = 1 where id = ?";
        jt.update(sql,id);
    }

    public StudentInfoByIdentityCardNoVo searchStudentInfoByIdentityCardNo(String identityCardNo) {
        String sql = "select yys.sex,yys.student_name,yys.current_school_name,yys.current_grade_name,yys.current_class_name,yyt.teacher_name as current_class_director_name,yys.id as student_id   from YCQ_YJS_STUDENT yys left join YCQ_YJS_TEACHER yyt on yys.current_class_director_id = yyt.id  where yys.del_flag = 0 and yys.identity_card_no = ? ";
        StudentInfoByIdentityCardNoVo studentInfoByIdentityCardNoVo = jt.queryForBean(StudentInfoByIdentityCardNoVo.class, sql, identityCardNo);
        return studentInfoByIdentityCardNoVo;
    }
}