package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentCourse;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourse.StudentCourseReq;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生课程表(YCQ_YJS_STUDENT_COURSE)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentCourseJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String BASE_SQL = "select * from YCQ_YJS_STUDENT_COURSE where del_flag = 0";

    private static final String SUM_COURSE_DURATION_SQL = "select sum(course_duration) from YCQ_YJS_STUDENT_COURSE where del_flag = 0 ";

    private static final String COUNT_SQL = "select count(*) from YCQ_YJS_STUDENT_COURSE where del_flag = 0";

    private static final String UPDATE_INFO_SQL = "update YCQ_YJS_STUDENT_COURSE t set t.residue_test_count=? where t.student_id= ? and t.paper_id= ? ";


    public PageRsp<YcqYjsStudentCourse> pageSearchStudentCourseInfo(PageReq<StudentCourseReq> req) {
        log.info("YcqYjsStudentCourseJdbcDao.pageSearchStudentCourseInfo.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        StringBuilder countSql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req.getQuery() != null) {
            // 学生ID
            if (req.getQuery().getStudentId() != null) {
                sql.append(" and student_id = ? ");
                countSql.append(" and student_id = ? ");
                param.add(req.getQuery().getStudentId());
            }
            // 专题ID
            if (req.getQuery().getSpecialId() != null) {
                sql.append(" and special_id = ? ");
                countSql.append(" and special_id = ? ");
                param.add(req.getQuery().getSpecialId());
            }
            // 专题类别
            if (StringUtils.isNotBlank(req.getQuery().getSpecialType())) {
                sql.append(" and special_type = ? ");
                countSql.append(" and special_type = ? ");
                param.add(req.getQuery().getSpecialType());
            }
        }

        sql.append(" order by id asc ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<YcqYjsStudentCourse> result;
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(YcqYjsStudentCourse.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<YcqYjsStudentCourse> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    public List<YcqYjsStudentCourse> selectStudentCourseInfoList(StudentCourseReq req) {
        log.info("YcqYjsStudentCourseJdbcDao.selectStudentCourseInfoList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 完成状态：0-未完成；1-已完成
            if (req.getCompleteStatus() != null) {
                sql.append(" and complete_status = ? ");
                param.add(req.getCompleteStatus());
            }
            // 专题ID
            if (req.getSpecialId() != null) {
                sql.append(" and special_id = ? ");
                param.add(req.getSpecialId());
            }
            // 专题类别
            if (StringUtils.isNotBlank(req.getSpecialType())) {
                sql.append(" and special_type = ? ");
                param.add(req.getSpecialType());
            }
            // 课程ID列表
            if (CollectionUtil.isNotEmpty(req.getCourseIdList())) {
                String itemSql = " and course_id in " + CommonUtil.fromListToStr(req.getCourseIdList());
                sql.append(itemSql);
            }
            // 专题ID列表
            if (CollectionUtil.isNotEmpty(req.getSpecialIdList())) {
                String itemSql = " and special_id in " + CommonUtil.fromListToStr(req.getSpecialIdList());
                sql.append(itemSql);
            }
        }
        sql.append(" order by id asc ");
        return jt.queryForBeans(YcqYjsStudentCourse.class, sql.toString(), param.toArray());
    }

    public Integer sumCourseDuration(StudentCourseReq req) {
        log.info("YcqYjsStudentCourseJdbcDao.sumCourseDuration.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(SUM_COURSE_DURATION_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 完成状态：0-未完成；1-已完成
            if (req.getCompleteStatus() != null) {
                sql.append(" and complete_status = ? ");
                param.add(req.getCompleteStatus());
            }
            // 专题ID
            if (req.getSpecialId() != null) {
                sql.append(" and special_id = ? ");
                param.add(req.getSpecialId());
            }
            // 专题类别
            if (StringUtils.isNotBlank(req.getSpecialType())) {
                sql.append(" and special_type = ? ");
                param.add(req.getSpecialType());
            }
            // 课程ID列表
            if (CollectionUtil.isNotEmpty(req.getCourseIdList())) {
                String itemSql = " and course_id in " + CommonUtil.fromListToStr(req.getCourseIdList());
                sql.append(itemSql);
            }
            // 专题ID列表
            if (CollectionUtil.isNotEmpty(req.getSpecialIdList())) {
                String itemSql = " and special_id in " + CommonUtil.fromListToStr(req.getSpecialIdList());
                sql.append(itemSql);
            }

            // 试卷id
            if (req.getPaperId() != null) {
                sql.append(" and paper_id = ? ");
                sql.append(req.getPaperId());
            }
        }
        return jt.queryForRawObject(Integer.class, sql.toString(), param.toArray());
    }

    public int updateStudentCourseById(YcqYjsStudentCourse ycqYjsStudentCourse) {
        int i =jt.update(UPDATE_INFO_SQL, new Object[]{ycqYjsStudentCourse.getResidueTestCount(), ycqYjsStudentCourse.getStudentId(), ycqYjsStudentCourse.getPaperId()});
        return i;
    }

}
