package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentCourseUploadVideo;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourseUploadVideo.StudentCourseUploadVideoReq;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生课程上传视频表(YCQ_YJS_STUDENT_COURSE_UPLOAD_VIDEO)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentCourseUploadVideoJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String SEECT_SQL = "select * from YCQ_YJS_STUDENT_COURSE_UPLOAD_VIDEO where del_flag = 0";

    private static final String COUNT_SQL = "select count(*) from YCQ_YJS_STUDENT_COURSE_UPLOAD_VIDEO where del_flag = 0";

    /**
     * 列表查询
     *
     * @param req
     * @return
     */
    public List<YcqYjsStudentCourseUploadVideo> selectList(StudentCourseUploadVideoReq req) {
        log.info("YcqYjsStudentCourseUploadVideoJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(SEECT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学生课程ID
            if (req.getStudentCourseId() != null) {
                sql.append(" and student_course_id = ? ");
                param.add(req.getStudentCourseId());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
        }
        sql.append(" order by id ");
        return jt.queryForBeans(YcqYjsStudentCourseUploadVideo.class, sql.toString(), param.toArray());
    }

    /**
     * 统计数量
     *
     * @param req
     * @return
     */
    public Integer count(StudentCourseUploadVideoReq req) {
        log.info("YcqYjsStudentCourseUploadVideoJdbcDao.count.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学生课程ID
            if (req.getStudentCourseId() != null) {
                sql.append(" and student_course_id = ? ");
                param.add(req.getStudentCourseId());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
        }
        return jt.queryForInt(sql.toString(), param.toArray());
    }

}