package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentHonorary;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryReq;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生荣誉表(YCQ_YJS_STUDENT_HONORARY)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentHonoraryJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String SEECT_SQL = "select * from YCQ_YJS_STUDENT_HONORARY where del_flag = 0";

    private static final String COUNT_SQL = "select count(*) from YCQ_YJS_STUDENT_HONORARY where del_flag = 0";

    /**
     * 列表查询学生荣誉数据
     *
     * @param req
     * @return
     */
    public List<YcqYjsStudentHonorary> selectList(StudentHonoraryReq req) {
        log.info("YcqYjsStudentHonoraryJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(SEECT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 荣誉称号类型
            if (req.getHonoraryTitleType() != null) {
                sql.append(" and honorary_title_type = ? ");
                param.add(req.getHonoraryTitleType());
            }
            // 学期类型
            if (req.getSemesterType() != null) {
                sql.append(" and semester_type = ? ");
                param.add(req.getSemesterType());
            }
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学校ID
            if (req.getSchoolId() != null) {
                sql.append(" and school_id = ? ");
                param.add(req.getSchoolId());
            }
            // 年级ID
            if (req.getGradeId() != null) {
                sql.append(" and grade_id = ? ");
                param.add(req.getGradeId());
            }
            // 班级ID
            if (req.getClassId() != null) {
                sql.append(" and class_id = ? ");
                param.add(req.getClassId());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
            // 学生ID列表
            if (CollectionUtil.isNotEmpty(req.getStudentIdList())) {
                String itemSql = " and student_id in " + CommonUtil.fromListToStr(req.getStudentIdList());
                sql.append(itemSql);
            }
        }
        sql.append(" order by created_time desc ");
        return jt.queryForBeans(YcqYjsStudentHonorary.class, sql.toString(), param.toArray());
    }

    /**
     * 统计学生荣誉数量
     *
     * @param req
     * @return
     */
    public Integer count(StudentHonoraryReq req) {
        log.info("YcqYjsStudentHonoraryJdbcDao.count.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 荣誉称号类型
            if (req.getHonoraryTitleType() != null) {
                sql.append(" and honorary_title_type = ? ");
                param.add(req.getHonoraryTitleType());
            }
            //荣誉称号以什么开头(校、区、市、全国)
            if (req.getHonoraryTitle() != null) {
                sql.append(" and honorary_title_type like ? ");
                param.add(req.getHonoraryTitle()+ "%");
            }

            // 学期类型
            if (req.getSemesterType() != null) {
                sql.append(" and semester_type = ? ");
                param.add(req.getSemesterType());
            }
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学校ID
            if (req.getSchoolId() != null) {
                sql.append(" and school_id = ? ");
                param.add(req.getSchoolId());
            }
            // 年级ID
            if (req.getGradeId() != null) {
                sql.append(" and grade_id = ? ");
                param.add(req.getGradeId());
            }
            // 班级ID
            if (req.getClassId() != null) {
                sql.append(" and class_id = ? ");
                param.add(req.getClassId());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
            // 学生ID列表
            if (CollectionUtil.isNotEmpty(req.getStudentIdList())) {
                String itemSql = " and student_id in " + CommonUtil.fromListToStr(req.getStudentIdList());
                sql.append(itemSql);
            }
            if(req.getHonoraryTitleType() != null){
                sql.append(" and honorary_level_type = ? ");
                param.add(req.getHonoraryTitleType());
            }
        }
        return jt.queryForInt(sql.toString(), param.toArray());
    }


    /**
     * 获取学生当前最新的荣誉称号
     * @param req
     * @return
     */
    public String honoraryTitleNow(StudentHonoraryReq req){
        StringBuilder sql = new StringBuilder(" select COALESCE(honorary_title,'') as honorary_title from YCQ_YJS_STUDENT_HONORARY where del_flag = 0 ");
        List<Object> param = new ArrayList<>();
        if (req != null) {

            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学校ID
            if (req.getSchoolId() != null) {
                sql.append(" and school_id = ? ");
                param.add(req.getSchoolId());
            }
            // 年级ID
            if (req.getGradeId() != null) {
                sql.append(" and grade_id = ? ");
                param.add(req.getGradeId());
            }
            // 班级ID
            if (req.getClassId() != null) {
                sql.append(" and class_id = ? ");
                param.add(req.getClassId());
            }

        }

        sql.append(" order by created_time desc limit 1 ");

        return jt.queryForRawObject(String.class,sql.toString(),param.toArray());
    }

}