package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudent;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.AppStudentArchivesVo;
import com.crunii.micro.service.oneaffair.dto.vo.student.StudentInfoVo;
import com.crunii.micro.service.oneaffair.dto.vo.student.StudentReq;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生表(YCQ_YJS_STUDENT)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String SEECT_SQL = "select * from ycq_yjs_student where del_flag = 0 ";

    private static final String LIST_BASE_SQL = "select distinct(s.id),s.*," +
            " (select count(1) from YCQ_YJS_STUDENT_HONORARY h where h.student_id = s.id and h.del_flag = 0) as studentHonoraryCount, " +
            " (select count(1) from YCQ_YJS_STUDENT_POSITION p where p.student_id = s.id and p.del_flag = 0) as studentPositionCount， " +
            " (select sum(co.course_duration) from YCQ_YJS_STUDENT_COURSE co where co.student_id = s.id and co.del_flag = 0 and co.complete_status = 1 and co.course_duration > 0 ) as courseDuration， " +
            " (select sum(d.total_time) from  YCQ_YJS_SERVICE_DURATION d where d.cert_no = s.identity_card_no and d.cert_type = '01') as serviceDuration, " +
            " (select count(distinct(m.id)) from  YCQ_YJS_STUDENT_MEDAL sm inner join ycq_yjs_medal m on sm.medal_id = m.id where sm.student_id = s.id and sm.del_flag = 0 and m.medal_type = 2 and sm.if_light = 1 and m.del_flag = 0) as xjMedalCount " +
            " from ycq_yjs_student s " +
            " where s.del_flag = 0 ";

    private static final String PAGE_BASE_SQL = "select distinct(s.id),s.*," +
            " (select count(1) from YCQ_YJS_STUDENT_HONORARY h where h.student_id = s.id and h.del_flag = 0) as studentHonoraryCount, " +
            " (select count(1) from YCQ_YJS_STUDENT_POSITION p where p.student_id = s.id and p.del_flag = 0) as studentPositionCount， " +
            " (select sum(co.course_duration) from YCQ_YJS_STUDENT_COURSE co where co.student_id = s.id and co.del_flag = 0 and co.complete_status = 1 and co.course_duration > 0 ) as courseDuration， " +
            " (select sum(d.total_time) from  YCQ_YJS_SERVICE_DURATION d where d.cert_no = s.identity_card_no and d.cert_type = '01') as serviceDuration, " +
            " (select count(distinct(m.id)) from  YCQ_YJS_STUDENT_MEDAL sm inner join ycq_yjs_medal m on sm.medal_id = m.id where sm.student_id = s.id and sm.del_flag = 0 and m.medal_type = 2 and sm.if_light = 1 and m.del_flag = 0) as xjMedalCount " +
            " from ycq_yjs_student s " +
            " inner join tsm_worker_org_all twoa on s.current_class_id = twoa.org_id " +
            " where s.del_flag = 0 ";

    private static final String PAGE_COUNT_SQL = "select count(distinct(s.id)) " +
            " from ycq_yjs_student s " +
            " inner join tsm_worker_org_all twoa on s.current_class_id = twoa.org_id " +
            " where s.del_flag = 0 ";

    /**
     * 列表查询学生数据
     *
     * @param req
     * @return
     */
    public List<StudentInfoVo> selectList(StudentReq req) {
        log.info("YcqYjsStudentJdbcDao.selectList.param：{}", req.toString());
        StringBuilder sql = new StringBuilder(LIST_BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生编号
            if (StringUtils.isNotBlank(req.getStudentNum())) {
                sql.append(" and s.student_num like ? ");
                param.add("%" + req.getStudentNum() + "%");
            }
            // 学生名称
            if (StringUtils.isNotBlank(req.getStudentName())) {
                sql.append(" and s.student_name like ? ");
                param.add("%" + req.getStudentName() + "%");
            }
            // 身份证号
            if (StringUtils.isNotBlank(req.getIdentityCardNo())) {
                sql.append(" and s.identity_card_no like ? ");
                param.add("%" + req.getIdentityCardNo() + "%");
            }
            // 手机号
            if (StringUtils.isNotBlank(req.getMobileLike())) {
                sql.append(" and s.mobile like ? ");
                param.add("%" + req.getMobileLike() + "%");
            }
            // 手机号
            if (StringUtils.isNotBlank(req.getMobile())) {
                sql.append(" and s.mobile = ? ");
                param.add(req.getMobile());
            }
            // 学校名称
            if (StringUtils.isNotBlank(req.getCurrentSchoolName())) {
                sql.append(" and s.current_school_name like ? ");
                param.add("%" + req.getCurrentSchoolName() + "%");
            }
            // 年级名称
            if (StringUtils.isNotBlank(req.getCurrentGradeName())) {
                sql.append(" and s.current_grade_name like ? ");
                param.add("%" + req.getCurrentGradeName() + "%");
            }
            // 班级名称
            if (StringUtils.isNotBlank(req.getCurrentClassName())) {
                sql.append(" and s.current_class_name like ? ");
                param.add("%" + req.getCurrentClassName() + "%");
            }
            // 班主任名称
            if (StringUtils.isNotBlank(req.getCurrentClassDirectorName())) {
                sql.append(" and s.current_class_director_name like ? ");
                param.add("%" + req.getCurrentClassDirectorName() + "%");
            }
            // 政治面貌
            if (req.getPoliticalStatusType() != null) {
                sql.append(" and s.political_status_type = ? ");
                param.add(req.getPoliticalStatusType());
            }
            // ID
            if (req.getId() != null) {
                sql.append(" and s.id = ? ");
                param.add(req.getId());
            }
            // 学校ID
            if (req.getCurrentSchoolId() != null) {
                sql.append(" and s.current_school_id = ? ");
                param.add(req.getCurrentSchoolId());
            }
            // 年级ID
            if (req.getCurrentGradeId() != null) {
                sql.append(" and s.current_grade_id = ? ");
                param.add(req.getCurrentGradeId());
            }
            // 班级ID
            if (req.getCurrentClassId() != null) {
                sql.append(" and s.current_class_id = ? ");
                param.add(req.getCurrentClassId());
            }
            // 班主任ID
            if (req.getCurrentClassDirectorId() != null) {
                sql.append(" and s.current_class_director_id = ? ");
                param.add(req.getCurrentClassDirectorId());
            }
            // 创建时间
            if (req.getStartCreatedTime() != null) {
                sql.append(" and s.created_time >= ? ");
                param.add(req.getStartCreatedTime());
            }
            if (req.getEndCreatedTime() != null) {
                sql.append(" and  ?  >= s.created_time ");
                param.add(req.getEndCreatedTime());
            }
            // 入团时间
            if (req.getStartJoinYouthLeagueTime() != null) {
                sql.append(" and s.join_youth_league_time >= ? ");
                param.add(req.getStartJoinYouthLeagueTime());
            }
            if (req.getEndJoinYouthLeagueTime() != null) {
                sql.append(" and  ?  >= s.join_youth_league_time ");
                param.add(req.getEndJoinYouthLeagueTime());
            }
            // 评分时间
            if (req.getStartScoreTime() != null) {
                sql.append(" and s.score_time >= ? ");
                param.add(req.getStartScoreTime());
            }
            if (req.getEndScoreTime() != null) {
                sql.append(" and  ?  >= s.score_time ");
                param.add(req.getEndScoreTime());
            }
            // 入队时间
            if (req.getStartJoinYoungPioneersTime() != null) {
                sql.append(" and s.join_young_pioneers_time >= ? ");
                param.add(req.getStartJoinYoungPioneersTime());
            }
            if (req.getEndJoinYoungPioneersTime() != null) {
                sql.append(" and  ?  >= s.join_young_pioneers_time ");
                param.add(req.getEndJoinYoungPioneersTime());
            }
            // 入党时间
            if (req.getStartJoinCommunistPartyTime() != null) {
                sql.append(" and s.join_communist_party_time >= ? ");
                param.add(req.getStartJoinCommunistPartyTime());
            }
            if (req.getEndJoinCommunistPartyTime() != null) {
                sql.append(" and  ?  >= s.join_communist_party_time ");
                param.add(req.getEndJoinCommunistPartyTime());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and s.id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
            // 学校ID列表
            if (CollectionUtil.isNotEmpty(req.getCurrentSchoolIdList())) {
                String itemSql = " and s.current_school_id in " + CommonUtil.fromListToStr(req.getCurrentSchoolIdList());
                sql.append(itemSql);
            }
            // 年级ID列表
            if (CollectionUtil.isNotEmpty(req.getCurrentGradeIdList())) {
                String itemSql = " and s.current_grade_id in " + CommonUtil.fromListToStr(req.getCurrentGradeIdList());
                sql.append(itemSql);
            }
            // 班级ID列表
            if (CollectionUtil.isNotEmpty(req.getCurrentClassIdList())) {
                String itemSql = " and s.current_class_id in " + CommonUtil.fromListToStr(req.getCurrentClassIdList());
                sql.append(itemSql);
            }
            // 班主任ID列表
            if (CollectionUtil.isNotEmpty(req.getCurrentClassDirectorIdList())) {
                String itemSql = " and s.current_class_director_id in " + CommonUtil.fromListToStr(req.getCurrentClassDirectorIdList());
                sql.append(itemSql);
            }
        }

        sql.append(" order by s.id desc");
        return jt.queryForBeans(StudentInfoVo.class, sql.toString(), param.toArray());
    }

    /**
     * 分页查询学生数据
     *
     * @param req
     * @return
     */
    public PageRsp<StudentInfoVo> pageSearchStudentInfo(PageReq<StudentReq> req) {
        log.info("YcqYjsStudentJdbcDao.pageSearchStudentInfo.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        // 数据权限
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        sql.append(" and twoa.worker_id = ? ");
        countSql.append(" and twoa.worker_id = ? ");
        param.add(currWorker.getLoginId());

        if (req != null && req.getQuery() != null) {
            // 学生编号
            if (StringUtils.isNotBlank(req.getQuery().getStudentNum())) {
                sql.append(" and s.student_num like ? ");
                countSql.append(" and s.student_num like ? ");
                param.add("%" + req.getQuery().getStudentNum() + "%");
            }
            // 学生名称
            if (StringUtils.isNotBlank(req.getQuery().getStudentName())) {
                sql.append(" and s.student_name like ? ");
                countSql.append(" and s.student_name like ? ");
                param.add("%" + req.getQuery().getStudentName() + "%");
            }
            // 中队名称-精准匹配
            if (StringUtils.isNotBlank(req.getQuery().getMiddleTeamName())) {
                sql.append(" and s.middle_team_name = ? ");
                countSql.append(" and s.middle_team_name = ? ");
                param.add(req.getQuery().getMiddleTeamName());
            }
            // 中队名称-模糊匹配
            if (StringUtils.isNotBlank(req.getQuery().getMiddleTeamNameLike())) {
                sql.append(" and s.middle_team_name like ? ");
                countSql.append(" and s.middle_team_name like ? ");
                param.add("%" + req.getQuery().getMiddleTeamNameLike() + "%");
            }
            // 多个学生名称列表-模糊搜索
            if (CollectionUtil.isNotEmpty(req.getQuery().getStudentNameLikeList())) {
                List<String> studentNameLikeList = req.getQuery().getStudentNameLikeList();
                sql.append(" and ( ");
                countSql.append(" and ( ");
                int count = 0;
                for (String studentNameLike : studentNameLikeList) {
                    if (count == 0) {
                        sql.append(" s.student_name like ? ");
                        countSql.append(" s.student_name like ? ");
                    } else {
                        sql.append(" or s.student_name like ? ");
                        countSql.append(" or s.student_name like ? ");
                    }
                    param.add("%" + studentNameLike + "%");
                    count++;
                }
                sql.append(" ) ");
                countSql.append(" ) ");
            }
            // 身份证号
            if (StringUtils.isNotBlank(req.getQuery().getIdentityCardNo())) {
                sql.append(" and s.identity_card_no like ? ");
                countSql.append(" and s.identity_card_no like ? ");
                param.add("%" + req.getQuery().getIdentityCardNo() + "%");
            }
            // 学校名称
            if (StringUtils.isNotBlank(req.getQuery().getCurrentSchoolName())) {
                sql.append(" and s.current_school_name like ? ");
                countSql.append(" and s.current_school_name like ? ");
                param.add("%" + req.getQuery().getCurrentSchoolName() + "%");
            }
            // 年级名称
            if (StringUtils.isNotBlank(req.getQuery().getCurrentGradeName())) {
                sql.append(" and s.current_grade_name like ? ");
                countSql.append(" and s.current_grade_name like ? ");
                param.add("%" + req.getQuery().getCurrentGradeName() + "%");
            }
            // 班级名称
            if (StringUtils.isNotBlank(req.getQuery().getCurrentClassName())) {
                sql.append(" and s.current_class_name like ? ");
                countSql.append(" and s.current_class_name like ? ");
                param.add("%" + req.getQuery().getCurrentClassName() + "%");
            }
            // 班主任名称
            if (StringUtils.isNotBlank(req.getQuery().getCurrentClassDirectorName())) {
                sql.append(" and s.current_class_director_name like ? ");
                countSql.append(" and s.current_class_director_name like ? ");
                param.add("%" + req.getQuery().getCurrentClassDirectorName() + "%");
            }
            // 政治面貌
            if (req.getQuery().getPoliticalStatusType() != null) {
                sql.append(" and s.political_status_type = ? ");
                countSql.append(" and s.political_status_type = ? ");
                param.add(req.getQuery().getPoliticalStatusType());
            }
            // ID
            if (req.getQuery().getId() != null) {
                sql.append(" and s.id = ? ");
                countSql.append(" and s.id = ? ");
                param.add(req.getQuery().getId());
            }
            // 学校ID
            if (req.getQuery().getCurrentSchoolId() != null) {
                sql.append(" and s.current_school_id = ? ");
                countSql.append(" and s.current_school_id = ? ");
                param.add(req.getQuery().getCurrentSchoolId());
            }
            // 年级ID
            if (req.getQuery().getCurrentGradeId() != null) {
                sql.append(" and s.current_grade_id = ? ");
                countSql.append(" and s.current_grade_id = ? ");
                param.add(req.getQuery().getCurrentGradeId());
            }
            // 班级ID
            if (req.getQuery().getCurrentClassId() != null) {
                sql.append(" and s.current_class_id = ? ");
                countSql.append(" and s.current_class_id = ? ");
                param.add(req.getQuery().getCurrentClassId());
            }
            // 班主任ID
            if (req.getQuery().getCurrentClassDirectorId() != null) {
                sql.append(" and s.current_class_director_id = ? ");
                countSql.append(" and s.current_class_director_id = ? ");
                param.add(req.getQuery().getCurrentClassDirectorId());
            }
            // 创建时间
            if (req.getQuery().getStartCreatedTime() != null) {
                sql.append(" and s.created_time >= ? ");
                countSql.append(" and s.created_time >= ? ");
                param.add(req.getQuery().getStartCreatedTime());
            }
            if (req.getQuery().getEndCreatedTime() != null) {
                sql.append(" and  ?  >= s.created_time ");
                countSql.append(" and  ?  >= s.created_time ");
                param.add(req.getQuery().getEndCreatedTime());
            }
            // 入团时间
            if (req.getQuery().getStartJoinYouthLeagueTime() != null) {
                sql.append(" and s.join_youth_league_time >= ? ");
                countSql.append(" and s.join_youth_league_time >= ? ");
                param.add(req.getQuery().getStartJoinYouthLeagueTime());
            }
            if (req.getQuery().getEndJoinYouthLeagueTime() != null) {
                sql.append(" and  ?  >= s.join_youth_league_time ");
                countSql.append(" and  ?  >= s.join_youth_league_time ");
                param.add(req.getQuery().getEndJoinYouthLeagueTime());
            }
            // 评分时间
            if (req.getQuery().getStartScoreTime() != null) {
                sql.append(" and s.score_time >= ? ");
                countSql.append(" and s.score_time >= ? ");
                param.add(req.getQuery().getStartScoreTime());
            }
            if (req.getQuery().getEndScoreTime() != null) {
                sql.append(" and  ?  >= s.score_time ");
                countSql.append(" and  ?  >= s.score_time ");
                param.add(req.getQuery().getEndScoreTime());
            }
            // 入队时间
            if (req.getQuery().getStartJoinYoungPioneersTime() != null) {
                sql.append(" and s.join_young_pioneers_time >= ? ");
                countSql.append(" and s.join_young_pioneers_time >= ? ");
                param.add(req.getQuery().getStartJoinYoungPioneersTime());
            }
            if (req.getQuery().getEndJoinYoungPioneersTime() != null) {
                sql.append(" and  ?  >= s.join_young_pioneers_time ");
                countSql.append(" and  ?  >= s.join_young_pioneers_time ");
                param.add(req.getQuery().getEndJoinYoungPioneersTime());
            }
            // 入党时间
            if (req.getQuery().getStartJoinCommunistPartyTime() != null) {
                sql.append(" and s.join_communist_party_time >= ? ");
                countSql.append(" and s.join_communist_party_time >= ? ");
                param.add(req.getQuery().getStartJoinCommunistPartyTime());
            }
            if (req.getQuery().getEndJoinCommunistPartyTime() != null) {
                sql.append(" and  ?  >= s.join_communist_party_time ");
                countSql.append(" and  ?  >= s.join_communist_party_time ");
                param.add(req.getQuery().getEndJoinCommunistPartyTime());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getIdList())) {
                String itemSql = " and s.id in " + CommonUtil.fromListToStr(req.getQuery().getIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 学校ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getCurrentSchoolIdList())) {
                String itemSql = " and s.current_school_id in " + CommonUtil.fromListToStr(req.getQuery().getCurrentSchoolIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 年级ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getCurrentGradeIdList())) {
                String itemSql = " and s.current_grade_id in " + CommonUtil.fromListToStr(req.getQuery().getCurrentGradeIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 班级ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getCurrentClassIdList())) {
                String itemSql = " and s.current_class_id in " + CommonUtil.fromListToStr(req.getQuery().getCurrentClassIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 数据权限班级ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getBaseClassIds())) {
                String itemSql = " and s.current_class_id in " + CommonUtil.fromListToStr(req.getQuery().getBaseClassIds());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 班主任ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getCurrentClassDirectorIdList())) {
                String itemSql = " and s.current_class_director_id in " + CommonUtil.fromListToStr(req.getQuery().getCurrentClassDirectorIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }

        }
        sql.append(" order by s.id desc ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<StudentInfoVo> result;
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(StudentInfoVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<StudentInfoVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    public List<YcqYjsStudent> selectListByIdentityCardNo(String identityCardNo) {
        if (StringUtils.isBlank(identityCardNo)) {
            return null;
        }
        String sql = SEECT_SQL + " and identity_card_no = ? ";
        return jt.query(sql, new BeanPropertyRowMapper<>(YcqYjsStudent.class), identityCardNo);
    }

    public List<YcqYjsStudent> selectListByMobile(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return null;
        }
        String sql = SEECT_SQL + " and mobile = ? ";
        return jt.query(sql, new BeanPropertyRowMapper<>(YcqYjsStudent.class), mobile);
    }

    /**
     * 列表查询学生数据
     *
     * @param req
     * @return
     */
    public List<YcqYjsStudent> selectInfoList(StudentReq req) {
        log.info("YcqYjsStudentJdbcDao.selectInfoList.param：{}", req.toString());
        StringBuilder sql = new StringBuilder(SEECT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生编号
            if (StringUtils.isNotBlank(req.getStudentNum())) {
                sql.append(" and student_num like ? ");
                param.add("%" + req.getStudentNum() + "%");
            }
            // 学生名称
            if (StringUtils.isNotBlank(req.getStudentName())) {
                sql.append(" and student_name like ? ");
                param.add("%" + req.getStudentName() + "%");
            }
            // 身份证号
            if (StringUtils.isNotBlank(req.getIdentityCardNo())) {
                sql.append(" and identity_card_no like ? ");
                param.add("%" + req.getIdentityCardNo() + "%");
            }
            // 手机号
            if (StringUtils.isNotBlank(req.getMobileLike())) {
                sql.append(" and mobile like ? ");
                param.add("%" + req.getMobileLike() + "%");
            }
            // 手机号
            if (StringUtils.isNotBlank(req.getMobile())) {
                sql.append(" and mobile = ? ");
                param.add(req.getMobile());
            }
            // 学校名称
            if (StringUtils.isNotBlank(req.getCurrentSchoolName())) {
                sql.append(" and current_school_name like ? ");
                param.add("%" + req.getCurrentSchoolName() + "%");
            }
            // 年级名称
            if (StringUtils.isNotBlank(req.getCurrentGradeName())) {
                sql.append(" and current_grade_name like ? ");
                param.add("%" + req.getCurrentGradeName() + "%");
            }
            // 班级名称
            if (StringUtils.isNotBlank(req.getCurrentClassName())) {
                sql.append(" and current_class_name like ? ");
                param.add("%" + req.getCurrentClassName() + "%");
            }
            // 班主任名称
            if (StringUtils.isNotBlank(req.getCurrentClassDirectorName())) {
                sql.append(" and current_class_director_name like ? ");
                param.add("%" + req.getCurrentClassDirectorName() + "%");
            }
            // 政治面貌
            if (req.getPoliticalStatusType() != null) {
                sql.append(" and political_status_type = ? ");
                param.add(req.getPoliticalStatusType());
            }
            // ID
            if (req.getId() != null) {
                sql.append(" and id = ? ");
                param.add(req.getId());
            }
            // 学校ID
            if (req.getCurrentSchoolId() != null) {
                sql.append(" and current_school_id = ? ");
                param.add(req.getCurrentSchoolId());
            }
            // 年级ID
            if (req.getCurrentGradeId() != null) {
                sql.append(" and current_grade_id = ? ");
                param.add(req.getCurrentGradeId());
            }
            // 班级ID
            if (req.getCurrentClassId() != null) {
                sql.append(" and current_class_id = ? ");
                param.add(req.getCurrentClassId());
            }
            // 班主任ID
            if (req.getCurrentClassDirectorId() != null) {
                sql.append(" and current_class_director_id = ? ");
                param.add(req.getCurrentClassDirectorId());
            }
            // 创建时间
            if (req.getStartCreatedTime() != null) {
                sql.append(" and created_time >= ? ");
                param.add(req.getStartCreatedTime());
            }
            if (req.getEndCreatedTime() != null) {
                sql.append(" and  ?  >= created_time ");
                param.add(req.getEndCreatedTime());
            }
            // 入团时间
            if (req.getStartJoinYouthLeagueTime() != null) {
                sql.append(" and join_youth_league_time >= ? ");
                param.add(req.getStartJoinYouthLeagueTime());
            }
            if (req.getEndJoinYouthLeagueTime() != null) {
                sql.append(" and  ?  >= join_youth_league_time ");
                param.add(req.getEndJoinYouthLeagueTime());
            }
            // 评分时间
            if (req.getStartScoreTime() != null) {
                sql.append(" and score_time >= ? ");
                param.add(req.getStartScoreTime());
            }
            if (req.getEndScoreTime() != null) {
                sql.append(" and  ?  >= score_time ");
                param.add(req.getEndScoreTime());
            }
            // 入队时间
            if (req.getStartJoinYoungPioneersTime() != null) {
                sql.append(" and join_young_pioneers_time >= ? ");
                param.add(req.getStartJoinYoungPioneersTime());
            }
            if (req.getEndJoinYoungPioneersTime() != null) {
                sql.append(" and  ?  >= join_young_pioneers_time ");
                param.add(req.getEndJoinYoungPioneersTime());
            }
            // 入党时间
            if (req.getStartJoinCommunistPartyTime() != null) {
                sql.append(" and join_communist_party_time >= ? ");
                param.add(req.getStartJoinCommunistPartyTime());
            }
            if (req.getEndJoinCommunistPartyTime() != null) {
                sql.append(" and  ?  >= join_communist_party_time ");
                param.add(req.getEndJoinCommunistPartyTime());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
            // 学校ID列表
            if (CollectionUtil.isNotEmpty(req.getCurrentSchoolIdList())) {
                String itemSql = " and current_school_id in " + CommonUtil.fromListToStr(req.getCurrentSchoolIdList());
                sql.append(itemSql);
            }
            // 年级ID列表
            if (CollectionUtil.isNotEmpty(req.getCurrentGradeIdList())) {
                String itemSql = " and current_grade_id in " + CommonUtil.fromListToStr(req.getCurrentGradeIdList());
                sql.append(itemSql);
            }
            // 班级ID列表
            if (CollectionUtil.isNotEmpty(req.getCurrentClassIdList())) {
                String itemSql = " and current_class_id in " + CommonUtil.fromListToStr(req.getCurrentClassIdList());
                sql.append(itemSql);
            }
            // 班主任ID列表
            if (CollectionUtil.isNotEmpty(req.getCurrentClassDirectorIdList())) {
                String itemSql = " and current_class_director_id in " + CommonUtil.fromListToStr(req.getCurrentClassDirectorIdList());
                sql.append(itemSql);
            }
            // 身份证号列表
            if (CollectionUtil.isNotEmpty(req.getIdentityCardNoList())) {
                String itemSql = " and identity_card_no in " + CommonUtil.fromStrListToStr2(req.getIdentityCardNoList());
                sql.append(itemSql);
            }
            // 手机号列表
            if (CollectionUtil.isNotEmpty(req.getMobileList())) {
                String itemSql = " and mobile in " + CommonUtil.fromStrListToStr2(req.getMobileList());
                sql.append(itemSql);
            }
        }

        sql.append(" order by id desc");
        return jt.queryForBeans(YcqYjsStudent.class, sql.toString(), param.toArray());
    }

    public AppStudentArchivesVo selectStudentById(BigInteger id) {
        StringBuilder sql = new StringBuilder(" select  yys.student_name,yys.current_grade_name,yys.current_class_name,yys.join_young_pioneers_time,");
        sql.append("yys.if_join_youth_Pioneers,yys.if_join_youth_league, yys.join_youth_league_time,yys.if_join_communist_party,");
        sql.append("yys.join_communist_party_time ,yyt.teacher_name   ");
        sql.append("from YCQ_YJS_STUDENT yys left join YCQ_YJS_TEACHER yyt on yys.current_class_director_id = yyt.id ");
        sql.append(" where yys.del_flag = 0  and yys.id = ? ");
        AppStudentArchivesVo appStudentArchivesVo = jt.queryForBean(AppStudentArchivesVo.class, sql.toString(), id);
        if (appStudentArchivesVo != null) {
            if (appStudentArchivesVo.getIfJoinYouthPioneers() != null && appStudentArchivesVo.getIfJoinYouthPioneers() == 1 && appStudentArchivesVo.getJoinYoungPioneersTime() != null) {
                appStudentArchivesVo.setYouthPioneersAge(DateUtil.ageOfNow(appStudentArchivesVo.getJoinYoungPioneersTime()));
            }
            if (appStudentArchivesVo.getIfJoinYouthLeague() != null && appStudentArchivesVo.getIfJoinYouthLeague() == 1 && appStudentArchivesVo.getJoinYouthLeagueTime() != null) {
                appStudentArchivesVo.setLeagueAge(DateUtil.ageOfNow(appStudentArchivesVo.getJoinYouthLeagueTime()));
            }
            if (appStudentArchivesVo.getIfJoinCommunistParty() != null && appStudentArchivesVo.getIfJoinCommunistParty() == 1 && appStudentArchivesVo.getJoinCommunistPartyTime() != null) {
                appStudentArchivesVo.setPartyAge(DateUtil.ageOfNow(appStudentArchivesVo.getJoinCommunistPartyTime()));
            }

        }
        return appStudentArchivesVo;
    }


}