package com.crunii.micro.service.oneaffair.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentMedalActivity;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedalActivity.StudentMedalActivityReq;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生奖章活动完成情况表(YCQ_YJS_STUDENT_MEDAL_ACTIVITY)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentMedalActivityJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String BASE_SQL = "select * from YCQ_YJS_STUDENT_MEDAL_ACTIVITY where del_flag = 0";

    private static final String COUNT_SQL = "select count(*) from YCQ_YJS_STUDENT_MEDAL_ACTIVITY where del_flag = 0";

    /**
     * 列表查询学生奖章活动数据
     *
     * @param req
     * @return
     */
    public List<YcqYjsStudentMedalActivity> selectList(StudentMedalActivityReq req) {
        log.info("YcqYjsStudentMedalActivityJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生奖章ID
            if (req.getStudentMedalId() != null) {
                sql.append(" and student_medal_id = ? ");
                param.add(req.getStudentMedalId());
            }
            // 奖章关联活动ID
            if (req.getMedalActivityId() != null) {
                sql.append(" and medal_activity_id = ? ");
                param.add(req.getMedalActivityId());
            }
            // 完成状态：0-未完成；1-已完成
            if (req.getCompleteStatus() != null) {
                sql.append(" and complete_status = ? ");
                param.add(req.getCompleteStatus());
            }
        }
        sql.append(" order by medal_activity_id ");
        return jt.queryForBeans(YcqYjsStudentMedalActivity.class, sql.toString(), param.toArray());
    }

    /**
     * 统计学生奖章活动数量
     *
     * @param req
     * @return
     */
    public Integer count(StudentMedalActivityReq req) {
        log.info("YcqYjsStudentMedalActivityJdbcDao.count.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生奖章ID
            if (req.getStudentMedalId() != null) {
                sql.append(" and student_medal_id = ? ");
                param.add(req.getStudentMedalId());
            }
            // 奖章关联活动ID
            if (req.getMedalActivityId() != null) {
                sql.append(" and medal_activity_id = ? ");
                param.add(req.getMedalActivityId());
            }
            // 完成状态：0-未完成；1-已完成
            if (req.getCompleteStatus() != null) {
                sql.append(" and complete_status = ? ");
                param.add(req.getCompleteStatus());
            }
        }
        return jt.queryForInt(sql.toString(), param.toArray());
    }

}
