package com.crunii.micro.service.oneaffair.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentMedalRule;
import com.crunii.micro.service.oneaffair.dto.vo.medalRule.StudentMedalRuleReq;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生争章规则完成情况表(YCQ_YJS_STUDENT_MEDAL_RULE)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentMedalRuleJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String BASE_SQL = "select * from YCQ_YJS_STUDENT_MEDAL_RULE where del_flag = 0";

    private static final String COUNT_SQL = "select count(*) from YCQ_YJS_STUDENT_MEDAL_RULE where del_flag = 0";

    /**
     * 列表查询学生争章规则数据
     *
     * @param req
     * @return
     */
    public List<YcqYjsStudentMedalRule> selectList(StudentMedalRuleReq req) {
        log.info("YcqYjsStudentMedalRuleJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生奖章ID
            if (req.getStudentMedalId() != null) {
                sql.append(" and student_medal_id = ? ");
                param.add(req.getStudentMedalId());
            }
            // 争章规则ID
            if (req.getMedalRuleId() != null) {
                sql.append(" and medal_rule_id = ? ");
                param.add(req.getMedalRuleId());
            }
            // 完成状态：0-未完成；1-已完成
            if (req.getCompleteStatus() != null) {
                sql.append(" and complete_status = ? ");
                param.add(req.getCompleteStatus());
            }
        }
        sql.append(" order by medal_rule_id ");
        return jt.queryForBeans(YcqYjsStudentMedalRule.class, sql.toString(), param.toArray());
    }

    /**
     * 统计学生争章规则数量
     *
     * @param req
     * @return
     */
    public Integer count(StudentMedalRuleReq req) {
        log.info("YcqYjsStudentMedalRuleJdbcDao.count.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生奖章ID
            if (req.getStudentMedalId() != null) {
                sql.append(" and student_medal_id = ? ");
                param.add(req.getStudentMedalId());
            }
            // 争章规则ID
            if (req.getMedalRuleId() != null) {
                sql.append(" and medal_rule_id = ? ");
                param.add(req.getMedalRuleId());
            }
            // 完成状态：0-未完成；1-已完成
            if (req.getCompleteStatus() != null) {
                sql.append(" and complete_status = ? ");
                param.add(req.getCompleteStatus());
            }
        }
        return jt.queryForInt(sql.toString(), param.toArray());
    }

}
