package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentPatriarch;
import com.crunii.micro.service.oneaffair.dto.vo.studentPatriarch.StudentPatriarchReq;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生家长表(YCQ_YJS_STUDENT_PATRIARCH)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentPatriarchJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String SEECT_SQL = "select * from YCQ_YJS_STUDENT_PATRIARCH where del_flag = 0";

    private static final String COUNT_SQL = "select count(*) from YCQ_YJS_STUDENT_PATRIARCH where del_flag = 0";

    /**
     * 列表查询学生家长数据
     *
     * @param req
     * @return
     */
    public List<YcqYjsStudentPatriarch> selectList(StudentPatriarchReq req) {
        log.info("YcqYjsStudentPatriarchJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(SEECT_SQL);
        List<Object> param = new ArrayList<>();
        if(req != null){
            // 主键
            if (req.getId() != null) {
                sql.append(" and id = ? ");
                param.add(req.getId());
            }
            // id列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                sql.append(" and id in " + CommonUtil.fromListToStr(req.getIdList()));
            }
            // 手机号-模糊查询
            if (StringUtils.isNotBlank(req.getMobileLike())) {
                sql.append(" and mobile like ? ");
                param.add("%" + req.getMobileLike() + "%");
            }
            // 手机号
            if (StringUtils.isNotBlank(req.getMobile())) {
                sql.append(" and mobile = ? ");
                param.add(req.getMobile());
            }
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学生ID列表
            if (CollectionUtil.isNotEmpty(req.getStudentIdList())) {
                sql.append(" and student_id in " + CommonUtil.fromListToStr(req.getStudentIdList()));
            }
            // 家长角色
            if (req.getPatriarchRole() != null) {
                sql.append(" and patriarch_role = ? ");
                param.add(req.getPatriarchRole());
            }
            // 家长姓名
            if (StringUtils.isNotBlank(req.getPatriarchName())) {
                sql.append(" and patriarch_name = ? ");
                param.add(req.getPatriarchName());
            }
            // 家长姓名-模糊查询
            if (StringUtils.isNotBlank(req.getPatriarchNameLike())) {
                sql.append(" and patriarch_name like ? ");
                param.add("%" + req.getPatriarchNameLike() + "%");
            }
        }
        sql.append(" order by id ");
        return jt.queryForBeans(YcqYjsStudentPatriarch.class, sql.toString(), param.toArray());
    }

    /**
     * 统计学生家长数量
     *
     * @param req
     * @return
     */
    public Integer count(StudentPatriarchReq req) {
        log.info("YcqYjsStudentPatriarchJdbcDao.count.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 主键
            if (req.getId() != null) {
                sql.append(" and id = ? ");
                param.add(req.getId());
            }
            // id列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                sql.append(" and id in " + CommonUtil.fromListToStr(req.getIdList()));
            }
            // 手机号-模糊查询
            if (StringUtils.isNotBlank(req.getMobileLike())) {
                sql.append(" and mobile like ? ");
                param.add("%" + req.getMobileLike() + "%");
            }
            // 手机号
            if (StringUtils.isNotBlank(req.getMobile())) {
                sql.append(" and mobile = ? ");
                param.add(req.getMobile());
            }
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学生ID列表
            if (CollectionUtil.isNotEmpty(req.getStudentIdList())) {
                sql.append(" and student_id in " + CommonUtil.fromListToStr(req.getStudentIdList()));
            }
            // 家长角色
            if (req.getPatriarchRole() != null) {
                sql.append(" and patriarch_role = ? ");
                param.add(req.getPatriarchRole());
            }
            // 家长姓名
            if (StringUtils.isNotBlank(req.getPatriarchName())) {
                sql.append(" and patriarch_name = ? ");
                param.add(req.getPatriarchName());
            }
            // 家长姓名-模糊查询
            if (StringUtils.isNotBlank(req.getPatriarchNameLike())) {
                sql.append(" and patriarch_name like ? ");
                param.add("%" + req.getPatriarchNameLike() + "%");
            }
        }
        return jt.queryForInt(sql.toString(), param.toArray());
    }

}