package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentPosition;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.StudentPositionReq;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生岗位表(YCQ_YJS_STUDENT_POSITION)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentPositionJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String SEECT_SQL = "select * from YCQ_YJS_STUDENT_POSITION where del_flag = 0";

    private static final String COUNT_SQL = "select count(*) from YCQ_YJS_STUDENT_POSITION where del_flag = 0";

    /**
     * 列表查询学生岗位数据
     *
     * @param req
     * @return
     */
    public List<YcqYjsStudentPosition> selectList(StudentPositionReq req) {
        log.info("YcqYjsStudentPositionJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(SEECT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生岗位类型
            if (req.getStudentPositionType() != null) {
                sql.append(" and student_position_type = ? ");
                param.add(req.getStudentPositionType());
            }
            // 学生岗位分类类型
            if (req.getStudentPositionClassType() != null) {
                sql.append(" and student_position_class_type = ? ");
                param.add(req.getStudentPositionClassType());
            }
            // 学期类型
            if (req.getSemesterType() != null) {
                sql.append(" and semester_type = ? ");
                param.add(req.getSemesterType());
            }
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学校ID
            if (req.getSchoolId() != null) {
                sql.append(" and school_id = ? ");
                param.add(req.getSchoolId());
            }
            // 年级ID
            if (req.getGradeId() != null) {
                sql.append(" and grade_id = ? ");
                param.add(req.getGradeId());
            }
            // 班级ID
            if (req.getClassId() != null) {
                sql.append(" and class_id = ? ");
                param.add(req.getClassId());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
            // 学生ID列表
            if (CollectionUtil.isNotEmpty(req.getStudentIdList())) {
                String itemSql = " and student_id in " + CommonUtil.fromListToStr(req.getStudentIdList());
                sql.append(itemSql);
            }
        }
        sql.append(" order by created_time desc ");
        return jt.queryForBeans(YcqYjsStudentPosition.class, sql.toString(), param.toArray());
    }

    /**
     * 统计学生岗位数量
     *
     * @param req
     * @return
     */
    public Integer count(StudentPositionReq req) {
        log.info("YcqYjsStudentPositionJdbcDao.count.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生岗位类型
            if (req.getStudentPositionType() != null) {
                sql.append(" and student_position_type = ? ");
                param.add(req.getStudentPositionType());
            }
            // 学生岗位分类类型
            if (req.getStudentPositionClassType() != null) {
                sql.append(" and student_position_class_type = ? ");
                param.add(req.getStudentPositionClassType());
            }
            // 学期类型
            if (req.getSemesterType() != null) {
                sql.append(" and semester_type = ? ");
                param.add(req.getSemesterType());
            }
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学校ID
            if (req.getSchoolId() != null) {
                sql.append(" and school_id = ? ");
                param.add(req.getSchoolId());
            }
            // 年级ID
            if (req.getGradeId() != null) {
                sql.append(" and grade_id = ? ");
                param.add(req.getGradeId());
            }
            // 班级ID
            if (req.getClassId() != null) {
                sql.append(" and class_id = ? ");
                param.add(req.getClassId());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
            // 学生ID列表
            if (CollectionUtil.isNotEmpty(req.getStudentIdList())) {
                String itemSql = " and student_id in " + CommonUtil.fromListToStr(req.getStudentIdList());
                sql.append(itemSql);
            }
        }
        return jt.queryForInt(sql.toString(), param.toArray());
    }


    /**
     * 学生当前最新岗位
     * @param req
     * @return
     */
    public YcqYjsStudentPosition nowStudentPosition(StudentPositionReq req){
        log.info("YcqYjsStudentPositionJdbcDao.nowStudentPosition.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(SEECT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学期类型
            if (req.getSemesterType() != null) {
                sql.append(" and semester_type = ? ");
                param.add(req.getSemesterType());
            }
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学校ID
            if (req.getSchoolId() != null) {
                sql.append(" and school_id = ? ");
                param.add(req.getSchoolId());
            }
            // 年级ID
            if (req.getGradeId() != null) {
                sql.append(" and grade_id = ? ");
                param.add(req.getGradeId());
            }
            // 班级ID
            if (req.getClassId() != null) {
                sql.append(" and class_id = ? ");
                param.add(req.getClassId());
            }

        }
        sql.append(" order by created_time desc limit 1 ");


        return jt.queryForBean(YcqYjsStudentPosition.class, sql.toString(), param.toArray());

    }


}