package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentSubCourse;
import com.crunii.micro.service.oneaffair.dto.vo.studentSubCourse.StudentSubCourseReq;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生子课程表(YCQ_YJS_STUDENT_SUB_COURSE)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentSubCourseJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String BASE_SQL = "select * from YCQ_YJS_STUDENT_SUB_COURSE where del_flag = 0";

    private static final String COUNT_SQL = "select count(*) from YCQ_YJS_STUDENT_SUB_COURSE where del_flag = 0";

    /**
     * 列表查询学生子课程数据
     *
     * @param req
     * @return
     */
    public List<YcqYjsStudentSubCourse> selectList(StudentSubCourseReq req) {
        log.info("YcqYjsStudentSubCourseJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生课程ID
            if (req.getStudentCourseId() != null) {
                sql.append(" and student_course_id = ? ");
                param.add(req.getStudentCourseId());
            }
            // 课程ID
            if (req.getCourseId() != null) {
                sql.append(" and course_id = ? ");
                param.add(req.getCourseId());
            }
            // 子课程ID
            if (req.getSubCourseId() != null) {
                sql.append(" and sub_course_id = ? ");
                param.add(req.getSubCourseId());
            }
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
        }
        sql.append(" order by id ");
        return jt.queryForBeans(YcqYjsStudentSubCourse.class, sql.toString(), param.toArray());
    }

    /**
     * 统计学生学生子课程
     *
     * @param req
     * @return
     */
    public Integer count(StudentSubCourseReq req) {
        log.info("YcqYjsStudentSubCourseJdbcDao.count.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生课程ID
            if (req.getStudentCourseId() != null) {
                sql.append(" and student_course_id = ? ");
                param.add(req.getStudentCourseId());
            }
            // 课程ID
            if (req.getCourseId() != null) {
                sql.append(" and course_id = ? ");
                param.add(req.getCourseId());
            }
            // 子课程ID
            if (req.getSubCourseId() != null) {
                sql.append(" and sub_course_id = ? ");
                param.add(req.getSubCourseId());
            }
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
        }
        return jt.queryForInt(sql.toString(), param.toArray());
    }

}
