package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentTest;
import com.crunii.micro.service.oneaffair.dto.vo.student.StudentInfoVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.StudentTestReq;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 学生答卷表(YCQ_YJS_STUDENT_TEST)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsStudentTestJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String BASE_SQL = "select * from YCQ_YJS_STUDENT_TEST where del_flag = 0";

    private static final String COUNT_SQL = "select count(*) from YCQ_YJS_STUDENT_TEST where del_flag = 0";

    /**
     * 列表查询学生答卷数据
     *
     * @param req
     * @return
     */
    public List<YcqYjsStudentTest> selectList(StudentTestReq req) {
        log.info("YcqYjsStudentTestJdbcDao.selectList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学生课程ID
            if (req.getStudentCourseId() != null) {
                sql.append(" and student_course_id = ? ");
                param.add(req.getStudentCourseId());
            }
            // 试卷ID
            if (req.getPaperId() != null) {
                sql.append(" and paper_id = ? ");
                param.add(req.getPaperId());
            }
            // 是否合格：0-不合格；1-合格
            if (req.getIfPass() != null) {
                sql.append(" and if_pass = ? ");
                param.add(req.getIfPass());
            }
            // 大于指定答题得分
            if (req.getAnswerScoreMin() != null) {
                sql.append(" and answer_score > ? ");
                param.add(req.getAnswerScoreMin());
            }
        }
        sql.append(" order by created_time desc");
        return jt.queryForBeans(YcqYjsStudentTest.class, sql.toString(), param.toArray());
    }

    /**
     * 统计学生答卷数量
     *
     * @param req
     * @return
     */
    public Integer count(StudentTestReq req) {
        log.info("YcqYjsStudentTestJdbcDao.count.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学生课程ID
            if (req.getStudentCourseId() != null) {
                sql.append(" and student_course_id = ? ");
                param.add(req.getStudentCourseId());
            }
            // 试卷ID
            if (req.getPaperId() != null) {
                sql.append(" and paper_id = ? ");
                param.add(req.getPaperId());
            }
            // 是否合格：0-不合格；1-合格
            if (req.getIfPass() != null) {
                sql.append(" and if_pass = ? ");
                param.add(req.getIfPass());
            }
            // 大于指定答题得分
            if (req.getAnswerScoreMin() != null) {
                sql.append(" and answer_score > ? ");
                param.add(req.getAnswerScoreMin());
            }
        }
        return jt.queryForInt(sql.toString(), param.toArray());
    }

    /**
     * 分页查询学生答卷
     * @param req
     * @return
     */
    public PageRsp<YcqYjsStudentTest> pageSearchStudentTest(PageReq<StudentTestReq> req) {
        log.info("YcqYjsStudentTestJdbcDao.pageSearchStudentTest.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        StringBuilder countSql = new StringBuilder(COUNT_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null && req.getQuery() != null) {
            // 学生ID
            if (req.getQuery().getStudentId() != null) {
                sql.append(" and student_id = ? ");
                countSql.append(" and student_id = ? ");
                param.add(req.getQuery().getStudentId());
            }
            // 学生课程ID
            if (req.getQuery().getStudentCourseId() != null) {
                sql.append(" and student_course_id = ? ");
                countSql.append(" and student_course_id = ? ");
                param.add(req.getQuery().getStudentCourseId());
            }
            // 试卷ID
            if (req.getQuery().getPaperId() != null) {
                sql.append(" and paper_id = ? ");
                countSql.append(" and paper_id = ? ");
                param.add(req.getQuery().getPaperId());
            }
            // 是否合格：0-不合格；1-合格
            if (req.getQuery().getIfPass() != null) {
                sql.append(" and if_pass = ? ");
                countSql.append(" and if_pass = ? ");
                param.add(req.getQuery().getIfPass());
            }
            // 大于指定答题得分
            if (req.getQuery().getAnswerScoreMin() != null) {
                sql.append(" and answer_score > ? ");
                countSql.append(" and answer_score > ? ");
                param.add(req.getQuery().getAnswerScoreMin());
            }

        }
        sql.append(" order by created_time desc");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<YcqYjsStudentTest> result;
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(YcqYjsStudentTest.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<YcqYjsStudentTest> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    public YcqYjsStudentTest selectMaxStudentTest(StudentTestReq req) {
        log.info("YcqYjsStudentTestJdbcDao.selectMaxStudentTest.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 学生ID
            if (req.getStudentId() != null) {
                sql.append(" and student_id = ? ");
                param.add(req.getStudentId());
            }
            // 学生课程ID
            if (req.getStudentCourseId() != null) {
                sql.append(" and student_course_id = ? ");
                param.add(req.getStudentCourseId());
            }
            // 试卷ID
            if (req.getPaperId() != null) {
                sql.append(" and paper_id = ? ");
                param.add(req.getPaperId());
            }
            // 是否合格：0-不合格；1-合格
            if (req.getIfPass() != null) {
                sql.append(" and if_pass = ? ");
                param.add(req.getIfPass());
            }
            // 大于指定答题得分
            if (req.getAnswerScoreMin() != null) {
                sql.append(" and answer_score > ? ");
                param.add(req.getAnswerScoreMin());
            }
        }
        sql.append(" order by answer_score desc limit 1");
        return jt.queryForBean(YcqYjsStudentTest.class, sql.toString(), param.toArray());
    }

}
