package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsTeacherBindingClass;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.TeacherBindingClassReq;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.TeacherInfoVo;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * 教师与班级的绑定表(YCQ_YJS_TEACHER_BINDING_ClASS)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsTeacherBindingClassJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    private static final String BASE_SQL = "select * from ycq_yjs_teacher_binding_class where del_flag = 0";
    private static final String BASE_SQL_1 = "select teacher_id from ycq_yjs_teacher_binding_class where del_flag = 0";

    public List<BigInteger> selectIdListByTeacherIds(List<BigInteger> teacherIds) {
        String sql = "select id from ycq_yjs_teacher_binding_class where del_flag = 0 and teacher_id in " + CommonUtil.fromListToStr(teacherIds);
        return JT.queryForList(sql,BigInteger.class);
    }

    public void clearBatch(List<BigInteger> idList) {
        if (!CollectionUtils.isEmpty(idList)) {
            String sql = "delete from ycq_yjs_teacher_binding_class where id in" + CommonUtil.fromListToStr(idList);
            JT.update(sql);
        }
    }

    public List<YcqYjsTeacherBindingClass> selectListByTeacherIds(List<BigInteger> teacherIds) {
        String sql = "select * from ycq_yjs_teacher_binding_class where del_flag = 0 and teacher_id in " + CommonUtil.fromListToStr(teacherIds);
        return JT.query(sql, new BeanPropertyRowMapper<>(YcqYjsTeacherBindingClass.class));
    }

    public List<BigInteger> selectTeacherIds(TeacherBindingClassReq req) {
        log.info("YcqYjsTeacherBindingClassJdbcDao.selectTeacherIds.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL_1);
        if(req != null) {
            // 年级ID列表
            if (CollectionUtil.isNotEmpty(req.getGradeIdList())) {
                String itemSql = " and grade_id in " + CommonUtil.fromListToStr(req.getGradeIdList());
                sql.append(itemSql);
            }
            // 班级ID列表
            if (CollectionUtil.isNotEmpty(req.getClassIdList())) {
                String itemSql = " and class_id in " + CommonUtil.fromListToStr(req.getClassIdList());
                sql.append(itemSql);
            }
            // 年级ID
            if (req.getGradeId() != null) {
                sql.append(" and grade_id = " + req.getGradeId());
            }
            // 班级ID
            if (req.getClassId() != null) {
                sql.append(" and class_id = " + req.getClassId());
            }
        }
        sql.append(" order by id asc ");
        return JT.queryForList(sql.toString(),BigInteger.class);
    }
}
