package com.crunii.micro.service.oneaffair.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsTeacher;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.TeacherInfoVo;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.TeacherReq;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 教师表(YCQ_YJS_TEACHER)表数据库访问层
 */
@Slf4j
@Repository
public class YcqYjsTeacherJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    private static final String BASE_SQL = "select * from ycq_yjs_teacher where del_flag = 0 ";

    private static final String COUNT_SQL = "select count(*) from ycq_yjs_teacher where del_flag = 0 ";

    private static final String PAGE_BASE_SQL = "select distinct(t.id),t.* from ycq_yjs_teacher t " +
            " inner join tsm_worker_org_all twoa on t.school_id = twoa.org_id " +
            " where t.del_flag = 0 ";

    private static final String PAGE_COUNT_SQL = "select count(distinct(t.id)) from ycq_yjs_teacher t " +
            " inner join tsm_worker_org_all twoa on t.school_id = twoa.org_id " +
            " where t.del_flag = 0 ";

    /**
     * 根据身份证查询教师数据
     *
     * @param identityCardNo
     * @return
     */
    public List<YcqYjsTeacher> selectListByIdentityCardNo(String identityCardNo) {
        if (StringUtils.isBlank(identityCardNo)) {
            return null;
        }
        String sql = BASE_SQL + " and identity_card_no = ? ";
        return jt.query(sql, new BeanPropertyRowMapper<>(YcqYjsTeacher.class), identityCardNo);
    }

    /**
     * 根据手机号查询教师数据
     *
     * @param mobile
     * @return
     */
    public List<YcqYjsTeacher> selectListByMobile(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return null;
        }
        String sql = BASE_SQL + " and mobile = ? ";
        return jt.query(sql, new BeanPropertyRowMapper<>(YcqYjsTeacher.class), mobile);
    }

    /**
     * 分页查询教师数据
     *
     * @param req
     * @return
     */
    public PageRsp<TeacherInfoVo> pageSearchTeacherInfo(PageReq<TeacherReq> req) {
        log.info("YcqYjsTeacherJdbcDao.pageSearchTeacherInfo.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();

        // 数据权限
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        sql.append(" and twoa.worker_id = ? ");
        countSql.append(" and twoa.worker_id = ? ");
        param.add(currWorker.getLoginId());

        if (req.getQuery() != null) {
            // 教师编号
            if (StringUtils.isNotBlank(req.getQuery().getTeacherNum())) {
                sql.append(" and t.teacher_num like ? ");
                countSql.append(" and t.teacher_num like ? ");
                param.add("%" + req.getQuery().getTeacherNum() + "%");
            }
            // 教师名称
            if (StringUtils.isNotBlank(req.getQuery().getTeacherName())) {
                sql.append(" and t.teacher_name like ? ");
                countSql.append(" and t.teacher_name like ? ");
                param.add("%" + req.getQuery().getTeacherName() + "%");
            }
            // 身份证号
            if (StringUtils.isNotBlank(req.getQuery().getIdentityCardNo())) {
                sql.append(" and t.identity_card_no like ? ");
                countSql.append(" and t.identity_card_no like ? ");
                param.add("%" + req.getQuery().getIdentityCardNo() + "%");
            }
            // 手机号
            if (StringUtils.isNotBlank(req.getQuery().getMobile())) {
                sql.append(" and t.mobile like ? ");
                countSql.append(" and t.mobile like ? ");
                param.add("%" + req.getQuery().getMobile() + "%");
            }
            // 性别
            if (req.getQuery().getSex() != null) {
                sql.append(" and t.sex = ? ");
                countSql.append(" and t.sex = ? ");
                param.add(req.getQuery().getSex());
            }
            // 教师职务类型
            if (req.getQuery().getTeacherPositionType() != null) {
                sql.append(" and t.teacher_position_type = ? ");
                countSql.append(" and t.teacher_position_type = ? ");
                param.add(req.getQuery().getTeacherPositionType());
            }
            // 政治面貌
            if (req.getQuery().getPoliticalStatusType() != null) {
                sql.append(" and t.political_status_type = ? ");
                countSql.append(" and t.political_status_type = ? ");
                param.add(req.getQuery().getPoliticalStatusType());
            }
            // 创建时间
            if (req.getQuery().getStartCreatedTime() != null) {
                sql.append(" and t.created_time >= ? ");
                countSql.append(" and t.created_time >= ? ");
                param.add(req.getQuery().getStartCreatedTime());
            }
            if (req.getQuery().getEndCreatedTime() != null) {
                sql.append(" and  ?  >= t.created_time ");
                countSql.append(" and  ?  >= t.created_time ");
                param.add(req.getQuery().getEndCreatedTime());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getIdList())) {
                String itemSql = " and t.id in " + CommonUtil.fromListToStr(req.getQuery().getIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 学校ID列表
            if (CollectionUtil.isNotEmpty(req.getQuery().getSchoolIdList())) {
                String itemSql = " and t.school_id in " + CommonUtil.fromListToStr(req.getQuery().getSchoolIdList());
                sql.append(itemSql);
                countSql.append(itemSql);
            }
            // 学校ID
            if (req.getQuery().getSchoolId() != null) {
                sql.append(" and t.school_id = ? ");
                countSql.append(" and t.school_id = ? ");
                param.add(req.getQuery().getSchoolId());
            }

        }

        sql.append(" order by t.created_time desc ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<TeacherInfoVo> result;
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(TeacherInfoVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<TeacherInfoVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

    /**
     * 列表查询教师数据
     *
     * @param req
     * @return
     */
    public List<TeacherInfoVo> selectTeacherInfoList(TeacherReq req) {
        log.info("YcqYjsTeacherJdbcDao.selectTeacherInfoList.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(BASE_SQL);
        List<Object> param = new ArrayList<>();
        if (req != null) {
            // 教师编号
            if (StringUtils.isNotBlank(req.getTeacherNum())) {
                sql.append(" and teacher_num like ? ");
                param.add("%" + req.getTeacherNum() + "%");
            }
            // 教师名称
            if (StringUtils.isNotBlank(req.getTeacherName())) {
                sql.append(" and teacher_name like ? ");
                param.add("%" + req.getTeacherName() + "%");
            }
            // 身份证号
            if (StringUtils.isNotBlank(req.getIdentityCardNo())) {
                sql.append(" and identity_card_no like ? ");
                param.add("%" + req.getIdentityCardNo() + "%");
            }
            // 性别
            if (req.getSex() != null) {
                sql.append(" and sex = ? ");
                param.add(req.getSex());
            }
            // 用户ID
            if (req.getUserId() != null) {
                sql.append(" and user_id = ? ");
                param.add(req.getUserId());
            }
            // 教师职务类型
            if (req.getTeacherPositionType() != null) {
                sql.append(" and teacher_position_type = ? ");
                param.add(req.getTeacherPositionType());
            }
            // 政治面貌
            if (req.getPoliticalStatusType() != null) {
                sql.append(" and political_status_type = ? ");
                param.add(req.getPoliticalStatusType());
            }
            // 创建时间
            if (req.getStartCreatedTime() != null) {
                sql.append(" and created_time >= ? ");
                param.add(req.getStartCreatedTime());
            }
            if (req.getEndCreatedTime() != null) {
                sql.append(" and  ?  >= created_time ");
                param.add(req.getEndCreatedTime());
            }
            // ID列表
            if (CollectionUtil.isNotEmpty(req.getIdList())) {
                String itemSql = " and id in " + CommonUtil.fromListToStr(req.getIdList());
                sql.append(itemSql);
            }
            // 学校ID列表
            if (CollectionUtil.isNotEmpty(req.getSchoolIdList())) {
                String itemSql = " and school_id in " + CommonUtil.fromListToStr(req.getSchoolIdList());
                sql.append(itemSql);
            }
            // 身份证号列表
            if (CollectionUtil.isNotEmpty(req.getIdentityCardNoList())) {
                String itemSql = " and identity_card_no in " + CommonUtil.fromStrListToStr2(req.getIdentityCardNoList());
                sql.append(itemSql);
            }
            // 手机号列表
            if (CollectionUtil.isNotEmpty(req.getMobileList())) {
                String itemSql = " and mobile in " + CommonUtil.fromStrListToStr2(req.getMobileList());
                sql.append(itemSql);
            }
            // 学校ID
            if (req.getSchoolId() != null) {
                sql.append(" and school_id = ? ");
                param.add(req.getSchoolId());
            }

        }
        sql.append(" order by created_time desc ");
        return jt.queryForBeans(TeacherInfoVo.class, sql.toString(), param.toArray());
    }

}
