package com.crunii.micro.service.oneaffair.manager;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.oneaffair.constants.OneAffairConstants;
import com.crunii.micro.service.oneaffair.dto.excel.GoodStudentInfoExcel;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsGoodStudent;
import com.crunii.micro.service.oneaffair.dto.vo.goodstudent.*;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.enums.ShowFlagEnum;
import com.crunii.micro.service.oneaffair.service.GoodStudentService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import com.crunii.micro.service.oneaffair.util.CharacterCodeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class GoodStudentManager {

    @Resource
    private GoodStudentService goodStudentService;

    @Value("${upload.url}")
    private String urlPrefix;

    /**
     * 新建优秀学生信息
     * @param createReq
     * @throws Exception
     */
    public void createGoodStudent(GoodStudentCreateReq createReq) throws Exception {
        try {
            log.info("GoodStudentManager.createGoodStudent.param：{}", createReq.toString());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                createReq.setCreatedName(currWorker.getWorkerName());
            }
            if(createReq.getStudentPhotoUrl() != null){
                createReq.setStudentPhotoUrl(urlPrefix + createReq.getStudentPhotoUrl());
            }
            // 优秀学生编号 : 业务首字母大写(GSGL）+ 时间(秒) + 4位随机码(26个 英文字母+数字)
            String teacherNum = CharacterCodeUtil.generateCharacterCode(OneAffairConstants.GOOD_STUDENT_NUM_PREFIX);
            createReq.setStudentNum(teacherNum);
            createReq.setCreatedTime(createDate);
            createReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
            createReq.setShowFlag(ShowFlagEnum.UN_SHOW.code());
            goodStudentService.insertOne(createReq);
        } catch (BusinessException e) {
            log.error("GoodStudentManager.createGoodStudent.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentManager.createGoodStudent.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 修改优秀学生信息
     *
     * @param updateReq
     */
    public void updateGoodStudent(GoodStudentUpdateReq updateReq) {
        try {
            log.info("GoodStudentManager.updateGoodStudent.param：{}", updateReq.toString());
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            if(updateReq.getStudentPhotoUrl() != null && updateReq.getIfStudentPhotoUrl()){
                updateReq.setStudentPhotoUrl(urlPrefix + updateReq.getStudentPhotoUrl());
            }
            updateReq.setLastUpdatedTime(updateDate);
            goodStudentService.updateOne(updateReq);
        } catch (BusinessException e) {
            log.error("GoodStudentManager.updateGoodStudent.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentManager.updateGoodStudent.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 批量修改优秀学生的展出状态
     *
     * @param updateReq
     */
    public void updateGoodStudentShowFlag(GoodStudentUpdateShowFlagReq updateReq) {
        try {
            log.info("GoodStudentManager.updateGoodStudentShowFlag.param：{}", updateReq.toString());
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            updateReq.setLastUpdatedTime(updateDate);
            goodStudentService.updateGoodStudentShowFlag(updateReq);
        } catch (BusinessException e) {
            log.error("GoodStudentManager.updateGoodStudentShowFlag.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentManager.updateGoodStudentShowFlag.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 删除优秀学生信息
     *
     * @param deleteReq
     */
    public void deleteGoodStudent(GoodStudentDeleteReq deleteReq) {
        try {
            log.info("GoodStudentManager.deleteGoodStudent.param：{}", deleteReq.toString());
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                deleteReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                deleteReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            deleteReq.setLastUpdatedTime(updateDate);
            goodStudentService.deleteBatch(deleteReq);
        } catch (BusinessException e) {
            log.error("GoodStudentManager.deleteGoodStudent.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentManager.deleteGoodStudent.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 分页查询优秀学生信息
     * @param req
     * @return
     * @throws Exception
     */
    public PageRsp<GoodStudentVo> pageSearchGoodStudent(PageReq<GoodStudentReq> req) throws Exception {
        log.info("GoodStudentManager.pageSearchGoodStudent.param：{}", req.toString());
        try {
            return goodStudentService.pageSearchGoodStudent(req);
        } catch (BusinessException e) {
            log.error("GoodStudentManager.pageSearchGoodStudent.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentManager.pageSearchGoodStudent.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 优秀学生信息excel导出
     * @param response
     * @param req
     */
    public void exportGoodStudentExcel(HttpServletResponse response, GoodStudentReq req) {
        try {
            log.info("GoodStudentManager.exportGoodStudentExcel.param：{}", req.toString());
            List<YcqYjsGoodStudent> voList = goodStudentService.selectList(req);
            List<GoodStudentInfoExcel> excelList = BeanCopyUtil.convert(voList, GoodStudentInfoExcel.class);
            String fileName = "优秀学生信息";
            export(response, fileName, excelList);
        } catch (BusinessException e) {
            log.error("GoodStudentManager.exportGoodStudentExcel.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("GoodStudentManager.exportGoodStudentExcel.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export(HttpServletResponse response, String fileName, List<GoodStudentInfoExcel> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=utf-8''" + encodedFileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), GoodStudentInfoExcel.class)
                    .sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy())
                    .doWrite(dataList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("GoodStudentManager.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 自定义Excel导出策略，设置表头和数据行的字体和高度
     *
     * @return Excel导出策略
     */
    private HorizontalCellStyleStrategy getHeightAndFontStrategy() {
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 11);
        headWriteFont.setBold(true);
        headWriteCellStyle.setWriteFont(headWriteFont);
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
        WriteFont contentWriteFont = new WriteFont();
        contentWriteFont.setFontHeightInPoints((short) 11);
        contentWriteCellStyle.setWriteFont(contentWriteFont);
        contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
    }

    public List<Map<String, String>> searchGoodStudentHonoraryTitleTypeList(String studentNum) throws Exception {
        PageReq<GoodStudentReq> pageReq = new PageReq<>();
        pageReq.setPageNo(1);
        pageReq.setPageSize(Integer.MAX_VALUE);

        GoodStudentReq goodStudentReq = new GoodStudentReq();
        goodStudentReq.setStudentNum(StringUtils.isNoneBlank(studentNum) ? studentNum : goodStudentReq.getStudentNum());
        pageReq.setQuery(goodStudentReq);

        PageRsp<GoodStudentVo> pageRsp = pageSearchGoodStudent(pageReq);
        List<Map<String, String>> honoraryTitleTypeList = new ArrayList<>();
        if (CollectionUtils.isEmpty(pageRsp.getResult())){
            return honoraryTitleTypeList;
        }

        List<GoodStudentVo> goodStudentVos = pageRsp.getResult().stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(x->x.getHonoraryTitleType()))), ArrayList::new));
        goodStudentVos.forEach(x->{
            Map<String, String> map = new HashMap<>();
            map.put("honoraryTitleType", x.getHonoraryTitleType());
            map.put("honoraryTitle", x.getHonoraryTitle());
            honoraryTitleTypeList.add(map);
        });

        return honoraryTitleTypeList;
    }


}
