package com.crunii.micro.service.oneaffair.manager;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.oneaffair.dto.excel.*;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsImportRecord;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordReq;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordUpdateReq;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordVo;
import com.crunii.micro.service.oneaffair.enums.IfFlagEnum;
import com.crunii.micro.service.oneaffair.enums.TaskCompleteStatusEnum;
import com.crunii.micro.service.oneaffair.enums.TaskOperateTypeEnum;
import com.crunii.micro.service.oneaffair.service.ImportRecordService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import dm.jdbc.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ImportRecordManager {

    @Resource
    private ImportRecordService importRecordService;
    @Resource
    private TeacherManager teacherManager;
    @Resource
    private StudentManager studentManager;

    /**
     * 分页查询批量导入记录信息
     *
     * @param req
     * @return
     */
    public PageRsp<ImportRecordVo> pageSearchImportRecord(PageReq<ImportRecordReq> req) {
        log.info("ImportRecordManager.pageSearchImportRecord.param：{}", req.toString());
        try {
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker == null || req == null || req.getQuery() == null) {
                return null;
            }
            ImportRecordReq query = req.getQuery();
            query.setCreatedId(new BigInteger(currWorker.workerId()));
            req.setQuery(query);
            return importRecordService.pageSearch(req);
        } catch (BusinessException e) {
            log.error("ImportRecordManager.pageSearchImportRecord.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordManager.pageSearchImportRecord.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 列表查询批量导入记录信息
     *
     * @param req
     * @return
     */
    public List<ImportRecordVo> selectImportRecordList(ImportRecordReq req) {
        log.info("ImportRecordManager.selectImportRecordList.param：{}", req.toString());
        try {
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker == null || req == null) {
                return null;
            }
            req.setCreatedId(new BigInteger(currWorker.workerId()));
            return BeanCopyUtil.convert(importRecordService.selectList(req), ImportRecordVo.class);
        } catch (BusinessException e) {
            log.error("ImportRecordManager.selectImportRecordList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordManager.selectImportRecordList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 批量导入的结果excel导出
     *
     * @param response
     * @param importRecordId
     */
    public void exportImportResultExcel(HttpServletResponse response, BigInteger importRecordId) {
        try {
            log.info("ImportRecordManager.exportImportResultExcel.param：{}", importRecordId.toString());
            YcqYjsImportRecord ycqYjsImportRecord = importRecordService.selectOneById(importRecordId);
            if (ycqYjsImportRecord == null || StringUtil.isEmpty(ycqYjsImportRecord.getDataJson())) {
                throw new BusinessException("批量导入的结果不存在，无法下载");
            }
            if (ycqYjsImportRecord.getCompleteStatus() == null || ycqYjsImportRecord.getCompleteStatus().intValue() != TaskCompleteStatusEnum.COMPLETED.code().intValue()) {
                throw new BusinessException("批量导入任务未执行完成，无法下载批量导入结果");
            }
            String dataJson = ycqYjsImportRecord.getDataJson();
            String fileName = ycqYjsImportRecord.getFileName();
            Integer operateType = ycqYjsImportRecord.getOperateType();

            fileName = StringUtil.isEmpty(fileName) ? "" : fileName.replace(".xlsx", "").replace(".xls", "").replace(".csv", "");
            fileName = fileName + "导入结果";
            if (operateType != null && operateType.intValue() == TaskOperateTypeEnum.TASK_TYPE_1.code().intValue()) {
                List<TeacherInfoImportExcel> teacherInfoImportExcelList = JSONArray.parseArray(dataJson, TeacherInfoImportExcel.class);
                List<TeacherInfoResultExportExcel> dataList = BeanCopyUtil.convert(teacherInfoImportExcelList, TeacherInfoResultExportExcel.class);
                export1(response, fileName, dataList);
            } else if (operateType != null && operateType.intValue() == TaskOperateTypeEnum.TASK_TYPE_2.code().intValue()) {
                List<StudentInfoImportExcel> studentInfoImportExcelList = JSONArray.parseArray(dataJson, StudentInfoImportExcel.class);
                List<StudentInfoResultExportExcel> dataList = BeanCopyUtil.convert(studentInfoImportExcelList, StudentInfoResultExportExcel.class);
                export2(response, fileName, dataList);
            } else if (operateType != null && operateType.intValue() == TaskOperateTypeEnum.TASK_TYPE_3.code().intValue()) {
                List<StudentInfoTwoImportExcel> studentInfoImportExcelList = JSONArray.parseArray(dataJson, StudentInfoTwoImportExcel.class);
                List<StudentInfoTwoResultExportExcel> dataList = BeanCopyUtil.convert(studentInfoImportExcelList, StudentInfoTwoResultExportExcel.class);
                export3(response, fileName, dataList);
            } else if (operateType != null && operateType.intValue() == TaskOperateTypeEnum.TASK_TYPE_4.code().intValue()) {
                List<StudentInfoThreeImportExcel> studentInfoImportExcelList = JSONArray.parseArray(dataJson, StudentInfoThreeImportExcel.class);
                List<StudentInfoThreeResultExportExcel> dataList = BeanCopyUtil.convert(studentInfoImportExcelList, StudentInfoThreeResultExportExcel.class);
                export4(response, fileName, dataList);
            } else {
                return;
            }
        } catch (BusinessException e) {
            log.error("ImportRecordManager.exportImportResultExcel.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordManager.exportImportResultExcel.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export1(HttpServletResponse response, String fileName, List<TeacherInfoResultExportExcel> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=utf-8''" + encodedFileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), TeacherInfoResultExportExcel.class)
                    .sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy())
                    .doWrite(dataList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("GoodStudentManager.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export2(HttpServletResponse response, String fileName, List<StudentInfoResultExportExcel> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=utf-8''" + encodedFileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), StudentInfoResultExportExcel.class)
                    .sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy())
                    .doWrite(dataList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("GoodStudentManager.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export3(HttpServletResponse response, String fileName, List<StudentInfoTwoResultExportExcel> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=utf-8''" + encodedFileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), StudentInfoTwoResultExportExcel.class)
                    .sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy())
                    .doWrite(dataList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("GoodStudentManager.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export4(HttpServletResponse response, String fileName, List<StudentInfoThreeResultExportExcel> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=utf-8''" + encodedFileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), StudentInfoThreeResultExportExcel.class)
                    .sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy())
                    .doWrite(dataList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("GoodStudentManager.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 自定义Excel导出策略，设置表头和数据行的字体和高度
     *
     * @return Excel导出策略
     */
    private HorizontalCellStyleStrategy getHeightAndFontStrategy() {
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 11);
        headWriteFont.setBold(true);
        headWriteCellStyle.setWriteFont(headWriteFont);
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
        WriteFont contentWriteFont = new WriteFont();
        contentWriteFont.setFontHeightInPoints((short) 11);
        contentWriteCellStyle.setWriteFont(contentWriteFont);
        contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
    }

    /**
     * 处理任务
     *
     * @return
     */
    public void executeTask() {
        log.info("ImportRecordManager.executeTask.param");
        try {
            BigInteger updatedId = new BigInteger("0");
            String updatedName = "系统";
            ImportRecordReq importRecordReq = new ImportRecordReq();
            importRecordReq.setCompleteStatus(TaskCompleteStatusEnum.UN_COMPLETE.code());
            List<YcqYjsImportRecord> ycqYjsImportRecords = importRecordService.selectList(importRecordReq);
            if (!CollectionUtils.isEmpty(ycqYjsImportRecords)) {
                for (YcqYjsImportRecord importRecord : ycqYjsImportRecords) {
                    if (importRecord.getOperateType() != null && importRecord.getOperateType().intValue() == TaskOperateTypeEnum.TASK_TYPE_1.code().intValue()) {
                        executeImportTeacherTask(importRecord.getId(), updatedId, updatedName);
                    }else{
                        continue;
                    }
                }
            }
        } catch (BusinessException e) {
            log.error("ImportRecordManager.executeTask.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordManager.executeTask.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public void reExecuteTask(BigInteger importRecordId) throws BusinessException {
        log.info("ImportRecordManager.reExecuteTask.param：{}", importRecordId);
        try {
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker == null) {
                throw new BusinessException("请登录完成后再进行任务重试");
            }
            YcqYjsImportRecord importRecord = importRecordService.selectOneById(importRecordId);
            if (importRecord == null) {
                throw new BusinessException("任务不存在，重试失败");
            }
            if (importRecord.getOperateType() != null && importRecord.getOperateType().intValue() == TaskOperateTypeEnum.TASK_TYPE_1.code().intValue()) {
                executeImportTeacherTask(importRecord.getId(), new BigInteger(currWorker.workerId()), currWorker.getWorkerName());
            }
        } catch (BusinessException e) {
            log.error("ImportRecordManager.reExecuteTask.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordManager.reExecuteTask.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 处理教师导入任务
     *
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public void executeImportTeacherTask(BigInteger importRecordId, BigInteger updatedId, String updatedName) throws BusinessException {
        log.info("ImportRecordManager.executeImportTeacherTask.param：{}", importRecordId);
        try {
            Date updateDate = new Date();
            YcqYjsImportRecord importRecord = importRecordService.selectOneById(importRecordId);
            if (importRecord == null) {
                return;
            }
            String dataJson = importRecord.getDataJson();
            if (StringUtil.isEmpty(dataJson)) {
                return;
            }
            List<TeacherInfoImportExcel> teacherInfoImportExcelList = JSONArray.parseArray(dataJson, TeacherInfoImportExcel.class);
            for (TeacherInfoImportExcel teacherInfoImportExcel : teacherInfoImportExcelList) {
                String errorMessage = "";
                try {
                    teacherManager.bindingTeacherToUserById(teacherInfoImportExcel.getTeacherId());
                } catch (BusinessException e) {
                    log.error("ImportRecordManager.executeImportTeacherTask.error:{}", e.getMessage());
                    errorMessage = e.getMessage();
                } catch (Exception e) {
                    log.error("ImportRecordManager.executeImportTeacherTask.error:{}", e.getMessage());
                    e.printStackTrace();
                    errorMessage = "系统内部异常，请联系管理员！";
                }
                if (StringUtil.isEmpty(errorMessage)) {
                    teacherInfoImportExcel.setCompleteStatus(IfFlagEnum.TRUE.code());
                    teacherInfoImportExcel.setErrorMessage(errorMessage);
                } else {
                    teacherInfoImportExcel.setCompleteStatus(IfFlagEnum.FALSE.code());
                    teacherInfoImportExcel.setErrorMessage(errorMessage);
                }
            }
            importRecord.setCompleteStatus(TaskCompleteStatusEnum.COMPLETED.code());
            importRecord.setCompleteTime(updateDate);
            List<TeacherInfoImportExcel> successList = teacherInfoImportExcelList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code()).collect(Collectors.toList());
            int SuccessSize = CollectionUtils.isEmpty(successList) ? 0 : successList.size();
            importRecord.setErrorSize(teacherInfoImportExcelList.size() - SuccessSize);
            importRecord.setSuccessSize(SuccessSize);
            importRecord.setLastUpdatedId(updatedId);
            importRecord.setLastUpdatedName(updatedName);
            importRecord.setLastUpdatedTime(updateDate);
            importRecord.setDataJson(JSON.toJSONString(teacherInfoImportExcelList));
            importRecordService.updateOne(BeanCopyUtil.convert(importRecord, ImportRecordUpdateReq.class));
        } catch (BusinessException e) {
            log.error("ImportRecordManager.executeImportTeacherTask.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordManager.executeImportTeacherTask.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 处理学生导入任务
     *
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public void executeImportStudentTask(BigInteger importRecordId, BigInteger updatedId, String updatedName) throws BusinessException {
        log.info("ImportRecordManager.executeImportStudentTask.param");
        try {
            Date updateDate = new Date();
            YcqYjsImportRecord importRecord = importRecordService.selectOneById(importRecordId);
            if (importRecord == null) {
                return;
            }
            String dataJson = importRecord.getDataJson();
            if (StringUtil.isEmpty(dataJson)) {
                return;
            }
            List<StudentInfoImportExcel> studentInfoImportExcelList = JSONArray.parseArray(dataJson, StudentInfoImportExcel.class);
            for (StudentInfoImportExcel studentInfoImportExcel : studentInfoImportExcelList) {
                String errorMessage = "";
                try {
                    // 学生绑定用户
                    studentManager.bindingStudentToUser(studentInfoImportExcel.getStudentId());
                } catch (BusinessException e) {
                    log.error("ImportRecordManager.executeImportStudentTask.error:{}", e.getMessage());
                    errorMessage = e.getMessage();
                } catch (Exception e) {
                    log.error("ImportRecordManager.executeImportStudentTask.error:{}", e.getMessage());
                    e.printStackTrace();
                    errorMessage = "系统内部异常，请联系管理员！";
                }
                if (StringUtil.isEmpty(errorMessage)) {
                    studentInfoImportExcel.setCompleteStatus(IfFlagEnum.TRUE.code());
                    studentInfoImportExcel.setErrorMessage(errorMessage);
                } else {
                    studentInfoImportExcel.setCompleteStatus(IfFlagEnum.FALSE.code());
                    studentInfoImportExcel.setErrorMessage(errorMessage);
                }
            }
            importRecord.setCompleteStatus(TaskCompleteStatusEnum.COMPLETED.code());
            importRecord.setCompleteTime(updateDate);
            List<StudentInfoImportExcel> successList = studentInfoImportExcelList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code()).collect(Collectors.toList());
            int SuccessSize = CollectionUtils.isEmpty(successList) ? 0 : successList.size();
            importRecord.setErrorSize(studentInfoImportExcelList.size() - SuccessSize);
            importRecord.setSuccessSize(SuccessSize);
            importRecord.setLastUpdatedId(updatedId);
            importRecord.setLastUpdatedName(updatedName);
            importRecord.setLastUpdatedTime(updateDate);
            importRecord.setDataJson(JSON.toJSONString(studentInfoImportExcelList));
            importRecordService.updateOne(BeanCopyUtil.convert(importRecord, ImportRecordUpdateReq.class));
        } catch (BusinessException e) {
            log.error("ImportRecordManager.executeImportStudentTask.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordManager.executeImportStudentTask.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 处理学生导入任务
     *
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public void executeImportStudentTaskTwo(BigInteger importRecordId, BigInteger updatedId, String updatedName) throws BusinessException {
        log.info("ImportRecordManager.executeImportStudentTaskTwo.param");
        try {
            Date updateDate = new Date();
            YcqYjsImportRecord importRecord = importRecordService.selectOneById(importRecordId);
            if (importRecord == null) {
                return;
            }
            String dataJson = importRecord.getDataJson();
            if (StringUtil.isEmpty(dataJson)) {
                return;
            }
            List<StudentInfoTwoImportExcel> studentInfoImportExcelList = JSONArray.parseArray(dataJson, StudentInfoTwoImportExcel.class);
            for (StudentInfoTwoImportExcel studentInfoImportExcel : studentInfoImportExcelList) {
                String errorMessage = "";
                try {
                    // 学生绑定用户
                    studentManager.bindingStudentToUser(studentInfoImportExcel.getStudentId());
                } catch (BusinessException e) {
                    log.error("ImportRecordManager.executeImportStudentTaskTwo.error:{}", e.getMessage());
                    errorMessage = e.getMessage();
                } catch (Exception e) {
                    log.error("ImportRecordManager.executeImportStudentTaskTwo.error:{}", e.getMessage());
                    e.printStackTrace();
                    errorMessage = "系统内部异常，请联系管理员！";
                }
                if (StringUtil.isEmpty(errorMessage)) {
                    studentInfoImportExcel.setCompleteStatus(IfFlagEnum.TRUE.code());
                    studentInfoImportExcel.setErrorMessage(errorMessage);
                } else {
                    studentInfoImportExcel.setCompleteStatus(IfFlagEnum.FALSE.code());
                    studentInfoImportExcel.setErrorMessage(errorMessage);
                }
            }
            importRecord.setCompleteStatus(TaskCompleteStatusEnum.COMPLETED.code());
            importRecord.setCompleteTime(updateDate);
            List<StudentInfoTwoImportExcel> successList = studentInfoImportExcelList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code()).collect(Collectors.toList());
            int SuccessSize = CollectionUtils.isEmpty(successList) ? 0 : successList.size();
            importRecord.setErrorSize(studentInfoImportExcelList.size() - SuccessSize);
            importRecord.setSuccessSize(SuccessSize);
            importRecord.setLastUpdatedId(updatedId);
            importRecord.setLastUpdatedName(updatedName);
            importRecord.setLastUpdatedTime(updateDate);
            importRecord.setDataJson(JSON.toJSONString(studentInfoImportExcelList));
            importRecordService.updateOne(BeanCopyUtil.convert(importRecord, ImportRecordUpdateReq.class));
        } catch (BusinessException e) {
            log.error("ImportRecordManager.executeImportStudentTaskTwo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordManager.executeImportStudentTaskTwo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 处理学生导入任务
     *
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public void executeImportStudentTaskThree(BigInteger importRecordId, BigInteger updatedId, String updatedName) throws BusinessException {
        log.info("ImportRecordManager.executeImportStudentTaskThree.param");
        try {
            Date updateDate = new Date();
            YcqYjsImportRecord importRecord = importRecordService.selectOneById(importRecordId);
            if (importRecord == null) {
                return;
            }
            String dataJson = importRecord.getDataJson();
            if (StringUtil.isEmpty(dataJson)) {
                return;
            }
            List<StudentInfoThreeImportExcel> studentInfoImportExcelList = JSONArray.parseArray(dataJson, StudentInfoThreeImportExcel.class);
            for (StudentInfoThreeImportExcel studentInfoImportExcel : studentInfoImportExcelList) {
                String errorMessage = "";
                try {
                    // 学生绑定用户
                    studentManager.bindingStudentToUser(studentInfoImportExcel.getStudentId());
                } catch (BusinessException e) {
                    log.error("ImportRecordManager.executeImportStudentTaskThree.error:{}", e.getMessage());
                    errorMessage = e.getMessage();
                } catch (Exception e) {
                    log.error("ImportRecordManager.executeImportStudentTaskThree.error:{}", e.getMessage());
                    e.printStackTrace();
                    errorMessage = "系统内部异常，请联系管理员！";
                }
                if (StringUtil.isEmpty(errorMessage)) {
                    studentInfoImportExcel.setCompleteStatus(IfFlagEnum.TRUE.code());
                    studentInfoImportExcel.setErrorMessage(errorMessage);
                } else {
                    studentInfoImportExcel.setCompleteStatus(IfFlagEnum.FALSE.code());
                    studentInfoImportExcel.setErrorMessage(errorMessage);
                }
            }
            importRecord.setCompleteStatus(TaskCompleteStatusEnum.COMPLETED.code());
            importRecord.setCompleteTime(updateDate);
            List<StudentInfoThreeImportExcel> successList = studentInfoImportExcelList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code()).collect(Collectors.toList());
            int SuccessSize = CollectionUtils.isEmpty(successList) ? 0 : successList.size();
            importRecord.setErrorSize(studentInfoImportExcelList.size() - SuccessSize);
            importRecord.setSuccessSize(SuccessSize);
            importRecord.setLastUpdatedId(updatedId);
            importRecord.setLastUpdatedName(updatedName);
            importRecord.setLastUpdatedTime(updateDate);
            importRecord.setDataJson(JSON.toJSONString(studentInfoImportExcelList));
            importRecordService.updateOne(BeanCopyUtil.convert(importRecord, ImportRecordUpdateReq.class));
        } catch (BusinessException e) {
            log.error("ImportRecordManager.executeImportStudentTaskThree.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("ImportRecordManager.executeImportStudentTaskThree.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }


}
