package com.crunii.micro.service.oneaffair.manager;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.StudyCenterService;
import com.crunii.micro.feign.client.dto.YcqYjsCourseMgrResp;
import com.crunii.micro.feign.client.dto.YcqYjsCoursesubMgrResp;
import com.crunii.micro.feign.client.dto.YcqYjsSpecialMgr;
import com.crunii.micro.service.oneaffair.constants.OneAffairConstants;
import com.crunii.micro.service.oneaffair.dto.oneaffair.*;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveChangeReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourse.*;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourseUploadVideo.StudentCourseUploadVideoCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourseUploadVideo.StudentCourseUploadVideoDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourseUploadVideo.StudentCourseUploadVideoReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentQuestion.StudentQuestionCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentQuestion.StudentQuestionInfoVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentQuestion.StudentQuestionReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentSubCourse.*;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.*;
import com.crunii.micro.service.oneaffair.enums.*;
import com.crunii.micro.service.oneaffair.service.*;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import com.crunii.micro.service.oneaffair.util.CharacterCodeUtil;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import dm.jdbc.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class StudentCourseManager {

    @Resource
    private StudentCourseService studentCourseService;
    @Resource
    private StudyCenterService studyCenterService;
    @Resource
    private StudentTestService studentTestService;
    @Resource
    private StudentQuestionService studentQuestionService;
    @Resource
    private StudentSubCourseService studentSubCourseService;
    @Resource
    private ArchiveService archiveService;
    @Resource
    private StudentService studentService;
    @Resource
    private StudentCourseUploadVideoService studentCourseUploadVideoService;

    @Value("${upload.url}")
    private String urlPrefix;

    public List<StudentCourseInfoVo> selectStudentCourseInfoList(StudentCourseReq req) throws Exception {
        log.info("StudentCourseManager.selectStudentCourseInfoList.param：{}", req.toString());
        try {
            // 查询专题类型下的课程信息
            List<YcqYjsCourseMgrResp> dataList = queryCourseInfoBySpecialType(req);
            List<StudentCourseInfoVo> result = new ArrayList<>();
            if (!CollectionUtils.isEmpty(dataList)) {
                // 查询学生的完成情况
                List<YcqYjsStudentCourse> studentCourseList = studentCourseService.selectStudentCourseInfoList(req);
                Map<String, List<YcqYjsStudentCourse>> studentCourseMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(studentCourseList)) {
                    studentCourseMap = studentCourseList.stream().collect(Collectors.groupingBy(YcqYjsStudentCourse::getGroupId));
                }
                for (YcqYjsCourseMgrResp item : dataList) {
                    StudentCourseInfoVo studentCourseInfoVo = new StudentCourseInfoVo();
                    // 学习状态：0-未学习；1-进行中；2-已学习
                    studentCourseInfoVo.setStudyStatus(StudyStatusEnum.UN_STUDYING.code());
                    List<YcqYjsStudentCourse> ycqYjsStudentCourses = studentCourseMap.get(item.getSpecialId() + ":" + item.getId());
                    if (!CollectionUtils.isEmpty(ycqYjsStudentCourses)) {
                        YcqYjsStudentCourse studentCourse = ycqYjsStudentCourses.get(0);
                        BeanUtils.copyProperties(studentCourse, studentCourseInfoVo);
                        studentCourseInfoVo.setStudyStatus(StudyStatusEnum.IN_STUDYING.code());
                        if (studentCourse.getCompleteStatus() != null && studentCourse.getCompleteStatus().intValue() == CompleteStatusEnum.COMPLETED.code().intValue()) {
                            studentCourseInfoVo.setStudyStatus(StudyStatusEnum.COMPLETE.code());
                        }
                    }
                    // 学生ID
                    studentCourseInfoVo.setStudentId(req.getStudentId());
                    // 专题ID
                    studentCourseInfoVo.setSpecialId(item.getSpecialId());
                    // 专题类别
                    studentCourseInfoVo.setSpecialType(item.getSpecialType().toString());
                    // 专题类别名称
                    studentCourseInfoVo.setSpecialName(item.getSpecialName());
                    // 课程ID
                    studentCourseInfoVo.setCourseId(item.getId());
                    // 课程名称
                    studentCourseInfoVo.setCourseName(item.getCourseName());
                    // 课程时长
                    studentCourseInfoVo.setCourseDuration(item.getCourseDuration());
                    // 课程学分
                    studentCourseInfoVo.setCourseCredit(item.getCourseCredit());
                    // 课程描述
                    studentCourseInfoVo.setCourseDescription(item.getCourseDescription());
                    // 课程封面
                    studentCourseInfoVo.setCourseCover(item.getCourseCover());
                    // 附件
                    studentCourseInfoVo.setAttachmentUrl(item.getAttachment());
                    studentCourseInfoVo.setIsSubCourse(item.getIsSubcourse());
                    result.add(studentCourseInfoVo);
                }
            }
            return result;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.selectStudentCourseInfoList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.selectStudentCourseInfoList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    public List<StudentCourseInfoVo> selectStudentCourseListInStudy(StudentCourseReq req) throws Exception {
        log.info("StudentCourseManager.selectStudentCourseListInStudy.param：{}", req.toString());
        try {
            return BeanCopyUtil.convert(studentCourseService.selectStudentCourseInfoList(req), StudentCourseInfoVo.class);
        } catch (BusinessException e) {
            log.error("StudentCourseManager.selectStudentCourseListInStudy.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.selectStudentCourseListInStudy.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 查询专题类型下的课程信息
     *
     * @param req
     * @return
     */
    public List<YcqYjsCourseMgrResp> queryCourseInfoBySpecialType(StudentCourseReq req) {
        YcqYjsSpecialMgr specialMgrReq = new YcqYjsSpecialMgr();
        specialMgrReq.setId(req.getSpecialId());
        specialMgrReq.setSpecialType(req.getSpecialType() != null ? Integer.valueOf(req.getSpecialType()) : null);
        specialMgrReq.setAffiliatedBusinessId(req.getAffiliatedBusinessId());
        Result<List<YcqYjsCourseMgrResp>> listResult = studyCenterService.queryCourseInfoBySpecialType(specialMgrReq);
        return listResult.getData();
    }

    /**
     * 统计学生学习课程情况
     *
     * @param req
     * @return
     */
    /*public StudentCourseSumVo sumStudentCourse(StudentCourseReq req) throws Exception {
        log.info("StudentCourseManager.sumStudentCourse.param：{}", req.toString());
        try {
            StudentCourseSumVo studentCourseSumVo = new StudentCourseSumVo();
            // 查询专题类型下的课程信息
            List<YcqYjsCourseMgrResp> dataList = queryCourseInfoBySpecialType(req);
            if (!CollectionUtils.isEmpty(dataList)) {
                List<BigInteger> courseIdList = dataList.stream().map(YcqYjsCourseMgrResp::getId).distinct().collect(Collectors.toList());
                List<BigInteger> specialIdList = dataList.stream().map(YcqYjsCourseMgrResp::getSpecialId).distinct().collect(Collectors.toList());
                int totalCount = dataList.size();
                req.setCourseIdList(courseIdList);
                req.setSpecialIdList(specialIdList);
                // 查询学生的完成情况
                List<YcqYjsStudentCourse> studentCourseList = studentCourseService.selectStudentCourseInfoList(req);
                if (!CollectionUtils.isEmpty(studentCourseList)) {
                    int studyCount = studentCourseList.size();
                    List<YcqYjsStudentCourse> completeStudentCourseList = studentCourseList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == CompleteStatusEnum.COMPLETED.code().intValue()).collect(Collectors.toList());
                    int completeCount = completeStudentCourseList.size();
                    Integer completeCourseDuration = studentCourseList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == CompleteStatusEnum.COMPLETED.code().intValue() && item.getCourseDuration() != null).map(YcqYjsStudentCourse::getCourseDuration).reduce((x, y) -> x + y).orElse(0);
                    studentCourseSumVo.setCompleteCourseDuration(completeCourseDuration);
                    studentCourseSumVo.setCompleteCount(completeCount);
                    studentCourseSumVo.setInStudyCount(studyCount - completeCount);
                    studentCourseSumVo.setUnStudyCount(totalCount - studyCount);
                } else {
                    studentCourseSumVo.setUnStudyCount(totalCount);
                }
            }
            return studentCourseSumVo;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.sumStudentCourse.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.sumStudentCourse.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }*/

    /**
     * 统计学生学习课程情况
     *
     * @param req
     * @return
     */
    public StudentCourseSumVo sumStudentCourse(StudentCourseReq req) throws Exception {
        log.info("StudentCourseManager.sumStudentCourse.param：{}", req.toString());
        try {
            StudentCourseSumVo studentCourseSumVo = new StudentCourseSumVo();
            // 查询专题类型下的课程信息
            List<YcqYjsCourseMgrResp> dataList = queryCourseInfoBySpecialType(req);
            if (!CollectionUtils.isEmpty(dataList)) {
                List<BigInteger> courseIdList = dataList.stream().map(YcqYjsCourseMgrResp::getId).distinct().collect(Collectors.toList());
                List<BigInteger> specialIdList = dataList.stream().map(YcqYjsCourseMgrResp::getSpecialId).distinct().collect(Collectors.toList());
                int totalCount = dataList.size();
                req.setCourseIdList(courseIdList);
                req.setSpecialIdList(specialIdList);
                // 查询学生的完成情况
                List<YcqYjsStudentCourse> studentCourseList = studentCourseService.selectStudentCourseInfoList(req);
                if (!CollectionUtils.isEmpty(studentCourseList)) {
                    int studyCount = studentCourseList.size();
                    List<YcqYjsStudentCourse> completeStudentCourseList = studentCourseList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == CompleteStatusEnum.COMPLETED.code().intValue()).collect(Collectors.toList());
                    int completeCount = completeStudentCourseList.size();
                    studentCourseSumVo.setCompleteCount(completeCount);
                    studentCourseSumVo.setInStudyCount(studyCount - completeCount);
                    studentCourseSumVo.setUnStudyCount(totalCount - studyCount);
                } else {
                    studentCourseSumVo.setUnStudyCount(totalCount);
                }
                // 学习时长
                StudentCourseReq studentCourseReq = new StudentCourseReq();
                studentCourseReq.setStudentId(req.getStudentId());
                studentCourseReq.setCompleteStatus(CompleteStatusEnum.COMPLETED.code());
                Integer completeCourseDuration = studentCourseService.sumCourseDuration(studentCourseReq);
                if(completeCourseDuration != null){
                    studentCourseSumVo.setCompleteCourseDuration(completeCourseDuration);
                }
            }
            return studentCourseSumVo;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.sumStudentCourse.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.sumStudentCourse.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }


    /**
     * 新建学生课程
     *
     * @param createReq
     */
    public YcqYjsStudentCourse createStudentCourse(StudentCourseCreateReq createReq) {
        try {
            log.info("StudentCourseManager.createStudentCourse.param：{}", createReq.toString());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                createReq.setCreatedName(currWorker.getWorkerName());
            }
            createReq.setCreatedTime(createDate);
            // 校验是否存在
            StudentCourseReq studentCourseReq = new StudentCourseReq();
            studentCourseReq.setStudentId(createReq.getStudentId());
            studentCourseReq.setSpecialId(createReq.getSpecialId());
            studentCourseReq.setSpecialType(createReq.getSpecialType());
            studentCourseReq.setCourseIdList(Arrays.asList(createReq.getCourseId()));
            List<YcqYjsStudentCourse> studentCourseList = studentCourseService.selectStudentCourseInfoList(studentCourseReq);
            if (!CollectionUtils.isEmpty(studentCourseList)) {
                throw new BusinessException("学生课程数据已维护，操作失败");
            }
            createReq.setIfAnswer(IfFlagEnum.FALSE.code());
            createReq.setLearnProgress(10);// 新建学生课程进度10%
            createReq.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
            createReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
            createReq.setGroupId(createReq.getSpecialId() + ":" + createReq.getCourseId());
            YcqYjsStudentCourse studentCourse = studentCourseService.insertOne(createReq);
            // 子课程
            Result<YcqYjsCourseMgrResp> ycqYjsCourseMgrRespResult = studyCenterService.queryById(createReq.getCourseId());
            YcqYjsCourseMgrResp courseMgrResp = ycqYjsCourseMgrRespResult.getData();
            if (courseMgrResp == null) {
                throw new BusinessException("课程数据不存在，操作失败");
            }
            if (courseMgrResp.getIsSubcourse() != null && courseMgrResp.getIsSubcourse().intValue() == IfFlagEnum.TRUE.code()) {
                List<YcqYjsCoursesubMgrResp> subCourseList = courseMgrResp.getSubCourseList();
                if (!CollectionUtils.isEmpty(subCourseList)) {
                    List<StudentSubCourseCreateReq> studentSubCourseCreateReqList = new ArrayList<>();
                    for (YcqYjsCoursesubMgrResp ycqYjsCoursesubMgrResp : subCourseList) {
                        StudentSubCourseCreateReq item = new StudentSubCourseCreateReq();
                        item.setSubCourseId(ycqYjsCoursesubMgrResp.getId());
                        item.setStudentId(createReq.getStudentId());
                        item.setCourseId(createReq.getCourseId());
                        item.setStudentCourseId(studentCourse.getId());
                        item.setCourseVideoDuration(null);
                        item.setViewVideoDuration(0);
                        item.setViewProgress(0);
                        item.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
                        item.setCompleteTime(null);
                        item.setDelFlag(DelFlagEnum.UN_DELETE.code());
                        item.setCreatedId(createReq.getCreatedId());
                        item.setCreatedName(createReq.getCreatedName());
                        item.setCreatedTime(createReq.getCreatedTime());
                        studentSubCourseCreateReqList.add(item);
                    }
                    if (!CollectionUtils.isEmpty(studentSubCourseCreateReqList)) {
                        studentSubCourseService.insertBatch(studentSubCourseCreateReqList);
                    }
                }
            }
            return studentCourse;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.createStudentCourse.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.createStudentCourse.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public Integer getLearnProgress(YcqYjsStudentCourse studentCourse, YcqYjsCourseMgrResp courseMgrResp, YcqYjsStudent studentInfo, Worker currWorker, Date updateDate) {
        try {
            Integer learnProgress = 0;
            if (studentCourse != null && courseMgrResp != null) {
                if (studentCourse.getCompleteStatus() != null && studentCourse.getCompleteStatus().intValue() == CompleteStatusEnum.COMPLETED.code().intValue()) {
                    learnProgress = 100;
                } else {
                    // 进度计算:
                    learnProgress = 10;
                    // 判断是否有视频
                    String courseId = courseMgrResp.getCourseId();
                    if (StringUtil.isEmpty(courseId)) {
                        learnProgress += 50;
                    } else {
                        Integer viewProgress = studentCourse.getViewProgress() == null ? 0 : studentCourse.getViewProgress();
                        learnProgress += viewProgress / 2;
                    }
                    Integer isTestquestions = courseMgrResp.getIsTestquestions();// 是否有试题(1:有，0:没有)
                    if (isTestquestions != null && isTestquestions.intValue() == 1) {
                        // 有试题并且合格
                        if (studentCourse.getPaperId() != null && studentCourse.getIfPass() != null && studentCourse.getIfPass().intValue() == IfFlagEnum.TRUE.code().intValue()) {
                            learnProgress += 20;
                        }
                    } else {
                        learnProgress += 20;
                    }
                    Integer isUploadvideo = courseMgrResp.getIsUploadvideo();// 是否上传视频(1:有，0:没有)
                    if (isUploadvideo != null && isUploadvideo.intValue() == 1) {
                        if (studentCourse.getIfUploadVideo() != null && studentCourse.getIfUploadVideo().intValue() == IfFlagEnum.TRUE.code()) {
                            learnProgress += 20;
                        }
                    } else {
                        learnProgress += 20;
                    }
                    if (learnProgress.intValue() >= 100) {
                        learnProgress = 100;
                        // 同步档案数据
                        // 需要校验
                        ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                        archiveChangeReq.setStudentId(studentCourse.getStudentId().toString());
                        archiveChangeReq.setActivityType(ActivityTypeEnum.ACTIVITY_TYPE_4.code().toString());
                        archiveChangeReq.setEventType(EventTypeEnum.EVENT_1.code());
                        archiveChangeReq.setEventId(studentCourse.getId().toString());
                        archiveChangeReq.setParticipateTime(updateDate);
                        archiveChangeReq.setClassName(studentInfo.getCurrentGradeName() + studentInfo.getSemesterName());
                        // 在“$时间”，完成“$专题名称$课程名称”学习，并完成视频上传
                        // 在“$时间”，完成“$专题名称$课程名称"学习，并完成答题
                        StringBuffer name = new StringBuffer("完成" + studentCourse.getSpecialName() + "--" + studentCourse.getCourseName() + "学习");
                        if (isTestquestions != null && isTestquestions.intValue() == 1) {
                            name.append("，并完成答题");
                        }
                        if (isUploadvideo != null && isUploadvideo.intValue() == 1) {
                            name.append("，并完成视频上传");
                        }
                        archiveChangeReq.setName(name.toString());
                        archiveService.insert(archiveChangeReq, currWorker);
                    }
                }
            }
            return learnProgress;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.getLearnProgress.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.getLearnProgress.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 列表查询学生课程上传视频
     *
     * @param req
     * @return
     * @throws Exception
     */
    public List<YcqYjsStudentCourseUploadVideo> selectStudentCourseUploadVideoList(StudentCourseUploadVideoReq req) throws Exception {
        log.info("StudentCourseManager.selectStudentCourseUploadVideoList.param：{}", req.toString());
        try {
            return studentCourseUploadVideoService.selectList(req);
        } catch (BusinessException e) {
            log.error("StudentCourseManager.selectStudentCourseUploadVideoList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.selectStudentCourseUploadVideoList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 学生课程上传视频
     *
     * @param req
     */
    @Transactional(rollbackFor = Exception.class)
    public void studentCourseUploadVideo(StudentCourseUploadVideoCreateReq req) throws Exception {
        try {
            log.info("StudentCourseManager.studentCourseUploadVideo.param：{}", req.toString());
            // 校验学生课程是否存在
            YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(req.getStudentCourseId());
            if (ycqYjsStudentCourse == null) {
                throw new BusinessException("学生课程数据未维护，操作失败");
            }
            // 校验学生是否存在
            YcqYjsStudent studentInfo = studentService.selectOneById(ycqYjsStudentCourse.getStudentId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            // 校验课程是否存在
            Result<YcqYjsCourseMgrResp> ycqYjsCourseMgrRespResult = studyCenterService.queryById(ycqYjsStudentCourse.getCourseId());
            YcqYjsCourseMgrResp courseMgrResp = ycqYjsCourseMgrRespResult.getData();
            if (courseMgrResp == null) {
                throw new BusinessException("课程数据不存在，操作失败");
            }
            // 校验最多上传三个视频
            StudentCourseUploadVideoReq studentCourseUploadVideoReq = new StudentCourseUploadVideoReq();
            studentCourseUploadVideoReq.setStudentCourseId(req.getStudentCourseId());
            List<YcqYjsStudentCourseUploadVideo> studentCourseUploadVideoList = studentCourseUploadVideoService.selectList(studentCourseUploadVideoReq);
            if (!CollectionUtils.isEmpty(studentCourseUploadVideoList) && studentCourseUploadVideoList.size() > 2) {
                throw new BusinessException("最多上传三个视频");
            }
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                req.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                req.setCreatedName(currWorker.getWorkerName());
                ycqYjsStudentCourse.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                ycqYjsStudentCourse.setLastUpdatedName(currWorker.getWorkerName());
            }
            ycqYjsStudentCourse.setLastUpdatedTime(updateDate);
            req.setCreatedTime(updateDate);
            req.setDelFlag(DelFlagEnum.UN_DELETE.code());
            // 保存上传视频
            studentCourseUploadVideoService.insertOne(req);
            ycqYjsStudentCourse.setIfUploadVideo(IfFlagEnum.TRUE.code());
            // 学生课程学习进度
            Integer learnProgress = getLearnProgress(ycqYjsStudentCourse, courseMgrResp, studentInfo, currWorker,updateDate);
            if (learnProgress.intValue() >= 100) {
                ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.COMPLETED.code());
                ycqYjsStudentCourse.setLearnProgress(100);
                if (ycqYjsStudentCourse.getCompleteStatus() == null || ycqYjsStudentCourse.getCompleteStatus().intValue() != CompleteStatusEnum.COMPLETED.code().intValue()) {
                    ycqYjsStudentCourse.setCompleteTime(updateDate);
                }
            } else {
                ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
                ycqYjsStudentCourse.setLearnProgress(learnProgress);
            }
            studentCourseService.updateStudentCourseVideo(BeanCopyUtil.convert(ycqYjsStudentCourse, StudentCourseUploadVideoVo.class));
        } catch (BusinessException e) {
            log.error("StudentCourseManager.studentCourseUploadVideo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.studentCourseUploadVideo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /***
     * 删除学生课程的上传视频
     * @param req
     */
    @Transactional(rollbackFor = Exception.class)
    public void deleteStudentCourseVideo(StudentCourseVideoDeleteReq req) throws Exception {
        try {
            log.info("StudentCourseManager.deleteStudentCourseVideo.param：{}", req.toString());
            // 校验学生课程是否存在
            YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(req.getStudentCourseId());
            if (ycqYjsStudentCourse == null) {
                throw new BusinessException("学生课程数据未维护，操作失败");
            }
            // 校验学生是否存在
            YcqYjsStudent studentInfo = studentService.selectOneById(ycqYjsStudentCourse.getStudentId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            // 校验课程是否存在
            Result<YcqYjsCourseMgrResp> ycqYjsCourseMgrRespResult = studyCenterService.queryById(ycqYjsStudentCourse.getCourseId());
            YcqYjsCourseMgrResp courseMgrResp = ycqYjsCourseMgrRespResult.getData();
            if (courseMgrResp == null) {
                throw new BusinessException("课程数据不存在，操作失败");
            }
            // 校验视频是否存在
            StudentCourseUploadVideoReq studentCourseUploadVideoReq = new StudentCourseUploadVideoReq();
            studentCourseUploadVideoReq.setStudentCourseId(req.getStudentCourseId());
            studentCourseUploadVideoReq.setIdList(Arrays.asList(req.getStudentCourseUploadVideoId()));
            List<YcqYjsStudentCourseUploadVideo> studentCourseUploadVideoList = studentCourseUploadVideoService.selectList(studentCourseUploadVideoReq);
            if (CollectionUtils.isEmpty(studentCourseUploadVideoList)) {
                throw new BusinessException("上传视频数据不存在，操作失败");
            }
            StudentCourseUploadVideoDeleteReq studentCourseUploadVideoDeleteReq = new StudentCourseUploadVideoDeleteReq();
            studentCourseUploadVideoDeleteReq.setIdList(Arrays.asList(req.getStudentCourseUploadVideoId()));
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                studentCourseUploadVideoDeleteReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                studentCourseUploadVideoDeleteReq.setLastUpdatedName(currWorker.getWorkerName());
                ycqYjsStudentCourse.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                ycqYjsStudentCourse.setLastUpdatedName(currWorker.getWorkerName());
            }
            ycqYjsStudentCourse.setLastUpdatedTime(updateDate);
            studentCourseUploadVideoDeleteReq.setLastUpdatedTime(updateDate);
            studentCourseUploadVideoService.deleteBatch(studentCourseUploadVideoDeleteReq);
            // 判断上传视频还存在不
            studentCourseUploadVideoReq.setIdList(null);
            studentCourseUploadVideoList = studentCourseUploadVideoService.selectList(studentCourseUploadVideoReq);
            if (CollectionUtils.isEmpty(studentCourseUploadVideoList)) {
                ycqYjsStudentCourse.setIfUploadVideo(IfFlagEnum.FALSE.code());
                // 学生课程学习进度
                Integer learnProgress = getLearnProgress(ycqYjsStudentCourse, courseMgrResp, studentInfo, currWorker,updateDate);
                if (learnProgress.intValue() >= 100) {
                    ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.COMPLETED.code());
                    ycqYjsStudentCourse.setLearnProgress(100);
                    if (ycqYjsStudentCourse.getCompleteStatus() == null || ycqYjsStudentCourse.getCompleteStatus().intValue() != CompleteStatusEnum.COMPLETED.code().intValue()) {
                        ycqYjsStudentCourse.setCompleteTime(updateDate);
                    }
                } else {
                    ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
                    ycqYjsStudentCourse.setLearnProgress(learnProgress);
                }
                studentCourseService.updateStudentCourseVideo(BeanCopyUtil.convert(ycqYjsStudentCourse, StudentCourseUploadVideoVo.class));
            }
        } catch (BusinessException e) {
            log.error("StudentCourseManager.deleteStudentCourseVideo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.deleteStudentCourseVideo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /***
     * 学生课程视频观看进度
     * @param req
     */
    public void studentCourseVideoViewProgress(StudentCourseVideoViewProgressReq req) {
        try {
            log.info("StudentCourseManager.studentCourseVideoViewProgress.param：{}", req.toString());
            YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(req.getStudentCourseId());
            if (ycqYjsStudentCourse == null) {
                throw new BusinessException("学生课程数据未维护，操作失败");
            }
            // 校验学生是否存在
            YcqYjsStudent studentInfo = studentService.selectOneById(ycqYjsStudentCourse.getStudentId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                ycqYjsStudentCourse.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                ycqYjsStudentCourse.setLastUpdatedName(currWorker.getWorkerName());
            }
            ycqYjsStudentCourse.setLastUpdatedTime(updateDate);
            Integer courseVideoDuration = req.getCourseVideoDuration() == null ? 0 : req.getCourseVideoDuration();
            Integer viewVideoDuration = req.getViewVideoDuration() == null ? 0 : req.getViewVideoDuration();
            Integer oldViewVideoDuration = ycqYjsStudentCourse.getViewVideoDuration() == null ? 0 : ycqYjsStudentCourse.getViewVideoDuration();
            Integer viewProgress = ycqYjsStudentCourse.getViewProgress() == null ? 0 : ycqYjsStudentCourse.getViewProgress();
            // 观看进度大于之前的观看进度 并且没有观看完成
            if (courseVideoDuration != 0 && viewVideoDuration.intValue() >= oldViewVideoDuration.intValue() && viewProgress != 100) {
                ycqYjsStudentCourse.setViewVideoDuration(req.getViewVideoDuration());
                ycqYjsStudentCourse.setCourseVideoDuration(req.getCourseVideoDuration());
                if (courseVideoDuration.intValue() == viewVideoDuration.intValue()) {
                    ycqYjsStudentCourse.setViewProgress(100);
                } else {
                    ycqYjsStudentCourse.setViewProgress((viewVideoDuration * 100) / courseVideoDuration);
                }
                Result<YcqYjsCourseMgrResp> ycqYjsCourseMgrRespResult = studyCenterService.queryById(ycqYjsStudentCourse.getCourseId());
                YcqYjsCourseMgrResp courseMgrResp = ycqYjsCourseMgrRespResult.getData();
                if (courseMgrResp == null) {
                    throw new BusinessException("课程数据不存在，操作失败");
                }
                Integer learnProgress = getLearnProgress(ycqYjsStudentCourse, courseMgrResp, studentInfo, currWorker, updateDate);
                if (learnProgress.intValue() >= 100) {
                    ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.COMPLETED.code());
                    ycqYjsStudentCourse.setLearnProgress(100);
                    if (ycqYjsStudentCourse.getCompleteStatus() == null || ycqYjsStudentCourse.getCompleteStatus().intValue() != CompleteStatusEnum.COMPLETED.code().intValue()) {
                        req.setCompleteTime(updateDate);
                    }
                } else {
                    ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
                    ycqYjsStudentCourse.setLearnProgress(learnProgress);
                }
                StudentCourseVideoViewProgressReq studentCourseVideoViewProgressReq = BeanCopyUtil.convert(ycqYjsStudentCourse, StudentCourseVideoViewProgressReq.class);
                studentCourseVideoViewProgressReq.setStudentCourseId(ycqYjsStudentCourse.getId());
                studentCourseService.studentCourseVideoViewProgress(studentCourseVideoViewProgressReq);
            }
        } catch (BusinessException e) {
            log.error("StudentCourseManager.studentCourseVideoViewProgress.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.studentCourseVideoViewProgress.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 根据学生课程ID列表查询学生子课程信息
     *
     * @param studentCourseId
     * @return
     */
    public List<StudentSubCourseInfoVo> selectStudentSubCourseInfoListByStudentCourseId(BigInteger studentCourseId) {
        try {
            log.info("StudentCourseManager.selectStudentSubCourseInfoListByStudentCourseId.param：{}", studentCourseId);
            List<StudentSubCourseInfoVo> studentSubCourseInfoVos = new ArrayList<>();
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            // 查询学生课程
            YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(studentCourseId);
            if (ycqYjsStudentCourse == null) {
                throw new BusinessException("学生课程数据不存在，操作失败");
            }
            // 子课程
            Result<YcqYjsCourseMgrResp> ycqYjsCourseMgrRespResult = studyCenterService.queryById(ycqYjsStudentCourse.getCourseId());
            YcqYjsCourseMgrResp courseMgrResp = ycqYjsCourseMgrRespResult.getData();
            if (courseMgrResp == null) {
                throw new BusinessException("课程数据不存在，操作失败");
            }
            List<YcqYjsCoursesubMgrResp> subCourseList = courseMgrResp.getSubCourseList();
            if (!CollectionUtils.isEmpty(subCourseList)) {
                // 学生子课程
                StudentSubCourseReq studentSubCourseReq = new StudentSubCourseReq();
                studentSubCourseReq.setStudentCourseId(studentCourseId);
                List<YcqYjsStudentSubCourse> studentSubCourseList = studentSubCourseService.selectList(studentSubCourseReq);
                Map<BigInteger, List<YcqYjsStudentSubCourse>> studentSubCourseMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(studentSubCourseList)) {
                    studentSubCourseMap = studentSubCourseList.stream().collect(Collectors.groupingBy(YcqYjsStudentSubCourse::getSubCourseId));
                    // 删除不维护的数据
                    List<BigInteger> subCourseIdList = subCourseList.stream().map(YcqYjsCoursesubMgrResp::getId).collect(Collectors.toList());
                    if (!CollectionUtils.isEmpty(subCourseIdList)) {
                        List<BigInteger> studentSubCourseIdList = studentSubCourseList.stream().filter(item -> item != null && (item.getSubCourseId() == null || !subCourseIdList.contains(item.getSubCourseId()))).map(YcqYjsStudentSubCourse::getId).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(studentSubCourseIdList)) {
                            StudentSubCourseDeleteReq deleteReq = new StudentSubCourseDeleteReq();
                            deleteReq.setIdList(studentSubCourseIdList);
                            if (currWorker != null) {
                                deleteReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                                deleteReq.setLastUpdatedName(currWorker.getWorkerName());
                            }
                            deleteReq.setLastUpdatedTime(createDate);
                            studentSubCourseService.deleteBatch(deleteReq);
                        }
                    }
                }
                // 查询学生课程ID
                for (YcqYjsCoursesubMgrResp ycqYjsCoursesubMgrResp : subCourseList) {
                    StudentSubCourseInfoVo item = new StudentSubCourseInfoVo();
                    BeanUtils.copyProperties(ycqYjsCoursesubMgrResp, item);
                    List<YcqYjsStudentSubCourse> ycqYjsStudentSubCourses = studentSubCourseMap.get(ycqYjsCoursesubMgrResp.getId());
                    if (!CollectionUtils.isEmpty(ycqYjsStudentSubCourses)) {
                        YcqYjsStudentSubCourse ycqYjsStudentSubCourse = ycqYjsStudentSubCourses.get(0);
                        item.setCourseVideoDuration(ycqYjsStudentSubCourse.getCourseVideoDuration());
                        item.setViewVideoDuration(ycqYjsStudentSubCourse.getViewVideoDuration());
                        item.setViewProgress(ycqYjsStudentSubCourse.getViewProgress());
                        item.setCompleteStatus(ycqYjsStudentSubCourse.getCompleteStatus());
                        item.setCompleteTime(ycqYjsStudentSubCourse.getCompleteTime());
                        item.setStudentCourseId(ycqYjsStudentSubCourse.getStudentCourseId());
                        item.setStudentId(ycqYjsStudentSubCourse.getStudentId());
                        item.setStudentSubCourseId(ycqYjsStudentSubCourse.getId());
                    } else {
                        item.setCourseVideoDuration(null);
                        item.setViewVideoDuration(0);
                        item.setViewProgress(0);
                        item.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
                        item.setCompleteTime(null);
                        item.setStudentCourseId(ycqYjsStudentCourse.getId());
                        item.setStudentId(ycqYjsStudentCourse.getStudentId());
                        item.setStudentSubCourseId(null);
                    }
                    item.setSubCourseId(ycqYjsCoursesubMgrResp.getId());
                    item.setSubCourseNum(ycqYjsCoursesubMgrResp.getSubCourseId());
                    item.setCourseDuration(ycqYjsCoursesubMgrResp.getCourseDuration() == null ? 0 : ycqYjsCoursesubMgrResp.getCourseDuration());
                    studentSubCourseInfoVos.add(item);
                }
            }
            return studentSubCourseInfoVos;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.selectStudentSubCourseInfoListByStudentCourseId.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.selectStudentSubCourseInfoListByStudentCourseId.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 新建学生子课程
     *
     * @param createReq
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public YcqYjsStudentSubCourse createStudentSubCourse(StudentSubCourseCreateReq createReq) throws Exception {
        try {
            log.info("StudentCourseManager.createStudentSubCourse.param：{}", createReq.toString());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                createReq.setCreatedName(currWorker.getWorkerName());
            }
            createReq.setCreatedTime(createDate);
            // 查询学生课程
            YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(createReq.getStudentCourseId());
            if (ycqYjsStudentCourse == null) {
                throw new BusinessException("学生课程数据不存在，操作失败");
            }
            // 判断子课程是否存在
            StudentSubCourseReq studentSubCourseReq = new StudentSubCourseReq();
            studentSubCourseReq.setStudentCourseId(createReq.getStudentCourseId());
            studentSubCourseReq.setSubCourseId(createReq.getSubCourseId());
            List<YcqYjsStudentSubCourse> studentSubCourseList = studentSubCourseService.selectList(studentSubCourseReq);
            if (!CollectionUtils.isEmpty(studentSubCourseList)) {
                throw new BusinessException("学生子课程数据已维护，操作失败");
            }
            // 整合数据
            StudentSubCourseCreateReq item = new StudentSubCourseCreateReq();
            item.setSubCourseId(createReq.getSubCourseId());
            item.setStudentId(ycqYjsStudentCourse.getStudentId());
            item.setCourseId(ycqYjsStudentCourse.getCourseId());
            item.setStudentCourseId(ycqYjsStudentCourse.getId());
            item.setCourseVideoDuration(null);
            item.setViewVideoDuration(0);
            item.setViewProgress(0);
            item.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
            item.setCompleteTime(null);
            item.setDelFlag(DelFlagEnum.UN_DELETE.code());
            item.setCreatedId(createReq.getCreatedId());
            item.setCreatedName(createReq.getCreatedName());
            item.setCreatedTime(createReq.getCreatedTime());
            return studentSubCourseService.insertOne(item);
        } catch (BusinessException e) {
            log.error("StudentCourseManager.createStudentSubCourse.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.createStudentSubCourse.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 学生子课程视频观看进度
     *
     * @param req
     */
    @Transactional(rollbackFor = Exception.class)
    public BigInteger studentSubCourseVideoViewProgress(StudentSubCourseViewProgressReq req) throws Exception {
        try {
            log.info("StudentCourseManager.studentSubCourseVideoViewProgress.param：{}", req.toString());
            // 查询学生课程
            YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(req.getStudentCourseId());
            if (ycqYjsStudentCourse == null) {
                throw new BusinessException("学生课程数据不存在，操作失败");
            }
            // 校验学生是否存在
            YcqYjsStudent studentInfo = studentService.selectOneById(ycqYjsStudentCourse.getStudentId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            Result<YcqYjsCourseMgrResp> ycqYjsCourseMgrRespResult = studyCenterService.queryById(ycqYjsStudentCourse.getCourseId());
            YcqYjsCourseMgrResp courseMgrResp = ycqYjsCourseMgrRespResult.getData();
            if (courseMgrResp == null) {
                throw new BusinessException("课程数据不存在，操作失败");
            }
            if (req.getStudentSubCourseId() == null) {
                // 新增
                StudentSubCourseCreateReq studentSubCourseCreateReq = new StudentSubCourseCreateReq();
                studentSubCourseCreateReq.setSubCourseId(req.getSubCourseId());
                studentSubCourseCreateReq.setStudentCourseId(req.getStudentCourseId());
                YcqYjsStudentSubCourse studentSubCourse = createStudentSubCourse(studentSubCourseCreateReq);
                req.setStudentSubCourseId(studentSubCourse.getId());
            }
            YcqYjsStudentSubCourse studentSubCourse = studentSubCourseService.selectOneById(req.getStudentSubCourseId());
            if (studentSubCourse == null) {
                throw new BusinessException("学生子课程数据未维护，操作失败");
            }
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                studentSubCourse.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                studentSubCourse.setLastUpdatedName(currWorker.getWorkerName());

                ycqYjsStudentCourse.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                ycqYjsStudentCourse.setLastUpdatedName(currWorker.getWorkerName());
            }
            studentSubCourse.setLastUpdatedTime(updateDate);
            ycqYjsStudentCourse.setLastUpdatedTime(updateDate);

            Integer courseVideoDuration = req.getCourseVideoDuration() == null ? 0 : req.getCourseVideoDuration();
            Integer viewVideoDuration = req.getViewVideoDuration() == null ? 0 : req.getViewVideoDuration();
            Integer oldViewVideoDuration = studentSubCourse.getViewVideoDuration() == null ? 0 : studentSubCourse.getViewVideoDuration();
            Integer completeStatus = studentSubCourse.getCompleteStatus() == null ? CompleteStatusEnum.UN_COMPLETE.code() : studentSubCourse.getCompleteStatus();
            // 观看进度大于之前的观看进度 并且没有观看完成
            if (courseVideoDuration != 0 && viewVideoDuration.intValue() > oldViewVideoDuration.intValue() && completeStatus != CompleteStatusEnum.COMPLETED.code()) {
                studentSubCourse.setCourseVideoDuration(courseVideoDuration);
                studentSubCourse.setViewVideoDuration(viewVideoDuration);
                if (courseVideoDuration.intValue() == viewVideoDuration.intValue()) {
                    studentSubCourse.setViewProgress(100);
                    studentSubCourse.setCompleteStatus(CompleteStatusEnum.COMPLETED.code());
                    studentSubCourse.setCompleteTime(updateDate);
                } else {
                    studentSubCourse.setViewProgress((viewVideoDuration * 100) / courseVideoDuration);
                    studentSubCourse.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
                    studentSubCourse.setCompleteTime(null);
                }
                studentSubCourseService.updateOne(BeanCopyUtil.convert(studentSubCourse, StudentSubCourseUpdateReq.class));
            }
            // 更新观看进度
            List<StudentSubCourseInfoVo> studentSubCourseInfoVos = selectStudentSubCourseInfoListByStudentCourseId(ycqYjsStudentCourse.getId());
            if (!CollectionUtils.isEmpty(studentSubCourseInfoVos)) {
                // 40 50 60
                Integer totalViewProgress = studentSubCourseInfoVos.stream().filter(item -> item != null && item.getViewProgress() != null).map(StudentSubCourseInfoVo::getViewProgress).reduce((x, y) -> x + y).orElse(0);
                int size = studentSubCourseInfoVos.size();
                ycqYjsStudentCourse.setViewProgress(totalViewProgress / size);
            }
            // 更新课程的学习进度
            Integer learnProgress = getLearnProgress(ycqYjsStudentCourse, courseMgrResp, studentInfo, currWorker,updateDate);
            if (learnProgress.intValue() >= 100) {
                ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.COMPLETED.code());
                ycqYjsStudentCourse.setLearnProgress(100);
                if (ycqYjsStudentCourse.getCompleteStatus() == null || ycqYjsStudentCourse.getCompleteStatus().intValue() != CompleteStatusEnum.COMPLETED.code().intValue()) {
                    ycqYjsStudentCourse.setCompleteTime(updateDate);
                }
            } else {
                ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
                ycqYjsStudentCourse.setLearnProgress(learnProgress);
            }
            StudentCourseVideoViewProgressReq studentCourseVideoViewProgressReq = BeanCopyUtil.convert(ycqYjsStudentCourse, StudentCourseVideoViewProgressReq.class);
            studentCourseVideoViewProgressReq.setStudentCourseId(ycqYjsStudentCourse.getId());
            studentCourseService.studentCourseVideoViewProgress(studentCourseVideoViewProgressReq);
            return req.getStudentSubCourseId();
        } catch (BusinessException e) {
            log.error("StudentCourseManager.studentSubCourseVideoViewProgress.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.studentSubCourseVideoViewProgress.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 减考试次数-新增式
     *
     * @param updateReq
     * @throws Exception
     */
    public YcqYjsStudentCourse minusResidueTestCount(MinusResidueTestCountReq updateReq) throws Exception {
        try {
            log.info("StudentCourseManager.submitStudentTest.param：{}", updateReq.toString());
            YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(updateReq.getStudentCourseId());
            if (ycqYjsStudentCourse == null) {
                throw new BusinessException("学生课程数据未维护，操作失败");
            }
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                ycqYjsStudentCourse.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                ycqYjsStudentCourse.setLastUpdatedName(currWorker.getWorkerName());
            }
            ycqYjsStudentCourse.setLastUpdatedTime(createDate);
            Integer allowTimes = updateReq.getAllowTimes();
            // 计算剩余考试次数
            if (allowTimes != null) {
                // 不是第一次考试 且 同一套试卷
                if (ycqYjsStudentCourse.getPaperId() != null && (ycqYjsStudentCourse.getPaperId()).equals(updateReq.getPaperId())) {
                    allowTimes = ycqYjsStudentCourse.getResidueTestCount();
                }
                if (allowTimes > 0) {
                    allowTimes--;
                } else {
                    allowTimes = 0;
                }
            }
            ycqYjsStudentCourse.setResidueTestCount(allowTimes);
            ycqYjsStudentCourse.setPaperId(updateReq.getPaperId());
            // 更新学生课程数据
            studentCourseService.studentCourseSubmitTest(BeanCopyUtil.convert(ycqYjsStudentCourse, StudentCourseSubmitTestReq.class));
            return ycqYjsStudentCourse;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.submitStudentTest.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.submitStudentTest.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 学生交卷-新增式
     *
     * @param createReq
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public YcqYjsStudentTest submitStudentTest(StudentTestCreateReq createReq) throws Exception {
        try {
            log.info("StudentCourseManager.submitStudentTest.param：{}", createReq.toString());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                createReq.setCreatedName(currWorker.getWorkerName());
            }
            createReq.setCreatedTime(createDate);
            createReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
            // 计算考试分数
            List<StudentQuestionCreateReq> studentQuestionCreateReqList = createReq.getStudentQuestionCreateReqList();
            if (CollectionUtils.isEmpty(studentQuestionCreateReqList)) {
                throw new BusinessException("学生答题不能为空");
            }
            YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(createReq.getStudentCourseId());
            if (ycqYjsStudentCourse == null) {
                throw new BusinessException("学生课程数据未维护，操作失败");
            }
            // 校验学生是否存在
            YcqYjsStudent studentInfo = studentService.selectOneById(ycqYjsStudentCourse.getStudentId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            ycqYjsStudentCourse.setPaperId(createReq.getPaperId());
            // 课程进度
            Result<YcqYjsCourseMgrResp> ycqYjsCourseMgrRespResult = studyCenterService.queryById(ycqYjsStudentCourse.getCourseId());
            YcqYjsCourseMgrResp courseMgrResp = ycqYjsCourseMgrRespResult.getData();
            if (courseMgrResp == null) {
                throw new BusinessException("课程数据不存在，操作失败");
            }
            // 组装答卷数据
            Integer singleScore = createReq.getSingleScore() == null ? 0 : createReq.getSingleScore();// 单选题分数
            Integer multiScore = createReq.getMultiScore() == null ? 0 : createReq.getMultiScore();// 多选题分数
            Integer judgeScore = createReq.getJudgeScore() == null ? 0 : createReq.getJudgeScore();// 判断题分数
            studentQuestionCreateReqList.stream().forEach(item -> {
                Integer questionScore = 0;
                if (item.getQuestionType() != null && item.getQuestionType().intValue() == QuestionTypeEnum.SINGLE.code().intValue()) {
                    questionScore = singleScore;
                } else if (item.getQuestionType() != null && item.getQuestionType().intValue() == QuestionTypeEnum.MULTI.code().intValue()) {
                    questionScore = multiScore;
                } else if (item.getQuestionType() != null && item.getQuestionType().intValue() == QuestionTypeEnum.JUDGE.code().intValue()) {
                    questionScore = judgeScore;
                }
                String rightOptionIds = CommonUtil.transferListToStr(item.getRightOptionIdList());
                String answerOptionIds = CommonUtil.transferListToStr(item.getAnswerOptionIdList());
                Integer answerQuestionScore = 0;
                Integer ifAnswerRight = IfFlagEnum.FALSE.code();
                if (!StringUtil.isEmpty(rightOptionIds) && !StringUtil.isEmpty(answerOptionIds) && rightOptionIds.equals(answerOptionIds)) {
                    answerQuestionScore = questionScore;
                    ifAnswerRight = IfFlagEnum.TRUE.code();
                }
                item.setQuestionScore(questionScore);// 题目分数
                item.setAnswerQuestionScore(answerQuestionScore);// 答题得分
                item.setRightOptionIds(rightOptionIds);// 正确选项ID列表，逗号隔开
                item.setAnswerOptionIds(answerOptionIds);// 答题选项ID列表，逗号隔开
                item.setIfAnswerRight(ifAnswerRight);// 是否答对：0-否；1-是
                item.setCreatedId(createReq.getCreatedId());
                item.setCreatedName(createReq.getCreatedName());
                item.setCreatedTime(createReq.getCreatedTime());
                item.setDelFlag(DelFlagEnum.UN_DELETE.code());
            });
            // 答题得分
            Integer passScore = createReq.getPassScore();
            List<Integer> trueAnswerQuestionScoreList = studentQuestionCreateReqList.stream().filter(item -> item != null && item.getIfAnswerRight() != null && item.getIfAnswerRight().intValue() == IfFlagEnum.TRUE.code().intValue() && item.getAnswerQuestionScore() != null).map(StudentQuestionCreateReq::getAnswerQuestionScore).collect(Collectors.toList());
            Integer answerScore = CollectionUtils.isEmpty(trueAnswerQuestionScoreList) ? 0 : trueAnswerQuestionScoreList.stream().reduce((x, y) -> x + y).orElse(0);
            // 是否合格：0-不合格；1-合格
            Integer ifPass = IfFlagEnum.FALSE.code();
            if (passScore != null && answerScore != null && answerScore.intValue() >= passScore.intValue()) {
                ifPass = IfFlagEnum.TRUE.code();
            }
            // 答对题目数
            List<StudentQuestionCreateReq> rightQuestionList = studentQuestionCreateReqList.stream().filter(item -> item != null && item.getIfAnswerRight() != null && item.getIfAnswerRight().intValue() == IfFlagEnum.TRUE.code().intValue()).collect(Collectors.toList());
            Integer rightQuestionCount = CollectionUtils.isEmpty(rightQuestionList) ? 0 : rightQuestionList.size();
            Integer questionCount = studentQuestionCreateReqList.size();
            createReq.setAnswerScore(answerScore);
            createReq.setIfPass(ifPass);
            createReq.setRightQuestionCount(rightQuestionCount);
            createReq.setErrorQuestionCount(questionCount - rightQuestionCount);
            createReq.setIfAnswer(IfFlagEnum.TRUE.code());
            // 保存学生答卷
            YcqYjsStudentTest ycqYjsStudentTest = studentTestService.insertOne(createReq);
            // 保存学生答题
            studentQuestionCreateReqList.stream().forEach(item -> {
                item.setStudentTestId(ycqYjsStudentTest.getId());
            });
            studentQuestionService.insertBatch(studentQuestionCreateReqList);
            // 查询最大分数
            StudentTestReq studentTestReq = new StudentTestReq();
            studentTestReq.setStudentCourseId(createReq.getStudentCourseId());
            studentTestReq.setPaperId(createReq.getPaperId());
            YcqYjsStudentTest ycqYjsStudentTest1 = studentTestService.selectMaxStudentTest(studentTestReq);
            ycqYjsStudentCourse.setAnswerScore(ycqYjsStudentTest1.getAnswerScore());
            ycqYjsStudentCourse.setIfPass(ycqYjsStudentTest1.getIfPass());
            Integer learnProgress = getLearnProgress(ycqYjsStudentCourse, courseMgrResp, studentInfo, currWorker,createDate);
            if (learnProgress.intValue() >= 100) {
                ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.COMPLETED.code());
                ycqYjsStudentCourse.setLearnProgress(100);
                if (ycqYjsStudentCourse.getCompleteStatus() == null || ycqYjsStudentCourse.getCompleteStatus().intValue() != CompleteStatusEnum.COMPLETED.code().intValue()) {
                    ycqYjsStudentCourse.setCompleteTime(createDate);
                }
            } else {
                ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
                ycqYjsStudentCourse.setLearnProgress(learnProgress);
            }
            ycqYjsStudentCourse.setLastUpdatedId(createReq.getCreatedId());
            ycqYjsStudentCourse.setLastUpdatedName(createReq.getCreatedName());
            ycqYjsStudentCourse.setLastUpdatedTime(createReq.getCreatedTime());
            ycqYjsStudentCourse.setIfAnswer(IfFlagEnum.TRUE.code());
            // 更新学生课程数据
            studentCourseService.studentCourseSubmitTest(BeanCopyUtil.convert(ycqYjsStudentCourse, StudentCourseSubmitTestReq.class));
            return ycqYjsStudentTest;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.submitStudentTest.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.submitStudentTest.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 初始化答卷并扣减考试次数-更新式
     *
     * @param updateReq
     * @throws Exception
     */
    public InitStudentTestVo initStudentTest(InitStudentTestReq updateReq) throws Exception {
        try {
            log.info("StudentCourseManager.initStudentTest.param：{}", updateReq.toString());
            YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(updateReq.getStudentCourseId());
            if (ycqYjsStudentCourse == null) {
                throw new BusinessException("学生课程数据未维护，操作失败");
            }
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                ycqYjsStudentCourse.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                ycqYjsStudentCourse.setLastUpdatedName(currWorker.getWorkerName());
            }
            ycqYjsStudentCourse.setLastUpdatedTime(createDate);
            Integer allowTimes = updateReq.getAllowTimes();
            // 计算剩余考试次数
            if (allowTimes != null) {
                // 不是第一次考试 且 同一套试卷
                if (ycqYjsStudentCourse.getPaperId() != null && (ycqYjsStudentCourse.getPaperId()).equals(updateReq.getPaperId())) {
                    allowTimes = ycqYjsStudentCourse.getResidueTestCount();
                }
                if (allowTimes > 0) {
                    allowTimes--;
                } else {
                    allowTimes = 0;
                }
            }
            ycqYjsStudentCourse.setResidueTestCount(allowTimes);
            ycqYjsStudentCourse.setPaperId(updateReq.getPaperId());
            // 更新学生课程数据
            studentCourseService.studentCourseSubmitTest(BeanCopyUtil.convert(ycqYjsStudentCourse, StudentCourseSubmitTestReq.class));
            // 新增答卷
            StudentTestCreateReq studentTestCreateReq = new StudentTestCreateReq();
            studentTestCreateReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
            if (currWorker != null) {
                studentTestCreateReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                studentTestCreateReq.setCreatedName(currWorker.getWorkerName());
            }
            studentTestCreateReq.setCreatedTime(createDate);
            studentTestCreateReq.setStudentId(ycqYjsStudentCourse.getStudentId());
            studentTestCreateReq.setStudentCourseId(ycqYjsStudentCourse.getId());
            studentTestCreateReq.setPaperId(updateReq.getPaperId());
            studentTestCreateReq.setPaperName(updateReq.getPaperName());
            studentTestCreateReq.setAnswerScore(0);
            studentTestCreateReq.setAnswerDuration(0);
            studentTestCreateReq.setIfAnswer(IfFlagEnum.FALSE.code());
            studentTestCreateReq.setIfPass(IfFlagEnum.FALSE.code());
            studentTestCreateReq.setIfShowScore(updateReq.getIfShowScore());
            studentTestCreateReq.setIfExam(updateReq.getIfExam());
            studentTestCreateReq.setTestScore(updateReq.getTestScore());
            studentTestCreateReq.setPassScore(updateReq.getPassScore());
            studentTestCreateReq.setQuestionCount(updateReq.getQuestionCount());
            studentTestCreateReq.setRightQuestionCount(0);
            studentTestCreateReq.setErrorQuestionCount(updateReq.getQuestionCount());
            YcqYjsStudentTest ycqYjsStudentTest = studentTestService.insertOne(studentTestCreateReq);
            InitStudentTestVo initStudentTestVo = new InitStudentTestVo();
            initStudentTestVo.setResidueTestCount(allowTimes);
            initStudentTestVo.setStudentTestId(ycqYjsStudentTest.getId());
            return initStudentTestVo;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.initStudentTest.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.initStudentTest.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 学生交卷-更新式
     *
     * @param updateReq
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public YcqYjsStudentTest submitStudentTestByUpdate(StudentTestUpdateReq updateReq) throws Exception {
        try {
            log.info("StudentCourseManager.submitStudentTestByUpdate.param：{}", updateReq.toString());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            updateReq.setLastUpdatedTime(createDate);
            // 学生答卷
            YcqYjsStudentTest studentTest = studentTestService.selectOneById(updateReq.getId());
            if(studentTest == null){
                throw new BusinessException("学生答卷数据未初始化，操作失败");
            }
            if(studentTest.getIfAnswer() == null || studentTest.getIfAnswer().intValue() != IfFlagEnum.FALSE.code()){
                throw new BusinessException("当前学生已交卷， 不能重复交卷");
            }
            if(updateReq.getStudentId() == null || !updateReq.getStudentId().equals(studentTest.getStudentId())){
                throw new BusinessException("不允许提交别人的答卷，操作失败");
            }
            // 校验学生是否存在
            YcqYjsStudent studentInfo = studentService.selectOneById(updateReq.getStudentId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            // 计算考试分数
            List<StudentQuestionCreateReq> studentQuestionCreateReqList = updateReq.getStudentQuestionCreateReqList();
            if (CollectionUtils.isEmpty(studentQuestionCreateReqList)) {
                throw new BusinessException("学生答题不能为空");
            }
            YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(studentTest.getStudentCourseId());
            if (ycqYjsStudentCourse == null) {
                throw new BusinessException("学生课程数据未维护，操作失败");
            }
            ycqYjsStudentCourse.setPaperId(studentTest.getPaperId());
            // 课程进度
            Result<YcqYjsCourseMgrResp> ycqYjsCourseMgrRespResult = studyCenterService.queryById(ycqYjsStudentCourse.getCourseId());
            YcqYjsCourseMgrResp courseMgrResp = ycqYjsCourseMgrRespResult.getData();
            if (courseMgrResp == null) {
                throw new BusinessException("课程数据不存在，操作失败");
            }
            // 组装答卷数据
            Integer singleScore = updateReq.getSingleScore() == null ? 0 : updateReq.getSingleScore();// 单选题分数
            Integer multiScore = updateReq.getMultiScore() == null ? 0 : updateReq.getMultiScore();// 多选题分数
            Integer judgeScore = updateReq.getJudgeScore() == null ? 0 : updateReq.getJudgeScore();// 判断题分数
            studentQuestionCreateReqList.stream().forEach(item -> {
                Integer questionScore = 0;
                if (item.getQuestionType() != null && item.getQuestionType().intValue() == QuestionTypeEnum.SINGLE.code().intValue()) {
                    questionScore = singleScore;
                } else if (item.getQuestionType() != null && item.getQuestionType().intValue() == QuestionTypeEnum.MULTI.code().intValue()) {
                    questionScore = multiScore;
                } else if (item.getQuestionType() != null && item.getQuestionType().intValue() == QuestionTypeEnum.JUDGE.code().intValue()) {
                    questionScore = judgeScore;
                }
                String rightOptionIds = CommonUtil.transferListToStr(item.getRightOptionIdList());
                String answerOptionIds = CommonUtil.transferListToStr(item.getAnswerOptionIdList());
                Integer answerQuestionScore = 0;
                Integer ifAnswerRight = IfFlagEnum.FALSE.code();
                if (!StringUtil.isEmpty(rightOptionIds) && !StringUtil.isEmpty(answerOptionIds) && rightOptionIds.equals(answerOptionIds)) {
                    answerQuestionScore = questionScore;
                    ifAnswerRight = IfFlagEnum.TRUE.code();
                }
                item.setQuestionScore(questionScore);// 题目分数
                item.setAnswerQuestionScore(answerQuestionScore);// 答题得分
                item.setRightOptionIds(rightOptionIds);// 正确选项ID列表，逗号隔开
                item.setAnswerOptionIds(answerOptionIds);// 答题选项ID列表，逗号隔开
                item.setIfAnswerRight(ifAnswerRight);// 是否答对：0-否；1-是
                item.setCreatedId(updateReq.getLastUpdatedId());
                item.setCreatedName(updateReq.getLastUpdatedName());
                item.setCreatedTime(updateReq.getLastUpdatedTime());
                item.setDelFlag(DelFlagEnum.UN_DELETE.code());
            });
            // 答题得分
            Integer passScore = updateReq.getPassScore();
            List<Integer> trueAnswerQuestionScoreList = studentQuestionCreateReqList.stream().filter(item -> item != null && item.getIfAnswerRight() != null && item.getIfAnswerRight().intValue() == IfFlagEnum.TRUE.code().intValue() && item.getAnswerQuestionScore() != null).map(StudentQuestionCreateReq::getAnswerQuestionScore).collect(Collectors.toList());
            Integer answerScore = CollectionUtils.isEmpty(trueAnswerQuestionScoreList) ? 0 : trueAnswerQuestionScoreList.stream().reduce((x, y) -> x + y).orElse(0);
            // 是否合格：0-不合格；1-合格
            Integer ifPass = IfFlagEnum.FALSE.code();
            if (passScore != null && answerScore != null && answerScore.intValue() >= passScore.intValue()) {
                ifPass = IfFlagEnum.TRUE.code();
            }
            // 答对题目数
            List<StudentQuestionCreateReq> rightQuestionList = studentQuestionCreateReqList.stream().filter(item -> item != null && item.getIfAnswerRight() != null && item.getIfAnswerRight().intValue() == IfFlagEnum.TRUE.code().intValue()).collect(Collectors.toList());
            Integer rightQuestionCount = CollectionUtils.isEmpty(rightQuestionList) ? 0 : rightQuestionList.size();
            Integer questionCount = studentQuestionCreateReqList.size();
            updateReq.setAnswerScore(answerScore);
            updateReq.setIfPass(ifPass);
            updateReq.setRightQuestionCount(rightQuestionCount);
            updateReq.setErrorQuestionCount(questionCount - rightQuestionCount);
            updateReq.setIfAnswer(IfFlagEnum.TRUE.code());
            // 更新学生答卷
            YcqYjsStudentTest ycqYjsStudentTest = studentTestService.updateOne(updateReq);
            // 保存学生答题
            studentQuestionCreateReqList.stream().forEach(item -> {
                item.setStudentTestId(updateReq.getId());
            });
            studentQuestionService.insertBatch(studentQuestionCreateReqList);
            // 查询最大分数
            StudentTestReq studentTestReq = new StudentTestReq();
            studentTestReq.setStudentCourseId(studentTest.getStudentCourseId());
            studentTestReq.setPaperId(studentTest.getPaperId());
            YcqYjsStudentTest ycqYjsStudentTest1 = studentTestService.selectMaxStudentTest(studentTestReq);
            ycqYjsStudentCourse.setAnswerScore(ycqYjsStudentTest1.getAnswerScore());
            ycqYjsStudentCourse.setIfPass(ycqYjsStudentTest1.getIfPass());
            Integer learnProgress = getLearnProgress(ycqYjsStudentCourse, courseMgrResp, studentInfo, currWorker,createDate);
            if (learnProgress.intValue() >= 100) {
                ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.COMPLETED.code());
                ycqYjsStudentCourse.setLearnProgress(100);
                if (ycqYjsStudentCourse.getCompleteStatus() == null || ycqYjsStudentCourse.getCompleteStatus().intValue() != CompleteStatusEnum.COMPLETED.code().intValue()) {
                    ycqYjsStudentCourse.setCompleteTime(createDate);
                }
            } else {
                ycqYjsStudentCourse.setCompleteStatus(CompleteStatusEnum.UN_COMPLETE.code());
                ycqYjsStudentCourse.setLearnProgress(learnProgress);
            }
            ycqYjsStudentCourse.setLastUpdatedId(updateReq.getLastUpdatedId());
            ycqYjsStudentCourse.setLastUpdatedName(updateReq.getLastUpdatedName());
            ycqYjsStudentCourse.setLastUpdatedTime(updateReq.getLastUpdatedTime());
            ycqYjsStudentCourse.setIfAnswer(IfFlagEnum.TRUE.code());
            // 更新学生课程数据
            studentCourseService.studentCourseSubmitTest(BeanCopyUtil.convert(ycqYjsStudentCourse, StudentCourseSubmitTestReq.class));
            return ycqYjsStudentTest;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.submitStudentTestByUpdate.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.submitStudentTestByUpdate.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 查询学生答卷详情
     *
     * @param studentTestId
     * @return
     */
    public StudentTestInfoVo selectStudentTestInfo(BigInteger studentTestId) {
        try {
            log.info("StudentCourseManager.selectStudentTestInfo.param：{}", studentTestId);
            StudentTestInfoVo studentTestInfoVo = new StudentTestInfoVo();
            YcqYjsStudentTest ycqYjsStudentTest = studentTestService.selectOneById(studentTestId);
            if (ycqYjsStudentTest != null) {
                BeanUtils.copyProperties(ycqYjsStudentTest, studentTestInfoVo);
                studentTestInfoVo.setPaperId(String.valueOf(ycqYjsStudentTest.getPaperId()));
                // 学生答题列表
                StudentQuestionReq studentQuestionReq = new StudentQuestionReq();
                studentQuestionReq.setStudentTestId(ycqYjsStudentTest.getId());
                List<YcqYjsStudentQuestion> ycqYjsStudentQuestions = studentQuestionService.selectList(studentQuestionReq);
                if (!CollectionUtils.isEmpty(ycqYjsStudentQuestions)) {
                    List<StudentQuestionInfoVo> studentQuestionInfoVoList = BeanCopyUtil.convert(ycqYjsStudentQuestions, StudentQuestionInfoVo.class);
                    studentTestInfoVo.setStudentQuestionInfoVoList(studentQuestionInfoVoList);
                }
                // 剩余考试次数
                YcqYjsStudentCourse ycqYjsStudentCourse = studentCourseService.selectOneById(ycqYjsStudentTest.getStudentCourseId());
                if (ycqYjsStudentCourse != null) {
                    studentTestInfoVo.setResidueTestCount(ycqYjsStudentCourse.getResidueTestCount());
                }
                // 排名：学生课程 - 试卷 - 本次考试排名 => 大于当前分数的数量
                StudentTestReq studentTestReq = new StudentTestReq();
                studentTestReq.setAnswerScoreMin(ycqYjsStudentTest.getAnswerScore());
                studentTestReq.setPaperId(ycqYjsStudentTest.getPaperId());
                studentTestReq.setStudentCourseId(ycqYjsStudentTest.getStudentCourseId());
                Integer count = studentTestService.count(studentTestReq);
                studentTestInfoVo.setRankNum(++count);
            }
            return studentTestInfoVo;
        } catch (BusinessException e) {
            log.error("StudentCourseManager.selectStudentTestInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.selectStudentTestInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 分页查询学生答卷
     *
     * @param req
     * @return
     */
    public PageRsp<YcqYjsStudentTest> pageSearchStudentTest(PageReq<StudentTestReq> req) {
        try {
            log.info("StudentCourseManager.pageSearchStudentTest.param：{}", req.toString());
            return studentTestService.pageSearchStudentTest(req);
        } catch (BusinessException e) {
            log.error("StudentCourseManager.pageSearchStudentTest.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.pageSearchStudentTest.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }


    /**
     * 获取视频唯一标识
     *
     * @return
     */
    public String getFileNumber() {
        try {
            log.info("StudentCourseManager.getFileNumber");
            return CharacterCodeUtil.generateCharacterCode(OneAffairConstants.STUDENT_COURSE_VIDEO_NUM_PREFIX);
        } catch (BusinessException e) {
            log.error("StudentCourseManager.getFileNumber.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentCourseManager.getFileNumber.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

}
