package com.crunii.micro.service.oneaffair.manager;

import cn.hutool.core.util.DesensitizedUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.builder.ExcelWriterBuilder;
import com.alibaba.excel.write.builder.ExcelWriterSheetBuilder;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSON;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.BeanUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.SysService;
import com.crunii.micro.feign.client.dto.WorkerC;
import com.crunii.micro.service.oneaffair.constants.OneAffairConstants;
import com.crunii.micro.service.oneaffair.dto.excel.*;
import com.crunii.micro.service.oneaffair.dto.excel.listener.StudentImportExcelListener;
import com.crunii.micro.service.oneaffair.dto.excel.listener.StudentThreeImportExcelListener;
import com.crunii.micro.service.oneaffair.dto.excel.listener.StudentTwoImportExcelListener;
import com.crunii.micro.service.oneaffair.dto.oneaffair.*;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveChangeReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveSelectReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveUpdateReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveVo;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.evaluate.SystemIndexReq;
import com.crunii.micro.service.oneaffair.dto.vo.evaluate.YcqYjsEvaluateIndexVo;
import com.crunii.micro.service.oneaffair.dto.vo.honor.HonorReq;
import com.crunii.micro.service.oneaffair.dto.vo.student.*;
import com.crunii.micro.service.oneaffair.dto.vo.studentBindingUser.StudentBindingUserCreateVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentBindingUser.StudentBindingUserReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourse.StudentCourseReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourse.StudentCourseSumVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.*;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.StudentMedalInfoReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPatriarch.StudentPatriarchReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.*;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.TeacherInfoVo;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.TeacherReq;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.TsmOrgReq;
import com.crunii.micro.service.oneaffair.enums.*;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsServiceDurationRepo;
import com.crunii.micro.service.oneaffair.service.*;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import com.crunii.micro.service.oneaffair.util.CharacterCodeUtil;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import dm.jdbc.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.springframework.data.domain.Example;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

import static com.alibaba.excel.support.ExcelTypeEnum.CSV;

@Slf4j
@Service
public class StudentManager {

    @Resource
    private StudentService studentService;
    @Resource
    private StudentPatriarchService studentPatriarchService;
    @Resource
    private StudentMedalManager studentMedalManager;
    @Resource
    private ArchiveService archiveService;
    @Resource
    private StudentCourseManager studentCourseManager;
    @Resource
    private StudentCourseService studentCourseService;
    @Resource
    private StudentPositionService studentPositionService;
    @Resource
    private StudentHonoraryService studentHonoraryService;
    @Resource
    private EvaluateIndexService evaluateIndexService;
    @Resource
    private SysService sysService;
    @Resource
    private StudentBindingUserService studentBindingUserService;
    @Resource
    private YcqYjsServiceDurationRepo ycqYjsServiceDurationRepo;
    @Resource
    private TeacherService teacherService;
    @Resource
    private TeacherBindingClassService teacherBindingClassService;
    @Resource
    private OrgService orgService;
    @Resource
    private ImportRecordService importRecordService;


    /**
     * 通过家长信息查询学生ID
     *
     * @param patriarchMobile
     * @param patriarchName
     * @return
     */
    private List<BigInteger> getStudentIdsByPatriarchPhoneAndName(String patriarchMobile, String patriarchName) {
        if (patriarchName == null && patriarchMobile == null) {
            return null;
        }
        StudentPatriarchReq studentPatriarchReq = new StudentPatriarchReq();
        studentPatriarchReq.setMobileLike(patriarchMobile);
        studentPatriarchReq.setPatriarchNameLike(patriarchName);
        List<YcqYjsStudentPatriarch> studentInfoVos = studentPatriarchService.selectList(studentPatriarchReq);
        if (CollectionUtils.isEmpty(studentInfoVos)) {
            return null;
        }
        return studentInfoVos.stream().map(YcqYjsStudentPatriarch::getStudentId).distinct().collect(Collectors.toList());
    }

    /**
     * 分页查询
     *
     * @param req
     * @return
     * @throws Exception
     */
    public PageRsp<StudentInfoVo> pageSearchStudentInfo(PageReq<StudentReq> req) throws Exception {
        try {
            log.info("StudentManager.selectStudentInfo.param：{}", req.toString());
            if (req != null && req.getQuery() != null) {
                List<BigInteger> studentIdList = getStudentIdsByPatriarchPhoneAndName(req.getQuery().getPatriarchMobile(), req.getQuery().getPatriarchName());
                // 通过家长信息查询学生ID
                if (!CollectionUtils.isEmpty(studentIdList)) {
                    StudentReq query = req.getQuery();
                    query.setIdList(studentIdList);
                    req.setQuery(query);
                }
                // 过滤
                Worker currWorker = CurrWorkerUtil.getCurrWorker();
                if (currWorker != null) {
                    log.info("StudentManager.currWorker.param：{}", currWorker.toString());
                    TeacherReq teacherReq = new TeacherReq();
                    teacherReq.setUserId(new BigInteger(String.valueOf(currWorker.getLoginId())));
                    List<TeacherInfoVo> teacherInfoVos = teacherService.selectTeacherInfoList(teacherReq);
                    if (!CollectionUtils.isEmpty(teacherInfoVos)) {
                        List<BigInteger> teacherIds = teacherInfoVos.stream().map(TeacherInfoVo::getId).collect(Collectors.toList());
                        List<YcqYjsTeacherBindingClass> teacherBindingClassList = teacherBindingClassService.selectListByTeacherIds(teacherIds);
                        if (!CollectionUtils.isEmpty(teacherBindingClassList)) {
                            // 班级
                            List<BigInteger> classIds = teacherBindingClassList.stream().filter(item -> item != null && item.getClassId() != null).map(YcqYjsTeacherBindingClass::getClassId).collect(Collectors.toList());
                            if (!CollectionUtils.isEmpty(classIds)) {
                                StudentReq query = req.getQuery();
                                query.setBaseClassIds(classIds);
                                req.setQuery(query);
                            }
                        }
                    }
                }
            }
            PageRsp<StudentInfoVo> studentInfoVoPageRsp = studentService.pageSearchStudentInfo(req);
            List<StudentInfoVo> studentInfoVoList = studentInfoVoPageRsp.getResult();
            if (!CollectionUtils.isEmpty(studentInfoVoList)) {
                List<BigInteger> studentIds = studentInfoVoList.stream().map(StudentInfoVo::getId).collect(Collectors.toList());
                // 家长信息
                StudentPatriarchReq studentPatriarchReq = new StudentPatriarchReq();
                studentPatriarchReq.setStudentIdList(studentIds);
                List<YcqYjsStudentPatriarch> studentPatriarcheList = studentPatriarchService.selectList(studentPatriarchReq);
                Map<BigInteger, List<YcqYjsStudentPatriarch>> studentPatriarchMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(studentPatriarcheList)) {
                    studentPatriarchMap = studentPatriarcheList.stream().collect(Collectors.groupingBy(YcqYjsStudentPatriarch::getStudentId));
                }
                // 指标体系 - 12
                SystemIndexReq systemIndexReq = new SystemIndexReq();
                systemIndexReq.setSystemId(new BigInteger("12"));
                systemIndexReq.setIndexLevel(1);
                List<YcqYjsEvaluateIndexVo> evaluateIndexVoList = evaluateIndexService.searchIndexListInfo(systemIndexReq);
                Map<BigInteger, List<YcqYjsStudentPatriarch>> finalStudentPatriarchMap = studentPatriarchMap;
                studentInfoVoList.forEach(item -> {

                    item.setStudentPatriarchList(finalStudentPatriarchMap.get(item.getId()));

                    Integer teacherScore = item.getTeacherScore() == null ? 0 : item.getTeacherScore();
                    // 入团积分计算
                    Integer youthLeagueScore = 0;
                    Double serviceDuration = item.getServiceDuration() == null ? 0 : item.getServiceDuration();// 服务时长: 小时
                    Integer courseDuration = item.getCourseDuration() == null ? 0 : item.getCourseDuration();// 课程时长：分钟
                    int xjMedalCount = item.getXjMedalCount();//星级章个数
                    int studentPositionCount = item.getStudentPositionCount();//学生岗位个数
                    int studentHonoraryCount = item.getStudentHonoraryCount();//学生荣誉称号个数
                    if (!CollectionUtils.isEmpty(evaluateIndexVoList)) {
                        // 志愿服务大于20小时 - 10506
                        List<YcqYjsEvaluateIndexVo> evaluateIndexVoList1 = evaluateIndexVoList.stream().filter(dto -> dto != null && dto.getId() != null && dto.getId().equals(new BigInteger("35"))).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(evaluateIndexVoList1)) {
                            YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo = evaluateIndexVoList1.get(0);
                            int score = ycqYjsEvaluateIndexVo.getScore() == null ? 0 : ycqYjsEvaluateIndexVo.getScore().intValue();
                            if (serviceDuration > 20) {
                                youthLeagueScore += score;
                            }
                        }
                        // 视频学习时长超过3分钟 - 10507
                        List<YcqYjsEvaluateIndexVo> evaluateIndexVoList2 = evaluateIndexVoList.stream().filter(dto -> dto != null && dto.getId() != null && dto.getId().equals(new BigInteger("36"))).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(evaluateIndexVoList2)) {
                            YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo = evaluateIndexVoList2.get(0);
                            int score = ycqYjsEvaluateIndexVo.getScore() == null ? 0 : ycqYjsEvaluateIndexVo.getScore().intValue();
                            if (courseDuration > 180) {
                                youthLeagueScore += score;
                            }
                        }
                        // 荣誉岗位超过一个 2
                        List<YcqYjsEvaluateIndexVo> evaluateIndexVoList3 = evaluateIndexVoList.stream().filter(dto -> dto != null && dto.getId() != null && dto.getId().equals(new BigInteger("39"))).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(evaluateIndexVoList3)) {
                            YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo = evaluateIndexVoList3.get(0);
                            int score = ycqYjsEvaluateIndexVo.getScore() == null ? 0 : ycqYjsEvaluateIndexVo.getScore().intValue();
                            if (studentPositionCount > 0) {
                                youthLeagueScore += score * studentPositionCount;
                            }
                        }
                        // 荣誉称号超过一个 3
                        List<YcqYjsEvaluateIndexVo> evaluateIndexVoList4 = evaluateIndexVoList.stream().filter(dto -> dto != null && dto.getId() != null && dto.getId().equals(new BigInteger("38"))).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(evaluateIndexVoList4)) {
                            YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo = evaluateIndexVoList4.get(0);
                            int score = ycqYjsEvaluateIndexVo.getScore() == null ? 0 : ycqYjsEvaluateIndexVo.getScore().intValue();
                            if (studentHonoraryCount > 0) {
                                youthLeagueScore += score * studentHonoraryCount;
                            }
                        }
                        // 星级章超过一个 1
                        List<YcqYjsEvaluateIndexVo> evaluateIndexVoList5 = evaluateIndexVoList.stream().filter(dto -> dto != null && dto.getId() != null && dto.getId().equals(new BigInteger("37"))).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(evaluateIndexVoList5)) {
                            YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo = evaluateIndexVoList5.get(0);
                            int score = ycqYjsEvaluateIndexVo.getScore() == null ? 0 : ycqYjsEvaluateIndexVo.getScore().intValue();
                            if (xjMedalCount > 0) {
                                youthLeagueScore += score * xjMedalCount;
                            }
                        }
                    }
                    item.setYouthLeagueScore(youthLeagueScore);
                    item.setTotalScore((teacherScore + youthLeagueScore) / 2);
                });
            }
            if (!CollectionUtils.isEmpty(studentInfoVoList)) {
                studentInfoVoPageRsp.setResult(studentInfoVoList);
            }
            return studentInfoVoPageRsp;
        } catch (BusinessException e) {
            log.error("StudentManager.pageSearchStudentInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.pageSearchStudentInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 列表查询学生数据
     *
     * @param req
     * @return
     * @throws Exception
     */
    public List<StudentInfoVo> selectStudentInfoList(StudentReq req) throws Exception {
        try {
            log.info("StudentManager.selectStudentInfo.param：{}", req.toString());
            List<BigInteger> studentIdList = getStudentIdsByPatriarchPhoneAndName(req.getPatriarchMobile(), req.getPatriarchName());
            // 通过家长信息查询学生ID
            if (!CollectionUtils.isEmpty(studentIdList)) {
                req.setIdList(studentIdList);
            }
            List<StudentInfoVo> studentInfoVoList = studentService.selectList(req);
            if (!CollectionUtils.isEmpty(studentInfoVoList)) {
                List<BigInteger> studentIds = studentInfoVoList.stream().map(StudentInfoVo::getId).collect(Collectors.toList());
                StudentPatriarchReq studentPatriarchReq = new StudentPatriarchReq();
                studentPatriarchReq.setStudentIdList(studentIds);
                List<YcqYjsStudentPatriarch> studentPatriarcheList = studentPatriarchService.selectList(studentPatriarchReq);
                Map<BigInteger, List<YcqYjsStudentPatriarch>> studentPatriarchMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(studentPatriarcheList)) {
                    studentPatriarchMap = studentPatriarcheList.stream().collect(Collectors.groupingBy(YcqYjsStudentPatriarch::getStudentId));
                }
                Map<BigInteger, List<YcqYjsStudentPatriarch>> finalStudentPatriarchMap = studentPatriarchMap;
                // 指标体系 - 12
                SystemIndexReq systemIndexReq = new SystemIndexReq();
                systemIndexReq.setIndexLevel(1);
                systemIndexReq.setSystemId(new BigInteger("12"));
                List<YcqYjsEvaluateIndexVo> evaluateIndexVoList = evaluateIndexService.searchIndexListInfo(systemIndexReq);
                studentInfoVoList.forEach(item -> {
                    item.setStudentPatriarchList(finalStudentPatriarchMap.get(item.getId()));
                    Integer teacherScore = item.getTeacherScore() == null ? 0 : item.getTeacherScore();
                    // 入团积分计算
                    Integer youthLeagueScore = 0;
                    Double serviceDuration = item.getServiceDuration() == null ? 0 : item.getServiceDuration();// 服务时长: 小时
                    Integer courseDuration = item.getCourseDuration() == null ? 0 : item.getCourseDuration();// 课程时长：分钟
                    int xjMedalCount = item.getXjMedalCount();//星级章个数
                    int studentPositionCount = item.getStudentPositionCount();//学生岗位个数
                    int studentHonoraryCount = item.getStudentHonoraryCount();//学生荣誉称号个数
                    if (!CollectionUtils.isEmpty(evaluateIndexVoList)) {
                        // 志愿服务大于20小时 - 10506
                        List<YcqYjsEvaluateIndexVo> evaluateIndexVoList1 = evaluateIndexVoList.stream().filter(dto -> dto != null && dto.getId() != null && dto.getId().equals(new BigInteger("35"))).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(evaluateIndexVoList1)) {
                            YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo = evaluateIndexVoList1.get(0);
                            int score = ycqYjsEvaluateIndexVo.getScore() == null ? 0 : ycqYjsEvaluateIndexVo.getScore().intValue();
                            if (serviceDuration > 20) {
                                youthLeagueScore += score;
                            }
                        }
                        // 视频学习时长超过3分钟 - 10507
                        List<YcqYjsEvaluateIndexVo> evaluateIndexVoList2 = evaluateIndexVoList.stream().filter(dto -> dto != null && dto.getId() != null && dto.getId().equals(new BigInteger("36"))).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(evaluateIndexVoList2)) {
                            YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo = evaluateIndexVoList2.get(0);
                            int score = ycqYjsEvaluateIndexVo.getScore() == null ? 0 : ycqYjsEvaluateIndexVo.getScore().intValue();
                            if (courseDuration > 180) {
                                youthLeagueScore += score;
                            }
                        }
                        // 荣誉岗位超过一个
                        List<YcqYjsEvaluateIndexVo> evaluateIndexVoList3 = evaluateIndexVoList.stream().filter(dto -> dto != null && dto.getId() != null && dto.getId().equals(new BigInteger("39"))).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(evaluateIndexVoList3)) {
                            YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo = evaluateIndexVoList3.get(0);
                            int score = ycqYjsEvaluateIndexVo.getScore() == null ? 0 : ycqYjsEvaluateIndexVo.getScore().intValue();
                            if (studentPositionCount > 0) {
                                youthLeagueScore += score * studentPositionCount;
                            }
                        }
                        // 荣誉称号超过一个
                        List<YcqYjsEvaluateIndexVo> evaluateIndexVoList4 = evaluateIndexVoList.stream().filter(dto -> dto != null && dto.getId() != null && dto.getId().equals(new BigInteger("38"))).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(evaluateIndexVoList4)) {
                            YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo = evaluateIndexVoList4.get(0);
                            int score = ycqYjsEvaluateIndexVo.getScore() == null ? 0 : ycqYjsEvaluateIndexVo.getScore().intValue();
                            if (studentHonoraryCount > 0) {
                                youthLeagueScore += score * studentHonoraryCount;
                            }
                        }
                        // 星级章超过一个
                        List<YcqYjsEvaluateIndexVo> evaluateIndexVoList5 = evaluateIndexVoList.stream().filter(dto -> dto != null && dto.getId() != null && dto.getId().equals(new BigInteger("37"))).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(evaluateIndexVoList5)) {
                            YcqYjsEvaluateIndexVo ycqYjsEvaluateIndexVo = evaluateIndexVoList5.get(0);
                            int score = ycqYjsEvaluateIndexVo.getScore() == null ? 0 : ycqYjsEvaluateIndexVo.getScore().intValue();
                            if (xjMedalCount > 0) {
                                youthLeagueScore += score * xjMedalCount;
                            }
                        }
                    }
                    item.setYouthLeagueScore(youthLeagueScore);
                    item.setTotalScore((teacherScore + youthLeagueScore) / 2);
                });
            }
            return studentInfoVoList;
        } catch (BusinessException e) {
            log.error("StudentManager.selectStudentInfoList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.selectStudentInfoList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 查询学生详情
     *
     * @param studentId
     * @return
     * @throws Exception
     */
    public StudentInfoVo selectStudentInfo(BigInteger studentId) throws Exception {
        try {
            log.info("StudentManager.selectStudentInfo.param：{}", studentId);
            if (studentId == null) {
                return null;
            }
            StudentReq req = new StudentReq();
            req.setId(studentId);
            List<StudentInfoVo> studentInfoVos = selectStudentInfoList(req);
            if (CollectionUtils.isEmpty(studentInfoVos)) {
                return null;
            }
            StudentInfoVo studentInfoVo = studentInfoVos.get(0);
            // 家长列表
            StudentPatriarchReq studentPatriarchReq = new StudentPatriarchReq();
            studentPatriarchReq.setStudentId(studentId);
            List<YcqYjsStudentPatriarch> studentPatriarcheList = studentPatriarchService.selectList(studentPatriarchReq);
            studentInfoVo.setStudentPatriarchList(studentPatriarcheList);
            // 生荣誉称号列表
            StudentHonoraryReq studentHonoraryReq = new StudentHonoraryReq();
            studentHonoraryReq.setStudentId(studentId);
            List<YcqYjsStudentHonorary> ycqYjsStudentHonoraries = studentHonoraryService.selectList(studentHonoraryReq);
            studentInfoVo.setStudentHonoraryList(ycqYjsStudentHonoraries);
            studentInfoVo.setStudentHonoraryCount(CollectionUtils.isEmpty(ycqYjsStudentHonoraries) ? 0 : ycqYjsStudentHonoraries.size());
            // 学生岗位评级列表
            StudentPositionReq studentPositionReq = new StudentPositionReq();
            studentPositionReq.setStudentId(studentId);
            List<YcqYjsStudentPosition> ycqYjsStudentPositions = studentPositionService.selectList(studentPositionReq);
            studentInfoVo.setStudentPositionList(ycqYjsStudentPositions);
            studentInfoVo.setStudentPositionCount(CollectionUtils.isEmpty(ycqYjsStudentPositions) ? 0 : ycqYjsStudentPositions.size());
            studentPositionReq.setGradeId(studentInfoVo.getCurrentGradeId());
            studentPositionReq.setSemesterType(studentInfoVo.getSemesterType());
            List<YcqYjsStudentPosition> studentPositionList = studentPositionService.selectList(studentPositionReq);
            if (!CollectionUtils.isEmpty(studentPositionList)) {
                YcqYjsStudentPosition studentPosition = studentPositionList.get(0);
                studentInfoVo.setStudentPositionType(studentPosition.getStudentPositionType());
                studentInfoVo.setStudentPosition(studentPosition.getStudentPosition());
                studentInfoVo.setStudentPositionClassType(studentPosition.getStudentPositionClassType());
                studentInfoVo.setStudentPositionClass(studentPosition.getStudentPositionClass());
                studentInfoVo.setPositionLevel(studentPosition.getPositionLevel());
                studentInfoVo.setPositionLevelType(studentPosition.getPositionLevelType());
                studentInfoVo.setPositionTime(studentPosition.getPositionTime());
            }
            return studentInfoVo;
        } catch (BusinessException e) {
            log.error("StudentManager.selectStudentInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.selectStudentInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 根据身份证查询少先队员的学生信息详情
     *
     * @param identityCardNo
     * @return
     * @throws Exception
     */
    public Result<StudentDetailVo> selectStudentByIdentityCardNo(String identityCardNo) throws Exception {
        try {
            log.info("StudentManager.selectStudentByIdentityCardNo.param：{}", identityCardNo);
            List<YcqYjsStudent> ycqYjsStudents = studentService.selectListByIdentityCardNo(identityCardNo);
            if (CollectionUtils.isEmpty(ycqYjsStudents)) {
                return Result.ofSuccessWithMsg(null, "未匹配到信息");
            }
            ycqYjsStudents = ycqYjsStudents.stream().filter(item -> item != null && item.getPoliticalStatusType() != null && item.getPoliticalStatusType().equals(PoliticalStatusTypeEnum.YOUNG_PIONEERS.code())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(ycqYjsStudents)) {
                return Result.ofSuccessWithMsg(null, "未匹配到信息");
            }
            return Result.ofSuccess(BeanCopyUtil.convert(ycqYjsStudents.get(0), StudentDetailVo.class));
        } catch (BusinessException e) {
            log.error("StudentManager.selectStudentByIdentityCardNo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.selectStudentByIdentityCardNo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 根据身份证和姓名查询少先队员的学生信息详情
     *
     * @param identityCardNo
     * @param studentName
     * @return
     * @throws Exception
     */
    public Result<StudentDetailVo> selectStudentByIdentityCardNoAndStudentName(String identityCardNo, String studentName) throws Exception {
        try {
            log.info("StudentManager.selectStudentByIdentityCardNoAndStudentName.param：identityCardNo:{},studentName:{}", identityCardNo, studentName);
            List<YcqYjsStudent> ycqYjsStudents = studentService.selectListByIdentityCardNo(identityCardNo);
            if (CollectionUtils.isEmpty(ycqYjsStudents)) {
                return Result.ofSuccessWithMsg(null, "未匹配到信息");
            }
            ycqYjsStudents = ycqYjsStudents.stream().filter(item -> item != null && item.getPoliticalStatusType() != null && item.getPoliticalStatusType().equals(PoliticalStatusTypeEnum.YOUNG_PIONEERS.code())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(ycqYjsStudents)) {
                return Result.ofSuccessWithMsg(null, "未匹配到信息");
            }
            StudentDetailVo studentDetailVo = BeanCopyUtil.convert(ycqYjsStudents.get(0), StudentDetailVo.class);
            if (studentDetailVo == null || StringUtil.isEmpty(studentName) || !studentName.equals(studentDetailVo.getStudentName())) {
                return Result.ofSuccessWithMsg(null, "未匹配到信息");
            }
            return Result.ofSuccess(studentDetailVo);
        } catch (BusinessException e) {
            log.error("StudentManager.selectStudentByIdentityCardNoAndStudentName.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.selectStudentByIdentityCardNoAndStudentName.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 新建学生数据
     *
     * @param createReq
     * @throws Exception
     */
    public void createStudentInfo(StudentCreateReq createReq) throws Exception {
        try {
            log.info("StudentManager.createStudentInfo.param：{}", createReq.toString());
            Date createDate = new Date();
            // 身份证验证
            String identityCardNo = createReq.getIdentityCardNo();
            if (StringUtil.isEmpty(identityCardNo)) {
                throw new BusinessException("身份证号必填");
            }
            if (!CommonUtil.isIDCard(identityCardNo)) {
                throw new BusinessException("身份证号格式不正确");
            }
            // 身份证验重
            List<YcqYjsStudent> studentInfoListByIdentityCardNo = studentService.selectListByIdentityCardNo(identityCardNo);
            if (!CollectionUtils.isEmpty(studentInfoListByIdentityCardNo)) {
                throw new BusinessException("该学生身份证信息已存在无法新增");
            }
            // 手机号验证
            String mobile = createReq.getMobile();
            if (!StringUtil.isEmpty(mobile)) {
                if (!CommonUtil.ifMobile(mobile)) {
                    throw new BusinessException("手机号格式不正确");
                }
                // 手机号验重
                /*List<YcqYjsStudent> studentInfoListByMobile = studentService.selectListByMobile(mobile);
                if (!CollectionUtils.isEmpty(studentInfoListByMobile)) {
                    throw new BusinessException("该学生手机号信息已存在无法新增");
                }*/
            }
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                createReq.setCreatedName(currWorker.getWorkerName());
            }
            // 教师编号 : 业务首字母大写(如JSGL）+ 时间(秒) + 4位随机码(26个 英文字母+数字)
            String studentNum = CharacterCodeUtil.generateCharacterCode(OneAffairConstants.STUDENT_NUM_PREFIX);
            createReq.setStudentNum(studentNum);
            createReq.setCreatedTime(createDate);
            createReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
            createReq.setIfJoinYouthPioneers(JoinYouthPioneersFlagEnum.NO.code());
            createReq.setIfJoinCommunistParty(JoinCommunistPartyFlagEnum.NO.code());
            createReq.setIfJoinYouthLeague(JoinYouthLeagueFlagEnum.NO.code());
            if (createReq.getPoliticalStatusType() != null) {
                if (PoliticalStatusTypeEnum.YOUTH_LEAGUE.code().equals(createReq.getPoliticalStatusType())) {
                    // 团队
                    createReq.setIfJoinYouthLeague(JoinYouthLeagueFlagEnum.YES.code());
                    createReq.setIfJoinYouthPioneers(JoinYouthPioneersFlagEnum.YES.code());
                    createReq.setJoinCommunistPartyTime(null);
                } else if (PoliticalStatusTypeEnum.YOUNG_PIONEERS.code().equals(createReq.getPoliticalStatusType())) {
                    // 少先队
                    createReq.setIfJoinYouthPioneers(JoinYouthPioneersFlagEnum.YES.code());
                    createReq.setJoinYouthLeagueTime(null);
                    createReq.setJoinCommunistPartyTime(null);
                } else if (PoliticalStatusTypeEnum.THE_MASSES.code().equals(createReq.getPoliticalStatusType())) {
                    // 群众
                    createReq.setJoinYoungPioneersTime(null);
                    createReq.setJoinYouthLeagueTime(null);
                    createReq.setJoinCommunistPartyTime(null);
                }
            }
            // 中队
            if (!StringUtil.isEmpty(createReq.getCurrentGradeName()) && !StringUtil.isEmpty(createReq.getCurrentClassName())) {
                createReq.setMiddleTeamName(createReq.getCurrentGradeName().replace("年级", "") + "(" + createReq.getCurrentClassName().replace("班", "") + ")中队");
            } else {
                createReq.setMiddleTeamName(null);
            }
            YcqYjsStudent studentInfo = studentService.createStudentInfo(createReq);
            // 档案初始
            List<ArchiveChangeReq> archiveCreateList = new ArrayList<>();
            // 是否入队
            if (studentInfo.getIfJoinYouthPioneers() != null && studentInfo.getIfJoinYouthPioneers().intValue() == IfFlagEnum.TRUE.code().intValue()) {
                ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                archiveChangeReq.setStudentId(studentInfo.getId().toString());
                archiveChangeReq.setActivityType(ActivityTypeEnum.ACTIVITY_TYPE_4.code().toString());
                archiveChangeReq.setEventType(EventTypeEnum.EVENT_3.code());
                archiveChangeReq.setEventId(studentInfo.getId().toString());
                archiveChangeReq.setParticipateTime(studentInfo.getJoinYoungPioneersTime());
                archiveChangeReq.setClassName(studentInfo.getCurrentGradeName() + studentInfo.getSemesterName());
                archiveChangeReq.setName("加入中国少先队");
                if (currWorker != null) {
                    archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                    archiveChangeReq.setUserName(currWorker.getWorkerName());
                }
                archiveCreateList.add(archiveChangeReq);
            }
            // 是否入团
            if (studentInfo.getIfJoinYouthLeague() != null && studentInfo.getIfJoinYouthLeague().intValue() == IfFlagEnum.TRUE.code().intValue()) {
                ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                archiveChangeReq.setStudentId(studentInfo.getId().toString());
                archiveChangeReq.setActivityType(ActivityTypeEnum.ACTIVITY_TYPE_5.code().toString());
                archiveChangeReq.setEventType(EventTypeEnum.EVENT_6.code());
                archiveChangeReq.setEventId(studentInfo.getId().toString());
                archiveChangeReq.setParticipateTime(studentInfo.getJoinYouthLeagueTime());
                archiveChangeReq.setClassName(studentInfo.getCurrentGradeName() + studentInfo.getSemesterName());
                archiveChangeReq.setName("被推优入团");
                if (currWorker != null) {
                    archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                    archiveChangeReq.setUserName(currWorker.getWorkerName());
                }
                archiveCreateList.add(archiveChangeReq);
            }
            // 存储档案数据
            if (!CollectionUtils.isEmpty(archiveCreateList)) {
                archiveService.insertBatch(archiveCreateList, currWorker);
            }
        } catch (BusinessException e) {
            log.error("StudentManager.createStudentInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.createStudentInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 建立学生与用户的绑定关系
     *
     * @param studentId
     * @throws Exception
     */
    public void bindingStudentToUser(BigInteger studentId) throws Exception {
        try {
            log.info("StudentManager.bindingStudentToUser.param：{}", studentId.toString());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            YcqYjsStudent ycqYjsStudent = studentService.selectOneById(studentId);
            if (ycqYjsStudent == null) {
                throw new BusinessException("学生数据不存在，同步创建用户失败");
            }
            StudentBindingUserReq studentBindingUserReq = new StudentBindingUserReq();
            studentBindingUserReq.setStudentId(ycqYjsStudent.getId());
            List<YcqYjsStudentBindingUser> studentBindingUserList = studentBindingUserService.selectList(studentBindingUserReq);
            if (!CollectionUtils.isEmpty(studentBindingUserList)) {
                return;
            }
            // 新增用户数据
            // 新增
            WorkerC workerC = new WorkerC();
            workerC.setOrgId(ycqYjsStudent.getCurrentSchoolId().toString());
            workerC.setMobile(ycqYjsStudent.getMobile());
            workerC.setSex(ycqYjsStudent.getSex());
            workerC.setWorkerName(ycqYjsStudent.getStudentName());
            workerC.setLoginCode(ycqYjsStudent.getMobile());
            workerC.setState(IfFlagEnum.TRUE.code());
            workerC.setRoleIds(Arrays.asList(UseRoleEnum.STUDENT.code()));
            log.info("新增学生数据时同步新增用户数据：{}", workerC);
            Result<Integer> worker = sysService.createWorkerInfo(workerC);
            if (worker == null || worker.getData() == null) {
                throw new BusinessException("新增学生数据时同步新增用户数据失败");
            }
            BigInteger loginId = new BigInteger(String.valueOf(worker.getData().intValue()));
            // 绑定
            StudentBindingUserCreateVo studentBindingUserCreateVo = new StudentBindingUserCreateVo();
            studentBindingUserCreateVo.setCreatedId(ycqYjsStudent.getCreatedId());
            studentBindingUserCreateVo.setCreatedTime(ycqYjsStudent.getCreatedTime());
            studentBindingUserCreateVo.setDelFlag(DelFlagEnum.UN_DELETE.code());
            studentBindingUserCreateVo.setInFlag(IfFlagEnum.TRUE.code());
            studentBindingUserCreateVo.setStudentId(studentBindingUserReq.getId());
            studentBindingUserCreateVo.setUserId(loginId);
            studentBindingUserCreateVo.setUserType(UserTypeEnum.STUDENT.code());
            studentBindingUserService.insertOne(studentBindingUserCreateVo);
        } catch (BusinessException e) {
            log.error("StudentManager.bindingStudentToUser.error:{}", e);
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.bindingStudentToUser.error:{}", e);
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 修改学生信息
     *
     * @param updateReq
     * @throws Exception
     */
    public void updateStudentInfo(StudentUpdateReq updateReq) throws Exception {
        try {
            log.info("StudentManager.updateStudentInfo.param：{}", updateReq.toString());
            Date updateDate = new Date();
            // 校验学生是否存在
            YcqYjsStudent studentInfo = studentService.selectOneById(updateReq.getId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            // 判断脱敏后是否修改
            // 手机号
            String mobile = updateReq.getMobile();
            String oldMobile = studentInfo.getMobile();
            if (!StringUtil.isEmpty(mobile) && !StringUtil.isEmpty(oldMobile)) {
                if (mobile.equals(DesensitizedUtil.mobilePhone(oldMobile))) {
                    updateReq.setMobile(oldMobile);
                } else {
                    // 手机号验证
                    if (!CommonUtil.ifMobile(mobile)) {
                        throw new BusinessException("手机号格式不正确");
                    }
                }
            }
            // 身份证
            String identityCardNo = updateReq.getIdentityCardNo();
            if (StringUtil.isEmpty(identityCardNo)) {
                throw new BusinessException("身份证号必填");
            }
            if (identityCardNo.equals(DesensitizedUtil.mobilePhone(studentInfo.getIdentityCardNo()))) {
                updateReq.setIdentityCardNo(studentInfo.getIdentityCardNo());
            } else {
                // 身份证验证
                if (!CommonUtil.isIDCard(identityCardNo)) {
                    throw new BusinessException("身份证号格式不正确");
                }
                // 身份证验重
                List<YcqYjsStudent> studentInfoList = studentService.selectListByIdentityCardNo(identityCardNo);
                if (!CollectionUtils.isEmpty(studentInfoList)
                        && (studentInfoList.size() > 1 || !studentInfoList.get(0).getId().equals(updateReq.getId()))) {
                    throw new BusinessException("该学生身份证信息已存在无法修改");
                }
            }
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            updateReq.setLastUpdatedTime(updateDate);
            // 中队
            if (!StringUtil.isEmpty(updateReq.getCurrentGradeName()) && !StringUtil.isEmpty(updateReq.getCurrentClassName())) {
                updateReq.setMiddleTeamName(updateReq.getCurrentGradeName().replace("年级", "") + "(" + updateReq.getCurrentClassName().replace("班", "") + ")中队");
            } else {
                updateReq.setMiddleTeamName(null);
            }
            studentService.updateStudentInfo(updateReq);
        } catch (BusinessException e) {
            log.error("StudentManager.updateStudentInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.updateStudentInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 编辑学生荣誉称号
     *
     * @param updateReq
     */
    @Transactional(rollbackFor = Exception.class)
    public void editStudentsHonorary(StudentHonoraryEditReq updateReq) throws Exception {
        try {
            log.info("StudentManager.editStudentsHonorary.param：{}", updateReq.toString());
            Date updateDate = new Date();
            // 校验学生是否存在
            YcqYjsStudent studentInfo = studentService.selectOneById(updateReq.getStudentId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            updateReq.setLastUpdatedTime(updateDate);
            // 需要处理的荣誉称号
            List<StudentHonoraryBaseReq> studentHonoraryBaseReqList = updateReq.getStudentHonoraryBaseReqList();
            // 验重
            if (!CollectionUtils.isEmpty(studentHonoraryBaseReqList)) {
                // 校验
                checkHonoraryBase(studentHonoraryBaseReqList);
                // 分组ID: gradeId:semesterType:honoraryLevelType:honoraryTitleType
                List<String> groupIdList = new ArrayList<>();
                for (StudentHonoraryBaseReq item : studentHonoraryBaseReqList) {
                    item.setSchoolId(studentInfo.getCurrentSchoolId());
                    item.setSchoolName(studentInfo.getCurrentSchoolName());
                    String groupId = item.getGradeId() + ":" + item.getSemesterType() + ":" + item.getHonoraryLevelType() + ":" + item.getHonoraryTitleType();
                    if (groupIdList.contains(groupId)) {
                        throw new BusinessException("同一年级同一学期已存在相同荣誉称号，请重新选择");
                    } else {
                        groupIdList.add(groupId);
                    }
                    item.setHonoraryFullTitle(item.getHonoraryLevel() + item.getHonoraryTitle());
                }
            }
            // 新增
            List<StudentHonoraryBaseReq> createList = studentHonoraryBaseReqList.stream().filter(item -> item != null && item.getId() == null).collect(Collectors.toList());
            List<StudentHonoraryCreateReq> studentHonoraryCreateReqList = BeanCopyUtil.convert(createList, StudentHonoraryCreateReq.class);
            // 修改
            List<StudentHonoraryBaseReq> updateList = studentHonoraryBaseReqList.stream().filter(item -> item != null && item.getId() != null).collect(Collectors.toList());
            List<StudentHonoraryUpdateReq> studentHonoraryUpdateReqList = BeanCopyUtil.convert(updateList, StudentHonoraryUpdateReq.class);
            String activityType = ActivityTypeEnum.ACTIVITY_TYPE_4.code().toString();
            // 团员
            if (studentInfo.getPoliticalStatus() != null && studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.YOUTH_LEAGUE.code())) {
                activityType = ActivityTypeEnum.ACTIVITY_TYPE_5.code().toString();
            } else if (studentInfo.getPoliticalStatus() != null && (studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.COMMUNIST_READY_PARTY_MEMBER.code()) || studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.COMMUNIST_PARTY_MEMBER.code()))) {
                activityType = ActivityTypeEnum.ACTIVITY_TYPE_6.code().toString();
            }
            // 删除
            // 已有荣誉称号
            StudentHonoraryReq studentHonoraryReq = new StudentHonoraryReq();
            studentHonoraryReq.setStudentId(updateReq.getStudentId());
            List<YcqYjsStudentHonorary> studentHonoraryList = studentHonoraryService.selectList(studentHonoraryReq);
            if (!CollectionUtils.isEmpty(studentHonoraryList)) {
                List<BigInteger> deleteStudentHonoraryIds = studentHonoraryList.stream().map(YcqYjsStudentHonorary::getId).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(studentHonoraryUpdateReqList)) {
                    List<BigInteger> updateIdList = studentHonoraryUpdateReqList.stream().map(StudentHonoraryUpdateReq::getId).collect(Collectors.toList());
                    deleteStudentHonoraryIds.removeAll(updateIdList);
                }
                if (!CollectionUtils.isEmpty(deleteStudentHonoraryIds)) {
                    StudentHonoraryDeleteReq studentHonoraryDeleteReq = new StudentHonoraryDeleteReq();
                    studentHonoraryDeleteReq.setIdList(deleteStudentHonoraryIds);
                    studentHonoraryDeleteReq.setLastUpdatedId(updateReq.getLastUpdatedId());
                    studentHonoraryDeleteReq.setLastUpdatedName(updateReq.getLastUpdatedName());
                    studentHonoraryDeleteReq.setLastUpdatedTime(updateReq.getLastUpdatedTime());
                    studentHonoraryService.deleteBatch(studentHonoraryDeleteReq);
                    // 同步档案数据
                    ArchiveSelectReq archiveSelectReq = new ArchiveSelectReq();
                    archiveSelectReq.setStudentId(studentInfo.getId().toString());
                    archiveSelectReq.setEventType(EventTypeEnum.EVENT_2.code());
                    List<String> deleteStudentHonoraryIdStrList = deleteStudentHonoraryIds.stream().map(String::valueOf).collect(Collectors.toList());
                    archiveSelectReq.setEventIdList(deleteStudentHonoraryIdStrList);
                    List<ArchiveVo> archiveVoList = archiveService.getList(archiveSelectReq);
                    if (!CollectionUtils.isEmpty(archiveVoList)) {
                        List<String> archiveVoIds = archiveVoList.stream().map(ArchiveVo::getId).distinct().collect(Collectors.toList());
                        archiveService.deleteBatch(archiveVoIds, currWorker);
                    }
                }
            }
            // 新增
            if (!CollectionUtils.isEmpty(studentHonoraryCreateReqList)) {
                studentHonoraryCreateReqList.stream().forEach(item -> {
                    item.setStudentId(updateReq.getStudentId());
                    item.setCreatedId(updateReq.getLastUpdatedId());
                    item.setCreatedName(updateReq.getLastUpdatedName());
                    item.setCreatedTime(updateReq.getLastUpdatedTime());
                    item.setDelFlag(DelFlagEnum.UN_DELETE.code());
                });
                List<YcqYjsStudentHonorary> studentHonorarys = studentHonoraryService.insertBatch(studentHonoraryCreateReqList);
                if (!CollectionUtils.isEmpty(studentHonorarys)) {
                    // 档案初始
                    List<ArchiveChangeReq> archiveCreateList = new ArrayList<>();
                    for (YcqYjsStudentHonorary studentHonorary : studentHonorarys) {
                        // 档案数据
                        ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                        archiveChangeReq.setStudentId(studentInfo.getId().toString());
                        archiveChangeReq.setActivityType(activityType);
                        archiveChangeReq.setEventType(EventTypeEnum.EVENT_2.code());
                        archiveChangeReq.setEventId(studentHonorary.getId().toString());
                        archiveChangeReq.setParticipateTime(studentHonorary.getCreatedTime());
                        archiveChangeReq.setClassName(studentInfo.getCurrentGradeName() + studentInfo.getSemesterName());
                        archiveChangeReq.setName("获得" + studentHonorary.getHonoraryFullTitle());
                        if (currWorker != null) {
                            archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                            archiveChangeReq.setUserName(currWorker.getWorkerName());
                        }
                        archiveCreateList.add(archiveChangeReq);
                    }
                    // 存储档案数据
                    if (!CollectionUtils.isEmpty(archiveCreateList)) {
                        archiveService.insertBatch(archiveCreateList, currWorker);
                    }
                }
            }
            // 修改
            if (!CollectionUtils.isEmpty(studentHonoraryUpdateReqList)) {
                studentHonoraryUpdateReqList.stream().forEach(item -> {
                    item.setLastUpdatedId(updateReq.getLastUpdatedId());
                    item.setLastUpdatedName(updateReq.getLastUpdatedName());
                    item.setLastUpdatedTime(updateReq.getLastUpdatedTime());
                });
                studentHonoraryService.updateBatch(studentHonoraryUpdateReqList);
                // 同步档案数据
                Map<BigInteger, StudentHonoraryUpdateReq> studentHonoraryUpdateMap = studentHonoraryUpdateReqList.stream().collect(Collectors.toMap(StudentHonoraryUpdateReq::getId, item -> item));
                ArchiveSelectReq archiveSelectReq = new ArchiveSelectReq();
                archiveSelectReq.setStudentId(studentInfo.getId().toString());
                archiveSelectReq.setEventType(EventTypeEnum.EVENT_2.code());
                List<String> updateStudentHonoraryIdStrList = studentHonoraryUpdateReqList.stream().map(StudentHonoraryUpdateReq::getId).map(String::valueOf).collect(Collectors.toList());
                archiveSelectReq.setEventIdList(updateStudentHonoraryIdStrList);
                List<ArchiveVo> archiveVoList = archiveService.getList(archiveSelectReq);
                List<ArchiveUpdateReq> archiveUpdateReqList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(archiveVoList)) {
                    archiveVoList.stream().forEach(item -> {
                        if (item.getEventId() != null) {
                            StudentHonoraryUpdateReq studentHonoraryUpdateReq = studentHonoraryUpdateMap.get(new BigInteger(item.getEventId()));
                            if (studentHonoraryUpdateReq != null) {
                                ArchiveUpdateReq archiveUpdateReq = new ArchiveUpdateReq();
                                archiveUpdateReq.setName("获得" + studentHonoraryUpdateReq.getHonoraryFullTitle());
                                archiveUpdateReq.setId(item.getId());
                                archiveUpdateReq.setLastUpdatedId(updateReq.getLastUpdatedId() == null ? null : updateReq.getLastUpdatedId().toString());
                                archiveUpdateReq.setLastUpdatedTime(updateReq.getLastUpdatedTime());
                                archiveUpdateReqList.add(archiveUpdateReq);
                            }
                        }
                    });
                    if (!CollectionUtils.isEmpty(archiveUpdateReqList)) {
                        archiveService.updateBatch(archiveUpdateReqList);
                    }
                }
            }
        } catch (BusinessException e) {
            log.error("StudentManager.editStudentsHonorary.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.editStudentsHonorary.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }


    /**
     * 批量设置学生荣誉称号
     *
     * @param batchCreateReq
     */
    @Transactional(rollbackFor = Exception.class)
    public void batchCreateStudentHonorary(StudentHonoraryBatchCreateReq batchCreateReq) throws Exception {
        try {
            log.info("StudentManager.batchCreateStudentHonorary.param：{}", batchCreateReq.toString());
            Date createDate = new Date();
            batchCreateReq.setCreatedTime(createDate);
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                batchCreateReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                batchCreateReq.setCreatedName(currWorker.getWorkerName());
            }
            List<StudentHonoraryBaseReq> studentHonoraryBaseReqList = batchCreateReq.getStudentHonoraryBaseReqList();
            // 校验
            checkHonoraryBase(studentHonoraryBaseReqList);
            // 校验学生是否存在
            List<BigInteger> studentIdList = batchCreateReq.getStudentIdList();
            if (CollectionUtils.isEmpty(studentIdList)) {
                throw new BusinessException("学生必选");
            }
            List<YcqYjsStudent> ycqYjsStudents = studentService.selectListByIds(studentIdList);
            if (CollectionUtils.isEmpty(ycqYjsStudents) || ycqYjsStudents.size() != studentIdList.size()) {
                throw new BusinessException("所选学生中存在未维护的学生，操作失败");
            }
            studentIdList = ycqYjsStudents.stream().map(YcqYjsStudent::getId).collect(Collectors.toList());
            Map<BigInteger, YcqYjsStudent> studentMap = ycqYjsStudents.stream().collect(Collectors.toMap(YcqYjsStudent::getId, item -> item));
            // 学生已有荣誉称号
            StudentHonoraryReq studentHonoraryReq = new StudentHonoraryReq();
            studentHonoraryReq.setStudentIdList(studentIdList);
            List<YcqYjsStudentHonorary> studentHonoraryList = studentHonoraryService.selectList(studentHonoraryReq);
            Map<BigInteger, List<YcqYjsStudentHonorary>> studentHonoraryMapByStudentId = new HashMap<>();
            if (!CollectionUtils.isEmpty(studentHonoraryList)) {
                studentHonoraryMapByStudentId = studentHonoraryList.stream().collect(Collectors.groupingBy(YcqYjsStudentHonorary::getStudentId));
            }
            //分组ID: gradeId:semesterType:honoraryLevelType:honoraryTitleType 同年级同学期相同荣誉称号只能有一个
            studentHonoraryBaseReqList.stream().forEach(item -> {
                item.setGroupId(item.getGradeId() + ":" + item.getSemesterType() + ":" + item.getHonoraryLevelType() + ":" + item.getHonoraryTitleType());
            });
            List<StudentHonoraryCreateReq> studentHonoraryCreateReqList = new ArrayList<>();
            for (BigInteger studentId : studentIdList) {
                YcqYjsStudent studentInfo = studentMap.get(studentId);
                if (studentInfo == null) {
                    continue;
                }
                // 学生已有荣誉称号
                List<YcqYjsStudentHonorary> ycqYjsStudentHonoraries = studentHonoraryMapByStudentId.get(studentId);
                List<String> groupIdList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(ycqYjsStudentHonoraries)) {
                    ycqYjsStudentHonoraries.stream().forEach(item -> groupIdList.add(item.getGradeId() + ":" + item.getSemesterType() + ":" + item.getHonoraryLevelType() + ":" + item.getHonoraryTitleType()));
                }
                // 过滤
                List<StudentHonoraryBaseReq> createStudentHonoraryList = studentHonoraryBaseReqList.stream().filter(item -> item != null && item.getGroupId() != null && !groupIdList.contains(item.getGroupId())).collect(Collectors.toList());
                for (StudentHonoraryBaseReq item : createStudentHonoraryList) {
                    StudentHonoraryCreateReq studentHonoraryCreateReq = new StudentHonoraryCreateReq();
                    String honoraryFullTitle = item.getHonoraryTitle() + item.getHonoraryTitle();
                    BeanUtil.copyProperties(item, studentHonoraryCreateReq);
                    studentHonoraryCreateReq.setStudentId(studentId);
                    studentHonoraryCreateReq.setSchoolId(studentInfo.getCurrentSchoolId());
                    studentHonoraryCreateReq.setSchoolName(studentInfo.getCurrentSchoolName());
                    studentHonoraryCreateReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
                    studentHonoraryCreateReq.setCreatedId(batchCreateReq.getCreatedId());
                    studentHonoraryCreateReq.setCreatedName(batchCreateReq.getCreatedName());
                    studentHonoraryCreateReq.setCreatedTime(batchCreateReq.getCreatedTime());
                    studentHonoraryCreateReq.setHonoraryFullTitle(honoraryFullTitle);
                    studentHonoraryCreateReqList.add(studentHonoraryCreateReq);

                }
            }
            if (!CollectionUtils.isEmpty(studentHonoraryCreateReqList)) {
                List<YcqYjsStudentHonorary> ycqYjsStudentHonoraries = studentHonoraryService.insertBatch(studentHonoraryCreateReqList);
                if (!CollectionUtils.isEmpty(ycqYjsStudentHonoraries)) {
                    // 档案初始
                    List<ArchiveChangeReq> archiveCreateList = new ArrayList<>();
                    for (YcqYjsStudentHonorary item : ycqYjsStudentHonoraries) {
                        YcqYjsStudent studentInfo = studentMap.get(item.getStudentId());
                        if (studentInfo == null) {
                            continue;
                        }
                        String activityType = ActivityTypeEnum.ACTIVITY_TYPE_4.code().toString();
                        // 团员
                        if (studentInfo.getPoliticalStatus() != null && studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.YOUTH_LEAGUE.code())) {
                            activityType = ActivityTypeEnum.ACTIVITY_TYPE_5.code().toString();
                        } else if (studentInfo.getPoliticalStatus() != null && (studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.COMMUNIST_READY_PARTY_MEMBER.code()) || studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.COMMUNIST_PARTY_MEMBER.code()))) {
                            activityType = ActivityTypeEnum.ACTIVITY_TYPE_6.code().toString();
                        }
                        // 记录档案
                        // 档案数据
                        ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                        archiveChangeReq.setStudentId(studentInfo.getId().toString());
                        archiveChangeReq.setActivityType(activityType);
                        archiveChangeReq.setEventType(EventTypeEnum.EVENT_2.code());
                        archiveChangeReq.setEventId(item.getId().toString());
                        archiveChangeReq.setParticipateTime(batchCreateReq.getCreatedTime());
                        archiveChangeReq.setClassName(studentInfo.getCurrentGradeName() + studentInfo.getSemesterName());
                        archiveChangeReq.setName("获得" + item.getHonoraryFullTitle());
                        if (currWorker != null) {
                            archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                            archiveChangeReq.setUserName(currWorker.getWorkerName());
                        }
                        archiveCreateList.add(archiveChangeReq);
                    }
                    // 存储档案数据
                    if (!CollectionUtils.isEmpty(archiveCreateList)) {
                        archiveService.insertBatch(archiveCreateList, currWorker);
                    }
                }
            }
        } catch (BusinessException e) {
            log.error("StudentManager.batchCreateStudentHonorary.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.batchCreateStudentHonorary.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    public void checkHonoraryBase(List<StudentHonoraryBaseReq> studentHonoraryBaseReqList) {
        try {
            log.info("StudentManager.checkHonoraryBase.param：{}", studentHonoraryBaseReqList.toString());
            if (CollectionUtils.isEmpty(studentHonoraryBaseReqList)) {
                throw new BusinessException("荣誉称号不能为空");
            }
            for (StudentHonoraryBaseReq item : studentHonoraryBaseReqList) {
                // 荣誉称号必选: honoraryTitleType honoraryTitle
                if(StringUtil.isEmpty(item.getHonoraryTitleType()) || StringUtil.isEmpty(item.getHonoraryTitle())){
                    throw new BusinessException("荣誉称号必选");
                }
                // 学期必选: semesterName semesterType
                if(StringUtil.isEmpty(item.getSemesterName()) || StringUtil.isEmpty(item.getSemesterType())){
                    throw new BusinessException("学期必选");
                }
                // 年级必选: gradeId gradeName
                if(StringUtil.isEmpty(item.getGradeName()) || item.getGradeId() == null){
                    throw new BusinessException("年级必选");
                }
                // 班级必选: classId className
                if(StringUtil.isEmpty(item.getClassName()) || item.getClassId() == null){
                    throw new BusinessException("班级必选");
                }
                // 荣誉称号级别必选: honoraryLevelType honoraryLevel
                if(StringUtil.isEmpty(item.getHonoraryLevelType()) || StringUtil.isEmpty(item.getHonoraryLevel())){
                    throw new BusinessException("荣誉称号级别必选");
                }
                // 获取荣誉称号时间必填：honoraryTime
                if(item.getHonoraryTime() == null){
                    throw new BusinessException("获取荣誉称号时间必填");
                }
            }
        } catch (BusinessException e) {
            log.error("StudentManager.checkHonoraryBase.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.checkHonoraryBase.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 查询指定学生的学生荣誉称号列表
     *
     * @param studentId
     * @return
     */
    public List<YcqYjsStudentHonorary> selectStudentsHonoraryList(BigInteger studentId) {
        try {
            log.info("StudentManager.selectStudentsHonoraryList.param：{}", studentId);
            if (studentId == null) {
                throw new BusinessException("学生必选");
            }
            StudentHonoraryReq studentHonoraryReq = new StudentHonoraryReq();
            studentHonoraryReq.setStudentId(studentId);
            return studentHonoraryService.selectList(studentHonoraryReq);
        } catch (BusinessException e) {
            log.error("StudentManager.selectStudentsHonoraryList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.selectStudentsHonoraryList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 编辑学生岗位评级
     *
     * @param updateReq
     */
    @Transactional(rollbackFor = Exception.class)
    public void editStudentPosition(StudentPositionEditReq updateReq) throws Exception {
        try {
            log.info("StudentManager.editStudentsPosition.param：{}", updateReq.toString());
            Date updateDate = new Date();
            // 校验学生是否存在
            YcqYjsStudent studentInfo = studentService.selectOneById(updateReq.getStudentId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            updateReq.setLastUpdatedTime(updateDate);
            // 需要处理的学生岗位评级
            List<StudentPositionBaseReq> studentPositionBaseReqList = updateReq.getStudentPositionBaseReqList();
            // 验重
            if (!CollectionUtils.isEmpty(studentPositionBaseReqList)) {
                // 校验
                checkPositionBase(studentPositionBaseReqList);
                // 分组ID: 分组ID: gradeId:semesterType:studentPositionClassType:positionLevelType:studentPositionType
                List<String> groupIdList = new ArrayList<>();
                for (StudentPositionBaseReq item : studentPositionBaseReqList) {
                    item.setSchoolId(studentInfo.getCurrentSchoolId());
                    item.setSchoolName(studentInfo.getCurrentSchoolName());
                    String groupId = item.getGradeId() + ":" + item.getSemesterType() + ":" + item.getStudentPositionClassType() + ":" + item.getPositionLevelType() + ":" + item.getStudentPositionType();
                    if (groupIdList.contains(groupId)) {
                        // 学生岗位验重
                        throw new BusinessException("同一年级同一学期已存在相同岗位，请重新选择");
                    } else {
                        groupIdList.add(groupId);
                    }
                }
            }
            // 新增
            List<StudentPositionBaseReq> createList = studentPositionBaseReqList.stream().filter(item -> item != null && item.getId() == null).collect(Collectors.toList());
            List<StudentPositionCreateReq> studentPositionCreateReqList = BeanCopyUtil.convert(createList, StudentPositionCreateReq.class);
            // 修改
            List<StudentPositionBaseReq> updateList = studentPositionBaseReqList.stream().filter(item -> item != null && item.getId() != null).collect(Collectors.toList());
            List<StudentPositionUpdateReq> studentPositionUpdateReqList = BeanCopyUtil.convert(updateList, StudentPositionUpdateReq.class);
            String activityType = ActivityTypeEnum.ACTIVITY_TYPE_4.code().toString();
            // 团员
            if (studentInfo.getPoliticalStatus() != null && studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.YOUTH_LEAGUE.code())) {
                activityType = ActivityTypeEnum.ACTIVITY_TYPE_5.code().toString();
            } else if (studentInfo.getPoliticalStatus() != null && (studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.COMMUNIST_READY_PARTY_MEMBER.code()) || studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.COMMUNIST_PARTY_MEMBER.code()))) {
                activityType = ActivityTypeEnum.ACTIVITY_TYPE_6.code().toString();
            }
            // 删除
            // 已有学生岗位评级
            StudentPositionReq studentPositionReq = new StudentPositionReq();
            studentPositionReq.setStudentId(updateReq.getStudentId());
            List<YcqYjsStudentPosition> studentPositionList = studentPositionService.selectList(studentPositionReq);
            if (!CollectionUtils.isEmpty(studentPositionList)) {
                List<BigInteger> deleteStudentPositionIds = studentPositionList.stream().map(YcqYjsStudentPosition::getId).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(studentPositionUpdateReqList)) {
                    List<BigInteger> updateIdList = studentPositionUpdateReqList.stream().map(StudentPositionUpdateReq::getId).collect(Collectors.toList());
                    deleteStudentPositionIds.removeAll(updateIdList);
                }
                if (!CollectionUtils.isEmpty(deleteStudentPositionIds)) {
                    StudentPositionDeleteReq studentPositionDeleteReq = new StudentPositionDeleteReq();
                    studentPositionDeleteReq.setIdList(deleteStudentPositionIds);
                    studentPositionDeleteReq.setLastUpdatedId(updateReq.getLastUpdatedId());
                    studentPositionDeleteReq.setLastUpdatedName(updateReq.getLastUpdatedName());
                    studentPositionDeleteReq.setLastUpdatedTime(updateReq.getLastUpdatedTime());
                    studentPositionService.deleteBatch(studentPositionDeleteReq);
                    // 同步档案数据
                    ArchiveSelectReq archiveSelectReq = new ArchiveSelectReq();
                    archiveSelectReq.setStudentId(studentInfo.getId().toString());
                    archiveSelectReq.setEventType(EventTypeEnum.EVENT_4.code());
                    List<String> deleteIdList = deleteStudentPositionIds.stream().map(String::valueOf).collect(Collectors.toList());
                    archiveSelectReq.setEventIdList(deleteIdList);
                    List<ArchiveVo> archiveVoList = archiveService.getList(archiveSelectReq);
                    if (!CollectionUtils.isEmpty(archiveVoList)) {
                        List<String> archiveVoIds = archiveVoList.stream().map(ArchiveVo::getId).distinct().collect(Collectors.toList());
                        archiveService.deleteBatch(archiveVoIds, currWorker);
                    }
                }
            }
            // 新增
            if (!CollectionUtils.isEmpty(studentPositionCreateReqList)) {
                studentPositionCreateReqList.stream().forEach(item -> {
                    item.setStudentId(updateReq.getStudentId());
                    item.setCreatedId(updateReq.getLastUpdatedId());
                    item.setCreatedName(updateReq.getLastUpdatedName());
                    item.setCreatedTime(updateReq.getLastUpdatedTime());
                    item.setDelFlag(DelFlagEnum.UN_DELETE.code());
                });
                List<YcqYjsStudentPosition> studentPositions = studentPositionService.insertBatch(studentPositionCreateReqList);
                if (!CollectionUtils.isEmpty(studentPositions)) {
                    // 档案初始
                    List<ArchiveChangeReq> archiveCreateList = new ArrayList<>();
                    for (YcqYjsStudentPosition studentPosition : studentPositions) {
                        // 档案数据
                        ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                        archiveChangeReq.setStudentId(studentInfo.getId().toString());
                        archiveChangeReq.setActivityType(activityType);
                        archiveChangeReq.setEventType(EventTypeEnum.EVENT_4.code());
                        archiveChangeReq.setEventId(studentPosition.getId().toString());
                        archiveChangeReq.setParticipateTime(studentPosition.getCreatedTime());
                        archiveChangeReq.setClassName(studentInfo.getCurrentGradeName() + studentInfo.getSemesterName());
                        archiveChangeReq.setName("获得" + studentPosition.getPositionLevel() + studentPosition.getStudentPosition());
                        if (currWorker != null) {
                            archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                            archiveChangeReq.setUserName(currWorker.getWorkerName());
                        }
                        archiveCreateList.add(archiveChangeReq);
                    }
                    // 存储档案数据
                    if (!CollectionUtils.isEmpty(archiveCreateList)) {
                        archiveService.insertBatch(archiveCreateList, currWorker);
                    }
                }
            }
            // 修改
            if (!CollectionUtils.isEmpty(studentPositionUpdateReqList)) {
                studentPositionUpdateReqList.stream().forEach(item -> {
                    item.setLastUpdatedId(updateReq.getLastUpdatedId());
                    item.setLastUpdatedName(updateReq.getLastUpdatedName());
                    item.setLastUpdatedTime(updateReq.getLastUpdatedTime());
                });
                studentPositionService.updateBatch(studentPositionUpdateReqList);
                // 同步档案数据
                Map<BigInteger, StudentPositionUpdateReq> studenPositionUpdateMap = studentPositionUpdateReqList.stream().collect(Collectors.toMap(StudentPositionUpdateReq::getId, item -> item));
                ArchiveSelectReq archiveSelectReq = new ArchiveSelectReq();
                archiveSelectReq.setStudentId(studentInfo.getId().toString());
                archiveSelectReq.setEventType(EventTypeEnum.EVENT_4.code());
                List<String> updateStudentPositionIdStrList = studentPositionUpdateReqList.stream().map(StudentPositionUpdateReq::getId).map(String::valueOf).collect(Collectors.toList());
                archiveSelectReq.setEventIdList(updateStudentPositionIdStrList);
                List<ArchiveVo> archiveVoList = archiveService.getList(archiveSelectReq);
                if (!CollectionUtils.isEmpty(archiveVoList)) {
                    List<ArchiveUpdateReq> archiveUpdateReqList = new ArrayList<>();
                    archiveVoList.stream().forEach(item -> {
                        if (item.getEventId() != null) {
                            StudentPositionUpdateReq studentPositionUpdateReq = studenPositionUpdateMap.get(new BigInteger(item.getEventId()));
                            if (studentPositionUpdateReq != null) {
                                ArchiveUpdateReq archiveUpdateReq = new ArchiveUpdateReq();
                                archiveUpdateReq.setName("获得" + studentPositionUpdateReq.getPositionLevel() + studentPositionUpdateReq.getStudentPosition());
                                archiveUpdateReq.setId(item.getId());
                                archiveUpdateReq.setLastUpdatedId(updateReq.getLastUpdatedId() == null ? null : updateReq.getLastUpdatedId().toString());
                                archiveUpdateReq.setLastUpdatedTime(updateReq.getLastUpdatedTime());
                                archiveUpdateReqList.add(archiveUpdateReq);
                            }
                        }
                    });
                    if (!CollectionUtils.isEmpty(archiveUpdateReqList)) {
                        archiveService.updateBatch(archiveUpdateReqList);
                    }
                }
            }
        } catch (BusinessException e) {
            log.error("StudentManager.editStudentPosition.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.editStudentPosition.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 批量设置学生学生岗位评级
     *
     * @param batchCreateReq
     */
    @Transactional(rollbackFor = Exception.class)
    public void batchCreateStudentPosition(StudentPositionBatchCreateReq batchCreateReq) throws Exception {
        try {
            log.info("StudentManager.batchCreateStudentPosition.param：{}", batchCreateReq.toString());
            Date createDate = new Date();
            batchCreateReq.setCreatedTime(createDate);
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                batchCreateReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                batchCreateReq.setCreatedName(currWorker.getWorkerName());
            }
            List<StudentPositionBaseReq> studentPositionBaseReqList = batchCreateReq.getStudentPositionBaseReqList();
            checkPositionBase(studentPositionBaseReqList);
            // 校验学生是否存在
            List<BigInteger> studentIdList = batchCreateReq.getStudentIdList();
            if (CollectionUtils.isEmpty(studentIdList)) {
                throw new BusinessException("学生必选");
            }
            List<YcqYjsStudent> ycqYjsStudents = studentService.selectListByIds(studentIdList);
            if (CollectionUtils.isEmpty(ycqYjsStudents) || ycqYjsStudents.size() != studentIdList.size()) {
                throw new BusinessException("所选学生中存在未维护的学生，操作失败");
            }
            studentIdList = ycqYjsStudents.stream().map(YcqYjsStudent::getId).collect(Collectors.toList());
            Map<BigInteger, YcqYjsStudent> studentMap = ycqYjsStudents.stream().collect(Collectors.toMap(YcqYjsStudent::getId, item -> item));
            // 学生已有学生岗位评级
            StudentPositionReq studentPositionReq = new StudentPositionReq();
            studentPositionReq.setStudentIdList(studentIdList);
            List<YcqYjsStudentPosition> studentPositionList = studentPositionService.selectList(studentPositionReq);
            Map<BigInteger, List<YcqYjsStudentPosition>> studentPositionMapByStudentId = new HashMap<>();
            if (!CollectionUtils.isEmpty(studentPositionList)) {
                studentPositionMapByStudentId = studentPositionList.stream().collect(Collectors.groupingBy(YcqYjsStudentPosition::getStudentId));
            }
            //分组ID: gradeId:semesterType:studentPositionClassType:positionLevelType:studentPositionType
            studentPositionBaseReqList.stream().forEach(item -> {
                item.setGroupId(item.getGradeId() + ":" + item.getSemesterType() + ":" + item.getStudentPositionClassType() + ":" + item.getPositionLevelType() + ":" + item.getStudentPositionType());
            });
            List<StudentPositionCreateReq> studentPositionCreateReqList = new ArrayList<>();
            for (BigInteger studentId : studentIdList) {
                YcqYjsStudent studentInfo = studentMap.get(studentId);
                if (studentInfo == null) {
                    continue;
                }
                // 学生已有学生岗位评级
                List<YcqYjsStudentPosition> ycqYjsStudentHonoraries = studentPositionMapByStudentId.get(studentId);
                List<String> groupIdList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(ycqYjsStudentHonoraries)) {
                    ycqYjsStudentHonoraries.stream().forEach(item -> groupIdList.add(item.getGradeId() + ":" + item.getSemesterType() + ":" + item.getStudentPositionClassType() + ":" + item.getPositionLevelType() + ":" + item.getStudentPositionType()));
                }
                // 过滤
                List<StudentPositionBaseReq> createStudentPositionList = studentPositionBaseReqList.stream().filter(item -> item != null && item.getGroupId() != null && !groupIdList.contains(item.getGroupId())).collect(Collectors.toList());
                for (StudentPositionBaseReq item : createStudentPositionList) {
                    StudentPositionCreateReq studentPositionCreateReq = new StudentPositionCreateReq();
                    BeanUtil.copyProperties(item, studentPositionCreateReq);
                    studentPositionCreateReq.setStudentId(studentId);
                    studentPositionCreateReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
                    studentPositionCreateReq.setCreatedId(batchCreateReq.getCreatedId());
                    studentPositionCreateReq.setCreatedName(batchCreateReq.getCreatedName());
                    studentPositionCreateReq.setCreatedTime(batchCreateReq.getCreatedTime());
                    studentPositionCreateReq.setSchoolId(studentInfo.getCurrentSchoolId());
                    studentPositionCreateReq.setSchoolName(studentInfo.getCurrentSchoolName());
                    studentPositionCreateReqList.add(studentPositionCreateReq);
                }
            }
            // 存储档案数据
            if (!CollectionUtils.isEmpty(studentPositionCreateReqList)) {
                List<YcqYjsStudentPosition> ycqYjsStudentPositions = studentPositionService.insertBatch(studentPositionCreateReqList);
                if (!CollectionUtils.isEmpty(ycqYjsStudentPositions)) {
                    // 档案初始
                    List<ArchiveChangeReq> archiveCreateList = new ArrayList<>();
                    for (YcqYjsStudentPosition item : ycqYjsStudentPositions) {
                        YcqYjsStudent studentInfo = studentMap.get(item.getStudentId());
                        if (studentInfo == null) {
                            continue;
                        }
                        String activityType = ActivityTypeEnum.ACTIVITY_TYPE_4.code().toString();
                        // 团员
                        if (studentInfo.getPoliticalStatus() != null && studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.YOUTH_LEAGUE.code())) {
                            activityType = ActivityTypeEnum.ACTIVITY_TYPE_5.code().toString();
                        } else if (studentInfo.getPoliticalStatus() != null && (studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.COMMUNIST_READY_PARTY_MEMBER.code()) || studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.COMMUNIST_PARTY_MEMBER.code()))) {
                            activityType = ActivityTypeEnum.ACTIVITY_TYPE_6.code().toString();
                        }
                        // 记录档案
                        // 档案数据
                        ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                        archiveChangeReq.setStudentId(studentInfo.getId().toString());
                        archiveChangeReq.setActivityType(activityType);
                        archiveChangeReq.setEventType(EventTypeEnum.EVENT_2.code());
                        archiveChangeReq.setEventId(item.getId().toString());
                        archiveChangeReq.setParticipateTime(batchCreateReq.getCreatedTime());
                        archiveChangeReq.setClassName(studentInfo.getCurrentGradeName() + studentInfo.getSemesterName());
                        archiveChangeReq.setName("获得" + item.getPositionLevel() + item.getStudentPosition());
                        if (currWorker != null) {
                            archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                            archiveChangeReq.setUserName(currWorker.getWorkerName());
                        }
                        archiveCreateList.add(archiveChangeReq);
                    }
                    if (!CollectionUtils.isEmpty(archiveCreateList)) {
                        archiveService.insertBatch(archiveCreateList, currWorker);
                    }
                }
            }
        } catch (BusinessException e) {
            log.error("StudentManager.batchCreateStudentPosition.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.batchCreateStudentPosition.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    public void checkPositionBase(List<StudentPositionBaseReq> studentPositionBaseReqList) {
        try {
            log.info("StudentManager.checkHonoraryBase.param：{}", studentPositionBaseReqList.toString());
            if (CollectionUtils.isEmpty(studentPositionBaseReqList)) {
                throw new BusinessException("学生岗位评级不能为空");
            }
            for (StudentPositionBaseReq item : studentPositionBaseReqList) {
                // 学期必选: semesterName semesterType
                if(StringUtil.isEmpty(item.getSemesterName()) || StringUtil.isEmpty(item.getSemesterType())){
                    throw new BusinessException("学期必选");
                }
                // 年级必选: gradeId gradeName
                if(StringUtil.isEmpty(item.getGradeName()) || item.getGradeId() == null){
                    throw new BusinessException("年级必选");
                }
                // 班级必选: classId className
                if(StringUtil.isEmpty(item.getClassName()) || item.getClassId() == null){
                    throw new BusinessException("班级必选");
                }
                // 学生岗位分类必选: studentPositionClassType studentPositionClass
                if(StringUtil.isEmpty(item.getStudentPositionClassType()) || StringUtil.isEmpty(item.getStudentPositionClass())){
                    throw new BusinessException("学生岗位分类必选");
                }
                // 学生岗位必选: studentPositionType studentPosition
                if(StringUtil.isEmpty(item.getStudentPositionType()) || StringUtil.isEmpty(item.getStudentPosition())){
                    throw new BusinessException("学生岗位必选");
                }
                // 学生岗位级别必选: positionLevelType positionLevel
                if(StringUtil.isEmpty(item.getPositionLevelType()) || StringUtil.isEmpty(item.getPositionLevel())){
                    throw new BusinessException("学生岗位级别必选");
                }
                // 获取岗位时间必填: positionTime
                if(item.getPositionTime() == null){
                    throw new BusinessException("获取岗位时间必填");
                }
            }
        } catch (BusinessException e) {
            log.error("StudentManager.checkHonoraryBase.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.checkHonoraryBase.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 查询指定学生的学生岗位评级列表
     *
     * @param studentId
     * @return
     */
    public List<YcqYjsStudentPosition> selectStudentsPositionList(BigInteger studentId) {
        try {
            log.info("StudentManager.selectStudentsPositionList.param：{}", studentId);
            if (studentId == null) {
                throw new BusinessException("学生必选");
            }
            StudentPositionReq studentPositionReq = new StudentPositionReq();
            studentPositionReq.setStudentId(studentId);
            return studentPositionService.selectList(studentPositionReq);
        } catch (BusinessException e) {
            log.error("StudentManager.selectStudentsPositionList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.selectStudentsPositionList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 设置学生入队时间
     *
     * @param enqueueReq
     */
    public void enqueueStudent(StudentEnqueueReq enqueueReq) {
        try {
            log.info("StudentManager.enqueueStudent.param：{}", enqueueReq.toString());
            Date updateDate = new Date();
            // 校验学生是否存在
            List<YcqYjsStudent> studentList = studentService.selectListByIds(enqueueReq.getIdList());
            if (CollectionUtils.isEmpty(studentList) || studentList.size() != enqueueReq.getIdList().size()) {
                throw new BusinessException("学生不存在，操作失败");
            }
            // 校验学生是否是群众
            List<String> politicalStatusTypeList = studentList.stream().filter(item -> item.getPoliticalStatusType() != null).map(YcqYjsStudent::getPoliticalStatusType).distinct().collect(Collectors.toList());
            if (CollectionUtils.isEmpty(politicalStatusTypeList) || politicalStatusTypeList.size() > 1
                    || !PoliticalStatusTypeEnum.THE_MASSES.code().equals(politicalStatusTypeList.get(0))
            ) {
                throw new BusinessException("学生的政治面貌是群众才能入队，操作失败");
            }
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                enqueueReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                enqueueReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            enqueueReq.setLastUpdatedTime(updateDate);
            enqueueReq.setPoliticalStatus(PoliticalStatusTypeEnum.YOUNG_PIONEERS.desc());
            enqueueReq.setPoliticalStatusType(PoliticalStatusTypeEnum.YOUNG_PIONEERS.code());
            enqueueReq.setIfJoinYouthPioneers(IfFlagEnum.TRUE.code());
            studentService.enqueueStudent(enqueueReq);
            // 档案初始
            List<ArchiveChangeReq> archiveCreateList = new ArrayList<>();
            for (YcqYjsStudent ycqYjsStudent : studentList) {
                ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                archiveChangeReq.setStudentId(ycqYjsStudent.getId().toString());
                archiveChangeReq.setActivityType(ActivityTypeEnum.ACTIVITY_TYPE_4.code().toString());
                archiveChangeReq.setEventType(EventTypeEnum.EVENT_3.code());
                archiveChangeReq.setEventId(ycqYjsStudent.getId().toString());
                archiveChangeReq.setParticipateTime(enqueueReq.getJoinYoungPioneersTime());
                archiveChangeReq.setClassName(ycqYjsStudent.getCurrentGradeName() + ycqYjsStudent.getSemesterName());
                archiveChangeReq.setName("加入中国少先队");
                if (currWorker != null) {
                    archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                    archiveChangeReq.setUserName(currWorker.getWorkerName());
                }
                archiveCreateList.add(archiveChangeReq);
            }
            // 存储档案数据
            if (!CollectionUtils.isEmpty(archiveCreateList)) {
                archiveService.insertBatch(archiveCreateList, currWorker);
            }
        } catch (BusinessException e) {
            log.error("StudentManager.enqueueStudent.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.enqueueStudent.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 学生退出少先队
     *
     * @param updateReq
     */
    public void quitEnqueueByStudent(StudentBatchUpdateReq updateReq) throws Exception {
        try {
            log.info("StudentManager.quitEnqueueByStudent.param：{}", updateReq.toString());
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            updateReq.setLastUpdatedTime(updateDate);
            // 校验学生是否存在
            List<YcqYjsStudent> studentList = studentService.selectListByIds(updateReq.getIdList());
            if (CollectionUtils.isEmpty(studentList) || studentList.size() != updateReq.getIdList().size()) {
                throw new BusinessException("学生不存在，操作失败");
            }
            // 校验学生是否是群众
            List<String> politicalStatusTypeList = studentList.stream().filter(item -> item.getPoliticalStatusType() != null).map(YcqYjsStudent::getPoliticalStatusType).distinct().collect(Collectors.toList());
            if (CollectionUtils.isEmpty(politicalStatusTypeList) || politicalStatusTypeList.size() > 1
                    || !PoliticalStatusTypeEnum.YOUNG_PIONEERS.code().equals(politicalStatusTypeList.get(0))
            ) {
                throw new BusinessException("学生的政治面貌是少先队员才能退出少先队，操作失败");
            }
            studentService.quitEnqueueByStudent(updateReq);
            // 同步档案数据
            List<String> studentIdList = studentList.stream().map(YcqYjsStudent::getId).map(String::valueOf).collect(Collectors.toList());
            ArchiveSelectReq archiveSelectReq = new ArchiveSelectReq();
            archiveSelectReq.setStudentIdList(studentIdList);
            archiveSelectReq.setEventType(EventTypeEnum.EVENT_3.code());
            archiveSelectReq.setEventIdList(studentIdList);
            List<ArchiveVo> archiveVoList = archiveService.getList(archiveSelectReq);
            if (!CollectionUtils.isEmpty(archiveVoList)) {
                List<String> archiveVoIds = archiveVoList.stream().map(ArchiveVo::getId).distinct().collect(Collectors.toList());
                archiveService.deleteBatch(archiveVoIds, currWorker);
            }
        } catch (BusinessException e) {
            log.error("StudentManager.quitEnqueueByStudent.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.quitEnqueueByStudent.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 学生退出共青团
     *
     * @param updateReq
     */
    public void quitLeagueByStudent(StudentBatchUpdateReq updateReq) throws Exception {
        try {
            log.info("StudentManager.quitLeagueByStudent.param：{}", updateReq.toString());
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            updateReq.setLastUpdatedTime(updateDate);
            // 校验学生是否存在
            List<YcqYjsStudent> studentList = studentService.selectListByIds(updateReq.getIdList());
            if (CollectionUtils.isEmpty(studentList) || studentList.size() != updateReq.getIdList().size()) {
                throw new BusinessException("学生不存在，操作失败");
            }
            // 校验学生是否是群众
            List<String> politicalStatusTypeList = studentList.stream().filter(item -> item.getPoliticalStatusType() != null).map(YcqYjsStudent::getPoliticalStatusType).distinct().collect(Collectors.toList());
            if (CollectionUtils.isEmpty(politicalStatusTypeList) || politicalStatusTypeList.size() > 1
                    || !PoliticalStatusTypeEnum.YOUTH_LEAGUE.code().equals(politicalStatusTypeList.get(0))
            ) {
                throw new BusinessException("学生的政治面貌是共青团员才能退出共青团，操作失败");
            }
            studentService.quitLeagueByStudent(updateReq);
            // 同步档案数据
            List<String> studentIdList = studentList.stream().map(YcqYjsStudent::getId).map(String::valueOf).collect(Collectors.toList());
            ArchiveSelectReq archiveSelectReq = new ArchiveSelectReq();
            archiveSelectReq.setStudentIdList(studentIdList);
            archiveSelectReq.setEventType(EventTypeEnum.EVENT_6.code());
            archiveSelectReq.setEventIdList(studentIdList);
            List<ArchiveVo> archiveVoList = archiveService.getList(archiveSelectReq);
            if (!CollectionUtils.isEmpty(archiveVoList)) {
                List<String> archiveVoIds = archiveVoList.stream().map(ArchiveVo::getId).distinct().collect(Collectors.toList());
                archiveService.deleteBatch(archiveVoIds, currWorker);
            }
        } catch (BusinessException e) {
            log.error("StudentManager.quitLeagueByStudent.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.quitLeagueByStudent.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 删除学生信息
     *
     * @param updateReq
     */
    public void deleteStudentInfo(StudentInfoDeleteReq updateReq) throws Exception {
        try {
            log.info("StudentManager.StudentInfoDeleteReq.param：{}", updateReq.toString());
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            updateReq.setLastUpdatedTime(updateDate);
            studentService.deleteBatch(updateReq);
        } catch (BusinessException e) {
            log.error("StudentManager.StudentInfoDeleteReq.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.StudentInfoDeleteReq.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 推优入团
     *
     * @param updateReq
     */
    public void promoteBestToLeague(PromoteBestToLeagueReq updateReq) {
        try {
            log.info("StudentManager.promoteBestToLeague.param：{}", updateReq.toString());
            Date updateDate = new Date();
            // 校验学生是否存在
            List<YcqYjsStudent> studentList = studentService.selectListByIds(updateReq.getIdList());
            if (CollectionUtils.isEmpty(studentList) || studentList.size() != updateReq.getIdList().size()) {
                throw new BusinessException("学生不存在，操作失败");
            }
            // 校验学生是否是团队
            List<String> politicalStatusTypeList = studentList.stream().filter(item -> item.getPoliticalStatusType() != null).map(YcqYjsStudent::getPoliticalStatusType).distinct().collect(Collectors.toList());
            if (CollectionUtils.isEmpty(politicalStatusTypeList) || politicalStatusTypeList.size() > 1
                    || !PoliticalStatusTypeEnum.YOUNG_PIONEERS.code().equals(politicalStatusTypeList.get(0))
            ) {
                throw new BusinessException("学生的政治面貌是少先队才能被推优入团，操作失败");
            }
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            updateReq.setJoinYouthLeagueTime(updateDate);
            updateReq.setIfJoinYouthLeague(JoinYouthLeagueFlagEnum.YES.code());
            updateReq.setPoliticalStatus(PoliticalStatusTypeEnum.YOUTH_LEAGUE.getName());
            updateReq.setPoliticalStatusType(PoliticalStatusTypeEnum.YOUTH_LEAGUE.code());
            updateReq.setLastUpdatedTime(updateDate);
            studentService.promoteBestToLeague(updateReq);
            // 档案初始
            List<ArchiveChangeReq> archiveCreateList = new ArrayList<>();
            studentList = studentService.selectListByIds(updateReq.getIdList());
            for (YcqYjsStudent ycqYjsStudent : studentList) {
                ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                archiveChangeReq.setStudentId(ycqYjsStudent.getId().toString());
                archiveChangeReq.setActivityType(ActivityTypeEnum.ACTIVITY_TYPE_5.code().toString());
                archiveChangeReq.setEventType(EventTypeEnum.EVENT_6.code());
                archiveChangeReq.setEventId(ycqYjsStudent.getId().toString());
                archiveChangeReq.setParticipateTime(ycqYjsStudent.getJoinYouthLeagueTime());
                archiveChangeReq.setClassName(ycqYjsStudent.getCurrentGradeName() + ycqYjsStudent.getSemesterName());
                archiveChangeReq.setName("被推优入团");
                if (currWorker != null) {
                    archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                    archiveChangeReq.setUserName(currWorker.getWorkerName());
                }
                archiveCreateList.add(archiveChangeReq);
            }
            // 存储档案数据
            if (!CollectionUtils.isEmpty(archiveCreateList)) {
                archiveService.insertBatch(archiveCreateList, currWorker);
            }
        } catch (BusinessException e) {
            log.error("StudentManager.promoteBestToLeague.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.promoteBestToLeague.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 导师评分
     *
     * @param updateReq
     */
    public void teacherScore(TeacherScoreReq updateReq) {
        try {
            log.info("StudentManager.teacherScore.param：{}", updateReq.toString());
            Date updateDate = new Date();
            // 校验学生是否存在
            List<YcqYjsStudent> studentList = studentService.selectListByIds(updateReq.getIdList());
            if (CollectionUtils.isEmpty(studentList) || studentList.size() != updateReq.getIdList().size()) {
                throw new BusinessException("学生不存在，操作失败");
            }
            // 校验学生是否是团队
            List<String> politicalStatusTypeList = studentList.stream().filter(item -> item.getPoliticalStatusType() != null).map(YcqYjsStudent::getPoliticalStatusType).distinct().collect(Collectors.toList());
            if (CollectionUtils.isEmpty(politicalStatusTypeList) || politicalStatusTypeList.size() > 1
                    || !PoliticalStatusTypeEnum.YOUNG_PIONEERS.code().equals(politicalStatusTypeList.get(0))
            ) {
                throw new BusinessException("学生的政治面貌是团队才能被导师评分，操作失败");
            }
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            updateReq.setLastUpdatedTime(updateDate);
            studentService.teacherScore(updateReq);
        } catch (BusinessException e) {
            log.error("StudentManager.teacherScore.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.teacherScore.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 学生信息excel导出
     *
     * @param response
     * @param req
     */
    public void exportStudentExcel(HttpServletResponse response, StudentReq req) {
        try {
            log.info("StudentManager.exportStudentExcel.param：{}", req.toString());
            List<StudentInfoVo> studentInfoVoList = selectStudentInfoList(req);
            String fileName = "学生信息";
            export(response, fileName, studentInfoVoList, req.getPoliticalStatusType());
        } catch (BusinessException e) {
            log.error("StudentManager.exportStudentExcel.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.exportStudentExcel.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export(HttpServletResponse response, String fileName, List<StudentInfoVo> dataList, String politicalStatusType) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=utf-8''" + encodedFileName + ".xlsx");
            ExcelWriterBuilder write;
            if (politicalStatusType != null && politicalStatusType.equals(PoliticalStatusTypeEnum.YOUNG_PIONEERS.code())) {
                write = EasyExcel.write(response.getOutputStream(), StudentInfoTwoExcel.class);
            } else if (politicalStatusType != null && politicalStatusType.equals(PoliticalStatusTypeEnum.YOUTH_LEAGUE.code())) {
                write = EasyExcel.write(response.getOutputStream(), StudentInfoThreeExcel.class);
            } else {
                write = EasyExcel.write(response.getOutputStream(), StudentInfoExcel.class);
            }
            ExcelWriterSheetBuilder sheet1 = write.sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy());
            if (politicalStatusType != null && politicalStatusType.equals(PoliticalStatusTypeEnum.YOUNG_PIONEERS.code())) {
                List<StudentInfoTwoExcel> studentInfoTwoExcelList = BeanCopyUtil.convert(dataList, StudentInfoTwoExcel.class);
                sheet1.doWrite(studentInfoTwoExcelList);
            } else if (politicalStatusType != null && politicalStatusType.equals(PoliticalStatusTypeEnum.YOUTH_LEAGUE.code())) {
                List<StudentInfoThreeExcel> studentInfoThreeExcelList = BeanCopyUtil.convert(dataList, StudentInfoThreeExcel.class);
                sheet1.doWrite(studentInfoThreeExcelList);
            } else {
                List<StudentInfoExcel> studentInfoExcelList = BeanCopyUtil.convert(dataList, StudentInfoExcel.class);
                sheet1.doWrite(studentInfoExcelList);
            }
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("StudentManager.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export(HttpServletResponse response, String fileName, List<StudentInfoExportExcel> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=utf-8''" + encodedFileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), StudentInfoExportExcel.class)
                    .sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy())
                    .doWrite(dataList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("StudentManager.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void exportTwo(HttpServletResponse response, String fileName, List<StudentInfoTwoExportExcel> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=utf-8''" + encodedFileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), StudentInfoTwoExportExcel.class)
                    .sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy())
                    .doWrite(dataList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("StudentManager.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void exportThree(HttpServletResponse response, String fileName, List<StudentInfoThreeExportExcel> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=utf-8''" + encodedFileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), StudentInfoThreeExportExcel.class)
                    .sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy())
                    .doWrite(dataList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("StudentManager.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 自定义Excel导出策略，设置表头和数据行的字体和高度
     *
     * @return Excel导出策略
     */
    private HorizontalCellStyleStrategy getHeightAndFontStrategy() {
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 11);
        headWriteFont.setBold(true);
        headWriteCellStyle.setWriteFont(headWriteFont);
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
        WriteFont contentWriteFont = new WriteFont();
        contentWriteFont.setFontHeightInPoints((short) 11);
        contentWriteCellStyle.setWriteFont(contentWriteFont);
        contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
    }

    /**
     * 推优入团报告
     *
     * @param studentId
     * @param studentNum
     * @return
     */
    public Map<String, String> promoteBestToLeagueReport(Long studentId, String studentNum) throws Exception {
        YcqYjsStudent ycqYjsStudent = studentService.selectOneById(BigInteger.valueOf(studentId));

        // 1.总分
        PageReq<StudentReq> pageReq = new PageReq<>();
        pageReq.setPageNo(1);
        pageReq.setPageSize(1);
        StudentReq studentReq = new StudentReq();
        studentReq.setStudentNum(studentNum);
        pageReq.setQuery(studentReq);
        PageRsp<StudentInfoVo> studentInfoVoPageRsp = pageSearchStudentInfo(pageReq);
        int totalScore = null != studentInfoVoPageRsp.getResult() && studentInfoVoPageRsp.getResult().size() > 0 ? studentInfoVoPageRsp.getResult().get(0).getTotalScore() : 0;

        // 2.星级奖章数
        StudentMedalInfoReq studentMedalInfoReq = new StudentMedalInfoReq();
        studentMedalInfoReq.setStudentId(BigInteger.valueOf(studentId));
        studentMedalInfoReq.setIfLight(IfFlagEnum.TRUE.code());
        studentMedalInfoReq.setMedalType("2");      // 星级奖章
        int medalCount = studentMedalManager.selectJoinMedalInfoList(studentMedalInfoReq).size();

        // 3.荣誉称号数
        PageReq<HonorReq> honorReqPageReq = new PageReq<>();
        honorReqPageReq.setPageNo(1);
        honorReqPageReq.setPageSize(Integer.MAX_VALUE);
        HonorReq honorReq = new HonorReq();
        honorReq.setStudentId(BigInteger.valueOf(studentId));
        honorReq.setStudentNum(studentNum);
        honorReq.setHonorType(OneAffairConstants.HONOR_TYPE_TITLE);
        honorReqPageReq.setQuery(honorReq);
        Long HonoraryTitleCount = archiveService.pageSearchMyHonor(honorReqPageReq).getTotalNum();

        // 4.服务时长
        YcqYjsServiceDuration ycqYjsServiceDuration = new YcqYjsServiceDuration();
        ycqYjsServiceDuration.setMobile(ycqYjsStudent.getMobile());     // 手机号
        ycqYjsServiceDuration.setCertNo(ycqYjsStudent.getIdentityCardNo());     // 身份证号
        ycqYjsServiceDuration.setDelFlag(CommonConstants.LOG_FLAG_NO);
        List<YcqYjsServiceDuration> serviceDurations = ycqYjsServiceDurationRepo.findAll(Example.of(ycqYjsServiceDuration));
        ycqYjsServiceDuration = CollectionUtils.isEmpty(serviceDurations) ? null : serviceDurations.get(0);

        // 5.学习时长
        StudentCourseReq studentCourseReq = new StudentCourseReq();
        studentCourseReq.setStudentId(BigInteger.valueOf(studentId));
        studentCourseReq.setCompleteStatus(CommonConstants.LOG_FLAG_YES);
        StudentCourseSumVo studentCourseSumVo = studentCourseManager.sumStudentCourse(studentCourseReq);

        // 6.荣誉职务
        PageReq<HonorReq> positionTypeCountPageReq = new PageReq<>();
        positionTypeCountPageReq.setPageNo(1);
        positionTypeCountPageReq.setPageSize(Integer.MAX_VALUE);
        HonorReq positionTypeCountReq = new HonorReq();
        positionTypeCountReq.setStudentId(BigInteger.valueOf(studentId));
        positionTypeCountReq.setStudentNum(studentNum);
        positionTypeCountReq.setHonorType(OneAffairConstants.HONOR_TYPE_POST);
        positionTypeCountPageReq.setQuery(positionTypeCountReq);
        Long positionTypeCount = archiveService.pageSearchMyHonor(positionTypeCountPageReq).getTotalNum();

        // 数据汇总
        Map<String, String> reportMap = new HashMap<>();
        reportMap.put("totalScore", totalScore + "");      // 总分
        reportMap.put("medalCount", medalCount + "");      // 星级奖章数
        reportMap.put("honoraryTitleCount", HonoraryTitleCount + "");      // 荣誉称号数
        reportMap.put("serviceDuration", null != ycqYjsServiceDuration ? ycqYjsServiceDuration.getTotalTime() + "" : "0");      // 服务时长(分钟)
        reportMap.put("studyDuration", studentCourseSumVo.getCompleteCourseDuration() + "");      // 学习时长(分钟)
        reportMap.put("positionTypeCount", positionTypeCount + "");      // 荣誉职务数

        return reportMap;
    }

    /**
     * Excel模板导入批量新增队前学生信息
     *
     * @param file
     * @return
     * @throws BusinessException
     */
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public Boolean importStudentInfo(HttpServletResponse response, MultipartFile file) throws BusinessException {
        try (InputStream inputStream = file.getInputStream()) {
            log.info("StudentManager.importStudentInfo.param：{}", file.getName());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker == null) {
                throw new BusinessException("请登录完成后再进行Excel模板导入批量新增学生信息操作");
            }
            Boolean ifSuccess = true;
            if (file == null) {
                throw new BusinessException("导入文件不能为空");
            }
            String filename = file.getOriginalFilename();
            assert filename != null;
            if (!filename.contains(".")) {
                throw new BusinessException("导入文件不能没有后缀");
            }
            String suffix = filename.substring(filename.lastIndexOf('.'));
            Set<String> extSet = new HashSet<>(6);
            extSet.add(".xlsx");
            extSet.add(".xls");
            extSet.add(".csv");
            boolean flag = extSet.contains(suffix);
            if (!flag) {
                throw new BusinessException("文件后缀名只能为【" + extSet.toString() + "】格式");
            }
            boolean ifCsv = ".csv".equals(suffix);
            // 读取Excel
            StudentImportExcelListener excelListener = new StudentImportExcelListener();
            if (ifCsv) {
                EasyExcel.read(inputStream, StudentInfoImportExcel.class, excelListener).excelType(CSV).charset(Charset.forName("GBK")).headRowNumber(2).sheet(0).doRead();
            } else {
                EasyExcel.read(inputStream, StudentInfoImportExcel.class, excelListener).headRowNumber(2).sheet(0).doRead();
            }
            List<StudentInfoImportExcel> importExcelList = excelListener.getExcelData();
            if (CollectionUtils.isEmpty(importExcelList)) {
                throw new BusinessException("导入文件内容不能为空");
            }
            // 判断是否校验通过
            long count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
            if (count == 0) {
                // 校验所属学校
                List<String> schoolNameList = importExcelList.stream().map(StudentInfoImportExcel::getCurrentSchoolName).distinct().collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(schoolNameList) && schoolNameList.size() > 1) {
                    throw new BusinessException("单次只能导入同一所学校的学生数据");
                }
                TsmOrgReq tsmOrgReq = new TsmOrgReq();
                tsmOrgReq.setParentOrgId(12069L);
                tsmOrgReq.setOrgNameList(schoolNameList);
                List<TsmOrg> tsmOrgList = orgService.selectList(tsmOrgReq);
                Map<String, List<TsmOrg>> tsmOrgMap = new HashMap<>();
                Map<String, List<TeacherInfoVo>> classDirectorInfoMap = new HashMap<>();
                Map<String, List<TeacherInfoVo>> tutorTeacherMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(tsmOrgList)) {
                    tsmOrgMap = tsmOrgList.stream().collect(Collectors.groupingBy(TsmOrg::getOrgName));
                    // 班主任
                    List<String> classDirectorNameList = importExcelList.stream().map(StudentInfoImportExcel::getCurrentClassDirectorName).distinct().collect(Collectors.toList());
                    TeacherReq teacherReq = new TeacherReq();
                    teacherReq.setTeacherNameList(classDirectorNameList);
                    teacherReq.setSchoolId(new BigInteger(String.valueOf(tsmOrgList.get(0).getOrgId())));
                    List<TeacherInfoVo> classDirectorInfoList = teacherService.selectTeacherInfoList(teacherReq);
                    if (!CollectionUtils.isEmpty(classDirectorInfoList)) {
                        classDirectorInfoMap = classDirectorInfoList.stream().collect(Collectors.groupingBy(TeacherInfoVo::getTeacherName));
                    }
                    // 辅导老师
                    List<String> tutorTeacherNameList = importExcelList.stream().map(StudentInfoImportExcel::getTutorTeacherName).distinct().collect(Collectors.toList());
                    teacherReq.setTeacherNameList(tutorTeacherNameList);
                    List<TeacherInfoVo> tutorTeacherList = teacherService.selectTeacherInfoList(teacherReq);
                    if (!CollectionUtils.isEmpty(tutorTeacherList)) {
                        tutorTeacherMap = tutorTeacherList.stream().collect(Collectors.groupingBy(TeacherInfoVo::getTeacherName));
                    }
                }
                // 校验所属组织
                List<String> youthCommitteeOrgNameList = importExcelList.stream().filter(item -> item != null && !StringUtil.isEmpty(item.getYouthCommitteeOrgName())).map(StudentInfoImportExcel::getYouthCommitteeOrgName).distinct().collect(Collectors.toList());
                tsmOrgReq.setParentOrgId(null);
                tsmOrgReq.setOrgNameList(youthCommitteeOrgNameList);
                List<TsmOrg> youthCommitteeOrgList = orgService.selectList(tsmOrgReq);
                Map<String, List<TsmOrg>> youthCommitteeOrgMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(youthCommitteeOrgList)) {
                    youthCommitteeOrgMap = youthCommitteeOrgList.stream().collect(Collectors.groupingBy(TsmOrg::getOrgName));
                }
                Map<String, List<TsmOrg>> finalTsmOrgMap = tsmOrgMap;
                Map<String, List<TsmOrg>> finalYouthCommitteeOrgMap = youthCommitteeOrgMap;
                Map<String, List<TeacherInfoVo>> finalClassDirectorInfoMap = classDirectorInfoMap;
                Map<String, List<TeacherInfoVo>> finalTutorTeacherMap = tutorTeacherMap;
                importExcelList.stream().forEach(item -> {
                    StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                    // 校验学校名称
                    List<TsmOrg> schoolList = finalTsmOrgMap.get(item.getCurrentSchoolName());
                    if (CollectionUtils.isEmpty(schoolList)) {
                        errorMessage.append("【学校名称】在组织管理中不存在 ");
                    } else {
                        item.setCurrentSchoolId(new BigInteger(schoolList.get(0).getOrgId().toString()));
                    }
                    // 校验所属组织
                    String youthCommitteeOrgName = item.getYouthCommitteeOrgName();
                    if (!StringUtil.isEmpty(youthCommitteeOrgName)) {
                        List<TsmOrg> youthCommitteeOrgs = finalYouthCommitteeOrgMap.get(item.getYouthCommitteeOrgName());
                        if (CollectionUtils.isEmpty(youthCommitteeOrgs)) {
                            errorMessage.append("【所属组织】在组织管理中不存在 ");
                        } else {
                            item.setYouthCommitteeOrgId(new BigInteger(youthCommitteeOrgs.get(0).getOrgId().toString()));
                        }
                    }
                    // 班主任姓名
                    String classDirectorName = item.getCurrentClassDirectorName();
                    if (!StringUtil.isEmpty(classDirectorName)) {
                        List<TeacherInfoVo> teacherInfoVos = finalClassDirectorInfoMap.get(classDirectorName);
                        if (CollectionUtils.isEmpty(teacherInfoVos)) {
                            errorMessage.append("【班主任姓名】在教师管理中不存在 ");
                        } else {
                            item.setCurrentClassDirectorId(teacherInfoVos.get(0).getId());
                        }
                    }
                    // 辅导老师姓名
                    String tutorTeacherName = item.getTutorTeacherName();
                    if (!StringUtil.isEmpty(tutorTeacherName)) {
                        List<TeacherInfoVo> teacherInfoVos = finalTutorTeacherMap.get(tutorTeacherName);
                        if (CollectionUtils.isEmpty(teacherInfoVos)) {
                            errorMessage.append("【辅导老师姓名】在教师管理中不存在 ");
                        } else {
                            item.setTutorTeacherId(teacherInfoVos.get(0).getId());
                        }
                    }
                    // 是否检验通过
                    if (!StringUtil.isEmpty(errorMessage.toString().trim())) {
                        item.setVerifyStatus(IfFlagEnum.FALSE.code());
                        item.setErrorMessage(errorMessage.toString());
                    } else {
                        item.setVerifyStatus(IfFlagEnum.TRUE.code());
                    }
                });
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            } else {
                // 导出失败原因
                ifSuccess = false;
            }
            // 校验年级.班级
            if (ifSuccess) {
                // 校验年级.班级
                List<Long> schoolIdList = importExcelList.stream().map(StudentInfoImportExcel::getCurrentSchoolId).distinct().map(item -> new Long(item.toString())).collect(Collectors.toList());
                // 年级
                TsmOrgReq tsmOrgReq = new TsmOrgReq();
                tsmOrgReq.setParentOrgIdList(schoolIdList);
                List<TsmOrg> gradeList = orgService.selectList(tsmOrgReq);
                List<Long> gradeIdList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(gradeList)) {
                    gradeIdList = gradeList.stream().map(TsmOrg::getOrgId).collect(Collectors.toList());
                }
                // 班级
                List<TsmOrg> classList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(gradeIdList)) {
                    tsmOrgReq.setParentOrgIdList(gradeIdList);
                    classList = orgService.selectList(tsmOrgReq);
                }
                List<TsmOrg> finalClassList = classList;
                importExcelList.stream().forEach(item -> {
                    StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                    // 根据学校ID获取年级和班级
                    BigInteger schoolId = item.getCurrentSchoolId();
                    List<TsmOrg> gradeInfoBySchoolList = gradeList.stream().filter(grade -> grade != null
                            && grade.getParentOrgId() != null && String.valueOf(grade.getParentOrgId()).equals(String.valueOf(schoolId))
                    ).collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(gradeInfoBySchoolList)) {
                        errorMessage.append("【年级】在组织管理中未维护 ");
                        item.setVerifyStatus(IfFlagEnum.FALSE.code());
                        item.setErrorMessage(errorMessage.toString());
                    } else {
                        // 年级
                        String gradeName = item.getCurrentGradeName();
                        List<TsmOrg> gradeInfoList = gradeInfoBySchoolList.stream().filter(vo -> vo != null && vo.getOrgName() != null && vo.getOrgName().equals(gradeName)).collect(Collectors.toList());
                        if (CollectionUtils.isEmpty(gradeInfoList)) {
                            errorMessage.append("【年级】在组织管理中未维护 ");
                            item.setVerifyStatus(IfFlagEnum.FALSE.code());
                            item.setErrorMessage(errorMessage.toString());
                        } else {
                            TsmOrg gradeInfo = gradeInfoList.get(0);
                            Long gradeId = gradeInfo.getOrgId();
                            item.setCurrentGradeId(BigInteger.valueOf(gradeId));
                            // 班级
                            String className = item.getCurrentClassName();
                            List<TsmOrg> classInfoList = finalClassList.stream().filter(vo -> vo != null && vo.getOrgName() != null && vo.getOrgName().equals(className) && vo.getParentOrgId() != null && vo.getParentOrgId().equals(gradeId)).collect(Collectors.toList());
                            if (CollectionUtils.isEmpty(classInfoList)) {
                                errorMessage.append("【班级】在组织管理中未维护 ");
                                item.setVerifyStatus(IfFlagEnum.FALSE.code());
                                item.setErrorMessage(errorMessage.toString());
                            } else {
                                TsmOrg classInfo = classInfoList.get(0);
                                Long classId = classInfo.getOrgId();
                                item.setCurrentClassId(BigInteger.valueOf(classId));
                            }
                        }
                    }
                });
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            }
            // 身份证验重：新增数据中身份证重复
            if (ifSuccess) {
                // 新增数据中的重复身份证
                List<String> repeatIdentityCardNoList = importExcelList.stream().map(StudentInfoImportExcel::getIdentityCardNo)
                        .collect(Collectors.toMap(e -> e, e -> 1, (a, b) -> a + b))
                        .entrySet().stream().filter(entry -> entry.getValue() > 1)
                        .map(entry -> entry.getKey()).collect(Collectors.toList());
                // 数据库中的重复身份证
                List<String> identityCardNoList = importExcelList.stream().map(StudentInfoImportExcel::getIdentityCardNo).distinct().collect(Collectors.toList());
                StudentReq studentReq = new StudentReq();
                studentReq.setIdentityCardNoList(identityCardNoList);
                List<YcqYjsStudent> studentInfoVoListByIdentityCardNo = studentService.selectInfoList(studentReq);
                List<String> existIdentityCardNoList = studentInfoVoListByIdentityCardNo.stream().map(YcqYjsStudent::getIdentityCardNo).collect(Collectors.toList());
                if (
                        !CollectionUtils.isEmpty(repeatIdentityCardNoList) || !CollectionUtils.isEmpty(existIdentityCardNoList)
                ) {
                    importExcelList.stream().forEach(item -> {
                        StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                        if (!CollectionUtils.isEmpty(repeatIdentityCardNoList) && repeatIdentityCardNoList.contains(item.getIdentityCardNo())) {
                            errorMessage.append("【身份证号】出现重复数据 ");
                        }
                        if (!CollectionUtils.isEmpty(existIdentityCardNoList) && existIdentityCardNoList.contains(item.getIdentityCardNo())) {
                            errorMessage.append("【身份证号】在学生管理中已维护 ");
                        }
                        // 是否检验通过
                        if (!StringUtil.isEmpty(errorMessage.toString().trim())) {
                            item.setVerifyStatus(IfFlagEnum.FALSE.code());
                            item.setErrorMessage(errorMessage.toString());
                        } else {
                            item.setVerifyStatus(IfFlagEnum.TRUE.code());
                        }
                    });
                }
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            }
            // 校验失败
            if (!ifSuccess) {
                filename = filename.replace(".xlsx", "").replace(".xls", "").replace(".csv", "");
                String fileName = filename + "_校验失败结果";
                export(response, fileName, BeanCopyUtil.convert(importExcelList, StudentInfoExportExcel.class));
            } else {
                // 批量新建
                List<StudentCreateReq> createList = BeanCopyUtil.convert(importExcelList, StudentCreateReq.class);
                createList.stream().forEach(item -> {
                    String studentNum = CharacterCodeUtil.generateCharacterCode(OneAffairConstants.STUDENT_NUM_PREFIX);
                    item.setStudentNum(studentNum);
                    item.setCreatedTime(createDate);
                    item.setDelFlag(DelFlagEnum.UN_DELETE.code());
                    if (currWorker != null) {
                        item.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                        item.setCreatedName(currWorker.getWorkerName());
                    }
                    // 中队名称
                    if (!StringUtil.isEmpty(item.getCurrentGradeName()) && !StringUtil.isEmpty(item.getCurrentClassName())) {
                        item.setMiddleTeamName(item.getCurrentGradeName().replace("年级", "") + "(" + item.getCurrentClassName().replace("班", "") + ")中队");
                    } else {
                        item.setMiddleTeamName(null);
                    }
                    item.setIfJoinYouthPioneers(IfFlagEnum.FALSE.code());
                    item.setJoinYoungPioneersTime(null);
                    item.setIfJoinYouthLeague(IfFlagEnum.FALSE.code());
                    item.setJoinYouthLeagueTime(null);
                    item.setIfJoinCommunistParty(IfFlagEnum.FALSE.code());
                    item.setJoinCommunistPartyTime(null);
                    item.setPoliticalStatus(PoliticalStatusTypeEnum.THE_MASSES.desc());
                    item.setPoliticalStatusType(PoliticalStatusTypeEnum.THE_MASSES.code());
                });
                List<YcqYjsStudent> ycqYjsStudents = studentService.insertBatch(createList);
                Map<String, List<YcqYjsStudent>> studentMap = ycqYjsStudents.stream().collect(Collectors.groupingBy(YcqYjsStudent::getIdentityCardNo));
                importExcelList.stream().forEach(item -> {
                    List<YcqYjsStudent> studentList = studentMap.get(item.getIdentityCardNo());
                    if (!CollectionUtils.isEmpty(studentList)) {
                        YcqYjsStudent student = studentList.get(0);
                        item.setStudentId(student.getId());
                        item.setStudentNum(student.getStudentNum());
                        item.setCompleteStatus(IfFlagEnum.TRUE.code());
                        item.setErrorMessage("");
                    } else {
                        item.setCompleteStatus(IfFlagEnum.FALSE.code());
                    }
                });
                // 任务入库
                ImportRecordCreateReq createReq = new ImportRecordCreateReq();
                createReq.setCreatedTime(createDate);
                createReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
                if (currWorker != null) {
                    createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                    createReq.setCreatedName(currWorker.getWorkerName());
                }
                createReq.setCompleteStatus(TaskCompleteStatusEnum.COMPLETED.code());
                createReq.setCompleteTime(createDate);
                List<StudentInfoImportExcel> successList = importExcelList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code()).collect(Collectors.toList());
                int SuccessSize = CollectionUtils.isEmpty(successList) ? 0 : successList.size();
                createReq.setErrorSize(importExcelList.size() - SuccessSize);
                createReq.setSuccessSize(SuccessSize);
                createReq.setTotalSize(importExcelList.size());
                createReq.setFileName(filename);
                createReq.setOperateType(TaskOperateTypeEnum.TASK_TYPE_2.code());
                createReq.setDataJson(JSON.toJSONString(importExcelList));
                importRecordService.insertOne(createReq);
            }
            return ifSuccess;
        } catch (BusinessException e) {
            log.error("StudentManager.importStudentInfo.error:" + e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.importStudentInfo.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * Excel模板导入批量新增入队学生信息
     *
     * @param file
     * @return
     * @throws BusinessException
     */
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public Boolean importStudentInfoTwo(HttpServletResponse response, MultipartFile file) throws BusinessException {
        try (InputStream inputStream = file.getInputStream()) {
            log.info("StudentManager.importStudentInfoTwo.param：{}", file.getName());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker == null) {
                throw new BusinessException("请登录完成后再进行Excel模板导入批量新增学生信息操作");
            }
            Boolean ifSuccess = true;
            if (file == null) {
                throw new BusinessException("导入文件不能为空");
            }
            String filename = file.getOriginalFilename();
            assert filename != null;
            if (!filename.contains(".")) {
                throw new BusinessException("导入文件不能没有后缀");
            }
            String suffix = filename.substring(filename.lastIndexOf('.'));
            Set<String> extSet = new HashSet<>(6);
            extSet.add(".xlsx");
            extSet.add(".xls");
            extSet.add(".csv");
            boolean flag = extSet.contains(suffix);
            if (!flag) {
                throw new BusinessException("文件后缀名只能为【" + extSet.toString() + "】格式");
            }
            boolean ifCsv = ".csv".equals(suffix);
            // 读取Excel
            StudentTwoImportExcelListener excelListener = new StudentTwoImportExcelListener();
            if (ifCsv) {
                EasyExcel.read(inputStream, StudentInfoTwoImportExcel.class, excelListener).excelType(CSV).charset(Charset.forName("GBK")).headRowNumber(2).sheet(0).doRead();
            } else {
                EasyExcel.read(inputStream, StudentInfoTwoImportExcel.class, excelListener).headRowNumber(2).sheet(0).doRead();
            }
            List<StudentInfoTwoImportExcel> importExcelList = excelListener.getExcelData();
            if (CollectionUtils.isEmpty(importExcelList)) {
                throw new BusinessException("导入文件内容不能为空");
            }
            // 判断是否校验通过
            long count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
            if (count == 0) {
                // 校验所属学校
                List<String> schoolNameList = importExcelList.stream().map(StudentInfoTwoImportExcel::getCurrentSchoolName).distinct().collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(schoolNameList) && schoolNameList.size() > 1) {
                    throw new BusinessException("单次只能导入同一所学校的学生数据");
                }
                TsmOrgReq tsmOrgReq = new TsmOrgReq();
                tsmOrgReq.setParentOrgId(12069L);
                tsmOrgReq.setOrgNameList(schoolNameList);
                List<TsmOrg> tsmOrgList = orgService.selectList(tsmOrgReq);
                Map<String, List<TsmOrg>> tsmOrgMap = new HashMap<>();
                Map<String, List<TeacherInfoVo>> classDirectorInfoMap = new HashMap<>();
                Map<String, List<TeacherInfoVo>> tutorTeacherMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(tsmOrgList)) {
                    tsmOrgMap = tsmOrgList.stream().collect(Collectors.groupingBy(TsmOrg::getOrgName));
                    // 班主任
                    List<String> classDirectorNameList = importExcelList.stream().map(StudentInfoTwoImportExcel::getCurrentClassDirectorName).distinct().collect(Collectors.toList());
                    TeacherReq teacherReq = new TeacherReq();
                    teacherReq.setTeacherNameList(classDirectorNameList);
                    teacherReq.setSchoolId(new BigInteger(String.valueOf(tsmOrgList.get(0).getOrgId())));
                    List<TeacherInfoVo> classDirectorInfoList = teacherService.selectTeacherInfoList(teacherReq);
                    if (!CollectionUtils.isEmpty(classDirectorInfoList)) {
                        classDirectorInfoMap = classDirectorInfoList.stream().collect(Collectors.groupingBy(TeacherInfoVo::getTeacherName));
                    }
                    // 辅导老师
                    List<String> tutorTeacherNameList = importExcelList.stream().map(StudentInfoTwoImportExcel::getTutorTeacherName).distinct().collect(Collectors.toList());
                    teacherReq.setTeacherNameList(tutorTeacherNameList);
                    List<TeacherInfoVo> tutorTeacherList = teacherService.selectTeacherInfoList(teacherReq);
                    if (!CollectionUtils.isEmpty(tutorTeacherList)) {
                        tutorTeacherMap = tutorTeacherList.stream().collect(Collectors.groupingBy(TeacherInfoVo::getTeacherName));
                    }
                }
                // 校验所属组织
                List<String> youthCommitteeOrgNameList = importExcelList.stream().filter(item -> item != null && !StringUtil.isEmpty(item.getYouthCommitteeOrgName())).map(StudentInfoTwoImportExcel::getYouthCommitteeOrgName).distinct().collect(Collectors.toList());
                tsmOrgReq.setParentOrgId(null);
                tsmOrgReq.setOrgNameList(youthCommitteeOrgNameList);
                List<TsmOrg> youthCommitteeOrgList = orgService.selectList(tsmOrgReq);
                Map<String, List<TsmOrg>> youthCommitteeOrgMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(youthCommitteeOrgList)) {
                    youthCommitteeOrgMap = youthCommitteeOrgList.stream().collect(Collectors.groupingBy(TsmOrg::getOrgName));
                }
                Map<String, List<TsmOrg>> finalTsmOrgMap = tsmOrgMap;
                Map<String, List<TsmOrg>> finalYouthCommitteeOrgMap = youthCommitteeOrgMap;
                Map<String, List<TeacherInfoVo>> finalClassDirectorInfoMap = classDirectorInfoMap;
                Map<String, List<TeacherInfoVo>> finalTutorTeacherMap = tutorTeacherMap;
                importExcelList.stream().forEach(item -> {
                    StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                    // 校验学校名称
                    List<TsmOrg> schoolList = finalTsmOrgMap.get(item.getCurrentSchoolName());
                    if (CollectionUtils.isEmpty(schoolList)) {
                        errorMessage.append("【学校名称】在组织管理中不存在 ");
                    } else {
                        item.setCurrentSchoolId(new BigInteger(schoolList.get(0).getOrgId().toString()));
                    }
                    // 校验所属组织
                    String youthCommitteeOrgName = item.getYouthCommitteeOrgName();
                    if (!StringUtil.isEmpty(youthCommitteeOrgName)) {
                        List<TsmOrg> youthCommitteeOrgs = finalYouthCommitteeOrgMap.get(item.getYouthCommitteeOrgName());
                        if (CollectionUtils.isEmpty(youthCommitteeOrgs)) {
                            errorMessage.append("【所属组织】在组织管理中不存在 ");
                        } else {
                            item.setYouthCommitteeOrgId(new BigInteger(youthCommitteeOrgs.get(0).getOrgId().toString()));
                        }
                    }
                    // 班主任姓名
                    String classDirectorName = item.getCurrentClassDirectorName();
                    if (!StringUtil.isEmpty(classDirectorName)) {
                        List<TeacherInfoVo> teacherInfoVos = finalClassDirectorInfoMap.get(classDirectorName);
                        if (CollectionUtils.isEmpty(teacherInfoVos)) {
                            errorMessage.append("【班主任姓名】在教师管理中不存在 ");
                        } else {
                            item.setCurrentClassDirectorId(teacherInfoVos.get(0).getId());
                        }
                    }
                    // 辅导老师姓名
                    String tutorTeacherName = item.getTutorTeacherName();
                    if (!StringUtil.isEmpty(tutorTeacherName)) {
                        List<TeacherInfoVo> teacherInfoVos = finalTutorTeacherMap.get(tutorTeacherName);
                        if (CollectionUtils.isEmpty(teacherInfoVos)) {
                            errorMessage.append("【辅导老师姓名】在教师管理中不存在 ");
                        } else {
                            item.setTutorTeacherId(teacherInfoVos.get(0).getId());
                        }
                    }
                    // 是否检验通过
                    if (!StringUtil.isEmpty(errorMessage.toString().trim())) {
                        item.setVerifyStatus(IfFlagEnum.FALSE.code());
                        item.setErrorMessage(errorMessage.toString());
                    } else {
                        item.setVerifyStatus(IfFlagEnum.TRUE.code());
                    }
                });
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            } else {
                // 导出失败原因
                ifSuccess = false;
            }
            // 校验年级.班级
            if (ifSuccess) {
                // 校验年级.班级
                List<Long> schoolIdList = importExcelList.stream().map(StudentInfoTwoImportExcel::getCurrentSchoolId).distinct().map(item -> new Long(item.toString())).collect(Collectors.toList());
                // 年级
                TsmOrgReq tsmOrgReq = new TsmOrgReq();
                tsmOrgReq.setParentOrgIdList(schoolIdList);
                List<TsmOrg> gradeList = orgService.selectList(tsmOrgReq);
                List<Long> gradeIdList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(gradeList)) {
                    gradeIdList = gradeList.stream().map(TsmOrg::getOrgId).collect(Collectors.toList());
                }
                // 班级
                List<TsmOrg> classList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(gradeIdList)) {
                    tsmOrgReq.setParentOrgIdList(gradeIdList);
                    classList = orgService.selectList(tsmOrgReq);
                }
                List<TsmOrg> finalClassList = classList;
                importExcelList.stream().forEach(item -> {
                    StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                    // 根据学校ID获取年级和班级
                    BigInteger schoolId = item.getCurrentSchoolId();
                    List<TsmOrg> gradeInfoBySchoolList = gradeList.stream().filter(grade -> grade != null
                            && grade.getParentOrgId() != null && String.valueOf(grade.getParentOrgId()).equals(String.valueOf(schoolId))
                    ).collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(gradeInfoBySchoolList)) {
                        errorMessage.append("【年级】在组织管理中未维护 ");
                        item.setVerifyStatus(IfFlagEnum.FALSE.code());
                        item.setErrorMessage(errorMessage.toString());
                    } else {
                        // 年级
                        String gradeName = item.getCurrentGradeName();
                        List<TsmOrg> gradeInfoList = gradeInfoBySchoolList.stream().filter(vo -> vo != null && vo.getOrgName() != null && vo.getOrgName().equals(gradeName)).collect(Collectors.toList());
                        if (CollectionUtils.isEmpty(gradeInfoList)) {
                            errorMessage.append("【年级】在组织管理中未维护 ");
                            item.setVerifyStatus(IfFlagEnum.FALSE.code());
                            item.setErrorMessage(errorMessage.toString());
                        } else {
                            TsmOrg gradeInfo = gradeInfoList.get(0);
                            Long gradeId = gradeInfo.getOrgId();
                            item.setCurrentGradeId(BigInteger.valueOf(gradeId));
                            // 班级
                            String className = item.getCurrentClassName();
                            List<TsmOrg> classInfoList = finalClassList.stream().filter(vo -> vo != null && vo.getOrgName() != null && vo.getOrgName().equals(className) && vo.getParentOrgId() != null && vo.getParentOrgId().equals(gradeId)).collect(Collectors.toList());
                            if (CollectionUtils.isEmpty(classInfoList)) {
                                errorMessage.append("【班级】在组织管理中未维护 ");
                                item.setVerifyStatus(IfFlagEnum.FALSE.code());
                                item.setErrorMessage(errorMessage.toString());
                            } else {
                                TsmOrg classInfo = classInfoList.get(0);
                                Long classId = classInfo.getOrgId();
                                item.setCurrentClassId(BigInteger.valueOf(classId));
                            }
                        }
                    }
                });
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            }
            // 身份证验重：新增数据中身份证重复 / 手机号验重：新增数据中手机号重复
            if (ifSuccess) {
                // 新增数据中的重复身份证
                List<String> repeatIdentityCardNoList = importExcelList.stream().map(StudentInfoTwoImportExcel::getIdentityCardNo)
                        .collect(Collectors.toMap(e -> e, e -> 1, (a, b) -> a + b))
                        .entrySet().stream().filter(entry -> entry.getValue() > 1)
                        .map(entry -> entry.getKey()).collect(Collectors.toList());
                // 数据库中的重复身份证
                List<String> identityCardNoList = importExcelList.stream().map(StudentInfoTwoImportExcel::getIdentityCardNo).distinct().collect(Collectors.toList());
                StudentReq studentReq = new StudentReq();
                studentReq.setIdentityCardNoList(identityCardNoList);
                List<YcqYjsStudent> studentInfoVoListByIdentityCardNo = studentService.selectInfoList(studentReq);
                List<String> existIdentityCardNoList = studentInfoVoListByIdentityCardNo.stream().map(YcqYjsStudent::getIdentityCardNo).collect(Collectors.toList());
                if (
                        !CollectionUtils.isEmpty(repeatIdentityCardNoList) || !CollectionUtils.isEmpty(existIdentityCardNoList)
                ) {
                    importExcelList.stream().forEach(item -> {
                        StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                        if (!CollectionUtils.isEmpty(repeatIdentityCardNoList) && repeatIdentityCardNoList.contains(item.getIdentityCardNo())) {
                            errorMessage.append("【身份证号】出现重复数据 ");
                        }
                        if (!CollectionUtils.isEmpty(existIdentityCardNoList) && existIdentityCardNoList.contains(item.getIdentityCardNo())) {
                            errorMessage.append("【身份证号】在学生管理中已维护 ");
                        }
                        // 是否检验通过
                        if (!StringUtil.isEmpty(errorMessage.toString().trim())) {
                            item.setVerifyStatus(IfFlagEnum.FALSE.code());
                            item.setErrorMessage(errorMessage.toString());
                        } else {
                            item.setVerifyStatus(IfFlagEnum.TRUE.code());
                        }
                    });
                }
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            }
            // 校验失败
            if (!ifSuccess) {
                filename = filename.replace(".xlsx", "").replace(".xls", "").replace(".csv", "");
                String fileName = filename + "_校验失败结果";
                exportTwo(response, fileName, BeanCopyUtil.convert(importExcelList, StudentInfoTwoExportExcel.class));
            } else {
                // 批量新建
                List<StudentCreateReq> createList = BeanCopyUtil.convert(importExcelList, StudentCreateReq.class);
                createList.stream().forEach(item -> {
                    String studentNum = CharacterCodeUtil.generateCharacterCode(OneAffairConstants.STUDENT_NUM_PREFIX);
                    item.setStudentNum(studentNum);
                    item.setCreatedTime(createDate);
                    item.setDelFlag(DelFlagEnum.UN_DELETE.code());
                    if (currWorker != null) {
                        item.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                        item.setCreatedName(currWorker.getWorkerName());
                    }
                    // 中队名称
                    if (!StringUtil.isEmpty(item.getCurrentGradeName()) && !StringUtil.isEmpty(item.getCurrentClassName())) {
                        item.setMiddleTeamName(item.getCurrentGradeName().replace("年级", "") + "(" + item.getCurrentClassName().replace("班", "") + ")中队");
                    } else {
                        item.setMiddleTeamName(null);
                    }
                    item.setIfJoinYouthLeague(IfFlagEnum.FALSE.code());
                    item.setJoinYouthLeagueTime(null);
                    item.setIfJoinCommunistParty(IfFlagEnum.FALSE.code());
                    item.setJoinCommunistPartyTime(null);
                    item.setPoliticalStatus(PoliticalStatusTypeEnum.YOUNG_PIONEERS.desc());
                    item.setPoliticalStatusType(PoliticalStatusTypeEnum.YOUNG_PIONEERS.code());
                });
                List<YcqYjsStudent> ycqYjsStudents = studentService.insertBatch(createList);
                Map<String, List<YcqYjsStudent>> studentMap = ycqYjsStudents.stream().collect(Collectors.groupingBy(YcqYjsStudent::getIdentityCardNo));
                // 档案初始
                List<ArchiveChangeReq> archiveCreateList = new ArrayList<>();
                importExcelList.stream().forEach(item -> {
                    List<YcqYjsStudent> studentList = studentMap.get(item.getIdentityCardNo());
                    if (!CollectionUtils.isEmpty(studentList)) {
                        YcqYjsStudent student = studentList.get(0);
                        item.setStudentId(student.getId());
                        item.setStudentNum(student.getStudentNum());
                        // 入队档案
                        ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                        archiveChangeReq.setStudentId(student.getId().toString());
                        archiveChangeReq.setActivityType(ActivityTypeEnum.ACTIVITY_TYPE_4.code().toString());
                        archiveChangeReq.setEventType(EventTypeEnum.EVENT_3.code());
                        archiveChangeReq.setEventId(student.getId().toString());
                        archiveChangeReq.setParticipateTime(student.getJoinYoungPioneersTime());
                        archiveChangeReq.setClassName(student.getCurrentGradeName() + student.getSemesterName());
                        archiveChangeReq.setName("加入中国少先队");
                        if (currWorker != null) {
                            archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                            archiveChangeReq.setUserName(currWorker.getWorkerName());
                        }
                        archiveCreateList.add(archiveChangeReq);
                        item.setCompleteStatus(IfFlagEnum.TRUE.code());
                        item.setErrorMessage("");
                    } else {
                        item.setCompleteStatus(IfFlagEnum.FALSE.code());
                    }
                });
                // 存储档案数据
                if (!CollectionUtils.isEmpty(archiveCreateList)) {
                    archiveService.insertBatch(archiveCreateList, currWorker);
                }
                // 任务入库
                ImportRecordCreateReq createReq = new ImportRecordCreateReq();
                createReq.setCreatedTime(createDate);
                createReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
                if (currWorker != null) {
                    createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                    createReq.setCreatedName(currWorker.getWorkerName());
                }
                createReq.setCompleteStatus(TaskCompleteStatusEnum.COMPLETED.code());
                createReq.setCompleteTime(createDate);
                List<StudentInfoTwoImportExcel> successList = importExcelList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code()).collect(Collectors.toList());
                int SuccessSize = CollectionUtils.isEmpty(successList) ? 0 : successList.size();
                createReq.setErrorSize(importExcelList.size() - SuccessSize);
                createReq.setSuccessSize(SuccessSize);
                createReq.setTotalSize(importExcelList.size());
                createReq.setFileName(filename);
                createReq.setOperateType(TaskOperateTypeEnum.TASK_TYPE_3.code());
                createReq.setDataJson(JSON.toJSONString(importExcelList));
                importRecordService.insertOne(createReq);
            }
            return ifSuccess;
        } catch (BusinessException e) {
            log.error("StudentManager.importStudentInfoTwo.error:" + e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.importStudentInfoTwo.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * Excel模板导入批量新增推优入团学生信息
     *
     * @param file
     * @return
     * @throws BusinessException
     */
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public Boolean importStudentInfoThree(HttpServletResponse response, MultipartFile file) throws BusinessException {
        try (InputStream inputStream = file.getInputStream()) {
            log.info("StudentManager.importStudentInfoThree.param：{}", file.getName());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker == null) {
                throw new BusinessException("请登录完成后再进行Excel模板导入批量新增学生信息操作");
            }
            Boolean ifSuccess = true;
            if (file == null) {
                throw new BusinessException("导入文件不能为空");
            }
            String filename = file.getOriginalFilename();
            assert filename != null;
            if (!filename.contains(".")) {
                throw new BusinessException("导入文件不能没有后缀");
            }
            String suffix = filename.substring(filename.lastIndexOf('.'));
            Set<String> extSet = new HashSet<>(6);
            extSet.add(".xlsx");
            extSet.add(".xls");
            extSet.add(".csv");
            boolean flag = extSet.contains(suffix);
            if (!flag) {
                throw new BusinessException("文件后缀名只能为【" + extSet.toString() + "】格式");
            }
            boolean ifCsv = ".csv".equals(suffix);
            // 读取Excel
            StudentThreeImportExcelListener excelListener = new StudentThreeImportExcelListener();
            if (ifCsv) {
                EasyExcel.read(inputStream, StudentInfoThreeImportExcel.class, excelListener).excelType(CSV).charset(Charset.forName("GBK")).headRowNumber(2).sheet(0).doRead();
            } else {
                EasyExcel.read(inputStream, StudentInfoThreeImportExcel.class, excelListener).headRowNumber(2).sheet(0).doRead();
            }
            List<StudentInfoThreeImportExcel> importExcelList = excelListener.getExcelData();
            if (CollectionUtils.isEmpty(importExcelList)) {
                throw new BusinessException("导入文件内容不能为空");
            }
            // 判断是否校验通过
            long count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
            if (count == 0) {
                // 校验所属学校
                List<String> schoolNameList = importExcelList.stream().map(StudentInfoThreeImportExcel::getCurrentSchoolName).distinct().collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(schoolNameList) && schoolNameList.size() > 1) {
                    throw new BusinessException("单次只能导入同一所学校的学生数据");
                }
                TsmOrgReq tsmOrgReq = new TsmOrgReq();
                tsmOrgReq.setParentOrgId(12069L);
                tsmOrgReq.setOrgNameList(schoolNameList);
                List<TsmOrg> tsmOrgList = orgService.selectList(tsmOrgReq);
                Map<String, List<TsmOrg>> tsmOrgMap = new HashMap<>();
                Map<String, List<TeacherInfoVo>> classDirectorInfoMap = new HashMap<>();
                Map<String, List<TeacherInfoVo>> tutorTeacherMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(tsmOrgList)) {
                    tsmOrgMap = tsmOrgList.stream().collect(Collectors.groupingBy(TsmOrg::getOrgName));
                    // 班主任
                    List<String> classDirectorNameList = importExcelList.stream().map(StudentInfoThreeImportExcel::getCurrentClassDirectorName).distinct().collect(Collectors.toList());
                    TeacherReq teacherReq = new TeacherReq();
                    teacherReq.setTeacherNameList(classDirectorNameList);
                    teacherReq.setSchoolId(new BigInteger(String.valueOf(tsmOrgList.get(0).getOrgId())));
                    List<TeacherInfoVo> classDirectorInfoList = teacherService.selectTeacherInfoList(teacherReq);
                    if (!CollectionUtils.isEmpty(classDirectorInfoList)) {
                        classDirectorInfoMap = classDirectorInfoList.stream().collect(Collectors.groupingBy(TeacherInfoVo::getTeacherName));
                    }
                    // 辅导老师
                    List<String> tutorTeacherNameList = importExcelList.stream().map(StudentInfoThreeImportExcel::getTutorTeacherName).distinct().collect(Collectors.toList());
                    teacherReq.setTeacherNameList(tutorTeacherNameList);
                    List<TeacherInfoVo> tutorTeacherList = teacherService.selectTeacherInfoList(teacherReq);
                    if (!CollectionUtils.isEmpty(tutorTeacherList)) {
                        tutorTeacherMap = tutorTeacherList.stream().collect(Collectors.groupingBy(TeacherInfoVo::getTeacherName));
                    }
                }
                // 校验所属组织
                List<String> youthCommitteeOrgNameList = importExcelList.stream().filter(item -> item != null && !StringUtil.isEmpty(item.getYouthCommitteeOrgName())).map(StudentInfoThreeImportExcel::getYouthCommitteeOrgName).distinct().collect(Collectors.toList());
                tsmOrgReq.setParentOrgId(null);
                tsmOrgReq.setOrgNameList(youthCommitteeOrgNameList);
                List<TsmOrg> youthCommitteeOrgList = orgService.selectList(tsmOrgReq);
                Map<String, List<TsmOrg>> youthCommitteeOrgMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(youthCommitteeOrgList)) {
                    youthCommitteeOrgMap = youthCommitteeOrgList.stream().collect(Collectors.groupingBy(TsmOrg::getOrgName));
                }
                Map<String, List<TsmOrg>> finalTsmOrgMap = tsmOrgMap;
                Map<String, List<TsmOrg>> finalYouthCommitteeOrgMap = youthCommitteeOrgMap;
                Map<String, List<TeacherInfoVo>> finalClassDirectorInfoMap = classDirectorInfoMap;
                Map<String, List<TeacherInfoVo>> finalTutorTeacherMap = tutorTeacherMap;
                importExcelList.stream().forEach(item -> {
                    StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                    // 校验学校名称
                    List<TsmOrg> schoolList = finalTsmOrgMap.get(item.getCurrentSchoolName());
                    if (CollectionUtils.isEmpty(schoolList)) {
                        errorMessage.append("【学校名称】在组织管理中不存在 ");
                    } else {
                        item.setCurrentSchoolId(new BigInteger(schoolList.get(0).getOrgId().toString()));
                    }
                    // 校验所属组织
                    String youthCommitteeOrgName = item.getYouthCommitteeOrgName();
                    if (!StringUtil.isEmpty(youthCommitteeOrgName)) {
                        List<TsmOrg> youthCommitteeOrgs = finalYouthCommitteeOrgMap.get(item.getYouthCommitteeOrgName());
                        if (CollectionUtils.isEmpty(youthCommitteeOrgs)) {
                            errorMessage.append("【所属组织】在组织管理中不存在 ");
                        } else {
                            item.setYouthCommitteeOrgId(new BigInteger(youthCommitteeOrgs.get(0).getOrgId().toString()));
                        }
                    }
                    // 班主任姓名
                    String classDirectorName = item.getCurrentClassDirectorName();
                    if (!StringUtil.isEmpty(classDirectorName)) {
                        List<TeacherInfoVo> teacherInfoVos = finalClassDirectorInfoMap.get(classDirectorName);
                        if (CollectionUtils.isEmpty(teacherInfoVos)) {
                            errorMessage.append("【班主任姓名】在教师管理中不存在 ");
                        } else {
                            item.setCurrentClassDirectorId(teacherInfoVos.get(0).getId());
                        }
                    }
                    // 辅导老师姓名
                    String tutorTeacherName = item.getTutorTeacherName();
                    if (!StringUtil.isEmpty(tutorTeacherName)) {
                        List<TeacherInfoVo> teacherInfoVos = finalTutorTeacherMap.get(tutorTeacherName);
                        if (CollectionUtils.isEmpty(teacherInfoVos)) {
                            errorMessage.append("【辅导老师姓名】在教师管理中不存在 ");
                        } else {
                            item.setTutorTeacherId(teacherInfoVos.get(0).getId());
                        }
                    }
                    // 是否检验通过
                    if (!StringUtil.isEmpty(errorMessage.toString().trim())) {
                        item.setVerifyStatus(IfFlagEnum.FALSE.code());
                        item.setErrorMessage(errorMessage.toString());
                    } else {
                        item.setVerifyStatus(IfFlagEnum.TRUE.code());
                    }
                });
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            } else {
                // 导出失败原因
                ifSuccess = false;
            }
            // 校验年级.班级
            if (ifSuccess) {
                // 校验年级.班级
                List<Long> schoolIdList = importExcelList.stream().map(StudentInfoThreeImportExcel::getCurrentSchoolId).distinct().map(item -> new Long(item.toString())).collect(Collectors.toList());
                // 年级
                TsmOrgReq tsmOrgReq = new TsmOrgReq();
                tsmOrgReq.setParentOrgIdList(schoolIdList);
                List<TsmOrg> gradeList = orgService.selectList(tsmOrgReq);
                List<Long> gradeIdList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(gradeList)) {
                    gradeIdList = gradeList.stream().map(TsmOrg::getOrgId).collect(Collectors.toList());
                }
                // 班级
                List<TsmOrg> classList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(gradeIdList)) {
                    tsmOrgReq.setParentOrgIdList(gradeIdList);
                    classList = orgService.selectList(tsmOrgReq);
                }
                List<TsmOrg> finalClassList = classList;
                importExcelList.stream().forEach(item -> {
                    StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                    // 根据学校ID获取年级和班级
                    BigInteger schoolId = item.getCurrentSchoolId();
                    List<TsmOrg> gradeInfoBySchoolList = gradeList.stream().filter(grade -> grade != null
                            && grade.getParentOrgId() != null && String.valueOf(grade.getParentOrgId()).equals(String.valueOf(schoolId))
                    ).collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(gradeInfoBySchoolList)) {
                        errorMessage.append("【年级】在组织管理中未维护 ");
                        item.setVerifyStatus(IfFlagEnum.FALSE.code());
                        item.setErrorMessage(errorMessage.toString());
                    } else {
                        // 年级
                        String gradeName = item.getCurrentGradeName();
                        List<TsmOrg> gradeInfoList = gradeInfoBySchoolList.stream().filter(vo -> vo != null && vo.getOrgName() != null && vo.getOrgName().equals(gradeName)).collect(Collectors.toList());
                        if (CollectionUtils.isEmpty(gradeInfoList)) {
                            errorMessage.append("【年级】在组织管理中未维护 ");
                            item.setVerifyStatus(IfFlagEnum.FALSE.code());
                            item.setErrorMessage(errorMessage.toString());
                        } else {
                            TsmOrg gradeInfo = gradeInfoList.get(0);
                            Long gradeId = gradeInfo.getOrgId();
                            item.setCurrentGradeId(BigInteger.valueOf(gradeId));
                            // 班级
                            String className = item.getCurrentClassName();
                            List<TsmOrg> classInfoList = finalClassList.stream().filter(vo -> vo != null && vo.getOrgName() != null && vo.getOrgName().equals(className) && vo.getParentOrgId() != null && vo.getParentOrgId().equals(gradeId)).collect(Collectors.toList());
                            if (CollectionUtils.isEmpty(classInfoList)) {
                                errorMessage.append("【班级】在组织管理中未维护 ");
                                item.setVerifyStatus(IfFlagEnum.FALSE.code());
                                item.setErrorMessage(errorMessage.toString());
                            } else {
                                TsmOrg classInfo = classInfoList.get(0);
                                Long classId = classInfo.getOrgId();
                                item.setCurrentClassId(BigInteger.valueOf(classId));
                            }
                        }
                    }
                });
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            }
            // 身份证验重：新增数据中身份证重复 / 手机号验重：新增数据中手机号重复
            if (ifSuccess) {
                // 新增数据中的重复身份证
                List<String> repeatIdentityCardNoList = importExcelList.stream().map(StudentInfoThreeImportExcel::getIdentityCardNo)
                        .collect(Collectors.toMap(e -> e, e -> 1, (a, b) -> a + b))
                        .entrySet().stream().filter(entry -> entry.getValue() > 1)
                        .map(entry -> entry.getKey()).collect(Collectors.toList());
                // 数据库中的重复身份证
                List<String> identityCardNoList = importExcelList.stream().map(StudentInfoThreeImportExcel::getIdentityCardNo).distinct().collect(Collectors.toList());
                StudentReq studentReq = new StudentReq();
                studentReq.setIdentityCardNoList(identityCardNoList);
                List<YcqYjsStudent> studentInfoVoListByIdentityCardNo = studentService.selectInfoList(studentReq);
                List<String> existIdentityCardNoList = studentInfoVoListByIdentityCardNo.stream().map(YcqYjsStudent::getIdentityCardNo).collect(Collectors.toList());
                if (
                        !CollectionUtils.isEmpty(repeatIdentityCardNoList) || !CollectionUtils.isEmpty(existIdentityCardNoList)
                ) {
                    importExcelList.stream().forEach(item -> {
                        StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                        if (!CollectionUtils.isEmpty(repeatIdentityCardNoList) && repeatIdentityCardNoList.contains(item.getIdentityCardNo())) {
                            errorMessage.append("【身份证号】出现重复数据 ");
                        }
                        if (!CollectionUtils.isEmpty(existIdentityCardNoList) && existIdentityCardNoList.contains(item.getIdentityCardNo())) {
                            errorMessage.append("【身份证号】在学生管理中已维护 ");
                        }
                        // 是否检验通过
                        if (!StringUtil.isEmpty(errorMessage.toString().trim())) {
                            item.setVerifyStatus(IfFlagEnum.FALSE.code());
                            item.setErrorMessage(errorMessage.toString());
                        } else {
                            item.setVerifyStatus(IfFlagEnum.TRUE.code());
                        }
                    });
                }
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            }
            // 校验失败
            if (!ifSuccess) {
                filename = filename.replace(".xlsx", "").replace(".xls", "").replace(".csv", "");
                String fileName = filename + "_校验失败结果";
                exportThree(response, fileName, BeanCopyUtil.convert(importExcelList, StudentInfoThreeExportExcel.class));
            } else {
                // 批量新建
                List<StudentCreateReq> createList = BeanCopyUtil.convert(importExcelList, StudentCreateReq.class);
                createList.stream().forEach(item -> {
                    String studentNum = CharacterCodeUtil.generateCharacterCode(OneAffairConstants.STUDENT_NUM_PREFIX);
                    item.setStudentNum(studentNum);
                    item.setCreatedTime(createDate);
                    item.setDelFlag(DelFlagEnum.UN_DELETE.code());
                    if (currWorker != null) {
                        item.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                        item.setCreatedName(currWorker.getWorkerName());
                    }
                    // 中队名称
                    if (!StringUtil.isEmpty(item.getCurrentGradeName()) && !StringUtil.isEmpty(item.getCurrentClassName())) {
                        item.setMiddleTeamName(item.getCurrentGradeName().replace("年级", "") + "(" + item.getCurrentClassName().replace("班", "") + ")中队");
                    } else {
                        item.setMiddleTeamName(null);
                    }
                    item.setIfJoinCommunistParty(IfFlagEnum.FALSE.code());
                    item.setJoinCommunistPartyTime(null);
                    item.setPoliticalStatus(PoliticalStatusTypeEnum.YOUTH_LEAGUE.desc());
                    item.setPoliticalStatusType(PoliticalStatusTypeEnum.YOUTH_LEAGUE.code());
                });
                List<YcqYjsStudent> ycqYjsStudents = studentService.insertBatch(createList);
                Map<String, List<YcqYjsStudent>> studentMap = ycqYjsStudents.stream().collect(Collectors.groupingBy(YcqYjsStudent::getIdentityCardNo));
                // 档案初始
                List<ArchiveChangeReq> archiveCreateList = new ArrayList<>();
                importExcelList.stream().forEach(item -> {
                    List<YcqYjsStudent> studentList = studentMap.get(item.getIdentityCardNo());
                    if (!CollectionUtils.isEmpty(studentList)) {
                        YcqYjsStudent student = studentList.get(0);
                        item.setStudentId(student.getId());
                        item.setStudentNum(student.getStudentNum());
                        // 入队档案
                        ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                        archiveChangeReq.setStudentId(student.getId().toString());
                        archiveChangeReq.setActivityType(ActivityTypeEnum.ACTIVITY_TYPE_4.code().toString());
                        archiveChangeReq.setEventType(EventTypeEnum.EVENT_3.code());
                        archiveChangeReq.setEventId(student.getId().toString());
                        archiveChangeReq.setParticipateTime(student.getJoinYoungPioneersTime());
                        archiveChangeReq.setClassName(student.getCurrentGradeName() + student.getSemesterName());
                        archiveChangeReq.setName("加入中国少先队");
                        if (currWorker != null) {
                            archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                            archiveChangeReq.setUserName(currWorker.getWorkerName());
                        }
                        archiveCreateList.add(archiveChangeReq);
                        // 入团档案
                        ArchiveChangeReq archiveChangeReq1 = new ArchiveChangeReq();
                        archiveChangeReq1.setStudentId(student.getId().toString());
                        archiveChangeReq1.setActivityType(ActivityTypeEnum.ACTIVITY_TYPE_5.code().toString());
                        archiveChangeReq1.setEventType(EventTypeEnum.EVENT_6.code());
                        archiveChangeReq1.setEventId(student.getId().toString());
                        archiveChangeReq1.setParticipateTime(student.getJoinYouthLeagueTime());
                        archiveChangeReq1.setClassName(student.getCurrentGradeName() + student.getSemesterName());
                        archiveChangeReq1.setName("被推优入团");
                        if (currWorker != null) {
                            archiveChangeReq1.setUserId(String.valueOf(currWorker.getLoginId()));
                            archiveChangeReq1.setUserName(currWorker.getWorkerName());
                        }
                        archiveCreateList.add(archiveChangeReq1);
                        item.setCompleteStatus(IfFlagEnum.TRUE.code());
                        item.setErrorMessage("");
                    } else {
                        item.setCompleteStatus(IfFlagEnum.FALSE.code());
                    }
                });
                // 存储档案数据
                if (!CollectionUtils.isEmpty(archiveCreateList)) {
                    archiveService.insertBatch(archiveCreateList, currWorker);
                }
                // 任务入库
                ImportRecordCreateReq createReq = new ImportRecordCreateReq();
                createReq.setCreatedTime(createDate);
                createReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
                if (currWorker != null) {
                    createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                    createReq.setCreatedName(currWorker.getWorkerName());
                }
                createReq.setCompleteStatus(TaskCompleteStatusEnum.COMPLETED.code());
                createReq.setCompleteTime(createDate);
                List<StudentInfoThreeImportExcel> successList = importExcelList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code()).collect(Collectors.toList());
                int SuccessSize = CollectionUtils.isEmpty(successList) ? 0 : successList.size();
                createReq.setErrorSize(importExcelList.size() - SuccessSize);
                createReq.setSuccessSize(SuccessSize);
                createReq.setTotalSize(importExcelList.size());
                createReq.setFileName(filename);
                createReq.setOperateType(TaskOperateTypeEnum.TASK_TYPE_4.code());
                createReq.setDataJson(JSON.toJSONString(importExcelList));
                importRecordService.insertOne(createReq);
            }
            return ifSuccess;
        } catch (BusinessException e) {
            log.error("StudentManager.importStudentInfoThree.error:" + e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentManager.importStudentInfoThree.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

}
