package com.crunii.micro.service.oneaffair.manager;

import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.oneaffair.dto.oneaffair.*;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveChangeReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveSelectReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveVo;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalActivityInfoVo;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalReq;
import com.crunii.micro.service.oneaffair.dto.vo.medal.MedalRuleCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.medalRule.StudentMedalRuleCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.medalRule.StudentMedalRuleReq;
import com.crunii.micro.service.oneaffair.dto.vo.medalRule.StudentMedalRuleUpdateReq;
import com.crunii.micro.service.oneaffair.dto.vo.medalRule.StudentMedalRuleVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.*;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedalActivity.StudentMedalActivityCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedalActivity.StudentMedalActivityReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedalActivity.StudentMedalActivityUpdateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedalActivity.StudentMedalActivityVo;
import com.crunii.micro.service.oneaffair.enums.*;
import com.crunii.micro.service.oneaffair.service.*;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class StudentMedalManager {


    @Resource
    private StudentMedalService studentMedalService;
    @Resource
    private StudentMedalRuleService studentMedalRuleService;
    @Resource
    private StudentMedalActivityService studentMedalActivityService;
    @Resource
    private MedalService medalService;
    @Resource
    private MedalRuleService medalRuleService;
    @Resource
    private MedalActivityService medalActivityService;
    @Resource
    private StudentService studentService;
    @Resource
    private ArchiveService archiveService;

    /**
     * 查询指定学生待点亮奖章信息列表
     *
     * @param studentId
     * @return
     */
    public List<YcqYjsMedal> selectWaitLightMedalInfoList(BigInteger studentId) {
        try {
            log.info("StudentMedalManager.selectWaitLightMedalInfoList.param：{}", studentId);
            YcqYjsStudent ycqYjsStudent = studentService.selectOneById(studentId);
            if (ycqYjsStudent == null) {
                return Lists.newArrayList();
            }
            MedalReq medalReq = new MedalReq();
            // 排出奖章
            StudentMedalReq studentMedalReq = new StudentMedalReq();
            studentMedalReq.setIfLight(IfFlagEnum.TRUE.code());
            studentMedalReq.setStudentId(studentId);
            List<YcqYjsStudentMedal> studentMedalList = studentMedalService.selectList(studentMedalReq);
            if (!CollectionUtils.isEmpty(studentMedalList)) {
                List<BigInteger> medalIdList = studentMedalList.stream().filter(item -> item != null && item.getMedalId() != null).map(YcqYjsStudentMedal::getMedalId).distinct().collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(medalIdList)) {
                    medalReq.setUnIds(medalIdList);
                }
            }
            // 学期
            List<String> ownerSemesters = new ArrayList<>();
            ownerSemesters.add(SemesterEnum.ALL_YEAR_TERM.code());
            ownerSemesters.add(ycqYjsStudent.getSemesterType());
            medalReq.setOwnerSemesters(ownerSemesters);
            // 组织ID
            medalReq.setOrgId(ycqYjsStudent.getCurrentSchoolId().toString());
            // 年级ID
            medalReq.setOwnerGrade(ycqYjsStudent.getCurrentGradeId().toString());
            List<YcqYjsMedal> ycqYjsMedals = medalService.selectList(medalReq);
            if(!CollectionUtils.isEmpty(ycqYjsMedals)){
                ycqYjsMedals = ycqYjsMedals.stream().sorted(Comparator.comparing(YcqYjsMedal::getCreatedTime)).collect(Collectors.toList());
            }
            return ycqYjsMedals;
        } catch (BusinessException e) {
            log.error("StudentMedalManager.selectWaitLightMedalInfoList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.selectWaitLightMedalInfoList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 根据奖章类型分组查询指定学生待点亮奖章信息列表
     * @param studentId
     * @return
     */
    public List<MedalGroupByMedalTypeInfo> selectWaitLightMedalInfoListGroupByMedalType(BigInteger studentId) {
        try {
            log.info("StudentMedalManager.selectWaitLightMedalInfoListGroupByMedalType.param：{}", studentId);
            return groupYcqYjsMedalListByMedalType(selectWaitLightMedalInfoList(studentId));
        } catch (BusinessException e) {
            log.error("StudentMedalManager.selectWaitLightMedalInfoListGroupByMedalType.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.selectWaitLightMedalInfoListGroupByMedalType.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    public List<MedalGroupByMedalTypeInfo> groupYcqYjsMedalListByMedalType(List<YcqYjsMedal> ycqYjsMedals){
        try {
            log.info("StudentMedalManager.groupYcqYjsMedalListByMedalType.param：{}", ycqYjsMedals.toString());
            if(CollectionUtils.isEmpty(ycqYjsMedals)){
                return Lists.newArrayList();
            }
            List<MedalGroupByMedalTypeInfo> voList = new ArrayList<>();
            Map<String, List<YcqYjsMedal>> medalMap = ycqYjsMedals.stream().collect(Collectors.groupingBy(YcqYjsMedal::getMedalType));
            medalMap.forEach((k,v)->{
                MedalGroupByMedalTypeInfo vo = new MedalGroupByMedalTypeInfo();
                vo.setMedalType(v.get(0).getMedalType());
                vo.setMedalTypeName(v.get(0).getMedalTypeName());
                vo.setMedalList(v);
                voList.add(vo);
            });
            voList.stream().sorted(Comparator.comparing(MedalGroupByMedalTypeInfo::getMedalType)).collect(Collectors.toList());
            return voList;
        } catch (BusinessException e) {
            log.error("StudentMedalManager.groupYcqYjsMedalListByMedalType.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.groupYcqYjsMedalListByMedalType.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 查询指定学生参与的奖章信息列表
     *
     * @param req
     * @return
     */
    public List<StudentMedalInfo> selectJoinMedalInfoList(StudentMedalInfoReq req) {
        try {
            log.info("StudentMedalManager.selectLightMedalInfoList.param：{}", req.toString());
            BigInteger studentId = req.getStudentId();
            YcqYjsStudent ycqYjsStudent = studentService.selectOneById(studentId);
            if (ycqYjsStudent == null) {
                return Lists.newArrayList();
            }
            MedalReq medalReq = new MedalReq();
            medalReq.setOwnerGrade(req.getGradeCode());
            medalReq.setOwnerSemester(req.getSemesterCode());
            // 是否当前学期：false-否；true-是
            if(req.getIfCurrent() != null && req.getIfCurrent()){
                // 上下学期
                medalReq.setOwnerSemester(ycqYjsStudent.getSemesterType());
                // 年级code
                medalReq.setOwnerGrade(ycqYjsStudent.getCurrentGradeId().toString());
            }
            medalReq.setMedalName(req.getMedalName());
            medalReq.setMedalType(req.getMedalType());
            // 已点亮奖章
            StudentMedalReq studentMedalReq = new StudentMedalReq();
            studentMedalReq.setIfLight(req.getIfLight());
            studentMedalReq.setStudentId(studentId);
            List<YcqYjsStudentMedal> studentMedalList = studentMedalService.selectList(studentMedalReq);
            if (CollectionUtils.isEmpty(studentMedalList)) {
                return Lists.newArrayList();
            }
            Map<BigInteger, List<YcqYjsStudentMedal>> studentMedalMap = studentMedalList.stream().collect(Collectors.groupingBy(YcqYjsStudentMedal::getMedalId));
            List<BigInteger> medalIdList = studentMedalList.stream().filter(item -> item != null && item.getMedalId() != null).map(YcqYjsStudentMedal::getMedalId).distinct().collect(Collectors.toList());
            if (CollectionUtils.isEmpty(medalIdList)) {
                return Lists.newArrayList();
            }
            medalReq.setIds(medalIdList);
            List<YcqYjsMedal> ycqYjsMedals = medalService.selectList(medalReq);
            List<StudentMedalInfo> studentMedalInfos = BeanCopyUtil.convert(ycqYjsMedals, StudentMedalInfo.class);
            if (CollectionUtils.isEmpty(studentMedalInfos)) {
                return Lists.newArrayList();
            }
            studentMedalInfos.stream().forEach(item -> {
                List<YcqYjsStudentMedal> ycqYjsStudentMedals = studentMedalMap.get(item.getId());
                if (!CollectionUtils.isEmpty(ycqYjsStudentMedals) && ycqYjsStudentMedals.get(0) != null) {
                    YcqYjsStudentMedal ycqYjsStudentMedal = ycqYjsStudentMedals.get(0);
                    item.setIfLight(ycqYjsStudentMedal.getIfLight());
                    item.setLightProgress(ycqYjsStudentMedal.getLightProgress());
                    item.setLightTime(ycqYjsStudentMedal.getLightTime());
                    item.setStudentId(ycqYjsStudentMedal.getStudentId());
                    item.setStudentMedalId(ycqYjsStudentMedal.getId());
                    item.setStudentMedalCreatedTime(ycqYjsStudentMedal.getCreatedTime());
                }
            });
            return studentMedalInfos;
        } catch (BusinessException e) {
            log.error("StudentMedalManager.selectLightMedalInfoList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.selectLightMedalInfoList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }


    public List<StudentMedalGroupByGradeAndMedalTypeInfo> selectJoinMedalInfoListGroupByGradeAndMedalType(StudentMedalInfoReq req) {
        try {
            log.info("StudentMedalManager.selectJoinMedalInfoListGroupByGradeAndMedalType.param：{}", req.toString());
            return selectJoinMedalInfoListGroupByGradeAndMedalType(selectJoinMedalInfoList(req));
        } catch (BusinessException e) {
            log.error("StudentMedalManager.selectJoinMedalInfoListGroupByGradeAndMedalType.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.selectJoinMedalInfoListGroupByGradeAndMedalType.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    public List<StudentMedalGroupByGradeInfo> selectJoinMedalInfoListGroupByGrade(StudentMedalInfoReq req) {
        try {
            log.info("StudentMedalManager.selectJoinMedalInfoListGroupByGrade.param：{}", req.toString());
            return selectJoinMedalInfoListGroupByGrade(selectJoinMedalInfoList(req));
        } catch (BusinessException e) {
            log.error("StudentMedalManager.selectJoinMedalInfoListGroupByGrade.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.selectJoinMedalInfoListGroupByGrade.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    public List<StudentMedalGroupByMedalTypeInfo> groupMedalInfoListByMedalType(List<StudentMedalInfo> studentMedalInfoList){
        try {
            log.info("StudentMedalManager.groupMedalInfoListByMedalType.param：{}", studentMedalInfoList.toString());
            if(CollectionUtils.isEmpty(studentMedalInfoList)){
                return Lists.newArrayList();
            }
            List<StudentMedalGroupByMedalTypeInfo> voList = new ArrayList<>();
            Map<String, List<StudentMedalInfo>> medalMap = studentMedalInfoList.stream().collect(Collectors.groupingBy(StudentMedalInfo::getMedalType));
            medalMap.forEach((k,v)->{
                StudentMedalGroupByMedalTypeInfo vo = new StudentMedalGroupByMedalTypeInfo();
                vo.setMedalType(v.get(0).getMedalType());
                vo.setMedalTypeName(v.get(0).getMedalTypeName());
                vo.setStudentMedalInfoList(v);
                voList.add(vo);
            });
            voList.stream().sorted(Comparator.comparing(StudentMedalGroupByMedalTypeInfo::getMedalType)).collect(Collectors.toList());
            return voList;
        } catch (BusinessException e) {
            log.error("StudentMedalManager.groupMedalInfoListByMedalType.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.groupMedalInfoListByMedalType.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    public List<StudentMedalGroupByGradeAndMedalTypeInfo> selectJoinMedalInfoListGroupByGradeAndMedalType(List<StudentMedalInfo> studentMedalInfoList){
        try {
            log.info("StudentMedalManager.selectJoinMedalInfoListGroupByGradeAndMedalType.param：{}", studentMedalInfoList.toString());
            if(CollectionUtils.isEmpty(studentMedalInfoList)){
                return Lists.newArrayList();
            }
            studentMedalInfoList.stream().forEach(item ->{
                item.setGroupId(item.getOwnerGrade() + ":" + item.getOwnerSemester());
            });
            List<StudentMedalGroupByGradeAndMedalTypeInfo> voList = new ArrayList<>();
            Map<String, List<StudentMedalInfo>> medalMap = studentMedalInfoList.stream().collect(Collectors.groupingBy(StudentMedalInfo::getGroupId));
            medalMap.forEach((k,v)->{
                if(!CollectionUtils.isEmpty(v)){
                    StudentMedalGroupByGradeAndMedalTypeInfo vo = new StudentMedalGroupByGradeAndMedalTypeInfo();
                    StudentMedalInfo studentMedalInfo = v.get(0);
                    vo.setGroupId(studentMedalInfo.getGroupId());
                    vo.setOwnerGrade(studentMedalInfo.getOwnerGrade());
                    vo.setOwnerGradeName(studentMedalInfo.getOwnerGradeName());
                    vo.setOwnerSemester(studentMedalInfo.getOwnerSemester());
                    vo.setOwnerSemesterName(studentMedalInfo.getOwnerSemesterName());
                    vo.setStudentMedalCreatedTime(studentMedalInfo.getStudentMedalCreatedTime());
                    vo.setStudentMedalGroupByMedalTypeInfoList(groupMedalInfoListByMedalType(v));
                    voList.add(vo);
                }
            });
            voList.stream().sorted(Comparator.comparing(StudentMedalGroupByGradeAndMedalTypeInfo::getStudentMedalCreatedTime)).collect(Collectors.toList());
            return voList;
        } catch (BusinessException e) {
            log.error("StudentMedalManager.selectJoinMedalInfoListGroupByGradeAndMedalType.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.selectJoinMedalInfoListGroupByGradeAndMedalType.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    public List<StudentMedalGroupByGradeInfo> selectJoinMedalInfoListGroupByGrade(List<StudentMedalInfo> studentMedalInfoList){
        try {
            log.info("StudentMedalManager.selectJoinMedalInfoListGroupByGrade.param：{}", studentMedalInfoList.toString());
            if(CollectionUtils.isEmpty(studentMedalInfoList)){
                return Lists.newArrayList();
            }
            studentMedalInfoList.stream().forEach(item ->{
                item.setGroupId(item.getOwnerGrade() + ":" + item.getOwnerSemester());
            });
            List<StudentMedalGroupByGradeInfo> voList = new ArrayList<>();
            Map<String, List<StudentMedalInfo>> medalMap = studentMedalInfoList.stream().collect(Collectors.groupingBy(StudentMedalInfo::getGroupId));
            medalMap.forEach((k,v)->{
                if(!CollectionUtils.isEmpty(v)){
                    StudentMedalGroupByGradeInfo vo = new StudentMedalGroupByGradeInfo();
                    StudentMedalInfo studentMedalInfo = v.get(0);
                    vo.setGroupId(studentMedalInfo.getGroupId());
                    vo.setOwnerGrade(studentMedalInfo.getOwnerGrade());
                    vo.setOwnerGradeName(studentMedalInfo.getOwnerGradeName());
                    vo.setOwnerSemester(studentMedalInfo.getOwnerSemester());
                    vo.setOwnerSemesterName(studentMedalInfo.getOwnerSemesterName());
                    vo.setStudentMedalCreatedTime(studentMedalInfo.getStudentMedalCreatedTime());
                    vo.setStudentMedalInfoList(v);
                    voList.add(vo);
                }
            });
            voList.stream().sorted(Comparator.comparing(StudentMedalGroupByGradeInfo::getStudentMedalCreatedTime)).collect(Collectors.toList());
            return voList;
        } catch (BusinessException e) {
            log.error("StudentMedalManager.selectJoinMedalInfoListGroupByGrade.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.selectJoinMedalInfoListGroupByGrade.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 新增学生奖章
     *
     * @param createReq
     */
    @Transactional(rollbackFor = Exception.class)
    public void createStudentMedalInfo(StudentMedalCreateReq createReq) throws Exception {
        try {
            log.info("StudentMedalManager.createStudentMedalInfo.param：{}", createReq.toString());
            // 查询是否维护
            StudentMedalReq studentMedalReq = new StudentMedalReq();
            studentMedalReq.setStudentId(createReq.getStudentId());
            studentMedalReq.setMedalId(createReq.getMedalId());
            List<YcqYjsStudentMedal> ycqYjsStudentMedals = studentMedalService.selectList(studentMedalReq);
            if (!CollectionUtils.isEmpty(ycqYjsStudentMedals)) {
                throw new BusinessException("当前奖章学生已争章，不能重复争章");
            }
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                createReq.setCreatedName(currWorker.getWorkerName());
            }
            createReq.setCreatedTime(createDate);
            createReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
            // 是否点亮：0-否；1-是 / 点亮进度，单位百分比 / 点亮时间
            createReq.setIfLight(IfFlagEnum.FALSE.code());
            int totalCount = 0;
            int successCount = 0;
            // 是否上传奖章凭证
            MedalRuleCreateReq medalRuleCreateReq = medalRuleService.selectMedalRuleInfo(createReq.getMedalId());
            if (medalRuleCreateReq.getNeedVoucher() != null && medalRuleCreateReq.getNeedVoucher().intValue() == IfFlagEnum.TRUE.code().intValue()) {
                // 需要上传奖章
                totalCount += 1;
                if (createReq.getVoucherCompleteStatus() != null && createReq.getVoucherCompleteStatus().intValue() == IfFlagEnum.TRUE.code().intValue()) {
                    successCount += 1;
                    createReq.setVoucherCompleteTime(createDate);
                }
            }
            // 争章规则列表
            List<StudentMedalRuleCreateReq> studentMedalRuleCreateList = createReq.getStudentMedalRuleCreateList();
            if (!CollectionUtils.isEmpty(studentMedalRuleCreateList)) {
                int studentMedalRuleCount = studentMedalRuleCreateList.size();
                totalCount += studentMedalRuleCount;
                long studentMedalRuleSuccessCount = studentMedalRuleCreateList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code().intValue()).count();
                successCount += studentMedalRuleSuccessCount;
            }
            // 学生奖章活动列表
            List<StudentMedalActivityCreateReq> studentMedalActivityCreateList = createReq.getStudentMedalActivityCreateList();
            if (!CollectionUtils.isEmpty(studentMedalActivityCreateList)) {
                int studentMedalActivityCount = studentMedalActivityCreateList.size();
                totalCount += studentMedalActivityCount;
                long studentMedalActivitySuccessCount = studentMedalActivityCreateList.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code().intValue()).count();
                successCount += studentMedalActivitySuccessCount;
            }
            createReq.setLightProgress(0);
            if (totalCount > 0 && successCount > 0) {
                if (totalCount <= successCount) {
                    createReq.setLightProgress(100);
                } else {
                    Integer lightProgress = (successCount * 100) / totalCount;
                    createReq.setLightProgress(lightProgress);
                }
            }
            YcqYjsStudentMedal ycqYjsStudentMedal = studentMedalService.insertOne(createReq);
            // 争章规则列表
            if (!CollectionUtils.isEmpty(studentMedalRuleCreateList)) {
                studentMedalRuleCreateList.stream().forEach(item -> {
                    item.setCreatedId(item.getCreatedId());
                    item.setCreatedName(item.getCreatedName());
                    item.setCreatedTime(item.getCreatedTime());
                    item.setDelFlag(DelFlagEnum.UN_DELETE.code());
                    item.setStudentMedalId(ycqYjsStudentMedal.getId());
                    if (item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code().intValue()) {
                        item.setCompleteTime(createDate);
                    } else {
                        item.setCompleteStatus(IfFlagEnum.FALSE.code().intValue());
                    }
                });
                studentMedalRuleService.insertBatch(studentMedalRuleCreateList);
            }
            // 学生奖章活动列表
            if (!CollectionUtils.isEmpty(studentMedalActivityCreateList)) {
                studentMedalActivityCreateList.stream().forEach(item -> {
                    item.setCreatedId(item.getCreatedId());
                    item.setCreatedName(item.getCreatedName());
                    item.setCreatedTime(item.getCreatedTime());
                    item.setDelFlag(DelFlagEnum.UN_DELETE.code());
                    item.setStudentMedalId(ycqYjsStudentMedal.getId());
                    if (item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code().intValue()) {
                        item.setCompleteTime(createDate);
                    } else {
                        item.setCompleteStatus(IfFlagEnum.FALSE.code().intValue());
                    }
                });
                studentMedalActivityService.insertBatch(studentMedalActivityCreateList);
            }
        } catch (BusinessException e) {
            log.error("StudentMedalManager.createStudentMedalInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.createStudentMedalInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 查询奖章详情
     * @param req
     * @return
     */
    public StudentMedalDetailVo selectStudentMedalDetail(StudentMedalDetailReq req) {
        try {
            StudentMedalDetailVo studentMedalDetailVo = new StudentMedalDetailVo();
            if (req == null) {
                return studentMedalDetailVo;
            }
            log.info("StudentMedalManager.selectStudentMedalDetail.param：{}", req.toString());
            // 查询奖章
            YcqYjsMedal ycqYjsMedal = medalService.selectInfo(req.getMedalId());
            if (ycqYjsMedal == null) {
                return studentMedalDetailVo;
            }
            BeanUtils.copyProperties(ycqYjsMedal, studentMedalDetailVo);
            studentMedalDetailVo.setStudentId(req.getStudentId());
            studentMedalDetailVo.setIfLight(IfFlagEnum.FALSE.code());
            studentMedalDetailVo.setLightProgress(0);
            // 查询争章规则列表
            MedalRuleCreateReq medalRuleCreateReq = medalRuleService.selectMedalRuleInfo(req.getMedalId());
            studentMedalDetailVo.setNeedVoucher(medalRuleCreateReq.getNeedVoucher());
            List<StudentMedalRuleVo> studentMedalRuleVoList = BeanCopyUtil.convert(medalRuleCreateReq.getRuleInfos(), StudentMedalRuleVo.class);
            // 奖章活动列表
            List<MedalActivityInfoVo> medalActivityInfoVos = medalService.selectActivityList(req.getMedalId());
            List<StudentMedalActivityVo> studentMedalActivityVoList = BeanCopyUtil.convert(medalActivityInfoVos, StudentMedalActivityVo.class);
            // 查询是否维护
            StudentMedalReq studentMedalReq = new StudentMedalReq();
            studentMedalReq.setStudentId(req.getStudentId());
            studentMedalReq.setMedalId(req.getMedalId());
            List<YcqYjsStudentMedal> ycqYjsStudentMedals = studentMedalService.selectList(studentMedalReq);
            if (!CollectionUtils.isEmpty(ycqYjsStudentMedals)) {
                // 已维护过
                YcqYjsStudentMedal ycqYjsStudentMedal = ycqYjsStudentMedals.get(0);
                Integer ifLight = ycqYjsStudentMedal.getIfLight();
                studentMedalDetailVo.setIfLight(ycqYjsStudentMedal.getIfLight());
                studentMedalDetailVo.setLightProgress(ycqYjsStudentMedal.getLightProgress());
                studentMedalDetailVo.setLightTime(ycqYjsStudentMedal.getLightTime());
                studentMedalDetailVo.setStudentMedalId(ycqYjsStudentMedal.getId());
                studentMedalDetailVo.setStudentId(ycqYjsStudentMedal.getStudentId());
                studentMedalDetailVo.setVoucherCompleteStatus(ycqYjsStudentMedal.getVoucherCompleteStatus());
                studentMedalDetailVo.setVoucherCompleteTime(ycqYjsStudentMedal.getVoucherCompleteTime());
                studentMedalDetailVo.setVoucherTime(ycqYjsStudentMedal.getVoucherTime());
                studentMedalDetailVo.setVoucherUrl(ycqYjsStudentMedal.getVoucherUrl());
                // 已有争章规则列表
                StudentMedalRuleReq studentMedalRuleReq = new StudentMedalRuleReq();
                studentMedalRuleReq.setStudentMedalId(ycqYjsStudentMedal.getId());
                List<YcqYjsStudentMedalRule> studentMedalRules = studentMedalRuleService.selectList(studentMedalRuleReq);
                Map<BigInteger, List<YcqYjsStudentMedalRule>> studentMedalRuleMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(studentMedalRules)) {
                    studentMedalRuleMap = studentMedalRules.stream().collect(Collectors.groupingBy(YcqYjsStudentMedalRule::getMedalRuleId));
                }
                if (!CollectionUtils.isEmpty(studentMedalRuleVoList)) {
                    Map<BigInteger, List<YcqYjsStudentMedalRule>> finalStudentMedalRuleMap = studentMedalRuleMap;
                    studentMedalRuleVoList.stream().forEach(item -> {
                        List<YcqYjsStudentMedalRule> ycqYjsStudentMedalRules = finalStudentMedalRuleMap.get(item.getId());
                        if (!CollectionUtils.isEmpty(ycqYjsStudentMedalRules)) {
                            YcqYjsStudentMedalRule studentMedalRule = ycqYjsStudentMedalRules.get(0);
                            item.setCompleteStatus(studentMedalRule.getCompleteStatus());
                            item.setCompleteTime(studentMedalRule.getCompleteTime());
                            item.setStudentMedalId(studentMedalRule.getStudentMedalId());
                            item.setStudentMedalRuleId(studentMedalRule.getId());
                        } else {
                            item.setCompleteStatus(IfFlagEnum.FALSE.code());
                        }
                    });
                    studentMedalRuleVoList = studentMedalRuleVoList.stream().filter(item -> item != null && item.getStudentMedalRuleId() != null).collect(Collectors.toList());
                }
                // 学生奖章活动列表
                StudentMedalActivityReq studentMedalActivityReq = new StudentMedalActivityReq();
                studentMedalActivityReq.setStudentMedalId(ycqYjsStudentMedal.getId());
                List<YcqYjsStudentMedalActivity> studentMedalActivities = studentMedalActivityService.selectList(studentMedalActivityReq);
                Map<BigInteger, List<YcqYjsStudentMedalActivity>> studentMedalActivitieMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(studentMedalActivities)) {
                    studentMedalActivitieMap = studentMedalActivities.stream().collect(Collectors.groupingBy(YcqYjsStudentMedalActivity::getMedalActivityId));
                }
                if (!CollectionUtils.isEmpty(studentMedalActivityVoList)) {
                    Map<BigInteger, List<YcqYjsStudentMedalActivity>> finalStudentMedalActivitieMap = studentMedalActivitieMap;
                    studentMedalActivityVoList.stream().forEach(item -> {
                        List<YcqYjsStudentMedalActivity> studentMedalActivitys = finalStudentMedalActivitieMap.get(new BigInteger(item.getId()));
                        if (!CollectionUtils.isEmpty(studentMedalActivitys)) {
                            YcqYjsStudentMedalActivity studentMedalActivity = studentMedalActivitys.get(0);
                            item.setCompleteStatus(studentMedalActivity.getCompleteStatus());
                            item.setCompleteTime(studentMedalActivity.getCompleteTime());
                            item.setStudentMedalId(studentMedalActivity.getStudentMedalId());
                            item.setStudentMedalActivityId(studentMedalActivity.getId());
                        } else {
                            item.setCompleteStatus(IfFlagEnum.FALSE.code());
                        }
                    });
                    studentMedalActivityVoList = studentMedalActivityVoList.stream().filter(item -> item != null && item.getStudentMedalActivityId() != null).collect(Collectors.toList());
                }
            }else{
                if(!CollectionUtils.isEmpty(studentMedalRuleVoList)){
                    studentMedalRuleVoList.stream().forEach(item -> {
                        item.setCompleteStatus(IfFlagEnum.FALSE.code());
                    });
                }
                if(!CollectionUtils.isEmpty(studentMedalActivityVoList)){
                    studentMedalActivityVoList.stream().forEach(item -> {
                        item.setCompleteStatus(IfFlagEnum.FALSE.code());
                    });
                }
            }
            studentMedalDetailVo.setStudentMedalRuleVoList(studentMedalRuleVoList);
            studentMedalDetailVo.setStudentMedalActivityVoList(studentMedalActivityVoList);
            return studentMedalDetailVo;
        } catch (BusinessException e) {
            log.error("StudentMedalManager.selectStudentMedalDetail.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.selectStudentMedalDetail.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 修改学生奖章
     * @param req
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public void updateStudentMedalInfo(StudentMedalDetailUpdateVo req) throws Exception {
        try {
            log.info("StudentMedalManager.updateStudentMedalInfo.param：{}", req.toString());
            // 查询是否维护
            YcqYjsStudentMedal studentMedal = studentMedalService.selectOneById(req.getStudentMedalId());
            if (studentMedal == null) {
                throw new BusinessException("当前奖章学生未争章，不能修改争章记录");
            }
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                req.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                req.setLastUpdatedName(currWorker.getWorkerName());
            }
            req.setLastUpdatedTime(updateDate);
            studentMedal.setLastUpdatedId(req.getLastUpdatedId());
            studentMedal.setLastUpdatedName(req.getLastUpdatedName());
            studentMedal.setLastUpdatedTime(req.getLastUpdatedTime());
            // 上传奖章凭证
            if(req.getIfVoucher()){
                studentMedal.setVoucherCompleteStatus(IfFlagEnum.TRUE.code());
                studentMedal.setVoucherCompleteTime(updateDate);
            }
            // 争章规则列表
            if(!CollectionUtils.isEmpty(req.getStudentMedalRuleIdList())){
                List<BigInteger> studentMedalRuleIdList = req.getStudentMedalRuleIdList();
                StudentMedalRuleUpdateReq updateReq = new StudentMedalRuleUpdateReq();
                updateReq.setCompleteStatus(IfFlagEnum.TRUE.code());
                updateReq.setCompleteTime(updateDate);
                updateReq.setIdList(studentMedalRuleIdList);
                updateReq.setLastUpdatedId(req.getLastUpdatedId());
                updateReq.setLastUpdatedName(req.getLastUpdatedName());
                updateReq.setLastUpdatedTime(req.getLastUpdatedTime());
                studentMedalRuleService.updateBatch(updateReq);
            }
            // 学生奖章活动列表
            if(!CollectionUtils.isEmpty(req.getStudentMedalActivityIdList())){
                List<BigInteger> studentMedalActivityIdList = req.getStudentMedalActivityIdList();
                StudentMedalActivityUpdateReq updateReq = new StudentMedalActivityUpdateReq();
                updateReq.setCompleteStatus(IfFlagEnum.TRUE.code());
                updateReq.setCompleteTime(updateDate);
                updateReq.setIdList(studentMedalActivityIdList);
                updateReq.setLastUpdatedId(req.getLastUpdatedId());
                updateReq.setLastUpdatedName(req.getLastUpdatedName());
                updateReq.setLastUpdatedTime(req.getLastUpdatedTime());
                studentMedalActivityService.updateBatch(updateReq);
            }
            // 计算进度
            // 是否点亮：0-否；1-是 / 点亮进度，单位百分比 / 点亮时间
            int totalCount = 0;
            int successCount = 0;
            // 是否上传奖章凭证
            MedalRuleCreateReq medalRuleCreateReq = medalRuleService.selectMedalRuleInfo(studentMedal.getMedalId());
            if (medalRuleCreateReq.getNeedVoucher() != null && medalRuleCreateReq.getNeedVoucher().intValue() == IfFlagEnum.TRUE.code().intValue()) {
                // 需要上传奖章
                totalCount += 1;
                if (studentMedal.getVoucherCompleteStatus() != null && studentMedal.getVoucherCompleteStatus().intValue() == IfFlagEnum.TRUE.code().intValue()) {
                    successCount += 1;
                }
            }
            // 争章规则列表
            StudentMedalRuleReq studentMedalRuleReq = new StudentMedalRuleReq();
            studentMedalRuleReq.setStudentMedalId(studentMedal.getId());
            List<YcqYjsStudentMedalRule> studentMedalRules = studentMedalRuleService.selectList(studentMedalRuleReq);
            if (!CollectionUtils.isEmpty(studentMedalRules)) {
                int studentMedalRuleCount = studentMedalRules.size();
                totalCount += studentMedalRuleCount;
                long studentMedalRuleSuccessCount = studentMedalRules.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code().intValue()).count();
                successCount += studentMedalRuleSuccessCount;
            }
            // 学生奖章活动列表
            StudentMedalActivityReq studentMedalActivityReq = new StudentMedalActivityReq();
            studentMedalActivityReq.setStudentMedalId(studentMedal.getId());
            List<YcqYjsStudentMedalActivity> studentMedalActivities = studentMedalActivityService.selectList(studentMedalActivityReq);
            if (!CollectionUtils.isEmpty(studentMedalActivities)) {
                int studentMedalActivityCount = studentMedalActivities.size();
                totalCount += studentMedalActivityCount;
                long studentMedalActivitySuccessCount = studentMedalActivities.stream().filter(item -> item != null && item.getCompleteStatus() != null && item.getCompleteStatus().intValue() == IfFlagEnum.TRUE.code().intValue()).count();
                successCount += studentMedalActivitySuccessCount;
            }
            if (totalCount > 0 && successCount > 0) {
                if (totalCount <= successCount) {
                    studentMedal.setLightProgress(100);
                } else {
                    Integer lightProgress = (successCount * 100) / totalCount;
                    studentMedal.setLightProgress(lightProgress);
                }
            }
            studentMedalService.updateOne(BeanCopyUtil.convert(studentMedal,StudentMedalUpdateReq.class));
        } catch (BusinessException e) {
            log.error("StudentMedalManager.updateStudentMedalInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.updateStudentMedalInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 修改学生奖章的点亮状态
     * @param req
     */
    public void updateStudentMedalLightStatus(StudentMedalLightStatusUpdateVo req) {
        try {
            log.info("StudentMedalManager.updateStudentMedalLightStatus.param：{}", req.toString());
            // 查询是否维护
            YcqYjsStudentMedal studentMedal = studentMedalService.selectOneById(req.getStudentMedalId());
            if (studentMedal == null) {
                throw new BusinessException("当前奖章学生未争章，不能修改奖章的点亮状态");
            }
            // 校验学生是否存在
            YcqYjsStudent studentInfo = studentService.selectOneById(studentMedal.getStudentId());
            if (studentInfo == null) {
                throw new BusinessException("学生不存在，操作失败");
            }
            // 查询学生数据
            String activityType = ActivityTypeEnum.ACTIVITY_TYPE_4.code().toString();
            // 团员
            if(studentInfo.getPoliticalStatus() != null && studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.YOUTH_LEAGUE.code())){
                activityType = ActivityTypeEnum.ACTIVITY_TYPE_5.code().toString();
            }else if(studentInfo.getPoliticalStatus() != null && (studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.COMMUNIST_READY_PARTY_MEMBER.code()) || studentInfo.getPoliticalStatus().equals(PoliticalStatusTypeEnum.COMMUNIST_PARTY_MEMBER.code()))){
                activityType = ActivityTypeEnum.ACTIVITY_TYPE_6.code().toString();
            }
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                studentMedal.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                studentMedal.setLastUpdatedName(currWorker.getWorkerName());
            }
            studentMedal.setLastUpdatedTime(updateDate);
            if(req.getIfVoucher() != null && req.getIfVoucher()){
                // 点亮奖章
                if(studentMedal.getLightProgress() == null || studentMedal.getLightProgress().intValue() != 100){
                    throw new BusinessException("争章任务未全部完成，点亮奖章操作失败");
                }
                studentMedal.setIfLight(IfFlagEnum.TRUE.code());
                studentMedal.setLightTime(updateDate);
                // 同步 档案数据
                ArchiveChangeReq archiveChangeReq = new ArchiveChangeReq();
                archiveChangeReq.setStudentId(studentInfo.getId().toString());
                archiveChangeReq.setActivityType(activityType);
                archiveChangeReq.setEventType(EventTypeEnum.EVENT_5.code());
                archiveChangeReq.setEventId(studentMedal.getId().toString());
                archiveChangeReq.setParticipateTime(studentMedal.getLightTime());
                archiveChangeReq.setClassName(studentInfo.getCurrentGradeName() + studentInfo.getSemesterName());
                // 查询奖章数据
                YcqYjsMedal ycqYjsMedal = medalService.selectInfo(studentMedal.getMedalId());
                archiveChangeReq.setName("获得一枚" + ycqYjsMedal.getMedalTypeName() + "--" + ycqYjsMedal.getMedalName());
                // 图片
                archiveChangeReq.setImg(ycqYjsMedal.getMedalPicKey());
                if (currWorker != null) {
                    archiveChangeReq.setUserId(String.valueOf(currWorker.getLoginId()));
                    archiveChangeReq.setUserName(currWorker.getWorkerName());
                }
                archiveService.insert(archiveChangeReq,currWorker);
            }else{
                // 取消点亮
                studentMedal.setIfLight(IfFlagEnum.FALSE.code());
                studentMedal.setLightTime(null);
                // 同步档案数据
                ArchiveSelectReq archiveSelectReq = new ArchiveSelectReq();
                archiveSelectReq.setStudentId(studentInfo.getId().toString());
                archiveSelectReq.setEventType(EventTypeEnum.EVENT_5.code());
                archiveSelectReq.setEventId(studentMedal.getId().toString());
                List<ArchiveVo> archiveVoList = archiveService.getList(archiveSelectReq);
                if(!CollectionUtils.isEmpty(archiveVoList)){
                    List<String> archiveVoIds = archiveVoList.stream().map(ArchiveVo::getId).distinct().collect(Collectors.toList());
                    archiveService.deleteBatch(archiveVoIds,currWorker);
                }
            }
            studentMedalService.updateOne(BeanCopyUtil.convert(studentMedal,StudentMedalUpdateReq.class));
        } catch (BusinessException e) {
            log.error("StudentMedalManager.updateStudentMedalLightStatus.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentMedalManager.updateStudentMedalLightStatus.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

}
