package com.crunii.micro.service.oneaffair.manager;

import cn.hutool.core.util.DesensitizedUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSON;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.SysService;
import com.crunii.micro.feign.client.dto.OrgNode;
import com.crunii.micro.feign.client.dto.WorkerC;
import com.crunii.micro.service.oneaffair.constants.OneAffairConstants;
import com.crunii.micro.service.oneaffair.dto.excel.TeacherInfoExportExcel;
import com.crunii.micro.service.oneaffair.dto.excel.TeacherInfoImportExcel;
import com.crunii.micro.service.oneaffair.dto.excel.listener.TeacherImportExcelListener;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsTeacher;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsTeacherBindingClass;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.*;
import com.crunii.micro.service.oneaffair.enums.*;
import com.crunii.micro.service.oneaffair.service.ImportRecordService;
import com.crunii.micro.service.oneaffair.service.OrgService;
import com.crunii.micro.service.oneaffair.service.TeacherBindingClassService;
import com.crunii.micro.service.oneaffair.service.TeacherService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import com.crunii.micro.service.oneaffair.util.CharacterCodeUtil;
import com.crunii.micro.service.oneaffair.util.CommonUtil;
import dm.jdbc.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.springframework.beans.BeanUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

import static com.alibaba.excel.support.ExcelTypeEnum.CSV;

@Slf4j
@Service
public class TeacherManager {

    @Resource
    private TeacherService teacherService;
    @Resource
    private TeacherBindingClassService teacherBindingClassService;
    @Resource
    private SysService sysService;
    @Resource
    private OrgService orgService;
    @Resource
    private ImportRecordService importRecordService;

    /**
     * 分页查询教师信息
     *
     * @param req
     * @return
     * @throws Exception
     */
    public PageRsp<TeacherInfoVo> pageSearchTeacherInfo(PageReq<TeacherReq> req) throws Exception {
        log.info("TeacherManager.pageSearchTeacherInfo.param：{}", req.toString());
        try {
            if (req != null && req.getQuery() != null && (req.getQuery().getClassId() != null || req.getQuery().getGradeId() != null
                    || !CollectionUtils.isEmpty(req.getQuery().getClassIdList()) || !CollectionUtils.isEmpty(req.getQuery().getGradeIdList())
            )) {
                TeacherReq query = req.getQuery();
                List<BigInteger> teacherIds = teacherBindingClassService.selectTeacherIds(BeanCopyUtil.convert(query, TeacherBindingClassReq.class));
                if (CollectionUtils.isEmpty(query.getIdList())) {
                    query.setIdList(teacherIds);
                    req.setQuery(query);
                }
            }
            PageRsp<TeacherInfoVo> teacherInfoVoPageRsp = teacherService.pageSearchTeacherInfo(req);
            List<TeacherInfoVo> teacherInfoVoList = teacherInfoVoPageRsp.getResult();
            if (!CollectionUtils.isEmpty(teacherInfoVoList)) {
                List<BigInteger> teacherIdList = teacherInfoVoList.stream().map(TeacherInfoVo::getId).distinct().collect(Collectors.toList());
                List<YcqYjsTeacherBindingClass> teacherBindingClassList = teacherBindingClassService.selectListByTeacherIds(teacherIdList);
                if (!CollectionUtils.isEmpty(teacherBindingClassList)) {
                    Map<BigInteger, List<YcqYjsTeacherBindingClass>> teacherBindingClassMap = teacherBindingClassList.stream().collect(Collectors.groupingBy(YcqYjsTeacherBindingClass::getTeacherId));
                    teacherInfoVoList.forEach(item -> {
                        item.setTeacherBindingClassList(teacherBindingClassMap.get(item.getId()));
                    });
                    teacherInfoVoPageRsp.setResult(teacherInfoVoList);
                }
            }
            return teacherInfoVoPageRsp;
        } catch (BusinessException e) {
            log.error("TeacherManager.pageSearchTeacherInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherManager.pageSearchTeacherInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 新建教师信息
     *
     * @param createReq
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public void createTeacherInfo(TeacherInfoCreateReq createReq) throws Exception {
        try {
            log.info("TeacherManager.createTeacherInfo.param：{}", createReq.toString());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                createReq.setCreatedName(currWorker.getWorkerName());
            }
            // 身份证验证
            String identityCardNo = createReq.getIdentityCardNo();
            if(StringUtil.isEmpty(identityCardNo)){
                throw new BusinessException("身份证号必填");
            }
            if(!CommonUtil.isIDCard(identityCardNo)){
                throw new BusinessException("身份证号格式不正确");
            }
            // 手机号验证
            String mobile = createReq.getMobile();
            if(StringUtil.isEmpty(mobile)){
                throw new BusinessException("手机号必填");
            }
            if(!CommonUtil.ifMobile(mobile)){
                throw new BusinessException("手机号格式不正确");
            }
            // 身份证验重
            List<YcqYjsTeacher> studentInfoList = teacherService.selectListByIdentityCardNo(identityCardNo);
            if (!CollectionUtils.isEmpty(studentInfoList)) {
                throw new BusinessException("该教师身份证信息已存在无法新增");
            }
            // 手机号验重
            List<YcqYjsTeacher> studentInfoListByMobile = teacherService.selectListByMobile(mobile);
            if (!CollectionUtils.isEmpty(studentInfoListByMobile)) {
                throw new BusinessException("该教师手机号信息已存在无法新增");
            }
            // 教师编号 : 业务首字母大写(如JSGL）+ 时间(秒) + 4位随机码(26个 英文字母+数字)
            String teacherNum = CharacterCodeUtil.generateCharacterCode(OneAffairConstants.TEACHER_NUM_PREFIX);
            createReq.setTeacherNum(teacherNum);
            createReq.setCreatedTime(createDate);
            createReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
            YcqYjsTeacher ycqYjsTeacher = teacherService.insertOne(createReq);
            if (ycqYjsTeacher == null) {
                throw new BusinessException("教师数据新增失败");
            }
            // 负责年级/班级必选
            List<GradeInfo> gradeInfoList = createReq.getGradeInfoList();
            List<YcqYjsTeacherBindingClass> teacherBindingClassList = transfer(gradeInfoList, ycqYjsTeacher.getId(), createReq.getCreatedId(), createDate);
            if (!CollectionUtils.isEmpty(teacherBindingClassList)) {
                List<TeacherBindingClassCreateReq> teacherBindingClassCreateList = BeanCopyUtil.convert(teacherBindingClassList, TeacherBindingClassCreateReq.class);
                teacherBindingClassService.insertBatch(teacherBindingClassCreateList);
            }
            // 新增用户数据
            // 新增
            WorkerC workerC = new WorkerC();
            workerC.setOrgId(createReq.getSchoolId().toString());
            workerC.setMobile(createReq.getMobile());
            workerC.setSex(createReq.getSex());
            workerC.setWorkerName(createReq.getTeacherName());
            workerC.setState(IfFlagEnum.TRUE.code());
            workerC.setLoginCode(createReq.getMobile());
            workerC.setRoleIds(Arrays.asList(UseRoleEnum.TEACHER.code()));
            Result<Integer> worker = sysService.createWorkerInfo(workerC);
            if (worker == null || worker.getData() == null) {
                throw new BusinessException("新增学生数据时同步新增用户数据失败");
            }
            BigInteger loginId = new BigInteger(String.valueOf(worker.getData()));
            // 绑定
            ycqYjsTeacher.setUserId(loginId);
            teacherService.updateOne(BeanCopyUtil.convert(ycqYjsTeacher, TeacherInfoUpdateReq.class));
        } catch (BusinessException e) {
            log.error("TeacherManager.createTeacherInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherManager.createTeacherInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 建立教师与用户的绑定关系
     *
     * @param teacherId
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public void bindingTeacherToUserById(BigInteger teacherId) throws Exception {
        try {
            log.info("TeacherManager.bindingTeacherToUserById.param：{}", teacherId.toString());
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            YcqYjsTeacher ycqYjsTeacher = teacherService.selectOneById(teacherId);
            if(ycqYjsTeacher == null){
                throw new BusinessException("教师数据不存在，同步创建用户失败");
            }
            if(ycqYjsTeacher.getUserId() != null){
                return;
            }
            // 新增用户数据
            WorkerC workerC = new WorkerC();
            workerC.setOrgId(ycqYjsTeacher.getSchoolId().toString());
            workerC.setMobile(ycqYjsTeacher.getMobile());
            workerC.setSex(ycqYjsTeacher.getSex());
            workerC.setWorkerName(ycqYjsTeacher.getTeacherName());
            workerC.setState(IfFlagEnum.TRUE.code());
            workerC.setLoginCode(ycqYjsTeacher.getMobile());
            workerC.setRoleIds(Arrays.asList(UseRoleEnum.TEACHER.code()));
            log.info("新增教师数据时同步新增用户数据: {}",workerC);
            Result<Integer> worker = sysService.createWorkerInfo(workerC);
            if (worker == null || worker.getData() == null) {
                log.error("新增教师数据时同步新增用户数据失败: {}",worker);
                throw new BusinessException("新增教师数据时同步新增用户数据失败");
            }
            BigInteger loginId = new BigInteger(String.valueOf(worker.getData()));
            // 绑定
            ycqYjsTeacher.setUserId(loginId);
            if(currWorker != null){
                ycqYjsTeacher.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                ycqYjsTeacher.setLastUpdatedName(currWorker.getWorkerName());
            }
            ycqYjsTeacher.setLastUpdatedTime(updateDate);
            teacherService.updateOne(BeanCopyUtil.convert(ycqYjsTeacher, TeacherInfoUpdateReq.class));
        } catch (BusinessException e) {
            log.error("TeacherManager.createTeacherInfo.error:{}", e);
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherManager.createTeacherInfo.error:{}", e);
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }



    public List<YcqYjsTeacherBindingClass> transfer(List<GradeInfo> gradeInfoList, BigInteger teacherId, BigInteger createdId, Date createDate) {
        if (CollectionUtils.isEmpty(gradeInfoList)) {
            return null;
        }
        List<YcqYjsTeacherBindingClass> teacherBindingClassList = new ArrayList<>();
        for (GradeInfo gradeInfo : gradeInfoList) {
            List<ClassInfo> classInfoList = gradeInfo.getClassInfoList();
            if (!CollectionUtils.isEmpty(classInfoList)) {
                for (ClassInfo classInfo : classInfoList) {
                    YcqYjsTeacherBindingClass item = BeanCopyUtil.convert(classInfo, YcqYjsTeacherBindingClass.class);
                    BeanUtils.copyProperties(gradeInfo, item);
                    if (item.getGradeId() != null && item.getClassId() != null) {
                        item.setGroupId(item.getGradeId() + ":" + item.getClassId());
                        item.setTeacherId(teacherId);
                        item.setCreatedTime(createDate);
                        item.setCreatedId(createdId);
                        item.setDelFlag(DelFlagEnum.UN_DELETE.code());
                        item.setIfAll(gradeInfo.getIfAll());
                        teacherBindingClassList.add(item);
                    }
                }
            }
        }
        return teacherBindingClassList;
    }

    /**
     * 修改教师信息
     *
     * @param updateReq
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public void updateStudentInfo(TeacherInfoUpdateReq updateReq) throws Exception {
        try {
            log.info("TeacherManager.updateStudentInfo.param：{}", updateReq.toString());
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                updateReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                updateReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            // 校验教师是否存在
            YcqYjsTeacher teacherInfo = teacherService.selectOneById(updateReq.getId());
            if (teacherInfo == null) {
                throw new BusinessException("教师数据未维护，操作失败");
            }
            updateReq.setLastUpdatedTime(updateDate);
            // 判断脱敏后是否修改
            // 手机号
            String mobile = updateReq.getMobile();
            if(StringUtil.isEmpty(mobile)){
                throw new BusinessException("手机号必填");
            }
            if (mobile.equals(DesensitizedUtil.mobilePhone(teacherInfo.getMobile()))) {
                updateReq.setMobile(teacherInfo.getMobile());
            } else {
                // 手机号验证
                if(!CommonUtil.ifMobile(mobile)){
                    throw new BusinessException("手机号格式不正确");
                }
                // 手机号验重
                List<YcqYjsTeacher> studentInfoList = teacherService.selectListByMobile(mobile);
                if (!CollectionUtils.isEmpty(studentInfoList)
                        && (studentInfoList.size() > 1 || !studentInfoList.get(0).getId().equals(updateReq.getId()))) {
                    throw new BusinessException("该教师手机号信息已存在无法修改");
                }
            }
            // 身份证
            String identityCardNo = updateReq.getIdentityCardNo();
            if(StringUtil.isEmpty(identityCardNo)){
                throw new BusinessException("身份证号必填");
            }
            if (identityCardNo.equals(DesensitizedUtil.mobilePhone(teacherInfo.getIdentityCardNo()))) {
                updateReq.setIdentityCardNo(teacherInfo.getIdentityCardNo());
            } else {
                // 身份证验证
                if(!CommonUtil.isIDCard(identityCardNo)){
                    throw new BusinessException("身份证号格式不正确");
                }
                // 身份证验重
                List<YcqYjsTeacher> studentInfoList = teacherService.selectListByIdentityCardNo(identityCardNo);
                if (!CollectionUtils.isEmpty(studentInfoList)
                        && (studentInfoList.size() > 1 || !studentInfoList.get(0).getId().equals(updateReq.getId()))) {
                    throw new BusinessException("该教师身份证信息已存在无法修改");
                }
            }
            teacherService.updateOne(updateReq);
            // 负责年级/班级必选
            List<YcqYjsTeacherBindingClass> teacherBindingClassList = transfer(updateReq.getGradeInfoList(), updateReq.getId(), updateReq.getLastUpdatedId(), updateDate);
            if (!CollectionUtils.isEmpty(teacherBindingClassList)) {
                List<BigInteger> idList = teacherBindingClassService.selectIdListByTeacherIds(Arrays.asList(updateReq.getId()));
                if (!CollectionUtils.isEmpty(idList)) {
                    teacherBindingClassService.clearBatch(idList);
                }
                List<TeacherBindingClassCreateReq> teacherBindingClassCreateList = BeanCopyUtil.convert(teacherBindingClassList, TeacherBindingClassCreateReq.class);
                teacherBindingClassService.insertBatch(teacherBindingClassCreateList);
            }
        } catch (BusinessException e) {
            log.error("TeacherManager.updateStudentInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherManager.updateStudentInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 删除教师信息
     *
     * @param deleteReq
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public void deleteTeacherInfo(TeacherInfoDeleteReq deleteReq) throws Exception {
        try {
            log.info("TeacherManager.deleteTeacherInfo.param：{}", deleteReq.toString());
            Date updateDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if (currWorker != null) {
                deleteReq.setLastUpdatedId(BigInteger.valueOf(currWorker.getLoginId()));
                deleteReq.setLastUpdatedName(currWorker.getWorkerName());
            }
            // 校验教师是否存在
            List<YcqYjsTeacher> ycqYjsTeachers = teacherService.selectListByIds(deleteReq.getIdList());
            if (CollectionUtils.isEmpty(ycqYjsTeachers)) {
                throw new BusinessException("教师数据未维护，操作失败");
            }
            List<BigInteger> teacherIdList = ycqYjsTeachers.stream().map(YcqYjsTeacher::getId).distinct().collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(teacherIdList)) {
                deleteReq.setIdList(teacherIdList);
                deleteReq.setLastUpdatedTime(updateDate);
                teacherService.deleteBatch(deleteReq);
                List<BigInteger> teacherBindingClassIdList = teacherBindingClassService.selectIdListByTeacherIds(teacherIdList);
                if (!CollectionUtils.isEmpty(teacherBindingClassIdList)) {
                    TeacherBindingClassDeleteReq teacherBindingClassDeleteReq = new TeacherBindingClassDeleteReq();
                    teacherBindingClassDeleteReq.setIdList(teacherBindingClassIdList);
                    teacherBindingClassDeleteReq.setLastUpdatedId(deleteReq.getLastUpdatedId());
                    teacherBindingClassDeleteReq.setLastUpdatedTime(updateDate);
                    teacherBindingClassService.deleteBatch(teacherBindingClassDeleteReq);
                }
            }
        } catch (BusinessException e) {
            log.error("TeacherManager.deleteTeacherInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherManager.deleteTeacherInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 通过父组织Id获取子组织(数据权限过滤)
     *
     * @param parentOrgId
     * @return
     */
    public List<OrgNode> getOrgChildrenByParentOrgId(Long parentOrgId) {
        try {
            log.info("TeacherManager.getOrgChildrenByParentOrgId.param：{}", parentOrgId);
            Result<List<OrgNode>> orgChildren = sysService.getOrgChildrenByParentOrgId(parentOrgId);
            if (orgChildren == null || !Result.DEFAULT_CODE.equals(orgChildren.getCode())) {
                throw new BusinessException("通过父组织Id获取子组织失败");
            }
            log.info("sysService.getOrgChildrenByParentOrgId.result：{}", orgChildren);
            List<OrgNode> dataList = orgChildren.getData();
            if (!CollectionUtils.isEmpty(dataList)) {
                // 过滤
                Worker currWorker = CurrWorkerUtil.getCurrWorker();
                if (currWorker != null) {
                    log.info("StudentManager.currWorker.result：{}", currWorker.toString());
                    TeacherReq teacherReq = new TeacherReq();
                    teacherReq.setUserId(new BigInteger(String.valueOf(currWorker.getLoginId())));
                    List<TeacherInfoVo> teacherInfoVos = teacherService.selectTeacherInfoList(teacherReq);
                    log.info("teacherService.selectTeacherInfoList.result：{}", teacherInfoVos);
                    if (!CollectionUtils.isEmpty(teacherInfoVos)) {
                        List<BigInteger> teacherIds = teacherInfoVos.stream().map(TeacherInfoVo::getId).collect(Collectors.toList());
                        List<YcqYjsTeacherBindingClass> teacherBindingClassList = teacherBindingClassService.selectListByTeacherIds(teacherIds);
                        log.info("teacherBindingClassService.selectListByTeacherIds.result：{}", teacherBindingClassList);
                        if (!CollectionUtils.isEmpty(teacherBindingClassList)) {
                            List<BigInteger> orgIds = new ArrayList<>();
                            // 年级
                            List<BigInteger> gradeIds = teacherBindingClassList.stream().filter(item -> item != null && item.getGradeId() != null).map(YcqYjsTeacherBindingClass::getGradeId).collect(Collectors.toList());
                            orgIds.addAll(gradeIds);
                            // 班级
                            List<BigInteger> classIds = teacherBindingClassList.stream().filter(item -> item != null && item.getClassId() != null).map(YcqYjsTeacherBindingClass::getClassId).collect(Collectors.toList());
                            orgIds.addAll(classIds);
                            if (!CollectionUtils.isEmpty(orgIds)) {
                                List<String> orgIdStrList = orgIds.stream().distinct().map(intValue -> String.valueOf(intValue)).collect(Collectors.toList());
                                log.info("teacherBindingClassService.orgIds.result：{}", orgIdStrList);
                                log.info("teacherBindingClassService.data.result：{}", dataList);
                                dataList = dataList.stream().filter(item -> item != null && item.getOrgId() != null && orgIdStrList.contains((String.valueOf(item.getOrgId())))).collect(Collectors.toList());
                            }
                        }
                    }
                }
            }
            return dataList;
        } catch (BusinessException e) {
            log.error("TeacherManager.getOrgChildrenByParentOrgId.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherManager.getOrgChildrenByParentOrgId.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * Excel模板导入批量新增教师信息
     *
     * @param file
     * @return
     * @throws BusinessException
     */
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public Boolean importTeacherInfo(HttpServletResponse response, MultipartFile file) throws BusinessException {
        try (InputStream inputStream = file.getInputStream()) {
            log.info("TeacherManager.importTeacherInfo.param：{}", file.getName());
            Date createDate = new Date();
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            if(currWorker == null){
                throw new BusinessException("请登录完成后再进行Excel模板导入批量新增教师信息操作");
            }
            Boolean ifSuccess = true;
            if (file == null) {
                throw new BusinessException("导入文件不能为空");
            }
            String filename = file.getOriginalFilename();
            assert filename != null;
            if (!filename.contains(".")) {
                throw new BusinessException("导入文件不能没有后缀");
            }
            String suffix = filename.substring(filename.lastIndexOf('.'));
            Set<String> extSet = new HashSet<>(6);
            extSet.add(".xlsx");
            extSet.add(".xls");
            extSet.add(".csv");
            boolean flag = extSet.contains(suffix);
            if (!flag) {
                throw new BusinessException("文件后缀名只能为【" + extSet.toString() + "】格式");
            }
            boolean ifCsv = ".csv".equals(suffix);
            // 读取Excel
            TeacherImportExcelListener excelListener = new TeacherImportExcelListener();
            if (ifCsv) {
                EasyExcel.read(inputStream, TeacherInfoImportExcel.class, excelListener).excelType(CSV).charset(Charset.forName("GBK")).headRowNumber(2).sheet(0).doRead();
            } else {
                EasyExcel.read(inputStream, TeacherInfoImportExcel.class, excelListener).headRowNumber(2).sheet(0).doRead();
            }
            List<TeacherInfoImportExcel> importExcelList = excelListener.getExcelData();
            if (CollectionUtils.isEmpty(importExcelList)) {
                throw new BusinessException("导入文件内容不能为空");
            }
            // 判断是否校验通过
            long count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
            if (count == 0) {
                // 校验所属学校
                List<String> schoolNameList = importExcelList.stream().map(TeacherInfoImportExcel::getSchoolName).distinct().collect(Collectors.toList());
                TsmOrgReq tsmOrgReq = new TsmOrgReq();
                tsmOrgReq.setParentOrgId(12069L);
                tsmOrgReq.setOrgNameList(schoolNameList);
                List<TsmOrg> tsmOrgList = orgService.selectList(tsmOrgReq);
                Map<String, List<TsmOrg>> tsmOrgMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(tsmOrgList)) {
                    tsmOrgMap = tsmOrgList.stream().collect(Collectors.groupingBy(TsmOrg::getOrgName));
                }
                // 校验所属组织
                List<String> youthCommitteeOrgNameList = importExcelList.stream().filter(item -> item != null && !StringUtil.isEmpty(item.getYouthCommitteeOrgName())).map(TeacherInfoImportExcel::getYouthCommitteeOrgName).distinct().collect(Collectors.toList());
                tsmOrgReq.setParentOrgId(null);
                tsmOrgReq.setOrgNameList(youthCommitteeOrgNameList);
                List<TsmOrg> youthCommitteeOrgList = orgService.selectList(tsmOrgReq);
                Map<String, List<TsmOrg>> youthCommitteeOrgMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(youthCommitteeOrgList)) {
                    youthCommitteeOrgMap = youthCommitteeOrgList.stream().collect(Collectors.groupingBy(TsmOrg::getOrgName));
                }
                Map<String, List<TsmOrg>> finalTsmOrgMap = tsmOrgMap;
                Map<String, List<TsmOrg>> finalYouthCommitteeOrgMap = youthCommitteeOrgMap;
                importExcelList.stream().forEach(item -> {
                    StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                    // 校验所属学校
                    List<TsmOrg> schoolList = finalTsmOrgMap.get(item.getSchoolName());
                    if (CollectionUtils.isEmpty(schoolList)) {
                        errorMessage.append("【所属学校】在组织管理中不存在 ");
                    } else {
                        item.setSchoolId(new BigInteger(schoolList.get(0).getOrgId().toString()));
                    }
                    // 校验所属组织
                    String youthCommitteeOrgName = item.getYouthCommitteeOrgName();
                    if (!StringUtil.isEmpty(youthCommitteeOrgName)) {
                        List<TsmOrg> youthCommitteeOrgs = finalYouthCommitteeOrgMap.get(item.getYouthCommitteeOrgName());
                        if (CollectionUtils.isEmpty(youthCommitteeOrgs)) {
                            errorMessage.append("【所属组织】在组织管理中不存在 ");
                        } else {
                            item.setYouthCommitteeOrgId(new BigInteger(youthCommitteeOrgs.get(0).getOrgId().toString()));
                        }
                    }
                    // 是否检验通过
                    if (!StringUtil.isEmpty(errorMessage.toString().trim())) {
                        item.setVerifyStatus(IfFlagEnum.FALSE.code());
                        item.setErrorMessage(errorMessage.toString());
                    } else {
                        item.setVerifyStatus(IfFlagEnum.TRUE.code());
                    }
                });
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            } else {
                // 导出失败原因
                ifSuccess = false;
            }
            // 校验年级.班级
            if (ifSuccess) {
                // 校验年级.班级
                List<Long> schoolIdList = importExcelList.stream().map(TeacherInfoImportExcel::getSchoolId).distinct().map(item -> new Long(item.toString())).collect(Collectors.toList());
                // 年级
                TsmOrgReq tsmOrgReq = new TsmOrgReq();
                tsmOrgReq.setParentOrgIdList(schoolIdList);
                List<TsmOrg> gradeList = orgService.selectList(tsmOrgReq);
                List<Long> gradeIdList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(gradeList)) {
                    gradeIdList = gradeList.stream().map(TsmOrg::getOrgId).collect(Collectors.toList());
                }
                // 班级
                Map<Long, List<TsmOrg>> classByParentMap = new HashMap<>();
                if (!CollectionUtils.isEmpty(gradeIdList)) {
                    tsmOrgReq.setParentOrgIdList(gradeIdList);
                    List<TsmOrg> classList = orgService.selectList(tsmOrgReq);
                    if (!CollectionUtils.isEmpty(classList)) {
                        classByParentMap = classList.stream().collect(Collectors.groupingBy(TsmOrg::getParentOrgId));
                    }
                }
                Map<Long, List<TsmOrg>> finalClassByParentMap = classByParentMap;
                importExcelList.stream().forEach(item -> {
                    StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                    // 根据学校ID获取年级和班级
                    BigInteger schoolId = item.getSchoolId();
                    List<TsmOrg> gradeInfoBySchoolList = gradeList.stream().filter(grade -> grade != null
                            && grade.getParentOrgId() != null && String.valueOf(grade.getParentOrgId()).equals(String.valueOf(schoolId))
                    ).collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(gradeInfoBySchoolList)) {
                        errorMessage.append("【负责年级/班级】在组织管理中未维护 ");
                        item.setVerifyStatus(IfFlagEnum.FALSE.code());
                        item.setErrorMessage(errorMessage.toString());
                    } else {
                        // 处理年级班级：二年级:一班,一年级:一班/二班
                        String gradeClassStr = item.getGradeClassStr();
                        String[] gradeClassItemArray = gradeClassStr.split(",");
                        List<TeacherBindingClassCreateReq> teacherBindingClassCreateList = new ArrayList<>();
                        Boolean ifFlag = true;
                        for (String gradeClassItem : gradeClassItemArray) {
                            TeacherBindingClassCreateReq gradeClassCreateItem = new TeacherBindingClassCreateReq();
                            gradeClassCreateItem.setSchoolId(schoolId);
                            gradeClassCreateItem.setSchoolName(item.getSchoolName());
                            gradeClassCreateItem.setIfAll(IfFlagEnum.FALSE.code());
                            gradeClassCreateItem.setDelFlag(DelFlagEnum.UN_DELETE.code());
                            if (currWorker != null) {
                                gradeClassCreateItem.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                            }
                            gradeClassCreateItem.setCreatedTime(createDate);
                            // 一年级:一班/二班
                            String[] gradeClass = gradeClassItem.split(":");
                            String gradeName = gradeClass[0];
                            gradeClassCreateItem.setGradeName(gradeName);
                            // 获取年级
                            List<TsmOrg> gradeInfoList = gradeInfoBySchoolList.stream().filter(vo -> vo != null && vo.getOrgName() != null && vo.getOrgName().equals(gradeName)).collect(Collectors.toList());
                            if (CollectionUtils.isEmpty(gradeInfoList)) {
                                ifFlag = false;
                                return;
                            }
                            TsmOrg gradeInfo = gradeInfoList.get(0);
                            Long gradeId = gradeInfo.getOrgId();
                            gradeClassCreateItem.setGradeId(BigInteger.valueOf(gradeId));
                            // 班级
                            List<TsmOrg> classList = finalClassByParentMap.get(gradeId);
                            if (CollectionUtils.isEmpty(classList)) {
                                ifFlag = false;
                                return;
                            }
                            Map<String, List<TsmOrg>> classInfoByNameMap = classList.stream().collect(Collectors.groupingBy(TsmOrg::getOrgName));
                            String classNameStr = gradeClass[1];
                            String[] classNameArray = classNameStr.split("/");
                            for (String className : classNameArray) {
                                TeacherBindingClassCreateReq teacherBindingClassCreateReq = BeanCopyUtil.convert(gradeClassCreateItem, TeacherBindingClassCreateReq.class);
                                teacherBindingClassCreateReq.setClassName(className);
                                // 获取班级
                                List<TsmOrg> classInfoList = classInfoByNameMap.get(className);
                                if (CollectionUtils.isEmpty(classInfoList)) {
                                    ifFlag = false;
                                    return;
                                }
                                TsmOrg classInfo = classInfoList.get(0);
                                Long classId = classInfo.getOrgId();
                                teacherBindingClassCreateReq.setClassId(BigInteger.valueOf(classId));
                                teacherBindingClassCreateReq.setGroupId(gradeId + ":" + classId);
                                teacherBindingClassCreateList.add(teacherBindingClassCreateReq);
                            }
                            if (!ifFlag) {
                                return;
                            }
                        }
                        if (!ifFlag) {
                            errorMessage.append("【负责年级/班级】在组织管理中未维护 ");
                            item.setVerifyStatus(IfFlagEnum.FALSE.code());
                            item.setErrorMessage(errorMessage.toString());
                        } else {
                            item.setTeacherBindingClassCreateList(teacherBindingClassCreateList);
                        }
                    }
                });
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            }
            // 身份证验重：新增数据中身份证重复 / 手机号验重：新增数据中手机号重复
            if (ifSuccess) {
                // 新增数据中的重复身份证
                List<String> repeatIdentityCardNoList = importExcelList.stream().map(TeacherInfoImportExcel::getIdentityCardNo)
                        .collect(Collectors.toMap(e -> e, e -> 1, (a, b) -> a + b))
                        .entrySet().stream().filter(entry -> entry.getValue() > 1)
                        .map(entry -> entry.getKey()).collect(Collectors.toList());
                // 新增数据中的重复手机号
                List<String> repeatMobileList = importExcelList.stream().map(TeacherInfoImportExcel::getMobile)
                        .collect(Collectors.toMap(e -> e, e -> 1, (a, b) -> a + b))
                        .entrySet().stream().filter(entry -> entry.getValue() > 1)
                        .map(entry -> entry.getKey()).collect(Collectors.toList());
                // 数据库中的重复身份证
                List<String> identityCardNoList = importExcelList.stream().map(TeacherInfoImportExcel::getIdentityCardNo).distinct().collect(Collectors.toList());
                TeacherReq teacherReq = new TeacherReq();
                teacherReq.setIdentityCardNoList(identityCardNoList);
                List<TeacherInfoVo> teacherInfoVoListByIdentityCardNo = teacherService.selectTeacherInfoList(teacherReq);
                List<String> existIdentityCardNoList = teacherInfoVoListByIdentityCardNo.stream().map(TeacherInfoVo::getIdentityCardNo).collect(Collectors.toList());
                // 数据库中的重复手机号
                List<String> mobileList = importExcelList.stream().map(TeacherInfoImportExcel::getMobile).distinct().collect(Collectors.toList());
                teacherReq.setIdentityCardNoList(null);
                teacherReq.setMobileList(mobileList);
                List<TeacherInfoVo> teacherInfoVoListByMobile = teacherService.selectTeacherInfoList(teacherReq);
                List<String> existMobileList = teacherInfoVoListByMobile.stream().map(TeacherInfoVo::getMobile).collect(Collectors.toList());
                if (
                        !CollectionUtils.isEmpty(repeatIdentityCardNoList) || !CollectionUtils.isEmpty(repeatMobileList) ||
                                !CollectionUtils.isEmpty(existIdentityCardNoList) || !CollectionUtils.isEmpty(existMobileList)

                ) {
                    importExcelList.stream().forEach(item -> {
                        StringBuilder errorMessage = item.getErrorMessage() != null ? new StringBuilder(item.getErrorMessage()) : new StringBuilder();
                        if (!CollectionUtils.isEmpty(repeatIdentityCardNoList) && repeatIdentityCardNoList.contains(item.getIdentityCardNo())) {
                            errorMessage.append("【身份证号】出现重复数据 ");
                        }
                        if (!CollectionUtils.isEmpty(repeatMobileList) && repeatMobileList.contains(item.getMobile())) {
                            errorMessage.append("【手机号】出现重复数据 ");
                        }
                        if (!CollectionUtils.isEmpty(existIdentityCardNoList) && existIdentityCardNoList.contains(item.getIdentityCardNo())) {
                            errorMessage.append("【身份证号】在教师管理中已维护 ");
                        }
                        if (!CollectionUtils.isEmpty(existMobileList) && existMobileList.contains(item.getMobile())) {
                            errorMessage.append("【手机号】在教师管理中已维护 ");
                        }
                        // 是否检验通过
                        if (!StringUtil.isEmpty(errorMessage.toString().trim())) {
                            item.setVerifyStatus(IfFlagEnum.FALSE.code());
                            item.setErrorMessage(errorMessage.toString());
                        } else {
                            item.setVerifyStatus(IfFlagEnum.TRUE.code());
                        }
                    });
                }
                // 判断是否校验通过
                count = importExcelList.stream().filter(item -> item != null && item.getVerifyStatus() != null && item.getVerifyStatus().intValue() == IfFlagEnum.FALSE.code().intValue()).count();
                if (count > 0) {
                    // 导出失败原因
                    ifSuccess = false;
                }
            }
            // 校验失败
            if (!ifSuccess) {
                filename = filename.replace(".xlsx", "").replace(".xls", "").replace(".csv", "");
                String fileName = filename + "校验失败结果";
                export(response, fileName, BeanCopyUtil.convert(importExcelList, TeacherInfoExportExcel.class));
            } else {
                // 批量新建教师数据
                List<TeacherInfoCreateReq> teacherInfoCreateList = BeanCopyUtil.convert(importExcelList, TeacherInfoCreateReq.class);
                teacherInfoCreateList.stream().forEach(item -> {
                    String teacherNum = CharacterCodeUtil.generateCharacterCode(OneAffairConstants.TEACHER_NUM_PREFIX);
                    item.setTeacherNum(teacherNum);
                    item.setCreatedTime(createDate);
                    item.setDelFlag(DelFlagEnum.UN_DELETE.code());
                    if (currWorker != null) {
                        item.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                        item.setCreatedName(currWorker.getWorkerName());
                    }
                    item.setRoleId(UseRoleEnum.TEACHER.code());
                    item.setRoleName(UseRoleEnum.TEACHER.getName());
                });
                List<YcqYjsTeacher> ycqYjsTeachers = teacherService.insertBatch(teacherInfoCreateList);
                Map<String, List<YcqYjsTeacher>> teacherMap = ycqYjsTeachers.stream().collect(Collectors.groupingBy(YcqYjsTeacher::getIdentityCardNo));
                List<TeacherBindingClassCreateReq> teacherBindingClassCreateList = new ArrayList<>();
                importExcelList.stream().forEach(item -> {
                    List<TeacherBindingClassCreateReq> teacherBindingClassList = item.getTeacherBindingClassCreateList();
                    List<YcqYjsTeacher> teacherList = teacherMap.get(item.getIdentityCardNo());
                    if (!CollectionUtils.isEmpty(teacherList)) {
                        YcqYjsTeacher teacher = teacherList.get(0);
                        item.setTeacherId(teacher.getId());
                        item.setTeacherNum(teacher.getTeacherNum());
                        if (!CollectionUtils.isEmpty(teacherBindingClassList)) {
                            teacherBindingClassList.stream().forEach(vo -> {
                                vo.setTeacherId(teacher.getId());
                            });
                            teacherBindingClassCreateList.addAll(teacherBindingClassList);
                            item.setTeacherBindingClassCreateList(teacherBindingClassList);
                        }
                    }
                });
                if (!CollectionUtils.isEmpty(ycqYjsTeachers)) {
                    teacherBindingClassService.insertBatch(teacherBindingClassCreateList);
                }
                // 任务入库
                ImportRecordCreateReq createReq = new ImportRecordCreateReq();
                createReq.setCreatedTime(createDate);
                createReq.setDelFlag(DelFlagEnum.UN_DELETE.code());
                if (currWorker != null) {
                    createReq.setCreatedId(BigInteger.valueOf(currWorker.getLoginId()));
                    createReq.setCreatedName(currWorker.getWorkerName());
                }
                createReq.setErrorSize(0);
                createReq.setSuccessSize(0);
                createReq.setTotalSize(importExcelList.size());
                createReq.setFileName(filename);
                createReq.setCompleteStatus(TaskCompleteStatusEnum.UN_COMPLETE.code());
                createReq.setOperateType(TaskOperateTypeEnum.TASK_TYPE_1.code());
                createReq.setDataJson(JSON.toJSONString(importExcelList));
                importRecordService.insertOne(createReq);
            }
            return ifSuccess;
        } catch (BusinessException e) {
            log.error("TeacherManager.importTeacherInfo.error:" + e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("TeacherManager.importTeacherInfo.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export(HttpServletResponse response, String fileName, List<TeacherInfoExportExcel> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=utf-8''" + encodedFileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), TeacherInfoExportExcel.class)
                    .sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy())
                    .doWrite(dataList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("TeacherManager.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 自定义Excel导出策略，设置表头和数据行的字体和高度
     *
     * @return Excel导出策略
     */
    private HorizontalCellStyleStrategy getHeightAndFontStrategy() {
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 11);
        headWriteFont.setBold(true);
        headWriteCellStyle.setWriteFont(headWriteFont);
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
        WriteFont contentWriteFont = new WriteFont();
        contentWriteFont.setFontHeightInPoints((short) 11);
        contentWriteCellStyle.setWriteFont(contentWriteFont);
        contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
    }

}
