package com.crunii.micro.service.oneaffair.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentCourse;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourse.*;

import java.math.BigInteger;
import java.util.List;

/**
 * 学生课程表(YCQ_YJS_STUDENT_COURSE)表服务接口
 */
public interface StudentCourseService {

    /**
     * 根据ID查询学生课程
     *
     * @param id
     * @return
     */
    YcqYjsStudentCourse selectOneById(BigInteger id);

    /**
     * 根据ID列表查询学生课程
     *
     * @param idList
     * @return
     */
    List<YcqYjsStudentCourse> selectListByIds(List<BigInteger> idList);

    /**
     * 分页查询学生课程数据
     *
     * @param req
     * @return
     */
    PageRsp<YcqYjsStudentCourse> pageSearchStudentCourseInfo(PageReq<StudentCourseReq> req);

    /**
     * 列表查询学生课程数据
     *
     * @param req
     * @return
     */
    List<YcqYjsStudentCourse> selectStudentCourseInfoList(StudentCourseReq req);

    /**
     * 统计学生课程时长
     *
     * @param req
     * @return
     */
    Integer sumCourseDuration(StudentCourseReq req);

    /**
     * 单个新增学生课程数据
     *
     * @param createReq
     * @return
     */
    YcqYjsStudentCourse insertOne(StudentCourseCreateReq createReq);

    /**
     * 批量新增学生课程数据
     *
     * @param createReqList
     * @return
     */
    List<YcqYjsStudentCourse> insertBatch(List<StudentCourseCreateReq> createReqList);

    /**
     * 更新学生课程的上传视频
     * @param req
     */
    void updateStudentCourseVideo(StudentCourseUploadVideoVo req);

    /**
     * 更新学生课程视频观看进度
     * @param req
     */
    void studentCourseVideoViewProgress(StudentCourseVideoViewProgressReq req);

    /**
     * 学生课程提交试卷
     * @param req
     */
    void studentCourseSubmitTest(StudentCourseSubmitTestReq req);

    /**
     * 根据用户ID、试卷id修改考生考试记录
     *
     * @param req
     * @return
     */
    Result<Object> updateStudentCourseById(StudentCourseReq req);

}
