package com.crunii.micro.service.oneaffair.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentTest;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.StudentTestCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.StudentTestDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.StudentTestReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.StudentTestUpdateReq;

import java.math.BigInteger;
import java.util.List;

/**
 * 学生答卷表(YCQ_YJS_STUDENT_TEST)表服务接口
 */
public interface StudentTestService {

    /**
     * 根据ID查询学生答卷数据
     *
     * @param id
     * @return
     */
    YcqYjsStudentTest selectOneById(BigInteger id);

    /**
     * 根据ID列表查询学生答卷数据
     *
     * @param idList
     * @return
     */
    List<YcqYjsStudentTest> selectListByIds(List<BigInteger> idList);

    /**
     * 列表查询学生答卷数据
     *
     * @param req
     * @return
     */
    List<YcqYjsStudentTest> selectList(StudentTestReq req);

    /**
     * 查询最高分数学生答卷数据
     *
     * @param req
     * @return
     */
    YcqYjsStudentTest selectMaxStudentTest(StudentTestReq req);

    /**
     * 分页查询学生答卷
     * @param req
     * @return
     */
    PageRsp<YcqYjsStudentTest> pageSearchStudentTest(PageReq<StudentTestReq> req);

    /**
     * 统计学生答卷数量
     *
     * @param req
     * @return
     */
    Integer count(StudentTestReq req);

    /**
     * 单个新增学生答卷数据
     *
     * @param createReq
     * @return
     */
    YcqYjsStudentTest insertOne(StudentTestCreateReq createReq);

    /**
     * 批量新增学生答卷数据
     *
     * @param createReqList
     * @return
     */
    List<YcqYjsStudentTest> insertBatch(List<StudentTestCreateReq> createReqList);

    /**
     * 批量删除学生答卷数据
     *
     * @param deleteReq
     */
    void deleteBatch(StudentTestDeleteReq deleteReq);

    /**
     * 单个修改学生答卷数据
     *
     * @param updateReq
     * @return
     */
    YcqYjsStudentTest updateOne(StudentTestUpdateReq updateReq);

}
