package com.crunii.micro.service.oneaffair.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsTeacher;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.*;

import java.math.BigInteger;
import java.util.List;

/**
 * 教师表(YCQ_YJS_TEACHER)表服务接口
 */
public interface TeacherService {

    /**
     * 根据身份证查询教师数据
     * @param identityCardNo 身份证
     * @return
     */
    List<YcqYjsTeacher> selectListByIdentityCardNo(String identityCardNo);

    /**
     * 根据手机号查询教师数据
     * @param mobile 手机号
     * @return
     */
    List<YcqYjsTeacher> selectListByMobile(String mobile);

    /**
     * 根据ID查询教师数据
     * @param id
     * @return
     */
    YcqYjsTeacher selectOneById(BigInteger id);

    /**
     * 根据ID列表查询教师数据
     * @param idList
     * @return
     */
    List<YcqYjsTeacher> selectListByIds(List<BigInteger> idList);

    /**
     * 分页查询教师数据
     * @param req
     * @return
     */
    PageRsp<TeacherInfoVo> pageSearchTeacherInfo(PageReq<TeacherReq> req);

    /**
     * 列表查询教师数据
     * @param req
     * @return
     */
    List<TeacherInfoVo> selectTeacherInfoList(TeacherReq req);

    /**
     * 单个新增教师数据
     * @param createReq
     * @return
     */
    YcqYjsTeacher insertOne(TeacherInfoCreateReq createReq);

    /**
     * 批量新增教师数据
     * @param createReqList
     * @return
     */
    List<YcqYjsTeacher> insertBatch(List<TeacherInfoCreateReq> createReqList);

    /**
     * 单个修改教师数据
     * @param updateReq
     */
    void updateOne(TeacherInfoUpdateReq updateReq);

    /**
     * 批量删除教师数据
     * @param deleteReq
     */
    void deleteBatch(TeacherInfoDeleteReq deleteReq);

}
