package com.crunii.micro.service.oneaffair.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.oneaffair.commo.base.ExamplePredicate;
import com.crunii.micro.service.oneaffair.commo.base.FieldRestriction;
import com.crunii.micro.service.oneaffair.commo.base.ValueInPredicate;
import com.crunii.micro.service.oneaffair.constants.OneAffairConstants;
import com.crunii.micro.service.oneaffair.dto.oneaffair.*;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveChangeReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveSelectReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveUpdateReq;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveVo;
import com.crunii.micro.service.oneaffair.dto.vo.honor.HonorReq;
import com.crunii.micro.service.oneaffair.dto.vo.honor.HonorVo;
import com.crunii.micro.service.oneaffair.dto.vo.honor.MthMatchVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.StudentMedalInfo;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.StudentMedalInfoReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.StudentPositionReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.StudentPositionUpdateReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.enums.IfFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.*;
import com.crunii.micro.service.oneaffair.manager.StudentMedalManager;
import com.crunii.micro.service.oneaffair.service.ArchiveService;
import com.crunii.micro.service.oneaffair.service.GoodStudentService;
import com.crunii.micro.service.oneaffair.service.StudentHonoraryService;
import com.crunii.micro.service.oneaffair.service.StudentPositionService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ArchiveServiceImpl implements ArchiveService {
    @Resource
    private YcqYjsArchiveJdbcDao jdbcDao;
    @Resource
    private YcqYjsArchiveRepo archiveRepo;
    @Autowired
    private GoodStudentService goodStudentService;
    @Resource
    private YcqYjsStudentRepo ycqYjsStudentRepo;
    @Autowired
    private StudentHonoraryService studentHonoraryService;
    @Autowired
    private StudentPositionService studentPositionService;
    @Autowired
    private YcqYjsMedalActivityRepo ycqYjsMedalActivityRepo;
    @Autowired
    private StudentMedalManager studentMedalManager;
    @Autowired
    private YcqYjsMedalRepo ycqYjsMedalRepo;

    @Override
    public void insert(ArchiveChangeReq archiveChangeReq, Worker worker){
        YcqYjsArchive ycqYjsArchive = new YcqYjsArchive();
        BeanUtil.copyProperties(archiveChangeReq, ycqYjsArchive);
        ycqYjsArchive.setDelFlag(0);
        Date date = new Date();
        ycqYjsArchive.setCreateTime(date);
        ycqYjsArchive.setLastUpdatedTime(date);
        String userId="001";//获取操作人的id
        if (worker != null) {
            ycqYjsArchive.setCreatedId(worker.getLoginId()+"");
            ycqYjsArchive.setLastUpdatedId(worker.getLoginId()+"");
        }
        ycqYjsArchive.setId(generateRandomId(16));
        archiveRepo.save(ycqYjsArchive);
    }

    @Override
    public void insertBatch(List<ArchiveChangeReq> archiveChangeReqList, Worker worker){
        if(CollectionUtils.isNotEmpty(archiveChangeReqList)){
            Date date = new Date();
            List<YcqYjsArchive> ycqYjsArchiveList = BeanCopyUtil.convert(archiveChangeReqList, YcqYjsArchive.class);
            ycqYjsArchiveList.stream().forEach(item -> {
                item.setId(generateRandomId(16));
                item.setDelFlag(0);
                item.setCreateTime(date);
                if (worker != null) {
                    item.setCreatedId(worker.getLoginId()+"");
                }
            });
            if(CollectionUtils.isNotEmpty(ycqYjsArchiveList)){
                archiveRepo.saveAll(ycqYjsArchiveList);
            }
        }
    }

    @Override
    public boolean update(ArchiveChangeReq req, Worker worker){
        YcqYjsArchive ycqYjsArchive = new YcqYjsArchive();
        ArchiveSelectReq archiveSelectReq = new ArchiveSelectReq();
        BeanUtil.copyProperties(req, ycqYjsArchive);
        archiveSelectReq.setId(req.getId());
        if (getList(archiveSelectReq).size()==0){
            return false;
        }
        ycqYjsArchive.setDelFlag(0);
        ycqYjsArchive.setLastUpdatedTime(new Date());
        if (worker != null) {
            ycqYjsArchive.setLastUpdatedId(worker.getLoginId()+"");
        }

        return jdbcDao.updateArchive(ycqYjsArchive);
    }

    private String generateRandomId(int length) {
        Random random = new Random();
        StringBuilder sb = new StringBuilder(length);

        for (int i = 0; i < length; i++) {
            int digit = random.nextInt(10); // 生成0-9之间的随机数字
            sb.append(digit);
        }

        return sb.toString();
    }

    @Override
    public boolean delete(String id, Worker worker){
        YcqYjsArchive ycqYjsArchive = new YcqYjsArchive();
        ycqYjsArchive.setDelFlag(1);
        ycqYjsArchive.setId(id);
        ycqYjsArchive.setLastUpdatedTime(new Date());
        if (worker != null) {
            ycqYjsArchive.setLastUpdatedId(worker.getLoginId()+"");
        }
        return jdbcDao.updateArchive(ycqYjsArchive);
    }

    /**
     * 批量删除
     * @param ids
     * @param worker
     */
    @Override
    public void deleteBatch(List<String> ids, Worker worker) {
        if(CollectionUtils.isNotEmpty(ids)){
            Date nowDate = new Date();
            List<YcqYjsArchive> deleteEntityList = archiveRepo.findAllById(ids);
            if (!org.springframework.util.CollectionUtils.isEmpty(deleteEntityList)) {
                deleteEntityList.stream().forEach(item -> {
                    item.setDelFlag(DelFlagEnum.DELETED.code());
                    item.setLastUpdatedTime(nowDate);
                    if (worker != null) {
                        item.setLastUpdatedId(worker.getLoginId()+"");
                    }
                });
                archiveRepo.saveAllAndFlush(deleteEntityList);
            }
        }
    }

    @Override
    public void updateBatch(List<ArchiveUpdateReq> updateReqList) {
        if(!org.springframework.util.CollectionUtils.isEmpty(updateReqList)){
            List<String> ids = updateReqList.stream().map(ArchiveUpdateReq::getId).collect(Collectors.toList());
            List<YcqYjsArchive> updateEntityList = archiveRepo.findAllById(ids);
            if(!CollectionUtils.isEmpty(updateEntityList)){
                Map<String, ArchiveUpdateReq> updateMap = updateReqList.stream().collect(Collectors.toMap(ArchiveUpdateReq::getId, item -> item));
                updateEntityList.stream().forEach(item->{
                    ArchiveUpdateReq updateReq = updateMap.get(item.getId());
                    if(updateReq != null){
                        BeanUtils.copyProperties(updateReq,item);
                    }
                });
                archiveRepo.saveAllAndFlush(updateEntityList);
            }
        }
    }

    @Override
    public List<ArchiveVo> getList(ArchiveSelectReq req) {
        List<ArchiveVo> archiveVos = jdbcDao.selectList(req);
        return archiveVos;
    }

    @Override
    public PageRsp<ArchiveVo> pageSearchArchiveInfo(PageReq<ArchiveSelectReq> req) {
        PageRsp<ArchiveVo> archiveVoPageRsp = jdbcDao.pageSearchStudentInfo(req);
        return archiveVoPageRsp;
    }

    @Override
    public PageRsp<HonorVo> pageSearchMyHonor(PageReq<HonorReq> req) {
        PageRsp<HonorVo> pageRsp = new PageRsp<>();

        // 1.荣誉称号
        if (OneAffairConstants.HONOR_TYPE_TITLE == req.getQuery().getHonorType()){
            StudentHonoraryReq studentHonoraryReq = new StudentHonoraryReq();
            studentHonoraryReq.setStudentId(req.getQuery().getStudentId());
            List<YcqYjsStudentHonorary> ycqYjsStudentHonoraries = studentHonoraryService.selectList(studentHonoraryReq);
            ycqYjsStudentHonoraries = null == ycqYjsStudentHonoraries ? new ArrayList<>() : ycqYjsStudentHonoraries;

            pageRsp.setPageNo(req.getPageNo());
            pageRsp.setPageSize(req.getPageSize());
            List<HonorVo> honorVos = ycqYjsStudentHonoraries.parallelStream().map(x->{
                HonorVo honorVo = new HonorVo();
                BeanUtil.copyProperties(x, honorVo);
                honorVo.setCreateTime(x.getCreatedTime());
                return honorVo;
            }).collect(Collectors.toList());
            pageRsp.setResult(honorVos);
            pageRsp.setTotalNum(honorVos.size());

        // 2.赛事活动
        }else if (OneAffairConstants.HONOR_TYPE_MATH == req.getQuery().getHonorType()){
            PageReq<ArchiveSelectReq> archiveSelectReqPageReq = new PageReq<>();
            archiveSelectReqPageReq.setPageNo(req.getPageNo());
            archiveSelectReqPageReq.setPageSize(req.getPageSize());

            ArchiveSelectReq archiveSelectReq = new ArchiveSelectReq();
            archiveSelectReq.setStudentId(req.getQuery().getStudentId() + "");
            archiveSelectReq.setActivityType(OneAffairConstants.ACTIVITY_TYPE_MATCH + "");
            archiveSelectReq.setAppCode(req.getQuery().getHonorTitle());
            archiveSelectReq.setParticipateStartTime(req.getQuery().getStartCreatedTime());
            archiveSelectReq.setParticipateEndTime(req.getQuery().getEndCreatedTime());
            archiveSelectReq.setEventId(req.getQuery().getHonorTitle());        // 具体赛事活动
            archiveSelectReqPageReq.setQuery(archiveSelectReq);

            PageRsp<ArchiveVo> archiveVoPageRsp = pageSearchArchiveInfo(archiveSelectReqPageReq);
            BeanUtil.copyProperties(archiveVoPageRsp, pageRsp);
            List<HonorVo> honorVos = archiveVoPageRsp.getResult().parallelStream().map(x->{
                HonorVo honorVo = new HonorVo();
                BeanUtil.copyProperties(x, honorVo);
                honorVo.setCreateTime(x.getParticipateTime());
                return honorVo;
            }).collect(Collectors.toList());
            pageRsp.setResult(honorVos);
            pageRsp.setTotalNum(honorVos.size());

        // 3.荣誉职务
        }else if (OneAffairConstants.HONOR_TYPE_POST == req.getQuery().getHonorType()){
            StudentPositionReq studentPositionReq = new StudentPositionReq();
            studentPositionReq.setStudentId(req.getQuery().getStudentId());
            List<YcqYjsStudentPosition> ycqYjsStudentPositions = studentPositionService.selectList(studentPositionReq);
            ycqYjsStudentPositions = null == ycqYjsStudentPositions ? new ArrayList<>() : ycqYjsStudentPositions;

            pageRsp.setPageNo(req.getPageNo());
            pageRsp.setPageSize(req.getPageSize());
            List<HonorVo> honorVos = ycqYjsStudentPositions.parallelStream().map(x->{
                HonorVo honorVo = new HonorVo();
                BeanUtil.copyProperties(x, honorVo);
                honorVo.setCreateTime(x.getCreatedTime());
                return honorVo;
            }).collect(Collectors.toList());
            pageRsp.setResult(honorVos);
            pageRsp.setTotalNum(honorVos.size());
        }

        pageRsp.setResult(null == pageRsp.getResult() ? new ArrayList<>() : pageRsp.getResult());
        return pageRsp;
    }

    @Override
    public PageRsp<HonorVo> pageSearchMyAcceptMthMatchInfo(PageReq<HonorReq> req) {
        PageRsp<HonorVo> pageRsp = new PageRsp<>();
        PageReq<ArchiveSelectReq> archiveSelectReqPageReq = new PageReq<>();
        archiveSelectReqPageReq.setPageNo(req.getPageNo());
        archiveSelectReqPageReq.setPageSize(req.getPageSize());

        ArchiveSelectReq archiveSelectReq = new ArchiveSelectReq();
        archiveSelectReq.setStudentId(req.getQuery().getStudentId() + "");
        archiveSelectReq.setActivityType(OneAffairConstants.ACTIVITY_TYPE_MATCH + "");  // 活动类型：赛事活动
        archiveSelectReq.setParticipateStartTime(req.getQuery().getStartCreatedTime());
        archiveSelectReq.setParticipateEndTime(req.getQuery().getEndCreatedTime());
        archiveSelectReqPageReq.setQuery(archiveSelectReq);

        PageRsp<ArchiveVo> archiveVoPageRsp = pageSearchArchiveInfo(archiveSelectReqPageReq);
        BeanUtil.copyProperties(archiveVoPageRsp, pageRsp);
        List<HonorVo> honorVos = archiveVoPageRsp.getResult().parallelStream().map(x->{
            HonorVo honorVo = new HonorVo();
            BeanUtil.copyProperties(x, honorVo);
            return honorVo;
        }).collect(Collectors.toList());
        pageRsp.setResult(honorVos);

        return pageRsp;
    }

    @Override
    public MthMatchVo getMthMatchInfo(Long eventId, Long studentId) {
        // 1.赛事
        MthMatchVo mthMatchVo = jdbcDao.getMthMatchInfo(eventId);

        // 2.赛事活动关联的奖章
        YcqYjsMedalActivity medalActivity = new YcqYjsMedalActivity();
        medalActivity.setActivityId(BigInteger.valueOf(eventId));
        medalActivity.setDelFlag(CommonConstants.LOG_FLAG_NO);
        List<YcqYjsMedalActivity> medalActivities = ycqYjsMedalActivityRepo.findAll(Example.of(medalActivity));
        if (CollectionUtils.isNotEmpty(medalActivities)){

            // 3.学生点亮的奖章
            StudentMedalInfoReq studentMedalInfoReq = new StudentMedalInfoReq();
            studentMedalInfoReq.setStudentId(BigInteger.valueOf(studentId));
            studentMedalInfoReq.setIfLight(IfFlagEnum.TRUE.code());
            List<StudentMedalInfo> studentMedalInfos = studentMedalManager.selectJoinMedalInfoList(studentMedalInfoReq);
            if (CollectionUtils.isNotEmpty(studentMedalInfos)){
                // 4.赛事活动的奖被学生点亮
                List<BigInteger> medalIds = studentMedalInfos.stream().map(x->x.getStudentMedalId()).collect(Collectors.toList());
                medalActivities = medalActivities.stream().filter(x->medalIds.contains(x.getMedalId())).collect(Collectors.toList());

                // 5.奖章信息
                if (CollectionUtils.isNotEmpty(medalActivities)){
                    List<FieldRestriction<YcqYjsMedal>> fieldRestrictions = new ArrayList<>();
                    fieldRestrictions.add(new ValueInPredicate<YcqYjsMedal, Long>("id", medalActivities.stream().map(x->Long.valueOf(x + "")).collect(Collectors.toList())));
                    mthMatchVo.setYcqYjsMedals(ycqYjsMedalRepo.findAll(ExamplePredicate.of(Example.of(new YcqYjsMedal()), fieldRestrictions)));

                }
            }
        }

        mthMatchVo.setYcqYjsMedals(null == mthMatchVo.getYcqYjsMedals() ? new ArrayList<>() : mthMatchVo.getYcqYjsMedals());
        return mthMatchVo;
    }

}
