package com.crunii.micro.service.oneaffair.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsEvaluateSystem;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsSystemOrgRel;
import com.crunii.micro.service.oneaffair.dto.vo.evaluate.*;
import com.crunii.micro.service.oneaffair.jdbc.*;
import com.crunii.micro.service.oneaffair.service.EvaluateSystemService;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;

@Service
public class EvaluateSystemServiceImpl implements EvaluateSystemService {

    @Resource
    private YcqYjsEvaluateSystemRepo ycqYjsEvaluateSystemRepo;

    @Resource
    private YcqYjsEvaluateSystemDao ycqYjsEvaluateSystemDao;

    @Resource
    private YcqAppConfigUrlDao ycqAppConfigUrlDao;

    @Resource
    private TsmOrgDao tsmOrgDao;

    @Resource
    private TsmOrgRepo orgRepo;

    @Resource
    private YcqYjsSystemOrgRelRepo ycqYjsSystemOrgRelRepo;


    @Override
    public PageRsp<EvaluateSystemVo> pageSearchSystemInfo(PageReq<EvaluateSystemReq> req) {
        return ycqYjsEvaluateSystemDao.pageSearchSystemInfo(req);
    }

    @Override
    public void createSystemInfo(EvaluateSystemCreateReq createReq) {
        YcqYjsEvaluateSystem ycqYjsEvaluateSystem = new YcqYjsEvaluateSystem();
        BeanUtil.copyProperties(createReq, ycqYjsEvaluateSystem);
        Date curr = new Date();
        ycqYjsEvaluateSystem.setDelFlag(0);
        ycqYjsEvaluateSystem.setCreateTime(curr);
        //ycqYjsEvaluateSystem.setCreateWorker(Objects.requireNonNull(CurrWorkerUtil.getCurrWorker()).getLoginId());
       YcqYjsEvaluateSystem save = ycqYjsEvaluateSystemRepo.save(ycqYjsEvaluateSystem);
       createYcqYjsSystemOrgRel(createReq.getTsmOrgVoList(),save.getId());
    }

    /**
     * 体系和机构绑定
     * @param tsmOrgVos
     * @param id
     */
    private void createYcqYjsSystemOrgRel(List<TsmOrgVo> tsmOrgVos,BigInteger id){
        List<YcqYjsSystemOrgRel> byProp = ycqYjsSystemOrgRelRepo.findByProp("systemId", id);
        if(!CollectionUtils.isEmpty(byProp)){
            ycqYjsSystemOrgRelRepo.deleteAll(byProp);
        }
        List<YcqYjsSystemOrgRel> ycqYjsSystemOrgRels = new ArrayList<>();
        if(!CollectionUtils.isEmpty(tsmOrgVos)){
            for(TsmOrgVo tsmOrgVo : tsmOrgVos){
                YcqYjsSystemOrgRel ycqYjsSystemOrgRel = new YcqYjsSystemOrgRel();
                ycqYjsSystemOrgRel.setSystemId(id);
                ycqYjsSystemOrgRel.setOrgName(tsmOrgVo.getOrgName());
                ycqYjsSystemOrgRel.setOrgId(new BigInteger(tsmOrgVo.getOrgId()));
                ycqYjsSystemOrgRels.add(ycqYjsSystemOrgRel);
            }
            ycqYjsSystemOrgRelRepo.saveAll(ycqYjsSystemOrgRels);
        }
    }
    @Override
    public void updateSystemInfo(EvaluateSystemUpdateReq updateReq) {
        YcqYjsEvaluateSystem ycqYjsEvaluateSystem = ycqYjsEvaluateSystemRepo.getById(updateReq.getId());
        if(Objects.isNull(ycqYjsEvaluateSystem)){
            throw new BusinessException("找不到该记录");
        }
        BeanUtil.copyProperties(updateReq, ycqYjsEvaluateSystem);
        Date curr = new Date();
        ycqYjsEvaluateSystem.setUpdateTime(curr);
        //ycqYjsEvaluateSystem.setUpdateWorker(Objects.requireNonNull(CurrWorkerUtil.getCurrWorker()).getLoginId());
        ycqYjsEvaluateSystemRepo.save(ycqYjsEvaluateSystem);
        createYcqYjsSystemOrgRel(updateReq.getTsmOrgVoList(),updateReq.getId());
    }

    @Override
    public void deleteSystemInfo(BigInteger id) {
        YcqYjsEvaluateSystem ycqYjsEvaluateSystem = ycqYjsEvaluateSystemRepo.getById(id);
        if(Objects.isNull(ycqYjsEvaluateSystem)){
            throw new BusinessException("找不到该记录");
        }
        ycqYjsEvaluateSystem.setDelFlag(1);
        ycqYjsEvaluateSystemRepo.save(ycqYjsEvaluateSystem);
    }

    @Override
    public List<AppConfigVo> searchAppConfig(String appName) {
        return ycqAppConfigUrlDao.searchAppConfig(appName);
    }

    @Override
    public List<TsmOrgVo> searchTsmOrg(BigInteger parentOrgId) {
        return tsmOrgDao.searchTsmOrg(parentOrgId);
    }

    @Override
    public TsmOrgVoChildrenVo searchTsmOrgAndSchool(BigInteger orgId) {
        List<TsmOrgVoChildrenVo> tsmOrg = tsmOrgDao.searchTsmOrgById(orgId);
        if (CollectionUtils.isEmpty(tsmOrg)) {
            throw new BusinessException("找不到该记录");
        }
        TsmOrgVoChildrenVo tsmOrgVoChildrenVo = tsmOrg.get(0);
        List<TsmOrgVo> list = tsmOrgDao.searchTsmOrg(orgId);
        tsmOrgVoChildrenVo.setChildrenList(list);
        return tsmOrgVoChildrenVo;
    }

    @Override
    public List<TsmOrgVo> getParentTsmOrg(BigInteger orgId) {
        TsmOrg child = orgRepo.findOrgById(orgId.toString());
        if (child!=null){
            TsmOrg parent = orgRepo.findOrgById(child.getParentOrgId().toString());
            TsmOrgVo tsmOrgVo = new TsmOrgVo();
            BeanUtil.copyProperties(parent, tsmOrgVo);
            return Collections.singletonList(tsmOrgVo);
        }
        return null;
    }

}
