package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsGoodStudent;
import com.crunii.micro.service.oneaffair.dto.vo.goodstudent.*;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsGoodStudentJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsGoodStudentRepo;
import com.crunii.micro.service.oneaffair.service.GoodStudentService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 优秀学生表(YCQ_YJS_GOOD_STUDENT)表服务实现类
 */
@Slf4j
@Service
public class GoodStudentServiceImpl implements GoodStudentService {

    @Resource
    private YcqYjsGoodStudentRepo goodStudentRepo;
    @Resource
    private YcqYjsGoodStudentJdbcDao goodStudentJdbcDao;

    /**
     * 单个新增优秀学生数据
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsGoodStudent insertOne(GoodStudentCreateReq createReq) {
        log.info("GoodStudentServiceImpl.insertGoodStudent.param：{}", createReq.toString());
        return goodStudentRepo.save(BeanCopyUtil.convert(createReq, YcqYjsGoodStudent.class));
    }

    /**
     * 单个修改优秀学生数据
     * @param updateReq
     */
    @Override
    public void updateOne(GoodStudentUpdateReq updateReq) {
        log.info("GoodStudentServiceImpl.updateOne.param：{}", updateReq.toString());
        // 校验优秀学生是否存在
        YcqYjsGoodStudent goodStudent = selectOneById(updateReq.getId());
        if (goodStudent != null) {
            BeanUtils.copyProperties(updateReq, goodStudent);
            goodStudentRepo.saveAndFlush(goodStudent);
        }
    }

    /**
     * 根据ID查询优秀学生数据
     * @param id 主键ID
     * @return
     */
    @Override
    public YcqYjsGoodStudent selectOneById(BigInteger id) {
        log.info("GoodStudentServiceImpl.selectOneById.param：{}", id);
        return goodStudentRepo.getOne(id);
    }

    /**
     * 批量删除优秀学生数据 - 逻辑删除
     * @param deleteReq
     */
    @Override
    public void deleteBatch(GoodStudentDeleteReq deleteReq) {
        log.info("GoodStudentServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsGoodStudent> goodStudentList = goodStudentRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(goodStudentList)) {
            goodStudentList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            goodStudentRepo.saveAllAndFlush(goodStudentList);
        }
    }

    /**
     * 批量修改优秀学生的展出状态
     * @param updateReq
     */
    @Override
    public void updateGoodStudentShowFlag(GoodStudentUpdateShowFlagReq updateReq) {
        log.info("GoodStudentServiceImpl.updateGoodStudentShowFlag.param：{}", updateReq.toString());
        List<YcqYjsGoodStudent> goodStudentList = goodStudentRepo.findAllById(updateReq.getIdList());
        if (!CollectionUtils.isEmpty(goodStudentList)) {
            goodStudentList.stream().forEach(item -> {
                BeanUtils.copyProperties(updateReq, item);
            });
            goodStudentRepo.saveAllAndFlush(goodStudentList);
        }
    }

    /**
     * 分页查询优秀学生数据
     * @param req
     * @return
     */
    @Override
    public PageRsp<GoodStudentVo> pageSearchGoodStudent(PageReq<GoodStudentReq> req) {
        log.info("GoodStudentServiceImpl.pageSearchGoodStudent.param：{}", req.toString());
        return goodStudentJdbcDao.pageSearchGoodStudent(req);
    }

    /**
     * 列表查询优秀学生数据
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsGoodStudent> selectList(GoodStudentReq req) {
        log.info("GoodStudentServiceImpl.selectList.param：{}", req.toString());
        return goodStudentJdbcDao.selectList(req);
    }

}
