package com.crunii.micro.service.oneaffair.service.impl;

import cn.hutool.core.date.DateUtil;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudent;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentPosition;
import com.crunii.micro.service.oneaffair.dto.vo.homePage.HomePageVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourse.StudentCourseReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourse.StudentCourseSumVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.StudentMedalReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.StudentPositionReq;
import com.crunii.micro.service.oneaffair.enums.MedalMarkEnum;
import com.crunii.micro.service.oneaffair.enums.MedalTypeEnum;
import com.crunii.micro.service.oneaffair.enums.PoliticalStatusTypeEnum;
import com.crunii.micro.service.oneaffair.jdbc.*;
import com.crunii.micro.service.oneaffair.manager.StudentCourseManager;
import com.crunii.micro.service.oneaffair.service.HomePageService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class HomePageServiceImpl implements HomePageService {

    @Resource
    private YcqYjsStudentRepo ycqYjsStudentRepo;

    @Resource
    private YcqYjsStudentPositionRepo ycqYjsStudentPositionRepo;

    @Resource
    private YcqYjsStudentHonoraryJdbcDao ycqYjsStudentHonoraryJdbcDao;

    @Resource
    private YcqYjsServiceDurationJdbcDao ycqYjsServiceDurationJdbcDao;

    @Resource
    private YcqYjsStudentMedalJdbcDao ycqYjsStudentMedalJdbcDao;

    @Resource
    private YcqYjsStudentCourseJdbcDao ycqYjsStudentCourseJdbcDao;

    @Resource
    private StudentCourseManager studentCourseManager;

    @Resource
    private MthRegisterJdbcDao mthRegisterJdbcDao;

    @Resource
    private YcqYjsStudentPositionJdbcDao ycqYjsStudentPositionJdbcDao;

    @Value("${affiliatedBusinessId:1715277690642243584}")
    private String affiliatedBusinessId;

    @Value("${specialType:10084001}")
    private String specialType;

    @Override
    public HomePageVo index(BigInteger studentId) throws Exception {
        HomePageVo homePageVo = new HomePageVo();
        YcqYjsStudent ycqYjsStudent = ycqYjsStudentRepo.getOne(studentId);
        homePageVo.setStudentId(studentId);
        homePageVo.setStudentName(ycqYjsStudent.getStudentName());
        homePageVo.setPoliticalStatusType(PoliticalStatusTypeEnum.getDesc(ycqYjsStudent.getPoliticalStatusType()));

        if(PoliticalStatusTypeEnum.THE_MASSES.getCode().equals(ycqYjsStudent.getPoliticalStatusType()) || PoliticalStatusTypeEnum.COMMUNIST_READY_PARTY_MEMBER.getCode().equals(ycqYjsStudent.getPoliticalStatusType())){
            homePageVo.setPoliticalAge(0);
        }else if(PoliticalStatusTypeEnum.YOUNG_PIONEERS.getCode().equals(ycqYjsStudent.getPoliticalStatusType())){
            homePageVo.setPoliticalAge(DateUtil.ageOfNow(ycqYjsStudent.getJoinYoungPioneersTime()));
        }else if(PoliticalStatusTypeEnum.YOUTH_LEAGUE.getCode().equals(ycqYjsStudent.getPoliticalStatusType())){
            homePageVo.setPoliticalAge(DateUtil.ageOfNow(ycqYjsStudent.getJoinYouthLeagueTime()));
        }else if(PoliticalStatusTypeEnum.COMMUNIST_PARTY_MEMBER.getCode().equals(ycqYjsStudent.getPoliticalStatusType())){
            homePageVo.setPoliticalAge(DateUtil.ageOfNow(ycqYjsStudent.getJoinCommunistPartyTime()));
        }

        //成长值
        homePageVo.setGrowthValue(calculatedGrowthWalue(studentId));

        //星级章
        StudentMedalReq req2 = new StudentMedalReq();
        req2.setStudentId(studentId);
        req2.setMedalType(MedalTypeEnum.STAR.getValue());
        req2.setIfLight(1);
        homePageVo.setStarLevel(ycqYjsStudentMedalJdbcDao.medalStarNow(req2));


        //学生岗位
        StudentPositionReq studentPositionReq = new StudentPositionReq();
        studentPositionReq.setStudentId(studentId);
        studentPositionReq.setGradeId(ycqYjsStudent.getCurrentGradeId());
        studentPositionReq.setSemesterType( ycqYjsStudent.getSemesterType());
        studentPositionReq.setClassId(ycqYjsStudent.getCurrentClassId());
        YcqYjsStudentPosition uniqueByProps = ycqYjsStudentPositionJdbcDao.nowStudentPosition(studentPositionReq);
        if(uniqueByProps != null){
            homePageVo.setStudentPositionClass(uniqueByProps.getStudentPositionClass()+uniqueByProps.getStudentPosition());
        }

        //荣誉称号累计
        StudentHonoraryReq req = new StudentHonoraryReq();
        req.setStudentId(studentId);
        Integer countHonoraryTitle = ycqYjsStudentHonoraryJdbcDao.count(req);
        homePageVo.setHonoraryTitleNow(countHonoraryTitle);

        //服务时长
        homePageVo.setServiceDurationTotal(ycqYjsServiceDurationJdbcDao.totalTime(studentId));;


        //学习时长
        StudentCourseReq studentCourseReq = new StudentCourseReq();
        studentCourseReq.setStudentId(studentId);
        studentCourseReq.setCompleteStatus(1);
        Integer i = ycqYjsStudentCourseJdbcDao.sumCourseDuration(studentCourseReq);
        //秒转化成小时
        BigDecimal bigDecimal = BigDecimal.valueOf(i == null ? 0 : i).divide(new BigDecimal("3600"), 2, BigDecimal.ROUND_DOWN);
        homePageVo.setStudyTotal(bigDecimal);


        //队前教育
        StudentCourseReq studentCourseReq1 = new StudentCourseReq();
        studentCourseReq1.setStudentId(studentId);
        studentCourseReq1.setAffiliatedBusinessId(new BigInteger(affiliatedBusinessId));
        studentCourseReq1.setSpecialType(specialType);
        StudentCourseSumVo studentCourseSumVo = callSumStudentCourse(studentCourseReq1);
        if(studentCourseSumVo != null){
             //队前教育->已完成
             homePageVo.setPreTeamEducationFinish(studentCourseSumVo.getCompleteCount());
            //队前教育->进行中
            homePageVo.setPreTeamEducationUnderway(studentCourseSumVo.getInStudyCount());
            //队前教育->未完成
            homePageVo.setPreTeamEducationUncompleted(studentCourseSumVo.getUnStudyCount());
        }



        //红领巾争章
        List<Map<String, Object>> mapList = ycqYjsStudentMedalJdbcDao.queryByStudent(studentId);
        if(!CollectionUtils.isEmpty(mapList)){
           for(Map<String,Object> map : mapList){
               //红领巾争章->基础章
               if(MedalTypeEnum.BASE.getValue().equals(map.get("medal_type"))){
                   homePageVo.setMedalBase(Integer.parseInt(map.get("count").toString()));
               //红领巾争章->星级章
               }else if(MedalTypeEnum.STAR.getValue().equals(map.get("medal_type"))){
                   homePageVo.setMedalStar(Integer.parseInt(map.get("count").toString()));
               //红领巾争章->特色章
               }else if(MedalTypeEnum.CHARACTERISTIC.getValue().equals(map.get("medal_type"))){
                   homePageVo.setMedalCharacteristic(Integer.parseInt(map.get("count").toString()));
               }
           }
        }


        //荣誉墙

        //荣誉墙->荣誉称号累计
        homePageVo.setHonoraryTitle(countHonoraryTitle);
        //荣誉墙->比赛获奖



        //我的实践
        //我的实践->志愿活动


        //我的实践->赛事活动
        homePageVo.setMyPracticeCompetition(mthRegisterJdbcDao.countMthRegister(studentId));;


        //我的实践->奖章数
        StudentMedalReq studentMedalReq2 = new StudentMedalReq();
        studentMedalReq2.setStudentId(studentId);
        studentMedalReq2.setIfLight(1);
        homePageVo.setMyPracticeMedal(ycqYjsStudentMedalJdbcDao.countByMedalType(studentMedalReq2));

        return homePageVo;
    }

    /**
     * 计算成长值
     * @param studentId
     * @return
     */
    private BigDecimal calculatedGrowthWalue(BigInteger studentId) throws Exception {
        BigDecimal total = new BigDecimal("0");
        //六知: 每个1分，满分6分
        //六会: 每个1分，满分6分。
        //一做: 完成即得5分
        StudentCourseReq studentCourseReq1 = new StudentCourseReq();
        studentCourseReq1.setStudentId(studentId);
        studentCourseReq1.setAffiliatedBusinessId(new BigInteger(affiliatedBusinessId));
        studentCourseReq1.setSpecialType(specialType);
        studentCourseReq1.setSpecialName("六知");
        StudentCourseSumVo studentCourseSumVo = callSumStudentCourse(studentCourseReq1);
        if(studentCourseSumVo != null && studentCourseSumVo.getCompleteCount() != null){
            if(studentCourseSumVo.getCompleteCount() >= 6){
                total = total.add(new BigDecimal("6"));
            }else{
                total = total.add(new BigDecimal(studentCourseSumVo.getCompleteCount()));
            }
        }

        studentCourseReq1.setSpecialName("六会");
        StudentCourseSumVo studentCourseSumVo2 = callSumStudentCourse(studentCourseReq1);
        if(studentCourseSumVo2 != null && studentCourseSumVo2.getCompleteCount() != null) {
            if (studentCourseSumVo2.getCompleteCount() >= 6) {
                total = total.add(new BigDecimal("6"));
            } else {
                total = total.add(new BigDecimal(studentCourseSumVo2.getCompleteCount()));
            }
        }

        studentCourseReq1.setSpecialName("一做");
        StudentCourseSumVo studentCourseSumVo3 = callSumStudentCourse(studentCourseReq1);
        if(studentCourseSumVo3 != null &&studentCourseSumVo3.getCompleteCount() != null) {
            if (studentCourseSumVo2.getCompleteCount() > 0) {
                total = total.add(new BigDecimal("5"));
            }
        }


        //基础章: 每个基础章计2分
        //特色章: 每个特色章计2分。
        //获得一次一星章计5分
        //获得一次二星章计10分
        //获得一次三星章计20分
        //获得一次四星章计35分
        //获得一次五星章计50分
        StudentMedalReq studentMedalReq = new StudentMedalReq();
        studentMedalReq.setStudentId(studentId);
        studentMedalReq.setIfLight(1);
        //基础章
        studentMedalReq.setMedalType(MedalTypeEnum.BASE.getValue());
        Integer base = ycqYjsStudentMedalJdbcDao.countByMedalType(studentMedalReq);
        if(base != null){
            total = total.add(new BigDecimal("2").multiply(new BigDecimal(base)));
        }
        //特色章
        studentMedalReq.setMedalType(MedalTypeEnum.CHARACTERISTIC.getValue());
        Integer special = ycqYjsStudentMedalJdbcDao.countByMedalType(studentMedalReq);
        if(special != null){
            total = total.add(new BigDecimal("2").multiply(new BigDecimal(special)));
        }
        //一星章
        studentMedalReq.setMedalType(MedalTypeEnum.CHARACTERISTIC.getValue());
        studentMedalReq.setMedalMark(MedalMarkEnum.ONE_STAR.getValueCode());
        Integer one = ycqYjsStudentMedalJdbcDao.countByMedalType(studentMedalReq);
        if(one != null){
            total = total.add(new BigDecimal("5").multiply(new BigDecimal(one)));
        }
        //二星章
        studentMedalReq.setMedalMark(MedalMarkEnum.ONE_STAR.getValueCode());
        Integer two = ycqYjsStudentMedalJdbcDao.countByMedalType(studentMedalReq);
        if(two != null){
            total = total.add(new BigDecimal("10").multiply(new BigDecimal(two)));
        }
        //三星章
        studentMedalReq.setMedalMark(MedalMarkEnum.ONE_STAR.getValueCode());
        Integer three = ycqYjsStudentMedalJdbcDao.countByMedalType(studentMedalReq);
        if(three != null){
            total = total.add(new BigDecimal("20").multiply(new BigDecimal(three)));
        }
        //四星章
        studentMedalReq.setMedalMark(MedalMarkEnum.ONE_STAR.getValueCode());
        Integer four = ycqYjsStudentMedalJdbcDao.countByMedalType(studentMedalReq);
        if(four != null){
            total = total.add(new BigDecimal("35").multiply(new BigDecimal(four)));
        }
        //五星章
        studentMedalReq.setMedalMark(MedalMarkEnum.ONE_STAR.getValueCode());
        Integer five = ycqYjsStudentMedalJdbcDao.countByMedalType(studentMedalReq);
        if(five != null){
            total = total.add(new BigDecimal("50").multiply(new BigDecimal(five)));
        }


        //志愿服务: 时长每2个小时计1分
        BigDecimal serviceDurationTotal = ycqYjsServiceDurationJdbcDao.totalTime(studentId);
        if(serviceDurationTotal != null){
            BigDecimal divide = serviceDurationTotal.divide(new BigDecimal("2"),0, BigDecimal.ROUND_HALF_UP);
            total = total.add(divide);
        }

        //获得校级荣誉每次计5分
        //获得区级荣誉每次计15分
        //获得市级荣誉每次计30分
        //获得全国级荣誉每次计50分
        //获得国际级荣誉每次计60分
        StudentHonoraryReq req = new StudentHonoraryReq();
        req.setStudentId(studentId);
        //校级
        req.setHonoraryLevelType("4");
        Integer shcool = ycqYjsStudentHonoraryJdbcDao.count(req);
        if(shcool != null){
            total = total.add(new BigDecimal("5").multiply(new BigDecimal(shcool)));
        }
        //区级
        req.setHonoraryLevelType("3");
        Integer area = ycqYjsStudentHonoraryJdbcDao.count(req);
        if(area != null){
            total = total.add(new BigDecimal("15").multiply(new BigDecimal(area)));
        }
        //市级
        req.setHonoraryLevelType("2");
        Integer city = ycqYjsStudentHonoraryJdbcDao.count(req);
        if(city != null){
            total = total.add(new BigDecimal("30").multiply(new BigDecimal(city)));
        }
        //全国
        req.setHonoraryLevelType("1");
        Integer country = ycqYjsStudentHonoraryJdbcDao.count(req);
        if(country != null){
            total = total.add(new BigDecimal("50").multiply(new BigDecimal(country)));
        }
        //国际   产品经理说没有国际级荣誉
      /*  req.setHonoraryLevelType("5");
        Integer internation = ycqYjsStudentHonoraryJdbcDao.count(req);
        if(internation != null){
            total = total.add(new BigDecimal("60").multiply(new BigDecimal(internation)));
        }*/


        //担任小队干部每学期计5分
        //担任中队干部每学期计10分
        //担任大队干部每学期计15分
        Map<String,Object> props = new HashMap<>();
        props.put("studentId", studentId);
        props.put("delFlag", 0);
        List<YcqYjsStudentPosition> uniqueByProps = ycqYjsStudentPositionRepo.findByProps(props);
        Map<String, List<YcqYjsStudentPosition>> studentPositionCollect = uniqueByProps.stream().collect(Collectors.groupingBy(YcqYjsStudentPosition::getStudentPositionClass));
        List<YcqYjsStudentPosition> small = studentPositionCollect.get("小队");
        List<YcqYjsStudentPosition> middle = studentPositionCollect.get("中队");
        List<YcqYjsStudentPosition> big = studentPositionCollect.get("大队");
        if(! CollectionUtils.isEmpty(small) ){
            total = total.add(new BigDecimal(small.size()).multiply(new BigDecimal("5")));
        }
        if(! CollectionUtils.isEmpty(middle) ){
            total = total.add(new BigDecimal(middle.size()).multiply(new BigDecimal("10")));
        }
        if(! CollectionUtils.isEmpty(big) ){
            total = total.add(new BigDecimal(big.size()).multiply(new BigDecimal("15")));
        }
        return total;
    }

    /**
     * 调用查询课程统计接口
     * @param studentCourseReq
     * @return
     */
    private StudentCourseSumVo callSumStudentCourse(StudentCourseReq studentCourseReq){
        StudentCourseSumVo studentCourseSumVo = null;
        try {
            studentCourseSumVo = studentCourseManager.sumStudentCourse(studentCourseReq);
        } catch (Exception e) {
            log.error("调用: studentCourseManager.sumStudentCourse(studentCourseReq) : " + e.getMessage());
            return null;
        }
        return studentCourseSumVo;
    }

}
