package com.crunii.micro.service.oneaffair.service.impl;

import cn.hutool.core.util.DesensitizedUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentBindingUser;
import com.crunii.micro.service.oneaffair.dto.vo.idengtitychoose.*;
import com.crunii.micro.service.oneaffair.dto.vo.studentBindingUser.StudentBindingUserReq;
import com.crunii.micro.service.oneaffair.enums.SexTypeEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentBindingUserJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentBindingUserRepo;
import com.crunii.micro.service.oneaffair.service.IdentityChooseService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class IdentityChooseServiceImpl implements IdentityChooseService {

    @Resource
    private YcqYjsStudentBindingUserJdbcDao ycqYjsStudentBindingUserJdbcDao;

    @Resource
    private YcqYjsStudentBindingUserRepo ycqYjsStudentBindingUserRepo;

    @Override
    public List<StudentInfoByUserVo> searchStudentInfoByUser(BigInteger userId) {
        List<StudentInfoByUserVo> studentInfoByUserVos = ycqYjsStudentBindingUserJdbcDao.searchStudentInfoByUser(userId);
        if(! CollectionUtils.isEmpty(studentInfoByUserVos)){
            for(StudentInfoByUserVo studentInfoByUserVo : studentInfoByUserVos){
                studentInfoByUserVo.setIdentityCardNo(DesensitizedUtil.idCardNum(studentInfoByUserVo.getIdentityCardNo(),4,4));
                studentInfoByUserVo.setSexStr(SexTypeEnum.getDesc(studentInfoByUserVo.getSex()));
            }
        }
        return studentInfoByUserVos;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public StudentInfoByUserVo bindStudent(BindStudentReq bindStudentReq) {
        StudentInfoByUserVo studentInfoByUserVo = new StudentInfoByUserVo();

        //查询该用户下已经绑定的学生信息
        List<StudentInfoByUserVo> studentInfoByUserVos = ycqYjsStudentBindingUserJdbcDao.searchStudentInfoByUser(bindStudentReq.getUserId());

        List<YcqYjsStudentBindingUser> studentBindingUserList = new ArrayList<>();

        for(String identityCardNo : bindStudentReq.getIdentityCardNos()){
            StudentInfoByIdentityCardNoVo studentInfoByIdentityCardNoVo = ycqYjsStudentBindingUserJdbcDao.searchStudentInfoByIdentityCardNo(identityCardNo);
            if(studentInfoByIdentityCardNoVo == null){
                throw new BusinessException("该学生不存在");
            }
            StudentBindingUserReq studentBindingUserReq = new StudentBindingUserReq();
            studentBindingUserReq.setStudentId(studentInfoByIdentityCardNoVo.getStudentId());
            studentBindingUserReq.setUserId(bindStudentReq.getUserId());
            studentBindingUserReq.setUserType(bindStudentReq.getUserType());
            List<YcqYjsStudentBindingUser> ycqYjsStudentBindingUserList = ycqYjsStudentBindingUserJdbcDao.selectList(studentBindingUserReq);
            //该学生已绑定用户，请勿重复绑定
            if(!CollectionUtils.isEmpty(ycqYjsStudentBindingUserList)){
                throw new BusinessException("身份证号：" + identityCardNo + "已绑定用户，请勿重复绑定");
            }
            YcqYjsStudentBindingUser ycqYjsStudentBindingUser = new YcqYjsStudentBindingUser();
            ycqYjsStudentBindingUser.setStudentId(studentInfoByIdentityCardNoVo.getStudentId());
            ycqYjsStudentBindingUser.setUserId(bindStudentReq.getUserId());
            ycqYjsStudentBindingUser.setUserType(bindStudentReq.getUserType());
            ycqYjsStudentBindingUser.setDelFlag(0);
            ycqYjsStudentBindingUser.setInFlag(CollectionUtils.isEmpty(studentInfoByUserVos) ? 1 : 0);
            ycqYjsStudentBindingUser.setCreatedTime(new Date());
            //  ycqYjsStudentBindingUser.setCreatedId(Objects.requireNonNull(CurrWorkerUtil.getCurrWorker()).getLoginId());
            studentBindingUserList.add(ycqYjsStudentBindingUser);
            if(CollectionUtils.isEmpty(studentInfoByUserVos)){
                studentInfoByUserVo.setIdentityCardNo(DesensitizedUtil.idCardNum(identityCardNo,4,4));
                studentInfoByUserVo.setStudentId(studentInfoByIdentityCardNoVo.getStudentId());
                studentInfoByUserVo.setStudentName(studentInfoByIdentityCardNoVo.getStudentName());
                studentInfoByUserVo.setSexStr(SexTypeEnum.getDesc(studentInfoByUserVo.getSex()));
                studentInfoByUserVo.setInFlag(1);
            }else{
                for(StudentInfoByUserVo infoByUserVo : studentInfoByUserVos){
                    if(1 == infoByUserVo.getInFlag()){
                        studentInfoByUserVo.setIdentityCardNo(DesensitizedUtil.idCardNum(identityCardNo,4,4));
                        studentInfoByUserVo.setStudentId(infoByUserVo.getStudentId());
                        studentInfoByUserVo.setStudentName(infoByUserVo.getStudentName());
                        studentInfoByUserVo.setSexStr(SexTypeEnum.getDesc(studentInfoByUserVo.getSex()));
                        studentInfoByUserVo.setInFlag(1);
                    }
                }
            }
        }
        if(! CollectionUtils.isEmpty(studentBindingUserList)) {
            log.info("用户绑定学生参数："+ studentBindingUserList);
            ycqYjsStudentBindingUserRepo.saveAllAndFlush(studentBindingUserList);
        }

        return studentInfoByUserVo;
    }

    @Override
    public void deleteBindStudentById(BigInteger id) {
        ycqYjsStudentBindingUserJdbcDao.deleteBindStudentById(id);
    }

    @Override
    public StudentInfoByIdentityCardNoVo searchStudentInfoByIdentityCardNo(IdentityCardNoVo identityCardNo) {
        return ycqYjsStudentBindingUserJdbcDao.searchStudentInfoByIdentityCardNo(identityCardNo.getIdentityCardNo());
    }

    @Override
    public void cutBindStudent(CutBindStudentVo cutBindStudentVo) {
        List<YcqYjsStudentBindingUser> byProp = ycqYjsStudentBindingUserRepo.findByProp("userId", cutBindStudentVo.getUserId());
        for(YcqYjsStudentBindingUser ycqYjsStudentBindingUser : byProp){
            if(cutBindStudentVo.getStudentId().equals(ycqYjsStudentBindingUser.getStudentId())){
                ycqYjsStudentBindingUser.setInFlag(1);
            }else {
                ycqYjsStudentBindingUser.setInFlag(0);
            }
        }
        ycqYjsStudentBindingUserRepo.saveAllAndFlush(byProp);
    }
}
