package com.crunii.micro.service.oneaffair.service.impl;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsImportRecord;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordReq;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordUpdateReq;
import com.crunii.micro.service.oneaffair.dto.vo.ImportRecord.ImportRecordVo;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsImportRecordJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsImportRecordRepo;
import com.crunii.micro.service.oneaffair.service.ImportRecordService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 批量导入记录表(YCQ_YJS_IMPORT_RECORD)服务实现类
 */
@Slf4j
@Service
public class ImportRecordServiceImpl implements ImportRecordService {
    @Resource
    private YcqYjsImportRecordJdbcDao ImportRecordJdbcDao;
    @Resource
    private YcqYjsImportRecordRepo ImportRecordRepo;

    /**
     * 根据ID查询导入记录数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsImportRecord selectOneById(BigInteger id) {
        log.info("ImportRecordServiceImpl.selectOneById.param：{}", id);
        return ImportRecordRepo.getOne(id);
    }

    /**
     * 列表查询导入记录数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsImportRecord> selectList(ImportRecordReq req) {
        log.info("ImportRecordServiceImpl.selectList.param：{}", req.toString());
        return ImportRecordJdbcDao.selectList(req);
    }

    /**
     * 分页查询导入记录数据
     *
     * @param req
     * @return
     */
    @Override
    public PageRsp<ImportRecordVo> pageSearch(PageReq<ImportRecordReq> req) {
        log.info("ImportRecordServiceImpl.pageSearch.param：{}", req.toString());
        return ImportRecordJdbcDao.pageSearch(req);
    }

    /**
     * 单个新增导入记录数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsImportRecord insertOne(ImportRecordCreateReq createReq) {
        log.info("ImportRecordServiceImpl.insertOne.param：{}", createReq.toString());
        return ImportRecordRepo.save(BeanCopyUtil.convert(createReq, YcqYjsImportRecord.class));
    }

    /**
     * 单个修改导入记录数据
     *
     * @param updateReq
     * @return
     */
    @Override
    public void updateOne(ImportRecordUpdateReq updateReq) {
        log.info("ImportRecordServiceImpl.updateOne.param：{}", updateReq.toString());
        YcqYjsImportRecord updateEntity = ImportRecordRepo.getOne(updateReq.getId());
        if (updateEntity != null) {
            BeanUtils.copyProperties(updateReq, updateEntity);
            ImportRecordRepo.saveAndFlush(updateEntity);
        }
    }


}