package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsMedalActivity;
import com.crunii.micro.service.oneaffair.dto.vo.medal.*;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.enums.MedalActivityStatusEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsMedalActivityJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsMedalActivityRepo;
import com.crunii.micro.service.oneaffair.service.MedalActivityService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 奖章表(YCQ_YJS_MEDAL)表服务实现类
 */
@Slf4j
@Service
public class MedalActivityServiceImpl implements MedalActivityService {

    @Resource
    private YcqYjsMedalActivityRepo medalActivityRepo;

    @Resource
    private YcqYjsMedalActivityJdbcDao ycqYjsMedalActivityJdbcDao;

    @Override
    public void batchCreateMedalActivity(MedalActivityCreateReq medalActivityCreateReq, Worker worker) {
        // 校验参数
        if (!CollectionUtils.isEmpty(medalActivityCreateReq.getActivityInfos())) {
            Date curr = new Date();
            BigInteger medalId = medalActivityCreateReq.getMedalId();
            Integer createdId = worker != null ? worker.getLoginId() : -1;
            List<YcqYjsMedalActivity> ycqYjsMedalActivityList = new ArrayList<>();
            medalActivityCreateReq.getActivityInfos().forEach(activityInfo -> {
                long count = medalActivityRepo.countByMedalIdAndActivityIdAndAppCodeAndDelFlag(medalId, activityInfo.getActivityId(), activityInfo.getAppCode(), DelFlagEnum.UN_DELETE.getCode());
                // 查询是否已存在数据
                if (count > 0) {
                    log.info("已存在关联数据");
                } else {
                    YcqYjsMedalActivity ycqYjsMedalActivity = new YcqYjsMedalActivity();
                    ycqYjsMedalActivity.setMedalId(medalId);
                    ycqYjsMedalActivity.setCreatedId(createdId);
                    ycqYjsMedalActivity.setActivityId(activityInfo.getActivityId());
                    ycqYjsMedalActivity.setAppCode(activityInfo.getAppCode());
                    ycqYjsMedalActivity.setCreatedId(createdId);
                    ycqYjsMedalActivity.setCreatedTime(curr);
                    ycqYjsMedalActivity.setLastUpdatedId(createdId);
                    ycqYjsMedalActivity.setLastUpdatedTime(curr);
                    ycqYjsMedalActivity.setDelFlag(0);
                    ycqYjsMedalActivity.setStatus(MedalActivityStatusEnum.JOIN.getValue());
                    ycqYjsMedalActivityList.add(ycqYjsMedalActivity);
                }
            });
            medalActivityRepo.saveAll(ycqYjsMedalActivityList);
        } else {
            throw new BusinessException("批量增加奖章关联活动参数异常");
        }
    }

    @Override
    public void deleteMedalActivityInfo(MedalActivityRemoveReq medalActivityRemoveReq) {
        // 判断参数是否正常
        if (!CollectionUtils.isEmpty(medalActivityRemoveReq.getActivityInfos()) && medalActivityRemoveReq.getMedalId() != null) {
            ycqYjsMedalActivityJdbcDao.deleteMedalInfo(medalActivityRemoveReq.getMedalId(),
                    medalActivityRemoveReq.getActivityInfos().stream()
                    .map(MedalActivityReq::getActivityId) // 提取activityId字段的值
                    .collect(Collectors.toList()));
        } else {
            throw new BusinessException("批量删除奖章关联活动参数异常");
        }
    }

    @Override
    public void updateMedalActivityStatus(MedalActivityStatusReq medalActivityStatusReq) {
        // 判断参数是否正常
        if (medalActivityStatusReq.getActivityId() != null && medalActivityStatusReq.getMedalId() != null && StringUtils.isNotEmpty(medalActivityStatusReq.getStatus())) {
            ycqYjsMedalActivityJdbcDao.updateMedalActivityStatus(medalActivityStatusReq.getMedalId(), medalActivityStatusReq.getActivityId(), medalActivityStatusReq.getStatus());
        } else {
            throw new BusinessException("修改奖章关联活动状态参数异常");
        }
    }

    @Override
    public List<YcqYjsMedalActivity> selectList(MedalActivityInfoReq req) {
        log.info("MedalActivityServiceImpl.selectList.param：{}", req.toString());
        return ycqYjsMedalActivityJdbcDao.selectList(req);
    }

}