package com.crunii.micro.service.oneaffair.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsMedal;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsMedalRule;
import com.crunii.micro.service.oneaffair.dto.vo.medal.*;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.enums.MedalRuleTypeEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsMedalRepo;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsMedalRuleJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsMedalRuleRepo;
import com.crunii.micro.service.oneaffair.service.MedalRuleService;
import com.crunii.micro.service.oneaffair.service.MedalService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 奖章表(YCQ_YJS_MEDAL)表服务实现类
 */
@Slf4j
@Service
public class MedalRuleServiceImpl implements MedalRuleService {

    @Resource
    private YcqYjsMedalRepo medalRepo;

    @Resource
    private YcqYjsMedalRuleRepo medalRuleRepo;

    @Resource
    private YcqYjsMedalRuleJdbcDao medalRuleJdbcDao;

    @Resource
    private TsmWorkerRepo tsmWorkerRepo;

    @Resource
    private MedalService medalService;

    @Override
    public PageRsp<MedalRuleInfoVo> pageSearchMedalRuleInfo(PageReq<MedalReq> req, Worker worker) throws Exception {
        try {
            log.info("MedalRuleServiceImpl.pageSearchMedalRuleInfo.param：{}", req.toString());
            List<TsmOrg> tsmOrgList = medalService.selectSchoolByUserId(worker);
            req.getQuery().setOrgIds(tsmOrgList.stream().map(TsmOrg::getOrgId).map(String::valueOf).collect(Collectors.toList()));
            PageRsp<MedalRuleInfoVo> medalRuleInfoVoPageRsp =  medalRuleJdbcDao.pageSearchMedalRuleInfo(req);
            medalRuleInfoVoPageRsp.getResult().stream().forEach(medalRuleInfoVo -> {
                //奖章判断规则列表
                medalRuleInfoVo.setRuleContents(Arrays.asList(medalRuleInfoVo.getRuleContent().split("\\|\\|\\|")));
                //规则判断方式
                List<String> ruleTypeList = Arrays.asList(medalRuleInfoVo.getRuleType().split("\\|\\|\\|"));
                medalRuleInfoVo.setRuleTypeName(
                        ruleTypeList.stream()
                                .map(ruleType -> {
                                    MedalRuleTypeEnum ruleTypeEnum = MedalRuleTypeEnum.fromCode(ruleType);
                                    if (ruleTypeEnum != null) {
                                        return ruleTypeEnum.getReferred();
                                    } else {
                                        // 处理未找到对应映射的情况
                                        return ""; // 或者其他默认值
                                    }
                                })
                                .filter(ruleTypeName -> !ruleTypeName.isEmpty()) // 过滤掉空字符串
                                .collect(Collectors.toCollection(LinkedHashSet::new)));
            });
            return medalRuleInfoVoPageRsp;
        } catch (BusinessException e) {
            log.error("MedalRuleServiceImpl.pageSearchMedalRuleInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("MedalRuleServiceImpl.pageSearchMedalRuleInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public Result<Boolean> batchCreateMedalRule(MedalRuleCreateReq medalRuleCreateReq, Worker worker) {
        // 校验参数
        if (!CollectionUtils.isEmpty(medalRuleCreateReq.getRuleInfos())) {
            Date curr = new Date();
            BigInteger medalId = medalRuleCreateReq.getMedalId();
            YcqYjsMedal ycqYjsMedal = medalRepo.getOne(medalId);
            // 判断是否有奖章
            if (ycqYjsMedal != null) {
                String ownerGrade = ycqYjsMedal.getOwnerGrade();
                // 查询奖章、年级下是否已添加规则
                if (medalRuleRepo.countByMedalIdAndOwnerGradeAndDelFlag(medalId,ownerGrade,DelFlagEnum.UN_DELETE.getCode()) > 0) {
                    return Result.ofFail("此奖章在该年级下已存在争章规则，请勿重复添加");
                } else {
                    Integer createdId =  worker != null ? worker.getLoginId() : -1;
                    String createdName = worker != null ? worker.getWorkerName() : "";
                    List<YcqYjsMedalRule> ycqYjsMedalRuleList = new ArrayList<>();
                    medalRuleCreateReq.getRuleInfos().forEach(ruleInfo -> {
                        YcqYjsMedalRule ycqYjsMedalRule = new YcqYjsMedalRule();
                        ycqYjsMedalRule.setId(null);
                        ycqYjsMedalRule.setMedalId(medalId);
                        ycqYjsMedalRule.setCreatedId(createdId);
                        ycqYjsMedalRule.setRuleContent(ruleInfo.getRuleContent());
                        ycqYjsMedalRule.setRuleType(ruleInfo.getRuleType());
                        ycqYjsMedalRule.setNeedVoucher(medalRuleCreateReq.getNeedVoucher());
                        ycqYjsMedalRule.setSystemRuleCode(ruleInfo.getSystemRuleCode());
                        ycqYjsMedalRule.setCreatedId(createdId);
                        ycqYjsMedalRule.setCreatedTime(curr);
                        ycqYjsMedalRule.setCreatedName(createdName);
                        ycqYjsMedalRule.setLastUpdatedId(createdId);
                        ycqYjsMedalRule.setLastUpdatedTime(curr);
                        ycqYjsMedalRule.setDelFlag(0);
                        ycqYjsMedalRule.setOrgId(ycqYjsMedal.getOrgId());
                        ycqYjsMedalRule.setOrgName(ycqYjsMedal.getOrgName());
                        ycqYjsMedalRule.setOwnerGrade(ownerGrade);
                        ycqYjsMedalRule.setOwnerGradeName(ycqYjsMedal.getOwnerGradeName());
                        ycqYjsMedalRuleList.add(ycqYjsMedalRule);
                    });
                    medalRuleRepo.saveAll(ycqYjsMedalRuleList);
                    return Result.ofSuccess(true);
                }
            } else {
                return Result.ofFail("未查询到该奖章信息");
            }
        } else {
            throw new BusinessException("批量增加争章规则参数异常");
        }
    }

    @Override
    public Result<Boolean> updateMedalRule(MedalRuleUpdateReq medalRuleUpdateReq,Worker worker) {
        // 校验参数
        if (!CollectionUtils.isEmpty(medalRuleUpdateReq.getRuleInfos())) {
            Date curr = new Date();
            BigInteger medalId = medalRuleUpdateReq.getMedalId();
            YcqYjsMedal ycqYjsMedal = medalRepo.getOne(medalId);
            // 判断是否有奖章
            if (ycqYjsMedal != null) {
                Integer lastUpdatedId = worker != null ? worker.getLoginId() : -1;
                String createdName = worker != null ? worker.getWorkerName() : "";
                List<YcqYjsMedalRule> ycqYjsMedalRuleList = new ArrayList<>();
                medalRuleUpdateReq.getRuleInfos().forEach(ruleInfo -> {
                    YcqYjsMedalRule ycqYjsMedalRule = new YcqYjsMedalRule();
                    ycqYjsMedalRule.setId(null);
                    ycqYjsMedalRule.setMedalId(medalId);
                    ycqYjsMedalRule.setCreatedId(lastUpdatedId);
                    ycqYjsMedalRule.setRuleContent(ruleInfo.getRuleContent());
                    ycqYjsMedalRule.setRuleType(ruleInfo.getRuleType());
                    ycqYjsMedalRule.setNeedVoucher(medalRuleUpdateReq.getNeedVoucher());
                    ycqYjsMedalRule.setSystemRuleCode(ruleInfo.getSystemRuleCode());
                    ycqYjsMedalRule.setCreatedId(lastUpdatedId);
                    ycqYjsMedalRule.setCreatedName(createdName);
                    ycqYjsMedalRule.setCreatedTime(curr);
                    ycqYjsMedalRule.setLastUpdatedId(lastUpdatedId);
                    ycqYjsMedalRule.setLastUpdatedTime(curr);
                    ycqYjsMedalRule.setDelFlag(0);
                    ycqYjsMedalRule.setOrgId(ycqYjsMedal.getOrgId());
                    ycqYjsMedalRule.setOrgName(ycqYjsMedal.getOrgName());
                    ycqYjsMedalRule.setOwnerGrade(ycqYjsMedal.getOwnerGrade());
                    ycqYjsMedalRule.setOwnerGradeName(ycqYjsMedal.getOwnerGradeName());
                    ycqYjsMedalRuleList.add(ycqYjsMedalRule);
                });
                // 删除原先关联争章规则
                medalRuleJdbcDao.deleteMedalRule(medalId);
                // 新增新的规则
                medalRuleRepo.saveAll(ycqYjsMedalRuleList);
                return Result.ofSuccess(true);
            } else {
                return Result.ofFail("未查询到该奖章信息");
            }
        } else {
            throw new BusinessException("批量增加争章规则参数异常");
        }
    }

    @Override
    public void delMedalRule(BigInteger id) {
        // 删除原先关联争章规则
        medalRuleJdbcDao.deleteMedalRule(id);
    }

    @Override
    public MedalRuleCreateReq selectMedalRuleInfo(BigInteger medalId) throws Exception {
        MedalRuleCreateReq medalRuleCreateReq = new MedalRuleCreateReq();
        List<YcqYjsMedalRule> ycqYjsMedalRules = medalRuleRepo.findByMedalIdAndDelFlag(medalId, DelFlagEnum.UN_DELETE.getCode());
        //判断是否存在数据
        if (!CollectionUtils.isEmpty(ycqYjsMedalRules)) {
            medalRuleCreateReq.setMedalId(medalId);
            medalRuleCreateReq.setNeedVoucher(ycqYjsMedalRules.get(0).getNeedVoucher());
            List<MedalRuleReq> medalRuleReqList = new ArrayList<>();
            ycqYjsMedalRules.forEach(ycqYjsMedalRule -> {
                MedalRuleReq medalRuleReq = new MedalRuleReq();
                BeanUtil.copyProperties(ycqYjsMedalRule, medalRuleReq);
                medalRuleReqList.add(medalRuleReq);
            });
            medalRuleCreateReq.setRuleInfos(medalRuleReqList);
        } else {
            log.info("当前暂无该奖章数据");
        }
        return medalRuleCreateReq;
    }

}