package com.crunii.micro.service.oneaffair.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.TsmPropertyValue;
import com.crunii.micro.domain.entity.main.TsmWorkerOrg;
import com.crunii.micro.domain.entity.main.YcqAppConfigUrl;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.domain.repo.main.TsmPropertyValueRepo;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;
import com.crunii.micro.domain.repo.main.YcqAppConfigUrlRepo;
import com.crunii.micro.service.oneaffair.dto.oneaffair.*;
import com.crunii.micro.service.oneaffair.dto.vo.Archive.ArchiveVo;
import com.crunii.micro.service.oneaffair.dto.vo.medal.*;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.StudentMedalReq;
import com.crunii.micro.service.oneaffair.enums.*;
import com.crunii.micro.service.oneaffair.jdbc.*;
import com.crunii.micro.service.oneaffair.service.MedalService;
import com.crunii.micro.service.oneaffair.util.CharacterCodeUtil;
import com.google.common.base.Joiner;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 奖章表(YCQ_YJS_MEDAL)表服务实现类
 */
@Slf4j
@Service
public class MedalServiceImpl implements MedalService {

    @Resource
    private YcqYjsMedalJdbcDao medalJdbcDao;

    @Resource
    private YcqYjsMedalRepo ycqYjsMedalRepo;

    @Resource
    private YcqYjsActivityJdbcDao activityJdbcDao;

    @Resource
    private YcqAppConfigUrlRepo ycqAppConfigUrlRepo;

    @Resource
    private TsmWorkerRepo tsmWorkerRepo;

    @Resource
    private YcqYjsMedalRuleRepo ycqYjsMedalRuleRepo;

    @Resource
    private YcqYjsMedalUnifyRepo ycqYjsMedalUnifyRepo;

    @Resource
    private TsmOrgRepo tsmOrgRepo;

    @Resource
    private YcqYjsMedalAppRepo ycqYjsMedalAppRepo;

    @Resource
    private YcqYjsMedalAppJdbcDao medalAppJdbcDao;

    @Resource
    private TsmPropertyValueRepo tsmPropertyValueRepo;

    @Resource
    private YcqYjsStudentMedalJdbcDao ycqYjsStudentMedalJdbcDao;

    @Resource
    private YcqYjsStudentRepo ycqYjsStudentRepo;

    @Autowired
    private OrgJdbcDao orgJdbcDao;

    @Value("${upload.url}")
    private String urlPrefix;

    private static final String ACTIVITY_SEARCH_METHOD = "PageSearch";

    private static final String ACTIVITY_SEARCH_NO_METHOD = "PageSearchNo";

    private Set<String> allowedAppCodes = new HashSet<>(Arrays.asList("matchManagement"));


    @Override
    public List<MedalUnifyInfoVo> searchAvailableMedalUnifyInfo(MedalReq req) throws Exception {
        try {
            List<YcqYjsMedalUnify> medalUnifyList = ycqYjsMedalUnifyRepo.findAvailableMedalUnifyInfo(DelFlagEnum.UN_DELETE.getCode(), 0, ObjectUtil.isEmpty(req) ? "" : req.getMedalType(), ObjectUtil.isEmpty(req) ? "" : req.getMedalMark());
            // 判断是否有可添加统一奖章
            if (CollectionUtils.isEmpty(medalUnifyList)) {
                return new ArrayList<>();
            } else {
                return medalUnifyList.stream()
                        .map(ycqYjsMedalUnify -> {
                            MedalUnifyInfoVo medalUnifyInfoVo = new MedalUnifyInfoVo();
                            BeanUtil.copyProperties(ycqYjsMedalUnify, medalUnifyInfoVo);
                            return medalUnifyInfoVo;
                        })
                        .collect(Collectors.toList());
            }
        } catch (BusinessException e) {
            log.error("MedalServiceImpl.searchAvailableMedalUnifyInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("MedalServiceImpl.searchAvailableMedalUnifyInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public MedalUnifyInfoVo searcMedalUnifyInfoByMedalNo(String medalNo) throws Exception {
        MedalUnifyInfoVo medalUnifyInfoVo = new MedalUnifyInfoVo();
        List<YcqYjsMedalUnify> medalUnifyList = ycqYjsMedalUnifyRepo.findByMedalNoAndDelFlag(medalNo, DelFlagEnum.UN_DELETE.getCode());
        //查询是否有数据
        if (medalUnifyList.size()>0) {
            copyPropertiesNonNull(medalUnifyList.get(0), medalUnifyInfoVo);
            return medalUnifyInfoVo;
        } else {
            log.info("暂时当前统一编号数据");
            return null;
        }

    }

    @Override
    public PageRsp<MedalInfoVo> pageSearchMedalInfo(PageReq<MedalReq> req, Worker worker) throws Exception {
        try {
            log.info("MedalServiceImpl.pageSearchMedalInfo.param：{}", req.toString());
            List<TsmOrg> tsmOrgList = selectSchoolByUserId(worker);
            req.getQuery().setOrgIds(tsmOrgList.stream().map(TsmOrg::getOrgId).map(String::valueOf).collect(Collectors.toList()));
            return medalJdbcDao.pageSearchMedalInfo(req);
        } catch (BusinessException e) {
            log.error("MedalServiceImpl.pageSearchMedalInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("MedalServiceImpl.pageSearchMedalInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public List<YcqYjsMedal> selectList(MedalReq req) throws Exception {
        try {
            log.info("MedalServiceImpl.selectList.param：{}", req.toString());
            return medalJdbcDao.selectList(req);
        } catch (BusinessException e) {
            log.error("MedalServiceImpl.selectList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("MedalServiceImpl.selectList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public List<MedalInfoMarkVo> selectListGroupByMedalMark(MedalReq req) throws Exception {
        try {
            log.info("MedalServiceImpl.selectListGroupByMedalMark.param：{}", req.toString());
            List<MedalInfoMarkVo> medalInfoMarkVoList = new ArrayList<>();
            req.setDisplayStatus(0);
            // 10/18需求变更，查询出该学生所在学校、年级、学期(上学期能看到上学期和全年的章，下学期能看到下学期和全年)的章,是否参与/点亮不筛选
            YcqYjsStudent ycqYjsStudent = ycqYjsStudentRepo.getOne(req.getStudentId());
            if (ycqYjsStudent != null) {
                req.setOrgId(String.valueOf(ycqYjsStudent.getCurrentSchoolId()));
                req.setOwnerGrade(String.valueOf(ycqYjsStudent.getCurrentGradeId()));
                List<String> ownerSemesters = new ArrayList<>();
                ownerSemesters.add(ycqYjsStudent.getSemesterType());
                ownerSemesters.add(MedalSemesterEnum.ALL_YEAR.getValue());
                req.setOwnerSemesters(ownerSemesters);
                req.setOwnerSemester(null);
                List<MedalStudentVo> ycqYjsMedalList = medalJdbcDao.selectStudentMedalList(req);
                // 根据字典标识分类
                List<TsmPropertyValue> tsmPropertyValueList = tsmPropertyValueRepo.findByProp("propCode", MedalPropertyEnum.MEDAL_MARK.getValue());
                // 判断是否查询所有章，无视类型
                if (StringUtils.isNotEmpty(req.getMedalType())) {
                    //判断是否为基础章、星级章
                    if (MedalTypeEnum.BASE.getValue().equals(req.getMedalType()) || MedalTypeEnum.STAR.getValue().equals(req.getMedalType())) {
                        // 查询该类型范围类的标识枚举
                        List<String> medalCodeList = MedalMarkEnum.fromTypeCode(req.getMedalType());
                        tsmPropertyValueList = tsmPropertyValueList.stream()
                                .filter(tsmPropertyValue -> medalCodeList.contains(tsmPropertyValue.getValueCode()))
                                .collect(Collectors.toList());
                    } else {
                        // 其他奖章 包含特色章
                        List<String> valueCodesList = Arrays.stream(MedalMarkEnum.values())
                                .map(MedalMarkEnum::getValueCode)
                                .collect(Collectors.toList());
                        tsmPropertyValueList =  tsmPropertyValueList.stream()
                                .filter(tsmPropertyValue -> !valueCodesList.contains(tsmPropertyValue.getValueCode()))
                                .collect(Collectors.toList());
                    }
                } else {
                    log.info("不筛选奖章类型");
                }
                // 根据标识分类
                tsmPropertyValueList.forEach(tsmPropertyValue -> {
                    MedalInfoMarkVo medalInfoMarkVo = new MedalInfoMarkVo();
                    medalInfoMarkVo.setMedalMark(tsmPropertyValue.getValueCode());
                    medalInfoMarkVo.setMedalMarkName(tsmPropertyValue.getValueName());
                    medalInfoMarkVo.setMedalType(MedalMarkEnum.fromValueCode(tsmPropertyValue.getValueCode()) != null ? MedalMarkEnum.fromValueCode(tsmPropertyValue.getValueCode()).getTypeCode() : MedalTypeEnum.CHARACTERISTIC.getValue());
                    medalInfoMarkVo.setMedalTypeName(MedalMarkEnum.fromValueCode(tsmPropertyValue.getValueCode()) != null ? MedalMarkEnum.fromValueCode(tsmPropertyValue.getValueCode()).getTypeName() : MedalTypeEnum.CHARACTERISTIC.getName());
                    medalInfoMarkVo.setYcqYjsMedalList(ycqYjsMedalList.stream()
                            .filter(medal -> tsmPropertyValue.getValueCode().equals(medal.getMedalMark()))
                            .collect(Collectors.toList()));
                    medalInfoMarkVoList.add(medalInfoMarkVo);
                });
                return medalInfoMarkVoList;
            } else {
                throw new BusinessException("未找到相关学生信息");
            }
        } catch (BusinessException e) {
            log.error("MedalServiceImpl.selectListGroupByMedalMark.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("MedalServiceImpl.selectListGroupByMedalMark.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public List<MedalInfoTypeVo> selectListGroupByMedalType(MedalStudentReq req) throws Exception {
        try {
            log.info("MedalServiceImpl.selectListGroupByMedalType.param：{}", req.toString());
            List<MedalInfoTypeVo> medalInfoTypeVoList = new ArrayList<>();
            req.setDisplayStatus(0);
            // 根据学生id获取奖章ids
            StudentMedalReq studentMedalReq = new StudentMedalReq();
            studentMedalReq.setStudentId(req.getStudentId());
            studentMedalReq.setIfLight(IfFlagEnum.TRUE.code());
            List<YcqYjsStudentMedal> ycqYjsStudentMedalList = ycqYjsStudentMedalJdbcDao.selectList(studentMedalReq);
            if (CollectionUtils.isEmpty(ycqYjsStudentMedalList)) {
                return Lists.newArrayList();
            } else {
                req.setIds(ycqYjsStudentMedalList.stream().map(YcqYjsStudentMedal::getMedalId).collect(Collectors.toList()));
                List<YcqYjsMedal> ycqYjsMedalList = medalJdbcDao.selectList(req);
                // 根据字典类型分类
                List<TsmPropertyValue> tsmPropertyValueList = tsmPropertyValueRepo.findByProp("propCode", MedalPropertyEnum.MEDAL_TYPE.getValue());
                // 根据标识分类
                tsmPropertyValueList.forEach(tsmPropertyValue -> {
                    MedalInfoTypeVo medalInfoTypeVo = new MedalInfoTypeVo();
                    medalInfoTypeVo.setMedalType(tsmPropertyValue.getValueCode());
                    medalInfoTypeVo.setMedalTypeName(tsmPropertyValue.getValueName());
                    medalInfoTypeVo.setYcqYjsMedalList(ycqYjsMedalList.stream()
                            .filter(medal -> tsmPropertyValue.getValueCode().equals(medal.getMedalType()))
                            .collect(Collectors.toList()));
                    medalInfoTypeVoList.add(medalInfoTypeVo);
                });
                return medalInfoTypeVoList;
            }
        } catch (BusinessException e) {
            log.error("MedalServiceImpl.selectListGroupByMedalType.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("MedalServiceImpl.selectListGroupByMedalType.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public YcqYjsMedal selectInfo(BigInteger id) throws Exception {
        return ycqYjsMedalRepo.getOne(id);
    }

    @Override
    public MedalStudentLightVo selectMedalLight(StudentActivityReq studentActivityReq) throws Exception {
        return medalJdbcDao.selectMedalLight(studentActivityReq);
    }

    @Override
    public Result<Boolean> createMedalInfo(MedalCreateReq createReq, Worker worker) {
        if (StringUtils.isEmpty(createReq.getMedalLevel())) {
            throw new BusinessException("奖章级别不存在，操作失败");
        } else {
            Date curr = new Date();
            YcqYjsMedal ycqYjsMedal = new YcqYjsMedal();
            copyPropertiesNonNull(createReq, ycqYjsMedal);
            // 判断奖章级别
            if (MedalLevelEnum.UNIFY.getValue().equals(createReq.getMedalLevel())) {
                //统一奖章
                long count = ycqYjsMedalRepo.countByUnifyMedalNoAndOrgIdAndOwnerGradeAndOwnerSemesterAndDelFlag(createReq.getUnifyMedalNo(),worker.getOrgId(),createReq.getOwnerGrade(),createReq.getOwnerSemester(), DelFlagEnum.UN_DELETE.getCode());
                //判断同学校、年级、学期是否已存在当前统一奖章
                if (count > 0 ){
                    //已存在
                    return Result.ofFail("当前全国统一奖章已绑定该学校同年级学期下,不可重复绑定");
                } else {
                    ycqYjsMedal.setUnifyMedalNo(createReq.getUnifyMedalNo());
                }
            } else {
                //自定义奖章
                ycqYjsMedal.setUnifyMedalNo(null);
            }
            ycqYjsMedal.setMedalNo(CharacterCodeUtil.generateCharacterCode("JZ"));
            ycqYjsMedal.setMedalLevelName(MedalLevelEnum.fromValue(createReq.getMedalLevel()).getName());
            ycqYjsMedal.setLastUpdatedTime(curr);
            ycqYjsMedal.setCreatedTime(curr);
            ycqYjsMedal.setDisplayStatus(0); //10/18需求变更，默认展示
            ycqYjsMedal.setDelFlag(0);
            if (worker != null) {
                ycqYjsMedal.setCreatedId(worker.getLoginId());
                ycqYjsMedal.setCreatedName(worker.getWorkerName());
                ycqYjsMedal.setLastUpdatedId(worker.getLoginId());
                if (StringUtils.isEmpty(ycqYjsMedal.getOrgId())) {
                    ycqYjsMedal.setOrgId(worker.getOrgId());
                }
                TsmOrg tsmOrg = tsmOrgRepo.findOrgById(ycqYjsMedal.getOrgId());
                if(tsmOrg != null) {
                    ycqYjsMedal.setOrgName(tsmOrg.getOrgName());
                }
            }
            ycqYjsMedalRepo.save(ycqYjsMedal);
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Boolean> createMedalInfoMulti(MedalCreateReq createReq, Worker worker) {
        if (StringUtils.isEmpty(createReq.getMedalLevel())) {
            throw new BusinessException("奖章级别不存在，操作失败");
        } else {
            Date curr = new Date();
            //判断是否多选年级
            if (CollectionUtils.isEmpty(createReq.getOwnerGradeList())) {
                createReq.getOwnerGradeList().add(createReq.getOwnerGrade());
                createReq.getOwnerGradeNameList().add(createReq.getOwnerGradeName());
            }
            String medalNo = CharacterCodeUtil.generateCharacterCode("JZ");
            List<YcqYjsMedal> ycqYjsMedalList = new ArrayList<>();
            for (int i = 0; i<createReq.getOwnerGradeList().size(); i++) {
                YcqYjsMedal ycqYjsMedal = new YcqYjsMedal();
                createReq.setOwnerGrade(createReq.getOwnerGradeList().get(i));
                createReq.setOwnerGradeName(createReq.getOwnerGradeNameList().get(i));
                copyPropertiesNonNull(createReq, ycqYjsMedal);
                // 判断奖章级别
                if (MedalLevelEnum.UNIFY.getValue().equals(createReq.getMedalLevel())) {
                    //统一奖章
                    long count = ycqYjsMedalRepo.countByUnifyMedalNoAndOrgIdAndOwnerGradeAndOwnerSemesterAndDelFlag(createReq.getUnifyMedalNo(),worker.getOrgId(),createReq.getOwnerGrade(),createReq.getOwnerSemester(), DelFlagEnum.UN_DELETE.getCode());
                    //判断同学校、年级、学期是否已存在当前统一奖章
                    if (count > 0 ){
                        //已存在
                        return Result.ofFail("当前全国统一奖章已绑定该学校同年级学期下,不可重复绑定");
                    } else {
                        ycqYjsMedal.setUnifyMedalNo(createReq.getUnifyMedalNo());
                    }
                } else {
                    //自定义奖章
                    ycqYjsMedal.setUnifyMedalNo(null);
                }
                ycqYjsMedal.setMedalNo(medalNo);
                ycqYjsMedal.setMedalLevelName(MedalLevelEnum.fromValue(createReq.getMedalLevel()).getName());
                ycqYjsMedal.setLastUpdatedTime(curr);
                ycqYjsMedal.setCreatedTime(curr);
                ycqYjsMedal.setDisplayStatus(0); //10/18需求变更，默认展示
                ycqYjsMedal.setDelFlag(0);
                if (worker != null) {
                    ycqYjsMedal.setCreatedId(worker.getLoginId());
                    ycqYjsMedal.setCreatedName(worker.getWorkerName());
                    ycqYjsMedal.setLastUpdatedId(worker.getLoginId());
                    if (StringUtils.isEmpty(ycqYjsMedal.getOrgId())) {
                        ycqYjsMedal.setOrgId(worker.getOrgId());
                    }
                    TsmOrg tsmOrg = tsmOrgRepo.findOrgById(ycqYjsMedal.getOrgId());
                    if(tsmOrg != null) {
                        ycqYjsMedal.setOrgName(tsmOrg.getOrgName());
                    }
                }
                ycqYjsMedalList.add(ycqYjsMedal);
            }
            ycqYjsMedalRepo.saveAll(ycqYjsMedalList);
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Boolean> updateMedalInfo(MedalUpdateReq updateReq, Worker worker) {
        YcqYjsMedal ycqYjsMedal = ycqYjsMedalRepo.getOne(updateReq.getId());
        // 查询奖章是否存在
        if (ycqYjsMedal == null) {
            throw new BusinessException("奖章不存在，操作失败");
        } else {
            // 判断根据奖章编号、年级id是否已存在数据
            if (ycqYjsMedalRepo.countByMedalNoAndOwnerGradeAndDelFlagAndIdNot(ycqYjsMedal.getMedalNo(),updateReq.getOwnerGrade(),DelFlagEnum.UN_DELETE.getCode(),updateReq.getId()) > 0) {
                return Result.ofFail("当前年级已存在该奖章，无法修改");
            }
            if (worker != null) {
                ycqYjsMedal.setLastUpdatedId(worker.getLoginId());
            }
            updateReq.setMedalLevelName(MedalLevelEnum.fromValue(updateReq.getMedalLevel()) != null ? MedalLevelEnum.fromValue(updateReq.getMedalLevel()).getName() : "");
            copyPropertiesNonNull(updateReq, ycqYjsMedal);
            TsmOrg tsmOrg = tsmOrgRepo.findOrgById(updateReq.getOrgId());
            if(tsmOrg != null) {
                ycqYjsMedal.setOrgName(tsmOrg.getOrgName());
            }
            // 判断奖章级别
            if (MedalLevelEnum.UNIFY.getValue().equals(updateReq.getMedalLevel())) {
                //统一奖章
                long count = ycqYjsMedalRepo.countByUnifyMedalNoAndOrgIdAndOwnerGradeAndOwnerSemesterAndDelFlagAndIdNot(updateReq.getUnifyMedalNo(),worker.getOrgId(),updateReq.getOwnerGrade(),updateReq.getOwnerSemester(), DelFlagEnum.UN_DELETE.getCode(), updateReq.getId());
                //判断同学校、年级、学期是否已存在当前统一奖章
                if (count > 0 ){
                    //已存在
                    return Result.ofFail("当前全国统一奖章已绑定该学校同年级学期下,不可重复绑定");
                } else {
                    ycqYjsMedal.setUnifyMedalNo(updateReq.getUnifyMedalNo());
                }
            } else {
                //自定义奖章
                ycqYjsMedal.setUnifyMedalNo(null);
            }
            ycqYjsMedalRepo.save(ycqYjsMedal);

            //同步更新争章规则中的组织、年级信息
            List<YcqYjsMedalRule> ycqYjsMedalRuleList = ycqYjsMedalRuleRepo.findByMedalIdAndDelFlag(updateReq.getId(), DelFlagEnum.UN_DELETE.getCode());
            if (!CollectionUtils.isEmpty(ycqYjsMedalRuleList)) {
                ycqYjsMedalRuleList.forEach(ycqYjsMedalRule -> {
                    ycqYjsMedalRule.setOrgId(ycqYjsMedal.getOrgId());
                    ycqYjsMedalRule.setOrgName(ycqYjsMedal.getOrgName());
                    ycqYjsMedalRule.setOwnerGrade(ycqYjsMedal.getOwnerGrade());
                    ycqYjsMedalRule.setOwnerGradeName(ycqYjsMedal.getOwnerGradeName());
                });
                ycqYjsMedalRuleRepo.saveAll(ycqYjsMedalRuleList);
            }
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Boolean> updateMedalInfoMulti(MedalUpdateReq updateReq, Worker worker) {
        YcqYjsMedal ycqYjsMedal = ycqYjsMedalRepo.getOne(updateReq.getId());
        // 查询奖章是否存在
        if (ycqYjsMedal == null) {
            throw new BusinessException("奖章不存在，操作失败");
        } else {
            if (worker != null) {
                ycqYjsMedal.setLastUpdatedId(worker.getLoginId());
            }
            updateReq.setMedalLevelName(MedalLevelEnum.fromValue(updateReq.getMedalLevel()) != null ? MedalLevelEnum.fromValue(updateReq.getMedalLevel()).getName() : "");
            //判断是否多选年级
            if (!CollectionUtils.isEmpty(updateReq.getOwnerGradeList())) {
                updateReq.setOwnerGrade(String.join(",", updateReq.getOwnerGradeList()));
                updateReq.setOwnerGradeName(String.join(",", updateReq.getOwnerGradeNameList()));
            }
            copyPropertiesNonNull(updateReq, ycqYjsMedal);
            TsmOrg tsmOrg = tsmOrgRepo.findOrgById(updateReq.getOrgId());
            if(tsmOrg != null) {
                ycqYjsMedal.setOrgName(tsmOrg.getOrgName());
            }
            // 判断奖章级别
            if (MedalLevelEnum.UNIFY.getValue().equals(updateReq.getMedalLevel())) {
                //统一奖章
                long count = ycqYjsMedalRepo.countByUnifyMedalNoAndOrgIdAndOwnerGradeAndOwnerSemesterAndDelFlagAndIdNot(updateReq.getUnifyMedalNo(),worker.getOrgId(),updateReq.getOwnerGrade(),updateReq.getOwnerSemester(), DelFlagEnum.UN_DELETE.getCode(), updateReq.getId());
                //判断同学校、年级、学期是否已存在当前统一奖章
                if (count > 0 ){
                    //已存在
                    return Result.ofFail("当前全国统一奖章已绑定该学校同年级学期下,不可重复绑定");
                } else {
                    ycqYjsMedal.setUnifyMedalNo(updateReq.getUnifyMedalNo());
                }
            } else {
                //自定义奖章
                ycqYjsMedal.setUnifyMedalNo(null);
            }
            ycqYjsMedalRepo.save(ycqYjsMedal);
        }
        return Result.ofSuccess(true);
    }

    @Override
    public void deleteMedalInfo(BigInteger id) {
        MedalInfoVo medalInfoVo = medalJdbcDao.selectOneById(id);
        // 判断是否存在数据
        if (medalInfoVo == null) {
            throw new BusinessException("奖章不存在，操作失败");
        } else {
            medalJdbcDao.deleteMedalInfo(id);
            //同步删除争章规则对应奖章数据
            List<YcqYjsMedalRule> ycqYjsMedalRuleList = ycqYjsMedalRuleRepo.findByMedalIdAndDelFlag(id, DelFlagEnum.UN_DELETE.getCode());
            if (!CollectionUtils.isEmpty(ycqYjsMedalRuleList)) {
                ycqYjsMedalRuleList.forEach(ycqYjsMedalRule -> {
                    ycqYjsMedalRule.setDelFlag(DelFlagEnum.DELETED.getCode());
                });
                ycqYjsMedalRuleRepo.saveAll(ycqYjsMedalRuleList);
            }
        }
    }

    @Override
    public void changeDisplayStatus(MedalDisplayStatusReq displayStatusReq, Worker worker) {
        MedalInfoVo medalInfoVo = medalJdbcDao.selectOneById(displayStatusReq.getId());
        // 查询奖章是否存在
        if (medalInfoVo == null) {
            throw new BusinessException("奖章不存在，操作失败");
        } else {
            if (worker != null) {
                displayStatusReq.setLastUpdatedId(worker.getLoginId());
            }
        }
        medalJdbcDao.changeDisplayStatus(displayStatusReq);
    }

    @Override
    public List<MedalAppConfigVo> queryAppConfigList(Worker worker) {
        List<YcqAppConfigUrl> ycqAppConfigUrlList = ycqAppConfigUrlRepo.findYcqAppConfigUrlByStateOrderBySortAsc(CommonConstants.STATE_VALID);
        List<MedalAppConfigVo> medalAppConfigVoList = new ArrayList<>();
        ycqAppConfigUrlList.forEach(e ->{
            MedalAppConfigVo medalAppConfigVo = new MedalAppConfigVo();
            e.setLogoUrl(urlPrefix + e.getLogoUrl());
            BeanUtil.copyProperties(e, medalAppConfigVo);

            long count = ycqYjsMedalAppRepo.countByAppCodeAndCreatedIdAndDelFlag(e.getAppCode(),worker.getLoginId(),0);
            // 查询当前用户是否关联此app配置
            if (count > 0) {
                medalAppConfigVo.setUseStatus("1");
            } else {
                medalAppConfigVo.setUseStatus("0");
            }
            medalAppConfigVoList.add(medalAppConfigVo);
            }
        );
        return medalAppConfigVoList;
    }

    @Override
    public PageRsp<MedalActivityInfoVo> pageSearchMedalNoActivityInfo(PageReq<ActivityReq> req) {
        // 判断参数是否正常
        if (!allowedAppCodes.contains(req.getQuery().getAppCode()) || ObjectUtil.isEmpty(req.getQuery().getMedalId())) {
            // 如果appType不在允许的集合中，直接返回
            PageRsp<MedalActivityInfoVo> result = new PageRsp<>();
            result.setResult(new ArrayList<>());
            return result;
        } else {
            String methodName = req.getQuery().getAppCode() + ACTIVITY_SEARCH_NO_METHOD;
            try {
                // 使用反射调用不同的DAO方法
                Method method = YcqYjsActivityJdbcDao.class.getMethod(methodName, PageReq.class);
                return (PageRsp<MedalActivityInfoVo>) method.invoke(activityJdbcDao, req);
            } catch (Exception e) {
                log.error("MedalServiceImpl.pageSearchMedalNoActivityInfo.error:{}", e.getMessage());
                e.printStackTrace();
                throw new BusinessException("系统内部异常，请联系管理员！");
            }
        }
    }

    @Override
    public PageRsp<MedalActivityInfoVo> pageSearchMedalActivityInfo(PageReq<ActivityReq> req) throws Exception {
        // 判断参数是否正常
        if (!allowedAppCodes.contains(req.getQuery().getAppCode()) || ObjectUtil.isEmpty(req.getQuery().getMedalId())) {
            // 如果appType不在允许的集合中，直接返回
            PageRsp<MedalActivityInfoVo> result = new PageRsp<>();
            result.setResult(new ArrayList<>());
            return result;
        } else {
            String methodName = req.getQuery().getAppCode() + ACTIVITY_SEARCH_METHOD;
            try {
                // 使用反射调用不同的DAO方法
                Method method = YcqYjsActivityJdbcDao.class.getMethod(methodName, PageReq.class);
                return (PageRsp<MedalActivityInfoVo>) method.invoke(activityJdbcDao, req);
            } catch (Exception e) {
                log.error("MedalServiceImpl.pageSearchMedalActivityInfo.error:{}", e.getMessage());
                e.printStackTrace();
                throw new BusinessException("系统内部异常，请联系管理员！");
            }
        }
    }

    @Override
    public List<MedalActivityInfoVo> selectActivityList(BigInteger medal) throws Exception {
        try {
            log.info("MedalServiceImpl.selectActivityList.param：{}", medal);
            // 判断参数是否正常
            if (medal == null) {
                log.error("MedalServiceImpl.selectList.error:查询参数异常");
                throw new BusinessException("查询参数异常");
            } else {
                return activityJdbcDao.selectList(medal);
            }
        } catch (BusinessException e) {
            log.error("MedalServiceImpl.selectActivityList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("MedalServiceImpl.selectActivityList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public List<ArchiveVo> selectStudentActivityList(StudentActivityReq studentActivityReq) throws Exception {
        try {
            // 判断参数是否正常
            if (studentActivityReq == null || StringUtils.isEmpty(studentActivityReq.getStudentId()) || studentActivityReq.getMedalId() == null) {
                log.error("MedalServiceImpl.selectStudentActivityList.error:查询参数异常");
                throw new BusinessException("查询参数异常");
            } else {
                return activityJdbcDao.selectStudentActivityList(studentActivityReq.getStudentId(),studentActivityReq.getMedalId());
            }
        } catch (BusinessException e) {
            log.error("MedalServiceImpl.selectStudentActivityList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("MedalServiceImpl.selectStudentActivityList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public List<TsmOrg> selectSchoolByUserId(Worker worker) throws Exception {
        if (worker != null) {
            List<Long> orgIds = new ArrayList<>();
            List<TsmWorkerOrg>tsmWorkerOrgList = orgJdbcDao.findByWorkId(worker.getLoginId());
            // 加入不递归数据
            orgIds.addAll(tsmWorkerOrgList.stream()
                    .filter(tsmWorkerOrg -> tsmWorkerOrg.getRecursiveOrNot() == 2L)
                    .map(TsmWorkerOrg::getOrgId)
                    .collect(Collectors.toList()));
            // 加入递归数据
            List<Long> recursiveOrgIds = tsmWorkerOrgList.stream()
                    .filter(tsmWorkerOrg -> tsmWorkerOrg.getRecursiveOrNot() == 1L)
                    .map(TsmWorkerOrg::getOrgId)
                    .collect(Collectors.toList());
            for (Long recursiveOrgId : recursiveOrgIds) {
                // 递归查询所有下级组织
                List<Long> orgList = new ArrayList<>();
                orgList.add(recursiveOrgId);
                listChildSchool(orgList, String.valueOf(recursiveOrgId));
                orgIds.addAll(orgList);
            }
            // 获取所有学校数据
            return orgJdbcDao.findSchoolByOrgIds(orgIds);
        } else {
            log.info("获取用户信息异常");
        }
        return new ArrayList<>();
    }

    /**
     * 查询当前用户的学校下的年级信息
     */
    @Override
    public List<MedalOrgGradeInfoVo> selectMedalOrgGrade(Worker worker) throws Exception {
        List<MedalOrgGradeInfoVo> gradeInfoVoList = new ArrayList<>();
        if (worker != null) {
            TsmOrg tsmOrg = tsmOrgRepo.findOrgById(worker.getOrgId());
            //判断是否有学校权限
            if (tsmOrg != null) {
                List<TsmOrg> orgList = tsmOrgRepo.findByParentOrgIdAndState(tsmOrg.getOrgId(), 1);
                orgList.forEach(childTsmOrg -> {
                    MedalOrgGradeInfoVo medalOrgGradeInfoVo = new MedalOrgGradeInfoVo();
                    medalOrgGradeInfoVo.setOrgGradeCode(childTsmOrg.getOrgCode());
                    medalOrgGradeInfoVo.setOrgGradeId(childTsmOrg.getOrgId());
                    medalOrgGradeInfoVo.setOrgGradeName(childTsmOrg.getOrgName());
                    gradeInfoVoList.add(medalOrgGradeInfoVo);
                });
            } else {
                log.info("该登录用户暂无学校组织");
            }
        } else {
            log.info("获取用户信息异常");
        }
        return gradeInfoVoList;
    }

    @Override
    public List<TsmPropertyValue> selectMedalMarkByType(String medalTypeCode) throws Exception {
        // 判断是否有奖章类型参数
        if (StringUtils.isEmpty(medalTypeCode)) {
            return Lists.newArrayList();
        } else {
            // 查询字典-奖章标识
            List<TsmPropertyValue> tsmPropertyValueList = tsmPropertyValueRepo.findByProp("propCode", MedalPropertyEnum.MEDAL_MARK.getValue());
            // 判断奖章类型是否为基础章、星级章
            if (MedalTypeEnum.BASE.getValue().equals(medalTypeCode) || MedalTypeEnum.STAR.getValue().equals(medalTypeCode)) {
                // 查询该类型范围类的标识枚举
                List<String> medalCodeList = MedalMarkEnum.fromTypeCode(medalTypeCode);
                return tsmPropertyValueList.stream()
                        .filter(tsmPropertyValue -> medalCodeList.contains(tsmPropertyValue.getValueCode()))
                        .collect(Collectors.toList());
            } else {
                // 其他奖章 包含特色章
                List<String> valueCodesList = Arrays.stream(MedalMarkEnum.values())
                        .map(MedalMarkEnum::getValueCode)
                        .collect(Collectors.toList());
                return tsmPropertyValueList.stream()
                        .filter(tsmPropertyValue -> !valueCodesList.contains(tsmPropertyValue.getValueCode()))
                        .collect(Collectors.toList());
            }
        }
    }

    @Override
    public List<MedalAppInfoVo> selectUserMedalApp(Worker worker) {
        List<MedalAppInfoVo> medalAppInfoVoList = new ArrayList<>();
        if (worker != null) {
            List<YcqYjsMedalApp> appList = ycqYjsMedalAppRepo.findByCreatedIdAndDelFlagOrderByCreatedTimeDesc(worker.getLoginId(), 0);
            appList.forEach(ycqYjsMedalApp -> {
                MedalAppInfoVo medalAppInfoVo = new MedalAppInfoVo();
                medalAppInfoVo.setId(ycqYjsMedalApp.getId());
                medalAppInfoVo.setAppCode(ycqYjsMedalApp.getAppCode());
                medalAppInfoVo.setAppName(ycqYjsMedalApp.getAppName());
                medalAppInfoVoList.add(medalAppInfoVo);
            });
        } else {
            log.info("获取用户信息异常");
        }
        return medalAppInfoVoList;
    }

    @Override
    public void batchCreateMedalApp(List<MedalAppCreateReq> medalAppCreateReqList, Worker worker) {
        // 判断业务是否为空
        if (!CollectionUtils.isEmpty(medalAppCreateReqList) && worker != null) {
            Date curr = new Date();
            List<YcqYjsMedalApp> ycqYjsMedalAppList = new ArrayList<>();
            medalAppCreateReqList.forEach(medalAppCreateReq -> {
               YcqYjsMedalApp ycqYjsMedalApp = new YcqYjsMedalApp();
               ycqYjsMedalApp.setAppName(medalAppCreateReq.getAppName());
               ycqYjsMedalApp.setAppCode(medalAppCreateReq.getAppCode());
               ycqYjsMedalApp.setCreatedId(worker.getLoginId());
               ycqYjsMedalApp.setCreatedTime(curr);
               ycqYjsMedalApp.setLastUpdatedId(worker.getLoginId());
               ycqYjsMedalApp.setLastUpdatedTime(curr);
               ycqYjsMedalApp.setDelFlag(0);
               ycqYjsMedalAppList.add(ycqYjsMedalApp);
            });
            // 删除原有所有关联app
            medalAppJdbcDao.deleteMedalAppByUser(worker.getLoginId());
            //添加新的关联配置
            ycqYjsMedalAppRepo.saveAll(ycqYjsMedalAppList);
        } else{
            log.info("无需新增业务");
        }
    }

    @Override
    public void deleteMedalApp(BigInteger id) {
        YcqYjsMedalApp ycqYjsMedalApp = ycqYjsMedalAppRepo.getOne(id);
        // 判断是否存在数据
        if (ycqYjsMedalApp == null) {
            throw new BusinessException("奖章不存在，操作失败");
        } else {
            medalAppJdbcDao.deleteMedalApp(id);
        }
    }


    @Override
    public List<MedalBaseInfoVo> searchNoRuleMedal(Worker worker) throws Exception {
        List<YcqYjsMedal> ycqYjsMedalList;
        List<BigInteger> ids = ycqYjsMedalRuleRepo.findExistRuleMedal();
        // 判断是否存在已配置过争章规则
        if (CollectionUtils.isEmpty(ids)) {
            ycqYjsMedalList = ycqYjsMedalRepo.findByDelFlag(DelFlagEnum.UN_DELETE.getCode());
        } else {
            ycqYjsMedalList = ycqYjsMedalRepo.findByIdNotInAndDelFlag(ids, DelFlagEnum.UN_DELETE.getCode());
        }
        return ycqYjsMedalList.stream()
                .map(ycqYjsMedal -> {
                    MedalBaseInfoVo medalBaseInfoVo = new MedalBaseInfoVo();
                    BeanUtil.copyProperties(ycqYjsMedal, medalBaseInfoVo);
                    return medalBaseInfoVo;
                })
                .distinct()
                .collect(Collectors.toList());
    }

    @Override
    public List<MedalGradeInfoVo> searchMedalGradeByMedalNo(MedalNoReq medalNoReq, Worker worker) throws Exception {
        List<BigInteger> ids = ycqYjsMedalRuleRepo.findExistRuleMedal();
        List<YcqYjsMedal> ycqYjsMedalList = ycqYjsMedalRepo.findByMedalNoAndDelFlag(medalNoReq.getMedalNo(),DelFlagEnum.UN_DELETE.getCode());
        return ycqYjsMedalList.stream()
                .map(ycqYjsMedal -> {
                    MedalGradeInfoVo medalGradeInfoVo = new MedalGradeInfoVo();
                    BeanUtil.copyProperties(ycqYjsMedal, medalGradeInfoVo);
                    // 判断是否已配置过规则
                    if (ids.contains(ycqYjsMedal.getId())) {
                        medalGradeInfoVo.setUseFlag(MedalUseTypeEnum.USE.getValue());
                    } else {
                        medalGradeInfoVo.setUseFlag(MedalUseTypeEnum.UN_USE.getValue());
                    }
                    return medalGradeInfoVo;
                })
                .distinct()
                .collect(Collectors.toList());
    }

    @Override
    public List<TsmOrg> selectGradeInfoByMedalId(BigInteger medalId) throws Exception {
        YcqYjsMedal ycqYjsMedal = ycqYjsMedalRepo.getOne(medalId);
        // 判断是否存在奖章
        if (ycqYjsMedal != null) {
            List<String> ownerGradeList = Arrays.asList(ycqYjsMedal.getOwnerGrade().split(","));
            List<Long> longList = ownerGradeList.stream().map(Long::parseLong).collect(Collectors.toList());
            return tsmOrgRepo.findByOrgIdIn(longList);
        } else {
            throw new BusinessException("未找到该奖章信息");
        }
    }

    private void copyPropertiesNonNull(Object source, Object target) {
        if (source != null) {
            final BeanWrapper src = new BeanWrapperImpl(source);
            PropertyDescriptor[] pds = src.getPropertyDescriptors();

            List<String> nullProperties = new ArrayList<>();
            for (PropertyDescriptor pd : pds) {
                String propertyName = pd.getName();
                if (src.getPropertyValue(propertyName) == null) {
                    nullProperties.add(propertyName);
                }
            }
            String[] nullPropertyNames = nullProperties.toArray(new String[0]);
            BeanUtil.copyProperties(source, target, nullPropertyNames);
        }
    }

    /**
     * 递归查询重庆市学校组织的所有下级id
     *
     * @param orgList
     * @param orgId
     * @return
     */
    private void listChildSchool(List<Long> orgList, String orgId) {
        //只有学校组织的递归查询
        if (OrgEnum.SCHOOL.getValue().equals(orgId)) {
            List<TsmOrg> orgs = orgJdbcDao.listByParentOrgIds(orgId);
            if (orgs.size() > 0) {
                List<Long> orgIds = orgs.stream().map(TsmOrg::getOrgId).collect(Collectors.toList());
                orgList.addAll(orgIds);
                String strOrgIds = Joiner.on(",").join(orgIds);
                this.listChildSchool(orgList, strOrgIds);
            }
        }
    }
}