package com.crunii.micro.service.oneaffair.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsMedal;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsMedalUnify;
import com.crunii.micro.service.oneaffair.dto.vo.medal.*;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.enums.MedalLevelEnum;
import com.crunii.micro.service.oneaffair.enums.OrgEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsMedalJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsMedalRepo;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsMedalUnifyJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsMedalUnifyRepo;
import com.crunii.micro.service.oneaffair.service.MedalUnifyService;
import com.crunii.micro.service.oneaffair.util.CharacterCodeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.beans.PropertyDescriptor;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 奖章表(YCQ_YJS_MEDAL_UNIFY)表服务实现类
 */
@Slf4j
@Service
public class MedalUnifyServiceImpl implements MedalUnifyService {

    @Resource
    private YcqYjsMedalUnifyJdbcDao medalUnifyJdbcDao;

    @Resource
    private YcqYjsMedalUnifyRepo ycqYjsMedalUnifyRepo;

    @Resource
    private YcqYjsMedalJdbcDao medalJdbcDao;

    @Resource
    private YcqYjsMedalRepo ycqYjsMedalRepo;


    @Override
    public PageRsp<MedalUnifyInfoVo> pageSearchMedalUnifyInfo(PageReq<MedalReq> req) throws Exception {
        try {
            log.info("MedalUnifyServiceImpl.pageSearchMedalUnifyInfo.param：{}", req.toString());
            return medalUnifyJdbcDao.pageSearchMedalUnifyInfo(req);
        } catch (BusinessException e) {
            log.error("MedalUnifyServiceImpl.pageSearchMedalUnifyInfo.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("MedalUnifyServiceImpl.pageSearchMedalUnifyInfo.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public Result<Boolean> createMedalUnifyInfo(MedalUnifyCreateReq createReq, Worker worker) {
        // 判断是否已存在相同名称奖章名称
        if (ycqYjsMedalUnifyRepo.countByMedalNameAndDelFlag(createReq.getMedalName(),DelFlagEnum.UN_DELETE.getCode()) > 0) {
            return Result.ofFail("已存在相同名称的统一奖章，请修改名称");
        } else {
            Date curr = new Date();
            YcqYjsMedalUnify ycqYjsMedalUnify = new YcqYjsMedalUnify();
            if (worker != null) {
                ycqYjsMedalUnify.setCreatedId(worker.getLoginId());
                ycqYjsMedalUnify.setCreatedName(worker.getWorkerName());
                ycqYjsMedalUnify.setLastUpdatedId(worker.getLoginId());
            }
            BeanUtil.copyProperties(createReq, ycqYjsMedalUnify);
            ycqYjsMedalUnify.setMedalNo(CharacterCodeUtil.generateCharacterCode("TYJZ"));
            //固定统一奖章
            ycqYjsMedalUnify.setLastUpdatedTime(curr);
            ycqYjsMedalUnify.setCreatedTime(curr);
            ycqYjsMedalUnify.setDisplayStatus(0); //10/18需求变更，默认展示
            ycqYjsMedalUnify.setDelFlag(0);
            ycqYjsMedalUnifyRepo.save(ycqYjsMedalUnify);
            return Result.ofSuccess(true);
        }
    }

    @Override
    public Result<Boolean> updateMedalUnifyInfo(MedalUnifyUpdateReq updateReq, Worker worker) {
        // 判断是否已存在相同名称奖章名称
        if (ycqYjsMedalUnifyRepo.countByMedalNameAndDelFlagAndIdNot(updateReq.getMedalName(),DelFlagEnum.UN_DELETE.getCode(),updateReq.getId()) > 0) {
            return Result.ofFail("已存在相同名称的统一奖章，请修改名称");
        } else {
            YcqYjsMedalUnify ycqYjsMedalUnify = ycqYjsMedalUnifyRepo.getOne(updateReq.getId());
            // 查询统一奖章是否存在
            if (ycqYjsMedalUnify == null) {
                throw new BusinessException("奖章不存在，操作失败");
            } else {
                copyPropertiesNonNull(updateReq, ycqYjsMedalUnify);
                if (worker != null) {
                    ycqYjsMedalUnify.setLastUpdatedId(worker.getLoginId());
                }
                ycqYjsMedalUnify.setLastUpdatedTime(new Date());
                ycqYjsMedalUnifyRepo.save(ycqYjsMedalUnify);

                // 查询是否有已关联奖章数据，如果有需要同步更新
                List<YcqYjsMedal> ycqYjsMedalList = ycqYjsMedalRepo.findByDelFlagAndUnifyMedalNoAndMedalLevel(DelFlagEnum.UN_DELETE.getCode(),ycqYjsMedalUnify.getMedalNo(), MedalLevelEnum.UNIFY.getValue());
                if (CollectionUtils.isEmpty(ycqYjsMedalList)) {
                    log.info("无需更新奖章表数据");
                } else {
                    List<YcqYjsMedal> updateMedalList = new ArrayList<>();
                    ycqYjsMedalList.forEach(ycqYjsMedal -> {
                        ycqYjsMedal.setMedalName(ycqYjsMedalUnify.getMedalName());
                        ycqYjsMedal.setMedalPicKey(ycqYjsMedalUnify.getMedalPicKey());
                        ycqYjsMedal.setMedalType(ycqYjsMedalUnify.getMedalType());
                        ycqYjsMedal.setMedalMark(ycqYjsMedalUnify.getMedalMark());
                        ycqYjsMedal.setIntroduction(ycqYjsMedalUnify.getIntroduction());
                        ycqYjsMedal.setMedalTypeName(ycqYjsMedalUnify.getMedalTypeName());
                        ycqYjsMedal.setMedalMarkName(ycqYjsMedalUnify.getMedalMarkName());
                        ycqYjsMedal.setLastUpdatedTime(new Date());
                        if (worker != null) {
                            ycqYjsMedal.setLastUpdatedId(worker.getLoginId());
                        }
                        updateMedalList.add(ycqYjsMedal);
                    });
                    ycqYjsMedalRepo.saveAll(updateMedalList);
                }
            }
            return Result.ofSuccess(true);
        }
    }

    @Override
    public void deleteMedalUnifyInfo(BigInteger id) {
        YcqYjsMedalUnify ycqYjsMedalUnify = ycqYjsMedalUnifyRepo.getOne(id);
        // 判断是否存在数据
        if (ycqYjsMedalUnify == null) {
            throw new BusinessException("奖章不存在，操作失败");
        } else {
            String unifyMedalNo = ycqYjsMedalUnify.getMedalNo();
            medalUnifyJdbcDao.deleteMedalUnifyInfo(id);

            //判断是否有关联奖章数据
            List<YcqYjsMedal> ycqYjsMedalList = ycqYjsMedalRepo.findByDelFlagAndUnifyMedalNoAndMedalLevel(DelFlagEnum.UN_DELETE.getCode(),unifyMedalNo, MedalLevelEnum.UNIFY.getValue());
            if (CollectionUtils.isEmpty(ycqYjsMedalList)) {
                log.info("无关联奖章数据");
            } else {
                ycqYjsMedalList.forEach(ycqYjsMedal -> {
                    medalJdbcDao.deleteMedalInfo(ycqYjsMedal.getId());
                });
            }
        }
    }

    @Override
    public void changeDisplayStatus(MedalDisplayStatusReq displayStatusReq, Worker worker) {
        YcqYjsMedalUnify ycqYjsMedalUnify = ycqYjsMedalUnifyRepo.getOne(displayStatusReq.getId());
        // 查询奖章是否存在
        if (ycqYjsMedalUnify == null) {
            throw new BusinessException("奖章不存在，操作失败");
        } else {
            if (worker != null) {
                displayStatusReq.setLastUpdatedId(worker.getLoginId());
            }
        }
        medalUnifyJdbcDao.changeDisplayStatus(displayStatusReq);
    }

    @Override
    public List<MedalUnifySchoolInfoVo> searchMedalUnifySchoolInfo(MedalSchoolReq medalSchoolReq) {
        YcqYjsMedalUnify ycqYjsMedalUnify = ycqYjsMedalUnifyRepo.getOne(medalSchoolReq.getId());
        // 查询奖章是否存在
        if (ycqYjsMedalUnify == null) {
            throw new BusinessException("奖章不存在，操作失败");
        } else {
            List<YcqYjsMedal> ycqYjsMedalList = new ArrayList<>();
            // 判断查询条件是否带组织code
            if (CollectionUtils.isEmpty(medalSchoolReq.getOrgId()) || medalSchoolReq.getOrgId().contains(OrgEnum.SCHOOL.getValue())) {
                ycqYjsMedalList = ycqYjsMedalRepo.findByDelFlagAndUnifyMedalNoAndMedalLevel(DelFlagEnum.UN_DELETE.getCode(),ycqYjsMedalUnify.getMedalNo(), MedalLevelEnum.UNIFY.getValue());
            } else {
                ycqYjsMedalList = ycqYjsMedalRepo.findByDelFlagAndUnifyMedalNoAndMedalLevelAndOrgIdIn(DelFlagEnum.UN_DELETE.getCode(),ycqYjsMedalUnify.getMedalNo(), MedalLevelEnum.UNIFY.getValue(), medalSchoolReq.getOrgId());
            }
            List<MedalUnifySchoolInfoVo> schoolInfoVos = new ArrayList<>();
            ycqYjsMedalList.stream().forEach(ycqYjsMedal -> {
                MedalUnifySchoolInfoVo medalUnifySchoolInfoVo = new MedalUnifySchoolInfoVo();
                copyPropertiesNonNull(ycqYjsMedal, medalUnifySchoolInfoVo);
                schoolInfoVos.add(medalUnifySchoolInfoVo);
            });
            return schoolInfoVos;
        }
    }

    @Override
    public List<YcqYjsMedal> searchMedalInfoByUnifyMedalNo(String medalNo) {
        if (StringUtils.isEmpty(medalNo)) {
            throw new BusinessException("奖章编号异常，操作失败");
        } else {
            return ycqYjsMedalRepo.findByDelFlagAndUnifyMedalNoAndMedalLevel(DelFlagEnum.UN_DELETE.getCode(),medalNo, MedalLevelEnum.UNIFY.getValue());
        }
    }

    private void copyPropertiesNonNull(Object source, Object target) {
        if (source != null) {
            final BeanWrapper src = new BeanWrapperImpl(source);
            PropertyDescriptor[] pds = src.getPropertyDescriptors();

            List<String> nullProperties = new ArrayList<>();
            for (PropertyDescriptor pd : pds) {
                String propertyName = pd.getName();
                if (src.getPropertyValue(propertyName) == null) {
                    nullProperties.add(propertyName);
                }
            }
            String[] nullPropertyNames = nullProperties.toArray(new String[0]);
            BeanUtil.copyProperties(source, target, nullPropertyNames);
        }
    }
}