package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentBindingUser;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentHonorary;
import com.crunii.micro.service.oneaffair.dto.vo.studentBindingUser.StudentBindingUserCreateVo;
import com.crunii.micro.service.oneaffair.dto.vo.studentBindingUser.StudentBindingUserReq;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentBindingUserJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentBindingUserRepo;
import com.crunii.micro.service.oneaffair.service.StudentBindingUserService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import dm.jdbc.filter.stat.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 学生与用户的绑定表(YCQ_YJS_STUDENT_BINDING_USER)表服务实现类
 */
@Slf4j
@Service
public class StudentBindingUserServiceImpl implements StudentBindingUserService {

    @Resource
    private YcqYjsStudentBindingUserJdbcDao studentBindingUserJdbcDao;
    @Resource
    private YcqYjsStudentBindingUserRepo studentBindingUserRepo;

    @Override
    public YcqYjsStudentBindingUser selectOneById(BigInteger id) {
        try {
            log.info("StudentBindingUserServiceImpl.selectOneById.param：{}", id);
            return studentBindingUserJdbcDao.selectOneById(id);
        } catch (BusinessException e) {
            log.error("StudentBindingUserServiceImpl.selectOneById.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentBindingUserServiceImpl.selectOneById.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public List<YcqYjsStudentBindingUser> selectList(StudentBindingUserReq req) {
        try {
            log.info("StudentBindingUserServiceImpl.selectList.param：{}", req.toString());
            return studentBindingUserJdbcDao.selectList(req);
        } catch (BusinessException e) {
            log.error("StudentBindingUserServiceImpl.selectList.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("StudentBindingUserServiceImpl.selectList.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public YcqYjsStudentBindingUser insertOne(StudentBindingUserCreateVo createReq) {
        log.info("StudentBindingUserServiceImpl.insertOne.param：{}", createReq.toString());
        return studentBindingUserRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentBindingUser.class));
    }

}
