package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.feign.client.StudyCenterService;
import com.crunii.micro.feign.client.dto.TestPaperVO;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentCourse;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourse.*;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentCourseJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentCourseRepo;
import com.crunii.micro.service.oneaffair.service.StudentCourseService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.Date;
import java.util.List;

/**
 * 学生课程表(YCQ_YJS_STUDENT_COURSE)表服务实现类
 */
@Slf4j
@Service
public class StudentCourseServiceImpl implements StudentCourseService {


    @Resource
    private YcqYjsStudentCourseRepo studentCourseRepo;
    @Resource
    private YcqYjsStudentCourseJdbcDao studentCourseJdbcDao;

    @Resource
    private StudyCenterService studyCenterService;

    /**
     * 根据ID查询学生课程
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentCourse selectOneById(BigInteger id) {
        log.info("StudentCourseServiceImpl.selectOneById.param：{}", id);
        return studentCourseRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生课程
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentCourse> selectListByIds(List<BigInteger> idList) {
        log.info("StudentCourseServiceImpl.selectListByIds.param：{}", idList.toString());
        return studentCourseRepo.findAllById(idList);
    }

    /**
     * 分页查询学生课程数据
     *
     * @param req
     * @return
     */
    @Override
    public PageRsp<YcqYjsStudentCourse> pageSearchStudentCourseInfo(PageReq<StudentCourseReq> req) {
        log.info("StudentCourseServiceImpl.pageSearchStudentCourseInfo.param：{}", req.toString());
        return studentCourseJdbcDao.pageSearchStudentCourseInfo(req);
    }

    /**
     * 列表查询学生课程数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentCourse> selectStudentCourseInfoList(StudentCourseReq req) {
        log.info("StudentCourseServiceImpl.selectStudentCourseInfoList.param：{}", req.toString());
        return studentCourseJdbcDao.selectStudentCourseInfoList(req);
    }

    @Override
    public Integer sumCourseDuration(StudentCourseReq req) {
        log.info("StudentCourseServiceImpl.sumCourseDuration.param：{}", req.toString());
        return studentCourseJdbcDao.sumCourseDuration(req);
    }

    /**
     * 单个新增学生课程数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentCourse insertOne(StudentCourseCreateReq createReq) {
        log.info("StudentCourseServiceImpl.insertOne.param：{}", createReq.toString());
        return studentCourseRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentCourse.class));
    }

    /**
     * 批量新增学生课程数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentCourse> insertBatch(List<StudentCourseCreateReq> createReqList) {
        log.info("StudentCourseServiceImpl.insertBatch.param：{}", createReqList.toString());
        return studentCourseRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentCourse.class));
    }

    /**
     * 更新学生课程的上传视频
     * @param req
     */
    @Override
    public void updateStudentCourseVideo(StudentCourseUploadVideoVo req) {
        log.info("StudentCourseServiceImpl.updateStudentCourseVideo.param：{}", req.toString());
        YcqYjsStudentCourse updateEntity = selectOneById(req.getId());
        if (updateEntity != null) {
            BeanUtils.copyProperties(req, updateEntity);
            studentCourseRepo.saveAndFlush(updateEntity);
        }
    }

    /**
     * 更新学生课程视频观看进度
     * @param req
     */
    @Override
    public void studentCourseVideoViewProgress(StudentCourseVideoViewProgressReq req) {
        log.info("StudentCourseServiceImpl.studentCourseVideoViewProgress.param：{}", req.toString());
        YcqYjsStudentCourse updateEntity = selectOneById(req.getStudentCourseId());
        if (updateEntity != null) {
            BeanUtils.copyProperties(req, updateEntity);
            studentCourseRepo.saveAndFlush(updateEntity);
        }
    }

    /**
     * 学生课程提交试卷
     * @param req
     */
    @Override
    public void studentCourseSubmitTest(StudentCourseSubmitTestReq req) {
        log.info("StudentCourseServiceImpl.studentCourseSubmitTest.param：{}", req.toString());
        YcqYjsStudentCourse updateEntity = selectOneById(req.getId());
        if (updateEntity != null) {
            BeanUtils.copyProperties(req, updateEntity);
            studentCourseRepo.saveAndFlush(updateEntity);
        }
    }

    @Override
    public Result<Object> updateStudentCourseById(StudentCourseReq req) {
        log.info("StudentCourseServiceImpl.updateStudentCourseById.param：{}", req.toString());
        List<YcqYjsStudentCourse> list =studentCourseJdbcDao.selectStudentCourseInfoList(req);
        if (CollectionUtils.isEmpty(list)) {
            return Result.ofSuccess("未查询到该用户的考试信息");
        }
        // 查询试卷可考次数
        Result<TestPaperVO> result =studyCenterService.getTestPaperInfo(req.getPaperId().longValue());
        if (!result.isSuccess()) {
            return Result.ofSuccess("查询试卷信息失败");
        }
        TestPaperVO testPaperVO = result.getData();
        if (testPaperVO == null) {
            return Result.ofSuccess("查询试卷信息不存在");
        }
        YcqYjsStudentCourse ycqYjsStudentCourse = new YcqYjsStudentCourse();
        ycqYjsStudentCourse.setStudentId(req.getStudentId());
        ycqYjsStudentCourse.setPaperId(req.getPaperId());
        ycqYjsStudentCourse.setResidueTestCount(testPaperVO.getAllowTimes());
        ycqYjsStudentCourse.setLastUpdatedTime(new Date());
        int i = studentCourseJdbcDao.updateStudentCourseById(ycqYjsStudentCourse);
        return Result.ofSuccess(i);
    }

}
