package com.crunii.micro.service.oneaffair.service.impl;


import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentCourseUploadVideo;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourseUploadVideo.StudentCourseUploadVideoCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourseUploadVideo.StudentCourseUploadVideoDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentCourseUploadVideo.StudentCourseUploadVideoReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentCourseUploadVideoJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentCourseUploadVideoRepo;
import com.crunii.micro.service.oneaffair.service.StudentCourseUploadVideoService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 学生课程上传视频表(YCQ_YJS_STUDENT_COURSE_UPLOAD_VIDEO)服务实现类
 */
@Slf4j
@Service
public class StudentCourseUploadVideoServiceImpl implements StudentCourseUploadVideoService {

    @Resource
    private YcqYjsStudentCourseUploadVideoJdbcDao StudentCourseUploadVideoJdbcDao;
    @Resource
    private YcqYjsStudentCourseUploadVideoRepo StudentCourseUploadVideoRepo;

    /**
     * 根据ID查询学生课程上传视频数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentCourseUploadVideo selectOneById(BigInteger id) {
        log.info("StudentCourseUploadVideoServiceImpl.selectOneById.param：{}", id);
        return StudentCourseUploadVideoRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生课程上传视频数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentCourseUploadVideo> selectListByIds(List<BigInteger> idList) {
        log.info("StudentCourseUploadVideoServiceImpl.selectListByIds.param：{}", idList.toString());
        return StudentCourseUploadVideoRepo.findAllById(idList);
    }

    /**
     * 列表查询学生课程上传视频数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentCourseUploadVideo> selectList(StudentCourseUploadVideoReq req) {
        log.info("StudentCourseUploadVideoServiceImpl.selectList.param：{}", req.toString());
        return StudentCourseUploadVideoJdbcDao.selectList(req);
    }

    /**
     * 统计学生课程上传视频数量
     *
     * @param req
     * @return
     */
    @Override
    public Integer count(StudentCourseUploadVideoReq req) {
        log.info("StudentCourseUploadVideoServiceImpl.count.param：{}", req.toString());
        return StudentCourseUploadVideoJdbcDao.count(req);
    }

    /**
     * 单个新增学生课程上传视频数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentCourseUploadVideo insertOne(StudentCourseUploadVideoCreateReq createReq) {
        log.info("StudentCourseUploadVideoServiceImpl.insertOne.param：{}", createReq.toString());
        return StudentCourseUploadVideoRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentCourseUploadVideo.class));
    }

    /**
     * 批量新增学生课程上传视频数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentCourseUploadVideo> insertBatch(List<StudentCourseUploadVideoCreateReq> createReqList) {
        log.info("StudentCourseUploadVideoServiceImpl.insertBatch.param：{}", createReqList.toString());
        return StudentCourseUploadVideoRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentCourseUploadVideo.class));
    }


    /**
     * 批量删除学生课程上传视频数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentCourseUploadVideoDeleteReq deleteReq) {
        log.info("StudentCourseUploadVideoServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudentCourseUploadVideo> deleteEntityList = StudentCourseUploadVideoRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            StudentCourseUploadVideoRepo.saveAllAndFlush(deleteEntityList);
        }
    }


}