package com.crunii.micro.service.oneaffair.service.impl;


import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentHonorary;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryUpdateReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentHonoraryJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentHonoraryRepo;
import com.crunii.micro.service.oneaffair.service.StudentHonoraryService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 学生荣誉表(YCQ_YJS_STUDENT_HONORARY)服务实现类
 */
@Slf4j
@Service
public class StudentHonoraryServiceImpl implements StudentHonoraryService {

    @Resource
    private YcqYjsStudentHonoraryJdbcDao studentHonoraryJdbcDao;
    @Resource
    private YcqYjsStudentHonoraryRepo studentHonoraryRepo;

    /**
     * 根据ID查询学生荣誉数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentHonorary selectOneById(BigInteger id) {
        log.info("StudentHonoraryServiceImpl.selectOneById.param：{}", id);
        return studentHonoraryRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生荣誉数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentHonorary> selectListByIds(List<BigInteger> idList) {
        log.info("StudentHonoraryServiceImpl.selectListByIds.param：{}", idList.toString());
        return studentHonoraryRepo.findAllById(idList);
    }

    /**
     * 列表查询学生荣誉数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentHonorary> selectList(StudentHonoraryReq req) {
        log.info("StudentHonoraryServiceImpl.selectList.param：{}", req.toString());
        return studentHonoraryJdbcDao.selectList(req);
    }

    /**
     * 统计学生荣誉数量
     *
     * @param req
     * @return
     */
    @Override
    public Integer count(StudentHonoraryReq req) {
        log.info("StudentHonoraryServiceImpl.count.param：{}", req.toString());
        return studentHonoraryJdbcDao.count(req);
    }

    /**
     * 单个新增学生荣誉数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentHonorary insertOne(StudentHonoraryCreateReq createReq) {
        log.info("StudentHonoraryServiceImpl.insertOne.param：{}", createReq.toString());
        return studentHonoraryRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentHonorary.class));
    }

    /**
     * 批量新增学生荣誉数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentHonorary> insertBatch(List<StudentHonoraryCreateReq> createReqList) {
        log.info("StudentHonoraryServiceImpl.insertBatch.param：{}", createReqList.toString());
        return studentHonoraryRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentHonorary.class));
    }

    /**
     * 单个修改学生荣誉数据
     *
     * @param updateReq
     * @return
     */
    @Override
    public void updateOne(StudentHonoraryUpdateReq updateReq) {
        log.info("StudentHonoraryServiceImpl.updateOne.param：{}", updateReq.toString());
        YcqYjsStudentHonorary updateEntity = studentHonoraryRepo.getOne(updateReq.getId());
        if(updateEntity != null){
            BeanUtils.copyProperties(updateReq, updateEntity);
            studentHonoraryRepo.saveAndFlush(updateEntity);
        }
    }

    /**
     * 批量修改学生荣誉数据
     *
     * @param updateReqList
     * @return
     */
    @Override
    public void updateBatch(List<StudentHonoraryUpdateReq> updateReqList) {
        log.info("StudentHonoraryServiceImpl.updateBatch.param：{}", updateReqList.toString());
        if(!CollectionUtils.isEmpty(updateReqList)){
            List<BigInteger> ids = updateReqList.stream().map(StudentHonoraryUpdateReq::getId).collect(Collectors.toList());
            List<YcqYjsStudentHonorary> updateEntityList = studentHonoraryRepo.findAllById(ids);
            if(!CollectionUtils.isEmpty(updateEntityList)){
                Map<BigInteger, StudentHonoraryUpdateReq> studentHonoraryUpdateMap = updateReqList.stream().collect(Collectors.toMap(StudentHonoraryUpdateReq::getId, item -> item));
                updateEntityList.stream().forEach(item->{
                    StudentHonoraryUpdateReq studentHonoraryUpdateReq = studentHonoraryUpdateMap.get(item.getId());
                    if(studentHonoraryUpdateReq != null){
                        BeanUtils.copyProperties(studentHonoraryUpdateReq,item);
                    }
                });
                studentHonoraryRepo.saveAllAndFlush(updateEntityList);
            }
        }
    }

    /**
     * 批量删除学生荣誉数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentHonoraryDeleteReq deleteReq) {
        log.info("StudentHonoraryServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudentHonorary> deleteEntityList = studentHonoraryRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            studentHonoraryRepo.saveAllAndFlush(deleteEntityList);
        }
    }

}