package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentMedalActivity;

import com.crunii.micro.service.oneaffair.dto.vo.studentMedalActivity.StudentMedalActivityCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedalActivity.StudentMedalActivityDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedalActivity.StudentMedalActivityReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedalActivity.StudentMedalActivityUpdateReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentMedalActivityJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentMedalActivityRepo;
import com.crunii.micro.service.oneaffair.service.StudentMedalActivityService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 学生奖章活动完成情况表(YCQ_YJS_STUDENT_MEDAL_ACTIVITY)表服务实现类
 */
@Slf4j
@Service
public class StudentMedalActivityServiceImpl implements StudentMedalActivityService {

    @Resource
    private YcqYjsStudentMedalActivityJdbcDao studentMedalActivityJdbcDao;
    @Resource
    private YcqYjsStudentMedalActivityRepo studentMedalActivityRepo;

    /**
     * 根据ID查询学生奖章活动数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentMedalActivity selectOneById(BigInteger id) {
        log.info("StudentMedalActivityServiceImpl.selectOneById.param：{}", id);
        return studentMedalActivityRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生奖章活动数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentMedalActivity> selectListByIds(List<BigInteger> idList) {
        log.info("StudentMedalActivityServiceImpl.selectListByIds.param：{}", idList.toString());
        return studentMedalActivityRepo.findAllById(idList);
    }

    /**
     * 列表查询学生奖章活动数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentMedalActivity> selectList(StudentMedalActivityReq req) {
        log.info("StudentMedalActivityServiceImpl.selectList.param：{}", req.toString());
        return studentMedalActivityJdbcDao.selectList(req);
    }

    /**
     * 统计学生奖章活动数量
     *
     * @param req
     * @return
     */
    @Override
    public Integer count(StudentMedalActivityReq req) {
        log.info("StudentMedalActivityServiceImpl.count.param：{}", req.toString());
        return studentMedalActivityJdbcDao.count(req);
    }

    /**
     * 统计学生奖章活动数量
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentMedalActivity insertOne(StudentMedalActivityCreateReq createReq) {
        log.info("StudentMedalActivityServiceImpl.insertOne.param：{}", createReq.toString());
        return studentMedalActivityRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentMedalActivity.class));
    }

    /**
     * 批量新增学生奖章活动数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentMedalActivity> insertBatch(List<StudentMedalActivityCreateReq> createReqList) {
        log.info("StudentMedalActivityServiceImpl.insertBatch.param：{}", createReqList.toString());
        return studentMedalActivityRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentMedalActivity.class));
    }

    /**
     * 批量删除学生奖章活动数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentMedalActivityDeleteReq deleteReq) {
        log.info("StudentMedalActivityServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudentMedalActivity> deleteEntityList = studentMedalActivityRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            studentMedalActivityRepo.saveAllAndFlush(deleteEntityList);
        }
    }

    /**
     * 批量修改学生学生奖章活动数据
     *
     * @param updateReq
     */
    @Override
    public void updateBatch(StudentMedalActivityUpdateReq updateReq) {
        log.info("StudentMedalActivityServiceImpl.updateBatch.param：{}", updateReq.toString());
        List<YcqYjsStudentMedalActivity> deleteEntityList = studentMedalActivityRepo.findAllById(updateReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                BeanUtils.copyProperties(updateReq, item);
            });
            studentMedalActivityRepo.saveAllAndFlush(deleteEntityList);
        }

    }

}
