package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentMedalRule;
import com.crunii.micro.service.oneaffair.dto.vo.medalRule.StudentMedalRuleCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.medalRule.StudentMedalRuleDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.medalRule.StudentMedalRuleReq;
import com.crunii.micro.service.oneaffair.dto.vo.medalRule.StudentMedalRuleUpdateReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentMedalRuleJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentMedalRuleRepo;
import com.crunii.micro.service.oneaffair.service.StudentMedalRuleService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 学生答卷表(YCQ_YJS_STUDENT_TEST)表服务实现类
 */
@Slf4j
@Service
public class StudentMedalRuleServiceImpl implements StudentMedalRuleService {

    @Resource
    private YcqYjsStudentMedalRuleJdbcDao studentMedalRuleJdbcDao;
    @Resource
    private YcqYjsStudentMedalRuleRepo studentMedalRuleRepo;

    /**
     * 根据ID查询学生争章规则数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentMedalRule selectOneById(BigInteger id) {
        log.info("StudentMedalRuleServiceImpl.selectOneById.param：{}", id);
        return studentMedalRuleRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生争章规则数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentMedalRule> selectListByIds(List<BigInteger> idList) {
        log.info("StudentMedalRuleServiceImpl.selectListByIds.param：{}", idList.toString());
        return studentMedalRuleRepo.findAllById(idList);
    }

    /**
     * 列表查询学生争章规则数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentMedalRule> selectList(StudentMedalRuleReq req) {
        log.info("StudentMedalRuleServiceImpl.selectList.param：{}", req.toString());
        return studentMedalRuleJdbcDao.selectList(req);
    }

    /**
     * 统计学生争章规则数量
     *
     * @param req
     * @return
     */
    @Override
    public Integer count(StudentMedalRuleReq req) {
        log.info("StudentMedalRuleServiceImpl.count.param：{}", req.toString());
        return studentMedalRuleJdbcDao.count(req);
    }

    /**
     * 统计学生争章规则数量
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentMedalRule insertOne(StudentMedalRuleCreateReq createReq) {
        log.info("StudentMedalRuleServiceImpl.insertOne.param：{}", createReq.toString());
        return studentMedalRuleRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentMedalRule.class));
    }

    /**
     * 批量新增学生争章规则数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentMedalRule> insertBatch(List<StudentMedalRuleCreateReq> createReqList) {
        log.info("StudentMedalRuleServiceImpl.insertBatch.param：{}", createReqList.toString());
        return studentMedalRuleRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentMedalRule.class));
    }

    /**
     * 批量删除学生争章规则数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentMedalRuleDeleteReq deleteReq) {
        log.info("StudentMedalRuleServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudentMedalRule> deleteEntityList = studentMedalRuleRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            studentMedalRuleRepo.saveAllAndFlush(deleteEntityList);
        }
    }

    /**
     * 批量修改学生争章规则数据
     *
     * @param updateReq
     */
    @Override
    public void updateBatch(StudentMedalRuleUpdateReq updateReq) {
        log.info("StudentMedalRuleServiceImpl.updateBatch.param：{}", updateReq.toString());
        List<YcqYjsStudentMedalRule> deleteEntityList = studentMedalRuleRepo.findAllById(updateReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                BeanUtils.copyProperties(updateReq, item);
            });
            studentMedalRuleRepo.saveAllAndFlush(deleteEntityList);
        }
    }

}
