package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentMedal;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.StudentMedalCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.StudentMedalDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.StudentMedalReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentMedal.StudentMedalUpdateReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentMedalJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentMedalRepo;
import com.crunii.micro.service.oneaffair.service.StudentMedalService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 学生答卷表(YCQ_YJS_STUDENT_TEST)表服务实现类
 */
@Slf4j
@Service
public class StudentMedalServiceImpl implements StudentMedalService {

    @Resource
    private YcqYjsStudentMedalJdbcDao studentMedalJdbcDao;
    @Resource
    private YcqYjsStudentMedalRepo studentMedalRepo;

    /**
     * 根据ID查询学生奖章数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentMedal selectOneById(BigInteger id) {
        log.info("StudentMedalServiceImpl.selectOneById.param：{}", id);
        return studentMedalRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生奖章数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentMedal> selectListByIds(List<BigInteger> idList) {
        log.info("StudentMedalServiceImpl.selectListByIds.param：{}", idList.toString());
        return studentMedalRepo.findAllById(idList);
    }

    /**
     * 列表查询学生奖章数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentMedal> selectList(StudentMedalReq req) {
        log.info("StudentMedalServiceImpl.selectList.param：{}", req.toString());
        return studentMedalJdbcDao.selectList(req);
    }

    /**
     * 统计学生奖章数量
     *
     * @param req
     * @return
     */
    @Override
    public Integer count(StudentMedalReq req) {
        log.info("StudentMedalServiceImpl.count.param：{}", req.toString());
        return studentMedalJdbcDao.count(req);
    }

    /**
     * 单个新增学生奖章数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentMedal insertOne(StudentMedalCreateReq createReq) {
        log.info("StudentMedalServiceImpl.insertOne.param：{}", createReq.toString());
        return studentMedalRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentMedal.class));
    }

    /**
     * 批量新增学生奖章数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentMedal> insertBatch(List<StudentMedalCreateReq> createReqList) {
        log.info("StudentMedalServiceImpl.insertBatch.param：{}", createReqList.toString());
        return studentMedalRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentMedal.class));
    }

    /**
     * 批量删除学生奖章数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentMedalDeleteReq deleteReq) {
        log.info("StudentMedalServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudentMedal> deleteEntityList = studentMedalRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            studentMedalRepo.saveAllAndFlush(deleteEntityList);
        }
    }

    /**
     * 单个修改学生奖章数据
     *
     * @param updateReq
     */
    @Override
    public void updateOne(StudentMedalUpdateReq updateReq) {
        log.info("StudentMedalServiceImpl.updateOne.param：{}", updateReq.toString());
        YcqYjsStudentMedal entity = studentMedalRepo.getOne(updateReq.getId());
        if (entity != null) {
            BeanUtils.copyProperties(updateReq, entity);
            studentMedalRepo.saveAndFlush(entity);
        }
    }

}
