package com.crunii.micro.service.oneaffair.service.impl;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentPatriarch;
import com.crunii.micro.service.oneaffair.dto.vo.studentPatriarch.StudentPatriarchCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPatriarch.StudentPatriarchDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPatriarch.StudentPatriarchReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentPatriarchJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentPatriarchRepo;
import com.crunii.micro.service.oneaffair.service.StudentPatriarchService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 学生家长表服务实现类
 */
@Slf4j
@Service
public class StudentPatriarchServiceImpl implements StudentPatriarchService {

    @Resource
    private YcqYjsStudentPatriarchJdbcDao studentPatriarchJdbcDao;
    @Resource
    private YcqYjsStudentPatriarchRepo studentPatriarchRepo;

    /**
     * 根据ID查询学生家长数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentPatriarch selectOneById(BigInteger id) {
        log.info("StudentPatriarchServiceImpl.selectOneById.param：{}", id);
        return studentPatriarchRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生家长数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentPatriarch> selectListByIds(List<BigInteger> idList) {
        log.info("StudentPatriarchServiceImpl.selectListByIds.param：{}", idList.toString());
        return studentPatriarchRepo.findAllById(idList);
    }

    /**
     * 列表查询学生家长数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentPatriarch> selectList(StudentPatriarchReq req) {
        log.info("StudentPatriarchServiceImpl.selectList.param：{}", req.toString());
        return studentPatriarchJdbcDao.selectList(req);
    }

    /**
     * 统计学生家长数量
     *
     * @param req
     * @return
     */
    @Override
    public Integer count(StudentPatriarchReq req) {
        log.info("StudentPatriarchServiceImpl.count.param：{}", req.toString());
        return studentPatriarchJdbcDao.count(req);
    }

    /**
     * 单个新增学生家长数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentPatriarch insertOne(StudentPatriarchCreateReq createReq) {
        log.info("StudentPatriarchServiceImpl.insertOne.param：{}", createReq.toString());
        return studentPatriarchRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentPatriarch.class));
    }

    /**
     * 批量新增学生家长数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentPatriarch> insertBatch(List<StudentPatriarchCreateReq> createReqList) {
        log.info("StudentPatriarchServiceImpl.insertBatch.param：{}", createReqList.toString());
        return studentPatriarchRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentPatriarch.class));
    }

    /**
     * 批量删除学生家长数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentPatriarchDeleteReq deleteReq) {
        log.info("StudentPatriarchServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudentPatriarch> deleteEntityList = studentPatriarchRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            studentPatriarchRepo.saveAllAndFlush(deleteEntityList);
        }
    }

}