package com.crunii.micro.service.oneaffair.service.impl;


import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentHonorary;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentPosition;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryUpdateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.StudentPositionCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.StudentPositionDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.StudentPositionReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentPosition.StudentPositionUpdateReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentPositionJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentPositionRepo;
import com.crunii.micro.service.oneaffair.service.StudentPositionService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 学生岗位表(YCQ_YJS_STUDENT_POSITION)服务实现类
 */
@Slf4j
@Service
public class StudentPositionServiceImpl implements StudentPositionService {

    @Resource
    private YcqYjsStudentPositionJdbcDao studentPositionJdbcDao;
    @Resource
    private YcqYjsStudentPositionRepo studentPositionRepo;

    /**
     * 根据ID查询学生岗位数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentPosition selectOneById(BigInteger id) {
        log.info("StudentPositionServiceImpl.selectOneById.param：{}", id);
        return studentPositionRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生岗位数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentPosition> selectListByIds(List<BigInteger> idList) {
        log.info("StudentPositionServiceImpl.selectListByIds.param：{}", idList.toString());
        return studentPositionRepo.findAllById(idList);
    }

    /**
     * 列表查询学生岗位数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentPosition> selectList(StudentPositionReq req) {
        log.info("StudentPositionServiceImpl.selectList.param：{}", req.toString());
        return studentPositionJdbcDao.selectList(req);
    }

    /**
     * 统计学生岗位数量
     *
     * @param req
     * @return
     */
    @Override
    public Integer count(StudentPositionReq req) {
        log.info("StudentPositionServiceImpl.count.param：{}", req.toString());
        return studentPositionJdbcDao.count(req);
    }

    /**
     * 单个新增学生岗位数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentPosition insertOne(StudentPositionCreateReq createReq) {
        log.info("StudentPositionServiceImpl.insertOne.param：{}", createReq.toString());
        return studentPositionRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentPosition.class));
    }

    /**
     * 批量新增学生岗位数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentPosition> insertBatch(List<StudentPositionCreateReq> createReqList) {
        log.info("StudentPositionServiceImpl.insertBatch.param：{}", createReqList.toString());
        return studentPositionRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentPosition.class));
    }

    /**
     * 批量删除学生岗位数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentPositionDeleteReq deleteReq) {
        log.info("StudentPositionServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudentPosition> deleteEntityList = studentPositionRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            studentPositionRepo.saveAllAndFlush(deleteEntityList);
        }
    }

    /**
     * 单个修改学生岗位数据
     *
     * @param updateReq
     * @return
     */
    @Override
    public void updateOne(StudentPositionUpdateReq updateReq) {
        log.info("StudentPositionServiceImpl.updateOne.param：{}", updateReq.toString());
        YcqYjsStudentPosition updateEntity = studentPositionRepo.getOne(updateReq.getId());
        if(updateEntity != null){
            BeanUtils.copyProperties(updateReq, updateEntity);
            studentPositionRepo.saveAndFlush(updateEntity);
        }
    }

    /**
     * 批量修改学生岗位数据
     *
     * @param updateReqList
     * @return
     */
    @Override
    public void updateBatch(List<StudentPositionUpdateReq> updateReqList) {
        log.info("StudentPositionServiceImpl.updateBatch.param：{}", updateReqList.toString());
        if(!CollectionUtils.isEmpty(updateReqList)){
            List<BigInteger> ids = updateReqList.stream().map(StudentPositionUpdateReq::getId).collect(Collectors.toList());
            List<YcqYjsStudentPosition> updateEntityList = studentPositionRepo.findAllById(ids);
            if(!CollectionUtils.isEmpty(updateEntityList)){
                Map<BigInteger, StudentPositionUpdateReq> updateMap = updateReqList.stream().collect(Collectors.toMap(StudentPositionUpdateReq::getId, item -> item));
                updateEntityList.stream().forEach(item->{
                    StudentPositionUpdateReq updateReq = updateMap.get(item.getId());
                    if(updateReq != null){
                        BeanUtils.copyProperties(updateReq,item);
                    }
                });
                studentPositionRepo.saveAllAndFlush(updateEntityList);
            }
        }
    }


}