package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentQuestion;
import com.crunii.micro.service.oneaffair.dto.vo.studentQuestion.StudentQuestionCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentQuestion.StudentQuestionDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentQuestion.StudentQuestionReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentQuestionJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentQuestionRepo;
import com.crunii.micro.service.oneaffair.service.StudentQuestionService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 学生答卷表(YCQ_YJS_STUDENT_TEST)表服务实现类
 */
@Slf4j
@Service
public class StudentQuestionServiceImpl implements StudentQuestionService {

    @Resource
    private YcqYjsStudentQuestionJdbcDao studentQuestionJdbcDao;
    @Resource
    private YcqYjsStudentQuestionRepo studentQuestionRepo;

    /**
     * 根据ID查询学生答题数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentQuestion selectOneById(BigInteger id) {
        log.info("StudentQuestionServiceImpl.selectOneById.param：{}", id);
        return studentQuestionRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生答题数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentQuestion> selectListByIds(List<BigInteger> idList) {
        log.info("StudentQuestionServiceImpl.selectListByIds.param：{}", idList.toString());
        return studentQuestionRepo.findAllById(idList);
    }

    /**
     * 列表查询学生答题数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentQuestion> selectList(StudentQuestionReq req) {
        log.info("StudentQuestionServiceImpl.selectList.param：{}", req.toString());
        return studentQuestionJdbcDao.selectList(req);
    }

    /**
     * 单个新增学生答题数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentQuestion insertOne(StudentQuestionCreateReq createReq) {
        log.info("StudentQuestionServiceImpl.insertOne.param：{}", createReq.toString());
        return studentQuestionRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentQuestion.class));
    }

    /**
     * 批量新增学生答题数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentQuestion> insertBatch(List<StudentQuestionCreateReq> createReqList) {
        log.info("StudentQuestionServiceImpl.insertBatch.param：{}", createReqList.toString());
        return studentQuestionRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentQuestion.class));
    }

    /**
     * 批量删除学生答题数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentQuestionDeleteReq deleteReq) {
        log.info("StudentQuestionServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudentQuestion> deleteEntityList = studentQuestionRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            studentQuestionRepo.saveAllAndFlush(deleteEntityList);
        }
    }

}
