package com.crunii.micro.service.oneaffair.service.impl;


import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentHonorary;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentSubCourse;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentHonorary.StudentHonoraryUpdateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentSubCourse.StudentSubCourseCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentSubCourse.StudentSubCourseDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentSubCourse.StudentSubCourseReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentSubCourse.StudentSubCourseUpdateReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentHonoraryJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentHonoraryRepo;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentSubCourseJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentSubCourseRepo;
import com.crunii.micro.service.oneaffair.service.StudentHonoraryService;
import com.crunii.micro.service.oneaffair.service.StudentSubCourseService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 学生子课程表(YCQ_YJS_STUDENT_SUB_COURSE)服务实现类
 */
@Slf4j
@Service
public class StudentSubCourseServiceImpl implements StudentSubCourseService {

    @Resource
    private YcqYjsStudentSubCourseJdbcDao StudentSubCourseJdbcDao;
    @Resource
    private YcqYjsStudentSubCourseRepo StudentSubCourseRepo;

    /**
     * 根据ID查询学生子课程数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentSubCourse selectOneById(BigInteger id) {
        log.info("StudentSubCourseServiceImpl.selectOneById.param：{}", id);
        return StudentSubCourseRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生子课程数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentSubCourse> selectListByIds(List<BigInteger> idList) {
        log.info("StudentSubCourseServiceImpl.selectListByIds.param：{}", idList.toString());
        return StudentSubCourseRepo.findAllById(idList);
    }

    /**
     * 列表查询学生子课程数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentSubCourse> selectList(StudentSubCourseReq req) {
        log.info("StudentSubCourseServiceImpl.selectList.param：{}", req.toString());
        return StudentSubCourseJdbcDao.selectList(req);
    }

    /**
     * 统计学生子课程数量
     *
     * @param req
     * @return
     */
    @Override
    public Integer count(StudentSubCourseReq req) {
        log.info("StudentSubCourseServiceImpl.count.param：{}", req.toString());
        return StudentSubCourseJdbcDao.count(req);
    }

    /**
     * 单个新增学生子课程数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentSubCourse insertOne(StudentSubCourseCreateReq createReq) {
        log.info("StudentSubCourseServiceImpl.insertOne.param：{}", createReq.toString());
        return StudentSubCourseRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentSubCourse.class));
    }

    /**
     * 批量新增学生子课程数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentSubCourse> insertBatch(List<StudentSubCourseCreateReq> createReqList) {
        log.info("StudentSubCourseServiceImpl.insertBatch.param：{}", createReqList.toString());
        return StudentSubCourseRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentSubCourse.class));
    }

    /**
     * 单个修改学生子课程数据
     *
     * @param updateReq
     * @return
     */
    @Override
    public void updateOne(StudentSubCourseUpdateReq updateReq) {
        log.info("StudentSubCourseServiceImpl.updateOne.param：{}", updateReq.toString());
        YcqYjsStudentSubCourse updateEntity = StudentSubCourseRepo.getOne(updateReq.getId());
        if(updateEntity != null){
            BeanUtils.copyProperties(updateReq, updateEntity);
            StudentSubCourseRepo.saveAndFlush(updateEntity);
        }
    }

    /**
     * 批量修改学生子课程数据
     *
     * @param updateReqList
     * @return
     */
    @Override
    public void updateBatch(List<StudentSubCourseUpdateReq> updateReqList) {
        log.info("StudentSubCourseServiceImpl.updateBatch.param：{}", updateReqList.toString());
        if(!CollectionUtils.isEmpty(updateReqList)){
            List<BigInteger> ids = updateReqList.stream().map(StudentSubCourseUpdateReq::getId).collect(Collectors.toList());
            List<YcqYjsStudentSubCourse> updateEntityList = StudentSubCourseRepo.findAllById(ids);
            if(!CollectionUtils.isEmpty(updateEntityList)){
                Map<BigInteger, StudentSubCourseUpdateReq> StudentSubCourseUpdateMap = updateReqList.stream().collect(Collectors.toMap(StudentSubCourseUpdateReq::getId, item -> item));
                updateEntityList.stream().forEach(item->{
                    StudentSubCourseUpdateReq StudentSubCourseUpdateReq = StudentSubCourseUpdateMap.get(item.getId());
                    if(StudentSubCourseUpdateReq != null){
                        BeanUtils.copyProperties(StudentSubCourseUpdateReq,item);
                    }
                });
                StudentSubCourseRepo.saveAllAndFlush(updateEntityList);
            }
        }
    }

    /**
     * 批量删除学生子课程数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentSubCourseDeleteReq deleteReq) {
        log.info("StudentSubCourseServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudentSubCourse> deleteEntityList = StudentSubCourseRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            StudentSubCourseRepo.saveAllAndFlush(deleteEntityList);
        }
    }

}