package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsStudentTest;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.StudentTestCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.StudentTestDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.StudentTestReq;
import com.crunii.micro.service.oneaffair.dto.vo.studentTest.StudentTestUpdateReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentTestJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsStudentTestRepo;
import com.crunii.micro.service.oneaffair.service.StudentTestService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 学生答卷表(YCQ_YJS_STUDENT_TEST)表服务实现类
 */
@Slf4j
@Service
public class StudentTestServiceImpl implements StudentTestService {

    @Resource
    private YcqYjsStudentTestJdbcDao studentTestJdbcDao;
    @Resource
    private YcqYjsStudentTestRepo studentTestRepo;

    /**
     * 根据ID查询学生答卷数据
     *
     * @param id
     * @return
     */
    @Override
    public YcqYjsStudentTest selectOneById(BigInteger id) {
        log.info("StudentTestServiceImpl.selectOneById.param：{}", id);
        return studentTestRepo.getOne(id);
    }

    /**
     * 根据ID列表查询学生答卷数据
     *
     * @param idList
     * @return
     */
    @Override
    public List<YcqYjsStudentTest> selectListByIds(List<BigInteger> idList) {
        log.info("StudentTestServiceImpl.selectListByIds.param：{}", idList.toString());
        return studentTestRepo.findAllById(idList);
    }

    /**
     * 列表查询学生答卷数据
     *
     * @param req
     * @return
     */
    @Override
    public List<YcqYjsStudentTest> selectList(StudentTestReq req) {
        log.info("StudentTestServiceImpl.selectList.param：{}", req.toString());
        return studentTestJdbcDao.selectList(req);
    }

    /**
     * 查询最高分数学生答卷数据
     *
     * @param req
     * @return
     */
    @Override
    public YcqYjsStudentTest selectMaxStudentTest(StudentTestReq req) {
        log.info("StudentTestServiceImpl.selectMaxStudentTest.param：{}", req.toString());
        return studentTestJdbcDao.selectMaxStudentTest(req);
    }

    /**
     * 分页查询学生答卷
     * @param req
     * @return
     */
    @Override
    public PageRsp<YcqYjsStudentTest> pageSearchStudentTest(PageReq<StudentTestReq> req) {
        log.info("StudentTestServiceImpl.pageSearchStudentTest.param：{}", req.toString());
        return studentTestJdbcDao.pageSearchStudentTest(req);
    }

    /**
     * 统计学生答卷数量
     *
     * @param req
     * @return
     */
    @Override
    public Integer count(StudentTestReq req) {
        log.info("StudentTestServiceImpl.count.param：{}", req.toString());
        return studentTestJdbcDao.count(req);
    }

    /**
     * 单个新增学生答卷数据
     *
     * @param createReq
     * @return
     */
    @Override
    public YcqYjsStudentTest insertOne(StudentTestCreateReq createReq) {
        log.info("StudentTestServiceImpl.insertOne.param：{}", createReq.toString());
        return studentTestRepo.save(BeanCopyUtil.convert(createReq, YcqYjsStudentTest.class));
    }

    /**
     * 批量新增学生答卷数据
     *
     * @param createReqList
     * @return
     */
    @Override
    public List<YcqYjsStudentTest> insertBatch(List<StudentTestCreateReq> createReqList) {
        log.info("StudentTestServiceImpl.insertBatch.param：{}", createReqList.toString());
        return studentTestRepo.saveAll(BeanCopyUtil.convert(createReqList, YcqYjsStudentTest.class));
    }

    /**
     * 批量删除学生答卷数据
     *
     * @param deleteReq
     */
    @Override
    public void deleteBatch(StudentTestDeleteReq deleteReq) {
        log.info("StudentTestServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsStudentTest> deleteEntityList = studentTestRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                BeanUtils.copyProperties(deleteReq, item);
            });
            studentTestRepo.saveAllAndFlush(deleteEntityList);
        }
    }

    /**
     * 单个修改学生答卷数据
     *
     * @param updateReq
     */
    @Override
    public YcqYjsStudentTest updateOne(StudentTestUpdateReq updateReq) {
        log.info("StudentTestServiceImpl.updateOne.param：{}", updateReq.toString());
        YcqYjsStudentTest entity = studentTestRepo.getOne(updateReq.getId());
        if (entity != null) {
            BeanUtils.copyProperties(updateReq, entity);
            entity = studentTestRepo.saveAndFlush(entity);
        }
        return entity;
    }


}
