package com.crunii.micro.service.oneaffair.service.impl;

import com.crunii.micro.service.oneaffair.dto.oneaffair.YcqYjsTeacherBindingClass;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.TeacherBindingClassCreateReq;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.TeacherBindingClassDeleteReq;
import com.crunii.micro.service.oneaffair.dto.vo.teacher.TeacherBindingClassReq;
import com.crunii.micro.service.oneaffair.enums.DelFlagEnum;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsTeacherBindingClassJdbcDao;
import com.crunii.micro.service.oneaffair.jdbc.YcqYjsTeacherBindingClassRepo;
import com.crunii.micro.service.oneaffair.service.TeacherBindingClassService;
import com.crunii.micro.service.oneaffair.util.BeanCopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 教师与班级的绑定表(YCQ_YJS_TEACHER_BINDING_ClASS)表服务实现类
 */
@Slf4j
@Service
public class TeacherBindingClassServiceImpl implements TeacherBindingClassService {

    @Resource
    private YcqYjsTeacherBindingClassJdbcDao teacherBindingClassJdbcDao;
    @Resource
    private YcqYjsTeacherBindingClassRepo teacherBindingClassRepo;


    @Override
    public List<BigInteger> selectTeacherIds(TeacherBindingClassReq req) {
        log.info("TeacherBindingClassServiceImpl.selectTeacherIds.param：{}", req.toString());
        return teacherBindingClassJdbcDao.selectTeacherIds(req);
    }

    @Override
    public void insertBatch(List<TeacherBindingClassCreateReq> createList) {
        log.info("TeacherBindingClassServiceImpl.insertBatch.param：{}", createList.toString());
        teacherBindingClassRepo.saveAll(BeanCopyUtil.convert(createList, YcqYjsTeacherBindingClass.class));
    }

    @Override
    public List<BigInteger> selectIdListByTeacherIds(List<BigInteger> teacherIds) {
        log.info("TeacherBindingClassServiceImpl.selectIdListByTeacherIds.param：{}", teacherIds.toString());
        return teacherBindingClassJdbcDao.selectIdListByTeacherIds(teacherIds);
    }

    @Override
    public List<YcqYjsTeacherBindingClass> selectListByTeacherIds(List<BigInteger> teacherIds) {
        log.info("TeacherBindingClassServiceImpl.selectIdListByTeacherIds.param：{}", teacherIds.toString());
        return teacherBindingClassJdbcDao.selectListByTeacherIds(teacherIds);
    }

    @Override
    public void clearBatch(List<BigInteger> idList) {
        log.info("TeacherBindingClassServiceImpl.clearBatch.param：{}", idList.toString());
        teacherBindingClassJdbcDao.clearBatch(idList);
    }

    @Override
    public void deleteBatch(TeacherBindingClassDeleteReq deleteReq) {
        log.info("TeacherBindingClassServiceImpl.deleteBatch.param：{}", deleteReq.toString());
        List<YcqYjsTeacherBindingClass> deleteEntityList = teacherBindingClassRepo.findAllById(deleteReq.getIdList());
        if (!CollectionUtils.isEmpty(deleteEntityList)) {
            deleteEntityList.stream().forEach(item -> {
                item.setDelFlag(DelFlagEnum.DELETED.code());
                item.setLastUpdatedId(deleteReq.getLastUpdatedId());
                item.setLastUpdatedTime(deleteReq.getLastUpdatedTime());
            });
            teacherBindingClassRepo.saveAllAndFlush(deleteEntityList);
        }
    }

}
