package com.crunii.micro.service.oneaffair.util;

import com.google.common.collect.Lists;
import io.netty.util.internal.StringUtil;
import org.springframework.util.CollectionUtils;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class CommonUtil {

    /**
     * 将数组转化成字符串
     *
     * @param list
     * @return
     */
    public static String fromListToStr(List<BigInteger> list) {
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.toString().replace("[", "(").replace("]", ")").replace(" ", "").trim();
    }

    /**
     * 将数组转化成字符串
     *
     * @param list
     * @return
     */
    public static String fromStrListToStr1(List<String> list) {
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.toString().replace("[", "(").replace("]", ")").replace(" ", "").trim();
    }

    /**
     * 将数组转化成字符串
     *
     * @param list
     * @return
     */
    public static String fromStrListToStr2(List<String> list) {
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        StringBuilder sql = new StringBuilder(" ( ");
        int count = 0;
        for (String str : list) {
            if(count == 0){
                sql.append("'" + str + "'");
            }else{
                sql.append(" , '" + str + "'");
            }
            count++;
        }
        sql.append(" ) ");
        return sql.toString();
    }

    /**
     * 将数组转化成字符串
     *
     * @param list
     * @return
     */
    public static String fromListToStr2(List<Long> list) {
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.toString().replace("[", "(").replace("]", ")").replace(" ", "").trim();
    }

    /**
     * 将数组转化成字符串
     *
     * @param list
     * @return
     */
    public static String fromStrListToStr(List<String> list) {
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.toString().replace("[", "").replace("]", "").replace(" ", "").trim();
    }

    /**
     * 将数组转化成字符串
     *
     * @param list
     * @return
     */
    public static String transferListToStr(List<BigInteger> list) {
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        list = list.stream().sorted().collect(Collectors.toList());
        return list.toString().replace("[", "").replace("]", "").replace(" ", "").trim();
    }

    /**
     * 将字符串转化成数组
     *
     * @param listStr
     * @return
     */
    public static List<BigInteger> transferStrToList(String listStr) {
        if (StringUtil.isNullOrEmpty(listStr)) {
            return null;
        }
        String[] split = listStr.split(",");
        if (split == null || split.length == 0) {
            return null;
        }
        List<BigInteger> list = new ArrayList<>();
        for (String item : split) {
            list.add(new BigInteger(item.trim()));
        }
        list = list.stream().sorted().collect(Collectors.toList());
        return list;
    }

    /**
     * 将字符串转化成数组
     *
     * @param listStr
     * @return
     */
    public static List<String> transferStrToStrList(String listStr) {
        if (StringUtil.isNullOrEmpty(listStr)) {
            return Lists.newArrayList();
        }
        String[] split = listStr.split(",");
        if (split == null || split.length == 0) {
            return Lists.newArrayList();
        }
        List<String> list = new ArrayList<>();
        for (String item : split) {
            list.add(item.trim());
        }
        return list;
    }

    /**
     * 身份证验证
     * 身份证号码是由18位数字和最后一位可能是字母X组成的，其中前17位为地区、出生年月日和顺序号，最后一位为校验码
     * @param idCard
     * @return
     */
    public static boolean isIDCard(String idCard) {
        if(StringUtil.isNullOrEmpty(idCard)){
            return false;
        }
        String regex = "^\\d{17}[\\d|X|x]$";
        if (!idCard.matches(regex)) {
            return false;
        }
        int[] weights = {7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2};
        String[] codes = {"1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2"};
        int sum = 0;
        for (int i = 0; i < weights.length; i++) {
            int num = Integer.parseInt(idCard.substring(i, i + 1));
            sum += num * weights[i];
        }
        int mod = sum % 11;
        String code = codes[mod];
        String last = idCard.substring(17).toUpperCase();
        return last.equals(code);
    }


    public static boolean ifMobile(String mobile) {
        if(StringUtil.isNullOrEmpty(mobile)){
            return false;
        }
        String regex = "[1][3456789][0-9]{9}";
        return mobile.matches(regex);
    }
}
