package com.crunii.micro.service.oneaffair.util;

import jodd.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.Date;


/**
 * @author zhoujiabo
 * @date 2020/06/28
 * 时间工具类
 */
@Slf4j
@Service
public class DateUtil {
    private static final String CHINESE_DATE_FORMAT_STRING = "yyyy年MM月dd日";
    public static final String DATE_FORMAT_STRING = "yyyy-MM-dd";
    public static final String DATE_FORMAT_STRING_MONTH = "yyyy-MM";
    public static final String TIME_FORMAT_STRING = "yyyy-MM-dd HH:mm:ss";
    private static final String DATE_YEAR = "yyyy";
    public static final String DATE_FORMAT_YYYYMMDD = "yyyyMMdd";
    public static final String DATE_FORMAT_YYYYMMDDHHMMSSS = "yyyyMMddHHmmss";
    public static final String DATE_FORMAT_YYYYMM = "yyyyMM";
    public static final String DATE_FORMAT_MM = "MM";
    public static final String DATE_FORMAT_YYYY = "yyyy";
    private static final String DATE_DAY = "dd";
    private static final String DATE_MONTH_DAY = "MM-dd";
    public static final String DATE_YEAR_MONTH = "yyyy年MM月";
    public static final String DATE_YEAR_MONTH_DAY = "yyyy年MM月dd日";
    public static final String DATE_MONTH_DAY_1 = "MM月dd日";

    /**
     * 获取当月第几天的00:00:00
     *
     * @param date
     * @return
     */
    public static Date getDayByThisMonth(Date date, int days) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, 0);
        calendar.set(Calendar.DAY_OF_MONTH, days);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        return calendar.getTime();
    }

    /**
     * 校验时间格式是否为 yyyy-MM-dd
     *
     * @param date
     * @return
     */
    public static Boolean isDateVail(String date) {
        //用于指定 日期/时间 模式
        DateTimeFormatter dtf = DateTimeFormatter.ofPattern(CHINESE_DATE_FORMAT_STRING);
        boolean flag = true;
        try {
            //Java 8 新添API 用于解析日期和时间
            LocalDateTime.parse(date, dtf);
        } catch (Exception e) {
            flag = false;
        }
        return flag;
    }

    /*
     * return current time example "2021-06-23 20:17:04"
     * */
    public static String getCurrentTime() {
        return DateFormatUtils.format(new Date(), TIME_FORMAT_STRING);
    }

    /*
     * return current time example "20210618"
     * */
    public static String getCurrentDate() {
        return DateFormatUtils.format(new Date(), DATE_FORMAT_YYYYMMDD);
    }

    public static String getCurrentMonthDate() {
        return DateFormatUtils.format(new Date(), DATE_FORMAT_YYYYMM);
    }

    public static String getDateStr(Date date, String format) {
        return DateFormatUtils.format(date, format);
    }

    /**
     * 获得之前月份的最后一天
     *
     * @param year
     * @param month
     * @return
     */
    public static Date getLastDayOfMonth(int year, int month) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.MONTH, month - 1);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        return calendar.getTime();
    }

    /**
     * 获取前n天
     */
    public static Date getPrevDate(int days) {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE, Math.negateExact(Math.abs(days)));
        return calendar.getTime();
    }

    /**
     * 获取前n月
     */
    public static Date getPrevDateByMonth(int months, Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, Math.negateExact(Math.abs(months)));
        return calendar.getTime();
    }


    /**
     * 获得当前时间年份
     *
     * @return
     */
    public static Integer getSysYear() {
        int year = Calendar.getInstance().get(Calendar.YEAR);
        return year;
    }

    /**
     * 获取日期字符串：yyyy-MM
     *
     * @param date 日期
     * @return
     */
    public static String getMonthDateStr(final Date date) {
        return date != null ? DateFormatUtils.format(date, DATE_FORMAT_STRING_MONTH) : "";
    }

    /**
     * 获取日期字符串：yyyy-MM-dd
     *
     * @param date 日期
     * @return
     */
    public static String getDateStr(final Date date) {
        return date != null ? DateFormatUtils.format(date, DATE_FORMAT_STRING) : "";
    }

    /**
     * 获取时间字符串：yyyy-MM-dd HH:mm:ss
     *
     * @param date
     * @return
     */
    public static String getTimeStr(final Date date) {
        return date != null ? DateFormatUtils.format(date, TIME_FORMAT_STRING) : "";
    }

    /**
     * 获取日期年份：yyyy
     *
     * @param date
     * @return
     */
    public static String getYear(final Date date) {
        return DateFormatUtils.format(date, DATE_YEAR);
    }

    /**
     * 获取日期 当前日期几号
     *
     * @param date
     * @return
     */
    public static String getDay(final Date date) {
        return DateFormatUtils.format(date, DATE_DAY);
    }

    /**
     * 获取日期 当前月日
     *
     * @param date
     * @return
     */
    public static String getMonthDay(final Date date) {
        return DateFormatUtils.format(date, DATE_MONTH_DAY);
    }

    /**
     * 获取日期中文格式：yyyy年MM月dd日
     *
     * @param date
     * @return
     */
    public static String getChineseDate(final Date date) {
        return DateFormatUtils.format(date, CHINESE_DATE_FORMAT_STRING);
    }

    /**
     * 获取指定日期上个月的最后一天，
     *
     * @param date 如date不传则默认当前
     * @return
     */
    public static Date getLastDayOfPreMonthSecond(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date != null ? date : new Date());
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        return calendar.getTime();
    }


    /**
     * 获取获取当前月的第一天
     *
     * @return
     */
    public static Date getFirstDayCurrentMonth() {
        Date date = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);//指定时间
        cal.set(Calendar.DAY_OF_MONTH, 1);
        return getThisDateStartTimeByAssignDate(cal.getTime());
    }

    /**
     * 获取获取当前月的第一天
     *
     * @return
     */
    public static Date getFirstDayCurrentMonthSecond() {
        Date date = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);//指定时间
        cal.set(Calendar.DAY_OF_MONTH, 1);
        //将小时至00
        cal.set(Calendar.HOUR_OF_DAY, 0);
        //将分钟至00
        cal.set(Calendar.MINUTE, 0);
        //将秒至00
        cal.set(Calendar.SECOND, 0);
        return getThisDateStartTimeByAssignDate(cal.getTime());
    }

    /**
     * 获取当月的最后一天
     *
     * @return
     */
    public static Date getLastDayCurrentMonth() {
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date != null ? date : new Date());
        calendar.add(Calendar.MONTH, 1);
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        return calendar.getTime();
    }

    /**
     * 获取当月的最后一天
     *
     * @return
     */
    public static Date getLastDayCurrentMonthSecond(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date != null ? date : new Date());
        calendar.add(Calendar.MONTH, 1);
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        return calendar.getTime();
    }

    /**
     * 获取指定日期对应月的最后一天
     *
     * @param date 如date不传则默认当前
     * @return
     */
    public static Date getLastDayOfDate(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date != null ? date : new Date());
        calendar.add(Calendar.MONTH, 1);
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        return calendar.getTime();
    }

    /**
     * 字符串转日期
     *
     * @param dateStr
     * @param parttern
     * @return
     */
    public static Date parse(String dateStr, String parttern) {
        try {
            if(StringUtil.isBlank(dateStr)){
                return null;
            }
            return DateUtils.parseDate(dateStr, parttern);
        } catch (ParseException e) {
            return null;
        }
    }

    /**
     * 获取两个日期中较大日期
     *
     * @param date1
     * @param date2
     * @return
     */
    public static Date getMaxDate(Date date1, Date date2) {
        return date1.getTime() >= date2.getTime() ? date1 : date2;
    }


    /**
     * 获取当天日期
     *
     * @return
     */
    public static Date getNowDate() {
        return new Date();
    }


    /**
     * 取得给定字符串描述的日期对象，描述模式采用pattern指定的格式.
     *
     * @param dateStr 日期描述
     * @param pattern 日期模式
     * @return 给定字符串描述的日期对象。
     */
    public static Date getDateFromString(String dateStr, String pattern) {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        Date resDate = null;
        try {
            resDate = sdf.parse(dateStr);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return resDate;
    }

    /**
     * JAVA计算两个日期相差多少天(by date)
     *
     * @author zhengkai.blog.csdn.net
     */
    public static int daysBetween(Date date1, Date date2) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date1);
        long time1 = cal.getTimeInMillis();
        cal.setTime(date2);
        long time2 = cal.getTimeInMillis();
        long between_days = (time2 - time1) / (1000 * 3600 * 24);
        return Integer.parseInt(String.valueOf(between_days));
    }

    /**
     * 获得指定时间的本周的第一天
     *
     * @return
     */
    public static Date getWeekDayStartTimeByAssignDate(Date assignDate) {
        Calendar c = Calendar.getInstance();
        c.setTime(assignDate);//设置起时间
        try {
            int weekday = c.get(Calendar.DAY_OF_WEEK) - 2;
            c.add(Calendar.DATE, -weekday);
            SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_STRING);
            SimpleDateFormat longSdf = new SimpleDateFormat(TIME_FORMAT_STRING);
            c.setTime(longSdf.parse(sdf.format(c.getTime()) + " 00:00:00"));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return c.getTime();
    }

    /**
     * 获取当前时间的00:00:00
     *
     * @param assignDate
     * @return
     */
    public static Date getThisDateStartTimeByAssignDate(Date assignDate) {
        if(assignDate == null){
           return null;
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取当前时间的23:59:59
     *
     * @param assignDate
     * @return
     */
    public static Date getThisDateEndTimeByAssignDate(Date assignDate) {
        if(assignDate == null){
            return null;
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取获取上周的第一天
     *
     * @param assignDate
     * @return
     */
    public static Date getFirstDayOfLastWeekByAssignDate(Date assignDate) {
        Calendar cal = Calendar.getInstance();
        cal.setFirstDayOfWeek(Calendar.MONDAY);//以周一为首日
        cal.setTime(assignDate);//当前时间
        cal.add(Calendar.WEDNESDAY, -1);
        cal.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);//周一
        return getThisDateStartTimeByAssignDate(cal.getTime());
    }

    /**
     * 获取上周的最后一天
     *
     * @param assignDate
     * @return
     */
    public static Date getLastDayOfLastWeekByAssignDate(Date assignDate) {
        Calendar cal = Calendar.getInstance();
        cal.setFirstDayOfWeek(Calendar.MONDAY);//以周一为首日
        cal.setTime(assignDate);//当前时间
        cal.add(Calendar.WEDNESDAY, -1);
        cal.set(Calendar.DAY_OF_WEEK, Calendar.SUNDAY);//周日
        return getThisDateEndTimeByAssignDate(cal.getTime());
    }

    /**
     * 获取获取上个月的第一天
     *
     * @param assignDate
     * @return
     */
    public static Date getFirstDayOfLastMonthByAssignDate(Date assignDate) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(assignDate);//指定时间
        cal.add(Calendar.MONTH, -1);
        cal.set(Calendar.DAY_OF_MONTH, 1);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        return getThisDateStartTimeByAssignDate(cal.getTime());
    }

    /**
     * 获取上个月的最后一天
     *
     * @param assignDate
     * @return
     */
    public static Date getLastDayOfLastMonthByAssignDate(Date assignDate) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(assignDate);//指定时间
        cal.add(Calendar.MONTH, 0);
        cal.set(Calendar.DAY_OF_MONTH, 0);
        cal.set(Calendar.HOUR_OF_DAY, 23);
        cal.set(Calendar.MINUTE, 59);
        cal.set(Calendar.SECOND, 59);
        return getThisDateEndTimeByAssignDate(cal.getTime());
    }

    /**
     * 获取获取当前月的第一天
     *
     * @param assignDate
     * @return
     */
    public static Date getFirstDayOfMonthByAssignDate(Date assignDate) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(assignDate);//指定时间
        cal.set(Calendar.DAY_OF_MONTH, 1);
        return getThisDateStartTimeByAssignDate(cal.getTime());
    }

    /**
     * 增加年
     *
     * @param assignDate
     * @return
     */
    public static Date addYear(Date assignDate, int years) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);
        calendar.add(Calendar.YEAR, years);
        return calendar.getTime();
    }

    /**
     * 增加年
     *
     * @param assignDate
     * @return
     */
    public static Date addMonth(Date assignDate, int month) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);
        calendar.add(Calendar.MONTH, month);
        return calendar.getTime();
    }

    /**
     * 增加天
     *
     * @param assignDate
     * @return
     */
    public static Date addDay(Date assignDate, int days) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);
        calendar.add(Calendar.DAY_OF_MONTH, days);
        return calendar.getTime();
    }

    /**
     * 增加小时
     *
     * @param assignDate
     * @return
     */
    public static Date addHours(Date assignDate, int hours) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);
        calendar.add(Calendar.HOUR, hours);
        return calendar.getTime();
    }

    /**
     * 判断该日期是否是该月的第一天
     *
     * @param date 需要判断的日期
     * @return
     */
    public static boolean ifFirstDayOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        System.out.println(calendar.get(Calendar.MONTH));
        return calendar.get(Calendar.DAY_OF_MONTH) == 1;
    }

    /**
     * 获取获取当前月的第一天(设置毫秒数为0)
     *
     * @param assignDate
     * @return
     */
    public static Date getFirstDayOfMonthByAssignDateV2(Date assignDate) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(assignDate);//指定时间
        cal.set(Calendar.DAY_OF_MONTH, 1);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime();
    }


    /**
     * 获取获取当前月的最后一天
     *
     * @param assignDate
     * @return
     */
    public static Date getLastDayOfMonthByAssignDate(Date assignDate) {
        Calendar c = Calendar.getInstance();
        c.setTime(assignDate);//指定时间
        int month = c.get(c.MONTH) + 1;
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);//指定时间
        calendar.set(Calendar.MONTH, month);
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        return getThisDateEndTimeByAssignDate(calendar.getTime());
    }

    /**
     * 获取当前季度
     */
    public static Integer getQuarterByAssignDate(Date assignDate) {
        Calendar c = Calendar.getInstance();
        c.setTime(assignDate);//指定时间
        int month = c.get(c.MONTH) + 1;
        int quarter = 0;
        if (month >= 1 && month <= 3) {
            quarter = 1;
        } else if (month >= 4 && month <= 6) {
            quarter = 2;
        } else if (month >= 7 && month <= 9) {
            quarter = 3;
        } else {
            quarter = 4;
        }
        return quarter;
    }

    /**
     * 获取本季度的第一天
     *
     * @param
     */
    public static Date getFirstDayOfQuarterByAssignDate(Date assignDate) {
        Integer quarter = getQuarterByAssignDate(assignDate);
        // 设置本年的季
        Calendar quarterCalendar = Calendar.getInstance();
        quarterCalendar.setTime(assignDate);//指定时间
        switch (quarter) {
            case 1:
                quarterCalendar.set(Calendar.MONTH, 0);
                break;
            case 2:
                quarterCalendar.set(Calendar.MONTH, 3);
                break;
            case 3:
                quarterCalendar.set(Calendar.MONTH, 6);
                break;
            case 4:
                quarterCalendar.set(Calendar.MONTH, 9);
                break;
        }
        quarterCalendar.set(Calendar.DATE, 1);
        //将小时至00
        quarterCalendar.set(Calendar.HOUR_OF_DAY, 0);
        //将分钟至00
        quarterCalendar.set(Calendar.MINUTE, 0);
        //将秒至00
        quarterCalendar.set(Calendar.SECOND, 0);
        return getThisDateStartTimeByAssignDate(quarterCalendar.getTime());
    }

    /**
     * 获取本季度的最后一天
     *
     * @param
     */
    public static Date getLastDayOfQuarterByAssignDate(Date assignDate) {
        Integer quarter = getQuarterByAssignDate(assignDate);
        // 设置本年的季
        Calendar quarterCalendar = Calendar.getInstance();
        quarterCalendar.setTime(assignDate);//指定时间
        switch (quarter) {
            case 1: // 3-31
                quarterCalendar.set(Calendar.MONTH, 3);
                break;
            case 2: // 6-30
                quarterCalendar.set(Calendar.MONTH, 6);
                break;
            case 3: // 9-30
                quarterCalendar.set(Calendar.MONTH, 9);
                break;
            case 4: // 12-31
                quarterCalendar.set(Calendar.MONTH, 12);
                break;
        }
        quarterCalendar.set(Calendar.DAY_OF_MONTH, 0);
        return getThisDateEndTimeByAssignDate(quarterCalendar.getTime());
    }

    /**
     * 获取上个季度的第一天
     *
     * @param
     */
    public static Date getFirstDayOfLastQuarterByAssignDate(Date assignDate) {
        Integer quarter = getQuarterByAssignDate(assignDate) - 1;
        // 设置本年的季
        Calendar quarterCalendar = Calendar.getInstance();
        quarterCalendar.setTime(assignDate);//指定时间
        int lastYear = quarterCalendar.get(quarterCalendar.YEAR) - 1;
        switch (quarter) {
            case 0: // 去年的第四季度: 10-1
                quarterCalendar.set(Calendar.YEAR, lastYear);
                quarterCalendar.set(Calendar.MONTH, 9);
                break;
            case 1: // 1-1
                quarterCalendar.set(Calendar.MONTH, 0);
                break;
            case 2: // 4-1
                quarterCalendar.set(Calendar.MONTH, 3);
                break;
            case 3: // 7-1
                quarterCalendar.set(Calendar.MONTH, 6);
                break;
        }
        quarterCalendar.set(Calendar.DAY_OF_MONTH, 1);
        quarterCalendar.set(Calendar.HOUR_OF_DAY, 0);
        quarterCalendar.set(Calendar.MINUTE, 0);
        quarterCalendar.set(Calendar.SECOND, 0);
        return getThisDateStartTimeByAssignDate(quarterCalendar.getTime());
    }

    /**
     * 获取上个季度的最后一天
     *
     * @param
     */
    public static Date getLastDayOfLastQuarterByAssignDate(Date assignDate) {
        Integer quarter = getQuarterByAssignDate(assignDate) - 1;
        // 设置本年的季
        Calendar quarterCalendar = Calendar.getInstance();
        quarterCalendar.setTime(assignDate);//指定时间
        int lastYear = quarterCalendar.get(quarterCalendar.YEAR) - 1;
        switch (quarter) {
            case 0: // 去年的第四季度: 12-31
                quarterCalendar.set(Calendar.YEAR, lastYear);
                quarterCalendar.set(Calendar.MONTH, 12);
                break;
            case 1: // 3-31
                quarterCalendar.set(Calendar.MONTH, 3);
                break;
            case 2:// 6-30
                quarterCalendar.set(Calendar.MONTH, 6);
                break;
            case 3:// 9-30
                quarterCalendar.set(Calendar.MONTH, 9);
                break;
        }
        quarterCalendar.set(Calendar.DAY_OF_MONTH, 0);
        quarterCalendar.set(Calendar.HOUR_OF_DAY, 23);
        quarterCalendar.set(Calendar.MINUTE, 59);
        quarterCalendar.set(Calendar.SECOND, 59);
        return getThisDateEndTimeByAssignDate(quarterCalendar.getTime());
    }

    /**
     * 获取当前半年的第一天
     */
    public static Date getFirstDayOfHalfYearByAssignDate(Date assignDate) {
        Calendar c = Calendar.getInstance();
        c.setTime(assignDate);//指定时间
        int month = c.get(c.MONTH) + 1;
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);//指定时间
        if (month >= 1 && month <= 6) {// 上半年
            calendar.set(Calendar.MONTH, 0);
        } else {// 下半年
            calendar.set(Calendar.MONTH, 6);
        }
        calendar.set(Calendar.DATE, 1);
        return getThisDateStartTimeByAssignDate(calendar.getTime());
    }

    /**
     * 获取当前半年的最后一天
     */
    public static Date getLastDayOfHalfYearByAssignDate(Date assignDate) {
        Calendar c = Calendar.getInstance();
        c.setTime(assignDate);//指定时间
        int month = c.get(c.MONTH) + 1;
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);//指定时间
        if (month >= 1 && month <= 6) {// 上半年
            calendar.set(Calendar.MONTH, 6);
        } else {// 下半年
            calendar.set(Calendar.MONTH, 12);
        }
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        return getThisDateEndTimeByAssignDate(calendar.getTime());
    }

    /**
     * 获取前半年的第一天
     */
    public static Date getFirstDayOfLastHalfYearByAssignDate(Date assignDate) {
        Calendar c = Calendar.getInstance();
        c.setTime(assignDate);//指定时间
        int month = c.get(c.MONTH) + 1;
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);//指定时间
        if (month >= 1 && month <= 6) {// 上半年
            calendar.set(Calendar.YEAR, calendar.get(Calendar.YEAR) - 1);
            calendar.set(Calendar.MONTH, 6);
        } else {// 下半年
            calendar.set(Calendar.MONTH, 0);
        }
        calendar.set(Calendar.DATE, 1);
        return getThisDateStartTimeByAssignDate(calendar.getTime());
    }

    /**
     * 获取前半年的最后一天
     */
    public static Date getLastDayOfLastHalfYearByAssignDate(Date assignDate) {
        Calendar c = Calendar.getInstance();
        c.setTime(assignDate);//指定时间
        int month = c.get(c.MONTH) + 1;
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);//指定时间
        if (month >= 1 && month <= 6) {// 上半年
            calendar.set(Calendar.YEAR, calendar.get(Calendar.YEAR) - 1);
            calendar.set(Calendar.MONTH, 12);
        } else {// 下半年
            calendar.set(Calendar.MONTH, 6);
        }
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        return getThisDateEndTimeByAssignDate(calendar.getTime());
    }

    /**
     * 获取当前全年的第一天
     */
    public static Date getFirstDayOfFullYearByAssignDate(Date assignDate) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);//指定时间
        calendar.set(Calendar.MONTH, 0);
        calendar.set(Calendar.DATE, 1);
        return getThisDateStartTimeByAssignDate(calendar.getTime());
    }

    /**
     * 获取当前全年的最后一天
     */
    public static Date getLastDayOfFullYearByAssignDate(Date assignDate) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);//指定时间
        calendar.set(Calendar.MONTH, 12);
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        return getThisDateEndTimeByAssignDate(calendar.getTime());
    }

    /**
     * 获取前年的第一天
     */
    public static Date getFirstDayOfLastFullYearByAssignDate(Date assignDate) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);//指定时间
        calendar.set(Calendar.YEAR, calendar.get(Calendar.YEAR) - 1);
        calendar.set(Calendar.MONTH, 0);
        calendar.set(Calendar.DATE, 1);
        return getThisDateStartTimeByAssignDate(calendar.getTime());
    }

    /**
     * 获取前年的最后一天
     */
    public static Date getLastDayOfLastFullYearByAssignDate(Date assignDate) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(assignDate);//指定时间
        calendar.set(Calendar.YEAR, calendar.get(Calendar.YEAR) - 1);
        calendar.set(Calendar.MONTH, 12);
        calendar.set(Calendar.DAY_OF_MONTH, 0);
        return getThisDateEndTimeByAssignDate(calendar.getTime());
    }

    /**
     * 获取系统的第一天
     */
    public static Date getFirstDayOfSystem() {
        Calendar calendar = Calendar.getInstance();
        calendar.clear();
        calendar.set(Calendar.MONTH, 0);
        calendar.set(Calendar.DATE, 1);
        return getThisDateStartTimeByAssignDate(calendar.getTime());
    }

    /**
     * 生成时间区间
     *
     * @param bizStartTime
     * @param bizEndTime
     * @return
     */
    public static String generateTimeInterval(Date bizStartTime, Date bizEndTime) {
        if (bizStartTime == null || bizEndTime == null) return null;
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return simpleDateFormat.format(bizStartTime) + "~" + simpleDateFormat.format(bizEndTime);
    }

    /**
     * 获取当前时间的00:00:00
     *
     * @param date
     * @return
     */
    public static Date getThisDateStartTime(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取当前星期几
     *
     * @return
     */
    public static Integer getWeek() {
        Calendar calendar;
        calendar = Calendar.getInstance();
        String week;
        week = calendar.get(calendar.DAY_OF_WEEK) - 1 + "";
        if ("0".equals(week)) {
            week = "7";
        }
        return Integer.valueOf(week);
    }

    /**
     * 验证时间格式
     *
     * <ol>
     * <li>yyyy-MM-dd HH:mm:ss</li>
     * <li>yyyy/MM/dd HH:mm:ss</li>
     * <li>yyyy.MM.dd HH:mm:ss</li>
     * <li>yyyy年MM月dd日 HH时mm分ss秒</li>
     * <li>yyyy-MM-dd</li>
     * <li>yyyy/MM/dd</li>
     * <li>yyyy.MM.dd</li>
     * <li>HH:mm:ss</li>
     * <li>HH时mm分ss秒</li>
     * <li>yyyy-MM-dd HH:mm</li>
     * <li>yyyy-MM-dd HH:mm:ss.SSS</li>
     * <li>yyyy-MM-dd HH:mm:ss.SSSSSS</li>
     * <li>yyyyMMddHHmmss</li>
     * <li>yyyyMMddHHmmssSSS</li>
     * <li>yyyyMMdd</li>
     * <li>EEE, dd MMM yyyy HH:mm:ss z</li>
     * <li>EEE MMM dd HH:mm:ss zzz yyyy</li>
     * <li>yyyy-MM-dd'T'HH:mm:ss'Z'</li>
     * <li>yyyy-MM-dd'T'HH:mm:ss.SSS'Z'</li>
     * <li>yyyy-MM-dd'T'HH:mm:ssZ</li>
     * <li>yyyy-MM-dd'T'HH:mm:ss.SSSZ</li>
     * <li>yyyy年MM期</li>
     * </ol>
     *
     * @param timeParamString
     * @return
     */
    public static Date validatedDateTime(String timeParamString) {
        Date parse;
        try {
            parse = cn.hutool.core.date.DateUtil.parse(timeParamString);
        } catch (Exception e) {
            int index = 0;
            if (timeParamString.length() < 10) {
                timeParamString = timeParamString + "01";
            }
            int length = timeParamString.length();
            while (index < length) {
                if (!Character.isDigit(timeParamString.charAt(index))) {
                    if (index != length - 1) {
                        timeParamString = timeParamString.replace(timeParamString.charAt(index), '-');
                    } else {
                        timeParamString = timeParamString.replace(timeParamString.charAt(index), ' ').trim();
                    }
                }
                index++;
            }
            try {
                parse = cn.hutool.core.date.DateUtil.parse(timeParamString);
            } catch (Exception exception) {
                return null;
            }
        }
        String format = cn.hutool.core.date.DateUtil.format(parse, "yyyy-MM-dd");
        if ("1970-01-01".equals(format)) {
            return null;
        }
        return parse;
    }

}

