package com.crunii.micro.service.pay.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.pay.dto.Product;
import com.crunii.micro.service.pay.service.WxPayH5;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/9 9:52
 */
@Slf4j
@Api(tags = "微信支付v3")
@RestController
@RequestMapping("/auth/WxPayController")
public class WxPayController {

    @Resource
    private WxPayH5 wxPayH5;

    @ApiOperation(value="纯H5支付(不建议在APP端使用)")
    @RequestMapping(value="/h5pay",method= RequestMethod.POST)
    public Result<String> h5pay(HttpServletRequest request, @RequestBody @Validated Product product) throws Exception {
       return wxPayH5.h5Pay(request,product);
    }

    @ApiOperation(value="商户订单号查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "outTradeNo", value = "商户订单号", required = true)
    })
    @RequestMapping(value="/queryTradeNo",method= RequestMethod.GET)
    public Result<String> queryTradeNo(@RequestParam("outTradeNo") String outTradeNo) throws Exception {
        return wxPayH5.h5OutTradeNo(outTradeNo);
    }

    @ApiOperation(value="关闭订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "outTradeNo", value = "商户订单号", required = true)
    })
    @RequestMapping(value="/queryTradeNo",method= RequestMethod.POST)
    public Result<String> closeTradeNo(@RequestParam("outTradeNo") String outTradeNo) throws Exception {
        return wxPayH5.h5CloseOutTradeNo(outTradeNo);
    }
}
