package com.crunii.micro.service.pay.util;

import com.wechat.pay.contrib.apache.httpclient.WechatPayHttpClientBuilder;
import com.wechat.pay.contrib.apache.httpclient.auth.PrivateKeySigner;
import com.wechat.pay.contrib.apache.httpclient.auth.Verifier;
import com.wechat.pay.contrib.apache.httpclient.auth.WechatPay2Credentials;
import com.wechat.pay.contrib.apache.httpclient.auth.WechatPay2Validator;
import com.wechat.pay.contrib.apache.httpclient.cert.CertificatesManager;
import com.wechat.pay.contrib.apache.httpclient.util.PemUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.Resource;

import java.io.File;
import java.nio.charset.StandardCharsets;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.Collections;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/9 9:33
 */
public class WechatPayUtil {

    /**
     * 商户号 （微信商户平台可以获取）
     */
    @Value("${wechat-pay.merchant-id}")
    private static String merchantId;

    /**
     * API_V3  （微信商户平台配置）
     */
    @Value("${wechat-pay.api-v3}")
    private static String API_V3;

    /**
     * 证书序列号（微信商户平台配置）
     */
    @Value("${wechat-pay.cert-id}")
    private static String CERT_ID ;

    /**
     * 私钥 （微信商户平台配置apiclient_key.pem内容）
     */
    private static final String PRIVATE_KEY_PATH = "ice/channel_config/wechat/apiclient_key.pem";

    /**
     * 证书路径 （自行设置）
     */
    private static final String CERT_PATH = "ice/channel_config/wechat/apiclient_cert.pem";


    /**
     * 初始化  clientBuilder和证书管理器
     *
     * @return com.wechat.pay.contrib.apache.httpclient.WechatPayHttpClientBuilder
     */
    public static WechatPayHttpClientBuilder initClientBuilder() throws Exception{
        // 读取证书及私钥
        Resource resourcePrivateKey = new FileSystemResource(new File(PRIVATE_KEY_PATH));
        PrivateKey privateKey = PemUtil.loadPrivateKey(resourcePrivateKey.getInputStream());
        Resource resourceCert = new FileSystemResource(new File(CERT_PATH));
        X509Certificate cert = PemUtil.loadCertificate(resourceCert.getInputStream());
        // 微信证书管理器
        CertificatesManager certificatesManager = CertificatesManager.getInstance();
        certificatesManager.putMerchant(
                merchantId,
                new WechatPay2Credentials(
                        merchantId,
                        new PrivateKeySigner(CERT_ID, privateKey)
                ),
                API_V3.getBytes(StandardCharsets.UTF_8)
        );
        Verifier verifier = certificatesManager.getVerifier(merchantId);
        // 初始化
        WechatPayHttpClientBuilder builder = WechatPayHttpClientBuilder.create()
                .withMerchant(merchantId, CERT_ID, privateKey)
                .withWechatPay(Collections.singletonList(cert))
                .withValidator(new WechatPay2Validator(verifier));
        return builder;
    }

    /**
     * 使用java正则表达式去掉多余的.与0
     */
    public static String subZeroAndDot(String s) {
        if (s.indexOf(".") > 0) {
            // 去掉多余的0
            s = s.replaceAll("0+?$", "");
            // 如最后一位是.则去掉
            s = s.replaceAll("[.]$", "");
        }
        return s;
    }
}
