package com.crunii.micro.service.qm.api.domain.dto.qf;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.*;
import java.io.Serializable;

/**
 * @author wangchen create 2023/11/9 9:09
 */
@Data
@ApiModel(description = "青峰创建参数")
public class LoanRecordDto implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty("贷款人姓名")
    @NotBlank(message = "贷款人姓名不能为空")
    private String loanName;

    @ApiModelProperty(value = "贷款人性别（0女 1男）")
    @NotNull(message = "贷款人性别不能为空")
    private Integer loanSex;

    @ApiModelProperty(value = "身份证号")
    @NotBlank(message = "身份证号不能为空")
    @Pattern(regexp = "^[1-9]\\d{5}(18|19|([23]\\d))\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$", message = "身份证号码格式错误")
    private String idCard;

    @ApiModelProperty(value = "政治面貌")
    @NotNull(message = "政治面貌不能为空")
    private Integer politicalOutlook;

    @ApiModelProperty(value = "联系电话")
    @NotBlank(message = "联系电话不能为空")
    @Pattern(regexp = "^1[3456789]\\d{9}$",message = "联系电话格式错误")
    private String loanPhone;

    @ApiModelProperty(value = "借款人性质")
    @NotNull(message = "借款人性质不能为空")
    private Integer loanNature;

    @ApiModelProperty(value = "教育程度")
    @NotNull(message = "教育程度不能为空")
    private Integer eduLevel;

    @ApiModelProperty(value = "申请金额（万）")
    @NotNull(message = "申请金额（万）不能为空")
    @DecimalMin(value = "0",message = "申请金额不能为负")
    private Double applyAmount;

    @ApiModelProperty(value = "贷款类型")
    @NotNull(message = "贷款类型不能为空")
    private Integer loanType;

    @ApiModelProperty(value = "意向承贷银行")
    @NotNull(message = "意向承贷银行不能为空")
    private Long intentionLoanBank;

    @ApiModelProperty(value = "借贷用途")
    @NotBlank(message = "借贷用途不能为空")
    private String loanUse;

    @ApiModelProperty(value = "公司名称")
    @NotBlank(message = "公司名称不能为空")
    private String companyName;

    @ApiModelProperty(value = "注册地址 ")
    @NotNull(message = "注册地址不能为空")
    private Long areaId;

    @ApiModelProperty(value = "所属行业")
    @NotNull(message = "所属行业不能为空")
    private Long industry;

    @ApiModelProperty(value = "企业类型（借款人类型）")
    @NotNull(message = "企业类型不能为空")
    private Integer enterpriseType;

    @ApiModelProperty(value = "企业经营负债额（万）")
    @NotNull(message = "企业经营负债额不能为空")
    @DecimalMin(value = "0",message = "企业经营负债额不能为负")
    private Double operatingLiabilities;

    @ApiModelProperty(value = "上一年营收（万）")
    @NotNull(message = "上一年营收不能为空")
    @DecimalMin(value = "0",message = "上一年营收不能为负")
    private Double lastYearIncome;

    @ApiModelProperty(value = "上一年税收（万）")
    @NotNull(message = "上一年税收不能为空")
    @DecimalMin(value = "0",message = "上一年税收不能为负")
    private Double lastYearTaxation;

    @ApiModelProperty(value = "员工数量（个）")
    @NotNull(message = "员工数量不能为空")
    @Min(value = 0,message = "员工数量不能为负")
    private Integer staffNumber;

    @ApiModelProperty(value = "员工中大学生数量（个）")
    @NotNull(message = "员工中大学生数量不能为空")
    @Min(value = 0,message = "员工中大学生数量不能为负")
    private Integer collegeStudentStaffNumber;

    @ApiModelProperty(value = "备注")
    private String notes;
}
