package com.crunii.micro.domain.entity.main;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;

/**
 * 青马-用户审核记录;
 *
 * @author : http://www.chiner.pro
 * @date : 2023-9-19
 */
@Entity
@Getter
@Setter
@ToString
@DynamicInsert
@DynamicUpdate
@Table(name = "YCQ_QM_USER_AUDIT_RECORD")
@EqualsAndHashCode(of = { "recordId" })
@GenericGenerator(name = "idGenerator", strategy = "com.crunii.micro.core.distributed.DistributedSnowFlakeId")
@ApiModel(value = "青马-用户审核记录", description = "")
public class YcqQmUserAuditRecord implements Serializable {

	private static final long serialVersionUID = 1L;
	/**
	 * 记录ID
	 */
	@Id
	@GeneratedValue(generator = "idGenerator")
	@Column(name = "record_id")
	@ApiModelProperty(name = "记录ID", notes = "")
	private Long recordId;

	/**
	 * 青马用户ID（被审核人）
	 */
	@Column(name = "user_info_id")
	@ApiModelProperty(name = "青马用户ID（被审核人）", notes = "")
	private Long userInfoId;

	/**
	 * 审核状态;0审核不通过 1审核通过 -1待审核
	 */
	@Column(name = "audit_status")
	@ApiModelProperty(name = "审核状态", notes = "0审核不通过 1审核通过 -1待审核")
	private Integer auditStatus;

	/**
	 * 审核人id
	 */
	@Column(name = "audit_worker_id")
	@ApiModelProperty(name = "审核人id", notes = "")
	private Integer auditWorkerId;

	/**
	 * 审核意见
	 */
	@Column(name = "note")
	@ApiModelProperty(name = "审核意见", notes = "")
	private String note;

	/**
	 * 审核时间
	 */
	@Column(name = "audit_time")
	@ApiModelProperty(name = "审核时间", notes = "")
	private Date auditTime;


}
