package com.crunii.micro.domain.entity.main;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;

 /**
 * 青马工程-用户信息表;
 * @author : http://www.chiner.pro
 * @date : 2023-9-18
 */
@Entity
@Getter
@Setter
@ToString
@DynamicInsert
@DynamicUpdate
@Table(name="YCQ_QM_USER_INFO")
@EqualsAndHashCode(of = { "id" })
@GenericGenerator(name = "idGenerator", strategy = "com.crunii.micro.core.distributed.DistributedSnowFlakeId")
@ApiModel(value = "青马工程-用户信息表",description = "用户信息表")
public class YcqQmUserInfo implements Serializable{

    private static final long serialVersionUID = 1L;
    /** 业务标识;生成雪花id */
    @Id
    @GeneratedValue(generator = "idGenerator")
    @Column(name="id")
    @ApiModelProperty(name = "业务标识",notes = "生成雪花id")
    private Long id;
    
    /** 用户ID;使用青春重庆的用户id */
    @Column(name="worker_id")
    @ApiModelProperty(name = "用户ID",notes = "使用青春重庆的用户id")
    private Integer workerId;
    
    /** 性别;-1未知 0女 1男 */
    @Column(name="sex")
    @ApiModelProperty(name = "性别",notes = "-1未知 0女 1男")
    private Integer sex;
    
    /** 身份;分为1学员 2导师 */
    @Column(name="user_identity")
    @ApiModelProperty(name = "身份",notes = "分为1学员 2导师")
    private Integer userIdentity;
    
    /** 组织ID;省市区 */
    @Column(name="org_id")
    @ApiModelProperty(name = "组织ID",notes = "省市区")
    private Long orgId;

     /** 区域id（org_type=9的组织id）;区域id（org_type=9的组织id） */
     @Column(name="area_id")
     @ApiModelProperty(name = "区域id（org_type=9的组织id）",notes = "区域id（org_type=9的组织id）")
     private Long areaId;
    
    /** 政治面貌;静态值，群众、共青团员、预备党员、党员 */
    @Column(name="political_outlook")
    @ApiModelProperty(name = "政治面貌",notes = "静态值，群众、共青团员、预备党员、党员")
    private Integer politicalOutlook;
    
    /** 最高学历;静态值：大专、本科、研究生 */
    @Column(name="education")
    @ApiModelProperty(name = "最高学历",notes = "静态值：大专、本科、研究生")
    private Integer education;
    
    /** 身份证号;加密存储 */
    @Column(name="card_id")
    @ApiModelProperty(name = "身份证号",notes = "加密存储")
    private String cardId;
    
    /** 工作单位 */
    @Column(name="work_unit")
    @ApiModelProperty(name = "工作单位",notes = "工作单位")
    private String workUnit;
    
    /** 职务;静态值配置 */
    @Column(name="worker_job")
    @ApiModelProperty(name = "职务",notes = "手动输入")
    private String workerJob;
    
    /** 审核状态;审核状态：-1审核中 1通过 0驳回 */
    @Column(name="audit_status")
    @ApiModelProperty(name = "审核状态",notes = "审核状态：-1审核中 1通过 0驳回")
    private Integer auditStatus;
    
//    /** 擅长领域 */
//    @Column(name="adept_field")
//    @ApiModelProperty(name = "擅长领域",notes = "")
//    private String adeptField;
    
    /** 创建人id */
    @Column(name="creator_id")
    @ApiModelProperty(name = "创建人id",notes = "创建人id")
    private Integer creatorId;
    
    /** 创建时间 */
    @Column(name="create_time")
    @ApiModelProperty(name = "创建时间",notes = "创建时间")
    private Date createTime;
    
    /** 修改人id */
    @Column(name="updater_id")
    @ApiModelProperty(name = "修改人id",notes = "修改人id")
    private Integer updaterId;
    
    /** 修改时间 */
    @Column(name="update_time")
    @ApiModelProperty(name = "修改时间",notes = "修改时间")
    private Date updateTime;
    
    /** 是否删除，学员淘汰状态0未淘汰1已淘汰 */
    @Column(name="deleted")
    @ApiModelProperty(name = "是否删除，学员淘汰状态0未淘汰1已淘汰",notes = "1删除  0不删除")
    private Integer deleted;

     /** 用户编号 */
     @Column(name="user_code")
     @ApiModelProperty(name = "用户编号",notes = "用户编号")
     private String userCode;
}