package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.Book.AppBookInfo;
import com.crunii.micro.service.qm.api.domain.dto.Book.AppBookInfoVo;
import com.crunii.micro.service.qm.api.domain.dto.Book.AppSearchBookDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.AppBookListDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.AppBookListInfoDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.AppBookListVo;
import com.crunii.micro.service.qm.service.AppBookListService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/19 19:00
 */
@Api(tags = "app书单相关")
@RestController
@RequestMapping("/auth/AppBookListController")
public class AppBookListController {

    @Resource
    private AppBookListService service;

    @ApiOperation("app书单分页")
    @PostMapping("/bookListPage")
    public Result<PageRsp<AppBookListVo>> bookListPage(@RequestBody @ApiParam PageReq<AppBookListDto> req) {
        return Result.ofSuccess(service.bookListPage(req));
    }

    @ApiOperation("app查看书单详情")
    @PostMapping("/viewBookList")
    public Result<PageRsp<AppBookInfo>> viewBookList(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @ApiParam PageReq<AppBookListInfoDto> req) {
        return Result.ofSuccess(service.viewBookList(worker, req));
    }

    @ApiOperation("app模糊搜索书籍")
    @PostMapping("/searchBook")
    public Result<PageRsp<AppBookInfo>> searchBook(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @ApiParam PageReq<AppSearchBookDto> req) {
        return Result.ofSuccess(service.searchBook(worker, req));
    }

    @ApiOperation("app查看书籍详情")
    @GetMapping("/viewBookInfo")
    public Result<AppBookInfoVo> viewBookInfo(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("bookId") Long bookId) {
        return service.viewBookInfo(worker, bookId);
    }
}
