package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.belowExcellent.*;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameVo;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.TeacherListDto;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.TeacherListVo;
import com.crunii.micro.service.qm.service.BelowExcellentService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.util.List;

/**
 * @author xiayinqiang
 * @date 2023/9/27 0027 9:54:55
 */
@Api(tags = "推优相关")
@RestController
@RequestMapping("/auth/BelowExcellentController")
public class BelowExcellentController {

	@Autowired BelowExcellentService belowExcellentService;

	@ApiOperation(value = "根据学员id查询上级组织")
	@ApiImplicitParam(name = "workerId", value = "学员id", required = true)
	@GetMapping("/selectParentOrgByWorkerId")
	public Result<PageOrgByOrgNameVo> selectParentOrgByWorkerId(@CurrWorker  @ApiIgnore Worker worker, Integer workerId) {
		return belowExcellentService.selectParentOrgByWorkerId(workerId);
	}

	@ApiOperation("推优")
	@PostMapping("/insertBelowExcellent")
	public Result<String> insertBelowExcellent(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @ApiParam @Validated InsertBelowExcellentDto dto) {
		return Result.ofSuccess(belowExcellentService.insertBelowExcellent(worker, dto));
	}

	@ApiOperation(value = "分页查询下级推优列表")
	@PostMapping("/pageBelowExcellent")
	public Result<PageRsp<PageBelowExcellentVo>> pageBelowExcellent(
			@CurrWorker  @ApiIgnore Worker worker, @RequestBody @ApiParam @Validated PageReq<PageBelowExcellentDto> req) {
		return Result.ofSuccess(belowExcellentService.pageBelowExcellent(req, worker));
	}

	@ApiOperation(value = "下级推优列表给学员分配班级和导师")
	@PostMapping("/insertStuClassesAndTeach")
	public Result<String> insertStuClassesAndTeach(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @ApiParam @Validated InsertStuClassesAndTeachDto dto) {
		return Result.ofSuccess(belowExcellentService.insertStuClassesAndTeach(worker, dto));
	}

	@ApiOperation(value = "下级推优-查询导师列表")
	@RequestMapping(path = "/teacherList", method = RequestMethod.POST)
	@ResponseBody
	public Result<List<TeacherListVo>> teacherList(@RequestBody @Valid TeacherListDto req) {
		return Result.ofSuccess(belowExcellentService.teacherList(req));
	}

	@ApiOperation(value = "批量导出")
	@PostMapping(path = "/exportByList")
	public void exportByList(@RequestBody @ApiParam @Validated ExportByListDto dto, HttpServletResponse response) throws IOException {
//		try {
			belowExcellentService.exportByList(dto, response);
//		} catch (Exception e) {
//			e.printStackTrace();
//			return Result.ofFail("导出失败");
//		}
//		return Result.ofSuccess("导出成功");
	}


	@ApiOperation(value = "根据学员id查询推荐入党记录")
	@ApiImplicitParam(name = "workerId", value = "学员id", required = true)
	@GetMapping("/selectIntoPartyByWorkerId")
	public Result<List<SelectIntoPartyVo>> selectIntoPartyByWorkerId(@CurrWorker  @ApiIgnore Worker worker,@Validated @NotNull(message = "学员id不能为空") Integer workerId) {
		return belowExcellentService.selectIntoPartyByWorkerId(worker,workerId);
	}

	@ApiOperation("新增或编辑推荐入党")
	@PostMapping("/insertIntoParty")
	public Result<String> insertIntoParty(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @ApiParam @Validated InsertIntoPartyDto dto) {
		return Result.ofSuccess(belowExcellentService.insertIntoParty(worker, dto));
	}

	@ApiOperation(value = "删除学员推荐入党记录")
	@ApiImplicitParam(name = "recordId", value = "记录主键id", required = true)
	@GetMapping("/deleteIntoPartyById")
	public Result<String> deleteIntoPartyById(@CurrWorker  @ApiIgnore Worker worker,@Validated @NotNull(message = "id不能为空") Long recordId) {
		return belowExcellentService.deleteIntoPartyById(worker,recordId);
	}


	@ApiOperation(value = "分页查询学员被拒绝两次列表")
	@PostMapping("/pageStuRefuseTwice")
	public Result<PageRsp<PageStuRefuseTwiceVo>> pageStuRefuseTwice(
			@CurrWorker  @ApiIgnore Worker worker, @RequestBody @ApiParam @Validated PageReq<PageStuRefuseTwiceDto> req) {
		return Result.ofSuccess(belowExcellentService.pageStuRefuseTwice(req, worker));
	}



}
