package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.Book.*;
import com.crunii.micro.service.qm.service.BookService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/18 18:55
 */
@Api(tags = "书籍管理web后台")
@RestController
@RequestMapping("/auth/BookController")
public class BookController {

    @Resource
    private BookService bookService;

    @ApiOperation("书籍分页")
    @PostMapping("/bookPage")
    public Result<PageRsp<BookVo>> bookPage(@RequestBody @ApiParam PageReq<BookDto> req) {
        return Result.ofSuccess(bookService.bookPage(req));
    }

    @ApiOperation("新增书籍")
    @PostMapping("/createBook")
    public Result<String> createBook(@CurrWorker  @ApiIgnore Worker worker, @Validated @RequestBody BookC bookC) {
        return bookService.createBook(worker,bookC);
    }

    @ApiOperation("修改书籍")
    @PostMapping("/updateBook")
    public Result<String> updateBook(@CurrWorker  @ApiIgnore Worker worker,@Validated @RequestBody BookU bookU) {
        return bookService.updateBook(worker,bookU);
    }

    @ApiOperation("删除书籍-支持批量")
    @PostMapping("/deleteBook")
    public Result<String> deleteBook(@CurrWorker  @ApiIgnore Worker worker,@RequestBody List<Long> bookId) {
        return bookService.deleteBook(worker,bookId);
    }

    @ApiOperation("查看书籍")
    @GetMapping("/viewBook")
    public Result<BookResVo> viewBook(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("bookId") Long bookId) {
        return bookService.viewBook(worker,bookId);
    }

    @ApiOperation("移除关联书单")
    @GetMapping("/removeBookList")
    public Result<String> removeBookList(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("bookId") Long bookId,@RequestParam("bookListId") Long bookListId) {
        return bookService.removeBookList(worker,bookId,bookListId);
    }

    @ApiOperation("上架/下架")
    @GetMapping("/updateStatus")
    public Result<String> updateStatus(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("bookId") Long bookId, @RequestParam("bookStatus") Integer bookStatus) {
        return bookService.updateStatus(worker,bookId,bookStatus);
    }
}
