package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.Book.BookInfoVo;
import com.crunii.micro.service.qm.api.domain.dto.bookList.*;
import com.crunii.micro.service.qm.service.BookListService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/19 11:31
 */
@Api(tags = "书单管理web后台")
@RestController
@RequestMapping("/auth/BookListController")
public class BookListController {

    @Resource
    private BookListService bookListService;

    @ApiOperation("书单分页")
    @PostMapping("/bookListPage")
    public Result<PageRsp<BookListVo>> bookListPage(@RequestBody @ApiParam PageReq<BookListDto> req) {
        return Result.ofSuccess(bookListService.bookListPage(req));
    }

    @ApiOperation("新增书单")
    @PostMapping("/createBookList")
    public Result<String> createBookList(@CurrWorker  @ApiIgnore Worker worker, @Validated @RequestBody BookListC bookListC) {
        return bookListService.createBookList(worker, bookListC);
    }

    @ApiOperation("编辑书单")
    @PostMapping("/updateBookList")
    public Result<String> updateBookList(@CurrWorker  @ApiIgnore Worker worker, @Validated @RequestBody BookListU bookListU) {
        return bookListService.updateBookList(worker, bookListU);
    }

    @ApiOperation("查看书单")
    @GetMapping("/viewBookList")
    public Result<BookListResVo> viewBookList(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("bookListId") Long bookListId) {
        return bookListService.viewBookList(worker, bookListId);
    }

    @ApiOperation("移除书籍")
    @GetMapping("/removeBook")
    public Result<String> removeBook(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("relId") Long relId) {
        return bookListService.removeBook(worker, relId);
    }

    @ApiOperation("删除书单-支持批量")
    @PostMapping("/deleteBookList")
    public Result<String> deleteBookList(@CurrWorker  @ApiIgnore Worker worker, @RequestBody List<Long> bookListIds) {
        return bookListService.deleteBookList(worker, bookListIds);
    }

    @ApiOperation("上架/下架")
    @GetMapping("/updateStatus")
    public Result<String> updateStatus(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("bookListId") Long bookListId, @RequestParam("bookListStatus") Integer bookListStatus) {
        return bookListService.updateStatus(worker, bookListId, bookListStatus);
    }

    @ApiOperation("关联书单")
    @PostMapping("/associatedBookList")
    public Result<String> associatedBookList(@CurrWorker  @ApiIgnore Worker worker, @Validated @RequestBody AssociatedBookListDto dto) {
        return bookListService.associatedBookList(worker, dto);
    }

    @ApiOperation("查询书单关联书籍信息")
    @PostMapping("/queryAssociateBook")
    public Result<PageRsp<BookInfoVo>> queryAssociateBook(@RequestBody @ApiParam PageReq<AppBookListInfoDto> req) {
        return Result.ofSuccess(bookListService.queryAssociateBook(req));
    }

}
