package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.classesMgr.*;
import com.crunii.micro.service.qm.service.ClassesMgrService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.List;

/**
 * @author: LG
 * @date: 2023/9/18 19:34
 */
@Api(tags = "班级和课程管理")
@RestController
@RequestMapping("/auth/ClassesMgrController")
public class ClassesMgrController {

    @Resource
    private ClassesMgrService classesMgrService;

    @ApiOperation("查询班级类型")
    @GetMapping("/queryClassesType")
    public Result<List<QueryClassesTypeVO>> queryClassesType(@RequestParam("levelCode") String levelCode){
        return classesMgrService.queryClassesType(levelCode);
    }

    @ApiOperation("查询班级级别")
    @PostMapping("/queryClassesLevel")
    public Result<List<QueryClassesTypeVO>> queryClassesLevel(){
        return classesMgrService.queryClassesLevel();
    }

    @ApiOperation("查询班级级别（仅班级新增处使用）")
    @GetMapping("/queryClassesLevelByAdd")
    public Result<List<QueryClassesTypeVO>> queryClassesLevelByAdd(@RequestParam("orgLevel") String orgLevel){
        return classesMgrService.queryClassesLevelByAdd(orgLevel);
    }

    @ApiOperation("查询班级信息")
    @PostMapping("/queryClasses")
    public Result<PageRsp<QueryClassesVO>> queryClasses(@RequestBody PageReq<QueryClassesDTO> req){
        return classesMgrService.queryClasses(req);
    }

    @ApiOperation("查询班级信息详情")
    @GetMapping("/queryClassesDetail")
    public Result<QueryClassesVO> queryClassesDetail(@RequestParam("classesId") Long classesId){
        return classesMgrService.queryClassesDetail(classesId);
    }

    @ApiOperation("查询班级下学员")
    @PostMapping("/queryClassesStu")
    public Result<PageRsp<stuVO>> queryClassesStu(@RequestBody PageReq<QueryClassesStuDTO> req ){
        return classesMgrService.queryClassesStu(req);
    }

    @ApiOperation("批量删除班级")
    @PostMapping("/deleteClasses")
    public Result<Object> deleteClasses(@CurrWorker  @ApiIgnore Worker worker,@RequestBody List<Long> classesId){
        return classesMgrService.deleteClasses(classesId);
    }

    @ApiOperation("新增班级")
    @PostMapping("/saveClasses")
    public Result<Object> saveClasses(@CurrWorker  @ApiIgnore Worker worker,@RequestBody @Validated SaveClassesDTO saveClassesDTO){
        return classesMgrService.saveClasses(worker,saveClassesDTO);
    }

    @ApiOperation("编辑班级")
    @PostMapping("/updateClasses")
    public Result<Object> updateClasses(@CurrWorker  @ApiIgnore Worker worker,@RequestBody @Validated SaveClassesDTO saveClassesDTO){
        return classesMgrService.saveClasses(worker,saveClassesDTO);
    }

    @ApiOperation("新增课程")
    @PostMapping("/saveCourse")
    public Result<Object> saveCourse(@CurrWorker  @ApiIgnore Worker worker,@RequestBody @Validated SaveCourseDTO saveCourseDTO) throws ParseException {
        return classesMgrService.saveCourse(worker,saveCourseDTO);
    }

    @ApiOperation("编辑课程")
    @PostMapping("/updateCourse")
    public Result<Object> updateCourse(@CurrWorker  @ApiIgnore Worker worker,@RequestBody @Validated SaveCourseDTO saveCourseDTO) throws ParseException {
        return classesMgrService.updateCourse(worker,saveCourseDTO);
    }

    @ApiOperation("批量删除课程")
    @PostMapping("/deleteCourse")
    public Result<Object>deleteCourse(@CurrWorker  @ApiIgnore Worker worker,@RequestBody List<Long> courseIds){
        return classesMgrService.deleteCourse(worker,courseIds);
    }

    @ApiOperation("查看课程")
    @PostMapping("/queryCourse")
    public Result<PageRsp<QueryCourseVO>> queryCourse(@RequestBody PageReq<QueryCourseDTO> req){
        return classesMgrService.queryCourse(req);
    }

    @ApiOperation("查看课程详情")
    @GetMapping("/queryCourseDetail")
    public Result<QueryCourseVO> queryCourseDetail(@RequestParam("courseId") Long courseId){
        return classesMgrService.queryCourseDetail(courseId);
    }

    @ApiOperation("组织评价学生")
    @PostMapping("/evaluateStu")
    public Result<Object> evaluateStu(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @Validated EvaluateStuDTO evaluateStuDTO){
        return classesMgrService.updateEvaluateStu(worker,evaluateStuDTO);
    }

    @ApiOperation("班级启用或停用")
    @PostMapping("/updateClassesStatus")
    public Result<Object> updateClassesStatus(@CurrWorker  @ApiIgnore Worker worker,@RequestBody @Validated UpdateClassesStatusDTO updateClassesStatusDTO ){
        return classesMgrService.updateClassesStatus(worker,updateClassesStatusDTO);
    }

    @ApiOperation("新增班级学员查询")
    @PostMapping("/queryAddClassesStu")
    public Result<PageRsp<stuVO>> queryAddClassesStu(@RequestBody PageReq<QueryClassesStuDTO> req){
        return classesMgrService.queryAddClassesStu(req);
    }




}
