package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.courseware.*;
import com.crunii.micro.service.qm.service.CoursewareService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/18 12:05
 */
@Api(tags = "课件管理web后台")
@RestController
@RequestMapping("/auth/CoursewareController")
public class CoursewareController {

    @Resource
    private CoursewareService coursewareService;

    @ApiOperation("课件分页")
    @PostMapping("/coursewarePage")
    public Result<PageRsp<CoursewareVo>> coursewarePage(@RequestBody @ApiParam PageReq<CoursewareDto> req) {
        return Result.ofSuccess(coursewareService.coursewarePage(req));
    }

    @ApiOperation("新增课件")
    @PostMapping("/createCourseware")
    public Result<String> createCourseware(@CurrWorker  @ApiIgnore Worker worker,@Validated @RequestBody CoursewareC coursewareC) {
        return coursewareService.createCourseware(worker,coursewareC);
    }

    @ApiOperation("修改课件")
    @PostMapping("/updateCourseware")
    public Result<String> updateCourseware(@CurrWorker  @ApiIgnore Worker worker,@Validated @RequestBody CoursewareU coursewareU) {
        return coursewareService.updateCourseware(worker,coursewareU);
    }

    @ApiOperation("删除课件-支持批量")
    @PostMapping("/deleteCourseware")
    public Result<String> deleteCourseware(@CurrWorker  @ApiIgnore Worker worker,@RequestBody List<Long> certificateId) {
        return coursewareService.deleteCourseware(worker,certificateId);
    }

    @ApiOperation("查看课件")
    @GetMapping("/viewCourseware")
    public Result<CoursewareResVo> viewCourseware(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("certificateId") Long certificateId) {
        return coursewareService.viewCourseware(worker,certificateId);
    }

    @ApiOperation("上架/下架")
    @GetMapping("/updateStatus")
    public Result<String> updateStatus(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("certificateId") Long certificateId, @RequestParam("certificateStatus") Integer certificateStatus) {
        return coursewareService.updateStatus(worker,certificateId,certificateStatus);
    }
}
