package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.homepage.vo.HomePageBookListVo;
import com.crunii.micro.service.qm.api.domain.dto.homepage.vo.HomeStuCountVo;
import com.crunii.micro.service.qm.service.HomePageService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * @author xia
 * @date 2023/9/18 0018 15:08:40
 */
@Api(tags = "主页相关")
@Validated
@RestController
@RequestMapping("/auth/HomePageController")
public class HomePageController {

	@Autowired HomePageService homePageService;

	/**
	 * 查询首页展示书单书籍列表，书单查前四个，每个书单里的书籍查前三个
	 */
	@ApiOperation(value = "查询首页展示书单书籍列表")
	@GetMapping("/selectHomeBookList")
	public Result<List<HomePageBookListVo>> selectHomeBookList(@CurrWorker  @ApiIgnore Worker worker) {
		return homePageService.selectHomeBookList();
	}

	/**
	 * 导师端查询首页展示学员各种数量
	 */
	@ApiOperation(value = "导师端查询首页展示学员各种数量")
	@GetMapping("/selectHomeStuCount")
	public Result<HomeStuCountVo> selectHomeStuCount(@CurrWorker  @ApiIgnore Worker worker) {
		return homePageService.selectHomeStuCount(worker.getLoginId());
	}

	/**
	 * 查询导师聘用时长或学员入学时间
	 */
	@ApiOperation(value = "查询导师聘用时长或学员入学时间")
	@GetMapping("/selectTeacherExpandDate")
	public Result<String> selectTeacherExpandDate(@CurrWorker  @ApiIgnore Worker worker) {
		return homePageService.selectTeacherExpandDate(worker.getLoginId());
	}





}
