package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQfAreaIndustry;
import com.crunii.micro.service.qm.api.domain.dto.qf.LoanPlanAuthDto;
import com.crunii.micro.service.qm.api.domain.dto.qf.LoanPlanAuthVo;
import com.crunii.micro.service.qm.api.domain.dto.qf.LoanPlanDetailVo;
import com.crunii.micro.service.qm.api.domain.dto.qf.LoanRecordDto;
import com.crunii.micro.service.qm.service.QfPlanAuditService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author wangchen create 2023/11/8 19:14
 */
@Api(tags = "青峰app")
@RestController
@RequestMapping("/auth/QfPlanAuditController")
public class QfPlanAuditController {

    private final QfPlanAuditService qfAppService;

    public QfPlanAuditController(QfPlanAuditService qfAppService) {
        this.qfAppService = qfAppService;
    }

    @ApiOperation("新增计划申请")
    @PostMapping("/createPlan")
    public Result<String> createPlan(@Validated @RequestBody LoanRecordDto loanRecord, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(qfAppService.createPlan(loanRecord,worker));
    }

    @ApiOperation("查询区域和行业")
    @GetMapping("/findAreaIndustry")
    public Result<List<YcqQfAreaIndustry>> findAreaIndustry(@RequestParam Integer type){
        return Result.ofSuccess(qfAppService.findAreaIndustry(type));
    }

    @ApiOperation("保留接口，写入区域和行业")
    @PostMapping("/createAreaIndustry")
    public Result<String> createAreaIndustry(@RequestBody List<YcqQfAreaIndustry> ycqQfAreaIndustry){
        return Result.ofSuccess(qfAppService.createAreaIndustry(ycqQfAreaIndustry));
    }

    @ApiOperation("删除区域或行业")
    @GetMapping("/delAreaIndustry")
    public Result<String> delAreaIndustry(@RequestParam Long areaIndustryId){
        return Result.ofSuccess(qfAppService.delAreaIndustry(areaIndustryId));
    }

    @ApiOperation("查询我的资金帮扶")
    @PostMapping("/pageMyLoanStatistics")
    public Result<PageRsp<LoanPlanAuthVo>> pageMyLoanStatistics(@RequestBody PageReq<LoanPlanAuthDto> dto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(qfAppService.pageMyLoanStatistics(dto,worker));
    }

    @ApiOperation("根据id查询详情")
    @PostMapping("/getLoanPlanDetail")
    public Result<LoanPlanDetailVo> getLoanPlanDetail(@RequestParam("loanId") Long loanId , @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(qfAppService.getLoanPlanDetail(loanId,worker));
    }

}
