package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQfBank;
import com.crunii.micro.domain.entity.main.YcqQfLoanRecord;
import com.crunii.micro.service.qm.api.domain.dto.ExcelDataInfo;
import com.crunii.micro.service.qm.api.domain.dto.qf.*;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.ImportStuDataDto;
import com.crunii.micro.service.qm.service.QfPlanManageService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * @author wangchen create 2023/11/9 11:23
 */
@Api(tags = "青峰web")
@RestController
@RequestMapping("/auth/QfPlanManageController")
public class QfPlanManageController {

    public static final String EXCEL_FILE_XLSX = ".xlsx";
    public static final String EXCEL_FILE_XLS = ".xls";

    private final QfPlanManageService qfPlanManageService;

    public QfPlanManageController(QfPlanManageService qfPlanManageService) {
        this.qfPlanManageService = qfPlanManageService;
    }

    @ApiOperation("分页查询计划申请")
    @PostMapping("/pageLoanPlan")
    public Result<PageRsp<LoanPlanDetailVo>> pageLoanPlan(@RequestBody PageReq<LoanPlanDto> req){
        return Result.ofSuccess(qfPlanManageService.pageLoanPlan(req));
    }

    @ApiOperation("根据id删除计划")
    @GetMapping("/delLoanPlan")
    public Result<String> delLoanPlan(Long loanId, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(qfPlanManageService.delLoanPlan(loanId,worker));
    }

    @ApiOperation("查询银行")
    @PostMapping("/queryBank")
    public Result<List<YcqQfBankVo>> queryBank(@RequestParam(defaultValue = "0") Long parentBankId){
        return Result.ofSuccess(qfPlanManageService.queryBank(parentBankId));
    }

    @ApiOperation("导入银行")
    @PostMapping("/insertBank")
    public Result<String> insertBank(@RequestBody List<YcqQfBank> req){
        return Result.ofSuccess(qfPlanManageService.insertBank(req));
    }

    @ApiOperation("导出计划申请")
    @PostMapping("/exportLoanPlan")
    public void exportLoanPlan(@RequestBody LoanPlanDto req){
        qfPlanManageService.exportLoanPlan(req);
    }

    @ApiOperation("导出计划申请-通过id")
    @PostMapping("/exportLoanPlanByIds")
    public void exportLoanPlanByIds(@RequestBody List<Long> loanIds){
        qfPlanManageService.exportLoanPlanByIds(loanIds);
    }

    @ApiOperation("导入计划数据验证")
    @PostMapping("/importLoanPlanVerify")
    public Result<ExcelDataInfo> importLoanPlanVerify(MultipartFile file){
        boolean b = file != null && !file.isEmpty() && StringUtils.isNotBlank(file.getOriginalFilename()) && (
                file.getOriginalFilename().indexOf(EXCEL_FILE_XLSX) > 0
                        || file.getOriginalFilename().indexOf(EXCEL_FILE_XLS) > 0);
        if (b){
            return qfPlanManageService.importLoanPlanVerify(file);
        }
        return Result.ofFail("非excel文件无法导入");
    }

    @ApiOperation("导入计划申请")
    @PostMapping("/importLoanPlan")
    public Result<String> importLoanPlan(@RequestBody ImportStuDataDto dto){
       return qfPlanManageService.importLoanPlan(dto);
    }

    @ApiOperation("查询资金帮扶统计信息")
    @GetMapping("/getLoanStatistics")
    public Result<LoanPlanSimpleVo> getLoanStatistics(){
        return Result.ofSuccess(qfPlanManageService.getLoanStatistics());
    }


    @ApiOperation("导入计划数据")
    @GetMapping("/saveByImport")
    public Result<ExcelDataInfo> saveByImport(MultipartFile file){
        boolean b = file != null && !file.isEmpty() && StringUtils.isNotBlank(file.getOriginalFilename()) && (
                file.getOriginalFilename().indexOf(EXCEL_FILE_XLSX) > 0
                        || file.getOriginalFilename().indexOf(EXCEL_FILE_XLS) > 0);
        if (b){
            return qfPlanManageService.saveByImport(file);
        }
        return Result.ofFail("非excel文件无法导入");
    }

}
