package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;
import com.crunii.micro.service.qm.service.StudentAppService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.ArrayList;
import java.util.List;

/**
 * @author wangchen create 2023/9/20 10:15
 */
@Api(tags = "学员app")
@RestController
@RequestMapping("/auth/StudentAppController")
public class StudentAppController {

    private final StudentAppService studentAppService;

    public StudentAppController(StudentAppService studentAppService) {
        this.studentAppService = studentAppService;
    }

    @ApiOperation(value = "验证是否绑定导师 true有绑定导师")
    @PostMapping("/verifyIsBindTeacher")
    public Result<Boolean> verifyIsBindTeacher(@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.verifyIsBindTeacher(worker.getLoginId()));
    }

    @ApiOperation(value = "查询导师列表-当未绑定导师时")
    @PostMapping("/pageTeacherBase")
    public Result<PageRsp<TeacherAppBaseVo>> pageTeacherBase(@RequestBody PageReq<TeacherAppBaseDto> pageReq, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.pageTeacherBase(pageReq,worker));
    }

    @ApiOperation(value = "查询我的导师-绑定导师时")
    @PostMapping("/findMyTeacher")
    public Result<List<TeacherAppBaseVo>> findMyTeacher(@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.findMyTeacher(worker));
    }

    @ApiOperation(value = "查询导师荣誉 && 擅长领域")
    @PostMapping("/getTeacherExpand")
    public Result<HonorFieldVo> getTeacherExpand(@RequestBody WorkerHonorDto workerHonorDto,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.getTeacherExpand(workerHonorDto.getWorkerId()));
    }

    @ApiOperation("跟踪培养页-自我培养")
    @PostMapping("/getSelfCultivation")
    public Result<SelfCultivationVo> getSelfCultivation(@RequestBody TrackingCultivationDto trackingCultivationDto,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.getSelfCultivation(trackingCultivationDto,worker));
    }

    @ApiOperation("跟踪培养页-成长发展分页-人才举荐")
    @PostMapping("/pageSelfGrowth")
    public Result<PageRsp<GrowthVo>> pageSelfGrowth(@RequestBody PageReq<TrackingCultivationDto> pageReq, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.pageSelfGrowth(pageReq,worker));
    }

    @ApiOperation("跟踪培养页-成长发展分页-推优入党")
    @PostMapping("/findParty")
    public Result<List<GrowthPartyVo>> findParty( @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.findParty(worker));
    }

    @ApiOperation("跟踪培养-作用发挥")
    @PostMapping("/pageExert")
    public Result<PageRsp<ExertVo>> pageExert(@RequestBody PageReq<TrackingCultivationDto> trackingCultivationDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.pageExert(trackingCultivationDto,worker));
    }

    @ApiOperation("跟踪培养页-导师评价")
    @PostMapping("/getStuTraineeExpand")
    public Result<PageRsp<TeacherTraineeVo>> getStuTraineeExpand(@RequestBody PageReq<TrackingCultivationDto> pageReq,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.studentAppService(pageReq,worker));
    }

    @ApiOperation(value = "查询审核流程")
    @PostMapping("/findAuditRecord")
    public Result<List<Object>> findAuditRecord(@RequestBody WorkerHonorDto workerHonorDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(new ArrayList<>());
    }

    @ApiOperation("跟踪培养页-自我培养录入/修改信息")
    @PostMapping("/enterSelfCultivation")
    public Result<String> enterSelfCultivation(@RequestBody SelfCultivationDto selfCultivationDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.saveSelfCultivation(selfCultivationDto,worker));
    }

    @ApiOperation("跟踪培养页-录入/修改发挥作用信息")
    @PostMapping("/enterExert")
    public Result<String> enterExert(@RequestBody ExertCreateDto exertCreateDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.saveExert(exertCreateDto,worker));
    }


    @ApiOperation("跟踪培养页-录入人才举荐")
    @PostMapping("/enterRecommend")
    public Result<String> enterRecommend(@RequestBody RecommendCreateDto recommendCreateDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.saveRecommend(recommendCreateDto,worker));
    }

    @ApiOperation("跟踪培养-自我培养-志愿服务分页 && 其他")
    @PostMapping("/getVolunteer")
    public Result<List<VolunteerVo>> getVolunteer(@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.getVolunteer(worker));
    }

    @ApiOperation("学员详情-跟踪培养-删除自我培养")
    @GetMapping("/delSelfCultivation")
    public Result<String> delSelfCultivation(@RequestParam("selfCultivationId") Long selfCultivationId,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.delSelfCultivation(selfCultivationId,worker));
    }

    @ApiOperation("学员详情-跟踪培养-删除作用发挥")
    @GetMapping("/delExert")
    public Result<String> delExert(@RequestParam("exertId") Long exertId,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.delExert(exertId,worker));
    }

    @ApiOperation("修改学习强国积分")
    @PostMapping("/modifyLearIntegral")
    public Result<String> modifyLearIntegral(@RequestBody LearIntegral learIntegral,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.modifyLearIntegral(learIntegral,worker));
    }

    @ApiOperation("学员详情-跟踪培养-删除成长发展")
    @GetMapping("/delSelfGrowth")
    public Result<String> delSelfGrowth(@RequestParam("groupId") Long groupId,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.delSelfGrowth(groupId,worker));
    }

    @ApiOperation("申请导师")
    @GetMapping("/applyTeacher")
    public Result<String> applyTeacher(@RequestParam("workerId") Integer workerId,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.applyTeacher(workerId,worker));
    }

    @ApiOperation("查看当前用户申请状态")
    @PostMapping("/findStuAuditState")
    public Result<StudentAuditVo> findStuAuditState(@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(studentAppService.findStuAuditState(worker));
    }
}
