package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.ViewTeacherDto;
import com.crunii.micro.service.qm.api.domain.dto.ExcelDataInfo;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.TeacherAppBaseVo;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ConcentratedCultivationVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.StuExpandDto;
import com.crunii.micro.service.qm.service.StudentManageWebService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @author baoxy
 * @date 2023-09-18
 */
@Api(tags = "学员管理web后台")
@RestController
@RequestMapping("/auth/StudentManageWebController")
public class StudentManageWebController {

    @Resource
    private StudentManageWebService studentManageWebService;

    @ApiOperation(value = "分页查询待审核,审核不通过,审核通过列表")
    @RequestMapping(path = "/pageWaitExamine", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<PassExamineVo>> pageExamine(@RequestBody @Valid PageReq<PassExamineDto> req, @CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(studentManageWebService.pageExamine(req, worker));
    }

    @ApiOperation(value = "查看学员基本信息")
    @RequestMapping(path = "/viewStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<ViewStudentVo> viewStudent(@RequestBody ViewStudentDto dto, @CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(studentManageWebService.viewStudent(dto, worker));
    }

    @ApiOperation(value = "创建学员")
    @RequestMapping(path = "/createStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> createStudent(@RequestBody CreateStudentDto dto, @CurrWorker @ApiIgnore Worker worker) {
        return studentManageWebService.createStudent(dto, worker);
    }

    @ApiOperation(value = "编辑学员")
    @RequestMapping(path = "/updateStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateStudent(@RequestBody @Valid UpdateStudentDto dto, @CurrWorker @ApiIgnore Worker worker) {
        return studentManageWebService.updateStudent(dto, worker);
    }

    @ApiOperation(value = "批量删除学员")
    @RequestMapping(path = "/batchDelStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> batchDelStudent(@RequestBody List<DelStudentDto> dtoList, @CurrWorker @ApiIgnore Worker worker) {
        return studentManageWebService.batchDelStudent(dtoList, worker);
    }

    @ApiOperation(value = "批量审核")
    @RequestMapping(path = "/batchExamine", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> batchExamine(@RequestBody List<BatchExamineDto> dtoList, @CurrWorker @ApiIgnore Worker worker) {
        return studentManageWebService.batchExamine(dtoList, worker);
    }

    @ApiOperation(value = "查询待审核,审核不通过,审核通过,下级推优的数量")
    @RequestMapping(path = "/searchNum", method = RequestMethod.POST)
    @ResponseBody
    public Result<SearchNumVo> searchNum(@RequestBody SearchNumDto dto, @CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(studentManageWebService.searchNum(dto,worker));
    }

    @ApiOperation(value = "模板下载")
    @RequestMapping(path = "/downLoadTemplate", method = RequestMethod.POST)
    @ResponseBody
    public void downLoadTemplate(HttpServletResponse response) {
        studentManageWebService.downLoadTemplate(response);
    }

    @ApiOperation(value = "校验学生的Excel文件")
    @RequestMapping(path = "/verifyStuExcel", method = RequestMethod.POST)
    @ResponseBody
    public Result<ExcelDataInfo> verifyStuExcel(@RequestPart("file") MultipartFile file) {
        return studentManageWebService.verifyStuExcel(file);
    }

    @ApiOperation(value = "Excel导入学员数据")
    @RequestMapping(path = "/importStuData", method = RequestMethod.POST)
    @ResponseBody
    public Result<String> importStuData(@RequestBody ImportStuDataDto dto, @ApiIgnore @CurrWorker Worker worker) {
        return studentManageWebService.importStuData(dto, worker);
    }

    @ApiOperation(value = "导出审核通过列表")
    @RequestMapping(path = "/exportPassExamine", method = RequestMethod.POST)
    @ResponseBody
    public void exportPassExamine(@RequestBody ExportExamineDto dto) {
        studentManageWebService.exportPassExamine(dto);
    }

    @ApiOperation(value = "查看学员-导师信息")
    @RequestMapping(path = "/viewTeacher", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<TeacherAppBaseVo>> viewTeacher(@Valid @RequestBody ViewTeacherDto dto) {
        return Result.ofSuccess(studentManageWebService.viewTeacher(dto));
    }

    @ApiOperation(value = "查看学员-集中培养")
    @RequestMapping(path = "/concentrateCulture", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<ConcentratedCultivationVo>> concentrateCulture(@RequestBody PageReq<StuExpandDto> req) {
        return Result.ofSuccess(studentManageWebService.concentrateCulture(req));
    }

    @ApiOperation(value = "导师设置")
    @RequestMapping(path = "/teacherSet", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> teacherSet(@RequestBody TeacherSetDto dto, @CurrWorker @ApiIgnore Worker worker) {
        return studentManageWebService.teacherSet(dto, worker);
    }

    @ApiOperation(value = "导师设置-查询导师列表")
    @RequestMapping(path = "/teacherList", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<TeacherListVo>> teacherList(@RequestBody @Valid TeacherListDto req) {
        return Result.ofSuccess(studentManageWebService.teacherList(req));
    }

    @ApiOperation(value = "导师设置-已绑定的导师列表")
    @RequestMapping(path = "/bindingTeacherList", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<TeacherListVo>> bindingTeacherList(@RequestBody TeacherBindingListDto dto) {
        return Result.ofSuccess(studentManageWebService.bindingTeacherList(dto));
    }

    @ApiOperation(value = "分配班级")
    @RequestMapping(path = "/shareClasses", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> shareClasses(@RequestBody @Valid ShareClassesDto dto, @CurrWorker @ApiIgnore Worker worker) {
        return studentManageWebService.shareClasses(dto, worker);
    }

    @ApiOperation(value = "上传结业证书")
    @RequestMapping(path = "/uploadCertificate", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> uploadCertificate(@RequestBody UploadCertificateDto dto, @CurrWorker @ApiIgnore Worker worker) {
        return studentManageWebService.uploadCertificate(dto, worker);
    }

    @ApiOperation(value = "验证是否更改学员信息")
    @RequestMapping(path = "/validUpdateStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> validUpdateStudent(@RequestBody UpdateStudentDto dto, @CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(studentManageWebService.validUpdateStudent(dto, worker));
    }
}
