package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;
import com.crunii.micro.service.qm.service.TeacherAppService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * @author wangchen create 2023/9/19 11:14
 */
@Api(tags = "导师app")
@RestController
@RequestMapping("/auth/TeacherAppController")
public class TeacherAppController {

    private final TeacherAppService teacherAppService;

    public TeacherAppController(TeacherAppService teacherAppService) {
        this.teacherAppService = teacherAppService;
    }

    @ApiOperation("我的学员页-分页查询学员 ")
    @PostMapping("/pageAuditMyStu")
    public Result<PageRsp<MyStuBaseVo>> pageAuditMyStu(@Valid @RequestBody PageReq<MyStuBaseDto> req, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.pageAuditMyStu(req,worker));
    }

    @ApiOperation("我的学员页-接受 || 拒绝学员")
    @PostMapping("/updateStuAudit")
    public Result<String> updateStuAudit(@RequestBody StuAuditDto stuAuditDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.updateStuAudit(stuAuditDto,worker));
    }

    @ApiOperation("我的学员页-评价学员")
    @PostMapping("/addEvaluateStu")
    public Result<String> addEvaluateStu(@RequestBody EvaluateStuDto evaluateStuDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.addEvaluateStu(evaluateStuDto,worker));
    }

    @ApiOperation("我的学员页-移除学员")
    @GetMapping("/delMyStu")
    public Result<String> delMyStu(@RequestParam("workerId") Integer workerId,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.delMyStu(workerId,worker));
    }

    @ApiOperation("学员详情-荣誉&优势-荣誉")
    @PostMapping("/pageHonor")
    public Result<PageRsp<WorkerHonorVo>> pageHonor(@Valid @RequestBody PageReq<StuExpandDto> pageReq){
        return Result.ofSuccess(teacherAppService.pageHonor(pageReq));
    }
    @ApiOperation("学员详情-荣誉&优势-优势")
    @PostMapping("/getAdvantage")
    public Result<String> getAdvantage(@Valid @RequestBody StuExpandDto stuExpandDto){
        return Result.ofSuccess(teacherAppService.getAdvantage(stuExpandDto));
    }

    @ApiOperation("学员详情-集中培养")
    @PostMapping("/pageConcentratedCultivationExpand")
    public Result<PageRsp<ConcentratedCultivationVo>> pageConcentratedCultivationExpand(@RequestBody PageReq<StuExpandDto> pageReq){
        return Result.ofSuccess(teacherAppService.pageConcentratedCultivationExpand(pageReq));
    }

    @ApiOperation("学员详情-导师评价")
    @PostMapping("/pageStuTraineeExpand")
    public Result<PageRsp<TeacherTraineeVo>> pageStuTraineeExpand(@RequestBody PageReq<TrackingCultivationDto> pageReq,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.pageStuTraineeExpand(pageReq,worker));
    }

    @ApiOperation("学员详情-跟踪培养-自我培养")
    @PostMapping("/getSelfCultivation")
    public Result<SelfCultivationVo> getSelfCultivation(@RequestBody TrackingCultivationDto trackingCultivationDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.getSelfCultivation(trackingCultivationDto,worker));
    }

    @ApiOperation("学员详情-跟踪培养-作用发挥")
    @PostMapping("/pageExert")
    public Result<PageRsp<ExertVo>> pageExert(@RequestBody PageReq<TrackingCultivationDto> trackingCultivationDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.pageExert(trackingCultivationDto,worker));
    }


    @ApiOperation("学员详情-跟踪培养-成长发展")
    @PostMapping("/pageGrowth")
    public Result<PageRsp<GrowthVo>> pageGrowth(@RequestBody PageReq<TrackingCultivationDto> trackingCultivationDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.pageGrowth(trackingCultivationDto,worker));
    }

    @ApiOperation("跟踪培养页-成长发展-推优入党")
    @PostMapping("/findParty")
    public Result<List<GrowthPartyVo>> findParty(@RequestBody TrackingCultivationDto pageReq, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.findParty(pageReq,worker));
    }

    @ApiOperation("学员详情-跟踪培养-自我培养-志愿服务分页 && 其他分页")
    @PostMapping("/getVolunteer")
    public Result<List<VolunteerVo>> getVolunteer(@RequestBody StuExpandDto stuExpandDto){
        return Result.ofSuccess(teacherAppService.getVolunteer(stuExpandDto));
    }

    @ApiOperation("添加学员页-分页查询学生（未申请导师，未匹配导师）")
    @PostMapping("/pageNoTeacherStu")
    public Result<PageRsp<MyStuBaseVo>> pageNoTeacherStu(@Valid @RequestBody PageReq<MyStuBaseDto> req,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.pageNoTeacherStu(req,worker));
    }

    @ApiOperation("添加学员页-选中学员")
    @PostMapping("/chooseStu")
    public Result<String> chooseStu(@RequestBody List<Integer> workerIds,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherAppService.addStuList(workerIds,worker));
    }

    @ApiOperation("查询班级级别")
    @PostMapping("/findLevelType")
    public Result<List<CodeValueVo>> findLevelType(){
        return Result.ofSuccess(teacherAppService.findLevelType());
    }

    @ApiOperation("查询班级年度")
    @PostMapping("/findClassesYear")
    public Result<List<String>> findClassesYear(){
        return Result.ofSuccess(teacherAppService.findClassesYear());
    }
}
