package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.SearchNumVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.EvaluateStuDto;
import com.crunii.micro.service.qm.service.TeacherManageService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @author wangchen create 2023/9/18 11:21
 */
@Api(tags = "导师管理web后台")
@RestController
@RequestMapping("/auth/TeacherManageController")
public class TeacherManageController {

    private final TeacherManageService teacherManageService;

    public TeacherManageController(TeacherManageService teacherManageService) {
        this.teacherManageService = teacherManageService;
    }

    @ApiOperation("创建/修改导师")
    @PostMapping("/modifyTeacher")
    public Result<String> modifyTeacher(@Valid @RequestBody TeacherDetailModifyDto teacherDetailModifyDto, @ApiIgnore @CurrWorker Worker worker) {
        return Result.ofSuccess(teacherManageService.modifyTeacher(teacherDetailModifyDto, worker));
    }

    @ApiOperation("验证是否更改")
    @PostMapping("/validUpdateTeacher")
    public Result<Boolean> validUpdateTeacher(@Valid @RequestBody TeacherDetailModifyDto teacherDetailModifyDto, @ApiIgnore @CurrWorker Worker worker) {
        return Result.ofSuccess(teacherManageService.validUpdateTeacher(teacherDetailModifyDto, worker));
    }

    @ApiOperation("保存聘书图片")
    @PostMapping("/saveExpandFileKey")
    public Result<String> saveExpandFileKey(@RequestBody @Valid TeacherExpandFileKeyDto teacherExpandFileKeyDto, @ApiIgnore @CurrWorker Worker worker) {
        return Result.ofSuccess(teacherManageService.saveExpandFileKey(teacherExpandFileKeyDto, worker));
    }

    @ApiOperation("分页查询导师")
    @PostMapping("/pageTeacher")
    public Result<PageRsp<TeacherBaseVo>> pageTeacher(@RequestBody @Valid PageReq<TeacherDto> pageReq) {
        return Result.ofSuccess(teacherManageService.pageTeacher(pageReq));
    }

    @ApiOperation("查询导师信息")
    @GetMapping("/getTeacherInfo")
    public Result<TeacherDetailVo> getTeacherInfo(@RequestParam("workerId") Integer workerId) {
        return Result.ofSuccess(teacherManageService.getTeacherInfo(workerId));
    }

    @ApiOperation("学员详情-荣誉&优势-荣誉")
    @PostMapping("/pageHonor")
    public Result<PageRsp<WorkerHonorVo>> pageHonor(@Valid @RequestBody PageReq<StuExpandDto> pageReq){
        return Result.ofSuccess(teacherManageService.pageHonor(pageReq));
    }

    @ApiOperation("学员详情-荣誉&优势-优势")
    @PostMapping("/getAdvantage")
    public Result<String> getAdvantage(@Valid @RequestBody StuExpandDto stuExpandDto){
        return Result.ofSuccess(teacherManageService.getAdvantage(stuExpandDto));
    }

    @ApiOperation("学员详情-集中培养")
    @PostMapping("/pageConcentratedCultivationExpand")
    public Result<PageRsp<ConcentratedCultivationVo>> pageConcentratedCultivationExpand(@RequestBody PageReq<StuExpandDto> pageReq){
        return Result.ofSuccess(teacherManageService.pageConcentratedCultivationExpand(pageReq));
    }

    @ApiOperation("学员详情-集中培养评价")
    @PostMapping("/evaluateConcentratedCultivation")
    public Result<String> evaluateConcentratedCultivation(@RequestBody EvaluateStuDto evaluateStuDto,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherManageService.evaluateConcentratedCultivation(evaluateStuDto,worker));
    }

    @ApiOperation("学员详情-导师评价")
    @PostMapping("/pageStuTraineeExpand")
    public Result<PageRsp<TeacherTraineeVo>> pageStuTraineeExpand(@RequestBody PageReq<TrackingCultivationDto> pageReq){
        return Result.ofSuccess(teacherManageService.pageStuTraineeExpand(pageReq));
    }

    @ApiOperation("学员详情-跟踪培养-自我培养")
    @PostMapping("/getSelfCultivation")
    public Result<SelfCultivationVo> getSelfCultivation(@RequestBody TrackingCultivationDto trackingCultivationDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherManageService.getSelfCultivation(trackingCultivationDto,worker));
    }

    @ApiOperation("学员详情-跟踪培养-自我培养-志愿服务 && 其他")
    @PostMapping("/getVolunteer")
    public Result<List<VolunteerVo>> getVolunteer(@RequestBody StuHonorDto stuExpandDto){
        return Result.ofSuccess(teacherManageService.getVolunteer(stuExpandDto));
    }

    @ApiOperation("学员详情-跟踪培养-作用发挥")
    @PostMapping("/pageExert")
    public Result<PageRsp<ExertVo>> pageExert(@RequestBody PageReq<ExertDto> req, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherManageService.pageExert(req,worker));
    }

    @ApiOperation("学员详情-跟踪培养-成长发展")
    @PostMapping("/pageGrowth")
    public Result<PageRsp<GrowthVo>> pageGrowth(@RequestBody PageReq<GroupCultivationDto> req, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(teacherManageService.pageGrowth(req,worker));
    }

    @ApiOperation("查询学员信息")
    @GetMapping("/getStudentInfo")
    public Result<List<StudentDetailVo>> getStudentInfo(@RequestParam("workerId") Integer workerId) {
        return teacherManageService.getStudentInfo(workerId);
    }

    @ApiOperation("审核导师")
    @PostMapping("/updateAuditTeacher")
    public Result<String> updateAuditTeacher(@RequestBody @Valid AuditTeacherDto auditTeacherDto, @ApiIgnore @CurrWorker Worker worker) {
        return Result.ofSuccess(teacherManageService.updateAuditTeacher(auditTeacherDto, worker));
    }

    @ApiOperation("批量删除导师校验 ")
    @PostMapping("/validateDelBatchTeacher")
    public Result<String> validateDelBatchTeacher(@RequestBody List<Integer> workerIds, @ApiIgnore @CurrWorker Worker worker) {
        return Result.ofSuccess(teacherManageService.validateDelBatchTeacher(workerIds, worker));
    }

    @ApiOperation("批量删除导师")
    @PostMapping("/delBatchTeacher")
    public Result<String> delBatchTeacher(@RequestBody List<Integer> workerIds, @ApiIgnore @CurrWorker Worker worker) {
        return Result.ofSuccess(teacherManageService.delBatchTeacher(workerIds, worker));
    }

    @ApiOperation("导出excel模板")
    @GetMapping("/exportTemplate")
    public void exportTemplate(HttpServletResponse response) {
        teacherManageService.exportTemplate(response);
    }

    @ApiOperation("导出导师-(通过workerId导出)")
    @PostMapping("/exportTeacherByIds")
    public void exportTeacher(@RequestBody List<Integer> workerIds) {
         teacherManageService.exportTeacher(workerIds);
    }

    @ApiOperation("导出导师信息-(条件导出)")
    @PostMapping("/batchExportTeacher")
    public void batchExportTeacher(@RequestBody TeacherDto teacherDto) {
         teacherManageService.batchExportTeacher(teacherDto);
    }

    @ApiOperation("Excel校验")
    @PostMapping(value = "/verifyExcel")
    public Result<ExcelDataInfo> verifyExcel(@RequestPart("file") MultipartFile file) {
        return teacherManageService.verifyExcel(file);
    }

    @ApiOperation("Excel导入导师数据")
    @PostMapping("/importTeacherData")
    public Result<String> importTeacherData(@ApiIgnore @CurrWorker Worker worker,@RequestParam("dataKey") String dataKey) {
        return teacherManageService.importTeacherData(worker,dataKey);
    }

    @ApiOperation("分页查询学生（未申请导师，未匹配导师）")
    @PostMapping("/pageNoTeacherStu")
    public Result<PageRsp<StudentVo>> pageNoTeacherStu(@Valid @RequestBody PageReq<Integer> req){
        return Result.ofSuccess(teacherManageService.pageNoTeacherStu(req));
    }

    @ApiOperation("在聘/解聘")
    @GetMapping("/updateStatus")
    public Result<String> updateStatus(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("workerId") Integer workerId, @RequestParam("expandStatus") Integer expandStatus) {
        return teacherManageService.updateStatus(worker, workerId, expandStatus);
    }

    @ApiOperation("查询已设置的学员")
    @GetMapping("/provisionedStu")
    public Result<List<StudentVo>> provisionedStu(@CurrWorker  @ApiIgnore Worker worker, @RequestParam("workerId") Integer workerId) {
        return teacherManageService.provisionedStu(worker, workerId);
    }

    @ApiOperation("设置学员")
    @PostMapping("/saveStudent")
    public Result<String> saveStudent(@CurrWorker  @ApiIgnore Worker worker,@Valid @RequestBody StudentDto dto) {
        return teacherManageService.saveStudent(worker, dto);
    }

    @ApiOperation(value = "查询待审核,审核不通过,审核通过")
    @PostMapping(path = "/searchNum")
    public Result<SearchNumVo> searchNum(@CurrWorker  @ApiIgnore Worker worker,Long orgId) {
        return Result.ofSuccess(teacherManageService.searchNum(worker,orgId));
    }
}
