package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.teachingRoute.*;
import com.crunii.micro.service.qm.service.TeachingRouteService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * @author denggl
 * date 2023/9/18
 */

@Api(tags = "教学路线web后台")
@RestController
@RequestMapping("/auth/YcqQmTeachingRouteController")
public class TeachingRouteController {
    @Autowired
    private TeachingRouteService ycqQmTeachingRouteService;

    @ApiOperation("web分页线路列表")
    @PostMapping("/pageSearchRoute")
    Result<PageRsp<TeachingRouteVo>> pageSearchRoute(@RequestBody PageReq<TeachingRouteDto> pageReq, @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(ycqQmTeachingRouteService.pageSearchRoute(pageReq, worker));

    }


    @ApiOperation("上下架")
    @GetMapping("/updateRouteState")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "routeId", value = "线路id", required = true),
            @ApiImplicitParam(name = "routeStatus", value = "1 上架 0 下架", required = true)
    })
    public Result<String> updateRouteState(@CurrWorker  @ApiIgnore Worker worker,@RequestParam Long routeId,@RequestParam Integer  routeStatus ) {
        ycqQmTeachingRouteService.updateRouteState(worker,routeId,routeStatus);

        return Result.ofSuccess();
    }


    @ApiOperation("web查看路线详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "routeId", value = "路线id", required = true)
    })
    @GetMapping("routeDetails")
    Result<TeachingRouteDetailsVo> routeDetails(@RequestParam Long routeId, @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(ycqQmTeachingRouteService.routeDetails(routeId, worker));
    }
    @ApiOperation("编辑线路信息")
    @PostMapping("/updateRoute")
    Result<String> updateRoute(@RequestBody @Validated TeachingRouteU teachingRouteU, @CurrWorker  @ApiIgnore Worker worker) {
        ycqQmTeachingRouteService.updateRoute(teachingRouteU, worker);
        return Result.ofSuccess();
    }

    @ApiOperation("删除路线（可批量）传路线id集合")
    @PostMapping("delRoute")
    Result<String> delRoute(@RequestBody List<Long> routeId, @CurrWorker  @ApiIgnore Worker worker) {
        ycqQmTeachingRouteService.delRoute(routeId, worker);
        return Result.ofSuccess();

    }

    @ApiOperation("获取线路的节点")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "routeId", value = "路线id", required = true)
    })
    @GetMapping("getListNode")
    Result<List<TeachingRouteNodeVo>> getListNode(@RequestParam Long routeId, @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(ycqQmTeachingRouteService.getlistNode(routeId, worker));
    }

    @ApiOperation("编辑线路节点")
    @PostMapping("updateRouteNode")
    Result<String> updateRouteNode(@RequestBody  @Validated List<TeachingRouteNodeDto> ycqQmTeachingRouteNodeVo, @CurrWorker  @ApiIgnore Worker worker) {
        ycqQmTeachingRouteService.updateRouteNode(ycqQmTeachingRouteNodeVo, worker);
        return Result.ofSuccess();
    }

    @ApiOperation("删除线路节点（可批量）传routeNodeId集合")
    @PostMapping("delRouteNode")
    Result<String> delRouteNode(@RequestBody List<Long> routeNodeId, @CurrWorker  @ApiIgnore Worker worker) {
        ycqQmTeachingRouteService.delRouteNode(routeNodeId, worker);
        return Result.ofSuccess();
    }
}
