package com.crunii.micro.service.qm.controller;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.trainingTrain.*;
import com.crunii.micro.service.qm.service.TrainingTrainService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * @author denggl
 * date 2023/9/19
 */

@Api(tags = "培训基地web后台")
@RestController
@RequestMapping("/auth/YcqQmTrainingTrainController")
public class TrainingTrainController {

    @Autowired
    private TrainingTrainService trainingTrainService;

    @ApiOperation("web分页查询基地信息")
    @PostMapping("/pageTrain")
    public Result<PageRsp<TrainingTrainVo>> pageTrain(@CurrWorker  @ApiIgnore Worker worker, @RequestBody PageReq<TrainingTrainDto> ycqQmTrainingTrainDto) {
        return Result.ofSuccess(trainingTrainService.pageTrain(worker,ycqQmTrainingTrainDto));
    }


    @ApiOperation("web查看基地详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "baseId", value = "基地id", required = true)
    })
    @GetMapping("/selectTrainDetails")
    public Result<TrainingTrainDetailsVo> selectTrinDetails(Long baseId, @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(trainingTrainService.selectTrinDetails(baseId,worker));
    }



    @ApiOperation("上下架")
    @PostMapping("/updateTrainState")

    public Result<String> updateTrainState(@CurrWorker  @ApiIgnore Worker worker, @RequestBody TrainStatusDto trainStatus) {
        trainingTrainService.updateTrainState(worker,trainStatus);
        return Result.ofSuccess();
    }




    @ApiOperation("更新基地信息")
    @PostMapping("/updateTrain")
    public Result<String> updateTrain(@CurrWorker  @ApiIgnore Worker worker,@RequestBody @Validated TrainingTrainU trainingTrainU) {
          trainingTrainService.updateTrain(worker, trainingTrainU);
        return Result.ofSuccess();
    }

    @ApiOperation("删除基地（可批量）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "baseId", value = "基地id", required = true)
    })
    @PostMapping ("/delTrain")
    public Result<String> delTrain(@RequestBody   List<Long> baseId, @CurrWorker  @ApiIgnore Worker worker) {
        trainingTrainService.delTrain(baseId,worker);
        return Result.ofSuccess();
    }

    @ApiOperation("查看基地打卡记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "baseId", value = "基地id", required = true)
    })
    @GetMapping("/selectClockRecord")
    public Result<List<TrainingTrainClockVo>> selectClockRecord(@RequestParam Long baseId, @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(trainingTrainService.selectClockRecord(baseId,worker));
    }




}
