package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.UserHonorDto;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.SelectUserHonorVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.SelectUserHonorVo;
import com.crunii.micro.service.qm.service.UserHonorService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @author xia
 * @version 1.0
 * @date 2023/9/18 0018 15:08:40
 */
@Api(tags = "用户荣誉相关")
@Validated
@RestController
@RequestMapping("/auth/UserHonorController")
public class UserHonorController {

	@Autowired UserHonorService userhonorService;

	/**
	 * 查询用户荣誉及擅长领域
	 */
	@ApiOperation("查询用户荣誉及擅长领域")
	@GetMapping("/selectUserHonor")
	public Result<SelectUserHonorVo> selectUserHonor(@CurrWorker  @ApiIgnore Worker worker) {
		return userhonorService.selectUserHonor(worker.getLoginId());
	}

	/**
	 * 新增或修改用户荣誉或擅长领域
	 */
	@ApiOperation("新增或修改用户荣誉")
	@PostMapping("/insertUserHonor")
	public Result<String> insertUserHonor(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @ApiParam UserHonorDto dto) {
		return userhonorService.insertUserHonor(worker,dto);
	}

	/**
	 * 删除用户荣誉或擅长领域
	 */
	@ApiOperation("删除用户荣誉")
	@ApiImplicitParam(name = "honorId", value = "荣誉主键id",required = true)
	@GetMapping("/deleteUserHonor")
	public Result<String> deleteUserHonor(@CurrWorker  @ApiIgnore Worker worker,Long honorId) {
		return userhonorService.deleteUserHonor(worker,honorId);
	}



}
