package com.crunii.micro.service.qm.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.org.OrgTreeReqDto;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameDto;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameVo;
import com.crunii.micro.service.qm.api.domain.dto.org.vo.OrgTreeVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.UserInfoDto;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.CheckUserInfoVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.ClassesLevelVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.ClassesTypeVo;
import com.crunii.micro.service.qm.service.UserInfoService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * @author xia
 * @version 1.0
 * @date 2023/9/18 0018 15:08:40
 */
@Api(tags = "用户信息录入相关")
@RestController
@RequestMapping("/auth/UserInfoController")
public class UserInfoController {

	@Autowired UserInfoService userInfoService;

	/**
	 * 判断用户是否已录入信息
	 */
	@ApiOperation("判断用户是否已录入信息")
	@GetMapping("/checkUserInfo")
	public Result<CheckUserInfoVo> checkUserInfo(@CurrWorker  @ApiIgnore Worker worker) {
		return userInfoService.checkUserInfo(worker.getLoginId());
	}

	/**
	 * 指标体系评价列表
	 */
	@ApiOperation("新增或修改用户信息")
	@PostMapping("/insertUserInfo")
	public Result<String> insertUserInfo(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @ApiParam @Validated UserInfoDto dto) {
		return userInfoService.insertUserInfo(worker,dto);
	}


	@ApiOperation(value = "查询所有班级包含的年份")
	@GetMapping( "/selectClassesYearByAll")
	public Result<List<String>> selectClassesYearByAll(@CurrWorker  @ApiIgnore Worker worker) {
		return userInfoService.selectClassesYearByAll();
	}

	@ApiOperation(value = "根据年份查询班级的所有级别")
	@ApiImplicitParam(name = "classesYear", value = "班级年份",required = true)
	@GetMapping( "/selectClassesLevelByYear")
	public Result<List<ClassesLevelVo>> selectClassesLevelByYear(@CurrWorker  @ApiIgnore Worker worker, String classesYear) {
		return userInfoService.selectClassesLevelByYear(classesYear);
	}

	@ApiOperation(value = "根据年度和班级级别查询班级类型")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "classesYear", value = "班级年度",required = true),
			@ApiImplicitParam(name = "levelCode", value = "班级级别编码",required = true),
			@ApiImplicitParam(name = "orgId", value = "班级组织id",required = true)
	})
	@GetMapping( "/selectClassesTypeByYearAndLevel")
	public Result<List<ClassesTypeVo>> selectClassesTypeByYearAndLevel(@CurrWorker  @ApiIgnore Worker worker, String classesYear, String levelCode,Long orgId) {
		return userInfoService.selectClassesTypeByYearAndLevel(worker,classesYear,levelCode,orgId);
	}

	/**
	 * 根据组织名称查询组织信息
	 */
	@ResponseBody
	@PostMapping("/pageOrgByOrgName")
	@ApiOperation("根据组织名称查询组织信息")
	public Result<PageRsp<PageOrgByOrgNameVo>> pageOrgByOrgName(@RequestBody @ApiParam @Validated PageReq<PageOrgByOrgNameDto> req){
		return Result.ofSuccess(userInfoService.pageOrgByOrgName(req));
	}

	@ApiOperation(value = "青马工程查询组织树")
	@RequestMapping(path = "/getOrgChildrenTree", method = RequestMethod.POST)
	@ResponseBody
	public Result<List<OrgTreeVo>> getOrgChildrenTree(@CurrWorker @ApiIgnore Worker worker, @RequestBody @ApiParam OrgTreeReqDto param) {
		Result<List<OrgTreeVo>> result = new Result<>();
		result.setData(userInfoService.getOrgChildrenTree(worker, param));
		return result;
	}

	@ApiOperation(value = "APP根据班级级别查询组织")
	@RequestMapping(path = "/getOrgChildrenTreeByOrgLevel", method = RequestMethod.POST)
	@ResponseBody
	public Result<List<OrgTreeVo>> getOrgChildrenTreeByOrgLevel(@CurrWorker @ApiIgnore Worker worker, @RequestBody @ApiParam OrgTreeReqDto param) {
		Result<List<OrgTreeVo>> result = new Result<>();
		result.setData(userInfoService.getOrgChildrenTreeByOrgLevel(worker, param));
		return result;
	}


}
