package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.qm.api.domain.dto.Book.AppBookInfo;
import com.crunii.micro.service.qm.api.domain.dto.Book.AppBookInfoVo;
import com.crunii.micro.service.qm.api.domain.dto.Book.AppSearchBookDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.AppBookListDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.AppBookListInfoDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.AppBookListVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/20 9:25
 */
@Repository
public class AppBookListDao {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String PAGE_BOOK_LIST_COUNT_SQL = "select COUNT(*) from YCQ_QM_BOOK_LIST a where 1 = 1 ";

    private static final String PAGE_BOOK_LIST_SQL = "select * from YCQ_QM_BOOK_LIST a where 1 = 1 ";

    private static final String PAGE_BOOK_SQL = "SELECT * FROM YCQ_QM_BOOK a WHERE 1=1 AND a.deleted=0 AND a.book_status=1 AND a.book_id IN (\n" +
            "SELECT book_id FROM YCQ_QM_BOOK_LIST_REL WHERE deleted=0 AND book_list_id=?) ";

    private static final String PAGE_BOOK_COUNT_SQL = "SELECT COUNT(*) FROM YCQ_QM_BOOK a WHERE 1=1 AND a.deleted=0 AND a.book_status=1 AND a.book_id IN (\n" +
            "SELECT book_id FROM YCQ_QM_BOOK_LIST_REL WHERE deleted=0 AND book_list_id=?) ";

    public PageRsp<AppBookListVo> pageList(PageReq<AppBookListDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_BOOK_LIST_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_BOOK_LIST_COUNT_SQL);
        Map<String, Object> map = new HashMap<>(2);

        // 书单名称
        if (StrUtil.isNotBlank(req.getQuery().getBookListName())) {
            sb.append(" and a.book_list_name like concat('%',:name,'%') ");
            sbCount.append(" and a.book_list_name like concat('%',:name,'%') ");
            map.put("name", req.getQuery().getBookListName());
        }
        // 书单主题
        if (ObjectUtil.isNotEmpty(req.getQuery().getBookListTheme())) {
            sb.append(" and a.book_list_theme in (:themes) ");
            sbCount.append(" and a.book_list_theme in (:themes) ");
            map.put("themes", req.getQuery().getBookListTheme());
        }
        // 逻辑删除
        sb.append(" and a.deleted =0 ");
        sbCount.append(" and a.deleted =0 ");

        int total = NPJT.queryForInt(sbCount.toString(), map);
        List<AppBookListVo> result;
        String dbType = NPJT.getDbType();
        if (total > 0) {
            sb.append(" order by a.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, map, sb, req);

            result = NPJT.queryForBeans(AppBookListVo.class, sb.toString(), map);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<AppBookListVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public PageRsp<AppBookInfo> pageBookList(PageReq<AppBookListInfoDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_BOOK_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_BOOK_COUNT_SQL);
        List<Object> params = new ArrayList<>();

        if (ObjectUtil.isNotEmpty(req.getQuery().getBookListId())) {
            params.add(req.getQuery().getBookListId());
        }

        int total = jt.queryForInt(sbCount.toString(), params.toArray());
        List<AppBookInfo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(" order by a.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jt.queryForBeans(AppBookInfo.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<AppBookInfo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public PageRsp<AppBookInfo> pageSearchBook(PageReq<AppSearchBookDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_BOOK_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_BOOK_COUNT_SQL);
        List<Object> params = new ArrayList<>();

        // 书单id
        if (ObjectUtil.isNotEmpty(req.getQuery().getBookListId())) {
            params.add(req.getQuery().getBookListId());
        }

        //书籍名称
        if (StrUtil.isNotBlank(req.getQuery().getBookName())) {
            sb.append(" and a.book_name like concat('%',?,'%') ");
            sbCount.append(" and a.book_name like concat('%',?,'%') ");
            params.add(req.getQuery().getBookName());
        }
        int total = jt.queryForInt(sbCount.toString(), params.toArray());
        List<AppBookInfo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(" order by a.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jt.queryForBeans(AppBookInfo.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<AppBookInfo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public AppBookInfoVo queryBookInfo(Long bookId){
        String sql = "SELECT book_id ,book_name ,book_author ,book_desc ,update_worker_name ,create_time,file_key FROM YCQ_QM_BOOK WHERE deleted = 0 AND book_status  = 1 AND  book_id = ?";
        return jt.queryForBean(AppBookInfoVo.class,sql,bookId);
    }

}
