package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.qm.api.domain.dto.courseware.AppCoursewareDto;
import com.crunii.micro.service.qm.api.domain.dto.courseware.AppCoursewareVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/20 16:21
 */
@Repository
public class AppCoursewareDao {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String PAGE_COURSEWARE_COUNT_SQL = "select COUNT(*) from YCQ_QM_COURSEWARE a where 1 = 1 ";

    private static final String PAGE_COURSEWARE_SQL = "select * from YCQ_QM_COURSEWARE a where 1 = 1 ";

    public PageRsp<AppCoursewareVo> pageList(PageReq<AppCoursewareDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_COURSEWARE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COURSEWARE_COUNT_SQL);
        Map<String, Object> map = new HashMap<>(3);
        // 课件主题
        if (ObjectUtil.isNotEmpty(req.getQuery().getCertificateTopic())) {
            sb.append(" and  a.certificate_topic in (:topics) ");
            sbCount.append(" and a.certificate_topic in (:topics) ");
            map.put("topics", req.getQuery().getCertificateTopic());
        }
        // 课件类型
        if (ObjectUtil.isNotEmpty(req.getQuery().getCertificateType())) {
            sb.append(" and  a.certificate_type in (:types) ");
            sbCount.append(" and a.certificate_type in (:types) ");
            map.put("types", req.getQuery().getCertificateType());
        }
        // 课件名称
        if (StrUtil.isNotBlank(req.getQuery().getCertificateName())) {
            sb.append(" and a.certificate_name like concat('%',:name,'%') ");
            sbCount.append(" and a.certificate_name like concat('%',:name,'%') ");
            map.put("name", req.getQuery().getCertificateName());
        }

        //上架状态
        sb.append(" and a.certificate_status =1 ");
        sbCount.append(" and a.certificate_status =1 ");

        // 逻辑删除
        sb.append(" and a.deleted =0 ");
        sbCount.append(" and a.deleted =0 ");

        int total = NPJT.queryForInt(sbCount.toString(), map);
        List<AppCoursewareVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(" order by a.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, map, sb, req);
            result = NPJT.queryForBeans(AppCoursewareVo.class, sb.toString(), map);
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<AppCoursewareVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

}
