package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.qm.api.domain.dto.belowExcellent.*;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameVo;
import com.crunii.micro.service.qm.util.ListUtil;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author xiayinqiang
 * @date 2023/9/27 0027 10:48:37
 */
@Repository
public class BelowExcellentDao {

	@Resource(name = "mainJT")
	private BaseJdbcTemplate jt;

	/**
	 * 根据学员id查询上级组织
	 *
	 * @param workerId 学员id
	 * @return PageOrgByOrgNameVo
	 */
	public PageOrgByOrgNameVo selectParentOrgByWorkerId(Integer workerId) {
		String sql = " SELECT C.org_id,C.org_code,C.org_name,C.org_type,C.org_level FROM YCQ_QM_USER_INFO A "
				+ "INNER JOIN TSM_ORG B ON A.org_id = B.org_id INNER JOIN TSM_ORG C ON B.parent_org_id = C.org_id "
				+ "WHERE a.deleted = 0 AND A.worker_id = ? ";
		return jt.queryForBean(PageOrgByOrgNameVo.class, sql, workerId);
	}

	/**
	 * 分页查询学员信息
	 *
	 * @param req 入参实体
	 * @param orgIdList
	 * @return PageRsp<PageBelowExcellentVo>
	 */
	public PageRsp<PageBelowExcellentVo> pageBelowExcellent(PageReq<PageBelowExcellentDto> req, List<Long> orgIdList) {
		StringBuilder sql = new StringBuilder(
				"SELECT A.id as userInfoId,A.worker_id,A.user_code,TW.worker_name,TW.mobile,O.org_id,O.org_name,A.worker_job,A.create_time,A.update_time,\n"
						+ "  CASE WHEN TSR.passCount>=2 THEN '2' WHEN TS.correlation_state=2 OR TS.correlation_state=1 THEN '3' WHEN TS.correlation_state=0 THEN '0' ELSE '1' END AS teacherMate,\n"
						+ " CASE WHEN TSR.passCount>=2 THEN '未匹配' WHEN TS.correlation_state=2 OR TS.correlation_state=1 THEN '已匹配' WHEN TS.correlation_state=0 THEN '待审核' ELSE '未选择' END AS teacherMateName,\n"
						+ " BX.recommend_time FROM YCQ_QM_USER_INFO A\n"
						+ " INNER JOIN YCQ_QM_BELOW_EXCELLEN BX ON A.worker_id = BX.stu_worker_id AND BX.deleted=0 AND BX.config_type = 0\n"
						+ " INNER JOIN TSM_WORKER TW ON A.worker_id = TW.login_id AND TW.state = 1\n"
						+ " INNER JOIN TSM_ORG O ON A.org_id = O.org_id AND O.state = 1\n"
						+ " LEFT JOIN (SELECT TSR.stu_worker_id,COUNT(TSR.stu_worker_id) AS passCount FROM YCQ_QM_TEACHER_STU_REJECT TSR \n"
						+ " WHERE TSR.deleted=0 GROUP BY TSR.stu_worker_id) TSR ON A.worker_id = TSR.stu_worker_id\n"
						+ " LEFT JOIN (SELECT TS.stu_worker_id,TS.correlation_state FROM YCQ_QM_TEACHER_STU TS\n"
						+ " WHERE TS.deleted=0 AND TS.binding=1 ) TS ON TS.stu_worker_id=A.worker_id");
		StringBuilder countSql = new StringBuilder("SELECT count(*) FROM YCQ_QM_USER_INFO A\n"
				+ " INNER JOIN YCQ_QM_BELOW_EXCELLEN BX ON A.worker_id = BX.stu_worker_id AND BX.deleted=0 AND BX.config_type = 0\n"
				+ " INNER JOIN TSM_WORKER TW ON A.worker_id = TW.login_id AND TW.state = 1\n"
				+ " INNER JOIN TSM_ORG O ON A.org_id = O.org_id AND O.state = 1\n"
				+ " LEFT JOIN (SELECT TSR.stu_worker_id,COUNT(TSR.stu_worker_id) AS passCount FROM YCQ_QM_TEACHER_STU_REJECT TSR \n"
				+ " WHERE TSR.deleted=0 GROUP BY TSR.stu_worker_id) TSR ON A.worker_id = TSR.stu_worker_id\n"
				+ " LEFT JOIN (SELECT TS.stu_worker_id,TS.correlation_state FROM YCQ_QM_TEACHER_STU TS\n"
				+ " WHERE TS.deleted=0 AND TS.binding=1 ) TS ON TS.stu_worker_id=A.worker_id ");
		List<Object> params = new ArrayList<>();

		//班级id
		if (ObjectUtil.isNotEmpty(req.getQuery().getClassesId())) {
			sql.append(" INNER JOIN YCQ_QM_USER_CLASSES_REL CR ON A.worker_id = CR.stu_worker_id AND CR.deleted = 0 AND CR.classes_id = ? ");
			countSql.append(" INNER JOIN YCQ_QM_USER_CLASSES_REL CR ON A.worker_id = CR.stu_worker_id AND CR.deleted = 0 AND CR.classes_id = ? ");
			params.add(req.getQuery().getClassesId());
		}

		sql.append(" WHERE A.deleted = 0 AND A.user_identity=1 ");
		countSql.append(" WHERE A.deleted = 0 AND A.user_identity=1 ");
		//组织id
		if (ListUtil.hasValue(orgIdList)) {
			String addSql = " and A.org_id in " +orgIdList.stream().map(i->" ? ").collect(Collectors.joining(",","(",")"));
			sql.append(addSql);
			countSql.append(addSql);
			params.addAll(orgIdList);
		}
//		if (ObjectUtil.isNotEmpty(req.getQuery().getOrgId())) {
//			sql.append(" and A.org_id = ? ");
//			countSql.append(" and A.org_id = ? ");
//			params.add(req.getQuery().getOrgId());
//		}
		//性别
		if (ObjectUtil.isNotEmpty(req.getQuery().getSex())) {
			sql.append(" and A.sex = ? ");
			countSql.append(" and A.sex = ? ");
			params.add(req.getQuery().getSex());
		}
		//职务
		if (ObjectUtil.isNotEmpty(req.getQuery().getWorkerJob())) {
			sql.append(" and A.worker_job like concat('%',?,'%') ");
			countSql.append(" and A.worker_job like concat('%',?,'%') ");
			params.add(req.getQuery().getWorkerJob());
		}
		//导师匹配状态
		if (ObjectUtil.isNotEmpty(req.getQuery().getTeacherMate())) {
			if (req.getQuery().getTeacherMate() == 2) {//未匹配:被拒绝两次(1个导师拒绝两次，或不同导师累计拒绝两次)
				sql.append(" AND TSR.passCount>=2 ");
				countSql.append(" AND TSR.passCount>=2 ");
			} else if (req.getQuery().getTeacherMate() == 3) {//已匹配:被导师接受申请或者被导师选中
				sql.append(" AND (TS.correlation_state=2 OR TS.correlation_state=1) ");
				countSql.append(" AND (TS.correlation_state=2 OR TS.correlation_state=1) ");
			} else if (req.getQuery().getTeacherMate() == 0) {//待审核:学员申请了导师,导师还未同意接收
				sql.append("  AND TS.correlation_state=0 ");
				countSql.append(" AND TS.correlation_state=0 ");
			} else if (req.getQuery().getTeacherMate() == 1) {//未选择:未进行导师选择或未被导师选择,或选择了一次被导师拒绝了
				sql.append("  AND (TSR.passCount<2 AND (TS.correlation_state=3 OR TS.stu_worker_id IS NULL)) ");
				countSql.append(" AND (TSR.passCount<2 AND (TS.correlation_state=3 OR TS.stu_worker_id IS NULL)) ");
			}
		}
		//学员编号
		if (ObjectUtil.isNotEmpty(req.getQuery().getUserCode())) {
			sql.append(" and A.user_code = ? ");
			countSql.append(" and A.user_code = ? ");
			params.add(req.getQuery().getUserCode());
		}
		//学员名称
		if (ObjectUtil.isNotEmpty(req.getQuery().getWorkerName())) {
			sql.append(" and TW.worker_name like concat('%',?,'%') ");
			countSql.append(" and TW.worker_name like concat('%',?,'%') ");
			params.add(req.getQuery().getWorkerName());
		}
		//学员手机
		if (ObjectUtil.isNotEmpty(req.getQuery().getMobile())) {
			sql.append(" and TW.mobile = ? ");
			countSql.append(" and TW.mobile = ? ");
			params.add(req.getQuery().getMobile());
		}
		//开始时间
		if (ObjectUtil.isNotEmpty(req.getQuery().getStartTime())) {
			sql.append(" AND BX.recommend_time >= ? ");
			countSql.append(" AND BX.recommend_time >= ? ");
			params.add(req.getQuery().getStartTime());
		}
		//结束时间
		if (ObjectUtil.isNotEmpty(req.getQuery().getEndTime())) {
			sql.append(" AND BX.recommend_time <= ? ");
			countSql.append(" AND BX.recommend_time <= ? ");
			params.add(req.getQuery().getEndTime());
		}

		int total = jt.queryForInt(countSql.toString(), params.toArray());
		List<PageBelowExcellentVo> pageBelowExcellentVos;
		String dbType = jt.getDbType();
		if (total > 0) {
			//根据数据库类型设置limit
			SqlUtil.appendLimit(dbType, params, sql, req);
			sql.append(" order by recommend_time desc ");
			pageBelowExcellentVos = jt.queryForBeans(PageBelowExcellentVo.class, sql.toString(), params.toArray());
		} else {
			pageBelowExcellentVos = new ArrayList<>(0);
		}
		PageRsp<PageBelowExcellentVo> page = new PageRsp<>();
		page.setPageNo(req.getPageNo());
		page.setPageSize(req.getPageSize());
		page.setTotalNum(total);
		page.setResult(pageBelowExcellentVos);
		// 为手机计算是否由下一页
		page.next();

		return page;

	}

	/**
	 * 导出数据
	 *
	 * @param dto 入参实体
	 * @return List<ExportByListVo>
	 */
	public List<ExportByListVo> exportByList(ExportByListDto dto) {
		List<Object> params = new ArrayList<>(dto.getUserInfoList());
		String sql =
				"SELECT A.id as userInfoId,A.worker_id,A.user_code,TW.worker_name,TW.mobile,O.org_name,A.worker_job,A.create_time,A.update_time,\n"
						+ " CASE WHEN TSR.passCount>=2 THEN '未匹配' WHEN TS.correlation_state=2 OR TS.correlation_state=1 THEN '已匹配' ELSE '未选择' END AS correlationStateDesc,\n"
						+ " BX.recommend_time FROM YCQ_QM_USER_INFO A\n"
						+ " INNER JOIN YCQ_QM_BELOW_EXCELLEN BX ON A.worker_id = BX.stu_worker_id AND BX.deleted=0 AND BX.config_type = 0\n"
						+ " INNER JOIN TSM_WORKER TW ON A.worker_id = TW.login_id AND TW.state = 1\n"
						+ " INNER JOIN TSM_ORG O ON A.org_id = O.org_id AND O.state = 1\n"
						+ " LEFT JOIN (SELECT TSR.stu_worker_id,COUNT(TSR.stu_worker_id) AS passCount FROM YCQ_QM_TEACHER_STU_REJECT TSR \n"
						+ " WHERE TSR.deleted=0 GROUP BY TSR.stu_worker_id) TSR ON A.worker_id = TSR.stu_worker_id\n"
						+ " LEFT JOIN (SELECT TS.stu_worker_id,TS.correlation_state FROM YCQ_QM_TEACHER_STU TS\n"
						+ " WHERE TS.deleted=0 AND TS.binding=1 ) TS ON TS.stu_worker_id=A.worker_id WHERE A.deleted = 0 AND A.user_identity=1 ";
		if (!CollectionUtils.isEmpty(dto.getUserInfoList())) {
			sql = sql + " AND A.id IN " + dto.getUserInfoList().stream().map(i -> " ? ").collect(Collectors.joining(",", "(", ")"));
		}
		return jt.queryForBeans(ExportByListVo.class, sql, params.toArray());
	}

	/**
	 * 根据学员id集合查询所有班级信息
	 *
	 * @param workerIdList 学员id集合
	 * @return List<PageBelowExecllentClassesVo>
	 */
	public List<PageBelowExecllentClassesVo> selectClassesInfoByWorkerIds(List<Integer> workerIdList) {
		List<Object> params = new ArrayList<>(workerIdList);
		String sql =
				"  SELECT C.org_id as classesOrgId,O.org_name as classesOrgName,CR.classes_id,C.classes_code,CR.stu_worker_id,D.type_code,D.type_name,D.level_code,D.level_name,C.classes_year\n"
						+ " FROM YCQ_QM_USER_CLASSES_REL CR\n"
						+ " INNER JOIN YCQ_QM_CLASSES_INFO C ON C.classes_id = CR.classes_id AND C.deleted=0\n"
						+ " INNER JOIN YCQ_QM_CLASSES_TYPE D ON C.classes_type_id = D.classes_type_id AND D.deleted=0\n"
						+ " INNER JOIN TSM_ORG O ON C.org_id = O.org_id AND O.state = 1\n"
						+ " WHERE CR.deleted = 0 "
						+ " AND CR.stu_worker_id IN " + workerIdList.stream().map(i -> " ? ").collect(Collectors.joining(",", "(", ")"));
		return jt.queryForBeans(PageBelowExecllentClassesVo.class, sql, params.toArray());
	}

	/**
	 * 根据学员id查询推荐入党记录
	 *
	 * @param workerId 学员id
	 */
	public List<SelectIntoPartyVo> selectIntoPartyByWorkerId(Integer workerId) {
		String sql = " SELECT A.record_id,A.stu_worker_id,A.worker_job,A.recommend_worker_id,A.recommend_reason,A.recommend_time FROM YCQ_QM_INTO_PARTY A WHERE A.deleted=0 and A.stu_worker_id = ? ";
		return jt.queryForBeans(SelectIntoPartyVo.class, sql, workerId);
	}

	public PageRsp<PageStuRefuseTwiceVo> pageStuRefuseTwice(PageReq<PageStuRefuseTwiceDto> req, List<Long> orgIdList) {
		StringBuilder sql = new StringBuilder(
				"SELECT A.id as userInfoId,A.worker_id,A.user_code,TW.worker_name,TW.mobile,O.org_id,O.org_name,A.worker_job,A.create_time,A.update_time,\n"
						+ "  CASE WHEN TSR.passCount>=2 THEN '2' WHEN TS.correlation_state=2 OR TS.correlation_state=1 THEN '3' WHEN TS.correlation_state=0 THEN '0' ELSE '1' END AS teacherMate,\n"
						+ " CASE WHEN TSR.passCount>=2 THEN '未匹配' WHEN TS.correlation_state=2 OR TS.correlation_state=1 THEN '已匹配' WHEN TS.correlation_state=0 THEN '待审核' ELSE '未选择' END AS teacherMateName\n"
						+ " FROM YCQ_QM_USER_INFO A\n"
					    + " INNER JOIN TSM_WORKER TW ON A.worker_id = TW.login_id AND TW.state = 1\n"
						+ " INNER JOIN TSM_ORG O ON A.org_id = O.org_id AND O.state = 1\n"
						+ " LEFT JOIN (SELECT TSR.stu_worker_id,COUNT(TSR.stu_worker_id) AS passCount FROM YCQ_QM_TEACHER_STU_REJECT TSR \n"
						+ " WHERE TSR.deleted=0 GROUP BY TSR.stu_worker_id) TSR ON A.worker_id = TSR.stu_worker_id\n"
						+ " LEFT JOIN (SELECT TS.stu_worker_id,TS.correlation_state FROM YCQ_QM_TEACHER_STU TS\n"
						+ " WHERE TS.deleted=0 AND TS.binding=1 ) TS ON TS.stu_worker_id=A.worker_id");
		StringBuilder countSql = new StringBuilder("SELECT count(*) FROM YCQ_QM_USER_INFO A\n"
				+ " INNER JOIN TSM_WORKER TW ON A.worker_id = TW.login_id AND TW.state = 1\n"
				+ " INNER JOIN TSM_ORG O ON A.org_id = O.org_id AND O.state = 1\n"
				+ " LEFT JOIN (SELECT TSR.stu_worker_id,COUNT(TSR.stu_worker_id) AS passCount FROM YCQ_QM_TEACHER_STU_REJECT TSR \n"
				+ " WHERE TSR.deleted=0 GROUP BY TSR.stu_worker_id) TSR ON A.worker_id = TSR.stu_worker_id\n"
				+ " LEFT JOIN (SELECT TS.stu_worker_id,TS.correlation_state FROM YCQ_QM_TEACHER_STU TS\n"
				+ " WHERE TS.deleted=0 AND TS.binding=1 ) TS ON TS.stu_worker_id=A.worker_id ");
		List<Object> params = new ArrayList<>();

		//班级id
		if (ObjectUtil.isNotEmpty(req.getQuery().getClassesId())) {
			sql.append(" INNER JOIN YCQ_QM_USER_CLASSES_REL CR ON A.worker_id = CR.stu_worker_id AND CR.deleted = 0 AND CR.classes_id = ? ");
			countSql.append(" INNER JOIN YCQ_QM_USER_CLASSES_REL CR ON A.worker_id = CR.stu_worker_id AND CR.deleted = 0 AND CR.classes_id = ? ");
			params.add(req.getQuery().getClassesId());
		}

		sql.append(" WHERE A.deleted = 0 AND A.user_identity=1 AND TSR.passCount>=2 ");
		countSql.append(" WHERE A.deleted = 0 AND A.user_identity=1 AND TSR.passCount>=2 ");
		//组织id
		if (ListUtil.hasValue(orgIdList)) {
			String addSql = " and A.org_id in " +orgIdList.stream().map(i->" ? ").collect(Collectors.joining(",","(",")"));
			sql.append(addSql);
			countSql.append(addSql);
			params.addAll(orgIdList);
		}
//		if (ObjectUtil.isNotEmpty(req.getQuery().getOrgId())) {
//			sql.append(" and A.org_id = ? ");
//			countSql.append(" and A.org_id = ? ");
//			params.add(req.getQuery().getOrgId());
//		}
		//性别
		if (ObjectUtil.isNotEmpty(req.getQuery().getSex())) {
			sql.append(" and A.sex = ? ");
			countSql.append(" and A.sex = ? ");
			params.add(req.getQuery().getSex());
		}
		//职务
		if (ObjectUtil.isNotEmpty(req.getQuery().getWorkerJob())) {
			sql.append(" and A.worker_job like concat('%',?,'%') ");
			countSql.append(" and A.worker_job like concat('%',?,'%') ");
			params.add(req.getQuery().getWorkerJob());
		}
		//导师匹配状态
		if (ObjectUtil.isNotEmpty(req.getQuery().getTeacherMate())) {
			if (req.getQuery().getTeacherMate() == 2) {//未匹配:被拒绝两次(1个导师拒绝两次，或不同导师累计拒绝两次)
				sql.append(" AND TSR.passCount>=2 ");
				countSql.append(" AND TSR.passCount>=2 ");
			} else if (req.getQuery().getTeacherMate() == 3) {//已匹配:被导师接受申请或者被导师选中
				sql.append(" AND (TS.correlation_state=2 OR TS.correlation_state=1) ");
				countSql.append(" AND (TS.correlation_state=2 OR TS.correlation_state=1) ");
			} else if (req.getQuery().getTeacherMate() == 0) {//待审核:学员申请了导师,导师还未同意接收
				sql.append("  AND TS.correlation_state=0 ");
				countSql.append(" AND TS.correlation_state=0 ");
			} else if (req.getQuery().getTeacherMate() == 1) {//未选择:未进行导师选择或未被导师选择,或选择了一次被导师拒绝了
				sql.append("  AND (TSR.passCount<2 AND (TS.correlation_state=3 OR TS.stu_worker_id IS NULL)) ");
				countSql.append(" AND (TSR.passCount<2 AND (TS.correlation_state=3 OR TS.stu_worker_id IS NULL)) ");
			}
		}
		//学员编号
		if (ObjectUtil.isNotEmpty(req.getQuery().getUserCode())) {
			sql.append(" and A.user_code = ? ");
			countSql.append(" and A.user_code = ? ");
			params.add(req.getQuery().getUserCode());
		}
		//学员名称
		if (ObjectUtil.isNotEmpty(req.getQuery().getWorkerName())) {
			sql.append(" and TW.worker_name like concat('%',?,'%') ");
			countSql.append(" and TW.worker_name like concat('%',?,'%') ");
			params.add(req.getQuery().getWorkerName());
		}
		//学员手机
		if (ObjectUtil.isNotEmpty(req.getQuery().getMobile())) {
			sql.append(" and TW.mobile = ? ");
			countSql.append(" and TW.mobile = ? ");
			params.add(req.getQuery().getMobile());
		}
		//开始时间
		if (ObjectUtil.isNotEmpty(req.getQuery().getStartTime())) {
			sql.append(" AND A.create_time >= ? ");
			countSql.append(" AND A.create_time >= ? ");
			params.add(req.getQuery().getStartTime());
		}
		//结束时间
		if (ObjectUtil.isNotEmpty(req.getQuery().getEndTime())) {
			sql.append(" AND A.create_time <= ? ");
			countSql.append(" AND A.create_time <= ? ");
			params.add(req.getQuery().getEndTime());
		}

		int total = jt.queryForInt(countSql.toString(), params.toArray());
		List<PageStuRefuseTwiceVo> voList;
		String dbType = jt.getDbType();
		if (total > 0) {
			//根据数据库类型设置limit
			SqlUtil.appendLimit(dbType, params, sql, req);
			sql.append(" order by A.update_time,A.create_time desc ");
			voList = jt.queryForBeans(PageStuRefuseTwiceVo.class, sql.toString(), params.toArray());
		} else {
			voList = new ArrayList<>(0);
		}
		PageRsp<PageStuRefuseTwiceVo> page = new PageRsp<>();
		page.setPageNo(req.getPageNo());
		page.setPageSize(req.getPageSize());
		page.setTotalNum(total);
		page.setResult(voList);
		// 为手机计算是否由下一页
		page.next();

		return page;
	}

	public int selectTeachStu(Integer stuWorkerId) {
		String sql = "SELECT count(*) FROM YCQ_QM_TEACHER_STU a WHERE a.deleted = 0 AND (a.correlation_state=2 OR a.correlation_state=1) AND a.stu_worker_id = ? ";
		return jt.queryForInt(sql, stuWorkerId);
	}
}
