package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.qm.api.domain.dto.Book.BookDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.BookListInfo;
import com.crunii.micro.service.qm.api.domain.dto.Book.BookVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/18 19:03
 */
@Repository
public class BookDao {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String PAGE_BOOK_COUNT_SQL = "select COUNT(*) from YCQ_QM_BOOK a where 1 = 1 ";

    private static final String PAGE_BOOK_SQL = "select * from YCQ_QM_BOOK a where 1 = 1 ";

    public PageRsp<BookVo> pageList(PageReq<BookDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_BOOK_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_BOOK_COUNT_SQL);
        List<Object> params = new ArrayList<>();

        // 书籍名称
        if (StrUtil.isNotBlank(req.getQuery().getBookName())) {
            sb.append(" and a.book_name like concat('%',?,'%') ");
            sbCount.append(" and a.book_name like concat('%',?,'%') ");
            params.add(req.getQuery().getBookName());
        }
        // 资源编号
        if (StrUtil.isNotBlank(req.getQuery().getBookCode())) {
            sb.append(" and a.book_code like concat('%',?,'%') ");
            sbCount.append(" and a.book_code like concat('%',?,'%') ");
            params.add(req.getQuery().getBookCode());
        }
        // 上架下架
        if (Objects.nonNull(req.getQuery().getBookStatus())) {
            sb.append(" and  a.book_status = ? ");
            sbCount.append(" and a.book_status = ? ");
            params.add(req.getQuery().getBookStatus());
        }
        // 书籍作者
        if (StrUtil.isNotBlank(req.getQuery().getBookAuthor())) {
            sb.append(" and a.book_author like concat('%',?,'%') ");
            sbCount.append(" and a.book_author like concat('%',?,'%') ");
            params.add(req.getQuery().getBookAuthor());
        }
        // 上传人姓名
        if (StrUtil.isNotBlank(req.getQuery().getUpdateWorkerName())) {
            sb.append(" and a.update_worker_name like concat('%',?,'%') ");
            sbCount.append(" and a.update_worker_name like concat('%',?,'%') ");
            params.add(req.getQuery().getUpdateWorkerName());
        }
        // 开始时间
        if (StrUtil.isNotBlank(req.getQuery().getStartTime())) {
            sb.append(" and a.create_time >= ? ");
            sbCount.append(" and a.create_time >= ? ");
            params.add(req.getQuery().getStartTime());
        }
        // 结束时间
        if (StrUtil.isNotBlank(req.getQuery().getEndTime())) {
            sb.append(" and a.create_time <= ? ");
            sbCount.append(" and a.create_time <= ? ");
            params.add(req.getQuery().getEndTime());
        }
        // 逻辑删除
        sb.append(" and a.deleted =0 ");
        sbCount.append(" and a.deleted =0 ");

        int total = jt.queryForInt(sbCount.toString(), params.toArray());
        List<BookVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(" order by a.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jt.queryForBeans(BookVo.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<BookVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public List<BookListInfo> queryBookListInfo(Long bookId) {
        String sql = "SELECT book_list_id,book_list_name,book_list_theme,book_list_status,create_time FROM YCQ_QM_BOOK_LIST WHERE deleted=0 AND book_list_id IN (\n" +
                "SELECT book_list_id FROM YCQ_QM_BOOK_LIST_REL WHERE book_id= ? AND deleted=0)";
        return jt.queryForBeans(BookListInfo.class, sql, bookId);
    }
}
