package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.qm.api.domain.dto.Book.BookInfo;
import com.crunii.micro.service.qm.api.domain.dto.Book.BookInfoVo;
import com.crunii.micro.service.qm.api.domain.dto.bookList.AppBookListInfoDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.BookListCount;
import com.crunii.micro.service.qm.api.domain.dto.bookList.BookListDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.BookListVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/19 14:19
 */
@Repository
public class BookListDao {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String PAGE_BOOK_LIST_COUNT_SQL = "select COUNT(*) from YCQ_QM_BOOK_LIST a where 1 = 1 ";
    private static final String PAGE_BOOK_LIST_SQL = "select * from YCQ_QM_BOOK_LIST a where 1 = 1 ";

    private static final String QUERY_BOOK_LIST_COUNT_SQL = "SELECT COUNT(*) FROM YCQ_QM_BOOK a WHERE a.deleted=0 AND a.book_status=1 AND a.book_id IN (\n" +
            "SELECT book_id FROM YCQ_QM_BOOK_LIST_REL WHERE deleted=0 AND book_list_id=?)";

    private static final String QUERY_BOOK_LIST_SQL = "SELECT a.book_id,a.book_name FROM YCQ_QM_BOOK a WHERE a.deleted=0 AND a.book_status=1 AND a.book_id IN (\n" +
            "SELECT book_id FROM YCQ_QM_BOOK_LIST_REL WHERE deleted=0 AND book_list_id=?)";

    public PageRsp<BookListVo> pageList(PageReq<BookListDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_BOOK_LIST_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_BOOK_LIST_COUNT_SQL);
        List<Object> params = new ArrayList<>();

        // 书单主题
        if(ObjectUtil.isNotEmpty(req.getQuery().getBookListTheme())){
            sb.append(" and  a.book_list_theme = ? ");
            sbCount.append(" and a.book_list_theme = ? ");
            params.add(req.getQuery().getBookListTheme());
        }
        // 书单名称
        if (StrUtil.isNotBlank(req.getQuery().getBookListName())) {
            sb.append(" and a.book_list_name like concat('%',?,'%') ");
            sbCount.append(" and a.book_list_name like concat('%',?,'%') ");
            params.add(req.getQuery().getBookListName());
        }
        // 资源编号
        if (StrUtil.isNotBlank(req.getQuery().getBookListCode())) {
            sb.append(" and a.book_list_code like concat('%',?,'%') ");
            sbCount.append(" and a.book_list_code like concat('%',?,'%') ");
            params.add(req.getQuery().getBookListCode());
        }
        // 上架下架
        if (Objects.nonNull(req.getQuery().getBookListStatus())) {
            sb.append(" and  a.book_list_status = ? ");
            sbCount.append(" and a.book_list_status = ? ");
            params.add(req.getQuery().getBookListStatus());
        }
        // 开始时间
        if (StrUtil.isNotBlank(req.getQuery().getStartTime())) {
            sb.append(" and a.create_time >= ? ");
            sbCount.append(" and a.create_time >= ? ");
            params.add(req.getQuery().getStartTime());
        }
        // 结束时间
        if (StrUtil.isNotBlank(req.getQuery().getEndTime())) {
            sb.append(" and a.create_time <= ? ");
            sbCount.append(" and a.create_time <= ? ");
            params.add(req.getQuery().getEndTime());
        }
        // 逻辑删除
        sb.append(" and a.deleted =0 ");
        sbCount.append(" and a.deleted =0 ");

        int total = jt.queryForInt(sbCount.toString(), params.toArray());
        List<BookListVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(" order by a.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jt.queryForBeans(BookListVo.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<BookListVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public List<BookListCount> queryBookListCount(List<Long> bookListIds) {
        String sql = "SELECT book_list_id as bookListId ,COUNT(1) as associatedBook FROM YCQ_QM_BOOK_LIST_REL WHERE deleted = 0 and book_list_id IN (:bookListIds) GROUP BY book_list_id";
        Map<String, Object> map = new HashMap<>(2);
        map.put("bookListIds", bookListIds);
        return NPJT.queryForBeans(BookListCount.class, sql, map);
    }

    public List<BookInfo> queryBookInfo(Long bookListId){
        String sql = "SELECT b.rel_id,a.book_name,a.book_author,a.book_desc,a.create_time,a.file_key FROM YCQ_QM_BOOK a INNER JOIN YCQ_QM_BOOK_LIST_REL b ON a.book_id=b.book_id AND  b.deleted=0 AND b.book_list_id=? WHERE a.deleted=0";
        return jt.queryForBeans(BookInfo.class,sql,bookListId);
    }

   public PageRsp<BookInfoVo> queryBookInfos(PageReq<AppBookListInfoDto> req){
        StringBuilder sb = new StringBuilder(QUERY_BOOK_LIST_SQL);

        StringBuilder sbCount = new StringBuilder(QUERY_BOOK_LIST_COUNT_SQL);

        List<Object> params = new ArrayList<>();
        // 书单id
        if (ObjectUtil.isNotEmpty(req.getQuery().getBookListId())) {
            params.add(req.getQuery().getBookListId());
        }
        int total = jt.queryForInt(sbCount.toString(), params.toArray());
        List<BookInfoVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(" order by a.book_name ");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jt.queryForBeans(BookInfoVo.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<BookInfoVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;

    }
}
