package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.entity.main.YcqQmCourseSectionRel;
import com.crunii.micro.service.qm.api.domain.dto.appFoster.QueryClassesCourseVO;
import com.crunii.micro.service.qm.api.domain.dto.appFoster.QueryEvaluateStuVO;
import com.crunii.micro.service.qm.api.domain.dto.appFoster.QuerySectionVO;
import com.crunii.micro.service.qm.api.domain.dto.classesMgr.*;
import jodd.util.StringUtil;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: LG
 * @date: 2023/9/18 19:42
 */
@Repository
public class ClassesMgrDao {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;

    public List<QueryClassesTypeVO> queryClassesType(String levelCode){
        String sql = "select * from YCQ_QM_CLASSES_TYPE where deleted = 0 and level_code = ? ";
        return jt.queryForBeans(QueryClassesTypeVO.class,sql,levelCode);
    }

    public List<QueryClassesTypeVO> queryClassesLevel(List<String> orgLevel){
        String collect = orgLevel.stream().map(i -> "?").collect(Collectors.joining(",", "(", ")"));
        String sql = "select level_code ,level_name,level_org_tier from YCQ_QM_CLASSES_TYPE where deleted = 0 and level_code in "+ collect +" group by level_code,level_name,level_org_tier ";
        return jt.queryForBeans(QueryClassesTypeVO.class,sql,orgLevel.toArray());
    }

    public List<QueryClassesTypeVO> queryClassesLevel(){
        String sql = "select level_code ,level_name,level_org_tier from YCQ_QM_CLASSES_TYPE where deleted = 0 group by level_code,level_name,level_org_tier ";
        return jt.queryForBeans(QueryClassesTypeVO.class,sql);
    }

    public QueryClassesVO queryClassesDetail(Long classesId){
        String sql = "select a.*,c.org_name,b.level_code,b.level_name,b.type_code,b.type_name,(select count(*) from YCQ_QM_USER_CLASSES_REL where deleted=0 and classes_id = a.classes_id ) as classesStuNum  from YCQ_QM_CLASSES_INFO a left join YCQ_QM_CLASSES_TYPE b on a.classes_type_id=b.classes_type_id left join TSM_ORG c on a.org_id=c.org_id where a.deleted = 0 and a.classes_id = ? ";
        return jt.queryForBean(QueryClassesVO.class,sql,classesId);
    }

    public PageRsp<QueryClassesVO> queryClasses(PageReq<QueryClassesDTO> req){
        QueryClassesDTO query = req.getQuery();
        String sql = "select a.*,c.org_name,b.level_code,b.level_name,b.type_code,b.type_name,(select count(*) from YCQ_QM_USER_CLASSES_REL as c right join YCQ_QM_USER_INFO as d on c.stu_worker_id =d.worker_id  where c.deleted=0 and d.deleted = 0 and c.classes_id = a.classes_id ) as classesStuNum  from YCQ_QM_CLASSES_INFO a left join YCQ_QM_CLASSES_TYPE b on a.classes_type_id=b.classes_type_id left join TSM_ORG c on a.org_id=c.org_id where a.deleted = 0 ";
        String sqlCount = "select count(*) from YCQ_QM_CLASSES_INFO a left join YCQ_QM_CLASSES_TYPE b on a.classes_type_id=b.classes_type_id where a.deleted = 0  ";
        ArrayList<Object> params = new ArrayList<>(20);

        if (ObjectUtil.isNotEmpty(query.getOrgId())){
            sql = sql + " and a.org_id = ? ";
            sqlCount = sqlCount + " and a.org_id = ? ";
            params.add(query.getOrgId());
        }

        if (StringUtil.isNotBlank(query.getClassesYear())){
            sql = sql + " and a.classes_year = ? ";
            sqlCount = sqlCount + " and a.classes_year = ? ";
            params.add(query.getClassesYear());
        }
        if (StringUtil.isNotBlank(query.getClassesLevelCode()) && StringUtil.isBlank(query.getClassesTypeCode())){
            sql = sql + " and a.classes_type_id in (select classes_type_id from  YCQ_QM_CLASSES_TYPE where level_code = ?) ";
            sqlCount = sqlCount + " and a.classes_type_id in (select classes_type_id from  YCQ_QM_CLASSES_TYPE where level_code = ?) ";
            params.add(query.getClassesLevelCode());
        }
        if (StringUtil.isNotBlank(query.getClassesTypeCode())){
            sql = sql + " and a.classes_type_id in (select classes_type_id from  YCQ_QM_CLASSES_TYPE where type_code = ?) ";
            sqlCount = sqlCount + " and a.classes_type_id in (select classes_type_id from  YCQ_QM_CLASSES_TYPE where type_code = ?) ";
            params.add(query.getClassesTypeCode());
        }
        if (ObjectUtil.isNotEmpty(query.getClassesStatus())){
            sql = sql + " and a.classes_status = ? ";
            sqlCount = sqlCount + " and a.classes_status = ? ";
            params.add(query.getClassesStatus());
        }
        if (ObjectUtil.isNotEmpty(query.getStartTime())){
            sql = sql +" and a.create_time >=? ";
            sqlCount = sqlCount +" and a.create_time >=? ";
            params.add(query.getStartTime());
        }
        if (ObjectUtil.isNotEmpty(query.getEndTime())){
            sql = sql +" and a.create_time <=? ";
            sqlCount = sqlCount +" and a.create_time <=? ";
            params.add(query.getEndTime());
        }
        sql = sql + " order by a.create_time desc ";
        return queryPageBase(req, sql, sqlCount, QueryClassesVO.class, params);
    }

    public void deleteClasses(List<Long> classesId){
        String collect = classesId.stream().map(i -> "?").collect(Collectors.joining(",", "(", ")"));
        String sql = "update YCQ_QM_CLASSES_INFO set deleted = 1 where deleted = 0 and classes_id in "+collect;
        jt.update(sql,classesId.toArray());
    }

    public void deleteSection(Long courseId){
        String sql = "update YCQ_QM_COURSE_SECTION set deleted = 1 where course_id = ? ";
        jt.update(sql,courseId);
    }

    public void deleteCourse(Long courseId){
        String sql = "update YCQ_QM_COURSE set deleted = 1 where course_id = ? ";
        jt.update(sql,courseId);
    }

    public Integer classesStuNum(Long classesId){
        String sql = "select count(*) from YCQ_QM_USER_CLASSES_REL where deleted = 0 and classes_id = ? ";
        return jt.queryForRawObject(Integer.class,sql,classesId);
    }

    public String queryClassesCode(Long classesId){
        String sql = "select classes_code from YCQ_QM_CLASSES_INFO where deleted = 0 and classes_id = ? ";
        return jt.queryForRawObject(String.class,sql,classesId);
    }

    public Integer courseStuNum(Long courseId){
        String sql = "select count(*) from YCQ_QM_USER_COURSE where deleted = 0 and course_id = ? ";
        return jt.queryForRawObject(Integer.class,sql,courseId);
    }

    public PageRsp<QueryCourseVO> queryCourse(PageReq<QueryCourseDTO> req){
        QueryCourseDTO query = req.getQuery();
        String sql = "select a.* from YCQ_QM_COURSE a  where a.deleted = 0  ";
        String sqlCount = "select count(*) from YCQ_QM_COURSE a where a.deleted = 0 ";
        ArrayList<Object> params = new ArrayList<>(20);
        if (ObjectUtil.isNotEmpty(query.getOrgId())){
            sql = sql + " and a.org_id = ? " ;
            sqlCount = sqlCount + " and a.org_id = ? " ;
            params.add(query.getOrgId());
        }
        if (ObjectUtil.isNotEmpty(query.getClassesId())){
            sql = sql + " and a.classes_id = ? ";
            sqlCount = sqlCount +" and a.classes_id = ? ";
            params.add(query.getClassesId());
        }
        if (StringUtil.isNotBlank(query.getCourseSubject())){
            sql = sql + " and a.course_subject = ? ";
            sqlCount = sqlCount +" and a.course_subject = ? ";
            params.add(query.getCourseSubject());
        }
        if (ObjectUtil.isNotEmpty(query.getCourseState())){
            //已结束
            if(query.getCourseState().equals(CommonConstants.STATE_INVALID)){
                sql = sql + " and now()>a.start_time and now()>a.end_time ";
                sqlCount = sqlCount +" and now()>a.start_time and now()>a.end_time ";
            }
            //即将开始
            if (query.getCourseState().equals(CommonConstants.MENU_TYPE_MENU)){
                sql = sql + " and now()<a.start_time  ";
                sqlCount = sqlCount +" and now()<a.start_time  ";
            }
            //正在进行
            if (query.getCourseState().equals(CommonConstants.MENU_TYPE_BUTTON)){
                sql = sql + "  and now()>a.start_time and now()<a.end_time  ";
                sqlCount = sqlCount +"  and now()>a.start_time and now()<a.end_time  ";
            }
        }
        if (StringUtil.isNotBlank(query.getCourseCode())){
            sql = sql + " and a.course_code = ? ";
            sqlCount = sqlCount +" and a.course_code = ? ";
            params.add(query.getCourseCode());
        }
        if (StringUtil.isNotBlank(query.getCourseName())){
            sql = sql + " and a.course_name like concat('%',?,'%') ";
            sqlCount = sqlCount +" and a.course_name like concat('%',?,'%') ";
            params.add(query.getCourseName());
        }
        if (ObjectUtil.isNotEmpty(query.getStartTime())){
            sql = sql +" and a.create_time >=? ";
            sqlCount = sqlCount +" and a.create_time >=? ";
            params.add(query.getStartTime());
        }
        if (ObjectUtil.isNotEmpty(query.getEndTime())){
            sql = sql +" and a.create_time <=? ";
            sqlCount = sqlCount +" and a.create_time <=? ";
            params.add(query.getEndTime());
        }


        sql = sql + " order by a.create_time desc ";
        return queryPageBase(req,sql,sqlCount,QueryCourseVO.class,params);
    }

    public List<SectionVO> querySection(Long courseId){
        String sql = " select *, TO_CHAR(section_start_time,'yyyy-mm-dd') as dateDay,TO_CHAR(section_start_time,'Hh24:mi') as hmStarTime ,TO_CHAR(section_end_time,'Hh24:mi') as hmEndTime from YCQ_QM_COURSE_SECTION where deleted = 0 and course_id = ? order by section_start_time  ";
        return jt.queryForBeans(SectionVO.class,sql,courseId);
    }

    public QueryCourseVO queryCourseVO(Long courseId){
        String sql = "select * from YCQ_QM_COURSE where deleted = 0 and course_id = ? ";
        return jt.queryForBean(QueryCourseVO.class,sql,courseId);
    }

    public List<QueryUserClassesVO> queryUserClasses(Integer workerId){
        String sql ="select a.*,b.type_name as classesTypeName from YCQ_QM_CLASSES_INFO a left join YCQ_QM_CLASSES_TYPE b on a.classes_type_id = b.classes_type_id  where a.deleted = 0 and a.classes_id in (select classes_id from YCQ_QM_USER_CLASSES_REL where deleted = 0 and stu_worker_id = ? )";
        return jt.queryForBeans(QueryUserClassesVO.class,sql,workerId);
    }

    public List<QueryClassesCourseVO> queryClassesCourseVOList(Long classesId){
        String sql = "select * from YCQ_QM_COURSE where deleted = 0 and classes_id = ? ";
        return jt.queryForBeans(QueryClassesCourseVO.class,sql,classesId);
    }

    public List<Long> stuParticipationCourseSection(Integer workerId){
        String sql = "select course_section_id from YCQ_QM_USER_COURSE_SECTION where deleted = 0 and stu_worker_id = ? ";
        return jt.queryForRawObjects(Long.class,sql,workerId);
    }

    public List<Long> stuParticipationCourse(Integer workerId){
        String sql = "select course_id from YCQ_QM_USER_COURSE where deleted = 0 and stu_worker_id = ? ";
        return jt.queryForRawObjects(Long.class,sql,workerId);
    }

    public List<QuerySectionVO> querySectionList(Long courseId){
        String sql = "select * from YCQ_QM_COURSE_SECTION where deleted = 0 and course_id = ? ";
        return jt.queryForBeans(QuerySectionVO.class,sql,courseId);
    }

    public Integer queryUserCourse(Integer workerId,Long courseId){
        String sql = " select count(*) from YCQ_QM_USER_COURSE where deleted = 0 and stu_worker_id = ? and course_id = ? ";
        return jt.queryForRawObject(Integer.class,sql,workerId,courseId);
    }

    public Integer queryUserCourseSection(Integer workerId,Long courseId){
        String sql = " select count(*) from YCQ_QM_USER_COURSE_SECTION where deleted = 0 and stu_worker_id = ? and course_section_id = ? ";
        return jt.queryForRawObject(Integer.class,sql,workerId,courseId);
    }

    public QueryEvaluateStuVO queryUserCourseVO(Integer workerId, Long courseId){
        String sql = "select a.*,b.worker_name as evaluateWorkerName from YCQ_QM_USER_COURSE a left join TSM_WORKER b on a.stu_worker_id=b.login_id where a.deleted = 0 and a.stu_worker_id = ? and a.course_id = ?";
        return jt.queryForBean(QueryEvaluateStuVO.class,sql,workerId,courseId);
    }

    public QueryEvaluateStuVO queryClassesYearType(Long courseId){
        String sql = "select a.classes_year,c.org_id,c.org_name,b.type_name as classesTypeName,b.level_name as classesLevelName from YCQ_QM_CLASSES_INFO a left join YCQ_QM_CLASSES_TYPE b on a.classes_type_id = b.classes_type_id left join TSM_ORG c on a.org_id = c.org_id where a.deleted = 0 and b.deleted = 0 and a.classes_id = (select classes_id from YCQ_QM_COURSE where deleted = 0 and course_id = ?)";
        return jt.queryForBean(QueryEvaluateStuVO.class,sql,courseId);
    }

    public Integer queryClassesCount(Long ClassesTypeId,String classesYear){
        String sql  = "select count(*) from YCQ_QM_CLASSES_INFO where deleted = 0 and classes_type_id = ? and classes_year = ? ";
        return jt.queryForRawObject(Integer.class,sql,ClassesTypeId,classesYear);
    }

    public void deleteCourseSectionRel(Long courseId){
        String sql = "delete from YCQ_QM_COURSE_SECTION_REL where course_id = ? ";
        jt.update(sql,courseId);
    }

    public List<YcqQmCourseSectionRel> queryCourseSectionRel(Long courseId){
        String sql = "select * from YCQ_QM_COURSE_SECTION_REL where course_id = ?  ";
        return jt.queryForBeans(YcqQmCourseSectionRel.class,sql,courseId);
    }

    public Integer whetherClassesType(SaveClassesDTO saveClassesDTO){
        String sql = "select count(*) from YCQ_QM_CLASSES_INFO where deleted = 0 and classes_year = ? and classes_type_id = ? and org_id = ?  ";
        return jt.queryForRawObject(Integer.class,sql,saveClassesDTO.getClassesYear(),saveClassesDTO.getClassesTypeId(),saveClassesDTO.getOrgId());
    }

    public PageRsp<stuVO> queryAddClassesStu(PageReq<QueryClassesStuDTO> req){
        String sql = "select a.worker_id as stuWorkerId,b.worker_name from YCQ_QM_USER_INFO a left join TSM_WORKER b on a.worker_id =b.login_id where a.deleted = 0 and a.user_identity = 1 and a.audit_status = 1 and a.worker_id in(select stu_worker_id from YCQ_QM_USER_CLASSES_REL where deleted = 0 and classes_id = ? ) UNION select a.worker_id as stuWorkerId,b.worker_name from YCQ_QM_USER_INFO a left join TSM_WORKER b on a.worker_id =b.login_id where a.deleted = 0 and a.user_identity = 1 and a.audit_status = 1 and a.worker_id not in(select stu_worker_id from YCQ_QM_USER_CLASSES_REL where deleted = 0 )";
        String sqlCount = "select (select count(*) from YCQ_QM_USER_INFO a left join TSM_WORKER b on a.worker_id =b.login_id where a.deleted = 0 and a.user_identity = 1 and a.audit_status = 1 and a.worker_id in(select stu_worker_id from YCQ_QM_USER_CLASSES_REL where deleted = 0 and classes_id = ? )) + (select COUNT(*) from YCQ_QM_USER_INFO a left join TSM_WORKER b on a.worker_id =b.login_id where a.deleted = 0 and a.user_identity = 1 and a.audit_status = 1 and a.worker_id NOT in(select stu_worker_id from YCQ_QM_USER_CLASSES_REL where deleted = 0 )) as total";
        ArrayList<Object> params = new ArrayList<>(20);
        params.add(req.getQuery().getClassesId());
        return queryPageBase(req,sql,sqlCount,stuVO.class,params);
    }

    public List<stuVO> queryClassesStu(Long classesId){
        String sql = "select a.stu_worker_id,b.worker_name from YCQ_QM_USER_CLASSES_REL a left join TSM_WORKER b on a.stu_worker_id = b.login_id where a.deleted = 0 and a.classes_id = ? ";
        return jt.queryForBeans(stuVO.class,sql,classesId);
    }

    public PageRsp<stuVO> queryClassesStu(PageReq<QueryClassesStuDTO> req){
        String sql = "select a.stu_worker_id,b.worker_name from YCQ_QM_USER_CLASSES_REL a left join TSM_WORKER b on a.stu_worker_id = b.login_id left join YCQ_QM_USER_INFO c on c.worker_id =b.login_id where a.deleted = 0 and c.deleted = 0 and a.classes_id = ?    ";
        String sqlCount = "select count(*) from YCQ_QM_USER_CLASSES_REL a left join TSM_WORKER b on a.stu_worker_id = b.login_id left join YCQ_QM_USER_INFO c on c.worker_id =b.login_id where a.deleted = 0 and c.deleted = 0 and a.classes_id = ?  ";
        ArrayList<Object> params = new ArrayList<>(20);
        params.add(req.getQuery().getClassesId());
        return queryPageBase(req,sql,sqlCount,stuVO.class,params);
    }

    /**
     * 分页查询base
     * @param req
     * @param sql
     * @param countSql
     * @param clazz
     * @param params
     * @param <T>
     * @return
     */
    public <T> PageRsp<T> queryPageBase(PageReq<?> req, String sql, String countSql, Class<T> clazz, List<Object> params){
        StringBuilder sb = new StringBuilder(sql);
        StringBuilder sbCount = new StringBuilder(countSql);

        int total = jt.queryForInt(sbCount.toString(), params.toArray());
        List<T> values = null;
        if (total > 0) {
            sb.append(" limit ? offset ? ");
            params.add(req.getPageSize());
            params.add(req.getMyStart());
            values = jt.queryForBeans(clazz, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<T>(0);
        }
        PageRsp<T> page = new PageRsp<T>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }
}
