package com.crunii.micro.service.qm.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQmExertLink;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.*;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.TeacherBindingListDto;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.ViewStudentDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.ExertDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.GroupCultivationDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.StuHonorDto;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/9/21 18:23
 */
@Repository
public class CommonDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public PageRsp<WorkerHonorVo> getStuExpand(PageReq<StuExpandDto> req) {
        StringBuilder sb = new StringBuilder("SELECT acquisition_time,honor_desc FROM YCQ_QM_USER_HONOR WHERE honor_type = 1 AND deleted =0 AND worker_id = ?");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_QM_USER_HONOR WHERE honor_type = 1 AND deleted =0 AND worker_id = ?");
        List<Object> params = new ArrayList<>();
        Integer workerId = req.getQuery().getWorkerId();
        params.add(workerId);
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<WorkerHonorVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by acquisition_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(WorkerHonorVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<WorkerHonorVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<String> getAdvantage(Integer workerId) {
        String sql = "SELECT honor_desc FROM YCQ_QM_USER_HONOR WHERE honor_type = 2 AND deleted =0 AND worker_id = ? ";
        return jdbcTemplate.queryForRawObjects(String.class,sql,workerId);
    }

    public PageRsp<ConcentratedCultivationVo> pageConcentratedCultivationExpand(PageReq<StuExpandDto> pageReq) {
        StringBuilder sb = new StringBuilder("SELECT B.start_time,A.evaluate_time,B.course_name,B.course_subject,A.evaluate_result,A.operation_completion,C.classes_year,D.level_name,D.type_name,A.user_course_id,A.presentation_evaluate FROM YCQ_QM_USER_COURSE A LEFT JOIN YCQ_QM_COURSE B ON A.course_id = B.course_id LEFT JOIN YCQ_QM_CLASSES_INFO C ON B.classes_id =C.classes_id LEFT JOIN YCQ_QM_CLASSES_TYPE D ON C.classes_type_id =D.classes_type_id WHERE stu_worker_id = ? ");
        StringBuilder sbCount = new StringBuilder("SELECT COUNT(1) FROM YCQ_QM_USER_COURSE A LEFT JOIN YCQ_QM_COURSE B ON A.course_id = B.course_id LEFT JOIN YCQ_QM_CLASSES_INFO C ON B.classes_id =C.classes_id LEFT JOIN YCQ_QM_CLASSES_TYPE D ON C.classes_type_id =D.classes_type_id WHERE stu_worker_id = ?");
        List<Object> params = new ArrayList<>();
        Integer workerId = pageReq.getQuery().getWorkerId();
        params.add(workerId);
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<ConcentratedCultivationVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by A.evaluate_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = jdbcTemplate.queryForBeans(ConcentratedCultivationVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<ConcentratedCultivationVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public PageRsp<TeacherTraineeVo> pageStuTraineeExpand(PageReq<TrackingCultivationDto> pageReq) {
        StringBuilder sb = new StringBuilder(" SELECT  D.worker_name,B.classes_year,C.level_name,A.evaluate_context,A.create_time,C.type_name FROM YCQ_QM_TUTOR_EVALUATE A\n" +
                " LEFT JOIN YCQ_QM_CLASSES_INFO B ON B.classes_id =A.classes_id\n" +
                " LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id = C.classes_type_id\n" +
                " LEFT JOIN TSM_WORKER D ON A.teach_worker_id = D.login_id\n" +
                " WHERE A.stu_worker_id = ? ");
        StringBuilder sbCount = new StringBuilder(" SELECT count(1) FROM YCQ_QM_TUTOR_EVALUATE A\n" +
                " LEFT JOIN YCQ_QM_CLASSES_INFO B ON B.classes_id =A.classes_id\n" +
                " LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id = C.classes_type_id\n" +
                " LEFT JOIN TSM_WORKER D ON A.teach_worker_id = D.login_id\n" +
                " WHERE A.stu_worker_id = ? ");
        List<Object> params = new ArrayList<>();
        TrackingCultivationDto query = pageReq.getQuery();
        Integer workerId = query.getWorkerId();
        params.add(workerId);
        if(Objects.nonNull(query.getStartTime())){
            sb.append(" AND A.create_time >= ?");
            sbCount.append(" AND A.create_time >= ?");
            params.add(query.getStartTime());
        }
        if(Objects.nonNull(query.getEndTime())){
            sb.append(" AND A.create_time <= ?");
            sbCount.append(" AND A.create_time <= ?");
            params.add(query.getEndTime());
        }
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<TeacherTraineeVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" ORDER BY A.updater_time DESC");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = jdbcTemplate.queryForBeans(TeacherTraineeVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<TeacherTraineeVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }
    public LearIntegral findIntegralByWorkerId(int loginId) {
        String sql = "SELECT integral,update_time,self_cultivation_id FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ? AND deleted = 0 AND cultivation_type =1 limit 1";
        return jdbcTemplate.queryForBean(LearIntegral.class,sql,loginId);
    }

    public List<WorkerBookVo> findBookInfoByWorkerId(int loginId) {
        String sql = "SELECT self_cultivation_id,book_name,experience,complete_time as createTime FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ? AND cultivation_type = 2 AND deleted = 0 order by createTime desc";
        return jdbcTemplate.queryForBeans(WorkerBookVo.class,sql,loginId);
    }

    public List<VolunteerVo> findVolunteerByWorkerId(int loginId) {
        String sql = "SELECT self_cultivation_id,experience,create_time FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ? AND cultivation_type = 3 AND deleted = 0 order by create_time desc limit 1";
        return jdbcTemplate.queryForBeans(VolunteerVo.class,sql,loginId);
    }

    public List<VolunteerVo> findOtherByWorkerId(int loginId) {
        String sql = "SELECT self_cultivation_id,experience,create_time FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ? AND cultivation_type = 4 AND deleted = 0 order by create_time desc limit 1";
        return jdbcTemplate.queryForBeans(VolunteerVo.class,sql,loginId);
    }

    public PageRsp<WorkerBookVo> pageBookRead(PageReq<StuExpandDto> pageReq) {
        StringBuilder sb = new StringBuilder("SELECT self_cultivation_id,book_name,experience,create_time FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ? AND cultivation_type = 2 AND deleted = 0 ");
        StringBuilder sbCount = new StringBuilder(" SELECT count(1) FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ? AND cultivation_type = 2 AND deleted = 0 ");
        List<Object> params = new ArrayList<>();
        Integer workerId = pageReq.getQuery().getWorkerId();
        params.add(workerId);
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<WorkerBookVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by create_time desc ");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = jdbcTemplate.queryForBeans(WorkerBookVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<WorkerBookVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public PageRsp<VolunteerVo> pageVolunteerOrOther(PageReq<StuExpandDto> pageReq, int cultivationType) {
        StringBuilder sb = new StringBuilder("SELECT self_cultivation_id,book_name,experience,create_time FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ? AND cultivation_type = ? AND deleted = 0 ");
        StringBuilder sbCount = new StringBuilder(" SELECT count(1) FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ? AND cultivation_type = ? AND deleted = 0 ");
        List<Object> params = new ArrayList<>();
        Integer workerId = pageReq.getQuery().getWorkerId();
        params.add(workerId);
        params.add(cultivationType);
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<VolunteerVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by create_time desc ");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = jdbcTemplate.queryForBeans(VolunteerVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<VolunteerVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public PageRsp<ExertVo> pageExert(PageReq<ExertDto> pageReq) {
        StringBuilder sb = new StringBuilder("SELECT exert_id,cover_number,exert_type,experience,complete_time as createTime FROM YCQ_QM_EXERT WHERE worker_id = ? AND deleted = 0 ");
        StringBuilder sbCount = new StringBuilder(" SELECT count(1) FROM YCQ_QM_EXERT WHERE worker_id = ? AND deleted = 0 ");
        List<Object> params = new ArrayList<>();
        ExertDto query = pageReq.getQuery();
        Integer workerId = query.getWorkerId();
        params.add(workerId);
        if(Objects.nonNull(query.getStartTime())){
            sb.append(" and complete_time >= ?");
            sbCount.append(" and complete_time >= ?");
            params.add(query.getStartTime());
        }
        if(Objects.nonNull(query.getEndTime())){
            sb.append(" and complete_time <= ?");
            sbCount.append(" and complete_time <= ?");
            params.add(query.getEndTime());
        }
        if(Objects.nonNull(query.getExertType())){
            sb.append(" and exert_type = ?");
            sbCount.append(" and exert_type = ?");
            params.add(query.getExertType());
        }
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<ExertVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by complete_time desc ");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = jdbcTemplate.queryForBeans(ExertVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<ExertVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    @InjectProperty
    public PageRsp<GrowthVo> pageGrowth(PageReq<GroupCultivationDto> pageReq) {
        StringBuilder sb = new StringBuilder("SELECT A.group_id,A.complete_time AS createTime,A.recommend_type,A.recommend_reason,B.work_unit FROM YCQ_QM_GROUP A LEFT JOIN YCQ_QM_USER_INFO B ON A.worker_id = B.worker_id WHERE A.worker_id = ? AND A.deleted = 0 ");
        StringBuilder sbCount = new StringBuilder(" SELECT COUNT(1) FROM YCQ_QM_GROUP A LEFT JOIN YCQ_QM_USER_INFO B ON A.worker_id = B.worker_id WHERE A.worker_id = ? AND A.deleted = 0 ");
        List<Object> params = new ArrayList<>();
        GroupCultivationDto query = pageReq.getQuery();
        Integer workerId = query.getWorkerId();
        params.add(workerId);
        if(Objects.nonNull(query.getStartTime())){
            sb.append(" and A.complete_time  >= ?");
            sbCount.append(" and A.complete_time  >= ?");
            params.add(query.getStartTime());
        }
        if(Objects.nonNull(query.getEndTime())){
            sb.append(" and A.complete_time  <= ?");
            sbCount.append(" and A.complete_time  <= ?");
            params.add(query.getEndTime());
        }
        if(Objects.nonNull(query.getRecommendType())){
            sb.append(" and A.recommend_type = ?");
            sbCount.append(" and A.recommend_type = ?");
            params.add(query.getRecommendType());
        }
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<GrowthVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by A.complete_time desc ");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = jdbcTemplate.queryForBeans(GrowthVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<GrowthVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<VolunteerVo> getVolunteer(StuHonorDto stuHonorDto) {
        StringBuilder sb = new StringBuilder("SELECT self_cultivation_id,cultivation_type,experience,complete_time as createTime FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ?  AND deleted = 0 ");
        List<Object> params = new ArrayList<>();
        params.add(stuHonorDto.getWorkerId());
        if(Objects.nonNull(stuHonorDto.getStartTime())){
            sb.append(" and complete_time >= ?");
            params.add(stuHonorDto.getStartTime());
        }
        if(Objects.nonNull(stuHonorDto.getEndTime())){
            sb.append(" and complete_time <= ?");
            params.add(stuHonorDto.getEndTime());
        }
        if(Objects.nonNull(stuHonorDto.getExperience())){
            sb.append(" and cultivation_type = ?");
            params.add(stuHonorDto.getExperience());
        }else {
            sb.append(" AND cultivation_type in(3,4)");
        }
        sb.append(" order by createTime desc");

        return jdbcTemplate.queryForBeans(VolunteerVo.class,sb.toString(),params.toArray());
    }

    public void updateUserClasses(Long classesId){
        String sql = "update YCQ_QM_USER_CLASSES_REL set deleted = 1 where deleted = 0 and classes_id = ? ";
        jdbcTemplate.update(sql,classesId);

    }

    public List<ExertLinkUrl> getUrlsByExertIds(List<Long> collect) {
        String sql = "SELECT url,exert_id FROM YCQ_QM_EXERT_LINK WHERE exert_id IN(:ids) and deleted = 0";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",collect);
        return baseNpJdbcTemplate.queryForBeans(ExertLinkUrl.class,sql,params);
    }

    public List<GrowthPartyVo> findParty(TrackingCultivationDto dto) {
        String sql = "SELECT recommend_reason,worker_job,recommend_time FROM YCQ_QM_INTO_PARTY WHERE stu_worker_id = ? AND deleted = 0 ORDER BY recommend_time";
        return jdbcTemplate.queryForBeans(GrowthPartyVo.class,sql,dto.getWorkerId());
    }

    public void delUserByWorkerIds(List<Integer> workerIds, Date nowTime, int loginId) {
        String sql = "UPDATE YCQ_QM_USER_INFO SET deleted = 1,update_time = :nowTime,updater_id = :workerId WHERE worker_id IN(:ids)";
        Map<String, Object> params = new HashMap<>(6);
        params.put("ids", workerIds);
        params.put("nowTime", nowTime);
        params.put("workerId", loginId);
        baseNpJdbcTemplate.update(sql, params);
    }

    public void delTeacherExpandByWorkerIds(List<Integer> workerIds, Date nowTime, int loginId) {
        String sql = "UPDATE YCQ_QM_TEACHER_EXPAND SET deleted = 1,update_time = :nowTime,updater_id = :workerId WHERE worker_id IN(:ids)";
        Map<String, Object> params = new HashMap<>(6);
        params.put("ids", workerIds);
        params.put("nowTime", nowTime);
        params.put("workerId", loginId);
        baseNpJdbcTemplate.update(sql, params);
    }

    public void delTeacherUser(List<Integer> workerIds, Date nowTime, int loginId,boolean isTeacher) {
        String sql = "UPDATE YCQ_QM_TEACHER_STU SET deleted = 1,update_time = :nowTime,updater_id = :workerId ";
        if(isTeacher){
            sql += " WHERE teach_worker_id IN(:ids)";
        }else {
            sql += " WHERE stu_worker_id IN(:ids)";
        }
        Map<String, Object> params = new HashMap<>(6);
        params.put("ids", workerIds);
        params.put("nowTime", nowTime);
        params.put("workerId", loginId);
        baseNpJdbcTemplate.update(sql, params);
    }

    public void delWorkerReject(List<Integer> workerIds,boolean isTeacher) {
        String sql = "UPDATE YCQ_QM_TEACHER_STU_REJECT SET deleted = 1  ";
        if(isTeacher){
            sql += "WHERE teach_worker_id IN (:ids)";
        }else {
            sql += "WHERE stu_worker_id IN (:ids)";
        }
        Map<String, Object> params = new HashMap<>(4);
        params.put("ids", workerIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void delClassRelRecord(List<Integer> stuWorkerIds) {
        String sql = "UPDATE YCQ_QM_USER_CLASSES_REL SET deleted = 1 WHERE stu_worker_id in(:ids)";
        Map<String, Object> params = new HashMap<>(4);
        params.put("ids", stuWorkerIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public List<TeacherAppBaseVo> findMyTeacher(Integer stuWorkerId) {
        String sql = "SELECT A.teach_worker_id as workerId,B.user_code,B.card_id,A.update_time AS endTime,A.start_time,C.sex,O.org_name as area_name,B.work_unit,B.worker_job,B.political_outlook,C.head_portrait_pic_key,C.worker_name,C.mobile,B.education,B.area_id,A.binding\n" +
                "FROM YCQ_QM_TEACHER_STU A \n" +
                "LEFT JOIN YCQ_QM_USER_INFO B ON A.teach_worker_id = B.worker_id\n" +
                "LEFT JOIN TSM_ORG O ON b.area_id = O.org_id\n" +
                "LEFT JOIN TSM_WORKER C ON A.teach_worker_id = C.login_id\n" +
                "WHERE stu_worker_id = ? AND correlation_state IN (1,2)  AND  A.deleted = 0 AND B.audit_status =1 AND B.deleted = 0 order by start_time desc";
        return jdbcTemplate.queryForBeans(TeacherAppBaseVo.class,sql,stuWorkerId);
    }

    public List<WorkerIdLeveName> getWorkerLevelName(List<Integer> collect) {
        String sql = "SELECT worker_id,B.level_name FROM YCQ_QM_TEACHER_EXPAND A LEFT JOIN YCQ_QM_CLASSES_TYPE B ON A.level_code =B.level_code AND A.deleted=0 WHERE A.worker_id IN(:ids) and level_name is not null GROUP BY  worker_id,B.level_name";
        Map<String,Object> params = new HashMap<>(3);
        params.put("ids",collect);
        return baseNpJdbcTemplate.queryForBeans(WorkerIdLeveName.class,sql,params);
    }

    public void delHonor(List<Integer> teacherWorkerId, Date nowTime, int loginId) {
        String sql = "UPDATE YCQ_QM_USER_HONOR SET deleted = 1,update_time = :updateTime,updater_id = :updateId WHERE worker_id in(:ids) and deleted=0";
        Map<String,Object> params = new HashMap<>(6);
        params.put("ids",teacherWorkerId);
        params.put("updateTime",nowTime);
        params.put("updateId",loginId);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void delUserCourse(List<Integer> stuWorkerIds,  int loginId) {
        String sql = "UPDATE YCQ_QM_USER_COURSE SET deleted = 1,updater_id = :updateId WHERE stu_worker_id in(:ids) and deleted=0";
        Map<String,Object> params = new HashMap<>(6);
        params.put("ids",stuWorkerIds);
        params.put("updateId",loginId);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void delSelfCultivation(List<Integer> stuWorkerIds, Date nowTime, int loginId) {
        String sql = "UPDATE YCQ_QM_SELF_CULTIVATION SET deleted = 1,update_time = :updateTime,updater_id = :updateId WHERE worker_id in(:ids) and deleted=0";
        Map<String,Object> params = new HashMap<>(6);
        params.put("ids",stuWorkerIds);
        params.put("updateTime",nowTime);
        params.put("updateId",loginId);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void delExert(List<Integer> stuWorkerIds) {
        String sql = "UPDATE YCQ_QM_EXERT SET deleted = 1 WHERE worker_id in(:ids) and deleted=0";
        Map<String,Object> params = new HashMap<>(6);
        params.put("ids",stuWorkerIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void delGroup(List<Integer> stuWorkerIds, Date nowTime, int loginId) {
        String sql = "UPDATE YCQ_QM_GROUP SET deleted = 1,update_time = :updateTime,updater_id = :updateId WHERE worker_id in(:ids) and deleted=0";
        Map<String,Object> params = new HashMap<>(6);
        params.put("ids",stuWorkerIds);
        params.put("updateTime",nowTime);
        params.put("updateId",loginId);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void updateTsmWorker(String workerName, Integer sex, Integer loginId) {
        String sql = "UPDATE TSM_WORKER SET state = 1 ,worker_name = ?,sex=? WHERE login_id = ?";
        jdbcTemplate.update(sql,workerName,sex,loginId);
    }

    public void delTutorTrainee(List<Integer> stuWorkerIds, Date nowTime, int loginId,boolean isTeacher) {
        String sql = "UPDATE YCQ_QM_TUTOR_EVALUATE SET deleted = 1,updater_time = :updateTime,updater_id = :updateId WHERE deleted=0";
        if(isTeacher){
            sql += "and stu_worker_id in(:ids)";
        }else {
            sql += "and teach_worker_id in(:ids)";
        }
        Map<String,Object> params = new HashMap<>(6);
        params.put("ids",stuWorkerIds);
        params.put("updateTime",nowTime);
        params.put("updateId",loginId);
    }

    public PageRsp<ExertVo> pageExertTwo(PageReq<ExertDto> req) {
        StringBuilder sb = new StringBuilder("SELECT * FROM YCQ_QM_EXERT WHERE deleted =0 AND worker_id = ? ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_QM_EXERT WHERE deleted =0 AND worker_id = ? ");
        List<Object> params = new ArrayList<>();
        ExertDto query = req.getQuery();
        params.add(query.getWorkerId());
        if(Objects.nonNull(query.getExertType())){
            sb.append(" and exert_type = ?");
            sbCount.append(" and exert_type = ?");
            params.add(query.getExertType());
        }
        if(Objects.nonNull(query.getStartTime())){
            sb.append(" and complete_time >= ?");
            sbCount.append(" and complete_time >= ?");
            params.add(query.getStartTime());
        }
        if(Objects.nonNull(query.getEndTime())){
            sb.append(" and complete_time <= ?");
            sbCount.append(" and complete_time <= ?");
            params.add(query.getEndTime());
        }
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<ExertVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by complete_time desc ");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(ExertVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<ExertVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<ExertFileTypeDto> findPicVido(List<Long> collect) {
        String sql = "SELECT exert_id,file_id,pic_url,file_key FROM YCQ_QM_EXERT_LINK WHERE deleted =0 AND exert_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",collect);
        return baseNpJdbcTemplate.queryForBeans(ExertFileTypeDto.class,sql,params);
    }

    public List<VideoPic> findVideoPicList(List<String> collect2) {
        String sql = "SELECT picture_url,file_id,file_name FROM YCQ_QM_VIDEO WHERE deleted = 0 AND file_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",collect2);
        return baseNpJdbcTemplate.queryForBeans(VideoPic.class,sql,params);
    }

    public void updateTeacherStu(List<Integer> stuWorkerIds, Date nowTime, int loginId) {
        String sql = "UPDATE YCQ_QM_TEACHER_STU SET binding = 0,updater_id = :updateId,update_time = :updateTime WHERE stu_worker_id IN(:ids) AND binding = 1 AND correlation_state in(1,2) AND deleted =0";
        Map<String,Object> params = new HashMap<>(2);
        params.put("updateId",loginId);
        params.put("ids",stuWorkerIds);
        params.put("updateTime",nowTime);
        baseNpJdbcTemplate.update(sql,params);
    }
}
