package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.qm.api.domain.dto.courseware.CoursewareDto;
import com.crunii.micro.service.qm.api.domain.dto.courseware.CoursewareResInfoVo;
import com.crunii.micro.service.qm.api.domain.dto.courseware.CoursewareVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/18 14:38
 */
@Repository
public class CoursewareDao {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String PAGE_COURSEWARE_COUNT_SQL = "select COUNT(*) from YCQ_QM_COURSEWARE a where 1 = 1 ";

    private static final String PAGE_COURSEWARE_SQL = "select * from YCQ_QM_COURSEWARE a where 1 = 1 ";

    public PageRsp<CoursewareVo> pageList(PageReq<CoursewareDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_COURSEWARE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COURSEWARE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        // 课件主题
        if (Objects.nonNull(req.getQuery().getCertificateTopic())) {
            sb.append(" and  a.certificate_topic = ? ");
            sbCount.append(" and a.certificate_topic = ? ");
            params.add(req.getQuery().getCertificateTopic());
        }
        // 状态
        if (Objects.nonNull(req.getQuery().getCertificateStatus())) {
            sb.append(" and  a.certificate_status = ? ");
            sbCount.append(" and a.certificate_status = ? ");
            params.add(req.getQuery().getCertificateStatus());
        }
        // 课件类型
        if (Objects.nonNull(req.getQuery().getCertificateType())) {
            sb.append(" and  a.certificate_type = ? ");
            sbCount.append(" and a.certificate_type = ? ");
            params.add(req.getQuery().getCertificateType());
        }
        // 课件名称
        if (StrUtil.isNotBlank(req.getQuery().getCertificateName())) {
            sb.append(" and a.certificate_name like concat('%',?,'%') ");
            sbCount.append(" and a.certificate_name like concat('%',?,'%') ");
            params.add(req.getQuery().getCertificateName());
        }
        // 资源编号
        if (StrUtil.isNotBlank(req.getQuery().getResNumber())) {
            sb.append(" and a.res_number like concat('%',?,'%') ");
            sbCount.append(" and a.res_number like concat('%',?,'%') ");
            params.add(req.getQuery().getResNumber());
        }
        // 上传人姓名
        if (StrUtil.isNotBlank(req.getQuery().getWorkerName())) {
            sb.append(" and a.worker_name like concat('%',?,'%') ");
            sbCount.append(" and a.worker_name like concat('%',?,'%') ");
            params.add(req.getQuery().getWorkerName());
        }
        // 开始时间
        if (StrUtil.isNotBlank(req.getQuery().getStartTime())) {
            sb.append(" and a.create_time >= ? ");
            sbCount.append(" and a.create_time >= ? ");
            params.add(req.getQuery().getStartTime());
        }
        // 结束时间
        if (StrUtil.isNotBlank(req.getQuery().getEndTime())) {
            sb.append(" and a.create_time <= ? ");
            sbCount.append(" and a.create_time <= ? ");
            params.add(req.getQuery().getEndTime());
        }
        // 逻辑删除
        sb.append(" and a.deleted =0 ");
        sbCount.append(" and a.deleted =0 ");

        int total = jt.queryForInt(sbCount.toString(), params.toArray());
        List<CoursewareVo> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            sb.append(" order by a.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jt.queryForBeans(CoursewareVo.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<CoursewareVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public CoursewareResInfoVo getCoursewareResInfoVo(Long fileId){
        String sql = "SELECT file_name AS resName,file_size AS resSize FROM YCQ_OBS_FILE WHERE state =1 and id =?";
        return jt.queryForBean(CoursewareResInfoVo.class,sql,fileId);
    }

    public CoursewareResInfoVo getCoursewareVideoInfoVo(String fileId){
        String sql = "SELECT file_name AS resName,file_size AS resSize,picture_url AS resUrl FROM YCQ_QM_VIDEO  WHERE deleted =0 and file_id =?";
        return jt.queryForBean(CoursewareResInfoVo.class,sql,fileId);
    }
}
