package com.crunii.micro.service.qm.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.entity.main.YcqQmBook;
import com.crunii.micro.service.qm.api.domain.dto.homepage.vo.HomePageBookListVo;
import com.crunii.micro.service.qm.api.domain.dto.homepage.vo.HomePageBookVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.UserHonorVo;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author xiayinqiang
 * @date 2023/9/18 0018 15:15:33
 */
@Repository
public class HomePageDao {
	@Resource(name = "mainJT")
	private BaseJdbcTemplate jt;


	/**
	 * 查询需要在首页展示的最新的四条书单id
	 *
	 * @return List<Long>
	 */
	public List<HomePageBookListVo> selectBookListByFour() {
		String sql = " SELECT a.book_list_id,a.book_list_name,a.book_list_theme FROM YCQ_QM_BOOK_LIST a INNER JOIN YCQ_QM_BOOK_LIST_REL B ON A.book_list_id =B.book_list_id \n"
				+ " INNER JOIN YCQ_QM_BOOK C ON B.book_id = C.book_id \n"
				+ " WHERE a.deleted = 0 AND b.deleted = 0 AND a.book_list_show_location = 1 AND a.book_list_status = 1 AND C.deleted = 0 AND C.book_status = 1 \n"
				+ " GROUP BY A.book_list_id,a.book_list_name,a.book_list_theme ORDER BY A.book_list_id DESC LIMIT 4 ";
		return jt.queryForBeans(HomePageBookListVo.class, sql);
	}

	/**
	 * 根据书单查询书籍
	 *
	 * @param bookListId 书单id
	 * @return List<HomePageBookVo>
	 */
	public List<HomePageBookVo> selectBookByBookListId(Long bookListId) {
		String sql =
				" SELECT a.book_list_id,a.book_id,b.book_name,b.book_author,b.file_key FROM YCQ_QM_BOOK_LIST_REL a INNER JOIN YCQ_QM_BOOK b ON a.book_id = b.book_id AND a.deleted = 0 AND b.deleted = 0 WHERE b.book_status = 1\n"
						+ " AND a.book_list_id = ? "
						+ " ORDER BY a.update_time DESC LIMIT 3 ";
		return jt.queryForBeans(HomePageBookVo.class, sql, bookListId);
	}

	/**
	 * 查询学员数量
	 * @param loginId 导师人员id
	 * @return List<Map<String, Object>>
	 */
	public List<Map<String, Object>> selectStuCountByCorrelationState(int loginId) {
		String sql = " SELECT a.correlation_state,count(*) as con FROM YCQ_QM_TEACHER_STU a WHERE a.deleted = 0 AND a.binding = 1 AND a.teach_worker_id = ? GROUP BY a.correlation_state  ";
		return jt.queryForList(sql,loginId);
	}

	/**
	 * 查询导师选择学员数量
	 * @param loginId 导师人员id
	 * @return int
	 */
	public int selectStuCountByChooseState(int loginId) {
		String sql = " SELECT count(*) FROM YCQ_QM_TEACHER_STU a WHERE a.deleted = 0 AND a.teach_worker_id = ? AND a.choose_state = 1 AND a.binding = 1 ";
		return jt.queryForInt(sql,loginId);
	}
}
