package com.crunii.micro.service.qm.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.LoanPlanImportTemp;
import com.crunii.micro.domain.entity.main.TsmPropertyValue;
import com.crunii.micro.domain.entity.main.YcqQfAreaIndustry;
import com.crunii.micro.domain.entity.main.YcqQfBank;
import com.crunii.micro.service.qm.api.domain.dto.qf.*;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wangchen create 2023/11/9 14:32
 */
@Repository
public class QfPlanManageDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public PageRsp<LoanPlanDetailVo> pageLoanPlan(PageReq<LoanPlanDto> req) {
        StringBuilder sb = new StringBuilder("SELECT *,CONCAT_WS('/',(SELECT bank_name FROM YCQ_QF_BANK WHERE bank_id = b.parent_bank_id),b.bank_name) AS bank_name ,b.bank_id,(SELECT area_industry_name FROM YCQ_QF_AREA_INDUSTRY WHERE area_id = area_industry_id) AS areaName,(SELECT area_industry_name FROM YCQ_QF_AREA_INDUSTRY WHERE industry = area_industry_id) AS industryName FROM YCQ_QF_LOAN_RECORD a LEFT JOIN YCQ_QF_BANK b ON a.intention_loan_bank =b.bank_id  where a.deleted = 0 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_QF_LOAN_RECORD where deleted = 0 ");
        List<Object> params = new ArrayList<>(10);
        LoanPlanDto loanPlanDto = req.getQuery();
        if(StringUtils.hasText(loanPlanDto.getLoanName())){
            sb.append(" and loan_name like concat('%',?,'%')");
            sbCount.append(" and loan_name like concat('%',?,'%')");
            params.add(loanPlanDto.getLoanName());
        }

        if(StringUtils.hasText(loanPlanDto.getIdCard())){
            sb.append(" and id_card like concat('%',?,'%')");
            sbCount.append(" and id_card like concat('%',?,'%')");
            params.add(loanPlanDto.getIdCard());
        }

        if (Objects.nonNull(loanPlanDto.getLoanType())) {
            sb.append(" and loan_type = ? ");
            sbCount.append(" and loan_type = ? ");
            params.add(loanPlanDto.getLoanType());
        }

        if (Objects.nonNull(loanPlanDto.getIntentionLoanBank())) {
            sb.append(" and intention_loan_bank = ? ");
            sbCount.append(" and intention_loan_bank = ? ");
            params.add(loanPlanDto.getIntentionLoanBank());
        }

        if(Objects.nonNull(loanPlanDto.getAreaId())){
            sb.append(" and area_id = ? ");
            sbCount.append(" and area_id = ? ");
            params.add(loanPlanDto.getAreaId());
        }

        if (Objects.nonNull(loanPlanDto.getStartTime())) {
            sb.append(" and create_time >= ? ");
            sbCount.append(" and create_time >= ? ");
            params.add(loanPlanDto.getStartTime());
        }

        if (Objects.nonNull(loanPlanDto.getEndTime())) {
            sb.append(" and create_time <= ? ");
            sbCount.append(" and create_time <= ? ");
            params.add(loanPlanDto.getEndTime());
        }

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<LoanPlanDetailVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(LoanPlanDetailVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<LoanPlanDetailVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<LoanPlanExcelVo> exportLoanPlan(LoanPlanDto req) {
        String sql = "SELECT loan_id, loan_name, loan_sex, id_card, political_outlook, loan_phone, loan_nature, edu_level, enterprise_type, apply_amount, loan_type, CONCAT_WS('/',(SELECT bank_name FROM YCQ_QF_BANK WHERE bank_id = b.parent_bank_id),b.bank_name) AS bank_name, loan_use, company_name, (SELECT area_industry_name FROM YCQ_QF_AREA_INDUSTRY WHERE area_id = area_industry_id) AS areaName ,(SELECT area_industry_name FROM YCQ_QF_AREA_INDUSTRY WHERE industry = area_industry_id) industryName, operating_liabilities, last_year_income, last_year_taxation, staff_number, college_student_staff_number, create_time,notes, apply_code FROM YOUTH_CQ.YCQ_QF_LOAN_RECORD a LEFT JOIN YCQ_QF_BANK b ON a.intention_loan_bank =b.bank_id WHERE a.deleted = 0 ";
        List<Object> params = new ArrayList<>(10);
        if(StringUtils.hasText(req.getLoanName())){
            sql += " and loan_name like concat('%',?,'%')";
            params.add(req.getLoanName());
        }
        if(Objects.nonNull(req.getAreaId())){
            sql += " and area_id = ? ";
            params.add(req.getAreaId());
        }

        if(StringUtils.hasText(req.getIdCard())){
            sql += " and id_card like concat('%',?,'%')";
            params.add(req.getIdCard());
        }

        if (Objects.nonNull(req.getLoanType())) {
            sql += " and loan_type = ? ";
            params.add(req.getLoanType());
        }

        if (Objects.nonNull(req.getIntentionLoanBank())) {
            sql += " and intention_loan_bank = ? ";
            params.add(req.getIntentionLoanBank());
        }

        if (Objects.nonNull(req.getStartTime())) {
            sql += " and create_time >= ? ";
            params.add(req.getStartTime());
        }

        if (Objects.nonNull(req.getEndTime())) {
            sql += " and create_time <= ? ";
            params.add(req.getEndTime());
        }
        sql += " order by create_time desc";
        return jdbcTemplate.queryForBeans(LoanPlanExcelVo.class,sql,params.toArray());
    }

    public void batchUpdateHandle(List<Long> loanIds) {
        String sql = "UPDATE YCQ_QF_LOAN_RECORD SET handle = 1 WHERE loan_id IN (:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",loanIds);
        baseNpJdbcTemplate.update(sql,params);
    }


    public void batchUpdateLoanInfoByApplyCode(List<LoanPlanImportTemp> list) {
//       String sql = "UPDATE YCQ_QF_LOAN_RECORD SET loan_amount = :loanAmount, loan_successful = :loanSuccessful WHERE apply_code = :applyCode";
//
//        NamedParameterJdbcTemplate namedParameterJdbcTemplate = new NamedParameterJdbcTemplate(jdbcTemplate);
//
//        List<MapSqlParameterSource> batchArgs = new ArrayList<>();
//        for (LoanPlanImportTemp obj : list) {
//            MapSqlParameterSource parameterSource = new MapSqlParameterSource();
//            parameterSource.addValue("loanAmount", obj.getLoanAmount());
//            parameterSource.addValue("loanSuccessful", obj.getLoanSuccessful());
//            parameterSource.addValue("applyCode", obj.getApplyCode());
//            batchArgs.add(parameterSource);
//        }


//        jdbcTemplate.batchUpdate(sql, list, list.size(), (ps, item) -> {
//            ps.setBigDecimal(1, item.getLoanAmount());
//            ps.setString(2, item.getLoanSuccessful());
//            ps.setLong(3, item.getApplyCode());
//        });

        String sql = "UPDATE YCQ_QF_LOAN_RECORD SET loan_amount = ?, loan_successful = ? WHERE apply_code = ? ";
        int[][] rowCounts = jdbcTemplate.batchUpdate(sql, list, 5, (ps, argument) -> {
            ps.setBigDecimal(1,argument.getLoanAmount());
            ps.setString(2, argument.getLoanSuccessful());
            ps.setLong(3,argument.getApplyCode());
        });
    }

    public void updateLoanInfoByApplyCode(LoanPlanImportTemp importTemp) {
//       String sql = "UPDATE YCQ_QF_LOAN_RECORD SET loan_amount = :loanAmount, loan_successful = :loanSuccessful WHERE apply_code = :applyCode";
//
//        NamedParameterJdbcTemplate namedParameterJdbcTemplate = new NamedParameterJdbcTemplate(jdbcTemplate);
//
//        List<MapSqlParameterSource> batchArgs = new ArrayList<>();
//        for (LoanPlanImportTemp obj : list) {
//            MapSqlParameterSource parameterSource = new MapSqlParameterSource();
//            parameterSource.addValue("loanAmount", obj.getLoanAmount());
//            parameterSource.addValue("loanSuccessful", obj.getLoanSuccessful());
//            parameterSource.addValue("applyCode", obj.getApplyCode());
//            batchArgs.add(parameterSource);
//        }


//        jdbcTemplate.batchUpdate(sql, list, list.size(), (ps, item) -> {
//            ps.setBigDecimal(1, item.getLoanAmount());
//            ps.setString(2, item.getLoanSuccessful());
//            ps.setLong(3, item.getApplyCode());
//        });

        String sql = "UPDATE YCQ_QF_LOAN_RECORD SET loan_amount = ?, loan_successful = ? WHERE apply_code = ? ";
        jdbcTemplate.update(sql, importTemp.getLoanAmount(), importTemp.getLoanSuccessful(), importTemp.getApplyCode());
    }

    public List<LoanPlanSimpleVo.LoanUserInfo> getLatestLoanList() {
        String sql = "select loan_name,(SELECT area_industry_name FROM YCQ_QF_AREA_INDUSTRY WHERE area_id = area_industry_id) AS areaName, loan_amount \n" +
                "from YCQ_QF_LOAN_RECORD where loan_successful = '是' and deleted = 0 order by create_time desc limit 0,20";
       return jdbcTemplate.queryForBeans(LoanPlanSimpleVo.LoanUserInfo.class, sql);
    }

    public LoanPlanSimpleVo loanStatistics() {
        String sql = "select count(*) loanCount,sum(loan_amount) loanTotalAmount from YCQ_QF_LOAN_RECORD where  loan_successful = '是' and deleted = 0 ";
        return jdbcTemplate.queryForBean(LoanPlanSimpleVo.class, sql);
    }

    public Integer loanApplierTotal() {
        String sql = "select count(*) from YCQ_QF_LOAN_RECORD where deleted = 0";
        return jdbcTemplate.queryForInt(sql);
    }

    public List<YcqQfBankVo> queryBank(Long parentBankId) {
        String sql = "SELECT bank_id,bank_name,is_leaf FROM YCQ_QF_BANK WHERE parent_bank_id = ?";
        return jdbcTemplate.queryForBeans(YcqQfBankVo.class,sql,parentBankId);
    }

    public Map<String, Long> getMap(){
        String sql = "select area_industry_id, area_industry_name from YCQ_QF_AREA_INDUSTRY ";
        return jdbcTemplate.queryForBeans(YcqQfAreaIndustry.class, sql).stream().collect(Collectors.toMap(YcqQfAreaIndustry::getAreaIndustryName, YcqQfAreaIndustry::getAreaIndustryId));
    }

    public Map<String, List<TsmPropertyValue>> getPropertyMap(){
        String sql = "select * from TSM_PROPERTY_VALUE where prop_code in('qf_loan_type','qf_political_outlook','qf_loan_nature','qf_intention_loan_bank','qf_edu_level','qf_enterprise_type')";
        return jdbcTemplate.queryForBeans(TsmPropertyValue.class, sql).stream().collect(Collectors.groupingBy(TsmPropertyValue::getPropCode));
    }

    public List<LoanPlanExcelVo> findPlanByLoanIds(List<Long> loanIds) {
        String sql = "SELECT loan_id, loan_name, loan_sex, id_card, political_outlook, loan_phone, loan_nature, edu_level, enterprise_type, apply_amount, loan_type, b.bank_name, loan_use, company_name, (SELECT area_industry_name FROM YCQ_QF_AREA_INDUSTRY WHERE area_id = area_industry_id) AS areaName ,(SELECT area_industry_name FROM YCQ_QF_AREA_INDUSTRY WHERE industry = area_industry_id) industryName, operating_liabilities, last_year_income, last_year_taxation, staff_number, college_student_staff_number, create_time,notes, apply_code FROM YOUTH_CQ.YCQ_QF_LOAN_RECORD a LEFT JOIN YCQ_QF_BANK b ON a.intention_loan_bank =b.bank_id WHERE loan_id IN(:ids) ";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",loanIds);
        return baseNpJdbcTemplate.queryForBeans(LoanPlanExcelVo.class,sql,params);
    }

    public PageRsp<LoanPlanAuthVo> pageMyLoanStatistics(PageReq<LoanPlanAuthDto> dto, int loginId) {
        StringBuilder sb = new StringBuilder("SELECT loan_id,loan_name,create_time,apply_code FROM YCQ_QF_LOAN_RECORD WHERE creator_id = ? AND deleted = 0 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_QF_LOAN_RECORD WHERE creator_id = ? AND deleted = 0 ");
        List<Object> params = new ArrayList<>(10);
        LoanPlanAuthDto loanPlanDto = dto.getQuery();
        params.add(loginId);
        Integer audit = 0;
        if(audit.equals(loanPlanDto.getAuthState())){
            sb.append("  AND loan_successful IS null ");
            sbCount.append("  AND loan_successful IS null ");
        }else {
            sb.append("  AND loan_successful IS not null ");
            sbCount.append("  AND loan_successful IS not null ");
        }


        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<LoanPlanAuthVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, dto);
            values = jdbcTemplate.queryForBeans(LoanPlanAuthVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<LoanPlanAuthVo> page = new PageRsp<>();
        page.setPageNo(dto.getPageNo());
        page.setPageSize(dto.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public LoanPlanDetailVo findPlanDetail(Long loanId) {
        String sql = "SELECT loan_id,loan_name,loan_sex,id_card,political_outlook,loan_phone,loan_nature,edu_level,apply_amount,\n" +
                "enterprise_type,loan_type,(SELECT bank_name FROM YCQ_QF_BANK WHERE bank_id = a.intention_loan_bank) AS bankName,\n" +
                "loan_use,company_name,(SELECT area_industry_name FROM YCQ_QF_AREA_INDUSTRY WHERE area_industry_id = a.industry) industryName,\n" +
                "operating_liabilities,last_year_income,last_year_taxation,staff_number,college_student_staff_number,create_time,notes,\n" +
                "apply_code,loan_amount,loan_successful,(SELECT area_industry_name FROM YCQ_QF_AREA_INDUSTRY WHERE area_industry_id = a.area_id) areaName\n" +
                "FROM YCQ_QF_LOAN_RECORD a  WHERE loan_id = ? ORDER BY create_time desc";
        return jdbcTemplate.queryForBean(LoanPlanDetailVo.class,sql,loanId);
    }
}
