package com.crunii.micro.service.qm.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQmTeacherStu;
import com.crunii.micro.service.qm.api.domain.dto.WorkerHonorVo;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.StudentAuditVo;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.TeacherAppBaseDto;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.TeacherAppBaseVo;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.WorkerIdLeveName;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/9/20 10:26
 */
@Repository
public class StudentAppDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public Boolean findTeacherIsExist(int loginId) {
        String sql = "SELECT count(1) FROM YCQ_QM_TEACHER_STU A LEFT JOIN YCQ_QM_USER_INFO B ON A.teach_worker_id = B.worker_id WHERE A.stu_worker_id = ?  AND correlation_state in(1,2) AND A.binding = 1 AND A.deleted = 0 AND B.deleted = 0 AND B.audit_status =1 ";
        return jdbcTemplate.queryForRawObject(Boolean.class,sql,loginId);
    }

    public Integer findStuIsApply(int loginId) {
        String sql = " SELECT count(1) FROM YCQ_QM_TEACHER_STU WHERE stu_worker_id = ? AND correlation_state <> 3 and binding = 1 AND deleted = 0 limit 1";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,loginId);
    }

    public PageRsp<TeacherAppBaseVo> pageTeacherBase(PageReq<TeacherAppBaseDto> pageReq,List<Integer> teacherIds,List<String> levelCodes) {
        StringBuilder sb = new StringBuilder("SELECT * FROM (SELECT A.*,C.level_code ,O.org_name \n" +
                "as areaName,B.mobile,B.worker_name,B.head_portrait_pic_key,(SELECT count(1) \n" +
                "FROM YCQ_QM_TEACHER_STU A LEFT JOIN YCQ_QM_USER_INFO B ON A.stu_worker_id =B.worker_id \n" +
                "WHERE teach_worker_id =A.worker_id  AND A.deleted = 0 AND A.correlation_state in(1,2) \n" +
                "AND A.binding = 1 AND B.deleted=0 AND B.audit_status =1 )  stuNumber,\n" +
                "(SELECT count(1)  FROM YCQ_QM_TEACHER_STU a LEFT JOIN YCQ_QM_USER_INFO b ON a.stu_worker_id = b.worker_id\n" +
                "WHERE teach_worker_id = A.worker_id AND b.audit_status=1 AND b.deleted = 0 AND a.correlation_state =0 AND a.binding=1 AND a.deleted=0 LIMIT 1) AS choose\n" +
                "FROM YCQ_QM_USER_INFO A \n" +
                "LEFT JOIN TSM_ORG O ON A.area_id = O.org_id LEFT JOIN TSM_WORKER B ON A.worker_id = B.login_id \n" +
                "LEFT JOIN YCQ_QM_TEACHER_EXPAND C ON A.worker_id =C.worker_id WHERE A.user_identity = 2 \n" +
                "AND A.deleted = 0 AND A.audit_status =1  \n" +
                "AND C.deleted=0  AND C.expand_status =1) where 1 = 1 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM (SELECT A.*,C.level_code ,O.org_name \n" +
                "as areaName,B.mobile,B.worker_name,B.head_portrait_pic_key,(SELECT count(1) \n" +
                "FROM YCQ_QM_TEACHER_STU A LEFT JOIN YCQ_QM_USER_INFO B ON A.stu_worker_id =B.worker_id \n" +
                "WHERE teach_worker_id =A.worker_id  AND A.deleted = 0 AND A.correlation_state in(1,2) \n" +
                "AND A.binding = 1 AND B.deleted=0 AND B.audit_status =1 )  stuNumber,\n" +
                "(SELECT count(1)  FROM YCQ_QM_TEACHER_STU a LEFT JOIN YCQ_QM_USER_INFO b ON a.stu_worker_id = b.worker_id\n" +
                "WHERE teach_worker_id = A.worker_id AND b.audit_status=1 AND b.deleted = 0 AND a.correlation_state =0 AND a.binding=1 AND a.deleted=0 LIMIT 1) AS choose\n" +
                "FROM YCQ_QM_USER_INFO A \n" +
                "LEFT JOIN TSM_ORG O ON A.area_id = O.org_id LEFT JOIN TSM_WORKER B ON A.worker_id = B.login_id \n" +
                "LEFT JOIN YCQ_QM_TEACHER_EXPAND C ON A.worker_id =C.worker_id WHERE A.user_identity = 2 \n" +
                "AND A.deleted = 0 AND A.audit_status =1  \n" +
                "AND C.deleted=0  AND C.expand_status =1) where 1 = 1 ");
        Map<String,Object> params = new HashMap<>(8);
        TeacherAppBaseDto query = pageReq.getQuery();

        if(!CollectionUtils.isEmpty(levelCodes)){
            sb.append("and level_code in(:codes) ");
            sbCount.append("and level_code in(:codes) ");
            params.put("codes",levelCodes);
        }
        if(StringUtils.hasText(query.getWorkerName())){
            sb.append("AND worker_name like concat('%',:name,'%') ");
            sbCount.append("AND worker_name like concat('%',:name,'%') ");
            params.put("name",query.getWorkerName());
        }
        if(Objects.nonNull(query.getAreaId())){
            sb.append(" and area_id = :areaId");
            sbCount.append(" and area_id = :areaId");
            params.put("areaId",query.getAreaId());
        }

        if(Objects.nonNull(query.getIsChoose()) && query.getIsChoose() == 1){
            sb.append(" and stuNumber <5 ");
            sbCount.append(" and stuNumber <5 ");
            if(!CollectionUtils.isEmpty(teacherIds)){
                sb.append(" and worker_id in (:workerIds)");
                sbCount.append(" and worker_id in (:workerIds) ");
                params.put("workerIds",teacherIds);
            }
        }

        int total = baseNpJdbcTemplate.queryForInt(sbCount.toString(), params);
        List<TeacherAppBaseVo> values;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by choose desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = baseNpJdbcTemplate.queryForBeans(TeacherAppBaseVo.class, sb.toString(), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<TeacherAppBaseVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<WorkerIdLeveName> getWorkerLevelName(List<Integer> collect) {
        String sql = "SELECT worker_id,B.level_name FROM YCQ_QM_TEACHER_EXPAND A LEFT JOIN YCQ_QM_CLASSES_TYPE B ON A.level_code =B.level_code AND A.deleted=0 WHERE A.worker_id IN(:ids) AND level_name IS  NOT NULL GROUP BY  worker_id,B.level_name";
        Map<String,Object> params = new HashMap<>(3);
        params.put("ids",collect);
        return baseNpJdbcTemplate.queryForBeans(WorkerIdLeveName.class,sql,params);
    }


    public List<WorkerHonorVo> findTeacherHonor(int loginId) {
        String sql = "SELECT acquisition_time,honor_desc FROM YCQ_QM_USER_HONOR WHERE worker_id = ? AND honor_type = 1 AND deleted =0 order by acquisition_time desc";
        return jdbcTemplate.queryForBeans(WorkerHonorVo.class,sql,loginId);
    }

    public List<String> findTeacherField(int loginId) {
        String sql = "SELECT honor_desc FROM YCQ_QM_USER_HONOR WHERE worker_id = ? AND honor_type = 2 AND deleted = 0";
        return jdbcTemplate.queryForRawObjects(String.class,sql,loginId);
    }

    public Long findStuClassesId(Integer workerId) {
        String sql = "SELECT A.classes_id  FROM YCQ_QM_USER_CLASSES_REL A LEFT JOIN YCQ_QM_CLASSES_INFO B ON A.classes_id  =B.classes_id LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id WHERE stu_worker_id = ? ORDER BY level_name LIMIT 1";
        return jdbcTemplate.queryForRawObject(Long.class,sql,workerId);
    }

    public List<Integer> findAuditWorkerId(int loginId) {
        String sql = "SELECT teach_worker_id FROM YCQ_QM_TEACHER_STU WHERE stu_worker_id = ? AND correlation_state = 0 AND deleted = 0 ";
        return jdbcTemplate.queryForRawObjects(Integer.class,sql,loginId);
    }

    public List<Integer> findRejectTeacherId(int loginId) {
        String sql = "SELECT DISTINCT(teach_worker_id) FROM YCQ_QM_TEACHER_STU_REJECT WHERE stu_worker_id = ? AND deleted = 0 ";
        return jdbcTemplate.queryForRawObjects(Integer.class,sql,loginId);
    }

    public YcqQmTeacherStu findTeacherStu(Integer teacherId, Worker stuId) {
        String sql = "SELECT * FROM YCQ_QM_TEACHER_STU WHERE teach_worker_id = ? AND stu_worker_id = ? and binding = 1 AND deleted = 0 ";
        return jdbcTemplate.queryForBean(YcqQmTeacherStu.class,sql,teacherId,stuId.getLoginId());
    }


    public String findFlowId(Integer workerId, int loginId) {
        String sql = " SELECT flow_instance_id FROM YCQ_QM_FLOW_RECORD WHERE service_id = ( SELECT tutor_trainee_id FROM YCQ_QM_TEACHER_STU WHERE teach_worker_id = ? AND stu_worker_id = ? AND deleted = 0 AND binding = 1 ORDER BY create_time DESC LIMIT 1 )";
        return jdbcTemplate.queryForRawObject(String.class,sql,workerId,loginId);
    }

    public List<String> findLevelCodes(int loginId) {
        String sql = "SELECT c.level_code FROM YCQ_QM_USER_CLASSES_REL a LEFT JOIN YCQ_QM_CLASSES_INFO b ON a.classes_id =b.classes_id LEFT JOIN YCQ_QM_CLASSES_TYPE c ON\tb.classes_type_id = c.classes_type_id WHERE a.deleted =0 AND b.deleted =0 AND c.deleted =0 AND  a.stu_worker_id =? ORDER BY c.level_code LIMIT 1";
        return jdbcTemplate.queryForRawObjects(String.class,sql,loginId);
    }

    public StudentAuditVo findStuAuditState(int loginId) {
        String sql = "SELECT teach_worker_id as workerId,tutor_trainee_id,correlation_state as auditState,(SELECT worker_name FROM TSM_WORKER WHERE login_id = teach_worker_id ) AS workerName FROM YCQ_QM_TEACHER_STU WHERE stu_worker_id = ? AND binding = 1 AND deleted = 0 order by update_time desc limit 1";
        return jdbcTemplate.queryForBean(StudentAuditVo.class,sql,loginId);
    }

    public Integer findTeacherStuCount(Integer teachWorkerId) {
        String sql = "SELECT count(1) FROM YCQ_QM_TEACHER_STU WHERE teach_worker_id = ? and correlation_state in(1,2) AND binding = 1 AND deleted = 0 ";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,teachWorkerId);
    }

    public void delAuditRecord(Long tutorTraineeId) {
        String sql = "UPDATE YCQ_QM_TEACHER_STU SET deleted = 1 WHERE tutor_trainee_id = ?";
        jdbcTemplate.update(sql,tutorTraineeId);
    }

    public StudentAuditVo findIsBinging(int loginId) {
        String sql = "SELECT teach_worker_id,tutor_trainee_id,correlation_state as auditState,(SELECT worker_name FROM TSM_WORKER WHERE login_id = teach_worker_id ) AS workerName FROM YCQ_QM_TEACHER_STU WHERE stu_worker_id = ? AND binding = 0 AND deleted = 0 order by update_time desc limit 1";
        return jdbcTemplate.queryForBean(StudentAuditVo.class,sql,loginId);
    }

    public Integer findStuRejectCount(int loginId) {
        String sql = "SELECT count(1) FROM YCQ_QM_TEACHER_STU_REJECT WHERE stu_worker_id = ? AND deleted = 0";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,loginId);
    }

    public String findUpdateWorkerName(int loginId) {
        String sql = "SELECT (SELECT worker_name FROM TSM_WORKER WHERE login_id = updater_id ) AS workerName FROM YCQ_QM_TEACHER_STU WHERE stu_worker_id = ? AND deleted = 0 order by update_time desc limit 1";
        return jdbcTemplate.queryForRawObject(String.class,sql,loginId);
    }

    public void delExertLink(Long exertId) {
        String sql =  "UPDATE YCQ_QM_EXERT_LINK SET deleted = 1 WHERE exert_id = ? AND deleted = 0";
        jdbcTemplate.update(sql,exertId);
    }

    public void updateVideo(Collection<String> videoDelIds) {
        String sql = "UPDATE YCQ_QM_VIDEO set deleted = 0 WHERE file_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",videoDelIds);
        baseNpJdbcTemplate.update(sql,params);
    }
}
