package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.entity.main.YcqQmClassesInfo;
import com.crunii.micro.domain.entity.main.YcqQmUserAuditRecord;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.*;
import com.crunii.micro.service.qm.util.ListUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author baoxy
 * @date 2023-09-18
 */
@Repository
public class StudentManageWebDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public static final String PAGE_WAIT_EXAMINE_SQL = "select * from (select distinct users.id as user_info_id, users.worker_id, " +
            "users.user_code,worker.worker_name,worker.mobile, users.org_id,org.org_name, users.worker_job,users.create_time, " +
            "users.update_time from ycq_qm_user_info users,ycq_qm_classes_info classes,tsm_worker worker,tsm_org org where " +
            "users.worker_id = worker.login_id  and users.org_id = org.org_id " +
            "and audit_status = ? and user_identity = '1' and classes.deleted = '0' and classes.classes_status = '1' ";

    public static final String PAGE_WAIT_EXAMINE_COUNT_SQL = "select count(1) from (select distinct users.id, users.worker_id, " +
            "users.user_code, worker.worker_name, worker.mobile,users.org_id,org.org_name, users.worker_job,users.create_time," +
            "users.update_time from ycq_qm_user_info users,ycq_qm_classes_info classes, tsm_worker worker,tsm_org org " +
            "where users.worker_id = worker.login_id  and users.org_id = org.org_id " +
            "and audit_status = ? and user_identity = '1' and classes.deleted = '0' and classes.classes_status = '1' ";

    public static final String VIEW_STUDENT_SQL = "select distinct users.id as user_info_id, users.worker_id, worker.worker_name," +
            "users.card_id,users.work_unit,users.worker_job,users.org_id,org.org_name,users.area_id, area.org_name as area_name," +
            "users.political_outlook, worker.mobile,users.sex, users.education,users.user_code from ycq_qm_user_info users," +
            "tsm_worker worker, tsm_org org,tsm_org area where users.id = ? and users.worker_id = worker.login_id and " +
            "users.org_id = org.org_id and users.area_id = area.org_id ";

    public static final String PAGE_PASS_EXAMINE_SQL = "select * from (select a.*, d.record_id,case when count_b >=2 then '2' when " +
            "c.correlation_state = 1 or c.correlation_state = 2 then 3 when correlation_state = 0 then 0 else 1 end as teacher_mate " +
            "from (select distinct users.id as user_info_id,users.worker_id,worker.worker_name, users.card_id,org.org_id," +
            "org.org_name,users.worker_job,users.political_outlook,worker.mobile,users.sex,users.education,users.user_code, " +
            "users.create_time,users.update_time,users.deleted as stu_state " +
            "from ycq_qm_user_info users, ycq_qm_classes_info classes,tsm_worker worker," +
            "tsm_org org where classes.deleted = 0 and classes.classes_status = 1 and audit_status = ? " +
            "and user_identity = 1 and users.worker_id = worker.login_id and users.org_id = org.org_id ";

    public static final String PAGE_PASS_EXAMINE_COUNT_SQL = "select count(1) from (select a.*, d.record_id,case when count_b >=2 then '2' when " +
            "c.correlation_state = 1 or c.correlation_state = 2 then 3 when correlation_state = 0 then 0 else 1 end as teacher_mate " +
            "from (select distinct users.id as user_info_id,users.worker_id,worker.worker_name, users.card_id,org.org_id," +
            "org.org_name,users.worker_job,users.political_outlook,worker.mobile,users.sex,users.education,users.user_code, " +
            "users.create_time,users.update_time from ycq_qm_user_info users, ycq_qm_classes_info classes,tsm_worker worker," +
            "tsm_org org where classes.deleted = 0 and classes.classes_status = 1 and audit_status = ? " +
            "and user_identity = 1 and users.worker_id = worker.login_id and users.org_id = org.org_id ";

    public static final String VIEW_TEACHER_SQL = "select a.*,worker.worker_name, worker.mobile ,area.org_name as area_name," +
            "classes.classes_year,type.type_name, type.level_name from (select users.id as user_info_id, users.worker_id,users.card_id," +
            "users.area_id,users.political_outlook,users.sex,users.education,users.user_code,stu.stu_classes_id as classes_id from " +
            "(select stu.stu_classes_id, stu.teach_worker_id from ycq_qm_user_info users, ycq_qm_teacher_stu stu where users.worker_id = " +
            "stu.stu_worker_id and users.id = ? and stu.deleted = ?) stu inner join ycq_qm_user_info users on " +
            "stu.teach_worker_id = users.worker_id) a,ycq_qm_classes_info classes,ycq_qm_classes_type type," +
            "tsm_worker worker,tsm_org area where a.classes_id = classes.classes_id and classes.classes_type_id = " +
            "type.classes_type_id and a.worker_id = worker.login_id and a.area_id = area.org_id";

    public static final String TEACHER_LIST_SQL = "select worker_name, mobile, worker_id as teach_worker_id from " +
            "(select distinct case when tutor_trainee_id is not null then count(tutor_trainee_id) else '0' end as num, worker_name, mobile, " +
            "worker_id from (select a.worker_name,a.mobile,a.worker_id, stu.tutor_trainee_id from (select worker_name,mobile," +
            "info.worker_id from ycq_qm_user_info info,tsm_worker worker,ycq_qm_teacher_expand expand where " +
            "info.worker_id = worker.login_id and info.user_identity = '2' and audit_status = '1' and expand.expand_status = '1' " +
            "and expand.deleted = '0' and info.worker_id = expand.worker_id ";

    public static final String BINDING_TEACHER_LIST_SQL = "select worker.worker_name,worker.mobile, a.teach_worker_id from " +
            "(select stu.teach_worker_id from ycq_qm_user_info info, (select * from ycq_qm_teacher_stu where deleted = ?) stu " +
            "where info.worker_id = stu.stu_worker_id and info.id = ? and binding = '1' AND correlation_state IN (1,2)) a " +
            "inner join tsm_worker worker on a.teach_worker_id = worker.login_id";

    public static final String EXPORT_PASS_EXAMINE_SQL = "select * from (select a.*,d.record_id," +
            "case when count_b >=2 then '2' when c.correlation_state = 1 or c.correlation_state = 2 then 3 when correlation_state = 0 " +
            "then 0 else 1 end as teacher_mate from (select distinct users.id as user_info_id," +
            "users.worker_id,worker.worker_name, users.card_id,org.org_id,org.org_name,users.worker_job,users.political_outlook," +
            "worker.mobile,users.sex,users.education,users.user_code, users.create_time,users.update_time," +
            "users.deleted as stu_state from ycq_qm_user_info users," +
            "ycq_qm_classes_info classes,tsm_worker worker,tsm_org org where classes.deleted = '0' and " +
            "classes.classes_status = '1' and audit_status = '1' and user_identity = '1' and users.worker_id = worker.login_id and " +
            "users.org_id = org.org_id ) a left join (select count(stu_worker_id) as count_b, stu_worker_id from ycq_qm_teacher_stu_reject " +
            "where deleted = '0' group by stu_worker_id) b on a.worker_id = b.stu_worker_id  left join " +
            "(select correlation_state, stu_worker_id from ycq_qm_teacher_stu where deleted = '0' and binding = '1' ) c " +
            "on a.worker_id = c.stu_worker_id left join (select * from ycq_qm_below_excellen where deleted = '0' and config_type = '0') " +
            "d on a.worker_id = d.stu_worker_id) where record_id is null ";

    public PageRsp<PassExamineVo> pageWaitExamine(PageReq<PassExamineDto> req, List<Long> orgIdList) {
        StringBuilder sql = new StringBuilder(PAGE_WAIT_EXAMINE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_WAIT_EXAMINE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        param.add(req.getQuery().getAuditStatus());
        commonQuery(sql, countSql, param, orgIdList, req.getQuery().getSex(), req.getQuery().getWorkerJob(),
                req.getQuery().getUserCode(), req.getQuery().getWorkerName(), req.getQuery().getMobile(), req.getQuery().getStartTime(),
                req.getQuery().getEndTime());
        sql.append(") a ");
        countSql.append(") a ");
        // 筛选班级id
        if (ObjectUtil.isNotEmpty(req.getQuery().getClassesId())) {
            sql.append("inner join ycq_qm_user_classes_rel rel on a.worker_id = rel.stu_worker_id and rel.deleted = 0 and rel.classes_id = ?");
            countSql.append("inner join ycq_qm_user_classes_rel rel on a.worker_id = rel.stu_worker_id and rel.deleted = 0 and rel.classes_id = ?");
            param.add(req.getQuery().getClassesId());
        }
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dyType = jt.getDbType();
        List<PassExamineVo> result;
        if (total > 0) {
            sql.append(" order by update_time desc, create_time desc ");
            SqlUtil.appendLimit(dyType, param, sql, req);
            result = jt.queryForBeans(PassExamineVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<PassExamineVo> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageNo());
        rsp.setResult(result);
        rsp.setTotalNum(total);
        return rsp;
    }

    public ViewStudentVo viewStudent(ViewStudentDto dto) {
        return jt.queryForBean(ViewStudentVo.class, VIEW_STUDENT_SQL, dto.getUserInfoId());
    }

    public List<ViewTeacherVo> viewTeacher(ViewStudentDto dto) {
        return jt.queryForBeans(ViewTeacherVo.class, VIEW_TEACHER_SQL, dto.getUserInfoId(), QmConstants.DELETED_0);
    }

    public int waitExamineCount(SearchNumDto dto, List<Long> orgIdList) {
        StringBuilder countSql = new StringBuilder(PAGE_WAIT_EXAMINE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        param.add(QmConstants.AUDIT_STATUS_WAIT);
        commonQuery(new StringBuilder(), countSql, param, orgIdList, dto.getSex(), dto.getWorkerJob(),
                dto.getUserCode(), dto.getWorkerName(), dto.getMobile(), dto.getStartTime(), dto.getEndTime());
        countSql.append(") a ");
        // 筛选班级id
        if (ObjectUtil.isNotEmpty(dto.getClassesId())) {
            countSql.append("inner join ycq_qm_user_classes_rel rel on a.worker_id = rel.stu_worker_id and rel.deleted = 0 " +
                    "and rel.classes_id = ?");
            param.add(dto.getClassesId());
        }
        return jt.queryForInt(countSql.toString(), param.toArray());
    }

    public int notPassExamineCount(SearchNumDto dto, List<Long> orgIdList) {
        StringBuilder countSql = new StringBuilder(PAGE_WAIT_EXAMINE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        param.add(QmConstants.AUDIT_STATUS_NO);
        commonQuery(new StringBuilder(), countSql, param, orgIdList, dto.getSex(), dto.getWorkerJob(),
                dto.getUserCode(), dto.getWorkerName(), dto.getMobile(), dto.getStartTime(),
                dto.getEndTime());
        countSql.append(") a ");
        // 筛选班级id
        if (ObjectUtil.isNotEmpty(dto.getClassesId())) {
            countSql.append("inner join ycq_qm_user_classes_rel rel on a.worker_id = rel.stu_worker_id and rel.deleted = 0 " +
                    "and rel.classes_id = ?");
            param.add(dto.getClassesId());
        }
        return jt.queryForInt(countSql.toString(), param.toArray());
    }

    public int passExamineCount(SearchNumDto dto, List<Long> orgIdList) {
        StringBuilder countSql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonPassExamine(new StringBuilder(), countSql, param, orgIdList, dto.getClassesId(), dto.getSex(), dto.getWorkerJob(),
                dto.getUserCode(), dto.getWorkerName(), dto.getMobile(), dto.getStartTime(), dto.getEndTime(), dto.getTeacherMate());
        return jt.queryForInt(countSql.toString(), param.toArray());
    }

    public int belowExcellentCount(SearchNumDto dto, List<Long> orgIdList) {
        StringBuilder countSql = new StringBuilder("SELECT count(*) FROM YCQ_QM_USER_INFO A\n"
                + " INNER JOIN YCQ_QM_BELOW_EXCELLEN BX ON A.worker_id = BX.stu_worker_id AND BX.deleted=0 AND BX.config_type = 0 \n"
                + " INNER JOIN TSM_WORKER TW ON A.worker_id = TW.login_id AND TW.state = 1\n"
                + " INNER JOIN TSM_ORG O ON A.org_id = O.org_id AND O.state = 1\n"
                + " LEFT JOIN (SELECT TSR.stu_worker_id,COUNT(TSR.stu_worker_id) AS passCount FROM YCQ_QM_TEACHER_STU_REJECT TSR \n"
                + " WHERE TSR.deleted=0 GROUP BY TSR.stu_worker_id) TSR ON A.worker_id = TSR.stu_worker_id\n"
                + " LEFT JOIN (SELECT TS.stu_worker_id,TS.correlation_state FROM YCQ_QM_TEACHER_STU TS\n"
                + " WHERE TS.deleted=0 AND TS.binding=1 ) TS ON TS.stu_worker_id=A.worker_id ");
        List<Object> params = new ArrayList<>();

        //班级id
        if (ObjectUtil.isNotEmpty(dto.getClassesId())) {
            countSql.append(" INNER JOIN YCQ_QM_USER_CLASSES_REL CR ON A.worker_id = CR.stu_worker_id AND CR.deleted = 0 AND CR.classes_id = ? ");
            params.add(dto.getClassesId());
        }
        countSql.append(" WHERE A.user_identity=1 ");
        //组织id
        if (ListUtil.hasValue(orgIdList)) {
            String addSql = " and A.org_id in " + orgIdList.stream().map(i -> " ? ").collect(Collectors.joining(",", "(", ")"));
            countSql.append(addSql);
            params.addAll(orgIdList);
        }
//        if (ObjectUtil.isNotEmpty(dto.getOrgId())) {
//            countSql.append(" and A.org_id = ? ");
//            params.add(dto.getOrgId());
//        }
        //性别
        if (ObjectUtil.isNotEmpty(dto.getSex())) {
            countSql.append(" and A.sex = ? ");
            params.add(dto.getSex());
        }
        //职务
        if (ObjectUtil.isNotEmpty(dto.getWorkerJob())) {
            countSql.append(" and A.worker_job like concat('%',?,'%') ");
            params.add(dto.getWorkerJob());
        }
        //导师匹配状态
        if (ObjectUtil.isNotEmpty(dto.getTeacherMate())) {
            //未匹配:被拒绝两次(1个导师拒绝两次，或不同导师累计拒绝两次)
            if (QmConstants.TEACHER_MATE_NOT.equals(dto.getTeacherMate())) {
                countSql.append(" AND TSR.passCount>=2 ");
                //已匹配:被导师接受申请或者被导师选中
            } else if (QmConstants.TEACHER_MATE_YES.equals(dto.getTeacherMate())) {
                countSql.append(" AND (TS.correlation_state=2 OR TS.correlation_state=1) ");
                //待审核:学员申请了导师,导师还未同意接收
            } else if (QmConstants.TEACHER_MATE_WAIT.equals(dto.getTeacherMate())) {
                countSql.append(" AND TS.correlation_state=0 ");
                //未选择:未进行导师选择或未被导师选择,或选择了一次被导师拒绝了
            } else if (QmConstants.TEACHER_MATE_NOT_SELECT.equals(dto.getTeacherMate())) {
                countSql.append(" AND (TSR.passCount<2 AND (TS.correlation_state=3 OR TS.stu_worker_id IS NULL)) ");
            }
        }
        //学员编号
        if (ObjectUtil.isNotEmpty(dto.getUserCode())) {
            countSql.append(" and A.user_code = ? ");
            params.add(dto.getUserCode());
        }
        //学员名称
        if (ObjectUtil.isNotEmpty(dto.getWorkerName())) {
            countSql.append(" and TW.worker_name like concat('%',?,'%') ");
            params.add(dto.getWorkerName());
        }
        //学员手机
        if (ObjectUtil.isNotEmpty(dto.getMobile())) {
            countSql.append(" and TW.mobile = ? ");
            params.add(dto.getMobile());
        }
        //开始时间
        if (ObjectUtil.isNotEmpty(dto.getStartTime())) {
            countSql.append(" AND BX.recommend_time >= ? ");
            params.add(dto.getStartTime());
        }
        //结束时间
        if (ObjectUtil.isNotEmpty(dto.getEndTime())) {
            countSql.append(" AND BX.recommend_time <= ? ");
            params.add(dto.getEndTime());
        }
        return jt.queryForInt(countSql.toString(), params.toArray());
    }

    /**
     * 查询被拒绝两次学员数量
     *
     * @param dto       入参实体
     * @param orgIdList 组织id列表
     * @return int
     */
    public int stuRefuseTwiceCount(SearchNumDto dto, List<Long> orgIdList) {
        StringBuilder countSql = new StringBuilder("SELECT DISTINCT count(*) FROM YCQ_QM_USER_INFO A\n"
                + " INNER JOIN TSM_WORKER TW ON A.worker_id = TW.login_id AND TW.state = 1\n"
                + " INNER JOIN TSM_ORG O ON A.org_id = O.org_id AND O.state = 1\n"
                + " LEFT JOIN (SELECT TSR.stu_worker_id,COUNT(TSR.stu_worker_id) AS passCount FROM YCQ_QM_TEACHER_STU_REJECT TSR \n"
                + " WHERE TSR.deleted=0 GROUP BY TSR.stu_worker_id) TSR ON A.worker_id = TSR.stu_worker_id\n"
                + " LEFT JOIN (SELECT TS.stu_worker_id,TS.correlation_state FROM YCQ_QM_TEACHER_STU TS\n"
                + " WHERE TS.deleted=0 AND TS.binding=1 ) TS ON TS.stu_worker_id=A.worker_id ");
        List<Object> params = new ArrayList<>();

        //班级id
        if (ObjectUtil.isNotEmpty(dto.getClassesId())) {
            countSql.append(" INNER JOIN YCQ_QM_USER_CLASSES_REL CR ON A.worker_id = CR.stu_worker_id AND CR.deleted = 0 AND CR.classes_id = ? ");
            params.add(dto.getClassesId());
        }
        countSql.append(" WHERE A.user_identity=1 AND TSR.passCount>=2 ");
        //组织id
        if (ListUtil.hasValue(orgIdList)) {
            String addSql = " and A.org_id in " + orgIdList.stream().map(i -> " ? ").collect(Collectors.joining(",", "(", ")"));
            countSql.append(addSql);
            params.addAll(orgIdList);
        }
//        if (ObjectUtil.isNotEmpty(dto.getOrgId())) {
//            countSql.append(" and A.org_id = ? ");
//            params.add(dto.getOrgId());
//        }
        //性别
        if (ObjectUtil.isNotEmpty(dto.getSex())) {
            countSql.append(" and A.sex = ? ");
            params.add(dto.getSex());
        }
        //职务
        if (ObjectUtil.isNotEmpty(dto.getWorkerJob())) {
            countSql.append(" and A.worker_job like concat('%',?,'%') ");
            params.add(dto.getWorkerJob());
        }

        //学员编号
        if (ObjectUtil.isNotEmpty(dto.getUserCode())) {
            countSql.append(" and A.user_code = ? ");
            params.add(dto.getUserCode());
        }
        //学员名称
        if (ObjectUtil.isNotEmpty(dto.getWorkerName())) {
            countSql.append(" and TW.worker_name like concat('%',?,'%') ");
            params.add(dto.getWorkerName());
        }
        //学员手机
        if (ObjectUtil.isNotEmpty(dto.getMobile())) {
            countSql.append(" and TW.mobile = ? ");
            params.add(dto.getMobile());
        }
        //开始时间
        if (ObjectUtil.isNotEmpty(dto.getStartTime())) {
            countSql.append(" AND A.create_time >= ? ");
            params.add(dto.getStartTime());
        }
        //结束时间
        if (ObjectUtil.isNotEmpty(dto.getEndTime())) {
            countSql.append(" AND A.create_time <= ? ");
            params.add(dto.getEndTime());
        }
        return jt.queryForInt(countSql.toString(), params.toArray());
    }


    public PageRsp<PassExamineVo> pagePassExamine(PageReq<PassExamineDto> req, List<Long> orgIdList) {
        StringBuilder sql = new StringBuilder();
        StringBuilder countSql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonPassExamine(sql, countSql, param, orgIdList, req.getQuery().getClassesId(), req.getQuery().getSex(),
                req.getQuery().getWorkerJob(), req.getQuery().getUserCode(), req.getQuery().getWorkerName(), req.getQuery().getMobile(),
                req.getQuery().getStartTime(), req.getQuery().getEndTime(), req.getQuery().getTeacherMate());
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dyType = jt.getDbType();
        List<PassExamineVo> result;
        if (total > 0) {
            sql.append(" order by update_time desc, create_time desc ");
            SqlUtil.appendLimit(dyType, param, sql, req);
            result = jt.queryForBeans(PassExamineVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<PassExamineVo> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageNo());
        rsp.setResult(result);
        rsp.setTotalNum(total);
        return rsp;
    }

    public List<PassExamineExcel> exportPassExamine(ExportExamineDto dto) {
        StringBuilder sql = new StringBuilder(EXPORT_PASS_EXAMINE_SQL);
        List<Object> param = new ArrayList<>();
        if (Objects.nonNull(dto.getUserInfoList())) {
            if (dto.getUserInfoList().size() > 0) {
                sql.append(" and user_info_id in (");
                dto.getUserInfoList().forEach(e -> {
                    sql.append("?,");
                    param.add(e);
                });
                sql.deleteCharAt(sql.length() - 1).append(") ");
            }
        }
        sql.append(" order by update_time desc, create_time desc ");
        return jt.queryForBeans(PassExamineExcel.class, sql.toString(), param.toArray());
    }

    public List<TeacherListVo> teacherList(TeacherListDto dto, String levelCode) {
        StringBuilder sql = new StringBuilder(TEACHER_LIST_SQL);
        List<Object> param = new ArrayList<>();

        if (QmConstants.CITY_LEVEL.equals(levelCode)) {
            sql.append(" and( expand.level_code = ? or  expand.level_code = ? )");
            param.add(levelCode);
            param.add(QmConstants.NATIONAL_LEVEL);
        } else {
            sql.append(" and expand.level_code = ? ");
            param.add(levelCode);
        }

        if (StringUtils.isNotBlank(dto.getQuery())) {
            sql.append(" and (worker.worker_name like ? or worker.mobile like ?) ");
            param.add("%" + dto.getQuery() + "%");
            param.add("%" + dto.getQuery() + "%");
        }
        sql.append(" ) a left join (select * from ycq_qm_teacher_stu where deleted = ?) stu on a.worker_id = stu.stu_worker_id) " +
                "group by worker_name, mobile,worker_id,tutor_trainee_id) where num < ?");
        param.add(QmConstants.DELETED_0);
        param.add(QmConstants.TEACHER_MAX_STU);
        return jt.queryForBeans(TeacherListVo.class, sql.toString(), param.toArray());
    }


    public void commonPassExamine(StringBuilder sql, StringBuilder countSql, List<Object> param, List<Long> orgIdList,
                                  Long classesId, Integer sex, String workerJob, String userCode, String workerName,
                                  String mobile, Date startTime, Date endTime, String teacherMate) {
        sql.append(PAGE_PASS_EXAMINE_SQL);
        countSql.append(PAGE_PASS_EXAMINE_COUNT_SQL);
        param.add(QmConstants.AUDIT_STATUS_YES);
        commonQuery(sql, countSql, param, orgIdList, sex, workerJob, userCode, workerName, mobile, startTime, endTime);
        sql.append(") a ");
        countSql.append(") a ");
        // 筛选班级id
        if (ObjectUtil.isNotEmpty(classesId)) {
            sql.append("inner join ycq_qm_user_classes_rel rel on a.worker_id = rel.stu_worker_id and rel.deleted = 0 and rel.classes_id = ?");
            countSql.append("inner join ycq_qm_user_classes_rel rel on a.worker_id = rel.stu_worker_id and rel.deleted = 0 and rel.classes_id = ?");
            param.add(classesId);
        }
        sql.append(" left join (select count(stu_worker_id) as count_b, stu_worker_id from ycq_qm_teacher_stu_reject where " +
                "deleted = 0 group by stu_worker_id) b on a.worker_id = b.stu_worker_id left join (select correlation_state, " +
                "stu_worker_id from ycq_qm_teacher_stu where deleted = '0' and binding = '1' ) c on a.worker_id = c.stu_worker_id left join " +
                "(select * from ycq_qm_below_excellen where deleted = '0' and config_type = '0') d on a.worker_id = d.stu_worker_id " +
                ") where record_id is null ");
        countSql.append(" left join (select count(stu_worker_id) as count_b, stu_worker_id from ycq_qm_teacher_stu_reject where " +
                "deleted = 0 group by stu_worker_id) b on a.worker_id = b.stu_worker_id left join (select correlation_state, " +
                "stu_worker_id from ycq_qm_teacher_stu where deleted = '0' and binding = '1' ) c on a.worker_id = c.stu_worker_id left join " +
                "(select * from ycq_qm_below_excellen where deleted = '0' and config_type = '0') d on a.worker_id = d.stu_worker_id " +
                ") where record_id is null ");
        if (StringUtils.isNotBlank(teacherMate)) {
            sql.append("and teacher_mate = ? ");
            countSql.append("and teacher_mate = ? ");
            param.add(teacherMate);
        }
    }

    private void commonQuery(StringBuilder sql, StringBuilder countSql, List<Object> param, List<Long> orgIdList, Integer sex,
                             String workerJob, String userCode, String workerName, String mobile, Date startTime, Date endTime) {
        if (ListUtil.hasValue(orgIdList)) {
            String addSql = " and users.org_id in " + orgIdList.stream().map(i -> " ? ").collect(Collectors.joining(",", "(", ")"));
            sql.append(addSql);
            countSql.append(addSql);
            param.addAll(orgIdList);
        }
//        if (StringUtils.isNotBlank(orgId)) {
//            // 这里要校验传过来orgId是不是根结点,如果是，则不需要带入下面的查询条件
//            if (!checkIsTopOrg(orgId)) {
//                sql.append(" and users.org_id = ? ");
//                countSql.append(" and users.org_id = ? ");
//                param.add(orgId);
//            }
//        }
        if (Objects.nonNull(sex)) {
            sql.append(" and users.sex = ? ");
            countSql.append(" and users.sex = ? ");
            param.add(sex);
        }
        if (StringUtils.isNotBlank(workerJob)) {
            sql.append(" and users.worker_job = ? ");
            countSql.append(" and users.worker_job = ? ");
            param.add(workerJob);
        }
        if (StringUtils.isNotBlank(userCode)) {
            sql.append(" and users.user_code = ? ");
            countSql.append(" and users.user_code = ? ");
            param.add(userCode);
        }
        if (StringUtils.isNotBlank(workerName)) {
            sql.append(" and worker.worker_name like ? ");
            countSql.append(" and worker.worker_name like ? ");
            param.add("%" + workerName + "%");
        }
        if (StringUtils.isNotBlank(mobile)) {
            sql.append(" and worker.mobile = ? ");
            countSql.append(" and worker.mobile = ? ");
            param.add(mobile);
        }
        if (Objects.nonNull(startTime) && Objects.nonNull(endTime)) {
            sql.append(" and users.create_time >= ? and users.create_time <= ? ");
            countSql.append(" and users.create_time >= ? and users.create_time <= ? ");
            param.add(startTime);
            param.add(endTime);
        }
    }

    public Integer findLoginIdByMobile(String mobile) {
        String sql = "select login_id from tsm_worker where mobile = ? and state = ? ";
        return jt.queryForRawObject(Integer.class, sql, mobile, QmConstants.STATUS);
    }

    public Integer findQmStudentExist(Integer loginId) {
        String sql = "select count(1) from ycq_qm_user_info where worker_id = ? ";
        return jt.queryForRawObject(Integer.class, sql, loginId);
    }

    public int checkRepeatWorkerName(String name, String mobile) {
        String sql = "select count(1) from tsm_worker where worker_name = ? and mobile = ? and state = ? ";
        return jt.queryForInt(sql, name, mobile, QmConstants.STATUS);
    }

    public YcqQmUserAuditRecord latestNote(Long userInfoId) {
        // 取出最新一条审核意见数据
        String sql = "select note from ycq_qm_user_audit_record where user_info_id = ? order by audit_time desc limit 1";
        return jt.queryForBean(YcqQmUserAuditRecord.class, sql, userInfoId);
    }

    public List<ClassesInfoVo> getClassesInfo(Long userInfoId) {
        String sql = "select * from (select distinct classes.classes_id,classes.classes_code,users.worker_id as stu_worker_id,type.type_code, " +
                "type.type_name, type.level_name, type.level_code, classes.classes_year,org.org_name as classesOrgName," +
                "classes.org_id as classesOrgId,type.level_org_tier from ycq_qm_user_info users,ycq_qm_classes_info classes, " +
                "ycq_qm_user_classes_rel rel,ycq_qm_classes_type type, tsm_org org where " +
                "users.worker_id = rel.stu_worker_id and classes.classes_id = rel.classes_id and classes.classes_type_id = type.classes_type_id " +
                "and users.id = ? and classes.deleted = ? and classes.classes_status = ? and rel.deleted = ? and org.org_id=classes.org_id " +
                ") order by level_code desc";
        List<Object> param = new ArrayList<>();
        param.add(userInfoId);
        param.add(QmConstants.DELETED_0);
        param.add(QmConstants.STATUS);
        param.add(QmConstants.DELETED_0);
        return jt.queryForBeans(ClassesInfoVo.class, sql, param.toArray());
    }

    public List<TeacherListVo> bindingTeacherList(TeacherBindingListDto dto) {
        StringBuilder sql = new StringBuilder(BINDING_TEACHER_LIST_SQL);
        List<Object> param = new ArrayList<>();
        param.add(QmConstants.DELETED_0);
        param.add(dto.getUserInfoId());
        if (StringUtils.isNotBlank(dto.getQuery())) {
            sql.append(" and (worker.worker_name like ? or worker.mobile like ?) ");
            param.add("%" + dto.getQuery() + "%");
            param.add("%" + dto.getQuery() + "%");
        }
        return jt.queryForBeans(TeacherListVo.class, sql.toString(), param.toArray());
    }

    public Long getOneClassesId(Integer workerId) {
        String sql = "select classes.classes_id from ycq_qm_user_classes_rel rel,ycq_qm_classes_info classes," +
                "ycq_qm_classes_type type where rel.stu_worker_id = ? and rel.deleted = ? and type.deleted = ? " +
                "and rel.classes_id = classes.classes_id and classes.classes_type_id = type.classes_type_id order by type.level_code " +
                "asc limit 1";
        List<Object> param = new ArrayList<>();
        param.add(workerId);
        param.add(QmConstants.DELETED_0);
        param.add(QmConstants.DELETED_0);
        return jt.queryForRawObject(Long.class, sql, param.toArray());
    }

    public String getOneLevelCode(Integer workerId) {
        String sql = "select type.level_code from ycq_qm_user_classes_rel rel,ycq_qm_classes_info classes," +
                "ycq_qm_classes_type type where rel.stu_worker_id = ? and rel.deleted = '0' and type.deleted = '0' " +
                "and classes.classes_status = '1' and rel.classes_id = classes.classes_id and classes.classes_type_id = " +
                "type.classes_type_id order by type.level_code asc limit 1";
        List<Object> param = new ArrayList<>();
        param.add(workerId);
        return jt.queryForRawObject(String.class, sql, param.toArray());
    }

    public Long checkOrgId(String orgId) {
        String sql = "SELECT org_id FROM TSM_ORG WHERE org_id = ?";
        return jt.queryForRawObject(Long.class, sql, Long.valueOf(orgId));
    }

    public String getValueCode(Integer propId, String valueName) {
        String sql = "SELECT value_code FROM TSM_PROPERTY_VALUE WHERE state =1 AND  prop_id = ? AND value_name =?";
        return jt.queryForRawObject(String.class, sql, propId, valueName);
    }

    public YcqQmClassesInfo getClassesId(String classesCode) {
        String sql = "select classes_id from ycq_qm_classes_info where classes_code = ? ";
        return jt.queryForBean(YcqQmClassesInfo.class, sql, classesCode);
    }

    public Long getAreaId(String orgName) {
        String sql = "SELECT org_id FROM TSM_ORG WHERE org_name = ?";
        return jt.queryForRawObject(Long.class, sql, orgName);
    }

    public Long getRelId(Long beforeClassesId, Integer stuWorkerId) {
        String sql = "select rel.rel_id from ycq_qm_classes_info info, ycq_qm_user_classes_rel rel where info.classes_id = rel.classes_id " +
                "and rel.classes_id = ? and stu_worker_id = ? and rel.deleted = ?";
        List<Object> param = new ArrayList<>();
        param.add(beforeClassesId);
        param.add(stuWorkerId);
        param.add(QmConstants.DELETED_0);
        return jt.queryForRawObject(Long.class, sql, param.toArray());
    }

    public String checkPhone(String phone) {
        String sql = "SELECT B.mobile FROM YCQ_QM_USER_INFO A LEFT JOIN TSM_WORKER B ON A.worker_id=B.login_id WHERE B.mobile=?";
        return jt.queryForRawObject(String.class, sql, phone);
    }
}
