package com.crunii.micro.service.qm.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.CodeValueVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.MyStuBaseDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.MyStuBaseVo;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/9/19 11:15
 */
@Repository
public class TeacherAppDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    public PageRsp<MyStuBaseVo> pageAuditMyStu(PageReq<MyStuBaseDto> req, Worker worker) {
        StringBuilder sb = new StringBuilder("SELECT C.worker_id ,B.worker_name,B.mobile,C.area_id,O.org_name as area_name,C.worker_job,C.political_outlook,C.education,F.classes_year,F.type_name,B.head_portrait_pic_key AS headUrl\n" +
                "FROM YCQ_QM_TEACHER_STU A LEFT JOIN TSM_WORKER B ON A.stu_worker_id = B.login_id\n" +
                " LEFT JOIN YCQ_QM_USER_INFO C ON A.stu_worker_id = C.worker_id\n" +
                " LEFT JOIN TSM_ORG O ON C.area_id = O.org_id\n" +
                " LEFT JOIN （SELECT A.stu_worker_id,B.classes_year,MIN(C.level_code), C.type_name,C.level_code FROM YCQ_QM_USER_CLASSES_REL A \n" +
                " LEFT JOIN YCQ_QM_CLASSES_INFO B ON A.classes_id =B.classes_id LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id GROUP BY stu_worker_id,type_name,level_code,classes_year, A.deleted\n" +
                " HAVING C.type_name = （SELECT max(C.type_name) FROM YCQ_QM_USER_CLASSES_REL D\n" +
                " LEFT JOIN YCQ_QM_CLASSES_INFO B ON D.classes_id =B.classes_id  LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id WHERE D.stu_worker_id =  A.stu_worker_id AND A.deleted=0）\n" +
                "）F ON A.stu_worker_id  = F.stu_worker_id WHERE  A.teach_worker_id = :workerId  AND C.deleted = 0  AND A.deleted = 0 and A.binding = 1 and C.audit_status = 1 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_QM_TEACHER_STU A LEFT JOIN TSM_WORKER B ON A.stu_worker_id = B.login_id\n" +
                " LEFT JOIN YCQ_QM_USER_INFO C ON A.stu_worker_id = C.worker_id\n" +
                " LEFT JOIN TSM_ORG O ON C.area_id = O.org_id\n" +
                " LEFT JOIN （SELECT A.stu_worker_id,B.classes_year,MIN(C.level_code), C.type_name,C.level_code FROM YCQ_QM_USER_CLASSES_REL A \n" +
                " LEFT JOIN YCQ_QM_CLASSES_INFO B ON A.classes_id =B.classes_id LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id GROUP BY stu_worker_id,type_name,level_code,classes_year, A.deleted\n" +
                " HAVING C.type_name = （SELECT max(C.type_name) FROM YCQ_QM_USER_CLASSES_REL D\n" +
                " LEFT JOIN YCQ_QM_CLASSES_INFO B ON D.classes_id =B.classes_id  LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id WHERE D.stu_worker_id =  A.stu_worker_id AND A.deleted=0）\n" +
                "）F ON A.stu_worker_id  = F.stu_worker_id WHERE  A.teach_worker_id = :workerId  AND C.deleted = 0  AND A.deleted = 0 and A.binding = 1  and C.audit_status = 1 ");
        Map<String,Object> params = new HashMap<>(16);
        params.put("workerId",worker.getLoginId());
        MyStuBaseDto query = req.getQuery();
        if(query.getCorrelationState().equals(QmConstants.USER_CORRELATION_STATE_1)){
            sb.append("AND correlation_state in(:correlationState,2) and binding = 1");
            sbCount.append("AND correlation_state in(:correlationState,2) and binding = 1");
        }else {
            sb.append("AND correlation_state = :correlationState ");
            sbCount.append("AND correlation_state = :correlationState ");
        }
        params.put("correlationState",query.getCorrelationState());
        if(StringUtils.hasText(query.getName())){
            sb.append("and ( F.type_name like concat('%',:name,'%') or B.worker_name like concat('%',:name,'%')) ");
            sbCount.append("and ( F.type_name like concat('%',:name,'%') or B.worker_name like concat('%',:name,'%')) ");
            params.put("name",query.getName());
        }

        if(!CollectionUtils.isEmpty(query.getEducation())){
            sb.append("and C.education in (:education) ");
            sbCount.append("and C.education in (:education) ");
            params.put("education",query.getEducation());
        }

        if(!CollectionUtils.isEmpty(query.getPoliticalOutlook())){
            sb.append("and C.political_outlook in (:outLook) ");
            sbCount.append("and C.political_outlook in (:outLook) ");
            params.put("outLook",query.getPoliticalOutlook());
        }

        if(!CollectionUtils.isEmpty(query.getClassesYear())){
            sb.append("and E.classes_year in (:classesYear) ");
            sbCount.append("and E.classes_year in (:classesYear) ");
            params.put("classesYear",query.getClassesYear());
        }

        if(!CollectionUtils.isEmpty(query.getLevelCode())){
            sb.append("and F.level_code in (:levelCode) ");
            sbCount.append("and F.level_code in (:levelCode)");
            params.put("levelCode",query.getLevelCode());
        }
        if(Objects.nonNull(query.getAreaId())){
            sb.append("and C.area_id = :areaId ");
            sbCount.append("and C.area_id = :areaId");
            params.put("areaId",query.getAreaId());
        }

        int total = baseNpJdbcTemplate.queryForInt(sbCount.toString(), params);
        List<MyStuBaseVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by A.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = baseNpJdbcTemplate.queryForBeans(MyStuBaseVo.class, sb.toString(), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<MyStuBaseVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public Long findIsAudit(Integer userWorkerId, int teacherWorkerId) {
        String sql = "SELECT tutor_trainee_id FROM YCQ_QM_TEACHER_STU WHERE teach_worker_id = ? AND stu_worker_id = ? AND correlation_state = 0 AND deleted = 0";
        return jdbcTemplate.queryForRawObject(Long.class,sql,teacherWorkerId,userWorkerId);
    }


    public Long findClassesId(Integer workerId) {
        String sql = "SELECT classes_id FROM YCQ_QM_USER_CLASSES_REL WHERE stu_worker_id = ? AND deleted = 0  ORDER BY update_time LIMIT 1";
        return jdbcTemplate.queryForRawObject(Long.class,sql,workerId);
    }

    public void updateState(Integer workerId, int loginId) {
        String sql = "UPDATE YCQ_QM_TEACHER_STU SET correlation_state = 2 WHERE stu_worker_id = ? AND teach_worker_id = ? and binding = 1 AND deleted = 0";
        jdbcTemplate.update(sql,workerId,loginId);
    }

    public void delMyStuBinding(Integer workerId, int loginId) {
        String sql = "UPDATE YCQ_QM_TEACHER_STU SET binding = 0 WHERE teach_worker_id = ? AND stu_worker_id = ? AND deleted = 0";
        jdbcTemplate.update(sql,loginId,workerId);
    }

    public PageRsp<MyStuBaseVo> pageNoTeacherStu(PageReq<MyStuBaseDto> req,String levelCode) {
        StringBuilder sb = new StringBuilder("SELECT B.area_id,B.worker_id ,O.org_name as areaName,C.head_portrait_pic_key AS headUrl,C.worker_name ,C.mobile ,B.worker_job ,B.political_outlook ,B.education,F.classes_year,F.type_name,C.head_portrait_pic_key headUrl FROM YCQ_QM_USER_INFO B  \n" +
                "LEFT JOIN TSM_WORKER C ON B.worker_id =C.login_id\n" +
                "LEFT JOIN TSM_ORG O ON B.area_id =O.org_id\n" +
                "LEFT JOIN (SELECT A.stu_worker_id,MIN(C.level_code), type_name,MAX(B.classes_year) classes_year,C.level_code FROM YCQ_QM_USER_CLASSES_REL A\n" +
                "LEFT JOIN YCQ_QM_CLASSES_INFO B ON A.classes_id =B.classes_id\n" +
                "LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id GROUP BY stu_worker_id,type_name,level_code,A.deleted\n" +
                "HAVING C.type_name = （SELECT max(C.type_name) FROM YCQ_QM_USER_CLASSES_REL D\n" +
                "LEFT JOIN YCQ_QM_CLASSES_INFO B ON D.classes_id =B.classes_id \n" +
                "LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id WHERE D.stu_worker_id =  A.stu_worker_id AND A.deleted=0）) F ON B.worker_id =F.stu_worker_id\n" +
                "WHERE user_identity = 1  and B.audit_status = 1  AND B.deleted = 0 AND worker_id NOT IN (SELECT stu_worker_id FROM YCQ_QM_TEACHER_STU WHERE (correlation_state <> 3) AND binding = 1 AND deleted = 0) and F.level_code in (:levelCode) ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_QM_USER_INFO B  \n" +
                "LEFT JOIN TSM_WORKER C ON B.worker_id =C.login_id\n" +
                "LEFT JOIN TSM_ORG O ON B.area_id =O.org_id\n" +
                "LEFT JOIN (SELECT A.stu_worker_id,MIN(C.level_code), type_name,MAX(B.classes_year) classes_year,C.level_code FROM YCQ_QM_USER_CLASSES_REL A\n" +
                "LEFT JOIN YCQ_QM_CLASSES_INFO B ON A.classes_id =B.classes_id\n" +
                "LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id GROUP BY stu_worker_id,type_name,level_code,A.deleted\n" +
                "HAVING C.type_name = （SELECT max(C.type_name) FROM YCQ_QM_USER_CLASSES_REL D\n" +
                "LEFT JOIN YCQ_QM_CLASSES_INFO B ON D.classes_id =B.classes_id \n" +
                "LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id WHERE D.stu_worker_id =  A.stu_worker_id AND A.deleted=0）) F ON B.worker_id =F.stu_worker_id\n" +
                "WHERE user_identity = 1 and B.audit_status = 1 AND B.deleted = 0 AND worker_id NOT IN (SELECT stu_worker_id FROM YCQ_QM_TEACHER_STU WHERE (correlation_state <> 3) AND binding = 1 AND deleted = 0) and F.level_code in (:levelCode)  ");
        Map<String,Object> params = new HashMap<>(16);
        MyStuBaseDto query = req.getQuery();
        params.put("levelCode",levelCode);
        if(StringUtils.hasText(query.getName())){
            sb.append("and ( F.type_name like concat('%',:name,'%') or C.worker_name like concat('%',:name,'%')) ");
            sbCount.append("and ( F.type_name like concat('%',:name,'%') or C.worker_name like concat('%',:name,'%')) ");
            params.put("name",query.getName());
        }

        if(!CollectionUtils.isEmpty(query.getEducation())){
            sb.append("and B.education in (:edu) ");
            sbCount.append("and B.education in (:edu)");
            params.put("edu",query.getEducation());
        }

        if(!CollectionUtils.isEmpty(query.getPoliticalOutlook())){
            sb.append("and B.political_outlook in (:outLook) ");
            sbCount.append("and B.political_outlook in (:outLook) ");
            params.put("outLook",query.getPoliticalOutlook());
        }

        if(!CollectionUtils.isEmpty(query.getClassesYear())){
            sb.append("and F.classes_year in (:classesYear) ");
            sbCount.append("and F.classes_year in (:classesYear) ");
            params.put("classesYear",query.getClassesYear());
        }

        if(Objects.nonNull(query.getAreaId())){
            sb.append("and B.area_id = :areaId ");
            sbCount.append("and B.area_id = :areaId");
            params.put("areaId",query.getAreaId());
        }

        int total = baseNpJdbcTemplate.queryForInt(sbCount.toString(), params);
        List<MyStuBaseVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by B.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = baseNpJdbcTemplate.queryForBeans(MyStuBaseVo.class, sb.toString(), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<MyStuBaseVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public void delTeacherReject(List<Integer> workerIds) {
        String sql = "UPDATE YCQ_QM_TEACHER_STU_REJECT SET deleted = 1  WHERE stu_worker_id IN (:ids)";
        Map<String, Object> params = new HashMap<>(4);
        params.put("ids", workerIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public Integer findStuCount(int loginId) {
        String sql = " SELECT count(1) FROM YCQ_QM_TEACHER_STU WHERE teach_worker_id = ? AND correlation_state in(1,2) AND binding = 1 AND deleted = 0";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,loginId);
    }

    public Long findStuClassesId(Integer workerId) {
        String sql = "SELECT A.classes_id  FROM YCQ_QM_USER_CLASSES_REL A LEFT JOIN YCQ_QM_CLASSES_INFO B ON A.classes_id  =B.classes_id LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id WHERE stu_worker_id = ? ORDER BY level_name LIMIT 1";
        return jdbcTemplate.queryForRawObject(Long.class,sql,workerId);
    }

    public List<CodeValueVo> findLevelType() {
        String sql = "SELECT level_code as code,level_name AS value FROM YCQ_QM_CLASSES_TYPE WHERE deleted = 0 GROUP BY level_code,level_name";
        return jdbcTemplate.queryForBeans(CodeValueVo.class,sql);
    }

    public List<String> findClassesYear() {
        String sql = "SELECT DISTINCT(classes_year) FROM YCQ_QM_CLASSES_INFO WHERE deleted = 0 ORDER BY classes_year DESC LIMIT 5";
        return jdbcTemplate.queryForRawObjects(String.class,sql);
    }

    public String findTeacherLevelCode(int loginId) {
        String sql = "SELECT level_code FROM YCQ_QM_TEACHER_EXPAND WHERE worker_id = ? AND deleted = 0";
        return jdbcTemplate.queryForRawObject(String.class,sql,loginId);
    }

    public Long findIsAudit(Integer userWorkerId) {
        String sql = "SELECT tutor_trainee_id FROM YCQ_QM_TEACHER_STU WHERE stu_worker_id = ? AND correlation_state <> 3 and binding =1 AND deleted = 0";
        return jdbcTemplate.queryForRawObject(Long.class,sql,userWorkerId);
    }
}
