package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.WorkerBookVo;
import com.crunii.micro.service.qm.api.domain.dto.WorkerHonorVo;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.LearIntegral;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.SearchNumVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.MyStuBaseVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.*;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;
import sun.security.pkcs11.P11Util;

import javax.annotation.Resource;
import java.util.*;

import static com.crunii.micro.service.qm.api.constants.QmConstants.SCHOOL_LEVEL;

/**
 * @author wangchen create 2023/9/18 11:24
 */
@Repository
public class TeacherManageDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate baseNpJdbcTemplate;

    @Resource
    private TsmOrgRepo tsmOrgRepo;


    public PageRsp<TeacherBaseVo> pageTeacher(PageReq<TeacherDto> pageReq, List<Long> orgIdList) {
        StringBuilder sb = new StringBuilder("SELECT A.id,A.user_code,A.worker_id,B.worker_name,B.mobile,A.org_id,c.org_name,a.area_id,d.org_name AS areaName,A.worker_job,A.create_time,A.update_time FROM YCQ_QM_USER_INFO A LEFT JOIN TSM_WORKER B ON A.worker_id=B.login_id LEFT JOIN TSM_ORG C ON a.org_id=c.org_id LEFT JOIN TSM_ORG D ON a.area_id=D.org_id WHERE A.deleted=0 AND A.user_identity=2 AND A.audit_status=:auditState ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_QM_USER_INFO A LEFT JOIN TSM_WORKER B ON A.worker_id=B.login_id LEFT JOIN TSM_ORG C ON a.org_id=c.org_id LEFT JOIN TSM_ORG D ON a.area_id=D.org_id WHERE A.deleted=0 AND A.user_identity=2 AND A.audit_status=:auditState ");
        Map<String,Object> params = new HashMap<>(16);
        TeacherDto query = pageReq.getQuery();
        params.put("auditState",query.getAuditStatus());
        if (CollectionUtil.isNotEmpty(orgIdList)) {
            sb.append(" and A.org_id in (:orgId) ");
            sbCount.append(" and A.org_id in (:orgId) ");
            params.put("orgId",query.getOrgId());
        }
        if (Objects.nonNull(query.getSex())) {
            sb.append(" and A.sex = :sex ");
            sbCount.append(" and A.sex = :sex ");
            params.put("sex",query.getSex());
        }
        if (StringUtils.hasText(query.getWorkerJob())) {
            sb.append(" and A.worker_job like concat('%',:workerJob,'%') ");
            sbCount.append(" and A.worker_job like concat('%',:workerJob,'%') ");
            params.put("workerJob",query.getWorkerJob());
        }
        if (Objects.nonNull(query.getEducation())) {
            sb.append(" and A.education = :education ");
            sbCount.append(" and A.education = :education ");
            params.put("education",query.getEducation());
        }
        if (StringUtils.hasText(query.getUserCode())) {
            sb.append(" and A.user_code like concat('%',:userCode,'%') ");
            sbCount.append(" and A.user_code like concat('%',:userCode,'%') ");
            params.put("userCode",query.getUserCode());
        }

        if (StringUtils.hasText(query.getWorkerName())) {
            sb.append(" and B.worker_name like concat('%',:workerName,'%') ");
            sbCount.append(" and B.worker_name like concat('%',:workerName,'%') ");
            params.put("workerName",query.getWorkerName());
        }

        if (StringUtils.hasText(query.getMobile())) {
            sb.append(" and B.mobile = :mobile ");
            sbCount.append(" and B.mobile = :mobile ");
            params.put("mobile",query.getMobile());
        }

        if (Objects.nonNull(query.getStartTime())) {
            sb.append(" and A.create_time >= :startTime");
            sbCount.append(" and A.create_time >= :startTime");
            params.put("startTime",query.getStartTime());
        }

        if (Objects.nonNull(query.getEndTime())) {
            sb.append(" and A.create_time <= :endTime");
            sbCount.append(" and A.create_time <= :endTime");
            params.put("endTime",query.getEndTime());
        }

        int total = baseNpJdbcTemplate.queryForInt(sbCount.toString(), params);
        List<TeacherBaseVo> values;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by A.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = baseNpJdbcTemplate.queryForBeans(TeacherBaseVo.class, sb.toString(), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<TeacherBaseVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public PageRsp<TeacherBaseVo> pageTeacherNew(PageReq<TeacherDto> pageReq) {
        StringBuilder sb = new StringBuilder("SELECT A.id,A.user_code,A.worker_id,B.worker_name,B.mobile,A.org_id,c.org_name,a.area_id,d.org_name AS areaName,A.worker_job,A.create_time,A.update_time FROM YCQ_QM_USER_INFO A LEFT JOIN TSM_WORKER B ON A.worker_id=B.login_id LEFT JOIN TSM_ORG C ON a.org_id=c.org_id LEFT JOIN TSM_ORG D ON a.area_id=D.org_id WHERE A.deleted=0 AND A.user_identity=2 AND A.audit_status=:auditStatus ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_QM_USER_INFO A LEFT JOIN TSM_WORKER B ON A.worker_id=B.login_id LEFT JOIN TSM_ORG C ON a.org_id=c.org_id LEFT JOIN TSM_ORG D ON a.area_id=D.org_id WHERE A.deleted=0 AND A.user_identity=2 AND A.audit_status=:auditStatus ");
        Map<String, Object> params = new HashMap<>(6);
        TeacherDto query = pageReq.getQuery();
        params.put("auditStatus",query.getAuditStatus());
        //if (StringUtils.hasText(query.getOrgId())) {
        //    sb.append(" and A.org_id = ? ");
        //    sbCount.append(" and A.org_id = ? ");
        //    params.add(query.getOrgId());
        //}
        if (Objects.nonNull(query.getSex())) {
            sb.append(" and A.sex = :sex ");
            sbCount.append(" and A.sex = :sex ");
            params.put("sex",query.getSex());
        }
        if (StringUtils.hasText(query.getWorkerJob())) {
            sb.append(" and A.worker_job = :job ");
            sbCount.append(" and A.worker_job = :job ");
            params.put("job",query.getWorkerJob());
        }
        if (Objects.nonNull(query.getEducation())) {
            sb.append(" and A.education = :education ");
            sbCount.append(" and A.education = :education ");
            params.put("education",query.getEducation());
        }
        if (StringUtils.hasText(query.getUserCode())) {
            sb.append(" and A.user_code like concat('%',:code,'%') ");
            sbCount.append(" and A.user_code like concat('%',:code,'%') ");
            params.put("code",query.getUserCode());
        }

        if (StringUtils.hasText(query.getWorkerName())) {
            sb.append(" and B.worker_name like concat('%',:name,'%') ");
            sbCount.append(" and B.worker_name like concat('%',:name,'%') ");
            params.put("name",query.getWorkerName());
        }

        if (StringUtils.hasText(query.getMobile())) {
            sb.append(" and B.mobile = :mobile ");
            sbCount.append(" and B.mobile = :mobile ");
            params.put("mobile",query.getMobile());
        }

        if (Objects.nonNull(query.getStartTime())) {
            sb.append(" and A.create_time >= :startTime");
            sbCount.append(" and A.create_time >= :startTime");
            params.put("startTime",query.getStartTime());
        }

        if (Objects.nonNull(query.getEndTime())) {
            sb.append(" and A.create_time <= :endTime");
            sbCount.append(" and A.create_time <= :endTime");
            params.put("endTime",query.getEndTime());
        }

        int total = baseNpJdbcTemplate.queryForInt(sbCount.toString(), params);
        List<TeacherBaseVo> values;
        String dbType = baseNpJdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by A.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = baseNpJdbcTemplate.queryForBeans(TeacherBaseVo.class, sb.toString(), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<TeacherBaseVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<String> orgList(String orgId){
        TsmOrg org = tsmOrgRepo.findOrgById(orgId);
        if(SCHOOL_LEVEL.equals(org.getOrgLevel())){
            // 查询以下组织
            return getOrgChildren(orgId);
        }
        return Collections.singletonList(orgId);
    }

    public List<String> getOrgChildren(String orgId){
        String sql = "SELECT a.org_id FROM tsm_org a LEFT JOIN tsm_worker b ON a.manage_people=b.login_id WHERE a.parent_org_id=?";
        return jdbcTemplate.queryForRawObjects(String.class,sql,orgId);
    }

    public List<TeacherExpandDto> findExpandStatus(List<Integer> workerIdList) {
        String sql = "SELECT worker_id,expand_status FROM YCQ_QM_TEACHER_EXPAND WHERE worker_id in(:ids) and deleted = 0";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", workerIdList);
        return baseNpJdbcTemplate.queryForBeans(TeacherExpandDto.class, sql, params);
    }

    public TeacherBaseVo findBaseTeacher(Integer workerId) {
        String sql = "SELECT A.id,B.worker_name,B.mobile,A.card_id,A.sex,A.org_id,d.org_name AS orgName,A.area_id,E.org_name AS areaName,A.education,A.political_outlook,A.worker_job,A.work_unit,A.user_code FROM YCQ_QM_USER_INFO A LEFT JOIN TSM_WORKER B ON A.worker_id=B.login_id LEFT JOIN YCQ_QM_USER_HONOR C ON A.worker_id=c.worker_id LEFT JOIN TSM_ORG D ON a.org_id=d.org_id LEFT JOIN TSM_ORG E ON a.area_id=E.org_id WHERE A.deleted=0 AND A.worker_id=? LIMIT 1";
        return jdbcTemplate.queryForBean(TeacherBaseVo.class, sql, workerId);
    }

    public List<WorkerHonorVo> findWorkerHonor(Integer workerId) {
        String sql = "SELECT acquisition_time,honor_desc FROM YCQ_QM_USER_HONOR WHERE honor_type = 1 and deleted = 0 AND worker_id = ? order by create_time desc";
        return jdbcTemplate.queryForBeans(WorkerHonorVo.class, sql, workerId);
    }

    public String findAuditNote(Long userInfoId) {
        String sql = "SELECT note FROM YCQ_QM_USER_AUDIT_RECORD WHERE user_info_id = ? ORDER BY audit_time DESC limit 1";
        return jdbcTemplate.queryForRawObject(String.class, sql, userInfoId);
    }

    public TeacherExpandDto findTeacherExpand(Integer workerId) {
        String sql = "SELECT A.file_key ,B.level_code,B.level_name FROM YCQ_QM_TEACHER_EXPAND A LEFT JOIN YCQ_QM_CLASSES_TYPE B ON A.level_code =B.level_code WHERE A.deleted = 0 AND A.worker_id = ? limit 1";
        return jdbcTemplate.queryForBean(TeacherExpandDto.class, sql, workerId);
    }

    public List<Integer> findStuByTeacherIds(List<Integer> workerIds) {
        String sql = "SELECT teach_worker_id,count(1) stuCount FROM YCQ_QM_TUTOR_TRAINEE WHERE correlation_state = 1 AND teacher_status = 1 AND deleted = 0 AND  teach_worker_id in(:ids) GROUP BY teach_worker_id";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", workerIds);
        return baseNpJdbcTemplate.queryForRawObjects(Integer.class, sql, params);
    }

    public List<String> findWorkerNames(List<Integer> workerIdList) {
        String sql = "SELECT worker_name FROM TSM_WORKER WHERE login_id in(:ids)";
        Map<String, Object> params = new HashMap<>(2);
        params.put("ids", workerIdList);
        return baseNpJdbcTemplate.queryForRawObjects(String.class, sql, params);
    }

    public void delTeacherByWorkerIds(List<Integer> workerIds, Date nowTime, int loginId) {
        String sql = "UPDATE YCQ_QM_USER_INFO SET deleted = 1,update_time = :nowTime,updater_id = :workerId WHERE worker_id IN(:ids) AND user_identity = 2";
        Map<String, Object> params = new HashMap<>(6);
        params.put("ids", workerIds);
        params.put("nowTime", nowTime);
        params.put("workerId", loginId);
        baseNpJdbcTemplate.update(sql, params);
    }

    public void delTeacherExpandByWorkerIds(List<Integer> workerIds, Date nowTime, int loginId) {
        String sql = "UPDATE YCQ_QM_TEACHER_EXPAND SET deleted = 1,update_time = :nowTime,updater_id = :workerId WHERE worker_id IN(:ids)";
        Map<String, Object> params = new HashMap<>(6);
        params.put("ids", workerIds);
        params.put("nowTime", nowTime);
        params.put("workerId", loginId);
        baseNpJdbcTemplate.update(sql, params);
    }

    public void delTutorTraineeByWorkerIds(List<Integer> workerIds, Date nowTime, int loginId) {
        String sql = "UPDATE YCQ_QM_TUTOR_TRAINEE SET deleted = 1,update_time = :nowTime,updater_id = :workerId WHERE teach_worker_id IN(:ids)";
        Map<String, Object> params = new HashMap<>(6);
        params.put("ids", workerIds);
        params.put("nowTime", nowTime);
        params.put("workerId", loginId);
        baseNpJdbcTemplate.update(sql, params);
    }

    public List<ExportTeacher> findExportTeacherByIds(List<Integer> ids) {
        StringBuilder sb = new StringBuilder("SELECT A.user_code,C.worker_name,C.mobile,A.card_id,CASE WHEN A.sex=0 THEN '女' ELSE '男' END AS sex,d.org_name,A.education,A.political_outlook,A.worker_job,A.work_unit,e.level_name FROM YCQ_QM_USER_INFO A LEFT JOIN YCQ_QM_TEACHER_EXPAND B ON A.worker_id=B.worker_id LEFT JOIN TSM_WORKER C ON A.worker_id=C.login_id LEFT JOIN TSM_ORG D ON a.org_id=d.org_id LEFT JOIN (\n" +
                "SELECT level_code,level_name FROM YCQ_QM_CLASSES_TYPE WHERE deleted=0 GROUP BY level_code,level_name) E ON b.level_code=e.level_code WHERE A.deleted=0 AND B.deleted=0 ");
        Map<String, Object> params = new HashMap<>(6);
        if (CollectionUtil.isNotEmpty(ids)){
            sb.append("AND A.worker_id IN (:workerIds)");
            params.put("workerIds", ids);
        }
        return baseNpJdbcTemplate.queryForBeans(ExportTeacher.class, sb.toString(), params);
    }


    public List<ExportTeacher> findExportTeacher(TeacherDto query) {
        StringBuilder sb = new StringBuilder("SELECT A.user_code,C.worker_name,C.mobile,A.card_id,CASE WHEN A.sex=0 THEN '女' ELSE '男' END AS sex,d.org_name,A.education,A.political_outlook,A.worker_job,A.work_unit,e.level_name FROM YCQ_QM_USER_INFO A LEFT JOIN YCQ_QM_TEACHER_EXPAND B ON A.worker_id=B.worker_id LEFT JOIN TSM_WORKER C ON A.worker_id=C.login_id LEFT JOIN TSM_ORG D ON a.org_id=d.org_id LEFT JOIN (\n" +
                "SELECT level_code,level_name FROM YCQ_QM_CLASSES_TYPE WHERE deleted=0 GROUP BY level_code,level_name) E ON b.level_code=e.level_code WHERE A.deleted=0 AND B.deleted=0");
        List<Object> params = new ArrayList<>();
        if (Objects.nonNull(query.getOrgId())) {
            sb.append(" and A.org_id = ? ");
            params.add(query.getOrgId());
        }
        if (Objects.nonNull(query.getSex())) {
            sb.append(" and A.sex = ? ");
            params.add(query.getSex());
        }
        if (StringUtils.hasText(query.getWorkerJob())) {
            sb.append(" and A.worker_job = ? ");
            params.add(query.getWorkerJob());
        }
        if (Objects.nonNull(query.getEducation())) {
            sb.append(" and A.education = ? ");
            params.add(query.getEducation());
        }
        if (StringUtils.hasText(query.getUserCode())) {
            sb.append(" and A.user_code like concat('%',?,'%') ");
            params.add(query.getEducation());
        }

        if (StringUtils.hasText(query.getWorkerName())) {
            sb.append(" and C.worker_name like concat('%',?,'%') ");
            params.add(query.getWorkerName());
        }

        if (StringUtils.hasText(query.getMobile())) {
            sb.append(" and C.mobile = ? ");
            params.add(query.getMobile());
        }

        if (Objects.nonNull(query.getStartTime())) {
            sb.append(" and A.create_time >= ?");
            params.add(query.getStartTime());
        }

        if (Objects.nonNull(query.getEndTime())) {
            sb.append(" and A.create_time <= ?");
            params.add(query.getEndTime());
        }
        return jdbcTemplate.queryForBeans(ExportTeacher.class, sb.toString(), params.toArray());
    }


    public Integer findLoginIdByMobile(String mobile) {
        String sql = "SELECT login_id FROM TSM_WORKER WHERE mobile = ?";
        return jdbcTemplate.queryForRawObject(Integer.class, sql, mobile);
    }

    public Integer findQmTeacherExist(Integer loginId) {
        String sql = "SELECT count(1) FROM YCQ_QM_USER_INFO WHERE worker_id = ? AND deleted = 0";
        return jdbcTemplate.queryForRawObject(Integer.class, sql, loginId);
    }

    public List<TsNumVo> queryTsNum(List<Integer> workerIdList) {
        String sql = "SELECT teach_worker_id,COUNT(1) AS stuNumber FROM YCQ_QM_TEACHER_STU a LEFT JOIN YCQ_QM_USER_INFO b ON a.stu_worker_id=b.worker_id WHERE a.deleted=0 AND b.deleted=0 AND binding=1 AND user_identity=1  AND b.audit_status =1 AND correlation_state IN (1,2) AND teach_worker_id IN (:teachWorkerIds) GROUP BY teach_worker_id";
        Map<String, Object> params = new HashMap<>(2);
        params.put("teachWorkerIds", workerIdList);
        return baseNpJdbcTemplate.queryForBeans(TsNumVo.class, sql, params);
    }

    public List<StudentDetailVo> getStudentInfo(Integer workerId) {
        String sql = "SELECT b.login_id AS workerId,B.worker_name,B.mobile,C.org_id,D.worker_job,D.political_outlook,C.classes_year,e.org_name,F.file_key \n" +
                "FROM YCQ_QM_TEACHER_STU A LEFT JOIN TSM_WORKER B ON A.stu_worker_id = B.login_id\n" +
                "LEFT JOIN YCQ_QM_USER_INFO D ON A.stu_worker_id = D.worker_id\n" +
                "LEFT JOIN TSM_ORG E ON d.area_id = e.org_id\n" +
                "LEFT JOIN YCQ_QM_CLASSES_INFO C ON C.classes_id = A.stu_classes_id\n" +
                "LEFT JOIN YCQ_QM_STU_EXPAND F ON A.stu_worker_id  = F.worker_id\n" +
                "WHERE A.teach_worker_id = ? AND (correlation_state = 1 OR correlation_state = 2)\n" +
                "AND A.binding = 1 AND A.deleted = 0 AND d.deleted = 0 AND d.audit_status = 1";
        return jdbcTemplate.queryForBeans(StudentDetailVo.class, sql, workerId);
    }

    public String findTeacherAudit(Integer workerId) {
        String sql = "SELECT honor_desc FROM YCQ_QM_USER_HONOR WHERE honor_type = 2 and deleted = 0 AND worker_id = ? order by create_time desc limit 1";
        return jdbcTemplate.queryForRawObject(String.class, sql, workerId);
    }

    public String  getValueCode(Integer propId, String valueName) {
        String sql = "SELECT value_code FROM TSM_PROPERTY_VALUE WHERE state =1 AND  prop_id = ? AND value_name =?";
        return jdbcTemplate.queryForRawObject(String.class, sql, propId, valueName);
    }

    public Long getAreaId(String orgName) {
        String sql = "SELECT org_id FROM TSM_ORG WHERE parent_org_id=500000 AND org_name=?";
        return jdbcTemplate.queryForRawObject(Long.class, sql,orgName);
    }

    public Long getOrgId(Long orgId) {
        String sql = "SELECT org_id FROM TSM_ORG WHERE org_id=?";
        return jdbcTemplate.queryForRawObject(Long.class, sql,orgId);
    }



    public LearIntegral findIntegralByWorkerId(int loginId) {
        String sql = "SELECT integral,self_cultivation_id FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ? AND deleted = 0 AND cultivation_type =1 limit 1";
        return jdbcTemplate.queryForBean(LearIntegral.class, sql, loginId);
    }

    public List<WorkerBookVo> findBookInfoByWorkerId(int loginId) {
        String sql = "SELECT self_cultivation_id,book_name,experience,complete_time AS createTime FROM YCQ_QM_SELF_CULTIVATION WHERE worker_id = ? AND cultivation_type = 2 AND deleted = 0 order by createTime desc";
        return jdbcTemplate.queryForBeans(WorkerBookVo.class, sql, loginId);
    }

    public PageRsp<StudentVo> pageNoTeacherStu(PageReq<Integer> req,String levelCode) {
        StringBuilder sb = new StringBuilder("SELECT B.login_id as stuWorkerId ,B.worker_name  FROM YCQ_QM_USER_INFO A \n" +
                "LEFT JOIN TSM_WORKER B ON A.worker_id =B.login_id \n" +
                "LEFT JOIN (SELECT A.stu_worker_id,MIN(C.level_code), type_name,MAX(B.classes_year) classes_year,C.level_code FROM YCQ_QM_USER_CLASSES_REL A\n" +
                "LEFT JOIN YCQ_QM_CLASSES_INFO B ON A.classes_id =B.classes_id\n" +
                " LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id GROUP BY stu_worker_id,type_name,level_code,A.deleted\n" +
                " HAVING C.type_name = （SELECT max(C.type_name) FROM YCQ_QM_USER_CLASSES_REL D\n" +
                "LEFT JOIN YCQ_QM_CLASSES_INFO B ON D.classes_id =B.classes_id \n" +
                "LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id WHERE D.stu_worker_id =  A.stu_worker_id AND A.deleted=0)) F\n" +
                "ON A.worker_id =F.stu_worker_id WHERE A.user_identity = 1 AND  A.deleted = 0 AND A.audit_status = 1 AND  F.level_code = ? " +
                "AND B.login_id NOT IN (SELECT stu_worker_id FROM YCQ_QM_TEACHER_STU WHERE correlation_state IN(1,2) AND binding = 1 AND deleted = 0)");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_QM_USER_INFO A \n" +
                "LEFT JOIN TSM_WORKER B ON A.worker_id =B.login_id \n" +
                "LEFT JOIN (SELECT A.stu_worker_id,MIN(C.level_code), type_name,MAX(B.classes_year) classes_year,C.level_code FROM YCQ_QM_USER_CLASSES_REL A\n" +
                "LEFT JOIN YCQ_QM_CLASSES_INFO B ON A.classes_id =B.classes_id\n" +
                " LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id GROUP BY stu_worker_id,type_name,level_code,A.deleted\n" +
                " HAVING C.type_name = （SELECT max(C.type_name) FROM YCQ_QM_USER_CLASSES_REL D\n" +
                "LEFT JOIN YCQ_QM_CLASSES_INFO B ON D.classes_id =B.classes_id \n" +
                "LEFT JOIN YCQ_QM_CLASSES_TYPE C ON B.classes_type_id =C.classes_type_id WHERE D.stu_worker_id =  A.stu_worker_id AND A.deleted=0)) F\n" +
                "ON A.worker_id =F.stu_worker_id WHERE A.user_identity = 1 AND  A.deleted = 0 AND A.audit_status = 1 AND F.level_code = ? " +
                "AND B.login_id NOT IN (SELECT stu_worker_id FROM YCQ_QM_TEACHER_STU WHERE correlation_state IN(1,2) AND binding = 1 AND deleted = 0)");
        List<Object> params = new ArrayList<>();
        params.add(levelCode);
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<StudentVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by A.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(StudentVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<StudentVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<StudentVo> queryStudent(Integer workerId){
        String sql ="SELECT a.stu_worker_id,b.worker_name FROM YCQ_QM_TEACHER_STU a LEFT JOIN YCQ_QM_USER_INFO c ON a.stu_worker_id = c.worker_id LEFT JOIN TSM_WORKER B ON a.stu_worker_id=b.login_id WHERE a.deleted=0 AND c.deleted =0 AND binding=1 and user_identity = 1 AND correlation_state IN (1,2) AND teach_worker_id=?";
        return jdbcTemplate.queryForBeans(StudentVo.class,sql,workerId);
    }

    public List<Integer> queryStudentIds(Integer workerId){
        String sql ="SELECT a.stu_worker_id FROM YCQ_QM_TEACHER_STU a LEFT JOIN YCQ_QM_USER_INFO c ON a.stu_worker_id = c.worker_id LEFT JOIN TSM_WORKER B ON a.stu_worker_id=b.login_id WHERE a.deleted=0 AND c.deleted =0 AND binding=1 and user_identity = 1 AND correlation_state IN (1,2) AND teach_worker_id=?";
        return jdbcTemplate.queryForRawObjects(Integer.class,sql,workerId);
    }

    public SearchNumVo searchNum(List<Long> orgIds) {
        String sql = "SELECT count(1) FROM ycq_qm_user_info a LEFT JOIN TSM_ORG b ON a.org_id=b.org_id WHERE a.user_identity=2 AND a.audit_status= :auditState AND a.deleted= :deleted ";

        Map<String,Object> params = new HashMap<>(4);
        params.put("auditState",QmConstants.AUDIT_STATUS_WAIT);
        params.put("deleted",QmConstants.AUDIT_STATUS_NO);
        if(CollectionUtil.isNotEmpty(orgIds)){
            sql += " AND b.org_id in(:orgIds)";
            params.put("orgIds",orgIds);
        }
        int waitExamine = baseNpJdbcTemplate.queryForInt(sql,params);
        params.put("auditState",QmConstants.AUDIT_STATUS_NO);
        int notPassExamine = baseNpJdbcTemplate.queryForInt(sql,params);
        params.put("auditState",QmConstants.AUDIT_STATUS_YES);
        int passExamine = baseNpJdbcTemplate.queryForInt(sql,params);

        SearchNumVo vo = new SearchNumVo();
        vo.setWaitExamine(waitExamine);
        vo.setNotPassExamine(notPassExamine);
        vo.setPassExamine(passExamine);
        return vo;
    }

    public void delTeacherReject(List<Integer> workerIds) {
        String sql = "UPDATE YCQ_QM_TEACHER_STU_REJECT SET deleted = 1  WHERE teach_worker_id IN (:ids)";
        Map<String, Object> params = new HashMap<>(4);
        params.put("ids", workerIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public String findTeacherLevelCode(int loginId) {
        String sql = "SELECT level_code FROM YCQ_QM_TEACHER_EXPAND WHERE worker_id = ? AND deleted = 0";
        return jdbcTemplate.queryForRawObject(String.class,sql,loginId);
    }

    public List<String> queryGraduationFileKeys(List<Integer> workerIds){
        String sql = "SELECT file_key FROM YCQ_QM_STU_EXPAND WHERE deleted =0 AND worker_id IN (:workerIds)";
        Map<String, Object> params = new HashMap<>(2);
        params.put("workerIds", workerIds);
        return baseNpJdbcTemplate.queryForRawObjects(String.class,sql,params);
    }

    public void delTeacherBindingStu(List<Integer> needDelStuIds, Integer teacherId, Worker worker) {
        String sql = "UPDATE YCQ_QM_TEACHER_STU SET deleted = 1,update_time=:updateTime,updater_id = :updateId WHERE teach_worker_id = :teacherWorkerId and correlation_state in(0,3) AND stu_worker_id IN(:stuWorkerIds) AND deleted = 0";
        Map<String, Object> params = new HashMap<>(8);
        params.put("updateTime", new Date());
        params.put("updateId", worker.getLoginId());
        params.put("teacherWorkerId", teacherId);
        params.put("stuWorkerIds",needDelStuIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public void updateTeacherBindingStu(List<Integer> needDelStuIds, Integer workerId, Worker worker) {
        String sql = "UPDATE YCQ_QM_TEACHER_STU SET binding = 0 ,update_time=:updateTime,updater_id = :updateId WHERE teach_worker_id = :teachWorkerId AND  stu_worker_id in(:stuWorkerIds) and binding = 1 AND deleted = 0 AND correlation_state in(1,2)";
        Map<String, Object> params = new HashMap<>(8);
        params.put("updateTime", new Date());
        params.put("updateId", worker.getLoginId());
        params.put("teachWorkerId", workerId);
        params.put("stuWorkerIds",needDelStuIds);
        baseNpJdbcTemplate.update(sql,params);
    }

    public String findLevelCodeByOrgId(Long orgId) {
        String sql = "SELECT org_level FROM TSM_ORG WHERE org_id = ? ";
        return jdbcTemplate.queryForRawObject(String.class,sql,orgId);
    }

    public List<Long> findSchoolOrgByOrgId(Long orgId) {
        String sql = "SELECT org_id FROM TSM_ORG START WITH org_id=? CONNECT BY PRIOR  org_id=parent_org_id";
        return jdbcTemplate.queryForRawObjects(Long.class,sql,orgId);
    }

    public List<Integer> findAllStuByTeacherWorkerIds(List<Integer> workerIds) {
        String sql = "SELECT a.stu_worker_id FROM YCQ_QM_TEACHER_STU a LEFT JOIN YCQ_QM_USER_INFO b ON a.stu_worker_id = b.worker_id WHERE b.user_identity=1 AND b.audit_status=1 AND b.deleted = 0 AND a.correlation_state in(1,2) AND a.binding=1 AND a.deleted = 0 AND a.teach_worker_id in(:ids)";
        Map<String, Object> params = new HashMap<>(8);
        params.put("ids", workerIds);
        return baseNpJdbcTemplate.queryForRawObjects(Integer.class,sql,params);
    }

    public String checkPhone(String phone){
        String sql ="SELECT B.mobile FROM YCQ_QM_USER_INFO A LEFT JOIN TSM_WORKER B ON A.worker_id=B.login_id WHERE A.deleted=0 AND B.mobile=?";
        return jdbcTemplate.queryForRawObject(String.class,sql,phone);
    }
}
