package com.crunii.micro.service.qm.jdbc;


import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.qm.api.domain.dto.teachingRoute.TeachingRouteDto;
import com.crunii.micro.service.qm.api.domain.dto.teachingRoute.TeachingRouteNodeVo;
import com.crunii.micro.service.qm.api.domain.dto.teachingRoute.TeachingRouteVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author denggl
 * date 2023/9/18
 */
@Repository
public class TeachingRouteDao {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;


    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String pageSearchRouteSql=" select route_id,route_code,route_name,route_theme,route_briefing,route_status,create_time from YCQ_QM_TEACHING_ROUTE where deleted=0";
    private static final String countPageSearchRouteSql="select count(*) from YCQ_QM_TEACHING_ROUTE where deleted=0";
    private static final String listNodeByRouteIdSql=" select route_node_id,start_time,end_time,content from  YCQ_QM_TEACHING_ROUTE_NODE  where teaching_route_id=? and deleted=0 order by start_time ";
    private static final String selectCountByCreateId=" select count(*) from YCQ_QM_TEACHING_ROUTE where creator_id=? and TO_CHAR(create_time, 'YYYY-MM-DD') = TO_CHAR(SYSDATE, 'YYYY-MM-DD')";
    public PageRsp<TeachingRouteVo> pageSearchRoute(PageReq<TeachingRouteDto> req) {
        StringBuilder sql =new StringBuilder(pageSearchRouteSql);
        StringBuilder countSql=new StringBuilder(countPageSearchRouteSql);
        Map<String, Object> params = new HashMap<>(20);
         if (ObjectUtil.isNotEmpty(req.getQuery().getRouteCode())){
             sql.append(" and route_code =:routeCode ");
             countSql.append(" and route_code =:routeCode");
             params.put("routeCode",req.getQuery().getRouteCode());
         }

         if (ObjectUtil.isNotEmpty(req.getQuery().getRouteTheme())){
             sql.append(" and route_theme in (:routeTheme)");
             countSql.append(" and route_theme in (:routeTheme) ");
             params.put("routeTheme",req.getQuery().getRouteTheme());
         }


         if ((ObjectUtil.isNotEmpty(req.getQuery().getRouteName()))){
             sql.append(" and route_name like '%"+req.getQuery().getRouteName()+"%' ");
             countSql.append(" and route_name like '%"+req.getQuery().getRouteName()+"%'");

         }

         if (ObjectUtil.isNotEmpty(req.getQuery().getStartTime())){
             sql.append(" and create_time>=:startTime ");
             countSql.append(" and create_time>=:startTime ");
             params.put("startTime",req.getQuery().getStartTime());

         }

         if (ObjectUtil.isNotEmpty(req.getQuery().getEndTime())){
             sql.append(" and create_time<=:endTime ");
             countSql.append(" and create_time<=:endTime ");
             params.put("endTime",req.getQuery().getEndTime());
         }

        if (ObjectUtil.isNotEmpty(req.getQuery().getRouteStatus())){
            sql.append(" and route_status=:routeStatus ");
            countSql.append(" and route_status=:routeStatus ");
            params.put("routeStatus",req.getQuery().getRouteStatus());
        }
        int total = NPJT.queryForInt(countSql.toString(), params);
         List<TeachingRouteVo> result=new ArrayList<>();
        if (total>0){
            String dbType = NPJT.getDbType();
            SqlUtil.appendLimit(dbType, params, sql, req);
            sql.append(" order by create_time desc ");
            result = NPJT.queryForBeans(TeachingRouteVo.class, sql.toString(), params);
        }
        PageRsp<TeachingRouteVo> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        rsp.setTotalNum(total);
        rsp.setResult(result);
        rsp.next();
        return rsp;
    }


    public List<TeachingRouteNodeVo>  listNodeByRouteId(Long id){
        return jt.queryForBeans(TeachingRouteNodeVo.class,listNodeByRouteIdSql,id);
    }


    public int selectCountByCreateId(long workerId){
        return jt.queryForInt(selectCountByCreateId,workerId);
    }
}
