package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.YcqObsFile;
import com.crunii.micro.domain.entity.main.YcqQmTrainingTrainLabel;
import com.crunii.micro.service.qm.api.domain.dto.trainingTrain.TrainLabel;
import com.crunii.micro.service.qm.api.domain.dto.trainingTrain.TrainingTrainClockVo;
import com.crunii.micro.service.qm.api.domain.dto.trainingTrain.TrainingTrainDto;
import com.crunii.micro.service.qm.api.domain.dto.trainingTrain.TrainingTrainVo;
import org.springframework.jdbc.core.ParameterizedPreparedStatementSetter;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.*;

/**
 * @author denggl
 * date 2023/9/19
 */
@Repository
public class TrainingTrainDao {


    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;


    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String pageTrainSql = " select base_id,train_code,train_status,train_name,train_region,train_category,train_label,create_time from YCQ_QM_TRAINING_TRAIN where deleted=0 ";
    private static final String countpageTrainSql = " select count(*) from YCQ_QM_TRAINING_TRAIN where deleted=0 ";
    private static final String selectClockRecord = " SELECT i.user_identity, w.worker_name as clockName,w.mobile as phone,t.clock_state as satus,w.head_portrait_pic_key as headSculpture,t.clock_id,t.clock_time,t.woker_id   FROM " +
            " YCQ_QM_TRAINING_TRAIN_CLOCK t  LEFT JOIN TSM_WORKER w  ON  t.woker_id =w.login_id   LEFT JOIN YCQ_QM_USER_INFO  i  ON i.worker_id= t.woker_id  WHERE t.train_id=?  order by t.clock_time desc";
    private static final String selectCountByCreateId = " select count(*) from YCQ_QM_TRAINING_TRAIN where  creator_id=? and TO_CHAR(create_time, 'YYYY-MM-DD') = TO_CHAR(SYSDATE, 'YYYY-MM-DD') ";
    private  static final String updateTrainPicState=" update ycq_obs_file set state=0 where object_key=? and  service_code=? and service_id=?";
    private static final String getOrgSql="  SELECT org_id ,org_name  FROM tsm_org WHERE  parent_org_id=500000";
    private static final String getPicSql =" select service_id,object_key,create_time from ycq_obs_file where  service_id in (:serviceId) and state=1 and service_code=:serviceCode order by create_time";
    private static final String findByTrainIdsAndDeletedSql=" select label_id,label_value,train_id as baseId from YCQ_QM_TRAINING_TRAIN_LABEL where deleted=:deleted and train_id in (:trId)";
    public PageRsp<TrainingTrainVo> pageTrain(PageReq<TrainingTrainDto> ycqQmTrainingTrainDto) {
        StringBuilder sql = new StringBuilder(pageTrainSql);
        StringBuilder countSql = new StringBuilder(countpageTrainSql);
        Map<String, Object> params = new HashMap<>(20);
        //区域
        if ((ObjectUtil.isNotEmpty(ycqQmTrainingTrainDto.getQuery().getTrainRegion()))) {
            sql.append(" and train_region=:trainRegion ");
            countSql.append(" and train_region=:trainRegion ");
            params.put("trainRegion", ycqQmTrainingTrainDto.getQuery().getTrainRegion());
        }
        //标签
        if (ObjectUtil.isNotEmpty(ycqQmTrainingTrainDto.getQuery().getTrainLabel())) {

            sql.append(" and base_id in ( select DISTINCT(train_id) from YCQ_QM_TRAINING_TRAIN_LABEL where label_value in (:trainLabel) and deleted=0 ) ");
            countSql.append(" and base_id in ( select DISTINCT(train_id) from YCQ_QM_TRAINING_TRAIN_LABEL where label_value in (:trainLabel) and deleted=0)");
            params.put("trainLabel", ycqQmTrainingTrainDto.getQuery().getTrainLabel());
        }
        //类别

        if (ObjectUtil.isNotEmpty(ycqQmTrainingTrainDto.getQuery().getTrainCategory())) {
            sql.append(" and train_category in (:trainCategory)");
            countSql.append(" and train_category in (:trainCategory) ");
            params.put("trainCategory", ycqQmTrainingTrainDto.getQuery().getTrainCategory());
        }
        //状态

        if (ObjectUtil.isNotEmpty(ycqQmTrainingTrainDto.getQuery().getTrainStatus())) {
            sql.append(" and train_status=:trainStatus ");
            countSql.append(" and train_status=:trainStatus");
            params.put("trainStatus", ycqQmTrainingTrainDto.getQuery().getTrainStatus());
        }
        //名称
        if (ObjectUtil.isNotEmpty(ycqQmTrainingTrainDto.getQuery().getTrainName())) {
            sql.append(" and train_name like '%"+ycqQmTrainingTrainDto.getQuery().getTrainName()+"%' ");
            countSql.append(" and train_name like'%"+ycqQmTrainingTrainDto.getQuery().getTrainName()+"%' ");

        }
        //编号

        if (ObjectUtil.isNotEmpty(ycqQmTrainingTrainDto.getQuery().getTrainCode())) {
            sql.append(" and train_code=:trainCode ");
            countSql.append(" and train_code=:trainCode ");
            params.put("trainCode", ycqQmTrainingTrainDto.getQuery().getTrainCode());
        }
        if (ObjectUtil.isNotEmpty(ycqQmTrainingTrainDto.getQuery().getStartTime())) {
            sql.append(" and create_time>=:statrTime ");
            countSql.append(" and create_time>=:statrTime");
            params.put("statrTime", ycqQmTrainingTrainDto.getQuery().getStartTime());

        }

        if (ObjectUtil.isNotEmpty(ycqQmTrainingTrainDto.getQuery().getEndTime())) {
            sql.append(" and create_time<= :endTime");
            countSql.append(" and create_time<= :endTime ");
            params.put("endTime", ycqQmTrainingTrainDto.getQuery().getEndTime());
        }
        int total = NPJT.queryForInt(countSql.toString(), params);
        List<TrainingTrainVo> result = new ArrayList<>();
        if (total > 0) {
            String dbType = jt.getDbType();
            SqlUtil.appendLimit(dbType, params, sql, ycqQmTrainingTrainDto);
            sql.append(" order by create_time desc ");
            result = NPJT.queryForBeans(TrainingTrainVo.class, sql.toString(), params);
        }
        PageRsp<TrainingTrainVo> rsp = new PageRsp<>();
        rsp.setPageNo(ycqQmTrainingTrainDto.getPageNo());
        rsp.setPageSize(ycqQmTrainingTrainDto.getPageSize());
        rsp.setTotalNum(total);
        rsp.setResult(result);
        rsp.next();
        return rsp;
    }

    public List<TrainingTrainClockVo> selectClockRecord(Long trId, Worker worker) {
        return jt.queryForBeans(TrainingTrainClockVo.class, selectClockRecord, trId);
    }

    public int selectCountByCreateId(long workerId) {
        return jt.queryForInt(selectCountByCreateId, workerId);
    }


    public void updateTrainPicState(List<String> listKey,Long serviceId){
        int[][] rowCounts = jt.batchUpdate(updateTrainPicState, listKey, 5, new ParameterizedPreparedStatementSetter<String>() {
            @Override
            public void setValues(PreparedStatement ps, String argument) throws SQLException {
                ps.setString(1,argument);
                ps.setString(2, CommonConstants.OBS_FILE_PXJD);
                ps.setLong(3,serviceId);
            }

        });

    }

    public List<TsmOrg> getOrg(){
        return jt.queryForBeans(TsmOrg.class,getOrgSql);
    }

    public List<YcqObsFile> getPic(List<Long> trId){
        Map<String, Object> params = new HashMap<>(20);
        params.put("serviceId",trId);
        params.put("serviceCode",CommonConstants.OBS_FILE_PXJD);
        return NPJT.queryForBeans(YcqObsFile.class,getPicSql,params);
    }

    public List<TrainLabel> findByTrainIdsAndDeleted(List<Long> trId, int deleted) {
        Map<String, Object> params = new HashMap<>(20);
        params.put("deleted",deleted);
        params.put("trId",trId);
        return NPJT.queryForBeans(TrainLabel.class,findByTrainIdsAndDeletedSql,params);

    }
}
