package com.crunii.micro.service.qm.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.entity.main.YcqQmUserAuditRecord;
import com.crunii.micro.domain.entity.main.YcqQmUserInfo;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.org.OrgTreeReqDto;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameDto;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameVo;
import com.crunii.micro.service.qm.api.domain.dto.org.vo.OrgTreeVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.CheckUserInfoVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.ClassesLevelVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.ClassesTypeVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.HomePageClassesVo;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author xiayinqiang
 * @date 2023/9/18 0018 15:15:33
 */
@Repository
public class UserInfoDao {
	@Resource(name = "mainJT")
	private BaseJdbcTemplate jt;

	/**
	 * 根据用户查询用户信息
	 * @param loginId 用户id
	 * @return CheckUserInfoVo
	 */
	public CheckUserInfoVo selectUserInfoByWorkerId(int loginId) {
		String sql = " SELECT a.*,o.org_name,CASE WHEN o1.parent_org_id = 0 OR a.area_id IS null then o1.org_name ELSE '重庆市/'||o1.org_name END AS areaName "
				+ "FROM YCQ_QM_USER_INFO a left JOIN TSM_ORG o ON a.org_id = o.org_id AND o.state = 1 "
				+ "left JOIN TSM_ORG o1 ON a.area_id = o1.org_id AND o1.state = 1 WHERE a.deleted = 0  AND a.worker_id = ? ";
		return jt.queryForBean(CheckUserInfoVo.class, sql, loginId);
	}

	/**
	 * 根据青马用户id查询最新的审核记录
	 * @param id 青马用户id
	 * @return YcqQmUserAuditRecord
	 */
	public YcqQmUserAuditRecord selectAuditRecordByUserId(Long id) {
		String sql = " SELECT * FROM YCQ_QM_USER_AUDIT_RECORD a WHERE a.user_info_id = ? ORDER BY a.audit_time desc LIMIT 1 ";
		return jt.queryForBean(YcqQmUserAuditRecord.class, sql, id);
	}


	/**
	 * 查询需要展示的班级信息
	 * @param workerId 人员id
	 * @return List<HomePageClassesVo>
	 */
	public List<HomePageClassesVo> selectClassesByWorkerId(Integer workerId) {
		String sql = "SELECT b.*,c.level_name,c.type_name FROM YCQ_QM_USER_CLASSES_REL a INNER JOIN YCQ_QM_CLASSES_INFO b ON a.classes_id = b.classes_id AND a.deleted = 0 AND b.deleted = 0 "
				+ " INNER JOIN YCQ_QM_CLASSES_TYPE c ON b.classes_type_id = c.classes_type_id "
				+ " WHERE a.stu_worker_id = ? AND b.classes_status = 1  ";
		return jt.queryForBeans(HomePageClassesVo.class, sql, workerId);
	}

	/**
	 * 查询用户的第一个班级
	 * @param workerId 用户id
	 * @return YcqQmClassesInfo
	 */
	public String selectFirstClassesByWorkerId(Integer workerId) {
		String sql = " SELECT b.classes_year FROM YCQ_QM_USER_CLASSES_REL a INNER JOIN YCQ_QM_CLASSES_INFO B ON a.classes_id = b.classes_id AND a.deleted = 0 AND b.deleted = 0\n"
				+ "WHERE a.stu_worker_id = ? AND b.classes_status = 1 ORDER BY a.create_time ASC LIMIT 1 ";
		return jt.queryForRawObject(String.class, sql, workerId);
	}

	/**
	 * 查询所有班级包含的年份
	 * @return List<String>
	 */
	public List<String> selectClassesYearByAll() {
		String sql = " SELECT a.classes_year FROM YCQ_QM_CLASSES_INFO a where a.deleted = 0 AND a.classes_status = 1 GROUP BY a.classes_year ORDER BY a.classes_year desc ";
		return jt.queryForRawObjects(String.class, sql);
	}

	/**
	 * 根据年份查询班级的所有级别
	 *
	 * @param classesYear 班级年份
	 * @return Result<List<ClassesLevelVo>>
	 */
	public List<ClassesLevelVo> selectClassesLevelByYear(String classesYear) {
		String sql = "SELECT b.level_code,b.level_org_tier,b.level_name FROM YCQ_QM_CLASSES_INFO a INNER JOIN YCQ_QM_CLASSES_TYPE b ON a.classes_type_id = b.classes_type_id "
				+ "where a.deleted = 0 AND b.deleted = 0 AND a.classes_year = ? GROUP BY b.level_code,b.level_org_tier,b.level_name ORDER BY b.level_code ";
		return jt.queryForBeans(ClassesLevelVo.class, sql,classesYear);
	}

	/**
	 *根据年度和类型查询班级列表
	 * @param classesYear 班级年度
	 * @param levelCode     班级级别code
	 * @param orgId 班级组织id
	 * @return Result<List < ClassesVo>>
	 */
	public List<ClassesTypeVo> selectClassesTypeByYearAndLevel(String classesYear, String levelCode, Long orgId) {
		String sql = " SELECT a.classes_id,b.type_code,b.type_name FROM YCQ_QM_CLASSES_INFO a INNER JOIN YCQ_QM_CLASSES_TYPE b ON a.classes_type_id = b.classes_type_id "
				+ " where a.deleted = 0 AND b.deleted = 0 AND a.classes_status = 1 AND a.classes_year = ? AND b.level_code = ? AND a.org_id = ? ORDER BY b.type_code ";
		return jt.queryForBeans(ClassesTypeVo.class, sql, classesYear,levelCode,orgId);
	}



	/**
	 * 查询导师级别名称
	 * @param workerId 用户id
	 * @return ClassesLevelVo
	 */
	public ClassesLevelVo selectLevelByWorkerId(Integer workerId) {
		String sql = " SELECT DISTINCT b.level_code,b.level_name FROM YCQ_QM_TEACHER_EXPAND a INNER JOIN YCQ_QM_CLASSES_TYPE b ON a.level_code = b.level_code WHERE a.deleted = 0 AND b.deleted = 0 AND a.worker_id = ? ";
		return jt.queryForBean(ClassesLevelVo.class, sql,workerId);
	}

	/**
	 * 根据组织名称查询组织信息
	 * @param req 入参实体
	 * @return PageRsp<PageOrgByOrgNameVo>
	 */
	public PageRsp<PageOrgByOrgNameVo> pageOrgByOrgName(PageReq<PageOrgByOrgNameDto> req) {
		StringBuilder sb = new StringBuilder("SELECT * FROM TSM_ORG A WHERE A.state = 1 ");
		StringBuilder sbCount = new StringBuilder("SELECT count(*) FROM TSM_ORG A WHERE A.state = 1 ");
		List<Object> params = new ArrayList<>();
		if(StringUtils.hasText(req.getQuery().getOrgName().trim())){
			sb.append(" and org_name like concat('%',?,'%')");
			sbCount.append(" and org_name like concat('%',?,'%')");
			params.add(req.getQuery().getOrgName().trim());
		}else{
			throw new BusinessException("组织名称不能为空");
		}

		int total = jt.queryForInt(sbCount.toString(), params.toArray());
		List<PageOrgByOrgNameVo> ycqLeagueVos;
		String dbType = jt.getDbType();
		if (total > 0) {
			//根据数据库类型设置limit
			SqlUtil.appendLimit(dbType, params, sb, req);
			sb.append(" order by sort_id");
			ycqLeagueVos = jt.queryForBeans(PageOrgByOrgNameVo.class, sb.toString(), params.toArray());
		} else {
			ycqLeagueVos = new ArrayList<>(0);
		}
		PageRsp<PageOrgByOrgNameVo> page = new PageRsp<>();
		page.setPageNo(req.getPageNo());
		page.setPageSize(req.getPageSize());
		page.setTotalNum(total);
		page.setResult(ycqLeagueVos);
		// 为手机计算是否由下一页
		page.next();

		return page;
	}

	public List<PageOrgByOrgNameVo> findParentOrgByOrgId(Long orgId) {
		String sql = "SELECT A.org_name FROM TSM_ORG A WHERE A.org_id in(SELECT org_id FROM TSM_ORG START WITH org_id= ? CONNECT BY PRIOR parent_org_id=org_id) AND  org_id <> ? ";
		return jt.queryForBeans(PageOrgByOrgNameVo.class, sql,orgId,orgId);
	}

	public List<String> findParentOrgNameByOrgId(Long orgId) {
		String sql = "SELECT A.org_name FROM TSM_ORG A WHERE A.org_id in(SELECT org_id FROM TSM_ORG START WITH org_id= ? CONNECT BY PRIOR parent_org_id=org_id) AND  org_id <> ? ";
		return jt.queryForRawObjects(String.class, sql,orgId,orgId);
	}

	public List<OrgTreeVo> findOrgChildrenTree(OrgTreeReqDto param, String orgId) {

		List<String> orgLevelListLeaf = new ArrayList<>();
		orgLevelListLeaf.add(QmConstants.DISTRICT_LEVEL);
		orgLevelListLeaf.add(QmConstants.SCHOOL_LEVEL);
		orgLevelListLeaf.add(QmConstants.FACULTY_LEVEL);
		orgLevelListLeaf.add(QmConstants.ENTERPRISE_LEVEL);
		orgLevelListLeaf.add(QmConstants.SECONDARY_UNIT_LEVEL);

		List<String> orgLevelList = new ArrayList<>(orgLevelListLeaf);
		orgLevelList.add(QmConstants.NATIONAL_LEVEL);
		orgLevelList.add(QmConstants.CITY_LEVEL);

		StringBuilder sql = new StringBuilder("SELECT a.org_id,a.parent_org_id,a.org_code,a.parent_org_code,a.org_name,a.org_desc,a.sort_id,a.state,a.create_time,a.update_time, a.org_level,\n"
				+ " case when (SELECT COUNT(*) FROM tsm_org b WHERE b.parent_org_id=a.org_id AND b.org_level in "+orgLevelListLeaf.stream().map(i->" ? ").collect(Collectors.joining(",","(",")"))+")>0  then FALSE ELSE TRUE end isLeaf \n"
				+ " from tsm_org a WHERE a.state=1 AND a.org_level in "+orgLevelList.stream().map(i->" ? ").collect(Collectors.joining(",","(",")")));
		List<Object> argList = new ArrayList<>();

		argList.addAll(orgLevelListLeaf);
		argList.addAll(orgLevelList);


		//上级组织id
		if(ObjectUtil.isNotEmpty(param.getParentOrgId())){
			sql.append(" and a.parent_org_id=? ");
			argList.add(param.getParentOrgId());
		}else{
			sql.append(" and a.org_id=? ");
			argList.add(Long.valueOf(orgId));
		}
		//组织名称模糊
		if(ObjectUtil.isNotEmpty(param.getOrgName())){
			sql.append(" and a.org_name LIKE concat('%',?,'%') ");
			argList.add(param.getOrgName());
		}
		//组织类型
		if(ObjectUtil.isNotEmpty(param.getOrgType())){
			sql.append(" and a.org_type=? ");
			argList.add(param.getOrgType());
		}
		//组织级别
		if(ObjectUtil.isNotEmpty(param.getOrgLevel())){
			sql.append(" and a.org_level=? ");
			argList.add(param.getOrgType());
		}

		return jt.queryForBeans(OrgTreeVo.class, sql.toString(), argList.toArray());
	}

	public List<OrgTreeVo> getOrgChildrenTreeByOrgLevel(OrgTreeReqDto param) {

		List<String> orgLevelListLeaf = new ArrayList<>();
		orgLevelListLeaf.add(QmConstants.DISTRICT_LEVEL);
		orgLevelListLeaf.add(QmConstants.SCHOOL_LEVEL);
		orgLevelListLeaf.add(QmConstants.FACULTY_LEVEL);
		orgLevelListLeaf.add(QmConstants.ENTERPRISE_LEVEL);
		orgLevelListLeaf.add(QmConstants.SECONDARY_UNIT_LEVEL);

		List<String> orgLevelList = new ArrayList<>(orgLevelListLeaf);
		orgLevelList.add(QmConstants.NATIONAL_LEVEL);
		orgLevelList.add(QmConstants.CITY_LEVEL);

		StringBuilder sql = new StringBuilder("SELECT a.org_id,a.parent_org_id,a.org_code,a.parent_org_code,a.org_name,a.org_desc,a.sort_id,a.state,a.create_time,a.update_time, a.org_level,\n"
				+ " case when (SELECT COUNT(*) FROM tsm_org b WHERE b.parent_org_id=a.org_id AND b.org_level in"+orgLevelListLeaf.stream().map(i->" ? ").collect(Collectors.joining(",","(",")"))+")>0  then FALSE ELSE TRUE end isLeaf \n"
				+ " from tsm_org a WHERE a.state=1 AND a.org_level in"+orgLevelList.stream().map(i->" ? ").collect(Collectors.joining(",","(",")")));
		List<Object> argList = new ArrayList<>();


		argList.addAll(orgLevelListLeaf);
		argList.addAll(orgLevelList);

		//上级组织id
		if(ObjectUtil.isNotEmpty(param.getParentOrgId())){
			sql.append(" and a.parent_org_id=? ");
			argList.add(param.getParentOrgId());
		}else if(ObjectUtil.isNotEmpty(param.getOrgLevel())){//组织级别
			String orgLevel = param.getOrgLevel();
			if(QmConstants.FACULTY_LEVEL.equals(orgLevel)){
				//院系级查寻高校列表
				orgLevel = QmConstants.SCHOOL_LEVEL;
			}else if(QmConstants.SECONDARY_UNIT_LEVEL.equals(orgLevel)){
				orgLevel = QmConstants.ENTERPRISE_LEVEL;
			}

			sql.append(" and a.org_level=? ");
			argList.add(orgLevel);
		}else{
			throw new BusinessException("参数有误！");
		}
//		//组织名称模糊
//		if(ObjectUtil.isNotEmpty(param.getOrgName())){
//			sql.append(" and a.org_name LIKE concat('%',?,'%') ");
//			argList.add(param.getOrgName());
//		}
//		//组织类型
//		if(ObjectUtil.isNotEmpty(param.getOrgType())){
//			sql.append(" and a.org_type=? ");
//			argList.add(param.getOrgType());
//		}


		return jt.queryForBeans(OrgTreeVo.class, sql.toString(), argList.toArray());
	}

	public YcqQmUserInfo findOrgIdByClassesId(Long classesId) {
		String sql = "select org_id from ycq_qm_classes_info where classes_id = ? ";
		return jt.queryForBean(YcqQmUserInfo.class, sql, classesId);
	}
}
