package com.crunii.micro.service.qm.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.belowExcellent.*;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameVo;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.TeacherListDto;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.TeacherListVo;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * @author xiayinqiang
 * @date 2023/9/27 0027 9:56:29
 */
public interface BelowExcellentService {

	/**
	 * 根据学员id查询上级组织
	 * @param workerId 学员id
	 * @return Result<PageOrgByOrgNameVo>
	 */
	Result<PageOrgByOrgNameVo> selectParentOrgByWorkerId(Integer workerId);
	/**
	 * 分页查询下级推优列表
	 * @param req 入参实体
	 * @param worker 人员实体
	 * @return PageRsp<PageBelowExcellentVo>
	 */
	PageRsp<PageBelowExcellentVo> pageBelowExcellent(PageReq<PageBelowExcellentDto> req, Worker worker);

	/**
	 * 推优
	 *
	 * @param worker 人员实体
	 * @param dto 入参实体
	 * @return String
	 */
	String insertBelowExcellent(Worker worker, InsertBelowExcellentDto dto);
	/**
	 * 下级推优列表给学员分配班级和导师
	 * @param worker 人员实体
	 * @param dto 入参实体
	 * @return String
	 */
	String insertStuClassesAndTeach(Worker worker, InsertStuClassesAndTeachDto dto);

	/**
	 * 根据学员id查询推荐入党记录
	 * @param worker 人员实体
	 * @param workerId 学员id
	 * @return Result<List<SelectIntoPartyVo>>
	 */
	Result<List<SelectIntoPartyVo>> selectIntoPartyByWorkerId(Worker worker, Integer workerId);

	/**
	 * 新增或编辑推荐入党
	 * @param worker 人员实体
	 * @param dto 入参实体
	 * @return String
	 */
	String insertIntoParty(Worker worker, InsertIntoPartyDto dto);

	/**
	 * 删除学员推荐入党记录
	 * @param worker 人员实体
	 * @param recordId 入参实体
	 * @return String
	 */
	Result<String> deleteIntoPartyById(Worker worker, Long recordId);

	/**
	 * 导出
	 * @param dto 入参实体
	 * @param response 请求
	 * @return String
	 */
	void exportByList(ExportByListDto dto, HttpServletResponse response) throws IOException;

	/**
	 * 分页查询学员被拒绝两次列表
	 * @param req 入参实体
	 * @param worker 人员实体
	 * @return PageRsp<PageStuRefuseTwiceVo>
	 */
	PageRsp<PageStuRefuseTwiceVo> pageStuRefuseTwice(PageReq<PageStuRefuseTwiceDto> req, Worker worker);

	/**
	 * 下级推优查询导师列表
	 * @param req 入参实体
	 * @return List<TeacherListVo>
	 */
	List<TeacherListVo> teacherList(TeacherListDto req);
}
