package com.crunii.micro.service.qm.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQmUserCourse;
import com.crunii.micro.domain.entity.main.YcqQmUserInfo;
import com.crunii.micro.service.qm.api.domain.dto.appFoster.GroupCourseVO;
import com.crunii.micro.service.qm.api.domain.dto.appFoster.GroupSectionVO;
import com.crunii.micro.service.qm.api.domain.dto.appFoster.QueryEvaluateStuVO;
import com.crunii.micro.service.qm.api.domain.dto.classesMgr.*;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import springfox.documentation.annotations.ApiIgnore;

import java.text.ParseException;
import java.util.List;

/**
 * @author: LG
 * @date: 2023/9/18 19:39
 */
public interface ClassesMgrService {

    /**
     * 查询班级类型
     * @param levelCode
     * @return
     */
    Result<List<QueryClassesTypeVO>> queryClassesType(String levelCode);

    /**
     * 查询班级级别
     * @return
     */
    Result<List<QueryClassesTypeVO>> queryClassesLevelByAdd(String orgLevel);

    /**
     * 查询班级级别
     * @return
     */
    Result<List<QueryClassesTypeVO>> queryClassesLevel();

    /**
     *查询班级信息
     * @param req
     * @return
     */
    Result<PageRsp<QueryClassesVO>> queryClasses(PageReq<QueryClassesDTO> req);

    /**
     * 批量删除班级
     * @param classesId
     * @return
     */
    Result<Object> deleteClasses(List<Long> classesId);

    /**
     * 新增班级
     * @param worker
     * @param saveClassesDTO
     * @return
     */
    Result<Object> saveClasses(Worker worker, SaveClassesDTO saveClassesDTO);


    /**\
     * 新增课程
     * @param worker
     * @param saveCourseDTO
     * @return
     * @throws ParseException
     */
    Result<Object> saveCourse(Worker worker,SaveCourseDTO saveCourseDTO) throws ParseException;

    /**
     * 编辑课程
     * @param worker
     * @param saveCourseDTO
     * @return
     * @throws ParseException
     */
    Result<Object> updateCourse(Worker worker, SaveCourseDTO saveCourseDTO) throws ParseException;

    /**
     * 批量删除课程
     * @param worker
     * @param courseIds
     * @return
     */
    Result<Object>deleteCourse(Worker worker,List<Long> courseIds);

    /**
     * 查看课程
     * @param req
     * @return
     */
    Result<PageRsp<QueryCourseVO>> queryCourse(PageReq<QueryCourseDTO> req);

    /**
     * 查看课程详情
     * @param courseId
     * @return
     */
    Result<QueryCourseVO> queryCourseDetail(Long courseId);

    /**
     * 用户参与课程
     * @param worker
     * @param courseId
     * @return
     */
    Result<Object> saveParticipationCourse(Worker worker, Long courseId,Long courseSectionId);

    /**
     * 根据用户查看他的所在各个班级
     * @param worker
     * @return
     */
    Result<List<QueryUserClassesVO>> queryUserClasses(Worker worker);

    /**
     * 根据班级查所有课程
     * @param worker
     * @param classesId
     * @return
     */
    Result<List<GroupCourseVO>> queryClassesCourse(Worker worker, Long classesId);

    /**
     * 根据课程查节次
     * @param worker
     * @param courseId
     * @return
     */
    Result<List<GroupSectionVO>> queryCourseSection(Worker worker,Long courseId);

    /**
     * 组织评价学生
     * @param worker
     * @param evaluateStuDTO
     * @return
     */
    Result<Object> updateEvaluateStu(Worker worker,EvaluateStuDTO evaluateStuDTO);

    /**
     * 参看评价
     * @param worker
     * @param courseId
     * @return
     */
    Result<QueryEvaluateStuVO> queryEvaluateStu(Worker worker, Long courseId);

    /**
     * 班级启用或停用
     * @param worker
     * @param updateClassesStatusDTO
     * @return
     */
    Result<Object> updateClassesStatus(Worker worker, UpdateClassesStatusDTO updateClassesStatusDTO );

    /**
     * 新增课程学员查询
     * @param req
     * @return
     */
    Result<PageRsp<stuVO>> queryAddClassesStu(PageReq<QueryClassesStuDTO> req);

    /**
     * 查询班级信息详情
     * @param classesId
     * @return
     */
    Result<QueryClassesVO> queryClassesDetail(Long classesId);

    /**
     * 查询班级下学员
     * @param req
     * @return
     */
    Result<PageRsp<stuVO>> queryClassesStu( PageReq<QueryClassesStuDTO> req );
}
