package com.crunii.micro.service.qm.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.TeacherAppBaseVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.ExertDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.GroupCultivationDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.StuHonorDto;

import java.util.List;

/**
 * @author xiayinqiang
 * @date 2023/9/21 0021 16:13:37
 */
public interface CommonService {
	/**
	 * 保存用户与班级关联关系,一个学员多个班级
	 * @param operatorId 操作人id
	 * @param loginId 用户id集合
	 * @param classesIdList 班级id集合
	 */
	void insertUserClassesRel(int operatorId, Integer loginId, List<Long> classesIdList);

	/**
	 * 保存用户与班级关联关系，一个班级多个学员
	 * @param operatorId 操作人id
	 * @param loginIdList 用户id集合
	 * @param classesId 班级id
	 */
	void insertUserClassesRel(int operatorId, List<Integer> loginIdList, Long classesId);


	/**
	 * 查询学员 -- 荣誉
	 * @param req Integer = workerId
	 * @return data
	 */
	PageRsp<WorkerHonorVo> getStuExpand(PageReq<StuExpandDto> req);

	/**
	 * 查询学员 -- 优势
	 * @param workerId
	 * @return
	 */
	String getAdvantage(Integer workerId);

	/**
	 * 查询学员-分页查询集中培养
	 * @param pageReq 分页参数
	 * @return data
	 */
	PageRsp<ConcentratedCultivationVo> pageConcentratedCultivationExpand(PageReq<StuExpandDto> pageReq);

	/**
	 * 查询学员-导师评价
	 * @param pageReq 分页查询
	 * @return  data
	 */
	PageRsp<TeacherTraineeVo> pageStuTraineeExpand(PageReq<TrackingCultivationDto> pageReq);
	/**
	 * 新增或修改导师级别
	 * @param operatorId
	 * @param loginId
	 * @param levelCode
	 */
	void insertTeacherExpand(int operatorId, int loginId, String levelCode);


	/**
	 * 查询跟踪-自我培养
	 * @param trackingCultivationDto 参数
	 * @param worker worker
	 * @return data
	 */
	SelfCultivationVo getSelfCultivation(TrackingCultivationDto trackingCultivationDto, Worker worker);

	/**
	 * 读书分页
	 * @param pageReq 请求参数
	 * @return data
	 */
	PageRsp<WorkerBookVo> pageBookRead(PageReq<StuExpandDto> pageReq);

	/**
	 * 查询志愿服务分页
	 * @param pageReq 分页参数
	 * @return data
	 */
	PageRsp<VolunteerVo> pageVolunteer(PageReq<StuExpandDto> pageReq);

	/**
	 * 查询其他分页
	 * @param pageReq 分页参数
	 * @return data
	 */
	PageRsp<VolunteerVo> pageOther(PageReq<StuExpandDto> pageReq);

	/**
	 * 分页查询作用发挥
	 * @param req 分页参数
	 * @return data
	 */
	PageRsp<ExertVo> pageExert(PageReq<ExertDto> req);


	/**
	 * 分页查询成长发展
	 * @param req 请求参数
	 * @return data
	 */
	PageRsp<GrowthVo> pageGrowth(PageReq<GroupCultivationDto> req);

	/**
	 * 志愿服务&其他
	 * @param stuHonorDto 参数
	 * @return data
	 */
	List<VolunteerVo> getVolunteer(StuHonorDto stuHonorDto);

	/**
	 * 查询推优入党
	 * @param pageReq req
	 * @return data
	 */
	List<GrowthPartyVo> findParty(TrackingCultivationDto pageReq);

	/**
	 * 删除导师
	 * @param teacherWorkerIds teacherIds
	 */
	void delTeacherInfo(List<Integer> teacherWorkerIds,Worker worker);

	/**
	 * 删除学员
	 * @param stuWorkerIds stuWorkerIds
	 */
	void delWorkerInfo(List<Integer> stuWorkerIds,Worker worker);

	/**
	 * 查询学员历史导师 以及当前导师
	 * @param workerId worker
 	 * @return data
	 */
	List<TeacherAppBaseVo> findMyTeacher(Integer workerId);

	/**  todo 临时 更新用户feign调用？青马注册更新tsm_worker？
	 * 更新用户名和性别
	 * @param workerName 用户名
	 * @param loginId loginId
	 * @param sex sex
	 */
	void updateWorker(Integer loginId,String workerName,Integer sex);
}
