package com.crunii.micro.service.qm.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQfBank;
import com.crunii.micro.domain.entity.main.YcqQfLoanRecord;
import com.crunii.micro.service.qm.api.domain.dto.ExcelDataInfo;
import com.crunii.micro.service.qm.api.domain.dto.qf.*;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.ImportStuDataDto;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author wangchen create 2023/11/9 11:27
 */
public interface QfPlanManageService {

    /**
     * 分页查询计划申请
     * @param req 分页参数
     * @return data
     */
    PageRsp<LoanPlanDetailVo> pageLoanPlan(PageReq<LoanPlanDto> req);


    /**
     * 删除计划
     * @param loanId 主键id
     * @param worker 当前登录用户
     * @return str
     */
    String delLoanPlan(Long loanId, Worker worker);

    /**
     * 导出计划申请
     * @param req 请求
     */
    void exportLoanPlan(LoanPlanDto req);

    Result<ExcelDataInfo> importLoanPlanVerify(MultipartFile file);

    Result<String> importLoanPlan(ImportStuDataDto dto);

    LoanPlanSimpleVo getLoanStatistics();

    /**
     * 通过父银行查询下级
     * @param parentBankId 父银行id
     * @return data
     */
    List<YcqQfBankVo> queryBank(Long parentBankId);

    Result<ExcelDataInfo> saveByImport(MultipartFile file);

    /**
     * 添加银行
     * @param req 请求
     * @return str
     */
    String insertBank(List<YcqQfBank> req);

    /**
     * 根据id导出
     * @param loanIds ids
     */
    void exportLoanPlanByIds(List<Long> loanIds);

}
