package com.crunii.micro.service.qm.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;

import java.util.List;

/**
 * @author wangchen create 2023/9/20 10:26
 */
public interface StudentAppService {

    /**
     * 验证当前学员是否绑定导师
     * @param loginId 学员id
     * @return true 已绑定
     */
    Boolean verifyIsBindTeacher(int loginId);

    /**
     * 分页查询老师基本信息
     * @param pageReq 分页参数
     * @param worker 当前用户
     * @return data
     */
    PageRsp<TeacherAppBaseVo> pageTeacherBase(PageReq<TeacherAppBaseDto> pageReq, Worker worker);

    /**
     * 查询我的导师  包括历史导师
     * @param worker 当前用户
     * @return data
     */
    List<TeacherAppBaseVo> findMyTeacher(Worker worker);

    /**
     * 查询审核流程
     * @param worker 当前用户
     * @param workerHonorDto 请求参数
     * @return data
     */

    /**
     * 录入自我培养
     * @param selfCultivationDto 参数
     * @param worker worker
     * @return str
     */
    String saveSelfCultivation(SelfCultivationDto selfCultivationDto, Worker worker);

    /**
     * 录入作用发挥
     * @param exertCreateDto 参数
     * @param worker worker
     * @return str
     */
    String saveExert(ExertCreateDto exertCreateDto, Worker worker);

    /**
     * 录入人才举荐
     * @param recommendCreateDto 参数
     * @param worker worker
     * @return str
     */
    String saveRecommend(RecommendCreateDto recommendCreateDto, Worker worker);

    /**
     * 获取优势
     * @param worker worker
     * @return data
     */
    String getAdvantage(Worker worker);

    /**
     * 自我培养
     * @param trackingCultivationDto 分页参数
     * @param worker worker
     * @return data
     */
    SelfCultivationVo getSelfCultivation(TrackingCultivationDto trackingCultivationDto, Worker worker);


    /**
     * 成长发展分页
     * @param pageReq req
     * @param worker worker
     * @return data
     */
    PageRsp<GrowthVo> pageSelfGrowth(PageReq<TrackingCultivationDto> pageReq, Worker worker);

    /**
     * 作用发挥
     * @param trackingCultivationDto 分页参数
     * @param worker worker
     * @return data
     */
    PageRsp<ExertVo> pageExert(PageReq<TrackingCultivationDto> trackingCultivationDto, Worker worker);

    /**
     * 导师评价
     * @param pageReq req
     * @param worker worker
     * @return data
     */
    PageRsp<TeacherTraineeVo> studentAppService(PageReq<TrackingCultivationDto> pageReq, Worker worker);

    /**
     * 志愿服务 && 其他
     * @param worker 参数
     * @return data
     */
    List<VolunteerVo> getVolunteer(Worker worker);

    /**
     * 删除自我培养
     * @param selfCultivationId 自我培养id
     * @param worker worker
     * @return str
     */
    String delSelfCultivation(Long selfCultivationId, Worker worker);

    /**
     * 删除作用发挥
     * @param exertId id
     * @param worker worker
     * @return data
     */
    String delExert(Long exertId, Worker worker);

    /**
     * 删除成长发展
     * @param groupId id
     * @param worker worker
     * @return str
     */
    String delSelfGrowth(Long groupId, Worker worker);

    /**
     * 获取导师荣誉 & 擅长领域
     * @param loginId 当前用户id
     * @return data
     */
    HonorFieldVo getTeacherExpand(int loginId);

    /**
     * 申请导师
     * @param teacherId 导师id
     * @param stuId  学员
     * @return str
     */
    String applyTeacher(Integer teacherId, Worker stuId);

    /**
     * 修改学习强哥积分
     * @param learIntegral 参数
     * @param worker worker
     * @return str
     */
    String modifyLearIntegral(LearIntegral learIntegral, Worker worker);

    /**
     * 查询推优入党
     * @param worker 当前用户
     * @return data
     */
    List<GrowthPartyVo> findParty(Worker worker);

    /**
     * 查询当前用户申请状态
     * @param worker worker
     * @return data
     */
    StudentAuditVo findStuAuditState(Worker worker);

}
