package com.crunii.micro.service.qm.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.ViewTeacherDto;
import com.crunii.micro.service.qm.api.domain.dto.ExcelDataInfo;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.TeacherAppBaseVo;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ConcentratedCultivationVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.StuExpandDto;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author baoxy
 * @date 2023-09-18
 */
public interface StudentManageWebService {

    /**
     * 分页查询待审核,审核不通过,审核通过列表
     *
     * @param req 分页参数
     * @return PageRsp<PassExamineVo>
     */
    PageRsp<PassExamineVo> pageExamine(PageReq<PassExamineDto> req, Worker worker);

    /**
     * 查看学员基本信息
     *
     * @param dto 请求参数
     * @return ViewStudentVo
     */
    ViewStudentVo viewStudent(ViewStudentDto dto, Worker worker);

    /**
     * 创建学员
     *
     * @param dto    请求参数
     * @param worker 当前用户
     * @return data
     */
    Result<Object> createStudent(CreateStudentDto dto, Worker worker);

    /**
     * 编辑学员
     *
     * @param dto    请求参数
     * @param worker 当前用户
     * @return data
     */
    Result<Object> updateStudent(UpdateStudentDto dto, Worker worker);

    /**
     * 批量删除学员
     *
     * @param dtoList 删除学员列表
     * @param worker  当前用户
     * @return data
     */
    Result<Object> batchDelStudent(List<DelStudentDto> dtoList, Worker worker);

    /**
     * 批量审核
     *
     * @param dtoList 审核列表
     * @param worker  当前用户
     * @return data
     */
    Result<Object> batchExamine(List<BatchExamineDto> dtoList, Worker worker);

    /**
     * 查询待审核,审核不通过,审核通过,下级推优的数量
     *
     * @param dto    请求参数
     * @param worker
     * @return SearchNumVo
     */
    SearchNumVo searchNum(SearchNumDto dto, Worker worker);

    /**
     * 模板下载
     *
     * @param response 响应
     */
    void downLoadTemplate(HttpServletResponse response);

    /**
     * 校验学生的Excel文件
     *
     * @param file 文件参数
     * @return data
     */
    Result<ExcelDataInfo> verifyStuExcel(MultipartFile file);

    /**
     * Excel导入学员数据
     *
     * @param dto    数据key
     * @param worker 当前用户
     * @return data
     */
    Result<String> importStuData(ImportStuDataDto dto, Worker worker);

    /**
     * 导出审核通过列表
     *
     * @param dto 请求参数
     */
    void exportPassExamine(ExportExamineDto dto);

    /**
     * 查看学员-导师信息
     *
     * @param dto 请求参数
     * @return List<ViewTeacherVo>
     */
    List<TeacherAppBaseVo> viewTeacher(ViewTeacherDto dto);

    /**
     * 查看学员-集中培养
     *
     * @param req 请求参数
     * @return PageRsp<ConcentratedCultivationVo>
     */
    PageRsp<ConcentratedCultivationVo> concentrateCulture(PageReq<StuExpandDto> req);

    /**
     * 导师设置
     *
     * @param dto    请求参数
     * @param worker 当前用户
     * @return data
     */
    Result<Object> teacherSet(TeacherSetDto dto, Worker worker);

    /**
     * 导师设置-查询导师列表
     *
     * @param dto 请求参数
     * @return List<TeacherListVo>
     */
    List<TeacherListVo> teacherList(TeacherListDto dto);

    /**
     * 导师设置-已绑定的导师列表
     *
     * @param dto 请求参数
     * @return List<TeacherListVo>
     */
    List<TeacherListVo> bindingTeacherList(TeacherBindingListDto dto);

    /**
     * 分配班级
     *
     * @param dto    请求参数
     * @param worker 当前用户
     * @return data
     */
    Result<Object> shareClasses(ShareClassesDto dto, Worker worker);

    /**
     * 验证是否更改学员
     *
     * @param dto 请求参数
     * @return Boolean类型
     */
    Boolean validUpdateStudent(UpdateStudentDto dto, Worker worker);

    /**
     * 上传结业证书
     *
     * @param dto    请求参数
     * @param worker 当前用户
     * @return data
     */
    Result<Object> uploadCertificate(UploadCertificateDto dto, Worker worker);
}
