package com.crunii.micro.service.qm.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;

import java.util.List;

/**
 * @author wangchen create 2023/9/19 11:15
 */
public interface TeacherAppService {

    /**
     * 分页查询我的学生
     * @param req 分页参数
     * @param worker worker
     * @return data
     */
    PageRsp<MyStuBaseVo> pageAuditMyStu(PageReq<MyStuBaseDto> req,Worker worker);


    /**
     * 分页查询荣誉
     * @param pageReq 请求参数
     * @return data
     */
    PageRsp<WorkerHonorVo> pageHonor(PageReq<StuExpandDto> pageReq);

    /**
     * 获取学员优势
     * @param stuExpandDto 请求参数
     * @return str
     */
    String getAdvantage(StuExpandDto stuExpandDto);

    /**
     * 更新用户申请信息
     * @param stuAuditDto 请求参数
     * @param worker 当前用户
     * @return str
     */
    String updateStuAudit(StuAuditDto stuAuditDto, Worker worker);

    /**
     * 评价学员
     * @param evaluateStuDto 评价参数
     * @param worker worker
     * @return str
     */
    String addEvaluateStu(EvaluateStuDto evaluateStuDto, Worker worker);

    /**
     * 移除学员
     * @param workerId 用户id
     * @param worker 当前登录用户
     * @return str
     */
    String delMyStu(Integer workerId,Worker worker);

    /**
     * 分页查询集中培养
     * @param pageReq 分页参数
     * @return data
     */
    PageRsp<ConcentratedCultivationVo> pageConcentratedCultivationExpand(PageReq<StuExpandDto> pageReq);

    /**
     * 分页查询导师评价
     * @param pageReq 分页参数
     * @param worker worker
     * @return data
     */
    PageRsp<TeacherTraineeVo> pageStuTraineeExpand(PageReq<TrackingCultivationDto> pageReq,Worker worker);

    /**
     * 我的学员页-获取学员扩展信息-跟踪培养
     * @param trackingCultivationDto 参数
     * @param worker worker
     * @return data
     */
    SelfCultivationVo getSelfCultivation(TrackingCultivationDto trackingCultivationDto, Worker worker);


    /**
     * 作用发展
     * @param trackingCultivationDto 请求参数
     * @param worker worker
     * @return data
     */
    PageRsp<ExertVo> pageExert(PageReq<TrackingCultivationDto> trackingCultivationDto, Worker worker);

    /**
     * 成长发展
     * @param trackingCultivationDto 参数
     * @param worker worker
     * @return data
     */
    PageRsp<GrowthVo> pageGrowth(PageReq<TrackingCultivationDto> trackingCultivationDto, Worker worker);

    /**
     * 分页查询没有绑定导师的学员
     * @param req req
     * @param worker worker
     * @return data
     */
    PageRsp<MyStuBaseVo> pageNoTeacherStu(PageReq<MyStuBaseDto> req,Worker worker);

    /**
     * 获取志愿服务和其他
     * @param stuExpandDto 参数
     * @return data
     */
    List<VolunteerVo> getVolunteer(StuExpandDto stuExpandDto);

    /**
     * 添加学员
     * @param workerIds workerIds
     * @param worker worker
     * @return str
     */
    String addStuList(List<Integer> workerIds,Worker worker);

    /**
     * 查询班级级别
     * @return data
     */
    List<CodeValueVo> findLevelType();

    /**
     * 查询班级年度
     * @return classersYear
     */
    List<String> findClassesYear();

    /**
     * 查询推优入党
     * @param pageReq 请求参数
     * @param worker worker
     * @return data
     */
    List<GrowthPartyVo> findParty(TrackingCultivationDto pageReq, Worker worker);
}
