package com.crunii.micro.service.qm.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.SearchNumVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.EvaluateStuDto;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author wangchen create 2023/9/18 11:22
 */
public interface TeacherManageService {

    /**
     * 分页查询导师
     * @param pageReq 分页参数
     * @return data
     */
    PageRsp<TeacherBaseVo> pageTeacher(PageReq<TeacherDto> pageReq);

    /**
     * 获取导师信息
     * @param workerId 导师id
     * @return data
     */
    TeacherDetailVo getTeacherInfo(Integer workerId);

    /**
     * 获取学员信息
     * @param workerId 用户id
     * @return data
     */
    Result<List<StudentDetailVo>> getStudentInfo(Integer workerId);

    /**
     * 审核用户
     * @param auditTeacherDto 审核参数
     * @param worker 当前用户
     * @return str
     */
    String updateAuditTeacher(AuditTeacherDto auditTeacherDto, Worker worker);


    /**
     * 批量删除导师
     * @param workerIds 删除导师的id
     * @param worker 当前用户
     * @return str
     */
    String delBatchTeacher(List<Integer> workerIds, Worker worker);

    /**
     * 导出模板
     * @param response 返回
     */
    void exportTemplate(HttpServletResponse response);

    /**
     * 添加/修改导师 当有id时修改
     * @param teacherDetailModifyDto 参数
     * @param worker 当前用户
     * @return str
     */
    String modifyTeacher(TeacherDetailModifyDto teacherDetailModifyDto,Worker worker);

    /**
     * 保存聘书图片
     * @param teacherExpandFileKeyDto 请求参数
     * @param worker 当前用户
     * @return str
     */
    String saveExpandFileKey(TeacherExpandFileKeyDto teacherExpandFileKeyDto, Worker worker);

    /**
     * 校验删除导师
     * @param workerIds 导师ids
     * @param worker 操作人
     * @return str
     */
    String validateDelBatchTeacher(List<Integer> workerIds, Worker worker);

    /**
     * 导出导师
     * @param workerIds 导师ids
     */
    void exportTeacher(List<Integer> workerIds);

    /**
     * 批量导出导师
     * @param teacherDto 导师导师
     */
    void batchExportTeacher(TeacherDto teacherDto);

    /**
     * 校验excel
     * @param file excel文件
     * @return data
     */
    Result<ExcelDataInfo> verifyExcel(MultipartFile file);

    /**
     * 分页查询荣誉
     * @param pageReq 请求参数
     * @return data
     */
    PageRsp<WorkerHonorVo> pageHonor(PageReq<StuExpandDto> pageReq);

    /**
     * 获取学员优势
     * @param stuExpandDto 请求参数
     * @return str
     */
    String getAdvantage(StuExpandDto stuExpandDto);

    /**
     * 分页查询集中培养
     * @param pageReq 分页参数
     * @return data
     */
    PageRsp<ConcentratedCultivationVo> pageConcentratedCultivationExpand(PageReq<StuExpandDto> pageReq);

    /**
     * 分页查询导师评价
     * @param pageReq 分页参数
     * @return data
     */
    PageRsp<TeacherTraineeVo> pageStuTraineeExpand(PageReq<TrackingCultivationDto> pageReq);

    /**
     * 我的学员页-获取学员扩展信息-跟踪培养
     * @param trackingCultivationDto 参数
     * @param worker worker
     * @return data
     */
    SelfCultivationVo getSelfCultivation(TrackingCultivationDto trackingCultivationDto, Worker worker);


    /**
     * 作用发挥
     * @param req 参数
     * @param worker worker
     * @return data
     */
    PageRsp<ExertVo> pageExert(PageReq<ExertDto> req, Worker worker);

    /**
     * 成长发展
     * @param req 参数
     * @param worker worker
     * @return data
     */
    PageRsp<GrowthVo> pageGrowth(PageReq<GroupCultivationDto> req, Worker worker);

    /**
     * Excel导入导师数据
     * @param dataKey 唯一值
     * @param worker dangqian用户
     * @return str
     */
    Result<String> importTeacherData(Worker worker,String dataKey);

    /**
     * 获取志愿服务和其他
     * @param stuHonorDto req
     * @return data
     */
    List<VolunteerVo> getVolunteer(StuHonorDto stuHonorDto);

    /**
     * 分页查询没有绑定导师的学员
     * @param req req
     * @return data
     */
    PageRsp<StudentVo> pageNoTeacherStu(PageReq<Integer> req);

    /**
     * 修改在聘/解聘状态
     * @param worker 当前用户
     * @param workerId 导师id
     * @param expandStatus 解聘/在聘
     * @return str
     */
    Result<String> updateStatus(Worker worker,Integer workerId,Integer expandStatus);

    /**
     * 查询已设置的学员
     * @param worker 当前用户
     * @param workerId 导师id
     * @return data
     */
    Result<List<StudentVo>> provisionedStu(Worker worker,Integer workerId);

    /**
     * 设置学员
     * @param worker 当前用户
     * @param dto 保存的学员ids
     * @return str
     */
    Result<String> saveStudent(Worker worker,StudentDto dto);

    /**
     * 评价学习交流
     * @param evaluateStuDto 参数
     * @param worker worker
     * @return str
     */
    String evaluateConcentratedCultivation(EvaluateStuDto evaluateStuDto,Worker worker);

    /**
     * 查询待审核,审核不通过,审核通过
     * @param orgId 组织id
     * @param worker worker
     * @return data
     */
    SearchNumVo searchNum(Worker worker,Long orgId);

    /**
     * 验证是否修改导师
     * @param teacherDetailModifyDto 参数
     * @param worker worekr
     * @return true 已修改
     */
    Boolean validUpdateTeacher(TeacherDetailModifyDto teacherDetailModifyDto, Worker worker);
}
