package com.crunii.micro.service.qm.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qm.api.domain.dto.org.OrgTreeReqDto;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameDto;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameVo;
import com.crunii.micro.service.qm.api.domain.dto.org.vo.OrgTreeVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.UserInfoDto;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.*;

import java.util.List;

/**
 * @author xia
 * @version 1.0
 * @date 2023/9/18 0018 15:11:23
 */
public interface UserInfoService {

	/**
	 * 判断用户是否已录入信息
	 * @param loginId 用户id
	 * @return Result<CheckUserInfoVo>
	 */
	Result<CheckUserInfoVo> checkUserInfo(int loginId);

	/**
	 * 新增用户
	 *
	 * @param worker 人员实体
	 * @param dto 入参实体
	 * @return Result<String>
	 */
	Result<String> insertUserInfo(Worker worker, UserInfoDto dto);

	/**
	 * 根据年度和类型查询班级列表
	 * @param worker 人员实体
	 * @param classesYear 班级年度
	 * @param levelCode 班级级别code
	 * @param orgId 班级组织id
	 * @return Result<List<ClassesVo>>
	 */
	Result<List<ClassesTypeVo>> selectClassesTypeByYearAndLevel(Worker worker, String classesYear, String levelCode, Long orgId);

	/**
	 * 查询用户信息
	 * @param loginId 人员id
	 * @return Result<UserInfoVo>
	 */
	Result<UserInfoVo> selectUserInfo(int loginId);

	/**
	 * 查询所有班级包含的年份
	 * @return Result<List<String>>
	 */
	Result<List<String>> selectClassesYearByAll();

	/**
	 * 根据年份查询班级的所有级别
	 * @param classesYear 班级年份
	 * @return Result<List<ClassesLevelVo>>
	 */
	Result<List<ClassesLevelVo>> selectClassesLevelByYear(String classesYear);

	/**
	 * 根据组织名称查询组织信息
	 * @param req 入参实体
	 * @return PageRsp<PageOrgByOrgNameVo>
	 */
	PageRsp<PageOrgByOrgNameVo> pageOrgByOrgName(PageReq<PageOrgByOrgNameDto> req);

	/**
	 * 青马工程查询组织树
	 * @param worker 人员实体
	 * @param param 入参实体
	 * @return List<OrgTreeVo>
	 */
	List<OrgTreeVo> getOrgChildrenTree(Worker worker, OrgTreeReqDto param);

	/**
	 *APP根据班级级别查询组织
	 * @param worker 人员实体
	 * @param param 入参实体
	 * @return List<OrgTreeVo>
	 */
	List<OrgTreeVo> getOrgChildrenTreeByOrgLevel(Worker worker, OrgTreeReqDto param);
}
