package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qm.api.domain.dto.Book.AppBookInfo;
import com.crunii.micro.service.qm.api.domain.dto.Book.AppBookInfoVo;
import com.crunii.micro.service.qm.api.domain.dto.Book.AppSearchBookDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.AppBookListDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.AppBookListInfoDto;
import com.crunii.micro.service.qm.api.domain.dto.bookList.AppBookListVo;
import com.crunii.micro.service.qm.jdbc.AppBookListDao;
import com.crunii.micro.service.qm.service.AppBookListService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/20 9:24
 */
@Service
@Slf4j
public class AppBookListServiceImpl implements AppBookListService {

    @Value("${upload.url}")
    private String urlPrefix;

    @Resource
    private AppBookListDao dao;

    @Override
    @InjectProperty
    public PageRsp<AppBookListVo> bookListPage(PageReq<AppBookListDto> req) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
       // log.info(JSONUtil.toJsonStr(currWorker));
        return dao.pageList(req);
    }

    @Override
    public PageRsp<AppBookInfo> viewBookList(Worker worker, PageReq<AppBookListInfoDto> req) {
        PageRsp<AppBookInfo> pageRsp = dao.pageBookList(req);
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        log.info(JSONUtil.toJsonStr(currWorker));
        List<AppBookInfo> result = pageRsp.getResult();
        if (CollectionUtil.isNotEmpty(result)) {
            result.forEach(x -> {
                if (StrUtil.isNotBlank(x.getFileKey())) {
                    x.setUrl(urlPrefix + x.getFileKey());
                }
            });
        }
        pageRsp.setResult(result);
        return pageRsp;
    }

    @Override
    public PageRsp<AppBookInfo> searchBook(Worker worker, PageReq<AppSearchBookDto> req) {
        PageRsp<AppBookInfo> pageRsp = dao.pageSearchBook(req);
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        log.info(JSONUtil.toJsonStr(currWorker));
        List<AppBookInfo> result = pageRsp.getResult();
        if (CollectionUtil.isNotEmpty(result)) {
            result.forEach(x -> {
                if (StrUtil.isNotBlank(x.getFileKey())) {
                    x.setUrl(urlPrefix + x.getFileKey());
                }
            });
        }
        pageRsp.setResult(result);
        return pageRsp;
    }

    @Override
    public Result<AppBookInfoVo> viewBookInfo(Worker worker, Long bookId) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        // log.info(JSONUtil.toJsonStr(currWorker));
        AppBookInfoVo bookInfo = dao.queryBookInfo(bookId);
        if (ObjectUtil.isNotEmpty(bookInfo)) {
            if (StrUtil.isNotBlank(bookInfo.getFileKey())) {
                bookInfo.setUrl(urlPrefix + bookInfo.getFileKey());
            }
        }
        return Result.ofSuccess(bookInfo);
    }

}
