package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQmCourseware;
import com.crunii.micro.domain.entity.main.YcqQmVideo;
import com.crunii.micro.domain.repo.main.YcqQmCoursewareRepo;
import com.crunii.micro.domain.repo.main.YcqQmVideoRepo;
import com.crunii.micro.service.qm.api.domain.dto.courseware.AppCoursewareDto;
import com.crunii.micro.service.qm.api.domain.dto.courseware.AppCoursewareInfoVo;
import com.crunii.micro.service.qm.api.domain.dto.courseware.AppCoursewareVo;
import com.crunii.micro.service.qm.jdbc.AppCoursewareDao;
import com.crunii.micro.service.qm.service.AppCoursewareService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

import static com.crunii.micro.service.qm.api.constants.QmConstants.CERTIFICATE_TYPE_PDF;
import static com.crunii.micro.service.qm.api.constants.QmConstants.DELETED_0;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/20 16:12
 */
@Service
@Slf4j
public class AppCoursewareServiceImpl implements AppCoursewareService {

    @Value("${upload.url}")
    private String urlPrefix;

    @Resource
    private AppCoursewareDao dao;

    @Resource
    private YcqQmCoursewareRepo repo;

    @Resource
    private YcqQmVideoRepo videoRepo;

    @Override
    @InjectProperty
    public PageRsp<AppCoursewareVo> coursewarePage(PageReq<AppCoursewareDto> req) {
        PageRsp<AppCoursewareVo> pageList = dao.pageList(req);
        List<AppCoursewareVo> result = pageList.getResult();
        for (AppCoursewareVo vo : result) {
            // pdf
            if (vo.getCertificateType().equals(CERTIFICATE_TYPE_PDF)) {
                if (StrUtil.isNotBlank(vo.getFileKey())) {
                    vo.setResUrl(urlPrefix + vo.getFileKey());
                }
            }else {
                // 视频
                YcqQmVideo video = videoRepo.findByFileIdAndDeleted(vo.getFileId(), DELETED_0);
                if (ObjectUtil.isNotEmpty(video)) {
                    vo.setResUrl(video.getPictureUrl());
                }
            }
        }
        pageList.setResult(result);
        return pageList;
    }

    @Override
    @InjectProperty
    public Result<AppCoursewareInfoVo> viewCourseware(Worker worker, Long certificateId) {
        AppCoursewareInfoVo vo = new AppCoursewareInfoVo();
        YcqQmCourseware courseware = repo.findByCertificateIdAndDeleted(certificateId, CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(courseware)) {
            throw new BusinessException("课件不存在");
        }
        BeanUtils.copyProperties(courseware, vo);
        // pdf
        if (courseware.getCertificateType().equals(CERTIFICATE_TYPE_PDF)) {
            if (StrUtil.isNotBlank(courseware.getFileKey())) {
                vo.setResUrl(urlPrefix + courseware.getFileKey());
            }
        }else {
            // 视频
            YcqQmVideo video = videoRepo.findByFileIdAndDeleted(vo.getFileId(), DELETED_0);
            if (ObjectUtil.isNotEmpty(video)) {
                vo.setResUrl(video.getPictureUrl());
            }
        }
        return Result.ofSuccess(vo);
    }
}
