package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.belowExcellent.*;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameVo;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.TeacherBindingListDto;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.TeacherListDto;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.TeacherListVo;
import com.crunii.micro.service.qm.jdbc.BelowExcellentDao;
import com.crunii.micro.service.qm.jdbc.StudentManageWebDao;
import com.crunii.micro.service.qm.service.BelowExcellentService;
import com.crunii.micro.service.qm.util.ListUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qm.api.constants.QmConstants.DELETED_0;

/**
 * @author xiayinqiang
 * @date 2023/9/27 0027 9:56:39
 */
@Slf4j
@Service
public class BelowExcellentServiceImpl implements BelowExcellentService {

	@Resource BelowExcellentDao belowExcellentDao;
	@Resource YcqQmBelowExcellenRepo ycqQmBelowExcellenRepo;
	@Resource YcqQmUserInfoRepo ycqQmUserInfoRepo;
	@Resource YcqQmIntoPartyRepo ycqQmIntoPartyRepo;
	@Resource YcqQmUserClassesRelRepo ycqQmUserClassesRelRepo;
	@Resource YcqQmTeacherStuRepo ycqQmTeacherStuRepo;
	@Resource
	private TeacherManageServiceImpl teacherManageServiceImpl;
	@Resource
	private StudentManageWebDao studentManageWebDao;

	/**
	 * 根据学员id查询上级组织
	 *
	 * @param workerId 学员id
	 * @return Result<PageOrgByOrgNameVo>
	 */
	@Override public Result<PageOrgByOrgNameVo> selectParentOrgByWorkerId(Integer workerId) {
		Result<PageOrgByOrgNameVo> result = new Result<>();
		PageOrgByOrgNameVo pageOrgByOrgNameVo = belowExcellentDao.selectParentOrgByWorkerId(workerId);
		if (ObjectUtil.isEmpty(pageOrgByOrgNameVo)) {
			return Result.ofFail("该学员组织已是最高级别");
		}
		result.setData(belowExcellentDao.selectParentOrgByWorkerId(workerId));
		return result;
	}

	/**
	 * 推优
	 *
	 * @param worker 人员实体
	 * @param dto    入参实体
	 * @return String
	 */
	@Override public String insertBelowExcellent(Worker worker, InsertBelowExcellentDto dto) {
		Date curr = new Date();
		YcqQmUserInfo ycqQmUserInfo = ycqQmUserInfoRepo.findByWorkerIdAndDeleted(dto.getStuWorkerId(), QmConstants.DELETED_0);
		if (ObjectUtil.isEmpty(ycqQmUserInfo)) {
			throw new BusinessException("学员不存在或已失效");
		}
		List<YcqQmBelowExcellen> qmBelowExcellenList = ycqQmBelowExcellenRepo
				.findByStuWorkerIdAndConfigTypeAndDeleted(dto.getStuWorkerId(), QmConstants.BE_CONFIG_TYPE_0, QmConstants.DELETED_0);
		if (ListUtil.hasValue(qmBelowExcellenList)) {
			throw new BusinessException("该学员推优后未配置班级或导师");
		}
//		//判断是否有关联的导师，没有关联的导师则不允许推优
//		int count = belowExcellentDao.selectTeachStu(dto.getStuWorkerId());
//		if (count <= 0) {
//			throw new BusinessException("该学员未关联导师");
//		}

		YcqQmBelowExcellen qmBelowExcellen = new YcqQmBelowExcellen();
		BeanUtils.copyProperties(dto, qmBelowExcellen);
		qmBelowExcellen.setOldOrgId(ycqQmUserInfo.getOrgId());
		qmBelowExcellen.setRecommendWorkerId(worker.getLoginId());
		qmBelowExcellen.setRecommendTime(curr);
		qmBelowExcellen.setCreatorId(worker.getLoginId());
		qmBelowExcellen.setCreateTime(curr);
		qmBelowExcellen.setUpdaterId(worker.getLoginId());
		qmBelowExcellen.setUpdateTime(curr);
		qmBelowExcellen.setDeleted(QmConstants.DELETED_0);
		qmBelowExcellen.setConfigType(QmConstants.BE_CONFIG_TYPE_0);

		ycqQmBelowExcellenRepo.saveAndFlush(qmBelowExcellen);

		//修改学员组织
		ycqQmUserInfo.setOrgId(dto.getOrgId());
		ycqQmUserInfo.setUpdaterId(worker.getLoginId());
		ycqQmUserInfo.setUpdateTime(new Date());
		ycqQmUserInfoRepo.saveAndFlush(ycqQmUserInfo);

		return "推优成功";
	}

	/**
	 * 下级推优列表给学员分配班级和导师
	 *
	 * @param worker 人员实体
	 * @param dto    入参实体
	 * @return String
	 */
	@Override public String insertStuClassesAndTeach(Worker worker, InsertStuClassesAndTeachDto dto) {
		Date curr = new Date();
		//新增班级
		YcqQmUserClassesRel ycqQmUserClassesRel = new YcqQmUserClassesRel();
		ycqQmUserClassesRel.setClassesId(dto.getClassesId());
		ycqQmUserClassesRel.setStuWorkerId(dto.getStuWorkerId());
		ycqQmUserClassesRel.setCreatorId(worker.getLoginId());
		ycqQmUserClassesRel.setCreateTime(curr);
		ycqQmUserClassesRel.setUpdaterId(worker.getLoginId());
		ycqQmUserClassesRel.setUpdateTime(curr);
		ycqQmUserClassesRel.setDeleted(QmConstants.DELETED_0);

		//关联导师
		//先取消原来绑定的导师
		List<YcqQmTeacherStu> qmTeacherStuList = ycqQmTeacherStuRepo
				.findAllByStuWorkerIdAndBindingAndDeleted(dto.getStuWorkerId(), QmConstants.TEACH_STU_BINDING_1, QmConstants.DELETED_0);
		if (ListUtil.hasValue(qmTeacherStuList)) {
			qmTeacherStuList.forEach(e -> {
				e.setBinding(QmConstants.TEACH_STU_BINDING_0);
				e.setUpdateTime(curr);
				e.setUpdaterId(worker.getLoginId());
			});
			ycqQmTeacherStuRepo.saveAllAndFlush(qmTeacherStuList);
		}
		//再关联现在的导师
		YcqQmTeacherStu stu = new YcqQmTeacherStu();
		//这里班级直接使用新选择的这个班级，这个班级就是学员最高组织的班级
		stu.setStuClassesId(dto.getClassesId());
		stu.setStuWorkerId(dto.getStuWorkerId());
		stu.setTeachWorkerId(dto.getTeachWorkerId());
		stu.setCorrelationState(QmConstants.USER_CORRELATION_STATE_1);
		// 选中状态 0 学生申请 1老师添加 2管理员添加,这里选2管理员添加
		stu.setChooseState(2);
		// 是否绑定 0已解绑 1已绑定 这里选1已绑定
		stu.setBinding(QmConstants.TEACH_STU_BINDING_1);
		stu.setCreatorId(worker.getLoginId());
		stu.setCreateTime(curr);
		stu.setUpdaterId(worker.getLoginId());
		stu.setUpdateTime(curr);
		stu.setDeleted(QmConstants.DELETED_0);
		stu.setStartTime(curr);
		//修改推优表配置状态
		List<YcqQmBelowExcellen> qmBelowExcellenList = ycqQmBelowExcellenRepo
				.findByStuWorkerIdAndConfigTypeAndDeleted(dto.getStuWorkerId(), QmConstants.BE_CONFIG_TYPE_0, QmConstants.DELETED_0);
		qmBelowExcellenList.forEach(qmBelowExcellen -> {
			qmBelowExcellen.setUpdaterId(worker.getLoginId());
			qmBelowExcellen.setUpdateTime(curr);
			qmBelowExcellen.setConfigType(QmConstants.BE_CONFIG_TYPE_1);
		});

		ycqQmBelowExcellenRepo.saveAllAndFlush(qmBelowExcellenList);

		ycqQmUserClassesRelRepo.saveAndFlush(ycqQmUserClassesRel);
		ycqQmTeacherStuRepo.saveAndFlush(stu);

		return "设置成功";
	}

	/**
	 * 分页查询下级推优列表
	 *
	 * @param req    入参实体
	 * @param worker 人员实体
	 * @return PageRsp<PageBelowExcellentVo>
	 */
	@Override public PageRsp<PageBelowExcellentVo> pageBelowExcellent(PageReq<PageBelowExcellentDto> req, Worker worker) {

		// 查询组织下所有组织
		List<Long> orgIdList = teacherManageServiceImpl.getOrgIdList(Long.valueOf(req.getQuery().getOrgId()));
		if (Objects.isNull(orgIdList)) {
			return new PageRsp<>();
		}

		PageRsp<PageBelowExcellentVo> prsVo = belowExcellentDao.pageBelowExcellent(req, orgIdList);
		List<PageBelowExcellentVo> result = prsVo.getResult();
		if (CollectionUtils.isEmpty(result)) {
			return prsVo;
		}
		//查询学员班级信息
		List<Integer> workerIdList = result.stream().map(PageBelowExcellentVo::getWorkerId).collect(Collectors.toList());
		List<PageBelowExecllentClassesVo> classesVoList = belowExcellentDao.selectClassesInfoByWorkerIds(workerIdList);
		if (!CollectionUtils.isEmpty(classesVoList)) {
			for (PageBelowExcellentVo pageBelowExcellentVo : result) {
				List<PageBelowExecllentClassesVo> classesVos = new ArrayList<>();
				for (PageBelowExecllentClassesVo classesVo : classesVoList) {
					if (pageBelowExcellentVo.getWorkerId().equals(classesVo.getStuWorkerId())) {
						classesVos.add(classesVo);
					}
				}
				pageBelowExcellentVo.setClassesVoList(classesVos);
			}
		}
		return prsVo;
	}

	/**
	 * 导出
	 *
	 * @param dto      入参实体
	 * @param response 请求
	 */
	@Override public void exportByList(ExportByListDto dto, HttpServletResponse response) throws IOException {

		List<ExportByListVo> listAll = belowExcellentDao.exportByList(dto);
		if (!CollectionUtils.isEmpty(listAll)) {
			//查询学员班级信息
			List<Integer> workerIdList = listAll.stream().map(ExportByListVo::getWorkerId).collect(Collectors.toList());
			List<PageBelowExecllentClassesVo> classesVoList = belowExcellentDao.selectClassesInfoByWorkerIds(workerIdList);
			if (!CollectionUtils.isEmpty(classesVoList)) {
				for (ExportByListVo exportByListVo : listAll) {
					StringBuilder sb = new StringBuilder();
					for (PageBelowExecllentClassesVo classesVo : classesVoList) {
						if (exportByListVo.getWorkerId().equals(classesVo.getStuWorkerId())) {
							sb.append(classesVo.getClassesCode()).append("/").append(classesVo.getClassesOrgName()).append("/")
									.append(classesVo.getClassesYear()).append("/").append(classesVo.getLevelName()).append("/")
									.append(classesVo.getTypeName()).append("\r\n");
						}
					}
					exportByListVo.setClassesVoList(sb.toString());
				}
			}
		}
		ExcelUtil.writeSingleExcel("export", "sheet1", listAll, ExportByListVo.class);
	}

	/**
	 * 分页查询学员被拒绝两次列表
	 *
	 * @param req    入参实体
	 * @param worker 人员实体
	 * @return PageRsp<PageStuRefuseTwiceVo>
	 */
	@Override public PageRsp<PageStuRefuseTwiceVo> pageStuRefuseTwice(PageReq<PageStuRefuseTwiceDto> req, Worker worker) {
		// 查询组织下所有组织
		List<Long> orgIdList = teacherManageServiceImpl.getOrgIdList(Long.valueOf(req.getQuery().getOrgId()));
		if (Objects.isNull(orgIdList)) {
			return new PageRsp<>();
		}

		PageRsp<PageStuRefuseTwiceVo> prsVo = belowExcellentDao.pageStuRefuseTwice(req, orgIdList);
		List<PageStuRefuseTwiceVo> result = prsVo.getResult();
		if (CollectionUtils.isEmpty(result)) {
			return prsVo;
		}
		//查询学员班级信息
		List<Integer> workerIdList = result.stream().map(PageStuRefuseTwiceVo::getWorkerId).collect(Collectors.toList());
		List<PageBelowExecllentClassesVo> classesVoList = belowExcellentDao.selectClassesInfoByWorkerIds(workerIdList);
		if (!CollectionUtils.isEmpty(classesVoList)) {
			for (PageStuRefuseTwiceVo vo : result) {
				List<PageBelowExecllentClassesVo> classesVos = new ArrayList<>();
				for (PageBelowExecllentClassesVo classesVo : classesVoList) {
					if (vo.getWorkerId().equals(classesVo.getStuWorkerId())) {
						classesVos.add(classesVo);
					}
				}
				vo.setClassesVoList(classesVos);
			}
		}
		return prsVo;
	}

	/**
	 * 下级推优查询导师列表
	 *
	 * @param req 入参实体
	 * @return List<TeacherListVo>
	 */
	@Override public List<TeacherListVo> teacherList(TeacherListDto req) {
		String getOneLevelCode = studentManageWebDao.getOneLevelCode(req.getStuWorkerId());
		if (ObjectUtil.isNotEmpty(req.getLevelCode())) {
			getOneLevelCode = req.getLevelCode();
		}
		List<TeacherListVo> voList = new ArrayList<>();
		List<TeacherListVo> teacherListVo = studentManageWebDao.teacherList(req, getOneLevelCode);
		YcqQmUserInfo userInfo = ycqQmUserInfoRepo.findByWorkerIdAndDeleted(req.getStuWorkerId(), DELETED_0);
		if (ObjectUtil.isEmpty(userInfo)) {
			throw new BusinessException("学员无效");
		}
		//查询已经绑定的导师
		TeacherBindingListDto bindingListDto = new TeacherBindingListDto();
		bindingListDto.setUserInfoId(userInfo.getId());
		List<TeacherListVo> bindingTeacherList = studentManageWebDao.bindingTeacherList(bindingListDto);
		if(ListUtil.hasValue(bindingTeacherList)){
			TeacherListVo listVo = bindingTeacherList.get(0);
			for (TeacherListVo vo : teacherListVo) {
				if (!vo.getTeachWorkerId().equals(listVo.getTeachWorkerId())) {
					voList.add(vo);
				}
			}
		}else{
			//没有匹配的导师
			voList.addAll(teacherListVo);
		}

		return voList;
	}

	/**
	 * 根据学员id查询推荐入党记录
	 *
	 * @param worker   人员实体
	 * @param workerId 学员id
	 * @return Result<List < SelectIntoPartyVo>>
	 */
	@Override public Result<List<SelectIntoPartyVo>> selectIntoPartyByWorkerId(Worker worker, Integer workerId) {
		Result<List<SelectIntoPartyVo>> result = new Result<>();
		result.setData(belowExcellentDao.selectIntoPartyByWorkerId(workerId));
		return result;
	}

	/**
	 * 新增或编辑推荐入党
	 *
	 * @param worker 人员实体
	 * @param dto    入参实体
	 * @return String
	 */
	@Override public String insertIntoParty(Worker worker, InsertIntoPartyDto dto) {
		YcqQmIntoParty ycqQmIntoParty;
		Date curr = new Date();
		if (ObjectUtil.isNotEmpty(dto.getRecordId())) {
			ycqQmIntoParty = ycqQmIntoPartyRepo.findByRecordIdAndDeleted(dto.getRecordId(), QmConstants.DELETED_0);
			if (ObjectUtil.isNotEmpty(ycqQmIntoParty)) {
				BeanUtils.copyProperties(dto, ycqQmIntoParty);
				ycqQmIntoParty.setRecommendWorkerId(worker.getLoginId());
				ycqQmIntoParty.setRecommendTime(curr);
				ycqQmIntoParty.setUpdaterId(worker.getLoginId());
				ycqQmIntoParty.setUpdateTime(curr);
				ycqQmIntoPartyRepo.saveAndFlush(ycqQmIntoParty);
				return "修改成功";
			}
		}

		ycqQmIntoParty = new YcqQmIntoParty();
		BeanUtils.copyProperties(dto, ycqQmIntoParty);
		ycqQmIntoParty.setRecommendWorkerId(worker.getLoginId());
		ycqQmIntoParty.setRecommendTime(curr);
		ycqQmIntoParty.setCreatorId(worker.getLoginId());
		ycqQmIntoParty.setCreateTime(curr);
		ycqQmIntoParty.setUpdaterId(worker.getLoginId());
		ycqQmIntoParty.setUpdateTime(curr);
		ycqQmIntoParty.setDeleted(QmConstants.DELETED_0);
		ycqQmIntoPartyRepo.saveAndFlush(ycqQmIntoParty);

		return "推荐成功";
	}

	/**
	 * 删除学员推荐入党记录
	 *
	 * @param worker   人员实体
	 * @param recordId 入参实体
	 * @return String
	 */
	@Override public Result<String> deleteIntoPartyById(Worker worker, Long recordId) {
		YcqQmIntoParty ycqQmIntoParty = ycqQmIntoPartyRepo.findByRecordIdAndDeleted(recordId, QmConstants.DELETED_0);
		if (ObjectUtil.isNotEmpty(ycqQmIntoParty)) {
			ycqQmIntoParty.setUpdaterId(worker.getLoginId());
			ycqQmIntoParty.setUpdateTime(new Date());
			ycqQmIntoParty.setDeleted(QmConstants.DELETED_1);
			ycqQmIntoPartyRepo.saveAndFlush(ycqQmIntoParty);
		}
		return Result.ofSuccess("删除成功");
	}

}
