package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.appFoster.*;
import com.crunii.micro.service.qm.api.domain.dto.classesMgr.*;
import com.crunii.micro.service.qm.jdbc.ClassesMgrDao;
import com.crunii.micro.service.qm.service.ClassesMgrService;
import com.crunii.micro.service.qm.service.CommonService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author: LG
 * @date: 2023/9/18 19:42
 */
@Service
public class ClassesMgrServiceImpl implements ClassesMgrService {

    @Resource
    private ClassesMgrDao classesMgrDao;

    @Resource
    private YcqQmClassesInfoRepo ycqQmClassesInfoRepo;

    @Resource
    private YcqQmCourseSectionRepo ycqQmCourseSectionRepo;

    @Resource
    private YcqQmCourseRepo ycqQmCourseRepo;

    @Resource
    private YcqQmUserCourseRepo ycqQmUserCourseRepo;

    @Resource
    private CommonService commonService;

    @Resource
    private YcqQmUserCourseSectionRepo ycqQmUserCourseSectionRepo;

    @Resource
    private YcqQmCourseSectionRelRepo ycqQmCourseSectionRelRepo;

    @Override
    public Result<List<QueryClassesTypeVO>> queryClassesType(String levelCode){
        return Result.ofSuccess(classesMgrDao.queryClassesType(levelCode));
    }

    @Override
    public Result<List<QueryClassesTypeVO>> queryClassesLevelByAdd(String orgLevel) {
        List<String> orgList = new ArrayList<>();
        orgList.add(orgLevel);
        if(QmConstants.CITY_LEVEL.equals(orgLevel)){
            orgList.add(QmConstants.NATIONAL_LEVEL);
        }
        return Result.ofSuccess(classesMgrDao.queryClassesLevel(orgList));
    }

    @Override
    public Result<List<QueryClassesTypeVO>> queryClassesLevel() {
        return Result.ofSuccess(classesMgrDao.queryClassesLevel());
    }

    @Override
    public Result<PageRsp<QueryClassesVO>> queryClasses(PageReq<QueryClassesDTO> req) {
        return Result.ofSuccess(classesMgrDao.queryClasses(req));
    }

    @Override
    public Result<Object> deleteClasses(List<Long> classesId) {
        if (CollectionUtil.isEmpty(classesId)){
            throw new BusinessException("请传入id");
        }
        for (Long s : classesId) {
            Integer integer = classesMgrDao.classesStuNum(s);
            if (!integer.equals(CommonConstants.STATE_INVALID)){
//                String s1 = classesMgrDao.queryClassesCode(s);
                throw new BusinessException("该班级下还有在读学员，不允许删除");
            }
        }

        classesMgrDao.deleteClasses(classesId);
        return Result.ofSuccess("删除成功");
    }

    @Override
    public Result<Object> saveClasses(Worker worker, SaveClassesDTO saveClassesDTO) {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }
        YcqQmClassesInfo ycqQmClassesInfo = new YcqQmClassesInfo();

        if(ObjectUtil.isNotEmpty(saveClassesDTO.getClassesId())){
            ycqQmClassesInfo.setClassesId(saveClassesDTO.getClassesId());
        }else{
            //判断是否有本年度同一班级类型
            Integer whetherClassesType = classesMgrDao.whetherClassesType(saveClassesDTO);
            if (!whetherClassesType.equals(CommonConstants.STATE_INVALID)){
                throw new BusinessException("班级类型已存在");
            }
        }

        ycqQmClassesInfo.setOrgId(saveClassesDTO.getOrgId());
        ycqQmClassesInfo.setClassesCode(makeCode("BJ"));
        ycqQmClassesInfo.setClassesStatus(CommonConstants.STATE_VALID);
        ycqQmClassesInfo.setClassesYear(saveClassesDTO.getClassesYear());
        ycqQmClassesInfo.setClassesTypeId(saveClassesDTO.getClassesTypeId());
        ycqQmClassesInfo.setCreateTime(new Date());
        ycqQmClassesInfo.setCreatorId(worker.getLoginId());
        ycqQmClassesInfo.setUpdateTime(new Date());
        ycqQmClassesInfo.setUpdaterId(worker.getLoginId());
        ycqQmClassesInfo.setDeleted(CommonConstants.STATE_INVALID);
        YcqQmClassesInfo save = ycqQmClassesInfoRepo.save(ycqQmClassesInfo);

        commonService.insertUserClassesRel(worker.getLoginId(),saveClassesDTO.getLoginIds(),save.getClassesId());

        return Result.ofSuccess("新增成功");
    }

    @Override
    public Result<Object> saveCourse(Worker worker, SaveCourseDTO saveCourseDTO) throws ParseException {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }


        YcqQmCourse ycqQmCourse = new YcqQmCourse();
        if (ObjectUtil.isNotEmpty(saveCourseDTO.getCourseId())){
            ycqQmCourse.setCourseId(saveCourseDTO.getCourseId());
        }

        ycqQmCourse.setCourseName(saveCourseDTO.getCourseName());
        ycqQmCourse.setCourseSubject(saveCourseDTO.getCourseSubject());
        ycqQmCourse.setClassesId(saveCourseDTO.getClassesId());
        ycqQmCourse.setOrgId(saveCourseDTO.getOrgId());
        ycqQmCourse.setStartTime(saveCourseDTO.getStartTime());
        ycqQmCourse.setEndTime(saveCourseDTO.getEndTime());
        ycqQmCourse.setDeleted(CommonConstants.STATE_INVALID);
        ycqQmCourse.setCreateTime(new Date());
        ycqQmCourse.setCreatorId(worker.getLoginId());
        ycqQmCourse.setUpdateTime(new Date());
        ycqQmCourse.setUpdaterId(worker.getLoginId());
        ycqQmCourse.setCourseCode(makeCode("KC"));
        ycqQmCourse.setSectionNumber(countCourseNumber(saveCourseDTO));
        YcqQmCourse saveYcqQmCourse = ycqQmCourseRepo.save(ycqQmCourse);

        List<YcqQmCourseSection> listSection = new ArrayList<>();
        List<YcqQmCourseSectionRel> listCourseDay = new ArrayList<>();

        List<CourseSectionDTO> sectionDTOS = saveCourseDTO.getSectionDTOS();
        for (CourseSectionDTO sectionDTO : sectionDTOS) {
            // 存 天 数据
            YcqQmCourseSectionRel ycqQmCourseSectionRel = new YcqQmCourseSectionRel();
            ycqQmCourseSectionRel.setCourseId(saveYcqQmCourse.getCourseId());
            ycqQmCourseSectionRel.setCourseDay(sectionDTO.getDate());
            listCourseDay.add(ycqQmCourseSectionRel);

            List<SectionDTO> sonSectionDTOS = sectionDTO.getSonSectionDTOS();
            for (SectionDTO sonSectionDTO : sonSectionDTOS) {
                YcqQmCourseSection ycqQmCourseSection = new YcqQmCourseSection();
                String startTimeStr = sectionDTO.getDate() + sonSectionDTO.getSectionTime().get(0);
                String endTimeStr = sectionDTO.getDate() + sonSectionDTO.getSectionTime().get(1);
                Date startTime = new SimpleDateFormat("yyyy-MM-ddHH:mm").parse(startTimeStr);
                Date endTime = new SimpleDateFormat("yyyy-MM-ddHH:mm").parse(endTimeStr);

                if (startTime.after(endTime)){
                    throw new BusinessException("请传入正确的节次起止时间");
                }
                ycqQmCourseSection.setSectionStartTime(startTime);
                ycqQmCourseSection.setSectionEndTime(endTime);
                ycqQmCourseSection.setContentCourse(sonSectionDTO.getContentCourse());
                ycqQmCourseSection.setLecturer(sonSectionDTO.getLecturer());
                ycqQmCourseSection.setDeleted(CommonConstants.STATE_INVALID);
                ycqQmCourseSection.setCreatorId(worker.getLoginId());
                ycqQmCourseSection.setCourseId(saveYcqQmCourse.getCourseId());
                ycqQmCourseSection.setCreateTime(new Date());

                listSection.add(ycqQmCourseSection);
            }
        }
        ycqQmCourseSectionRelRepo.saveAll(listCourseDay);
        ycqQmCourseSectionRepo.saveAll(listSection);


        return Result.ofSuccess("新增成功");
    }

    private int countCourseNumber(SaveCourseDTO saveCourseDTO) {
        int i = 0;
        for (CourseSectionDTO sectionDTO : saveCourseDTO.getSectionDTOS()) {
            for (SectionDTO sonSectionDTO : sectionDTO.getSonSectionDTOS()) {
                i++;
            }
        }
        return i;
    }

    @Override
    public Result<Object> updateCourse(Worker worker, SaveCourseDTO saveCourseDTO) throws ParseException {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }

        classesMgrDao.deleteCourseSectionRel(saveCourseDTO.getCourseId());
        classesMgrDao.deleteSection(saveCourseDTO.getCourseId());
        saveCourse(worker, saveCourseDTO);
        return Result.ofSuccess("修改成功");
    }

    @Override
    public Result<Object> deleteCourse(Worker worker, List<Long> courseIds) {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }

        if (CollectionUtil.isEmpty(courseIds)){
            throw new BusinessException("请选课程");
        }

        Result<Object> result = new Result<>();
        List<Long> unCourse = new ArrayList<>();
        for (Long courseId : courseIds) {
            Integer integer = classesMgrDao.courseStuNum(courseId);
            if (!integer.equals(CommonConstants.STATE_INVALID)){
                unCourse.add(courseId);
            }
        }
        if(CollectionUtil.isEmpty(unCourse)){
            for (Long s : courseIds) {
                classesMgrDao.deleteCourseSectionRel(s);
                classesMgrDao.deleteSection(s);
                classesMgrDao.deleteCourse(s);
            }
            result.setMsg("删除成功");
            result.setSuccess(true);
            return result;
        }

        result.setData(unCourse);
        result.setSuccess(false);
        result.setMsg("选中课程已有学员参与，无法删除！");
        return result;


    }

    @Override
    public Result<PageRsp<QueryCourseVO>> queryCourse(PageReq<QueryCourseDTO> req) {
        PageRsp<QueryCourseVO> queryCourseVOPageRsp = classesMgrDao.queryCourse(req);
        List<QueryCourseVO> result = queryCourseVOPageRsp.getResult();
        Date nowDate = new Date();
        if(CollectionUtil.isNotEmpty(result)){
            for (QueryCourseVO queryCourseVO : result) {
                QueryEvaluateStuVO queryEvaluateStuVO1 = classesMgrDao.queryClassesYearType(queryCourseVO.getCourseId());
                if (ObjectUtil.isNotEmpty(queryEvaluateStuVO1)){
                    queryCourseVO.setClassesLevelName(queryEvaluateStuVO1.getClassesLevelName());
                    queryCourseVO.setClassesTypeName(queryEvaluateStuVO1.getClassesTypeName());
                    queryCourseVO.setClassesYear(queryEvaluateStuVO1.getClassesYear());
                    queryCourseVO.setOrgName(queryEvaluateStuVO1.getOrgName());
                    judgeTense(nowDate,queryCourseVO);
                }
            }
        }
        return Result.ofSuccess(queryCourseVOPageRsp);
    }

    private void judgeTense(Date nowDate, QueryCourseVO queryCourseVO) {
        //判断课程进行
        //在startime之后 且 在endtime之前或相等  为正在进行
        if(nowDate.after(queryCourseVO.getStartTime()) && nowDate.before(queryCourseVO.getEndTime())){
            queryCourseVO.setCourseState(CommonConstants.MENU_TYPE_BUTTON);
            queryCourseVO.setCourseStateText("正在进行");
        }

        //在startime之前 为未开始
        if (nowDate.before(queryCourseVO.getStartTime()) ){
            queryCourseVO.setCourseState(CommonConstants.MENU_TYPE_MENU);
            queryCourseVO.setCourseStateText("未开始");
        }

        //在startime之后 在endtime之后 为 已结束
        if (nowDate.after(queryCourseVO.getStartTime()) && nowDate.after(queryCourseVO.getEndTime())){
            queryCourseVO.setCourseState(CommonConstants.STATE_INVALID);
            queryCourseVO.setCourseStateText("已结束");
        }
    }

    @Override
    public Result<QueryCourseVO> queryCourseDetail(Long courseId) {
        QueryCourseVO queryCourseVO = classesMgrDao.queryCourseVO(courseId);
        List<SectionVO> sectionVOS = classesMgrDao.querySection(courseId);
        List<YcqQmCourseSectionRel> ycqQmCourseSectionRels = classesMgrDao.queryCourseSectionRel(courseId);
        Map<String, List<SectionVO>> collect = sectionVOS.stream().collect(Collectors.groupingBy(SectionVO::getDateDay));

        List<CourseSectionDTO> sectionDTOS = new ArrayList<>();

        for (YcqQmCourseSectionRel ycqQmCourseSectionRel : ycqQmCourseSectionRels) {
            CourseSectionDTO courseSectionDTO = new CourseSectionDTO();
            List<SectionVO> secVOS = collect.get(ycqQmCourseSectionRel.getCourseDay());
            if (CollectionUtil.isNotEmpty(secVOS)) {
                List<SectionDTO> sonSectionDTOS = new ArrayList<>();
                for (SectionVO sectionVO : secVOS) {
                    SectionDTO sectionDTO = new SectionDTO();
                    sectionDTO.setContentCourse(sectionVO.getContentCourse());
                    sectionDTO.setLecturer(sectionVO.getLecturer());
                    List<String> timeList = new ArrayList<>(2);
                    timeList.add(sectionVO.getHmStarTime());
                    timeList.add(sectionVO.getHmEndTime());
                    sectionDTO.setSectionTime(timeList);
                    sonSectionDTOS.add(sectionDTO);
                }
                courseSectionDTO.setSonSectionDTOS(sonSectionDTOS);
            }
            courseSectionDTO.setDate(ycqQmCourseSectionRel.getCourseDay());
            sectionDTOS.add(courseSectionDTO);
        }
        if (ObjectUtil.isNotNull(queryCourseVO)) {
            //注入 课程节次
            queryCourseVO.setSectionDTOS(sectionDTOS);
            //注入班级 year type level
            QueryEvaluateStuVO queryEvaluateStuVO1 = classesMgrDao.queryClassesYearType(queryCourseVO.getCourseId());
            if (ObjectUtil.isNotEmpty(queryEvaluateStuVO1)){
                queryCourseVO.setClassesLevelName(queryEvaluateStuVO1.getClassesLevelName());
                queryCourseVO.setClassesTypeName(queryEvaluateStuVO1.getClassesTypeName());
                queryCourseVO.setClassesYear(queryEvaluateStuVO1.getClassesYear());
                queryCourseVO.setOrgId(queryEvaluateStuVO1.getOrgId());
                queryCourseVO.setOrgName(queryEvaluateStuVO1.getOrgName());
            }
        }else{
            throw new BusinessException("不存在此课程");
        }
        Date nowDate = new Date();
        //判断课程进行时
        judgeTense(nowDate,queryCourseVO);

        return Result.ofSuccess(queryCourseVO);
    }

    @Override
    public Result<Object> saveParticipationCourse(Worker worker, Long courseId,Long courseSectionId) {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }

        Integer integer = classesMgrDao.queryUserCourse(worker.getLoginId(), courseId);
        if (integer.equals(CommonConstants.STATE_INVALID)){
            //存大课程参与表
            YcqQmUserCourse ycqQmUserCourse = new YcqQmUserCourse();
            ycqQmUserCourse.setStuWorkerId(worker.getLoginId());
            ycqQmUserCourse.setCourseId(courseId);
            ycqQmUserCourse.setCreateTime(new Date());
            ycqQmUserCourse.setDeleted(CommonConstants.STATE_INVALID);
            ycqQmUserCourse.setCreatorId(worker.getLoginId());
            ycqQmUserCourse.setUpdaterId(worker.getLoginId());
            ycqQmUserCourseRepo.save(ycqQmUserCourse);
        }

        Integer integer2 = classesMgrDao.queryUserCourseSection(worker.getLoginId(), courseSectionId);
        if (integer2.equals(CommonConstants.STATE_INVALID)){
            //存子课时参与表
            YcqQmUserCourseSection ycqQmUserCourseSection = new YcqQmUserCourseSection();
            ycqQmUserCourseSection.setCreateTime(new Date());
            ycqQmUserCourseSection.setDeleted(CommonConstants.STATE_INVALID);
            ycqQmUserCourseSection.setStuWorkerId(worker.getLoginId());
            ycqQmUserCourseSection.setCourseSectionId(courseSectionId);
            ycqQmUserCourseSectionRepo.save(ycqQmUserCourseSection);
            return Result.ofSuccess("参加成功");
        }else{
            throw new BusinessException("已参与此课时");
        }



    }

    @Override
    public Result<List<QueryUserClassesVO>> queryUserClasses(Worker worker) {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }

        return Result.ofSuccess(classesMgrDao.queryUserClasses(worker.getLoginId()));
    }

    @Override
    public Result<List<GroupCourseVO>> queryClassesCourse(Worker worker, Long classesId) {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }

        List<QueryClassesCourseVO> queryClassesCourseVOS = classesMgrDao.queryClassesCourseVOList(classesId);
        List<Long> participationCourse = classesMgrDao.stuParticipationCourse(worker.getLoginId());
        Date nowDate = new Date();
        //分类 时间 ，根据结束时间和开始时间， 找出参与课程
        //正在进行：1 ， 已结束：0 ， 未开始：2
        List<GroupCourseVO> resultList = new ArrayList<>();
        GroupCourseVO inProgress = new GroupCourseVO();
        inProgress.setOngoingState("1");
        GroupCourseVO finished = new GroupCourseVO();
        finished.setOngoingState("0");
        GroupCourseVO beStar = new GroupCourseVO();
        beStar.setOngoingState("2");

        List<QueryClassesCourseVO> inProgressList = new ArrayList<>();
        List<QueryClassesCourseVO> finishedList = new ArrayList<>();
        List<QueryClassesCourseVO> beStarList = new ArrayList<>();

        for (QueryClassesCourseVO queryClassesCourseVO : queryClassesCourseVOS) {
            //在startime之后 且 在endtime之前或相等  为正在进行
            if(nowDate.after(queryClassesCourseVO.getStartTime()) && nowDate.before(queryClassesCourseVO.getEndTime())){
                inProgressList.add(queryClassesCourseVO);
            }

            //在startime之前 为未开始
            if (nowDate.before(queryClassesCourseVO.getStartTime()) ){
                beStarList.add(queryClassesCourseVO);
            }

            //在startime之后 在endtime之后 为 已结束
            if (nowDate.after(queryClassesCourseVO.getEndTime()) && nowDate.after(queryClassesCourseVO.getEndTime())){
                finishedList.add(queryClassesCourseVO);
            }
            //设置是否参与
            if (CollectionUtil.isNotEmpty(participationCourse)) {
                if (participationCourse.contains(queryClassesCourseVO.getCourseId())) {
                    queryClassesCourseVO.setWhetherParticipation(true);
                }
            }
        }
        inProgress.setQueryClassesCourseVOList(inProgressList);
        finished.setQueryClassesCourseVOList(finishedList);
        beStar.setQueryClassesCourseVOList(beStarList);
        resultList.add(inProgress);
        resultList.add(finished);
        resultList.add(beStar);

        return Result.ofSuccess(resultList);
    }

    @Override
    public Result<List<GroupSectionVO>> queryCourseSection(Worker worker, Long courseId) {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }

        List<QuerySectionVO> querySectionVOS = classesMgrDao.querySectionList(courseId);
        List<Long> participationCourseSection = classesMgrDao.stuParticipationCourseSection(worker.getLoginId());

        Date nowDate = new Date();
        List<GroupSectionVO> resultList = new ArrayList<>();
        GroupSectionVO inProgress = new GroupSectionVO();
        inProgress.setOngoingState("1");
        GroupSectionVO finished = new GroupSectionVO();
        finished.setOngoingState("0");
        GroupSectionVO beStar = new GroupSectionVO();
        beStar.setOngoingState("2");

        List<QuerySectionVO> inProgressList = new ArrayList<>();
        List<QuerySectionVO> finishedList = new ArrayList<>();
        List<QuerySectionVO> beStarList = new ArrayList<>();

        for (QuerySectionVO querySectionVO : querySectionVOS) {
            //在startime之后 且 在endtime之前或相等  为正在进行
            if(nowDate.after(querySectionVO.getSectionStartTime()) && nowDate.before(querySectionVO.getSectionEndTime())){
                inProgressList.add(querySectionVO);
            }

            //在startime之前 为未开始
            if (nowDate.before(querySectionVO.getSectionStartTime()) ){
                beStarList.add(querySectionVO);
            }

            //在startime之后 在endtime之后 为 已结束
            if (nowDate.after(querySectionVO.getSectionStartTime()) && nowDate.after(querySectionVO.getSectionEndTime())){
                finishedList.add(querySectionVO);
            }

            //设置是否参与
            if (CollectionUtil.isNotEmpty(participationCourseSection)) {
                if (participationCourseSection.contains(querySectionVO.getSectionId())) {
                    querySectionVO.setWhetherParticipation(true);
                }
            }
        }
        inProgress.setQuerySectionVOList(inProgressList);
        finished.setQuerySectionVOList(finishedList);
        beStar.setQuerySectionVOList(beStarList);
        resultList.add(inProgress);
        resultList.add(finished);
        resultList.add(beStar);

        return Result.ofSuccess(resultList);
    }

    @Override
    public Result<Object> updateEvaluateStu(Worker worker, EvaluateStuDTO evaluateStuDTO) {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }

        Optional<YcqQmUserCourse> byId = ycqQmUserCourseRepo.findById(evaluateStuDTO.getUserCourseId());
        if (byId.isPresent()){
            YcqQmUserCourse ycqQmUserCourse = byId.get();
            ycqQmUserCourse.setPresentationEvaluate(evaluateStuDTO.getPresentationEvaluate());
            ycqQmUserCourse.setEvaluateResult(evaluateStuDTO.getEvaluateResult());
            ycqQmUserCourse.setOperationCompletion(evaluateStuDTO.getOperationCompletion());
            ycqQmUserCourse.setEvaluateWorkerId(worker.getLoginId());
            ycqQmUserCourse.setEvaluateTime(new Date());
            ycqQmUserCourse.setUpdaterId(worker.getLoginId());
            ycqQmUserCourseRepo.save(ycqQmUserCourse);
            return Result.ofSuccess("评价成功");
        }else{
            throw new BusinessException("数据异常");
        }

    }

    @Override
    public Result<QueryEvaluateStuVO> queryEvaluateStu(Worker worker, Long courseId) {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker=worker1;
        }
        QueryEvaluateStuVO queryEvaluateStuVO = new QueryEvaluateStuVO();

        try{
            queryEvaluateStuVO = classesMgrDao.queryUserCourseVO(worker.getLoginId(), courseId);
        }catch (Exception e){
            throw new BusinessException("数据异常");
        }
        if (ObjectUtil.isNotEmpty(queryEvaluateStuVO)){
            QueryEvaluateStuVO queryEvaluateStuVO1 = classesMgrDao.queryClassesYearType(courseId);
            if (ObjectUtil.isNotEmpty(queryEvaluateStuVO1)){
                queryEvaluateStuVO.setClassesLevelName(queryEvaluateStuVO1.getClassesLevelName());
                queryEvaluateStuVO.setClassesTypeName(queryEvaluateStuVO1.getClassesTypeName());
                queryEvaluateStuVO.setClassesYear(queryEvaluateStuVO1.getClassesYear());
                queryEvaluateStuVO.setOrgId(queryEvaluateStuVO1.getOrgId());
                queryEvaluateStuVO.setOrgName(queryEvaluateStuVO1.getOrgName());
            }
        }

        return Result.ofSuccess(queryEvaluateStuVO);
    }

    /*private  <T> void fillYearTypeLevel(Class<T> t, Long courseId) throws NoSuchFieldException, IllegalAccessException {
        QueryEvaluateStuVO queryEvaluateStuVO1 = classesMgrDao.queryClassesYearType(courseId);
        if (ObjectUtil.isNotEmpty(queryEvaluateStuVO1)){
            Field classesLevelName = t.getField("classesLevelName");
            classesLevelName.setAccessible(true);
            classesLevelName.set(t,queryEvaluateStuVO1.getClassesLevelName());
            t.setClassesTypeName(queryEvaluateStuVO1.getClassesTypeName());
            t.setClassesYear(queryEvaluateStuVO1.getClassesYear());
        }
    }*/

    @Override
    public Result<Object> updateClassesStatus(Worker worker, UpdateClassesStatusDTO updateClassesStatusDTO ) {
        Integer i = 0;
        if (i.equals(updateClassesStatusDTO.getClassesStatus())){
            Integer integer = classesMgrDao.classesStuNum(updateClassesStatusDTO.getClassesId());
            if (!integer.equals(i)){
                throw new BusinessException("该班级下还有在读学员，不允许禁用");
            }
        }
        Optional<YcqQmClassesInfo> byId = ycqQmClassesInfoRepo.findById(updateClassesStatusDTO.getClassesId());

        if (byId.isPresent()){
            YcqQmClassesInfo ycqQmClassesInfo = byId.get();
            ycqQmClassesInfo.setClassesStatus(updateClassesStatusDTO.getClassesStatus());
            ycqQmClassesInfo.setUpdaterId(worker.getLoginId());
            ycqQmClassesInfo.setUpdateTime(new Date());
            ycqQmClassesInfoRepo.save(ycqQmClassesInfo);
        }else {
            throw new BusinessException("数据异常");
        }
        return Result.ofSuccess("修改成功");
    }

    @Override
    public Result<PageRsp<stuVO>> queryAddClassesStu(PageReq<QueryClassesStuDTO> req) {
        return Result.ofSuccess(classesMgrDao.queryAddClassesStu(req));
    }

    @Override
    public Result<QueryClassesVO> queryClassesDetail(Long classesId) {
        QueryClassesVO queryClassesVO = classesMgrDao.queryClassesDetail(classesId);
        // 注入学员
        /*if (ObjectUtil.isNotNull(queryClassesVO)){
                List<stuVO> stuVOS = classesMgrDao.queryClassesStu(queryClassesVO.getClassesId());
                queryClassesVO.setStuList(stuVOS);
        }*/
        return Result.ofSuccess(queryClassesVO);
    }

    @Override
    public Result<PageRsp<stuVO>> queryClassesStu(PageReq<QueryClassesStuDTO> req) {
        return Result.ofSuccess(classesMgrDao.queryClassesStu(req));
    }

    private String makeCode(String code) {
        SimpleDateFormat yyyyMMddHHmmss = new SimpleDateFormat("yyyyMMddHHmmss");
        String format = yyyyMMddHHmmss.format(new Date());
        String random = String.valueOf(new Random().nextInt(9000) + 1000);
        String courseCode = code + format + random;
        return courseCode;
    }


}
