package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.YcqQmClassesInfoRepo;
import com.crunii.micro.domain.repo.main.YcqQmSelfCultivationRepo;
import com.crunii.micro.domain.repo.main.YcqQmTeacherExpandRepo;
import com.crunii.micro.domain.repo.main.YcqQmUserClassesRelRepo;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.*;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.TeacherBindingListDto;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.ViewStudentDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.ExertDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.GroupCultivationDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.StuHonorDto;
import com.crunii.micro.service.qm.jdbc.CommonDao;
import com.crunii.micro.service.qm.service.CommonService;
import com.crunii.micro.service.qm.util.ListUtil;
import com.crunii.micro.service.qm.util.UserCardIdDecryptUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qm.api.constants.QmConstants.TEACHER_NOW;
import static com.crunii.micro.service.qm.api.constants.QmConstants.UNBOUND;

/**
 * @author xiayinqiang
 * @date 2023/9/21 0021 16:14:02
 */
@Service
public class CommonServiceImpl implements CommonService {

	@Value("${upload.url}")
	private String urlPrefix;

	@Autowired YcqQmUserClassesRelRepo ycqQmUserClassesRelRepo;
	@Autowired YcqQmClassesInfoRepo ycqQmClassesInfoRepo;
	@Resource private CommonDao commonDao;
	@Autowired YcqQmTeacherExpandRepo ycqQmTeacherExpandRepo;
	@Resource private YcqQmSelfCultivationRepo ycqQmSelfCultivationRepo;

	/**
	 * 保存用户与班级关联关系
	 *
	 * @param operatorId    操作人id
	 * @param loginId       用户id
	 * @param classesIdList 班级id集合
	 */
	@Override public void insertUserClassesRel(int operatorId, Integer loginId, List<Long> classesIdList) {
		Date curr = new Date();
		List<YcqQmClassesInfo> qmClassesInfoList = ycqQmClassesInfoRepo.findByClassesIdInAndDeleted(classesIdList, QmConstants.DELETED_0);
		if (!ListUtil.hasValue(qmClassesInfoList)) {
			throw new BusinessException("不存在任何一个班级");
		}
		//查询学员关联的所有班级
		List<YcqQmUserClassesRel> classesRelList = ycqQmUserClassesRelRepo.findByStuWorkerIdAndDeleted(loginId, QmConstants.DELETED_0);
		if (!CollectionUtils.isEmpty(classesIdList)) {
			ycqQmUserClassesRelRepo.deleteAll(classesRelList);
		}

		List<YcqQmUserClassesRel> relList = new ArrayList<>();
		for (YcqQmClassesInfo ycqQmClassesInfo : qmClassesInfoList) {
			YcqQmUserClassesRel ycqQmUserClassesRel = new YcqQmUserClassesRel();
			ycqQmUserClassesRel.setClassesId(ycqQmClassesInfo.getClassesId());
			ycqQmUserClassesRel.setStuWorkerId(loginId);
			ycqQmUserClassesRel.setCreatorId(operatorId);
			ycqQmUserClassesRel.setCreateTime(curr);
			ycqQmUserClassesRel.setUpdaterId(operatorId);
			ycqQmUserClassesRel.setUpdateTime(curr);
			ycqQmUserClassesRel.setDeleted(QmConstants.DELETED_0);

			relList.add(ycqQmUserClassesRel);
		}
		ycqQmUserClassesRelRepo.saveAllAndFlush(relList);
	}

	/**
	 * 保存用户与班级关联关系，一个班级多个学员
	 *
	 * @param operatorId  操作人id
	 * @param loginIdList 用户id集合
	 * @param classesId   班级id
	 */
	@Override public void insertUserClassesRel(int operatorId, List<Integer> loginIdList, Long classesId) {
		Date curr = new Date();
		YcqQmClassesInfo qmClassesInfo = ycqQmClassesInfoRepo.findByClassesIdAndDeleted(classesId, QmConstants.DELETED_0);
		if (ObjectUtil.isEmpty(qmClassesInfo)) {
			throw new BusinessException("未查询到对应班级");
		}

		commonDao.updateUserClasses(classesId);

		if(CollectionUtil.isNotEmpty(loginIdList)){
			List<YcqQmUserClassesRel> relList = new ArrayList<>();
			for (Integer loginId : loginIdList) {
				YcqQmUserClassesRel ycqQmUserClassesRel = new YcqQmUserClassesRel();
				ycqQmUserClassesRel.setClassesId(qmClassesInfo.getClassesId());
				ycqQmUserClassesRel.setStuWorkerId(loginId);
				ycqQmUserClassesRel.setCreatorId(operatorId);
				ycqQmUserClassesRel.setCreateTime(curr);
				ycqQmUserClassesRel.setUpdaterId(operatorId);
				ycqQmUserClassesRel.setUpdateTime(curr);
				ycqQmUserClassesRel.setDeleted(QmConstants.DELETED_0);
				relList.add(ycqQmUserClassesRel);
			}
			ycqQmUserClassesRelRepo.saveAllAndFlush(relList);
		}

	}

	@Override
	public PageRsp<WorkerHonorVo> getStuExpand(PageReq<StuExpandDto> req) {
		return commonDao.getStuExpand(req);
	}

	@Override
	public String getAdvantage(Integer workerId) {
		List<String> advantage = commonDao.getAdvantage(workerId);
		return String.join(";", advantage);
	}

	@Override
	public PageRsp<ConcentratedCultivationVo> pageConcentratedCultivationExpand(PageReq<StuExpandDto> pageReq) {
		return commonDao.pageConcentratedCultivationExpand(pageReq);
	}

	@Override
	public PageRsp<TeacherTraineeVo> pageStuTraineeExpand(PageReq<TrackingCultivationDto> pageReq) {
		return commonDao.pageStuTraineeExpand(pageReq);
	}

	/**
	 * 新增或修改导师级别
	 *
	 * @param operatorId 操作人id
	 * @param loginId    导师id
	 * @param levelCode  导师级别
	 */
	@Override public void insertTeacherExpand(int operatorId, int loginId, String levelCode) {
		Date curr = new Date();
		YcqQmTeacherExpand ycqQmTeacherExpand = ycqQmTeacherExpandRepo
				.findByWorkerIdAndDeleted(loginId, QmConstants.DELETED_0);
		if (ObjectUtil.isEmpty(ycqQmTeacherExpand)) {
			ycqQmTeacherExpand = new YcqQmTeacherExpand();
			ycqQmTeacherExpand.setWorkerId(loginId);
			ycqQmTeacherExpand.setLevelCode(levelCode);//班级级别
			ycqQmTeacherExpand.setCreatorId(operatorId);
			ycqQmTeacherExpand.setCreateTime(curr);
			ycqQmTeacherExpand.setUpdaterId(operatorId);
			ycqQmTeacherExpand.setUpdateTime(curr);
			ycqQmTeacherExpand.setDeleted(QmConstants.DELETED_0);
		} else {
			ycqQmTeacherExpand.setLevelCode(levelCode);//班级级别
			ycqQmTeacherExpand.setUpdaterId(operatorId);
			ycqQmTeacherExpand.setUpdateTime(curr);
		}
		ycqQmTeacherExpandRepo.saveAndFlush(ycqQmTeacherExpand);
	}

	@Override
	public SelfCultivationVo getSelfCultivation(TrackingCultivationDto trackingCultivationDto, Worker worker) {
		SelfCultivationVo selfCultivationVo = new SelfCultivationVo();
		Date now = new Date();
		// 获取学习强国积分
		LearIntegral integral = commonDao.findIntegralByWorkerId(trackingCultivationDto.getWorkerId());
		if (Objects.isNull(integral)) {
			YcqQmSelfCultivation ycqQmSelfCultivation = new YcqQmSelfCultivation();
			ycqQmSelfCultivation.setCultivationType(1);
			ycqQmSelfCultivation.setWorkerId(trackingCultivationDto.getWorkerId());
			ycqQmSelfCultivation.setIntegral(0);
			ycqQmSelfCultivation.setCreatorId(worker.getLoginId());
			ycqQmSelfCultivation.setCreateTime(now);
			ycqQmSelfCultivation.setUpdaterId(worker.getLoginId());
			ycqQmSelfCultivation.setUpdateTime(now);
			ycqQmSelfCultivation.setDeleted(0);
			YcqQmSelfCultivation ycqQmSelfCultivation1 = ycqQmSelfCultivationRepo.saveAndFlush(ycqQmSelfCultivation);
			integral = new LearIntegral();
			integral.setSelfCultivationId(ycqQmSelfCultivation1.getSelfCultivationId());
			integral.setUpdateTime(now);
			integral.setIntegral(0);
		}
		selfCultivationVo.setLearIntegral(integral);
		// 书籍
		List<WorkerBookVo> workerBookVos = commonDao.findBookInfoByWorkerId(trackingCultivationDto.getWorkerId());
		selfCultivationVo.setBookInfo(workerBookVos);
		return selfCultivationVo;
	}

	@Override
	public PageRsp<WorkerBookVo> pageBookRead(PageReq<StuExpandDto> pageReq) {
		return commonDao.pageBookRead(pageReq);
	}

	@Override
	public PageRsp<VolunteerVo> pageVolunteer(PageReq<StuExpandDto> pageReq) {
		return commonDao.pageVolunteerOrOther(pageReq, 3);
	}

	@Override
	public PageRsp<VolunteerVo> pageOther(PageReq<StuExpandDto> pageReq) {
		return commonDao.pageVolunteerOrOther(pageReq, 4);
	}

	@Override
	public PageRsp<ExertVo> pageExert(PageReq<ExertDto> req) {
		PageRsp<ExertVo> pageExertTwo = commonDao.pageExertTwo(req);
		List<ExertVo> result = pageExertTwo.getResult();
		if(CollectionUtils.isEmpty(result)){
			return pageExertTwo;
		}
		List<Long> collect = result.stream().map(ExertVo::getExertId).collect(Collectors.toList());
		List<ExertFileTypeDto> exertFileTypeDtos = commonDao.findPicVido(collect);
		if(CollectionUtils.isEmpty(exertFileTypeDtos)){
			return pageExertTwo;
		}
		// 添加视频图片地址
		List<String> collect2 = exertFileTypeDtos.stream().filter(x -> Objects.nonNull(x.getFileId())).map(ExertFileTypeDto::getFileId).collect(Collectors.toList());
		List<VideoPic> videoPicList = commonDao.findVideoPicList(collect2);
		Map<String, VideoPic> collect3 = videoPicList.stream().collect(Collectors.toMap(VideoPic::getFileId, Function.identity()));
		for (ExertFileTypeDto exertFileTypeDto : exertFileTypeDtos) {
			VideoPic videoPic = collect3.get(exertFileTypeDto.getFileId());
			exertFileTypeDto.setVideoPicUrl(Objects.isNull(videoPic)?null:videoPic.getPictureUrl());
			exertFileTypeDto.setFileName(Objects.isNull(videoPic)?null:videoPic.getFileName());
		}

		Map<Long, List<ExertFileTypeDto>> collect1 = exertFileTypeDtos.stream().collect(Collectors.groupingBy(ExertFileTypeDto::getExertId));
		for (ExertVo exertVo : result) {
			exertVo.setFileList(collect1.get(exertVo.getExertId()));
		}
		return pageExertTwo;
	}

	@Override
	public PageRsp<GrowthVo> pageGrowth(PageReq<GroupCultivationDto> req) {
		return commonDao.pageGrowth(req);
	}

	@Override
	public List<VolunteerVo> getVolunteer(StuHonorDto stuHonorDto) {
		return commonDao.getVolunteer(stuHonorDto);
	}

	@Override
	public List<GrowthPartyVo> findParty(TrackingCultivationDto dto) {
		return commonDao.findParty(dto);
	}

	@Override
	public void delTeacherInfo(List<Integer> teacherWorkerId,Worker worker) {
		Date nowTime = new Date();
		// 删除基本表
		commonDao.delUserByWorkerIds(teacherWorkerId,nowTime,worker.getLoginId());
		// 删除扩展表
		commonDao.delTeacherExpandByWorkerIds(teacherWorkerId,nowTime,worker.getLoginId());
		// 删除导师与学员关系表
		commonDao.delTeacherUser(teacherWorkerId,nowTime,worker.getLoginId(),true);
		// 删除拒绝表
		commonDao.delWorkerReject(teacherWorkerId,true);
		// 删除导师荣誉
		commonDao.delHonor(teacherWorkerId,nowTime,worker.getLoginId());
		// 删除导师评价表
		commonDao.delTutorTrainee(teacherWorkerId,nowTime,worker.getLoginId(),true);
	}

	@Override
	public void delWorkerInfo(List<Integer> stuWorkerIds,Worker worker) {
		Date nowTime = new Date();
		// 删除基本表
		commonDao.delUserByWorkerIds(stuWorkerIds,nowTime,worker.getLoginId());
		// 删除拒绝表
		commonDao.delWorkerReject(stuWorkerIds,false);
		// 导师与学员关联关系为已解绑
		commonDao.updateTeacherStu(stuWorkerIds,nowTime,worker.getLoginId());
		// 无需删除学员关联信息 可能后续会，暂时保留
	/*	// 删除学员班级关联表
		commonDao.delClassRelRecord(stuWorkerIds);
		// 删除学员荣誉
		commonDao.delHonor(stuWorkerIds,nowTime,worker.getLoginId());
		// 删除导师与学员关联关系
		commonDao.delTeacherUser(stuWorkerIds,nowTime,worker.getLoginId(),false);
		// 删除学员集中培养
		commonDao.delUserCourse(stuWorkerIds,worker.getLoginId());
		// 删除学员自我培养 todo 视频，图片，文件未处理
		commonDao.delSelfCultivation(stuWorkerIds,nowTime,worker.getLoginId());
		// 删除学员作用发挥
		commonDao.delExert(stuWorkerIds);
		// 删除学员成长发挥
		commonDao.delGroup(stuWorkerIds,nowTime,worker.getLoginId());
		// 删除导师评价表
		commonDao.delTutorTrainee(stuWorkerIds,nowTime,worker.getLoginId(),false);*/
	}

	@Override
	public List<TeacherAppBaseVo> findMyTeacher(Integer stuWorkerId) {
		List<TeacherAppBaseVo> myTeacher = commonDao.findMyTeacher(stuWorkerId);
		if(CollectionUtils.isEmpty(myTeacher)){
			return myTeacher;
		}
		List<Integer> collect = myTeacher.stream().map(TeacherAppBaseVo::getWorkerId).collect(Collectors.toList());
		Map<Integer, String> workerLevelName = getWorkerLevelName(collect);
		myTeacher.forEach(x->{
			x.setLevelName(workerLevelName.get(x.getWorkerId()));
			if(!x.isBinding()){
				x.setBindingTrans(UNBOUND);
			}else {
				x.setBindingTrans(TEACHER_NOW);
				x.setEndTime(null);
			}
			if(StringUtils.hasText(x.getHeadPortraitPicKey())){
				x.setHeadPortraitPicKey(urlPrefix+x.getHeadPortraitPicKey());
			}
			// 处理身份证号
			x.setCardId(UserCardIdDecryptUtil.decryptUserCardId(x.getCardId(),false));
		});
		return myTeacher;
	}

	@Override
	public void updateWorker(Integer loginId,String workerName, Integer sex) {
		commonDao.updateTsmWorker(workerName,sex,loginId);
	}

	/**
	 * 获取导师worker的级别名称
	 */
	private Map<Integer, String> getWorkerLevelName(List<Integer> collect) {
		List<WorkerIdLeveName> list = commonDao.getWorkerLevelName(collect);
		Map<Integer, String> collect1 = new HashMap<>(16);
		if(!CollectionUtils.isEmpty(list)){
			collect1 = list.stream().collect(Collectors.toMap(WorkerIdLeveName::getWorkerId, WorkerIdLeveName::getLevelName));
		}
		return collect1;
	}
}
