package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdcardUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQfAreaIndustry;
import com.crunii.micro.domain.entity.main.YcqQfLoanRecord;
import com.crunii.micro.domain.repo.main.YcqQfAreaIndustryRepo;
import com.crunii.micro.domain.repo.main.YcqQfLoanRecordRepo;
import com.crunii.micro.service.qm.api.domain.dto.qf.LoanPlanAuthDto;
import com.crunii.micro.service.qm.api.domain.dto.qf.LoanPlanAuthVo;
import com.crunii.micro.service.qm.api.domain.dto.qf.LoanPlanDetailVo;
import com.crunii.micro.service.qm.api.domain.dto.qf.LoanRecordDto;
import com.crunii.micro.service.qm.jdbc.QfPlanManageDao;
import com.crunii.micro.service.qm.service.QfPlanAuditService;
import com.crunii.micro.service.qm.service.counter.RedisCounter;
import com.crunii.micro.service.qm.util.CharacterLengthUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Objects;

import static com.crunii.micro.service.qm.api.constants.QmConstants.*;

/**
 * @author wangchen create 2023/11/9 10:22
 */
@Service
public class QfPlanAuditServiceImpl implements QfPlanAuditService {

    @Resource
    private YcqQfLoanRecordRepo ycqQfLoanRecordRepo;

    @Resource
    private YcqQfAreaIndustryRepo ycqQfAreaIndustryRepo;

    @Resource
    private QfPlanManageDao qfPlanManageDao;

    @Resource
    private RedisCounter redisCounter;

    @Override
    public String createPlan(LoanRecordDto loanRecord, Worker worker) {
        // 验证字符串长度
        validatedCharacter(loanRecord);
        // 查询当前身份证是否有数据
        Date now = new Date();
        int loginId = worker.getLoginId();
        YcqQfLoanRecord ycqQfLoanRecord = ycqQfLoanRecordRepo.findByIdCardAndHandle(loanRecord.getIdCard(), 0);
        if(Objects.isNull(ycqQfLoanRecord)){
            ycqQfLoanRecord = new YcqQfLoanRecord();
            ycqQfLoanRecord.setCreatorId(loginId);
            ycqQfLoanRecord.setCreateTime(now);
            String incrementStr;
            long increment = redisCounter.increment();
            if (increment < 10){
                incrementStr = "00" + increment;
            }else if (increment  < 100){
                incrementStr = "0" + increment;
            }else {
                incrementStr = increment + "";
            }
            ycqQfLoanRecord.setApplyCode(Long.parseLong(DateUtils.formatToDate(now) + incrementStr));
        }
        BeanUtil.copyProperties(loanRecord,ycqQfLoanRecord);
        ycqQfLoanRecord.setUpdaterId(loginId);
        ycqQfLoanRecord.setUpdateTime(now);
        ycqQfLoanRecord.setDeleted(0);
        ycqQfLoanRecordRepo.saveAndFlush(ycqQfLoanRecord);
        return "保存成功";
    }

    @Override
    public String createAreaIndustry(List<YcqQfAreaIndustry> ycqQfAreaIndustry) {
        ycqQfAreaIndustryRepo.saveAllAndFlush(ycqQfAreaIndustry);
        return "保存成功";
    }

    @Override
    public List<YcqQfAreaIndustry> findAreaIndustry(Integer areaIndustryType) {
        return ycqQfAreaIndustryRepo.findByAreaIndustryType(areaIndustryType);
    }

    @Override
    public String delAreaIndustry(Long areaIndustryId) {
        ycqQfAreaIndustryRepo.deleteById(areaIndustryId);
        ycqQfAreaIndustryRepo.flush();
        return "删除成功";
    }

    @Override
    public PageRsp<LoanPlanAuthVo> pageMyLoanStatistics(PageReq<LoanPlanAuthDto> dto, Worker worker) {
        return qfPlanManageDao.pageMyLoanStatistics(dto,worker.getLoginId());
    }

    @Override
    @InjectProperty
    public LoanPlanDetailVo getLoanPlanDetail(Long loanId, Worker worker) {
        LoanPlanDetailVo planDetail = qfPlanManageDao.findPlanDetail(loanId);
        int age = 0;
        if(Objects.isNull(planDetail)){
           throw new BusinessException("文章不存在");
        }
        try {
            age = IdcardUtil.getAgeByIdCard(planDetail.getIdCard());
        } catch (Exception ignore) {
        }
        planDetail.setAge(age);
        return planDetail;
    }

    private void validatedCharacter(LoanRecordDto loanRecord) {
        String loanName = loanRecord.getLoanName();
        String loanUse = loanRecord.getLoanUse();
        String companyName = loanRecord.getCompanyName();
        String notes = loanRecord.getNotes();
       if(CharacterLengthUtil.isExceedLength(loanName,QF_NAME_LENGTH)){
            throw new BusinessException("姓名长度不能超过"+QF_NAME_LENGTH+"字符");
       }
        if(CharacterLengthUtil.isExceedLength(loanUse,QF_USE_COMPANY_NAME_LENGTH)){
            throw new BusinessException("借贷用途不能超过"+QF_USE_COMPANY_NAME_LENGTH+"字符");
        }
        if(CharacterLengthUtil.isExceedLength(companyName,QF_USE_COMPANY_NAME_LENGTH)){
            throw new BusinessException("公司名称不能超过"+QF_USE_COMPANY_NAME_LENGTH+"字符");
        }
        if(CharacterLengthUtil.isExceedLength(notes,QF_NOTES_LENGTH)){
            throw new BusinessException("备注长度不能超过"+QF_NOTES_LENGTH+"字符");
        }
    }
}
