package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.BeanUtil;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.core.cache.TsmPropertyCache;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.LoanPlanImportTemp;
import com.crunii.micro.domain.entity.main.TsmPropertyValue;
import com.crunii.micro.domain.entity.main.YcqQfBank;
import com.crunii.micro.domain.entity.main.YcqQfLoanRecord;
import com.crunii.micro.domain.repo.main.LoanPlanImportTempRepo;
import com.crunii.micro.domain.repo.main.YcqQfBankRepo;
import com.crunii.micro.domain.repo.main.YcqQfLoanRecordRepo;
import com.crunii.micro.service.qm.api.domain.dto.ExcelDataInfo;
import com.crunii.micro.service.qm.api.domain.dto.ExcelMistakeData;
import com.crunii.micro.service.qm.api.domain.dto.MistakeDataInfo;
import com.crunii.micro.service.qm.api.domain.dto.qf.*;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.ImportStuDataDto;
import com.crunii.micro.service.qm.api.enums.SexEnum;
import com.crunii.micro.service.qm.executors.ImportLoanExecutor;
import com.crunii.micro.service.qm.jdbc.QfPlanManageDao;
import com.crunii.micro.service.qm.service.QfPlanManageService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qm.api.constants.QmConstants.*;

/**
 * @author wangchen create 2023/11/9 11:27
 */
@Service
@Slf4j
public class QfPlanManageServiceImpl implements QfPlanManageService {

    @Resource
    private QfPlanManageDao qfPlanManageDao;

    @Resource
    private YcqQfLoanRecordRepo ycqQfLoanRecordRepo;

    @Resource
    private TsmPropertyCache tsmPropertyCache;

    @Autowired
    private RedissonClient redissonClient;

    @Autowired
    private ImportLoanExecutor executor;

    @Resource
    private LoanPlanImportTempRepo loanPlanImportTempRepo;

    @Resource
    private YcqQfBankRepo ycqQfBankRepo;

    private static final String IMPORT_LOAN_EXCEL_LOCK = "import_loan_excel_lock";

    @Override
    @InjectProperty
    public PageRsp<LoanPlanDetailVo> pageLoanPlan(PageReq<LoanPlanDto> req) {
        PageRsp<LoanPlanDetailVo> rsp = qfPlanManageDao.pageLoanPlan(req);
        List<LoanPlanDetailVo> result = rsp.getResult();
        if (CollectionUtil.isEmpty(result)) {
            return rsp;
        }
        Date now = new Date();
        List<Long> idList = new ArrayList<>();
        for (LoanPlanDetailVo loanPlanVo : result) {
            int ageByIdCard = 0;
            try {
                ageByIdCard = IdcardUtil.getAgeByIdCard(loanPlanVo.getIdCard(), now);
            } catch (Exception e) {
                idList.add(loanPlanVo.getLoanId());
            }
            loanPlanVo.setAge(ageByIdCard);
        }
        if(!CollectionUtils.isEmpty(idList)){
            log.info("解析身份证号错误，id集合{}", JSONUtil.toJsonStr(idList));
        }
        return rsp;
    }

    @Override
    public String delLoanPlan(Long loanId, Worker worker) {
        if (Objects.isNull(loanId)) {
            throw new BusinessException("参数不能为空");
        }
        YcqQfLoanRecord one = ycqQfLoanRecordRepo.getOne(loanId);
        if (Objects.isNull(one)) {
            throw new BusinessException("无数据");
        }
        one.setDeleted(1);
        one.setUpdaterId(worker.getLoginId());
        one.setUpdateTime(new Date());
        ycqQfLoanRecordRepo.saveAndFlush(one);
        return "删除成功";
    }

    @Override
    public void exportLoanPlan(LoanPlanDto req) {
        List<LoanPlanExcelVo> loanPlanExcelVos = qfPlanManageDao.exportLoanPlan(req);
        if (CollectionUtil.isEmpty(loanPlanExcelVos)) {
            throw new BusinessException("无数据导出");
        }
        List<Long> loanIds = loanPlanExcelVos.stream().map(LoanPlanExcelVo::getLoanId).collect(Collectors.toList());
        qfPlanManageDao.batchUpdateHandle(loanIds);
        // 静态值翻译
        staticValueTrans(loanPlanExcelVos);
        // 导出
        try {
            ExcelUtil.writeSingleExcel("青锋计划审核表", null, loanPlanExcelVos, LoanPlanExcelVo.class);
        } catch (IOException e) {
            throw new BusinessException("导出失败");
        }
    }

    @Override
    public Result<ExcelDataInfo> importLoanPlanVerify(MultipartFile file) {
        LoanPlanImportVo excelData = new LoanPlanImportVo();
        ExcelDataInfo info = new ExcelDataInfo();
        List<LoanPlanImportVo> excelDataList;
        List<MistakeDataInfo> lists = new ArrayList<>();
        try {
            excelDataList = ExcelUtil.readSingleExcel(file, excelData, 1);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("导入失败");
        }
        if (CollectionUtil.isEmpty(excelDataList)) {
            throw new BusinessException("解析异常,查看数据是否规范");
        } else if (excelDataList.size() > EXCEL_MAX_RAW) {
            throw new BusinessException("超出最大导入条数");
        } else {
            List<LoanPlanImportVo> collect = excelDataList.stream().filter(item -> item.getLoanAmount() != null || StringUtils.isNotBlank(item.getLoanSuccessful())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(collect)) {
                throw new BusinessException("解析异常,查看数据是否规范");
            }
            lists = this.validParam(collect);
            if (CollectionUtils.isEmpty(lists)) {
                // 可以批量写入的数据
                List<LoanPlanImportTemp> tempList = new ArrayList<>();
                String dataKey = "loan" + UUIDUtil.getUUID();
                collect.forEach(e -> {
                    LoanPlanImportTemp ycqQmStuExcelTemp = new LoanPlanImportTemp();
                    BeanUtil.copyPropertiesIgnoreNull(e, ycqQmStuExcelTemp);
                    ycqQmStuExcelTemp.setDataKey(dataKey);
                    // 身份证第一次加密
                    tempList.add(ycqQmStuExcelTemp);
                });
                loanPlanImportTempRepo.saveAllAndFlush(tempList);
                info.setDataKey(dataKey);
            }
        }
        info.setLists(lists);
        return Result.ofSuccess(info);
    }

    @Override
    public Result<String> importLoanPlan(ImportStuDataDto dto) {
        String dataKey = dto.getDataKey();
        if (StringUtils.isBlank(dataKey)) {
            return Result.ofFail("参数不合法");
        }
        List<LoanPlanImportTemp> byDataKey = loanPlanImportTempRepo.findByDataKey(dataKey);
        if (CollectionUtils.isEmpty(byDataKey)) {
            return Result.ofFail("数据已被处理");
        }
        CompletableFuture.runAsync(() -> {
            byDataKey.forEach(item -> {
                qfPlanManageDao.updateLoanInfoByApplyCode(item);
            });
        }, executor.getExecutor());
        loanPlanImportTempRepo.deleteByDataKey(dataKey);
        Result<String> result = new Result<>();
        result.setSuccess(true);
        result.setData(String.valueOf(byDataKey.size()));
        result.setMsg("导入成功");
        return result;
    }

    @Override
    public LoanPlanSimpleVo getLoanStatistics() {
        List<LoanPlanSimpleVo.LoanUserInfo> list = qfPlanManageDao.getLatestLoanList();
        if (CollectionUtils.isEmpty(list)) {
            return new LoanPlanSimpleVo();
        }

        LoanPlanSimpleVo loanPlanSimpleVo = qfPlanManageDao.loanStatistics();

        loanPlanSimpleVo.setApplierCount(qfPlanManageDao.loanApplierTotal());

        BigDecimal divide = loanPlanSimpleVo.getLoanTotalAmount().setScale(2, RoundingMode.HALF_UP);
        loanPlanSimpleVo.setLoanTotalAmount(divide);
        loanPlanSimpleVo.setLoanUserInfo(list);
        list.forEach(item -> {
            String loanName = item.getLoanName();
            if (StringUtils.isNotBlank(loanName)) {
                String pattern = null;
                String replacement = null;
                if (loanName.length() <= 2) {
                    pattern = "(\\W)[\\w\\W]*";
                    replacement = "$1*";
                } else {
                    pattern = "(\\W)[\\w\\W]*(\\W)";
                    replacement = "$1*$2";
                }
                item.setLoanName(loanName.replaceAll(pattern, replacement));
            }
            BigDecimal loanAmount = item.getLoanAmount();
            if (null != loanAmount) {
                String bigDecimalStr = loanAmount.setScale(2, RoundingMode.HALF_UP).toString() + "万元";
                item.setLoanAmountStr(bigDecimalStr);
            }
        });
        return loanPlanSimpleVo;
    }

    @Override
    public List<YcqQfBankVo> queryBank(Long parentBankId) {
        return qfPlanManageDao.queryBank(parentBankId);
    }

    private List<MistakeDataInfo> validParam(List<LoanPlanImportVo> loanPlanImportVos) {
        List<Long> repeatableCode = loanPlanImportVos.stream().map(LoanPlanImportVo::getApplyCode).filter(Objects::nonNull)
                .collect(Collectors.groupingBy(code -> code, Collectors.counting()))
                .entrySet().stream()
                .filter(entry -> entry.getValue() > 1)
                .map(Map.Entry::getKey)
                .collect(Collectors.toList());
        List<MistakeDataInfo> arr = new ArrayList<>();
        for (int i = 0; i < loanPlanImportVos.size(); i++) {
            List<ExcelMistakeData> list = new ArrayList<>();
            ExcelMistakeData data = new ExcelMistakeData();
            data.setName("行数");
            data.setValue(String.valueOf(i + 2));
            data.setFlag(true);
            list.add(data);
            LoanPlanImportVo loanPlanImportVo = loanPlanImportVos.get(i);
            Long applyCode = loanPlanImportVo.getApplyCode();
            if (applyCode == null) {
                ExcelMistakeData mistakeInfo = getMistakeInfo("编号", "编号不能为空", false, 0);
                list.add(mistakeInfo);
            }
            BigDecimal loanAmount = loanPlanImportVo.getLoanAmount();
            String loanSuccessful = loanPlanImportVo.getLoanSuccessful();
            if (StringUtils.isNotBlank(loanSuccessful) && null == loanAmount) {
                ExcelMistakeData mistakeInfo = getMistakeInfo("放贷金额", "放贷金额不能为空", false, 0);
                list.add(mistakeInfo);
            }
            if (StringUtils.isBlank(loanSuccessful) && null != loanAmount) {
                ExcelMistakeData mistakeInfo = getMistakeInfo("是否贷款成功", "是否贷款成功不能为空", false, 0);
                list.add(mistakeInfo);
            }
            if (null != loanAmount && loanAmount.compareTo(BigDecimal.ZERO) < 0) {
                ExcelMistakeData mistakeInfo = getMistakeInfo("放贷金额", "放贷金额不能为负数", false, 0);
                list.add(mistakeInfo);
            }
            if (applyCode != null && repeatableCode.contains(applyCode)) {
                ExcelMistakeData mistakeInfo = getMistakeInfo("编号", "编号重复", false, 0);
                list.add(mistakeInfo);
            }
            List<Integer> state = list.stream().map(ExcelMistakeData::getState).collect(Collectors.toList());
            if (state.contains(0)) {
                MistakeDataInfo dataInfo = new MistakeDataInfo();
                dataInfo.setMistakeDataInfo(list);
                arr.add(dataInfo);
            }
        }
        return arr;
    }

    @Override
    public Result<ExcelDataInfo> saveByImport(MultipartFile file) {
        LoanPlanImportVo excelData = new LoanPlanImportVo();
        ExcelDataInfo info = new ExcelDataInfo();
        List<LoanPlanImportVo> excelDataList;
        List<MistakeDataInfo> lists = new ArrayList<>();
        // 银行数据量不大，查全部
        List<YcqQfBank> all = ycqQfBankRepo.findAll();
        Map<String, Long> bankMao = all.stream().collect(Collectors.toMap(YcqQfBank::getBankName, YcqQfBank::getBankId));
        try {
            excelDataList = ExcelUtil.readSingleExcel(file, excelData, 1);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("导入失败");
        }
        if (CollectionUtil.isEmpty(excelDataList)) {
            throw new BusinessException("解析异常,查看数据是否规范");
        } else if (excelDataList.size() > 1000) {
            throw new BusinessException("超出最大导入条数");
        } else {
            //  List<LoanPlanImportVo> collect = excelDataList.stream().filter(item -> item.getLoanAmount() != null || StringUtils.isNotBlank(item.getLoanSuccessful())).collect(Collectors.toList());
//            if (CollectionUtils.isEmpty(collect)){
//                throw new BusinessException("解析异常,查看数据是否规范");
//            }
//            lists = this.validParam(collect);
//            if (CollectionUtils.isEmpty(lists)){
            Map<String, Long> map = qfPlanManageDao.getMap();
            List<YcqQfLoanRecord> ycqQfLoanRecords = new ArrayList<>();
            Map<String, List<TsmPropertyValue>> propertyCacheMap = qfPlanManageDao.getPropertyMap();
            // 可以批量写入的数据
            excelDataList.forEach(item -> {
                YcqQfLoanRecord record = new YcqQfLoanRecord();
                record.setApplyCode(item.getApplyCode());

                record.setLoanName(item.getLoanName());
                if (Objects.equals("男", item.getLoanSexTrans())) {
                    record.setLoanSex(1);
                } else {
                    record.setLoanSex(0);
                }
                record.setIdCard(item.getIdCard());
                record.setPoliticalOutlook(this.strConvert(propertyCacheMap, "qf_political_outlook", item.getPoliticalOutlookTrans()));
                record.setLoanPhone(item.getLoanPhone());
                record.setLoanNature(this.strConvert(propertyCacheMap, "qf_loan_nature", item.getLoanNatureTrans()));
                record.setEduLevel(this.strConvert(propertyCacheMap, "qf_edu_level", item.getEduLevelTrans()));
                record.setEnterpriseType(this.strConvert(propertyCacheMap, "qf_enterprise_type", item.getEnterpriseTypeTrans()));
                if (record.getEnterpriseType() == null) {
                    record.setEnterpriseType(1);
                }
                record.setLoanType(this.strConvert(propertyCacheMap, "qf_loan_type", item.getEnterpriseTypeTrans()));
                if (record.getLoanType() == null) {
                    record.setLoanType(1);
                }
                record.setApplyAmount(Double.valueOf(item.getApplyAmount()));
                Long bankId = bankMao.get(item.getIntentionLoanBankTrans());
                record.setIntentionLoanBank(Objects.nonNull(bankId)?bankId:1);
                record.setLoanUse(item.getLoanUse());
                record.setCompanyName(item.getCompanyName());
                record.setAreaId(map.get(item.getAreaName()));
                record.setIndustry(map.get(item.getIndustryName()));
                record.setOperatingLiabilities(Double.valueOf(item.getOperatingLiabilities()));
                record.setLastYearIncome(Double.valueOf(item.getLastYearIncome()));
                record.setLastYearTaxation(Double.valueOf(item.getLastYearTaxation()));
                record.setStaffNumber(Integer.valueOf(item.getStaffNumber()));
                record.setCreateTime(item.getCreateTime());
                record.setCollegeStudentStaffNumber(Integer.valueOf(item.getCollegeStudentStaffNumber()));
                record.setLoanAmount(item.getLoanAmount());
                Long areaId = record.getAreaId();
                if (null == areaId) {
                    record.setAreaId(1L);
                }
                Long industry = record.getIndustry();
                if (null == industry) {
                    record.setIndustry(1L);
                }
                Long intentionLoanBank = record.getIntentionLoanBank();
                if (null == intentionLoanBank) {
                    record.setIntentionLoanBank(1L);
                }
                record.setNotes(item.getNotes());
                String loanSuccessful = item.getLoanSuccessful();
                if (Objects.equals(loanSuccessful, "是")) {
                    record.setLoanSuccessful(loanSuccessful);
                } else {
                    record.setLoanSuccessful("否");
                }
                record.setLoanAmount(item.getLoanAmount());
                ycqQfLoanRecords.add(record);
            });
            ycqQfLoanRecordRepo.saveAllAndFlush(ycqQfLoanRecords);
        }
//        }
        info.setLists(lists);
        return Result.ofSuccess(info);
    }

    @Override
    public String insertBank(List<YcqQfBank> req) {
        ycqQfBankRepo.saveAllAndFlush(req);
        return "添加成功";
    }

    @Override
    public void exportLoanPlanByIds(List<Long> loanIds) {
        List<LoanPlanExcelVo> loanPlanExcelVos = qfPlanManageDao.findPlanByLoanIds(loanIds);
        if (CollectionUtil.isEmpty(loanPlanExcelVos)) {
            throw new BusinessException("无数据导出");
        }
        List<Long> loanIdList = loanPlanExcelVos.stream().map(LoanPlanExcelVo::getLoanId).collect(Collectors.toList());
        qfPlanManageDao.batchUpdateHandle(loanIdList);
        // 静态值翻译
        staticValueTrans(loanPlanExcelVos);
        // 导出
        try {
            ExcelUtil.writeSingleExcel("青锋计划审核表", null, loanPlanExcelVos, LoanPlanExcelVo.class);
        } catch (IOException e) {
            throw new BusinessException("导出失败");
        }
    }

    private ExcelMistakeData getMistakeInfo(String name, String errText, boolean flag, int state) {
        ExcelMistakeData excelMistakeData = new ExcelMistakeData();
        excelMistakeData.setName(name);
        excelMistakeData.setErrText(errText);
        excelMistakeData.setFlag(flag);
        excelMistakeData.setState(state);
        return excelMistakeData;
    }

    /**
     * 静态展翻译
     */
    private void staticValueTrans(List<LoanPlanExcelVo> loanPlanExcelVos) {
        Date now = new Date();
        List<Long> ids = new ArrayList<>();
        for (LoanPlanExcelVo loanPlanExcelVo : loanPlanExcelVos) {
            int ageByIdCard = 0;
            try {
                ageByIdCard = IdcardUtil.getAgeByIdCard(loanPlanExcelVo.getIdCard(), now);
            } catch (Exception e) {
                ids.add(loanPlanExcelVo.getLoanId());
            }
            loanPlanExcelVo.setAge(ageByIdCard);
            loanPlanExcelVo.setLoanTypeTrans(tsmPropertyCache.get("qf_loan_type", loanPlanExcelVo.getLoanType().toString()));
            // 静态值翻译
            loanPlanExcelVo.setLoanSexTrans(SexEnum.getSexValue(loanPlanExcelVo.getLoanSex()));
            loanPlanExcelVo.setPoliticalOutlookTrans(tsmPropertyCache.get("qf_political_outlook", loanPlanExcelVo.getPoliticalOutlook().toString()));
            loanPlanExcelVo.setLoanNatureTrans(tsmPropertyCache.get("qf_loan_nature", loanPlanExcelVo.getLoanNature().toString()));
            loanPlanExcelVo.setEduLevelTrans(tsmPropertyCache.get("qf_edu_level", loanPlanExcelVo.getEduLevel().toString()));
            loanPlanExcelVo.setEnterpriseTypeTrans(tsmPropertyCache.get("qf_enterprise_type", loanPlanExcelVo.getEnterpriseType().toString()));
            // 增加后缀
            loanPlanExcelVo.setApplyAmount(loanPlanExcelVo.getApplyAmount() + QF_UNIT_RMB);
            loanPlanExcelVo.setOperatingLiabilities(loanPlanExcelVo.getOperatingLiabilities() + QF_UNIT_RMB);
            loanPlanExcelVo.setLastYearIncome(loanPlanExcelVo.getLastYearIncome() + QF_UNIT_RMB);
            loanPlanExcelVo.setLastYearTaxation(loanPlanExcelVo.getLastYearTaxation() + QF_UNIT_RMB);
            loanPlanExcelVo.setStaffNumber(loanPlanExcelVo.getStaffNumber() + QF_UNIT_PEOPLE);
            loanPlanExcelVo.setCollegeStudentStaffNumber(loanPlanExcelVo.getCollegeStudentStaffNumber() + QF_UNIT_PEOPLE);
        }
        if(!CollectionUtils.isEmpty(ids)){
            log.info("导出数据身份证异常,id->{}",JSONUtil.toJsonStr(ids));
        }
    }

    private Integer strConvert(Map<String, List<TsmPropertyValue>> propertyCacheMap, String propCode, String str) {
        if (StringUtils.isBlank(str)) {
            return null;
        }
        if (null == propertyCacheMap) {
            return null;
        }
        List<TsmPropertyValue> values = propertyCacheMap.get(propCode);
        if (!CollectionUtils.isEmpty(values)) {
            String s = values.stream().collect(Collectors.toMap(TsmPropertyValue::getValueName, TsmPropertyValue::getValueCode)).get(str);
            if (!StringUtils.isBlank(s)) {
                return Integer.parseInt(s);
            }
        }
        return null;
    }
}
