package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.BeanUtil;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.feign.client.SysService;
import com.crunii.micro.feign.client.dto.WorkerC;
import com.crunii.micro.feign.client.dto.WorkerCreateDTO;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.ViewTeacherDto;
import com.crunii.micro.service.qm.api.domain.dto.ExcelDataInfo;
import com.crunii.micro.service.qm.api.domain.dto.ExcelMistakeData;
import com.crunii.micro.service.qm.api.domain.dto.MistakeDataInfo;
import com.crunii.micro.service.qm.api.domain.dto.WorkerHonorVo;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.TeacherAppBaseVo;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ConcentratedCultivationVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.StuExpandDto;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.SelectUserHonorVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.UserHonorVo;
import com.crunii.micro.service.qm.jdbc.StudentManageWebDao;
import com.crunii.micro.service.qm.service.CommonService;
import com.crunii.micro.service.qm.service.StudentManageWebService;
import com.crunii.micro.service.qm.service.TeacherManageService;
import com.crunii.micro.service.qm.service.UserHonorService;
import com.crunii.micro.service.qm.util.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qm.api.constants.QmConstants.EXCEL_MAX_RAW;

/**
 * @author baoxy
 * @date 2023-09-18
 */
@Service
@Slf4j
public class StudentManageWebServiceImpl implements StudentManageWebService {

    @Resource
    private StudentManageWebDao studentManageWebDao;

    @Resource
    private TeacherManageService teacherManageService;

    @Resource
    private UserHonorService userHonorService;

    @Resource
    private YcqQmUserInfoRepo ycqQmUserInfoRepo;

    @Resource
    private YcqQmUserClassesRelRepo ycqQmUserClassesRelRepo;

    @Resource
    private YcqQmUserHonorRepo ycqQmUserHonorRepo;

    @Resource
    private YcqQmTeacherStuRepo ycqQmTeacherStuRepo;
    @Resource
    private YcqQmTeacherStuRejectRepo ycqQmTeacherStuRejectRepo;

    @Resource
    private YcqQmUserAuditRecordRepo ycqQmUserAuditRecordRepo;

    @Resource
    private SysService sysService;

    @Resource
    private YcqQmStuExpandRepo ycqQmStuExpandRepo;

    @Resource
    private ExcelStuVerify excelStuVerify;

    @Resource
    private TeacherManageServiceImpl teacherManageServiceImpl;

    @Resource
    private YcqQmStuExcelTempRepo ycqQmStuExcelTempRepo;

    @Resource
    private CommonService commonService;

    @Value("${upload.url}")
    private String urlPrefix;

    @Override
    public PageRsp<PassExamineVo> pageExamine(PageReq<PassExamineDto> req, Worker worker) {
        PageRsp<PassExamineVo> rsp = null;
        // 查询组织下所有组织
        List<Long> orgIdList = teacherManageServiceImpl.getOrgIdList(Long.valueOf(req.getQuery().getOrgId()));
        if (Objects.isNull(orgIdList)) {
            return new PageRsp<>();
        }

        if (QmConstants.AUDIT_STATUS_WAIT == req.getQuery().getAuditStatus() ||
                QmConstants.AUDIT_STATUS_NO == req.getQuery().getAuditStatus()) {
            rsp = studentManageWebDao.pageWaitExamine(req, orgIdList);
        }
        if (QmConstants.AUDIT_STATUS_YES == req.getQuery().getAuditStatus()) {
            rsp = studentManageWebDao.pagePassExamine(req, orgIdList);
            List<PassExamineVo> result = rsp.getResult();
            result.forEach(e -> {
                if (QmConstants.TEACHER_MATE_WAIT.equals(e.getTeacherMate())) {
                    e.setTeacherMateName("待审核");
                }
                if (QmConstants.TEACHER_MATE_NOT_SELECT.equals(e.getTeacherMate())) {
                    e.setTeacherMateName("未选择");
                }
                if (QmConstants.TEACHER_MATE_NOT.equals(e.getTeacherMate())) {
                    e.setTeacherMateName("未匹配");
                }
                if (QmConstants.TEACHER_MATE_YES.equals(e.getTeacherMate())) {
                    e.setTeacherMateName("已匹配");
                }
            });
            rsp.setResult(result);
        }
        assert rsp != null;
        List<PassExamineVo> result = rsp.getResult();

        // 查询出学员关联的班级列表
        result.forEach(e -> {
            List<ClassesInfoVo> classesVoList = studentManageWebDao.getClassesInfo(e.getUserInfoId());
            e.setClassesVoList(classesVoList);
            if (classesVoList.size() > 0) {
                // 2024-01-05更改:因为学员的班级列表已固定为1个，因此这里可编辑班级直接传列表唯一一个班级数据即可
                e.setUpdateClassesInfo(classesVoList.get(0));
                // 如果是超管账号，则取出等级最高的那个班级，dao层的sql语句已倒序，取出第一个即可
/*                if (QmConstants.SUPER_ADMIN_WORKER_ID == worker.getLoginId()) {
                    e.setUpdateClassesInfo(classesVoList.get(0));
                } else {
                    classesVoList.forEach(t -> {
                        // 当前用户的组织id要与班级组织id匹配才能
                        if (e.getOrgId().equals(t.getClassesOrgId())) {
                            // 可编辑的班级信息
                            e.setUpdateClassesInfo(t);
                        }
                    });
                }*/
            }
        });
        rsp.setResult(result);
        return rsp;
    }

    @Override
    @InjectProperty
    public ViewStudentVo viewStudent(ViewStudentDto dto, Worker worker) {
        ViewStudentVo viewStudentVo = studentManageWebDao.viewStudent(dto);
        // 给性别附上文字 1男0女
        viewStudentVo.setSexName(Objects.equals(viewStudentVo.getSex(), "1") ? QmConstants.SEX_MALE_NAME : QmConstants.SEX_FEMALE_NAME);
        // 身份证解密
        String decrypt = Sm2Util.decrypt(QmConstants.PRIVATE_KEY, false, viewStudentVo.getCardId());
        String decrypt1 = Sm2Util.decrypt(QmConstants.PRIVATE_WEB_KEY, true, decrypt);
        // 2023-09-29:与前端沟通后不再对身份证号进行脱敏处理
        viewStudentVo.setCardId(decrypt1);
        List<ClassesInfoVo> classesVoList = studentManageWebDao.getClassesInfo(dto.getUserInfoId());
        // 查询出学员关联的班级列表
        viewStudentVo.setClassesVoList(classesVoList);
        if (classesVoList.size() > 0) {
            // 2024-01-05更改:因为学员的班级列表已固定为1个，因此这里可编辑班级直接传列表唯一一个班级数据即可
            viewStudentVo.setUpdateClassesInfo(classesVoList.get(0));
            // 如果是超管账号，则取出等级最高的那个班级，dao层的sql语句已倒序，取出第一个即可
/*            if (QmConstants.SUPER_ADMIN_WORKER_ID == worker.getLoginId()) {
                viewStudentVo.setUpdateClassesInfo(classesVoList.get(0));
            } else {
                classesVoList.forEach(t -> {
                    // 当前用户的组织id要与班级组织id匹配才能
                    if (worker.getOrgId().equals(t.getClassesOrgId())) {
                        // 可编辑的班级信息
                        viewStudentVo.setUpdateClassesInfo(t);
                    }
                });
            }*/

        }

        // 取出最新的一条审核意见数据
        YcqQmUserAuditRecord latestNote = studentManageWebDao.latestNote(dto.getUserInfoId());
        if (Objects.nonNull(latestNote)) {
            viewStudentVo.setNote(latestNote.getNote());
        }
        // 调用查询荣誉service
        Result<SelectUserHonorVo> result = userHonorService.selectUserHonor(viewStudentVo.getWorkerId());
        SelectUserHonorVo selectUserHonorVo = result.getData();
        viewStudentVo.setAdeptField(selectUserHonorVo.getAdeptField());
        viewStudentVo.setHonorList(selectUserHonorVo.getUserHonorVoList());
        // 2023-10-09 11:06 新增学员详情查看结业证书的功能
        YcqQmStuExpand ycqQmStuExpand = ycqQmStuExpandRepo.findByWorkerIdAndDeleted(viewStudentVo.getWorkerId(), QmConstants.DELETED_0);
        if (Objects.nonNull(ycqQmStuExpand)) {
            if (StringUtils.isNotBlank(ycqQmStuExpand.getFileKey())) {
                viewStudentVo.setFileKey(ycqQmStuExpand.getFileKey());
                viewStudentVo.setFileUrl(urlPrefix + ycqQmStuExpand.getFileKey());
            }
        }
        return viewStudentVo;
    }

    @Override
    public Result<Object> createStudent(CreateStudentDto dto, Worker worker) {
        // 查询是否有当前用户
        Integer loginId = studentManageWebDao.findLoginIdByMobile(dto.getMobile());
        if (Objects.nonNull(loginId)) {
            // todo 临时 更新用户feign调用？青马注册更新tsm_worker？
            commonService.updateWorker(loginId, dto.getWorkerName(), dto.getSex());
            // 查询青马是否有该学员用户
            Integer exist = studentManageWebDao.findQmStudentExist(loginId);
            if (exist > 0) {
                return Result.ofFail("用户已存在");
            } else {
                createQmUser(dto, loginId, worker);
            }
        } else {
            // 用feign调用创建用户
            WorkerC workerC = new WorkerC();
            workerC.setMobile(dto.getMobile());
            workerC.setSex(dto.getSex());
            workerC.setWorkerName(dto.getWorkerName());
            workerC.setState(1);
            workerC.setOrgId(String.valueOf(dto.getOrgId()));
            Result<WorkerCreateDTO> createWorker = sysService.createWorker(workerC);
            if (!createWorker.isSuccess()) {
                log.info("创建用户失败，feign接口调用失败！参数->{}", JSONUtil.toJsonStr(workerC));
                throw new BusinessException("新增失败");
            }
            // 获得新增后的loginId
            Integer workerId = createWorker.getData().getLoginId();
            createQmUser(dto, workerId, worker);
        }
        return Result.ofSuccess("新增成功");
    }

    @Override
    public Result<Object> updateStudent(UpdateStudentDto dto, Worker worker) {
        YcqQmUserInfo info = ycqQmUserInfoRepo.getOne(dto.getUserInfoId());
        info.setPoliticalOutlook(dto.getPoliticalOutlook());
        info.setSex(dto.getSex());
        info.setAreaId(dto.getAreaId());
        info.setEducation(dto.getEducation());
        // 2023-09-29修改:前端会直接传加密后的身份证号数据,这里需要再加密一次
        info.setCardId(Sm2Util.encrypt(QmConstants.PUBLIC_KEY, false, dto.getCardId()));
        info.setWorkerJob(dto.getWorkerJob());
        info.setWorkUnit(dto.getWorkUnit());
        info.setUpdaterId(worker.getLoginId());
        info.setUpdateTime(new Date());
        // 修改学员关联的班级信息
        updateClasses(dto.getClassesId(), dto.getBeforeClassesId(), dto.getWorkerId(), worker);
        // 修改个人擅长
        if (StringUtils.isNotBlank(dto.getAdeptField())) {
            updateAdeptField(dto.getAdeptField(), dto.getWorkerId(), worker);
        }
        // 修改用户的荣誉信息列表
        if (Objects.nonNull(dto.getHonorList())) {
            updateUserHonor(dto.getHonorList(), dto.getWorkerId(), worker);
        }

        // 2023-10-09 15:03: 新增编辑信息的时候添加可编辑审核状态,方便前段调用
        if (Objects.nonNull(dto.getAuditStatus())) {
            info.setAuditStatus(dto.getAuditStatus());
            // 查询审核记录的信息
            List<YcqQmUserAuditRecord> findRecordList = ycqQmUserAuditRecordRepo.findAllByUserInfoId(dto.getUserInfoId());
            if (findRecordList.size() > 0) {
                findRecordList.forEach(t -> {
                    t.setUserInfoId(dto.getUserInfoId());
                    t.setAuditStatus(dto.getAuditStatus());
                    t.setAuditWorkerId(worker.getLoginId());
                    t.setNote(dto.getNote());
                    t.setAuditTime(new Date());
                });
                ycqQmUserAuditRecordRepo.saveAllAndFlush(findRecordList);
            } else {
                // 列表为空，则需要新增值
                YcqQmUserAuditRecord record = new YcqQmUserAuditRecord();
                record.setUserInfoId(dto.getUserInfoId());
                record.setAuditStatus(dto.getAuditStatus());
                record.setAuditWorkerId(worker.getLoginId());
                record.setNote(dto.getNote());
                record.setAuditTime(new Date());
                ycqQmUserAuditRecordRepo.saveAndFlush(record);
            }
        }

        return Result.ofSuccess("修改成功");
    }

    @Override
    public Result<Object> batchDelStudent(List<DelStudentDto> dtoList, Worker worker) {
        List<Long> idList = dtoList.stream().map(DelStudentDto::getUserInfoId).collect(Collectors.toList());
        List<YcqQmUserInfo> ycqQmUserInfoList = ycqQmUserInfoRepo.findAllByIdIn(idList);
        List<Integer> stuWorkerIds = ycqQmUserInfoList.stream().map(YcqQmUserInfo::getWorkerId).collect(Collectors.toList());
        commonService.delWorkerInfo(stuWorkerIds, worker);
//        List<YcqQmUserInfo> list = new ArrayList<>();
//        dtoList.forEach(e -> {
//            YcqQmUserInfo info = ycqQmUserInfoRepo.getOne(e.getUserInfoId());
//            info.setDeleted(QmConstants.DELETED_1);
//            info.setUpdaterId(worker.getLoginId());
//            info.setUpdateTime(new Date());
//            list.add(info);
//        });
//        //批量删除学员关联班级关系
//        List<Integer> collect = list.stream().map(YcqQmUserInfo::getWorkerId).collect(Collectors.toList());
//        studentManageWebDao.deleteClassesStu(collect);
//        // 批量删除学员关联导师的关系
//        studentManageWebDao.deleteTeacherStu(collect, worker.getLoginId());
//        ycqQmUserInfoRepo.saveAllAndFlush(list);
        return Result.ofSuccess("删除成功");
    }

    @Override
    public Result<Object> batchExamine(List<BatchExamineDto> dtoList, Worker worker) {
        List<YcqQmUserInfo> infoList = new ArrayList<>();
        List<YcqQmUserAuditRecord> recordList = new ArrayList<>();
        dtoList.forEach(e -> {
            YcqQmUserInfo info = ycqQmUserInfoRepo.getOne(e.getUserInfoId());
            info.setAuditStatus(e.getAuditStatus());
            info.setUpdaterId(worker.getLoginId());
            info.setUpdateTime(new Date());
            infoList.add(info);

            // 查询审核记录的信息
            List<YcqQmUserAuditRecord> findRecordList = ycqQmUserAuditRecordRepo.findAllByUserInfoId(e.getUserInfoId());
            if (findRecordList.size() > 0) {
                findRecordList.forEach(t -> {
                    t.setUserInfoId(e.getUserInfoId());
                    t.setAuditStatus(e.getAuditStatus());
                    t.setAuditWorkerId(worker.getLoginId());
                    t.setNote(e.getNote());
                    t.setAuditTime(new Date());
                });
                recordList.addAll(findRecordList);
            } else {
                // 若列表为空，则要新增值
                YcqQmUserAuditRecord record = new YcqQmUserAuditRecord();
                record.setUserInfoId(e.getUserInfoId());
                record.setAuditStatus(e.getAuditStatus());
                record.setAuditWorkerId(worker.getLoginId());
                record.setNote(e.getNote());
                record.setAuditTime(new Date());
                recordList.add(record);
            }
            // 推送工作流
    /*        if(EnvUtil.getEnvIsOnline()){
                teacherManageServiceImpl.pushFlow(e.getUserInfoId(), worker);
            }*/
        });
        ycqQmUserInfoRepo.saveAllAndFlush(infoList);
        ycqQmUserAuditRecordRepo.saveAllAndFlush(recordList);
        return Result.ofSuccess("审核成功");
    }

    @Override
    public SearchNumVo searchNum(SearchNumDto dto, Worker worker) {
        SearchNumVo vo = new SearchNumVo();
        // 查询组织下所有组织
        if (ObjectUtil.isEmpty(dto.getOrgId())) {
            dto.setOrgId(worker.getOrgId());
        }
        List<Long> orgIdList = teacherManageServiceImpl.getOrgIdList(Long.valueOf(dto.getOrgId()));
        if (Objects.isNull(orgIdList)) {
            return vo;
        }

        vo.setWaitExamine(studentManageWebDao.waitExamineCount(dto, orgIdList));
        vo.setNotPassExamine(studentManageWebDao.notPassExamineCount(dto, orgIdList));
        vo.setPassExamine(studentManageWebDao.passExamineCount(dto, orgIdList));
        vo.setBelowExcellent(studentManageWebDao.belowExcellentCount(dto, orgIdList));
        vo.setStuRefuseTwiceCount(studentManageWebDao.stuRefuseTwiceCount(dto, orgIdList));
        return vo;
    }

    @Override
    public void downLoadTemplate(HttpServletResponse response) {
        InputStream inputStream = null;
        OutputStream outputStream = null;
        try {
            inputStream = Thread.currentThread().getContextClassLoader().getResourceAsStream("templates/学员标准导入模版.xlsx");
            outputStream = response.getOutputStream();
            String fileName = "学员标准导入模版";
            // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
            String newFileName = null;
            try {
                newFileName = URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            // 当客户端请求的资源是一个可下载的资源（这里的“可下载”是指浏览器会弹出下载框或者下载界面）时，对这个可下载资源的描述（例如下载框中的文件名称）就是来源于该头域。
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + newFileName + ".xlsx");
            // 服务器告诉浏览器它发送的数据属于什么文件类型，也就是响应数据的MIME类型
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            // 关闭缓存（HTTP/1.1）
            response.setHeader("Cache-Control", "no-store");
            // 关闭缓存（HTTP/1.0）
            response.setHeader("Pragma", "no-cache");
            // 缓存有效时间
            response.setDateHeader("Expires", 0);
            IoUtil.copy(inputStream, outputStream);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            IoUtil.close(inputStream);
            IoUtil.close(outputStream);
        }
    }

    @Override
    public Result<ExcelDataInfo> verifyStuExcel(MultipartFile file) {
        ExcelStuData excelStuData = new ExcelStuData();
        ExcelDataInfo info = new ExcelDataInfo();
        List<ExcelStuData> excelStuDataList;
        List<MistakeDataInfo> lists = new ArrayList<>();
        List<ExcelMistakeData> dataList;
        try {
            excelStuDataList = ExcelUtil.readSingleExcel(file, excelStuData, 2);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("导入失败");
        }
        if (CollectionUtil.isEmpty(excelStuDataList)) {
            throw new BusinessException("解析异常,查看数据是否规范");
        } else if (excelStuDataList.size() > EXCEL_MAX_RAW) {
            throw new BusinessException("超出最大导入条数");
        }
        // 开始校验参数
        // 所有手机号
        List<String> phones = excelStuDataList.stream().map(ExcelStuData::getPhone).filter(Objects::nonNull).collect(Collectors.toList());
        // 所有身份证号
        List<String> idCards = excelStuDataList.stream().map(ExcelStuData::getIdCard).filter(Objects::nonNull).collect(Collectors.toList());
        for (ExcelStuData stuData : excelStuDataList) {
            dataList = excelStuVerify.data(stuData, String.valueOf(excelStuDataList.indexOf(stuData) + 3),
                    teacherManageServiceImpl.filterData(phones), teacherManageServiceImpl.filterData(idCards));
            if (CollectionUtil.isNotEmpty(dataList)) {
                MistakeDataInfo dataInfo = new MistakeDataInfo();
                dataInfo.setMistakeDataInfo(dataList);
                lists.add(dataInfo);
            }
        }
        // 校验通过保存临时表
        if (CollectionUtil.isEmpty(lists)) {
            List<YcqQmStuExcelTemp> tempList = new ArrayList<>();
            String dataKey = "stu" + UUIDUtil.getUUID();
            excelStuDataList.forEach(e -> {
                YcqQmStuExcelTemp ycqQmStuExcelTemp = new YcqQmStuExcelTemp();
                BeanUtil.copyPropertiesIgnoreNull(e, ycqQmStuExcelTemp);
                ycqQmStuExcelTemp.setDataKey(dataKey);
                // 身份证第一次加密
                ycqQmStuExcelTemp.setIdCard(Sm2Util.encrypt(QmConstants.PUBLIC_WEB_KEY, true, e.getIdCard()));
                tempList.add(ycqQmStuExcelTemp);
            });
            ycqQmStuExcelTempRepo.saveAllAndFlush(tempList);
            info.setDataKey(dataKey);
        }
        // 返回错误信息
        info.setLists(lists);
        return Result.ofSuccess(info);
    }

    @Override
    public Result<String> importStuData(ImportStuDataDto dto, Worker worker) {
        List<YcqQmStuExcelTemp> ycqQmStuExcelTempList = ycqQmStuExcelTempRepo.findAllByDataKey(dto.getDataKey());
        if (CollectionUtil.isEmpty(ycqQmStuExcelTempList)) {
            log.info("导入数据为空!!!");
            throw new BusinessException("导入失败,请联系管理员");
        }
        ycqQmStuExcelTempList.forEach(e -> {
            CreateStudentDto createStudentDto = new CreateStudentDto();
            createStudentDto.setWorkerName(e.getName());
            createStudentDto.setMobile(e.getPhone());
            createStudentDto.setCardId(e.getIdCard());
            // 组织
            createStudentDto.setOrgId(Long.valueOf(e.getAffiliation()));
            // 区域, 注意:这里值是区域id,不是中文名称
            createStudentDto.setAreaId(studentManageWebDao.getAreaId(e.getRegion()));
            createStudentDto.setPoliticalOutlook(Integer.valueOf(studentManageWebDao.
                    getValueCode(QmConstants.POLITICAL_OUTLOOK, e.getPoliticalAspect())));
            createStudentDto.setSex(QmConstants.SEX_MALE_NAME.equals(e.getSex()) ? 1 : 0);
            createStudentDto.setEducation(Integer.valueOf(studentManageWebDao.getValueCode(QmConstants.EDUCATION,
                    e.getHighestEducation())));
            createStudentDto.setWorkUnit(e.getWorkplace());
            createStudentDto.setWorkerJob(e.getOffice());
            YcqQmClassesInfo classesInfoList = studentManageWebDao.getClassesId(e.getClassesCode());
            if (Objects.nonNull(classesInfoList)) {
                createStudentDto.setClassesId(classesInfoList.getClassesId());
            }
            try {
                createStudent(createStudentDto, worker);
            } catch (Exception exception) {
                log.info("---------->导入导师失败,{}", exception.getMessage());
                throw new BusinessException("导入失败,请联系管理员");
            }
        });
        // 导入成功后删除临时表数据
        ycqQmStuExcelTempRepo.deleteAll(ycqQmStuExcelTempList);
        Result<String> result = new Result<>();
        result.setSuccess(true);
        result.setData(String.valueOf(ycqQmStuExcelTempList.size()));
        result.setMsg("导入成功");
        return result;
    }


    @Override
    public void exportPassExamine(ExportExamineDto dto) {
        List<PassExamineExcel> excelList = studentManageWebDao.exportPassExamine(dto);
        excelList.forEach(e -> {
            if (QmConstants.TEACHER_MATE_WAIT.equals(e.getTeacherMate())) {
                e.setTeacherMate("待审核");
            }
            if (QmConstants.TEACHER_MATE_NOT_SELECT.equals(e.getTeacherMate())) {
                e.setTeacherMate("未选择");
            }
            if (QmConstants.TEACHER_MATE_NOT.equals(e.getTeacherMate())) {
                e.setTeacherMate("未匹配");
            }
            if (QmConstants.TEACHER_MATE_YES.equals(e.getTeacherMate())) {
                e.setTeacherMate("已匹配");
            }
            // 将学员状态返回的数字设置为汉字导出
            if (QmConstants.DELETED_0 == Integer.parseInt(e.getStuState())) {
                e.setStuState("正常");
            } else {
                e.setStuState("已淘汰");
            }

            // 获取班级信息列表
            List<ClassesInfoVo> classesInfoVoList = studentManageWebDao.getClassesInfo(e.getUserInfoId());
            if (classesInfoVoList.size() > 0) {
                StringBuilder typeName = new StringBuilder();
                // 这里导出列表暂定为多个班级用逗号分隔
                classesInfoVoList.forEach(t -> typeName.append(t.getTypeName()).append(","));
                typeName.deleteCharAt(typeName.length() - 1);
                e.setTypeName(typeName.toString());
            }
        });
        try {
            ExcelUtil.writeSingleExcel("审核通过列表", null, excelList, PassExamineExcel.class);
        } catch (IOException e) {
            throw new BusinessException("导出失败");
        }
    }

    @Override
    @InjectProperty
    public List<TeacherAppBaseVo> viewTeacher(ViewTeacherDto dto) {
        return commonService.findMyTeacher(dto.getWorkerId());
    }

    @Override
    public PageRsp<ConcentratedCultivationVo> concentrateCulture(PageReq<StuExpandDto> req) {
        return teacherManageService.pageConcentratedCultivationExpand(req);
    }

    @Override
    public Result<Object> teacherSet(TeacherSetDto dto, Worker worker) {
        Date curr = new Date();
        // 导师设置规定只能绑定一个导师,因此需要将之前已绑定的导师的绑定状态设置为解绑
        List<YcqQmTeacherStu> teacherStuList = ycqQmTeacherStuRepo.findAllByStuWorkerIdAndBindingAndDeleted(dto.getStuWorkerId(),
                QmConstants.STATUS, QmConstants.DELETED_0);
        if (ListUtil.hasValue(teacherStuList)) {
            // 这里有绑定导师
            // 解除绑定后要将绑定状态设置为已解绑0
            teacherStuList.forEach(e -> {
                e.setBinding(QmConstants.TEACH_STU_BINDING_0);
                e.setUpdateTime(curr);
                e.setUpdaterId(worker.getLoginId());
            });
            ycqQmTeacherStuRepo.saveAllAndFlush(teacherStuList);
        }

        // 这里要判断页面的操作，可能是解绑已绑定的导师,这个时候前端不会传teacherWorkerId字段,因此不能新增关联表的数据
        if (Objects.nonNull(dto.getTeachWorkerId())) {
            YcqQmTeacherStu stu = new YcqQmTeacherStu();
            // 获取学生已关联的最高等级班级
            Long getOneClassesId = studentManageWebDao.getOneClassesId(dto.getStuWorkerId());
            if (Objects.nonNull(getOneClassesId)) {
                stu.setStuClassesId(getOneClassesId);
            }
            stu.setStuWorkerId(dto.getStuWorkerId());
            stu.setTeachWorkerId(dto.getTeachWorkerId());
            stu.setCorrelationState(QmConstants.USER_CORRELATION_STATE_1);
            // 选中状态 0 学生申请 1老师添加 ,2管理员添加,这里选2管理员添加
            stu.setChooseState(QmConstants.CHOOSE_STATE_2);
            // 是否绑定 0已解绑 1已绑定 这里选
            stu.setBinding(QmConstants.STATUS);
            stu.setCreatorId(worker.getLoginId());
            stu.setCreateTime(curr);
            stu.setUpdaterId(worker.getLoginId());
            stu.setUpdateTime(curr);
            stu.setDeleted(QmConstants.DELETED_0);
            stu.setStartTime(curr);
            ycqQmTeacherStuRepo.saveAndFlush(stu);

            //新增导师关联需要将该学员的导师拒绝记录表里面的记录删除
            List<YcqQmTeacherStuReject> teacherStuRejectList = ycqQmTeacherStuRejectRepo
                    .findAllByStuWorkerIdAndDeleted(dto.getStuWorkerId(), QmConstants.DELETED_0);
            if (ListUtil.hasValue(teacherStuRejectList)) {
                teacherStuRejectList.forEach(teacherStuReject -> {
                    teacherStuReject.setUpdaterId(worker.getLoginId());
                    teacherStuReject.setUpdateTime(curr);
                    teacherStuReject.setDeleted(QmConstants.DELETED_1);
                });
                ycqQmTeacherStuRejectRepo.saveAllAndFlush(teacherStuRejectList);
            }

        }
        return Result.ofSuccess("绑定成功");
    }

    @Override
    public List<TeacherListVo> teacherList(TeacherListDto dto) {
        String getOneLevelCode = studentManageWebDao.getOneLevelCode(dto.getStuWorkerId());
        if (ObjectUtil.isNotEmpty(dto.getLevelCode())) {
            getOneLevelCode = dto.getLevelCode();
        }
        return studentManageWebDao.teacherList(dto, getOneLevelCode);
    }

    @Override
    public List<TeacherListVo> bindingTeacherList(TeacherBindingListDto dto) {
        return studentManageWebDao.bindingTeacherList(dto);
    }

    @Override
    public Result<Object> shareClasses(ShareClassesDto dto, Worker worker) {
        // 超管账号不受orgId的限制
        updateClasses(dto.getClassesId(), dto.getBeforeClassesId(), dto.getWorkerId(), worker);
        return Result.ofSuccess("操作成功");
    }

    @Override
    public Boolean validUpdateStudent(UpdateStudentDto dto, Worker worker) {
        // 如果有值被修改则返回true,没有则返回false
        YcqQmUserInfo ycqQmUserInfo = ycqQmUserInfoRepo.getOne(dto.getUserInfoId());
        // 判断用户的基本信息
        boolean bool = validStudentBasicInfo(dto, ycqQmUserInfo);
        if (bool) {
            return true;
        }
        // 判断班级信息
        // 传值的班级id
        Long dtoClassesId = dto.getClassesId();
//        Long userInfoClassesId = studentManageWebDao.getClassesId(Long.valueOf(worker.getOrgId()), dto.getWorkerId());
        Long beforeClassesId = dto.getBeforeClassesId();

        // 判断班级是否有修改
        if (!dtoClassesId.equals(beforeClassesId)) {
            return true;
        }
        // 获取荣誉列表信息和擅长领域
        // 调用查询荣誉service
        Result<SelectUserHonorVo> result = userHonorService.selectUserHonor(ycqQmUserInfo.getWorkerId());
        String adeptField = result.getData().getAdeptField();
        List<UserHonorVo> userHonorVoList = result.getData().getUserHonorVoList();
        // 判断擅长领域
        if (!dto.getAdeptField().equals(adeptField)) {
            return true;
        }
        // 判断荣誉列表大小
        if (dto.getHonorList().size() != userHonorVoList.size()) {
            return true;
        }
        // 传值的荣誉列表
        List<WorkerHonorVo> dtoHonorList = new ArrayList<>();
        dto.getHonorList().forEach(e -> {
            WorkerHonorVo honorVo = new WorkerHonorVo();
            honorVo.setHonorDesc(e.getHonorDesc());
            honorVo.setAcquisitionTime(e.getAcquisitionTime());
            dtoHonorList.add(honorVo);
        });
        // 用户已有的荣誉列表
        List<WorkerHonorVo> userInfoHonorList = new ArrayList<>();
        userHonorVoList.forEach(e -> {
            WorkerHonorVo honorVo = new WorkerHonorVo();
            honorVo.setHonorDesc(e.getHonorDesc());
            honorVo.setAcquisitionTime(e.getAcquisitionTime());
            userInfoHonorList.add(honorVo);
        });
        // 获取按时间排序后的荣誉列表
        List<WorkerHonorVo> sortDtoHonorList = dtoHonorList.stream().sorted(
                (o1, o2) -> DateUtil.compare(o1.getAcquisitionTime(), o2.getAcquisitionTime())).collect(Collectors.toList());
        List<WorkerHonorVo> sortUserInfoHonorList =
                userInfoHonorList.stream().sorted(
                        (o1, o2) -> DateUtil.compare(o1.getAcquisitionTime(), o2.getAcquisitionTime())).collect(Collectors.toList());
        // 判断荣誉列表的数据是否有改动
        String dtoHonor = JSONUtil.toJsonStr(sortDtoHonorList);
        String userInfoHonor = JSONUtil.toJsonStr(sortUserInfoHonorList);
        // 若荣誉列表不相等，说明有改动，则返回ture,反之返回false
        return !dtoHonor.equals(userInfoHonor);
    }

    @Override
    public Result<Object> uploadCertificate(UploadCertificateDto dto, Worker worker) {
        YcqQmStuExpand ycqQmStuExpand = ycqQmStuExpandRepo.findByWorkerIdAndDeleted(dto.getWorkerId(), QmConstants.DELETED_0);
        // 判断:若有值，则在这个数据上修改即可.若没有值,则需要新增数据
        if (Objects.nonNull(ycqQmStuExpand)) {
            ycqQmStuExpand.setFileKey(dto.getFileKey());
            ycqQmStuExpand.setUpdaterId(worker.getLoginId());
            ycqQmStuExpand.setUpdateTime(new Date());
            ycqQmStuExpandRepo.saveAndFlush(ycqQmStuExpand);
        } else {
            YcqQmStuExpand newYcqQmStuExpand = new YcqQmStuExpand();
            newYcqQmStuExpand.setWorkerId(dto.getWorkerId());
            newYcqQmStuExpand.setFileKey(dto.getFileKey());
            newYcqQmStuExpand.setCreatorId(worker.getLoginId());
            newYcqQmStuExpand.setCreateTime(new Date());
            newYcqQmStuExpand.setUpdaterId(worker.getLoginId());
            newYcqQmStuExpand.setUpdateTime(new Date());
            newYcqQmStuExpand.setDeleted(QmConstants.DELETED_0);
            ycqQmStuExpandRepo.saveAndFlush(newYcqQmStuExpand);
        }
        return Result.ofSuccess("上传成功");
    }

    public void createQmUser(CreateStudentDto dto, Integer workerId, Worker worker) {
        YcqQmUserInfo info = new YcqQmUserInfo();
        info.setWorkerId(workerId);
        info.setSex(dto.getSex());
        // 身份为学员1
        info.setUserIdentity(QmConstants.USER_IDENTITY_STUDENT);
        info.setOrgId(dto.getOrgId());
        info.setAreaId(dto.getAreaId());
        info.setPoliticalOutlook(dto.getPoliticalOutlook());
        info.setEducation(dto.getEducation());
        // 加密身份证号
        // 2023-09-29修改:前端会直接传加密后的身份证号数据,这里需要再加密一次
        info.setCardId(Sm2Util.encrypt(QmConstants.PUBLIC_KEY, false, dto.getCardId()));
        info.setWorkUnit(dto.getWorkUnit());
        info.setWorkerJob(dto.getWorkerJob());
        // 2023-10-08 14:27修改:web端新增学员不需要审核,直接算审核通过
        info.setAuditStatus(QmConstants.AUDIT_STATUS_YES);
        info.setCreatorId(worker.getLoginId());
        info.setCreateTime(new Date());
        info.setUpdaterId(worker.getLoginId());
        info.setUpdateTime(new Date());
        info.setDeleted(QmConstants.DELETED_0);
        // 年度用系统当前年份即可
        LocalDate localDate = LocalDate.now();
        info.setUserCode(UserCodeUtil.getStuCode(localDate.getYear()));
        ycqQmUserInfoRepo.saveAndFlush(info);
        createUserHonor(dto.getHonorList(), dto.getAdeptField(), workerId, worker);
        List<YcqQmUserClassesRel> saveRelList = new ArrayList<>();
        YcqQmUserClassesRel ycqQmUserClassesRel = new YcqQmUserClassesRel();
        ycqQmUserClassesRel.setStuWorkerId(workerId);
        ycqQmUserClassesRel.setClassesId(dto.getClassesId());
        ycqQmUserClassesRel.setCreatorId(worker.getLoginId());
        ycqQmUserClassesRel.setCreateTime(new Date());
        ycqQmUserClassesRel.setUpdaterId(worker.getLoginId());
        ycqQmUserClassesRel.setUpdateTime(new Date());
        ycqQmUserClassesRel.setDeleted(QmConstants.DELETED_0);
        saveRelList.add(ycqQmUserClassesRel);
        ycqQmUserClassesRelRepo.saveAllAndFlush(saveRelList);
    }

    public void createUserHonor(List<CreateStudentHonorDto> honorList, String adeptField, Integer workerId, Worker worker) {
        // 个人擅长非必选,因此这里需要判断是否有值
        if (StringUtils.isNotBlank(adeptField)) {
            saveAdeptFieldHonor(adeptField, workerId, worker);
        }
        // 荣誉列表非必选,因此这里需要判断是否有值
        if (Objects.nonNull(honorList)) {
            honorList.forEach(e -> saveHonor(e.getAcquisitionTime(), e.getHonorDesc(), workerId, worker));
        }
    }

    public void updateClasses(Long classesId, Long beforeClassesId, Integer workerId, Worker worker) {
        // 查询一个组织下的班级关联关系
        Long relId = studentManageWebDao.getRelId(beforeClassesId, workerId);
        if (Objects.nonNull(relId)) {
            YcqQmUserClassesRel ycqQmUserClassesRel = ycqQmUserClassesRelRepo.getOne(relId);
            ycqQmUserClassesRel.setClassesId(classesId);
            ycqQmUserClassesRel.setUpdateTime(new Date());
            ycqQmUserClassesRel.setUpdaterId(worker.getLoginId());
            ycqQmUserClassesRelRepo.saveAndFlush(ycqQmUserClassesRel);
        }
    }

    public void updateAdeptField(String adeptField, Integer workerId, Worker worker) {
        // 修改擅长领域
        List<YcqQmUserHonor> adeptFieldHonor = ycqQmUserHonorRepo.findByWorkerIdAndHonorTypeAndDeleted(workerId,
                QmConstants.HONOR_TYPE_2, QmConstants.DELETED_0);
        if (Objects.nonNull(adeptFieldHonor)) {
            adeptFieldHonor.forEach(e -> {
                e.setHonorDesc(adeptField);
                e.setUpdateTime(new Date());
                e.setUpdaterId(worker.getLoginId());
            });
            // 若没有值，则新增擅长领域数据
            if (adeptFieldHonor.size() == 0) {
                YcqQmUserHonor ycqQmUserHonor = new YcqQmUserHonor();
                ycqQmUserHonor.setWorkerId(workerId);
                ycqQmUserHonor.setAcquisitionTime(new Date());
                ycqQmUserHonor.setHonorDesc(adeptField);
                ycqQmUserHonor.setDeleted(QmConstants.DELETED_0);
                ycqQmUserHonor.setHonorType(QmConstants.HONOR_TYPE_2);
                ycqQmUserHonor.setCreatorId(worker.getLoginId());
                ycqQmUserHonor.setCreateTime(new Date());
                ycqQmUserHonor.setUpdaterId(worker.getLoginId());
                ycqQmUserHonor.setUpdateTime(new Date());
                ycqQmUserHonorRepo.saveAndFlush(ycqQmUserHonor);
            }
        }
        ycqQmUserHonorRepo.saveAllAndFlush(adeptFieldHonor);
    }

    public void updateUserHonor(List<UpdateStudentHonorDto> honorList, Integer workerId, Worker worker) {
        // 先查询用户已有的荣誉列表
        List<YcqQmUserHonor> ycqQmUserHonors = ycqQmUserHonorRepo.findByWorkerIdAndHonorTypeAndDeleted(workerId, QmConstants.HONOR_TYPE_1,
                QmConstants.DELETED_0);

        //用户已有荣誉列表的id值列表
        List<Long> userHonorId = ycqQmUserHonors.stream().map(YcqQmUserHonor::getHonorId).collect(Collectors.toList());
        // 前端传来的荣誉id值列表
        List<Long> updateHonorId = new ArrayList<>();
        // 保存数据的列表
        List<YcqQmUserHonor> saveList = new ArrayList<>();
        honorList.forEach(e -> {
            if (Objects.nonNull(e.getHonorId())) {
                // 荣誉id值不为空,此时为编辑荣誉
                YcqQmUserHonor honor = ycqQmUserHonorRepo.getOne(e.getHonorId());
                honor.setHonorDesc(e.getHonorDesc());
                honor.setAcquisitionTime(e.getAcquisitionTime());
                honor.setUpdateTime(new Date());
                honor.setUpdaterId(worker.getLoginId());
                saveList.add(honor);
                updateHonorId.add(e.getHonorId());
            } else {
                // 若荣誉id值为空, 说明是新增的荣誉
                saveHonorList(saveList, e.getAcquisitionTime(), e.getHonorDesc(), workerId, worker);
            }
        });
        // 取出userHonorId中有但updateHonorId没有的id列表
        userHonorId.removeAll(updateHonorId);
        // 此时updateHonorId列表若有值,说明该值为需要逻辑删除的id值
        userHonorId.forEach(e -> {
            YcqQmUserHonor honor = ycqQmUserHonorRepo.getOne(e);
            honor.setDeleted(QmConstants.DELETED_1);
            honor.setUpdaterId(worker.getLoginId());
            honor.setUpdateTime(new Date());
            saveList.add(honor);
        });
        ycqQmUserHonorRepo.saveAllAndFlush(saveList);
    }

    public void saveHonor(Date acquisitionTime, String honorDesc, Integer workerId, Worker worker) {
        YcqQmUserHonor honor = new YcqQmUserHonor();
        honor.setWorkerId(workerId);
        honor.setAcquisitionTime(acquisitionTime);
        honor.setHonorDesc(honorDesc);
        honor.setCreatorId(worker.getLoginId());
        honor.setCreateTime(new Date());
        honor.setUpdaterId(worker.getLoginId());
        honor.setUpdateTime(new Date());
        honor.setDeleted(QmConstants.DELETED_0);
        // 荣誉类型：1荣誉2擅长领域
        honor.setHonorType(QmConstants.HONOR_TYPE_1);
        ycqQmUserHonorRepo.saveAndFlush(honor);
    }

    public void saveHonorList(List<YcqQmUserHonor> saveList, Date acquisitionTime, String honorDesc, Integer workerId, Worker worker) {
        YcqQmUserHonor honor = new YcqQmUserHonor();
        honor.setWorkerId(workerId);
        honor.setAcquisitionTime(acquisitionTime);
        honor.setHonorDesc(honorDesc);
        honor.setCreatorId(worker.getLoginId());
        honor.setCreateTime(new Date());
        honor.setUpdaterId(worker.getLoginId());
        honor.setUpdateTime(new Date());
        honor.setDeleted(QmConstants.DELETED_0);
        // 荣誉类型：1荣誉2擅长领域
        honor.setHonorType(QmConstants.HONOR_TYPE_1);
        saveList.add(honor);
    }

    public void saveAdeptFieldHonor(String adeptField, Integer workerId, Worker worker) {
        YcqQmUserHonor adeptFieldHonor = new YcqQmUserHonor();
        adeptFieldHonor.setWorkerId(workerId);
        adeptFieldHonor.setAcquisitionTime(new Date());
        adeptFieldHonor.setHonorDesc(adeptField);
        adeptFieldHonor.setCreatorId(worker.getLoginId());
        adeptFieldHonor.setCreateTime(new Date());
        adeptFieldHonor.setUpdaterId(worker.getLoginId());
        adeptFieldHonor.setUpdateTime(new Date());
        adeptFieldHonor.setDeleted(QmConstants.DELETED_0);
        // 荣誉类型：1荣誉2擅长领域
        adeptFieldHonor.setHonorType(QmConstants.HONOR_TYPE_2);
        ycqQmUserHonorRepo.saveAndFlush(adeptFieldHonor);
    }

    public Boolean validStudentBasicInfo(UpdateStudentDto dto, YcqQmUserInfo ycqQmUserInfo) {
        // 判断身份证信息
        //前端会直接传加密后的身份证号数据,这里需要解密
        String dtoCardId = Sm2Util.decrypt(QmConstants.PRIVATE_WEB_KEY, true, dto.getCardId());
        // 查询用户已有的身份证信息, 并进行两次解密
        String decrypt = Sm2Util.decrypt(QmConstants.PRIVATE_KEY, false, ycqQmUserInfo.getCardId());
        String userInfoCardId = Sm2Util.decrypt(QmConstants.PRIVATE_WEB_KEY, true, decrypt);
        // 将传来的身份证信息和用户已有信息进行校验, 若值不一样说明被修改
        if (!dtoCardId.equals(userInfoCardId)) {
            return true;
        }
        // 判断性别,这里传值和数据库的性别数据都是静态值数字
        if (!dto.getSex().equals(ycqQmUserInfo.getSex())) {
            return true;
        }
        // 判断区域id
        if (!dto.getAreaId().equals(ycqQmUserInfo.getAreaId())) {
            return true;
        }
        // 判断最高学历,这里传值和数据库的最高学历数据都是静态值数字
        if (!dto.getEducation().equals(ycqQmUserInfo.getEducation())) {
            return true;
        }
        // 判断政治面貌,这里传值和数据库的政治面貌数据都是静态值数字
        if (!dto.getPoliticalOutlook().equals(ycqQmUserInfo.getPoliticalOutlook())) {
            return true;
        }
        // 判断学员职务
        if (!dto.getWorkerJob().equals(ycqQmUserInfo.getWorkerJob())) {
            return true;
        }
        // 判断工作单位
        return !dto.getWorkUnit().equals(ycqQmUserInfo.getWorkUnit());
    }
}
