package com.crunii.micro.service.qm.service.impl;
import java.util.*;

import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQmFlowRecord;
import com.crunii.micro.domain.entity.main.YcqQmTeacherStu;
import com.crunii.micro.domain.entity.main.YcqQmTeacherStuReject;
import com.crunii.micro.domain.entity.main.YcqQmTutorEvaluate;
import com.crunii.micro.domain.repo.main.YcqQmFlowRecordRepo;
import com.crunii.micro.domain.repo.main.YcqQmTeacherStuRejectRepo;
import com.crunii.micro.domain.repo.main.YcqQmTeacherStuRepo;
import com.crunii.micro.domain.repo.main.YcqQmTutorEvaluateRepo;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.ExertDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.GroupCultivationDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.StuHonorDto;
import com.crunii.micro.service.qm.api.enums.EvaluateResultEnum;
import com.crunii.micro.service.qm.api.enums.OperationCompletionEnum;
import com.crunii.micro.service.qm.api.enums.StuAuditFlowEnum;
import com.crunii.micro.service.qm.jdbc.TeacherAppDao;
import com.crunii.micro.service.qm.service.CommonService;
import com.crunii.micro.service.qm.service.TeacherAppService;
import com.crunii.micro.service.qm.util.EnvUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;

/**
 * @author wangchen create 2023/9/19 11:15
 */
@Slf4j
@Service
public class TeacherAppServiceImpl implements TeacherAppService {

    @Value("${upload.url}")
    private String urlPrefix;

    @Resource
    private CommonService commonService;

    @Resource
    private YcqQmFlowRecordRepo ycqQmFlowRecordRepo;

    @Resource
    private YcqQmTeacherStuRejectRepo ycqQmTeacherStuRejectRepo;

    @Resource
    private YcqQmTeacherStuRepo ycqQmTeacherStuRepo;

    @Resource
    private YcqQmTutorEvaluateRepo ycqQmTutorEvaluateRepo;

    private final TeacherAppDao teacherAppDao;

    public TeacherAppServiceImpl(TeacherAppDao teacherAppDao) {
        this.teacherAppDao = teacherAppDao;
    }

    @Override
    @InjectProperty
    public PageRsp<MyStuBaseVo> pageAuditMyStu(PageReq<MyStuBaseDto> req,Worker worker) {
        if(Objects.isNull(req.getQuery().getCorrelationState())){
            throw new BusinessException("状态不能为空");
        }
        PageRsp<MyStuBaseVo> rsp = teacherAppDao.pageAuditMyStu(req, worker);
        List<MyStuBaseVo> result = rsp.getResult();
        if(CollectionUtils.isEmpty(result)){
            return rsp;
        }
        result.forEach(x->{
            if(StringUtils.hasText(x.getHeadUrl())){
                x.setHeadUrl(urlPrefix+x.getHeadUrl());
            }
        });
        return rsp;
    }

    @Override
    public PageRsp<WorkerHonorVo> pageHonor(PageReq<StuExpandDto> pageReq) {
        return commonService.getStuExpand(pageReq);
    }

    @Override
    public String getAdvantage(StuExpandDto stuExpandDto) {
        return commonService.getAdvantage(stuExpandDto.getWorkerId());
    }

    @Override
    public String updateStuAudit(StuAuditDto stuAuditDto, Worker worker) {
        Integer correlationState = stuAuditDto.getCorrelationState();
        if(!QmConstants.PASS_STATE.equals(correlationState) && !QmConstants.REJECTED_STATE.equals(correlationState)){
            throw new BusinessException("状态有误");
        }
        Date now = new Date();
        Integer userWorkerId = stuAuditDto.getWorkerId();
        int teacherWorkerId = worker.getLoginId();
        // 当前用户是否申请该导师
        Long tutorTraineeId = teacherAppDao.findIsAudit(userWorkerId,teacherWorkerId);
        if(Objects.isNull(tutorTraineeId)){
            throw new BusinessException("该学生未申请");
        }
        // 更新用户状态
        YcqQmTeacherStu one = ycqQmTeacherStuRepo.getOne(tutorTraineeId);
        one.setCorrelationState(correlationState);
        one.setUpdaterId(worker.getLoginId());
        one.setUpdateTime(now);
        one.setStartTime(QmConstants.REJECTED_STATE.equals(correlationState)?null:now);
        ycqQmTeacherStuRepo.saveAndFlush(one);
        // 如果已拒绝
        if(QmConstants.REJECTED_STATE.equals(correlationState)){
            YcqQmTeacherStuReject ycqQmTeacherStuReject = new YcqQmTeacherStuReject();
            ycqQmTeacherStuReject.setTeachWorkerId(worker.getLoginId());
            ycqQmTeacherStuReject.setStuWorkerId(userWorkerId);
            ycqQmTeacherStuReject.setCreatorId(worker.getLoginId());
            ycqQmTeacherStuReject.setCreateTime(now);
            ycqQmTeacherStuReject.setUpdaterId(worker.getLoginId());
            ycqQmTeacherStuReject.setUpdateTime(now);
            ycqQmTeacherStuReject.setDeleted(0);
            ycqQmTeacherStuRejectRepo.saveAndFlush(ycqQmTeacherStuReject);
        }else {
            // 查询已添加学员个数
            Integer count = teacherAppDao.findStuCount(worker.getLoginId());
            if(count > QmConstants.TEACHER_MAX_STU){
                throw new BusinessException("导师已关联5个学员");
            }
            // 删除已拒绝记录
            List<YcqQmTeacherStuReject> byTeachWorkerIdAndStuWorkerIdAndDeleted = ycqQmTeacherStuRejectRepo.findByTeachWorkerIdAndStuWorkerIdAndDeleted(worker.getLoginId(), userWorkerId, 0);
            if(!CollectionUtils.isEmpty(byTeachWorkerIdAndStuWorkerIdAndDeleted)){
                for (YcqQmTeacherStuReject ycqQmTeacherStuReject : byTeachWorkerIdAndStuWorkerIdAndDeleted) {
                    ycqQmTeacherStuReject.setDeleted(1);
                }
                ycqQmTeacherStuRejectRepo.saveAllAndFlush(byTeachWorkerIdAndStuWorkerIdAndDeleted);
            }
        }
        // 查询审核实例id
        YcqQmFlowRecord byServiceId = ycqQmFlowRecordRepo.findByServiceId(tutorTraineeId);
/*        if(Objects.isNull(byServiceId)){
            throw new BusinessException("流程未到达");
        }*/
        if(EnvUtil.getEnvIsOnline()){
//            pushFlow(worker, correlationState, byServiceId);
        }
        return "更新成功";
    }

/*    private void pushFlow(Worker worker, Integer correlationState, YcqQmFlowRecord byServiceId) {
        // 远程审核推进流程
        FlowCommitDTO flowCommitDTO = new FlowCommitDTO();
        FlowCommitorInfoDTO flowCommitorInfoDTO = new FlowCommitorInfoDTO();
        flowCommitorInfoDTO.setUserId((long) worker.getLoginId());
        flowCommitorInfoDTO.setUserName(worker.getWorkerName());
        String orgId = worker.getOrgId();
        flowCommitorInfoDTO.setOrgId(Objects.nonNull(orgId)?Long.valueOf(orgId):null);
        flowCommitorInfoDTO.setOrgName(worker.getOrgName());
        flowCommitDTO.setFlowInstanceId(byServiceId.getFlowInstanceId());
        flowCommitDTO.setTaskInstanceId(byServiceId.getNodeInstanceId());
        flowCommitDTO.setCommitor(flowCommitorInfoDTO);
        if(QmConstants.REJECTED_STATE.equals(correlationState)){
            flowCommitDTO.setProcessResult(StuAuditFlowEnum.getAuditDesc(QmConstants.STU_AUDIT_REJECT));
        }else {
            flowCommitDTO.setProcessResult(StuAuditFlowEnum.getAuditDesc(QmConstants.STU_AUDIT_PASS));
        }
        CommonResult<CruniiCommitTaskResult> result = flowInstanceApi.commitProcess(flowCommitDTO);
        if(!result.isSuccess()){
            log.info("调用推进流程失败,返回-》{}-参数-》{}", JSONUtil.toJsonStr(result),JSONUtil.toJsonStr(flowCommitDTO));
            throw new BusinessException("审核流程调用失败");
        }
    }*/

    @Override
    public String addEvaluateStu(EvaluateStuDto evaluateStuDto, Worker worker) {
        Date now = new Date();
        Long classesId = teacherAppDao.findClassesId(evaluateStuDto.getWorkerId());
        YcqQmTutorEvaluate ycqQmTutorEvaluate = new YcqQmTutorEvaluate();
        ycqQmTutorEvaluate.setStuWorkerId(evaluateStuDto.getWorkerId());
        ycqQmTutorEvaluate.setTeachWorkerId(worker.getLoginId());
        ycqQmTutorEvaluate.setClassesId(classesId);
        ycqQmTutorEvaluate.setEvaluateContext(evaluateStuDto.getEvaluateContext());
        ycqQmTutorEvaluate.setCreatorId(worker.getLoginId());
        ycqQmTutorEvaluate.setCreateTime(now);
        ycqQmTutorEvaluate.setUpdaterTime(now);
        ycqQmTutorEvaluate.setUpdaterId(worker.getLoginId());
        ycqQmTutorEvaluate.setDeleted(0);
        ycqQmTutorEvaluateRepo.saveAndFlush(ycqQmTutorEvaluate);
        // 修改状态为已评价
        teacherAppDao.updateState(evaluateStuDto.getWorkerId(),worker.getLoginId());
        return "评价成功";
    }

    @Override
    public String delMyStu(Integer workerId,Worker worker) {
        teacherAppDao.delMyStuBinding(workerId,worker.getLoginId());
        return "删除成功";
    }

    @Override
    public PageRsp<ConcentratedCultivationVo> pageConcentratedCultivationExpand(PageReq<StuExpandDto> pageReq) {
        PageRsp<ConcentratedCultivationVo> concentratedCultivationVoPageRsp = commonService.pageConcentratedCultivationExpand(pageReq);
        List<ConcentratedCultivationVo> result = concentratedCultivationVoPageRsp.getResult();
        if(CollectionUtils.isEmpty(result)){
            return concentratedCultivationVoPageRsp;
        }
        for (ConcentratedCultivationVo concentratedCultivationVo : result) {
            Integer evaluateResult = concentratedCultivationVo.getEvaluateResult();
            concentratedCultivationVo.setEvaluateResultTrans(EvaluateResultEnum.getTrans(evaluateResult));
            Integer operationCompletion = concentratedCultivationVo.getOperationCompletion();
            concentratedCultivationVo.setOperationCompletionTrans(OperationCompletionEnum.getTrans(operationCompletion));
        }
        return concentratedCultivationVoPageRsp;
    }

    @Override
    public PageRsp<TeacherTraineeVo> pageStuTraineeExpand(PageReq<TrackingCultivationDto> pageReq,Worker worker) {
        return commonService.pageStuTraineeExpand(pageReq);
    }


    @Override
    public SelfCultivationVo getSelfCultivation(TrackingCultivationDto trackingCultivationDto, Worker worker) {
        return commonService.getSelfCultivation(trackingCultivationDto,worker);
    }

    @Override
    @InjectProperty
    public PageRsp<ExertVo> pageExert(PageReq<TrackingCultivationDto> pageReq, Worker worker) {
        TrackingCultivationDto query = pageReq.getQuery();
        ExertDto exertDto = new ExertDto();
        exertDto.setWorkerId(query.getWorkerId());
        exertDto.setStartTime(query.getStartTime());
        exertDto.setEndTime(query.getEndTime());

        PageReq<ExertDto> req = new PageReq<>();
        req.setQuery(exertDto);
        req.setPageNo(pageReq.getPageNo());
        req.setPageSize(pageReq.getPageSize());
        req.setOrderProps(pageReq.getOrderProps());
        return commonService.pageExert(req);
    }

    @Override
    @InjectProperty
    public PageRsp<GrowthVo> pageGrowth(PageReq<TrackingCultivationDto> trackingCultivationDto, Worker worker) {
        TrackingCultivationDto query = trackingCultivationDto.getQuery();
        GroupCultivationDto groupCultivationDto = new GroupCultivationDto();
        groupCultivationDto.setWorkerId(query.getWorkerId());
        groupCultivationDto.setStartTime(query.getStartTime());
        groupCultivationDto.setEndTime(query.getEndTime());

        PageReq<GroupCultivationDto> req = new PageReq<>();
        req.setQuery(groupCultivationDto);
        req.setPageNo(trackingCultivationDto.getPageNo());
        req.setPageSize(trackingCultivationDto.getPageSize());
        req.setOrderProps(trackingCultivationDto.getOrderProps());
        return commonService.pageGrowth(req);
    }

    @Override
    @InjectProperty
    public PageRsp<MyStuBaseVo> pageNoTeacherStu(PageReq<MyStuBaseDto> req,Worker worker) {
        // 查询导师级别
        String levelCode = teacherAppDao.findTeacherLevelCode(worker.getLoginId());
        PageRsp<MyStuBaseVo> rsp = teacherAppDao.pageNoTeacherStu(req,levelCode);
        List<MyStuBaseVo> result = rsp.getResult();
        if(CollectionUtils.isEmpty(result)){
            return rsp;
        }
        result.forEach(x->{
            if(StringUtils.hasText(x.getHeadUrl())){
                x.setHeadUrl(urlPrefix+x.getHeadUrl());
            }
        });
        return rsp;
    }

    @Override
    @InjectProperty
    public List<VolunteerVo> getVolunteer(StuExpandDto stuExpandDto) {
        StuHonorDto stuHonorDto = new StuHonorDto();
        stuHonorDto.setWorkerId(stuExpandDto.getWorkerId());
        return commonService.getVolunteer(stuHonorDto);
    }

    @Override
    public String addStuList(List<Integer> workerIds,Worker worker) {
        Integer stuCount = teacherAppDao.findStuCount(worker.getLoginId());
        if(stuCount+workerIds.size()>QmConstants.TEACHER_MAX_STU){
            throw new BusinessException("已存在5名学员");
        }
        teacherAppDao.delTeacherReject(workerIds);
        Date date = new Date();
        List<YcqQmTeacherStu> list = new ArrayList<>();
        for (Integer workerId : workerIds) {
            // 查询学员是否有申请记录
            Long isAudit = teacherAppDao.findIsAudit(workerId);
            if(Objects.nonNull(isAudit)){
                throw new BusinessException("选中的学员已申请导师！");
            }
            Long stuClassesId = teacherAppDao.findStuClassesId(workerId);
            YcqQmTeacherStu ycqQmTeacherStu = new YcqQmTeacherStu();
            ycqQmTeacherStu.setStuClassesId(stuClassesId);
            ycqQmTeacherStu.setTeachWorkerId(worker.getLoginId());
            ycqQmTeacherStu.setStuWorkerId(workerId);
            ycqQmTeacherStu.setCorrelationState(1);
            ycqQmTeacherStu.setChooseState(1);
            ycqQmTeacherStu.setBinding(1);
            ycqQmTeacherStu.setStartTime(date);
            ycqQmTeacherStu.setCreatorId(worker.getLoginId());
            ycqQmTeacherStu.setCreateTime(date);
            ycqQmTeacherStu.setUpdaterId(worker.getLoginId());
            ycqQmTeacherStu.setUpdateTime(date);
            ycqQmTeacherStu.setDeleted(0);
            list.add(ycqQmTeacherStu);
        }
        ycqQmTeacherStuRepo.saveAllAndFlush(list);
        return "添加成功";
    }

    @Override
    public List<CodeValueVo> findLevelType() {
        return teacherAppDao.findLevelType();
    }

    @Override
    public List<String> findClassesYear() {
        return teacherAppDao.findClassesYear();
    }

    @Override
    public List<GrowthPartyVo> findParty(TrackingCultivationDto pageReq, Worker worker) {
        return commonService.findParty(pageReq);
    }

}
