package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.core.cache.TsmPropertyCache;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.feign.client.SysService;
import com.crunii.micro.feign.client.dto.WorkerC;
import com.crunii.micro.feign.client.dto.WorkerCreateDTO;
import com.crunii.micro.service.qm.api.domain.dto.*;
import com.crunii.micro.service.qm.api.domain.dto.studentApp.LearIntegral;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.SearchNumVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.ExertVo;
import com.crunii.micro.service.qm.api.domain.dto.teacherApp.*;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.EvaluateStuDto;
import com.crunii.micro.service.qm.api.domain.dto.teacherWeb.*;
import com.crunii.micro.service.qm.api.enums.EvaluateResultEnum;
import com.crunii.micro.service.qm.api.enums.OperationCompletionEnum;
import com.crunii.micro.service.qm.jdbc.TeacherManageDao;
import com.crunii.micro.service.qm.service.CommonService;
import com.crunii.micro.service.qm.service.TeacherManageService;
import com.crunii.micro.service.qm.util.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ReflectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qm.api.constants.QmConstants.*;

/**
 * @author wangchen create 2023/9/18 11:22
 */
@Slf4j
@Service
public class TeacherManageServiceImpl implements TeacherManageService {

    @Value("${upload.url}")
    private String urlPrefix;

    @Resource
    private YcqQmUserInfoRepo ycqQmUserInfoRepo;

    @Resource
    private SysService sysService;

    @Resource
    private YcqQmSelfCultivationRepo ycqQmSelfCultivationRepo;

    @Resource
    private CommonService commonService;

    @Resource
    private YcqQmUserAuditRecordRepo ycqQmUserAuditRecordRepo;

    @Resource
    private YcqQmUserCourseRepo ycqQmUserCourseRepo;

    @Resource
    private YcqQmTeacherExpandRepo ycqQmTeacherExpandRepo;

    private final TeacherManageDao teacherManageDao;

    @Resource
    private YcqQmUserHonorRepo honorRepo;

    @Resource
    private YcqQmFlowRecordRepo ycqQmFlowRecordRepo;

    @Resource
    private ExcelVerify excelVerify;

    @Resource
    private YcqQmExcelTempRepo excelTempRepo;

    @Resource
    private YcqQmTeacherExpandRepo expandRepo;

    @Resource
    private YcqQmTeacherStuRepo stuRepo;

    @Resource
    private YcqQmUserClassesRelRepo classesRelRepo;

    @Resource
    private TsmPropertyCache cache;

    public TeacherManageServiceImpl(TeacherManageDao teacherManageDao) {
        this.teacherManageDao = teacherManageDao;
    }

    @Override
    @InjectProperty
    public PageRsp<TeacherBaseVo> pageTeacher(PageReq<TeacherDto> pageReq) {
        // 查询组织下所有组织
        List<Long> orgIdList = getOrgIdList(pageReq.getQuery().getOrgId());
        if(Objects.isNull(orgIdList)){
            return new PageRsp<>();
        }
        PageRsp<TeacherBaseVo> rsp = teacherManageDao.pageTeacher(pageReq,orgIdList);
        List<TeacherBaseVo> result = rsp.getResult();
        if (CollectionUtils.isEmpty(result)) {
            return rsp;
        }
        // 通过worker查询聘用状态
        List<Integer> workerIdList = result.stream().map(TeacherBaseVo::getWorkerId).collect(Collectors.toList());
        List<TeacherExpandDto> expandStatusList = teacherManageDao.findExpandStatus(workerIdList);
        // 获取导师学员数量
        List<TsNumVo> tsNumVos = teacherManageDao.queryTsNum(workerIdList);
        Map<Integer, Integer> map = tsNumVos.stream().collect(Collectors.toMap(TsNumVo::getTeachWorkerId, TsNumVo::getStuNumber));
        if (!CollectionUtils.isEmpty(expandStatusList)) {
            Map<Integer, Integer> workerExpandStatusMap = expandStatusList.stream().filter(x -> Objects.nonNull(x.getExpandStatus())).collect(Collectors.toMap(TeacherExpandDto::getWorkerId, TeacherExpandDto::getExpandStatus));
            result.forEach(teacherVo -> {
                        Integer suNumber = map.get(teacherVo.getWorkerId());
                        teacherVo.setExpandStatus(workerExpandStatusMap.get(teacherVo.getWorkerId()));
                        teacherVo.setStuMaxNumber(TEACHER_MAX_STU);
                        teacherVo.setStuNumber(ObjectUtil.isEmpty(suNumber) ? 0 : suNumber);
                    }
            );
        }
        result.forEach(x -> x.setAuditStatus(pageReq.getQuery().getAuditStatus()));
        return rsp;
    }

    /**
     * 获取下级组织  超级管理员设null，区县组织不查询下级，学校组织查询下级所有组织id
     */
    public List<Long> getOrgIdList(Long orgId) {
        List<Long> orgIdList = new ArrayList<>();
        if(TOP_ORG_ID.equals(orgId) || Objects.isNull(orgId)){
            return orgIdList;
        }
        String levelCode = teacherManageDao.findLevelCodeByOrgId(orgId);
        if(!StringUtils.hasText(levelCode)){
            return orgIdList;
        }
        // 区县级
        if(DISTRICT_LEVEL.equals(levelCode.trim())){
            orgIdList.add(orgId);
            return orgIdList;
        }
        // 校级 需查询下所有组织
        return teacherManageDao.findSchoolOrgByOrgId(orgId);
    }

    @Override
    @InjectProperty
    public TeacherDetailVo getTeacherInfo(Integer workerId) {
        TeacherDetailVo teacherDetailVo = new TeacherDetailVo();
        // 查询导师基本信息
        TeacherBaseVo teacherBaseVo = teacherManageDao.findBaseTeacher(workerId);
        if (Objects.isNull(teacherBaseVo)) {
            throw new BusinessException("未查询到导师信息");
        }
        teacherBaseVo.setCardId(UserCardIdDecryptUtil.decryptUserCardId(teacherBaseVo.getCardId(),false));
        teacherDetailVo.setTeacherBase(teacherBaseVo);
        // 查询荣誉
        List<WorkerHonorVo> workerHonorList = teacherManageDao.findWorkerHonor(workerId);
        teacherDetailVo.setWorkerHonorList(workerHonorList);
        // 查询擅长领域
        String honorDesc = teacherManageDao.findTeacherAudit(workerId);
        teacherDetailVo.setHonorDesc(honorDesc);
        // 查询审核意见
        String note = teacherManageDao.findAuditNote(teacherBaseVo.getId());
        teacherDetailVo.setNote(note);
        // 查询聘书url & 擅长领域
        TeacherExpandDto teacherExpand = teacherManageDao.findTeacherExpand(workerId);
        if (ObjectUtil.isNotEmpty(teacherExpand)) {
            if (StringUtils.hasText(teacherExpand.getFileKey())) {
                teacherDetailVo.setExpandUrl(urlPrefix + teacherExpand.getFileKey());
            }
            teacherDetailVo.setLevelCode(teacherExpand.getLevelCode());
            teacherDetailVo.setLevelName(teacherExpand.getLevelName());
        }
        return teacherDetailVo;
    }

    @Override
    @InjectProperty
    public Result<List<StudentDetailVo>> getStudentInfo(Integer workerId) {
        List<StudentDetailVo> studentInfo = teacherManageDao.getStudentInfo(workerId);
        if (CollUtil.isNotEmpty(studentInfo)) {
            studentInfo.forEach(x -> {
                if (StringUtils.hasText(x.getFileKey())) {
                    x.setIsGraduation("0");
                }else {
                    x.setIsGraduation("1");
                }
            });
        }

        return Result.ofSuccess(studentInfo);
    }

    @Override
    public String saveExpandFileKey(TeacherExpandFileKeyDto teacherExpandFileKeyDto, Worker worker) {
        YcqQmTeacherExpand byWorkerIdAndDeleted = ycqQmTeacherExpandRepo.findByWorkerIdAndDeleted(teacherExpandFileKeyDto.getWorkerId(), 0);
        if (Objects.isNull(byWorkerIdAndDeleted)) {
            throw new BusinessException("无用户");
        }
        byWorkerIdAndDeleted.setFileKey(teacherExpandFileKeyDto.getFileKey());
        byWorkerIdAndDeleted.setUpdaterId(worker.getLoginId());
        byWorkerIdAndDeleted.setUpdateTime(new Date());
        ycqQmTeacherExpandRepo.saveAndFlush(byWorkerIdAndDeleted);
        return "更新成功";
    }

    @Override
    @InjectProperty
    public PageRsp<StudentVo> pageNoTeacherStu(PageReq<Integer> req) {
        // 查询levelCode
        String teacherLevelCode = teacherManageDao.findTeacherLevelCode(req.getQuery());
        PageRsp<StudentVo> rsp = teacherManageDao.pageNoTeacherStu(req, teacherLevelCode);
        List<StudentVo> studentVos = teacherManageDao.queryStudent(req.getQuery());
        List<StudentVo> result = rsp.getResult();
        result.addAll(studentVos);
        return rsp;
    }

    @Override
    public Result<String> updateStatus(Worker worker, Integer workerId, Integer expandStatus) {
        YcqQmTeacherExpand data = expandRepo.findByWorkerIdAndDeleted(workerId, 0);
        if (ObjectUtil.isEmpty(data)) {
            throw new BusinessException("操作失败");
        }
        // 解聘需要判断是否存在未结业学员
        if (expandStatus == STATUS_INVALID) {
            List<Integer> vos = teacherManageDao.queryStudentIds(workerId);
            if (CollUtil.isNotEmpty(vos)) {
                List<String> files = teacherManageDao.queryGraduationFileKeys(vos);
                if (files.size() != vos.size()) {
                    throw new BusinessException("操作失败，存在未结业的学员");
                }
            }
        }
        data.setExpandStatus(expandStatus);
        data.setUpdateTime(new Date());
        data.setUpdaterId(worker.getLoginId());
        expandRepo.saveAndFlush(data);
        return Result.ofSuccess("操作成功");
    }

    @Override
    public Result<List<StudentVo>> provisionedStu(Worker worker, Integer workerId) {
        return Result.ofSuccess(teacherManageDao.queryStudent(workerId));
    }

    @Override
    public Result<String> saveStudent(Worker worker, StudentDto dto) {
        // 导师是否存在
        YcqQmUserInfo data = ycqQmUserInfoRepo.findByWorkerIdAndDeleted(dto.getWorkerId(), 0);
        if (ObjectUtil.isEmpty(data)) {
            throw new BusinessException("导师不存在");
        }
        // 已绑定学员
        List<Integer> list = teacherManageDao.queryStudentIds(dto.getWorkerId());
        // 前端传入学员
        List<Integer> studentIds = dto.getStudentIds();
        // 需删除的学员
        List<Integer> needDelStuIds = new ArrayList<>();
        // 需新增的学员
        List<Integer> needInsertStuIds = new ArrayList<>();
        for (Integer integer : list) {
            if(!studentIds.contains(integer)){
                needDelStuIds.add(integer);
            }
        }
        for (Integer studentId : studentIds) {
            if(!list.contains(studentId)){
                needInsertStuIds.add(studentId);
            }
        }

        // 新增
        Date now = new Date();
        if(!CollectionUtils.isEmpty(needInsertStuIds)){
            // 删除之前导师与学员的申请以及拒绝记录
            teacherManageDao.delTeacherBindingStu(needInsertStuIds,dto.getWorkerId(),worker);
        }
        for (Integer id : needInsertStuIds) {
            // 查询班级id
            List<YcqQmUserClassesRel> classesRels = classesRelRepo.findByStuWorkerIdAndDeleted(id, 0);
            if (CollectionUtils.isEmpty(classesRels)) {
                throw new BusinessException("学员班级为空");
            }
            YcqQmTeacherStu stu = new YcqQmTeacherStu();
            stu.setStuClassesId(classesRels.get(0).getClassesId());
            stu.setTeachWorkerId(dto.getWorkerId());
            stu.setStuWorkerId(id);
            stu.setCorrelationState(USER_CORRELATION_STATE_1);
            stu.setBinding(1);
            stu.setChooseState(2);
            stu.setStartTime(now);
            stu.setCreatorId(worker.getLoginId());
            stu.setCreateTime(now);
            stu.setUpdaterId(worker.getLoginId());
            stu.setUpdateTime(now);
            stu.setDeleted(0);
            stuRepo.saveAndFlush(stu);
        }

        if(!CollectionUtils.isEmpty(needDelStuIds)){
            // 解绑
            teacherManageDao.updateTeacherBindingStu(needDelStuIds,dto.getWorkerId(),worker);
        }
        return Result.ofSuccess("操作成功");
    }


    @Override
    public String evaluateConcentratedCultivation(EvaluateStuDto evaluateStuDto, Worker worker) {
        if (Objects.isNull(evaluateStuDto.getUserCourseId())) {
            throw new BusinessException("数据异常，无该课程");
        }
        YcqQmUserCourse one = ycqQmUserCourseRepo.getOne(evaluateStuDto.getUserCourseId());
        one.setPresentationEvaluate(evaluateStuDto.getPresentationEvaluate());
        one.setEvaluateWorkerId(worker.getLoginId());
        one.setEvaluateTime(new Date());
        ycqQmUserCourseRepo.saveAndFlush(one);
        return "评价成功";
    }

    @Override
    public SearchNumVo searchNum(Worker worker, Long orgId) {
        List<Long> orgIdList = getOrgIdList(orgId);
        return teacherManageDao.searchNum(orgIdList);
    }

    @Override
    public Boolean validUpdateTeacher(TeacherDetailModifyDto teacherDetailModifyDto, Worker worker) {
        // 对比导师基本信息是否修改
        return validateTeacher(teacherDetailModifyDto);
    }

    /**
     * 验证导师是否修改
     */
    private Boolean validateTeacher(TeacherDetailModifyDto teacherDetailModifyDto) {
        Integer workerId = teacherDetailModifyDto.getWorkerId();
        if (Objects.isNull(workerId)) {
            return true;
        }
        // 判断基本信息
        TeacherDetailVo teacherInfo = getTeacherInfo(workerId);
        TeacherBaseModifyDto teacherBase = teacherDetailModifyDto.getTeacherBase();
        teacherBase.setCardId(Sm2Util.decrypt(PRIVATE_WEB_KEY, true, teacherBase.getCardId()));
        TeacherBaseVo teacherBase1 = teacherInfo.getTeacherBase();
        boolean b = validateObj(teacherBase, teacherBase1);
        if (b) {
            return true;
        }
        // 判断擅长领域
        WorkerHonorVo field = teacherDetailModifyDto.getField();
        String honorDesc = teacherInfo.getHonorDesc();
        if (Objects.isNull(field)) {
            if (StringUtils.hasText(honorDesc)) {
                return true;
            }
        }else {
            if (!field.getHonorDesc().equals(honorDesc)) {
                return true;
            }
        }
        List<WorkerHonorVo> workerHonorList = teacherDetailModifyDto.getWorkerHonorList();
        List<WorkerHonorVo> workerHonorList1 = teacherInfo.getWorkerHonorList();
        if (workerHonorList.size() != workerHonorList1.size()) {
            return true;
        }
        List<WorkerHonorVo> collect = workerHonorList.stream().sorted(
                (o1, o2) -> DateUtil.compare(o1.getAcquisitionTime(), o2.getAcquisitionTime())
        ).collect(Collectors.toList());
        List<WorkerHonorVo> collect2 = workerHonorList1.stream().sorted(
                (o1, o2) -> DateUtil.compare(o1.getAcquisitionTime(), o2.getAcquisitionTime())
        ).collect(Collectors.toList());
        String s = JSONUtil.toJsonStr(collect);
        String s1 = JSONUtil.toJsonStr(collect2);
        if (!s.equals(s1)) {
            return true;
        }
        // 判断班级级别
        String levelCode1 = teacherBase.getLevelCode();
        String levelCode = teacherInfo.getLevelCode();
        if(!StringUtils.hasText(levelCode) && !StringUtils.hasText(levelCode1)){
            return true;
        }
        return !levelCode.equals(levelCode1);
    }

    /**
     * 判断是否变动 true，2个对象值已发生改变
     */
    private boolean validateObj(Object compare, Object coverCompare) {
        Field[] declaredFields = compare.getClass().getDeclaredFields();
        Field[] declaredFields1 = coverCompare.getClass().getDeclaredFields();
        try {
            for (Field declaredField : declaredFields) {
                ReflectionUtils.makeAccessible(declaredField);
                for (Field field : declaredFields1) {
                    ReflectionUtils.makeAccessible(field);
                    if (declaredField.getName().equals(field.getName())) {
                        Object o = declaredField.get(compare);
                        Object o1 = field.get(coverCompare);
                        if (!o.equals(o1)) {
                            return true;
                        }
                    }
                }
            }
        } catch (IllegalAccessException e) {
            return true;
        }
        return false;
    }

    @Override
    public String updateAuditTeacher(AuditTeacherDto auditTeacherDto, Worker worker) {
        List<YcqQmUserInfo> ycqQmUserInfoList = new ArrayList<>();
        List<YcqQmUserAuditRecord> ycqQmUserAuditRecordArrayList = new ArrayList<>();
        List<Integer> workerId = auditTeacherDto.getWorkerId();
        List<YcqQmTeacherExpand> ycqQmTeacherExpands = new ArrayList<>();
        for (Integer aLong : workerId) {
            YcqQmUserInfo one = ycqQmUserInfoRepo.findByWorkerIdAndDeleted(aLong, 0);
            if (Objects.isNull(one)) {
                throw new BusinessException("无修改用户");
            }
            Date now = new Date();
            if (!one.getAuditStatus().equals(auditTeacherDto.getAuditStatus())) {
                one.setAuditStatus(auditTeacherDto.getAuditStatus());
                one.setUpdateTime(now);
                one.setUpdaterId(worker.getLoginId());
                ycqQmUserInfoList.add(one);
            }
            // 更新审核记录表
            YcqQmUserAuditRecord ycqQmUserAuditRecord = new YcqQmUserAuditRecord();
            ycqQmUserAuditRecord.setUserInfoId(one.getId());
            ycqQmUserAuditRecord.setAuditStatus(auditTeacherDto.getAuditStatus());
            ycqQmUserAuditRecord.setAuditWorkerId(worker.getLoginId());
            ycqQmUserAuditRecord.setNote(auditTeacherDto.getNote());
            ycqQmUserAuditRecord.setAuditTime(now);
            ycqQmUserAuditRecordArrayList.add(ycqQmUserAuditRecord);
            // 更新导师聘用状态
            YcqQmTeacherExpand byWorkerIdAndDeleted = ycqQmTeacherExpandRepo.findByWorkerIdAndDeleted(aLong, 0);
            byWorkerIdAndDeleted.setExpandStatus(1);
            ycqQmTeacherExpands.add(byWorkerIdAndDeleted);
            // 推送工作流
     /*      if(EnvUtil.getEnvIsOnline()){
               pushFlow(one.getId(),worker);
           }*/
        }
        if (!CollectionUtils.isEmpty(ycqQmUserInfoList)) {
            ycqQmUserInfoRepo.saveAllAndFlush(ycqQmUserInfoList);
        }
        if(!CollectionUtils.isEmpty(ycqQmTeacherExpands)){
            ycqQmTeacherExpandRepo.saveAllAndFlush(ycqQmTeacherExpands);
        }
        if (!CollectionUtils.isEmpty(ycqQmUserAuditRecordArrayList)) {
            ycqQmUserAuditRecordRepo.saveAllAndFlush(ycqQmUserAuditRecordArrayList);
        }
        return "审核成功";
    }

/*    public void pushFlow(Long userInfoId,Worker worker) {
        // 推送流程
        YcqQmFlowRecord byServiceId = ycqQmFlowRecordRepo.findByServiceId(userInfoId);
        if(Objects.isNull(byServiceId)){
            throw new BusinessException("流程未到达");
        }
        // 远程审核推进流程
        FlowCommitDTO flowCommitDTO = new FlowCommitDTO();
        FlowCommitorInfoDTO flowCommitorInfoDTO = new FlowCommitorInfoDTO();
        flowCommitorInfoDTO.setUserId((long) worker.getLoginId());
        flowCommitorInfoDTO.setUserName(worker.getWorkerName());
        String orgId = worker.getOrgId();
        flowCommitorInfoDTO.setOrgId(Objects.nonNull(orgId)?Long.valueOf(orgId):null);
        flowCommitorInfoDTO.setOrgName(worker.getOrgName());
        flowCommitDTO.setFlowInstanceId(byServiceId.getFlowInstanceId());
        flowCommitDTO.setTaskInstanceId(byServiceId.getNodeInstanceId());
        flowCommitDTO.setCommitor(flowCommitorInfoDTO);
        CommonResult<CruniiCommitTaskResult> result = flowInstanceApi.commitProcess(flowCommitDTO);
        if(!result.isSuccess()){
            log.info("调用推进流程失败,返回-》{}-参数-》{}", JSONUtil.toJsonStr(result),JSONUtil.toJsonStr(flowCommitDTO));
            throw new BusinessException("审核流程调用失败");
        }
    }*/

    @Override
    public String delBatchTeacher(List<Integer> workerIds, Worker worker) {
        // 查询导师下所有学员
        List<Integer> stuWorkerIds = teacherManageDao.findAllStuByTeacherWorkerIds(workerIds);
        if(!CollectionUtils.isEmpty(stuWorkerIds)){
            throw new BusinessException("导师下还拥有学员");
        }
        // 查询学员是否结业 todo
        commonService.delTeacherInfo(workerIds,worker);
        return "删除成功";
    }

    @Override
    public void exportTemplate(HttpServletResponse response) {
        responseSetting(response, "导师标准导入模版", ".xlsx",
                "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        InputStream inputStream = null;
        OutputStream outputStream = null;
        try {
            // 读取文件的输入流
            inputStream = Thread.currentThread().getContextClassLoader().getResourceAsStream("templates/导师标准导入模版.xlsx");
            outputStream = response.getOutputStream();
            IoUtil.copy(inputStream, outputStream);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            IoUtil.close(inputStream);
            IoUtil.close(outputStream);
        }
    }

    public void responseSetting(HttpServletResponse response, String fileName, String suffix, String contentType) {
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String newFileName = null;
        try {
            newFileName = URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        // 当客户端请求的资源是一个可下载的资源（这里的“可下载”是指浏览器会弹出下载框或者下载界面）时，对这个可下载资源的描述（例如下载框中的文件名称）就是来源于该头域。
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + newFileName + suffix);
        // 服务器告诉浏览器它发送的数据属于什么文件类型，也就是响应数据的MIME类型
        response.setContentType(contentType);
        response.setCharacterEncoding("utf-8");
        // 关闭缓存（HTTP/1.1）
        response.setHeader("Cache-Control", "no-store");
        // 关闭缓存（HTTP/1.0）
        response.setHeader("Pragma", "no-cache");
        // 缓存有效时间
        response.setDateHeader("Expires", 0);
    }

    @Override
    public String modifyTeacher(TeacherDetailModifyDto teacherDetailModifyDto, Worker worker) {
        if (Objects.isNull(teacherDetailModifyDto.getAuditStatus())) {
            throw new BusinessException("审核状态不能为空");
        }
        // 查询是新增 | 修改
        Integer workerId = teacherDetailModifyDto.getWorkerId();
        if (Objects.nonNull(workerId)) {
            updateTeacher(teacherDetailModifyDto, worker);
        } else {
            createTeacher(teacherDetailModifyDto, worker);
        }
        return "操作成功";
    }

    /**
     * 创建导师
     */
    private void createTeacher(TeacherDetailModifyDto teacherDetailModifyDto, Worker tsmWorker) {
        Date nowTime = new Date();
        TeacherBaseModifyDto teacherBase = teacherDetailModifyDto.getTeacherBase();
        // 查询是否有当前用户
        Integer loginId = teacherManageDao.findLoginIdByMobile(teacherBase.getMobile());
        if (Objects.nonNull(loginId)) {
            // todo 临时 更新用户feign调用？青马注册更新tsm_worker？
            commonService.updateWorker(loginId,teacherBase.getWorkerName(),teacherBase.getSex());
            // 有用户查询青马是否有该导师
            Integer count = teacherManageDao.findQmTeacherExist(loginId);
            if (count > 0) {
                throw new BusinessException("用户已存在");
            } else {
                // 创建青马用户
                createQmUser(loginId, teacherDetailModifyDto, tsmWorker, nowTime);
            }
        } else {
            // 远程叫用创建worker
            WorkerC workerC = new WorkerC();
            workerC.setMobile(teacherBase.getMobile());
            workerC.setSex(teacherBase.getSex());
            workerC.setWorkerName(teacherBase.getWorkerName());
            workerC.setState(1);
            workerC.setOrgId(teacherBase.getOrgId() + "");
            Result<WorkerCreateDTO> worker = sysService.createWorker(workerC);
            if (!worker.isSuccess()) {
                log.info("创建用户失败，feign接口调用失败！参数-》{}", JSONUtil.toJsonStr(workerC));
                throw new BusinessException("新增失败");
            }
            Integer workerId = worker.getData().getLoginId();
            // 创建青马用户
            createQmUser(workerId, teacherDetailModifyDto, tsmWorker, nowTime);
        }
    }

    /**
     * 创建青马用户
     */
    private void createQmUser(Integer workerId, TeacherDetailModifyDto teacherDetailModifyDto, Worker tsmWorker, Date nowTime) {
        TeacherBaseModifyDto teacherBase = teacherDetailModifyDto.getTeacherBase();
        List<WorkerHonorVo> workerHonorList = teacherDetailModifyDto.getWorkerHonorList();
        WorkerHonorVo field = teacherDetailModifyDto.getField();
        YcqQmUserInfo ycqQmUserInfo = new YcqQmUserInfo();
        ycqQmUserInfo.setWorkerId(workerId);
        ycqQmUserInfo.setSex(teacherBase.getSex());
        ycqQmUserInfo.setUserIdentity(2);
        ycqQmUserInfo.setOrgId(teacherBase.getOrgId());
        ycqQmUserInfo.setAreaId(teacherBase.getAreaId());
        ycqQmUserInfo.setPoliticalOutlook(teacherBase.getPoliticalOutlook());
        ycqQmUserInfo.setEducation(teacherBase.getEducation());
        ycqQmUserInfo.setCardId(Sm2Util.encrypt(PUBLIC_KEY, false, teacherBase.getCardId()));
        ycqQmUserInfo.setWorkUnit(teacherBase.getWorkUnit());
        ycqQmUserInfo.setWorkerJob(teacherBase.getWorkerJob());
        ycqQmUserInfo.setAuditStatus(1);
        ycqQmUserInfo.setCreatorId(tsmWorker.getLoginId());
        ycqQmUserInfo.setCreateTime(nowTime);
        ycqQmUserInfo.setUpdaterId(tsmWorker.getLoginId());
        ycqQmUserInfo.setUpdateTime(nowTime);
        ycqQmUserInfo.setDeleted(0);
        int year = DateUtil.year(nowTime);
        ycqQmUserInfo.setUserCode(UserCodeUtil.getTeacherCode(year));
        YcqQmUserInfo ycqQmUserInfo1 = ycqQmUserInfoRepo.saveAndFlush(ycqQmUserInfo);
        // 创建导师扩展表
        YcqQmTeacherExpand ycqQmTeacherExpand = new YcqQmTeacherExpand();
        ycqQmTeacherExpand.setWorkerId(ycqQmUserInfo1.getWorkerId());
        ycqQmTeacherExpand.setExpandStatus(1);
        ycqQmTeacherExpand.setLevelCode(teacherBase.getLevelCode());
        ycqQmTeacherExpand.setCreatorId(tsmWorker.getLoginId());
        ycqQmTeacherExpand.setCreateTime(nowTime);
        ycqQmTeacherExpand.setUpdaterId(tsmWorker.getLoginId());
        ycqQmTeacherExpand.setUpdateTime(nowTime);
        ycqQmTeacherExpand.setDeleted(0);
        ycqQmTeacherExpandRepo.saveAndFlush(ycqQmTeacherExpand);
        // 创建荣誉 && 擅长领域
        createHonor(workerId, workerHonorList, field);
        // 创建审核记录
        YcqQmUserAuditRecord ycqQmUserAuditRecord = new YcqQmUserAuditRecord();
        ycqQmUserAuditRecord.setUserInfoId(ycqQmUserInfo1.getId());
        ycqQmUserAuditRecord.setAuditStatus(teacherDetailModifyDto.getAuditStatus());
        ycqQmUserAuditRecord.setAuditWorkerId(tsmWorker.getLoginId());
        ycqQmUserAuditRecord.setNote(teacherDetailModifyDto.getNote());
        ycqQmUserAuditRecord.setAuditTime(nowTime);
        ycqQmUserAuditRecordRepo.saveAndFlush(ycqQmUserAuditRecord);
    }

    /**
     * 创建荣誉
     */
    private void createHonor(Integer workerId, List<WorkerHonorVo> list, WorkerHonorVo field) {
        List<YcqQmUserHonor> honorList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            for (WorkerHonorVo honorVo : list) {
                YcqQmUserHonor honor = new YcqQmUserHonor();
                honor.setWorkerId(workerId);
                honor.setAcquisitionTime(honorVo.getAcquisitionTime());
                honor.setHonorDesc(honorVo.getHonorDesc());
                honor.setCreatorId(workerId);
                honor.setCreateTime(new Date());
                honor.setUpdaterId(workerId);
                honor.setUpdateTime(new Date());
                honor.setHonorType(1);
                honorList.add(honor);
            }
        }
        // 擅长领域
        if (Objects.nonNull(field)) {
            YcqQmUserHonor honor = new YcqQmUserHonor();
            honor.setWorkerId(workerId);
            honor.setAcquisitionTime(new Date());
            honor.setHonorDesc(field.getHonorDesc());
            honor.setCreatorId(workerId);
            honor.setCreateTime(new Date());
            honor.setUpdaterId(workerId);
            honor.setUpdateTime(new Date());
            honor.setHonorType(2);
            honorList.add(honor);
        }
        honorRepo.saveAllAndFlush(honorList);
    }

    private void updateTeacher(TeacherDetailModifyDto dto, Worker worker) {
        Integer workId = dto.getWorkerId();
        TeacherBaseModifyDto teacherBase = dto.getTeacherBase();
        List<WorkerHonorVo> workerHonorList = dto.getWorkerHonorList();
        WorkerHonorVo field = dto.getField();
        // 基本信息
        YcqQmUserInfo info = ycqQmUserInfoRepo.findByWorkerIdAndDeleted(workId, 0);
        // 扩展信息
        YcqQmTeacherExpand expand = ycqQmTeacherExpandRepo.findByWorkerIdAndDeleted(workId, 0);
        // 荣誉
        List<YcqQmUserHonor> honors = honorRepo.findByWorkerIdAndDeleted(workId, 0);
        // 擅长领域
        if (ObjectUtil.isNotEmpty(info)) {
            info.setSex(teacherBase.getSex());
            info.setOrgId(teacherBase.getOrgId());
            info.setAreaId(teacherBase.getAreaId());
            info.setPoliticalOutlook(teacherBase.getPoliticalOutlook());
            info.setEducation(teacherBase.getEducation());
            info.setWorkUnit(teacherBase.getWorkUnit());
            info.setWorkerJob(teacherBase.getWorkerJob());
            info.setAuditStatus(dto.getAuditStatus());
            info.setUpdaterId(workId);
            info.setUpdateTime(new Date());
            ycqQmUserInfoRepo.saveAndFlush(info);
        }
        if (ObjectUtil.isNotEmpty(expand)) {
            expand.setExpandStatus(1);
            expand.setLevelCode(teacherBase.getLevelCode());
            expand.setUpdaterId(workId);
            expand.setUpdateTime(new Date());
            ycqQmTeacherExpandRepo.saveAndFlush(expand);
        }
        // 荣誉先删除再更新
        if (CollectionUtil.isNotEmpty(honors)) {
            honors.forEach(x -> {
                x.setUpdateTime(new Date());
                x.setUpdaterId(workId);
                x.setDeleted(1);
            });
            honorRepo.saveAllAndFlush(honors);
        }
        // 新增荣誉 && 擅长领域
        createHonor(workId, workerHonorList, field);
        // 创建审核记录
        YcqQmUserAuditRecord ycqQmUserAuditRecord = new YcqQmUserAuditRecord();
        ycqQmUserAuditRecord.setUserInfoId(info.getId());
        ycqQmUserAuditRecord.setAuditStatus(dto.getAuditStatus());
        ycqQmUserAuditRecord.setAuditWorkerId(worker.getLoginId());
        ycqQmUserAuditRecord.setNote(dto.getNote());
        ycqQmUserAuditRecord.setAuditTime(new Date());
        ycqQmUserAuditRecordRepo.saveAndFlush(ycqQmUserAuditRecord);
    }

    @Override
    public String validateDelBatchTeacher(List<Integer> workerIds, Worker worker) {
        // 查询导师下所有学员
        List<Integer> stuWorkerIds = teacherManageDao.findAllStuByTeacherWorkerIds(workerIds);
        if(CollectionUtil.isNotEmpty(stuWorkerIds)){
            throw new BusinessException("导师下还拥有学员");
        }
    /*    // 查询导师下是否还绑定学员
        List<Integer> workerIdList = teacherManageDao.findStuByTeacherIds(workerIds);
        // 绑定学员查导师名称
        if (!CollectionUtils.isEmpty(workerIdList)) {
            // 查出用户名称
            List<String> workerNames = teacherManageDao.findWorkerNames(workerIdList);
            String join = String.join(",", workerNames);
            throw new BusinessException(join + "下有正在进行的课程，无法删除");
        }*/
        return "";
    }

    @Override
    public void exportTeacher(List<Integer> workerIds) {
        List<ExportTeacher> exportTeacherList = teacherManageDao.findExportTeacherByIds(workerIds);
        try {
            if (CollectionUtil.isNotEmpty(exportTeacherList)) {
                exportTeacherList.forEach(x -> {
                    String decrypt = Sm2Util.decrypt(PRIVATE_KEY, false, x.getCardId());
                    decrypt = Sm2Util.decrypt(PRIVATE_WEB_KEY, true, decrypt);
                    x.setCardId(DesensitizedUtil.idCardNum(decrypt, 1, 2));
                    x.setEducationTrans(cache.get("qm-politicalOutlook", x.getEducation().toString()));
                    x.setPoliticalOutlookTrans(cache.get("qm-politicalOutlook", x.getPoliticalOutlook().toString()));
                });
            }
            ExcelUtil.writeSingleExcel("导师信息", null, exportTeacherList, ExportTeacher.class);
        } catch (IOException e) {
            throw new BusinessException("导出失败");
        }

    }

    @Override
    public void batchExportTeacher(TeacherDto teacherDto) {
        List<ExportTeacher> exportTeacherList = teacherManageDao.findExportTeacher(teacherDto);
        try {
            if (CollectionUtil.isNotEmpty(exportTeacherList)) {
                exportTeacherList.forEach(x -> {
                    String decrypt = Sm2Util.decrypt(PRIVATE_KEY, false, x.getCardId());
                    decrypt = Sm2Util.decrypt(PRIVATE_WEB_KEY, true, decrypt);
                    x.setCardId(DesensitizedUtil.idCardNum(decrypt, 1, 2));
                    x.setEducationTrans(cache.get("qm-politicalOutlook", x.getEducation().toString()));
                    x.setPoliticalOutlookTrans(cache.get("qm-politicalOutlook", x.getPoliticalOutlook().toString()));
                });
            }
            ExcelUtil.writeSingleExcel("导师信息", "导师信息", exportTeacherList, ExportTeacher.class);
        } catch (IOException e) {
            throw new BusinessException("导出失败");
        }

    }

    @Override
    public Result<ExcelDataInfo> verifyExcel(MultipartFile file) {
        ExcelData data = new ExcelData();
        ExcelDataInfo info = new ExcelDataInfo();
        List<ExcelData> excelData;
        List<MistakeDataInfo> lists = new ArrayList<>();
        List<ExcelMistakeData> dataList;
        List<YcqQmExcelTemp> tempList = new ArrayList<>();
        try {
            excelData = ExcelUtil.readSingleExcel(file, data, 2);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("导入失败");
        }
        if (CollectionUtil.isEmpty(excelData)) {
            throw new BusinessException("解析异常,查看数据是否规范");
        } else if (excelData.size() > EXCEL_MAX_RAW) {
            throw new BusinessException("超出最大导入条数");
        }
        // 开始校验参数
        // 所有手机号
        List<String> phones = excelData.stream().map(ExcelData::getPhone).filter(Objects::nonNull).collect(Collectors.toList());
        // 所有身份证号
        List<String> idCards = excelData.stream().map(ExcelData::getIdCard).filter(Objects::nonNull).collect(Collectors.toList());
        for (ExcelData x : excelData) {
            dataList = excelVerify.data(x, String.valueOf(excelData.indexOf(x) + 3), filterData(phones), filterData(idCards));
            if (CollectionUtil.isNotEmpty(dataList)) {
                MistakeDataInfo dataInfo = new MistakeDataInfo();
                dataInfo.setMistakeDataInfo(dataList);
                lists.add(dataInfo);
            }
        }
        // 校验通过保存临时表
        if (CollectionUtil.isEmpty(lists)) {
            String key = "JY" + UUIDUtil.getUUID();
            // 保存数据库
            excelData.forEach(x -> {
                YcqQmExcelTemp temp = new YcqQmExcelTemp();
                temp.setDataKey(key);
                BeanUtils.copyProperties(x, temp);
                // 身份证第一次加密
                temp.setIdCard(Sm2Util.encrypt(PUBLIC_WEB_KEY, true, x.getIdCard()));
                tempList.add(temp);
            });
            excelTempRepo.saveAllAndFlush(tempList);
            info.setDataKey(key);
        }
        // 返回错误信息
        info.setLists(lists);
        return Result.ofSuccess(info);
    }

    @Override
    public PageRsp<WorkerHonorVo> pageHonor(PageReq<StuExpandDto> pageReq) {
        return commonService.getStuExpand(pageReq);
    }

    @Override
    public String getAdvantage(StuExpandDto stuExpandDto) {
        return commonService.getAdvantage(stuExpandDto.getWorkerId());
    }

    @Override
    public PageRsp<ConcentratedCultivationVo> pageConcentratedCultivationExpand(PageReq<StuExpandDto> pageReq) {
        PageRsp<ConcentratedCultivationVo> concentratedCultivationVoPageRsp = commonService.pageConcentratedCultivationExpand(pageReq);
        List<ConcentratedCultivationVo> result = concentratedCultivationVoPageRsp.getResult();
        if (CollectionUtils.isEmpty(result)) {
            return concentratedCultivationVoPageRsp;
        }
        for (ConcentratedCultivationVo concentratedCultivationVo : result) {
            Integer evaluateResult = concentratedCultivationVo.getEvaluateResult();
            concentratedCultivationVo.setEvaluateResultTrans(EvaluateResultEnum.getTrans(evaluateResult));
            Integer operationCompletion = concentratedCultivationVo.getOperationCompletion();
            concentratedCultivationVo.setOperationCompletionTrans(OperationCompletionEnum.getTrans(operationCompletion));
        }
        return concentratedCultivationVoPageRsp;
    }

    @Override
    public PageRsp<TeacherTraineeVo> pageStuTraineeExpand(PageReq<TrackingCultivationDto> pageReq) {
        return commonService.pageStuTraineeExpand(pageReq);
    }

    @Override
    public SelfCultivationVo getSelfCultivation(TrackingCultivationDto trackingCultivationDto, Worker worker) {
        SelfCultivationVo selfCultivationVo = new SelfCultivationVo();
        // 获取学习强国积分
        LearIntegral integral = teacherManageDao.findIntegralByWorkerId(trackingCultivationDto.getWorkerId());
        if (Objects.isNull(integral)) {
            YcqQmSelfCultivation ycqQmSelfCultivation = new YcqQmSelfCultivation();
            ycqQmSelfCultivation.setCultivationType(1);
            ycqQmSelfCultivation.setWorkerId(trackingCultivationDto.getWorkerId());
            ycqQmSelfCultivation.setIntegral(0);
            ycqQmSelfCultivation.setCreatorId(worker.getLoginId());
            ycqQmSelfCultivation.setCreateTime(new Date());
            ycqQmSelfCultivation.setDeleted(worker.getLoginId());
            YcqQmSelfCultivation ycqQmSelfCultivation1 = ycqQmSelfCultivationRepo.saveAndFlush(ycqQmSelfCultivation);
            integral = new LearIntegral();
            integral.setSelfCultivationId(ycqQmSelfCultivation1.getSelfCultivationId());
            integral.setIntegral(0);
        }
        selfCultivationVo.setLearIntegral(integral);
        // 书籍
        List<WorkerBookVo> workerBookVos = teacherManageDao.findBookInfoByWorkerId(trackingCultivationDto.getWorkerId());
        selfCultivationVo.setBookInfo(workerBookVos);
        return selfCultivationVo;
    }

    @Override
    @InjectProperty
    public PageRsp<ExertVo> pageExert(PageReq<ExertDto> req, Worker worker) {
        return commonService.pageExert(req);
    }

    @Override
    @InjectProperty
    public PageRsp<GrowthVo> pageGrowth(PageReq<GroupCultivationDto> req, Worker worker) {
        return commonService.pageGrowth(req);
    }

    @Override
    public Result<String> importTeacherData(Worker worker, String dataKey) {
        Result<String> result = new Result<>();
        List<YcqQmExcelTemp> tempData = excelTempRepo.findAllByDataKey(dataKey);
        if (CollectionUtil.isEmpty(tempData)) {
            log.info("导入数据为空!!!");
            throw new BusinessException("导入失败,请联系管理员");
        }
        tempData.forEach(x -> {
            TeacherDetailModifyDto teacherDetailModifyDto = new TeacherDetailModifyDto();
            TeacherBaseModifyDto dto = new TeacherBaseModifyDto();
            dto.setWorkerName(x.getName());
            dto.setMobile(x.getPhone());
            dto.setSex("男".equals(x.getSex()) ? 1 : 0);
            dto.setCardId(x.getIdCard());
            dto.setLevelCode(getValueCode(CLASS_LEVEL, x.getClassLevel()));
            dto.setPoliticalOutlook(Integer.valueOf(getValueCode(POLITICAL_OUTLOOK, x.getPoliticalAspect())));
            dto.setEducation(Integer.valueOf(getValueCode(EDUCATION, x.getHighestEducation())));
            dto.setWorkerJob(x.getOffice());
            // 组织
            dto.setOrgId(Long.valueOf(x.getAffiliation()));
            // 区域
            dto.setAreaId(getAreaId(x.getRegion()));
            dto.setWorkUnit(x.getWorkplace());
            teacherDetailModifyDto.setTeacherBase(dto);
            teacherDetailModifyDto.setAuditStatus(1);
            try {
                modifyTeacher(teacherDetailModifyDto, worker);
            } catch (Exception e) {
                log.info("---------->导入导师失败,{}", e.getMessage());
                throw new BusinessException("导入失败,请联系管理员");
            }
        });
        // 导入成功后删除临时表数据
        excelTempRepo.deleteAll(tempData);
        result.setSuccess(true);
        result.setData(String.valueOf(tempData.size()));
        result.setMsg("导入成功");
        return result;
    }

    @Override
    @InjectProperty
    public List<VolunteerVo> getVolunteer(StuHonorDto stuHonorDto) {
        return commonService.getVolunteer(stuHonorDto);
    }

    /**
     * 通过静态值名称获取code
     *
     * @return
     */
    public String getValueCode(Integer propId, String valueName) {
        return teacherManageDao.getValueCode(propId, valueName);
    }

    /**
     * 查询区域id
     *
     * @param areaName
     * @return
     */
    public Long getAreaId(String areaName) {
        Long areaId;
        if ("重庆市".equals(areaName)) {
            areaId = 500000L;
        } else {
            areaId = teacherManageDao.getAreaId(areaName);
        }
        return areaId;
    }

    /**
     * 查询机构
     *
     * @param orgId
     * @return
     */
    public Long getOrgId(Long orgId) {
        return teacherManageDao.getOrgId(orgId);
    }

    /**
     * 找出重复的手机号/身份证号
     *
     * @param list
     * @return
     */
    public List<String> filterData(List<String> list) {
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream()
                .collect(Collectors.groupingBy(phoneNumber -> phoneNumber, Collectors.counting()))
                .entrySet().stream()
                .filter(entry -> entry.getValue() > 1)
                .map(Map.Entry::getKey)
                .collect(Collectors.toList());
    }

}