package com.crunii.micro.service.qm.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQmTeachingRoute;
import com.crunii.micro.domain.entity.main.YcqQmTeachingRouteNode;
import com.crunii.micro.domain.repo.main.YcqQmTeachingRouteNodeRepo;
import com.crunii.micro.domain.repo.main.YcqQmTeachingRouteRepo;

import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.teachingRoute.*;
import com.crunii.micro.service.qm.jdbc.TeachingRouteDao;
import com.crunii.micro.service.qm.service.TeachingRouteService;
import com.crunii.micro.service.qm.util.ChineseConvertToPinyin;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author denggl
 * date 2023/9/18
 */
@Service
public class TeachingRouteServiceImpl implements TeachingRouteService {
    @Autowired
    private TeachingRouteDao ycqQmTeachingRouteDao;

    @Autowired
    private YcqQmTeachingRouteRepo ycqQmTeachingRouteRepo;

    @Autowired
    private YcqQmTeachingRouteNodeRepo ycqQmTeachingRouteNodeRepo;

    @InjectProperty
    @Override
    public PageRsp<TeachingRouteVo> pageSearchRoute(PageReq<TeachingRouteDto> ycqQmTeachingRouteVoPageReq, Worker worker) {
        return ycqQmTeachingRouteDao.pageSearchRoute(ycqQmTeachingRouteVoPageReq);
    }
    @InjectProperty
    @Override
    public TeachingRouteDetailsVo routeDetails(Long routeId, Worker worker) {
        YcqQmTeachingRoute ycqQmTeachingRoute = ycqQmTeachingRouteRepo.findById(routeId).orElse(new YcqQmTeachingRoute());
        if (ycqQmTeachingRoute == null) {
            return null;
        }
        //组装基本信息和节点信息
        TeachingRouteDetailsVo ycqQmTeachingRouteDetails = new TeachingRouteDetailsVo();
        BeanUtils.copyProperties(ycqQmTeachingRoute, ycqQmTeachingRouteDetails);
        ycqQmTeachingRouteDetails.setRouteNodeDtoList(getlistNode(routeId, worker));
        return ycqQmTeachingRouteDetails;
    }

    @Override
    public void updateRoute(TeachingRouteU teachingRouteU, Worker worker) {
        YcqQmTeachingRoute ycqQmTeachingRoute = new YcqQmTeachingRoute();

        //routeId为空表示新增，否则为修改编辑
        if (teachingRouteU.getRouteId() == null) {
            ycqQmTeachingRoute.setCreateTime(new Date());
            ycqQmTeachingRoute.setCreatorId(worker.getLoginId());
            ycqQmTeachingRoute.setDeleted(0);
            BeanUtils.copyProperties(teachingRouteU, ycqQmTeachingRoute);
            ycqQmTeachingRoute.setRouteCode(getRouteCode(worker,teachingRouteU.getRouteName()));

        } else {
            ycqQmTeachingRoute= ycqQmTeachingRouteRepo.findById(teachingRouteU.getRouteId()).orElse(new YcqQmTeachingRoute());
            if (ycqQmTeachingRoute==null){
                throw new BusinessException("编辑失败,线路不存在！");
            }
            ycqQmTeachingRoute.setUpdateTime(new Date());
            ycqQmTeachingRoute.setUpdaterId(worker.getLoginId());
            ycqQmTeachingRoute.setDeleted(QmConstants.DELETED_0);

            BeanUtils.copyProperties(teachingRouteU, ycqQmTeachingRoute,new String[]{"createTime","creatorId,routeCode"});
        }
        YcqQmTeachingRoute save = ycqQmTeachingRouteRepo.saveAndFlush(ycqQmTeachingRoute);
        //是否设置线路节点
        if(!CollectionUtils.isEmpty(teachingRouteU.getListNode())){
            teachingRouteU.getListNode().forEach(data->{
                data.setTeachingRouteId(save.getRouteId());
            });

           updateRouteNode(teachingRouteU.getListNode(),worker);
        }


    }

    @Override
    public void delRoute(List<Long> lid, Worker worker) {
        //只能删除未下架的数据
        List<YcqQmTeachingRoute> allById = ycqQmTeachingRouteRepo.findAllById(lid);
        List<YcqQmTeachingRoute> collect = allById.stream().filter(data -> data.getRouteStatus() == 1).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(collect)) {
            throw new BusinessException("删除失败,含有未下架的路线！");
        }
        ycqQmTeachingRouteRepo.updateDeletedByRouteId(worker.getLoginId(),new Date(),lid);
    }

    @Override
    public void updateRouteNode(List<TeachingRouteNodeDto> ycqQmTeachingRouteNodeDto, Worker worker) {
        //过滤是否新增
        List<YcqQmTeachingRouteNode> addCollect = ycqQmTeachingRouteNodeDto.stream().filter(data -> data.getRouteNodeId() == null).map(node -> {
            checkNodeTime(node.getStartTime(),node.getEndTime());
            checkNodeContent(node.getContent());
            YcqQmTeachingRouteNode ycqQmTeachingRouteNode = new YcqQmTeachingRouteNode();
            BeanUtils.copyProperties(node, ycqQmTeachingRouteNode);
            ycqQmTeachingRouteNode.setCreateTime(new Date());
            ycqQmTeachingRouteNode.setCreatorId(worker.getLoginId());
            ycqQmTeachingRouteNode.setDeleted(QmConstants.DELETED_0);
            return ycqQmTeachingRouteNode;
        }).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(addCollect)) {
            ycqQmTeachingRouteNodeRepo.saveAll(addCollect);
        }

        //修改编辑
        List<YcqQmTeachingRouteNode> updateCcollect = ycqQmTeachingRouteNodeDto.stream().filter(data -> data.getRouteNodeId() != null).map(node -> {
            YcqQmTeachingRouteNode ycqQmTeachingRouteNode = ycqQmTeachingRouteNodeRepo.findById(node.getRouteNodeId()).orElse(new YcqQmTeachingRouteNode());
          if (ycqQmTeachingRouteNode!=null) {
              checkNodeTime(node.getStartTime(),node.getEndTime());
              checkNodeContent(node.getContent());
              BeanUtils.copyProperties(node, ycqQmTeachingRouteNode, new String[]{"createTime", "creatorId"});
              ycqQmTeachingRouteNode.setUpdateTime(new Date());
              ycqQmTeachingRouteNode.setUpdaterId(worker.getLoginId());
              ycqQmTeachingRouteNode.setDeleted(QmConstants.DELETED_0);

          }
            return ycqQmTeachingRouteNode;

        }).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(updateCcollect)) {
            ycqQmTeachingRouteNodeRepo.saveAll(updateCcollect);
        }
    }


    @Override
    public void delRouteNode(List<Long> lid, Worker worker) {
         ycqQmTeachingRouteNodeRepo.updateDeletedByRouteId(worker.getLoginId(),new Date(),lid);
    }

    @Override
    public List<TeachingRouteNodeVo> getlistNode(Long routeId, Worker worker) {
        return ycqQmTeachingRouteDao.listNodeByRouteId(routeId);
    }

    @Override
    public void updateRouteState(Worker worker, Long routeId, Integer routeStatus) {
        YcqQmTeachingRoute ycqQmTeachingRoute = ycqQmTeachingRouteRepo.findById(routeId).orElse(new YcqQmTeachingRoute() );
        if (ycqQmTeachingRoute!=null){
            ycqQmTeachingRoute.setUpdaterId(worker.getLoginId());
            ycqQmTeachingRoute.setUpdateTime(new Date());
            ycqQmTeachingRoute.setRouteStatus(routeStatus);
            ycqQmTeachingRouteRepo.save(ycqQmTeachingRoute);
        }
    }

    /**
     * 根据规则生成的课程编号，格式为“模块拼音首字母+导师拼音首字母+创建时间(yymmddhhmmss)+四位数字(升序)，示例;JXLX2023080712000000001
     * 需要获取获取现有数量
     *
     * @return
     */
    public String getRouteCode(Worker worker,String routeName) {
        int count = ycqQmTeachingRouteDao.selectCountByCreateId(worker.getLoginId());//现有数量
      /*  String teacher = ChineseConvertToPinyin.toFirstChar(routeName);   //老师拼音大写字母
        if (!Objects.nonNull(teacher)){
            teacher="";
        }*/
        String crateTiem = DateUtil.format(new Date(), DateUtil.defaultDT1Pattern);
        return QmConstants.JXLX + crateTiem + getNumCode(count);
    }
    /**
     * 获取4位序列
     * @param num
     * @return
     */
    private static String getNumCode(int num) {
        num++;
        String formattedNumber = String.format("%04d", num);
        return formattedNumber;
    }


    void checkNodeTime(Date startTime,Date endTime ){

        if (startTime.compareTo(endTime)>0){
            throw new BusinessException("节点开始时间不能大于结束时间！");

        }
    }



    void  checkNodeContent(String content){
        if (StringUtils.isBlank(content)){
            throw new BusinessException("节点路线内容必填！");

        }
    }
}
