package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;

import com.crunii.micro.domain.dto.obs.ObsServiceCodeDto;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.trainingTrain.*;
import com.crunii.micro.service.qm.jdbc.TrainingTrainDao;
import com.crunii.micro.service.qm.jdbc.UserInfoDao;
import com.crunii.micro.service.qm.service.TrainingTrainService;

import lombok.extern.log4j.Log4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author denggl
 * date 2023/9/19
 */
@Service
@Log4j
public class TrainingTrainServiceImpl implements TrainingTrainService {
    @Autowired
    private TrainingTrainDao ycqQmTrainingTrainDao;
    @Autowired
    private YcqQmTrainingTrainRepo ycqQmTrainingTrainRepo;
    @Autowired
    private YcqQmTrainingTrainClockRepo ycqQmTrainingTrainClockRepo;
    @Autowired
    private ObsService obsService;

    @Value("${upload.url}")
    private String urlPrefix;

    @Resource
    private YcqQmTrainingTrainLabelRepo ycqQmTrainingTrainLabelRepo;

    @InjectProperty
    @Override
    public PageRsp<TrainingTrainVo> pageTrain(Worker worker, PageReq<TrainingTrainDto> ycqQmTrainingTrainDto) {
        PageRsp<TrainingTrainVo> pageTrain = ycqQmTrainingTrainDao.pageTrain(ycqQmTrainingTrainDto);
        if (CollectionUtil.isEmpty(pageTrain.getResult())){
            return pageTrain;
        }
       //获取所有基地id
        List<Long> collect = pageTrain.getResult().stream().map(item -> item.getBaseId()).collect(Collectors.toList());
        //获取所有图片
        Map<Long,List<TrainPic>> obsFiles = getObsFiles(collect);
         //获取所有标签
        Map<Long, List<TrainLabel>> trainLabel = getTrainLabel(collect);
        //获取所有区域
        Map<Long,String>  orgMap= getOrg();

        pageTrain.getResult().forEach(data -> {
            data.setTrainLabel(trainLabel.get(data.getBaseId()));
            data.setTrainRegionName(orgMap.get(data.getTrainRegion()));
            List<TrainPic> trainPics = obsFiles.get(data.getBaseId());
            if (!CollectionUtil.isEmpty(trainPics)) {
                //列表查询只需展示一张
                data.setTeachingRoutePic(trainPics.get(0));
            }
            //标签数据
        });
        return pageTrain;
    }

    @Override
    public void updateTrain(Worker worker, TrainingTrainU trainingTrainU) {

        YcqQmTrainingTrain ycqQmTrainingTrain = new YcqQmTrainingTrain();
        // 新增
        if (trainingTrainU.getBaseId() == null) {
            ycqQmTrainingTrain.setCreateTime(new Date());
            ycqQmTrainingTrain.setCreatorId(worker.getLoginId());
            BeanUtils.copyProperties(trainingTrainU, ycqQmTrainingTrain);
            ycqQmTrainingTrain.setTrainCode(getTrainCode(worker, trainingTrainU.getTrainName()));
        } else {
            ycqQmTrainingTrain = ycqQmTrainingTrainRepo.findById(trainingTrainU.getBaseId()).orElse(new YcqQmTrainingTrain());
            //防止创建信息被覆盖
            BeanUtils.copyProperties(trainingTrainU, ycqQmTrainingTrain, new String[]{"createTime", "creatorId","trainCode"});
            ycqQmTrainingTrain.setUpdateTime(new Date());
            ycqQmTrainingTrain.setUpdaterId(worker.getLoginId());


        }

        YcqQmTrainingTrain trainingTrain = ycqQmTrainingTrainRepo.saveAndFlush(ycqQmTrainingTrain);
        //将基地id关于obs关联
        updateTrainPic(trainingTrainU.getListFile(),trainingTrain.getBaseId());
        //将标签和基地关联
        updateTrainLabel(trainingTrainU.getTrainLabel(), trainingTrain.getBaseId());

    }



    @InjectProperty
    @Override
    public TrainingTrainDetailsVo selectTrinDetails(Long trId, Worker worker) {
        YcqQmTrainingTrain ycqQmTrainingTrain = ycqQmTrainingTrainRepo.getOne(trId);

        if (ycqQmTrainingTrain == null) {
            return null;
        }
        List<Long> list=new ArrayList<>();
        list.add(trId);
        TrainingTrainDetailsVo trainingTrainVo = new TrainingTrainDetailsVo();
        BeanUtils.copyProperties(ycqQmTrainingTrain, trainingTrainVo);
        trainingTrainVo.setTeachingRoutePic(getObsFiles(list).get(trId));
        trainingTrainVo.setTrainLabel(getTrainLabel(list).get(trId));
        trainingTrainVo.setTrainRegionName(getOrg().get(ycqQmTrainingTrain.getTrainRegion()));
        return trainingTrainVo;
    }

    @Override
    public void delTrain(List<Long> trId, Worker worker) {
        List<YcqQmTrainingTrain> allById = ycqQmTrainingTrainRepo.findAllById(trId);
        List<YcqQmTrainingTrain> collect = allById.stream().filter(data -> data.getTrainStatus() == 1).collect(Collectors.toList());
        if (!CollectionUtil.isEmpty(collect)) {
            throw new BusinessException("存在启用基地，请先禁用，再删除！");
        }
        ycqQmTrainingTrainRepo.updateDeletedByrouteBaseId(worker.getLoginId(),new Date(),trId);
    }

    @Override
    public List<TrainingTrainClockVo> selectClockRecord(Long trId, Worker worker) {
        List<TrainingTrainClockVo> trainingTrainClockVos = ycqQmTrainingTrainDao.selectClockRecord(trId, worker);
        trainingTrainClockVos.forEach(data -> {
                if (StringUtils.hasText(data.getHeadSculpture())) {
                    data.setHeadSculptureUrl(urlPrefix + data.getHeadSculpture());
            }
        });
        return trainingTrainClockVos;
    }

    @Override
    public String trainClock(TrainClockDto trainClockDto, Worker worker) {
        Optional<YcqQmTrainingTrain> byId = ycqQmTrainingTrainRepo.findById(trainClockDto.getTrainId());
        if (!byId.isPresent()){
            throw new BusinessException("基地不存在");
        }
        YcqQmTrainingTrainClock ycqQmTrainingTrainClock = new YcqQmTrainingTrainClock();
        ycqQmTrainingTrainClock.setLatitude(trainClockDto.getLatitude());
        ycqQmTrainingTrainClock.setLongitude(trainClockDto.getLongitude());
        ycqQmTrainingTrainClock.setWokerId(worker.getLoginId());
        ycqQmTrainingTrainClock.setTrainId(trainClockDto.getTrainId());
        ycqQmTrainingTrainClock.setClockTime(new Date());
        ycqQmTrainingTrainClock.setClockState(trainClockDto.getClockState());
        ycqQmTrainingTrainClockRepo.saveAndFlush(ycqQmTrainingTrainClock);
        //查询打卡次数
        Long countByWokerId = ycqQmTrainingTrainClockRepo.countByWokerIdAndClockState((worker.getLoginId()),QmConstants.CLOCK_STATE);
        return String.valueOf(countByWokerId);
    }

    @Override
    public void updateTrainState(Worker worker, TrainStatusDto trainStatus) {
        ycqQmTrainingTrainRepo.updatetStatusByBaseId(trainStatus.getTrainStatus(),worker.getLoginId(),new Date(),trainStatus.getTrainId());


    }
    @InjectProperty
    @Override
    public PageRsp<TrainingTrainVo> AppPageTrain(Worker worker, PageReq<TrainingTrainDto> ycqQmTrainingTrainDto) {
        ycqQmTrainingTrainDto.getQuery().setTrainStatus(QmConstants.STATUS);
        return  pageTrain(worker,ycqQmTrainingTrainDto);
    }

    @Override
    public Long clocked(Long baseId, Worker worker) {
        Long countByWokerId = ycqQmTrainingTrainClockRepo.countByWokerIdAndTrainIdAndClockState(worker.getLoginId(),baseId,QmConstants.CLOCK_STATE);
        return countByWokerId;
    }

    /**
     * 根据规则生成的课程编号，格式为“模块拼音首字母+导师拼音首字母+创建时间(yymmddhhmmss)+四位数字(升序)，示例;JD2023080712000000001
     *
     * @param worker
     * @return
     */
    public String getTrainCode(Worker worker, String name) {
        //现有数量
        int count = ycqQmTrainingTrainDao.selectCountByCreateId(worker.getLoginId());
   /*     //基地音大写字母
        String teacher = ChineseConvertToPinyin.toFirstChar(name);
        if (!Objects.nonNull(teacher)) {
            teacher = "";
        }*/
        String crateTiem = DateUtil.format(new Date(), DateUtil.defaultDT1Pattern);
        return QmConstants.PXJD + crateTiem + getNumCode(count);
    }



    /**
     * 获取4位序列
     *
     * @param num
     * @return
     */
    private static String getNumCode(int num) {
        num++;
        String formattedNumber = String.format("%04d", num);
           return formattedNumber;
    }


    public List<FileVo> saveFile(MultipartFile[] files) {
        if (files.length == 0) {
            return null;
        }
        //判断文件个数和大小
        if (files.length > 3) {
            throw new BusinessException("最多上传3张！");
        }
        for (MultipartFile file : files) {
            long fileSize = file.getSize();
            // 转换为MB
            double fileSizeInMB = (double) fileSize / (1024 * 1024);
            //最大上传5M
            if (fileSizeInMB > 5) {
                throw new BusinessException("最大上传5M！");
            }
        }
        List<FileVo> fileVoList = new ArrayList<>();
        for (MultipartFile file : files) {
            try {
                Result<FileVo> fileVoResult = obsService.feignFileUpload(file);
                if (!fileVoResult.isSuccess()) {
                    throw new BusinessException("图片上传失败！");
                }
                fileVoList.add(fileVoResult.getData());
            } catch (Exception e) {
                throw new BusinessException("图片上传失败！");
            }

        }

        return fileVoList;
    }




    public Map<Long,List<TrainPic>> getObsFiles(List<Long> baseId) {
     return ycqQmTrainingTrainDao.getPic(baseId).stream().map(data -> {
            TrainPic teachingRoutePic = new TrainPic();
            teachingRoutePic.setPicKey(data.getObjectKey());
            teachingRoutePic.setPicKeyUrl(urlPrefix + data.getObjectKey());
            teachingRoutePic.setBaseId(data.getServiceId());
            return teachingRoutePic;
        }).collect(Collectors.groupingBy(TrainPic::getBaseId));
    }

    void updateTrainLabel(List<Integer> list, Long baseId) {
        List<YcqQmTrainingTrainLabel> byTrainIdAndDeleted = ycqQmTrainingTrainLabelRepo.findByTrainIdAndDeleted(baseId, QmConstants.DELETED_0);

        //删除
        List<YcqQmTrainingTrainLabel> delCollect = byTrainIdAndDeleted.stream().filter(abel -> !list.contains(abel.getLabelValue())).map(data -> {
            data.setDeleted(QmConstants.DELETED_1);
            return data;
        }).collect(Collectors.toList());

        List<Integer> exist = byTrainIdAndDeleted.stream().map(item -> item.getLabelValue()).collect(Collectors.toList());
        //新增
        List<YcqQmTrainingTrainLabel> addCollect = list.stream().filter(value -> !exist.contains(value)).map(item -> {
            YcqQmTrainingTrainLabel ycqQmTrainingTrainLabel = new YcqQmTrainingTrainLabel();
            ycqQmTrainingTrainLabel.setLabelValue(item);
            ycqQmTrainingTrainLabel.setTrainId(baseId);
            return ycqQmTrainingTrainLabel;
        }).collect(Collectors.toList());
        addCollect.addAll(delCollect);
        ycqQmTrainingTrainLabelRepo.saveAllAndFlush(addCollect);
    }

     void updateTrainPic(List<TeachingRoutePicdDto> listFile, Long baseId) {
         List<Long> list=new ArrayList<>();
         list.add(baseId);
         List<String>  updateCollect = listFile.stream().map(item -> item.getPicKey()).collect(Collectors.toList());
         List<TrainPic> obsFiles = getObsFiles(list).get(baseId);
         if (!CollectionUtil.isEmpty(obsFiles)){
             //删除
             List<String> delCollect = obsFiles.stream().filter(pic -> !updateCollect.contains(pic.getPicKey())).map(item ->
                     item.getPicKey()
             ).collect(Collectors.toList());
             if (!CollectionUtil.isEmpty(delCollect)) {
                 ycqQmTrainingTrainDao.updateTrainPicState(delCollect,baseId);
             }
         }
             //新增
             List<ObsServiceCodeDto> fileCollect = listFile.stream().filter(teachingRoutePicdDto -> teachingRoutePicdDto.getFileId() != null).map(file -> {
                 ObsServiceCodeDto obsServiceCodeDtos = new ObsServiceCodeDto();
                 obsServiceCodeDtos.setFileId(file.getFileId());
                 obsServiceCodeDtos.setServiceCode(CommonConstants.OBS_FILE_PXJD);
                 obsServiceCodeDtos.setServiceId(baseId);
                 return obsServiceCodeDtos;
             }).collect(Collectors.toList());

         if (!CollectionUtil.isEmpty(fileCollect)) {
             obsService.updateObsServiceCode(fileCollect);
         }
         }


    public  Map<Long, List<TrainLabel>>  getTrainLabel(List<Long> baseId) {
        List<TrainLabel> byTrainIdsAndDeleted = ycqQmTrainingTrainDao.findByTrainIdsAndDeleted(baseId, QmConstants.DELETED_0);
        return   byTrainIdsAndDeleted.stream().collect(Collectors.groupingBy(TrainLabel::getBaseId));


    }




   public Map<Long,String> getOrg(){
       Map<Long,String>  orgMap= ycqQmTrainingTrainDao.getOrg().stream().collect(Collectors.toMap(TsmOrg::getOrgId,TsmOrg::getOrgName));
       return orgMap;
   }



}
