package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQmUserHonor;
import com.crunii.micro.domain.repo.main.YcqQmUserHonorRepo;
import com.crunii.micro.domain.repo.main.YcqQmUserInfoRepo;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.UserHonorDto;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.SelectUserHonorVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.UserHonorVo;
import com.crunii.micro.service.qm.jdbc.UserHonorDao;
import com.crunii.micro.service.qm.service.UserHonorService;
import com.crunii.micro.service.qm.util.ListUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author xia
 * @version 1.0
 * @date 2023/9/18 0018 15:12:32
 */
@Slf4j
@Service
public class UserHonorServiceImpl implements UserHonorService {

	@Autowired YcqQmUserHonorRepo ycqQmUserHonorRepo;
	@Autowired YcqQmUserInfoRepo ycqQmUserInfoRepo;

	@Autowired UserHonorDao userhonorDao;

	/**
	 * 查询用户荣誉
	 *
	 * @param loginId 人员id
	 * @return Result<SelectUserHonorVo>
	 */
	@Override public Result<SelectUserHonorVo> selectUserHonor(int loginId) {
		Result<SelectUserHonorVo> result = new Result<>();
		SelectUserHonorVo selectUserHonorVo = new SelectUserHonorVo();
		List<YcqQmUserHonor> ycqQmUserHonorList = ycqQmUserHonorRepo.findByWorkerIdAndDeleted(loginId, QmConstants.DELETED_0);
		if(ListUtil.hasValue(ycqQmUserHonorList)){
			List<UserHonorVo> userhonorVoList = new ArrayList<>();
			for(YcqQmUserHonor ycqQmUserHonor:ycqQmUserHonorList){
				if(QmConstants.HONOR_TYPE_2==ycqQmUserHonor.getHonorType()){
					selectUserHonorVo.setAdeptField(ycqQmUserHonor.getHonorDesc());
				}else{
					UserHonorVo userhonorVo = new UserHonorVo();
					BeanUtils.copyProperties(ycqQmUserHonor,userhonorVo);
					userhonorVoList.add(userhonorVo);
				}
			}
			selectUserHonorVo.setUserHonorVoList(userhonorVoList);
		}
		result.setData(selectUserHonorVo);
		return result;
	}

	/**
	 * 新增或修改用户荣誉
	 *
	 * @param worker 人员实体
	 * @param dto    入参实体
	 * @return Result<String>
	 */
	@Override public Result<String> insertUserHonor(Worker worker, UserHonorDto dto) {
		Result<String> result = new Result<>();
		if(QmConstants.HONOR_TYPE_1==dto.getHonorType()&&ObjectUtil.isEmpty(dto.getAcquisitionTime())){
			return Result.ofFail("荣誉获奖时间不能为空");
		}
		Date curr = new Date();
		YcqQmUserHonor ycqQmUserHonor;
		if (ObjectUtil.isEmpty(dto.getHonorId())) {
			//新增
			ycqQmUserHonor = new YcqQmUserHonor();
			BeanUtils.copyProperties(dto, ycqQmUserHonor);
			if(QmConstants.HONOR_TYPE_2==dto.getHonorType()){
				//查询人员以前是否录入过擅长领域
				List<YcqQmUserHonor> ycqQmUserHonorList = ycqQmUserHonorRepo
						.findByWorkerIdAndHonorTypeAndDeleted(worker.getLoginId(), QmConstants.HONOR_TYPE_2, QmConstants.DELETED_0);
				if(ycqQmUserHonorList!=null&&ycqQmUserHonorList.size()>0){
					//以前录入过则删掉旧的，保证人员只有一条擅长领域
					ycqQmUserHonorRepo.deleteAll(ycqQmUserHonorList);
				}
				ycqQmUserHonor.setAcquisitionTime(curr);//擅长领域录入时间
			}
			ycqQmUserHonor.setWorkerId(worker.getLoginId());
			ycqQmUserHonor.setCreatorId(worker.getLoginId());
			ycqQmUserHonor.setCreateTime(curr);
			ycqQmUserHonor.setUpdaterId(worker.getLoginId());
			ycqQmUserHonor.setUpdateTime(curr);
			ycqQmUserHonor.setDeleted(QmConstants.DELETED_0);
		} else {
			//修改
			ycqQmUserHonor = ycqQmUserHonorRepo.findByHonorIdAndDeleted(dto.getHonorId(), QmConstants.DELETED_0);
			if (ObjectUtil.isEmpty(ycqQmUserHonor)) {
				return Result.ofFail("未查询到该记录");
			} else {
				if(QmConstants.HONOR_TYPE_2==dto.getHonorType()){
					ycqQmUserHonor.setAcquisitionTime(curr);//擅长领域录入时间
				}else{
					ycqQmUserHonor.setAcquisitionTime(dto.getAcquisitionTime());//荣誉获得时间
				}
				ycqQmUserHonor.setHonorDesc(dto.getHonorDesc());
				ycqQmUserHonor.setUpdaterId(worker.getLoginId());
				ycqQmUserHonor.setUpdateTime(curr);
			}
		}
		ycqQmUserHonorRepo.saveAndFlush(ycqQmUserHonor);
		result.setData("录入成功");
		return result;
	}

	/**
	 * 删除用户荣誉
	 *
	 * @param worker  人员实体
	 * @param honorId 荣誉主键id
	 * @return Result<String>
	 */
	@Override public Result<String> deleteUserHonor(Worker worker, Long honorId) {
		Result<String> result = new Result<>();
		Date curr = new Date();
		YcqQmUserHonor ycqQmUserHonor = ycqQmUserHonorRepo.findByHonorIdAndDeleted(honorId, QmConstants.DELETED_0);
		if (ObjectUtil.isEmpty(ycqQmUserHonor)) {
			return Result.ofFail("未查询到该记录");
		} else {
			ycqQmUserHonor.setUpdaterId(worker.getLoginId());
			ycqQmUserHonor.setUpdateTime(curr);
			ycqQmUserHonor.setDeleted(QmConstants.DELETED_1);
		}
		ycqQmUserHonorRepo.saveAndFlush(ycqQmUserHonor);
		result.setData("删除成功");
		return result;
	}


}
