package com.crunii.micro.service.qm.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQmUserAuditRecord;
import com.crunii.micro.domain.entity.main.YcqQmUserInfo;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.qm.api.constants.QmConstants;
import com.crunii.micro.service.qm.api.domain.dto.org.OrgTreeReqDto;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameDto;
import com.crunii.micro.service.qm.api.domain.dto.org.PageOrgByOrgNameVo;
import com.crunii.micro.service.qm.api.domain.dto.org.vo.OrgTreeVo;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.UserInfoDto;
import com.crunii.micro.service.qm.api.domain.dto.userinfo.vo.*;
import com.crunii.micro.service.qm.config.QmFlowConfig;
import com.crunii.micro.service.qm.jdbc.UserInfoDao;
import com.crunii.micro.service.qm.service.CommonService;
import com.crunii.micro.service.qm.service.UserInfoService;
import com.crunii.micro.service.qm.util.ListUtil;
import com.crunii.micro.service.qm.util.Sm2Util;
import com.crunii.micro.service.qm.util.UserCodeUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

import static com.crunii.micro.service.qm.api.constants.QmConstants.*;

/**
 * @author xia
 * @version 1.0
 * @date 2023/9/18 0018 15:12:32
 */
@Slf4j
@Service
public class UserInfoServiceImpl implements UserInfoService {

	@Autowired YcqQmUserInfoRepo ycqQmUserInfoRepo;

	@Resource
	private QmFlowConfig qmFlowConfig;

	@Autowired YcqQmUserClassesRelRepo ycqQmUserClassesRelRepo;
	@Autowired YcqQmClassesInfoRepo ycqQmClassesInfoRepo;
	@Autowired YcqQmTeacherExpandRepo ycqQmTeacherExpandRepo;

	@Autowired UserInfoDao userInfoDao;

	@Autowired CommonService commonService;
	@Resource
	private YcqQmFlowRecordRepo ycqQmFlowRecordRepo;
	/**
	 * 判断用户是否已录入信息
	 *
	 * @param loginId 用户id
	 * @return Result<CheckUserInfoVo>
	 */
	@Override
	@InjectProperty
	public Result<CheckUserInfoVo> checkUserInfo(int loginId) {
		Result<CheckUserInfoVo> result = new Result<>();
		YcqQmUserInfo userInfo = ycqQmUserInfoRepo.findByWorkerId(loginId);
		if(ObjectUtil.isNotEmpty(userInfo)&&userInfo.getDeleted()== DELETED_1&&userInfo.getUserIdentity()== USER_IDENTITY_STUDENT){
			CheckUserInfoVo vo = new CheckUserInfoVo();
			vo.setStuState(DELETED_1);//学员已被淘汰
			result.setData(vo);
			return result;
		}

		CheckUserInfoVo checkUserInfoVo = userInfoDao.selectUserInfoByWorkerId(loginId);
		if (ObjectUtil.isNotEmpty(checkUserInfoVo)) {
			//身份证解密
			checkUserInfoVo.setCardId(Sm2Util.decrypt(QmConstants.PRIVATE_KEY, false, checkUserInfoVo.getCardId()));

			//查询审核意见
			if (QmConstants.AUDIT_STATUS_WAIT != checkUserInfoVo.getAuditStatus()) {
				//审核失败和审核通过均需要查询审核时间
				YcqQmUserAuditRecord ycqQmUserAuditRecord = userInfoDao.selectAuditRecordByUserId(checkUserInfoVo.getId());
				if (ObjectUtil.isNotEmpty(ycqQmUserAuditRecord)) {
					checkUserInfoVo.setAuditStatus(ycqQmUserAuditRecord.getAuditStatus());
					checkUserInfoVo.setAuditTime(ycqQmUserAuditRecord.getAuditTime());
					checkUserInfoVo.setNote(ycqQmUserAuditRecord.getNote());
				}
			}
			//判断是导师还是学员
			if (checkUserInfoVo.getUserIdentity() == QmConstants.USER_IDENTITY_STUDENT) {
				//查询班级信息
				List<String> classesList = new ArrayList<>();
				List<HomePageClassesVo> classesVoList = userInfoDao.selectClassesByWorkerId(checkUserInfoVo.getWorkerId());

				if (ListUtil.hasValue(classesVoList)) {
					for (HomePageClassesVo homePageClassesVo : classesVoList) {
						String classDesc = homePageClassesVo.getClassesYear()
								+ "/" + homePageClassesVo.getLevelName()
								+ "/" + homePageClassesVo.getTypeName();

						classesList.add(classDesc);
					}
				}
				checkUserInfoVo.setClassesList(classesList);
			} else if (checkUserInfoVo.getUserIdentity() == QmConstants.USER_IDENTITY_TEACHER) {
				//查询导师级别
				ClassesLevelVo classesLevelVo = userInfoDao
						.selectLevelByWorkerId(checkUserInfoVo.getWorkerId());
				if (ObjectUtil.isNotEmpty(classesLevelVo)) {
					checkUserInfoVo.setLevelCode(classesLevelVo.getLevelCode());
					checkUserInfoVo.setLevelName(classesLevelVo.getLevelName());
				}

			}
			result.setData(checkUserInfoVo);
		}
		return result;
	}

	/**
	 * 新增用户
	 *
	 * @param worker 人员实体
	 * @param dto    入参实体
	 * @return Result<String>
	 */
	@Override public Result<String> insertUserInfo(Worker worker, UserInfoDto dto) {
		Result<String> result = new Result<>();
		Date curr = new Date();
		//查询是否录入过，有则修改，没有则新增
		YcqQmUserInfo userInfo = ycqQmUserInfoRepo.findByWorkerId(worker.getLoginId());
		if (ObjectUtil.isEmpty(userInfo)) {
			//新增
			userInfo = new YcqQmUserInfo();
			BeanUtils.copyProperties(dto, userInfo);
			userInfo.setWorkerId(worker.getLoginId());
			userInfo.setAuditStatus(QmConstants.AUDIT_STATUS_WAIT);
			userInfo.setCreatorId(worker.getLoginId());
			userInfo.setCreateTime(curr);
			userInfo.setUpdaterId(worker.getLoginId());
			userInfo.setUpdateTime(curr);
			userInfo.setDeleted(QmConstants.DELETED_0);
			//加密身份证
			userInfo.setCardId(Sm2Util.encrypt(QmConstants.PUBLIC_KEY, false, userInfo.getCardId()));

			if (dto.getUserIdentity() == QmConstants.USER_IDENTITY_STUDENT) {
				if (ListUtil.hasValue(dto.getClassesIdList())) {
					//新增班级与学员关联
					commonService.insertUserClassesRel(worker.getLoginId(), worker.getLoginId(), dto.getClassesIdList());
					// 新增用户的组织id,先根据班级id查询
					YcqQmUserInfo findOrgId = userInfoDao.findOrgIdByClassesId(dto.getClassesIdList().get(0));
					userInfo.setOrgId(findOrgId.getOrgId());
				}
				String firstClassesYear = userInfoDao.selectFirstClassesByWorkerId(userInfo.getWorkerId());
				if (ObjectUtil.isEmpty(firstClassesYear)) {
					firstClassesYear = String.valueOf(LocalDate.now().getYear());
				}
				String stuCode = UserCodeUtil.getStuCode(Integer.valueOf(firstClassesYear));
				userInfo.setUserCode(stuCode);
			} else if (dto.getUserIdentity() == QmConstants.USER_IDENTITY_TEACHER) {
				//新增或修改导师级别
				commonService.insertTeacherExpand(worker.getLoginId(), userInfo.getWorkerId(), dto.getLevelCode());
				userInfo.setUserCode(UserCodeUtil.getTeacherCode(DateUtil.year(curr)));
			} else {
				return Result.ofFail("用户身份类型有误");
			}
		} else {
			if(userInfo.getDeleted()== DELETED_1){
				throw new BusinessException("学员已被淘汰，无法添加!");
			}
			//有值，修改
			BeanUtils.copyProperties(dto, userInfo);
			//加密身份证
			userInfo.setCardId(Sm2Util.encrypt(QmConstants.PUBLIC_KEY, false, userInfo.getCardId()));

			userInfo.setAuditStatus(QmConstants.AUDIT_STATUS_WAIT);
			userInfo.setUpdaterId(worker.getLoginId());
			userInfo.setUpdateTime(curr);

			if (dto.getUserIdentity() == QmConstants.USER_IDENTITY_STUDENT) {
				if (ListUtil.hasValue(dto.getClassesIdList())) {
					//新增班级与学员关联
					commonService.insertUserClassesRel(worker.getLoginId(), worker.getLoginId(), dto.getClassesIdList());
					// 更改用户的组织id,先根据班级id查询
					YcqQmUserInfo findOrgId = userInfoDao.findOrgIdByClassesId(dto.getClassesIdList().get(0));
					userInfo.setOrgId(findOrgId.getOrgId());
				}
				String firstClassesYear = userInfoDao.selectFirstClassesByWorkerId(userInfo.getWorkerId());
				if (ObjectUtil.isEmpty(firstClassesYear)) {
					firstClassesYear = String.valueOf(LocalDate.now().getYear());
				}
				String stuCode = UserCodeUtil.getStuCode(Integer.valueOf(firstClassesYear));
				userInfo.setUserCode(stuCode);
			} else if (dto.getUserIdentity() == QmConstants.USER_IDENTITY_TEACHER) {
				//新增或修改导师级别
				commonService.insertTeacherExpand(worker.getLoginId(), userInfo.getWorkerId(), dto.getLevelCode());
				userInfo.setUserCode(UserCodeUtil.getTeacherCode(DateUtil.year(curr)));
			}
		}

		YcqQmUserInfo ycqQmUserInfo = ycqQmUserInfoRepo.saveAndFlush(userInfo);

	/*	if(EnvUtil.getEnvIsOnline()){
			createFlow(worker,ycqQmUserInfo.getId());
		}*/
		result.setData("录入成功");
		return result;
	}

	/**
	 * 创建工作流
	 */
/*	private void createFlow(Worker worker, Long tutorTraineeId) {
		FlowIdProperty insetUser = qmFlowConfig.getInsertUser();
		FlowStartDTO flowStartDTO = new FlowStartDTO();
		flowStartDTO.setFlowDeployId(insetUser.getDeployId());
		flowStartDTO.setFlowModuleId(insetUser.getModuleId());
		flowStartDTO.setBizId(String.valueOf(tutorTraineeId));
		flowStartDTO.setFlowTitle(INSERT_USER_TITLE);
		FlowStarterInfoDTO flowStarterInfoDTO = new FlowStarterInfoDTO();
		flowStarterInfoDTO.setUserId((long) worker.getLoginId());
		flowStarterInfoDTO.setUserName(worker.getWorkerName());
		String orgId = worker.getOrgId();
		flowStarterInfoDTO.setOrgId(Objects.nonNull(orgId)?Long.valueOf(orgId):null);
		flowStarterInfoDTO.setOrgName(worker.getOrgName());
		flowStartDTO.setStarter(flowStarterInfoDTO);
		CommonResult<CruniiStartProcessResult> result;
		try {
			result = flowInstanceApi.startProcess(flowStartDTO);
		} catch (Exception e) {
			log.info("远程调用流程执行失败@！参数-》{}", JsonUtil.getJsonString(flowStartDTO));
			throw new BusinessException("执行流程调用失败！");
		}
		if(!result.isSuccess()){
			throw new BusinessException("执行流程调用失败！");
		}
		CruniiStartProcessResult data = result.getData();
		String flowInstanceId = data.getFlowInstanceId();
		String nodeInstanceId = data.getActiveTaskInstance().getNodeInstanceId();

		// 用于查询流程状态
		YcqQmFlowRecord ycqQmFlowRecord = ycqQmFlowRecordRepo.findByServiceId(tutorTraineeId);
		if(Objects.isNull(ycqQmFlowRecord)){
			ycqQmFlowRecord = new YcqQmFlowRecord();
		}
		ycqQmFlowRecord.setServiceId(tutorTraineeId);
		ycqQmFlowRecord.setFlowInstanceId(flowInstanceId);
		ycqQmFlowRecord.setCreatorId(worker.getLoginId());
		ycqQmFlowRecord.setCreateTime(new Date());
		ycqQmFlowRecord.setNodeInstanceId(nodeInstanceId);
		ycqQmFlowRecordRepo.saveAndFlush(ycqQmFlowRecord);
	}*/


	/**
	 * 根据年度和类型查询班级列表
	 *
	 * @param worker      人员实体
	 * @param classesYear 班级年度
	 * @param levelCode   班级级别code
	 * @param orgId       班级组织id
	 * @return Result<List < ClassesVo>>
	 */
	@Override
	@InjectProperty
	public Result<List<ClassesTypeVo>> selectClassesTypeByYearAndLevel(Worker worker, String classesYear, String levelCode, Long orgId) {
		Result<List<ClassesTypeVo>> result = new Result<>();
		result.setData(userInfoDao.selectClassesTypeByYearAndLevel(classesYear, levelCode, orgId));
		return result;
	}

	/**
	 * 查询用户信息
	 *
	 * @param loginId 人员id
	 * @return Result<UserInfoVo>
	 */
	@Override public Result<UserInfoVo> selectUserInfo(int loginId) {
		Result<UserInfoVo> result = new Result<>();
		UserInfoVo userInfoVo = new UserInfoVo();
		YcqQmUserInfo userInfo = ycqQmUserInfoRepo.findByWorkerIdAndDeleted(loginId, QmConstants.DELETED_0);
		if (ObjectUtil.isEmpty(userInfo)) {
			BeanUtils.copyProperties(userInfo, userInfoVo);
		} else {
			return Result.ofFail("未查询到该用户信息");
		}
		result.setData(userInfoVo);
		return result;
	}

	/**
	 * 查询所有班级包含的年份
	 *
	 * @return Result<List < String>>
	 */
	@Override public Result<List<String>> selectClassesYearByAll() {
		Result<List<String>> result = new Result<>();
		result.setData(userInfoDao.selectClassesYearByAll());
		return result;
	}

	/**
	 * 根据年份查询班级的所有级别
	 *
	 * @param classesYear 班级年份
	 * @return Result<List < ClassesLevelVo>>
	 */
	@Override public Result<List<ClassesLevelVo>> selectClassesLevelByYear(String classesYear) {
		Result<List<ClassesLevelVo>> result = new Result<>();
		result.setData(userInfoDao.selectClassesLevelByYear(classesYear));
		return result;
	}

	/**
	 * 根据组织名称查询组织信息
	 *
	 * @param req 入参实体
	 * @return PageRsp < PageOrgByOrgNameVo>
	 */
	@Override public PageRsp<PageOrgByOrgNameVo> pageOrgByOrgName(PageReq<PageOrgByOrgNameDto> req) {
		PageRsp<PageOrgByOrgNameVo> rsp = userInfoDao.pageOrgByOrgName(req);
		List<PageOrgByOrgNameVo> result = rsp.getResult();
		if (CollectionUtils.isEmpty(result)) {
			return rsp;
		}

		for (PageOrgByOrgNameVo pageOrgByOrgNameVo : result) {

			List<String> parentOrgNameList = userInfoDao.findParentOrgNameByOrgId(pageOrgByOrgNameVo.getOrgId());

			if (CollectionUtils.isEmpty(parentOrgNameList)) {
				continue;
			}
			StringBuilder parentOrgNameAllBuilder = new StringBuilder();
			for (String parentOrgName : parentOrgNameList) {
				parentOrgNameAllBuilder.append(parentOrgName + " < ");
			}
			String parentOrgNameAll = parentOrgNameAllBuilder.substring(0, parentOrgNameAllBuilder.length() - 3);
			pageOrgByOrgNameVo.setParentOrgNameAll(parentOrgNameAll);

//			List<PageOrgByOrgNameVo> list = userInfoDao.findParentOrgByOrgId(pageOrgByOrgNameVo.getOrgId());
//			List<PageOrgByOrgNameVo> parentOrgList = new LinkedList<>();
//			if(CollectionUtils.isEmpty(list)){
//				continue;
//			}
//			Long parentOrgId = pageOrgByOrgNameVo.getParentOrgId();
//			Map<Long,PageOrgByOrgNameVo> mapOrg = new HashMap<>(list.size()*2);
//			for (PageOrgByOrgNameVo orgTreeVo : list) {
//				mapOrg.put(orgTreeVo.getOrgId(),orgTreeVo);
//			}
//			for (int i = 0; i < mapOrg.size(); i++) {
//				PageOrgByOrgNameVo parentOrg = mapOrg.get(parentOrgId);
//				parentOrgList.add(parentOrg);
//				if(Objects.nonNull(parentOrg) && parentOrg.getParentOrgId()!=null){
//					parentOrgId = parentOrg.getParentOrgId();
//				}else {
//					break;
//				}
//			}
//			pageOrgByOrgNameVo.setParentOrgList(parentOrgList);
		}

		return rsp;
	}

	/**
	 * 青马工程查询组织树
	 *
	 * @param worker 人员实体
	 * @param param  入参实体
	 * @return List<OrgTreeVo>
	 */
	@Override public List<OrgTreeVo> getOrgChildrenTree(Worker worker, OrgTreeReqDto param) {
		return  userInfoDao.findOrgChildrenTree(param,worker.getOrgId());
	}

	/**
	 * APP根据班级级别查询组织
	 *
	 * @param worker 人员实体
	 * @param param  入参实体
	 * @return List<OrgTreeVo>
	 */
	@Override public List<OrgTreeVo> getOrgChildrenTreeByOrgLevel(Worker worker, OrgTreeReqDto param) {
		return  userInfoDao.getOrgChildrenTreeByOrgLevel(param);
	}

}
