package com.crunii.micro.service.qm.util;

import cn.hutool.core.lang.RegexPool;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.entity.main.YcqQmClassesInfo;
import com.crunii.micro.domain.entity.main.YcqQmUserInfo;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;
import com.crunii.micro.domain.repo.main.YcqQmClassesInfoRepo;
import com.crunii.micro.domain.repo.main.YcqQmUserInfoRepo;
import com.crunii.micro.service.qm.api.domain.dto.ExcelMistakeData;
import com.crunii.micro.service.qm.api.domain.dto.studentWeb.ExcelStuData;
import com.crunii.micro.service.qm.jdbc.StudentManageWebDao;
import com.crunii.micro.service.qm.jdbc.TeacherManageDao;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qm.api.constants.QmConstants.*;

/**
 * @author baoxy
 * @date 2023-10-10
 */
@Component
public class ExcelStuVerify {

    @Resource
    private TsmWorkerRepo repo;

    @Resource
    private YcqQmUserInfoRepo infoRepo;

    List<ExcelMistakeData> list;

    @Resource
    private TeacherManageDao teacherManageDao;

    @Resource
    private StudentManageWebDao studentManageWebDao;

    @Resource
    private YcqQmClassesInfoRepo ycqQmClassesInfoRepo;

    public List<ExcelMistakeData> data(ExcelStuData excelStuData, String raw, List<String> phones, List<String> idCards) {
        list = new ArrayList<>();
        // 行数
        ExcelMistakeData data = new ExcelMistakeData();
        data.setName("行数");
        data.setValue(raw);
        data.setFlag(true);
        list.add(data);

        name(excelStuData);
        phone(excelStuData, phones);
        sex(excelStuData);
        idCard(excelStuData, idCards);
        affiliation(excelStuData);
//        teacher(excelStuData);
        classesCode(excelStuData);
        politicalAspect(excelStuData);
        highestEducation(excelStuData);
        office(excelStuData);
        region(excelStuData);
        workplace(excelStuData);
        belongingSystem(excelStuData);
        List<Integer> state = list.stream().map(ExcelMistakeData::getState).collect(Collectors.toList());
        if (state.contains(0)) {
            return list;
        } else {
            list = new ArrayList<>();
        }
        return list;
    }

    /**
     * 姓名
     */
    public void name(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("姓名");
        if (StrUtil.isEmpty(excelStuData.getName())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            mistakeData.setFlag(true);
            mistakeData.setValue(excelStuData.getName());
        }
        list.add(mistakeData);
    }

    /**
     * 手机号
     */
    public void phone(ExcelStuData excelStuData, List<String> phones) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("手机号");
        if (StrUtil.isEmpty(excelStuData.getPhone())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
//            TsmWorker worker = repo.findByMobileAndState(excelStuData.getPhone(), 1);

            String checkPhone = studentManageWebDao.checkPhone(excelStuData.getPhone());
            if (!Pattern.matches(RegexPool.MOBILE, excelStuData.getPhone())) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("不符合规范");
                mistakeData.setState(0);
            } else if (StrUtil.isNotEmpty(checkPhone)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("手机号已存在");
                mistakeData.setState(0);
                // 判断手机号是否重复
            } else if (phones.stream().anyMatch(phoneNumber -> phoneNumber.equals(excelStuData.getPhone()))) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("手机号重复");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelStuData.getPhone());
                mistakeData.setFlag(true);
            }
        }
        list.add(mistakeData);
    }

    /**
     * 所属组织
     */
    public void affiliation(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("所属组织");
        if (StrUtil.isEmpty(excelStuData.getAffiliation())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            if(NumberUtil.isNumber(excelStuData.getAffiliation())){
                Long orgId = studentManageWebDao.checkOrgId(excelStuData.getAffiliation());
                if (ObjectUtil.isEmpty(orgId)) {
                    mistakeData.setFlag(false);
                    mistakeData.setErrText("组织不存在");
                    mistakeData.setState(0);
                } else {
                    mistakeData.setValue(excelStuData.getAffiliation());
                    mistakeData.setFlag(true);
                }
            }else {
                mistakeData.setFlag(false);
                mistakeData.setErrText("应传入组织ID,详情看导入模板");
                mistakeData.setState(0);
            }
        }

        list.add(mistakeData);
    }

    /**
     * 性别
     */
    public void sex(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("性别");
        if (StrUtil.isEmpty(excelStuData.getSex())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else if (!("男".equals(excelStuData.getSex()) || "女".equals(excelStuData.getSex()))) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("不符合规范");
            mistakeData.setState(0);
        } else {
            mistakeData.setValue(excelStuData.getSex());
            mistakeData.setFlag(true);
        }
        list.add(mistakeData);
    }

    /**
     * 身份证
     */
    public void idCard(ExcelStuData excelStuData, List<String> idCards) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("身份证");
        if (StrUtil.isEmpty(excelStuData.getIdCard())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else if (!Pattern.matches(ID_CARD_REGEX, excelStuData.getIdCard())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("不符合规范");
            mistakeData.setState(0);
        } else if (idCards.stream().anyMatch(phoneNumber -> phoneNumber.equals(excelStuData.getIdCard()))) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("身份证重复");
            mistakeData.setState(0);
        } else {
            YcqQmUserInfo info = infoRepo.findByCardIdAndDeleted(excelStuData.getIdCard(), 0);
            if (ObjectUtil.isNotEmpty(info)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("身份证号已存在");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelStuData.getIdCard());
                mistakeData.setFlag(true);
            }
        }
        list.add(mistakeData);
    }

    /**
     * 班级级别
     */
    public void classesCode(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("所属班级");
        if (StrUtil.isEmpty(excelStuData.getClassesCode())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            if(isAlphanumeric(excelStuData.getClassesCode())){
                // 校验传值的班级编码是否存在
                YcqQmClassesInfo ycqQmClassesInfo = ycqQmClassesInfoRepo.findByClassesCodeAndDeleted(excelStuData.getClassesCode(), DELETED_0);
                if (ObjectUtil.isEmpty(ycqQmClassesInfo)) {
                    mistakeData.setFlag(false);
                    mistakeData.setErrText("所属班级不存在");
                    mistakeData.setState(0);
                } else {
                    mistakeData.setValue(excelStuData.getClassesCode());
                    mistakeData.setFlag(true);
                }
            }else {
                mistakeData.setFlag(false);
                mistakeData.setErrText("班级编码不符合规范，详情看导入模板");
                mistakeData.setState(0);
            }
        }

        list.add(mistakeData);
    }

    /**
     * 校验是否是字母加数字
     */
    private boolean isAlphanumeric(String value) {
        String regex = "^[a-zA-Z0-9]+$";
        return Pattern.matches(regex, value);
    }


    /**
     * 政治面貌
     */
    public void politicalAspect(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("政治面貌");
        if (StrUtil.isEmpty(excelStuData.getPoliticalAspect())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            String code = teacherManageDao.getValueCode(POLITICAL_OUTLOOK, excelStuData.getPoliticalAspect());
            if (ObjectUtil.isEmpty(code)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("政治面貌不存在");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelStuData.getPoliticalAspect());
                mistakeData.setFlag(true);
            }
        }
        list.add(mistakeData);
    }

    /**
     * 最高学历
     */
    public void highestEducation(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("最高学历");
        if (StrUtil.isEmpty(excelStuData.getHighestEducation())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            String code = teacherManageDao.getValueCode(EDUCATION, excelStuData.getHighestEducation());
            if (ObjectUtil.isEmpty(code)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("最高学历不存在");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelStuData.getHighestEducation());
                mistakeData.setFlag(true);
            }
        }
        list.add(mistakeData);
    }

    /**
     * 职务
     */
    public void office(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("职务");
        if (StrUtil.isEmpty(excelStuData.getOffice())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            mistakeData.setValue(excelStuData.getOffice());
            mistakeData.setFlag(true);
        }
        list.add(mistakeData);
    }

    /**
     * 区域
     */
    public void region(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("区域");
        if (StrUtil.isEmpty(excelStuData.getRegion())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            Long areaId = studentManageWebDao.getAreaId(excelStuData.getRegion());
            if (ObjectUtil.isEmpty(areaId)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("区域不存在");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelStuData.getRegion());
                mistakeData.setFlag(true);
            }
        }
        list.add(mistakeData);
    }

    /**
     * 工作单位
     */
    public void workplace(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("工作单位");
        if (StrUtil.isEmpty(excelStuData.getWorkplace())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            mistakeData.setValue(excelStuData.getWorkplace());
            mistakeData.setFlag(true);
        }
        list.add(mistakeData);
    }

    /**
     * 所属系统
     */
    public void belongingSystem(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("所属系统");
        if (StrUtil.isEmpty(excelStuData.getBelongingSystem())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            mistakeData.setValue(excelStuData.getBelongingSystem());
            mistakeData.setFlag(true);
        }
        list.add(mistakeData);
    }

}
