package com.crunii.micro.service.qm.util;

import cn.hutool.core.lang.RegexPool;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.entity.main.YcqQmUserInfo;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;
import com.crunii.micro.domain.repo.main.YcqQmUserInfoRepo;
import com.crunii.micro.service.qm.api.domain.dto.ExcelData;
import com.crunii.micro.service.qm.api.domain.dto.ExcelMistakeData;
import com.crunii.micro.service.qm.jdbc.TeacherManageDao;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qm.api.constants.QmConstants.*;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/9/22 15:11
 */
@Component
public class ExcelVerify {

    @Resource
    private TsmWorkerRepo repo;

    @Resource
    private YcqQmUserInfoRepo infoRepo;

    List<ExcelMistakeData> list;

    @Resource
    private TeacherManageDao teacherManageDao;

    public List<ExcelMistakeData> data(ExcelData excelData, String raw, List<String> phones, List<String> idCards) {
        list = new ArrayList<>();
        // 行数
        ExcelMistakeData data = new ExcelMistakeData();
        data.setName("行数");
        data.setValue(raw);
        data.setFlag(true);
        list.add(data);

        name(excelData);
        phone(excelData, phones);
        affiliation(excelData);
        sex(excelData);
        idCard(excelData, idCards);
        classLevel(excelData);
        politicalAspect(excelData);
        highestEducation(excelData);
        office(excelData);
        region(excelData);
        workplace(excelData);
        belongingSystem(excelData);
        List<Integer> state = list.stream().map(ExcelMistakeData::getState).collect(Collectors.toList());
        if (state.contains(0)) {
            return list;
        } else {
            list = new ArrayList<>();
        }
        return list;
    }

    /**
     * 姓名
     */
    public void name(ExcelData excelData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("姓名");
        if (StrUtil.isEmpty(excelData.getName())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            mistakeData.setFlag(true);
            mistakeData.setValue(excelData.getName());
        }
        list.add(mistakeData);
    }

    /**
     * 手机号
     */
    public void phone(ExcelData excelData, List<String> phones) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("手机号");
        if (StrUtil.isEmpty(excelData.getPhone())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
//            TsmWorker worker = repo.findByMobileAndState(excelData.getPhone(), 1);
            String checkPhone = teacherManageDao.checkPhone(excelData.getPhone());
            if (!Pattern.matches(RegexPool.MOBILE, excelData.getPhone())) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("不符合规范");
                mistakeData.setState(0);
            } else if (StrUtil.isNotEmpty(checkPhone)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("手机号已存在");
                mistakeData.setState(0);
                // 判断手机号是否重复
            } else if (phones.stream().anyMatch(phoneNumber -> phoneNumber.equals(excelData.getPhone()))) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("手机号重复");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelData.getPhone());
                mistakeData.setFlag(true);
            }
        }
        list.add(mistakeData);
    }

    /**
     * 机构
     */
    public void affiliation(ExcelData excelData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("机构");
        if (StrUtil.isEmpty(excelData.getAffiliation())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            if(NumberUtil.isNumber(excelData.getAffiliation())){
                Long orgId = teacherManageDao.getOrgId(Long.parseLong(excelData.getAffiliation()));
                if (ObjectUtil.isEmpty(orgId)) {
                    mistakeData.setFlag(false);
                    mistakeData.setErrText("机构不存在");
                    mistakeData.setState(0);
                } else {
                    mistakeData.setValue(excelData.getAffiliation());
                    mistakeData.setFlag(true);
                }
            }else {
                mistakeData.setFlag(false);
                mistakeData.setErrText("机构ID,详情看导入模板");
                mistakeData.setState(0);
            }
        }

        list.add(mistakeData);
    }

    /**
     * 性别
     */
    public void sex(ExcelData excelData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("性别");
        if (StrUtil.isEmpty(excelData.getSex())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else if (!("男".equals(excelData.getSex()) || "女".equals(excelData.getSex()))) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("不符合规范");
            mistakeData.setState(0);
        } else {
            mistakeData.setValue(excelData.getSex());
            mistakeData.setFlag(true);
        }
        list.add(mistakeData);
    }

    /**
     * 身份证
     */
    public void idCard(ExcelData excelData, List<String> idCards) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("身份证");
        if (StrUtil.isEmpty(excelData.getIdCard())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else if (!Pattern.matches(ID_CARD_REGEX, excelData.getIdCard())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("不符合规范");
            mistakeData.setState(0);
        } else if (idCards.stream().anyMatch(phoneNumber -> phoneNumber.equals(excelData.getIdCard()))) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("身份证重复");
            mistakeData.setState(0);
        } else {
            YcqQmUserInfo info = infoRepo.findByCardIdAndDeleted(excelData.getIdCard(), 0);
            if (ObjectUtil.isNotEmpty(info)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("身份证号已存在");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelData.getIdCard());
                mistakeData.setFlag(true);
            }
        }
        list.add(mistakeData);
    }

    /**
     * 班级级别
     */
    public void classLevel(ExcelData excelData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("班级级别");
        if (StrUtil.isEmpty(excelData.getClassLevel())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            String code = teacherManageDao.getValueCode(CLASS_LEVEL, excelData.getClassLevel());
            if (ObjectUtil.isEmpty(code)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("班级级别不存在");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelData.getClassLevel());
                mistakeData.setFlag(true);
            }
        }

        list.add(mistakeData);
    }

    /**
     * 政治面貌
     */
    public void politicalAspect(ExcelData excelData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("政治面貌");
        if (StrUtil.isEmpty(excelData.getPoliticalAspect())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            String code = teacherManageDao.getValueCode(POLITICAL_OUTLOOK, excelData.getPoliticalAspect());
            if (ObjectUtil.isEmpty(code)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("政治面貌不存在");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelData.getPoliticalAspect());
                mistakeData.setFlag(true);
            }
        }
        list.add(mistakeData);
    }

    /**
     * 最高学历
     */
    public void highestEducation(ExcelData excelData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("最高学历");
        if (StrUtil.isEmpty(excelData.getHighestEducation())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            String code = teacherManageDao.getValueCode(EDUCATION, excelData.getHighestEducation());
            if (ObjectUtil.isEmpty(code)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("最高学历不存在");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelData.getHighestEducation());
                mistakeData.setFlag(true);
            }
        }
        list.add(mistakeData);
    }

    /**
     * 职务
     */
    public void office(ExcelData excelData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("职务");
        if (StrUtil.isEmpty(excelData.getOffice())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            mistakeData.setValue(excelData.getOffice());
            mistakeData.setFlag(true);
        }
        list.add(mistakeData);
    }

    /**
     * 区域
     */
    public void region(ExcelData excelData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("区域");
        if (StrUtil.isEmpty(excelData.getRegion())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            Long areaId = teacherManageDao.getAreaId(excelData.getRegion());
            if (ObjectUtil.isEmpty(areaId)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("区域不存在");
                mistakeData.setState(0);
            } else {
                mistakeData.setValue(excelData.getRegion());
                mistakeData.setFlag(true);
            }
        }
        list.add(mistakeData);
    }

    /**
     * 工作单位
     */
    public void workplace(ExcelData excelData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("工作单位");
        if (StrUtil.isEmpty(excelData.getWorkplace())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            mistakeData.setValue(excelData.getWorkplace());
            mistakeData.setFlag(true);
        }
        list.add(mistakeData);
    }

    /**
     * 所属系统
     */
    public void belongingSystem(ExcelData excelData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("所属系统");
        if (StrUtil.isEmpty(excelData.getBelongingSystem())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            mistakeData.setValue(excelData.getBelongingSystem());
            mistakeData.setFlag(true);
        }
        list.add(mistakeData);
    }

}
