/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.qm.util;

import cn.hutool.core.util.HexUtil;
import cn.hutool.crypto.ECKeyUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.SM2;
import org.bouncycastle.crypto.params.ECPrivateKeyParameters;
import org.bouncycastle.crypto.params.ECPublicKeyParameters;

/**
 * @author shenhs create 2023/6/2 19:43
 */
public class Sm2Util {
	/**
	 * 加密
	 * @param publicKey 公钥
	 * @param isHex 是否是16进制key
	 * @param str 加密字符串
	 * @return String
	 */
	public static String encrypt(String publicKey,boolean isHex,String str){
		if(isHex){
			ECPublicKeyParameters ecPublicKeyParameters = ECKeyUtil.toSm2PublicParams(publicKey);
			SM2 sm2 = new SM2(null, ecPublicKeyParameters);
			byte[] encStr = sm2.encrypt(str, KeyType.PublicKey);
			return HexUtil.encodeHexStr(encStr);
		}else{
			SM2 sm2 = new SM2(null, publicKey);
			return sm2.encryptBase64(str, KeyType.PublicKey);
		}
	}

	/**
	 * 解密
	 * @param privateKey 私钥
	 * @param isHex 是否是16进制key
	 * @param str 加密字符串
	 * @return String
	 */
	public static String decrypt(String privateKey,boolean isHex,String str){
		if(isHex){
			ECPrivateKeyParameters privateKeyParameters = ECKeyUtil.toSm2PrivateParams(privateKey);
			SM2 sm2 = new SM2(privateKeyParameters, null);
			// 在密文前面固定加04，以解决解密时报错：Invalid point encoding 0x5c
			if(!str.startsWith("04")){
				str = "04"+str;
			}
			return sm2.decryptStr(str, KeyType.PrivateKey);
		}else{
			SM2 sm2 = new SM2(privateKey, null);
			return sm2.decryptStr(str,KeyType.PrivateKey);
		}
	}

	public static void main(String[] args) {
		SM2 sm2 = new SM2();
		sm2.initKeys();
		System.out.println("pk："+sm2.getPublicKeyBase64());
		System.out.println("prk："+sm2.getPrivateKeyBase64());
	}
}
