package com.crunii.micro.service.qzh.api.domain.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

@Entity
@Table(name = "YCQ_YJS_STUDENT_COURSE")
@NamedQuery(name = "YcqYjsStudentCourse.findAll", query = "SELECT t FROM YcqYjsStudentCourse t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = {"id"})
@ApiModel(description = "学生课程表")
public class YcqYjsStudentCourse implements Serializable {

    private static final long serialVersionUID = 1L;


    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @ApiModelProperty(name = "id", notes = "主键")
    @Column(name = "id")
    private BigInteger id;

    @Column(name = "student_id")
    @ApiModelProperty(name = "studentId", notes = "学生ID")
    private BigInteger studentId;

    @Column(name = "special_id")
    @ApiModelProperty(name = "specialId", notes = "专题ID")
    private BigInteger specialId;

    @Column(name = "course_id")
    @ApiModelProperty(name = "courseId", notes = "课程ID")
    private BigInteger courseId;

    @Size(max= 128,message="分组ID不能超过128")
    @Column(name = "group_id")
    @ApiModelProperty(name = "groupId", notes = "分组ID：specialId:courseId ")
    private String groupId;

    @Size(max= 128,message="课程名称不能超过128")
    @Column(name = "course_name")
    @ApiModelProperty(name = "courseName", notes = "课程名称")
    private String courseName;

    @Size(max= 128,message="专题类别不能超过128")
    @Column(name = "special_type")
    @ApiModelProperty(name = "specialType", notes = "专题类别")
    private String specialType;

    @Size(max= 128,message="专题类别名称不能超过128")
    @Column(name = "special_name")
    @ApiModelProperty(name = "specialName", notes = "专题类别名称")
    private String specialName;

    @Size(max= 900,message="附件不能超过900")
    @Column(name = "attachment_url")
    @ApiModelProperty(name = "attachmentUrl", notes = "附件，多个逗号隔开")
    private String attachmentUrl;

    @Column(name = "paper_id")
    @ApiModelProperty(name = "paperId", notes = "试卷ID")
    private BigInteger paperId;

    @Column(name = "if_answer")
    @ApiModelProperty(name = "ifAnswer", notes = "是否提交过答卷")
    private Integer ifAnswer;

    @Column(name = "residue_test_count")
    @ApiModelProperty(name = "residueTestCount", notes = "剩余考试次数")
    private Integer residueTestCount;

    @Column(name = "answer_score")
    @ApiModelProperty(name = "answerScore", notes = "答题得分")
    private Integer answerScore;

    @Column(name = "if_pass")
    @ApiModelProperty(name = "ifPass", notes = "是否合格：0-不合格；1-合格")
    private Integer ifPass;

    @Column(name = "course_duration")
    @ApiModelProperty(name = "courseDuration", notes = "课程时长：分钟")
    private Integer courseDuration;

    @Column(name = "course_credit")
    @ApiModelProperty(name = "courseCredit", notes = "课程学分")
    private Integer courseCredit;

    @Column(name = "view_progress")
    @ApiModelProperty(name = "viewProgress",notes = "观看进度，单位百分比")
    private Integer viewProgress;

    @Column(name = "course_video_duration")
    @ApiModelProperty(name = "courseVideoDuration", notes = "课程视频时长，单位：秒")
    private Integer courseVideoDuration;

    @Column(name = "view_video_duration")
    @ApiModelProperty(name = "viewVideoDuration", notes = "观看课程视频时长，单位：秒")
    private Integer viewVideoDuration;

    @Column(name = "learn_progress")
    @ApiModelProperty(name = "learnProgress", notes = "学习进度，单位百分比")
    private Integer learnProgress;

    @Column(name = "complete_status")
    @ApiModelProperty(name = "completeStatus", notes = "完成状态：0-未完成；1-已完成")
    private Integer completeStatus;

    @Column(name = "complete_time")
    @ApiModelProperty(name = "completeTime", notes = "完成时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date completeTime;

    @Column(name = "del_flag")
    @ApiModelProperty(name = "delFlag", notes = "是否删除：0-未删除；1-已删除")
    private Integer delFlag;

    @Column(name = "if_upload_video")
    @ApiModelProperty(name = "ifUploadVideo", notes = "是否上传视频")
    private Integer ifUploadVideo;

    @Column(name = "created_time")
    @ApiModelProperty(name = "createdTime", notes = "创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createdTime;

    @Column(name = "created_id")
    @ApiModelProperty(name = "createdId", notes = "创建人ID")
    private BigInteger createdId;

    @Size(max= 255,message="创建人名称不能超过255")
    @Column(name = "created_name")
    @ApiModelProperty(name = "createdName", notes = "创建人名称")
    private String createdName;

    @Column(name = "last_updated_time")
    @ApiModelProperty(name = "lastUpdatedTime", notes = "最后更新时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastUpdatedTime;

    @Column(name = "last_updated_id")
    @ApiModelProperty(name = "lastUpdatedId", notes = "最后更新人ID")
    private BigInteger lastUpdatedId;

    @Size(max= 255,message="最后更新人名称不能超过255")
    @Column(name = "last_updated_name")
    @ApiModelProperty(name = "lastUpdatedName", notes = "最后更新人名称")
    private String lastUpdatedName;

}