package com.crunii.micro.service.qzh.api.domain.dto.activityPublish;

import com.crunii.micro.service.qzh.api.domain.dto.document.CoverImageDto;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormsReqDTO;
import io.swagger.annotations.ApiModel;
import java.util.List;
import javax.persistence.Column;
import javax.validation.constraints.*;
import lombok.Data;
import org.hibernate.validator.constraints.Length;
import org.springframework.validation.annotation.Validated;

/**
 * 活动新增实体类
 * @Author: zwx
 * @CreateTime: 2023-09-18  19:36:02
 * @Description: 活动新增请求参数
 */
@Data
@ApiModel(description = "活动新增请求参数")
public class ActivityAddReqDTO {
    /**
     * 活动标题
     */
    @NotBlank(message = "活动标题必填")
    @Length(max = 50,message = "活动标题不允许超过50个字符")
    private String activityTitle;

    /**
     * 活动开始时间
     */
    @NotBlank(message = "活动开始时间不能为空")
    @Pattern(regexp = "^\\d{4}-\\d{2}-\\d{2}\\s\\d{2}:\\d{2}:\\d{2}$", message = "时间格式必须是yyyy-MM-dd HH:mm:ss")
    private String startTime;
    /**
     * 活动结束时间
     */
    @NotBlank(message = "活动结束时间不能为空")
    @Pattern(regexp = "^\\d{4}-\\d{2}-\\d{2}\\s\\d{2}:\\d{2}:\\d{2}$", message = "时间格式必须是yyyy-MM-dd HH:mm:ss")
    private String endTime;

    /**
     * 所属系统id
     */
    @NotBlank(message = "所属业务必填")
    private String businessBelong;

    @NotNull(message = "所属类型必填")
    /**
     * 活动类型id
     */
    private Integer activityTypeId;

    /**
     * 参与对象（如果所有组织可见，默认传1）
     */
    @NotNull(message = "参与对象不能为空")
    private List<ActivityOrgDTO> orgIdList;


    /**
     * 是否限定人数（1-限制，0-不限制)
     */
    @Min(value = 0,message = "是否限定人数参数格式不正确")
    @Max(value = 1,message = "是否限定人数参数格式不正确")
    @NotNull(message = "是否限定人数必填")
    private Integer isLimit;

    /**
     * 限定人数
     */
    private Integer limitCount;

    /**
     * 男性人数（青春之约使用）
     */
    private Integer maleCount = 0;

    /**
     * 女性限制人数（青春之约使用）
     */
    private Integer femaleCount = 0;


    /**
     * 活动地址
     */
    @Length(max = 50,message = "活动地址不允许超过50个字符")
    private String activityAddress;

    /**
     * 联系方式
     */
    private String contactInformation;

    /**
     * 活动举办方
     */
    @Length(max = 100,message = "活动举办方不允许超过100个字符")
    private String sponsor;

    /**
     * 活动图片
     */
    private List<CoverImageDto>  headPicUrlList;


    /**
     * 是否需要报名（1-需要，0-不需要）
     */
    @Min(value = 0,message = "是否需要报名参数格式不正确")
    @Max(value = 1,message = "是否需要报名参数格式不正确")
    @NotNull(message = "是否需要报名必填")
    private Integer isApply;

    /**
     * 报名开始时间
     */
    private String applyStartTime;

    /**
     * 报名结束时间
     */
    private String applyEndTime;

    /**
     * 自定义报名表
     */
    private DynamicFormsReqDTO applyForm;

    /**
     * 是否需要签到签退 0否 1是
     */
    @NotNull(message = "是否签到签退必填")
    private Integer neededSign;

    /**
     * 签到签退方式
     */
    private ActivitySignDTO signDTO;

    /**
     *
     * 报名是否需要审核（0否 1是）
     */
    private Integer neededAudit;

    /**
     * 活动评价填写 0默认模板 1动态表单 2不需要反馈
     */
    @NotNull(message = "活动反馈填写必填")
    private Integer replyType;

    /**
     * 自定义反馈表
     */
    private DynamicFormsReqDTO dynamicForm;

    /**
     * 活动内容
     */
    private String activityContent;

    /**
     * 活动内容图片
     */
    private List<CoverImageDto> coverImageFile;

    /**
     * 所在区域
     */
    @NotNull(message = "所在区域不能为空")
    private List<ZoneAddDTO> zoneAppAddDTOList;

    @Data
    @Validated
    public static class ZoneAddDTO {
        /**
         * 区域类型 市级1，区县级2，校级3
         */
        @Min(value = 1, message = "最小值不能小于1")
        @Max(value = 3, message = "最大值不饿能大于3")
        private Integer zoneType;

        /**
         * 属性值（如果为全部或者只选了第一级，则传1）
         */

        @NotBlank(message = "区域code不能为空")
        private String valueCode;

        /**
         * 上一级属性值，如果为第一级，默认传0
         */
        @NotBlank(message = "上级区域code不能为空")
        private String parentCode = "0";

        /**
         * 属性名称
         */
        @NotBlank(message = "区域名称不能为空")
        private String valueName;


    }
}