package com.crunii.micro.service.qzh.api.domain.dto.activityPublish;

import com.crunii.micro.service.qzh.api.domain.dto.DefaultTemplateParticipationVO;

import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormColumnDTO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormColumnVO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormRecordVO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormVO;
import io.swagger.annotations.ApiModel;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.*;
import java.util.List;

/**
 * @Author: 参与详情
 * @CreateTime: 2023-09-18  19:33:30
 * @Description: 查询活动列表响应参数
 */
@Data
@ApiModel(description = "参与详情")
public class ActivityJoinDetailRespVO {
    /**
     * 活动id
     */
    @NotNull(message = "活动id")
    private Long id;
    /**
     * 活动标题
     */
    @NotBlank(message = "活动标题")
    @Length(max = 50,message = "活动标题不允许超过50个字符")
    private String activityTitle;

    /**
     * 所属业务code
     */
    private String businessBelong;


    /**
     * 所属业务
     */
    private String businessBelongName;

    /**
     * 活动类型（1-青年突击队活动，2-青年文明号活动，3-青年安全生产示范岗活动，4-维护青少年权益岗活动）
     */
    private String activityTypeName;

    /**
     * 活动开始时间
     */
    @NotBlank(message = "活动开始时间不能为空")
    @Pattern(regexp = "^\\d{4}-\\d{2}-\\d{2}\\s\\d{2}:\\d{2}:\\d{2}$", message = "时间格式必须是yyyy-MM-dd HH:mm:ss")
    private String startTime;
    /**
     * 活动结束时间
     */
    @NotBlank(message = "活动结束时间不能为空")
    @Pattern(regexp = "^\\d{4}-\\d{2}-\\d{2}\\s\\d{2}:\\d{2}:\\d{2}$", message = "时间格式必须是yyyy-MM-dd HH:mm:ss")
    private String endTime;


    /**
     * 活动地址
     */
    @Length(max = 50,message = "活动地址不允许超过50个字符")
    private String activityAddress;

    /**
     * 活动举办方
     */
    @Length(max = 50,message = "活动举办方不允许超过50个字符")
    private String sponsor;

    /**
     * 联系方式
     */
    @Pattern(regexp = "^0\\d{2,3}-\\d{7,8}$|^1[3-9]\\d{9}$", message = "无效的联系方式")
    private String contactInformation;

    /**
     * 活动状态（1-启用，0-未启用）
     */
    @Min(value = 0,message = "活动状态参数格式为(1-启用，0-未启用)")
    @Max(value = 1,message = "活动状态参数格式为(1-启用，0-未启用)")
    private Integer activityState;

    /**
     * 是否限定人数（1-限制，0-不限制)
     */
    @Min(value = 0,message = "是否限定人数参数格式不正确")
    @Max(value = 1,message = "是否限定人数参数格式不正确")
    private Integer isLimit;

    /**
     * 限制人数
     */
    private Integer limitCount;

    /**
     * 是否需要报名（1-需要，0-不需要）
     */
    @Min(value = 0,message = "是否需要报名参数格式不正确")
    @Max(value = 1,message = "是否需要报名参数格式不正确")
    @NotNull(message = "是否需要报名必填")
    private Integer isApply;

    /**
     * 报名是否需要审核（0否 1是）
     */
    private Integer neededAudit;

    /**
     * 是否需要签到签退 0否 1是
     */
    private Integer neededSign;


    /**
     * 内容是否审核（0否 1是）
     */
    private Integer contentNeededAudit;

    /**
     * 自定义表单模板信息
     */
    DynamicFormVO formVO;

    /**
     * 自定义表单数据
     */
    DynamicFormRecordVO recordVO;

    /**
     * 参与人信息 + 默认模板
     */
    private DefaultTemplateParticipationVO participationDetailsVO;

    /**
     * 报名表模板heads
     */
    String applyJson;

    /**
     * 报名表数据json
     */
    DynamicFormRecordVO applyRecordJson;


    /**
     * 是否存在报名表
     */
    Integer needApply;
}
