package com.crunii.micro.domain.entity.main;

import java.time.LocalDateTime;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import java.io.Serializable;

/**
 * 消息发送记录表(MessageSendLogTask)
 *
 * @author zwx
 * @since 2023-10-27 10:24:48
 */
@Entity
@Table(name = "ycq_message_record")
@NamedQuery(name = "MessageSendLogTask.findAll", query="SELECT t FROM MessageSendLogTask t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@GenericGenerator(name = "idGenerator", strategy = "com.crunii.micro.core.distributed.DistributedSnowFlakeId")
@EqualsAndHashCode(of= {"id"})
@ApiModel(description = "消息发送记录表")
public class MessageSendLogTask implements Serializable {
    private static final long serialVersionUID = 868943754814221907L;

    @Id
    @GeneratedValue(generator = "idGenerator")
    @Column(name = "id")
    @ApiModelProperty(value = "id")
    private Long id;

    @Column(name = "send_id")
    @ApiModelProperty(value = "发送ID")
    private Long sendId;

    @Column(name = "send_way")
    @ApiModelProperty(value = "发送方式(1短信、2站内、3全部)")
    private String sendWay;

    @Column(name = "message_type")
    @ApiModelProperty(value = "消息类型(1PC 、2APP 、3公众号)")
    private String messageType;

    @Column(name = "receiver")
    @ApiModelProperty(value = "消息接收人")
    private String receiver;

    @Column(name = "message_content")
    @ApiModelProperty(value = "消息发送内容")
    private String messageContent;

    @Column(name = "is_read")
    @ApiModelProperty(value = "消息是否已读（0未读、1已读）")
    private Integer isRead;

    @Column(name = "data_source")
    @ApiModelProperty(value = "消息来源")
    private String dataSource;

    @Column(name = "data_title")
    @ApiModelProperty(value = "消息标题")
    private String dataTitle;

    @Column(name = "notify_object")
    @ApiModelProperty(value = "通知对象")
    private String notifyObject;

    @Column(name = "push_terminal")
    @ApiModelProperty(value = "推送终端")
    private String pushTerminal;

    @Column(name = "push_time")
    @ApiModelProperty(value = "推送时间")
    private LocalDateTime pushTime;

    @Column(name = "user_name")
    @ApiModelProperty(value = "用户名")
    private String userName;

    @Column(name = "mobile_phone")
    @ApiModelProperty(value = "手机")
    private String mobilePhone;

    @Column(name = "account_number")
    @ApiModelProperty(value = "账号")
    private String accountNumber;

    @Column(name = "message_classify")
    @ApiModelProperty(value = "消息分类")
    private String messageClassify;

    @Column(name = "login_id")
    @ApiModelProperty(value = "接收人id")
    private Long loginId;

    @Column(name = "send_login_id")
    @ApiModelProperty(value = "发送人id")
    private Long sendLoginId;

    @Column(name = "message_send_task_id")
    @ApiModelProperty(value = "message_send_task_id")
    private Long messageSendTaskId;

    @Column(name = "path_and_json")
    @ApiModelProperty(value = "路由+对应参数")
    private String pathAndJson;

    @Column(name = "expand_param")
    @ApiModelProperty(value = "拓展参数")
    private String expandParam;

}
