package com.crunii.micro.domain.entity.main;

import java.time.LocalDateTime;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import javax.persistence.*;
import java.io.Serializable;

/**
 * 消息发送任务表(MessageSendTask)
 *
 * @author zwx
 * @since 2023-10-27 10:24:48
 */
@Entity
@Table(name = "MESSAGE_SEND_TASK")
@NamedQuery(name = "MessageSendTask.findAll", query="SELECT t FROM MessageSendTask t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of= {"ID"})  
@ApiModel(description = "消息发送任务表")
public class MessageSendTask implements Serializable {
    private static final long serialVersionUID = 868943754814221907L;
    
    @Id
	@GeneratedValue(strategy=GenerationType.IDENTITY)
    @Column(name ="ID")
	@ApiModelProperty(value = "序号")
    private Long id;
    
    /**
     * 发送范围(1全用户、2角色、3组织【全用户可以用 user、role 、org、单独的用户id等】、4单独用户)
     */
    @Column(name = "SCOPE_TYPE")  
    @ApiModelProperty(value = "发送范围(1全用户、2角色(角色id)、3组织(组织id)、4单独用户)")
    private Integer scopeType;  
    /**
     * 范围数据(逗号拼接ID)
     */
    @Column(name = "SCOPE_DATA")  
    @ApiModelProperty(value = "范围数据(全用户(不传id)、2角色(传角色id)、3组织(传组织id)、4单独用户(传用户id))【多个id英文逗号拼接】")
    private String scopeData;  
    /**
     * 发送方式( 1短信 、2站内、3全部)
     */
    @Column(name = "SEND_WAY")  
    @ApiModelProperty(value = "发送方式( 1短信 、2站内)")
    private String sendWay;
    /**
     * 消息类型(1PC 、2APP 、3全部)
     */
    @Column(name = "MESSAGE_TYPE")  
    @ApiModelProperty(value = "消息类型(1PC 、2APP 、3公众号)")
    private String messageType;
    /**
     * 消息发送内容
     */
    @Column(name = "MESSAGE_CONTENT")  
    @ApiModelProperty(value = "消息发送内容")  
    private String messageContent;  
    /**
     * 发/推送状态(1已发、0未发)
     */
    @Column(name = "SEND_STATE")  
    @ApiModelProperty(value = "发/推送状态(1已发、0未发)")  
    private Integer sendState;  
    /**
     * 创建时间
     */
    @Column(name = "CREATE_TIME")  
    @ApiModelProperty(value = "创建时间")  
    private LocalDateTime createTime;
    /**
     * 创建人
     */
    @Column(name = "CREATE_USER")
    @ApiModelProperty(value = "创建人")
    private Integer createUser;
    /**
     * 更新人
     */
    @Column(name = "UPDATE_USER")
    @ApiModelProperty(value = "更新人")
    private Integer updateUser;
    /**
     * 更新时间
     */
    @Column(name = "UPDATE_TIME")  
    @ApiModelProperty(value = "更新时间")  
    private LocalDateTime updateTime;

    /**
     * 消息来源
     */
    @Column(name = "DATA_SOURCE")
    @ApiModelProperty(value = "消息来源")
    private String dataSource;
    /**
     * 消息分类
     */
    @Column(name = "MESSAGE_CLASSIFY")
    @ApiModelProperty(value = "消息分类")
    private String messageClassify;

    /**
     * 模板ID
     */
    @Column(name = "TEMPLATE_ID")
    @ApiModelProperty(value = "模板ID")
    private Long templateId;

    /**
     * 发送人id
     */
    @Column(name = "SEND_LOGIN_ID")
    @ApiModelProperty(value = "发送人id")
    private Long sendLoginId;

    /**
     * 用于推送跳转
     */
    @Column(name = "PATH_AND_JSON")
    @ApiModelProperty(value = "路由+对应参数")
    private String pathAndJson;

    @Column(name = "FILL_DATA")
    @ApiModelProperty(value = "JSON填充数据")
    private String fillData;

    @ApiModelProperty(value = "拓展参数")
    @Column(name = "EXPAND_PARAM")
    private String expandParam;

}
