package com.crunii.micro.domain.entity.main;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateTimeSerializer;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.Date;

/**
 * 维权岗管理表
 *
 * @TableName YCQ_QZH_RIGHTS_PROTECTION
 */
@Entity
@Table(name = "YCQ_QZH_RIGHTS_PROTECTION")
@NamedQuery(name = "YcqQzhRightsProtection.findAll", query = "SELECT t FROM YcqQzhRightsProtection t")
@DynamicInsert
@DynamicUpdate
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(of = {"id"})
@ApiModel(description = "YCQ_QZH_RIGHTS_PROTECTION表")
public class YcqQzhRightsProtection implements Serializable, Cloneable {

    /**
     * 序号
     */
    @Id
    @Column(name = "id")
    @ApiModelProperty(value = "id")
    private Long id;

    /**
     * 申报编号
     */
    @Column(name = "applicant_code")
    private String applicantCode;

    /**
     * 单位名称
     */
    @Column(name = "unit_name")
    private String unitName;

    /**
     * 原单位名称
     */
    @Column(name = "old_unit_name")
    private String oldUnitName;

    /**
     * 申请人组织id
     */
    @Column(name = "applicant_org_id")
    private Long applicantOrgId;


    /**
     * 通讯地址
     */
    @Column(name = "address")
    private String address;

    /**
     * 联系电话
     */
    @Column(name = "phone")
    private String phone;

    /**
     * 邮编
     */
    @Column(name = "zip_code")
    private String zipCode;

    /**
     * 所属团组织
     */
    @Column(name = "organization_id")
    private Long organizationId;

    /**
     * 所属团组织
     *
     */
    @Column(name = "organization_name")
    private String  organizationName;

    /**
     * 原所属团组织
     */
    @Column(name = "old_organization_id")
    private Long oldOrganizationId;

    /**
     * 原所属团组织
     *
     */
    @Column(name = "old_organization_name")
    private String  oldOrganizationName;

    /**
     * 所属系统
     */
    @Column(name = "system_id")
    private Long systemId;

    /**
     * 申请人id
     */
    @Column(name = "applicant_id")
    private Long applicantId;


    /**
     * 申请时间
     */
    @Column(name = "apply_time")
    @JsonSerialize(using = LocalDateTimeSerializer.class)
    private LocalDateTime applyTime;

    /**
     * 审核意见
     */
    @Column(name = "opinion")
    private Integer opinion;

    /**
     * 原因
     */
    @Column(name = "reason")
    private String reason;

    /**
     * 复核备注
     */
    @Column(name = "check_notes")
    private String checkNotes;

    /**
     * 状态
     */
    @Column(name = "status")
    private String status;

    /**
     * 类型
     */
    @Column(name = "apply_type")
    private Integer applyType;

    /**
     * 关联政策
     */
    @Column(name = "policy_id")
    private Long policyId;

    /**
     * 创建方案
     */
    @Column(name = "programme")
    private String programme;

    /**
     * 备注
     */
    @Column(name = "notes")
    private String notes;

    /**
     * 当前环节
     */
    @Column(name = "link")
    private String link;

    /**
     * 命名时间
     */
    @Column(name = "naming_time")
    private String namingTime;

    /**
     * 命名文件号
     */
    @Column(name = "naming_file_code")
    private String namingFileCode;

    /**
     * 市级内设机构
     */
    @Column(name = "municipal_level_internal_institutions")
    private Integer municipalLevelInternalInstitutions;


    /**
     * 创建人ID
     */
    @Column(name = "create_id")
    private Long createId;

    /**
     * 创建人姓名
     */
    @Column(name = "create_by")
    private String createBy;

    /**
     * 创建时间
     */
    @Column(name = "create_time")
    @JsonSerialize(using = LocalDateTimeSerializer.class)
    private LocalDateTime createTime;

    /**
     * 更新人ID
     */
    @Column(name = "update_id")
    private Long updateId;

    /**
     * 更新人姓名
     */
    @Column(name = "update_by")
    private String updateBy;

    /**
     * 更新时间
     */
    @Column(name = "update_time")
    @JsonSerialize(using = LocalDateTimeSerializer.class)
    private LocalDateTime updateTime;

    /**
     * 删除标识
     */
    @Column(name = "del_flag")
    private Integer delFlag;


    /**
     * 关联数据id
     */
    @Column(name = "association_id")
    private Long associationId;

    /**
     * systemState
     */
    @Column(name = "system_state")
    private Integer systemState;

    /**
     * 所属主管单位
     */
    @Column(name = "manager_org_id")
    private Long managerOrgId;

    /**
     * 所属主管单位
     *
     */
    @Column(name = "manager_org_name")
    private String managerOrgName;

    /**
     * 负责人
     */
    @Column(name = "header")
    private String header;

    /**
     * 成为权益岗时间
     */
    @Column(name = "create_pro_time")
    private String createProTime;

    @ApiModelProperty("流程实例id")
    @Column(name = "flow_instance_id")
    private String flowInstanceId;

    @ApiModelProperty("流程节点id")
    @Column(name = "node_instance_id")
    private String nodeInstanceId;

    @ApiModelProperty("历史节点")
    @Column(name = "old_node_keys")
    private String oldNodeKeys;

    @ApiModelProperty("市级复核填报单位")
    @Column(name = "filling_unit")
    private String fillingUnit;
}
