package com.crunii.micro.domain.entity.main;

import java.time.LocalDateTime;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import java.io.Serializable;

@Entity
@Table(name = "YOUTH_SAFETY_MODEL_RECOMMEND")
@NamedQuery(name = "YouthSafetyModelRecommend.findAll", query="SELECT t FROM YouthSafetyModelRecommend t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@GenericGenerator(name = "idGenerator", strategy = "com.crunii.micro.core.distributed.DistributedSnowFlakeId")
@EqualsAndHashCode(of= {"id"})  
@ApiModel(description = "青安岗推荐申报")
public class YouthSafetyModelRecommend implements Serializable {
    private static final long serialVersionUID = 295667750794577280L;
    
    @Id
	@GeneratedValue(generator = "idGenerator")
    @Column(name ="id")
	@ApiModelProperty(value = "青安岗推荐表id")
    private Long id;
    /**
     * 申报单位名称
     */
    @Column(name = "apply_org_name")  
    @ApiModelProperty(value = "申报单位名称")  
    private String applyOrgName;  
    /**
     * 所属团组织id
     */
    @Column(name = "affiliation_org_id")
    @ApiModelProperty(value = "所属团组织id")  
    private String affiliationOrgId;
    /**
     * 所属团组织名称
     */
    @Column(name = "affiliation_org_name")
    @ApiModelProperty(value = "所属团组织名称")  
    private String affiliationOrgName;
    /**
     * 申报编号
     */
    @Column(name = "code")
    @ApiModelProperty(value = "申报编号")  
    private String code;

    @Column(name = "leader_id_card")
    @ApiModelProperty("岗长身份证")
    private String leaderIdCard;
    /**
     * 岗长姓名
     */
    @Column(name = "leader_name")
    @ApiModelProperty(value = "岗长姓名")  
    private String leaderName;
    /**
     * 联系电话
     */
    @Column(name = "leader_phone")
    @ApiModelProperty(value = "联系电话")  
    private String leaderPhone;
    /**
     * 创建单位职工人数
     */
    @Column(name = "workers_num")
    @ApiModelProperty(value = "创建单位职工人数")  
    private Integer workersNum;
    /**
     * 35 岁以下青年人数
     */
    @Column(name = "youth_num")
    @ApiModelProperty(value = "35 岁以下青年人数")  
    private Integer youthNum;
    /**
     * 申报类型
     */
    @Column(name = "remark")
    @ApiModelProperty(value = "申报类型")  
    private String remark;
    /**
     * 是否删除(1删除,0未删除)
     */
    @Column(name = "del_flag")
    @ApiModelProperty(value = "是否删除(1删除,0未删除)")  
    private Integer delFlag;
    /**
     * 创建人id
     */
    @Column(name = "create_id")  
    @ApiModelProperty(value = "创建人id")  
    private Long createId;  
    /**
     * 创建人
     */
    @Column(name = "creator")
    @ApiModelProperty(value = "创建人")  
    private String creator;
    /**
     * 创建时间
     */
    @Column(name = "create_time")  
    @ApiModelProperty(value = "创建时间")  
    private LocalDateTime createTime;  
    /**
     * 更新人id
     */
    @Column(name = "update_id")  
    @ApiModelProperty(value = "更新人id")  
    private Long updateId;  
    /**
     * 更新人
     */
    @Column(name = "updator")
    @ApiModelProperty(value = "更新人")  
    private String updator;
    /**
     * 更新时间
     */
    @Column(name = "update_time")  
    @ApiModelProperty(value = "更新时间")  
    private LocalDateTime updateTime;  
    /**
     * 主要事迹
     */
    @Column(name = "major_achievement")
    @ApiModelProperty(value = "主要事迹")  
    private String majorAchievement;
    /**
     * 申报id
     */
    @Column(name = "qag_apply_id")
    @ApiModelProperty(value = "申报id")
    private Long qagApplyId;

    /**
     * 是否为草稿
     */
    @Column(name = "is_draft")
    @ApiModelProperty(value = "是否为草稿(1是，0否)")
    private Integer isDraft;
}
