package com.crunii.micro.domain.repo.main;

import com.crunii.micro.domain.common.BaseRepo;

import com.crunii.micro.domain.entity.main.YcqActivityPeopleAudit;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @author lyx
 * @date 2023/9/7
 */
@Repository
public interface YcqActivityPeopleAuditRepo extends BaseRepo<YcqActivityPeopleAudit,Long> {
    /**
     * 根据用户id和活动id查询唯一用户
     * @param userId 用户id
     * @param activityId 活动id
     * @return
     */
    YcqActivityPeopleAudit findByUserIdAndActivityIdAndIsDeleted(Long userId,Long activityId,Integer isDeleted);


    List<YcqActivityPeopleAudit> findByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(Long userId,Long activityId,Integer isDeleted,Integer registrationStatus);


    YcqActivityPeopleAudit findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(Long userId,Long activityId,Integer isDeleted,Integer registrationStatus);

    YcqActivityPeopleAudit findTopByUserIdAndActivityIdAndIsDeletedAndRegistrationStatusOrderByRegistrationTimeDesc(Long userId, Long activityId, Integer isDeleted, Integer registrationStatus);

    List<YcqActivityPeopleAudit> findByUserIdAndIsDeleted(Long userId,Integer isDeleted);

    List<YcqActivityPeopleAudit> findByUserIdAndActivityIdInAndIsDeleted(Long userId, List<Long> activityIds,Integer isDeleted);

    /**
     * 根据活动id查询用户已报名的人员
     * @param activityId 活动id
     * @param registrationStatus 报名状态
     * @return
     */
    List<YcqActivityPeopleAudit> findByActivityIdAndRegistrationStatus(Long activityId,Integer registrationStatus);

    /**
     * 根据一组ID查询对应的用户列表
     * @param ids 一组ID列表
     * @return 对应ID的用户列表
     */
    List<YcqActivityPeopleAudit> findByIdIn(List<Long> ids);

    List<YcqActivityPeopleAudit> findByActivityIdAndIsDeleted(Long activityId,Integer isDeleted);

    /**
     * @param processState 流程状态  1: 立即报名 2: 立即签到 3: 立即签退 4: 内容填写 5: 已报满 6: 被退回 7: 不展示
     * @param activityId 活动id
     * @param userId 用户id
     * @return
     */
    @Modifying
    @Query(value = "UPDATE ycq_activity_people_audit SET process_state = ?1 WHERE activity_id =?2 AND user_id = ?3 ",nativeQuery = true)
    void updateProcessStateByActivityIdAndUserId(Integer processState,Long activityId, Long userId);

    @Modifying
    @Query(value = "UPDATE ycq_activity_people_audit SET process_state = ?1,registration_status=?4 WHERE activity_id =?2 AND user_id = ?3 ",nativeQuery = true)
    void updateProcessStateByActivityIdAndUserIdAndRegistrationStatus(Integer processState,Long activityId, Long userId,Integer registrationStatus);


    /**
     * 根据活动和用户id逻辑删除
     * @param activityId 活动id
     * @param userId 用户id
     */
    @Modifying
    @Query(value = "UPDATE ycq_activity_people_audit SET is_deleted = 1 WHERE activity_id = ?1 AND user_id = ?2", nativeQuery = true)
    void deleteByActivityIdAndUserId(Long activityId, Long userId);

    /**
     * 根据活动id逻辑删除所有用户
     * @param activityId 活动id
     */
    @Modifying
    @Query(value = "UPDATE ycq_activity_people_audit SET is_deleted = 0 WHERE activity_id = ?1 ", nativeQuery = true)
    void deleteByActivityId(Long activityId);

}
