package com.crunii.micro.domain.repo.main;

import com.crunii.micro.domain.common.BaseRepo;
import com.crunii.micro.domain.entity.main.YcqMienEvaluate;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Set;

import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;

/**
 * @Author thw
 * @date 2023/9/11
 * @description 风采评定表Repo
 */
@Repository
public interface YcqMienEvaluateRepo extends BaseRepo<YcqMienEvaluate,Long> {

  /***
   * @Description: 通过风采ids查询风采是否已经评定过（多个风采会关联同一个指标，一个风采不能关联多个指标）
   * @Author: thw
   * @Date: 2023/9/13 19:06
   * @Params:
   * @param ids
   * @return: java.util.List<com.crunii.micro.domain.entity.main.YcqMienEvaluate>
   **/
  List<YcqMienEvaluate> findAllByMienIdIn(Set<Long> ids);
  
  /**
   * 根据风采id删除对应评定内容-逻辑删除
   * @author leixb
   * @since 2023-11-23 16:15
   * @param mienId 风采id
   * @param updateBy 更新人
   * @param updateId 更新人id
   * @param updateTime 更新时间
   */
  @Transactional(rollbackOn = {Exception.class})
  @Modifying
  @Query("UPDATE YcqMienEvaluate SET delFlag = 1, updateBy = :updateBy, updateId = :updateId, updateTime = :updateTime WHERE mienId = :mienId AND delFlag = 0 ")
  void updateDelFlagByMienId(@Param("mienId") Long mienId, @Param("updateBy") String updateBy, @Param("updateId") Long updateId, @Param("updateTime") LocalDateTime updateTime);
  
  /**
   * 根据风采id和指标id更新指标id
   * @param mienId 风采id
   * @param oldIndicatorId 旧指标id
   * @param newIndicatorId 新指标id
   * @param updateBy 更新人
   * @param updateId 更新人id
   * @param updateTime 更新时间
   */
  @Transactional(rollbackOn = {Exception.class})
  @Modifying
  @Query("UPDATE YcqMienEvaluate SET indicatorId = :newIndicatorId, updateBy = :updateBy, updateId = :updateId, updateTime = :updateTime WHERE mienId = :mienId AND indicatorId = :oldIndicatorId AND delFlag = 0 ")
  void updateByMienIdAndIndicatorId(@Param( "mienId") Long mienId, @Param("oldIndicatorId") Long oldIndicatorId, @Param( "newIndicatorId") Long newIndicatorId, @Param("updateBy") String updateBy, @Param("updateId") Long updateId, @Param("updateTime") LocalDateTime updateTime);
}
