package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.*;
import com.crunii.micro.service.qzh.dto.TestPaperExtentVO;
import com.crunii.micro.service.qzh.service.CivilizationReportService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author wangchen create 2023/11/13 16:51
 */
@Validated
@Api(tags = "文明号申报")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/CivilizationReportController")
public class CivilizationReportController {

    private CivilizationReportService civilizationReportService;

    @Autowired
    public CivilizationReportController(CivilizationReportService civilizationReportService) {
        this.civilizationReportService = civilizationReportService;
    }

    @ApiOperation("文明号申报创建")
    @PostMapping("/crateCivilizationReport")
    public Result<String> crateCivilizationReport(@RequestBody @Validated CivilizationReportDto civilizationReportDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.crateCivilizationReport(civilizationReportDto,worker));
    }

    @ApiOperation("文明号申报草稿创建")
    @PostMapping("/crateCivilizationReportDraft")
    public Result<String> crateCivilizationReportDraft(@RequestBody CivilizationReportDto civilizationReportDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.crateCivilizationReportDraft(civilizationReportDto,worker));
    }


    @ApiOperation("文明号申報分頁查詢")
    @PostMapping("/pageCivilizationReport")
    public Result<PageRsp<CivilizationReportVo>> pageCivilizationReport(@RequestBody PageReq<CivilizationReportPageDto> req, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.pageCivilizationReport(req,worker));
    }

    @ApiOperation("查询关联政策")
    @GetMapping("/findDoc")
    public Result<List<DocVo>> findDoc(@RequestParam(value = "declarationType") Integer declarationType){
        return Result.ofSuccess(civilizationReportService.findDoc(declarationType));
    }

    @ApiOperation("申报编辑回显（当为草稿的情况）")
    @GetMapping("/findDraft")
    public Result<CivilizationReportDto> findDraft(@RequestParam(value = "declarationId") Long declarationId, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.findDraft(declarationId,worker));
    }

    @ApiOperation("删除申报（当为草稿的情况）")
    @GetMapping("/delDraft")
    public Result<String> delDraft(@RequestParam(value = "declarationId") Long declarationId, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.delDraft(declarationId,worker));
    }

    @ApiOperation("初评 || 审核 || 初核")
    @PostMapping("/batchReview")
    public Result<String> batchReview(@RequestBody ReviewDto reviewDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.updateReview(reviewDto,worker));
    }

    @ApiOperation("可回退流程")
    @GetMapping("/getBackFlowIds")
    public Result<List<Integer>> getBackFlowIds(@RequestParam(value = "declarationId") Long declarationId, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.getBackFlowIds(declarationId,worker));
    }

    @ApiOperation("回退流程")
    @PostMapping("/backFlow")
    public Result<String> backFlow(@RequestBody BackFlowDto backFlowDto, @ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.updateBackFlow(backFlowDto,worker));
    }

    @ApiOperation("文明号申报查看-当前登录人-分页查询")
    @PostMapping("/pageMyselfReport")
    public Result<PageRsp<CivilizationReportVo>> pageMyselfReport(@RequestBody PageReq<Object> req,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.pageMyselfReport(req,worker));
    }

    @ApiOperation("查询pdf")
    @GetMapping("/findPdf")
    public ResponseEntity<byte[]> findPdf(@RequestParam(value = "declarationId") Long declarationId, HttpServletResponse response){
        return civilizationReportService.findPdf(declarationId,response);
    }

    @ApiOperation("文明号-考试结果")
    @PostMapping("/createExamResult")
    public Result<String> createExamResult(@RequestBody ExamResultDto req){
        return Result.ofSuccess(civilizationReportService.createExamResult(req));
    }

    @ApiOperation("查询当前用户权限 0 查看 1 市级 2 区县")
    @PostMapping("/findWorkerRole")
    public Result<Integer> findWorkerRole(@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.findWorkerRole(worker));
    }

    @ApiOperation("查询当前用户是否有考试 考试编码 10909 复核\n" +
            "10910  创建和复核\n" +
            "10908 创建")
    @PostMapping("/findWorkerExam")
    public Result<TestPaperExtentVO> findWorkerExam(@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.findWorkerExam(worker));
    }

    @ApiOperation("查询已考次数")
    @PostMapping("/findWorkerExamCount")
    public Result<TestPaperVO> findWorkerExamCount(@RequestParam("paperId")Long paperId ,@ApiIgnore @CurrWorker Worker worker){
        return Result.ofSuccess(civilizationReportService.findWorkerExamCount(paperId,worker));
    }
}
