package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qzh.dto.*;
import com.crunii.micro.service.qzh.service.IndicatorAppraiseService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @Author thw
 * @date 2023/9/5
 * @description 指标体系评价
 */
@Validated
@Api(tags = "指标体系评价")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/indicatorAppraiseController")
public class IndicatorAppraiseController {

  private final IndicatorAppraiseService indicatorAppraiseService;

  @PostMapping(value = "/queryAppraiseList")
  @ApiOperation("指标体系评价列表")
  public Result<List<IndicatorAppraiseRespDTO>> queryAppraiseList( @ApiIgnore @CurrWorker Worker worker, @RequestBody IndicatorAppraiseReqDTO indicatorAppraiseReqDTO){
    return indicatorAppraiseService.queryAppraiseList(worker , indicatorAppraiseReqDTO);
  }

  @PostMapping("/appraiseMienPage")
  @ApiOperation("指标体系评价-评定分页")
  public Result<PageRsp<IndicatorAppraiseDecideRespDTO>> appraiseMienPage( @ApiIgnore @CurrWorker Worker worker , @Validated @RequestBody PageReq<IndicatorAppraiseDecideReqDTO> req){
    return Result.ofSuccess(indicatorAppraiseService.appraiseMienPage(worker , req));
  }

  @PostMapping("/doMienAppraise")
  @ApiOperation("指标体系评价-风采评定")
  public Result<String> doMienEvaluate(@ApiIgnore @CurrWorker Worker worker,@Validated @RequestBody MienAppraiseReqDTO dto){
    return indicatorAppraiseService.doMienEvaluate(worker , dto);
  }

  @PostMapping("/findIndexSystemVersion")
  @ApiOperation("查询指标体系版本下拉")
  public Result<List<IndexSystemSimpleRespDTO>> findIndexSystemVersion(@ApiIgnore @CurrWorker Worker worker){
    return indicatorAppraiseService.findIndexSystemVersion(worker);
  }


  @PostMapping("/findPostInformation")
  @ApiOperation("查询符合条件的维权岗信息")
  public Result<List<CoincidenceOrganizationRespDTO>> findPostInformation(@ApiIgnore @CurrWorker Worker worker, String unitName){
    return indicatorAppraiseService.findPostInformation(worker, unitName);
  }


}
