package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.qzh.api.domain.dto.activity.ParticipantDetailDTO;
import com.crunii.micro.service.qzh.api.domain.dto.ParticipantDetailVO;
import com.crunii.micro.service.qzh.service.ParticipantDetailsService;
import com.crunii.micro.service.qzh.util.EasyExcelUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;


@Api(tags = "活动管理-参与详情管理")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/participantDetailsController")
public class ParticipantDetailsController {

  private final ParticipantDetailsService participantDetailsService;


  /**
   * excel导入文件模板地址
   */
  public static final String EXCEL_DEMO_PATH = "static/word/活动参与详情表.xlsx";


  @ApiOperation("查询列表")
  @PostMapping("/queryList")
  public Result<PageRsp<ParticipantDetailVO>> queryList(@RequestBody @Validated PageReq<ParticipantDetailDTO> req) {
    return participantDetailsService.queryList(req);
  }

  @ApiOperation(value = "批量导出")
  @RequestMapping(path = "/batchExport", method = RequestMethod.POST)
  public void exportParticipantDetailsExcel( @RequestBody ParticipantDetailDTO dto){
    participantDetailsService.exportParticipantDetailsExcel(dto);
  }

  @ApiOperation("活动参与详情导入")
  @PostMapping("/batchImport")
  public Result doImportDate(@RequestBody @Validated ParticipantDetailDTO dto) {
      return participantDetailsService.doImportDate(dto);
  }


  @ApiOperation("校验excel")
  @PostMapping("/verifyExcel")
  public Result doVerifyExcel(MultipartFile file) {
    if (!EasyExcelUtils.isExcelFile(file)) {
      return new Result<>("文件类型出错,请上传正确的文件!",Boolean.FALSE);
    }
    if (file.getSize() > 5 * 1024 * 1024) {
      return new Result<>("文件大小超过5MB限制！", Boolean.FALSE);
    }
    EasyExcelUtils.validateExcelHeaders(EXCEL_DEMO_PATH,file);
    return participantDetailsService.doVerifyExcel(file);


  }

  @ApiOperation(value = "活动参与导入模板下载")
  @GetMapping("/downloadExcelTemp")
  public Object downloadExcelTemp() {
    ResponseEntity<InputStreamResource> response = null;
    try {
      response = EasyExcelUtils.downloadExcelDemo(EXCEL_DEMO_PATH, "活动参与导入模板");
    } catch (Exception e) {
    throw new BusinessException("活动参与详情导入模板下载报错！");
    }
    return response;
  }

}
