package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qzh.api.domain.dto.activity.CreateActivityReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activityPublish.*;
import com.crunii.micro.service.qzh.api.domain.dto.form.*;
import com.crunii.micro.service.qzh.dto.BelongBusinessVo;
import com.crunii.micro.service.qzh.service.PublishActivityService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * @Author zjw
 * @Description 活动2.0
 * @Date  2023/9/18 15:09:18
 **/

@Api(tags = "活动2.0")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/publishActivityController")
public class PublishActivityController {

    @Autowired
    private PublishActivityService service;


    /**
     * 创建活动/复制活动
     * @param dto
     * @return
     */
    @ApiOperation("创建活动")
    @PostMapping("/createActivity")
    public Result<Long> createActivity(@RequestBody @ApiParam @Valid  ActivityAddReqDTO dto) {
        return service.createActivityAdd(dto);
    }

    /**
     * 活动列表
     * @param dto
     * @return
     */
    @ApiOperation("活动列表")
    @PostMapping("/pageActivity")
    public Result<PageRsp<ActivityQueryRespDTO>> pageActivity(@RequestBody @ApiParam PageReq<ActivityQueryReqDTO> dto) {
        return service.activityQuery(dto);
    }


    /**
     * 查看活动
     * @param id
     * @return
     */
    @ApiOperation("查看活动")
    @GetMapping("/getActivity/{id}")
    public Result<ActivityDetailRespDTO> doActivityDetail(@PathVariable("id") Long id) {
        return service.activityDetail(id);
    }

    @ApiOperation("查看原因")
    @PostMapping("/getCause")
    public Result<YcqActivityCauseVO> doCause(@RequestBody @Validated YcqActivityCauseDTO dto) {
        return service.doCause(dto.getActivityId());
    }

    /**
     * 修改活动
     * @param dto
     * @return
     */
    @ApiOperation("修改活动")
    @PostMapping("/updateActivity")
    public Result<String> updateActivity(@RequestBody @ApiParam @Valid ActivityEditReqDTO dto) {
        return service.doActivityEdit(dto);
    }

    /**
     * 修改活动状态
     * @param dto
     * @return
     */
    @ApiOperation("修改活动状态")
    @PostMapping("/updateActivityState")
    public Result<Long> updateActivityState(@RequestBody @ApiParam  @Valid ActivityStateEditReqDTO dto) {
        return service.updateActivityState(dto);
    }


    /**
     * 删除活动
     * @param id
     * @return
     */
    @ApiOperation("删除活动")
    @GetMapping("/deleteActivity/{id}")
    public Result<Boolean> deleteActivity(@PathVariable("id") Long id) {
        return service.doActivityDelete(id);
    }

//    @ApiOperation("复制活动")
//    @GetMapping("/copyActivity/{id}")
//    public Result<Long> copyActivity(@PathVariable("id") Long id) {
//        return service.copyActivity(id);
//    }

    /**
     * @description: 获取二维码
     * @author: limy
     * @date: 2023/11/30 17:00
     * @param: [req]
     * @return: com.crunii.micro.common.dto.Result<com.crunii.micro.service.qzh.api.domain.dto.activityPublish.ActivityQrCodeDTO>
     **/
    @ApiOperation("获取二维码")
    @PostMapping("/getQrCode")
    public Result<List<ActivityQrCodeDTO>> getQrCode(@RequestBody ActivitySignInReqVo req){
        return service.createQrCode(req);
    }



    @ApiOperation("APP-查询活动类型列表")
    @PostMapping("/findActivityTypeList")
    public Result<List<YcqActivityTypeVO>> findActivityTypeList(@RequestBody List<String> codeList){
        return Result.ofSuccess(service.findActivityTypeList(codeList));
    }

    @ApiOperation("查看用户报名提交信息")
    @PostMapping("/getUserApplyInfo")
    public Result<ActivityApplyRecordVO> getApplyInfo(@RequestBody ActivityJoinDetailQueryReqDTO dto){
        return service.getApplyInfo(dto);
    }

    /**
     * 参与详情
     */
    @ApiOperation("参与详情")
    @PostMapping("/getJoinDetail")
    public Result<ActivityJoinDetailRespVO> getJoinDetail(@RequestBody ActivityJoinDetailQueryReqDTO dto){
        return service.getJoinDetail(dto);

    }

    /**
     * APP-活动列表（分页）
     * @param dto
     * @return
     */
    @ApiOperation("APP-活动列表（分页）")
    @PostMapping("/activityQueryForApp")
    public Result<PageRsp<ActivityAppQueryRespDTO>> pageActivityQueryForApp(@RequestBody @ApiParam PageReq<ActivityAppQueryReqDTO> dto) {
        return service.activityQueryForApp(dto);
    }

    @ApiOperation("APP-我的活动列表")
    @PostMapping("/activityQueryForAppQzh")
    public Result<PageRsp<ActivityAppQueryRespDTO>> activityQueryForAppQzh(@RequestBody @ApiParam PageReq<ActivityAppQueryReqDTO> dto) {
        return service.activityQueryForAppQzh(dto);
    }

    /**
     * APP-活动详情
     * @param id
     * @return
     */
    @ApiOperation("APP-活动详情")
    @GetMapping("/activityDetailForApp/{id}")
    public Result<AppActivityDetailRespDTO> pageActivityDetailForApp(@PathVariable("id") Long id) {
        return service.doActivityDetailForApp(id);
    }

    /**
     * APP-报名表表格结构查询（base64编码）
     * @param id
     * @return
     */
    @ApiOperation("APP-报名表表格结构查询")
    @GetMapping("/getApplyFormById/{id}")
    public Result<DynamicFormVO> getApplyFormById(@PathVariable("id") Long id){
        return service.getApplyFormByActivityId(id);
    }

    /**
     * APP-修改活动浏览次数
     * @param id
     * @return
     */
    @ApiOperation("APP-修改活动浏览次数")
    @GetMapping("/updateBrowserCount/{id}")
    public Result<Boolean> updateBrowserCount(@PathVariable("id") Long id){
        service.updateBrowserCount(id);
        return Result.ofSuccess(true);
    }
    /**
     * APP-提交报名表（json提交需要转base64）
     */
    @ApiOperation("APP-提交报名表（json转base64）")
    @PostMapping("/submit")
    public Result<Boolean> submit( @RequestBody DynamicFormRecordSubmitDTO dto){
        return service.submitForm(dto);
    }

    @ApiOperation("取消报名")
    @GetMapping("/cancelApply/{id}")
    public Result<String> doCancelApply(@PathVariable("id") Long id){
        return service.doCancelApply(id);
    }


    /**
     * 活动内容表头（动态表单）
     */
    @ApiOperation("活动内容表头（动态表单）")
    @GetMapping("/getDynamicFormByActivityId/{id}")
    public Result<DynamicFormVO> getDynamicFormByActivityId(@PathVariable("id") Long id){
        return null;
    }

    /**
     * 活动内容（动态表单）
     */
    @ApiOperation("活动内容数据部分（动态表单）")
    @PostMapping("/pageDynamicFormRecord")
    public Result<PageRsp<DynamicFormRecordPageVO>> pageDynamicFormRecord(@RequestBody PageReq<DynamicFormRecordPageDTO> dto){
        return null;
    }


    /**
     * 导出活动内容（动态表单）
     */
    @ApiOperation("导出活动内容（动态表单）")
    @PostMapping("/exportDynamicFormRecord")
    public Result<Boolean> exportDynamicFormRecord(@RequestBody DynamicFormRecordPageDTO dto){
        return null;
    }





//    @ApiOperation("活动报名APP")
//    @PostMapping("/activityApplyForApp")
//    public Result doActivityApply(@RequestParam Long id) {
//        return service.doActivityApply(id);
//    }

//    @ApiOperation("活动报名人数详情APP")
//    @PostMapping("/activityApplyDetailForApp")
//    public Result doActivityApplyDetailForApp(@RequestBody @ApiParam PageReq<ActivityApplyDetailReqDTO> dto) {
//        return service.appActivityApplyDetail(dto,true);
//    }
//
//    @ApiOperation("其他活动参与详情的查看")
//    @PostMapping("/userApplyDetail")
//    public Result userApplyDetail(@RequestBody @ApiParam @Valid ActivityJoinDetailQueryReqDTO dto) {
//        return service.doUserApplyDetail(dto.getId());
//    }
//
//    @ApiOperation("活动状态")
//    @PostMapping("/activityShutDown")
//    public Result activityShutDown(@RequestBody @ApiParam @Valid ActivityStateReqDTO dto) {
//        return service.doActivityShutDown(dto);
//    }
//
//    @ApiOperation("青字号参与活动")
//    @PostMapping("/qZhJoinActivity")
//    public Result qZhJoinActivity(@RequestBody @ApiParam @Valid CreateActivityReqDTO dto) {
//        //青字号活动 发布风采
//        return service.doQZhJoinActivity(dto);
//    }
//
//    @ApiOperation("青字号业务活动参与详情的查看")
//    @PostMapping("/activityJoinDetailQuery")
//    public Result activityJoinDetailQuery(@RequestBody @ApiParam @Valid ActivityJoinDetailQueryReqDTO dto) {
//        return service.doActivityJoinDetailQuery(dto);
//    }
//
//    @ApiOperation("其他业务参与活动(提交表单)APP")
//    @PostMapping("/otherJoinActivity")
//    public Result otherJoinActivity(@RequestBody @ApiParam DynamicFormRecordReqDTO dto) {
//        //其他业务 填写表单
//        return service.doOtherJoinActivity(dto);
//    }

    @ApiOperation("其他业务参与活动表单模板查看")
    @PostMapping("/otherJoinActivityForm")
    public Result otherJoinActivityForm(@RequestParam Long id) {
        return service.doOtherJoinActivityForm(id);
    }

    @ApiOperation("风采列表查询")
    @PostMapping("/showListQuery")
    public Result<PageRsp<ShowListQueryRespDTO>> showListQuery(@RequestBody @ApiParam PageReq<ShowListQueryReqDTO> dto) {
        return service.doShowListQuery(dto);
    }

    @ApiOperation("业务系统下拉框")
    @PostMapping("/businessSelect")
    public Result<BusinessSelectRespDTO> businessSelect() {
        return service.businessSelect();
    }

    @ApiOperation("活动报名人数列表查询")
    @PostMapping("/activityApplyDetail")
    public Result doActivityApplyDetail(@RequestBody @ApiParam PageReq<ActivityApplyDetailReqDTO> dto) {
        return service.activityApplyDetail(dto,false);
    }

    /**
     * 查询所属业务
     */
    @RequestMapping(path = "/getBelongBUsiness", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<BelongBusinessVo>> getBelongBUsiness() {
        return service.getBelongBUsiness();
    }

    /**
     * APP-我的评价
     * @param dto
     * @return
     */
    @RequestMapping(path = "/queryActivityEvaluateList", method = RequestMethod.POST)
    public Result<PageRsp<YcqActivityEvaluateVO>> queryActivityEvaluateList(@RequestBody PageReq<ActivityAppQueryReqDTO> dto){
        return Result.ofSuccess(service.queryActivityEvaluateList(dto));
    }

    /**
     * 活动报道-关联活动数据
     * @return
     */
    @RequestMapping(path = "/queryActivityBaseInfo", method = RequestMethod.POST)
    public Result<PageRsp<ActivityBaseVO>> queryActivityBaseInfo(@RequestBody PageReq<ActivityBaseDTO> req){
        return Result.ofSuccess(service.queryActivityBaseInfo(req));
    }

    /**
     * 活动发布/下架/上架
     * @return
     */
    @RequestMapping(path = "/updatePublishStatus", method = RequestMethod.POST)
    public Result<Boolean> updatePublishStatus(@Validated @RequestBody ActivityPublishStatusEditDTO dto){
        return Result.ofSuccess(service.updatePublishStatus(dto));
    }

    /**
     * 活动-审核列表
     * @return
     */
    @RequestMapping(path = "/pageAuditActivity", method = RequestMethod.POST)
    public Result<PageRsp<ActivityQueryAuditRespDTO>> pageAuditActivity(@RequestBody PageReq<ActivityQueryAuditDTO> dto) {
        return service.pageAuditActivity(dto);
    }
}
