package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.qzh.api.domain.dto.document.CreateEventReportDTO;
import com.crunii.micro.service.qzh.api.domain.dto.document.PageEventReportReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.document.PageEventReportRespDTO;
import com.crunii.micro.service.qzh.api.domain.dto.document.YcqEventReportDTO;
import com.crunii.micro.service.qzh.service.YcqEventReportService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @program: microservice
 * @description: 活动报道controller
 * @author: xiang xin
 * @create: 2024-01-15 15:34
 */
@Api(tags = "活动报道")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/eventReportController")
public class YcqEventReportController {
    private final YcqEventReportService ycqEventReportService;

    @ApiOperation("创建活动报道")
    @PostMapping("/createEventReport")
    public Result<Void> createEventReport(@RequestBody @Validated @ApiParam CreateEventReportDTO dto) {
        return ycqEventReportService.createEventReport(dto);
    }

    @ApiOperation("分页查询活动报道")
    @PostMapping("/pageEventReport")
    public Result<PageRsp<PageEventReportRespDTO>> pageEventReport(@RequestBody @ApiParam PageReq<PageEventReportReqDTO> dto) {
        return ycqEventReportService.pageEventReport(dto);
    }

    @ApiOperation("APP分页查询活动报道")
    @PostMapping("/pageEventReportApp")
    public Result<PageRsp<PageEventReportRespDTO>> pageEventReportApp(@RequestBody @ApiParam PageReq<PageEventReportReqDTO> dto) {
        return ycqEventReportService.pageEventReportApp(dto);
    }

    @ApiOperation("修改活动报道")
    @PostMapping("/updateEventReport")
    public Result updateEventReport(@RequestBody @Validated @ApiParam CreateEventReportDTO dto) {
        return ycqEventReportService.updateEventReport(dto);
    }

    @ApiOperation("修改状态(发布/下架)")
    @PostMapping("/updateType")
    public Result updateType(@RequestBody @Validated @ApiParam YcqEventReportDTO dto) {
        return ycqEventReportService.updateType(dto);
    }


    @ApiOperation("查看活动报道详情")
    @PostMapping("/doEventReport")
    public Result<PageEventReportRespDTO> doEventReport(@RequestBody @Validated @ApiParam YcqEventReportDTO dto) {
        return ycqEventReportService.doEventReport(dto);
    }

    @ApiOperation("删除活动报道")
    @PostMapping("/deleteEventReport")
    public Result deleteYcqQzhActivity(@RequestBody @Validated @ApiParam YcqEventReportDTO dto) {
        return ycqEventReportService.deleteYcqQzhActivity(dto);
    }
}
