package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.qzh.api.domain.dto.activity.*;
import com.crunii.micro.service.qzh.service.YcqQzhActivityService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description: 风采Controller
 * @Param:
 * @return:
 * @Author: xiang xin
 * @Date: 2023/9/18
 */
@Api(tags = "风采")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/activityController")
public class YcqQzhActivityController {

    private final YcqQzhActivityService ycqQzhActivityService;

    @ApiOperation("创建风采")
    @PostMapping("/createYcqQzhActivity")
    public Result<Void> createAnnouncement(@RequestBody @Validated @ApiParam CreateActivityReqDTO dto) {
        return ycqQzhActivityService.createAnnouncement(dto);
    }

    @ApiOperation("分页查询风采")
    @PostMapping("/pageYcqQzhActivity")
    public Result<PageRsp<PageActivityRspDTO>> pageYcqQzhActivity(@RequestBody @ApiParam PageReq<PageActivityReqDTO> dto) {
        return ycqQzhActivityService.pageYcqQzhActivity(dto);
    }

    @ApiOperation("修改风采")
    @PostMapping("/updateYcqQzhActivity")
    public Result updateYcqQzhActivity(@RequestBody @Validated @ApiParam CreateActivityReqDTO dto) {
        return ycqQzhActivityService.updateYcqQzhActivity(dto);
    }

    @ApiOperation("查看风采详情")
    @PostMapping("/doFindDetails")
    public Result<ActivityDetailsRspDTO> doFindDetails(@RequestBody @Validated @ApiParam OperationActivityReqDTO dto) {
        return ycqQzhActivityService.doFindDetails(dto);
    }

    @ApiOperation("删除风采")
    @PostMapping("/deleteYcqQzhActivity")
    public Result deleteYcqQzhActivity(@RequestBody @Validated @ApiParam OperationActivityReqDTO dto) {
        return ycqQzhActivityService.deleteYcqQzhActivity(dto);
    }

    @ApiOperation("转发风采")
    @PostMapping("/doForwardYcqQzhActivity")
    public Result doForwardYcqQzhActivity(@RequestBody @Validated @ApiParam CreateActivityReqDTO dto) {
        return ycqQzhActivityService.doForwardYcqQzhActivity(dto);
    }

    @ApiOperation("查询所属对象")
    @PostMapping("/findParent")
    public Result<List<ParentDTO>> findParent(@RequestParam("type") Integer type) {
        return ycqQzhActivityService.findParent(type);
    }

//    @ApiOperation("通用记录风采浏览量")
//    @PostMapping("/doBrowseActivity")
//    public Result doBrowseActivity(@RequestBody @Validated @ApiParam OperationActivityReqDTO dto) {
//        return ycqQzhActivityService.doBrowseActivity(dto);
//    }

//    @ApiOperation("APP端获取最新10条数据")
//    @PostMapping("/getAnnouncementNewest")
//    public Result<List<PageAnnouncementRspDTO>> getAnnouncementNewest() {
//        return announcementService.getAnnouncementNewest();
//    }

    @ApiOperation("查询关联指标")
    @PostMapping("/findTarget")
    public Result<List<ActivityTargetDTO>> findTarget() {
        List<ActivityTargetDTO> result = ycqQzhActivityService.findTarget();
        return Result.ofSuccess(result);
    }
}
