package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.qzh.api.domain.dto.announcement.*;
import com.crunii.micro.service.qzh.service.YcqQzhAnnouncementService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Description: 公告公示Controller
 * @Param:
 * @return:
 * @Author: xiang xin
 * @Date: 2023/9/18
 */
@Api(tags = "公告公示")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/announcementController")
public class YcqQzhAnnouncementController {

    private final YcqQzhAnnouncementService announcementService;

    @ApiOperation("创建公告公示")
    @PostMapping("/createAnnouncement")
    public Result createAnnouncement(@RequestBody @Validated @ApiParam CreateAnnouncementReqDTO dto) {
        return announcementService.createAnnouncement(dto);
    }

    @ApiOperation("分页查询公告公示")
    @PostMapping("/pageAnnouncement")
    public Result<PageRsp<PageAnnouncementRspDTO>> pageAnnouncement(@RequestBody @ApiParam PageReq<PageAnnouncementReqDTO> dto) {
        return announcementService.pageAnnouncement(dto);
    }

    @ApiOperation("查看公告公示详情")
    @PostMapping("/findAnnouncementDetailsById")
    public Result findAnnouncementDetailsById(@RequestBody @ApiParam OperationAnnouncementReqDTO dto) {
        return announcementService.findAnnouncementDetailsById(dto);
    }

    @ApiOperation("编辑公告公示")
    @PostMapping("/updateAnnouncement")
    public Result updateAnnouncement(@RequestBody @Validated @ApiParam CreateAnnouncementReqDTO dto) {
        return announcementService.updateAnnouncement(dto);
    }

    @ApiOperation("修改公告公示状态")
    @PostMapping("/doDownAnnouncement")
    public Result doDownAnnouncement(@RequestBody @Validated @ApiParam OperationAnnouncementReqDTO dto) {
        return announcementService.doDownAnnouncement(dto);
    }

    @ApiOperation("删除公告公示")
    @PostMapping("/deleteAnnouncement")
    public Result deleteAnnouncement(@RequestBody @Validated @ApiParam OperationAnnouncementReqDTO dto) {
        return announcementService.deleteAnnouncement(dto);
    }

    @ApiOperation("置顶/取消置顶公告公示")
    @PostMapping("/doTopAnnouncement")
    public Result doTopAnnouncement(@RequestBody @Validated @ApiParam OperationAnnouncementReqDTO dto) {
        return announcementService.doTopAnnouncement(dto);
    }

//    @ApiOperation("获取最新五条公告公示")
//    @PostMapping("/getAnnouncementNewest")
//    public Result<List<PageAnnouncementRspDTO>> getAnnouncementNewest() {
//        return announcementService.getAnnouncementNewest();
//    }

//    @ApiOperation("查找申报关联工单")
//    @PostMapping("/findWorkerOrder")
//    public Result<List<WorkerOrderRspDTO>> findWorkerOrder(@RequestBody @ApiParam WorkerOrderReqDTO dto) {
//        return announcementService.findWorkerOrder(dto);
//    }

//    @ApiOperation("记录公告浏览量")
//    @PostMapping("/doBrowseAnnouncement")
//    public Result doBrowseAnnouncement(@RequestBody @Validated  @ApiParam OperationAnnouncementReqDTO dto) {
//        return Result.ofSuccess(announcementService.doBrowseAnnouncement(dto));
//    }

//    @ApiOperation("取消点赞")
//    @PostMapping("/doBackBrowseAnnouncement")
//    public Result doBackBrowseAnnouncement(@RequestBody @ApiParam OperationAnnouncementReqDTO dto) {
//        return Result.ofSuccess();
//    }
}
