package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.qzh.api.domain.dto.commandDo.*;
import com.crunii.micro.service.qzh.service.YcqQzhCommandoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import static org.reflections.Reflections.log;

/**
 * @author ly
 * @date 2023/9/11 18:43
 */
@Validated
@Api(tags = "青字号/青字号管理/突击队", description = "the 青字号/青字号管理/突击队 API")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/ycqQzhCommandoController")
public class YcqQzhCommandoController {

	@Autowired
	private YcqQzhCommandoService commandoService;

	@ApiOperation("创建突击队")
	@PostMapping("/save")
	public Result save(@RequestBody @ApiParam SaveCommandoReqDTO dto) {
		try {
			return commandoService.save(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

//
//    @ApiOperation("突击队申请人草稿查询   -----  没用接口")
//    @PostMapping("/queryCommandoDraft")
//    public Result queryCommandoDraft()
//    {
//        return commandoService.queryCommandoDraft();
//    }

	@ApiOperation("突击队分页审批人查询")
	@PostMapping("/queryCommandoByExamine")
	public Result queryCommandoByExamine(@RequestBody @ApiParam PageReq<QueryCommandoReqDTO> dto) {
		try {
			return commandoService.queryCommandoByExamine(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队分页申请人查询 ")
	@PostMapping("/queryCommandoByApplicant")
	public Result queryCommandoByApplicant(@RequestBody @ApiParam PageReq<QueryCommandoReqDTO> dto) {
		try {
			return commandoService.queryCommandoByApplicant(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队修改队长 ")
	@PostMapping("/changeCommandoLeader")
	public Result updateCommandoLeader(@RequestBody @ApiParam ChangeLeaderCommandoReqDTO dto) {

		try {
			return commandoService.updateCommandoLeader(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队修改 ")
	@PostMapping("/updateCommando")
	public Result updateCommando(@RequestBody @ApiParam CommandoUpdateReqDTO dto) {
		try {
			return commandoService.updateCommando(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队删除 ")
	@PostMapping("/deleteCommando")
	public Result deleteCommando(String id) {
		try {
			return commandoService.deleteCommando(id);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队详情查询 ")
	@PostMapping("/queryById")
	public Result queryById(String id) {
		try {
			return commandoService.queryById(id);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队审核 ")
	@PostMapping("/examineCommando")
	public Result saveExamineCommando(@RequestBody @ApiParam ExamineCommandoReqDTO dto) {
		try {
			return commandoService.saveExamineCommando(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队申报页面变更记录草稿终止任务 ")
	@PostMapping("/saveEndChangeTask")
	public Result saveEndChangeTask(@RequestBody @ApiParam ExamineCommandoReqDTO dto) {
		try {
			return commandoService.saveEndChangeTask(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队排行榜 ")
	@PostMapping("/rankingCommandoList")
	public Result rankingCommandoList(@RequestBody @ApiParam PageReq<CommandoRankListReqDTO> pageReq) {
		try {
			return commandoService.updateRankingCommandoList(pageReq);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队申请人变更记录查询 ")
	@PostMapping("/queryApplicantChange")
	public Result queryApplicantChange(@RequestBody @ApiParam PageReq<QueryCommandoReqDTO> dto) {
		try {
			return commandoService.queryApplicantChange(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队审批人变更记录查询 ")
	@PostMapping("/queryApplicantExamine")
	public Result queryApplicantExamine(@RequestBody @ApiParam PageReq<QueryCommandoReqDTO> dto) {
		try {
			return commandoService.queryApplicantExamine(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("查询突击队变更队长的变更数据")
	@PostMapping("/queryChangeData")
	public Result queryApplicantChange(@RequestBody @ApiParam QueryCommandoReqDTO dto) {
		try {
			return commandoService.queryChangeData(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队流程信息查询")
	@PostMapping("/queryFlow")
	public Result queryFlow(@RequestBody @ApiParam QueryCommandoReqDTO dto) {
		try {
			return commandoService.queryFlow(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

	@ApiOperation("突击队统计待当前用户处理的数据条数")
	@PostMapping("/pendingNum")
	public Result pendingNum(@RequestBody @ApiParam QueryCommandoReqDTO dto) {
		try {
			return commandoService.pendingNum(dto);
		} catch (Exception e) {
			log.info(e.getMessage());
			return   Result.ofFail("数据异常，请联系管理员");
		}
	}

}
