package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhDemoDto;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhDemoVo;
import com.crunii.micro.service.qzh.service.YcqQzhDemoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.List;

import static org.reflections.Reflections.log;

/**
 * YcqQzhDemoController
 *
 * @author xiangjunming
 * @date 2023/9/4
 */
@Validated
@Api(tags = "demo")
@RequiredArgsConstructor
@RestController
@RequestMapping("/YcqQzhDemoController")
public class YcqQzhDemoController {

    private final YcqQzhDemoService service;

    /**
     * 分页查询
     */
    @ApiOperation("分页查询")
    @PostMapping("/queryPageList")
    public Result<PageRsp<YcqQzhDemoVo>> queryPageList(@RequestBody @ApiParam PageReq<YcqQzhDemoDto> req) {
        log.info("req:" + req.toString());
        return Result.ofSuccess(service.queryPageList(req));
    }

    /**
     * 未分页列表
     */
    @ApiOperation("未分页列表")
    @PostMapping("/queryList")
    public Result<List<YcqQzhDemoVo>> queryList(@RequestBody @ApiParam YcqQzhDemoDto req) {
        return Result.ofSuccess(service.queryList(req));
    }

    /**
     * 获取详情
     */
    @ApiOperation("获取详情")
    @GetMapping("/getInfo")
    public Result<YcqQzhDemoVo> getInfo(@ApiParam("主键") @NotNull(message = "主键不能为空") @RequestParam("id") Long id) {
        return Result.ofSuccess(service.queryById(id));
    }

    /**
     * 新增
     */
    @ApiOperation("新增")
    @PostMapping("/add")
    public Result<Void> add(@Validated(AddGroup.class) @RequestBody YcqQzhDemoDto dto) {
        service.insertByDto(dto);
        return Result.ofSuccess();
    }

    /**
     * 修改
     */
    @ApiOperation("修改")
    @PostMapping("/edit")
    public Result<Void> edit(@Validated(EditGroup.class) @RequestBody YcqQzhDemoDto dto) {
        service.updateByDto(dto);
        return Result.ofSuccess();
    }

    /**
     * 删除
     */
    @ApiOperation("删除")
    @PostMapping("/remove")
    public Result<Void> remove(@ApiParam("主键串") @NotEmpty(message = "主键不能为空") @RequestParam("ids") List<Long> ids) {
        service.deleteByIds(ids);
        return Result.ofSuccess();
    }
}
