package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.qzh.api.domain.dto.document.*;
import com.crunii.micro.service.qzh.service.YcqQzhDocumentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Description: 政策Controller
 * @Param:
 * @return:
 * @Author: xiang xin
 * @Date: 2023/9/19
 */
@Api(tags = "政策")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/documentController")
public class YcqQzhDocumentController {

    private final YcqQzhDocumentService ycqQzhDocumentService;

    @ApiOperation("新建政策")
    @PostMapping("/createDocument")
    public Result createDocument(@RequestBody @Validated @ApiParam CreateDocumentReqDTO dto) {
        return ycqQzhDocumentService.createDocument(dto);
    }

    @ApiOperation("分页查询政策")
    @PostMapping("/pageDocument")
    public Result<PageRsp<PageDocumentRspDTO>> pageDocument(@RequestBody @ApiParam PageReq<PageDocumentReqDTO> dto) {
        return ycqQzhDocumentService.pageDocument(dto);
    }

    @ApiOperation("编辑政策")
    @PostMapping("/updateDocument")
    public Result updateDocument(@RequestBody @Validated @ApiParam CreateDocumentReqDTO dto) {
        return ycqQzhDocumentService.updateDocument(dto);
    }

    @ApiOperation("查看政策详情")
    @PostMapping("/findDocumentDetailsById")
    public Result<DocumentDetailsRspDTO> findDocumentDetailsById(@RequestBody @Validated @ApiParam OperationDocumentReqDTO dto) {
        return ycqQzhDocumentService.findDocumentDetailsById(dto);
    }

    @ApiOperation("修改政策状态")
    @PostMapping("/doDownDocument")
    public Result doDownDocument(@RequestBody @Validated @ApiParam OperationDocumentReqDTO dto) {
        return ycqQzhDocumentService.doDownDocument(dto);
    }

    @ApiOperation("删除政策")
    @PostMapping("/deleteDocument")
    public Result deleteDocument(@RequestBody @Validated @ApiParam OperationDocumentReqDTO dto) {
        return ycqQzhDocumentService.deleteDocument(dto);
    }

    @ApiOperation("置顶政策与取消置顶")
    @PostMapping("/doTopDocument")
    public Result doTopDocument(@RequestBody @Validated @ApiParam OperationDocumentReqDTO dto) {
        return ycqQzhDocumentService.doTopDocument(dto);
    }

//    @ApiOperation("文明号申报查询可关联政策")
//    @PostMapping("/doFindDocumentByApplyType")
//    public Result doFindDocumentByApplyType(@RequestBody @ApiParam OperationDocumentReqDTO dto) {
//        return ycqQzhDocumentService.doFindDocumentByApplyType(dto);
//    }

//    @ApiOperation("记录政策浏览量")
//    @PostMapping("/doBrowseDocument")
//    public Result doBrowseDocument(@RequestBody @Validated @ApiParam OperationDocumentReqDTO dto) {
//        return ycqQzhDocumentService.doBrowseDocument(dto);
//    }
}
