package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.dynamicform.YcqQzhDynamicForm;
import com.crunii.micro.domain.entity.main.dynamicform.YcqQzhDynamicFormRecord;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormRecordReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicRecordDetailReqDTO;
import com.crunii.micro.service.qzh.service.YcqQzhDynamicFormService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
 * @MethodName
 * @Description 动态表单模板
 * @Author Jiang
 * @Date 2023/9/27 11:05
 */
@Api(tags = "动态表单模板", description = "动态表单模板 API")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/YcqQzhDynamicFormController")
public class YcqQzhDynamicFormController {

    @Autowired
    YcqQzhDynamicFormService ycqQzhDynamicFormService;


    /**
     * @MethodName createIndicatorSystem
     * @Description PC 新增活动动态表单模板
     * @param: dto
     * @param: worker
     * @return: com.crunii.micro.common.dto.Result<java.lang.String>
     * @Author Jiang
     * @Date 2023/9/27 19:31
     */
    @ApiOperation("新增活动动态表单模板")
    @PostMapping("/createDynamicForm")
    public Result createDynamicForm(@CurrWorker @ApiIgnore Worker worker, @RequestBody DynamicFormReqDTO dynamicFormReq) {
        return ycqQzhDynamicFormService.createDynamicForm(worker, dynamicFormReq);
    }

    /**
     * @MethodName findDynamicFormByActivityId
     * @Description APP端和PC可以公用 通过活动Id获取活动表单数据
     * @param: worker
     * @param: activityId
     * @return: com.crunii.micro.common.dto.Result<java.util.List < com.crunii.micro.service.qzh.api.domain.dto.HomePageTopRespDTO>>
     * @Author Jiang
     * @Date 2023/9/27 21:59
     */
    @ApiOperation("APP端和PC可以公用 通过活动Id获取活动表单数据")
    @RequestMapping(value = "/findDynamicFormByActivityId", method = RequestMethod.POST)
    public Result<YcqQzhDynamicForm> findDynamicFormByActivityId(@CurrWorker @ApiIgnore Worker worker, @RequestParam Long activityId) {
        return ycqQzhDynamicFormService.findDynamicFormByActivityId(worker, activityId);
    }


    /**
     * @MethodName createIndicatorSystem
     * @Description PC 通过活动Id和用户获取活动表单数据详情
     * @param: dto
     * @param: worker
     * @return: com.crunii.micro.common.dto.Result<java.lang.String>
     * @Author Jiang
     * @Date 2023/9/27 19:31
     */
    @ApiOperation("PC 通过活动Id和用户获取活动表单数据详情")
    @PostMapping("/findDynamicFormByActivityIdAndUserId")
    public Result<YcqQzhDynamicFormRecord> findDynamicFormByActivityIdAndUserId(@CurrWorker @ApiIgnore Worker worker, @RequestBody DynamicRecordDetailReqDTO dto) {
        return ycqQzhDynamicFormService.findDynamicFormByActivityIdAndUserId(worker, dto);
    }


    /**
     * @MethodName findDynamicFormByActivityId
     * @Description APP端 参加活动提交表单数据
     * @param: worker
     * @param: activityId
     * @return: com.crunii.micro.common.dto.Result<java.util.List < com.crunii.micro.service.qzh.api.domain.dto.HomePageTopRespDTO>>
     * @Author Jiang
     * @Date 2023/9/27 21:59
     */
    @ApiOperation("APP端 参加活动提交表单数据")
    @RequestMapping(value = "/createDynamicFormRecord", method = RequestMethod.POST)
    public Result createDynamicFormRecord(@CurrWorker @ApiIgnore Worker worker, @RequestBody DynamicFormRecordReqDTO dynamicFormRecord) {
        return ycqQzhDynamicFormService.createDynamicFormRecord(worker, dynamicFormRecord);
    }
}
