package com.crunii.micro.service.qzh.controller;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qzh.api.domain.dto.*;
import com.crunii.micro.service.qzh.service.YcqQzhHomePageService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

import static org.reflections.Reflections.log;

/**
 * 青子号首页相关
 *
 * @author jiang
 * @date 2023/9/11 18:43
 */
@Validated
@Api(tags = "青子号首页相关", description = "青子号首页相关 API")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/YcqQzhHomePage")
public class YcqQzhHomePageController {


    @Autowired
    YcqQzhHomePageService ycqQzhHomePageService;


    /**
     * @MethodName pageIndicatorVersion
     * @Description 个人信息及活跃度相关信息（累计登录天数、风采发布量）
     * @param: worker
     * @return: com.crunii.micro.common.dto.Result<com.crunii.micro.service.qzh.api.domain.dto.HomePageUserRelatedRespDTO>
     * @Author Jiang
     * @Date 2023/9/23 9:47
     */
    @ApiOperation("个人信息及活跃度相关信息（累计登录天数、风采发布量）")
    @RequestMapping(value = "/findPersonalActivity", method = RequestMethod.POST)
    public Result<HomePageUserRelatedRespDTO> findPersonalActivity(@CurrWorker @ApiIgnore Worker worker) {
        return ycqQzhHomePageService.findPersonalActivity(worker);
    }


    /**
     * @MethodName findPolicyAndNoticeAndElegantDemeanourTop
     * @Description 首页 政策指引、公告公示、风采展示最新10条
     * @param: worker
     * @return: com.crunii.micro.common.dto.Result<com.crunii.micro.common.dto.PageRsp < com.crunii.micro.service.qzh.api.domain.dto.HomePageTopRespDTO>>
     * @Author Jiang
     * @Date 2023/9/23 10:09
     */
    @ApiOperation("首页 政策指引、公告公示、风采展示最新10条")
    @RequestMapping(value = "/findPolicyAndNoticeAndElegantDemeanourTop", method = RequestMethod.POST)
    public Result<List<HomePageTopRespDTO>> findPolicyAndNoticeAndElegantDemeanourTop(@CurrWorker @ApiIgnore Worker worker, @RequestBody HomePageTopReqDTO homePageTopReq) {
        return ycqQzhHomePageService.findPolicyAndNoticeAndElegantDemeanourTop(worker, homePageTopReq);
    }


}
