package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.validate.AddGroup;
import com.crunii.micro.common.validate.EditGroup;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qzh.api.domain.dto.*;
import com.crunii.micro.service.qzh.service.YcqQzhIndicatorManageService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Api(tags = "指标体系管理")
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/indicatorSystemManageController")
public class YcqQzhIndicatorManageController {

    private final YcqQzhIndicatorManageService service;

    @ApiOperation("指标版本查询")
    @PostMapping("/findIndicatorVersion")
    public Result<List<YcqQzhIndicatorVersionVo>> findIndicatorVersion(@CurrWorker @ApiIgnore Worker worker) {
        return Result.ofSuccess(service.findIndicatorVersion(worker));
    }

    /**
     * 指标版本新增
     */
    @ApiOperation("指标版本新增")
    @PostMapping("/createIndicatorVersion")
    public Result<Void> createIndicatorVersion(@Validated(AddGroup.class) @RequestBody YcqQzhIndicatorVersionDto dto, @CurrWorker @ApiIgnore Worker worker) {
        service.createIndicatorVersion(dto,worker);
        return Result.ofSuccessMsg("新增成功!");
    }

    /**
     * 指标版本修改
     */
    @ApiOperation("指标版本修改")
    @PostMapping("/updateIndicatorVersion")
    public Result<Void> updateIndicatorVersion(@Validated(EditGroup.class) @RequestBody YcqQzhIndicatorVersionDto dto, @CurrWorker @ApiIgnore Worker worker) {
        service.updateIndicatorVersion(dto,worker);
        return Result.ofSuccessMsg("修改成功!");
    }

    /**
     * 指标版本删除
     */
    @ApiOperation("指标版本删除")
    @PostMapping("/deleteIndicatorVersion")
    public Result<Void> deleteIndicatorVersion(@RequestBody List<Long> ids, @CurrWorker @ApiIgnore Worker worker) {
        return service.deleteIndicatorVersion(ids, worker);
    }

    @ApiOperation("指标体系查询")
    @PostMapping("/findIndicatorSystem")
    public Result<List<YcqQzhIndicatorSystemVo>> findIndicatorSystem(@RequestBody YcqQzhIndicatorSystemDto req) {
        return Result.ofSuccess(service.findIndicatorSystem(req));
    }

    /**
     * 指标体系新增
     */
    @ApiOperation("指标体系新增")
    @PostMapping("/createIndicatorSystem")
    public Result<Void> createIndicatorSystem(@RequestBody YcqQzhIndicatorSystemDto dto, @CurrWorker  @ApiIgnore Worker worker) {
        return service.createIndicatorSystem(dto, worker);
    }

    /**
     * 指标体系修改
     */
    @ApiOperation("指标体系修改")
    @PostMapping("/updateIndicatorSystem")
    public Result<Void> updateIndicatorSystem(@RequestBody YcqQzhIndicatorSystemDto dto, @CurrWorker  @ApiIgnore Worker worker) {
        return service.updateIndicatorSystem(dto,worker);
    }

    /**
     * 指标指标体系
     */
    @ApiOperation("指标体系删除")
    @PostMapping("/deleteIndicatorSystem")
    public Result<Void> deleteIndicatorSystem(@RequestBody List<Long> ids, @CurrWorker  @ApiIgnore Worker worker) {
        return service.deleteIndicatorSystem(ids,worker);
    }

    @ApiOperation(value = "指标体系导出")
    @PostMapping(path = "/exportIndicatorSystem")
    public void exportIndicatorSystem(HttpServletResponse response, @RequestBody YcqQzhIndicatorSystemDto dto, @CurrWorker @ApiIgnore Worker worker) {
        service.exportIndicatorSystem(response, dto);
    }

    @ApiOperation("指标体系下拉")
    @PostMapping("/findIndicatorDropdownBox")
    public Result<List<YcqQzhIndicatorIndexRespDTO>> pageIndicatorVersion(@Validated @RequestBody YcqQzhIndicatorIndexReqDTO req) {
        return Result.ofSuccess(service.findIndicatorDropdownBox(req));
    }
}
