package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhOnlineRecordDTO;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhOnlineTrainDTO;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhOnlineTrainTypeDTO;
import com.crunii.micro.service.qzh.service.YcqQzhOnlineTrainService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;


/**
 * @author lmlksy
 * @date 2023/11/21 10:42
 */
@Api(tags = "线上培训", description = "线上培训")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/YcqQzhOnlineTrainController")
public class YcqQzhOnlineTrainController {

    private final YcqQzhOnlineTrainService onlineTrainService;

    @ApiOperation("创建")
    @PostMapping("/create")
    public Result<Boolean> create(@RequestBody YcqQzhOnlineTrainDTO dto){
        return onlineTrainService.create(dto);
    }

    @ApiOperation("创建-观看记录")
    @PostMapping("/createRecord")
    public Result<Boolean> createRecord(@RequestBody YcqQzhOnlineRecordDTO dto){
        return onlineTrainService.createRecord(dto);
    }

    @ApiOperation("编辑")
    @PostMapping("/update")
    public Result<Boolean> update(@RequestBody YcqQzhOnlineTrainDTO dto){
        return onlineTrainService.update(dto);
    }
    /**
     * 详情
     */
    @ApiOperation("详情")
    @GetMapping("/getOne")
    public Result<YcqQzhOnlineTrainDTO> getOne(@ApiParam("主键") @NotNull(message = "主键不能为空") @RequestParam("id") Long id,@RequestParam("type") Long type) {
        return onlineTrainService.getOne(id,type);
    }
    /**
     * 删除
     */
    @ApiOperation("删除")
    @GetMapping("/delete")
    public Result<Boolean> delete(@ApiParam("主键") @NotNull(message = "主键不能为空") @RequestParam("id") Long id) {
        return onlineTrainService.delete(id);
    }

    @ApiOperation("置顶")
    @PostMapping("/isTop")
    public Result<Boolean> isTop(@RequestBody YcqQzhOnlineTrainDTO dto){
        return onlineTrainService.isTop(dto);
    }

    @ApiOperation("分页查询")
    @PostMapping("/trainPage")
    public Result<PageRsp<YcqQzhOnlineTrainDTO>> trainPage(@RequestBody PageReq<YcqQzhOnlineTrainDTO> req) {
        return onlineTrainService.trainPage(req);
    }

    @ApiOperation("观看记录分页查询")
    @PostMapping("/recordPage")
    public Result<PageRsp<YcqQzhOnlineRecordDTO>> recordPage(@RequestBody PageReq<YcqQzhOnlineTrainDTO> req) {
        return onlineTrainService.recordPage(req);
    }

    @ApiOperation("青创专题培训类型集合")
    @PostMapping("/getTrainList")
    public Result<List<YcqQzhOnlineTrainTypeDTO>> getTrainList(@RequestBody PageReq<YcqQzhOnlineTrainDTO> req) {
        return onlineTrainService.getTrainList(req);
    }
}
