package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.entity.main.YcqQzhRightsProtection;
import com.crunii.micro.domain.entity.main.YcqQzhRightsProtectionAnnex;
import com.crunii.micro.domain.entity.main.document.YcqQzhAnnex;
import com.crunii.micro.service.qzh.api.domain.dto.IndicatorAppraiseRespDTO;
import com.crunii.micro.service.qzh.api.domain.dto.document.DocumentListRspDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.*;
import com.crunii.micro.service.qzh.dto.AddCreateDTO;
import com.crunii.micro.service.qzh.service.YcqQzhProtectionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.List;

@Validated
@Api(tags = "维权岗")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/rightsProtectionController")
public class YcqQzhProtectionController {

    @Resource
    private YcqQzhProtectionService service;

    @ApiOperation("申报记录分页查询")
    @PostMapping("/recordPage")
    public Result<PageRsp<RightsProtectionRecordPageRespDTO>> recordPage(@RequestBody PageReq<Object> req) {
        return service.recordPage(req);
    }

    @ApiOperation("修改记录分页查询")
    @PostMapping("/updatePage")
    public Result<PageRsp<RightsProtectionRecordPageRespDTO>> updatePage(@RequestBody PageReq<Object> req) {
        return service.updatePage(req);
    }

    @ApiOperation("申报草稿")
    @PostMapping("/saveDraft")
    public Result<YcqQzhRightsProtection> saveDraft(@RequestBody(required = false) RightsProtectionCreateReqDTO req) {
        return service.saveDraft(req);
    }


    @ApiOperation("申报创建")
    @PostMapping("/create")
    public Result<Boolean> saveCreate(@RequestBody RightsProtectionCreateReqDTO req) {
        return service.saveCreate(req);
    }


    @ApiOperation("创建申报查看")
    @PostMapping("/getCreate")
    public Result<RightsProtectionCreateQueryRespDTO> get(@RequestParam("id") Long id) {
        return service.getCreate(id);
    }

    @ApiOperation("申报删除")
    @PostMapping("/delete")
    public Result<Boolean> delete(@RequestParam("id") Long id) {
        return service.delete(id);
    }


    @ApiOperation("申报修改")
    @PostMapping("/update")
    public Result<Boolean> update(@RequestBody RightsProtectionUpdateReqDTO req) {
        return service.update(req);
    }


    @ApiOperation("申报评审")
    @PostMapping("/operate")
    public Result<Object> operate(@RequestBody RightsProtectionOperateReqDTO req) {
        return service.doOperate(req);
    }


    @ApiOperation("申报公示")
    @PostMapping("/publicity")
    public Result<Boolean> publicity(@RequestBody RightsProtectionPublicityReqDTO req) {
        return service.doPublicity(req);
    }

    @ApiOperation("评审退回")
    @PostMapping("/doBack")
    public Result<Object> doBack(@RequestBody RightsProtectionOperateReqDTO req) {
        return service.doBack(req);
    }

    @ApiOperation("新增申报接口")
    public Result<Boolean> saveCreate(@RequestBody AddCreateDTO dto) {
        return service.addCreate(dto);
    }

    @ApiOperation("关联政策")
    @PostMapping("/queryPolicy")
    Result<List<DocumentListRspDTO>> queryPolicy(@RequestParam("type") Integer type) {
        return service.queryPolicy(type);
    }

    @ApiOperation("获取上一环节的附件")
    @PostMapping("/getAnnex")
    Result<RightsProtectionAnnexDTO> getAnnex(@RequestParam("id") Long id) {
        return service.getAnnex(id);
    }


    @ApiOperation("申报流程显示")
    @PostMapping("/getFlows")
    Result<List<RightsProtectionFlowDTO>> getFlows(@RequestParam("id") Long id) {
        return service.getFlows(id);
    }

    @ApiOperation("申报清单分页查询")
    @PostMapping("/inventoryPage")
    public Result<PageRsp<RightsProtectionInventoryPageRespDTO>> inventoryPage(@RequestBody PageReq<RightsProtectionInventoryPageReqDTO> req) {
        return service.inventoryPage(req);
    }

}
