package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.cavalryTrumpet.YcqYnCaptainChangeFlow;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.*;
import com.crunii.micro.service.qzh.service.YcqYnDeclarationRecordService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.data.repository.query.Param;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author: LG
 * @date: 2023/11/14 16:04
 */
@Validated
@Api(tags = "青年号-申报记录管理")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/ycqYnDeclarationRecordController")
public class YcqYnDeclarationRecordController {

    @Resource
    private YcqYnDeclarationRecordService ycqYnDeclarationRecordService;

    @ApiOperation("查询青年文明号申报-变更记录条数")
    @GetMapping("/queryDeclarationRecordChangeNumber")
    public Result<QueryNumVO> queryDeclarationRecordChangeNumber(@CurrWorker  @ApiIgnore Worker worker,@Param("queryType")Integer queryType ){
        return ycqYnDeclarationRecordService.queryDeclarationRecordChangeNumber(worker,queryType);
    }

    @ApiOperation("查询青年文明号申报-变更记录-根据worker")
    @PostMapping("/queryDeclarationRecordChangeByWorker")
    public Result<PageRsp<QueryDeclarationRecordChangeVO>> queryDeclarationRecordChangeByWorker(@CurrWorker  @ApiIgnore Worker worker,@RequestBody @ApiParam PageReq<QueryDeclarationRecordChangeDTO> dto){
        return ycqYnDeclarationRecordService.queryDeclarationRecordChangeByWorker(worker,dto);
    }

    @ApiOperation("查询青年文明号申报-变更记录")
    @PostMapping("/queryDeclarationRecordChange")
    public Result<PageRsp<QueryDeclarationRecordChangeVO>> queryDeclarationRecordChange(@CurrWorker  @ApiIgnore Worker worker,@RequestBody @ApiParam PageReq<CivilizationReportPageDto> dto){
        return ycqYnDeclarationRecordService.queryDeclarationRecordChange(worker,dto);
    }

    @ApiOperation("变更号长")
    @PostMapping("/changeCaptain")
    public Result<String> changeCaptain(@CurrWorker  @ApiIgnore Worker worker,@RequestBody @ApiParam ChangeCaptainDTO dto){
        return ycqYnDeclarationRecordService.saveChangeCaptain(worker,dto);
    }

    @ApiOperation("变更号长流程查询")
    @GetMapping("/queryDeclarationRecordChangeFlow")
    public Result<List<QueryDeclarationRecordChangeFlowVO>> queryDeclarationRecordChangeFlow(@CurrWorker  @ApiIgnore Worker worker, @Param("declarationId")Long declarationId ){
        return ycqYnDeclarationRecordService.queryDeclarationRecordChangeFlow(worker,declarationId);
    }

    @ApiOperation("查询变更前后号长")
    @GetMapping("/queryCaptainInfo")
    public Result<QueryCaptainInfoVO> queryCaptainInfo(@Param("changeId") Long changeId ){
        return ycqYnDeclarationRecordService.queryCaptainInfo(changeId);
    }


    @ApiOperation("变更号长-审核通过")
    @PostMapping("/auditCaptainPass")
    public Result<String> auditCaptainPass( @CurrWorker  @ApiIgnore Worker worker,@RequestBody @ApiParam AuditCaptainDTO dto ){
        return ycqYnDeclarationRecordService.updateAuditCaptainPass(worker,dto);
    }

    @ApiOperation("变更号长-审核不通过")
    @PostMapping("/auditCaptainNotPass")
    public Result<String> auditCaptainNotPass(@CurrWorker  @ApiIgnore Worker worker,@RequestBody @ApiParam AuditCaptainDTO dto ){
        return ycqYnDeclarationRecordService.updateAuditCaptainNotPass(worker,dto);
    }

    @ApiOperation("曾被认定类别管理-新增")
    @PostMapping("/addConfig")
    public Result<String> addConfig(@CurrWorker  @ApiIgnore Worker worker,@RequestBody AddConfigDTO dto){
        return Result.ofSuccess(ycqYnDeclarationRecordService.addConfig(worker,dto));
    }

    @ApiOperation("曾被认定类别管理-置无效 || 有效")
    @PostMapping("/deleteConfig")
    public Result<String> deleteConfig(@CurrWorker  @ApiIgnore Worker worker,@RequestBody ConfigEffectiveDto configEffectiveDto ){
        return Result.ofSuccess( ycqYnDeclarationRecordService.deleteConfig(worker,configEffectiveDto));
    }

    @ApiOperation("曾被认定类别管理-编辑")
    @PostMapping("/updateConfig")
    public Result<String> updateConfig(@CurrWorker @ApiIgnore Worker worker, @RequestBody UpdateConfigDTO dto){
        return Result.ofSuccess(ycqYnDeclarationRecordService.updateConfig(worker,dto));
    }

    @ApiOperation("曾被认定类别管理-查询")
    @PostMapping("/queryConfig")
    public Result<PageRsp<QueryConfigVO>> queryConfig(@RequestBody @ApiParam PageReq<QueryConfigDTO> dto){
        return Result.ofSuccess(ycqYnDeclarationRecordService.queryConfig(dto));
    }
}
