package com.crunii.micro.service.qzh.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.*;
import com.crunii.micro.service.qzh.service.YouthSafetyModelRecommendService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @Author: zwx
 * @CreateTime: 2023-11-13  16:13:41
 * @Description: 青年安全生产示范岗推荐申报
 */

@Api(tags = "青年安全生产示范岗推荐申报")
@RequiredArgsConstructor
@RestController
@RequestMapping("/auth/YouthSafetyModelRecommend")
public class YouthSafetyModelRecommendController {

    @Autowired
    YouthSafetyModelRecommendService youthSafetyModelRecommendService;

    @ApiOperation("青年安全生产示范岗推荐申报新增")
    @PostMapping("/recommendCreate")
    public Result youthSafetyModelRecommendCreate(@RequestBody YouthSafetyModelRecommendReqDTO dto) {
        return youthSafetyModelRecommendService.createRecommend(dto);
    }

    @ApiOperation("青年安全生产示范岗推荐申报详情")
    @PostMapping("/recommendDetail")
    public Result youthSafetyModelRecommendDetail(@RequestParam Long id) {
        return youthSafetyModelRecommendService.recommendDtail(id);
    }

    @ApiOperation("推荐申报填入返回草稿")
    @PostMapping("/fillIn")
    public Result youthSafetyModelFillIn(@RequestParam Long id) {
        return youthSafetyModelRecommendService.doRecommendFillIn(id);
    }

    @ApiOperation("变更岗长申请")
    @PostMapping("/updateLeader")
    public Result youthSafetyModelUpdateLeader(@RequestBody YouthSafetyChangeLeaderReqDTO dto) {
        return youthSafetyModelRecommendService.doChangeLeader(dto);
    }

    @ApiOperation("变更岗长审核")
    @PostMapping("/examineLeaderChange")
    public Result leaderChangeExamine(@RequestBody ExamineChangeLeaderDTO dto) {
        return youthSafetyModelRecommendService.doLeaderChangeExamine(dto);
    }

    @ApiOperation("当前用户变更记录申请分页查询")
    @PostMapping("/leaderChangeListPage")
    public Result leaderChangeListPage(@RequestBody PageReq<LeaderChangeListPageReqDTO> req) {
        return youthSafetyModelRecommendService.leaderChangeListPage(req);
    }

    @ApiOperation("申报变更清单审核分页查询")
    @PostMapping("/applyChangeListPage")
    public Result applyChangeListPage(@RequestBody PageReq<LeaderChangeListPageReqDTO> req) {
        return youthSafetyModelRecommendService.applyChangeListPage(req);
    }

    @ApiOperation("申报预览")
    @PostMapping("/applyPdfExport")
    public ResponseEntity<byte[]> applyPdfExport(@RequestParam String id, HttpServletRequest request, HttpServletResponse response) {
        return youthSafetyModelRecommendService.applyPdfExport(id,request,response);
    }

    @ApiOperation("推荐申报预览")
    @PostMapping("/recommendPdfExport")
    public ResponseEntity<byte[]> recommendPdfExport(@RequestParam String id, HttpServletRequest request, HttpServletResponse response) {
        return youthSafetyModelRecommendService.recommendPdfExport(id,request,response);
    }


    @ApiOperation("变更记录终止任务")
    @PostMapping("/leaderChangeDelete")
    public Result leaderChangeDelete(@RequestBody IdReqDTO dto) {
        Long id = Long.valueOf(dto.getId());
        return youthSafetyModelRecommendService.doLeaderChangeDelete(id);
    }

    @ApiOperation("变更记录编辑")
    @PostMapping("/leaderChangeEdit")
    public Result leaderChangeEdit(@RequestBody LeaderChangeEditDTO dto) {
        return youthSafetyModelRecommendService.doLeaderChangeEdit(dto);
    }

    @ApiOperation("变更流程记录")
    @PostMapping("/flowHistory")
    public Result flowHistory(@RequestParam String id) {
        return youthSafetyModelRecommendService.flowHistory(id);
    }


}
