package com.crunii.micro.service.qzh.enums;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author lmlksy
 * @date 2023/12/18 14:56
 */
public enum PushTerminalEnum {
    PC(1, "PC"),
    APP(2, "APP"),
    MANAGEMENT_CONSOLE(3, "管理台"),
    PUBLIC_ACCOUNT(4, "公众号");

    private final int code;
    private final String description;

    PushTerminalEnum(int code, String description) {
        this.code = code;
        this.description = description;
    }

    public int getCode() {
        return code;
    }

    public String getDescription() {
        return description;
    }

    public static String getDescriptionsByCodes(String codes) {
        List<String> descriptions = Arrays.stream(codes.split(","))
                .map(String::trim)
                .map(Integer::parseInt)
                .map(code -> Arrays.stream(values())
                        .filter(endpoint -> endpoint.getCode() == code)
                        .map(PushTerminalEnum::getDescription)
                        .findFirst()
                        .orElse(""))
                .collect(Collectors.toList());

        return String.join(",", descriptions);
    }

    public static void main(String[] args) {
        // 测试将字符串转换为枚举列表
        String messageType1 = "1"; // 单个code
        String messageType2 = "1,3"; // 多个code

        String descriptionsByCodes = PushTerminalEnum.getDescriptionsByCodes(messageType1);
        System.out.println(descriptionsByCodes);
    }
}
