package com.crunii.micro.service.qzh.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.cavalryTrumpet.YcqYnFlowRecord;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.*;
import com.crunii.micro.service.qzh.dto.TestPaperExtentVO;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wangchen create 2023/11/13 16:53
 */
@Repository
public class CivilizationReportDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate npJdbcTemplate;

    public PageRsp<CivilizationReportVo> pageCivilizationReport(PageReq<CivilizationReportPageDto> pageReq, List<Long> declarationIds) {
        StringBuilder sb = new StringBuilder("SELECT a.*,b.document_title as docName,c.worker_name,(SELECT org_name FROM TSM_ORG WHERE a.org_id = org_id) orgName FROM YCQ_YN_DECLARATION a LEFT JOIN YCQ_QZH_DOCUMENT b ON a.doc_id = b.id LEFT JOIN TSM_WORKER c ON a.creator_id = c.login_id  WHERE  a.deleted = 0 and a.draft = 0 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_YN_DECLARATION a LEFT JOIN YCQ_QZH_DOCUMENT b ON a.doc_id = b.id LEFT JOIN TSM_WORKER c ON a.creator_id = c.login_id  WHERE a.deleted = 0 and a.draft = 0 ");
        Map<String, Object> params = appendParams(pageReq, declarationIds, sb, sbCount);

        int total = npJdbcTemplate.queryForInt(sbCount.toString(), params);
        List<CivilizationReportVo> values;
        String dbType = npJdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by a.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = npJdbcTemplate.queryForBeans(CivilizationReportVo.class, sb.toString(), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<CivilizationReportVo> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    private Map<String, Object> appendParams(PageReq<CivilizationReportPageDto> pageReq, List<Long> declarationIds, StringBuilder sb, StringBuilder sbCount) {
        Map<String,Object> params = new HashMap<>(32);
        sb.append(" and a.declaration_id in(:ids)");
        sbCount.append(" and a.declaration_id in(:ids)");
        params.put("ids", declarationIds);
        CivilizationReportPageDto query = pageReq.getQuery();
        if(StringUtils.hasText(query.getCollectiveName())){
            sb.append(" and a.collective_name like CONCAT('%',:collectiveName,'%')");
            sbCount.append(" and a.collective_name like CONCAT('%',:collectiveName,'%')");
            params.put("collectiveName",query.getCollectiveName());
        }
        if(Objects.nonNull(query.getOrgId())){
            sb.append(" and a.org_id = :orgId ");
            sbCount.append(" and a.org_id = :orgId ");
            params.put("orgId",query.getOrgId());
        }
        if(StringUtils.hasText(query.getUnit())){
            sb.append(" and a.unit like CONCAT('%',:unit,'%')");
            sbCount.append(" and a.unit like CONCAT('%',:unit,'%')");
            params.put("unit",query.getUnit());
        }
        if(StringUtils.hasText(query.getDeclarationName())){
            sb.append(" and a.declaration_name like CONCAT('%',:declarationName,'%')");
            sbCount.append(" and a.declaration_name like CONCAT('%',:declarationName,'%')");
            params.put("declarationName",query.getDeclarationName());
        }
        if(Objects.nonNull(query.getAuditState())){
            sb.append(" and a.audit_state = :auditState ");
            sbCount.append(" and a.audit_state = :auditState ");
            params.put("auditState",query.getAuditState());
        }

        if(StringUtils.hasText(query.getCreateWorker())){
            sb.append(" and c.worker_name like CONCAT('%',:createWorker,'%')");
            sbCount.append(" and c.worker_name like CONCAT('%',:createWorker,'%')");
            params.put("createWorker",query.getCreateWorker());
        }
        if(Objects.nonNull(query.getStartTime())){
            sb.append(" and a.create_time >= :startTime ");
            sbCount.append(" and a.create_time >= :startTime ");
            params.put("startTime",query.getStartTime());
        }
        if(Objects.nonNull(query.getEndTime())){
            sb.append(" and a.create_time <= :endTime ");
            sbCount.append(" and a.create_time <= :endTime ");
            params.put("endTime",query.getEndTime());
        }
        if(StringUtils.hasText(query.getDocName())){
            sb.append(" and b.document_title like CONCAT('%',:docName,'%')");
            sbCount.append(" and b.document_title like CONCAT('%',:docName,'%')");
            params.put("docName",query.getDocName());
        }
        if(Objects.nonNull(query.getDeclarationType())){
            sb.append(" and a.declaration_type = :declarationType ");
            sbCount.append(" and a.declaration_type = :declarationType ");
            params.put("declarationType",query.getDeclarationType());
        }
        if(Objects.nonNull(query.getCategory())){
            sb.append(" and a.category = :category ");
            sbCount.append(" and a.category = :category ");
            params.put("category",query.getCategory());
        }

        if(Objects.nonNull(query.getDistribution())){
            sb.append(" and a.distribution = :distribution ");
            sbCount.append(" and a.distribution = :distribution ");
            params.put("distribution",query.getDistribution());
        }
        return params;
    }

    /**
     * 查询政策 需要传入？暂停 declarationType 后续可能使用
     */
    public List<DocVo> findDoc(Integer declarationType) {
        String sql = "select id AS docId,document_title AS docName from ycq_qzh_document where document_type=2 and del_flag=0 ";
        return jdbcTemplate.queryForBeans(DocVo.class,sql);
    }

    public void updateAuditState(List<Long> declarationIds, int auditState,Integer loginId,Date now) {
        String sql = "UPDATE YCQ_YN_DECLARATION SET audit_state = :auditState,last_review_comments = 1,updater_id = :loginId,update_time = :updateTime where declaration_id in(:ids)";
        Map<String,Object> params = new HashMap<>(4);
        params.put("auditState",auditState);
        params.put("ids",declarationIds);
        params.put("loginId",loginId);
        params.put("updateTime",now);
        npJdbcTemplate.update(sql,params);
    }

    public void rejectFlow(List<Long> declarationIds,Integer loginId,Date now) {
        String sql = "UPDATE YCQ_YN_DECLARATION SET audit_state = 8 ,last_review_comments = 2,updater_id = :loginId,update_time = :updateTime where declaration_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",declarationIds);
        params.put("loginId",loginId);
        params.put("updateTime",now);
        npJdbcTemplate.update(sql,params);
    }

    public void updateRatingLevel(List<Long> declarationIds,int loginId,Date now,int ratingLevel) {
        String sql = "UPDATE YCQ_YN_DECLARATION SET rating_level = :ratingLevel,updater_id = :loginId,update_time = :updateTime  where declaration_id in(:ids)";
        Map<String,Object> params = new HashMap<>(10);
        params.put("ids",declarationIds);
        params.put("ratingLevel",ratingLevel);
        params.put("loginId",loginId);
        params.put("updateTime",now);
        npJdbcTemplate.update(sql,params);
    }

    public List<ReviewVo> findByIds(List<Long> declarationIds) {
        String sql = "SELECT audit_state,org_id FROM YCQ_YN_DECLARATION WHERE deleted = 0 and declaration_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",declarationIds);
        return npJdbcTemplate.queryForBeans(ReviewVo.class,sql,params);
    }

    public YcqYnFlowRecord findFlowId(Long declarationId) {
        String sql = "SELECT * FROM YCQ_YN_FLOW_RECORD WHERE service_id = ? ORDER BY create_time DESC LIMIT 1";
        return jdbcTemplate.queryForBean(YcqYnFlowRecord.class,sql,declarationId);
    }

    public int findCount(List<Long> declarationIds, List<Integer> auditStates) {
        String sql = "SELECT count(1) FROM YCQ_YN_DECLARATION WHERE deleted = 0 AND declaration_id in(:ids) AND audit_state in(:auditStates)";
        Map<String,Object> params = new HashMap<>(3);
        params.put("ids",declarationIds);
        params.put("auditStates",auditStates);
        return npJdbcTemplate.queryForInt(sql,params);
    }

    public PdfWorkerInfoVo findWorkerInfo(Integer workerId) {
        String sql = "SELECT worker_name,(SELECT org_name FROM TSM_ORG WHERE org_id = CAST(a.org_id AS BIGINT)) AS orgName,mobile FROM TSM_WORKER a WHERE login_id = ?";
        return jdbcTemplate.queryForBean(PdfWorkerInfoVo.class,sql,workerId);
    }

    public PageRsp<CivilizationReportVo> pageMyselfReport(PageReq<Object> req, Worker worker) {
        StringBuilder sb = new StringBuilder("SELECT a.*,b.document_title as docName,c.worker_name,(SELECT org_name FROM TSM_ORG WHERE a.org_id = org_id) orgName FROM YCQ_YN_DECLARATION a LEFT JOIN YCQ_QZH_DOCUMENT b ON a.doc_id = b.id LEFT JOIN TSM_WORKER c ON a.creator_id = c.login_id  WHERE  a.deleted = 0 and (a.creator_id = ? OR (a.login_id=? and a.audit_state <> 0)) ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_YN_DECLARATION a LEFT JOIN YCQ_QZH_DOCUMENT b ON a.doc_id = b.id LEFT JOIN TSM_WORKER c ON a.creator_id = c.login_id  WHERE a.deleted = 0  and (a.creator_id = ? OR a.login_id=?) ");
        List<Object> params = new ArrayList<>(4);
        params.add(worker.getLoginId());
        params.add(worker.getLoginId());
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<CivilizationReportVo> values;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by a.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            values = jdbcTemplate.queryForBeans(CivilizationReportVo.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<CivilizationReportVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public Integer findLoginIdByMobile(String mobile) {
        String sql = "SELECT login_id FROM TSM_WORKER WHERE mobile = ?";
        return jdbcTemplate.queryForRawObject(Integer.class, sql, mobile);
    }

    public List<Long> findDeclarationIdByLoginId(Integer workerId) {
        String sql = "SELECT declaration_id FROM YCQ_YN_DECLARATION WHERE deleted = 0 AND audit_state = 4 AND  login_id = ?";
        return jdbcTemplate.queryForRawObjects(Long.class,sql,workerId);
    }

    public String findOrgNameByOrgId(Long orgId) {
        String sql = "SELECT org_name FROM TSM_ORG WHERE org_id = ?";
        return jdbcTemplate.queryForRawObject(String.class,sql,orgId);
    }

    public List<Long> isExistInitEvaluation(List<Long> declarationIds) {
        String sql = "SELECT declaration_id FROM YCQ_YN_INITIAL_EVALUATION WHERE declaration_id in(:ids)";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",declarationIds);
        return npJdbcTemplate.queryForRawObjects(Long.class,sql,params);
    }

    public OrgIdNameDto findOrgCountyId(Long orgId) {
        String sql = "SELECT org_id,org_name FROM TSM_ORG WHERE org_id = ?";
        return jdbcTemplate.queryForBean(OrgIdNameDto.class,sql,orgId);
    }

    public List<String> findCanQueryOrgIds(List<Long> orgIds) {
        String sql = "SELECT DISTINCT(org_id) FROM TSM_ORG WHERE org_id NOT in(:ids) START WITH org_id in(:ids) CONNECT BY PRIOR parent_org_id=org_id";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",orgIds);
        return npJdbcTemplate.queryForRawObjects(String.class,sql,params);
    }

    public List<Long> findServiceIds(QueryOrgDto queryOrgDto) {
        String sql = "SELECT service_id FROM YCQ_QZH_QUERY_ORG WHERE LOCATE(?,str_org_id)>0 AND report_type = ? AND deleted = 0";
        return jdbcTemplate.queryForRawObjects(Long.class,sql,Long.parseLong(queryOrgDto.getOrgId()),queryOrgDto.getReportType());
    }


    public List<KeyValueDto> findNextOrgId(List<Long> declarationIds,boolean isCountry) {
        String sql = "SELECT declaration_id AS id,(%s) AS value FROM YCQ_YN_DECLARATION a WHERE declaration_id in(:ids)";
        if(isCountry){
            sql = String.format(sql, "a.org_id");
        }else {
            sql = String.format(sql, "SELECT org_id FROM TSM_ORG WHERE parent_org_id = 0 AND org_type = '1'");
        }
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",declarationIds);
        return npJdbcTemplate.queryForBeans(KeyValueDto.class,sql,params);
    }

    public OrgIdNameDto findTopOrg() {
        String sql = "SELECT org_id,org_name FROM TSM_ORG WHERE parent_org_id = 0 AND org_type = '1'";
        return jdbcTemplate.queryForBean(OrgIdNameDto.class,sql);
    }

    public List<String> findCanQueryOrgContainIds(List<Long> orgIds) {
        String sql = "SELECT DISTINCT(org_id) FROM TSM_ORG START WITH org_id in(:ids) CONNECT BY PRIOR parent_org_id=org_id";
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",orgIds);
        return npJdbcTemplate.queryForRawObjects(String.class,sql,params);
    }

    public List<Long> findNextDeclarationIds(List<Long> declarationIds,boolean isCountry) {
        String sql = "SELECT declaration_id FROM YCQ_YN_DECLARATION WHERE declaration_id in(:ids) ";
        if(isCountry){
            sql += "  AND audit_state = 1";
        }else {
            sql += " AND audit_state <> 1";
        }
        Map<String,Object> params = new HashMap<>(2);
        params.put("ids",declarationIds);
        return npJdbcTemplate.queryForRawObjects(Long.class,sql,params);
    }

    public Integer findOrgLevel(String orgId) {
        String sql = "SELECT org_level FROM TSM_ORG WHERE org_id = ?";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,Long.parseLong(orgId));
    }

    public int findHandleCountryCount(String orgId) {
        String sql = "SELECT count(1) FROM YCQ_YN_DECLARATION WHERE org_id = ? AND deleted = 0 AND audit_state = 1";
        return jdbcTemplate.queryForInt(sql,orgId);
    }

    public int findHandleCityCount(String orgId) {
        String sql = "SELECT count(1) FROM YCQ_YN_DECLARATION a WHERE audit_state NOT IN(0,1,4,7,8) AND deleted = 0";
        return jdbcTemplate.queryForInt(sql);
    }

    public int findCurrentReportCount(int loginId) {
        String sql = "SELECT count(1) FROM YCQ_YN_DECLARATION WHERE creator_id = ? AND audit_state = 0 AND deleted = 0";
        return jdbcTemplate.queryForInt(sql,loginId);
    }

    public int findWorkerExam(int loginId) {
        String sql = "SELECT CASE WHEN \"category\" = 1 THEN 10908 WHEN \"category\" = 2 THEN 10909 ELSE 0 end  FROM YCQ_YN_DECLARATION WHERE audit_state = 4 AND deleted = 0 AND login_id = ? limit 1";
        return jdbcTemplate.queryForInt(sql,loginId);
    }

    public TestPaperExtentVO findIsCreateExam(int loginId) {
        String sql = "SELECT id AS studentCourseId,student_id FROM YCQ_YJS_STUDENT_COURSE WHERE student_id = ? AND del_flag = 0 limit 1";
        return jdbcTemplate.queryForBean(TestPaperExtentVO.class,sql,loginId);
    }

    public Long findPaperId(String s) {
        String sql = "SELECT id FROM YCQ_YJS_TEST_PAPER WHERE type_code = ? AND del_flag = 0 limit 1";
        return jdbcTemplate.queryForRawObject(Long.class,sql,s);
    }

    public int findWorkerExamCount(Long paperId, Worker worker) {
        String sql = "SELECT residue_test_count FROM YCQ_YJS_STUDENT_COURSE WHERE student_id =? AND paper_id = ?";
        return jdbcTemplate.queryForInt(sql,worker.getLoginId(),paperId);
    }

    public TestPaperVO findPaperInfo(Long paperId) {
        String sql = "SELECT id,answer_duration ,start_time ,end_time ,allow_times ,pass_score ,total_score ,total_count FROM YCQ_YJS_TEST_PAPER WHERE id = ?";
        return jdbcTemplate.queryForBean(TestPaperVO.class,sql,paperId);
    }
}
