package com.crunii.micro.service.qzh.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qzh.dto.CommandoActiveDetailsIssueRespDTO;
import com.crunii.micro.service.qzh.dto.CommandoActiveDetailsLoginRespDTO;
import com.crunii.micro.service.qzh.dto.IntegrationDetailsCommonReqDTO;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import javax.annotation.Resource;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

/**
 * @Author thw
 * @date 2023/9/9
 * @description 突击队积分明细Dao
 */
@Repository
public class CommandoActiveDetailsDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    // 突击队风采发布积分分页
    private static final String ISSUE_PAGE_BASE_SQL =
//            " select ROWNUM orderNumber , a.id , a.state , a.create_time , a.group_org_id ,   b.activity_title title, a.commando_name , b.create_by workerName,to_char(b.CREATE_TIME, 'yyyy-MM-dd hh24:mi:ss') as issueTime , 1 score from YCQ_QZH_COMMANDO a " +
        // 由于前端的对应问题而修改
            " select  a.id , a.state , a.create_time , a.group_org_id ,   b.activity_title workerName, a.commando_name , b.create_by phone ,to_char(b.CREATE_TIME, 'yyyy-MM-dd hh24:mi:ss') as loginTime , 1 score from YCQ_QZH_COMMANDO a " +
                    "inner join YCQ_QZH_ACTIVITY b on b.parent_id = a.id and b.activity_type = 1 and b.DEL_FLAG = 0 where a.state = 3 and a.del_flag = 0 ";

    private static final String ISSUE_PAGE_COUNT_SQL =
            " select count(1) from YCQ_QZH_COMMANDO a " +
                    "inner join YCQ_QZH_ACTIVITY b on b.parent_id = a.id and b.activity_type = 1 and b.DEL_FLAG = 0  " +
                    "where a.state = 3 and a.del_flag = 0 ";

    // 突击队登录积分分页
//    private static final String LOGIN_PAGE_BASE_SQL = " select ROWNUM orderNumber , a.commando_name   from YCQ_QZH_COMMANDO a where a.state = 3 ";
    private static final String LOGIN_PAGE_BASE_SQL = "  SELECT  b.LOGIN_ID ,  a.id , a.group_org_id , a.commando_code, a.create_time , a.team_leader_phone phone, a.commando_name, b.worker_name, b.MOBILE ,     "
        + "    to_char(c.max_time, 'yyyy-MM-dd hh24:mi:ss') as LOGIN_TIME , 1 score        "
        + "    FROM  YCQ_QZH_COMMANDO a    "
        + "    inner join TSM_WORKER  b  on   (a.CREATE_ID=b.LOGIN_ID or b.MOBILE=a.TEAM_LEADER_PHONE ) and b.STATE = 1           "
        + "    inner join ( "
        + "        select MOBILE , max(login_time) max_time , substr(max(login_time),1,10) date_time from TSM_LOGIN_LOG  "
        + "        group by MOBILE , substr(LOGIN_TIME,1,10) order by date_time desc  "
        + "    ) c on c.MOBILE = b.MOBILE "
        + "    where a.DEL_FLAG= 0 and a.state = 3  and date_format(c.max_time,'yyyy-MM-dd HH:mm:ss')  >= date_format(a.create_time,'yyyy-MM-dd HH:mm:ss') ";
    //    private static final String LOGIN_PAGE_COUNT_SQL = " select count(*) from YCQ_QZH_COMMANDO a where a.state = 3 ";
    private static final String LOGIN_PAGE_COUNT_SQL = " SELECT  count(1)  FROM  YCQ_QZH_COMMANDO a    "
        + "inner join TSM_WORKER  b  on   (a.CREATE_ID=b.LOGIN_ID or b.MOBILE=a.TEAM_LEADER_PHONE ) and b.STATE = 1           "
        + "inner join ( "
        + "    select MOBILE , max(login_time) max_time , substr(max(login_time),1,10) date_time from TSM_LOGIN_LOG  "
        + "    group by MOBILE , substr(LOGIN_TIME,1,10) order by date_time desc  "
        + ") c on c.MOBILE = b.MOBILE "
        + "where a.DEL_FLAG= 0 and a.state = 3  and date_format(c.max_time,'yyyy-MM-dd HH:mm:ss')  >= date_format(a.create_time,'yyyy-MM-dd HH:mm:ss') ";


    /** 突击队发布积分明细 */
    public PageRsp<CommandoActiveDetailsIssueRespDTO> issueScorePage(Worker currWorker,
                                                                     PageReq<IntegrationDetailsCommonReqDTO> req) {
        StringBuilder sb = new StringBuilder(ISSUE_PAGE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(ISSUE_PAGE_COUNT_SQL);
        IntegrationDetailsCommonReqDTO query = req.getQuery();
        List<Object> params = new ArrayList<>();

        // 通过突击队主键id，查询已发布的风采
        sb.append(" and a.id = ? ");
        sbCount.append(" and a.id = ? ");
        params.add(Long.valueOf(query.getId()));

        // 通用查询
        commonConditionSet(sb,sbCount,params,query);

        sb.append("  order by loginTime desc  ");

        sb = new StringBuilder(" select ROWNUM orderNumber, a.*  from ( " + sb + ") a");
//        sbCount = new StringBuilder(" select count(1) from ( " + sbCount + ") a");

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<CommandoActiveDetailsIssueRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jdbcTemplate.queryForBeans(CommandoActiveDetailsIssueRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<CommandoActiveDetailsIssueRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    /** 突击队登录积分明细 */
    public PageRsp<CommandoActiveDetailsLoginRespDTO> loginScorePage(Worker currWorker,
                                                                     PageReq<IntegrationDetailsCommonReqDTO> req) {
        StringBuilder sb = new StringBuilder(LOGIN_PAGE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(LOGIN_PAGE_COUNT_SQL);
        IntegrationDetailsCommonReqDTO query = req.getQuery();
        List<Object> params = new ArrayList<>();

        // 通过突击队主键id，查询已发布的风采
        sb.append(" and a.id = ? ");
        sbCount.append(" and a.id = ? ");
        params.add(Long.valueOf(query.getId()));

        // 通用查询
        commonConditionSet(sb,sbCount,params,query);

        sb.append(" order by LOGIN_TIME desc ");

        sb = new StringBuilder(" select ROWNUM orderNumber, a.*  from ( " + sb + ") a");
//        sbCount = new StringBuilder(" select count(1) from ( " + sbCount + ") a");

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<CommandoActiveDetailsLoginRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jdbcTemplate.queryForBeans(CommandoActiveDetailsLoginRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<CommandoActiveDetailsLoginRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }



    /**
     * 通用条件查询
     */
    private void commonConditionSet(StringBuilder sb, StringBuilder sbCount, List<Object> params, IntegrationDetailsCommonReqDTO dto) {
        // 查询组织
//        if (StringUtils.hasText(dto.getOrgId())) {
//            sb.append(" and a.group_org_id = ? ");
//            if (Objects.nonNull(sbCount)) {
//                sbCount.append(" and a.group_org_id = ? ");
//            }
//            params.add(dto.getOrgId());
//        }

        // 开始日期
        if (StringUtils.hasText(dto.getStartDate())) {
            sb.append(" and to_date(a.create_time,'yyyy-mm-dd hh24:mi:ss') >= ? ");
            if (Objects.nonNull(sbCount)) {
                sbCount.append(" and to_date(a.create_time,'yyyy-mm-dd hh24:mi:ss') >= ? ");
            }
            params.add(dto.getStartDate() +" 00:00:00");
        }

        // 结束日期
        if (StringUtils.hasText(dto.getEndDate())) {
            sb.append(" and to_date(a.create_time,'yyyy-mm-dd hh24:mi:ss') <= ? ");
            if (Objects.nonNull(sbCount)) {
                sbCount.append(" and to_date(a.create_time,'yyyy-mm-dd hh24:mi:ss') <= ? ");
            }
            params.add(dto.getEndDate() + " 23:59:59");
        }
    }
}
