package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO;
import com.crunii.micro.service.qzh.dto.*;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

/**
 * @Author thw
 * @date 2023/9/7
 * @description 青年突击队信息统计
 */
@Repository
public class CommandoCountInfoDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    /* 通过突击队的唯一 commando_code分组，突击队不会重复 */
    private static final String COMMANDO_TOTAL_SQL = " select distinct a.commando_code , person_num  ,  concat_ws('-', team_leader_name, team_leader_card) leaderNameAndCard, count(parent_id) fengcaiGoupNumber ,"
        + "    LISTAGG ( distinct orgidList , ',')  orgidList"
        + "    from  YCQ_QZH_COMMANDO a          "
        + "    left join YCQ_QZH_ACTIVITY b on a.id = b.parent_id  and  b.activity_type = 1 and b.DEL_FLAG = 0 and a.del_flag = 0         "
        + "    left join (   "
        + "       select  LISTAGG ( distinct k.org_id , ',')  WITHIN GROUP (ORDER BY k.org_id)  orgidList , k.commando_code    "
        + "       from YCQ_QZH_COMMANDO a       "
        + "       inner join ycq_qzh_commando_org k on k.commando_code = a.commando_code group by k.commando_code   "
        + "    ) h on h.commando_code = a.commando_code and a.del_flag = 0   "
        + "    where  a.state = 3 and a.del_flag = 0  ";


    /** 冻结状态sql */
    private static final String COMMANDO_STATE = "  select count(*) from (  "
        + "    SELECT  distinct a.commando_code , "
        + "    (CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6 and k.label = 'true' )  THEN '冻结' ELSE '正常' END ) commandoState  "
        + "    FROM    "
        + "    ycq_qzh_commando a           "
        + "    LEFT JOIN tsm_worker b ON a.create_id = b.login_id  "
        + "    LEFT JOIN (  "
        + "        select a.commando_code , count(parent_id) mien_count ,max(b.create_time) max_time ,  "
        + "         (case when ((count(parent_id) = 0 and  TIMESTAMPDIFF( MONTH, max(a.create_time) , now()) > 6)  "
        + "         or (count(parent_id) > 0 and  TIMESTAMPDIFF( MONTH, max(b.create_time) , now()) > 6))  "
        + "         then 'true' else 'false' end) label  "
        + "         from ycq_qzh_commando a  "
        + "         LEFT JOIN YCQ_QZH_ACTIVITY b on a.id = b.parent_id and b.activity_type = 1  and a.state = 3 and a.del_flag = 0 and b.del_flag = 0  "
        + "         group by a.commando_code  "
        + "    ) k ON k.commando_code = a.commando_code   " ;

    private static final String COMMANDO_TYPE_SQL = " select"
        + "  type typeTitle,"
        + "  count(type) typeNumber,"
        + "  sum(person_num) typeTeamMemberNumber ,"
        + "  LISTAGG ( distinct orgidList , ',')  orgidList"
        + " from"
        + "  YCQ_QZH_COMMANDO a"
        + "  left join (   "
        + "        select  LISTAGG ( distinct  k.org_id , ',') WITHIN GROUP (ORDER BY k.org_id)  orgidList , k.commando_code    "
        + "        from YCQ_QZH_COMMANDO a       "
        + "        inner join ycq_qzh_commando_org k on k.commando_code = a.commando_code and a.del_flag = 0"
        + "        group by k.commando_code   "
        + "    ) h on h.commando_code = a.commando_code and a.del_flag = 0    "
        + " where a.state = 3 and a.del_flag = 0 ";

    // 青年青年突击队活跃度积分分页sql（突击队为已通过，风采为突击队风采以及已经发布的风采）
    private static final String COMMANDO_PAGE_BASE_SQL =
            "  select distinct a.id, a.create_time , a.create_id , a.del_flag , a.commando_code ,a.commando_name,a.group_org_id, a.group_org_name orgName , a.team_leader_name,a.team_leader_phone,a.type, a.state ,          "
                + "   case when d.fengcaiGoupNumber is not null then d.fengcaiGoupNumber else 0 end fengcaiGoupNumber,           "
                + "   case when e.POINT is not null then e.POINT else 0 end loginAmount,  max_time ,       "
                + "   ifnull((d.fengcaiGoupNumber +  e.POINT), 0) score     "
                + "   from YCQ_QZH_COMMANDO a          "
                + "   inner join (           "
                + "   select  commando_code , count(b.parent_id) fengcaiGoupNumber from YCQ_QZH_COMMANDO a            "
                + "   left join YCQ_QZH_ACTIVITY b on b.parent_id = a.id and b.activity_type = 1 and b.DEL_FLAG = 0 and a.del_flag = 0 and a.state = 3            "
                + "   group by  commando_code            "
                + "   ) d on d.commando_code = a.commando_code           "
                + "   inner join (           "
                + "    SELECT a.commando_code, a.COMMANDO_NAME , count(c.MOBILE) point  ,   max(max_time) max_time     "
                + "    FROM YCQ_QZH_COMMANDO a             "
                + "    left join TSM_WORKER  b  on  (a.CREATE_ID=b.LOGIN_ID or b.MOBILE=a.TEAM_LEADER_PHONE ) and b.STATE =1 and a.state = 3 and a.del_flag = 0 and a.state = 3           "
                + "    left join (      "
                + "     select MOBILE , max(login_time) max_time , substr(max(login_time),1,10) date_time from TSM_LOGIN_LOG       "
                + "        group by MOBILE , substr(LOGIN_TIME,1,10)  order by date_time  desc       "
                + "     ) c on c.MOBILE = b.MOBILE and  date_format(max_time,'yyyy-MM-dd HH:mm:ss')  >= date_format(a.create_time,'yyyy-MM-dd HH:mm:ss') "
                + "    group by  a.commando_code , a.COMMANDO_NAME           "
                + "   ) e on e.commando_code= a.commando_code    "
                + "   where a.state = 3  and a.del_flag = 0    ";
    private static final String COMMANDO_PAGE_COUNT_SQL =
              " select distinct a.id, a.create_time , a.create_id , a.del_flag , a.commando_code ,a.commando_name,a.group_org_id, a.group_org_name orgName , a.team_leader_name,a.team_leader_phone,a.type, a.state ,          "
                  + "   case when d.fengcaiGoupNumber is not null then d.fengcaiGoupNumber else 0 end fengcaiGoupNumber,           "
                  + "   case when e.POINT is not null then e.POINT else 0 end loginAmount,  max_time ,       "
                  + "   ifnull((d.fengcaiGoupNumber +  e.POINT), 0) score     "
                  + "   from YCQ_QZH_COMMANDO a          "
                  + "   inner join (           "
                  + "   select  commando_code , count(b.parent_id) fengcaiGoupNumber from YCQ_QZH_COMMANDO a            "
                  + "   left join YCQ_QZH_ACTIVITY b on b.parent_id = a.id and b.activity_type = 1 and b.DEL_FLAG = 0 and a.del_flag = 0 and a.state = 3            "
                  + "   group by  commando_code            "
                  + "   ) d on d.commando_code = a.commando_code           "
                  + "   inner join (           "
                  + "    SELECT a.commando_code, a.COMMANDO_NAME , count(c.MOBILE) point  ,   max(max_time) max_time     "
                  + "    FROM YCQ_QZH_COMMANDO a             "
                  + "    left join TSM_WORKER  b  on  (a.CREATE_ID=b.LOGIN_ID or b.MOBILE=a.TEAM_LEADER_PHONE ) and b.STATE =1 and a.state = 3 and a.del_flag = 0 and a.state = 3           "
                  + "    left join (      "
                  + "     select MOBILE , max(login_time) max_time , substr(max(login_time),1,10) date_time from TSM_LOGIN_LOG       "
                  + "        group by MOBILE , substr(LOGIN_TIME,1,10)  order by date_time  desc       "
                  + "     ) c on c.MOBILE = b.MOBILE and  date_format(max_time,'yyyy-MM-dd HH:mm:ss')  >= date_format(a.create_time,'yyyy-MM-dd HH:mm:ss') "
                  + "    group by  a.commando_code , a.COMMANDO_NAME           "
                  + "   ) e on e.commando_code= a.commando_code    "
                  + "   where a.state = 3  and a.del_flag = 0  ";


    /**
     * @param currWorker
     * @param req
     * @Description: 青年青年突击队活跃度积分分页
     * @Author: thw
     * @Date: 2023/9/8 14:44
     * @Params:
     * @return: com.crunii.micro.common.dto.PageRsp<com.crunii.micro.service.qzh.api.domain.dto.CommandoActiveIntegralRespDTO>
     **/
    public PageRsp<CommandoActiveIntegralRespDTO> activeIntegralPage(Worker currWorker,
                                                                     PageReq<CommandoCountInfoCommonReqDTO> req,List<String> commandoList) {
        StringBuilder sb = new StringBuilder(COMMANDO_PAGE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(COMMANDO_PAGE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        // 查询条件对象
        CommandoCountInfoCommonReqDTO query = req.getQuery();

        // 权限控制
        if (ObjectUtil.isNotEmpty(commandoList)){
            String inSql = getInSql(commandoList, "a.commando_code", params);
            sb.append("and " + inSql);
            sbCount.append("and " +inSql);
        }else {
            PageRsp<CommandoActiveIntegralRespDTO> page = new PageRsp<>();
            page.setPageNo(req.getPageNo());
            page.setPageSize(req.getPageSize());
            page.setTotalNum(0);
            page.setResult(null);
            return page;
        }

        /*//增加申请人
        if (ObjectUtil.isNotEmpty(currWorker.getLoginId())){
            if (ObjectUtil.isNotEmpty(commandoList)){
                sb.append(" union \n");
                sbCount.append(" union \n");
                sb.append(COMMANDO_PAGE_BASE_SQL);
                sbCount.append(COMMANDO_PAGE_COUNT_SQL);
            }

            sb.append(" and  a.create_id = " + "\'" + currWorker.getLoginId() + "\' "  );
            sbCount.append(" and  a.create_id = " + "\'" + currWorker.getLoginId() + "\' "  );
        }*/

        // 通用条件查询
        commonConditionSet(sb, sbCount, params, query);


        if (ObjectUtil.isNotEmpty(query.getOrderDesc())){
            sb.append(" order by score " + query.getOrderDesc());
        }else {
            sb.append(" order by score desc ");
        }

        sb = new StringBuilder(" select ROWNUM orderNumber, a.*  from ( " + sb + ") a where 1 = 1 ");
        sbCount = new StringBuilder(" select count(1) from ( " + sbCount + ") a where 1 = 1 ");



        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<CommandoActiveIntegralRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jdbcTemplate.queryForBeans(CommandoActiveIntegralRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<CommandoActiveIntegralRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    /**
     * @param dto
     * @Description: 查询突击队汇总信息
     * @Author: thw
     * @Date: 2023/9/8 14:44
     * @Params:
     * @return: com.crunii.micro.service.qzh.api.domain.dto.CommandoGroupInfoRespDTO
     **/
    public List<CommandoInfoDTO> countGroupInfo(Worker currWorker,CommandoCountInfoCommonReqDTO dto ,List<String> commandoList) {
        StringBuilder sb = new StringBuilder(COMMANDO_TOTAL_SQL);
        List<Object> params = new ArrayList<>();

        /*//增加申请人
        if (ObjectUtil.isNotEmpty(currWorker.getLoginId())){
            sb.append(" and ( a.create_id = " + "\'" + currWorker.getLoginId() + "\' "  );
            if (ObjectUtil.isEmpty(commandoList)){
                sb.append(" ) ");
            }
        }

        // 权限控制
        if (ObjectUtil.isNotEmpty(commandoList)){
            String sign = " and ";
            String inSql = getInSql(commandoList, "a.commando_code", params);
            if (ObjectUtil.isNotEmpty(currWorker.getLoginId())){
                sign = " or ";
            }
            sb.append(sign + inSql);
            if (ObjectUtil.isNotEmpty(currWorker.getLoginId())){
                sb.append(" ) ");
            }
        }*/
        if (ObjectUtil.isNotEmpty(commandoList)){
            String inSql = getInSql(commandoList, "a.commando_code", params);
            sb.append("and " + inSql);
        }else {
            return Collections.emptyList();
        }

        // 通用条件查询
        commonConditionSet(sb,null,params,dto);
        sb.append(" group by  a.commando_code , person_num , team_leader_name , team_leader_card  ");

        return jdbcTemplate.queryForBeans(CommandoInfoDTO.class, sb.toString(), params.toArray());
    }


    /**
     * @param dto
     * @Description: 查询突击队类别信息
     * @Author: thw
     * @Date: 2023/9/8 14:45
     * @Params:
     * @return: java.util.List<com.crunii.micro.service.qzh.api.domain.dto.CommandoTypeRatioRespDTO>
     **/
    public List<CommandoTypeRatioRespDTO> findTypeAndNumber(Worker currWorker,CommandoCountInfoCommonReqDTO dto,List<String> commandoList) {
        StringBuilder sb = new StringBuilder(COMMANDO_TYPE_SQL);
        List<Object> params = new ArrayList<>();

        /*//增加申请人
        if (ObjectUtil.isNotEmpty(currWorker.getLoginId())){
            sb.append(" and ( a.create_id = " + "\'" + currWorker.getLoginId() + "\' "  );
            if (ObjectUtil.isEmpty(commandoList)){
                sb.append(" ) ");
            }
        }

        // 权限控制
        if (ObjectUtil.isNotEmpty(commandoList)){
            String sign = " and ";
            String inSql = getInSql(commandoList, "a.commando_code", params);
            if (ObjectUtil.isNotEmpty(currWorker.getLoginId())){
                sign = " or ";
            }
            sb.append(sign + inSql);
            if (ObjectUtil.isNotEmpty(currWorker.getLoginId())){
                sb.append(" ) ");
            }
        }*/
        if (ObjectUtil.isNotEmpty(commandoList)){
            String inSql = getInSql(commandoList, "a.commando_code", params);
            sb.append("and " + inSql);
        }else {
            return Collections.emptyList();
        }
        // 通用查询
        commonConditionSet(sb, null, params, dto);
        sb.append(" group by type , a.commando_code  ORDER BY typeTeamMemberNumber asc ");
        return jdbcTemplate.queryForBeans(CommandoTypeRatioRespDTO.class, sb.toString(), params.toArray());
    }

    /**
     * 通用条件查询
     */
    private void commonConditionSet(StringBuilder sb, StringBuilder sbCount, List<Object> params, CommandoCountInfoCommonReqDTO dto) {
        // 查询组织
/*        if (StringUtils.hasText(dto.getOrgId())) {
            sb.append(" and a.group_org_id = ? ");
            if (Objects.nonNull(sbCount)) {
                sbCount.append(" and a.group_org_id = ? ");
            }
            params.add(dto.getOrgId());
        }*/

        // 开始日期
        if (StringUtils.hasText(dto.getStartDate())) {
            sb.append(" and to_date(a.create_time,'yyyy-mm-dd hh24:mi:ss') >= ? ");
            if (Objects.nonNull(sbCount)) {
                sbCount.append(" and to_date(a.create_time,'yyyy-mm-dd hh24:mi:ss') >= ? ");
            }
            params.add(dto.getStartDate() +" 00:00:00");
        }

        // 结束日期
        if (StringUtils.hasText(dto.getEndDate())) {
            sb.append(" and to_date(a.create_time,'yyyy-mm-dd hh24:mi:ss') <= ? ");
            if (Objects.nonNull(sbCount)) {
                sbCount.append(" and to_date(a.create_time,'yyyy-mm-dd hh24:mi:ss') <= ? ");
            }
            params.add(dto.getEndDate() + " 23:59:59");
        }
    }


    /** 统计队伍状态冻结数 */
    public Integer getCommandoState(Worker currWorker,CommandoCountInfoCommonReqDTO dto,String commandoInSql) {
        StringBuilder sb = new StringBuilder(COMMANDO_STATE);
        List<Object> params = new ArrayList<>();
        // 通过唯一编码获取冻结状态
        if (ObjectUtil.isNotEmpty(commandoInSql)){
            sb.append(" and a.commando_code in " + commandoInSql);
        }
        // 查询
        commonConditionSet(sb, null, params, dto);

//        //增加申请人
//        if (ObjectUtil.isNotEmpty(currWorker.getLoginId())){
//            sb.append(" or ( a.create_id = " + "\'" + currWorker.getLoginId() + "\' and a.state = 3  and a.del_flag = 0 )"  );
//        }

        sb.append(" ) where commandoState = '冻结' ");

        return jdbcTemplate.queryForObject(sb.toString(), Integer.class, params.toArray());
    }

    public List<CommandoOrganizationRespDTO> findCoincidenceOrganization(Worker currWorker) {
        String organSql = "select org_id , org_name from tsm_org where state = 1 ";
        return jdbcTemplate.queryForBeans(CommandoOrganizationRespDTO.class,organSql);
    }


    /** 查询用户相关的组织信息 */
    public List<TsmOrg> findOrgContent(String orgId){
        String sql =" select w.org_id org_code , w.registration_system , o.org_id , o.org_name , o.org_type , o.manage_people from tsm_worker w  "
            + "left join tsm_org o on w.org_id = o.org_code and w.state = 1 and o.state = 1 ";
        return jdbcTemplate.queryForBeans(TsmOrg.class,sql);
    }

    public List<Long> findAllOrgAuth(int loginId) {
        String SQL = " select distinct org_id from TSM_WORKER_ORG_ALL where worker_id =" + loginId;
        return jdbcTemplate.queryForRawObjects(Long.class, SQL);
    }

    public List<TsmOrgContentDTO> findOrgManagePeople(String orgType,String orgLevel) {
        StringBuilder sql = new StringBuilder("select  t.*  from tsm_org t where t.state = 1 ");
        ArrayList<Object> params = new ArrayList<>();

        if (StringUtils.hasText(orgType)){
            sql.append(" and t.org_type = ? " );
            params.add(orgType);
        }

        if (StringUtils.hasText(orgLevel)){
            sql.append(" and t.org_level = ? " );
            params.add(orgLevel);
        }

        return jdbcTemplate.queryForBeans(TsmOrgContentDTO.class, sql.toString(),params.toArray());
    }

    /** 通过目前权限组织查询突击队赋权组织关系表 */
    public List<String> findIdByOrgIdIn(String permissionOrgIdStr) {
        if (ObjectUtil.isNotEmpty(permissionOrgIdStr)){
            String sql = "SELECT distinct commando_code from YCQ_QZH_COMMANDO_ORG  where org_id in ( " + permissionOrgIdStr + " )";
            List<String> ids = jdbcTemplate.queryForRawObjects(String.class,sql);
            return ids;
        }
        return Collections.emptyList();
    }



    /**
     * 拼接in查询
     *
     * @param list     in里边的值
     * @param paramKey sql中in的key
     * @param params   查询参数
     */
    public static String getInSql(List list, String paramKey, List<Object> params) {
        StringBuffer sbSql = new StringBuffer();
        sbSql.append(" ( " + paramKey + " in ( ");
        for (int i = 0; i < list.size(); i++) {
            sbSql.append("?,");
            if ((i + 1) % 1000 == 0 && (i + 1) < list.size()) {
                //去掉最后的逗号
                sbSql.deleteCharAt(sbSql.length() - 1);
                sbSql.append(" ) OR ").append(paramKey).append(" IN (");
            }
            params.add(list.get(i));
        }
        //去掉最后的逗号
        sbSql.deleteCharAt(sbSql.length() - 1);
        sbSql.append(" ) ) ");
        return sbSql.toString();
    }

    public List<QueryCommandoRespDTO> findCommonedContent(Worker worker) {
        StringBuilder sb = new StringBuilder(" select * from YCQ_QZH_COMMANDO a where  a.del_flag = 0 and a.state = 3   ");
        List<Object> params = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(worker) && ObjectUtil.isNotEmpty(worker.getLoginId())){
            sb.append(" and a.create_id = ? ");
            params.add(worker.getLoginId());
        }
        return jdbcTemplate.queryForBeans(QueryCommandoRespDTO.class, sb.toString(), params.toArray());
    }
}
