/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved. <a>http://www.cqcis.com</a>
 */
package com.crunii.micro.service.qzh.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.qzh.api.domain.dto.ParticipantDetailVO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.DefaultTemplateDTO;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.QagPageRsp;
import com.crunii.micro.service.qzh.dto.DefaultTemplateVO;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: 抽取题目Jdbc
 * @author: limy
 * @date: 2023/9/7 18:53
 **/
@Repository
public class DefaultTemplateJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    private static final String PAGE_BASE_SQL = "SELECT id,user_id,activity_id, phone_number phoneNumber,publisher,to_char(publish_time, 'yyyy-MM-dd hh24:mi:ss') publishTime," +
            "content,is_publicity isPublicity,activity_rating activityRating,is_anonymity isAnonymity FROM YCQ_ACTIVITY_PARTICIPATION_DETAILS WHERE 1=1 ";
    private static final String PAGE_COUNT_SQL = "SELECT COUNT(*) FROM YCQ_ACTIVITY_PARTICIPATION_DETAILS WHERE 1=1 ";

    public QagPageRsp<DefaultTemplateVO> queryList(PageReq<DefaultTemplateDTO> req) {
        DefaultTemplateDTO query = req.getQuery();
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> params = new ArrayList<>();


        sb.append("AND activity_id = ? ");
        sbCount.append("AND activity_id = ? ");
        params.add(query.getActivityId());
        //查询活动表

        // 固定查询条件
        if (query.getOrganizationId() != null) {
            sb.append("AND organization_id = ? ");
            sbCount.append("AND organization_id = ? ");
            params.add(query.getOrganizationId());
        }

        if (query.getActivityRating() != null) {
            sb.append("AND activity_rating = ? ");
            sbCount.append("AND activity_rating = ? ");
            params.add(query.getActivityRating());
        }
        if (query.getAuditState() != null) {
            sb.append("AND audit_state = ? ");
            sbCount.append("AND audit_state = ? ");
            params.add(query.getAuditState());
        }
        // 模糊查询参数
        if (query.getTitle() != null) {
            sb.append("AND title LIKE ? ");
            sbCount.append("AND title LIKE ? ");
            params.add("%"+query.getTitle()+"%");

        }
        if (query.getPhoneNumber() != null) {
            sb.append("AND phone_number LIKE ? ");
            sbCount.append("AND phone_number LIKE ? ");
            params.add("%"+query.getPhoneNumber()+"%");

        }

        if (query.getPublisher() != null) {
            sb.append("AND publisher LIKE ? ");
            sbCount.append("AND publisher LIKE ? ");
            params.add("%"+query.getPublisher()+"%");
        }

        // 时间范围查询
        if (query.getStartPublicationTime() != null && query.getEndPublicationTime() != null) {
            sb.append("AND publish_time BETWEEN ? AND ? ");
            sbCount.append("AND publish_time BETWEEN ? AND ? ");
            params.add(query.getStartPublicationTime());
            params.add(query.getEndPublicationTime());
        }

        sb.append(" order by is_publicity ");
        // 获取总数
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<DefaultTemplateVO> result =new ArrayList<>();
        if (total > 0) {
            String dbType = jdbcTemplate.getDbType();
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            // 查询结果
             result = jdbcTemplate.queryForBeans(DefaultTemplateVO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        QagPageRsp<DefaultTemplateVO> page = new QagPageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
//        page.setIsBatchAudit(contentNeededAudit);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }


}
