package com.crunii.micro.service.qzh.jdbc;

import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.qzh.dto.dynamicform.DynamicFormTableReqDTO;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.query.internal.NativeQueryImpl;
import org.hibernate.transform.Transformers;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName DynamicFormService
 * @Description 动态表单
 * @Author Jiang
 * @Date 2023/10/13
 * @Version 1.0
 */
@Repository
public class DynamicFormDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate mainJT;


    /**
     * @MethodName createTable
     * @Description 根据json数据组件字段信息创建表
     * @param: createSql 建表sql语句
     * @Author Jiang
     * @Date 2023/10/14 19:20
     */
    public void createTable(String createSql) {
        mainJT.execute(createSql);
    }


    /**
     * @MethodName insertByTableName
     * @Description  通过表名保存数据到对应的表中
     * @param: toString
     * @param: toArray
     * @Author Jiang
     * @Date 2023/10/16 19:36
     */
    public void insertByTableName(String insertSql, Object[] params) {
        mainJT.update(insertSql, params);
    }


    /**
     * @MethodName findTableDataByBusinessIdAndTableName
     * @Description 通过业务id和表名查询动态表单提交的表单数据的值
     * @param: dynamicFormDao
     * @return: java.util.Map<java.lang.String, java.lang.Object>
     * @Author Jiang
     * @Date 2023/10/13 18:21
     */
    public Map<String, Object> findTableDataByBusinessIdAndTableName(DynamicFormTableReqDTO dynamicFormTableReq) {
        if (StringUtils.isBlank(dynamicFormTableReq.getTableName())) {
            throw new BusinessException("表名必传!");
        }
        StringBuilder sb = new StringBuilder("select  * from " + dynamicFormTableReq.getTableName() + " where 1=1 ");
        List<Object> params = new ArrayList<>();
        if (StringUtils.isNotBlank(dynamicFormTableReq.getBusinessId())) {
            sb.append(" AND business_id = ? ");
            params.add(dynamicFormTableReq.getBusinessId());
        }
        int i = mainJT.queryForInt(sb.toString(), params.toArray());
        if (i > 0) {
            return mainJT.queryForMap(sb.toString(), params.toArray());
        }
        return new HashMap<>();
    }



}
