package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qzh.dto.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import org.springframework.stereotype.Repository;

/**
 * @Author thw
 * @date 2023/9/5
 * @description 指标体系评价jdbc
 */
@Repository
public class IndicatorAppraiseDao {

  @Resource(name = "mainJT")
  protected BaseJdbcTemplate jdbcTemplate;

  @Resource(name = "mainNPJT")
  protected BaseNPJdbcTemplate NPJT;

  // 分组聚合sql
  private static final String findListSql = " select  a.id , count(b.indicator_system_id )  activeNumber , LISTAGG (b.id::varchar , ',') WITHIN GROUP (ORDER BY b.id) AS ids  from  YCQ_QZH_INDICATOR_SYSTEM  a   "
      + " left  join YCQ_QZH_ACTIVITY b on a.id = b.indicator_system_id  and b.del_flag = 0 ";

  // 风采评定分页
  private static final String PAGE_APPRAISE_BASE_SQL = " select ROWNUM orderNumber , a.id , a.activity_title title, m.id mid , m.mien_id , ifnull(m.state, 0) state , "
      + "case when (m.state = 0)  then '待评定' when m.state = 1 then '已通过' when m.state = 2 then '已驳回' end stateName  "
      + "from YCQ_QZH_ACTIVITY a "
      + "left join YCQ_MIEN_EVALUATE m on a.id = m.mien_id  "
      + "where a.del_flag = 0 and m.del_flag = 0 ";
  private static final String PAGE_APPRAISE_COUNT_SQL = " select count(*) from YCQ_QZH_ACTIVITY a "
      + "left join YCQ_MIEN_EVALUATE m on a.id = m.mien_id  "
      + "where a.del_flag = 0 and m.del_flag = 0 ";

    // 指标列表导出
    // private static final String EXPORT_APPRAISE_SQL =
    //     " select  h.*, k.activity_type, k.parent_id, k.waitState, k.yesState, k.noState from YCQ_QZH_INDICATOR_SYSTEM h  \n"
    //         + " left join \n"
    //         + " (select a.id, b.activity_type, b.parent_id, sum(case when m.state = 0 then 1 else 0 end) as waitState, \n"
    //         + " sum(case when m.state = 1 then 1 else 0 end) as yesState, sum(case when m.state = 2 then 1 else 0 end) as noState \n"
    //         + " from YCQ_QZH_INDICATOR_SYSTEM  a \n"
    //         + " left join YCQ_QZH_ACTIVITY b on a.id = b.indicator_system_id and b.activity_type = 4  and b.del_flag = 0 \n"
    //         + " left join YCQ_MIEN_EVALUATE m on a.id = m.indicator_id  and m.del_flag = 0 \n"
    //         + " group by a.id, b.activity_type, b.parent_id ) k \n"
    //         + " on h.id = k.id   \n"
    //         + " where h.deleted = 0 and indicator_version_id = ? ";
    /** 指标列表导出 */
    private static final String EXPORT_APPRAISE_SQL =
        " select sys.*, src.parent_id, src.activity_type, src.wait_state, src.yes_state, src.no_state\n"
            + " FROM ycq_qzh_indicator_system sys LEFT JOIN\n"
            + " (SELECT b.parent_id, a.indicator_id, b.activity_type, sum(case when a.state = 0 then 1 else 0 end) as wait_state,\n"
            + " sum(case when a.state = 1 then 1 else 0 end) as yes_state, sum(case when a.state = 2 then 1 else 0 end) as no_state\n"
            + " FROM ycq_mien_evaluate a \n"
            + " JOIN ycq_qzh_activity b ON a.mien_id  = b.id AND a.indicator_id = b.indicator_system_id \n"
            + " WHERE a.del_flag = 0 AND b.activity_type = 4 AND b.del_flag = 0 \n"
            + " GROUP BY b.parent_id, a.indicator_id, b.activity_type) src ON sys.id = src.indicator_id\n"
            + " WHERE sys.deleted = 0 AND sys.indicator_version_id = ? ";
  
  // public List<IndicatorAppraiseRespDTO> queryAppraiseListOld(Worker worker, IndicatorAppraiseReqDTO dto) {
  //   StringBuilder groupSql = new StringBuilder(findListSql);
  //   List<Object> params = new ArrayList<>();
  //
  //   // 且通过风采表的ACTIVITY_TYPE = 4 可以得到权益岗相关的风采，风采里面有关联了指标体系id的。并且可以通过 ACTIVITY_TYPE = 4 与 PARENT_ID 锁定某一个权益岗的指标内容
  //   if (ObjectUtil.isNotEmpty(dto.getRightsId())){
  //     groupSql.append(" and b.parent_id = ?  and b.activity_type = 4  ");
  //     params.add(Long.valueOf(dto.getRightsId()));
  //     groupSql.append(" group by a.id  ");
  //   }else {
  //     groupSql.append(" group by a.id  ");
  //   }
  //
  //   // 最终sql
  //   StringBuilder sb = new StringBuilder(" select  h.* , h.id indexId , k.activeNumber , k.ids from  YCQ_QZH_INDICATOR_SYSTEM h  "
  //       + "   left join    "
  //       + "   (  " + groupSql + "    ) k on h.id = k.id   "
  //       + "   where h.deleted = '1' ");
  //
  //   if (Objects.isNull(dto.getVersionId())){
  //     throw new BusinessException("指标体系版本主键id必传!");
  //   }else {
  //     // 某个指标版本下的指标体系内容
  //     sb.append(" and h.indicator_version_id = ? ");
  //     params.add(Long.valueOf(dto.getVersionId()));
  //   }
  //
  //   // 评价要点
  //   if (StrUtil.isNotBlank(dto.getEvaluationPoints())) {
  //     sb.append(" and h.evaluation_points like concat('%',?,'%') ");
  //     params.add(dto.getEvaluationPoints());
  //   }
  //   // 一级指标
  //   if (StrUtil.isNotBlank(dto.getPrimaryIndicators())) {
  //     sb.append(" and h.primary_indicators like concat('%',?,'%') ");
  //     params.add(dto.getPrimaryIndicators());
  //   }
  //   // 二级指标
  //   if (StrUtil.isNotBlank(dto.getSecondaryIndicators())) {
  //     sb.append(" and h.secondary_indicators like concat('%',?,'%') ");
  //     params.add(dto.getSecondaryIndicators());
  //   }
  //
  //   sb.append(" order by  h.indicator_version_id , h.evaluation_points  , h.primary_indicators  ");
  //
  //   List<IndicatorAppraiseRespDTO> result = jdbcTemplate.queryForBeans(IndicatorAppraiseRespDTO.class, sb.toString(), params.toArray());
  //   return result;
  // }

    /**
     * 查询指标体系评价列表
     * @author leixb
     * @since 2023-11-23 18:53
     * @param worker 用户信息
     * @param dto 请求参数
     * @return List<IndicatorAppraiseRespDTO>
     */
    public List<IndicatorAppraiseRespDTO> queryAppraiseList(Worker worker, IndicatorAppraiseReqDTO dto) {
        StringBuilder sql = new StringBuilder(
            " SELECT sys.id index_id, evaluation_points, primary_indicators, secondary_indicators, score, COALESCE(act.active_number,0) active_number\n"
          + " FROM ycq_qzh_indicator_system sys LEFT JOIN \n"
          + " (SELECT a.id, count(b.indicator_system_id) active_number FROM ycq_qzh_indicator_system a \n"
          + " LEFT JOIN ycq_qzh_activity b ON  a.id = b.indicator_system_id \n"
          + " WHERE a.deleted = 0 AND a.indicator_version_id = ? \n"
          + " AND b.del_flag = 0 AND b.activity_type = 4 AND b.parent_id = ? GROUP BY a.id) act \n"
          + " ON sys.id = act.id WHERE sys.deleted = 0 AND sys.indicator_version_id = ? \n");
        List<Object> params = new ArrayList<>();
        params.add(dto.getVersionId());
        params.add(dto.getRightsId());
        params.add(dto.getVersionId());
        // 评价要点
        if (StrUtil.isNotBlank(dto.getEvaluationPoints())) {
          sql.append(" and evaluation_points like concat('%',?,'%') ");
          params.add(dto.getEvaluationPoints());
        }
        // 一级指标
        if (StrUtil.isNotBlank(dto.getPrimaryIndicators())) {
          sql.append(" and primary_indicators like concat('%',?,'%') ");
          params.add(dto.getPrimaryIndicators());
        }
        // 二级指标
        if (StrUtil.isNotBlank(dto.getSecondaryIndicators())) {
          sql.append(" and secondary_indicators like concat('%',?,'%') ");
          params.add(dto.getSecondaryIndicators());
        }
        sql.append(" ORDER BY evaluation_points, primary_indicators, create_time ");
        return jdbcTemplate.queryForBeans(IndicatorAppraiseRespDTO.class, sql.toString(), params.toArray());
    }
  
    /**
     * 风采评定分页
     * @author leixb
     * @since 2023-11-23 18:50
     * @param currWorker 用户
     * @param req 请求参数
     * @return PageRsp<IndicatorAppraiseDecideRespDTO>
     */
    public PageRsp<IndicatorAppraiseDecideRespDTO> appraiseMienPage( Worker currWorker,
        PageReq<IndicatorAppraiseDecideReqDTO> req) {
        StringBuilder sb = new StringBuilder(" SELECT ROWNUM order_number, a.id, a.activity_title title, a.indicator_system_id, m.id mid, m.state, \n"
                + " CASE WHEN m.state = 0 THEN '待评定' WHEN m.state = 1 then '已通过' WHEN m.state = 2 THEN '已驳回' END state_name \n"
                + " FROM ycq_qzh_activity a JOIN ycq_mien_evaluate m ON a.id = m.mien_id AND a.indicator_system_id = m.indicator_id \n"
                + " WHERE a.del_flag = 0 AND m.del_flag = 0 AND a.parent_id = ? AND m.indicator_id = ? ");
        StringBuilder sbCount = new StringBuilder(" SELECT count(*) FROM ycq_qzh_activity a JOIN ycq_mien_evaluate m ON a.id = m.mien_id AND a.indicator_system_id = m.indicator_id  \n"
          + " WHERE a.del_flag = 0 AND m.del_flag = 0 AND a.parent_id = ? AND m.indicator_id = ? ");
        List<Object> params = new ArrayList<>();
        IndicatorAppraiseDecideReqDTO query = req.getQuery();
        params.add(query.getRightsId());
        params.add(query.getIndexId());
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<IndicatorAppraiseDecideRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
          //根据数据库类型设置limit
          SqlUtil.appendLimit(dbType, params, sb, req);
          result = jdbcTemplate.queryForBeans(IndicatorAppraiseDecideRespDTO.class, sb.toString(), params.toArray());
        } else {
          result = new ArrayList<>(0);
        }
        PageRsp<IndicatorAppraiseDecideRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

  /**
   * @Description: 风采评定分页
   * @Author: thw
   * @Date: 2023/9/11 15:49
   * @Params:
   * @param currWorker
   * @param req
   * @return: com.crunii.micro.common.dto.PageRsp<com.crunii.micro.service.qzh.api.domain.dto.IndicatorAppraiseDecideRespDTO>
   **/
  public PageRsp<IndicatorAppraiseDecideRespDTO> appraiseMienPageOld(Worker currWorker,
      PageReq<IndicatorAppraiseDecideReqDTO> req) {
    StringBuilder sb = new StringBuilder(PAGE_APPRAISE_BASE_SQL);
    StringBuilder sbCount = new StringBuilder(PAGE_APPRAISE_COUNT_SQL);
    List<Object> params = new ArrayList<>();
    IndicatorAppraiseDecideReqDTO query = req.getQuery();
    String inSql  = "";
    if (ObjectUtil.isNotEmpty(query.getIds())){
      inSql = Arrays.stream(query.getIds().split(",")).collect(Collectors.joining(",", "(", ")"));
      sb.append(" and m.mien_id in " + inSql);
      sbCount.append(" and m.mien_id in " + inSql);
    }

    int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
    List<IndicatorAppraiseDecideRespDTO> result;
    String dbType = jdbcTemplate.getDbType();
    if (total > 0) {
      //根据数据库类型设置limit
      SqlUtil.appendLimit(dbType, params, sb, req);

      result = jdbcTemplate.queryForBeans(IndicatorAppraiseDecideRespDTO.class, sb.toString(), params.toArray());
    } else {
      result = new ArrayList<>(0);
    }
    PageRsp<IndicatorAppraiseDecideRespDTO> page = new PageRsp<>();
    page.setPageNo(req.getPageNo());
    page.setPageSize(req.getPageSize());
    page.setTotalNum(total);
    page.setResult(result);
    // 为手机计算是否由下一页
    page.next();

    return page;
  }

  /** 查询指标体系版本内容 */
  public List<IndexSystemSimpleRespDTO> findIndexSystemVersion() {
    String sql = " SELECT id ,version_name FROM ycq_qzh_indicator_version WHERE deleted = 0 ORDER BY create_time DESC ";
    return jdbcTemplate.queryForBeans(IndexSystemSimpleRespDTO.class, sql);
  }


  /**
   * 满足条件的查询维权岗
   * 条件：维权岗创建流程为公示通过，状态为创建中的
   * */
  public List<CoincidenceOrganizationRespDTO> findPostInformation(Worker currWorker, String unitName) {
      String sql = " SELECT id, unit_name FROM ycq_qzh_rights_protection p \n"
          + "LEFT JOIN (SELECT parent_id,MAX(update_time) update_time, MAX(create_time) create_time FROM ycq_qzh_activity \n"
          + "WHERE activity_type = 4 AND del_flag = 0 GROUP BY parent_id) a ON p.id = a.parent_id \n"
          + "WHERE p.link = '1006' AND p.del_flag = 0 \n"
          + "ORDER BY COALESCE(a.update_time,a.create_time,p.create_time) DESC ";
      List<Object> params = new ArrayList<>();
      if (StrUtil.isNotEmpty(unitName)) {
          sql += " AND p.unit_name like concat('%',?,'%') ";
          params.add(unitName);
      }
      return jdbcTemplate.queryForBeans(CoincidenceOrganizationRespDTO.class, sql, params.toArray());
  }

  /** 导出指标体系内容 */
  public List<ExportIndexSystemRespDTO> exportIndexSystemContent(Worker currWorker, String inSql, boolean label, String versionId) {
      StringBuilder sb = new StringBuilder(EXPORT_APPRAISE_SQL);
      // label=true,说明查询到的维权岗下拉存在数据
      if (label) {
          // sb.append(" and k.parent_id in " + inSql + " order by h.evaluation_points, h.primary_indicators,
          // h.create_time ");
          sb.append(" AND src.parent_id IN ").append(inSql).append(" ORDER BY sys.evaluation_points, sys.primary_indicators, sys.create_time ");
      }
      return jdbcTemplate.queryForBeans(ExportIndexSystemRespDTO.class, sb.toString(), Integer.parseInt(versionId));
  }
}
