/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved. <a>http://www.cqcis.com</a>
 */
package com.crunii.micro.service.qzh.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.qzh.api.domain.dto.activity.ParticipantDetailDTO;
import com.crunii.micro.service.qzh.api.domain.dto.ParticipantDetailVO;
import com.crunii.micro.service.qzh.dto.IndicatorAppraiseDecideRespDTO;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: 抽取题目Jdbc
 * @author: limy
 * @date: 2023/9/7 18:53
 **/
@Repository
public class ParticipantDetailJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    private static final String PAGE_BASE_SQL = "SELECT * FROM ycq_activity_people_audit WHERE is_deleted = 0 ";
    private static final String PAGE_COUNT_SQL = "SELECT COUNT(*) FROM ycq_activity_people_audit WHERE is_deleted = 0 ";

    public PageRsp<ParticipantDetailVO> queryList(PageReq<ParticipantDetailDTO> req) {
        ParticipantDetailDTO query = req.getQuery();
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> params = new ArrayList<>();

        sb.append("AND activity_id = ? ");
        sbCount.append("AND activity_id = ? ");
        params.add(query.getActivityId());

//        //todo 修改只展示报名通过的数据
//        sb.append("AND registration_status = 1 ");
//        sbCount.append("AND registration_status = 1 ");

        //姓名和联系方式的模糊查询条件
        if (query.getUserName() != null) {
            sb.append("AND user_name LIKE ? ");
            sbCount.append("AND user_name LIKE ? ");
            params.add("%" + query.getUserName() + "%");
        }

        if (query.getContactMethod() != null) {
            sb.append("AND contact_method LIKE ? ");
            sbCount.append("AND contact_method LIKE ? ");
            params.add("%" + query.getContactMethod() + "%");
        }

        // 报名状态、签到结果、签退结果固定查询条件
        if (query.getRegistrationStatus() != null) {
            sb.append("AND registration_status = ? ");
            sbCount.append("AND registration_status = ? ");
            params.add(query.getRegistrationStatus());
        }

        if (query.getCheckInResult() != null) {
            sb.append("AND check_in_result = ? ");
            sbCount.append("AND check_in_result = ? ");
            params.add(query.getCheckInResult());
        }

        if (query.getCheckOutResult() != null) {
            sb.append("AND check_out_result = ? ");
            sbCount.append("AND check_out_result = ? ");
            params.add(query.getCheckOutResult());
        }

        // 时间范围查询
        if (query.getStartRegistrationTime() != null && query.getEndRegistrationTime() != null) {
            sb.append("AND registration_time BETWEEN ? AND ? ");
            sbCount.append("AND registration_time BETWEEN ? AND ? ");
            params.add(query.getStartRegistrationTime());
            params.add(query.getEndRegistrationTime());
        }

        if (query.getStartCheckInTime() != null && query.getEndCheckInTime() != null) {
            sb.append("AND check_in_time BETWEEN ? AND ? ");
            sbCount.append("AND check_in_time BETWEEN ? AND ? ");
            params.add(query.getStartCheckInTime());
            params.add(query.getEndCheckInTime());
        }

        if (query.getStartCheckOutTime() != null && query.getEndCheckOutTime() != null) {
            sb.append("AND check_out_time BETWEEN ? AND ? ");
            sbCount.append("AND check_out_time BETWEEN ? AND ? ");
            params.add(query.getStartCheckOutTime());
            params.add(query.getEndCheckOutTime());
        }
        //报名状态排序
        sb.append(" ORDER BY registration_status DESC ");
        List<ParticipantDetailVO> result =new ArrayList<>();
        PageRsp<ParticipantDetailVO> page = new PageRsp<>();
        // 获取总数
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        if (total > 0) {
            String dbType = jdbcTemplate.getDbType();
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            // 查询结果
            result = jdbcTemplate.queryForBeans(ParticipantDetailVO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }

        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }


}
