package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmMenu;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.LeaderChangeListPageReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.LeaderChangeLogPageRespDTO;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.QagPageRsp;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.QagShowUserDTO;
import com.crunii.micro.service.qzh.api.enums.QagChangeLeaderEnum;
import com.crunii.micro.service.qzh.dto.YcqQzhQagApplyRespDTO;
import com.crunii.module.flow.api.vo.FlowInstanceRespVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Repository
public class QagApplyLeaderChangeLogDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private final static String APPLY_CHANGE_LIST_PAGE_SQL = "  SELECT cl.*,cl.createdor creator,cl.created_time createTime,         " +
            "                                                    case when button=0  AND b.org_id=cl.affiliation_org_id then 1         " +
            "                                                    when button=1 AND b.org_code='QCCQ3537136584' then 1         " +
            "                                                    else 0 end button         " +
            "                                                    FROM QAG_APPLY_LEADER_CHANGE_LOG cl                  " +
            "                                                    left join                                      " +
            "                                                       （SELECT case when m.menu_name='初核' then 0     " +
            "                                                        when m.menu_name='复核' then 1 end as button,tw.org_id,o.org_code          " +
            "                                                       FROM TSM_WORKER tw                 " +
            "                                                            LEFT JOIN TSM_WORKER_ROLE wr ON tw.login_id = wr.login_id                 " +
            "                                                            LEFT JOIN  TSM_ROLE r ON r.role_id = wr.role_id                  " +
            "                                                            left join TSM_ROLE_MENU rm on r.role_id=rm.role_id                  " +
            "                                                            left join TSM_MENU m on rm.menu_code=m.menu_code " +
            "                                                            left join TSM_ORG o on o.org_id=tw.org_id WHERE m.state=1              " +
            "                                                            AND  m.menu_type=2  AND  m.parent_menu_code=? AND tw.login_id =?               " +
            "                                                            AND m.menu_name in('初核','复核')) b             " +
            "                                                       on cl.state=b.button             " +
            "                                                       WHERE  cl.is_deleted=0 ";

    private final static String SUPER_APPLY_CHANGE_LIST_PAGE_SQL = " SELECT cl.*,cl.createdor creator,cl.created_time createTime,                " +
            "                                       case when cl.state=0  AND tw.org_id=cl.affiliation_org_id then 1                  " +
            "                                          when cl.state=1 AND o.org_code='QCCQ3537136584' then 1                  " +
            "                                          else 0 end button               " +
            "                                       FROM (select cl.*,? as login_id from QAG_APPLY_LEADER_CHANGE_LOG cl) cl                " +
            "                                       left join TSM_WORKER tw on tw.login_id=cl.login_id  and tw.login_id=?  " +
            "                                       left join TSM_ORG o on o.org_id=tw.org_id  " +
            "                                       where cl.is_deleted=0  ";

    private final static String CHANGE_LIST_PAGE_SQL = "SELECT cl.*,createdor creator,created_time createTime,   " +
            "CASE WHEN is_deleted=1 THEN -1 else state end state, mr.major_achievement,                                       " +
            "CASE WHEN is_deleted=1 THEN '终止任务'     " +
            "WHEN cl.state=0 THEN '待初核'     " +
            "WHEN cl.state=1 THEN '待复核'   " +
            "WHEN cl.state=2 THEN '已通过'                                           " +
            "WHEN cl.state=3 THEN '草稿' END stateName,  " +
            "CASE WHEN cl.is_deleted=1 THEN 1  " +
            "WHEN  cl.state=3 THEN 0 " +
            "WHEN  cl.state=2 THEN 1 else 1 END sortState   " +
            "FROM QAG_APPLY_LEADER_CHANGE_LOG cl                                       " +
            "LEFT JOIN YOUTH_SAFETY_MODEL_RECOMMEND mr ON cl.qag_apply_id=mr.qag_apply_id                                      " +
            "WHERE 1=1 ";

    private final static String APPLY_CHANGE_LIST_COUNT_SQL = " SELECT count(*)    " +
            "       FROM QAG_APPLY_LEADER_CHANGE_LOG cl              " +
            "       left join                                  " +
            "          （SELECT case when m.menu_name='初核' then 0 " +
            "           when m.menu_name='复核' then 1 end as button,tw.org_id         " +
            "          FROM TSM_WORKER tw             " +
            "               LEFT JOIN TSM_WORKER_ROLE wr ON tw.login_id = wr.login_id             " +
            "               LEFT JOIN  TSM_ROLE r ON r.role_id = wr.role_id              " +
            "               left join TSM_ROLE_MENU rm on r.role_id=rm.role_id              " +
            "               left join TSM_MENU m on rm.menu_code=m.menu_code WHERE m.state=1          " +
            "               AND  m.menu_type=2  AND  m.parent_menu_code=? AND tw.login_id =?           " +
            "               AND m.menu_name in('初核','复核')) b         " +
            "          on cl.state=b.button         " +
            "          WHERE  cl.is_deleted=0 ";

    private final static String SUPER_APPLY_CHANGE_LIST_COUNT_SQL = "SELECT count(*)          " +
            "                          FROM (select cl.*,? as login_id from QAG_APPLY_LEADER_CHANGE_LOG cl) cl            " +
            "                          left join TSM_WORKER tw on tw.org_id=cl.login_id  and tw.login_id=?       " +
            "                          where cl.is_deleted=0 ";

    private final static String CHANGE_LIST_COUNT_SQL = "SELECT COUNT(*) FROM QAG_APPLY_LEADER_CHANGE_LOG cl WHERE 1=1 ";

    private boolean validatedSuper(Worker worker) {
        if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            return true;
        }
        return false;
    }

    public QagPageRsp<LeaderChangeLogPageRespDTO> applyChangeListPage(PageReq<LeaderChangeListPageReqDTO> req, List<FlowInstanceRespVO> list, Long totalNum, TsmMenu tsmMenu,List<Long> ids) {

        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        StringBuilder sb = null;
        StringBuilder sbCount = null;
        List<Object> params = new ArrayList<>();
        if (validatedSuper(currWorker)) {
            sb = new StringBuilder(SUPER_APPLY_CHANGE_LIST_PAGE_SQL);
            sbCount = new StringBuilder(SUPER_APPLY_CHANGE_LIST_COUNT_SQL);
            params.add(currWorker.getLoginId());
            params.add(currWorker.getLoginId());
        } else {
            sb = new StringBuilder(APPLY_CHANGE_LIST_PAGE_SQL);
            sbCount = new StringBuilder(APPLY_CHANGE_LIST_COUNT_SQL);
            params.add(tsmMenu.getMenuCode());
            params.add(currWorker.getLoginId());
            if (CollectionUtils.isNotEmpty(ids)) {
                sb.append(" and cl.id in([ids])");
                sbCount.append(" and cl.id in([ids])");
                sb = new StringBuilder(sb.toString().replace("[ids]", StringUtils.strip(ids.toString(), "[]")));
                sbCount = new StringBuilder(sbCount.toString().replace("[ids]", StringUtils.strip(ids.toString(), "[]")));
            } else {
                sb.append(" and 1=2");
                sbCount.append(" and 1=2");
            }

        }
        LeaderChangeListPageReqDTO dto = req.getQuery();

        if (StrUtil.isNotBlank(dto.getCode())) {
            sb.append(" and cl.code like concat('%',?,'%') ");
            sbCount.append(" and cl.code like concat('%',?,'%') ");
            params.add(dto.getCode());
        }
        if (StrUtil.isNotBlank(dto.getApplyOrgName())) {
            sb.append(" and cl.apply_org_name like concat('%',?,'%') ");
            sbCount.append(" and cl.apply_org_name like concat('%',?,'%') ");
            params.add(dto.getApplyOrgName());
        }
        if (StrUtil.isNotBlank(dto.getAffiliationOrgName())) {
            sb.append(" and cl.affiliation_org_name like concat('%',?,'%') ");
            sbCount.append(" and cl.affiliation_org_name like concat('%',?,'%') ");
            params.add(dto.getAffiliationOrgName());
        }

        if (StrUtil.isNotBlank(dto.getAffiliationOrgId())) {
            sb.append(" and cl.affiliation_org_id = ? ");
            sbCount.append(" and cl.affiliation_org_id = ? ");
            params.add(dto.getAffiliationOrgId());
        }
        if (StrUtil.isNotBlank(dto.getLeaderName())) {
            sb.append(" and cl.leader_name like concat('%',?,'%') ");
            sbCount.append(" and cl.leader_name like concat('%',?,'%') ");
            params.add(dto.getLeaderName());
        }
        if (StrUtil.isNotBlank(dto.getLeaderPhone())) {
            sb.append(" and cl.leader_phone like concat('%',?,'%') ");
            sbCount.append(" and cl.leader_phone like concat('%',?,'%') ");
            params.add(dto.getLeaderPhone());
        }

        if (StrUtil.isNotBlank(dto.getLeaderIdCard())) {
            sb.append(" and cl.leader_id_card like concat('%',?,'%') ");
            sbCount.append(" and cl.leader_id_card like concat('%',?,'%') ");
            params.add(dto.getLeaderIdCard());
        }

        if (StrUtil.isNotBlank(dto.getType())) {
            sb.append(" and cl.type like concat('%',?,'%') ");
            sbCount.append(" and cl.type like concat('%',?,'%') ");
            params.add(dto.getType());
        }

        if (Objects.nonNull(dto.getState())) {
            sb.append(" and cl.state = ? ");
            sbCount.append(" and cl.state= ? ");
            params.add(dto.getState());
        }

        if (StringUtils.isNotBlank(dto.getRemark())) {
            sb.append(" and cl.remark like concat('%',?,'%') ");
            sbCount.append(" and cl.remark like concat('%',?,'%') ");
            params.add(dto.getRemark());
        }

        if (StringUtils.isNotBlank(dto.getCreator())) {
            sb.append(" and cl.createdor like concat('%',?,'%') ");
            sbCount.append(" and cl.createdor like concat('%',?,'%') ");
            params.add(dto.getCreator());
        }

        if (StrUtil.isNotBlank(dto.getStartCreateTime()) && StrUtil.isNotBlank(dto.getEndCreateTime())) {
            sb.append(" and cl.cteated_time <= to_date(?,'yyyy-mm-dd hh24:mi:ss') and cl.cteated_time>=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and cl.cteated_time <= to_date(?,'yyyy-mm-dd hh24:mi:ss') and cl.cteated_time>=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(dto.getStartCreateTime());
            params.add(dto.getEndCreateTime());
        }

        if (StrUtil.isNotBlank(dto.getCreateTime())) {
            sb.append(" and to_char(cl.created_time,'yyyy-mm-dd') = ? ");
            sbCount.append(" and to_char(cl.created_time,'yyyy-mm-dd') = ? ");
            params.add(dto.getCreateTime());
        }

        if (StringUtils.isNotBlank(dto.getDistributionType())) {
            sb.append(" and cl.distribution_type like concat('%',?,'%') ");
            sbCount.append(" and cl.distribution_type like concat('%',?,'%') ");
            params.add(dto.getDistributionType());
        }

        sb.append(" order by button desc,cl.updated_time desc ");


//        sb.append(" and flow_instance_id in  ");
//        sbCount.append(" and flow_instance_id in  ");
//
//        String str = "";
//        for (int i = 0; i < list.size(); i++) {
//            if (i == 0) {
//                str += "'" + list.get(i).getFlowInstanceId() + "'";
//            } else {
//                str += ",'" + list.get(i).getFlowInstanceId() + "'";
//            }
//        }
////        params.add("("+str+")");
//
//        sb.append("("+str+")");
//        sbCount.append("("+str+")");

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<LeaderChangeLogPageRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(LeaderChangeLogPageRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        if (result.size()>0){
            for (LeaderChangeLogPageRespDTO leaderChangeLogPageRespDTO : result) {
                leaderChangeLogPageRespDTO.setLeaderPhone(getSubStr(leaderChangeLogPageRespDTO.getLeaderPhone()));
                leaderChangeLogPageRespDTO.setLeaderIdCard(getSubStr(leaderChangeLogPageRespDTO.getLeaderIdCard()));
                leaderChangeLogPageRespDTO.setLeaderPhoneNew(getSubStr(leaderChangeLogPageRespDTO.getLeaderPhoneNew()));
                leaderChangeLogPageRespDTO.setLeaderIdCardNew(getSubStr(leaderChangeLogPageRespDTO.getLeaderIdCardNew()));
            }
        }
        QagPageRsp<LeaderChangeLogPageRespDTO> page = new QagPageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

    public PageRsp<LeaderChangeLogPageRespDTO> leaderChangeListPage(PageReq<LeaderChangeListPageReqDTO> req, Worker worker) {
        StringBuilder sb = new StringBuilder(CHANGE_LIST_PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(CHANGE_LIST_COUNT_SQL);
        List<Object> params = new ArrayList<>();

        sb.append(" and cl.create_id = ? or cl.leader_phone=? ");
        sbCount.append(" and cl.create_id = ? or cl.leader_phone=? ");
        params.add(worker.getLoginId());
        params.add(worker.getMobile());

        sb.append(" order by sortState,cl.updated_time desc");

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<LeaderChangeLogPageRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(LeaderChangeLogPageRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }

        if (result.size()>0){
            for (LeaderChangeLogPageRespDTO leaderChangeLogPageRespDTO : result) {
                leaderChangeLogPageRespDTO.setLeaderPhone(getSubStr(leaderChangeLogPageRespDTO.getLeaderPhone()));
                leaderChangeLogPageRespDTO.setLeaderIdCard(getSubStr(leaderChangeLogPageRespDTO.getLeaderIdCard()));
                leaderChangeLogPageRespDTO.setLeaderPhoneNew(getSubStr(leaderChangeLogPageRespDTO.getLeaderPhoneNew()));
                leaderChangeLogPageRespDTO.setLeaderIdCardNew(getSubStr(leaderChangeLogPageRespDTO.getLeaderIdCardNew()));
            }
        }

        PageRsp<LeaderChangeLogPageRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

    public String getSubStr(String str){
        if (StrUtil.isNotBlank(str)) {
            int length = str.length();
            String firstStr=str.substring(0,3);
            String lastStr=str.substring(length-4);
            return firstStr+"****"+lastStr;
        }
        return "";
    }


    public TsmOrg getExamineOrg(QagChangeLeaderEnum e,Long orgId) {
        String sql = "SELECT *  FROM TSM_ORG  where org_level=? START WITH org_id=?  CONNECT BY PRIOR parent_org_id=org_id";
        List<Object> params = new ArrayList<>();
        params.add(e.getLevel());
        params.add(orgId);
        return jdbcTemplate.queryForBean(TsmOrg.class, sql, params.toArray());
    }

    public List<TsmOrg> getExamineUpAllOrg(Long orgId) {
        String sql = "SELECT *  FROM TSM_ORG  START WITH org_id=?  CONNECT BY PRIOR parent_org_id=org_id";
        List<Object> params = new ArrayList<>();
        params.add(orgId);
        return jdbcTemplate.queryForBeans(TsmOrg.class, sql, params.toArray());
    }

    public Integer applyChangeTotal(Worker currWorker) {
        StringBuilder sbCount = new StringBuilder(CHANGE_LIST_COUNT_SQL);
        sbCount.append(" and create_id = ? ");
        List<Object> params = new ArrayList<>();
        params.add(currWorker.getLoginId());
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        return total;
    }

    public List<QagShowUserDTO> queryShowUser(String flowInstanceId) {
        String sql = "select NODE_ASSIGENN,NODE_ASSIGENN_NAME,NODE_ASSIGENN_ORG,CREATE_TIME  " +
                "from CRUNII_FLOW_PROCESS_HISTORY where flow_instance_id = ? " +
                "and node_key='958d6e24-4602-4884-b4ec-18297eecf63c' and PROCESS_TYPE=1 order by create_time desc limit 1 ";
        return jdbcTemplate.queryForBeans(QagShowUserDTO.class,sql,flowInstanceId);
    }

    public Integer countChangePending(TsmMenu tsmMenu,Worker currWorker,List<Long> ids) {
        
        String superSql = "  select count(*) from (SELECT                 " +
                "                                                        case when cl.state=0  AND tw.org_id=cl.affiliation_org_id then 1                      " +
                "                                                           when cl.state=1 AND o.org_code='QCCQ3537136584' then 1                      " +
                "                                                           else 0 end button                   " +
                "                                                        FROM (select cl.*,? as login_id from QAG_APPLY_LEADER_CHANGE_LOG cl) cl                    " +
                "                                                        left join TSM_WORKER tw on tw.login_id=cl.login_id  and tw.login_id=? " +
                "                                                        left join TSM_ORG o on o.org_id=tw.org_id " +
                "                                                        where cl.is_deleted=0 ) where button=1 ";
        
        String sql = "  SELECT         " +
                "                                                                     case when button=0  AND b.org_id=cl.affiliation_org_id then 1             " +
                "                                                                     when button=1 AND b.org_code='QCCQ3537136584' then 1             " +
                "                                                                     else 0 end button             " +
                "                                                                     FROM QAG_APPLY_LEADER_CHANGE_LOG cl                      " +
                "                                                                     left join                                          " +
                "                                                                        （SELECT case when m.menu_name='初核' then 0         " +
                "                                                                         when m.menu_name='复核' then 1 end as button,tw.org_id,o.org_code                " +
                "                                                                        FROM TSM_WORKER tw                     " +
                "                                                                             LEFT JOIN TSM_WORKER_ROLE wr ON tw.login_id = wr.login_id                     " +
                "                                                                             LEFT JOIN  TSM_ROLE r ON r.role_id = wr.role_id                      " +
                "                                                                             left join TSM_ROLE_MENU rm on r.role_id=rm.role_id                      " +
                "                                                                             left join TSM_MENU m on rm.menu_code=m.menu_code " +
                "                                                                             left join TSM_ORG o on o.org_id=tw.org_id WHERE m.state=1                  " +
                "                                                                             AND  m.menu_type=2  AND  m.parent_menu_code=? AND tw.login_id = ?                  " +
                "                                                                             AND m.menu_name in('初核','复核')) b                 " +
                "                                                                        on cl.state=b.button                 " +
                "                                                                        WHERE  cl.is_deleted=0  ";
        if (validatedSuper(currWorker)) {
            ArrayList<Object> params = new ArrayList<>();
            params.add(currWorker.getLoginId());
            params.add(currWorker.getLoginId());
            return jdbcTemplate.queryForInt(superSql, params.toArray());
        } else {
            StringBuilder sb = new StringBuilder(sql);
            if (CollectionUtils.isNotEmpty(ids)) {
                sb.append(" and cl.id in([ids])");
                sb = new StringBuilder(sb.toString().replace("[ids]", StringUtils.strip(ids.toString(), "[]")));
            } else {
                sb.append(" and 1=2");
            }
            ArrayList<Object> params = new ArrayList<>();
            params.add(tsmMenu.getMenuCode());
            params.add(currWorker.getLoginId());
            sb.insert(0,"select count(*) from ( ");
            sb.append(" ) where button=1 ");
            return jdbcTemplate.queryForInt(sb.toString(), params.toArray());
        }
    }
}
