/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved. <a>http://www.cqcis.com</a>
 */
package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.qzh.api.domain.dto.RegistrationAuditVO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.DefaultTemplateDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.RegistrationAuditDTO;
import com.crunii.micro.service.qzh.dto.DefaultTemplateVO;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: 抽取题目Jdbc
 * @author: limy
 * @date: 2023/9/7 18:53
 **/
@Repository
public class YcqActivityPeopleAuditJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    private static final String PAGE_BASE_SQL = "SELECT * FROM YCQ_ACTIVITY_PEOPLE_AUDIT WHERE is_deleted = 0 ";
    private static final String PAGE_COUNT_SQL = "SELECT COUNT(*) FROM YCQ_ACTIVITY_PEOPLE_AUDIT WHERE is_deleted = 0 ";

    public PageRsp<RegistrationAuditVO> queryList(PageReq<RegistrationAuditDTO> req) {

        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(req.getQuery().getActivityId())){
            sb.append( " and activity_id = ? ");
            sbCount.append( " and activity_id = ? ");
            params.add(req.getQuery().getActivityId());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getAuditState())){
            sb.append( " and registration_status = ? ");
            sbCount.append( " and registration_status = ? ");
            params.add(req.getQuery().getAuditState());
        }
        sb.append(" order by registration_status desc");
        // 获取总数
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<RegistrationAuditVO> result =new ArrayList<>();
        if (total > 0) {
            String dbType = jdbcTemplate.getDbType();
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            // 查询结果
             result = jdbcTemplate.queryForBeans(RegistrationAuditVO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<RegistrationAuditVO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }


}
