package com.crunii.micro.service.qzh.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqActivityParticipationDetails;
import com.crunii.micro.domain.entity.main.YcqDynamicForm;
import com.crunii.micro.domain.entity.main.YcqDynamicRecord;
import com.crunii.micro.domain.entity.main.dynamicform.YcqQzhDynamicFormColumnValue;
import com.crunii.micro.domain.entity.main.dynamicform.YcqQzhDynamicFormRecord;
import com.crunii.micro.domain.repo.main.YcqActivityParticipationDetailsRepo;
import com.crunii.micro.domain.repo.main.YcqDynamicFormRepo;
import com.crunii.micro.domain.repo.main.YcqDynamicRecordRepo;
import com.crunii.micro.service.qzh.api.domain.dto.activity.DefaultTemplateDTO;
import com.crunii.micro.service.qzh.dto.DefaultTemplateVO;
import com.crunii.micro.service.qzh.dto.dynamicform.DynamicFormColumnValueDTO;
import com.crunii.micro.service.qzh.dto.dynamicform.DynamicFormReqDTO;
import com.crunii.micro.service.qzh.dto.dynamicform.DynamicFormValeRspDTO;
import com.crunii.micro.service.qzh.dto.dynamicform.YcqDynamicRecordDTO;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Repository
public class YcqDynamicFormDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    @Resource
    private YcqDynamicRecordRepo formRecordRepo;

    @Resource
    private YcqDynamicFormRepo formRepo;

    @Resource
    private YcqActivityParticipationDetailsRepo ycqActivityParticipationDetailsRepo;

    private static final String PAGE_BASE_SQL = "select a.* from YCQ_DYNAMIC_RECORD a ,YCQ_DYNAMIC_FORM b  where a.IS_DELETE = 0 and b.id=a.BUSINESS_ID  and  b.BUSINESS_ID = ?";

    private static final String LIST_ACTIVE_BASE_SQL = "select  a.*,c.registration_audit_time registrationTime,c.check_in_time,c.check_out_time,(select mobile from TSM_WORKER where login_id=a.create_id) createPhone,c.check_in_time,c.check_out_time ,case when c.check_in_result==0 then '失败' else '' end checkInResultName,case when c.check_out_result==0 then '失败' else '' end checkOutResultName, case when c.registration_status=0 then '不通过' when c.registration_status=1 then '通过' when c.registration_status=1 then '待审核' else '' end status from YCQ_DYNAMIC_RECORD a left join YCQ_DYNAMIC_FORM b on  b.id=a.BUSINESS_ID left join  YCQ_ACTIVITY_PEOPLE_AUDIT c  on c.business_belong_id=a.id and c.is_deleted = 0  where a.IS_DELETE = 0    and  b.BUSINESS_ID = ?";
    private static final String PAGE_ACTIVE_BASE_SQL ="select  a.*,c.id applicationId,c.registration_status,c.registration_audit_time registrationTime,c.check_in_time,c.check_out_time, (select mobile from TSM_WORKER where login_id=a.create_id) createPhone,c.check_in_time,c.check_out_time ,case when c.check_in_result==0 then '失败' else '' end checkInResultName,case when c.check_out_result==0 then '失败' else '' end checkOutResultName, case when c.registration_status=0 then '不通过' when c.registration_status=1 then '通过' when c.registration_status=1 then '待审核' else '' end status from YCQ_DYNAMIC_RECORD a left join YCQ_DYNAMIC_FORM b on  b.id=a.BUSINESS_ID left join  YCQ_ACTIVITY_PEOPLE_AUDIT c  on c.business_belong_id=a.id and c.is_deleted = 0  where a.IS_DELETE = 0    and  b.BUSINESS_ID = ?";
    private static final String GET_VALUE_LIST_SQL = "select * from YCQ_DYNAMIC_VALUE_RECORD where IS_DELETE = 0 and  BUSINESS_ID = ? ORDER BY COLUMN_KEY ";
    private static final String PAGE_COUNT_SQL = "select COUNT(*) from YCQ_DYNAMIC_RECORD a ,YCQ_DYNAMIC_FORM b  where a.IS_DELETE = 0 and b.id=a.BUSINESS_ID  and  b.BUSINESS_ID = ?";
    private static final String PAGE_ACTIVE_COUNT_SQL = "select COUNT(*) from YCQ_DYNAMIC_RECORD a left join YCQ_DYNAMIC_FORM b on  b.id=a.BUSINESS_ID left join  YCQ_ACTIVITY_PEOPLE_AUDIT c  on c.business_belong_id=a.id and c.is_deleted = 0 where a.IS_DELETE = 0    and  b.BUSINESS_ID = ? ";
    private static final String COUNT_SQL = "select COUNT(*) from YCQ_DYNAMIC_RECORD a ,YCQ_DYNAMIC_FORM b  where a.IS_DELETE = 0 and b.id=a.BUSINESS_ID  and  b.BUSINESS_ID = ? and b.type = ? ";

    private static final String LOGIN_ID_SQL = "select b.login_id  from TSM_ROLE a ,TSM_WORKER_ROLE b,TSM_WORKER c where  a.role_name='活动审核管理员' and a.role_id=b.role_id and c.login_id=b.login_id and c.org_id = ? ";
    public PageRsp<DynamicFormValeRspDTO> getPage(PageReq<DynamicFormReqDTO> req) {
        Long id = req.getQuery().getId();

        DynamicFormReqDTO query = req.getQuery();
        query.setType(2);
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder listSb = new StringBuilder(GET_VALUE_LIST_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        params.add(id);
        if (query.getName() != null && !query.getName().equals("")) {
            sb.append(" and a.CREATE_NAME like concat('%',?,'%') ");
            sbCount.append(" and a.CREATE_NAME like concat('%',?,'%') ");
            params.add(query.getName());
        }

        if (query.getType() != null) {
            sb.append(" and b.type = ? ");
            sbCount.append(" and b.type = ?");
            params.add(query.getType());
        }

        // 时间范围查询
        if (query.getBeginTime() != null) {
            sb.append("AND a.CREATE_TIME >= ? ");
            sbCount.append("AND a.CREATE_TIME >= ? ");
            params.add(query.getBeginTime());
        }
        if (query.getEndTime() != null) {
            sb.append("AND a.CREATE_TIME <= ? ");
            sbCount.append("AND a.CREATE_TIME <= ? ");
            params.add(query.getEndTime());
        }

        // 获取总数
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        // 查询结果
        List<DynamicFormValeRspDTO> result = new ArrayList<DynamicFormValeRspDTO>();
        List<YcqDynamicRecord> list = jdbcTemplate.queryForBeans(YcqDynamicRecord.class, sb.toString(), params.toArray());
        PageRsp<DynamicFormValeRspDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        if (list != null && list.size() > 0) {
            for (YcqDynamicRecord record : list) {
                List<Object> listParams = new ArrayList<>();
                List<DynamicFormColumnValueDTO> DynamicFormColumnValues = new ArrayList<>();
                //添加id
                DynamicFormColumnValueDTO idDto=new DynamicFormColumnValueDTO();
                idDto.setColumnName("ID");
                idDto.setValue(record.getId()+"");
                idDto.setColumnDescribe("id");
                DynamicFormColumnValues.add(idDto);
                DynamicFormValeRspDTO dto=new DynamicFormValeRspDTO();
                listParams.add(record.getId());
                List<YcqQzhDynamicFormColumnValue> values = jdbcTemplate.queryForBeans(YcqQzhDynamicFormColumnValue.class, listSb.toString(), listParams.toArray());
                if (values!=null&&values.size()>0){
                    for (YcqQzhDynamicFormColumnValue value : values) {
                        DynamicFormColumnValueDTO valueDTO=new DynamicFormColumnValueDTO();
                        valueDTO.setColumnName(value.getColumnKey());
                        valueDTO.setValue((value.getColumnValue()==null||value.getColumnValue().equals(""))?"-":value.getColumnValue());
                        valueDTO.setColumnDescribe(value.getColumnDes());
                        valueDTO.setType(value.getColumnType());
                        DynamicFormColumnValues.add(valueDTO);
                    }
                    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

                    //createBy
                    DynamicFormColumnValueDTO createBy=new DynamicFormColumnValueDTO();
                    createBy.setColumnName("createBy");
                    createBy.setValue(record.getCreateName());
                    createBy.setColumnDescribe("发布者");
                    DynamicFormColumnValues.add(createBy);

                    //createTime
                    DynamicFormColumnValueDTO createTime=new DynamicFormColumnValueDTO();
                    createTime.setColumnName("createTime");
                    createTime.setValue(record.getCreateTime()==null?"":df.format(record.getCreateTime()));
                    createTime.setColumnDescribe("发布时间");
                    DynamicFormColumnValues.add(createTime);

                    //createId
                    DynamicFormColumnValueDTO createId=new DynamicFormColumnValueDTO();
                    createId.setColumnName("createId");
                    createId.setValue(record.getCreateId()+"");
                    createId.setColumnDescribe("发布者Id");
                    DynamicFormColumnValues.add(createId);

                    dto.setList(DynamicFormColumnValues);
                }

                result.add(dto);
            }
        }

        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<List<String>> getList(DynamicFormReqDTO query) {
        List<List<String>> result = new ArrayList<>();

        Long id = query.getId();
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder listSb = new StringBuilder(GET_VALUE_LIST_SQL);
        List<Object> params = new ArrayList<>();
        params.add(id);
        if (query.getName() != null && !query.getName().equals("")) {
            sb.append(" and a.CREATE_NAME like concat('%',?,'%') ");
            params.add(query.getName());
        }
        if (query.getType() != null) {
            sb.append(" and b.type = ? ");
            params.add(query.getType());
        }

        //批量选择
        if (query.getIds() != null && query.getIds().size()>0) {
            String str = query.getIds().stream().map(String::valueOf).collect(Collectors.joining(","));
            sb.append(" and a.id in ( "+str+" ) ");
        }

        // 时间范围查询
        if (query.getBeginTime() != null) {
            sb.append("AND a.CREATE_TIME >= ? ");
            params.add(query.getBeginTime());
        }
        if (query.getEndTime() != null) {
            sb.append("AND a.CREATE_TIME <= ? ");
            params.add(query.getEndTime());
        }

        // 查询结果
        List<YcqQzhDynamicFormRecord> list = jdbcTemplate.queryForBeans(YcqQzhDynamicFormRecord.class, sb.toString(), params.toArray());
        if (list != null && list.size() > 0) {
            for (YcqQzhDynamicFormRecord record : list) {
                List<String> one = new ArrayList<>();
                List<Object> listParams = new ArrayList<>();
//                one.add(record.getId()+"");
                listParams.add(record.getId());
                List<YcqQzhDynamicFormColumnValue> values = jdbcTemplate.queryForBeans(YcqQzhDynamicFormColumnValue.class, listSb.toString(), listParams.toArray());
                if (values!=null&&values.size()>0){
                    for (YcqQzhDynamicFormColumnValue value : values) {
                        if (value.getColumnType()!=null&&!value.getColumnType().equals("content")&&!value.getColumnType().equals("image")&&!value.getColumnType().equals("title"))
                        one.add((value.getColumnValue()==null||value.getColumnValue().equals(""))?"-":value.getColumnValue());
                    }
                    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    one.add(record.getCreateName());
                    one.add(record.getCreateTime()==null?"":df.format(record.getCreateTime()));

                }
                result.add(one);
            }
        }
        return result;
    }


    public List<List<String>> getApplicationList(DynamicFormReqDTO query,Integer type) {
        List<List<String>> result = new ArrayList<>();

        Long id = query.getActivityId();
        query.setType(1);
        StringBuilder sb = new StringBuilder(LIST_ACTIVE_BASE_SQL);
        StringBuilder listSb = new StringBuilder(GET_VALUE_LIST_SQL);
        List<Object> params = new ArrayList<>();
        params.add(id);
        if (query.getName() != null && !query.getName().equals("")) {
            sb.append(" and a.CREATE_NAME like concat('%',?,'%') ");
            params.add(query.getName());
        }
        if (query.getType() != null) {
            sb.append(" and b.type = ? ");
            params.add(query.getType());
        }

        if (query.getAuditStatus() != null&&!query.getAuditStatus().equals("")) {
            sb.append(" and c.registration_status = ? ");
            params.add(query.getAuditStatus());
        }

        //批量选择
        if (query.getIds() != null && query.getIds().size()>0) {
            String str = query.getIds().stream().map(String::valueOf).collect(Collectors.joining(","));
            sb.append(" and a.id in ( "+str+" ) ");
        }
//姓名和联系方式的模糊查询条件
        if (query.getUserName() != null) {
            sb.append("AND c.user_name LIKE ? ");
            params.add("%" + query.getUserName() + "%");
        }

        if (query.getContactMethod() != null) {
            sb.append("AND c.contact_method LIKE ? ");
            params.add("%" + query.getContactMethod() + "%");
        }

        // 报名状态、签到结果、签退结果固定查询条件
        if (query.getRegistrationStatus() != null) {
            sb.append("AND c.registration_status = ? ");
            params.add(query.getRegistrationStatus());
        }

        if (query.getCheckInResult() != null) {
            sb.append("AND c.check_in_result = ? ");
            params.add(query.getCheckInResult());
        }

        if (query.getCheckOutResult() != null) {
            sb.append("AND c.check_out_result = ? ");
            params.add(query.getCheckOutResult());
        }

        // 时间范围查询
        if (query.getStartRegistrationTime() != null && query.getEndRegistrationTime() != null) {
            sb.append("AND c.registration_time BETWEEN ? AND ? ");
            params.add(query.getStartRegistrationTime());
            params.add(query.getEndRegistrationTime());
        }

        if (query.getStartCheckInTime() != null && query.getEndCheckInTime() != null) {
            sb.append("AND c.check_in_time BETWEEN ? AND ? ");
            params.add(query.getStartCheckInTime());
            params.add(query.getEndCheckInTime());
        }

        if (query.getStartCheckOutTime() != null && query.getEndCheckOutTime() != null) {
            sb.append("AND c.check_out_time BETWEEN ? AND ? ");
            params.add(query.getStartCheckOutTime());
            params.add(query.getEndCheckOutTime());
        }
        //报名状态排序
        sb.append(" ORDER BY c.registration_status DESC ");


        // 查询结果
        List<YcqDynamicRecordDTO> list = jdbcTemplate.queryForBeans(YcqDynamicRecordDTO.class, sb.toString(), params.toArray());
        if (list != null && list.size() > 0) {
            for (YcqDynamicRecordDTO record : list) {
                List<String> one = new ArrayList<>();
                List<Object> listParams = new ArrayList<>();
                listParams.add(record.getId());
                List<YcqQzhDynamicFormColumnValue> values = jdbcTemplate.queryForBeans(YcqQzhDynamicFormColumnValue.class, listSb.toString(), listParams.toArray());
                if (values!=null&&values.size()>0){
                    for (YcqQzhDynamicFormColumnValue value : values) {
                        if (value.getColumnType()!=null&&!value.getColumnType().equals("content")&&!value.getColumnType().equals("image")&&!value.getColumnType().equals("title"))
                            one.add((value.getColumnValue()==null||value.getColumnValue().equals(""))?"-":value.getColumnValue());
                    }

                    if (type==1){
                        one.add(record.getStatus());
                    }else {
                        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        one.add(record.getCreateName());
                        one.add(record.getCreatePhone());
                        one.add(record.getRegistrationTime());
                        one.add(record.getStatus());
                        one.add(record.getCheckInTime()==null?"":df.format(record.getCheckInTime()));
                        one.add(record.getCheckInResultName());
                        one.add(record.getCheckOutTime()==null?"":df.format(record.getCheckOutTime()));
                        one.add(record.getCheckOutResultName());
                    }

                }

                result.add(one);
            }
        }
        return result;
    }
    public PageRsp<DynamicFormValeRspDTO> getApplicationPage(PageReq<DynamicFormReqDTO> req) {

        Long id = req.getQuery().getActivityId();


        DynamicFormReqDTO query = req.getQuery();
        query.setType(1);
        StringBuilder sb = new StringBuilder(PAGE_ACTIVE_BASE_SQL);
        StringBuilder listSb = new StringBuilder(GET_VALUE_LIST_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_ACTIVE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        params.add(id);
        if (query.getName() != null && !query.getName().equals("")) {
            sb.append(" and a.CREATE_NAME like concat('%',?,'%') ");
            sbCount.append(" and a.CREATE_NAME like concat('%',?,'%') ");
            params.add(query.getName());
        }
        if (query.getType() != null) {
            sb.append(" and b.type = ? ");
            sbCount.append(" and b.type = ?");
            params.add(query.getType());
        }
        if (query.getAuditStatus() != null&&!query.getAuditStatus().equals("")) {
            sb.append(" and c.registration_status = ? ");
            sbCount.append(" and c.registration_status = ? ");
            params.add(query.getAuditStatus());
        }

        //姓名和联系方式的模糊查询条件
        if (query.getUserName() != null) {
            sb.append("AND c.user_name LIKE ? ");
            sbCount.append("AND c.user_name LIKE ? ");
            params.add("%" + query.getUserName() + "%");
        }

        if (query.getContactMethod() != null) {
            sb.append("AND c.contact_method LIKE ? ");
            sbCount.append("AND c.contact_method LIKE ? ");
            params.add("%" + query.getContactMethod() + "%");
        }

        // 报名状态、签到结果、签退结果固定查询条件
        if (query.getRegistrationStatus() != null) {
            sb.append("AND c.registration_status = ? ");
            sbCount.append("AND c.registration_status = ? ");
            params.add(query.getRegistrationStatus());
        }

        if (query.getCheckInResult() != null) {
            sb.append("AND c.check_in_result = ? ");
            sbCount.append("AND c.check_in_result = ? ");
            params.add(query.getCheckInResult());
        }

        if (query.getCheckOutResult() != null) {
            sb.append("AND c.check_out_result = ? ");
            sbCount.append("AND c.check_out_result = ? ");
            params.add(query.getCheckOutResult());
        }

        // 时间范围查询
        if (query.getStartRegistrationTime() != null && query.getEndRegistrationTime() != null) {
            sb.append("AND c.registration_time BETWEEN ? AND ? ");
            sbCount.append("AND c.registration_time BETWEEN ? AND ? ");
            params.add(query.getStartRegistrationTime());
            params.add(query.getEndRegistrationTime());
        }

        if (query.getStartCheckInTime() != null && query.getEndCheckInTime() != null) {
            sb.append("AND c.check_in_time BETWEEN ? AND ? ");
            sbCount.append("AND c.check_in_time BETWEEN ? AND ? ");
            params.add(query.getStartCheckInTime());
            params.add(query.getEndCheckInTime());
        }

        if (query.getStartCheckOutTime() != null && query.getEndCheckOutTime() != null) {
            sb.append("AND c.check_out_time BETWEEN ? AND ? ");
            sbCount.append("AND c.check_out_time BETWEEN ? AND ? ");
            params.add(query.getStartCheckOutTime());
            params.add(query.getEndCheckOutTime());
        }
        //报名状态排序
        sb.append(" ORDER BY c.registration_status DESC ");




        // 获取总数
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        // 查询结果
        List<DynamicFormValeRspDTO> result = new ArrayList<DynamicFormValeRspDTO>();
        List<YcqDynamicRecordDTO> list = jdbcTemplate.queryForBeans(YcqDynamicRecordDTO.class, sb.toString(), params.toArray());
        PageRsp<DynamicFormValeRspDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        if (list != null && list.size() > 0) {
            for (YcqDynamicRecordDTO record : list) {
                List<Object> listParams = new ArrayList<>();
                List<DynamicFormColumnValueDTO> DynamicFormColumnValues = new ArrayList<>();

                DynamicFormValeRspDTO dto=new DynamicFormValeRspDTO();
                listParams.add(record.getId());
                List<YcqQzhDynamicFormColumnValue> values = jdbcTemplate.queryForBeans(YcqQzhDynamicFormColumnValue.class, listSb.toString(), listParams.toArray());
                if (values!=null&&values.size()>0){
                    for (YcqQzhDynamicFormColumnValue value : values) {
                        DynamicFormColumnValueDTO valueDTO=new DynamicFormColumnValueDTO();
                        valueDTO.setColumnName(value.getColumnKey());
                        valueDTO.setValue((value.getColumnValue()==null||value.getColumnValue().equals(""))?"-":value.getColumnValue());
                        valueDTO.setColumnDescribe(value.getColumnDes());
                        valueDTO.setType(value.getColumnType());
                        DynamicFormColumnValues.add(valueDTO);
                    }
                    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//
//                    //createBy
//                    DynamicFormColumnValueDTO createBy=new DynamicFormColumnValueDTO();
//                    createBy.setColumnName("createBy");
//                    createBy.setValue(record.getCreateName());
//                    createBy.setColumnDescribe("发布者");
//                    DynamicFormColumnValues.add(createBy);
//
//                    //createTime
//                    DynamicFormColumnValueDTO createTime=new DynamicFormColumnValueDTO();
//                    createTime.setColumnName("createTime");
//                    createTime.setValue(record.getCreateTime()==null?"":df.format(record.getCreateTime()));
//                    createTime.setColumnDescribe("发布时间");
//                    DynamicFormColumnValues.add(createTime);
//
                    //createId
                    DynamicFormColumnValueDTO createId=new DynamicFormColumnValueDTO();
                    createId.setColumnName("createId");
                    createId.setValue(record.getCreateId()+"");
                    createId.setColumnDescribe("发布者Id");
                    DynamicFormColumnValues.add(createId);

                    //createName
                    DynamicFormColumnValueDTO createName=new DynamicFormColumnValueDTO();
                    createName.setColumnName("submitName");
                    createName.setValue(record.getCreateName());
                    createName.setColumnDescribe("提交人姓名");
                    DynamicFormColumnValues.add(createName);

                    //createPhone
                    DynamicFormColumnValueDTO createPhone=new DynamicFormColumnValueDTO();
                    createPhone.setColumnName("submitPhone");
                    createPhone.setValue(record.getCreatePhone());
                    createPhone.setColumnDescribe("提交人电话");
                    DynamicFormColumnValues.add(createPhone);

                     //status
                    DynamicFormColumnValueDTO status=new DynamicFormColumnValueDTO();
                    status.setColumnName("status");
                    status.setValue(record.getStatus());
                    status.setColumnDescribe("报名状态");
                    DynamicFormColumnValues.add(status);

                    //添加id
                    DynamicFormColumnValueDTO idDto=new DynamicFormColumnValueDTO();
                    idDto.setColumnName("ID");
                    idDto.setValue(record.getApplicationId()+"");
                    idDto.setColumnDescribe("id");
                    DynamicFormColumnValues.add(idDto);

                    //status
                    DynamicFormColumnValueDTO registrationStatus=new DynamicFormColumnValueDTO();
                    registrationStatus.setColumnName("registrationStatus");
                    registrationStatus.setValue(record.getRegistrationStatus());
                    registrationStatus.setColumnDescribe("报名TYPE");
                    DynamicFormColumnValues.add(registrationStatus);


                    //registrationTime
                    DynamicFormColumnValueDTO registrationTime=new DynamicFormColumnValueDTO();
                    registrationTime.setColumnName("submitTime");
                    registrationTime.setValue(record.getRegistrationTime());
                    registrationTime.setColumnDescribe("报名时间");
                    DynamicFormColumnValues.add(registrationTime);

                    //checkInTime
                    DynamicFormColumnValueDTO checkInTime=new DynamicFormColumnValueDTO();
                    checkInTime.setColumnName("signTime");
                    checkInTime.setValue(record.getCheckInTime()==null?"":df.format(record.getCheckInTime()));
                    checkInTime.setColumnDescribe("签到时间");
                    DynamicFormColumnValues.add(checkInTime);

                    //checkInResultName
                    DynamicFormColumnValueDTO checkInResultName=new DynamicFormColumnValueDTO();
                    checkInResultName.setColumnName("signResult");
                    checkInResultName.setValue(record.getCheckInResultName());
                    checkInResultName.setColumnDescribe("签到结果");
                    DynamicFormColumnValues.add(checkInResultName);

                    //checkOutTime
                    DynamicFormColumnValueDTO checkOutTime=new DynamicFormColumnValueDTO();
                    checkOutTime.setColumnName("signOutTime");
                    checkOutTime.setValue(record.getCheckInTime()==null?"":df.format(record.getCheckInTime()));
                    checkOutTime.setColumnDescribe("签退时间");
                    DynamicFormColumnValues.add(checkOutTime);

                    //checkInResultName
                    DynamicFormColumnValueDTO checkOutResultName=new DynamicFormColumnValueDTO();
                    checkOutResultName.setColumnName("signOutResult");
                    checkOutResultName.setValue(record.getCheckOutResultName());
                    checkOutResultName.setColumnDescribe("签退结果");
                    DynamicFormColumnValues.add(checkOutResultName);

                    //havaRecord
                    DynamicFormColumnValueDTO displayButton=new DynamicFormColumnValueDTO();
                    Integer num=0;
                    YcqDynamicForm byBusinessIdAndType = formRepo.getByBusinessIdAndType(id, 2);
                    if (byBusinessIdAndType!=null){
                        List<YcqDynamicRecord> records = formRecordRepo.findByBusinessIdAndCreateId(byBusinessIdAndType.getId() + "", record.getCreateId());
                        if (records!=null&&records.size()>0){
                            num=1;
                        }
                    }else {
                        YcqActivityParticipationDetails byActivityIdAndUserId = ycqActivityParticipationDetailsRepo.findByActivityIdAndUserId(id, (long) record.getCreateId());
                        if (byActivityIdAndUserId!=null){
                            num=1;
                        }
                    }

                    displayButton.setColumnName("displayButton");
                    displayButton.setValue(num+"");
                    displayButton.setColumnDescribe("是否填写");
                    DynamicFormColumnValues.add(displayButton);

                    dto.setList(DynamicFormColumnValues);
                }

                result.add(dto);
            }
        }

        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public Integer getApplicationPage(Integer activeId) {
        // 获取总数
        StringBuilder sbCount = new StringBuilder(COUNT_SQL);
        List<Object> params = new ArrayList<>();
        params.add(activeId);
        params.add(2);
        return jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
    }

    public List<Worker> getLoginIdS(Long id) {
        // 获取总数
        List<Object> params = new ArrayList<>();
        params.add(id);
        StringBuilder sb = new StringBuilder(LOGIN_ID_SQL);
        return jdbcTemplate.queryForBeans(Worker.class,sb.toString(), params.toArray());
    }

    public List<YcqDynamicForm> getFormByActiveIdS(List<Long> ids) {
        String str="";
        for (Long id : ids) {
            str+=id+",";
        }
        str = str.substring(0, str.length() - 1);
        // 获取总数
        StringBuilder sb = new StringBuilder("SELECT * FROM YCQ_DYNAMIC_FORM WHERE BUSINESS_ID in ( ");
        sb.append(str);
        sb.append(" ) and TYPE=1");
        return jdbcTemplate.queryForBeans(YcqDynamicForm.class,sb.toString());
    }

    public Integer getMaxTime(Long activeId) {
        StringBuilder sb = new StringBuilder("SELECT max(audit_times) FROM YCQ_ACTIVE_FLOW_NODE WHERE biz_id = ? ");
        List<Object> params = new ArrayList<>();
        params.add(activeId);
        return jdbcTemplate.queryForInt(sb.toString(), params.toArray());
    }

}
