package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.document.YcqPromiseYouth;
import com.crunii.micro.service.qzh.api.domain.dto.document.PageEventReportReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.document.PageEventReportRespDTO;
import com.crunii.micro.service.qzh.api.domain.dto.document.YcqEventReportDTO;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @program: microservice
 * @description: 活动报道Jdbc
 * @author: xiang xin
 * @create: 2024-01-15 17:59
 */
@Repository
public class YcqEventReportDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String PAGE_EVENT_REPORT_SQL = " select \n" +
            "a.id,\n" +
            "a.report_title,\n" +
            "a.report_type,\n" +
            "a.report_text,\n" +
            "a.activity_id,\n" +
            "a.CREATED_TIME,\n" +
            "a.CREATED_BY createdId,\n" +
            "b.worker_name as createdName,\n" +
            "c.org_id createdOrgId,\n" +
            "c.org_name createdOrgName,\n" +
            "d.ACTIVITY_TITLE activityName,\n" +
            "case \n" +
            "when a.report_type = 1 then '未发布'\n" +
            "when a.report_type = 2 then '已发布'\n" +
            "when a.report_type = 3 then '已下架'\n" +
            "else null end as reportTypeName\n" +
            "from YCQ_EVENT_REPORT a\n" +
            "left join TSM_WORKER b on a.CREATED_BY=b.login_id\n" +
            "left join TSM_ORG c on b.org_id=c.org_id\n" +
            "left join YCQ_PUBLISH_ACTIVITY d on a.activity_id = d.id\n" +
            "inner join tsm_worker_org_all twoa on c.org_id=twoa.org_id where 1=1 and del_flag = 0 ";
    private static final String PAGE_EVENT_REPORT_COUNT = " select \n" +
            "count(1)\n" +
            "from \"YOUTH_CQ\".\"YCQ_EVENT_REPORT\" a\n" +
            "left join TSM_WORKER b on a.CREATED_BY=b.login_id\n" +
            "left join TSM_ORG c on b.org_id=c.org_id\n" +
            "left join YCQ_PUBLISH_ACTIVITY d on a.activity_id = d.id\n" +
            "inner join tsm_worker_org_all twoa on c.org_id=twoa.org_id where 1=1 and del_flag = 0 ";

    private static final String PAGE_EVENT_REPORT_SQL_APP = " select \n" +
            "a.id,\n" +
            "a.report_title,\n" +
            "a.report_type,\n" +
            "a.report_text,\n" +
            "a.activity_id,\n" +
            "a.CREATED_TIME,\n" +
            "a.CREATED_BY createdId,\n" +
            "b.worker_name as createdName,\n" +
            "c.org_id createdOrgId,\n" +
            "c.org_name createdOrgName,\n" +
            "d.ACTIVITY_TITLE activityName,\n" +
            "case \n" +
            "when a.report_type = 1 then '未发布'\n" +
            "when a.report_type = 2 then '已发布'\n" +
            "when a.report_type = 3 then '已下架'\n" +
            "else null end as reportTypeName\n" +
            "from YCQ_EVENT_REPORT a\n" +
            "left join TSM_WORKER b on a.CREATED_BY=b.login_id\n" +
            "left join TSM_ORG c on b.org_id=c.org_id\n" +
            "left join YCQ_PUBLISH_ACTIVITY d on a.activity_id = d.id where 1=1 and del_flag = 0 ";
    private static final String PAGE_EVENT_REPORT_COUNT_APP = " select \n" +
            "count(1)\n" +
            "from \"YOUTH_CQ\".\"YCQ_EVENT_REPORT\" a\n" +
            "left join TSM_WORKER b on a.CREATED_BY=b.login_id\n" +
            "left join YCQ_PUBLISH_ACTIVITY d on a.activity_id = d.id\n" +
            "left join TSM_ORG c on b.org_id=c.org_id where 1=1 and del_flag = 0 ";

    public PageRsp<PageEventReportRespDTO> pageEventReport(PageReq<PageEventReportReqDTO> req, Worker worker) {
        StringBuilder sb = new StringBuilder(PAGE_EVENT_REPORT_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_EVENT_REPORT_COUNT);
        List<Object> params = new ArrayList<>();
        PageEventReportReqDTO query = req.getQuery();
        if (ObjectUtil.isNotEmpty(query)) {
            //鉴权
            sb.append(" and twoa.worker_id= ? ");
            sbCount.append(" and twoa.worker_id= ? ");
            params.add(worker.getLoginId());

            if (StrUtil.isNotBlank(query.getReportTitle())) {
                sb.append(" and a.report_title like concat('%',?,'%') ");
                sbCount.append(" and a.report_title like concat('%',?,'%') ");
                params.add(query.getReportTitle());
            }
            if (Objects.nonNull(req.getQuery().getReportType())) {
                sb.append(" and a.report_type = ? ");
                sbCount.append(" and a.report_type = ? ");
                params.add(query.getReportType());
            }
            //有效开始时间
            if (StrUtil.isNotBlank(query.getCreatedTimeStart())) {
                sb.append(" and a.CREATED_TIME>=? ");
                sbCount.append(" and a.CREATED_TIME>=? ");
                params.add(query.getCreatedTimeStart());
            }
            //有效结束时间
            if (StrUtil.isNotBlank(query.getCreatedTimeEnd())) {
                sb.append(" and a.CREATED_TIME<=? ");
                sbCount.append(" and a.CREATED_TIME<=? ");
                params.add(query.getCreatedTimeEnd());
            }
            sb.append(" order by a.CREATED_TIME desc");
        }
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<PageEventReportRespDTO> result;
        String dbType = jdbcTemplate.getDbType();

        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(PageEventReportRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<PageEventReportRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public PageEventReportRespDTO doEventReport(YcqEventReportDTO dto, Worker worker) {
        StringBuilder sb = new StringBuilder(PAGE_EVENT_REPORT_SQL_APP);
        if (null != dto.getId()) {
            sb.append(" and a.id = " + dto.getId());
        }
        List<PageEventReportRespDTO> pageEventReportRespDTOS = jdbcTemplate.queryForBeans(PageEventReportRespDTO.class, sb.toString());
        if (pageEventReportRespDTOS.size() > 0) {
            return pageEventReportRespDTOS.get(0);
        }
        return null;
    }

    public PageRsp<PageEventReportRespDTO> pageEventReportApp(PageReq<PageEventReportReqDTO> req, Worker worker) {
        StringBuilder sb = new StringBuilder(PAGE_EVENT_REPORT_SQL_APP);
        StringBuilder sbCount = new StringBuilder(PAGE_EVENT_REPORT_COUNT_APP);
        List<Object> params = new ArrayList<>();
        PageEventReportReqDTO query = req.getQuery();
        if (ObjectUtil.isNotEmpty(query)) {
            //已发布
            sb.append(" and a.report_type = 2 ");
            sbCount.append(" and a.report_type = 2 ");

            if (StrUtil.isNotBlank(query.getActivityId())) {
                sb.append(" and a.activity_id = ? ");
                sbCount.append(" and a.activity_id = ? ");
                params.add(query.getActivityId());
            }
            if (StrUtil.isNotBlank(query.getReportTitle())) {
                sb.append(" and a.report_title like concat('%',?,'%') ");
                sbCount.append(" and a.report_title like concat('%',?,'%') ");
                params.add(query.getReportTitle());
            }
            if (Objects.nonNull(req.getQuery().getReportType())) {
                sb.append(" and a.report_type = ? ");
                sbCount.append(" and a.report_type = ? ");
                params.add(query.getReportType());
            }
            //有效开始时间
            if (StrUtil.isNotBlank(query.getCreatedTimeStart())) {
                sb.append(" and a.CREATED_TIME>=? ");
                sbCount.append(" and a.CREATED_TIME>=? ");
                params.add(query.getCreatedTimeStart());
            }
            //有效结束时间
            if (StrUtil.isNotBlank(query.getCreatedTimeEnd())) {
                sb.append(" and a.CREATED_TIME<=? ");
                sbCount.append(" and a.CREATED_TIME<=? ");
                params.add(query.getCreatedTimeEnd());
            }
            sb.append(" order by a.CREATED_TIME desc");
        }
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<PageEventReportRespDTO> result;
        String dbType = jdbcTemplate.getDbType();

        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(PageEventReportRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<PageEventReportRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public List<YcqPromiseYouth> promiseYouth(Long loginId) {
        String SQL = "select * from YCQ_LM_USER where del_flag = 0 and status = 1 and auth_status = 2 and user_id = " + loginId;
        return jdbcTemplate.queryForBeans(YcqPromiseYouth.class, SQL);
    }

    public void delectAll(Long id) {
        //删除附件
        String sql1 = "delete from YCQ_EVENT_REPORT_ANNEX where type=1 and article_id=" + id;
        jdbcTemplate.execute(sql1);
        //删除封面
        String sql = "delete from YCQ_EVENT_REPORT_COVER where type=1 and article_id=" + id;
        jdbcTemplate.execute(sql);
    }
}
