package com.crunii.micro.service.qzh.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.entity.main.YcqAnswerRecord;
import com.crunii.micro.service.qzh.dto.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author lyx
 * @date 2023/9/5
 */
@Slf4j
@Repository
public class YcqExamJdbcDao {
	@Resource(name = "mainJT")
	protected BaseJdbcTemplate JT;

	private static final String examRecordSql = "select *from YCQ_EXAM_RECORD where detele = 1 ";

	public List<AppExamListRespDTO> getExamList() {
		String sql  = "select a.id,a.title examTitle,a.effective_time examTime,a.total_score exam_total_score, a.topic_num topicNum,a.STATE,"
				+ "(select count(*)from YCQ_EXAM_RECORD b where b.EXAM_ID = a.id) completeNum,c.FILE_ID examIcon "
				+ "from YCQ_YZYH_TEST a LEFT JOIN YCQ_YZYH_TEST_FILE c "
				+ "ON c.TEST_ID=a.id";
		return JT.query(sql, new BeanPropertyRowMapper<>(AppExamListRespDTO.class));
	}

	public YcqAnswerRecord getExamCountByWorkerAndId(Integer id,int userId) {
		StringBuilder sb = new StringBuilder(examRecordSql);
		sb.append("and id = ? ");
		sb.append("and userId = ? ");
		return JT.queryForBean(YcqAnswerRecord.class,sb.toString(),id,userId);
	}

	public AppExamInfoRespDTO getExamInfoById(Integer id) {
		String sql = "select a.id,a.title examName,a.effective_time examTime,a.TOPIC_NUM,a.total_score,a.PASS_SCORE, "
				+ "a.total_times residueTimes,  0 EXAM_TIMES,a.START_TIME examStartTime,a.end_time examEndTime,a.EXAM_OBJECT examTarget"
				+ " from YCQ_YZYH_TEST a where a.id = ? and deteled = 1";
		return JT.queryForBean(AppExamInfoRespDTO.class,sql,id);
	}

	public AppExamContentRespDTO getExamById(Integer id) {
		String sqlStr = "select a.id,a.title examName,a.EFFECTIVE_TIME examResidueTime from YCQ_YZYH_TEST  a where a.DELETED = 1 and id = ?";
		return JT.queryForBean(AppExamContentRespDTO.class,sqlStr,id);
	}

	public List<ExamTopicRespDTO> getTopicByTestId(Integer id) {
		String sql = "select a.test_id,a.topic_id,b.id,b.topic,case when b.topic_type = 1 then '单选' "
				+ "when b.topic_type = 2 then '多选'  when b.topic_type = 3 then '判断' end topic_type "
				+ "from YCQ_YZYH_TOPIC_REL a left join YCQ_TOPIC b on a.TOPIC_ID = b.ID where b.DELETED = 1 and a.TEST_ID = ? order by b.topic_type";
		return JT.queryForBeans(ExamTopicRespDTO.class,sql,id);
	}

	public List<ExamOptionRespDTO> getOptionByTopicId(List<Integer> topicsIds) {
		String sqlStr = "select a.id,a.OPTION,a.option_content,a,topic_id from YCQ_TOPIC_OPTION a where topic_id in (";
		StringBuilder sql = new StringBuilder(sqlStr);
		List<String> param = new ArrayList<>();
		for (int i = 0; i < topicsIds.size(); i++) {
			if (i == topicsIds.size() - 1) {
				sql.append(" ? )");
			} else {
				sql.append(" ?, ");
			}
			param.add(topicsIds.get(i).toString());
		}

		sql.append(" order by a.OPTION");
		return JT.queryForBeans(ExamOptionRespDTO.class,sql.toString(),param.toArray());
	}

	public List<ExamTopicRespDTO> getTopicByTestIdAndAnswer(Integer id) {
		String sql = "select a.test_id,a.topic_id,b.id,b.topic, b.topic_type "
				+ "from YCQ_YZYH_TOPIC_REL a left join YCQ_TOPIC b on a.TOPIC_ID = b.ID "
				+ "where b.DELETED = 1 and a.TEST_ID = ? order by b.topic_type";
		return JT.queryForBeans(ExamTopicRespDTO.class,sql,id);
	}
}
