package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhIndicatorSystemVo;
import com.crunii.micro.service.qzh.api.domain.dto.activity.OperationActivityReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.PageActivityReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.PageActivityRspDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.ParentDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activityPublish.ShowListQueryReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activityPublish.ShowListQueryRespDTO;
import com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 风采jdbc
 * @Param:
 * @return:
 * @Author: xiang xin
 * @Date: 2023/9/18
 */
@Repository
public class YcqQzhActivityJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;
    private static final String PAGE_Activity_BASE_SQL = " select a.id,a.activity_title,a.create_id,a.activity_text,a.activity_type,a.open_competence, \n" +
            "case when a.activity_type=1 then '青年突击队风采' when a.activity_type=2 then '青年文明号风采' \n" +
            "when a.activity_type=3 then '青年安全生产示范岗风采' when a.activity_type=4 then '维护青少年权益岗风采' else tpv.value_name end as activityTypeName, \n" +
            "case when a.open_competence=1 then '全部' when a.open_competence=2 then '对内' when a.open_competence=3 then '对外' else null end as openCompetenceName, \n" +
            "b.worker_name as createUserName,a.create_time as createTime,[AD]\n" +
            "(case when a.sort_num>0 then true else false end) as whetherTop, \n" +
            "case when (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=3) is not null \n" +
            "then (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=3)  else 0 end  as viewTotal \n" +
            "from ycq_qzh_activity a \n" +
            "left join tsm_property_value tpv on a.activity_type = tpv.value_code and tpv.prop_code = 'activityOtherType' " +
            "left join tsm_worker b on a.create_id=b.login_id [CD]\n" +
            "where a.del_flag=0 ";
    private static final String PAGE_Activity_COUNT_SQL = "select count(*) from ycq_qzh_activity a " +
            " left join tsm_property_value tpv on a.activity_type = tpv.value_code and tpv.prop_code = 'activityOtherType' " +
            " left join tsm_worker b on a.create_id=b.login_id [CD] where a.del_flag=0 ";

    private static final String DELETE_SQL = "update ycq_qzh_activity set del_flag=1, update_id=?, update_by=?, update_time=? where id in";

    public PageRsp<PageActivityRspDTO> pageActivity(PageReq<PageActivityReqDTO> req, List<Long> listIds, Worker worker) {
        StringBuilder sb = new StringBuilder(PAGE_Activity_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_Activity_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        PageActivityReqDTO query = req.getQuery();
        if (ObjectUtil.isNotEmpty(query)) {
            //标题
            if (StrUtil.isNotBlank(query.getActivityTitle())) {
                sb.append(" and a.activity_title like concat('%',?,'%') ");
                sbCount.append(" and a.activity_title like concat('%',?,'%') ");
                params.add(query.getActivityTitle());
            }
            //如果condition为app则只查询我自己的风采
            if (StrUtil.isNotBlank(query.getCondition()) && "myApp".equals(query.getCondition())) {
                sb.append(" and a.create_id = ? ");
                sbCount.append(" and a.create_id = ? ");
                params.add(worker.getLoginId());
            }
            //状态
//            if (Objects.nonNull(req.getQuery().getState())) {
//                sb.append(" and a.state = ? ");
//                sbCount.append(" and a.state = ? ");
//                params.add(query.getState());
//            }
            //类型
            if (Objects.nonNull(query.getActivityType())) {
                if (5 == query.getActivityType()) {
                    sb.append(" and a.activity_type not in (1,2,3,4) ");
                    sbCount.append(" and a.activity_type not in (1,2,3,4)  ");
                } else {
                    sb.append(" and a.activity_type = ? ");
                    sbCount.append(" and a.activity_type = ? ");
                    params.add(query.getActivityType());
                }
//                sb.append(" and a.activity_type = ? ");
//                sbCount.append(" and a.activity_type = ? ");
//                params.add(query.getActivityType());
                String replace = null;
                String replace1 = null;
                String replace2 = null;
                if (1 == query.getActivityType()) {
                    //关联突击队
                    replace = sb.toString().replace("[CD]", "left join YCQ_QZH_COMMANDO y1 on a.PARENT_ID =y1.id and y1.del_flag='0' ");
                    replace1 = replace.replace("[AD]", " y1.COMMANDO_NAME as parentName, y1.id as parentId, ");
                    replace2 = sbCount.toString().replace("[CD]", "left join YCQ_QZH_COMMANDO y1 on a.PARENT_ID =y1.id and y1.del_flag='0' ");
                    sb = new StringBuilder(replace1);
                    sbCount = new StringBuilder(replace2);
                } else if (2 == query.getActivityType()) {
                    //关联文明号  文明号逻辑需要修改
                    replace = sb.toString().replace("[CD]", "left join ycq_yn_declaration y2 on a.parent_id = y2.declaration_id and y2.deleted = 0 ");
                    replace1 = replace.replace("[AD]", " y2.collective_name as parentName, y2.declaration_id as parentId, ");
                    replace2 = sbCount.toString().replace("[CD]", "left join ycq_yn_declaration y2 on a.parent_id = y2.declaration_id and y2.deleted = 0 ");
                    sb = new StringBuilder(replace1);
                    sbCount = new StringBuilder(replace2);
                } else if (3 == query.getActivityType()) {
                    //关联示范岗
                    replace = sb.toString().replace("[CD]", "left join ycq_qzh_qag_apply y3 on a.parent_id =y3.id and y3.del_flag = 0  ");
                    replace1 = replace.replace("[AD]", " y3.apply_org_name as parentName, y3.id as parentId, ");
                    replace2 = sbCount.toString().replace("[CD]", "left join ycq_qzh_qag_apply y3 on a.parent_id =y3.id and y3.del_flag = 0  ");
                    sb = new StringBuilder(replace1);
                    sbCount = new StringBuilder(replace2);
                } else if (4 == query.getActivityType()) {
                    //关联权益岗
                    replace = sb.toString().replace("[CD]", "left join YCQ_QZH_RIGHTS_PROTECTION y4 on a.PARENT_ID =y4.id and y4.del_flag='0' ");
                    replace1 = replace.replace("[AD]", " y4.unit_name as parentName, y4.id as parentId, ");
                    replace2 = sbCount.toString().replace("[CD]", "left join YCQ_QZH_RIGHTS_PROTECTION y4 on a.PARENT_ID =y4.id and y4.del_flag='0' ");
                    sb = new StringBuilder(replace1);
                    sbCount = new StringBuilder(replace2);
                } else {
                    //其他风采 未关联其他表
                    replace = sb.toString().replace("[CD]", "");
                    replace1 = replace.replace("[AD]", "");
                    replace2 = sbCount.toString().replace("[CD]", "");
                    sb = new StringBuilder(replace1);
                    sbCount = new StringBuilder(replace2);
                }
            } else {
                String replace = sb.toString().replace("[CD]", "");
                String replace1 = replace.replace("[AD]", "");
                String replace2 = sbCount.toString().replace("[CD]", "");
                sb = new StringBuilder(replace1);
                sbCount = new StringBuilder(replace2);
            }
//            if ("app".equals(query.getCondition()) && null != listIds) {
//                String collect = listIds.stream().map(String::valueOf).collect(Collectors.joining(",", "(", ") "));
//                sb.append(" and (a.open_competence = 1 or (a.id in " + collect + " and a.open_competence =2 )) ");
//                sbCount.append(" and (a.open_competence = 1 or (a.id in " + collect + " and a.open_competence =2 )) ");
//            } else if ("app".equals(query.getCondition()) && null == listIds) {
//                sb.append(" and a.open_competence = 1 ");
//                sbCount.append(" and a.open_competence = 1 ");
//            } else if ("web".equals(query.getCondition()) && null != orgId) {
//                if (1 == query.getOpenCompetence()) {
//                    sb.append(" and (a.open_competence = 1 or (find_in_set(?,create_org_ids) and a.open_competence =2 )) ");
//                    sbCount.append(" and (a.open_competence = 1 or (find_in_set(?,create_org_ids) and a.open_competence =2 )) ");
//                    params.add(orgId);
//                } else if (2 == query.getOpenCompetence()) {
//                    sb.append(" and find_in_set(?,create_org_ids) and a.open_competence =2  ");
//                    sbCount.append(" and find_in_set(?,create_org_ids) and a.open_competence =2 ");
//                    params.add(orgId);
//                }
//            } else if ("web".equals(query.getCondition()) && null == orgId) {
            if (Objects.nonNull(query.getOpenCompetence())) {
                sb.append(" and a.open_competence = ? ");
                sbCount.append(" and a.open_competence = ? ");
                params.add(query.getOpenCompetence());
            }
//            } else {
//                new BusinessException("系统内部异常，请联系管理员!");
//            }
            //创建者
            if (Objects.nonNull(query.getCreateUserName())) {
                sb.append(" and b.worker_name like concat('%',?,'%') ");
                sbCount.append(" and b.worker_name like concat('%',?,'%') ");
                params.add(query.getCreateUserName());
            }
            //所属对象
            if (Objects.nonNull(query.getParentName())) {
                if (1 == query.getActivityType()) {
                    sb.append(" and y1.COMMANDO_NAME like concat('%',?,'%') ");
                    sbCount.append(" and y1.COMMANDO_NAME like concat('%',?,'%') ");
                } else if (2 == query.getActivityType()) {
                    sb.append(" and y2.id like concat('%',?,'%') ");
                    sbCount.append(" and y2.id like concat('%',?,'%') ");
                } else if (3 == query.getActivityType()) {
                    sb.append(" and y3.APPLY_ORG_NAME like concat('%',?,'%') ");
                    sbCount.append(" and y3.APPLY_ORG_NAME like concat('%',?,'%') ");
                } else if (4 == query.getActivityType()) {
                    sb.append(" and y4.unit like concat('%',?,'%') ");
                    sbCount.append(" and y4.unit like concat('%',?,'%') ");
                }
                params.add(query.getParentName());
            }
            // 开始日期
            if (Objects.nonNull(query.getBeginTime())) {
                sb.append(" and  a.CREATE_TIME >= ? ");
                sbCount.append(" and  a.CREATE_TIME >= ? ");
                params.add(query.getBeginTime());

            }
            // 结束日期
            if (Objects.nonNull(query.getEndTime())) {
                sb.append(" and a.CREATE_TIME <= ? ");
                sbCount.append(" and  a.CREATE_TIME <= ? ");
                params.add(query.getEndTime());
            }
            // 指标体系评价->关联风采点击传参
            if (Objects.nonNull(query.getParentId())) {
                sb.append(" and a.parent_id = ? ");
                sbCount.append(" and a.parent_id = ? ");
                params.add(Long.valueOf(query.getParentId()));
            }
            if (Objects.nonNull(query.getIndicatorSystemId())) {
                sb.append(" and a.indicator_system_id = ? ");
                sbCount.append(" and a.indicator_system_id = ? ");
                params.add(Long.valueOf(query.getIndicatorSystemId()));
            }
            sb.append(" order by a.create_time desc ");
        }
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<PageActivityRspDTO> result;
        String dbType = jdbcTemplate.getDbType();

        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(PageActivityRspDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<PageActivityRspDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    /**
     * 查看详细
     */
    public PageActivityRspDTO selectActivity(OperationActivityReqDTO activityReqDTO) {
        StringBuilder sb = new StringBuilder(PAGE_Activity_BASE_SQL);
        List<Object> params = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(activityReqDTO)) {
            if (Objects.nonNull(activityReqDTO.getType())) {
                String replace = null;
                String replace1 = null;
                if (1 == activityReqDTO.getType()) {
                    //关联突击队
                    replace = sb.toString().replace("[CD]", "left join YCQ_QZH_COMMANDO y1 on a.PARENT_ID =y1.id and y1.del_flag='0' ");
                    replace1 = replace.replace("[AD]", " y1.COMMANDO_NAME as parentName, y1.id as parentId, ");
                    sb = new StringBuilder(replace1);
                } else if (2 == activityReqDTO.getType()) {
                    //关联文明号  文明号逻辑需要修改
                    replace = sb.toString().replace("[CD]", "left join ycq_yn_declaration y2 on a.parent_id = y2.declaration_id and y2.deleted = 0 ");
                    replace1 = replace.replace("[AD]", " y2.collective_name as parentName, y2.declaration_id as parentId, ");
                    sb = new StringBuilder(replace1);
                } else if (3 == activityReqDTO.getType()) {
                    //关联示范岗
                    replace = sb.toString().replace("[CD]", "left join ycq_qzh_qag_apply y3 on a.parent_id =y3.id and y3.del_flag = 0  ");
                    replace1 = replace.replace("[AD]", " y3.apply_org_name as parentName, y3.id as parentId,");
                    sb = new StringBuilder(replace1);
                } else if (4 == activityReqDTO.getType()) {
                    //关联权益岗
                    replace = sb.toString().replace("[CD]", "left join YCQ_QZH_RIGHTS_PROTECTION y4 on a.PARENT_ID =y4.id and y4.del_flag='0' ");
                    replace1 = replace.replace("[AD]", " y4.unit_name as parentName, y4.id as parentId, ");
                    sb = new StringBuilder(replace1);
                } else {
                    // 其他
                    replace = sb.toString().replace("[CD]", "");
                    replace1 = replace.replace("[AD]", "");
                    sb = new StringBuilder(replace1);
                }
            } else {
                String replace = sb.toString().replace("[CD]", "");
                String replace1 = replace.replace("[AD]", "");
                sb = new StringBuilder(replace1);
            }
            //id
            if (Objects.nonNull(activityReqDTO.getId())) {
                sb.append(" and a.id = ? ");
                params.add(activityReqDTO.getId());
            }
        }
        List<PageActivityRspDTO> pageActivityRspDTOS = jdbcTemplate.queryForBeans(PageActivityRspDTO.class, sb.toString(), params.toArray());
        if (pageActivityRspDTOS.size() > 0) {
            return pageActivityRspDTOS.get(0);
        } else {
            return null;
        }
    }

    /**
     * 删除风采封面
     */
    public void deleteAnnex(Long id) {
        //删除封面
        String sql = "delete from ycq_qzh_cover where type=3 and article_id=" + id;
        jdbcTemplate.execute(sql);
    }

    /**
     * 删除风采
     *
     * @param ids
     * @param updateId
     * @param updateBy
     */
    public void deleteByIds(Collection<Long> ids, Long updateId, String updateBy) {
        String inIds = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ") "));
        StringBuilder sb = new StringBuilder(DELETE_SQL).append(inIds);
        jdbcTemplate.update(sb.toString(), updateId, updateBy, new Date());
    }

    /**
     * 查询所属对象
     *
     * @param type
     */
    public List<ParentDTO> findParent(Integer type) {
        String sql = null;
        //突击队
        if (ObjectUtil.isNotEmpty(type) && type.equals(1)) {
            sql = " select id,COMMANDO_NAME as name from YCQ_QZH_COMMANDO  where del_flag=0 and state = 3 ";
        }
        //文明号
        if (ObjectUtil.isNotEmpty(type) && type.equals(2)) {
            sql = " select a.id,b.COLLECTIVENESS_NAME as name from YCQ_CIVILIZATION_INVENTORY a \n" +
                    "left join YCQ_CIVILIZATION_REPORT b on a.id=b.MAIN_ID  where a.del_flag='0'";
        }
        //青年安全生产示范岗
        if (ObjectUtil.isNotEmpty(type) && type.equals(3)) {
            sql = " select id,APPLY_ORG_NAME as name from YCQ_QZH_YOUTH_SAFETY_MANAGEMENT  where del_flag='0'";
        }
        //青少年权益岗
        if (ObjectUtil.isNotEmpty(type) && type.equals(4)) {
            sql = "select id,unit_name as name from YCQ_QZH_RIGHTS_PROTECTION  where del_flag='0'";
        }
        return jdbcTemplate.queryForBeans(ParentDTO.class, sql);
    }

    public String selectOrgName(Long createId) {
        String SQL = "select distinct b.org_name from TSM_WORKER a left join TSM_ORG b on a.ORG_ID =b.ORG_ID where a.LOGIN_ID = " + createId;
        return jdbcTemplate.queryForObject(SQL, String.class);
    }

    public List<Long> selecetOrgs(int loginId) {
        String SQL = " select distinct org_id from TSM_WORKER_ORG_ALL where worker_id =" + loginId;
        return jdbcTemplate.queryForRawObjects(Long.class, SQL);
    }

    public List<Long> selecetIds(List<Long> orgIds) {
        String collect = orgIds.stream().map(String::valueOf).collect(Collectors.joining(",", "(", ") "));
        String SQL = "select distinct BUSINESS_ID from YCQ_QZH_BUSINESS_ALL where 1=1 and org_id in " + collect;
        return jdbcTemplate.queryForRawObjects(Long.class, SQL);
    }

    public PageRsp<ShowListQueryRespDTO> showListQueryByActId(PageReq<ShowListQueryReqDTO> dto) {
        StringBuilder sb = new StringBuilder("select qa.id,qa.activity_title,case when qb.viewTotal is null then 0 else qb.viewTotal end viewTotal, " +
                "qa.create_time from ycq_qzh_activity qa left join  " +
                "(select article_id,count(1) as viewTotal from ycq_qzh_browse  group by article_id) qb  " +
                "on qa.id=qb.article_id  where qa.del_flag=0 and qa.act_id = ?");
        StringBuilder sbCount = new StringBuilder("select count(*) from ycq_qzh_activity qa left join  " +
                "(select article_id,count(1) as viewTotal from ycq_qzh_browse  group by article_id) qb  " +
                "on qa.id=qb.article_id  where qa.del_flag=0 and qa.act_id = ?");
        ArrayList<Object> params = new ArrayList<>();
        params.add(dto.getQuery().getId().toString());
        int total = jdbcTemplate.queryForInt(sbCount.toString(), dto.getQuery().getId().toString());
        List<ShowListQueryRespDTO> result;
        String dbType = jdbcTemplate.getDbType();

        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, dto);
            result = jdbcTemplate.queryForBeans(ShowListQueryRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<ShowListQueryRespDTO> page = new PageRsp<>();
        page.setPageNo(dto.getPageNo());
        page.setPageSize(dto.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    public TsmOrg findParentOrgs(Long orgId) {
        String SQL = "select org_id,parent_org_id from tsm_org where org_id = " + orgId;
        return jdbcTemplate.queryForBean(TsmOrg.class, SQL);
    }

    public List<QueryCommandoRespDTO> queryCommandoByExamine(PageReq<QueryCommandoReqDTO> req, Worker worker, List<String> commandoCodes) {
        StringBuilder sb = new StringBuilder("    select * from ( select a.*,b.commando_state  from (  SELECT     a.*,    \n"
                + "                    \n"
                + "                       a.create_by applicant,   \n"
                + "                          a.create_time applyTime,     \n"
                + "                           a.GROUP_ORG_NAME orgName  FROM   \n"
                + "                              ycq_qzh_commando a    \n"
                + "                                LEFT JOIN tsm_worker b  \n"
                + "                                ON a.create_id = b.login_id WHERE    \n"
                + "                                  a.DEL_FLAG=0 and \n"
                + "                                  a.STATE!=1       ) as a LEFT join (                          \n"
                + "                                  \n"
                + "                                  \n"
                + "                                  \n"
                + "                                  select commando_code,commando_state from(   SELECT   a.commando_code, \n"
                + "                 (CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6 and k.label = 'true' )  THEN '冻结' ELSE '正常' END ) commando_state \n"
                + "                 FROM     \n"
                + "                 ycq_qzh_commando a            \n"
                + "                 LEFT JOIN tsm_worker b ON a.create_id = b.login_id   \n"
                + "                 LEFT JOIN (   \n"
                + "                    select a.commando_code , count(parent_id) mien_count ,max(b.create_time) max_time ,   \n"
                + "                     (case when ((count(parent_id) = 0 and  TIMESTAMPDIFF( MONTH, max(a.create_time) , now()) > 6)   \n"
                + "                     or (count(parent_id) > 0 and  TIMESTAMPDIFF( MONTH, max(b.create_time) , now()) > 6))   \n"
                + "                     then 'true' else 'false' end) label   \n"
                + "                     from ycq_qzh_commando a   \n"
                + "                     LEFT JOIN YCQ_QZH_ACTIVITY b on a.id = b.parent_id and b.activity_type = 1   and a.del_flag = 0 and b.del_flag = 0   \n"
                + "                     group by a.commando_code   \n"
                + "                 ) k ON k.commando_code = a.commando_code  ) where 1=1 ) b on a.commando_code=b.commando_code ) where 1=1   ");

        List<Object> params = new ArrayList<>();
        if (null != commandoCodes && commandoCodes.size() > 0) {
            String collect = commandoCodes.stream().map(code -> "\'" + code.toString() + "\'").collect(Collectors.joining(",", "(", ") "));
            sb.append("  and COMMANDO_CODE in  " + collect + "   ");
        }
        sb.append(" and state = ? ");
        params.add(3);
        return jdbcTemplate.queryForBeans(QueryCommandoRespDTO.class, sb.toString(), params.toArray());
    }

    public List<QueryCommandoRespDTO> queryCommandoByApplicant(Worker worker) {
        String PAGE_BASE_SQL = " SELECT  a.*,  ( CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6  or a.STATE <=2)  THEN '冻结' ELSE '正常' END ) commandoState, b.worker_name applicant, a.create_time applyTime,  a.GROUP_ORG_NAME orgName FROM  ycq_qzh_commando a  LEFT JOIN tsm_worker b ON a.create_id = b.login_id WHERE  a.DEL_FLAG=0 ";
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        List<Object> params = new ArrayList<>();
        sb.append(" and a.create_id = ? ");
        params.add(worker.getLoginId());
        sb.append(" and a.state = ? ");
        params.add(3);
        return jdbcTemplate.queryForBeans(QueryCommandoRespDTO.class, sb.toString(), params.toArray());
    }

    public List<String> selectCommandoCode(List<Long> allOrgList) {
        if (allOrgList.size() > 0) {
            String collect = allOrgList.stream().map(String::valueOf).collect(Collectors.joining(",", "(", ") "));
            String SQL = "select distinct COMMANDO_CODE from YCQ_QZH_COMMANDO_ORG where 1=1 and org_id in " + collect;
            return jdbcTemplate.queryForRawObjects(String.class, SQL);
        } else {
            return null;
        }

    }

    /**
     * 查询风险指标
     * 指标体系管理中当前时间所处指标版本体系有效期内的指标
     *
     * @author leixb
     * @since 2023-11-21 16:18
     */
    public List<YcqQzhIndicatorSystemVo> findTarget() {
        String sql = " SELECT sys.* FROM ycq_qzh_indicator_system sys \n" +
                "JOIN (SELECT id FROM ycq_qzh_indicator_version WHERE deleted = 0 \n" +
                "AND ? BETWEEN effective_time_start AND effective_time_end \n" +
                "ORDER BY effective_time_start DESC LIMIT 1) ver \n" +
                "ON sys.indicator_version_id = ver.id \n" +
                "WHERE sys.deleted = 0 AND secondary_indicators IS NOT NULL ORDER BY sys.create_time ";
        return jdbcTemplate.queryForBeans(YcqQzhIndicatorSystemVo.class, sql, LocalDate.now());
    }

    /**
     * 查询所属对象
     * 弃用
     *
     * @param type   类型 1-突击队，2-文明号，3-示范岗，4-权益岗
     * @param worker 用户
     * @return List<ParentDTO>
     * @author leixb
     * @since 2023-11-23 11:11
     */
    public List<ParentDTO> findParentOld(Integer type, Worker worker) {
        StringBuilder sql = new StringBuilder();
        if (1 == type) {
            sql.append(" SELECT id, commando_name name FROM ycq_qzh_commando WHERE state = 3 AND del_flag = 0 AND group_org_id = ? ");
        } else if (2 == type) {
            sql.append(" SELECT declaration_id id, collective_name name FROM ycq_yn_declaration WHERE audit_state = 7 AND deleted = 0 AND org_id = ?  ");
        } else if (3 == type) {
            sql.append(" SELECT id, apply_org_name name FROM ycq_qzh_qag_apply WHERE flow_state = 7 AND del_flag = 0 AND affiliation_org_id = ?  ");
        } else {
            sql.append(" SELECT id, unit_name name FROM ycq_qzh_rights_protection WHERE link = '1006' AND del_flag = 0 AND applicant_org_id = ? ");
        }
        sql.append(" ORDER BY create_time ");
        return jdbcTemplate.queryForBeans(ParentDTO.class, sql.toString(), worker.getOrgId());
    }

    private boolean validatedSuper(Worker worker) {
        if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            return true;
        }
        return false;
    }

}