package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.document.YcqQzhAnnouncement;
import com.crunii.micro.service.qzh.api.domain.dto.announcement.PageAnnouncementReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.announcement.PageAnnouncementRspDTO;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;


@Repository
public class YcqQzhAnnouncementJdbcDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String PAGE_ANNOUNCEMENT_BASE_SQL = " select a.*,\n" +
            "case when a.announcement_type=1 then '青年突击队公告' when a.announcement_type=2 then '青年文明号公告'\n" +
            "when a.announcement_type=3 then '青年安全生产示范岗公告' when a.announcement_type=4 then '维护青少年权益岗公告' else tpv.value_name end as announcementTypeName,\n" +
            "case when a.open_competence=1 then '全部' when a.open_competence=2 then '对内' when a.open_competence=3 then '对外' else null end as openCompetenceName,\n" +
            "b.worker_name as createUserName,a.create_time as createTime,\n" +
            "tpv.value_code, "+
            "case when a.state=1 then '已发布' when a.state=2 then '已下架' else null end as stateName,\n" +
            "(case when a.sort_num>0 then true else false end) as whetherTop,\n" +
            "case when (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=2) is not null \n" +
            "then (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=2)  else 0 end  as viewTotal \n" +
            "from ycq_qzh_announcement a\n" +
            "left join tsm_property_value tpv on a.announcement_type = tpv.value_code  and tpv.prop_code = 'announcementOtherType' "+
            "left join tsm_worker b on a.create_id=b.login_id\n" +
            "where a.del_flag=0 ";

    private static final String PAGE_ANNOUNCEMENT_COUNT_SQL = "select count(*) from ycq_qzh_announcement a "
        + " left join tsm_property_value tpv on a.announcement_type = tpv.value_code  and tpv.prop_code = 'announcementOtherType' "
        + " left join tsm_worker b on a.create_id=b.login_id where a.del_flag=0 ";

//    private static final String FIND_ANNOUNCEMENT_LIST_SQL = "select id,document_title from ycq_qzh_announcement where document_type=2 and apply_type=? and del_flag=0 ";

    private static final String DELETE_SQL = "update ycq_qzh_announcement set del_flag=1, update_id=?, update_by=?, update_time=? where id in";

    private static final String GET_ANNOUNCEMENT_NEWEST =
            " select id,announcement_title,case when announcement_type=1 then '青年突击队公告公示' when announcement_type=2 then '青年文明号公告公示'\n" +
                    "when announcement_type=3 then '青年安全生产示范岗公告公示' when announcement_type=4 then '维护青少年权益岗公告公示' else tpv.value_name end as announcementTypeName,\n" +
                    "b.worker_name as createUserName,a.create_time  as createTime,\n" +
                    "tpv.value_code, "+
                    "case when announcement_type=1 then 'danger' when announcement_type=2 then 'warning'\n" +
                    "when announcement_type=3 then 'success' when announcement_type=4 then 'info' else 'other' end as colour,\n" +
                    "case when (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=2) is not null\n" +
                    "then (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=2)  else 0 end  as viewTotal\n" +
                    "from ycq_qzh_announcement a\n" +
                    "left join tsm_property_value tpv on a.announcement_type = tpv.value_code  and tpv.prop_code = 'announcementOtherType' "+
                    "left join tsm_worker b on a.create_id=b.login_id where a.del_flag=0 and a.state =1 ";

    public PageRsp<PageAnnouncementRspDTO> pageAnnouncement(PageReq<PageAnnouncementReqDTO> req, List<Long> listIds, String orgId) {
        StringBuilder sb = new StringBuilder(PAGE_ANNOUNCEMENT_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_ANNOUNCEMENT_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        PageAnnouncementReqDTO query = req.getQuery();
        if (ObjectUtil.isNotEmpty(query)) {
            if (StrUtil.isNotBlank(query.getAnnouncementTitle())) {
                sb.append(" and a.announcement_title like concat('%',?,'%') ");
                sbCount.append(" and a.announcement_title like concat('%',?,'%') ");
                params.add(query.getAnnouncementTitle());
            }
            //类型
            if (Objects.nonNull(query.getAnnouncementType())) {
                if (5 == query.getAnnouncementType()) {
                    sb.append(" and a.announcement_type not in (1,2,3,4) ");
                    sbCount.append(" and a.announcement_type not in (1,2,3,4)  ");
                } else {
                    sb.append(" and a.announcement_type = ? ");
                    sbCount.append(" and a.announcement_type = ? ");
                    params.add(query.getAnnouncementType());
                }
            }
//            if ("app".equals(query.getCondition()) && null != listIds) {
//                String collect = listIds.stream().map(String::valueOf).collect(Collectors.joining(",", "(", ") "));
//                sb.append(" and (a.open_competence = 1 or (a.id in " + collect + " and a.open_competence =2 )) ");
//                sbCount.append(" and (a.open_competence = 1 or (a.id in " + collect + " and a.open_competence =2 )) ");
//            } else if ("app".equals(query.getCondition()) && null == listIds) {
//                sb.append(" and a.open_competence = 1 ");
//                sbCount.append(" and a.open_competence = 1 ");
//            } else if ("web".equals(query.getCondition()) && null != orgId) {
//                if (1 == query.getOpenCompetence()) {
//                    sb.append(" and (a.open_competence = 1 or (find_in_set(?,create_org_ids) and a.open_competence =2 )) ");
//                    sbCount.append(" and (a.open_competence = 1 or (find_in_set(?,create_org_ids) and a.open_competence =2 )) ");
//                    params.add(orgId);
//                } else if (2 == query.getOpenCompetence()) {
//                    sb.append(" and find_in_set(?,create_org_ids) and a.open_competence =2  ");
//                    sbCount.append(" and find_in_set(?,create_org_ids) and a.open_competence =2 ");
//                    params.add(orgId);
//                }
//            } else if ("web".equals(query.getCondition()) && null == orgId) {
            if (Objects.nonNull(query.getOpenCompetence())) {
                sb.append(" and a.open_competence = ? ");
                sbCount.append(" and a.open_competence = ? ");
                params.add(query.getOpenCompetence());
            }
//            } else {
//                new BusinessException("系统内部异常，请联系管理员!");
//            }
            if (Objects.nonNull(req.getQuery().getState())) {
                sb.append(" and a.state = ? ");
                sbCount.append(" and a.state = ? ");
                params.add(query.getState());
            }
            if (ObjectUtil.isNotEmpty(query.getCreateUserName())) {
                sb.append(" and b.worker_name like concat('%',?,'%') ");
                sbCount.append(" and b.worker_name like concat('%',?,'%') ");
                params.add(query.getCreateUserName());
            }
            //有效开始时间
            if (StringUtils.isNotEmpty(query.getBeginTime())) {
                sb.append(" and a.create_time>=? ");
                sbCount.append(" and a.create_time>=? ");
                params.add(query.getBeginTime());
            }
            //有效结束时间
            if (StringUtils.isNotEmpty(query.getEndTime())) {
                sb.append(" and a.create_time<=? ");
                sbCount.append(" and a.create_time<=? ");
                params.add(query.getEndTime());
            }
            //app仅查看已发布
            if (StringUtils.isNotEmpty(query.getCondition()) && query.getCondition().equals("app")) {
                sb.append(" and a.state=? ");
                sbCount.append(" and a.state=? ");
                params.add(1);
            }
            sb.append(" order by (case when a.sort_num is null then 0 else a.sort_num end) desc,a.create_time desc");
        }
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<PageAnnouncementRspDTO> result;
        String dbType = jdbcTemplate.getDbType();

        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(PageAnnouncementRspDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<PageAnnouncementRspDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    /**
     * 查看公告公示详情
     *
     * @param
     * @return
     */
//    public AnnouncementDetailsRspDTO getOneDetails(Long id) {
//        List<Object> params = new ArrayList<>();
//        params.add(id);
//        AnnouncementDetailsRspDTO result = jdbcTemplate.queryForBean(AnnouncementDetailsRspDTO.class, GET_ANNOUNCEMENT_ONE_SQL, params);
//        return result;
//    }
    public void deleteByIds(Collection<Long> ids, Long updateId, String updateBy) {
        String inIds = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ") "));
        StringBuilder sb = new StringBuilder(DELETE_SQL).append(inIds);
        jdbcTemplate.update(sb.toString(), updateId, updateBy, new Date());
    }

    /**
     * 删除公告公示附件以及封面
     */
    public void deleteAnnex(Long id) {
        //删除附件
        String sql1 = "delete from ycq_qzh_annex where type=2 and article_id=" + id;
        jdbcTemplate.execute(sql1);
        //删除封面
        String sql = "delete from ycq_qzh_cover where type=2 and article_id=" + id;
        jdbcTemplate.execute(sql);
    }

    /**
     * 查询当前排序序号后的数据
     */
    public List<YcqQzhAnnouncement> findListBySortNum(Integer sortNum) {
        String sql = "select * from  ycq_qzh_announcement where sort_num>" + sortNum;
        return jdbcTemplate.queryForBeans(YcqQzhAnnouncement.class, sql);
    }

//    "select * from (\n"+
//            "select a.id,a.announcement_title as title,a.create_time,\n"+
//            "case when a.announcement_type=1 then '青年突击队公告' when a.announcement_type=2 then '青年文明号公告'\n"+
//            "when a.announcement_type=3 then '青年安全生产示范岗公告' when a.announcement_type=4 then '维护青少年权益岗公告' else null end as typeName,\n"+
//            "case when (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=2) is not null \n"+
//            "then (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=2)  else 0 end  as viewTotal \n"+
//            "from ycq_qzh_announcement a\n"+
//            "left join tsm_worker b on a.create_id=b.login_id where a.del_flag=0\n"+
//            "union all\n"+
//            "select a.id,a.document_title as title,a.create_time,\n"+
//            "case when a.document_type=1 then '青年突击队政策'\n"+
//            "when a.document_type=2 then '青年文明号政策' when a.document_type=3 then '青年安全生产示范岗政策' \n"+
//            "when a.document_type=4 then '维护青少年权益岗政策' else null end as typeName,\n"+
//            "case when (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=1) is not null \n"+
//            "then (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=1)  else 0 end  as viewTotal \n"+
//            "from ycq_qzh_document a\n"+
//            "left join tsm_worker b on a.create_id=b.login_id where del_flag=0\n"+
//            "union all\n"+
//            "select a.id,a.activity_title as title,a.create_time, \n"+
//            "case when a.activity_type=1 then '青年突击队风采' when a.activity_type=2 then '青年文明号风采' \n"+
//            "when a.activity_type=3 then '青年安全生产示范岗风采' when a.activity_type=4 then '维护青少年权益岗风采' else null end as typeName, \n"+
//            "case when (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=3) is not null \n"+
//            "then (select count(1) from ycq_qzh_browse where article_id=a.id and record_type=3)  else 0 end  as viewTotal \n"+
//            "from ycq_qzh_activity a \n"+
//            "left join tsm_worker b on a.create_id=b.login_id where a.del_flag=0 ) a order by a.create_time desc limit 10"

    /**
     * 获取最新五条公告公示
     *
     * @return
     */
    public List<PageAnnouncementRspDTO> getAnnouncementNewest(List<Long> listIds) {
        StringBuilder sb = new StringBuilder(GET_ANNOUNCEMENT_NEWEST);
        sb.append(" and a.open_competence = 1 ");
        sb.append(" and a.announcement_type in(1,2,3,4) ");
        sb.append(" order by a.create_time desc limit 5 ");
        return jdbcTemplate.queryForBeans(PageAnnouncementRspDTO.class, sb.toString());
    }

    public String selectOrgName(Long createId) {
        String SQL = "select distinct b.org_name from TSM_WORKER a left join TSM_ORG b on a.ORG_ID =b.ORG_ID where a.LOGIN_ID = " + createId;
        return jdbcTemplate.queryForObject(SQL, String.class);
    }

    /**
     * 查询申报下所有公示
     */
//    public List<YcqQzhAnnouncement> findListByParentId(Long parentId) {
//        String sql = "select * from ycq_qzh_announcement where parent_id=" + parentId;
//        return jdbcTemplate.queryForBeans(YcqQzhAnnouncement.class, sql);
//    }

    /**
     * 查找申报关联工单
     */
//    public List<YcqQzhRightsProtection> findWorkerOrderByRightsProtection(WorkerOrderReqDTO dto) {
//        List<Object> params = new ArrayList<>();
//        StringBuilder sql = new StringBuilder("select a.id,a.applicant_code,a.apply_type,a.unit_name,a.organization,a.phone,a.applicant, " +
//                "  a.apply_time,a.status" +
//                "  from ycq_qzh_rights_protection a  where 1=1 ");
//        if (ObjectUtil.isNotEmpty(dto.getApplyType())) {
//            sql.append(" and a.apply_type=?");
//            if (dto.getApplyType().equals(1)) {
//                params.add("新建");
//            } else {
//                params.add("复核");
//            }
//        }
//        //申报编号
//        if (ObjectUtil.isNotEmpty(dto.getApplicantCode())) {
//            sql.append(" and a.applicant_code=?");
//            params.add(dto.getApplicantCode());
//        }
//        return jdbcTemplate.queryForBeans(YcqQzhRightsProtection.class, sql.toString(), params);
//    }
    public List<Long> selecetOrgs(int loginId) {
        String SQL = " select distinct org_id from TSM_WORKER_ORG_ALL where worker_id =" + loginId;
        return jdbcTemplate.queryForRawObjects(Long.class, SQL);
    }

    public List<Long> selecetIds(List<Long> orgIds) {
        String collect = orgIds.stream().map(String::valueOf).collect(Collectors.joining(",", "(", ") "));
        String SQL = "select distinct BUSINESS_ID from YCQ_QZH_BUSINESS_ALL where 1=1 and org_id in " + collect;
        return jdbcTemplate.queryForRawObjects(Long.class, SQL);
    }

    public TsmOrg findParentOrgs(Long orgId) {
        String SQL = "select org_id,parent_org_id from tsm_org where org_id = " + orgId;
        return jdbcTemplate.queryForBean(TsmOrg.class, SQL);
    }
}