/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved. <a>http://www.cqcis.com</a>
 */
package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.YcqQzhCommando;
import com.crunii.micro.domain.entity.main.YcqQzhCommandoFlow;
import com.crunii.micro.domain.entity.main.YcqQzhCommandoLog;
import com.crunii.micro.service.qzh.api.domain.dto.commandDo.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * demo
 *
 * @author xiangjunming
 * @date 2023/9/4
 */
@Repository
public class YcqQzhCommandoJdbcDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;


    private static final String PAGE_BASE_SQL = "SELECT " + "    a.*, " + "    ( CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6  or a.STATE <=2)  THEN '冻结' ELSE '正常' END ) commandoState, " + "    b.worker_name applicant, " + "    a.create_time applyTime, " + "    a.GROUP_ORG_NAME orgName  " + "FROM " + "    ycq_qzh_commando a " + "    LEFT JOIN tsm_worker b ON a.create_id = b.login_id " + "WHERE " + "    a.DEL_FLAG=0 ";


    private static final String RANK_SQL = " SELECT *,POINT collectNum FROM YCQ_QZH_COMMANDO_POINT  WHERE 1=1   ";

    //更改了字段 code 改为 COMMANDO_CODE
//    private static final String GET_ONE_SQL = "select b.*,b.POINT collectNum from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_POINT b where a.CODE=b.COMMANDO_CODE and (a.CREATE_ID=? or a.TEAM_LEADER_PHONE = ?) limit 1 ";
    private static final String GET_ONE_SQL = " select b.*,b.POINT collectNum from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_POINT b where a.COMMANDO_CODE=b.COMMANDO_CODE and (a.CREATE_ID=? or a.TEAM_LEADER_PHONE = ?) limit 1 ";
    private static final String GET_ONE_SQL_BY_APLLY = "select a.* from YCQ_QZH_COMMANDO a where  (a.CREATE_ID=? or a.TEAM_LEADER_PHONE = ?) limit 1 ";
    private static final String RANK_SQL_COUNT = " select count(*) from YCQ_QZH_COMMANDO_POINT a where 1=1 ";
    private static final String PAGE_COUNT_SQL = " select count(*) from ycq_qzh_commando a left  join  tsm_worker b on  a.create_id=b.login_id  left  join YCQ_LEAGUE_DETAIL c on  a.GROUP_ORG_ID=c.league_id where  a.DEL_FLAG=0 ";


    private static final String DELETE_SQL = "update ycq_qzh_commando set del_flag=1, update_id=?, update_by=?, update_time=? where id = ";

    private static final String EXAMINE_SQL = "update ycq_qzh_commando set EXAMINE_REASON=?,STATE=?, update_id=?, update_by=?, update_time=? where id in ";

    public PageRsp<QueryCommandoRespDTO> queryCommandoByExamine(PageReq<QueryCommandoReqDTO> req, Worker worker, List<String> commandoCodes) {
        StringBuilder sb = new StringBuilder("  SELECT     a.*,   \n"
                + "  \n"
                + "      a.create_by applicant,  \n"
                + "         a.create_time applyTime,    \n"
                + "          a.GROUP_ORG_NAME orgName  FROM  \n"
                + "             ycq_qzh_commando a   \n"
                + "               LEFT JOIN tsm_worker b \n"
                + "               ON a.create_id = b.login_id WHERE   \n"
                + "                 a.DEL_FLAG=0 and\n"
                + "                 a.STATE!=1\n   ");

        List<Object> params = new ArrayList<>();

        if (null != commandoCodes && commandoCodes.size() > 0) {
            String collect = commandoCodes.stream().map(code -> "\'" + code.toString() + "\'").collect(Collectors.joining(",", "(", ") "));
            sb.append("  and a.COMMANDO_CODE in  " + collect + "   ");
        }

        if (StrUtil.isNotBlank(req.getQuery().getCommandoName())) {
            sb.append(" and a.commando_name like concat('%',?,'%') ");
            params.add(req.getQuery().getCommandoName());
        }
        if (Objects.nonNull(req.getQuery().getGroupOrgId())) {
            sb.append(" and a.GROUP_ORG_ID = ? ");
            params.add(req.getQuery().getGroupOrgId());
        }
        if (Objects.nonNull(req.getQuery().getState())) {
            if (req.getQuery().getState() == 4) {
                sb.append(" and (a.state = 4 or a.state=5) ");
            } else {
                sb.append(" and a.state = ? ");
                params.add(req.getQuery().getState());
            }
        }
        if (Objects.nonNull(req.getQuery().getType())) {
            sb.append(" and a.type = ? ");
            params.add(req.getQuery().getType());
        }
        if (Objects.nonNull(req.getQuery().getDistribution())) {
            sb.append(" and a.distribution = ? ");
            params.add(req.getQuery().getDistribution());
        }
        if (Objects.nonNull(req.getQuery().getStartTime())) {
            sb.append(" and  to_char(a.CREATE_TIME,'yyyy-mm-dd') >=? ");
            params.add(req.getQuery().getStartTime());
        }
        if (Objects.nonNull(req.getQuery().getEndTime())) {
            sb.append(" and  to_char(a.CREATE_TIME,'yyyy-mm-dd') <=? ");
            params.add(req.getQuery().getEndTime());
        }

        //申请人
        if (Objects.nonNull(req.getQuery().getApplicant())) {
            sb.append(" and  a.create_by like concat('%',?,'%') ");
            params.add(req.getQuery().getApplicant());
        }
        if (Objects.nonNull(req.getQuery().getHaveThreeYearPlan())) {
            sb.append(" and  a.HAVE_THREE_YEAR_PLAN like concat('%',?,'%') ");
            params.add(req.getQuery().getHaveThreeYearPlan());
        }
        sb.append("  order by CREATE_TIME desc   ");
        String sbCount = "  select count(*) from (  " + sb + "   )   ";
        int total = jdbcTemplate.queryForInt(sbCount, params.toArray());
        List<QueryCommandoRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(QueryCommandoRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<QueryCommandoRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }

    /**
     * @description: 审核人查询申请记录  (自己分页用)
     * @param: [req, worker]
     * @return: com.crunii.micro.common.dto.PageRsp<com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO>
     * @author 13760
     * @date: 2023/10/18 16:04
     */
    public PageRsp<QueryCommandoRespDTO> queryCommandoForAudit(PageReq<QueryCommandoReqDTO> req, Worker worker, List<String> commandoCodes) {
        StringBuilder sb = new StringBuilder("    select * from ( select a.*,b.commando_state  from (  SELECT     a.*,    \n"
                + "                    \n"
                + "                       a.create_by applicant,   \n"
                + "                          a.create_time applyTime,     \n"
                + "                           a.GROUP_ORG_NAME orgName  FROM   \n"
                + "                              ycq_qzh_commando a    \n"
                + "                                LEFT JOIN tsm_worker b  \n"
                + "                                ON a.create_id = b.login_id WHERE    \n"
                + "                                  a.DEL_FLAG=0 and \n"
                + "                                  a.STATE!=1       ) as a LEFT join (                          \n"
                + "                                  \n"
                + "                                  \n"
                + "                                  \n"
                + "                                  select commando_code,commando_state from(   SELECT   a.commando_code, \n"
                + "                 (CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6 and k.label = 'true' )  THEN '冻结' ELSE '正常' END ) commando_state \n"
                + "                 FROM     \n"
                + "                 ycq_qzh_commando a            \n"
                + "                 LEFT JOIN tsm_worker b ON a.create_id = b.login_id   \n"
                + "                 LEFT JOIN (   \n"
                + "                    select a.commando_code , count(parent_id) mien_count ,max(b.create_time) max_time ,   \n"
                + "                     (case when ((count(parent_id) = 0 and  TIMESTAMPDIFF( MONTH, max(a.create_time) , now()) > 6)   \n"
                + "                     or (count(parent_id) > 0 and  TIMESTAMPDIFF( MONTH, max(b.create_time) , now()) > 6))   \n"
                + "                     then 'true' else 'false' end) label   \n"
                + "                     from ycq_qzh_commando a   \n"
                + "                     LEFT JOIN YCQ_QZH_ACTIVITY b on a.id = b.parent_id and b.activity_type = 1   and a.del_flag = 0 and b.del_flag = 0   \n"
                + "                     group by a.commando_code   \n"
                + "                 ) k ON k.commando_code = a.commando_code  ) where 1=1 ) b on a.commando_code=b.commando_code ) where 1=1   ");

        List<Object> params = new ArrayList<>();

        if (null != commandoCodes && commandoCodes.size() > 0) {
            String collect = commandoCodes.stream().map(code -> "\'" + code.toString() + "\'").collect(Collectors.joining(",", "(", ") "));
            sb.append("  and COMMANDO_CODE in  " + collect + "   ");
        }

        if (StrUtil.isNotBlank(req.getQuery().getCommandoName())) {
            sb.append(" and commando_name like concat('%',?,'%') ");
            params.add(req.getQuery().getCommandoName());
        }
        if (Objects.nonNull(req.getQuery().getGroupOrgId())) {
            sb.append(" and GROUP_ORG_ID = ? ");
            params.add(req.getQuery().getGroupOrgId());
        }
        if (Objects.nonNull(req.getQuery().getState())) {
            if (req.getQuery().getState() == 4) {
                sb.append(" and (state = 4 or state=5) ");
            } else {
                sb.append(" and state = ? ");
                params.add(req.getQuery().getState());
            }
        }
        if (Objects.nonNull(req.getQuery().getType())) {
            sb.append(" and type = ? ");
            params.add(req.getQuery().getType());
        }
        if (Objects.nonNull(req.getQuery().getDistribution())) {
            sb.append(" and distribution = ? ");
            params.add(req.getQuery().getDistribution());
        }
        if (Objects.nonNull(req.getQuery().getStartTime())) {
            sb.append(" and  to_char(CREATE_TIME,'yyyy-mm-dd') >=? ");
            params.add(req.getQuery().getStartTime());
        }
        if (Objects.nonNull(req.getQuery().getEndTime())) {
            sb.append(" and  to_char(CREATE_TIME,'yyyy-mm-dd') <=? ");
            params.add(req.getQuery().getEndTime());
        }

        //申请人
        if (Objects.nonNull(req.getQuery().getApplicant())) {
            sb.append(" and  create_by like concat('%',?,'%') ");
            params.add(req.getQuery().getApplicant());
        }
        if (Objects.nonNull(req.getQuery().getHaveThreeYearPlan())) {
            sb.append(" and  HAVE_THREE_YEAR_PLAN like concat('%',?,'%') ");
            params.add(req.getQuery().getHaveThreeYearPlan());
        }

        if (Objects.nonNull(req.getQuery().getCommandoState())) {
            sb.append(" and  commando_state like concat('%',?,'%') ");
            params.add(req.getQuery().getCommandoState());
        }

        sb.append("  order by CREATE_TIME desc   ");
        String sbCount = "  select count(*) from (  " + sb + "   )   ";
        int total = jdbcTemplate.queryForInt(sbCount, params.toArray());
        List<QueryCommandoRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(QueryCommandoRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<QueryCommandoRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }


    /**
     * @description: 审核人查询申请记录 返回list (给其他调用)
     * @param: [req, worker]
     * @return: com.crunii.micro.common.dto.PageRsp<com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO>
     * @author 13760
     * @date: 2023/10/18 16:04
     */
    public List<QueryCommandoRespDTO> queryCommandoForAuditList(QueryCommandoReqDTO req, Worker worker, List<String> commandoCodes) {
        StringBuilder sb = new StringBuilder("    select * from ( select a.*,b.commando_state  from (  SELECT     a.*,    \n"
                + "                    \n"
                + "                       a.create_by applicant,   \n"
                + "                          a.create_time applyTime,     \n"
                + "                           a.GROUP_ORG_NAME orgName  FROM   \n"
                + "                              ycq_qzh_commando a    \n"
                + "                                LEFT JOIN tsm_worker b  \n"
                + "                                ON a.create_id = b.login_id WHERE    \n"
                + "                                  a.DEL_FLAG=0 and \n"
                + "                                  a.STATE!=1       ) as a LEFT join (                          \n"
                + "                                  \n"
                + "                                  \n"
                + "                                  \n"
                + "                                  select commando_code,commando_state from(   SELECT   a.commando_code, \n"
                + "                 (CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6 and k.label = 'true' )  THEN '冻结' ELSE '正常' END ) commando_state \n"
                + "                 FROM     \n"
                + "                 ycq_qzh_commando a            \n"
                + "                 LEFT JOIN tsm_worker b ON a.create_id = b.login_id   \n"
                + "                 LEFT JOIN (   \n"
                + "                    select a.commando_code , count(parent_id) mien_count ,max(b.create_time) max_time ,   \n"
                + "                     (case when ((count(parent_id) = 0 and  TIMESTAMPDIFF( MONTH, max(a.create_time) , now()) > 6)   \n"
                + "                     or (count(parent_id) > 0 and  TIMESTAMPDIFF( MONTH, max(b.create_time) , now()) > 6))   \n"
                + "                     then 'true' else 'false' end) label   \n"
                + "                     from ycq_qzh_commando a   \n"
                + "                     LEFT JOIN YCQ_QZH_ACTIVITY b on a.id = b.parent_id and b.activity_type = 1   and a.del_flag = 0 and b.del_flag = 0   \n"
                + "                     group by a.commando_code   \n"
                + "                 ) k ON k.commando_code = a.commando_code  ) where 1=1 ) b on a.commando_code=b.commando_code ) where 1=1   ");

        List<Object> params = new ArrayList<>();

        if (null != commandoCodes && commandoCodes.size() > 0) {
            String collect = commandoCodes.stream().map(code -> "\'" + code.toString() + "\'").collect(Collectors.joining(",", "(", ") "));
            sb.append("  and COMMANDO_CODE in  " + collect + "   ");
        }

             return jdbcTemplate.queryForBeans(QueryCommandoRespDTO.class, sb.toString(), params.toArray());

    }


    public PageRsp<QueryCommandoRespDTO> pageListByApplicant(PageReq<QueryCommandoReqDTO> req, Worker worker) {
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        //申请人
        sb.append(" and a.create_id = ? ");
        sbCount.append(" and a.create_id = ? ");
        params.add(worker.getLoginId());
        sb.append("   order by CREATE_TIME desc  ");
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<QueryCommandoRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(QueryCommandoRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<QueryCommandoRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }


    public void deleteById(Long id, int updateId, String updateBy) {
        StringBuilder sb = new StringBuilder(DELETE_SQL).append(id);
        jdbcTemplate.update(sb.toString(), updateId, updateBy, new Date());
    }

//    public PageRsp<CommandoRankListDetilRespDTO> rankingCommandoPage(PageReq<CommandoRankListReqDTO> req, Worker currWorker) {
//
//        List<Object> params = new ArrayList<>();
//        params.add(currWorker.getLoginId());
//        params.add(currWorker.getMobile());
//        YcqQzhCommandoPoint commandoPoint = jdbcTemplate.queryForBean(YcqQzhCommandoPoint.class, GET_ONE_SQL, params.toArray());
//
//        StringBuilder sb = new StringBuilder(RANK_SQL);
//        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
//        List<Object> pageParams = new ArrayList<>();
//
//        if (Objects.nonNull(req.getQuery().getName())) {
//            sb.append(" and  COMMANDO_NAME like concat('%',?,'%') ");
//            sbCount.append(" and  COMMANDO_NAME like concat('%',?,'%') ");
//            params.add(req.getQuery().getName());
//        }
//
//        sb.append(" ORDER BY POINT DESC ");
//        int total = jdbcTemplate.queryForInt(sbCount.toString(), pageParams.toArray());
//        List<CommandoRankListDetilRespDTO> result;
//        String dbType = jdbcTemplate.getDbType();
//        if (total > 0) {
//            //根据数据库类型设置limit
//            SqlUtil.appendLimit(dbType, pageParams, sb, req);
//            result = jdbcTemplate.queryForBeans(CommandoRankListDetilRespDTO.class, sb.toString(), pageParams.toArray());
//            if (commandoPoint != null) {
//                CommandoRankListDetilRespDTO dto = new CommandoRankListDetilRespDTO();
//                dto.setCommandoName(commandoPoint.getCommandoName());
//                dto.setCollectNum(commandoPoint.getPoint() + "");
//                if (result.size() < 2) {
//                    result.add(1, dto);
//                }
//                if (result.size() < 1) {
//                    result.add(0, dto);
//                } else {
//                    result.add(2, dto);
//                }
//            }
//            for (int i = 0; i < result.size(); i++) {
//                result.get(i).setRankNum(i + 1 + (req.getPageNo() - 1) * req.getPageSize());
//            }
//        } else {
//            result = new ArrayList<>(0);
//        }
//        PageRsp<CommandoRankListDetilRespDTO> page = new PageRsp<>();
//        page.setPageNo(req.getPageNo());
//        page.setPageSize(req.getPageSize());
//        page.setTotalNum(total);
//        page.setResult(result);
//        // 为手机计算是否由下一页
//        page.next();
//        return page;
//    }

    public PageRsp<CommandoRankListDetilRespDTO> selectFcHd(PageReq<CommandoRankListReqDTO> req, Worker currWorker) {

        List<Object> params = new ArrayList<>();
        String sbSQL = " SELECT to_char(now(), 'yyyy-mm-dd HH:mm:ss') create_time,b.COMMANDO_CODE,b.COMMANDO_NAME,count(a.ID) collectNum FROM YCQ_QZH_COMMANDO b left join YCQ_QZH_ACTIVITY a on b.ID = a.PARENT_ID and a.DEL_FLAG = 0 and a.ACTIVITY_TYPE = 1\n" +
                "where b.DEL_FLAG = 0 and b.state =3 group by b.COMMANDO_CODE,b.COMMANDO_NAME ";
        String sbCountSQL = " SELECT COUNT(1) FROM ( SELECT b.COMMANDO_CODE,b.COMMANDO_NAME,count(a.ID) point FROM YCQ_QZH_COMMANDO b left join YCQ_QZH_ACTIVITY a on b.ID = a.PARENT_ID and a.DEL_FLAG = 0 and a.ACTIVITY_TYPE = 1\n" +
                "where b.DEL_FLAG = 0 and b.state =3 [CD] group by b.COMMANDO_CODE,b.COMMANDO_NAME )";
        StringBuilder sb = new StringBuilder(sbSQL);
        StringBuilder sbCount = new StringBuilder(sbCountSQL);
        List<Object> pageParams = new ArrayList<>();
        int total = 0;
        if (Objects.nonNull(req.getQuery().getName())) {
            sb.append(" and  COMMANDO_NAME like concat('%',?,'%') ");
            pageParams.add(req.getQuery().getName());
            String replace = sbCount.toString().replace("[CD]", " and  COMMANDO_NAME like concat('%',?,'%') ");
            total = jdbcTemplate.queryForInt(replace, pageParams.toArray());
        } else {
            String replace = sbCount.toString().replace("[CD]", "");
            total = jdbcTemplate.queryForInt(replace, pageParams.toArray());
        }
        sb.append(" ORDER BY collectNum DESC ");
        List<CommandoRankListDetilRespDTO> result;
        String dbType = jdbcTemplate.getDbType();

        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(CommandoRankListDetilRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }

        PageRsp<CommandoRankListDetilRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }


    /**
     * @description: 查询 队长，申请人，    是不是再其他 突击队里面了  突击队名称不重复，队长再一个突击队里
     * @param: [saveCommandoReqDTO, currWorker, code]
     * @return: com.crunii.micro.domain.entity.main.YcqQzhCommando
     * @author 13760
     * @date: 2023/9/18 14:51
     */
    public YcqQzhCommando checkData(SaveCommandoReqDTO saveCommandoReqDTO, Worker currWorker, String code) {

        String sb = "select * from YCQ_QZH_COMMANDO  where 1=1 and DEL_FLAG=0  and (state=2 or state=3 ) and TEAM_LEADER_CARD = ?  limit 1 ";
        List<Object> params = new ArrayList<>();
        //  params.add(currWorker.getLoginId());
        params.add(saveCommandoReqDTO.getTeamLeaderCard());
        return jdbcTemplate.queryForBean(YcqQzhCommando.class, sb, params.toArray());

    }

    /**
     * @description: t 突击队申请人 草稿查询
     * @param: [token, dto]
     * @return: com.crunii.micro.common.dto.Result
     * @author 13760
     * @date: 2023/9/18 14:54
     */
    public CommandoDetailsRespDTO findDraftByWorker(Worker currWorker) {


        String sb = "select * from YCQ_QZH_COMMANDO a where 1=1 and state=1 and  CREATE_ID=? limit 1 ";
        List<Object> params = new ArrayList<>();
        params.add(currWorker.getLoginId());
        return jdbcTemplate.queryForBean(CommandoDetailsRespDTO.class, sb, params.toArray());

    }

    /**
     * @description: 根据comandcode查看以前是不是有变更记录了
     * @param: [code]
     * @return: com.crunii.micro.domain.entity.main.YcqQzhCommandoLog
     * @author 13760
     * @date: 2023/9/19 14:28
     */
    public YcqQzhCommandoLog findMaxLogByCode(String code) {
        String sb = "select * from YCQ_QZH_COMMANDO_LOG where 1=1 and CREATE_TIME=( select max(CREATE_TIME)  from YCQ_QZH_COMMANDO_LOG  where 1=1 and commando_code=?  ) and commando_code=?  limit 1 ";
        List<Object> params = new ArrayList<>();
        if (code != null) {
            params.add(code);
            params.add(code);
        }
        return jdbcTemplate.queryForBean(YcqQzhCommandoLog.class, sb, params.toArray());
    }


    /**
     * @description: 申请人查询变更记录
     * @param: [dto]
     * @return: com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO
     * @author 13760
     * @date: 2023/9/19 19:40
     */
    public PageRsp<QueryCommandoChangeRespDTO> findChangeLeader(PageReq<QueryCommandoReqDTO> req, Worker worker, int type, List<String> commandoCodes) {

        //  String sb = "select a.id,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME,\n" + "a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE,\n" + "b.BATCH_NO_CHANGE\n" + "\n" + " from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1\n" + "and a.COMMANDO_CODE=b.COMMANDO_CODE\n" + "and b.FLOW_TYPE=2\n" + "and b.FLOW_CREATE_TIME=  (  select max(b.FLOW_CREATE_TIME) from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1\n" + "    and a.COMMANDO_CODE=b.COMMANDO_CODE\n" + "    and a.BATCH_NO_CHANGE=b.BATCH_NO_CHANGE\n" + "    and b.FLOW_TYPE=2  and a.DEL_FLAG=1  )   \n ";

//        StringBuffer sb = new StringBuffer("  select *，concat(COMMANDO_CODE,'-',BATCH_NO_CHANGE,APPLY_CHANGE_NUM\n"
//                + ") as number_id from (select a.id,a.distribution,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME,\n"
//                + "a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.WORK_SITUATION,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE,\n"
//                + "b.BATCH_NO_CHANGE,b.APPLY_CHANGE_NUM,a.CREATE_ID ,a.GROUP_ORG_ID\n"
//                + " from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1  and a.DEL_FLAG=0 \n"
//                + "and a.COMMANDO_CODE=b.COMMANDO_CODE\n"
//                + "and b.FLOW_TYPE=2 \n"
//                + "and b.id in (select id from  YCQ_QZH_COMMANDO_FLOW  where (FLOW_CREATE_TIME,APPLY_CHANGE_NUM,COMMANDO_CODE) in (\n"
//                + "select max(FLOW_CREATE_TIME),APPLY_CHANGE_NUM ,COMMANDO_CODE   from YCQ_QZH_COMMANDO_FLOW  where flow_type=2\n"
//                + "group by APPLY_CHANGE_NUM,COMMANDO_CODE\n"
//                + ")   ) ) where 1=1   ");


        StringBuilder sb = new StringBuilder("      select * from (  select a.*,b.commando_state from (\n"
                + "                  select *，concat(COMMANDO_CODE,'-',BATCH_NO_CHANGE,APPLY_CHANGE_NUM \n"
                + "                 ) as number_id from (select a.id,a.distribution,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME, \n"
                + "                 a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.WORK_SITUATION,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE, \n"
                + "                 b.BATCH_NO_CHANGE,b.APPLY_CHANGE_NUM,a.CREATE_ID ,a.GROUP_ORG_ID \n"
                + "                  from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1  and a.DEL_FLAG=0 \n"
                + "                 and a.COMMANDO_CODE=b.COMMANDO_CODE \n"
                + "                 and b.FLOW_TYPE=2 \n"
                + "                 and b.id in (select id from  YCQ_QZH_COMMANDO_FLOW  where (FLOW_CREATE_TIME,APPLY_CHANGE_NUM,COMMANDO_CODE) in ( \n"
                + "                 select max(FLOW_CREATE_TIME),APPLY_CHANGE_NUM ,COMMANDO_CODE   from YCQ_QZH_COMMANDO_FLOW  where flow_type=2 \n"
                + "                 group by APPLY_CHANGE_NUM,COMMANDO_CODE \n"
                + "                 )   ) ) where 1=1  \n"
                + "                 ) as a  left join ( \n"
                + "                  select commando_code,commando_state from(   SELECT   a.commando_code,  \n"
                + "                                (CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6 and k.label = 'true' )  THEN '冻结' ELSE '正常' END ) commando_state  \n"
                + "                                FROM      \n"
                + "                                ycq_qzh_commando a             \n"
                + "                                LEFT JOIN tsm_worker b ON a.create_id = b.login_id    \n"
                + "                                LEFT JOIN (    \n"
                + "                                   select a.commando_code , count(parent_id) mien_count ,max(b.create_time) max_time ,    \n"
                + "                                    (case when ((count(parent_id) = 0 and  TIMESTAMPDIFF( MONTH, max(a.create_time) , now()) > 6)    \n"
                + "                                    or (count(parent_id) > 0 and  TIMESTAMPDIFF( MONTH, max(b.create_time) , now()) > 6))    \n"
                + "                                    then 'true' else 'false' end) label    \n"
                + "                                    from ycq_qzh_commando a    \n"
                + "                                    LEFT JOIN YCQ_QZH_ACTIVITY b on a.id = b.parent_id and b.activity_type = 1   and a.del_flag = 0 and b.del_flag = 0    \n"
                + "                                    group by a.commando_code    \n"
                + "                                ) k ON k.commando_code = a.commando_code  ) where 1=1 \n"
                + "                 ) as b on a.commando_code=b.commando_code )  where 1=1   ");

        List<Object> params = new ArrayList<>();

        //type=1 创建人查询
        if (type == 1) {
            sb.append(" and  create_id=?   ");
            params.add(worker.getLoginId());
        }
        //type=2   审核人查组织下的
        if (null != commandoCodes && commandoCodes.size() > 0) {
            String collect = commandoCodes.stream().map(code -> "\'" + code.toString() + "\'").collect(Collectors.joining(",", "(", ") "));
            sb.append("  and COMMANDO_CODE in  " + collect + "   ");
        }
        if (StrUtil.isNotBlank(req.getQuery().getCommandoName())) {
            sb.append(" and commando_name like concat('%',?,'%') ");
            params.add(req.getQuery().getCommandoName());
        }
        if (Objects.nonNull(req.getQuery().getGroupOrgId())) {
            sb.append(" and GROUP_ORG_ID = ? ");
            params.add(req.getQuery().getGroupOrgId());
        }
        if (Objects.nonNull(req.getQuery().getState())) {
            if (req.getQuery().getState() == 4) {
                sb.append(" and (state = 4 or state=5 or state=6 ) ");
            } else {
                sb.append(" and state = ? ");
                params.add(req.getQuery().getState());
            }
        }
        if (Objects.nonNull(req.getQuery().getType())) {
            sb.append(" and type = ? ");
            params.add(req.getQuery().getType());
        }
        if (Objects.nonNull(req.getQuery().getDistribution())) {
            sb.append(" and a.distribution = ? ");
            params.add(req.getQuery().getDistribution());
        }
        if (Objects.nonNull(req.getQuery().getStartTime())) {
            sb.append(" and  to_char(apply_time,'yyyy-mm-dd') >=? ");
            params.add(req.getQuery().getStartTime());
        }
        if (Objects.nonNull(req.getQuery().getEndTime())) {
            sb.append(" and  to_char(apply_time,'yyyy-mm-dd') <=? ");
            params.add(req.getQuery().getEndTime());
        }

        //申请人
        if (Objects.nonNull(req.getQuery().getApplicant())) {
            sb.append(" and  applicant like concat('%',?,'%') ");
            params.add(req.getQuery().getApplicant());
        }
        if (Objects.nonNull(req.getQuery().getHaveThreeYearPlan())) {
            sb.append(" and  HAVE_THREE_YEAR_PLAN like concat('%',?,'%') ");
            params.add(req.getQuery().getHaveThreeYearPlan());
        }
        sb.append("  order by APPLY_TIME desc  ");
        String sbCount = "  select count(*) from (  " + sb + "   )   ";
        int total = jdbcTemplate.queryForInt(sbCount, params.toArray());
        List<QueryCommandoChangeRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(QueryCommandoChangeRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<QueryCommandoChangeRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;

    }


    /**
     * @description: 审批人查询变更记录
     * @param: [dto]
     * @return: com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO
     * @author 13760
     * @date: 2023/9/19 19:40
     */
    public PageRsp<QueryCommandoChangeRespDTO> findChangeForAudit(PageReq<QueryCommandoReqDTO> req, Worker worker, int type, List<String> commandoCodes) {

        //  String sb = "select a.id,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME,\n" + "a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE,\n" + "b.BATCH_NO_CHANGE\n" + "\n" + " from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1\n" + "and a.COMMANDO_CODE=b.COMMANDO_CODE\n" + "and b.FLOW_TYPE=2\n" + "and b.FLOW_CREATE_TIME=  (  select max(b.FLOW_CREATE_TIME) from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1\n" + "    and a.COMMANDO_CODE=b.COMMANDO_CODE\n" + "    and a.BATCH_NO_CHANGE=b.BATCH_NO_CHANGE\n" + "    and b.FLOW_TYPE=2  and a.DEL_FLAG=1  )   \n ";

        StringBuilder sb = new StringBuilder("      select * from (  select a.*,b.commando_state from (\n"
                + "                  select *，concat(COMMANDO_CODE,'-',BATCH_NO_CHANGE,APPLY_CHANGE_NUM \n"
                + "                 ) as number_id from (select a.id,a.distribution,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME, \n"
                + "                 a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.WORK_SITUATION,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE, \n"
                + "                 b.BATCH_NO_CHANGE,b.APPLY_CHANGE_NUM,a.CREATE_ID ,a.GROUP_ORG_ID \n"
                + "                  from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1  and a.DEL_FLAG=0 \n"
                + "                 and a.COMMANDO_CODE=b.COMMANDO_CODE \n"
                + "                 and b.FLOW_TYPE=2 \n"
                + "                 and b.id in (select id from  YCQ_QZH_COMMANDO_FLOW  where (FLOW_CREATE_TIME,APPLY_CHANGE_NUM,COMMANDO_CODE) in ( \n"
                + "                 select max(FLOW_CREATE_TIME),APPLY_CHANGE_NUM ,COMMANDO_CODE   from YCQ_QZH_COMMANDO_FLOW  where flow_type=2 \n"
                + "                 group by APPLY_CHANGE_NUM,COMMANDO_CODE \n"
                + "                 )   ) ) where 1=1  \n"
                + "                 ) as a  left join ( \n"
                + "                  select commando_code,commando_state from(   SELECT   a.commando_code,  \n"
                + "                                (CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6 and k.label = 'true' )  THEN '冻结' ELSE '正常' END ) commando_state  \n"
                + "                                FROM      \n"
                + "                                ycq_qzh_commando a             \n"
                + "                                LEFT JOIN tsm_worker b ON a.create_id = b.login_id    \n"
                + "                                LEFT JOIN (    \n"
                + "                                   select a.commando_code , count(parent_id) mien_count ,max(b.create_time) max_time ,    \n"
                + "                                    (case when ((count(parent_id) = 0 and  TIMESTAMPDIFF( MONTH, max(a.create_time) , now()) > 6)    \n"
                + "                                    or (count(parent_id) > 0 and  TIMESTAMPDIFF( MONTH, max(b.create_time) , now()) > 6))    \n"
                + "                                    then 'true' else 'false' end) label    \n"
                + "                                    from ycq_qzh_commando a    \n"
                + "                                    LEFT JOIN YCQ_QZH_ACTIVITY b on a.id = b.parent_id and b.activity_type = 1   and a.del_flag = 0 and b.del_flag = 0    \n"
                + "                                    group by a.commando_code    \n"
                + "                                ) k ON k.commando_code = a.commando_code  ) where 1=1 \n"
                + "                 ) as b on a.commando_code=b.commando_code )  where 1=1   ");

        List<Object> params = new ArrayList<>();

        //type=1 创建人查询
        if (type == 1) {
            sb.append(" and  create_id=?   ");
            params.add(worker.getLoginId());
        }
        //type=2   审核人查组织下的
        if (null != commandoCodes && commandoCodes.size() > 0) {
            String collect = commandoCodes.stream().map(code -> "\'" + code.toString() + "\'").collect(Collectors.joining(",", "(", ") "));
            sb.append("  and COMMANDO_CODE in  " + collect + "   ");
        }
        if (StrUtil.isNotBlank(req.getQuery().getCommandoName())) {
            sb.append(" and commando_name like concat('%',?,'%') ");
            params.add(req.getQuery().getCommandoName());
        }
        if (Objects.nonNull(req.getQuery().getGroupOrgId())) {
            sb.append(" and GROUP_ORG_ID = ? ");
            params.add(req.getQuery().getGroupOrgId());
        }
        if (Objects.nonNull(req.getQuery().getState())) {
            if (req.getQuery().getState() == 4) {
                sb.append(" and (state = 4 or state=5 or state=6 ) ");
            } else {
                sb.append(" and state = ? ");
                params.add(req.getQuery().getState());
            }
        }
        if (Objects.nonNull(req.getQuery().getType())) {
            sb.append(" and type = ? ");
            params.add(req.getQuery().getType());
        }
        if (Objects.nonNull(req.getQuery().getDistribution())) {
            sb.append(" and a.distribution = ? ");
            params.add(req.getQuery().getDistribution());
        }
        if (Objects.nonNull(req.getQuery().getStartTime())) {
            sb.append(" and  to_char(apply_time,'yyyy-mm-dd') >=? ");
            params.add(req.getQuery().getStartTime());
        }
        if (Objects.nonNull(req.getQuery().getEndTime())) {
            sb.append(" and  to_char(apply_time,'yyyy-mm-dd') <=? ");
            params.add(req.getQuery().getEndTime());
        }
        if (Objects.nonNull(req.getQuery().getCommandoState())) {
            sb.append(" and  commando_state like concat('%',?,'%') ");
            params.add(req.getQuery().getCommandoState());
        }
        //申请人
        if (Objects.nonNull(req.getQuery().getApplicant())) {
            sb.append(" and  applicant like concat('%',?,'%') ");
            params.add(req.getQuery().getApplicant());
        }
        if (Objects.nonNull(req.getQuery().getHaveThreeYearPlan())) {
            sb.append(" and  HAVE_THREE_YEAR_PLAN like concat('%',?,'%') ");
            params.add(req.getQuery().getHaveThreeYearPlan());
        }
        sb.append("  order by APPLY_TIME desc  ");


        String sbCount = "  select count(*) from (  " + sb + "   )   ";
        int total = jdbcTemplate.queryForInt(sbCount, params.toArray());
        List<QueryCommandoChangeRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(QueryCommandoChangeRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<QueryCommandoChangeRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return page;

    }





    /**
     * @description: 审批人查询变更记录
     * @param: [dto]
     * @return: com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO
     * @author 13760
     * @date: 2023/9/19 19:40
     */
    public List<QueryCommandoChangeRespDTO> findChangeForAuditList(QueryCommandoReqDTO req, Worker worker, int type, List<String> commandoCodes) {

        //  String sb = "select a.id,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME,\n" + "a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE,\n" + "b.BATCH_NO_CHANGE\n" + "\n" + " from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1\n" + "and a.COMMANDO_CODE=b.COMMANDO_CODE\n" + "and b.FLOW_TYPE=2\n" + "and b.FLOW_CREATE_TIME=  (  select max(b.FLOW_CREATE_TIME) from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1\n" + "    and a.COMMANDO_CODE=b.COMMANDO_CODE\n" + "    and a.BATCH_NO_CHANGE=b.BATCH_NO_CHANGE\n" + "    and b.FLOW_TYPE=2  and a.DEL_FLAG=1  )   \n ";

        StringBuilder sb = new StringBuilder("      select * from (  select a.*,b.commando_state from (\n"
                + "                  select *，concat(COMMANDO_CODE,'-',BATCH_NO_CHANGE,APPLY_CHANGE_NUM \n"
                + "                 ) as number_id from (select a.id,a.distribution,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME, \n"
                + "                 a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.WORK_SITUATION,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE, \n"
                + "                 b.BATCH_NO_CHANGE,b.APPLY_CHANGE_NUM,a.CREATE_ID ,a.GROUP_ORG_ID \n"
                + "                  from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1  and a.DEL_FLAG=0 \n"
                + "                 and a.COMMANDO_CODE=b.COMMANDO_CODE \n"
                + "                 and b.FLOW_TYPE=2 \n"
                + "                 and b.id in (select id from  YCQ_QZH_COMMANDO_FLOW  where (FLOW_CREATE_TIME,APPLY_CHANGE_NUM,COMMANDO_CODE) in ( \n"
                + "                 select max(FLOW_CREATE_TIME),APPLY_CHANGE_NUM ,COMMANDO_CODE   from YCQ_QZH_COMMANDO_FLOW  where flow_type=2 \n"
                + "                 group by APPLY_CHANGE_NUM,COMMANDO_CODE \n"
                + "                 )   ) ) where 1=1  \n"
                + "                 ) as a  left join ( \n"
                + "                  select commando_code,commando_state from(   SELECT   a.commando_code,  \n"
                + "                                (CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6 and k.label = 'true' )  THEN '冻结' ELSE '正常' END ) commando_state  \n"
                + "                                FROM      \n"
                + "                                ycq_qzh_commando a             \n"
                + "                                LEFT JOIN tsm_worker b ON a.create_id = b.login_id    \n"
                + "                                LEFT JOIN (    \n"
                + "                                   select a.commando_code , count(parent_id) mien_count ,max(b.create_time) max_time ,    \n"
                + "                                    (case when ((count(parent_id) = 0 and  TIMESTAMPDIFF( MONTH, max(a.create_time) , now()) > 6)    \n"
                + "                                    or (count(parent_id) > 0 and  TIMESTAMPDIFF( MONTH, max(b.create_time) , now()) > 6))    \n"
                + "                                    then 'true' else 'false' end) label    \n"
                + "                                    from ycq_qzh_commando a    \n"
                + "                                    LEFT JOIN YCQ_QZH_ACTIVITY b on a.id = b.parent_id and b.activity_type = 1   and a.del_flag = 0 and b.del_flag = 0    \n"
                + "                                    group by a.commando_code    \n"
                + "                                ) k ON k.commando_code = a.commando_code  ) where 1=1 \n"
                + "                 ) as b on a.commando_code=b.commando_code )  where 1=1   ");

        List<Object> params = new ArrayList<>();

        //type=1 创建人查询
        if (type == 1) {
            sb.append(" and  create_id=?   ");
            params.add(worker.getLoginId());
        }
        //type=2   审核人查组织下的
        if (null != commandoCodes && commandoCodes.size() > 0) {
            String collect = commandoCodes.stream().map(code -> "\'" + code.toString() + "\'").collect(Collectors.joining(",", "(", ") "));
            sb.append("  and COMMANDO_CODE in  " + collect + "   ");
        }
        sb.append("  order by APPLY_TIME desc  ");
        return jdbcTemplate.queryForBeans(QueryCommandoChangeRespDTO.class, sb.toString(), params.toArray());
    }





    /**
     * @description: 审核人查询变更记录
     * @param: [dto]
     * @return: com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO
     * @author 13760
     * @date: 2023/9/19 19:40
     */
    public List<QueryCommandoChangeRespDTO> findChangeLeaderByAudit(PageReq<QueryCommandoReqDTO> req, Worker worker, int type, List<String> commandoCodes) {

        //  String sb = "select a.id,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME,\n" + "a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE,\n" + "b.BATCH_NO_CHANGE\n" + "\n" + " from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1\n" + "and a.COMMANDO_CODE=b.COMMANDO_CODE\n" + "and b.FLOW_TYPE=2\n" + "and b.FLOW_CREATE_TIME=  (  select max(b.FLOW_CREATE_TIME) from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1\n" + "    and a.COMMANDO_CODE=b.COMMANDO_CODE\n" + "    and a.BATCH_NO_CHANGE=b.BATCH_NO_CHANGE\n" + "    and b.FLOW_TYPE=2  and a.DEL_FLAG=1  )   \n ";

        StringBuffer sb = new StringBuffer("  select * from (select a.id,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME,a.distribution,\n"
                + "a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.WORK_SITUATION,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE,\n"
                + "b.BATCH_NO_CHANGE,b.APPLY_CHANGE_NUM,a.CREATE_ID ,a.GROUP_ORG_ID\n"
                + " from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1  and a.DEL_FLAG=0 \n"
                + "and a.COMMANDO_CODE=b.COMMANDO_CODE\n"
                + "and b.FLOW_TYPE=2 \n"
                + "and b.id in (  select id from  YCQ_QZH_COMMANDO_FLOW  where  (FLOW_CREATE_TIME,COMMANDO_CODE) in (\n"
                + "select max(FLOW_CREATE_TIME),COMMANDO_CODE from (\n"
                + "select * from YCQ_QZH_COMMANDO_FLOW where (APPLY_CHANGE_NUM,COMMANDO_CODE)\n"
                + "in (\n"
                + "select max(APPLY_CHANGE_NUM),COMMANDO_CODE   from YCQ_QZH_COMMANDO_FLOW   where flow_type=2\n"
                + "group by COMMANDO_CODE\n"
                + ")) group by COMMANDO_CODE)   ) ) where 1=1     ");

        List<Object> params = new ArrayList<>();

        //type=1 创建人查询
        if (type == 1) {
            sb.append(" and  create_id=?   ");
            params.add(worker.getLoginId());
        }
        //type=2   审核人查组织下的
        if (null != commandoCodes && commandoCodes.size() > 0) {
            String collect = commandoCodes.stream().map(code -> "\'" + code.toString() + "\'").collect(Collectors.joining(",", "(", ") "));
            sb.append("  and COMMANDO_CODE in  " + collect + "   ");
        }
        if (StrUtil.isNotBlank(req.getQuery().getCommandoName())) {
            sb.append(" and commando_name like concat('%',?,'%') ");
            params.add(req.getQuery().getCommandoName());
        }
        if (Objects.nonNull(req.getQuery().getGroupOrgId())) {
            sb.append(" and GROUP_ORG_ID = ? ");
            params.add(req.getQuery().getGroupOrgId());
        }
        if (Objects.nonNull(req.getQuery().getState())) {
            if (req.getQuery().getState() == 4) {
                sb.append(" and (state = 4 or state=5) ");
            } else {
                sb.append(" and state = ? ");
                params.add(req.getQuery().getState());
            }
        }
        if (Objects.nonNull(req.getQuery().getType())) {
            sb.append(" and type = ? ");
            params.add(req.getQuery().getType());
        }
        if (Objects.nonNull(req.getQuery().getDistribution())) {
            sb.append(" and a.distribution = ? ");
            params.add(req.getQuery().getDistribution());
        }
        if (Objects.nonNull(req.getQuery().getStartTime())) {
            sb.append(" and  to_char(apply_time,'yyyy-mm-dd') >=? ");
            params.add(req.getQuery().getStartTime());
        }
        if (Objects.nonNull(req.getQuery().getEndTime())) {
            sb.append(" and  to_char(apply_time,'yyyy-mm-dd') <=? ");
            params.add(req.getQuery().getEndTime());
        }

        //申请人
        if (Objects.nonNull(req.getQuery().getApplicant())) {
            sb.append(" and  applicant like concat('%',?,'%') ");
            params.add(req.getQuery().getApplicant());
        }
        if (Objects.nonNull(req.getQuery().getHaveThreeYearPlan())) {
            sb.append(" and  HAVE_THREE_YEAR_PLAN like concat('%',?,'%') ");
            params.add(req.getQuery().getHaveThreeYearPlan());
        }
        sb.append("  order by APPLY_TIME desc  ");
        return jdbcTemplate.queryForBeans(QueryCommandoChangeRespDTO.class, sb.toString(), params.toArray());

    }

//    public void rankingCommando() {
//        String SQL = " select  b.COMMANDO_CODE,   b.COMMANDO_NAME, (a.point + b.point) point from  ( SELECT  b.COMMANDO_CODE,    b.COMMANDO_NAME,    count(a.ID) point   FROM  YCQ_QZH_COMMANDO b left join YCQ_QZH_ACTIVITY a on b.ID = a.PARENT_ID   and a.DEL_FLAG = 1  and a.ACTIVITY_TYPE = 1 where  b.DEL_FLAG = 1  group by   b.COMMANDO_CODE, b.COMMANDO_NAME ) a, ( SELECT     b.COMMANDO_CODE,  b.COMMANDO_NAME,   count(c.ID) point   FROM   YCQ_QZH_COMMANDO b   left join TSM_WORKER a on (    b.CREATE_ID = a.LOGIN_ID  or a.MOBILE = b.TEAM_LEADER_PHONE )   and a.STATE = 1 left join TSM_LOGIN_LOG c on a.MOBILE = c.MOBILE where    b.DEL_FLAG = 1  group by  b.COMMANDO_CODE,  b.COMMANDO_NAME   ) b where a.COMMANDO_CODE = b.COMMANDO_CODE ";
//        List<YcqQzhCommandoPoint> list = jdbcTemplate.queryForBeans(YcqQzhCommandoPoint.class, SQL);
//        for (YcqQzhCommandoPoint commando : list) {
//            commando.setCreateTime(new Date());
//        }
//        ycqQzhCommandoPointRepo.deleteAll();
//        ycqQzhCommandoPointRepo.saveAll(list);
//    }

    public void rankingCommando() {
        String DEL_SQL = " DELETE FROM YCQ_QZH_COMMANDO_POINT ";
        String INSERT_SQL = " INSERT into YCQ_QZH_COMMANDO_POINT(id ,COMMANDO_CODE,COMMANDO_NAME,point,CREATE_TIME) select ROWNUM, b.COMMANDO_CODE,  b.COMMANDO_NAME, (a.point + b.point) point,now() from  ( SELECT  b.COMMANDO_CODE,    b.COMMANDO_NAME,    count(a.ID) point   FROM  YCQ_QZH_COMMANDO b left join YCQ_QZH_ACTIVITY a on b.ID = a.PARENT_ID   and a.DEL_FLAG = 0  and a.ACTIVITY_TYPE = 1 where  b.DEL_FLAG = 0  group by   b.COMMANDO_CODE, b.COMMANDO_NAME ) a, ( SELECT     b.COMMANDO_CODE,  b.COMMANDO_NAME,   count(c.ID) point   FROM   YCQ_QZH_COMMANDO b   left join TSM_WORKER a on (    b.CREATE_ID = a.LOGIN_ID  or a.MOBILE = b.TEAM_LEADER_PHONE )   and a.STATE = 1 left join TSM_LOGIN_LOG c on a.MOBILE = c.MOBILE where    b.DEL_FLAG = 0  group by  b.COMMANDO_CODE,  b.COMMANDO_NAME   ) b where a.COMMANDO_CODE = b.COMMANDO_CODE ";

        jdbcTemplate.batchUpdate(DEL_SQL);
        jdbcTemplate.batchUpdate(INSERT_SQL);

    }


    /**
     * @description: 根据版本号找到对应是变更记录
     * @param: []
     * @return: com.crunii.micro.domain.entity.main.YcqQzhCommandoFlow
     * @author 13760
     * @date: 2023/9/20 14:36
     */
    public YcqQzhCommandoLog findByCodeAndBatchChange(String ycqQzhCommando, Integer batchChange) {
        StringBuffer sb = new StringBuffer("select * from YCQ_QZH_COMMANDO_LOG where 1=1  ");
        List<Object> params = new ArrayList<>();
        if (ycqQzhCommando != null && batchChange != null) {
            sb.append("  and  COMMANDO_CODE=? and BATCH_NO=? ");
            params.add(ycqQzhCommando);
            params.add(batchChange);
        }
        //TODO 防止 产生2条数据
        sb.append("   limit 1 ");
        return jdbcTemplate.queryForBean(YcqQzhCommandoLog.class, sb.toString(), params.toArray());

    }

    /**
     * @description: 突击队流程查询
     * @param: []
     * @return: com.crunii.micro.domain.entity.main.YcqQzhCommandoFlow
     * @author 13760
     * @date: 2023/9/20 14:36
     */
    public List<CommandoFlowRespDTO> findFlowByCommandoCode(String commandoCode) {
        StringBuffer sb = new StringBuffer("select \n" + "commando_code,\n" + "FLOW_NAME as deal_flow_name,\n" + "flow_worker as deal_worker,\n" + "flow_worker_org as deal_org_name,\n" + "FLOW_CREATE_TIME as deal_time,\n" + "DEAL_TEXT as deal_advice,\n" + " state as deal_state,"
                + "BATCH_NO_CHANGE,\n"
                + "BATCH_NO_APPLY, \n"
                + "APPLY_CHANGE_NUM" + " from YCQ_QZH_COMMANDO_FLOW where 1=1  ");
        List<Object> params = new ArrayList<>();
        if (null != commandoCode) {
            sb.append("  and  COMMANDO_CODE=?  ");
            params.add(commandoCode);

        }
        sb.append("  order by FLOW_CREATE_TIME ");
        return jdbcTemplate.queryForBeans(CommandoFlowRespDTO.class, sb.toString(), params.toArray());
    }

    /**
     * @description: 查询当前申请人 正在申请的单子条数
     * @param: [loginCode]
     * @return: int
     * @author 13760
     * @date: 2023/9/21 10:27
     */
    public int findStateById(Integer loginCode) {
        StringBuilder sb = new StringBuilder("  select count(*) from YCQ_QZH_COMMANDO where 1=1 and state=2  and  DEL_FLAG=0    ");
        List<Object> params = new ArrayList<>();
        if (null != loginCode) {
            sb.append("  and CREATE_ID=?   ");
            params.add(loginCode);
        }
        return jdbcTemplate.queryForInt(sb.toString(), params.toArray());
    }

    /**
     * @description: 查询之前的申报流程
     * @param: [commandoCode, batchNoApply]
     * @return: com.crunii.micro.domain.entity.main.YcqQzhCommandoFlow
     * @author 13760
     * @date: 2023/9/21 16:36
     */
    public YcqQzhCommandoFlow findFlow(String commandoCode, Integer batchNoApply, Integer batchNoChange, int type) {

        StringBuffer sb = new StringBuffer("select * from YCQ_QZH_COMMANDO_FLOW where 1=1 and FLOW_TYPE=" + type + " "
                + ""
                + " and  COMMANDO_CODE='"+commandoCode+"' and FLOW_CREATE_TIME= (select max(FLOW_CREATE_TIME) from YCQ_QZH_COMMANDO_FLOW where 1=1 and FLOW_TYPE=" + type + "     ");
        List<Object> params = new ArrayList<>();
        if (null != commandoCode) {
            sb.append("  and  COMMANDO_CODE=?  ");
            params.add(commandoCode);
        }
        if (null != batchNoApply) {
            sb.append("  and  BATCH_NO_APPLY=?  ");
            params.add(batchNoApply);
        }
        if (null != batchNoChange) {
            sb.append("  and  BATCH_NO_CHANGE=?  ");
            params.add(batchNoChange);
        }

        sb.append("  )  limit 1");
        return jdbcTemplate.queryForBean(YcqQzhCommandoFlow.class, sb.toString(), params.toArray());
    }


    /**
     * @description: 查询当前最新的状态
     * @param: [commandoCode, examineType]
     * @return: com.crunii.micro.domain.entity.main.YcqQzhCommandoFlow
     * @author 13760
     * @date: 2023/9/22 15:13
     */
    public YcqQzhCommandoFlow findFlowByCode(String commandoCode, Integer batchNoChange) {
        StringBuffer sb = new StringBuffer("select * from YCQ_QZH_COMMANDO_FLOW where 1=1  and FLOW_CREATE_TIME=\n"
                + "(\n"
                + "select max(FLOW_CREATE_TIME) from YCQ_QZH_COMMANDO_FLOW where 1=1 and FLOW_TYPE=2 "
                + "and COMMANDO_CODE='" + commandoCode + "'"
                + "and BATCH_NO_CHANGE='" + batchNoChange + "'  "
                + ")  limit 1");
        List<Object> params = new ArrayList<>();
        return jdbcTemplate.queryForBean(YcqQzhCommandoFlow.class, sb.toString(), params.toArray());
    }


    /**
     * @description: 查找以前是不是有流程了
     * @param: [commandoCode]
     * @return: com.crunii.micro.domain.entity.main.YcqQzhCommandoFlow
     * @author 13760
     * @date: 2023/9/22 17:08
     */
    public YcqQzhCommandoFlow findChangeNumFromFlow(String commandoCode) {
        StringBuilder sb = new StringBuilder("   select distinct COMMANDO_CODE ,max(APPLY_CHANGE_NUM) APPLY_CHANGE_NUM from  YCQ_QZH_COMMANDO_FLOW where 1=1 and FLOW_TYPE=2 and COMMANDO_CODE=?  "
                + "group by COMMANDO_CODE  ");
        List<Object> params = new ArrayList<>();
        if (null != commandoCode) {
            params.add(commandoCode);
        }
        return jdbcTemplate.queryForBean(YcqQzhCommandoFlow.class, sb.toString(), params.toArray());

    }


    /**
     * @description: 根据分公司查找可以审核的人
     * @param: [groupOrgId]
     * @return: java.util.List<com.crunii.micro.domain.dto.Worker>
     * @author 13760
     * @date: 2023/9/23 10:35
     */
    public List<Worker> findReviwed(String groupOrgId) {
        StringBuilder sb = new StringBuilder("   select * from\n"
                + " TSM_WORKER a left join \n"
                + " TSM_WORKER_ROLE b on a.LOGIN_ID=b.LOGIN_ID\n"
                + " left join TSM_ROLE c  on b.ROLE_ID=c.ROLE_ID\n"
                + "                where 1=1 and c.ROLE_NAME='青年突击队审核管理员'\n"
                + "                and a.ORG_ID =?  ");
        List<Object> params = new ArrayList<>();
        if (null != groupOrgId) {
            params.add(groupOrgId);
        }
        return jdbcTemplate.queryForBeans(Worker.class, sb.toString(), params.toArray());
    }


    /**
     * @description: 统计待处理条数
     * @param: [worker]
     * @return: int
     * @author 13760
     * @date: 2023/9/23 16:16
     */
    public int pendingNum(Worker worker) {
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        List<Object> params = new ArrayList<>();
        //申请人
        sb.append(" and a.create_id = ? ");
        params.add(worker.getLoginId());
        String sbCount = "     select count(*) from (    " + sb + " and a.state=1      )         ";
        return jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
    }

    /**
     * @description: 统计变更记录待处理条数
     * @param: [worker]
     * @return: int
     * @author 13760
     * @date: 2023/9/23 16:16
     */
    public int pendingChangeNum(Worker worker) {

        StringBuffer sb = new StringBuffer("select count(*) from (select a.id,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME,\n"
                + "a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE,\n"
                + "b.BATCH_NO_CHANGE,b.APPLY_CHANGE_NUM,a.CREATE_ID\n"
                + " from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1\n"
                + "and a.COMMANDO_CODE=b.COMMANDO_CODE\n"
                + "and b.FLOW_TYPE=2 \n"
                + "and b.id in (  select id from  YCQ_QZH_COMMANDO_FLOW  where (FLOW_CREATE_TIME,APPLY_CHANGE_NUM,COMMANDO_CODE) in (   \n"
                + "                                               select max(FLOW_CREATE_TIME),APPLY_CHANGE_NUM ,COMMANDO_CODE   from YCQ_QZH_COMMANDO_FLOW  where flow_type=2   \n"
                + "                                               group by APPLY_CHANGE_NUM,COMMANDO_CODE   \n"
                + "                                               )   )    ");

        List<Object> params = new ArrayList<>();
        //申请人
        sb.append(" and create_id = ? ");
        StringBuffer sbCount = new StringBuffer("     select * from (" + sb + " )  where 1=1 and  (state=1 or state=5) )");
        params.add(worker.getLoginId());
        return jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
    }


    /**
     * @description: 统计审核人申报记录待处理条数
     * @param: [worker]
     * @return: int
     * @author 13760
     * @date: 2023/9/23 16:16
     */
    public List<QueryCommandoRespDTO> pendingExApplyNum(Worker worker) {

//        StringBuilder sb = new StringBuilder("  SELECT     a.*,   \n"
//                + "  ( CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6  or a.STATE <=2)  THEN '冻结' ELSE '正常' END ) commandoState, \n"
//                + "      b.worker_name applicant,  \n"
//                + "         a.create_time applyTime,    \n"
//                + "          a.GROUP_ORG_NAME orgName  FROM  \n"
//                + "             ycq_qzh_commando a   \n"
//                + "               LEFT JOIN tsm_worker b \n"
//                + "               ON a.create_id = b.login_id WHERE   \n"
//                + "                 a.DEL_FLAG=0 and\n"
//                + "                 a.STATE!=1\n   ");
        StringBuilder sb = new StringBuilder("  SELECT a.*  FROM  ycq_qzh_commando a WHERE   a.DEL_FLAG = 0  and a.STATE != 1  ");

        List<Object> params = new ArrayList<>();


        return jdbcTemplate.queryForBeans(QueryCommandoRespDTO.class, sb.toString(), params.toArray());
    }


    /**
     * @description: 统计审核人变更记录待处理条数
     * @param: [worker]
     * @return: int
     * @author 13760
     * @date: 2023/9/23 16:16
     */
    public List<QueryCommandoChangeRespDTO> pendingExChangeNum(Worker worker) {

        StringBuffer sb = new StringBuffer(" select * from (select a.id,a.COMMANDO_CODE,'变更队长'as data_type,a.COMMANDO_NAME,\n"
                + "a.PROJECT_NAME,a.TYPE,a.HAVE_THREE_YEAR_PLAN,a.PERSON_NUM,a.TEAM_LEADER_NAME,a.TEAM_LEADER_CARD,a.TEAM_LEADER_PHONE,a.GROUP_ORG_NAME,b.APPLY_WORKER as applicant,b.APPLY_TIME,b.STATE,\n"
                + "b.BATCH_NO_CHANGE,b.APPLY_CHANGE_NUM,a.CREATE_ID,a.GROUP_ORG_ID\n"
                + " from YCQ_QZH_COMMANDO a, YCQ_QZH_COMMANDO_FLOW b where 1=1\n"
                + "and a.COMMANDO_CODE=b.COMMANDO_CODE\n"
                + "and b.FLOW_TYPE=2 \n"
                + "and b.id in (   select id from  YCQ_QZH_COMMANDO_FLOW  where (FLOW_CREATE_TIME,APPLY_CHANGE_NUM,COMMANDO_CODE) in (  \n"
                + "                                select max(FLOW_CREATE_TIME),APPLY_CHANGE_NUM ,COMMANDO_CODE   from YCQ_QZH_COMMANDO_FLOW  where flow_type=2  \n"
                + "                                group by APPLY_CHANGE_NUM,COMMANDO_CODE  \n"
                + "                                )   ) ) where 1=1   ");

        List<Object> params = new ArrayList<>();


        return jdbcTemplate.queryForBeans(QueryCommandoChangeRespDTO.class, sb.toString(), params.toArray());
    }

    /**
     * @description: 查询突击队以前的变更队长流程走完没有
     * @param: [commandoCode]
     * @return: com.crunii.micro.domain.entity.main.YcqQzhCommandoFlow
     * @author 13760
     * @date: 2023/9/25 14:46
     */
    public YcqQzhCommandoFlow findMaxFlow(String commandoCode) {
        StringBuffer sb = new StringBuffer(" select * from YCQ_QZH_COMMANDO_FLOW where 1=1 and FLOW_TYPE=2 "
                + "and COMMANDO_CODE=? "
                + "and FLOW_CREATE_TIME=(\n"
                + "select max(FLOW_CREATE_TIME) from   YCQ_QZH_COMMANDO_FLOW where 1=1  \n"
                + " and FLOW_TYPE=2 \n"
                + " and commando_code=? \n"
                + " and apply_change_num=(\n"
                + "select max(apply_change_num) from   YCQ_QZH_COMMANDO_FLOW where 1=1  \n"
                + "and FLOW_TYPE=2 \n"
                + "and commando_code=?  )\n"
                + " )  ");

        List<Object> params = new ArrayList<>();
        if (StringUtils.isNotEmpty(commandoCode)) {
            params.add(commandoCode);
            params.add(commandoCode);
            params.add(commandoCode);
        }
        return jdbcTemplate.queryForBean(YcqQzhCommandoFlow.class, sb.toString(), params.toArray());
    }

    /**
     * @description: 根据code查附件
     * @param: [commandoCode]
     * @return: java.util.List<com.crunii.micro.service.qzh.api.domain.dto.commandDo.FileDTO>
     * @author 13760
     * @date: 2023/9/26 10:59
     */
    public List<FileDTO> findFileByCommandoCode(String commandoCode) {
        StringBuffer sb = new StringBuffer(" select * from  YCQ_QZH_COMMANDO_FILE where 1=1 and state=1 ");

        List<Object> params = new ArrayList<>();
        if (StringUtils.isNotEmpty(commandoCode)) {
            sb.append("   and COMMANDO_CODE=?  ");
            params.add(commandoCode);
        }
        return jdbcTemplate.queryForBeans(FileDTO.class, sb.toString(), params.toArray());
    }


    /**
     * @description: //看新的队长是不是存在在其他突击队中
     * @param: [teamLeaderCard]
     * @return: com.crunii.micro.domain.entity.main.YcqQzhCommando
     * @author 13760
     * @date: 2023/9/27 11:17
     */
    public YcqQzhCommando findCommanDoByCard(String teamLeaderCard) {
        StringBuffer sb = new StringBuffer(" select * from  YCQ_QZH_COMMANDO where 1=1 and DEL_FLAG=0 and  (state=2 or state=3) and TEAM_LEADER_CARD=? limit 1 ");

        List<Object> params = new ArrayList<>();
        params.add(teamLeaderCard);
        return jdbcTemplate.queryForBean(YcqQzhCommando.class, sb.toString(), params.toArray());
    }


    /**
     * @description: 查询用户赋权的组织
     * @param: [currWorker]
     * @return: java.util.List<java.lang.Long>
     * @author 13760
     * @date: 2023/9/27 16:59
     */
    public List<Long> selecetOrgs(int loginId) {
        String SQL = " select distinct org_id from TSM_WORKER_ORG_ALL where worker_id =" + loginId;
        return jdbcTemplate.queryForRawObjects(Long.class, SQL);
    }

    /**
     * @description: 查询突击队名
     * @param: [teamLeaderCard]
     * @return: com.crunii.micro.domain.entity.main.YcqQzhCommando
     * @author 13760
     * @date: 2023/9/27 11:17
     */
    public YcqQzhCommando checkName(SaveCommandoReqDTO saveCommandoReqDTO) {
        StringBuffer sb = new StringBuffer(" select * from  YCQ_QZH_COMMANDO where 1=1 and DEL_FLAG=0 and COMMANDO_NAME=? and  (state=2 or state=3)  limit 1 ");

        List<Object> params = new ArrayList<>();
        params.add(saveCommandoReqDTO.getCommandoName());
        return jdbcTemplate.queryForBean(YcqQzhCommando.class, sb.toString(), params.toArray());
    }


    /**
     * @description: 冻结/正常 队伍状态
     * @param: [commandoCode]
     * @return: java.lang.String
     * @author 13760
     * @date: 2023/9/27 20:41
     */
    public String saveCommandoState(String commandoCode) {
        StringBuffer sb = new StringBuffer(" select commando_state from(   SELECT   a.commando_code,\n"
                + "(CASE WHEN (TIMESTAMPDIFF( MONTH, b.last_login_time, now()) > 6 and k.label = 'true' )  THEN '冻结' ELSE '正常' END ) commando_state\n"
                + "FROM    \n"
                + "ycq_qzh_commando a           \n"
                + "LEFT JOIN tsm_worker b ON a.create_id = b.login_id  \n"
                + "LEFT JOIN (  \n"
                + "    select a.commando_code , count(parent_id) mien_count ,max(b.create_time) max_time ,  \n"
                + "     (case when ((count(parent_id) = 0 and  TIMESTAMPDIFF( MONTH, max(a.create_time) , now()) > 6)  \n"
                + "     or (count(parent_id) > 0 and  TIMESTAMPDIFF( MONTH, max(b.create_time) , now()) > 6))  \n"
                + "     then 'true' else 'false' end) label  \n"
                + "     from ycq_qzh_commando a  \n"
                + "     LEFT JOIN YCQ_QZH_ACTIVITY b on a.id = b.parent_id and b.activity_type = 1   and a.del_flag = 0 and b.del_flag = 0  \n"
                + "     group by a.commando_code  \n"
                + ") k ON k.commando_code = a.commando_code  ) where 1=1 and commando_code=?  ");

        List<Object> params = new ArrayList<>();
        params.add(commandoCode);
        return jdbcTemplate.queryForRawObject(String.class, sb.toString(), params.toArray());
    }


    public YcqQzhCommando findCommanDoOne(String id) {
        StringBuffer sb = new StringBuffer(" select * from  YCQ_QZH_COMMANDO where 1=1 and id='" + id + "' ");
        List<Object> params = new ArrayList<>();
        return jdbcTemplate.queryForBean(YcqQzhCommando.class, sb.toString(), params.toArray());
    }


    public TsmOrg findParentOrgs(Long orgId) {
        String SQL = "select org_id,parent_org_id from tsm_org where org_id = '" + orgId + "' ";
        return jdbcTemplate.queryForBean(TsmOrg.class, SQL);
    }

    public List<String> selectCommandoCode(List<Long> allOrgList) {
        if (allOrgList.size() > 0) {
            String collect = allOrgList.stream().map(String::valueOf).collect(Collectors.joining(",", "(", ") "));
            String SQL = "select distinct COMMANDO_CODE from YCQ_QZH_COMMANDO_ORG where 1=1 and org_id in " + collect;
            return jdbcTemplate.queryForRawObjects(String.class, SQL);
        } else {
            return null;
        }

    }

    /**
     * @description: 查询用户是不是有青子队审核按钮
     * @param: [loginId]
     * @return: int
     * @author 13760
     * @date: 2023/10/5 10:35
     */
    public int findReviewButton(int loginId) {
        String sql = "select count(*) from TSM_WORKER_ROLE a\n"
                + "left join TSM_ROLE b\n"
                + "on a.ROLE_ID=b.ROLE_ID\n"
                + " where 1=1 and b.ROLE_NAME='青年突击队审核管理员'\n"
                + "and LOGIN_ID='" + loginId + "'";
        return jdbcTemplate.queryForRawObject(Integer.class, sql);
    }

    public List<FileDTO> findFlowFile(CommandoFlowRespDTO respDTO) {
        StringBuffer sb = new StringBuffer(" select * from  YCQ_QZH_COMMANDO_FILE where 1=1   ");
        Integer batchNoApply = respDTO.getBatchNoApply();
        String commandoCode = respDTO.getCommandoCode();
        List<Object> params = new ArrayList<>();
        if (StringUtils.isNotEmpty(commandoCode)) {
            sb.append("   and COMMANDO_CODE=?  ");
            params.add(commandoCode);
        }
        if (null != batchNoApply) {
            sb.append("   and BATCH_NO_APPLY=?  ");
            params.add(batchNoApply);
        }
        return jdbcTemplate.queryForBeans(FileDTO.class, sb.toString(), params.toArray());
    }

    public Integer selectLogin(String commandoCode) {
        StringBuffer sb = new StringBuffer(" SELECT count(1) from(SELECT distinct to_char(c.login_time, 'yyyy-mm-dd') FROM \n" +
                "YCQ_QZH_COMMANDO b left join TSM_WORKER a on b.CREATE_ID = a.LOGIN_ID \n" +
                "and a.STATE = 1 left join TSM_LOGIN_LOG c on a.MOBILE = c.MOBILE\n" +
                "where b.DEL_FLAG = 0 and b.state =3 and b.commando_code = ? and date_format(c.login_time,'yyyy-MM-dd HH:mm:ss')  > = date_format(b.create_time,'yyyy-MM-dd HH:mm:ss') group by c.login_time)");
        List<Object> params = new ArrayList<>();
        if (StringUtils.isNotEmpty(commandoCode)) {
            params.add(commandoCode);
            return jdbcTemplate.queryForRawObject(Integer.class, sb.toString(), params.toArray());
        } else {
            return 0;
        }
    }

    public Integer selectQueue(String commandoCode) {
        StringBuffer sb = new StringBuffer(" SELECT count(1) from(SELECT distinct to_char(c.login_time, 'yyyy-mm-dd') FROM \n" +
                "YCQ_QZH_COMMANDO b left join TSM_WORKER a on a.MOBILE = b.TEAM_LEADER_PHONE \n" +
                "and a.STATE = 1 left join TSM_LOGIN_LOG c on a.MOBILE = c.MOBILE\n" +
                "where b.DEL_FLAG = 0 and b.state =3 and b.commando_code = ? and date_format(c.login_time,'yyyy-MM-dd HH:mm:ss')  > = date_format(b.create_time,'yyyy-MM-dd HH:mm:ss') group by c.login_time)");
        List<Object> params = new ArrayList<>();
        if (StringUtils.isNotEmpty(commandoCode)) {
            params.add(commandoCode);
            return jdbcTemplate.queryForRawObject(Integer.class, sb.toString(), params.toArray());
        } else {
            return 0;
        }
    }

    public String findWorkerOrgName(String orgId) {

        String sql = "select  org_name from    TSM_ORG  where org_id='"+orgId+"'       ";
       return jdbcTemplate.queryForRawObject(String.class,sql);

    }

//    /**
//     * @description: 查询orgName
//     * @param: [orgId]
//     * @return: java.lang.String
//     * @author 13760
//     * @date: 2023/10/7 10:42
//     */
//	public String findOrgNameByWorker(String orgId) {
//        String sql = "select  org_full_name from    TSM_ORG  where org_id='"+orgId+"'       ";
//        return jdbcTemplate.queryForRawObject(String.class,sql);
//
//	}

//    public List<YcqQzhDemoVo> queryList(YcqQzhDemoDto req) {
//        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
//        List<Object> params = new ArrayList<>();
//        if (StrUtil.isNotBlank(req.getName())) {
//            sb.append(" and a.name like concat('%',?,'%') ");
//            params.add(req.getName());
//        }
//        if (Objects.nonNull(req.getId())) {
//            sb.append(" and a.id = ? ");
//            params.add(req.getId());
//        }
//        List<YcqQzhDemoVo> result = jdbcTemplate.queryForBeans(YcqQzhDemoVo.class, sb.toString(), params.toArray());
//        return result;
//    }
//
//
//    public void deleteByIds(Collection<Long> ids, Long updateId, String updateBy) {
//        String inIds = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ") "));
//        StringBuilder sb = new StringBuilder(DELETE_SQL).append(inIds);
//        jdbcTemplate.update(sb.toString(), updateId, updateBy, new Date());
//    }
}
