package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhDailySupervisionFileDto;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhDailySupervisionReqDto;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhDailySupervisionRespDto;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhIndicatorVersionVo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Repository
public class YcqQzhDailySupervisionJdbcDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String DAILY_SUPERVISION_BASE_SQL = " select id,id serialNumber,collective_name,location_unit,full_name,id_card,contact_information,create_by createUser,create_time,content from ycq_qzh_daily_supervision where deleted =0 ";

    private static final String DAILY_SUPERVISION_COUNT_SQL = "select count(*) from ycq_qzh_daily_supervision a where deleted != 1 ";

    public PageRsp<YcqQzhDailySupervisionRespDto> pageSupervisionManage(PageReq<YcqQzhDailySupervisionReqDto> req) {
        StringBuilder sb = new StringBuilder(DAILY_SUPERVISION_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(DAILY_SUPERVISION_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        if (req.getQuery() != null && StrUtil.isNotBlank(req.getQuery().getCollectiveName())) {
            sb.append(" and collective_name like concat('%',?,'%') ");
            sbCount.append(" and collective_name like concat('%',?,'%') ");
            params.add(req.getQuery().getCollectiveName());
        }
        if (req.getQuery() != null && StrUtil.isNotBlank(req.getQuery().getLocationUnit())) {
            sb.append(" and location_unit like concat('%',?,'%') ");
            sbCount.append(" and location_unit like concat('%',?,'%') ");
            params.add(req.getQuery().getLocationUnit());
        }
        if (req.getQuery() != null && StrUtil.isNotBlank(req.getQuery().getFullName())) {
            sb.append(" and full_name like concat('%',?,'%') or and id_card like concat('%',?,'%') or and contact_information like concat('%',?,'%') ");
            sbCount.append(" and full_name like concat('%',?,'%') ");
            params.add(req.getQuery().getFullName());
            params.add(req.getQuery().getFullName());
            params.add(req.getQuery().getFullName());
        }
        if (req.getQuery() != null && StrUtil.isNotBlank(req.getQuery().getCreateUserName())) {
            sb.append(" and create_by like concat('%',?,'%') ");
            sbCount.append(" and create_by like concat('%',?,'%') ");
            params.add(req.getQuery().getCreateUserName());
        }
        if(req.getQuery() != null && StrUtil.isNotBlank(req.getQuery().getCreateTimeStart())){
            sb.append(" and create_time >= ? ");
            sbCount.append(" and create_time >= ? ");
            params.add(req.getQuery().getCreateTimeStart());
        }
        if(req.getQuery() != null && StrUtil.isNotBlank(req.getQuery().getCreateTimeEnd())){
            sb.append(" and create_time <= ? ");
            sbCount.append(" and create_time <= ? ");
            params.add(req.getQuery().getCreateTimeStart());
        }
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqQzhDailySupervisionRespDto> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jdbcTemplate.queryForBeans(YcqQzhDailySupervisionRespDto.class, sb.toString(), params.toArray());
            for(YcqQzhDailySupervisionRespDto resp : result){
                if(StrUtil.isNotBlank(resp.getIdCard())){
                    String maskedIDCard = resp.getIdCard().substring(0, resp.getIdCard().length() - 4) + "****"; // 脱敏最后四位
                    resp.setIdCard(maskedIDCard);
                }
                StringBuilder fileSql = new StringBuilder(" select file_id from ycq_qzh_daily_supervision_file where daily_supervision_id = ? ");
                List<YcqQzhDailySupervisionFileDto> files= jdbcTemplate.queryForBeans(YcqQzhDailySupervisionFileDto.class, fileSql.toString(), resp.getId());
                if(files.size() > 0){
                    List<String> fileIds = new ArrayList<>();
                    for(YcqQzhDailySupervisionFileDto fileId : files){
                        if(StrUtil.isNotBlank(fileId.getFileId())){
                            fileIds.add(fileId.getFileId());
                        }
                    }
                    resp.setFiles(fileIds);
                }
            }
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqQzhDailySupervisionRespDto> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }
}
