package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.YcqQzhIndicatorSystem;
import com.crunii.micro.service.qzh.api.domain.dto.*;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Repository
public class YcqQzhIndicatorManageJdbcDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String VERSION_PAGE_ROLE_BASE_SQL = "select id, version_name, effective_time_start, effective_time_end, " +
        " CASE WHEN CURRENT_DATE BETWEEN effective_time_start AND effective_time_end THEN 'true' \n" +
        " ELSE 'false' END AS is_valid from ycq_qzh_indicator_version a where deleted = 0 ";

    private static final String VERSION_PAGE_ROLE_COUNT_SQL = "select count(*) from ycq_qzh_indicator_version a where deleted = 0 ";

    private static final String SYSTEM_PAGE_ROLE_BASE_SQL = "select * from ycq_qzh_indicator_system a where deleted = 0 ";

    private static final String SYSTEM_PAGE_ROLE_COUNT_SQL = "select count(*) from ycq_qzh_indicator_system a where deleted = 0 ";

    private static final String GET_EVALUATION_POINTS_BASE_SQL =
        " SELECT ROWNUM key_id, evaluation_points FROM ( SELECT evaluation_points, MIN(create_time) create_time\n"
            + " FROM ycq_qzh_indicator_system WHERE indicator_version_id = ? AND deleted = 0 AND evaluation_points IS NOT NULL \n"
            + " GROUP BY evaluation_points ORDER BY create_time ) a ";

    private static final String GET_PRIMARY_INDICATORS_BASE_SQL =
        " SELECT ROWNUM key_id, primary_indicators, evaluation_points FROM (\n"
            + " SELECT primary_indicators, evaluation_points, MIN(create_time) create_time\n"
            + " FROM ycq_qzh_indicator_system\n"
            + " WHERE indicator_version_id = ? AND deleted = 0 AND primary_indicators IS NOT NULL \n"
            + " GROUP BY primary_indicators, evaluation_points ORDER BY create_time ) a\n ";

    private static final String GET_ONE_LEVEL_S_SQL = "  select * from ycq_qzh_indicator_system where indicator_version_id =? and evaluation_points = ? and deleted = 0 and primary_indicators is null limit 1 ";

    private static final String GET_TWO_LEVEL_S_SQL = "  select * from ycq_qzh_indicator_system where indicator_version_id =? and evaluation_points = ? and primary_indicators = ? and deleted = 0 and secondary_indicators is null limit 1 ";


    public List<YcqQzhIndicatorVersionVo> findIndicatorVersion() {
        StringBuilder sb = new StringBuilder(VERSION_PAGE_ROLE_BASE_SQL);
        List<Object> params = new ArrayList<>();
        sb.append(" order by create_time desc, id desc ");
        return jdbcTemplate.queryForBeans(YcqQzhIndicatorVersionVo.class, sb.toString(), params.toArray());
    }

    public PageRsp<YcqQzhIndicatorVersionVo> pageIndicatorVersion(PageReq<Void> req) {
        StringBuilder sb = new StringBuilder(VERSION_PAGE_ROLE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(VERSION_PAGE_ROLE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqQzhIndicatorVersionVo> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            sb.append( " order by create_time desc, id desc ");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(YcqQzhIndicatorVersionVo.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqQzhIndicatorVersionVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<YcqQzhIndicatorSystemVo> findIndicatorSystem(YcqQzhIndicatorSystemDto req) {
        if (Objects.isNull(req) || Objects.isNull(req.getIndicatorVersionId())) {
            throw new BusinessException("版本id不能为空");
        }
        StringBuilder sb = new StringBuilder(SYSTEM_PAGE_ROLE_BASE_SQL);
        List<Object> params = new ArrayList<>();
        sb.append(" and a.indicator_version_id = ? ");
        params.add(req.getIndicatorVersionId());
        if (StrUtil.isNotBlank(req.getEvaluationPoints())) {
            sb.append(" and a.evaluation_points like concat('%',?,'%') ");
            params.add(req.getEvaluationPoints());
        }
        if (StrUtil.isNotBlank(req.getPrimaryIndicators())) {
            sb.append(" and a.primary_indicators like concat('%',?,'%') ");
            params.add(req.getPrimaryIndicators());
        }
        if (StrUtil.isNotBlank(req.getSecondaryIndicators())) {
            sb.append(" and a.secondary_indicators like concat('%',?,'%') ");
            params.add(req.getSecondaryIndicators());
        }
        sb.append(" order by evaluation_points, create_time, primary_indicators, secondary_indicators ");
        return jdbcTemplate.queryForBeans(YcqQzhIndicatorSystemVo.class, sb.toString(), params.toArray());
    }

    public PageRsp<YcqQzhIndicatorSystemVo> pageIndicatorSystem(PageReq<YcqQzhIndicatorSystemDto> req) {
        YcqQzhIndicatorSystemDto query = req.getQuery();
        if (Objects.isNull(query) || Objects.isNull(query.getIndicatorVersionId())) {
            throw new BusinessException("版本id不能为空");
        }
        StringBuilder sb = new StringBuilder(SYSTEM_PAGE_ROLE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(SYSTEM_PAGE_ROLE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        sb.append(" and a.indicator_version_id = ? ");
        sbCount.append(" and a.indicator_version_id = ? ");
        params.add(query.getIndicatorVersionId());

        if (StrUtil.isNotBlank(query.getEvaluationPoints())) {
            sb.append(" and a.evaluation_points like concat('%',?,'%') ");
            sbCount.append(" and a.evaluation_points like concat('%',?,'%') ");
            params.add(query.getEvaluationPoints());
        }
        if (StrUtil.isNotBlank(query.getPrimaryIndicators())) {
            sb.append(" and a.primary_indicators like concat('%',?,'%') ");
            sbCount.append(" and a.primary_indicators like concat('%',?,'%') ");
            params.add(query.getPrimaryIndicators());
        }
        if (StrUtil.isNotBlank(query.getSecondaryIndicators())) {
            sb.append(" and a.secondary_indicators like concat('%',?,'%') ");
            sbCount.append(" and a.secondary_indicators like concat('%',?,'%') ");
            params.add(query.getSecondaryIndicators());
        }
        sb.append(" order by evaluation_points, create_time, primary_indicators, secondary_indicators ");
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqQzhIndicatorSystemVo> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(YcqQzhIndicatorSystemVo.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqQzhIndicatorSystemVo> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<YcqQzhIndicatorIndexRespDTO> getEvaluationPoints(Long indicatorVersionId) {
        List<Object> params = new ArrayList<>();
        params.add(indicatorVersionId);
        return jdbcTemplate.queryForBeans(YcqQzhIndicatorIndexRespDTO.class, GET_EVALUATION_POINTS_BASE_SQL, params.toArray());
    }

    public List<YcqQzhIndicatorIndexRespDTO> getPrimaryIndicators(Long indicatorVersionId) {
        List<Object> params = new ArrayList<>();
        params.add(indicatorVersionId);
        return jdbcTemplate.queryForBeans(YcqQzhIndicatorIndexRespDTO.class, GET_PRIMARY_INDICATORS_BASE_SQL, params.toArray());
    }

    public YcqQzhIndicatorSystem getOneLevelC(YcqQzhIndicatorSystemDto req) {
        List<Object> params = new ArrayList<>();
        params.add(req.getIndicatorVersionId());
        params.add(req.getEvaluationPoints());
        return jdbcTemplate.queryForBean(YcqQzhIndicatorSystem.class, GET_ONE_LEVEL_S_SQL, params.toArray());
    }

    public YcqQzhIndicatorSystem getTwoLevelC(YcqQzhIndicatorSystemDto req) {
        List<Object> params = new ArrayList<>();
        params.add(req.getIndicatorVersionId());
        params.add(req.getEvaluationPoints());
        params.add(req.getPrimaryIndicators());
        return jdbcTemplate.queryForBean(YcqQzhIndicatorSystem.class, GET_TWO_LEVEL_S_SQL, params.toArray());
    }
}
