package com.crunii.micro.service.qzh.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQzhOnlineTrain;
import com.crunii.micro.domain.entity.main.YcqQzhOnlineTrainingVideo;
import com.crunii.micro.domain.entity.main.YcqQzhVideoBrowsingHistory;
import com.crunii.micro.domain.repo.main.YcqQzhOnlineTrainingVideoRepo;
import com.crunii.micro.domain.repo.main.YcqQzhVideoBrowsingHistoryRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhOnlineRecordDTO;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhOnlineTrainDTO;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhOnlineTrainVideoDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.RightsProtectionRecordPageRespDTO;
import com.crunii.micro.service.qzh.dto.video.VideoDto;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/10/12 16:42
 */
@Repository
public class YcqQzhOnlineTrainDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;


    @Resource
    private YcqQzhOnlineTrainingVideoRepo ycqQzhOnlineTrainingVideoRepo;

    @Resource
    private YcqQzhVideoBrowsingHistoryRepo ycqQzhVideoBrowsingHistoryRepo;


    private static final String PAGE_BASE_SQL = "select  a.id,a.TITLE title, a.IS_TOP isTop ,a.content content ,a.SECOND_TYPE,IF(a.TYPE='青创专题培训', a.SECOND_TYPE, TYPE) type,a.FILE_ID fileId,a.FILE_URL fileUrl, " +
            "        a.state state,a.CREATED_BY createdBy,a.CREATED_TIME createdTime, " +
            "        COUNT(c.id) num ,COUNT(distinct c.CREATED_BY) personNum ,(select worker_name from TSM_WORKER where login_id=a.CREATED_BY limit 1) as createdName " +
            "from YCQ_QZH_ONLINE_TRAINING a " +
            "left join YCQ_QZH_ONLINE_TRAINING_VIDEO  b on a.ID=b.TRAINING_ID " +
            "left join YCQ_QZH_VIDEO_BROWSING_HISTORY c on b.VIDEO_ID=c.BUSINESS_ID    where 1=1 ";
    private static final String GET_ONE_SQL = "select  a.id,a.TITLE title, a.IS_TOP isTop ,a.content content ,a.SECOND_TYPE,a.TYPE type,a.FILE_ID fileId,a.FILE_URL fileUrl, " +
            "        a.state state,  (select worker_name from tsm_worker where login_id=a.CREATED_BY)createdBy,a.CREATED_TIME createdTime, " +
            "        COUNT(c.id) num ,COUNT(distinct c.CREATED_BY) personNum " +
            "from YCQ_QZH_ONLINE_TRAINING a " +
            "left join YCQ_QZH_ONLINE_TRAINING_VIDEO  b on a.ID=b.TRAINING_ID " +
            "left join YCQ_QZH_VIDEO_BROWSING_HISTORY c on b.VIDEO_ID=c.BUSINESS_ID    where 1=1 ";

    private static final String LAST_SQL = " group by a.id,a.TITLE,a.TYPE,a.FILE_ID,a.FILE_URL,a.state,a.CREATED_BY ,a.content ,a.CREATED_TIME ,a.IS_TOP,a.UPDATED_TIME,a.SECOND_TYPE, a.TOP_TIME ORDER BY a.IS_TOP desc ,a.TOP_TIME desc,a.CREATED_TIME desc";
    private static final String PAGE_RECORD_PAGE_SQL = "select a.CREATED_BY id,a.CREATED_NAME,a.PHONE,MAX(a.CREATED_TIME)createdTime, count(a.id) number ,sum(a.VIDEO_TIME) videoTime from YCQ_QZH_VIDEO_BROWSING_HISTORY a  left join  YCQ_QZH_ONLINE_TRAINING_VIDEO b on  a.BUSINESS_ID=b.VIDEO_ID where b.TRAINING_ID=?  group by a.CREATED_NAME, a.PHONE,a.CREATED_BY  ";

    public Result<YcqQzhOnlineTrainDTO> getOne(Long id,Worker worker) {
        List<Object> params = new ArrayList<>();
        List<YcqQzhOnlineTrainVideoDTO> ycqQzhOnlineTrainVideoDTOS = new ArrayList<YcqQzhOnlineTrainVideoDTO>();
        StringBuilder sb = new StringBuilder(GET_ONE_SQL);
        sb.append("  and a.id = "+id+" ");
        sb.append(LAST_SQL);
        List<YcqQzhOnlineTrainDTO> list = NPJT.queryForBeans(YcqQzhOnlineTrainDTO.class, sb.toString(), params);
        if (list == null) {
            return Result.ofFail("未查到数据");
        }
        YcqQzhOnlineTrainDTO result=list.get(0);
        List<YcqQzhOnlineTrainingVideo> videos = ycqQzhOnlineTrainingVideoRepo.findByTrainingId(id);
        if (videos != null && videos.size() > 0) {
            for (YcqQzhOnlineTrainingVideo video : videos) {
                YcqQzhOnlineTrainVideoDTO ycqQzhOnlineTrainVideoDTO = new YcqQzhOnlineTrainVideoDTO();
                BeanUtils.copyProperties(video, ycqQzhOnlineTrainVideoDTO);
                List<YcqQzhVideoBrowsingHistory> byCreatedByAndVideoId = ycqQzhVideoBrowsingHistoryRepo.findByCreatedByAndBusinessIDAndIsComplete(Long.parseLong(worker.getLoginId() + ""), video.getVideoId(),"1");
                if (byCreatedByAndVideoId!=null&&byCreatedByAndVideoId.size()>0){
                    ycqQzhOnlineTrainVideoDTO.setIsComplete("1");
                }else {
                    ycqQzhOnlineTrainVideoDTO.setIsComplete("0");
                }
                ycqQzhOnlineTrainVideoDTOS.add(ycqQzhOnlineTrainVideoDTO);
            }
        }
        List<YcqQzhVideoBrowsingHistory> byBusinessID = ycqQzhVideoBrowsingHistoryRepo.findByBusinessID(id + "");
        if (byBusinessID!=null&&byBusinessID.size()>0){
            result.setRecordNum(byBusinessID.size());
        }else {
            result.setRecordNum(0);
        }
        result.setVideos(ycqQzhOnlineTrainVideoDTOS);
        return Result.ofSuccess(result);
    }

    public Result<PageRsp<YcqQzhOnlineTrainDTO>> trainPage(PageReq<YcqQzhOnlineTrainDTO> req) {
        YcqQzhOnlineTrainDTO query = req.getQuery();
        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);

        List<Object> params = new ArrayList<>();
        if (query.getTitle() != null && !query.getTitle().equals("")) {
            sb.append(" and a.TITLE like concat('%',?,'%') ");
            params.add(query.getTitle());
        }
        if (query.getType() != null && !query.getType().equals("")) {
            sb.append("AND a.TYPE = ? ");
            params.add(query.getType());
        }

        if (query.getSecondType() != null && !query.getSecondType().equals("")) {
            sb.append("AND a.SECOND_TYPE = ? ");
            params.add(query.getSecondType());
        }
        if (query.getState() != null && !query.getState().equals("")) {
            sb.append("AND a.state = ? ");
            params.add(query.getState());
        }

        // 时间范围查询
        if (query.getBeginTime() != null) {
            sb.append("AND a.CREATED_TIME >= ? ");
            params.add(query.getBeginTime());
        }
        if (query.getEndTime() != null) {
            sb.append("AND a.CREATED_TIME <= ? ");
            params.add(query.getEndTime());
        }
        sb.append(LAST_SQL);
        String sql = sb.toString();
        int total = jt.queryForInt(" select count(1)  from ( " + sql + " ) ", params.toArray());
        List<YcqQzhOnlineTrainDTO> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jt.queryForBeans(YcqQzhOnlineTrainDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqQzhOnlineTrainDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return Result.ofSuccess(page);
    }

    public Result<PageRsp<YcqQzhOnlineRecordDTO>> recordPage(PageReq<YcqQzhOnlineTrainDTO> req) {
        YcqQzhOnlineTrainDTO query = req.getQuery();
        StringBuilder sb = new StringBuilder(PAGE_RECORD_PAGE_SQL);
        List<Object> params = new ArrayList<>();
        params.add(query.getId());
        if (query.getType() != null && !query.getType().equals("")) {
            sb.append("AND a.TYPE = ? ");
            params.add(query.getType());
        }
        String sql = sb.toString();
        int total = jt.queryForInt(" select count(1)  from ( " + sql + " ) ", params.toArray());
        List<YcqQzhOnlineRecordDTO> result;
        String dbType = jt.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jt.queryForBeans(YcqQzhOnlineRecordDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqQzhOnlineRecordDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return Result.ofSuccess(page);
    }
}
