package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQzhRightsFlow;
import com.crunii.micro.domain.entity.main.YcqQzhRightsProtection;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author lyx
 * @date 2023/11/15
 */
@Repository
public class YcqQzhProtectionJdbcDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String PAGE_ROLE_BASE_SQL = "select  a.id,applicant_code,  "
            + "            case when apply_type = 0 then '创建'  "
            + "            when apply_type = 1 then '复核' end apply_type,  "
            + "            case when a.status = '1000' or a.status = '2000' then '草稿'  "
            + "            when a.status = '1001' or a.status = '1002' then '待初评'  "
            + "            when a.status = '1003' or a.status = '1004' then '待初核'  "
            + "            when a.status = '1005' then '待公示'  "
            + "            when a.status = '1006' then '公示通过'  "
            + "            when a.status = '2001' or a.status = '2002'  or a.status = '2003'  or a.status = '2004' then '待复核'  "
            + "            when a.status = '2005' then '复核通过'  "
            + "            end status,unit_name,address,  "
            + "            case  "
            + "            when a.link = '1000'  then '创建'  "
            + "            when a.link = '1001'  then '区县级主管单位初评及推荐'  "
            + "            when a.link = '1002' then '区县级团委初评及推荐'  "
            + "            when a.link = '1003' then '市级主管单位初核'  "
            + "            when a.link = '1004' then '市级团委初核'  "
            + "            when a.link = '1005' then '市级团委公示'  "
            + "            when a.link = '1006' then '公示'  "
            + "            when a.link = '2000' then '复核申报'  "
            + "            when a.link = '2001' then '区县级主管单位复核'  "
            + "            when a.link = '2002' then '区县级团委复核'  "
            + "            when a.link = '2003' then '市级主管单位评估及验收'  "
            + "            when a.link = '2004' then '市级团委评估及验收'  "
            + "            when a.link = '2005' then ''  "
            + "            end link ,a.phone,a.zip_code,c.value_name system, f.processed_date, "
            + "            case when a.link = '2000' then '1000' "
            + "            when a.link = '2001' then '1001'  "
            + "            when a.link = '2002' then '1002'  "
            + "            when a.link = '2003' then '1003'  "
            + "            when a.link = '2004' then '1004'  "
            + "            when a.link = '2005' then '1006'   else a.link  end orderBy,"
            + "            a.ORGANIZATION_NAME organization,a.system_state,a.manager_org_name managerOrg,b.worker_name applicant,  "
            + "            apply_time, a.status buttom,a.SYSTEM_ID,a.municipal_level_internal_institutions,a.manager_org_id,a.manager_org_name,a.ORGANIZATION_NAME,a.ORGANIZATION_ID  "
            + "            from YCQ_QZH_RIGHTS_PROTECTION a "
            + "            left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id "
            + "            left join  TSM_WORKER b on a.applicant_id = b.login_id  "
            + "            left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_code and prop_code = 'systemType' "
            + "            where a.del_flag = 0 and f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where fl.rights_id = a.id) ";

    private static final String PAGE_ROLE_COUNT_SQL = "select count(*)from YCQ_QZH_RIGHTS_PROTECTION a where del_flag=0";

    private static final String GET_CREAT_BY_ID_SQL = "select  a.id,a.applicant_code, f.id flow_id,a.unit_name,"
            + "             a.phone,zip_code,a.system_id,a.address,"
            + "                       a.ORGANIZATION_ID,a.ORGANIZATION_NAME,a.manager_org_ID,a.manager_org_name,a.create_pro_time,a.policy_id,"
            + "						  a.municipal_level_internal_institutions,programme,"
            + "                       apply_time from YCQ_QZH_RIGHTS_PROTECTION a "
            + "             left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id"
            + "             where a.del_flag = 0 and f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where fl.rights_id = a.id)"
            + "             and a.id = ?";

    private static final String PAGE_UPDATE_BASE_SQL = "select a.id,applicant_code, e.update_type,  "
            + "   case when apply_type = 0 then '创建'   "
            + "   when apply_type = 1 then '复核' end apply_type,   "
            + "   case when status = '1000' or status = '2000' then '草稿'   "
            + "   when status = '1001' or status = '1002' then '待初评'   "
            + "   when status = '1003' or status = '1004' then '待初核'   "
            + "   when status = '1005' then '待公示'   "
            + "   when status = '1006' then '公示通过'   "
            + "   when status = '2001' or status = '2002' or status = '2003'  or status = '2004' then '待复核'   "
            + "   when status = '2005' then '复核通过'   "
            + "             end status,unit_name,address,   "
            + "   case   "
            + "   when link = '1000'  then '创建'   "
            + "   when link = '1001'  then '区县级主管单位初评及推荐'   "
            + "   when link = '1002' then '区县级团委初评及推荐'   "
            + "   when link = '1003' then '市级主管单位初核'   "
            + "   when link = '1004' then '市级团委初核'   "
            + "   when link = '1005' then '市级团委公示'   "
            + "   when link = '1006' then '公示'   "
            + "   when link = '2000' then '复核申报'   "
            + "   when link = '2001' then '区县级主管单位复核'   "
            + "   when link = '2002' then '区县级团委复核'   "
            + "   when link = '2003' then '市级主管单位评估及验收'   "
            + "   when link = '2004' then '市级团委评估及验收'   "
            + "   when link = '2005' then ''   "
            + "             end link ,a.phone,zip_code,c.value_name system, e.old_data,e.new_data, "
            + "             a.organization_name organization,b.worker_name applicant,apply_time, status buttom   "
            + "             from YCQ_QZH_RIGHTS_PROTECTION_UPDATE e   "
            + "   left join YCQ_QZH_RIGHTS_PROTECTION a on e.rights_id = a.id  "
            + "   left join  TSM_WORKER b on a.applicant_id = b.login_id   "
            + "   left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_code and prop_code = 'systemType'    "
            + "   where a.del_flag=0 ";
    //	private static final String PAGE_UPDATE_COUNT_SQL = "select count(*)from YCQ_QZH_RIGHTS_PROTECTION_UPDATE where del_flag=0";
    private static final String PAGE_UPDATE_COUNT_SQL = " select count(*)" +
            "from YCQ_QZH_RIGHTS_PROTECTION_UPDATE e " +
            "left join YCQ_QZH_RIGHTS_PROTECTION a on e.rights_id = a.id " +
            "left join  TSM_WORKER b on a.applicant_id = b.login_id " +
            "left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_code and prop_code = 'systemType'  " +
            "left join TSM_ORG d on a.ORGANIZATION_ID = d.org_id " +
            "where a.del_flag=0 ";
    //查看复核详细
    private static final String SELECTCHECK = " select " +
            "a.id, " +
            "a.unit_name, " +
            "a.old_unit_name, " +
            "c.value_name system, " +
            "a.ORGANIZATION_ID, " +
            "a.ORGANIZATION_NAME, " +
            "a.old_organization_id, " +
            "a.old_organization_name, " +
            "a.manager_org_id, " +
            "a.manager_org_name, " +
            "b.worker_name applicant, " +
            "a.applicant_code, " +
            "a.phone, " +
            "address, " +
            "case when a.municipal_level_internal_institutions=1  " +
            "then '是' else '否' end as municipalLevelInternalInstitutionsName, " +
            "apply_time, " +
            "naming_time, " +
            "naming_file_code, " +
            "f.id flowId, " +
            "a.header, " +
            "a.municipal_level_internal_institutions, " +
            "a.system_id, " +
            "a.association_id, " +
            "a.zip_code " +
            "from YCQ_QZH_RIGHTS_PROTECTION a " +
            "left join  TSM_WORKER b on a.applicant_id = b.login_id " +
            "left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_code and prop_code = 'systemType'  " +
            "left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id " +
            "where a.del_flag = 0 and f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where fl.rights_id = a.id) ";

    public PageRsp<RightsProtectionRecordPageRespDTO> recordPageList(PageReq<Object> req, boolean b, int i) {
//		CurrWorkerUtil.mockCurrWorker();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        StringBuilder sb = new StringBuilder(PAGE_ROLE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_ROLE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        if (!b) {
            sb.append("and a.applicant_id = ? ");
            sbCount.append("and a.applicant_id = ? ");
            params.add(worker.getLoginId());
        }
        if (1 == i) {
            sb.append(" and (a.link = '1006' or a.link = '2005') ");
            sbCount.append(" and (a.link = '1006' or a.link = '2005') ");
        }
        sb.append("  order by orderBy,f.status,f.processed_date desc  ");
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<RightsProtectionRecordPageRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(RightsProtectionRecordPageRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<RightsProtectionRecordPageRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

    public PageRsp<RightsProtectionRecordPageRespDTO> updatePageList(PageReq<Object> req, boolean b) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        List<Object> params = new ArrayList<>();
        StringBuilder sb = new StringBuilder(PAGE_UPDATE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_UPDATE_COUNT_SQL);
        if (!b) {
            sb.append(" and a.applicant_id = ? ");
            sbCount.append(" and a.applicant_id = ? ");
            params.add(worker.getLoginId());
        }
        sb.append(" order by e.update_time desc");

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<RightsProtectionRecordPageRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(RightsProtectionRecordPageRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<RightsProtectionRecordPageRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }


    public YcqQzhRightsFlow findMaxSortByRightsId(Long rightsId) {
        String sql = "select *from YCQ_QZH_RIGHTS_FLOW where rights_id = ? "
                + "and sort = (select max(sort) from Ycq_Qzh_Rights_Flow where rights_id = ?)";
        return jdbcTemplate.queryForBean(YcqQzhRightsFlow.class, sql, rightsId, rightsId);
    }

    public String getDescById(Long systemId) {
        String sql = "select value_control address from TSM_PROPERTY_VALUE where value_code = ? and prop_code = 'systemType'";
        RightsProtectionRecordPageRespDTO dto = jdbcTemplate
                .queryForBean(RightsProtectionRecordPageRespDTO.class, sql, systemId);
        return dto.getAddress();
    }

    public RightsProtectionCreateQueryRespDTO getCreate(Long id) {
        StringBuilder sb = new StringBuilder(GET_CREAT_BY_ID_SQL);
//		if (0 == i){
//        	sb.append(" and f.is_draft = 0");
//		}
        RightsProtectionCreateQueryRespDTO dto = jdbcTemplate
                .queryForBean(RightsProtectionCreateQueryRespDTO.class, sb.toString(), id);

        return dto;
    }

//    public List<CheckMarketQueryDTO> findAllByIdInAndDelFlag(List<Long> ids, String orgId, int i, String code, CheckQueryDTO dto) {
//        String sql = "select a.unit_name,a.old_unit_name, b.value_name,d.id,d.file_type,d.file_url,d.file_new_name  from YCQ_QZH_RIGHTS_PROTECTION a "
//                + "   left join TSM_PROPERTY_VALUE b on a.SYSTEM_ID = b.value_id  "
//                + "   left join YCQ_QZH_RIGHTS_flow c on a.id = c.rights_id "
//                + "   left join YCQ_QZH_RIGHTS_PROTECTION_ANNEX d on c.id = d.flow_id "
//                + "   where a.del_flag = 0 and b.del_flag = 0 and d.del_flag = 0 "
//                + "   and a.id in(:ids) and d.annex_type = 1 "
//                + "   and c.should_org_id = :orgId and c.link = :code";
//        Map<String, Object> params = new HashMap<>();
//        params.put("ids", ids);
//        params.put("orgId", orgId);
//        params.put("code", code);
//        if (null != dto.getSystemId()) {
//            sql += " and a.systemId = :systemId";
//            params.put("systemId", dto.getSystemId());
//        }
//        return NPJT.queryForBeans(CheckMarketQueryDTO.class, sql, params);
//
//    }


    public RightsProtectionCheckQueryRespDTO getCheck(Long id) {
        StringBuilder sb = new StringBuilder(SELECTCHECK);
        sb.append(" and a.id = ? ");
        return jdbcTemplate.queryForBean(RightsProtectionCheckQueryRespDTO.class, sb.toString(), id);
    }

    public List<CheckIndexReqDTO> selectRights(Long id) {
        String sql = " select b.id,a.rights_id,a.rel,b.index_data from YCQ_QZH_RIGHTS_INDEX_REL a " +
                "left join YCQ_QZH_RIGHTS_INDEX b on a.index_id=b.id where 1=1 and a.del_flag=0 and b.del_flag=0 and a.rights_id = ? ";
        return jdbcTemplate.queryForBeans(CheckIndexReqDTO.class, sql, id);
    }

    private static final String PAGE_ROLE_BASE_INVENTORY_SUPPER_SQL = "select *from(select  "
            + "      a.id,a.applicant_code,a.apply_type apply_type_int,g.document_title  policy,a.policy_id, "
            + "      case when a.apply_type = 0 then '创建'   "
            + "      when a.apply_type = 1 then '复核' end apply_type, "
            + "      a.unit_name,a.address,a.phone,zip_code,a.system_id,c.value_name system,  "
            + "      a.ORGANIZATION_ID,a.ORGANIZATION_NAME organization,b.worker_name applicant,apply_time, a.status ,a.status type ,e.result, "
            + "      case when e.result = 0 then '通过' "
            + "      when e.result = 1 then '优秀'  "
            + "      when e.result = 2 then '合格'  "
            + "      when e.result = 3 then '不通过'  "
            + "      when e.result = 4 then '整改'  "
            + "      when e.result = 5 then '撤销命名' "
            + "      when e.result = 6 then '命名自动取消'  "
            + "   when e.result = 7 then '退回'  "
            + " end opinion ,f.processed_date,case when a.status = '500' then 2 else f.status end fsta "
            + "              from YCQ_QZH_RIGHTS_PROTECTION a   "
            + "              left join  TSM_WORKER b on a.applicant_id = b.login_id   "
            + "              left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_code and prop_code = 'systemType'    "
            + "              left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id "
            + "              left join YCQ_QZH_RIGHTS_FLOW e on e.id = f.parent_id "
            + "      left join YCQ_QZH_DOCUMENT g on a.policy_id = g.id "
            + "              where a.del_flag = 0 and  a.status != '1000' and a.status != '2000' "
            + "              and   f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where a.id = fl.rights_id)) a where 1=1 ";
    private static final String PAGE_ROLE_BASE_INVENTORY_SUPPER_SQL_COUNT = "select count(*) from( "
            + "     select   "
            + "                   a.id,a.applicant_code,a.apply_type apply_type_int,g.document_title,a.policy_id, "
            + "                   case when a.apply_type = 0 then '创建'    "
            + "                   when a.apply_type = 1 then '复核' end apply_type,  "
            + "                   a.unit_name,a.address,a.phone,zip_code,a.system_id,c.value_name system,   "
            + "                   a.ORGANIZATION_ID,a.ORGANIZATION_NAME organization,b.worker_name applicant,apply_time, a.status ,a.status type, e.result,  "
            + "                   case when e.result = 0 then '通过'  "
            + "                   when e.result = 1 then '优秀'   "
            + "                   when e.result = 2 then '合格'   "
            + "                   when e.result = 3 then '不通过'   "
            + "                   when e.result = 4 then '整改'   "
            + "                   when e.result = 5 then '撤销命名'  "
            + "                   when e.result = 6 then '命名自动取消'   "
            + "                when e.result = 7 then '退回'   "
            + "              end opinion ,f.processed_date,f.status fsta,a.del_flag  from YCQ_QZH_RIGHTS_PROTECTION a    "
            + "                           left join  TSM_WORKER b on a.applicant_id = b.login_id    "
            + "                           left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_code and prop_code = 'systemType'     "
            + "                           left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id  "
            + "                           left join YCQ_QZH_RIGHTS_FLOW e on e.id = f.parent_id  "
            + "                   left join YCQ_QZH_DOCUMENT g on a.policy_id = g.id  "
            + "                           where a.del_flag = 0  and  a.status != '1000' and a.status != '2000'  "
            + "                           and   f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where a.id = fl.rights_id)) a where 1=1 ";
    /*private static final String PAGE_ROLE_BASE_INVENTORY_SQL = "select  id,applicant_code,apply_type apply_type_int,document_title,policy_id,"
            + "  case when apply_type = 0 then '创建'  "
            + "              when apply_type = 1 then '复核' end apply_type,"
            + "              unit_name,address,phone,zip_code,system_id, system, "
            + "              ORGANIZATION_ID,organization,applicant,apply_time,  status, result,"
            + "              case when result = 0 then '通过'"
            + "              when result = 1 then '优秀' "
            + "              when result = 2 then '合格' "
            + "              when result = 3 then '不通过' "
            + "              when result = 4 then '整改' "
            + "              when result = 5 then '撤销命名'"
            + "              when result = 6 then '命名自动取消' "
            + " when result = 7 then '退回' "
            + " end opinion ,processed_date,status fsta"
            + "               from "
            + " (select "
            + "              a.id,a.applicant_code,a.apply_type,g.document_title,a.policy_id, "
            + "              a.unit_name,a.address,a.phone,zip_code,a.system_id,c.value_name system, "
            + "              a.ORGANIZATION_ID,a.ORGANIZATION_NAME organization,b.worker_name applicant,"
            + "				 apply_time, a.status , e.result,f.processed_date,f.status fsta"
            + "              from YCQ_QZH_RIGHTS_PROTECTION a  "
            + "              left join  TSM_WORKER b on a.applicant_id = b.login_id  "
            + "              left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_id  "
            + "              left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id"
            + "              left join YCQ_QZH_RIGHTS_FLOW e on e.id = f.parent_id"
            + "   left join YCQ_QZH_DOCUMENT g on a.policy_id = g.id"
            + "              where f.should_org_id = ? and f.status = 0  "
            + "              and a.del_flag = 0 "
            + "              union all  "
            + "              select "
            + "              a.id,a.applicant_code,a.apply_type,g.document_title,a.policy_id, "
            + "              a.unit_name,a.address,a.phone,zip_code,a.system_id,c.value_name system, "
            + "              a.ORGANIZATION_ID,a.ORGANIZATION_NAME organization,b.worker_name,apply_time,"
            + "				 a.status , e.result,e.processed_date,f.status fsta"
            + "              from YCQ_QZH_RIGHTS_PROTECTION a  "
            + "              left join  TSM_WORKER b on a.applicant_id = b.login_id  "
            + "              left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_id  "
            + "              left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id "
            + "              left join YCQ_QZH_RIGHTS_FLOW e on e.id = f.parent_id"
            + " 			 left join YCQ_QZH_DOCUMENT g on a.policy_id = g.id"
            + "              where f.should_org_id = ? and f.status = 1  "
            + "              and   f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where a.id = fl.rights_id and fl.status = 1) "
            + "               and a.del_flag = 0 "
            + "              union all "
            + "              select "
            + "              a.id,a.applicant_code,a.apply_type,g.document_title,a.policy_id,"
            + "              a.unit_name,a.address,a.phone,zip_code,a.system_id,c.value_name system, "
            + "              a.ORGANIZATION_ID,a.ORGANIZATION_NAME organization,b.worker_name,apply_time, "
            + "				 a.status , f.result,f.processed_date,f.status fsta"
            + "              from YCQ_QZH_RIGHTS_PROTECTION a  "
            + "              left join  TSM_WORKER b on a.applicant_id = b.login_id  "
            + "              left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_id  "
            + "              left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id "
            + " 			 left join YCQ_QZH_DOCUMENT g on a.policy_id = g.id"
            + "              where f.next_org_id = ? and f.status = 1  "
            + "              and (f.result = 3 or f.result = 4 or  f.result = 5 or f.result = 6)"
            + "              and   f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where a.id = fl.rights_id and fl.status = 1) "
            + "               and a.del_flag = 0) a where status != '1000' and status != '2000'";
    private static final String PAGE_ROLE_BASE_INVENTORY_SQL_COUNT = "select count(*)"
            + "              from"
            + "(select "
            + "             *"
            + "             from YCQ_QZH_RIGHTS_PROTECTION a  "
            + "             left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id "
            + "             where f.should_org_id = ? and f.status = 0  "
            + "             and a.del_flag = 0 "
            + "             union all  "
            + "             select "
            + "             *"
            + "             from YCQ_QZH_RIGHTS_PROTECTION a  "
            + "             left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id "
            + "             where f.should_org_id = ? and f.status = 1  "
            + "             and   f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where a.id = fl.rights_id and fl.status = 1) "
            + "              and a.del_flag = 0 "
            + "             union all "
            + "             select "
            + "             *"
            + "             from YCQ_QZH_RIGHTS_PROTECTION a  "
            + "             left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id "
            + "             where f.next_org_id = ? and f.status = 1  "
            + "             and (f.result = 3 or f.result = 4 or  f.result = 5 or f.result = 6)"
            + "             and   f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where a.id = fl.rights_id and fl.status = 1) "
            + "              and a.del_flag = 0) a where status != '1000' and status != '2000'";
*/
    private static final String PAGE_ROLE_BASE_INVENTORY_SQL ="select  id,applicant_code,apply_type_int,document_title policy,policy_id, "
            + "               case when apply_type = 0 then '创建'   "
            + "                           when apply_type = 1 then '复核' end apply_type, "
            + "                           unit_name,address,phone,zip_code,system_id, system,  "
            + "                           ORGANIZATION_ID,organization,applicant,apply_time,  status, result, "
            + "                           case when result = 0 then '通过' "
            + "                           when result = 1 then '优秀'  "
            + "                           when result = 2 then '合格'  "
            + "                           when result = 3 then '不通过'  "
            + "                           when result = 4 then '整改'  "
            + "                           when result = 5 then '撤销命名' "
            + "                           when result = 6 then '命名自动取消'  "
            + "              when result = 7 then '退回'  "
            + "              end opinion ,processed_date,fsta,type "
            + "                            from  "
            + "              (select  "
            + "                           a.id,a.applicant_code,a.apply_type,a.apply_type apply_type_int,g.document_title,a.policy_id,"
            + "                           a.unit_name,a.address,a.phone,zip_code,a.system_id,c.value_name system,  "
            + "                           a.ORGANIZATION_ID,a.ORGANIZATION_NAME organization,b.worker_name applicant, "
            + "                  apply_time, a.status ,a.status type, e.result,f.processed_date,f.status fsta "
            + "                           from YCQ_QZH_RIGHTS_PROTECTION a   "
            + "                           left join  TSM_WORKER b on a.applicant_id = b.login_id   "
            + "                           left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_code and prop_code = 'systemType'    "
            + "                           left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id "
            + "                           left join YCQ_QZH_RIGHTS_FLOW e on e.id = f.parent_id "
            + "                left join YCQ_QZH_DOCUMENT g on a.policy_id = g.id "
            + "                           where f.should_org_id = ? and f.status = 0   "
            + "                           and a.del_flag = 0  ";
    private static final String PAGE_ROLE_BASE_INVENTORY_SQL_COUNT ="select  count(*) "
            + "                            from  "
            + "              (select  "
            + "                           a.id,a.applicant_code,a.apply_type,a.apply_type apply_type_int,g.document_title,a.policy_id,"
            + "                           a.unit_name,a.address,a.phone,zip_code,a.system_id,c.value_name system,  "
            + "                           a.ORGANIZATION_ID,a.ORGANIZATION_NAME organization,b.worker_name applicant, "
            + "                  apply_time, a.status ,a.status type, e.result,f.processed_date,f.status fsta "
            + "                           from YCQ_QZH_RIGHTS_PROTECTION a   "
            + "                           left join  TSM_WORKER b on a.applicant_id = b.login_id   "
            + "                           left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_code and prop_code = 'systemType'    "
            + "                           left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id "
            + "                           left join YCQ_QZH_RIGHTS_FLOW e on e.id = f.parent_id "
            + "                left join YCQ_QZH_DOCUMENT g on a.policy_id = g.id "
            + "                           where f.should_org_id = ? and f.status = 0   "
            + "                           and a.del_flag = 0  ";
    private static final String PAGE_ROLE_BASE_INVENTORY_ADD_SQL = " union all "
            + "                   select    a.id,a.applicant_code,a.apply_type,a.apply_type apply_type_int,g.document_title,a.policy_id,  "
            + "                           a.unit_name,a.address,a.phone,zip_code,a.system_id,c.value_name system,  "
            + "                           a.ORGANIZATION_ID,a.ORGANIZATION_NAME organization,b.worker_name applicant, "
            + "                  apply_time, 1 status , a.status type, e.result,f.processed_date,case when a.status = '500' then 2 else 1 end fsta "
            + "                           from YCQ_QZH_RIGHTS_PROTECTION a   "
            + "                           left join  TSM_WORKER b on a.applicant_id = b.login_id   "
            + "                           left join TSM_PROPERTY_VALUE c on a.SYSTEM_ID = c.value_code and prop_code = 'systemType'    "
            + "                           left join YCQ_QZH_RIGHTS_FLOW f on a.id = f.rights_id "
            + "                           left join YCQ_QZH_RIGHTS_FLOW e on e.id = f.parent_id "
            + "                left join YCQ_QZH_DOCUMENT g on a.policy_id = g.id  "
            + "                    where a.id in([ids]) and f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where a.id = fl.rights_id) ";
    public PageRsp<RightsProtectionInventoryPageRespDTO> inventoryPageList(PageReq<RightsProtectionInventoryPageReqDTO> req, String workerOrgId, boolean b, List<Long> ids) {
        StringBuilder sb;
        StringBuilder sbCount;
        List<Object> params = new ArrayList<>();
        if (b) {
            //超级管理员
            sb = new StringBuilder(PAGE_ROLE_BASE_INVENTORY_SUPPER_SQL);
            sbCount = new StringBuilder(PAGE_ROLE_BASE_INVENTORY_SUPPER_SQL_COUNT);
        } else {
            sb = new StringBuilder(PAGE_ROLE_BASE_INVENTORY_SQL);
            sbCount = new StringBuilder(PAGE_ROLE_BASE_INVENTORY_SQL_COUNT);
            params.add(workerOrgId);

            if (CollectionUtils.isNotEmpty(ids)){
                sb.append(PAGE_ROLE_BASE_INVENTORY_ADD_SQL);
                sbCount.append(PAGE_ROLE_BASE_INVENTORY_ADD_SQL);
                //params.add(StringUtils.join(ids,","));
            }
            sb.append(") a where type != '1000' and type != '2000'");
            sbCount.append(") a where type != '1000' and type != '2000'");


        }
        if (StrUtil.isNotBlank(req.getQuery().getUnitName())) {
            sb.append(" and a.unit_name like concat('%',?,'%') ");
            sbCount.append(" and a.unit_name like concat('%',?,'%') ");
            params.add(req.getQuery().getUnitName());
        }
        if (Objects.nonNull(req.getQuery().getOrganization())) {
            sb.append(" and a.organization_id =?");
            sbCount.append(" and a.organization_id=?");
            params.add(req.getQuery().getOrganization());
        }
        if (StrUtil.isNotBlank(req.getQuery().getApplicant())) {
            sb.append(" and a.applicant like concat('%',?,'%') ");
            sbCount.append(" and a.applicant like concat('%',?,'%') ");
            params.add(req.getQuery().getApplicant());
        }
        if (StrUtil.isNotBlank(req.getQuery().getApplyStartTime()) && StrUtil.isNotBlank(req.getQuery().getApplyEndTime())) {
            sb.append(" and a.apply_time between ? and ? ");
            sbCount.append(" and a.apply_time between ? and ? ");
            params.add(req.getQuery().getApplyStartTime());
            params.add(req.getQuery().getApplyEndTime());
        }
        if (StrUtil.isNotBlank(req.getQuery().getStatus())) {
            sb.append(" and a.type =? ");
            sbCount.append(" and a.type =? ");
            params.add(req.getQuery().getStatus());
        }
        if (Objects.nonNull(req.getQuery().getApplyType())) {
            sb.append(" and a.apply_type_int =?");
            sbCount.append(" and a.apply_type_int=?");
            params.add(req.getQuery().getApplyType());
        }
        if (StrUtil.isNotBlank(req.getQuery().getPolicyId())) {
            sb.append(" and a.policy_id =? ");
            sbCount.append(" and a.policy_id=? ");
            params.add(req.getQuery().getPolicyId());
        }
        sb.append(" order by fsta,processed_date desc ");
        String scount = sbCount.toString().replace("[ids]", StringUtils.strip(ids.toString(), "[]"));

        int total = jdbcTemplate.queryForInt(scount, params.toArray());
        List<RightsProtectionInventoryPageRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dbType, params, sb, req);
            String sql = sb.toString().replace("[ids]", StringUtils.strip(ids.toString(), "[]"));
            result = jdbcTemplate.queryForBeans(RightsProtectionInventoryPageRespDTO.class, sql, params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<RightsProtectionInventoryPageRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

    public List<RightsProtectionFlowDTO> getFlows(Long id) {
        String sql1 = "select d.id flowId,b.processed_work workerName,c.org_name doOrgName,b.link linkName,b.link,b.sort,  "
                + "                 DATE_FORMAT(b.processed_date, '%Y-%m-%d %H:%i:%s') doDate,b.result option,b.msg from YCQ_QZH_RIGHTS_PROTECTION a  "
                + "                 left join YCQ_QZH_RIGHTS_FLOW b on a.id = b.rights_id  "
                + "                 left join TSM_ORG c on should_org_id = c.org_id    "
                + "                 left join YCQ_QZH_RIGHTS_FLOW d on d.parent_id = b.id "
                + "                 where a.del_flag= 0 and  a.id = ? and b.status = 1 and b.link!='500' and b.link!='1006' and b.link != '2005'  order by sort ";
        List<RightsProtectionFlowDTO> list = jdbcTemplate.queryForBeans(RightsProtectionFlowDTO.class, sql1, id);
        List<Long> ids = list.stream().map(RightsProtectionFlowDTO::getFlowId).collect(Collectors.toList());
        String sql2 = "select id,file_id, file_name,file_new_name,annex_type,file_url,flow_id from YCQ_QZH_RIGHTS_PROTECTION_ANNEX  "
                + "  where flow_id in (:ids)";

        Map<String, Object> params = new HashMap<>();
        params.put("ids", ids);
        List<RightsProtectionFlowAnnexDTO> annexs = NPJT.queryForBeans(RightsProtectionFlowAnnexDTO.class, sql2, params);

        for (RightsProtectionFlowDTO dto : list) {
            List<RightsProtectionFlowAnnexDTO> other = annexs.stream().filter(o -> o.getFlowId().equals(dto.getFlowId()) && o.getAnnexType() == 0)
                    .collect(Collectors.toList());
            List<RightsProtectionFlowAnnexDTO> annex = annexs.stream().filter(o -> o.getFlowId().equals(dto.getFlowId()) && o.getAnnexType() == 1)
                    .collect(Collectors.toList());

            List<RightsProtectionFlowAnnexDTO> re = annexs.stream().filter(o -> o.getFlowId().equals(dto.getFlowId()) && o.getAnnexType() == 2)
                    .collect(Collectors.toList());

            if (CollectionUtils.isNotEmpty(other)) {
                dto.setOtherAnnex(other);
            }
            if (CollectionUtils.isNotEmpty(annex)) {
                dto.setAnnex(annex.get(0));
            }

            if (CollectionUtils.isNotEmpty(re)) {
                dto.setReviewAnnex(re.get(0));
            }
        }
        return list;
    }

    public List<CheckMarketQueryDTO> findMarket(CheckQueryDTO dto, Worker worker, String button) {
        StringBuffer sql = new StringBuffer(" select " +
                "a.id, " +
                "b.value_name systemName, " +
                "a.unit_name, " +
                "a.old_unit_name, " +
                "a.SYSTEM_ID, " +
                "a.filling_unit, " +
                "case when c.link ==2004 then a.opinion else null end option " +
                "from YCQ_QZH_RIGHTS_PROTECTION a " +
                "left join TSM_PROPERTY_VALUE b on a.SYSTEM_ID = b.value_code and prop_code = 'systemType'   " +
                "left join YCQ_QZH_RIGHTS_FLOW c on a.id = c.rights_id " +
                "where a.del_flag = 0 and b.del_flag = 0 ");
        List<Object> params = new ArrayList<>();
        if (!CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            sql.append(" and c.should_org_id = ? ");
            params.add(worker.getOrgId());
        }
        if (null != dto.getIds()) {
            String ids = dto.getIds().stream().map(id -> "'" + id + "'").collect(
                    Collectors.joining(",", "(", ")"));
            sql.append(" and a.id in "+ids);
        }
        if (StringUtils.isNotBlank(button)) {
            sql.append(" and c.link = ? ");
            params.add(button);
        }
        if (null != dto.getSystemId()) {
            sql.append(" and a.system_id = ? ");
            params.add(dto.getSystemId());
        }
        return jdbcTemplate.queryForBeans(CheckMarketQueryDTO.class, sql.toString(), params.toArray());
    }

    public List<RightsProtectionAnnexDTO> findLast(CheckMarketQueryDTO checkMarketQueryDTO, Integer annexType) {
        StringBuffer SQL = new StringBuffer(" select " +
                "d.file_type," +
                "d.file_url," +
                "d.file_new_name," +
                "d.file_name   " +
                "from YCQ_QZH_RIGHTS_FLOW c " +
                "left join YCQ_QZH_RIGHTS_PROTECTION_ANNEX d on c.id = d.flow_id " +
                "where 1=1 ");
        List<Object> params = new ArrayList<>();
        if (null != checkMarketQueryDTO.getId()) {
            SQL.append(" and c.rights_id= ? ");
            params.add(checkMarketQueryDTO.getId());
            SQL.append(" and c.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW where rights_id = ? ) ");
            params.add(checkMarketQueryDTO.getId());
        }
        if (null != annexType) {
            SQL.append(" and d.annex_type = ? ");
            params.add(annexType);
        }
        return jdbcTemplate.queryForBeans(RightsProtectionAnnexDTO.class, SQL.toString(), params.toArray());
    }

    public PageRsp<RightsProtectionRecordPageRespDTO> recordPage(PageReq<ProtectionRecordDTO> req, boolean b, int i) {
//        CurrWorkerUtil.mockCurrWorker();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        StringBuilder sb = new StringBuilder(PAGE_ROLE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_ROLE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        if (!b) {
            sb.append("and a.applicant_id = ? ");
            sbCount.append("and a.applicant_id = ? ");
            params.add(worker.getLoginId());
        }
        if (1 == i) {
            sb.append(" and (a.link = '1006' or a.link = '2005') ");
            sbCount.append(" and (a.link = '1006' or a.link = '2005') ");
        }
        if (null != req.getQuery().getApplyType()) {
            sb.append(" and a.apply_type =  ? ");
            sbCount.append(" and a.apply_type = ? ");
            params.add(req.getQuery().getApplyType());
        }
        if (Objects.nonNull(req.getQuery().getApplyTimeStart())) {
            sb.append(" and  to_char(a.apply_time,'yyyy-mm-dd') >=? ");
            sbCount.append(" and  to_char(a.apply_time,'yyyy-mm-dd') >=? ");
            params.add(req.getQuery().getApplyTimeStart());
        }
        if (Objects.nonNull(req.getQuery().getApplyTimeEnd())) {
            sb.append(" and  to_char(a.apply_time,'yyyy-mm-dd') <=? ");
            sbCount.append(" and  to_char(a.apply_time,'yyyy-mm-dd') <=? ");
            params.add(req.getQuery().getApplyTimeEnd());
        }
        sb.append("  order by orderBy,f.status,f.processed_date desc  ");
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<RightsProtectionRecordPageRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(RightsProtectionRecordPageRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<RightsProtectionRecordPageRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

	public List<YcqQzhRightsProtection> findIdByWork(Long id) {
        String sql = "select a.id from YCQ_QZH_RIGHTS_PROTECTION a"
                + "   left join YCQ_QZH_RIGHTS_FLOW b on a.id = b.rights_id "
                + "   where b.status = 0 and b.should_org_id = ? and a.del_flag = 0";
        return jdbcTemplate.queryForBeans(YcqQzhRightsProtection.class,sql,id);
	}

    public RightsProtectionFlowOrgDTO findMaxSortByRightsIdAndLink(Long id, String link) {

        String sql = "select f.*,t.org_name shouldOrg,m.org_name nextOrg from YCQ_QZH_RIGHTS_FLOW f "
                + " left join tsm_org t on f.should_org_id = t.org_id "
                + " left join tsm_org m on f.next_org_id = m.org_id "
                + " where f.rights_id = ? and f.sort = (select max(sort) from YCQ_QZH_RIGHTS_FLOW fl where fl.rights_id = ? and link = ?) ";

        return jdbcTemplate.queryForBean(RightsProtectionFlowOrgDTO.class,sql,id,id,link);
    }
}
