/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved. <a>http://www.cqcis.com</a>
 */
package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.YcqQuestionBank;
import com.crunii.micro.domain.repo.main.YcqQuestionBankRepo;
import com.crunii.micro.service.qzh.api.domain.dto.*;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author zwx
 * @Description //TODO
 * @Date  2023/9/5 11:09:06
 **/
@Repository
public class YcqTopicJdbcDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    @Autowired
    YcqQuestionBankRepo ycqQuestionBankRepo;

    private static final String PAGE_TOPIC_BASE_SQL = "select * from ycq_topic a where a.deleted = 0 ";

    private static final String PAGE_TOPIC_COUNT_SQL = "select count(*) from ycq_topic a where a.deleted = 0 ";


    private static final String DELETE_SQL = "update ycq_topic set deleted=1, update_id=?, update_name=?, update_time=? where id in";

    public PageRsp<TopicRespDTO> pageList(PageReq<TopicListQueryReqDTO> req) {
        StringBuilder sb = new StringBuilder(PAGE_TOPIC_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_TOPIC_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        if (StrUtil.isNotBlank(req.getQuery().getTopic())) {
            sb.append(" and a.topic like concat('%',?,'%') ");
            sbCount.append(" and a.topic like concat('%',?,'%') ");
            params.add(req.getQuery().getTopic());
        }
        if (Objects.nonNull(req.getQuery().getTopicType())) {
            sb.append(" and a.topic_type = ? ");
            sbCount.append(" and a.topic_type = ? ");
            params.add(req.getQuery().getTopicType());
        }
        sb.append("and a.question_bank_id = ? ");
        sbCount.append("and a.question_bank_id = ? ");
        params.add(req.getQuery().getQuestionBankId());
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<TopicRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(TopicRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<TopicRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

    public Map<String, Object> selectAllTopicForQuestionBankId(List<String> topics,Integer questionBankId) {
        StringBuilder sb = new StringBuilder("select topic,id from ycq_topic a where a.deleted = 0 and a.question_bank_id = ? ");
        return jdbcTemplate.queryForMap(sb.toString(),questionBankId);
    }

    public List<Long> countTopicForQuestionBank(Long questionBankId) {
        StringBuilder sb = new StringBuilder("select count(*) as number from YCQ_TOPIC T where t.DELETED=0 and t.question_bank_id = ? GROUP BY t.TOPIC_TYPE ");
        List<Object> params = new ArrayList<>();
        params.add(questionBankId);
        List<Map<String, Object>> maps = jdbcTemplate.queryForList(sb.toString(), params.toArray());
        List<Long> result = new ArrayList<>();
        for (Map<String, Object> map : maps) {
            Long i = (Long) map.get("NUMBER");
            result.add(i);
        }
        return result;
    }
}
