package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.entity.main.cavalryTrumpet.YcqYnFlowRecord;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.*;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author: LG
 * @date: 2023/11/14 16:07
 */
@Repository
public class YcqYnDeclarationRecordJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate npJdbcTemplate;

    public List<QueryDeclarationRecordChangeFlowVO> queryDeclarationRecordChangeFlow(Long declarationId){
        String sql = "select process_type,captain_change_flow_id,declaration_id,flow_state as nodeName,handler_state as processComments, handler as nodeAssigennName ,handler_org as nodeAssigennOrgName, handle_time as createTime, reason as processResult from YCQ_YN_CAPTAIN_CHANGE_FLOW where declaration_id = ? order by createTime desc";
        return jdbcTemplate.queryForBeans(QueryDeclarationRecordChangeFlowVO.class,sql,declarationId);
    }

    public List<Long> queryChangeIds(Long orgId,boolean city){
        String sql;
        if (city){
            sql = "select change_id from YCQ_YN_CAPTAIN_CHANGE where audit_state = 1 and deleted = 0 and declaration_id in (select declaration_id from YCQ_YN_DECLARATION where deleted = 0 and org_id = (SELECT org_id FROM TSM_ORG WHERE org_id in(SELECT org_id FROM TSM_ORG WHERE parent_org_id = 0 AND org_type = '1')))";
            return jdbcTemplate.queryForRawObjects(Long.class,sql);
        }else {
            sql = "select change_id from YCQ_YN_CAPTAIN_CHANGE where audit_state = 0 and deleted = 0 and declaration_id in (select declaration_id from YCQ_YN_DECLARATION where deleted = 0 and org_id = ? ) ";
            return jdbcTemplate.queryForRawObjects(Long.class,sql,orgId);
        }
    }

    public Long queryCountOrgId(Long change){
        String sql = "select org_id from YCQ_YN_DECLARATION where declaration_id = (select declaration_id from YCQ_YN_CAPTAIN_CHANGE where change_id = ? )";
        return jdbcTemplate.queryForRawObject(Long.class,sql,change);
    }

    public Long queryOrgDel(Long delId){
        String sql = "select org_id from YCQ_YN_DECLARATION where deleted = 0 and declaration_id = ? ";
        return jdbcTemplate.queryForRawObject(Long.class,sql,delId);
    }

    public TsmWorker whetherCaptainWorker(String mobile){
        String sql = "select * from TSM_WORKER where  mobile = ?  limit 1";
        return jdbcTemplate.queryForBean(TsmWorker.class,sql,mobile);
    }

    public Integer queryWhetherChange(Long declarationId){
        String sql = "select count(*) from YCQ_YN_CAPTAIN_CHANGE where deleted = 0 and declaration_id = ? and audit_state  in (0,1)";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,declarationId);
    }

    public YcqYnFlowRecord queryFlowRecord(Long serviceId){
        String sql = "select * from YCQ_YN_FLOW_RECORD where  service_id = ? order by create_time desc limit 1 ";
        return jdbcTemplate.queryForBean(YcqYnFlowRecord.class,sql,serviceId);
    }

    public QueryCaptainInfoVO queryCaptainInfo(Long changeId){
        String sql = "select * from YCQ_YN_CAPTAIN_CHANGE where deleted = 0 and change_id = ? ";
        return jdbcTemplate.queryForBean(QueryCaptainInfoVO.class,sql,changeId);
    }

    public Integer queryChangeNumWorker(Integer workerId,Integer auditState){
        String sql = "select count(1) from YCQ_YN_CAPTAIN_CHANGE b left join YCQ_YN_DECLARATION a on a.declaration_id = b.declaration_id left join TSM_WORKER c on b.creator_id = c.login_id left join TSM_ORG d on a.org_id = d.org_id where a.deleted = 0 and b.deleted = 0 and b.creator_id = ? and b.audit_state = ? ";
        return jdbcTemplate.queryForRawObject(Integer.class,sql,workerId,auditState);
    }

    public Integer queryChangeNumOrg(List<Long> delIds,Integer auditState){
        String sql = "select count(1) from YCQ_YN_CAPTAIN_CHANGE b  left join YCQ_YN_DECLARATION a on a.declaration_id = b.declaration_id left join TSM_WORKER c on b.creator_id = c.login_id left join TSM_ORG d on a.org_id = d.org_id LEFT JOIN YCQ_QZH_DOCUMENT e ON a.doc_id = e.id  where a.deleted = 0 and b.deleted = 0 and a.declaration_id in(:delIds)  and b.audit_state = :auditState ";
        Map<String,Object> params = new HashMap<>(32);
        params.put("delIds",delIds);
        params.put("auditState",auditState);
        return npJdbcTemplate.queryForInt(sql, params);
    }

    public PageRsp<QueryDeclarationRecordChangeVO> queryDeclarationRecordChangeByWorker(Worker worker, PageReq<QueryDeclarationRecordChangeDTO> dto){
        String sql = "select a.declaration_id,a.collective_name,a.distribution,a.declaration_type,a.declaration_code,a.unit,a.category,a.org_id,a.declaration_name,a.id_card,a.phone_number,a.wechat ,b.change_state,b.change_id,b.audit_state,b.reason,b.creator_id,b.create_time,c.worker_name,d.org_name from YCQ_YN_CAPTAIN_CHANGE b left join YCQ_YN_DECLARATION a on a.declaration_id = b.declaration_id left join TSM_WORKER c on b.creator_id = c.login_id left join TSM_ORG d on a.org_id = d.org_id where a.deleted = 0 and b.deleted = 0 and b.creator_id = ? ";
        String sqlCount = "select count(1) from YCQ_YN_CAPTAIN_CHANGE b left join YCQ_YN_DECLARATION a on a.declaration_id = b.declaration_id left join TSM_WORKER c on b.creator_id = c.login_id left join TSM_ORG d on a.org_id = d.org_id where a.deleted = 0 and b.deleted = 0 and b.creator_id = ? ";
        ArrayList<Object> params = new ArrayList<>(20);
        params.add(worker.getLoginId());
        if (ObjectUtil.isNotEmpty(dto.getQuery().getAuditState())){
            if(dto.getQuery().getAuditState().equals(CommonConstants.STATE_VALID)){
                sql += " and b.audit_state in (0,1) ";
                sqlCount += " and b.audit_state in (0,1) ";
            }
        }
        sql += " order by b.create_time desc " ;

        return jdbcTemplate.queryPageBase(dto,sql,sqlCount,QueryDeclarationRecordChangeVO.class,params);
    }


    public PageRsp<QueryDeclarationRecordChangeVO> queryDeclarationRecordChange(List<Long> delIds, PageReq<CivilizationReportPageDto> dto){
        StringBuilder sb = new StringBuilder( "select a.declaration_id,a.collective_name,a.declaration_code,a.distribution,a.declaration_type,a.unit,a.category,a.org_id,a.declaration_name,a.id_card,a.phone_number,a.wechat ,b.change_state,b.change_id,b.audit_state,b.reason,b.creator_id,b.create_time,c.worker_name,d.org_name from YCQ_YN_CAPTAIN_CHANGE b  left join YCQ_YN_DECLARATION a on a.declaration_id = b.declaration_id left join TSM_WORKER c on b.creator_id = c.login_id left join TSM_ORG d on a.org_id = d.org_id LEFT JOIN YCQ_QZH_DOCUMENT e ON a.doc_id = e.id  where a.deleted = 0 and b.deleted = 0 and a.declaration_id in(:delIds) ");
        StringBuilder sbCount = new StringBuilder( "select count(1) from YCQ_YN_CAPTAIN_CHANGE b  left join YCQ_YN_DECLARATION a on a.declaration_id = b.declaration_id left join TSM_WORKER c on b.creator_id = c.login_id left join TSM_ORG d on a.org_id = d.org_id LEFT JOIN YCQ_QZH_DOCUMENT e ON a.doc_id = e.id  where a.deleted = 0 and b.deleted = 0 and a.declaration_id in(:delIds) ");
        Map<String,Object> params = new HashMap<>(32);
        params.put("delIds",delIds);
        CivilizationReportPageDto query = dto.getQuery();
        if(StringUtils.hasText(query.getCollectiveName())){
            sb.append(" and a.collective_name like CONCAT('%',:collectiveName,'%')");
            sbCount.append(" and a.collective_name like CONCAT('%',:collectiveName,'%')");
            params.put("collectiveName",query.getCollectiveName());
        }
        if(Objects.nonNull(query.getOrgId())){
            sb.append(" and a.org_id = :orgId ");
            sbCount.append(" and a.org_id = :orgId ");
            params.put("orgId",query.getOrgId());
        }
        if(StringUtils.hasText(query.getUnit())){
            sb.append(" and a.unit like CONCAT('%',:unit,'%')");
            sbCount.append(" and a.unit like CONCAT('%',:unit,'%')");
            params.put("unit",query.getUnit());
        }
        if(StringUtils.hasText(query.getDeclarationName())){
            sb.append(" and a.declaration_name like CONCAT('%',:declarationName,'%')");
            sbCount.append(" and a.declaration_name like CONCAT('%',:declarationName,'%')");
            params.put("declarationName",query.getDeclarationName());
        }
        if(Objects.nonNull(query.getAuditState())){
            Integer i = 3;
            if (query.getAuditState().equals(i)){
                sb.append(" and b.audit_state in (0,1) ");
                sbCount.append(" and b.audit_state in (0,1) ");
            }else{
                sb.append(" and b.audit_state = :auditState ");
                sbCount.append(" and b.audit_state = :auditState ");
                params.put("auditState",query.getAuditState());
            }
        }

        if(StringUtils.hasText(query.getCreateWorker())){
            sb.append(" and c.worker_name like CONCAT('%',:createWorker,'%')");
            sbCount.append(" and c.worker_name like CONCAT('%',:createWorker,'%')");
            params.put("createWorker",query.getCreateWorker());
        }
        if(Objects.nonNull(query.getStartTime())){
            sb.append(" and a.create_time >= :startTime ");
            sbCount.append(" and a.create_time >= :startTime ");
            params.put("startTime",query.getStartTime());
        }
        if(Objects.nonNull(query.getEndTime())){
            sb.append(" and a.create_time <= :endTime ");
            sbCount.append(" and a.create_time <= :endTime ");
            params.put("endTime",query.getEndTime());
        }
        if(StringUtils.hasText(query.getDocName())){
            sb.append(" and e.document_title like CONCAT('%',:docName,'%')");
            sbCount.append(" and e.document_title like CONCAT('%',:docName,'%')");
            params.put("docName",query.getDocName());
        }
        if(Objects.nonNull(query.getDeclarationType())){
            sb.append(" and a.declaration_type = :declarationType ");
            sbCount.append(" and a.declaration_type = :declarationType ");
            params.put("declarationType",query.getDeclarationType());
        }
        if(Objects.nonNull(query.getCategory())){
            sb.append(" and a.category = :category ");
            sbCount.append(" and a.category = :category ");
            params.put("category",query.getCategory());
        }

        if(Objects.nonNull(query.getDistribution())){
            sb.append(" and a.distribution = :distribution ");
            sbCount.append(" and a.distribution = :distribution ");
            params.put("distribution",query.getDistribution());
        }

        int total = npJdbcTemplate.queryForInt(sbCount.toString(), params);
        List<QueryDeclarationRecordChangeVO> values;
        String dbType = npJdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by b.create_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, dto);
            values = npJdbcTemplate.queryForBeans(QueryDeclarationRecordChangeVO.class, sb.toString(), params);
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<QueryDeclarationRecordChangeVO> page = new PageRsp<>();
        page.setPageNo(dto.getPageNo());
        page.setPageSize(dto.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;

//        return jdbcTemplate.queryPageBase(dto,sql,sqlCount,QueryDeclarationRecordChangeVO.class,params);
    }

    public PageRsp<QueryConfigVO> queryConfig(PageReq<QueryConfigDTO> pageReq) {
        StringBuilder sb = new StringBuilder("SELECT a.before_category_id ,a.category_name ,a.state,b.worker_name ,a.create_time ,a.update_time  FROM YCQ_YN_CATEGORY a LEFT JOIN TSM_WORKER b ON a.creator_id =b.login_id WHERE a.deleted = 0 ");
        StringBuilder sbCount = new StringBuilder("SELECT count(1) FROM YCQ_YN_CATEGORY a LEFT JOIN TSM_WORKER b ON a.creator_id =b.login_id WHERE a.deleted = 0 ");
        List<Object> params = new ArrayList<>(6);
        QueryConfigDTO query = pageReq.getQuery();
        if(StringUtils.hasText(query.getCategoryName())){
            sb.append(" and a.category_name like concat('%',?,'%')");
            sbCount.append(" and a.category_name like concat('%',?,'%')");
            params.add(query.getCategoryName());
        }
        if(StringUtils.hasText(query.getWorkerName())){
            sb.append(" and b.worker_name like concat('%',?,'%')");
            sbCount.append(" and b.worker_name like concat('%',?,'%')");
            params.add(query.getWorkerName());
        }
        if(Objects.nonNull(query.getState())){
            sb.append(" and a.state = ?");
            sbCount.append(" and a.state = ?");
            params.add(query.getState());
        }

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<QueryConfigVO> values;
        String dbType = npJdbcTemplate.getDbType();
        if (total > 0) {
            sb.append(" order by a.update_time desc");
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, pageReq);
            values = jdbcTemplate.queryForBeans(QueryConfigVO.class, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<>(0);
        }
        PageRsp<QueryConfigVO> page = new PageRsp<>();
        page.setPageNo(pageReq.getPageNo());
        page.setPageSize(pageReq.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }
}
