/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved. <a>http://www.cqcis.com</a>
 */
package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqYouthStartAppDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqYouthStartAppVO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqYouthStartDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqYouthStartVO;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: 抽取题目Jdbc
 * @author: limy
 * @date: 2023/9/7 18:53
 **/
@Repository
public class YcqYouthStartAppJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    private static final String PAGE_BASE_SQL = "SELECT id, policy_title, issue_number, policy_level, policy_type,  to_char(publish_time, 'yyyy-MM-dd hh24:mi:ss') publish_time,policy_text,priority,view_count FROM ycq_youth_start WHERE is_deleted = 0 and data_state = 1 ";
    private static final String PAGE_COUNT_SQL = "SELECT COUNT(*) FROM ycq_youth_start WHERE is_deleted = 0 ";

    public PageRsp<YcqYouthStartAppVO> queryList(PageReq<YcqYouthStartAppDTO> req) {

        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        YcqYouthStartAppDTO query = req.getQuery();
        // 固定查询条件
        if (query.getId() != null) {
            sb.append("AND id = ? ");
            sbCount.append("AND id = ? ");
            params.add(query.getId());
        }

        if (query.getPolicyLevel() != null) {
            sb.append("AND policy_level = ? ");
            sbCount.append("AND policy_level = ? ");
            params.add(query.getPolicyLevel());
        }

        if (CollectionUtil.isNotEmpty(query.getPolicyType())) {
            String itemSql = " and policy_type in " + fromListToStr2(query.getPolicyType());
            sb.append(itemSql);
            sbCount.append(itemSql);
        }
        // 添加政策标题模糊查询条件
        if (query.getPolicyTitle() != null) {
            sb.append("AND policy_title LIKE ? ");
            sbCount.append("AND policy_title LIKE ? ");
            params.add("%" + query.getPolicyTitle() + "%");
        }

        // 添加发文号模糊查询条件
        if (query.getIssueNumber() != null) {
            sb.append("AND issue_number LIKE ? ");
            sbCount.append("AND issue_number LIKE ? ");
            params.add("%" + query.getIssueNumber() + "%");
        }


        sb.append(" order by publish_time desc");
        // 获取总数
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqYouthStartAppVO> result =new ArrayList<>();
        if (total > 0) {
            String dbType = jdbcTemplate.getDbType();
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            // 查询结果
             result = jdbcTemplate.queryForBeans(YcqYouthStartAppVO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqYouthStartAppVO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }
    public static String fromListToStr2(List<Integer> list) {
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.toString().replace("[", "(").replace("]", ")").replace(" ", "").trim();
    }

}
