/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved. <a>http://www.cqcis.com</a>
 */
package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.qzh.api.domain.dto.RegistrationAuditVO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.RegistrationAuditDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqYouthStartDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqYouthStartVO;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: 抽取题目Jdbc
 * @author: limy
 * @date: 2023/9/7 18:53
 **/
@Repository
public class YcqYouthStartJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    private static final String PAGE_BASE_SQL = "SELECT id, policy_title, issue_number, policy_level, policy_type, publisher, to_char(publish_time, 'yyyy-MM-dd hh24:mi:ss') publish_time, view_count, data_state,policy_text,priority FROM ycq_youth_start WHERE is_deleted = 0 ";
    private static final String PAGE_COUNT_SQL = "SELECT COUNT(*) FROM ycq_youth_start WHERE is_deleted = 0 ";

    public PageRsp<YcqYouthStartVO> queryList(PageReq<YcqYouthStartDTO> req) {

        StringBuilder sb = new StringBuilder(PAGE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        YcqYouthStartDTO query = req.getQuery();
        // 固定查询条件
        if (query.getId() != null) {
            sb.append("AND id = ? ");
            sbCount.append("AND id = ? ");
            params.add(query.getId());
        }

        if (query.getPolicyLevel() != null) {
            sb.append("AND policy_level = ? ");
            sbCount.append("AND policy_level = ? ");
            params.add(query.getPolicyLevel());
        }

        if (query.getPolicyType() != null) {
            sb.append("AND policy_type = ? ");
            sbCount.append("AND policy_type = ? ");
            params.add(query.getPolicyType());
        }
        if (query.getDataState() != null) {
            sb.append("AND data_state = ? ");
            sbCount.append("AND data_state = ? ");
            params.add(query.getDataState());
        }
        // 添加政策标题模糊查询条件
        if (query.getPolicyTitle() != null) {
            sb.append("AND policy_title LIKE ? ");
            sbCount.append("AND policy_title LIKE ? ");
            params.add("%" + query.getPolicyTitle() + "%");
        }

        // 添加发文号模糊查询条件
        if (query.getIssueNumber() != null) {
            sb.append("AND issue_number LIKE ? ");
            sbCount.append("AND issue_number LIKE ? ");
            params.add("%" + query.getIssueNumber() + "%");
        }

        // 添加发布者模糊查询条件
        if (query.getPublisher() != null) {
            sb.append("AND publisher LIKE ? ");
            sbCount.append("AND publisher LIKE ? ");
            params.add("%" + query.getPublisher() + "%");
        }

        // 添加开始发布时间和结束发布时间范围查询条件
        if (query.getStartPublishTime() != null && query.getEndPublishTime() != null) {
            sb.append("AND publish_time BETWEEN ? AND ? ");
            sbCount.append("AND publish_time BETWEEN ? AND ? ");
            params.add(query.getStartPublishTime());
            params.add(query.getEndPublishTime());
        }

        sb.append(" order by publish_time desc");
        // 获取总数
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqYouthStartVO> result =new ArrayList<>();
        if (total > 0) {
            String dbType = jdbcTemplate.getDbType();
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            // 查询结果
             result = jdbcTemplate.queryForBeans(YcqYouthStartVO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqYouthStartVO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }


}
