/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved. <a>http://www.cqcis.com</a>
 */
package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.qzh.dto.YcqYzyhTestCountReqDto;
import com.crunii.micro.service.qzh.dto.YcqYzyhTestCountRespDto;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: 考试情况统计Jdbc
 * @author: limy
 * @date: 2023/9/7 14:50
 **/
@Repository
public class YcqYzyhTestCountJdbcDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String PAGE_ROLE_BASE_SQL = "SELECT \n" +
            "a.*,\n" +
            "rank() over (order by EXAM_SCORE desc,EXAM_TIME asc) as\n" +
            "rank\n" +
            "FROM \n" +
            "ycq_exam_record  a where a.DELETED=0 ";

    private static final String PAGE_ROLE_COUNT_SQL = "select count(*) from \n" +
            "(SELECT \n" +
            "s.*,\n" +
            "rank() over (order by EXAM_SCORE desc,EXAM_TIME asc) as\n" +
            "rank\n" +
            "FROM \n" +
            "ycq_exam_record  s where s.DELETED=0) a ";


    public PageRsp<YcqYzyhTestCountRespDto> examDetailsCount(PageReq<YcqYzyhTestCountReqDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_ROLE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_ROLE_COUNT_SQL);
        List<Object> params = new ArrayList<>();

        //关键字 姓名手机号
        if (StrUtil.isNotBlank(req.getQuery().getKeyWord())) {
//            sb.append(" and a.title like concat('%',?,'%') ");
//            sbCount.append(" and a.title like concat('%',?,'%') ");
//            params.add(req.getQuery().getTitle());
        }
        //团组织
        if (StrUtil.isNotBlank(req.getQuery().getOrgId())) {
            sb.append(" and a.org_id = ? ");
            sbCount.append(" and a.org_id = ? ");
            params.add(req.getQuery().getOrgId());
        }
        //所在集体
        if (StrUtil.isNotBlank(req.getQuery().getTeamId())) {
//            sb.append(" and a.org_id = ? ");
//            sbCount.append(" and a.org_id = ? ");
//            params.add(req.getQuery().getOrgId());
        }
        //状态
        if (ObjectUtil.isNotEmpty(req.getQuery().getState())) {
            sb.append(" and a.is_pass = ? ");
            sbCount.append(" and a.a.is_pass = ? ");
            params.add(req.getQuery().getState());
        }

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqYzyhTestCountRespDto> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jdbcTemplate.queryForBeans(YcqYzyhTestCountRespDto.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqYzyhTestCountRespDto> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public List<YcqYzyhTestCountRespDto> examDetailsCountExport(YcqYzyhTestCountReqDto req) {
        StringBuilder sb = new StringBuilder(PAGE_ROLE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_ROLE_COUNT_SQL);
        List<Object> params = new ArrayList<>();

        //关键字 姓名手机号
        if (StrUtil.isNotBlank(req.getKeyWord())) {
//            sb.append(" and a.title like concat('%',?,'%') ");
//            sbCount.append(" and a.title like concat('%',?,'%') ");
//            params.add(req.getQuery().getTitle());
        }
        //团组织
        if (StrUtil.isNotBlank(req.getOrgId())) {
            sb.append(" and a.org_id = ? ");
            sbCount.append(" and a.org_id = ? ");
            params.add(req.getOrgId());
        }
        //所在集体
        if (StrUtil.isNotBlank(req.getTeamId())) {
//            sb.append(" and a.org_id = ? ");
//            sbCount.append(" and a.org_id = ? ");
//            params.add(req.getQuery().getOrgId());
        }
        //状态
        if (ObjectUtil.isNotEmpty(req.getState())) {
            sb.append(" and a.is_pass = ? ");
            sbCount.append(" and a.is_pass = ? ");
            params.add(req.getState());
        }
        return jdbcTemplate.queryForBeans(YcqYzyhTestCountRespDto.class, sb.toString(), params.toArray());
    }

}
