/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved. <a>http://www.cqcis.com</a>
 */
package com.crunii.micro.service.qzh.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.qzh.dto.TestDetailRespDto;
import com.crunii.micro.service.qzh.dto.TestTopicRespDto;
import com.crunii.micro.service.qzh.dto.YcqYzyhTestReqDto;
import com.crunii.micro.service.qzh.dto.YcqYzyhTestRespDto;
import jodd.util.StringUtil;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description: 考试列表Jdbc
 * @author: limy
 * @date: 2023/9/6 10:01
 **/
@Repository
public class YcqYzyhTestJdbcDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;

    private static final String PAGE_ROLE_BASE_SQL = "select * from ycq_yzyh_test a left join \n" +
            "\n" +
            "(select EXAM_ID ,count(EXAM_ID) join_num from YCQ_EXAM_RECORD b where b.DELETED='0' group by b.EXAM_ID)c\n" +
            "\n" +
            "on a.id=c.exam_id\n" +
            "\n" +
            "where a.deleted = 0 ";

    private static final String PAGE_ROLE_COUNT_SQL = "select count(*) from ycq_yzyh_test a where a.deleted = 0 ";

    private static final String PAGE_ROLE_DETAIL_SQL = "select a.*,b.JOIN_NUM from YCQ_YZYH_TEST a\n" +
            " left join\n" +
            "(select exam_id,count(*) join_num from  ycq_exam_record group by exam_id)b\n" +
            "on a.id=b.EXAM_ID \n" +
            "where  a.DETELED=0 ";

    private static final String DELETE_SQL = "update ycq_yzyh_test set deleted=1, update_id=?, update_name=?, update_time=? where id in";

    private static final String OPERATE_SQL = "update ycq_yzyh_test set state=?, update_id=?, update_name=?, update_time=? where deleted = 0 and id=?";

    private static final String TOPIC_SQL = "select * from YCQ_TOPIC where id in\n" +
            "(select topic_id from YCQ_YZYH_TOPIC_REL where TEST_ID=?)";

    public PageRsp<YcqYzyhTestRespDto> pageList(PageReq<YcqYzyhTestReqDto> req) {
        StringBuilder sb = new StringBuilder(PAGE_ROLE_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_ROLE_COUNT_SQL);
        List<Object> params = new ArrayList<>();

        //标题
        if (StrUtil.isNotBlank(req.getQuery().getTitle())) {
            sb.append(" and a.title like concat('%',?,'%') ");
            sbCount.append(" and a.title like concat('%',?,'%') ");
            params.add(req.getQuery().getTitle());
        }
        //试卷类型
        if (StrUtil.isNotBlank(req.getQuery().getExamType())) {
            sb.append(" and a.exam_type = ? ");
            sbCount.append(" and a.exam_type = ? ");
            params.add(req.getQuery().getExamType());
        }

        if (ObjectUtil.isNotEmpty(req.getQuery().getTestTime())) {
            sb.append(" and a.start_time between ? and ? ");
            sbCount.append("and a.start_time between  ? and ? ");
            String pattern="yyyy-MM-dd HH:mm";
            params.add(DateUtil.parse(req.getQuery().getTestTime().get(0),pattern));
            params.add(DateUtil.parse(req.getQuery().getTestTime().get(1),pattern));
        }
        //状态
        if (ObjectUtil.isNotEmpty(req.getQuery().getState())) {
            sb.append(" and a.state = ? ");
            sbCount.append(" and a.state = ? ");
            params.add(req.getQuery().getState());
        }

        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqYzyhTestRespDto> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);

            result = jdbcTemplate.queryForBeans(YcqYzyhTestRespDto.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqYzyhTestRespDto> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }

    public TestDetailRespDto viewTest(String id){
        StringBuilder sb = new StringBuilder(PAGE_ROLE_DETAIL_SQL);
        List<Object> params = new ArrayList<>();
        if(StringUtil.isNotBlank(id)){
            sb.append("and a.id= ? ");
            params.add(id);
        }
        return  jdbcTemplate.queryForBean(TestDetailRespDto.class,sb.toString(),params.toArray());
    }

    public List<TestTopicRespDto> getTestTopics(String id){
        StringBuilder sb = new StringBuilder(TOPIC_SQL);
        return  jdbcTemplate.queryForBeans(TestTopicRespDto.class,sb.toString(),id);
    }

    public void deleteByIds(Collection<String> ids, Integer updateId, String updateBy) {
        String inIds = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ") "));
        StringBuilder sb = new StringBuilder(DELETE_SQL).append(inIds);
        jdbcTemplate.update(sb.toString(), updateId, updateBy, new Date());
    }


    public void operateTest(Integer operate, Integer updateId, String updateBy,String id) {
        StringBuilder sb = new StringBuilder(OPERATE_SQL);
        jdbcTemplate.update(sb.toString(),operate, updateId, updateBy, id,new Date());
    }
}
