package com.crunii.micro.service.qzh.jdbc.activitypublish;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.domain.entity.main.YcqPublishActivity;
import com.crunii.micro.service.qzh.api.domain.dto.activityPublish.*;
import com.crunii.micro.service.qzh.api.domain.dto.document.CoverImageDto;
import com.crunii.micro.service.qzh.dto.BelongBusinessVo;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import com.crunii.micro.service.qzh.dto.dynamicform.YcqActiveFlowNodeReqDTO;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;

/**
 * @Author: zwx
 * @CreateTime: 2023-09-19  10:04:30
 * @Description: TODO
 */
@Repository
@Validated
public class PublishActivityDao {
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    @Resource(name = "mainNPJT")
    protected BaseNPJdbcTemplate NPJT;


    private final static String PAGE_ACTIVITY_BASE_SQL = "select ypa.*,ypa.publish_time localPublishTime,ypa.start_time localStartTime, ypa.end_time localEndTime, case when (ypa.is_apply = 0 ) then '不需要' ELSE '需要' END AS isApplyStr " +
            "from ycq_publish_activity ypa " +
            "inner join tsm_worker_org_all twoa on ypa.org_id=twoa.org_id where twoa.worker_id= ? " +
            "and ypa.is_deleted=0 ";
    private final static String PAGE_ACTIVITY_COUNT_SQL = "select count(*) from ycq_publish_activity ypa " +
            " inner join tsm_worker_org_all twoa on ypa.org_id=twoa.org_id where twoa.worker_id= ? " +
            " and ypa.is_deleted=0";

    private final static String PAGE_ACTIVITY_AUDIT_SQL = "SELECT yafn.audit_result auditStatus ,ypa.*,ypa.publish_time localPublishTime,ypa.start_time localStartTime, ypa.end_time localEndTime, case when (ypa.is_apply = 0 ) then '不需要' ELSE '需要' END AS isApplyStr " +
            "FROM YCQ_ACTIVE_FLOW_NODE yafn INNER JOIN (SELECT biz_id, MAX(create_time) AS latest_create_time FROM YCQ_ACTIVE_FLOW_NODE where biz_type = 'active' and is_initial = 2 and org_id = ? GROUP BY biz_id) b ON yafn.biz_id = b.biz_id AND yafn.create_time = b.latest_create_time " +
            "inner join  ycq_publish_activity ypa on ypa.id = yafn.biz_id " +
            "where ypa.is_deleted=0 ";
    private final static String PAGE_ACTIVITY_AUDIT_COUNT_SQL = "select count(*) " +
            "FROM YCQ_ACTIVE_FLOW_NODE yafn INNER JOIN (SELECT biz_id, MAX(create_time) AS latest_create_time FROM YCQ_ACTIVE_FLOW_NODE where biz_type = 'active' and is_initial = 2 and org_id = ? GROUP BY biz_id) b ON yafn.biz_id = b.biz_id AND yafn.create_time = b.latest_create_time " +
            "inner join  ycq_publish_activity ypa on ypa.id = yafn.biz_id " +
            "where ypa.is_deleted=0 ";

    private final static String APP_PAGE_ACTIVITY_BASE_SQL = "select ypa.id,  max(ypa.ACTIVITY_TITLE) ACTIVITY_TITLE, max(ypa.LIMIT_COUNT) LIMIT_COUNT, max(ypa.AUDIT_COUNT) AUDIT_COUNT, max(ypa.START_TIME) START_TIME, max(ypa.END_TIME) END_TIME, max(ypa.BUSINESS_BELONG_LOGO_URL) BUSINESS_BELONG_LOGO_URL, max(ypa.ACTIVITY_ADDRESS) ACTIVITY_ADDRESS, max(ypa.ACTIVITY_TYPE_NAME) ACTIVITY_TYPE_NAME, max(ypa.APPLY_START_TIME) APPLY_START_TIME, max(ypa.APPLY_END_TIME) APPLY_END_TIME, max(ypa.ACTIVITY_STATUS_TYPE_ID) ACTIVITY_STATUS_TYPE_ID , max(ypa.create_time) create_time " +
            "from ycq_publish_activity ypa inner join YCQ_ACTIVITY_ORG yao on ypa.id = yao.ACTIVITY_ID and (yao.ORG_ID = ? or yao.org_id = 1) ";
    private final static String APP_PAGE_ACTIVITY_COUNT_SQL = "select count(*) from (" +
            "select ypa.id from ycq_publish_activity ypa inner join YCQ_ACTIVITY_ORG yao on ypa.id = yao.ACTIVITY_ID and (yao.ORG_ID = ? or yao.org_id = 1) ";

    private final static String APP_PAGE_ACTIVITY_PARTICIPATE_SQL = "select ypa.id,  max(ypa.ACTIVITY_TITLE) ACTIVITY_TITLE, max(ypa.LIMIT_COUNT) LIMIT_COUNT, max(ypa.AUDIT_COUNT) AUDIT_COUNT, max(ypa.START_TIME) START_TIME, max(ypa.END_TIME) END_TIME, max(ypa.BUSINESS_BELONG_LOGO_URL) BUSINESS_BELONG_LOGO_URL, max(ypa.ACTIVITY_ADDRESS) ACTIVITY_ADDRESS, max(ypa.ACTIVITY_TYPE_NAME) ACTIVITY_TYPE_NAME, max(ypa.APPLY_START_TIME) APPLY_START_TIME, max(ypa.APPLY_END_TIME) APPLY_END_TIME, max(ypa.ACTIVITY_STATUS_TYPE_ID) ACTIVITY_STATUS_TYPE_ID , max(ypa.create_time) create_time " +
            "from ycq_publish_activity ypa ";
    private final static String APP_PAGE_ACTIVITY_PARTICIPATE_COUNT_SQL = "select count(*) from (select ypa.id from ycq_publish_activity ypa ";


    private final static String APP_PAGE_ACTIVITY_NO_TOKEN_SQL = "select ypa.id,  max(ypa.ACTIVITY_TITLE) ACTIVITY_TITLE, max(ypa.LIMIT_COUNT) LIMIT_COUNT, max(ypa.AUDIT_COUNT) AUDIT_COUNT, max(ypa.START_TIME) START_TIME, max(ypa.END_TIME) END_TIME, max(ypa.BUSINESS_BELONG_LOGO_URL) BUSINESS_BELONG_LOGO_URL, max(ypa.ACTIVITY_ADDRESS) ACTIVITY_ADDRESS, max(ypa.ACTIVITY_TYPE_NAME) ACTIVITY_TYPE_NAME, max(ypa.APPLY_START_TIME) APPLY_START_TIME, max(ypa.APPLY_END_TIME) APPLY_END_TIME, max(ypa.ACTIVITY_STATUS_TYPE_ID) ACTIVITY_STATUS_TYPE_ID , max(ypa.create_time) create_time " +
            "from ycq_publish_activity ypa inner join YCQ_ACTIVITY_ORG yao on ypa.id = yao.ACTIVITY_ID and yao.org_id = 1 ";
    private final static String APP_PAGE_ACTIVITY_NO_TOKEN_COUNT_SQL = "select count(*) from (select ypa.id from  ycq_publish_activity ypa inner join YCQ_ACTIVITY_ORG yao on ypa.id = yao.ACTIVITY_ID and yao.org_id = 1 ";


    private final static String APP_PAGE_ACTIVITY_EVALUATE_SQL = "select ypa.id ,max(ypa.ACTIVITY_STATUS_TYPE_ID) ACTIVITY_STATUS_TYPE_ID, max(ypa.ACTIVITY_TITLE) ACTIVITY_TITLE, max(REPLY_TYPE) REPLY_TYPE from ycq_publish_activity ypa ";
    private final static String APP_PAGE_ACTIVITY_EVALUATE_COUNT_SQL = "select count(*) from (select ypa.id from ycq_publish_activity ypa ";

    public PageRsp<ActivityQueryRespDTO> pageList(PageReq<ActivityQueryReqDTO> req, Integer loginId) {
        StringBuilder sb = new StringBuilder(PAGE_ACTIVITY_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_ACTIVITY_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        params.add(loginId);
        ActivityQueryReqDTO query = req.getQuery();
        if (StrUtil.isNotBlank(query.getActivityTitle())) {
            sb.append(" and ypa.activity_title like concat('%',?,'%') ");
            sbCount.append(" and ypa.activity_title like concat('%',?,'%') ");
            params.add(query.getActivityTitle());
        }
        if (StrUtil.isNotBlank(query.getBusinessBelong())) {
            sb.append(" and ypa.business_belong = ? ");
            sbCount.append(" and ypa.business_belong = ? ");
            params.add(query.getBusinessBelong());
        }
        if (Objects.nonNull(query.getActivityTypeId())) {
            sb.append("and ypa.activity_type_id = ? ");
            sbCount.append("and ypa.activity_type_id = ? ");
            params.add(query.getActivityTypeId());
        }
        if (Objects.nonNull(query.getOrgId())) {
            sb.append("and ypa.org_id = ? ");
            sbCount.append("and ypa.org_id = ? ");
            params.add(query.getOrgId());
        }
        if (StrUtil.isNotBlank(query.getSponsor())) {
            sb.append(" and ypa.sponsor like concat('%',?,'%') ");
            sbCount.append(" and ypa.sponsor like concat('%',?,'%') ");
            params.add(query.getSponsor());
        }
        if (StrUtil.isNotBlank(query.getStartTime())) {
            sb.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getStartTime());
        }
        if (StrUtil.isNotBlank(query.getEndTime())) {
            sb.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getEndTime());
        }

        if (StrUtil.isNotBlank(query.getStartPublishTime())) {
            sb.append(" and ypa.publish_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.publish_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getStartPublishTime());
        }

        if (StrUtil.isNotBlank(query.getEndPublishTime())) {
            sb.append(" and ypa.publish_time <= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.publish_time <= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getEndPublishTime());
        }
        Integer activityStatusTypeId = query.getActivityStatusTypeId();
        if (null != activityStatusTypeId){
            sb.append("and ypa.ACTIVITY_STATUS_TYPE_ID = ? ");
            sbCount.append("and ypa.ACTIVITY_STATUS_TYPE_ID = ? ");
            params.add(activityStatusTypeId);
        }

        sb.append("order by ypa.create_time desc");
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<ActivityQueryRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(ActivityQueryRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<ActivityQueryRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

    public PageRsp<ActivityQueryAuditRespDTO> pageAuditList(PageReq<ActivityQueryAuditDTO> req, Long currentOrgId) {
        StringBuilder sb = new StringBuilder(PAGE_ACTIVITY_AUDIT_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_ACTIVITY_AUDIT_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        params.add(currentOrgId);
        ActivityQueryAuditDTO query = req.getQuery();
        if (StrUtil.isNotBlank(query.getActivityTitle())) {
            sb.append(" and ypa.activity_title like concat('%',?,'%') ");
            sbCount.append(" and ypa.activity_title like concat('%',?,'%') ");
            params.add(query.getActivityTitle());
        }
        if (StrUtil.isNotBlank(query.getBusinessBelong())) {
            sb.append(" and ypa.business_belong = ? ");
            sbCount.append(" and ypa.business_belong = ? ");
            params.add(query.getBusinessBelong());
        }
        if (Objects.nonNull(query.getActivityTypeId())) {
            sb.append("and ypa.activity_type_id = ? ");
            sbCount.append("and ypa.activity_type_id = ? ");
            params.add(query.getActivityTypeId());
        }
        if (Objects.nonNull(query.getOrgId())) {
            sb.append("and ypa.org_id = ? ");
            sbCount.append("and ypa.org_id = ? ");
            params.add(query.getOrgId());
        }
        if (StrUtil.isNotBlank(query.getSponsor())) {
            sb.append(" and ypa.sponsor like concat('%',?,'%') ");
            sbCount.append(" and ypa.sponsor like concat('%',?,'%') ");
            params.add(query.getSponsor());
        }
        if (StrUtil.isNotBlank(query.getStartTime())) {
            sb.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getStartTime());
        }
        if (StrUtil.isNotBlank(query.getEndTime())) {
            sb.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getEndTime());
        }

        if (StrUtil.isNotBlank(query.getStartPublishTime())) {
            sb.append(" and ypa.publish_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.publish_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getStartPublishTime());
        }

        if (StrUtil.isNotBlank(query.getEndPublishTime())) {
            sb.append(" and ypa.publish_time <= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.publish_time <= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getEndPublishTime());
        }
        Integer auditStatus = query.getAuditStatus();
        if (null != auditStatus){
            sb.append("and yafn.audit_result = ? ");
            sbCount.append("and yafn.audit_result = ? ");
            params.add(auditStatus);
        }

        sb.append("order by ypa.create_time desc");
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<ActivityQueryAuditRespDTO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(ActivityQueryAuditRespDTO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<ActivityQueryAuditRespDTO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

    public PageRsp<YcqPublishActivity> pageListForApp(PageReq<ActivityAppQueryReqDTO> req, Long currentOrgId) {
        StringBuilder sb = new StringBuilder(APP_PAGE_ACTIVITY_BASE_SQL);
        StringBuilder sbCount = new StringBuilder(APP_PAGE_ACTIVITY_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        params.add(currentOrgId);
        ActivityAppQueryReqDTO query = req.getQuery();
        // 所在区域
        List<ActivityAppQueryReqDTO.ZoneAppQueryDTO> zoneAppQueryDTOList = query.getZoneAppQueryDTOList();
        if (!CollectionUtils.isEmpty(zoneAppQueryDTOList)){
            appendZoneSql(zoneAppQueryDTOList, sb, sbCount, params, true);
        }else {
            sb.append(" where ypa.is_deleted=0 ");
            sbCount.append(" where ypa.is_deleted=0 ");
        }


        sb.append(" and PUT_ON_LIST = 1 ");
        sbCount.append(" and PUT_ON_LIST = 1 ");

        if (StrUtil.isNotBlank(query.getActivityTitle())) {
            sb.append(" and ypa.activity_title like concat('%',?,'%') ");
            sbCount.append(" and ypa.activity_title like concat('%',?,'%') ");
            params.add(query.getActivityTitle());
        }
        if (!CollectionUtils.isEmpty(query.getBusinessBelong())){
            String collect = String.join("','", query.getBusinessBelong());
            sb.append(" and ypa.business_belong in ( '").append(collect).append("') ");
            sbCount.append(" and ypa.business_belong in ( '").append(collect).append("') ");
        }
        if (Objects.nonNull(query.getActivityTypeId())) {
            sb.append("and ypa.activity_type_id = ? ");
            sbCount.append("and ypa.activity_type_id = ? ");
            params.add(query.getActivityTypeId());
        }
        // 状态
        Integer activityStatusTypeId = query.getActivityStatusTypeId();
        if (null != activityStatusTypeId){
            sb.append("and ypa.ACTIVITY_STATUS_TYPE_ID = ? ");
            sbCount.append("and ypa.ACTIVITY_STATUS_TYPE_ID = ? ");
            params.add(activityStatusTypeId);
        }
        if (StrUtil.isNotBlank(query.getStartTime())) {
            sb.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getStartTime());
        }
        if (StrUtil.isNotBlank(query.getEndTime())) {
            sb.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getEndTime());
        }
        Integer isLimit = query.getIsLimit();
        if (null != isLimit){
            sb.append("and ypa.is_limit = ? ");
            sbCount.append("and ypa.is_limit = ? ");
            params.add(isLimit);
        }

        sbCount.append(" group by ypa.id ) ypa1");
        sb.append(" group by ypa.id, ypa.create_time ");
        sb.append("order by ypa.create_time desc");
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqPublishActivity> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(YcqPublishActivity.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqPublishActivity> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

    private void appendZoneSql(List<ActivityAppQueryReqDTO.ZoneAppQueryDTO> list,StringBuilder sb ,StringBuilder sbCount, List<Object> params, boolean containsWhere){
        sb.append(" inner join YCQ_ACTIVITY_ZONE yaz on ypa.id = yaz.activity_id and (");
        sbCount.append(" inner join YCQ_ACTIVITY_ZONE yaz on ypa.id = yaz.activity_id and (");
        Map<Integer, ActivityAppQueryReqDTO.ZoneAppQueryDTO> collect1 = list.stream().collect(Collectors.toMap(ActivityAppQueryReqDTO.ZoneAppQueryDTO::getZoneType, Function.identity()));
        List<String> sqlList = new ArrayList<>(8);
        collect1.forEach((k,v) -> {
            Set<String> collect = v.getValueDTOList().stream().map(ActivityAppQueryReqDTO.ZoneAppQueryValueDTO::getValueCode).collect(Collectors.toSet());
            if (collect.contains("1") || collect.contains("2") || collect.contains("3")){
                sqlList.add("(yaz.ZONE_TYPE = ?)");
            }else {
                String sql = "(yaz.ZONE_TYPE = ? and yaz.value_code in ('1','2','3','" + String.join("','", collect) + "'))";
                sqlList.add(sql);
            }
            params.add(k);
        });
        for (int i = 0; i < sqlList.size(); i++) {
            if (i == 0){
                sb.append(sqlList.get(i));
                sbCount.append(sqlList.get(i));
            }else {
                sb.append(" or ").append(sqlList.get(i));
                sbCount.append(" or ").append(sqlList.get(i));
            }
        }
        if (containsWhere){
            sb.append(") where ypa.is_deleted=0 ");
            sbCount.append(") where ypa.is_deleted=0 ");
        }else {
            sb.append(") ");
            sbCount.append(") ");
        }
    }

    public PageRsp<YcqPublishActivity> pageListForAppWithoutToken(PageReq<ActivityAppQueryReqDTO> req) {
        StringBuilder sb = new StringBuilder(APP_PAGE_ACTIVITY_NO_TOKEN_SQL);
        StringBuilder sbCount = new StringBuilder(APP_PAGE_ACTIVITY_NO_TOKEN_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        ActivityAppQueryReqDTO query = req.getQuery();
        // 所在区域：
        List<ActivityAppQueryReqDTO.ZoneAppQueryDTO> zoneAppQueryDTOList = query.getZoneAppQueryDTOList();
        if (!CollectionUtils.isEmpty(zoneAppQueryDTOList)){
            appendZoneSql(zoneAppQueryDTOList, sb, sbCount, params, true);
        }else {
            sb.append(" where ypa.is_deleted=0 ");
            sbCount.append(" where ypa.is_deleted=0 ");
        }


        sb.append(" and PUT_ON_LIST = 1 ");
        sbCount.append(" and PUT_ON_LIST = 1 ");

        if (StrUtil.isNotBlank(query.getActivityTitle())) {
            sb.append(" and ypa.activity_title like concat('%',?,'%') ");
            sbCount.append(" and ypa.activity_title like concat('%',?,'%') ");
            params.add(query.getActivityTitle());
        }
        if (!CollectionUtils.isEmpty(query.getBusinessBelong())){
            String collect = String.join("','", query.getBusinessBelong());
            sb.append(" and ypa.business_belong in ( '").append(collect).append("') ");
            sbCount.append(" and ypa.business_belong in ( '").append(collect).append("') ");
        }
        if (Objects.nonNull(query.getActivityTypeId())) {
            sb.append("and ypa.activity_type_id = ? ");
            sbCount.append("and ypa.activity_type_id = ? ");
            params.add(query.getActivityTypeId());
        }
        // 状态
        Integer activityStatusTypeId = query.getActivityStatusTypeId();
        if (null != activityStatusTypeId){
            sb.append("and ypa.ACTIVITY_STATUS_TYPE_ID = ? ");
            sbCount.append("and ypa.ACTIVITY_STATUS_TYPE_ID = ? ");
            params.add(activityStatusTypeId);
        }
        if (StrUtil.isNotBlank(query.getStartTime())) {
            sb.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getStartTime());
        }
        if (StrUtil.isNotBlank(query.getEndTime())) {
            sb.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getEndTime());
        }
        Integer isLimit = query.getIsLimit();
        if (null != isLimit){
            sb.append("and ypa.is_limit = ? ");
            sbCount.append("and ypa.is_limit = ? ");
            params.add(isLimit);
        }

        sbCount.append(" group by ypa.id ) ypa1");
        sb.append(" group by ypa.id, ypa.create_time ");
        sb.append("order by ypa.create_time desc");
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqPublishActivity> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(YcqPublishActivity.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqPublishActivity> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

    public PageRsp<YcqPublishActivity> pageListForQZH(PageReq<ActivityAppQueryReqDTO> req, Collection<Long> activityIds) {
        StringBuilder sb = new StringBuilder(APP_PAGE_ACTIVITY_PARTICIPATE_SQL);
        StringBuilder sbCount = new StringBuilder(APP_PAGE_ACTIVITY_PARTICIPATE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        ActivityAppQueryReqDTO query = req.getQuery();
        // 所在区域：
        List<ActivityAppQueryReqDTO.ZoneAppQueryDTO> zoneAppQueryDTOList = query.getZoneAppQueryDTOList();
        if (!CollectionUtils.isEmpty(zoneAppQueryDTOList)){
            appendZoneSql(zoneAppQueryDTOList, sb, sbCount, params, true);
        }else {
            sb.append(" where ypa.is_deleted=0 ");
            sbCount.append(" where ypa.is_deleted=0 ");
        }

        String ids = activityIds.stream().map(item -> item + "").collect(Collectors.joining(","));
        sb.append(" and ypa.id in (").append(ids).append(") ").append(" and PUT_ON_LIST = 1 ");
        sbCount.append(" and ypa.id in (").append(ids).append(") ").append(" and PUT_ON_LIST = 1 ");

        if (StrUtil.isNotBlank(query.getActivityTitle())) {
            sb.append(" and ypa.activity_title like concat('%',?,'%') ");
            sbCount.append(" and ypa.activity_title like concat('%',?,'%') ");
            params.add(query.getActivityTitle());
        }
        if (!CollectionUtils.isEmpty(query.getBusinessBelong())){
            String collect = String.join("','", query.getBusinessBelong());
            sb.append(" and ypa.business_belong in ( '").append(collect).append("') ");
            sbCount.append(" and ypa.business_belong in ( '").append(collect).append("') ");
        }
        if (Objects.nonNull(query.getActivityTypeId())) {
            sb.append("and ypa.activity_type_id = ? ");
            sbCount.append("and ypa.activity_type_id = ? ");
            params.add(query.getActivityTypeId());
        }
        // 状态
        Integer activityStatusTypeId = query.getActivityStatusTypeId();
        if (null != activityStatusTypeId){
            sb.append("and ypa.ACTIVITY_STATUS_TYPE_ID = ? ");
            sbCount.append("and ypa.ACTIVITY_STATUS_TYPE_ID = ? ");
            params.add(activityStatusTypeId);
        }
        if (StrUtil.isNotBlank(query.getStartTime())) {
            sb.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getStartTime());
        }
        if (StrUtil.isNotBlank(query.getEndTime())) {
            sb.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getEndTime());
        }
        Integer isLimit = query.getIsLimit();
        if (null != isLimit){
            sb.append("and ypa.is_limit = ? ");
            sbCount.append("and ypa.is_limit = ? ");
            params.add(isLimit);
        }

        sbCount.append(" group by ypa.id ) ypa1");
        sb.append(" group by ypa.id, ypa.create_time ");
        sb.append("order by ypa.create_time desc");
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<YcqPublishActivity> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(YcqPublishActivity.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<YcqPublishActivity> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }
    public List<YcqPublishActivity> pageListForMyEvaluate(PageReq<ActivityAppQueryReqDTO> req, List<Long> ids) {
        StringBuilder sb = new StringBuilder(APP_PAGE_ACTIVITY_EVALUATE_SQL);
        StringBuilder sbCount = new StringBuilder(APP_PAGE_ACTIVITY_EVALUATE_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        ActivityAppQueryReqDTO query = req.getQuery();
        // 所在区域：
        // 所在区域：
        List<ActivityAppQueryReqDTO.ZoneAppQueryDTO> zoneAppQueryDTOList = query.getZoneAppQueryDTOList();
        if (!CollectionUtils.isEmpty(zoneAppQueryDTOList)){
            appendZoneSql(zoneAppQueryDTOList, sb, sbCount, params, false);
        }

        String idsStr = ids.stream().map(item -> ""+item ).collect(Collectors.joining(","));
        sb.append(" where ypa.id in (").append(idsStr).append(") and ypa.REPLY_TYPE = 0 ");
       // sbCount.append(" where id in (").append(idsStr).append(") ");

        sb.append(" and ypa.end_time < (current_timestamp) and end_time +3 > current_timestamp ");
       // sbCount.append(" and ypa.end_time < (current_timestamp) and end_time +3 > current_timestamp ");

        if (StrUtil.isNotBlank(query.getStartTime())) {
            sb.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.start_time >= to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getStartTime());
        }
        if (StrUtil.isNotBlank(query.getEndTime())) {
            sb.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            sbCount.append(" and ypa.end_time<=to_date(?,'yyyy-mm-dd hh24:mi:ss') ");
            params.add(query.getEndTime());
        }

        // 状态
        Integer activityStatusTypeId = query.getActivityStatusTypeId();
        if (null != activityStatusTypeId){
            sb.append("and ypa.ACTIVITY_STATUS_TYPE_ID = ? ");
            sbCount.append("and ypa.ACTIVITY_STATUS_TYPE_ID = ? ");
            params.add(activityStatusTypeId);
        }

        if (StrUtil.isNotBlank(query.getActivityTitle())) {
            sb.append(" and ypa.activity_title like concat('%',?,'%') ");
         //   sbCount.append(" and ypa.activity_title like concat('%',?,'%') ");
            params.add(query.getActivityTitle());
        }
        if (!CollectionUtils.isEmpty(query.getBusinessBelong())){
            String collect = String.join("','", query.getBusinessBelong());
            sb.append(" and ypa.business_belong in ( '").append(collect).append("') ");
          //  sbCount.append(" and ypa.business_belong in ( '").append(collect).append("') ");
        }
        if (Objects.nonNull(query.getActivityTypeId())) {
            sb.append("and ypa.activity_type_id = ? ");
         //   sbCount.append("and ypa.activity_type_id = ? ");
            params.add(query.getActivityTypeId());
        }
        Integer isLimit = query.getIsLimit();
        if (null != isLimit){
            sb.append("and ypa.is_limit = ? ");
         //   sbCount.append("and ypa.is_limit = ? ");
            params.add(isLimit);
        }

        sbCount.append(" group by ypa.id ) ypa1");
        sb.append(" group by ypa.id, ypa.create_time ");

        sb.append("order by ypa.create_time desc");
       // int total = jdbcTemplate.queryForInt(sbCount.toString(), objects);
        return jdbcTemplate.queryForBeans(YcqPublishActivity.class, sb.toString(), params.toArray());
//        List<YcqPublishActivity> result;
//        String dbType = jdbcTemplate.getDbType();
//        if (total > 0) {
//            //根据数据库类型设置limit
//            SqlUtil.appendLimit(dbType, params, sb, req);
//            result = jdbcTemplate.queryForBeans(YcqPublishActivity.class, sb.toString(), objects);
//        } else {
//            result = new ArrayList<>(0);
//        }
//        PageRsp<YcqPublishActivity> page = new PageRsp<>();
//        page.setPageNo(req.getPageNo());
//        page.setPageSize(req.getPageSize());
//        page.setTotalNum(total);
//        page.setResult(result);
//        page.next();
//        return page;
    }
    public int updateApplyCount(Long id) {
        StringBuilder stringBuilder = new StringBuilder("update ycq_publish_activity set applyer_count=applyer_count+1 where (applyer_count<limit_count or limit_count is null) and id=?");
        return jdbcTemplate.update(stringBuilder.toString(),id);
    }

    public int updateBrowserCount(Long id) {
        StringBuilder stringBuilder = new StringBuilder("update ycq_publish_activity set BROWSE_COUNT=BROWSE_COUNT+1 where  id=?");
        return jdbcTemplate.update(stringBuilder.toString(),id);
    }

    /**
     * 删除风采封面
     */
    public void deleteAnnex(Long id) {
        //删除封面
        String sql = "delete from ycq_qzh_cover where type=4 and article_id=" + id;
        jdbcTemplate.execute(sql);
    }

    public List<YcqAppConfigDTO> findSelectBusiness() {
        StringBuilder sb = new StringBuilder("select app_code,system_name from ycq_app_config_url where state=1 and system_name is not null");
        return jdbcTemplate.queryForBeans(YcqAppConfigDTO.class, sb.toString());
    }

    public void plusTimes(Long activityId){
        String sql = "update ycq_publish_activity set COPY_TIMES = COPY_TIMES + 1 where id = ?";
        jdbcTemplate.update(sql, activityId);
    }

    public void autoStartActivity(){
        Date date = new Date();
        String sql = "update ycq_publish_activity set activity_state = 1 where activity_state = 0 and IS_MANUAL_SHUTDOWN = 0 and START_TIME < ? and end_time > ?";
        jdbcTemplate.update(sql, date, date);
    }

    public void autoStopActivity(){
        Date date = new Date();
        String sql = "update ycq_publish_activity set activity_state = 0 where activity_state = 1 and end_time < ?";
        jdbcTemplate.update(sql, date);
    }

    public YcqAppConfigDTO findAccessInfoById(String appCode){
        String sql = "select a.system_name, a.logo_url from ycq_app_config_url a where app_code =  ?";
        return jdbcTemplate.queryForBean(YcqAppConfigDTO.class, sql, appCode);
    }

    public List<YcqActivityTypeVO> findActivityTypeList(List<String> codeList){
        String sql = "select value_name activityTypeName,value_id activityTypeId  from tsm_property_value where state=1 and prop_code in('";
        String sql1 = sql + String.join("','", codeList) + "')";
        return jdbcTemplate.queryForBeans(YcqActivityTypeVO.class, sql1);
    }

    public YcqAppConfigDTO findActivityTypeByCodeAndId(Integer id){
        String sql = "select value_code appCode, value_name systemName  from tsm_property_value  where value_id = ?";
        return jdbcTemplate.queryForBean(YcqAppConfigDTO.class, sql, id);
    }

    public int countByActivityTitle(String title){
        String sql = "select count(*) from ycq_publish_activity where is_deleted = 0 and  ACTIVITY_TITLE = ?";
        return jdbcTemplate.queryForInt(sql, title);
    }

    public int countByActivityTitleAndIdNotEqual(String title, Long id){
        String sql = "select count(*) from ycq_publish_activity where is_deleted = 0 and  ACTIVITY_TITLE = ? and id != ?";
        return jdbcTemplate.queryForInt(sql, title, id);
    }

    public FeedbackLimitDTO feedbackLimit(Long activityId){
        String sql = "select * from ycq_publish_activity where id = ?";
        YcqPublishActivity ycqPublishActivity = jdbcTemplate.queryForBean(YcqPublishActivity.class, sql, activityId);
        FeedbackLimitDTO feedbackLimitDTO = new FeedbackLimitDTO();
        if (null != ycqPublishActivity){
            if (Objects.equals(ycqPublishActivity.getIsLimit(), 1) && Objects.equals(ycqPublishActivity.getIsApply(), 0)){
                feedbackLimitDTO.setFeedbackLimit(true);
                feedbackLimitDTO.setLimitCount(ycqPublishActivity.getLimitCount());
            }
        }
        return feedbackLimitDTO;
    }

    /**
     * 返回true表示允许，false为不允许审核通过
     * @param activityId
     * @param count
     * @return
     */
    public boolean updateAuditCount(Long activityId, int count){
        boolean flag = false;
        if (count == 0){
            return flag = true;
        }
        String sql = "select audit_count, NEEDED_AUDIT,LIMIT_COUNT,IS_LIMIT   from ycq_publish_activity where id = ?";
        YcqPublishActivity ycqPublishActivity = jdbcTemplate.queryForBean(YcqPublishActivity.class, sql, activityId);
        if (Objects.equals(ycqPublishActivity.getIsLimit(), 1)){
            if (Objects.equals(ycqPublishActivity.getNeededAudit(), 1)){
                Integer limitCount = ycqPublishActivity.getLimitCount();
                Integer auditCount = ycqPublishActivity.getAuditCount();
                int i = limitCount - auditCount;
                flag = (i - count) >= 0;
                if (flag){
                    // 修改审核通过人数
                    String updateSql = "update ycq_publish_activity set audit_count = ? where id = ?";
                    auditCount+= count;
                    jdbcTemplate.update(updateSql, auditCount, activityId);
                }
            }
        }else {
            flag = true;
        }
        return flag;
    }



    public List<CoverImageDto> getByArticleIdAndTye(Long id, int type){
        String sql = "select file_id, file_key, file_name, file_type,file_size, pic_width,pic_height,file_url,type file_type  from ycq_qzh_cover where article_id = ? and type = ?";
        return jdbcTemplate.queryForBeans(CoverImageDto.class, sql, id, type);
    }

    public List<BelongBusinessVo> getBelongBUsiness() {
        String sql = " select * from ycq_app_config_url a where a.state = 1\n" +
                "  and app_code not in (select value_code from TSM_PROPERTY_VALUE where prop_code='activityBusinessExclude' and state=1) ";
        return jdbcTemplate.queryForBeans(BelongBusinessVo.class,sql);
    }

    /**
     * 审核状态修改
     * @param activityId 活动id
     * @param auditOrgId 当前审核组织id
     * @param resultType 审核结果 1审核通过 2审核通过并交上级审核  3审核不通过
     */
    public void updateAuditResultById(Long activityId, Long auditOrgId, Integer resultType){
        if (Objects.equals(1, resultType)){
            String sql = "update ycq_publish_activity set PUBLISH_AUDIT_STATUS = 1, PUT_ON_LIST = 3, ACTIVITY_STATUS_TYPE_ID = 2 where id = ?";
            jdbcTemplate.update(sql, activityId);
        }else if (Objects.equals(3, resultType)){
            String sql = "update ycq_publish_activity set PUBLISH_AUDIT_STATUS = 3 ,ACTIVITY_STATUS_TYPE_ID = 11 where id = ?";
            jdbcTemplate.update(sql, activityId);
        }else if (Objects.equals(2, resultType)){
            String sql = "update ycq_publish_activity set PUBLISH_AUDIT_STATUS = 2,ACTIVITY_STATUS_TYPE_ID = 10 where id = ?";
            jdbcTemplate.update(sql, activityId);
        }
    }

    public void reAudit(Long activityId){
        String sql = "update ycq_publish_activity set PUBLISH_AUDIT_STATUS = 4,ACTIVITY_STATUS_TYPE_ID = 1 where id = ?";
        jdbcTemplate.update(sql, activityId);
    }

    private final String BASE_ACTIVITY_SQL = "select ypa.id, ypa.ACTIVITY_TITLE from  ycq_publish_activity ypa inner join tsm_worker_org_all twoa on ypa.org_id=twoa.org_id where twoa.worker_id= ? and ypa.is_deleted=0";
    private final String BASE_ACTIVITY_COUNT_SQL = "select count(*) from  ycq_publish_activity ypa inner join tsm_worker_org_all twoa on ypa.org_id=twoa.org_id where twoa.worker_id= ? and ypa.is_deleted=0";
    public PageRsp<ActivityBaseVO> queryActivityBaseInfo(PageReq<ActivityBaseDTO> req, Integer loginId) {
        StringBuilder sb = new StringBuilder(BASE_ACTIVITY_SQL);
        StringBuilder sbCount = new StringBuilder(BASE_ACTIVITY_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        params.add(loginId);
        String activityTitle = req.getQuery().getActivityTitle();
        if (StrUtil.isNotBlank(activityTitle)){
            sb.append(" and ypa.activity_title like concat('%',?,'%') ");
            sbCount.append(" and ypa.activity_title like concat('%',?,'%') ");
            params.add(activityTitle);
        }
        sb.append("order by ypa.create_time desc");
        int total = jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<ActivityBaseVO> result;
        String dbType = jdbcTemplate.getDbType();
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            result = jdbcTemplate.queryForBeans(ActivityBaseVO.class, sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<ActivityBaseVO> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        page.next();
        return page;
    }

    public List<YcqPublishActivity> autoChangeStatusList(){
        String sql = "select id,ACTIVITY_STATUS_TYPE_ID  from ycq_publish_activity where is_deleted = 0 and ACTIVITY_STATUS_TYPE_ID in(3,4,5,6,7) and put_on_list = 1";
        return jdbcTemplate.queryForBeans(YcqPublishActivity.class, sql);
    }

    public void setStatus(Long id, Integer status){
        String sql = "update ycq_publish_activity set ACTIVITY_STATUS_TYPE_ID = ? where id = ?";
        jdbcTemplate.update(sql, status, id);
    }

    public YcqActiveFlowNodeReqDTO getInitNode(Long activityId){
        String sql = "select org_id,org_name from YCQ_ACTIVE_FLOW_NODE where biz_id = ? and is_initial = 1";
        return jdbcTemplate.queryForBean(YcqActiveFlowNodeReqDTO.class,sql,  activityId);
    }
}
