package com.crunii.micro.service.qzh.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.*;
import com.crunii.micro.service.qzh.dto.TestPaperExtentVO;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.http.ResponseEntity;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author wangchen create 2023/11/13 16:53
 */
public interface CivilizationReportService extends InitializingBean {

    /**
     * 文明号申报创建
     *
     * @param civilizationReportDto 创建参数
     * @param worker worker
     * @return str
     */
    String crateCivilizationReport(CivilizationReportDto civilizationReportDto, Worker worker);

    /**
     * 分页查询文明号申报记录
     *
     * @param req    分页参数
     * @param worker 当前用户
     * @return data
     */
    PageRsp<CivilizationReportVo> pageCivilizationReport(PageReq<CivilizationReportPageDto> req, Worker worker);

    /**
     * 查询政策
     *
     * @param declarationType 查询类型 1 创建 2 复核
     * @return 政策id+名字
     */
    List<DocVo> findDoc(Integer declarationType);

    /**
     * 编辑数据回显
     *
     * @param declarationId id
     * @param worker        worker
     * @return data
     */
    CivilizationReportDto findDraft(Long declarationId, Worker worker);

    /**
     * 删除申报
     *
     * @param declarationId 主键id
     * @param worker        worker
     * @return str
     */
    String delDraft(Long declarationId, Worker worker);

    /**
     * 1 批量初评 2 批量审核 3 批量初核
     *
     * @param reviewDto 请求参数
     * @param worker    worker
     * @return str
     */
    String updateReview(ReviewDto reviewDto, Worker worker);

    /**
     * 回退流程
     *
     * @param backFlowDto 参数
     * @param worker      worker
     * @return 回退流程
     */
    String updateBackFlow(BackFlowDto backFlowDto, Worker worker);

    /**
     * 分页查询我的申报
     *
     * @param worker 当前用户
     * @param req    分页参数
     * @return data
     */
    PageRsp<CivilizationReportVo> pageMyselfReport(PageReq<Object> req, Worker worker);

    /**
     * 获取可回得流程节点
     *
     * @param declarationId 主键id
     * @param worker        当前worker
     * @return data
     */
    List<Integer> getBackFlowIds(Long declarationId, Worker worker);

    /**
     * 查询pdf
     *
     * @param declarationId 主键id
     * @param response      响应
     * @return 流
     */
    ResponseEntity<byte[]> findPdf(Long declarationId, HttpServletResponse response);

    /**
     * 创建考试成绩信息
     *
     * @param req 请求参数
     * @return str
     */
    String createExamResult(ExamResultDto req);


    /**
     * 创建申报 可查询的组织
     *
     * @param queryOrgDto 创建参数
     * @param worker      当前用户
     * @return str
     */
    String saveQueryOrg(ReportOrgDto queryOrgDto, Worker worker);

    /**
     * 查询申报id
     *
     * @param queryOrgDto 查询参数
     * @return data
     */
    List<Long> queryServiceIds(QueryOrgDto queryOrgDto);

    /**
     * 查询当前用户组织权限 0 查看 1 市级 2 区县
     * @param worker 当前用户
     * @return 组织权限
     */
    Integer findWorkerRole(Worker worker);

    /**
     * 查询待处理清单个数
     * @param worker 当前用户
     * @return 待处理个数
     */
    int findReportCount(Worker worker);

    /**
     * 查询当前用户申报记录 草稿个数
     * @param worker 当前用户
     * @return 待处理个数
     */
    int findCurrentReportCount(Worker worker);

    /**
     * 保存草稿
     * @param civilizationReportDto 参数
     * @param worker 用户
     * @return data
     */
    String crateCivilizationReportDraft(CivilizationReportDto civilizationReportDto, Worker worker);

    /**
     * 查询当前用户是否有考试
     * @param worker 当前用户
     * @return 1有 0 无
     */
    TestPaperExtentVO findWorkerExam(Worker worker);

    /**
     * 查询考试此时
     * @param paperId 试卷id
     * @param worker 当前用户
     * @return 次数
     */
    TestPaperVO findWorkerExamCount(Long paperId, Worker worker);
}
