package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.json.JSONObject;
import com.alibaba.nacos.shaded.com.google.gson.Gson;
import com.alibaba.nacos.shaded.com.google.gson.JsonObject;
import com.crunii.micro.common.dto.Result;

import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqActivityPeopleAudit;
import com.crunii.micro.domain.entity.main.YcqActivitySign;

import com.crunii.micro.domain.entity.main.YcqPublishActivity;
import com.crunii.micro.domain.repo.main.PublishActivityRepo;
import com.crunii.micro.domain.repo.main.YcqActivityPeopleAuditRepo;
import com.crunii.micro.domain.repo.main.YcqActivitySignRepo;
import com.crunii.micro.service.qzh.api.domain.dto.activity.LocationDataDTO;
import com.crunii.micro.service.qzh.api.domain.dto.LocationDataVO;

import com.crunii.micro.service.qzh.api.domain.dto.activity.QrCodeInfo;
import com.crunii.micro.service.qzh.api.enums.ActivityButtonStateEnum;
import com.crunii.micro.service.qzh.config.ActivityStateFactory;
import com.crunii.micro.service.qzh.dto.YcqActivityPeopleAuditDTO;
import com.crunii.micro.service.qzh.service.*;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;


/**
 * @author lmlksy
 * @date 2023/11/21 10:52
 */
@Service
public class ActivityCheckInServiceImpl implements ActivityCheckInService, LocationDataValidator {

    @Autowired
    private YcqActivitySignRepo ycqActivitySignRepo;

    @Autowired
    private CheckInCommonLogicImpl checkInCommonLogicImpl;


    @Autowired
    private ActivityStateFactory activityStateFactory;

    @Autowired
    private RegistrationAuditService registrationAuditService;

    @Autowired
    private YcqEventReportService ycqEventReportService;

    @Autowired
    private PublishActivityRepo publishActivityRepo;


    @Override
    public Result<LocationDataVO> doCalculateDistance(LocationDataDTO locationDataDTO) {
        return checkIn(locationDataDTO.getLatitude(),locationDataDTO.getLongitude(),
                locationDataDTO.getActivityLatitude(), locationDataDTO.getActivityLongitude()
                );
    }

    @Override
    public Result<String> doLocation(LocationDataDTO locationDataDTO, Worker worker) {
//        Worker worker1 = new Worker();
//        worker1.setLoginId(9999);
//        worker=worker1;
        validateLocationDataDTO(locationDataDTO);
        validateActivityBusiness(locationDataDTO);
        Result<LocationDataVO> locationDataVOResult = checkIn(locationDataDTO.getLatitude(), locationDataDTO.getLongitude(),
                locationDataDTO.getActivityLatitude(), locationDataDTO.getActivityLongitude()
        );
        YcqActivityPeopleAuditDTO ycqActivityPeopleAuditDTO = new YcqActivityPeopleAuditDTO(locationDataDTO.getActivityId(), ActivityButtonStateEnum.SIGN_OUT_NOW.getCode());
        //新增用户
        registrationAuditService.saveYcqActivityPeopleAudit2(ycqActivityPeopleAuditDTO);
        if (!locationDataVOResult.isSuccess()){
            return Result.ofFail("当前不在签到范围内");
        }
        if (locationDataDTO.getClockTag()==1){
            Result<YcqActivityPeopleAudit> ycqActivityPeopleAuditResult =activityStateFactory.getActivityState(locationDataDTO.getClockTag()).handleActivityState(locationDataDTO, worker);
            ycqActivityPeopleAuditResult.getData().setCountTrue(1);
            return checkInCommonLogicImpl.checkIn(ycqActivityPeopleAuditResult.getData(),Boolean.TRUE);
        }else {
            Result<YcqActivityPeopleAudit> ycqActivityPeopleAuditResult =activityStateFactory.getActivityState(locationDataDTO.getClockTag()).handleActivityState(locationDataDTO, worker);
            ycqActivityPeopleAuditResult.getData().setCountTrue(2);
            return checkInCommonLogicImpl.checkIn(ycqActivityPeopleAuditResult.getData(),Boolean.FALSE);
        }

    }

    private void validateActivityBusiness(LocationDataDTO locationDataDTO) {
        YcqPublishActivity byIdAndIsDeleted = publishActivityRepo.findByIdAndIsDeleted(locationDataDTO.getActivityId(), 0);
        if (byIdAndIsDeleted.getBusinessBelong().equals("youthPromise")){
            Boolean aBoolean = ycqEventReportService.promiseYouth();
            if (!aBoolean){
                throw new BusinessException("当前活动需先前往青春之约进行注册！");
            }
        }
    }

    @Override
    public Result<String> doScan(LocationDataDTO locationDataDTO, Worker worker) {
//        Worker worker1 = new Worker();
//        worker1.setLoginId(9999);
//        worker=worker1;
        validateActivityBusiness(locationDataDTO);
        String qrId="";
        try {
            String qrCodeString = locationDataDTO.getQrCodeString();
            JSONObject jsonObject = new JSONObject(qrCodeString);
            // 获取 qrId 的值
            qrId = jsonObject.getStr("qrId");
        }catch (Exception e){
            throw new BusinessException("请使用活动二维码!");
        }
        if (qrId==null||qrId.equals("")){
            throw new BusinessException("请使用活动二维码!");
        }
        validateLocationDataDTO(locationDataDTO);

        YcqActivitySign sign = ycqActivitySignRepo.findByActivityId(locationDataDTO.getActivityId());

        YcqActivityPeopleAuditDTO ycqActivityPeopleAuditDTO = new YcqActivityPeopleAuditDTO(locationDataDTO.getActivityId(), ActivityButtonStateEnum.SIGN_OUT_NOW.getCode());
        //新增用户
        registrationAuditService.saveYcqActivityPeopleAudit2(ycqActivityPeopleAuditDTO);
        if (locationDataDTO.getClockTag()==1){
            if (sign == null||sign.getQdCode()==null) {
                throw new BusinessException("没有找到对应签到二维码信息！");
            }

            String[] qrCodes = sign.getQdCode().split(",");
            boolean found =false;
            for (String qrCode : qrCodes) {
                if (qrCode.equals(qrId)) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                throw new BusinessException("请确认扫描签到二维码正确!");
            }
            Result<YcqActivityPeopleAudit> ycqActivityPeopleAuditResult = activityStateFactory.getActivityState(locationDataDTO.getClockTag()).handleActivityState(locationDataDTO, worker);
            //这里已经算通过了，设置已扫描标识
            ycqActivityPeopleAuditResult.getData().setCountFalse(1);
            return checkInCommonLogicImpl.checkIn(ycqActivityPeopleAuditResult.getData(),Boolean.TRUE);
        }else {
            if (sign == null||sign.getQtCode()==null) {
                throw new BusinessException("没有找到对应签退二维码信息！");
            }

            String[] qtCodes = sign.getQtCode().split(",");
            boolean found =false;
            for (String qtCode : qtCodes) {
                if (qtCode.equals(qrId)) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                throw new BusinessException("请确认扫描签退二维码正确!");
            }
            Result<YcqActivityPeopleAudit> ycqActivityPeopleAuditResult = activityStateFactory.getActivityState(locationDataDTO.getClockTag()).handleActivityState(locationDataDTO, worker);
            ycqActivityPeopleAuditResult.getData().setCountFalse(2);
            return checkInCommonLogicImpl.checkIn(ycqActivityPeopleAuditResult.getData(),Boolean.FALSE);
        }

    }

    // 签到方法，根据距离进行签到提示
    public Result<LocationDataVO> checkIn(String lat1, String lon1, String lat2, String lon2) {
        try {
            double latitude1 = Double.parseDouble(lat1);
            double longitude1 = Double.parseDouble(lon1);
            double latitude2 = Double.parseDouble(lat2);
            double longitude2 = Double.parseDouble(lon2);

            double distance = calculateDistance(latitude1, longitude1, latitude2, longitude2);

            if (distance > 100) {
                return Result.ofFail("当前不在签到范围内！",new LocationDataVO().setDistance(Math.round(distance)));
            } else {
                return Result.ofSuccessWithMsg(new LocationDataVO().setDistance(Math.round(distance)),"到达目的地，是否签到！");
            }
        } catch (NumberFormatException e) {
            throw new BusinessException("输入格式错误，请输入正确的经度和纬度！");
        }
    }
    // 使用Haversine公式计算两个经纬度之间的距离（单位：米）
    public double calculateDistance(double lat1, double lon1, double lat2, double lon2) {
        final int R = 6371; // 地球半径（单位：千米）

        double latDistance = Math.toRadians(lat2 - lat1);
        double lonDistance = Math.toRadians(lon2 - lon1);
        double a = Math.sin(latDistance / 2) * Math.sin(latDistance / 2)
                + Math.cos(Math.toRadians(lat1)) * Math.cos(Math.toRadians(lat2))
                * Math.sin(lonDistance / 2) * Math.sin(lonDistance / 2);
        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        double distance = R * c * 1000; // 转换为米

        return distance;
    }

}
