package com.crunii.micro.service.qzh.service.impl;

import com.alibaba.excel.util.CollectionUtils;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.domain.dto.message.PushMsgTemplateReqBo;
import com.crunii.micro.domain.entity.main.YcqActivityPeopleAudit;
import com.crunii.micro.domain.entity.main.YcqActivitySign;
import com.crunii.micro.domain.entity.main.YcqPublishActivity;
import com.crunii.micro.domain.repo.main.MessageSendLogTaskRepo;
import com.crunii.micro.domain.repo.main.PublishActivityRepo;
import com.crunii.micro.domain.repo.main.YcqActivityPeopleAuditRepo;
import com.crunii.micro.domain.repo.main.YcqActivitySignRepo;
import com.crunii.micro.feign.client.MessageCommonService;
import com.crunii.micro.service.qzh.enums.ActivityPushMsgConstants;
import com.crunii.micro.service.qzh.service.ActivityMessageSendTaskService;
import feign.FeignException;
import lombok.extern.log4j.Log4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author lmlksy
 * @date 2024/1/8 15:18
 */
@Log4j
@Service
public class ActivityMessageSendTaskServiceImpl implements ActivityMessageSendTaskService {

    @Autowired
    PublishActivityRepo publishActivityRepo;

    @Autowired
    MessageCommonService messageCommonService;

    @Autowired
    YcqActivityPeopleAuditRepo ycqActivityPeopleAuditRepo;

    @Autowired
    YcqActivitySignRepo ycqActivitySignRepo;

    @Override
    @Scheduled(cron = "0 */10 * * * *") // 每25分钟执行一次
    public void doActivityMessageSenTask() {
        LocalDateTime currentTime = LocalDateTime.now();

        List<YcqPublishActivity> all = publishActivityRepo.findAll();

        if (!CollectionUtils.isEmpty(all)) {
            List<YcqPublishActivity> filteredList = all.stream()
                    .filter(activity -> activity.getActivityState() != null && activity.getActivityState() == 1)
                    .filter(activity -> activity.getIsDeleted() != null && activity.getIsDeleted() == 0)
                    .collect(Collectors.toList());
            log.info("filteredList:"+filteredList);
            for (YcqPublishActivity activity : filteredList) {
                LocalDateTime startTime = activity.getStartTime();
                List<YcqActivityPeopleAudit> ycqActivityPeopleAudits = ycqActivityPeopleAuditRepo.findByActivityIdAndIsDeleted(activity.getId(), 0);
                YcqActivitySign sign = ycqActivitySignRepo.findByActivityId(activity.getId());

                if (startTime != null) {
                    long minutesUntilStartTime = java.time.Duration.between(currentTime, startTime).toMinutes();
                    if (-minutesUntilStartTime >= -1440 && -minutesUntilStartTime < -1430) {
                        sendPushNotification(ActivityPushMsgConstants.SCENE_NAME_ACTIVITY_START_INFORM, ActivityPushMsgConstants.TITLE_ACTIVITY_START_INFORM, startTime, ycqActivityPeopleAudits);
                    }
                }
                if (sign!=null){
                    if (sign.getSignInStartTime() != null) {
                        LocalDateTime signInStartTime = DateUtils.dateToLocalDateTime(sign.getSignInStartTime());
                        Duration timeDifference = Duration.between(currentTime, signInStartTime);
                        log.info("当前时间:"+currentTime);
                        log.info("开始签到时间:"+signInStartTime);
                        log.info("timeDifference.toMinutes():"+timeDifference.toMinutes());
                        if (-timeDifference.toMinutes() >= -1440&& -timeDifference.toMinutes() < -1430) {
                            sendPushNotification(ActivityPushMsgConstants.SCENE_NAME_ACTIVITY_CHECK_IN, ActivityPushMsgConstants.TITLE_ACTIVITY_CHECK_IN, signInStartTime, ycqActivityPeopleAudits);
                        }
                    }

                    if (sign.getSignOutStartTime() != null) {
                        LocalDateTime signOutStartTime = DateUtils.dateToLocalDateTime(sign.getSignOutStartTime());
                        Duration timeDifference = Duration.between(currentTime, signOutStartTime);
                        log.info("sign.getSignInStartTime():"+signOutStartTime);
                        if (-timeDifference.toMinutes() >= -1440&& -timeDifference.toMinutes() < -1430) {
                            sendPushNotification(ActivityPushMsgConstants.SCENE_NAME_ACTIVITY_CHECK_OUT, ActivityPushMsgConstants.TITLE_ACTIVITY_CHECK_OUT, signOutStartTime, ycqActivityPeopleAudits);
                        }
                    }
                }
            }
        }
    }

    private void sendPushNotification(String sceneName, String title, LocalDateTime time, List<YcqActivityPeopleAudit> ycqActivityPeopleAudits) {

        Map<String, Object> params = new HashMap<>(16);
        params.put("time", DateUtils.formatLocalDateTime(time));
        PushMsgTemplateReqBo pushMsgTemplateReqBo = new PushMsgTemplateReqBo();
        pushMsgTemplateReqBo.setSystemName(ActivityPushMsgConstants.SYSTEM_NAME);
        pushMsgTemplateReqBo.setSceneName(sceneName);
        pushMsgTemplateReqBo.setTitle(title);
        pushMsgTemplateReqBo.setMap(params);
        if (!CollectionUtils.isEmpty(ycqActivityPeopleAudits)) {
            List<YcqActivityPeopleAudit> activityPeopleAudits = ycqActivityPeopleAudits.stream()
                    .filter(audit -> audit.getRegistrationStatus() != null && audit.getRegistrationStatus() == 1)
                    .filter(audit -> audit.getIsDeleted() != null && audit.getIsDeleted() == 0)
                    .collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(activityPeopleAudits)) {
                for (YcqActivityPeopleAudit e : activityPeopleAudits) {
                        log.info("调用远程:"+pushMsgTemplateReqBo+e.getUserId());
                        try {
                            messageCommonService.pushMsg(pushMsgTemplateReqBo,e.getUserId());
                        } catch (Exception e1) {
                            log.error("调用远程 pushMsg失败: " + e1.getMessage());
                            return;
                        }
                }
            }
            ycqActivityPeopleAuditRepo.saveAll(activityPeopleAudits);

        }
    }

}
