package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.domain.entity.main.YcqActivitySign;
import com.crunii.micro.domain.entity.main.YcqPublishActivity;
import com.crunii.micro.domain.repo.main.PublishActivityRepo;
import com.crunii.micro.domain.repo.main.YcqActivitySignRepo;
import com.crunii.micro.service.qzh.service.ActivityStatusCheckerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Date;

/**
 * @author lmlksy
 * @date 2024/1/17 9:53
 */
@Service
public class ActivityStatusCheckerServiceImpl implements ActivityStatusCheckerService {

    @Autowired
    private  PublishActivityRepo publishActivityRepo;

    @Autowired
    private YcqActivitySignRepo ycqActivitySignRepo;

    @Override
    public Integer getActivityStatusTypeId(Long activityId) {
        YcqPublishActivity activityDetails = fetchActivityById(activityId);

        YcqActivitySign ycqActivitySign = fetchActivitySignById(activityId);
        if (ObjectUtil.isEmpty(activityDetails)){
            throw new BusinessException("活动不能为空！");
        }
        if (activityDetails != null) {
            Integer publishAuditStatus = activityDetails.getPublishAuditStatus();
            Integer putOnList = activityDetails.getPutOnList();
            LocalDateTime startTime = activityDetails.getStartTime();
            LocalDateTime endTime = activityDetails.getEndTime();
            Integer isApply = activityDetails.getIsApply();
            Date applyStartTime = activityDetails.getApplyStartTime();
            Date applyEndTime = activityDetails.getApplyEndTime();
            Integer neededSign = activityDetails.getNeededSign();


//            if (isAuditPassedAndNotStarted(publishAuditStatus)) {
//                return 1;//待审核 PUBLISH_AUDIT_STATUS=0
//            } else if (isOnShelfPending(publishAuditStatus,putOnList)) {
//                return 2;//2待发布 PUBLISH_AUDIT_STATUS =1 and PUT_ON_LIST=2
//            } else if (isAuditPassedAndOnShelf(publishAuditStatus,putOnList, startTime)) {
            if (isAuditPassedAndOnShelf(publishAuditStatus,putOnList, startTime, isApply, applyStartTime)) {
                return 3;//3未开始 PUBLISH_AUDIT_STATUS=1 and PUT_ON_LIST=1 早于开始时间
            } else if (isApplyInProgress(isApply, applyStartTime, applyEndTime)) {
                return 4;//4报名中 IS_APPLY=1 判断当前时间在APPLY_START_TIME和APPLY_END_TIME中
            }  else if (isAfterActivityEnd(endTime)) {
                return 8;
            }

            if (!ObjectUtil.isEmpty(ycqActivitySign)&&neededSign==1){
                Date signInStartTime = ycqActivitySign.getSignInStartTime();
                Date signInEndTime = ycqActivitySign.getSignInEndTime();
                Date signOutStartTime = ycqActivitySign.getSignOutStartTime();
                Date signOutEndTime = ycqActivitySign.getSignOutEndTime();
                if (isNotInRegistrationSignAndNotInProgress(applyEndTime, signInStartTime, signInEndTime,signOutStartTime,signOutEndTime)) {
                    return 5;//5进行中 报名时间之后且在活动开始和结束时间，并且不在签到开始时间和签到结束时间内和不在签退开始时间和签退结束时间内
                } else if (isSignRequiredAndInProgress(neededSign, signInStartTime, signInEndTime)) {
                    return 6;  //NEEDED_SIGN=1，并且判断在签到开始时间和签到结束时间
                }else if (isSignRequiredAndInProgress(neededSign, signOutStartTime, signOutEndTime)) {
                    return 7; // 7: 签退中
                }
                if (isAuditPassedAndOnShelf2(startTime,applyEndTime)){
                    return 3;
                }
            }
        if (isWithinRange(startTime,endTime)){
            return 5;
        }

        }
    return 3;
    }

    private YcqPublishActivity fetchActivityById(Long activityId) {
        return publishActivityRepo.findByIdAndIsDeleted(activityId,0);
    }

    private YcqActivitySign fetchActivitySignById(Long activityId) {
        return ycqActivitySignRepo.findByActivityId(activityId);
    }

    private boolean isAuditPassedAndNotStarted(Integer publishAuditStatus) {
        return publishAuditStatus == 0;
    }

    private boolean isOnShelfPending(Integer publishAuditStatus,Integer putOnList) {
        return publishAuditStatus == 1 && putOnList == 2;
    }

    private boolean isAuditPassedAndOnShelf(Integer publishAuditStatus,Integer putOnList, LocalDateTime startTime, Integer isApply, Date applyStartTime) {
        LocalDateTime now = LocalDateTime.now();
        if (isApply == 0){
            return publishAuditStatus == 1 && now.isBefore(startTime);
        }
        return publishAuditStatus == 1 && now.isBefore(DateUtils.dateToLocalDateTime(applyStartTime));
    }

    private boolean isAuditPassedAndOnShelf2(LocalDateTime startTime,Date applyEndTime) {
        LocalDateTime now = LocalDateTime.now();
        if (startTime!=null&&applyEndTime!=null){
            return  now.isAfter(DateUtils.dateToLocalDateTime(applyEndTime))&&now.isBefore(startTime);
        }
        return false;
    }

    private boolean isApplyInProgress(int isApply, Date applyStartTime, Date applyEndTime) {
        return isApply == 1 && isWithinRange(applyStartTime, applyEndTime);
    }

    private boolean isNotInRegistrationSignAndNotInProgress(Date applyEndTime, Date signInStartTime, Date signInEndTime,
                                                            Date signOutStartTime, Date signOutEndTime) {

        if (applyEndTime!=null&&signInStartTime!=null&&signInEndTime!=null&&signOutStartTime!=null&&signOutEndTime!=null) {
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime applyEndDateTime = DateUtils.dateToLocalDateTime(applyEndTime);
            LocalDateTime signInStartDateTime = DateUtils.dateToLocalDateTime(signInStartTime);
            LocalDateTime signInEndDateTime = DateUtils.dateToLocalDateTime(signInEndTime);
            LocalDateTime signOutStartDateTime = DateUtils.dateToLocalDateTime(signOutStartTime);
            LocalDateTime signOutEndDateTime = DateUtils.dateToLocalDateTime(signOutEndTime);


            return now.isAfter(applyEndDateTime)
                    && now.isBefore(signInStartDateTime) && now.isAfter(signInEndDateTime)
                    && now.isBefore(signOutStartDateTime) && now.isAfter(signOutEndDateTime);
        }
        return false;
    }


    private boolean isSignRequiredAndInProgress(Integer neededSign, Date signInStartTime, Date signInEndTime) {
        if (neededSign == 1&&signInStartTime!=null&&signInEndTime!=null) {
            LocalDateTime now = LocalDateTime.now();

            LocalDateTime signInStartDateTime = DateUtils.dateToLocalDateTime(signInStartTime);
            LocalDateTime signInEndDateTime = DateUtils.dateToLocalDateTime(signInEndTime);

            return now.isAfter(signInStartDateTime) && now.isBefore(signInEndDateTime);
        }

        return false;
    }


    private boolean isAfterActivityEnd(LocalDateTime endTime) {
        return LocalDateTime.now().isAfter(endTime);
    }

    private boolean isWithinRange(Date start, Date end) {
        LocalDateTime now = LocalDateTime.now();
        return now.isAfter(DateUtils.dateToLocalDateTime(start))
                && now.isBefore(DateUtils.dateToLocalDateTime(end));
    }

    private boolean isWithinRange(LocalDateTime start, LocalDateTime end) {
        LocalDateTime now = LocalDateTime.now();
        if (start!=null&&end!=null){
            return now.isAfter(start) && now.isBefore(end);
        }
        return false;
    }
}
