package com.crunii.micro.service.qzh.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqActivityPeopleAudit;
import com.crunii.micro.domain.entity.main.YcqActivitySign;
import com.crunii.micro.domain.entity.main.YcqPublishActivity;
import com.crunii.micro.domain.repo.main.PublishActivityRepo;
import com.crunii.micro.domain.repo.main.YcqActivityPeopleAuditRepo;
import com.crunii.micro.domain.repo.main.YcqActivitySignRepo;
import com.crunii.micro.service.qzh.api.domain.dto.activity.LocationDataDTO;
import com.crunii.micro.service.qzh.api.enums.ActivityButtonStateEnum;
import com.crunii.micro.service.qzh.service.ActivityState;
import com.crunii.micro.service.qzh.service.CheckInCommonLogic;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Optional;

/** 签到校验
 * @author lmlksy
 * @date 2023/12/2 14:21
 */
@Service
public class CheckInActivityStateImpl implements ActivityState {

    @Autowired
    YcqActivityPeopleAuditRepo ycqActivityPeopleAuditRepo;

    @Autowired
    PublishActivityRepo publishActivityRepo;

    @Autowired
    YcqActivitySignRepo ycqActivitySignRepo;

    @Autowired
    private CheckOutCommonLogicImpl checkOutCommonLogicImpl;
    @Override
    public Result<YcqActivityPeopleAudit> handleActivityState(LocationDataDTO locationDataDTO, Worker worker) {

        //根据审核状态或流程状态
        YcqActivityPeopleAudit activityPeopleAudit = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(Long.valueOf(worker.getLoginId()), locationDataDTO.getActivityId(),0,1);

        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(locationDataDTO.getActivityId());
        if (publishActivity.isPresent()){
            YcqPublishActivity ycqPublishActivity = publishActivity.get();
            //需要报名
            if (ycqPublishActivity.getIsApply()!=0){
                if (activityPeopleAudit==null) {
                    throw new BusinessException("当前人员审核未通过，无法进行签到！");
                }
            }
            YcqActivitySign ycqActivitySign = ycqActivitySignRepo.findByActivityId(ycqPublishActivity.getId());
            if (ycqActivitySign.getSignInStartTime()==null){
                throw new BusinessException( "请先配置签到时间！");
            }
            if (LocalDateTime.now().isBefore(DateUtils.dateToLocalDateTime(ycqActivitySign.getSignInStartTime()))) {
                checkOutCommonLogicImpl.checkIn(activityPeopleAudit,Boolean.TRUE);
                throw new BusinessException("还未到达签到时间！");
            } else if (LocalDateTime.now().isAfter(DateUtils.dateToLocalDateTime(ycqActivitySign.getSignInEndTime()))) {
                checkOutCommonLogicImpl.checkIn(activityPeopleAudit,Boolean.FALSE);
                throw new BusinessException("已超过签到时间！");
            }
    }else {
            throw new BusinessException("当前活动不存在");
        }
        return Result.ofSuccess(activityPeopleAudit);
    }
}
