package com.crunii.micro.service.qzh.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqActivityPeopleAudit;
import com.crunii.micro.domain.entity.main.YcqActivitySign;
import com.crunii.micro.domain.entity.main.YcqPublishActivity;
import com.crunii.micro.domain.repo.main.PublishActivityRepo;
import com.crunii.micro.domain.repo.main.YcqActivityPeopleAuditRepo;
import com.crunii.micro.domain.repo.main.YcqActivitySignRepo;
import com.crunii.micro.service.qzh.api.domain.dto.activity.LocationDataDTO;
import com.crunii.micro.service.qzh.api.enums.ActivityButtonStateEnum;
import com.crunii.micro.service.qzh.service.ActivityState;
import com.crunii.micro.service.qzh.service.CheckInCommonLogic;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/** 签退状态
 * @author lmlksy
 * @date 2023/12/2 14:36
 */
@Service
public class CheckOutActivityStateImpl implements ActivityState {
    @Autowired
    YcqActivityPeopleAuditRepo ycqActivityPeopleAuditRepo;
    @Autowired
    YcqActivitySignRepo ycqActivitySignRepo;

    @Autowired
    CheckOutCommonLogicImpl checkOutCommonLogicImpl;

    @Autowired
    PublishActivityRepo publishActivityRepo;

    @Override
    public Result<YcqActivityPeopleAudit> handleActivityState(LocationDataDTO locationDataDTO, Worker worker) {
        YcqActivityPeopleAudit audit = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(Long.valueOf(worker.getLoginId()), locationDataDTO.getActivityId(),0,1);
       //如果单独配了一个签退 再次校验
        YcqActivitySign ycqActivitySign = ycqActivitySignRepo.findByActivityId(locationDataDTO.getActivityId());
       //todo 用户已过签到截至时间 或者为null就不在现在限制 是否签到
        if (ycqActivitySign.getSignInEndTime()!=null&&LocalDateTime.now().isBefore(DateUtils.dateToLocalDateTime(ycqActivitySign.getSignInEndTime()))) {
            if (ycqActivitySign.getSignInWay()!=null){
                if (audit==null||audit.getCheckInResult()==null||audit.getCheckInResult()!=1){
                    throw new BusinessException( "当前人员还未签到，无法签退！");
                }
            }
        }

        if (ycqActivitySign.getSignOutStartTime()==null){
            throw new BusinessException( "请先配置签退时间！");
        }
        if (LocalDateTime.now().isBefore(DateUtils.dateToLocalDateTime(ycqActivitySign.getSignOutStartTime()))) {
            checkOutCommonLogicImpl.checkIn(audit,Boolean.TRUE);
            throw new BusinessException("还未到达签退时间！");
        } else if (LocalDateTime.now().isAfter(DateUtils.dateToLocalDateTime(ycqActivitySign.getSignOutEndTime()))) {
            checkOutCommonLogicImpl.checkIn(audit,Boolean.FALSE);
            throw new BusinessException("已超过签退时间！");
        }
        return Result.ofSuccess(audit);
    }
}
