package com.crunii.micro.service.qzh.service.impl;
import java.math.BigInteger;

import java.io.*;
import java.util.Date;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.BooleanUtil;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.exception.enums.GlobalErrorCodeConstants;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.cavalryTrumpet.*;
import com.crunii.micro.domain.repo.main.cavalryTrumpet.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.StudyCenterService;
import com.crunii.micro.feign.client.SysService;
import com.crunii.micro.feign.client.dto.WorkerC;
import com.crunii.micro.feign.client.dto.WorkerCreateDTO;
import com.crunii.micro.service.qzh.api.OneAffairClient;
import com.crunii.micro.service.qzh.api.domain.dto.StudentCourseCreateReq;
import com.crunii.micro.service.qzh.api.domain.dto.YcqYjsStudentCourse;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.*;
import com.crunii.micro.service.qzh.api.enums.CivilizationBackFlowEnum;
import com.crunii.micro.service.qzh.api.enums.CivilizationReportCodeEnum;
import com.crunii.micro.service.qzh.api.enums.CivilizationReportFlowEnum;
import com.crunii.micro.service.qzh.dto.TestPaperExtentVO;
import com.crunii.micro.service.qzh.jdbc.CivilizationReportDao;
import com.crunii.micro.service.qzh.service.CivilizationReportService;
import com.crunii.micro.service.qzh.util.CharacterLengthUtil;
import com.crunii.module.flow.api.FlowInstanceApi;
import com.crunii.module.flow.api.vo.FlowInstanceReqVO;
import com.crunii.module.flow.api.vo.conf.FlowConfRespVO;
import com.crunii.module.flow.api.vo.pending.CruniiFlowProcessHistoryRespVO;
import com.crunii.module.flow.dto.*;
import com.didiglobal.turbo.engine.result.RollbackTaskResult;
import com.itextpdf.text.*;
import com.itextpdf.text.pdf.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qzh.api.constants.CivilizationConstants.*;

/**
 * @author wangchen create 2023/11/13 16:53
 */
@Service
@Slf4j
public class CivilizationReportServiceImpl implements CivilizationReportService {

    @Resource
    private CivilizationReportDao civilizationReportDao;

    @Resource
    private YcqYnDeclarationRepo ycqYnDeclarationRepo;

    @Resource
    private FlowInstanceApi flowInstanceApi;

    @Resource
    private YcqYnFlowRecordRepo ycqYnFlowRecordRepo;

    @Resource
    private YcqYnInitialEvaluationRepo ycqYnInitialEvaluationRepo;

    @Resource
    private SysService sysService;

    @Resource
    private YcqYnDeclarationExamRepo ycqYnDeclarationExamRepo;

    @Resource
    private YcqQzhQueryOrgRepo ycqQzhQueryOrgRepo;

    @Resource
    private OneAffairClient oneAffairClient;

    private Map<Integer, List<Integer>> backMap;


    @Override
    public String crateCivilizationReport(CivilizationReportDto civilizationReportDto, Worker worker) {
        if (Objects.isNull(worker.getOrgId())) {
            throw new BusinessException("无组织，无法申报！");
        }
        // 验证身份证号是否小于40岁
        validatedIdCard(civilizationReportDto.getIdCard(),false);
        // 字符长度校验
        validatedCharacter(civilizationReportDto,false);
        // 验证 职工总人数 & 35岁及以下青年人数 & 中国国籍人员数量
        validatedPeopleNum(civilizationReportDto.getEmployeesNumber(), civilizationReportDto.getUnder35(), civilizationReportDto.getCountrymenNumber());
        Date now = new Date();
        String declarationCode = getReportCode(civilizationReportDto.getDeclarationType());
        YcqYnDeclaration ycqYnDeclaration;
        if (Objects.nonNull(civilizationReportDto.getDeclarationId())) {
            ycqYnDeclaration = ycqYnDeclarationRepo.getOne(civilizationReportDto.getDeclarationId());
        } else {
            ycqYnDeclaration = new YcqYnDeclaration();
        }
        String flowInstanceId1 = ycqYnDeclaration.getFlowInstanceId();
        // 如果新增则会为空
        Long beforeDeclarationId = ycqYnDeclaration.getDeclarationId();
        ycqYnDeclaration.setDeclarationCode(declarationCode);
        BeanUtils.copyProperties(civilizationReportDto, ycqYnDeclaration);
        ycqYnDeclaration.setFlowInstanceId(flowInstanceId1);
        // 如果没有worker则创建
        Integer loginId = ifNotWorkerCreate(civilizationReportDto);
        ycqYnDeclaration.setAuditState(1);
        ycqYnDeclaration.setLoginId(loginId);
        ycqYnDeclaration.setCreateTime(now);
        ycqYnDeclaration.setCreatorId(worker.getLoginId());
        ycqYnDeclaration.setUpdaterId(worker.getLoginId());
        ycqYnDeclaration.setUpdateTime(now);
        ycqYnDeclaration.setDraft(0);
        YcqYnDeclaration ycqYnDeclaration1 = ycqYnDeclarationRepo.saveAndFlush(ycqYnDeclaration);
        Long declarationId = ycqYnDeclaration1.getDeclarationId();

        // 新建状态 查询当前组织组织名称
        OrgIdNameDto orgIdNameDto = civilizationReportDao.findOrgCountyId(ycqYnDeclaration1.getOrgId());
        String flowInstanceId;
        String nodeInstanceId;
        if (Objects.isNull(beforeDeclarationId)) {
            // 之前未保存草稿，则需启动工作流
            CruniiStartProcessResult flow = createFlow(worker, ycqYnDeclaration1);
            flowInstanceId = flow.getFlowInstanceId();
            nodeInstanceId = flow.getActiveTaskInstance().getNodeInstanceId();
        } else {
            YcqYnFlowRecord flowId = civilizationReportDao.findFlowId(declarationId);
            flowInstanceId = flowId.getFlowInstanceId();
            nodeInstanceId = flowId.getNodeInstanceId();
            saveCanQueryOrg(worker, ycqYnDeclaration1.getOrgId(), ycqYnDeclaration1.getDeclarationId());
        }
        saveCanQueryOrg(worker,ycqYnDeclaration1.getOrgId(),declarationId);
        pushInitialEvaluation(flowInstanceId, nodeInstanceId, orgIdNameDto, worker, declarationId);
        ycqYnDeclarationRepo.saveAndFlush(ycqYnDeclaration1);
        return ycqYnDeclaration1.getDeclarationId() + "";
    }

    public boolean validateWeixin(String weixinCode){
        if(!StringUtils.hasText(weixinCode)){
            return false;
        }
        String judge = "^[a-zA-Z]{1}[-_a-zA-Z0-9]{4,19}+$";
        Pattern pat = Pattern.compile(judge);
        Matcher mat = pat.matcher(weixinCode);
        return mat.matches();
    }


    /**
     * 保存具有查看权限的org
     */
    private void saveCanQueryOrg(Worker worker, Long orgId, Long serviceId) {
        // 查询管理界面可查看的组织id 1 当前用户上所有组织 2申报上所有组织
        List<Long> orgIds = new ArrayList<>();
        long l = Long.parseLong(worker.getOrgId());
        orgIds.add(l);
        orgIds.add(orgId);
        ReportOrgDto reportOrgDto = new ReportOrgDto();
        reportOrgDto.setReportType(1);
        reportOrgDto.setServiceId(serviceId);
        reportOrgDto.setOrgIds(orgIds);
        reportOrgDto.setContainCurrentOrg(true);
        this.saveQueryOrg(reportOrgDto, worker);
    }

    /**
     * 查询如果没有当前用户则创建
     */
    private Integer ifNotWorkerCreate(CivilizationReportDto civilizationReportDto) {
        Integer loginIdByMobile = civilizationReportDao.findLoginIdByMobile(civilizationReportDto.getPhoneNumber());
        if (Objects.nonNull(loginIdByMobile)) {
            return loginIdByMobile;
        }
        // 远程叫用创建worker
        WorkerC workerC = new WorkerC();
        workerC.setMobile(civilizationReportDto.getPhoneNumber());
        workerC.setSex(1);
        workerC.setWorkerName(civilizationReportDto.getDeclarationName());
        workerC.setState(1);
        workerC.setOrgId(civilizationReportDto.getOrgId() + "");
        Result<WorkerCreateDTO> worker = sysService.createWorker(workerC);
        if (!worker.isSuccess()) {
            log.info("创建用户失败，feign接口调用失败！参数-》{}", JSONUtil.toJsonStr(workerC));
            throw new BusinessException("新增失败");
        }
        return worker.getData().getLoginId();
    }

    /**
     * 推进到初评 || 草稿
     */
    private void pushInitialEvaluation(String flowInstanceId, String nodeInstanceId, OrgIdNameDto orgIdNameDto, Worker worker, Long declarationId) {
        // 推进流程到初评
        FlowCommitDTO flowCommitDTO = new FlowCommitDTO();
        FlowCommitorInfoDTO flowCommitorInfoDTO = new FlowCommitorInfoDTO();
        flowCommitDTO.setFlowInstanceId(flowInstanceId);
        flowCommitDTO.setTaskInstanceId(nodeInstanceId);

        // 设置提交人
        flowCommitorInfoDTO.setUserId((long) worker.getLoginId());
        flowCommitorInfoDTO.setUserName(worker.getWorkerName());
        String orgId = worker.getOrgId();
        flowCommitorInfoDTO.setOrgId(Objects.nonNull(worker.getOrgId()) ? Long.valueOf(orgId) : null);
        flowCommitorInfoDTO.setOrgName(worker.getOrgName());
        flowCommitDTO.setCommitor(flowCommitorInfoDTO);

        // 设置下一个处理人组织
        FlowTaskAssigennDTO flowTaskAssigennDTO = new FlowTaskAssigennDTO();
        List<IdNameDTO> idNameDTOS = new ArrayList<>();
        IdNameDTO idNameDTO = new IdNameDTO();
        idNameDTO.setId(orgIdNameDto.getOrgId());
        idNameDTO.setName(orgIdNameDto.getOrgName());
        idNameDTOS.add(idNameDTO);
        flowTaskAssigennDTO.setAssigennDept(idNameDTOS);
        flowCommitDTO.setAddedTaskAssigenn(flowTaskAssigennDTO);

        CommonResult<CruniiCommitTaskResult> resultCommonResult = flowInstanceApi.commitProcess(flowCommitDTO);
        isSuccess(resultCommonResult);
        CruniiCommitTaskResult data = resultCommonResult.getData();
        String nodeInstanceId1 = data.getActiveTaskInstance().getNodeInstanceId();
        // 用于查询流程状态
        YcqYnFlowRecord ycqYnFlowRecord = new YcqYnFlowRecord();
        ycqYnFlowRecord.setServiceId(declarationId);
        ycqYnFlowRecord.setFlowInstanceId(flowInstanceId);
        ycqYnFlowRecord.setNodeInstanceId(nodeInstanceId1);
        ycqYnFlowRecord.setFlowState(1);
        ycqYnFlowRecord.setCreatorId(worker.getLoginId());
        ycqYnFlowRecord.setCreateTime(new Date());
        ycqYnFlowRecordRepo.saveAndFlush(ycqYnFlowRecord);
    }

    /**
     * 启动工作流
     */
    private CruniiStartProcessResult createFlow(Worker worker, YcqYnDeclaration ycqYnDeclaration1) {
        CommonResult<FlowConfRespVO> voCommonResult = flowInstanceApi.getFlowConfInfoByFlowCode("a5408b679fc447d8a53ff3f2c7f2f153");
        isSuccess(voCommonResult);
        FlowConfRespVO data1 = voCommonResult.getData();
        FlowStartDTO flowStartDTO = new FlowStartDTO();
        flowStartDTO.setFlowDeployId(data1.getFlowDeployId());
        flowStartDTO.setFlowModuleId(data1.getFlowModuleId());
        flowStartDTO.setBizId(String.valueOf(ycqYnDeclaration1.getDeclarationId()));
        // 设置启动人
        FlowStarterInfoDTO flowStarterInfoDTO = new FlowStarterInfoDTO();
        flowStarterInfoDTO.setUserId((long) worker.getLoginId());
        flowStarterInfoDTO.setUserName(worker.getWorkerName());
        String orgId = worker.getOrgId();
        flowStarterInfoDTO.setOrgId(Objects.nonNull(worker.getOrgId()) ? Long.valueOf(orgId) : null);
        flowStarterInfoDTO.setOrgName(worker.getOrgName());
        flowStartDTO.setStarter(flowStarterInfoDTO);
        CommonResult<CruniiStartProcessResult> result;
        try {
            result = flowInstanceApi.startProcess(flowStartDTO);
        } catch (Exception e) {
            log.info("远程调用流程执行失败@！参数-》{}", JsonUtil.getJsonString(flowStartDTO));
            throw new BusinessException("执行流程调用失败！");
        }
        isSuccess(result);
        CruniiStartProcessResult data = result.getData();
        String flowInstanceId = data.getFlowInstanceId();
        String nodeInstanceId = data.getActiveTaskInstance().getNodeInstanceId();
        // 保存流程实例id，后续查询代办
        ycqYnDeclaration1.setFlowInstanceId(flowInstanceId);
        ycqYnDeclarationRepo.saveAndFlush(ycqYnDeclaration1);
        // 用于查询流程状态
        YcqYnFlowRecord ycqYnFlowRecord = new YcqYnFlowRecord();
        ycqYnFlowRecord.setServiceId(ycqYnDeclaration1.getDeclarationId());
        ycqYnFlowRecord.setFlowInstanceId(flowInstanceId);
        ycqYnFlowRecord.setNodeInstanceId(nodeInstanceId);
        ycqYnFlowRecord.setFlowState(3);
        ycqYnFlowRecord.setCreatorId(worker.getLoginId());
        ycqYnFlowRecord.setCreateTime(new Date());
        ycqYnFlowRecordRepo.saveAndFlush(ycqYnFlowRecord);
        return data;
    }

    /**
     * 判断是否成功
     */
    private void isSuccess(CommonResult<?> voCommonResult) {
        if (!GlobalErrorCodeConstants.SUCCESS.getCode().equals(voCommonResult.getCode())) {
            throw new BusinessException(voCommonResult.getMsg());
        }
    }


    /**
     * 字符长度校验
     */
    private void validatedCharacter(CivilizationReportDto civilizationReportDto,boolean isDraft) {
        String collectiveName = civilizationReportDto.getCollectiveName();
        if(validatedStrIsDraft(isDraft,collectiveName)){
            if (CharacterLengthUtil.isExceedLength(collectiveName, COLLECTIVE_NAME_MAX_LENGTH)) {
                throw new BusinessException("集体名称不能超过" + COLLECTIVE_NAME_MAX_LENGTH + "字符");
            }
        }
        String declarationName = civilizationReportDto.getDeclarationName();
        if(validatedStrIsDraft(isDraft,declarationName)){
            if (CharacterLengthUtil.isExceedLength(declarationName, DECLARATION_NAME_MAX_LENGTH)) {
                throw new BusinessException("号长姓名不能超过" + DECLARATION_NAME_MAX_LENGTH + "字符");
            }
        }
        String beforeCategory = civilizationReportDto.getBeforeCategory();
        if(validatedStrIsDraft(isDraft,beforeCategory)){
            if (CharacterLengthUtil.isExceedLength(beforeCategory, OTHER_MAX_LENGTH)) {
                throw new BusinessException("曾被认定类别不能超过" + OTHER_MAX_LENGTH + "字符");
            }
        }
        String wechat = civilizationReportDto.getWechat();
        if(validatedStrIsDraft(isDraft,wechat)){
            if (!validateWeixin(wechat)) {
                throw new BusinessException("微信号不符合规范");
            }
        }
    }


    /**
     * 验证是否草稿 str是否有值，true
     */
    private boolean validatedStrIsDraft(boolean isDraft, String str) {
        if(isDraft){
            return StringUtils.hasText(str);
        }
        return true;
    }

    @Override
    @InjectProperty
    public PageRsp<CivilizationReportVo> pageCivilizationReport(PageReq<CivilizationReportPageDto> req, Worker worker) {
        QueryOrgDto queryOrgDto = new QueryOrgDto();
        queryOrgDto.setReportType(1);
        queryOrgDto.setOrgId(worker.getOrgId());
        List<Long> list = this.queryServiceIds(queryOrgDto);
        if (CollectionUtils.isEmpty(list)) {
            return new PageRsp<>();
        }
        PageRsp<CivilizationReportVo> rsp = civilizationReportDao.pageCivilizationReport(req, list);
        List<CivilizationReportVo> result = rsp.getResult();
        if (CollectionUtils.isEmpty(result)) {
            return rsp;
        }
        String orgId = worker.getOrgId();
        handleData(result, orgId);
        return rsp;
    }

    /**
     * 处理返回结果
     */
    private void handleData(List<CivilizationReportVo> result, String orgId) {
        // 查询是否有报备表
        List<Long> declarationIds = result.stream().map(CivilizationReportVo::getDeclarationId).collect(Collectors.toList());
        List<Long> existDeclarationIds = civilizationReportDao.isExistInitEvaluation(declarationIds);
        // 不可操作状态
        List<Integer> notHandleAuditState = Arrays.asList(0,7,8);
        Map<Long, String> collect = new HashMap<>(16);
        // 查询当前用户是否具有权限审核
        if (StringUtils.hasText(orgId)) {
            // 查询具有区县级的declarationIds集合
            List<Long> initialEvaluationIds = civilizationReportDao.findNextDeclarationIds(declarationIds,true);
            // 查询具有市级权限
            List<Long> cityDeclarationIds =  civilizationReportDao.findNextDeclarationIds(declarationIds,false);
            List<KeyValueDto> list = new ArrayList<>();
            if(!CollectionUtils.isEmpty(initialEvaluationIds)){
                list.addAll(civilizationReportDao.findNextOrgId(initialEvaluationIds,true));
            }
            if(!CollectionUtils.isEmpty(cityDeclarationIds)){
                list.addAll(civilizationReportDao.findNextOrgId(cityDeclarationIds,false));
            }
            collect = list.stream().collect(Collectors.toMap(KeyValueDto::getId, KeyValueDto::getValue));
        }
        // 草稿状态
        Integer auditState = 0;
        // 处理占比 职工总人数 & 35岁及以下青年人数 & 中国国籍人员数量 均有要求不为空，无需处理为空情况
        for (CivilizationReportVo civilizationReportVo : result) {
            if (existDeclarationIds.contains(civilizationReportVo.getDeclarationId())) {
                civilizationReportVo.setReportName(civilizationReportVo.getDeclarationCode()+"-"+civilizationReportVo.getCollectiveName());
                civilizationReportVo.setExistReport(true);
            }
            if(!auditState.equals(civilizationReportVo.getAuditState())){
                Integer employeesNumber = civilizationReportVo.getEmployeesNumber();
                BigDecimal bigDecimal = new BigDecimal(employeesNumber);
                Integer under35 = civilizationReportVo.getUnder35();
                Integer countrymenNumber = civilizationReportVo.getCountrymenNumber();
                String s = under35(under35, bigDecimal);
                String country = country(countrymenNumber, bigDecimal);
                civilizationReportVo.setUnder35Proportion(s);
                civilizationReportVo.setCountrymenNumberProportion(country);
            }
            // 身份证脱敏
            String s1 = "";
            if(StringUtils.hasText(civilizationReportVo.getIdCard())){
                s1 = DesensitizedUtil.idCardNum(civilizationReportVo.getIdCard(), 14, 0);
            }
            civilizationReportVo.setIdCard(s1);
            // 查询改条数据是否有审核权限
            if (StringUtils.hasText(orgId)) {
                String s2 = collect.get(civilizationReportVo.getDeclarationId());
                // 当审核状态不在集合中不可操作
                if (orgId.equals(s2) && !notHandleAuditState.contains(civilizationReportVo.getAuditState())) {
                    civilizationReportVo.setHandleState(1);
                }
            }
        }

    }


    @Override
    public List<DocVo> findDoc(Integer declarationType) {
        return civilizationReportDao.findDoc(declarationType);
    }

    @Override
    public CivilizationReportDto findDraft(Long declarationId, Worker worker) {
        if (Objects.isNull(declarationId)) {
            throw new BusinessException("未传id");
        }
        YcqYnDeclaration one = ycqYnDeclarationRepo.findByDeclarationIdAndCreatorIdAndDeleted(declarationId, worker.getLoginId(), 0);
        if (Objects.isNull(one)) {
            throw new BusinessException("数据异常");
        }
        if (!DRAFT.equals(one.getDraft())) {
            throw new BusinessException("该申报不是草稿，无法编辑");
        }
        CivilizationReportDto civilizationReportDto = new CivilizationReportDto();
        BeanUtils.copyProperties(one, civilizationReportDto);
        String orgName = civilizationReportDao.findOrgNameByOrgId(one.getOrgId());
        civilizationReportDto.setOrgName(orgName);
        return civilizationReportDto;
    }

    @Override
    public String delDraft(Long declarationId, Worker worker) {
        YcqYnDeclaration one = ycqYnDeclarationRepo.findByDeclarationIdAndCreatorIdAndDeleted(declarationId, worker.getLoginId(), 0);
        if (Objects.isNull(one)) {
            return "删除成功";
        }
        if (!DRAFT.equals(one.getDraft())) {
            throw new BusinessException("该申报不是草稿，无法删除");
        }
        one.setDeleted(1);
        ycqYnDeclarationRepo.saveAndFlush(one);
        return "删除成功";
    }

    @Override
    public String updateReview(ReviewDto reviewDto, Worker worker) {
        List<Long> declarationIds = reviewDto.getDeclarationIds();
        List<ReviewVo> list = civilizationReportDao.findByIds(declarationIds);
        if (CollectionUtils.isEmpty(list)) {
            throw new BusinessException("申报记录不存在");
        }
        if (list.size() != declarationIds.size()) {
            throw new BusinessException("申报记录不存在");
        }
        // 判断是否通过
        boolean auditPass = AUDIT_PASS.equals(reviewDto.getAuditResult());
        switch (reviewDto.getAuditType()) {
            case 1:
                return initialEvaluation(reviewDto, worker, auditPass);
            case 2:
                return batchAudit(reviewDto, worker, auditPass);
            case 3:
                return archikaryon(reviewDto, worker, auditPass);
            default:
                throw new BusinessException("状态有误");
        }
    }


    @Override
    public String updateBackFlow(BackFlowDto backFlowDto, Worker worker) {

        YcqYnDeclaration ycqYnDeclaration = ycqYnDeclarationRepo.findByDeclarationIdAndDeleted(backFlowDto.getDeclarationId(), 0);
        if (Objects.isNull(ycqYnDeclaration)) {
            throw new BusinessException("无申报流程");
        }
        // 判断是否可以回退到当前流程
        boolean contains = backMap.get(ycqYnDeclaration.getAuditState()).contains(backFlowDto.getAuditState());
        if (!contains) {
            throw new BusinessException("无法回退");
        }
        // 如果是回退到草稿状态
        Integer auditSate = 0;
        if (auditSate.equals(backFlowDto.getAuditState())) {
            ycqYnDeclaration.setDraft(1);
        }
        ycqYnDeclaration.setAuditState(backFlowDto.getAuditState());
        ycqYnDeclarationRepo.saveAndFlush(ycqYnDeclaration);
        rollBackFlow(ycqYnDeclaration, worker, backFlowDto);
        return "更新成功";
    }

    /**
     * 回滚流程
     */
    private void rollBackFlow(YcqYnDeclaration ycqYnDeclaration, Worker worker, BackFlowDto backFlowDto) {
        YcqYnFlowRecord flowId = civilizationReportDao.findFlowId(ycqYnDeclaration.getDeclarationId());
        if (Objects.isNull(flowId)) {
            throw new BusinessException("未启动流程");
        }
        String flowNodeName = CivilizationBackFlowEnum.getFlowNodeName(backFlowDto.getAuditState());
        FlowInstanceReqVO vo = new FlowInstanceReqVO();
        vo.setFlowInstanceId(ycqYnDeclaration.getFlowInstanceId());
        CommonResult<List<CruniiFlowProcessHistoryRespVO>> result = flowInstanceApi.listProcessHistory(vo);
        isSuccess(result);
        // key不为空，去重key，转map
        Map<String, String> collect = result.getData().stream().filter(x -> StringUtils.hasText(x.getNodeName()) && StringUtils.hasText(x.getNodeKey()))
                .collect(Collectors.collectingAndThen(Collectors.toCollection(
                        () -> new TreeSet<>(Comparator.comparing(CruniiFlowProcessHistoryRespVO::getNodeName))
                ), ArrayList::new)).stream().collect(Collectors.toMap(CruniiFlowProcessHistoryRespVO::getNodeName, CruniiFlowProcessHistoryRespVO::getNodeKey));
        String s = collect.get(flowNodeName);
        if (!StringUtils.hasText(s)) {
            throw new BusinessException("回退状态有误");
        }
        FlowRollBackDTO flowRollBackDTO = new FlowRollBackDTO();
        flowRollBackDTO.setTargetNodeKey(s);
        flowRollBackDTO.setFlowInstanceId(ycqYnDeclaration.getFlowInstanceId());
        flowRollBackDTO.setProcessComments("回退");
        flowRollBackDTO.setProcessResult(backFlowDto.getReason());

        FlowCommitorInfoDTO commitorInfoDTO = new FlowCommitorInfoDTO();
        commitorInfoDTO.setUserId((long) worker.getLoginId());
        commitorInfoDTO.setUserName(worker.getWorkerName());
        String orgId = worker.getOrgId();
        commitorInfoDTO.setOrgId(Objects.isNull(orgId) ? null : Long.parseLong(orgId));
        commitorInfoDTO.setOrgName(worker.getOrgName());
        flowRollBackDTO.setCommitor(commitorInfoDTO);

        CommonResult<RollbackTaskResult> rollbackTaskResultCommonResult = flowInstanceApi.rollBackTargetNode(flowRollBackDTO);
        isSuccess(rollbackTaskResultCommonResult);
        RollbackTaskResult data = rollbackTaskResultCommonResult.getData();
        // 保存推进过程
        YcqYnFlowRecord ycqYnFlowRecord = new YcqYnFlowRecord();
        ycqYnFlowRecord.setServiceId(ycqYnDeclaration.getDeclarationId());
        ycqYnFlowRecord.setFlowInstanceId(data.getFlowInstanceId());
        ycqYnFlowRecord.setNodeInstanceId(data.getActiveTaskInstance().getNodeInstanceId());
        ycqYnFlowRecord.setFlowState(0);
        ycqYnFlowRecord.setCreatorId(worker.getLoginId());
        ycqYnFlowRecord.setCreateTime(new Date());
        ycqYnFlowRecordRepo.saveAndFlush(ycqYnFlowRecord);
    }

    @Override
    @InjectProperty
    public PageRsp<CivilizationReportVo> pageMyselfReport(PageReq<Object> req, Worker worker) {
        PageRsp<CivilizationReportVo> rsp = civilizationReportDao.pageMyselfReport(req, worker);
        List<CivilizationReportVo> result = rsp.getResult();
        if (CollectionUtils.isEmpty(result)) {
            return rsp;
        }
        handleData(result, null);
        result.forEach(x->{
            if(x.getCreatorId().equals(worker.getLoginId())){
                x.setIsDeclarationName(1);
            }
        });
        return rsp;
    }

    @Override
    public List<Integer> getBackFlowIds(Long declarationId, Worker worker) {
        YcqYnDeclaration ycqYnDeclaration = ycqYnDeclarationRepo.findByDeclarationIdAndDeleted(declarationId, 0);
        if (Objects.isNull(ycqYnDeclaration)) {
            throw new BusinessException("无此申报");
        }
        return backMap.get(ycqYnDeclaration.getAuditState());
    }

    @Override
    public ResponseEntity<byte[]> findPdf(Long declarationId, HttpServletResponse response) {
        YcqYnDeclaration ycqYnDeclaration = ycqYnDeclarationRepo.findByDeclarationIdAndDeleted(declarationId, 0);
        if (Objects.isNull(ycqYnDeclaration)) {
            throw new BusinessException("申报id不存在");
        }
        YcqYnInitialEvaluation ycqYnInitialEvaluation = ycqYnInitialEvaluationRepo.findByDeclarationId(declarationId);
        if (Objects.isNull(ycqYnInitialEvaluation)) {
            throw new BusinessException("未查询到初评人");
        }

        // 构建HTTP响应头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_PDF);
        headers.setContentDispositionFormData("attachment", "exportedFile.pdf");

        ByteArrayOutputStream outputStream = cratePdf(ycqYnDeclaration, ycqYnInitialEvaluation);

        byte[] byteArray = outputStream.toByteArray();
        return ResponseEntity.ok().headers(headers).body(byteArray);
    }

    @Override
    public String createExamResult(ExamResultDto req) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        List<Long> declarationIds = civilizationReportDao.findDeclarationIdByLoginId(req.getWorkerId());
        if (CollectionUtils.isEmpty(declarationIds)) {
            return "暂无需考试申报";
        }
        List<YcqYnDeclarationExam> list = new ArrayList<>();
        for (Long declarationId : declarationIds) {
            YcqYnDeclarationExam ycqYnDeclarationExam = new YcqYnDeclarationExam();
            ycqYnDeclarationExam.setDeclarationId(declarationId);
            ycqYnDeclarationExam.setPaperId(req.getPaperId());
            ycqYnDeclarationExam.setUserId(req.getWorkerId());
            ycqYnDeclarationExam.setTestScore(req.getTestScore());
            ycqYnDeclarationExam.setPassScore(req.getPassScore());
            ycqYnDeclarationExam.setAnswerScore(req.getPassScore());
            ycqYnDeclarationExam.setIfPass(req.getIfPass());
            list.add(ycqYnDeclarationExam);
        }
        Integer successState = 1;
        ycqYnDeclarationExamRepo.saveAllAndFlush(list);
        if (successState.equals(req.getIfPass())) {
            civilizationReportDao.updateAuditState(declarationIds, 5, worker.getLoginId(), new Date());
            commitSuccessFlow(declarationIds, worker, "考试通过");
        } else {
            rejectFlow(declarationIds, "考试未通过", worker);
        }
        return "推送成功";
    }

    @Override
    public String saveQueryOrg(ReportOrgDto queryOrgDto, Worker worker) {
        Date now = new Date();
        List<String> canQueryOrgIds ;
        if(queryOrgDto.isContainCurrentOrg()){
            canQueryOrgIds =  civilizationReportDao.findCanQueryOrgContainIds(queryOrgDto.getOrgIds());
        }else {
            canQueryOrgIds  = civilizationReportDao.findCanQueryOrgIds(queryOrgDto.getOrgIds());
        }
        Integer reportType = queryOrgDto.getReportType();
        Long serviceId = queryOrgDto.getServiceId();
        String join = String.join(",", canQueryOrgIds);

        YcqQzhQueryOrg ycqYzhQueryOrg = ycqQzhQueryOrgRepo.findByServiceIdAndDeleted(serviceId, 0);
        if (Objects.isNull(ycqYzhQueryOrg)) {
            ycqYzhQueryOrg = new YcqQzhQueryOrg();
            ycqYzhQueryOrg.setCreatorId(worker.getLoginId());
            ycqYzhQueryOrg.setCreateTime(now);
        } else {
            String strOrgId = ycqYzhQueryOrg.getStrOrgId();
            join = join + "," + strOrgId;
            // 去重
            join = Arrays.stream(join.split(",")).distinct().collect(Collectors.joining(","));
        }
        ycqYzhQueryOrg.setReportType(reportType);
        ycqYzhQueryOrg.setServiceId(serviceId);
        ycqYzhQueryOrg.setStrOrgId(join);
        ycqYzhQueryOrg.setUpdaterId(worker.getLoginId());
        ycqYzhQueryOrg.setUpdateTime(now);
        ycqQzhQueryOrgRepo.saveAndFlush(ycqYzhQueryOrg);
        return "保存成功";
    }

    @Override
    public List<Long> queryServiceIds(QueryOrgDto queryOrgDto) {
        return civilizationReportDao.findServiceIds(queryOrgDto);
    }

    @Override
    public Integer findWorkerRole(Worker worker) {
        String orgId = worker.getOrgId();
        if(Objects.isNull(orgId)){
            return 0;
        }
        OrgIdNameDto topOrg = civilizationReportDao.findTopOrg();
        if((topOrg.getOrgId()+"").equals(orgId)){
            return 2;
        }
        return 1;
    }

    @Override
    public int findReportCount(Worker worker) {
        String orgId = worker.getOrgId();
        // 当前用户组织为空则没有待处理
        if(!StringUtils.hasText(orgId)){
            return 0;
        }
        OrgIdNameDto topOrg = civilizationReportDao.findTopOrg();
        // 如是市级直接查全部
        if(topOrg.getOrgId().equals(Long.parseLong(orgId))){
            return civilizationReportDao.findHandleCityCount(orgId);
        }
        return civilizationReportDao.findHandleCountryCount(orgId);
    }

    @Override
    public int findCurrentReportCount(Worker worker) {
        return civilizationReportDao.findCurrentReportCount(worker.getLoginId());
    }


    @Override
    public String crateCivilizationReportDraft(CivilizationReportDto civilizationReportDto, Worker worker) {
        if (Objects.isNull(worker.getOrgId())) {
            throw new BusinessException("无组织，无法申报！");
        }
        // 验证身份证号是否小于40岁
        validatedIdCard(civilizationReportDto.getIdCard(),true);
        // 字符长度校验
        validatedCharacter(civilizationReportDto,true);

        Date now = new Date();
        String declarationCode = getReportCode(civilizationReportDto.getDeclarationType());
        YcqYnDeclaration ycqYnDeclaration;
        if (Objects.nonNull(civilizationReportDto.getDeclarationId())) {
            ycqYnDeclaration = ycqYnDeclarationRepo.getOne(civilizationReportDto.getDeclarationId());
        } else {
            ycqYnDeclaration = new YcqYnDeclaration();
        }
        String flowInstanceId1 = ycqYnDeclaration.getFlowInstanceId();
        // 如果有id，任务之前已经执行过工作流
        Long beforeDeclarationId = ycqYnDeclaration.getDeclarationId();
        ycqYnDeclaration.setDeclarationCode(declarationCode);
        BeanUtils.copyProperties(civilizationReportDto, ycqYnDeclaration);
        ycqYnDeclaration.setFlowInstanceId(flowInstanceId1);
        ycqYnDeclaration.setAuditState(0);
        ycqYnDeclaration.setCreateTime(now);
        ycqYnDeclaration.setCreatorId(worker.getLoginId());
        ycqYnDeclaration.setUpdaterId(worker.getLoginId());
        ycqYnDeclaration.setUpdateTime(now);
        ycqYnDeclaration.setDraft(1);
        YcqYnDeclaration ycqYnDeclaration1 = ycqYnDeclarationRepo.saveAndFlush(ycqYnDeclaration);

        ycqYnDeclaration.setAuditState(0);

        if (Objects.isNull(beforeDeclarationId)) {
            // 启动流程
            createFlow(worker, ycqYnDeclaration1);
        }
        ycqYnDeclarationRepo.saveAndFlush(ycqYnDeclaration1);
        return ycqYnDeclaration1.getDeclarationId() + "";
    }

    @Override
    public TestPaperExtentVO findWorkerExam(Worker worker) {
        int workerExam = civilizationReportDao.findWorkerExam(worker.getLoginId());
        if(workerExam != 0){
            updateResetExam(worker);
        }
        TestPaperExtentVO workerExam1 = createWorkerExam(worker,workerExam);
        workerExam1.setCategoryType(workerExam);
        return workerExam1;
    }

    @Override
    public TestPaperVO findWorkerExamCount(Long paperId, Worker worker) {
        TestPaperVO testPaperVO = civilizationReportDao.findPaperInfo(paperId);
        if(Objects.isNull(testPaperVO)){
            throw new BusinessException("未查询到试卷信息");
        }
        // 查询可考次数
        int workerExamCount = civilizationReportDao.findWorkerExamCount(paperId, worker);
        testPaperVO.setResidueTestCount(workerExamCount);
        testPaperVO.setFinnishCount(testPaperVO.getAllowTimes()-workerExamCount);
        return testPaperVO;
    }

    public TestPaperExtentVO createWorkerExam(Worker worker,Integer workerExam) {
        // 查询是否已经创建考试记录
        TestPaperExtentVO vo  =  civilizationReportDao.findIsCreateExam(worker.getLoginId());
        if(Objects.nonNull(vo)){
            return vo;
        }
        Long paperId = civilizationReportDao.findPaperId(workerExam+"");
        if(Objects.isNull(paperId)){
            throw new BusinessException("暂无考试试卷");
        }
        StudentCourseCreateReq studentCourseCreateReq = new StudentCourseCreateReq();
        studentCourseCreateReq.setStudentId(new BigInteger(String.valueOf(worker.getLoginId())));
        studentCourseCreateReq.setSpecialId(new BigInteger("-1"));
        studentCourseCreateReq.setCourseId(new BigInteger("-1"));
        studentCourseCreateReq.setPaperId(new BigInteger(paperId+""));
        studentCourseCreateReq.setSpecialType("-1");
        studentCourseCreateReq.setCourseDuration(100);
        studentCourseCreateReq.setIfAnswer(0);
        studentCourseCreateReq.setDelFlag(0);
        studentCourseCreateReq.setCreatedTime(new Date());
        studentCourseCreateReq.setCreatedId(new BigInteger(String.valueOf(worker.getLoginId())));
        studentCourseCreateReq.setCreatedName(worker.getWorkerName());
        Result<YcqYjsStudentCourse> studentCourse = oneAffairClient.createStudentCourse(studentCourseCreateReq);
        if(!studentCourse.isSuccess()){
            throw new BusinessException("调用创建接口失败。{}",studentCourse.getMsg());
        }
        YcqYjsStudentCourse data = studentCourse.getData();
        vo = new TestPaperExtentVO();
        vo.setStudentId(data.getStudentId().intValue());
        vo.setStudentCourseId(data.getId().intValue());
        return vo;
    }

    public Long updateResetExam(Worker worker) {
        // 查询当前用户是否有当前试卷 得考试记录
        List<YcqYnDeclarationExam> list = ycqYnDeclarationExamRepo.findByUserIdAndDeleted(worker.getLoginId(), 0);
        Long paperId = null;
        if(!CollectionUtils.isEmpty(list)){
            list.forEach(x->x.setDeleted(1));
            ycqYnDeclarationExamRepo.saveAllAndFlush(list);
            // 远程推送到考试中心重置服务
            for (YcqYnDeclarationExam x : list) {
                ExamResetDto examResetDto = new ExamResetDto();
                examResetDto.setStudentId(BigInteger.valueOf(worker.getLoginId()));
                paperId= x.getPaperId();
                examResetDto.setPaperId(BigInteger.valueOf(x.getPaperId()));
                Result<Object> integerResult = oneAffairClient.resetExamRecords(examResetDto);
                if(!integerResult.isSuccess()){
                    log.info("重置考试失败->{}",integerResult.getMsg());
                }
            }
        }
        return paperId;
    }


    /**
     * 初核
     */
    private String archikaryon(ReviewDto reviewDto, Worker worker, boolean auditPass) {
        Date now = new Date();
        List<Long> declarationIds = reviewDto.getDeclarationIds();
        List<Integer> auditStates = new ArrayList<>();
        auditStates.add(2);
        int count = civilizationReportDao.findCount(declarationIds, auditStates);
        if (declarationIds.size() != count) {
            throw new BusinessException("流程未到，不能初核");
        }
        // 验证是否可初核
        OrgIdNameDto topOrg = civilizationReportDao.findTopOrg();
        Long orgId = topOrg.getOrgId();
        if(!(orgId+"").equals(worker.getOrgId())){
            throw new BusinessException("无法初核");
        }
        String reason = reviewDto.getReason();
        if (auditPass) {
            civilizationReportDao.updateAuditState(declarationIds, 3, worker.getLoginId(), now);
            // 推进工作流
            commitSuccessFlow(declarationIds, worker, reason);
        } else {
            rejectFlow(declarationIds, reason, worker);
        }
        return "初核成功";
    }

    /**
     * 审核 处理
     */
    private String batchAudit(ReviewDto reviewDto, Worker worker, boolean auditPass) {
        Map<Integer,String> chineseName = new HashMap<>(6);
        chineseName.put(3,"待报备结果");
        chineseName.put(4,"待考试");
        chineseName.put(5,"待交叉检查");
        chineseName.put(6,"待命名");
        Date now = new Date();
        List<Long> declarationIds = reviewDto.getDeclarationIds();
        // 校验是否可审核
        List<Integer> auditStates = Arrays.asList(3, 4, 5, 6);
        int count = civilizationReportDao.findCount(declarationIds, auditStates);
        if (declarationIds.size() != count) {
            throw new BusinessException("流程未到，不能审核");
        }
        String reason = reviewDto.getReason();
        // 如果待命名状态 设置评定等级
        if (WAIT_NAME_RESULT.equals(reviewDto.getAuditState())) {
            civilizationReportDao.updateRatingLevel(declarationIds, worker.getLoginId(), now, reviewDto.getRatingLevel());
        }
        if (auditPass) {
            civilizationReportDao.updateAuditState(declarationIds, CivilizationReportFlowEnum.getNextAuditState(reviewDto.getAuditState()), worker.getLoginId(), now);
            // 推进工作流
            commitSuccessFlow(declarationIds, worker, reason);
        } else {
            rejectFlow(declarationIds, reason, worker);
        }
        return chineseName.get(reviewDto.getAuditState())+"审核成功";
    }

    /**
     * 初评 处理
     */
    private String initialEvaluation(ReviewDto reviewDto, Worker worker, boolean auditPass) {
        List<Long> declarationIds = reviewDto.getDeclarationIds();
        // 校验是否可初评
        List<Integer> auditStates = new ArrayList<>();
        auditStates.add(1);
        int count = civilizationReportDao.findCount(declarationIds, auditStates);
        if (declarationIds.size() != count) {
            throw new BusinessException("流程未到，不能初评");
        }

        String reason = reviewDto.getReason();
        if (auditPass) {
            civilizationReportDao.updateAuditState(declarationIds, 2, worker.getLoginId(),  new Date());
            // 初评通过，新增初评人记录表
            saveEvaluation(declarationIds, worker);
            // 推进工作流
            commitSuccessFlow(declarationIds, worker, reason);
        } else {
            rejectFlow(declarationIds, reason, worker);
        }
        return "初评成功";
    }

    private void saveEvaluation(List<Long> declarationIds, Worker worker) {
        List<YcqYnInitialEvaluation> list = new ArrayList<>();
        for (Long declarationId : declarationIds) {
            YcqYnInitialEvaluation byDeclarationId = ycqYnInitialEvaluationRepo.findByDeclarationId(declarationId);
            if (Objects.isNull(byDeclarationId)) {
                byDeclarationId = new YcqYnInitialEvaluation();
            }
            byDeclarationId.setDeclarationId(declarationId);
            byDeclarationId.setWorkerId(worker.getLoginId());
            byDeclarationId.setEvaluationTime(new Date());
            list.add(byDeclarationId);
        }
        ycqYnInitialEvaluationRepo.saveAllAndFlush(list);
    }

    private ByteArrayOutputStream cratePdf(YcqYnDeclaration declaration, YcqYnInitialEvaluation ycqYnInitialEvaluation) {
        PdfWorkerInfoVo workerInfoVo = civilizationReportDao.findWorkerInfo(ycqYnInitialEvaluation.getWorkerId());
        Date evaluationTime = ycqYnInitialEvaluation.getEvaluationTime();
        String auditDate = DateUtil.formatChineseDate(evaluationTime, false, false);
        String betweenYear;
        int year = DateUtil.year(evaluationTime);
        if (year % 2 == 0) {
            betweenYear = year + "-" + year + 1;
        } else {
            betweenYear = year - 1 + "-" + year;
        }

        ByteArrayOutputStream outputStream = null;
        PdfWriter pdfWriter;
        try {
            //创建一个文档(默认大小A4纸)
            outputStream = new ByteArrayOutputStream();
            Document document = new Document(PageSize.A4.rotate());
            pdfWriter = PdfWriter.getInstance(document, outputStream);
            document.open();

            BaseFont bfChinese = BaseFont.createFont("/font/AdobeSongStd-Light.otf", BaseFont.IDENTITY_H, BaseFont.NOT_EMBEDDED);
            Font blackFont = new Font(bfChinese);
            blackFont.setColor(BaseColor.BLACK);
            blackFont.setStyle(Font.BOLD);

            PdfPTable table0 = new PdfPTable(2);
            // 构建每个单元格
            PdfPCell cell1 = new PdfPCell(new Paragraph("附件1", blackFont));
            // 设置背景颜色
            cell1.setBackgroundColor(BaseColor.WHITE);
            // 设置高度
            cell1.setFixedHeight(40);
            // 设置内容水平居中显示
            cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
            // 设置垂直居中
            cell1.setVerticalAlignment(Element.ALIGN_TOP);
            cell1.setBorder(0);
            table0.addCell(cell1);

            Font redFont = new Font(bfChinese);
            redFont.setColor(163, 0, 20);
            // 构建每个单元格
            PdfPCell cell2 = new PdfPCell(new Paragraph("申报编码: " + declaration.getDeclarationCode()+"-"+declaration.getCollectiveName(), redFont));
            // 设置背景颜色
            cell2.setBackgroundColor(BaseColor.WHITE);
            // 设置高度
            cell2.setFixedHeight(40);
            // 设置内容水平居中显示
            cell2.setHorizontalAlignment(Element.ALIGN_RIGHT);
            // 设置垂直居中
            cell2.setVerticalAlignment(Element.ALIGN_TOP);
            cell2.setBorder(0);
            table0.addCell(cell2);
            document.add(table0);
            blackFont.setSize(20f);
            blackFont.setStyle(Font.BOLD);
            Paragraph textgraph1 = new Paragraph(betweenYear + "年度重庆市青年文明号创建（复核）报备表", blackFont);
            textgraph1.setAlignment(Element.ALIGN_CENTER);
            textgraph1.setSpacingBefore(30f);
            document.add(textgraph1);

            Font tableFont = new Font(bfChinese);
            tableFont.setSize(8);
            tableFont.setStyle(Font.NORMAL);
            PdfPTable table = new PdfPTable(10);
            table.setWidthPercentage(100.0f);
            table.setSpacingBefore(30);
            table.setHorizontalAlignment(0);
            buildTitle(tableFont, table);
            document.add(table);

            PdfPTable table2 = new PdfPTable(10);
            table2.completeRow();
            table2.setWidthPercentage(100.0f);
            table2.setHorizontalAlignment(0);

            PdfFormField checkBox = PdfFormField.createCheckBox(pdfWriter);
            checkBox.setFieldName("是");
            pdfWriter.addAnnotation(checkBox);
            String typeName1 = "创建";
            String typeName2 = "复核";
            Integer createType = 1;
            if (createType.equals(declaration.getDeclarationType())) {
                typeName1 = "√ " + typeName1;
            } else {
                typeName2 = "√ " + typeName2;
            }
            PdfPTable nestedTable = new PdfPTable(1);
            PdfPCell tableCell = getTableCell(typeName1, tableFont, 1, 25);
            nestedTable.addCell(tableCell);
            PdfPCell tableCell2 = getTableCell(typeName2, tableFont, 1, 25);
            nestedTable.addCell(tableCell2);
            nestedTable.completeRow();
            PdfPCell pdfPCell = new PdfPCell(nestedTable);
            table2.addCell(pdfPCell);

            Map<Integer, String> ratingLevel = new HashMap<>(6);
            ratingLevel.put(1, "市级");
            ratingLevel.put(2, "市级优秀");
            ratingLevel.put(3, "市级杰出");
            ratingLevel.put(declaration.getCategory(), "√ " + ratingLevel.get(declaration.getCategory()));

            table2.addCell(getTableCell(declaration.getCollectiveName(), tableFont, 2, 50));
            table2.addCell(getTableCell(declaration.getUnit(), tableFont, 1, 50));
            table2.addCell(getTableCell(ratingLevel.get(1) + "\n\n" + ratingLevel.get(2) + "\n\n" + ratingLevel.get(3), tableFont, 1, 50));
            table2.addCell(getTableCell(declaration.getBeforeCategory(), tableFont, 1, 50));
            table2.addCell(getTableCell(declaration.getDeclarationName(), tableFont, 1, 50));
            table2.addCell(getTableCell(declaration.getIdCard(), tableFont, 1, 50));

            PdfPTable nestedTable2 = new PdfPTable(1);
            PdfPCell tableCellLast = getTableCell("手机", tableFont, 1, 25);
            nestedTable2.addCell(tableCellLast);
            PdfPCell tableCell22 = getTableCell("微信号", tableFont, 1, 25);
            nestedTable2.addCell(tableCell22);
            nestedTable2.completeRow();
            PdfPCell pdfPCell2 = new PdfPCell(nestedTable2);
            table2.addCell(pdfPCell2);


            PdfPTable nestedTable3 = new PdfPTable(1);
            PdfPCell tableCellLast3 = getTableCell(declaration.getPhoneNumber(), tableFont, 1, 25);
            nestedTable3.addCell(tableCellLast3);
            PdfPCell tableCell3 = getTableCell(declaration.getWechat(), tableFont, 1, 25);
            nestedTable3.addCell(tableCell3);
            nestedTable3.completeRow();
            PdfPCell pdfPCell3 = new PdfPCell(nestedTable3);
            table2.addCell(pdfPCell3);
            document.add(table2);

            tableFont = new Font(bfChinese);
            blackFont.setColor(BaseColor.BLACK);
            blackFont.setStyle(Font.BOLD);

            Paragraph org = new Paragraph("所属组织: " + workerInfoVo.getOrgName(), tableFont);
            org.setSpacingBefore(40);
            org.setAlignment(Element.ALIGN_RIGHT);
            org.setIndentationRight(80);
            document.add(org);
            Paragraph worker = new Paragraph(workerInfoVo.getWorkerName(), tableFont);
            worker.setAlignment(Element.ALIGN_RIGHT);
            worker.setSpacingBefore(15);
            worker.setIndentationRight(40);
            document.add(worker);
            Paragraph date = new Paragraph(auditDate, tableFont);
            worker.setSpacingBefore(15);
            date.setAlignment(Element.ALIGN_RIGHT);
            date.setIndentationRight(50);
            document.add(date);

            Paragraph bei1 = new Paragraph("注：1：创建（复核）青年文明号集体名称：勾选“创建”或“复核”创建（复核）类别：勾选“市级杰出”或“市级优秀”或“市级”。", tableFont);
            bei1.setAlignment(Element.ALIGN_CENTER);
            bei1.setSpacingBefore(30);
            document.add(bei1);

            Paragraph bei2 = new Paragraph("2：曾被认定类别：填写“XX年度市级杰出青年文明号“XX年度市级优诱青年文明号“XX年度市级普通青年文明号及区县青年文明号等", tableFont);
            bei2.setIndentationLeft(45);
            document.add(bei2);

            Paragraph bei2Ext = new Paragraph("，并提供佐证材料。", tableFont);
            bei2Ext.setIndentationLeft(64);
            document.add(bei2Ext);

            Paragraph bei3 = new Paragraph("3：所属团组织：填写所属各区县（自治县）团委，两江新区，西部科学城重庆高新区、万盛经开区团（工）委，市各直属团组织名称并盖章。", tableFont);
            bei3.setIndentationLeft(45);
            document.add(bei3);
            // 添加水印
            PdfGState gs = new PdfGState();
            gs.setFillOpacity(0.2f);//改透明度
            gs.setStrokeOpacity(0.4f);
            PdfContentByte content = pdfWriter.getDirectContent();
            Rectangle pageSize = document.getPageSize();
            float width = pageSize.getWidth();
            float height = pageSize.getHeight();
            // 旋转角度
            String workerName = workerInfoVo.getWorkerName();
            String mobile = workerInfoVo.getMobile();
            float rotation = -30f;
            content.setFontAndSize(bfChinese, 20);
            content.setColorFill(BaseColor.BLUE);
            content.beginText();
            content.setGState(gs);
            String text = "青春重庆 " + workerName + " " + mobile;
            content.showTextAligned(Element.ALIGN_CENTER,text , width / 2, height / 5, rotation);
            content.showTextAligned(Element.ALIGN_LEFT, text, 100, height - 100, rotation);
            content.showTextAligned(Element.ALIGN_LEFT, text, width - 200, height - 100, rotation);
            content.endText();
            document.close();

        } catch (DocumentException | IOException e) {
            throw new BusinessException("创建pdf异常:" + e.getMessage());
        } finally {
            if (Objects.nonNull(outputStream)) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return outputStream;
    }


    private void buildTitle(Font tableFont, PdfPTable table) {
        PdfPCell pdfPCell1 = getTableCell("创建（复核）青年文明号集体名称", tableFont, 3, 40);
        table.addCell(pdfPCell1);
        PdfPCell pdfPCell2 = getTableCell("所在单位", tableFont, 1, 40);
        table.addCell(pdfPCell2);
        PdfPCell pdfPCell3 = getTableCell("创建（复核）类别", tableFont, 1, 40);

        table.addCell(pdfPCell3);
        PdfPCell pdfPCell4 = getTableCell("曾被认定类别", tableFont, 1, 40);

        table.addCell(pdfPCell4);
        PdfPCell pdfPCell5 = getTableCell("号长姓名", tableFont, 1, 40);

        table.addCell(pdfPCell5);
        PdfPCell pdfPCell6 = getTableCell("身份证号", tableFont, 1, 40);

        table.addCell(pdfPCell6);
        PdfPCell pdfPCell7 = getTableCell("联系方式", tableFont, 2, 40);
        table.addCell(pdfPCell7);
    }

    private PdfPCell getTableCell(String tableName, Font tableFont, int colspan, int height) {
        PdfPCell pdfPCell = new PdfPCell(new Phrase(tableName, tableFont));
        pdfPCell.setHorizontalAlignment(Element.ALIGN_CENTER);
        pdfPCell.setVerticalAlignment(Element.ALIGN_MIDDLE);
        pdfPCell.setUseBorderPadding(true);
        pdfPCell.setColspan(colspan);
        pdfPCell.setFixedHeight(height);
        return pdfPCell;
    }

    /**
     * 推进成功工作流
     */
    private void commitSuccessFlow(List<Long> declarationIds, Worker worker, String reason) {
        for (Long declarationId : declarationIds) {
            YcqYnFlowRecord one = civilizationReportDao.findFlowId(declarationId);
            if (Objects.isNull(one)) {
                throw new BusinessException("流程未启动");
            }
            // 推进流程
            FlowCommitDTO flowCommitDTO = new FlowCommitDTO();
            FlowCommitorInfoDTO flowCommitorInfoDTO = new FlowCommitorInfoDTO();
            flowCommitorInfoDTO.setUserId((long) worker.getLoginId());
            flowCommitorInfoDTO.setUserName(worker.getWorkerName());
            String orgId = worker.getOrgId();
            flowCommitorInfoDTO.setOrgId(Objects.nonNull(orgId) ? Long.valueOf(orgId) : null);
            flowCommitorInfoDTO.setOrgName(worker.getOrgName());
            flowCommitDTO.setFlowInstanceId(one.getFlowInstanceId());
            flowCommitDTO.setTaskInstanceId(one.getNodeInstanceId());
            flowCommitDTO.setCommitor(flowCommitorInfoDTO);
            flowCommitDTO.setProcessComments("通过");
            flowCommitDTO.setProcessResult(reason);
            Map<String, Object> flowVar = new HashMap<>(2);
            flowVar.put("auditState", 1);
            flowCommitDTO.setFlowVar(flowVar);

            // 设置下一个处理人组织
            FlowTaskAssigennDTO flowTaskAssigennDTO = new FlowTaskAssigennDTO();
            List<IdNameDTO> idNameDTOS = new ArrayList<>();
            IdNameDTO idNameDTO = new IdNameDTO();
            idNameDTO.setId(CommonConstants.SUPER_LOGIN_IDS.get(0).longValue());
            idNameDTOS.add(idNameDTO);
            flowTaskAssigennDTO.setAssigennUser(idNameDTOS);
            flowCommitDTO.setAddedTaskAssigenn(flowTaskAssigennDTO);

            CommonResult<CruniiCommitTaskResult> result = flowInstanceApi.commitProcess(flowCommitDTO);
            isSuccess(result);
            CruniiCommitTaskResult data = result.getData();
            // 保存推进过程
            YcqYnFlowRecord ycqYnFlowRecord = new YcqYnFlowRecord();
            ycqYnFlowRecord.setServiceId(declarationId);
            ycqYnFlowRecord.setFlowInstanceId(data.getFlowInstanceId());
            ycqYnFlowRecord.setNodeInstanceId(data.getActiveTaskInstance().getNodeInstanceId());
            ycqYnFlowRecord.setFlowState(1);
            ycqYnFlowRecord.setCreatorId(worker.getLoginId());
            ycqYnFlowRecord.setCreateTime(new Date());
            ycqYnFlowRecordRepo.saveAndFlush(ycqYnFlowRecord);
        }
    }

    /**
     * 驳回流程
     */
    private void rejectFlow(List<Long> declarationIds, String reason, Worker worker) {
        Date now = new Date();
        civilizationReportDao.rejectFlow(declarationIds, worker.getLoginId(), now);

        for (Long declarationId : declarationIds) {
            YcqYnFlowRecord one = civilizationReportDao.findFlowId(declarationId);
            if (Objects.isNull(one)) {
                throw new BusinessException("流程未启动");
            }
            // 推进流程
            FlowCommitDTO flowCommitDTO = new FlowCommitDTO();
            FlowCommitorInfoDTO flowCommitorInfoDTO = new FlowCommitorInfoDTO();
            flowCommitorInfoDTO.setUserId((long) worker.getLoginId());
            flowCommitorInfoDTO.setUserName(worker.getWorkerName());
            String orgId = worker.getOrgId();
            flowCommitorInfoDTO.setOrgId(Objects.nonNull(orgId) ? Long.valueOf(orgId) : null);
            flowCommitorInfoDTO.setOrgName(worker.getOrgName());
            flowCommitDTO.setFlowInstanceId(one.getFlowInstanceId());
            flowCommitDTO.setTaskInstanceId(one.getNodeInstanceId());

            // 超管特殊处理
            FlowTaskAssigennDTO flowTaskAssigennDTO = new FlowTaskAssigennDTO();
            List<IdNameDTO> list = new ArrayList<>();
            IdNameDTO idNameDTO = new IdNameDTO();
            idNameDTO.setId(9999L);
            list.add(idNameDTO);
            flowCommitDTO.setAddedTaskAssigenn(flowTaskAssigennDTO);

            flowCommitDTO.setCommitor(flowCommitorInfoDTO);
            flowCommitDTO.setProcessComments("驳回");
            flowCommitDTO.setProcessResult(reason);
            Map<String, Object> flowVar = new HashMap<>(2);
            flowVar.put("auditState", 2);
            flowCommitDTO.setFlowVar(flowVar);

            CommonResult<CruniiCommitTaskResult> result = flowInstanceApi.commitProcess(flowCommitDTO);
            isSuccess(result);
            CruniiCommitTaskResult data = result.getData();
            // 保存推进过程
            YcqYnFlowRecord ycqYnFlowRecord = new YcqYnFlowRecord();
            ycqYnFlowRecord.setServiceId(declarationId);
            ycqYnFlowRecord.setFlowInstanceId(data.getFlowInstanceId());
            ycqYnFlowRecord.setNodeInstanceId(data.getActiveTaskInstance().getNodeInstanceId());
            ycqYnFlowRecord.setFlowState(1);
            ycqYnFlowRecord.setCreatorId(worker.getLoginId());
            ycqYnFlowRecord.setCreateTime(new Date());
            ycqYnFlowRecordRepo.saveAndFlush(ycqYnFlowRecord);
        }
    }

    /**
     * 验证 职工总人数 & 35岁及以下青年人数 & 中国国籍人员数量
     */
    private void validatedPeopleNum(int employeesNumber, int under35, int countrymenNumber) {
        // 职工总人数
        if (MIN_EMPLOY_COUNT > employeesNumber || MAX_EMPLOY_COUNT < employeesNumber) {
            throw new BusinessException("录入人数需大于等于6人，小于等于200人");
        }
        BigDecimal employees = new BigDecimal(employeesNumber);
        // 35岁及以下青年人数
        under35(under35, employees);
        // 中国国籍人员数量
        country(countrymenNumber, employees);
    }

    private static String country(int countrymenNumber, BigDecimal employees) {
        BigDecimal compareCountryNum = new BigDecimal(COUNTRY_PROPORTION);
        BigDecimal countrymen = new BigDecimal(countrymenNumber);
        BigDecimal divide1 = countrymen.divide(employees, 2, RoundingMode.HALF_UP);
        if (divide1.compareTo(compareCountryNum) < 0) {
            throw new BusinessException("中国国籍人员占职工总人数80%以上");
        }
        return divide1.floatValue() * 100 + "%";
    }

    private static String under35(int under35, BigDecimal employees) {
        BigDecimal compareNum = new BigDecimal(AGE_35_PROPORTION);
        BigDecimal under = new BigDecimal(under35);
        BigDecimal divide = under.divide(employees, 2, RoundingMode.HALF_UP);
        if (divide.compareTo(compareNum) < 0) {
            throw new BusinessException("35岁及以下青年人数占职工总人数50%以上");
        }
        return divide.floatValue() * 100 + "%";
    }

    @Override
    public void afterPropertiesSet() {
        // 缓存可回退节点
        backMap = new HashMap<>(16);
        backMap.put(1, Collections.singletonList(0));
        backMap.put(2, Arrays.asList(0, 1));
    }


    /**
     * 校验年龄不能大于40岁
     */
    private void validatedIdCard(String idCard,boolean isDraft) {
        if(isDraft){
            if(!StringUtils.hasText(idCard)){
                return;
            }
            // 校验身份证是否填写正确
            if(!IdcardUtil.isValidCard(idCard)){
                throw new BusinessException("身份证需正确填写！");
            }
        }
        int ageByIdCard = IdcardUtil.getAgeByIdCard(idCard, new Date());
        if (ageByIdCard > MAX_AGE) {
            throw new BusinessException("身份证年龄需小于等于40岁");
        }
    }

    private static String getReportCode(Integer declarationType) {
        String codePrefix = CivilizationReportCodeEnum.getCodePrefix(declarationType);
        if (!StringUtils.hasText(codePrefix)) {
            throw new BusinessException("申报类型有误，请重新输入");
        }
        long date = DateUtil.current() / 1000;
        String s = RandomUtil.randomStringUpper(4);
        return codePrefix + date + s;
    }
}
