package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQzhCommando;
import com.crunii.micro.domain.entity.main.YcqQzhCommandoOrg;
import com.crunii.micro.domain.repo.main.YcqQzhCommandoOrgRepo;
import com.crunii.micro.domain.repo.main.YcqQzhCommandoRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.dto.CommandoActiveDetailsIssueRespDTO;
import com.crunii.micro.service.qzh.dto.CommandoActiveDetailsLoginRespDTO;
import com.crunii.micro.service.qzh.dto.IntegrationDetailsCommonReqDTO;
import com.crunii.micro.service.qzh.jdbc.CommandoActiveDetailsDao;
import com.crunii.micro.service.qzh.service.CommandoActiveDetailsService;
import com.crunii.micro.service.qzh.service.CommandoCountInfoService;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

/**
 * @Author thw
 * @date 2023/9/9
 * @description
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class CommandoActiveDetailsServiceImpl implements CommandoActiveDetailsService {

    private final CommandoActiveDetailsDao commandoActiveDetailsDao;

    private final CommandoCountInfoServiceImpl commandoCountInfoService;

    private final YcqQzhCommandoRepo ycqQzhCommandoRepo;

    private final YcqQzhCommandoOrgRepo ycqQzhCommandoOrgRepo;

    @Override
    public Result<PageRsp<CommandoActiveDetailsIssueRespDTO>> issueScorePage(Worker worker,
                                                                             PageReq<IntegrationDetailsCommonReqDTO> req) {

         // 参数校验
        checkParameter(req.getQuery());
        // 权限
//        checkPermissionContent(worker,req.getQuery().getId(), req.getQuery().getIds());

        PageRsp<CommandoActiveDetailsIssueRespDTO> result = commandoActiveDetailsDao
            .issueScorePage(worker, req);
        if (Objects.nonNull(result.getTotalNum()) && result.getTotalNum() > 0){
            for (CommandoActiveDetailsIssueRespDTO vo : result.getResult()) {
                // 发布次数
                vo.setIssueNumber((int) result.getTotalNum());
            }
        }
        return Result.ofSuccess(result);
    }


    @Override
    public Result<PageRsp<CommandoActiveDetailsLoginRespDTO>> loginScorePage(Worker worker,
                                                                             PageReq<IntegrationDetailsCommonReqDTO> req) {

        // 参数校验
        checkParameter(req.getQuery());
        // 权限
//        checkPermissionContent(worker,req.getQuery().getId(), req.getQuery().getIds());
        PageRsp<CommandoActiveDetailsLoginRespDTO> result = commandoActiveDetailsDao
            .loginScorePage(worker, req);

        if (Objects.nonNull(result.getTotalNum()) && result.getTotalNum() > 0){
            for (CommandoActiveDetailsLoginRespDTO respDTO : result.getResult()) {
                // 登录次数
                respDTO.setLoginNumber((int) result.getTotalNum());
            }
        }
        return Result.ofSuccess(result);
    }


    /** 校验某个突击队的查看权限 */
    public void checkPermissionContent(Worker worker,String id,List<Long> queryOrgIds) {

        List<Long> permissionOrgIds = commandoCountInfoService.checkWorkerAuth(worker,queryOrgIds);
        // 通过必传的突击队id判断是否有权限
        Optional<YcqQzhCommando> commandoOptional = ycqQzhCommandoRepo
            .findById(Long.valueOf(id));
        if (!commandoOptional.isPresent()){
            throw new BusinessException("根据id:"+ id + "未查询到突击队信息!");
        }
        YcqQzhCommando ycqQzhCommando = commandoOptional.get();
        List<YcqQzhCommandoOrg> commandoOrgs = ycqQzhCommandoOrgRepo.findByCommandoCode(ycqQzhCommando.getCommandoCode());
        Set<Long> existOrgIds = commandoOrgs.stream().map(e -> e.getOrgId()).collect(Collectors.toSet());
        // 取交集
        if (CollectionUtils.isNotEmpty(permissionOrgIds)){
            permissionOrgIds.retainAll(existOrgIds);
            if (permissionOrgIds.size() == 0){
                throw new BusinessException("没有突击队的查询权限!");
            }
        }
    }

    /** 参数校验 */
    public static void checkParameter(IntegrationDetailsCommonReqDTO dto) {
        if (Objects.isNull(dto)){
            throw new BusinessException("参数对象必传！");
        }
        if (ObjectUtil.isNotEmpty(dto) && ObjectUtil.isEmpty(dto.getId())) {
            throw new BusinessException("必传参数为空！");
        }
    }
}
