package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO;
import com.crunii.micro.service.qzh.dto.*;
import com.crunii.micro.service.qzh.jdbc.CommandoCountInfoDao;
import com.crunii.micro.service.qzh.service.CommandoCountInfoService;
import com.crunii.micro.service.qzh.service.YcqQzhCommandoService;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

/**
 * @Author thw
 * @date 2023/9/7
 * @description 3.3.1青年突击队信息统计
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class CommandoCountInfoServiceImpl implements CommandoCountInfoService {

    private final TsmOrgRepo tsmOrgRepo;

    private final CommandoCountInfoDao commandoCountInfoDao;

    private final YcqQzhCommandoService ycqQzhCommandoApiService;

   /* @Override
    public Result<List<CommandoOrganizationRespDTO>> findCoincidenceOrganization(Worker worker) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        return Result.ofSuccess(commandoCountInfoDao.findCoincidenceOrganization(currWorker));
    }*/


    @Override
    public Result<PageRsp<CommandoActiveIntegralRespDTO>> activeIntegralPage(Worker worker,
                                                                             PageReq<CommandoCountInfoCommonReqDTO> req) {
        // 获取权限List
        List<String> commandoCodeList = getPermissionList(worker, req.getQuery());

        /*  // 权限
        List<Long> permissionOrgIds = checkWorkerAuth(worker,req.getQuery().getIds());
        List<String> commandoList = null;
        if (CollectionUtils.isNotEmpty(permissionOrgIds)){
//            String permissionOrgIdStr = Joiner.on(",").join(permissionOrgIds);
            String permissionOrgIdStr = permissionOrgIds.stream().map(e -> "\'" + e + "\'")
                .collect(Collectors.joining(","));
            // 通过目前权限组织查询突击队赋权组织关系表
            commandoList = commandoCountInfoDao.findIdByOrgIdIn(permissionOrgIdStr);
        }*/
        PageRsp<CommandoActiveIntegralRespDTO> result = commandoCountInfoDao.activeIntegralPage(worker, req, commandoCodeList);
        return Result.ofSuccess(result);
    }


    @Override
    public Result<CommandoGroupInfoRespDTO> countGroupInfo(Worker worker,
                                                           CommandoCountInfoCommonReqDTO dto) {

        CommandoGroupInfoRespDTO respDTO = new CommandoGroupInfoRespDTO();

        // 权限判断
/*        List<Long> userOrgIds = checkWorkerAuth(worker,dto.getIds());
//        if (ObjectUtil.isNotEmpty(userOrgIds)){
            Iterator<CommandoInfoDTO> iterator = list.iterator();
            while (iterator.hasNext()) {
                CommandoInfoDTO infoDTO = iterator.next();
                String[] orgIds = infoDTO.getOrgidList().split(",");
                List<Long> orgIdList = Arrays.stream(orgIds).map(e -> Long.valueOf(e)).collect(Collectors.toList());
                // 权限校验
                boolean label = checkOrgId(userOrgIds, orgIdList);
                if (!label){
                    // 不满足则移除
                    iterator.remove();
                }
            }
            List<String> commandoList = null;
            if (CollectionUtils.isNotEmpty(userOrgIds)){
//            String permissionOrgIdStr = Joiner.on(",").join(permissionOrgIds);
                String permissionOrgIdStr = userOrgIds.stream().map(e -> "\'" + e + "\'")
                    .collect(Collectors.joining(","));
                // 通过目前权限组织查询突击队赋权组织关系表
                commandoList = commandoCountInfoDao.findIdByOrgIdIn(permissionOrgIdStr);
            }*/
        // 获取权限List
        List<String> commandoCodeList = getPermissionList(worker, dto);
            //统计突击队总数/队员信息/风采发布数
            List<CommandoInfoDTO> list = commandoCountInfoDao.countGroupInfo(worker, dto,commandoCodeList);
            if (CollectionUtils.isNotEmpty(list)){
                // 青年突击队总数
                int count = 0;
                int sumPersonNum = 0;
                int fengcaiNum = 0;
                StringBuilder cardStr = new StringBuilder("");
                String leaderNameAndCard = "";
                // 组装
                for (CommandoInfoDTO infoDTO : list) {
                    count++;
                    sumPersonNum += infoDTO.getPersonNum();
                    fengcaiNum += infoDTO.getFengcaiGoupNumber();
                    cardStr.append(infoDTO.getLeaderNameAndCard()).append(",");
                }
                leaderNameAndCard = cardStr.substring(0, cardStr.length() - 1);
                // 设置
                respDTO.setCommandoTotalNumber(count);
                respDTO.setCommandoMemberNumber(sumPersonNum);
                respDTO.setFengcaiReleaseTotalNumber(fengcaiNum);
                respDTO.setTeamLeaderCardList(leaderNameAndCard);

                //统计队长40岁以下
                Integer ageNumber = getAgeNumber(respDTO.getTeamLeaderCardList(), 40);
                respDTO.setTeamLeaderAgeNumber(ageNumber);
                //将idCard信息设置为null,以免泄露信息
                respDTO.setTeamLeaderCardList(null);
                //统计队伍状态冻结数
                List<String> commandoCodes = list.stream().map(e -> e.getCommandoCode())
                    .collect(Collectors.toList());
                String commandoInSql = "";
                // 冻结状态的权限处理
                if (ObjectUtil.isNotEmpty(commandoCodes)){
                    commandoInSql = commandoCodes.stream().map(e -> "\'" + e + "\'").collect(Collectors.joining(",", "(", ")"));
                }
                Integer commandoState = commandoCountInfoDao.getCommandoState(worker,dto,commandoInSql);
                respDTO.setRanksFreezeNumber(commandoState);
//                //当为null时，为每个字符设置初始值
//                checkValueAndSet(respDTO);
            }

//        }
        //当为null时，为每个字符设置初始值
        checkValueAndSet(respDTO);
        return Result.ofSuccess(respDTO);
    }

    @Override
    public Result<List<CommandoTypeRatioRespDTO>> findTypeAndNumber(Worker worker,
                                                                    CommandoCountInfoCommonReqDTO dto) {
        List<CommandoTypeRatioRespDTO> respDTOList = Collections.EMPTY_LIST;
       /* // 权限判断
//        List<Long> userOrgIds = checkWorkerAuth(worker,dto.getIds());
//        if (ObjectUtil.isNotEmpty(userOrgIds)){
            /*Iterator<CommandoTypeRatioRespDTO> iterator = respDTOList.iterator();
            while (iterator.hasNext()) {
                CommandoTypeRatioRespDTO infoDTO = iterator.next();
                String[] orgIds = infoDTO.getOrgidList().split(",");
                List<Long> orgIdList = Arrays.stream(orgIds).map(e -> Long.valueOf(e)).collect(Collectors.toList());
                // 权限校验
                boolean label = checkOrgId(userOrgIds, orgIdList);
                if (!label){
                    // 不满足则移除
                    iterator.remove();
                }
                infoDTO.setOrgidList(null);
            }
            List<String> commandoList = null;
            if (CollectionUtils.isNotEmpty(userOrgIds)){
//            String permissionOrgIdStr = Joiner.on(",").join(permissionOrgIds);
                String permissionOrgIdStr = userOrgIds.stream().map(e -> "\'" + e + "\'")
                    .collect(Collectors.joining(","));
                // 通过目前权限组织查询突击队赋权组织关系表
                commandoList = commandoCountInfoDao.findIdByOrgIdIn(permissionOrgIdStr);
            }*/
        // 获取权限List
        List<String> commandoCodeList = getPermissionList(worker, dto);
        respDTOList = commandoCountInfoDao.findTypeAndNumber(worker,dto,commandoCodeList);
        List<CommandoTypeRatioRespDTO> list = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(respDTOList)){
            // 突击队总数
            long count = respDTOList.stream().count();
            // 根据类型分组
            Map<String, List<CommandoTypeRatioRespDTO>> groupContent = respDTOList.stream()
                .collect(Collectors.groupingBy(CommandoTypeRatioRespDTO::getTypeTitle));
            groupContent.forEach((k, v) ->{
                CommandoTypeRatioRespDTO voDemo = new CommandoTypeRatioRespDTO();
                voDemo.setTypeCount(count);
                voDemo.setTypeTitle(k);
                voDemo.setTypeNumber((int) v.stream().count());
                voDemo.setTypeTeamMemberNumber(v.stream().mapToInt(CommandoTypeRatioRespDTO::getTypeTeamMemberNumber).sum());
                list.add(voDemo);
            });
            respDTOList = list;
        }

        return Result.ofSuccess(respDTOList);
    }

    /**
     * 当为null时，为每个字符设置初始值
     */
    private void checkValueAndSet(CommandoGroupInfoRespDTO respDTO) {
        if (Objects.nonNull(respDTO)) {
            // 青年突击队总数
            if (ObjectUtils.isEmpty(respDTO.getCommandoTotalNumber())) {
                respDTO.setCommandoTotalNumber(0);
            }
            // 青年突击队员总数
            if (ObjectUtils.isEmpty(respDTO.getCommandoMemberNumber())) {
                respDTO.setCommandoMemberNumber(0);
            }
            // 队伍状态冻结数
            if (ObjectUtils.isEmpty(respDTO.getRanksFreezeNumber())) {
                respDTO.setRanksFreezeNumber(0);
            }
            // 队长40岁以下
            if (ObjectUtils.isEmpty(respDTO.getTeamLeaderAgeNumber())) {
                respDTO.setTeamLeaderAgeNumber(0);
            }
            // 风采发布总数
            if (ObjectUtils.isEmpty(respDTO.getFengcaiReleaseTotalNumber())) {
                respDTO.setFengcaiReleaseTotalNumber(0);
            }
        }
    }

    /**
     * 获取队长年龄小于指定年龄的数量
     */
    public static int getAgeNumber(String idCard, int ageNum) {
        int ageCount = 0;
        if (StringUtils.isNotBlank(idCard)) {
            String[] idCardArr = idCard.split(",");
            checkIdCardLength(idCardArr);
            for (int i = 0; i < idCardArr.length; i++) {
                String[] infoArr = idCardArr[i].split("-");
                int age = getAge(infoArr[1]);
                if (age < ageNum) {
                    ageCount++;
                }
            }
        }
        return ageCount;
    }

    /**
     * 通过身份证号得到具体的年龄
     */
    public static int getAge(String idCard) {
        int year = DateUtil.getYear();
        int month = DateUtil.getMonth();
        int day = DateUtil.getDay();
        int idYear = Integer.valueOf(idCard.substring(6, 10));
        int idMonth = Integer.valueOf(idCard.substring(10, 12));
        int idDay = Integer.valueOf(idCard.substring(12, 14));
        if (month > idMonth) {
            return year - idYear;
        } else if ((month == idMonth) && (day > idDay)) {
            return year - idYear;
        } else {
            return year - idYear - 1;
        }
    }

    /**
     * 检验身份证号的长度是否为18位
     */
    private static void checkIdCardLength(String[] idCardArr) {
        // 身份证号的正则表达式匹配规则
        String reg = "^[1-9]\\d{5}(18|19|([23]\\d))\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$";
        Pattern pattern = Pattern.compile(reg);

        StringBuilder message = new StringBuilder();
        if (ObjectUtil.isNotEmpty(idCardArr)) {
            message.append("队长名为:");
            for (int i = 0; i < idCardArr.length; i++) {
                String[] split = idCardArr[i].split("-");
                Matcher matcher = pattern.matcher(split[1]);
                boolean label = matcher.matches();
                if (split[1].length() != 18 && label) {
                    if (i != 0) {
                        message.append(",");
                    }
                    message.append(split[0]);
                }
            }
            message.append("的身份证位数不对。");
            System.out.println(message.length());
            if (message.toString().length() > 14) {
                throw new BusinessException(message.toString());
            }
        }
    }

    /***
     * @Description:  判断当前用户是否有该条数据的权限
     * @Author: thw
     * @Date: 2023/10/6 9:36
     * @Params:
     * @param userPermissionList 用户权限
     * @param dataOrgIdList  数据存储的权限
     * @return: boolean
     **/
    public boolean checkOrgId(List<Long> userPermissionList , List<Long> dataOrgIdList){
        if (CollectionUtils.isNotEmpty(userPermissionList) && CollectionUtils.isNotEmpty(dataOrgIdList)){
            for (Long orgId : userPermissionList) {
                if (dataOrgIdList.contains(orgId)){
                    return true;
                }
            }
        }
        return false;
    }

    /** 通过 组织类型 和 组织级别，查询相关的组织管理员 */
    public List<TsmOrgContentDTO> findOrgManagePeople(String orgType , String orgLevel){
        return commandoCountInfoDao.findOrgManagePeople(orgType,orgLevel);
    }


    /** 查询用户所有的数据权限 */
    public List<Long> getWorkerAllAuth(Worker worker){
        return commandoCountInfoDao.findAllOrgAuth(worker.getLoginId());
    }


    /** 返回经过处理后的最终用户数据权限  OrgTreeReqDto 查看组织相关信息的DTO */
    public List<Long> checkWorkerAuth(Worker worker,List<Long> queryOrgIds){
        /*// 查询团市委组织
        List<TsmOrgContentDTO> cityLevels = findOrgManagePeople("1", "Citylevel");
//        Set<Long> cityLevelOrgId = cityLevels.stream().filter(e -> Objects.nonNull(e.getOrgId())).map(v -> v.getOrgId()).collect(Collectors.toSet());
        // 团市委组织管理员(团市委管理员：查看所有组织下的突击队统计信息)
        Set<Integer> cityManagePeoples = cityLevels.stream().filter(v -> Objects.nonNull(v.getManagePeople())).map(v -> v.getManagePeople())
            .collect(Collectors.toSet());
        // 区县团委管组织
        List<TsmOrgContentDTO> Districts = findOrgManagePeople("1", "Districtlevel");
        Set<Long> districtsOrgId = Districts.stream().filter(e -> Objects.nonNull(e.getOrgId())).map(e -> e.getOrgId()).collect(Collectors.toSet());
        // 区县团委管理员：查看本区县范围内的突击队统计信息
        Set<TsmOrgContentDTO> districtPeoples = Districts.stream()
            .filter(e -> Objects.nonNull(e.getManagePeople())).collect(Collectors.toSet());

        // 用户数据权限
        List<Long> workerAllAuth = getWorkerAllAuth(worker);

        // 不是团市委管理员则根据数据权限查看
        if (!cityManagePeoples.contains(worker.getLoginId())){
            // 不是区县团委管理员则根据数据权限查看
            if (!districtPeoples.contains(worker.getLoginId())){
                return workerAllAuth;
            }else {
                if (CollectionUtils.isNotEmpty(workerAllAuth) && CollectionUtils.isNotEmpty(districtsOrgId)) {
                    workerAllAuth.addAll(districtsOrgId);
                }
                return workerAllAuth;
            }
        }else {
            // 团市委管理员：查看所有组织下的突击队统计信息,所以不加组织限制条件
            return Collections.EMPTY_LIST;
        }*/
        List<Long> workerAllAuth = getWorkerAllAuth(worker);
        TsmOrg tsmOrg = tsmOrgRepo.findOrgById(worker.getOrgId());
        // 查看权限中是否包含本身的组织
        if (ObjectUtil.isNotEmpty(tsmOrg) && !workerAllAuth.contains(tsmOrg.getOrgId())){
            workerAllAuth.add(tsmOrg.getOrgId());
        }

//        if (ObjectUtil.isNotEmpty(workerAllAuth) && !workerAllAuth.contains(queryOrgIds)){
//            workerAllAuth.addAll(queryOrgIds);
//        }

        // 取用户权限组织与查询组织的交集
        if (ObjectUtil.isNotEmpty(workerAllAuth) && ObjectUtil.isNotEmpty(queryOrgIds)){
            workerAllAuth.retainAll(queryOrgIds);
        }

        return workerAllAuth;
    }

    /** 获取权限List */
    public List<String> getPermissionList(Worker worker,
        CommandoCountInfoCommonReqDTO dtoDemo) {
        // 获取突击队的
        ArrayList<QueryCommandoRespDTO> list = new ArrayList<>();
        QueryCommandoReqDTO dto = new QueryCommandoReqDTO();
        List<QueryCommandoRespDTO> queryCommandoRespDTOS = ycqQzhCommandoApiService
            .queryCommandoByExamineList(dto);

        List<QueryCommandoRespDTO> dtoList = commandoCountInfoDao.findCommonedContent(worker);
        if (ObjectUtil.isNotEmpty(queryCommandoRespDTOS)){
            list.addAll(queryCommandoRespDTOS);
        }
        if (ObjectUtil.isNotEmpty(dtoList)){
            list.addAll(dtoList);
        }

        // 突击队所显示的数据
        List<String> commandoCodeList = list.stream()
            .filter(e -> Integer.valueOf(3).equals(e.getState())).distinct()
            .map(e -> e.getCommandoCode()).collect(Collectors.toList());

        // 通过组织查询获取能显示数据
        if (ObjectUtil.isNotEmpty(dtoDemo.getIds())){
            String permissionOrgIdStr = dtoDemo.getIds().stream().map(e -> "\'" + e + "\'")
                .collect(Collectors.joining(","));
            List<String> commandoList = commandoCountInfoDao.findIdByOrgIdIn(permissionOrgIdStr);
            // 取交集
            commandoCodeList.retainAll(commandoList);
        }
        return commandoCodeList;
    }

}
