package com.crunii.micro.service.qzh.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.repo.main.PublishActivityRepo;
import com.crunii.micro.service.qzh.api.domain.dto.activity.DefaultAppTemplateDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.DefaultAppTemplateImageDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activityPublish.FeedbackLimitDTO;
import com.crunii.micro.service.qzh.api.enums.ActivityButtonStateEnum;
import com.crunii.micro.service.qzh.dto.YcqActivityPeopleAuditDTO;
import com.crunii.micro.service.qzh.jdbc.activitypublish.PublishActivityDao;
import com.crunii.micro.service.qzh.service.DefaultAppTemplateService;
import com.crunii.micro.service.qzh.service.RegistrationAuditService;
import com.crunii.micro.service.qzh.service.YcqEventReportService;
import com.sun.org.apache.bcel.internal.generic.I2F;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * @author lmlksy
 * @date 2023/11/25 16:21
 */
@Service
public class DefaultAppTemplateServiceImpl implements DefaultAppTemplateService {
    @Autowired
    PublishActivityRepo publishActivityRepo;

    @Autowired
    YcqActivityPeopleAuditRepo ycqActivityPeopleAuditRepo;

    @Autowired
    YcqActivityParticipationDetailsRepo ycqActivityParticipationDetailsRepo;

    @Autowired
    YcqActivityImageRepo ycqActivityImageRepo;

    @Autowired
    TsmWorkerRepo tsmWorkerRepo;

    @Autowired
    RegistrationAuditService registrationAuditService;

    @Autowired
    YcqEventReportService ycqEventReportService;

    @Override
    public Result<String> saveActivityContent(DefaultAppTemplateDTO dto, Worker worker) {
       if (dto.getId()==null){
           return Result.ofFail("活动id不能为空！");
       }
        YcqPublishActivity byIdAndIsDeleted = publishActivityRepo.findByIdAndIsDeleted(dto.getId(), 0);
        if (byIdAndIsDeleted.getBusinessBelong().equals("youthPromise")){
            Boolean aBoolean = ycqEventReportService.promiseYouth();
            if (!aBoolean){
                throw new BusinessException("当前活动需先前往青春之约进行注册！");
            }
        }
        YcqActivityParticipationDetails userIdAndActivityId = ycqActivityParticipationDetailsRepo.findByUserIdAndActivityId(Long.valueOf(worker.getLoginId()), dto.getId());
        if (userIdAndActivityId!=null){
            return Result.ofFail("该用户已经提交");
        }
        YcqPublishActivity publishActivity = publishActivityRepo.findByIdAndIsDeleted(dto.getId(), 0);
        if (publishActivity!=null&&publishActivity.getIsLimit()==1){
            List<YcqActivityParticipationDetails> byActivityId = ycqActivityParticipationDetailsRepo.findByActivityId(dto.getId());
            if (!CollectionUtils.isEmpty(byActivityId)){
                if (publishActivity.getLimitCount()>byActivityId.size()){
                    return Result.ofFail("该活动超过提交次数");
                }
            }

        }

        //活动反馈表）默认模板
        YcqActivityParticipationDetails ycqActivityParticipationDetails = new YcqActivityParticipationDetails();
//        ycqActivityParticipationDetails.setTitle(dto.getTitle());
        ycqActivityParticipationDetails.setIsAnonymity(dto.getIsAnonymity());
        ycqActivityParticipationDetails.setActivityRating(dto.getActivityRating());
        ycqActivityParticipationDetails.setActivityId(dto.getId());
        ycqActivityParticipationDetails.setContent(dto.getActivityContent());
        ycqActivityParticipationDetails.setUserId(Long.valueOf(worker.getLoginId()));
        //审核状态，在额外查询一次是否需要审核
//        YcqPublishActivity byIdAndIsDeleted = publishActivityRepo.findByIdAndIsDeleted(dto.getId(), 0);
        ycqActivityParticipationDetails.setIsPublicity(0);
//        if (byIdAndIsDeleted!=null){
//            if (byIdAndIsDeleted.getContentNeededAudit()==0){
//                ycqActivityParticipationDetails.setAuditState(1);
//                //修改公示为通过 不需要审核
//                ycqActivityParticipationDetails.setIsPublicity(0);
//            }else {
//                ycqActivityParticipationDetails.setIsPublicity(0);
//                ycqActivityParticipationDetails.setAuditState(2);
//            }
//
//        }
        ycqActivityParticipationDetails.setPublisher(worker.getWorkerName());
        ycqActivityParticipationDetails.setPublishTime(LocalDateTime.now());
        ycqActivityParticipationDetails.setPhoneNumber(worker.getMobile());
        ycqActivityParticipationDetails.setBrowseCount(0);
        TsmWorker tsmWorker = tsmWorkerRepo.findByLoginIdAndState(worker.getLoginId(), 1);
        ycqActivityParticipationDetails.setOrganizationId(Long.valueOf(tsmWorker.getOrgId()));
        YcqActivityParticipationDetails save = ycqActivityParticipationDetailsRepo.save(ycqActivityParticipationDetails);
        if (!CollectionUtils.isEmpty(dto.getImageUrls())) {
            List<YcqActivityImage> activityImageList = new ArrayList<>();
            for (DefaultAppTemplateImageDTO imageUrl : dto.getImageUrls()) {
                //图片表
                YcqActivityImage ycqActivityImage = new YcqActivityImage();
                ycqActivityImage.setActivityId(dto.getId());
                ycqActivityImage.setUserId(Long.valueOf(worker.getLoginId()));
                ycqActivityImage.setImageUrl(imageUrl.getImageUrl());
                ycqActivityImage.setYcqActivityParticipationDetailsId(save.getId());
                activityImageList.add(ycqActivityImage);
            }
            ycqActivityImageRepo.saveAll(activityImageList);

        }
        YcqActivityPeopleAuditDTO ycqActivityPeopleAuditDTO = new YcqActivityPeopleAuditDTO(dto.getId(), ActivityButtonStateEnum.HIDE.getCode());
        //新增用户
        registrationAuditService.saveYcqActivityPeopleAudit2(ycqActivityPeopleAuditDTO);
        return Result.ofSuccess();
    }

    @Override
    public Result<Integer> doQueryView(DefaultAppTemplateDTO dto) {

        if (dto.getId()!=null){
            //活动表
            YcqPublishActivity ycqPublishActivity = publishActivityRepo.findByIdAndIsDeleted(dto.getId(), 0);
            if (ycqPublishActivity==null){
                throw new BusinessException("当前活动不存在！");
            }
            if (ycqPublishActivity.getBrowseCount()==null||ycqPublishActivity.getBrowseCount()==0){
                ycqPublishActivity.setBrowseCount(1);
            }else {
                ycqPublishActivity.setBrowseCount(ycqPublishActivity.getBrowseCount()+1);
            }
            YcqPublishActivity save = publishActivityRepo.saveAndFlush(ycqPublishActivity);
            return Result.ofSuccess(save.getBrowseCount());
        }

        if (dto.getYcqActivityParticipationDetailsId()!=null){
            YcqActivityParticipationDetails ycqActivityParticipationDetails = ycqActivityParticipationDetailsRepo.findById(dto.getYcqActivityParticipationDetailsId()).orElse(null);
            if (ycqActivityParticipationDetails==null){
                throw new BusinessException("当前反馈不存在！");
            }
            if (ycqActivityParticipationDetails.getBrowseCount()==null||ycqActivityParticipationDetails.getBrowseCount()==0){
                ycqActivityParticipationDetails.setBrowseCount(1);
            }else {
                ycqActivityParticipationDetails.setBrowseCount(ycqActivityParticipationDetails.getBrowseCount()+1);
            }
            YcqActivityParticipationDetails details = ycqActivityParticipationDetailsRepo.saveAndFlush(ycqActivityParticipationDetails);
            return Result.ofSuccess(details.getBrowseCount());
        }
        return Result.ofSuccess();
    }

}
