package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.*;
import com.crunii.micro.service.qzh.api.domain.dto.activity.DefaultTemplateDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.DefaultTemplateParticipationDTO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormColumnVO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormRecordVO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormVO;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.QagPageRsp;
import com.crunii.micro.service.qzh.api.enums.ActivityButtonStateEnum;
import com.crunii.micro.service.qzh.dto.DefaultTemplateVO;
import com.crunii.micro.service.qzh.dto.dynamicform.DynamicFormColumnDTO;
import com.crunii.micro.service.qzh.jdbc.DefaultTemplateJdbcDao;
import com.crunii.micro.service.qzh.service.DefaultTemplateService;
import com.crunii.micro.service.qzh.service.YcqDynamicFormService;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qzh.api.constants.QzhConstants.WORKER_NOT_EXISTS;

/**
 * @author lmlksy
 * @date 2023/11/22 15:55
 */

@Service
public class DefaultTemplateServiceImpl implements DefaultTemplateService {

    @Autowired
    private DefaultTemplateJdbcDao defaultTemplateJdbcDao;

    @Autowired
    private TsmWorkerRepo tsmWorkerRepo;

    @Autowired
    private YcqActivityParticipationDetailsRepo ycqActivityParticipationDetailsRepo;

    @Autowired
    private YcqActivityPeopleAuditRepo ycqActivityPeopleAuditRepo;

    @Autowired
    private YcqActivityImageRepo ycqActivityImageRepo;

    @Autowired
    private PublishActivityRepo publishActivityRepo;

    @Autowired
    private TsmOrgRepo tsmOrgRepo;

    @Autowired
    private YcqDynamicFormService ycqDynamicFormService;

    @Autowired
    private YcqDynamicFormRepo ycqDynamicFormRepo;

    @Autowired
    private TsmPropertyValueRepo tsmPropertyValueRepo;

    @Override
    public Result<QagPageRsp<DefaultTemplateVO>> queryList(PageReq<DefaultTemplateDTO> req) {
        if (req.getQuery()==null&&req.getQuery().getActivityId()==null){
            return Result.ofFail("活动id不能为空！");
        }
        QagPageRsp<DefaultTemplateVO> defaultTemplateVOPageRsp = defaultTemplateJdbcDao.queryList(req);
//        if (defaultTemplateVOPageRsp!=null){
//            if (!CollectionUtils.isEmpty(defaultTemplateVOPageRsp.getResult())){
//                for (DefaultTemplateVO defaultTemplateVO : defaultTemplateVOPageRsp.getResult()) {
//                    TsmOrg tsmOrg = tsmOrgRepo.findByOrgIdAndState(defaultTemplateVO.getOrganizationId(), 1);
//                    if (tsmOrg!=null){
//                        defaultTemplateVO.setOrganizationName(tsmOrg.getOrgName());
//                    }
//                }
//            }
//        }
        return Result.ofSuccess(defaultTemplateVOPageRsp);
    }

    @Override
    public Result<String> doAudit(DefaultTemplateDTO req) {
        if (req.getAuditState()==null){
            return Result.ofFail("审核状态不能为空！");
        }
        YcqActivityParticipationDetails ycqActivityParticipationDetails = ycqActivityParticipationDetailsRepo.findById(req.getId()).orElse(null);
//        ycqActivityParticipationDetails.setAuditState(req.getAuditState());
//        ycqActivityParticipationDetails.setAuditTime(LocalDateTime.now());
        if (req.getCause()!=null){
            ycqActivityParticipationDetails.setCause(req.getCause());
        }
        ycqActivityParticipationDetailsRepo.save(ycqActivityParticipationDetails);
        return new Result("审核成功!",Boolean.TRUE);
    }

    @Override
    public Result<String> doPublicity(DefaultTemplateDTO req) {
        if (req.getIsPublicity()==null){
            return Result.ofFail("公示状态不能为空！");
        }
        YcqActivityParticipationDetails ycqActivityParticipationDetails = ycqActivityParticipationDetailsRepo.findById(req.getId()).orElse(null);
            if (req.getIsPublicity()==1){
                //设置活动的是否公示
//                ycqActivityParticipationDetails.setAuditState(3);
                ycqActivityParticipationDetails.setIsPublicity(req.getIsPublicity());
                ycqActivityParticipationDetails.setPublicityTime(LocalDateTime.now());
                ycqActivityParticipationDetailsRepo.saveAndFlush(ycqActivityParticipationDetails);
                return new Result<>("公示成功!",Boolean.TRUE);
            }else if (req.getIsPublicity()==0){
                ycqActivityParticipationDetails.setIsPublicity(req.getIsPublicity());
                //设置流程状态为不展示
                ycqActivityPeopleAuditRepo.updateProcessStateByActivityIdAndUserId(ActivityButtonStateEnum.HIDE.getCode(), ycqActivityParticipationDetails.getActivityId(), ycqActivityParticipationDetails.getUserId());
//                ycqActivityParticipationDetails.setAuditState(1);
                ycqActivityParticipationDetailsRepo.saveAndFlush(ycqActivityParticipationDetails);
                return new Result<>("取消公示成功!",Boolean.TRUE);
            }
        return Result.ofSuccess();
    }

    /**
     *   提供单个用户的信息和反馈模板信息
     * @param activityId 活动id
     * @param userId 用户id
     * @return
     */
    @Override
    public Result<DefaultTemplateParticipationVO> queryViewDetails(Long activityId,Long userId) {
        DefaultTemplateParticipationVO defaultTemplateParticipationVO = new DefaultTemplateParticipationVO();
        ParticipantDetailVO participantDetailVO = new ParticipantDetailVO();
        YcqPublishActivity publishActivity = publishActivityRepo.findByIdAndIsDeleted(activityId, 0);
        if (publishActivity==null){
            return Result.ofFail("当前活动不存在！");
        }

        YcqActivityPeopleAudit activityPeopleAudit = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(userId, activityId,0,1);
        if (activityPeopleAudit==null){
            return Result.ofFail("当前用户不存在！");
        }

        if (activityPeopleAudit.getRegistrationTime()!=null){
            //报名时间
            participantDetailVO.setRegistrationTime(DateUtils.formatLocalDateTimeWithoutSecond(activityPeopleAudit.getRegistrationTime()));
        }
        if (activityPeopleAudit.getRegistrationAuditTime()!=null){
            //报名审核时间
            if (publishActivity.getNeededAudit()==1){
                participantDetailVO.setRegistrationAuditTime(DateUtils.formatLocalDateTimeWithoutSecond(activityPeopleAudit.getRegistrationAuditTime()));
            }
        }
        if (activityPeopleAudit.getCheckInTime()!=null){
            //签到
            participantDetailVO.setCheckInTime((DateUtils.formatLocalDateTimeWithoutSecond(activityPeopleAudit.getCheckInTime())));
        }
        if (activityPeopleAudit.getCheckOutTime()!=null){
            participantDetailVO.setCheckOutTime((DateUtils.formatLocalDateTimeWithoutSecond(activityPeopleAudit.getCheckOutTime())));
        }
        //不需要报名
        if (publishActivity.getIsApply()!=null&&publishActivity.getIsApply()==0){
            if (activityPeopleAudit.getCheckOutTime()!=null){
                participantDetailVO.setParticipationTime((DateUtils.formatLocalDateTimeWithoutSecond(activityPeopleAudit.getCheckOutTime())));
            }
            if (activityPeopleAudit.getCheckInTime()!=null){
                //签到
                participantDetailVO.setParticipationTime((DateUtils.formatLocalDateTimeWithoutSecond(activityPeopleAudit.getCheckInTime())));
            }
        }else {
            if (activityPeopleAudit.getRegistrationTime()!=null){
                //报名时间
                participantDetailVO.setParticipationTime(DateUtils.formatLocalDateTimeWithoutSecond(activityPeopleAudit.getRegistrationTime()));
            }
        }

        //反馈
        YcqActivityParticipationDetails participationDetails = ycqActivityParticipationDetailsRepo.findByUserIdAndActivityId(userId, activityId);
        if (participationDetails!=null) {

            //不需要签到签退
            if (publishActivity.getNeededSign()!=null&&publishActivity.getNeededSign()==0){
                if (participationDetails.getPublishTime()!=null) {
                    participantDetailVO.setParticipationTime(DateUtils.formatLocalDateTimeWithoutSecond(participationDetails.getPublishTime()));
                }
            }
            //过程信息与反馈详情
            ParticipationDetailsVO participationDetailsVO = new ParticipationDetailsVO();
            if (participationDetails.getPublishTime()!=null){
                participationDetailsVO.setPublishTime(DateUtils.formatLocalDateTimeWithoutSecond(participationDetails.getPublishTime()));
            }
//            if (participationDetails.getAuditTime()!=null){
//                participationDetailsVO.setAuditTime(DateUtils.formatLocalDateTimeWithoutSecond(participationDetails.getAuditTime()));
//            }
            participationDetailsVO.setYcqActivityParticipationDetails(participationDetails.getId());
            BeanUtils.copyProperties(participationDetails, participationDetailsVO);

            List<YcqActivityImageVO> attachments = new ArrayList<>();
            List<YcqActivityImage> images = ycqActivityImageRepo.findByUserIdAndActivityId(userId, activityId);
            for (YcqActivityImage image : images) {
                YcqActivityImageVO ycqActivityImageVO = new YcqActivityImageVO();
                ycqActivityImageVO.setImageUrl(image.getImageUrl());
                attachments.add(ycqActivityImageVO);
            }
            participationDetailsVO.setAttachments(attachments);

            defaultTemplateParticipationVO.setParticipationDetailsVO(participationDetailsVO);
        }

        BeanUtils.copyProperties(activityPeopleAudit,participantDetailVO);
        if (publishActivity.getIsApply()!=null&&publishActivity.getIsApply()==0){
            participantDetailVO.setRegistrationTime(null);
            participantDetailVO.setRegistrationStatus(null);
            participantDetailVO.setRegistrationAuditTime(null);
        }

        defaultTemplateParticipationVO.setParticipantDetailVO(participantDetailVO);

        return Result.ofSuccess(defaultTemplateParticipationVO);
    }

    /**
     *  查询一个活动下所有的 已报名人员 已经所有的已公示反馈模板
     * @param activityId 活动id
     * @return
     */
    @Override
    public Result<AppActivityDetailsVO> queryViewDetails(Long activityId) {
        AppActivityDetailsVO appActivityDetailsVO = new AppActivityDetailsVO();
        List<YcqActivityPeopleAudit> peopleAuditList = ycqActivityPeopleAuditRepo.findByActivityIdAndIsDeleted(activityId,0);
        if (CollectionUtils.isEmpty(peopleAuditList)){
            return Result.ofSuccess(appActivityDetailsVO);
        }
        //去除掉待审核人员数据
        List<YcqActivityPeopleAudit> filteredList = peopleAuditList.stream()
                .filter(audit -> !audit.getRegistrationStatus().equals(2))
                .collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(filteredList)){
            //已报名人数去除掉未通过的
            appActivityDetailsVO.setRegisteredCount(Long.valueOf(filteredList.stream()
                    .filter(audit -> !audit.getRegistrationStatus().equals(0))
                    .collect(Collectors.toList()).size()));
            //报名情况集合
            List<AppActivityStatusVO> appActivityStatusVOList =new ArrayList<>();
            for (YcqActivityPeopleAudit activityPeopleAudit : filteredList) {
                AppActivityStatusVO appActivityStatusVO = new AppActivityStatusVO();
                //脱敏处理
                String userName = activityPeopleAudit.getUserName();
                if (userName != null && userName.length() > 1) {
                    StringBuilder maskedNameBuilder = new StringBuilder();
                    maskedNameBuilder.append(userName.charAt(0));
                    for (int i = 1; i < userName.length() - 1; i++) {
                        maskedNameBuilder.append("*");
                    }
                    maskedNameBuilder.append(userName.charAt(userName.length() - 1));
                    appActivityStatusVO.setUserName(maskedNameBuilder.toString());
                }
                appActivityStatusVO.setRegistrationStatus(activityPeopleAudit.getRegistrationStatus());
                appActivityStatusVOList.add(appActivityStatusVO);
            }
            appActivityDetailsVO.setAppActivityStatusVOList(appActivityStatusVOList);
        }

        //活动反馈内容
        List<YcqActivityParticipationDetails> participationDetails = ycqActivityParticipationDetailsRepo.findByActivityIdAndIsPublicity(activityId, 1);

        if (!CollectionUtils.isEmpty(participationDetails)){
            //参与详情反馈模板集合
            List<ParticipationDetailsVO> participationDetailsVOList =new ArrayList<>();
            for (YcqActivityParticipationDetails participationDetail : participationDetails) {
                //ParticipationDetailsVO
                ParticipationDetailsVO participationDetailsVO = new ParticipationDetailsVO();
                participationDetailsVO.setPublishTime(DateUtils.formatLocalDateTime(participationDetail.getPublishTime()));
                participationDetailsVO.setYcqActivityParticipationDetails(participationDetail.getId());
                BeanUtils.copyProperties(participationDetail,participationDetailsVO);
                TsmWorker tsmWorker = tsmWorkerRepo.findByLoginIdAndState(participationDetail.getUserId().intValue(), 1);
                if (tsmWorker!=null){
                    //脱敏处理
                    String userName = tsmWorker.getWorkerName();
                    if (userName != null && userName.length() > 1) {
                        StringBuilder maskedNameBuilder = new StringBuilder();
                        maskedNameBuilder.append(userName.charAt(0));
                        for (int i = 1; i < userName.length() - 1; i++) {
                            maskedNameBuilder.append("*");
                        }
                        maskedNameBuilder.append(userName.charAt(userName.length() - 1));
                        participationDetailsVO.setWorkerName(maskedNameBuilder.toString());
                    }
                    participationDetailsVO.setHeadPortraitPicKey(tsmWorker.getHeadPortraitPicKey());
                }
                //图片
                List<YcqActivityImageVO> attachments =new ArrayList<>();
                List<YcqActivityImage> images = ycqActivityImageRepo.findByUserIdAndActivityId(participationDetail.getUserId(), activityId);
                if (!CollectionUtils.isEmpty(images)){
                    for (YcqActivityImage image : images) {
                        YcqActivityImageVO imageVO = new YcqActivityImageVO();
                        imageVO.setImageUrl(image.getImageUrl());
                        attachments.add(imageVO);
                    }
                    participationDetailsVO.setAttachments(attachments);
                }
                participationDetailsVOList.add(participationDetailsVO);
            }
            appActivityDetailsVO.setParticipationDetailsVOList(participationDetailsVOList);
        }

        return Result.ofSuccess(appActivityDetailsVO);
    }

    @Override
    public Result<AppActivityEvaluateVO> queryAppViewDetails(AppActivityEvaluateDTO dto) {
        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(dto.getActivityId());
        if (!publishActivity.isPresent()) {
            throw new BusinessException("所选活动不存在");
        }
        YcqPublishActivity ycqPublishActivity = publishActivity.get();
        AppActivityEvaluateVO appActivityEvaluateVO = new AppActivityEvaluateVO();
        AppActivityPlayerInfoVO appActivityPlayerInfoVO = new AppActivityPlayerInfoVO();
        YcqActivityPeopleAudit peopleAudit = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(dto.getUserId(), dto.getActivityId(), 0,1);
        YcqActivityPeopleAudit peopleAudit2 = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(dto.getUserId(), dto.getActivityId(), 0,2);
        YcqActivityPeopleAudit peopleAudit0 = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(dto.getUserId(), dto.getActivityId(), 0,0);
        YcqActivityPeopleAudit peopleAudit00 = ycqActivityPeopleAuditRepo.findTopByUserIdAndActivityIdAndIsDeletedAndRegistrationStatusOrderByRegistrationTimeDesc(dto.getUserId(), dto.getActivityId(), 0,0);

        if (!ObjectUtil.isEmpty(peopleAudit2)){
            if (peopleAudit2.getRegistrationTime()!=null){
                appActivityPlayerInfoVO.setRegistrationTime(DateUtils.formatLocalDateTime(peopleAudit2.getRegistrationTime()));
            }

            if (peopleAudit2.getRegistrationAuditTime()!=null){
                appActivityPlayerInfoVO.setRegistrationAuditTime(DateUtils.formatLocalDateTime(peopleAudit2.getRegistrationAuditTime()));
            }
            if (peopleAudit2.getCheckInTime()!=null){
                appActivityPlayerInfoVO.setCheckInTime(DateUtils.formatLocalDateTime(peopleAudit2.getCheckInTime()));
            }
            if (peopleAudit2.getCheckOutTime()!=null){
                appActivityPlayerInfoVO.setCheckOutTime(DateUtils.formatLocalDateTime(peopleAudit2.getCheckOutTime()));
            }
            BeanUtils.copyProperties(peopleAudit2,appActivityPlayerInfoVO);
            if (ycqPublishActivity.getNeededAudit()==0){
                appActivityPlayerInfoVO.setRegistrationResult(null);
                appActivityPlayerInfoVO.setRegistrationAuditTime(null);
                appActivityPlayerInfoVO.setCause(null);
                appActivityPlayerInfoVO.setNoCause(null);
            }
            appActivityPlayerInfoVO.setRegistrationResult("成功");
            BeanUtils.copyProperties(peopleAudit2,appActivityPlayerInfoVO);
        }

        if (!ObjectUtil.isEmpty(peopleAudit)){
            if (peopleAudit.getRegistrationTime()!=null){
                appActivityPlayerInfoVO.setRegistrationTime(DateUtils.formatLocalDateTime(peopleAudit.getRegistrationTime()));
            }

            if (peopleAudit.getRegistrationAuditTime()!=null){
                appActivityPlayerInfoVO.setRegistrationAuditTime(DateUtils.formatLocalDateTime(peopleAudit.getRegistrationAuditTime()));
            }
            if (peopleAudit.getCheckInTime()!=null){
                appActivityPlayerInfoVO.setCheckInTime(DateUtils.formatLocalDateTime(peopleAudit.getCheckInTime()));
            }
            if (peopleAudit.getCheckOutTime()!=null){
                appActivityPlayerInfoVO.setCheckOutTime(DateUtils.formatLocalDateTime(peopleAudit.getCheckOutTime()));
            }

            if (ycqPublishActivity.getNeededAudit()==0){
                appActivityPlayerInfoVO.setRegistrationResult(null);
                appActivityPlayerInfoVO.setRegistrationAuditTime(null);
                appActivityPlayerInfoVO.setCause(null);
                appActivityPlayerInfoVO.setNoCause(null);
            }
            appActivityPlayerInfoVO.setRegistrationResult("成功");
            BeanUtils.copyProperties(peopleAudit,appActivityPlayerInfoVO);
        }

        if (peopleAudit00!=null&&peopleAudit==null&&peopleAudit2==null){
            if (peopleAudit00.getNoCause()!=null){
                List<String> valueCodeList = Arrays.asList(peopleAudit00.getNoCause().split(","));
                List<TsmPropertyValue> byValueCodeInAndState = tsmPropertyValueRepo.findByValueCodeInAndState(valueCodeList, 1);
                if (!CollectionUtils.isEmpty(byValueCodeInAndState)){
                    String valueNameString = byValueCodeInAndState.stream()
                            .map(TsmPropertyValue::getValueName)
                            .collect(Collectors.joining(","));
                    appActivityPlayerInfoVO.setNoCause(valueNameString);
                }
            }
            BeanUtils.copyProperties(peopleAudit00,appActivityPlayerInfoVO);
            appActivityPlayerInfoVO.setRegistrationResult("成功");
            if (ycqPublishActivity.getNeededAudit()==0){
                appActivityPlayerInfoVO.setRegistrationResult(null);
                appActivityPlayerInfoVO.setRegistrationAuditTime(null);
                appActivityPlayerInfoVO.setCause(null);
                appActivityPlayerInfoVO.setNoCause(null);
            }else {
                if (peopleAudit00.getRegistrationTime()!=null){
                    appActivityPlayerInfoVO.setRegistrationTime(DateUtils.formatLocalDateTime(peopleAudit00.getRegistrationTime()));
                }

                if (peopleAudit00.getRegistrationAuditTime()!=null){
                    appActivityPlayerInfoVO.setRegistrationAuditTime(DateUtils.formatLocalDateTime(peopleAudit00.getRegistrationAuditTime()));
                }
                if (peopleAudit00.getCheckInTime()!=null){
                    appActivityPlayerInfoVO.setCheckInTime(DateUtils.formatLocalDateTime(peopleAudit00.getCheckInTime()));
                }
                if (peopleAudit00.getCheckOutTime()!=null){
                    appActivityPlayerInfoVO.setCheckOutTime(DateUtils.formatLocalDateTime(peopleAudit00.getCheckOutTime()));
                }
            }

        }

        boolean needApply = Objects.equals(ycqPublishActivity.getIsApply(), 1);
        appActivityPlayerInfoVO.setNeedApply(0);
        if (ycqPublishActivity.getNeededSign()!=null){
            appActivityPlayerInfoVO.setNeededSign(ycqPublishActivity.getNeededSign());
        }

        if (ycqPublishActivity.getIsApply()!=null){
            appActivityPlayerInfoVO.setIsApply(ycqPublishActivity.getIsApply());
        }
        if(!ObjectUtil.isEmpty(peopleAudit)&&peopleAudit.getBusinessBelongId()!=null){
            YcqDynamicForm byBusinessIdAndType = ycqDynamicFormRepo.getByBusinessIdAndType(dto.getActivityId(), 1);
            if (!ObjectUtil.isEmpty(byBusinessIdAndType)){
                 appActivityPlayerInfoVO.setApplyJson(byBusinessIdAndType.getFormJson());
            }
            String detailsByActiveId = ycqDynamicFormService.getDetailsValueByActiveId(peopleAudit.getBusinessBelongId());
            if (needApply&&null != detailsByActiveId) {
                appActivityPlayerInfoVO.setNeedApply(1);
                DynamicFormRecordVO dynamicFormRecordVO = new DynamicFormRecordVO();
                dynamicFormRecordVO.setRecord(detailsByActiveId);
                appActivityPlayerInfoVO.setApplyRecordJson(dynamicFormRecordVO);
            }
        }
        appActivityEvaluateVO.setAppActivityPlayerInfoVO(appActivityPlayerInfoVO);

        YcqActivityParticipationDetails activityIdAndUserId = ycqActivityParticipationDetailsRepo.findByActivityIdAndUserId(dto.getActivityId(), dto.getUserId());
        if (!ObjectUtil.isEmpty(activityIdAndUserId)){
            ParticipationDetailsVO participationDetailsVO = new ParticipationDetailsVO();
            if (activityIdAndUserId.getPublishTime()!=null){
                participationDetailsVO.setPublishTime(DateUtils.formatLocalDateTime(activityIdAndUserId.getPublishTime()));
            }

            participationDetailsVO.setYcqActivityParticipationDetails(activityIdAndUserId.getId());
            BeanUtils.copyProperties(activityIdAndUserId,participationDetailsVO);
            TsmWorker tsmWorker = tsmWorkerRepo.findByLoginIdAndState(activityIdAndUserId.getUserId().intValue(), 1);
            if (tsmWorker!=null){
                participationDetailsVO.setWorkerName(tsmWorker.getWorkerName());
                participationDetailsVO.setHeadPortraitPicKey(tsmWorker.getHeadPortraitPicKey());
            }
            //图片
            List<YcqActivityImageVO> attachments =new ArrayList<>();
            List<YcqActivityImage> images = ycqActivityImageRepo.findByUserIdAndActivityId(activityIdAndUserId.getUserId(), dto.getActivityId());
            if (!CollectionUtils.isEmpty(images)){
                for (YcqActivityImage image : images) {
                    YcqActivityImageVO imageVO = new YcqActivityImageVO();
                    imageVO.setImageUrl(image.getImageUrl());
                    attachments.add(imageVO);
                }
                participationDetailsVO.setAttachments(attachments);
            }
            appActivityEvaluateVO.setParticipationDetailsVO(participationDetailsVO);
        }

        return Result.ofSuccess(appActivityEvaluateVO);
    }

    @Override
    public List<ParticipationDetailsVO> queryActivityEvaluate(List<Long> activityId) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        List<ParticipationDetailsVO> participationDetailsVOList = new ArrayList<>();

            List<YcqActivityParticipationDetails> byUserIdAndActivityId = ycqActivityParticipationDetailsRepo.findByActivityIdInAndUserId(activityId, Long.valueOf(worker.getLoginId()));
            if (!ObjectUtil.isEmpty(byUserIdAndActivityId)){
                for (YcqActivityParticipationDetails ycqActivityParticipationDetails : byUserIdAndActivityId) {
                    ParticipationDetailsVO participationDetailsVO = new ParticipationDetailsVO();
                    if (ycqActivityParticipationDetails.getPublishTime()!=null){
                        participationDetailsVO.setPublishTime(DateUtils.formatLocalDateTimeWithoutSecond(ycqActivityParticipationDetails.getPublishTime()));
                    }
                    participationDetailsVO.setYcqActivityParticipationDetails(ycqActivityParticipationDetails.getId());
                    BeanUtils.copyProperties(ycqActivityParticipationDetails,participationDetailsVO);
                    //图片
                    List<YcqActivityImageVO> attachments =new ArrayList<>();
                    List<YcqActivityImage> images = ycqActivityImageRepo.findByUserIdAndActivityId(Long.valueOf(worker.getLoginId()), ycqActivityParticipationDetails.getActivityId());

                    TsmWorker tsmWorker = tsmWorkerRepo.findByLoginIdAndState(ycqActivityParticipationDetails.getUserId().intValue(), 1);
                    if (tsmWorker!=null){
                        participationDetailsVO.setWorkerName(tsmWorker.getWorkerName());
                        participationDetailsVO.setHeadPortraitPicKey(tsmWorker.getHeadPortraitPicKey());
                    }
                    if (!CollectionUtils.isEmpty(images)){
                        for (YcqActivityImage image : images) {
                            YcqActivityImageVO imageVO = new YcqActivityImageVO();
                            imageVO.setImageUrl(image.getImageUrl());
                            attachments.add(imageVO);
                        }
                        participationDetailsVO.setAttachments(attachments);
                    }
                    participationDetailsVOList.add(participationDetailsVO);
                }

            }


        return participationDetailsVOList;
    }

    @Override
    public void deleteDefaultTemplate(Long activityId) {
        ycqActivityParticipationDetailsRepo.deleteByActivityId(activityId);
        ycqActivityImageRepo.deleteByActivityId(activityId);
    }

    @Override
    public Result<ParticipationDetailsVO> queryUserEvaluate(AppActivityEvaluateDTO dto) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        if (dto.getActivityId()==null){
            throw new BusinessException("活动id不能为空！");
        }
        ParticipationDetailsVO participationDetailsVO = new ParticipationDetailsVO();
        YcqActivityParticipationDetails byUserIdAndActivityId = ycqActivityParticipationDetailsRepo.findByUserIdAndActivityId(Long.valueOf(worker.getLoginId()), dto.getActivityId());
        if (!ObjectUtil.isEmpty(byUserIdAndActivityId)){
            BeanUtils.copyProperties(byUserIdAndActivityId,participationDetailsVO);
            TsmWorker tsmWorker = tsmWorkerRepo.findByLoginIdAndState(byUserIdAndActivityId.getUserId().intValue(), 1);
            if (tsmWorker!=null){
                participationDetailsVO.setWorkerName(tsmWorker.getWorkerName());
                participationDetailsVO.setHeadPortraitPicKey(tsmWorker.getHeadPortraitPicKey());
            }
            //图片
            List<YcqActivityImageVO> attachments =new ArrayList<>();
            List<YcqActivityImage> images = ycqActivityImageRepo.findByUserIdAndActivityId(Long.valueOf(worker.getLoginId()), byUserIdAndActivityId.getActivityId());
            if (!CollectionUtils.isEmpty(images)){
                for (YcqActivityImage image : images) {
                    YcqActivityImageVO imageVO = new YcqActivityImageVO();
                    imageVO.setImageUrl(image.getImageUrl());
                    attachments.add(imageVO);
                }
                participationDetailsVO.setAttachments(attachments);
            }
        }

        return Result.ofSuccess(participationDetailsVO);
    }


}
