package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONObject;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.RSAUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.dynamicform.YcqQzhDynamicForm;
import com.crunii.micro.domain.repo.main.dynamicform.YcqQzhDynamicFormRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormRecordReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormReqDTO;
import com.crunii.micro.service.qzh.dto.dynamicform.DynamicFormColumnDTO;
import com.crunii.micro.service.qzh.dto.dynamicform.DynamicFormColumnValueDTO;
import com.crunii.micro.service.qzh.dto.dynamicform.DynamicFormTableReqDTO;
import com.crunii.micro.service.qzh.jdbc.DynamicFormDao;
import com.crunii.micro.service.qzh.service.DynamicFormService;
import com.crunii.micro.service.qzh.utils.PinyinUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

import static com.crunii.micro.service.qzh.api.constants.QzhConstants.WORKER_NOT_EXISTS;

/**
 * @ClassName DynamicFormService
 * @Description 动态表单
 * @Author Jiang
 * @Date 2023/10/13
 * @Version 1.0
 */
@Service
public class DynamicFormServiceImpl implements DynamicFormService {

    @Autowired
    DynamicFormDao dynamicFormDao;

    @Autowired
    YcqQzhDynamicFormRepo dynamicFormRepo;


    /**
     * @MethodName saveDynamicFormJson
     * @Description 动态表单模板分页查询
     * @param: dynamicFormJsons
     * @return: com.crunii.micro.common.dto.Result
     * @Author Jiang
     * @Date 2023/10/13 17:14
     */
    @Override
    public PageRsp<YcqQzhDynamicForm> pageDynamicForm(PageReq<DynamicFormReqDTO> req) {
        return null;
    }

    /**
     * @MethodName saveDynamicFormJson
     * @Description 保存动态表单的json数据
     * @param: dynamicForm
     * @Author Jiang
     * @Date 2023/10/13 17:15
     */
    @Override
    public void saveDynamicFormJson(Worker worker, DynamicFormReqDTO dynamicFormReq) {

//        //获取json数据中的值
//        //将动态json数据存入数据库 YCQ_QZH_DYNAMIC_FORM 表中
//        Map properties = (Map) dynamicFormJsons.get("properties");
//        //给tableName添加前缀，标识为自定义
//        String tableName = "dynamic_" + properties.get("tableName");
//        YcqQzhDynamicForm dynamicFormJson = new YcqQzhDynamicForm();
//        dynamicFormJson.setFormName(tableName);
//        dynamicFormJson.setFormJson(dynamicFormJsons.toString());
//        dynamicFormRepo.save(dynamicFormJson);
//
//        //解析json中的控件信息，并存入 List<DynamicFormColumnDTO>中暂存
//        try {
//            List<DynamicFormColumnDTO> dynamicFormColumnDTOS = new ArrayList<>();
//            ObjectMapper objectMapper = new ObjectMapper();
//            JsonNode jsonNode = objectMapper.readTree(dynamicFormJsons.toString());
//            JsonNode propertiesNode = jsonNode.get("properties");
//            if (propertiesNode != null && propertiesNode.isObject()) {
//                // 遍历properties中的键值对
//                propertiesNode.fields().forEachRemaining(entry -> {
//                    String key = entry.getKey();
//                    JsonNode valueNode = entry.getValue();
//                    System.out.println("Key: " + key);
//                    System.out.println("valueNode" + valueNode);
//
//                    DynamicFormColumnDTO dto = new DynamicFormColumnDTO();
//                    dto.setColumnName(valueNode.get("widget").asText() + "_" + key);
//                    dto.setColumnDescribe(valueNode.get("title").asText());
//                    dynamicFormColumnDTOS.add(dto);
//                });
//            }
//            //1通过json数据解析出值 存入列表保存
//
//            //2、根据json数据在数据库中新创建表，每个动态表单会在【YCQ_QZH_DYNAMIC_FORM】 表中新建一条数据，然后会创建一张表 create table 语句，将通过 tableName（表名）生成表，json解析出信息作为字段名
//            createAutoTable(tableName, dynamicFormColumnDTOS);
//        } catch (JsonProcessingException e) {
//            throw new RuntimeException(e);
//        }


        //获取json数据中的值
        //将动态json数据存入数据库 YCQ_QZH_DYNAMIC_FORM 表中

        worker = getWorker();

        // 后期做成通用的把 YCQ_QZH_DYNAMIC_FORM 表业务id 移出
        //新增活动或编辑活动时直接到用这个方法 首先删除该活动的动态表单字端
        dynamicFormRepo.deleteByActivityId((dynamicFormReq.getActivityId()));

//        String formTableName = PinyinUtil.getPinyinInitials(dynamicFormReq.getFormName());
//        //自定义tableName
//        String tableName = "dynamic_form_" + formTableName;
        YcqQzhDynamicForm dynamicFormJson = new YcqQzhDynamicForm();
//        dynamicFormJson.setFormTableName(tableName);
        BeanUtils.copyProperties(dynamicFormReq, dynamicFormJson);
        dynamicFormJson.setCreateBy(Long.valueOf(worker.getLoginId()));
        dynamicFormJson.setCreateTime(new Date());
        dynamicFormJson.setDeleted(0);
        dynamicFormRepo.save(dynamicFormJson);

        //解析json中的控件信息，并存入 List<DynamicFormColumnDTO>中暂存
        try {
            List<DynamicFormColumnDTO> dynamicFormColumnDTOS = new ArrayList<>();
            ObjectMapper objectMapper = new ObjectMapper();
            JsonNode jsonNode = objectMapper.readTree(dynamicFormReq.getFormJson());
            JsonNode propertiesNode = jsonNode.get("properties");
            if (propertiesNode != null && propertiesNode.isObject()) {
                // 遍历properties中的键值对
                propertiesNode.fields().forEachRemaining(entry -> {
                    String key = entry.getKey();
                    JsonNode valueNode = entry.getValue();
                    DynamicFormColumnDTO dto = new DynamicFormColumnDTO();
                    dto.setColumnName(valueNode.get("widget").asText() + "_" + key);
                    dto.setColumnDescribe(valueNode.get("title").asText());
                    dynamicFormColumnDTOS.add(dto);
                });
            }
            //1通过json数据解析出值 存入列表保存 (未建-后期建表存)

            //2、根据json数据在数据库中新创建表，每个动态表单会在【YCQ_QZH_DYNAMIC_FORM】 表中新建一条数据，然后会创建一张表 create table 语句，将通过 tableName（表名）生成表，json解析出信息作为字段名
//            createAutoTable(tableName, dynamicFormColumnDTOS);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * @MethodName getWorker
     * @Description 获取用户
     * @return: com.crunii.micro.domain.dto.Worker
     * @Author Jiang
     * @Date 2023/10/16 16:09
     */
    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        return worker;
    }


    /**
     * @MethodName createAutoTable
     * @Description 根据json数据创建表
     * @param: tableName
     * @param: tableFields
     * @Author Jiang
     * @Date 2023/10/13 15:05
     */
    public void createAutoTable(String tableName, List<DynamicFormColumnDTO> dynamicFormColumnDTOS) {
        //组装建表语句
        String createSql = "CREATE TABLE " + tableName + "(  id BIGINT IDENTITY(1, 1) NOT NULL, CREATE_WORKER_ID VARCHAR2(255),UPDATE_WORKER_ID VARCHAR2(255),CREATE_WORKER_NAME VARCHAR2(255),UPDATE_WORKER_NAME VARCHAR2(255),CREATE_TIME dateTime,UPDATE_TIME dateTime,business_id VARCHAR2(255),";
        String comment = ";  ";
        for (int i = 0; i < dynamicFormColumnDTOS.size(); i++) {
            DynamicFormColumnDTO dynamicFormColumn = dynamicFormColumnDTOS.get(i);
            if (i == dynamicFormColumnDTOS.size() - 1) {
                String value = "\"" + dynamicFormColumn.getColumnName() + "\"" + " " + " VARCHAR2(255) " + " ) ";
                createSql += value;
                //字段注释拼接
                comment += " COMMENT ON COLUMN " + tableName + "." + "\"" + dynamicFormColumn.getColumnName() + "\"" + " IS " + "'" + dynamicFormColumn.getColumnDescribe() + "' ; ";

            } else {
                String value = "\"" + dynamicFormColumn.getColumnName() + "\"" + " " + " VARCHAR2(255) " + " , ";
                createSql += value;
                //字段注释拼接
                comment += " COMMENT ON COLUMN " + tableName + "." + "\"" + dynamicFormColumn.getColumnName() + "\"" + " IS " + "'" + dynamicFormColumn.getColumnDescribe() + "' ; ";
            }
        }
        System.out.println(createSql + comment);
//        dynamicFormDao.createTable(createSql + comment); TODO 不能连续执行SQL
        dynamicFormDao.createTable(createSql);
    }


    /**
     * @MethodName addTableData
     * @Description 通过表名新增动态表对应字段数据-（保存用户表单填的数据）
     * @param: tableData
     * @Author Jiang
     * @Date 2023/10/13 15:19
     */
    @Override
    public void saveTableData(Worker worker, DynamicFormRecordReqDTO dynamicFormRecord) {

//        // 将tableName存入tableData中
//        String tableName = "" + jsonObject.get("tableName");
//        Map<String, Object> tableData = new HashMap<>();
//        tableData.put("tableName", tableName);
//
//        // 自动生成32位UUID存入表的字段映射中，确保insert操作
//        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
//        Map<String, String> tableFieldMap = new HashMap();
//        tableFieldMap.put("id", uuid);
//        // processId, 将processId存入表的字段映射中
//        String processId = (String) jsonObject.get("processId");
//        tableFieldMap.put("processId", processId);
//        // 组装字段数据到字段映射中
//        Map<String, String> map = (Map) jsonObject.get("tableFiled");
//        tableFieldMap.putAll(map);
//
//        // 再将table的字段映射map存入tableData中
//        tableData.put("tableFieldMap", tableFieldMap);


        //解析json中的控件信息，并存入 List<DynamicFormColumnDTO>中暂存
        try {
            List<DynamicFormColumnValueDTO> dynamicFormColumnValue = new ArrayList<>();
            ObjectMapper objectMapper = new ObjectMapper();
            JsonNode jsonNode = objectMapper.readTree(dynamicFormRecord.getFormJson());
            JsonNode propertiesNode = jsonNode.get("properties");
            if (propertiesNode != null && propertiesNode.isObject()) {
                // 遍历properties中的键值对
                propertiesNode.fields().forEachRemaining(entry -> {
                    String key = entry.getKey();
                    JsonNode valueNode = entry.getValue();
                    DynamicFormColumnValueDTO columnValue = new DynamicFormColumnValueDTO();
                    columnValue.setColumnName(valueNode.get("widget").asText() + "_" + key);
                    columnValue.setColumnDescribe(valueNode.get("title").asText());
                    columnValue.setValue(valueNode.get("value").asText());
                    dynamicFormColumnValue.add(columnValue);
                });
            }
            //解析APP提交的json数据解析出用户填的值 通过表名存到对应的表中
            insertByTableName(worker, dynamicFormRecord.getFormTableName(), dynamicFormColumnValue);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }


    /**
     * @MethodName insertBytableName
     * @Description 解析APP提交的json数据解析出用户填的值 通过表名保存数据到对应的表中
     * @Author Jiang
     * @Date 2023/10/16 19:28
     */
    private void insertByTableName(Worker worker, String formTableName, List<DynamicFormColumnValueDTO> dynamicFormColumnValue) {

        StringBuilder insertSql = new StringBuilder();
        insertSql.append("insert into ").append(formTableName).append(" (");
        for (DynamicFormColumnValueDTO column : dynamicFormColumnValue) {

            insertSql.append("\"" + column.getColumnName() + "\"").append(",");
        }
        insertSql = new StringBuilder(insertSql.substring(0, insertSql.length() - 1));
        insertSql.append(" , CREATE_TIME");
        insertSql.append(" , CREATE_WORKER_NAME");
        insertSql.append(" , CREATE_WORKER_ID");
        insertSql.append(") values (");
        for (int i = 0; i < dynamicFormColumnValue.size() + 3; i++) {
            insertSql.append("?,");
        }
        insertSql = new StringBuilder(insertSql.substring(0, insertSql.length() - 1));

        insertSql.append(")");
        List<Object> params = new ArrayList<>();
        for (DynamicFormColumnValueDTO dynamicFormColumnValueDTO : dynamicFormColumnValue) {
            params.add(dynamicFormColumnValueDTO.getValue());
        }
        params.add(new Date());
        params.add(worker.getWorkerName());
        params.add(worker.getLoginId());
        dynamicFormDao.insertByTableName(insertSql.toString(), params.toArray());
    }


    /**
     * @MethodName findById
     * @Description 通过业务id查询该业务下的动态表单的json数据
     * @param: id
     * @return: com.crunii.micro.common.dto.Result<?>
     * @Author Jiang
     * @Date 2023/10/13 16:18
     */
    @Override
    public Result<YcqQzhDynamicForm> findByBusinessId(Long id) {
        YcqQzhDynamicForm dynamicFormJson = dynamicFormRepo.findByActivityIdAndDeleted(id, 0);
        if (null != dynamicFormJson) {
            return new Result<>(Boolean.TRUE, dynamicFormJson);
        }
        return new Result<>(Boolean.TRUE, new YcqQzhDynamicForm());
    }


    /**
     * @MethodName findTableDataByIdAndTableName
     * @Description 通过业务id和表名查询动态表单提交的表单数据的值
     * @param: dynamicFormTableReq
     * @return: com.crunii.micro.common.dto.Result<?>
     * @Author Jiang
     * @Date 2023/10/13 16:50
     */
    @Override
    public Result<Map<String, Object>> findTableDataByBusinessIdAndTableName(DynamicFormTableReqDTO dynamicFormTableReq) {
        Map<String, Object> map = dynamicFormDao.findTableDataByBusinessIdAndTableName(dynamicFormTableReq);
        return new Result<>(Boolean.TRUE, map);

    }


}
