package com.crunii.micro.service.qzh.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqAnswerRecord;
import com.crunii.micro.domain.entity.main.YcqExamRecord;
import com.crunii.micro.domain.entity.main.YcqTopicOption;
import com.crunii.micro.domain.entity.main.YcqYzyhTest;
import com.crunii.micro.domain.repo.main.YcqExamRecordRepo;
import com.crunii.micro.domain.repo.main.YcqTopicOptionRepo;
import com.crunii.micro.domain.repo.main.YcqYzyhTestRepo;

import com.crunii.micro.service.qzh.dto.*;
import com.crunii.micro.service.qzh.jdbc.YcqExamJdbcDao;
import com.crunii.micro.service.qzh.service.ExamService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


import java.time.LocalDate;
import java.util.List;
import java.util.Objects;

import java.util.stream.Collectors;

/**
 * @author lyx
 * @date 2023/9/5
 */
@Service
public class ExamServiceImpl implements ExamService {

	@Autowired
	private YcqExamJdbcDao ycqExamJdbcDao;
	@Autowired
	private YcqExamRecordRepo ycqExamRecordRepo;
	@Autowired
	private YcqYzyhTestRepo ycqYzyhTestRepo;
	@Autowired
	private YcqTopicOptionRepo ycqTopicOptionRepo;


	@Override
	public Result<List<AppExamListRespDTO>> queryExamList() {
		return Result.ofSuccess(ycqExamJdbcDao.getExamList());
	}

	@Override
	public Result<AppExamInfoRespDTO> queryExamInfoById(Worker worker,Integer id) {
		// 1 通过worker_id查询考试记录表，没有的话代表还没有考，有的话查询还有几次机会
		YcqAnswerRecord ycqAnswerRecord = ycqExamJdbcDao.getExamCountByWorkerAndId(id,worker.getLoginId());
		// 2 查询考试信息
		AppExamInfoRespDTO appExamInfoVO = ycqExamJdbcDao.getExamInfoById(id);
		if (Objects.nonNull(ycqAnswerRecord)){
			appExamInfoVO.setResidueTimes(appExamInfoVO.getExamTimes() -  ycqAnswerRecord.getExamTimes());
			appExamInfoVO.setExamTimes(ycqAnswerRecord.getExamTimes());
		}
		return Result.ofSuccess(appExamInfoVO);
	}

	@Override
	public Result<AppExamContentRespDTO> queryExamById(Worker worker,Integer id) {
		//通过考试id查询考试
		//AppExamContentRespDTO appExamInfoVO = ycqExamJdbcDao.getExamById(id);
		YcqYzyhTest ycqYzyhTest = ycqYzyhTestRepo.findById(String.valueOf(id)).orElse(null);
		if(Objects.isNull(ycqYzyhTest)){
			throw new BusinessException("未查询到考试！");
		}
		AppExamContentRespDTO appExamInfoVO = new AppExamContentRespDTO();
		appExamInfoVO.setId(Integer.valueOf(ycqYzyhTest.getId()));
		appExamInfoVO.setExamName(ycqYzyhTest.getTitle());
		YcqExamRecord ycqExamRecord = ycqExamRecordRepo.findByExamId(id);
		LocalDate now = LocalDate.now();
		//有考试记录修改，无则新增
		if(Objects.isNull(ycqExamRecord)){
			//新增一条考试记录，考试开始时间是现在
			ycqExamRecord = new YcqExamRecord();
			ycqExamRecord.setExamId(id);
			ycqExamRecord.setCreateTime(now);
			ycqExamRecord.setUserId(worker.getLoginId());
			ycqExamRecord.setOrgId(worker.getOrgId());
			ycqExamRecord.setCreateBy(worker.getWorkerName());
			ycqExamRecord.setExamTimes(1);
		}else{
			ycqExamRecord.setUpdateBy(worker.getWorkerName());
			ycqExamRecord.setUpdateTime(now);
			ycqExamRecord.setExamTimes(ycqExamRecord.getExamTimes()+1);
			//考试次数校验
			if(ycqExamRecord.getExamTimes() > ycqYzyhTest.getTotalTimes()){
				throw new BusinessException("考试次数已用完！");
			}
		}
		appExamInfoVO.setExamResidueTime(ycqYzyhTest.getEffectiveTime());
		ycqExamRecord.setStartTime(now);
		ycqExamRecordRepo.save(ycqExamRecord);
		//通过试卷id查询本次考试的所有题目
		//题目集合
		List<ExamTopicRespDTO> topics = ycqExamJdbcDao.getTopicByTestId(id);
		//拿到题目id的集合，查询所有题目的所有答案选项
		List<Integer> topicsIds = topics.stream().map(ExamTopicRespDTO::getTopicId).collect(Collectors.toList());
		List<ExamOptionRespDTO> options = ycqExamJdbcDao.getOptionByTopicId(topicsIds);
		for (ExamTopicRespDTO topic:topics) {
			List<ExamOptionRespDTO> collect =
					options.stream().filter(o -> o.getTopicId().equals(topic.getTopicId()))
							.collect(Collectors.toList());
			topic.setExamOptions(collect);
		}

		appExamInfoVO.setExamTopics(topics);
		return Result.ofSuccess(appExamInfoVO);
	}



	@Override
	public Result<String> doSubmit(AppExamReqDTO appExamReqDTO) {
		//先拿到试卷正确答案及每题类型分值
		Integer id = appExamReqDTO.getId();
		//提交的题和答案
		List<AppExamSubmitReqDTO> submits = appExamReqDTO.getAppExamSubmits();
		YcqYzyhTest ycqYzyhTest = ycqYzyhTestRepo.findById(String.valueOf(id)).orElse(null);
		if(Objects.isNull(ycqYzyhTest)){
			throw new BusinessException("未查询到考试！");
		}
		//拿到所有的题id
		List<Integer> topics = submits.stream().map(AppExamSubmitReqDTO::getTopicId).collect(Collectors.toList());
		//通过题拿到正确答案,题目类型前端返回，，，暂时Integer=1,2,3
		List<YcqTopicOption> rightOps =  ycqTopicOptionRepo.findByIdInAndDeleted(topics,1);
		int a =1;
		//通过循环比对每道题的答案，加上对应分值
		int score = 0;
		for (AppExamSubmitReqDTO submit : submits) {
			//拿到本题正确答案
			List<Integer> rightOp =
					rightOps.stream().filter(r -> r.getTopicId().equals(submit.getTopicId()))
							.map(YcqTopicOption::getId).collect(Collectors.toList());
			//拿到提交的答案
			List<Integer> submitOp = submit.getOptionIds();
			submitOp.stream().forEach(o -> Long.valueOf(o));
			//判断答案是否正确
			if (rightOp.containsAll(submitOp)){
				switch (a) {
				case 1:
					score+=ycqYzyhTest.getSingleRule();
				case 2:
					score+=ycqYzyhTest.getJudgeRule();
				case 3:
					score+=ycqYzyhTest.getMultiRule();
				}
			}
		}
		//分数计算完毕后，查询已经考了的分数，核对哪个分数更高
		YcqExamRecord ycqExamRecord = ycqExamRecordRepo.findByExamId(id);
		Double examScore = ycqExamRecord.getExamScore();
		if(examScore < score){
			//存入本次考试记录
		}
		return Result.ofSuccess("提交成功");
	}
}
