package com.crunii.micro.service.qzh.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.entity.main.MessageSendTask;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.feign.client.SmsService;
import com.crunii.micro.service.qzh.service.MessageSendLogSaveService;
import com.crunii.micro.service.qzh.service.MessageSendingStrategy;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author lmlksy
 * @date 2023/11/16 16:49
 */
@Service
public class GenericMessageSendingStrategy implements MessageSendingStrategy {

    @Autowired
    private SmsService smsService;

    @Autowired
    private MessageSendLogSaveService saveMessageSendLog;


    @Override
    public void doSendMessage(MessageSendTask task, List<TsmWorker> workers,String templateCode) {
        workers.forEach(worker -> {

            //发送方式短信或全部
            if (task.getSendWay().contains("1")||task.getSendWay().contains("3")) {
                if (templateCode!=null){
                    Result<String> stringResult = smsService.sendSms(worker.getMobile(), templateCode, task.getFillData());
                    if (!stringResult.isSuccess()){
                        throw new RuntimeException("短信微服务异常，请检查日志");
                    }
                }
            }

            doSaveLogAsync(task, worker);
        });

    }

    //todo 是否异步处理。。
    public void doSaveLogAsync(MessageSendTask task, TsmWorker worker) {
        saveMessageSendLog.doSaveMessageSendLog(task, worker);
    }
}
