package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.dto.CoincidenceOrganizationRespDTO;
import com.crunii.micro.service.qzh.dto.ExportIndexSystemRespDTO;
import com.crunii.micro.service.qzh.jdbc.IndicatorAppraiseDao;
import com.crunii.micro.service.qzh.service.IndicatorAppraiseExportService;
import com.crunii.micro.service.qzh.utils.ExcelpoiUtil;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

/**
 * @Author thw
 * @date 2023/9/14
 * @description 指标体系评价-导出指标评价
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class IndicatorAppraiseExportServiceImpl implements IndicatorAppraiseExportService {

    @Resource
    private final IndicatorAppraiseDao indicatorAppraiseDao;

    @Override
    public void exportIndexSystemContent(HttpServletResponse response, Worker worker, String versionId) {
        if (StringUtils.isBlank(versionId)) {
          throw new BusinessException("版本id不能为空");
        }
        // todo 权限
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        // 首先查询到维权岗的下拉列表
        List<CoincidenceOrganizationRespDTO> postInformation =
            indicatorAppraiseDao.findPostInformation(currWorker, null);
        List<String> ids = postInformation.stream().map(CoincidenceOrganizationRespDTO::getId).distinct().collect(Collectors.toList());
        String inSql = "";
        boolean label = false;
        if (CollectionUtils.isNotEmpty(ids)) {
            inSql = ids.stream().collect(Collectors.joining(",", "(", ")"));
            label = true;
        }
        // 得到所有维权岗的下的指标数据
        List<ExportIndexSystemRespDTO> listVO = indicatorAppraiseDao.exportIndexSystemContent(currWorker, inSql, label, versionId);
        // 通过维权岗id分组
        Map<Long, List<ExportIndexSystemRespDTO>> groupByIdList =
            listVO.stream().collect(Collectors.groupingBy(ExportIndexSystemRespDTO::getParentId));

        try (ServletOutputStream outputStream = response.getOutputStream()) {

            // 获取类上注解相关内容
            Map<Integer, String> easyExcelHeadMap = getEasyExcelHeadCount(ExportIndexSystemRespDTO.class);
            // 每个一sheet表的表头数组相同
            String[] headColumn = new String[easyExcelHeadMap.size()];
            for (int i = 0; i < easyExcelHeadMap.keySet().size(); i++) {
                headColumn[i] = easyExcelHeadMap.get(i);
            }
            // 存放sheet的id(key)和sheet名称
            TreeMap<Long, String> indexMap = new TreeMap<>();
            if (MapUtils.isNotEmpty(groupByIdList)) {
                for (Long key : groupByIdList.keySet()) {
                    for (int i = 0; i < postInformation.size(); i++) {
                        if (key.equals(Long.valueOf(postInformation.get(i).getId()))) {
                            String sheetName = postInformation.get(i).getUnitName();
                            // 直有当sheet里面有数据时，才创建一个sheet
                            if (CollectionUtils.isNotEmpty(groupByIdList.get(key))) {
                                indexMap.put(key, sheetName);
                            }
                        }
                    }
                }
            }

            /** 第一步，创建一个Workbook，对应一个Excel文件 */
            HSSFWorkbook wb = new HSSFWorkbook();
            HSSFCellStyle contentStyle = ExcelpoiUtil.createContentCellStyle(wb);
            // 存放有数据的Sheet与对应的key
            Map<Long, HSSFSheet> indexSheetTreeMap = new TreeMap<>();
            HSSFSheet sheet = null;
            Long keyId = null;
            if (MapUtils.isNotEmpty(indexMap)) {
                for (Long key : indexMap.keySet()) {
                    /** 第二步，在Workbook中添加一个sheet,对应Excel文件中的sheet */
                    sheet = wb.createSheet(indexMap.get(key));
                    keyId = key;
                    indexSheetTreeMap.put(key, sheet);
                }
            } else {
                sheet = wb.createSheet("Sheet");
                keyId = 0L;
                indexSheetTreeMap.put(0L, sheet);
            }

            for (Long key : indexSheetTreeMap.keySet()) {
                // 得到每一个sheet
                HSSFSheet tempSheet = indexSheetTreeMap.get(key);
                // 创建sheet样式和表头
                // 行号
                int rowNumber = 0;
                // 包装表头
                packHead(headColumn, wb, tempSheet, rowNumber);
                rowNumber++;
                // 获取每一个sheet的数据
                List<ExportIndexSystemRespDTO> listVOs = groupByIdList.get(key);
                // listVOs.forEach(System.out::println);
                if (CollectionUtils.isNotEmpty(listVOs)) {

                    // 第一列要合并的行数
                    int oneCellMergedNum = 0;
                    String oneCellInitValue = "null";
                    // 第二列要合并的行数
                    int twoCellMergedNum = 0;
                    String twoCellInitValue = "null";

                    int endRowIndex = 0;
                    int endCellIndex = 0;

                    int endRowIndex2 = 0;
                    int endCellIndex2 = 0;

                    // 每一行
                    HSSFRow tempRow = null;
                    for (int j = 0; j < listVOs.size(); j++) {
                        tempRow = tempSheet.createRow(rowNumber++);
                        tempRow.setHeight((short)500);
                        // 遍历每一条数据VO
                        ExportIndexSystemRespDTO vo = listVOs.get(j);
                        int cellNumber = 0;
                        // 每一列
                        for (int i = 0; i < headColumn.length; i++) {
                            HSSFCell tempCell = null;
                            // 评价要点
                            if (i == 0) {
                                // 本身内容占据一列(默认只占一列)
                                tempCell = tempRow.createCell(cellNumber++);
                                // 逗号字符串转义
                                String tempStr = commaTransferredMeaning(vo.getEvaluationPoints());
                                tempCell.setCellValue(tempStr);
                                // 比较同一列的值是否相等，相等就需要合并行
                                if (oneCellInitValue.equals(vo.getEvaluationPoints())) {
                                    oneCellMergedNum++;
                                } else {
                                    if (ObjectUtil.isNotEmpty(vo.getEvaluationPoints())) {
                                        // 每当比较的列值重置后，就需要将该合并的内容合并了
                                        oneCellInitValue = vo.getEvaluationPoints();
                                        if (oneCellMergedNum > 0) {
                                            endRowIndex = rowNumber;
                                            endCellIndex = cellNumber;
                                        }
                                    }
                                }
                            }

                            // 一级指标
                            if (i == 1) {
                                // 本身内容占据一列(默认只占一列)
                                tempCell = tempRow.createCell(cellNumber++);
                                String temp = commaTransferredMeaning(vo.getPrimaryIndicators());
                                tempCell.setCellValue(temp);
                                // 比较同一列的值是否相等，相等就需要合并行
                                if (twoCellInitValue.equals(vo.getPrimaryIndicators())) {
                                    twoCellMergedNum++;
                                } else {
                                    // 每当比较的列值重置后，就需要将该合并的内容合并了
                                    if (Objects.nonNull(vo.getPrimaryIndicators())) {
                                        twoCellInitValue = vo.getPrimaryIndicators();
                                        if (twoCellMergedNum > 0) {
                                            endRowIndex2 = rowNumber;
                                            endCellIndex2 = cellNumber;
                                        }
                                    }
                                }
                            }
                            if (i == 2) {
                                tempCell = tempRow.createCell(cellNumber++);
                                String temp = commaTransferredMeaning(vo.getSecondaryIndicators());
                                tempCell.setCellValue(temp);
                            }
                            if (i == 3) {
                                tempCell = tempRow.createCell(cellNumber++);
                                tempCell.setCellValue(vo.getScore());
                            }
                            if (i == 4) {
                                tempCell = tempRow.createCell(cellNumber++);
                                tempCell.setCellValue(vo.getWaitState());
                            }
                            if (i == 5) {
                                tempCell = tempRow.createCell(cellNumber++);
                                tempCell.setCellValue(vo.getYesState());
                            }
                            if (i == 6) {
                                tempCell = tempRow.createCell(cellNumber++);
                                tempCell.setCellValue(vo.getNoState());
                            }
                            tempCell.setCellStyle(contentStyle);
                        }

                        // 每一行结束之后判断，是否需要合并
                        if (endRowIndex > 0) {
                            tempSheet.addMergedRegion(new CellRangeAddress(endRowIndex - oneCellMergedNum - 1,
                                endRowIndex - 1, endCellIndex, endCellIndex));
                            oneCellMergedNum = 0;
                            endRowIndex = 0;
                        } else {
                            // 防止最后有数据重复后，却没有后续的不重复数据(注意：cellNumber此时都为字段个数 7）
                            if (oneCellMergedNum > 0 && (cellNumber - 7 == 0)) {
                                tempSheet.addMergedRegion(
                                    new CellRangeAddress(rowNumber - 1 - oneCellMergedNum, rowNumber - 1, 0, 0));
                                oneCellMergedNum = 0;
                            }
                        }

                        // 每一行结束之后判断，是否需要合并
                        if (endRowIndex2 > 0) {
                            tempSheet.addMergedRegion(new CellRangeAddress(endRowIndex2 - 1 - twoCellMergedNum,
                                endRowIndex2 - 1, endCellIndex2, endCellIndex2));
                            twoCellMergedNum = 0;
                            endRowIndex2 = 0;
                        } else {
                            // 防止最后有数据重复后，却没有后续的不重复数据
                            if (twoCellMergedNum > 0 && (cellNumber - 6 == 1)) {
                                tempSheet.addMergedRegion(
                                    new CellRangeAddress(rowNumber - 1 - twoCellMergedNum, rowNumber - 1, 1, 1));
                                twoCellMergedNum = 0;
                            }
                        }

                    }
                }

                List<Integer> allCellWidth = getAllCellWidth(headColumn, new ArrayList<Integer>(), 600);
                // 自适应列宽
                for (int i = 0; i < headColumn.length; i++) {
                    tempSheet.setColumnWidth(i, allCellWidth.get(i));
                }

                // ExcelpoiUtil.putWatermark(headColumn.length, wb, tempSheet, currWorker);
            }

            String fileName = "各个维护青少年权益岗对应的指标列表导出" + DateUtil.format(new Date(), DateUtil.defaultDT1Pattern);
                // + ExcelTypeEnum.XLS.getValue();
            response.setContentType("application/vnd.ms-excel;charset=utf-8");
            // 文件名称
            fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name());
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION,
                "attachment;filename=" + fileName + ".xlsx" );

            response.flushBuffer();
            wb.write(outputStream);
            if (null != outputStream) {
                outputStream.close();
            }
            wb.close();
        } catch (Exception e) {
            errorWrite(response, e);
        }
    }

    /** 逗号字符串转义 */
    private static String commaTransferredMeaning(String str) {
        if (StringUtils.isNotBlank(str)) {
            // 先判断字符里是否含有逗号
            if (str.endsWith(",") || str.endsWith("，")) {
                // 如果还有双引号，先将双引号转义，避免两边加了双引号后转义错误
                if (str.contains("\"")) {
                    str = str.replace("\"", "\"\"");
                }
                // 将逗号转义
                str = "\"" + str + "\"";
            }
        } else {
            str = "";
        }
        return str;
    }

    /**
     * 获取 easyExcel导出表头的内容
     */
    private static Map<Integer, String> getEasyExcelHeadCount(Class<?> head) {
        // @ExcelProperty
        Field[] fields = head.getDeclaredFields();
        Map<Integer, String> treeMap = new TreeMap<>();
        if (fields != null && fields.length > 0) {
            for (Field field : fields) {
                boolean exist = field.isAnnotationPresent(ExcelProperty.class);
                if (exist) {
                    int index = field.getAnnotation(ExcelProperty.class).index();
                    String[] value = field.getAnnotation(ExcelProperty.class).value();
                    String realValue = Arrays.stream(value).collect(Collectors.joining());
                    treeMap.put(index, realValue);
                }
            }
        }
        return treeMap;
    }

    /** 设置每列的宽度 */
    public List<Integer> getAllCellWidth(String[] headColumn, List<Integer> widthList, int initWidth) {
        if (ArrayUtils.isNotEmpty(headColumn)) {
            for (int i = 0; i < headColumn.length; i++) {
                widthList.add((headColumn[i].length() * 2) * initWidth);
            }
        }
        return widthList;
    }

    /** 创建sheet样式和表头 */
    private void packHead(String[] headColumn, HSSFWorkbook wb, HSSFSheet sheet, int rowNumber) {
        /** 第三步，设置样式以及字体样式 */
        // HSSFCellStyle titleStyle = ExcelpoiUtil.createTitleCellStyle(wb);
        HSSFCellStyle headerStyle = ExcelpoiUtil.createHeadCellStyle(wb);

        /** 第四步，创建标题 ,合并标题单元格 */
        HSSFRow oneRow = sheet.createRow(rowNumber);
        oneRow.setHeight((short)500);
        // 封装第一行表头内容
        int cellNumberOne = 0;
        for (int k = 0; k < headColumn.length; k++) {
            // 本身内容占据一列(默认只占一列)
            HSSFCell tempCell = oneRow.createCell(cellNumberOne++);
            tempCell.setCellValue(headColumn[k]);
            tempCell.setCellStyle(headerStyle);
        }
    }

    /** 设置导出错误响应 */
    private void errorWrite(HttpServletResponse response, Exception e) {
        log.error(e.getMessage(), e);
        response.setContentType("application/json");
        response.setCharacterEncoding("utf-8");
        response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR.value());
    }
    
    /**
     * 导出指标体系内容
     * @author leixb
     * @since 2023-11-25 15:29
     * @param response
     * @param worker
     * @param versionId
     */
    public void exportIndexSystemContentNew(HttpServletResponse response, Worker worker, String versionId) {
        if (StringUtils.isBlank(versionId)) {
            throw new BusinessException("版本id不能为空");
        }
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        // 首先查询到维权岗的下拉列表
        List<CoincidenceOrganizationRespDTO> postInformation =
            indicatorAppraiseDao.findPostInformation(currWorker, null);
        
        
    }

}
