package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqMienEvaluate;
import com.crunii.micro.domain.repo.main.YcqMienEvaluateRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.dto.*;
import com.crunii.micro.service.qzh.jdbc.IndicatorAppraiseDao;
import com.crunii.micro.service.qzh.service.IndicatorAppraiseService;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

/**
 * @Author thw
 * @date 2023/9/5
 * @description
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class IndicatorAppraiseServiceImpl implements IndicatorAppraiseService {

    private final IndicatorAppraiseDao indicatorAppraiseDao;

    private final YcqMienEvaluateRepo ycqMienEvaluateRepo;


    /**
     * @param req
     * @Description: 指标体系评价列表
     * @Author: thw
     * @Date: 2023/9/7 10:19
     * @Params:
     * @return: com.crunii.micro.common.dto.Result<java.util.List < com.crunii.micro.service.qzh.api.domain.dto.IndicatorAppraiseRespDTO>>
     */
    @Override
    public Result<List<IndicatorAppraiseRespDTO>> queryAppraiseList(Worker worker, IndicatorAppraiseReqDTO req) {
        // todo 权限
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (null != req) {
            if (StringUtils.isBlank(req.getVersionId())) {
                return Result.ofFail("指标版本id为空！");
            }
            if (StringUtils.isBlank(req.getRightsId())) {
                return Result.ofFail("维权岗id为空！");
            }
        }
        return Result.ofSuccess(indicatorAppraiseDao.queryAppraiseList(currWorker, req));
    }
  
    /**
     * 指标体系评价-评定分页
     * @author leixb
     * @since 2023-11-23 16:42
     * @param worker 用户
     * @param req 请求参数
     * @return PageRsp<IndicatorAppraiseDecideRespDTO>
     */
    @Override
    public PageRsp<IndicatorAppraiseDecideRespDTO> appraiseMienPage(Worker worker,
        PageReq<IndicatorAppraiseDecideReqDTO> req) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        return indicatorAppraiseDao.appraiseMienPage(currWorker, req);
    }
  
  /**
   * @Description: 指标体系评价-评定分页
   * @Author: thw
   * @Date: 2023/9/13 16:48
   * @Params:
   * @param worker
   * @param req
   * @return: com.crunii.micro.common.dto.Result<com.crunii.micro.common.dto.PageRsp<com.crunii.micro.service.qzh.api.domain.dto.IndicatorAppraiseDecideRespDTO>>
   **/
  // public Result<PageRsp<IndicatorAppraiseDecideRespDTO>> appraiseMienPageOld(Worker worker,
  //     PageReq<IndicatorAppraiseDecideReqDTO> req) {
  //
  //   Worker currWorker = CurrWorkerUtil.getCurrWorker();
  //
  //   if (ObjectUtil.isEmpty(req.getQuery().getIds())){
  //     throw new BusinessException("必传参数为空!");
  //   }
  //
  //   // 如果风采一次某没有评定过则需要新增在评定表中
  //   String ids = req.getQuery().getIds();
  //   // 获取风采id集合
  //   Set<Long> MienIdS = Arrays.stream(ids.split(",")).map(e -> Long.valueOf(e)).collect(Collectors.toSet());
  //   // 非第一次评定地风采
  //   List<YcqMienEvaluate> findInfo = ycqMienEvaluateRepo.findAllByMienIdIn(MienIdS);
  //   // 可以查询到的数据，说明非第一次
  //   Set<Long> existIdSet = findInfo.stream().map(e -> e.getMienId()).collect(Collectors.toSet());
  //   // 获取第一次评定的id列表
  //   List<Long> tempIdList = new ArrayList(MienIdS);
  //   if (CollectionUtils.isNotEmpty(tempIdList) && CollectionUtils.isNotEmpty(existIdSet)) {
  //     for (int i = 0; i < tempIdList.size(); i++) {
  //       if (existIdSet.contains(tempIdList.get(i))) {
  //         tempIdList.remove(tempIdList.get(i));
  //       }
  //     }
  //   }
  //
  //   YcqMienEvaluate entity = null;
  //   if (CollectionUtils.isNotEmpty(tempIdList)) {
  //     ArrayList<YcqMienEvaluate> ycqMienEvaluates = new ArrayList<>();
  //     for (Long id : tempIdList) {
  //       entity = new YcqMienEvaluate();
  //       // 指标体系主键
  //       entity.setIndicatorId(Long.valueOf(req.getQuery().getIndexId()));
  //       // 风采主键
  //       entity.setMienId(Long.valueOf(id));
  //       entity.setCreateId(Long.valueOf(currWorker.getLoginId()));
  //       entity.setCreatorBy(currWorker.getWorkerName());
  //       entity.setCreateTime(LocalDateTime.now());
  //       entity.setDelFlag(0);
  //       entity.setState(0);
  //       ycqMienEvaluates.add(entity);
  //     }
  //     // 将第一次评定地内容新增在表中
  //     ycqMienEvaluateRepo.saveAll(ycqMienEvaluates);
  //   }
  //
  //   PageRsp<IndicatorAppraiseDecideRespDTO> pageList = indicatorAppraiseDao
  //       .appraiseMienPage(currWorker, req);
  //   return Result.ofSuccess(pageList);
  // }


    /**
    * @Description: 评定风采
    * @Author: thw
    * @Date: 2023/9/18 16:57
    * @Params:
    * @param worker
    * @param dto
    * @return: com.crunii.micro.common.dto.Result<java.lang.String>
    **/
    @Override
    public Result<String> doMienEvaluate(Worker worker, MienAppraiseReqDTO dto) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        
        if (ObjectUtil.isEmpty(dto.getIds()) || ObjectUtil.isEmpty(dto.getLabel())) {
            throw new BusinessException("必传参数为空");
        }
        // 风采评定表主键ids
        Set<Long> ids = dto.getIds().stream().map(Long::valueOf).collect(Collectors.toSet());
        List<YcqMienEvaluate> allInfo = ycqMienEvaluateRepo.findAllById(ids);
        // 通过：true；驳回：false
        if (CollectionUtils.isNotEmpty(allInfo)) {
            for (YcqMienEvaluate updateEntity : allInfo) {
                if (dto.getLabel()) {
                    updateEntity.setState(1);
                } else {
                    updateEntity.setState(2);
                }
            updateEntity.setUpdateId((long) currWorker.getLoginId());
            updateEntity.setUpdateBy(currWorker.getWorkerName());
            updateEntity.setUpdateTime(LocalDateTime.now());
            }
            ycqMienEvaluateRepo.saveAll(allInfo);
        }else {
          throw new BusinessException("未查询到相关数据。");
        }
        return Result.ofSuccess("评定成功！");
    }


  /**
   * @Description: 查询指标体系版本下拉
   * @Author: thw
   * @Date: 2023/9/7 10:18
   * @Params:
   * @return: com.crunii.micro.common.dto.Result<java.util.List < com.crunii.micro.service.qzh.api.domain.dto.IndexSystemSimpleRespDTO>>
   */
  @Override
  public Result<List<IndexSystemSimpleRespDTO>> findIndexSystemVersion(Worker worker) {
    // todo 权限
    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    return Result.ofSuccess(indicatorAppraiseDao.findIndexSystemVersion());
  }

  /**
   * @Description: 查询符合条件的维权岗信息
   * @Author: thw
   * @Date: 2023/9/8 17:53
   * @Params:
   * @return: com.crunii.micro.common.dto.Result<java.util.List < com.crunii.micro.service.qzh.api.domain.dto.CoincidenceOrgrationRespDTO>>
   **/
  @Override
  public Result<List<CoincidenceOrganizationRespDTO>> findPostInformation(Worker worker, String unitName) {
    // todo 权限
    Worker currWorker = CurrWorkerUtil.getCurrWorker();
    List<CoincidenceOrganizationRespDTO> list = indicatorAppraiseDao.findPostInformation(currWorker, unitName);
    return Result.ofSuccess(list);
  }

}
