package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.util.CollectionUtils;

import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.feign.client.SmsService;
import com.crunii.micro.platform.components.Locker;
import com.crunii.micro.service.qzh.jdbc.TsmWorKerRoleJdbcDao;
import com.crunii.micro.service.qzh.service.MessageSendLogTaskService;
import com.crunii.micro.service.qzh.service.MessageSendingStrategy;

import lombok.extern.log4j.Log4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RAtomicLong;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.transaction.Transactional;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


/**
 * @author lmlksy
 * @date 2023/11/8 16:14
 */
@Log4j
@Service
public class MessageSendLogTaskServiceImpl implements MessageSendLogTaskService {

    @Autowired
    TsmWorkerRepo tsmWorkerRepo;

    @Autowired
    TsmWorKerRoleJdbcDao tsmWorKerRoleJdbcDao;

    @Autowired
    MessageSendTaskRepo messageSendTaskRepo;

    @Autowired
    MessageSendingStrategy messageSendingStrategy;

    @Autowired
    YcqMessageTemplateRepo ycqMessageTemplateRepo;

    //弃用
//    private Set<Long> inProcessTasks = ConcurrentHashMap.newKeySet();

    @Autowired
    private RedissonClient redissonClient;

    private static final String LOCK_KEY = "messageSendTaskLock";
    private static final long LOCK_EXPIRATION = 30; // 锁的过期时间，单位：秒

    // 定义任务状态的 Redis 键
    private static final String TASK_STATUS_KEY = "taskStatusKey";

    @Autowired
    SmsService smsService;
    @Override
    @Scheduled(cron = "*/5 * * * * *")
    public void doMessageSendLogTask() {
        LocalDateTime begin = LocalDateTime.now();
        RLock lock = redissonClient.getLock(LOCK_KEY);
        if (Objects.isNull(lock)) {
            return ;
        }

        boolean isLocked = false;
        RAtomicLong taskStatus =null;
        try {
            isLocked = lock.tryLock(0, LOCK_EXPIRATION, TimeUnit.SECONDS);
            taskStatus = redissonClient.getAtomicLong(TASK_STATUS_KEY);
            log.info("taskStatus开始状态"+taskStatus);
            if (isLocked) {
                // 获取任务状态
                if (taskStatus.compareAndSet(0, 1)) {
                    log.info("taskStatus进行中状态"+taskStatus);
                    try {
                        List<MessageSendTask> messageSendTaskList = messageSendTaskRepo.findBySendState(0);
                        List<TsmWorker> tsmWorkersList = tsmWorkerRepo.findByState(1);

                        if (!CollectionUtils.isEmpty(messageSendTaskList)) {
                            // 批量处理任务
                            List<MessageSendTask> tasksToProcess = new ArrayList<>();
                            for (MessageSendTask task : messageSendTaskList) {
//                        if (!inProcessTasks.contains(task.getId())) {
                                tasksToProcess.add(task);
//                        }
                            }
                            batchProcessTasks(tasksToProcess, tsmWorkersList);
                        }
                    }catch (Exception e){
                        log.error("短信任务执行异常:"+e);
                        throw new RuntimeException("短信任务执行异常{}",e);
                    }finally {
                        taskStatus.set(0);
                    }
                }
            }
            if (!isLocked) {
                log.info("尝试获取锁失败");
                return;
            }

        } catch (Exception e) {
            throw new RuntimeException("短信任务执行异常{}",e);
        } finally {
            LocalDateTime end = LocalDateTime.now();
            log.info("短信执行时间:"+Duration.between(begin, end).getSeconds());
            if (isLocked) {
                if (lock.isHeldByCurrentThread()){
                    taskStatus.set(0);
                    log.info("taskStatus结束状态"+taskStatus);
                    lock.unlock();
                }
            }

        }
    }

    private void batchProcessTasks(List<MessageSendTask> tasks, List<TsmWorker> tsmWorkersList) {
        for (MessageSendTask task : tasks) {
            try {
                doProcessTask(task, tsmWorkersList);
            } catch (Exception e) {
                log.error("处理task id:"+task.getId());
                throw new RuntimeException(e);
            }
        }
    }
    public void doProcessTask(MessageSendTask task, List<TsmWorker> tsmWorkersList) {
        try {
//            inProcessTasks.add(task.getId());
            MessageSendTask taskToUpdate = messageSendTaskRepo.getOne(task.getId());
            if (taskToUpdate.getSendState() == 0) {
                taskToUpdate.setSendState(1);
                messageSendTaskRepo.save(taskToUpdate);
                log.info("开始修改发送状态");
                List<TsmWorker> applicableWorkers = getWorkersForTaskScope(task, tsmWorkersList);
                YcqMessageTemplate ycqMessageTemplate = ycqMessageTemplateRepo.findByTemplateIdAndStatus(task.getTemplateId(), (byte) 1);
                if (ycqMessageTemplate == null || StringUtils.isEmpty(ycqMessageTemplate.getTemplateCode())) {
                    messageSendingStrategy.doSendMessage(task, applicableWorkers, null);
                } else {
                    messageSendingStrategy.doSendMessage(task, applicableWorkers, ycqMessageTemplate.getTemplateCode());
                }

            }
        }catch (Exception e){
            log.info("发送服务异常{}",e);
            throw new RuntimeException("发送服务异常{}",e);
        }finally {
            log.info("短信调度执行结束");
//            inProcessTasks.remove(task.getId());
        }
    }

    private List<TsmWorker> getWorkersForTaskScope(MessageSendTask task, List<TsmWorker> tsmWorkersList) {
        List<TsmWorker> filteredWorkers = new ArrayList<>();

        switch (task.getScopeType()) {
            //1全用户
            case 1:
                filteredWorkers = tsmWorkersList;
                break;
            case 2:
                //2角色
                List<TsmWorkerRole> roles = tsmWorKerRoleJdbcDao.findByRoleIds(Arrays.asList(task.getScopeData().split(",")));
                List<Integer> loginIds = roles.stream()
                        .map(TsmWorkerRole::getLoginId)
                        .collect(Collectors.toList());
                filteredWorkers = tsmWorkersList.stream()
                        .filter(worker -> loginIds.contains(Integer.valueOf(worker.getLoginId())))
                        .collect(Collectors.toList());
                break;
            case 3:
                //3组织
                List<String> orgIds = Arrays.asList(task.getScopeData().split(","))
                        .stream().map(String::valueOf).collect(Collectors.toList());
                filteredWorkers = tsmWorkersList.stream()
                        .filter(worker -> orgIds.contains(worker.getOrgId()))
                        .collect(Collectors.toList());
                break;
            case 4:
                //4单独用户
                List<Long> uniqueUserIds = Arrays.stream(task.getScopeData().split(","))
                        .map(Long::valueOf)
                        .collect(Collectors.toList());
                filteredWorkers = tsmWorkersList.stream()
                        .filter(worker -> uniqueUserIds.contains(Long.valueOf(worker.getLoginId())))
                        .collect(Collectors.toList());
                break;
            default:
                break;
        }
        return filteredWorkers;
    }
}
