package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.enums.AuditStatusEnums;
import com.crunii.micro.common.enums.ResultEnums;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.BeanUtil;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.*;
import com.crunii.micro.service.qzh.api.domain.dto.activity.ParticipantDetailDTO;
import com.crunii.micro.service.qzh.api.enums.ActivityButtonStateEnum;
import com.crunii.micro.service.qzh.dto.ParticipantDetailImportEL;
import com.crunii.micro.service.qzh.jdbc.ParticipantDetailJdbcDao;
import com.crunii.micro.service.qzh.jdbc.activitypublish.PublishActivityDao;
import com.crunii.micro.service.qzh.service.ParticipantDetailsService;
import com.crunii.micro.service.qzh.service.YcqDynamicFormService;
import com.crunii.micro.service.qzh.util.ExcelStuVerify;
import com.crunii.micro.service.qzh.utils.Sm2Util;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @author lmlksy
 * @date 2023/11/23 17:30
 */

@Service
public class ParticipantDetailsServiceImpl implements ParticipantDetailsService {

    @Autowired
    ParticipantDetailJdbcDao participantDetailJdbcDao;

    @Autowired
    YcqActivityPeopleAuditRepo ycqActivityPeopleAuditRepo;

    @Autowired
    TsmWorkerRepo tsmWorkerRepo;

    @Autowired
    TsmOrgRepo tsmOrgRepo;

    @Autowired
    ExcelStuVerify excelStuVerify;

    @Autowired
    PublishActivityDao publishActivityDao;

    @Autowired
    PublishActivityRepo publishActivityRepo;
    public static final int EXCEL_MAX_RAW = 500;

    @Autowired
    private ycqActivityPeopleAuditTempRepo ycqActivityPeopleAuditTempRepo;

    @Autowired
    private YcqActivityParticipationDetailsRepo ycqActivityParticipationDetailsRepo;

    @Autowired
    private YcqDynamicFormService ycqDynamicFormService;


    @Override
    public Result<PageRsp<ParticipantDetailVO>> queryList(PageReq<ParticipantDetailDTO> req) {
        PageRsp<ParticipantDetailVO> participantDetailVOPageRsp = participantDetailJdbcDao.queryList(req);
        if (!CollectionUtils.isEmpty(participantDetailVOPageRsp.getResult())){
            for (ParticipantDetailVO participantDetailVO : participantDetailVOPageRsp.getResult()) {
                YcqPublishActivity isDeleted = publishActivityRepo.findByIdAndIsDeleted(participantDetailVO.getActivityId(), 0);
                if (ObjectUtil.isEmpty(isDeleted)){
                    participantDetailVO.setDisplayButton(0);
                }else {
                    if (isDeleted.getReplyType()==2){
                        participantDetailVO.setDisplayButton(0);
                    }else if (isDeleted.getReplyType()==0){
                        YcqActivityParticipationDetails byActivityIdAndUserId = ycqActivityParticipationDetailsRepo.findByActivityIdAndUserId(participantDetailVO.getActivityId(), participantDetailVO.getUserId());
                        if (ObjectUtil.isEmpty(byActivityIdAndUserId)){
                            participantDetailVO.setDisplayButton(0);
                        }else {
                            participantDetailVO.setDisplayButton(1);
                        }
                    }else if (isDeleted.getReplyType()==1){
                        YcqActivityPeopleAudit ycqActivityPeopleAudit = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(participantDetailVO.getUserId(), participantDetailVO.getActivityId(), 0, 1);
                        if (ycqActivityPeopleAudit!=null){
                            if (ycqActivityPeopleAudit.getBusinessBelongId()!=null){
                                String valueByActiveId = ycqDynamicFormService.getDetailsValueByActiveId(ycqActivityPeopleAudit.getBusinessBelongId());
                                if (valueByActiveId==null){
                                    participantDetailVO.setDisplayButton(0);
                                }else {
                                    participantDetailVO.setDisplayButton(1);
                                }
                            }
                        }else {
                            participantDetailVO.setDisplayButton(0);
                        }
                    }
                }
                if (isDeleted.getIsApply()!=null&&isDeleted.getIsApply()==0){
                    participantDetailVO.setRegistrationTime(null);
                    participantDetailVO.setRegistrationStatus(null);
                    participantDetailVO.setRegistrationAuditTime(null);
                }
            }
        }
        return Result.ofSuccess(participantDetailVOPageRsp);
    }

    @Override
    public void exportParticipantDetailsExcel(ParticipantDetailDTO dto) {
        List<ParticipantDetailVO> result=new ArrayList<>();
        if (!CollectionUtils.isEmpty(dto.getIds())){
            List<YcqActivityPeopleAudit> byIdIn = ycqActivityPeopleAuditRepo.findByIdIn(dto.getIds());
            for (YcqActivityPeopleAudit activityPeopleAudit : byIdIn) {
                ParticipantDetailVO participantDetailVO = new ParticipantDetailVO();
                if (activityPeopleAudit.getRegistrationTime()!=null){
                    //报名时间
                    participantDetailVO.setRegistrationTime(DateUtils.formatLocalDateTime(activityPeopleAudit.getRegistrationTime()));
                }
                if (activityPeopleAudit.getCheckInTime()!=null){
                    //签到时间
                    participantDetailVO.setCheckInTime(DateUtils.formatLocalDateTime(activityPeopleAudit.getCheckInTime()));
                }
                if (activityPeopleAudit.getCheckOutTime()!=null){
                    //签退时间
                    participantDetailVO.setCheckOutTime(DateUtils.formatLocalDateTime(activityPeopleAudit.getCheckOutTime()));
                }
                participantDetailVO.setCheckInResult(activityPeopleAudit.getCheckInResult());
                participantDetailVO.setCheckOutResult(activityPeopleAudit.getCheckOutResult());
                BeanUtil.copyProperties(activityPeopleAudit,participantDetailVO);
                result.add(participantDetailVO);
            }
            result.sort(Comparator.comparing(ParticipantDetailVO::getRegistrationStatus).reversed());
        }else if (dto.getActivityId()!=null){
            List<YcqActivityPeopleAudit> byIdIn = ycqActivityPeopleAuditRepo.findByActivityIdAndIsDeleted(dto.getActivityId(),0);
            for (YcqActivityPeopleAudit activityPeopleAudit : byIdIn) {
                ParticipantDetailVO participantDetailVO = new ParticipantDetailVO();
                if (activityPeopleAudit.getRegistrationTime() != null) {
                    //报名时间
                    participantDetailVO.setRegistrationTime(DateUtils.formatLocalDateTime(activityPeopleAudit.getRegistrationTime()));
                }
                if (activityPeopleAudit.getCheckInTime() != null) {
                    //签到时间
                    participantDetailVO.setCheckInTime(DateUtils.formatLocalDateTime(activityPeopleAudit.getCheckInTime()));
                }
                if (activityPeopleAudit.getCheckOutTime() != null) {
                    //签退时间
                    participantDetailVO.setCheckOutTime(DateUtils.formatLocalDateTime(activityPeopleAudit.getCheckOutTime()));
                }
                participantDetailVO.setCheckInResult(activityPeopleAudit.getCheckInResult());
                participantDetailVO.setCheckOutResult(activityPeopleAudit.getCheckOutResult());
                BeanUtil.copyProperties(activityPeopleAudit, participantDetailVO);
                result.add(participantDetailVO);
            }
        }else if (dto.hasValues()){
            PageRsp<ParticipantDetailVO> participantDetailVOPageRsp = participantDetailJdbcDao.queryList(new PageReq<>(dto));
            result = participantDetailVOPageRsp.getResult();
            result.sort(Comparator.comparing(ParticipantDetailVO::getRegistrationStatus).reversed());
        }
        try {
            ExcelUtil.writeSingleExcel("活动参与详情表","sheet",result, ParticipantDetailVO.class);
        } catch (IOException e) {
            throw new BusinessException("导出失败");
        }
    }

    @Override
    public Result doImportDate(ParticipantDetailDTO dto) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException("获取当前登录用户失败");
        }
        Result<String> result = new Result<>();
        List<YcqActivityPeopleAuditTemp> tempData = ycqActivityPeopleAuditTempRepo.findAllByDataKey(dto.getDataKey());
        if (CollectionUtil.isEmpty(tempData)) {
            throw new BusinessException("导入失败,没有可导入数据！");
        }
        long count = tempData.stream()
                .map(YcqActivityPeopleAuditTemp::getUserName)
                .distinct()
                .count();
        boolean b = publishActivityDao.updateAuditCount(dto.getActivityId(),(int)count);
        if (!b){
            throw new BusinessException("导入不通过！活动已爆满");
        }
       List<YcqActivityPeopleAudit> peopleAudits = new ArrayList<>();
        tempData.forEach(x -> {


            YcqActivityPeopleAudit ycqActivityPeopleAudit = new YcqActivityPeopleAudit();
            //用户id 以及默认值 已经校验肯定能查询出来
            List<TsmWorker> tsmWorkers = tsmWorkerRepo.findByWorkerNameAndState(x.getUserName(), 1);
            //校验用户是否以及存在与YcqActivityPeopleAudit表
            YcqActivityPeopleAudit activityPeopleAudit = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(Long.valueOf(tsmWorkers.get(0).getLoginId()), dto.getActivityId(), 0,1);
            if (activityPeopleAudit!=null){
                activityPeopleAudit.setRegistrationStatus(x.getRegistrationStatus());
                activityPeopleAudit.setRegistrationTime(x.getRegistrationTime());
                activityPeopleAudit.setCheckInTime(x.getCheckInTime());
                activityPeopleAudit.setCheckInResult(x.getCheckInResult());
                activityPeopleAudit.setCheckOutTime(x.getCheckOutTime());
                activityPeopleAudit.setUserId(Long.valueOf(tsmWorkers.get(0).getLoginId()));
                activityPeopleAudit.setCheckOutResult(x.getCheckOutResult());
                activityPeopleAudit.setActivityId(dto.getActivityId());
                activityPeopleAudit.setIsDeleted(0);
                ycqActivityPeopleAuditRepo.save(activityPeopleAudit);
            }else {
                BeanUtil.copyProperties(x,ycqActivityPeopleAudit);
                ycqActivityPeopleAudit.setUserId(Long.valueOf(tsmWorkers.get(0).getLoginId()));
                ycqActivityPeopleAudit.setActivityId(dto.getActivityId());
                ycqActivityPeopleAudit.setPhoneNumber(tsmWorkers.get(0).getMobile());
                ycqActivityPeopleAudit.setCountFalse(0);
                ycqActivityPeopleAudit.setCountTrue(0);
                ycqActivityPeopleAudit.setAuditor(worker.getWorkerName());
                ycqActivityPeopleAudit.setAuditorId(Long.valueOf(worker.getLoginId()));

            }
            peopleAudits.add(ycqActivityPeopleAudit);
        });
        ycqActivityPeopleAuditRepo.saveAll(peopleAudits);
        // 导入成功后删除临时表数据
        ycqActivityPeopleAuditTempRepo.deleteAll(tempData);
        result.setSuccess(true);
        result.setData(String.valueOf(tempData.size()));
        result.setMsg("导入成功");
        return result;
    }

//    @Override
//    public Result doImportDate(InputStream inputStream) {
//        AnalysisEventListener<ParticipantDetailImportEL> analysisEventListener = new AnalysisEventListener<ParticipantDetailImportEL>() {
//            public final Integer MAX_SAVE_SIZE = 1000;
//            final List<YcqActivityPeopleAudit> ycqActivityPeopleAuditList = new ArrayList<>();
//            @Override
//            public void invoke(ParticipantDetailImportEL el, AnalysisContext analysisContext) {
////                String errorMsg = null;
//                String error =null;
//                try {
//                    YcqActivityPeopleAudit ycqActivityPeopleAudit = new YcqActivityPeopleAudit();
//                    //非空校验
////                    errorMsg = validateAddObj(el);
////                    if (errorMsg != null) {
////                        return;
////                    }
//                    //初始化实体对象
//                    error = initEntity(el, ycqActivityPeopleAudit);
//                    ycqActivityPeopleAuditList.add(ycqActivityPeopleAudit);
//                    if (ycqActivityPeopleAuditList.size() >= MAX_SAVE_SIZE) {
//                        ycqActivityPeopleAuditRepo.saveAll(ycqActivityPeopleAuditList);
//                        ycqActivityPeopleAuditList.clear();
//                    }
//                }  finally {
////                    if (StringUtils.isNotBlank(errorMsg)) {
////                        throw new BusinessException(
////                                "导入出错，在第" + analysisContext.readRowHolder().getRowIndex() + "行，错误信息：" + errorMsg);
////                    }
////                    if (StringUtils.isNotBlank(error)) {
////                        throw new BusinessException(
////                                "导入出错，在第" + analysisContext.readRowHolder().getRowIndex() + "行，错误信息：" +error);
////                    }
//                }
//            }
//
//            @Override
//            public void doAfterAllAnalysed(AnalysisContext analysisContext) {
//                ycqActivityPeopleAuditRepo.saveAll(ycqActivityPeopleAuditList);
//                ycqActivityPeopleAuditList.clear();
//            }
//        };
//        EasyExcel.read(inputStream, ParticipantDetailImportEL.class, analysisEventListener).doReadAll();
//        return Result.ofSuccessMsg("导入成功！");
//    }

    @Override
    public Result<ExcelDataInfo> doVerifyExcel(MultipartFile file) {
        ExcelStuData excelStuData = new ExcelStuData();
        ExcelDataInfo info = new ExcelDataInfo();
        List<ExcelStuData> excelStuDataList;
        List<MistakeDataInfo> lists = new ArrayList<>();
        List<ExcelMistakeData> dataList;
        try {
            excelStuDataList = ExcelUtil.readSingleExcel(file, excelStuData, 1);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("导入失败,请使用导入模板!");
        }
        if (CollectionUtil.isEmpty(excelStuDataList)) {
            throw new BusinessException("解析异常,查看数据是否规范");
        } else if (excelStuDataList.size() > EXCEL_MAX_RAW) {
            throw new BusinessException("超出最大导入条数");
        }
        // 开始校验参数
        for (ExcelStuData stuData : excelStuDataList) {
            dataList = excelStuVerify.data(stuData, String.valueOf(excelStuDataList.indexOf(stuData) + 2));
            if (CollectionUtil.isNotEmpty(dataList)) {
                MistakeDataInfo dataInfo = new MistakeDataInfo();
                dataInfo.setMistakeDataInfo(dataList);
                lists.add(dataInfo);
            }
        }
        // 校验通过保存临时表
        if (CollectionUtil.isEmpty(lists)) {
            List<YcqActivityPeopleAuditTemp> tempList = new ArrayList<>();
            String dataKey = "ycq" + UUIDUtil.getUUID();
            excelStuDataList.forEach(e -> {
                YcqActivityPeopleAuditTemp activityPeopleAuditTemp = new YcqActivityPeopleAuditTemp();
                BeanUtil.copyPropertiesIgnoreNull(e, activityPeopleAuditTemp);
                activityPeopleAuditTemp.setIsDeleted(0);
                //时间
                if (e.getRegistrationTime()!=null){
                    activityPeopleAuditTemp.setRegistrationTime(DateUtils.parseLocalDateTime(e.getRegistrationTime()));
                }
                if (e.getCheckinTime()!=null){
                    activityPeopleAuditTemp.setCheckInTime(DateUtils.parseLocalDateTime(e.getCheckinTime()));
                }
                if (e.getCheckoutTime()!=null){
                    activityPeopleAuditTemp.setCheckOutTime(DateUtils.parseLocalDateTime(e.getCheckoutTime()));
                }
                //报名状态
                if (e.getRegistrationStatusName() != null) {
                    Integer codeByDescription = AuditStatusEnums.getByDescription(e.getRegistrationStatusName());
                    activityPeopleAuditTemp.setRegistrationStatus(codeByDescription);
                    if (codeByDescription==0||codeByDescription==1){
                        activityPeopleAuditTemp.setRegistrationAuditTime(LocalDateTime.now());
                    }
                }else {
                    activityPeopleAuditTemp.setRegistrationStatus(1);
                    activityPeopleAuditTemp.setRegistrationAuditTime(LocalDateTime.now());
                }
                if (e.getCheckinResultName()!=null){
                    Integer codeByDescription = ResultEnums.getByDescription(e.getCheckinResultName());
                    activityPeopleAuditTemp.setCheckInResult(codeByDescription);
                }
                if (e.getCheckoutResultName()!=null){
                    Integer codeByDescription = ResultEnums.getByDescription(e.getCheckoutResultName());
                    activityPeopleAuditTemp.setCheckOutResult(codeByDescription);
                }

                activityPeopleAuditTemp.setDataKey(dataKey);
                tempList.add(activityPeopleAuditTemp);
                info.setDataKey(dataKey);
            });
            ycqActivityPeopleAuditTempRepo.saveAll(tempList);
        }
        // 返回错误信息
        info.setLists(lists);
        return Result.ofSuccess(info);
    }


//    @Override
//    public Result doVerifyExcel(InputStream inputStream) {
//        final int MAX_ROWS = 500;
//        AtomicInteger rowCount = new AtomicInteger(0);
//        AnalysisEventListener<ParticipantDetailImportEL> analysisEventListener = new AnalysisEventListener<ParticipantDetailImportEL>() {
//            @Override
//            public void invoke(ParticipantDetailImportEL el, AnalysisContext analysisContext) {
//                String errorMsg = null;
//                String error =null;
//                try {
//                    YcqActivityPeopleAudit ycqActivityPeopleAudit = new YcqActivityPeopleAudit();
//                    //非空校验
//                    errorMsg = validateAddObj(el);
//                    if (errorMsg != null) {
//                        return;
//                    }
//                    //校验实体对象
//                    error = initEntity(el, ycqActivityPeopleAudit);
//                    rowCount.incrementAndGet();
//                }  finally {
//                    if (StringUtils.isNotBlank(errorMsg)) {
//                        throw new BusinessException(
//                                "导入出错，在第" + analysisContext.readRowHolder().getRowIndex() + "行，错误信息：" + errorMsg);
//                    }
//                    if (StringUtils.isNotBlank(error)) {
//                        throw new BusinessException(
//                                "导入出错，在第" + analysisContext.readRowHolder().getRowIndex() + "行，错误信息：" +error);
//                    }
//                    if (rowCount.get() > MAX_ROWS) {
//                        throw new BusinessException(
//                                "单个数据量不允许超过500条！");
//                    }
//                }
//            }
//
//            @Override
//            public void doAfterAllAnalysed(AnalysisContext analysisContext) {
//
//            }
//        };
//        return Result.ofSuccessMsg("文件校验成功！");
//    }

    private String validateAddObj(ParticipantDetailImportEL el) {
        if (StringUtils.isBlank(el.getUserName())) {
            return "姓名不能为空";
        }
        if (el.getContactMethod() == null) {
            return "联系方式不能为空";
        }
        return null;
    }
    private String initEntity(ParticipantDetailImportEL el, YcqActivityPeopleAudit ycqActivityPeopleAudit) {
        List<TsmWorker> tsmWorkerList = tsmWorkerRepo.findByWorkerNameAndState(el.getUserName(), 1);
        if (!CollectionUtils.isEmpty(tsmWorkerList)){
            TsmWorker tsmWorker = tsmWorkerList.get(0);
            ycqActivityPeopleAudit.setUserId(Long.valueOf(tsmWorker.getLoginId()));
            ycqActivityPeopleAudit.setUserName(tsmWorker.getWorkerName());
            ycqActivityPeopleAudit.setPhoneNumber(tsmWorker.getPhone());
        }else {
            return "当前用户名称系统中不存在！";
        }
        if (el.getRegistrationStatusName() != null) {
            Integer codeByDescription = AuditStatusEnums.getByDescription(el.getRegistrationStatusName());
            ycqActivityPeopleAudit.setRegistrationStatus(codeByDescription);
        }else {
            ycqActivityPeopleAudit.setRegistrationStatus(1);
        }
        if (el.getRegistrationTime()!=null){
            ycqActivityPeopleAudit.setRegistrationTime(DateUtils.dateToLocalDateTime(el.getRegistrationTime()));
        }
        if (el.getCheckinTime()!=null){
            ycqActivityPeopleAudit.setCheckInTime(DateUtils.dateToLocalDateTime(el.getCheckinTime()));

        }
        if (el.getCheckoutTime()!=null){
            ycqActivityPeopleAudit.setCheckOutTime(DateUtils.dateToLocalDateTime(el.getCheckoutTime()));
        }
        if (el.getCheckinResultName()!=null){
            Integer codeByDescription = ResultEnums.getByDescription(el.getCheckinResultName());
            ycqActivityPeopleAudit.setCheckInResult(codeByDescription);
        }
        if (el.getCheckoutResultName()!=null){
            Integer codeByDescription = ResultEnums.getByDescription(el.getCheckoutResultName());
            ycqActivityPeopleAudit.setCheckOutResult(codeByDescription);
        }

        return null;
    }
}
