package com.crunii.micro.service.qzh.service.impl;

import static com.crunii.micro.common.constants.CommonConstants.ACTIVITY_SIGN_IDENTIFY;
import static com.crunii.micro.service.qzh.api.constants.QzhConstants.WORKER_NOT_EXISTS;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.entity.main.document.YcqQzhActivity;
import com.crunii.micro.domain.entity.main.document.YcqQzhCover;
import com.crunii.micro.domain.entity.main.dynamicform.YcqQzhDynamicForm;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.repo.main.activity.YcqQzhActivityRepo;
import com.crunii.micro.domain.repo.main.activitypublis.ActivityRegisterRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhCoverRepo;
import com.crunii.micro.domain.repo.main.dynamicform.YcqQzhDynamicFormRecordRepo;
import com.crunii.micro.domain.repo.main.dynamicform.YcqQzhDynamicFormRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.platform.components.Locker;
import com.crunii.micro.service.qzh.api.domain.dto.AppActivityDetailsVO;
import com.crunii.micro.service.qzh.api.domain.dto.DefaultTemplateParticipationVO;
import com.crunii.micro.service.qzh.api.domain.dto.ParticipationDetailsVO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.ActivityAppButtonDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.CreateActivityReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activity.OperationActivityReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.activityPublish.*;
import com.crunii.micro.service.qzh.api.domain.dto.document.CoverImageDto;
import com.crunii.micro.service.qzh.api.domain.dto.form.*;
import com.crunii.micro.service.qzh.api.enums.ActivityAuditStatusEnum;
import com.crunii.micro.service.qzh.api.enums.ActivityButtonStateEnum;
import com.crunii.micro.service.qzh.api.enums.ActivityStatusTypeEnum;
import com.crunii.micro.service.qzh.chain.activity.*;
import com.crunii.micro.service.qzh.dto.BelongBusinessVo;
import com.crunii.micro.service.qzh.dto.ButtonStateDTO;
import com.crunii.micro.service.qzh.dto.YcqActivityPeopleAuditDTO;
import com.crunii.micro.service.qzh.dto.dynamicform.DynamicFormColumnDTO;
import com.crunii.micro.service.qzh.dto.dynamicform.DynamicFormTableReqDTO;
import com.crunii.micro.service.qzh.dto.dynamicform.YcqActiveFlowNodeReqDTO;
import com.crunii.micro.service.qzh.jdbc.YcqActivityApplyRecordDao;
import com.crunii.micro.service.qzh.jdbc.YcqActivityPeopleAuditDao;
import com.crunii.micro.service.qzh.jdbc.YcqDynamicFormDao;
import com.crunii.micro.service.qzh.jdbc.YcqQzhActivityJdbcDao;
import com.crunii.micro.service.qzh.jdbc.activitypublish.ActivityRegisterDao;
import com.crunii.micro.service.qzh.jdbc.activitypublish.PublishActivityDao;
import com.crunii.micro.service.qzh.service.*;
import com.crunii.micro.service.qzh.utils.MemoryPageUtil;
import com.crunii.micro.service.qzh.utils.TimeUtils;
import com.crunii.micro.service.qzh.utils.TreeUtil;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.*;
import java.util.function.Function;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.imageio.ImageIO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RedissonClient;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

/**
 * @Author: zwx
 * @CreateTime: 2023-09-19  09:15:10
 * @Description: 活动发布
 */
@Slf4j
@Service
public class PublishActivityServiceImpl implements PublishActivityService {

    @Autowired
    PublishActivityRepo publishActivityRepo;

    @Autowired
    PublishActivityDao publishActivityDao;


    @Resource
    private YcqDynamicFormRepo formRepo;

    @Autowired
    ActivityRegisterRepo activityRegisterRepo;

    @Autowired
    ActivityRegisterDao activityRegisterDao;

    @Autowired
    RedissonClient redissonClient;

    @Resource
    private YcqDynamicRecordRepo formRecordRepo;

    @Autowired
    YcqQzhActivityService ycqQzhActivityService;

    @Resource
    YcqQzhActivityRepo ycqQzhActivityRepo;

    @Resource
    YcqQzhCoverRepo ycqQzhCoverRepo;

    @Resource
    YcqQzhActivityJdbcDao ycqQzhActivityJdbcDao;

    @Autowired
    YcqActivityParticipationDetailsRepo defaultTemplateRepo;

    @Resource
    YcqQzhDynamicFormRecordRepo ycqQzhDynamicFormRecordRepo;

    @Resource
    YcqQzhDynamicFormRepo ycqQzhDynamicFormRepo;

    @Resource
    TsmWorkerRepo tsmWorkerRepo;

    @Resource
    TsmOrgRepo tsmOrgRepo;

    @Resource
    RegistrationAuditService auditService;

    @Autowired
    private Locker locker;

    @Autowired
    private RedissonClient redisson;

    @Autowired
    private YcqActivitySignRepo signRepo;

    @Autowired
    private YcqActivityApplyFormRepo applyFormRepo;

    @Autowired
    private YcqActivityApplyRecordRepo applyRecordRepo;

    @Autowired
    private YcqActivityApplyRecordDao applyRecordDao;

    @Autowired
    YcqActivityPeopleAuditRepo ycqActivityPeopleAuditRepo;

    @Autowired
    YcqActivityPeopleAuditDao activityPeopleAuditDao;

    @Resource
    private YcqDynamicFormRepo dynamicFormRepo;

    @Resource
    private YcqDynamicRecordRepo dynamicRecordRepo;

    @Resource
    private YcqDynamicFormService dynamicFormService;

    @Resource
    private DefaultTemplateService defaultTemplateService;

    @Resource
    private TsmPropertyValueRepo tsmPropertyValueRepo;

    @Resource
    private YcqActivityZoneRepo activityZoneRepo;

    @Resource
    private YcqActivityOrgRepo activityOrgRepo;

    @Resource
    private ActivityStatusCheckerService activityStatusCheckerService;

    @Value("${upload.url}")
    private String urlPrefix;

    @Resource
    private YcqDynamicFormRepo ycqDynamicFormRepo;

    @Resource
    private YcqDynamicFormDao ycqDynamicFormDao;

    //活动报名锁参数
    private final static String ACTIVITY_APPLY = "LOCK:ACTIVITY:APPLY";
    //青字号活动参加锁参数
    private final static String ACTIVITY_QZH_JOIN = "LOCK:ACTIVITY:QZH:JOIN";
    //其他业务活动参加锁参数
    private final static String ACTIVITY_OTHER_JOIN = "LOCK:ACTIVITY:OTHER:JOIN";

    private static final Integer POSITIVE_NUMBER = 1;

    private static final String CHARSET = "utf-8";
    private static final String FORMAT_NAME = "JPG";
    // 二维码尺寸
    private static final int QRCODE_SIZE = 350;
    // 字体大小
    private static final int FONT_SIZE = 18;
    //二维码底部文字
    private static final String SIGN_IN_TIME = "签到时间：";
    private static final String SIGN_IN_CONDITION = "签到地点：";
    private static final String SIGN_OUT_TIME = "签退时间：";
    private static final String SIGN_OUT_CONDITION = "签退地点：";
    private static final String SYMBOL = "-";

    // 区域静态值code
    private static final String ZONE_CODE = "zone";
    private static final String[] MAP_KEY = {"locationNameStr", "locationNameList"};


    public static final Map<Integer, String> fixedActivityTypeMap;

    static {
        fixedActivityTypeMap = new HashMap<>();
        fixedActivityTypeMap.put(1001, "红色研学");
        fixedActivityTypeMap.put(1002, "志愿服务");
        fixedActivityTypeMap.put(1003, "科技创新");
        fixedActivityTypeMap.put(1004, "实习见习");
        fixedActivityTypeMap.put(1005, "劳动教育");
        fixedActivityTypeMap.put(1006, "其他");
    }

    @Override
    @Transactional
    public Result<Long> createActivityAdd(ActivityAddReqDTO dto) {
        // 入参数据校验
        String errMsg = addActivityValidate(dto);
        if (errMsg != null) {
            return Result.ofFail(errMsg);
        }

        // 解析所属系统
        YcqAppConfigDTO accessInfoById = publishActivityDao.findAccessInfoById(dto.getBusinessBelong());
        if (null == accessInfoById) {
            return Result.ofFail("系统信息不存在");
        }

        // 解析活动类型
        Integer activityTypeId = dto.getActivityTypeId();
        YcqPublishActivity ycqPublishActivity = new YcqPublishActivity();
        BeanUtils.copyProperties(dto, ycqPublishActivity);
        String typeName = fixedActivityTypeMap.get(activityTypeId);
        if (StringUtils.isBlank(typeName)) {
            YcqAppConfigDTO activityTypeByCodeAndId = publishActivityDao.findActivityTypeByCodeAndId(activityTypeId);
            if (null == activityTypeByCodeAndId) {
                return Result.ofFail("活动类型不存在");
            }
            ycqPublishActivity.setActivityTypeName(activityTypeByCodeAndId.getSystemName());
        } else {
            ycqPublishActivity.setActivityTypeName(typeName);
        }
        int i = publishActivityDao.countByActivityTitle(dto.getActivityTitle());
        if (i > 0) {
            return Result.ofFail("活动标题已存在");
        }
        List<ActivityAddReqDTO.ZoneAddDTO> zoneAppAddDTOList = dto.getZoneAppAddDTOList();
        if (CollectionUtils.isEmpty(zoneAppAddDTOList)){
            return Result.ofFail("所在区域必填");
        }

        //新增
        Worker currWorker = getWorker();
        int loginId = currWorker.getLoginId();
        String workerName = currWorker.getWorkerName();
        Long orgId = Long.parseLong(currWorker.getOrgId());
        String orgName = currWorker.getOrgName();
//        int loginId = 9999;
//        String workerName = "testww";
//        Long orgId = 12071L;
//        String orgName = "test";
        Date createTime = new Date();
        ycqPublishActivity.setStartTime(DateUtils.parseLocalDateTime(dto.getStartTime()));
        ycqPublishActivity.setEndTime(DateUtils.parseLocalDateTime(dto.getEndTime()));
        ycqPublishActivity.setApplyStartTime(DateUtils.parseString(dto.getApplyStartTime(), "yyyy-MM-dd HH:mm:ss"));
        ycqPublishActivity.setApplyEndTime(DateUtils.parseString(dto.getApplyEndTime(), "yyyy-MM-dd HH:mm:ss"));
        ycqPublishActivity.setContentPublishTime(LocalDateTime.now());
        ycqPublishActivity.setContentPublishOrgName(orgName);
        ycqPublishActivity.setOrgName(orgName);
        ycqPublishActivity.setIsDeleted(0);
        ycqPublishActivity.setIsTop(0);
        ycqPublishActivity.setPublisher(workerName);
        ycqPublishActivity.setCreateTime(createTime);
        ycqPublishActivity.setCreateUserId(loginId);
        ycqPublishActivity.setCreateUserName(workerName);
        ycqPublishActivity.setPublishTime(LocalDateTime.now());
        ycqPublishActivity.setOrgId(orgId);
        ycqPublishActivity.setBrowseCount(0);
        ycqPublishActivity.setIsManualShutdown(0);
        ycqPublishActivity.setBusinessBelongName(accessInfoById.getSystemName());
        ycqPublishActivity.setBusinessBelongLogoUrl(accessInfoById.getLogoUrl());
        ycqPublishActivity.setActivityTypeId(activityTypeId);
        ycqPublishActivity.setApplyerCount(0);
        ycqPublishActivity.setMaleCount(dto.getMaleCount());
        ycqPublishActivity.setFemaleCount(dto.getFemaleCount());
        ycqPublishActivity.setPutOnList(2);
        ycqPublishActivity.setPublishAuditStatus(4);
        ycqPublishActivity.setActivityStatusTypeId(1);
        // 创建活动
        YcqPublishActivity save = publishActivityRepo.saveAndFlush(ycqPublishActivity);
        Long id = save.getId();

        // 保存审核初始记录
        YcqActiveFlowNodeReqDTO ycqActiveFlowNodeReqDTO = new YcqActiveFlowNodeReqDTO();
        ycqActiveFlowNodeReqDTO.setActiveId(id);
        ycqActiveFlowNodeReqDTO.setCreateId((long)loginId);
        ycqActiveFlowNodeReqDTO.setCreateBy(workerName);
        ycqActiveFlowNodeReqDTO.setCreateTime(createTime);
        ycqActiveFlowNodeReqDTO.setIsInitial(1);
        ycqActiveFlowNodeReqDTO.setOrgId(orgId);
        ycqActiveFlowNodeReqDTO.setOrgName(orgName);
        ycqActiveFlowNodeReqDTO.setIsRelease(0);
        dynamicFormService.createNode(ycqActiveFlowNodeReqDTO);

        YcqActiveFlowNodeReqDTO ycqActiveFlowNodeReqDTO2 = new YcqActiveFlowNodeReqDTO();
        ycqActiveFlowNodeReqDTO2.setActiveId(id);
        ycqActiveFlowNodeReqDTO2.setOrgId(orgId);
        ycqActiveFlowNodeReqDTO2.setOrgName(orgName);
        ycqActiveFlowNodeReqDTO2.setIsRelease(1);
        dynamicFormService.createNode(ycqActiveFlowNodeReqDTO2);

        // 新增区域
        List<YcqActivityZone> ycqActivityZones = new ArrayList<>();
        zoneAppAddDTOList.forEach(item ->{
            YcqActivityZone ycqActivityZone = new YcqActivityZone();
            ycqActivityZone.setActivityId(id);
            ycqActivityZone.setZoneType(item.getZoneType());
            ycqActivityZone.setValueCode(item.getValueCode());
            ycqActivityZone.setValueName(item.getValueName());
            ycqActivityZone.setParentCode(item.getParentCode());
            ycqActivityZones.add(ycqActivityZone);
        });
        activityZoneRepo.saveAllAndFlush(ycqActivityZones);
        // 新增组织关联信息
        List<YcqActivityOrg> list = new ArrayList<>();
        List<ActivityOrgDTO> orgIdList = dto.getOrgIdList();
        orgIdList.forEach(item -> {
            YcqActivityOrg ycqActivityOrg = new YcqActivityOrg();
            ycqActivityOrg.setActivityId(id);
            ycqActivityOrg.setOrgId(item.getOrgId());
            ycqActivityOrg.setOrgName(item.getOrgName());
            list.add(ycqActivityOrg);
        });
        activityOrgRepo.saveAllAndFlush(list);

        // 有签到表，往签到表存数据
        Integer neededSign = dto.getNeededSign();
        if (Objects.equals(POSITIVE_NUMBER, neededSign)) {
            YcqActivitySign ycqActivitySign = new YcqActivitySign();
            ActivitySignDTO signDTO = dto.getSignDTO();
            BeanUtils.copyProperties(signDTO, ycqActivitySign);
            if (StringUtils.isNotBlank(signDTO.getSignInStartTime())) {
                ycqActivitySign.setSignInStartTime(DateUtils.parseString(signDTO.getSignInStartTime(), "yyyy-MM-dd HH:mm:ss"));
            }
            if (StringUtils.isNotBlank(signDTO.getSignInEndTime())) {
                ycqActivitySign.setSignInEndTime(DateUtils.parseString(signDTO.getSignInEndTime(), "yyyy-MM-dd HH:mm:ss"));
            }
            if (StringUtils.isNotBlank(signDTO.getSignOutStartTime())) {
                ycqActivitySign.setSignOutStartTime(DateUtils.parseString(signDTO.getSignOutStartTime(), "yyyy-MM-dd HH:mm:ss"));
            }
            if (StringUtils.isNotBlank(signDTO.getSignOutEndTime())) {
                ycqActivitySign.setSignOutEndTime(DateUtils.parseString(signDTO.getSignOutEndTime(), "yyyy-MM-dd HH:mm:ss"));
            }
            ycqActivitySign.setCreateTime(createTime);
            ycqActivitySign.setCreateId(loginId);
            ycqActivitySign.setRadius(100);
            ycqActivitySign.setActivityId(id);
            signRepo.saveAndFlush(ycqActivitySign);
        }

        // 保存封面图信息
        HashMap<Integer, List<CoverImageDto>> listHashMap = new HashMap<>(6);
        if (!CollectionUtils.isEmpty(dto.getCoverImageFile())) {
            // 4表示模板的图片
            listHashMap.put(4, dto.getCoverImageFile());
        }
        // 保存封面图信息
        if (!CollectionUtils.isEmpty(dto.getHeadPicUrlList())) {
            // 5表示app列表的图片list
            listHashMap.put(5, dto.getHeadPicUrlList());
        }
        this.saveAnnexAndCover(listHashMap, id);

        // 是否有自定义反馈，有则存动态表service
        if (Objects.equals(POSITIVE_NUMBER, dto.getReplyType())) {
            DynamicFormTableReqDTO dynamicFormTableReqDTO = new DynamicFormTableReqDTO();
            dynamicFormTableReqDTO.setFormJson(dto.getDynamicForm().getFormJson());
            dynamicFormTableReqDTO.setTableName(dto.getActivityTitle());
            dynamicFormTableReqDTO.setBusinessId(String.valueOf(id));
            dynamicFormService.doCreate(dynamicFormTableReqDTO, currWorker);
        }

        // 是否需要报名，有则存动态表service
        if (Objects.equals(POSITIVE_NUMBER, dto.getIsApply()) && dto.getApplyForm() != null) {
            DynamicFormsReqDTO applyForm = dto.getApplyForm();
            DynamicFormTableReqDTO dynamicFormTableReqDTO = new DynamicFormTableReqDTO();
            dynamicFormTableReqDTO.setFormJson(applyForm.getFormJson());
            dynamicFormTableReqDTO.setBusinessId(String.valueOf(id));
            dynamicFormTableReqDTO.setTableName(dto.getActivityTitle());
            dynamicFormService.doCreateActiveFrom(dynamicFormTableReqDTO, currWorker);
        }
        return Result.ofSuccess(id);
    }

    @Override
    public Result<PageRsp<ActivityQueryRespDTO>> activityQuery(PageReq<ActivityQueryReqDTO> dto) {
        Worker currWorker = getWorker();
        int loginId = currWorker.getLoginId();
//        int loginId = 9999;
        PageRsp<ActivityQueryRespDTO> activityQueryRespDTOPageRsp = publishActivityDao.pageList(dto, loginId);

        List<ActivityQueryRespDTO> result = activityQueryRespDTOPageRsp.getResult();
        if (!CollectionUtils.isEmpty(result)) {
            List<Long> collect = result.stream().map(ActivityQueryRespDTO::getId).collect(Collectors.toList());
            List<Long> replyIds = result.stream().filter(item -> !Objects.equals(item.getReplyType(), 2)).map(ActivityQueryRespDTO::getId).collect(Collectors.toList());
            Map<Long, YcqDynamicForm> objectObjectHashMap = new HashMap<>();
            if (!CollectionUtils.isEmpty(replyIds)){
                objectObjectHashMap = ycqDynamicFormDao.getFormByActiveIdS(replyIds).stream().collect(Collectors.toMap(YcqDynamicForm::getBusinessId, Function.identity()));
            }

            Map<Long, Integer> queryAuditCount = activityPeopleAuditDao.queryAuditCount(collect);
            List<Long> needSignIds = result.stream().filter(item -> Objects.equals(item.getNeededSign(), 1)).map(ActivityQueryRespDTO::getId).collect(Collectors.toList());
            Map<Long, YcqActivitySign> signMaps = new HashMap<>();
            if (!CollectionUtils.isEmpty(needSignIds)) {
                signMaps = signRepo.findByActivityIdIn(needSignIds).stream().collect(Collectors.toMap(YcqActivitySign::getActivityId, Function.identity()));
            }
            Map<Long, YcqActivitySign> finalSignMaps = signMaps;
            Map<Long, YcqDynamicForm> finalObjectObjectHashMap = objectObjectHashMap;
            result.forEach(item -> {
                if (Objects.equals(item.getNeededSign(), 1)) {
                    YcqActivitySign sign = finalSignMaps.get(item.getId());
                    if (null != sign) {
                        ActivitySignVO activitySignVO = new ActivitySignVO();
                        activitySignVO.setSignInWay(sign.getSignInWay());
                        activitySignVO.setSignOutWay(sign.getSignOutWay());
                        item.setSignVO(activitySignVO);
                    }
                }
                item.setStartTime(DateUtils.format(item.getLocalStartTime(), "yyyy-MM-dd HH:mm"));
                item.setEndTime(DateUtils.format(item.getLocalEndTime(), "yyyy-MM-dd HH:mm"));
                item.setPublishTime(DateUtils.format(item.getLocalPublishTime(), "yyyy-MM-dd HH:mm"));
                item.setApplyerCount(queryAuditCount.getOrDefault(item.getId(), 0));
                item.setActivityStatusTypeName(ActivityStatusTypeEnum.findByCode(item.getActivityStatusTypeId()));
                if (!Objects.equals(2, item.getReplyType())){
                    int flag = finalObjectObjectHashMap.get(item.getId()) == null ? 2 : 1;
                    item.setDynamicApplyForm(flag);
                }
            });
        }
        return Result.ofSuccess(activityQueryRespDTOPageRsp);
    }


    @InjectProperty
    @Override
    public Result<ActivityDetailRespDTO> activityDetail(Long id) {
        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(id);
        if (!publishActivity.isPresent()) {
            throw new BusinessException("所选活动不存在");
        }
        YcqPublishActivity ycqPublishActivity = publishActivity.get();
        ActivityDetailRespDTO activityQueryRespDTO = new ActivityDetailRespDTO();
        BeanUtils.copyProperties(ycqPublishActivity, activityQueryRespDTO);
        LocalDateTime contentPublishTime = ycqPublishActivity.getContentPublishTime();
        if (null != contentPublishTime) {
            activityQueryRespDTO.setContentPublishTime(DateUtils.formatLocalDateTime(contentPublishTime));
        }
        activityQueryRespDTO.setStartTime(ycqPublishActivity.getStartTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")));
        activityQueryRespDTO.setEndTime(ycqPublishActivity.getEndTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")));
        if (ycqPublishActivity.getApplyStartTime() != null) {
            activityQueryRespDTO.setApplyStartTime(DateUtils.formatDate(ycqPublishActivity.getApplyStartTime(), "yyyy-MM-dd HH:mm"));
        }
        if (ycqPublishActivity.getApplyEndTime() != null) {
            activityQueryRespDTO.setApplyEndTime(DateUtils.formatDate(ycqPublishActivity.getApplyEndTime(), "yyyy-MM-dd HH:mm"));
        }

        //返回自定义表单模板信息
        if (Objects.equals(ycqPublishActivity.getReplyType(), 1)) {
            DynamicFormVO dynamicFormVO = new DynamicFormVO();
            dynamicFormVO.setFormJson(dynamicFormService.getDetailsByActiveId(ycqPublishActivity.getId(), 2));
            activityQueryRespDTO.setDynamicForm(dynamicFormVO);
        }
        // 报名表
        if (Objects.equals(ycqPublishActivity.getIsApply(), POSITIVE_NUMBER)) {
            DynamicFormsReqDTO dynamicFormsReqDTO = new DynamicFormsReqDTO();
            dynamicFormsReqDTO.setFormJson(dynamicFormService.getDetailsByActiveId(ycqPublishActivity.getId(), 1));
            activityQueryRespDTO.setApplyForm(dynamicFormsReqDTO);
        }

        // 签到表
        if (Objects.equals(ycqPublishActivity.getNeededSign(), 1)) {
            YcqActivitySign ycqActivitySign = signRepo.findByActivityId(id);
            if (ycqActivitySign != null) {
                ActivitySignDTO activitySignDTO = new ActivitySignDTO();
                BeanUtils.copyProperties(ycqActivitySign, activitySignDTO);
                activityQueryRespDTO.setSignDTO(activitySignDTO);
                if (null != ycqActivitySign.getSignInStartTime()) {
                    activitySignDTO.setSignInStartTime(DateUtils.formatDate(ycqActivitySign.getSignInStartTime(), "yyyy-MM-dd HH:mm"));
                }
                if (null != ycqActivitySign.getSignInEndTime()) {
                    activitySignDTO.setSignInEndTime(DateUtils.formatDate(ycqActivitySign.getSignInEndTime(), "yyyy-MM-dd HH:mm"));
                }
                if (null != ycqActivitySign.getSignOutStartTime()) {
                    activitySignDTO.setSignOutStartTime(DateUtils.formatDate(ycqActivitySign.getSignOutStartTime(), "yyyy-MM-dd HH:mm"));
                }
                if (null != ycqActivitySign.getSignOutEndTime()) {
                    activitySignDTO.setSignOutEndTime(DateUtils.formatDate(ycqActivitySign.getSignOutEndTime(), "yyyy-MM-dd HH:mm"));
                }
            }
        }

        // 参与对象组织
        List<YcqActivityOrg> byActivityId1 = activityOrgRepo.findByActivityId(id);
        activityQueryRespDTO.setOrgIdList(byActivityId1.stream().map(item -> new ActivityOrgDTO(item.getOrgId(), item.getOrgName())).collect(Collectors.toList()));

        activityQueryRespDTO.setCoverImageFile(publishActivityDao.getByArticleIdAndTye(id, 4));
        //查询封面
        activityQueryRespDTO.setHeadPicUrlList(publishActivityDao.getByArticleIdAndTye(id, 5));

        // 查询区域信息
        List<YcqActivityZone> byActivityId = activityZoneRepo.findByActivityId(id);
        ArrayList<ActivityZoneTreeVO> treeList = new ArrayList<>();
        byActivityId.forEach(item -> {
            ActivityZoneTreeVO activityZoneTreeVO = new ActivityZoneTreeVO();
            BeanUtils.copyProperties(item, activityZoneTreeVO);
            treeList.add(activityZoneTreeVO);
        });
        //List<ActivityZoneTreeVO> activityZoneTreeVOS = TreeUtil.buildTree(treeList, ActivityZoneTreeVO::getValueCode, ActivityZoneTreeVO::getParentCode, ActivityZoneTreeVO::getChildren);
        activityQueryRespDTO.setZoneTree(treeList);

        return Result.ofSuccess(activityQueryRespDTO);
    }

    @Override
    public Result updateActivityState(ActivityStateEditReqDTO dto) {
        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(dto.getId());
        if (!publishActivity.isPresent()) {
            throw new BusinessException("活动不存在");
        }
        YcqPublishActivity ycqPublishActivity = publishActivity.get();
        if (dto.getActivityState() == 1 && LocalDateTime.now().isAfter(ycqPublishActivity.getEndTime())) {
            return Result.ofFail("当前活动时间已结束，无法进行启动");
        }
        if (Objects.equals(ycqPublishActivity.getActivityState(), dto.getActivityState())) {
            return Result.ofSuccess(dto.getId());
        }
        if (Objects.equals(0, dto.getActivityState()) && Objects.equals(1, ycqPublishActivity.getActivityState())) {
            ycqPublishActivity.setIsManualShutdown(1);
        }
        ycqPublishActivity.setActivityState(dto.getActivityState());
        publishActivityRepo.saveAndFlush(ycqPublishActivity);
        return Result.ofSuccess(dto.getId());
    }

    @Override
    public Result doActivityEdit(ActivityEditReqDTO dto) {
        String errMsg = editActivityValidate2(dto);
        if (errMsg != null) {
            return Result.ofFail(errMsg);
        }

        Worker currWorker = getWorker();
        int loginId = currWorker.getLoginId();
        String workerName = currWorker.getWorkerName();
        String orgId = currWorker.getOrgId();
        String orgName = currWorker.getOrgName();
//        int loginId = 9999;
//        String workerName = "currWorker";
//        String orgName = "testOrg";
//        String orgId = "1674703692521037822";
        Long id = dto.getId();
        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(id);
        if (!publishActivity.isPresent()) {
            return Result.ofFail("活动信息有误,请联系管理员");
        }

        // 查询是否有相同活动标题
        int count = publishActivityDao.countByActivityTitleAndIdNotEqual(dto.getActivityTitle(), id);
        if (count > 0){
            return Result.ofFail("活动标题已存在");
        }
        YcqPublishActivity ycqPublishActivity = publishActivity.get();
        Integer activityStatusTypeId = ycqPublishActivity.getActivityStatusTypeId();
        // 只有审核未通过的活动才允许修改
        if (!Objects.equals(activityStatusTypeId, 1) && !Objects.equals(activityStatusTypeId, 9) && !Objects.equals(activityStatusTypeId, 11)){
            return Result.ofFail("仅待审核/审核不通过/已下架的活动支持编辑");
        }
        // 类型为已下架的活动，只支持修改活动时间、报名时间、签到时间
        ActivitySignDTO signDTO = dto.getSignDTO();
        if (Objects.equals(activityStatusTypeId, 9)){
            ycqPublishActivity.setStartTime(DateUtils.parseLocalDateTime(dto.getStartTime()));
            ycqPublishActivity.setEndTime(DateUtils.parseLocalDateTime(dto.getEndTime()));
            ycqPublishActivity.setApplyStartTime(DateUtil.parse(dto.getApplyStartTime()));
            ycqPublishActivity.setApplyEndTime(DateUtil.parse(dto.getApplyEndTime()));
            YcqActivitySign sign = signRepo.findByActivityId(id);
            if (Objects.equals(POSITIVE_NUMBER, ycqPublishActivity.getNeededSign())){
                if (null != sign){
                    if (StringUtils.isNotBlank(signDTO.getSignInStartTime())){
                        sign.setSignInStartTime(DateUtils.parseString(signDTO.getSignInStartTime(), "yyyy-MM-dd HH:mm:ss"));
                    }
                    if (StringUtils.isNotBlank(signDTO.getSignInEndTime())){
                        sign.setSignInEndTime(DateUtils.parseString(signDTO.getSignInEndTime(), "yyyy-MM-dd HH:mm:ss"));
                    }
                    if (StringUtils.isNotBlank(signDTO.getSignOutStartTime())){
                        sign.setSignOutStartTime(DateUtils.parseString(signDTO.getSignOutStartTime(), "yyyy-MM-dd HH:mm:ss"));
                    }
                    if (StringUtils.isNotBlank(signDTO.getSignOutEndTime())){
                        sign.setSignOutEndTime(DateUtils.parseString(signDTO.getSignOutEndTime(), "yyyy-MM-dd HH:mm:ss"));
                    }
                    signRepo.saveAndFlush(sign);
                }
            }
            return Result.ofSuccess("编辑成功");
        }

        Integer oldApply = ycqPublishActivity.getIsApply();
        Integer newApply = dto.getIsApply();
        if (Objects.equals(1, oldApply) && Objects.equals(0,newApply)){
            dynamicFormService.deleteByIdAndType(id, 1);
        }else if (Objects.equals(1,newApply) && Objects.equals(0, oldApply)){
            // 新增
            DynamicFormsReqDTO applyForm = dto.getApplyForm();
            if (null != applyForm && StringUtils.isNotBlank(applyForm.getFormJson())){
                String formJson = applyForm.getFormJson();
                DynamicFormTableReqDTO dynamicFormTableReqDTO = new DynamicFormTableReqDTO();
                dynamicFormTableReqDTO.setFormJson(formJson);
                dynamicFormTableReqDTO.setBusinessId(String.valueOf(id));
                dynamicFormTableReqDTO.setTableName(dto.getActivityTitle());
                dynamicFormService.doCreateActiveFrom(dynamicFormTableReqDTO,currWorker);
            }
        } else if (Objects.equals(1, newApply) && Objects.equals(1, oldApply)) {
            DynamicFormsReqDTO applyForm = dto.getApplyForm();
            if (null == applyForm || StringUtils.isBlank(applyForm.getFormJson())){
                dynamicFormService.deleteByIdAndType(id, 1);
            }else {
                YcqDynamicForm byBusinessIdAndType = dynamicFormRepo.getByBusinessIdAndType(id, 1);
                if (null == byBusinessIdAndType){
                    DynamicFormTableReqDTO dynamicFormTableReqDTO = new DynamicFormTableReqDTO();
                    dynamicFormTableReqDTO.setFormJson(applyForm.getFormJson());
                    dynamicFormTableReqDTO.setBusinessId(String.valueOf(id));
                    dynamicFormTableReqDTO.setTableName(dto.getActivityTitle());
                    dynamicFormTableReqDTO.setType(1);
                    dynamicFormService.doCreateActiveFrom(dynamicFormTableReqDTO,currWorker);
                } else if (!Objects.equals(byBusinessIdAndType.getFormJson(), applyForm.getFormJson())) {
                    dynamicFormService.deleteByIdAndType(id, 1);
                    DynamicFormTableReqDTO dynamicFormTableReqDTO = new DynamicFormTableReqDTO();
                    dynamicFormTableReqDTO.setFormJson(applyForm.getFormJson());
                    dynamicFormTableReqDTO.setBusinessId(String.valueOf(id));
                    dynamicFormTableReqDTO.setTableName(dto.getActivityTitle());
                    dynamicFormTableReqDTO.setType(1);
                    dynamicFormService.doCreateActiveFrom(dynamicFormTableReqDTO,currWorker);
                }
            }
        }

        activityOrgRepo.deleteByActivityId(id);
        // 新增组织关联信息
        List<YcqActivityOrg> list = new ArrayList<>();
        List<ActivityOrgDTO> orgIdList = dto.getOrgIdList();
        orgIdList.forEach(item -> {
            YcqActivityOrg ycqActivityOrg = new YcqActivityOrg();
            ycqActivityOrg.setActivityId(id);
            ycqActivityOrg.setOrgId(item.getOrgId());
            ycqActivityOrg.setOrgName(item.getOrgName());
            list.add(ycqActivityOrg);
        });
        activityOrgRepo.saveAllAndFlush(list);

        //自定义表单模板
        Integer oldReply = ycqPublishActivity.getReplyType();
        Integer newReply = dto.getReplyType();
        if (Objects.equals(1, oldReply) && !Objects.equals(1,newReply)){
            dynamicFormService.deleteByIdAndType(id, 2);
        }else if (Objects.equals(1,newReply) && !Objects.equals(1, oldReply)){
            // 新增
            DynamicFormsReqDTO applyForm = dto.getDynamicForm();
            String formJson = applyForm.getFormJson();
            DynamicFormTableReqDTO dynamicFormTableReqDTO = new DynamicFormTableReqDTO();
            dynamicFormTableReqDTO.setFormJson(formJson);
            dynamicFormTableReqDTO.setBusinessId(String.valueOf(id));
            dynamicFormTableReqDTO.setTableName(dto.getActivityTitle());
            dynamicFormTableReqDTO.setType(2);
            dynamicFormService.doCreate(dynamicFormTableReqDTO,currWorker);
        } else if (Objects.equals(1, newReply) && Objects.equals(1, oldReply)) {
            DynamicFormsReqDTO applyForm = dto.getDynamicForm();
            YcqDynamicForm byBusinessIdAndType = dynamicFormRepo.getByBusinessIdAndType(id, 1);
            if (!Objects.equals(byBusinessIdAndType.getFormJson(), applyForm.getFormJson())){
                dynamicFormService.deleteByIdAndType(id, 2);
                DynamicFormTableReqDTO dynamicFormTableReqDTO = new DynamicFormTableReqDTO();
                dynamicFormTableReqDTO.setFormJson(applyForm.getFormJson());
                dynamicFormTableReqDTO.setBusinessId(String.valueOf(id));
                dynamicFormTableReqDTO.setTableName(dto.getActivityTitle());
                dynamicFormTableReqDTO.setType(2);
                dynamicFormService.doCreate(dynamicFormTableReqDTO,currWorker);
            }
        }


        BeanUtils.copyProperties(dto, ycqPublishActivity);
        // 解析所属系统
        YcqAppConfigDTO accessInfoById = publishActivityDao.findAccessInfoById(dto.getBusinessBelong());
        if (null == accessInfoById) {
            return Result.ofFail("系统信息不存在");
        }
        ycqPublishActivity.setBusinessBelongName(accessInfoById.getSystemName());
        // 解析活动类型
        Integer activityTypeId = dto.getActivityTypeId();
        String typeName = fixedActivityTypeMap.get(activityTypeId);
        if (StringUtils.isBlank(typeName)) {
            YcqAppConfigDTO activityTypeByCodeAndId = publishActivityDao.findActivityTypeByCodeAndId(activityTypeId);
            if (null == activityTypeByCodeAndId) {
                return Result.ofFail("活动类型不存在");
            }
            ycqPublishActivity.setActivityTypeName(activityTypeByCodeAndId.getSystemName());
        } else {
            ycqPublishActivity.setActivityTypeName(typeName);
        }
        ycqPublishActivity.setActivityTypeId(activityTypeId);
        ycqPublishActivity.setStartTime(DateUtils.parseLocalDateTime(dto.getStartTime()));
        ycqPublishActivity.setEndTime(DateUtils.parseLocalDateTime(dto.getEndTime()));
        ycqPublishActivity.setLimitCount(dto.getIsLimit() == 1 ? dto.getLimitCount() : null);
        Date date = new Date();
        ycqPublishActivity.setUpdateTime(date);
        ycqPublishActivity.setUpdateUserId(loginId);
        ycqPublishActivity.setUpdateUserName(workerName);
        ycqPublishActivity.setApplyStartTime(DateUtil.parse(dto.getApplyStartTime()));
        ycqPublishActivity.setApplyEndTime(DateUtil.parse(dto.getApplyEndTime()));
        publishActivityRepo.saveAndFlush(ycqPublishActivity);

        // 区域修改
        activityZoneRepo.deleteByActivityId(id);
        List<YcqActivityZone> ycqActivityZones = new ArrayList<>();
        dto.getZoneAppAddDTOList().forEach(item ->{
            YcqActivityZone ycqActivityZone = new YcqActivityZone();
            ycqActivityZone.setActivityId(id);
            ycqActivityZone.setZoneType(item.getZoneType());
            ycqActivityZone.setValueCode(item.getValueCode());
            ycqActivityZone.setValueName(item.getValueName());
            ycqActivityZone.setParentCode(item.getParentCode());
            ycqActivityZones.add(ycqActivityZone);
        });
        activityZoneRepo.saveAllAndFlush(ycqActivityZones);

        // 修改封面图信息
        ycqQzhCoverRepo.deleteByArticleId(id);
        HashMap<Integer, List<CoverImageDto>> listHashMap = new HashMap<>(6);
        if (!CollectionUtils.isEmpty(dto.getCoverImageFile())) {
            listHashMap.put(4, dto.getCoverImageFile());
        }

        if (!CollectionUtils.isEmpty(dto.getHeadPicUrlList())) {
            listHashMap.put(5, dto.getHeadPicUrlList());
        }
        this.saveAnnexAndCover(listHashMap, id);


        // 修改签到表
        YcqActivitySign sign = signRepo.findByActivityId(id);
        if (Objects.equals(POSITIVE_NUMBER, dto.getNeededSign())){
            if (null != sign){
                BeanUtils.copyProperties(signDTO, sign);
                sign.setRadius(100);
                signRepo.saveAndFlush(sign);
            }else {
                YcqActivitySign ycqActivitySign = new YcqActivitySign();
                BeanUtils.copyProperties(signDTO, ycqActivitySign);
                if (StringUtils.isNotBlank(signDTO.getSignInStartTime())){
                    ycqActivitySign.setSignInStartTime(DateUtils.parseString(signDTO.getSignInStartTime(), "yyyy-MM-dd HH:mm:ss"));
                }
                if (StringUtils.isNotBlank(signDTO.getSignInEndTime())){
                    ycqActivitySign.setSignInEndTime(DateUtils.parseString(signDTO.getSignInEndTime(), "yyyy-MM-dd HH:mm:ss"));
                }
                if (StringUtils.isNotBlank(signDTO.getSignOutStartTime())){
                    ycqActivitySign.setSignOutStartTime(DateUtils.parseString(signDTO.getSignOutStartTime(), "yyyy-MM-dd HH:mm:ss"));
                }
                if (StringUtils.isNotBlank(signDTO.getSignOutEndTime())){
                    ycqActivitySign.setSignOutEndTime(DateUtils.parseString(signDTO.getSignOutEndTime(), "yyyy-MM-dd HH:mm:ss"));
                }
                ycqActivitySign.setCreateTime(date);
                ycqActivitySign.setCreateId(loginId);
                ycqActivitySign.setRadius(100);
                ycqActivitySign.setActivityId(id);
                signRepo.saveAndFlush(ycqActivitySign);
            }
        }else {
            if (null != sign){
                signRepo.deleteById(sign.getId());
            }
        }

        Integer publishAuditStatus = ycqPublishActivity.getPublishAuditStatus();
        if (Objects.equals(publishAuditStatus, 3)){
            // 从初始节点开始审核
            YcqActiveFlowNodeReqDTO initNode = publishActivityDao.getInitNode(id);
            YcqActiveFlowNodeReqDTO ycqActiveFlowNodeReqDTO2 = new YcqActiveFlowNodeReqDTO();
            ycqActiveFlowNodeReqDTO2.setActiveId(id);
            ycqActiveFlowNodeReqDTO2.setOrgId(initNode.getOrgId());
            ycqActiveFlowNodeReqDTO2.setOrgName(initNode.getOrgName());
            ycqActiveFlowNodeReqDTO2.setIsRelease(1);
            ycqActiveFlowNodeReqDTO2.setIsUpdate(1);
            dynamicFormService.createNode(ycqActiveFlowNodeReqDTO2);
            publishActivityDao.reAudit(id);
        }

        return Result.ofSuccess("编辑成功");
    }

    @Override
    public Boolean updatePublishStatus(ActivityPublishStatusEditDTO dto){
        Long id = dto.getId();
        Integer putOnList = dto.getPutOnList();
        YcqPublishActivity one = publishActivityRepo.getOne(id);
        Integer activityStatusTypeId = one.getActivityStatusTypeId();
        if (Objects.equals(putOnList, 2)){
            // 3未开始 4报名中 5进行中 6签到中 7签退中 8已结束这几个状态支持下架操作
            Integer[] states = {3,4,5,6,7,8};
            List<Integer> ints = Arrays.asList(states);
            if (!ints.contains(activityStatusTypeId)){
                throw new BusinessException("当前活动状态不支持下架");
            }
            one.setActivityStatusTypeId(9);
            one.setPutOnList(putOnList);
        }else {
            if (!Objects.equals(9, activityStatusTypeId) && Objects.equals(3, putOnList)){
                throw new BusinessException("当前活动未下架");
            }
            if (Objects.equals(1, putOnList)){
                // 只有审核通过，可以发布
                Integer publishAuditStatus = one.getPublishAuditStatus();
                if (!Objects.equals(1, publishAuditStatus)){
                    throw new BusinessException("未审核通过，无法发布");
                }

            }
            one.setActivityStatusTypeId(activityStatusCheckerService.getActivityStatusTypeId(id));
            one.setPutOnList(1);
        }
        publishActivityRepo.saveAndFlush(one);
        return true;
    }

    @Override
    public PageRsp<ActivityBaseVO> queryActivityBaseInfo(PageReq<ActivityBaseDTO> req){
        Worker currWorker = getWorker();
        int loginId = currWorker.getLoginId();
//        Integer loginId = 9999;
        return publishActivityDao.queryActivityBaseInfo(req, loginId);
    }

    @Override
    public Result<PageRsp<ActivityQueryAuditRespDTO>> pageAuditActivity(PageReq<ActivityQueryAuditDTO> dto) {
        Worker currWorker = getWorker();
        Long orgId = Long.parseLong(currWorker.getOrgId());
//        Long orgId = 1674703692521037822L;
        PageRsp<ActivityQueryAuditRespDTO> activityQueryRespDTOPageRsp = publishActivityDao.pageAuditList(dto, orgId);

        List<ActivityQueryAuditRespDTO> result = activityQueryRespDTOPageRsp.getResult();
        if (!CollectionUtils.isEmpty(result)) {
            List<Long> collect = result.stream().map(ActivityQueryAuditRespDTO::getId).collect(Collectors.toList());
            // 根据组织id查询是否还有上级组织
            Set<Long> collect2 = activityPeopleAuditDao.queryTopOrgList().stream().map(YcqActivityOrg::getOrgId).collect(Collectors.toSet());

            Map<Long, Integer> queryAuditCount = activityPeopleAuditDao.queryAuditCount(collect);
            List<Long> needSignIds = result.stream().filter(item -> Objects.equals(item.getNeededSign(), 1)).map(ActivityQueryAuditRespDTO::getId).collect(Collectors.toList());
            Map<Long, YcqActivitySign> signMaps = new HashMap<>();
            if (!CollectionUtils.isEmpty(needSignIds)) {
                signMaps = signRepo.findByActivityIdIn(needSignIds).stream().collect(Collectors.toMap(YcqActivitySign::getActivityId, Function.identity()));
            }
            Map<Long, YcqActivitySign> finalSignMaps = signMaps;
            result.forEach(item -> {
                if (Objects.equals(item.getNeededSign(), 1)) {
                    YcqActivitySign sign = finalSignMaps.get(item.getId());
                    if (null != sign) {
                        ActivitySignVO activitySignVO = new ActivitySignVO();
                        activitySignVO.setSignInWay(sign.getSignInWay());
                        activitySignVO.setSignOutWay(sign.getSignOutWay());
                        item.setSignVO(activitySignVO);
                    }
                }
                item.setStartTime(DateUtils.format(item.getLocalStartTime(), "yyyy-MM-dd HH:mm"));
                item.setEndTime(DateUtils.format(item.getLocalEndTime(), "yyyy-MM-dd HH:mm"));
                item.setPublishTime(DateUtils.format(item.getLocalPublishTime(), "yyyy-MM-dd HH:mm"));

                item.setApplyerCount(queryAuditCount.getOrDefault(item.getId(), 0));
                item.setAuditStatusName(ActivityAuditStatusEnum.findByCode(item.getAuditStatus()));
                boolean contains = collect2.contains(orgId);
                if (contains){
                    item.setIsTop(1);
                }
            });
        }
        return Result.ofSuccess(activityQueryRespDTOPageRsp);
    }

    @Override
    public Result<PageRsp<ActivityAppQueryRespDTO>> activityQueryForApp(PageReq<ActivityAppQueryReqDTO> dto) {
        Worker currWorker = getWorker();
        Long orgId = Long.parseLong(currWorker.getOrgId());
//        Long orgId = 12071L;
        PageRsp<YcqPublishActivity> pageListForApp = publishActivityDao.pageListForApp(dto, orgId);
        List<YcqPublishActivity> result = pageListForApp.getResult();
        PageRsp<ActivityAppQueryRespDTO> res = new PageRsp<>();
        res.setPageSize(pageListForApp.getPageSize());
        res.setPageNo(pageListForApp.getPageNo());
        res.setTotalNum(pageListForApp.getTotalNum());
        if (CollectionUtils.isEmpty(result)){
            res.setResult(Collections.emptyList());
            res.next();
            return Result.ofSuccess(res);
        }
        // 用户信息
        List<Long> ids = result.stream().map(YcqPublishActivity::getId).collect(Collectors.toList());
//        Map<Long, List<YcqActivityPeopleAudit>> auditMap = new HashMap<>();
//        if (!CollectionUtils.isEmpty(ids)) {
//            List<YcqActivityPeopleAudit> list = activityPeopleAuditDao.findByUserIdAndActivityIdIn(ids, (long) loginId);
//            auditMap = list.stream().collect(Collectors.groupingBy(YcqActivityPeopleAudit::getActivityId));
//        }

        // 签到配置信息
        Map<Long, YcqActivitySign> signMaps = new HashMap<>();
        List<Long> needSignIds = result.stream().filter(item -> Objects.equals(item.getNeededSign(), 1)).map(YcqPublishActivity::getId).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(needSignIds)) {
            signMaps = signRepo.findByActivityIdIn(needSignIds).stream().collect(Collectors.toMap(YcqActivitySign::getActivityId, Function.identity()));
        }

        // 查询默认反馈
//        List<Long> defaultIds = result.stream().filter(item -> Objects.equals(item.getReplyType(), 0)).map(YcqPublishActivity::getId).collect(Collectors.toList());
//        Map<Long, List<YcqActivityParticipationDetails>> defaultTemplateMap = new HashMap<>();
//        if (!CollectionUtils.isEmpty(defaultIds)) {
//            defaultTemplateMap = defaultTemplateRepo.findByActivityIdInAndUserId(defaultIds, (long) loginId).stream().collect(Collectors.groupingBy(YcqActivityParticipationDetails::getActivityId));
//        }
//        List<Long> dynamicIds = result.stream().filter(item -> Objects.equals(item.getReplyType(), POSITIVE_NUMBER)).map(YcqPublishActivity::getId).collect(Collectors.toList());
//        // 查询自定义反馈
//        List<Long> dynamicFeedbackIds = new ArrayList<>();
//        if (!CollectionUtils.isEmpty(dynamicIds)) {
//            dynamicFeedbackIds = dynamicFormService.getRecordByUsrAndActiveIds(loginId, dynamicIds, 1).stream().map(item -> {
//                return (long) item.get("activeId");
//            }).collect(Collectors.toList());
//
//        }

        // 已审核通过人数
        Map<Long, Integer> queryAuditCountMap = new HashMap<>();
        List<Long> needAuditIds = result.stream().map(YcqPublishActivity::getId).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(needAuditIds)) {
            queryAuditCountMap = activityPeopleAuditDao.queryAuditCount(needAuditIds);
        }

        Map<Long, List<YcqQzhCover>> headLineMap = ycqQzhCoverRepo.findAllByArticleIdInAndType(ids, 5).stream().collect(Collectors.groupingBy(YcqQzhCover::getArticleId));

//        Map<Long, List<YcqActivityPeopleAudit>> finalAuditMap = auditMap;
//        Map<Long, List<YcqActivityParticipationDetails>> defaultFinalTemplateMap = defaultTemplateMap;
//        List<Long> dynamicFinalTemplateMap = dynamicFeedbackIds;
        Map<Long, Integer> finalQueryAuditCountMap = queryAuditCountMap;
        Map<Long, YcqActivitySign> finalSignMap = signMaps;
        List<ActivityAppQueryRespDTO> respDTOS = new ArrayList<>();
        result.forEach(item -> {
            Long id = item.getId();
            ActivityAppQueryRespDTO respDTO = new ActivityAppQueryRespDTO();
            BeanUtils.copyProperties(item, respDTO);
            LocalDateTime now = LocalDateTime.now();
            if (StringUtils.isNotBlank(item.getBusinessBelongLogoUrl())) {
                respDTO.setBusinessBelongLogoUrl(urlPrefix + item.getBusinessBelongLogoUrl());
            }
            List<YcqQzhCover> ycqQzhCovers = headLineMap.get(id);
            if (!CollectionUtils.isEmpty(ycqQzhCovers)) {
                YcqQzhCover ycqQzhCover = ycqQzhCovers.get(0);
                CoverImageDto coverImageDto = new CoverImageDto();
                BeanUtils.copyProperties(ycqQzhCover, coverImageDto);
                respDTO.setHeadPicUrl(coverImageDto);
            }
//            ActivityAppButtonDTO activityAppButtonDTO = this.setButtonState(item, new ActivityAppButtonDTO(), finalAuditMap.getOrDefault(id, Collections.emptyList()), finalQueryAuditCountMap, finalSignMap, defaultFinalTemplateMap.getOrDefault(id, Collections.emptyList()), dynamicFinalTemplateMap, loginId, false);
//            respDTO.setButtonState(activityAppButtonDTO.getButtonState());
//            respDTO.setButtonGrey(activityAppButtonDTO.getButtonGrey());
            respDTO.setAuditCount(finalQueryAuditCountMap.getOrDefault(id, 0));
            Date applyStartTime = item.getApplyStartTime();
            if (null != applyStartTime) {
                respDTO.setApplyStartTime(DateUtils.formatDate(applyStartTime, "yyyy-MM-dd HH:mm:ss"));
            }
            Date applyEndTime = item.getApplyEndTime();
            if (null != applyEndTime) {
                respDTO.setApplyEndTime(DateUtils.formatDate(applyEndTime, "yyyy-MM-dd HH:mm:ss"));
            }
            // 活动是否开始
            respDTO.setActivityStatusTypeName(ActivityStatusTypeEnum.findByCode(item.getActivityStatusTypeId()));
            if (Objects.equals(item.getNeededSign(), 1)) {
                YcqActivitySign sign = finalSignMap.get(id);
                if (sign != null) {
                    ActivitySignVO activitySignVO = new ActivitySignVO();
                    BeanUtils.copyProperties(sign, activitySignVO);
                    Date signInStartTime = sign.getSignInStartTime();
                    activitySignVO.setSignInStartTime(DateUtils.formatDate(signInStartTime, "yyyy-MM-dd HH:mm:ss"));
                    activitySignVO.setSignInEndTime(DateUtils.formatDate(sign.getSignInEndTime(), "yyyy-MM-dd HH:mm:ss"));
                    activitySignVO.setSignOutStartTime(DateUtils.formatDate(sign.getSignOutStartTime(), "yyyy-MM-dd HH:mm:ss"));
                    activitySignVO.setSignOutEndTime(DateUtils.formatDate(sign.getSignOutEndTime(), "yyyy-MM-dd HH:mm:ss"));
                    respDTO.setSignVO(activitySignVO);
                }
            }
            // 活动时间格式调整
            respDTO.setStartTime(DateUtils.formatLocalDateTime(item.getStartTime(), "yyyy-MM-dd HH:mm"));
            respDTO.setEndTime(DateUtils.formatLocalDateTime(item.getEndTime(), "yyyy-MM-dd HH:mm"));

            // 区域静态值转换
//            Map<String, Object> map = doZoneCodeToName(item.getLocationCode(),
//                    ZONE_CODE, ",");
//            respDTO.setLocationName(map.get(MAP_KEY[0]).toString());
//            respDTO.setLocationNameList((List<String>) map.get(MAP_KEY[1]));
            respDTO.setActivityStatusTypeName(ActivityStatusTypeEnum.findByCode(item.getActivityStatusTypeId()));
            respDTOS.add(respDTO);
        });
        res.setResult(respDTOS);
        res.next();
        return Result.ofSuccess(res);
    }


    @Override
    public Result<PageRsp<ActivityAppQueryRespDTO>> activityQueryForAppQzh(PageReq<ActivityAppQueryReqDTO> dto) {
        Worker currWorker = getWorker();
        int loginId = currWorker.getLoginId();
//        int loginId = 9999;
        PageRsp<ActivityAppQueryRespDTO> res = new PageRsp<>();
        res.setPageSize(dto.getPageSize());
        res.setPageNo(dto.getPageNo());
        // 查询当前用户报名的活动
        List<YcqActivityPeopleAudit> byUserIdAndIsDeleted = ycqActivityPeopleAuditRepo.findByUserIdAndIsDeleted((long) loginId, 0);
        if (CollectionUtils.isEmpty(byUserIdAndIsDeleted)) {
            res.setTotalNum(0);
        } else {
            Set<Long> activityIds = byUserIdAndIsDeleted.stream().map(YcqActivityPeopleAudit::getActivityId).collect(Collectors.toSet());
            PageRsp<YcqPublishActivity> pageListForApp = publishActivityDao.pageListForQZH(dto, activityIds);
            List<YcqPublishActivity> result = pageListForApp.getResult();
            res.setTotalNum(pageListForApp.getTotalNum());

            if (CollectionUtils.isEmpty(result)){
                res.setResult(Collections.emptyList());
                res.next();
                return Result.ofSuccess(res);
            }

            // 用户信息
            List<Long> ids = result.stream().map(YcqPublishActivity::getId).collect(Collectors.toList());
//            Map<Long, List<YcqActivityPeopleAudit>> auditMap = new HashMap<>();
//            if (!CollectionUtils.isEmpty(ids)) {
//                List<YcqActivityPeopleAudit> list = activityPeopleAuditDao.findByUserIdAndActivityIdIn(ids, (long) loginId);
//                auditMap = list.stream().collect(Collectors.groupingBy(YcqActivityPeopleAudit::getActivityId));
//            }

            // 签到配置信息
            Map<Long, YcqActivitySign> signMaps = new HashMap<>();
            List<Long> needSignIds = result.stream().filter(item -> Objects.equals(item.getNeededSign(), 1)).map(YcqPublishActivity::getId).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(needSignIds)) {
                signMaps = signRepo.findByActivityIdIn(needSignIds).stream().collect(Collectors.toMap(YcqActivitySign::getActivityId, Function.identity()));
            }

//            // 查询默认反馈
//            List<Long> defaultIds = result.stream().filter(item -> Objects.equals(item.getReplyType(), 0)).map(YcqPublishActivity::getId).collect(Collectors.toList());
//            Map<Long, List<YcqActivityParticipationDetails>> defaultTemplateMap = new HashMap<>();
//            if (!CollectionUtils.isEmpty(defaultIds)) {
//                defaultTemplateMap = defaultTemplateRepo.findByActivityIdInAndUserId(defaultIds, (long) loginId).stream().collect(Collectors.groupingBy(YcqActivityParticipationDetails::getActivityId));
//            }
//            List<Long> dynamicIds = result.stream().filter(item -> Objects.equals(item.getReplyType(), POSITIVE_NUMBER)).map(YcqPublishActivity::getId).collect(Collectors.toList());
//            // 查询自定义反馈
//            List<Long> dynamicFeedbackIds = new ArrayList<>();
//            if (!CollectionUtils.isEmpty(dynamicIds)) {
//                dynamicFeedbackIds = dynamicFormService.getRecordByUsrAndActiveIds(loginId, dynamicIds, 1).stream().map(item -> {
//                    return (long) item.get("activeId");
//                }).collect(Collectors.toList());
//
//            }

            Map<Long, List<YcqQzhCover>> headLineMap = ycqQzhCoverRepo.findAllByArticleIdInAndType(ids, 5).stream().collect(Collectors.groupingBy(YcqQzhCover::getArticleId));


            // 已审核通过人数
            Map<Long, Integer> queryAuditCountMap = new HashMap<>();
            List<Long> needAuditIds = result.stream().map(YcqPublishActivity::getId).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(needAuditIds)) {
                queryAuditCountMap = activityPeopleAuditDao.queryAuditCount(needAuditIds);
            }

//            Map<Long, List<YcqActivityPeopleAudit>> finalAuditMap = auditMap;
//            Map<Long, List<YcqActivityParticipationDetails>> defaultFinalTemplateMap = defaultTemplateMap;
//            List<Long> dynamicFinalTemplateMap = dynamicFeedbackIds;
            Map<Long, Integer> finalQueryAuditCountMap = queryAuditCountMap;
            Map<Long, YcqActivitySign> finalSignMap = signMaps;
            List<ActivityAppQueryRespDTO> respDTOS = new ArrayList<>();
            result.forEach(item -> {
                Long id = item.getId();
                ActivityAppQueryRespDTO respDTO = new ActivityAppQueryRespDTO();
                BeanUtils.copyProperties(item, respDTO);
                LocalDateTime now = LocalDateTime.now();
                if (StringUtils.isNotBlank(item.getBusinessBelongLogoUrl())) {
                    respDTO.setBusinessBelongLogoUrl(urlPrefix + item.getBusinessBelongLogoUrl());
                }
                List<YcqQzhCover> ycqQzhCovers = headLineMap.get(id);
                if (!CollectionUtils.isEmpty(ycqQzhCovers)) {
                    YcqQzhCover ycqQzhCover = ycqQzhCovers.get(0);
                    CoverImageDto coverImageDto = new CoverImageDto();
                    BeanUtils.copyProperties(ycqQzhCover, coverImageDto);
                    respDTO.setHeadPicUrl(coverImageDto);
                }
//                ActivityAppButtonDTO activityAppButtonDTO = this.setButtonState(item, new ActivityAppButtonDTO(), finalAuditMap.getOrDefault(id, Collections.emptyList()), finalQueryAuditCountMap, finalSignMap, defaultFinalTemplateMap.getOrDefault(id, Collections.emptyList()), dynamicFinalTemplateMap, loginId, false);
//                respDTO.setButtonState(activityAppButtonDTO.getButtonState());
//                respDTO.setButtonGrey(activityAppButtonDTO.getButtonGrey());
                respDTO.setAuditCount(finalQueryAuditCountMap.getOrDefault(id, 0));

                Date applyStartTime = item.getApplyStartTime();
                if (null != applyStartTime) {
                    respDTO.setApplyStartTime(DateUtils.formatDate(applyStartTime, "yyyy-MM-dd HH:mm:ss"));
                }
                Date applyEndTime = item.getApplyEndTime();
                if (null != applyEndTime) {
                    respDTO.setApplyEndTime(DateUtils.formatDate(applyEndTime, "yyyy-MM-dd HH:mm:ss"));
                }

                // 活动是否开始
//                if (now.isBefore(item.getStartTime()) && Objects.equals(item.getActivityState(), 0)) {
//                    respDTO.setActivityStateApp(1);
//                } else if (now.isAfter(item.getEndTime()) && Objects.equals(item.getActivityState(), 0)) {
//                    respDTO.setActivityStateApp(3);
//                } else {
//                    respDTO.setActivityStateApp(2);
//                }
                if (Objects.equals(item.getNeededSign(), 1)) {
                    YcqActivitySign sign = finalSignMap.get(id);
                    if (sign != null) {
                        ActivitySignVO activitySignVO = new ActivitySignVO();
                        BeanUtils.copyProperties(sign, activitySignVO);
                        Date signInStartTime = sign.getSignInStartTime();
                        activitySignVO.setSignInStartTime(DateUtils.formatDate(signInStartTime, "yyyy-MM-dd HH:mm:ss"));
                        activitySignVO.setSignInEndTime(DateUtils.formatDate(sign.getSignInEndTime(), "yyyy-MM-dd HH:mm:ss"));
                        activitySignVO.setSignOutStartTime(DateUtils.formatDate(sign.getSignOutStartTime(), "yyyy-MM-dd HH:mm:ss"));
                        activitySignVO.setSignOutEndTime(DateUtils.formatDate(sign.getSignOutEndTime(), "yyyy-MM-dd HH:mm:ss"));
                        respDTO.setSignVO(activitySignVO);
                    }
                }
                // 活动时间格式调整
                respDTO.setStartTime(DateUtils.formatLocalDateTime(item.getStartTime(), "yyyy-MM-dd HH:mm"));
                respDTO.setEndTime(DateUtils.formatLocalDateTime(item.getEndTime(), "yyyy-MM-dd HH:mm"));
                respDTO.setActivityStatusTypeName(ActivityStatusTypeEnum.findByCode(item.getActivityStatusTypeId()));
                // 区域静态值转换
//                Map<String, Object> map = doZoneCodeToName(item.getLocationCode(),
//                        ZONE_CODE, ",");
//                respDTO.setLocationName(map.get(MAP_KEY[0]).toString());
//                respDTO.setLocationNameList((List<String>) map.get(MAP_KEY[1]));

                respDTOS.add(respDTO);
            });
            res.setTotalNum(pageListForApp.getTotalNum());
            res.setResult(respDTOS);
        }
        res.next();
        return Result.ofSuccess(res);
    }

    @Override
    public Result<PageRsp<ActivityAppQueryRespDTO>> activityQueryForAppWithoutToken(PageReq<ActivityAppQueryReqDTO> dto) {
        PageRsp<YcqPublishActivity> pageListForApp = publishActivityDao.pageListForAppWithoutToken(dto);
        List<YcqPublishActivity> result = pageListForApp.getResult();
        PageRsp<ActivityAppQueryRespDTO> res = new PageRsp<>();
        res.setPageSize(pageListForApp.getPageSize());
        res.setPageNo(pageListForApp.getPageNo());
        res.setTotalNum(pageListForApp.getTotalNum());
        if (CollectionUtils.isEmpty(result)){
            res.setResult(Collections.emptyList());
            res.next();
            return Result.ofSuccess(res);
        }

        List<Long> collect = result.stream().map(YcqPublishActivity::getId).collect(Collectors.toList());
        Map<Long, List<YcqQzhCover>> headLineMap = ycqQzhCoverRepo.findAllByArticleIdInAndType(collect, 5).stream().collect(Collectors.groupingBy(YcqQzhCover::getArticleId));


        // 签到/签退配置信息
        Map<Long, YcqActivitySign> signMaps = new HashMap<>();
        List<Long> needSignIds = result.stream().filter(item -> Objects.equals(item.getNeededSign(), 1)).map(YcqPublishActivity::getId).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(needSignIds)) {
            signMaps = signRepo.findByActivityIdIn(needSignIds).stream().collect(Collectors.toMap(YcqActivitySign::getActivityId, Function.identity()));
        }

        // 已审核通过人数
        Map<Long, Integer> queryAuditCountMap = new HashMap<>();
        List<Long> needAuditIds = result.stream().map(YcqPublishActivity::getId).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(needAuditIds)) {
            queryAuditCountMap = activityPeopleAuditDao.queryAuditCount(needAuditIds);
        }

        Map<Long, Integer> finalQueryAuditCountMap = queryAuditCountMap;
        Map<Long, YcqActivitySign> finalSignMap = signMaps;
        List<ActivityAppQueryRespDTO> respDTOS = new ArrayList<>();
        result.forEach(item -> {
            Long id = item.getId();
            ActivityAppQueryRespDTO respDTO = new ActivityAppQueryRespDTO();
            BeanUtils.copyProperties(item, respDTO);
            LocalDateTime now = LocalDateTime.now();
            if (StringUtils.isNotBlank(item.getBusinessBelongLogoUrl())) {
                respDTO.setBusinessBelongLogoUrl(urlPrefix + item.getBusinessBelongLogoUrl());
            }
            ActivityAppButtonDTO activityAppButtonDTO = this.getButtonInfo(now, item, finalSignMap.get(id), finalQueryAuditCountMap.getOrDefault(id, 0));
            respDTO.setButtonState(activityAppButtonDTO.getButtonState());
            respDTO.setButtonGrey(activityAppButtonDTO.getButtonGrey());

            List<YcqQzhCover> ycqQzhCovers = headLineMap.get(id);
            if (!CollectionUtils.isEmpty(ycqQzhCovers)) {
                YcqQzhCover ycqQzhCover = ycqQzhCovers.get(0);
                CoverImageDto coverImageDto = new CoverImageDto();
                BeanUtils.copyProperties(ycqQzhCover, coverImageDto);
                respDTO.setHeadPicUrl(coverImageDto);
            }

            // 活动是否开始
//            if (now.isBefore(item.getStartTime()) && Objects.equals(item.getActivityState(), 0)) {
//                respDTO.setActivityStateApp(1);
//            } else if (now.isAfter(item.getEndTime()) && Objects.equals(item.getActivityState(), 0)) {
//                respDTO.setActivityStateApp(3);
//            } else {
//                respDTO.setActivityStateApp(2);
//            }
            if (Objects.equals(item.getNeededSign(), 1)) {
                YcqActivitySign sign = finalSignMap.get(id);
                if (sign != null) {
                    ActivitySignVO activitySignVO = new ActivitySignVO();
                    BeanUtils.copyProperties(sign, activitySignVO);
                    Date signInStartTime = sign.getSignInStartTime();
                    activitySignVO.setSignInStartTime(DateUtils.formatDate(signInStartTime, "yyyy-MM-dd HH:mm:ss"));
                    activitySignVO.setSignInEndTime(DateUtils.formatDate(sign.getSignInEndTime(), "yyyy-MM-dd HH:mm:ss"));
                    activitySignVO.setSignOutStartTime(DateUtils.formatDate(sign.getSignOutStartTime(), "yyyy-MM-dd HH:mm:ss"));
                    activitySignVO.setSignOutEndTime(DateUtils.formatDate(sign.getSignOutEndTime(), "yyyy-MM-dd HH:mm:ss"));
                    respDTO.setSignVO(activitySignVO);
                }
            }
            // 活动时间格式调整
            respDTO.setStartTime(DateUtils.formatLocalDateTime(item.getStartTime(), "yyyy-MM-dd HH:mm"));
            respDTO.setEndTime(DateUtils.formatLocalDateTime(item.getEndTime(), "yyyy-MM-dd HH:mm"));
            respDTO.setActivityStatusTypeName(ActivityStatusTypeEnum.findByCode(item.getActivityStatusTypeId()));
            // 区域静态值转换
//            Map<String, Object> map = doZoneCodeToName(item.getLocationCode(),
//                ZONE_CODE, ",");
//            respDTO.setLocationName(map.get(MAP_KEY[0]).toString());
//            respDTO.setLocationNameList((List<String>)map.get(MAP_KEY[1]));
            respDTOS.add(respDTO);
        });
        res.setResult(respDTOS);
        res.next();
        return Result.ofSuccess(res);
    }

    /**
     * 查询所属业务
     */
    @Override
    public Result<List<BelongBusinessVo>> getBelongBUsiness() {
        return Result.ofSuccess(publishActivityDao.getBelongBUsiness());
    }

    private ActivityAppButtonDTO getButtonInfo(LocalDateTime now, YcqPublishActivity item, YcqActivitySign sign, int auditCount) {
        ActivityAppButtonDTO buttonDTO = new ActivityAppButtonDTO();
        LocalDateTime startTime = item.getStartTime();
        LocalDateTime endTime = item.getEndTime();
        if (now.isAfter(endTime)) {
            buttonDTO.setButtonState(ActivityButtonStateEnum.HIDE.getCode());
            return buttonDTO;
        }
        Integer isApply = item.getIsApply();
        if (Objects.equals(isApply, 1)) {
            if (now.isAfter(DateUtils.dateToLocalDateTime(item.getApplyEndTime()))) {
                buttonDTO.setButtonState(ActivityButtonStateEnum.HIDE.getCode());
                return buttonDTO;
            }
            if (now.isBefore(DateUtils.dateToLocalDateTime(item.getApplyStartTime()))) {
                buttonDTO.setButtonGrey(1);
            }
            // 需要报名
            Integer isLimit = item.getIsLimit();
            if (Objects.equals(1, isLimit)) {      // 是否限制报名人数
                // 限制报名人数
                Integer limitCount = item.getLimitCount();
                if (auditCount >= limitCount) {
                    buttonDTO.setButtonState(ActivityButtonStateEnum.FULL_REGISTRATION.getCode());
                } else {
                    buttonDTO.setButtonState(ActivityButtonStateEnum.REGISTER.getCode());
                }
            } else {
                buttonDTO.setButtonState(ActivityButtonStateEnum.REGISTER.getCode());
            }
            return buttonDTO;
        } else {
            // 不需要报名
            // 查看下一个状态是什么
            Integer neededSign = item.getNeededSign();
            if (Objects.equals(neededSign, 1)) {
                Integer signInWay = sign.getSignInWay();
                if (null != signInWay) {
                    LocalDateTime signInStartTime = DateUtils.dateToLocalDateTime(sign.getSignInStartTime());
                    if (now.isBefore(signInStartTime)) {
                        buttonDTO.setButtonGrey(1);
                    }
                    LocalDateTime signInEndTime = DateUtils.dateToLocalDateTime(sign.getSignInEndTime());
                    if (now.isBefore(signInEndTime)) {
                        buttonDTO.setButtonState(ActivityButtonStateEnum.SIGN_IN_NOW.getCode());
                    }
                }
                Integer signOutWay = sign.getSignOutWay();
                if (null != signOutWay) {
                    if (buttonDTO.getButtonState() == null) {
                        LocalDateTime signOutStartTime = DateUtils.dateToLocalDateTime(sign.getSignOutStartTime());
                        if (now.isBefore(signOutStartTime)) {
                            buttonDTO.setButtonGrey(1);
                        }
                        LocalDateTime signOutEndTime = DateUtils.dateToLocalDateTime(sign.getSignOutEndTime());
                        if (now.isBefore(signOutEndTime)) {
                            buttonDTO.setButtonState(ActivityButtonStateEnum.SIGN_OUT_NOW.getCode());
                        }
                    }
                }
                if (null == buttonDTO.getButtonState()) {
                    buttonDTO.setButtonState(ActivityButtonStateEnum.EDIT.getCode());
                }
                return buttonDTO;
            } else {
                buttonDTO.setButtonState(ActivityButtonStateEnum.EDIT.getCode());
                return buttonDTO;
            }
        }
    }

    private void neededSignJudge(LocalDateTime now, YcqActivitySign sign, YcqPublishActivity item, ActivityAppQueryRespDTO respDTO, YcqActivityPeopleAudit auditInfo, Integer defaultState, boolean needApply) {
        if (Objects.equals(1, item.getNeededSign())) {
            if (null != sign) {
                ActivitySignVO activitySignVO = new ActivitySignVO();
                BeanUtils.copyProperties(sign, activitySignVO);
                Date signInStartTime = sign.getSignInStartTime();
                activitySignVO.setSignInStartTime(DateUtils.formatDate(signInStartTime, "yyyy-MM-dd HH:mm:ss"));
                activitySignVO.setSignInEndTime(DateUtils.formatDate(sign.getSignInEndTime(), "yyyy-MM-dd HH:mm:ss"));
                activitySignVO.setSignOutStartTime(DateUtils.formatDate(sign.getSignOutStartTime(), "yyyy-MM-dd HH:mm:ss"));
                activitySignVO.setSignOutEndTime(DateUtils.formatDate(sign.getSignOutEndTime(), "yyyy-MM-dd HH:mm:ss"));
                respDTO.setSignVO(activitySignVO);
                // 是否有人员数据，有直接取，没有表示未报名
                if (null == auditInfo) {
                    if (needApply) {
                        respDTO.setButtonState(defaultState);
                    } else {
                        respDTO.setButtonState(2);
                    }
                } else {
                    Integer processState = auditInfo.getProcessState();
                    respDTO.setButtonState(processState);
                    if (Objects.equals(processState, 2)) {
                        if (now.isBefore(DateUtils.dateToLocalDateTime(sign.getSignInStartTime()))) {
                            respDTO.setButtonGrey(1);
                        }
                    } else if (Objects.equals(processState, 3)) {
                        if (now.isBefore(DateUtils.dateToLocalDateTime(sign.getSignOutStartTime()))) {
                            respDTO.setButtonGrey(1);
                        }
                    }
                }
            } else {
                log.error("当前信息错误，设置需要报名，但是没有报名表，活动id：{}", item.getId());
            }
        } else {
            // 不需要签到
            // 是否有人员数据，有直接取，没有表示
            if (null == auditInfo) {
                if (needApply) {
                    respDTO.setButtonState(defaultState);
                } else {
                    respDTO.setButtonState(4);
                }
            } else {
                respDTO.setButtonState(auditInfo.getProcessState());
            }
        }
        if (auditInfo != null && Objects.equals(auditInfo.getRegistrationStatus(), 2)) {
            respDTO.setButtonGrey(1);
        }
    }

    // 责任链可以做优化，现在代码逻辑不够简洁不利于后期维护
    private ActivityAppButtonDTO setButtonState(YcqPublishActivity resource, ActivityAppButtonDTO target, List<YcqActivityPeopleAudit> ycqActivityPeopleAudits, Map<Long, Integer> auditCountMap, Map<Long, YcqActivitySign> finalSignMap, List<YcqActivityParticipationDetails> defaultTemplateList, List<Long> dynamicTemplateList, int loginId, boolean needCancelButton) {
        List<ActivityButtonStateFilter<ButtonStateDTO>> list = new ArrayList<>();
        Long activityId = resource.getId();
        Integer isApply = resource.getIsApply();
        boolean neededApply = Objects.equals(isApply, 1);
        boolean isApplyBool = false;
        LocalDateTime startTime = resource.getStartTime();
        LocalDateTime endTime = resource.getEndTime();


        ActivityDefaultStartFilter activityDefaultStartFilter = new ActivityDefaultStartFilter(endTime);
        list.add(activityDefaultStartFilter);
        Map<Long, YcqActivityPeopleAudit> userAuditInfo = ycqActivityPeopleAudits.stream().collect(Collectors.toMap(YcqActivityPeopleAudit::getUserId, Function.identity()));
        YcqActivityPeopleAudit ycqActivityPeopleAudit = userAuditInfo.get((long) loginId);

        // 需要报名
        if (neededApply) {
            LocalDateTime applyStartTime = DateUtils.dateToLocalDateTime(resource.getApplyStartTime());
            LocalDateTime applyEndTime = DateUtils.dateToLocalDateTime(resource.getApplyEndTime());
            // 未报名
            if (null == ycqActivityPeopleAudit) {
                // 是否有人数限制
                if (Objects.equals(resource.getIsLimit(), 1)) {
                    Integer orDefault = auditCountMap.getOrDefault(activityId, 0);
                    boolean isFull = orDefault >= resource.getLimitCount();
                    ActivityApplyFilter applyFilter = new ActivityApplyFilter(neededApply, isApplyBool, isFull, applyStartTime, applyEndTime, endTime);
                    list.add(applyFilter);
                } else {
                    ActivityApplyFilter applyFilter = new ActivityApplyFilter(neededApply, isApplyBool, false, applyStartTime, applyEndTime, endTime);
                    list.add(applyFilter);
                }

            } else {
                // 是否需要审核
                boolean neededAudit = Objects.equals(resource.getNeededAudit(), 1);
                if (neededAudit) {
                    ActivityAuditFilter activityAuditFilter = new ActivityAuditFilter(startTime, endTime, ycqActivityPeopleAudit.getRegistrationStatus());
                    list.add(activityAuditFilter);
                }
            }
        }

        // 签到
        Integer neededSign = resource.getNeededSign();
        if (Objects.equals(1, neededSign)) {
            YcqActivitySign signInfo = finalSignMap.getOrDefault(activityId, new YcqActivitySign());
            Integer signType = signInfo.getSignType();
            if (Objects.equals(0, signType)) {
                LocalDateTime signInStart = DateUtils.dateToLocalDateTime(signInfo.getSignInStartTime());
                LocalDateTime signInEnd = DateUtils.dateToLocalDateTime(signInfo.getSignInEndTime());
                LocalDateTime signOutStart = DateUtils.dateToLocalDateTime(signInfo.getSignOutStartTime());
                LocalDateTime signOutEnd = DateUtils.dateToLocalDateTime(signInfo.getSignOutEndTime());
                if (null != ycqActivityPeopleAudit) {
                    ActivitySignInFilter activitySignInFilter = new ActivitySignInFilter(signInStart, signInEnd, Objects.equals(ycqActivityPeopleAudit.getCheckInResult(), 1), endTime);
                    ActivitySignOutFilter activitySignOutFilter = new ActivitySignOutFilter(signOutStart, signOutEnd, Objects.equals(ycqActivityPeopleAudit.getCheckOutResult(), 1), endTime);
                    list.add(activitySignInFilter);
                    list.add(activitySignOutFilter);
                } else {
                    ActivitySignInFilter activitySignInFilter = new ActivitySignInFilter(signInStart, signInEnd, false, endTime);
                    ActivitySignOutFilter activitySignOutFilter = new ActivitySignOutFilter(signOutStart, signOutEnd, false, endTime);
                    list.add(activitySignInFilter);
                    list.add(activitySignOutFilter);
                }
            } else if (Objects.equals(1, signType)) {  //签到
                LocalDateTime signInStart = DateUtils.dateToLocalDateTime(signInfo.getSignInStartTime());
                LocalDateTime signInEnd = DateUtils.dateToLocalDateTime(signInfo.getSignInEndTime());
                if (null != ycqActivityPeopleAudit) {
                    ActivitySignInFilter activitySignInFilter = new ActivitySignInFilter(signInStart, signInEnd, Objects.equals(ycqActivityPeopleAudit.getCheckInResult(), 1), endTime);
                    list.add(activitySignInFilter);
                } else {
                    ActivitySignInFilter activitySignInFilter = new ActivitySignInFilter(signInStart, signInEnd, false, endTime);
                    list.add(activitySignInFilter);
                }

            } else {  // 签退
                LocalDateTime signOutStart = DateUtils.dateToLocalDateTime(signInfo.getSignOutStartTime());
                LocalDateTime signOutEnd = DateUtils.dateToLocalDateTime(signInfo.getSignOutEndTime());
                if (null != ycqActivityPeopleAudit) {
                    ActivitySignOutFilter activitySignOutFilter = new ActivitySignOutFilter(signOutStart, signOutEnd, Objects.equals(ycqActivityPeopleAudit.getCheckOutResult(), 1), endTime);
                    list.add(activitySignOutFilter);
                } else {
                    ActivitySignOutFilter activitySignOutFilter = new ActivitySignOutFilter(signOutStart, signOutEnd, false, endTime);
                    list.add(activitySignOutFilter);
                }
            }
        }


        // 查询用户反馈记录
        Integer replyType = resource.getReplyType();
        boolean isFeedback;
        if (Objects.equals(replyType, 0)) {
            // 默认模板
            isFeedback = defaultTemplateList.stream().anyMatch(item -> Objects.equals(item.getActivityId(), activityId));
        } else {
            isFeedback = dynamicTemplateList.contains(activityId);
        }
        boolean canFeedback = true;
        if (Objects.equals(isApply, 1)){
            if (ycqActivityPeopleAudit == null || !Objects.equals(ycqActivityPeopleAudit.getRegistrationStatus(), 1)){
                canFeedback = false;
            }
        }
        ActivityFeedbackFilter activityApplyFilter = new ActivityFeedbackFilter(endTime, isFeedback, !Objects.equals(2, replyType), canFeedback);
        list.add(activityApplyFilter);

//        ActivityDefaultEndFilter endFilter = new ActivityDefaultEndFilter();
//
//        list.add(endFilter);
        for (int i = 0; i < list.size(); i++) {
            if (i == (list.size() - 1)) {
                break;
            }
            ActivityButtonStateFilter<ButtonStateDTO> current = list.get(i);
            int nextInt = i + 1;
            ActivityButtonStateFilter<ButtonStateDTO> next = list.get(nextInt);
            current.setNext(next);
        }
        LocalDateTime now = LocalDateTime.now();
        ButtonStateDTO result = list.get(0).doFilter(now, false);
        target.setButtonState(result.getButtonState());
        target.setButtonGrey(result.getIsGrey());
        if (now.isBefore(endTime)){
            if (needCancelButton) {
                if (null != ycqActivityPeopleAudit) {
                    if (Objects.equals(ycqActivityPeopleAudit.getRegistrationStatus(), 2)) {
                        target.setCancelApply(1);
                    }
                }
            }
        }
        if (Objects.equals(6, target.getButtonState()) && now.isAfter(DateUtils.dateToLocalDateTime(resource.getApplyEndTime()))){
            target.setButtonState(7);
        }
        return target;
    }

    @Override
    public Result<AppActivityDetailRespDTO> doActivityDetailForApp(Long id) {
        Worker currWorker = getWorker();
        int loginId = currWorker.getLoginId();
 //       int loginId = 9999;
        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(id);
        if (!publishActivity.isPresent()) {
            throw new BusinessException("活动状态异常，请联系管理员");
        }
        YcqPublishActivity ycqPublishActivity = publishActivity.get();

        AppActivityDetailRespDTO appActivityQueryRespDTO = new AppActivityDetailRespDTO();
        BeanUtils.copyProperties(ycqPublishActivity, appActivityQueryRespDTO);
        appActivityQueryRespDTO.setActivityStatusTypeName(ActivityStatusTypeEnum.findByCode(appActivityQueryRespDTO.getActivityStatusTypeId()));
        //appActivityQueryRespDTO.setPublishTime(ycqPublishActivity.getPublishTime().format(DateTimeFormatter.ofPattern(DateUtils.DEFAULT_DATE_TIME_FORMAT)));
        appActivityQueryRespDTO.setStartTime(ycqPublishActivity.getStartTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")));
        appActivityQueryRespDTO.setEndTime(ycqPublishActivity.getEndTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")));
        Date applyStartTime = ycqPublishActivity.getApplyStartTime();
        if (null != applyStartTime) {
            appActivityQueryRespDTO.setApplyStartTime(DateUtils.format(applyStartTime, "yyyy-MM-dd HH:mm"));
        }
        Date applyEndTime = ycqPublishActivity.getApplyEndTime();
        if (null != applyEndTime) {
            appActivityQueryRespDTO.setApplyEndTime(DateUtils.format(applyEndTime, "yyyy-MM-dd HH:mm"));
        }
        HashMap<Long, YcqActivitySign> signHashMap = new HashMap<>(4);
        // 签到信息（如有）
        YcqActivitySign sign = signRepo.findByActivityId(id);
        if (Objects.equals(ycqPublishActivity.getNeededSign(), 1)) {
            if (sign != null) {
                signHashMap.put(id, sign);
                ActivitySignVO activitySignVO = new ActivitySignVO();
                BeanUtils.copyProperties(sign, activitySignVO);
                Date signInStartTime = sign.getSignInStartTime();
                activitySignVO.setSignInStartTime(DateUtils.formatDate(signInStartTime, "yyyy-MM-dd HH:mm:ss"));
                activitySignVO.setSignInEndTime(DateUtils.formatDate(sign.getSignInEndTime(), "yyyy-MM-dd HH:mm:ss"));
                activitySignVO.setSignOutStartTime(DateUtils.formatDate(sign.getSignOutStartTime(), "yyyy-MM-dd HH:mm:ss"));
                activitySignVO.setSignOutEndTime(DateUtils.formatDate(sign.getSignOutEndTime(), "yyyy-MM-dd HH:mm:ss"));
                appActivityQueryRespDTO.setSignVO(activitySignVO);
            }
        }

        appActivityQueryRespDTO.setHeadPicUrlList(publishActivityDao.getByArticleIdAndTye(id, 5));
        // 按钮状态
        List<YcqActivityPeopleAudit> ycqActivityPeopleAudits = new ArrayList<>(1);
        YcqActivityPeopleAudit ycqActivityPeopleAudit = activityPeopleAuditDao.queryByActivityIdAndLoginId(id, (long) loginId);
        if (null != ycqActivityPeopleAudit) {
            //todo 加入识别是否报名成功得验证
            appActivityQueryRespDTO.setIsApplication(1);
        } else {
            appActivityQueryRespDTO.setIsApplication(0);
        }
        YcqActivityPeopleAudit ycqActivityPeopleAudit1 = activityPeopleAuditDao.queryAuditInfoByActivityIdAndUserId(id, loginId);
        if (null != ycqActivityPeopleAudit1){
            ycqActivityPeopleAudits.add(ycqActivityPeopleAudit1);
        }
        List<YcqActivityParticipationDetails> defaultTemplateList = new ArrayList<>();
        List<Long> dynamicFeedbackIds = new ArrayList<>();
        if (Objects.equals(ycqPublishActivity.getReplyType(), 0)) {
            YcqActivityParticipationDetails byActivityIdAndUserId = defaultTemplateRepo.findByActivityIdAndUserId(id, (long) loginId);
            if (null != byActivityIdAndUserId) {
                defaultTemplateList.add(byActivityIdAndUserId);
            }
        } else if (Objects.equals(ycqPublishActivity.getReplyType(), 1)){
            dynamicFeedbackIds = dynamicFormService.getRecordByUsrAndActiveIds(loginId, Collections.singletonList(id), 2).stream().map(item -> {
                return (long) item.get("activeId");
            }).collect(Collectors.toList());
        }

        Map<Long, Integer> queryAuditCount = activityPeopleAuditDao.queryAuditCount(Collections.singletonList(id));
        ActivityAppButtonDTO buttonDTO = this.setButtonState(ycqPublishActivity, new ActivityAppButtonDTO(), ycqActivityPeopleAudits, queryAuditCount, signHashMap, defaultTemplateList, dynamicFeedbackIds, loginId, true);
        appActivityQueryRespDTO.setCancelApply(buttonDTO.getCancelApply());
        appActivityQueryRespDTO.setButtonGrey(buttonDTO.getButtonGrey());
        appActivityQueryRespDTO.setButtonState(buttonDTO.getButtonState());
//        LocalDateTime now = LocalDateTime.now();
//        ActivityAppQueryRespDTO respDTO = new ActivityAppQueryRespDTO();
//        YcqActivityPeopleAudit byUserIdAndActivityId = activityPeopleAuditDao.queryByActivityIdAndLoginId(id, (long) loginId);
//        if (Objects.equals(ycqPublishActivity.getIsApply(), POSITIVE_NUMBER)){
//            // 需要报名
//            this.neededSignJudge(now, sign, ycqPublishActivity, respDTO, byUserIdAndActivityId, 1, true);
//        }else {
//            // 不需要报名
//            this.neededSignJudge(now, sign, ycqPublishActivity, respDTO, byUserIdAndActivityId, 7, false);
//        }
//        appActivityQueryRespDTO.setButtonState(respDTO.getButtonState());
//        appActivityQueryRespDTO.setButtonGrey(respDTO.getButtonGrey());
//        // 如果当前未审核，则展示取消报名按钮
//        if (byUserIdAndActivityId != null && Objects.equals(byUserIdAndActivityId.getProcessState(), 2) && Objects.equals(1,ycqPublishActivity.getNeededAudit())){
//            appActivityQueryRespDTO.setCancelApply(1);
//        }


        //  默认模板数据/参与人信息
        Result<AppActivityDetailsVO> appActivityDetailsVOResult = defaultTemplateService.queryViewDetails(id);
        if (appActivityDetailsVOResult != null && appActivityDetailsVOResult.isSuccess()) {
            appActivityQueryRespDTO.setUserAndTemplateInfo(appActivityDetailsVOResult.getData());
        }

        // 是否需要提交报名信息
        appActivityQueryRespDTO.setNeededSubmit(0);
        Integer isApply = appActivityQueryRespDTO.getIsApply();
        if (Objects.equals(POSITIVE_NUMBER, isApply)) {
            // 是否有报名表
            YcqDynamicForm byBusinessIdAndType = ycqDynamicFormRepo.getByBusinessIdAndType(id, 1);
            if (null != byBusinessIdAndType) {
                appActivityQueryRespDTO.setNeededSubmit(1);
            }
            // 当前用户是否报名
            Integer integer = activityPeopleAuditDao.queryCurrentUserApplyInfo(id, (long) loginId);
            int i = integer > 0 ? 1 : 0;
            appActivityQueryRespDTO.setApplyFlag(i);
        }

        // 区域静态值转换
        List<YcqActivityZone> byActivityId = activityZoneRepo.findByActivityId(id);
        appActivityQueryRespDTO.setZoneNames(byActivityId.stream().filter(item -> !Objects.equals(item.getValueCode(), "1")).map(YcqActivityZone::getValueName).collect(Collectors.joining(",")));

        if (Objects.equals(ycqPublishActivity.getIsApply(), POSITIVE_NUMBER)) {
            appActivityQueryRespDTO.setApplyForm(dynamicFormService.getDetailsByActiveId(ycqPublishActivity.getId(), 1));
        }

        return Result.ofSuccess(appActivityQueryRespDTO);
    }


    @Override
    public Result<AppActivityDetailRespDTO> doActivityDetailForAppWithoutToken(Long id) {
        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(id);
        if (!publishActivity.isPresent()) {
            throw new BusinessException("活动状态异常，请联系管理员");
        }
        YcqPublishActivity ycqPublishActivity = publishActivity.get();

        AppActivityDetailRespDTO appActivityQueryRespDTO = new AppActivityDetailRespDTO();
        BeanUtils.copyProperties(ycqPublishActivity, appActivityQueryRespDTO);
        appActivityQueryRespDTO.setActivityStatusTypeName(ActivityStatusTypeEnum.findByCode(appActivityQueryRespDTO.getActivityStatusTypeId()));
        //appActivityQueryRespDTO.setPublishTime(ycqPublishActivity.getPublishTime().format(DateTimeFormatter.ofPattern(DateUtils.DEFAULT_DATE_TIME_FORMAT)));
        appActivityQueryRespDTO.setStartTime(ycqPublishActivity.getStartTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")));
        appActivityQueryRespDTO.setEndTime(ycqPublishActivity.getEndTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")));
        Date applyStartTime = ycqPublishActivity.getApplyStartTime();
        if (null != applyStartTime) {
            appActivityQueryRespDTO.setApplyStartTime(DateUtils.format(applyStartTime, "yyyy-MM-dd HH:mm"));
        }
        Date applyEndTime = ycqPublishActivity.getApplyEndTime();
        if (null != applyEndTime) {
            appActivityQueryRespDTO.setApplyEndTime(DateUtils.format(applyEndTime, "yyyy-MM-dd HH:mm"));
        }
        HashMap<Long, YcqActivitySign> signHashMap = new HashMap<>(4);
        // 签到信息（如有）
        YcqActivitySign sign = signRepo.findByActivityId(id);
        if (Objects.equals(ycqPublishActivity.getNeededSign(), 1)) {
            if (sign != null) {
                signHashMap.put(id, sign);
                ActivitySignVO activitySignVO = new ActivitySignVO();
                BeanUtils.copyProperties(sign, activitySignVO);
                Date signInStartTime = sign.getSignInStartTime();
                activitySignVO.setSignInStartTime(DateUtils.formatDate(signInStartTime, "yyyy-MM-dd HH:mm:ss"));
                activitySignVO.setSignInEndTime(DateUtils.formatDate(sign.getSignInEndTime(), "yyyy-MM-dd HH:mm:ss"));
                activitySignVO.setSignOutStartTime(DateUtils.formatDate(sign.getSignOutStartTime(), "yyyy-MM-dd HH:mm:ss"));
                activitySignVO.setSignOutEndTime(DateUtils.formatDate(sign.getSignOutEndTime(), "yyyy-MM-dd HH:mm:ss"));
                appActivityQueryRespDTO.setSignVO(activitySignVO);
            }
        }

        appActivityQueryRespDTO.setHeadPicUrlList(publishActivityDao.getByArticleIdAndTye(id, 5));
        // 按钮状态
        List<YcqActivityPeopleAudit> ycqActivityPeopleAudits = new ArrayList<>(1);
        YcqActivityPeopleAudit ycqActivityPeopleAudit = new YcqActivityPeopleAudit();
        appActivityQueryRespDTO.setIsApplication(0);

        List<YcqActivityParticipationDetails> defaultTemplateList = new ArrayList<>();
        List<Long> dynamicFeedbackIds = new ArrayList<>();

        Map<Long, Integer> queryAuditCount = activityPeopleAuditDao.queryAuditCount(Collections.singletonList(id));
        ActivityAppButtonDTO buttonDTO = this.setButtonState(ycqPublishActivity, new ActivityAppButtonDTO(), ycqActivityPeopleAudits, queryAuditCount, signHashMap, defaultTemplateList, dynamicFeedbackIds, 99999999, true);
        appActivityQueryRespDTO.setCancelApply(buttonDTO.getCancelApply());
        appActivityQueryRespDTO.setButtonGrey(buttonDTO.getButtonGrey());
        appActivityQueryRespDTO.setButtonState(buttonDTO.getButtonState());

        //  默认模板数据/参与人信息
        Result<AppActivityDetailsVO> appActivityDetailsVOResult = defaultTemplateService.queryViewDetails(id);
        if (appActivityDetailsVOResult != null && appActivityDetailsVOResult.isSuccess()) {
            appActivityQueryRespDTO.setUserAndTemplateInfo(appActivityDetailsVOResult.getData());
        }

        // 是否需要提交报名信息
        appActivityQueryRespDTO.setNeededSubmit(0);
        Integer isApply = appActivityQueryRespDTO.getIsApply();
        if (Objects.equals(POSITIVE_NUMBER, isApply)) {
            // 是否有报名表
            YcqDynamicForm byBusinessIdAndType = ycqDynamicFormRepo.getByBusinessIdAndType(id, 1);
            if (null != byBusinessIdAndType) {
                appActivityQueryRespDTO.setNeededSubmit(1);
            }
            // 当前用户是否报名
            appActivityQueryRespDTO.setApplyFlag(0);
        }

        // 区域静态值转换
        List<YcqActivityZone> byActivityId = activityZoneRepo.findByActivityId(id);
        appActivityQueryRespDTO.setZoneNames(byActivityId.stream().filter(item -> !Objects.equals(item.getValueCode(), "1")).map(YcqActivityZone::getValueName).collect(Collectors.joining(",")));

        if (Objects.equals(ycqPublishActivity.getIsApply(), POSITIVE_NUMBER)) {
            appActivityQueryRespDTO.setApplyForm(dynamicFormService.getDetailsByActiveId(ycqPublishActivity.getId(), 1));
        }

        return Result.ofSuccess(appActivityQueryRespDTO);
    }

    private String addActivityValidate(ActivityAddReqDTO dto) {
//        if (StringUtils.isNotBlank(dto.getActivityAddress()) && dto.getActivityAddress().length() > 50) {
//            return "活动地址不允许超过50个字符";
//        }
//        if (StringUtils.isBlank(dto.getActivityTitle())) {
//            return "活动标题内容为空或者超长";
//        }
//        Pattern pattern = Pattern.compile("^0\\d{2,3}-\\d{7,8}$|^1[3-9]\\d{9}$");
//        if (StringUtils.isNotBlank(dto.getContactInformation()) && Boolean.FALSE.equals(pattern.matcher(dto.getContactInformation()).find())) {
//            return "无效的联系方式";
//        }
//        pattern = Pattern.compile("^\\d{4}-\\d{2}-\\d{2}\\s\\d{2}:\\d{2}:\\d{2}$");
//        if (Boolean.FALSE.equals(pattern.matcher(dto.getStartTime()).find())) {
//            return "时间格式必须是yyyy-MM-dd HH:mm:ss";
//        }
//        if (Boolean.FALSE.equals(pattern.matcher(dto.getEndTime()).find())) {
//            return "时间格式必须是yyyy-MM-dd HH:mm:ss";
//        }
        if (dto.getIsLimit() == 1 && (null == dto.getLimitCount() || dto.getLimitCount() <=0)) {
            return "限定人数有误";
        }
        if (LocalDateTime.now().isAfter(LocalDateTime.parse(dto.getEndTime(), DateTimeFormatter.ofPattern(DateUtils.DEFAULT_DATE_TIME_FORMAT)))) {
            return "无法新增已经结束的活动";
        }
        Integer isApply = dto.getIsApply();
//        if (!(isApply == 0 || isApply == 1)) {
//            return "是否需要报名参数格式不正确";
//        }
        if (Objects.equals(isApply, 0)) {
            dto.setNeededAudit(0);
        } else {
            Integer neededAudit = dto.getNeededAudit();
            if (null == neededAudit) {
                return "报名是否需要审核必填";
            }
        }
//        if (!(dto.getIsLimit() == 0 || dto.getIsLimit() == 1)) {
//            return "是否限定人数参数格式不正确";
//        }
//        pattern = Pattern.compile("^[1-9]\\d*$");
//        if (StringUtils.isNotBlank(dto.getLimitCount()) && Boolean.FALSE.equals(pattern.matcher(dto.getLimitCount()).find())) {
//            return "限定人数必须是正整数";
//        }
//        if (StringUtils.isBlank(dto.getBusinessBelong())) {
//            return "所属业务必填";
//        }
        // 签到签退时间必须在活动时间之间
        Integer neededSign = dto.getNeededSign();
        LocalDateTime startTime = TimeUtils.get(dto.getStartTime());
        LocalDateTime endTime = TimeUtils.get(dto.getEndTime());
        if (Objects.equals(1, neededSign)) {
            ActivitySignDTO signDTO = dto.getSignDTO();
            if (null == signDTO) {
                return "签到配置信息必填";
            }
            Integer signInWay = signDTO.getSignInWay();
            Integer signOutWay = signDTO.getSignOutWay();
            if (null == signInWay && null == signOutWay) {
                return "签到或签退方式必选";
            }
            if (null == signOutWay) {
                signDTO.setSignType(1);
            } else if (null == signInWay) {
                signDTO.setSignType(2);
            } else {
                signDTO.setSignType(0);
            }
            Integer signType = signDTO.getSignType();
            String signInStartTime = signDTO.getSignInStartTime();
            String signInEndTime = signDTO.getSignInEndTime();
            String signOutStartTime = signDTO.getSignOutStartTime();
            String signOutEndTime = signDTO.getSignOutEndTime();
            if (Objects.equals(0, signType)) {
                String s = this.signInTimeValid(signInStartTime, signInEndTime, startTime, endTime);
                if (StringUtils.isNotBlank(s)) {
                    return s;
                }
                String s1 = this.signOutTimeValid(signOutStartTime, signOutEndTime, startTime, endTime, signInEndTime);
                if (StringUtils.isNotBlank(s1)) {
                    return s1;
                }
            } else if (Objects.equals(1, signType)) {
                String s = this.signInTimeValid(signInStartTime, signInEndTime, startTime, endTime);
                if (StringUtils.isNotBlank(s)) {
                    return s;
                }
            } else {
                String s = this.signOutTimeValid(signOutStartTime, signOutEndTime, startTime, endTime, null);
                if (StringUtils.isNotBlank(s)) {
                    return s;
                }
            }
        }
        // 报名时间必须早于活动最晚时间
        if (Objects.equals(isApply, 1)) {
            if (null != dto.getApplyForm()) {
                if (!StringUtils.isBlank(dto.getApplyForm().getFormJson())) {
                    String businessBelong = dto.getBusinessBelong();
                    if (Objects.equals(businessBelong, "youthPromise")){
                        if (!dto.getApplyForm().getFormJson().contains("idcared")){
                            return "所属业务为青春之约,报名表需要配置身份证";
                        }
                    }
                }
            }
            String applyEndTime1 = dto.getApplyEndTime();
            String applyStartTime1 = dto.getApplyStartTime();

            if (StringUtils.isBlank(applyEndTime1) || StringUtils.isBlank(applyStartTime1)) {
                return "报名时间不能为空";
            }
            LocalDateTime applyEndTime = TimeUtils.get(applyEndTime1);
            LocalDateTime applyStartTime = TimeUtils.get(applyStartTime1);
            if (applyEndTime.isBefore(LocalDateTime.now())){
                return "报名结束时间必须晚于当前时间";
            }
            if (applyEndTime.isAfter(startTime)) {
                return "报名结束时间必须早于活动开始时间";
            }
            if (applyEndTime.isBefore(applyStartTime)){
                return "报名开始时间必须早于报名结束时间";
            }

        }
        if (Objects.equals(dto.getReplyType(), 1)) {
            if (dto.getDynamicForm() == null || StringUtils.isBlank(dto.getDynamicForm().getFormJson())) {
                return "自定义模板需要配置";
            }
        }
        if (CollectionUtils.isEmpty(dto.getOrgIdList())){
            return "参与组织不能为空";
        }
        List<ActivityAddReqDTO.ZoneAddDTO> zoneAppAddDTOList = dto.getZoneAppAddDTOList();
        if (CollectionUtils.isEmpty(zoneAppAddDTOList)){
            return "所在区域不能为空";
        }
        return null;
    }

    private String editActivityValidate2(ActivityEditReqDTO dto) {
        if (dto.getIsLimit() == 1 && null == dto.getLimitCount()) {
            return "限定人数为必填项";
        }
        if (LocalDateTime.now().isAfter(LocalDateTime.parse(dto.getEndTime(), DateTimeFormatter.ofPattern(DateUtils.DEFAULT_DATE_TIME_FORMAT)))) {
            return "无法新增已经结束的活动";
        }
        Integer isApply = dto.getIsApply();
        if (Objects.equals(isApply, 0)) {
            dto.setNeededAudit(0);
        } else {
            Integer neededAudit = dto.getNeededAudit();
            if (null == neededAudit) {
                return "报名是否需要审核必填";
            }
        }
        if (CollectionUtils.isEmpty(dto.getOrgIdList())){
            return "参与组织不能为空";
        }
        List<ActivityEditReqDTO.ZoneEditDTO> zoneAppAddDTOList1 = dto.getZoneAppAddDTOList();
        if (CollectionUtils.isEmpty(zoneAppAddDTOList1)){
            return "所在区域必填";
        }
        // 签到签退时间必须在活动时间之间
        Integer neededSign = dto.getNeededSign();
        LocalDateTime startTime = TimeUtils.get(dto.getStartTime());
        LocalDateTime endTime = TimeUtils.get(dto.getEndTime());
        if (Objects.equals(1, neededSign)) {
            ActivitySignDTO signDTO = dto.getSignDTO();
            if (null == signDTO) {
                return "签到配置信息必填";
            }
            Integer signInWay = signDTO.getSignInWay();
            Integer signOutWay = signDTO.getSignOutWay();
            if (null == signInWay && null == signOutWay) {
                return "签到或签退方式必选";
            }
            if (null == signOutWay) {
                signDTO.setSignType(1);
            } else if (null == signInWay) {
                signDTO.setSignType(2);
            } else {
                signDTO.setSignType(0);
            }
            Integer signType = signDTO.getSignType();
            String signInStartTime = signDTO.getSignInStartTime();
            String signInEndTime = signDTO.getSignInEndTime();
            String signOutStartTime = signDTO.getSignOutStartTime();
            String signOutEndTime = signDTO.getSignOutEndTime();
            if (Objects.equals(0, signType)) {
                String s = this.signInTimeValid(signInStartTime, signInEndTime, startTime, endTime);
                if (StringUtils.isNotBlank(s)) {
                    return s;
                }
                String s1 = this.signOutTimeValid(signOutStartTime, signOutEndTime, startTime, endTime, signInEndTime);
                if (StringUtils.isNotBlank(s1)) {
                    return s1;
                }
            } else if (Objects.equals(1, signType)) {
                String s = this.signInTimeValid(signInStartTime, signInEndTime, startTime, endTime);
                if (StringUtils.isNotBlank(s)) {
                    return s;
                }
            } else {
                String s = this.signOutTimeValid(signOutStartTime, signOutEndTime, startTime, endTime, null);
                if (StringUtils.isNotBlank(s)) {
                    return s;
                }
            }
        }
        // 报名时间必须早于活动最晚时间
        if (Objects.equals(isApply, 1)) {
            if (null != dto.getApplyForm()) {
                if (!StringUtils.isBlank(dto.getApplyForm().getFormJson())) {
                    String businessBelong = dto.getBusinessBelong();
                    if (Objects.equals(businessBelong, "youthPromise")){
                        if (!dto.getApplyForm().getFormJson().contains("idcared")){
                            return "所属业务为青春之约,报名表需要配置身份证";
                        }
                    }
                }
            }
            String applyEndTime1 = dto.getApplyEndTime();
            String applyStartTime1 = dto.getApplyStartTime();

            if (StringUtils.isBlank(applyEndTime1) || StringUtils.isBlank(applyStartTime1)) {
                return "报名时间不能为空";
            }
            LocalDateTime applyEndTime = TimeUtils.get(applyEndTime1);
            LocalDateTime applyStartTime = TimeUtils.get(applyStartTime1);
            if (applyEndTime.isBefore(LocalDateTime.now())){
                return "报名结束时间必须晚于当前时间";
            }
            if (applyEndTime.isAfter(startTime)) {
                return "报名结束时间必须早于活动开始时间";
            }
            if (applyEndTime.isBefore(applyStartTime)){
                return "报名开始时间必须早于报名结束时间";
            }

        }
        if (Objects.equals(dto.getReplyType(), 1)) {
            if (dto.getDynamicForm() == null || StringUtils.isBlank(dto.getDynamicForm().getFormJson())) {
                return "自定义模板需要配置";
            }
        }
        return null;
    }

    private String signInTimeValid(String signInStartTime, String signInEndTime, LocalDateTime startTime, LocalDateTime endTime) {
        if (StringUtils.isBlank(signInEndTime) || StringUtils.isBlank(signInStartTime)) {
            return "签到时间不能为空";
        }
        LocalDateTime signInLocaleStart = TimeUtils.get(signInStartTime);
        LocalDateTime signInLocaleEnd = TimeUtils.get(signInEndTime);
        if (signInLocaleStart.isAfter(signInLocaleEnd)) {
            return "签到开始时间不能晚于签到结束时间";
        }
        if (signInLocaleStart.isBefore(startTime) || signInLocaleEnd.isAfter(endTime)) {
            return "签到时间时间必须在活动时间范围内";
        }
        return null;
    }

    private String signOutTimeValid(String signOutStartTime, String signOutEndTime, LocalDateTime startTime, LocalDateTime endTime, String signInEndTime) {
        if (StringUtils.isBlank(signOutStartTime) || StringUtils.isBlank(signOutEndTime)) {
            return "签退时间不能为空";
        }
        LocalDateTime signOutLocaleStart = TimeUtils.get(signOutStartTime);
        LocalDateTime signOutLocaleEnd = TimeUtils.get(signOutEndTime);
        if (signOutLocaleStart.isAfter(signOutLocaleEnd)) {
            return "签退开始时间不能晚于签退结束时间";
        }
        if (signOutLocaleStart.isBefore(startTime) || signOutLocaleEnd.isAfter(endTime)) {
            return "签退时间必须在活动时间范围内";
        }
        if (StringUtils.isNotBlank(signInEndTime)) {
            if (TimeUtils.get(signInEndTime).isAfter(signOutLocaleStart)) {
                return "签到时间不能晚于签退时间";
            }
        }
        return null;
    }

    private void saveAnnexAndCover(Map<Integer, List<CoverImageDto>> coverImageFile, Long id) {
        List<YcqQzhCover> coverImageDtos = new ArrayList<>(coverImageFile.size());
        //保存封面
        coverImageFile.forEach((type, value) -> {
            if (!CollectionUtils.isEmpty(value)) {
                value.forEach(s -> {
                    YcqQzhCover ycqQzhCover = new YcqQzhCover();
                    ycqQzhCover.setFileId(s.getFileId());
                    ycqQzhCover.setFileName(s.getFileName());
                    ycqQzhCover.setFileKey(s.getFileKey());
                    ycqQzhCover.setFileType(s.getFileType());
                    if (s.getFileSize() != null) {
                        ycqQzhCover.setFileSize(Integer.parseInt(s.getFileSize()));
                    }
                    ycqQzhCover.setFileUrl(s.getFileUrl());
                    ycqQzhCover.setPicHeight(s.getPicHeight());
                    ycqQzhCover.setPicWidth(s.getPicWidth());
                    ycqQzhCover.setArticleId(id);
                    ycqQzhCover.setType(type);
                    coverImageDtos.add(ycqQzhCover);
                });
            }
        });
        if (!CollectionUtils.isEmpty(coverImageDtos)) {
            ycqQzhCoverRepo.saveAllAndFlush(coverImageDtos);
        }
    }


    private String editActivityValidate(ActivityEditReqDTO dto) {
        if (dto.getId() == null) {
            return "活动id必传";
        }
        if (StringUtils.isNotBlank(dto.getActivityAddress()) && dto.getActivityAddress().length() > 50) {
            return "活动地址不允许超过50个字符";
        }
        if (StringUtils.isBlank(dto.getActivityTitle())) {
            return "活动标题必填";
        }
        if (dto.getActivityTitle().length() > 50) {
            return "活动标题不允许超过50个字符";
        }
        Pattern pattern = Pattern.compile("^0\\d{2,3}-\\d{7,8}$|^1[3-9]\\d{9}$");
        if (StringUtils.isNotBlank(dto.getContactInformation()) && Boolean.FALSE.equals(pattern.matcher(dto.getContactInformation()).find())) {
            return "无效的联系方式";
        }
        pattern = Pattern.compile("^\\d{4}-\\d{2}-\\d{2}\\s\\d{2}:\\d{2}:\\d{2}$");
        if (Boolean.FALSE.equals(pattern.matcher(dto.getStartTime()).find())) {
            return "时间格式必须是yyyy-MM-dd HH:mm:ss";
        }
        if (Boolean.FALSE.equals(pattern.matcher(dto.getEndTime()).find())) {
            return "时间格式必须是yyyy-MM-dd HH:mm:ss";
        }
//        if (!(dto.getIsApply() == 0 || dto.getIsApply() == 1)) {
//            return "是否需要报名参数格式不正确";
//        }
        if (!(dto.getIsLimit() == 0 || dto.getIsLimit() == 1)) {
            return "是否限定人数参数格式不正确";
        }
        pattern = Pattern.compile("^[1-9]\\d*$");
        if (null == dto.getLimitCount()) {
            return "限定人数必须是正整数";
        }
        if (StringUtils.isNotBlank(dto.getSponsor()) && dto.getSponsor().length() > 100) {
            return "活动举办方不允许超过100个字符";
        }
        if (StringUtils.isBlank(dto.getBusinessBelong())) {
            return "所属业务必填";
        }
        return null;
    }

    @Override
    public Result<Boolean> doActivityDelete(Long id) {
        Worker currWorker = getWorker();
        int loginId = currWorker.getLoginId();
    //    int loginId = 9999;
        YcqPublishActivity ycqPublishActivity = publishActivityRepo.findByIdAndIsDeleted(id, 0);
        if (ycqPublishActivity == null) {
            throw new BusinessException("所选活动不存在");
        }
        Integer[] arr = {1, 8, 9};
        if (!Arrays.asList(arr).contains(ycqPublishActivity.getActivityStatusTypeId())) {
            throw new BusinessException("当前活动状态不允许删除");
        }
        ycqPublishActivity.setIsDeleted(1);
        ycqPublishActivity.setUpdateTime(new Date());
        ycqPublishActivity.setUpdateUserId(loginId);
        publishActivityRepo.saveAndFlush(ycqPublishActivity);
        //删除关联的风采
        ycqQzhActivityRepo.updateIsDeleted(ycqPublishActivity.getId().toString());
        List<Long> list = new ArrayList<>();
        list.add((long) ycqPublishActivity.getIsDeleted());
        List<YcqQzhCover> allByArticleIdInAndType = ycqQzhCoverRepo.findAllByArticleIdInAndType(list, 4);
        if (!CollectionUtils.isEmpty(allByArticleIdInAndType)) {
            publishActivityDao.deleteAnnex(ycqPublishActivity.getId());
        }
        // 删除签到表
        Integer neededSign = ycqPublishActivity.getNeededSign();
        if (Objects.equals(POSITIVE_NUMBER, neededSign)) {
            signRepo.deleteByActivityId(id);
        }
        // 删除报名申请表配置    // 删除报名表数据
        if (Objects.equals(POSITIVE_NUMBER, ycqPublishActivity.getIsApply())) {
            applyFormRepo.deleteByActivityId(id);
            applyRecordRepo.deleteByActivityId(id);
        }
        // 删除自定义表单数据
        if (Objects.equals(POSITIVE_NUMBER, ycqPublishActivity.getReplyType())) {
            dynamicFormService.delete(id);
        } else {
            defaultTemplateService.deleteDefaultTemplate(id);
        }

        // 删除用户信息
        ycqActivityPeopleAuditRepo.deleteByActivityId(id);

        return Result.ofSuccess(true);
    }


    @Override
    public Result<PageRsp<ActivityRegisterRespDTO>> activityApplyDetail(PageReq<ActivityApplyDetailReqDTO> dto, Boolean isApp) {
        return Result.ofSuccess(activityRegisterDao.pageList(dto));
    }

    @Override
    public Result<PageRsp<String>> appActivityApplyDetail(PageReq<ActivityApplyDetailReqDTO> dto, Boolean isApp) {
        Worker currWorker = getWorker();
        PageRsp<ActivityRegisterRespDTO> pageRsp = activityRegisterDao.pageList(dto);
        List<ActivityRegisterRespDTO> result = pageRsp.getResult();
        List<String> names = result.stream().map(ActivityRegisterRespDTO::getName).collect(Collectors.toList());
        PageRsp<String> objectPageRsp = new PageRsp<>();
        objectPageRsp.setPageNo(pageRsp.getPageNo());
        objectPageRsp.setPageSize(pageRsp.getPageSize());
        objectPageRsp.setTotalNum(pageRsp.getTotalNum());
        objectPageRsp.setResult(names);
        objectPageRsp.setHasNext(pageRsp.isHasNext());
        return Result.ofSuccess(objectPageRsp);
    }

    @Override
    public Result<ActivtiyJoinDetailRespDTO> doUserApplyDetail(Integer id) {
//        Worker currWorker = getWorker();
//        ActivtiyJoinDetailRespDTO activtiyJoinDetailRespDTO = activityRegisterDao.userApplyDetail(id);
//        if (activtiyJoinDetailRespDTO == null) {
//            throw new BusinessException("报名详情不存在");
//        }
//        List<YcqAppConfigDTO> list = publishActivityDao.findSelectBusiness();
//        Map<String, String> businessMap = list.stream().collect(Collectors.toMap(YcqAppConfigDTO::getAppCode, YcqAppConfigDTO::getSystemName));
//        activtiyJoinDetailRespDTO.setBusinessBelongName(businessMap.get(activtiyJoinDetailRespDTO.getBusinessBelong()));
//        //
//        DynamicRecordDetailReqDTO dynamicRecordDetailReqDTO = new DynamicRecordDetailReqDTO();
//        dynamicRecordDetailReqDTO.setActivityId(activtiyJoinDetailRespDTO.getActivityId());
//        dynamicRecordDetailReqDTO.setWorkerId(activtiyJoinDetailRespDTO.getWorkerId());
//        Result<List<YcqQzhDynamicFormRecord>> formRecord = ycqQzhDynamicFormService.findDynamicFormByActivityIdAndUserId(currWorker, dynamicRecordDetailReqDTO);
//        List<YcqQzhDynamicFormRecord> data = formRecord.getData();
//        List<DynamicFormsReqDTO> dynamicFormsReqDTOS = new ArrayList<>();
//        for (YcqQzhDynamicFormRecord datum : data) {
//            DynamicFormsReqDTO dynamicFormsReqDTO = new DynamicFormsReqDTO();
//            BeanUtils.copyProperties(datum, dynamicFormsReqDTO);
//            dynamicFormsReqDTOS.add(dynamicFormsReqDTO);
//        }
//        activtiyJoinDetailRespDTO.setDynamicRecords(dynamicFormsReqDTOS);
//
//        return Result.ofSuccess(activtiyJoinDetailRespDTO);

        Worker currWorker = getWorker();
        ActivtiyJoinDetailRespDTO activtiyJoinDetailRespDTO = activityRegisterDao.userApplyDetail(id);
        if (activtiyJoinDetailRespDTO == null) {
            throw new BusinessException("报名详情不存在");
        }
        List<YcqAppConfigDTO> list = publishActivityDao.findSelectBusiness();
        Map<String, String> businessMap = list.stream().collect(Collectors.toMap(YcqAppConfigDTO::getAppCode, YcqAppConfigDTO::getSystemName));
        activtiyJoinDetailRespDTO.setBusinessBelongName(businessMap.get(activtiyJoinDetailRespDTO.getBusinessBelong()));
        //
        DynamicRecordDetailReqDTO dynamicRecordDetailReqDTO = new DynamicRecordDetailReqDTO();
        dynamicRecordDetailReqDTO.setActivityId(activtiyJoinDetailRespDTO.getActivityId());
        dynamicRecordDetailReqDTO.setWorkerId(activtiyJoinDetailRespDTO.getWorkerId());
//        Result<YcqQzhDynamicFormRecord> formRecord = ycqQzhDynamicFormService.findDynamicFormByActivityIdAndUserId(currWorker, dynamicRecordDetailReqDTO);
//        YcqQzhDynamicFormRecord data = formRecord.getData();
//        if (data!=null) {
//            DynamicFormsReqDTO dynamicFormsReqDTO = new DynamicFormsReqDTO();
//            BeanUtils.copyProperties(data, dynamicFormsReqDTO);
//            activtiyJoinDetailRespDTO.setDynamicRecords(dynamicFormsReqDTO);
//        }
        return Result.ofSuccess(activtiyJoinDetailRespDTO);
    }

    @Override
    public Result doActivityShutDown(ActivityStateReqDTO dto) {
        Worker currWorker = getWorker();
        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(dto.getId());
        if (!publishActivity.isPresent()) {
            return Result.ofFail("活动不存在");
        }
        YcqPublishActivity ycqPublishActivity = publishActivity.get();
        if (dto.getActivityState() == 1 && LocalDateTime.now().isAfter(ycqPublishActivity.getEndTime())) {
            return Result.ofFail("当前活动时间已结束，无法进行启动");
        }
        //手动关闭后不再自动开启
        if (dto.getActivityState() == 0) {
            ycqPublishActivity.setActivityState(0);
            ycqPublishActivity.setIsManualShutdown(1);
        } else {
            ycqPublishActivity.setActivityState(1);
            ycqPublishActivity.setIsManualShutdown(0);
        }
        publishActivityRepo.save(ycqPublishActivity);
        return Result.ofSuccess(dto.getActivityState() == 0 ? "禁用成功" : "启用成功");
    }

    @Override
    public Result doQZhJoinActivity(CreateActivityReqDTO dto) {
        Worker currWorker = getWorker();
        Result announcement = null;
        locker.lock(ACTIVITY_QZH_JOIN, currWorker.workerId());
        try {
            if (dto.getActId() == null) {
                throw new BusinessException("请传入活动ID");
            }
            Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(Long.parseLong(dto.getActId()));
            if (!publishActivity.isPresent()) {
                throw new BusinessException("活动不存在");
            }
            YcqPublishActivity ycqPublishActivity = publishActivity.get();
            if (ycqPublishActivity.getActivityState() == 0) {
                throw new BusinessException("活动已经禁用,无法参加");
            }
            if (LocalDateTime.now().isBefore(ycqPublishActivity.getStartTime()) || LocalDateTime.now().isAfter(ycqPublishActivity.getEndTime())) {
                throw new BusinessException("不在活动时间范围内");
            }
            TsmWorker tsmWorker = tsmWorkerRepo.findByLoginIdAndState(currWorker.getLoginId(), 1);
            TsmOrg tsmOrg = tsmOrgRepo.findByOrgIdAndState(Long.valueOf(tsmWorker.getOrgId()), 1);
            Integer isApply = ycqPublishActivity.getIsApply();
            YcqActivityRegister activityRegister = activityRegisterRepo.findByActivityIdAndWorkerIdAndIsDeleted(Long.parseLong(dto.getActId()), currWorker.getLoginId(), 0);
            if (isApply == 1) {
                if (activityRegister == null) {
                    throw new BusinessException("您还未报名");
                }
            }
            if (ycqPublishActivity.getIsLimit() == 1 && ycqPublishActivity.getApplyerCount().equals(ycqPublishActivity.getLimitCount())) {
                throw new BusinessException("报名人数已满");
            }
            YcqQzhActivity ycqQzhActivity = ycqQzhActivityRepo.findByActIdAndCreateId(dto.getActId(), String.valueOf(currWorker.getLoginId()));
            if (ycqQzhActivity != null) {
                throw new BusinessException("您已经发布过风采");
            }
            if (ycqPublishActivity.getIsApply() == 0) {
                if (activityRegister == null) {
                    int result = publishActivityDao.updateApplyCount(ycqPublishActivity.getId());
                    if (result != 1 && Objects.nonNull(ycqPublishActivity.getLimitCount())) {
                        throw new BusinessException("当前参与人数已满，无法参与活动");
                    }
                }
            }
            announcement = ycqQzhActivityService.createAnnouncement(dto);
            if (announcement.isSuccess()) {
                if (isApply == 0 && activityRegister == null) {
                    YcqActivityRegister ycqActivityRegister = new YcqActivityRegister();
                    ycqActivityRegister.setActivityId(Long.valueOf(dto.getActId()));
                    ycqActivityRegister.setWorkerId(currWorker.getLoginId());
                    ycqActivityRegister.setPhone(currWorker.getMobile());
                    ycqActivityRegister.setRegisterTime(LocalDateTime.now());
                    ycqActivityRegister.setName(currWorker.getWorkerName());
                    ycqActivityRegister.setOrganization(tsmOrg.getOrgName());
                    ycqActivityRegister.setIsDeleted(0);
                    activityRegisterRepo.save(ycqActivityRegister);
                }
                publishActivityRepo.save(ycqPublishActivity);
            }
        } finally {
            locker.unlock();
        }
        return announcement;
    }

    @Override
    public Result doActivityJoinDetailQuery(ActivityJoinDetailQueryReqDTO dto) {
        Worker currWorker = getWorker();
        Optional<YcqActivityRegister> byId = activityRegisterRepo.findById(dto.getId());
        if (!byId.isPresent()) {
            throw new BusinessException("活动参与详情不存在");
        }
        YcqActivityRegister ycqActivityRegister = byId.get();
        YcqQzhActivity byActIdAndCreateId = ycqQzhActivityRepo.findByActIdAndCreateId(ycqActivityRegister.getActivityId().toString(), String.valueOf(ycqActivityRegister.getWorkerId()));
        if (byActIdAndCreateId == null) {
            return Result.ofSuccess("无风采记录");
        }
        OperationActivityReqDTO operationActivityReqDTO = new OperationActivityReqDTO();
        operationActivityReqDTO.setId(byActIdAndCreateId.getId());
        operationActivityReqDTO.setType(byActIdAndCreateId.getActivityType());
        operationActivityReqDTO.setRecordType(4);
        return ycqQzhActivityService.doFindDetails(operationActivityReqDTO);
    }

    @Override
    public Result doShowListQuery(PageReq<ShowListQueryReqDTO> dto) {
        Worker currWorker = getWorker();
        PageRsp<ShowListQueryRespDTO> pageRsp = ycqQzhActivityJdbcDao.showListQueryByActId(dto);
        List<ShowListQueryRespDTO> result = pageRsp.getResult();

        List<Long> ids = result.stream().map(ShowListQueryRespDTO::getId).collect(Collectors.toList());
        List<YcqQzhCover> covers = ycqQzhCoverRepo.findAllByArticleIdInAndType(ids, 3);
        result.forEach(e -> {
            List<CoverImageDto> url = new ArrayList<>();
            covers.forEach(r -> {
                if (e.getId().equals(r.getArticleId())) {
                    CoverImageDto coverImageDto = new CoverImageDto();
                    BeanUtils.copyProperties(r, coverImageDto);
                    url.add(coverImageDto);
                }
            });
            e.setCoverImageFile(url);

        });
        return Result.ofSuccess(pageRsp);
    }

    @Override
    public Result<BusinessSelectRespDTO> businessSelect() {
        Worker currWorker = getWorker();
        List<YcqAppConfigDTO> list = publishActivityDao.findSelectBusiness();
        BusinessSelectRespDTO businessSelectRespDTO = new BusinessSelectRespDTO();
        businessSelectRespDTO.setBusinessSelect(list);
        return Result.ofSuccess(businessSelectRespDTO);
    }

    @Override
    public Result<YcqQzhDynamicForm> doOtherJoinActivityForm(Long id) {
//        Worker currWorker = getWorker();
//        Result<List<YcqQzhDynamicForm>> dynamicFormByActivityId = ycqQzhDynamicFormService.findDynamicFormByActivityId(currWorker, id);
//        List<YcqQzhDynamicForm> dynamicForms = dynamicFormByActivityId.getData();
//        ArrayList<OtherActivityFormDTO> dynamicFormsReqDTOS = new ArrayList<>();
//        for (YcqQzhDynamicForm dynamicForm : dynamicForms) {
//            OtherActivityFormDTO dynamicFormsReqDTO = new OtherActivityFormDTO();
//            BeanUtils.copyProperties(dynamicForm, dynamicFormsReqDTO);
//            dynamicFormsReqDTOS.add(dynamicFormsReqDTO);
//        }
//        OtherActivityFormRespDTO otherActivityFormRespDTO = new OtherActivityFormRespDTO();
//        otherActivityFormRespDTO.setDynamicForms(dynamicFormsReqDTOS);
//        return Result.ofSuccess(otherActivityFormRespDTO);

        Worker currWorker = getWorker();
        // Result<YcqQzhDynamicForm> dynamicFormByActivityId = ycqQzhDynamicFormService.findDynamicFormByActivityId(currWorker, id);
        // YcqQzhDynamicForm dynamicForms = dynamicFormByActivityId.getData();
        //return Result.ofSuccess(dynamicForms);
        return null;
    }

    @Override
    public Result doActivityApply(Long id) {
        Worker currWorker = getWorker();
        locker.lock(ACTIVITY_APPLY, currWorker.workerId());
        try {
            TsmWorker tsmWorker = tsmWorkerRepo.findByLoginIdAndState(currWorker.getLoginId(), 1);
            TsmOrg tsmOrg = tsmOrgRepo.findByOrgIdAndState(Long.valueOf(tsmWorker.getOrgId()), 1);
            Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(id);
            if (!publishActivity.isPresent()) {
                throw new BusinessException("活动信息有误,请联系管理员");
            }
            YcqPublishActivity ycqPublishActivity = publishActivity.get();
            if (ycqPublishActivity.getActivityState() == 0) {
                throw new BusinessException("活动未启用");
            }
            if (LocalDateTime.now().isAfter(ycqPublishActivity.getEndTime())) {
                throw new BusinessException("活动已经结束");
            }
            if (ycqPublishActivity.getIsApply() == 0) {
                throw new BusinessException("该活动无需报名");
            }
            YcqActivityRegister isExist = activityRegisterRepo.findByActivityIdAndWorkerIdAndIsDeleted(id, currWorker.getLoginId(), 0);
            if (isExist != null && ycqPublishActivity.getIsApply() == 1) {
                throw new BusinessException("该活动您已经报过名");
            }
            Map<Long, Integer> queryAuditCount = activityPeopleAuditDao.queryAuditCount(Collections.singletonList(id));
            Integer orDefault = queryAuditCount.getOrDefault(id, 0);
            if (ycqPublishActivity.getLimitCount() <= orDefault){
                throw new BusinessException("当前报名人数已满");
            }
            if (isExist == null) {
                YcqActivityRegister ycqActivityRegister = new YcqActivityRegister();
                ycqActivityRegister.setRegisterTime(LocalDateTime.now());
                ycqActivityRegister.setName(currWorker.getWorkerName());
                ycqActivityRegister.setPhone(currWorker.getMobile());
                ycqActivityRegister.setIsDeleted(0);
                ycqActivityRegister.setActivityId(id);
                ycqActivityRegister.setOrganization(tsmOrg.getOrgName());
                ycqActivityRegister.setWorkerId(currWorker.getLoginId());
                activityRegisterRepo.saveAndFlush(ycqActivityRegister);
            }
        } finally {
            locker.unlock();
        }
        return Result.ofSuccess("报名成功");
    }

    @Override
    public Result doOtherJoinActivity(DynamicFormRecordReqDTO dto) {
//        Worker worker = getWorker();
//        locker.lock(ACTIVITY_OTHER_JOIN, worker.workerId());
//        try {
//            TsmWorker tsmWorker = tsmWorkerRepo.findByLoginIdAndState(worker.getLoginId(), 1);
//            TsmOrg tsmOrg = tsmOrgRepo.findByOrgIdAndState(Long.valueOf(tsmWorker.getOrgId()),1);
//            Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(Integer.valueOf(dto.getBusinessId());
//            if (!publishActivity.isPresent()) {
//                throw new BusinessException("活动不存在");
//            }
//            YcqPublishActivity ycqPublishActivity = publishActivity.get();
//            if (ycqPublishActivity.getActivityState() == 0) {
//                throw new BusinessException("活动已经禁用,无法参加");
//            }
//            if (LocalDateTime.now().isBefore(ycqPublishActivity.getStartTime()) || LocalDateTime.now().isAfter(ycqPublishActivity.getEndTime())) {
//                throw new BusinessException("不在活动时间范围内");
//            }
//
//            YcqQzhDynamicFormRecord qzhDynamicFormRecords = ycqQzhDynamicFormRecordRepo.findByActivityIdAndCreateByAndDeleted(dto.getActivityId(),(long)worker.getLoginId() , 0);
//            if (qzhDynamicFormRecords != null) {
//                throw new BusinessException("您已经参加过活动");
//            }
//            Integer isApply = ycqPublishActivity.getIsApply();
//            YcqActivityRegister activityRegister = activityRegisterRepo.findByActivityIdAndWorkerIdAndIsDeleted(Integer.valueOf(dto.getActivityId().intValue()), worker.getLoginId(), 0);
//            if (isApply == 1) {
//                if (activityRegister == null) {
//                    throw new BusinessException("您还未报名");
//                }
//            }
//            if (ycqPublishActivity.getIsLimit() == 1 && ycqPublishActivity.getApplyerCount().equals(ycqPublishActivity.getLimitCount())) {
//                throw new BusinessException("报名人数已满");
//            }
//            if (ycqPublishActivity.getIsApply()==0) {
//                if (activityRegister==null) {
//                    int result = publishActivityDao.updateApplyCount(ycqPublishActivity.getId());
//                    if (result != 1 && Objects.nonNull(ycqPublishActivity.getLimitCount())) {
//                        throw new BusinessException("当前参与人数已满，无法参与活动");
//                    }
//                }
//            }
//            Result dynamicFormRecord = ycqQzhDynamicFormService.createDynamicFormRecord(worker, dto);
//            if (dynamicFormRecord.isSuccess()) {
//                if (isApply == 0 && activityRegister==null) {
//                    YcqActivityRegister ycqActivityRegister = new YcqActivityRegister();
//                    ycqActivityRegister.setActivityId(dto.getActivityId().intValue());
//                    ycqActivityRegister.setWorkerId(worker.getLoginId());
//                    ycqActivityRegister.setPhone(worker.getMobile());
//                    ycqActivityRegister.setRegisterTime(LocalDateTime.now());
//                    ycqActivityRegister.setName(worker.getWorkerName());
//                    ycqActivityRegister.setOrganization(tsmOrg.getOrgName());
//                    ycqActivityRegister.setIsDeleted(0);
//                    activityRegisterRepo.save(ycqActivityRegister);
//                }
//                publishActivityRepo.save(ycqPublishActivity);
//            }
//        } finally {
//            locker.unlock();
//        }
        return Result.ofSuccess("成功");
    }

    @Override
    public Result<DynamicFormVO> getApplyFormByActivityId(Long id) {
        YcqDynamicForm byId1 = formRepo.getByBusinessIdAndType(id,1);
        if (byId1 == null) {
            return Result.ofFail("未配置报名表");
        }
        DynamicFormVO dynamicFormVO = new DynamicFormVO();
        dynamicFormVO.setFormId(byId1.getId());
        dynamicFormVO.setFormJson(byId1.getFormJson());
        return Result.ofSuccess(dynamicFormVO);
    }

    @Override
    @Transactional
    public Result<Boolean> submitForm(DynamicFormRecordSubmitDTO dto) {
        Worker currWorker = getWorker();
        int loginId = currWorker.getLoginId();
        //     int loginId = 9999;
        Long activityId = dto.getActivityId();
        locker.lock(ACTIVITY_APPLY, String.valueOf(loginId));
        try {
            Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(activityId);
            if (!publishActivity.isPresent()) {
                throw new BusinessException("活动不存在");
            }
            YcqPublishActivity ycqPublishActivity = publishActivity.get();
            Integer isApply = ycqPublishActivity.getIsApply();
            if (isApply == 0) {
                throw new BusinessException("该活动无需报名");
            }
            LocalDateTime now = LocalDateTime.now();
            if (ycqPublishActivity.getActivityState() == 0 && now.isAfter(ycqPublishActivity.getEndTime())) {
                throw new BusinessException("活动未启用且已结束");
            }
            if (now.isAfter(DateUtils.dateToLocalDateTime(ycqPublishActivity.getApplyEndTime()))) {
                throw new BusinessException("活动已过报名时间");
            }
//            if (LocalDateTime.now().isAfter(ycqPublishActivity.getEndTime())) {
//                throw new BusinessException("活动已经结束");
//            }
            // 是否报满 ，根据审核人数判断
            if (Objects.equals(ycqPublishActivity.getIsLimit(), POSITIVE_NUMBER)) {
                int limitCount = ycqPublishActivity.getLimitCount() == null ? 0 : ycqPublishActivity.getLimitCount();
                Integer auditCount = activityPeopleAuditDao.queryAuditCountByActivityId(activityId);
                if (auditCount >= limitCount) {
                    throw new BusinessException("报名人数已满");
                }
            }
            // 判断是否需要提交报名表
            YcqActivityApplyForm byActivityId = applyFormRepo.findByActivityId(activityId);
            if (null != byActivityId) {
                // 不为null表示需要提交报名表
                YcqActivityApplyRecord byActivityIdAndCreateId = applyRecordDao.getByActivityIdAndUserId(activityId, loginId);
                if (null != byActivityIdAndCreateId) {
                    throw new BusinessException("已提交数据，无法再次提交");
                }
                YcqActivityApplyRecord ycqActivityApplyRecord = new YcqActivityApplyRecord();
                ycqActivityApplyRecord.setActivityId(activityId);
                ycqActivityApplyRecord.setCreateId(loginId);
                ycqActivityApplyRecord.setCreateTime(new Date());
                ycqActivityApplyRecord.setRecordJson(dto.getRecordJson());
                applyRecordRepo.saveAndFlush(ycqActivityApplyRecord);
            }
            publishActivityDao.updateApplyCount(ycqPublishActivity.getId());

            // 创建报名用户
            YcqActivityPeopleAuditDTO ycqActivityPeopleAuditDTO = new YcqActivityPeopleAuditDTO(activityId, 2);
            Integer neededAudit = ycqPublishActivity.getNeededAudit();
            if (Objects.equals(1, neededAudit)) {
                ycqActivityPeopleAuditDTO.setRegistrationStatus(2);
            }
            auditService.saveYcqActivityPeopleAudit(ycqActivityPeopleAuditDTO);
        } finally {
            locker.unlock();
        }
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional
    public Result<String> doCancelApply(Long id) {
        Worker currWorker = getWorker();
        int loginId = currWorker.getLoginId();
//               int loginId = 11846;
        YcqActivityPeopleAudit byUserIdAndActivityId = activityPeopleAuditDao.queryByActivityIdAndLoginIdAndStatus(id, (long) loginId);
        if (null == byUserIdAndActivityId) {
            return Result.ofFail("用户信息不存在待审核数据，请联系管理员");
        }
        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(id);
        if (!publishActivity.isPresent()) {
            throw new BusinessException("活动不存在");
        }
        YcqPublishActivity ycqPublishActivity = publishActivity.get();
        Integer registrationStatus = byUserIdAndActivityId.getRegistrationStatus();
        Integer neededAudit = ycqPublishActivity.getNeededAudit();
        if (!Objects.equals(POSITIVE_NUMBER, neededAudit)) {
            log.error("当前活动id：{}不需要报名，触发取消报名错误：用户id：{}", id, loginId);
            return Result.ofFail("当前活动无法取消报名");
        }
        if (!Objects.equals(registrationStatus, 2)) {
            return Result.ofFail("报名申请已审核，请刷新页面重试");
        }
        // 删除报名表信息
//        ycqActivityPeopleAuditRepo.deleteByActivityIdAndUserIdAndRegistrationStatus(byUserIdAndActivityId.getUserId(),byUserIdAndActivityId.getActivityId());
        ycqActivityPeopleAuditRepo.deleteById(byUserIdAndActivityId.getId());
        //删除记录表
        Long businessBelongId = byUserIdAndActivityId.getBusinessBelongId();
        if (byUserIdAndActivityId!=null&&businessBelongId!=null){
            formRecordRepo.deleteById(businessBelongId);
        }

//        applyRecordDao.delete(id, loginId);
//
//        // 删除用户信息
//        auditService.deleteByActivityId(id);

        return Result.ofSuccess("取消成功");
    }

    @Override
    public Result<ActivityJoinDetailRespVO> getJoinDetail(ActivityJoinDetailQueryReqDTO dto) {
        Integer loginId = dto.getUserId();
        // 查询活动基础数据
        Long activityId = dto.getActivityId();
        if (null == loginId || null == activityId) {
            return Result.ofFail("参数有误");
        }
        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(activityId);
        if (!publishActivity.isPresent()) {
            throw new BusinessException("所选活动不存在");
        }
        YcqActivityPeopleAudit activityPeopleAudit = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(Long.valueOf(loginId), dto.getActivityId(), 0,1);
        ActivityJoinDetailRespVO activityJoinDetailRespVO = new ActivityJoinDetailRespVO();
        YcqPublishActivity ycqPublishActivity = publishActivity.get();
        BeanUtils.copyProperties(ycqPublishActivity, activityJoinDetailRespVO);
        activityJoinDetailRespVO.setStartTime(ycqPublishActivity.getStartTime().format(DateTimeFormatter.ofPattern(DateUtils.DATE_TIME_FORMAT_WITHOUT_SECOND)));
        activityJoinDetailRespVO.setEndTime(ycqPublishActivity.getEndTime().format(DateTimeFormatter.ofPattern(DateUtils.DATE_TIME_FORMAT_WITHOUT_SECOND)));

        // 报名表
        boolean needApply = Objects.equals(ycqPublishActivity.getIsApply(), 1);
        activityJoinDetailRespVO.setNeedApply(0);
        if(activityPeopleAudit!=null&&activityPeopleAudit.getBusinessBelongId()!=null) {
            YcqDynamicForm byBusinessIdAndType = ycqDynamicFormRepo.getByBusinessIdAndType(dto.getActivityId(), 1);
            if (!ObjectUtil.isEmpty(byBusinessIdAndType)){
                activityJoinDetailRespVO.setApplyJson(byBusinessIdAndType.getFormJson());
            }
            String valueByActiveId = dynamicFormService.getDetailsValueByActiveId(activityPeopleAudit.getBusinessBelongId());
            if (needApply && null != valueByActiveId) {
                activityJoinDetailRespVO.setNeedApply(1);
                DynamicFormRecordVO dynamicFormRecordVO = new DynamicFormRecordVO();
                dynamicFormRecordVO.setRecord(valueByActiveId);
                activityJoinDetailRespVO.setApplyRecordJson(dynamicFormRecordVO);
            }
        }
        //  默认模板数据/参与人信息
        Result<DefaultTemplateParticipationVO> defaultTemplateParticipationVOResult = defaultTemplateService.queryViewDetails(activityId, (long) loginId);
        activityJoinDetailRespVO.setParticipationDetailsVO(defaultTemplateParticipationVOResult.getData());

        // 自定义表单
        Integer replyType = ycqPublishActivity.getReplyType();
        if (Objects.equals(1, replyType)) {
            DynamicFormVO dynamicFormVO = new DynamicFormVO();
            DynamicFormRecordVO dynamicFormRecordVO = new DynamicFormRecordVO();
            String detailsByActiveId = dynamicFormService.getDetailsByActiveId(activityId, 2);

            String detailsValueByActiveId = dynamicFormService.getFeedbackByActiveId(activityId, Long.valueOf(loginId),2);
            dynamicFormRecordVO.setRecord(detailsValueByActiveId);

            dynamicFormVO.setFormJson(detailsByActiveId);
            activityJoinDetailRespVO.setFormVO(dynamicFormVO);
            activityJoinDetailRespVO.setRecordVO(dynamicFormRecordVO);
        }
        return Result.ofSuccess(activityJoinDetailRespVO);
    }

    @Override
    public List<YcqActivityTypeVO> findActivityTypeList(List<String> codeList) {
        if (CollectionUtils.isEmpty(codeList)) {
            return Collections.emptyList();
        }
        return publishActivityDao.findActivityTypeList(codeList);
    }

    @Override
    public Result<ActivityApplyRecordVO> getApplyInfo(ActivityJoinDetailQueryReqDTO dto) {
        Integer loginId = dto.getUserId();
        // 查询活动基础数据
        Long activityId = dto.getActivityId();
        if (null == loginId || null == activityId) {
            return Result.ofFail("参数有误");
        }
        ActivityApplyRecordVO activityApplyRecordVO = new ActivityApplyRecordVO();
        YcqActivityApplyForm ycqActivityApplyForm = applyFormRepo.findByActivityId(activityId);
        if (ycqActivityApplyForm != null) {
            activityApplyRecordVO.setFormJson(ycqActivityApplyForm.getFormJson());
            YcqActivityApplyRecord byActivityIdAndCreateId = applyRecordDao.getByActivityIdAndUserId(activityId, loginId);
            if (null != byActivityIdAndCreateId) {
                activityApplyRecordVO.setRecord(byActivityIdAndCreateId.getRecordJson());
            }
        }
        return Result.ofSuccess(activityApplyRecordVO);
    }

    @Override
    public void updateBrowserCount(Long id) {
        publishActivityDao.updateBrowserCount(id);
    }

    @Override
    public Result<Long> copyActivity(Long id) {
        Worker worker = getWorker();
        int loginId = worker.getLoginId();
        String workerName = worker.getWorkerName();
//        int loginId = 9999;
//        String workerName = "test1";
        Optional<YcqPublishActivity> byId = publishActivityRepo.findById(id);
        if (!byId.isPresent()) {
            log.error("当前用户编辑活动数据异常，用户id：{}，活动id：{}", loginId, id);
            throw new BusinessException("error");
        }
        YcqPublishActivity old = byId.get();
        YcqPublishActivity ycqPublishActivity = new YcqPublishActivity();
        BeanUtils.copyProperties(old, ycqPublishActivity);
        Date date = new Date();
        ycqPublishActivity.setCreateUserId(loginId);
        ycqPublishActivity.setCreateUserName(workerName);
        ycqPublishActivity.setCreateTime(date);
        ycqPublishActivity.setId(null);
        int i = ycqPublishActivity.getCopyTimes() + 1;
        ycqPublishActivity.setActivityTitle(old.getActivityTitle() + "_" + i);
        ycqPublishActivity.setCopyTimes(0);
        ycqPublishActivity.setApplyerCount(0);
        ycqPublishActivity.setAuditCount(0);
        publishActivityDao.plusTimes(id);
        YcqPublishActivity save = publishActivityRepo.saveAndFlush(ycqPublishActivity);
        Long newId = save.getId();
        // 如果有签到信息 需要查询并保存
        Integer neededSign = ycqPublishActivity.getNeededSign();
        if (Objects.equals(neededSign, POSITIVE_NUMBER)) {
            YcqActivitySign oldSign = signRepo.findByActivityId(id);
            if (null != oldSign) {
                YcqActivitySign byActivityId = new YcqActivitySign();
                BeanUtils.copyProperties(oldSign, byActivityId);
                byActivityId.setId(null);
                byActivityId.setActivityId(newId);
                byActivityId.setCreateId(loginId);
                byActivityId.setCreateTime(date);
                signRepo.saveAndFlush(byActivityId);
            }
        }
        // 如果有自定义报名表 需要保存
        if (Objects.equals(ycqPublishActivity.getIsApply(), POSITIVE_NUMBER)) {
            YcqActivityApplyForm oldApply = applyFormRepo.findByActivityId(id);
            if (null != oldApply) {
                YcqActivityApplyForm byActivityId = new YcqActivityApplyForm();
                BeanUtils.copyProperties(oldApply, byActivityId);
                byActivityId.setId(null);
                byActivityId.setActivityId(newId);
                byActivityId.setCreateId(loginId);
                byActivityId.setCreateTime(date);
                applyFormRepo.saveAndFlush(byActivityId);
            }
        }

        // 自定义反馈信息
//        if (Objects.equals(ycqPublishActivity.getReplyType(), POSITIVE_NUMBER)) {
//            YcqDynamicForm oldForm = dynamicFormRepo.getByBusinessId(id);
//            if (null != oldForm) {
//                DynamicFormTableReqDTO dynamicFormTableReqDTO = new DynamicFormTableReqDTO();
//                dynamicFormTableReqDTO.setFormJson(oldForm.getFormJson());
//                dynamicFormTableReqDTO.setTableName(ycqPublishActivity.getActivityTitle());
//                dynamicFormTableReqDTO.setBusinessId(String.valueOf(newId));
//                dynamicFormService.create(dynamicFormTableReqDTO);
//            }
//        }

        return Result.ofSuccess(newId);
    }

    @Override
    public Result<YcqActivityCauseVO> doCause(Long activityId) {
        Worker worker = getWorker();
        YcqActivityPeopleAudit byUserIdAndActivityIdAndIsDeleted = ycqActivityPeopleAuditRepo.findTopByUserIdAndActivityIdAndIsDeletedAndRegistrationStatusOrderByRegistrationTimeDesc(Long.valueOf(worker.getLoginId()), activityId, 0,0);
        if (ObjectUtil.isEmpty(byUserIdAndActivityIdAndIsDeleted)) {
            throw new BusinessException("当前用户不存在!");
        }
        YcqActivityCauseVO ycqActivityCauseVO = new YcqActivityCauseVO();
        ycqActivityCauseVO.setCause(byUserIdAndActivityIdAndIsDeleted.getCause());
        ycqActivityCauseVO.setAuditor(byUserIdAndActivityIdAndIsDeleted.getAuditor());
        ycqActivityCauseVO.setAuditorId(byUserIdAndActivityIdAndIsDeleted.getAuditorId());
        if (byUserIdAndActivityIdAndIsDeleted.getNoCause()!=null){
            List<String> valueCodeList = Arrays.asList(byUserIdAndActivityIdAndIsDeleted.getNoCause().split(","));
            List<TsmPropertyValue> byValueCodeInAndState = tsmPropertyValueRepo.findByValueCodeInAndState(valueCodeList, 1);
            if (!CollectionUtils.isEmpty(byValueCodeInAndState)){
                String valueNameString = byValueCodeInAndState.stream()
                        .map(TsmPropertyValue::getValueName)
                        .collect(Collectors.joining(","));
                ycqActivityCauseVO.setNoCause(valueNameString);
            }
        }

        if (byUserIdAndActivityIdAndIsDeleted.getRegistrationAuditTime() != null) {
            ycqActivityCauseVO.setRegistrationAuditTime(DateUtils.formatLocalDateTime(byUserIdAndActivityIdAndIsDeleted.getRegistrationAuditTime()));

        }
        return Result.ofSuccess(ycqActivityCauseVO);
    }

    @Override
    public Result<List<ActivityQrCodeDTO>> createQrCode(ActivitySignInReqVo req) {
        //校验活动
        if (ObjectUtil.isEmpty(req.getActivityId()) || ObjectUtil.isEmpty(req.getSignType())) {
            throw new BusinessException("参数错误");
        }
        YcqPublishActivity activity = publishActivityRepo.getOne(req.getActivityId());
        if (ObjectUtil.isEmpty(activity)) {
            throw new BusinessException("当前活动不存在!");
        }
        //是否需要生成签到二维码
        YcqPublishActivity ycqPublishActivity = publishActivityRepo.findByIdAndIsDeleted(req.getActivityId(), 0);
        YcqActivitySign ycqActivitySign = signRepo.findByActivityId(req.getActivityId());
        if (ObjectUtil.isEmpty(ycqPublishActivity)) {
            throw new BusinessException("该活动不存在!");
        }
        if (ObjectUtil.isEmpty(ycqActivitySign)) {
            throw new BusinessException("该活动未配置签到信息!");
        }
        if (ObjectUtil.isNotEmpty(ycqActivitySign.getSignType())) {
            if (ObjectUtil.isEmpty(ycqActivitySign.getSignInWay()) || ObjectUtil.isEmpty(ycqActivitySign.getSignInWay())) {
                throw new BusinessException("该活动未配置签到信息!");
            }
        }
        if (ycqActivitySign.getSignType() == 1 && ObjectUtil.isNotEmpty(ycqActivitySign.getSignInWay()) && ycqActivitySign.getSignInWay().equals(2)) {
            throw new BusinessException("该活动不需要二维码!");
        }
        if (ycqActivitySign.getSignType() == 2 && ObjectUtil.isNotEmpty(ycqActivitySign.getSignOutWay()) && ycqActivitySign.getSignOutWay().equals(2)) {
            throw new BusinessException("该活动不需要二维码!");
        }
        if (ycqActivitySign.getSignType() == 0 && (ObjectUtil.isNotEmpty(ycqActivitySign.getSignInWay()) && ycqActivitySign.getSignInWay().equals(2)) && (ObjectUtil.isNotEmpty(ycqActivitySign.getSignOutWay()) && ycqActivitySign.getSignOutWay().equals(2))) {
            throw new BusinessException("该活动不需要二维码!");
        }

        if (ycqActivitySign.getSignType() == 0) {
            List<ActivityQrCodeDTO> codes = new ArrayList<>();
            if (ObjectUtil.isNotEmpty(ycqActivitySign.getSignInWay()) && ycqActivitySign.getSignInWay() != 2) {
                //二维码包含的信息
                ActivitySignInReqVo qrCodeMsg1 = new ActivitySignInReqVo();
                qrCodeMsg1.setActivityId(req.getActivityId());
                qrCodeMsg1.setSignType(1);
                qrCodeMsg1.setSignInWay(ycqActivitySign.getSignInWay());
                //二维码唯一标识
                String qrIdentifier1 = UUIDUtil.getUUID();
                qrCodeMsg1.setQrId(qrIdentifier1);
                //系统标识
                qrCodeMsg1.setIdentify(ACTIVITY_SIGN_IDENTIFY);
                //下方备注文字
                List<String> comments1 = new ArrayList<>();
                comments1.add("              签到二维码");

                //生成二维码
                String qrCode1 = null;
                try {
                    qrCode1 = createQrCode(qrCodeMsg1, comments1);
                } catch (IOException | WriterException e) {
                    log.info("生成二维码失败");
                    e.printStackTrace();
                }
                ActivityQrCodeDTO dto = new ActivityQrCodeDTO();
                dto.setQrCode(qrCode1);
                dto.setFileName(ycqPublishActivity.getActivityTitle() + "签到码");
                codes.add(dto);
                ycqActivitySign.setQdCode(com.alibaba.cloud.commons.lang.StringUtils.isNotEmpty(ycqActivitySign.getQdCode()) ? ycqActivitySign.getQdCode() + "," + qrIdentifier1 : qrIdentifier1);
            }

            if (ObjectUtil.isNotEmpty(ycqActivitySign.getSignInWay()) && ycqActivitySign.getSignOutWay() != 2) {
                ActivitySignInReqVo qrCodeMsg2 = new ActivitySignInReqVo();
                qrCodeMsg2.setActivityId(req.getActivityId());
                qrCodeMsg2.setSignType(2);
                qrCodeMsg2.setSignOutWay(ycqActivitySign.getSignOutWay());
                //二维码唯一标识
                String qrIdentifier2 = UUIDUtil.getUUID();
                qrCodeMsg2.setQrId(qrIdentifier2);
                //系统标识
                qrCodeMsg2.setIdentify(ACTIVITY_SIGN_IDENTIFY);
                //下方备注文字
                List<String> comments2 = new ArrayList<>();
                comments2.add("              签退二维码");

                String qrCode2 = null;
                try {
                    qrCode2 = createQrCode(qrCodeMsg2, comments2);
                } catch (IOException | WriterException e) {
                    log.info("生成二维码失败");
                    e.printStackTrace();
                }
                ActivityQrCodeDTO dto2 = new ActivityQrCodeDTO();
                dto2.setQrCode(qrCode2);
                dto2.setFileName(ycqPublishActivity.getActivityTitle() + "签退码");
                codes.add(dto2);
                ycqActivitySign.setQtCode(com.alibaba.cloud.commons.lang.StringUtils.isNotEmpty(ycqActivitySign.getQtCode()) ? ycqActivitySign.getQtCode() + "," + qrIdentifier2 : qrIdentifier2);

            }

            //保存二维码唯一标识
            signRepo.save(ycqActivitySign);
            return Result.ofSuccess(codes);
        }


        //二维码包含的信息
        ActivitySignInReqVo qrCodeMsg = new ActivitySignInReqVo();
        qrCodeMsg.setActivityId(req.getActivityId());
        qrCodeMsg.setSignType(ycqActivitySign.getSignType());
        //二维码唯一标识
        String qrIdentifier = UUIDUtil.getUUID();
        qrCodeMsg.setQrId(qrIdentifier);

        //系统标识
        qrCodeMsg.setIdentify(ACTIVITY_SIGN_IDENTIFY);
        //下方备注文字
        //SimpleDateFormat createTime = new SimpleDateFormat("MM-dd HH:ss");
        List<String> comments = new ArrayList<>();
        if (ycqActivitySign.getSignType() == 1) {
            qrCodeMsg.setSignInWay(ycqActivitySign.getSignInWay());
            ycqActivitySign.setQdCode(com.alibaba.cloud.commons.lang.StringUtils.isNotEmpty(ycqActivitySign.getQdCode()) ? ycqActivitySign.getQdCode() + "," + qrIdentifier : qrIdentifier);
            comments.add("             签到二维码");
        }
        if (ycqActivitySign.getSignType() == 2) {
            qrCodeMsg.setSignOutWay(ycqActivitySign.getSignOutWay());
            ycqActivitySign.setQtCode(com.alibaba.cloud.commons.lang.StringUtils.isNotEmpty(ycqActivitySign.getQtCode()) ? ycqActivitySign.getQtCode() + "," + qrIdentifier : qrIdentifier);
            comments.add("              签退二维码");
        }

        //生成二维码
        String qrCode = null;
        List<ActivityQrCodeDTO> code = new ArrayList<>();
        ActivityQrCodeDTO activityQrCodeDTO = new ActivityQrCodeDTO();
        try {
            qrCode = createQrCode(qrCodeMsg, comments);
        } catch (IOException | WriterException e) {
            log.info("生成二维码失败");
            e.printStackTrace();
        }
        activityQrCodeDTO.setQrCode(qrCode);
        activityQrCodeDTO.setFileName(ycqPublishActivity.getActivityTitle() + (ycqActivitySign.getSignType() == 1 ? "签到码" : "签退码"));
        code.add(activityQrCodeDTO);
        //保存二维码唯一标识
        signRepo.save(ycqActivitySign);
        return Result.ofSuccess(code);
    }

    @Override
    public PageRsp<YcqActivityEvaluateVO> queryActivityEvaluateList(PageReq<ActivityAppQueryReqDTO> dto){
        // 查询用户信息
        int loginId = getWorker().getLoginId();
        //int loginId = 9999;
        List<YcqActivityPeopleAudit> activityIds = activityPeopleAuditDao.queryParticipateActivityList((long) loginId);
        PageRsp<YcqActivityEvaluateVO> res = new PageRsp<>();
        res.setPageSize(dto.getPageSize());
        res.setPageNo(dto.getPageNo());
        if (CollectionUtils.isEmpty(activityIds)){
            res.setResult(Collections.emptyList());
            res.next();
            return res;
        }
        // 根据用户参与的活动id 过滤活动
        List<YcqPublishActivity> activityList = publishActivityDao.pageListForMyEvaluate(dto, activityIds.stream().map(YcqActivityPeopleAudit::getActivityId).collect(Collectors.toList()));
        if (CollectionUtils.isEmpty(activityList)){
            res.setResult(Collections.emptyList());
            res.next();
            return res;
        }

        // 根据满足条件的活动id列表查询已评价
        List<Long> collect = activityList.stream().map(YcqPublishActivity::getId).collect(Collectors.toList());
        // 封装评价信息
        List<YcqActivityEvaluateVO> ycqActivityEvaluateVOS = new ArrayList<>(activityList.size());
        activityList.forEach(item -> {
            YcqActivityEvaluateVO ycqActivityEvaluateVO = new YcqActivityEvaluateVO();
            ycqActivityEvaluateVO.setId(item.getId());
            ycqActivityEvaluateVO.setActivityTitle(item.getActivityTitle());
            ycqActivityEvaluateVO.setReplyType(item.getReplyType());
            ycqActivityEvaluateVOS.add(ycqActivityEvaluateVO);
        });


        Map<Long, ParticipationDetailsVO> collect1 = defaultTemplateService.queryActivityEvaluate(collect).stream().collect(Collectors.toMap(ParticipationDetailsVO::getActivityId, Function.identity()));

        ycqActivityEvaluateVOS.forEach(item -> {
            Long id = item.getId();
            ParticipationDetailsVO participationDetailsVO = collect1.get(id);
            if (null != participationDetailsVO){
                item.setDisplayButton(0);
                item.setDetails(participationDetailsVO);
            }
        });
        // 物理分页，获取前N条数据
        List<YcqActivityEvaluateVO> collect2 = ycqActivityEvaluateVOS.stream().sorted(Comparator.comparing(YcqActivityEvaluateVO::getDisplayButton).reversed()).collect(Collectors.toList());
        List<YcqActivityEvaluateVO> page = MemoryPageUtil.getPage(collect2, dto.getPageNo(), dto.getPageSize());

        List<Long> ids = page.stream().map(YcqActivityEvaluateVO::getId).collect(Collectors.toList());
        Map<Long, List<YcqQzhCover>> headLineMap = ycqQzhCoverRepo.findAllByArticleIdInAndType(ids, 5).stream().collect(Collectors.groupingBy(YcqQzhCover::getArticleId));
        page.forEach(item -> {
            List<YcqQzhCover> ycqQzhCovers = headLineMap.get(item.getId());
            if (!CollectionUtils.isEmpty(ycqQzhCovers)) {
                YcqQzhCover ycqQzhCover = ycqQzhCovers.get(0);
                CoverImageDto coverImageDto = new CoverImageDto();
                BeanUtils.copyProperties(ycqQzhCover, coverImageDto);
                item.setHeadPicUrl(coverImageDto);
            }
        });
        res.setResult(page);
        res.setTotalNum(activityList.size());
        res.next();
        return res;
    }


    /**
     * 生成二维码
     *
     * @param qrCodeMsg
     */
    private String createQrCode(ActivitySignInReqVo qrCodeMsg, List<String> comments) throws IOException, com.google.zxing.WriterException {
        Hashtable hints = new Hashtable();
        hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.M);
        hints.put(EncodeHintType.CHARACTER_SET, CHARSET);
        hints.put(EncodeHintType.MARGIN, 1);
        BitMatrix bitMatrix = new MultiFormatWriter().encode(JsonUtil.getJsonString(qrCodeMsg), BarcodeFormat.QR_CODE, QRCODE_SIZE, QRCODE_SIZE,
                hints);
        int width = bitMatrix.getWidth();
        int height = bitMatrix.getHeight();
        int tempHeight = height + 45;

        BufferedImage image = new BufferedImage(width, tempHeight, BufferedImage.TYPE_INT_RGB);
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                image.setRGB(x, y, bitMatrix.get(x, y) ? 0xFF000000 : 0xFFFFFFFF);
            }
        }
        //添加底部文字
        addFontImage(image, comments);
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ImageIO.write(image, FORMAT_NAME, outputStream);
        String base64Image = Base64.getEncoder().encodeToString(outputStream.toByteArray());
        return base64Image;
    }

    /**
     * 底部添加文字
     *
     * @param source
     */
    private void addFontImage(BufferedImage source, List<String> comments) {
        BufferedImage textImage = strToImage(QRCODE_SIZE, 45, comments);
        Graphics2D graph = source.createGraphics();
        //开启文字抗锯齿
        graph.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

        int width = textImage.getWidth(null);
        int height = textImage.getHeight(null);

        Image src = textImage;
        graph.drawImage(src, 0, QRCODE_SIZE, width, height, null);
        graph.dispose();
        return;
    }

    /**
     * 设置文字相关参数并转图片
     *
     * @param width
     * @param height
     * @param comments
     * @return
     */
    private BufferedImage strToImage(int width, int height, List<String> comments) {
        BufferedImage textImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = (Graphics2D) textImage.getGraphics();
        //开启文字抗锯齿
        g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g2.setBackground(Color.WHITE);
        g2.clearRect(0, 0, width, height);

        g2.setPaint(Color.BLACK);
        FontRenderContext context = g2.getFontRenderContext();
        InputStream chineseFontStream = getClass().getResourceAsStream("/font/华文仿宋.ttf");
        // 创建华文中宋字体对象
        try {
            Font font = Font.createFont(Font.TRUETYPE_FONT, chineseFontStream);
            font = font.deriveFont(Font.BOLD, FONT_SIZE);
            g2.setFont(font);
            //设置底部文字
            int num = 0;
            for (String comment : comments) {
                LineMetrics lineMetrics = font.getLineMetrics(comment, context);
                float y = num + lineMetrics.getHeight();
                g2.drawString(comment, 15, y);
                num += y;
            }
        } catch (FontFormatException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return textImage;
    }

    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        return worker;
    }

//    private String getValue(Integer id, String code){
//        ValueReq valueReq = new ValueReq();
//        valueReq.setPropCode(code);
//        PageReq<ValueReq> pageRsp = new PageReq<>();
//        pageRsp.setQuery(valueReq);
//        PageRsp<TsmPropertyValue> tsmPropertyValuePageRsp = propertyJdbcDao.pageValue(pageRsp);
//        List<TsmPropertyValue> values = tsmPropertyValuePageRsp.getResult();
//        if (CollectionUtils.isEmpty(values)){
//            throw new BusinessException("未完善配置信息，请联系管理员");
//        }
//        List<TsmPropertyValue> collect = values.stream().filter(item -> Objects.equals(item.getValueCode(), String.valueOf(id))).collect(Collectors.toList());
//        if (CollectionUtils.isEmpty(collect)){
//            throw new BusinessException("未查询到该位置配置，请联系管理员");
//        }
//        return collect.get(0).getValueName();
//    }


    /**
     * 区域静态值转换
     *
     * @param locationCode 区域静态值编码
     * @param proCode      静态值ProCode
     * @param separator    分隔符
     */
    private Map<String, Object> doZoneCodeToName(String locationCode, String proCode, String separator) {
        List<TsmPropertyValue> valueList = tsmPropertyValueRepo.findAllByPropCodeAndState(
                proCode, 1);
        Map<String, Object> map = new HashMap<>();
        StringBuilder locationName = new StringBuilder();
        List<String> locationList = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(valueList)) {
            Map<String, List<TsmPropertyValue>> codeMap = valueList.stream()
                    .collect(Collectors.groupingBy(TsmPropertyValue::getValueCode));

            if (ObjectUtil.isNotEmpty(locationCode) && ObjectUtil.isNotEmpty(codeMap)) {
                String[] split = locationCode.split(separator);

                for (String localCode : split) {
                    if (codeMap.containsKey(localCode)) {
                        locationList.add(codeMap.get(localCode).get(0).getValueName());
                        locationName.append(codeMap.get(localCode).get(0).getValueName()).append(separator);
                    }
                }
                if (locationName.length() > 1){
                    locationName.deleteCharAt(locationName.length() - 1);
                }
            }
        }
        if (ObjectUtil.isNotEmpty(locationName)) {
            map.put(MAP_KEY[0], locationName.toString());
        } else {
            map.put(MAP_KEY[0], "");
        }
        if (ObjectUtil.isNotEmpty(locationList)) {
            map.put(MAP_KEY[1], locationList);
        } else {
            map.put(MAP_KEY[1], Collections.emptyList());
        }
        return map;
    }
}
