package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpException;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONObject;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.common.utils.UUIDUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQzhVideos;
import com.crunii.micro.domain.repo.main.YcqQzhVideosRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.ds.uid.util.DateUtils;
import com.crunii.micro.service.qzh.dto.video.*;
import com.crunii.micro.service.qzh.jdbc.QzhVideoDao;
import com.crunii.micro.service.qzh.service.QzhVideoService;
import com.crunii.micro.service.qzh.util.UploadFileUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Map;

import static com.crunii.micro.service.qzh.enums.QmConstants.*;


/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/10/11 16:58
 */
@Service
@Slf4j
public class QzhVideoServiceImpl implements QzhVideoService {

    @Value("${obs.s3.bucketName}")
    private String bucketName;

    @Value("${obs.s3.endPoint}")
    private String endPointHost;

    @Value("${obs.s3.accessKey}")
    private String accessKey;

    @Value("${obs.s3.accessSecret}")
    private String accessSecret;

    @Value("${qzh.appKey:1}")
    private String appKey;

    @Value("${qzh.secretKey:1}")
    private String secretKey;

    @Value("${qzh.systemName:1}")
    private String systemName;

    @Value("${qzh.userName:1}")
    private String userName;

    @Value("${qzh.uploadFileNotificationUrl:1}")
    private String uploadFileNotificationUrl;

    @Value("${qzh.getFileUrl:1}")
    private String getFileUrl;

    @Value("${qzh.notifyUrl:1}")
    private String notifyUrl;

    @Value("${qzh.webPreviewUrl:1}")
    private String webPreviewUrl;

    @Value("${qzh.pictureUrlPrefix:1}")
    private String pictureUrlPrefix;

    @Resource
    private YcqQzhVideosRepo ycqQzhVideosRepo;

    @Resource
    private QzhVideoDao videoDao;


    @Override
    public Result<NotificationVo> videoFileUpload( MultipartFile file, String fileName) {
//        Worker worker = CurrWorkerUtil.getCurrWorker();
        Worker worker =new  Worker();
        worker.setLoginId(9999);
        worker.setWorkerName("test");
        YcqQzhVideos video = new YcqQzhVideos();
        String date = DateUtils.formatDate(new Date(), "yyyyMMdd");
        final String objectName = systemName + "/" + userName + "/" + date + "/originalVideo/";
        long size = file.getSize() / 1024;
        String uuidName;
        try {
            // 解析文件后缀
            int index = fileName.lastIndexOf(".");
            String suffix;
            if (index == -1 || (suffix = fileName.substring(index + 1)).isEmpty()) {
                return Result.ofFail("非视频文件，请重新上传");
            }
            //判断是否为视频文件(简单校验一下)
            if (!isVideoFile(suffix)) {
                return Result.ofFail("非视频文件，请重新上传");
            }
            // 获取文件信息
            uuidName = UUIDUtil.getUUID() + "." + suffix;
            String objectKey = objectName + uuidName;
            Map<String, Object> map = UploadFileUtil.signUrl(accessKey, accessSecret, endPointHost, bucketName, objectKey, 60 * 24 * 1);
            String actionUrl = map.get("fileUrl").toString();
            // 上传文件
            UploadFileUtil.uploadFile(actionUrl, file);
            // 保存数据库
            video.setFileName(fileName);
            video.setFileKey(objectKey);
            video.setFileType(suffix.toLowerCase());
            video.setFileSize(String.valueOf(size));
            video.setCreatorId(worker.getLoginId());
            video.setFileDate(date);
            video.setCreateTime(new Date());
        } catch (Exception e) {
            log.info("上传文件失败，{}", e.getMessage());
            return Result.ofFail("文件上传失败");
        }
        // 上传完成后通知转码
        NoticeDto dto = new NoticeDto();
        dto.setFileName(uuidName);
        dto.setUploadDate(date);
        NotificationVo vo = uploadFileNotification(worker, dto);
        vo.setFileName(fileName);
        if (ObjectUtil.isEmpty(vo)) {
            throw new BusinessException("文件转码失败");
        }
        video.setFileId(vo.getFileId());
        // 替换成公网访问
        String pictureUrl = vo.getPictureUrl();
        String newPictureUrl = pictureUrl.replace(pictureUrlPrefix, webPreviewUrl);
        video.setPictureUrl(newPictureUrl);
        ycqQzhVideosRepo.saveAndFlush(video);
        vo.setPictureUrl(newPictureUrl);
        return Result.ofSuccess(vo);
    }

    @Override
    public NotificationVo uploadFileNotification(Worker worker, NoticeDto noticeDto) {
        NotificationVo vo =new NotificationVo();
        String url = uploadFileNotificationUrl;
        NotificationDto dto = new NotificationDto();
        dto.setAppKey(appKey);
        dto.setNonce(getNonce());
        dto.setCurTime(System.currentTimeMillis() + "");
        dto.setCheckNum(getCheckNum(dto.getNonce(), dto.getCurTime()));
        dto.setSystemName(systemName);
        dto.setUserName(userName);
        dto.setUploadDate(noticeDto.getUploadDate());
        dto.setFileName(noticeDto.getFileName());
        dto.setNotifyURL(notifyUrl);
        String json = JsonUtil.getJsonString(dto);
        String result;
        try {
            result = HttpRequest.post(url)
                    .body(json)
                    .execute().body();
        } catch (HttpException e) {
            log.info("----------->{}", e.getMessage());
            throw new BusinessException("出错了，请联系管理员");
        }
        JSONObject jsonObject = new JSONObject(result);
        log.info("------------{}", jsonObject);
        String msg = (String) jsonObject.get("msg");
        if (!"OK".equals(msg)) {
            log.info("请求参数{}", json);
            throw new BusinessException(jsonObject.get("msg").toString());
        }
        vo.setFileId((String) jsonObject.get("fileID"));
        vo.setPictureUrl((String) jsonObject.get("pictureUrl"));
        return vo;
    }

    @Override
    public Result<String> getFileUrl(FileDto fileDto) {
                Worker worker = CurrWorkerUtil.getCurrWorker();
//        Worker worker =new  Worker();
//        worker.setLoginId(9999);
//        worker.setWorkerName("test");
        YcqQzhVideos video = ycqQzhVideosRepo.findByFileIdAndDeleted(fileDto.getFileId(), DELETED_0);
        if (ObjectUtil.isEmpty(video)) {
            return Result.ofFail(FILE_STATUS_MGS_ONE);
        }
        // 获取视频转化状态
        if (video.getFileStatus() == STATUS_INVALID) {
            return Result.ofFail(FILE_STATUS_MGS_TWO);
        }
        String url = getFileUrl;
        GetFileDto dto = new GetFileDto();
        dto.setAppKey(appKey);
        dto.setNonce(getNonce());
        dto.setCurTime(System.currentTimeMillis() + "");
        dto.setCheckNum(getCheckNum(dto.getNonce(), dto.getCurTime()));
        dto.setSystemName(systemName);
        dto.setUserName(userName);
        dto.setUploadDate(video.getFileDate());
        dto.setFileID(video.getFileId());
        dto.setFileType(fileDto.getFileType());
        String json = JsonUtil.getJsonString(dto);
        String result;
        try {
            result = HttpRequest.post(url)
                    .body(json)
                    .execute().body();
        } catch (HttpException e) {
            throw new BusinessException("出错了，请联系管理员");
        }
        JSONObject jsonObject = new JSONObject(result);
        String msg = (String) jsonObject.get("msg");
        if (!"OK".equals(msg)) {
            log.info("请求参数{},错误信息{}", json, jsonObject.get("msg").toString());
            throw new BusinessException("获取失败，请联系管理员");
        }
        // 替换成公网访问
        String oldFileUrl = (String) jsonObject.get("fileUrl");
        String fileUrl = oldFileUrl.replace(pictureUrlPrefix, webPreviewUrl);
        return Result.ofSuccess(fileUrl);
    }

    @Override
    public Result<String> queryRealTimeVideoAddress(Worker worker, String fileId) {
        YcqQzhVideos video = ycqQzhVideosRepo.findByFileIdAndDeleted(fileId, DELETED_0);
        // 根据fileNumber、tableName获取信息表记录
        if (ObjectUtil.isEmpty(video)) {
            return Result.ofFail(FILE_STATUS_MGS_ONE);
        }
        // 获取视频转化状态
        if (video.getFileStatus() == STATUS_INVALID) {
            return Result.ofFail(FILE_STATUS_MGS_TWO);
        }
        String url = getFileUrl;
        GetFileDto dto = new GetFileDto();
        dto.setAppKey(appKey);
        dto.setNonce(getNonce());
        dto.setCurTime(System.currentTimeMillis() + "");
        dto.setCheckNum(getCheckNum(dto.getNonce(), dto.getCurTime()));
        dto.setSystemName(systemName);
        dto.setUserName(userName);
        dto.setUploadDate(video.getFileDate());
        dto.setFileID(fileId);
        dto.setFileType(VIDEO_1080);
        String json = JsonUtil.getJsonString(dto);
        String result;
        try {
            result = HttpRequest.post(url)
                    .body(json)
                    .execute().body();
        } catch (HttpException e) {
            throw new BusinessException("出错了，请联系管理员");
        }
        JSONObject jsonObject = new JSONObject(result);
        String msg = (String) jsonObject.get("msg");
        if (!"OK".equals(msg)) {
            log.info("请求参数{}", jsonObject.get("msg").toString());
            Result.ofFail("获取视频失败");
        }
        // 替换成公网访问
        String oldFileUrl = (String) jsonObject.get("fileUrl");
        String fileUrl = oldFileUrl.replace(pictureUrlPrefix, webPreviewUrl);
        return Result.ofSuccess(fileUrl);
    }

    @Override
    public Result<String> updateFileStatus(VideoDto videoDto) {
        Result<String> result = new Result<>();
        videoDao.updateFileStatus(videoDto);
        result.setCode("0");
        result.setMsg("OK");
        return result;
    }

    @Override
    public Result<String> downloadVideo(String fileId) {
        YcqQzhVideos video = ycqQzhVideosRepo.findByFileIdAndDeleted(fileId, DELETED_0);
        if (ObjectUtil.isEmpty(video)) {
            return Result.ofFail("视频不存在");
        }
        Map<String, Object> returnMap = UploadFileUtil.downloadUrl(accessKey, accessSecret, endPointHost, bucketName, video.getFileKey(), 60 * 24 * 7);
        String fileUrl = (String) returnMap.get("fileUrl");
        if (StrUtil.isEmpty(fileUrl)) {
            return Result.ofFail("获取视频失败");
        }
        return Result.ofSuccess(fileUrl);
    }

    private String getCheckNum(String nonce, String curTime) {
        return SecureUtil.sha1(nonce + secretKey + curTime);
    }

    public String getNonce() {
        int randomNumber = (int) (Math.random() * 1000000);
        return String.format("%06d", randomNumber);
    }

    public boolean isVideoFile(String suffix) {
        String extension = suffix.toLowerCase();
        return "mp4".equalsIgnoreCase(extension)
                || "avi".equalsIgnoreCase(extension)
                || "mov".equalsIgnoreCase(extension)
                || "wmv".equalsIgnoreCase(extension)
                || "flv".equalsIgnoreCase(extension);
    }
}
