package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.excel.ExcelUtil;

import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.message.PushMsgTemplateReqBo;
import com.crunii.micro.domain.dto.message.SendMessageTaskReqDTO;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.entity.main.YcqActivityPeopleAudit;
import com.crunii.micro.domain.entity.main.YcqPublishActivity;
import com.crunii.micro.domain.repo.main.PublishActivityRepo;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;
import com.crunii.micro.domain.repo.main.YcqActivityPeopleAuditRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.MessageCommonService;
import com.crunii.micro.service.qzh.api.domain.dto.activity.RegistrationAuditDTO;
import com.crunii.micro.service.qzh.api.domain.dto.RegistrationAuditVO;
import com.crunii.micro.service.qzh.api.enums.ActivityButtonStateEnum;
import com.crunii.micro.service.qzh.dto.DefaultTemplateVO;
import com.crunii.micro.service.qzh.dto.YcqActivityPeopleAuditDTO;
import com.crunii.micro.service.qzh.enums.ActivityPushMsgConstants;
import com.crunii.micro.service.qzh.jdbc.YcqActivityPeopleAuditJdbcDao;
import com.crunii.micro.service.qzh.jdbc.activitypublish.PublishActivityDao;
import com.crunii.micro.service.qzh.service.RegistrationAuditService;
import com.crunii.micro.service.qzh.service.SendMessageTaskService;

import lombok.RequiredArgsConstructor;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;


/**
 * @author lmlksy
 * @date 2023/11/20 10:35
 */
@Service
@Slf4j
public class RegistrationAuditServiceImpl implements RegistrationAuditService {

    @Autowired
    YcqActivityPeopleAuditRepo ycqActivityPeopleAuditRepo;

    @Autowired
    YcqActivityPeopleAuditJdbcDao ycqActivityPeopleAuditJdbcDao;

    @Autowired
    TsmWorkerRepo tsmWorkerRepo;

    @Autowired
    PublishActivityDao publishActivityDao;

    @Autowired
    MessageCommonService messageCommonService;

    @Autowired
    PublishActivityRepo publishActivityRepo;


    @Override
    public Result<PageRsp<RegistrationAuditVO>> queryList(PageReq<RegistrationAuditDTO> req) {
        PageRsp<RegistrationAuditVO> registrationAuditVOPageRsp = ycqActivityPeopleAuditJdbcDao.queryList(req);
        return Result.ofSuccess(registrationAuditVOPageRsp);
    }

    @Override
    public void exportRegistrationAuditExcel(RegistrationAuditDTO dto) {
        //修改为根据活动id导出
        List<YcqActivityPeopleAudit> ycqActivityPeopleAudits = ycqActivityPeopleAuditRepo.findByActivityIdAndIsDeleted(dto.getActivityId(),0);
        List<RegistrationAuditVO> result = new ArrayList<>();
        if (!CollectionUtils.isEmpty(ycqActivityPeopleAudits)){
            for (YcqActivityPeopleAudit ycqActivityPeopleAudit : ycqActivityPeopleAudits) {
                RegistrationAuditVO registrationAuditVO = new RegistrationAuditVO();
                registrationAuditVO.setRegistrationTime(DateUtils.formatLocalDateTime(ycqActivityPeopleAudit.getRegistrationTime()));
                BeanUtils.copyProperties(ycqActivityPeopleAudit,registrationAuditVO);
                result.add(registrationAuditVO);
            }
            result.sort(Comparator.comparing(RegistrationAuditVO::getRegistrationStatus).reversed());
        }
        try {
            ExcelUtil.writeSingleExcel("报名审核人员表","sheet",result,RegistrationAuditVO.class);
        } catch (IOException e) {
            throw new BusinessException("导出失败！");
        }
    }

    @Override
    public Result doBatchAudit(RegistrationAuditDTO dto) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException("获取当前登录用户失败");
        }
        if (CollectionUtils.isEmpty(dto.getIds())){
            return Result.ofFail("id不能为空！");
        }
        if (dto.getAuditState()==null){
            return Result.ofFail("审核状态不能为空！");
        }
        List<YcqActivityPeopleAudit> ycqActivityPeopleAudits = ycqActivityPeopleAuditRepo.findByIdIn(dto.getIds());
        //添加判断 只能勾选待审核数据
        // 检查列表中是否存在 registrationStatus 为 0 或 1 的情况
        boolean hasInvalidStatus = ycqActivityPeopleAudits.stream()
                .anyMatch(audit -> audit.getRegistrationStatus() != null && (audit.getRegistrationStatus() == 0 || audit.getRegistrationStatus() == 1));
        if (hasInvalidStatus) {
            return Result.ofFail("只能选择待审核数据！");
        }
        YcqPublishActivity ycqPublishActivity = publishActivityRepo.findByIdAndIsDeleted(ycqActivityPeopleAudits.get(0).getActivityId(), 0);

        for (YcqActivityPeopleAudit e : ycqActivityPeopleAudits) {
            if (e.getCause()!=null){
            if (e.getCause().length() > 255) {
                return Result.ofFail("审核原因不能超过255字符！");
            }
            }
            //设置报名状态
            e.setRegistrationStatus(dto.getAuditState());
            //设置流程状态为不通过
            if (dto.getAuditState()==0){
                if (!CollectionUtils.isEmpty(dto.getNoCause())) {
                    String concatenatedString = String.join(",", dto.getNoCause());
                    e.setNoCause(concatenatedString);
                }

                e.setCause(dto.getCause());
                e.setProcessState(ActivityButtonStateEnum.REJECT.getCode());
                e.setAuditor(worker.getWorkerName());
                e.setAuditorId(Long.valueOf(worker.getLoginId()));
            }else {
                boolean b = publishActivityDao.updateAuditCount(e.getActivityId(), ycqActivityPeopleAudits.size());
                if (!b){
                    return Result.ofFail("审核不通过！活动已爆满");
                }
                e.setCause(dto.getCause());
                e.setProcessState(ActivityButtonStateEnum.SIGN_IN_NOW.getCode());
                e.setAuditor(worker.getWorkerName());
                e.setAuditorId(Long.valueOf(worker.getLoginId()));

                PushMsgTemplateReqBo pushMsgTemplateReqBo = new PushMsgTemplateReqBo();
                pushMsgTemplateReqBo.setSystemName(ActivityPushMsgConstants.SYSTEM_NAME);
                pushMsgTemplateReqBo.setSceneName(ActivityPushMsgConstants.SCENE_NAME_ACTIVITY_REGISTERED);
                pushMsgTemplateReqBo.setTitle(ActivityPushMsgConstants.TITLE_ACTIVITY_REGISTERED);
                Map<String, Object> params = new HashMap<>(16);
                params.put("activity", ycqPublishActivity.getActivityTitle());
                params.put("examine", "通过");
                pushMsgTemplateReqBo.setMap(params);
                log.info("活动报名审核开始调用远程pushMag");
                messageCommonService.pushMsg(pushMsgTemplateReqBo,e.getUserId());
            }
            e.setRegistrationAuditTime(LocalDateTime.now());
        }

        ycqActivityPeopleAuditRepo.saveAll(ycqActivityPeopleAudits);
        return Result.ofSuccessMsg("提交成功！");
    }

    @Override
    public void saveYcqActivityPeopleAudit(YcqActivityPeopleAuditDTO dto) {
//ActivityButtonStateEnum
        Worker tsmWorker = CurrWorkerUtil.getCurrWorker();
//        Worker worker1 = new Worker();
//        worker1.setLoginId(9999);
//        tsmWorker=worker1;
        //查询用户是否存在
        YcqActivityPeopleAudit peopleAudit = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(Long.valueOf(tsmWorker.getLoginId()), dto.getActivityId(),0,1);
        YcqActivityPeopleAudit peopleAudit2 = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(Long.valueOf(tsmWorker.getLoginId()), dto.getActivityId(),0,2);

        if (peopleAudit!=null){
            throw new BusinessException("用户已经报名成功！");
        }
        if (peopleAudit2!=null){
            throw new BusinessException("用户待审核状态中还需审核！");
        }
            YcqActivityPeopleAudit ycqActivityPeopleAudit = new YcqActivityPeopleAudit();
            ycqActivityPeopleAudit.setIsDeleted(0);
            ycqActivityPeopleAudit.setProcessState(dto.getProcessState());
            ycqActivityPeopleAudit.setUserId(Long.valueOf(tsmWorker.getLoginId()));
            ycqActivityPeopleAudit.setUserName(tsmWorker.getWorkerName());
            ycqActivityPeopleAudit.setPhoneNumber(tsmWorker.getMobile());
            ycqActivityPeopleAudit.setCountFalse(0);
            ycqActivityPeopleAudit.setCountTrue(0);
            ycqActivityPeopleAudit.setRegistrationStatus(dto.getRegistrationStatus());
            ycqActivityPeopleAudit.setRegistrationTime(LocalDateTime.now());
            ycqActivityPeopleAudit.setActivityId(dto.getActivityId());
            ycqActivityPeopleAudit.setContactMethod(tsmWorker.getMobile());
            ycqActivityPeopleAudit.setSex(dto.getSex());
            ycqActivityPeopleAudit.setBusinessBelongId(dto.getBusinessBelongId());
            ycqActivityPeopleAuditRepo.saveAndFlush(ycqActivityPeopleAudit);
    }


    public void saveYcqActivityPeopleAudit2(YcqActivityPeopleAuditDTO dto) {
//ActivityButtonStateEnum
        Worker tsmWorker = CurrWorkerUtil.getCurrWorker();
//        Worker worker1 = new Worker();
//        worker1.setLoginId(9999);
//        tsmWorker=worker1;
        //查询用户是否存在
        YcqActivityPeopleAudit ycqActivityPeopleAudit1 = ycqActivityPeopleAuditRepo.findFirstByUserIdAndActivityIdAndIsDeletedAndRegistrationStatus(Long.valueOf(tsmWorker.getLoginId()), dto.getActivityId(), 0, 1);
        if (ycqActivityPeopleAudit1!=null){
            ycqActivityPeopleAuditRepo.updateProcessStateByActivityIdAndUserId(dto.getProcessState(), dto.getActivityId(), Long.valueOf(tsmWorker.getLoginId()));
        }else {
            YcqActivityPeopleAudit ycqActivityPeopleAudit = new YcqActivityPeopleAudit();
            ycqActivityPeopleAudit.setIsDeleted(0);
            ycqActivityPeopleAudit.setProcessState(dto.getProcessState());
            ycqActivityPeopleAudit.setUserId(Long.valueOf(tsmWorker.getLoginId()));
            ycqActivityPeopleAudit.setUserName(tsmWorker.getWorkerName());
            ycqActivityPeopleAudit.setPhoneNumber(tsmWorker.getMobile());
            ycqActivityPeopleAudit.setCountFalse(0);
            ycqActivityPeopleAudit.setCountTrue(0);
            ycqActivityPeopleAudit.setRegistrationStatus(dto.getRegistrationStatus());
            ycqActivityPeopleAudit.setRegistrationTime(LocalDateTime.now());
            ycqActivityPeopleAudit.setActivityId(dto.getActivityId());
            ycqActivityPeopleAudit.setContactMethod(tsmWorker.getMobile());
            ycqActivityPeopleAudit.setSex(dto.getSex());
            ycqActivityPeopleAudit.setBusinessBelongId(dto.getBusinessBelongId());
            ycqActivityPeopleAuditRepo.saveAndFlush(ycqActivityPeopleAudit);
        }
    }

    /**
     *
     * @param activityId 活动id
     */
    @Override
    public void deleteByActivityId(Long activityId){
        Worker tsmWorker = CurrWorkerUtil.getCurrWorker();
        ycqActivityPeopleAuditRepo.deleteByActivityIdAndUserId(activityId, Long.valueOf(tsmWorker.getLoginId()));
    }

}
