package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import com.crunii.micro.common.dto.Result;

import com.crunii.micro.domain.entity.main.YcqSngHistory;
import com.crunii.micro.domain.entity.main.YcqSngSchool;
import com.crunii.micro.domain.entity.main.YcqSngStudent;
import com.crunii.micro.domain.repo.main.YcqSngHistoryReqo;
import com.crunii.micro.domain.repo.main.YcqSngSchoolReqo;
import com.crunii.micro.domain.repo.main.YcqSngStudentReqo;
import com.crunii.micro.service.qzh.api.domain.dto.History;
import com.crunii.micro.service.qzh.api.domain.dto.PutStudentDTO;
import com.crunii.micro.service.qzh.api.domain.dto.StudentInfo;

import com.crunii.micro.service.qzh.service.SNGService;
import com.crunii.micro.service.qzh.utils.Sm2Util;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author lmlkyc
 * @date 2023-09-26 20:44
 */
@Service
@Slf4j
public class SNGServiceImpl implements SNGService {

    private static String sm2_publicKey = "MFkwEwYHKoZIzj0CAQYIKoEcz1UBgi0DQgAEjUNP4NE2S6v+DhYUQ1u92ZYFuoAOvMMP/zlxWfTKS1l+mPfSpxph0rNec84wvMsR/FevXnOZM1GpgKZPKVCY1A==";
    private static String sm2_privateKey = "MIGTAgEAMBMGByqGSM49AgEGCCqBHM9VAYItBHkwdwIBAQQgvIbUXmqNHzwm9utXUg8zscFXAbEDM31dyKT+fhs7FZOgCgYIKoEcz1UBgi2hRANCAASNQ0/g0TZLq/4OFhRDW73ZlgW6gA68ww//OXFZ9MpLWX6Y99KnGmHSs15zzjC8yxH8V69ec5kzUamApk8pUJjU";


    @Autowired
    private YcqSngHistoryReqo ycqSngHistoryReqo;

    @Autowired
    private YcqSngSchoolReqo ycqSngSchoolReqo;

    @Autowired
    private YcqSngStudentReqo ycqSngStudentReqo;

    @Override
    public Result<Object> saveStudentHistoryFromSng(PutStudentDTO putStudentDTO) {
        log.info("putStudentDTO {} {} {}", putStudentDTO.getSysCode(), putStudentDTO.getTransactionId(), putStudentDTO.getEncode());
        Result<Object> result = checkAuthCodeParam(putStudentDTO);

        if (result != null) {
            return result;
        }
        // 根据 sysCode 值检查其有效性，如果不正确则返回错误
//		Sm2SecretProperty sm2SecretProperty = properties.get(putStudentDTO.getSysCode());
        if (!putStudentDTO.getSysCode().equals("CQSNG")) {
            return new Result<>(Boolean.FALSE, "2002", "sysCode不正确", null);
        }

        String jsonStr = "";

        try {
            jsonStr = Sm2Util.decrypt(sm2_privateKey, false, putStudentDTO.getEncode());
            log.info("encode解密参数:" + jsonStr);
        } catch (Exception e) {
            log.error("解密失败", e);
        }
        String batchCodeId = "";
        try {
            // 使用 Jackson 库来解析 JSON 字符串
            ObjectMapper objectMapper = new ObjectMapper();
            JsonNode jsonNode = objectMapper.readTree(jsonStr);

            // 获取 BatchCodeId 和 Count 参数的值
            batchCodeId = jsonNode.get("BatchCodeId").asText();
            int count = jsonNode.get("Count").asInt();

            // 获取 StudentInfos 数组节点
            JsonNode studentInfosArrayNode = jsonNode.get("StudentInfos");
            if (studentInfosArrayNode != null && studentInfosArrayNode.isArray()) {
                // 遍历 StudentInfos 数组，将每个元素解析为学生信息对象（StudentInfo）
                List<StudentInfo> studentInfoList = new ArrayList<>();

                for (JsonNode studentInfoNode : studentInfosArrayNode) {
                    StudentInfo studentInfo = objectMapper.treeToValue(studentInfoNode, StudentInfo.class);
                    studentInfoList.add(studentInfo);
                }
                for (StudentInfo studentInfo : studentInfoList) {

                    YcqSngStudent byVersionHash = ycqSngStudentReqo.findByStudentIdentity(studentInfo.getStudentIdentity());
                    if (byVersionHash != null) {
                        if (!byVersionHash.getVersionHash().equals(studentInfo.getVersionHash())) {
                            //版本不一致,就进行更新学生信息
                            if (studentInfo.getRegisterDate() != null) {
                                byVersionHash.setRegisterDate(LocalDateTimeUtil.parse(studentInfo.getRegisterDate(), "yyyy-MM-dd'T'HH:mm:ss"));
                            }

                            if (studentInfo.getLastAction() != null) {
                                byVersionHash.setLastAction(LocalDateTimeUtil.parse(studentInfo.getLastAction(), "yyyy-MM-dd'T'HH:mm:ss"));
                            }
                            byVersionHash.setBatchCodeId(batchCodeId);
                            BeanUtils.copyProperties(studentInfo, byVersionHash);
							log.info("===================byVersionHash+修改" + byVersionHash);
                            ycqSngStudentReqo.save(byVersionHash);
                            //更新学生学校信息
                            YcqSngSchool ycqSngSchool = ycqSngSchoolReqo.findByYcqSngStudentId(byVersionHash.getId());
                            BeanUtils.copyProperties(studentInfo.getSchoolInfo(), ycqSngSchool);
							log.info("===================ycqSngSchool-修改"+ycqSngSchool);
                            ycqSngSchoolReqo.save(ycqSngSchool);
                            //更新学生活动信息
                            ycqSngHistoryReqo.setDelFlagTo1ByYcqSngStudentId(byVersionHash.getId());
                            List<YcqSngHistory> ycqSngHistory = new ArrayList<>();
                            for (History source : studentInfo.getJoinHistories()) {
                                YcqSngHistory target = new YcqSngHistory();
                                if (source.getStart() != null) {
                                    target.setStart(LocalDateTimeUtil.parse(source.getStart(), "yyyy-MM-dd'T'HH:mm:ss"));
                                }
                                BeanUtils.copyProperties(source, target);
                                target.setYcqSngStudentId(byVersionHash.getId());
                                target.setDelFlag(0);
                                ycqSngHistory.add(target);
                            }
                            log.info("===================ycqSngHistory+修改" + ycqSngHistory.size());
                            ycqSngHistoryReqo.saveAll(ycqSngHistory);
                        }
                    } else {
                        //存储到YcqSngStudent
                        YcqSngStudent ycqSngStudent = new YcqSngStudent();
                        if (studentInfo.getRegisterDate() != null) {
                            ycqSngStudent.setRegisterDate(LocalDateTimeUtil.parse(studentInfo.getRegisterDate(), "yyyy-MM-dd'T'HH:mm:ss"));
                        }

                        if (studentInfo.getLastAction() != null) {
                            ycqSngStudent.setLastAction(LocalDateTimeUtil.parse(studentInfo.getLastAction(), "yyyy-MM-dd'T'HH:mm:ss"));
                        }
                        ycqSngStudent.setBatchCodeId(batchCodeId);
                        BeanUtils.copyProperties(studentInfo, ycqSngStudent);
						log.info("===================ycqSngStudent"+ycqSngStudent);
                        YcqSngStudent save = ycqSngStudentReqo.save(ycqSngStudent);
                        //存储到YcqSngStudent
                        YcqSngSchool ycqSngSchool = new YcqSngSchool();
                        BeanUtils.copyProperties(studentInfo.getSchoolInfo(), ycqSngSchool);
                        ycqSngSchool.setYcqSngStudentId(save.getId());
                        log.info("===================ycqSngSchool" + ycqSngSchool);
                        ycqSngSchoolReqo.save(ycqSngSchool);

                        //存储到YCQ_SNG_HISTORY
                        List<YcqSngHistory> ycqSngHistory = new ArrayList<>();
                        for (History source : studentInfo.getJoinHistories()) {
                            YcqSngHistory target = new YcqSngHistory();
                            if (source.getStart() != null) {
                                target.setStart(LocalDateTimeUtil.parse(source.getStart(), "yyyy-MM-dd'T'HH:mm:ss"));
                            }
                            BeanUtils.copyProperties(source, target);
                            target.setYcqSngStudentId(save.getId());
                            target.setDelFlag(0);
                            ycqSngHistory.add(target);
                        }
                        log.info("===================新增======" + ycqSngHistory.size());
                        ycqSngHistoryReqo.saveAll(ycqSngHistory);
                    }
                }
            }
            // 这里可以返回成功的结果，或者根据处理结果返回相应的结果
            return new Result(Boolean.TRUE, "0", "推送成功!", count);
        } catch (Exception e) {
            // 在解析或转换过程中可能会抛出异常
            e.printStackTrace();
            // 返回相应的错误信息
            return new Result<>(Boolean.FALSE, "-1", "推送失败!批次号:{}", batchCodeId);
        }
    }

    private Result<Object> checkAuthCodeParam(PutStudentDTO putStudentDTO) {
        // 检查是否缺少 encode 参数
        if (putStudentDTO.getEncode() == null || putStudentDTO.getEncode().isEmpty()) {
            return new Result(Boolean.FALSE, "1002", "缺少参数encode", null);
        }

        // 检查是否缺少 sysCode 参数
        if (putStudentDTO.getSysCode() == null || putStudentDTO.getSysCode().isEmpty()) {
            return new Result<>(Boolean.FALSE, "2001", "缺少参数sysCode", null);
        }

        return null;
    }
}
