package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.dto.message.SendMessageTaskReqDTO;
import com.crunii.micro.domain.entity.main.MessageSendTask;
import com.crunii.micro.domain.repo.main.MessageSendTaskRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.service.SendMessageTaskService;
import lombok.extern.slf4j.Slf4j;
import org.assertj.core.util.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: zwx
 * @CreateTime: 2023-10-27  10:30:40
 * @Description: TODO
 */
@Slf4j
@Service
public class SendMessageTaskServiceImpl implements SendMessageTaskService {

    private static final String MESSAGE_PUBLICKEY = "MFkwEwYHKoZIzj0CAQYIKoEcz1UBgi0DQgAEGNYRbNf0h7wQ0rmpFHEWJbSut7wnHvzUEYvEov3TB7bob5FfTPeMGlLxqENbV4z0Jk1qqO9u5wQ4L2hE+QkiXA==";

    private static final String MESSAGE_PRIVATEKEY = "MIGTAgEAMBMGByqGSM49AgEGCCqBHM9VAYItBHkwdwIBAQQgZLKg2Xx93kyDvacGLZGY2r3Wpu7jUwTo2LXbi7QYfSGgCgYIKoEcz1UBgi2hRANCAAQY1hFs1/SHvBDSuakUcRYltK63vCce/NQRi8Si/dMHtuhvkV9M94waUvGoQ1tXjPQmTWqo727nBDgvaET5CSJc";

    @Resource
    MessageSendTaskRepo messageSendTaskRepo;
    @Override
    public Result saveMessageTask(SendMessageTaskReqDTO taskReqDTO) {
        log.info("消息调用:",taskReqDTO.toString());
        Worker currWorker = CurrWorkerUtil.getCurrWorker();

//        String decryptData = null;
//        try{
//            decryptData = Sm2Util.decrypt(MESSAGE_PRIVATEKEY, false, param);
//            log.info("解密后的数据:"+decryptData);
//        } catch (Exception e) {
//            log.error("解密失败:"+e);
//        }

        try {
//            SendMessageTaskReqDTO taskReqDTO = JSONUtil.toBean(decryptData, SendMessageTaskReqDTO.class);
//            log.info("json转换后的数据:"+taskReqDTO.toString());

            //入参校验
            String errMsg = argumentValidated(taskReqDTO);
            if (errMsg !=null) {
                return Result.ofFail(errMsg);
            }
            MessageSendTask messageSendTask = new MessageSendTask();
            BeanUtils.copyProperties(taskReqDTO,messageSendTask);
            messageSendTask.setSendState(0);
            if(ObjectUtil.isNotEmpty(currWorker)){

                messageSendTask.setCreateUser(currWorker.getLoginId());
            }else {
                messageSendTask.setCreateUser(0);

            }
            messageSendTask.setCreateTime(LocalDateTime.now());
            messageSendTask.setFillData(taskReqDTO.getFillData());
            log.info("消息任务保存信息：-------->{}"+messageSendTask);
            messageSendTaskRepo.saveAndFlush(messageSendTask);
        } catch (Exception e) {
            log.error("保存失败:"+ e);
            throw new RuntimeException(e);
        }
        return Result.ofSuccess("成功");
    }



    private String argumentValidated(SendMessageTaskReqDTO dto) {
        if (dto.getScopeType()==null) {
            return "发送范围必传";
        }
        List<Integer> scopeType = Lists.newArrayList(1, 2, 3, 4);
        if (!scopeType.contains(dto.getScopeType())) {
            return "发送范围类型不存在";
        }
        if (dto.getScopeType()!=1 && StrUtil.isBlank(dto.getScopeData())) {
            return "非全用户范围数据必传";
        }
        if (StrUtil.isBlank(dto.getSendWay())) {
            return "发送方式必传";
        }
        Set<Integer> sendWayDto = Arrays.stream(dto.getSendWay().split(",")).map(Integer::new).collect(Collectors.toSet());
        List<Integer> sendWay = Lists.newArrayList(1, 2);
        if (!new HashSet<>(sendWay).containsAll(sendWayDto)) {
            return "发送方式类型不存在";
        }
        if (StrUtil.isBlank(dto.getMessageType())) {
            return "推送终端必传";
        }
        Set<Integer> messageTypeDto = Arrays.stream(dto.getMessageType().split(",")).map(Integer::new).collect(Collectors.toSet());
        List<Integer> messageType = Lists.newArrayList(1, 2, 3, 4);
//        if (sendWayDto.size()==1 && messageTypeDto.contains(1) && StringUtils.isNotBlank(dto.getMessageType())) {
//            return "发送方式只为短信无需传推送终端";
//        }
        if (!new HashSet<>(messageType).containsAll(messageTypeDto)) {
            return "推送终端不存在";
        }
        if (StrUtil.isBlank(dto.getMessageContent())) {
            return "消息内容必传";
        }
        if (StrUtil.isBlank(dto.getDataSource())) {
            return "消息来源必传";
        }
        if (StrUtil.isBlank(dto.getMessageClassify())) {
            return "消息分类必传";
        }
        if (Objects.isNull(dto.getTemplateId())) {
            return "模板ID必传";
        }
        return null;
    }
}
