package com.crunii.micro.service.qzh.service.impl;


import com.alibaba.excel.util.CollectionUtils;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.entity.main.TsmProperty;
import com.crunii.micro.domain.entity.main.TsmPropertyValue;
import com.crunii.micro.domain.repo.main.TsmPropertyRepo;
import com.crunii.micro.domain.repo.main.TsmPropertyValueRepo;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqActivityAuditReasonDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqActivityAuditReasonVO;
import com.crunii.micro.service.qzh.service.YcqActivityAuditReasonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;


/**
 * @author lmlksy
 * @date 2024/1/5 15:04
 */
@Service
public class YcqActivityAuditReasonServiceImpl implements YcqActivityAuditReasonService {

    @Autowired
    TsmPropertyValueRepo tsmPropertyValueRepo;

    @Autowired
    TsmPropertyRepo tsmPropertyRepo;
    @Override
    public Result<List<YcqActivityAuditReasonVO>> pageList() {
        List<TsmPropertyValue> tsmPropertyValueList = tsmPropertyValueRepo.findAllByPropCodeAndState("ACT_ACTIVITY_QFJH", 1);

        List<YcqActivityAuditReasonVO> activityAuditReasonVOList = new ArrayList<>();
        for (TsmPropertyValue tsmPropertyValue : tsmPropertyValueList) {
            YcqActivityAuditReasonVO activityAuditReasonVO = convertToYcqActivityAuditReasonVO(tsmPropertyValue);
            activityAuditReasonVOList.add(activityAuditReasonVO);
        }
        Collections.sort(activityAuditReasonVOList, Comparator.comparingLong(YcqActivityAuditReasonVO::getSortId).reversed());
        return Result.ofSuccess(activityAuditReasonVOList);
    }

    private YcqActivityAuditReasonVO convertToYcqActivityAuditReasonVO(TsmPropertyValue tsmPropertyValue) {
        YcqActivityAuditReasonVO activityAuditReasonVO = new YcqActivityAuditReasonVO();
        activityAuditReasonVO.setValueId(Long.valueOf(tsmPropertyValue.getValueId()));
        activityAuditReasonVO.setValueCode(tsmPropertyValue.getValueCode());
        activityAuditReasonVO.setValueName(tsmPropertyValue.getValueName());
        activityAuditReasonVO.setSortId(Long.valueOf(tsmPropertyValue.getSortId()));
        return activityAuditReasonVO;
    }
    @Override
    public Result<String> update(YcqActivityAuditReasonDTO dto) {
        if (dto.getValueId()==null){
            throw new BusinessException("id不能为空！");
        }
        TsmPropertyValue byValueId = tsmPropertyValueRepo.findByValueId(dto.getValueId().intValue());
        byValueId.setValueName(dto.getValueName());
        tsmPropertyValueRepo.save(byValueId);
        return Result.ofSuccess("操作成功");
    }

    @Override
    public Result<String> delete(YcqActivityAuditReasonDTO dto) {
        if (dto.getValueId()==null){
            throw new BusinessException("id不能为空！");
        }
        tsmPropertyValueRepo.deleteByValueId(dto.getValueId().intValue());
        return Result.ofSuccess("操作成功");
    }

    @Override
    public Result<String> saveCreate(YcqActivityAuditReasonDTO dto) {

        TsmPropertyValue tsmPropertyValue = new TsmPropertyValue();

        TsmProperty tsmProperty = tsmPropertyRepo.findByPropCodeAndState("ACT_ACTIVITY_QFJH", 1);
        if (ObjectUtils.isEmpty(tsmProperty)){
            throw new BusinessException("请检查静态值:活动管理-报名审核-原因不存在！");
        }

        tsmPropertyValue.setPropId(tsmProperty.getPropId());
        tsmPropertyValue.setValueCode(generateValueCode());
        tsmPropertyValue.setValueName(dto.getValueName());
        List<TsmPropertyValue>  maxSortIdList = tsmPropertyValueRepo.findMaxSortIdByPropId(tsmProperty.getPropId());
        int newSortId = 0;
        if (maxSortIdList != null && !maxSortIdList.isEmpty()) {
            TsmPropertyValue maxSortIdProperty = maxSortIdList.get(0);
            int maxSortId = maxSortIdProperty.getSortId();
            newSortId = maxSortId + 1;
            tsmPropertyValue.setSortId(newSortId);
        }
        tsmPropertyValue.setState(1);
        tsmProperty.setCreateTime(new Date());
        tsmPropertyValue.setPropCode(tsmProperty.getPropCode());
        tsmPropertyValue.setDelFlag(0);

        tsmPropertyValueRepo.save(tsmPropertyValue);

        return Result.ofSuccess("操作成功");
    }

    private static final Random RANDOM = new Random();

    public static String generateValueCode() {
        StringBuilder valueCode = new StringBuilder();

        int firstTwoDigits = RANDOM.nextInt(100);
        valueCode.append(String.format("%02d", firstTwoDigits));

        for (int i = 0; i < 4; i++) {
            char randomUppercaseLetter = (char) (RANDOM.nextInt(26) + 'A');
            valueCode.append(randomUppercaseLetter);
        }

        return valueCode.toString();
    }

    public static void main(String[] args) {
        System.out.println(generateValueCode());
    }
}
