package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.convert.impl.BeanConverter;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.YcqQzhAnnouncementApply;
import com.crunii.micro.domain.entity.main.document.YcqQzhAnnex;
import com.crunii.micro.domain.entity.main.document.YcqQzhAnnouncement;
import com.crunii.micro.domain.entity.main.document.YcqQzhCover;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.domain.repo.main.YcqQzhBusinessAllRepo;
import com.crunii.micro.domain.repo.main.announcement.YcqQzhAnnouncementApplyRepo;
import com.crunii.micro.domain.repo.main.announcement.YcqQzhAnnouncementRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhAnnexRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhBrowseRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhCoverRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.announcement.*;
import com.crunii.micro.service.qzh.api.domain.dto.document.AnnexDto;
import com.crunii.micro.service.qzh.api.domain.dto.document.CoverImageDto;
import com.crunii.micro.service.qzh.jdbc.YcqQzhAnnouncementJdbcDao;
import com.crunii.micro.service.qzh.service.YcqQzhAnnouncementService;
import com.crunii.micro.service.qzh.util.PictureEncapsulationUtil;
import com.crunii.micro.service.qzh.utils.BeanCopyUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qzh.api.constants.QzhConstants.*;

/**
 * @Description: 公告公示ServiceImpl
 * @Param:
 * @return:
 * @Author: xiang xin
 * @Date: 2023/9/18
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class YcqAnnouncementServiceImpl implements YcqQzhAnnouncementService {
    private final YcqQzhAnnouncementRepo ycqQzhAnnouncementRepo;
    private final YcqQzhAnnexRepo ycqQzhAnnexRepo;
    private final YcqQzhCoverRepo ycqQzhCoverRepo;
    private final YcqQzhAnnouncementJdbcDao ycqQzhAnnouncementJdbcDao;
    private final YcqQzhAnnouncementApplyRepo applyRepo;
    private final YcqQzhBrowseRepo ycqQzhBrowseRepo;
    private final YcqQzhBusinessAllRepo ycqQzhBusinessAllRepo;
    private final TsmOrgRepo tsmOrgRepo;
    @Resource
    private YcqQzhAnnouncementApplyRepo ycqQzhAnnouncementApplyRepo;
    @Resource
    private BeanCopyUtil beanCopyUtil;

    /**
     * 新建公告
     *
     * @param req
     * @return
     */
    @Override
    public Result<Object> createAnnouncement(CreateAnnouncementReqDTO req) {
        //校验请求参数
        validateCreateDocument(req);
        Worker worker = getWorker();
        Snowflake snowflake = new Snowflake();
        YcqQzhAnnouncement ycqQzhAnnouncement = new YcqQzhAnnouncement();
        ycqQzhAnnouncement.setId(snowflake.nextId());
        ycqQzhAnnouncement.setAnnouncementTitle(req.getAnnouncementTitle());
        ycqQzhAnnouncement.setAnnouncementType(req.getAnnouncementType());
        ycqQzhAnnouncement.setOpenCompetence(req.getOpenCompetence());
        ycqQzhAnnouncement.setAnnouncementText(req.getAnnouncementText());
        //新建默认已发布
        ycqQzhAnnouncement.setState(1);
        //新增和工单的关联
        if (null != req.getApplyOrders() && req.getApplyOrders().size() > 0) {
            for (ApplyOrder applyOrder : req.getApplyOrders()) {
                if(ObjectUtil.isNotEmpty(applyOrder.getApplyId())){
                    YcqQzhAnnouncementApply ycqQzhAnnouncementApply = new YcqQzhAnnouncementApply();
                    ycqQzhAnnouncementApply.setApplyId(applyOrder.getApplyId());
                    ycqQzhAnnouncementApply.setApplyName(applyOrder.getApplyName());
                    ycqQzhAnnouncementApply.setApplyType(req.getAnnouncementType());
                    ycqQzhAnnouncementApply.setAnnouncementId(ycqQzhAnnouncement.getId().toString());
                    applyRepo.save(ycqQzhAnnouncementApply);
                }
            }
        }
        //组织封装
//        orgSelect(worker, ycqQzhAnnouncement, req);
        //创建信息
        saveCreate(ycqQzhAnnouncement);
        ycqQzhAnnouncementRepo.save(ycqQzhAnnouncement);
        //存储附件和图片
        if (null != req.getSubsidiaryFile() && req.getSubsidiaryFile().size() > 0) {
            saveAnnex(req.getSubsidiaryFile(), ycqQzhAnnouncement.getId());
        }
        if (null != req.getCoverImageFile() && req.getCoverImageFile().size() > 0) {
            saveCover(req.getCoverImageFile(), ycqQzhAnnouncement.getId());
        }
        return new Result<>(CREATE_SUCCESS, Boolean.TRUE);

    }

    /**
     * 分页查询公告
     */
    @Override
    public Result<PageRsp<PageAnnouncementRspDTO>> pageAnnouncement(PageReq<PageAnnouncementReqDTO> pageAnnouncementReqDTO) {
        PageAnnouncementReqDTO query = pageAnnouncementReqDTO.getQuery();
        Worker worker = getWorker();
//        if (null == worker.getOrgId()) {
//            return Result.ofFail("该账号组织异常,请联系系统管理员");
//        }
//        if (null != query.getCondition() && "app".equals(query.getCondition())) {
//            List<Long> list = ycqQzhAnnouncementJdbcDao.selecetOrgs(worker.getLoginId());
//            List<Long> listIds = ycqQzhAnnouncementJdbcDao.selecetIds(list);
//            return new Result(Boolean.TRUE, isShow(ycqQzhAnnouncementJdbcDao.pageAnnouncement(pageAnnouncementReqDTO, listIds, worker.getOrgId()), worker));
//        } else if (null != query.getCondition() && "web".equals(query.getCondition())) {
//            if (null == query.getOpenCompetence()) {
//                return Result.ofFail("开放范围不能为空");
//            }
        PageRsp<PageAnnouncementRspDTO> show = isShow(ycqQzhAnnouncementJdbcDao.pageAnnouncement(pageAnnouncementReqDTO, null, worker.getOrgId()), worker);
        for (PageAnnouncementRspDTO pageAnnouncementRspDTO : show.getResult()) {
            List<YcqQzhAnnouncementApply> announcementApplies = ycqQzhAnnouncementApplyRepo.findByAnnouncementId(pageAnnouncementRspDTO.getId().toString());
            if(ObjectUtil.isNotEmpty(announcementApplies)){
                List<ApplyOrder> applyOrders = beanCopyUtil.copyList(announcementApplies, ApplyOrder.class);
                pageAnnouncementRspDTO.setApplyOrders(applyOrders);
            }
        }
        return new Result(Boolean.TRUE, show);
//        } else {
//            return Result.ofFail("位置字段不能为空");
//        }
    }

    /**
     * 对按钮展示进行打标
     *
     * @param pageDocumentRspDTOPageRsp
     * @return
     */
    public PageRsp<PageAnnouncementRspDTO> isShow(PageRsp<PageAnnouncementRspDTO> pageDocumentRspDTOPageRsp, Worker worker) {
        for (PageAnnouncementRspDTO pageAnnouncementRspDTO : pageDocumentRspDTOPageRsp.getResult()) {
            if (pageAnnouncementRspDTO.getCreateId().equals(String.valueOf(worker.getLoginId()))) {
                pageAnnouncementRspDTO.setIsShow(1);
            } else {
                pageAnnouncementRspDTO.setIsShow(2);
            }
            List<YcqQzhCover> oneByArticleId = ycqQzhCoverRepo.findOneByArticleId(pageAnnouncementRspDTO.getId(), 2);
            List<String> coverList = new ArrayList<>();
            if (oneByArticleId.size() > 0) {
                for (YcqQzhCover ycqQzhCover : oneByArticleId) {
                    coverList.add(ycqQzhCover.getFileUrl());
                }
            }
            pageAnnouncementRspDTO.setCoverUrl(coverList);
        }
        return pageDocumentRspDTOPageRsp;
    }

    /**
     * 查看公告公示详情
     *
     * @param request
     * @return
     */
    @Override
    public Result<AnnouncementDetailsRspDTO> findAnnouncementDetailsById(OperationAnnouncementReqDTO request) {
        YcqQzhAnnouncement one = ycqQzhAnnouncementRepo.getOne(request.getId());
        if (ObjectUtil.isEmpty(one) || (ObjectUtil.isNotEmpty(one.getDelFlag()) && one.getDelFlag().equals(1))) {
            return Result.ofFail(ANNOUNCEMENT_NOT_EXISTS);
        }
        AnnouncementDetailsRspDTO announcementDetailsRspDTO = new AnnouncementDetailsRspDTO();
        //浏览量
        Integer countArticleIdAndRecordType = ycqQzhBrowseRepo.findCountArticleIdAndRecordType(request.getId(), 2);
        if (null != countArticleIdAndRecordType) {
            announcementDetailsRspDTO.setViewTotal(countArticleIdAndRecordType);
        } else {
            announcementDetailsRspDTO.setViewTotal(0);
        }
        //封装来源
        if (null != one.getCreateId()) {
            String orgName = ycqQzhAnnouncementJdbcDao.selectOrgName(Long.valueOf(one.getCreateId()));
            if (null != orgName) {
                announcementDetailsRspDTO.setOrgName(orgName);
            }
        }
        announcementDetailsRspDTO.setId(one.getId());
        //标题
        announcementDetailsRspDTO.setAnnouncementTitle(one.getAnnouncementTitle());
        //类型
        announcementDetailsRspDTO.setAnnouncementType(one.getAnnouncementType());
        //开放范围
        announcementDetailsRspDTO.setOpenCompetence(one.getOpenCompetence());
        //创建时间
        announcementDetailsRspDTO.setCreateTime(one.getCreateTime());
        //文本内容
        announcementDetailsRspDTO.setAnnouncementText(one.getAnnouncementText());
        //创建人
        announcementDetailsRspDTO.setCreateUserName(one.getCreateBy());
        //查询附件
        if (ObjectUtil.isNotEmpty(announcementDetailsRspDTO)) {
            List<CoverImageDto> coverImageDtoList = ycqQzhCoverRepo.findOneByArticleId(request.getId(), 2).stream().map(PictureEncapsulationUtil::doCoverRsp).collect(Collectors.toList());
            List<AnnexDto> annexDtoList = ycqQzhAnnexRepo.findOneByArticleId(request.getId(), 2).stream().map(PictureEncapsulationUtil::doAnnexDtoConverter).collect(Collectors.toList());
            announcementDetailsRspDTO.setCoverImageFileId(coverImageDtoList);
            announcementDetailsRspDTO.setSubsidiaryFile(annexDtoList);
        }
        return new Result<>(Boolean.TRUE, announcementDetailsRspDTO);
    }

    /**
     * 编辑公告公示
     *
     * @param createAnnouncementReqDTO
     * @return
     */
    @Override
    public Result<String> updateAnnouncement(CreateAnnouncementReqDTO createAnnouncementReqDTO) {
        if (ObjectUtil.isEmpty(createAnnouncementReqDTO.getId())) {
            return Result.ofFail(ANNOUNCEMENT_REQUEST_ERROR);
        }
        //校验请求参数
        validateCreateDocument(createAnnouncementReqDTO);
        Worker worker = getWorker();
        YcqQzhAnnouncement ycqQzhAnnouncement = ycqQzhAnnouncementRepo.getOne(createAnnouncementReqDTO.getId());
        if (Objects.nonNull(ycqQzhAnnouncement.getCreateId()) && Objects.nonNull(worker.getLoginId())) {
            if (!ycqQzhAnnouncement.getCreateId().equals(String.valueOf(worker.getLoginId()))) {
                return Result.ofFail(ALL_OFF_HANDLE);
            }
        } else {
            return Result.ofFail(ALL_OFF_ACCOUNT);
        }
        if (2 == ycqQzhAnnouncement.getState()) {
            //校验参数
            validateCreateDocument(createAnnouncementReqDTO);
            ycqQzhAnnouncement.setId(createAnnouncementReqDTO.getId());
            ycqQzhAnnouncement.setAnnouncementTitle(createAnnouncementReqDTO.getAnnouncementTitle());
            ycqQzhAnnouncement.setAnnouncementType(createAnnouncementReqDTO.getAnnouncementType());
            ycqQzhAnnouncement.setOpenCompetence(createAnnouncementReqDTO.getOpenCompetence());
            ycqQzhAnnouncement.setAnnouncementText(createAnnouncementReqDTO.getAnnouncementText());
//            ycqQzhAnnouncement.setState(createAnnouncementReqDTO.getState());
            //删除之前的工单关联
            applyRepo.deleteByAnnouncementId(createAnnouncementReqDTO.getId().toString());
            if (null != createAnnouncementReqDTO.getApplyOrders() && createAnnouncementReqDTO.getApplyOrders().size() > 0) {
                for (ApplyOrder applyOrder : createAnnouncementReqDTO.getApplyOrders()) {
                    YcqQzhAnnouncementApply ycqQzhAnnouncementApply = new YcqQzhAnnouncementApply();
                    ycqQzhAnnouncementApply.setApplyId(applyOrder.getApplyId());
                    ycqQzhAnnouncementApply.setApplyName(applyOrder.getApplyName());
                    ycqQzhAnnouncementApply.setApplyType(createAnnouncementReqDTO.getAnnouncementType());
                    ycqQzhAnnouncementApply.setAnnouncementId(ycqQzhAnnouncement.getId().toString());
                    applyRepo.save(ycqQzhAnnouncementApply);
                }
            }
            //删除历史关联
//            ycqQzhBusinessAllRepo.deleteByBusinessId(createAnnouncementReqDTO.getId());
            //组织
//            orgSelect(worker, ycqQzhAnnouncement, createAnnouncementReqDTO);
            //保存修改人信息
            saveUpdate(ycqQzhAnnouncement);
            ycqQzhAnnouncementRepo.save(ycqQzhAnnouncement);
            //删除附件以及封面
            ycqQzhAnnouncementJdbcDao.deleteAnnex(ycqQzhAnnouncement.getId());
            //存储附件和图片
            if (null != createAnnouncementReqDTO.getSubsidiaryFile() && createAnnouncementReqDTO.getSubsidiaryFile().size() > 0) {
                saveAnnex(createAnnouncementReqDTO.getSubsidiaryFile(), createAnnouncementReqDTO.getId());
            }
            if (null != createAnnouncementReqDTO.getCoverImageFile() && createAnnouncementReqDTO.getCoverImageFile().size() > 0) {
                saveCover(createAnnouncementReqDTO.getCoverImageFile(), createAnnouncementReqDTO.getId());
            }
            return new Result<>(UPDATE_SUCCESS, Boolean.TRUE);
        } else if (1 == ycqQzhAnnouncement.getState()) {
            return Result.ofFail(ALL_OFF_SHELF);
        } else {
            return Result.ofFail(STATE_TRUE);
        }
    }

    /**
     * 修改公告公示状态
     *
     * @param dto
     * @return
     */
    @Override
    public Result<String> doDownAnnouncement(OperationAnnouncementReqDTO dto) {
        YcqQzhAnnouncement ycqQzhAnnouncement = ycqQzhAnnouncementRepo.getOne((long) dto.getId());
        if (ObjectUtil.isEmpty(ycqQzhAnnouncement) || (ObjectUtil.isNotEmpty(ycqQzhAnnouncement.getDelFlag()) && ycqQzhAnnouncement.getDelFlag().equals(1))) {
            return Result.ofFail(ANNOUNCEMENT_NOT_EXISTS);
        }
        if (null != dto.getState()) {
            Worker worker = getWorker();
            if (Objects.nonNull(ycqQzhAnnouncement.getCreateId()) && Objects.nonNull(worker.getLoginId())) {
                if (!ycqQzhAnnouncement.getCreateId().equals(String.valueOf(worker.getLoginId()))) {
                    return Result.ofFail(ALL_OFF_HANDLE);
                }
            } else {
                return Result.ofFail(ALL_OFF_ACCOUNT);
            }
            ycqQzhAnnouncement.setState(dto.getState());
            //保存修改人信息
            saveUpdate(ycqQzhAnnouncement);
            //如果再次发布则个更新发布时间和发布人信息
            if (1 == dto.getState()) {
                saveCreate(ycqQzhAnnouncement);
            }
            ycqQzhAnnouncementRepo.save(ycqQzhAnnouncement);
            return new Result<>(UPDATE_SUCCESS, Boolean.TRUE);
        } else {
            return Result.ofFail("state不能为空");
        }
    }

    /**
     * 删除公告公示
     *
     * @param deleteDto
     * @return
     */
    @Override
    public Result<Object> deleteAnnouncement(OperationAnnouncementReqDTO deleteDto) {
        Worker worker = getWorker();
        YcqQzhAnnouncement ycqQzhAnnouncement = ycqQzhAnnouncementRepo.getOne(deleteDto.getId());
        if (ObjectUtil.isEmpty(ycqQzhAnnouncement) || (ObjectUtil.isNotEmpty(ycqQzhAnnouncement.getDelFlag()) && ycqQzhAnnouncement.getDelFlag().equals(1))) {
            return Result.ofFail(ANNOUNCEMENT_NOT_EXISTS);
        }
        if (ObjectUtil.isNotEmpty(ycqQzhAnnouncement.getState()) && ycqQzhAnnouncement.getState().equals(1)) {
            return Result.ofFail(ANNOUNCEMENT_IS_PUBLISH);
        }
        if (Objects.nonNull(ycqQzhAnnouncement.getCreateId()) && Objects.nonNull(worker.getLoginId())) {
            if (!ycqQzhAnnouncement.getCreateId().equals(String.valueOf(worker.getLoginId()))) {
                return Result.ofFail(ALL_OFF_HANDLE);
            }
        } else {
            return Result.ofFail(ALL_OFF_ACCOUNT);
        }
        ycqQzhAnnouncementJdbcDao.deleteByIds(Collections.singleton(deleteDto.getId()), (long) worker.getLoginId(), worker.getWorkerName());
        return new Result<>(OPERATION_SUCCESS, Boolean.TRUE);

    }

    /**
     * 置顶/取消置顶公告公示
     */
    @Override
    public Result<String> doTopAnnouncement(OperationAnnouncementReqDTO putTopAnnouncementReqDTO) {
        YcqQzhAnnouncement ycqQzhAnnouncement = ycqQzhAnnouncementRepo.getOne(putTopAnnouncementReqDTO.getId());
        if (ObjectUtil.isEmpty(ycqQzhAnnouncement) || (ObjectUtil.isNotEmpty(ycqQzhAnnouncement.getDelFlag()) && ycqQzhAnnouncement.getDelFlag().equals(1))) {
            return Result.ofFail(ANNOUNCEMENT_NOT_EXISTS);
        }
        if (putTopAnnouncementReqDTO.getWhetherTop()) {
            //置顶
            Integer maxSort = ycqQzhAnnouncementRepo.findMaxSort();
            ycqQzhAnnouncement.setSortNum(maxSort != null ? maxSort + 1 : 1);
            ycqQzhAnnouncementRepo.save(ycqQzhAnnouncement);
        } else {
            List<YcqQzhAnnouncement> list = ycqQzhAnnouncementJdbcDao.findListBySortNum(ycqQzhAnnouncement.getSortNum());
            //取消置顶
            ycqQzhAnnouncement.setSortNum(0);
            ycqQzhAnnouncementRepo.save(ycqQzhAnnouncement);
            //小于当前序号的重新排序
            for (YcqQzhAnnouncement qzhAnnouncement : list) {
                qzhAnnouncement.setSortNum(qzhAnnouncement.getSortNum() - 1);
                ycqQzhAnnouncementRepo.save(qzhAnnouncement);
            }
        }
        return new Result<>(OPERATION_SUCCESS, Boolean.TRUE);
    }

    /**
     * 记录浏览量
     */

//    @Override
//    public Result doBrowseAnnouncement(OperationAnnouncementReqDTO browseDocumentReqDTO) {
//        YcqQzhAnnouncement ycqQzhAnnouncement = ycqQzhAnnouncementRepo.getOne(browseDocumentReqDTO.getId());
//        if (ObjectUtil.isEmpty(ycqQzhAnnouncement)) {
//            return Result.ofFail(DOCUMENT_NOT_EXISTS);
//        }
//        Worker worker = getWorker();
//        YcqQzhBrowse ycqQzhBrowse = new YcqQzhBrowse();
//        ycqQzhBrowse.setRecordType(2);
//        ycqQzhBrowse.setArticleId(ycqQzhAnnouncement.getId());
//        ycqQzhBrowse.setUserId((long) worker.getLoginId());
//        ycqQzhBrowse.setCreateTime(new Date());
//        ycqQzhBrowseRepo.save(ycqQzhBrowse);
//        return new Result<>(BROWSE_TRUE, Boolean.TRUE);
//    }

    /**
     * 取消点赞
     *
     * @param req
     * @return
     */
//    public Result<String> doBackBrowseAnnouncement(OperationAnnouncementReqDTO req) {
//        YcqQzhAnnouncement ycqQzhAnnouncement = ycqQzhAnnouncementRepo.getOne(req.getId());
//        if (ObjectUtil.isEmpty(ycqQzhAnnouncement)) {
//            return Result.ofFail(DOCUMENT_NOT_EXISTS);
//        }
//        Worker worker = getWorker();
//        YcqQzhBrowse ycqQzhBrowse = ycqQzhBrowseRepo.findOneByArticleId(3, ycqQzhAnnouncement.getId(), 2, (long) worker.getLoginId());
//        if (ObjectUtil.isNotEmpty(ycqQzhBrowse)) {
//            ycqQzhBrowse.setDelFlag(1);
//            ycqQzhBrowseRepo.save(ycqQzhBrowse);
//        }
//        return new Result<>(true);
//    }

    /**
     * 获取最新公告公示
     *
     * @return
     */
    @Override
    public Result<List<PageAnnouncementRspDTO>> getAnnouncementNewest() {
        List<PageAnnouncementRspDTO> announcementNewest = ycqQzhAnnouncementJdbcDao.getAnnouncementNewest(null);
        for (PageAnnouncementRspDTO pageAnnouncementRspDTO : announcementNewest) {
            List<YcqQzhCover> oneByArticleId = ycqQzhCoverRepo.findOneByArticleId(pageAnnouncementRspDTO.getId(), 2);
            List<String> coverList = new ArrayList<>();
            if (oneByArticleId.size() > 0) {
                for (YcqQzhCover ycqQzhCover : oneByArticleId) {
                    coverList.add(ycqQzhCover.getFileUrl());
                }
            }
            pageAnnouncementRspDTO.setCoverUrl(coverList);
        }
        return new Result<>(Boolean.TRUE, announcementNewest);
    }

    @Override
    public Result<PageRsp<PageAnnouncementRspDTO>> pageAnnouncementTourists(PageReq<PageAnnouncementReqDTO> pageAnnouncementReqDTO) {
        return new Result(Boolean.TRUE, isShowTourists(ycqQzhAnnouncementJdbcDao.pageAnnouncement(pageAnnouncementReqDTO, null, null)));

    }

    public PageRsp<PageAnnouncementRspDTO> isShowTourists(PageRsp<PageAnnouncementRspDTO> pageDocumentRspDTOPageRsp) {
        for (PageAnnouncementRspDTO pageAnnouncementRspDTO : pageDocumentRspDTOPageRsp.getResult()) {
            pageAnnouncementRspDTO.setIsShow(2);
            List<YcqQzhCover> oneByArticleId = ycqQzhCoverRepo.findOneByArticleId(pageAnnouncementRspDTO.getId(), 2);
            List<String> coverList = new ArrayList<>();
            if (oneByArticleId.size() > 0) {
                for (YcqQzhCover ycqQzhCover : oneByArticleId) {
                    coverList.add(ycqQzhCover.getFileUrl());
                }
            }
            //获取来源
            if (null != pageAnnouncementRspDTO.getCreateId()) {
                String orgName = ycqQzhAnnouncementJdbcDao.selectOrgName(Long.valueOf(pageAnnouncementRspDTO.getCreateId()));
                if (null != orgName) {
                    pageAnnouncementRspDTO.setOrgName(orgName);
                }
            }
            pageAnnouncementRspDTO.setCoverUrl(coverList);
        }
        return pageDocumentRspDTOPageRsp;
    }
    /**
     * 查找申报关联工单
     *
     * @param dto
     * @return
     */
//    @Override
//    public Result<List<WorkerOrderRspDTO>> findWorkerOrder(WorkerOrderReqDTO dto) {
//        List<WorkerOrderRspDTO> result = new ArrayList<>();
//        //维权岗
//        if (ObjectUtil.isNotEmpty(dto) && ObjectUtil.isNotEmpty(dto.getType()) && dto.getType().equals(4)) {
//            List<YcqQzhRightsProtection> data = ycqQzhAnnouncementJdbcDao.findWorkerOrderByRightsProtection(dto);
//            for (YcqQzhRightsProtection ycqQzhRightsProtection : data) {
//                WorkerOrderRspDTO dto1 = new WorkerOrderRspDTO();
//                dto1.setId(ycqQzhRightsProtection.getId());
//                dto1.setApplicantCode(ycqQzhRightsProtection.getApplicantCode());
//                dto1.setApplicantUserName(ycqQzhRightsProtection.getApplicant());
//                dto1.setAffiliatedUnitName(ycqQzhRightsProtection.getUnitName());
//                dto1.setOrganizationOrgName(ycqQzhRightsProtection.getOrganization());
//                dto1.setTelephoneNum(ycqQzhRightsProtection.getPhone());
//                if (ObjectUtil.isNotEmpty(ycqQzhRightsProtection.getPolicyId())) {
//                    YcqQzhDocument ycqQzhDocument = ycqQzhDocumentRepo.getOne(Long.valueOf(ycqQzhRightsProtection.getPolicyId()));
//                    if (ObjectUtil.isNotEmpty(ycqQzhDocument) && StringUtils.isNotEmpty(ycqQzhDocument.getDocumentTitle())) {
//                        dto1.setPolicyDocument(ycqQzhDocument.getDocumentTitle());
//                    }
//                }
//                result.add(dto1);
//            }
//        }
//        return new Result<><>(result);
//    }

//    /**
//     * 保存附件以及封面
//     *
//     * @param subsidiaryFile 附件
//     * @param coverImageFile 封面
//     * @param id             公告公示id
//     */
//    private void saveAnnexAndCover(List<AnnexDto> subsidiaryFile, List<CoverImageDto> coverImageFile, Long id) {
//        //保存附件
//        subsidiaryFile.forEach(s -> {
//            YcqQzhAnnex ycqQzhAnnex = new YcqQzhAnnex();
//            ycqQzhAnnex.setFileId(s.getFileId());
//            ycqQzhAnnex.setFileName(s.getFileName());
//            ycqQzhAnnex.setFileKey(s.getFileKey());
//            ycqQzhAnnex.setFileType(s.getFileType());
//            if (s.getFileSize() != null) {
//                ycqQzhAnnex.setFileSize(s.getFileSize());
//            }
//            ycqQzhAnnex.setFileUrl(s.getFileUrl());
//            ycqQzhAnnex.setArticleId(id);
//            ycqQzhAnnex.setType(2);
//            ycqQzhAnnexRepo.save(ycqQzhAnnex);
//        });
//        //保存封面
//        coverImageFile.forEach(s -> {
//            YcqQzhCover ycqQzhCover = new YcqQzhCover();
//            ycqQzhCover.setFileId(s.getFileId());
//            ycqQzhCover.setFileName(s.getFileName());
//            ycqQzhCover.setFileKey(s.getFileKey());
//            ycqQzhCover.setFileType(s.getFileType());
//            if (s.getFileSize() != null) {
//                ycqQzhCover.setFileSize(Integer.parseInt(s.getFileSize()));
//            }
//            ycqQzhCover.setFileUrl(s.getFileUrl());
//            ycqQzhCover.setPicHeight(s.getPicHeight());
//            ycqQzhCover.setPicWidth(s.getPicWidth());
//            ycqQzhCover.setArticleId(id);
//            ycqQzhCover.setType(2);
//            ycqQzhCoverRepo.save(ycqQzhCover);
//        });
//    }

    /**
     * 组织封装
     *
     * @param worker
     * @param ycqQzhAnnouncement
     * @param req
     */
//    private void orgSelect(Worker worker, YcqQzhAnnouncement ycqQzhAnnouncement, CreateAnnouncementReqDTO req) {
//        //todo 对传过来的组织也要进行递归晚上找父级
//        List<YcqQzhBusinessAll> businessAlls = new ArrayList<>();
//        if (null != req.getAuthorityOrgs() && req.getAuthorityOrgs().size() > 0) {
//            List<Long> orgAll = new ArrayList<>();
//            //先把数据的org_id传入list 然后对相同父级的数据做去重 再拿去递归
//            List<YcqAllOrganization> authorityOrgs = req.getAuthorityOrgs();
//            for (YcqAllOrganization authorityOrg : authorityOrgs) {
//                orgAll.add(authorityOrg.getOrgId());
//            }
//            //对传过来的数据进行去重
//            ArrayList<YcqAllOrganization> setOrgs = authorityOrgs.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
//                    new TreeSet<>(Comparator.comparing(YcqAllOrganization::getParentOrgId))), ArrayList::new));
//            if (setOrgs.size() > 0) {
//                for (YcqAllOrganization setOrg : setOrgs) {
//                    List<Long> newList = new ArrayList<>();
//                    findParentOrg(newList, setOrg.getOrgId());
//                    orgAll.addAll(newList);
//                }
//            }
//            //再次去重避免重复
//            List<Long> collect = orgAll.stream().distinct().collect(Collectors.toList());
//            for (Long longId : collect) {
//                YcqQzhBusinessAll ycqQzhBusinessAll = new YcqQzhBusinessAll();
//                if (null != ycqQzhAnnouncement.getId()) {
//                    ycqQzhBusinessAll.setBusinessId(ycqQzhAnnouncement.getId());
//                } else if (null != req.getId()) {
//                    ycqQzhBusinessAll.setBusinessId(req.getId());
//                }
//                ycqQzhBusinessAll.setBusinessType(2);
//                ycqQzhBusinessAll.setOrgId(longId);
//                businessAlls.add(ycqQzhBusinessAll);
//            }
//            ycqQzhBusinessAllRepo.saveAll(businessAlls);
//        }
//        //todo 反向递归找到上级组织id
//        if (null != worker.getOrgId()) {
//            List<Long> list = new ArrayList<>();
//            list.add(Long.valueOf(worker.getOrgId()));
//            //递归上级组织
//            findParentOrg(list, Long.valueOf(worker.getOrgId()));
//            String collect = list.stream().distinct().map(String::valueOf).collect(Collectors.joining(","));
//            ycqQzhAnnouncement.setCreateOrgIds(collect);
//        }
//    }

    /**
     * 逆向递归
     *
     * @param list,orgId
     */
    private void findParentOrg(List<Long> list, Long orgId) {
        TsmOrg tsmOrg = ycqQzhAnnouncementJdbcDao.findParentOrgs(orgId);
        if (0 == tsmOrg.getParentOrgId()) {
            list.add(tsmOrg.getOrgId());
        } else {
            list.add(tsmOrg.getParentOrgId());
            findParentOrg(list, tsmOrg.getParentOrgId());
        }
    }

    /**
     * 保存附件以及封面
     *
     * @param subsidiaryFile 附件
     * @param
     * @param id             政策id
     */
    private void saveAnnex(List<AnnexDto> subsidiaryFile, Long id) {
        //保存附件
        subsidiaryFile.forEach(s -> {
            YcqQzhAnnex ycqQzhAnnex = PictureEncapsulationUtil.doCrateConverter(s);
            ycqQzhAnnex.setArticleId(id);
            ycqQzhAnnex.setType(2);
            ycqQzhAnnexRepo.save(ycqQzhAnnex);
        });
    }

    private void saveCover(List<CoverImageDto> coverImageFile, Long id) {
        //保存封面
        coverImageFile.forEach(s -> {
            YcqQzhCover ycqQzhCover = PictureEncapsulationUtil.doCrateConverter(s);
            ycqQzhCover.setArticleId(id);
            ycqQzhCover.setType(2);
            ycqQzhCoverRepo.save(ycqQzhCover);
        });
    }

    /**
     * 检验公告公示请求参数
     *
     * @param req
     */
    private void validateCreateDocument(CreateAnnouncementReqDTO req) {
        if (ObjectUtil.isEmpty(req)) {
            throw new RuntimeException(DOCUMENT_REQUEST_IS_NULL);
        }
        //标题长度不能超过50个字符
        if (StringUtils.isNotEmpty(req.getAnnouncementTitle()) && req.getAnnouncementTitle().length() > 50) {
            throw new RuntimeException(DOCUMENT_TITLE_LENGTH_TOO_LONG);
        }
        //文件数量不能超过五个
        if (null != req.getSubsidiaryFile() && req.getSubsidiaryFile().size() > 5) {
            throw new RuntimeException(DOCUMENT_FILE_TOO_MUCH);
        }
    }

    /**
     * 获取登录用户
     *
     * @return
     */
    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        return worker;
    }

    /**
     * 保存新建信息
     *
     * @param ycqQzhDocument
     */
    private void saveCreate(YcqQzhAnnouncement ycqQzhDocument) {
        Worker worker = getWorker();
        if (ObjectUtil.isNotEmpty(worker)) {
            ycqQzhDocument.setCreateId(String.valueOf(worker.getLoginId()));
            ycqQzhDocument.setCreateBy(worker.getWorkerName());
            ycqQzhDocument.setCreateTime(new Date());
        }
    }

    /**
     * 保存修改信息
     *
     * @param ycqQzhDocument
     */
    private void saveUpdate(YcqQzhAnnouncement ycqQzhDocument) {
        Worker worker = getWorker();
        if (ObjectUtil.isNotEmpty(worker)) {
            ycqQzhDocument.setUpdateId((long) worker.getLoginId());
            ycqQzhDocument.setUpdateBy(worker.getWorkerName());
            ycqQzhDocument.setUpdateTime(new Date());
        }
    }
}