package com.crunii.micro.service.qzh.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.domain.dto.message.PushMsgTemplateReqBo;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.dynamicform.YcqQzhDynamicFormColumnValue;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.repo.main.dynamicform.YcqDynamicFormColumnValueRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.MessageCommonService;
import com.crunii.micro.platform.components.Locker;
import com.crunii.micro.service.qzh.api.domain.dto.activityPublish.FeedbackLimitDTO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormRecordSubmitDTO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormsReqDTO;
import com.crunii.micro.service.qzh.api.enums.ActivityButtonStateEnum;
import com.crunii.micro.service.qzh.dto.YcqActivityPeopleAuditDTO;
import com.crunii.micro.service.qzh.dto.dynamicform.*;
import com.crunii.micro.service.qzh.enums.ActivityPushMsgConstants;
import com.crunii.micro.service.qzh.jdbc.YcqActivityApplyRecordDao;
import com.crunii.micro.service.qzh.jdbc.YcqActivityPeopleAuditDao;
import com.crunii.micro.service.qzh.jdbc.YcqDynamicFormDao;
import com.crunii.micro.service.qzh.jdbc.activitypublish.PublishActivityDao;
import com.crunii.micro.service.qzh.service.RegistrationAuditService;
import com.crunii.micro.service.qzh.service.YcqDynamicFormService;
import com.crunii.micro.service.qzh.service.YcqEventReportService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.util.*;

@Service
@Slf4j
public class YcqDynamicFormServiceImpl implements YcqDynamicFormService {

    @Resource
    private YcqDynamicFormRepo formRepo;

    /** 第一代居民身份证长度 */
    private static final int FIRST_ID_CARD_LENGTH = 15;


    /** 第二代居民身份证长度 */
    private static final int SECOND_ID_CARD_LENGTH = 18;



    @Resource
    private YcqDynamicRecordRepo formRecordRepo;

    @Resource
    RegistrationAuditService auditService;

    private static final Integer POSITIVE_NUMBER = 1;

    @Resource
    private YcqActiveFlowNodeRepo ycqActiveFlowNodeRepo;

    @Autowired
    private Locker locker;

    //活动报名锁参数
    private final static String ACTIVITY_APPLY = "LOCK:ACTIVITY:APPLY";

    @Resource
    private PublishActivityRepo publishActivityRepo;

    @Autowired
    YcqActivityPeopleAuditDao activityPeopleAuditDao;


    @Autowired
    private YcqActivityApplyFormRepo applyFormRepo;

    @Autowired
    private YcqActivityApplyRecordDao applyRecordDao;


    @Autowired
    private YcqEventReportService ycqEventReportService;

    @Resource
    private TsmOrgRepo tsmOrgRepo;

    @Resource
    private YcqDynamicFormColumnValueRepo formColumnValueRepo;

    @Resource
    private YcqDynamicFormDao dao;

    @Resource
    private PublishActivityDao publishActivityDao;
    @Resource
    private YcqDynamicColumnRepo columnRepo;


    @Resource
    private RegistrationAuditService registrationAuditService;

    @Resource
    private MessageCommonService messageCommonService;

    /**
     * 额外带有用户信息保存
     *
     * @param dynamicForm
     * @param loginId
     * @return
     */
    @Override
    public Long saveContainUserInfo(DynamicFormsReqDTO dynamicForm, int loginId) {
        // 存formjson表
        YcqDynamicForm ycqDynamicForm = new YcqDynamicForm();
        BeanUtils.copyProperties(dynamicForm, ycqDynamicForm);
        YcqDynamicForm save = formRepo.saveAndFlush(ycqDynamicForm);

        // 将列解析出来存column表 todo::
//        String formJson = dynamicForm.getFormJson();
//        YcqDynamicColumn column = new YcqDynamicColumn();
//        column.setFormId(save.getId());
//        columnRepo.saveAll(null);
        // 解析json，生成业务表 todo::


        return save.getId();
    }

    @Override
    @Transactional
    public Result<Boolean> doCreate(DynamicFormTableReqDTO req, Worker currWorker) {
        YcqDynamicForm form = new YcqDynamicForm();
        form.setCreateId(currWorker.getLoginId());
        form.setCreateName(currWorker.getWorkerName());
        form.setFormJson(req.getFormJson());
        form.setBusinessId(Long.parseLong(req.getBusinessId()));
        form.setType(2);
        formRepo.saveAndFlush(form);

        String formJson = req.getFormJson();
        JSONObject object = JSON.parseObject(formJson);
        String schema = object.get("schema").toString();
        List<JSONObject> maps = JsonUtil.jsonToList(schema, JSONObject.class);
        ArrayList<YcqDynamicColumn> list = new ArrayList<>();
        for (JSONObject dto : maps) {
            YcqDynamicColumn ycqDynamicColumn = new YcqDynamicColumn();
            ycqDynamicColumn.setCreateId(currWorker.getLoginId());
            if (dto.get("ui")!=null){
                Object ui=dto.get("ui");
                String uiStr=JsonUtil.getJsonString(ui);
                JSONObject uiObj = JSON.parseObject(uiStr);
                ycqDynamicColumn.setColumnType(uiObj.get("type")==null?"":uiObj.get("type").toString());
            }

            ycqDynamicColumn.setColumnNameDesc(dto.get("title")==null?"":dto.get("title").toString());
            ycqDynamicColumn.setColumnName(dto.get("fieldKey")==null?"":dto.get("fieldKey").toString());
            ycqDynamicColumn.setIsDelete(0);
            ycqDynamicColumn.setFormId(form.getId());
            ycqDynamicColumn.setType(2);
            list.add(ycqDynamicColumn);
        }
        columnRepo.saveAllAndFlush(list);
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional
    public Result<Boolean> doCreateActiveFrom(DynamicFormTableReqDTO req, Worker currWorker) {

        YcqDynamicForm form = new YcqDynamicForm();
        form.setCreateId(currWorker.getLoginId());
        form.setCreateName(currWorker.getWorkerName());
        form.setFormJson(req.getFormJson());
        form.setBusinessId(Long.parseLong(req.getBusinessId()));
        form.setType(1);
        formRepo.saveAndFlush(form);

        String formJson = req.getFormJson();
        JSONObject object = JSON.parseObject(formJson);
        String schema = object.get("recordJson").toString();
        List<JSONObject> maps = JsonUtil.jsonToList(schema, JSONObject.class);
        ArrayList<YcqDynamicColumn> list = new ArrayList<>();
        for (JSONObject dto : maps) {
            if (dto.get("list")!=null){
                Object json=dto.get("list");
                String listStr=JsonUtil.getJsonString(json);
                List<Map> keys = JsonUtil.jsonToList(listStr, Map.class);
                if (keys!=null&&keys.size()>0){
                    for (Map map : keys) {
                        YcqDynamicColumn ycqDynamicColumn = new YcqDynamicColumn();
                        ycqDynamicColumn.setCreateId(currWorker.getLoginId());
                        ycqDynamicColumn.setColumnType("string");
                        ycqDynamicColumn.setColumnNameDesc(map.get("showName")==null?"":map.get("showName").toString());
                        ycqDynamicColumn.setColumnName(map.get("key")==null?"":map.get("key").toString());
                        ycqDynamicColumn.setIsDelete(0);
                        ycqDynamicColumn.setFormId(form.getId());
                        ycqDynamicColumn.setType(1);
                        list.add(ycqDynamicColumn);
                    }
                }
            }
        }
        columnRepo.saveAllAndFlush(list);
        return Result.ofSuccess(true);
    }

    @Override
    public void deleteByIdAndType(Long id, int type){
        formRepo.deleteByBusinessIdAndType(id, type);
    }

    @Override
    @Transactional
    public Result<Boolean> delete(Long id) {
        //删表单
        List<YcqDynamicForm> ycqDynamicForms = formRepo.getByBusinessId(id);
        if (ycqDynamicForms == null) {
            Result.ofFail("表单不存在");
        }
        formRepo.deleteAll(ycqDynamicForms);
        //删字段
        for (YcqDynamicForm form : ycqDynamicForms) {
            List<YcqDynamicColumn> ycqDynamicColumns = columnRepo.findByFormIdOrderByColumnName(form.getId());
            if (ycqDynamicColumns != null && ycqDynamicColumns.size() > 0) {
                columnRepo.deleteAll(ycqDynamicColumns);
            }
        }


        return Result.ofSuccess(true);
    }

    @Override
    @Transactional
    public Result<Boolean> updateForm(DynamicFormTableReqDTO req) {

        YcqDynamicForm form = formRepo.getByBusinessIdAndType(req.getFormId(),req.getType());
        if (form==null){
            Result.ofFail("修改表单错误");
        }
        Long id = form.getId();
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        String formJson = req.getFormJson();
        JSONObject object = JSON.parseObject(formJson);
        String schema = object.get("schema").toString();
        List<Map> maps = JsonUtil.jsonToList(schema, Map.class);
        ArrayList<YcqDynamicColumn> list = new ArrayList<>();
        for (Map dto : maps) {
            YcqDynamicColumn ycqDynamicColumn = new YcqDynamicColumn();
            ycqDynamicColumn.setCreateId(currWorker.getLoginId());
            ycqDynamicColumn.setColumnType(dto.get("type").toString());
            ycqDynamicColumn.setColumnNameDesc(dto.get("title").toString());
            ycqDynamicColumn.setColumnName(dto.get("fieldKey").toString());
            ycqDynamicColumn.setIsDelete(0);
            ycqDynamicColumn.setFormId(id);
            ycqDynamicColumn.setType(2);
            list.add(ycqDynamicColumn);
        }
        List<YcqDynamicColumn> columns = columnRepo.findByFormIdAndTypeOrderByColumnName(id,req.getType());
        if (columns != null && columns.size() > 0) {
            columnRepo.deleteAll(columns);
        }
        columnRepo.saveAll(list);
        form.setUpdateId(currWorker.getLoginId());
        form.setUpdateTime(new Date());
        form.setFormJson(req.getFormJson());
//        form.setFormName(req.getTableName());
        formRepo.saveAndFlush(form);


        return Result.ofSuccess(true);
    }


    @Override
    @Transactional
    public Result<Boolean> updateActiveForm(DynamicFormTableReqDTO req) {

        YcqDynamicForm form = formRepo.getByBusinessIdAndType(req.getFormId(),req.getType());
        if (form==null){
            Result.ofFail("修改表单错误");
        }
        Long id = form.getId();
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        String formJson = req.getFormJson();
        JSONObject object = JSON.parseObject(formJson);
        String schema = object.get("recordJson").toString();
        List<JSONObject> maps = JsonUtil.jsonToList(schema, JSONObject.class);
        ArrayList<YcqDynamicColumn> list = new ArrayList<>();
        for (JSONObject dto : maps) {
            if (dto.get("list")!=null){
                Object json=dto.get("list");
                String listStr=JsonUtil.getJsonString(json);
                List<Map> keys = JsonUtil.jsonToList(listStr, Map.class);
                if (keys!=null&&keys.size()>0){
                    for (Map map : keys) {
                        YcqDynamicColumn ycqDynamicColumn = new YcqDynamicColumn();
                        ycqDynamicColumn.setCreateId(currWorker.getLoginId());
                        ycqDynamicColumn.setColumnType("string");
                        ycqDynamicColumn.setColumnNameDesc(map.get("showName")==null?"":map.get("showName").toString());
                        ycqDynamicColumn.setColumnName(map.get("key")==null?"":map.get("key").toString());
                        ycqDynamicColumn.setIsDelete(0);
                        ycqDynamicColumn.setFormId(form.getId());
                        ycqDynamicColumn.setType(1);
                        list.add(ycqDynamicColumn);
                    }
                }
            }
        }
        List<YcqDynamicColumn> columns = columnRepo.findByFormIdAndTypeOrderByColumnName(id,req.getType());
        if (columns != null && columns.size() > 0) {
            columnRepo.deleteAll(columns);
        }
        columnRepo.saveAllAndFlush(list);
        form.setUpdateId(currWorker.getLoginId());
        form.setUpdateTime(new Date());
        form.setFormJson(req.getFormJson());
//        form.setFormName(req.getTableName());
        formRepo.saveAndFlush(form);
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional
    public Result<Boolean> createFormVale(DynamicFormValeReqDTO req) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
//                        Worker currWorker = new Worker();
//                currWorker.setLoginId(10329);
//                currWorker.setWorkerName("超级管理员");
        YcqDynamicForm form=formRepo.getByBusinessIdAndType(req.getId(),2);
        Long id = form.getId();
        List<YcqDynamicRecord> records=formRecordRepo.findByBusinessIdAndCreateId(id+"",currWorker.getLoginId());
        if (records!=null&&records.size()>0){
            return Result.ofFail("该用户已经提交");
        }
        FeedbackLimitDTO feedbackLimit=publishActivityDao.feedbackLimit(req.getId());
        if (feedbackLimit.isFeedbackLimit()){
            List<YcqDynamicRecord> ycqDynamicRecords=formRecordRepo.findByBusinessId(id+"");
            if (ycqDynamicRecords!=null&&ycqDynamicRecords.size()>=feedbackLimit.getLimitCount()){
                return Result.ofFail("该活动超过提交次数");
            }
        }


        List<DynamicFormColumnValueDTO> list = req.getList();
        ArrayList<YcqQzhDynamicFormColumnValue> records2 = new ArrayList<>();
        YcqDynamicRecord record = new YcqDynamicRecord();
        record.setCreateId(currWorker.getLoginId());
        record.setCreateName(currWorker.getWorkerName());
        record.setCreateTime(new Date());
        record.setBusinessId(id + "");
        formRecordRepo.save(record);
        if (list != null && list.size() > 0) {
            for (DynamicFormColumnValueDTO dto : list) {
                YcqQzhDynamicFormColumnValue recordValue = new YcqQzhDynamicFormColumnValue();
                recordValue.setCreateId(Long.parseLong(currWorker.getLoginId() + ""));
                recordValue.setCreateName(currWorker.getWorkerName());
                recordValue.setCreateTime(new Date());
                recordValue.setColumnKey(dto.getColumnName());
                recordValue.setColumnDes(dto.getColumnDescribe());
                recordValue.setColumnValue(dto.getValue());
                recordValue.setColumnType(dto.getType());
                recordValue.setBusinessId(record.getId());
                records2.add(recordValue);
            }
        }
        formColumnValueRepo.saveAll(records2);
        YcqActivityPeopleAuditDTO ycqActivityPeopleAuditDTO = new YcqActivityPeopleAuditDTO(req.getId(), ActivityButtonStateEnum.HIDE.getCode());
        ycqActivityPeopleAuditDTO.setBusinessBelongId(record.getId());
        registrationAuditService.saveYcqActivityPeopleAudit2(ycqActivityPeopleAuditDTO);
        return Result.ofSuccess(true);
    }



    @Override
    @Transactional
    public Result<Integer> createActiveFormVale(DynamicFormRecordSubmitDTO req) {
       String idCard="";
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        //创建数据
//                Worker currWorker = new Worker();
//                currWorker.setLoginId(9999);
//                currWorker.setWorkerName("超级管理员");
//                req.setRecordJson("{\"idcared\":\"战三\",\"name\":\"男\",\"phone\":\"15823987519\",\"sex\":\"南坪\"}");
//                currWorker.setLoginId(11846);
//                currWorker.setWorkerName("徐尧科");

        YcqDynamicRecord record = new YcqDynamicRecord();
        int loginId = currWorker.getLoginId();
        //     int loginId = 9999;
        Long activityId = req.getActivityId();
        locker.lock(ACTIVITY_APPLY, String.valueOf(loginId));
        try {
            Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(activityId);
            if (!publishActivity.isPresent()) {
                throw new BusinessException("活动不存在");
            }
            YcqPublishActivity ycqPublishActivity = publishActivity.get();
            Integer neededAudit = ycqPublishActivity.getNeededAudit();
            Integer isApply = ycqPublishActivity.getIsApply();
            if (isApply==null&&isApply == 0) {
                throw new BusinessException("该活动无需报名");
            }
            LocalDateTime now = LocalDateTime.now();
            if (now.isAfter(ycqPublishActivity.getEndTime())) {
                throw new BusinessException("活动未启用且已结束");
            }
            if (now.isAfter(DateUtils.dateToLocalDateTime(ycqPublishActivity.getApplyEndTime()))) {
                throw new BusinessException("活动已过报名时间");
            }




            //青春之约校验
            if (ycqPublishActivity.getBusinessBelong()!=null&&ycqPublishActivity.getBusinessBelong().equals("youthPromise")){
                Boolean aBoolean = ycqEventReportService.promiseYouth();
                if (!aBoolean){
                    return Result.ofSuccess(2);
                }

                Integer man = activityPeopleAuditDao.queryAuditCountByActivityIdAndSex(activityId,1);
                Integer women = activityPeopleAuditDao.queryAuditCountByActivityIdAndSex(activityId,0);
                Integer maleCount = ycqPublishActivity.getMaleCount();
                Integer femaleCount = ycqPublishActivity.getFemaleCount();
                if (man>=maleCount){
                    throw new BusinessException("男性报名已报满");
                }
                if (women>=femaleCount){
                    throw new BusinessException("女性报名已报满");
                }

            }

//            if (LocalDateTime.now().isAfter(ycqPublishActivity.getEndTime())) {
//                throw new BusinessException("活动已经结束");
//            }
            // 是否报满 ，根据审核人数判断
            if (Objects.equals(ycqPublishActivity.getIsLimit(), POSITIVE_NUMBER)) {
                int limitCount = ycqPublishActivity.getLimitCount() == null ? 0 : ycqPublishActivity.getLimitCount();
                Integer auditCount = activityPeopleAuditDao.queryAuditCountByActivityId(activityId);
                if (auditCount >= limitCount) {
                    throw new BusinessException("报名人数已满");
                }
            }



            // 判断是否需要提交报名表
            YcqDynamicForm byActivityId = formRepo.getByBusinessIdAndType(activityId,1);
            if (null != byActivityId) {
                //是否有审核通过/同通过 检验
                Integer num = activityPeopleAuditDao.queryAuditCountByActivityIdAndStatusAndUsrId(activityId,(long)loginId);
                if (num>0){
                    throw new BusinessException("已经报名了");
                }


                YcqDynamicForm form=formRepo.getByBusinessIdAndType(req.getActivityId(),req.getType());
                Long id = form.getId();

                ArrayList<YcqQzhDynamicFormColumnValue> records = new ArrayList<>();

                record.setCreateId(currWorker.getLoginId());
                record.setCreateName(currWorker.getWorkerName());
                record.setCreateTime(new Date());
                record.setBusinessId(id + "");
                formRecordRepo.save(record);

                String recordJson = req.getRecordJson();
                if (recordJson!=null&&!recordJson.equals("")){
                    Map<String, Object> stringObjectMap = JsonUtil.jsonToMap(recordJson);
                    if (stringObjectMap!=null){
                        Set<String>  set = stringObjectMap.keySet();
                        for(String key: set) {
                            YcqQzhDynamicFormColumnValue recordValue = new YcqQzhDynamicFormColumnValue();
                            recordValue.setCreateId(Long.parseLong(currWorker.getLoginId() + ""));
                            recordValue.setCreateName(currWorker.getWorkerName());
                            recordValue.setCreateTime(new Date());
                            recordValue.setColumnKey(key);
//                    recordValue.setColumnDes(dto.getColumnDescribe());
                            recordValue.setColumnValue(stringObjectMap.get(key)==null?"":stringObjectMap.get(key).toString());
                            recordValue.setColumnType("string");
                            recordValue.setBusinessId(record.getId());
                            records.add(recordValue);
                            if ("idcared".equals(key)){
                                idCard=stringObjectMap.get(key)==null?"":stringObjectMap.get(key).toString();
                            }
                        }
                    }
                }
                formColumnValueRepo.saveAll(records);
            }
            publishActivityDao.updateApplyCount(ycqPublishActivity.getId());

            // 创建报名用户
            YcqActivityPeopleAuditDTO ycqActivityPeopleAuditDTO = new YcqActivityPeopleAuditDTO(activityId, 2);
            if (Objects.equals(1, neededAudit)) {
                ycqActivityPeopleAuditDTO.setRegistrationStatus(2);
            }
            //设置男女
            if (!idCard.equals("")){
                Boolean isMan = isMan(idCard);
                if (isMan){
                    ycqActivityPeopleAuditDTO.setSex(1);
                }else {
                    ycqActivityPeopleAuditDTO.setSex(0);
                }

            }
            ycqActivityPeopleAuditDTO.setBusinessBelongId(record.getId());
            auditService.saveYcqActivityPeopleAudit(ycqActivityPeopleAuditDTO);
        } finally {
            locker.unlock();
        }
        return Result.ofSuccess(1);

    }

    /**
     * @desc 通过身份证获取性别
     * @auth llp
     * @date 2022/7/15 16:10
     * @param idCard 身份证
     * @return java.util.Map<java.lang.String,java.lang.Object>
     */
    public static Boolean isMan(String idCard){
        Boolean isMan=true;
        Map<String, Object> sexMap = new HashMap<>();
        // 默认值
        int sexInt = -1;
        String sexStr = "未知";
        // 15 位身份证
        if (idCard.length() == FIRST_ID_CARD_LENGTH){
            String sex = idCard.substring(14, 15);
            // 偶数表示女性，奇数表示男性
            if (Integer.parseInt(sex) % 2 == 0){
                isMan=false;
            }else {
                isMan=true;
            }
        }
        // 18 位身份证
        if (idCard.length() == SECOND_ID_CARD_LENGTH){
            String sex = idCard.substring(16, 17);
            // 偶数表示女性，奇数表示男性
            if (Integer.parseInt(sex) % 2 == 0){
                isMan=false;
            }else {
                isMan=true;
            }
        }


        return isMan;
    }





    @Override
    public Result<DynamicFormTableReqDTO> getDetailsById(Long id) {

        YcqDynamicRecord ycqDynamicRecord = formRecordRepo.getById(id);
        //表单id反查表单数据
        YcqDynamicForm form = formRepo.getById(Long.parseLong(ycqDynamicRecord.getBusinessId()));
        List<DynamicFormColumnDTO> resultList = new ArrayList<>();
        //反查填写数据
        List<YcqQzhDynamicFormColumnValue> records = formColumnValueRepo.findByBusinessIdOrderByCreateTimeDesc(id);
        DynamicFormTableReqDTO dto = new DynamicFormTableReqDTO();
        dto.setBusinessId(form.getId() + "");
        dto.setFormJson(form.getFormJson());
        if (records != null && records.size() > 0) {
            for (YcqQzhDynamicFormColumnValue record : records) {
                DynamicFormColumnDTO dynamicFormColumnDTO = new DynamicFormColumnDTO();
                dynamicFormColumnDTO.setValue(record.getColumnValue());
                dynamicFormColumnDTO.setColumnDescribe(record.getColumnDes());
                dynamicFormColumnDTO.setColumnName(record.getColumnKey());
                resultList.add(dynamicFormColumnDTO);
            }
        }
        dto.setList(resultList);
        return Result.ofSuccess(dto);
    }

    @Override
    public Result<PageRsp<DynamicFormValeRspDTO>> getPage(PageReq<DynamicFormReqDTO> req) {
        return Result.ofSuccess(dao.getPage(req));
    }

    @Override
    public Result<List<DynamicFormColumnDTO>> getHeads(Long id,Integer type) {
        YcqDynamicForm form=formRepo.getByBusinessIdAndType(id,type);
        List<YcqDynamicColumn> ycqDynamicColumns =new ArrayList<>();
        if (type==3){
            form=formRepo.getByBusinessIdAndType(id,1);
            ycqDynamicColumns = columnRepo.findByFormIdAndTypeOrderByColumnName(form.getId(),1);
        }else {
            form=formRepo.getByBusinessIdAndType(id,type);
            ycqDynamicColumns = columnRepo.findByFormIdAndTypeOrderByColumnName(form.getId(),type);
        }
        return Result.ofSuccess(getHeadColumns(ycqDynamicColumns,type));
    }

    private List<DynamicFormColumnDTO> getHeadColumns(List<YcqDynamicColumn> ycqDynamicColumns,Integer type) {
        List<DynamicFormColumnDTO> list = new ArrayList<DynamicFormColumnDTO>();
        if (ycqDynamicColumns != null && ycqDynamicColumns.size() > 0) {
            for (YcqDynamicColumn column : ycqDynamicColumns) {
                DynamicFormColumnDTO dynamicFormColumnDTO = new DynamicFormColumnDTO();
                dynamicFormColumnDTO.setType(column.getColumnType());
                dynamicFormColumnDTO.setIsDelete(column.getIsDelete());
                dynamicFormColumnDTO.setColumnName(column.getColumnName());
                dynamicFormColumnDTO.setColumnDescribe(column.getColumnNameDesc());
                if (column.getColumnType()!=null&&!column.getColumnType().equals("content")&&!column.getColumnType().equals("image")&&!column.getColumnType().equals("title")){
                    list.add(dynamicFormColumnDTO);
                }
            }
        }
        if (type==2){
            DynamicFormColumnDTO createTime = new DynamicFormColumnDTO();
            createTime.setType("String");
            createTime.setColumnDescribe("发布时间");
            createTime.setColumnName("createTime");
            DynamicFormColumnDTO createBy = new DynamicFormColumnDTO();
            createBy.setType("String");
            createBy.setColumnDescribe("评价人");
            createBy.setColumnName("createBy");
            list.add(createBy);
            list.add(createTime);
        }else if (type==1){
            DynamicFormColumnDTO status = new DynamicFormColumnDTO();
            status.setType("String");
            status.setColumnDescribe("报名状态");
            status.setColumnName("status");
            list.add(status);
        }else if (type==3){
            DynamicFormColumnDTO submitName = new DynamicFormColumnDTO();
            submitName.setType("String");
            submitName.setColumnDescribe("提交人姓名");
            submitName.setColumnName("submitName");
            list.add(submitName);

            DynamicFormColumnDTO submitPhone = new DynamicFormColumnDTO();
            submitPhone.setType("String");
            submitPhone.setColumnDescribe("提交人电话");
            submitPhone.setColumnName("submitPhone");
            list.add(submitPhone);

            DynamicFormColumnDTO submitTime = new DynamicFormColumnDTO();
            submitTime.setType("String");
            submitTime.setColumnDescribe("提交时间");
            submitTime.setColumnName("submitTime");
            list.add(submitTime);

            DynamicFormColumnDTO status = new DynamicFormColumnDTO();
            status.setType("String");
            status.setColumnDescribe("报名状态");
            status.setColumnName("status");
            list.add(status);

            DynamicFormColumnDTO signTime = new DynamicFormColumnDTO();
            signTime.setType("String");
            signTime.setColumnDescribe("签到时间");
            signTime.setColumnName("signTime");
            list.add(signTime);

            DynamicFormColumnDTO signResult = new DynamicFormColumnDTO();
            signResult.setType("String");
            signResult.setColumnDescribe("签到结果");
            signResult.setColumnName("signResult");
            list.add(signResult);


            DynamicFormColumnDTO signOutTime = new DynamicFormColumnDTO();
            signOutTime.setType("String");
            signOutTime.setColumnDescribe("签退时间");
            signOutTime.setColumnName("signOutTime");
            list.add(signOutTime);

            DynamicFormColumnDTO signOutResult = new DynamicFormColumnDTO();
            signOutResult.setType("String");
            signOutResult.setColumnDescribe("签退结果");
            signOutResult.setColumnName("signOutResult");
            list.add(signOutResult);

        }

        return list;
    }

    @Override
    public Result<Boolean> export(DynamicFormExportDTO req, HttpServletResponse response) {
        Long activeId = req.getActiveId();
        Optional<YcqPublishActivity> publishActivity = publishActivityRepo.findById(activeId);
        if (!publishActivity.isPresent()) {
            throw new BusinessException("活动不存在");
        }
        YcqPublishActivity ycqPublishActivity = publishActivity.get();
        YcqDynamicForm form=formRepo.getByBusinessIdAndType(activeId,2);
        if (form==null){
            return Result.ofFail("模板不存在");
        }
        //表头
        List<List<String>> one = new ArrayList<>();
        List<YcqDynamicColumn> ycqDynamicColumns = columnRepo.findByFormIdAndTypeOrderByColumnName(form.getId(),2);
        List<DynamicFormColumnDTO> dynamicFormColumnDTOS = getHeadColumns(ycqDynamicColumns,2);
        List<String> columns = new ArrayList<>();
        if (dynamicFormColumnDTOS != null && dynamicFormColumnDTOS.size() > 0) {
            for (DynamicFormColumnDTO dynamicFormColumnDTO : dynamicFormColumnDTOS) {
                columns.add(dynamicFormColumnDTO.getColumnDescribe() == null ? " " : dynamicFormColumnDTO.getColumnDescribe());
            }
        }
        one.add(columns);
        //数据
        DynamicFormReqDTO formReqDTO = new DynamicFormReqDTO();
        formReqDTO.setId(req.getActiveId());
        formReqDTO.setName(req.getName());
        formReqDTO.setBeginTime(req.getBeginTime());
        formReqDTO.setEndTime(req.getEndTime());
        formReqDTO.setIds(req.getIds());
        List<List<String>> values = dao.getList(formReqDTO);
        one.addAll(values);
        try {
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode(ycqPublishActivity.getActivityTitle()+"评价", "UTF-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream()).head(new ArrayList<>()).sheet("sheet1").doWrite(one);
        } catch (IOException e) {
            throw new BusinessException("输出流异常");
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Boolean> applicationExport(DynamicFormExportDTO req, HttpServletResponse response) {
        Long activeId = req.getActivityId();
        YcqDynamicForm form=formRepo.getByBusinessIdAndType(activeId,1);
        if (form==null){
            return Result.ofFail("模板不存在");
        }
        //表头
        List<List<String>> one = new ArrayList<>();
        List<YcqDynamicColumn> ycqDynamicColumns = columnRepo.findByFormIdAndTypeOrderByColumnName(form.getId(),1);
        List<DynamicFormColumnDTO> dynamicFormColumnDTOS = getHeadColumns(ycqDynamicColumns,1);
        List<String> columns = new ArrayList<>();
        if (dynamicFormColumnDTOS != null && dynamicFormColumnDTOS.size() > 0) {
            for (DynamicFormColumnDTO dynamicFormColumnDTO : dynamicFormColumnDTOS) {
                columns.add(dynamicFormColumnDTO.getColumnDescribe() == null ? " " : dynamicFormColumnDTO.getColumnDescribe());
            }
        }
        one.add(columns);
        //数据
        DynamicFormReqDTO formReqDTO = new DynamicFormReqDTO();
        formReqDTO.setActivityId(req.getActivityId());
        formReqDTO.setName(req.getName());
        formReqDTO.setBeginTime(req.getBeginTime());
        formReqDTO.setEndTime(req.getEndTime());
        formReqDTO.setIds(req.getIds());
        formReqDTO.setAuditStatus(req.getAuditStatus());
        List<List<String>> values = dao.getApplicationList(formReqDTO,1);
        one.addAll(values);
        try {
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("导出", "UTF-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream()).head(new ArrayList<>()).sheet("sheet1").doWrite(one);
        } catch (IOException e) {
//            throw new BusinessException("输出流异常");
        }
        return Result.ofSuccess(true);
    }


    @Override
    public Result<Boolean> applicationDetailsExport(DynamicFormExportDTO req, HttpServletResponse response) {
        Long activeId = req.getActivityId();
        YcqDynamicForm form=formRepo.getByBusinessIdAndType(activeId,1);
        if (form==null){
            return Result.ofFail("模板不存在");
        }
        //表头
        List<List<String>> one = new ArrayList<>();
        List<YcqDynamicColumn> ycqDynamicColumns = columnRepo.findByFormIdAndTypeOrderByColumnName(form.getId(),1);
        List<DynamicFormColumnDTO> dynamicFormColumnDTOS = getHeadColumns(ycqDynamicColumns,3);
        List<String> columns = new ArrayList<>();
        if (dynamicFormColumnDTOS != null && dynamicFormColumnDTOS.size() > 0) {
            for (DynamicFormColumnDTO dynamicFormColumnDTO : dynamicFormColumnDTOS) {
                columns.add(dynamicFormColumnDTO.getColumnDescribe() == null ? " " : dynamicFormColumnDTO.getColumnDescribe());
            }
        }
        one.add(columns);
        //数据
        DynamicFormReqDTO formReqDTO = new DynamicFormReqDTO();
        formReqDTO.setActivityId(req.getActivityId());
        formReqDTO.setName(req.getName());
        formReqDTO.setBeginTime(req.getBeginTime());
        formReqDTO.setEndTime(req.getEndTime());
        formReqDTO.setIds(req.getIds());
        formReqDTO.setUserName(req.getUserName());
        formReqDTO.setAuditStatus(req.getAuditStatus());
        formReqDTO.setUserName(req.getUserName());
        formReqDTO.setContactMethod(req.getContactMethod());
        formReqDTO.setRegistrationStatus(req.getRegistrationStatus());
        formReqDTO.setCheckInResult(req.getCheckInResult());
        formReqDTO.setCheckOutResult(req.getCheckOutResult());
        formReqDTO.setStartCheckOutTime(req.getStartCheckOutTime());
        formReqDTO.setStartCheckInTime(req.getStartCheckInTime());
        formReqDTO.setStartRegistrationTime(req.getStartRegistrationTime());
        formReqDTO.setEndCheckInTime(req.getEndCheckInTime());
        formReqDTO.setEndCheckOutTime(req.getEndCheckOutTime());
        formReqDTO.setEndRegistrationTime(req.getEndRegistrationTime());
        List<List<String>> values = dao.getApplicationList(formReqDTO,3);
        one.addAll(values);
        try {
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("导出", "UTF-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream()).head(new ArrayList<>()).sheet("sheet1").doWrite(one);
        } catch (IOException e) {
            throw new BusinessException("输出流异常");
        }
        return Result.ofSuccess(true);
    }


    @Override
    public String getDetailsByActiveId(Long id,Integer type) {
        YcqDynamicForm ycqDynamicForm = formRepo.getByBusinessIdAndType(id,type);
        return ycqDynamicForm == null ? null : ycqDynamicForm.getFormJson();
    }


    @Override
    public String getDetailsValueByActiveId(Long id) {
        YcqDynamicRecord record=formRecordRepo.getById(id);
        if (record == null) {
            return null;
        }
        List<YcqQzhDynamicFormColumnValue> values = formColumnValueRepo.findByBusinessIdAndCreateId(record.getId(), Long.parseLong(record.getCreateId()+""));
        List<DynamicFormColumnDTO> resultList = new ArrayList<>();
        if (values != null && values.size() > 0) {
            for (YcqQzhDynamicFormColumnValue columnValue : values) {
                DynamicFormColumnDTO dynamicFormColumnDTO = new DynamicFormColumnDTO();
                dynamicFormColumnDTO.setValue(columnValue.getColumnValue());
                dynamicFormColumnDTO.setColumnDescribe(columnValue.getColumnDes());
                dynamicFormColumnDTO.setColumnName(columnValue.getColumnKey());
                dynamicFormColumnDTO.setType(columnValue.getColumnType());
                resultList.add(dynamicFormColumnDTO);
            }
        }
        if (resultList.size() > 0) {
            return JsonUtil.getPrettyJsonString(resultList);
        }
        return null;
    }

    @Override
    public String getFeedbackByActiveId(Long id,Long loginId,Integer type) {
        YcqDynamicForm ycqDynamicForm = formRepo.getByBusinessIdAndType(id,type);
        if (ycqDynamicForm == null) {
            return null;
        }
//        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        List<YcqDynamicRecord> records = formRecordRepo.findByBusinessIdAndCreateId(ycqDynamicForm.getId() + "", Integer.parseInt(loginId+""));
        if (records == null) {
            return null;
        }
        if (records.size()==0){
            return null;
        }
        YcqDynamicRecord record = records.get(0);
        List<YcqQzhDynamicFormColumnValue> values = formColumnValueRepo.findByBusinessIdAndCreateId(record.getId(), loginId);
        List<DynamicFormColumnDTO> resultList = new ArrayList<>();
        if (values != null && values.size() > 0) {
            for (YcqQzhDynamicFormColumnValue columnValue : values) {
                DynamicFormColumnDTO dynamicFormColumnDTO = new DynamicFormColumnDTO();
                dynamicFormColumnDTO.setValue(columnValue.getColumnValue());
                dynamicFormColumnDTO.setColumnDescribe(columnValue.getColumnDes());
                dynamicFormColumnDTO.setColumnName(columnValue.getColumnKey());
                dynamicFormColumnDTO.setType(columnValue.getColumnType());
                resultList.add(dynamicFormColumnDTO);
            }
        }
        if (resultList.size() > 0) {
            return JsonUtil.getPrettyJsonString(resultList);
        }
        return null;
    }


    @Override
    public List<Map<String,Object>> getRecordByUsrAndActiveIds(Integer usrId,List<Long> ids,Integer type) {
        List<Map<String,Object>> list=new ArrayList<>();
        for (Long id : ids) {
            Map<String,Object> map=new HashMap<>();
            YcqDynamicForm form=formRepo.getByBusinessIdAndType(id,type);
            if (null == form){
                continue;
            }
            map.put("activeId",id);
            List<YcqDynamicRecord> record=formRecordRepo.findByBusinessIdAndCreateId(form.getId()+"",usrId);
            if (!CollectionUtils.isEmpty(record)){
                map.put("usrId",usrId);
                list.add(map);
            }
        }
        return list;
    }

    @Override
    @Transactional
    public Result<Boolean> createNode(YcqActiveFlowNodeReqDTO req) {
        YcqFlowNode ycqActiveFlowNode = new YcqFlowNode();
        ycqActiveFlowNode.setOrgId(req.getOrgId());
        ycqActiveFlowNode.setBizId(req.getActiveId());
        ycqActiveFlowNode.setOrgName(req.getOrgName());
        ycqActiveFlowNode.setCreateBy(req.getCreateBy());
        ycqActiveFlowNode.setCreateId(req.getCreateId());
        ycqActiveFlowNode.setCreateTime(req.getCreateTime()==null?new Date():req.getCreateTime());
        ycqActiveFlowNode.setIsInitial(req.getIsInitial());
        ycqActiveFlowNode.setBizType("active");
        ycqActiveFlowNode.setResultType(0);
        ycqActiveFlowNode.setAuditResult(0);
        List<YcqFlowNode> ycqFlowNodes = ycqActiveFlowNodeRepo.findByBizIdOrderByIsInitialDescCreateTimeDesc(req.getActiveId());
        if (ycqFlowNodes!=null&&ycqFlowNodes.size()>1){
            YcqFlowNode ycqFlowNode = ycqFlowNodes.get(0);
            if (ycqFlowNode.getResultType()==0&&ycqFlowNode.getIsInitial()!=1){
                return Result.ofSuccess();
            }
        }
        if (req.getIsUpdate()==1){
            Integer num = dao.getMaxTime(req.getActiveId());
            if (num==null){
                num=0;
            }else {
                num=num+1;
            }
            ycqActiveFlowNode.setAuditTimes(num);
        }else {
            ycqActiveFlowNode.setAuditTimes(0);
        }

        ycqActiveFlowNodeRepo.saveAndFlush(ycqActiveFlowNode);
        //     通过/不通过发送消息
        if (req.getIsRelease()!=null&&req.getIsRelease()==1){
            PushMsgTemplateReqBo pushMsgTemplateReqBo = new PushMsgTemplateReqBo();
            pushMsgTemplateReqBo.setSystemName(ActivityPushMsgConstants.SYSTEM_NAME);
            pushMsgTemplateReqBo.setSceneName(ActivityPushMsgConstants.SCENE_NAME_ACTIVITY_RELEASE);
            pushMsgTemplateReqBo.setTitle(ActivityPushMsgConstants.TITLE_ACTIVITY_RELEASE);
            YcqPublishActivity activity = publishActivityRepo.getById(req.getActiveId());
            //设置消息跳转参数
            HashMap<String, Object> paramMap = new HashMap<>();
            paramMap.put("id",req.getActiveId());
            paramMap.put("type","active");
            paramMap.put("isJump",1);
            pushMsgTemplateReqBo.setPathAndJson(JsonUtil.getJsonString(paramMap));

            //设置短信接收参数
            Map<String, Object> map = new LinkedHashMap<>();
            map.put("activity",activity.getActivityTitle());
            pushMsgTemplateReqBo.setMap(map);

            List<Worker> ids = dao.getLoginIdS(activity.getOrgId());
            if (ids!=null&&ids.size()>0){
                for (Worker worker : ids) {
                    log.info("发送消息-活动待审核消息： 用户 {} ",(long)worker.getLoginId());
                    messageCommonService.pushMsg(pushMsgTemplateReqBo,(long)worker.getLoginId());
                }
            }


        }
        return Result.ofSuccess();
    }

    @Override
    @Transactional
    public Result<Boolean> updateNode(YcqActiveFlowNodeReqDTO req) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
//        Worker currWorker = new Worker();
//        currWorker.setLoginId(9999);
//        currWorker.setWorkerName("超级管理员");
//        currWorker.setOrgId("1674703740113805312");
//        currWorker.setOrgName("高2022级21班团支部");

        List<YcqFlowNode> ycqFlowNodes = ycqActiveFlowNodeRepo.findByBizIdOrderByIsInitialDescCreateTimeDesc(req.getActiveId());
        YcqFlowNode ycqFlowNode = ycqFlowNodes.get(0);
        Integer auditTimes = ycqFlowNode.getAuditTimes();
        if (ycqFlowNode.getResultType()==1||ycqFlowNode.getResultType()==3){
            return Result.ofFail("已经审核通过/拒绝");
        }
        if (req.getResultType()==2) {
            TsmOrg org = tsmOrgRepo.getById(ycqFlowNode.getOrgId());
            if (org.getParentOrgId() == 0) {
                return Result.ofFail("已经是是最上级");
            }
        }
        ycqFlowNode.setBizId(req.getActiveId());
        ycqFlowNode.setCreateBy(currWorker.getWorkerName());
        ycqFlowNode.setCreateId(Long.parseLong(currWorker.getLoginId()+""));
        ycqFlowNode.setReason(req.getReason());
        ycqFlowNode.setResultType(req.getResultType());
        ycqFlowNode.setResultStr(req.getResultStr());
        ycqFlowNode.setCreateTime(new Date());
        ycqFlowNode.setApproveType(1);
        ycqFlowNode.setIsInitial(2);
        ycqFlowNode.setAuditResult(req.getResultType());
        ycqActiveFlowNodeRepo.saveAndFlush(ycqFlowNode);

        if (req.getResultType()==2){
            TsmOrg org = tsmOrgRepo.getById(ycqFlowNode.getOrgId());

            TsmOrg parentOrg = tsmOrgRepo.getById(org.getParentOrgId());
            YcqFlowNode ycqActiveFlowNode = new YcqFlowNode();
            ycqActiveFlowNode.setOrgId(parentOrg.getOrgId());
            ycqActiveFlowNode.setBizId(ycqFlowNode.getBizId());
            ycqActiveFlowNode.setOrgName(parentOrg.getOrgName());
            ycqActiveFlowNode.setResultType(0);
            ycqActiveFlowNode.setAuditResult(0);
            ycqActiveFlowNode.setCreateTime(new Date());
            ycqActiveFlowNode.setIsInitial(2);
            ycqActiveFlowNode.setAuditTimes(auditTimes);
            ycqActiveFlowNode.setBizType("active");
            ycqActiveFlowNodeRepo.saveAndFlush(ycqActiveFlowNode);
        }



        if (req.getResultType()==1||req.getResultType()==3){

            //通过驳回 分批次修改状态
            List<YcqFlowNode> byAuditTimesAndBizId = ycqActiveFlowNodeRepo.findByAuditTimesAndBizId(auditTimes, req.getActiveId());
            if (byAuditTimesAndBizId!=null&&byAuditTimesAndBizId.size()>0){
                for (YcqFlowNode flowNode : byAuditTimesAndBizId) {
                    flowNode.setAuditResult(req.getResultType());
                }
                ycqActiveFlowNodeRepo.saveAllAndFlush(byAuditTimesAndBizId);
            }
            //发消息
            PushMsgTemplateReqBo pushMsgTemplateReqBo = new PushMsgTemplateReqBo();
            pushMsgTemplateReqBo.setSystemName(ActivityPushMsgConstants.SYSTEM_NAME);
            pushMsgTemplateReqBo.setSceneName(ActivityPushMsgConstants.SCENE_NAME_ACTIVITY_AUDIT);
            pushMsgTemplateReqBo.setTitle(ActivityPushMsgConstants.TITLE_ACTIVITY_AUDIT);
            Map<String, Object> map = new LinkedHashMap<>();
            YcqPublishActivity activity = publishActivityRepo.getById(req.getActiveId());

            map.put("activity",activity.getActivityTitle());
            if (req.getResultType()==1){
                map.put("examine","通过");
            } else if (req.getResultType()==3) {
                map.put("examine","拒绝");
            }

            //设置消息跳转参数
            HashMap<String, Object> paramMap = new HashMap<>();
            paramMap.put("id",req.getActiveId());
            paramMap.put("type","active");
            pushMsgTemplateReqBo.setPathAndJson(JsonUtil.getJsonString(paramMap));

            pushMsgTemplateReqBo.setMap(map);
            try {
                messageCommonService.pushMsg(pushMsgTemplateReqBo,Long.parseLong(activity.getCreateUserId()+""));
            } catch (NumberFormatException e) {
            }
        }

        publishActivityDao.updateAuditResultById(req.getActiveId(),Long.parseLong(currWorker.getOrgId()),req.getResultType());

        return Result.ofSuccess();
    }

    @Override
    public Result<List<YcqActiveFlowNodeRspDTO>> getNodeList(YcqActiveFlowNodeReqDTO req) {
        ArrayList<YcqActiveFlowNodeRspDTO> list = new ArrayList<>();
        List<YcqFlowNode> flowNodes = ycqActiveFlowNodeRepo.findByBizIdOrderByIsInitialAscCreateTimeAsc(req.getActiveId());
        String dshzz="";
        if (flowNodes!=null&&flowNodes.size()>0){
            for (YcqFlowNode flowNode : flowNodes) {
                YcqActiveFlowNodeRspDTO ycqActiveFlowNodeRspDTO = new YcqActiveFlowNodeRspDTO();
                ycqActiveFlowNodeRspDTO.setFbr(flowNode.getCreateBy()==null?null:flowNode.getCreateBy());
                if ((flowNode.getResultType()!=null&&flowNode.getResultType()!=0)||flowNode.getIsInitial()==1){
                    if (flowNode.getIsInitial()==1){
                        ycqActiveFlowNodeRspDTO.setFbsj(flowNode.getCreateTime()==null?null:flowNode.getCreateTime());
                    }else {
                        ycqActiveFlowNodeRspDTO.setShsj(flowNode.getCreateTime()==null?null:flowNode.getCreateTime());
                    }


                }
                if (flowNode.getResultType()==0&&flowNode.getIsInitial()==2){
                    ycqActiveFlowNodeRspDTO.setDshzz(flowNode.getOrgName()==null?null:flowNode.getOrgName());
                }else {
                    ycqActiveFlowNodeRspDTO.setSszz(flowNode.getOrgName()==null?null:flowNode.getOrgName());

                }

                if (flowNode.getResultType()!=null){
                    if (flowNode.getResultType()==1){
                        ycqActiveFlowNodeRspDTO.setShjg("审核通过");
                    }else if (flowNode.getResultType()==2){
                        ycqActiveFlowNodeRspDTO.setShjg("审核通过并交上级审核");
                    }else if (flowNode.getResultType()==3){
                        ycqActiveFlowNodeRspDTO.setShjg("审核不通过");
                    }

                }
                ycqActiveFlowNodeRspDTO.setReason(flowNode.getReason());
                list.add(ycqActiveFlowNodeRspDTO);
                dshzz=flowNode.getOrgName();
            }
        }
        return Result.ofSuccess(list);
    }

    @Override
    public Result<PageRsp<DynamicFormValeRspDTO>> getApplicationPage(PageReq<DynamicFormReqDTO> req) {
        return Result.ofSuccess(dao.getApplicationPage(req));
    }
}
