package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.Result;

import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqCxlTz;
import com.crunii.micro.domain.entity.main.YcqCxlTzLog;
import com.crunii.micro.domain.repo.main.YcqCxlTzLogRepo;
import com.crunii.micro.domain.repo.main.YcqCxlTzRepo;
import com.crunii.micro.service.qzh.dto.YcqCxlTzReqDto;
import com.crunii.micro.service.qzh.enums.ImportExeclTypeEnum;
import com.crunii.micro.service.qzh.jdbc.YcqQdCxlJdbcDao;
import com.crunii.micro.service.qzh.service.YcqQdCxlSaveService;
import com.crunii.micro.service.qzh.service.YcqQdCxlService;
import com.crunii.micro.service.qzh.utils.ExcelUtil;
import groovy.util.logging.Slf4j;
import lombok.RequiredArgsConstructor;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.text.NumberFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @BelongsProject: microservice
 * @BelongsPackage: com.crunii.micro.service.qzh.service.impl
 * @Author: limy
 * @CreateTime: 2023-09-23  11:29
 * @Description: 青大参学率统计台账
 * @Version: 1.0
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class YcqQdCxlServiceImpl implements YcqQdCxlService {

    @Resource
    private YcqCxlTzRepo ycqCxlTzRepo;
    @Resource
    private YcqCxlTzLogRepo ycqCxlTzLogRepo;
    @Resource
    private YcqQdCxlJdbcDao ycqQdCxlJdbcDao;
    @Resource
    private YcqQdCxlSaveService ycqQdCxlSaveService;


    @Override
    public Result ImportTaiZhang(MultipartFile file, String acctMonth, Worker worker) throws IOException {
        //上传附件

        // 读取excel上传数据集合
        List<Map<Integer, String>> list ;
        try {
            list = ExcelUtil.readExcelContent(file);
        } catch (Exception e) {
            throw new RuntimeException("请上传正确的格式文档");
        }
        for (int i = 0; i < list.size(); i++) {

            if (list.size()<2){ throw new RuntimeException("请上传正确的格式文档");}
            Map<Integer, String> integerStringMap = list.get(0);
            Iterator<Map.Entry<Integer, String>> iterator = integerStringMap.entrySet().iterator();
            List<String> ss=new ArrayList<>();
            while (iterator.hasNext()){
                Map.Entry<Integer, String> next = iterator.next();
                Integer key = next.getKey();
                String value = next.getValue();
                ss.add(value);
            }
            //校验分公司固定表头
            if(!ss.contains(ImportExeclTypeEnum.TYPE_QUXIAN.getVal())&&
                    !ss.contains(ImportExeclTypeEnum.TYPE_GAOXIAO.getVal())&&
                    !ss.contains(ImportExeclTypeEnum.TYPE_CITY.getVal())){
                throw new RuntimeException("请上传正确的格式文档");
            }
            break;
        }

        String originalFilename = file.getOriginalFilename();
        //为了获取第几行开始取数据
        int count=0;
        //表头集合
        List<String> importColums = new ArrayList<>();
        //将列名与下标一对应存入一个map
        HashMap<Integer, String> columMap = new HashMap<>();
        //内容集合
        List<YcqCxlTz> importContents = new ArrayList<>();
        int row = 1;
        //取第一列的数据
        List<String>   oneLines=new ArrayList<>();
        // 将第一行的  存入一个list  然后装到其他列去
        Map<Integer, String> integerStringMap1 = list.get(count);
        for (Map.Entry<Integer, String> integerStringEntry : integerStringMap1.entrySet()) {
            oneLines.add(integerStringEntry.getValue());
        }

        for (int i = 0; i < oneLines.size(); i++) {
            //写入表头
            importColums.add(oneLines.get(i));       //列名加进表头
            columMap.put(i,oneLines.get(i));
        }

        //内容解析并保存
        for (int i1 = count+1; i1 < list.size(); i1++) {
            Map<Integer, String> integerStringMap = list.get(i1);
            if(integerStringMap.size()<importColums.size()){
                for (int size = integerStringMap.size(); size < importColums.size(); size++) {
                    integerStringMap.put(size,null);
                }
            }
            Iterator<Map.Entry<Integer, String>> iterator = integerStringMap.entrySet().iterator();

            // 行目录集合
            String areaName=null;
            while (iterator.hasNext()) {
                Map.Entry<Integer, String> next = iterator.next();
                Integer key = next.getKey();
                String value = next.getValue();
                //列数肯定是一致的 不然多的也没用
                if(key>=oneLines.size())
                {break;}
                String s1 = oneLines.get(key);
                if(key.equals(1)){
                    areaName=value;
                }
                //写入内容
                if(key>1){
                    YcqCxlTz content = new YcqCxlTz();
                    content.setMonthId(acctMonth);
                    content.setColum(columMap.get(key));
                    content.setAreaName(areaName);
                    if(oneLines.get(1).equals(ImportExeclTypeEnum.TYPE_QUXIAN.getVal())){
                      content.setExeclType(ImportExeclTypeEnum.TYPE_QUXIAN.getCode());
                    }else if(oneLines.get(1).equals(ImportExeclTypeEnum.TYPE_GAOXIAO.getVal())){
                        content.setExeclType(ImportExeclTypeEnum.TYPE_GAOXIAO.getCode());
                    }else if(oneLines.get(1).equals(ImportExeclTypeEnum.TYPE_CITY.getVal())){
                        content.setExeclType(ImportExeclTypeEnum.TYPE_CITY.getCode());
                    }
                    content.setValue(value);
                    content.setExportRow(row);
                    content.setCreatedId(2333);
                    content.setCreatedTime(LocalDateTime.now());
                    content.setColumNum(key+1);
                    importContents.add(content);
                }
            }
            row++;
        }

        //相关业务数据保存
        ycqCxlTzRepo.saveAll(importContents);
        return Result.ofSuccess("导入成功!");
    }

    //循环解析多个sheet
    @Override
    public Result createMultiTaiZhang(MultipartFile file, String acctMonth, Worker worker) throws IOException {
        // 读取excel上传数据集合
        String fileName = file.getOriginalFilename();
        assert fileName != null;
        String suffix = fileName.substring(fileName.lastIndexOf(".") + 1);
        InputStream in = file.getInputStream();
        int sheets;
        HSSFWorkbook wb=null;
        XSSFWorkbook wbx=null;
        if ("xls".equals(suffix)) {
            wb = new HSSFWorkbook(in);
            sheets = wb.getNumberOfSheets();
        } else if ("xlsx".equals(suffix)) {
             wbx = new XSSFWorkbook(in);
            sheets = wbx.getNumberOfSheets();
        } else {
            return Result.ofFail("文件格式错误!");
        }
        //内容集合
        List<YcqCxlTzReqDto> importContents = new ArrayList<>();
        //校验
        for (int i = 0; i < sheets; i++) {
            String sheetName = "xls".equals(suffix) ? wb.getSheetName(i) : wbx.getSheetName(i);
            if (ObjectUtil.isEmpty(sheetName)||sheetName.contains("Sheet")) {
                return Result.ofFail("第"+(i+1)+"个sheet无类别名!");
            }
            if(!sheetName.equals(ImportExeclTypeEnum.TYPE_QUXIAN.getVal())
                    &&!sheetName.equals(ImportExeclTypeEnum.TYPE_GAOXIAO.getVal())
                    &&!sheetName.equals(ImportExeclTypeEnum.TYPE_CITY.getVal())){
                return Result.ofFail("第"+(i+1)+"个sheet类别不正确!");
            }
            List<Map<Integer, String>> listData = ExcelUtil.readExcelToMap(file, i, 0, 0, -1);
            if (ObjectUtil.isEmpty(listData)) {
                return Result.ofFail("第"+(i+1)+"个Sheet无数据!");
            }
            if(listData.size()<4){
                return Result.ofFail("请填充"+(i+1)+"个Sheet的数据!");
            }
        }
        for (int i = 0; i < sheets; i++) {
            String sheetName= "xls".equals(suffix)?wb.getSheetName(i):wbx.getSheetName(i);
            List<Map<Integer, String>> listData = ExcelUtil.readExcelToMap(file, i, 0, 0, -1);
            //为了获取第几行开始取数据
            int count=1;
            //表头集合
            List<String> importColums = new ArrayList<>();
            int row = 1;

            List<String>   zeroLines=new ArrayList<>();
            Map<Integer, String> integerStringMap2 = listData.get(0);
            for (Map.Entry<Integer, String> integerStringEntry : integerStringMap2.entrySet()) {
                zeroLines.add(integerStringEntry.getValue());
            }
            //取第二行的数据
            List<String>   oneLines=new ArrayList<>();
            // 将第一行的  存入一个list  然后装到其他列去
            Map<Integer, String> integerStringMap1 = listData.get(count);
            for (Map.Entry<Integer, String> integerStringEntry : integerStringMap1.entrySet()) {
                oneLines.add(integerStringEntry.getValue());
            }

            List<String>  twoLines=new ArrayList<>();
            Map<Integer, String> integerStringMap3 = listData.get(count);
            for (Map.Entry<Integer, String> integerStringEntry : integerStringMap3.entrySet()) {
                twoLines.add(integerStringEntry.getValue());
            }

            if(!zeroLines.contains(ImportExeclTypeEnum.TYPE_QUXIAN.getVal())
            &&!zeroLines.contains(ImportExeclTypeEnum.TYPE_GAOXIAO.getVal())
            &&!zeroLines.contains(ImportExeclTypeEnum.TYPE_CITY.getVal())){
                return Result.ofFail("表头格式错误!");
            }
//            if(!zeroLines.contains("团员基数")||!zeroLines.contains("序号")){
//                return Result.ofFail("表头格式错误!");
//            }
            if(!oneLines.contains("平均参学人数")||!oneLines.contains("平均参学率")){
                return Result.ofFail("表头格式错误!");
            }
//            if(twoLines.contains("参学率")||twoLines.contains("参学人数")){
//                return Result.ofFail("表头格式错误!");
//            }

            for (int j = 0; j < oneLines.size(); j++) {
                //写入表头
                importColums.add(oneLines.get(j));       //列名加进表头
            }
            //内容解析并保存
            for (int i1 = 3; i1 < listData.size(); i1++) {
                Map<Integer, String> integerStringMap = listData.get(i1);
                if(integerStringMap.size()<importColums.size()){
                    for (int size = integerStringMap.size(); size < importColums.size(); size++) {
                        integerStringMap.put(size,null);
                    }
                }

                // 行目录集合
                String areaName=null;
                String applyBasic=null;
                List<YcqCxlTzReqDto> ycqCxlTzReqDtos = new ArrayList<>();
                for (Integer key = 0; key < integerStringMap.keySet().size(); key++) {
                    //列数肯定是一致的 不然多的也没用
                    if(key>=oneLines.size())
                    {break;}
                    String s1 = oneLines.get(key);
                    if(key.equals(1)){
                        areaName=integerStringMap.get(key);
                        ycqCxlTzReqDtos = ycqQdCxlJdbcDao.pageList(areaName,acctMonth,null);
                    }
                    if(key.equals(2)){
                        //团员基数
                        applyBasic=integerStringMap.get(key);
                    }
                    //写入内容
                    if(2<key&&key<integerStringMap.keySet().size()-2){
                        //若课程名称或团委名称为空，则相应参学率数据无需导入
                        if(ObjectUtil.isNotEmpty(s1)&&ObjectUtil.isNotEmpty(areaName)){
                            //若系统中已存在团委名称+课程名称的参学率相关数据，则做变更操作；若团委或课程与系统记录中的数据不同，则进行数据新增操作
                            List<YcqCxlTzReqDto> cxlTzReqList = ycqCxlTzReqDtos.stream().filter(ycqCxlTz -> ycqCxlTz.getColum().equals(s1)).collect(Collectors.toList());
                           // List<YcqCxlTz> collect =ObjectUtil.isEmpty(ycqCxlTzs)?null:cxlTzList.stream().filter(ObjectUtil.isNotEmpty(acctMonth) ? ycqCxlTz -> ycqCxlTz.getMonthId().equals(acctMonth) : ycqCxlTz -> true).collect(Collectors.toList());
                            YcqCxlTzReqDto content;
                            if(ObjectUtil.isNotEmpty(cxlTzReqList)){
                                content=cxlTzReqList.get(0);
                                content.setUpdatedTime(LocalDateTime.now());
                                content.setUpdatedId(worker.getLoginId());
                            }else{
                                content = new YcqCxlTzReqDto();
                                content.setCreatedId(worker.getLoginId());
                                content.setCreatedTime(LocalDateTime.now());
                            }
                            content.setColum(s1);
                            content.setAreaName(areaName);
                            content.setMonthId(acctMonth);
                            content.setExeclType(ImportExeclTypeEnum.getCodeByVal(sheetName).getCode());
                            content.setApplyBasic(ObjectUtil.isEmpty(applyBasic)?0:Double.valueOf(applyBasic).intValue());
                            //获取格式化对象
                            NumberFormat nt = NumberFormat.getPercentInstance();
                            //设置百分数精确度2即保留两位小数
                            nt.setMinimumFractionDigits(2);
                            if(ObjectUtil.isNotEmpty(integerStringMap.get(key))&&!isNumeric(integerStringMap.get(key))){
                                throw new BusinessException("文件内容中'"+integerStringMap.get(key)+"'"+"填写的不规范!");
                            }
                            if(ObjectUtil.isNotEmpty(integerStringMap.get(key+1))&&!isNumeric(integerStringMap.get(key+1))){
                                throw new BusinessException("文件内容中'"+integerStringMap.get(key+1)+"'"+"填写的不规范!");
                            }
                            content.setValue(ObjectUtil.isEmpty(integerStringMap.get(key+1))?"0":nt.format(Double.valueOf(integerStringMap.get(key+1))));
                            content.setValue2(ObjectUtil.isEmpty(integerStringMap.get(key))?0:Double.valueOf(integerStringMap.get(key)).intValue());
                            content.setExportRow(row);
                            content.setColumNum(key+1);
                            content.setIsAvg(0);

                            importContents.add(content);
                        }
                    }
                    //平均参学人数与参学率
                    if(ObjectUtil.isNotEmpty(areaName)&&key.equals(integerStringMap.keySet().size()-2)){
                        List<YcqCxlTzReqDto> cxlTzReqList = ycqCxlTzReqDtos.stream().filter(ycqCxlTz -> ycqCxlTz.getColum().equals("平均")).collect(Collectors.toList());
                       // List<YcqCxlTz> collect =ObjectUtil.isEmpty(ycqCxlTzs)?null:cxlTzList.stream().filter(ObjectUtil.isNotEmpty(acctMonth) ? ycqCxlTz -> ycqCxlTz.getMonthId().equals(acctMonth) : ycqCxlTz -> true).collect(Collectors.toList());
                        YcqCxlTzReqDto content;
                        if(ObjectUtil.isNotEmpty(cxlTzReqList)) {
                            content = cxlTzReqList.get(0);
                            content.setUpdatedTime(LocalDateTime.now());
                            content.setUpdatedId(worker.getLoginId());
                        }else {
                            content = new YcqCxlTzReqDto();
                            content.setCreatedId(worker.getLoginId());
                            content.setCreatedTime(LocalDateTime.now());
                        }
                        content.setColum("平均");
                        content.setAreaName(areaName);
                        content.setMonthId(acctMonth);
                        content.setExeclType(ImportExeclTypeEnum.getCodeByVal(sheetName).getCode());
                        content.setApplyBasic(ObjectUtil.isEmpty(applyBasic)?0:Double.valueOf(applyBasic).intValue());
                        //获取格式化对象
                        NumberFormat nt = NumberFormat.getPercentInstance();
                        //设置百分数精确度2即保留两位小数
                        nt.setMinimumFractionDigits(2);
                        if(ObjectUtil.isNotEmpty(integerStringMap.get(key))&&!isNumeric(integerStringMap.get(key))){
                            throw new BusinessException("文件内容中'"+integerStringMap.get(key)+"'"+"填写的不规范!");
                        }
                        if(ObjectUtil.isNotEmpty(integerStringMap.get(key+1))&&!isNumeric(integerStringMap.get(key+1))){
                            throw new BusinessException("文件内容中'"+integerStringMap.get(key+1)+"'"+"填写的不规范!");
                        }
                        content.setValue(ObjectUtil.isEmpty(integerStringMap.get(key+1))?"0":nt.format(Double.valueOf(integerStringMap.get(key+1))));
                        content.setValue2(ObjectUtil.isEmpty(integerStringMap.get(key))?0:Math.round(Float.valueOf(integerStringMap.get(key))));
                        content.setExportRow(row);
                        content.setColumNum(key+1);
                        content.setIsAvg(1);

                        importContents.add(content);
                    }
                }
                row++;
            }

        }

        //相关业务数据异步保存
        ycqQdCxlSaveService.createData(importContents,fileName,worker);
        return Result.ofSuccess("导入成功!");
    }


    private  boolean isNumeric(String str) {
        Pattern pattern = Pattern.compile("[0-9]*\\.?[0-9]+");
        Matcher matcher = pattern.matcher(str);
        boolean success = matcher.matches();
        return success;
    }


}
