package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.YcqMienEvaluate;
import com.crunii.micro.domain.entity.main.document.YcqQzhActivity;
import com.crunii.micro.domain.entity.main.document.YcqQzhBrowse;
import com.crunii.micro.domain.entity.main.document.YcqQzhCover;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.domain.repo.main.YcqMienEvaluateRepo;
import com.crunii.micro.domain.repo.main.YcqQzhBusinessAllRepo;
import com.crunii.micro.domain.repo.main.activity.YcqQzhActivityRepo;
import com.crunii.micro.domain.repo.main.activitypublis.ActivityRegisterRepo;
import com.crunii.micro.domain.repo.main.announcement.YcqQzhAnnouncementRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhBrowseRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhCoverRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhDocumentRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhIndicatorSystemVo;
import com.crunii.micro.service.qzh.api.domain.dto.activity.*;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.CivilizationReportPageDto;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.CivilizationReportVo;
import com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO;
import com.crunii.micro.service.qzh.api.domain.dto.document.CoverImageDto;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.RightsProtectionInventoryPageReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.RightsProtectionInventoryPageRespDTO;
import com.crunii.micro.service.qzh.dto.YcqQzhQagApplyReqDTO;
import com.crunii.micro.service.qzh.dto.YcqQzhQagApplyRespDTO;
import com.crunii.micro.service.qzh.jdbc.YcqQzhActivityJdbcDao;
import com.crunii.micro.service.qzh.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qzh.api.constants.QzhConstants.*;

/**
 * @Description: 风采ServiceImpl
 * @Param:
 * @return:
 * @Author: xiang xin
 * @Date: 2023/9/18
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class YcqQzhActivityServiceImpl implements YcqQzhActivityService {

    private final YcqQzhActivityRepo ycqQzhActivityRepo;
    private final YcqQzhActivityJdbcDao ycqQzhActivityJdbcDao;
    private final YcqQzhCoverRepo ycqQzhCoverRepo;
    private final YcqQzhBrowseRepo ycqQzhBrowseRepo;
    private final ActivityRegisterRepo activityRegisterRepo;
    private final YcqQzhDocumentRepo ycqQzhDocumentRepo;
    private final YcqQzhAnnouncementRepo ycqQzhAnnouncementRepo;
    private final YcqQzhBusinessAllRepo ycqQzhBusinessAllRepo;
    private final TsmOrgRepo tsmOrgRepo;
    /**
     * 风采评定表
     */
    private final YcqMienEvaluateRepo ycqMienEvaluateRepo;
    /**
     * 突击队
     */
    private final YcqQzhCommandoService ycqQzhCommandoService;
    /**
     * 文明号
     */
    private final CivilizationReportService civilizationReportService;
    /**
     * 青安岗
     */
    private final YcqQzhQagApplyService ycqQzhQagApplyService;
    /**
     * 维权岗
     */
    private final YcqQzhProtectionService ycqQzhProtectionService;

    /**
     * 类型 1-突击队，2-文明号，3-示范岗，4-权益岗
     */
    private final static Integer[] TYPE_ARR = new Integer[]{1, 2, 3, 4};

    /**
     * 创建风采
     *
     * @param dto
     * @return
     */
    @Override
    public Result<Void> createAnnouncement(CreateActivityReqDTO dto) {
        // 验证
        validateCreateDocument(dto);
        Snowflake snowflake = new Snowflake();
        Worker worker = getWorker();
        YcqQzhActivity ycqQzhActivity = new YcqQzhActivity();
        ycqQzhActivity.setId(snowflake.nextId());
        //标题
        ycqQzhActivity.setActivityTitle(dto.getActivityTitle());
        //类型
        ycqQzhActivity.setActivityType(dto.getActivityType());
        //文本
        ycqQzhActivity.setActivityText(dto.getActivityText());
        // 需求吴宇佳说开放范围目前不需要，前后端都屏蔽掉，2023-12-18，这里设置默认1-全部
        ycqQzhActivity.setOpenCompetence(1);
        //所属对象ID
        ycqQzhActivity.setParentId(dto.getParentId());
        //关联活动 还需要加判断
        // ycqQzhActivity.setActId(dto.getActId());
        //位置
        ycqQzhActivity.setCondition(dto.getCondition());
        //组织封装
        // orgSelect(worker, ycqQzhActivity, dto);
        //指标id
        ycqQzhActivity.setIndicatorSystemId(dto.getTargetId());
        //存储创建信息
        saveCreate(ycqQzhActivity);
        YcqQzhActivity saveAct = ycqQzhActivityRepo.save(ycqQzhActivity);
        // 如果风采类型为4，即权益岗，需要将风采信息新增到风采评定表 ycq_mien_evaluate
        if (4 == dto.getActivityType() && null != saveAct.getIndicatorSystemId()) {
            createYcqMienEvaluate(saveAct);
        }
        //存储附件和图片
        if (null != dto.getCoverImageFile() && dto.getCoverImageFile().size() > 0) {
            saveAnnexAndCover(dto.getCoverImageFile(), ycqQzhActivity.getId());
        }
        return Result.ofSuccessMsg(CREATE_SUCCESS);
    }

    private void createYcqMienEvaluate(YcqQzhActivity saveAct) {
        YcqMienEvaluate evaluate = new YcqMienEvaluate();
        evaluate.setMienId(saveAct.getId());
        evaluate.setIndicatorId(saveAct.getIndicatorSystemId());
        // 解释：0：待评定；1：已通过；2：已驳回
        evaluate.setState(0);
        evaluate.setDelFlag(0);
        evaluate.setCreateId(Long.valueOf(saveAct.getCreateId()));
        evaluate.setCreatorBy(saveAct.getCreateBy());
        evaluate.setCreateTime(LocalDateTime.now());
        ycqMienEvaluateRepo.save(evaluate);
    }

    /**
     * 分页查询风采
     *
     * @param dto
     * @return
     */
    @Override
    public Result<PageRsp<PageActivityRspDTO>> pageYcqQzhActivity(PageReq<PageActivityReqDTO> dto) {
        PageActivityReqDTO query = dto.getQuery();
        Worker worker = getWorker();
//        if (null == worker.getOrgId()) {
//            return Result.ofFail("该账号组织异常,请联系系统管理员");
//        }
//        if (null != query.getCondition() && "app".equals(query.getCondition())) {
//            List<Long> list = ycqQzhActivityJdbcDao.selecetOrgs(worker.getLoginId());
//            List<Long> listIds = ycqQzhActivityJdbcDao.selecetIds(list);
//            return new Result(Boolean.TRUE, isShow(ycqQzhActivityJdbcDao.pageActivity(dto, listIds, worker.getOrgId()), worker));
//        } else if (null != query.getCondition() && "web".equals(query.getCondition())) {
//            if (null == query.getOpenCompetence()) {
//                return Result.ofFail("开放范围不能为空");
//            }
        return new Result(Boolean.TRUE, isShow(ycqQzhActivityJdbcDao.pageActivity(dto, null, worker), worker));
//        } else {
//            return Result.ofFail("位置字段不能为空");
//        }
    }

    /**
     * 对按钮展示进行打标
     *
     * @param pageDocumentRspDTOPageRsp
     * @return
     */
    public PageRsp<PageActivityRspDTO> isShow(PageRsp<PageActivityRspDTO> pageDocumentRspDTOPageRsp, Worker worker) {
        for (PageActivityRspDTO pageActivityRspDTO : pageDocumentRspDTOPageRsp.getResult()) {
            if (String.valueOf(pageActivityRspDTO.getCreateId()).equals(String.valueOf(worker.getLoginId()))) {
                pageActivityRspDTO.setIsShow(1);
            } else {
                pageActivityRspDTO.setIsShow(2);
            }
            List<YcqQzhCover> oneByArticleId = ycqQzhCoverRepo.findOneByArticleId(pageActivityRspDTO.getId(), 3);
            List<String> coverList = new ArrayList<>();
            if (oneByArticleId.size() > 0) {
                for (YcqQzhCover ycqQzhCover : oneByArticleId) {
                    coverList.add(ycqQzhCover.getFileUrl());
                }
            }
            pageActivityRspDTO.setCoverUrl(coverList);
        }
        return pageDocumentRspDTOPageRsp;
    }

    /**
     * 修改风采
     *
     * @param dto
     * @return
     */
    @Override
    public Result updateYcqQzhActivity(CreateActivityReqDTO dto) {
        if (ObjectUtil.isEmpty(dto) || ObjectUtil.isEmpty(dto.getId())) {
            return Result.ofFail(REQUEST_ERROR);
        }
        validateCreateDocument(dto);
        Worker worker = getWorker();
        YcqQzhActivity ycqQzhActivity = ycqQzhActivityRepo.getOne(dto.getId());
        // 获取未修改的类型和指标id
        Integer oldActivityType = ycqQzhActivity.getActivityType();
        Long oldIndicatorSystemId = ycqQzhActivity.getIndicatorSystemId();
        if (Objects.nonNull(ycqQzhActivity.getCreateId()) && Objects.nonNull(worker.getLoginId())) {
            if (!String.valueOf(ycqQzhActivity.getCreateId()).equals(String.valueOf(worker.getLoginId()))) {
                return Result.ofFail(ALL_OFF_HANDLE);
            }
        } else {
            return Result.ofFail(ALL_OFF_ACCOUNT);
        }
        ycqQzhActivity.setId(dto.getId());
        ycqQzhActivity.setActivityTitle(dto.getActivityTitle());
        ycqQzhActivity.setActivityType(dto.getActivityType());
        ycqQzhActivity.setActivityText(dto.getActivityText());
        // 需求吴宇佳说开放范围目前不需要，前后端都屏蔽掉，2023-12-18，这里设置默认1-全部
        ycqQzhActivity.setOpenCompetence(1);
        ycqQzhActivity.setParentId(dto.getParentId());
        ycqQzhActivity.setCondition(dto.getCondition());
        ycqQzhActivity.setIndicatorSystemId(dto.getTargetId());
        saveUpdate(ycqQzhActivity);
        //删除历史关联
//        ycqQzhBusinessAllRepo.deleteByBusinessId(dto.getId());
        //组织封装
//        orgSelect(worker, ycqQzhActivity, dto);
        YcqQzhActivity saveAct = ycqQzhActivityRepo.save(ycqQzhActivity);
        // 更新风采评定表 ycq_mien_evaluate
        if (4 == dto.getActivityType()){
            updateYcqMienEvaluate(saveAct, oldIndicatorSystemId);
        }
        //删除封面
        ycqQzhActivityJdbcDao.deleteAnnex(dto.getId());
        //保存新封面
        if (null != dto.getCoverImageFile() && dto.getCoverImageFile().size() > 0) {
            saveAnnexAndCover(dto.getCoverImageFile(), dto.getId());
        }
        return new Result<>(UPDATE_SUCCESS, Boolean.TRUE);
    }

    private void updateYcqMienEvaluate(YcqQzhActivity saveAct, Long oldIndicatorSystemId) {
        Worker worker = getWorker();
        if (null == saveAct.getIndicatorSystemId() && null != oldIndicatorSystemId) {
            ycqMienEvaluateRepo.updateDelFlagByMienId(saveAct.getId(), worker.getWorkerName(), (long) worker.getLoginId(), LocalDateTime.now());
        } else if (null != saveAct.getIndicatorSystemId() && null != oldIndicatorSystemId){
            ycqMienEvaluateRepo.updateByMienIdAndIndicatorId(saveAct.getId(), oldIndicatorSystemId, saveAct.getIndicatorSystemId(), worker.getWorkerName(), (long) worker.getLoginId(), LocalDateTime.now());
        } else if (null != saveAct.getIndicatorSystemId() && null == oldIndicatorSystemId) {
            // 新增风采评定表 ycq_mien_evaluate
            createYcqMienEvaluate(saveAct);
        }
    }

    /**
     * 获取风采详情
     *
     * @param dto
     * @return
     */
    @Override
    public Result<ActivityDetailsRspDTO> doFindDetails(OperationActivityReqDTO dto) {
        if (ObjectUtil.isEmpty(dto) || ObjectUtil.isEmpty(dto.getId())) {
            return Result.ofFail(REQUEST_ERROR);
        }
        PageActivityRspDTO ycqQzhActivity = ycqQzhActivityJdbcDao.selectActivity(dto);
        if (null == ycqQzhActivity) {
            return Result.ofFail("该数据异常,查询失败!!!");
        }
        ActivityDetailsRspDTO activityDetailsRsp = new ActivityDetailsRspDTO();
        Integer countArticleIdAndRecordType = ycqQzhBrowseRepo.findCountArticleIdAndRecordType(dto.getId(), 3);
        if (null != countArticleIdAndRecordType) {
            activityDetailsRsp.setViewTotal(countArticleIdAndRecordType);
        } else {
            activityDetailsRsp.setViewTotal(0);
        }
        //获取来源
        if (null != ycqQzhActivity.getCreateId()) {
            String orgName = ycqQzhActivityJdbcDao.selectOrgName(Long.valueOf(ycqQzhActivity.getCreateId()));
            if (null != orgName) {
                activityDetailsRsp.setOrgName(orgName);
            }
        }
        activityDetailsRsp.setId(ycqQzhActivity.getId());
        activityDetailsRsp.setActivityTitle(ycqQzhActivity.getActivityTitle());
        activityDetailsRsp.setActivityType(ycqQzhActivity.getActivityType());
        activityDetailsRsp.setActivityTypeName(ycqQzhActivity.getActivityTypeName());
        activityDetailsRsp.setActivityText(ycqQzhActivity.getActivityText());
        activityDetailsRsp.setOpenCompetence(ycqQzhActivity.getOpenCompetence());
        activityDetailsRsp.setCreateTime(ycqQzhActivity.getCreateTime());
        activityDetailsRsp.setParentId(ycqQzhActivity.getParentId());
        activityDetailsRsp.setParentName(ycqQzhActivity.getParentName());
        activityDetailsRsp.setCreateUserName(ycqQzhActivity.getCreateUserName());
        //查询封面
        if (ObjectUtil.isNotEmpty(activityDetailsRsp)) {
            List<YcqQzhCover> collect = ycqQzhCoverRepo.findOneByArticleId(dto.getId(), 3).stream().collect(Collectors.toList());
            List<CoverImageDto> coverImageDtoList = new ArrayList<>();
            for (YcqQzhCover ycqQzhCover : collect) {
                CoverImageDto coverImageDto = new CoverImageDto();
                coverImageDto.setFileId(ycqQzhCover.getFileId());
                coverImageDto.setFileKey(ycqQzhCover.getFileKey());
                coverImageDto.setFileName(ycqQzhCover.getFileName());
                if (ycqQzhCover.getFileSize() != null) {
                    coverImageDto.setFileSize(String.valueOf(ycqQzhCover.getFileSize()));
                }
                coverImageDto.setFileType(ycqQzhCover.getFileType());
                coverImageDto.setFileUrl(ycqQzhCover.getFileUrl());
                coverImageDto.setPicHeight(ycqQzhCover.getPicHeight());
                coverImageDto.setPicWidth(ycqQzhCover.getPicWidth());
                coverImageDtoList.add(coverImageDto);
            }
            activityDetailsRsp.setCoverImageFile(coverImageDtoList);
        }
        return new Result(Boolean.TRUE, activityDetailsRsp);
    }

    /**
     * 删除风采
     *
     * @param dto
     * @return
     */
    @Override
    public Result deleteYcqQzhActivity(OperationActivityReqDTO dto) {
        Worker worker = getWorker();
        if (ObjectUtil.isEmpty(dto) || ObjectUtil.isEmpty(dto.getId())) {
            return Result.ofFail(REQUEST_ERROR);
        }
        YcqQzhActivity ycqQzhActivity = ycqQzhActivityRepo.getOne(dto.getId());
        if (ObjectUtil.isEmpty(ycqQzhActivity) || (ObjectUtil.isNotEmpty(ycqQzhActivity.getDelFlag()) && ycqQzhActivity.getDelFlag().equals(1))) {
            return Result.ofFail(ACTIVITY_NOT_EXISTS);
        }
        if (Objects.nonNull(ycqQzhActivity.getCreateId()) && Objects.nonNull(worker.getLoginId())) {
            if (!String.valueOf(ycqQzhActivity.getCreateId()).equals(String.valueOf(worker.getLoginId()))) {
                return Result.ofFail(ALL_OFF_HANDLE);
            }
        } else {
            return Result.ofFail(ALL_OFF_ACCOUNT);
        }
        ycqQzhActivityJdbcDao.deleteByIds(Collections.singleton(dto.getId()), (long) worker.getLoginId(), worker.getWorkerName());
        // 更新风采评定表 ycq_mien_evaluate
        if (4 == ycqQzhActivity.getActivityType()) {
            ycqMienEvaluateRepo.updateDelFlagByMienId(dto.getId(), worker.getWorkerName(), (long) worker.getLoginId(), LocalDateTime.now());
        }
        return new Result<>(OPERATION_SUCCESS, Boolean.TRUE);
    }

    /**
     * 转发风采
     *
     * @param dto
     * @return
     */
    @Override
    public Result doForwardYcqQzhActivity(CreateActivityReqDTO dto) {
        //验证
        validateCreateDocument(dto);
        Snowflake snowflake = new Snowflake();
        Worker worker = getWorker();
        YcqQzhActivity ycqQzhActivity = new YcqQzhActivity();
        ycqQzhActivity.setId(snowflake.nextId());
        //标题
        ycqQzhActivity.setActivityTitle(dto.getActivityTitle());
        //类型
        ycqQzhActivity.setActivityType(dto.getActivityType());
        //文本
        ycqQzhActivity.setActivityText(dto.getActivityText());
        // 需求吴宇佳说开放范围目前不需要，前后端都屏蔽掉，2023-12-18，这里设置默认1-全部
        ycqQzhActivity.setOpenCompetence(1);
        //所属对象ID
        ycqQzhActivity.setParentId(dto.getParentId());
        //转发的时候与活动无关
//        ycqQzhActivity.setActId(Optional.ofNullable(dto.getActId()).orElse(null));
        //位置
        ycqQzhActivity.setCondition(dto.getCondition());
        //组织封装
//        orgSelect(worker, ycqQzhActivity, dto);
        // 指标
        ycqQzhActivity.setIndicatorSystemId(dto.getTargetId());
        //存储创建信息
        saveCreate(ycqQzhActivity);
        YcqQzhActivity saveAct = ycqQzhActivityRepo.save(ycqQzhActivity);
        // 如果风采类型为4，即权益岗，需要将风采信息新增到风采评定表 ycq_mien_evaluate
        if (4 == dto.getActivityType() && null != saveAct.getIndicatorSystemId()) {
            createYcqMienEvaluate(saveAct);
        }
        //存储附件和图片
        if (null != dto.getCoverImageFile() && dto.getCoverImageFile().size() > 0) {
            saveAnnexAndCover(dto.getCoverImageFile(), ycqQzhActivity.getId());
        }
        return new Result<>(CREATE_SUCCESS, Boolean.TRUE);
    }

    /**
     * 记录浏览量
     *
     * @param dto
     * @return
     */
    @Override
    public Result doBrowseActivity(OperationActivityReqDTO dto) {
        if (null != dto.getRecordType()) {
            YcqQzhBrowse ycqQzhBrowse = new YcqQzhBrowse();
            //1政策,2公告公示,3风采
            if (1 == dto.getRecordType()) {
                ycqQzhBrowse.setRecordType(1);
//                Optional<YcqQzhDocument> byId = ycqQzhDocumentRepo.findById(dto.getId());
//                if (byId.isPresent()) {
                ycqQzhBrowse.setArticleId(dto.getId());
//                } else {
//                    return Result.ofFail("该政策不存在");
//                }
            } else if (2 == dto.getRecordType()) {
                ycqQzhBrowse.setRecordType(2);
//                Optional<YcqQzhAnnouncement> byId = ycqQzhAnnouncementRepo.findById(dto.getId());
//                if (byId.isPresent()) {
                ycqQzhBrowse.setArticleId(dto.getId());
//                } else {
//                    return Result.ofFail("该公告公示不存在");
//                }
            } else if (3 == dto.getRecordType()) {
                ycqQzhBrowse.setRecordType(3);
//                Optional<YcqQzhActivity> byId = ycqQzhActivityRepo.findById(dto.getId());
//                if (byId.isPresent()) {
                ycqQzhBrowse.setArticleId(dto.getId());
//                } else {
//                    return Result.ofFail("该风采不存在");
//                }
            } else {
                return Result.ofFail("recordType为空或类型错误");
            }
            Worker worker = CurrWorkerUtil.getCurrWorker();
            if (null != worker) {
                ycqQzhBrowse.setUserId(Long.valueOf(worker.getLoginId()));
            }
            ycqQzhBrowse.setCreateTime(new Date());
            ycqQzhBrowseRepo.save(ycqQzhBrowse);
            return new Result<>(BROWSE_TRUE, Boolean.TRUE);
        } else {
            return Result.ofFail("浏览类型不能为空");
        }
    }

    /**
     * 查询所属对象
     * 新增风采，所属对象状态
     * 当风采类型为青年突击队风采时，所属对象下拉列表显示所有“已审核通过”的突击队名称；
     * 当风采类型为青年文明号风采时，所属对象下拉列表显示所有“已认定”的文明号名称；
     * 当风采类型为青年安全生产示范岗风采时，所属对象下拉列表显示所有“已认定”的青安岗名称；
     * 当风采类型为维护青少年权益岗风采时，所属对象下拉列表显示所有“已公示”的维权岗名称；
     * 突击队表：ycq_qzh_commando	  state = 3
     * 文明号表：ycq_yn_declaration	audit_state = 7
     * 青安岗表：ycq_qzh_qag_apply	flow_state  =  7
     * 权益岗表：ycq_qzh_rights_protection	link = '1006'
     * 弃用
     *
     * @param type 类型 1-突击队，2-文明号，3-示范岗，4-权益岗
     * @return Result<List < ParentDTO>>
     */
    public Result<List<ParentDTO>> findParentOld(Integer type) {
        if (null == type) {
            Result.ofFail(REQUEST_ERROR);
        }
        if (null != type && Arrays.asList(TYPE_ARR).contains(type)) {
            Result.ofFail("参数错误！");
        }
        Worker worker = getWorker();
        return Result.ofSuccess(ycqQzhActivityJdbcDao.findParentOld(type, worker));
    }

    /**
     * 查询所属对象
     * 新增风采，所属对象状态
     * 当风采类型为青年突击队风采时，所属对象下拉列表显示所有“已审核通过”的突击队名称；
     * 当风采类型为青年文明号风采时，所属对象下拉列表显示所有“已认定”的文明号名称；
     * 当风采类型为青年安全生产示范岗风采时，所属对象下拉列表显示所有“已认定”的青安岗名称；
     * 当风采类型为维护青少年权益岗风采时，所属对象下拉列表显示所有“已公示”的维权岗名称；
     * 突击队表：ycq_qzh_commando	  state = 3
     * 文明号表：ycq_yn_declaration	audit_state = 7
     * 青安岗表：ycq_qzh_qag_apply	flow_state  =  7
     * 权益岗表：ycq_qzh_rights_protection	link = '1006'
     *
     * @param type 类型 1-突击队，2-文明号，3-示范岗，4-权益岗
     * @return Result<List < ParentDTO>>
     */
    @Override
    public Result<List<ParentDTO>> findParent(Integer type) {
        if (ObjectUtil.isEmpty(type)) {
            Result.ofFail(REQUEST_ERROR);
        }
        // mockCurrWorker();
        Worker currWorker = getWorker();
        // 1-突击队，2-文明号，3-示范岗，4-权益岗
        if (1 == type) {
            List<QueryCommandoRespDTO> allCommando = new ArrayList<>();
            QueryCommandoReqDTO queryCommandoReqDTO = new QueryCommandoReqDTO();
            //调用
            List<QueryCommandoRespDTO> pageRspResult = ycqQzhCommandoService.queryCommandoByExamineList(queryCommandoReqDTO);
            List<QueryCommandoRespDTO> pageRspResult1 = ycqQzhActivityJdbcDao.queryCommandoByApplicant(currWorker);
            allCommando.addAll(pageRspResult);
            allCommando.addAll(pageRspResult1);
            List<ParentDTO> listCommando = new ArrayList<>();
            for (QueryCommandoRespDTO queryCommandoRespDTO : allCommando) {
                if (3 == queryCommandoRespDTO.getState()) {
                    ParentDTO parentDTO = new ParentDTO();
                    parentDTO.setId(queryCommandoRespDTO.getId());
                    parentDTO.setName(queryCommandoRespDTO.getCommandoName());
                    listCommando.add(parentDTO);
                }
            }
            ArrayList<ParentDTO> collect = listCommando.stream().collect(Collectors.collectingAndThen(
                    Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(ParentDTO::getId))), ArrayList::new));
            return new Result<>(Boolean.TRUE, collect);
        } else if (2 == type) {
            PageReq<CivilizationReportPageDto> pageReq = new PageReq<>();
            CivilizationReportPageDto req = new CivilizationReportPageDto();
            req.setAuditState(7);
            pageReq.setPageSize(Integer.MAX_VALUE);
            pageReq.setQuery(req);
            List<CivilizationReportVo> result = civilizationReportService.pageCivilizationReport(pageReq, currWorker).getResult();
            List<ParentDTO> resList = new ArrayList<>();
            for (CivilizationReportVo vo : result) {
                if (7 == vo.getAuditState()) {
                    ParentDTO parentDTO = new ParentDTO();
                    parentDTO.setId(vo.getDeclarationId().toString());
                    parentDTO.setName(vo.getCollectiveName());
                    resList.add(parentDTO);
                }
            }
            ArrayList<ParentDTO> collect = resList.stream().collect(Collectors.collectingAndThen(
                    Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(ParentDTO::getId))), ArrayList::new));
            return new Result<>(Boolean.TRUE, collect);
        } else if (3 == type) {
            PageReq<YcqQzhQagApplyReqDTO> pageReq = new PageReq<>();
            YcqQzhQagApplyReqDTO req = new YcqQzhQagApplyReqDTO();
            req.setFlowState(7);
            pageReq.setPageSize(Integer.MAX_VALUE);
            pageReq.setQuery(req);
            List<YcqQzhQagApplyRespDTO> result = ycqQzhQagApplyService.examinePage(pageReq).getData().getResult();
            List<ParentDTO> resList = new ArrayList<>();
            for (YcqQzhQagApplyRespDTO vo : result) {
                if (7 == vo.getFlowState()) {
                    ParentDTO parentDTO = new ParentDTO();
                    parentDTO.setId(vo.getId().toString());
                    parentDTO.setName(vo.getApplyOrgName());
                    resList.add(parentDTO);
                }
            }
            ArrayList<ParentDTO> collect = resList.stream().collect(Collectors.collectingAndThen(
                    Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(ParentDTO::getId))), ArrayList::new));
            return new Result<>(Boolean.TRUE, collect);
        } else if (4 == type) {
            PageReq<RightsProtectionInventoryPageReqDTO> pageReq = new PageReq<>();
            RightsProtectionInventoryPageReqDTO req = new RightsProtectionInventoryPageReqDTO();
            req.setStatus("1006");
            pageReq.setPageSize(Integer.MAX_VALUE);
            pageReq.setQuery(req);
            List<RightsProtectionInventoryPageRespDTO> result = ycqQzhProtectionService.inventoryPage(pageReq).getData().getResult();
            List<ParentDTO> resList = new ArrayList<>();
            for (RightsProtectionInventoryPageRespDTO vo : result) {
                ParentDTO parentDTO = new ParentDTO();
                parentDTO.setId(vo.getId().toString());
                parentDTO.setName(vo.getUnitName());
                resList.add(parentDTO);
            }
            ArrayList<ParentDTO> collect = resList.stream().collect(Collectors.collectingAndThen(
                    Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(ParentDTO::getId))), ArrayList::new));
            return new Result<>(Boolean.TRUE, collect);
        } else {
            return Result.ofFail("类型错误！");
        }
    }

    /**
     * 查询关联指标
     *
     * @return List<ActivityTargetDTO>
     * @author leixb
     * @since 2023-11-21 16:17
     */
    @Override
    public List<ActivityTargetDTO> findTarget() {
        List<YcqQzhIndicatorSystemVo> resVoList = ycqQzhActivityJdbcDao.findTarget();
        List<String> uniqueEvaluationPoints = resVoList.stream()
                .map(YcqQzhIndicatorSystemVo::getEvaluationPoints)
                .filter(StringUtils::isNotBlank)
                .distinct()
                .collect(Collectors.toList());
        AtomicInteger i = new AtomicInteger(1);
        List<ActivityTargetDTO> resList = uniqueEvaluationPoints.stream()
                .map(evaluationPoints -> {
                    ActivityTargetDTO dto = new ActivityTargetDTO();
                    dto.setValue(String.valueOf(i.getAndIncrement()));
                    dto.setLabel(evaluationPoints);
                    return dto;
                }).collect(Collectors.toList());
        AtomicInteger o = new AtomicInteger(1);
        resList.forEach(dto -> {
            List<String> voListOne = resVoList.stream()
                    .filter(vo -> dto.getLabel().equals(vo.getEvaluationPoints()))
                    .map(YcqQzhIndicatorSystemVo::getPrimaryIndicators)
                    .filter(StringUtils::isNotBlank)
                    .distinct()
                    .collect(Collectors.toList());
            List<ActivityTargetDTO> childrenDto = voListOne.stream()
                    .map(primaryIndicators -> {
                        ActivityTargetDTO childrenOne = new ActivityTargetDTO();
                        childrenOne.setValue(String.valueOf(o.getAndIncrement()));
                        childrenOne.setLabel(primaryIndicators);
                        return childrenOne;
                    }).collect(Collectors.toList());
            dto.setChildren(childrenDto);

            childrenDto.forEach(child -> {
                List<YcqQzhIndicatorSystemVo> voListTwo = resVoList.stream()
                        .filter(vo -> dto.getLabel().equals(vo.getEvaluationPoints())
                                && child.getLabel().equals(vo.getPrimaryIndicators())
                                && StringUtils.isNotBlank(vo.getSecondaryIndicators()))
                        .collect(Collectors.toList());
                List<ActivityTargetDTO> childrenDtoSub = voListTwo.stream()
                        .map(vo -> {
                            ActivityTargetDTO childTwo = new ActivityTargetDTO();
                            childTwo.setValue(String.valueOf(vo.getId()));
                            childTwo.setLabel(vo.getSecondaryIndicators());
                            return childTwo;
                        }).collect(Collectors.toList());
                child.setChildren(childrenDtoSub);
            });
        });
        return resList;
    }

    @Override
    public Result<PageRsp<PageActivityRspDTO>> pageYcqQzhActivityTourists(PageReq<PageActivityReqDTO> dto) {
        return new Result(Boolean.TRUE, isShowTourists(ycqQzhActivityJdbcDao.pageActivity(dto, null, null)));
    }

    /**
     * 对按钮展示进行打标
     *
     * @param pageDocumentRspDTOPageRsp
     * @return
     */
    public PageRsp<PageActivityRspDTO> isShowTourists(PageRsp<PageActivityRspDTO> pageDocumentRspDTOPageRsp) {
        for (PageActivityRspDTO pageActivityRspDTO : pageDocumentRspDTOPageRsp.getResult()) {
            pageActivityRspDTO.setIsShow(2);
            List<YcqQzhCover> oneByArticleId = ycqQzhCoverRepo.findOneByArticleId(pageActivityRspDTO.getId(), 3);
            List<String> coverList = new ArrayList<>();
            if (oneByArticleId.size() > 0) {
                for (YcqQzhCover ycqQzhCover : oneByArticleId) {
                    coverList.add(ycqQzhCover.getFileUrl());
                }
            }
            //获取来源
            if (null != pageActivityRspDTO.getCreateId()) {
                String orgName = ycqQzhActivityJdbcDao.selectOrgName(Long.valueOf(pageActivityRspDTO.getCreateId()));
                if (null != orgName) {
                    pageActivityRspDTO.setOrgName(orgName);
                }
            }
            pageActivityRspDTO.setCoverUrl(coverList);
        }
        return pageDocumentRspDTOPageRsp;
    }

    /**
     * 组织封装
     *
     * @param worker
     * @param ycqQzhActivity
     * @param req
     */
//    private void orgSelect(Worker worker, YcqQzhActivity ycqQzhActivity, CreateActivityReqDTO req) {
//        //todo 对传过来的组织也要进行递归晚上找父级
//        List<YcqQzhBusinessAll> businessAlls = new ArrayList<>();
//        if (null != req.getAuthorityOrgs() && req.getAuthorityOrgs().size() > 0) {
//            List<Long> orgAll = new ArrayList<>();
//            //先把数据的org_id传入list 然后对相同父级的数据做去重 再拿去递归
//            List<YcqAllOrganization> authorityOrgs = req.getAuthorityOrgs();
//            for (YcqAllOrganization authorityOrg : authorityOrgs) {
//                orgAll.add(authorityOrg.getOrgId());
//            }
//            //对传过来的数据进行去重
//            ArrayList<YcqAllOrganization> setOrgs = authorityOrgs.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
//                    new TreeSet<>(Comparator.comparing(YcqAllOrganization::getParentOrgId))), ArrayList::new));
//            if (setOrgs.size() > 0) {
//                for (YcqAllOrganization setOrg : setOrgs) {
//                    List<Long> newList = new ArrayList<>();
//                    findParentOrg(newList, setOrg.getOrgId());
//                    orgAll.addAll(newList);
//                }
//            }
//            //再次去重避免重复
//            List<Long> collect = orgAll.stream().distinct().collect(Collectors.toList());
//            for (Long longId : collect) {
//                YcqQzhBusinessAll ycqQzhBusinessAll = new YcqQzhBusinessAll();
//                if (null != ycqQzhActivity.getId()) {
//                    ycqQzhBusinessAll.setBusinessId(ycqQzhActivity.getId());
//                } else if (null != req.getId()) {
//                    ycqQzhBusinessAll.setBusinessId(req.getId());
//                }
//                ycqQzhBusinessAll.setBusinessType(3);
//                ycqQzhBusinessAll.setOrgId(longId);
//                businessAlls.add(ycqQzhBusinessAll);
//            }
//            ycqQzhBusinessAllRepo.saveAll(businessAlls);
//        }
//        //todo 反向递归找到上级组织id
//        if (null != worker.getOrgId()) {
//            List<Long> list = new ArrayList<>();
//            list.add(Long.valueOf(worker.getOrgId()));
//            //递归上级组织
//            findParentOrg(list, Long.valueOf(worker.getOrgId()));
//            String collect = list.stream().distinct().map(String::valueOf).collect(Collectors.joining(","));
//            ycqQzhActivity.setCreateOrgIds(collect);
//        }
//    }

    /**
     * 逆向递归
     *
     * @param list,orgId
     */
    private void findParentOrg(List<Long> list, Long orgId) {
        TsmOrg tsmOrg = ycqQzhActivityJdbcDao.findParentOrgs(orgId);
        if (0 == tsmOrg.getParentOrgId()) {
            list.add(tsmOrg.getOrgId());
        } else {
            list.add(tsmOrg.getParentOrgId());
            findParentOrg(list, tsmOrg.getParentOrgId());
        }
    }


    /**
     * 检验风采请求参数
     *
     * @param req
     */
    private void validateCreateDocument(CreateActivityReqDTO req) {
        if (ObjectUtil.isEmpty(req)) {
            throw new BusinessException(DOCUMENT_REQUEST_IS_NULL);
        }
        //标题长度不能超过50个字符
        if (StringUtils.isNotEmpty(req.getActivityTitle()) && req.getActivityTitle().length() > 50) {
            throw new BusinessException(DOCUMENT_TITLE_LENGTH_TOO_LONG);
        }
        //文件数量不能超过五个
        if (null != req.getCoverImageFile() && req.getCoverImageFile().size() > 9) {
            throw new BusinessException(ACTIVITY_FILE_TOO_MUCH);
        }
    }

    /**
     * 保存附件以及封面
     *
     * @param coverImageFile 封面
     * @param id             风采id
     */
    private void saveAnnexAndCover(List<CoverImageDto> coverImageFile, Long id) {
        //保存封面
        coverImageFile.forEach(s -> {
            YcqQzhCover ycqQzhCover = new YcqQzhCover();
            ycqQzhCover.setFileId(s.getFileId());
            ycqQzhCover.setFileName(s.getFileName());
            ycqQzhCover.setFileKey(s.getFileKey());
            ycqQzhCover.setFileType(s.getFileType());
            if (s.getFileSize() != null) {
                ycqQzhCover.setFileSize(Integer.parseInt(s.getFileSize()));
            }
            ycqQzhCover.setFileUrl(s.getFileUrl());
            ycqQzhCover.setPicHeight(s.getPicHeight());
            ycqQzhCover.setPicWidth(s.getPicWidth());
            ycqQzhCover.setArticleId(id);
            ycqQzhCover.setType(3);
            ycqQzhCoverRepo.save(ycqQzhCover);
        });
    }

    /**
     * 获取登录用户
     *
     * @return
     */
    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        return worker;
//        if (worker == null) {
//            worker = new Worker();
//            worker.setLoginId(10075);
//            worker.setWorkerName("龚松涛");
//        }
//        if (ObjectUtil.isEmpty(worker)) {
//            //throw new RuntimeException(WORKER_NOT_EXISTS);
//        }
//        return worker;
    }

    /**
     * 保存新建信息
     *
     * @param ycqQzhActivity
     */
    private void saveCreate(YcqQzhActivity ycqQzhActivity) {
        Worker worker = getWorker();
        if (ObjectUtil.isNotEmpty(worker)) {
            ycqQzhActivity.setCreateId(String.valueOf(worker.getLoginId()));
            ycqQzhActivity.setCreateBy(worker.getWorkerName());
            ycqQzhActivity.setCreateTime(new Date());
        }
    }

    /**
     * 保存修改信息
     *
     * @param ycqQzhActivity
     */
    private void saveUpdate(YcqQzhActivity ycqQzhActivity) {
        Worker worker = getWorker();
        if (ObjectUtil.isNotEmpty(worker)) {
            ycqQzhActivity.setUpdateId((long) worker.getLoginId());
            ycqQzhActivity.setUpdateBy(worker.getWorkerName());
            ycqQzhActivity.setUpdateTime(new Date());
        }
    }
}