package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.commandDo.*;
import com.crunii.micro.service.qzh.converter.dcoument.YcqQzhCommandoLocalConverter;
import com.crunii.micro.service.qzh.jdbc.YcqQzhCommandoJdbcDao;
import com.crunii.micro.service.qzh.service.YcqQzhCommandoService;
import lombok.RequiredArgsConstructor;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@ToString
@Service
@Slf4j
@RequiredArgsConstructor
public class YcqQzhCommandoApiImpl implements YcqQzhCommandoService {

	private final YcqQzhCommandoRepo repo;

	private final YcqQzhCommandoJdbcDao jdbcDao;

	private final YcqQzhCommandoFileRepo fileRepo;

	private final YcqQzhCommandoOrgRepo commandoOrgRepo;
	@Autowired
	private YcqQzhCommandoLogRepo logRepo;

	@Autowired
	private YcqQzhCommandoFlowRepo flowRepo;

	@Autowired
	private YcqQzhCommandoReviwedRepo reviwedRepo;

	@Autowired
	private YcqQzhCommandoLocalConverter commandoLocalConverter;

	private Worker getWorker() {
		Worker worker = CurrWorkerUtil.getCurrWorker();
		String orgId = worker.getOrgId();
		String orgName=jdbcDao.findWorkerOrgName(orgId);
		if (null==worker.getOrgName()){
			worker.setOrgName(orgName);
		}
		return worker;
	}

	/**
	 * @description: 1:  申请人-创建突击队 （青字号申请页面）
	 * @date: 2023/9/15 17:29
	 */
	@Override
	public Result<String> save(SaveCommandoReqDTO saveCommandoReqDTO) {
		Worker currWorker = getWorker();

		//如果前端没有传id  证明新增内容
		if (null == saveCommandoReqDTO.getId()) {
			//  如果是保存草稿   传0  正式数据传1
			String isDraft = saveCommandoReqDTO.getIsDraft();
			if ("0".equals(isDraft)) {
				//草稿
				YcqQzhCommando entity = commandoLocalConverter.saveDtoToEntity(saveCommandoReqDTO);
				entity.setCreateId(String.valueOf(currWorker.getLoginId()));
				entity.setCreateBy(currWorker.getWorkerName());
				entity.setCreateTime(new Date());
				entity.setState(1);
				entity.setDelFlag(0);
				entity.setCommandoCode(getCode());
				repo.save(entity);

				// 保存附件内容
				//先删除再增加
				updateFile(saveCommandoReqDTO, entity);

				return Result.ofSuccess();
			} else {

				//校验数据
				try {
					checkSaveData(saveCommandoReqDTO);
				} catch (Exception e) {
					return Result.ofFail(e.getMessage());
				}
				//前端没处理--处理空格
				saveCommandoReqDTO.setCommandoName(saveCommandoReqDTO.getCommandoName().trim());
				saveCommandoReqDTO.setTeamLeaderCard(saveCommandoReqDTO.getTeamLeaderCard().trim());
				//申请人 队长  只能在同一只突击队
				//    队长 只能再一个突击队    突击队队名不能重复

				//1.先查突击队队名重复吗？   查 存在的 且 任务状态为 待审核和通过的
				YcqQzhCommando oldCommanDoName = jdbcDao.checkName(saveCommandoReqDTO);
				if (oldCommanDoName != null) {
					return Result.ofFail("突击队名称已经存在");
				}
				//1.查突击队队长在其他突击里没有   查 存在的 且 任务状态为 待审核和通过的
				YcqQzhCommando leaderCommando = jdbcDao.checkData(saveCommandoReqDTO, currWorker, getCode());
				if (leaderCommando != null) {
					return Result.ofFail("该队长已经存在，请重新选择");
				}

				//申请人只允许一条正在进行的流程
//				int count = jdbcDao.findStateById(currWorker.getLoginId());
//				if (count != 0) {
//					return Result.ofFail("有正在进行的流程，请先处理。");
//				}
				YcqQzhCommando entity = commandoLocalConverter.saveDtoToEntity(saveCommandoReqDTO);
				entity.setCreateId(String.valueOf(currWorker.getLoginId()));
				entity.setCreateBy(currWorker.getWorkerName());
				entity.setCreateTime(new Date());
				entity.setState(2);
				entity.setDelFlag(0);
				entity.setCommandoCode(getCode());
				entity.setBatchNoApply(1);
				repo.save(entity);

				//将团组织的上级组织都存起来，好让管理界面的人通过组织直接查看数据
				saveAllOrg(saveCommandoReqDTO, entity);

				//附件
				List<FileDTO> saveFile = saveCommandoReqDTO.getSaveFile();
				try {
					checkFileAndSave(saveFile);
				} catch (Exception e) {
					return Result.ofFail(e.getMessage());
				}
				updateFile(saveCommandoReqDTO, entity);

				//创建流程信息
				YcqQzhCommandoFlow flow = new YcqQzhCommandoFlow();
				flow.setCommandoCode(entity.getCommandoCode());
				flow.setFlowCreateTime(new Date());
				flow.setFlowType(1);
				flow.setFlowName("申报");
				flow.setFlowWorker(currWorker.getWorkerName());
				flow.setFlowWorkerCode(currWorker.getLoginCode());
				flow.setFlowWorkerOrg(currWorker.getOrgName());
				flow.setFlowWorkerOrgId(currWorker.getOrgId());
				flow.setBatchNoApply(entity.getBatchNoApply());
				flow.setState(2);
				flow.setApplyWorker(currWorker.getWorkerName());
				flow.setApplyWorkerCode(currWorker.getLoginCode());
				flow.setApplyTime(new Date());
				flowRepo.save(flow);

				//创建需要的审核人的  组织的并且审核按钮的
				//根据org_id  查所有的人 然后再插入
				String groupOrgId = saveCommandoReqDTO.getGroupOrgId();
				//
				List<Worker> workers = jdbcDao.findReviwed(groupOrgId);
				//删除以前的
				reviwedRepo.deleteApply(entity.getCommandoCode(), entity.getBatchNoApply());
				if (workers != null) {
					List<YcqQzhCommandoReviwed> reviwedList = new ArrayList<>();
					for (Worker worker : workers) {

						YcqQzhCommandoReviwed one = new YcqQzhCommandoReviwed();
						one.setBatchNoApply(entity.getBatchNoApply());
						one.setCommandoCode(entity.getCommandoCode());
						one.setCreateTime(new Date());
						one.setReviewedWorker(worker.getWorkerName());
						one.setReviewedWorkerCode(worker.getLoginCode());
						one.setType(1);
						reviwedList.add(one);
					}
					reviwedRepo.saveAll(reviwedList);
				}

				return Result.ofSuccess();
			}

		} else {
			//传了id 证明是编辑

			Optional<YcqQzhCommando> byId = repo.findById(saveCommandoReqDTO.getId());
			YcqQzhCommando ycqQzhCommando = byId.get();
			if (!(ycqQzhCommando.getState() == 1 || ycqQzhCommando.getState() == 4)) {
				return Result.ofFail("草稿状态才能编辑");
			}
			//  如果是保存草稿   传0  正式数据传1
			String isDraft = saveCommandoReqDTO.getIsDraft();
			if ("0".equals(isDraft)) {

				//草稿
				saveCommando(saveCommandoReqDTO, currWorker, ycqQzhCommando);
				ycqQzhCommando.setState(1);
				ycqQzhCommando.setDelFlag(0);
				repo.save(ycqQzhCommando);

				// 保存附件内容
				//先删除再增加
				updateFile(saveCommandoReqDTO, ycqQzhCommando);

				return Result.ofSuccess();
			} else {

				//申请人只允许一条正在进行的流程
//				int count = jdbcDao.findStateById(currWorker.getLoginId());
//				if (count != 0) {
//					return Result.ofFail("有正在进行的流程，请先处理。");
//				}
				//校验数据
				try {
					checkSaveData(saveCommandoReqDTO);
				} catch (Exception e) {
					return Result.ofFail(e.getMessage());
				}
				//前端没处理--处理空格
				saveCommandoReqDTO.setCommandoName(saveCommandoReqDTO.getCommandoName().trim());
				saveCommandoReqDTO.setTeamLeaderCard(saveCommandoReqDTO.getTeamLeaderCard().trim());
				//申请人 队长  只能在同一只突击队
				//1.先查突击队队名重复吗？   查 存在的 且 任务状态为 待审核和通过的
				YcqQzhCommando oldCommanDoName = jdbcDao.checkName(saveCommandoReqDTO);
				if (oldCommanDoName != null) {
					return Result.ofFail("突击队名称已经存在");
				}
				//1.查突击队队长在其他突击里没有   查 存在的 且 任务状态为 待审核和通过的
				YcqQzhCommando leaderCommando = jdbcDao.checkData(saveCommandoReqDTO, currWorker, getCode());
				if (leaderCommando != null) {
					return Result.ofFail("该队长已经存在，请重新选择");
				}

				saveCommando(saveCommandoReqDTO, currWorker, ycqQzhCommando);
				ycqQzhCommando.setState(2);
				ycqQzhCommando.setDelFlag(0);
				Integer batchNoApply = ycqQzhCommando.getBatchNoApply();
				if (batchNoApply == null) {
					batchNoApply = 1;
				}
				int i = batchNoApply + 1;
				ycqQzhCommando.setBatchNoApply(i);
				repo.save(ycqQzhCommando);

				// 保存附件内容
				List<FileDTO> saveFile = saveCommandoReqDTO.getSaveFile();
				try {
					checkFileAndSave(saveFile);
				} catch (Exception e) {
					return Result.ofFail(e.getMessage());
				}
				updateFile(saveCommandoReqDTO, ycqQzhCommando);

				//保存组织关系
				saveAllOrg(saveCommandoReqDTO, ycqQzhCommando);

				//创建流程信息
				YcqQzhCommandoFlow flow = new YcqQzhCommandoFlow();
				flow.setCommandoCode(ycqQzhCommando.getCommandoCode());
				flow.setFlowCreateTime(new Date());
				flow.setFlowType(1);
				flow.setFlowName("申报");
				flow.setFlowWorker(currWorker.getWorkerName());
				flow.setFlowWorkerCode(currWorker.getLoginCode());
				flow.setFlowWorkerOrg(currWorker.getOrgName());
				flow.setFlowWorkerOrgId(currWorker.getOrgId());
				flow.setBatchNoApply(i);
				flow.setApplyWorker(currWorker.getWorkerName());
				flow.setApplyWorkerCode(currWorker.getLoginCode());
				flow.setApplyTime(new Date());
				flow.setState(2);
				flowRepo.save(flow);

				//创建需要的审核人的
				//根据org_id  查所有的人 然后再插入
				String groupOrgId = saveCommandoReqDTO.getGroupOrgId();
				//删除之前的
				reviwedRepo.deleteApply(ycqQzhCommando.getCommandoCode(), ycqQzhCommando.getBatchNoApply());
				List<Worker> workers = jdbcDao.findReviwed(groupOrgId);
				if (workers != null) {
					List<YcqQzhCommandoReviwed> reviwedList = new ArrayList<>();
					for (Worker worker : workers) {

						YcqQzhCommandoReviwed one = new YcqQzhCommandoReviwed();
						one.setBatchNoApply(ycqQzhCommando.getBatchNoApply());
						one.setCommandoCode(ycqQzhCommando.getCommandoCode());
						one.setCreateTime(new Date());
						one.setReviewedWorker(worker.getWorkerName());
						one.setReviewedWorkerCode(worker.getLoginCode());
						one.setType(1);
						reviwedList.add(one);
					}
					reviwedRepo.saveAll(reviwedList);
				}

				return Result.ofSuccess();
			}
		}

	}

	private void saveCommando(SaveCommandoReqDTO saveCommandoReqDTO, Worker currWorker, YcqQzhCommando ycqQzhCommando) {
		ycqQzhCommando.setCommandoName(saveCommandoReqDTO.getCommandoName());
		ycqQzhCommando.setProjectName(saveCommandoReqDTO.getProjectName());
		ycqQzhCommando.setGroupOrgName(saveCommandoReqDTO.getGroupOrgName());
		ycqQzhCommando.setGroupOrgId(saveCommandoReqDTO.getGroupOrgId());
		ycqQzhCommando.setType(saveCommandoReqDTO.getType());
		ycqQzhCommando.setTeamLeaderName(saveCommandoReqDTO.getTeamLeaderName());
		ycqQzhCommando.setTeamLeaderCard(saveCommandoReqDTO.getTeamLeaderCard());
		ycqQzhCommando.setTeamLeaderPhone(saveCommandoReqDTO.getTeamLeaderPhone());
		ycqQzhCommando.setPersonNum(saveCommandoReqDTO.getPersonNum());
		ycqQzhCommando.setWorkSituation(saveCommandoReqDTO.getWorkSituation());
		ycqQzhCommando.setFileUrl(saveCommandoReqDTO.getFileUrl());
		ycqQzhCommando.setDistribution(saveCommandoReqDTO.getDistribution());
		ycqQzhCommando.setFileType(saveCommandoReqDTO.getFileType());
		ycqQzhCommando.setFileName(saveCommandoReqDTO.getFileName());
		ycqQzhCommando.setHaveThreeYearPlan(saveCommandoReqDTO.getHaveThreeYearPlan());
		ycqQzhCommando.setUpdateId(String.valueOf(currWorker.getLoginId()));
		ycqQzhCommando.setUpdateBy(currWorker.getWorkerName());
		ycqQzhCommando.setUpdateTime(new Date());
	}

	private void saveAllOrg(SaveCommandoReqDTO saveCommandoReqDTO, YcqQzhCommando entity) {

		//先删除之前的
		String commandoCode = entity.getCommandoCode();
		commandoOrgRepo.deleteByCommandoCode(commandoCode);

		//将团组织的上级组织都存起来，好让管理界面的人通过组织直接查看数据
		String groupOrgId1 = saveCommandoReqDTO.getGroupOrgId();
		List<Long> orgIds = new ArrayList<>();
		orgIds.add(Long.valueOf(groupOrgId1));
		findParentOrg(orgIds, Long.valueOf(groupOrgId1));
		//
		if (orgIds.size() > 0) {
			List<YcqQzhCommandoOrg> orgs = new ArrayList<>();
			for (Long orgId : orgIds) {
				YcqQzhCommandoOrg one = new YcqQzhCommandoOrg();
				one.setCommandoCode(entity.getCommandoCode());
				one.setOrgId(orgId);
				one.setCreateTime(new Date());
				orgs.add(one);
			}
			commandoOrgRepo.saveAll(orgs);
		}

	}

	/**
	 * @description: 附件验证和插入
	 * @date: 2023/9/18 15:34
	 */
	private void checkFileAndSave(List<FileDTO> saveFile) throws Exception {

		// //附件限制  5个  类型为 pdf/doc/xlx
		if (saveFile != null) {

			if (saveFile.size() > 5) {
				throw new Exception("最多上传5个附件");
			}
			for (FileDTO fileDTO : saveFile) {
				if (fileDTO.getFileSize() > 2048) {
					throw new Exception("每个文件大小请小于2M");
				}
				List<String> fileType = new ArrayList<>();
				fileType.add("xlsx");
				fileType.add("docx");
				fileType.add("xls");
				fileType.add("doc");
				fileType.add("pdf");
				if (!fileType.contains(fileDTO.getFileType())) {
					throw new Exception("请上传.docx/.xlsx/.pdf类型文件");
				}
			}
		}

	}

	/**
	 * @description: 附件插入
	 * @date: 2023/9/18 15:34
	 */
	private void updateFile(SaveCommandoReqDTO saveCommandoReqDTO, YcqQzhCommando entity) {
		fileRepo.updateFileState(entity.getCommandoCode());
		if (saveCommandoReqDTO.getSaveFile() != null) {
			List<YcqQzhCommandoFile> files = new ArrayList<>();
			for (FileDTO fileDTO : saveCommandoReqDTO.getSaveFile()) {

				YcqQzhCommandoFile one = new YcqQzhCommandoFile();
				one.setCommandoCode(entity.getCommandoCode());
				one.setState(1);
				one.setFileId(fileDTO.getFileId());
				one.setFileKey(fileDTO.getFileKey());
				one.setFileName(fileDTO.getFileName());
				one.setFileUrl(fileDTO.getFileUrl());
				one.setFileSize(fileDTO.getFileSize());
				one.setFileType(fileDTO.getFileType());
				one.setName(fileDTO.getName());
				one.setCreateTime(new Date());
				one.setBatchNoApply(entity.getBatchNoApply());
				files.add(one);
			}
			fileRepo.saveAll(files);
		}
	}

	/**
	 * @description: 校验提交数据
	 * @param: [saveCommandoReqDTO]
	 * @return: void
	 * @author 13760
	 * @date: 2023/9/18 15:34
	 */
	private void checkSaveData(SaveCommandoReqDTO saveCommandoReqDTO) throws Exception {
		//必填项
		// 青年突击队名称  文本，50字
		//建设项目名称     文本，50字
		//所属团组织
		//所属类别
		//是否为三年行动计划突击队
		//队长姓名    文本，10个字
		//队长身份证号   18位
		//队长电话   11位数字
		//队员人数  正整数>5
		String commandoName = saveCommandoReqDTO.getCommandoName();
		if (null == commandoName || commandoName.length() > 50) {
			throw new Exception("青年突击队名称为必填项且文字不超过50字");
		}
		String projectName = saveCommandoReqDTO.getProjectName();
//		if (null == projectName || projectName.length() > 50) {
//			throw new Exception("建设项目名称为必填项且文字不超过50字");
//		}
		if (ObjectUtil.isNotEmpty(projectName) && projectName.length() > 50) {
			throw new Exception("建设项目名称文字不超过50字");
		}
		String groupOrgName = saveCommandoReqDTO.getGroupOrgName();
		if (null == groupOrgName) {
			throw new Exception("所属团组织为必填项");
		}
		String type = saveCommandoReqDTO.getType();
		if (null == type) {
			throw new Exception("所属类别为必填项");
		}
		String teamLeaderName = saveCommandoReqDTO.getTeamLeaderName();
		if (null == teamLeaderName || teamLeaderName.length() > 10) {
			throw new Exception("队长姓名为必填项且不超过10字");
		}
		String teamLeaderCard = saveCommandoReqDTO.getTeamLeaderCard();
		String patt = "^[1-9]\\d{5}(18|19|([23]\\d))\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$";
		Pattern r = Pattern.compile(patt);
		Matcher matcher = r.matcher(teamLeaderCard);
		boolean b = matcher.find();
//        if (!b){
//            throw new Exception("请填写正确身份证");
//        }
		Long personNum = saveCommandoReqDTO.getPersonNum();
		if (null == personNum || personNum < 5) {
			throw new Exception("队员人数需大于等于5人");
		}
		//非必填项

		//工作任务情况	1000字
		String workSituation = saveCommandoReqDTO.getWorkSituation();
		if (null != workSituation && workSituation.length() > 1000) {
			throw new Exception("工作情况在1000字以内");
		}
		//附件	.docx/.xlsx/.pdf,小于5个，每个小于2M

	}

	/**
	 * @description: 3：突击队修改队长   修改（青字号页面  -修改 登陆人修改队长数据）
	 * @date: 2023/9/15 17:33
	 */
	@Override
	public Result<String> updateCommandoLeader(ChangeLeaderCommandoReqDTO dto) {

		Worker currWorker = getWorker();
		YcqQzhCommando commando = repo.getOne(Long.valueOf(dto.getId()));

		//待审核  和 已通过的才能变更队长
		if (!(commando.getState() == 2 || commando.getState() == 3)) {
			return Result.ofFail("待审核或已通过的数据才能变更队长");
		}
		//看新的队长是不是存在在其他突击队中
		if (!dto.getTeamLeaderCard().equals(commando.getTeamLeaderCard())) {
			YcqQzhCommando qzhCommando = jdbcDao.findCommanDoByCard(dto.getTeamLeaderCard());
			if (qzhCommando != null) {
				return Result.ofFail("变更的新队长已经存在于其他突击队");
			}
		}
		//传了applyChangeNum 证明是变更记录那修改   在那基础上修改
		if (dto.getApplyChangeNum() == null) {
			//根据comandoCode  查询突击队以前的变更队长流程走完没有
			YcqQzhCommandoFlow old = jdbcDao.findMaxFlow(commando.getCommandoCode());
			if (old != null) {
				long state = old.getState();
				//已通过3   终止6   不通过是回到草稿5
				if (!(state == 3 || state == 6)) {
					return Result.ofFail("存在正在进行的变更流程");
				}
			}
		}
		//在流程中的才有变更队长    记录变更队长的操作日志
		YcqQzhCommandoLog log = new YcqQzhCommandoLog();
		//根据comandcode查看以前是不是有变更记录了
		YcqQzhCommandoLog logOld = jdbcDao.findMaxLogByCode(commando.getCommandoCode());
		int batchNoChange;
		if (logOld != null) {
			batchNoChange = logOld.getBatchNo() + 1;
		} else {
			batchNoChange = 1;
		}
		//待审核 2
		log.setState(2);
		log.setCommandoCode(commando.getCommandoCode());
		log.setTeamLeaderCard(commando.getTeamLeaderCard());
		log.setTeamLeaderName(commando.getTeamLeaderName());
		log.setTeamLeaderPhone(commando.getTeamLeaderPhone());
		log.setTeamLeaderCardNew(dto.getTeamLeaderCard());
		log.setTeamLeaderNameNew(dto.getTeamLeaderName());
		log.setTeamLeaderPhoneNew(dto.getTeamLeaderPhone());
		log.setCreateTime(new Date());
		log.setCreateBy(currWorker.getWorkerName());
		log.setCreateId(currWorker.getLoginCode());
		log.setBatchNo(batchNoChange);
		logRepo.save(log);

		//变更队长记录到流程中，变更了审核成功了才会更改到主表中
		YcqQzhCommandoFlow flow = new YcqQzhCommandoFlow();
		flow.setCommandoCode(commando.getCommandoCode());
		flow.setFlowCreateTime(new Date());
		flow.setFlowType(2);
		flow.setFlowName("变更队长");
		flow.setFlowWorker(currWorker.getWorkerName());
		flow.setFlowWorkerCode(currWorker.getLoginCode());
		flow.setFlowWorkerOrg(currWorker.getOrgName());
		flow.setFlowWorkerOrgId(currWorker.getOrgId());
		flow.setBatchNoChange(batchNoChange);
		flow.setState(2);
		flow.setApplyWorker(currWorker.getWorkerName());
		flow.setApplyWorkerCode(currWorker.getLoginCode());
		flow.setApplyTime(new Date());

		if (dto.getApplyChangeNum() == null) {
			//证明是新的流程
			//根据code 查找以前是不是有流程了
			YcqQzhCommandoFlow changeNumFromFlow = jdbcDao.findChangeNumFromFlow(commando.getCommandoCode());
			if (changeNumFromFlow == null) {
				flow.setApplyChangeNum(1);
			} else {
				flow.setApplyChangeNum(changeNumFromFlow.getApplyChangeNum() + 1);
			}
		} else {
			flow.setApplyChangeNum(dto.getApplyChangeNum());
		}
		flowRepo.save(flow);

		//将变更队长的版本存在主表中
		commando.setBatchNoChange(batchNoChange);
		repo.save(commando);
		//创建需要的审核人的
		//根据org_id  查所有的人 然后再插入
		String groupOrgId = commando.getGroupOrgId();
		//
		reviwedRepo.deleteChange(commando.getCommandoCode(), flow.getBatchNoChange());
		List<Worker> workers = jdbcDao.findReviwed(groupOrgId);
		if (workers != null) {
			List<YcqQzhCommandoReviwed> reviwedList = new ArrayList<>();
			for (Worker worker : workers) {

				YcqQzhCommandoReviwed one = new YcqQzhCommandoReviwed();
				one.setBatchNoChange(flow.getBatchNoChange());
				one.setCommandoCode(commando.getCommandoCode());
				one.setCreateTime(new Date());
				one.setReviewedWorker(worker.getWorkerName());
				one.setReviewedWorkerCode(worker.getLoginCode());
				one.setType(2);
				reviwedList.add(one);
			}
			reviwedRepo.saveAll(reviwedList);
		}

		return Result.ofSuccess();
	}

	/**
	 * @description: 突击队修改   修改（青字号页面  -修改 登陆人修改申请数据）
	 * @date: 2023/9/15 17:32
	 */
	@Override
	public Result<String> updateCommando(CommandoUpdateReqDTO commandoUpdateReqDTO) {

		Worker currWorker = getWorker();
		//草稿才能编辑：
		//判断是不是在草稿状态
		Optional<YcqQzhCommando> byId = repo.findById(Long.valueOf(commandoUpdateReqDTO.getId()));
		YcqQzhCommando ycqQzhCommando = byId.get();
		Integer state = ycqQzhCommando.getState();
		if (state != 1) {
			return Result.ofFail("草稿状态才能编辑");
		}
		YcqQzhCommando entity = commandoLocalConverter.updateDtoToEntity(commandoUpdateReqDTO);
		entity.setUpdateId(currWorker.getLoginId() + "");
		entity.setUpdateBy(currWorker.getWorkerName());
		entity.setUpdateTime(new Date());
		repo.saveAndFlush(entity);
		return Result.ofSuccess();
	}

	/**
	 * @description: 4 突击队删除   删除（青字号页面  -删除 登陆人删除申请的数据）
	 * @date: 2023/9/15 17:31
	 */
	@Override
	public Result<String> deleteCommando(String id) {
		Worker currWorker = getWorker();
		jdbcDao.deleteById(Long.valueOf(id), currWorker.getLoginId(), currWorker.getWorkerName());
		return Result.ofSuccess();
	}

	/**
	 * @description: 突击队详情查询   （查询单条数据详细的信息）
	 * @date: 2023/9/15 17:31
	 */
	@Override
	public Result<CommandoDetailsRespDTO> queryById(String id) {

		YcqQzhCommando commando = jdbcDao.findCommanDoOne(id);
		CommandoDetailsRespDTO dto = commandoLocalConverter.entityToVo(commando);
		String commandoCode = dto.getCommandoCode();
		List<FileDTO> fileDTOS = setFileData(commandoCode);
		dto.setFile(fileDTOS);
		return Result.ofSuccess(dto);
	}

	/**
	 * @description:突击队分页申请人查询 2  查询（青字号页面  -查询 登陆人所申请的数据）
	 * @date: 2023/9/15 17:30
	 */
	@Override
	public Result<PageRsp<QueryCommandoRespDTO>> queryCommandoByApplicant(PageReq<QueryCommandoReqDTO> queryCommandoReqDTO) {

		Worker currWorker = getWorker();
		PageRsp<QueryCommandoRespDTO> rsp = jdbcDao.pageListByApplicant(queryCommandoReqDTO, currWorker);

		//关联附件
		List<QueryCommandoRespDTO> result = rsp.getResult();
		if (result != null) {
			for (QueryCommandoRespDTO queryCommandoRespDTO : result) {
				String commandoCode = queryCommandoRespDTO.getCommandoCode();
				List<FileDTO> fileDTOS = setFileData(commandoCode);
				queryCommandoRespDTO.setFile(fileDTOS);
				//插入队伍状态
				String commandoState = setCommandoState(commandoCode);
				queryCommandoRespDTO.setCommandoState(commandoState);
			}
		}

		return Result.ofSuccess(rsp);
	}

	/**
	 * @description:突击队分页审批人查询 （突击队管理页面）
	 * @date: 2023/9/15 16:33
	 */
	@Override
	public Result<PageRsp<QueryCommandoRespDTO>> queryCommandoByExamine(PageReq<QueryCommandoReqDTO> queryCommandoReqDTO) {

		Worker currWorker = getWorker();

		//  审批人 可以看到哪些id 查看权限 所属组织下的所有团组织 （   比如B团组织再A下面   小木是A团组织的   小木能看到  A，B所有的  ）
		List<Long> allOrgList = new ArrayList<>();
		allOrgList.add(Long.valueOf(currWorker.getOrgId()));
		List<Long> orgList = jdbcDao.selecetOrgs(currWorker.getLoginId());
		allOrgList.addAll(orgList);
		List<String> commandoCodes = jdbcDao.selectCommandoCode(allOrgList);
		if (null == commandoCodes || commandoCodes.size() == 0) {
			PageRsp<QueryCommandoRespDTO> resultPage = new PageRsp();
			return Result.ofSuccess(resultPage);
		}
		// PageRsp<QueryCommandoRespDTO> rsp = jdbcDao.queryCommandoByExamine(queryCommandoReqDTO, currWorker, commandoCodes);
		PageRsp<QueryCommandoRespDTO> rsp = jdbcDao.queryCommandoForAudit(queryCommandoReqDTO, currWorker, commandoCodes);
		List<QueryCommandoRespDTO> result = rsp.getResult();
		if (result != null) {
			result.stream().map(audit -> audit = this.setShowFlag(audit)).collect(Collectors.toList());
			//插入附件   和插入队伍状态
			for (QueryCommandoRespDTO queryCommandoRespDTO : result) {
				String commandoCode = queryCommandoRespDTO.getCommandoCode();
				List<FileDTO> fileDTOS = setFileData(commandoCode);
				queryCommandoRespDTO.setFile(fileDTOS);
			}
		}

		return Result.ofSuccess(rsp);
	}

	/**
	 * @description:突击队分页审批人查询 给其他页面调用  返回LIst （突击队管理页面）
	 * @date: 2023/9/15 16:33
	 */
	@Override
	public List<QueryCommandoRespDTO> queryCommandoByExamineList(QueryCommandoReqDTO queryCommandoReqDTO) {

		Worker currWorker = getWorker();

		//  审批人 可以看到哪些id 查看权限 所属组织下的所有团组织 （   比如B团组织再A下面   小木是A团组织的   小木能看到  A，B所有的  ）
		List<Long> allOrgList = new ArrayList<>();
		allOrgList.add(Long.valueOf(currWorker.getOrgId()));
		List<Long> orgList = jdbcDao.selecetOrgs(currWorker.getLoginId());
		allOrgList.addAll(orgList);
		List<String> commandoCodes = jdbcDao.selectCommandoCode(allOrgList);
		if (null == commandoCodes || commandoCodes.size() == 0) {
			List<QueryCommandoRespDTO> resultPage = new ArrayList<>();
			return resultPage;
		}
		List<QueryCommandoRespDTO> result = jdbcDao.queryCommandoForAuditList(queryCommandoReqDTO, currWorker, commandoCodes);
		if (result != null) {
			result.stream().map(audit -> audit = this.setShowFlag(audit)).collect(Collectors.toList());
			//插入附件   和插入队伍状态
			for (QueryCommandoRespDTO queryCommandoRespDTO : result) {
				String commandoCode = queryCommandoRespDTO.getCommandoCode();
				List<FileDTO> fileDTOS = setFileData(commandoCode);
				queryCommandoRespDTO.setFile(fileDTOS);
			}
		}
		return result;
	}

	/**
	 * @description: 给每条数据插入队伍状态
	 * @param: [commandoCode]
	 * @return: java.lang.String
	 * @author 13760
	 * @date: 2023/9/27 11:05
	 */
	private String setCommandoState(String commandoCode) {

		return jdbcDao.saveCommandoState(commandoCode);
	}

	/**
	 * @description: 给每条数据添加 是不是有显示审核按钮
	 * @param: [audit]
	 * @return: com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoRespDTO
	 * @author 13760
	 * @date: 2023/9/23 14:39
	 */
	private QueryCommandoRespDTO setShowFlag(QueryCommandoRespDTO audit) {

		Worker worker = getWorker();
		//   审核 权限 是给  所属团组织 配了按钮的人看的
		String groupOrgId = audit.getGroupOrgId();
		if (worker.getOrgId().equals(groupOrgId)) {
			//所属组织一样 并且配有审核按钮
			//查看是不是配有审核按钮
			int num = jdbcDao.findReviewButton(worker.getLoginId());
			if (num == 0) {
				audit.setShowFlag(false);
			} else {
				audit.setShowFlag(true);
			}
		} else {
			audit.setShowFlag(false);
		}
		return audit;
	}

	/**
	 * @description:突击队审核 （突击队管理页面）
	 * @date: 2023/9/15 17:30
	 */
	@Override
	public Result<String> saveExamineCommando(ExamineCommandoReqDTO dto) {

		Worker currWorker = getWorker();

		List<String> ids = new ArrayList<>();
		if (Objects.nonNull(dto.getCommandoId())) {
			ids = Arrays.asList(dto.getCommandoId().split(","));
		}
		for (String id : ids) {
			//看是变更队长的审核  还是正常申报的审核
			//正常的申报审核
			if (dto.getExamineType() == 1) {

				// 申报记录的  审核   3通过 （状态为已通过）  4：回退（状态变更为草稿1）  5 不通过（ 状态为已驳回 流程结束）
				Optional<YcqQzhCommando> byId = repo.findById(Long.valueOf(id));
				boolean present = byId.isPresent();
				if (present) {
					YcqQzhCommando ycqQzhCommando = byId.get();
					//后端判断一下  待审核状态的才能审核
					if (ycqQzhCommando.getState() != 2) {
						return Result.ofFail("待审核状态才能审核");
					}
					//  只有数据的团组织人  才能审核
					long examineResult = dto.getExamineResult();
					if (examineResult == 4) {
						// 如果是回退  就到草稿状态去
						ycqQzhCommando.setState(1);
					} else {
						ycqQzhCommando.setState(dto.getExamineResult());
					}
					ycqQzhCommando.setExamineReason(dto.getExamineReason());
					if (dto.getExamineResult() == 5 && StringUtils.isEmpty(dto.getExamineReason())) {
						//不通过必须填写理由
						return Result.ofFail("不通过请说明理由");
					}
					// 新增 审核流程
					YcqQzhCommandoFlow flow = new YcqQzhCommandoFlow();
					//查询之前申报的信息
					YcqQzhCommandoFlow oldFlow = jdbcDao.findFlow(ycqQzhCommando.getCommandoCode(), ycqQzhCommando.getBatchNoApply(), null, 1);
					if (oldFlow != null) {
						flow.setApplyWorker(oldFlow.getApplyWorker());
						flow.setApplyWorkerCode(oldFlow.getApplyWorkerCode());
						flow.setApplyTime(oldFlow.getApplyTime());
					} else {
						flow.setApplyWorker(currWorker.getWorkerName());
						flow.setApplyWorkerCode(currWorker.getLoginCode());
						flow.setApplyTime(new Date());
					}
					flow.setFlowName("审核");
					flow.setCommandoCode(ycqQzhCommando.getCommandoCode());
					flow.setFlowType(1);
					flow.setFlowCreateTime(new Date());
					flow.setFlowWorker(currWorker.getWorkerName());
					flow.setFlowWorkerCode(currWorker.getLoginCode());
					flow.setFlowWorkerOrg(currWorker.getOrgName());
					flow.setFlowWorkerOrgId(currWorker.getOrgId());
					flow.setState(dto.getExamineResult());
					flow.setBatchNoApply(ycqQzhCommando.getBatchNoApply());
					flow.setDealText(dto.getExamineReason());
					repo.save(ycqQzhCommando);
					flowRepo.save(flow);
				}
			} else {

				//   变更队长的 审核  ： 3  通过 （状态为已通过）   5 不通过（ 状态为为草稿 ）
				//是变更队长的审核
				Optional<YcqQzhCommando> byId = repo.findById(Long.valueOf(id));
				boolean present = byId.isPresent();
				if (present) {
					YcqQzhCommando ycqQzhCommando = byId.get();
					//   只有数据的团组织人 配了审核按钮的 才能审核
					YcqQzhCommandoFlow flow = new YcqQzhCommandoFlow();
					Integer batchNoChange1 = ycqQzhCommando.getBatchNoChange();
					YcqQzhCommandoFlow old = jdbcDao.findFlowByCode(ycqQzhCommando.getCommandoCode(), batchNoChange1);
					//查询之前申报的信息
					YcqQzhCommandoFlow oldFlow = jdbcDao.findFlow(ycqQzhCommando.getCommandoCode(), null, batchNoChange1, 2);
					if (oldFlow != null) {
						flow.setApplyWorker(oldFlow.getApplyWorker());
						flow.setApplyWorkerCode(oldFlow.getApplyWorkerCode());
						flow.setApplyTime(oldFlow.getApplyTime());
					} else {
						flow.setApplyWorker(currWorker.getWorkerName());
						flow.setApplyWorkerCode(currWorker.getLoginCode());
						flow.setApplyTime(new Date());
					}
					flow.setFlowName("变更队长审核");
					flow.setCommandoCode(ycqQzhCommando.getCommandoCode());
					flow.setFlowType(2);
					flow.setFlowCreateTime(new Date());
					flow.setFlowWorker(currWorker.getWorkerName());
					flow.setFlowWorkerCode(currWorker.getLoginCode());
					flow.setFlowWorkerOrg(currWorker.getOrgName());
					flow.setFlowWorkerOrgId(currWorker.getOrgId());
					flow.setBatchNoChange(batchNoChange1);
					flow.setState(dto.getExamineResult());
					if (old != null) {
						flow.setApplyChangeNum(old.getApplyChangeNum());
					} else {
						flow.setApplyChangeNum(1);
					}

					if (dto.getExamineResult() == 5 && StringUtils.isEmpty(dto.getExamineReason())) {
						//不通过必须填写理由
						return Result.ofFail("不通过请说明理由");
					}
					flow.setDealText(dto.getExamineReason());
					//如果审核通过 则将主表的变更队长信息更改了
					flowRepo.save(flow);
					if (dto.getExamineResult() == 3) {
						//查出 变更队长记录表的信息
						//通过code+batchChange
						String commandoCode = ycqQzhCommando.getCommandoCode();
						Integer batchNoChange = ycqQzhCommando.getBatchNoChange();
						YcqQzhCommandoLog log = jdbcDao.findByCodeAndBatchChange(commandoCode, batchNoChange);
						if (log != null) {
							ycqQzhCommando.setTeamLeaderCard(log.getTeamLeaderCardNew());
							ycqQzhCommando.setTeamLeaderName(log.getTeamLeaderNameNew());
							ycqQzhCommando.setTeamLeaderPhone(log.getTeamLeaderPhoneNew());
							ycqQzhCommando.setUpdateBy(log.getCreateBy());
							ycqQzhCommando.setUpdateId(log.getCreateId() + "");
							ycqQzhCommando.setUpdateTime(log.getCreateTime());
							repo.save(ycqQzhCommando);
						}
					}
				}
			}
		}
		return Result.ofSuccess();
	}

	/**
	 * @description: 突击队排行榜
	 * @date: 2023/9/15 17:32
	 */
	@Override
	public Result<PageRsp<CommandoRankListDetilRespDTO>> updateRankingCommandoList(PageReq<CommandoRankListReqDTO> pageReq) {
		//        Worker currWorker = CurrWorkerUtil.getCurrWorker();
		Worker currWorker = getWorker();
		// 定时任务改为这里执行 临时使用 这里直接 获取突击队排行入库
//        jdbcDao.rankingCommando();
//        return Result.ofSuccess(jdbcDao.rankingCommandoPage(pageReq, currWorker));
		//按积分排名
		//1、登录积分：队长和创建人，每天最多分别积1分
		//2、风采或活动积分：关联了当前突击队的风采或活动，发布几个积几分
//        Worker currWorker = new Worker();
//        currWorker.setLoginId(9999);
//        currWorker.setMobile("13996280848");
		PageRsp<CommandoRankListDetilRespDTO> commandoRankListDetilRespDTOPageRsp = jdbcDao.selectFcHd(pageReq, currWorker);
		//再去获取登录的积分相加
		for (CommandoRankListDetilRespDTO commandoRankListDetilRespDTO : commandoRankListDetilRespDTOPageRsp.getResult()) {
			//队长和创建人分别去找对应的登录次数
			Integer loginNumber = jdbcDao.selectLogin(commandoRankListDetilRespDTO.getCommandoCode());
			Integer loginQueue = jdbcDao.selectQueue(commandoRankListDetilRespDTO.getCommandoCode());
			Integer amount = commandoRankListDetilRespDTO.getCollectNum() + loginNumber + loginQueue;
			commandoRankListDetilRespDTO.setCollectNum(amount);
		}
		List<CommandoRankListDetilRespDTO> collect = commandoRankListDetilRespDTOPageRsp.getResult().stream()
				.sorted(Comparator.comparing(CommandoRankListDetilRespDTO::getCollectNum).reversed()).collect(Collectors.toList());
		for (int i = 0; i < collect.size(); i++) {
			if (1 == pageReq.getPageNo()) {
				collect.get(i).setRankNum(i + 1);
			} else {
				collect.get(i).setRankNum((pageReq.getPageNo() - 1) * 10 + i + 1);
			}
		}
		commandoRankListDetilRespDTOPageRsp.setResult(collect);
		return Result.ofSuccess(commandoRankListDetilRespDTOPageRsp);
	}

	/**
	 * @description: 当前登陆人突击队草稿查询
	 * @date: 2023/9/15 17:32
	 */
	@Override
	public Result<CommandoDetailsRespDTO> queryCommandoDraft() {

		Worker currWorker = getWorker();
		CommandoDetailsRespDTO respDTO = jdbcDao.findDraftByWorker(currWorker);
		return new Result<>(true, respDTO);
	}

	/**
	 * @description: 突击队申请人变更记录查询
	 * @param: [token, dto]
	 * @return: com.crunii.micro.common.dto.Result
	 * @author 13760
	 * @date: 2023/9/18 14:54
	 */
	@Override
	public Result<PageRsp<QueryCommandoChangeRespDTO>> queryApplicantChange(PageReq<QueryCommandoReqDTO> dto) {

		Worker worker = getWorker();

		//只查询变更了队长的记录
		//状态(草稿 1 待审核2  已通过3   5不通过=草稿  6终止)
		// List<QueryCommandoChangeRespDTO> result = jdbcDao.findChangeLeader(dto, worker, 1, null);
		PageRsp<QueryCommandoChangeRespDTO> changeLeader = jdbcDao.findChangeLeader(dto, worker, 1, null);
		List<QueryCommandoChangeRespDTO> result = changeLeader.getResult();
		if (result != null) {
			for (QueryCommandoChangeRespDTO queryCommandoChangeRespDTO : result) {
				//5不通过=草稿
				if (queryCommandoChangeRespDTO.getState() == 5) {
					queryCommandoChangeRespDTO.setState(1);
				}
				String commandoCode = queryCommandoChangeRespDTO.getCommandoCode();
				List<FileDTO> fileDTOS = setFileData(commandoCode);
				queryCommandoChangeRespDTO.setFile(fileDTOS);
			}
		}
		return new Result<>(true, changeLeader);
	}

	/**
	 * @description: 突击队审批人变更记录查询
	 * @param: [token, dto]
	 * @return: com.crunii.micro.common.dto.Result
	 * @author 13760
	 * @date: 2023/9/18 14:54
	 */
	@Override
	public Result<PageRsp<QueryCommandoChangeRespDTO>> queryApplicantExamine(PageReq<QueryCommandoReqDTO> dto) {

		Worker worker = getWorker();

		//  审批人 可以看到哪些id 查看权限 所属组织下的所有团组织 （   比如B团组织再A下面   小木是A团组织的   小木能看到  A，B所有的  ）
		List<Long> allOrgList = new ArrayList<>();
		allOrgList.add(Long.valueOf(worker.getOrgId()));
		List<Long> orgList = jdbcDao.selecetOrgs(worker.getLoginId());
		allOrgList.addAll(orgList);
		List<String> commandoCodes = jdbcDao.selectCommandoCode(allOrgList);
		if (null == commandoCodes || commandoCodes.size() == 0) {
			PageRsp<QueryCommandoChangeRespDTO> resultPage = new PageRsp();
			return Result.ofSuccess(resultPage);
		}
		//只查询变更了队长的记录
		//状态(草稿 1 待审核2  已通过3   5不通过=草稿  6终止)
		//List<QueryCommandoChangeRespDTO> result = jdbcDao.findChangeLeader(dto, worker, 2, commandoCodes);
		PageRsp<QueryCommandoChangeRespDTO> changeForAudit = jdbcDao.findChangeForAudit(dto, worker, 2, commandoCodes);
		List<QueryCommandoChangeRespDTO> result = changeForAudit.getResult();
		if (result != null) {
			result.stream().map(audit -> audit = this.setShowFlagChange(audit)).collect(Collectors.toList());

			for (QueryCommandoChangeRespDTO queryCommandoChangeRespDTO : result) {
				String commandoCode = queryCommandoChangeRespDTO.getCommandoCode();
				List<FileDTO> fileDTOS = setFileData(commandoCode);
				queryCommandoChangeRespDTO.setFile(fileDTOS);
			}
		}
		return new Result<>(true, changeForAudit);
	}

	/**
	 * @description: 审核人变更记录查询  给其他接口调用
	 * @param: [dto]
	 * @return: java.util.List<com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoChangeRespDTO>
	 * @author 13760
	 * @date: 2023/10/20 15:28
	 */
	@Override
	public List<QueryCommandoChangeRespDTO> queryApplicantExamineList(QueryCommandoReqDTO dto){
		Worker worker = getWorker();

		//  审批人 可以看到哪些id 查看权限 所属组织下的所有团组织 （   比如B团组织再A下面   小木是A团组织的   小木能看到  A，B所有的  ）
		List<Long> allOrgList = new ArrayList<>();
		allOrgList.add(Long.valueOf(worker.getOrgId()));
		List<Long> orgList = jdbcDao.selecetOrgs(worker.getLoginId());
		allOrgList.addAll(orgList);
		List<String> commandoCodes = jdbcDao.selectCommandoCode(allOrgList);
		if (null == commandoCodes || commandoCodes.size() == 0) {
			List<QueryCommandoChangeRespDTO> resultPage = new ArrayList<>();
			return resultPage;
		}
		//只查询变更了队长的记录
		//状态(草稿 1 待审核2  已通过3   5不通过=草稿  6终止)
		List<QueryCommandoChangeRespDTO> result = jdbcDao.findChangeForAuditList(dto, worker, 2, commandoCodes);
		if (result != null) {
			result.stream().map(audit -> audit = this.setShowFlagChange(audit)).collect(Collectors.toList());

			for (QueryCommandoChangeRespDTO queryCommandoChangeRespDTO : result) {
				String commandoCode = queryCommandoChangeRespDTO.getCommandoCode();
				List<FileDTO> fileDTOS = setFileData(commandoCode);
				queryCommandoChangeRespDTO.setFile(fileDTOS);
			}
		}
		return result;

	}
	/**
	 * @description: 变更队长的审批人
	 * @param: [audit]
	 * @return: com.crunii.micro.service.qzh.api.domain.dto.commandDo.QueryCommandoChangeRespDTO
	 * @author 13760
	 * @date: 2023/9/23 15:10
	 */
	private QueryCommandoChangeRespDTO setShowFlagChange(QueryCommandoChangeRespDTO audit) {
		Worker worker = getWorker();
		String groupOrgId = audit.getGroupOrgId();
		if (worker.getOrgId().equals(groupOrgId)) {
			//所属组织一样 并且配有审核按钮
			//查看是不是配有审核按钮
			int num = jdbcDao.findReviewButton(worker.getLoginId());
			if (num == 0) {
				audit.setShowFlag(false);
			} else {
				audit.setShowFlag(true);
			}
		} else {
			audit.setShowFlag(false);
		}
		return audit;
	}

	/**
	 * @description: 查询变更队长的详细信息（前后）
	 * @param: [dto]
	 * @return: com.crunii.micro.common.dto.Result<com.crunii.micro.service.qzh.api.domain.dto.commandDo.ChangeLeaderCommandoResqDTO>
	 * @author 13760
	 * @date: 2023/9/20 15:05
	 */

	@Override
	public Result<ChangeLeaderCommandoRespDTO> queryChangeData(QueryCommandoReqDTO dto) {

		Integer batchNoChange = dto.getBatchNoChange();
		String commandoCode = dto.getCommandoCode();
		YcqQzhCommandoLog byCodeAndBatchChange = jdbcDao.findByCodeAndBatchChange(commandoCode, batchNoChange);
		ChangeLeaderCommandoRespDTO result = new ChangeLeaderCommandoRespDTO();
		if (null != byCodeAndBatchChange) {
			result.setCommandoCode(byCodeAndBatchChange.getCommandoCode());
			result.setTeamLeaderCard(byCodeAndBatchChange.getTeamLeaderCard());
			result.setTeamLeaderName(byCodeAndBatchChange.getTeamLeaderName());
			result.setTeamLeaderPhone(byCodeAndBatchChange.getTeamLeaderPhone());
			result.setTeamLeaderCardNew(byCodeAndBatchChange.getTeamLeaderCardNew());
			result.setTeamLeaderNameNew(byCodeAndBatchChange.getTeamLeaderNameNew());
			result.setTeamLeaderPhoneNew(byCodeAndBatchChange.getTeamLeaderPhoneNew());
		}
		return new Result<>(true, result);
	}

	/**
	 * @description: 突击队流程查询
	 * @param: [token, dto]
	 * @return: com.crunii.micro.common.dto.Result
	 * @author 13760
	 * @date: 2023/9/18 14:54
	 */
	@Override
	public Result<List<CommandoFlowRespDTO>> queryFlow(QueryCommandoReqDTO dto) {

		List<CommandoFlowRespDTO> respDTOS = jdbcDao.findFlowByCommandoCode(dto.getCommandoCode());
		if (respDTOS != null) {
			//审核的步骤  把附件信息填进去
			// respDTOS.stream().map(audit -> audit = this.setFileData(audit)).collect(Collectors.toList());
			//附件信息
			for (CommandoFlowRespDTO respDTO : respDTOS) {
				//审核步骤需要附件
				if ("申报".equals(respDTO.getDealFlowName())) {
					List<FileDTO> fileDTOS = setFlowFileData(respDTO);
					respDTO.setFile(fileDTOS);
				}
			}
		}
		return new Result<>(true, respDTOS);
	}

	private List<FileDTO> setFlowFileData(CommandoFlowRespDTO respDTO) {
		return jdbcDao.findFlowFile(respDTO);
	}

	/**
	 * @description: 流程详细  把审核流程的  关联附件进去
	 * @param: [audit]
	 * @return: com.crunii.micro.service.qzh.api.domain.dto.commandDo.CommandoFlowRespDTO
	 * @author 13760
	 * @date: 2023/9/25 16:05
	 */
	private List<FileDTO> setFileData(String commandoCode) {
		return jdbcDao.findFileByCommandoCode(commandoCode);
	}

	/**
	 * @description: 突击队申报页面变更记录草稿终止任务
	 * @date: 2023/9/22 14:16
	 */
	@Override
	public Result saveEndChangeTask(ExamineCommandoReqDTO dto) {

		Worker currWorker = getWorker();
		// 变更队长 终止任务 终止任务   状态给 6 代表终止任务
		// 草稿状态下才能终止
		//查询当前 1 和 5都为草稿状态
		YcqQzhCommandoFlow old = jdbcDao.findFlowByCode(dto.getCommandoCode(), dto.getBatchNoChange());
		if (!(old.getState() == 1 || old.getState() == 5)) {
			return Result.ofFail("草稿状态才能终止任务");
		}
		//新建流程 终止
		// 新增 审核流程
		YcqQzhCommandoFlow flow = new YcqQzhCommandoFlow();
		//查询之前申报的信息
		flow.setApplyWorker(currWorker.getWorkerName());
		flow.setApplyWorkerCode(currWorker.getLoginCode());
		flow.setApplyTime(new Date());
		flow.setFlowName("终止变更任务");
		flow.setCommandoCode(dto.getCommandoCode());
		flow.setFlowType(2);
		flow.setFlowCreateTime(new Date());
		flow.setFlowWorker(currWorker.getWorkerName());
		flow.setFlowWorkerCode(currWorker.getLoginCode());
		flow.setFlowWorkerOrg(currWorker.getOrgName());
		flow.setFlowWorkerOrgId(currWorker.getOrgId());
		flow.setState(dto.getExamineResult());
		flow.setBatchNoChange(dto.getBatchNoChange());
		flow.setApplyChangeNum(old.getApplyChangeNum());
		flowRepo.save(flow);
		return Result.ofSuccess();
	}

	/**
	 * @description: 突击队统计待当前用户处理的数据条数
	 * @param: [type]
	 * @return: com.crunii.micro.common.dto.Result
	 * @author 13760
	 * @date: 2023/9/23 15:47
	 */
	@Override
	public Result<CommandoPendingNumRespDTO> pendingNum(QueryCommandoReqDTO dto) {
		String type = dto.getType();
		CommandoPendingNumRespDTO result = new CommandoPendingNumRespDTO();
		Worker worker = getWorker();
		//传1 代表查 申报页面的
		if ("1".equals(type)) {
			//申报清单的
			int applyNum = jdbcDao.pendingNum(worker);
			int applyChangeNum = jdbcDao.pendingChangeNum(worker);
			//变更记录的
			result.setApplyNum(applyNum);
			result.setChangeNum(applyChangeNum);
		} else {
			List<QueryCommandoRespDTO> respDTOS = jdbcDao.pendingExApplyNum(worker);
			//List<QueryCommandoRespDTO> respDTOS = this.queryCommandoByExamineList(dto);
			List<QueryCommandoChangeRespDTO> changeNum = jdbcDao.pendingExChangeNum(worker);
			//List<QueryCommandoChangeRespDTO> changeNum = this.queryApplicantExamineList(dto);
			if (respDTOS != null) {

				List<QueryCommandoRespDTO> collect = respDTOS.stream().map(audit -> audit = this.setShowFlag(audit)).collect(Collectors.toList());
				List<QueryCommandoRespDTO> collect1 = collect.stream().filter(s -> s.getState() == 2).collect(Collectors.toList());
				List<QueryCommandoRespDTO> collect2 = collect1.stream().filter(QueryCommandoRespDTO::getShowFlag).collect(Collectors.toList());
				int size = collect2.size();
				result.setExamineApplyNum(size);
			}
			if (changeNum != null) {
				List<QueryCommandoChangeRespDTO> collect3 = changeNum.stream().map(audit -> audit = this.setShowFlagChange(audit))
						.collect(Collectors.toList());
				List<QueryCommandoChangeRespDTO> collect4 = collect3.stream().filter(s -> s.getState() == 2)
						.filter(QueryCommandoChangeRespDTO::getShowFlag).collect(Collectors.toList());
				int changeSize = collect4.size();
				result.setExamineChangeNum(changeSize);
			}
		}

		return new Result<>(true, result);
	}

	public static String getCode() {
		String code = "TJD";
		SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
		Date date = new Date();
		String now = format.format(date);
		code += now;
		char[] arr = new char[62];
		for (int i = 0; i < 26; i++) {
			arr[i] = (char) ('A' + i);
		}
		for (int i = 0; i < 26; i++) {
			arr[i + 26] = (char) ('a' + i);
		}
		for (int i = 0; i < 10; i++) {
			arr[i + 52] = (char) (48 + i);
		}
		Random random = new Random();
		String str = "";
		for (int i = 0; i < 4; i++) {
			str += arr[random.nextInt(62)];
		}
		code += str;
		return code;
	}

	/**
	 * 逆向递归
	 *
	 * @param list,orgId
	 */
	private void findParentOrg(List<Long> list, Long orgId) {
		TsmOrg tsmOrg = jdbcDao.findParentOrgs(orgId);
		if (0 == tsmOrg.getParentOrgId()) {
			list.add(tsmOrg.getOrgId());
		} else {
			list.add(tsmOrg.getParentOrgId());
			findParentOrg(list, tsmOrg.getParentOrgId());
		}
	}

}
