package com.crunii.micro.service.qzh.service.impl;


import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.document.YcqQzhAnnex;
import com.crunii.micro.domain.entity.main.document.YcqQzhCover;
import com.crunii.micro.domain.entity.main.document.YcqQzhDocument;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.domain.repo.main.YcqQzhBusinessAllRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhAnnexRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhBrowseRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhCoverRepo;
import com.crunii.micro.domain.repo.main.document.YcqQzhDocumentRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.document.*;
import com.crunii.micro.service.qzh.jdbc.YcqQzhDocumentJdbcDao;
import com.crunii.micro.service.qzh.service.YcqQzhDocumentService;
import com.crunii.micro.service.qzh.util.PictureEncapsulationUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qzh.api.constants.QzhConstants.*;

/**
 * @Description: 政策ServiceImpl
 * @Param:
 * @return:
 * @Author: xiang xin
 * @Date: 2023/9/18
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class YcqQzhDocumentServiceImpl implements YcqQzhDocumentService {

    private final YcqQzhDocumentRepo ycqQzhDocumentRepo;
    private final YcqQzhDocumentJdbcDao ycqQzhDocumentJdbcDao;
    private final YcqQzhCoverRepo ycqQzhCoverRepo;
    private final YcqQzhAnnexRepo ycqQzhAnnexRepo;
    private final YcqQzhBrowseRepo ycqQzhBrowseRepo;
    private final YcqQzhBusinessAllRepo ycqQzhBusinessAllRepo;
    private final TsmOrgRepo tsmOrgRepo;


    /**
     * 创建政策
     *
     * @param req
     * @return
     */
    @Override
    public Result<String> createDocument(CreateDocumentReqDTO req) {
        //验证
        validateCreateDocument(req);
        Worker worker = getWorker();
        Snowflake snowflake = new Snowflake();
        YcqQzhDocument ycqQzhDocument = new YcqQzhDocument();
        ycqQzhDocument.setId(snowflake.nextId());
        ycqQzhDocument.setDocumentTitle(req.getDocumentTitle());
        ycqQzhDocument.setDocumentType(req.getDocumentType());
        ycqQzhDocument.setDocumentText(req.getDocumentText());
        ycqQzhDocument.setOpenCompetence(req.getOpenCompetence());
        //政策文件号可以为空
        ycqQzhDocument.setDocumentNum(Optional.ofNullable(req.getDocumentNum()).orElse(null));
        //创建状态默认为1
        ycqQzhDocument.setState(1);
        if (null != req.getApplyEntrance() && "1".equals(req.getApplyEntrance())) {
            if (null != req.getBeginTime()) {
                ycqQzhDocument.setBeginTime(req.getBeginTime());
            } else {
                return Result.ofFail("有效开始时间不能为空");
            }
            if (null != req.getEndTime()) {
                ycqQzhDocument.setEndTime(req.getEndTime());
            } else {
                return Result.ofFail("有效结束时间不能为空");
            }
            if (null != req.getApplyType()) {
                ycqQzhDocument.setApplyType(req.getApplyType());
            } else {
                return Result.ofFail("申请类型不能为空");
            }
            ycqQzhDocument.setApplyEntrance("1");
        } else {
            ycqQzhDocument.setApplyEntrance("2");
        }
        //封装创建人信息
        saveCreate(ycqQzhDocument);
        //组织封装
//        orgSelect(worker, ycqQzhDocument, req);
        //创建的时候就是已发布
        ycqQzhDocument.setState(1);
        //存储
        ycqQzhDocumentRepo.save(ycqQzhDocument);
        //存储附件和图片
        if (null != req.getSubsidiaryFile() && req.getSubsidiaryFile().size() > 0) {
            saveAnnex(req.getSubsidiaryFile(), ycqQzhDocument.getId());
        }
        if (null != req.getCoverImageFile() && req.getCoverImageFile().size() > 0) {
            saveCover(req.getCoverImageFile(), ycqQzhDocument.getId());
        }
        return new Result(CREATE_SUCCESS, Boolean.TRUE);
    }

    /**
     * 分页查询政策
     *
     * @param pageDocumentReqDTO
     * @return
     */
    @Override
    public Result<PageRsp<PageDocumentRspDTO>> pageDocument(PageReq<PageDocumentReqDTO> pageDocumentReqDTO) {
        PageDocumentReqDTO query = pageDocumentReqDTO.getQuery();
        Worker worker = getWorker();
//        if (null == worker.getLoginId()) {
//            return Result.ofFail("该账号组织异常,请联系系统管理员");
//        }
//        if (null != query.getCondition() && "app".equals(query.getCondition())) {
//            List<Long> list = ycqQzhDocumentJdbcDao.selecetOrgs(worker.getLoginId());
//            List<Long> listIds = ycqQzhDocumentJdbcDao.selecetIds(list);
//            return new Result(Boolean.TRUE, isShow(ycqQzhDocumentJdbcDao.pageDocument(pageDocumentReqDTO, listIds, worker.getOrgId()), worker));
//        } else if (null != query.getCondition() && "web".equals(query.getCondition())) {
//            if (null == query.getOpenCompetence()) {
//                return Result.ofFail("开放范围不能为空");
//            }
        return new Result(Boolean.TRUE, isShow(ycqQzhDocumentJdbcDao.pageDocument(pageDocumentReqDTO, null, worker.getOrgId()), worker));
//        } else {
//            return Result.ofFail("位置字段不能为空");
//        }
    }

    /**
     * 对按钮展示进行打标
     *
     * @param pageDocumentRspDTOPageRsp
     * @return
     */
    public PageRsp<PageDocumentRspDTO> isShow(PageRsp<PageDocumentRspDTO> pageDocumentRspDTOPageRsp, Worker worker) {
        for (PageDocumentRspDTO pageDocumentRspDTO : pageDocumentRspDTOPageRsp.getResult()) {
            if (pageDocumentRspDTO.getCreateId().equals(String.valueOf(worker.getLoginId()))) {
                pageDocumentRspDTO.setIsShow(1);
            } else {
                pageDocumentRspDTO.setIsShow(2);
            }
            List<YcqQzhCover> oneByArticleId = ycqQzhCoverRepo.findOneByArticleId(pageDocumentRspDTO.getId(), 1);
            List<String> coverList = new ArrayList<>();
            if (oneByArticleId.size() > 0) {
                for (YcqQzhCover ycqQzhCover : oneByArticleId) {
                    coverList.add(ycqQzhCover.getFileUrl());
                }
            }
            pageDocumentRspDTO.setCoverUrl(coverList);
        }
        return pageDocumentRspDTOPageRsp;
    }

    /**
     * 编辑政策
     *
     * @param createDocumentReqDTO
     * @return
     */
    @Override
    public Result<String> updateDocument(CreateDocumentReqDTO createDocumentReqDTO) {
        if (ObjectUtil.isEmpty(createDocumentReqDTO.getId())) {
            return Result.ofFail(REQUEST_ERROR);
        }
        validateCreateDocument(createDocumentReqDTO);
        YcqQzhDocument ycqQzhDocument = ycqQzhDocumentRepo.getOne(createDocumentReqDTO.getId());
        //当数据状态为已下架的时候才可以编辑
        if (2 == ycqQzhDocument.getState()) {
            //验证参数
            Worker worker = getWorker();
            if (Objects.nonNull(ycqQzhDocument.getCreateId()) && Objects.nonNull(worker.getLoginId())) {
                if (!ycqQzhDocument.getCreateId().equals(String.valueOf(worker.getLoginId()))) {
                    return Result.ofFail(ALL_OFF_HANDLE);
                }
            } else {
                return Result.ofFail(ALL_OFF_ACCOUNT);
            }
            saveUpdate(ycqQzhDocument);
            ycqQzhDocument.setDocumentTitle(createDocumentReqDTO.getDocumentTitle());
            ycqQzhDocument.setDocumentType(createDocumentReqDTO.getDocumentType());
            ycqQzhDocument.setDocumentText(createDocumentReqDTO.getDocumentText());
            //政策文件号
            ycqQzhDocument.setDocumentNum(Optional.ofNullable(createDocumentReqDTO.getDocumentNum()).orElse(null));
            ycqQzhDocument.setOpenCompetence(createDocumentReqDTO.getOpenCompetence());
//            ycqQzhDocument.setState(1);
            //申请入口1是2否
            if (null != createDocumentReqDTO.getApplyEntrance() && "1".equals(createDocumentReqDTO.getApplyEntrance())) {
                ycqQzhDocument.setBeginTime(createDocumentReqDTO.getBeginTime());
                ycqQzhDocument.setEndTime(createDocumentReqDTO.getEndTime());
                ycqQzhDocument.setApplyType(createDocumentReqDTO.getApplyType());
                ycqQzhDocument.setApplyEntrance("1");
            } else {
                ycqQzhDocument.setApplyEntrance("2");
            }
            //删除历史关联
//            ycqQzhBusinessAllRepo.deleteByBusinessId(createDocumentReqDTO.getId());
            //组织封装
//            orgSelect(worker, ycqQzhDocument, createDocumentReqDTO);
            ycqQzhDocumentRepo.save(ycqQzhDocument);
            //删除旧封面以及附件
            ycqQzhDocumentJdbcDao.deleteAnnex(ycqQzhDocument.getId());
            //保存新封面以及附件
            if (null != createDocumentReqDTO.getSubsidiaryFile() && createDocumentReqDTO.getSubsidiaryFile().size() > 0) {
                saveAnnex(createDocumentReqDTO.getSubsidiaryFile(), ycqQzhDocument.getId());
            }
            if (null != createDocumentReqDTO.getCoverImageFile() && createDocumentReqDTO.getCoverImageFile().size() > 0) {
                saveCover(createDocumentReqDTO.getCoverImageFile(), ycqQzhDocument.getId());
            }
            return new Result(UPDATE_SUCCESS, Boolean.TRUE);
        } else if (1 == ycqQzhDocument.getState()) {
            return Result.ofFail(ALL_OFF_SHELF);
        } else {
            return Result.ofFail(STATE_TRUE);
        }
    }

    /**
     * 政策修改状态
     *
     * @param operationDocumentReqDTO
     * @return
     */
    @Override
    public Result<String> doDownDocument(OperationDocumentReqDTO operationDocumentReqDTO) {
        //校验是否公示
        //校验是否存在申报数据
        YcqQzhDocument ycqQzhDocument = ycqQzhDocumentRepo.getOne(operationDocumentReqDTO.getId());
        if (null != operationDocumentReqDTO.getState()) {
            ycqQzhDocument.setState(operationDocumentReqDTO.getState());
            Worker worker = getWorker();
            if (Objects.nonNull(ycqQzhDocument.getCreateId()) && Objects.nonNull(worker.getLoginId())) {
                if (!ycqQzhDocument.getCreateId().equals(String.valueOf(worker.getLoginId()))) {
                    return Result.ofFail(ALL_OFF_HANDLE);
                }
            } else {
                return Result.ofFail(ALL_OFF_ACCOUNT);
            }
            saveUpdate(ycqQzhDocument);
            //如果再次发布则个更新发布时间和发布人信息
            if (1 == operationDocumentReqDTO.getState()) {
                saveCreate(ycqQzhDocument);
            }
            ycqQzhDocumentRepo.save(ycqQzhDocument);
            return new Result<>(UPDATE_SUCCESS, Boolean.TRUE);
        } else {
            return Result.ofFail("state不能为空");
        }
    }

    /**
     * 删除政策
     *
     * @param deleteDocumentPostRequest
     * @return
     */
    @Override
    public Result<String> deleteDocument(OperationDocumentReqDTO deleteDocumentPostRequest) {
        Worker worker = getWorker();
        YcqQzhDocument ycqQzhDocument = ycqQzhDocumentRepo.getOne(deleteDocumentPostRequest.getId());
        if (ObjectUtil.isEmpty(ycqQzhDocument) || (ObjectUtil.isNotEmpty(ycqQzhDocument.getDelFlag()) && ycqQzhDocument.getDelFlag().equals(1))) {
            return Result.ofFail(DOCUMENT_NOT_EXISTS);
        }
        if (ObjectUtil.isNotEmpty(ycqQzhDocument.getState()) && ycqQzhDocument.getState().equals(1)) {
            return Result.ofFail(DOCUMENT_IS_PUBLISH);
        }
        if (Objects.nonNull(ycqQzhDocument.getCreateId()) && Objects.nonNull(worker.getLoginId())) {
            if (!ycqQzhDocument.getCreateId().equals(String.valueOf(worker.getLoginId()))) {
                return Result.ofFail(ALL_OFF_HANDLE);
            }
        } else {
            return Result.ofFail(ALL_OFF_ACCOUNT);
        }
        //todo 是否存在申报数据
        ycqQzhDocumentJdbcDao.deleteByIds(Collections.singleton(ycqQzhDocument.getId()), worker.getLoginId(), worker.getWorkerName());
        return new Result(OPERATION_SUCCESS, Boolean.TRUE);

    }

    /**
     * 置顶与取消置顶政策
     *
     * @param operationDocumentReqDTO
     * @return
     */
    @Override
    public Result<String> doTopDocument(OperationDocumentReqDTO operationDocumentReqDTO) {
        YcqQzhDocument one = ycqQzhDocumentRepo.getOne(operationDocumentReqDTO.getId());
        if (ObjectUtil.isEmpty(one)) {
            return Result.ofFail(DOCUMENT_NOT_EXISTS);
        }
        saveUpdate(one);
        if (operationDocumentReqDTO.getWhetherTop()) {
            //置顶
            Integer maxSort = ycqQzhDocumentRepo.findMaxSort();
            one.setSortNum(maxSort != null ? maxSort + 1 : 1);
            ycqQzhDocumentRepo.save(one);
        } else {
            List<YcqQzhDocument> list = ycqQzhDocumentJdbcDao.findListBySortNum(one.getSortNum());
            //取消置顶
            one.setSortNum(0);
            ycqQzhDocumentRepo.save(one);
            //小于当前序号的重新排序
            for (YcqQzhDocument qzhDocument : list) {
                qzhDocument.setSortNum(qzhDocument.getSortNum() - 1);
                ycqQzhDocumentRepo.save(qzhDocument);
            }
        }
        return new Result<>(OPERATION_SUCCESS, Boolean.TRUE);
    }

    /**
     * 文明号申报查询可关联政策
     */
    @Override
    public Result<List<DocumentListRspDTO>> doFindDocumentByApplyType(OperationDocumentReqDTO reqDTO) {
        return new Result<>(Boolean.TRUE, ycqQzhDocumentJdbcDao.queryList(reqDTO.getType(), reqDTO.getApplyType()));
    }

    @Override
    public Result<PageRsp<PageDocumentRspDTO>> pageDocumentTourists(PageReq<PageDocumentReqDTO> dto) {
        return new Result(Boolean.TRUE, isShowTourists(ycqQzhDocumentJdbcDao.pageDocument(dto, null, null)));
    }

    public PageRsp<PageDocumentRspDTO> isShowTourists(PageRsp<PageDocumentRspDTO> pageDocumentRspDTOPageRsp) {
        for (PageDocumentRspDTO pageDocumentRspDTO : pageDocumentRspDTOPageRsp.getResult()) {
            pageDocumentRspDTO.setIsShow(2);
            List<YcqQzhCover> oneByArticleId = ycqQzhCoverRepo.findOneByArticleId(pageDocumentRspDTO.getId(), 1);
            List<String> coverList = new ArrayList<>();
            if (oneByArticleId.size() > 0) {
                for (YcqQzhCover ycqQzhCover : oneByArticleId) {
                    coverList.add(ycqQzhCover.getFileUrl());
                }
            }
            pageDocumentRspDTO.setCoverUrl(coverList);
            //获取来源
            if (null != pageDocumentRspDTO.getCreateId()) {
                String orgName = ycqQzhDocumentJdbcDao.selectOrgName(Long.valueOf(pageDocumentRspDTO.getCreateId()));
                if (null != orgName) {
                    pageDocumentRspDTO.setOrgName(orgName);
                }
            }
        }
        return pageDocumentRspDTOPageRsp;
    }

    /**
     * 查看政策详情
     *
     * @param operationDocumentReqDTO
     * @return
     */
    @Override
    public Result<DocumentDetailsRspDTO> findDocumentDetailsById(OperationDocumentReqDTO operationDocumentReqDTO) {

        YcqQzhDocument ycqQzhDocument = ycqQzhDocumentRepo.getOne(operationDocumentReqDTO.getId());
        if (ObjectUtil.isEmpty(ycqQzhDocument) || (ObjectUtil.isNotEmpty(ycqQzhDocument.getDelFlag()) && ycqQzhDocument.getDelFlag().equals(1))) {
            return Result.ofFail(DOCUMENT_NOT_EXISTS);
        }
        //浏览量
        DocumentDetailsRspDTO documentDetailsRsp = new DocumentDetailsRspDTO();
        Integer countArticleIdAndRecordType = ycqQzhBrowseRepo.findCountArticleIdAndRecordType(ycqQzhDocument.getId(), 1);
        if (null != countArticleIdAndRecordType) {
            documentDetailsRsp.setViewTotal(countArticleIdAndRecordType);
        } else {
            documentDetailsRsp.setViewTotal(0);
        }
        //获取来源
        if (null != ycqQzhDocument.getCreateId()) {

            String orgName = ycqQzhDocumentJdbcDao.selectOrgName(Long.valueOf(ycqQzhDocument.getCreateId()));
            if (null != orgName) {
                documentDetailsRsp.setOrgName(orgName);
            }
        }
        documentDetailsRsp.setId(ycqQzhDocument.getId());
        documentDetailsRsp.setDocumentNum(ycqQzhDocument.getDocumentNum());
        documentDetailsRsp.setDocumentText(ycqQzhDocument.getDocumentText());
        documentDetailsRsp.setDocumentTitle(ycqQzhDocument.getDocumentTitle());
        documentDetailsRsp.setDocumentType(ycqQzhDocument.getDocumentType());
        //当入口为是的时候才又有效时间和截至时间
        if ("1".equals(ycqQzhDocument.getApplyEntrance())) {
            documentDetailsRsp.setBeginTime(ycqQzhDocument.getBeginTime());
            documentDetailsRsp.setEndTime(ycqQzhDocument.getEndTime());
            documentDetailsRsp.setApplyType(ycqQzhDocument.getApplyType());
            if (1 == ycqQzhDocument.getApplyType()) {
                documentDetailsRsp.setApplyName("新建");
            } else {
                documentDetailsRsp.setApplyName("复核");
            }
        }
        documentDetailsRsp.setApplyEntrance(ycqQzhDocument.getApplyEntrance());
        documentDetailsRsp.setOpenCompetence(ycqQzhDocument.getOpenCompetence());
        documentDetailsRsp.setState(ycqQzhDocument.getState());
        documentDetailsRsp.setCreateTime(ycqQzhDocument.getCreateTime());
        documentDetailsRsp.setCreateUserName(ycqQzhDocument.getCreateBy());
        List<AnnexDto> annexes = ycqQzhAnnexRepo.findOneByArticleId(ycqQzhDocument.getId(), 1).stream().map(PictureEncapsulationUtil::doAnnexDtoConverter).collect(Collectors.toList());
        List<CoverImageDto> covers = ycqQzhCoverRepo.findOneByArticleId(ycqQzhDocument.getId(), 1).stream().map(PictureEncapsulationUtil::doCoverRsp).collect(Collectors.toList());
        documentDetailsRsp.setSubsidiaryFile(annexes);
        documentDetailsRsp.setCoverImageFile(covers);
        return new Result<>(Boolean.TRUE, documentDetailsRsp);

    }


    /**
     * 记录浏览量
     */
//    @Override
//    public Result<String> doBrowseDocument(OperationDocumentReqDTO operationDocumentReqDTO) {
//        YcqQzhDocument ycqQzhDocument = ycqQzhDocumentRepo.getOne(operationDocumentReqDTO.getId());
//        if (ObjectUtil.isEmpty(ycqQzhDocument)) {
//            return Result.ofFail(DOCUMENT_NOT_EXISTS);
//        }
//        Worker worker = getWorker();
//        YcqQzhBrowse ycqQzhBrowse = new YcqQzhBrowse();
//        ycqQzhBrowse.setRecordType(1);
//        ycqQzhBrowse.setArticleId(ycqQzhDocument.getId());
//        ycqQzhBrowse.setUserId((long) worker.getLoginId());
//        ycqQzhBrowse.setCreateTime(new Date());
//        ycqQzhBrowseRepo.save(ycqQzhBrowse);
//        return new Result<>(BROWSE_TRUE, Boolean.TRUE);
//    }

    /**
     * 取消点赞
     *
     * @param req
     * @return
     */
//    @Override
//    public Result<String> doBackBrowseDocument(OperationDocumentReqDTO req) {
//        YcqQzhDocument ycqQzhDocument = ycqQzhDocumentRepo.getOne(req.getId());
//        if (ObjectUtil.isEmpty(ycqQzhDocument)) {
//            return Result.ofFail(DOCUMENT_NOT_EXISTS);
//        }
//        Worker worker = getWorker();
//        YcqQzhBrowse ycqQzhBrowse = ycqQzhBrowseRepo.findOneByArticleId(1, ycqQzhDocument.getId(), 2, (long) worker.getLoginId());
//        if (ObjectUtil.isNotEmpty(ycqQzhBrowse)) {
//            ycqQzhBrowse.setDelFlag(1);
//            ycqQzhBrowseRepo.save(ycqQzhBrowse);
//        }
//        return new Result<>(true);
//    }

    /**
     * 组织封装
     *
     * @param worker
     * @param ycqQzhDocument
     * @param req
     */
//    private void orgSelect(Worker worker, YcqQzhDocument ycqQzhDocument, CreateDocumentReqDTO req) {
//        //todo 对传过来的组织也要进行递归晚上找父级
//        List<YcqQzhBusinessAll> businessAlls = new ArrayList<>();
//        if (null != req.getAuthorityOrgs() && req.getAuthorityOrgs().size() > 0) {
//            List<Long> orgAll = new ArrayList<>();
//            //先把数据的org_id传入list 然后对相同父级的数据做去重 再拿去递归
//            List<YcqAllOrganization> authorityOrgs = req.getAuthorityOrgs();
//            for (YcqAllOrganization authorityOrg : authorityOrgs) {
//                orgAll.add(authorityOrg.getOrgId());
//            }
//            //对传过来的数据进行去重
//            ArrayList<YcqAllOrganization> setOrgs = authorityOrgs.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
//                    new TreeSet<>(Comparator.comparing(YcqAllOrganization::getParentOrgId))), ArrayList::new));
//            if (setOrgs.size() > 0) {
//                for (YcqAllOrganization setOrg : setOrgs) {
//                    List<Long> newList = new ArrayList<>();
//                    findParentOrg(newList, setOrg.getOrgId());
//                    orgAll.addAll(newList);
//                }
//            }
//            //再次去重避免重复
//            List<Long> collect = orgAll.stream().distinct().collect(Collectors.toList());
//            for (Long longId : collect) {
//                YcqQzhBusinessAll ycqQzhBusinessAll = new YcqQzhBusinessAll();
//                if (null != ycqQzhDocument.getId()) {
//                    ycqQzhBusinessAll.setBusinessId(ycqQzhDocument.getId());
//                } else if (null != req.getId()) {
//                    ycqQzhBusinessAll.setBusinessId(req.getId());
//                }
//                ycqQzhBusinessAll.setBusinessType(1);
//                ycqQzhBusinessAll.setOrgId(longId);
//                businessAlls.add(ycqQzhBusinessAll);
//            }
//            ycqQzhBusinessAllRepo.saveAll(businessAlls);
//        }
//        //todo 反向递归找到上级组织id
//        if (null != worker.getOrgId()) {
//            List<Long> list = new ArrayList<>();
//            list.add(Long.valueOf(worker.getOrgId()));
//            //递归上级组织
//            findParentOrg(list, Long.valueOf(worker.getOrgId()));
//            String collect = list.stream().distinct().map(String::valueOf).collect(Collectors.joining(","));
//            ycqQzhDocument.setCreateOrgIds(collect);
//        }
//    }

    /**
     * 逆向递归
     *
     * @param list,orgId
     */
    private void findParentOrg(List<Long> list, Long orgId) {
        TsmOrg tsmOrg = ycqQzhDocumentJdbcDao.findParentOrgs(orgId);
        if (0 == tsmOrg.getParentOrgId()) {
            list.add(tsmOrg.getOrgId());
        } else {
            list.add(tsmOrg.getParentOrgId());
            findParentOrg(list, tsmOrg.getParentOrgId());
        }
    }

    /**
     * 保存新建信息
     *
     * @param ycqQzhDocument
     */
    private void saveCreate(YcqQzhDocument ycqQzhDocument) {
        Worker worker = getWorker();
        if (ObjectUtil.isNotEmpty(worker)) {
            ycqQzhDocument.setCreateId(String.valueOf(worker.getLoginId()));
            ycqQzhDocument.setCreateBy(worker.getWorkerName());
            ycqQzhDocument.setCreateTime(new Date());
        }
    }

    /**
     * 保存修改信息
     *
     * @param ycqQzhDocument
     */
    private void saveUpdate(YcqQzhDocument ycqQzhDocument) {
        Worker worker = getWorker();
        if (ObjectUtil.isNotEmpty(worker)) {
            ycqQzhDocument.setUpdateId((long) worker.getLoginId());
            ycqQzhDocument.setUpdateBy(worker.getWorkerName());
            ycqQzhDocument.setUpdateTime(new Date());
        }
    }

    /**
     * 获取登录用户
     *
     * @return
     */
    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        return worker;
    }

    private void saveAnnex(List<AnnexDto> subsidiaryFile, Long id) {
        //保存附件
        subsidiaryFile.forEach(s -> {
            YcqQzhAnnex ycqQzhAnnex = PictureEncapsulationUtil.doCrateConverter(s);
            ycqQzhAnnex.setArticleId(id);
            ycqQzhAnnex.setType(1);
            ycqQzhAnnexRepo.save(ycqQzhAnnex);
        });
    }

    private void saveCover(List<CoverImageDto> coverImageFile, Long id) {
        //保存封面
        coverImageFile.forEach(s -> {
            YcqQzhCover ycqQzhCover = PictureEncapsulationUtil.doCrateConverter(s);
            ycqQzhCover.setArticleId(id);
            ycqQzhCover.setType(1);
            ycqQzhCoverRepo.save(ycqQzhCover);
        });
    }

    /**
     * 校验政策新增数据
     *
     * @param req
     */
    private void validateCreateDocument(CreateDocumentReqDTO req) {
        //请求参数不能为空
        if (ObjectUtil.isEmpty(req)) {
            throw new RuntimeException(DOCUMENT_REQUEST_IS_NULL);
        }
        //标题长度不能超过50个字符
        if (StringUtils.isNotEmpty(req.getDocumentTitle()) && req.getDocumentTitle().length() > 50) {
            throw new RuntimeException(DOCUMENT_TITLE_LENGTH_TOO_LONG);
        }
        //文件号长度不能超过50个字符
        if (StringUtils.isNotEmpty(req.getDocumentNum()) && req.getDocumentNum().length() > 50) {
            throw new RuntimeException(DOCUMENT_NUM_LENGTH_TOO_LONG);
        }
        //文件数量不能超过五个
        if (null != req.getSubsidiaryFile() && req.getSubsidiaryFile().size() > 5) {
            throw new RuntimeException(DOCUMENT_FILE_TOO_MUCH);
        }
        //todo 校验文件大小
    }
}