package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.dynamicform.YcqQzhDynamicForm;
import com.crunii.micro.domain.entity.main.dynamicform.YcqQzhDynamicFormRecord;
import com.crunii.micro.domain.repo.main.dynamicform.YcqQzhDynamicFormRecordRepo;
import com.crunii.micro.domain.repo.main.dynamicform.YcqQzhDynamicFormRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormRecordReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicFormReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.form.DynamicRecordDetailReqDTO;
import com.crunii.micro.service.qzh.service.DynamicFormService;
import com.crunii.micro.service.qzh.service.YcqQzhDynamicFormService;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

import static com.crunii.micro.service.qzh.api.constants.QzhConstants.WORKER_NOT_EXISTS;

/**
 * @ClassName YcqQzhDynamicFormService
 * @Description 动态表单模板
 * @Author Jiang
 * @Date 2023/9/27
 * @Version 1.0
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class YcqQzhDynamicFormServiceImpl implements YcqQzhDynamicFormService {


    @Autowired
    YcqQzhDynamicFormRepo ycqQzhDynamicFormRepo;

    @Autowired
    YcqQzhDynamicFormRecordRepo ycqQzhDynamicFormRecordRepo;

    @Autowired
    DynamicFormService dynamicFormService;


    /**
     * /**
     *
     * @MethodName createIndicatorSystem
     * @Description 新增活动动态表单模板
     * @param: dto
     * @param: worker
     * @return: com.crunii.micro.common.dto.Result<java.lang.String>
     * @Author Jiang
     * @Date 2023/9/27 19:31
     */
    @Override
    public Result createDynamicForm(Worker worker, DynamicFormReqDTO dynamicFormReq) {
//        worker = getWorker();
//        //新增活动或编辑活动时直接到用这个方法 首先删除该活动的动态表单字端
//        ycqQzhDynamicFormRepo.deleteByActivityId((dynamicFormReq.getDynamicForms().get(0).getActivityId()));
//
//        List<YcqQzhDynamicForm> ycqQzhDynamicForms = new ArrayList<>();
//        int i = 0;
//        for (DynamicFormsReqDTO dynamicForm : dynamicFormReq.getDynamicForms()) {
//            i++;
//            YcqQzhDynamicForm ycqQzhDynamicForm = new YcqQzhDynamicForm();
//            BeanUtils.copyProperties(dynamicForm, ycqQzhDynamicForm);
//            ycqQzhDynamicForm.setCreateBy((long)worker.getLoginId());
//            ycqQzhDynamicForm.setCreateTime(new Date());
//            ycqQzhDynamicForm.setLabel(dynamicForm.getType()+i);
//            ycqQzhDynamicForm.setSort(i);
//            ycqQzhDynamicForm.setDeleted(0);
//            ycqQzhDynamicForms.add(ycqQzhDynamicForm);
//        }
//        ycqQzhDynamicFormRepo.saveAll(ycqQzhDynamicForms);
//        return new Result<>("新增成功!", Boolean.TRUE);
        worker = getWorker();
        //新增活动或编辑活动时直接到用这个方法 首先删除该活动的动态表单字端
        ycqQzhDynamicFormRepo.deleteByActivityId((dynamicFormReq.getActivityId()));

        YcqQzhDynamicForm ycqQzhDynamicForm = new YcqQzhDynamicForm();
        BeanUtils.copyProperties(dynamicFormReq, ycqQzhDynamicForm);
        ycqQzhDynamicForm.setCreateBy((long) worker.getLoginId());
        ycqQzhDynamicForm.setCreateTime(new Date());
        ycqQzhDynamicForm.setDeleted(0);
        ycqQzhDynamicFormRepo.save(ycqQzhDynamicForm);
        return new Result<>("新增成功!", Boolean.TRUE);
    }


    /**
     * @MethodName createIndicatorSystem
     * @Description PC 通过活动Id和用户获取活动表单数据详情
     * @param: dto
     * @param: worker
     * @return: com.crunii.micro.common.dto.Result<java.lang.String>
     * @Author Jiang
     * @Date 2023/9/27 19:31
     */
    @Override
    public Result<YcqQzhDynamicFormRecord> findDynamicFormByActivityIdAndUserId(Worker worker, DynamicRecordDetailReqDTO dto) {
        YcqQzhDynamicFormRecord qzhDynamicFormRecords = ycqQzhDynamicFormRecordRepo.findByActivityIdAndCreateByAndDeleted(dto.getActivityId(), dto.getWorkerId(), 0);
        return new Result<>(Boolean.TRUE, qzhDynamicFormRecords);
    }

    /**
     * @MethodName findDynamicFormByActivityId
     * @Description APP端和PC可以公用 通过活动Id获取活动表单数据
     * @param: worker
     * @param: activityId
     * @return: com.crunii.micro.common.dto.Result<java.util.List < com.crunii.micro.service.qzh.api.domain.dto.HomePageTopRespDTO>>
     * @Author Jiang
     * @Date 2023/9/27 21:59
     */
    @Override
    public Result<YcqQzhDynamicForm> findDynamicFormByActivityId(Worker worker, Long activityId) {
        YcqQzhDynamicForm dynamicFormList = ycqQzhDynamicFormRepo.findByActivityIdAndDeleted(activityId,0);
        return new Result<>(Boolean.TRUE, dynamicFormList);
    }


    /**
     * @MethodName findDynamicFormByActivityId
     * @Description APP端 参加活动提交表单数据
     * @param: worker
     * @param: activityId
     * @return: com.crunii.micro.common.dto.Result<java.util.List < com.crunii.micro.service.qzh.api.domain.dto.HomePageTopRespDTO>>
     * @Author Jiang
     * @Date 2023/9/27 21:59
     */
    @Override
    public Result createDynamicFormRecord(Worker worker, DynamicFormRecordReqDTO dynamicFormRecord) {
        worker = getWorker();
//        List<YcqQzhDynamicFormRecord> ycqQzhDynamicFormRecords = new ArrayList<>();
//        for (DynamicFormsReqDTO dynamicRecord : dynamicFormRecord.getDynamicRecords()) {
//            YcqQzhDynamicFormRecord ycqQzhDynamicFormRecord = new YcqQzhDynamicFormRecord();
//            ycqQzhDynamicFormRecord.setCreateBy((long) worker.getLoginId());
//            ycqQzhDynamicFormRecord.setCreateTime(new Date());
//            ycqQzhDynamicFormRecord.setDeleted(0);
//            BeanUtils.copyProperties(dynamicRecord, ycqQzhDynamicFormRecord);
//            ycqQzhDynamicFormRecords.add(ycqQzhDynamicFormRecord);
//        }
//
//        ycqQzhDynamicFormRecordRepo.saveAll(ycqQzhDynamicFormRecords);
//        return new Result<>("新增成功!", Boolean.TRUE);

        YcqQzhDynamicFormRecord ycqQzhDynamicFormRecord = new YcqQzhDynamicFormRecord();
        ycqQzhDynamicFormRecord.setCreateBy((long) worker.getLoginId());
        ycqQzhDynamicFormRecord.setCreateTime(new Date());
        ycqQzhDynamicFormRecord.setDeleted(0);
        BeanUtils.copyProperties(dynamicFormRecord, ycqQzhDynamicFormRecord);

        //保存用户提交的表单json记录
        ycqQzhDynamicFormRecordRepo.save(ycqQzhDynamicFormRecord);

        //解析用户提交的json数据保存到表中（保存用户表单填的数据）
//        dynamicFormService.saveTableData(worker, dynamicFormRecord);

        return new Result<>("新增成功!", Boolean.TRUE);
    }

    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        return worker;
    }

}
