package com.crunii.micro.service.qzh.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQzhIndicatorSystem;
import com.crunii.micro.domain.entity.main.YcqQzhIndicatorVersion;
import com.crunii.micro.domain.repo.main.YcqQzhIndicatorSystemRepo;
import com.crunii.micro.domain.repo.main.YcqQzhIndicatorVersionRepo;
import com.crunii.micro.service.qzh.api.domain.dto.*;
import com.crunii.micro.service.qzh.converter.YcqQzhIndicatorSystemConverter;
import com.crunii.micro.service.qzh.jdbc.YcqQzhIndicatorManageJdbcDao;
import com.crunii.micro.service.qzh.service.YcqQzhIndicatorManageService;
import com.crunii.micro.service.qzh.util.CustomImageModifyHandler;
import com.crunii.micro.service.qzh.util.MultiColumnMergeStrategy;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.*;

@Service
@Slf4j
@RequiredArgsConstructor
public class YcqQzhIndicatorManageServiceImpl implements YcqQzhIndicatorManageService {
    private final YcqQzhIndicatorManageJdbcDao jdbcDao;

    private final YcqQzhIndicatorVersionRepo versionRepo;

    private final YcqQzhIndicatorSystemRepo systemRepo;

    private final YcqQzhIndicatorSystemConverter systemConverter;

    /** 指标体系类型常量值
     * 1=评价要点，2=一级指标，3=二级指标
     * */
    private static final Integer[] TYPE_ARR = new Integer[] {1, 2, 3};


    @Override
    public List<YcqQzhIndicatorVersionVo> findIndicatorVersion(Worker worker) {
        return jdbcDao.findIndicatorVersion();
    }

    @Override
    public void createIndicatorVersion(YcqQzhIndicatorVersionDto req, Worker worker) {
        YcqQzhIndicatorVersion entity = new YcqQzhIndicatorVersion();
        BeanUtils.copyProperties(req, entity);
        entity.setCreateId((long) worker.getLoginId());
        entity.setCreateBy(worker.getWorkerName());
        entity.setCreateTime(new Date());
        entity.setDeleted(0);
        versionRepo.save(entity);
    }

    @Override
    public void updateIndicatorVersion(YcqQzhIndicatorVersionDto req, Worker worker) {
        YcqQzhIndicatorVersion entity = versionRepo.getById(req.getId());
        BeanUtils.copyProperties(req, entity);
        entity.setUpdateId((long) worker.getLoginId());
        entity.setUpdateBy(worker.getWorkerName());
        entity.setUpdateTime(new Date());
        versionRepo.save(entity);
    }

    @Override
    public Result<Void> deleteIndicatorVersion(List<Long> ids, Worker worker) {
        if ( null == ids || ids.size() == 0) {
            return Result.ofFail("请选择要删除的版本！");
        }
        for(int i=0; i<ids.size(); i++){
            YcqQzhIndicatorVersion entity = versionRepo.getById(ids.get(i));
            entity.setUpdateId((long) worker.getLoginId());
            entity.setUpdateBy(worker.getWorkerName());
            entity.setUpdateTime(new Date());
            entity.setDeleted(1);
            versionRepo.save(entity);
        }
       return Result.ofSuccessMsg("删除成功!");
    }

    @Override
    public List<YcqQzhIndicatorSystemVo> findIndicatorSystem(YcqQzhIndicatorSystemDto req) {
        return jdbcDao.findIndicatorSystem(req);
    }

    @Override
    public Result<Void> createIndicatorSystem(YcqQzhIndicatorSystemDto req,Worker worker) {
        String msg = checkReq(req, 1);
        if (!"".equals(msg)) {
            return Result.ofFail(msg);
        }
        if(TYPE_ARR[0].equals(req.getType())){
            // 评价要点
            YcqQzhIndicatorSystem entity = new YcqQzhIndicatorSystem();
            BeanUtils.copyProperties(req, entity, "id", "primaryIndicators", "secondaryIndicators", "score");
            commonInfo(entity, worker);
            systemRepo.save(entity);
            return Result.ofSuccessMsg("新增成功！");
        } else if (TYPE_ARR[1].equals(req.getType())){
            // 一级指标，先查询是否存在有评价要点但没有一级指标的数据
            YcqQzhIndicatorSystem one = jdbcDao.getOneLevelC(req);
            if(one != null){
                one.setPrimaryIndicators(req.getPrimaryIndicators());
                systemRepo.save(one);
            } else {
                YcqQzhIndicatorSystem entity = new YcqQzhIndicatorSystem();
                BeanUtils.copyProperties(req, entity, "id", "secondaryIndicators", "score");
                commonInfo(entity, worker);
                systemRepo.save(entity);
            }
            return Result.ofSuccessMsg("新增成功！");
        } else if (TYPE_ARR[2].equals(req.getType())){
            // 二级指标，先查询是否存在有评价要点和一级指标但没有二级指标的数据
            YcqQzhIndicatorSystem two = jdbcDao.getTwoLevelC(req);
            if(two!=null){
                two.setSecondaryIndicators(req.getSecondaryIndicators());
                two.setScore(req.getScore());
                systemRepo.save(two);
            } else{
                YcqQzhIndicatorSystem entity = new YcqQzhIndicatorSystem();
                BeanUtils.copyProperties(req, entity, "id");
                commonInfo(entity, worker);
                systemRepo.save(entity);
            }
            return Result.ofSuccessMsg("新增成功！");
        } else {
            return Result.ofFail("类型错误！");
        }
    }

    private void commonInfo(YcqQzhIndicatorSystem entity, Worker worker) {
        entity.setCreateId((long) worker.getLoginId());
        entity.setCreateBy(worker.getWorkerName());
        entity.setCreateTime(new Date());
        entity.setDeleted(0);
    }

    /**
     * 参数校验
     * @author leixb
     * @since 2023-11-17 15:53
     * @param req 请求参数
     * @param checkType 校验类型  null:下拉查询 1：新增 2：修改
     * @return String
     */
    private String checkReq(YcqQzhIndicatorSystemDto req, Integer checkType) {
        if (null == req) {
            return "请求参数为空！";
        }
        if (null == req.getIndicatorVersionId()) {
            return "指标版本id为空！";
        }
        // 修改时不校验类型
        if (null == req.getType() && 2 != checkType){
            return "类型为空！";
        }

        if (null != req.getType() && !Arrays.asList(TYPE_ARR).contains(req.getType())) {
            return "类型错误！";
        }

        if (null != checkType) {
            if (2 == checkType && null == req.getId()) {
                return "id为空！";
            }
            if (TYPE_ARR[0].equals(req.getType())) {
                // 评价要点
                if (StringUtils.isEmpty(req.getEvaluationPoints())) {
                    return "评价要点为空！";
                }
                if (StringUtils.isNotEmpty(req.getEvaluationPoints()) && req.getEvaluationPoints().length() > 20) {
                    return "评价要点长度不能超过20字符！";
                }
            } else {
                // 类型：一级指标、二级指标；修改时没有类型，全部都要校验
                if (StringUtils.isEmpty(req.getEvaluationPoints())) {
                    return "评价要点为空！";
                }
                if (StringUtils.isEmpty(req.getPrimaryIndicators())) {
                    return "一级指标为空！";
                }
                if (StringUtils.isNotEmpty(req.getPrimaryIndicators()) && req.getPrimaryIndicators().length() > 200) {
                    return "一级指标长度不能超过200字符！";
                }
                if (TYPE_ARR[2].equals(req.getType()) || 2 == checkType) {
                    // 二级指标和修改时校验二级指标
                    if (StringUtils.isEmpty(req.getSecondaryIndicators())) {
                        return "二级指标为空！";
                    }
                    if (StringUtils.isNotEmpty(req.getSecondaryIndicators()) && req.getSecondaryIndicators().length() > 200) {
                        return "二级指标长度不能超过200字符！";
                    }
                }
            }
        }
        return "";
    }

    @Override
    public Result<Void> updateIndicatorSystem(YcqQzhIndicatorSystemDto req, Worker worker) {
        String msg = checkReq(req, 2);
        if (!"".equals(msg)) {
            return Result.ofFail(msg);
        }
        YcqQzhIndicatorSystem entity = systemRepo.getById(req.getId());
        BeanUtils.copyProperties(req, entity, "id", "indicatorVersionId");
        entity.setUpdateId((long) worker.getLoginId());
        entity.setUpdateBy(worker.getWorkerName());
        entity.setUpdateTime(new Date());
        systemRepo.save(entity);
        return Result.ofSuccessMsg("修改成功！");
    }

    @Override
    public Result<Void> deleteIndicatorSystem(List<Long> ids,Worker worker) {
        if (null == ids || ids.size() == 0) {
            return Result.ofFail("id为空！");
        }
        for(int i=0; i < ids.size(); i++){
            YcqQzhIndicatorSystem entity = systemRepo.getById(ids.get(i));
            entity.setUpdateId((long) worker.getLoginId());
            entity.setUpdateBy(worker.getWorkerName());
            entity.setUpdateTime(new Date());
            entity.setDeleted(1);
            systemRepo.save(entity);
        }
        return Result.ofSuccessMsg("删除成功！");
    }

    @Override
    public void exportIndicatorSystem(HttpServletResponse response, YcqQzhIndicatorSystemDto req) {
//        PageReq<YcqQzhIndicatorSystemDto> page = new PageReq<>();
//        page.setQuery(req);
//        page.setPageSize(Integer.MAX_VALUE);
        List<YcqQzhIndicatorSystemVo> excelList = jdbcDao.findIndicatorSystem(req);
        String fileName = "指标体系导出" + DateUtil.format(new Date(), DateUtil.defaultDT1Pattern);
        try {
            fileName = URLEncoder.encode(fileName, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        commonSetResponse(response, fileName);
        // 写入数据
        try {
            EasyExcel.write(response.getOutputStream(), YcqQzhIndicatorSystemVo.class)
                // 表头阴影
                .registerWriteHandler(new CustomImageModifyHandler())
                // 自动列宽
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                // 指定列合并
                .registerWriteHandler(new MultiColumnMergeStrategy(excelList.size(),0,1))
                .sheet("指标体系").doWrite(excelList);

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public List<YcqQzhIndicatorIndexRespDTO> findIndicatorDropdownBox(YcqQzhIndicatorIndexReqDTO req) {
        if(TYPE_ARR[1].equals(req.getType())){
            // 评价要点下拉
            return jdbcDao.getEvaluationPoints(req.getIndicatorVersionId());
        }
        if(TYPE_ARR[2].equals(req.getType())){
            // 一级指标下拉
            return jdbcDao.getPrimaryIndicators(req.getIndicatorVersionId());

        }
        return new ArrayList<>();
    }

    public void commonSetResponse(HttpServletResponse response, String fileName) {
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
    }
}
