package com.crunii.micro.service.qzh.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQzhOnlineTrain;
import com.crunii.micro.domain.entity.main.YcqQzhOnlineTrainingVideo;
import com.crunii.micro.domain.entity.main.YcqQzhVideoBrowsingHistory;
import com.crunii.micro.domain.repo.main.YcqQzhOnlineTrainRepo;
import com.crunii.micro.domain.repo.main.YcqQzhOnlineTrainingVideoRepo;
import com.crunii.micro.domain.repo.main.YcqQzhVideoBrowsingHistoryRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhOnlineRecordDTO;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhOnlineTrainDTO;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhOnlineTrainTypeDTO;
import com.crunii.micro.service.qzh.api.domain.dto.YcqQzhOnlineTrainVideoDTO;
import com.crunii.micro.service.qzh.dto.video.FileDto;
import com.crunii.micro.service.qzh.jdbc.YcqQzhOnlineTrainDao;
import com.crunii.micro.service.qzh.service.QzhVideoService;
import com.crunii.micro.service.qzh.service.YcqQzhOnlineTrainService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


@Service
@Slf4j
public class YcqQzhOnlineTrainServiceImpl implements YcqQzhOnlineTrainService {
    @Resource
    private YcqQzhOnlineTrainRepo ycqQzhOnlineTrainRepo;
    @Resource
    private YcqQzhOnlineTrainingVideoRepo ycqQzhOnlineTrainingVideoRepo;
    @Resource
    private YcqQzhVideoBrowsingHistoryRepo ycqQzhVideoBrowsingHistoryRepo;
    @Resource
    private YcqQzhOnlineTrainDao ycqQzhOnlineTrainDao;

    @Resource
    private QzhVideoService videoService;


    @Override
    public Result<Boolean> create(YcqQzhOnlineTrainDTO dto) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
//        Worker worker = new Worker();
//        worker.setLoginId(9999);
//        worker.setWorkerName("test");
        YcqQzhOnlineTrain ycqQzhOnlineTrain = new YcqQzhOnlineTrain();
        BeanUtils.copyProperties(dto, ycqQzhOnlineTrain);
        ycqQzhOnlineTrain.setCreatedBy(Long.parseLong(worker.getLoginId() + ""));
        ycqQzhOnlineTrain.setCreatedTime(new Date());
        ycqQzhOnlineTrain.setIsTop(0);
        ycqQzhOnlineTrainRepo.saveAndFlush(ycqQzhOnlineTrain);
        if (dto.getVideos() != null && dto.getVideos().size() > 0) {
            List<YcqQzhOnlineTrainingVideo> videos = new ArrayList<>();
            for (YcqQzhOnlineTrainVideoDTO video : dto.getVideos()) {
                YcqQzhOnlineTrainingVideo ycqQzhOnlineTrainingVideo = new YcqQzhOnlineTrainingVideo();
                BeanUtils.copyProperties(video, ycqQzhOnlineTrainingVideo);
                ycqQzhOnlineTrainingVideo.setTrainingId(ycqQzhOnlineTrain.getId());
                ycqQzhOnlineTrainingVideo.setCreatedBy(Long.parseLong(worker.getLoginId() + ""));
                ycqQzhOnlineTrainingVideo.setCreatedTime(new Date());
                videos.add(ycqQzhOnlineTrainingVideo);
            }
            ycqQzhOnlineTrainingVideoRepo.saveAllAndFlush(videos);
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Boolean> delete(Long id) {
        YcqQzhOnlineTrain train = ycqQzhOnlineTrainRepo.getOne(id);
        if (train == null) {
            return Result.ofFail("未查询到数据");
        }
        if (train.getState() != null && !train.getState().equals("下架")) {
            return Result.ofFail("不允许删除");
        }
        ycqQzhOnlineTrainRepo.deleteById(id);
        List<YcqQzhOnlineTrainingVideo> videos = ycqQzhOnlineTrainingVideoRepo.findByTrainingId(id);
        if (videos != null && videos.size() > 0) {
            ycqQzhOnlineTrainingVideoRepo.deleteAll(videos);
        }
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional
    public Result<Boolean> update(YcqQzhOnlineTrainDTO dto) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
//        Worker worker = new Worker();
//        worker.setLoginId(9999);
//        worker.setWorkerName("test");
        YcqQzhOnlineTrain train = ycqQzhOnlineTrainRepo.getOne(dto.getId());
        if (train.getState() != null && !train.getState().equals("下架")) {
            return Result.ofFail("不允许修改");
        }
        train.setUpdatedBy(Long.parseLong(worker.getLoginId() + ""));
        train.setUpdatedTime(new Date());
        train.setContent(dto.getContent());
        train.setFileId(dto.getFileId());
        train.setType(dto.getType());
        train.setState(dto.getState());
        train.setFileUrl(dto.getFileUrl());
        train.setTitle(dto.getTitle());
        train.setSecondType(dto.getSecondType());
        ycqQzhOnlineTrainRepo.saveAndFlush(train);
        //删
        List<YcqQzhOnlineTrainingVideo> TrainingVideos = ycqQzhOnlineTrainingVideoRepo.findByTrainingId(dto.getId());
        if (TrainingVideos != null && TrainingVideos.size() > 0) {
            ycqQzhOnlineTrainingVideoRepo.deleteAll(TrainingVideos);
        }
        if (dto.getVideos() != null && dto.getVideos().size() > 0) {
            List<YcqQzhOnlineTrainingVideo> videos = new ArrayList<>();
            for (YcqQzhOnlineTrainVideoDTO video : dto.getVideos()) {
                YcqQzhOnlineTrainingVideo ycqQzhOnlineTrainingVideo = new YcqQzhOnlineTrainingVideo();
                BeanUtils.copyProperties(video, ycqQzhOnlineTrainingVideo);
                ycqQzhOnlineTrainingVideo.setTrainingId(train.getId());
                ycqQzhOnlineTrainingVideo.setCreatedTime(new Date());
                ycqQzhOnlineTrainingVideo.setCreatedBy(Long.parseLong(worker.getLoginId() + ""));
                videos.add(ycqQzhOnlineTrainingVideo);
            }
            ycqQzhOnlineTrainingVideoRepo.saveAllAndFlush(videos);
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<YcqQzhOnlineTrainDTO> getOne(Long id, Long type) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
//        Worker worker = new Worker();
//        worker.setLoginId(9999);
//        worker.setWorkerName("test");
//        worker.setPhone("12345679111");
        if (type == 1) {
            YcqQzhVideoBrowsingHistory ycqQzhVideoBrowsingHistory = new YcqQzhVideoBrowsingHistory();
            ycqQzhVideoBrowsingHistory.setCreatedBy(Long.parseLong(worker.getLoginId() + ""));
            ycqQzhVideoBrowsingHistory.setCreatedName(worker.getWorkerName());
            ycqQzhVideoBrowsingHistory.setPhone(worker.getMobile());
            ycqQzhVideoBrowsingHistory.setBusinessID(id + "");
            ycqQzhVideoBrowsingHistory.setCreatedTime(new Date());
            ycqQzhVideoBrowsingHistoryRepo.saveAndFlush(ycqQzhVideoBrowsingHistory);
        }
        Result<YcqQzhOnlineTrainDTO> result = ycqQzhOnlineTrainDao.getOne(id, worker);
        if (result.getData() != null && result.getData().getVideos() != null && result.getData().getVideos().size() > 0) {
            YcqQzhOnlineTrainDTO data = result.getData();
            List<YcqQzhOnlineTrainVideoDTO> videos = data.getVideos();
            for (YcqQzhOnlineTrainVideoDTO video : videos) {
                String videoId = video.getVideoId();
                FileDto dto = new FileDto();
                dto.setFileId(videoId);
                dto.setFileType("1");
                Result<String> strResult = videoService.getFileUrl(dto);
                if (strResult.getCode() != null && strResult.getCode().equals("0")) {
                    video.setVideoUrl(strResult.getData());
                }
            }

        }

        return result;
    }

    @Override
    public Result<PageRsp<YcqQzhOnlineTrainDTO>> trainPage(PageReq<YcqQzhOnlineTrainDTO> req) {
        return ycqQzhOnlineTrainDao.trainPage(req);
    }

    @Override
    public Result<PageRsp<YcqQzhOnlineRecordDTO>> recordPage(PageReq<YcqQzhOnlineTrainDTO> req) {
        return ycqQzhOnlineTrainDao.recordPage(req);
    }

    @Override
    public Result<Boolean> isTop(YcqQzhOnlineTrainDTO dto) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
//        Worker worker = new Worker();
//        worker.setLoginId(9999);
//        worker.setWorkerName("test");
//        worker.setPhone("12345679111");
        if (dto.getIsTop() != null && dto.getIsTop() == 1) {
            List<YcqQzhOnlineTrain> byIsTop = ycqQzhOnlineTrainRepo.findByIsTop(dto.getIsTop());
            if (byIsTop != null && byIsTop.size() >= 5) {
                return Result.ofFail("最多置顶五条信息，请先取消其他置顶");
            }
        }
        YcqQzhOnlineTrain train = ycqQzhOnlineTrainRepo.getOne(dto.getId());
        train.setUpdatedBy(Long.parseLong(worker.getLoginId() + ""));
        train.setUpdatedTime(new Date());
        if (dto.getIsTop() != null) {
            train.setIsTop(dto.getIsTop());
            if (dto.getIsTop() == 1) {
                train.setTopTime(new Date());
            } else {
                train.setTopTime(null);
            }
        }
        if (dto.getState() != null && !dto.getState().equals("")) {
            train.setState(dto.getState());
        }
        ycqQzhOnlineTrainRepo.saveAndFlush(train);
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Boolean> createRecord(YcqQzhOnlineRecordDTO dto) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
//        Worker worker = new Worker();
//        worker.setLoginId(9999);
//        worker.setWorkerName("test");
//        worker.setPhone("12345679111");
        YcqQzhVideoBrowsingHistory ycqQzhVideoBrowsingHistory = new YcqQzhVideoBrowsingHistory();
        ycqQzhVideoBrowsingHistory.setCreatedBy(Long.parseLong(worker.getLoginId() + ""));
        ycqQzhVideoBrowsingHistory.setCreatedName(worker.getWorkerName());
        ycqQzhVideoBrowsingHistory.setPhone(worker.getMobile());
        ycqQzhVideoBrowsingHistory.setIsComplete(dto.getIsComplete());
        ycqQzhVideoBrowsingHistory.setBusinessID(dto.getBusinessID());
        ycqQzhVideoBrowsingHistory.setVideoTime(dto.getVideoTime());
        ycqQzhVideoBrowsingHistory.setCreatedTime(new Date());
        ycqQzhVideoBrowsingHistoryRepo.saveAndFlush(ycqQzhVideoBrowsingHistory);
        return Result.ofSuccess(true);
    }

    @Override
    public Result<List<YcqQzhOnlineTrainTypeDTO>> getTrainList(PageReq<YcqQzhOnlineTrainDTO> req) {
        List<String> types = new ArrayList<>();
        List<YcqQzhOnlineTrainTypeDTO> result = new ArrayList<>();
        types.add("乡村振兴培训");
        types.add("科技创新培训");
        types.add("建功西部陆海新通道建设培训");
        types.add("建功成渝地区双城经济全建设培训");
        List<YcqQzhOnlineTrain> bySecondType;
        for (String type : types) {
            String title=req.getQuery()==null?null:req.getQuery().getTitle();
            if (title==null){
                bySecondType = ycqQzhOnlineTrainRepo.findBySecondTypeAndState(type,"公示");
            }else {
                 bySecondType = ycqQzhOnlineTrainRepo.findBySecondTypeAndStateAndTitleLike(type,"公示","%"+title+"%");
            }

            if (bySecondType != null && bySecondType.size() > 0) {
                YcqQzhOnlineTrainTypeDTO ycqQzhOnlineTrainTypeDTO = new YcqQzhOnlineTrainTypeDTO();
                List<YcqQzhOnlineTrainDTO> ycqQzhOnlineTrainDTOS = new ArrayList<>();
                for (YcqQzhOnlineTrain ycqQzhOnlineTrain : bySecondType) {
                    YcqQzhOnlineTrainDTO ycqQzhOnlineTrainDTO = new YcqQzhOnlineTrainDTO();
                    BeanUtils.copyProperties(ycqQzhOnlineTrain, ycqQzhOnlineTrainDTO);
                    ycqQzhOnlineTrainDTOS.add(ycqQzhOnlineTrainDTO);
                }
                ycqQzhOnlineTrainTypeDTO.setType(type);
                ycqQzhOnlineTrainTypeDTO.setList(ycqQzhOnlineTrainDTOS);
                result.add(ycqQzhOnlineTrainTypeDTO);
            } else {
                YcqQzhOnlineTrainTypeDTO ycqQzhOnlineTrainTypeDTO = new YcqQzhOnlineTrainTypeDTO();
                ycqQzhOnlineTrainTypeDTO.setType(type);
                result.add(ycqQzhOnlineTrainTypeDTO);
            }
        }

        return Result.ofSuccess(result);
    }
}
