package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.TsmRoleDto;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.ReportOrgDto;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.*;
import com.crunii.micro.service.qzh.converter.RightsProtectionConvert;
import com.crunii.micro.service.qzh.enums.LinkStatusEnum;
import com.crunii.micro.service.qzh.enums.TypeEnum;
import com.crunii.micro.service.qzh.jdbc.YcqQzhProtectionJdbcDao;
import com.crunii.micro.service.qzh.service.CivilizationReportService;
import com.crunii.micro.service.qzh.service.YcqQzhProtectionCheckService;
import com.crunii.micro.service.qzh.util.CodeUtil;
import com.crunii.module.flow.api.FlowInstanceApi;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author lyx
 * @date 2023/11/15
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class YcqQzhProtectionCheckServiceImpl implements YcqQzhProtectionCheckService {

    @Resource
    private YcqQzhProtectionJdbcDao jdbcDao;
    @Resource
    private RightsProtectionRepo repo;
    @Resource
    private RightsProtectionAnnexRepo annexRepo;
    @Resource
    private YcqQzhRightsFlowRepo ycqQzhRightsFlowRepo;
    @Resource
    private YcqQzhRightsIndexRepo ycqQzhRightsIndexRepo;
    @Resource
    private YcqQzhRightsIndexRelRepo ycqQzhRightsIndexRelRepo;

    private final FlowInstanceApi flowInstanceApi;

    @Resource
    private CivilizationReportService civilizationReportService;

    //区县主管单位
    private static final Integer QX_MAN_AUDIT_ROLE = 10128;

    //区县团组织
    private static final Integer QX_ORG_AUDIT_ROLE = 10129;
    //市级主管单位
    private static final Integer CQ_MAN_AUDIT_ROLE = 10130;
    //市级团组织
    private static final Integer CQ_ORG_AUDIT_ROLE = 10131;

    //区县主管单位标识
    private static final Integer QX_MAN_AUDIT = 0;
    //区县团组织标识
    private static final Integer QX_ORG_AUDIT = 1;
    //市级主管单位标识
    private static final Integer CQ_MAN_AUDIT = 2;
    //市级团组织标识
    private static final Integer CQ_ORG_AUDIT = 3;

    @Override
    public Result<Long> doDraft(RightsProtectionCheckReqDTO req) {
//        CurrWorkerUtil.mockCurrWorker();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        // 提交为草稿，草稿不需要进入流程不调用doSystem方法
        Snowflake snowflake = new Snowflake();
        long flowId = snowflake.nextId();
        YcqQzhRightsProtection entity;
        LocalDateTime now = LocalDateTime.now();
        Long workId = Long.valueOf(worker.getOrgId());
        validateDraft(req);
        //新增草稿
        if (req.getId() == null) {
            entity = RightsProtectionConvert.INSTANCE.convert(req);
            entity.setApplicantCode(CodeUtil.codeRule(2));
            entity.setApplicantId(Long.valueOf(worker.getLoginId()));
            entity.setApplicantOrgId(workId);
            entity.setApplyType(TypeEnum.APPLYTYPE_1.getType());
            entity.setStatus(LinkStatusEnum.CHECK_0.getCode());
            entity.setLink(LinkStatusEnum.CHECK_0.getCode());
            entity.setCreateId(Long.valueOf(worker.getLoginId()));
            entity.setCreateBy(worker.getWorkerName());
            entity.setApplyTime(now);
            entity.setCreateTime(now);
            if (null != req.getUnitflag()) {
                entity.setUnitName(req.getNewUnitName());
                entity.setOldUnitName(req.getUnitName());
            }
            if (null != req.getOrgflag()) {
                entity.setOrganizationId(req.getNewOrganizationId());
                entity.setOrganizationName(req.getNewOrganizationName());
                entity.setOldOrganizationId(req.getOrganizationId());
                entity.setOldOrganizationName(req.getOrganizationName());
            }
            Long entityId = snowflake.nextId();
            entity.setId(entityId);
            saveFlow(workId, workId, workId, entityId, flowId, LinkStatusEnum.CHECK_0.getCode(), null, null, null, worker);
            saveAnnexs(req, entity, worker, flowId, 0);

        } else {
            //修改为草稿
            Long rightsProtectionId = req.getId();
            entity = repo.findById(rightsProtectionId).get();
            BeanUtils.copyProperties(req, entity);
            if (null != req.getUnitflag()) {
                entity.setUnitName(req.getNewUnitName());
                entity.setOldUnitName(req.getUnitName());
            }
            if (null != req.getOrgflag()) {
                entity.setOrganizationId(req.getNewOrganizationId());
                entity.setOrganizationName(req.getNewOrganizationName());
                entity.setOldOrganizationId(req.getOrganizationId());
                entity.setOldOrganizationName(req.getOrganizationName());
            }
            entity.setApplicantId(Long.valueOf(worker.getLoginId()));
            entity.setStatus(LinkStatusEnum.CHECK_0.getCode());
            entity.setUpdateId(Long.valueOf(worker.getLoginId()));
            entity.setUpdateBy(worker.getWorkerName());
            // 处理附件
            // 处理附件
            YcqQzhRightsFlow flow1 = jdbcDao.findMaxSortByRightsId(rightsProtectionId);
            saveAnnexs(req, entity, worker, flow1.getId(), 0);
        }
        entity = repo.save(entity);
        //保存指标关联表
        saveIndicator(req.getReviewIndicators(), entity);
        return new Result<>(entity.getId());
    }

    @Override
    public Result<Object> doOperate(RightsProtectionOperateReqDTO req) {
//        CurrWorkerUtil.mockCurrWorker();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        YcqQzhRightsProtection entity = repo.findByIdAndDelFlag(req.getId(), 0);
        Long proId = Long.valueOf(worker.getOrgId());
        Long shouldId = null;
        Long nextId = null;
        Integer opinion = req.getOpinion();
        if (opinion == 0 && Objects.isNull(req.getScanAnnex())) {
            throw new BusinessException("通过必须上传盖章版扫描件");
        }
        YcqQzhRightsFlow flow1 = jdbcDao.findMaxSortByRightsId(entity.getId());
        //判断组织
        validatedOrgAudit(worker, flow1);
        //角色判断
        if (LinkStatusEnum.CHECK_1.getCode().equals(entity.getLink())) {
            validatedRoleAudit(worker, 0);
        } else if (LinkStatusEnum.CHECK_2.getCode().equals(entity.getLink())) {
            validatedRoleAudit(worker, 1);
        }
        Snowflake snowflake = new Snowflake();
        long flowId = snowflake.nextId();
        String linkStr = "";
        if (0 == opinion) {
            if (Objects.isNull(req.getScanAnnex())) {
                throw new BusinessException("通过审核必须上传盖章版扫描件");
            }
            entity.setOpinion(opinion);
            if (ObjectUtil.isNotEmpty(req.getReason())) {
                if (req.getReason().length() <= 50) {
                    entity.setReason(req.getReason());
                } else {
                    throw new BusinessException("原因不能超过50字符!");
                }
            }
            entity.setUpdateBy(worker.getWorkerName());
            entity.setUpdateId(Long.valueOf(worker.getLoginId()));
            //确定下一步流程
            if (LinkStatusEnum.CHECK_1.getCode().equals(entity.getLink())) {
                entity.setStatus(LinkStatusEnum.CHECK_2.getCode());
                entity.setLink(LinkStatusEnum.CHECK_2.getCode());
                entity.setManagerOrgId(req.getNextOrgId());
                entity.setManagerOrgName(req.getNextOrgName());
                linkStr = LinkStatusEnum.CHECK_1.getLink();
                shouldId = entity.getOrganizationId();
                nextId = req.getNextOrgId();
            } else if (LinkStatusEnum.CHECK_2.getCode().equals(entity.getLink())) {
                entity.setOrganizationId(req.getNextOrgId());
                entity.setOrganizationName(req.getNextOrgName());
                if (TypeEnum.TYPE_0.getType().equals(entity.getSystemState())) {
                    entity.setStatus(LinkStatusEnum.CHECK_3.getCode());
                    entity.setLink(LinkStatusEnum.CHECK_3.getCode());
                    shouldId = entity.getManagerOrgId();
                    nextId = req.getNextOrgId();
                } else {
                    entity.setStatus(LinkStatusEnum.CHECK_4.getCode());
                    entity.setLink(LinkStatusEnum.CHECK_4.getCode());
                    shouldId = req.getNextOrgId();
                    nextId = entity.getOrganizationId();
                }
                linkStr = LinkStatusEnum.CHECK_2.getLink();
            } else {
                throw new BusinessException("类型不存在");
            }

        } else if (opinion == 3) {
            //直接结束
            entity.setStatus(LinkStatusEnum.END.getCode());
            entity.setLink(LinkStatusEnum.END.getCode());
            shouldId = entity.getApplicantOrgId();
            nextId = entity.getApplicantOrgId();
        } else {
            throw new BusinessException("类型不存在");
        }
        if (ObjectUtil.isNotEmpty(req.getScanAnnex())) {
            YcqQzhRightsProtectionAnnex annexEntity = null;
            annexEntity = RightsProtectionConvert.INSTANCE.convert(req.getScanAnnex());
            annexEntity.setApplicantCode(entity.getApplicantCode());
            annexEntity.setAnnexType(TypeEnum.ANNEXTYPE_1.getType());
            annexEntity.setFlowId(flowId);
            annexEntity.setId(snowflake.nextId());
            annexEntity.setDelFlag(0);
            annexEntity.setFileNewName(entity.getApplicantCode() + "-" + linkStr + "-" + worker.getOrgName());
            annexRepo.save(annexEntity);
        }
        //添加可以查看的权限
        ReportOrgDto dto = new ReportOrgDto();
        dto.setReportType(3);
        dto.setServiceId(entity.getId());
        dto.setContainCurrentOrg(true);
        List<Long> orgs = new ArrayList<>();
        orgs.add(Long.valueOf(worker.getOrgId()));
        orgs.add(shouldId);
        orgs.add(nextId);
        dto.setOrgIds(orgs);
        civilizationReportService.saveQueryOrg(dto, worker);
        //工作流结束
        saveFlow(proId, shouldId, nextId, entity.getId(), flowId, entity.getLink(), opinion, req.getReason(), null, worker);
        repo.save(entity);
        return Result.ofSuccess("审核成功！");

    }

    @Override
    public Result<List<YcqQzhRightsIndex>> getIndicator() {
        List<YcqQzhRightsIndex> allByDelFlag = ycqQzhRightsIndexRepo.findAllByDelFlag(0);
        return new Result<>(allByDelFlag);
    }

    @Override
    public Result<CheckMarketQueryWorkerDTO> getMarket(CheckQueryDTO dto) {
//        CurrWorkerUtil.mockCurrWorker();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        CheckMarketQueryWorkerDTO checkMarketQueryWorkerDTO = new CheckMarketQueryWorkerDTO();
        Set<String> listCode = new HashSet<>();
        for (Long id : dto.getIds()) {
            YcqQzhRightsFlow flow1 = jdbcDao.findMaxSortByRightsId(id);
            //判断组织
            validatedOrgAudit(worker, flow1);
            //角色判断
            if (LinkStatusEnum.CHECK_3.getCode().equals(flow1.getLink())) {
                validatedRoleAudit(worker, 2);
            } else if (LinkStatusEnum.CHECK_4.getCode().equals(flow1.getLink())) {
                validatedRoleAudit(worker, 3);
            }
            listCode.add(flow1.getLink());
        }
        if (0 == listCode.size() || listCode.size() > 1) {
            throw new BusinessException("勾选的数据存在不同流程数据或未勾选,请核对!");
        }
        //市主管
        Object[] objects = listCode.toArray();
        String linkCode = objects[0].toString();
        if (LinkStatusEnum.CHECK_3.getCode().equals(linkCode)) {
            //查询主表相关数据
            List<CheckMarketQueryDTO> marketList = jdbcDao.findMarket(dto, worker, LinkStatusEnum.CHECK_3.getCode());
            if (marketList.size() > 0) {
                //查询上一环节盖章版附件 通过主表ID去flow表找到对应流程上一级别 然后通过flowId去找到对应上一环节盖章版附件
                for (CheckMarketQueryDTO checkMarketQueryDTO : marketList) {
                    List<RightsProtectionAnnexDTO> annexDTO = jdbcDao.findLast(checkMarketQueryDTO, 1);
                    if (annexDTO.size() > 0) {
                        checkMarketQueryDTO.setScanAnnexWeb(annexDTO.get(0));
                    }
                }
                checkMarketQueryWorkerDTO.setMarkets(marketList);
            }
            //市团市
        } else if (LinkStatusEnum.CHECK_4.getCode().equals(linkCode)) {
            //查询主表相关数据
            List<CheckMarketQueryDTO> marketList = jdbcDao.findMarket(dto, worker, LinkStatusEnum.CHECK_4.getCode());
            if (marketList.size() > 0) {
                //
                for (CheckMarketQueryDTO checkMarketQueryDTO : marketList) {
                    //上一环节盖章附件
                    List<RightsProtectionAnnexDTO> annexDTO = jdbcDao.findLast(checkMarketQueryDTO, 1);
                    //上一环节汇总附件
                    List<RightsProtectionAnnexDTO> annexDTOMany = jdbcDao.findLast(checkMarketQueryDTO, 2);
                    if (annexDTO.size() > 0) {
                        checkMarketQueryDTO.setScanAnnexWeb(annexDTO.get(0));
                    }
                    if (annexDTOMany.size() > 0) {
                        checkMarketQueryDTO.setScanAnnexMany(annexDTOMany.get(0));
                    }
                }
                String fillingUnit = marketList.get(0).getFillingUnit();
                checkMarketQueryWorkerDTO.setFillingUnit(fillingUnit);
            }
            checkMarketQueryWorkerDTO.setMarkets(marketList);
        }
        return new Result<>(checkMarketQueryWorkerDTO);
    }

    @Override
    public Result<Object> doMarketCheck(CheckMarketOperateDTO dto) {
//        CurrWorkerUtil.mockCurrWorker();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        List<YcqQzhRightsProtection> entitys = repo
                .findAllByIdInAndDelFlag(dto.getMarkets().stream().map(CheckMarketQueryDTO::getId).collect(Collectors.toList()), 0);
        if (LinkStatusEnum.CHECK_3.getCode().equals(entitys.get(0).getLink())) {
            for (CheckMarketQueryDTO queryDTO : dto.getMarkets()) {
                if ((queryDTO.getOption() == 1 || queryDTO.getOption() == 2) && Objects.isNull(queryDTO.getScanAnnex())) {
                    throw new BusinessException("合格或优秀必须上传盖章版扫描件！");
                }
            }
        }
        if (LinkStatusEnum.CHECK_3.getCode().equals(entitys.get(0).getLink()) && ObjectUtil.isNotNull(dto.getFillingUnit()) && dto.getFillingUnit().length() > 50) {
            throw new BusinessException("填报单位长度不能超过50字!");
        }
        if (LinkStatusEnum.CHECK_3.getCode().equals(entitys.get(0).getLink()) && Objects.isNull(dto.getAnnex())) {
            throw new BusinessException("复核汇总表必须上传!");
        }

        Snowflake snowflake = new Snowflake();
        List<YcqQzhRightsProtectionAnnex> annexList = new ArrayList<>();
        for (YcqQzhRightsProtection entity : entitys) {
            List<CheckMarketQueryDTO> list = dto.getMarkets().stream().filter(o -> o.getId().equals(entity.getId())).collect(Collectors.toList());
            Integer opinion = list.get(0).getOption();
            long proId;
            long shouldId;
            long flowId = snowflake.nextId();
            CheckMarketQueryDTO dto1 = list.get(0);
            if (list.size() != 1) {
                throw new BusinessException("数据异常！");
            } else {
                if (LinkStatusEnum.CHECK_4.getCode().equals(entitys.get(0).getLink()) && (opinion.equals(3) || opinion.equals(4) || opinion.equals(5) || opinion.equals(6))) {
                    //直接结束
                    entity.setStatus(LinkStatusEnum.END.getCode());
                    entity.setLink(LinkStatusEnum.END.getCode());
                    proId = Long.valueOf(worker.getOrgId());
                    shouldId = Long.valueOf(worker.getOrgId());
                    if (null != dto1.getScanAnnex()) {
                        YcqQzhRightsProtectionAnnex annexEntity = RightsProtectionConvert.INSTANCE.convert(dto1.getScanAnnex());
                        annexEntity.setId(snowflake.nextId());
                        annexEntity.setApplicantCode(entity.getApplicantCode());
                        annexEntity.setAnnexType(TypeEnum.ANNEXTYPE_1.getType());
                        annexEntity.setFlowId(flowId);
                        annexEntity.setDelFlag(0);
                        annexList.add(annexEntity);
                    }
                } else {
                    if (LinkStatusEnum.CHECK_3.getCode().equals(entity.getLink())) {
                        entity.setLink(LinkStatusEnum.CHECK_4.getCode());
                        entity.setStatus(LinkStatusEnum.CHECK_4.getCode());
                        proId = Long.valueOf(worker.getOrgId());
                        shouldId = entity.getOrganizationId();
                        if (null == dto1.getScanAnnex()) {
                            throw new BusinessException("盖章版附件不能为空!");
                        }
                    } else {
                        entity.setLink(LinkStatusEnum.CHECK_5.getCode());
                        entity.setStatus(LinkStatusEnum.CHECK_5.getCode());
                        proId = Long.valueOf(worker.getOrgId());
                        shouldId = Long.valueOf(worker.getOrgId());
                    }
                    //盖章版扫描件
                    if (null != dto1.getScanAnnex()) {
                        YcqQzhRightsProtectionAnnex annexEntity1 = RightsProtectionConvert.INSTANCE.convert(dto1.getScanAnnex());
                        annexEntity1.setId(snowflake.nextId());
                        annexEntity1.setApplicantCode(entity.getApplicantCode());
                        annexEntity1.setAnnexType(TypeEnum.ANNEXTYPE_1.getType());
                        annexEntity1.setFlowId(flowId);
                        annexEntity1.setDelFlag(0);
                        annexList.add(annexEntity1);
                    }
                    //写入填写单位复核
                    entity.setFillingUnit(dto.getFillingUnit());
                    entity.setOpinion(dto1.getOption());
                    repo.save(entity);
                    if (null != dto.getAnnex()) {
                        //复核汇总表
                        YcqQzhRightsProtectionAnnex annexEntity2 = RightsProtectionConvert.INSTANCE.convert(dto.getAnnex());
                        annexEntity2.setAnnexType(TypeEnum.ANNEXTYPE_2.getType());
                        annexEntity2.setDelFlag(0);
                        annexEntity2.setApplicantCode(entity.getApplicantCode());
                        annexEntity2.setId(snowflake.nextId());
                        annexEntity2.setFlowId(flowId);
                        annexList.add(annexEntity2);
                    }
                }
                if (null != dto1.getOldUnitName()) {
                    saveFlow(proId, shouldId, null, entity.getId(), flowId, entity.getLink(), dto1.getOption(), dto1.getOldUnitName(), null, worker);
                } else {
                    saveFlow(proId, shouldId, null, entity.getId(), flowId, entity.getLink(), dto1.getOption(), null, null, worker);
                }
            }
        }
        annexRepo.saveAll(annexList);
        return new Result<>(true);
    }

    @Override
    public Result<RightsProtectionCheckQueryRespDTO> getCheck(Long id) {
        RightsProtectionCheckQueryRespDTO check = jdbcDao.getCheck(id);
        //关联指标
        List<CheckIndexReqDTO> checkIndexReqDTOList = jdbcDao.selectRights(id);
        check.setReviewIndicators(checkIndexReqDTOList);
        // 扫描件1  0其他附件
        List<YcqQzhRightsProtectionAnnex> type = annexRepo.findAllByFlowIdAndDelFlag(check.getFlowId(), 0);
        List<YcqQzhRightsProtectionAnnex> others = type.stream().filter(o -> o.getAnnexType() == 0).collect(Collectors.toList());
        List<RightsProtectionAnnexDTO> list = new ArrayList<>();
        for (YcqQzhRightsProtectionAnnex other : others) {
            RightsProtectionAnnexDTO dto1 = new RightsProtectionAnnexDTO();
            BeanUtils.copyProperties(other, dto1);
            list.add(dto1);
        }
        check.setOtherAnnex(list);
        List<YcqQzhRightsProtectionAnnex> annex = type.stream().filter(o -> o.getAnnexType() == 1).collect(Collectors.toList());
        if (annex.size() == 1) {
            RightsProtectionAnnexDTO dto1 = new RightsProtectionAnnexDTO();
            BeanUtils.copyProperties(annex.get(0), dto1);
            check.setScanAnnex(dto1);
            return new Result<>(check);
        }
        return new Result<>(check);
    }

    @Override
    public Result<String> annexTool(CheckQueryDTO dto) {
//        CurrWorkerUtil.mockCurrWorker();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        Optional<YcqQzhRightsProtection> byId = repo.findById(dto.getId());
        if (byId.isPresent()) {
            //application_code-流程名字-组织名字
            YcqQzhRightsProtection ycqQzhRightsProtection = byId.get();
            StringBuffer name = new StringBuffer();
            name.append(ycqQzhRightsProtection.getApplicantCode());
            if (null != worker.getOrgName()) {
                if (LinkStatusEnum.CHECK_3.getCode().equals(ycqQzhRightsProtection.getLink())) {
                    name.append("-" + LinkStatusEnum.CHECK_3.getLink());
                    name.append("-" + ycqQzhRightsProtection.getManagerOrgName());
                } else if (LinkStatusEnum.CHECK_4.getCode().equals(ycqQzhRightsProtection.getLink())) {
                    name.append("-" + LinkStatusEnum.CHECK_4.getLink());
                    name.append("-" + ycqQzhRightsProtection.getOrganizationName());
                }
            } else {
                return Result.ofFail("当前账号对应组织名称异常,请联系系统管理员");
            }
            return new Result<>(name.toString());
        }
        return Result.ofFail("数据异常,未查询到对应数据");
    }

    @Override
    public Result<PageRsp<RightsProtectionRecordPageRespDTO>> recordPage(PageReq<ProtectionRecordDTO> req) {
//        CurrWorkerUtil.mockCurrWorker();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        PageRsp<RightsProtectionRecordPageRespDTO> resp = jdbcDao.recordPage(req, validatedSuper(worker), 1);
        if (resp.getResult().size() > 0) {
            recordPageOrg(resp.getResult());
        }
        return new Result<>(resp);
    }

    /**
     * @Description: 处理团组织跟主管单位回显问题
     * @Param: [req]
     * @return: void
     * @Author: xiang xin
     * @Date: 2023/12/9
     */
    public void recordPageOrg(List<RightsProtectionRecordPageRespDTO> req) {
        for (RightsProtectionRecordPageRespDTO entity : req) {
            String link = "";
            if (TypeEnum.APPLYTYPE_0.getVal().equals(entity.getApplyType())) {
                if (entity.getSystemState() == 0) {
                    if (entity.getMunicipalLevelInternalInstitutions() == 0) {
                        link = LinkStatusEnum.CREATE_1.getCode();
                    } else {
                        link = LinkStatusEnum.CREATE_3.getCode();
                    }
                } else {
                    if (entity.getMunicipalLevelInternalInstitutions() == 0) {
                        link = LinkStatusEnum.CREATE_2.getCode();
                    } else {
                        link = LinkStatusEnum.CREATE_4.getCode();
                    }
                }
            } else if (TypeEnum.APPLYTYPE_1.getVal().equals(entity.getApplyType())) {
                if (entity.getSystemState() == 0) {
                    if (entity.getMunicipalLevelInternalInstitutions() == 0) {
                        link = LinkStatusEnum.CHECK_1.getCode();
                    } else {
                        link = LinkStatusEnum.CHECK_3.getCode();
                    }
                } else {
                    if (entity.getMunicipalLevelInternalInstitutions() == 0) {
                        link = LinkStatusEnum.CHECK_2.getCode();
                    } else {
                        link = LinkStatusEnum.CHECK_4.getCode();
                    }
                }
            }
            RightsProtectionFlowOrgDTO flow = jdbcDao.findMaxSortByRightsIdAndLink(entity.getId(), link);
            //根据市级内设机构 和 systemState 查询提交的第一个环节的should和next
            //再判断分别是哪个主管单位和团组织
            if (entity.getSystemState() == 0) {
                entity.setManagerOrgId(flow.getShouldOrgId());
                entity.setManagerOrgName(flow.getShouldOrg());
                entity.setOrganizationId(flow.getNextOrgId());
                entity.setOrganizationName(flow.getNextOrg());
            } else {
                entity.setOrganizationId(flow.getShouldOrgId());
                entity.setOrganizationName(flow.getShouldOrg());
            }
        }
    }


    @Override
    public Result<Object> doCheck(RightsProtectionCheckReqDTO req) {
//        CurrWorkerUtil.mockCurrWorker();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        // 提交为草稿，草稿不需要进入流程不调用doSystem方法
        Snowflake snowflake = new Snowflake();
        long flowId = snowflake.nextId();
        YcqQzhRightsProtection entity;
        //提交为复核
        //新增为复核
        // 数据和权限校验
        validate(req);
        validateDraft(req);
        if (req.getId() == null) {
            throw new BusinessException("请先保存为草稿，打印生成的文件再上传提交");
        } else {
            //修改为复核
            Long rightsProtectionId = req.getId();
            entity = repo.findById(rightsProtectionId).get();
            BeanUtils.copyProperties(req, entity);
            entity.setUpdateId(Long.valueOf(worker.getLoginId()));
            entity.setUpdateBy(worker.getWorkerName());
            entity.setApplyTime(LocalDateTime.now());
            if (null != req.getUnitflag()) {
                entity.setUnitName(req.getNewUnitName());
                entity.setOldUnitName(req.getUnitName());
            }
            if (null != req.getOrgflag()) {
                entity.setOrganizationId(req.getNewOrganizationId());
                entity.setOrganizationName(req.getNewOrganizationName());
                entity.setOldOrganizationId(req.getOrganizationId());
                entity.setOldOrganizationName(req.getOrganizationName());
            }
            RightsProtectionAnnexDTO scanAnnex = req.getScanAnnex();
            scanAnnex.setFileNewName(entity.getApplicantCode() + "-申报复核-" + worker.getOrgName());
//			saveAnnexs(req, entity, worker, flowId);
        }
        //  查询静态表中的值 根据value_id 查询 value_desc是不是以1开头
        String b = jdbcDao.getDescById(req.getSystemId());
        Long orgId = Long.valueOf(worker.getOrgId());
        boolean code = false;
        if (StringUtils.isNotEmpty(b)) {
            code = b.startsWith("1");
        }
        Long shouldId = null;
        Long nextId = null;
        //确定下一步流程
        if (code) {
            if (req.getMunicipalLevelInternalInstitutions().equals(TypeEnum.TYPE_1.getType())) {
                entity.setStatus(LinkStatusEnum.CHECK_4.getCode());
                entity.setLink(LinkStatusEnum.CHECK_4.getCode());
                // TODO 2023年11月15日16:09:01 组织打标
                if (null != req.getOrgflag()) {
                    entity.setOrganizationId(req.getNewOrganizationId());
                    entity.setOrganizationName(req.getNewOrganizationName());
                    shouldId = req.getNewOrganizationId();
                } else {
                    shouldId = req.getOrganizationId();
                }
            } else {
                entity.setStatus(LinkStatusEnum.CHECK_2.getCode());
                entity.setLink(LinkStatusEnum.CHECK_2.getCode());
                if (null != req.getOrgflag()) {
                    entity.setOrganizationId(req.getNewOrganizationId());
                    entity.setOrganizationName(req.getNewOrganizationName());
                    shouldId = req.getNewOrganizationId();
                } else {
                    shouldId = req.getOrganizationId();
                }
                nextId = req.getManagerOrgId();
            }
            entity.setSystemState(1);
        } else {
            if (req.getMunicipalLevelInternalInstitutions().equals(TypeEnum.TYPE_1.getType())) {
                entity.setStatus(LinkStatusEnum.CHECK_3.getCode());
                entity.setLink(LinkStatusEnum.CHECK_3.getCode());
            } else {
                entity.setStatus(LinkStatusEnum.CHECK_1.getCode());
                entity.setLink(LinkStatusEnum.CHECK_1.getCode());
            }
            shouldId = req.getManagerOrgId();
            if (null != req.getOrgflag()) {
                entity.setOrganizationId(req.getNewOrganizationId());
                entity.setOrganizationName(req.getNewOrganizationName());
                nextId = req.getNewOrganizationId();
            } else {
                nextId = req.getOrganizationId();
            }
            entity.setSystemState(0);
        }
        //添加可以查看的组织权限
        ReportOrgDto dto = new ReportOrgDto();
        dto.setReportType(3);
        dto.setServiceId(entity.getId());
        dto.setContainCurrentOrg(true);
        List<Long> orgs = new ArrayList<>();
        orgs.add(Long.valueOf(worker.getOrgId()));
        if (Objects.nonNull(req.getManagerOrgId())) {
            orgs.add(req.getManagerOrgId());
        }
        if (null != req.getOrgflag()) {
            orgs.add(req.getNewOrganizationId());
        } else {
            orgs.add(req.getOrganizationId());
        }
        dto.setOrgIds(orgs);
        civilizationReportService.saveQueryOrg(dto, worker);
        entity.setApplyTime(LocalDateTime.now());
        saveFlow(orgId, shouldId, nextId, req.getId(), flowId, entity.getLink(), null, null, null, worker);
        repo.saveAndFlush(entity);
        saveAnnexs(req, entity, worker, flowId, 1);
        //保存指标关联表
        saveIndicator(req.getReviewIndicators(), entity);
        return new Result<>(true);
    }

    private void saveAnnexs(RightsProtectionCheckReqDTO dto, YcqQzhRightsProtection entity, Worker worker, Long flowId, Integer isDraf) {
        ArrayList<YcqQzhRightsProtectionAnnex> annexes = new ArrayList<>();
        Snowflake snowflake = new Snowflake();
        if (isDraf == 0) {
            List<YcqQzhRightsProtectionAnnex> allByFlowId = annexRepo.findAllByFlowIdAndDelFlag(flowId, 0);
            if (CollectionUtils.isNotEmpty(allByFlowId)) {
                allByFlowId.forEach(o -> o.setDelFlag(1));
                annexes.addAll(allByFlowId);
            }
        }
        if (Objects.nonNull(dto.getScanAnnex())) {
            YcqQzhRightsProtectionAnnex scanAnnex = RightsProtectionConvert.INSTANCE.convert(dto.getScanAnnex());
            scanAnnex.setId(snowflake.nextId());
            scanAnnex.setApplicantCode(entity.getApplicantCode());
            scanAnnex.setFlowId(flowId);
            scanAnnex.setBusinessId(entity.getId());
            scanAnnex.setCreateBy(worker.getWorkerName());
            scanAnnex.setCreateId(Long.valueOf(worker.getLoginId()));
            scanAnnex.setCreateTime(LocalDateTime.now());
            scanAnnex.setAnnexType(TypeEnum.ANNEXTYPE_1.getType());
            annexes.add(scanAnnex);
        }
        if (CollectionUtils.isNotEmpty(dto.getOtherAnnex())) {
            for (RightsProtectionAnnexDTO annex : dto.getOtherAnnex()) {
                YcqQzhRightsProtectionAnnex annexEntity = RightsProtectionConvert.INSTANCE.convert(annex);
                annexEntity.setId(snowflake.nextId());
                annexEntity.setApplicantCode(entity.getApplicantCode());
                annexEntity.setFlowId(flowId);
                annexEntity.setBusinessId(entity.getId());
                annexEntity.setCreateBy(worker.getWorkerName());
                annexEntity.setCreateId(Long.valueOf(worker.getLoginId()));
                annexEntity.setCreateTime(LocalDateTime.now());
                annexEntity.setAnnexType(TypeEnum.ANNEXTYPE_0.getType());
                annexes.add(annexEntity);
            }
        }
        if (CollectionUtils.isNotEmpty(annexes)) {
            annexRepo.saveAllAndFlush(annexes);
        }
    }

    private void validate(RightsProtectionCheckReqDTO rightsProtectionCheckReqDTO) {
        if (StringUtils.isEmpty(rightsProtectionCheckReqDTO.getUnitName())) {
            throw new BusinessException("单位名称不能为空！");
        }
        if (Objects.nonNull(rightsProtectionCheckReqDTO.getUnitflag()) && StringUtils.isEmpty(rightsProtectionCheckReqDTO.getNewUnitName())) {
            throw new BusinessException("选择单位变更后，变更后单位名称不能为空！");
        }
        if (Objects.isNull(rightsProtectionCheckReqDTO.getSystemId())) {
            throw new BusinessException("所属系统不能为空！");
        }
        if (Objects.isNull(rightsProtectionCheckReqDTO.getOrganizationId())) {
            throw new BusinessException("所属团组织不能为空！");
        }
        if (Objects.nonNull(rightsProtectionCheckReqDTO.getOrgflag()) && Objects.isNull(rightsProtectionCheckReqDTO.getNewOrganizationId())) {
            throw new BusinessException("选择团组织变更后，变更后所属团组织不能为空！");
        }
        if (StringUtils.isEmpty(rightsProtectionCheckReqDTO.getPhone())) {
            throw new BusinessException("联系电话不能为空！");
        }
        if (StringUtils.isEmpty(rightsProtectionCheckReqDTO.getAddress())) {
            throw new BusinessException("通讯地址不能为空！");
        }
        if (StringUtils.isEmpty(rightsProtectionCheckReqDTO.getHeader())) {
            throw new BusinessException("权益岗负责人不能为空！");
        }
        if (null == rightsProtectionCheckReqDTO.getNamingTime()) {
            throw new BusinessException("命名时间不能为空！");
        }
        if (StringUtils.isEmpty(rightsProtectionCheckReqDTO.getNamingFileCode())) {
            throw new BusinessException("命名文件号不能为空！");
        }
        if (CollectionUtils.isEmpty(rightsProtectionCheckReqDTO.getReviewIndicators())) {
            throw new BusinessException("复合指标不能为空！");
        }
        if (Objects.isNull(rightsProtectionCheckReqDTO.getScanAnnex())) {
            throw new BusinessException("请上传盖章版扫描件！");
        }
        if (Objects.nonNull(rightsProtectionCheckReqDTO.getOtherAnnex()) &&
                rightsProtectionCheckReqDTO.getOtherAnnex().size() > 5) {
            throw new BusinessException("请上传最多5个附件！");
        }
        if (StringUtils.isEmpty(rightsProtectionCheckReqDTO.getZipCode())) {
            throw new BusinessException("邮编不能为空！");
        } else if (rightsProtectionCheckReqDTO.getZipCode().length() != 6) {
            throw new BusinessException("邮编格式不对！");
        }
    }

    private void saveIndicator(List<CheckIndexReqDTO> reviewIndicators, YcqQzhRightsProtection entity) {

        List<YcqQzhRightsIndexRel> list = ycqQzhRightsIndexRelRepo.findAllByRightsIdAndDelFlag(entity.getId(), 0);

        if ((CollectionUtils.isNotEmpty(list))) {
            list.stream().forEach(o -> o.setDelFlag(1));
        }
        if (reviewIndicators.size() > 0) {
            for (CheckIndexReqDTO index : reviewIndicators) {
                YcqQzhRightsIndexRel ycqQzhRightsIndexRel = new YcqQzhRightsIndexRel();
                ycqQzhRightsIndexRel.setIndexId(index.getId());
                ycqQzhRightsIndexRel.setRightsId(entity.getId());
                ycqQzhRightsIndexRel.setRel(index.getRel());
                ycqQzhRightsIndexRel.setDelFlag(0);
                list.add(ycqQzhRightsIndexRel);
            }
        }
        ycqQzhRightsIndexRelRepo.saveAllAndFlush(list);
    }

    private void saveFlow(Long processedOrgId, Long shouldId, Long nextId, Long rightsId, Long flowId, String link, Integer option, String reason, Long annexId, Worker woker) {
        // 根据选择条件查询第一次和第二次审核的组织，存入审核流程表
        YcqQzhRightsFlow flow = new YcqQzhRightsFlow();
        flow.setId(flowId);
        flow.setRightsId(rightsId);
        flow.setProcessedOrgId(processedOrgId);//所填单位的组织id
        flow.setShouldOrgId(shouldId);
        flow.setNextOrgId(nextId);
        flow.setLink(link);
        if (LinkStatusEnum.CHECK_5.getCode().equals(link)
                || LinkStatusEnum.END.getCode().equals(link)) {
            flow.setStatus(1);
        } else {
            flow.setStatus(0);
        }
        flow.setProcessedWork(woker.getWorkerName());
        flow.setProcessedDate(LocalDateTime.now());
        //flow.setAnnexId(annexId);
        List<YcqQzhRightsFlow> ycqQzhRightsFlows = new ArrayList<>();
        YcqQzhRightsFlow flow1 = jdbcDao.findMaxSortByRightsId(rightsId);
        if (Objects.nonNull(flow1)) {
            flow1.setStatus(1);
            flow1.setResult(option);
            flow1.setMsg(reason);
            flow.setSort(flow1.getSort() + 1);
            flow.setParentId(flow1.getId());
            ycqQzhRightsFlows.add(flow1);
        } else {
            flow.setSort(0);
        }
        ycqQzhRightsFlows.add(flow);
        ycqQzhRightsFlowRepo.saveAllAndFlush(ycqQzhRightsFlows);
    }

    private boolean validatedOrgAudit(Worker worker, YcqQzhRightsFlow flow1) {
        if (Long.valueOf(worker.getOrgId()).equals(flow1.getShouldOrgId())) {
            return true;
        }
        if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            return true;
        }
        throw new BusinessException("数据权限不足！");
    }

    /**
     * 判断角色权限
     *
     * @param worker worker
     * @return 重庆市团委权限true，区县团委false
     */
    private boolean validatedRoleAudit(Worker worker, int code) {
        if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            return true;
        }
        List<TsmRoleDto> roles = worker.getRoles();
        List<Integer> roleIds = roles.stream().map(TsmRoleDto::getRoleId).collect(Collectors.toList());

        if (code == QX_MAN_AUDIT) {
            // 判断是否区县级主管单位审核
            if (roleIds.contains(QX_MAN_AUDIT_ROLE)) {
                return true;
            }
        } else if (code == QX_ORG_AUDIT) {
            // 判断是否区县级团组织审核
            if (roleIds.contains(QX_ORG_AUDIT_ROLE)) {
                return true;
            }
        } else if (code == CQ_MAN_AUDIT) {
            // 判断是否市级主管单位审核
            if (roleIds.contains(CQ_MAN_AUDIT_ROLE)) {
                return true;
            }
        } else if (code == CQ_ORG_AUDIT) {
            // 判断是否市级团组织审核
            if (roleIds.contains(CQ_ORG_AUDIT_ROLE)) {
                return true;
            }
        }
        throw new BusinessException("数据中有该角色权限无法处理,请核对!");
    }

    private boolean validatedSuper(Worker worker) {
        if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            return true;
        }
        return false;
    }

    private void validateDraft(RightsProtectionCheckReqDTO req) {
        if (StringUtils.isNotEmpty(req.getUnitName()) && req.getUnitName().length() > 50) {
            throw new BusinessException("单位名称输入超长，不超过50字！");
        }
        if (StringUtils.isNotEmpty(req.getNewUnitName()) && req.getNewUnitName().length() > 50) {
            throw new BusinessException("变更后单位名称输入超长，不超过50字！");
        }
        if (StringUtils.isNotEmpty(req.getHeader()) && req.getHeader().length() > 50) {
            throw new BusinessException("权益岗负责人，不超过50字！");
        }
        if (StringUtils.isNotEmpty(req.getPhone()) && req.getPhone().length() != 11) {
            throw new BusinessException("联系电话格式不对！");
        }
        if (StringUtils.isNotEmpty(req.getAddress()) && req.getAddress().length() > 100) {
            throw new BusinessException("通讯地址超长，不超过100字！");

        }
        if (StringUtils.isNotEmpty(req.getZipCode()) && req.getZipCode().length() != 6) {
            throw new BusinessException("邮编格式不对！");
        }
        if (Objects.nonNull(req.getOtherAnnex())
                && req.getOtherAnnex().size() > 5) {
            throw new BusinessException("请上传最多5个附件！");
        }
    }
}
