package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.spring.SpringUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQzhQagApply;
import com.crunii.micro.domain.entity.main.YcqQzhRightsProtection;
import com.crunii.micro.domain.repo.main.RightsProtectionRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.ds.uid.util.DateUtils;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.CheckMarketQueryDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.CheckMarketQueryWorkerDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.RightsProtectionCheckReqDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.RightsProtectionCreateReqDTO;
import com.crunii.micro.service.qzh.dto.YcqQzhRightsIndexRespDTO;
import com.crunii.micro.service.qzh.enums.CheckMessageEnum;
import com.crunii.micro.service.qzh.jdbc.YcqQzhRightsIndexJdbcDao;
import com.crunii.micro.service.qzh.service.YcqQzhProtectionCheckService;
import com.crunii.micro.service.qzh.service.YcqQzhProtectionReportService;
import com.crunii.micro.service.qzh.service.YcqQzhProtectionService;
import com.crunii.micro.service.qzh.utils.FileUtil;
import com.crunii.micro.service.qzh.utils.PDFAddWatermark;
import com.crunii.micro.service.qzh.utils.WatermarkUtil;
import com.crunii.micro.service.qzh.utils.XwpfTUtil;
import com.itextpdf.text.*;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qzh.api.constants.QzhConstants.WORKER_NOT_EXISTS;

/**
 * @BelongsProject: microservice
 * @BelongsPackage: com.crunii.micro.service.qzh.service.impl
 * @Author: limy
 * @CreateTime: 2023-11-17  15:46
 * @Description: 权益岗申报预览导出
 * @Version: 1.0
 */
@Service
public class YcqQzhProtectionReportServiceImpl implements YcqQzhProtectionReportService {
    @Resource
    private YcqQzhProtectionService ycqQzhProtectionService;
    @Resource
    private ObsService obsService;
    @Resource
    private RightsProtectionRepo rightsProtectionRepo;
    @Resource
    private YcqQzhRightsIndexJdbcDao ycqQzhRightsIndexJdbcDao;

    @Override
    public ResponseEntity<byte[]> createPreview(HttpServletRequest request, HttpServletResponse response, String id) throws IOException {
        Long aLong = Long.valueOf(id);
       // CurrWorkerUtil.mockCurrWorker();
        Worker worker = getWorker();
        Optional<YcqQzhRightsProtection> byId = rightsProtectionRepo.findById(aLong);
        if (!byId.isPresent()) {
            throw new BusinessException("申报不存在");
        }

        try {
            YcqQzhRightsProtection data = byId.get();
            // 构建HTTP响应头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_PDF);
            headers.setContentDispositionFormData("attachment", "exportedFile.pdf");

            ByteArrayOutputStream outputStream = getPdfWithWatermark(data, worker, response);

            byte[] byteArray = outputStream.toByteArray();
            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(byteArray);
            ByteArrayOutputStream byteArrayOutputStream = PDFAddWatermark.addWatermarkToPDF(byteArrayInputStream, "青春重庆 " + worker.getWorkerName() + " " + worker.getPhone());
            return ResponseEntity.ok().headers(headers).body(byteArrayOutputStream.toByteArray());
        } catch (DocumentException | IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public ResponseEntity<byte[]> checkPreview(HttpServletRequest request, HttpServletResponse response, String id) {
        Long aLong = Long.valueOf(id);
        //CurrWorkerUtil.mockCurrWorker();
        Worker worker = getWorker();
        Optional<YcqQzhRightsProtection> byId = rightsProtectionRepo.findById(aLong);
        if (!byId.isPresent()) {
            throw new BusinessException("申报不存在");
        }
        try {
            YcqQzhRightsProtection ycqQzhRightsProtection = byId.get();
            // 构建HTTP响应头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_PDF);
            headers.setContentDispositionFormData("attachment", URLEncoder.encode("全国维护青少年权益岗复核表","utf-8")+".pdf");

            ByteArrayOutputStream outputStream = getPdfWithCheck(ycqQzhRightsProtection, worker, response);

            byte[] byteArray = outputStream.toByteArray();
            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(byteArray);
            ByteArrayOutputStream byteArrayOutputStream = PDFAddWatermark.addWatermarkToPDF(byteArrayInputStream, "青春重庆 " + worker.getWorkerName() + " " + worker.getPhone());
            return ResponseEntity.ok().headers(headers).body(byteArrayOutputStream.toByteArray());
        } catch (DocumentException | IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public ResponseEntity<byte[]> cityPreview(HttpServletRequest request, HttpServletResponse response, CheckMarketQueryWorkerDTO dto) {
       //CurrWorkerUtil.mockCurrWorker();
        Worker worker = getWorker();
        try {
            // 构建HTTP响应头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_PDF);
            //设置响应字符集
            response.setCharacterEncoding("UTF-8");
            headers.setContentDispositionFormData("attachment", URLEncoder.encode("全国维护青少年权益岗复核情况汇总表","utf-8")+".pdf");

            // 生成pdf
            ByteArrayOutputStream outputStream = getPdfWithCity(dto, worker, response);
            byte[] byteArray = outputStream.toByteArray();

            // 水印生成
            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(byteArray);
            ByteArrayOutputStream byteArrayOutputStream = PDFAddWatermark.addWatermarkToPDF(byteArrayInputStream, "青春重庆 " + worker.getWorkerName() + " " + worker.getPhone());
            return ResponseEntity.ok().headers(headers).body(byteArrayOutputStream.toByteArray());
        } catch (DocumentException | IOException e) {
            throw new RuntimeException(e);
        }
    }

    private ByteArrayOutputStream getPdfWithWatermark(YcqQzhRightsProtection data, Worker worker, HttpServletResponse response) throws DocumentException, IOException {

        //创建一个文档(默认大小A4纸)
        Document document = new Document();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        PdfWriter.getInstance(document,byteArrayOutputStream);
        document.open();

        BaseFont bfChinese = BaseFont.createFont("/font/AdobeSongStd-Light.otf",BaseFont.IDENTITY_H,BaseFont.NOT_EMBEDDED);

        Font blackFont = new Font(bfChinese);
        blackFont.setColor(BaseColor.BLACK);

        PdfPTable table0 = new PdfPTable(2);
        // 构建每个单元格
        PdfPCell cell1 = new PdfPCell(new Paragraph("附件1",new Font(bfChinese, 12f, Font.BOLD)));
        // 设置背景颜色
        cell1.setBackgroundColor(BaseColor.WHITE);
        // 设置高度
        cell1.setFixedHeight(30);
        // 设置内容水平居中显示
        cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
        // 设置垂直居中
        cell1.setVerticalAlignment(Element.ALIGN_TOP);
        cell1.setBorder(0);
        table0.addCell(cell1);

        Font redFont = new Font(bfChinese);
        //redFont.setSize(15f);
        //redFont.setColor(BaseColor.BLACK);
        redFont.setColor(BaseColor.RED);
        // 构建每个单元格
        PdfPCell cell2 = new PdfPCell(new Paragraph("申报编号: "+data.getApplicantCode(),redFont));
        // 设置背景颜色
        cell2.setBackgroundColor(BaseColor.WHITE);
        // 设置高度
        cell2.setFixedHeight(30);
        // 设置内容水平居中显示
        cell2.setHorizontalAlignment(Element.ALIGN_RIGHT);
        // 设置垂直居中
        cell2.setVerticalAlignment(Element.ALIGN_TOP);
        cell2.setBorder(0);
        table0.addCell(cell2);
        document.add(table0);
        blackFont.setSize(20f);
        Paragraph textgraph1 = new Paragraph("创建单位申报表",new Font(bfChinese, 20f, Font.BOLD));
        textgraph1.setAlignment(Element.ALIGN_CENTER);
        textgraph1.setSpacingBefore(10f);
        textgraph1.setLeading(10f);
        document.add(textgraph1);

        blackFont.setSize(10f);
        PdfPTable mainTable1 = new PdfPTable(6);
        mainTable1.setSpacingBefore(16f);
        mainTable1.addCell(getPdfPCell("单位名称",blackFont,1,1,40));
        mainTable1.addCell(getPdfPCell(data.getUnitName(),blackFont,5,2,40));
        document.add(mainTable1);

        PdfPTable mainTable2 = new PdfPTable(6);
        mainTable2.addCell(getPdfPCell("通讯地址",blackFont,1,1,40));
        mainTable2.addCell(getPdfPCell(data.getAddress(),blackFont,5,2,40));
        document.add(mainTable2);

        PdfPTable mainTable3 = new PdfPTable(6);
        mainTable3.addCell(getPdfPCell("联系电话",blackFont,1,1,40));
        mainTable3.addCell(getPdfPCell(data.getPhone(),blackFont,3,1,40));
        mainTable3.addCell(getPdfPCell("邮编",blackFont,1,1,40));
        mainTable3.addCell(getPdfPCell(data.getZipCode(),blackFont,1,1,40));
        document.add(mainTable3);

        PdfPTable mainTable4 = new PdfPTable(6);
        mainTable4.addCell(getPdfPCell("何时被命名为省级维护青少年权益岗称号\n（不作要求，有则填，无则空白）",blackFont,4,1,80));
        mainTable4.addCell(getPdfPCell(data.getCreateProTime(),blackFont,2,1,80));
        document.add(mainTable4);

        PdfPTable mainTable5 = new PdfPTable(12);
        mainTable5.addCell(getPdfPCell("创\n" +
                "建\n" +
                "方\n" +
                "案",blackFont,1,1,500));
        mainTable5.addCell(getPdfPCell2(ObjectUtil.isEmpty(data.getProgramme())?"":data.getProgramme().replaceAll("<[^>]*>",""),blackFont,11,1,500));
        document.add(mainTable5);

        PdfPTable mainTable6 = new PdfPTable(12);
        mainTable6.addCell(getPdfPCell("地市\n级主\n管单\n位意\n见",blackFont,1,1,160));
        mainTable6.addCell(getPdfPCellSeal("签 章       \n\n年  月  日  \n",blackFont,5,1,160));
        mainTable6.addCell(getPdfPCell("地市\n级团\n委意\n见",blackFont,1,1,160));
        mainTable6.addCell(getPdfPCellSeal("签 章       \n\n年  月  日  \n",blackFont,5,1,160));
        document.add(mainTable6);

        PdfPTable mainTable7 = new PdfPTable(12);
        mainTable7.addCell(getPdfPCell("省级\n" +
                "主管\n" +
                "单位\n" +
                "意见",blackFont,1,1,160));
        mainTable7.addCell(getPdfPCellSeal("签 章       \n\n年  月  日  \n",blackFont,11,1,160));
        document.add(mainTable7);

        PdfPTable mainTable8 = new PdfPTable(12);
        mainTable8.addCell(getPdfPCell("省级\n" +
                "团委\n" +
                "意见",blackFont,1,1,160));
        mainTable8.addCell(getPdfPCellSeal("签 章       \n\n年  月  日  \n",blackFont,11,1,160));
        document.add(mainTable8);


        document.close();

        return byteArrayOutputStream;
    }


    private ByteArrayOutputStream getPdfWithCheck(YcqQzhRightsProtection data, Worker worker, HttpServletResponse response) throws DocumentException, IOException {

        //创建一个文档(默认大小A4纸)
        Document document = new Document();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        PdfWriter.getInstance(document,byteArrayOutputStream);
        document.open();

        BaseFont bfChinese = BaseFont.createFont("/font/AdobeSongStd-Light.otf",BaseFont.IDENTITY_H, BaseFont.EMBEDDED);

        Font font = new Font(bfChinese);
        font.setSize(15f);
        font.setColor(BaseColor.RED);
        Paragraph textgraph1 = new Paragraph("申报编码:"+data.getApplicantCode(),font);
        textgraph1.setAlignment(Element.ALIGN_RIGHT);
        textgraph1.setSpacingBefore(30f);
        document.add(textgraph1);

        font.setColor(BaseColor.BLACK);
        Paragraph textgraph2 = new Paragraph("附件1",new Font(bfChinese, 15f, Font.BOLD));
        textgraph2.setAlignment(Element.ALIGN_LEFT);
        textgraph2.setSpacingBefore(5f);
        document.add(textgraph2);

        font.setSize(20f);
        Paragraph textgraph3 = new Paragraph("全国维护青少年权益岗复核表",new Font(bfChinese, 20f, Font.BOLD));
        textgraph3.setAlignment(Element.ALIGN_CENTER);
        textgraph3.setSpacingBefore(5f);
        document.add(textgraph3);

        font.setSize(10f);
        PdfPTable mainTable1 = new PdfPTable(6);
        mainTable1.setSpacingBefore(10f);
        mainTable1.addCell(getPdfPCell("单位名称",font,1,1,40));
        mainTable1.addCell(getPdfPCell(data.getUnitName(),font,5,1,40));
        mainTable1.addCell(getPdfPCell("通讯地址",font,1,1,40));
        mainTable1.addCell(getPdfPCell(data.getAddress(),font,5,1,40));
        mainTable1.addCell(getPdfPCell("权益岗负责人",font,1,1,40));
        mainTable1.addCell(getPdfPCell(data.getHeader(),font,1,1,40));
        mainTable1.addCell(getPdfPCell("联系电话",font,1,1,40));
        mainTable1.addCell(getPdfPCell(data.getPhone(),font,3,1,40));
        mainTable1.addCell(getPdfPCell("命名时间",font,1,1,40));
        mainTable1.addCell(getPdfPCell(ObjectUtil.isEmpty(data.getNamingTime())?"":data.getNamingTime().toString(),font,5,1,40));
        document.add(mainTable1);

        PdfPTable mainTable2 = new PdfPTable(12);
        font.setSize(30f);
        List<YcqQzhRightsIndexRespDTO> list = ycqQzhRightsIndexJdbcDao.getIndexAndRel(data.getId());
        mainTable2.addCell(getPdfPCell("复核\n指标",font,1,ObjectUtil.isEmpty(list)?1:list.size(),60));
        font.setSize(10f);
        mainTable2.addCell(getPdfPCell("指标描述",font,8,1,60));
        mainTable2.addCell(getPdfPCell("自查情况\n" +
                "（是/否）",font,3,1,60));
        if(ObjectUtil.isNotEmpty(list)){
            for (YcqQzhRightsIndexRespDTO ycqQzhRightsIndexRespDTO : list) {
                mainTable2.addCell(getPdfPCell(ycqQzhRightsIndexRespDTO.getIndexData(),font,8,1,40));
                mainTable2.addCell(getPdfPCell(ycqQzhRightsIndexRespDTO.getRel().equals(0)?"否":"是",font,3,1,40));
            }
        }

//        mainTable2.addCell(getPdfPCell("拥护中国共产党领导，深入学习贯彻习近平新时代中国特色社会主义思想，遵守党规党纪和法律法规",font,8,1,38));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,38));
//        mainTable2.addCell(getPdfPCell("立足本系统基本职能，创造性开展服务青少年发展、维护青少年权益专项工作、专题活动，取得良好成效",font,8,1,38));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,38));
//        mainTable2.addCell(getPdfPCell("及时受理青少年咨询和投诉，建立与共青团组织及其12355青少年服务台（站、点）的个案转介、工作会商、联合处置机制",font,8,1,50));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,50));
//        mainTable2.addCell(getPdfPCell("有明确的创建工作保障措施和工作机制，确定专门机构或者指定专门人员从事青少年发展和权益维护工作，配套相应工作经费保障创建工作",font,8,1,50));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,50));
//        mainTable2.addCell(getPdfPCell("对服务青少年发展和权益维护的具体事项和标准作出公开承诺，在工作场所显著位置公布服务内容，树立良好社会形象",font,8,1,50));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,50));
//        mainTable2.addCell(getPdfPCell("所在单位或者组织机构主要负责人受到刑事处罚或者党纪政务重处分",font,8,1,38));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,38));
//        mainTable2.addCell(getPdfPCell("因工作职能变更不再直接从事权益岗创建项目相关工作",font,8,1,38));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,38));
//        mainTable2.addCell(getPdfPCell("严重违反党和国家法律法规、方针政策，造成不良社会影响",font,8,1,38));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,38));
//        mainTable2.addCell(getPdfPCell("在青少年权益维护方面履职不力，创建承诺未能兑现，社会评价不佳，工作质量严重下滑",font,8,1,38));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,38));
//        mainTable2.addCell(getPdfPCell("在复核中弄虚作假，存在欺报、瞒报、贿审等行为",font,8,1,38));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,38));
//        mainTable2.addCell(getPdfPCell("存在严重损害全国维护青少年权益岗形象等其他情形",font,8,1,38));
//        mainTable2.addCell(getPdfPCell("是",font,3,1,38));

        document.add(mainTable2);

        font.setSize(10f);
        PdfPTable mainTable3 = new PdfPTable(12);
        mainTable3.addCell(getPdfPCell("区县\n主管\n单位\n意见",font,1,1,160));
        mainTable3.addCell(getPdfPCellSeal("签 章       \n\n年  月  日  \n",font,11,1,160));
        document.add(mainTable3);

        PdfPTable mainTable4 = new PdfPTable(12);
        mainTable4.addCell(getPdfPCell("区县\n团委\n意见",font,1,1,160));
        mainTable4.addCell(getPdfPCellSeal("签 章       \n\n年  月  日  \n",font,11,1,160));
        document.add(mainTable4);

        PdfPTable mainTable5 = new PdfPTable(12);
        mainTable5.addCell(getPdfPCell("省级\n" +
                "主管\n" +
                "单位\n" +
                "意见",font,1,1,160));
        mainTable5.addCell(getPdfPCellSeal("签 章       \n\n年  月  日  \n",font,11,1,160));
        document.add(mainTable5);

        PdfPTable mainTable6 = new PdfPTable(12);
        mainTable6.addCell(getPdfPCell("省级\n" +
                "团委\n" +
                "意见",font,1,1,160));
        mainTable6.addCell(getPdfPCellSeal("签 章       \n\n年  月  日  \n",font,11,1,160));
        document.add(mainTable6);


        document.close();

        return byteArrayOutputStream;
    }

    private ByteArrayOutputStream getPdfWithCity(CheckMarketQueryWorkerDTO dto, Worker worker, HttpServletResponse response) throws DocumentException, IOException {

        //创建一个文档(默认大小A4纸)
        Document document = new Document();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        PdfWriter.getInstance(document,byteArrayOutputStream);
        document.open();

        BaseFont bfChinese = BaseFont.createFont("/font/AdobeSongStd-Light.otf",BaseFont.IDENTITY_H, BaseFont.EMBEDDED);

        Font font = new Font(bfChinese);

        font.setColor(BaseColor.BLACK);

        //font.setSize(15f);
        Paragraph textgraph1 = new Paragraph("附件2",new Font(bfChinese, 15f, Font.BOLD));
        textgraph1.setAlignment(Element.ALIGN_LEFT);
        textgraph1.setIndentationLeft(30f);// 左缩进
        textgraph1.setSpacingBefore(20f);
        textgraph1.setLeading(15f);
        document.add(textgraph1);

        //font.setSize(20f);
        Paragraph textgraph2 = new Paragraph("全国维护青少年权益岗复核情况汇总表",new Font(bfChinese, 20f, Font.BOLD));
        textgraph2.setAlignment(Element.ALIGN_CENTER);
        textgraph1.setSpacingBefore(20f);
        textgraph2.setLeading(15f);
        document.add(textgraph2);

        font.setSize(10f);
//        Paragraph textgraph3 = new Paragraph("填报单位（盖章）："+getChunk(dto.getWorketOrg()),font);
//        textgraph3.setAlignment(Element.ALIGN_LEFT);
//        textgraph3.setLeading(10);
//        document.add(textgraph3);
        Paragraph textgraph3 = new Paragraph();
        Font underlineFont = new Font(bfChinese, 10, Font.UNDERLINE);
        Phrase ph = new Phrase();
        //块
        ph.add(new Chunk("填报单位（盖章）：", font));
        ph.add(new Chunk(ObjectUtil.isEmpty(dto.getFillingUnit())?"":dto.getFillingUnit(), underlineFont));
        Phrase ph1 = new Phrase();
        ph1.add(new Chunk("                   填报时间：", font));
        ph1.add(new Chunk(DateUtils.formatDate(new Date(), com.crunii.micro.common.utils.DateUtils.DEFAULT_DATE_TIME_FORMAT), underlineFont));
        textgraph3.add(ph);
        textgraph3.add(ph1);
        textgraph3.setSpacingBefore(20f);//段落与段落的间距，我们一个ph就组成一个段落
        textgraph3.setAlignment(Element.ALIGN_CENTER);//设置对齐方式，右对齐
        textgraph3.setLeading(15f);//行间距
        document.add(textgraph3);

//        Paragraph textgraph4 = new Paragraph("填报时间："+getChunk(dto.getDate()),font);
//        textgraph4.setAlignment(Element.ALIGN_RIGHT);
//        textgraph3.setLeading(0);
//        document.add(textgraph4);


        PdfPTable mainTable1 = new PdfPTable(10);
        mainTable1.setSpacingBefore(20f);
        mainTable1.addCell(getPdfPCell("系统",font,1,1,100));
        mainTable1.addCell(getPdfPCell("命名单位\n（规范简称）",font,4,1,100));
        mainTable1.addCell(getPdfPCell("复核意见（优秀、合格、整改、命名自动取消、撤销命名）",font,2,1,100));
        mainTable1.addCell(getPdfPCell("备注\n（如有更名，请在本栏填写原命名单位名称）",font,3,1,100));

        document.add(mainTable1);


        if(ObjectUtil.isNotEmpty(dto.getMarkets())){
            Map<String, List<CheckMarketQueryDTO>> collect = dto.getMarkets().stream().collect(Collectors.groupingBy(CheckMarketQueryDTO::getSystemName, Collectors.toList()));
            for (String s : collect.keySet()) {
                PdfPTable mainTable2 = new PdfPTable(10);
                mainTable2.addCell(getPdfPCell(s,font,1,collect.get(s).size(),40));
                for (CheckMarketQueryDTO checkMarketQueryDTO : collect.get(s)) {
                    mainTable2.addCell(getPdfPCell(checkMarketQueryDTO.getUnitName(),font,4,1,40));
                    mainTable2.addCell(getPdfPCell(ObjectUtil.isEmpty(checkMarketQueryDTO.getOption())?"": CheckMessageEnum.getValByCode(checkMarketQueryDTO.getOption()).getVal(),font,2,1,40));
                    mainTable2.addCell(getPdfPCell(checkMarketQueryDTO.getOldUnitName(),font,3,1,40));
                }
                document.add(mainTable2);
            }
        }
        


        document.close();

        return byteArrayOutputStream;
    }

    public static PdfPCell getPdfPCell (String value,Font font,int colspan,int rowspan,Integer Height) {
        PdfPCell cell = new PdfPCell(new Phrase(value,font));
        cell.setUseBorderPadding(true);
        cell.setBorderColor(BaseColor.BLACK);
        cell.setFixedHeight(Height);
        cell.setHorizontalAlignment(Element.ALIGN_CENTER);
        cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
        cell.setColspan(colspan);
        cell.setRowspan(rowspan);
        return cell;
    }

    public static PdfPCell getPdfPCell2 (String value,Font font,int colspan,int rowspan,Integer Height) {
        Phrase ph = new Phrase(value, font);
        ph.setLeading(10f);
        //ph.setFirstLineIndent(12f);
        PdfPCell cell = new PdfPCell(ph);
        cell.setUseBorderPadding(true);
        cell.setBorderColor(BaseColor.BLACK);
        cell.setFixedHeight(Height);
        cell.setPaddingTop(10f);
//        cell.setHorizontalAlignment(Element.ALIGN_LEFT);
//        cell.setVerticalAlignment(Element.ALIGN_TOP);
        cell.setColspan(colspan);
        cell.setRowspan(rowspan);
        cell.setIndent(10f);
        cell.setLeading(1f,1f);
        return cell;
    }


    public static PdfPCell getPdfPCellSeal (String value,Font font,int colspan,int rowspan,Integer Height) {
        PdfPCell cell = new PdfPCell(new Phrase(value,font));
        cell.setUseBorderPadding(true);
        cell.setBorderColor(BaseColor.BLACK);
        cell.setFixedHeight(Height);
        cell.setPaddingRight(10f);
        cell.setPaddingBottom(10f);
        cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
        cell.setVerticalAlignment(Element.ALIGN_BOTTOM);
        cell.setColspan(colspan);
        cell.setRowspan(rowspan);
        return cell;
    }

    /**
     * 设置下划线
     *
     * @return
     */
    public static Chunk getChunk(String str) {
        // 创建Chunk对象，设置下划线的厚度为0.1
        Chunk underline = new Chunk(str);
        underline.setUnderline(0.1f, -1f);
        // 创建Chunk对象，设置下划线的厚度为1
        Chunk strike = new Chunk(str);
        strike.setUnderline(1f, 3f);
        return strike;
    }


    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        return worker;
    }
    public void beiyong() throws IOException {
        //        Result<YcqQzhRightsProtection> check = ycqQzhProtectionService.saveDraft(req);
//        YcqQzhRightsProtection data = check.getData();
        Map<String,Object> params = new HashMap<>();
//        params.put("${code}",data.getApplicantCode());
//        params.put("${unitName}",data.getUnitName());
//        params.put("${address}",data.getAddress());
//        params.put("${phone}",data.getPhone());
//        params.put("${zipCode}",data.getZipCode());
//        params.put("${programme}",data.getProgramme());
        params.put("${code}","oewdsnkkcmxdos");
        params.put("${unitName}","重庆市某某单位");

        InputStream is = getClass().getClassLoader().getResourceAsStream("static/word/创建单位申报表.docx");
        //InputStream is = new FileInputStream("C:\\Users\\17627\\Desktop\\1.docx");
        XWPFDocument doc = new XWPFDocument(is);
        XwpfTUtil.replaceInPara(doc, params);
        XwpfTUtil.replaceInTable(doc, params);
        WatermarkUtil.waterMarkDocXDocument(doc,"张三 18890764599");

        //String fileName = "D://test17.pdf";
//        response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
//        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        //OutputStream os = response.getOutputStream();

        //OutputStream os = new FileOutputStream(fileName);
        MultipartFile multipartFile = FileUtil.xwpfDocumentToCommonsMultipartFile(doc, "test777.docx");
        Result<FileVo> fileVoResult = obsService.feignFileUpload(multipartFile);


        //doc.write(os);
        //XwpfTUtil.close(os);
        XwpfTUtil.close(is);
    }
}
