package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.TsmRoleDto;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqQzhRightsFlow;
import com.crunii.micro.domain.entity.main.YcqQzhRightsProtection;
import com.crunii.micro.domain.entity.main.YcqQzhRightsProtectionAnnex;
import com.crunii.micro.domain.entity.main.YcqQzhRightsProtectionUpdate;
import com.crunii.micro.domain.repo.main.RightsProtectionAnnexRepo;
import com.crunii.micro.domain.repo.main.RightsProtectionRepo;
import com.crunii.micro.domain.repo.main.YcqQzhRightsFlowRepo;
import com.crunii.micro.domain.repo.main.YcqQzhRightsProtectionUpdateRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.QueryOrgDto;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.ReportOrgDto;
import com.crunii.micro.service.qzh.api.domain.dto.document.DocumentListRspDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.*;
import com.crunii.micro.service.qzh.converter.RightsProtectionConvert;
import com.crunii.micro.service.qzh.dto.AddCreateDTO;
import com.crunii.micro.service.qzh.enums.LinkStatusEnum;
import com.crunii.micro.service.qzh.enums.TypeEnum;
import com.crunii.micro.service.qzh.jdbc.YcqQzhDocumentJdbcDao;
import com.crunii.micro.service.qzh.jdbc.YcqQzhProtectionJdbcDao;
import com.crunii.micro.service.qzh.service.CivilizationReportService;
import com.crunii.micro.service.qzh.service.YcqQzhProtectionService;
import com.crunii.micro.service.qzh.util.CodeUtil;
import com.crunii.module.flow.api.FlowInstanceApi;
import com.crunii.module.flow.api.vo.conf.FlowConfRespVO;
import com.crunii.module.flow.dto.*;
import com.didiglobal.turbo.engine.model.InstanceData;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author lyx
 * @date 2023/11/15
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class YcqQzhProtectionServiceImpl implements YcqQzhProtectionService {

    @Resource
    private YcqQzhProtectionJdbcDao jdbcDao;
    @Resource
    private RightsProtectionRepo repo;
    @Resource
    private YcqQzhRightsProtectionUpdateRepo upadteRepo;
    @Resource
    private RightsProtectionAnnexRepo annexRepo;
    @Resource
    private YcqQzhRightsFlowRepo ycqQzhRightsFlowRepo;
    @Resource
    private YcqQzhDocumentJdbcDao ycqQzhDocumentJdbcDao;
    @Resource
    private CivilizationReportService civilizationReportService;

    private final FlowInstanceApi flowInstanceApi;

    //区县主管单位
    private static final Integer QX_MAN_AUDIT_ROLE = 10128;

    //区县团组织
    private static final Integer QX_ORG_AUDIT_ROLE = 10129;
    //市级主管单位
    private static final Integer CQ_MAN_AUDIT_ROLE = 10130;
    //市级团组织
    private static final Integer CQ_ORG_AUDIT_ROLE = 10131;

    //区县主管单位标识
    private static final Integer QX_MAN_AUDIT = 0;
    //区县团组织标识
    private static final Integer QX_ORG_AUDIT = 1;
    //市级主管单位标识
    private static final Integer CQ_MAN_AUDIT = 2;
    //市级团组织标识
    private static final Integer CQ_ORG_AUDIT = 3;

    /**
     * 申报记录查询
     *
     * @return
     */
    @Override
    public Result<PageRsp<RightsProtectionRecordPageRespDTO>> recordPage(PageReq<Object> dto) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        PageRsp<RightsProtectionRecordPageRespDTO> resp = jdbcDao.recordPageList(dto, validatedSuper(worker), 0);
        return new Result<>(resp);
    }

    @Override
    public Result<PageRsp<RightsProtectionRecordPageRespDTO>> updatePage(PageReq<Object> req) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        PageRsp<RightsProtectionRecordPageRespDTO> resp = jdbcDao.updatePageList(req, validatedSuper(worker));
        return new Result<>(resp);
    }

    @Override
    public Result<Object> doBack(RightsProtectionOperateReqDTO req) {
        Snowflake snowflake = new Snowflake();
        long flowId = snowflake.nextId();
        Worker worker = CurrWorkerUtil.getCurrWorker();
        Integer opinion = req.getOpinion();
        YcqQzhRightsProtection entity = repo.findByIdAndDelFlag(req.getId(), 0);
        YcqQzhRightsFlow flow1 = jdbcDao.findMaxSortByRightsId(entity.getId());
        //判断组织
        validatedOrgAudit(worker, flow1);
        //角色判断
        //复核
        if (0 == entity.getApplyType()) {
            if (LinkStatusEnum.CREATE_1.getCode().equals(entity.getLink())) {
                validatedRoleAudit(worker, 0);
            } else if (LinkStatusEnum.CREATE_2.getCode().equals(entity.getLink())) {
                validatedRoleAudit(worker, 1);
            } else if (LinkStatusEnum.CREATE_3.getCode().equals(entity.getLink())) {
                validatedRoleAudit(worker, 2);
            } else if (LinkStatusEnum.CREATE_4.getCode().equals(entity.getLink())) {
                validatedRoleAudit(worker, 3);
            }
        } else if (1 == entity.getApplyType()) {
            if (LinkStatusEnum.CHECK_1.getCode().equals(entity.getLink())) {
                validatedRoleAudit(worker, 0);
            } else if (LinkStatusEnum.CHECK_2.getCode().equals(entity.getLink())) {
                validatedRoleAudit(worker, 1);
            } else if (LinkStatusEnum.CHECK_3.getCode().equals(entity.getLink())) {
                validatedRoleAudit(worker, 2);
            } else if (LinkStatusEnum.CHECK_4.getCode().equals(entity.getLink())) {
                validatedRoleAudit(worker, 3);
            }
        }
        Long proId = Long.valueOf(worker.getOrgId());
        Long shouldId = null;
        Long nextId = null;
        if (opinion == 7) {
            //退回,原因必填且最多80字
            if (StringUtils.isEmpty(req.getReason())){
                throw new BusinessException("退回原因必填");
            }
            if (req.getReason().length()>80){
                throw new BusinessException("退回原因超长，请控制在80字以内！");
            }
            String link = "";
            if (0 == entity.getApplyType()) {
                entity.setStatus(LinkStatusEnum.CREATE_0.getCode());
                entity.setLink(LinkStatusEnum.CREATE_0.getCode());
                //查询草稿状态提交后的两个组织，附件，再设置到现在的草稿状态
                if (entity.getSystemState() == 0){
                    if (entity.getMunicipalLevelInternalInstitutions() == 0){
                        link = LinkStatusEnum.CREATE_1.getCode();
                    }else {
                        link = LinkStatusEnum.CREATE_3.getCode();
                    }
                }else{
                    if (entity.getMunicipalLevelInternalInstitutions() == 0){
                        link = LinkStatusEnum.CREATE_2.getCode();
                    }else {
                        link = LinkStatusEnum.CREATE_4.getCode();
                    }
                }
            } else if (1 == entity.getApplyType()) {
                entity.setStatus(LinkStatusEnum.CHECK_0.getCode());
                entity.setLink(LinkStatusEnum.CHECK_0.getCode());
                if (entity.getSystemState() == 0){
                    if (entity.getMunicipalLevelInternalInstitutions() == 0){
                        link = LinkStatusEnum.CHECK_1.getCode();
                    }else {
                        link = LinkStatusEnum.CHECK_3.getCode();
                    }
                }else{
                    if (entity.getMunicipalLevelInternalInstitutions() == 0){
                        link = LinkStatusEnum.CHECK_2.getCode();
                    }else {
                        link = LinkStatusEnum.CHECK_4.getCode();
                    }
                }
            }
            RightsProtectionFlowOrgDTO flow = jdbcDao.findMaxSortByRightsIdAndLink(entity.getId(),link);
            //根据市级内设机构 和 systemState 查询提交的第一个环节的should和next
            //再判断分别是哪个主管单位和团组织
            if (entity.getSystemState() == 0){
                entity.setManagerOrgId(flow.getShouldOrgId());
                entity.setManagerOrgName(flow.getShouldOrg());
                entity.setOrganizationId(flow.getNextOrgId());
                entity.setOrganizationName(flow.getNextOrg());
            }else{
                entity.setOrganizationId(flow.getShouldOrgId());
                entity.setOrganizationName(flow.getShouldOrg());
            }
            //把初始附件设置到当前的草稿中
            List<YcqQzhRightsProtectionAnnex> annexes = annexRepo.findAllByFlowIdAndDelFlag(flow.getId(), 0);
            List<YcqQzhRightsProtectionAnnex> annexeNews = new ArrayList<>();
            for (YcqQzhRightsProtectionAnnex annex : annexes) {
                YcqQzhRightsProtectionAnnex annexNew = new YcqQzhRightsProtectionAnnex();
                BeanUtils.copyProperties(annex,annexNew);
                annexNew.setId(snowflake.nextId());
                annexNew.setFlowId(flowId);
                annexeNews.add(annexNew);
            }
            shouldId = entity.getApplicantOrgId();
            nextId = entity.getApplicantOrgId();
            annexRepo.saveAll(annexeNews);
        }
        saveFlow(proId, shouldId, nextId, entity.getId(), flowId, entity.getLink(), opinion, req.getReason(), 1, worker);
        repo.save(entity);
        return Result.ofSuccess("操作完成！");
    }

    @Override
    public Result<RightsProtectionAnnexDTO> getAnnex(Long id) {
        YcqQzhRightsFlow flow1 = jdbcDao.findMaxSortByRightsId(id);
        if (null == flow1) {
            return Result.ofFail("未查到相关数据");
        }
        YcqQzhRightsProtectionAnnex annex = annexRepo.findAllByFlowIdAndAnnexTypeAndDelFlag(flow1.getId(), 1,0);
        if (null == annex) {
            return Result.ofFail("未查到相关附件");
        }
        RightsProtectionAnnexDTO dto = new RightsProtectionAnnexDTO();
        BeanUtils.copyProperties(annex, dto);
        return new Result<>(dto);
    }

    @Override
    public Result<List<RightsProtectionFlowDTO>> getFlows(Long id) {
        List<RightsProtectionFlowDTO> list = jdbcDao.getFlows(id);
        return new Result<>(list);
    }

    @Override
    public Result<PageRsp<RightsProtectionInventoryPageRespDTO>> inventoryPage(PageReq<RightsProtectionInventoryPageReqDTO> req) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        QueryOrgDto dto = new QueryOrgDto();
        dto.setReportType(3);
        dto.setOrgId(worker.getOrgId());
        List<Long> ids = civilizationReportService.queryServiceIds(dto);
        //查询自己需要处理的id是不是在ids里面，是的话需要从ids中排除掉
        List<YcqQzhRightsProtection> workBus = jdbcDao.findIdByWork(Long.valueOf(worker.getOrgId()));
        if (CollectionUtils.isNotEmpty(workBus)){
            List<Long> workBusIds = workBus.stream()
                    .map(YcqQzhRightsProtection::getId).collect(Collectors.toList());
            ids = ids.stream().filter(o -> !workBusIds.contains(o)).collect(Collectors.toList());
        }

        String workerOrgId = worker.getOrgId();
        PageRsp<RightsProtectionInventoryPageRespDTO> resp = jdbcDao.inventoryPageList(req, workerOrgId, validatedSuper(worker),ids);
        return new Result<PageRsp<RightsProtectionInventoryPageRespDTO>>(resp);
    }

    @Override
    public Result<YcqQzhRightsProtection> saveDraft(RightsProtectionCreateReqDTO req) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        // 提交为草稿，草稿不需要进入流程不调用doSystem方法
        Snowflake snowflake = new Snowflake();
        Long workId = Long.valueOf(worker.getLoginId());
        YcqQzhRightsProtection entity;
        LocalDateTime now = LocalDateTime.now();


        validateDraft(req, worker);
        //新增草稿
        if (Objects.isNull(req) || req.getId() == null) {
            long flowId = snowflake.nextId();
            entity = RightsProtectionConvert.INSTANCE.convert(req);
            if (Objects.isNull(entity)) {
                entity = new YcqQzhRightsProtection();
            }
            entity.setApplicantCode(CodeUtil.codeRule(1));
            entity.setApplicantId(workId);
            entity.setApplicantOrgId(Long.valueOf(worker.getOrgId()));
            entity.setApplyType(TypeEnum.APPLYTYPE_0.getType());
            entity.setStatus(LinkStatusEnum.CREATE_0.getCode());
            entity.setLink(LinkStatusEnum.CREATE_0.getCode());
            entity.setCreateId(workId);
            entity.setCreateBy(worker.getWorkerName());
            entity.setApplyTime(now);
            entity.setCreateTime(now);
            entity.setDelFlag(0);
            Long entityId = snowflake.nextId();
            saveFlow(Long.valueOf(worker.getOrgId()), Long.valueOf(worker.getOrgId()), Long.valueOf(worker.getOrgId()), entityId, flowId,
                    LinkStatusEnum.CREATE_0.getCode(), null, null, 0, worker);
            saveAnnexs(req, entity, worker, flowId, 0);
            entity.setId(entityId);
            repo.save(entity);
        } else {
            //修改为草稿
            Long rightsProtectionId = req.getId();
            entity = repo.findById(rightsProtectionId).get();
            BeanUtils.copyProperties(req, entity);
            entity.setStatus(LinkStatusEnum.CREATE_0.getCode());
            entity.setUpdateId(workId);
            entity.setUpdateBy(worker.getWorkerName());
            repo.save(entity);
            // 处理附件
            YcqQzhRightsFlow flow1 = jdbcDao.findMaxSortByRightsId(rightsProtectionId);
            saveAnnexs(req, entity, worker, flow1.getId(),0);
        }

        return new Result<>(entity);
    }

    private void validateDraft(RightsProtectionCreateReqDTO req, Worker worker) {
        if (StringUtils.isNotEmpty(req.getUnitName())) {
            List<YcqQzhRightsProtection> unitEns = repo.findAllByUnitNameAndApplyTypeAndDelFlag(req.getUnitName(), 0, 0);
            if (CollectionUtils.isNotEmpty(unitEns)) {
                throw new BusinessException("此单位已申报或正在申报中！");
            }
        }

        if (StringUtils.isNotEmpty(req.getUnitName()) && req.getUnitName().length() > 50) {
            throw new BusinessException("单位名称输入超长，不超过50字！");
        }
        if (StringUtils.isNotEmpty(req.getPhone())&&req.getPhone().length() != 11) {
            throw new BusinessException("联系电话格式不对！");
        }
        if (StringUtils.isNotEmpty(req.getAddress())&&req.getAddress().length() > 100) {
            throw new BusinessException("通讯地址超长，不超过100字！");

        }
        if (StringUtils.isNotEmpty(req.getZipCode())&&req.getZipCode().length() != 6) {
            throw new BusinessException("邮编格式不对！");
        }

        if (StringUtils.isNotEmpty(req.getCreateProTime())
                && req.getCreateProTime().length()>50) {
            throw new BusinessException("何时命名权益岗超长，不超过50字！");
        }

        if (StringUtils.isNotEmpty(req.getProgramme())&&req.getProgramme().length() > 2000) {
            throw new BusinessException("创建方案超长，不超过2000字！");
        }

        if (Objects.nonNull(req.getOtherAnnex())
                && req.getOtherAnnex().size() > 5) {
            throw new BusinessException("请上传最多5个附件！");
        }

    }

    @Override
    public Result<Boolean> saveCreate(RightsProtectionCreateReqDTO req) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
//        Worker worker = CurrWorkerUtil.getCurrWorker();
        Snowflake snowflake = new Snowflake();
        Long workId = Long.valueOf(worker.getLoginId());
        long flowId = snowflake.nextId();
        //提交为创建
        //新增为创建
        // 数据和权限校验
        validate(req, worker);
        YcqQzhRightsProtection entity;
        if (req.getId() == null) {
            throw new BusinessException("请先保存为草稿，打印生成的文件再上传提交");
//			entity = RightsProtectionConvert.INSTANCE.convert(req);
//			entity.setApplicantCode(CodeUtil.codeRule(1));
//			entity.setCreateId(workId);
//			entity.setCreateBy(worker.getWorkerName());
//			entity.setLink(LinkStatusEnum.CREATE_1.getCode());
//			entity.setStatus(LinkStatusEnum.CREATE_1.getCode());
//			Long entityId = snowflake.nextId();
//			saveAnnexs(req, entity, worker, flowId);
//			entity.setId(entityId);
//			repo.save(entity);
        } else {
            //草稿提交为创建
            Long rightsProtectionId = req.getId();
            entity = repo.findById(rightsProtectionId).get();
            List<YcqQzhRightsProtection>  unitEns = repo.findAllByUnitNameAndApplyTypeAndDelFlag(req.getUnitName(),0,0);
            if(CollectionUtils.isNotEmpty(unitEns)){
                throw new BusinessException("此单位已申报或正在申报中！");
            }
            BeanUtils.copyProperties(req, entity);
            entity.setStatus(LinkStatusEnum.CREATE_1.getCode());
            entity.setLink(LinkStatusEnum.CREATE_1.getCode());
            entity.setApplyTime(LocalDateTime.now());
            entity.setUpdateId(workId);
            entity.setUpdateBy(worker.getWorkerName());
            RightsProtectionAnnexDTO scanAnnex = req.getScanAnnex();
            scanAnnex.setFileNewName(entity.getApplicantCode() + "-申报创建-" + worker.getOrgName());
            log.info("用户组织名：----------------------" + worker.getOrgName());
            saveAnnexs(req, entity, worker, flowId, 1);
        }

        //  查询静态表中的值 根据value_id 查询 value_desc是不是以1开头
        String b = jdbcDao.getDescById(req.getSystemId());

        boolean code = false;
        if (StringUtils.isNotEmpty(b)) {
            code = b.startsWith("1");
        }
        Long orgId = Long.valueOf(worker.getOrgId());
        Long shouldId = null;
        Long nextId = null;
        //确定下一步流程
        if (code) {
            if (req.getMunicipalLevelInternalInstitutions().equals(TypeEnum.TYPE_1.getType())) {
                entity.setStatus(LinkStatusEnum.CREATE_4.getCode());
                entity.setLink(LinkStatusEnum.CREATE_4.getCode());
                // TODO 2023年11月15日16:09:01 组织打标
                shouldId = req.getOrganizationId();
				nextId = Long.valueOf(worker.getOrgId());
            } else {
                entity.setStatus(LinkStatusEnum.CREATE_2.getCode());
                entity.setLink(LinkStatusEnum.CREATE_2.getCode());
                shouldId = req.getOrganizationId();
                nextId = req.getManagerOrgId();
            }
            entity.setSystemState(1);
        } else {
            if (req.getMunicipalLevelInternalInstitutions().equals(TypeEnum.TYPE_1.getType())) {
                entity.setStatus(LinkStatusEnum.CREATE_3.getCode());
                entity.setLink(LinkStatusEnum.CREATE_3.getCode());

            } else {
                entity.setStatus(LinkStatusEnum.CREATE_1.getCode());
                entity.setLink(LinkStatusEnum.CREATE_1.getCode());
            }
			// 第一次查询单位区县级org_id
			shouldId = req.getManagerOrgId();
			// 第二次查询团组织区县级org_id
			nextId = req.getOrganizationId();
			entity.setSystemState(0);
        }
        //添加可以查看的组织权限
        ReportOrgDto dto = new ReportOrgDto();
        dto.setReportType(3);
        dto.setServiceId(entity.getId());
        dto.setContainCurrentOrg(true);
        List<Long> orgs = new ArrayList<>();
        orgs.add(Long.valueOf(worker.getOrgId()));
        if(Objects.nonNull(req.getManagerOrgId())){
            orgs.add(req.getManagerOrgId());
        }
        orgs.add(req.getOrganizationId());
        dto.setOrgIds(orgs);
        civilizationReportService.saveQueryOrg(dto,worker);
        entity.setApplyTime(LocalDateTime.now());
        saveFlow(orgId, shouldId, nextId, req.getId(), flowId, entity.getLink(), null, null, 1, worker);
        repo.save(entity);

        return new Result<>(true);
    }

    @Override
    public Result<RightsProtectionCreateQueryRespDTO> getCreate(Long id) {
        RightsProtectionCreateQueryRespDTO dto = jdbcDao.getCreate(id);

        List<YcqQzhRightsProtectionAnnex> type = annexRepo.findAllByFlowIdAndDelFlag(dto.getFlowId(),0);
        List<YcqQzhRightsProtectionAnnex> others = type.stream().filter(o -> o.getAnnexType() == 0).collect(Collectors.toList());
        List<RightsProtectionAnnexDTO> list = new ArrayList<>();
        for (YcqQzhRightsProtectionAnnex other : others) {
            RightsProtectionAnnexDTO dto1 = new RightsProtectionAnnexDTO();
            BeanUtils.copyProperties(other, dto1);
            list.add(dto1);
        }
        if (CollectionUtils.isNotEmpty(list)) {
            dto.setOtherAnnex(list);
        }

        List<YcqQzhRightsProtectionAnnex> annex = type.stream().filter(o -> o.getAnnexType() == 1).collect(Collectors.toList());
        if (annex.size() == 1) {
            RightsProtectionAnnexDTO dto1 = new RightsProtectionAnnexDTO();
            BeanUtils.copyProperties(annex.get(0), dto1);
            dto.setScanAnnex(dto1);
        } else if (annex.size() == 0) {
            //不处理
        } else {
            throw new BusinessException("数据异常");
        }

        return new Result<RightsProtectionCreateQueryRespDTO>(dto);
    }

    @Override
    public Result<Boolean> delete(Long id) {
        YcqQzhRightsProtection protection = repo.findById(id).get();
        if (Objects.isNull(protection)) {
            throw new BusinessException("未找到数据");
        } else {
        	if (!LinkStatusEnum.CREATE_0.getCode().equals(protection.getLink())
			&& !LinkStatusEnum.CHECK_0.getCode().equals(protection.getLink())){
				throw new BusinessException("已在流程中，不能删除！");
			}
            protection.setDelFlag(1);
            repo.save(protection);
        }
        return new Result<>(true);
    }

    @Override
    public Result<Boolean> update(RightsProtectionUpdateReqDTO req) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        YcqQzhRightsProtection entity = repo.findByIdAndDelFlag(req.getId(), 0);
        List<YcqQzhRightsProtectionUpdate> list = new ArrayList<>();
        Snowflake snowflake = new Snowflake();
        if (!entity.getPhone().equals(req.getPhone())) {
            YcqQzhRightsProtectionUpdate en = new YcqQzhRightsProtectionUpdate();
            en.setOldData(entity.getPhone());
            en.setNewData(req.getPhone());
            entity.setPhone(req.getPhone());
            en.setId(snowflake.nextId());
            en.setRightsId(entity.getId());
            en.setUpdateType("修改联系电话");
            en.setUpdateBy(Long.valueOf(worker.workerId()));
            en.setUpdateTime(LocalDateTime.now());
            list.add(en);
        }
        if (!entity.getZipCode().equals(req.getZipCode())) {
            YcqQzhRightsProtectionUpdate en = new YcqQzhRightsProtectionUpdate();
            en.setOldData(entity.getZipCode());
            en.setNewData(req.getZipCode());
            entity.setZipCode(req.getZipCode());
            en.setId(snowflake.nextId());
            en.setRightsId(entity.getId());
            en.setUpdateType("修改邮编");
            en.setUpdateBy(Long.valueOf(worker.workerId()));
            en.setUpdateTime(LocalDateTime.now());
            list.add(en);
        }
        if (!entity.getAddress().equals(req.getAddress())) {
            YcqQzhRightsProtectionUpdate en = new YcqQzhRightsProtectionUpdate();
            en.setOldData(entity.getAddress());
            en.setNewData(req.getAddress());
            entity.setAddress(req.getAddress());
            en.setId(snowflake.nextId());
            en.setRightsId(entity.getId());
            en.setUpdateType("修改通讯地址");
            en.setUpdateBy(Long.valueOf(worker.workerId()));
            en.setUpdateTime(LocalDateTime.now());
            list.add(en);
        }
        repo.save(entity);
        upadteRepo.saveAll(list);
        return new Result<>(true);
    }

    @Override
    public Result<Object> doOperate(RightsProtectionOperateReqDTO req) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        YcqQzhRightsProtection entity = repo.findByIdAndDelFlag(req.getId(), 0);
//		if (!entity.getLink().equals(req.getButton())) {
//			throw new BusinessException("数据异常！");
//		}
        Long proId = Long.valueOf(worker.getOrgId());
        Long shouldId = null;
        Long nextId = null;
        Integer opinion = req.getOpinion();
        YcqQzhRightsFlow flow1 = jdbcDao.findMaxSortByRightsId(entity.getId());
        //判断组织
        validatedOrgAudit(worker, flow1);
        //角色判断
        if (LinkStatusEnum.CREATE_1.getCode().equals(entity.getLink())) {
            validatedRoleAudit(worker, 0);
        } else if (LinkStatusEnum.CREATE_2.getCode().equals(entity.getLink())) {
            validatedRoleAudit(worker, 1);
        } else if (LinkStatusEnum.CREATE_3.getCode().equals(entity.getLink())) {
            validatedRoleAudit(worker, 2);
        } else if (LinkStatusEnum.CREATE_4.getCode().equals(entity.getLink()) || LinkStatusEnum.CREATE_5.getCode().equals(entity.getLink())) {
            validatedRoleAudit(worker, 3);
        }
        if ((opinion == 0 || opinion == 1 || opinion == 2) && Objects.isNull(req.getScanAnnex())) {
            throw new BusinessException("通过必须上传盖章版扫描件");
        }
        String reason = req.getReason();
        if(StringUtils.isNotEmpty(reason) && reason.length()>80){
            throw new BusinessException("备注超长！请控制在80字以内！");
        }
        Snowflake snowflake = new Snowflake();
        long flowId = snowflake.nextId();
        String linkStr = "";
        //0通过，1优秀，2合格，3不通过，4，整改 5撤销命名 6 命名自动取消 7退回
        YcqQzhRightsProtectionAnnex annexEntity = null;
        if (opinion == 0 || opinion == 1 || opinion == 2) {
            if (Objects.isNull(req.getScanAnnex())) {
                throw new BusinessException("通过审核必须上传盖章版扫描件");
            }
            //已通过审核，修改新增审核流程，主表，附件表
            entity.setOpinion(opinion);
            entity.setReason(reason);
            entity.setUpdateBy(worker.getWorkerName());
            entity.setUpdateId(Long.valueOf(worker.getLoginId()));
            annexEntity = RightsProtectionConvert.INSTANCE.convert(req.getScanAnnex());
            annexEntity.setId(snowflake.nextId());
            annexEntity.setApplicantCode(entity.getApplicantCode());
            annexEntity.setAnnexType(TypeEnum.ANNEXTYPE_1.getType());
            annexEntity.setFlowId(flowId);
            annexEntity.setDelFlag(0);
            //确定下一步流程
            //区县级主管单位初评
            if (LinkStatusEnum.CREATE_1.getCode().equals(entity.getLink())) {
                entity.setStatus(LinkStatusEnum.CREATE_2.getCode());
                entity.setLink(LinkStatusEnum.CREATE_2.getCode());
                entity.setManagerOrgId(req.getNextOrgId());
                entity.setManagerOrgName(req.getNextOrgName());
                linkStr = LinkStatusEnum.CREATE_1.getLink();
                shouldId = entity.getOrganizationId();
                nextId = req.getNextOrgId();
            } else if (LinkStatusEnum.CREATE_2.getCode().equals(entity.getLink())) {
                entity.setOrganizationId(req.getNextOrgId());
                entity.setOrganizationName(req.getNextOrgName());
                if (TypeEnum.TYPE_0.getType().equals(entity.getSystemState())) {
                    entity.setStatus(LinkStatusEnum.CREATE_3.getCode());
                    entity.setLink(LinkStatusEnum.CREATE_3.getCode());
                    shouldId = entity.getManagerOrgId();
                    nextId = req.getNextOrgId();
                } else {
                    entity.setStatus(LinkStatusEnum.CREATE_4.getCode());
                    entity.setLink(LinkStatusEnum.CREATE_4.getCode());
                    shouldId = req.getNextOrgId();
                    nextId = entity.getOrganizationId();
                }
                linkStr = LinkStatusEnum.CREATE_2.getLink();
            } else if (LinkStatusEnum.CREATE_3.getCode().equals(entity.getLink())) {
                entity.setStatus(LinkStatusEnum.CREATE_4.getCode());
                entity.setLink(LinkStatusEnum.CREATE_4.getCode());
                linkStr = LinkStatusEnum.CREATE_3.getLink();
                //市级团委初核的id
                shouldId = entity.getOrganizationId();
                //市级团委公示
                nextId = entity.getOrganizationId();
                //市级团委初核
            } else if (LinkStatusEnum.CREATE_4.getCode().equals(entity.getLink())) {
                entity.setStatus(LinkStatusEnum.CREATE_5.getCode());
                entity.setLink(LinkStatusEnum.CREATE_5.getCode());
                shouldId = entity.getOrganizationId();
                nextId = entity.getApplicantOrgId();
                linkStr = LinkStatusEnum.CREATE_4.getLink();
            } else if (LinkStatusEnum.CREATE_5.getCode().equals(entity.getLink())) {
                entity.setLink(LinkStatusEnum.CREATE_6.getCode());
                entity.setStatus(LinkStatusEnum.CREATE_6.getCode());
                shouldId = entity.getApplicantOrgId();
                nextId = entity.getApplicantOrgId();
            }
            //添加可以查看的权限
            ReportOrgDto dto = new ReportOrgDto();
            dto.setReportType(3);
            dto.setServiceId(entity.getId());
            dto.setContainCurrentOrg(true);
            List<Long> orgs = new ArrayList<>();
            orgs.add(Long.valueOf(worker.getOrgId()));
            orgs.add(shouldId);
            orgs.add(nextId);
            dto.setOrgIds(orgs);
            civilizationReportService.saveQueryOrg(dto,worker);

            log.info("-------------------------" + worker.getOrgName());
            annexEntity.setFileNewName(entity.getApplicantCode() + "-" + linkStr + "-" + worker.getOrgName());
            annexRepo.save(annexEntity);
        } else if (opinion == 3 || opinion == 4 || opinion == 5 || opinion == 6) {
            //直接结束
            entity.setStatus(LinkStatusEnum.END.getCode());
            entity.setLink(LinkStatusEnum.END.getCode());
            shouldId = entity.getApplicantOrgId();
            nextId = entity.getApplicantOrgId();
            if(Objects.nonNull(req.getScanAnnex())){
                annexEntity = RightsProtectionConvert.INSTANCE.convert(req.getScanAnnex());
                annexEntity.setId(snowflake.nextId());
                annexEntity.setApplicantCode(entity.getApplicantCode());
                annexEntity.setAnnexType(TypeEnum.ANNEXTYPE_1.getType());
                annexEntity.setFlowId(flowId);
                annexEntity.setDelFlag(0);
                annexEntity.setFileNewName(entity.getApplicantCode() + "-" + linkStr + "-" + worker.getOrgName());
                annexRepo.save(annexEntity);
            }
        } else {
            throw new BusinessException("类型不存在");
        }
        saveFlow(proId, shouldId, nextId, entity.getId(), flowId, entity.getLink(), opinion, req.getReason(), 1, worker);
        repo.save(entity);
        return Result.ofSuccess("审核成功！");
    }


    @Override
    public Result<Boolean> doPublicity(RightsProtectionPublicityReqDTO req) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        Integer opinion = req.getOpinion();
        YcqQzhRightsProtection entity = repo.findByIdAndDelFlag(req.getId(), 0);
        YcqQzhRightsFlow flow1 = jdbcDao.findMaxSortByRightsId(req.getId());
        validatedRoleAudit(worker, 3);
        validatedOrgAudit(worker, flow1);
        Snowflake snowflake = new Snowflake();
        long flowId = snowflake.nextId();

        if (opinion == 0) {
            entity.setLink(LinkStatusEnum.CREATE_6.getCode());
            entity.setStatus(LinkStatusEnum.CREATE_6.getCode());
        } else if (opinion == 3) {
            //直接结束
            entity.setStatus(LinkStatusEnum.END.getCode());
            entity.setLink(LinkStatusEnum.END.getCode());
        } else {
            throw new BusinessException("该方式不存在");
        }
        saveFlow(Long.valueOf(worker.getOrgId()), entity.getApplicantOrgId(), entity.getApplicantOrgId(), entity.getId(), flowId, entity.getLink(),
                opinion, req.getPublicityNotes(), 1, worker);
        return new Result<>(true);
    }

    @Override
    public Result<Boolean> addCreate(AddCreateDTO dto) {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        YcqQzhRightsProtection entity = RightsProtectionConvert.INSTANCE.convert(dto);
        entity.setApplicantCode(CodeUtil.codeRule(1));
        entity.setApplicantId(Long.valueOf(worker.getLoginId()));
        entity.setApplyType(TypeEnum.APPLYTYPE_0.getType());
        if (Objects.isNull(dto.getId())) {
            entity.setCreateId(Long.valueOf(worker.getLoginId()));
            entity.setCreateBy(worker.getWorkerName());
        } else {
            entity.setUpdateId(Long.valueOf(worker.getLoginId()));
            entity.setUpdateBy(worker.getWorkerName());
        }
        //存入文件，1创建盖章版扫描件，2

        repo.save(entity);
        return new Result<>(true);
    }

    @Override
    public Result<List<DocumentListRspDTO>> queryPolicy(Integer type) {
        List<DocumentListRspDTO> list = ycqQzhDocumentJdbcDao.queryListForTypeAndState(4, type, 1);
        return Result.ofSuccess(list);
    }

    private void saveAnnexs(RightsProtectionCreateReqDTO rightsProtectionCreateReqDTO, YcqQzhRightsProtection entity, Worker worker, Long flowId, int isDraf) {
        ArrayList<YcqQzhRightsProtectionAnnex> annexes = new ArrayList<>();
        Snowflake snowflake = new Snowflake();
        if (isDraf == 0){
            List<YcqQzhRightsProtectionAnnex> allByFlowId = annexRepo.findAllByFlowIdAndDelFlag(flowId,0);
            if (CollectionUtils.isNotEmpty(allByFlowId)){
                allByFlowId.forEach(o -> o.setDelFlag(1));
                annexes.addAll(allByFlowId);
            }
        }
        if (Objects.nonNull(rightsProtectionCreateReqDTO)) {
            //盖章版扫描件
            if (Objects.nonNull(rightsProtectionCreateReqDTO.getScanAnnex())) {
                YcqQzhRightsProtectionAnnex scanAnnex = RightsProtectionConvert.INSTANCE.convert(rightsProtectionCreateReqDTO.getScanAnnex());
                scanAnnex.setId(snowflake.nextId());
                scanAnnex.setApplicantCode(entity.getApplicantCode());
                scanAnnex.setFlowId(flowId);
                scanAnnex.setBusinessId(rightsProtectionCreateReqDTO.getId());
                scanAnnex.setCreateBy(worker.getWorkerName());
                scanAnnex.setCreateId(Long.valueOf(worker.getLoginId()));
                scanAnnex.setCreateTime(LocalDateTime.now());
                scanAnnex.setAnnexType(TypeEnum.ANNEXTYPE_1.getType());
                annexes.add(scanAnnex);
            }
            //其他附件
            if (CollectionUtils.isNotEmpty(rightsProtectionCreateReqDTO.getOtherAnnex())) {
                for (RightsProtectionAnnexDTO annex : rightsProtectionCreateReqDTO.getOtherAnnex()) {
                    YcqQzhRightsProtectionAnnex annexEntity = RightsProtectionConvert.INSTANCE.convert(annex);
                    annexEntity.setId(snowflake.nextId());
                    annexEntity.setApplicantCode(entity.getApplicantCode());
                    annexEntity.setFlowId(flowId);
                    annexEntity.setBusinessId(rightsProtectionCreateReqDTO.getId());
                    annexEntity.setCreateBy(worker.getWorkerName());
                    annexEntity.setCreateId(Long.valueOf(worker.getLoginId()));
                    annexEntity.setCreateTime(LocalDateTime.now());
                    annexEntity.setAnnexType(TypeEnum.ANNEXTYPE_0.getType());
                    annexes.add(annexEntity);
                }
            }
        }
        if (CollectionUtils.isNotEmpty(annexes)) {
            annexRepo.saveAllAndFlush(annexes);
        }
    }

    private void saveFlow(Long processedOrgId, Long shouldId, Long nextId, Long rightsId, Long flowId, String link,
                          Integer option, String reason, int isDraf, Worker work) {
        // 根据选择条件查询第一次和第二次审核的组织，存入审核流程表
        YcqQzhRightsFlow flow = new YcqQzhRightsFlow();
        flow.setId(flowId);
        flow.setRightsId(rightsId);
        flow.setProcessedOrgId(processedOrgId);//所填单位的组织id
        flow.setProcessedWork(work.getWorkerName());
        flow.setProcessedDate(LocalDateTime.now());
        flow.setShouldOrgId(shouldId);
        flow.setNextOrgId(nextId);
        flow.setLink(link);
        if (LinkStatusEnum.CREATE_6.getCode().equals(link)
                || LinkStatusEnum.END.getCode().equals(link)) {
            flow.setStatus(1);
        } else if (LinkStatusEnum.CHECK_5.getCode().equals(link)
                || LinkStatusEnum.END.getCode().equals(link)) {
            flow.setStatus(1);
        } else {
            flow.setStatus(0);
        }
        flow.setIsDraft(isDraf);
        List<YcqQzhRightsFlow> ycqQzhRightsFlows = new ArrayList<>();
        YcqQzhRightsFlow flow1 = jdbcDao.findMaxSortByRightsId(rightsId);
        if (Objects.nonNull(flow1)) {
            flow1.setStatus(1);
            flow1.setResult(option);
            flow1.setMsg(reason);
            flow.setSort(flow1.getSort() + 1);
            flow.setParentId(flow1.getId());
            ycqQzhRightsFlows.add(flow1);
        } else {
            flow.setSort(0);
        }
        ycqQzhRightsFlows.add(flow);
        ycqQzhRightsFlowRepo.saveAllAndFlush(ycqQzhRightsFlows);
    }

    /**
     * 校验~数据
     */
    private void validate(RightsProtectionCreateReqDTO rightsProtectionCreateReqDTO, Worker worker) {
        if (StringUtils.isEmpty(rightsProtectionCreateReqDTO.getUnitName())) {
            throw new BusinessException("单位名称不能为空！");
        } else if (rightsProtectionCreateReqDTO.getUnitName().length() > 50) {
            throw new BusinessException("单位名称输入超长，不超过50字！");
        }
        if (Objects.isNull(rightsProtectionCreateReqDTO.getSystemId())) {
            throw new BusinessException("所属系统不能为空！");
        }
        if (Objects.isNull(rightsProtectionCreateReqDTO.getOrganizationId())) {
            throw new BusinessException("所属团组织不能为空！");
        }
        String b = jdbcDao.getDescById(rightsProtectionCreateReqDTO.getSystemId());
        boolean code = false;
        if (StringUtils.isNotEmpty(b)) {
            code = b.startsWith("1");
        }

        if (!code && Objects.isNull(rightsProtectionCreateReqDTO.getManagerOrgId())) {
            throw new BusinessException("所属主管单位不能为空！");
        }

        if (StringUtils.isEmpty(rightsProtectionCreateReqDTO.getPhone())) {
            throw new BusinessException("联系电话不能为空！");
        } else if (rightsProtectionCreateReqDTO.getPhone().length() != 11) {
            throw new BusinessException("联系电话格式不对！");
        }
        if (StringUtils.isEmpty(rightsProtectionCreateReqDTO.getAddress())) {
            throw new BusinessException("通讯地址不能为空！");
        } else if (rightsProtectionCreateReqDTO.getAddress().length() > 100) {
            throw new BusinessException("通讯地址超长，不超过100字！");

        }
        if (StringUtils.isEmpty(rightsProtectionCreateReqDTO.getZipCode())) {
            throw new BusinessException("邮编不能为空！");
        } else if (rightsProtectionCreateReqDTO.getZipCode().length() != 6) {
            throw new BusinessException("邮编格式不对！");
        }
        if (StringUtils.isNotEmpty(rightsProtectionCreateReqDTO.getCreateProTime())
        && rightsProtectionCreateReqDTO.getCreateProTime().length()>50) {
            throw new BusinessException("何时命名权益岗超长，不超过50字！");
        }
        if (StringUtils.isEmpty(rightsProtectionCreateReqDTO.getProgramme()) ||
        "<p><br></p>".equals(rightsProtectionCreateReqDTO.getProgramme())) {
            throw new BusinessException("创建方案不能为空！");
        } else if (rightsProtectionCreateReqDTO.getZipCode().length() > 2000) {
            throw new BusinessException("创建方案超长，不超过2000字！");
        }

        if (Objects.isNull(rightsProtectionCreateReqDTO.getScanAnnex())) {
            throw new BusinessException("请上传盖章版扫描件！");
        }

        if (Objects.nonNull(rightsProtectionCreateReqDTO.getOtherAnnex())
                && rightsProtectionCreateReqDTO.getOtherAnnex().size() > 5) {
            throw new BusinessException("请上传最多5个附件！");
        }
    }

    /**
     * @Description: 启动工作流
     * @Param: []
     * @return: void
     * @Author: xiang xin
     * @Date: 2023/11/20
     */
    private Integer firingWorkerFlow(Worker worker, RightsProtectionCreateReqDTO req, YcqQzhRightsProtection entity) {
        //启动工作流
        CommonResult<FlowConfRespVO> result = flowInstanceApi.getFlowConfInfoByFlowCode("babf25d7e299489183063bc4a2ff80a7");
        FlowConfRespVO data = result.getData();
        String deployId = data.getFlowDeployId();
        String moduleId = data.getFlowModuleId();
        //流程信息
        FlowStartDTO dto = new FlowStartDTO();
        dto.setFlowModuleId(moduleId);
        dto.setFlowDeployId(deployId);
        //创建信息
        FlowStarterInfoDTO starter = new FlowStarterInfoDTO();
        starter.setOrgId(Long.parseLong(worker.getOrgId()));
        starter.setOrgName(worker.getOrgName());
        starter.setUserId(Long.parseLong(worker.getLoginId() + ""));
        starter.setUserName(worker.getWorkerName());
        dto.setStarter(starter);
        //下一步处理组织
        FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
        ArrayList<IdNameDTO> orgs = new ArrayList<>();
        IdNameDTO idNameDTO = new IdNameDTO();
        idNameDTO.setId(Long.parseLong(worker.getOrgId()));
        idNameDTO.setName(worker.getOrgName());
        orgs.add(idNameDTO);
        addedTaskAssigenn.setAssigennDept(orgs);
        dto.setAddedTaskAssigenn(addedTaskAssigenn);
        String flowInstanceId;
        String nodeInstanceId;
        CommonResult<CruniiStartProcessResult> startResult;
        //设置流程变量后面判断
        List<InstanceData> variables = new ArrayList<>();
        InstanceData instanceData = new InstanceData();
        instanceData.setKey("isDraft");
        instanceData.setValue(req.getIsDraft());
        variables.add(instanceData);
        dto.setVariables(variables);
        try {
            startResult = flowInstanceApi.startProcess(dto);
            CruniiStartProcessResult startData = startResult.getData();
            flowInstanceId = startData.getFlowInstanceId();
            nodeInstanceId = startData.getActiveTaskInstance().getNodeInstanceId();
        } catch (Exception e) {
            return 0;
        }
        if (startResult.getCode() == 00 && startResult.getData().getErrCode() < 2000) {
        } else {
            return 0;
        }
        entity.setFlowInstanceId(flowInstanceId);
        entity.setNodeInstanceId(nodeInstanceId);
        return 1;
    }

    /**
     * @Description: 草稿->创建
     * @Param:
     * @return:
     * @Author: xiang xin
     * @Date: 2023/11/20
     */
    private Integer CraeteWorkerFlow(Worker worker, RightsProtectionCreateReqDTO req, YcqQzhRightsProtection entity) {
        //接入工作流
        FlowCommitDTO dto = new FlowCommitDTO();
        dto.setFlowInstanceId(entity.getFlowInstanceId());
        dto.setTaskInstanceId(entity.getNodeInstanceId());
        FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
        commitor.setOrgId(Long.parseLong(worker.getOrgId()));
        commitor.setOrgName(worker.getOrgName());
        commitor.setUserId(Long.parseLong(worker.getLoginId() + ""));
        commitor.setUserName(worker.getWorkerName());
        dto.setCommitor(commitor);
        //设置流程变量后面判断
        Map<String, Object> map = new HashMap<>();
        map.put("municipal", req.getMunicipalLevelInternalInstitutions());
        map.put("systemState", req.getSystemState());
        dto.setFlowVar(map);
        dto.setCommitFlow(true);
        //设置组织
        FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
        ArrayList<IdNameDTO> orgs = new ArrayList<>();
        IdNameDTO idNameDTO = new IdNameDTO();
        if (0 == req.getSystemState()) {
            //主管单位初评managerOrgId
            idNameDTO.setId((req.getManagerOrgId()));
            //todo 需要新增字段
            idNameDTO.setName("组织名称");
        } else {
            //团委初评organizationId
            idNameDTO.setId((req.getOrganizationId()));
            //todo 需要新增字段
            idNameDTO.setName("组织名称");
        }
        orgs.add(idNameDTO);
        addedTaskAssigenn.setAssigennDept(orgs);
        dto.setAddedTaskAssigenn(addedTaskAssigenn);
        CommonResult<CruniiCommitTaskResult> startResult = flowInstanceApi.commitProcess(dto);
        if (startResult.getCode() == 00 && startResult.getData().getErrCode() < 2000) {
            //todo 如果回退需要注意修改状态
            String nodeInstanceId = startResult.getData().getActiveTaskInstance().getNodeInstanceId();
            entity.setNodeInstanceId(nodeInstanceId);
        } else {
            return 0;
        }
        //审核下一步开始结束 历史节点
        String oldNodeKeys = entity.getOldNodeKeys() == null ? "" : entity.getOldNodeKeys();
        String modelKey = startResult.getData().getActiveTaskInstance().getModelKey();
        entity.setOldNodeKeys(oldNodeKeys + "," + modelKey);
        return 1;
    }

    /**
     * @Description: 创建->审核
     * @Param: [worker, req, entity]
     * @return: java.lang.Integer
     * @Author: xiang xin
     * @Date: 2023/11/20
     */
    private Integer examineWorkerFlow(Worker worker, RightsProtectionOperateReqDTO req, YcqQzhRightsProtection entity, Long shouldId, Map<String, Object> map, Integer type) {
        //工作流开始工作
        FlowCommitDTO dto = new FlowCommitDTO();
        dto.setFlowInstanceId(entity.getFlowInstanceId());
        dto.setTaskInstanceId(entity.getNodeInstanceId());
        //设置提交人
        FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
        commitor.setOrgId(Long.parseLong(worker.getOrgId()));
        commitor.setOrgName(worker.getOrgName());
        commitor.setUserId(Long.parseLong(worker.getLoginId() + ""));
        commitor.setUserName(worker.getWorkerName());
        dto.setCommitor(commitor);
        //设置通过原因 1通过 2不通过 3优秀 4合格
        if (1 == type) {
            dto.setProcessResult("通过");
        } else if (2 == type) {
            dto.setProcessResult("不通过");
        } else if (3 == type) {
            dto.setProcessResult("优秀");
        } else if (4 == type) {
            dto.setProcessResult("合格");
        }
        dto.setProcessComments(req.getReason());
        //传入判断依据
        if (!map.isEmpty()) {
            dto.setFlowVar(map);
            dto.setCommitFlow(true);
        }
        //组织数据 todo 加组织名称
        FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
        ArrayList<IdNameDTO> orgs = new ArrayList<>();
        IdNameDTO idNameDTO = new IdNameDTO();
        idNameDTO.setId(shouldId);
        idNameDTO.setName("组织名称");
        orgs.add(idNameDTO);
        addedTaskAssigenn.setAssigennDept(orgs);
        dto.setAddedTaskAssigenn(addedTaskAssigenn);
        CommonResult<CruniiCommitTaskResult> startResult = flowInstanceApi.commitProcess(dto);
        if (startResult.getCode() == 00 && startResult.getData().getErrCode() < 2000) {
            String nodeInstanceId = startResult.getData().getActiveTaskInstance().getNodeInstanceId();
            entity.setNodeInstanceId(nodeInstanceId);
        } else {
            return 0;
        }
        String oldNodeKeys = entity.getOldNodeKeys() == null ? "" : entity.getOldNodeKeys();
        String modelKey = startResult.getData().getActiveTaskInstance().getModelKey();
        entity.setOldNodeKeys(oldNodeKeys + "," + modelKey);
        return 1;
    }

    /**
     * @Description: 审核->公示
     * @Param:
     * @return:
     * @Author: xiang xin
     * @Date: 2023/11/21
     */
    private Integer publicityWorkerFlow(Worker worker, RightsProtectionPublicityReqDTO req, YcqQzhRightsProtection entity, Long shouldId) {
        //接入工作流
        FlowCommitDTO dto = new FlowCommitDTO();
        dto.setFlowInstanceId(entity.getFlowInstanceId());
        dto.setTaskInstanceId(entity.getNodeInstanceId());
        FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
        commitor.setOrgId(shouldId);
        commitor.setOrgName(worker.getOrgName());
        commitor.setUserId(Long.parseLong(worker.getLoginId() + ""));
        commitor.setUserName(worker.getWorkerName());
        dto.setCommitor(commitor);
        //设置流程变量后面判断
        Map<String, Object> map = new HashMap<>();
        map.put("opinion", req.getOpinion());
        dto.setFlowVar(map);
        dto.setCommitFlow(true);
        //组织数据 todo 加组织名称
        CommonResult<CruniiCommitTaskResult> startResult = flowInstanceApi.commitProcess(dto);
        if (startResult.getCode() == 00 && startResult.getData().getErrCode() < 2000) {
            //todo 如果回退需要注意修改状态
            String nodeInstanceId = startResult.getData().getActiveTaskInstance().getNodeInstanceId();
            entity.setNodeInstanceId(nodeInstanceId);
        } else {
            return 0;
        }
        //审核下一步开始结束 历史节点
        String oldNodeKeys = entity.getOldNodeKeys() == null ? "" : entity.getOldNodeKeys();
        String modelKey = startResult.getData().getActiveTaskInstance().getModelKey();
        entity.setOldNodeKeys(oldNodeKeys + "," + modelKey);
        return 1;
    }

    /**
     * 判断角色权限
     *
     * @param worker worker
     * @return 重庆市团委权限true，区县团委false
     */
    private boolean validatedRoleAudit(Worker worker, int code) {
        List<TsmRoleDto> roles = worker.getRoles();
        List<Integer> roleIds = roles.stream().map(TsmRoleDto::getRoleId).collect(Collectors.toList());

        if (code == QX_MAN_AUDIT) {
            // 判断是否区县级主管单位审核
            if (roleIds.contains(QX_MAN_AUDIT_ROLE)) {
                return true;
            }
        } else if (code == QX_ORG_AUDIT) {
            // 判断是否区县级团组织审核
            if (roleIds.contains(QX_ORG_AUDIT_ROLE)) {
                return true;
            }
        } else if (code == CQ_MAN_AUDIT) {
            // 判断是否市级主管单位审核
            if (roleIds.contains(CQ_MAN_AUDIT_ROLE)) {
                return true;
            }
        } else if (code == CQ_ORG_AUDIT) {
            // 判断是否市级团组织审核
            if (roleIds.contains(CQ_ORG_AUDIT_ROLE)) {
                return true;
            }
        }
        if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            return true;
        }
        throw new BusinessException("角色权限不足！");
    }

    private boolean validatedOrgAudit(Worker worker, YcqQzhRightsFlow flow1) {
        if (Long.valueOf(worker.getOrgId()).equals(flow1.getShouldOrgId())) {
            return true;
        }
        if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            return true;
        }
        throw new BusinessException("数据权限不足！");
    }

    private boolean validatedSuper(Worker worker) {
        if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            return true;
        }
        return false;
    }
}
