package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.entity.main.cavalryTrumpet.YcqQzhQueryOrg;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.repo.main.cavalryTrumpet.YcqQzhQueryOrgRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.QueryOrgDto;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.ReportOrgDto;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.FlowHistoryFilesRespVO;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.QagFlowHistoryRespVO;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.QagPageRsp;
import com.crunii.micro.service.qzh.dto.*;
import com.crunii.micro.service.qzh.jdbc.QagApplyLeaderChangeLogDao;
import com.crunii.micro.service.qzh.jdbc.YcqQzhQagApplyJdbcDao;
import com.crunii.micro.service.qzh.service.CivilizationReportService;
import com.crunii.micro.service.qzh.service.YcqQzhQagApplyService;
import com.crunii.module.flow.api.FlowInstanceApi;
import com.crunii.module.flow.api.vo.FlowInstanceReqVO;
import com.crunii.module.flow.api.vo.conf.FlowConfRespVO;
import com.crunii.module.flow.api.vo.pending.CruniiFlowProcessHistoryRespVO;
import com.crunii.module.flow.dto.*;
import com.didiglobal.turbo.engine.model.InstanceData;
import com.didiglobal.turbo.engine.result.RollbackTaskResult;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.compress.utils.Sets;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class YcqQzhQagApplyServiceImpl implements YcqQzhQagApplyService {

    private final YcqQzhQagApplyRepo repo;

    private final YcqQzhQagApplyJdbcDao jdbcDao;

    private final FlowInstanceApi flowInstanceApi;

    private final YouthSafetyModelRecommendRepo youthSafetyModelRecommendRepo;

    private final QagApplyLeaderChangeLogDao leaderChangeLogDao;

    private final QagApplyLeaderChangeLogDao qagApplyLeaderChangeLogDao;

    private final TsmMenuRepo tsmMenuRepo;

    private final TsmWorkerRepo tsmWorkerRepo;

    private final TsmOrgRepo tsmOrgRepo;

    private final YouthSafetyRecommendFileRepo youthSafetyRecommendFileRepo;

    private final static String CREATE_DECLARE = "b5ca4a69-6947-4d46-b836-ca536fa5fd37";

    private final static String FIRST_REVIEW = "958d6e24-4602-4884-b4ec-18297eecf63c";

    private final static String SECOND_REVIEW = "eb49e60d-bd0d-4f82-9338-d893d6b4ad99";

    private final static String FILL = "3f40c92b-2801-4355-bdbd-89e347bd509c";

    private final static String FIRST_CHECK = "98d5b426-e032-429e-97da-f819097b9b52";

    private final static String SECOND_CHECK = "91a932f4-e853-4a0e-8caa-6ff2da5b09e2";

    private final static String PASS = "e52e28aa-6e64-4adf-9480-e402a9d524cb";

    @Resource
    private CivilizationReportService civilizationReportService;

    @Resource
    QagApplyLeaderChangeLogRepo qagApplyLeaderChangeLogRepo;

    @Resource
    YcqQzhQueryOrgRepo ycqQzhQueryOrgRepo;

    @Override
    public Result<String> create(YcqQzhQagApplyReqDTO req) {

        Worker currWorker = CurrWorkerUtil.getCurrWorker();

        List<YcqQzhQagApply> idCardCheck =  repo.findAllByLeaderIdCardAndDelFlagAndFlowStateNotIn(req.getLeaderIdCard(),0,Arrays.asList(0,8));

        if (CollectionUtils.isNotEmpty(idCardCheck) && req.getFlowState()!=0) {
            throw new BusinessException("同一岗长身份证号只能存在一个申报中");
        }

        List<QagApplyLeaderChangeLog> logList = qagApplyLeaderChangeLogRepo.findByLeaderIdCardNewAndIsDeletedAndStateIn(req.getLeaderIdCard(),0,Lists.newArrayList(0,1,3));

        if (CollectionUtils.isNotEmpty(logList)) {
            throw new BusinessException("当前岗长已经存在于其他变更流程中");
        }

        //启动
        CommonResult<FlowConfRespVO> result = flowInstanceApi.getFlowConfInfoByFlowCode("07a23260a8e7458f83291e09708e48ca");
        FlowConfRespVO data = result.getData();
        String deployId = data.getFlowDeployId();
        String moduleId = data.getFlowModuleId();
        //流程信息
        FlowStartDTO dto = new FlowStartDTO();
        dto.setFlowModuleId(moduleId);
        dto.setFlowDeployId(deployId);
        //创建信息
        FlowStarterInfoDTO starter = new FlowStarterInfoDTO();
        starter.setOrgId(Long.parseLong(currWorker.getOrgId()));
        starter.setOrgName(currWorker.getOrgName());
        starter.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
        starter.setUserName(currWorker.getWorkerName());
        dto.setStarter(starter);
        //特殊新增
        //审批组织
        FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
        ArrayList<IdNameDTO> orgs = new ArrayList<>();
        IdNameDTO idNameDTO = new IdNameDTO();
        //组织数据
        idNameDTO.setId(Long.parseLong(currWorker.getOrgId()));
        idNameDTO.setName(currWorker.getOrgName());

        orgs.add(idNameDTO);
        addedTaskAssigenn.setAssigennDept(orgs);

        dto.setAddedTaskAssigenn(addedTaskAssigenn);
        String flowInstanceId;
        String nodeInstanceId;
        String nodeKey;
        CommonResult<CruniiStartProcessResult> startResult;

        //设置流程变量后面判断
        List<InstanceData> variables = new ArrayList<>();
        InstanceData instanceData = new InstanceData();
        instanceData.setKey("state");
        instanceData.setValue(req.getFlowState());
        variables.add(instanceData);
        dto.setVariables(variables);

        try {
            startResult = flowInstanceApi.startProcess(dto);
            CruniiStartProcessResult startData = startResult.getData();
            flowInstanceId = startData.getFlowInstanceId();
            nodeKey = startData.getActiveTaskInstance().getModelKey();
            nodeInstanceId = startData.getActiveTaskInstance().getNodeInstanceId();
        } catch (Exception e) {
            return Result.ofFail("流程启动失败");
        }
        if (startResult.getCode() == 00 && startResult.getData().getErrCode() < 2000) {

        } else {
            return Result.ofFail("流程启动失败");
        }

        //结束
        YcqQzhQagApply entity = new YcqQzhQagApply();
        BeanUtils.copyProperties(req, entity);
        entity.setCreateId(Long.valueOf(currWorker.getLoginId()));
        entity.setCreateTime(new Date());
        entity.setCreator(currWorker.getWorkerName());
        entity.setUpdator(currWorker.getWorkerName());
        entity.setUpdateTime(new Date());
        entity.setDelFlag(0);
        entity.setCode(getCode());
        entity.setLastAdvice("提交");
        entity.setFlowInstanceId(flowInstanceId);
        entity.setNodeInstanceId(nodeInstanceId);
        entity.setOldNodeKeys(nodeKey);

        //不保存草稿默认提交一次草稿
        if (req.getFlowState()!=0) {
            createDratf(entity,currWorker);
        }
        YcqQzhQagApply save = repo.save(entity);

        //添加可以查看的组织权限
        ReportOrgDto authDto = new ReportOrgDto();
        authDto.setReportType(2);
        authDto.setServiceId(entity.getId());
        authDto.setContainCurrentOrg(true);
        List<Long> authOrgs = new ArrayList<>();
        authOrgs.add(Long.valueOf(currWorker.getOrgId()));
        if(Objects.nonNull(entity.getAffiliationOrgId())){
            authOrgs.add(Long.valueOf(entity.getAffiliationOrgId()));
        }

        Long cityOrgCode = jdbcDao.getCityOrgCode();
        authOrgs.add(Long.valueOf(currWorker.getOrgId()));
        authOrgs.add(cityOrgCode);
        authDto.setOrgIds(authOrgs);
        civilizationReportService.saveQueryOrg(authDto,currWorker);
        return Result.ofSuccessMsg("操作成功");
    }

    private void createDratf(YcqQzhQagApply save,Worker currWorker) {
        //审核通过到草稿
        FlowCommitDTO draftdto = new FlowCommitDTO();
        draftdto.setFlowInstanceId(save.getFlowInstanceId());
        draftdto.setTaskInstanceId(save.getNodeInstanceId());
        //设置提交人
        FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
        commitor.setOrgId(Long.parseLong(currWorker.getOrgId()));
        commitor.setOrgName(currWorker.getOrgName());
        commitor.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
        commitor.setUserName(currWorker.getWorkerName());
        draftdto.setCommitor(commitor);
        //处理结果处理意见
        draftdto.setProcessResult("草稿提交");
        draftdto.setProcessComments("");
        //设置流程变量后面判断
        Map<String, Object> map = new HashMap<>();
        map.put("state", 1);
        draftdto.setFlowVar(map);
        draftdto.setCommitFlow(true);

        //设置组织
        //审批组织
        FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
        ArrayList<IdNameDTO> orgs = new ArrayList<>();
        IdNameDTO idNameDTO = new IdNameDTO();
        idNameDTO.setId(Long.parseLong(save.getAffiliationOrgId()));
        idNameDTO.setName(save.getAffiliationOrgName());
        orgs.add(idNameDTO);
        addedTaskAssigenn.setAssigennDept(orgs);
        draftdto.setAddedTaskAssigenn(addedTaskAssigenn);
        CommonResult<CruniiCommitTaskResult> draftResult = flowInstanceApi.commitProcess(draftdto);
        if (draftResult.getCode() == 00 && draftResult.getData().getErrCode() < 2000) {
            String nodeInstanceId = draftResult.getData().getActiveTaskInstance().getNodeInstanceId();
            save.setNodeInstanceId(nodeInstanceId);
            save.setFlowState(1);
        } else {
            throw new BusinessException("流程启动失败");
        }
        String oldNodeKeys = save.getOldNodeKeys() == null ? "" : save.getOldNodeKeys();
        String modelKey = draftResult.getData().getActiveTaskInstance().getModelKey();
        save.setOldNodeKeys(oldNodeKeys + "," + modelKey);
    }

    @Override
    public Result<YcqQzhQagApply> getDetails(Long id) {
        YcqQzhQagApply apply = repo.getById(id);
        return Result.ofSuccess(apply);
    }

    @Override
    public Result<Boolean> delete(Long id) {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        List<Long> list = new ArrayList();
        list.add(id);
        jdbcDao.deleteByIds(list, Long.valueOf(currWorker.getLoginId()));

        Optional<YcqQzhQagApply> byId = repo.findById(id);
        YcqQzhQagApply ycqQzhQagApply = byId.get();
        FlowBatchFinishDTO flowBatchFinishDTO = new FlowBatchFinishDTO();
        flowBatchFinishDTO.setFlowInstanceId(Sets.newHashSet(ycqQzhQagApply.getFlowInstanceId()));
        flowInstanceApi.batchFinish(flowBatchFinishDTO);
        return Result.ofSuccessMsg("操作成功");
    }

    @Override
    public Result<Boolean> update(YcqQzhQagApplyReqDTO req) {
        
        YcqQzhQagApply apply = repo.getById(req.getId());

        List<YcqQzhQagApply> idCardCheck =  repo.findAllByLeaderIdCardAndDelFlagAndFlowStateNotIn(req.getLeaderIdCard(),0,Arrays.asList(0,8));
        List<QagApplyLeaderChangeLog> logList = qagApplyLeaderChangeLogRepo.findByLeaderIdCardNewAndIsDeletedAndStateIn(req.getLeaderIdCard(),0,Lists.newArrayList(0,1,3));
        if (StringUtils.isNotBlank(req.getLeaderIdCard()) && !req.getLeaderIdCard().contains("*")) {
            if (CollectionUtils.isNotEmpty(logList)) {
                throw new BusinessException("当前岗长已经存在于其他变更流程中");
            }

            if (CollectionUtils.isNotEmpty(idCardCheck) && req.getFlowState()!=0) {
//                if (StrUtil.isNotBlank(req.getLeaderIdCard()) && apply.getLeaderIdCard()!=null && apply.getLeaderIdCard().equals(req.getLeaderIdCard())) {
                    throw new BusinessException("同一岗长身份证号只能存在一个申报中");
//                }
            }
        }

        if (StringUtils.isNotBlank(req.getLeaderIdCard()) && req.getLeaderIdCard().contains("*")) {
            idCardCheck =  repo.findAllByLeaderIdCardAndDelFlagAndFlowStateNotIn(apply.getLeaderIdCard(),0,Arrays.asList(0,8));
            logList = qagApplyLeaderChangeLogRepo.findByLeaderIdCardNewAndIsDeletedAndStateIn(apply.getLeaderIdCard(),0,Lists.newArrayList(0,1,3));
            if (CollectionUtils.isNotEmpty(idCardCheck) && req.getFlowState()!=0) {
                throw new BusinessException("同一岗长身份证号只能存在一个申报中");
            }

            if (CollectionUtils.isNotEmpty(logList)) {
                throw new BusinessException("当前岗长已经存在于其他变更流程中");
            }

        }

        //设置提交人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        String modelKey = "";
        if (req.getFlowState() == 1) {
            FlowCommitDTO dto = new FlowCommitDTO();
            dto.setFlowInstanceId(apply.getFlowInstanceId());
            dto.setTaskInstanceId(apply.getNodeInstanceId());

            FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
            commitor.setOrgId(Long.parseLong(currWorker.getOrgId()));
            commitor.setOrgName(currWorker.getOrgName());
            commitor.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
            commitor.setUserName(currWorker.getWorkerName());
            dto.setCommitor(commitor);
            //处理结果处理意见
            dto.setProcessResult("通过");
            dto.setProcessComments("通过");
            //设置流程变量后面判断
            Map<String, Object> map = new HashMap<>();
            map.put("state", 1);
            dto.setFlowVar(map);
            dto.setCommitFlow(true);
            //特殊新增
            //设置组织
            FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
            ArrayList<IdNameDTO> orgs = new ArrayList<>();
            IdNameDTO idNameDTO = new IdNameDTO();
            idNameDTO.setId(Long.parseLong(req.getAffiliationOrgId()));
            idNameDTO.setName(req.getAffiliationOrgName());
            orgs.add(idNameDTO);
            addedTaskAssigenn.setAssigennDept(orgs);
            dto.setAddedTaskAssigenn(addedTaskAssigenn);
            CommonResult<CruniiCommitTaskResult> startResult = flowInstanceApi.commitProcess(dto);
            if (startResult.getCode() == 00 && startResult.getData().getErrCode() < 2000) {
                String nodeInstanceId = startResult.getData().getActiveTaskInstance().getNodeInstanceId();
                apply.setNodeInstanceId(nodeInstanceId);
                apply.setFlowState(1);
                apply.setLastAdvice("提交");
                modelKey = startResult.getData().getActiveTaskInstance().getModelKey();
            } else {
                return Result.ofFail("流程执行失败");
            }
            //审核下一步开始结束
        }
        apply.setPolicyId(req.getPolicyId());
        apply.setAffiliationOrgId(req.getAffiliationOrgId());
        apply.setAffiliationOrgName(req.getAffiliationOrgName());
        apply.setApplyOrgName(req.getApplyOrgName());
        apply.setWorkersNum(req.getWorkersNum());
        apply.setYouthNum(req.getYouthNum());
        apply.setLeaderName(req.getLeaderName());

        if (req.getLeaderPhone()!=null&&!req.getLeaderPhone().contains("*")){
            apply.setLeaderPhone(req.getLeaderPhone());
        }
        if (req.getLeaderIdCard()!=null&&!req.getLeaderIdCard().contains("*")){
            apply.setLeaderIdCard(req.getLeaderIdCard());
        }
        apply.setRemark(req.getRemark());
        apply.setDistributionType(req.getDistributionType());
        apply.setFileId(req.getFileId());
        apply.setFileName(req.getFileName());
        String oldNodeKeys = apply.getOldNodeKeys() == null ? "" : apply.getOldNodeKeys();
        apply.setOldNodeKeys(oldNodeKeys + "," + modelKey);
        apply.setUpdator(currWorker.getWorkerName());
        apply.setUpdateTime(new Date());
        YcqQzhQagApply ycqQzhQagApply = repo.saveAndFlush(apply);

        //添加可以查看的组织权限
        ReportOrgDto authDto = new ReportOrgDto();
        authDto.setReportType(2);
        authDto.setServiceId(ycqQzhQagApply.getId());
        authDto.setContainCurrentOrg(true);
        List<Long> authOrgs = new ArrayList<>();
        authOrgs.add(Long.valueOf(currWorker.getOrgId()));
        if(Objects.nonNull(req.getAffiliationOrgId())){
            authOrgs.add(Long.valueOf(req.getAffiliationOrgId()));
        }
        authOrgs.add(Long.valueOf(currWorker.getOrgId()));
        authDto.setOrgIds(authOrgs);
        civilizationReportService.saveQueryOrg(authDto,currWorker);
        return Result.ofSuccessMsg("操作成功");
    }

    @Override
    public Result<PageRsp<YcqQzhQagApplyRespDTO>> page(PageReq<YcqQzhQagApplyReqDTO> req) {

        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        //查出当前人青安岗的按钮权限
        List<TsmMenu> byMenuUrl = tsmMenuRepo.findByMenuUrl("/cyanfont/cyanfontManager/teenagerSecurePost");
        if (byMenuUrl.size()!=1) {
            throw new BusinessException("菜单权限数据异常，请联系管理员");
        }
        Long cityOrgCode = jdbcDao.getCityOrgCode();
        TsmMenu tsmMenu = byMenuUrl.get(0);
        PageRsp<YcqQzhQagApplyRespDTO> ycqQzhQagApplyRespDTOPageRsp = jdbcDao.pageList(req, currWorker,tsmMenu,cityOrgCode);
        List<YcqQzhQagApplyRespDTO> result = ycqQzhQagApplyRespDTOPageRsp.getResult();

        for (YcqQzhQagApplyRespDTO ycqQzhQagApplyRespDTO : result) {
            if (currWorker.getLoginId()!=ycqQzhQagApplyRespDTO.getCreateId()) {
                ycqQzhQagApplyRespDTO.setIsApplicant(0);
            }
            if (ycqQzhQagApplyRespDTO.getFlowState()==1 && ycqQzhQagApplyRespDTO.getButton()==1) {
                ycqQzhQagApplyRespDTO.setPreviewState(1);
            }
            if (ycqQzhQagApplyRespDTO.getFlowState()>1 && ycqQzhQagApplyRespDTO.getFlowState()<4) {
                ycqQzhQagApplyRespDTO.setPreviewState(1);
            }
            if (ycqQzhQagApplyRespDTO.getFlowState()==4 && ycqQzhQagApplyRespDTO.getButton()==1) {
                ycqQzhQagApplyRespDTO.setPreviewState(2);
            }
            if (ycqQzhQagApplyRespDTO.getFlowState()==4 && ycqQzhQagApplyRespDTO.getButton()==0) {
                ycqQzhQagApplyRespDTO.setPreviewState(1);
            }
            if (ycqQzhQagApplyRespDTO.getFlowState()>4) {
                ycqQzhQagApplyRespDTO.setPreviewState(2);
            }
            if (ycqQzhQagApplyRespDTO.getFlowState()==8) {
                if (ycqQzhQagApplyRespDTO.getOldNodeKeys().contains(SECOND_REVIEW)) {
                    ycqQzhQagApplyRespDTO.setPreviewState(1);
                }
                if (ycqQzhQagApplyRespDTO.getOldNodeKeys().contains(SECOND_CHECK)) {
                    ycqQzhQagApplyRespDTO.setPreviewState(2);
                }
            }
        }
        ycqQzhQagApplyRespDTOPageRsp.setResult(result);
        return Result.ofSuccess(ycqQzhQagApplyRespDTOPageRsp);
    }

    @Override
    public Result<PageRsp<YcqQzhQagApplyRespDTO>> examinePage(PageReq<YcqQzhQagApplyReqDTO> req) {

        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        //查出当前人青安岗的按钮权限
        List<TsmMenu> byMenuUrl = tsmMenuRepo.findByMenuUrl("/cyanfont/cyanfontManager/teenagerSecurePost");
        if (byMenuUrl.size()!=1) {
            throw new BusinessException("菜单权限数据异常，请联系管理员");
        }
        TsmMenu tsmMenu = byMenuUrl.get(0);
        List<String> CurrentAuth = jdbcDao.queryCurrentUserAuth(tsmMenu,currWorker).stream().map(TsmMenu::getMenuName).collect(Collectors.toList());

        QueryOrgDto dto = new QueryOrgDto();
        dto.setReportType(2);
        dto.setOrgId(currWorker.getOrgId());
        List<Long> ids = civilizationReportService.queryServiceIds(dto);
        //根据权限展示当前审核状态
        QagPageRsp<YcqQzhQagApplyRespDTO> ycqQzhQagApplyRespDTOPageRsp = jdbcDao.examinePage(req,tsmMenu,ids);
        List<YcqQzhQagApplyRespDTO> result = ycqQzhQagApplyRespDTOPageRsp.getResult();

        // 非超管处理
        if (!validatedSuper(currWorker)) {
            if (CurrentAuth.contains("申报批量审核")) {
                ycqQzhQagApplyRespDTOPageRsp.setIsBatchAudit(1);
            }
            for (YcqQzhQagApplyRespDTO data : result) {
                if (CurrentAuth.contains("申报查看")) {
                    data.setViewButton(1);
                }
                if (currWorker.getLoginId()!=data.getCreateId()) {
                    data.setIsApplicant(0);
                }
                if (data.getFlowState()==1 && data.getButton()==1) {
                    data.setPreviewState(1);
                }
                if (data.getFlowState()>1 && data.getFlowState()<4) {
                    data.setPreviewState(1);
                }
                if (data.getFlowState()==4 && data.getButton()==1) {
                    data.setPreviewState(2);
                }
                if (data.getFlowState()==4 && data.getButton()==0) {
                    data.setPreviewState(1);
                }
                if (data.getFlowState()>4 && data.getFlowState()<8) {
                    data.setPreviewState(2);
                }
                if (data.getFlowState()==8) {
                    if (data.getOldNodeKeys().contains(SECOND_REVIEW)) {
                        data.setPreviewState(1);
                    }
                    if (data.getOldNodeKeys().contains(SECOND_CHECK)) {
                        data.setPreviewState(2);
                    }
                }
                //回退
                if (CurrentAuth.contains("回退") && !Lists.newArrayList(8,7,6).contains(data.getFlowState())) {
                    if (Lists.newArrayList(1,4).contains(data.getFlowState())) {
                        data.setBackButton(1);
                    }
                    if (Lists.newArrayList(2,5,6).contains(data.getFlowState())) {
                        data.setBackButton(1);
                    }
                    if (currWorker.getLoginId()!=data.getCreateId()) {
                        data.setIsApplicant(0);
                    }
                }
            }
            ycqQzhQagApplyRespDTOPageRsp.setResult(result);
        } else {
            ycqQzhQagApplyRespDTOPageRsp.setIsBatchAudit(1);
            //超管处理
            for (YcqQzhQagApplyRespDTO data : result) {
                if (data.getFlowState()==1 && data.getButton()==1) {
                    data.setPreviewState(1);
                }
                if (data.getFlowState()>1 && data.getFlowState()<4) {
                    data.setPreviewState(1);
                }
                if (data.getFlowState()==4 && data.getButton()==1) {
                    data.setPreviewState(2);
                }
                if (data.getFlowState()>4) {
                    data.setPreviewState(2);
                }
                //回退
                if (!Lists.newArrayList(8,7,6).contains(data.getFlowState())) {
                    data.setBackButton(1);
                }
                data.setViewButton(1);
            }
            ycqQzhQagApplyRespDTOPageRsp.setResult(result);
        }
        return Result.ofSuccess(ycqQzhQagApplyRespDTOPageRsp);
    }

    private boolean validatedSuper(Worker worker) {
        if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            return true;
        }
        return false;
    }

    @Override
    public Result audit(YcqQzhQagAuditReqDTO req) {

        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        List<Long> ids = req.getIds().stream().map(Long::valueOf).collect(Collectors.toList());
        List<YcqQzhQagApply> allycqQzhQagApply = repo.findAllById(ids);
        for (YcqQzhQagApply ycqQzhQagApply : allycqQzhQagApply) {
            if (Lists.newArrayList(0,3,8).contains(ycqQzhQagApply.getFlowState())) {
                throw new BusinessException("草稿、待填写、已驳回状态不可审核");
            }
        }
        for (Long id : ids) {
            //审核下一步开始
            YcqQzhQagApply apply = repo.getById(id);
            FlowCommitDTO dto = new FlowCommitDTO();
            dto.setFlowInstanceId(apply.getFlowInstanceId());
            dto.setTaskInstanceId(apply.getNodeInstanceId());
            //设置提交人
            FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
            commitor.setOrgId(Long.parseLong(currWorker.getOrgId()));
            commitor.setOrgName(currWorker.getOrgName());
            commitor.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
            commitor.setUserName(currWorker.getWorkerName());
            dto.setCommitor(commitor);
            //处理结果处理意见
            dto.setProcessResult(req.getState() == 1 ? "通过" : "驳回");
            dto.setProcessComments(req.getReason());
            //设置流程变量后面判断
            Map<String, Object> map = new HashMap<>();
            map.put("state", req.getState());
            dto.setFlowVar(map);
            dto.setCommitFlow(true);

            if (req.getState() == 1) {
                //特殊新增
                //设置组织
                FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
                ArrayList<IdNameDTO> orgs = new ArrayList<>();
                IdNameDTO idNameDTO = new IdNameDTO();
                Long cityOrgCode = jdbcDao.getCityOrgCode();
                if (apply.getFlowState()==1 || apply.getFlowState()==4 || apply.getFlowState()==5 || apply.getFlowState()==6) {
                    idNameDTO.setId(cityOrgCode);
                    idNameDTO.setName("团重庆市委");
                }
                if (apply.getFlowState()==3) {
                    idNameDTO.setId(Long.valueOf(apply.getAffiliationOrgId()));
                    idNameDTO.setName(apply.getAffiliationOrgName());
                }
                if (apply.getFlowState()==2) {
                    TsmWorker worker = tsmWorkerRepo.findByLoginIdAndState(apply.getCreateId().intValue(), 1);
                    TsmOrg byOrgIdAndState = tsmOrgRepo.findByOrgIdAndState(Long.valueOf(worker.getOrgId()), 1);
                    idNameDTO.setId(Long.valueOf(worker.getOrgId()));
                    idNameDTO.setName(byOrgIdAndState.getOrgName());
                }
                orgs.add(idNameDTO);
                addedTaskAssigenn.setAssigennDept(orgs);
                dto.setAddedTaskAssigenn(addedTaskAssigenn);
            }
            CommonResult<CruniiCommitTaskResult> startResult = flowInstanceApi.commitProcess(dto);
            if (startResult.getCode() == 00 && startResult.getData().getErrCode() < 2000) {
                String nodeInstanceId = startResult.getData().getActiveTaskInstance().getNodeInstanceId();
                apply.setNodeInstanceId(nodeInstanceId);
                apply.setFlowState(req.getState() == 1 ? apply.getFlowState() + 1 : 8);
                apply.setUpdateTime(new Date());
                apply.setUpdator(currWorker.getWorkerName());
                apply.setLastAdvice(req.getState()==1?"通过":"不通过");
            } else {
                return Result.ofFail("流程执行失败");
            }
            //审核下一步开始结束

            String oldNodeKeys = apply.getOldNodeKeys() == null ? "" : apply.getOldNodeKeys();
            String modelKey = startResult.getData().getActiveTaskInstance().getModelKey();
            apply.setOldNodeKeys(oldNodeKeys + "," + modelKey);
            repo.saveAndFlush(apply);

            if (req.getState()!=1) {
                FlowBatchFinishDTO flowBatchFinishDTO = new FlowBatchFinishDTO();
                flowBatchFinishDTO.setFlowInstanceId(Sets.newHashSet(apply.getFlowInstanceId()));
                flowInstanceApi.batchFinish(flowBatchFinishDTO);
            }
        }
        return Result.ofSuccessMsg("操作成功");
    }

    @Override
    public Result<Boolean> back(YcqQzhQagBackReqDTO req) {

        if (StrUtil.isNotBlank(req.getReason()) && req.getReason().length()>50) {
            throw new BusinessException("回退说明不能超过50个字符");
        }

        YcqQzhQagApply apply = repo.getById(req.getId());
        FlowRollBackDTO dto = new FlowRollBackDTO();
        dto.setFlowInstanceId(apply.getFlowInstanceId());
        if (apply.getFlowState()==1 || apply.getFlowState()==2) {
            dto.setTargetNodeKey(CREATE_DECLARE);
        }
        if (apply.getFlowState()==4 || apply.getFlowState()==5) {
            dto.setTargetNodeKey(FILL);
        }
        //设置提交人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
        commitor.setOrgId(Long.parseLong(currWorker.getOrgId()));
        commitor.setOrgName(currWorker.getOrgName());
        commitor.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
        commitor.setUserName(currWorker.getWorkerName());
        dto.setCommitor(commitor);
        dto.setProcessResult("回退");
        dto.setProcessComments(req.getReason());
        CommonResult<RollbackTaskResult> result = flowInstanceApi.rollBackTargetNode(dto);

        String nodeInstanceId;
        if (result.getCode() == 00 && result.getData().getErrCode() < 2000) {
            nodeInstanceId = result.getData().getActiveTaskInstance().getNodeInstanceId();
        } else {
            return Result.ofFail("流程回退失败");
        }

        //终止当前正在发起变更流程
        if (apply.getFlowState() == 1 || apply.getFlowState() == 2) {
            QagApplyLeaderChangeLog terminateLog = qagApplyLeaderChangeLogRepo.findByQagApplyIdAndIsDeletedAndStateIn(apply.getId(), 0, Lists.newArrayList(0, 1, 3));
            if (terminateLog!=null) {
                terminateLog.setIsDeleted(1);
                terminateLog.setUpdateId((long) (currWorker.getLoginId()));
                terminateLog.setUpdatedor(currWorker.getWorkerName());
                terminateLog.setUpdatedTime(LocalDateTime.now());
                qagApplyLeaderChangeLogRepo.saveAndFlush(terminateLog);
            }
            // 清除掉组织数据
            Long cityOrgCode = jdbcDao.getCityOrgCode();
            YcqQzhQueryOrg ycqYzhQueryOrg = ycqQzhQueryOrgRepo.findByServiceIdAndDeleted(apply.getId(), 0);
            ycqYzhQueryOrg.setStrOrgId(cityOrgCode.toString());
            ycqYzhQueryOrg.setUpdaterId(currWorker.getLoginId());
            ycqYzhQueryOrg.setUpdateTime(new Date());
            ycqQzhQueryOrgRepo.saveAndFlush(ycqYzhQueryOrg);
        }

        //需要回退的节点
        String nodeKey = "";

        if (apply.getFlowState()==1 || apply.getFlowState()==2) {
            apply.setFlowState(0);
            nodeKey=CREATE_DECLARE;
        }
        if (apply.getFlowState()==4 || apply.getFlowState()==5) {
            apply.setFlowState(3);
            nodeKey=FILL;
        }
        apply.setNodeInstanceId(nodeInstanceId);
        String oldNodeKeys = apply.getOldNodeKeys();
        String[] split = oldNodeKeys.split(",");
        String oldKey = Arrays.stream(getSubArray(split, nodeKey)).collect(Collectors.joining(","));

        apply.setOldNodeKeys(oldKey);
        apply.setUpdator(currWorker.getWorkerName());
        apply.setUpdateTime(new Date());
        //推荐申报回退业务
        if (apply.getFlowState() == 3) {
            YouthSafetyModelRecommend recommend = youthSafetyModelRecommendRepo.findByQagApplyId(req.getId());
            if (recommend!=null){
                recommend.setIsDraft(1);
                youthSafetyModelRecommendRepo.saveAndFlush(recommend);
            }
        }



        repo.saveAndFlush(apply);
        return Result.ofSuccessMsg("操作成功");
    }

    private static String[] getSubArray(String[] array, String target) {
        // 查找目标字符在数组中的索引
        int index = Arrays.asList(array).indexOf(target);
        // 如果目标字符不在数组中或者是第一个字符，则返回原数组
        if (index == -1 || index == 0) {
            return array;
        }
        // 获取目标字符之前的子数组
        return Arrays.copyOfRange(array, 0, index + 1);
    }

    @Override
    public Result<List<CruniiFlowProcessHistoryRespVO>> backList(YcqQzhQagBackReqDTO req) {
        YcqQzhQagApply apply = repo.getById(req.getId());
        FlowInstanceReqVO dto = new FlowInstanceReqVO();
        dto.setFlowInstanceId(apply.getFlowInstanceId());
        CommonResult<List<CruniiFlowProcessHistoryRespVO>> result = flowInstanceApi.listProcessHistory(dto);
        List<CruniiFlowProcessHistoryRespVO> data = result.getData();
        if (data != null && data.size() > 0) {
            data.remove(data.size() - 1);
        }
        List<CruniiFlowProcessHistoryRespVO> list = data.stream().filter(distinctByKey(CruniiFlowProcessHistoryRespVO::getNodeKey)).collect(Collectors.toList());
        ArrayList<CruniiFlowProcessHistoryRespVO> resultList = new ArrayList<>();
        String oldNodeKeys = apply.getOldNodeKeys();
        if (oldNodeKeys == null || oldNodeKeys.equals("")) {
            return Result.ofSuccess(new ArrayList<CruniiFlowProcessHistoryRespVO>());
        } else {
            for (CruniiFlowProcessHistoryRespVO vo : list) {
                if (apply.getFlowState()==1 && "初评".equals(vo.getNodeName())) {
                    continue;
                }
                if (apply.getFlowState()==2 && "复评".equals(vo.getNodeName())) {
                    continue;
                }
                if (apply.getFlowState()==4 && "初核".equals(vo.getNodeName())) {
                    continue;
                }
                if (apply.getFlowState()==5 && "复核".equals(vo.getNodeName())) {
                    continue;
                }
                if (vo.getNodeKey() != null && oldNodeKeys.contains(vo.getNodeKey())) {
                    resultList.add(vo);
                }

            }

        }
        Result<List<CruniiFlowProcessHistoryRespVO>> resultBack = new Result<>();
        resultBack.setData(resultList);
        resultBack.setMsg("操作成功");
        return resultBack;
    }

    @Override
    public Result<YcqQzhQagApplyTotalRespDTO> getApplyTotal() {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        Integer applyTotal = jdbcDao.applyTotal(currWorker);
        Integer applyChangeTotal = leaderChangeLogDao.applyChangeTotal(currWorker);
        YcqQzhQagApplyTotalRespDTO dto = new YcqQzhQagApplyTotalRespDTO();
        dto.setApplyNum(applyTotal);
        dto.setApplyChangeNum(applyChangeTotal);
        return Result.ofSuccess(dto);
    }


    @Override
    public Result<YcqQzhQagApplyTotalRespDTO> getExamineTotal() {

        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        // 变更审核记录
        List<TsmMenu> byMenuUrl = tsmMenuRepo.findByMenuUrl("/cyanfont/cyanfontManager/teenagerSecurePost");
        if (byMenuUrl.size()!=1) {
            throw new BusinessException("菜单权限数据异常，请联系管理员");
        }

        QueryOrgDto dto = new QueryOrgDto();
        dto.setReportType(2);
        dto.setOrgId(currWorker.getOrgId());
        List<Long> ids = civilizationReportService.queryServiceIds(dto);

        TsmMenu tsmMenu = byMenuUrl.get(0);
        Integer num = jdbcDao.countPending(tsmMenu,currWorker,ids);
        Integer changeNum = qagApplyLeaderChangeLogDao.countChangePending(tsmMenu,currWorker,ids);
        YcqQzhQagApplyTotalRespDTO ycqQzhQagApplyTotalRespDTO = new YcqQzhQagApplyTotalRespDTO();
        ycqQzhQagApplyTotalRespDTO.setApplyChangeNum(changeNum);
        ycqQzhQagApplyTotalRespDTO.setApplyNum(num);
        return Result.ofSuccess(ycqQzhQagApplyTotalRespDTO);
    }

    @Override
    public Result<List<QagFlowHistoryRespVO>> flowHistory(FlowInstanceReqVO flowInstanceReqVO) {
        YcqQzhQagApply ycqQzhQagApply = repo.findByFlowInstanceIdAndDelFlag(flowInstanceReqVO.getFlowInstanceId(),0);
        YouthSafetyModelRecommend qagApplyId = youthSafetyModelRecommendRepo.findByQagApplyId(ycqQzhQagApply.getId());
        List<YouthSafetyRecommendFile> filse = null;
        if (qagApplyId!=null) {
            filse = youthSafetyRecommendFileRepo.findByRecommendId(qagApplyId.getId());
        }
        CommonResult<List<CruniiFlowProcessHistoryRespVO>> listCommonResult = flowInstanceApi.listProcessHistory(flowInstanceReqVO);
        List<CruniiFlowProcessHistoryRespVO> data = listCommonResult.getData();
        List<QagFlowHistoryRespVO> qagFlowHistoryRespVOS = new ArrayList<>();
        for (CruniiFlowProcessHistoryRespVO datum : data) {
            QagFlowHistoryRespVO qagFlowHistoryRespVO = new QagFlowHistoryRespVO();
            BeanUtils.copyProperties(datum,qagFlowHistoryRespVO);
            if ("创建集体申报".equals(datum.getNodeName())) {
                // 查出流程文件
                if (ycqQzhQagApply!=null && ycqQzhQagApply.getFileId()!=null && StrUtil.isNotBlank(ycqQzhQagApply.getFileName())) {
                    FlowHistoryFilesRespVO flowHistoryFilesRespVO = new FlowHistoryFilesRespVO();
                    flowHistoryFilesRespVO.setFileId(ycqQzhQagApply.getFileId());
                    flowHistoryFilesRespVO.setFileName(ycqQzhQagApply.getFileName());
                    qagFlowHistoryRespVO.setFiles(Lists.newArrayList(flowHistoryFilesRespVO));
                }
            }
            if ("推荐表填写".equals(datum.getNodeName())) {
                //
                if (CollectionUtils.isNotEmpty(filse)) {
                    ArrayList<FlowHistoryFilesRespVO> flowHistoryFilesRespVOS = new ArrayList<>();
                    for (YouthSafetyRecommendFile youthSafetyRecommendFile : filse) {
                        FlowHistoryFilesRespVO flowHistoryFilesRespVO = new FlowHistoryFilesRespVO();
                        BeanUtils.copyProperties(youthSafetyRecommendFile,flowHistoryFilesRespVO);
                        flowHistoryFilesRespVOS.add(flowHistoryFilesRespVO);
                    }
                    qagFlowHistoryRespVO.setFiles(flowHistoryFilesRespVOS);
                }
            }
            qagFlowHistoryRespVOS.add(qagFlowHistoryRespVO);
        }

        return Result.ofSuccess(qagFlowHistoryRespVOS);
    }

    private static <T> Predicate<T> distinctByKey(Function<? super T, Object> keyExtractor) {
        Map<Object, Boolean> seen = new ConcurrentHashMap<>();
        return t -> seen.putIfAbsent(keyExtractor.apply(t), Boolean.TRUE) == null;
    }


    public static String getCode() {
        String code = "QAG";
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
        Date date = new Date();
        String now = format.format(date);
        code += now;
        char[] arr = new char[62];
        for (int i = 0; i < 26; i++) {
            arr[i] = (char) ('A' + i);
        }
        for (int i = 0; i < 26; i++) {
            arr[i + 26] = (char) ('a' + i);
        }
        for (int i = 0; i < 10; i++) {
            arr[i + 52] = (char) (48 + i);
        }
        Random random = new Random();
        String str = "";
        for (int i = 0; i < 4; i++) {
            str += arr[random.nextInt(62)];
        }
        code += str;
        return code;
    }
}
