package com.crunii.micro.service.qzh.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqTrainingCenterFile;
import com.crunii.micro.domain.repo.main.YcqTrainingCenterFileRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.YcqTrainingCenterFileDTO;
import com.crunii.micro.service.qzh.api.domain.dto.YcqTrainingCenterFileVo;
import com.crunii.micro.service.qzh.dto.video.FileDto;
import com.crunii.micro.service.qzh.jdbc.YcqTrainingCenterFileJdbcDao;
import com.crunii.micro.service.qzh.service.QzhVideoService;
import com.crunii.micro.service.qzh.service.YcqTrainingCenterFileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Objects;


@Service
@Slf4j
public class YcqTrainingCenterFileServiceImpl implements YcqTrainingCenterFileService {

    @Resource
    private YcqTrainingCenterFileRepo ycqTrainingCenterFileRepo;
    @Resource
    private YcqTrainingCenterFileJdbcDao jdbcDao;

    @Resource
    private QzhVideoService videoService;

    @Override
    public Result<Void> create(YcqTrainingCenterFileDTO dto) {
                Worker worker = CurrWorkerUtil.getCurrWorker();
//        Worker worker = new Worker();
//        worker.setLoginId(9999);
//        worker.setWorkerName("test");
        if (dto.getSort() != null) {
            List<YcqTrainingCenterFile> files = ycqTrainingCenterFileRepo.findBySort(dto.getSort());
            if (files != null && files.size() > 0) {
                return Result.ofFail("排序重复");
            }
        }


        YcqTrainingCenterFile ycqTrainingCenterFile = new YcqTrainingCenterFile();
        ycqTrainingCenterFile.setFileId(dto.getFileId());
        ycqTrainingCenterFile.setFileName(dto.getFileName());
        ycqTrainingCenterFile.setFileUrl(dto.getFileUrl());
        ycqTrainingCenterFile.setBusiness(dto.getBusiness());
        ycqTrainingCenterFile.setRemark(dto.getRemark());
        ycqTrainingCenterFile.setSort(dto.getSort());
        ycqTrainingCenterFile.setType(dto.getType());
        ycqTrainingCenterFile.setCreateBy(worker.getLoginId() + "");
        ycqTrainingCenterFile.setCreateTime(new Date());
        ycqTrainingCenterFile.setIsAppShow(dto.getIsAppShow());

        if (dto.getType() != null && dto.getType() != 1) {
            Boolean checkFileType = checkFileType(dto.getFileUrl());
            if (!checkFileType) {
                return Result.ofFail("上传文件类型有问题");
            }

        }
        ycqTrainingCenterFileRepo.saveAndFlush(ycqTrainingCenterFile);
        return Result.ofSuccess();
    }

    private Boolean checkFileType(String url) {
        if (url != null && (url.contains(".doc") || url.contains(".docx") || url.contains(".xls") || url.contains(".pdf") || url.contains(".jpg") || url.contains(".jpeg") || url.contains(".png"))) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public Result<Void> delete(Long id) {
        YcqTrainingCenterFile one = ycqTrainingCenterFileRepo.getOne(id);
        if (one == null) {
            return Result.ofFail("没有数据");
        }
        ycqTrainingCenterFileRepo.delete(one);
        return Result.ofSuccess();
    }

    @Override
    public Result<PageRsp<YcqTrainingCenterFileVo>> page(PageReq<YcqTrainingCenterFileDTO> req) {
        PageRsp<YcqTrainingCenterFileVo> rsp = jdbcDao.pageList(req);
        return Result.ofSuccess(rsp);
    }

    @Override
    public Result<Void> update(YcqTrainingCenterFileDTO dto) {
        YcqTrainingCenterFile ycqTrainingCenterFile = ycqTrainingCenterFileRepo.getOne(dto.getId());
        if (ycqTrainingCenterFile == null) {
            return Result.ofFail("没有数据");
        }

        if (dto.getSort() != null&&ycqTrainingCenterFile.getSort()!=null&&ycqTrainingCenterFile.getSort()!=dto.getSort()) {
            List<YcqTrainingCenterFile> files = ycqTrainingCenterFileRepo.findBySort(dto.getSort());
            if (files != null && files.size() > 0) {
                return Result.ofFail("排序重复");
            }
        }

        ycqTrainingCenterFile.setFileId(dto.getFileId());
        ycqTrainingCenterFile.setFileName(dto.getFileName());
        ycqTrainingCenterFile.setFileUrl(dto.getFileUrl());
        ycqTrainingCenterFile.setBusiness(dto.getBusiness());
        ycqTrainingCenterFile.setRemark(dto.getRemark());
        ycqTrainingCenterFile.setType(dto.getType());
        ycqTrainingCenterFile.setSort(dto.getSort());
        ycqTrainingCenterFile.setIsAppShow(dto.getIsAppShow());

        if (dto.getType() != null && dto.getType() != 1) {
            Boolean checkFileType = checkFileType(dto.getFileUrl());
            if (!checkFileType) {
                return Result.ofFail("上传文件类型有问题");
            }

        }
        ycqTrainingCenterFileRepo.saveAndFlush(ycqTrainingCenterFile);
        return Result.ofSuccess();
    }

    @Override
    public Result<List<YcqTrainingCenterFileVo>> list(YcqTrainingCenterFileDTO req) {
        return Result.ofSuccess(jdbcDao.list(req));
    }


}
