package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.core.annotation.InjectProperty;
import com.crunii.micro.domain.dto.TsmRoleDto;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.entity.main.cavalryTrumpet.YcqYnCaptainChange;
import com.crunii.micro.domain.entity.main.cavalryTrumpet.YcqYnCaptainChangeFlow;
import com.crunii.micro.domain.entity.main.cavalryTrumpet.YcqYnCategory;
import com.crunii.micro.domain.entity.main.cavalryTrumpet.YcqYnDeclaration;
import com.crunii.micro.domain.repo.main.cavalryTrumpet.YcqYnCaptainChangeFlowRepo;
import com.crunii.micro.domain.repo.main.cavalryTrumpet.YcqYnCaptainChangeRepo;
import com.crunii.micro.domain.repo.main.cavalryTrumpet.YcqYnCategoryRepo;
import com.crunii.micro.domain.repo.main.cavalryTrumpet.YcqYnDeclarationRepo;
import com.crunii.micro.feign.client.SysService;
import com.crunii.micro.feign.client.dto.WorkerC;
import com.crunii.micro.feign.client.dto.WorkerCreateDTO;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.*;
import com.crunii.micro.service.qzh.jdbc.YcqYnDeclarationRecordJdbcDao;
import com.crunii.micro.service.qzh.service.CivilizationReportService;
import com.crunii.micro.service.qzh.service.YcqYnDeclarationRecordService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author: LG
 * @date: 2023/11/14 16:06
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class YcqYnDeclarationRecordServiceImpl implements YcqYnDeclarationRecordService {

    @Resource
    private YcqYnDeclarationRecordJdbcDao ycqYnDeclarationRecordJdbcDao;

    @Resource
    private YcqYnCategoryRepo ycqYnCategoryRepo;

    @Resource
    private YcqYnDeclarationRepo ycqYnDeclarationRepo;

    @Resource
    private YcqYnCaptainChangeRepo ycqYnCaptainChangeRepo;

    @Resource
    private CivilizationReportService civilizationReportService;

    @Resource
    private YcqYnCaptainChangeFlowRepo ycqYnCaptainChangeFlowRepo;

    @Resource
    private SysService sysService;


    private static final  Integer COUNTY_AUDIT_TYPE = 0;
    private static final  Integer CITY_AUDIT_TYPE = 1;

    private static final  String CITY_ORG_ID = "1674703692521037822";


    @Override
    public Result<QueryNumVO> queryDeclarationRecordChangeNumber(Worker worker, Integer queryType) {
        // type 0:workerId查条数，1：orgId条数
        QueryNumVO queryNumVO = new QueryNumVO();
        int reportCount = 0;
        Integer changeNum = 0;
        if(queryType.equals(CommonConstants.STATE_INVALID)){
            reportCount = civilizationReportService.findCurrentReportCount(worker);
            // changeNum 区分用户是区县还是市级
            if (worker.getOrgId().equals(CITY_ORG_ID)){
                changeNum = ycqYnDeclarationRecordJdbcDao.queryChangeNumWorker(worker.getLoginId(),CITY_AUDIT_TYPE);
            }else {
                changeNum = ycqYnDeclarationRecordJdbcDao.queryChangeIds(Long.valueOf(worker.getOrgId()),false).size();
//                changeNum = ycqYnDeclarationRecordJdbcDao.queryChangeNumWorker(worker.getLoginId(),COUNTY_AUDIT_TYPE);
            }

        }else if(queryType.equals(CommonConstants.STATE_VALID)){
            reportCount = civilizationReportService.findReportCount(worker);
            QueryOrgDto queryOrgDto = new QueryOrgDto();
            queryOrgDto.setOrgId(worker.getOrgId());
            queryOrgDto.setReportType(CommonConstants.STATE_VALID);
            List<Long> delIds = civilizationReportService.queryServiceIds(queryOrgDto);
            // changeNum 区分用户是区县还是市级
            if (worker.getOrgId().equals(CITY_ORG_ID)){
                changeNum = ycqYnDeclarationRecordJdbcDao.queryChangeNumOrg(delIds,CITY_AUDIT_TYPE);
            }else {
                changeNum = ycqYnDeclarationRecordJdbcDao.queryChangeIds(Long.valueOf(worker.getOrgId()),false).size();
//                changeNum = ycqYnDeclarationRecordJdbcDao.queryChangeNumOrg(delIds,COUNTY_AUDIT_TYPE);
            }
        }

        queryNumVO.setChangeNum(changeNum);
        queryNumVO.setDelNum(reportCount);

        return Result.ofSuccess(queryNumVO);
    }

    @Override
    public Result<List<QueryDeclarationRecordChangeFlowVO>> queryDeclarationRecordChangeFlow(Worker worker, Long declarationId) {
        return Result.ofSuccess(ycqYnDeclarationRecordJdbcDao.queryDeclarationRecordChangeFlow(declarationId));
    }

    @Override
    @InjectProperty
    public Result<PageRsp<QueryDeclarationRecordChangeVO>> queryDeclarationRecordChangeByWorker(Worker worker, PageReq<QueryDeclarationRecordChangeDTO> dto) {
        return Result.ofSuccess(ycqYnDeclarationRecordJdbcDao.queryDeclarationRecordChangeByWorker(worker,dto));

    }

    @Override
    @InjectProperty
    public Result<PageRsp<QueryDeclarationRecordChangeVO>> queryDeclarationRecordChange(Worker worker, PageReq<CivilizationReportPageDto> dto) {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(10192);
            worker=worker1;
        }
        QueryOrgDto queryOrgDto = new QueryOrgDto();
        queryOrgDto.setOrgId(worker.getOrgId());
        queryOrgDto.setReportType(CommonConstants.STATE_VALID);
        List<Long> delIds = civilizationReportService.queryServiceIds(queryOrgDto);

        PageRsp<QueryDeclarationRecordChangeVO> queryDeclarationRecordChangeVOPageRsp = ycqYnDeclarationRecordJdbcDao.queryDeclarationRecordChange(delIds, dto);
        List<QueryDeclarationRecordChangeVO> result = queryDeclarationRecordChangeVOPageRsp.getResult();

        //workerOrg是重庆市org的话
        List<Long> changeAuditIds = new ArrayList<>();
        if (worker.getOrgId().equals(CITY_ORG_ID)){
            //查出来的id 赋值true
            for (QueryDeclarationRecordChangeVO r : result) {
                if (r.getAuditState().equals(CITY_AUDIT_TYPE)){
                    r.setWhetherAudit(true);
                }
            }
        }else {
            changeAuditIds = ycqYnDeclarationRecordJdbcDao.queryChangeIds(Long.valueOf(worker.getOrgId()),false);
            markWhetherAudit(result,COUNTY_AUDIT_TYPE,changeAuditIds);
        }

        return Result.ofSuccess(queryDeclarationRecordChangeVOPageRsp);
    }

    private void markWhetherAudit(List<QueryDeclarationRecordChangeVO> result, Integer cityAuditType,List<Long> changeAuditIds) {
        //查出来的id 赋值true
        for (QueryDeclarationRecordChangeVO r : result) {
            if (changeAuditIds.contains(r.getChangeId()) && r.getAuditState().equals(cityAuditType)){
                r.setWhetherAudit(true);
            }
        }
    }

    @Override
    public Result<String> saveChangeCaptain(Worker worker, ChangeCaptainDTO dto) {
        Integer integer = ycqYnDeclarationRecordJdbcDao.queryWhetherChange(dto.getDeclarationId());
        if (!integer.equals(CommonConstants.STATE_INVALID)){
            throw new BusinessException("存在正在进行中的变更流程");
        }

        YcqYnDeclaration byDeclarationIdAndDeleted = ycqYnDeclarationRepo.findByDeclarationIdAndDeleted(dto.getDeclarationId(), CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(byDeclarationIdAndDeleted)){
            throw  new BusinessException("数据异常");
        }
        //号长不能变更号长
        Integer i = worker.getLoginId();
        if (i.equals(byDeclarationIdAndDeleted.getLoginId())){
            throw  new BusinessException("号长无权限更改");
        }

        //相同数据不做处理
        if (dto.getAfterDeclarationName().equals(byDeclarationIdAndDeleted.getDeclarationName()) && dto.getAfterIdCard().equals(byDeclarationIdAndDeleted.getIdCard()) && dto.getAfterPhoneNumber().equals(byDeclarationIdAndDeleted.getPhoneNumber()) &&dto.getAfterWechat().equals(byDeclarationIdAndDeleted.getWechat())){
            return Result.ofSuccess("提交成功");
        }

        //存变更表
        YcqYnCaptainChange ycqYnCaptainChange = new YcqYnCaptainChange();
        BeanUtils.copyProperties(dto,ycqYnCaptainChange);
        ycqYnCaptainChange.setChangeDeclarationName(byDeclarationIdAndDeleted.getDeclarationName());
        ycqYnCaptainChange.setChangeIdCard(byDeclarationIdAndDeleted.getIdCard());
        ycqYnCaptainChange.setChangePhoneNumber(byDeclarationIdAndDeleted.getPhoneNumber());
        ycqYnCaptainChange.setChangeWechat(byDeclarationIdAndDeleted.getWechat());
        ycqYnCaptainChange.setChangeState(CommonConstants.STATE_VALID);
        ycqYnCaptainChange.setAuditState(CommonConstants.STATE_INVALID);
        ycqYnCaptainChange.setCreatorId(worker.getLoginId());
        ycqYnCaptainChange.setCreateTime(new Date());
        ycqYnCaptainChange.setUpdaterId(worker.getLoginId());
        ycqYnCaptainChange.setUpdateTime(new Date());
        ycqYnCaptainChange.setDeleted(CommonConstants.STATE_NO);
        ycqYnCaptainChangeRepo.save(ycqYnCaptainChange);

        //存变更流程表
        YcqYnCaptainChangeFlow ycqYnCaptainChangeFlow = new YcqYnCaptainChangeFlow();
        ycqYnCaptainChangeFlow.setDeclarationId(byDeclarationIdAndDeleted.getDeclarationId());
        ycqYnCaptainChangeFlow.setFlowState("流程发起");
        ycqYnCaptainChangeFlow.setHandlerState("提交");
        ycqYnCaptainChangeFlow.setHandler(worker.getWorkerName());
        ycqYnCaptainChangeFlow.setHandlerOrg(worker.getOrgName());
        ycqYnCaptainChangeFlow.setHandleTime(new Date());
        ycqYnCaptainChangeFlow.setProcessType(CommonConstants.MENU_TYPE_DIR);
        ycqYnCaptainChangeFlow.setCreatorId(worker.getLoginId());
        ycqYnCaptainChangeFlowRepo.save(ycqYnCaptainChangeFlow);
        return Result.ofSuccess("提交成功");
    }

    @Override
    public Result<QueryCaptainInfoVO> queryCaptainInfo(Long changeId) {
        return Result.ofSuccess(ycqYnDeclarationRecordJdbcDao.queryCaptainInfo(changeId));
    }

    @Override
    public Result<String> updateAuditCaptainPass(Worker worker, AuditCaptainDTO dto) {
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            List<TsmRoleDto> list = new ArrayList<>();
            TsmRoleDto tsmRoleDto = new TsmRoleDto();
            list.add(tsmRoleDto);
            worker1.setRoles(list);
            worker=worker1;
        }
        //更新变更表状态
        for (Long changeId : dto.getChangeIds()) {
            YcqYnCaptainChange byChangeIdAndDeleted = ycqYnCaptainChangeRepo.findByChangeIdAndDeleted(changeId, CommonConstants.STATE_INVALID);
            if (ObjectUtil.isEmpty(byChangeIdAndDeleted)){
                throw new BusinessException("请传入有效变更id");
            }
            YcqYnCaptainChangeFlow ycqYnCaptainChangeFlow = new YcqYnCaptainChangeFlow();
            if (byChangeIdAndDeleted.getAuditState().equals(CITY_AUDIT_TYPE)){
                validatedOrgAudit(worker,CITY_ORG_ID);
                byChangeIdAndDeleted.setAuditState(CommonConstants.MENU_TYPE_BUTTON);
                //如果是市级审核通过，变更号长存档申报表
                saveDelarationCaptain(worker,byChangeIdAndDeleted);
                ycqYnCaptainChangeFlow.setFlowState("市团委审核");
            }else if (byChangeIdAndDeleted.getAuditState().equals(COUNTY_AUDIT_TYPE)){
                Long countOrgId = ycqYnDeclarationRecordJdbcDao.queryCountOrgId(byChangeIdAndDeleted.getChangeId());
                if (ObjectUtil.isEmpty(countOrgId)){
                    throw new BusinessException("申报组织id数据异常");
                }
                validatedOrgAudit(worker,countOrgId.toString());
                ycqYnCaptainChangeFlow.setFlowState("区县委审核");
                byChangeIdAndDeleted.setAuditState(CommonConstants.STATE_VALID);
            }else {
                throw new BusinessException("请选择待审核状态记录");
            }
            byChangeIdAndDeleted.setUpdateTime(new Date());
            byChangeIdAndDeleted.setUpdaterId(worker.getLoginId());
            byChangeIdAndDeleted.setReason(dto.getReason());
            ycqYnCaptainChangeRepo.saveAndFlush(byChangeIdAndDeleted);

            //存变更流程表
            ycqYnCaptainChangeFlow.setDeclarationId(byChangeIdAndDeleted.getDeclarationId());
            ycqYnCaptainChangeFlow.setHandlerState("通过");
            ycqYnCaptainChangeFlow.setHandler(worker.getWorkerName());
            ycqYnCaptainChangeFlow.setHandlerOrg(worker.getOrgName());
            ycqYnCaptainChangeFlow.setHandleTime(new Date());
            ycqYnCaptainChangeFlow.setReason(dto.getReason());
            ycqYnCaptainChangeFlow.setProcessType(CommonConstants.MENU_TYPE_MENU);
            ycqYnCaptainChangeFlow.setCreatorId(worker.getLoginId());
            ycqYnCaptainChangeFlowRepo.save(ycqYnCaptainChangeFlow);
        }
        return Result.ofSuccess("审核成功");
    }

    private void saveDelarationCaptain(Worker worker, YcqYnCaptainChange byChangeIdAndDeleted) {
        YcqYnDeclaration byDeclarationIdAndDeleted = ycqYnDeclarationRepo.findByDeclarationIdAndDeleted(byChangeIdAndDeleted.getDeclarationId(), CommonConstants.STATE_NO);
        if (ObjectUtil.isEmpty(byDeclarationIdAndDeleted)){
            throw new BusinessException("数据有误");
        }
        TsmWorker tsmWorker = ycqYnDeclarationRecordJdbcDao.whetherCaptainWorker(byChangeIdAndDeleted.getAfterPhoneNumber());
        if (ObjectUtil.isEmpty(tsmWorker)){
            // 新建worker
            WorkerC workerC = new WorkerC();
            workerC.setMobile(byChangeIdAndDeleted.getAfterPhoneNumber());
            workerC.setWorkerName(byChangeIdAndDeleted.getAfterDeclarationName());
            workerC.setState(CommonConstants.STATE_VALID);
            Long orgId = ycqYnDeclarationRecordJdbcDao.queryOrgDel(byChangeIdAndDeleted.getDeclarationId());
            if (ObjectUtil.isNotEmpty(orgId)) {
                workerC.setOrgId(orgId.toString());
            }
            Result<WorkerCreateDTO> captainWorker = sysService.createWorker(workerC);
            if (!captainWorker.isSuccess()) {
                log.info("创建用户失败，feign接口调用失败！参数-》{}", JSONUtil.toJsonStr(workerC));
                throw new BusinessException(captainWorker.getMsg());
            }
            byDeclarationIdAndDeleted.setLoginId(captainWorker.getData().getLoginId());
        }else {
            byDeclarationIdAndDeleted.setLoginId(tsmWorker.getLoginId());
        }
        byDeclarationIdAndDeleted.setDeclarationName(byChangeIdAndDeleted.getAfterDeclarationName());
        byDeclarationIdAndDeleted.setIdCard(byChangeIdAndDeleted.getAfterIdCard());
        byDeclarationIdAndDeleted.setPhoneNumber(byChangeIdAndDeleted.getAfterPhoneNumber());
        byDeclarationIdAndDeleted.setWechat(byChangeIdAndDeleted.getAfterWechat());
        byDeclarationIdAndDeleted.setUpdaterId(worker.getLoginId());
        byDeclarationIdAndDeleted.setUpdateTime(new Date());
        ycqYnDeclarationRepo.saveAndFlush(byDeclarationIdAndDeleted);
    }

    @Override
    public Result<String> updateAuditCaptainNotPass(Worker worker, AuditCaptainDTO dto) {
        for (Long changeId : dto.getChangeIds()) {
            Long countOrgId = ycqYnDeclarationRecordJdbcDao.queryCountOrgId(changeId);
            if (ObjectUtil.isEmpty(countOrgId)){
                throw new BusinessException("申报组织id数据异常");
            }
            YcqYnCaptainChangeFlow ycqYnCaptainChangeFlow = new YcqYnCaptainChangeFlow();
            if (worker.getOrgId().equals(CITY_ORG_ID) ){
                ycqYnCaptainChangeFlow.setFlowState("市团委审核");
            }else if (worker.getOrgId().equals(countOrgId.toString())){
                ycqYnCaptainChangeFlow.setFlowState("区县团委审核");
            }else {
                throw new BusinessException("权限不足！");
            }

            YcqYnCaptainChange byChangeIdAndDeleted = ycqYnCaptainChangeRepo.findByChangeIdAndDeleted(changeId, CommonConstants.STATE_INVALID);
            byChangeIdAndDeleted.setAuditState(CommonConstants.LOGIN_FAIL_ERR_PWD);
            byChangeIdAndDeleted.setUpdateTime(new Date());
            byChangeIdAndDeleted.setUpdaterId(worker.getLoginId());
            byChangeIdAndDeleted.setReason(dto.getReason());
            ycqYnCaptainChangeRepo.saveAndFlush(byChangeIdAndDeleted);

            //存变更流程表
            ycqYnCaptainChangeFlow.setDeclarationId(byChangeIdAndDeleted.getDeclarationId());
            ycqYnCaptainChangeFlow.setHandlerState("驳回");
            ycqYnCaptainChangeFlow.setHandler(worker.getWorkerName());
            ycqYnCaptainChangeFlow.setHandlerOrg(worker.getOrgName());
            ycqYnCaptainChangeFlow.setHandleTime(new Date());
            ycqYnCaptainChangeFlow.setReason(dto.getReason());
            ycqYnCaptainChangeFlow.setProcessType(CommonConstants.MENU_TYPE_BUTTON);
            ycqYnCaptainChangeFlow.setCreatorId(worker.getLoginId());
            ycqYnCaptainChangeFlowRepo.save(ycqYnCaptainChangeFlow);
        }



        return Result.ofSuccess("审核操作成功");
    }

    @Override
    public String addConfig(Worker worker, AddConfigDTO dto) {
        Date now = new Date();
        int loginId = worker.getLoginId();
        YcqYnCategory ycqYnCategory = new YcqYnCategory();
        BeanUtils.copyProperties(dto,ycqYnCategory);
        ycqYnCategory.setCreatorId(loginId);
        ycqYnCategory.setCreateTime(now);
        saveUpdate(worker,ycqYnCategory);
        YcqYnCategory save = ycqYnCategoryRepo.saveAndFlush(ycqYnCategory);
        return save.getBeforeCategoryId().toString();
    }

    @Override
    public String deleteConfig(Worker worker, ConfigEffectiveDto configEffectiveDto) {
        YcqYnCategory byId = ycqYnCategoryRepo.findByBeforeCategoryIdAndDeleted(configEffectiveDto.getBeforeCategoryId(),0);
        if(Objects.isNull(byId)){
            throw new BusinessException("无效id");
        }
        byId.setState(configEffectiveDto.getState());
        saveUpdate(worker, byId);
        ycqYnCategoryRepo.saveAndFlush(byId);
        return "修改成功";
    }

    @Override
    public String updateConfig(Worker worker, UpdateConfigDTO dto) {
        YcqYnCategory ycqYnCategory = ycqYnCategoryRepo.findByBeforeCategoryIdAndDeleted(dto.getBeforeCategoryId(), 0);
        if(Objects.isNull(ycqYnCategory)){
            throw new BusinessException("无效主键");
        }
        ycqYnCategory.setCategoryName(dto.getCategoryName());
        ycqYnCategory.setState(dto.getState());
        saveUpdate(worker,ycqYnCategory);
        ycqYnCategoryRepo.saveAndFlush(ycqYnCategory);
        return "更新成功";
    }

    @Override
    public PageRsp<QueryConfigVO> queryConfig(PageReq<QueryConfigDTO> dto) {
        return ycqYnDeclarationRecordJdbcDao.queryConfig(dto);
    }

    /**
     * 保存修改信息
     */
    private void saveUpdate(Worker worker, YcqYnCategory byId) {
        byId.setUpdateTime(new Date());
        byId.setUpdaterId(worker.getLoginId());
    }



    /**
     * 判断角色权限是否有此权限
     * @param worker worker
     * @return
     */
    private boolean validatedRoleAudit(Worker worker,Integer role) {
        List<TsmRoleDto> roles = worker.getRoles();
        List<Integer> roleIds = roles.stream().map(TsmRoleDto::getRoleId).collect(Collectors.toList());
        // 判断是否重庆市委审核
        if(roleIds.contains(role)){
            return true;
        }else{
            throw new BusinessException("权限不足！");
        }

    }

    private boolean validatedOrgAudit(Worker worker, String orgId) {
        if (worker.getOrgId().equals(orgId)){
            return true;
        }else {
            throw new BusinessException("权限不足！");
        }
    }
}
