package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqYouthCommonFile;
import com.crunii.micro.domain.entity.main.YcqYouthStart;
import com.crunii.micro.domain.repo.main.YcqYouthCommonFileRepo;
import com.crunii.micro.domain.repo.main.YcqYouthStartRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.platform.components.Locker;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.CommonFile;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqYouthStartDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqYouthStartUncheckDTO;
import com.crunii.micro.service.qzh.api.domain.dto.rightsProtection.YcqYouthStartVO;
import com.crunii.micro.service.qzh.jdbc.YcqYouthStartJdbcDao;
import com.crunii.micro.service.qzh.service.YcqYouthStartService;
import org.redisson.client.RedisClient;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author lmlksy
 * @date 2023/12/12 10:55
 */
@Service
public class YcqYouthStartServiceImpl implements YcqYouthStartService {

    @Autowired
    YcqYouthStartJdbcDao ycqYouthStartJdbcDao;

    @Autowired
    YcqYouthStartRepo ycqYouthStartRepo;

    @Autowired
    YcqYouthCommonFileRepo ycqYouthCommonFileRepo;

    @Autowired
    Locker locker;
    @Override
    public Result<PageRsp<YcqYouthStartVO>> pageList(PageReq<YcqYouthStartDTO> req) {
        PageRsp<YcqYouthStartVO> ycqYouthStartVOPageRsp = ycqYouthStartJdbcDao.queryList(req);
        if (!CollectionUtils.isEmpty(ycqYouthStartVOPageRsp.getResult())){
            for (YcqYouthStartVO ycqYouthStartVO : ycqYouthStartVOPageRsp.getResult()) {
                //id不为null 设置正文以及文件
                List<YcqYouthCommonFile> ycqYouthCommonFileList = ycqYouthCommonFileRepo.findByYcqYouthStartId(ycqYouthStartVO.getId());
                if (!CollectionUtils.isEmpty(ycqYouthCommonFileList)){
                    List<CommonFile> files=new ArrayList<>();
                    for (YcqYouthCommonFile ycqYouthCommonFile : ycqYouthCommonFileList) {
                        CommonFile commonFile = new CommonFile();
                        BeanUtils.copyProperties(ycqYouthCommonFile,commonFile);
                        files.add(commonFile);
                    }
                    ycqYouthStartVO.setFiles(files);
                }
            }

        }
        // 对查询结果进行重新排序
        List<YcqYouthStartVO> sortedList = ycqYouthStartVOPageRsp.getResult().stream()
                .sorted(Comparator.comparing(YcqYouthStartVO::getPriority, Comparator.reverseOrder()) // 优先级降序排序
                        .thenComparing(YcqYouthStartVO::getPublishTime, Comparator.reverseOrder())) // 时间降序排序
                .collect(Collectors.toList());



        ycqYouthStartVOPageRsp.setResult(sortedList);
        return Result.ofSuccess(ycqYouthStartVOPageRsp);
    }

    @Override
    public Result<String> update(YcqYouthStartDTO dto) {
        Worker worker = getWorker();
        if (dto.getId()==null) {
            throw new BusinessException("id不能为空！");
        }
        // 检查政策是否为下架状态，若是，则允许编辑
        Optional<YcqYouthStart> ycqYouthStart = ycqYouthStartRepo.findById(dto.getId());
        if (ycqYouthStart.isPresent()) {
            YcqYouthStart start = ycqYouthStart.get();
            if (start.getDataState()!=0){
                throw new BusinessException("只有下架状态的政策允许编辑！");
            }
            // 修改政策标题、发文号、政策层级、政策类型等信息
            start.setPolicyTitle(dto.getPolicyTitle());
            start.setIssueNumber(dto.getIssueNumber());
            start.setPolicyLevel(dto.getPolicyLevel());
            start.setDataState(dto.getDataState());
            start.setPolicyType(dto.getPolicyType());
            start.setPolicyText(dto.getPolicyText());
            start.setEditor(worker.getWorkerName());
            start.setEditorId(Long.valueOf(worker.getLoginId()));
            ycqYouthStartRepo.save(start);
            //修改图片
            if (!CollectionUtils.isEmpty(dto.getFiles())){
                ycqYouthCommonFileRepo.deleteByYcqYouthStartId(dto.getId());
                for (CommonFile file : dto.getFiles()) {
                    YcqYouthCommonFile ycqYouthCommonFile = new YcqYouthCommonFile();
                    BeanUtils.copyProperties(file,ycqYouthCommonFile);
                    ycqYouthCommonFile.setYcqYouthStartId(dto.getId());
                    ycqYouthCommonFileRepo.save(ycqYouthCommonFile);
                }
            }
            return Result.ofSuccessMsg("更新成功！");
        } else {
            return Result.ofFail("当前政策不存在！");
        }
    }

    @Override
    public Result<String> delete(YcqYouthStartDTO dto) {
        if (dto.getId()==null){
            throw new BusinessException("id不能为空！");
        }
        ycqYouthStartRepo.markAsDeletedById(dto.getId());
        ycqYouthCommonFileRepo.deleteByYcqYouthStartId(dto.getId());

        return Result.ofSuccessMsg("删除成功！");
    }

    @Override
    public Result<String> doPublicity(YcqYouthStartDTO dto) {
        if (dto.getId()==null){
            throw new BusinessException("id不能为空！");
        }
        if (dto.getDataState()==null){
            throw new BusinessException("公示状态不能为空！");
        }
        Optional<YcqYouthStart> ycqYouthStart = ycqYouthStartRepo.findById(dto.getId());
        if (ycqYouthStart.isPresent()) {
            YcqYouthStart start = ycqYouthStart.get();
            start.setDataState(dto.getDataState());
            ycqYouthStartRepo.save(start);

            return Result.ofSuccessMsg("操作成功！");
        } else {
            return Result.ofFail("当前政策不存在！");
        }
    }

    @Override
    public Result<String> doPriority(YcqYouthStartDTO dto) {

        String lockId = "priority_lock_" + dto.getId();
        try {
            locker.lock("priority", lockId);

            if (dto.getId() == null) {
                throw new BusinessException("id不能为空！");
            }
            if (dto.getIsTop() == null) {
                throw new BusinessException("置顶标识不能为空！");
            }
            Optional<YcqYouthStart> ycqYouthStart = ycqYouthStartRepo.findById(dto.getId());
            YcqYouthStart youthStart = ycqYouthStart.get();
            if (dto.getIsTop()==1){
                Integer maxPriority = ycqYouthStartRepo.findMaxPriority();
                if (maxPriority == 5) {
                    throw new BusinessException("最多置顶五条政策信息，请先取消其他置顶政策！");
                }
                if (youthStart.getPriority() > 0) {
                    throw new BusinessException("当前政策信息已经置顶！");
                }
                youthStart.setPriority(maxPriority + 1);

            }else {
                youthStart.setPriority(0);
                List<YcqYouthStart> topItems = ycqYouthStartRepo.findByPriorityNot(0);
                for (YcqYouthStart item : topItems) {
                    if (item.getId().equals(youthStart.getId())) {
                        continue;
                    }
                    int priority = item.getPriority();
                    if (priority > youthStart.getPriority()) {
                        item.setPriority(priority - 1);
                        ycqYouthStartRepo.save(item);
                    }
                }

            }
            ycqYouthStartRepo.save(youthStart);
            return Result.ofSuccessMsg("操作成功！");
        } finally {
            locker.unlock();
        }
    }

    @Override
    public Result<String> saveCreate(YcqYouthStartDTO dto) {
        Worker worker = getWorker();
        YcqYouthStart ycqYouthStart = new YcqYouthStart();
        ycqYouthStart.setPolicyTitle(dto.getPolicyTitle());
        ycqYouthStart.setIssueNumber(dto.getIssueNumber());
        ycqYouthStart.setPolicyLevel(dto.getPolicyLevel());
        ycqYouthStart.setPolicyType(dto.getPolicyType());
        ycqYouthStart.setPublisher(worker.getWorkerName());
        ycqYouthStart.setPublisherId(Long.valueOf(worker.getLoginId()));
        ycqYouthStart.setPublishTime(LocalDateTime.now());
        ycqYouthStart.setViewCount(0);
        ycqYouthStart.setDataState(dto.getDataState());
        ycqYouthStart.setIsDeleted(0);
        ycqYouthStart.setPolicyText(dto.getPolicyText());
        ycqYouthStart.setPriority(0);
        YcqYouthStart save = ycqYouthStartRepo.save(ycqYouthStart);

        if (!CollectionUtils.isEmpty(dto.getFiles())){
            List<YcqYouthCommonFile> ycqYouthCommonFiles = new ArrayList<>();
            for (CommonFile file : dto.getFiles()) {
                YcqYouthCommonFile ycqYouthCommonFile = new YcqYouthCommonFile();
                BeanUtils.copyProperties(file,ycqYouthCommonFile);
                ycqYouthCommonFile.setYcqYouthStartId(save.getId());
                ycqYouthCommonFiles.add(ycqYouthCommonFile);
            }
            ycqYouthCommonFileRepo.saveAll(ycqYouthCommonFiles);
        }
        return Result.ofSuccessMsg("新增成功！");
    }

    @Override
    public Result<YcqYouthStartVO> pageListUncheck(YcqYouthStartUncheckDTO dto) {
        if (dto.getId()==null){
           throw new BusinessException("id不能为空！") ;

        }
        YcqYouthStartVO ycqYouthStartVO = new YcqYouthStartVO();
        Optional<YcqYouthStart> byId = ycqYouthStartRepo.findById(dto.getId());
        if (byId.isPresent()){
            YcqYouthStart ycqYouthStart = byId.get();
            if (ycqYouthStart.getPublishTime()!=null){
                ycqYouthStartVO.setPublishTime(DateUtils.formatLocalDateTime(ycqYouthStart.getPublishTime()));
            }
            if (ycqYouthStart.getPublisherId()!=null){
                ycqYouthStartVO.setPublisherId(String.valueOf(ycqYouthStart.getPublisherId()));
            }
            BeanUtils.copyProperties(ycqYouthStart,ycqYouthStartVO);
            //id不为null 设置正文以及文件
            List<YcqYouthCommonFile> ycqYouthCommonFileList = ycqYouthCommonFileRepo.findByYcqYouthStartId(ycqYouthStart.getId());
            if (!CollectionUtils.isEmpty(ycqYouthCommonFileList)){
                List<CommonFile> files=new ArrayList<>();
                for (YcqYouthCommonFile ycqYouthCommonFile : ycqYouthCommonFileList) {
                    CommonFile commonFile = new CommonFile();
                    BeanUtils.copyProperties(ycqYouthCommonFile,commonFile);
                    files.add(commonFile);
                }
                ycqYouthStartVO.setFiles(files);
            }

        }
        return Result.ofSuccess(ycqYouthStartVO);
    }

    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException("获取当前登录用户失败");
        }
        return worker;
    }
}
