package com.crunii.micro.service.qzh.service.impl;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.TopicImportReqDTO;
import com.crunii.micro.service.qzh.api.enums.TopicTypeEnum;
import com.crunii.micro.service.qzh.dto.*;
import com.crunii.micro.service.qzh.jdbc.YcqChooseTopicJdbcDao;
import com.crunii.micro.service.qzh.jdbc.YcqTopicJdbcDao;
import com.crunii.micro.service.qzh.jdbc.YcqYzyhTestCountJdbcDao;
import com.crunii.micro.service.qzh.jdbc.YcqYzyhTestJdbcDao;
import com.crunii.micro.service.qzh.service.YcqYzyhTestService;
import com.crunii.micro.service.qzh.util.CustomImageModifyHandler;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;


import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qzh.api.constants.QzhConstants.*;

/**
 * @BelongsProject: microservice
 * @BelongsPackage: com.crunii.micro.service.qzh.service.impl
 * @Author: limy
 * @CreateTime: 2023-09-05  14:25
 * @Description: 应知应会考试
 * @Version: 1.0
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class YcqYzyhTestServiceImpl implements YcqYzyhTestService {
    @Resource
    private YcqYzyhTestRepo yzyhTestRepo;
    @Resource
    private YcqYzyhTopicRelRepo ycqYzyhTopicRelRepo;
    @Resource
    private YcqYzyhTestJdbcDao ycqYzyhTestJdbcDao;
    @Resource
    private YcqYzyhTestCountJdbcDao ycqYzyhTestCountJdbcDao;
    @Resource
    private YcqChooseTopicJdbcDao ycqChooseTopicJdbcDao;
    @Resource
    private YcqYzyhTestFileRepo ycqYzyhTestFileRepo;
    @Resource
    private YcqTopicRepo ycqTopicRepo;
    @Resource
    private YcqTopicJdbcDao ycqTopicJdbcDao;
    @Resource
    private YcqTopicOptionRepo ycqTopicOptionRepo;

    private  ConcurrentHashMap<String,Object> chmap=new ConcurrentHashMap<>();

    private static final String[] MULTIPLE_OPTION = new String[]{"A","B","C","D","AB","AC","AD","BC","BD","CD","ABC","ABD","ACD","BCD","ABCD"};



    /**
     * @description: 添加试卷
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Result<String> addTest(AddYcqYzyhTestReqDto addYcqYzyhTestReqDto,Worker worker) {
       // Worker worker = CurrWorkerUtil.getCurrWorker();
        //不是批量导入题目
        if(ObjectUtil.isNotEmpty(addYcqYzyhTestReqDto)&&!addYcqYzyhTestReqDto.getTopicType().equals(3)){
            List<Integer> topicList = addYcqYzyhTestReqDto.getTopicList();
            List<Integer> topicIds = topicList.stream().collect(Collectors.toList());
            //题目是否在题库中存在
            List<YcqTopic> topics = ycqTopicRepo.findAllById(topicIds);
            List<Integer> deleteds = topics.stream().map(e -> e.getDeleted()).filter(f -> f.equals(1)).collect(Collectors.toList());
            if(CollectionUtils.isEmpty(topics)||ObjectUtil.isNotEmpty(deleteds)||topics.size()<topicIds.size()){
                return Result.ofFail("有不存在的题目，请重新选择!");
            }
            YcqYzyhTest ycqYzyhTest = new YcqYzyhTest();
            BeanUtils.copyProperties(addYcqYzyhTestReqDto,ycqYzyhTest);
            List<YcqYzyhTest> ycqYzyhTests = yzyhTestRepo.findbyTitle(addYcqYzyhTestReqDto.getTitle());
            if(!CollectionUtils.isEmpty(ycqYzyhTests)){
                return Result.ofFail("已有相同名称的试卷，请检查!");
            }
            Snowflake snowflake = new Snowflake();
            ycqYzyhTest.setId(String.valueOf(snowflake.nextId()));
            ycqYzyhTest.setPublishTime(LocalDateTime.now());
            // ycqYzyhTest.setPublishWorkerId(String.valueOf(worker.getLoginId()));
            ycqYzyhTest.setPublishWorkerId("111");
            // ycqYzyhTest.setPublishWorkerName(worker.getWorkerName());
            ycqYzyhTest.setPublishWorkerName("22");
            String pattern="yyyy-MM-dd HH:mm";
            Date start = DateUtil.parse(addYcqYzyhTestReqDto.getTestTime().get(0), pattern);
            LocalDateTime startTime=LocalDateTime.ofInstant(start.toInstant(), ZoneId.systemDefault());
            Date end = DateUtil.parse(addYcqYzyhTestReqDto.getTestTime().get(1), pattern);
            LocalDateTime endTime=LocalDateTime.ofInstant(end.toInstant(), ZoneId.systemDefault());
            if(LocalDateTime.now().isAfter(startTime)||LocalDateTime.now().isAfter(endTime)){
                return Result.ofFail("不能选择过去的时间!");
            }
            ycqYzyhTest.setStartTime(startTime);
            ycqYzyhTest.setEndTime(endTime);
            ycqYzyhTest.setCreatedBy(111);
            ycqYzyhTest.setCreatedTime(LocalDateTime.now());
            ycqYzyhTest.setDeleted(0);
            ycqYzyhTest.setState(0);

            YcqYzyhTest save = yzyhTestRepo.save(ycqYzyhTest);
            this.setSchelude(save);

            List<YcqYzyhTestFile> list = new ArrayList<>();
            addYcqYzyhTestReqDto.getFileIds().forEach(s -> {
                list.add(YcqYzyhTestFile.builder().testId(save.getId()).fileId(s).createId(Long.valueOf(worker.getLoginId()))
                        .createBy(worker.getWorkerName()).createTime(LocalDateTime.now()).build());
            });
            if (list != null && list.size() > 0) {
                ycqYzyhTestFileRepo.saveAll(list);
            }

            //存入考试与题目关联表
            ArrayList<YcqYzyhTopicRel> ycqYzyhTopicRels = new ArrayList<>();
            for (Integer topicId : topicList) {
                YcqYzyhTopicRel ycqYzyhTopicRel = new YcqYzyhTopicRel();
                ycqYzyhTopicRel.setTestId(save.getId());
                ycqYzyhTopicRel.setTopicId(topicId);
                ycqYzyhTopicRel.setCreatedBy(333);
                ycqYzyhTopicRel.setCreatedTime(LocalDateTime.now());
                ycqYzyhTopicRels.add(ycqYzyhTopicRel);
            }
            ycqYzyhTopicRelRepo.saveAll(ycqYzyhTopicRels);

        }
        return new Result<>(CREATE_SUCCESS);
    }

    /**
     * @description: 抽取题目
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    @Override
    public Result<List<TestTopicRespDto>> chooseTopic(ChooseTopicReqDto chooseTopicReqDto,Worker worker) {
        List<TestTopicRespDto> testTopicRespDtos = ycqChooseTopicJdbcDao.queryList(chooseTopicReqDto);
        return Result.ofSuccess(testTopicRespDtos);
    }

    /**
     * @description: 删除考试
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    @Override
    public Result<String> delTest(List<String> requestBody,Worker worker) {
        //Worker worker = CurrWorkerUtil.getCurrWorker();
        ycqYzyhTestJdbcDao.deleteByIds(requestBody,worker.getLoginId(),worker.getWorkerName());
        return new Result<>(OPERATION_SUCCESS);
    }

    /**
     * @description: 操作考试
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    @Override
    public Result<String> doTest(OperateTestRepDto operateTestRepDto,Worker worker) {
        //Worker worker = CurrWorkerUtil.getCurrWorker();
        YcqYzyhTest yzyhTest = yzyhTestRepo.findById(operateTestRepDto.getTestId()).orElse(null);
        if(ObjectUtil.isEmpty(yzyhTest)){
            return Result.ofFail("该考试不存在!");
        }
        //开始
        if(ObjectUtil.isNotEmpty(operateTestRepDto.getOperate())&&operateTestRepDto.getOperate().equals(1)){
            if(LocalDateTime.now().isAfter(yzyhTest.getStartTime())){
                return Result.ofFail("考试已开始!");
            }
            this.cancelSchelude(yzyhTest.getId()+"start");
        }
        //结束
        if(ObjectUtil.isNotEmpty(operateTestRepDto.getOperate())&&operateTestRepDto.getOperate().equals(2)){
            if(LocalDateTime.now().isAfter(yzyhTest.getStartTime())){
                return Result.ofFail("考试已结束!");
            }
            this.cancelSchelude(yzyhTest.getId()+"end");
        }
        ycqYzyhTestJdbcDao.operateTest(operateTestRepDto.getOperate(),worker.getLoginId(),worker.getWorkerName(),yzyhTest.getId());

        return new Result<>(OPERATION_SUCCESS);
    }

    /**
     * @description: 考试情况统计分页查询
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    @Override
    public Result<PageRsp<YcqYzyhTestCountRespDto>> examDetailsCount(PageReq<YcqYzyhTestCountReqDto> ycqYzyhTestCountReqDto,Worker worker) {
        PageRsp<YcqYzyhTestCountRespDto> ycqYzyhTestCountRespDtoPageRsp = ycqYzyhTestCountJdbcDao.examDetailsCount(ycqYzyhTestCountReqDto);
        return Result.ofSuccess(ycqYzyhTestCountRespDtoPageRsp);
    }

    /**
     * @description: 考试情况统计导出
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    @Override
    public void examDetailsCountExport(HttpServletResponse response,YcqYzyhTestCountReqDto ycqYzyhTestCountReqDto,Worker worker) throws IOException {
        List<YcqYzyhTestCountRespDto> list = ycqYzyhTestCountJdbcDao.examDetailsCountExport(ycqYzyhTestCountReqDto);
        // HttpServletResponse response =((ServletWebRequest) RequestContextHolder.getRequestAttributes()).getResponse();
        String fileName = null;
        try {
            fileName = URLEncoder.encode("考试情况统计", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
        // 写入数据
        EasyExcel.write(response.getOutputStream(), YcqYzyhTestCountRespDto.class).registerWriteHandler(new CustomImageModifyHandler()).
                sheet("sheet").doWrite(list);

    }

    /**
     * @description: 考试列表分页查询
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    @Override
    public Result<PageRsp<YcqYzyhTestRespDto>> pageTest(PageReq<YcqYzyhTestReqDto> ycqYzyhTestReqDto,Worker worker) {
        PageRsp<YcqYzyhTestRespDto> ycqYzyhTestRespDtoPageRsp = ycqYzyhTestJdbcDao.pageList(ycqYzyhTestReqDto);
        for (YcqYzyhTestRespDto ycqYzyhTestRespDto : ycqYzyhTestRespDtoPageRsp.getResult()) {
            ycqYzyhTestRespDto.setTopicList(ycqYzyhTestJdbcDao.getTestTopics(ycqYzyhTestRespDto.getId()));
            ycqYzyhTestRespDto.setCountScore(ycqYzyhTestRespDto.getPassScore()+"/"+ycqYzyhTestRespDto.getTotalScore());
            ycqYzyhTestRespDto.setTestTime(ycqYzyhTestRespDto.getStartTime()+"至"+ycqYzyhTestRespDto.getEndTime());
        }
        return Result.ofSuccess(ycqYzyhTestRespDtoPageRsp);
    }

    /**
     * @description: 批量导入题目
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    @Override
    public Result<List<TestTopicRespDto>> topicsImport( MultipartFile file,Integer questionBankId,Worker worker) throws IOException {

        // 通过EasyExcel工具读取excel表中内容
//        EasyExcel.read(file.getInputStream(), TopicImportReqDTO.class, new TopicImportListener(this, file, questionBankId, worker))
//                .headRowNumber(1).sheet().doReadSync();


        try {
            List<TopicImportReqDTO> list = EasyExcel.read(file.getInputStream(), TopicImportReqDTO.class, null)
                    .ignoreEmptyRow(true).sheet(2).doReadSync();
            Result<List<TopicImportReqDTO>> listResult = saveData(list, questionBankId);
            List<TopicImportReqDTO> erroList = listResult.getData();
            if (erroList.size()==0) {
                return Result.ofSuccess();
            }
            File outputFile = new File("C:\\Users\\ASUS\\Desktop\\test.xlsx");
            EasyExcel.write(outputFile, TopicImportReqDTO.class).sheet().doWrite(list);
//            return Result.ofSuccessWithMsg(erroList,listResult.getMsg());
        } catch (IOException e) {
            log.error("导入失败信息:"+e);
            return Result.ofFail("导入失败,请联系管理员");
        }
        return null;
    }

    private Result<List<TopicImportReqDTO>> saveData(List<TopicImportReqDTO> con,Integer questionBankId) {
        Integer totalSize = con.size(), successSize = 0, failSize = 0;
        List<TopicImportReqDTO> erro = new ArrayList<>();
        List<TopicImportReqDTO> correct = new ArrayList<>();
        List<String> topics = con.stream().map(TopicImportReqDTO::getTopic).distinct().collect(Collectors.toList());
        Map<String,Object> topicMap= ycqTopicJdbcDao.selectAllTopicForQuestionBankId(topics,questionBankId);
        for (TopicImportReqDTO topicImportReqDTO : con) {
            if (StringUtils.isNotBlank(topicImportReqDTO.getTopic())) {
                failSize++;
                topicImportReqDTO.setImportErroCause("题目不能为空");
                erro.add(topicImportReqDTO);
                continue;
            }
            if (StringUtils.isNotBlank(topicImportReqDTO.getAnswer())) {
                failSize++;
                topicImportReqDTO.setImportErroCause("答案不能为空");
                erro.add(topicImportReqDTO);
                continue;
            }
            if (topicMap.containsKey(topicImportReqDTO.getTopic())) {
                failSize++;
                topicImportReqDTO.setImportErroCause("题目在该题库中已经存在");
                erro.add(topicImportReqDTO);
                continue;
            }
            if (!StringUtils.containsAny(topicImportReqDTO.getTopicType(), TopicTypeEnum.Judgment.getName(),TopicTypeEnum.Single.getName(),TopicTypeEnum.Mulity.getName())) {
                failSize++;
                topicImportReqDTO.setImportErroCause("题目类型不存在");
                erro.add(topicImportReqDTO);
                continue;
            }
            //单选题的校验
            if (TopicTypeEnum.Single.getName().equals(topicImportReqDTO.getTopicType())) {
                if (!StringUtils.containsAny(topicImportReqDTO.getAnswer(),"A","B","C","D")) {
                    failSize++;
                    topicImportReqDTO.setImportErroCause("答案类型不正确");
                    erro.add(topicImportReqDTO);
                    continue;
                }
                if (topicImportReqDTO.getOptionA().isEmpty()
                        || topicImportReqDTO.getOptionB().isEmpty()
                        || topicImportReqDTO.getOptionC().isEmpty()
                        || topicImportReqDTO.getOptionD().isEmpty()) {
                    failSize++;
                    topicImportReqDTO.setImportErroCause("ABCD选项不能存在空选项");
                    erro.add(topicImportReqDTO);
                    continue;
                }
            }
            //多选题的校验
            if (TopicTypeEnum.Mulity.getName().equals(topicImportReqDTO.getTopicType())) {
                if (StringUtils.containsAny(topicImportReqDTO.getAnswer(),MULTIPLE_OPTION)) {
                    failSize++;
                    topicImportReqDTO.setImportErroCause("答案类型不正确");
                    erro.add(topicImportReqDTO);
                    continue;
                }
                if (topicImportReqDTO.getOptionA().isEmpty()
                        || topicImportReqDTO.getOptionB().isEmpty()
                        || topicImportReqDTO.getOptionC().isEmpty()
                        || topicImportReqDTO.getOptionD().isEmpty()) {
                    failSize++;
                    topicImportReqDTO.setImportErroCause("ABCD选项不能存在空选项");
                    erro.add(topicImportReqDTO);
                    continue;
                }
            }
            //判断题的校验
            if (TopicTypeEnum.Judgment.getName().equals(topicImportReqDTO.getTopicType())) {
                if (StringUtils.containsAny(topicImportReqDTO.getAnswer(),"A","B")) {
                    failSize++;
                    topicImportReqDTO.setImportErroCause("答案类型不正确");
                    erro.add(topicImportReqDTO);
                    continue;
                }
                if (topicImportReqDTO.getOptionA().isEmpty() || topicImportReqDTO.getOptionB().isEmpty()) {
                    failSize++;
                    topicImportReqDTO.setImportErroCause("AB选项不能存在空选项");
                    erro.add(topicImportReqDTO);
                    continue;
                }
            }
            correct.add(topicImportReqDTO);

            if (correct.size()>=1000) {
                successSize += correct.size();
                SaveTopic(correct,questionBankId);
                correct.clear();
            }
        }
        String msg = "导入成功,总数:" + totalSize + "条,成功:" + successSize + "失败:" + failSize+"条";
        return Result.ofSuccessWithMsg(erro,msg);
    }

    public void SaveTopic(List<TopicImportReqDTO> correct,Integer questionBankId) {
        List<YcqTopic> list = new ArrayList<>();
        HashMap<Integer,List<YcqTopicOption>> optionMap = new HashMap<>();
        for (int i = 0; i < correct.size(); i++) {
            TopicImportReqDTO importReqDTO = correct.get(i);
            YcqTopic ycqTopic = new YcqTopic();
            ycqTopic.setDeleted(0);
            ycqTopic.setTopicType(TopicTypeEnum.findCodeByVal(importReqDTO.getTopicType()));
            ycqTopic.setQuestionBankId(questionBankId);
            ycqTopic.setRemark(importReqDTO.getAttr());
            list.add(ycqTopic);
            List<YcqTopicOption> options = new ArrayList<>();
            for(int j = 0; j<4; j++) {
                YcqTopicOption ycqTopicOption = new YcqTopicOption();
                ycqTopicOption.setOption(matchOption(j));
                ycqTopicOption.setOptionContent(importReqDTO.getOptionA());
                ycqTopicOption.setIsAnswer(importReqDTO.getAnswer().contains(ycqTopicOption.getOption())?1:0);
                options.add(ycqTopicOption);
            }
            optionMap.put(i,options);
        }
        List<YcqTopic> ycqTopics = ycqTopicRepo.saveAllAndFlush(list);
        for (int i = 0; i < ycqTopics.size(); i++) {
            YcqTopic ycqTopic = ycqTopics.get(i);
            List<YcqTopicOption> options = optionMap.get(i);
            for (YcqTopicOption option : options) {
                option.setTopicId(ycqTopic.getId());
            }
        }

        List<YcqTopicOption> optionSave = optionMap.values().stream()
                .flatMap(List::stream)
                .collect(Collectors.toList());
        ycqTopicOptionRepo.saveAllAndFlush(optionSave);
    }


    private String matchOption(int id) {
        HashMap<Integer, Character> matchMap = new HashMap<>(24);
        for (int i = 0; i < 25; i++) {
            matchMap.put(i, (char) ('A' + i));
        }
        return String.valueOf(matchMap.get(id));
    }

    /**
     * @description: 编辑试卷
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    @Override
    public Result<String> updateTest(AddYcqYzyhTestReqDto addYcqYzyhTestReqDto,Worker worker) {
        YcqYzyhTest ycqYzyhTest = yzyhTestRepo.findById(addYcqYzyhTestReqDto.getId()).orElse(null);
        if(ObjectUtil.isNotEmpty(ycqYzyhTest)){
            return Result.ofFail("该试卷不存在!");
        }
        if(!ycqYzyhTest.getState().equals(0)){
            return Result.ofFail("该试卷不能编辑!");
        }

       // Worker worker = CurrWorkerUtil.getCurrWorker();
        //不是批量导入题目
        if(ObjectUtil.isNotEmpty(addYcqYzyhTestReqDto)&&!addYcqYzyhTestReqDto.getTopicType().equals(3)){
            List<Integer> topicList = addYcqYzyhTestReqDto.getTopicList();
            List<Integer> topicIds = topicList.stream().collect(Collectors.toList());
            //题目是否在题库中存在
            List<YcqTopic> topics = ycqTopicRepo.findAllById(topicIds);
            List<Integer> deleteds = topics.stream().map(e -> e.getDeleted()).filter(f -> f.equals(1)).collect(Collectors.toList());
            if(CollectionUtils.isEmpty(topics)||ObjectUtil.isNotEmpty(deleteds)||topics.size()<topicIds.size()){
                return Result.ofFail("有不存在的题目，请重新选择!");
            }

            List<YcqYzyhTest> ycqYzyhTests = yzyhTestRepo.findbyTitle(addYcqYzyhTestReqDto.getTitle());
            if(!CollectionUtils.isEmpty(ycqYzyhTests)){
                return Result.ofFail("已有相同名称的试卷，请检查!");
            }
            this.cancelSchelude(ycqYzyhTest.getId()+"start");
            this.cancelSchelude(ycqYzyhTest.getId()+"end");
            BeanUtils.copyProperties(addYcqYzyhTestReqDto,ycqYzyhTest);

            String pattern="yyyy-MM-dd HH:mm";
            Date start = DateUtil.parse(addYcqYzyhTestReqDto.getTestTime().get(0), pattern);
            LocalDateTime startTime=LocalDateTime.ofInstant(start.toInstant(), ZoneId.systemDefault());
            Date end = DateUtil.parse(addYcqYzyhTestReqDto.getTestTime().get(1), pattern);
            LocalDateTime endTime=LocalDateTime.ofInstant(end.toInstant(), ZoneId.systemDefault());
            ycqYzyhTest.setStartTime(startTime);
            ycqYzyhTest.setEndTime(endTime);
            ycqYzyhTest.setUpdatedBy(worker.getLoginId());
            ycqYzyhTest.setUpdatedTime(LocalDateTime.now());
            ycqYzyhTest.setState(0);

            YcqYzyhTest save = yzyhTestRepo.save(ycqYzyhTest);
            this.setSchelude(save);

            ycqYzyhTestFileRepo.deteleByTestId(save.getId());
            Date date = new Date();
            List<YcqYzyhTestFile> list =new ArrayList<>();
            addYcqYzyhTestReqDto.getFileIds().forEach(s -> {
                list.add(YcqYzyhTestFile.builder().testId(save.getId()).fileId(s).updateId(Long.valueOf(worker.getLoginId()))
                        .updateBy(worker.getWorkerName()).createTime(LocalDateTime.now()).build());
            });
            if (list != null && list.size() > 0) {
                ycqYzyhTestFileRepo.saveAll(list);
            }

            //存入考试与题目关联表
            ycqYzyhTopicRelRepo.deteleByTestId(save.getId());
            ArrayList<YcqYzyhTopicRel> ycqYzyhTopicRels = new ArrayList<>();
            for (Integer topicId : topicList) {
                YcqYzyhTopicRel ycqYzyhTopicRel = new YcqYzyhTopicRel();
                ycqYzyhTopicRel.setTestId(save.getId());
                ycqYzyhTopicRel.setTopicId(topicId);
                ycqYzyhTopicRels.add(ycqYzyhTopicRel);
            }
            ycqYzyhTopicRelRepo.saveAll(ycqYzyhTopicRels);

        }
        return new Result<>(UPDATE_SUCCESS);
    }

    /**
     * @description: 查看试卷
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    @Override
    public Result<TestDetailRespDto> viewTest(String id,Worker worker) {
        TestDetailRespDto testDetailRespDtos = ycqYzyhTestJdbcDao.viewTest(id);
        testDetailRespDtos.setTopicList(ycqYzyhTestJdbcDao.getTestTopics(id));
        return Result.ofSuccess(testDetailRespDtos);
    }


    /**
     * @description: 设置定时任务更改考试状态
     * @author: limy
     * @date: 2023/9/5 14:59
     **/
    public void setSchelude(YcqYzyhTest ycqYzyhTest){
        TimerTask startTask = new TimerTask() {
            @Override
            public void run() {
                YcqYzyhTest yzyhTest = yzyhTestRepo.findById(ycqYzyhTest.getId()).orElse(null);
                yzyhTest.setState(1);
                yzyhTestRepo.save(yzyhTest);

            }
        };
        TimerTask endTask = new TimerTask() {
            @Override
            public void run() {
                YcqYzyhTest yzyhTest = yzyhTestRepo.findById(ycqYzyhTest.getId()).orElse(null);
                yzyhTest.setState(2);
                yzyhTestRepo.save(yzyhTest);

            }
        };

        Timer timer1 = new Timer("Timer1");
        if(!chmap.containsKey(ycqYzyhTest.getId()+"start")){
            chmap.put(ycqYzyhTest.getId()+"start",timer1);
        }
        Timer timer2 = new Timer("Timer2");
        if(!chmap.containsKey(ycqYzyhTest.getId()+"end")){
            chmap.put(ycqYzyhTest.getId()+"end",timer2);
        }
        timer1.schedule(startTask, Date.from(ycqYzyhTest.getStartTime().atZone(ZoneId.systemDefault()).toInstant()));
        timer2.schedule(endTask, Date.from(ycqYzyhTest.getEndTime().atZone(ZoneId.systemDefault()).toInstant()));
    }

    /**
     * @description: 取消定时任务
     * @author: limy
     * @date: 2023/9/7 16:13
     * @param: [key]
     * @return: void
     **/
    public void cancelSchelude(String key){
        Timer timer = (Timer) chmap.get(key);
        timer.cancel();
        chmap.remove(key,timer);
    }
}
