package com.crunii.micro.service.qzh.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.QueryOrgDto;
import com.crunii.micro.service.qzh.api.domain.dto.cavalryTrumpet.ReportOrgDto;
import com.crunii.micro.service.qzh.api.domain.dto.youthSafetyModel.*;
import com.crunii.micro.service.qzh.jdbc.QagApplyLeaderChangeLogDao;
import com.crunii.micro.service.qzh.jdbc.YcqQzhQagApplyJdbcDao;
import com.crunii.micro.service.qzh.service.CivilizationReportService;
import com.crunii.micro.service.qzh.service.YouthSafetyModelRecommendService;
import com.crunii.micro.service.qzh.utils.PDFAddWatermark;
import com.crunii.module.flow.api.FlowInstanceApi;
import com.crunii.module.flow.api.vo.FlowInstanceReqVO;
import com.crunii.module.flow.api.vo.conf.FlowConfRespVO;
import com.crunii.module.flow.api.vo.pending.CruniiFlowProcessHistoryRespVO;
import com.crunii.module.flow.dto.*;
import com.didiglobal.turbo.engine.model.InstanceData;
import com.didiglobal.turbo.engine.result.RollbackTaskResult;
import com.itextpdf.text.*;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.compress.utils.Sets;
import org.assertj.core.util.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static com.crunii.micro.service.qzh.api.constants.QzhConstants.WORKER_NOT_EXISTS;

/**
 * @Author: zwx
 * @CreateTime: 2023-11-13  16:20:16
 * @Description: 青年安全生产示范岗推荐申报
 */
@Service
public class YouthSafetyModelRecommendServiceImpl implements YouthSafetyModelRecommendService {

    @Resource
    YouthSafetyModelRecommendRepo youthSafetyModelRecommendRepo;

    @Resource
    YouthSafetyRecommendFileRepo youthSafetyRecommendFileRepo;

    @Resource
    YcqQzhQagApplyRepo ycqQzhQagApplyRepo;

    @Resource
    QagApplyLeaderChangeLogRepo qagApplyLeaderChangeLogRepo;

    @Autowired
    QagApplyLeaderChangeLogDao jdbc;

    @Autowired
    FlowInstanceApi flowInstanceApi;

    @Resource
    TsmOrgRepo tsmOrgRepo;

    @Resource
    TsmWorkerRepo tsmWorkerRepo;

    @Resource
    TsmMenuRepo tsmMenuRepo;

    @Autowired
    YcqQzhQagApplyJdbcDao jdbcDao;

    @Resource
    private CivilizationReportService civilizationReportService;

    @Override
    public Result createRecommend(YouthSafetyModelRecommendReqDTO dto) {

        Worker currWorker = getWorker();


        Optional<YcqQzhQagApply> options = ycqQzhQagApplyRepo.findById(dto.getQagApplyId());
        if (!options.isPresent()) {
            throw new BusinessException("申报数据不存在");
        }

//        List<YcqQzhQagApply> idCardCheck =  ycqQzhQagApplyRepo.findAllByLeaderIdCardAndDelFlagAndFlowStateNot(dto.getLeaderIdCard(),0,8);

        YcqQzhQagApply ycqQzhQagApply = options.get();

        YouthSafetyModelRecommend qagApply = youthSafetyModelRecommendRepo.findByQagApplyIdAndIsDraft(dto.getQagApplyId(),0);
        if (qagApply!=null) {
            throw new BusinessException("已经存在推荐申报");
        }
        YouthSafetyModelRecommend drart = youthSafetyModelRecommendRepo.findByQagApplyIdAndIsDraft(dto.getQagApplyId(),1);
        if (dto.getIsDraft().equals(1)) {
            //保存为草稿
            YouthSafetyModelRecommend youthSafetyModelRecommend = voToEntity(dto, currWorker,ycqQzhQagApply,drart);
            YouthSafetyModelRecommend saveRecommend = youthSafetyModelRecommendRepo.save(youthSafetyModelRecommend);
            //删除之前的草稿附件
            youthSafetyRecommendFileRepo.deleteByRecommendId(saveRecommend.getId());
            //保存附件
            saveFile(dto,saveRecommend.getId());
        } else {
//            if (CollectionUtils.isNotEmpty(idCardCheck)) {
//                throw new BusinessException("同一岗长身份证号只能存在一个申报中");
//            }
            dataCheck(dto);
            fileCheck(dto);
            FlowCommitDTO flowdto = new FlowCommitDTO();
            flowdto.setFlowInstanceId(ycqQzhQagApply.getFlowInstanceId());
            flowdto.setTaskInstanceId(ycqQzhQagApply.getNodeInstanceId());
            //设置提交人
            FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
            commitor.setOrgId(Long.parseLong(currWorker.getOrgId()));
            commitor.setOrgName(currWorker.getOrgName());
            commitor.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
            commitor.setUserName(currWorker.getWorkerName());
            flowdto.setCommitor(commitor);

            //处理结果处理意见
            flowdto.setProcessResult("");
            flowdto.setProcessComments("");
            //设置流程变量后面判断
            Map<String, Object> map = new HashMap<>();
            map.put("state", 1);
            flowdto.setFlowVar(map);
            flowdto.setCommitFlow(true);

            //特殊新增
            //设置组织
            FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
            ArrayList<IdNameDTO> orgs = new ArrayList<>();
            IdNameDTO idNameDTO = new IdNameDTO();
            idNameDTO.setId(Long.valueOf(ycqQzhQagApply.getAffiliationOrgId()));
            idNameDTO.setName(ycqQzhQagApply.getAffiliationOrgName());
            orgs.add(idNameDTO);
            addedTaskAssigenn.setAssigennDept(orgs);
            flowdto.setAddedTaskAssigenn(addedTaskAssigenn);


            CommonResult<CruniiCommitTaskResult> startResult = flowInstanceApi.commitProcess(flowdto);
            if (startResult.getCode() == 00 && startResult.getData().getErrCode() < 2000) {
                String nodeInstanceId = startResult.getData().getActiveTaskInstance().getNodeInstanceId();
                ycqQzhQagApply.setNodeInstanceId(nodeInstanceId);
                ycqQzhQagApply.setFlowState(ycqQzhQagApply.getFlowState()+1);
                ycqQzhQagApply.setUpdateTime(new Date());
                ycqQzhQagApply.setUpdator(currWorker.getWorkerName());

                YouthSafetyModelRecommend youthSafetyModelRecommend = voToEntity(dto, currWorker,ycqQzhQagApply,drart);
                YouthSafetyModelRecommend saveRecommend = youthSafetyModelRecommendRepo.save(youthSafetyModelRecommend);
                //删除之前的草稿附件
                youthSafetyRecommendFileRepo.deleteByRecommendId(saveRecommend.getId());
                //保存附件
                saveFile(dto,saveRecommend.getId());
                ycqQzhQagApplyRepo.save(ycqQzhQagApply);

            } else {
                return Result.ofFail("流程执行失败");
            }

            //审核下一步开始结束
            String oldNodeKeys = ycqQzhQagApply.getOldNodeKeys() == null ? "" : ycqQzhQagApply.getOldNodeKeys();
            String modelKey = startResult.getData().getActiveTaskInstance().getModelKey();
            ycqQzhQagApply.setOldNodeKeys(oldNodeKeys + "," + modelKey);

        }
        return Result.ofSuccessMsg("操作成功");
    }

    private void saveFile(YouthSafetyModelRecommendReqDTO dto,Long id) {
        List<YouthSafetyRecommendFile> fileList = new ArrayList<>();
        dto.getFiles().forEach(e -> {
            YouthSafetyRecommendFile item = new YouthSafetyRecommendFile();
            BeanUtils.copyProperties(e,item);
            item.setRecommendId(id);
            item.setCreateTime(LocalDateTime.now());
            fileList.add(item);
        });
        youthSafetyRecommendFileRepo.saveAll(fileList);
    }

    private void dataCheck(YouthSafetyModelRecommendReqDTO dto) {
        if (dto.getMajorAchievement().replaceAll("<[^>]*>","").length()>500) {
            throw new BusinessException("主要事迹不能大于500个字符");
        }
        checkIdCard(dto.getLeaderIdCard());
        if (dto.getWorkersNum()<3 || dto.getWorkersNum() >200) {
            throw new BusinessException("职工人数需大于等于3人，小于等于200人");
        }
        if ((double) dto.getYouthNum() /dto.getWorkersNum() < 0.5) {
            throw new BusinessException("35岁以下青年人数占比需大于等于50%");
        }
    }

    private void fileCheck(YouthSafetyModelRecommendReqDTO dto) {
        if (dto.getFiles()==null) {
            throw new BusinessException("盖章版扫描件必传");
        }
        //tozwx 校验文件数量
        long noStampCount = dto.getFiles().stream().filter(e -> e.getIsStamp() != 1).count();
        if (noStampCount > 5){
            throw new BusinessException("其它附件数需小于5个");
        }
        long stampCount = dto.getFiles().stream().filter(e -> e.getIsStamp() == 1).count();
        if (stampCount != 1){
            throw new BusinessException("盖章扫描件需传1个附件");
        }
        dto.getFiles().forEach(e->{
            //tozwx 校验文件大小
            if (e.getFileSize()>2048) {
                throw new BusinessException("附件大小需小于2M");
            }
            if (e.getIsStamp()==1 && !"pdf".equals(e.getFileType())) {
                throw new BusinessException("盖章扫描件只能传pdf格式");
            }
            if (e.getIsStamp()==0 && !Lists.newArrayList("docx","doc","xls","xlsx","pdf").contains(e.getFileType())) {
                throw new BusinessException("其他附件只能传word、excel、pdf文档");
            }
        });

    }

    private YouthSafetyModelRecommend voToEntity(YouthSafetyModelRecommendReqDTO dto,Worker worker,YcqQzhQagApply ycqQzhQagApply,YouthSafetyModelRecommend drart) {

        YouthSafetyModelRecommend youthSafetyModelRecommend = new YouthSafetyModelRecommend();
        youthSafetyModelRecommend.setId(Objects.nonNull(drart)?drart.getId():null);
        youthSafetyModelRecommend.setApplyOrgName(ycqQzhQagApply.getApplyOrgName());
        youthSafetyModelRecommend.setAffiliationOrgId(ycqQzhQagApply.getAffiliationOrgId());
        youthSafetyModelRecommend.setAffiliationOrgName(tsmOrgRepo.findOrgById(dto.getAffiliationOrgId()).getOrgName());
        youthSafetyModelRecommend.setCode(ycqQzhQagApply.getCode());
        youthSafetyModelRecommend.setLeaderIdCard(ycqQzhQagApply.getLeaderIdCard());
        youthSafetyModelRecommend.setLeaderName(ycqQzhQagApply.getLeaderName());
        youthSafetyModelRecommend.setLeaderPhone(ycqQzhQagApply.getLeaderPhone());
        youthSafetyModelRecommend.setWorkersNum(ycqQzhQagApply.getWorkersNum());
        youthSafetyModelRecommend.setYouthNum(ycqQzhQagApply.getYouthNum());
        youthSafetyModelRecommend.setRemark(ycqQzhQagApply.getRemark());
        youthSafetyModelRecommend.setCreator(worker.getWorkerName());
        youthSafetyModelRecommend.setCreateId((long)worker.getLoginId());
        youthSafetyModelRecommend.setCreateTime(LocalDateTime.now());
        youthSafetyModelRecommend.setUpdator(worker.getWorkerName());
        youthSafetyModelRecommend.setUpdateId((long)worker.getLoginId());
        youthSafetyModelRecommend.setUpdateTime(LocalDateTime.now());
        youthSafetyModelRecommend.setDelFlag(0);
        youthSafetyModelRecommend.setMajorAchievement(dto.getMajorAchievement());
        youthSafetyModelRecommend.setQagApplyId(ycqQzhQagApply.getId());
        youthSafetyModelRecommend.setIsDraft(dto.getIsDraft());

        youthSafetyModelRecommend.setCreator(worker.getWorkerName());
        youthSafetyModelRecommend.setCreateId((long)worker.getLoginId());
        youthSafetyModelRecommend.setCreateTime(LocalDateTime.now());
        youthSafetyModelRecommend.setDelFlag(0);
        return youthSafetyModelRecommend;
    }

    @Override
    public Result recommendDtail(Long id) {

        Optional<YcqQzhQagApply> byId = ycqQzhQagApplyRepo.findById(id);
        YouthSafetyModelRecommend youthSafetyModelRecommend = youthSafetyModelRecommendRepo.findByQagApplyIdAndIsDraft(byId.get().getId(),0);
        if (youthSafetyModelRecommend == null) {
            Result<Object> result = new Result<>();
            result.setData(null);
            result.setSuccess(Boolean.TRUE);
            return result;
        }

        YouthSafetyModelRecommendRespDTO youthSafetyModelRecommendRespDTO = new YouthSafetyModelRecommendRespDTO();
        BeanUtils.copyProperties(youthSafetyModelRecommend,youthSafetyModelRecommendRespDTO);
        youthSafetyModelRecommendRespDTO.setLeaderPhone(getSubStr(youthSafetyModelRecommend.getLeaderPhone()));
        youthSafetyModelRecommendRespDTO.setLeaderIdCard(getSubStr(youthSafetyModelRecommend.getLeaderIdCard()));
        youthSafetyModelRecommendRespDTO.setMajorAchievement(StrUtil.isNotBlank(youthSafetyModelRecommend.getMajorAchievement())?youthSafetyModelRecommendRespDTO.getMajorAchievement().replaceAll("<[^>]*>",""): youthSafetyModelRecommend.getMajorAchievement());

        List<YouthSafetyRecommendFile> files = youthSafetyRecommendFileRepo.findByRecommendId(youthSafetyModelRecommend.getId());
        if (CollectionUtils.isNotEmpty(files)) {
            List<RecommendFileReqDTO> fileList = new ArrayList<>();
            for (YouthSafetyRecommendFile file : files) {
                RecommendFileReqDTO recommendFileReqDTO = new RecommendFileReqDTO();
                BeanUtils.copyProperties(file,recommendFileReqDTO);
                fileList.add(recommendFileReqDTO);
            }
            youthSafetyModelRecommendRespDTO.setFiles(fileList);
        }
        return Result.ofSuccess(youthSafetyModelRecommendRespDTO);
    }

    @Override
    public Result doRecommendFillIn(Long id) {

        Worker worker = getWorker();
        Optional<YcqQzhQagApply> optionalYcqQzhQagApply = ycqQzhQagApplyRepo.findById(id);
        if (!optionalYcqQzhQagApply.isPresent()) {
            throw new BusinessException("所选申报不存在");
        }
        YouthSafetyModelRecommend youthSafetyModelRecommend = youthSafetyModelRecommendRepo.findByQagApplyId(id);
        if (youthSafetyModelRecommend==null) {
            YouthSafetyModelRecommend newRecommend = new YouthSafetyModelRecommend();
            BeanUtils.copyProperties(optionalYcqQzhQagApply.get(),newRecommend);
            newRecommend.setIsDraft(1);
            newRecommend.setQagApplyId(optionalYcqQzhQagApply.get().getId());
            newRecommend.setCreator(worker.getWorkerName());
            newRecommend.setCreateId((long)worker.getLoginId());
            newRecommend.setCreateTime(LocalDateTime.now());
            newRecommend.setUpdateId(null);
            newRecommend.setUpdator(null);
            newRecommend.setUpdateTime(null);
            YouthSafetyModelRecommend save = youthSafetyModelRecommendRepo.save(newRecommend);

            RecommendFillInRespDTO recommendFillInRespDTO = new RecommendFillInRespDTO();
            BeanUtils.copyProperties(save,recommendFillInRespDTO);
            recommendFillInRespDTO.setLeaderIdCard(getSubStr(save.getLeaderIdCard()));
            recommendFillInRespDTO.setLeaderPhone(getSubStr(save.getLeaderPhone()));
            return Result.ofSuccess(recommendFillInRespDTO);
        } else {
            if (youthSafetyModelRecommend.getIsDraft()==0) {
                throw new BusinessException("已经存在推荐申报");
            }
            RecommendFillInRespDTO recommendFillInRespDTO = new RecommendFillInRespDTO();
            List<YouthSafetyRecommendFile> files = youthSafetyRecommendFileRepo.findByRecommendId(youthSafetyModelRecommend.getId());
            if (CollectionUtils.isNotEmpty(files)) {
                List<RecommendFileReqDTO> fileList = new ArrayList<>();
                for (YouthSafetyRecommendFile file : files) {
                    RecommendFileReqDTO recommendFileReqDTO = new RecommendFileReqDTO();
                    BeanUtils.copyProperties(file,recommendFileReqDTO);
                    fileList.add(recommendFileReqDTO);
                }
                recommendFillInRespDTO.setFiles(fileList);
            }
            BeanUtils.copyProperties(youthSafetyModelRecommend,recommendFillInRespDTO);
            recommendFillInRespDTO.setLeaderIdCard(getSubStr(youthSafetyModelRecommend.getLeaderIdCard()));
            recommendFillInRespDTO.setLeaderPhone(getSubStr(youthSafetyModelRecommend.getLeaderPhone()));
            return Result.ofSuccess(recommendFillInRespDTO);
        }
    }

    private String getSubStr(String str){
        if (StrUtil.isNotBlank(str)) {
            int length = str.length();
            String firstStr=str.substring(0,3);
            String lastStr=str.substring(length-4);
            return firstStr+"****"+lastStr;
        }
        return "";
    }

    @Override
    public Result doChangeLeader(YouthSafetyChangeLeaderReqDTO dto) {

        Worker currWorker = getWorker();
        Optional<YcqQzhQagApply> optional = ycqQzhQagApplyRepo.findById(dto.getId());
        if (!optional.isPresent()) {
            throw new BusinessException("所选申报不存在");
        }
        YcqQzhQagApply ycqQzhQagApply = optional.get();
        QagApplyLeaderChangeLog query = qagApplyLeaderChangeLogRepo.findByQagApplyIdAndIsDeletedAndStateIn(dto.getId(),0, Lists.newArrayList(0,1,3));
        if (query!=null) {
            throw new BusinessException("当前已经存在发起变更申请流程");
        }
        List<YcqQzhQagApply> idCardCheck =  ycqQzhQagApplyRepo.findAllByLeaderIdCardAndDelFlagAndFlowStateNotIn(dto.getLeaderIdCard(),0,Arrays.asList(0,8));

        List<QagApplyLeaderChangeLog> logList = qagApplyLeaderChangeLogRepo.findByLeaderIdCardNewAndIsDeletedAndStateIn(dto.getLeaderIdCard(),0,Lists.newArrayList(0,1,3));

        if (CollectionUtils.isNotEmpty(logList)) {
            throw new BusinessException("当前岗长已经存在于其他变更流程中");
        }

        if (CollectionUtils.isNotEmpty(idCardCheck) && (!dto.getLeaderIdCard().contains("*") && !ycqQzhQagApply.getLeaderIdCard().equals(dto.getLeaderIdCard()))) {
            throw new BusinessException("同一岗长身份证号只能存在一个申报中");
        }

        // 初始化一个流程
        // 启动
        CommonResult<FlowConfRespVO> result = flowInstanceApi.getFlowConfInfoByFlowCode("2c48b54e979243cbad685c527d9016a5");
        FlowConfRespVO data = result.getData();
        String deployId = data.getFlowDeployId();
        String moduleId = data.getFlowModuleId();
        //流程信息
        FlowStartDTO flowdto = new FlowStartDTO();
        flowdto.setFlowModuleId(moduleId);
        flowdto.setFlowDeployId(deployId);
        //创建信息
        FlowStarterInfoDTO starter = new FlowStarterInfoDTO();
        starter.setOrgId(Long.parseLong(currWorker.getOrgId()));
        starter.setOrgName(currWorker.getOrgName());
        starter.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
        starter.setUserName(currWorker.getWorkerName());
        flowdto.setStarter(starter);
        //审批组织
        FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
        ArrayList<IdNameDTO> orgs = new ArrayList<>();
        IdNameDTO idNameDTO = new IdNameDTO();
        idNameDTO.setId(Long.parseLong(currWorker.getOrgId()));
        idNameDTO.setName(currWorker.getOrgName());
        orgs.add(idNameDTO);
        addedTaskAssigenn.setAssigennDept(orgs);
        flowdto.setAddedTaskAssigenn(addedTaskAssigenn);
        CommonResult<CruniiStartProcessResult> startResult;

        //设置流程变量后面判断
        List<InstanceData> variables = new ArrayList<>();
        InstanceData instanceData = new InstanceData();
        instanceData.setKey("state");
        instanceData.setValue(1);
        variables.add(instanceData);
        flowdto.setVariables(variables);

        String flowInstanceId = null;
        String nodeInstanceId = null;
        try {
            startResult = flowInstanceApi.startProcess(flowdto);
            CruniiStartProcessResult startData = startResult.getData();
            flowInstanceId = startData.getFlowInstanceId();
            nodeInstanceId = startData.getActiveTaskInstance().getNodeInstanceId();
        } catch (Exception e) {
            throw new BusinessException("流程启动失败");
        }

        //生成变更记录
        QagApplyLeaderChangeLog qagApplyLeaderChangeLog = new QagApplyLeaderChangeLog();
        BeanUtils.copyProperties(ycqQzhQagApply,qagApplyLeaderChangeLog);
        qagApplyLeaderChangeLog.setQagApplyId(dto.getId());
        qagApplyLeaderChangeLog.setLeaderIdCardNew(dto.getLeaderIdCard());
        qagApplyLeaderChangeLog.setLeaderNameNew(dto.getLeaderName());
        qagApplyLeaderChangeLog.setLeaderPhoneNew(dto.getLeaderPhone());
        qagApplyLeaderChangeLog.setState(0);
        qagApplyLeaderChangeLog.setIsDeleted(0);
        qagApplyLeaderChangeLog.setType("变更岗长");
        qagApplyLeaderChangeLog.setFlowInstanceId(flowInstanceId);
        qagApplyLeaderChangeLog.setNodeInstanceId(nodeInstanceId);
        qagApplyLeaderChangeLog.setCreatedor(currWorker.getWorkerName());
        qagApplyLeaderChangeLog.setCreatedTime(LocalDateTime.now());
        qagApplyLeaderChangeLog.setCreateId((long)currWorker.getLoginId());
        qagApplyLeaderChangeLog.setUpdatedor(currWorker.getWorkerName());
        qagApplyLeaderChangeLog.setUpdatedTime(LocalDateTime.now());
        qagApplyLeaderChangeLog.setUpdateId((long)currWorker.getLoginId());

        // 初始直接通过草稿到初核
        createDratf(qagApplyLeaderChangeLog,currWorker);

        QagApplyLeaderChangeLog save = qagApplyLeaderChangeLogRepo.save(qagApplyLeaderChangeLog);

        //增加查看组织权限
        //添加可以查看的组织权限
        ReportOrgDto authDto = new ReportOrgDto();
        authDto.setReportType(2);
        authDto.setServiceId(save.getId());
        authDto.setContainCurrentOrg(true);
        List<Long> authOrgs = new ArrayList<>();
        authOrgs.add(Long.valueOf(currWorker.getOrgId()));
        if(Objects.nonNull(save.getAffiliationOrgId())){
            authOrgs.add(Long.valueOf(save.getAffiliationOrgId()));
        }
        Long cityOrgCode = jdbcDao.getCityOrgCode();
        authOrgs.add(Long.valueOf(currWorker.getOrgId()));
        authOrgs.add(cityOrgCode);
        authDto.setOrgIds(authOrgs);
        civilizationReportService.saveQueryOrg(authDto,currWorker);

        return Result.ofSuccessMsg("操作成功");
    }

    private void createDratf(QagApplyLeaderChangeLog qagApplyLeaderChangeLog,Worker currWorker) {
        //审核通过到草稿
        FlowCommitDTO draftdto = new FlowCommitDTO();
        draftdto.setFlowInstanceId(qagApplyLeaderChangeLog.getFlowInstanceId());
        draftdto.setTaskInstanceId(qagApplyLeaderChangeLog.getNodeInstanceId());
        //设置提交人
        FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
        commitor.setOrgId(Long.parseLong(currWorker.getOrgId()));
        commitor.setOrgName(currWorker.getOrgName());
        commitor.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
        commitor.setUserName(currWorker.getWorkerName());
        draftdto.setCommitor(commitor);
        //处理结果处理意见
        draftdto.setProcessResult("草稿提交");
        draftdto.setProcessComments("");
        //设置流程变量后面判断
        Map<String, Object> map = new HashMap<>();
        map.put("state", 1);
        draftdto.setFlowVar(map);
        draftdto.setCommitFlow(true);

        //设置组织
        //审批组织
        FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
        ArrayList<IdNameDTO> orgs = new ArrayList<>();
        IdNameDTO idNameDTO = new IdNameDTO();
        idNameDTO.setId(Long.parseLong(qagApplyLeaderChangeLog.getAffiliationOrgId()));
        idNameDTO.setName(qagApplyLeaderChangeLog.getAffiliationOrgName());
        orgs.add(idNameDTO);
        addedTaskAssigenn.setAssigennDept(orgs);
        draftdto.setAddedTaskAssigenn(addedTaskAssigenn);
        CommonResult<CruniiCommitTaskResult> draftResult = flowInstanceApi.commitProcess(draftdto);
        if (draftResult.getCode() == 00 && draftResult.getData().getErrCode() < 2000) {
            String nodeInstanceId = draftResult.getData().getActiveTaskInstance().getNodeInstanceId();
            qagApplyLeaderChangeLog.setNodeInstanceId(nodeInstanceId);
        } else {
            throw new BusinessException("流程启动失败");
        }
        String oldNodeKeys = qagApplyLeaderChangeLog.getOldNodeKeys() == null ? "" : qagApplyLeaderChangeLog.getOldNodeKeys();
        String modelKey = draftResult.getData().getActiveTaskInstance().getModelKey();
        qagApplyLeaderChangeLog.setOldNodeKeys(oldNodeKeys + "," + modelKey);
    }


    @Override
    public Result applyChangeListPage(PageReq<LeaderChangeListPageReqDTO> req) {

        Worker currWorker = getWorker();

        //查出当前人青安岗的按钮权限
        List<TsmMenu> byMenuUrl = tsmMenuRepo.findByMenuUrl("/cyanfont/cyanfontManager/teenagerSecurePost");
        if (byMenuUrl.size()!=1) {
            throw new BusinessException("菜单权限数据异常，请联系管理员");
        }
        TsmMenu tsmMenu = byMenuUrl.get(0);
        List<String> CurrentAuth = jdbcDao.queryCurrentUserAuth(tsmMenu,currWorker).stream().map(TsmMenu::getMenuName).collect(Collectors.toList());

        QueryOrgDto dto = new QueryOrgDto();
        dto.setReportType(2);
        dto.setOrgId(currWorker.getOrgId());
        List<Long> ids = civilizationReportService.queryServiceIds(dto);

        QagPageRsp<LeaderChangeLogPageRespDTO> leaderChangeLogPageRespDTOPageRsp = jdbc.applyChangeListPage(req, null, null,tsmMenu,ids);
        List<LeaderChangeLogPageRespDTO> result = leaderChangeLogPageRespDTOPageRsp.getResult();

        // 非超管处理
        if (!validatedSuper(currWorker)) {
            if(CurrentAuth.contains("变更批量审核")) {
                leaderChangeLogPageRespDTOPageRsp.setIsBatchAudit(1);
            }
            for (LeaderChangeLogPageRespDTO data : result) {
                if (CurrentAuth.contains("变更查看")) {
                    data.setViewButton(1);
                }
            }
            leaderChangeLogPageRespDTOPageRsp.setResult(result);
        } else {
            leaderChangeLogPageRespDTOPageRsp.setIsBatchAudit(1);
            //超管处理
            for (LeaderChangeLogPageRespDTO data : result) {
                data.setViewButton(1);
            }
            leaderChangeLogPageRespDTOPageRsp.setResult(result);
        }

        return Result.ofSuccess(leaderChangeLogPageRespDTOPageRsp);

    }

    private boolean validatedSuper(Worker worker) {
        if (CommonConstants.SUPER_LOGIN_IDS.contains(worker.getLoginId())) {
            return true;
        }
        return false;
    }

    @Override
    public Result leaderChangeListPage(PageReq<LeaderChangeListPageReqDTO> req) {
        Worker worker = getWorker();
        PageRsp<LeaderChangeLogPageRespDTO> leaderChangeLogPageRespDTOPageRsp = jdbc.leaderChangeListPage(req, worker);
        List<LeaderChangeLogPageRespDTO> result = leaderChangeLogPageRespDTOPageRsp.getResult();
        for (LeaderChangeLogPageRespDTO leaderChangeLogPageRespDTO : result) {
            if (worker.getLoginId()!=leaderChangeLogPageRespDTO.getCreateId()) {
                leaderChangeLogPageRespDTO.setIsApplicant(0);
            }
        }
        leaderChangeLogPageRespDTOPageRsp.setResult(result);
        return Result.ofSuccess(leaderChangeLogPageRespDTOPageRsp);
    }

    @Override
    public ResponseEntity<byte[]> applyPdfExport(String id, HttpServletRequest request, HttpServletResponse response) {

        try {
            // 构建HTTP响应头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_PDF);
            //设置响应字符集
            response.setCharacterEncoding("UTF-8");
            headers.setContentDispositionFormData("attachment", URLEncoder.encode("重庆市青年安全示范岗创建集体报备表","utf-8")+".pdf");

            Long aLong = Long.valueOf(id);
            Worker worker = getWorker();
            Optional<YcqQzhQagApply> byId = ycqQzhQagApplyRepo.findById(aLong);
            if (!byId.isPresent()) {
    //            throw new BusinessException("申报不存在");
                return ResponseEntity.ok().headers(headers).body(new byte[0]);
            }

            YcqQzhQagApply ycqQzhQagApply = byId.get();
            List<String > content = new ArrayList<>();
            content.add("1");
            content.add(ycqQzhQagApply.getApplyOrgName());
            content.add(ycqQzhQagApply.getWorkersNum().toString());
            content.add(ycqQzhQagApply.getYouthNum().toString());
            content.add(ycqQzhQagApply.getLeaderName());
            content.add(ycqQzhQagApply.getLeaderIdCard());
            content.add(ycqQzhQagApply.getLeaderPhone());
            content.add("1".equals(ycqQzhQagApply.getRemark())?"申报十佳":"申报市级");

            // 生成pdf
            ByteArrayOutputStream outputStream = getPdfWithWatermark(content, worker, ycqQzhQagApply);
            byte[] byteArray = outputStream.toByteArray();

            // 水印生成
            TsmWorker worker1 = tsmWorkerRepo.findByLoginIdAndState(worker.getLoginId(), 1);
            TsmOrg byOrgIdAndState = tsmOrgRepo.findByOrgIdAndState(Long.valueOf(worker1.getOrgId()), 1);
            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(byteArray);
            ByteArrayOutputStream byteArrayOutputStream = PDFAddWatermark.addWatermarkToPDF(byteArrayInputStream, "青春重庆 " + worker.getWorkerName() + " " + worker1.getMobile());
            return ResponseEntity.ok().headers(headers).body(byteArrayOutputStream.toByteArray());
        } catch (DocumentException | IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public ResponseEntity<byte[]> recommendPdfExport(String id, HttpServletRequest request, HttpServletResponse response) {

        try {
            // 构建HTTP响应头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_PDF);
            //设置响应字符集
            response.setCharacterEncoding("UTF-8");
            headers.setContentDispositionFormData("attachment", URLEncoder.encode("重庆市青年安全示范岗推荐表","utf-8")+".pdf");
            Long aLong = Long.valueOf(id);
            Worker worker = getWorker();
            YouthSafetyModelRecommend youthSafetyModelRecommend = youthSafetyModelRecommendRepo.findByQagApplyId(aLong);
            if (youthSafetyModelRecommend==null) {
                return ResponseEntity.ok().headers(headers).body(new byte[0]);
            }

            List<String > content = new ArrayList<>();
            content.add(youthSafetyModelRecommend.getCode());
            content.add(youthSafetyModelRecommend.getApplyOrgName());
            content.add(youthSafetyModelRecommend.getLeaderName());
            content.add("1".equals(youthSafetyModelRecommend.getRemark())?"申报十佳":"申报市级");
            content.add(youthSafetyModelRecommend.getWorkersNum().toString());
            content.add(youthSafetyModelRecommend.getYouthNum().toString());
            content.add(youthSafetyModelRecommend.getLeaderPhone());
            content.add(StrUtil.isNotBlank(youthSafetyModelRecommend.getMajorAchievement())?youthSafetyModelRecommend.getMajorAchievement().replaceAll("<[^>]*>",""):"");


            ByteArrayOutputStream outputStream = getPdfWithWatermarkForRecommend(content, worker);

            byte[] byteArray = outputStream.toByteArray();
            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(byteArray);
            TsmWorker worker1 = tsmWorkerRepo.findByLoginIdAndState(worker.getLoginId(), 1);
            TsmOrg byOrgIdAndState = tsmOrgRepo.findByOrgIdAndState(Long.valueOf(worker1.getOrgId()), 1);
            ByteArrayOutputStream byteArrayOutputStream = PDFAddWatermark.addWatermarkToPDF(byteArrayInputStream, "青春重庆 " + worker.getWorkerName() + " " + worker1.getMobile());
            return ResponseEntity.ok().headers(headers).body(byteArrayOutputStream.toByteArray());
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Result doLeaderChangeExamine(ExamineChangeLeaderDTO req) {

        Worker currWorker = getWorker();
        List<Long> ids = req.getIds().stream().map(Long::valueOf).collect(Collectors.toList());
        List<QagApplyLeaderChangeLog> allById = qagApplyLeaderChangeLogRepo.findAllById(ids);
        if (CollectionUtils.isEmpty(allById)) {
            throw new BusinessException("变更记录不存在");
        }
        Long cityOrgCode = jdbcDao.getCityOrgCode();
        for (QagApplyLeaderChangeLog changeLog : allById) {
            // 流程已经结束
            if (changeLog.getState()==2 || changeLog.getState() ==3) {
                throw new BusinessException("当前流程已经结束");
            }
            // 审批通过
            if (req.getState()==1) {
                //接入工作流
                //审核下一步开始
                FlowCommitDTO dto = new FlowCommitDTO();
                dto.setFlowInstanceId(changeLog.getFlowInstanceId());
                dto.setTaskInstanceId(changeLog.getNodeInstanceId());
                //设置提交人
                FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
                commitor.setOrgId(Long.parseLong(currWorker.getOrgId()));
                commitor.setOrgName(currWorker.getOrgName());
                commitor.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
                commitor.setUserName(currWorker.getWorkerName());
                dto.setCommitor(commitor);
                //处理结果处理意见
                dto.setProcessResult(req.getState() == 1 ? "通过" : "驳回");
                dto.setProcessComments(req.getReason());
                //设置流程变量后面判断
                Map<String, Object> map = new HashMap<>();
                map.put("state", req.getState());
                dto.setFlowVar(map);
                dto.setCommitFlow(true);

                if (req.getState() == 1 && changeLog.getState()!=2) {
                    //特殊新增
                    //设置组织
                    FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
                    ArrayList<IdNameDTO> orgs = new ArrayList<>();
                    IdNameDTO idNameDTO = new IdNameDTO();
                    idNameDTO.setId(cityOrgCode);
                    idNameDTO.setName("团重庆市委");
                    orgs.add(idNameDTO);
                    addedTaskAssigenn.setAssigennDept(orgs);
                    dto.setAddedTaskAssigenn(addedTaskAssigenn);
                }
                CommonResult<CruniiCommitTaskResult> startResult = flowInstanceApi.commitProcess(dto);
                if (startResult.getCode() == 00 && startResult.getData().getErrCode() < 2000) {
                    String nodeInstanceId = startResult.getData().getActiveTaskInstance().getNodeInstanceId();
                    changeLog.setNodeInstanceId(nodeInstanceId);
                    changeLog.setState(req.getState() == 1 ? changeLog.getState() + 1 : 3);
                    changeLog.setUpdateId((long)currWorker.getLoginId());
                    changeLog.setUpdatedor(currWorker.getWorkerName());
                    changeLog.setUpdatedTime(LocalDateTime.now());
                    if (changeLog.getState()==2) {
                        YcqQzhQagApply ycqQzhQagApply = ycqQzhQagApplyRepo.findById(changeLog.getQagApplyId()).get();
                        ycqQzhQagApply.setLeaderIdCard(changeLog.getLeaderIdCardNew());
                        ycqQzhQagApply.setLeaderName(changeLog.getLeaderNameNew());
                        ycqQzhQagApply.setLeaderPhone(changeLog.getLeaderPhoneNew());
                        ycqQzhQagApplyRepo.save(ycqQzhQagApply);
                    }
                } else {
                    return Result.ofFail("流程执行失败");
                }
                //审核下一步开始结束
                String oldNodeKeys = changeLog.getOldNodeKeys() == null ? "" : changeLog.getOldNodeKeys();
                String modelKey = startResult.getData().getActiveTaskInstance().getModelKey();
                changeLog.setOldNodeKeys(oldNodeKeys + "," + modelKey);
                //接入工作流完成
                qagApplyLeaderChangeLogRepo.saveAndFlush(changeLog);
            } else {
                // 回退到草稿节点
                String nodeKey = "e94167a5-cf8a-4627-b221-4583df30e0e5";

                FlowRollBackDTO dto = new FlowRollBackDTO();
                dto.setFlowInstanceId(changeLog.getFlowInstanceId());
                dto.setTargetNodeKey(nodeKey);
                //设置提交人
                FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
                commitor.setOrgId(Long.parseLong(currWorker.getOrgId()));
                commitor.setOrgName(currWorker.getOrgName());
                commitor.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
                commitor.setUserName(currWorker.getWorkerName());
                dto.setCommitor(commitor);
                dto.setProcessResult("回退");
                dto.setProcessComments(req.getReason());
                CommonResult<RollbackTaskResult> result = flowInstanceApi.rollBackTargetNode(dto);

                String nodeInstanceId;
                if (result.getCode() == 00 && result.getData().getErrCode() < 2000) {
                    nodeInstanceId = result.getData().getActiveTaskInstance().getNodeInstanceId();
                    changeLog.setState(3);
                    changeLog.setUpdateId((long)currWorker.getLoginId());
                    changeLog.setUpdatedor(currWorker.getWorkerName());
                    changeLog.setUpdatedTime(LocalDateTime.now());
                } else {
                    return Result.ofFail("流程回退失败");
                }

                changeLog.setNodeInstanceId(nodeInstanceId);
//                String oldNodeKeys = changeLog.getOldNodeKeys();

//                String oldKey = "";
//                for (String key : oldNodeKeys.split(",")) {
//                    if (key != null && key.equals(nodeKey)) {
//                        oldKey += key;
//                    }
//                }
                changeLog.setOldNodeKeys(nodeKey);
                qagApplyLeaderChangeLogRepo.saveAndFlush(changeLog);
            }

        }
        return Result.ofSuccessMsg("操作成功");
    }

    @Override
    public Result doLeaderChangeDelete(Long id) {

        Worker worker = getWorker();
        Optional<QagApplyLeaderChangeLog> byId = qagApplyLeaderChangeLogRepo.findById(id);
        if (!byId.isPresent()) {
            throw new BusinessException("变更记录不存在");
        }
        QagApplyLeaderChangeLog qagApplyLeaderChangeLog = byId.get();
        if (qagApplyLeaderChangeLog.getState()!=3) {
            throw new BusinessException("当前状态不可终止");
        }
        if (!Long.valueOf(worker.getLoginId()).equals(qagApplyLeaderChangeLog.getCreateId())) {
            throw new BusinessException("非创建人不可终止");
        }
        qagApplyLeaderChangeLogRepo.updateLeaderChangeLog(id);
        qagApplyLeaderChangeLog.setUpdatedor(worker.getWorkerName());
        qagApplyLeaderChangeLog.setUpdateId((long)worker.getLoginId());
        qagApplyLeaderChangeLog.setUpdatedTime(LocalDateTime.now());
        qagApplyLeaderChangeLogRepo.save(qagApplyLeaderChangeLog);

        FlowBatchFinishDTO flowBatchFinishDTO = new FlowBatchFinishDTO();
        flowBatchFinishDTO.setFlowInstanceId(Sets.newHashSet(qagApplyLeaderChangeLog.getFlowInstanceId()));
        flowInstanceApi.batchFinish(flowBatchFinishDTO);
        return Result.ofSuccessMsg("终止成功");
    }

    @Override
    public Result doLeaderChangeEdit(LeaderChangeEditDTO dto) {
        Worker currWorker = getWorker();
        Optional<QagApplyLeaderChangeLog> byId = qagApplyLeaderChangeLogRepo.findById(dto.getId());
        if (!byId.isPresent()) {
            throw new BusinessException("变更记录不存在");
        }
        QagApplyLeaderChangeLog qagApplyLeaderChangeLog = byId.get();
        List<YcqQzhQagApply> idCardCheck =  ycqQzhQagApplyRepo.findAllByLeaderIdCardAndDelFlagAndFlowStateNot(dto.getLeaderIdCard(),0,8);

        if (!dto.getLeaderIdCard().contains("*")) {
            if (CollectionUtils.isNotEmpty(idCardCheck) && !qagApplyLeaderChangeLog.getLeaderIdCard().equals(dto.getLeaderIdCard())) {
                throw new BusinessException("同一岗长身份证号只能存在一个申报中");
            }
            List<QagApplyLeaderChangeLog> logList = qagApplyLeaderChangeLogRepo.findByLeaderIdCardNewAndIsDeletedAndStateIn(dto.getLeaderIdCard(),0,Lists.newArrayList(0,1,3));

            if (CollectionUtils.isNotEmpty(logList)) {
                throw new BusinessException("当前岗长已经存在于其他变更流程中");
            }
        }

        QagApplyLeaderChangeLog query = qagApplyLeaderChangeLogRepo.findByQagApplyIdAndIsDeletedAndStateIn(qagApplyLeaderChangeLog.getQagApplyId(),0, Lists.newArrayList(0,1));
        if (query!=null) {
            throw new BusinessException("当前已经存在发起变更申请流程");
        }
        if (qagApplyLeaderChangeLog.getState()!=3) {
            throw new BusinessException("当前状态不可编辑");
        }
        checkIdCard(dto.getLeaderIdCard());
        if (dto.getLeaderIdCard()!=null && !dto.getLeaderIdCard().contains("*")) {
            qagApplyLeaderChangeLog.setLeaderIdCardNew(dto.getLeaderIdCard());
        }
        qagApplyLeaderChangeLog.setLeaderNameNew(dto.getLeaderName());
        if (dto.getLeaderPhone()!=null && !dto.getLeaderPhone().contains("*")) {
            qagApplyLeaderChangeLog.setLeaderPhoneNew(dto.getLeaderPhone());
        }
        qagApplyLeaderChangeLog.setState(0);
        qagApplyLeaderChangeLog.setUpdatedor(currWorker.getWorkerName());
        qagApplyLeaderChangeLog.setUpdateId((long)currWorker.getLoginId());
        qagApplyLeaderChangeLog.setUpdatedTime(LocalDateTime.now());


        //审核下一步开始
        FlowCommitDTO draftdto = new FlowCommitDTO();
        draftdto.setFlowInstanceId(qagApplyLeaderChangeLog.getFlowInstanceId());
        draftdto.setTaskInstanceId(qagApplyLeaderChangeLog.getNodeInstanceId());
        //设置提交人
        FlowCommitorInfoDTO commitor = new FlowCommitorInfoDTO();
        commitor.setOrgId(Long.parseLong(currWorker.getOrgId()));
        commitor.setOrgName(currWorker.getOrgName());
        commitor.setUserId(Long.parseLong(currWorker.getLoginId() + ""));
        commitor.setUserName(currWorker.getWorkerName());
        draftdto.setCommitor(commitor);
        //处理结果处理意见
        draftdto.setProcessResult("草稿提交");
        draftdto.setProcessComments("");
        //设置流程变量后面判断
        Map<String, Object> map = new HashMap<>();
        map.put("state", 1);
        draftdto.setFlowVar(map);
        draftdto.setCommitFlow(true);

        //审批组织
        FlowTaskAssigennDTO addedTaskAssigenn = new FlowTaskAssigennDTO();
        ArrayList<IdNameDTO> orgs = new ArrayList<>();
        IdNameDTO idNameDTO = new IdNameDTO();
        //组织数据
        idNameDTO.setId(Long.parseLong(qagApplyLeaderChangeLog.getAffiliationOrgId()));
        idNameDTO.setName(qagApplyLeaderChangeLog.getAffiliationOrgName());
        orgs.add(idNameDTO);
        addedTaskAssigenn.setAssigennDept(orgs);

        //设置组织
        draftdto.setAddedTaskAssigenn(addedTaskAssigenn);
        CommonResult<CruniiCommitTaskResult> startResult = flowInstanceApi.commitProcess(draftdto);
        if (startResult.getCode() == 00 && startResult.getData().getErrCode() < 2000) {
            String nodeInstanceId = startResult.getData().getActiveTaskInstance().getNodeInstanceId();
            qagApplyLeaderChangeLog.setNodeInstanceId(nodeInstanceId);
        } else {
            return Result.ofFail("流程启动失败");
        }
        String oldNodeKeys = qagApplyLeaderChangeLog.getOldNodeKeys() == null ? "" : qagApplyLeaderChangeLog.getOldNodeKeys();
        String modelKey = startResult.getData().getActiveTaskInstance().getModelKey();
        qagApplyLeaderChangeLog.setOldNodeKeys(oldNodeKeys + "," + modelKey);

        qagApplyLeaderChangeLogRepo.save(qagApplyLeaderChangeLog);
        return Result.ofSuccessMsg("操作成功");
    }

    @Override
    public Result flowHistory(String id) {
        List<QagApplyLeaderChangeLog> changeLogs = qagApplyLeaderChangeLogRepo.findByQagApplyIdOrderByCreatedTimeDesc(Long.valueOf(id));
        List<String> flowInstances = changeLogs.stream().map(QagApplyLeaderChangeLog::getFlowInstanceId).collect(Collectors.toList());
        List<List<CruniiFlowProcessHistoryRespVO>> historyList = new ArrayList<>();
        for (String flowInstance : flowInstances) {
            FlowInstanceReqVO flowInstanceReqVO = new FlowInstanceReqVO();
            flowInstanceReqVO.setFlowInstanceId(flowInstance);
            CommonResult<List<CruniiFlowProcessHistoryRespVO>> listCommonResult = flowInstanceApi.listProcessHistory(flowInstanceReqVO);
            historyList.add(listCommonResult.getData());
        }

        return Result.ofSuccess(historyList);
    }

    public void checkIdCard(String idCard) {
        if (idCard.contains("*")) {
            return;
        }
        boolean matches = Pattern.matches("^(\\d{15}$|^\\d{18}$|^\\d{17}(\\d|X|x))$", idCard);
        if (!matches) {
            throw new BusinessException("身份证格式不正确");
        }

        String year;
        String monthDay;
        year = idCard.substring(6,10);
        monthDay = idCard.substring(10,14);
        //获取当前时间字符串如：2022-1128
        String nowTimeStr = new SimpleDateFormat("yyyy-MMdd").format(new Date());
        String yearNow = nowTimeStr.substring(0, 4);// 当前年份
        String monthDayNow = nowTimeStr.substring(5, 9);// 当前月日
        int age = Integer.parseInt(yearNow) - Integer.parseInt(year);
        //age减一的情况 ：用户月日大于当前月日（开头可以为0的4位数int）
        if (Integer.parseInt(monthDay) > Integer.parseInt(monthDayNow)) {
            age = age - 1;
        }
        if (age>=40) {
            throw new BusinessException("岗长年龄需小于等于40岁");
        };
    }

    private ByteArrayOutputStream getPdfWithWatermarkForRecommend(List<String > content, Worker worker) throws DocumentException, IOException {
        //创建一个文档(默认大小A4纸)
        Document document = new Document();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        PdfWriter.getInstance(document, byteArrayOutputStream);
        document.open();

        BaseFont bfChinese = BaseFont.createFont("/font/AdobeSongStd-Light.otf",BaseFont.IDENTITY_H, BaseFont.EMBEDDED);

        Font blackFont = new Font(bfChinese);
        blackFont.setColor(BaseColor.BLACK);

        PdfPTable table = new PdfPTable(2);
        // 构建每个单元格
        PdfPCell cell1 = new PdfPCell(new Paragraph("附件2",blackFont));
        // 设置背景颜色
        cell1.setBackgroundColor(BaseColor.WHITE);
        // 设置高度
        cell1.setFixedHeight(40);
        // 设置内容水平居中显示
        cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
        // 设置垂直居中
        cell1.setVerticalAlignment(Element.ALIGN_TOP);
        cell1.setBorder(0);
        table.addCell(cell1);

        Font redFont = new Font(bfChinese);
        redFont.setColor(BaseColor.BLACK);
        redFont.setColor(BaseColor.RED);
        // 构建每个单元格
        PdfPCell cell2 = new PdfPCell(new Paragraph("申报编号:"+content.get(0),redFont));
        // 设置背景颜色
        cell2.setBackgroundColor(BaseColor.WHITE);
        // 设置高度
        cell2.setFixedHeight(40);
        // 设置内容水平居中显示
        cell2.setHorizontalAlignment(Element.ALIGN_RIGHT);
        // 设置垂直居中
        cell2.setVerticalAlignment(Element.ALIGN_TOP);
        cell2.setBorder(0);
        table.addCell(cell2);
        document.add(table);

        blackFont.setSize(20f);
        blackFont.setStyle(1);
        Paragraph textgraph1 = new Paragraph(LocalDateTime.now().getYear()+"年度重庆市青年安全生产示范岗推荐表",blackFont);
        textgraph1.setAlignment(Element.ALIGN_CENTER);
        textgraph1.setSpacingBefore(30f);
        document.add(textgraph1);
        blackFont.setStyle(-1);

        blackFont.setSize(10f);
        PdfPTable mainTable1 = new PdfPTable(6);
        mainTable1.setSpacingBefore(16f);
        mainTable1.addCell(getPdfPCell("申报单位名称",blackFont,1,1,40));
        mainTable1.addCell(getPdfPCell(content.get(1), blackFont,5,2,40));
        document.add(mainTable1);

        PdfPTable mainTable2 = new PdfPTable(6);
        mainTable2.addCell(getPdfPCell("岗长",blackFont,1,1,40));
        mainTable2.addCell(getPdfPCell(content.get(2),blackFont,3,1,40));
        mainTable2.addCell(getPdfPCell("申报类别",blackFont,1,1,40));
        mainTable2.addCell(getPdfPCell(content.get(3),blackFont,1,1,40));
        document.add(mainTable2);

        PdfPTable mainTable3 = new PdfPTable(6);
        mainTable3.addCell(getPdfPCell("创建单位职工人数",blackFont,1,1,80));
        mainTable3.addCell(getPdfPCell(content.get(4),blackFont,1,1,80));
        mainTable3.addCell(getPdfPCell("35岁以下青年人数",blackFont,1,1,80));
        mainTable3.addCell(getPdfPCell(content.get(5),blackFont,1,1,80));
        mainTable3.addCell(getPdfPCell("联系电话",blackFont,1,1,80));
        mainTable3.addCell(getPdfPCell(content.get(6),blackFont,1,1,80));
        document.add(mainTable3);


        PdfPTable mainTable4 = new PdfPTable(12);
        mainTable4.setSpacingBefore(0);
//        mainTable4.addCell(getPdfPCell("主\n要\n事\n迹",blackFont,1,1,160));
//        mainTable4.addCell(getPdfPCell(StrUtil.isEmpty(content.get(7)) ?"（2000字左右，可附页)":content.get(7),blackFont,11,1,160));

        PdfPCell mainTable4Cell1 = new PdfPCell(new Phrase("主\n要\n事\n迹",blackFont));
        mainTable4Cell1.setUseBorderPadding(true);
        mainTable4Cell1.setBorderColor(BaseColor.BLACK);
        mainTable4Cell1.setFixedHeight(160);
        mainTable4Cell1.setMinimumHeight(50f);
        mainTable4Cell1.setHorizontalAlignment(Element.ALIGN_CENTER);
        mainTable4Cell1.setVerticalAlignment(Element.ALIGN_MIDDLE);
        mainTable4Cell1.setColspan(1);
        mainTable4Cell1.setRowspan(1);
        mainTable4.addCell(mainTable4Cell1);
        document.add(mainTable4);

        PdfPCell mainTable4Cell2 = new PdfPCell(new Phrase(StrUtil.isEmpty(content.get(7)) ?"（500字左右，可附页)":content.get(7),blackFont));
        mainTable4Cell2.setBorderColor(BaseColor.BLACK);
        mainTable4Cell2.setIndent(20f);
        mainTable4Cell2.setFixedHeight(160);
        mainTable4Cell2.setMinimumHeight(50f);
        mainTable4Cell2.setPadding(8f);
        mainTable4Cell2.setHorizontalAlignment(Element.ALIGN_LEFT);
        mainTable4Cell2.setVerticalAlignment(Element.ALIGN_CENTER);
        mainTable4Cell2.setColspan(11);
        mainTable4Cell2.setRowspan(1);
        mainTable4.addCell(mainTable4Cell2);
        document.add(mainTable4);

        PdfPTable mainTable5 = new PdfPTable(12);
        mainTable5.addCell(getPdfPCell("企\n业\n意\n见",blackFont,1,1,160));
        mainTable5.addCell(getPdfPCellSeal("(盖 章)       \n\n年  月  日  \n",blackFont,11,1,160));
        document.add(mainTable5);


        PdfPTable mainTable6 = new PdfPTable(12);
        mainTable6.addCell(getPdfPCell("推\n荐\n单\n位\n团\n组\n织\n意\n见",blackFont,1,1,160));
        mainTable6.addCell(getPdfPCellSeal("(盖 章)       \n\n年  月  日  \n",blackFont,5,1,160));
        mainTable6.addCell(getPdfPCell("上\n级\n安\n全\n监\n管\n部\n门\n意\n见",blackFont,1,1,160));
        mainTable6.addCell(getPdfPCellSeal("(盖 章)       \n\n年  月  日  \n",blackFont,5,1,160));
        document.add(mainTable6);

        PdfPTable mainTable7 = new PdfPTable(12);
        mainTable7.addCell(getPdfPCell("组\n委\n会\n办\n公\n室\n意\n见",blackFont,1,1,160));
//        mainTable7.addCell(getPdfPCellSeal("(共青团重庆市委盖章)       \n\n年  月  日  \n",blackFont,5,1,160));

        PdfPCell cell1Before = new PdfPCell(new Phrase("(共青团重庆市委盖章)       \n\n年  月  日  \n",blackFont));
        cell1Before.setUseBorderPadding(true);
        cell1Before.setBorderColor(BaseColor.BLACK);
        cell1Before.setFixedHeight(160);
        cell1Before.setPaddingRight(10f);
        cell1Before.setPaddingBottom(10f);
        cell1Before.setHorizontalAlignment(Element.ALIGN_RIGHT);
        cell1Before.setVerticalAlignment(Element.ALIGN_BOTTOM);
        cell1Before.disableBorderSide(8);
        cell1Before.setColspan(5);
        cell1Before.setRowspan(1);
        mainTable7.addCell(cell1Before);

        //中间
        PdfPCell cell = new PdfPCell(new Phrase("",blackFont));
        cell.setBorderColor(BaseColor.BLACK);
        cell.setFixedHeight(160);
        cell.setPaddingRight(10f);
        cell.setPaddingBottom(10f);
        cell.disableBorderSide(12);
        cell.setColspan(1);
        cell.setRowspan(1);
        mainTable7.addCell(cell);

        PdfPCell cell1After = new PdfPCell(new Phrase("(重庆市应急管理局盖章)       \n\n年  月  日  \n",blackFont));
        cell1After.setUseBorderPadding(true);
        cell1After.setBorderColor(BaseColor.BLACK);
        cell1After.setFixedHeight(160);
        cell1After.setPaddingRight(10f);
        cell1After.setPaddingBottom(10f);
        cell1After.setHorizontalAlignment(Element.ALIGN_RIGHT);
        cell1After.setVerticalAlignment(Element.ALIGN_BOTTOM);
        cell1After.disableBorderSide(4);
        cell1After.setColspan(5);
        cell1After.setRowspan(1);
        mainTable7.addCell(cell1After);
        document.add(mainTable7);

        Paragraph textgraph = new Paragraph("                        注:推荐单位团组织为各区县(自治县)团委，两江新区、西部科学城重庆高新区、万盛经开区团(工)\n"
                +"                        委，市各直属团组织和相关单位；申报类别注明申报十佳、申报市级；该附件一式三份。",blackFont);
        textgraph1.setAlignment(Element.ALIGN_CENTER);
        document.add(textgraph);
        document.close();
        return byteArrayOutputStream;
    }

    private ByteArrayOutputStream getPdfWithWatermark(List<String > content, Worker worker, YcqQzhQagApply ycqQzhQagApply) throws DocumentException, IOException {

        //创建一个文档(默认大小A4纸)
        Document document = new Document();
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        PdfWriter.getInstance(document,byteArrayOutputStream);
        document.open();

        BaseFont bfChinese = BaseFont.createFont("/font/AdobeSongStd-Light.otf",BaseFont.IDENTITY_H, BaseFont.EMBEDDED);

        Font font = new Font(bfChinese);
        font.setColor(BaseColor.RED);
        Paragraph textgraph1 = new Paragraph("申报编号:"+ycqQzhQagApply.getCode(),font);
        textgraph1.setAlignment(Element.ALIGN_RIGHT);
        textgraph1.setSpacingBefore(40f);
        document.add(textgraph1);

        font.setColor(BaseColor.BLACK);

        font.setSize(15f);
        Paragraph textgraph2 = new Paragraph("附件1",font);
        textgraph2.setAlignment(Element.ALIGN_LEFT);
        textgraph2.setSpacingBefore(20f);
        document.add(textgraph2);

        font.setSize(20f);
        font.setStyle(1);
        Paragraph textgraph3 = new Paragraph(LocalDateTime.now().getYear()+"年度重庆市青年安全生产示范岗\n" +
                "创建集体报备表",font);
        textgraph3.setAlignment(Element.ALIGN_CENTER);
        textgraph3.setSpacingBefore(40f);
        document.add(textgraph3);
        font.setStyle(-1);

        // 添加表格，4列
        PdfPTable table = new PdfPTable(8);
        // 设置表格宽度比例为%100
        table.setWidthPercentage(100);
        // 设置表格的宽度
//        table.setTotalWidth(500);
        // 也可以每列分别设置宽度
        float[] widthArr = new float[] { 30, 120, 60, 60,60,120,60,60 };
        String[] tableTitleArray = new String[] {"序号","申报单位名称","创建单位职工人数","35岁以下青年人数","岗长姓名","岗长身份证号","联系电话","备注"};
        table.setTotalWidth(widthArr);
        // 锁住宽度
        table.setLockedWidth(true);
        // 设置表格上面空白宽度
        table.setSpacingBefore(40f);
        // 设置表格下面空白宽度
        table.setSpacingAfter(10f);
        // 设置表格默认为无边框
        table.getDefaultCell().setBorder(0);
//        PdfContentByte cb = writer.getDirectContent();

        font.setSize(8f);

        for (String tableTitle : tableTitleArray) {
            // 构建每个单元格
            PdfPCell cell1 = new PdfPCell(new Paragraph(tableTitle,font));
            // 边框颜色
            cell1.setBorderColor(BaseColor.BLACK);
            // 设置背景颜色
            cell1.setBackgroundColor(BaseColor.WHITE);
            // 设置跨两行
            cell1.setRowspan(2);
            // 设置高度
            cell1.setFixedHeight(40);
            // 设置内容水平居中显示
            cell1.setHorizontalAlignment(Element.ALIGN_CENTER);
            // 设置垂直居中
            cell1.setVerticalAlignment(Element.ALIGN_MIDDLE);
            table.addCell(cell1);
        }

        for (String value : content) {
            // 构建每个单元格
            PdfPCell cell1 = new PdfPCell(new Paragraph(value,font));
            // 边框颜色
            cell1.setBorderColor(BaseColor.BLACK);
            // 设置背景颜色
            cell1.setBackgroundColor(BaseColor.WHITE);
            // 设置跨两行
            cell1.setRowspan(2);
            // 设置高度
            cell1.setFixedHeight(40);
            // 设置内容水平居中显示
            cell1.setHorizontalAlignment(Element.ALIGN_CENTER);
            // 设置垂直居中
            cell1.setVerticalAlignment(Element.ALIGN_MIDDLE);
            table.addCell(cell1);
        }

        document.add(table);


        font.setSize(12f);
        Paragraph textgraph4 = new Paragraph("注：1.推荐单位团组织为各区县( 自治县) 团委，两江新区、西部科学城重庆高新区、\n",font);
        textgraph4.setAlignment(Element.ALIGN_LEFT);
        textgraph4.setSpacingBefore(10f);
        document.add(textgraph4);

        Paragraph textgraph5 = new Paragraph("\t  万盛经开区团(工)委，市各直属团组织和相关单位；\n",font);
        textgraph5.setAlignment(Element.ALIGN_LEFT);
        textgraph5.setSpacingBefore(10f);
        textgraph5.setIndentationLeft(20f);
        document.add(textgraph5);

        Paragraph textgraph6 = new Paragraph("    2.“备注”栏请注明“申报十佳”或“申报市级”。",font);
        textgraph6.setAlignment(Element.ALIGN_LEFT);
        textgraph6.setSpacingBefore(10f);
        textgraph6.setIndentationLeft(13f);
        document.add(textgraph6);

        Paragraph textgraph7 = new Paragraph("推荐单位团组织:",font);
        textgraph7.setAlignment(Element.ALIGN_RIGHT);
        textgraph7.setSpacingBefore(40f);
        textgraph7.setIndentationRight(200f);
        document.add(textgraph7);

        List<QagShowUserDTO> qagShowUserDTOS = jdbc.queryShowUser(ycqQzhQagApply.getFlowInstanceId());

        if (CollectionUtils.isNotEmpty(qagShowUserDTOS)) {
            QagShowUserDTO qagShowUserDTO = qagShowUserDTOS.get(0);
            LocalDateTime createTime = qagShowUserDTO.getCreateTime();
            TsmWorker worker1 = tsmWorkerRepo.findByLoginIdAndState(Integer.valueOf(qagShowUserDTO.getNodeAssigenn()), 1);
            TsmOrg byOrgIdAndState = tsmOrgRepo.findByOrgIdAndState(Long.valueOf(worker1.getOrgId()), 1);
            Paragraph textgraph8 = new Paragraph(byOrgIdAndState.getOrgName()+" "+worker1.getWorkerName(),font);
            textgraph8.setAlignment(Element.ALIGN_RIGHT);
            textgraph8.setSpacingBefore(13f);
            textgraph8.setIndentationRight(92f);
            document.add(textgraph8);

            Paragraph textgraph9 = new Paragraph(createTime.getYear()+" 年 "+createTime.getMonth().getValue()+" 月 "+createTime.getDayOfMonth()+" 日",font);
            textgraph9.setAlignment(Element.ALIGN_RIGHT);
            textgraph9.setSpacingBefore(13f);
            textgraph9.setIndentationRight(92f);
            document.add(textgraph9);
        } else {
            TsmWorker worker1 = tsmWorkerRepo.findByLoginIdAndState(worker.getLoginId(), 1);
            TsmOrg byOrgIdAndState = tsmOrgRepo.findByOrgIdAndState(Long.valueOf(worker1.getOrgId()), 1);
            Paragraph textgraph8 = new Paragraph(byOrgIdAndState.getOrgName()+" "+worker1.getWorkerName(),font);
            textgraph8.setAlignment(Element.ALIGN_RIGHT);
            textgraph8.setSpacingBefore(13f);
            textgraph8.setIndentationRight(92f);
            document.add(textgraph8);

            Paragraph textgraph9 = new Paragraph(LocalDateTime.now().getYear()+" 年 "+LocalDateTime.now().getMonth().getValue()+" 月 "+LocalDateTime.now().getDayOfMonth()+" 日",font);
            textgraph9.setAlignment(Element.ALIGN_RIGHT);
            textgraph9.setSpacingBefore(13f);
            textgraph9.setIndentationRight(92f);
            document.add(textgraph9);
        }


        document.close();

        return byteArrayOutputStream;
    }

    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException(WORKER_NOT_EXISTS);
        }
        return worker;
    }

    public static PdfPCell getPdfPCell (String value,Font font,int colspan,int rowspan,Integer Height) {
        PdfPCell cell = new PdfPCell(new Phrase(value,font));
        cell.setUseBorderPadding(true);
        cell.setBorderColor(BaseColor.BLACK);
        cell.setFixedHeight(Height);
        cell.setHorizontalAlignment(Element.ALIGN_CENTER);
        cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
        cell.setColspan(colspan);
        cell.setRowspan(rowspan);
        return cell;
    }


    public static PdfPCell getPdfPCellSeal (String value,Font font,int colspan,int rowspan,Integer Height) {
        PdfPCell cell = new PdfPCell(new Phrase(value,font));
        cell.setUseBorderPadding(true);
        cell.setBorderColor(BaseColor.BLACK);
        cell.setFixedHeight(Height);
        cell.setPaddingRight(10f);
        cell.setPaddingBottom(10f);
        cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
        cell.setVerticalAlignment(Element.ALIGN_BOTTOM);
        cell.setColspan(colspan);
        cell.setRowspan(rowspan);
        return cell;
    }
}
