package com.crunii.micro.service.qzh.task;

import com.alibaba.excel.util.CollectionUtils;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.MessageSendTaskRepo;
import com.crunii.micro.domain.repo.main.PublishActivityRepo;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;
import com.crunii.micro.domain.repo.main.YcqMessageTemplateRepo;
import com.crunii.micro.feign.client.SmsService;
import com.crunii.micro.service.qzh.jdbc.TsmWorKerRoleJdbcDao;
import com.crunii.micro.service.qzh.jdbc.activitypublish.PublishActivityDao;
import com.crunii.micro.service.qzh.service.ActivityStatusCheckerService;
import com.crunii.micro.service.qzh.service.MessageSendLogTaskService;
import com.crunii.micro.service.qzh.service.MessageSendingStrategy;
import lombok.extern.log4j.Log4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RAtomicLong;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


/**
 * @author lmlksy
 * @date 2023/11/8 16:14
 */
@Log4j
@Service
public class ActivityStatusTypeTask  {

    @Autowired
    private RedissonClient redissonClient;

    private static final String LOCK_KEY = "activityStatusTypeTask";
    private static final long LOCK_EXPIRATION = 30; // 锁的过期时间，单位：秒

    // 定义任务状态的 Redis 键
    private static final String TASK_STATUS_KEY = "activityTaskStatusKey";

    @Autowired
    ActivityStatusCheckerService activityStatusCheckerService;

    @Autowired
    PublishActivityRepo publishActivityRepo;

    @Resource
    private PublishActivityDao activityDao;

    @Scheduled(cron = "*/60 * * * * *")
    public void doActivityStatusTypeTask() {
        RLock lock = redissonClient.getLock(LOCK_KEY);
        if (Objects.isNull(lock)) {
            return ;
        }

        boolean isLocked = false;
        RAtomicLong taskStatus =null;
        try {
            isLocked = lock.tryLock(0, LOCK_EXPIRATION, TimeUnit.SECONDS);
            taskStatus = redissonClient.getAtomicLong(TASK_STATUS_KEY);
            if (isLocked) {
                if (taskStatus.compareAndSet(0, 1)) {
                    try {

//                        List<YcqPublishActivity> all = publishActivityRepo.findAll();
//                        List<YcqPublishActivity> filteredList = all.stream()
//                                .filter(activity -> activity.getIsDeleted() != null && activity.getIsDeleted() == 0)
//                                .filter(activity -> activity.getActivityStatusTypeId() == null || (activity.getActivityStatusTypeId() != null && activity.getActivityStatusTypeId() != 9))
//                                .filter(activity -> activity.getActivityStatusTypeId() == null || (activity.getActivityStatusTypeId() != null && activity.getActivityStatusTypeId() != 1))
//                                .filter(activity -> activity.getActivityStatusTypeId() == null || (activity.getActivityStatusTypeId() != null && activity.getActivityStatusTypeId() != 2))
//                                .collect(Collectors.toList());
//                        if (!CollectionUtils.isEmpty(filteredList)) {
//                            // 批量处理任务
//                            List<YcqPublishActivity> tasksToProcess = new ArrayList<>();
//                            for (YcqPublishActivity task : filteredList) {
//                                tasksToProcess.add(task);
//                            }
//                            batchProcessTasks(tasksToProcess);
//                        }
                        List<YcqPublishActivity> list = activityDao.autoChangeStatusList();
                        if (!CollectionUtils.isEmpty(list)){
                            list.forEach(item -> {
                                doProcessTask(item);
                            });
                        }

                    }catch (Exception e){
                        log.error("活动任务执行异常:"+e);
                    }finally {
                        taskStatus.set(0);
                    }
                }
            }
            if (!isLocked) {
                log.info("尝试获取锁失败");
                return;
            }

        } catch (Exception e) {
            log.error("获取锁异常");
            return;
        } finally {
            if (isLocked&&lock.isHeldByCurrentThread()) {
                    taskStatus.set(0);
                    lock.unlock();
                }
        }
    }

    private void batchProcessTasks(List<YcqPublishActivity> tasks) {
        for (YcqPublishActivity task : tasks) {
            try {
                doProcessTask(task);
            } catch (Exception e) {
                log.error("处理task id异常:"+task.getId()+e);
                return;
            }
        }
    }
    public void doProcessTask(YcqPublishActivity task) {
        try {
            Integer activityStatusTypeId = activityStatusCheckerService.getActivityStatusTypeId(task.getId());
            if (!Objects.equals(activityStatusTypeId, task.getActivityStatusTypeId())){
                activityDao.setStatus(task.getId(), activityStatusTypeId);
            }
        }catch (Exception e){
            log.info("修改活动状态异常{}",e);
            return;
        }finally {
            log.info("活动调度执行结束");
        }
    }

}
