package com.crunii.micro.service.qzh.util;

import cn.hutool.core.lang.RegexPool;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;

import com.crunii.micro.common.enums.AuditStatusEnums;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;

import com.crunii.micro.domain.repo.main.YcqActivityPeopleAuditRepo;
import com.crunii.micro.service.qzh.api.domain.dto.ExcelMistakeData;
import com.crunii.micro.service.qzh.api.domain.dto.ExcelStuData;
import org.apache.ivy.util.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author baoxy
 * @date 2023-10-10
 */
@Component
public class ExcelStuVerify {

    @Autowired
    TsmWorkerRepo tsmWorkerRepo;
    List<ExcelMistakeData> list;

    public List<ExcelMistakeData> data(ExcelStuData excelStuData, String raw) {
        list = new ArrayList<>();
        // 行数
        ExcelMistakeData data = new ExcelMistakeData();
        data.setName("行数");
        data.setValue(raw);
        data.setFlag(true);
        list.add(data);
        name(excelStuData);
        contactMethod(excelStuData);
        registrationStatusName(excelStuData);
        registrationTime(excelStuData);
        checkinTime(excelStuData);
        checkoutTime(excelStuData);
        List<Integer> state = list.stream().map(ExcelMistakeData::getState).collect(Collectors.toList());
        if (state.contains(0)) {
            return list;
        } else {
            list = new ArrayList<>();
        }
        return list;
    }

    /**
     * 姓名
     */
    public void name(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("姓名");

        if (StrUtil.isEmpty(excelStuData.getUserName())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            List<TsmWorker> tsmWorkers = tsmWorkerRepo.findByWorkerNameAndState(excelStuData.getUserName(), 1);
            if (CollectionUtils.isEmpty(tsmWorkers)){
                mistakeData.setFlag(false);
                mistakeData.setErrText("当前用户不存在");
                mistakeData.setState(0);
            }else {
                mistakeData.setFlag(true);
                mistakeData.setValue(excelStuData.getUserName());
            }
        }
        list.add(mistakeData);
    }

    /**
     * 手机号
     */
    public void contactMethod(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("联系方式");
        if (StrUtil.isEmpty(excelStuData.getContactMethod())) {
            mistakeData.setFlag(false);
            mistakeData.setErrText("未填写");
            mistakeData.setState(0);
        } else {
            mistakeData.setFlag(true);
            mistakeData.setValue(excelStuData.getContactMethod());
        }
        list.add(mistakeData);
    }

    public void registrationStatusName(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("报名状态");
        if (!StrUtil.isEmpty(excelStuData.getRegistrationStatusName())){
            Integer codeByDescription = AuditStatusEnums.getByDescription(excelStuData.getRegistrationStatusName());
            if (codeByDescription==null){
                mistakeData.setFlag(false);
                mistakeData.setErrText("报名状态名称填写异常");
                mistakeData.setState(0);
            }else {
                mistakeData.setFlag(true);
                mistakeData.setValue(excelStuData.getRegistrationStatusName());
            }
        }
        list.add(mistakeData);
    }

    public void registrationTime(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("报名时间");

        if (excelStuData.getRegistrationTime() != null) {
            String regexPattern = "\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2}";
            String datetimeString = excelStuData.getRegistrationTime().toString();

            if (!datetimeString.matches(regexPattern)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("报名时间格式填写异常！格式为yyyy-MM-dd HH:mm:ss");
                mistakeData.setState(0);
            } else {
                mistakeData.setFlag(true);
                mistakeData.setValue(datetimeString);
            }
        }
        list.add(mistakeData);
    }

    public void checkinTime(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("签到时间");

        if (excelStuData.getCheckinTime() != null) {
            String regexPattern = "\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2}";
            String datetimeString = excelStuData.getCheckinTime().toString();

            if (!datetimeString.matches(regexPattern)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("签到时间格式填写异常！格式为yyyy-MM-dd HH:mm:ss");
                mistakeData.setState(0);
            } else {
                mistakeData.setFlag(true);
                mistakeData.setValue(datetimeString);
            }
        }
        list.add(mistakeData);
    }

    public void checkoutTime(ExcelStuData excelStuData) {
        ExcelMistakeData mistakeData = new ExcelMistakeData();
        mistakeData.setName("签退时间");

        if (excelStuData.getCheckoutTime() != null) {
            String regexPattern = "\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2}";
            String datetimeString = excelStuData.getCheckoutTime().toString();

            if (!datetimeString.matches(regexPattern)) {
                mistakeData.setFlag(false);
                mistakeData.setErrText("签退时间格式填写异常！格式为yyyy-MM-dd HH:mm:ss");
                mistakeData.setState(0);
            } else {
                mistakeData.setFlag(true);
                mistakeData.setValue(datetimeString);
            }
        }
        list.add(mistakeData);
    }



}
