
package com.crunii.micro.service.qzh.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.hssf.eventusermodel.*;
import org.apache.poi.hssf.eventusermodel.dummyrecord.LastCellOfRowDummyRecord;
import org.apache.poi.hssf.record.*;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/***
 * @Author jiang
 * @Description 导入
 * @Date 2021/10/20
 * @Param  * @param null
 * @return
 **/
public class Excel2003Reader implements HSSFListener {
    protected static final Log log = LogFactory.getLog(Excel2003Reader.class);

    private POIFSFileSystem fs;
    private HSSFRequest request;
    private HSSFEventFactory factory;
    private SSTRecord sstRecord;
    private FormatTrackingHSSFListener formatlistener;

    private int rowSize;

    private List<String> rowData = new ArrayList<String>(10);

    private List<Map<Integer, String>> data = new ArrayList<Map<Integer, String>>(500);

    /**
     * @return the data
     */
    public List<Map<Integer, String>> getData() {
        return data;
    }

    /**
     * @param data the data to set
     */
    public void setData(List<Map<Integer, String>> data) {
        this.data = data;
    }

    /**
     * @return the rowSize
     */
    public int getRowSize() {
        return rowSize;
    }

    /**
     * @param rowSize the rowSize to set
     */
    public void setRowSize(int rowSize) {
        this.rowSize = rowSize;
    }

    public static void main(String[] args) throws Exception {
        Excel2003Reader reader = new Excel2003Reader(new FileInputStream(new File("D:\\1.xls")));
        reader.readData();
        System.out.println(reader.getData());
        System.out.println(reader.getRowSize());
    }

    public Excel2003Reader(InputStream inputStream) {
        try {
            this.fs = new POIFSFileSystem(inputStream);
        } catch (IOException e) {
            log.error("build POIFSFileSystem error!", e);
            throw new RuntimeException(e);
        }
        this.formatlistener = new FormatTrackingHSSFListener(new MissingRecordAwareHSSFListener(this));
        this.factory = new HSSFEventFactory();
        this.request = new HSSFRequest();
        this.request.addListenerForAllRecords(formatlistener);
    }

    public void readData() {
        try {
            factory.processWorkbookEvents(request, fs);
        } catch (IOException e) {
            log.error("readData error!", e);
            throw new RuntimeException(e);
        }
    }

    @Override
    public void processRecord(Record record) {
        rowSize++;
        switch (record.getSid()) {
            case LabelRecord.sid:
                LabelRecord lr = (LabelRecord) record;
                rowData.add(lr.getColumn(), lr.getValue().trim());
                break;
            case SSTRecord.sid:
                sstRecord = (SSTRecord) record;
                break;
            case LabelSSTRecord.sid: // 单元格为字符串类型
                LabelSSTRecord lsrec = (LabelSSTRecord) record;
                if (sstRecord == null) {
                    rowData.add("");
                } else {
                    rowData.add(lsrec.getColumn(), sstRecord.getString(lsrec.getSSTIndex()).toString().trim());
                }
                break;
            case NumberRecord.sid: // 单元格为数字类型
                NumberRecord numrec = (NumberRecord) record;
                rowData.add(numrec.getColumn(), formatlistener.formatNumberDateCell(numrec).trim());
                break;
            default:
                break;
        }

        if (record instanceof LastCellOfRowDummyRecord) {
            if (rowData.size() > 0) {
                Map<Integer, String> map = new HashMap<Integer, String>(rowData.size());
                for (int i = 0; i < rowData.size(); i++) {
                    map.put(i, rowData.get(i));
                }
                data.add(map);
                // 清空容器
                rowData.clear();
            }
        }

    }

}
