
package com.crunii.micro.service.qzh.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.ss.util.CellReference;
import org.apache.poi.util.SAXHelper;
import org.apache.poi.xssf.eventusermodel.ReadOnlySharedStringsTable;
import org.apache.poi.xssf.eventusermodel.XSSFReader;
import org.apache.poi.xssf.eventusermodel.XSSFSheetXMLHandler;
import org.apache.poi.xssf.eventusermodel.XSSFSheetXMLHandler.SheetContentsHandler;
import org.apache.poi.xssf.model.StylesTable;
import org.apache.poi.xssf.usermodel.XSSFComment;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/***
 * @Author jiang
 * @Description 导入
 * @Date 2021/10/20
 * @Param  * @param null
 * @return
 **/
public class Excel2007Reader implements SheetContentsHandler {
	protected static final Log log = LogFactory.getLog(Excel2007Reader.class);

	private int rowSize;

	private List<String> rowData = new ArrayList<String>(10);

	private List<Map<Integer, String>> data = new ArrayList<Map<Integer, String>>(500);

	private OPCPackage p;

	private InputStream inputStream;

	public static void main(String[] args) throws Exception {
		Excel2007Reader reader = new Excel2007Reader(new FileInputStream(new File("D:\\2.xlsx")));
		reader.readData();
		System.out.println(reader.getData());
		System.out.println(reader.getRowSize());
	}

	public Excel2007Reader(InputStream inputStream) {
		try {
			p = OPCPackage.open(inputStream);
			this.inputStream = inputStream;
		} catch (Exception e) {
			log.error("OPCPackage.open error!", e);
			throw new RuntimeException(e);
		}
	}

	private void processSheet(StylesTable styles, ReadOnlySharedStringsTable strings, InputStream sheetInputStream) throws SAXException, ParserConfigurationException, IOException {
		XMLReader sheetParser = SAXHelper.newXMLReader();
		sheetParser.setContentHandler(new XSSFSheetXMLHandler(styles, strings, this, false));
		sheetParser.parse(new InputSource(sheetInputStream));
	}

	public void readData() {
		InputStream sheetInputStream = null;
		try {
			XSSFReader xssfReader = new XSSFReader(p);
			StylesTable styles = xssfReader.getStylesTable();
			ReadOnlySharedStringsTable strings = new ReadOnlySharedStringsTable(p);
			sheetInputStream = xssfReader.getSheetsData().next();
			processSheet(styles, strings, sheetInputStream);
		} catch (Exception e) {
			log.error("readData error!", e);
			throw new RuntimeException("readData error!");
		} finally {
			try {
				if (p != null) {
					p.close();
				}
			} catch (Exception e1) {
				log.error("p.close() error!", e1);
			}
			if (sheetInputStream != null) {
				try {
					sheetInputStream.close();
				} catch (Exception e) {
					log.error("sheetInputStream.close() error!", e);
				}
			}
			if (this.inputStream != null) {
				try {
					this.inputStream.close();
				} catch (Exception e) {
					log.error("inputStream.close() error!", e);
				}
			}
		}
	}

	/**
	 * @see SheetContentsHandler#startRow(int)
	 */
	@Override
	public void startRow(int rowNum) {
		rowSize++;
	}

	/**
	 * @see SheetContentsHandler#endRow()
	 */
	@Override
	public void endRow(int rowNum) {
		if (rowData.size() > 0) {
			Map<Integer, String> map = new HashMap<Integer, String>(rowData.size());
			for (int i = 0; i < rowData.size(); i++) {
				map.put(i, rowData.get(i));
			}
			data.add(map);
			// 清空容器
			rowData.clear();
		}
	}

	/**
	 * @see SheetContentsHandler#cell(String, String)
	 */
	@Override
	public void cell(String cellReference, String formattedValue, XSSFComment comment) {
		if (cellReference != null) {
			int col = (new CellReference(cellReference)).getCol();
			// 对表格空数据进行过滤
			for(int i = rowData.size(); i < col; i++){
				rowData.add(i, "");
			}
			rowData.add(col, formattedValue);
		}
	}

	/**
	 * @see SheetContentsHandler#headerFooter(String, boolean, String)
	 */
	@Override
	public void headerFooter(String text, boolean isHeader, String tagName) {

	}

	/**
	 * @return the rowSize
	 */
	public int getRowSize() {
		return rowSize;
	}

	/**
	 * @param rowSize the rowSize to set
	 */
	public void setRowSize(int rowSize) {
		this.rowSize = rowSize;
	}

	/**
	 * @return the rowData
	 */
	public List<String> getRowData() {
		return rowData;
	}

	/**
	 * @param rowData the rowData to set
	 */
	public void setRowData(List<String> rowData) {
		this.rowData = rowData;
	}

	/**
	 * @return the data
	 */
	public List<Map<Integer, String>> getData() {
		return data;
	}

	/**
	 * @param data the data to set
	 */
	public void setData(List<Map<Integer, String>> data) {
		this.data = data;
	}

}
