
package com.crunii.micro.service.qzh.utils;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import groovy.util.logging.Slf4j;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.openxml4j.util.ZipSecureFile;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/***
 * @Author jiang
 * @Description 导入
 * @Date 2021/10/20
 * @Param  * @param null
 * @return
 **/
@Slf4j
public class ExcelUtil {
	protected static final Log log = LogFactory.getLog(ExcelUtil.class);

	public static final int CONCURRENT_XLS_NUM = 50;

	public static final int MAX_EXPORT_NUM = 5000;

	public static List<Map<Integer, String>> readExcelContent(MultipartFile file) {
		String fileName = file.getOriginalFilename();
		assert fileName != null;
		String suffix = fileName.substring(fileName.lastIndexOf(".") + 1);
		try {
			if ("xls".equals(suffix)) {
				ZipSecureFile.setMinInflateRatio(-1.0d);
				Excel2003Reader reader = new Excel2003Reader(file.getInputStream());
				reader.readData();
				return reader.getData();
			} else if ("xlsx".equals(suffix)) {
				InputStream inputStream = file.getInputStream();
				ZipSecureFile.setMinInflateRatio(-1.0d);
				Excel2007Reader reader = new Excel2007Reader(inputStream);
				reader.readData();
				return reader.getData();
			} else {
				throw new RuntimeException("文件格式错误!");
			}
		} catch (Exception e) {
			log.error("文件读取错误!", e);
			throw new RuntimeException("文件读取错误!");
		}

	}



	public static List<Map<Integer, String>> readExcelToMap(MultipartFile file, int sheetNo, int startRowNo, int headRow, int readRow) throws IOException {
		List<Map<Integer, String>> resultList = new ArrayList<>();

		EasyExcel.read(file.getInputStream(), new AnalysisEventListener<Map<Integer, String>>() {
			@Override
			public void invoke(Map<Integer, String> data, AnalysisContext context) {
				//当前行号
				Integer currentRowNo = context.readRowHolder().getRowIndex();
				//判断当前行号是否在读取范围内
				if (currentRowNo >= startRowNo) {
					//如果读取所有，或者 当前行没有达到行数限制  则解析数据
					if (readRow == -1 || currentRowNo < startRowNo + readRow) {
						resultList.add(data);
					}
				}
			}
			@Override
			public void doAfterAllAnalysed(AnalysisContext context) {
				log.info("excel数据解析完成,sheetName:{}，sheetNo:{}"+context.readSheetHolder().getSheetName()+context.readSheetHolder().getSheetNo());
			}

		}).sheet(sheetNo).headRowNumber(headRow).doRead();

		return resultList;
	}


	public static  Map<String, Map<String, List<String>>> parseMuiltSheetExcel(File file) throws Exception{
		FileInputStream in = new FileInputStream(file);
		HSSFWorkbook wb = new HSSFWorkbook(in);
		//XSSFWorkbook wb = new XSSFWorkbook(in);
		Map<String, Map<String, List<String>>> sheetMap = new LinkedHashMap<>();

		for (int sheetIndex = 0; sheetIndex < wb.getNumberOfSheets(); sheetIndex++) {
			HSSFSheet sheet = wb.getSheetAt(sheetIndex);
			String sheetName = sheet.getSheetName();
			for (int rowNum = 0; rowNum <= sheet.getLastRowNum(); rowNum++) {
				HSSFRow row = sheet.getRow(rowNum);
				int lie = row.getPhysicalNumberOfCells();
				HSSFRow titleRow = sheet.getRow(0); // 获取标题行
				// 循环Excel的列
				for (int c = 0; c < lie; c++) {
					HSSFCell titleCell = titleRow.getCell(c);// 获取标题
					// 标题
					String title = titleCell.toString();;
					Cell cell = row.getCell(c);
					// 具体的值
					String value = "";
					if (cell != null) {
						switch (cell.getCellTypeEnum()) {
							case STRING:
								value = cell.getStringCellValue();
								break;
							case NUMERIC:
								if (HSSFDateUtil.isCellDateFormatted(cell)) {
									Date date = cell.getDateCellValue();
									if (date != null) {
										value = new SimpleDateFormat("yyyy-MM-dd").format(date);
									} else {
										value = "";
									}
								} else {
									value = new DecimalFormat("0").format(cell.getNumericCellValue());
								}
								break;
							case FORMULA:
								// 导入时如果为公式生成的数据则无值
								if (!cell.getStringCellValue().equals("")) {
									value = cell.getStringCellValue();
								} else {
									value = cell.getNumericCellValue() + "";
								}
								break;
							case BLANK:
								break;
							case ERROR:
								value = "";
								break;
							case BOOLEAN:
								value = (cell.getBooleanCellValue() == true ? "Y" : "N");
								break;
							default:
								value = "";
						}
						// 判断本sheet页是否已经存入map中
						if (sheetMap.get(sheetName) != null) {
							// 获取sheet页
							Map<String, List<String>> dataMap = sheetMap.get(sheetName);
							List<String> data = dataMap.get(title);
							if (data!=null){
								data.add(value);
							}else{
								dataMap.put(title,new ArrayList<>());
							}
						}else{
							sheetMap.put(sheetName,new LinkedHashMap<>());
						}
					}
				}
			}
		}
		in.close();
		return sheetMap;
	}

}
