package com.crunii.micro.service.qzh.utils;
import com.crunii.micro.common.utils.DateUtils;
import com.crunii.micro.domain.dto.Worker;
import java.awt.Color;
import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.springframework.http.MediaType;

/**
 * @description: 合并单元格
 * @create: 2021-07-28 10:49
 **/

public class ExcelpoiUtil {


  /**
   * 创建标题样式
   *
   * @param wb
   * @return
   */
  public static HSSFCellStyle createTitleCellStyle(HSSFWorkbook wb) {
    HSSFCellStyle cellStyle = wb.createCellStyle();
    cellStyle.setAlignment(HorizontalAlignment.CENTER);//水平居中
    cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);//垂直对齐
    cellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
    cellStyle.setFillForegroundColor(IndexedColors.GREY_40_PERCENT.getIndex());//背景颜色

    HSSFFont headerFont1 = (HSSFFont) wb.createFont(); // 创建字体样式
    headerFont1.setBold(true); //字体加粗
    headerFont1.setFontName("黑体"); // 设置字体类型
    headerFont1.setFontHeightInPoints((short) 15); // 设置字体大小
    cellStyle.setFont(headerFont1); // 为标题样式设置字体样式

    return cellStyle;
  }


  /**
   * 创建表头样式
   *
   * @param wb
   * @return
   */
  public static HSSFCellStyle createHeadCellStyle(HSSFWorkbook wb) {
    HSSFCellStyle cellStyle = wb.createCellStyle();
    cellStyle.setWrapText(true);// 设置自动换行
    cellStyle.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());//背景颜色
    cellStyle.setAlignment(HorizontalAlignment.CENTER); //水平居中
    cellStyle.setVerticalAlignment(VerticalAlignment.CENTER); //垂直对齐
    cellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
    cellStyle.setBottomBorderColor(IndexedColors.BLACK.index);
    cellStyle.setBorderBottom(BorderStyle.THIN); //下边框
    cellStyle.setBorderLeft(BorderStyle.THIN); //左边框
    cellStyle.setBorderRight(BorderStyle.THIN); //右边框
    cellStyle.setBorderTop(BorderStyle.THIN); //上边框

    HSSFFont headerFont = (HSSFFont) wb.createFont(); // 创建字体样式
    headerFont.setBold(true); //字体加粗
    headerFont.setFontName("黑体"); // 设置字体类型
    headerFont.setFontHeightInPoints((short) 12); // 设置字体大小
    cellStyle.setFont(headerFont); // 为标题样式设置字体样式

    return cellStyle;
  }

  /**
   * 创建内容样式
   *
   * @param wb
   * @return
   */
  public static HSSFCellStyle createContentCellStyle(HSSFWorkbook wb) {
    HSSFCellStyle cellStyle = wb.createCellStyle();
    cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);// 垂直居中
    cellStyle.setAlignment(HorizontalAlignment.CENTER);// 水平居中
    cellStyle.setWrapText(true);// 设置自动换行
    cellStyle.setBorderBottom(BorderStyle.THIN); //下边框
    cellStyle.setBorderLeft(BorderStyle.THIN); //左边框
    cellStyle.setBorderRight(BorderStyle.THIN); //右边框
    cellStyle.setBorderTop(BorderStyle.THIN); //上边框

    // 生成12号字体
    HSSFFont font = wb.createFont();
    font.setColor((short) 8);
    font.setFontHeightInPoints((short) 12);
    cellStyle.setFont(font);

    return cellStyle;
  }


  public static void putWaterRemarkToExcel(Workbook wb, Sheet sheet, BufferedImage waterRemarkPath,
      int startXCol,
      int startYRow, int betweenXCol, int betweenYRow, int XCount, int YCount, int waterRemarkWidth,
      int waterRemarkHeight, Worker worker) throws IOException {
    //加载图片
    ByteArrayOutputStream byteArrayOut = new ByteArrayOutputStream();
    if (null == waterRemarkPath) {
      throw new RuntimeException("向Excel上面打印水印，读取水印图片失败(2)。");
    }
    ImageIO.write(waterRemarkPath, "png", byteArrayOut);

    // 开始打水印
    Drawing drawing = sheet.createDrawingPatriarch();

    // 按照共需打印多少行水印进行循环
    for (int yCount = 0; yCount < YCount; yCount++) {
      // 按照每行需要打印多少个水印进行循环
      for (int xCount = 0; xCount < XCount; xCount++) {
        // 创建水印图片位置
        int xIndexInteger = startXCol + (xCount * waterRemarkWidth) + (xCount * betweenXCol);
        int yIndexInteger = startYRow + (yCount * waterRemarkHeight) + (yCount * betweenYRow);
        /*
         * 参数定义： 第一个参数是（x轴的开始节点）； 第二个参数是（是y轴的开始节点）； 第三个参数是（是x轴的结束节点）；
         * 第四个参数是（是y轴的结束节点）； 第五个参数是（是从Excel的第几列开始插入图片，从0开始计数）；
         * 第六个参数是（是从excel的第几行开始插入图片，从0开始计数）； 第七个参数是（图片宽度，共多少列）；
         * 第8个参数是（图片高度，共多少行）；
         */
        ClientAnchor anchor = drawing
            .createAnchor(0, 0, 0, 0, xIndexInteger, yIndexInteger,
                xIndexInteger + waterRemarkWidth,
                yIndexInteger + waterRemarkHeight);

        Picture pic = drawing
            .createPicture(anchor,
                wb.addPicture(byteArrayOut.toByteArray(), Workbook.PICTURE_TYPE_PNG));
        pic.resize();
      }
    }
    byteArrayOut.flush();
    byteArrayOut.close();
    sheet.protectSheet(worker.getLoginCode());
  }

  public static <T> void putWatermark(Workbook wb, Sheet sheet, Worker worker) throws Exception {
    putWatermark(1, wb, sheet, worker);
  }

  /**
   * 水印
   *
   * @param fieldListSize 导出表头的长度
   * @param wb            工作簿
   * @param sheet         表格
   * @param worker        用户信息
   * @since 2023-07-21 12:28
   */
  public static <T> void putWatermark(Integer fieldListSize, Workbook wb, Sheet sheet,
      Worker worker) throws Exception {
    // 根据导出列数计算图片的宽度
    //int width = 500;
    int width = 800;
    // 根据导出数据计算图片的高度
    //int height = 200;
    int height = 300;
    GraphicsEnvironment e1 = GraphicsEnvironment.getLocalGraphicsEnvironment();
    String[] fontName = e1.getAvailableFontFamilyNames();
    String fontType = "微软雅黑";
    Integer fontStyle = java.awt.Font.PLAIN;
    Integer fontSize = 20;
    java.awt.Font font = new java.awt.Font(fontType, fontStyle, fontSize);
    Color color = Color.red;
    String time = DateUtils
        .formatLocalDateTime(LocalDateTime.now(), DateUtils.DEFAULT_DATE_TIME_FORMAT2);
    String[] texts = new String[]{"用户信息及重要数据严禁泄漏", "账号：" + worker.getPhone(), "日期：" + time};
    // 获取有文字水印的透明色图片
    BufferedImage water = createWaterMark(width, height, texts, color, font, -25.00, 0.3f);
    //获取excel实际所占行
    int row = sheet.getFirstRowNum() + sheet.getLastRowNum();
    //导入图片
    //putWaterRemarkToExcel(wb, sheet, water, 0, 0, 6, 8, 1, row / 10 + 1, 0, 0, worker);
    putWaterRemarkToExcel(wb, sheet, water, 3, 10, 10, 30, fieldListSize / 15 + 1, row / 30 + 1, 0,
        0, worker);

  }


  public static BufferedImage createWaterMark(int width, int height, String[] text, Color color,
      java.awt.Font font, Double degree, float alpha) throws IOException {
    BufferedImage image = new BufferedImage(width, height,
        BufferedImage.TYPE_INT_RGB);// 获取bufferedImage对象
    Graphics2D g2d = image.createGraphics(); // 获取Graphics2d对象
    image = g2d.getDeviceConfiguration()
        .createCompatibleImage(width, height, Transparency.TRANSLUCENT);
    g2d.dispose();
    g2d = image.createGraphics();
    g2d.setColor(color); //设置字体颜色和透明度
    g2d.setStroke(new BasicStroke(1)); // 设置字体
    g2d.setFont(font); // 设置字体类型  加粗 大小
    //        g2d.rotate(Math.toRadians(-10), (double) image.getWidth() / 2, (double) image.getHeight() / 2);//设置倾斜度
    // 设置水印旋转
    if (null != degree) {
      //注意rotate函数参数theta，为弧度制，故需用Math.toRadians转换一下
      //以矩形区域中央为圆心旋转
      g2d.rotate(Math.toRadians(degree), (double) image.getWidth() / 2,
          (double) image.getHeight() / 2);
    }
    //计算打印字符中的最长值
    Integer length = Integer.valueOf(0);
    for (String str : text) {
      if (length < str.length()) {
        length = str.length();
      }
    }
    // 字体长度
    int markWidth = font.getSize() * (length);
    // 字体高度
    int markHeight = font.getSize();
    FontRenderContext context = g2d.getFontRenderContext();
    Rectangle2D bounds = font.getStringBounds(text[text.length - 1], context);
    double x = (width - markWidth) / 2;
    double y = (height - markHeight) / 10 / 2;
    double ascent = -bounds.getY();
    double baseY = y + ascent;
    // 写入水印文字原定高度过小，所以累计写水印，增加高度
    //设置透明度:1.0f为透明度 ，值从0-1.0，依次变得不透明
    g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
    g2d.drawString(text[0], (int) x, (int) (baseY += markHeight));
    g2d.drawString(text[1], (int) x, (int) (baseY += markHeight));
    g2d.drawString(text[2], (int) x, (int) (baseY += markHeight));
    // 释放对象
    g2d.dispose();
    return image;
  }

  /**
   * 浏览器下载excel
   *
   * @param fileName
   * @param wb
   * @param response
   */

  private static void buildExcelDocument(String fileName, Workbook wb,
      HttpServletResponse response) {
    try {
      response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
      response.setHeader("Content-Disposition",
          "attachment;filename=" + URLEncoder.encode(fileName, "utf-8"));
      response.flushBuffer();
      wb.write(response.getOutputStream());
    } catch (IOException e) {
      e.printStackTrace();
    }
  }


}
