package com.crunii.micro.service.qzh.utils;

import com.itextpdf.text.FontProvider;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.pdf.PdfDocument;
import com.itextpdf.text.pdf.PdfWriter;
import fr.opensagres.poi.xwpf.converter.xhtml.Base64EmbedImgManager;
import fr.opensagres.poi.xwpf.converter.xhtml.XHTMLConverter;
import fr.opensagres.poi.xwpf.converter.xhtml.XHTMLOptions;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringWriter;
import java.util.Map;

/**
 * @BelongsProject: microservice
 * @BelongsPackage: com.crunii.micro.service.qzh.utils
 * @Author: limy
 * @CreateTime: 2023-11-17  14:57
 * @Description: 转为文件
 * @Version: 1.0
 */
public class FileUtil {

    /**
     * XWPFDocument 转 MultipartFile（CommonsMultipartFile）
     *
     * @param document 文档对象
     * @param fileName 文件名
     * @return
     */
    public static MultipartFile xwpfDocumentToCommonsMultipartFile(XWPFDocument document, String fileName) {
        //XWPFDocument转FileItem
        FileItemFactory factory = new DiskFileItemFactory(16, null);
        FileItem fileItem = factory.createItem("textField", "text/plain", true, fileName);
        try {
            OutputStream os = fileItem.getOutputStream();
            document.write(os);
            os.close();
            //FileItem转MultipartFile
            MultipartFile multipartFile = new CommonsMultipartFile(fileItem);
            return multipartFile;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * docx文件转html
     * 参考： https://wooaooo.com/2022/02/12/Java/POI/java-poi5.2.0-word-to-html/
     * @param  data 需要填充的数据。
     */
    public static String word2007ToHtml(XWPFDocument document, Map<String, String> data)
            throws IOException {


        //  带图片的word，则将图片转为base64编码，保存在一个页面中
        XHTMLOptions options = XHTMLOptions.create().indent(3).setImageManager(new Base64EmbedImgManager());
        //若设置true,表示外层没有html\head\body标签，设为false，保留html等标签。
        options.setFragment(true);
        //转换为html
        StringWriter writer = new StringWriter();
        XHTMLConverter xhtmlConverter = (XHTMLConverter) XHTMLConverter.getInstance();
        xhtmlConverter.convert(document, writer, options);
        String html = writer.toString();
        // 解析html并优化布局
        Document doc = Jsoup.parseBodyFragment(html);
        //去掉页边距和固定宽度
//        Elements elements = doc.getElementsByTag("div");
//        for(Element element : elements){
//            element.attr("style","line-height:2em;margin-left: 3em; margin-right: 3em;");
//        }
//        //设置所有table的样式
//        Elements tables = doc.getElementsByTag("table");
//        for (Element table : tables) {
//            table.attr("style", "border-collapse: collapse;");
//        }
        //文本替换。
        html = doc.outerHtml();
        for(Map.Entry<String,String> item : data.entrySet()){
            html = html.replace(item.getKey(),item.getValue());
        }
        return html;
    }

}
