package com.crunii.micro.service.record.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.excel.EasyExcel;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.record.dto.*;
import com.crunii.micro.service.record.service.RecordEventsService;
import com.crunii.micro.service.record.utils.CustomImageModifyHandler;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import com.crunii.micro.common.dto.Result;

import java.io.IOException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;


/**
 * @author: LG
 * @date: 2023/6/25 15:31
 */
@Api(tags = "纪实系统")
@RestController
@RequestMapping("/auth/RecordEventsController")
public class RecordEventsController {

    @Resource
    private RecordEventsService recordEventsService;

    @ApiOperation(value = "修改个人信息")
    @RequestMapping(path = "/modifyPersonalInformation", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> modifyPersonalInformation(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @Validated ModifyPersonalInformationDto modifyPersonalInformationDto){
        return recordEventsService.updatePersonalInformation(worker,modifyPersonalInformationDto);
    }

    @ApiOperation(value = "个人修改信息回显")
    @RequestMapping(path = "/echoPersonalInformation", method = RequestMethod.GET)
    @ResponseBody
    public Result<EchoPersonalVo> echoPersonalInformation(@CurrWorker  @ApiIgnore Worker worker){
        return Result.ofSuccess(recordEventsService.echoPersonalInformation(worker));
    }

    @ApiOperation(value = "任务打卡")
    @RequestMapping(path = "/taskPunch", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> taskPunch(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @Validated TaskPunchDto taskPunchDto){
        return recordEventsService.saveEventTask(worker,taskPunchDto);
    }

    @ApiOperation(value = "任务查看")
    @PostMapping(path = "/taskLookOver")
    @ResponseBody
    public Result<PageRsp<TaskLookOverVo>> taskLookOver(@CurrWorker  @ApiIgnore Worker worker, @RequestBody  PageReq<Object> keyword){
        return Result.ofSuccess(recordEventsService.taskLookOver(worker,keyword));
    }

    @ApiOperation(value = "完成情况天数")
    @RequestMapping(path = "/doneCondition", method = RequestMethod.GET)
    @ResponseBody
    public Result<DoneConditionVo> doneCondition(@CurrWorker  @ApiIgnore Worker worker){
        return Result.ofSuccess(recordEventsService.doneCondition(worker));
    }

    @ApiOperation(value = "纪实数据统计列表查询")
    @RequestMapping(path = "/statisticalList", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<TaskLookOverVo>> statisticalList(@RequestBody PageReq<StatisticalListDto> req){
        return Result.ofSuccess(recordEventsService.statisticalList(req));
    }

    @ApiOperation(value = "个人信息修改校验")
    @RequestMapping(path = "/informationModificationCheck", method = RequestMethod.GET)
    @ResponseBody
    public Result informationModificationCheck(@CurrWorker  @ApiIgnore Worker worker){
        return recordEventsService.informationModificationCheck(worker);
    }

    @ApiOperation(value = "团市委直属机关查询")
    @RequestMapping(path = "/queryLeagueCode", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<LeagueCodeVo>> queryLeagueCode(){
        return Result.ofSuccess(recordEventsService.queryLeagueCode());
    }

    @ApiOperation(value = "excel导出数据统计",produces = "application/octet-stream")
    @RequestMapping(path = "/excelOutPut", method = RequestMethod.POST)
    public void excelOutPut(HttpServletResponse response ,@RequestBody List<String> eventIds ) throws IOException {

        if (CollectionUtil.isEmpty(eventIds)){
            throw new BusinessException("请传id");
        }

        List<TaskLookOverExcel> list = recordEventsService.excelOutPut(eventIds);

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String format = simpleDateFormat.format(new Date());

        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        String fileName = URLEncoder.encode("密切联系青年"+format, "UTF-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
        // 写入数据
        EasyExcel.write(response.getOutputStream(), TaskLookOverExcel.class).registerWriteHandler(new CustomImageModifyHandler()).sheet("sheet").doWrite(list);

    }



}
