package com.crunii.micro.service.record.jdbc;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.record.dto.*;
import jodd.util.StringUtil;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: LG
 * @date: 2023/6/25 17:18
 */
@Repository
public class RecordEventsDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public EchoPersonalVo queryEchoPersonalInformation(Integer workerId){
        String sql ="select a.*, b.league_name as orgIdText from TSM_WORKER a left join YCQ_LEAGUE b on a.org_id = b.league_code where a.state = 1 and a.login_id = ? ";
        return jt.queryForBean(EchoPersonalVo.class,sql,workerId);
    }

    public List<LeagueCodeVo> queryLeagueCode(){
        String sql = "select league_code , league_name from YCQ_LEAGUE where type = 1 ";
        return jt.queryForBeans(LeagueCodeVo.class,sql);
    }

    public String queryLeagueName(String orgId){
        String sql = "select league_name from YCQ_LEAGUE where league_code = ? ";
        return jt.queryForRawObject(String.class,sql,orgId);
    }

    public List<Integer> queryWorkerId(StatisticalListDto statisticalListDto){
        String sql = "select login_id from TSM_WORKER where state = 1 ";
        ArrayList<Object> params = new ArrayList<>();
        if (StringUtil.isNotBlank(statisticalListDto.getRegistrationSystem())) {
            sql = sql + " and registration_system = ? ";
            params.add(Integer.valueOf(statisticalListDto.getRegistrationSystem()));

            if (StringUtil.isNotBlank(statisticalListDto.getLeagueCode())) {
                sql = sql + " and org_id in (select org_id from TSM_ORG where ori_org_code = ? ) ";
                params.add(statisticalListDto.getLeagueCode());
            }
        }



        if (StringUtil.isNotBlank(statisticalListDto.getLeagueCadreName())){
            sql = sql + " and worker_name like concat('%',?,'%') ";
            params.add(statisticalListDto.getLeagueCadreName());
        }
        List<Integer> integers = jt.queryForRawObjects(Integer.class, sql, params.toArray());
        if (CollectionUtil.isEmpty(integers)){
            integers.add(-1);
        }
        return integers;
    }

    public PageRsp<TaskLookOverVo> taskLookOver (Worker worker, PageReq<Object> req){
        String sql = "select * from YCQ_EVENTS_TASK where state = 1 and upload_worker_id = ? order by task_time desc";
        String sqlCount = "select count(*) from YCQ_EVENTS_TASK where state = 1 and upload_worker_id = ? ";
        ArrayList<Object> params = new ArrayList<>();
        params.add(worker.getLoginId());
        return queryPageBase(req, sql, sqlCount, TaskLookOverVo.class, params);
    }

    public List<TaskLookOverVo> taskLookOver (List<String> eventsIds){
        String collectSql = eventsIds.stream().map(i -> "?").collect(Collectors.joining(",", "(", ")"));
        String sql = "select a.*,b.worker_name as uploadWorkerName,b.registration_system as registrationSystem ,b.org_id as orgId,c.org_name as orgIdText from YCQ_EVENTS_TASK as a left join TSM_WORKER as b on a.upload_worker_id = b.login_id left join TSM_ORG as c on c.org_id = b.org_id where  a.state = 1 and a.events_id in " +collectSql;
        ArrayList<Object> params = new ArrayList<>(Arrays.asList(eventsIds.toArray()));
        return jt.queryForBeans(TaskLookOverVo.class,sql,params.toArray());
    }

    public PageRsp<TaskLookOverVo> taskLookOver ( PageReq<StatisticalListDto> req){
        String sql = "select a.*,b.worker_name as uploadWorkerName,b.registration_system as registrationSystem ,b.org_id as orgId,b.position, c.org_name as orgIdText from YCQ_EVENTS_TASK as a left join TSM_WORKER as b on a.upload_worker_id = b.login_id left join TSM_ORG as c on c.org_id = b.org_id where  a.state = 1 " ;
        String sqlCount = "select count(*) from YCQ_EVENTS_TASK as a left join TSM_WORKER as b on a.upload_worker_id = b.login_id left join YCQ_LEAGUE as c on c.league_code = b.org_id where  a.state = 1 ";
        ArrayList<Object> params = new ArrayList<>();
        String workerSql = "select login_id from TSM_WORKER where state = 1 ";

        if (StringUtil.isNotBlank(req.getQuery().getRegistrationSystem())) {
            workerSql = workerSql + " and registration_system = ? ";
            params.add(Integer.valueOf(req.getQuery().getRegistrationSystem()));

//            if (StringUtil.isNotBlank(req.getQuery().getLeagueCode())) {
//                workerSql = workerSql + " and org_id in (select org_id from TSM_ORG where ori_org_code = ? ) ";
//                params.add(req.getQuery().getLeagueCode());
//            }
            if (StringUtil.isNotBlank(req.getQuery().getOrgId())) {
                workerSql = workerSql + " and org_id = ? ";
                params.add(req.getQuery().getOrgId());
            }
        }
        if (StringUtil.isNotBlank(req.getQuery().getLeagueCadreName())){
            workerSql = workerSql + " and worker_name like concat('%',?,'%') ";
            params.add(req.getQuery().getLeagueCadreName());
        }
            sql = sql +" and a.upload_worker_id in ("+ workerSql +")" ;
            sqlCount = sqlCount + " and a.upload_worker_id in ("+ workerSql +")";



        //拼接时间段sql
        if (ObjectUtil.isNotEmpty(req.getQuery().getStarTime())){
            sql = sql +" and a.task_time >=? ";
            sqlCount = sqlCount +" and a.task_time >=? ";
            params.add(req.getQuery().getStarTime());
        }
        if (ObjectUtil.isNotEmpty(req.getQuery().getEndTime())){
            sql = sql +" and a.task_time <=? ";
            sqlCount = sqlCount +" and a.task_time <=? ";
            params.add(req.getQuery().getEndTime());
        }
        sql = sql + "order by a.task_time desc";
        return queryPageBase(req, sql, sqlCount, TaskLookOverVo.class, params);
    }

    public List<TaskLookOverVo> taskLookOver (Worker worker){
        String sql = "select * from YCQ_EVENTS_TASK where YEAR(task_time)=YEAR(NOW()) and  state = 1 and upload_worker_id = ? ";
        return jt.queryForBeans(TaskLookOverVo.class,sql,worker.getLoginId());
    }

    public void updateObsState(Long serviceId){
        String sql = "update YCQ_OBS_FILE set state = 0 where service_code = ? and service_id = ? ";
        jt.update(sql,CommonConstants.OBS_FILE_RECORD_CODE,serviceId);

    }

    public List<ObsFileVo> queryObsPic(Long serviceId){
        String sql = "select object_key as fileKey,id as fileId from YCQ_OBS_FILE where state = 1 and service_code = ? and service_id = ? ";
        return jt.queryForBeans(ObsFileVo.class,sql, CommonConstants.OBS_FILE_RECORD_CODE,serviceId);
    }



    /**
     * 分页查询base
     * @param req
     * @param sql
     * @param countSql
     * @param clazz
     * @param params
     * @param <T>
     * @return
     */
    public <T> PageRsp<T> queryPageBase(PageReq<?> req, String sql, String countSql, Class<T> clazz, List<Object> params){
        StringBuilder sb = new StringBuilder(sql);
        StringBuilder sbCount = new StringBuilder(countSql);

        int total = jt.queryForInt(sbCount.toString(), params.toArray());
        List<T> values = null;
        if (total > 0) {
            sb.append(" limit ? offset ? ");
            params.add(req.getPageSize());
            params.add(req.getMyStart());
            values = jt.queryForBeans(clazz, sb.toString(), params.toArray());
        } else {
            values = new ArrayList<T>(0);
        }
        PageRsp<T> page = new PageRsp<T>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(values);
        // 为手机计算是否由下一页
        page.next();

        return page;
    }
}
