package com.crunii.micro.service.redscarf.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;

import com.crunii.micro.service.redscarf.service.CollectiveLearnService;
import com.crunii.micro.web.mvc.CurrWorker;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author baoxy
 * @date 2023-09-02
 */
@Api(tags = "集体学习")
@RestController
@RequestMapping("/auth/CollectiveLearnController")
public class CollectiveLearnController {

    @Resource
    private CollectiveLearnService collectiveLearnService;

    @ApiOperation(value = "集体学习提交内容")
    @RequestMapping(path = "/submitContent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> submitContent(@RequestBody CollectiveLearnDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        return collectiveLearnService.submitContent(dto, worker);
    }

    @ApiOperation(value = "集体学习审核")
    @RequestMapping(path = "/examine", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> examine(@RequestBody CollectiveLearnExamineDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        return collectiveLearnService.examine(dto, worker);
    }

    @ApiOperation(value = "集体学习查询审核弹窗")
    @RequestMapping(path = "/searchExamine", method = RequestMethod.POST)
    @ResponseBody
    public Result<CollectiveLearnSearchExamineVO> searchExamine(@RequestBody CollectiveLearnSearchExamineDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        return collectiveLearnService.searchExamine(dto, worker);
    }

    @ApiOperation(value = "集体学习查询未审核的数量接口")
    @RequestMapping(path = "/notExamineNum", method = RequestMethod.POST)
    @ResponseBody
    public Result<CollectiveLearnNotExamineNumVO> notExamineNum(@RequestBody CollectiveLearnNotExamineNumDTO dto,
                                                                @CurrWorker  @ApiIgnore Worker worker) {
        return collectiveLearnService.notExamineNum(dto, worker);
    }

    @ApiOperation(value = "集体学习查询未100%学习率的学季下拉框接口")
    @RequestMapping(path = "/searchNotCompleteQuarter", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<SearchQuarterVO>> searchNotCompleteQuarter(@RequestBody SearchNotCompleteQuarterDTO dto,
                                                                 @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(collectiveLearnService.searchNotCompleteQuarter(dto, worker));
    }

    @ApiOperation(value = "集体学习查询未100%学习率的学期下拉框接口")
    @RequestMapping(path = "/searchNotCompleteSemester", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<SearchSemesterVO>> searchNotCompleteSemester(@RequestBody SearchNotCompleteSemesterDTO dto,
                                                              @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(collectiveLearnService.searchNotCompleteSemester(dto, worker));
    }
}
