package com.crunii.micro.service.redscarf.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.redscarf.service.LearnDataService;
import com.crunii.micro.web.mvc.CurrWorker;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

/**
 * @author baoxy
 * @date 2023-09-02
 */
@Api(tags = "学习数据列表")
@RestController
@RequestMapping("/auth/LearnDataController")
public class LearnDataController {

    @Resource
    private LearnDataService learnDataService;

    @ApiOperation(value = "分页查询市级权限列表")
    @RequestMapping(path = "/pageSearchCityAccess", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<CityAccessVO>> pageSearchCityAccess(@RequestBody PageReq<CityAccessDTO> req, @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(learnDataService.pageSearchCityAccess(req, worker));
    }

    @ApiOperation(value = "分页查询区县权限列表")
    @RequestMapping(path = "/pageSearchCountyAccess", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<CountyAccessVO>> pageSearchCountyAccess(@RequestBody PageReq<CountyAccessDTO> req, @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(learnDataService.pageSearchCountyAccess(req, worker));
    }

    @ApiOperation(value = "分页查询学校权限列表")
    @RequestMapping(path = "/pageSearchSchoolAccess", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<SchoolAccessVO>> pageSearchSchoolAccess(@RequestBody PageReq<SchoolAccessDTO> req,
                                                                  @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(learnDataService.pageSearchSchoolAccess(req, worker));
    }

    @ApiOperation(value = "分页查询中队权限列表")
    @RequestMapping(path = "/pageSearchSquadronAccess", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<SquadronAccessVO>> pageSearchSquadronAccess(@RequestBody PageReq<SquadronAccessDTO> req,
                                                                      @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(learnDataService.pageSearchSquadronAccess(req, worker));
    }

    @ApiOperation(value = "分页查询中队详情列表")
    @RequestMapping(path = "/pageSearchSquadronDetails", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<SquadronDetailsVO>> pageSearchSquadronDetails(@RequestBody PageReq<SquadronDetailsDTO> req,
                                                                        @CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(learnDataService.pageSearchSquadronDetails(req, worker));
    }

    @ApiOperation(value = "导出市级权限列表")
    @RequestMapping(path = "/exportCityAccess", method = RequestMethod.POST)
    @ResponseBody
    public void exportCityAccess(HttpServletResponse response, @RequestBody CityAccessDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        learnDataService.exportCityAccess(response, dto, worker);
    }

    @ApiOperation(value = "导出区县权限列表")
    @RequestMapping(path = "/exportCountyAccess", method = RequestMethod.POST)
    @ResponseBody
    public void exportCountyAccess(HttpServletResponse response, @RequestBody CountyAccessDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        learnDataService.exportCountyAccess(response, dto, worker);
    }

    @ApiOperation(value = "导出学校权限列表")
    @RequestMapping(path = "/exportSchoolAccess", method = RequestMethod.POST)
    @ResponseBody
    public void exportSchoolAccess(HttpServletResponse response, @RequestBody SchoolAccessDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        learnDataService.exportSchoolAccess(response, dto, worker);
    }

    @ApiOperation(value = "导出中队权限列表")
    @RequestMapping(path = "/exportSquadronAccess", method = RequestMethod.POST)
    @ResponseBody
    public void exportSquadronAccess(HttpServletResponse response, @RequestBody SquadronAccessDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        learnDataService.exportSquadronAccess(response, dto, worker);
    }

    @ApiOperation(value = "导出中队详情列表")
    @RequestMapping(path = "/exportSquadronDetails", method = RequestMethod.POST)
    @ResponseBody
    public void exportSquadronDetails(HttpServletResponse response, @RequestBody SquadronDetailsDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        learnDataService.exportSquadronDetails(response, dto, worker);
    }

    @ApiOperation(value = "查询用户权限类型")
    @RequestMapping(path = "/workerAccess", method = RequestMethod.POST)
    @ResponseBody
    public Result<WorkerAccessVO> workerAccess(@CurrWorker  @ApiIgnore Worker worker) {
        return Result.ofSuccess(learnDataService.workerAccess(worker));
    }
}
