package com.crunii.micro.service.redscarf.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.redscarf.service.QuarterEnterService;
import com.crunii.micro.web.mvc.CurrWorker;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;

/**
 * @author baoxy
 * @date 2023-09-02
 */
@Api(tags = "课程录入功能")
@RestController
@RequestMapping("/auth/QuarterEnterController")
public class QuarterEnterController {

    @Resource
    private QuarterEnterService quarterEnterService;

    @ApiOperation(value = "分页查询学季信息")
    @RequestMapping(path = "/pageSearchQuarter", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<QuarterVO>> pageSearchQuarter(@RequestBody PageReq<Object> req) {
        return Result.ofSuccess(quarterEnterService.pageSearchQuarter(req));
    }

    @ApiOperation(value = "分页查询学季详情信息")
    @RequestMapping(path = "/pageSearchQuarterDetails", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<QuarterDetailsVO>> pageSearchQuarterDetails(@RequestBody PageReq<QuarterDetailsDTO> req) {
        return Result.ofSuccess(quarterEnterService.pageSearchQuarterDetails(req));
    }

    @ApiOperation(value = "新增学季")
    @RequestMapping(path = "/createQuarter", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> createQuarter(@RequestBody CreateQuarterDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        return quarterEnterService.createQuarter(dto, worker);
    }

    @ApiOperation(value = "编辑学季")
    @RequestMapping(path = "/updateQuarter", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateQuarter(@RequestBody UpdateQuarterDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        return quarterEnterService.updateQuarter(dto, worker);
    }

    @ApiOperation(value = "新增学期")
    @RequestMapping(path = "/createSemester", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> createSemester(@RequestBody CreateSemesterDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        return quarterEnterService.createSemester(dto, worker);
    }

    @ApiOperation(value = "编辑学期")
    @RequestMapping(path = "/updateSemester", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateSemester(@RequestBody UpdateSemesterDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        return quarterEnterService.updateSemester(dto, worker);
    }

    @ApiOperation(value = "批量删除学季")
    @RequestMapping(path = "/dealQuarter", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> dealQuarter(@RequestBody DelQuarterDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        return quarterEnterService.dealQuarter(dto, worker);
    }

    @ApiOperation(value = "批量删除学期")
    @RequestMapping(path = "/delSemester", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> delSemester(@RequestBody DelSemesterDTO dto, @CurrWorker  @ApiIgnore Worker worker) {
        return quarterEnterService.delSemester(dto, worker);
    }
}
