package com.crunii.micro.service.redscarf.controller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.redscarf.service.UserSideService;
import com.crunii.micro.web.mvc.CurrWorker;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author: LG
 * @date: 2023/8/31 14:13
 */
@Api(tags = "红领巾爱学习用户端")
@RestController
@RequestMapping("/auth/userSideController")
public class UserSideController {

    @Resource
    private UserSideService userSideService;

    @ApiOperation(value = "判断用户手机号是否录入学生")
    @RequestMapping(path = "/whetherSchoolInformation", method = RequestMethod.POST)
    @ResponseBody
    public Result<WhetherSchoolVO> whetherSchoolInformation(@CurrWorker  @ApiIgnore Worker worker){
        return userSideService.whetherSchoolInformation(worker);
    }

    @ApiOperation(value = "用户当前选中学生")
    @RequestMapping(path = "/updateUserStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateUserStudent(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @Validated UserAddStudentDTO userAddStudentDTO){
        return userSideService.updateUserStudent(worker,userAddStudentDTO);
    }

    @ApiOperation(value = "查看所有学季")
    @RequestMapping(path = "/queryLearnSeason", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<QueryLearnSeasonVO>> queryLearnSeason(){
        return userSideService.queryLearnSeason();
    }

    @ApiOperation(value = "验证学生身份证号和姓名与IRS进行匹配查询")
    @RequestMapping(path = "/queryStudentInformation", method = RequestMethod.POST)
    @ResponseBody
    public Result<QueryStudentInformationVO> queryStudentInformation(@RequestBody @Validated QueryStudentInformationDTO queryStudentInformationDTO){
        return userSideService.queryStudentInformation(queryStudentInformationDTO);
    }

    @ApiOperation(value = "录入学生信息")
    @RequestMapping(path = "/saveStudentInformation", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> saveStudentInformation(@CurrWorker  @ApiIgnore Worker worker, @RequestBody SaveStudentInformationDTO saveStudentInformationDTO){
        return userSideService.saveStudentInformation(worker,saveStudentInformationDTO);
    }

    @ApiOperation(value = "删除学生")
    @RequestMapping(path = "/deleteStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> deleteStudent(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @Validated UserAddStudentDTO userAddStudentDTO){
        return userSideService.deleteStudent(worker,userAddStudentDTO);
    }

    @ApiOperation(value = "标记用户完成的课程")
    @RequestMapping(path = "/updateLearningSeason", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateLearningSeason(@CurrWorker  @ApiIgnore Worker worker, @RequestBody @Validated UpdateLearningSeasonDTO updateLearningSeasonDTO){
        return userSideService.updateLearningSeason(worker,updateLearningSeasonDTO);
    }

    @ApiOperation(value = "查看学期根据各个学生学习情况划分")
    @RequestMapping(path = "/queryStudentLearnSemester", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<SemesterDTO>> queryStudentLearnSemester(@RequestBody @Validated QueryStudentLearnSemesterDTO queryStudentLearnSemesterDTO){
        return userSideService.queryStudentLearnSemester(queryStudentLearnSemesterDTO);
    }

    @ApiOperation(value = "用户查看所绑定的所有学生")
    @RequestMapping(path = "/queryUserStudent", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<QueryStudentInformationVO>> queryUserStudent(@CurrWorker  @ApiIgnore Worker worker){
        return userSideService.queryUserStudent(worker);
    }

    @ApiOperation(value = "我的奖章跳转页面")
    @RequestMapping(path = "/skipPage", method = RequestMethod.POST)
    @ResponseBody
    public Result<String> skipPage(@CurrWorker  @ApiIgnore Worker worker){
        return userSideService.skipPage(worker);
    }

}
