package com.crunii.micro.service.redscarf.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.mircro.service.redscarf.constants.RedscarfConstants;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author baoxy
 * @date 2023-09-09
 */
@Repository
public class CollectiveLearnDao {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;

    public static final String SEARCH_EXAMINE_SQL = "select a.collective_Learn_id, " +
            "b.quarter, b.quarter_theme_name,c.semester, c.semester_theme_name," +
            "concat(a.class_grade,'级',a.detachment,'中队') as gradeDetachment," +
            "d.worker_name  AS presenter, TO_CHAR(a.create_time,'yyyy-MM-dd hh24:mi') as create_time, a.location from " +
            "ycq_redscarf_collective_learn a,ycq_redscarf_quarter b,ycq_redscarf_semester c, tsm_worker d where " +
            "a.quarter_id = b.quarter_id AND a.semester_id = c.semester_id AND a.login_id  = d.login_id and " +
            "a.collective_Learn_id = ? and a.deleted = ? and b.deleted = ? and c.deleted = ? ";

    public static final String NOT_EXAMINE_NUM_SQL = "select count(1) as num from ycq_redscarf_collective_learn a, " +
            "ycq_redscarf_quarter q, ycq_redscarf_semester s where is_examine = ? and a.deleted = ? and school_name = ? " +
            "and q.deleted = ? and s.deleted = ? and a.quarter_id = q.quarter_id and a.semester_id = s.semester_id ";

    public static final String SEARCH_NOT_COMPLETE_QUARTER_SQL = "select quarter_id,quarter,quarter_theme_name from " +
            "(select b.quarter_id,b.semester_id,b.quarter,b.quarter_theme_name, " +
            "CASE WHEN round(stu_learn * 1.0/stu_total *100,2) IS NOT NULL THEN round(stu_learn * 1.0/stu_total *100,2) " +
            "ELSE '0' end as learn_rate " +
            "from (select count(1) as stu_learn,class_grade,detachment,quarter_id,semester_id from ycq_redscarf_stu stu," +
            "ycq_redscarf_stu_record record where stu.redscarf_id = record.redscarf_id and stu.deleted = ? and record.deleted = ? " +
            "and detachment = ? and school_name = ? and class_grade = ? GROUP BY class_grade,detachment,quarter_id,semester_id) a " +
            "RIGHT JOIN (SELECT DISTINCT sum(stu_num) AS stu_total,  class_grade,detachment, q.quarter_id, semester_id, quarter," +
            "quarter_theme_name, semester, semester_theme_name from ycq_redscarf_stu_test ,ycq_redscarf_quarter q, ycq_redscarf_semester s where " +
            "q.quarter_id = s.quarter_id and q.deleted = ? and s.deleted = ? and detachment = ? and school_name = ? and class_grade = ? " +
            "GROUP BY class_grade,detachment, q.quarter_id, semester_id,quarter, quarter_theme_name,semester,semester_theme_name) b " +
            "ON a.class_grade = b.class_grade AND a.detachment = b.detachment AND a.semester_id = b.semester_id) where learn_rate != 100 ";

    public static final String SEARCH_NOT_COMPLETE_SEMESTER_SQL = "select semester_id,semester,semester_theme_name from " +
            "(select b.semester_id,b.semester,b.semester_theme_name, " +
            "CASE WHEN round(stu_learn * 1.0/stu_total *100,2) IS NOT NULL THEN round(stu_learn * 1.0/stu_total *100,2) " +
            "ELSE '0' end as learn_rate " +
            "from (select count(1) as stu_learn,class_grade,detachment,quarter_id,semester_id from ycq_redscarf_stu stu," +
            "ycq_redscarf_stu_record record where stu.redscarf_id = record.redscarf_id and stu.deleted = ? and record.deleted = ? " +
            "and detachment = ? and school_name = ? and class_grade = ? and quarter_id = ? GROUP BY class_grade,detachment," +
            "quarter_id,semester_id) a RIGHT JOIN (SELECT DISTINCT sum(stu_num) AS stu_total, class_grade,detachment, q.quarter_id, " +
            "semester_id, quarter,quarter_theme_name, semester, semester_theme_name from ycq_redscarf_stu_test ,ycq_redscarf_quarter q, " +
            "ycq_redscarf_semester s where q.quarter_id = s.quarter_id and q.deleted = ? and s.deleted = ? and detachment = ? and " +
            "school_name = ? and class_grade = ? and q.quarter_id = ? GROUP BY class_grade,detachment, q.quarter_id, semester_id,quarter," +
            "quarter_theme_name,semester,semester_theme_name) b ON a.class_grade = b.class_grade AND a.detachment = b.detachment " +
            "AND a.semester_id = b.semester_id) where learn_rate != 100 ";

    public CollectiveLearnSearchExamineVO searchExamine(CollectiveLearnSearchExamineDTO dto) {
        List<Object> param = new ArrayList<>();
        param.add(dto.getCollectiveLearnId());
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        return jt.queryForBean(CollectiveLearnSearchExamineVO.class, SEARCH_EXAMINE_SQL, param.toArray());
    }

    public CollectiveLearnNotExamineNumVO notExamineNum(CollectiveLearnNotExamineNumDTO dto) {
        List<Object> param = new ArrayList<>();
        param.add(RedscarfConstants.ZERO_INTEGER);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(dto.getSchoolName());
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        return jt.queryForBean(CollectiveLearnNotExamineNumVO.class, NOT_EXAMINE_NUM_SQL, param.toArray());
    }

    public List<SearchQuarterVO> searchNotCompleteQuarter(SearchNotCompleteQuarterDTO dto) {
        List<Object> param = new ArrayList<>();
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(dto.getSquadron());
        param.add(dto.getSchool());
        param.add(dto.getClassGrade());
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(dto.getSquadron());
        param.add(dto.getSchool());
        param.add(dto.getClassGrade());
        return jt.queryForBeans(SearchQuarterVO.class, SEARCH_NOT_COMPLETE_QUARTER_SQL, param.toArray());
    }

    public List<SearchSemesterVO> searchNotCompleteSemester(SearchNotCompleteSemesterDTO dto) {
        List<Object> param = new ArrayList<>();
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(dto.getSquadron());
        param.add(dto.getSchool());
        param.add(dto.getClassGrade());
        param.add(dto.getQuarterId());
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(dto.getSquadron());
        param.add(dto.getSchool());
        param.add(dto.getClassGrade());
        param.add(dto.getQuarterId());
        return jt.queryForBeans(SearchSemesterVO.class, SEARCH_NOT_COMPLETE_SEMESTER_SQL, param.toArray());
    }
}
