package com.crunii.micro.service.redscarf.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.mircro.service.redscarf.constants.RedscarfConstants;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author baoxy
 * @date 2023-09-05
 */
@Repository
public class LearnDataDao {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;

    public static final String PAGE_SEARCH_CITY_ACCESS_SQL = "select b.area_name AS county, " +
            "b.quarter,b.quarter_id, b.quarter_theme_name, b.semester, b.semester_id, b.semester_theme_name, " +
            "CASE WHEN round(stu_learn * 1.0/stu_total *100,2) IS NOT NULL THEN concat(round(stu_learn * 1.0/stu_total *100,2), '%') " +
            "ELSE '0%' end as learn_rate from (select count(1) as stu_learn,area_name,quarter_id,semester_id from " +
            "ycq_redscarf_stu stu, ycq_redscarf_stu_record record where stu.redscarf_id  = record.redscarf_id and stu.deleted = ? " +
            "and record.deleted = ?  ";

    public static final String PAGE_SEARCH_CITY_ACCESS_COUNT_SQL = "select count(1) from (select count(1) as stu_learn, " +
            "area_name,quarter_id,semester_id FROM ycq_redscarf_stu stu, ycq_redscarf_stu_record record " +
            "where stu.redscarf_id  = record.redscarf_id and stu.deleted = ? and record.deleted = ?  ";

    public static final String PAGE_SEARCH_COUNTY_ACCESS_SQL = "select b.school_name as school, b.quarter,b.quarter_id," +
            "b.quarter_theme_name, b.semester, b.semester_id, b.semester_theme_name," +
            "CASE WHEN round(stu_learn * 1.0/stu_total *100,2) IS NOT NULL THEN concat(round(stu_learn * 1.0/stu_total *100,2), '%') " +
            "ELSE '0%' end as learn_rate from (select count(1) AS stu_learn,school_name ,quarter_id,semester_id from " +
            "ycq_redscarf_stu stu, ycq_redscarf_stu_record record where stu.redscarf_id = record.redscarf_id and stu.deleted = ? " +
            "and record.deleted = ? and area_name = ? ";

    public static final String PAGE_SEARCH_COUNTY_ACCESS_COUNT_SQL = "select count(1) from (select count(1) AS stu_learn,school_name," +
            "quarter_id,semester_id from ycq_redscarf_stu stu, ycq_redscarf_stu_record record " +
            "where stu.redscarf_id = record.redscarf_id and stu.deleted = ? and record.deleted = ? and area_name = ? ";

    public static final String PAGE_SEARCH_SCHOOL_ACCESS_SQL = "select b.class_grade,b.detachment as squadron," +
            "b.quarter,b.quarter_id, b.quarter_theme_name,b.semester,b.semester_id, b.semester_theme_name," +
            "CASE WHEN round(stu_learn * 1.0/stu_total *100,2) IS NOT NULL THEN concat(round(stu_learn * 1.0/stu_total *100,2), '%') " +
            "ELSE '0%' end as learn_rate ";

    public static final String PAGE_SEARCH_SCHOOL_ACCESS_COUNT_SQL = "select count(1) from (select count(1) AS stu_learn,class_grade," +
            "detachment,quarter_id,semester_id from ycq_redscarf_stu stu, ycq_redscarf_stu_record record " +
            "where stu.redscarf_id  = record.redscarf_id and stu.deleted = ? and record.deleted = ? and school_name = ? ";

    public static final String PAGE_SEARCH_SQUADRON_ACCESS_SQL = "select b.quarter_id,b.semester_id," +
            "b.quarter,b.quarter_theme_name, b.semester,b.semester_theme_name, " +
            "CASE WHEN round(stu_learn * 1.0/stu_total *100,2) IS NOT NULL THEN concat(round(stu_learn * 1.0/stu_total *100,2), '%') " +
            "ELSE '0%' end as learn_rate " +
            "from (select count(1) as stu_learn,class_grade,detachment,quarter_id,semester_id from ycq_redscarf_stu stu," +
            "ycq_redscarf_stu_record record where stu.redscarf_id = record.redscarf_id and stu.deleted = ? and record.deleted = ? " +
            "and detachment = ? and school_name = ? and class_grade = ? GROUP BY class_grade,detachment,quarter_id,semester_id) a " +
            "RIGHT JOIN (SELECT DISTINCT sum(stu_num) AS stu_total,  class_grade,detachment, q.quarter_id, semester_id, quarter," +
            "quarter_theme_name, semester, semester_theme_name from ycq_redscarf_stu_test ,ycq_redscarf_quarter q, ycq_redscarf_semester s where " +
            "q.quarter_id = s.quarter_id and q.deleted = ? and s.deleted = ? and detachment = ? and school_name = ? and class_grade = ? ";

    public static final String PAGE_SEARCH_SQUADRON_ACCESS_COUNT_SQL = "select count(1) from " +
            "(select count(1) as stu_learn,class_grade,detachment,quarter_id,semester_id from ycq_redscarf_stu stu," +
            "ycq_redscarf_stu_record record where stu.redscarf_id = record.redscarf_id and stu.deleted = ? and record.deleted = ? " +
            "and detachment = ? and school_name = ? and class_grade = ? GROUP BY class_grade,detachment,quarter_id,semester_id) a " +
            "RIGHT JOIN (SELECT DISTINCT sum(stu_num) AS stu_total,  class_grade,detachment, q.quarter_id, semester_id, quarter," +
            "quarter_theme_name, semester, semester_theme_name from ycq_redscarf_stu_test ,ycq_redscarf_quarter q, ycq_redscarf_semester s where " +
            "q.quarter_id = s.quarter_id and q.deleted = ? and s.deleted = ? and detachment = ? and school_name = ? and class_grade = ? ";

    public static final String PAGE_SEARCH_SQUADRON_DETAILS_SQL = "select a.stu_name as name, a.redscarf_id as student_id," +
            "case when b.stu_record_id is not null then '1' else  '0' end AS is_learn,quarter, quarter_theme_name,semester," +
            "semester_theme_name from (select redscarf_id,stu_name, q.quarter_id, semester_id, quarter, quarter_theme_name, " +
            "semester, semester_theme_name from ycq_redscarf_stu stu,ycq_redscarf_quarter q, ycq_redscarf_semester s where " +
            "q.quarter_id = s.quarter_id and stu.deleted = ? and q.deleted = ? and s.deleted = ? and detachment = ? " +
            "and school_name = ? and class_grade = ? and q.quarter_id = ? and s.semester_id = ? ";

    public static final String PAGE_SEARCH_SQUADRON_DETAILS_COUNT_SQL = "select count(1) from (select redscarf_id,stu_name," +
            "q.quarter_id, semester_id, quarter, quarter_theme_name,semester, semester_theme_name from ycq_redscarf_stu stu," +
            "ycq_redscarf_quarter q, ycq_redscarf_semester s where q.quarter_id = s.quarter_id and stu.deleted = ? and q.deleted = ? " +
            "and s.deleted = ? and detachment = ? and school_name = ? and class_grade = ? and q.quarter_id = ? and s.semester_id = ?";

    public PageRsp<CityAccessVO> pageSearchCityAccess(PageReq<CityAccessDTO> req) {
        StringBuilder sql = new StringBuilder();
        StringBuilder countSql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchCityAccess(sql, countSql, param, req.getQuery().getCounty(), req.getQuery().getQuarter(),
                req.getQuery().getSemester());
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dyType = jt.getDbType();
        List<CityAccessVO> result;
        if (total > 0) {
            sql.append(" order by learn_rate desc ");
            SqlUtil.appendLimit(dyType, param, sql, req);
            result = jt.queryForBeans(CityAccessVO.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<CityAccessVO> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        rsp.setTotalNum(total);
        rsp.setResult(result);
        return rsp;
    }

    public List<CityAccessExcel> excelSearchCityAccess(CityAccessDTO dto) {
        StringBuilder sql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchCityAccess(sql, new StringBuilder(), param, dto.getCounty(), dto.getQuarter(),
                dto.getSemester());
        sql.append(" order by learn_rate desc ");
        return jt.queryForBeans(CityAccessExcel.class, sql.toString(), param.toArray());
    }

    public void commonSearchCityAccess(StringBuilder sql, StringBuilder countSql, List<Object> param, List<String> county,
                                       List<String> quarter, List<String> semester) {
        sql.append(PAGE_SEARCH_CITY_ACCESS_SQL);
        countSql.append(PAGE_SEARCH_CITY_ACCESS_COUNT_SQL);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        commonAreaName(county, sql, countSql, param);
        sql.append(" GROUP BY area_name,quarter_id,semester_id) a RIGHT JOIN (SELECT DISTINCT sum(stu_num) AS stu_total, " +
                "area_name, q.quarter_id, semester_id, quarter, quarter_theme_name, semester, semester_theme_name from " +
                "ycq_redscarf_stu_test,ycq_redscarf_quarter q, ycq_redscarf_semester s where q.quarter_id = s.quarter_id " +
                "and q.deleted = ? and s.deleted = ? ");
        countSql.append(" GROUP BY area_name,quarter_id,semester_id) a RIGHT JOIN (SELECT DISTINCT sum(stu_num) AS stu_total, " +
                "area_name, q.quarter_id, semester_id, quarter, quarter_theme_name, semester, semester_theme_name from " +
                "ycq_redscarf_stu_test,ycq_redscarf_quarter q, ycq_redscarf_semester s where q.quarter_id = s.quarter_id " +
                "and q.deleted = ? and s.deleted = ? ");
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        commonAreaName(county, sql, countSql, param);
        commonQuarterAndSemester(quarter, semester, sql, countSql, param);
        sql.append(" GROUP BY area_name, q.quarter_id, semester_id,quarter, quarter_theme_name," +
                "semester, semester_theme_name ) b ON a.area_name = b.area_name AND a.semester_id = b.semester_id ");
        countSql.append(" GROUP BY area_name, q.quarter_id, semester_id,quarter, quarter_theme_name," +
                "semester, semester_theme_name ) b ON a.area_name = b.area_name AND a.semester_id = b.semester_id ");
    }

    public PageRsp<CountyAccessVO> pageSearchCountyAccess(PageReq<CountyAccessDTO> req) {
        StringBuilder sql = new StringBuilder();
        StringBuilder countSql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchCountyAccess(sql, countSql, param, req.getQuery().getCounty(), req.getQuery().getSchool(),
                req.getQuery().getQuarter(), req.getQuery().getSemester());
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dyType = jt.getDbType();
        List<CountyAccessVO> result;
        if (total > 0) {
            sql.append(" order by learn_rate desc ");
            SqlUtil.appendLimit(dyType, param, sql, req);
            result = jt.queryForBeans(CountyAccessVO.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<CountyAccessVO> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        rsp.setTotalNum(total);
        rsp.setResult(result);
        return rsp;
    }

    public List<CountyAccessExcel> excelSearchCountyAccess(CountyAccessDTO dto) {
        StringBuilder sql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchCountyAccess(sql, new StringBuilder(), param, dto.getCounty(), dto.getSchool(), dto.getQuarter(),
                dto.getSemester());
        sql.append(" order by learn_rate desc ");
        return jt.queryForBeans(CountyAccessExcel.class, sql.toString(), param.toArray());
    }

    public void commonSearchCountyAccess(StringBuilder sql, StringBuilder countSql, List<Object> param, String county,
                                         List<String> school, List<String> quarter, List<String> semester) {
        sql.append(PAGE_SEARCH_COUNTY_ACCESS_SQL);
        countSql.append(PAGE_SEARCH_COUNTY_ACCESS_COUNT_SQL);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(county);
        commonSchoolName(school, sql, countSql, param);
        sql.append(" GROUP BY school_name,quarter_id,semester_id) a RIGHT JOIN (SELECT DISTINCT sum(stu_num) AS stu_total, " +
                "school_name, q.quarter_id, semester_id, quarter, quarter_theme_name,semester, semester_theme_name from " +
                "ycq_redscarf_stu_test,ycq_redscarf_quarter q, ycq_redscarf_semester s where q.quarter_id = s.quarter_id " +
                "and q.deleted = ? and s.deleted = ? and area_name = ? ");
        countSql.append(" GROUP BY school_name,quarter_id,semester_id) a RIGHT JOIN (SELECT DISTINCT sum(stu_num) AS stu_total, " +
                "school_name, q.quarter_id, semester_id, quarter, quarter_theme_name,semester, semester_theme_name from " +
                "ycq_redscarf_stu_test,ycq_redscarf_quarter q, ycq_redscarf_semester s where q.quarter_id = s.quarter_id " +
                "and q.deleted = ? and s.deleted = ? and area_name = ? ");
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(county);
        commonSchoolName(school, sql, countSql, param);
        commonQuarterAndSemester(quarter, semester, sql, countSql, param);
        sql.append(" GROUP BY school_name, q.quarter_id, semester_id,quarter, quarter_theme_name," +
                "semester, semester_theme_name ) b ON a.school_name = b.school_name AND a.semester_id = b.semester_id ");
        countSql.append(" GROUP BY school_name, q.quarter_id, semester_id,quarter, quarter_theme_name," +
                "semester, semester_theme_name ) b ON a.school_name = b.school_name AND a.semester_id = b.semester_id ");
    }

    public PageRsp<SchoolAccessVO> pageSearchSchoolAccess(PageReq<SchoolAccessDTO> req) {
        StringBuilder sql = new StringBuilder();
        StringBuilder countSql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchSchoolAccess(sql, countSql, param, req.getQuery().getSchool(), req.getQuery().getClassGrade(),
                req.getQuery().getSquadron(), req.getQuery().getQuarter(), req.getQuery().getSemester(), req.getQuery().getOperateType());
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dyType = jt.getDbType();
        List<SchoolAccessVO> result;
        if (total > 0) {
            sql.append(" order by learn_rate desc ");
            SqlUtil.appendLimit(dyType, param, sql, req);
            result = jt.queryForBeans(SchoolAccessVO.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<SchoolAccessVO> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        rsp.setTotalNum(total);
        rsp.setResult(result);
        return rsp;
    }

    public List<SchoolAccessExcel> excelSearchSchoolAccess(SchoolAccessDTO dto) {
        StringBuilder sql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchSchoolAccess(sql, new StringBuilder(), param, dto.getSchool(), dto.getClassGrade(), dto.getSquadron(),
                dto.getQuarter(), dto.getSemester(), dto.getOperateType());
        sql.append(" order by learn_rate desc ");
        return jt.queryForBeans(SchoolAccessExcel.class, sql.toString(), param.toArray());
    }

    public List<SchoolAccessExcelExamine> excelSearchSchoolAccessExamine(SchoolAccessDTO dto) {
        StringBuilder sql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchSchoolAccess(sql, new StringBuilder(), param, dto.getSchool(), dto.getClassGrade(), dto.getSquadron(),
                dto.getQuarter(), dto.getSemester(), dto.getOperateType());
        sql.append(" order by learn_rate desc ");
        return jt.queryForBeans(SchoolAccessExcelExamine.class, sql.toString(), param.toArray());
    }

    public void commonSearchSchoolAccess(StringBuilder sql, StringBuilder countSql, List<Object> param, String school,
                                         List<String> grade, List<String> squadron, List<String> quarter, List<String> semester,
                                         String operateType) {
        sql.append(PAGE_SEARCH_SCHOOL_ACCESS_SQL);
        countSql.append(PAGE_SEARCH_SCHOOL_ACCESS_COUNT_SQL);
        if (StringUtils.isNotBlank(operateType)) {
            sql.append(" ,c.examine_state, c.collective_Learn_id ");
        }
        sql.append("from (select count(1) AS stu_learn,class_grade,detachment,quarter_id,semester_id from ycq_redscarf_stu stu, " +
                "ycq_redscarf_stu_record record where stu.redscarf_id  = record.redscarf_id and stu.deleted = ? and record.deleted = ? " +
                "and school_name = ? ");
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(school);
        commonGradeAndDetachment(grade, squadron, sql, countSql, param);
        sql.append(" GROUP BY class_grade,detachment,quarter_id,semester_id) a RIGHT JOIN (SELECT DISTINCT sum(stu_num) AS stu_total, " +
                "class_grade,detachment, q.quarter_id, semester_id, quarter, quarter_theme_name, semester, semester_theme_name from " +
                "ycq_redscarf_stu_test ,ycq_redscarf_quarter q, ycq_redscarf_semester s where q.quarter_id = s.quarter_id and " +
                "q.deleted = ? and s.deleted = ? and school_name = ? ");
        countSql.append(" GROUP BY class_grade,detachment,quarter_id,semester_id) a RIGHT JOIN (SELECT DISTINCT sum(stu_num) AS stu_total, " +
                "class_grade,detachment, q.quarter_id, semester_id, quarter, quarter_theme_name, semester, semester_theme_name from " +
                "ycq_redscarf_stu_test ,ycq_redscarf_quarter q, ycq_redscarf_semester s where q.quarter_id = s.quarter_id and " +
                "q.deleted = ? and s.deleted = ? and school_name = ? ");
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(school);
        commonGradeAndDetachment(grade, squadron, sql, countSql, param);
        commonQuarterAndSemester(quarter, semester, sql, countSql, param);
        sql.append(" GROUP BY class_grade,detachment, q.quarter_id, semester_id,quarter, quarter_theme_name," +
                "semester, semester_theme_name) b ON a.class_grade = b.class_grade AND a.detachment = b.detachment " +
                "and a.semester_id = b.semester_id ");
        countSql.append(" GROUP BY class_grade,detachment, q.quarter_id, semester_id,quarter, quarter_theme_name," +
                "semester, semester_theme_name) b ON a.class_grade = b.class_grade AND a.detachment = b.detachment " +
                "and a.semester_id = b.semester_id ");
        // 集体学习审核的关联sql
        if (StringUtils.isNotBlank(operateType)) {
            sql.append(" INNER JOIN ycq_redscarf_collective_learn c ON b.quarter_id = c.quarter_id and b.semester_id = c.semester_id " +
                    "and b.class_grade = c.class_grade and b.detachment = c.detachment and c.is_examine = ? ");
            countSql.append(" INNER JOIN ycq_redscarf_collective_learn c ON b.quarter_id = c.quarter_id and b.semester_id = c.semester_id " +
                    "and b.class_grade = c.class_grade and b.detachment = c.detachment and c.is_examine = ? ");
            param.add(operateType);
        }
    }

    public PageRsp<SquadronAccessVO> pageSearchSquadronAccess(PageReq<SquadronAccessDTO> req) {
        StringBuilder sql = new StringBuilder();
        StringBuilder countSql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchSquadronAccess(sql, countSql, param, req.getQuery().getSquadron(), req.getQuery().getSchool(),
                req.getQuery().getClassGrade(), req.getQuery().getQuarter(), req.getQuery().getSemester());
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<SquadronAccessVO> result;
        if (total > 0) {
            sql.append(" order by learn_rate desc ");
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(SquadronAccessVO.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<SquadronAccessVO> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        rsp.setTotalNum(total);
        rsp.setResult(result);
        return rsp;
    }

    public List<SquadronAccessExcel> excelSearchSquadronAccess(SquadronAccessDTO dto) {
        StringBuilder sql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchSquadronAccess(sql, new StringBuilder(), param, dto.getSquadron(), dto.getSchool(), dto.getClassGrade(),
                dto.getQuarter(), dto.getSemester());
        sql.append(" order by learn_rate desc ");
        return jt.queryForBeans(SquadronAccessExcel.class, sql.toString(), param.toArray());
    }

    public void commonSearchSquadronAccess(StringBuilder sql, StringBuilder countSql, List<Object> param,
                                           String squadron, String school, Integer classGrade, List<String> quarter, List<String> semester) {
        sql.append(PAGE_SEARCH_SQUADRON_ACCESS_SQL);
        countSql.append(PAGE_SEARCH_SQUADRON_ACCESS_COUNT_SQL);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(squadron);
        param.add(school);
        param.add(classGrade);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(squadron);
        param.add(school);
        param.add(classGrade);
        commonQuarterAndSemester(quarter, semester, sql, countSql, param);
        sql.append(" GROUP BY class_grade,detachment, q.quarter_id, semester_id,quarter, quarter_theme_name,semester," +
                "semester_theme_name) b ON a.class_grade = b.class_grade AND a.detachment = b.detachment AND a.semester_id = b.semester_id");
        countSql.append(" GROUP BY class_grade,detachment, q.quarter_id, semester_id,quarter, quarter_theme_name,semester," +
                "semester_theme_name) b ON a.class_grade = b.class_grade AND a.detachment = b.detachment AND a.semester_id = b.semester_id");
    }

    public PageRsp<SquadronDetailsVO> pageSearchSquadronDetails(PageReq<SquadronDetailsDTO> req) {
        StringBuilder sql = new StringBuilder();
        StringBuilder countSql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchSquadronDetails(sql, countSql, param, req.getQuery().getSquadron(), req.getQuery().getSchool(),
                req.getQuery().getClassGrade(), req.getQuery().getName(), req.getQuery().getQuarterId(), req.getQuery().getSemesterId());
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<SquadronDetailsVO> result;
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(SquadronDetailsVO.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<SquadronDetailsVO> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        rsp.setTotalNum(total);
        rsp.setResult(result);
        return rsp;
    }

    public List<SquadronDetailsExcel> excelSearchSquadronDetails(SquadronDetailsDTO dto) {
        StringBuilder sql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonSearchSquadronDetails(sql, new StringBuilder(), param, dto.getSquadron(), dto.getSchool(), dto.getClassGrade(),
                dto.getName(), dto.getQuarterId(), dto.getSemesterId());
        return jt.queryForBeans(SquadronDetailsExcel.class, sql.toString(), param.toArray());
    }

    public void commonSearchSquadronDetails(StringBuilder sql, StringBuilder countSql, List<Object> param, String squadron,
                                            String school, Integer classGrade, List<String> name, String quarterId, String semesterId) {
        sql.append(PAGE_SEARCH_SQUADRON_DETAILS_SQL);
        countSql.append(PAGE_SEARCH_SQUADRON_DETAILS_COUNT_SQL);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(squadron);
        param.add(school);
        param.add(classGrade);
        param.add(quarterId);
        param.add(semesterId);
        if (Objects.nonNull(name)) {
            if (name.size() > 0) {
                sql.append(" and stu_name in (");
                countSql.append(" and stu_name in (");
                commonSqlIn(name, sql, countSql, param);
            }
        }
        sql.append(" ) a LEFT JOIN (select redscarf_id, quarter_id, semester_id, stu_record_id from ycq_redscarf_stu_record " +
                "where deleted = ?) b ON a.redscarf_id = b.redscarf_id AND a.semester_id = b.semester_id");
        countSql.append(" ) a LEFT JOIN (select redscarf_id, quarter_id, semester_id, stu_record_id from ycq_redscarf_stu_record " +
                "where deleted = ?) b ON a.redscarf_id = b.redscarf_id AND a.semester_id = b.semester_id");
        param.add(RedscarfConstants.NOT_DELETED);
    }

    public void commonAreaName(List<String> county, StringBuilder sql, StringBuilder countSql, List<Object> param) {
        if (Objects.nonNull(county)) {
            if (county.size() > 0) {
                sql.append(" and area_name in (");
                countSql.append(" and area_name in (");
                commonSqlIn(county, sql, countSql, param);
            }
        }
    }

    public void commonSchoolName(List<String> school, StringBuilder sql, StringBuilder countSql, List<Object> param) {
        if (Objects.nonNull(school)) {
            if (school.size() > 0) {
                sql.append(" and school_name in (");
                countSql.append(" and school_name in (");
                commonSqlIn(school, sql, countSql, param);
            }
        }

    }

    public void commonGradeAndDetachment(List<String> grade, List<String> squadron, StringBuilder sql, StringBuilder countSql,
                                         List<Object> param) {
        if (Objects.nonNull(grade)) {
            if (grade.size() > 0) {
                sql.append(" and class_grade in (");
                countSql.append(" and class_grade in (");
                commonSqlIn(grade, sql, countSql, param);
            }
        }
        if (Objects.nonNull(squadron)) {
            if (squadron.size() > 0) {
                sql.append(" and detachment in (");
                countSql.append(" and detachment in (");
                commonSqlIn(squadron, sql, countSql, param);
            }
        }
    }

    public void commonQuarterAndSemester(List<String> quarter, List<String> semester, StringBuilder sql, StringBuilder countSql,
                                         List<Object> param) {
        if (Objects.nonNull(quarter)) {
            if (quarter.size() > 0) {
                sql.append(" and q.quarter_id in (");
                countSql.append(" and q.quarter_id in (");
                commonSqlIn(quarter, sql, countSql, param);
            }
        }
        if (Objects.nonNull(semester)) {
            if (semester.size() > 0) {
                sql.append(" and s.semester_id in (");
                countSql.append(" and s.semester_id in (");
                commonSqlIn(semester, sql, countSql, param);
            }
        }
    }

    private void commonSqlIn(List<String> list, StringBuilder sql, StringBuilder countSql, List<Object> param) {
        list.forEach(e -> {
            sql.append("?,");
            countSql.append("?,");
            param.add(e);
        });
        sql.deleteCharAt(sql.length() - 1).append(") ");
        countSql.deleteCharAt(countSql.length() - 1).append(") ");
    }
}
