package com.crunii.micro.service.redscarf.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author baoxy
 * @date 2024-01-04
 */
@Repository
public class LearnRecordDao {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate jt;

    public PageRsp<LearnRecordVO> pageLearnRecord(PageReq<LearnRecordDto> req) {
        StringBuilder sql = new StringBuilder();
        StringBuilder countSql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonLearnRecord(sql, countSql, param, req.getQuery().getStuName());
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<LearnRecordVO> result;
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(LearnRecordVO.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<LearnRecordVO> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        rsp.setTotalNum(total);
        rsp.setResult(result);
        return rsp;
    }

    public List<LearnRecordExcel> exportLearnRecord(LearnRecordDto dto) {
        StringBuilder sql = new StringBuilder();
        List<Object> param = new ArrayList<>();
        commonLearnRecord(sql, new StringBuilder(), param, dto.getStuName());
        return jt.queryForBeans(LearnRecordExcel.class, sql.toString(), param.toArray());
    }

    public void commonLearnRecord(StringBuilder sql, StringBuilder countSql, List<Object> param, String stuName) {
        sql.append("select a.stu_name ,a.redscarf_id,quarter, quarter_theme_name,semester, semester_theme_name from (select " +
                "stu.redscarf_id,stu.stu_name, record.quarter_id, record.semester_id,record.create_time " +
                "from (select * from ycq_redscarf_stu_record where deleted = '0') record inner join " +
                "ycq_redscarf_stu stu on record.redscarf_id = stu.redscarf_id ");
        countSql.append("select count(1) from (select stu.redscarf_id,stu.stu_name, record.quarter_id, record.semester_id," +
                "record.create_time from (select * from ycq_redscarf_stu_record where deleted = '0') record inner join " +
                "ycq_redscarf_stu stu on record.redscarf_id = stu.redscarf_id ");
        if (StringUtils.hasText(stuName)) {
            sql.append(" and stu_name like ? ");
            countSql.append(" and stu_name like ? ");
            param.add("%" + stuName + "%");
        }
        sql.append(") a,ycq_redscarf_quarter q, ycq_redscarf_semester s where a.quarter_id = q.quarter_id and " +
                "a.semester_id = s.semester_id order by a.create_time desc");
        countSql.append(") a,ycq_redscarf_quarter q, ycq_redscarf_semester s where a.quarter_id = q.quarter_id and " +
                "a.semester_id = s.semester_id order by a.create_time desc");
    }

}
