package com.crunii.micro.service.redscarf.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.mircro.service.redscarf.constants.RedscarfConstants;
import com.crunii.mircro.service.redscarf.domain.dto.QuarterDetailsDTO;
import com.crunii.mircro.service.redscarf.domain.dto.QuarterDetailsVO;
import com.crunii.mircro.service.redscarf.domain.dto.QuarterVO;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author baoxy
 * @date 2023-09-04
 */
@Repository
public class QuarterEnterDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public static final String PAGE_SEARCH_QUARTER_SQL = "select quarter_id, quarter, quarter_theme_name as theme_name, " +
            "sort from ycq_redscarf_quarter where deleted = ? ";

    public static final String PAGE_SEARCH_QUARTER_COUNT_SQL = "select count(1) from ycq_redscarf_quarter where deleted = ? ";

    public static final String PAGE_SEARCH_QUARTER_DETAILS_SQL = "select semester_id, semester," +
            "semester_theme_name as theme_name, sort,learning_link as learn_link from ycq_redscarf_semester " +
            "where deleted = ? and quarter_id = ? ";

    public static final String PAGE_SEARCH_QUARTER_DETAILS_COUNT_SQL = "select count(1) from ycq_redscarf_semester where deleted = ? " +
            "and quarter_id = ?";

    public PageRsp<QuarterVO> pageSearchQuarter(PageReq<Object> req) {
        StringBuilder sql = new StringBuilder(PAGE_SEARCH_QUARTER_SQL);
        List<Object> param = new ArrayList<>();
        param.add(RedscarfConstants.NOT_DELETED);
        int total = jt.queryForInt(PAGE_SEARCH_QUARTER_COUNT_SQL, param.toArray());
        List<QuarterVO> quarterVOList;
        String dbType = jt.getDbType();
        if (total > 0) {
            sql.append(" order by sort desc,create_time desc ");
            SqlUtil.appendLimit(dbType, param, sql, req);
            quarterVOList = jt.queryForBeans(QuarterVO.class, sql.toString(), param.toArray());
        } else {
            quarterVOList = new ArrayList<>();
        }
        PageRsp<QuarterVO> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        rsp.setTotalNum(total);
        rsp.setResult(quarterVOList);
        return rsp;
    }

    public PageRsp<QuarterDetailsVO> pageSearchQuarterDetails(PageReq<QuarterDetailsDTO> req) {
        StringBuilder sql = new StringBuilder(PAGE_SEARCH_QUARTER_DETAILS_SQL);
        List<Object> param = new ArrayList<>();
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(req.getQuery().getQuarterId());
        int total = jt.queryForInt(PAGE_SEARCH_QUARTER_DETAILS_COUNT_SQL, param.toArray());
        List<QuarterDetailsVO> quarterDetailsVOList;
        String dyType = jt.getDbType();
        if (total > 0) {
            sql.append(" order by sort desc,create_time desc ");
            SqlUtil.appendLimit(dyType, param, sql, req);
            quarterDetailsVOList = jt.queryForBeans(QuarterDetailsVO.class, sql.toString(), param.toArray());
        } else {
            quarterDetailsVOList = new ArrayList<>();
        }
        PageRsp<QuarterDetailsVO> rsp = new PageRsp<>();
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        rsp.setTotalNum(total);
        rsp.setResult(quarterDetailsVOList);
        return rsp;
    }

    public int checkQuarterSort(int sort, boolean update, String quarterId) {
        String sql = "select count(1) from ycq_redscarf_quarter where deleted = ? and sort = ?";
        List<Object> param = new ArrayList<>();
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(sort);
        if (update) {
            sql += " and quarter_id != ?";
            param.add(quarterId);
        }
        return jt.queryForInt(sql, param.toArray());
    }

    public int checkSemesterSort(int sort, boolean update, String semesterId, String quarterId) {
        String sql = "select count(1) from ycq_redscarf_semester where deleted = ? and sort = ? and quarter_id = ? ";
        List<Object> param = new ArrayList<>();
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(sort);
        param.add(quarterId);
        if (update) {
            sql += " and semester_id != ?";
            param.add(semesterId);
        }
        return jt.queryForInt(sql, param.toArray());
    }

    public int checkRepeatQuarter(String quarter, boolean update, String quarterId) {
        String sql = "select count(1) from ycq_redscarf_quarter where quarter = ? and deleted = ? ";
        List<Object> param = new ArrayList<>();
        param.add(quarter);
        param.add(RedscarfConstants.NOT_DELETED);
        if (update) {
            sql += " and quarter_id != ?";
            param.add(quarterId);
        }
        return jt.queryForInt(sql, param.toArray());
    }

    public int checkRepeatSemester(String semester, boolean update, String semesterId, String quarterId) {
        String sql = "select count(1) from ycq_redscarf_semester where semester = ? and deleted = ? and quarter_id = ? ";
        List<Object> param = new ArrayList<>();
        param.add(semester);
        param.add(RedscarfConstants.NOT_DELETED);
        param.add(quarterId);
        if (update) {
            sql += " and semester_id != ?";
            param.add(semesterId);
        }
        return jt.queryForInt(sql, param.toArray());
    }
}
