package com.crunii.micro.service.redscarf.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqRedscarfCollectiveLearn;
import com.crunii.micro.domain.entity.main.YcqRedscarfCollectiveLearnPic;
import com.crunii.micro.domain.entity.main.YcqRedscarfWorkerRoleTest;
import com.crunii.micro.domain.repo.main.YcqRedscarfCollectiveLearnPicRepo;
import com.crunii.micro.domain.repo.main.YcqRedscarfCollectiveLearnRepo;
import com.crunii.micro.domain.repo.main.YcqRedscarfWorkerRoleTestRepo;
import com.crunii.micro.service.redscarf.jdbc.CollectiveLearnDao;
import com.crunii.micro.service.redscarf.service.CollectiveLearnService;
import com.crunii.mircro.service.redscarf.constants.RedscarfConstants;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author baoxy
 * @date 2023-09-02
 */
@Service
public class CollectiveLearnServiceImpl implements CollectiveLearnService {

    @Resource
    private CollectiveLearnDao collectiveLearnDao;

    @Resource
    private YcqRedscarfCollectiveLearnRepo ycqRedscarfCollectiveLearnRepo;

    @Resource
    private YcqRedscarfCollectiveLearnPicRepo ycqRedscarfCollectiveLearnPicRepo;

    @Resource
    private YcqRedscarfWorkerRoleTestRepo ycqRedscarfWorkerRoleTestRepo;

    @Value("${upload.url}")
    private String urlPrefix;

    @Override
    public Result<Object> submitContent(CollectiveLearnDTO dto, Worker worker) {
        if (StringUtils.isNotBlank(dto.getLocation())) {
            if (dto.getLocation().length() > RedscarfConstants.LENGTH_FIFTY) {
                return Result.ofFail("地点不能超过50个字,请重新填写");
            }
        }
        YcqRedscarfCollectiveLearn learn = new YcqRedscarfCollectiveLearn();
        learn.setLoginId(worker.getLoginId());
        learn.setQuarterId(Long.valueOf(dto.getQuarterId()));
        learn.setSemesterId(Long.valueOf(dto.getSemesterId()));
        learn.setLocation(dto.getLocation());
        YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
        // 没有配置权限的用户直接返回空数据
        if (Objects.isNull(test)) {
            return new Result<>();
        }
        // 若实体类的区县名称county没有值,说明此时用户没有学校权限，有中队权限.页面调用直接调用中队权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的中队
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        if (StringUtils.isBlank(dto.getDetachment())) {
            learn.setDetachment(test.getDetachment());
        } else {
            learn.setDetachment(dto.getDetachment());
        }
        if (StringUtils.isBlank(dto.getSchoolName())) {
            learn.setSchoolName(test.getSchoolName());
        } else {
            learn.setSchoolName(dto.getSchoolName());
        }
        if (Objects.isNull(dto.getClassGrade())) {
            learn.setClassGrade(test.getClassGrade());
        } else {
            learn.setClassGrade(dto.getClassGrade());
        }
        learn.setDeleted(RedscarfConstants.NOT_DELETED);
        learn.setIsExamine(RedscarfConstants.ZERO_INTEGER);
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        try {
            Date date = dateFormat.parse(dto.getTime());
            learn.setCreateTime(date);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Long collectiveLearnId = ycqRedscarfCollectiveLearnRepo.saveAndFlush(learn).getCollectiveLearnId();
        List<YcqRedscarfCollectiveLearnPic> learnPicList = new ArrayList<>();
        dto.getPicList().forEach(e -> {
            YcqRedscarfCollectiveLearnPic learnPic = new YcqRedscarfCollectiveLearnPic();
            learnPic.setCollectiveLearnId(collectiveLearnId);
            learnPic.setDeleted(RedscarfConstants.NOT_DELETED);
            learnPic.setFileKey(e);
            learnPicList.add(learnPic);
        });
        ycqRedscarfCollectiveLearnPicRepo.saveAllAndFlush(learnPicList);
        return Result.ofSuccess(collectiveLearnId);
    }

    @Override
    public Result<Object> examine(CollectiveLearnExamineDTO dto, Worker worker) {
        YcqRedscarfCollectiveLearn learn = ycqRedscarfCollectiveLearnRepo.getOne(Long.valueOf(dto.getCollectiveLearnId()));
        learn.setIsExamine(RedscarfConstants.ONE_INTEGER);
        learn.setExamineState(dto.getIsPass());
        ycqRedscarfCollectiveLearnRepo.saveAndFlush(learn);
        return Result.ofSuccess("操作成功!");
    }

    @Override
    public Result<CollectiveLearnSearchExamineVO> searchExamine(CollectiveLearnSearchExamineDTO dto, Worker worker) {
        CollectiveLearnSearchExamineVO vo = collectiveLearnDao.searchExamine(dto);
        vo.setQuarter(RedscarfConstants.checkQuarter(vo.getQuarter()) + " " + vo.getQuarterThemeName());
        vo.setSemester(RedscarfConstants.checkSemester(vo.getSemester()) + " " + vo.getSemesterThemeName());
        List<YcqRedscarfCollectiveLearnPic> learnPicList = ycqRedscarfCollectiveLearnPicRepo.findAllByCollectiveLearnId
                (Long.valueOf(dto.getCollectiveLearnId()));
        List<String> picList = new ArrayList<>();
        learnPicList.forEach(e -> picList.add(urlPrefix + e.getFileKey()));
        vo.setPicList(picList);
        return Result.ofSuccess(vo);
    }

    @Override
    public Result<CollectiveLearnNotExamineNumVO> notExamineNum(CollectiveLearnNotExamineNumDTO dto, Worker worker) {
        // 若实体类的学校名称schoolName没有值,说明此时用户没有区县权限.页面调用直接调用学校权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的学校
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        if (StringUtils.isBlank(dto.getSchoolName())) {
            YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
            // 没有配置权限的用户直接返回空数据
            if (Objects.isNull(test)) {
                return new Result<>();
            }
            dto.setSchoolName(test.getSchoolName());
        }
        return Result.ofSuccess(collectiveLearnDao.notExamineNum(dto));
    }

    @Override
    public List<SearchQuarterVO> searchNotCompleteQuarter(SearchNotCompleteQuarterDTO dto, Worker worker) {
        // 若实体类的中队,学校,年级没有值,说明此时用户没有学校权限.页面调用直接调用中队权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的中队,学校,年级
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
        // 没有配置权限的用户直接返回空数据
        if (Objects.isNull(test)) {
            return new ArrayList<>();
        }
        if (StringUtils.isBlank(dto.getSquadron())) {
            dto.setSquadron(test.getDetachment());
        }
        if (StringUtils.isBlank(dto.getSchool())) {
            dto.setSchool(test.getSchoolName());
        }
        if (Objects.isNull(dto.getClassGrade())) {
            dto.setClassGrade(test.getClassGrade());
        }
        List<SearchQuarterVO> list = collectiveLearnDao.searchNotCompleteQuarter(dto);
        list.forEach(e -> e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName()));
        return list;
    }

    @Override
    public List<SearchSemesterVO> searchNotCompleteSemester(SearchNotCompleteSemesterDTO dto, Worker worker) {
        // 若实体类的中队,学校,年级没有值,说明此时用户没有学校权限.页面调用直接调用中队权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的中队,学校,年级
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
        // 没有配置权限的用户直接返回空数据
        if (Objects.isNull(test)) {
            return new ArrayList<>();
        }
        if (StringUtils.isBlank(dto.getSquadron())) {
            dto.setSquadron(test.getDetachment());
        }
        if (StringUtils.isBlank(dto.getSchool())) {
            dto.setSchool(test.getSchoolName());
        }
        if (Objects.isNull(dto.getClassGrade())) {
            dto.setClassGrade(test.getClassGrade());
        }
        List<SearchSemesterVO> list = collectiveLearnDao.searchNotCompleteSemester(dto);
        list.forEach(e -> e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName()));
        return list;
    }
}
