package com.crunii.micro.service.redscarf.service.impl;

import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqRedscarfQuarter;
import com.crunii.micro.domain.entity.main.YcqRedscarfSemester;
import com.crunii.micro.domain.entity.main.YcqRedscarfWorkerRoleTest;
import com.crunii.micro.domain.repo.main.YcqRedscarfQuarterRepo;
import com.crunii.micro.domain.repo.main.YcqRedscarfSemesterRepo;
import com.crunii.micro.domain.repo.main.YcqRedscarfWorkerRoleTestRepo;
import com.crunii.micro.service.redscarf.jdbc.CommonListDao;
import com.crunii.micro.service.redscarf.service.CommonListService;
import com.crunii.mircro.service.redscarf.constants.RedscarfConstants;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author baoxy
 * @date 2023-09-11
 */
@Service
public class CommonListServiceImpl implements CommonListService {

    @Resource
    private CommonListDao commonListDao;

    @Resource
    private YcqRedscarfQuarterRepo ycqRedscarfQuarterRepo;

    @Resource
    private YcqRedscarfSemesterRepo ycqRedscarfSemesterRepo;

    @Resource
    private YcqRedscarfWorkerRoleTestRepo ycqRedscarfWorkerRoleTestRepo;

    @Override
    public List<SearchQuarterVO> searchQuarter() {
        List<YcqRedscarfQuarter> ycqRedscarfQuarterList = ycqRedscarfQuarterRepo.findAllByDeleted(RedscarfConstants.NOT_DELETED);
        List<SearchQuarterVO> searchQuarterVOList = new ArrayList<>();
        ycqRedscarfQuarterList.forEach(e -> {
            SearchQuarterVO vo = new SearchQuarterVO();
            vo.setQuarterId(e.getQuarterId().toString());
            // 将学季组合成第一季度 主题名称,例如第一季度 XXX主提或者特辑 XXX主题
            vo.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            searchQuarterVOList.add(vo);
        });
        return searchQuarterVOList;
    }

    @Override
    public List<SearchSemesterVO> searchSemester(SearchSemesterDTO dto) {
        List<YcqRedscarfSemester> semesterList = ycqRedscarfSemesterRepo.findAllByDeletedAndQuarterIdIn(RedscarfConstants.NOT_DELETED,
                dto.getQuarterId());
        List<SearchSemesterVO> searchSemesterVOList = new ArrayList<>();
        if (semesterList.size() > 0) {
            semesterList.forEach(e -> {
                SearchSemesterVO vo = new SearchSemesterVO();
                vo.setSemesterId(e.getSemesterId().toString());
                // 将学季组合成第一期 主题名称,例如第一期 XXX主提
                vo.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
                searchSemesterVOList.add(vo);
            });
        }
        return searchSemesterVOList;
    }

    @Override
    public List<SearchCountyVO> searchCounty() {
        return commonListDao.searchCounty();
    }

    @Override
    public List<SearchSchoolVO> searchSchool(SearchSchoolDTO dto, Worker worker) {
        // 若实体类的区县名称county没有值,说明此时用户没有市级权限，有区县权限.页面调用直接调用区县权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的区县
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        if (StringUtils.isBlank(dto.getCounty())) {
            YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
            // 没有配置权限的用户直接返回空数据
            if (Objects.isNull(test)) {
                return new ArrayList<>();
            }
            dto.setCounty(test.getAreaName());
        }
        return commonListDao.searchSchool(dto);
    }

    @Override
    public List<SearchClassGradeVO> searchClassGrade(SearchClassGradeDTO dto, Worker worker) {
        // 若实体类的学校名称school没有值,说明此时用户没有区县权限.页面调用直接调用学校权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的学校
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        if (StringUtils.isBlank(dto.getSchool())) {
            YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
            // 没有配置权限的用户直接返回空数据
            if (Objects.isNull(test)) {
                return new ArrayList<>();
            }
            dto.setSchool(test.getSchoolName());
        }
        return commonListDao.searchClassGrade(dto);
    }

    @Override
    public List<SearchSquadronVO> searchSquadron(SearchSquadronDTO dto, Worker worker) {
        // 若实体类的学校名称school没有值,说明此时用户没有区县权限.页面调用直接调用学校权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的学校
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        if (StringUtils.isBlank(dto.getSchool())) {
            YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
            // 没有配置权限的用户直接返回空数据
            if (Objects.isNull(test)) {
                return new ArrayList<>();
            }
            dto.setSchool(test.getSchoolName());
        }
        List<SearchSquadronVO> voList = commonListDao.searchSquadron(dto);
        voList.forEach(e -> e.setSquadron(e.getSquadron() + RedscarfConstants.DETACHMENT_NAME));
        return voList;
    }

    @Override
    public List<SearchStudentVO> searchStudent(SearchStudentDTO dto, Worker worker) {
        // 若实体类的中队,学校,年级没有值,说明此时用户没有学校权限.页面调用直接调用中队权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的中队,学校,年级
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
        // 没有配置权限的用户直接返回空数据
        if (Objects.isNull(test)) {
            return new ArrayList<>();
        }
        if (StringUtils.isBlank(dto.getSquadron())) {
            dto.setSquadron(test.getDetachment());
        }
        if (StringUtils.isBlank(dto.getSchool())) {
            dto.setSchool(test.getSchoolName());
        }
        if (Objects.isNull(dto.getClassGrade())) {
            dto.setClassGrade(test.getClassGrade());
        }
        return commonListDao.searchStudent(dto);
    }
}
