package com.crunii.micro.service.redscarf.service.impl;

import com.alibaba.excel.EasyExcel;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqRedscarfWorkerRoleTest;
import com.crunii.micro.domain.repo.main.YcqRedscarfWorkerRoleTestRepo;
import com.crunii.micro.service.redscarf.jdbc.LearnDataDao;
import com.crunii.micro.service.redscarf.service.LearnDataService;
import com.crunii.micro.service.redscarf.utils.CustomImageModifyHandler;
import com.crunii.mircro.service.redscarf.constants.RedscarfConstants;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Objects;

/**
 * @author baoxy
 * @date 2023-09-02
 */
@Service
public class LearnDataServiceImpl implements LearnDataService {

    @Resource
    private LearnDataDao learnDataDao;

    @Resource
    private YcqRedscarfWorkerRoleTestRepo ycqRedscarfWorkerRoleTestRepo;

    @Override
    public PageRsp<CityAccessVO> pageSearchCityAccess(PageReq<CityAccessDTO> req, Worker worker) {
        PageRsp<CityAccessVO> rsp = learnDataDao.pageSearchCityAccess(req);
        List<CityAccessVO> result = rsp.getResult();
        result.forEach(e -> {
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
        });
        rsp.setResult(result);
        return rsp;
    }

    @Override
    public PageRsp<CountyAccessVO> pageSearchCountyAccess(PageReq<CountyAccessDTO> req, Worker worker) {
        // 若实体类的区县名称county没有值,说明此时用户没有市级权限.页面调用直接调用区县权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的区县
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        if (StringUtils.isBlank(req.getQuery().getCounty())) {
            YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
            // 没有配置权限的用户直接返回空数据
            if (Objects.isNull(test)) {
                return new PageRsp<>();
            }
            req.getQuery().setCounty(test.getAreaName());
        }
        PageRsp<CountyAccessVO> rsp = learnDataDao.pageSearchCountyAccess(req);
        List<CountyAccessVO> result = rsp.getResult();
        result.forEach(e -> {
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
        });
        rsp.setResult(result);
        return rsp;
    }

    @Override
    public PageRsp<SchoolAccessVO> pageSearchSchoolAccess(PageReq<SchoolAccessDTO> req, Worker worker) {
        // 若实体类的学校名称schoolName没有值,说明此时用户没有区县权限.页面调用直接调用学校权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的学校
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        if (StringUtils.isBlank(req.getQuery().getSchool())) {
            YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
            // 没有配置权限的用户直接返回空数据
            if (Objects.isNull(test)) {
                return new PageRsp<>();
            }
            req.getQuery().setSchool(test.getSchoolName());
        }
        PageRsp<SchoolAccessVO> rsp = learnDataDao.pageSearchSchoolAccess(req);
        List<SchoolAccessVO> result = rsp.getResult();
        result.forEach(e -> {
            e.setSquadron(e.getSquadron() + RedscarfConstants.DETACHMENT_NAME);
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
        });
        rsp.setResult(result);
        return rsp;
    }

    @Override
    public PageRsp<SquadronAccessVO> pageSearchSquadronAccess(PageReq<SquadronAccessDTO> req, Worker worker) {
        // 若实体类的中队,学校,年级没有值,说明此时用户没有学校权限.页面调用直接调用中队权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的中队,学校,年级
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
        // 没有配置权限的用户直接返回空数据
        if (Objects.isNull(test)) {
            return new PageRsp<>();
        }
        if (StringUtils.isBlank(req.getQuery().getSquadron())) {
            req.getQuery().setSquadron(test.getDetachment());
        }
        if (StringUtils.isBlank(req.getQuery().getSchool())) {
            req.getQuery().setSchool(test.getSchoolName());
        }
        if (Objects.isNull(req.getQuery().getClassGrade())) {
            req.getQuery().setClassGrade(test.getClassGrade());
        }

        PageRsp<SquadronAccessVO> rsp = learnDataDao.pageSearchSquadronAccess(req);
        List<SquadronAccessVO> result = rsp.getResult();
        result.forEach(e -> {
            e.setSchool(req.getQuery().getSchool());
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
        });
        return rsp;
    }

    @Override
    public PageRsp<SquadronDetailsVO> pageSearchSquadronDetails(PageReq<SquadronDetailsDTO> req, Worker worker) {
        // 若实体类的中队,学校,年级没有值,说明此时用户没有学校权限.页面调用直接调用中队权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的中队,学校,年级
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
        // 没有配置权限的用户直接返回空数据
        if (Objects.isNull(test)) {
            return new PageRsp<>();
        }
        if (StringUtils.isBlank(req.getQuery().getSquadron())) {
            req.getQuery().setSquadron(test.getDetachment());
        }
        if (StringUtils.isBlank(req.getQuery().getSchool())) {
            req.getQuery().setSchool(test.getSchoolName());
        }
        if (Objects.isNull(req.getQuery().getClassGrade())) {
            req.getQuery().setClassGrade(test.getClassGrade());
        }
        PageRsp<SquadronDetailsVO> rsp = learnDataDao.pageSearchSquadronDetails(req);
        List<SquadronDetailsVO> result = rsp.getResult();
        result.forEach(e -> {
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
        });
        rsp.setResult(result);
        return rsp;
    }

    @Override
    public void exportCityAccess(HttpServletResponse response, CityAccessDTO dto, Worker worker) {
        List<CityAccessExcel> excelList = learnDataDao.excelSearchCityAccess(dto);
        excelList.forEach(e -> {
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
        });
        String fileName = null;
        try {
            fileName = URLEncoder.encode("市级权限列表", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        commonSetResponse(response, fileName);
        // 写入数据
        try {
            EasyExcel.write(response.getOutputStream(), CityAccessExcel.class).registerWriteHandler(new CustomImageModifyHandler()).
                    sheet("sheet").doWrite(excelList);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void exportCountyAccess(HttpServletResponse response, CountyAccessDTO dto, Worker worker) {
        // 若实体类的区县名称county没有值,说明此时用户没有市级权限.页面调用直接调用区县权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的区县
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        if (StringUtils.isBlank(dto.getCounty())) {
            YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
            // 没有配置权限的用户直接返回空数据
            if (Objects.isNull(test)) {
                return;
            }
            dto.setCounty(test.getAreaName());
        }
        List<CountyAccessExcel> excelList = learnDataDao.excelSearchCountyAccess(dto);
        excelList.forEach(e -> {
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
        });
        String fileName = null;
        try {
            fileName = URLEncoder.encode("区县权限列表", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        commonSetResponse(response, fileName);
        // 写入数据
        try {
            EasyExcel.write(response.getOutputStream(), CountyAccessExcel.class).registerWriteHandler(new CustomImageModifyHandler()).
                    sheet("sheet").doWrite(excelList);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void exportSchoolAccess(HttpServletResponse response, SchoolAccessDTO dto, Worker worker) {
        // 若实体类的学校名称school没有值,说明此时用户没有区县权限.页面调用直接调用学校权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的学校
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        if (StringUtils.isBlank(dto.getSchool())) {
            YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
            // 没有配置权限的用户直接返回空数据
            if (Objects.isNull(test)) {
                return;
            }
            dto.setSchool(test.getSchoolName());
        }

        String fileName = null;
        try {
            fileName = URLEncoder.encode("学校权限列表", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        commonSetResponse(response, fileName);
        // 写入数据
        try {
            if (StringUtils.isNotBlank(dto.getOperateType())) {
                if (RedscarfConstants.ONE_INTEGER.equals(Integer.valueOf(dto.getOperateType()))) {
                    // 已审核的学校权限列表需要添加审核状态的字样
                    List<SchoolAccessExcelExamine> excelList = learnDataDao.excelSearchSchoolAccessExamine(dto);
                    excelList.forEach(e -> {
                        String examineState = e.getExamineState().equals(RedscarfConstants.ONE_STRING) ? RedscarfConstants.PASS_FIELD :
                                RedscarfConstants.NOT_PASS_FIELD;
                        e.setExamineState(examineState);
                        e.setSquadron(e.getSquadron() + RedscarfConstants.DETACHMENT_NAME);
                        e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
                        e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
                    });
                    EasyExcel.write(response.getOutputStream(), SchoolAccessExcelExamine.class).registerWriteHandler(new CustomImageModifyHandler()).
                            sheet("sheet").doWrite(excelList);
                } else {
                    List<SchoolAccessExcel> excelList = learnDataDao.excelSearchSchoolAccess(dto);
                    excelList.forEach(e -> {
                        e.setSquadron(e.getSquadron() + RedscarfConstants.DETACHMENT_NAME);
                        e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
                        e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
                    });
                    EasyExcel.write(response.getOutputStream(), SchoolAccessExcel.class).registerWriteHandler(new CustomImageModifyHandler()).
                            sheet("sheet").doWrite(excelList);
                }
            } else {
                List<SchoolAccessExcel> excelList = learnDataDao.excelSearchSchoolAccess(dto);
                excelList.forEach(e -> {
                    e.setSquadron(e.getSquadron() + RedscarfConstants.DETACHMENT_NAME);
                    e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
                    e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
                });
                EasyExcel.write(response.getOutputStream(), SchoolAccessExcel.class).registerWriteHandler(new CustomImageModifyHandler()).
                        sheet("sheet").doWrite(excelList);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void exportSquadronAccess(HttpServletResponse response, SquadronAccessDTO dto, Worker worker) {
        // 若实体类的中队,学校,年级没有值,说明此时用户没有学校权限.页面调用直接调用中队权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的中队,学校,年级
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
        // 没有配置权限的用户直接返回空数据
        if (Objects.isNull(test)) {
            return;
        }
        if (StringUtils.isBlank(dto.getSquadron())) {
            dto.setSquadron(test.getDetachment());
        }
        if (StringUtils.isBlank(dto.getSchool())) {
            dto.setSchool(test.getSchoolName());
        }
        if (Objects.isNull(dto.getClassGrade())) {
            dto.setClassGrade(test.getClassGrade());
        }
        List<SquadronAccessExcel> excelList = learnDataDao.excelSearchSquadronAccess(dto);
        excelList.forEach(e -> {
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
        });
        String fileName = null;
        try {
            fileName = URLEncoder.encode("中队权限列表", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        commonSetResponse(response, fileName);
        // 写入数据
        try {
            EasyExcel.write(response.getOutputStream(), SquadronAccessExcel.class).registerWriteHandler(new CustomImageModifyHandler()).
                    sheet("sheet").doWrite(excelList);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void exportSquadronDetails(HttpServletResponse response, SquadronDetailsDTO dto, Worker worker) {
        // 若实体类的中队,学校,年级没有值,说明此时用户没有学校权限.页面调用直接调用中队权限列表.
        // 因此需要从用户实体类调用暂定的权限表来查询用户当前的中队,学校,年级
        // 注意:暂定权限表是临时的,后续会用正式的权限数据表
        YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
        // 没有配置权限的用户直接返回空数据
        if (Objects.isNull(test)) {
            return;
        }
        if (StringUtils.isBlank(dto.getSquadron())) {
            dto.setSquadron(test.getDetachment());
        }
        if (StringUtils.isBlank(dto.getSchool())) {
            dto.setSchool(test.getSchoolName());
        }
        if (Objects.isNull(dto.getClassGrade())) {
            dto.setClassGrade(test.getClassGrade());
        }
        List<SquadronDetailsExcel> excelList = learnDataDao.excelSearchSquadronDetails(dto);
        excelList.forEach(e -> {
            // 判断是否学习(1是,0否)
            String isLearnExcel = e.getIsLearn() == 1 ? "是" : "否";
            e.setIsLearnExcel(isLearnExcel);
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
        });
        String fileName = null;
        try {
            fileName = URLEncoder.encode("中队详情列表", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        commonSetResponse(response, fileName);
        // 写入数据
        try {
            EasyExcel.write(response.getOutputStream(), SquadronDetailsExcel.class).registerWriteHandler(new CustomImageModifyHandler()).
                    sheet("sheet").doWrite(excelList);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public WorkerAccessVO workerAccess(Worker worker) {
        YcqRedscarfWorkerRoleTest test = ycqRedscarfWorkerRoleTestRepo.findByLoginId(worker.getLoginId());
        WorkerAccessVO vo = new WorkerAccessVO();
        if (Objects.nonNull(test)) {
            vo.setRoleType(test.getRoleType());
            if (RedscarfConstants.ROLE_TYPE_2 == test.getRoleType()) {
                // 用户为区县权限
                vo.setCommonName(test.getAreaName());
            }
            if (RedscarfConstants.ROLE_TYPE_3 == test.getRoleType()) {
                // 用户为学校权限
                vo.setCommonName(test.getSchoolName());
            }
            if (RedscarfConstants.ROLE_TYPE_4 == test.getRoleType()) {
                // 用户为中队权限, 该值为年级和中队的拼接,例如2023级-一中队
                String commonName = test.getClassGrade() + "级-" + test.getDetachment() + RedscarfConstants.DETACHMENT_NAME;
                vo.setCommonName(commonName);
            }
        }
        return vo;
    }

    public void commonSetResponse(HttpServletResponse response, String fileName) {
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
    }

}
