package com.crunii.micro.service.redscarf.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.redscarf.jdbc.LearnRecordDao;
import com.crunii.micro.service.redscarf.service.LearnRecordService;
import com.crunii.mircro.service.redscarf.constants.RedscarfConstants;
import com.crunii.mircro.service.redscarf.domain.dto.LearnRecordDto;
import com.crunii.mircro.service.redscarf.domain.dto.LearnRecordExcel;
import com.crunii.mircro.service.redscarf.domain.dto.LearnRecordVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * @author baoxy
 * @date 2024-01-04
 */
@Service
public class LearnRecordServiceImpl implements LearnRecordService {

    @Resource
    private LearnRecordDao learnRecordDao;

    @Override
    public PageRsp<LearnRecordVO> pageLearnRecord(PageReq<LearnRecordDto> req) {
        PageRsp<LearnRecordVO> rsp = learnRecordDao.pageLearnRecord(req);
        List<LearnRecordVO> result = rsp.getResult();
        result.forEach(e -> {
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
            // 是否学习固定为是
            e.setIsLearn(1);
        });
        return rsp;
    }

    @Override
    public void exportLearnRecord(LearnRecordDto dto) {
        List<LearnRecordExcel> exportLearnRecord = learnRecordDao.exportLearnRecord(dto);
        exportLearnRecord.forEach(e -> {
            e.setQuarter(RedscarfConstants.checkQuarter(e.getQuarter()) + " " + e.getQuarterThemeName());
            e.setSemester(RedscarfConstants.checkSemester(e.getSemester()) + " " + e.getSemesterThemeName());
        });
        try {
            ExcelUtil.writeSingleExcel("学习记录列表", null, exportLearnRecord, LearnRecordExcel.class);
        } catch (IOException e) {
            throw new BusinessException("导出失败");
        }
    }
}
