package com.crunii.micro.service.redscarf.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqRedscarfQuarter;
import com.crunii.micro.domain.entity.main.YcqRedscarfSemester;
import com.crunii.micro.domain.repo.main.YcqRedscarfQuarterRepo;
import com.crunii.micro.domain.repo.main.YcqRedscarfSemesterRepo;
import com.crunii.micro.service.redscarf.jdbc.QuarterEnterDao;
import com.crunii.micro.service.redscarf.service.QuarterEnterService;
import com.crunii.mircro.service.redscarf.constants.RedscarfConstants;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author baoxy
 * @date 2023-09-02
 */
@Service
public class QuarterEnterServiceImpl implements QuarterEnterService {

    @Resource
    private QuarterEnterDao quarterEnterDao;

    @Resource
    private YcqRedscarfQuarterRepo ycqRedscarfQuarterRepo;

    @Resource
    private YcqRedscarfSemesterRepo ycqRedscarfSemesterRepo;

    @Override
    public PageRsp<QuarterVO> pageSearchQuarter(PageReq<Object> req) {
        PageRsp<QuarterVO> rsp = quarterEnterDao.pageSearchQuarter(req);
        List<QuarterVO> result = rsp.getResult();
        result.forEach(e -> e.setQuarter(RedscarfConstants.checkQuarter((e.getQuarter()))));
        rsp.setResult(result);
        return rsp;
    }

    @Override
    public PageRsp<QuarterDetailsVO> pageSearchQuarterDetails(PageReq<QuarterDetailsDTO> req) {
        PageRsp<QuarterDetailsVO> rsp = quarterEnterDao.pageSearchQuarterDetails(req);
        List<QuarterDetailsVO> result = rsp.getResult();
        result.forEach(e -> e.setSemester(RedscarfConstants.checkSemester((e.getSemester()))));
        rsp.setResult(result);
        return rsp;
    }

    @Override
    public Result<Object> createQuarter(CreateQuarterDTO dto, Worker worker) {
        Result<Object> checkThemeNameLength = checkThemeNameLength(dto.getThemeName());
        if (Objects.nonNull(checkThemeNameLength)) {
            return checkThemeNameLength;
        }
        Result<Object> checkRepeatQuarter = checkRepeatQuarter(dto.getQuarter(), false, null);
        if (Objects.nonNull(checkRepeatQuarter)) {
            return checkRepeatQuarter;
        }
        // 需求明说可以不做重复校验
//        Result<Object> checkQuarterSort = checkQuarterSort(dto.getSort(), false, null);
//        if (Objects.nonNull(checkQuarterSort)) {
//            return checkQuarterSort;
//        }
        YcqRedscarfQuarter ycqRedscarfQuarter = new YcqRedscarfQuarter();
        ycqRedscarfQuarter.setQuarter(dto.getQuarter());
        ycqRedscarfQuarter.setQuarterThemeName(dto.getThemeName());
        ycqRedscarfQuarter.setSort(dto.getSort());
        ycqRedscarfQuarter.setCreatorName(worker.getWorkerName());
        ycqRedscarfQuarter.setUpdaterName(worker.getWorkerName());
        ycqRedscarfQuarter.setCreateTime(new Date());
        ycqRedscarfQuarter.setUpdateTime(new Date());
        ycqRedscarfQuarter.setDeleted(RedscarfConstants.NOT_DELETED);
        Long id = ycqRedscarfQuarterRepo.saveAndFlush(ycqRedscarfQuarter).getQuarterId();
        return Result.ofSuccess(id);
    }

    @Override
    public Result<Object> updateQuarter(UpdateQuarterDTO dto, Worker worker) {
        Result<Object> checkThemeNameLength = checkThemeNameLength(dto.getThemeName());
        if (Objects.nonNull(checkThemeNameLength)) {
            return checkThemeNameLength;
        }
        Result<Object> checkRepeatQuarter = checkRepeatQuarter(dto.getQuarter(), true, dto.getQuarterId());
        if (Objects.nonNull(checkRepeatQuarter)) {
            return checkRepeatQuarter;
        }
        // 需求明说可以不做重复校验
//        Result<Object> checkQuarterSort = checkQuarterSort(dto.getSort(), true, dto.getQuarterId());
//        if (Objects.nonNull(checkQuarterSort)) {
//            return checkQuarterSort;
//        }
        YcqRedscarfQuarter ycqRedscarfQuarter = ycqRedscarfQuarterRepo.getOne(Long.valueOf(dto.getQuarterId()));
        ycqRedscarfQuarter.setQuarter(dto.getQuarter());
        ycqRedscarfQuarter.setQuarterThemeName(dto.getThemeName());
        ycqRedscarfQuarter.setSort(dto.getSort());
        ycqRedscarfQuarter.setUpdaterName(worker.getWorkerName());
        ycqRedscarfQuarter.setUpdateTime(new Date());
        ycqRedscarfQuarterRepo.saveAndFlush(ycqRedscarfQuarter);
        return Result.ofSuccess("操作成功!");
    }

    @Override
    public Result<Object> createSemester(CreateSemesterDTO dto, Worker worker) {
        Result<Object> checkThemeNameLength = checkThemeNameLength(dto.getThemeName());
        if (Objects.nonNull(checkThemeNameLength)) {
            return checkThemeNameLength;
        }
        Result<Object> checkRepeatSemester = checkRepeatSemester(dto.getSemester(), false, null, dto.getQuarterId());
        if (Objects.nonNull(checkRepeatSemester)) {
            return checkRepeatSemester;
        }
        // 需求明说可以不做重复校验
//        Result<Object> checkSemesterSort = checkSemesterSort(dto.getSort(), false, null, dto.getQuarterId());
//        if (Objects.nonNull(checkSemesterSort)) {
//            return checkSemesterSort;
//        }
        YcqRedscarfSemester ycqRedscarfSemester = new YcqRedscarfSemester();
        ycqRedscarfSemester.setQuarterId(Long.valueOf(dto.getQuarterId()));
        ycqRedscarfSemester.setSemester(dto.getSemester());
        ycqRedscarfSemester.setSemesterThemeName(dto.getThemeName());
        ycqRedscarfSemester.setLearningLink(dto.getLearnLink());
        ycqRedscarfSemester.setCreatorName(worker.getWorkerName());
        ycqRedscarfSemester.setUpdaterName(worker.getWorkerName());
        ycqRedscarfSemester.setCreateTime(new Date());
        ycqRedscarfSemester.setUpdateTime(new Date());
        ycqRedscarfSemester.setSort(dto.getSort());
        ycqRedscarfSemester.setDeleted(RedscarfConstants.NOT_DELETED);
        Long id = ycqRedscarfSemesterRepo.saveAndFlush(ycqRedscarfSemester).getSemesterId();
        return Result.ofSuccess(id);
    }

    @Override
    public Result<Object> updateSemester(UpdateSemesterDTO dto, Worker worker) {
        Result<Object> checkThemeNameLength = checkThemeNameLength(dto.getThemeName());
        if (Objects.nonNull(checkThemeNameLength)) {
            return checkThemeNameLength;
        }
        Result<Object> checkRepeatSemester = checkRepeatSemester(dto.getSemester(), true, dto.getSemesterId(), dto.getQuarterId());
        if (Objects.nonNull(checkRepeatSemester)) {
            return checkRepeatSemester;
        }
        // 需求明说可以不做重复校验
//        Result<Object> checkSemesterSort = checkSemesterSort(dto.getSort(), true, dto.getSemesterId(), dto.getQuarterId());
//        if (Objects.nonNull(checkSemesterSort)) {
//            return checkSemesterSort;
//        }
        YcqRedscarfSemester ycqRedscarfSemester = ycqRedscarfSemesterRepo.getOne(Long.valueOf(dto.getSemesterId()));
        ycqRedscarfSemester.setSemester(dto.getSemester());
        ycqRedscarfSemester.setLearningLink(dto.getLearnLink());
        ycqRedscarfSemester.setSemesterThemeName(dto.getThemeName());
        ycqRedscarfSemester.setSort(dto.getSort());
        ycqRedscarfSemester.setUpdaterName(worker.getWorkerName());
        ycqRedscarfSemester.setUpdateTime(new Date());
        ycqRedscarfSemesterRepo.saveAndFlush(ycqRedscarfSemester);
        return Result.ofSuccess("操作成功!");
    }

    @Override
    public Result<Object> dealQuarter(DelQuarterDTO dto, Worker worker) {
        List<YcqRedscarfQuarter> deleteList = new ArrayList<>();
        dto.getQuarterIdList().forEach(e -> {
            YcqRedscarfQuarter ycqRedscarfQuarter = ycqRedscarfQuarterRepo.getOne(Long.valueOf(e));
            ycqRedscarfQuarter.setDeleted(RedscarfConstants.DELETED);
            ycqRedscarfQuarter.setUpdateTime(new Date());
            ycqRedscarfQuarter.setUpdaterName(worker.getWorkerName());
            deleteList.add(ycqRedscarfQuarter);
        });
        ycqRedscarfQuarterRepo.saveAllAndFlush(deleteList);
        return Result.ofSuccess("操作成功!");
    }

    @Override
    public Result<Object> delSemester(DelSemesterDTO dto, Worker worker) {
        List<YcqRedscarfSemester> deleteList = new ArrayList<>();
        dto.getSemesterIdList().forEach(e -> {
            YcqRedscarfSemester ycqRedscarfSemester = ycqRedscarfSemesterRepo.getOne(Long.valueOf(e));
            ycqRedscarfSemester.setDeleted(RedscarfConstants.DELETED);
            ycqRedscarfSemester.setUpdaterName(worker.getWorkerName());
            ycqRedscarfSemester.setUpdateTime(new Date());
            deleteList.add(ycqRedscarfSemester);
        });
        ycqRedscarfSemesterRepo.saveAllAndFlush(deleteList);
        return Result.ofSuccess("操作成功!");
    }

/*    public Result<Object> checkQuarterSort(int sort, boolean update, String quarterId) {
        int count = quarterEnterDao.checkQuarterSort(sort, update, quarterId);
        if (count > 0) {
            return Result.ofFail("排序有重复数据,请重新填写");
        }
        return null;
    }*/

/*    public Result<Object> checkSemesterSort(int sort, boolean update, String semesterId, String quarterId) {
        int count = quarterEnterDao.checkSemesterSort(sort, update, semesterId, quarterId);
        if (count > 0) {
            return Result.ofFail("排序有重复数据,请重新填写");
        }
        return null;
    }*/

    public Result<Object> checkThemeNameLength(String themeName) {
        if (StringUtils.isNotBlank(themeName)) {
            if (themeName.length() > RedscarfConstants.LENGTH_FIFTY) {
                return Result.ofFail("主题名称不能超过50个字,请重新填写");
            }
        }
        return null;
    }

    public Result<Object> checkRepeatQuarter(String quarter, boolean update, String quarterId) {
        if (StringUtils.isNotBlank(quarter)) {
            // 2023-12-28 20:34 特辑可以多条数据，因此不需要判断特辑
            // 当传值为特辑时，不需要判断是否有重复数据
            if (!RedscarfConstants.SPECIAL_EDITION.equals(quarter)) {
                int total = quarterEnterDao.checkRepeatQuarter(quarter, update, quarterId);
                if (total > 0) {
                    return Result.ofFail("学季有重复数据,请重新填写");
                }
            }
        }
        return null;
    }

    public Result<Object> checkRepeatSemester(String semester, boolean update, String semesterId, String quarterId) {
        if (StringUtils.isNotBlank(semester)) {
            // 2023-12-28 20:34 特辑可以多条数据，因此不需要判断特辑
            // 当传值为特辑时，不需要判断是否有重复数据
            if (!RedscarfConstants.SPECIAL_EDITION.equals(semester)) {
                int total = quarterEnterDao.checkRepeatSemester(semester, update, semesterId, quarterId);
                if (total > 0) {
                    return Result.ofFail("学期有重复数据,请重新填写");
                }
            }
        }
        return null;
    }
}
