package com.crunii.micro.service.redscarf.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqRedscarfStu;
import com.crunii.micro.domain.entity.main.YcqRedscarfStuRecord;
import com.crunii.micro.domain.repo.main.YcqRedscarfStuRecordRepo;
import com.crunii.micro.domain.repo.main.YcqRedscarfStuRepo;
import com.crunii.micro.service.redscarf.jdbc.UserSideDao;
import com.crunii.micro.service.redscarf.service.UserSideService;
import com.crunii.mircro.service.redscarf.constants.RedscarfConstants;
import com.crunii.mircro.service.redscarf.domain.dto.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author: LG
 * @date: 2023/9/5 14:41
 */
@Service
@Slf4j
public class UserSideServiceImpl  implements UserSideService {

    @Resource
    private UserSideDao userSideDao;

    @Resource
    private YcqRedscarfStuRepo ycqRedscarfStuRepo;

    @Resource
    private YcqRedscarfStuRecordRepo ycqRedscarfStuRecordRepo;

    @Value("${skipPageLink}")
    private String skipPageLink;

    @Override
    public Result<List<QueryLearnSeasonVO>> queryLearnSeason() {
        List<QueryLearnSeasonVO> queryLearnSeasonVOs = userSideDao.queryLearnSeason();
        queryLearnSeasonVOs.forEach(s -> s.setQuarter(RedscarfConstants.checkQuarter(s.getQuarter())));
        return Result.ofSuccess(queryLearnSeasonVOs);
    }

    @Override
    public Result<Object> updateUserStudent(Worker worker, UserAddStudentDTO userAddStudentDTO) {
        userSideDao.updateUserStudent(worker,userAddStudentDTO);
        return Result.ofSuccess("成功");
    }

    @Override
    public Result<Object> saveStudentInformation(Worker worker, SaveStudentInformationDTO saveStudentInformationDTO) {
        YcqRedscarfStu ycqRedscarfStu = new YcqRedscarfStu();
        ycqRedscarfStu.setLoginId(worker.getLoginId());
        String areaName = saveStudentInformationDTO.getAreaName();
        ycqRedscarfStu.setAreaName(StringUtils.hasText(areaName)?areaName:"潼南区");

        String schoolName = saveStudentInformationDTO.getSchoolName();
        ycqRedscarfStu.setSchoolName(StringUtils.hasText(schoolName)?schoolName:"重庆市潼南区塘坝镇胜利小学校");

        Integer classGrade = saveStudentInformationDTO.getClassGrade();
        ycqRedscarfStu.setClassGrade(Objects.nonNull(classGrade)?classGrade:2022);

        String detachment = saveStudentInformationDTO.getDetachment();
        ycqRedscarfStu.setDetachment(StringUtils.hasText(detachment)?detachment:"一");
        ycqRedscarfStu.setStuName(saveStudentInformationDTO.getStuName());
        ycqRedscarfStu.setStuIdCard(saveStudentInformationDTO.getStuIdCard());

        ycqRedscarfStu.setPickStu(CommonConstants.STATE_INVALID);
        ycqRedscarfStu.setCreatorName(worker.getWorkerName());
        ycqRedscarfStu.setUpdaterName(worker.getWorkerName());
        ycqRedscarfStu.setDeleted(CommonConstants.STATE_INVALID);
        ycqRedscarfStuRepo.save(ycqRedscarfStu);

        return Result.ofSuccess("录入成功");
    }

    @Override
    public Result<Object> deleteStudent(Worker worker, UserAddStudentDTO userAddStudentDTO) {/*
        String env = System.getProperty("spring.profiles.active");
        if("local".equals(env)){
            Worker worker1 = new Worker();
            worker1.setLoginId(9999);
            worker1.setWorkerName("ss");
            worker=worker1;
        }
*/

        userSideDao.deleteUserStudent(worker,userAddStudentDTO);
        return Result.ofSuccess("删除成功");
    }

    @Override
    public Result<Object> updateLearningSeason(Worker worker, UpdateLearningSeasonDTO updateLearningSeasonDTO) {
        int checkRepeatStuRecord = userSideDao.checkRepeatStuRecord(updateLearningSeasonDTO.getRedscarfId(), updateLearningSeasonDTO.getQuarterId(),
                updateLearningSeasonDTO.getSemesterId());
        // 如果已有记录，则不录入数据.因此只有在记录为0时，菜录入数据
        if (checkRepeatStuRecord == 0) {
            YcqRedscarfStuRecord ycqRedscarfStuRecord = new YcqRedscarfStuRecord();
            ycqRedscarfStuRecord.setQuarterId(Long.valueOf(updateLearningSeasonDTO.getQuarterId()));
            ycqRedscarfStuRecord.setRedscarfId(Long.valueOf(updateLearningSeasonDTO.getRedscarfId()));
            ycqRedscarfStuRecord.setSemesterId(Long.valueOf(updateLearningSeasonDTO.getSemesterId()));
            ycqRedscarfStuRecord.setCreatorName(worker.getWorkerName());
            ycqRedscarfStuRecord.setUpdaterName(worker.getWorkerName());
            ycqRedscarfStuRecord.setCreateTime(new Date());
            ycqRedscarfStuRecord.setDeleted(CommonConstants.STATE_INVALID);
            ycqRedscarfStuRecordRepo.save(ycqRedscarfStuRecord);
        }
        return Result.ofSuccess("标记完成");
    }

    @Override
    public Result<List<SemesterDTO>> queryStudentLearnSemester(QueryStudentLearnSemesterDTO queryStudentLearnSemesterDTO) {
        List<SemesterDTO> semesterDTOS = userSideDao.queryStudentLearnSemester(queryStudentLearnSemesterDTO);
        semesterDTOS.forEach(s -> s.setSemester(RedscarfConstants.checkSemester(s.getSemester())));
        return Result.ofSuccess(semesterDTOS);
    }

    @Override
    public Result<List<QueryStudentInformationVO>> queryUserStudent(Worker worker) {
        return Result.ofSuccess(userSideDao.queryUserStudent(worker));
    }

    @Override
    public Result<WhetherSchoolVO> whetherSchoolInformation(Worker worker) {
        WhetherSchoolVO whetherSchoolVO = userSideDao.whetherSchoolInformation(worker);
        if (ObjectUtil.isNotEmpty(whetherSchoolVO)){
            whetherSchoolVO.setWhetherFillOut(true);
            return Result.ofSuccess(whetherSchoolVO);
        }
        WhetherSchoolVO empty = new WhetherSchoolVO();
        empty.setWhetherFillOut(false);
        return Result.ofSuccess(empty);
    }

    @Override
    public Result<QueryStudentInformationVO> queryStudentInformation(QueryStudentInformationDTO queryStudentInformationDTO) {
        QueryStudentInformationVO queryStudentInformationVO = new QueryStudentInformationVO();
        queryStudentInformationVO.setAreaName("潼南区");
        queryStudentInformationVO.setClassGrade(2022);
        queryStudentInformationVO.setDetachment("一");
        queryStudentInformationVO.setSchoolName("重庆市潼南区塘坝镇胜利小学校");
        queryStudentInformationVO.setStuName(queryStudentInformationDTO.getStuName());
        queryStudentInformationVO.setStuIdCard(queryStudentInformationDTO.getStuIdCard());
        return Result.ofSuccess(queryStudentInformationVO);
    }

    @Override
    public Result<String> skipPage(Worker worker) {
        return Result.ofSuccess(skipPageLink);
    }


}
