package com.crunii.micro.service.report;

import com.alibaba.druid.spring.boot.autoconfigure.DruidDataSourceAutoConfigure;
import com.crunii.micro.common.utils.JasyptEncryptorUtil;
import com.crunii.micro.config.ClientVersionRibbonConfiguration;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.data.redis.RedisAutoConfiguration;
import org.springframework.boot.autoconfigure.data.redis.RedisReactiveAutoConfiguration;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.autoconfigure.transaction.TransactionAutoConfiguration;
import org.springframework.boot.web.embedded.undertow.UndertowServletWebServerFactory;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cloud.loadbalancer.annotation.LoadBalancerClients;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.EnableMBeanExport;
import org.springframework.context.annotation.ImportResource;
import org.springframework.jmx.support.RegistrationPolicy;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.util.StringUtils;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/7/5 10:16
 */
@EnableFeignClients(basePackages = {"com.crunii.micro.feign"})
@ImportResource({ "classpath:spring/*.xml" })
@EnableCaching
@EnableScheduling
@EnableMBeanExport(registration = RegistrationPolicy.IGNORE_EXISTING)
@SpringBootApplication(
        exclude = {
                RedisAutoConfiguration.class,
                RedisReactiveAutoConfiguration.class,
                TransactionAutoConfiguration.class,
                DataSourceAutoConfiguration.class,
                DruidDataSourceAutoConfigure.class })
@LoadBalancerClients(defaultConfiguration = ClientVersionRibbonConfiguration.class)
public class ReportApplication {

    public static void main(String[] args) {
        JasyptEncryptorUtil.setConfigClientProps();
        JasyptEncryptorUtil.setAdminClientProps();
        // 接netty重复设置的问题
        System.setProperty("es.set.netty.runtime.available.processors", "false");
        String env = System.getProperty("spring.profiles.active");
        if (!StringUtils.hasText(env)) {
            // local or dev or prod
            env = "local";
            System.setProperty("spring.profiles.active", env);
        }
        SpringApplication.run(ReportApplication.class, args);
    }

    @Bean
    public UndertowServletWebServerFactory createUndertowServletWebServerFactory() {
        return new UndertowServletWebServerFactory();
    }

}
