package com.crunii.micro.service.report.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.report.dto.cockpit.*;
import com.crunii.micro.service.report.service.CockpitDataService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

@Api(tags = "驾驶舱数据接口")
@RestController
@RequestMapping("/auth/CockpitDataController")
public class CockpitDataController {

    @Resource
    private CockpitDataService service;

    @ApiOperation("获取属性")
    @PostMapping("/getAttribute")
    public Result<List<CockpitDataGetAttributeResp>> getAttribute(@RequestBody CockpitDataGetAttributeReq req){
        return service.getAttribute(req);
    }

    @ApiOperation("查询")
    @PostMapping("/query")
    public Result<Map<String, Object>> query(@RequestBody CockpitDataQueryReq req){
        return service.query(req);
    }

    @ApiOperation("分页查询")
    @PostMapping("/pageQuery")
    public Result<PageRsp<Map<String, Object>>> pageQuery(@RequestBody PageReq<CockpitDataQueryReq> req){
        return service.pageQuery(req);
    }

    @ApiOperation("新增")
    @PostMapping("/insert")
    public Result<Boolean> insert(@RequestBody CockpitDataInsertReq req){
        return service.insert(req);
    }

    @ApiOperation("修改")
    @PostMapping("/update")
    public Result<Boolean> update(@RequestBody CockpitDataUpdateReq req){
        return service.update(req);
    }

    @ApiOperation("删除")
    @PostMapping("/delete")
    public Result<Boolean> delete(@RequestBody CockpitDataDeleteReq req){
        return service.delete(req);
    }

    @ApiOperation("批量删除")
    @PostMapping("/batchDelete")
    public Result<Boolean> batchDelete(@RequestBody CockpitDataBatchDeleteReq req){
        return service.batchDelete(req);
    }

    @ApiOperation("获取所有表名")
    @GetMapping("/getNames")
    public Result<List<CockpitDataGetNamesResp>> getNames(){
        return service.getNames();
    }

    @ApiOperation("获取属性值")
    @PostMapping("/getAttributeValue")
    public Result<List<CockpitDataGetAttributeValueResp>> getAttributeValue(@RequestBody CockpitDataGetAttributeValueReq req){
        return service.getAttributeValue(req);
    }

    @ApiOperation("下载模板")
    @PostMapping("/download")
    public Result<Boolean> download(@RequestBody CockpitDataDownloadReq req){
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletResponse response = attributes.getResponse();
        return service.download(req, response);
    }

    @ApiOperation("导入Excel")
    @PostMapping("/import")
    public Result<Object> importExcel(MultipartFile fileName, String tableName){
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletResponse response = attributes.getResponse();
        return service.importExcel(fileName, response, tableName);
    }

    @ApiOperation(value = "获取可筛选属性值")
    @RequestMapping(path = "/getFilterMapValue", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<BasicDataColumnValueResp>> getFilterMapValue(@RequestBody @ApiParam BasicDataColumnValueReq req) {
        List<BasicDataColumnValueResp> resp = service.getFilterMapValue(req);
        Result<List<BasicDataColumnValueResp>> result = new Result();
        result.setData(resp);
        return result;
    }

    @ApiOperation("获取字段")
    @PostMapping("/getField")
    public Result<List<CockpitDataGetFieldResp>> getField(@RequestBody CockpitDataGetAttributeReq req){
        return service.getField(req);
    }
}
