package com.crunii.micro.service.report.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.report.dto.*;

import com.crunii.micro.service.report.service.ModelManagerService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

@Api(value = "模型管理相关")
@Controller
@Slf4j
@RequestMapping("/auth/model/ModelManagerController")
public class ModelManagerController {

    @Autowired
    private ModelManagerService modelManagerService;

    @ApiOperation(value = "模型分页")
    @RequestMapping(path = "/pageModel", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<ModelRes>> pageModel(@RequestBody @ApiParam PageReq<ModelReq> req) {
        Result<PageRsp<ModelRes>> result = new Result<>();
        result.setData(modelManagerService.pageModel(req));
        return result;
    }

    @ApiOperation(value = "创建模型")
    @RequestMapping(path = "/createModel", method = RequestMethod.POST)
    @ResponseBody
    public Result<String> createModel(@RequestBody @ApiParam ModelCVo modelCVo) {
        modelManagerService.createModel(modelCVo);
        return new Result<>();
    }

    @ApiOperation(value = "更新模型")
    @RequestMapping(path = "/updateModel", method = RequestMethod.POST)
    @ResponseBody
    public Result<String> updateModel(@RequestBody @ApiParam ModelUVo modelUVo) {
        modelManagerService.updateModel(modelUVo);
        return new Result<>();
    }

    @ApiOperation(value = "复制模型")
    @RequestMapping(path = "/copyModel", method = RequestMethod.POST)
    @ResponseBody
    public Result<String> copyModel(@RequestBody @ApiParam ModelPVo modelPVo) {
        modelManagerService.doCopyModel(modelPVo);
        return new Result<>();
    }

    @ApiOperation(value = "获取模型")
    @RequestMapping(path = "/getModel", method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "modelId", value = "模型ID", dataType = "int", required = true)})
    @ResponseBody
    public Result<ModelResOneVo> getModel(Long modelId) {
        ModelResOneVo one = modelManagerService.getModel(modelId);
        return new Result<>(one);
    }

    @ApiOperation(value = "发布模型")
    @RequestMapping(path = "/publishModel", method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "modelId", value = "模型ID", dataType = "int", required = true)})
    @ResponseBody
    public Result<String> publishModel(Long modelId, @RequestParam(required = false) String parentMenuCode,
                                       @CurrWorker  @ApiIgnore Worker worker) {
        Result<String> result = new Result<>();
        result.setData(modelManagerService.doPublishModel(modelId, parentMenuCode));
        return result;
    }

    @ApiOperation(value = "保存列")
    @RequestMapping(path = "/saveColumn", method = RequestMethod.POST)
    @ResponseBody
    public Result<String> saveColumn(@RequestBody @ApiParam ColumnCVo columnCVo) {
        Result<String> result = new Result<>();
        result.setData(modelManagerService.saveColumn(columnCVo));
        return result;
    }

    @ApiOperation(value = "模型分页")
    @RequestMapping(path = "/getModelListFixed", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<ModelResFixed>> getModelListFixed(@RequestBody @ApiParam GetModeListFixedReq req) {
        List<ModelResFixed> list = modelManagerService.getModelListFixed(req);
        return new Result(list);
    }

    @ApiOperation(value = "获取模型")
    @RequestMapping(path = "/getModelFixed", method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "name", value = "模型ID", required = true)})
    @ResponseBody
    public Result<List<ModelResOneFixedVo>> getModelFixed(String name) {
        return modelManagerService.doGetModelChilds(name);
    }

}
