package com.crunii.micro.service.report.jdbc;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.StrPool;
import com.alibaba.druid.DbType;
import com.alibaba.druid.sql.SQLUtils;
import com.crunii.micro.common.exception.MicroException;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.report.dto.cockpit.map.grid.CockpitGridMapDataReqVO;
import com.crunii.micro.service.report.dto.cockpit.map.grid.CockpitGridMapDataRespVO;
import com.crunii.micro.service.report.dto.cockpit.map.grid.CockpitGridMapYouthNumRespVO;
import com.crunii.micro.service.report.dto.cockpit.map.position.CockpitPositionAreaSumRespVO;
import com.crunii.micro.service.report.dto.cockpit.map.position.CockpitPositionDataReqVO;
import com.crunii.micro.service.report.dto.cockpit.map.position.CockpitPositionDataRespVO;
import com.crunii.micro.service.report.service.CockpitMapService;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.crunii.micro.service.report.constants.CockpitMapErrorCodeConstants.GRID_MAP_GRANULARITY_NOT_FOUND;

@Repository
public class CockpitMapDAO {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    private static final Map<Integer, String> QUERY_TABLE_MAP;

    private static final Map<String, List<String>> QUERY_TABLE_COLUMN;

    static {
        HashMap<Integer, String> queryTableMap = new HashMap<>();
        HashMap<String, List<String>> queryTableColumn = new HashMap<>();

        // 1171栅格
        queryTableMap.put(1171, "map_grid_youth_cq_all_wgs84");
        queryTableColumn.put("map_grid_youth_cq_all_wgs84", CollUtil.newArrayList(
                "id", "area_name", "youth_num", "center_lng", "center_lat",
                "left_up_lng", "left_up_lat", "right_down_lng", "right_down_lat"));

        // 20栅格
        queryTableMap.put(20, "map_grid_youth_cq_g20_wgs84");
        queryTableColumn.put("map_grid_youth_cq_g20_wgs84", CollUtil.newArrayList(
                "id", "area_name", "youth_num", "center_lng", "center_lat",
                "left_up_lng", "left_up_lat", "right_down_lng", "right_down_lat"));

        // 100栅格
        queryTableMap.put(100, "map_grid_youth_cq_g100_wgs84");
        queryTableColumn.put("map_grid_youth_cq_g100_wgs84", CollUtil.newArrayList(
                "id", "area_name", "youth_num", "center_lng", "center_lat",
                "left_up_lng", "left_up_lat", "right_down_lng", "right_down_lat"));

        // 200栅格
        queryTableMap.put(200, "map_grid_youth_cq_g200_wgs84");
        queryTableColumn.put("map_grid_youth_cq_g200_wgs84", CollUtil.newArrayList(
                "id", "area_name", "youth_num", "center_lng", "center_lat",
                "left_up_lng", "left_up_lat", "right_down_lng", "right_down_lat"));

        // 500栅格
        queryTableMap.put(500, "map_grid_youth_cq_g500_wgs84");
        queryTableColumn.put("map_grid_youth_cq_g500_wgs84", CollUtil.newArrayList(
                "id", "area_name", "youth_num", "center_lng", "center_lat",
                "left_up_lng", "left_up_lat", "right_down_lng", "right_down_lat"));

        QUERY_TABLE_MAP = Collections.unmodifiableMap(queryTableMap);
        QUERY_TABLE_COLUMN = Collections.unmodifiableMap(queryTableColumn);
    }

    public List<CockpitGridMapDataRespVO> listCockpitGridMapData(CockpitGridMapDataReqVO reqVO) {
        String tableName = QUERY_TABLE_MAP.get(reqVO.getMapGranularity());
        if (tableName == null) {
            throw new MicroException(GRID_MAP_GRANULARITY_NOT_FOUND);
        }
        List<String> tableColumn = QUERY_TABLE_COLUMN.get(tableName);
        if (tableColumn == null) {
            return Collections.emptyList();
        }
        tableColumn = tableColumn.stream().map(o -> "a." + o).collect(Collectors.toList());
        String sql = " SELECT " + String.join(StrPool.COMMA, tableColumn) +
                " FROM " + tableName + " a " +
                " WHERE a.center_lng >= ? AND a.center_lat >= ? " +
                " AND a.center_lng <= ? AND a.center_lat <= ? ";
        List<Object> params = CollUtil.newArrayList(reqVO.getLeftDownLng(), reqVO.getLeftDownLat(), reqVO.getRightUpLng(), reqVO.getRightUpLat());
        sql = SQLUtils.format(sql, DbType.dm, params);
        return jdbcTemplate.queryForBeans(CockpitGridMapDataRespVO.class, sql);
    }

    public CockpitGridMapYouthNumRespVO getCockpitGridMapYouthNum(CockpitGridMapDataReqVO reqVO) {
        String tableName = QUERY_TABLE_MAP.get(reqVO.getMapGranularity());
        if (tableName == null) {
            throw new MicroException(GRID_MAP_GRANULARITY_NOT_FOUND);
        }
        String baseSql = " SELECT MIN(a.youth_num) AS min_youth_num, MAX(a.youth_num) AS max_youth_num " +
                " FROM " + tableName + " a " +
                " WHERE area_name IN ";

        List<Object> mainAreaNames = new ArrayList<>(CockpitMapService.MAIN_AREA_NAME_SET);
        String[] mainAreaNameArr = new String[mainAreaNames.size()];
        Arrays.fill(mainAreaNameArr, "?");
        String mainSql = baseSql + "(" + String.join(", ", mainAreaNameArr) + ")";
        mainSql = SQLUtils.format(mainSql, DbType.dm, mainAreaNames);
        Map<String, Object> mainResultMap = jdbcTemplate.queryForMap(mainSql);

        List<Object> otherAreaNames = new ArrayList<>(CockpitMapService.OTHER_AREA_NAME_SET);
        String[] otherAreaNameArr = new String[otherAreaNames.size()];
        Arrays.fill(otherAreaNameArr, "?");
        String otherSql = baseSql + "(" + String.join(", ", otherAreaNameArr) + ")";
        otherSql = SQLUtils.format(otherSql, DbType.dm, otherAreaNames);
        Map<String, Object> otherResultMap = jdbcTemplate.queryForMap(otherSql);

        CockpitGridMapYouthNumRespVO result = new CockpitGridMapYouthNumRespVO();
        result.setMapGranularity(reqVO.getMapGranularity());
        result.setMainMinNum((Integer) mainResultMap.get("min_youth_num"));
        result.setMainMaxNum((Integer) mainResultMap.get("max_youth_num"));
        result.setOtherMinNum((Integer) otherResultMap.get("min_youth_num"));
        result.setOtherMaxNum((Integer) otherResultMap.get("max_youth_num"));
        return result;
    }

    public List<CockpitPositionAreaSumRespVO> listCockpitPositionDataAreaSum(CockpitPositionDataReqVO reqVO) {
        if (reqVO.getTypes().length == 0) {
            return Collections.emptyList();
        }
        List<Object> params = CollUtil.newArrayList(reqVO.getTypes());
        String[] otherAreaNameArr = new String[reqVO.getTypes().length];
        Arrays.fill(otherAreaNameArr, "?");
        String sql = " SELECT MIN(a.id) AS id, a.area_name, COUNT(1) AS position_num, " +
                " b.flag_lng AS area_lng, b.flag_lat AS area_lat, " +
                " b.center_position_lng, b.center_position_lat " +
                " FROM map_position_youth_cq a " +
                " LEFT JOIN map_area_flag b ON a.area_name = b.flag_name " +
                " WHERE a.position_type IN (" + String.join(", ", otherAreaNameArr) + " ) " +
                " GROUP BY a.area_name, b.flag_lng, b.flag_lat, b.center_position_lng, b.center_position_lat ";
        sql = SQLUtils.format(sql, DbType.dm, params);
        return jdbcTemplate.queryForBeans(CockpitPositionAreaSumRespVO.class, sql);
    }

    public List<CockpitPositionDataRespVO> listCockpitPositionMapData(CockpitPositionDataReqVO reqVO) {
        if (reqVO.getTypes().length == 0) {
            return Collections.emptyList();
        }
        List<Object> params = CollUtil.newArrayList(reqVO.getTypes());
        String[] otherAreaNameArr = new String[reqVO.getTypes().length];
        Arrays.fill(otherAreaNameArr, "?");
        String sql = " SELECT a.id, a.position_code, a.position_name, a.position_type, " +
                " a.area_name, a.position_address, a.position_lng, a.position_lat " +
                " FROM map_position_youth_cq a " +
                " WHERE a.position_type IN (" + String.join(", ", otherAreaNameArr) + " ) " +
                " AND a.position_lng >= ? AND a.position_lat >= ? " +
                " AND a.position_lng <= ? AND a.position_lat <= ? ";
        params.addAll(CollUtil.newArrayList(reqVO.getLeftDownLng(), reqVO.getLeftDownLat(), reqVO.getRightUpLng(), reqVO.getRightUpLat()));
        sql = SQLUtils.format(sql, DbType.dm, params);
        return jdbcTemplate.queryForBeans(CockpitPositionDataRespVO.class, sql);
    }

    /**
     * 阵地地图数据同步
     * 使用map_position_youth_cq表中的数据更新map_area_flag表
     *
     * @return 更新行数
     */
    public int positionToAreaFlag() {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        int workerId = currWorker != null ? currWorker.getLoginId() : 10130;
        String workerName = currWorker != null ? currWorker.getWorkerName() : "胡人天";
        String sql = " UPDATE map_area_flag a  " +
                " SET FLAG_DESCRIPTION = b.valuea, UPDATE_WORKER_ID = " + workerId + ", UPDATE_WORKER_NAME = '" + workerName + "', UPDATE_TIME = NOW() " +
                " FROM ( " +
                "  SELECT area_name, '{\"num\":'||SUM(position_num)||',\"names\": \"'||REPLACE(WM_CONCAT(position_type||position_num||'个'), ',', '\\n')||'\"}' AS valuea " +
                "  FROM ( " +
                "   SELECT area_name, position_type, count(*) AS position_num " +
                "   FROM map_position_youth_cq  " +
                "   GROUP BY area_name, position_type  " +
                "   ORDER BY area_name, position_type " +
                "  ) t " +
                "  GROUP BY area_name " +
                "  UNION ALL  " +
                "  SELECT '重庆主城' AS area_name, '{\"num\":'||SUM(position_num)||',\"names\": \"'||REPLACE(WM_CONCAT(position_type||position_num||'个'), ',', '\\n')||'\"}' AS valuea " +
                "  FROM ( " +
                "   SELECT position_type, count(*) AS position_num " +
                "   FROM map_position_youth_cq  " +
                "   WHERE area_name IN ('巴南', '渝中', '南岸', '两江', '沙坪坝', '北碚', '大渡口', '渝北', '九龙坡', '江北') " +
                "   GROUP BY position_type " +
                "   ORDER BY position_type " +
                "  ) t " +
                " ) b " +
                " WHERE a.flag_name= b.area_name ";
        return jdbcTemplate.update(sql);
    }

}
