package com.crunii.micro.service.report.jdbc;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.entity.main.TsmMenu;
import com.crunii.micro.domain.entity.main.YcqCockpitModel;
import com.crunii.micro.domain.entity.main.YcqCockpitModelColum;
import com.crunii.micro.domain.repo.main.TsmMenuRepo;
import com.crunii.micro.domain.repo.main.YcqCockpitModelColumRepo;
import com.crunii.micro.domain.repo.main.YcqCockpitModelRepo;
import com.crunii.micro.service.report.dto.*;
import com.crunii.micro.service.report.dto.cockpit.CockpitDataGetFieldResp;
import com.crunii.micro.service.report.enums.DatabaseTypeEnum;
import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.*;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

/**
 * @author wangchen create 2023/7/14 10:14
 */
@Repository
public class ModelManagerDao {


    private static final String MODEL_CODE_PRE = "FLO";
    private static final String getNextMenuCodeSql = "select menu_code from tsm_menu where parent_menu_code=? ";

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;
    @Autowired
    private YcqCockpitModelRepo floModelRepo;
    @Autowired
    private YcqCockpitModelColumRepo floColumnRepo;
    @Autowired
    private TsmMenuRepo tsmMenuRepo;


    public PageRsp<ModelRes> pageModel(PageReq<ModelReq> req) {
        StringBuilder sb = new StringBuilder(
                "select a.* from YCQ_COCKPIT_MODEL a where status = 1 ");
        StringBuilder sbCount = new StringBuilder("select count(1) from YCQ_COCKPIT_MODEL a where status = 1 ");
        List<Object> params = new ArrayList<>();

        if (req.getQuery().getType() != null) {
            sb.append(" and a.type = ? ");
            sbCount.append(" and a.type = ? ");
            params.add(req.getQuery().getType());
        }

        if (StringUtils.hasText(req.getQuery().getName())) {
            sb.append(" and a.name like concat('%',?,'%') ");
            sbCount.append(" and a.name like concat('%',?,'%') ");
            params.add(req.getQuery().getName());
        }
        if (StringUtils.hasText(req.getQuery().getRemark())) {
            sb.append(" and a.remark like concat('%',?,'%') ");
            sbCount.append(" and a.remark like concat('%',?,'%') ");
            params.add(req.getQuery().getRemark());
        }
        int total = this.JT.queryForInt(sbCount.toString(), params.toArray());
        List<ModelRes> roles = null;
        if (total > 0) {
            sb.append(" order by a.create_time desc limit ?,? ");
            params.add(req.getMyStart());
            params.add(req.getPageSize());
            roles = this.JT.queryForBeans(ModelRes.class, sb.toString(), params.toArray());
        } else {
            roles = new ArrayList<>(0);
        }
        PageRsp<ModelRes> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(roles);
        // 为手机计算是否由下一页
        page.next();
        return page;
    }


    public void createModel(ModelCVo modelCVo) {
        boolean isOk = false;
        int i = this.JT.queryForInt("select count(1) from YCQ_COCKPIT_MODEL where name = ? and status = 1 ",
                new Object[]{modelCVo.getName()});
        if (i > 0) {
            isOk = true;
        }
        if (isOk) {
            throw new BusinessException("模型已经存在");
        }
        YcqCockpitModel model = new YcqCockpitModel();
        BeanUtils.copyProperties(modelCVo, model);
        model.setCode(MODEL_CODE_PRE + new Date().getTime());
        model.setState(1);
        model.setStatus(1);
        model.setCreateTime(new Date());
        this.floModelRepo.save(model);
    }


    public void updateModel(ModelUVo modelUVo) {
        YcqCockpitModel model = this.floModelRepo.getOne(modelUVo.getId());
        if (model.getState() == 2) {
            throw new BusinessException("模型已经发布，无法修改系统");
        }
        if (!model.getName().equals(modelUVo.getName())) {
            if (model.getState() == 2) {
                throw new BusinessException("模型已经发布，无法修改名称");
            }
            boolean isOk = false;
            int i = this.JT.queryForInt("select count(1) from YCQ_COCKPIT_MODEL where name = ? and status = 1 and id != ? ",
                    new Object[]{modelUVo.getName(), modelUVo.getId()});
            if (i > 0) {
                isOk = true;
            }
            if (isOk) {
                throw new BusinessException("模型已经存在");
            }
        }
        BeanUtils.copyProperties(modelUVo, model, "id", "status");
        this.floModelRepo.save(model);
        this.floModelRepo.flush();
    }


    public void doCopyModel(ModelPVo modelPVo) {


        YcqCockpitModel newModel = new YcqCockpitModel();
        BeanUtils.copyProperties(modelPVo, newModel, "id");
        newModel.setState(1);
        newModel.setStatus(1);
        newModel.setCreateTime(new Date());
        this.floModelRepo.save(newModel);

        Map<String, Object> map = new HashMap<>();
        map.put("status", 1);
        map.put("modelId", modelPVo.getId());
        List<YcqCockpitModelColum> columnList = this.floColumnRepo.findByProps(map);
    }

    private void addColumn(Long modelId, List<YcqCockpitModelColum> columnList, String loginId) {
        this.JT.batchUpdate(
                "insert into YCQ_COCKPIT_MODEL_COLUMN(MODEL_ID,NAME,TYPE,INT_SIZE,DEC_SIZE,IS_DESENSITIZATION,DEFAULT_VAL,REMARK,CREATE_TIME,CREATE_USER,STATUS, IS_NEED,IS_SHOW_COLUMN) values (?,?,?,?,?,?,?,?,now(),?,?,?,?)",
                new BatchPreparedStatementSetter() {

                    public void setValues(PreparedStatement ps, int i) throws SQLException {
                        YcqCockpitModelColum column = columnList.get(i);
                        ps.setLong(1, modelId);
                        ps.setString(2, column.getName().toUpperCase());
                        ps.setString(3, column.getType());
                        if (column.getIntSize() != null) {
                            ps.setInt(4, column.getIntSize());
                        } else {
                            ps.setNull(4, Types.NULL);
                        }
                        if (column.getDecSize() != null) {
                            ps.setInt(5, column.getDecSize());
                        } else {
                            ps.setNull(5, Types.NULL);
                        }
                        ps.setBoolean(6, column.isDesensitization());
                        if (StringUtils.hasText(column.getDefaultVal())) {
                            ps.setString(7, column.getDefaultVal());
                        } else {
                            ps.setNull(7, Types.NULL);
                        }
                        if (StringUtils.hasText(column.getRemark())) {
                            ps.setString(8, column.getRemark());
                        } else {
                            ps.setNull(8, Types.NULL);
                        }
                        ps.setString(9, loginId);
                        ps.setInt(10, 1);
                        ps.setBoolean(11, column.isNeed());
                        ps.setInt(12, column.getIsShowColumn() == null ? 1 : column.getIsShowColumn());
                    }


                    public int getBatchSize() {
                        return columnList.size();
                    }
                });
    }

    @Transactional
    public String doPublishModel(Long modelId, String parentMenuCode) {
        YcqCockpitModel model = this.floModelRepo.getOne(modelId);
        if (model == null) {
            throw new BusinessException("未找到modelId为" + modelId + "的模型");
        }
        Date curr = new Date();
        TsmMenu menu = new TsmMenu();
        menu.setCreateTime(curr);
        menu.setState(1);
        menu.setMenuType(1);
        menu.setParentMenuCode(parentMenuCode);
        menu.setMenuCode(getNextMenuCode(parentMenuCode));
        menu.setMenuName(model.getRemark());
        menu.setMenuUrl("/basaltable/basicdata/" + model.getName());
        List<TsmMenu> list = tsmMenuRepo.findByMenuUrl("/basaltable/basicdata/" + model.getName());
        if (list != null && list.size() > 0) {
            for (TsmMenu tsmMenu : list) {
                tsmMenuRepo.delete(tsmMenu);
            }
        }
        tsmMenuRepo.save(menu);
        tsmMenuRepo.flush();
        model.setState(2);
        floModelRepo.saveAndFlush(model);
        return null;
    }

    public static void main(String[] args) {

        System.out.println(UUID.randomUUID().toString().replace("-", ""));
    }

    public ModelResOneVo getModel(Long modelId) {
        YcqCockpitModel model = this.floModelRepo.getOne(modelId);
        ModelResOneVo one = new ModelResOneVo();
        one.setId(modelId);
        one.setName(model.getName());
        one.setRemark(model.getRemark());
        one.setState(model.getState());
        List<Object> params = new ArrayList<>();
        String sql = "SELECT fc.* FROM YCQ_COCKPIT_MODEL_COLUMN fc LEFT JOIN YCQ_COCKPIT_MODEL fm ON fc.MODEL_ID = fm.ID WHERE  fc.STATUS = 1 AND fm.STATUS = 1 and fc.name!='ID' and fc.name!='CREATE_TIME' and fc.name!='CREATE_WORKER_ID' and fc.name!='UPDATE_TIME' and fc.name!='UPDATE_WORKER_ID' and fc.name!='UPDATE_WORKER_NAME' and fc.name!='CREATE_WORKER_NAME' and fm.id =  ";
        StringBuilder sb = new StringBuilder(sql).append(modelId);
        List<ColumnVo> list = JT.queryForBeans(ColumnVo.class, sb.toString(), params.toArray());
        one.setList(list);
        return one;
    }

    private String addColumn(YcqCockpitModelColum column) {
        StringBuffer sql = new StringBuffer();
        if ("ID".equals(column.getName().toUpperCase()) || "FLO_WORK_ID".equals(column.getName().toUpperCase())
                || "FLO_FORM_ID".equals(column.getName().toUpperCase())
                || "SYSTEM_ID".equals(column.getName().toUpperCase())) {
            throw new BusinessException("ID、FLO_WORK_ID、FLO_FORM_ID,SYSTEM_ID为固定字段");
        }
        sql.append("`").append(column.getName()).append("` ");
        DatabaseTypeEnum.MYSQL mysqlType = DatabaseTypeEnum.getMysqlType(column.getType());
        sql.append(mysqlType.value).append(" ");
        if (mysqlType.isIntSize) {
            sql.append("(").append(column.getIntSize() == null ? 0 : column.getIntSize());
            if (mysqlType.isDecSize) {
                sql.append(",").append(column.getDecSize() == null ? 0 : column.getDecSize()).append(") ");
            } else {
                sql.append(") ");
            }
        }
        if (mysqlType.isString || mysqlType == DatabaseTypeEnum.MYSQL.Date) {
            if (StringUtils.hasText(column.getDefaultVal())) {
                sql.append("DEFAULT").append(" '").append(column.getDefaultVal()).append("' ");
            }
        } else {
            if (StringUtils.hasText(column.getDefaultVal())) {
                sql.append("DEFAULT ").append(column.getDefaultVal()).append(" ");
            }
        }
        if (StringUtils.hasText(column.getRemark())) {
            sql.append("COMMENT").append(" '").append(column.getRemark()).append("' ");
        }
        return sql.toString();
    }

    @Transactional
    public String saveColumn(ColumnCVo columnCVo) {
        YcqCockpitModel model = this.floModelRepo.getOne(columnCVo.getModelId());
        List<ColumnVo> list = columnCVo.getList();
        String isExistSql = "select count(*) from sysobjects where name = ?";
        String copySql = "CREATE TABLE " + model.getName() + "_copy" + " AS  SELECT * FROM  " + model.getName();
        String copyTableCountSql = "select COUNT(1) from (select object_name table_name,created create_time from" +
                "    user_objects where object_type='TABLE' or object_type='VIEW')" +
                "WHERE  TABLE_NAME='" + model.getName() + "_copy'";
        String dropSql = "drop table " + model.getName();
        String copyDataSql = "INSERT INTO " + model.getName();
        String dropCopySql = "drop table " + model.getName() + "_copy";
        //先把零时表删除
        Integer num = this.JT.queryForInt(copyTableCountSql);
        if (num > 0) {
            this.JT.execute(dropCopySql);
        }
        String selectSql = "select * from YCQ_COCKPIT_MODEL_COLUMN where  NAME !='ID' and MODEL_ID = " + columnCVo.getModelId();
        List<CockpitDataGetFieldResp> columns = JT.queryForBeans(CockpitDataGetFieldResp.class, selectSql);
        int count = this.JT.queryForInt(isExistSql, model.getName());
        if (count > 0) {
            this.JT.execute(copySql);
            this.JT.execute(dropSql);
        }
        String createSql = "CREATE TABLE " + model.getName() + "(  id BIGINT IDENTITY(1, 1) NOT NULL, CREATE_WORKER_ID VARCHAR2(255)," +
                "UPDATE_WORKER_ID VARCHAR2(255)," +
                "CREATE_WORKER_NAME VARCHAR2(255)," +
                "UPDATE_WORKER_NAME VARCHAR2(255)," +
                "CREATE_TIME dateTime," +
                "UPDATE_TIME dateTime," +
                " NOT CLUSTER PRIMARY KEY(\"id\"), ";
        for (int i = 0; i < list.size(); i++) {
            ColumnVo columnVo = list.get(i);
            if (i == list.size() - 1) {
                String value = columnVo.getName() + " " + DatabaseTypeEnum.getMysqlType(columnVo.getType()).value + " ) ";
                createSql += value;
            } else {
                String value = columnVo.getName() + " " + DatabaseTypeEnum.getMysqlType(columnVo.getType()).value + " , ";
                createSql += value;
            }
        }
        String colunms = "";
        for (int i = 0; i < columns.size(); i++) {
            CockpitDataGetFieldResp columnVo = columns.get(i);
            if (i == columns.size() - 1) {
                colunms += columnVo.getName();
            } else {
                colunms += columnVo.getName() + ",";
            }
        }

        this.JT.execute(createSql);
        this.JT.execute("SET IDENTITY_INSERT " + model.getName() + " ON");
        if (!colunms.equals("")) {
            copyDataSql += " (" + colunms + ") ";
            copyDataSql += " SELECT " + colunms;
            copyDataSql += " FROM  " + model.getName() + "_copy";
            //迁移数据
            this.JT.execute(copyDataSql);
        }

        //删除临时表
        Integer numCopy = this.JT.queryForInt(copyTableCountSql);
        if (numCopy > 0) {
            this.JT.execute(dropCopySql);
        }

        List<YcqCockpitModelColum> addColumnList = list.stream().map(columnVo -> {
            YcqCockpitModelColum YcqCockpitModelColum = new YcqCockpitModelColum();
            BeanUtils.copyProperties(columnVo, YcqCockpitModelColum, "id");
            return YcqCockpitModelColum;
        }).collect(toList());
        this.JT.update("delete from YCQ_COCKPIT_MODEL_COLUMN where MODEL_ID = ? ", new Object[]{columnCVo.getModelId()});
        YcqCockpitModelColum id = new YcqCockpitModelColum();
        id.setName("ID");
        id.setRemark("id");
        id.setIsShowColumn(1);
        id.setType("Long");
        YcqCockpitModelColum createTime = new YcqCockpitModelColum();
        createTime.setName("CREATE_TIME");
        createTime.setRemark("创建时间");
        createTime.setIsShowColumn(1);
        createTime.setType("Date");
        YcqCockpitModelColum createId = new YcqCockpitModelColum();
        createId.setName("CREATE_WORKER_ID");
        createId.setRemark("创建人id");
        createId.setIsShowColumn(0);
        createId.setType("String");
        YcqCockpitModelColum updateTime = new YcqCockpitModelColum();
        updateTime.setName("UPDATE_TIME");
        updateTime.setRemark("更新时间");
        updateTime.setIsShowColumn(1);
        updateTime.setType("Date");
        YcqCockpitModelColum updateId = new YcqCockpitModelColum();
        updateId.setName("UPDATE_WORKER_ID");
        updateId.setRemark("修改人id");
        updateId.setIsShowColumn(0);
        updateId.setType("String");
        YcqCockpitModelColum createName = new YcqCockpitModelColum();
        createName.setName("CREATE_WORKER_NAME");
        createName.setRemark("创建人名字");
        createName.setIsShowColumn(1);
        createName.setType("String");
        YcqCockpitModelColum updateName = new YcqCockpitModelColum();
        updateName.setName("UPDATE_WORKER_NAME");
        updateName.setRemark("更新人名字");
        updateName.setIsShowColumn(1);
        updateName.setType("String");
        addColumnList.add(id);
        addColumnList.add(createTime);
        addColumnList.add(createId);
        addColumnList.add(updateTime);
        addColumnList.add(updateId);
        addColumnList.add(createName);
        addColumnList.add(updateName);
        this.addColumn(columnCVo.getModelId(), addColumnList, null);
        return null;
    }


    public ModelResOneFixedVo getModelFixed(String name) {
        Map<String, Object> map = new HashMap<>();
        map.put("status", 1);
        map.put("name", name);
        YcqCockpitModel model = this.floModelRepo.getUniqueByProps(map);

        ModelResOneFixedVo one = new ModelResOneFixedVo();
        one.setValue(model.getName());
        one.setLabel(model.getRemark());

        Map<String, Object> mapC = new HashMap<>();
        mapC.put("status", 1);
        mapC.put("modelId", model.getId());
        List<YcqCockpitModelColum> columnList = this.floColumnRepo.findByProps(mapC);
        List<ColumnFixedVo> list = columnList.stream().map(YcqCockpitModelColum -> {
            ColumnFixedVo column = new ColumnFixedVo();
            column.setLabel(YcqCockpitModelColum.getRemark());
            column.setValue(YcqCockpitModelColum.getName());
            return column;
        }).collect(toList());
        one.setChildren(list);
        return one;
    }

    public Result<List<ModelResOneFixedVo>> doGetModelChilds(String name) {
        Set<String> names = Sets.newHashSet(StrUtil.split(name, ","));
//        List<ModelTableColumnDto> list = modelSqlBuilder.doGeTableColumns(names);
        List<ModelTableColumnDto> list = getDoGeTableColumns(names);

        Map<String, ModelTableColumnDto> tableMap = Maps.newHashMap();
        Map<String, List<ModelTableColumnDto>> columnMap = Maps.newHashMap();
        if (ObjectUtil.isNotEmpty(list)) {
            for (ModelTableColumnDto tableColumn : list) {
                if (!tableMap.containsKey(tableColumn.getTableName())) {
                    tableMap.put(tableColumn.getTableName(), tableColumn);
                }
                if (!columnMap.containsKey(tableColumn.getTableName())) {
                    columnMap.put(tableColumn.getTableName(), Lists.newArrayList());
                }
                columnMap.get(tableColumn.getTableName()).add(tableColumn);
            }
        }
        List<ModelResOneFixedVo> models = Lists.newArrayList();
        tableMap.forEach((k, v) -> {
            ModelResOneFixedVo one = new ModelResOneFixedVo();
            models.add(one);
            one.setValue(v.getTableName());
            one.setLabel(v.getTableDesc());
            List<ColumnFixedVo> columns = Lists.newArrayList();
            one.setChildren(columns);
            columnMap.get(k).forEach(c -> {
                ColumnFixedVo column = new ColumnFixedVo();
                column.setLabel(c.getColumnDesc());
                column.setValue(c.getColumnName());
                columns.add(column);
            });
        });
        return Result.ofSuccess(models);
    }


    public List<ModelResFixed> getModelListFixed(GetModeListFixedReq req) {
        Map<String, Object> map = new HashMap<>();
        map.put("status", 1);
        map.put("state", 2);
//        map.put("type", 1); // 必须为基础数据类型
        // map.put("systemId", req.getSystemId());
        List<YcqCockpitModel> list = this.floModelRepo.findByProps(map);

        List<ModelResFixed> resFixedList = list.stream()
                .map(model -> {
                    ModelResFixed res = new ModelResFixed();
                    res.setValue(model.getName());
                    res.setLabel(model.getRemark());
                    return res;
                }).collect(toList());
        return resFixedList;
    }


    private List<ModelTableColumnDto> getDoGeTableColumns(Set<String> ids) {
        List<Object> params = new ArrayList<>();
        String sql = "SELECT fm.NAME AS tableName,fm.REMARK AS tableDesc,fc.name AS columnName,fc.REMARK AS columnDesc FROM YCQ_COCKPIT_MODEL_COLUMN fc LEFT JOIN YCQ_COCKPIT_MODEL fm ON fc.MODEL_ID = fm.ID WHERE  fc.STATUS = 1 AND fm.STATUS = 1 and fc.name!='ID' and fc.name!='CREATE_TIME' and fc.name!='CREATE_WORKER_ID' and fc.name!='UPDATE_TIME' and fc.name!='UPDATE_WORKER_ID' and fc.name!='UPDATE_WORKER_NAME' and fc.name!='CREATE_WORKER_NAME' and fm.NAME in  ";
        String inIds = "(" + ids.stream().map(item -> "'" + item + "'").collect(Collectors.joining(StrUtil.COMMA)) + ")";
        StringBuilder sb = new StringBuilder(sql).append(inIds);
        return JT.queryForBeans(ModelTableColumnDto.class, sb.toString(), params.toArray());
    }


    public String getNextMenuCode(String parentMenuCode) {
        String code;
        List<String> codes = JT.queryForRawObjects(String.class, getNextMenuCodeSql, parentMenuCode);
        if (codes.isEmpty()) {
            code = parentMenuCode + "001"+UUID.randomUUID();
        } else {
            code = UUID.randomUUID().toString();
        }
        return code;
    }

//    private static String getNextCode(List<String> codes) {
//        Collections.sort(codes);
//        String last = codes.get(codes.size() - 1);
//        Integer next = Integer.valueOf(last.substring(last.length() - 3)) + 1;
//        return last.substring(0, last.length() - 3) + Strings.padStart(next.toString(), 3, '0');
//    }
}
