package com.crunii.micro.service.report.listener;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import org.springframework.jdbc.core.BatchPreparedStatementSetter;

import javax.annotation.Resource;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.*;

public class BasicDataExcelListener extends AnalysisEventListener<Map<Integer, Object>> {

    @Resource(name = "mainJT")
    private BaseJdbcTemplate JT;

    private List<Map<Integer, Object>> list = new ArrayList<>();

    private List<Map<String, Object>> columnInfoList;

    private List<Map<String, Object>> errorList;

    private Integer workerId;

    private Integer succCount=0;

    private String workerName;

    private final Integer COUNT = 50;

    private CountNum count;

    private String exeSql;

//    private final Integer systemId;

    public BasicDataExcelListener(List<Map<String, Object>> columnInfoList, BaseJdbcTemplate JT, Worker worker, Integer systemId, String exeSql) {
        this.columnInfoList = columnInfoList;
        this.errorList = new ArrayList<>();
        this.workerId = worker.getLoginId();
        this.workerName = worker.getWorkerName();
        this.count = new CountNum();
        this.JT = JT;
        this.exeSql = exeSql;
//        this.systemId = systemId;
    }

    @Override
    public void invoke(Map<Integer, Object> data, AnalysisContext analysisContext) {
        // 数据校验
//        boolean isError = false;

        int errorNum=0;
        StringBuilder errorMsg = new StringBuilder();
        Map<String, Object> errorMap = new HashMap<>();

        if (data.size() > columnInfoList.size()) {
            throw new BusinessException("导入列长度不能超过基础数据列长度!");
        }

        if (data.size() < columnInfoList.size()) {
            while ((columnInfoList.size() - data.size()) > 0) {
                data.put(data.size(), null);
            }
        }

        for (int index = 0; index < columnInfoList.size(); index++) {

            Object value = data.get(index);
            String type = columnInfoList.get(index).get("columnType").toString();
            String remark = columnInfoList.get(index).get("columnRemark").toString();
            errorMap.put(remark, value);
            if (ObjectUtil.isNotNull(value)) {
                if ("Integer".equals(type)) {
                    errorNum = isInteger(value, errorMsg,errorNum);
                } else if ("Double".equals(type)) {
                    errorNum = isDouble(value, errorMsg,errorNum);
                } else if ("Date".equals(type)) {
                    errorNum = isDate(value, errorMsg,errorNum);
                }
            }
        }

        if (errorNum>0) {
            errorMap.put("错误信息", errorMsg.toString());
            errorList.add(errorMap);
        } else {
            list.add(data);
            count.setSuccCount(count.getSuccCount()+1);
        }

//        if (list.size() >0) {
//            saveData();
//            list.clear();
//        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        saveData();
    }

    @Override
    public void onException(Exception exception, AnalysisContext context) throws Exception {
    }

    public void saveData() {
        try {
            if (list!=null&&list.size()>0){
                for (List<Map<Integer, Object>> maps : getSubList(500, list)) {
                    String dataSql = "";
                    if (maps!=null){
                        for (int i = 0; i < maps.size(); i++) {
                            String str = "";
                            for (int index = 0; index < columnInfoList.size(); index++) {
                                Map<Integer, Object> map = maps.get(i);
                                if (index == columnInfoList.size() - 1) {
                                    str += map.get(index)!=null?"'" + map.get(index) + "'":"''";
                                } else {
                                    str += map.get(index)!=null?"'" + map.get(index) + "'":"''";
                                    str += ",";
                                }
                            }
                            str+=",'"+workerId+"'";
                            str+=",'"+workerName+"'";
                            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                            str+=",'"+sdf.format(new Date())+"'";
                            if (i == maps.size() - 1) {
                                dataSql += " (" + str + ") ";
                            } else {
                                dataSql += " (" + str + ") , ";
                            }
                        }
                        JT.execute(exeSql + dataSql);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("sql语句批量执行失败!");
        }
    }


    public static <T>List<List<T>> getSubList(int length,List<T> list){
        int size = list.size();
        int temp = size / length + 1;
        boolean result = size % length == 0;
        List<List<T>> subList = new ArrayList<>();
        for (int i = 0; i < temp; i++) {
            if (i == temp - 1) {
                if (result) {
                    break;
                }
                subList.add(list.subList(length * i, size)) ;
            } else {
                subList.add(list.subList(length * i, length * (i + 1))) ;
            }
        }
        return subList;
    }

    private static class CountNum {
        private Integer succCount=0;

        public CountNum() {
            setSuccCount(0);
        }

        public Integer getSuccCount() {
            return succCount;
        }

        public void setSuccCount(Integer succCount) {
            this.succCount = succCount;
        }
    }

    public Integer getSuccCount() {
        return count.getSuccCount();
    }

    public List<Map<String, Object>> getErrorList() {
        return this.errorList;
    }

    private int isInteger(Object value, StringBuilder errorMsg, int errorNum) {
        try {
            Integer.parseInt(String.valueOf(value));
        } catch (Exception e) {
            errorMsg.append(value + "转换成整数失败!\n");
            return errorNum+1;
        }
        return errorNum;
    }

    private int isDouble(Object value, StringBuilder errorMsg, int errorNum) {
        try {
            Double.parseDouble(String.valueOf(value));
        } catch (Exception e) {
            errorMsg.append(value + "转换成浮点数失败!\n");
            return errorNum+1;
        }
        return errorNum;
    }

    private int isDate(Object value, StringBuilder errorMsg, int errorNum) {
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            sdf.parse(String.valueOf(value));
        } catch (Exception e) {
            errorMsg.append("转换成时间格式为:{yyyy-MM-dd HH:mm:ss}失败!");
            return errorNum+1;
        }
        return errorNum;
    }
}
