package com.crunii.micro.service.report.service;

import cn.hutool.core.collection.CollUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.report.dto.cockpit.map.grid.CockpitGridMapDataReqVO;
import com.crunii.micro.service.report.dto.cockpit.map.grid.CockpitGridMapYouthNumRespVO;
import com.crunii.micro.service.report.dto.cockpit.map.grid.GridMapDataRespVO;
import com.crunii.micro.service.report.dto.cockpit.map.position.CockpitPositionAreaSumRespVO;
import com.crunii.micro.service.report.dto.cockpit.map.position.CockpitPositionDataReqVO;
import com.crunii.micro.service.report.dto.cockpit.map.position.CockpitPositionDataRespVO;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;
import java.util.Set;

public interface CockpitMapService {

    BigDecimal CQ_LEFT_DOWN_LNG = new BigDecimal("105.28976");
    BigDecimal CQ_LEFT_DOWN_LAT = new BigDecimal("28.160225");
    BigDecimal CQ_RIGHT_UP_LNG = new BigDecimal("110.199858");
    BigDecimal CQ_RIGHT_UP_LAT = new BigDecimal("32.201187");
    Set<String> MAIN_AREA_NAME_SET = Collections.unmodifiableSet(
            CollUtil.newHashSet("巴南", "渝中", "南岸", "两江", "沙坪坝", "北碚", "大渡口",
                    "渝北", "九龙坡", "江北")
    );
    Set<String> OTHER_AREA_NAME_SET = Collections.unmodifiableSet(
            CollUtil.newHashSet("忠县", "永川", "开州", "垫江", "璧山", "万州", "南川", "长寿",
                    "涪陵", "秀山", "大足", "綦江", "合川", "潼南", "江津", "云阳", "铜梁", "城口",
                    "巫山", "巫溪", "梁平", "武隆", "石柱", "奉节", "丰都", "黔江", "彭水", "酉阳",
                    "荣昌")
    );

    /**
     * 查询驾驶舱栅格图列表
     *
     * @param reqVO 请求参数
     * @return 驾驶舱栅格图列表
     */
    Result<List<GridMapDataRespVO>> listCockpitGridMapData(CockpitGridMapDataReqVO reqVO);

    /**
     * 查询驾驶舱栅格图-人数极值
     *
     * @param reqVO 请求参数
     * @return 人数极值
     */
    Result<CockpitGridMapYouthNumRespVO> getCockpitGridMapYouthNum(CockpitGridMapDataReqVO reqVO);

    /**
     * 查询驾驶舱阵地区域地图列表
     *
     * @param reqVO 请求参数
     * @return 驾驶舱阵地区域地图列表
     */
    Result<List<CockpitPositionAreaSumRespVO>> listCockpitPositionDataAreaSum(CockpitPositionDataReqVO reqVO);

    /**
     * 查询驾驶舱阵地地图列表
     *
     * @param reqVO 请求参数
     * @return 驾驶舱阵地地图列表
     */
    Result<List<CockpitPositionDataRespVO>> listCockpitPositionMapData(CockpitPositionDataReqVO reqVO);

    /**
     * 阵地地图数据同步
     * 使用map_position_youth_cq表中的数据更新map_area_flag表
     *
     * @return 更新行数
     */
    Result<Integer> positionToAreaFlag();

}
