package com.crunii.micro.service.report.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.builder.ExcelWriterBuilder;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.RSAUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.YcqCockpitModelColum;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.report.dto.cockpit.*;
import com.crunii.micro.service.report.listener.BasicDataExcelListener;
import com.crunii.micro.service.report.service.CockpitDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;
import sun.misc.BASE64Encoder;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class CockpitDataServiceImpl implements CockpitDataService {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jdbcTemplate;

    /**
     * 获取属性
     */
    @Override
    public Result<List<CockpitDataGetAttributeResp>> getAttribute(CockpitDataGetAttributeReq req) {
        String tableName = req.getTableName();
        try {
            tableName = RSAUtil.decryptHex(tableName);
        } catch (Exception e) {
            throw new BusinessException("请加密!");
        }
        String selectIdSql = "select id from YCQ_COCKPIT_MODEL where name = " + "'" + tableName + "'";
        Long id = jdbcTemplate.queryForObject(selectIdSql, Long.class);
        String selectSql = "select * from YCQ_COCKPIT_MODEL_COLUMN where MODEL_ID = " + id;
        List<YcqCockpitModelColum> data = jdbcTemplate.queryForBeans(YcqCockpitModelColum.class, selectSql);
        LinkedList<CockpitDataGetAttributeResp> result = new LinkedList<>();
        CockpitDataGetAttributeResp header = null;
        List<CockpitDataGetAttributeResp> mainList = new ArrayList<>();
        List<CockpitDataGetAttributeResp> secondList = new ArrayList<>();
        for (int i = 0; i < data.size(); i++) {
            YcqCockpitModelColum resp = data.get(i);
            CockpitDataGetAttributeResp resp2 = new CockpitDataGetAttributeResp();
            BeanUtils.copyProperties(resp, resp2);
            String name = resp2.getName();
            if (name.equalsIgnoreCase("ID")) {
                header = resp2;
                header.setIsAddColumn(0);
            } else if (name.equalsIgnoreCase("CREATE_TIME")
                    || name.equalsIgnoreCase("UPDATE_TIME")
                    || name.equalsIgnoreCase("CREATE_WORKER_ID")
                    || name.equalsIgnoreCase("UPDATE_WORKER_ID")
                    || name.equalsIgnoreCase("CREATE_WORKER_NAME")
                    || name.equalsIgnoreCase("UPDATE_WORKER_NAME")) {
                resp2.setIsAddColumn(0);
                secondList.add(resp2);
            } else {
                resp2.setIsAddColumn(1);
                mainList.add(resp2);
            }
        }
        if (header != null) {
            result.addFirst(header);
        }
        result.addAll(mainList);
        result.addAll(secondList);
        return new Result(result);
    }

    /**
     * 查询
     */
    @Override
    public Result<Map<String, Object>> query(CockpitDataQueryReq req) {
        String tableName = req.getTableName();
        List<BasicDataColumnReq> basicDataColumnList = req.getBasicDataColumnList();
        List<BasicDataColumnReq> orderColumnList = req.getOrderColumnList();
        Integer type = req.getOrderType();
        try {
            tableName = RSAUtil.decryptHex(tableName);
        } catch (Exception e) {
            throw new BusinessException("请加密!");
        }
        String tableExistSql = "select count(*) from sysobjects where name = ?";
        if (jdbcTemplate.queryForInt(tableExistSql, tableName) == 0) {
            throw new BusinessException("表不存在!");
        }
        List<YcqCockpitModelColum> list = getColum(tableName);
        String colum = "";
        if (list != null && list.size() > 0) {
            for (YcqCockpitModelColum ycqCockpitModelColum : list) {
                if (ycqCockpitModelColum.getType() != null && ycqCockpitModelColum.getType().equals("Date")) {
                    colum = colum + "  TO_CHAR(" + ycqCockpitModelColum.getName() + ", 'YYYY-MM-DD') as " + ycqCockpitModelColum.getName() + " ,  ";
                } else {
                    colum = colum + ycqCockpitModelColum.getName() + " , ";
                }
            }
            colum = colum.substring(0, colum.length() - 2);
        }
        StringBuilder sb = new StringBuilder("select  * from " + tableName + " where 1=1 ");
        List<Object> params = new ArrayList<>();

        if (basicDataColumnList != null && !basicDataColumnList.isEmpty()) {
            for (BasicDataColumnReq column : basicDataColumnList) {
                String columnName = null;
                Object columnValue = column.getColumnValue();
                try {
                    columnName = RSAUtil.decryptHex(column.getColumnName());
                } catch (Exception e) {
                    throw new BusinessException("列名请编码!");
                }
                String sign = column.getSign();
                // 等于任意一个或者不等于任意一个
                if (sign.equals("in") || sign.equals("not in")) {
                    sb.append(" and " + columnName + " " + sign);
                    if (columnValue instanceof List) {
                        sb.append("(");
                        List<Object> lst = ((List) columnValue);
                        for (Object o : lst) {
                            sb.append("?,");
                            params.add(o);
                        }
                        sb = new StringBuilder(sb.substring(0, sb.length() - 1));
                        sb.append(") ");
                    } else {
                        sb.append(" (?) ");
                        params.add(columnValue);
                    }

                } else if (sign.equals("like") || sign.equals("not like")) { // 包含或者不包含
                    sb.append(" and " + columnName + " " + sign + " concat('%',?,'%')");
                    params.add(columnValue);
                } else if (sign.equals("is null") || sign.equals("is not null")) { // 为null,或者不为null
                    sb.append(" and " + columnName + " " + sign);
                } else { // 大于小于等于
                    sb.append(" and " + columnName + " " + sign + " ? ");
                    params.add(columnValue);
                }
            }
        }

        if (orderColumnList != null && orderColumnList.size() > 0) {
            sb.append(" ORDER BY  " );
            for (BasicDataColumnReq basicDataColumnReq : orderColumnList) {
                String columnName = basicDataColumnReq.getColumnName();
                try {
                    columnName = RSAUtil.decryptHex(columnName);
                } catch (Exception e) {
                    throw new BusinessException("列名请编码!");
                }
                sb.append(" "+columnName + ",");
            }
            sb = new StringBuilder(sb.substring(0, sb.length() - 1));
            if (type != null && type == 1) {
                sb.append(" ASC ");
            } else if (type != null && type == 2) {
                sb.append(" DESC ");
            }
        }



        List<Map<String, Object>> result = new ArrayList<>();
        result = this.jdbcTemplate.queryForList(sb.toString(), params.toArray());
        return new Result(result);
    }

    /**
     * 分页查询
     */
    @Override
    public Result<PageRsp<Map<String, Object>>> pageQuery(PageReq<CockpitDataQueryReq> req) {
        String tableName = req.getQuery().getTableName();
        List<BasicDataColumnReq> basicDataColumnList = req.getQuery().getBasicDataColumnList();
        List<BasicDataColumnReq> orderColumnList = req.getQuery().getOrderColumnList();
        Integer type = req.getQuery().getOrderType();
        try {
            tableName = RSAUtil.decryptHex(tableName);
        } catch (Exception e) {
            throw new BusinessException("请加密!");
        }
        String tableExistSql = "select count(*) from sysobjects where name = ?";
        if (jdbcTemplate.queryForInt(tableExistSql, tableName) == 0) {
            throw new BusinessException("表不存在!");
        }
        List<YcqCockpitModelColum> list = getColum(tableName);
        String colum = "";
        if (list != null && list.size() > 0) {
            for (YcqCockpitModelColum ycqCockpitModelColum : list) {
                if (ycqCockpitModelColum.getType() != null && ycqCockpitModelColum.getType().equals("Date")) {
                    colum = colum + "  TO_CHAR(" + ycqCockpitModelColum.getName() + ", 'YYYY-MM-DD') as " + ycqCockpitModelColum.getName() + " ,  ";
                } else {
                    colum = colum + ycqCockpitModelColum.getName() + " , ";
                }
            }
        }
        colum = colum.substring(0, colum.length() - 2);
        StringBuilder sb = new StringBuilder("select  * from " + tableName + " where 1=1 ");
        StringBuilder sbCount = new StringBuilder("select count(1) from " + tableName + " where 1=1 ");
        List<Object> params = new ArrayList<>();

        if (basicDataColumnList != null && !basicDataColumnList.isEmpty()) {
            for (BasicDataColumnReq column : basicDataColumnList) {
                String columnName = null;
                Object columnValue = column.getColumnValue();
                try {
                    columnName = RSAUtil.decryptHex(column.getColumnName());
                } catch (Exception e) {
                    throw new BusinessException("列名请编码!");
                }
                String sign = column.getSign();
                sign = RSAUtil.decryptHex(sign);
                // 等于任意一个或者不等于任意一个
                if (sign.equals("in") || sign.equals("not in")) {
                    sb.append(" and " + columnName + " " + sign);
                    sbCount.append(" and " + columnName + " " + sign);
                    if (columnValue instanceof List) {
                        sb.append("(");
                        sbCount.append("(");
                        List<Object> lst = ((List) columnValue);
                        for (Object o : lst) {
                            sb.append("?,");
                            sbCount.append("?,");
                            params.add(o);
                        }
                        sb = new StringBuilder(sb.substring(0, sb.length() - 1));
                        sbCount = new StringBuilder(sbCount.substring(0, sbCount.length() - 1));
                        sb.append(") ");
                        sbCount.append(") ");
                    } else {
                        sb.append(" (?) ");
                        sbCount.append(" (?) ");
                        params.add(columnValue);
                    }

                } else if (sign.equals("like") || sign.equals("not like")) { // 包含或者不包含
                    sb.append(" and " + columnName + " " + sign + " concat('%',?,'%')");
                    sbCount.append(" and " + columnName + " " + sign + " concat('%',?,'%')");
                    params.add(columnValue);
                } else if (sign.equals("is null") || sign.equals("is not null")) { // 为null,或者不为null
                    sb.append(" and " + columnName + " " + sign);
                    sbCount.append(" and " + columnName + " " + sign);
                } else { // 大于小于等于
                    sb.append(" and " + columnName + " " + sign + " ? ");
                    sbCount.append(" and " + columnName + " " + sign + " ? ");
                    params.add(columnValue);
                }
            }
        }

        if (orderColumnList != null && orderColumnList.size() > 0) {
            sb.append(" ORDER BY  " );
            for (BasicDataColumnReq basicDataColumnReq : orderColumnList) {
                String columnName = basicDataColumnReq.getColumnName();
                try {
                    columnName = RSAUtil.decryptHex(columnName);
                } catch (Exception e) {
                    throw new BusinessException("列名请编码!");
                }
                sb.append( " "+columnName + ",");
            }
            sb = new StringBuilder(sb.substring(0, sb.length() - 1));
            if (type != null && type == 1) {
                sb.append(" ASC ");
            } else if (type != null && type == 2) {
                sb.append(" DESC ");
            }
        }



        int total = this.jdbcTemplate.queryForInt(sbCount.toString(), params.toArray());
        List<Map<String, Object>> result = new ArrayList<>();
        if (total > 0) {
            sb.append("  limit ?,? ");
            params.add(req.getMyStart());
            params.add(req.getPageSize());

            result = this.jdbcTemplate.queryForList(sb.toString(), params.toArray());
        } else {
            result = new ArrayList<>(0);
        }
        PageRsp<Map<String, Object>> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(result);
        // 为手机计算是否由下一页
        page.next();
        return new Result(page);
    }

    /**
     * 新增
     */
    @Override
    public Result<Boolean> insert(CockpitDataInsertReq req) {
        if (req == null) {
            throw new BusinessException("无任何数据传入");
        }

        Worker worker = getWorker();
        String tableName = req.getTableName();
        try {
            tableName = RSAUtil.decryptHex(tableName);
        } catch (Exception e) {
            throw new BusinessException("请加密!");
        }
        StringBuilder insertSql = new StringBuilder();
        insertSql.append("insert into ").append(tableName).append(" (");
        for (CockpitDataColumn column : req.getColumnList()) {
            String columnName = "";
            try {
                columnName = RSAUtil.decryptHex(column.getColumnName());
            } catch (Exception e) {
                throw new BusinessException("入参未加密!");
            }
            insertSql.append(columnName).append(",");
        }
        insertSql = new StringBuilder(insertSql.substring(0, insertSql.length() - 1));
        insertSql.append(" , CREATE_TIME");
        insertSql.append(" , CREATE_WORKER_NAME");
        insertSql.append(" , CREATE_WORKER_ID");
        insertSql.append(") values (");
        for (int i = 0; i < req.getColumnList().size() + 3; i++) {
            insertSql.append("?,");
        }
        insertSql = new StringBuilder(insertSql.substring(0, insertSql.length() - 1));

        insertSql.append(")");
        List<Object> params = new ArrayList<>();
        for (CockpitDataColumn Column : req.getColumnList()) {
            params.add(Column.getColumnValue());
        }
//        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
//        params.add("'"+sdf.format(new Date())+"'");
        params.add(new Date());
        params.add(worker.getWorkerName());
        params.add(worker.getLoginId());
        jdbcTemplate.update(insertSql.toString(), params.toArray());
        return new Result(true);
    }

    /**
     * 修改
     */
    @Override
    public Result<Boolean> update(CockpitDataUpdateReq req) {
        if (req == null) {
            throw new BusinessException("无任何数据传入");
        }
        String tableName = req.getTableName();
        try {
            tableName = RSAUtil.decryptHex(tableName);
        } catch (Exception e) {
            throw new BusinessException("请加密!");
        }
        StringBuilder updateSql = new StringBuilder();
        updateSql.append("update ").append(tableName).append(" set ");
        for (CockpitDataColumn column : req.getColumnList()) {
            String name = "";
            try {
                name = RSAUtil.decryptHex(column.getColumnName());
            } catch (Exception e) {
                throw new BusinessException("入参未加密!");
            }
            if (!name.equals("id")) {
                updateSql.append(name).append(" = ?,");
            }

        }

        updateSql = new StringBuilder(updateSql.substring(0, updateSql.length() - 1));
        updateSql.append(",UPDATE_WORKER_ID = ?");
        updateSql.append(",UPDATE_WORKER_NAME = ?");
        updateSql.append(",UPDATE_TIME = ?");
        updateSql.append(" where id = ").append(req.getId());
        List<Object> params = new ArrayList<>();
        for (CockpitDataColumn column : req.getColumnList()) {
            String name = "";
            try {
                name = RSAUtil.decryptHex(column.getColumnName());
            } catch (Exception e) {
                throw new BusinessException("入参未加密!");
            }
            if (!name.equals("id")) {
                params.add(column.getColumnValue());
            }
        }
        Worker worker = getWorker();
        params.add(worker.getLoginId());
        params.add(worker.getWorkerName());
        params.add(new Date());
        jdbcTemplate.update(updateSql.toString(), params.toArray());
        return new Result(true);
    }

    /**
     * 删除
     */
    @Override
    public Result<Boolean> delete(CockpitDataDeleteReq req) {
        if (req == null) {
            throw new BusinessException("无任何数据传入");
        }
        String tableName = req.getTableName();
        try {
            tableName = RSAUtil.decryptHex(tableName);
        } catch (Exception e) {
            throw new BusinessException("请加密!");
        }
        StringBuilder deleteSql = new StringBuilder();
        deleteSql.append("delete from ").append(tableName).append(" where id = ").append(req.getId());
        jdbcTemplate.update(deleteSql.toString());
        return new Result(true);
    }

    /**
     * 批量删除
     */
    @Override
    public Result<Boolean> batchDelete(CockpitDataBatchDeleteReq req) {
        if (req == null) {
            throw new BusinessException("无任何数据传入");
        }
        String tableName = req.getTableName();
        try {
            tableName = RSAUtil.decryptHex(tableName);
        } catch (Exception e) {
            throw new BusinessException("请加密!");
        }
        List<Long> ids = req.getIdList();
        StringBuilder condition = new StringBuilder("(");
        for (int i = 0; i < ids.size(); i++) {
            condition.append("?,");
        }
        condition = new StringBuilder(condition.substring(0, condition.length() - 1));
        condition.append(")");
        String deleteSql = "delete from " + tableName + " where id in " + condition.toString();
        jdbcTemplate.update(deleteSql, ids.toArray());
        return new Result(true);
    }

    /**
     * 获取所有表名
     */
    @Override
    public Result<List<CockpitDataGetNamesResp>> getNames() {
        String selectSql = "select * from YCQ_COCKPIT_MODEL";
        List<CockpitDataGetNamesResp> result = jdbcTemplate.queryForBeans(CockpitDataGetNamesResp.class, selectSql);
        return new Result(result);
    }

    /**
     * 获取属性值
     */
    @Override
    public Result<List<CockpitDataGetAttributeValueResp>> getAttributeValue(CockpitDataGetAttributeValueReq req) {
        String tableName = req.getTableName();
        String columnName = req.getColumnName();
        try {
            tableName = RSAUtil.decryptHex(req.getTableName());
            columnName = RSAUtil.decryptHex(req.getColumnName());
        } catch (Exception e) {
            throw new BusinessException("入参请加密!");
        }
        if (!StringUtils.hasText(tableName) || !StringUtils.hasText(columnName)) {
            throw new BusinessException("tableName或columnName必须传入值!");
        }
        String sql = "SELECT DISTINCT " + columnName + " AS VALUE  FROM " + tableName + " WHERE 1=1 AND " + columnName + " IS NOT NULL ";
        List<CockpitDataGetAttributeValueResp> result = jdbcTemplate.queryForBeans(CockpitDataGetAttributeValueResp.class, sql);
        return new Result(result);
    }

    /**
     * 下载模板
     */
    @Override
    public Result<Boolean> download(CockpitDataDownloadReq req, HttpServletResponse response) {
        if (req == null) {
            throw new BusinessException("请求参数为null");
        }
        String tableName = req.getTableName();
        try {
            tableName = RSAUtil.decryptHex(tableName);
        } catch (Exception e) {
            throw new BusinessException("参数tableName未编码!");
        }
        String selectIdSql = "select id from YCQ_COCKPIT_MODEL where name = " + "'" + tableName + "'";
        Long id = jdbcTemplate.queryForObject(selectIdSql, Long.class);
        List<String> columns = listColumns(id);
        try {
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");
            // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
//            String fileName = jdbcTemplate.queryForRawObject(String.class, "select REMARK from YCQ_COCKPIT_MODEL where name = ? limit 1 ", tableName);
//            // 如果没有找到数据模型对应的备注名称 默认 导出数据
//            if (StrUtil.isBlank(fileName)) {
//                fileName = tableName;
//            }
            String fileName = URLEncoder.encode("导入模板", "UTF-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream()).head(dynanichHead(id, columns)).sheet("sheet1").doWrite(new ArrayList());
        } catch (IOException e) {
            throw new BusinessException("输出流异常");
        }
        return new Result(true);
    }

    private List<String> listColumns(Long id) {
        String selectSql = "select NAME from YCQ_COCKPIT_MODEL_COLUMN fc where  fc.name!='ID' and fc.name!='CREATE_TIME' and fc.name!='CREATE_WORKER_ID' and fc.name!='UPDATE_TIME' and fc.name!='UPDATE_WORKER_ID' and fc.name!='UPDATE_WORKER_NAME' and fc.name!='CREATE_WORKER_NAME' and MODEL_ID = " + id;
        return jdbcTemplate.queryForList(selectSql, String.class).stream().filter(column -> {
            if (column.equalsIgnoreCase("ID") || column.equalsIgnoreCase("SYSTEM_ID") || column.equalsIgnoreCase("CREATE_TIME") ||
                    column.equalsIgnoreCase("UPDATE_TIME") || column.equalsIgnoreCase("CREATE_WORKER_ID") || column.equalsIgnoreCase("UPDATE_WORKER_ID")
                    || column.equalsIgnoreCase("CREATE_WORKER_NAME") || column.equalsIgnoreCase("UPDATE_WORKER_NAME")) {
                return false;
            }
            return true;
        }).collect(Collectors.toList());
    }

    private List<List<String>> dynanichHead(Long id, List<String> columns) {
        List<List<String>> list = new ArrayList<List<String>>();
        for (String column : columns) {
            List<String> head = new ArrayList<>();
            head.add(parseHeadInfoToComment(id, column));
            list.add(head);
        }
        return list;
    }

    private String parseHeadInfoToComment(Long id, String columnName) {
        String selectSql = "select REMARK from YCQ_COCKPIT_MODEL_COLUMN where MODEL_ID = ? and NAME = ?";
        String column = jdbcTemplate.queryForObject(selectSql, String.class, id, columnName);
        if (StringUtils.isEmpty(column)) {
            return columnName;
        }
        return column;
    }

    /**
     * 导入Excel
     */
    @Override
    public Result<Object> importExcel(MultipartFile excel, HttpServletResponse response, String tableName) {
        try {
            tableName = RSAUtil.decryptHex(tableName);
        } catch (Exception e) {
            throw new BusinessException("tableName请加密!");
        }
        String selectIdSql = "select id from YCQ_COCKPIT_MODEL where name = " + "'" + tableName + "'";
        Long id = jdbcTemplate.queryForObject(selectIdSql, Long.class);
        List<Map<String, Object>> columnInfoList = getColumnList(id);
        List<String> nativeColumnInfoList = getNativeColumnList(id);

        if (ObjectUtil.isNull(columnInfoList) || columnInfoList.isEmpty()) {
            throw new BusinessException("无任何字段!");
        }

        if (ObjectUtil.isNull(nativeColumnInfoList) || nativeColumnInfoList.isEmpty()) {
            throw new BusinessException("数据库无对应表字段!");
        }

        isContainNativeColumns(columnInfoList, nativeColumnInfoList);

        List<String> errorColumns = new ArrayList<>();
        StringBuilder sql = new StringBuilder("INSERT INTO ").append(tableName).append("(");
//        StringBuilder valueBody = new StringBuilder("(  ");
//        final int systemId = jdbcTemplate.queryForInt("SELECT SYSTEM_ID FROM FLO_MODEL WHERE TYPE = ? AND NAME = ? ", 0, tableName);
        for (Map<String, Object> columninfo : columnInfoList) {
            String columnName = columninfo.get("columnName").toString();
            String columnRemark = columninfo.get("columnRemark").toString();
            errorColumns.add(columnRemark);
//            sql.append("`" + columnName + "`,");
            sql.append(columnName + ",");
//            valueBody.append("?,");
        }
        sql = new StringBuilder(sql.substring(0, sql.length() - 1));
//        valueBody = new StringBuilder(valueBody.substring(0, valueBody.length() - 1));
//        sql.append(" `SYSTEM_ID`, `CREATE_TIME`, `CREATE_WORKER_ID`, `CREATE_WORKER_NAME`) VALUES ");
        sql.append(" , CREATE_WORKER_ID, CREATE_WORKER_NAME,CREATE_TIME ) VALUES ");
//        valueBody.append(" ?, ?, ?, ?) ");
//        valueBody.append(")");
//        sql.append(valueBody);
        errorColumns.add("错误信息");
        Worker worker = getWorker();
        BasicDataExcelListener listener = new BasicDataExcelListener(columnInfoList, jdbcTemplate, worker, null, sql.toString());
        try {
            EasyExcel.read(excel.getInputStream(), listener).sheet().headRowNumber(1).doRead();
            if (listener.getErrorList().size() > 0) {
                try {
                    ByteArrayOutputStream os = new ByteArrayOutputStream();
                    ExcelWriterBuilder builder = new ExcelWriterBuilder();
                    builder.sheet("错误信息");
                    builder.head(dynanichHead(errorColumns.toArray(new String[0])));
                    builder.file(os);
                    builder.needHead(true);
                    WriteSheet sheet = new WriteSheet();
                    sheet.setSheetNo(1);
                    sheet.setSheetName("错误信息");
                    ExcelWriter writer = builder.build();
                    writer.write(parseResultSet(errorColumns.toArray(new String[0]), listener.getErrorList()), sheet);
                    writer.finish();
                    /*ExcelWriter excelWriter = EasyExcel.write().withTemplate(excel.getInputStream()).file(os).build();
                    WriteSheet writeSheet = EasyExcel.writerSheet().head(dynanichHead(errorColumns.toArray(new String[0]))).build();
                    excelWriter.fill(parseResultSet(errorColumns.toArray(new String[0]), errorList), writeSheet);
                    excelWriter.finish();*/
                    String excel_base64 = new BASE64Encoder().encode(os.toByteArray());

                    Map<String, Object> map = new HashMap<>();
                    map.put("successCount", listener.getSuccCount());
                    map.put("errorCount", listener.getErrorList().size());
                    map.put("excel", excel_base64);

                    return new Result<>(map);

                    /*response.setContentType("application/vnd.ms-excel");
                    response.setCharacterEncoding("utf-8");
                    // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系  菜单名+表名

                    String fileName = "导入错误信息";
                    response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");*/
                    // 输出错误excel表格
                    //EasyExcel.write(response.getOutputStream()).head(dynanichHead(errorColumns.toArray(new String[0]))).sheet("基础数据集").doWrite(parseResultSet(errorColumns.toArray(new String[0]), errorList));
                } catch (Exception e) {
                    throw new BusinessException("输出错误信息excel失败!");
                }
            }
        } catch (Exception e) {
            throw new BusinessException("excel文件读取失败!");
        }
        return new Result<>("导入数据成功!", true);
    }

    private List<Map<String, Object>> getColumnList(Long id) {
        String selectSql = "select NAME as columnName, TYPE as columnType, REMARK as columnRemark from YCQ_COCKPIT_MODEL_COLUMN fc where  fc.name!='ID' and fc.name!='CREATE_TIME' and fc.name!='CREATE_WORKER_ID' and fc.name!='UPDATE_TIME' and fc.name!='UPDATE_WORKER_ID' and fc.name!='UPDATE_WORKER_NAME' and fc.name!='CREATE_WORKER_NAME' and MODEL_ID = " + id;
        return jdbcTemplate.queryForList(selectSql);
    }

    private List<String> getNativeColumnList(Long id) {
        String selectSql = "select NAME from YCQ_COCKPIT_MODEL_COLUMN where MODEL_ID = " + id;
        return jdbcTemplate.queryForList(selectSql, String.class);
    }

    private void isContainNativeColumns(List<Map<String, Object>> columnInfoList, List<String> nativeColumnInfoList) {
        for (Map<String, Object> columnInfo : columnInfoList) {
            String columnName = String.valueOf(columnInfo.get("columnName"));
            if (!nativeColumnInfoList.contains(columnName)) {
                throw new BusinessException("字段:{" + columnName + "}不存在!");
            }
        }
    }

    private List<List<String>> dynanichHead(String[] columns) {
        List<List<String>> list = new ArrayList<List<String>>();
        for (String column : columns) {
            List<String> head = new ArrayList<>();
            head.add(column);
            list.add(head);
        }
        return list;
    }

    private List<List<String>> parseResultSet(String[] columns, List<Map<String, Object>> result) {
        List<List<String>> lists = new ArrayList<>();
        for (Map<String, Object> map : result) {
            List<String> list = new ArrayList<>();
            for (String column : columns) {
                list.add(map.get(column) == null ? "" : map.get(column).toString());
            }
            lists.add(list);
        }
        return lists;
    }

    private Worker getWorker() {
        Worker worker = CurrWorkerUtil.getCurrWorker();
        if (ObjectUtil.isEmpty(worker)) {
            throw new RuntimeException("获取当前登录用户失败");
        }
        return worker;

    }

    @Override
    public List<BasicDataColumnValueResp> getFilterMapValue(BasicDataColumnValueReq req) {
        String tableName = req.getTableName();
        String columnName = req.getColumnName();

        try {
            tableName = RSAUtil.decryptHex(req.getTableName());
            columnName = RSAUtil.decryptHex(req.getColumnName());
        } catch (Exception e) {
            throw new BusinessException("入参请加密!");
        }

        if (!StringUtils.hasText(tableName) || !StringUtils.hasText(columnName)) {
            throw new BusinessException("tableName或columnName必须传入值!");
        }
        String sql = "SELECT DISTINCT " + columnName + " AS VALUE  FROM " + tableName + " WHERE 1=1 AND " + columnName
                + " IS NOT NULL ";

        return this.jdbcTemplate.queryForBeans(BasicDataColumnValueResp.class, sql);
    }

    private List<YcqCockpitModelColum> getColum(String tableName) {
        String selectIdSql = "select id from YCQ_COCKPIT_MODEL where name = " + "'" + tableName + "'";
        Long id = jdbcTemplate.queryForObject(selectIdSql, Long.class);
        String selectSql = "select * from YCQ_COCKPIT_MODEL_COLUMN where MODEL_ID = " + id;
        return jdbcTemplate.queryForBeans(YcqCockpitModelColum.class, selectSql);
    }

    /**
     * 获取字段
     */
    @Override
    public Result<List<CockpitDataGetFieldResp>> getField(CockpitDataGetAttributeReq req) {
        String tableName = req.getTableName();
        try {
            tableName = RSAUtil.decryptHex(tableName);
        } catch (Exception e) {
            throw new BusinessException("请加密!");
        }
        String selectIdSql = "select id from YCQ_COCKPIT_MODEL where name = " + "'" + tableName + "'";
        Long id = jdbcTemplate.queryForObject(selectIdSql, Long.class);
        String selectSql = "select * from YCQ_COCKPIT_MODEL_COLUMN where MODEL_ID = " + id;
        List<CockpitDataGetFieldResp> result = jdbcTemplate.queryForBeans(CockpitDataGetFieldResp.class, selectSql);
        for(int i = result.size()-1; i>=0; i--){
            String name = result.get(i).getName();
            if(name.equalsIgnoreCase("ID")
             ||name.equalsIgnoreCase("CREATE_TIME")
             ||name.equalsIgnoreCase("UPDATE_TIME")
             ||name.equalsIgnoreCase("CREATE_WORKER_ID")
             ||name.equalsIgnoreCase("UPDATE_WORKER_ID")
             ||name.equalsIgnoreCase("CREATE_WORKER_NAME")
             ||name.equalsIgnoreCase("UPDATE_WORKER_NAME")) {
                result.remove(i);
            }
        }
        return new Result(result);
    }

}
