package com.crunii.micro.service.report.service.impl;

import com.crunii.micro.service.report.constants.ReportConstants;
import com.crunii.micro.service.report.dto.*;
import com.crunii.micro.service.report.jdbc.FiveGoalDao;
import com.crunii.micro.service.report.service.FiveGoalService;
import lombok.extern.slf4j.Slf4j;
import org.junit.rules.Stopwatch;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;
import org.springframework.util.StringUtils;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * @author wangchen create 2023/7/14 10:13
 */
@Slf4j
@Service
public class FiveGoalServiceImpl implements FiveGoalService {

    private final FiveGoalDao fiveGoalDao;

    public FiveGoalServiceImpl(FiveGoalDao fiveGoalDao) {
        this.fiveGoalDao = fiveGoalDao;
    }

    @Override
    public Object findFiveGoal(String valueCode) {
        if (ReportConstants.OVERALL_CONTRIBUTION.equals(valueCode)) {
            OverallContributionVo overallContributionVo = fiveGoalDao.findNowContributionByType(valueCode);
            OverallContributionVo overallContribution = fiveGoalDao.findGoalContributionByType(valueCode);
            overallContributionVo.setTarget(overallContribution);
            return overallContributionVo;
        }
        FiveGoalVo fiveGoalNow = fiveGoalDao.findFiveGoalNowByType(valueCode);
        FiveGoalVo fiveGoal = fiveGoalDao.findFiveGoalByType(valueCode);
        return chooseType(fiveGoalNow, valueCode, fiveGoal);
    }

    /**
     * 返回对象
     */
    private Object chooseType(FiveGoalVo fiveGoalNow, String valueCode, FiveGoalVo fiveGoal) {
        switch (valueCode) {
            case "1":
                return handleLeadership(fiveGoalNow,fiveGoal);
            case "2":
                return handleOrganizationalPower(fiveGoalNow,fiveGoal);
            case "3":
                return handleServicePower(fiveGoalNow,fiveGoal);
            case "5":
                return handleStrictlyGovernTheRegiment(fiveGoalNow,fiveGoal);
            default:
                return null;
        }
    }

    /**
     * 从严治党
     */
    private StrictlyGovernTheRegimentVo handleStrictlyGovernTheRegiment(FiveGoalVo fiveGoalByType,FiveGoalVo fiveGoal) {
        StrictlyGovernTheRegimentVo strictlyGovernTheRegimentVo = new StrictlyGovernTheRegimentVo();
        List<String> names = new ArrayList<>();
        names.add(fiveGoalByType.getLeaderWithRateName());
        names.add(fiveGoalByType.getBaseOrgRateName());
        names.add(fiveGoalByType.getTeacherPartyMemberRateName());
        names.add(fiveGoalByType.getYouthAntiCorruptionActivitiesName());
        List<String> values = new ArrayList<>();
        values.add(fiveGoalByType.getLeaderWithRate());
        values.add(fiveGoalByType.getBaseOrgRate());
        values.add(fiveGoalByType.getTeacherPartyMemberRate());
        values.add(fiveGoalByType.getYouthAntiCorruptionActivities());
        List<String> goal = new ArrayList<>();
        goal.add(fiveGoal.getLeaderWithRate());
        goal.add(fiveGoal.getBaseOrgRate());
        goal.add(fiveGoal.getTeacherPartyMemberRate());
        goal.add(fiveGoal.getYouthAntiCorruptionActivities());
        NameValueVo build = NameValueVo.builder().names(names).values(values).goal(goal).build();
        strictlyGovernTheRegimentVo.setNameValue(build);
        return strictlyGovernTheRegimentVo;
    }

    /**
     * 服务力
     */
    private ServicePowerVo handleServicePower(FiveGoalVo fiveGoalByType,FiveGoalVo fiveGoal) {
        ServicePowerVo servicePowerVo = new ServicePowerVo();
        List<String> names = new ArrayList<>();
        names.add(fiveGoalByType.getYouthPsychologyPlaceName());
        names.add(fiveGoalByType.getYouthHomeName());
        names.add(fiveGoalByType.getOutsideSchoolEduName());
        names.add(fiveGoalByType.getHelpCollegeWorkName());
        List<String> values = new ArrayList<>();
        values.add(fiveGoalByType.getYouthPsychologyPlace());
        values.add(fiveGoalByType.getYouthHome());
        values.add(fiveGoalByType.getOutsideSchoolEdu());
        values.add(fiveGoalByType.getHelpCollegeWork());
        List<String> goal = new ArrayList<>();
        goal.add(fiveGoal.getYouthPsychologyPlace());
        goal.add(fiveGoal.getYouthHome());
        goal.add(fiveGoal.getOutsideSchoolEdu());
        goal.add(fiveGoal.getHelpCollegeWork());
        NameValueVo build = NameValueVo.builder().names(names).goal(goal).values(values).build();
        servicePowerVo.setServicePower(build);
        return servicePowerVo;
    }

    /**
     * 组织力
     */
    private OrganizationalPowerVo handleOrganizationalPower(FiveGoalVo fiveGoalByType,FiveGoalVo fiveGoal) {
        OrganizationalPowerVo organizationalPowerVo = new OrganizationalPowerVo();
        // 处理覆盖率
        List<String> rateName = new ArrayList<>();
        rateName.add(fiveGoalByType.getOrgCoverName());
        rateName.add(fiveGoalByType.getExcellentEnterOrgRateName());
        rateName.add(fiveGoalByType.getStudyConnectRateName());
        rateName.add(fiveGoalByType.getExcellentEnterCcpRateName());
        List<String> rate = new ArrayList<>();
        rate.add(fiveGoalByType.getOrgCover());
        rate.add(fiveGoalByType.getExcellentEnterOrgRate());
        rate.add(fiveGoalByType.getStudyConnectRate());
        rate.add(fiveGoalByType.getExcellentEnterCcpRate());
        // 处理目标
        List<String> goal = new ArrayList<>();
        goal.add(fiveGoal.getOrgCover());
        goal.add(fiveGoal.getExcellentEnterOrgRate());
        goal.add(fiveGoal.getStudyConnectRate());
        goal.add(fiveGoal.getExcellentEnterCcpRate());
        NameValueVo nameValueVo = NameValueVo.builder().names(rateName).values(rate).goal(goal).build();
        organizationalPowerVo.setRate(nameValueVo);
        // 处理其他
        List<String> names = new ArrayList<>();
        names.add(fiveGoalByType.getNonPublicOrgName());
        names.add(fiveGoalByType.getOrgDensityName());
        List<String> values = new ArrayList<>();
        values.add(fiveGoalByType.getNonPublicOrg());
        values.add(fiveGoalByType.getOrgDensity());
        List<String> goalOther = new ArrayList<>();
        goalOther.add(fiveGoal.getNonPublicOrg());
        goalOther.add(fiveGoal.getOrgDensity());
        NameValueVo nameValue = NameValueVo.builder().names(names).goal(goalOther).values(values).build();
        nameValue.setNames(names);
        nameValue.setValues(values);
        organizationalPowerVo.setOther(nameValue);
        return organizationalPowerVo;
    }

    /**
     * 处理引领力
     */
    private LeadershipVo handleLeadership(FiveGoalVo fiveGoalByType, FiveGoalVo fiveGoal) {
        LeadershipVo leadershipVo = new LeadershipVo();
        leadershipVo.setUnit("个");
        leadershipVo.setLeagueOrgCount(fiveGoalByType.getLeagueOrgCount());
        // 处理覆盖率
        List<String> rateName = new ArrayList<>();
        rateName.add(fiveGoalByType.getEduCoverageRateName());
        rateName.add(fiveGoalByType.getYouthPoliticsBackboneName());
        List<String> rate = new ArrayList<>();
        rate.add(fiveGoalByType.getEduCoverageRate());
        rate.add(fiveGoalByType.getYouthPoliticsBackbone());
        List<String> goal = new ArrayList<>();
        goal.add(fiveGoal.getEduCoverageRate());
        goal.add(fiveGoal.getYouthPoliticsBackbone());
        NameValueVo nameValueVo = NameValueVo.builder().names(rateName).goal(goal).values(rate).build();
        leadershipVo.setRate(nameValueVo);
        // 处理排名
        List<GoalRankVo> goalRankVos = new ArrayList<>();
        GoalRankVo goalRankVo = new GoalRankVo();
        goalRankVo.setName(fiveGoalByType.getOneName());
        goalRankVo.setRankName(fiveGoalByType.getRankOneName());
        goalRankVo.setRank(fiveGoalByType.getOneRank());
        goalRankVos.add(goalRankVo);

        GoalRankVo goalRank = new GoalRankVo();
        goalRank.setName(fiveGoalByType.getTwoName());
        goalRank.setRankName(fiveGoalByType.getRankTwoName());
        goalRank.setRank(fiveGoalByType.getTwoRank());
        goalRankVos.add(goalRank);

        GoalRankVo goalRankThree = new GoalRankVo();
        goalRankThree.setName(fiveGoalByType.getThreeName());
        goalRankThree.setRankName(fiveGoalByType.getRankThreeName());
        goalRankThree.setRank(fiveGoalByType.getThreeRank());
        goalRankVos.add(goalRankThree);
        leadershipVo.setRank(goalRankVos);
        return leadershipVo;
    }

}
