package com.crunii.micro.service.report.service.impl;

import com.crunii.micro.service.report.constants.ReportConstants;
import com.crunii.micro.service.report.dto.LeagueIndustryCountVo;
import com.crunii.micro.service.report.dto.LeagueOrgVo;
import com.crunii.micro.service.report.dto.LeagueProportionVo;
import com.crunii.micro.service.report.enums.LeagueNameEnum;
import com.crunii.micro.service.report.jdbc.LeagueSituationDao;
import com.crunii.micro.service.report.service.LeagueSituationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * @author wangchen create 2023/7/12 17:02
 */
@Slf4j
@Service
public class LeagueSituationServiceImpl implements LeagueSituationService {


    private final LeagueSituationDao leagueSituationDao;

    public LeagueSituationServiceImpl(LeagueSituationDao leagueSituationDao) {
        this.leagueSituationDao = leagueSituationDao;
    }

    @Override
    public LeagueOrgVo findSituation(String valueCode) {
         return chooseSituation(valueCode);
    }

    @Override
    public LeagueOrgVo findSituationOrg() {
        LeagueOrgVo leagueOrgVo = new LeagueOrgVo();
        // 查询组织总数
        Integer allLeagueOrgCount = leagueSituationDao.findAllLeagueOrgCount();
        if(allLeagueOrgCount.equals(0)){
            leagueOrgVo.setLeagueOrgNum("0");
            leagueOrgVo.setUnit("人");
            return leagueOrgVo;
        }
        // 保存组织总数
        handleNum(leagueOrgVo,allLeagueOrgCount);
        // 查询组织类型百分比
        List<LeagueProportionVo> leagueOrgProportionVos = leagueSituationDao.findLeagueOrgProportion();
        handleProportion(leagueOrgVo,leagueOrgProportionVos,allLeagueOrgCount);
        // 查询领域分布
        List<LeagueIndustryCountVo> leagueIndustryCountVos = leagueSituationDao.findLeagueOrgIndustry();
        leagueIndustryCountVos = handleLeagueIndustry(leagueIndustryCountVos);
        leagueOrgVo.setLeagueIndustries(leagueIndustryCountVos);
        return leagueOrgVo;
    }

    @Override
    public LeagueOrgVo findSituationMember() {
        LeagueOrgVo leagueOrgVo = new LeagueOrgVo();
        // 查询团员总数
        Integer allLeagueOrgCount = leagueSituationDao.findAllLeagueMemberCount();
        if(allLeagueOrgCount.equals(0)){
            leagueOrgVo.setLeagueOrgNum("0");
            leagueOrgVo.setUnit("人");
            return leagueOrgVo;
        }
        // 保存团员总数
        leagueOrgVo.setLeagueOrgNum(allLeagueOrgCount+"");
        leagueOrgVo.setUnit("人");
        // 查询团员类型百分比
        List<LeagueProportionVo> leagueOrgProportionVos = leagueSituationDao.findLeagueMemberProportion();
        handleProportion(leagueOrgVo,leagueOrgProportionVos,allLeagueOrgCount);
        // 查询领域分布
        List<LeagueIndustryCountVo> leagueIndustryCountVos = leagueSituationDao.findLeagueMemberIndustry();
        leagueIndustryCountVos = handleLeagueIndustry(leagueIndustryCountVos);
        leagueOrgVo.setLeagueIndustries(leagueIndustryCountVos);
        return leagueOrgVo;
    }

    @Override
    public LeagueOrgVo findSituationLeader() {
        LeagueOrgVo leagueOrgVo = new LeagueOrgVo();
        // 查询团干部总数
        Integer allLeagueOrgCount = leagueSituationDao.findAllLeagueLeaderCount();
        if(allLeagueOrgCount.equals(0)){
            leagueOrgVo.setLeagueOrgNum("0");
            leagueOrgVo.setUnit("人");
            return leagueOrgVo;
        }
        // 保存团干部总数
        leagueOrgVo.setLeagueOrgNum(allLeagueOrgCount+"");
        leagueOrgVo.setUnit("人");
        // 查询团干部类型百分比
        List<LeagueProportionVo> leagueOrgProportionVos = leagueSituationDao.findLeagueLeaderProportion();
        handleProportion(leagueOrgVo,leagueOrgProportionVos,allLeagueOrgCount);
        // 查询团干部领域分布
        List<LeagueIndustryCountVo> leagueIndustryCountVos = leagueSituationDao.findLeagueLeaderIndustry();
        leagueIndustryCountVos = handleLeagueIndustry(leagueIndustryCountVos);
        leagueOrgVo.setLeagueIndustries(leagueIndustryCountVos);
        return leagueOrgVo;
    }

    /**
     * 更具类别选择
     */
    private LeagueOrgVo chooseSituation(String valueCode) {
        // 备注 4 为少年队，暂无数据
        switch (valueCode){
            case "1":
                LeagueOrgVo situationOrg = this.findSituationOrg();
                situationOrg.setTitle(LeagueNameEnum.getName(valueCode));
                return situationOrg;
            case "2":
                LeagueOrgVo situationMember = this.findSituationMember();
                situationMember.setTitle(LeagueNameEnum.getName(valueCode));
                return situationMember;
            case "3":
                LeagueOrgVo situationLeader = this.findSituationLeader();
                situationLeader.setTitle(LeagueNameEnum.getName(valueCode));
                return situationLeader;
            case "4":
                return null;
            default:
                return null;
        }
    }

    /**
     * 处理领域分类
     */
    private List<LeagueIndustryCountVo> handleLeagueIndustry(List<LeagueIndustryCountVo> leagueIndustryCountVos) {
        List<LeagueIndustryCountVo> list = new ArrayList<>();
        LeagueIndustryCountVo leagueIndustryCount = new LeagueIndustryCountVo();
        leagueIndustryCount.setIndustryName("学校");
        leagueIndustryCount.setIndustryNameCount("0");
        list.add(leagueIndustryCount);
        LeagueIndustryCountVo noPublicIndustryCount = new LeagueIndustryCountVo();
        noPublicIndustryCount.setIndustryName("非公企业");
        noPublicIndustryCount.setIndustryNameCount("0");
        list.add(noPublicIndustryCount);
        LeagueIndustryCountVo organIndustryCount = new LeagueIndustryCountVo();
        organIndustryCount.setIndustryName("机关事业单位");
        organIndustryCount.setIndustryNameCount("0");
        list.add(organIndustryCount);
        for (LeagueIndustryCountVo leagueIndustryCountVo : leagueIndustryCountVos) {
            String industryName = leagueIndustryCountVo.getIndustryName();
            // 学校
            if(ReportConstants.SCHOOL_INDUSTRY_NAME.contains(industryName)){
                int i = Integer.parseInt(leagueIndustryCountVo.getIndustryNameCount());
                leagueIndustryCount.setIndustryNameCount(Integer.parseInt(leagueIndustryCount.getIndustryNameCount())+i+"");
                continue;
            }
            // 非公企业
            if(ReportConstants.NO_PUBLIC_INDUSTRY_NAME.contains(industryName)){
                int i = Integer.parseInt(leagueIndustryCountVo.getIndustryNameCount());
                noPublicIndustryCount.setIndustryNameCount(Integer.parseInt(leagueIndustryCount.getIndustryNameCount())+i+"");
                continue;
            }
            // 机关事业单位
            if(ReportConstants.ORGAN_INDUSTRY_NAME.contains(industryName)){
                int i = Integer.parseInt(leagueIndustryCountVo.getIndustryNameCount());
                organIndustryCount.setIndustryNameCount(Integer.parseInt(leagueIndustryCount.getIndustryNameCount())+i+"");
                continue;
            }
            list.add(leagueIndustryCountVo);
        }
        return list;
    }


    /**
     * 处理百分比
     */
    private void handleProportion(LeagueOrgVo leagueOrgVo, List<LeagueProportionVo> leagueOrgProportionVos,Integer allLeagueOrgCount) {
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(1);
        df.setMinimumFractionDigits(1);
        for (LeagueProportionVo leagueOrgProportionVo : leagueOrgProportionVos) {
            String sysTypeProportion = df.format(leagueOrgProportionVo.getSysTypeCount() *100.00 / allLeagueOrgCount);
            leagueOrgProportionVo.setSysTypeProportion(sysTypeProportion);
        }
        leagueOrgVo.setLeagueOrgTypes(leagueOrgProportionVos);
    }

    /**
     * 处理总数数据
     */
    private void handleNum(LeagueOrgVo leagueOrgVo, Integer allLeagueOrgCount) {
        DecimalFormat df = new DecimalFormat("#.0");
        String str = df.format(allLeagueOrgCount/10000.0);
        leagueOrgVo.setLeagueOrgNum(str);
        leagueOrgVo.setUnit("万个");
    }
}
