package com.crunii.micro.service.report.service.impl;

import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.service.report.dto.MapEduVo;
import com.crunii.micro.service.report.jdbc.MapJdbc;
import com.crunii.micro.service.report.service.MapService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/7/14 14:39
 */
@Service
@Slf4j
public class MapServiceImpl implements MapService {

    @Resource
    private MapJdbc mapJdbc;

    @Resource
    private RedisTemplate<String,List<MapEduVo>> redisTemplate;

    @Override
    public List<MapEduVo> educationList() {
        List<MapEduVo> mapEduVos = redisTemplate.opsForValue().get(RedisKeys.DASHBOARD_MAP_KEY);
        if(CollectionUtils.isEmpty(mapEduVos)){
            mapEduVos = flushMapCash();
        }
        return mapEduVos;
    }

    @Override
    public List<MapEduVo> flushMapCash(){
        List<MapEduVo> mapEduVos  = mapJdbc.queryEduList();
        if(CollectionUtils.isEmpty(mapEduVos)){
            return new ArrayList<>();
        }
        Map<String, MapEduVo> collect = mapEduVos.stream().collect(Collectors.toMap(MapEduVo::getName, Function.identity(), (k1, k2) -> k1));
        List<MapEduVo> mapEduVo = mapJdbc.findLeagueCount();
        for (MapEduVo eduVo : mapEduVo) {
            MapEduVo mapEduVo1 = collect.get(eduVo.getName());
            eduVo.setInstitution(mapEduVo1.getInstitution());
            eduVo.setJuvenile(mapEduVo1.getJuvenile());
            eduVo.setEvents(mapEduVo1.getEvents());
            eduVo.setTeenHome(mapEduVo1.getTeenHome());
        }
        redisTemplate.opsForValue().set(RedisKeys.DASHBOARD_MAP_KEY,mapEduVo, 1,TimeUnit.DAYS);
        return mapEduVo;
    }
}
