package com.crunii.micro.service.report.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.entity.main.YcqCqzyscService;
import com.crunii.micro.domain.entity.main.YcqCqzyscServiceTime;
import com.crunii.micro.service.report.dto.ServiceTimeVo;
import com.crunii.micro.service.report.dto.ServiceVo;
import com.crunii.micro.service.report.dto.VolunteeringVo;
import com.crunii.micro.service.report.jdbc.OrgPowerJdbc;
import com.crunii.micro.service.report.service.OrgPowerService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/7/12 17:31
 */
@Service
@Slf4j
public class OrgPowerServiceImpl implements OrgPowerService {

    @Resource
    private OrgPowerJdbc orgPowerJdbc;

    @Override
    public Result<VolunteeringVo> queryVolunteering() {

        VolunteeringVo volunteeringVo = new VolunteeringVo();
        List<ServiceTimeVo> timeVoList = new ArrayList<>();
        List<String> month = new ArrayList<>();
        List<Integer> actNum = new ArrayList<>();
        List<Integer> applicantNum = new ArrayList<>();
        ServiceVo serviceVos = new ServiceVo();

        // 查询服务时间
        List<YcqCqzyscServiceTime> list = orgPowerJdbc.queryServiceTime();
        // 总服务时间
        double sumServiceTime = list.stream().mapToDouble(YcqCqzyscServiceTime::getServiceTime).sum();
        // 总人数
        int sumServiceNum = list.stream().mapToInt(YcqCqzyscServiceTime::getServiceNum).sum();
        // 获取排名前3数据
        list = list.stream().limit(3).collect(Collectors.toList());
        for (YcqCqzyscServiceTime time : list) {
            ServiceTimeVo vo = new ServiceTimeVo();
            vo.setOrgName(time.getAreaName());
            double timePercentage = div(time.getServiceTime(), sumServiceTime, 2);
            vo.setPercentage(String.valueOf((float) (timePercentage * 100)));
            timeVoList.add(vo);
        }
        // 查询志愿服务
        List<YcqCqzyscService> serviceList = orgPowerJdbc.queryService();
        // 获取最近6个月数据
        serviceList = serviceList.stream().skip(Math.max(0, serviceList.size() - 6)).collect(Collectors.toList());
        for (YcqCqzyscService service : serviceList) {
            // 转化月份
            month.add(convert(service.getMonth().substring(service.getMonth().length() - 2)));
            actNum.add(service.getEventsNum());
            applicantNum.add(service.getApplyNum());
        }
        serviceVos.setMonth(month);
        serviceVos.setActNum(actNum);
        serviceVos.setApplicantNum(applicantNum);
        // 构造响应实体
        String serviceNum = String.valueOf((float) div(sumServiceNum, 10000, 2));
        volunteeringVo.setTotalNum(serviceNum);
        volunteeringVo.setServiceTimeVo(timeVoList);
        volunteeringVo.setServiceVo(serviceVos);
        return Result.ofSuccess(volunteeringVo);
    }

    /**
     * 提供（相对）精确的除法运算。当发生除不尽的情况时，由scale参数指
     * 定精度，以后的数字四舍五入。
     *
     * @param v1    被除数
     * @param v2    除数
     * @param scale 表示表示需要精确到小数点以后几位。
     * @return 两个参数的商
     */
    public double div(double v1, double v2, int scale) {
        if (scale < 0) {
            throw new IllegalArgumentException(
                    "The scale must be a positive integer or zero");
        }
        BigDecimal b1 = BigDecimal.valueOf(v1);
        BigDecimal b2 = BigDecimal.valueOf(v2);
        return b1.divide(b2, scale, RoundingMode.HALF_UP).doubleValue();
    }

    /**
     * 月份转换 01转一月、02转二月
     *
     * @param input
     * @return
     */
    public static String convert(String input) {
        String[] months = input.split(",");
        StringBuilder result = new StringBuilder();
        String[] monthNames = new String[] {"一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"};
        for (String month : months) {
            int monthNumber = Integer.parseInt(month);
            String monthName = monthNames[monthNumber - 1];
            result.append(monthName).append(",");
        }
        result.deleteCharAt(result.length() - 1);
        return result.toString();
    }

}
