package com.crunii.micro.service.report.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.entity.main.YcqReportEducation;
import com.crunii.micro.service.report.dto.BudgetVo;
import com.crunii.micro.service.report.dto.EducationVo;
import com.crunii.micro.service.report.dto.SchoolEduVo;
import com.crunii.micro.service.report.jdbc.EducationJdbc;
import com.crunii.micro.service.report.service.ServicePowerService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/7/13 17:44
 */
@Service
@Slf4j
public class ServicePowerServiceImpl implements ServicePowerService {

    @Resource
    private EducationJdbc jdbc;

    @Override
    public Result<EducationVo> outSchoolEducation() {
        EducationVo educationVo = new EducationVo();
        // 五年投入经费
        BudgetVo budgetVos = new BudgetVo();
        List<String> years = new ArrayList<>();
        List<String> budgets = new ArrayList<>();

        // 查询校外教育
        List<YcqReportEducation> educations = jdbc.queryEducation();
        // 获取最近5年数据
        educations = educations.stream().skip(Math.max(0, educations.size() - 5)).collect(Collectors.toList());

        for (YcqReportEducation a : educations) {
            years.add(a.getYear());
            budgets.add(a.getFunds());
        }
        budgetVos.setYear(years);
        budgetVos.setBudget(budgets);

        // 获取最新一年数据
        YcqReportEducation b = educations.get(educations.size() - 1);
        // 校外教育
        List<SchoolEduVo> eduVos1 = jdbc.querySchoolEdu(b.getYear());
        // 师生比
        List<SchoolEduVo> eduVos2 = jdbc.queryTsRatio(b.getYear());

        //构造响应实体
        educationVo.setBudgetVos(budgetVos);
        educationVo.setSchoolEduVos(eduVos1);
        educationVo.setTsRatioVos(eduVos2);

        return Result.ofSuccess(educationVo);
    }
}
