package com.crunii.micro.service.report.service.impl;

import com.crunii.micro.common.excel.ExcelUtil;
import com.crunii.micro.service.report.dto.TestDto;
import com.crunii.micro.service.report.jdbc.TestDao;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author wangchen create 2023/7/12 18:19
 */
@Service
@Slf4j
public class TestServiceImp {

    private final TestDao testDao;

    public TestServiceImp(TestDao testDao) {
        this.testDao = testDao;
    }

    public String updateLeague(MultipartFile file,Integer num) {
        TestDto testDto = new TestDto();
        List<TestDto> classes = null;
        try {
            classes = ExcelUtil.readSingleExcel(file,testDto, num-1);
        } catch (Exception e) {
            e.printStackTrace();
        }
        for (TestDto aClass : classes) {
            log.info("开始同步");
            if(!StringUtils.hasText(aClass.getType()) || !StringUtils.hasText(aClass.getFullName())){
                log.info("跳过此行数-----------行数：{}",num);
                continue;
            }
            List<String> leagueCodes;
            // 查询是否有组织
            if(StringUtils.hasText(aClass.getName())){
                leagueCodes = testDao.findLeagueOrg(aClass.getName());
                if(leagueCodes.size()>1){
                    leagueCodes = testDao.findFullLeagueOrg(aClass.getFullName());
                }
            }else {
                leagueCodes = testDao.findFullLeagueOrg(aClass.getFullName());
            }

            if(leagueCodes.size() != 1){
                log.error("不能识别组织，组织名称：{},组织全称:{}",aClass.getName(),aClass.getFullName());
            }
            String leagueCode = leagueCodes.get(0);
            if(!StringUtils.hasText(leagueCode)){
                log.error("未查询到组织编码，组织名称：{},组织全称:{}",aClass.getName(),aClass.getFullName());
            }
            // 查询组织及其以下组织code
            List<String> leagueCodeIds = testDao.findLeagueCode(leagueCode);
            // 更新
            testDao.updateType(leagueCodeIds,aClass.getType());
            log.info("同步完成组织，----------->名称：{},组织全称:{}",aClass.getName(),aClass.getFullName());
        }
        log.info("--------------数据同步完成-----------------------");
        return "保存成功";
    }
}
