package com.crunii.micro.service.report.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.format.DatePrinter;
import cn.hutool.core.io.unit.DataUnit;
import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.service.report.dto.WebsiteGatherVo;
import com.crunii.micro.service.report.jdbc.WebsiteGatherDao;
import com.crunii.micro.service.report.service.WebsiteGatherService;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @author wangchen create 2023/7/31 15:54
 */
@Service
public class WebsiteGatherServiceImpl implements WebsiteGatherService {

    @Resource
    private RedisTemplate<String,WebsiteGatherVo> redisTemplate;

    private final WebsiteGatherDao websiteGatherDao;

    public WebsiteGatherServiceImpl(WebsiteGatherDao websiteGatherDao) {
        this.websiteGatherDao = websiteGatherDao;
    }

    @Override
    public WebsiteGatherVo getWebsiteGather() {
        WebsiteGatherVo websiteGatherVo = redisTemplate.opsForValue().get(RedisKeys.WEBSITE_GATHER_KEY);
        if(Objects.isNull(websiteGatherVo)){
            websiteGatherVo =  flushWebsiteGather();
        }
        return websiteGatherVo;
    }

    @Override
    public WebsiteGatherVo flushWebsiteGather() {
        Date date = new Date();
        String yesterday = DateUtil.format(date, "yyyy-MM-dd");
        DateTime lastMonthDay = DateUtil.offsetMonth(date, -1);
        WebsiteGatherVo websiteGatherVo = websiteGatherDao.findRegisterAccessActiveCount(yesterday,lastMonthDay);
        String pageId = websiteGatherDao.findHotAccessPageId();
        String pageName = websiteGatherDao.findPageNameByMenuCode(pageId);
        if(!StringUtils.hasText(pageName)){
            pageName = websiteGatherDao.findPageNameByAppCode(pageId);
        }
        websiteGatherVo.setHotAccess(pageName);
        redisTemplate.opsForValue().set(RedisKeys.WEBSITE_GATHER_KEY,websiteGatherVo,1, TimeUnit.DAYS);
        return websiteGatherVo;
    }
}
