package com.crunii.micro.service.report.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.entity.main.YcqReportYouthSituation;
import com.crunii.micro.service.report.dto.DegreeVo;
import com.crunii.micro.service.report.dto.RatioVo;
import com.crunii.micro.service.report.dto.YouthPopulationVo;
import com.crunii.micro.service.report.dto.YouthSituationVo;
import com.crunii.micro.service.report.jdbc.YouthSituationJdbc;
import com.crunii.micro.service.report.service.YouthSituationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/7/13 9:52
 */
@Service
@Slf4j
public class YouthSituationServiceImpl implements YouthSituationService {

    @Resource
    private YouthSituationJdbc jdbc;

    @Override
    public Result<YouthSituationVo> queryYouthSituation() {
        YouthSituationVo situationVo = new YouthSituationVo();
        YouthPopulationVo vo = new YouthPopulationVo();
        RatioVo ratioVo = new RatioVo();
        List<String> years = new ArrayList<>();
        List<String> population = new ArrayList<>();

        List<YcqReportYouthSituation> situationList = jdbc.queryYouthSituation();
        // 展示最近五年数据
        situationList = situationList.stream().skip(Math.max(0, situationList.size() - 5)).collect(Collectors.toList());
        for (YcqReportYouthSituation s : situationList) {
            years.add(s.getYear());
            population.add(s.getPeopleNum());
        }
        // 青年人口变化
        vo.setYears(years);
        vo.setPopulation(population);
        situationVo.setYouthPopulationVo(vo);

        // 男女比例
        YcqReportYouthSituation b = situationList.get(situationList.size() - 1);
        ratioVo.setYear(b.getYear());
        ratioVo.setManNum(b.getMenNum());
        ratioVo.setWomanNum(b.getWomenNum());
        situationVo.setRatioVo(ratioVo);

        // 学历组成
        List<DegreeVo> degreeList = jdbc.queryDegree(b.getYear());
        situationVo.setDegreeVo(degreeList);

        return Result.ofSuccess(situationVo);
    }
}
