package com.crunii.micro.service.sms.util;

import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;

/**
 * @author ql
 * @date 2022/6/1 17:23
 * @version 1.0
 */
@Component
public class RedisUtils {

    @Resource
    private RedisTemplate<String, Object> redisTemplate;

    /**
     * 写入缓存，永不过期
     */
    public boolean set(String key, Object value) {
        boolean flag;
        try {
            redisTemplate.opsForValue().set(key, value);
            flag = true;
        } catch (Exception ex) {
            flag = false;
        }
        return flag;
    }

    /**
     * 设置带过期时间的缓存
     */
    public boolean set(String key, Object value, long expire) {
        boolean flag;
        try {
            if (expire > 0) {
                redisTemplate.opsForValue().set(key, value, expire, TimeUnit.MINUTES);
            } else {
                this.set(key, value);
            }
            flag = true;
        } catch (Exception ex) {
            flag = false;
        }
        return flag;
    }

    public boolean put(String key, String value, long expire) {
        boolean flag;
        try {
            if (expire > 0) {
                redisTemplate.opsForValue().set(key, value, expire, TimeUnit.SECONDS);
            } else {
                this.set(key, value);
            }
            flag = true;
        } catch (Exception ex) {
            flag = false;
        }
        return flag;
    }

    /**
     * 取值方法
     */
    public Object getValue(String key) {
        return redisTemplate.opsForValue().get(key);
    }

    /**
     * 删除方法
     */
    public boolean del(String key) {
        return redisTemplate.delete(key);

    }

    /**
     * 判断某个key是否存在
     */
    public boolean exist(String key) {
        Boolean flag = false;
        try {
            flag = redisTemplate.hasKey(key);

            return flag;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return flag;
    }


    /**
     * 从redis中获取key对应的过期时间;
     * 如果该值有过期时间，就返回相应的过期时间;
     * 如果该值没有设置过期时间，就返回-1;
     * 如果没有该key，就返回-2;
     */
    public long expire(String key) {
        return redisTemplate.opsForValue().getOperations().getExpire(key);
    }


}
