/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.sso.config;

import com.crunii.micro.common.utils.JsonUtil;
import lombok.Getter;
import lombok.Setter;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author shenhs create 2023/6/2 17:28
 */
@Getter
@Setter
@Component
@ConfigurationProperties(prefix = "sso.sm2")
public class Sm2SecretKeyProperties implements InitializingBean {

	List<Sm2SecretProperty> secrets = new ArrayList<>();

	Map<String,Sm2SecretProperty> sm2SecretPropertyMap = new HashMap();

	@Override public void afterPropertiesSet() throws Exception {
		sm2SecretPropertyMap = secrets.stream().collect(Collectors.toMap(Sm2SecretProperty::getSysCode, Function.identity()));
		System.out.println(JsonUtil.getJsonString(sm2SecretPropertyMap));
	}

	/**
	 * 是否包含这个系统编码
	 * @param sysCode
	 * @return
	 */
	public boolean containsSysCode(String sysCode){
		return sm2SecretPropertyMap.containsKey(sysCode);
	}

	/**
	 * 根据系统编码获取相关密钥信息
	 * @param sysCode
	 * @return
	 */
	public Sm2SecretProperty get(String sysCode){
		return sm2SecretPropertyMap.get(sysCode);
	}

}
