package com.crunii.micro.service.sso.service.impl;

import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.sso.SSOConstant;
import com.crunii.micro.service.sso.config.HeadlineLoginInfoProperties;
import com.crunii.micro.service.sso.config.Sm2SecretKeyProperties;
import com.crunii.micro.service.sso.config.Sm2SecretProperty;
import com.crunii.micro.service.sso.constants.SsoConstants;
import com.crunii.micro.service.sso.dto.*;
import com.crunii.micro.service.sso.jdbc.OrgJdbc;
import com.crunii.micro.service.sso.service.GetService;
import com.crunii.micro.service.sso.service.ParamCheckService;
import com.crunii.micro.service.sso.utils.Sm2Util;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/7/4 9:38
 */
@Service
@Slf4j
public class GetServiceImpl implements GetService {

    @Resource
    private OrgJdbc orgJdbc;

    @Autowired
    private Sm2SecretKeyProperties properties;

    @Autowired
    private ParamCheckService paramCheckService;

    @Autowired
    private HeadlineLoginInfoProperties headlineLoginInfoProperties;


    @Override
    public Result<Object> getOrgInfo(OrgDto dto) {
        log.info("获取组织信息请求参数,{}", dto);
        Sm2SecretProperty sm2SecretProperty = properties.get(dto.getSysCode());
        String jsonStr = "";
        try {
            jsonStr = Sm2Util.decrypt(sm2SecretProperty.getPrivateKey(), sm2SecretProperty.isHex(), dto.getEncode());
        } catch (Exception e) {
            log.error("解密失败", e);
            return paramCheckService.getResult(SSOConstant.DECRYPT_ERROR);
        }
        log.info("时间戳:{}", jsonStr);

        List<OrgVo> orgVoList = orgJdbc.getOrgInfo();
        return Result.ofSuccess(orgVoList);
    }

    @Override
    public Result<String> getHeadline() {
        return Result.ofSuccess(handleUserLoginWay());
    }


    /**
     * 处理账号密码登录方式
     *  返回登陆后url连接
     */
    private String handleUserLoginWay() {
        HeadlineLoginInfo loginInfo = headlineLoginInfoProperties.getLoginInfo();
        if(Objects.isNull(loginInfo)){
            throw new BusinessException("未找到对应配置信息");
        }
        LoginReq loginReq = LoginReq.builder().username(loginInfo.getUsername()).password(loginInfo.getPassword()).build();
        String resultJson;
        try {
            resultJson = HttpUtil.post(loginInfo.getLoginUrl(), JSONUtil.toJsonStr(loginReq), SsoConstants.LOGIN_TIMEOUT);
        } catch (Exception e) {
            throw new BusinessException("登录接口调用失败");
        }
        JSONObject jsonObject = JSONUtil.parseObj(resultJson);
        if(!SsoConstants.LOGIN_SUCCESS_CODE.equals(jsonObject.get(SsoConstants.LOGIN_CODE))){
            throw new BusinessException("接口调用失败，返回信息===>{}",jsonObject.get(SsoConstants.LOGIN_MESSAGE));
        }
        Object o = jsonObject.get(SsoConstants.LOGIN_DATA);
        LoginLocationVo loginLocationVo = JSONUtil.toBean(o.toString(), LoginLocationVo.class);
        return loginLocationVo.getLoginLocationHref();
    }
}
