/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.sso.service.impl;

import cn.hutool.core.util.StrUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.sso.SSOConstant;
import com.crunii.micro.service.sso.config.Sm2SecretKeyProperties;
import com.crunii.micro.service.sso.service.ParamCheckService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @author shenhs create 2023/6/2 19:20
 */
@Service
public class ParamCheckServiceImpl implements ParamCheckService {

	@Autowired
	private Sm2SecretKeyProperties properties;

	private static Map<String,Result> ERROR_RESULT_MAP = new HashMap<>();

	static {
		ERROR_RESULT_MAP.put(SSOConstant.DECRYPT_ERROR, Result.ofFail(SSOConstant.DECRYPT_ERROR,"解密失败,请检查sysCode与publicKey是否匹配"));
		ERROR_RESULT_MAP.put(SSOConstant.IS_BLANK_AUTH_CODE, Result.ofFail(SSOConstant.IS_BLANK_AUTH_CODE,"缺少参数authCode"));
		ERROR_RESULT_MAP.put(SSOConstant.IS_BLANK_SYS_CODE, Result.ofFail(SSOConstant.IS_BLANK_SYS_CODE,"缺少参数sysCode"));
		ERROR_RESULT_MAP.put(SSOConstant.ERROR_AUTH_CODE, Result.ofFail(SSOConstant.ERROR_AUTH_CODE,"authCode无效或过期"));
		ERROR_RESULT_MAP.put(SSOConstant.IS_BLANK_ENCODE, Result.ofFail(SSOConstant.IS_BLANK_ENCODE,"缺少参数encode"));
		ERROR_RESULT_MAP.put(SSOConstant.ERROR_SYS_CODE, Result.ofFail(SSOConstant.ERROR_SYS_CODE,"sysCode不正确"));
		ERROR_RESULT_MAP.put(SSOConstant.ERROR_TOKEN, Result.ofFail(SSOConstant.ERROR_TOKEN,"token无效或过期"));
		ERROR_RESULT_MAP.put(SSOConstant.ERROR_SMS_CODE, Result.ofFail(SSOConstant.ERROR_SMS_CODE,"验证码错误"));
		ERROR_RESULT_MAP.put(SSOConstant.ERROR_USER_OR_PASSWD, Result.ofFail(SSOConstant.ERROR_USER_OR_PASSWD,"用户名密码错误"));
		ERROR_RESULT_MAP.put(SSOConstant.ERROR_CAPTCHA, Result.ofFail(SSOConstant.ERROR_CAPTCHA,"图片验证码错误"));


	}

	public Result<Object> getResult(String errorCode){
		return ERROR_RESULT_MAP.get(errorCode);
	}

	@Override public Result<Object> checkAuthCodeParam(String sysCode) {
		if(StrUtil.isBlank(sysCode)){
			return ERROR_RESULT_MAP.get(SSOConstant.IS_BLANK_AUTH_CODE);
		}
		if(!properties.containsSysCode(sysCode)){
			return ERROR_RESULT_MAP.get(SSOConstant.ERROR_SYS_CODE);
		}
		return null;
	}
}
