package com.crunii.micro.service.sso.service.impl;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.domain.entity.main.YcqStudyPlacard;
import com.crunii.micro.domain.repo.main.YcqStudyPlacardRepo;
import com.crunii.micro.service.sso.SSOConstant;
import com.crunii.micro.service.sso.config.Sm2SecretKeyProperties;
import com.crunii.micro.service.sso.config.Sm2SecretProperty;
import com.crunii.micro.service.sso.dto.HotPlacardVo;
import com.crunii.micro.service.sso.dto.PushDto;
import com.crunii.micro.service.sso.jdbc.PushJdbc;
import com.crunii.micro.service.sso.service.ParamCheckService;
import com.crunii.micro.service.sso.service.PushService;
import com.crunii.micro.service.sso.utils.Sm2Util;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/25 17:05
 */
@Service
@Slf4j
public class PushServiceImpl implements PushService {

    @Autowired
    private Sm2SecretKeyProperties properties;

    @Autowired
    private ParamCheckService paramCheckService;

    @Resource
    private YcqStudyPlacardRepo placardRepo;

    @Resource
    private PushJdbc pushJdbc;

    @Override
    public Result<Object> saveHotPlacard(PushDto pushDto) {
        log.info("saveHotPlacard {} {} {}", pushDto.getSysCode(), pushDto.getTransactionId(), pushDto.getSysCode());
        String encode = pushDto.getEncode();
        Sm2SecretProperty sm2SecretProperty = properties.get(pushDto.getSysCode());
        String jsonStr = "";
        List<YcqStudyPlacard> list = new ArrayList<>();
        try {
            jsonStr = Sm2Util.decrypt(sm2SecretProperty.getPrivateKey(),sm2SecretProperty.isHex(), encode);
        } catch (Exception e) {
            log.error("解密失败", e);
            return paramCheckService.getResult(SSOConstant.DECRYPT_ERROR);
        }
        List<HotPlacardVo> voList = JsonUtil.jsonToList(jsonStr, HotPlacardVo.class);
        if (CollectionUtils.isEmpty(voList)) {
            return Result.ofFail("推送数据不能为空");
        }
        // 推送前先删除系统编码下的数据
        pushJdbc.deletePlacard(pushDto.getSysCode());
        // 新增
        voList.forEach(x -> {
            YcqStudyPlacard placard = new YcqStudyPlacard();
            placard.setCreateTime(new Date());
            placard.setSysCode(pushDto.getSysCode());
            BeanUtils.copyProperties(x, placard);
            list.add(placard);
        });

        //// 判断图片id是否重复
        //boolean isNameDuplicate = voList.stream()
        //        .map(HotPlacardVo::getPlacardId)
        //        .distinct()
        //        .count() != voList.size();
        //if (isNameDuplicate) {
        //    return Result.ofFail("存在重复的图片id");
        //} else {
        //    voList.forEach(x -> {
        //        YcqStudyPlacard repo = placardRepo.findByPlacardId(x.getPlacardId());
        //        if (ObjectUtil.isNotEmpty(repo)) {
        //            BeanUtils.copyProperties(x, repo);
        //            list.add(repo);
        //        } else {
        //            YcqStudyPlacard placard = new YcqStudyPlacard();
        //            placard.setSysCode(pushDto.getSysCode());
        //            BeanUtils.copyProperties(x, placard);
        //            list.add(placard);
        //        }
        //    });
        //}
        placardRepo.saveAllAndFlush(list);
        return Result.ofSuccess("推送成功");
    }
}
