/**
 * Copyright 2008-2009. Chongqing Communications Industry Services Co.,Ltd Information Technology Branch. All rights reserved.
 * <a>http://www.crunii.com</a>
 */
package com.crunii.micro.service.sso.service.impl;

import cn.hutool.crypto.SmUtil;
import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.common.utils.NanoIdUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.sso.config.SSOCacheProperty;
import com.crunii.micro.service.sso.dto.LoginVo;
import com.crunii.micro.service.sso.dto.SSOWorkerInfo;
import com.crunii.micro.service.sso.service.SSOCacheService;
import org.redisson.api.RMapCache;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;

/**
 * @author shenhs create 2023/6/2 20:07
 */
@Service
public class SSOCacheServiceImpl implements SSOCacheService, InitializingBean {

    String env = System.getProperty("spring.profiles.active");

    @Autowired
    private RedissonClient redissonClient;

    @Autowired
    private SSOCacheProperty cacheProperty;

    private RMapCache<String, String> authCodeMap = null;
    private RMapCache<String, SSOWorkerInfo> tokenInfoMap = null;

    @Override
    public void afterPropertiesSet() throws Exception {
        if (authCodeMap == null) {
            authCodeMap = redissonClient.getMapCache(RedisKeys.SSO_AUTH_CODE_CACHE_KEY);
        }
        if (tokenInfoMap == null) {
            tokenInfoMap = redissonClient.getMapCache(RedisKeys.SSO_TOKEN_CACHE_KEY);
        }
//		redissonClient.getScoredSortedSet();
//		RSortedSet<ZsetTest> sortedSet = redissonClient.getSortedSet("ZSET_TEST");
//		ZsetTest zsetTest4 = new ZsetTest("张wq",3);
//		ZsetTest zsetTest = new ZsetTest("张三",1);
//		ZsetTest zsetTest2 = new ZsetTest("张er",2);
//		ZsetTest zsetTest3 = new ZsetTest("张wb",6);
//		ZsetTest zsetTest5 = new ZsetTest("张wa",9);
//		ZsetTest zsetTest6 = new ZsetTest("张wb",5);
//		sortedSet.add(zsetTest4);
//		sortedSet.add(zsetTest);
//		sortedSet.add(zsetTest2);
//		sortedSet.add(zsetTest3);
//		sortedSet.add(zsetTest5);
//		sortedSet.add(zsetTest6);

//		sortedSet.

    }

    public String put(String authCode, Worker worker) {
        String sm3Code = SmUtil.sm3(authCode);
        String token = NanoIdUtil.randomNanoId();
        authCodeMap.put(sm3Code, token, cacheProperty.getAuthCodeTimeout(), TimeUnit.SECONDS);
        SSOWorkerInfo ssoWorkerInfo = null;
        if (env.equals("dev")) {
            ssoWorkerInfo = new SSOWorkerInfo(112233, "zhangsan", "1761236781", "aholcqckop==", "测试组织");
        } else {
            int loginId = worker.getLoginId();
            String mobile = worker.getMobile();
            String workerName = worker.getWorkerName();
            ssoWorkerInfo = new SSOWorkerInfo(loginId, workerName, mobile, worker.getOrgId(), worker.getOrgName());
        }
        tokenInfoMap.put(token, ssoWorkerInfo, cacheProperty.getTokenTimeout(), TimeUnit.SECONDS);
        return token;
    }

    @Override
    public LoginVo loginPut(String authCode, Worker worker) {
        String sm3Code = SmUtil.sm3(authCode);
        String token = NanoIdUtil.randomNanoId();
        authCodeMap.put(sm3Code, token, cacheProperty.getAuthCodeTimeout(), TimeUnit.SECONDS);
        SSOWorkerInfo ssoWorkerInfo = null;
        if (env.equals("dev")) {
            ssoWorkerInfo = new SSOWorkerInfo(112233, "谌红霜", "17602346573", "", "");
        } else {
            int loginId = worker.getLoginId();
            String mobile = worker.getMobile();
            String workerName = worker.getWorkerName();
            ssoWorkerInfo = new SSOWorkerInfo(loginId, workerName, mobile, "", "");
        }
        tokenInfoMap.put(token, ssoWorkerInfo, cacheProperty.getTokenTimeout(), TimeUnit.SECONDS);
        LoginVo vo = new LoginVo();
        vo.setToken(token);
        vo.setExpires(cacheProperty.getTokenTimeout());
        return vo;
    }

    @Override
    public String getToken(String authCode) {
        String sm3Code = SmUtil.sm3(authCode);
        return authCodeMap.get(sm3Code);
    }

    @Override
    public SSOWorkerInfo getWorkerInfo(String token) {
        SSOWorkerInfo ssoWorkerInfo = tokenInfoMap.get(token);
        return ssoWorkerInfo;
    }

    public static void main(String[] args) {
//		System.out.println();
    }

}
