package com.crunii.micro.service.sso.service.impl;

import cn.hutool.json.JSONUtil;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.domain.entity.main.*;
import com.crunii.micro.domain.repo.main.*;
import com.crunii.micro.service.sso.SSOConstant;
import com.crunii.micro.service.sso.config.Sm2SecretKeyProperties;
import com.crunii.micro.service.sso.config.Sm2SecretProperty;
import com.crunii.micro.service.sso.dto.*;
import com.crunii.micro.service.sso.jdbc.PushJdbc;
import com.crunii.micro.service.sso.service.ParamCheckService;
import com.crunii.micro.service.sso.service.VolunteerPushService;
import com.crunii.micro.service.sso.utils.JsonSerilizable;
import com.crunii.micro.service.sso.utils.Sm2Util;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author qiaolin
 * @version 1.0
 * @date 2023/6/27 15:38
 */
@Service
@Slf4j
public class VolunteerPushServiceImpl implements VolunteerPushService {

    public static final String NEW_URL = "https://youth.cq.cqyl.org.cn:11021/qcttpic/approve/cover_img";
    @Autowired
    private Sm2SecretKeyProperties properties;

    @Autowired
    private ParamCheckService paramCheckService;

    @Resource
    private YcqCqzyscServiceRepo serviceRepo;

    @Resource
    private YcqCqzyscServiceTimeRepo serviceTimeRepo;

    @Resource
    private YcqHeadlineArticleRepo ycqHeadlineArticleRepo;

    @Resource
    private YcqHeadlineArticleColumnRepo ycqHeadlineArticleColumnRepo;

    @Resource
    private YcqYouthcallCampusEducationRepo ycqYouthcallCampusEducationRepo;

    @Resource
    private PushJdbc pushJdbc;


    @Override
    public Result<Object> saveService(PushDto pushDto) {
        log.info("saveService {} {} {}", pushDto.getSysCode(), pushDto.getTransactionId(), pushDto.getSysCode());
        String encode = pushDto.getEncode();
        Sm2SecretProperty sm2SecretProperty = properties.get(pushDto.getSysCode());
        String jsonStr = "";
        try {
            jsonStr = Sm2Util.decrypt(sm2SecretProperty.getPrivateKey(),sm2SecretProperty.isHex(), encode);
        } catch (Exception e) {
            log.error("解密失败", e);
            return paramCheckService.getResult(SSOConstant.DECRYPT_ERROR);
        }
        List<YcqCqzyscService> voList = JsonUtil.jsonToList(jsonStr, YcqCqzyscService.class);
        if (CollectionUtils.isEmpty(voList)) {
            return Result.ofFail("推送数据不能为空");
        }
        // 删除相同日期的数据
        pushJdbc.deleteService(voList.get(0).getDateTime());
        //保存
        serviceRepo.saveAllAndFlush(voList);
        return Result.ofSuccess("志愿服务数据推送成功");
    }

    @Override
    public Result<Object> saveServiceTime(PushDto pushDto) {
        log.info("saveServiceTime {} {} {}", pushDto.getSysCode(), pushDto.getTransactionId(), pushDto.getSysCode());
        String encode = pushDto.getEncode();
        Sm2SecretProperty sm2SecretProperty = properties.get(pushDto.getSysCode());
        String jsonStr = "";
        try {
            jsonStr = Sm2Util.decrypt(sm2SecretProperty.getPrivateKey(),sm2SecretProperty.isHex(), encode);
        } catch (Exception e) {
            log.error("解密失败", e);
            return paramCheckService.getResult(SSOConstant.DECRYPT_ERROR);
        }
        List<YcqCqzyscServiceTime> voList = JsonUtil.jsonToList(jsonStr, YcqCqzyscServiceTime.class);
        if (CollectionUtils.isEmpty(voList)) {
            return Result.ofFail("推送数据不能为空");
        }
        // 删除相同日期的数据
        pushJdbc.deleteServiceTime(voList.get(0).getDateTime());
        // 保存
        serviceTimeRepo.saveAllAndFlush(voList);
        return Result.ofSuccess("志愿服务数据推送成功");
    }

    @Override
    public Result<Object> pushYouthHeadlines(PushDto pushDto) {
        log.info("pushYouthHeadlines {} {},encode: {}", pushDto.getSysCode(), pushDto.getTransactionId(), pushDto.getEncode());
        String encode = pushDto.getEncode();
        Sm2SecretProperty sm2SecretProperty = properties.get(pushDto.getSysCode());
        String jsonStr = "";
        try {
            jsonStr = Sm2Util.decrypt(sm2SecretProperty.getPrivateKey(),sm2SecretProperty.isHex(), encode);
        } catch (Exception e) {
            log.error("解密失败", e);
            return paramCheckService.getResult(SSOConstant.DECRYPT_ERROR);
        }
        List<HeadlineArticleDto> voList = JsonUtil.jsonToList(jsonStr, HeadlineArticleDto.class);


        if (CollectionUtils.isEmpty(voList)) {
            return Result.ofFail("推送数据不能为空");
        }
        log.info("pushYouthHeadlines,parse json:[{}]", jsonStr);
        this.replaceUri(voList);
        List<String> collect = voList.stream().map(HeadlineArticleDto::getHeadlineArticleId).collect(Collectors.toList());
        // 查询封装已经推送的数据
        List<ArticleInfoVo> articleInfoVos = pushJdbc.findExistArticleIds(collect);
        Map<String, ArticleInfoVo> existArticleIds = new HashMap<>(16);
        if(! CollectionUtils.isEmpty(articleInfoVos)){
            // 删除二级栏目
            List<Long> collect1 = articleInfoVos.stream().map(ArticleInfoVo::getHeadlineId).collect(Collectors.toList());
            pushJdbc.deleteColumn(collect1);
            existArticleIds  = articleInfoVos.stream().collect(Collectors.toMap(ArticleInfoVo::getHeadlineArticleId, Function.identity()));
        }
        // 处理已存在在的id,则做更新处理
        for (HeadlineArticleDto headlineArticleDto : voList) {
            YcqHeadlineArticle ycqHeadlineArticle = new YcqHeadlineArticle();
//            ycqHeadlineArticle.setReleaseTime(new Date());
            BeanUtils.copyProperties(headlineArticleDto,ycqHeadlineArticle);
            List<YcqHeadlineArticleColumn> columnTwos = headlineArticleDto.getColumnTwos();
            ArticleInfoVo articleInfoVo = existArticleIds.get(ycqHeadlineArticle.getHeadlineArticleId());
            if(Objects.nonNull(articleInfoVo)){
                // 收藏 点赞
                ycqHeadlineArticle.setHeadlineId(articleInfoVo.getHeadlineId());
                ycqHeadlineArticle.setLikesCount(articleInfoVo.getLikesCount());
                ycqHeadlineArticle.setReadingQuantity(articleInfoVo.getReadingQuantity());
            }
            ycqHeadlineArticle.setState(1);
            // 保存
            YcqHeadlineArticle ycqHeadlineArticle1 = ycqHeadlineArticleRepo.saveAndFlush(ycqHeadlineArticle);
            columnTwos.forEach(x->{
                x.setHeadlineId(ycqHeadlineArticle1.getHeadlineId());
                x.setDeleted("0");
            });
            ycqHeadlineArticleColumnRepo.saveAllAndFlush(columnTwos);
        }

        return Result.ofSuccess("青春头条数据推送成功");
    }

    public void replaceUri(List<HeadlineArticleDto> source){
        source.forEach(item -> {
            String coverPic = item.getCoverPic();
            item.setResourcePic(coverPic);
            if (StringUtils.hasText(coverPic)){
                int i = coverPic.lastIndexOf("/");
                String substring = coverPic.substring(i);
                String newUrl = NEW_URL + substring;
                item.setCoverPic(newUrl);
            }
        });
    }

    @Override
    public Result<Object> headlinesTop(PushDto pushDto) {
        log.info("headlinesTop {} {} ,{}", pushDto.getSysCode(), pushDto.getTransactionId(), pushDto.getEncode());
        String encode = pushDto.getEncode();
        Sm2SecretProperty sm2SecretProperty = properties.get(pushDto.getSysCode());
        String jsonStr = "";
        try {
            jsonStr = Sm2Util.decrypt(sm2SecretProperty.getPrivateKey(),sm2SecretProperty.isHex(), encode);
        } catch (Exception e) {
            log.error("解密失败", e);
            return paramCheckService.getResult(SSOConstant.DECRYPT_ERROR);
        }
        log.info("headlinesTop info:[{}]", jsonStr);
        HeadlinesTopDto headlinesTopDto = JSONUtil.toBean(jsonStr, HeadlinesTopDto.class);
        if(Objects.isNull(headlinesTopDto)){
            return Result.ofFail("无数据！");
        }
        YcqHeadlineArticle byHeadlineArticleId = ycqHeadlineArticleRepo.findByHeadlineArticleId(headlinesTopDto.getHeadlineArticleId());
        if(Objects.isNull(byHeadlineArticleId)){
            return Result.ofFail("无此文章");
        }
        if(byHeadlineArticleId.getIsTop().equals(headlinesTopDto.getIsTop())){
            return Result.ofSuccess("青春头条置顶成功");
        }
        byHeadlineArticleId.setIsTop(headlinesTopDto.getIsTop());
        byHeadlineArticleId.setUpdateTime(new Date());
        ycqHeadlineArticleRepo.saveAndFlush(byHeadlineArticleId);
        return Result.ofSuccess("青春头条置顶成功");
    }

    @Override
    public Result<Object> offCampusEducation(PushDto pushDto) {
        log.info("offCampusEducation {} {} {}", pushDto.getSysCode(), pushDto.getTransactionId(), pushDto.getSysCode());
        String encode = pushDto.getEncode();
        Sm2SecretProperty sm2SecretProperty = properties.get(pushDto.getSysCode());
        String jsonStr = "";
        try {
            jsonStr = Sm2Util.decrypt(sm2SecretProperty.getPrivateKey(),sm2SecretProperty.isHex(), encode);
        } catch (Exception e) {
            log.error("解密失败", e);
            return paramCheckService.getResult(SSOConstant.DECRYPT_ERROR);
        }
        YcqYouthcallCampusEducationVo ycqYouthcallCampusEducationVo = JSONUtil.toBean(jsonStr, YcqYouthcallCampusEducationVo.class);
        if(Objects.isNull(ycqYouthcallCampusEducationVo)){
            return Result.ofFail("无数据！");
        }
        // 轮播图集合
        List<CarouselChart> carouselCharts = ycqYouthcallCampusEducationVo.getCarouselCharts();
        // 名师推荐
        List<RecommendByTeachers> recommendByTeachers = ycqYouthcallCampusEducationVo.getRecommendByTeachers();
        // 优秀学员
        List<ExcellentStudent> excellentStudents = ycqYouthcallCampusEducationVo.getExcellentStudents();
        // 兴趣课程
        String interestCoursesUrl = ycqYouthcallCampusEducationVo.getInterestCoursesUrl();
        // 实践活动
        String practicalActivity  = ycqYouthcallCampusEducationVo.getPracticalActivity();
        YcqYouthcallCampusEducation ycqYouthcallCampusEducation = new YcqYouthcallCampusEducation();
        try {
            ycqYouthcallCampusEducation.setInterestCoursesUrl(interestCoursesUrl);
            ycqYouthcallCampusEducation.setPracticalActivity(practicalActivity);
            ycqYouthcallCampusEducation.setCarouselChart(JsonSerilizable.serilizableForList(carouselCharts).getBytes());
            ycqYouthcallCampusEducation.setRecommendByTeachers(JsonSerilizable.serilizableForList(recommendByTeachers).getBytes());
            ycqYouthcallCampusEducation.setExcellentStudent(JsonSerilizable.serilizableForList(excellentStudents).getBytes());
            ycqYouthcallCampusEducation.setDelFlag("0");
            ycqYouthcallCampusEducation.setCreateTime(new Date());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        ycqYouthcallCampusEducationRepo.saveAndFlush(ycqYouthcallCampusEducation);
        return Result.ofSuccess("青呼我为-校外教育推送成功");
    }
}
