package com.crunii.micro.service.studycenter.common.base;

import org.springframework.data.domain.Example;
import org.springframework.data.jpa.convert.QueryByExamplePredicateBuilder;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.jpa.repository.query.EscapeCharacter;
import org.springframework.lang.NonNull;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.ArrayList;
import java.util.List;

/**
 * Author：dadm
 * Date：2023-09-21 10:26
 * Description：<描述>
 */
public class ExamplePredicate<T> implements Specification<T> {

    private final List<FieldRestriction<T>> fieldRestrictionList;
    private final Example<T> example;

    public ExamplePredicate(Example<T> example, List<FieldRestriction<T>> fieldRestrictionList) {
        this.fieldRestrictionList = fieldRestrictionList;
        this.example = example;
    }

    public static <T> ExamplePredicate<T> of (Example<T> example, List<FieldRestriction<T>> fieldRestrictionList) {
        return new ExamplePredicate<T>(example, fieldRestrictionList);
    }

    @Override
    public Predicate toPredicate(@NonNull Root<T> root, @NonNull CriteriaQuery<?> query, @NonNull CriteriaBuilder criteriaBuilder) {
        List<Predicate> extPredicates =  new ArrayList<>();
        // 先加进去原有的匹配条件
        Predicate predicate = QueryByExamplePredicateBuilder.getPredicate(root, criteriaBuilder, example, EscapeCharacter.DEFAULT);
        extPredicates.add(predicate);
        //　加入自定义的陪陪条件
        for (FieldRestriction<T> fieldRestriction : fieldRestrictionList) {
            extPredicates.addAll(fieldRestriction.addRestriction(root, criteriaBuilder));
        }
        return criteriaBuilder.and(extPredicates.toArray(new Predicate[0]));
    }


}
