package com.crunii.micro.service.studycenter.controller;

import cn.hutool.json.JSONUtil;
//import com.crunii.framework.api.encrypt.annotation.EncryptRequestBody;
import com.crunii.framework.api.encrypt.annotation.EncryptRequestBody;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.studycenter.dto.studycenter.*;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsCourseMgrVo;
import com.crunii.micro.service.studycenter.service.YcqYjsCourseMgrService;
import com.crunii.micro.service.studycenter.service.YcqYjsSpecialMgrService;
import com.crunii.micro.service.studycenter.service.impl.YcqYjsServiceDurationServiceImpl;
import com.crunii.micro.service.studycenter.util.CharacterCodeUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigInteger;
import java.util.List;

//import com.crunii.framework.api.encrypt.annotation.EncryptRequestBody;

/**
 * @author : dinglian
 * @date : 2023-9-18
 */
@Slf4j
@Api(tags = "课程管理")
@RestController
@RequestMapping("/ycqYjsCourseMgr/auth")
public class YcqYjsCourseMgrController {

    @Resource
    private YcqYjsCourseMgrService ycqYjsCourseMgrServiceImpl;

    @Resource
    private YcqYjsSpecialMgrService ycqYjsSpecialMgrServiceImpl;

    @Resource
    private YcqYjsServiceDurationServiceImpl YcqYjsServiceDurationServiceImpl;

    @ApiOperation(value = "查询所有课程管理")
    @RequestMapping(path = "paginQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<YcqYjsCourseMgrRspPage>> paginQuery(@RequestBody PageReq<YcqYjsCourseMgrVo> req) {
        try {
            return ycqYjsCourseMgrServiceImpl.paginQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.paginQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.paginQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新增课程管理")
    @RequestMapping(path = "create", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> create(@Valid @RequestBody YcqYjsCourseMgrVo ycqYjsCourseMgrvo) {
        try {
            return ycqYjsCourseMgrServiceImpl.insert(ycqYjsCourseMgrvo);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "课程名称校验")
    @RequestMapping(path = "courseNameCheck", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> courseNameCheck(@RequestBody YcqYjsCourseMgr ycqYjsCourseMgr) {
        try {
            return ycqYjsCourseMgrServiceImpl.courseNameCheck(ycqYjsCourseMgr);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.update.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.update.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改课程管理")
    @RequestMapping(path = "update", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> update(@Valid @RequestBody YcqYjsCourseMgrVo ycqYjsCourseMgrVo) {
        try {
            return ycqYjsCourseMgrServiceImpl.update(ycqYjsCourseMgrVo);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.update.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.update.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除课程管理")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/del", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> del(BigInteger id) {
        try {
            return ycqYjsCourseMgrServiceImpl.deleteById(id);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.deleteYcqYjsCourseTestRel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.deleteYcqYjsCourseTestRel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "更改课程状态")
    @RequestMapping(path = "updateCourseStatus", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateCourseStatus(@Valid @RequestBody YcqYjsCourseMgr ycqYjsCourseMgr) {
        try {
            return ycqYjsCourseMgrServiceImpl.updateCourseStatus(ycqYjsCourseMgr);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.update.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.update.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "根据id查询课程管理")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/queryById", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYjsCourseMgrResp> queryById(@RequestParam("id")BigInteger id) {
        try {
            return ycqYjsCourseMgrServiceImpl.queryById(id);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.queryById.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.queryById.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "专题下拉列表")
    @RequestMapping(path = "querySpecialInfoList", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> querySpecialInfoList() {
        try {
            List<YcqYjsSpecialMgr> list = ycqYjsSpecialMgrServiceImpl.querySpecialInfoList();
            return Result.ofSuccess(list);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除课程管理关联试卷、题目关系")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/deleteYcqYjsCourseTestRel", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> deleteYcqYjsCourseTestRel(@Valid @RequestBody YcqYjsCourseTestRel ycqYjsCourseTestRel) {
        try {
            return ycqYjsCourseMgrServiceImpl.deleteYcqYjsCourseTestRel(ycqYjsCourseTestRel);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.deleteYcqYjsCourseTestRel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.deleteYcqYjsCourseTestRel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据试卷id查询关联的课程信息")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/queryYcqYjsCourseByTestId", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<YcqYjsCourseMgr>> queryYcqYjsCourseByTestId(@Valid @RequestBody YcqYjsCourseTestRel ycqYjsCourseTestRel) {
        try {
            return ycqYjsCourseMgrServiceImpl.queryYcqYjsCourseByTestId(ycqYjsCourseTestRel);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.deleteYcqYjsCourseTestRel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.deleteYcqYjsCourseTestRel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "上传课程视频")
    @RequestMapping(path = "/uploadCourseVideo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> uploadCourseVideo(@RequestParam(value="file") MultipartFile file,
                                            @RequestParam("fileNumber") String fileNumber,
                                            @RequestParam("tableName") String tableName) {
        try {
            return  ycqYjsCourseMgrServiceImpl.uploadCourseVideo(file, fileNumber, tableName);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.uploadCourseVideo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.uploadCourseVideo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "通知流媒体转码")
    @RequestMapping(path = "/transcodingVideo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> transcodingVideo(@RequestParam("userName") String userName,
                                           @RequestParam("uploadDate") String uploadDate,
                                           @RequestParam("fileName") String fileName) {
        try {
            return  ycqYjsCourseMgrServiceImpl.transcodingVideo(userName, uploadDate, fileName);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.uploadCourseVideo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.uploadCourseVideo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "获取视频文件下载地址")
    @RequestMapping(path = "/downloadCourseVideo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> downloadCourseVideo(@RequestParam("fileNumber") String fileNumber,
                                            @RequestParam("tableName") String tableName) {
        try {
            return  ycqYjsCourseMgrServiceImpl.downloadCourseVideo(fileNumber, tableName);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.downloadCourseVideo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.downloadCourseVideo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "获取视频实时播放地址")
    @RequestMapping(path = "/queryRealTimeVideoAddress", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> queryRealTimeVideoAddress(@RequestBody YcqYjsUploadVideo YcqYjsUploadVideo) {
        try {
            return  ycqYjsCourseMgrServiceImpl.queryRealTimeVideoAddress(YcqYjsUploadVideo.getFileNumber(), YcqYjsUploadVideo.getTableName());
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.downloadCourseVideo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.downloadCourseVideo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "生成编号")
    @RequestMapping(path = "generateNumber", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> generateNumber(String code) {
        try {
            String generateCode = CharacterCodeUtil.generateCharacterCode(code);
            return Result.ofSuccess(generateCode);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.generateNumber.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.generateNumber.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @PostMapping("/api-encrypt")
    public Result<String> encrypt(@EncryptRequestBody YcqYjsCourseMgr ycqYjsCourseMgr) {
        return Result.ofSuccess(JSONUtil.toJsonStr(ycqYjsCourseMgr));
    };

    @PostMapping("/batchInsertExcel")
    public Result<String> batchInsertExcel(YcqYjsCourseMgr ycqYjsCourseMgr) {
        YcqYjsServiceDurationServiceImpl.batchInsert();
        return Result.ofSuccess();
    };
}
