package com.crunii.micro.service.studycenter.controller;

import com.crunii.framework.api.encrypt.annotation.EncryptRequestBody;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.studycenter.annotation.Desensitization;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsNegativePerson;
import com.crunii.micro.service.studycenter.dto.vo.NegativePersonCondition;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsNegativePersonVo;
import com.crunii.micro.service.studycenter.service.YcqYjsNegativePersonService;
import dm.jdbc.filter.stat.util.JSONUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;

/**
 * Author：Charler
 * Date：2023-09-20 17:21
 * Description：<描述>
 */
@Slf4j
@Api(tags = "负面人员清单api接口")
@RestController
@RequestMapping("/YcqYjsNegativePerson/auth")
public class YcqYjsNegativePersonController {

    @Autowired
    private YcqYjsNegativePersonService negativePersonService;

    @ApiOperation(value = "新建负面人员")
    @RequestMapping(path = "/encryptionRequired/createNegativePerson", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createNegativePerson(@Validated @EncryptRequestBody @ApiParam YcqYjsNegativePersonVo createReq) {
        log.info("YcqYjsNegativePersonController.createNegativePerson.param：{}", createReq.toString());
        try {
            negativePersonService.createNegativePerson(createReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsNegativePersonController.createNegativePerson.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsNegativePersonController.createNegativePerson.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除负面人员")
    @RequestMapping(path = "/delNegativePerson", method = RequestMethod.GET)
    @ResponseBody
    public Result<Boolean> delNegativePerson(@RequestParam("id") BigInteger id) {
        log.info("YcqYjsNegativePersonController.delNegativePerson.param：{}", id);
        try {
            negativePersonService.delNegativePerson(id);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsNegativePersonController.delNegativePerson.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsNegativePersonController.delNegativePerson.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改负面人员信息")
    @RequestMapping(path = "/encryptionRequired/updateNegativePerson", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateNegativePerson(@Validated @EncryptRequestBody YcqYjsNegativePersonVo updateReq) {
        log.info("YcqYjsNegativePersonController.updateNegativePerson.param：{}", updateReq.toString());
        try {
            negativePersonService.updateNegativePerson(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsNegativePersonController.updateNegativePerson.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsNegativePersonController.updateNegativePerson.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询负面人员信息")
    @RequestMapping(path = "/getNegativePerson", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYjsNegativePerson> getNegativePerson(@RequestParam("id") BigInteger id) {
        log.info("YcqYjsNegativePersonController.getNegativePerson.param：{}", id);
        try {
            YcqYjsNegativePerson negativePerson = negativePersonService.getNegativePersonInfo(id);
            return Result.ofSuccess(negativePerson);
        } catch (BusinessException e) {
            log.error("YcqYjsNegativePersonController.getNegativePerson.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsNegativePersonController.getNegativePerson.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "分页查询负面人员")
    @RequestMapping(path = "/pageSearchNegativePerson", method = RequestMethod.POST)
    @Desensitization
    @ResponseBody
    public Result<PageRsp<YcqYjsNegativePersonVo>> pageSearchNegativePerson(@RequestBody PageReq<NegativePersonCondition> req) {
        log.info("YcqYjsNegativePersonController.pageSearchNegativePerson.param：{}", req.toString());
        try {
            return Result.ofSuccess(negativePersonService.pageSearchNegativePerson(req));
        } catch (BusinessException e) {
            log.error("YcqYjsNegativePersonController.pageSearchNegativePerson.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsNegativePersonController.pageSearchNegativePerson.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation("负面人员清单excel导出")
    @RequestMapping(value = "/exportNegativePersonExcel", method = {RequestMethod.POST})
    @ResponseBody
    public Result<Boolean> exportNegativePersonExcel(HttpServletResponse response, @RequestBody NegativePersonCondition req) {
        log.info("YcqYjsNegativePersonController.exportNegativePersonExcel.param：{}", req.toString());
        try {
            negativePersonService.exportNegativePersonExcel(response,req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsNegativePersonController.exportNegativePersonExcel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsNegativePersonController.exportNegativePersonExcel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }
}


