package com.crunii.micro.service.studycenter.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.studycenter.dto.studycenter.*;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsCouresSpecialVo;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsSpecialMgrVo;
import com.crunii.micro.service.studycenter.service.YcqYjsSpecialMgrService;
import com.crunii.micro.service.studycenter.util.CharacterCodeUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigInteger;
import java.util.List;

/**
 * @author : dinglian
 * @date : 2023-9-18
 */
@Slf4j
@Api(tags = "专题管理")
@RestController
@RequestMapping("/ycqYjsSpecialMgr/auth")
public class YcqYjsSpecialMgrController {

    @Resource
    private YcqYjsSpecialMgrService ycqYjsSpecialMgrServiceImpl;

    @ApiOperation(value = "查询所有专题管理")
    @RequestMapping(path = "paginQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<YcqYjsSpecialMgrDto>> paginQuery(@RequestBody PageReq<YcqYjsSpecialMgrVo> req) {
        try {
            return ycqYjsSpecialMgrServiceImpl.paginQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYjsSpecialMgrController.paginQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsSpecialMgrController.paginQuery.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新增专题管理")
    @RequestMapping(path = "create", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> create(@Valid @RequestBody YcqYjsSpecialMgrVo ycqYjsSpecialMgrVo) {
        try {
            return ycqYjsSpecialMgrServiceImpl.insert(ycqYjsSpecialMgrVo);
        } catch (BusinessException e) {
            log.error("YcqYjsSpecialMgrController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsSpecialMgrController.create.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改专题管理")
    @RequestMapping(path = "update", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> update(@Valid @RequestBody YcqYjsSpecialMgrVo ycqYjsSpecialMgrVo) {
        try {
            return ycqYjsSpecialMgrServiceImpl.update(ycqYjsSpecialMgrVo);
        } catch (BusinessException e) {
            log.error("YcqYjsSpecialMgrController.update.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsSpecialMgrController.update.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除专题管理")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/del", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> del(BigInteger id) {
        try {
            return ycqYjsSpecialMgrServiceImpl.deleteById(id);
        } catch (BusinessException e) {
            log.error("YcqYjsSpecialMgrController.del.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsSpecialMgrController.del.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "根据id查询专题管理")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/queryById", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYjsSpecialMgrDto> queryById(BigInteger id) {
        try {
            return ycqYjsSpecialMgrServiceImpl.queryById(id);
        } catch (BusinessException e) {
            log.error("YcqYjsSpecialMgrController.queryById.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsSpecialMgrController.queryById.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "专题管理关联课程")
    @RequestMapping(path = "createRelationship", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> createRelationship(@Valid @RequestBody YcqYjsCouresSpecialVo YcqYjsCouresSpecialvo) {
        try {
            return ycqYjsSpecialMgrServiceImpl.createRelationship(YcqYjsCouresSpecialvo);
        } catch (BusinessException e) {
            log.error("YcqYjsSpecialMgrController.createRelationship.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsSpecialMgrController.createRelationship.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "生成编号")
    @RequestMapping(path = "generateNumber", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> generateNumber(String code) {
        try {
            String generateCode = CharacterCodeUtil.generateCharacterCode(code);
            return Result.ofSuccess(generateCode);
        } catch (BusinessException e) {
            log.error("YcqYjsSpecialMgrController.generateNumber.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsSpecialMgrController.generateNumber.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "关联课程下拉展示")
    @RequestMapping(path = "relatedCourses", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> relatedCourses() {
        try {
            return ycqYjsSpecialMgrServiceImpl.relatedCourses();
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.generateNumber.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.generateNumber.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据专题分类查询课程信息")
    @RequestMapping(path = "queryCourseInfoBySpecialType", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<YcqYjsCourseMgrResp>> queryCourseInfoBySpecialType(@RequestBody YcqYjsSpecialMgr ycqYjsSpecialMgr) {
        try {
            return ycqYjsSpecialMgrServiceImpl.queryCourseInfoBySpecialType(ycqYjsSpecialMgr);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.paginQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.paginQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }
}
