package com.crunii.micro.service.studycenter.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.studycenter.dto.vo.TestPaperCondition;
import com.crunii.micro.service.studycenter.dto.vo.TestPaperReq;
import com.crunii.micro.service.studycenter.dto.vo.TestPaperVO;
import com.crunii.micro.service.studycenter.service.YcqYjsTestPaperService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * Author：dadm
 * Date：2023-09-20 17:21
 * Description：<描述>
 */
@Slf4j
@Api(tags = "试卷管理接口")
@RestController
@RequestMapping("/ycqYjsTestPaper/auth")
public class YcqYjsTestPaperController {

    @Autowired
    private YcqYjsTestPaperService testPaperService;

    @ApiOperation(value = "新建试卷")
    @RequestMapping(path = "/createTestPaper", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createTestPaper(@Validated @RequestBody @ApiParam TestPaperReq createReq) {
        log.info("TestPaperController.createStudentInfo.param：{}", createReq.toString());
        try {
            testPaperService.createTestPaper(createReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("TestPaperController.createTestPaper.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TestPaperController.createTestPaper.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除试卷")
    @RequestMapping(path = "/delTestPaper", method = RequestMethod.GET)
    @ResponseBody
    public Result<Boolean> delTestPaper(@RequestParam("id") Long id) {
        log.info("TestPaperController.delTestPaper.param：{}", id);
        try {
            testPaperService.delTestPaper(id);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("TestPaperController.delTestPaper.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TestPaperController.delTestPaper.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改试卷")
    @RequestMapping(path = "/updateTestPaper", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateTestPaper(@Validated @RequestBody TestPaperReq updateReq) {
        log.info("TestPaperController.updateTestPaper.param：{}", updateReq.toString());
        try {
            testPaperService.updateTestPaper(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("TestPaperController.updateTestPaper.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TestPaperController.updateTestPaper.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询试卷详情")
    @RequestMapping(path = "/getTestPaperInfo", method = RequestMethod.GET)
    @ResponseBody
    public Result<TestPaperVO> getTestPaperInfo(@RequestParam("id") Long id) {
        log.info("TestPaperController.getTestPaperInfo.param：{}", id);
        try {
            TestPaperVO testPaperVO = testPaperService.getTestPaperInfo(id);
            return Result.ofSuccess(testPaperVO);
        } catch (BusinessException e) {
            log.error("TestPaperController.getTestPaperInfo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TestPaperController.getTestPaperInfo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "分页查询试卷")
    @RequestMapping(path = "/pageSearchTestPaper", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<TestPaperVO>> pageSearchTestPaper(@RequestBody PageReq<TestPaperCondition> req) {
        log.info("TestPaperController.pageSearchTestPaper.param：{}", req.toString());
        try {
            return Result.ofSuccess(testPaperService.pageSearchTestPaper(req));
        } catch (BusinessException e) {
            log.error("TestPaperController.pageSearchTestPaper.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TestPaperController.pageSearchTestPaper.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "启用|禁用")
    @RequestMapping(path = "/startOrEndTestPaper", method = RequestMethod.GET)
    @ResponseBody
    public Result<Boolean> startOrEndTestPaper(@RequestParam("id") Long id,
                                               @RequestParam("status") Integer status) {
        log.info("TestPaperController.startOrEndTestPaper.param：{}", id);
        try {
            testPaperService.startOrEndTestPaper(id, status);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("TestPaperController.startOrEndTestPaper.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TestPaperController.startOrEndTestPaper.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


    @ApiOperation(value = "上传图片")
    @RequestMapping(path = "/uploadImage", method = RequestMethod.POST)
    @ResponseBody
    public Result<FileVo> uploadImage(@RequestParam MultipartFile file) {
        try {
            return Result.ofSuccess(testPaperService.doStoreImage(file));
        } catch (BusinessException e) {
            log.error("TestPaperController.uploadImage.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("TestPaperController.uploadImage.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}


