package com.crunii.micro.service.studycenter.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestion;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestionReq;
import com.crunii.micro.service.studycenter.dto.vo.ImportVO;
import com.crunii.micro.service.studycenter.service.YcqYjsTestPaperQuestionRelService;
import com.crunii.micro.service.studycenter.service.YcqYjsTestQuestionService;
import dm.jdbc.filter.stat.util.JSONUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigInteger;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author : Charler
 * @date : 2023-9-21
 */
@Slf4j
@Api(tags = "试题管理")
@RestController
@RequestMapping("/exam/YcqYjsTestQuestionController/auth")
public class YcqYjsTestQuestionController {

    @Resource
    private YcqYjsTestQuestionService testQuestionService;

    @Autowired
    private YcqYjsTestPaperQuestionRelService  testPaperQuestionRelService;

    @ApiOperation(value = "查询所有")
    @RequestMapping(path = "searchAll", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<YcqYjsTestQuestion>> searchAll() {
        Result<List<YcqYjsTestQuestion>> result = new Result<>();
        result.setData(testQuestionService.searchAll());
        return result;
    }

    @ApiOperation(value = "分页查询所有试题")
    @RequestMapping(path = "pageQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<YcqYjsTestQuestion>> pageQuery(@RequestBody PageReq<YcqYjsTestQuestionReq> req) {
        log.info("YcqYjsTestQuestionController.pageQuery.param：{}", req.toString());
        Result<PageRsp<YcqYjsTestQuestion>> result = new Result<>();
        // 分页查询
        result.setData(testQuestionService.pageYcqYjsTestQuestion(req));
        return result;
    }

    @ApiOperation(value = "分页查询展示相关试题")
    @RequestMapping(path = "pageQueryByIds", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<YcqYjsTestQuestion>> pageQueryByIds(@RequestBody PageReq<YcqYjsTestQuestionReq> req) {
        log.info("YcqYjsTestQuestionController.pageQueryByIds.param：{}", req.toString());
        Result<PageRsp<YcqYjsTestQuestion>> result = new Result<>();
        // 分页查询
        result.setData(testQuestionService.pageYcqYjsTestQuestionByIds(req));
        return result;
    }

    @ApiOperation(value = "新增试题")
    @RequestMapping(path = "create", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> create(@Valid @RequestBody YcqYjsTestQuestionReq creatReq) {
        // 保存新增试题功能
        return testQuestionService.createTestQuestionInfo(creatReq);
    }

    @ApiOperation(value = "修改试题")
    @RequestMapping(path = "update", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> update(@Valid @RequestBody YcqYjsTestQuestionReq updateReq) {
        return testQuestionService.updateTestQuestionInfo(updateReq);
    }

    @ApiOperation(value = "删除试题")
    @ApiImplicitParams( @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger") )
    @RequestMapping(path = "/del", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> del(BigInteger id) {
        int testPaperCountByQuestionId = testPaperQuestionRelService.getTestPaperCountByQuestionId(id);
        if (testPaperCountByQuestionId != 0) {
            throw new BusinessException("此试题存在关联试卷，无法删除");
        }
        return testQuestionService.deleteById(id);
    }

    @ApiOperation(value = "批量删除试题")
    @ApiImplicitParams( @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger") )
    @RequestMapping(path = "/deleteByIds", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> deleteByIds(@RequestBody YcqYjsTestQuestionReq updateReq) {
        Map<BigInteger, Integer> mapByQuestionIds = testPaperQuestionRelService.getTestPaperCountMapByQuestionIds(updateReq.getIds());
        List<BigInteger> filterIds = mapByQuestionIds.entrySet().stream()
                .filter(entry -> entry.getValue() != 0)
                .map(Map.Entry::getKey)
                .collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(filterIds)) {
            return Result.ofFail("-1", "存在关联试卷，无法删除：", filterIds);
        }
        return testQuestionService.deleteByIds(updateReq.getIds());
    }


    @ApiOperation(value = "根据id查询试题详情")
    @RequestMapping(path = "/queryById", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYjsTestQuestionReq> queryById(@RequestParam("id") BigInteger id) {
        log.info("YcqYjsTestQuestionController.queryById.param：{}", JSONUtils.toJSONString(id));
        try {
            return testQuestionService.queryById(id);
        } catch (BusinessException e) {
            log.error("YcqYjsTestQuestionController.queryById.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsTestQuestionController.queryById.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据ids查询试题列表")
    @ApiImplicitParams(@ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger"))
    @RequestMapping(path = "/queryByIds", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<YcqYjsTestQuestionReq>> queryByIds(@RequestBody YcqYjsTestQuestionReq updateReq) {
        return testQuestionService.queryByIds(updateReq.getIds());
    }

    @RequestMapping(path = "/importTestQuestionExcel", method = RequestMethod.POST)
    @ApiOperation("批量导入试题")
    @ResponseBody
    public Result<ImportVO> importTestQuestionExcel(@RequestParam(value="file") MultipartFile file,
                                                    @RequestParam(value = "organizeId", required = false) BigInteger organizeId,
                                                    @RequestParam(value = "organizeName", required = false) String organizeName) {
        log.info("YcqYjsTestQuestionController.importTestQuestionExcel.param：{},{}", JSONUtils.toJSONString(organizeId), JSONUtils.toJSONString(organizeName));
        try {
            ImportVO importVO = testQuestionService.createImportTestQuestionExcel(file, organizeId, organizeName);
            return Result.ofSuccess(importVO);
        } catch (BusinessException e) {
            log.error("YcqYjsTestQuestionController.importTestQuestionExcel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsTestQuestionController.importTestQuestionExcel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
