package com.crunii.micro.service.studycenter.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTopicClassify;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsTopicClassifyCondition;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsTopicClassifyVo;
import com.crunii.micro.service.studycenter.service.YcqYjsTopicClassifyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigInteger;

/**
 * Author：Charler
 * Date：2023-09-20 17:21
 * Description：<描述>
 */
@Slf4j
@Api(tags = "题目分类api接口")
@RestController
@RequestMapping("/YcqYjsTopicClassify/auth")
public class YcqYjsTopicClassifyController {

    @Autowired
    private YcqYjsTopicClassifyService topicClassifyService;


    @ApiOperation(value = "新建题目分类信息")
    @RequestMapping(path = "/createTopicClassify", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createTopicClassify(@Validated @RequestBody @ApiParam YcqYjsTopicClassifyVo createReq) {
        log.info("YcqYjsTopicClassifyController.createTopicClassify.param：{}", createReq.toString());
        try {
            topicClassifyService.createTopicClassify(createReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsTopicClassifyController.createTopicClassify.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsTopicClassifyController.createTopicClassify.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除题目分类")
    @RequestMapping(path = "/delTopicClassify", method = RequestMethod.GET)
    @ResponseBody
    public Result<Boolean> delTopicClassify(@RequestParam("id") BigInteger id) {
        log.info("YcqYjsTopicClassifyController.delTopicClassify.param：{}", id);
        try {
            topicClassifyService.delTopicClassify(id);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsTopicClassifyController.delTopicClassify.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsTopicClassifyController.delTopicClassify.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改题目分类信息")
    @RequestMapping(path = "/updateTopicClassify", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateTopicClassify(@Validated @RequestBody YcqYjsTopicClassifyVo updateReq) {
        log.info("YcqYjsTopicClassifyController.updateTopicClassify.param：{}", updateReq.toString());
        try {
            topicClassifyService.updateTopicClassify(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsTopicClassifyController.updateTopicClassify.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsTopicClassifyController.updateTopicClassify.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询题目分类详情")
    @RequestMapping(path = "/getTopicClassify", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYjsTopicClassify> getTopicClassify(@RequestParam("id") BigInteger id) {
        log.info("YcqYjsTopicClassifyController.getTopicClassify.param：{}", id);
        try {
            YcqYjsTopicClassify topicClassify = topicClassifyService.getTopicClassify(id);
            return Result.ofSuccess(topicClassify);
        } catch (BusinessException e) {
            log.error("YcqYjsTopicClassifyController.getTopicClassify.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsTopicClassifyController.getTopicClassify.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "分页查询题目分类列表")
    @RequestMapping(path = "/pageSearchTopicClassify", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<YcqYjsTopicClassifyVo>> pageSearchTopicClassify(@RequestBody PageReq<YcqYjsTopicClassifyCondition> req) {
        log.info("YcqYjsTopicClassifyController.pageSearchTopicClassify.param：{}", req.toString());
        try {
            return Result.ofSuccess(topicClassifyService.pageSearchTopicClassify(req));
        } catch (BusinessException e) {
            log.error("YcqYjsTopicClassifyController.pageSearchTopicClassify.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsTopicClassifyController.pageSearchTopicClassify.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }


}


