package com.crunii.micro.service.studycenter.controller;

import com.crunii.framework.api.encrypt.annotation.EncryptRequestBody;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.studycenter.annotation.Desensitization;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsUnmatchedPerson;
import com.crunii.micro.service.studycenter.dto.vo.NegativePersonCondition;
import com.crunii.micro.service.studycenter.dto.vo.UnmatchedPersonCondition;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsUnmatchedPersonVo;
import com.crunii.micro.service.studycenter.service.YcqYjsUnmatchedPersonService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;

/**
 * Author：Charler
 * Date：2023-09-20 17:21
 * Description：<描述>
 */
@Slf4j
@Api(tags = "不匹配人员清单api接口")
@RestController
@RequestMapping("/YcqYjsUnmatchedPerson/auth")
public class YcqYjsUnmatchedPersonController {

    @Autowired
    private YcqYjsUnmatchedPersonService unmatchedPersonService;

    @ApiOperation(value = "新建不匹配人员")
    @RequestMapping(path = "/encryptionRequired/createUnmatchedPerson", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createUnmatchedPerson(@Validated @EncryptRequestBody @ApiParam YcqYjsUnmatchedPersonVo createReq) {
        log.info("YcqYjsUnmatchedPersonController.createUnmatchedPerson.param：{}", createReq.toString());
        try {
            unmatchedPersonService.createUnmatchedPerson(createReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsUnmatchedPersonController.createUnmatchedPerson.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsUnmatchedPersonController.createUnmatchedPerson.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除不匹配人员")
    @RequestMapping(path = "/delUnmatchedPerson", method = RequestMethod.GET)
    @ResponseBody
    public Result<Boolean> delUnmatchedPerson(@RequestParam("id") BigInteger id) {
        log.info("YcqYjsUnmatchedPersonController.delUnmatchedPerson.param：{}", id);
        try {
            unmatchedPersonService.delUnmatchedPerson(id);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsUnmatchedPersonController.delUnmatchedPerson.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsUnmatchedPersonController.delUnmatchedPerson.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改不匹配人员信息")
    @RequestMapping(path = "/encryptionRequired/updateUnmatchedPerson", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateUnmatchedPerson(@Validated @EncryptRequestBody @ApiParam YcqYjsUnmatchedPersonVo updateReq) {
        log.info("YcqYjsUnmatchedPersonController.updateUnmatchedPerson.param：{}", updateReq.toString());
        try {
            unmatchedPersonService.updateUnmatchedPerson(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsUnmatchedPersonController.updateUnmatchedPerson.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsUnmatchedPersonController.updateUnmatchedPerson.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询不匹配人员详情")
    @RequestMapping(path = "/getUnmatchedPerson", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYjsUnmatchedPerson> getUnmatchedPerson(@RequestParam("id") BigInteger id) {
        log.info("YcqYjsUnmatchedPersonController.getUnmatchedPerson.param：{}", id);
        try {
            YcqYjsUnmatchedPerson unmatchedPerson = unmatchedPersonService.getUnmatchedPerson(id);
            return Result.ofSuccess(unmatchedPerson);
        } catch (BusinessException e) {
            log.error("YcqYjsUnmatchedPersonController.getUnmatchedPerson.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsUnmatchedPersonController.getUnmatchedPerson.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "分页查询不匹配人员列表")
    @RequestMapping(path = "/pageSearchUnmatchedPerson", method = RequestMethod.POST)
    @Desensitization
    @ResponseBody
    public Result<PageRsp<YcqYjsUnmatchedPersonVo>> pageSearchUnmatchedPerson(@RequestBody PageReq<UnmatchedPersonCondition> req) {
        log.info("YcqYjsUnmatchedPersonController.pageSearchUnmatchedPerson.param：{}", req.toString());
        try {
            return Result.ofSuccess(unmatchedPersonService.pageSearchUnmatchedPerson(req));
        } catch (BusinessException e) {
            log.error("YcqYjsUnmatchedPersonController.pageSearchUnmatchedPerson.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsUnmatchedPersonController.pageSearchUnmatchedPerson.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "同步不匹配人员")
    @RequestMapping(path = "/synchronizeInfo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> synchronizeInfo(@RequestBody PageReq<UnmatchedPersonCondition> req) {
        log.info("YcqYjsUnmatchedPersonController.synchronizeInfo.param：{}",req.toString());
        return Result.ofSuccess();
    }

    @ApiOperation("不匹配人员清单excel导出")
    @RequestMapping(value = "/exportUnmatchedPersonExcel", method = {RequestMethod.POST})
    @ResponseBody
    public Result<Boolean> exportUnmatchedPersonExcel(HttpServletResponse response, @RequestBody UnmatchedPersonCondition req) {
        log.info("YcqYjsUnmatchedPersonController.exportUnmatchedPersonExcel.param：{}", req.toString());
        try {
            unmatchedPersonService.exportUnmatchedPersonExcel(response, req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsUnmatchedPersonController.exportUnmatchedPersonExcel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsUnmatchedPersonController.exportUnmatchedPersonExcel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }
}


