package com.crunii.micro.service.studycenter.controller;

import com.crunii.framework.api.encrypt.annotation.EncryptRequestBody;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.studycenter.annotation.Desensitization;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsYearServiceTime;
import com.crunii.micro.service.studycenter.dto.vo.UnmatchedPersonCondition;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsYearServiceTimeVo;
import com.crunii.micro.service.studycenter.dto.vo.YearServiceTimeCondition;
import com.crunii.micro.service.studycenter.service.YcqYjsYearServiceTimeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;

/**
 * Author：Charler
 * Date：2023-09-20 17:21
 * Description：<描述>
 */
@Slf4j
@Api(tags = "年度服务时长预警api接口")
@RestController
@RequestMapping("/YcqYjsYearServiceTime/auth")
public class YcqYjsYearServiceTimeController {

    @Autowired
    private YcqYjsYearServiceTimeService yearServiceTimeService;


    @ApiOperation(value = "新建年度服务时长信息")
    @RequestMapping(path = "/encryptionRequired/createYearServiceTime", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> createYearServiceTime(@Validated @EncryptRequestBody @ApiParam YcqYjsYearServiceTimeVo createReq) {
        log.info("YcqYjsYearServiceTimeController.createYearServiceTime.param：{}", createReq.toString());
        try {
            yearServiceTimeService.createYearServiceTime(createReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsYearServiceTimeController.createYearServiceTime.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsYearServiceTimeController.createYearServiceTime.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除年度服务时长人员")
    @RequestMapping(path = "/delYearServiceTime", method = RequestMethod.GET)
    @ResponseBody
    public Result<Boolean> delYearServiceTime(@RequestParam("id") BigInteger id) {
        log.info("YcqYjsYearServiceTimeController.delYearServiceTime.param：{}", id);
        try {
            yearServiceTimeService.delYearServiceTime(id);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsYearServiceTimeController.delYearServiceTime.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsYearServiceTimeController.delYearServiceTime.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改年度服务时长信息")
    @RequestMapping(path = "/updateYearServiceTime", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> updateYearServiceTime(@Validated @RequestBody YcqYjsYearServiceTimeVo updateReq) {
        log.info("YcqYjsYearServiceTimeController.updateYearServiceTime.param：{}", updateReq.toString());
        try {
            yearServiceTimeService.updateYearServiceTime(updateReq);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsYearServiceTimeController.updateYearServiceTime.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsYearServiceTimeController.updateYearServiceTime.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询年度服务时长人员详情")
    @RequestMapping(path = "/getYearServiceTime", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYjsYearServiceTime> getYearServiceTime(@RequestParam("id") BigInteger id) {
        log.info("YcqYjsYearServiceTimeController.getYearServiceTime.param：{}", id);
        try {
            YcqYjsYearServiceTime yearServiceTime = yearServiceTimeService.getYearServiceTime(id);
            return Result.ofSuccess(yearServiceTime);
        } catch (BusinessException e) {
            log.error("YcqYjsYearServiceTimeController.getYearServiceTime.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsYearServiceTimeController.getYearServiceTime.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "分页查询年度服务时长人员列表")
    @RequestMapping(path = "/pageSearchYearServiceTime", method = RequestMethod.POST)
    @Desensitization
    @ResponseBody
    public Result<PageRsp<YcqYjsYearServiceTimeVo>> pageSearchYearServiceTime(@RequestBody PageReq<YearServiceTimeCondition> req) {
        log.info("YcqYjsYearServiceTimeController.pageSearchYearServiceTime.param：{}", req.toString());
        try {
            return Result.ofSuccess(yearServiceTimeService.pageSearchYearServiceTime(req));
        } catch (BusinessException e) {
            log.error("YcqYjsYearServiceTimeController.pageSearchYearServiceTime.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsYearServiceTimeController.pageSearchYearServiceTime.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("年度服务时长清单excel导出")
    @RequestMapping(value = "/exportYearServiceTimeExcel", method = {RequestMethod.POST})
    @ResponseBody
    public Result<Boolean> exportYearServiceTimeExcel(HttpServletResponse response, @RequestBody YearServiceTimeCondition req) {
        log.info("YcqYjsYearServiceTimeController.exportYearServiceTimeExcel.param：{}", req.toString());
        try {
            yearServiceTimeService.exportYearServiceTimeExcel(response, req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsYearServiceTimeController.exportYearServiceTimeExcel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsYearServiceTimeController.exportYearServiceTimeExcel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}


