package com.crunii.micro.service.studycenter.handle;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.entity.main.YcqAppConfigUrl;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTopicClassify;
import com.crunii.micro.service.studycenter.dto.vo.ExcelImportTestQuestionModel;
import com.crunii.micro.service.studycenter.dto.vo.ImportVO;
import org.apache.commons.lang3.StringUtils;

import java.math.BigInteger;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Author：dadm
 * Date：2023-09-22 11:01
 * Description：<描述>
 */
public class ExcelImportRegionSiteListener extends AnalysisEventListener<ExcelImportTestQuestionModel> {


    private YcqYjsTestPaperHandle ycqYjsTestPaperHandle;
    private List<ExcelImportTestQuestionModel> models;
    private Map<String, YcqAppConfigUrl> ycqAppConfigUrlMap;
    private Map<String, YcqYjsTopicClassify> ycqYjsTopicClassifyMap;
    private  BigInteger organizeIdTestPaper;
    private ImportVO importVO;

    public ExcelImportRegionSiteListener(YcqYjsTestPaperHandle ycqYjsTestPaperHandle,
                                         List<ExcelImportTestQuestionModel> models,
                                         Map<String, YcqAppConfigUrl> ycqAppConfigUrlMap,
                                         Map<String, YcqYjsTopicClassify> ycqYjsTopicClassifyMap,
                                         BigInteger organizeIdTestPaper,
                                         ImportVO importVO){

        this.ycqYjsTestPaperHandle = ycqYjsTestPaperHandle;
        this.models = models;
        this.ycqAppConfigUrlMap = ycqAppConfigUrlMap;
        this.ycqYjsTopicClassifyMap = ycqYjsTopicClassifyMap;
        this.organizeIdTestPaper = organizeIdTestPaper;
        this.importVO = importVO;
    }

    @Override
    public void invoke(ExcelImportTestQuestionModel model, AnalysisContext analysisContext) {
        model.setOrganizeId(null != ycqAppConfigUrlMap.get(model.getOrganizationName()) ? BigInteger.valueOf(ycqAppConfigUrlMap.get(model.getOrganizationName()).getAppId()) : null);
        model.setOrganizeIdTestPaper(organizeIdTestPaper);

        BigInteger organizeIdTopicClassify = null != organizeIdTestPaper ? organizeIdTestPaper : model.getOrganizeId();
        model.setTopicClassifyId(null != organizeIdTopicClassify && StringUtils.isNotEmpty(model.getTopicClassifyName()) && null != ycqYjsTopicClassifyMap.get(organizeIdTopicClassify + model.getTopicClassifyName()) ?
                ycqYjsTopicClassifyMap.get(organizeIdTopicClassify + model.getTopicClassifyName()).getId() : null);
        models.add(model);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        // 导入数据量校验
        if (models.size() < 2){
            throw new BusinessException("导入数据为空");
        }

        // 导入数据标题校验
        if (!ycqYjsTestPaperHandle.checkImportTestQuestionExcelTitle(models.get(0))){
            throw new BusinessException("excel格式错误");
        }

        // 导入数据格式校验
        models.remove(0);
        ycqYjsTestPaperHandle.checkImportTestQuestionExcel(models);

        importVO.getSuccessList().addAll(models.stream().filter(x->x.getSuccess()).collect(Collectors.toList()));
        importVO.getFailList().addAll(models.stream().filter(x->!x.getSuccess()).collect(Collectors.toList()));
        importVO.setCountAll(models.size());        // 导入总数
        importVO.setCountSuccess(importVO.getSuccessList().size());      // 导入成功数
        importVO.setCountFail(importVO.getFailList().size());     // 导入失败数
    }



}