package com.crunii.micro.service.studycenter.handle;

import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.studycenter.constants.StudyCenterConstants;
import com.crunii.micro.service.studycenter.dto.vo.ExcelImportTestQuestionModel;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * Author：dadm
 * Date：2023-09-22 11:01
 * Description：<描述>
 */
@Component
public class YcqYjsTestPaperHandle {

    /**EXCEL格式-xlsx*/
    String FORMAT_EXCEL_XLSX = "xlsx";
    /**EXCEL格式-xls*/
    String FORMAT_EXCEL_XLS = "xls";

    public Boolean checkExcelFormat(MultipartFile file){
        if (file.getSize() == 0) {
            throw new BusinessException("文件内容为空, 请重新选择文");
        }

        String fileName = file.getOriginalFilename();
        if (!fileName.endsWith(FORMAT_EXCEL_XLS) && !fileName.endsWith(FORMAT_EXCEL_XLSX)) {
            throw new BusinessException("文件格式不支持，请使用以EXCEL 2007/2010（*.xlsx）格式文件或者*.xls");
        }

        return Boolean.TRUE;
    }

    public void checkImportTestQuestionExcel(List<ExcelImportTestQuestionModel> models){
        for (int i = 0;i < models.size();i++){
            ExcelImportTestQuestionModel model = models.get(i);
            model.setNo(i + 1);

            // 1.字段判空
            if (!checkExportTestQuestionExcelEmpty(model)){
                continue;
            }

            // 2.题型判断
            if (null == StudyCenterConstants.QUESTION_TOPIC_TYPE.get(model.getTopicType())){
                model.setSuccess(Boolean.FALSE);
                model.setFailReason("题型不存在");
                continue;
            }

            // 3.正确答案判断(不可为空)
            if (!checkExportTestQuestionExcelAnswerOption(model)){
                continue;
            }

            // 4.所属业务判断
            if (null == model.getOrganizeId()){
                model.setSuccess(Boolean.FALSE);
                model.setFailReason("所属业务不存在");
                continue;
            }

            // 5.所属业务是否相同判断(试卷所属业务和试题所属业务)
            if (null != model.getOrganizeIdTestPaper() && model.getOrganizeIdTestPaper().longValue() != model.getOrganizeId().longValue()){
                model.setSuccess(Boolean.FALSE);
                model.setFailReason("试题和试卷所属业务不同");
                continue;
            }

            // 6.题目分类校验
            if (null == model.getTopicClassifyId()){
                model.setSuccess(Boolean.FALSE);
                model.setFailReason("分类不存在");
                continue;
            }

        }


    }

    public Boolean checkExportTestQuestionExcelEmpty(ExcelImportTestQuestionModel model) {
        model.setFailReason(StringUtils.isEmpty(model.getTopic()) ? "题目不可为空" : model.getFailReason());
        model.setFailReason(StringUtils.isEmpty(model.getTopicType()) ? "题型不可为空" : model.getFailReason());
        model.setFailReason(StringUtils.isEmpty(model.getOptionA()) ? "选项A不可为空" : model.getFailReason());
        model.setFailReason(StringUtils.isEmpty(model.getOptionB()) ? "选项B不可为空" : model.getFailReason());
        model.setFailReason(StringUtils.isEmpty(model.getAnswerOption()) ? "正确答案不可为空" : model.getFailReason());
        model.setFailReason(StringUtils.isEmpty(model.getTopicClassifyName()) ? "分类不可为空" : model.getFailReason());
        model.setFailReason(StringUtils.isEmpty(model.getOrganizationName()) ? "所属业务不可为空" : model.getFailReason());
        model.setSuccess(StringUtils.isNoneBlank(model.getFailReason()) ? false : true);
        return model.getSuccess();
    }

    public Boolean checkExportTestQuestionExcelAnswerOption(ExcelImportTestQuestionModel model) {
        model.setFailReason(model.getAnswerOption().contains(StudyCenterConstants.QUESTION_TOPIC_OPTION_C) && StringUtils.isEmpty(model.getOptionC()) ? "正确答案C不可为空" : model.getFailReason());  // 1.选项C判断
        model.setFailReason(model.getAnswerOption().contains(StudyCenterConstants.QUESTION_TOPIC_OPTION_D) && StringUtils.isEmpty(model.getOptionD()) ? "正确答案D不可为空" : model.getFailReason());  // 2.选项D判断
        model.setFailReason(model.getAnswerOption().contains(StudyCenterConstants.QUESTION_TOPIC_OPTION_E) && StringUtils.isEmpty(model.getOptionE()) ? "正确答案E不可为空" : model.getFailReason());  // 3.选项E判断
        model.setFailReason(model.getAnswerOption().contains(StudyCenterConstants.QUESTION_TOPIC_OPTION_F) && StringUtils.isEmpty(model.getOptionF()) ? "正确答案F不可为空" : model.getFailReason());  // 4.选项F判断
        model.setSuccess(StringUtils.isNoneBlank(model.getFailReason()) ? false : true);
        return model.getSuccess();
    }

    public Boolean checkImportTestQuestionExcelTitle(ExcelImportTestQuestionModel model) {
        model.setSuccess(StringUtils.isEmpty(model.getTopic()) || !model.getTopic().contains("题目") ? false : model.getSuccess());
        model.setSuccess(StringUtils.isEmpty(model.getTopicType()) || !model.getTopicType().contains("题型") ? false : model.getSuccess());
        model.setSuccess(StringUtils.isEmpty(model.getOptionA()) || !model.getOptionA().contains("选项A") ? false : model.getSuccess());
        model.setSuccess(StringUtils.isEmpty(model.getOptionB()) || !model.getOptionB().contains("选项B") ? false : model.getSuccess());
        model.setSuccess(StringUtils.isEmpty(model.getOptionC()) || !model.getOptionC().contains("选项C") ? false : model.getSuccess());
        model.setSuccess(StringUtils.isEmpty(model.getOptionD()) || !model.getOptionD().contains("选项D") ? false : model.getSuccess());
        model.setSuccess(StringUtils.isEmpty(model.getOptionE()) || !model.getOptionE().contains("选项E") ? false : model.getSuccess());
        model.setSuccess(StringUtils.isEmpty(model.getOptionF()) || !model.getOptionF().contains("选项F") ? false : model.getSuccess());
        model.setSuccess(StringUtils.isEmpty(model.getAnswerOption()) || !model.getAnswerOption().contains("正确答案") ? false : model.getSuccess());
        model.setSuccess(StringUtils.isEmpty(model.getTopicClassifyName()) || !model.getTopicClassifyName().contains("分类") ? false : model.getSuccess());
        return model.getSuccess();
    }

    public Boolean checkExportTestQuestionTopicClassify(ExcelImportTestQuestionModel model) {


        return model.getSuccess();
    }
}
