package com.crunii.micro.service.studycenter.jdbc;



import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseMgr;


import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseMgrResp;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseMgrRspPage;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseTestRel;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsCourseMgrVo;
import com.crunii.micro.service.studycenter.util.BaseJdbcTemplateUtil;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 试题表;
 *
 * @author : dinglian
 * @date : 2023-9-18
 */
@Repository
public class YcqYjsCourseMgrJdbcDao {

    public static final String PAGE_SQL ="select a.*, " +
            " (select WM_CONCAT(c.special_name)  from YCQ_YJS_COURES_SPECIAL b,YCQ_YJS_SPECIAL_MGR c where b.special_id=c.id and b.course_id= a.id and c.del_flag='0') as special_names," +
            " (select count(1) from YCQ_YJS_COURSESUB_MGR d where d.course_id = a.id) as sum_subcoures," +
            " case when a.test_type=1 then (select f.title from  YCQ_YJS_COURSE_TEST_REL e,YCQ_YJS_TEST_PAPER f where a.id = e.course_id and e.test_type=1 and e.test_id = f.id) end as paper_title, " +
            " case when a.test_type=2 then (select count(1) from  YCQ_YJS_COURSE_TEST_REL e,YCQ_YJS_TEST_QUESTION g where a.id = e.course_id and e.test_type=2 and e.test_id = g.id) end as sum_questtions " +
            " from YCQ_YJS_COURSE_MGR a where 1 = 1 and a.del_flag=0 ";

    public static final String PAGE_COUNT_SQL ="select count(1) from YCQ_YJS_COURSE_MGR a where 1 = 1 and a.del_flag=0 ";

    public static final String DELETE_SQL ="Update YCQ_YJS_COURSE_MGR a set a.del_flag=1  where id = ? ";

    public static final String QUERY_SQL ="SELECT t.*, " +
            " (select a.value_name from tsm_property_value a,tsm_property b " +
            "             WHERE a.prop_id=b.prop_id AND t.course_type = a.value_code " +
            "             AND b.prop_code='yjs-courseType') as course_type_name," +
            " (select a.value_name from tsm_property_value a,tsm_property b " +
            "             WHERE a.prop_id=b.prop_id AND t.course_labels = a.value_code " +
            "             AND b.prop_code='yjs-courseLabels') as course_labels_name," +
            " (select WM_CONCAT(b.special_name)  from YCQ_YJS_COURES_SPECIAL a,YCQ_YJS_SPECIAL_MGR b where a.course_id=t.id and a.special_id=b.id and b.del_flag='0') as special_name" +
            "  FROM YCQ_YJS_COURSE_MGR t where t.id = ? ";

    // 课程分类查询
    public static final String QUERY_COURSE_TYPE_SQL="select t.course_type," +
            " (select a.value_name from tsm_property_value a,tsm_property b " +
            " WHERE a.prop_id=b.prop_id AND t.course_type = a.value_code " +
            " AND b.prop_code='yjs-courseType') as course_type_name, " +
            " t.id,t.course_name from YCQ_YJS_COURSE_MGR t where t.del_flag=0 ";

    // 修改语句
    public static final String UPDATE_SQL ="update YCQ_YJS_COURSE_MGR a " +
            " set a.course_name=?,a.course_description=?,a.course_type=?,a.course_labels=?," +
            " a.status=?,a.update_time=?,a.is_subcourse=?,a.is_testquestions=?,a.test_type=?," +
            " a.attachment=?,a.is_uploadvideo=?,a.uploadvideo_guide=?,a.course_duration=?," +
            " a.course_credit=?,a.video_name=?,a.is_drag_play=?,a.is_multiply_play=?,a.attachment_name=?,a.course_cover=? where a.id=?";

    // 修改课程状态语句
    public static final String UPDATE_COURSE_STATUS_SQL ="update YCQ_YJS_COURSE_MGR a " +
            " set a.status=?,a.update_time=? where a.id=?";

    // 根据试卷id查询所关联的课程
    public static final String QUERY_COURSE_BY_TEST=" select id from YCQ_YJS_COURSE_TEST_REL t where t.test_type=? and t.test_id=? ";

    // 判断课程名称是否重复
    public static final String QUERY_COURSENAME_CHECK=" select count(1) from YCQ_YJS_COURSE_MGR a where a.course_name=? ";
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    public PageRsp<YcqYjsCourseMgrRspPage> paginQuery(PageReq<YcqYjsCourseMgrVo> req) {
        StringBuilder sb = new StringBuilder(PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);

        List<Object> params = new ArrayList<>();
        if (req.getQuery()!=null) {
            // 课程编号
            if (StringUtils.hasText(req.getQuery().getCourseId())) {
                sb.append(" and a.course_id like concat('%',?,'%') ");
                sbCount.append(" and a.course_id like concat('%',?,'%') ");
                params.add(req.getQuery().getCourseId());
            }
            // 课程名称
            if (StringUtils.hasText(req.getQuery().getCourseName())) {
                sb.append(" and a.course_name like concat('%',?,'%') ");
                sbCount.append(" and a.course_name like concat('%',?,'%') ");
                params.add(req.getQuery().getCourseName());
            }
            // 课程分类
            if (req.getQuery().getCourseType() != null) {
                sb.append(" and a.course_type = ? ");
                sbCount.append(" and a.course_type =? ");
                params.add(req.getQuery().getCourseType());
            }
            // 课程标签
            if (StringUtils.hasText(req.getQuery().getCourseLabels())) {
                sb.append(" and a.course_labels like  concat('%',?,'%') ");
                sbCount.append(" and a.course_labels like  concat('%',?,'%') ");
                params.add(req.getQuery().getCourseLabels());
            }

            // 关联专题
            if (req.getQuery().getSpecialSubjectId() != null) {
                sb.append(" and a.special_subject_id = ? ");
                sbCount.append(" and a.special_subject_id =? ");
                params.add(req.getQuery().getSpecialSubjectId());
            }

            // 发布时间
            if (StringUtils.hasText(req.getQuery().getCreateTimeBegin())) {
                sb.append(" and a.create_time >= ? ");
                sbCount.append(" and a.create_time  >= ? ");
                params.add(req.getQuery().getCreateTimeBegin());
            }

            if (StringUtils.hasText(req.getQuery().getCreateTimeEnd())) {
                sb.append(" and a.create_time <= ? ");
                sbCount.append(" and a.create_time  <= ? ");
                params.add(req.getQuery().getCreateTimeEnd());
            }

            // 状态
            if (req.getQuery().getStatus() != null) {
                sb.append(" and a.status = ? ");
                sbCount.append(" and a.status  = ? ");
                params.add(req.getQuery().getStatus());
            }

            // 关联专题id
            if (req.getQuery().getSpecialId() != null) {
                sb.append(" and exists (select 1 from YCQ_YJS_COURES_SPECIAL s where s.special_id=? and s.course_id=a.id) ");
                sbCount.append(" and exists (select 1 from YCQ_YJS_COURES_SPECIAL s where s.special_id=? and s.course_id=a.id) ");
                params.add(req.getQuery().getSpecialId());
            }
        }
        sb.append("and a.del_flag=0 order by a.create_time desc ");
        int pageNo = req.getPageNo();
        int pageSize = req.getPageSize();
        int count = JT.queryForInt(sbCount.toString(), params.toArray());
        List list = new ArrayList();
        if (count>0) {
            list = queryForPage(sb.toString(), pageNo, pageSize, params);
        }

        PageRsp<YcqYjsCourseMgrRspPage> rsp = new PageRsp<>();
        rsp.setResult(list);
        rsp.setTotalNum(count);
        rsp.setPageNo(pageNo);
        rsp.setPageSize(pageSize);

        return rsp;
    }

    public List queryForPage(String sql, int pageNo, int pageSize, List<Object> params) {
        int offset = (pageNo - 1) * pageSize;
        sql = sql + " limit " + pageSize + " offset " + offset;
        return JT.queryForBeans(YcqYjsCourseMgrRspPage.class, sql, params.toArray());
    }

    public YcqYjsCourseMgr insert(YcqYjsCourseMgr ycqYjsCourseMgr) {
        ycqYjsCourseMgr.setCreateTime(new Date());
        ycqYjsCourseMgr.setDelFlag(0);
        String insertsql = BaseJdbcTemplateUtil.insert(YcqYjsCourseMgr.class);
        Object[] objs = BaseJdbcTemplateUtil.getValues(ycqYjsCourseMgr);
        PreparedStatementCreator create = new PreparedStatementCreator() {
            //利用传统的方法进行JDBC查询
            @Override
            public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
                //插入用户，返回主键
                PreparedStatement preparedStatement = connection.prepareStatement(insertsql,
                        PreparedStatement.RETURN_GENERATED_KEYS);
                for (int i = 0; i < objs.length; i++) {
                    preparedStatement.setObject(i+1, objs[i]);
                }
                return preparedStatement;
            }
        };
        //2.创建keyHoldle
        GeneratedKeyHolder keyHolder = new GeneratedKeyHolder();
        JT.update(create, keyHolder);
        //获取生成的主键key
        Long key = keyHolder.getKey().longValue();
        BigInteger id = new BigInteger(String.valueOf(key));
        ycqYjsCourseMgr.setId(id);
        return ycqYjsCourseMgr;
    }

    public void update(YcqYjsCourseMgr ycqYjsCourseMgr)  {
        ycqYjsCourseMgr.setUpdateTime(new Date());
        try {
//            String updatesql = BaseJdbcTemplateUtil.update(ycqYjsCourseMgr);
           JT.update(UPDATE_SQL, ycqYjsCourseMgr.getCourseName(),ycqYjsCourseMgr.getCourseDescription(),ycqYjsCourseMgr.getCourseType(),
                   ycqYjsCourseMgr.getCourseLabels(),ycqYjsCourseMgr.getStatus(),ycqYjsCourseMgr.getUpdateTime(),ycqYjsCourseMgr.getIsSubcourse(),
                   ycqYjsCourseMgr.getIsTestquestions(),ycqYjsCourseMgr.getTestType(),ycqYjsCourseMgr.getAttachment(),ycqYjsCourseMgr.getIsUploadvideo(),
                   ycqYjsCourseMgr.getUploadvideoGuide(),ycqYjsCourseMgr.getCourseDuration(),ycqYjsCourseMgr.getCourseCredit(),
                   ycqYjsCourseMgr.getVideoName(),ycqYjsCourseMgr.getIsDragPlay(),ycqYjsCourseMgr.getIsMultiplyPlay(),ycqYjsCourseMgr.getAttachmentName(),ycqYjsCourseMgr.getCourseCover(),
                   ycqYjsCourseMgr.getId()
           );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void deleteById(BigInteger id) {
        JT.update(DELETE_SQL, new Object[]{id});
    }

    public YcqYjsCourseMgrResp queryById(BigInteger id) {
        YcqYjsCourseMgrResp ycqYjsCourseMgrResp = JT.queryForBean(YcqYjsCourseMgrResp.class, QUERY_SQL, new Object[]{id});
        // 根据课程类型查询关联试卷试题
        return ycqYjsCourseMgrResp;
    }

    // 课程分类查询
    public Map<Integer,List<YcqYjsCourseMgrResp>>  queryCourseType() {
        List<YcqYjsCourseMgrResp> list = JT.queryForBeans(YcqYjsCourseMgrResp.class, QUERY_COURSE_TYPE_SQL);
        Map<Integer,List<YcqYjsCourseMgrResp>>  returnMap = list.stream().collect(Collectors.groupingBy(YcqYjsCourseMgr::getCourseType));
        return returnMap;
    }

    // 根据试卷、试题id关联的课程
    public List queryCourseByTest(int testType,  BigInteger testId) {
        List<YcqYjsCourseTestRel> list = JT.queryForBeans(YcqYjsCourseTestRel.class, QUERY_COURSE_BY_TEST, new Object[] {testType, testId});
        return list;
    }

    // 更改课程状态
    public int updateCourseStatus(YcqYjsCourseMgr ycqYjsCourseMgr) {
        ycqYjsCourseMgr.setUpdateTime(new Date());
        int i =JT.update(UPDATE_COURSE_STATUS_SQL, ycqYjsCourseMgr.getStatus(), ycqYjsCourseMgr.getUpdateTime(), ycqYjsCourseMgr.getId());
        return i;
    }

    // 判断课程名称是否重复
    public int courseNameCheck(String courseName) {
        int i = JT.queryForInt(QUERY_COURSENAME_CHECK, new Object[]{courseName});
        return i;
    }
}
