package com.crunii.micro.service.studycenter.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseMgr;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseTestRel;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestPaper;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestion;
import com.crunii.micro.service.studycenter.util.BaseJdbcTemplateUtil;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Repository
public class YcqYjsCourseTestRelDao {

    public static final String DELETE_SQL ="delete from YCQ_YJS_COURSE_TEST_REL a  where a.course_id = ? and a.test_type=? and a.test_id = ?";

    /**
     *  关联题库表查询
     * */
    public static final String QUERY_TEST_SQL = "select * from YCQ_YJS_COURSE_TEST_REL a where a.test_type='2' and a.course_id = ? ";
    /**
     *  关联试卷表查询
     * */
    public static final String QUERY_PAPER_SQL = "select * from YCQ_YJS_COURSE_TEST_REL a left join YCQ_YJS_TEST_PAPER b " +
            " on a.test_id = b.id  where a.course_id = ? ";

    /**
     * 根据试卷id，查询课程信息
     * */
    public static final String QUERY_COURSE_BYTESTID_SQL = " select * from YCQ_YJS_COURSE_MGR m where 1=1 " +
            " and exists (select * from YCQ_YJS_COURSE_TEST_REL l where  m.id =l.course_id and l.test_type=? and l.test_id=?) ";

    // 批量删除试卷、题目关联关系
    public static final String BATH_DELETE_BY_COURSE_ID_SQL = "delete from YCQ_YJS_COURSE_TEST_REL a where a.course_id=?";

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    /**
     * 新增
     * */
    public void insert(YcqYjsCourseTestRel ycqYjsCourseTestRel) {
        ycqYjsCourseTestRel.setCreateTime(new Date());
        ycqYjsCourseTestRel.setDelFlag(0);
        String insertsql = BaseJdbcTemplateUtil.insert(YcqYjsCourseTestRel.class);
        JT.update(insertsql, BaseJdbcTemplateUtil.getValues(ycqYjsCourseTestRel));
    }

    /**
     * 批量新增
     * */
    public void batchInsert(List<YcqYjsCourseTestRel> list) {
        YcqYjsCourseTestRel ycqYjsCourseTestRel = null;
        List<Object[]> listObj = new ArrayList<>();
        Object[] obj = null;
        for (int i = 0; i < list.size(); i++) {
            ycqYjsCourseTestRel = list.get(i);
            ycqYjsCourseTestRel.setCreateTime(new Date());
            ycqYjsCourseTestRel.setDelFlag(0);
            obj = BaseJdbcTemplateUtil.getValues(ycqYjsCourseTestRel);
            listObj.add(obj);
        }
        String insertsql = BaseJdbcTemplateUtil.insert(YcqYjsCourseTestRel.class);
        JT.batchUpdate(insertsql, listObj);
    }

    /**
     * 物理删除
     * */
    public void deleteYcqYjsCourseTestRel(YcqYjsCourseTestRel ycqYjsCourseTestRel) {
        JT.update(DELETE_SQL, new Object[]{ycqYjsCourseTestRel.getCourseId(), ycqYjsCourseTestRel.getTestType(), ycqYjsCourseTestRel.getTestId()});
    }

    /**
     * 根据课程id删除关联关系
     * */
    public void deleteBathCourseById(BigInteger courseId) {
        JT.update(BATH_DELETE_BY_COURSE_ID_SQL, new Object[]{courseId});
    }

    /**
     *  关联题库表查询
     * */
    public List<YcqYjsCourseTestRel> queryTestByCourseId(BigInteger courseId) {
        List<YcqYjsCourseTestRel> list = JT.queryForBeans(YcqYjsCourseTestRel.class, QUERY_TEST_SQL, new Object[]{courseId});
        return list;
    }

    /**
     *  关联试卷表查询
     * */
    public List<YcqYjsTestPaper> queryPaperByCourseId(BigInteger courseId) {
        List<YcqYjsTestPaper> list = JT.queryForBeans(YcqYjsTestPaper.class, QUERY_PAPER_SQL, new Object[]{courseId});
        return list;
    }

    public List<YcqYjsCourseMgr>  queryYcqYjsCourseByTestId(YcqYjsCourseTestRel ycqYjsCourseTestRel){
        List<YcqYjsCourseMgr> list = JT.queryForBeans(YcqYjsCourseMgr.class, QUERY_COURSE_BYTESTID_SQL, new Object[]{ycqYjsCourseTestRel.getTestType(),ycqYjsCourseTestRel.getTestId()});
        return list;
    }
}
