package com.crunii.micro.service.studycenter.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.common.jdbc.BaseNPJdbcTemplate;
import com.crunii.micro.service.studycenter.dto.studycenter.*;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsSpecialMgrVo;
import com.crunii.micro.service.studycenter.util.BaseJdbcTemplateUtil;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository
public class YcqYjsSpecialMgrDao {
    public static final String PAGE_SQL ="select a.*, " +
            " (select u.app_name from YCQ_APP_CONFIG_URL u where u.app_id= affiliated_business_id) as affiliatedBusinessName," +
            " (select count(1) from YCQ_YJS_COURES_SPECIAL b,YCQ_YJS_COURSE_MGR c where b.course_id=c.id and  a.id = b.special_id and c.del_flag='0') as sum_coures," +
            " (select sum(c.course_duration) from YCQ_YJS_COURES_SPECIAL b,YCQ_YJS_COURSE_MGR c where a.id = b.special_id and b.course_id=c.id and c.del_flag='0') as course_duratios, " +
            " (select sum(c.course_credit) from YCQ_YJS_COURES_SPECIAL b,YCQ_YJS_COURSE_MGR c where a.id = b.special_id and b.course_id=c.id and c.del_flag='0') as course_credits " +
            "from YCQ_YJS_SPECIAL_MGR a where 1 = 1 and a.del_flag=0 ";

    public static final String PAGE_COUNT_SQL ="select count(1) from YCQ_YJS_SPECIAL_MGR a where 1 = 1 and a.del_flag=0 ";

    public static final String DELETE_SQL ="Update YCQ_YJS_SPECIAL_MGR a set a.del_flag=1  where a.id = ? ";

    public static final String QUERY_SQL ="select a.*, " +
            " (select u.app_name from YCQ_APP_CONFIG_URL u where u.app_id= affiliated_business_id) as affiliatedBusinessName," +
            " (select count(1) from YCQ_YJS_COURES_SPECIAL b,YCQ_YJS_COURSE_MGR c where b.course_id=c.id and  a.id = b.special_id and c.del_flag='0') as sum_coures," +
            " (select sum(c.course_duration) from YCQ_YJS_COURES_SPECIAL b,YCQ_YJS_COURSE_MGR c where a.id = b.special_id and b.course_id=c.id and c.del_flag='0') as course_duratios, " +
            " (select sum(c.course_credit) from YCQ_YJS_COURES_SPECIAL b,YCQ_YJS_COURSE_MGR c where a.id = b.special_id and b.course_id=c.id and c.del_flag='0') as course_credits " +
            "from YCQ_YJS_SPECIAL_MGR a where a.id=? and a.del_flag=0 ";

    public static final String QUERY_COURSE_SQL ="select c.id,c.course_name from YCQ_YJS_COURES_SPECIAL b,YCQ_YJS_COURSE_MGR c  where c.del_flag='0' and b.course_id=c.id and b.special_id = ? ";

    public static final String UPDATE_SQL ="Update YCQ_YJS_SPECIAL_MGR a set a.special_name=?, " +
            " a.affiliated_business_id=?,a.status=? ,a.period_validity=?, " +
            " a.validity_time=?,a.special_type=? " +
            " where a.id = ?";

    public static final String QUERY_COURSEINFO_SPECIALTYPE="select " +
            " a.special_number," +
            " a.id as specialId," +
            " a.special_name," +
            " a.affiliated_business_id," +
            " a.status as specialStatus, " +
            " a.period_validity," +
            " a.validity_time," +
            " a.special_type," +
            " (select t.value_name from TSM_PROPERTY_VALUE t where t.prop_code='yjs-topicClassification' and a.special_type=t.value_code) as special_type_name,  " +
            " c.* from YCQ_YJS_SPECIAL_MGR a,YCQ_YJS_COURES_SPECIAL b,YCQ_YJS_COURSE_MGR c" +
            " where a.id=b.special_id and b.course_id=c.id and a.del_flag = 0 and c.del_flag = 0 and a.status=1 and c.status=1 ";

    // 专题下拉列表
    public static final String QUERY_SPECIALINFO_SQL="SELECT id , special_name from YCQ_YJS_SPECIAL_MGR";

    // app端专题查询接口
    public static final String APP_QUERY_SPECIALINFO_SQL = "SELECT id , special_name from YCQ_YJS_SPECIAL_MGR a where a.del_flag = 0 and a.status=1 ";

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    /**
     * 分页查询
     * */
    public PageRsp<YcqYjsSpecialMgrDto> paginQuery(PageReq<YcqYjsSpecialMgrVo> req) {
        StringBuilder sb = new StringBuilder(PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);

        List<Object> params = new ArrayList<>();
        if (req.getQuery()!=null) {
            // 专题编号
            if (StringUtils.hasText(req.getQuery().getSpecialNumber())) {
                sb.append(" and a.special_number like concat('%',?,'%') ");
                sbCount.append(" and a.special_number like concat('%',?,'%') ");
                params.add(req.getQuery().getSpecialNumber());
            }
            // 专题名称
            if (StringUtils.hasText(req.getQuery().getSpecialName())) {
                sb.append(" and a.special_name like concat('%',?,'%') ");
                sbCount.append(" and a.special_name like concat('%',?,'%') ");
                params.add(req.getQuery().getSpecialName());
            }
            // 所属业务
            if ( req.getQuery().getAffiliatedBusinessId()!=null ) {
                sb.append(" and a.affiliated_business_id = ? ");
                sbCount.append(" and a.affiliated_business_id =? ");
                params.add(req.getQuery().getAffiliatedBusinessId());
            }
            // 专题状态
            if ( req.getQuery().getStatus()!=null ) {
                sb.append(" and a.status = ? ");
                sbCount.append(" and a.status =? ");
                params.add(req.getQuery().getStatus());
            }

            // 创建时间
            if ( StringUtils.hasText(req.getQuery().getCreateTimeBegin())) {
                sb.append(" and a.create_time >= ? ");
                sbCount.append(" and a.create_time  >= ? ");
                params.add(req.getQuery().getCreateTimeBegin());
            }

            if ( StringUtils.hasText(req.getQuery().getCreateTimeEnd()) ) {
                sb.append(" and a.create_time <= ? ");
                sbCount.append(" and a.create_time  <= ? ");
                params.add(req.getQuery().getCreateTimeEnd());
            }
        }


        sb.append(" and a.del_flag=0 order by a.create_time desc ");
        int pageNo = req.getPageNo();
        int pageSize = req.getPageSize();
        int count = JT.queryForInt(sbCount.toString(), params.toArray());
        List list = new ArrayList();

        if (count>0) {
            list = queryForPage(sb.toString(), pageNo, pageSize, params);
        }

        PageRsp<YcqYjsSpecialMgrDto> rsp = new PageRsp<>();
        rsp.setResult(list);
        rsp.setTotalNum(count);
        rsp.setPageNo(pageNo);
        rsp.setPageSize(pageSize);

        return rsp;
    }

    public List queryForPage(String sql, int pageNo, int pageSize, List<Object> params) {
        int offset = (pageNo - 1) * pageSize;
        sql = sql + " limit " + pageSize + " offset " + offset;
        return JT.queryForBeans(YcqYjsSpecialMgrDto.class, sql, params.toArray());
    }

    /**
     * 插入
     * */
    public YcqYjsSpecialMgr insert(YcqYjsSpecialMgr ycqYjsSpecialMgr) {
        ycqYjsSpecialMgr.setCreateTime(new Date());
        ycqYjsSpecialMgr.setDelFlag(0);
        String insertsql = BaseJdbcTemplateUtil.insert(YcqYjsSpecialMgr.class);
        Object[] objs = BaseJdbcTemplateUtil.getValues(ycqYjsSpecialMgr);
        PreparedStatementCreator create = new PreparedStatementCreator() {
            //利用传统的方法进行JDBC查询
            @Override
            public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
                //插入用户，返回主键
                PreparedStatement preparedStatement = connection.prepareStatement(insertsql,
                        PreparedStatement.RETURN_GENERATED_KEYS);
                for (int i = 0; i < objs.length; i++) {
                    preparedStatement.setObject(i+1, objs[i]);
                }
                return preparedStatement;
            }
        };
        //2.创建keyHoldle
        GeneratedKeyHolder keyHolder = new GeneratedKeyHolder();
        JT.update(create, keyHolder);
        //获取生成的主键key
        Long key = keyHolder.getKey().longValue();
        BigInteger id = new BigInteger(String.valueOf(key));
        ycqYjsSpecialMgr.setId(id);
        return ycqYjsSpecialMgr;
    }

    /**
     * 修改
     * */
    public void update(YcqYjsSpecialMgr ycqYjsSpecialMgr)  {
        ycqYjsSpecialMgr.setUpdateTime(new Date());
        try {
            JT.update(UPDATE_SQL, ycqYjsSpecialMgr.getSpecialName(), ycqYjsSpecialMgr.getAffiliatedBusinessId(), ycqYjsSpecialMgr.getStatus(),
                    ycqYjsSpecialMgr.getPeriodValidity(),ycqYjsSpecialMgr.getValidityTime(),ycqYjsSpecialMgr.getSpecialType(), ycqYjsSpecialMgr.getId());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 逻辑删除
     * */
    public void deleteById(BigInteger id) {
        JT.update(DELETE_SQL, new Object[]{id});
    }

    /**
     * 根据id查询对象
     * */
    public YcqYjsSpecialMgrDto queryById(BigInteger id) {
        YcqYjsSpecialMgrDto ycqYjsSpecialMgrDto = JT.queryForBean (YcqYjsSpecialMgrDto.class, QUERY_SQL, new Object[]{id});
        return ycqYjsSpecialMgrDto;
    }

    public List<YcqYjsCourseMgrRsp> queryYcqYjsCourseMgrBySpecialId(BigInteger id) {
        // 查询关联的课程
        List<YcqYjsCourseMgrRsp> list = JT.queryForBeans(YcqYjsCourseMgrRsp.class, QUERY_COURSE_SQL, new Object[]{id});
        return list;
    }

    public List<YcqYjsCourseMgrResp> queryCourseInfoBySpecialType(YcqYjsSpecialMgr ycqYjsSpecialMgr) {
        StringBuffer sql= new StringBuffer(QUERY_COURSEINFO_SPECIALTYPE);
        List<Object> params = new ArrayList<>();

        // 专题ID
        if (ycqYjsSpecialMgr.getId() !=null ) {
            sql.append(" and a.id = ? ");
            params.add(ycqYjsSpecialMgr.getId());
        }
        // 所属业务
        if ( ycqYjsSpecialMgr.getAffiliatedBusinessId()!=null ) {
            sql.append(" and a.affiliated_business_id = ? ");
            params.add(ycqYjsSpecialMgr.getAffiliatedBusinessId());
        }

        // 专题分类
        if ( ycqYjsSpecialMgr.getSpecialType()!=null ) {
            sql.append(" and a.special_type = ? ");
            params.add(ycqYjsSpecialMgr.getSpecialType());
        }
        return JT.queryForBeans(YcqYjsCourseMgrResp.class, sql.toString(), params.toArray());
    }

    public List<YcqYjsSpecialMgr> querySpecialInfoList() {
        List<YcqYjsSpecialMgr> list = JT.queryForBeans(YcqYjsSpecialMgr.class, QUERY_SPECIALINFO_SQL);
        return list;
    }

    public List<YcqYjsSpecialMgr> queryAppVideoList(YcqYjsSpecialMgr ycqYjsSpecialMgr) {
        StringBuffer sql= new StringBuffer(APP_QUERY_SPECIALINFO_SQL);
        List<Object> params = new ArrayList<>();
        // 所属业务
        if ( ycqYjsSpecialMgr.getAffiliatedBusinessId()!=null ) {
            sql.append(" and a.affiliated_business_id = ? ");
            params.add(ycqYjsSpecialMgr.getAffiliatedBusinessId());
        }
        List<YcqYjsSpecialMgr> list = JT.queryForBeans(YcqYjsSpecialMgr.class, sql.toString(), params.toArray());
        return list;
    }
}
