package com.crunii.micro.service.studycenter.jdbc;

import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.studycenter.constants.StudyCenterConstants;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestPaper;
import com.crunii.micro.service.studycenter.dto.vo.TestPaperCondition;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Author：dadm
 * Date：2023-10-12 10:03
 * Description：<描述>
 */
@Repository
public class YcqYjsTestPaperJdbcRepo {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;
    public static final String PAGE_COUNT_SQL ="SELECT count(1) FROM YCQ_YJS_TEST_PAPER WHERE 1 = 1 and del_flag=0 ";
    private static final String PAGE_SQL = "SELECT * FROM YCQ_YJS_TEST_PAPER WHERE 1=1 and del_flag=0 ";

    public PageRsp<YcqYjsTestPaper> paginQuery(PageReq<TestPaperCondition> req) {
        StringBuilder sb = new StringBuilder(PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        @Valid TestPaperCondition reqQuery = req.getQuery();
        List<Object> params = new ArrayList<>();
        if (req.getQuery()!=null) {
            // 标题title
            if (StringUtils.hasText(reqQuery.getTitle())) {
                sb.append(" AND title like concat('%',?,'%') ");
                sbCount.append(" AND title like concat('%',?,'%') ");
                params.add(reqQuery.getTitle());
            }

            // 所属业务编码
            if (StringUtils.hasText(reqQuery.getBusinessCode())) {
                sb.append(" AND business_code = ?");
                sbCount.append(" AND business_code = ? ");
                params.add(reqQuery.getBusinessCode());
            }

            // 试卷类型比编码
            if (StringUtils.hasText(reqQuery.getTypeCode())) {
                sb.append(" AND type_code = ?");
                sbCount.append(" AND type_code = ? ");
                params.add(reqQuery.getTypeCode());
            }

            // 启用禁用状态
            if (null != reqQuery.getEnableStatus()) {
                sb.append(" AND  status = ?");
                sbCount.append(" AND status = ? ");
                params.add(reqQuery.getEnableStatus());
            }

            // 试卷状态-未开始
            if (null != reqQuery.getStatus() && reqQuery.getStatus() == StudyCenterConstants.TEST_PAPER_STATUS_NO ) {
                sb.append(" AND start_time > ? ");
                sbCount.append(" AND start_time  > ? ");
                params.add(DateUtil.format(new Date(), DateUtil.defaultDTPattern));

                sb.append(" AND  is_exam = ?");
                sbCount.append(" AND is_exam = ? ");
                params.add(CommonConstants.LOG_FLAG_YES);
            }

            // 试卷状态-进行中
            if (null != reqQuery.getStatus() && reqQuery.getStatus() == StudyCenterConstants.TEST_PAPER_STATUS_START ) {
                String currentTime = DateUtil.format(new Date(), DateUtil.defaultDTPattern);
                String sqlQuery = "AND (( start_time <= '" +  currentTime + "' AND end_time >= '" + currentTime + "') OR is_exam = 0)";
                sb.append(sqlQuery);
                sbCount.append(sqlQuery);
            }

            // 试卷状态-已结束
            if (null != reqQuery.getStatus() && reqQuery.getStatus() == StudyCenterConstants.TEST_PAPER_STATUS_END ) {
                sb.append(" AND end_time < ? ");
                sbCount.append(" AND end_time  < ? ");
                params.add(DateUtil.format(new Date(), DateUtil.defaultDTPattern));

                sb.append(" AND  is_exam = ?");
                sbCount.append(" AND is_exam = ? ");
                params.add(CommonConstants.LOG_FLAG_YES);
            }

            // 考试开始时间
            if (StringUtils.hasText(reqQuery.getStartTime())) {
                String sql = " AND ((start_time >= '" + reqQuery.getStartTime() +  "' AND start_time <= '" + reqQuery.getEndTime() + "' ) OR ( end_time >= '" + reqQuery.getStartTime() + "' AND end_time <= '" + reqQuery.getEndTime() + "' ))";
                sb.append(sql);
                sbCount.append(sql);
            }

        }
        sb.append("AND del_flag=0 ORDER BY status DESC, publish_time DESC ");

        int total = JT.queryForInt(sbCount.toString(), params.toArray());
        List<YcqYjsTestPaper> ycqYjsTestPapers = null;
        String dbType = JT.getDbType();
        // 判断数量
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            ycqYjsTestPapers = JT.queryForBeans(YcqYjsTestPaper.class, sb.toString(), params.toArray());
        } else {
            ycqYjsTestPapers = new ArrayList<>(0);
        }
        PageRsp<YcqYjsTestPaper> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(ycqYjsTestPapers);

        return page;
    }


    public PageRsp<YcqYjsTestPaper> paginRelQuery(PageReq<TestPaperCondition> req) {
        StringBuilder sb = new StringBuilder(PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        @Valid TestPaperCondition reqQuery = req.getQuery();
        List<Object> params = new ArrayList<>();
        if (req.getQuery()!=null) {
            // 标题title
            if (StringUtils.hasText(reqQuery.getTitle())) {
                sb.append(" AND title like concat('%',?,'%') ");
                sbCount.append(" AND title like concat('%',?,'%') ");
                params.add(reqQuery.getTitle());
            }

            // 所属业务编码
            if (StringUtils.hasText(reqQuery.getBusinessCode())) {
                sb.append(" AND business_code = ?");
                sbCount.append(" AND business_code = ? ");
                params.add(reqQuery.getBusinessCode());
            }

            // 试卷类型比编码
            if (StringUtils.hasText(reqQuery.getTypeCode())) {
                sb.append(" AND type_code = ?");
                sbCount.append(" AND type_code = ? ");
                params.add(reqQuery.getTypeCode());
            }

            // 启用禁用状态
            if (null != reqQuery.getEnableStatus()) {
                sb.append(" AND  status = ?");
                sbCount.append(" AND status = ? ");
                params.add(reqQuery.getEnableStatus());
            }

            // 考试开始时间
            if (StringUtils.hasText(reqQuery.getStartTime())) {
                sb.append(" AND start_time <= ? ");
                sbCount.append(" AND start_time  <= ? ");
                params.add(reqQuery.getStartTime());
            }

            // 考试结束时间
            if (StringUtils.hasText(reqQuery.getEndTime())) {
                sb.append(" AND end_time >= ? ");
                sbCount.append(" AND end_time  >= ? ");
                params.add(reqQuery.getEndTime());
            }

            if (CollectionUtils.isNotEmpty(reqQuery.getPaperIds())) {
                String paperIdsStr = reqQuery.getPaperIds().stream().map(x->"'" + x  + "'").collect(Collectors.joining(","));
                sb.append(" AND id NOT IN (" + String.join(",", paperIdsStr) + ") ");
                sbCount.append(" AND id NOT IN (" + String.join(",", paperIdsStr) + ") ");
            }

        }
        sb.append(" AND (( start_time <= '" +  reqQuery.getCurrentTime() + "' AND end_time >= '" + reqQuery.getCurrentTime() + "') OR is_exam = 0) AND status =1");     // 考试中或者习题
        sbCount.append(" AND (( start_time <= '" +  reqQuery.getCurrentTime() + "' AND end_time >= '" + reqQuery.getCurrentTime() + "') OR is_exam = 0)  AND status = 1");

        sb.append(" AND status = 1 AND del_flag=0 ORDER BY status DESC, publish_time DESC ");

        int total = JT.queryForInt(sbCount.toString(), params.toArray());
        List<YcqYjsTestPaper> ycqYjsTestPapers = null;
        String dbType = JT.getDbType();
        // 判断数量
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            ycqYjsTestPapers = JT.queryForBeans(YcqYjsTestPaper.class, sb.toString(), params.toArray());
        } else {
            ycqYjsTestPapers = new ArrayList<>(0);
        }
        PageRsp<YcqYjsTestPaper> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(ycqYjsTestPapers);

        return page;
    }

}
