package com.crunii.micro.service.studycenter.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestion;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestionReq;
import com.crunii.micro.service.studycenter.util.BaseJdbcTemplateUtil;
import com.crunii.micro.service.studycenter.util.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 试题表;
 *
 * @author : Charler
 * @date : 2023-9-21
 */
@Slf4j
@Repository
public class YcqYjsTestQuestionJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    public static final String DELETE_SQL = "Update YCQ_YJS_TEST_QUESTION a set a.del_flag=1 , a.UPDATED_TIME = ? where id = ? ";

    public static final String QUERY_SQL = "SELECT * FROM YCQ_YJS_TEST_QUESTION a where a.del_flag=0 and id = ? ";

    public static final String QUERY_CLASSFIYID_SQL = "SELECT * FROM YCQ_YJS_TEST_QUESTION a where a.del_flag=0 and topic_classify_id = ? ";

    private static final String QUERYIDS_SQL = "select * from YCQ_YJS_TEST_QUESTION a where a.del_flag=0 and id in ";

    private static final String QUERYDELIDS_SQL = "select * from YCQ_YJS_TEST_QUESTION a where a.del_flag=0 and id not in ";

    private static final String PAGE_SQL = "select * from YCQ_YJS_TEST_QUESTION a where 1=1 and a.del_flag=0 ";

    private static final String PAGE_COUNT_SQL = "select count(*) from YCQ_YJS_TEST_QUESTION a where 1=1 and a.del_flag=0";

    public static final String UPDATE_SQL = "Update YCQ_YJS_TEST_QUESTION a set a.topic_classify_name=?, " +
            " a.topic=? ,a.topic_type=? ,a.topic_classify_id=?  where a.id = ?";

    public static final String UPDATE_CLASSIFY_SQL = "Update YCQ_YJS_TEST_QUESTION a set a.topic_classify_name=?, " +
            " a.organize_id=? ,a.organize_name=?  where a.id = ?";

    public List<YcqYjsTestQuestion> searchAll() {
        String sql = "select * from YCQ_YJS_TEST_QUESTION where del_flag=0 ";
        // 查询数据集合
        List<YcqYjsTestQuestion> list = JT.query(sql, new BeanPropertyRowMapper<>(YcqYjsTestQuestion.class));
        return list;
    }

    /**
     * 分页查询题库相关信息
     * @param req
     * @return
     */
    public PageRsp<YcqYjsTestQuestion> pageQuery(PageReq<YcqYjsTestQuestionReq> req) {
        StringBuilder sb = new StringBuilder(PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);
        @Valid YcqYjsTestQuestionReq reqQuery = req.getQuery();
        List<Object> params = new ArrayList<>();

        if (reqQuery.getTopicClassifyId() != null) {
            sb.append(" and a.topic_classify_id = ?");
            sbCount.append(" and a.topic_classify_id = ? ");
            params.add(reqQuery.getTopicClassifyId());
        }
        // 分类名称
        if (StringUtils.hasText(reqQuery.getTopicClassifyName())) {
            sb.append(" and a.topic_classify_name like concat('%',?,'%') ");
            sbCount.append(" and a.topic_classify_name like concat('%',?,'%') ");
            params.add(reqQuery.getTopicClassifyName());
        }
        // 题目
        if (StringUtils.hasText(reqQuery.getTopic())) {
            sb.append(" and a.topic like concat('%',?,'%') ");
            sbCount.append(" and a.topic like concat('%',?,'%') ");
            params.add(reqQuery.getTopic());
        }
        // 类型
        if (reqQuery.getTopicType() != null) {
            sb.append(" and a.topic_type = ? ");
            sbCount.append(" and a.topic_type =? ");
            params.add(reqQuery.getTopicType());
        }
        // 业务id
        if (reqQuery.getOrganizeId() != null) {
            sb.append(" and a.organize_id = ? ");
            sbCount.append(" and a.organize_id = ? ");
            params.add(reqQuery.getOrganizeId());
        }

        sb.append(" and a.del_flag=0 order by a.CREATED_TIME desc ");

        int total = JT.queryForInt(sbCount.toString(), params.toArray());
        List<YcqYjsTestQuestion> yjsTestQuestions = null;
        String dbType = JT.getDbType();
        // 判断数量
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            yjsTestQuestions = JT.queryForBeans(YcqYjsTestQuestion.class, sb.toString(), params.toArray());
        } else {
            yjsTestQuestions = new ArrayList<>(0);
        }
        PageRsp<YcqYjsTestQuestion> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(yjsTestQuestions);

        return page;
    }


    /**
     * 插入数据
     * @param testQuestion
     * @return
     */
    public YcqYjsTestQuestion insert(YcqYjsTestQuestion testQuestion) {
        String insertsql = BaseJdbcTemplateUtil.insert(YcqYjsTestQuestion.class);
        Object[] objs = BaseJdbcTemplateUtil.getValues(testQuestion);
        PreparedStatementCreator create = new PreparedStatementCreator() {
            //利用传统的方法进行JDBC查询
            @Override
            public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
                //插入用户，返回主键
                PreparedStatement preparedStatement = connection.prepareStatement(insertsql,
                        PreparedStatement.RETURN_GENERATED_KEYS);
                for (int i = 0; i < objs.length; i++) {
                    preparedStatement.setObject(i + 1, objs[i]);
                }
                return preparedStatement;
            }
        };
        //2.创建keyHoldle
        GeneratedKeyHolder keyHolder = new GeneratedKeyHolder();
        JT.update(create, keyHolder);
        //获取生成的主键key
        Long key = keyHolder.getKey().longValue();
        BigInteger id = new BigInteger(String.valueOf(key));
        testQuestion.setId(id);
        return testQuestion;
    }


    /**
     * 更新试题信息
     * @param testQuestion
     */
    public void update(YcqYjsTestQuestion testQuestion) {
        testQuestion.setUpdatedTime(new Date());
        testQuestion.setUpdatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));

        try {
            JT.update(UPDATE_SQL, testQuestion.getTopicClassifyName(), testQuestion.getTopic(), testQuestion.getTopicType(), testQuestion.getTopicClassifyId(), testQuestion.getId());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 更新试题信息
     * @param testQuestion
     */
    public void updateByClassify(YcqYjsTestQuestion testQuestion) {
        testQuestion.setUpdatedTime(new Date());
        testQuestion.setUpdatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));

        try {
            JT.update(UPDATE_CLASSIFY_SQL, testQuestion.getTopicClassifyName(), testQuestion.getOrganizeId(), testQuestion.getOrganizeName(), testQuestion.getId());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 根据id删除试题信息
     * @param id
     */
    public void deleteById(BigInteger id) {
        JT.update(DELETE_SQL, new Date(), new Object[]{id});
    }

    /**
     * 根据ids批量删除试题信息
     * @param ids
     */
    public void deleteByIds(List<BigInteger> ids) {
        if (CollectionUtils.isNotEmpty(ids)) {
            // 构建动态的IN子句
            String inClause = ids.stream().map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining(", ", "(", ")")); // 生成"(1, 2, 3)"格式的子句
            String sql = "Update YCQ_YJS_TEST_QUESTION a set a.del_flag=1 , a.UPDATED_TIME = ? where id in  " + inClause;
            JT.update(sql, new Date());
        } else {
            log.info("无需删除试题数据");
        }
    }

    /**
     * 根据id查询试题表信息
     * @param id
     * @return
     */
    public YcqYjsTestQuestion queryById(BigInteger id) {
        YcqYjsTestQuestion testQuestion = JT.queryForBean(YcqYjsTestQuestion.class, QUERY_SQL, new Object[]{id});
        return testQuestion;
    }

    /**
     * 根据题目分类id查询
     * @param id
     * @return
     */
    public List<YcqYjsTestQuestion> queryByClassifyId(BigInteger id) {
        List<YcqYjsTestQuestion> testQuestions = JT.queryForBeans(YcqYjsTestQuestion.class, QUERY_CLASSFIYID_SQL, new Object[]{id});
        return testQuestions;
    }

    public List<YcqYjsTestQuestion> getTestQuestions(List<BigInteger> ids) {
        // 如果ids集合为空返回空
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptyList();
        } else {
            String inSql = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ")"));
            String sql = QUERYIDS_SQL + inSql;
            return JT.queryForBeans(YcqYjsTestQuestion.class, sql);
        }
    }

    public List<YcqYjsTestQuestion> getTestQuestionsByDelIds(YcqYjsTestQuestionReq req) {
        List<BigInteger> ids = req.getIds();
        // 如果ids集合为空返回空
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptyList();
        } else {
            StringBuilder sb = new StringBuilder(PAGE_SQL);
            List<Object> params = new ArrayList<>();
            // id集合
            if (!CollectionUtils.isEmpty(req.getIds())) {
                // 构建动态的IN子句
                String inSql = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ")"));
                sb.append(" and a.id not in  ").append(inSql);
            }
            // 业务id
            if (req.getOrganizeId() != null) {
                sb.append(" and a.organize_id = ? ");
                params.add(req.getOrganizeId());
            }
            // 题目
            if (StringUtils.hasText(req.getTopic())) {
                sb.append(" and a.topic = ? ");
                params.add(req.getTopic());
            }
            // 题目分类字典id
            if (req.getTopicClassifyId() != null) {
                sb.append(" and a.topic_classify_id = ? ");
                params.add(req.getTopicClassifyId());
            }
            // 题目类型（1单选题2多选题3判断题）
            if (req.getTopicType() != null) {
                sb.append(" and a.topic_type = ? ");
                params.add(req.getTopicType());
            }
            sb.append(" order by a.CREATED_TIME desc ");
            return JT.queryForBeans(YcqYjsTestQuestion.class, sb.toString(),params.toArray());
        }
    }
}
