package com.crunii.micro.service.studycenter.jdbc;

import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestion;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTopicOption;
import com.crunii.micro.service.studycenter.util.BaseJdbcTemplateUtil;
import com.crunii.micro.service.studycenter.util.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 试题表;
 *
 * @author : Charler
 * @date : 2023-9-21
 */
@Slf4j
@Repository
public class YcqYjsTopicOptionJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    public static final String DELETE_SQL ="Update YCQ_YJS_TOPIC_OPTION a set a.del_flag=1  where id = ? ";

    public static final String QUERY_SQL ="SELECT * FROM YCQ_YJS_TOPIC_OPTION a where a.del_flag=0 and id = ? ";

    public static final String QUERYTESTID_SQL ="SELECT * FROM YCQ_YJS_TOPIC_OPTION a where a.del_flag=0 and test_question_id = ?";

    public static final String QUERYTESTIDs_SQL ="SELECT * FROM YCQ_YJS_TOPIC_OPTION a where a.del_flag=0 and test_question_id in ";

    public static final String QUERYDELIDS_SQL ="SELECT * FROM YCQ_YJS_TOPIC_OPTION a where a.del_flag=0 and test_question_id not in ";


    public static final String UPDATE_SQL = "Update YCQ_YJS_TOPIC_OPTION a set a.OPTION=?, " +
            " a.option_content=? ,a.is_answer=?  where a.id = ?";



    public List<YcqYjsTopicOption> searchAll() {
        String sql = "select * from YCQ_YJS_TOPIC_OPTION where 1 = 1";
        // 查询数据集合
        List<YcqYjsTopicOption> list = JT.query(sql, new BeanPropertyRowMapper<>(YcqYjsTopicOption.class));
        return list;
    }


    public void batchInsert(List<YcqYjsTopicOption> list) {
        YcqYjsTopicOption topicOption = null;
        List<Object[]> listObj = new ArrayList<>();
        Object[] obj = null;
        for (int i = 0; i < list.size(); i++) {
            topicOption = list.get(i);
            obj = BaseJdbcTemplateUtil.getValues(topicOption);
            listObj.add(obj);
        }
        String insertsql = BaseJdbcTemplateUtil.insert(YcqYjsTopicOption.class);
        JT.batchUpdate(insertsql, listObj);
    }

    /**
     * 更新试题选项信息
     * @param topicOption
     */
    public void update(YcqYjsTopicOption topicOption) {
        topicOption.setUpdatedTime(new Date());
        topicOption.setUpdatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));
        try {
            JT.update(UPDATE_SQL, topicOption.getOption(), topicOption.getOptionContent(), topicOption.getIsAnswer(), topicOption.getId());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    public void deleteById(BigInteger id) {
        JT.update(DELETE_SQL, new Object[]{id});
    }

    public YcqYjsTopicOption queryById(BigInteger id) {
        YcqYjsTopicOption topicOption = JT.queryForBean(YcqYjsTopicOption.class, QUERY_SQL, new Object[]{id});
        return topicOption;
    }

    public List<YcqYjsTopicOption> queryByTestId(BigInteger id) {
        List<YcqYjsTopicOption> topicOptions = JT.queryForBeans(YcqYjsTopicOption.class, QUERYTESTID_SQL, new Object[]{id});
        return topicOptions;
    }

    public List<YcqYjsTopicOption> queryByTestIds(List<BigInteger> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptyList();
        } else {
            String inSql = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ")"));
            String sql = QUERYTESTIDs_SQL + inSql;
            return JT.queryForBeans(YcqYjsTopicOption.class, sql);
        }
    }

    public List<YcqYjsTopicOption> queryByDelIds(List<BigInteger> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptyList();
        } else {
            String inSql = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ")"));
            String sql = QUERYDELIDS_SQL + inSql + " order by a.CREATED_TIME desc";
            return JT.queryForBeans(YcqYjsTopicOption.class, sql);
        }
    }
}