package com.crunii.micro.service.studycenter.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsYearServiceTime;
import com.crunii.micro.service.studycenter.dto.vo.YearServiceTimeCondition;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 年度时长;
 *
 * @author : Charler
 * @date : 2023-9-21
 */
@Slf4j
@Repository
public class YcqYjsYearServiceTimeJdbcDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    private static final String PAGE_SQL = "select * from YCQ_YJS_YAER_SERVICETIME a where 1=1 ";

    private static final String PAGE_COUNT_SQL = "select count(*) from YCQ_YJS_YAER_SERVICETIME a where 1=1 ";

    public static final String QUERYIDS_SQL ="SELECT * FROM YCQ_YJS_YAER_SERVICETIME a where a.del_flag=0 and id in ";

    /**
     * 分页查询年度服务时长相关信息
     * @param req
     * @return
     */
    public PageRsp<YcqYjsYearServiceTime> pageQuery(PageReq<YearServiceTimeCondition> req) {
        YearServiceTimeCondition reqQuery = req.getQuery();
        StringBuilder sb = new StringBuilder(PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);

        List<Object> params = new ArrayList<>();
        // 身份类型
        if (StringUtils.hasText(reqQuery.getIdentityType())) {
            sb.append(" and a.identity_Type like concat('%',?,'%') ");
            sbCount.append(" and a.identity_Type like concat('%',?,'%') ");
            params.add(reqQuery.getIdentityType());
        }
        // 人员编号
        if (StringUtils.hasText(reqQuery.getPersonNumber())) {
            sb.append(" and a.person_number like concat('%',?,'%') ");
            sbCount.append(" and a.person_number like concat('%',?,'%') ");
            params.add(reqQuery.getPersonNumber());
        }

        // 人员姓名
        if (StringUtils.hasText(reqQuery.getPersonName())) {
            sb.append(" and a.person_name like concat('%',?,'%') ");
            sbCount.append(" and a.person_name like concat('%',?,'%') ");
            params.add(reqQuery.getPersonName());
        }

        // 手机号
        if (StringUtils.hasText(reqQuery.getPhoneNumber())) {
            sb.append(" and a.phone_number like concat('%',?,'%') ");
            sbCount.append(" and a.phone_number like concat('%',?,'%') ");
            params.add(reqQuery.getPhoneNumber());
        }
        // 年度服务时长
        if (StringUtils.hasText(reqQuery.getYearServiceTime())) {
            sb.append(" and a.year_service_time like concat('%',?,'%') ");
            sbCount.append(" and a.year_service_timeyear_service_time like concat('%',?,'%') ");
            params.add(reqQuery.getYearServiceTime());
        }

        // 开始时间
        if (StringUtils.hasText(reqQuery.getStartTime())) {
            sb.append(" and a.CREATED_TIME >= ?");
            sbCount.append(" and a.CREATED_TIME >= ?");
            params.add(reqQuery.getStartTime());
        }

        // 结束时间
        if (StringUtils.hasText(reqQuery.getEndTime())) {
            sb.append(" and a.CREATED_TIME <= ?");
            sbCount.append(" and a.CREATED_TIME <= ?");
            params.add(reqQuery.getEndTime());
        }
        sb.append(" and a.del_flag=0 order by a.CREATED_TIME desc ");

        int total = JT.queryForInt(sbCount.toString(), params.toArray());
        List<YcqYjsYearServiceTime> ycqYjsYearServiceTimes = null;
        String dbType = JT.getDbType();
        // 判断数量
        if (total > 0) {
            //根据数据库类型设置limit
            SqlUtil.appendLimit(dbType, params, sb, req);
            ycqYjsYearServiceTimes = JT.queryForBeans(YcqYjsYearServiceTime.class, sb.toString(), params.toArray());
        } else {
            ycqYjsYearServiceTimes = new ArrayList<>(0);
        }
        PageRsp<YcqYjsYearServiceTime> page = new PageRsp<>();
        page.setPageNo(req.getPageNo());
        page.setPageSize(req.getPageSize());
        page.setTotalNum(total);
        page.setResult(ycqYjsYearServiceTimes);

        return page;
    }

    public List<YcqYjsYearServiceTime> queryByIds(List<BigInteger> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptyList();
        } else {
            String inSql = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ")"));
            String sql = QUERYIDS_SQL + inSql + " order by a.CREATED_TIME desc";
            return JT.queryForBeans(YcqYjsYearServiceTime.class, sql);
        }
    }
}
