package com.crunii.micro.service.studycenter.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseMgr;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseMgrResp;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseMgrRspPage;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseTestRel;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsCourseMgrVo;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigInteger;
import java.util.List;


/**
 * 一件事-课程管理;(YCQ_YJS_COURSE_MGR)表服务接口
 * @author : http://www.chiner.pro
 * @date : 2023-9-20
 */
public interface YcqYjsCourseMgrService{
    /** 
     * 通过ID查询单条数据 
     *
     * @param id 主键
     * @return 实例对象
     */
    Result<YcqYjsCourseMgrResp> queryById(BigInteger id);
    /** 
     * 分页查询
     *
     * @param req 筛选条件
     * @return 查询结果
     */
    Result<PageRsp<YcqYjsCourseMgrRspPage>> paginQuery(PageReq<YcqYjsCourseMgrVo> req);
    /** 
     * 新增数据
     *
     * @param ycqYjsCourseMgrvo 实例对象
     * @return 实例对象
     */
    Result<Object> insert(YcqYjsCourseMgrVo ycqYjsCourseMgrvo);
    /** 
     * 更新数据
     *
     * @param ycqYjsCourseMgrvo 实例对象
     * @return 实例对象
     */
    Result<Object> update(YcqYjsCourseMgrVo ycqYjsCourseMgrvo);

    // 更改课程状态
    Result<Object> updateCourseStatus(YcqYjsCourseMgr ycqYjsCourseMgr);
    /** 
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    Result<Object> deleteById(BigInteger id);

    /**
     * 通过主键删除课程、试题试卷关联关系
     *
     * @param ycqYjsCourseTestRel 关联关系类
     * @return 是否成功
     */
    Result<Object> deleteYcqYjsCourseTestRel(YcqYjsCourseTestRel ycqYjsCourseTestRel);

    /**
     * 根据试卷id查询关联的课程信息
     *
     * @param ycqYjsCourseTestRel 关联关系类
     * @return 是否成功
     */
    Result<List<YcqYjsCourseMgr>> queryYcqYjsCourseByTestId(YcqYjsCourseTestRel ycqYjsCourseTestRel);

    /**
     * 上传文件
     *
     * @param file 文件
     * @return 是否成功
     */
    Result<Object> uploadCourseVideo(MultipartFile file, String fileNumber, String tableName);

    Result<Object> downloadCourseVideo(String fileNumber, String tableName);

    Result<Object> updateCourseVideoStatus(String fileId);

    Result<Object> queryRealTimeVideoAddress(String fileNumber, String tableName);

    Result<Object> courseNameCheck(YcqYjsCourseMgr ycqYjsCourseMgr);

    Result<Object> generateUploadAddress(String userName,String systemName,String originalVideo,String fileName);

    Result<Object> transcodingVideo(String userName, String uploadDate, String fileName);
}