package com.crunii.micro.service.studycenter.service.impl;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONObject;
import com.alibaba.druid.support.json.JSONUtils;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.studycenter.constants.YcqYjsCourseMgrConstants;
import com.crunii.micro.service.studycenter.dto.studycenter.*;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsCourseMgrVo;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsCourseMgrJdbcDao;

import com.crunii.micro.service.studycenter.jdbc.YcqYjsCourseTestRelDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsCoursesubMgrDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsUploadVideoDao;
import com.crunii.micro.service.studycenter.service.YcqYjsTestQuestionService;
import com.crunii.micro.service.studycenter.util.*;
import com.obs.services.ObsClient;
import com.obs.services.exception.ObsException;
import com.obs.services.model.PutObjectRequest;
import com.obs.services.model.PutObjectResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import com.crunii.micro.service.studycenter.service.YcqYjsCourseMgrService;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.*;
import java.math.BigInteger;
import java.net.InetAddress;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 一件事-课程管理;(YCQ_YJS_COURSE_MGR)表服务实现类
 * @author : http://www.chiner.pro
 * @date : 2023-9-20
 */
@Slf4j
@Service
public class YcqYjsCourseMgrServiceImpl implements YcqYjsCourseMgrService{

    @Value("${streaming.systemName:qqcqglxt}")
    private String systemName;

    @Value("${streaming.url:http://10.169.25.11:60600/}")
    private String url;

    @Value("${streaming.appKey:806823245}")
    private String appKey;

    @Value("${streaming.appSecret:tys27nfaedf5d4cah16c6a34mejg8fc118sddj3}")
    private String appSecret;

    @Value("${streaming.notifyUrl:http://10.169.25.5:11024/streaming/updateCourseVideoStatus}")
    private String notifyUrl;

    @Value("${upload.url}")
    private String urlPrefix;

    @Value("${obs.s3.accessKey}")
    private String accessKey;

    @Value("${obs.s3.accessSecret}")
    private String accessSecret;

    @Value("${obs.s3.bucketName}")
    private String bucketName;

    @Value("${obs.s3.endPoint}")
    private String endPoint;

    @Resource
    private YcqYjsCourseMgrJdbcDao ycqYjsCourseMgrJdbcDao;

    @Resource
    private YcqYjsCoursesubMgrDao ycqYjsCoursesubMgrDao;

    @Resource
    private YcqYjsCourseTestRelDao ycqYjsCourseTestRelDao;

    @Resource
    private YcqYjsUploadVideoDao ycqYjsUploadVideoDao;

    @Autowired
    private YcqYjsTestQuestionService testQuestionService;

    @Resource
    private ObsService obsService;

    @Autowired
    private Environment environment;

    @Override
    public Result<PageRsp<YcqYjsCourseMgrRspPage>> paginQuery(PageReq<YcqYjsCourseMgrVo> req) {
        PageRsp<YcqYjsCourseMgrRspPage> rsp = ycqYjsCourseMgrJdbcDao.paginQuery(req);
        return Result.ofSuccess(rsp);
    }

    @Override
    public Result<Object> insert(YcqYjsCourseMgrVo ycqYjsCourseMgrvo) {
        YcqYjsCourseMgr ycqYjsCourseMgr = BeanUtils.copyProperties(ycqYjsCourseMgrvo, YcqYjsCourseMgr.class);
        // 封面拼接地址
        String courseCover = ycqYjsCourseMgr.getCourseCover();
        if (StringUtils.hasText(courseCover)) {
            ycqYjsCourseMgr.setCourseCover(urlPrefix+courseCover);
        }
        // 上传附件地址
        String attachment = ycqYjsCourseMgr.getAttachment();
        if (StringUtils.hasText(attachment)) {
            ycqYjsCourseMgr.setAttachment(urlPrefix+attachment);
        }
        // 获取当前登陆人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (currWorker!=null) {
            ycqYjsCourseMgr.setCreatorId(currWorker.getLoginId());
            ycqYjsCourseMgr.setCreatorName(currWorker.getWorkerName());
        }

        // 如果有字课程则，获取子课程汇总时长,填充到课程时长
        Integer isSubcourse = ycqYjsCourseMgr.getIsSubcourse();
        List<YcqYjsCoursesubMgr> subList = ycqYjsCourseMgrvo.getSubList();
        if (isSubcourse== YcqYjsCourseMgrConstants.IS_SUBCOURSE_YES) {
            Integer courseDuration = 0;
            for (YcqYjsCoursesubMgr ycqYjsCoursesubMgr : subList) {
                courseDuration = courseDuration + ycqYjsCoursesubMgr.getCourseDuration();
            }
            ycqYjsCourseMgr.setCourseDuration(courseDuration);
        }

        // 先保存课程记录获取课程id后，再进行试卷、试题保存
        ycqYjsCourseMgr = ycqYjsCourseMgrJdbcDao.insert(ycqYjsCourseMgr);

        // 如果有试题
        Integer isTestquestions = ycqYjsCourseMgr.getIsTestquestions();
        if (isTestquestions== YcqYjsCourseMgrConstants.IS_TEST_QUESTIONS_YES) {
            List<YcqYjsCourseTestRel> courseTestList = ycqYjsCourseMgrvo.getCourseTestList();
            for (YcqYjsCourseTestRel ycqYjsCourseTestRel : courseTestList) {
                ycqYjsCourseTestRel.setCourseId(ycqYjsCourseMgr.getId());
            }
            // testType测试类型(1:选择试卷，2:选择试题)
            ycqYjsCourseTestRelDao.batchInsert(courseTestList);
        }

        // 如果有字课程则，需要保存子课程
        if (isSubcourse== YcqYjsCourseMgrConstants.IS_SUBCOURSE_YES) {
            for (YcqYjsCoursesubMgr ycqYjsCoursesubMgr : subList) {
                ycqYjsCoursesubMgr.setCourseId(ycqYjsCourseMgr.getId());
            }
            ycqYjsCoursesubMgrDao.batchInsert(subList);
        }

        return Result.ofSuccess(ycqYjsCourseMgr);
    }

    @Override
    public Result<Object>  update(YcqYjsCourseMgrVo ycqYjsCourseMgrvo) {
        // 封面拼接地址
        String courseCover = ycqYjsCourseMgrvo.getCourseCover();
        if (StringUtils.hasText(courseCover) && !courseCover.contains("http")) {
            ycqYjsCourseMgrvo.setCourseCover(urlPrefix+courseCover);
        }
        // 上传附件地址
        String attachment = ycqYjsCourseMgrvo.getAttachment();
        if (StringUtils.hasText(attachment) && !attachment.contains("http")) {
            ycqYjsCourseMgrvo.setAttachment(urlPrefix+attachment);
        }

        // 更新子课程信息,删除主课程关联的子课程
        List<YcqYjsCoursesubMgr> subList = ycqYjsCourseMgrvo.getSubList();
        ycqYjsCoursesubMgrDao.batchDeleteBycourseId(ycqYjsCourseMgrvo.getId());
        if (!CollectionUtils.isEmpty(subList)) {
            Integer courseDuration = 0;
            for (YcqYjsCoursesubMgr ycqYjsCoursesubMgr : subList) {
                ycqYjsCoursesubMgr.setCourseId(ycqYjsCourseMgrvo.getId());
                courseDuration = courseDuration + ycqYjsCoursesubMgr.getCourseDuration();
            }
            ycqYjsCoursesubMgrDao.batchInsert(subList);
            ycqYjsCourseMgrvo.setCourseDuration(courseDuration);
        }
        // 批量更新试卷、试题关联关系
        List<YcqYjsCourseTestRel> courseTestList = ycqYjsCourseMgrvo.getCourseTestList();
        if (!CollectionUtils.isEmpty(courseTestList)) {
            ycqYjsCourseTestRelDao.deleteBathCourseById(ycqYjsCourseMgrvo.getId());
            for (YcqYjsCourseTestRel ycqYjsCourseTestRel : courseTestList) {
                ycqYjsCourseTestRel.setCourseId(ycqYjsCourseMgrvo.getId());
            }
            // testType测试类型(1:选择试卷，2:选择试题)
            ycqYjsCourseTestRelDao.batchInsert(courseTestList);
        }
        YcqYjsCourseMgr ycqYjsCourseMgr = BeanUtils.copyProperties(ycqYjsCourseMgrvo, YcqYjsCourseMgr.class);
        ycqYjsCourseMgrJdbcDao.update(ycqYjsCourseMgr);
        return Result.ofSuccess(ycqYjsCourseMgrvo);
    }

    @Override
    public Result<Object> updateCourseStatus(YcqYjsCourseMgr ycqYjsCourseMgr) {
        int i = ycqYjsCourseMgrJdbcDao.updateCourseStatus(ycqYjsCourseMgr);
        return Result.ofSuccess(i);
    }

    @Override
    public Result<Object> deleteById(BigInteger id) {
        ycqYjsCourseMgrJdbcDao.deleteById(id);
        return Result.ofSuccess(id);
    }

    @Override
    public Result<Object> deleteYcqYjsCourseTestRel(YcqYjsCourseTestRel ycqYjsCourseTestRel) {
        ycqYjsCourseTestRelDao.deleteYcqYjsCourseTestRel(ycqYjsCourseTestRel);
        return Result.ofSuccess(ycqYjsCourseTestRel);
    }

    @Override
    public Result<List<YcqYjsCourseMgr>> queryYcqYjsCourseByTestId(YcqYjsCourseTestRel ycqYjsCourseTestRel) {
        List<YcqYjsCourseMgr> list = ycqYjsCourseTestRelDao.queryYcqYjsCourseByTestId(ycqYjsCourseTestRel);
        return Result.ofSuccess(list);
    }

    @Override
    public Result<YcqYjsCourseMgrResp> queryById(BigInteger id) {
        YcqYjsCourseMgrResp ycqYjsCourseMgrResp = ycqYjsCourseMgrJdbcDao.queryById(id);
        // YcqYjsCourseMgrResp ycqYjsCourseMgrResp = BeanUtils.copyProperties(ycqYjsCourseMgr, YcqYjsCourseMgrResp.class);
        // 查询子课程集合
        List<YcqYjsCoursesubMgrResp> subList = ycqYjsCoursesubMgrDao.queryCourseById(id);
        ycqYjsCourseMgrResp.setSubCourseList(subList);
        // 查询试卷、试题集合
        Integer testType = ycqYjsCourseMgrResp.getTestType();
        if (testType== YcqYjsCourseMgrConstants.IS_TEST_QUESTIONS_ONE) {
            List<YcqYjsTestPaper> list  = ycqYjsCourseTestRelDao.queryPaperByCourseId(id);
            ycqYjsCourseMgrResp.setPaperList(list);
        }
        if (testType== YcqYjsCourseMgrConstants.IS_TEST_QUESTIONS_TWO) {
            List<YcqYjsCourseTestRel> list = ycqYjsCourseTestRelDao.queryTestByCourseId(id);
            // 试题携带答案
            // 2.根据题目ids查询题目
            List<BigInteger> ids = list.stream().map(e-> e.getTestId()).collect(Collectors.toList());
            List<YcqYjsTestQuestionReq> testQuestionList =  testQuestionService.queryByIds(ids).getData();
            // 3.排序：单选题 多选题 判断题
            testQuestionList.sort(Comparator.comparing(x -> x.getTopicType()));
            ycqYjsCourseMgrResp.setTestQuestionList(testQuestionList);
        }
        Result<YcqYjsCourseMgrResp> result = new Result<>();
        result.setData(ycqYjsCourseMgrResp);
        return result;
    }

    @Override
    public Result<Object> uploadCourseVideo(MultipartFile file, String fileNumber, String tableName) {
        // 获取文件大小
        Long fileSize = file.getSize();
        // 上传文件构造的地址
        String userName = "admin";
        // 获取当前登陆人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (currWorker!=null) {
            userName = currWorker.getLoginCode();
        }
        Date day=new Date();
        SimpleDateFormat sdf= new SimpleDateFormat("yyyyMMdd");
        String uploadDate = sdf.format(day);
        String fileName= file.getOriginalFilename();

//        String uploadPath=systemName+"/"+userName+"/"+uploadDate+"/originalVideo/"+fileName;
//        // 获取预上传地址
//
//        Map<String, Object> map = UploadFileUtil.signUrl(accessKey, accessSecret, endPoint ,bucketName, uploadPath, 60 * 24 * 1 );
//        String actionUrl= map.get("fileUrl").toString();
//        System.out.println("上传地址为:"+actionUrl);
//        // 上传文件
//        String resultUpload = UploadFileUtil.uploadFile(actionUrl, file);
        try {
            postObs(file);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
//        log.info("上传视频fileNumber:{},上传文件大小:{},上传结果:{}",fileNumber, fileSize,resultUpload);

        // 通知流媒体转码
        Map requestMap=new HashMap();
        requestMap.put("systemName",systemName);
        requestMap.put("userName" ,userName);
        requestMap.put("uploadDate" ,uploadDate);
        requestMap.put("fileName" ,fileName);

        if (StringUtils.isEmpty(notifyUrl)) {
            try{
                String ip = InetAddress.getLocalHost().getHostAddress();
                String port = environment.getProperty("server.port");
                String environmentName = environment.getProperty("spring.application.name") ;
                notifyUrl = "http://"+ip+":"+port+"/api/"+environmentName+"/streaming/updateCourseVideoStatus";

            }catch (Exception e) {
                log.error("获取环境信息失败:{}", e.getMessage());
            }
        }
        requestMap.put("notifyURL", notifyUrl);

        log.info("请求参数:{}", JsonUtil.getJsonString(requestMap));
        String curTime = String.valueOf(new Date().getTime());
        String nonce = HttpClientUtil.getStringRandom(22);
        String checkSum = HttpClientUtil.getCheckSum(nonce, appSecret, curTime);
        requestMap.put("appKey", appKey);
        requestMap.put("nonce", nonce);
        requestMap.put("curTime", curTime);
        requestMap.put("checkNum", checkSum);
        String returnStr = HttpClientUtil.doPost(url+UploadFileUtil.UPLOAD_FILENOTIFICATION,requestMap);
        log.info("上传视频fileNumber:{},流媒体响应结果:{}",fileNumber, returnStr);
        //String returnStr1 = "{'code':'0','msg':'OK','fileID':'78229307D3D6F222','pictureUrl':'http://102.2.12.224:18080/sjzl/78229307D3D6F222_720p.mp4?token=fajfaggfajfgjadfgadfdaj'}";
        JSONObject json=new JSONObject(returnStr);
        YcqYjsUploadVideo ycqYjsUploadVideo=new YcqYjsUploadVideo();
        String msg =json.getStr("msg");
        if ("OK".equals(msg)) {
            String fileID = json.getStr("fileID");
            String pictureUrl = json.getStr("pictureUrl");
            String fileDuration = json.getStr("fileDuration");
            // 创建视频信息转码表记录
            ycqYjsUploadVideo.setFileNumber(fileNumber);
            ycqYjsUploadVideo.setTableName(tableName);
            ycqYjsUploadVideo.setFileName(fileName);
            String[] fileArr=fileName.split("\\.");
            ycqYjsUploadVideo.setFileType(null);
            if (fileArr.length==2) {
                ycqYjsUploadVideo.setFileType(fileArr[1]);
            }

            ycqYjsUploadVideo.setFileSize(new BigInteger(fileSize.toString()));
            ycqYjsUploadVideo.setFileStatus(1);

            ycqYjsUploadVideo.setFileId(fileID);
            ycqYjsUploadVideo.setSystemName(systemName);
            ycqYjsUploadVideo.setUserName(userName);
            ycqYjsUploadVideo.setUploadDate(uploadDate);
            // 获取视频时长
            ycqYjsUploadVideo.setFileDuration(fileDuration);
            // 获取视频封面
            pictureUrl = pictureUrl.replaceAll(url, "/");
            ycqYjsUploadVideo.setPictureUrl(pictureUrl);
            // 保存前先删除
            ycqYjsUploadVideoDao.delete(ycqYjsUploadVideo.getFileNumber(),ycqYjsUploadVideo.getTableName());
            ycqYjsUploadVideo=ycqYjsUploadVideoDao.insert(ycqYjsUploadVideo);
        } else {
            return Result.ofFail(returnStr);
        }

        return Result.ofSuccess(ycqYjsUploadVideo);
    }

    private void postObs(MultipartFile file) throws IOException {
        String userName = "admin";
        // 获取当前登陆人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (currWorker!=null) {
//            userName = currWorker.getWorkerName();
            userName = currWorker.getLoginCode();
        }
        Date day=new Date();
        SimpleDateFormat sdf= new SimpleDateFormat("yyyyMMdd");
        String uploadDate = sdf.format(day);
        String fileName= file.getOriginalFilename();

        String objectKey=systemName+"/"+userName+"/"+uploadDate+"/originalVideo/"+fileName;

        ObsClient obsClient = new ObsClient(accessKey, accessSecret, endPoint);
        File fileFile = transferToFile(file);
        PutObjectRequest request = new PutObjectRequest (bucketName, objectKey, fileFile);

        try {
            PutObjectResult result = obsClient.putObject(request);
            System.out.println("Upload successful! ETag: " + result.getEtag());
        } catch (ObsException e) {
            System.out.println("Error uploading file: " + e.getErrorCode() + " - " + e.getErrorMessage());
        } finally {
            obsClient.close();
            File del = new File(fileFile.toURI());
            del.delete();
        }
    }

    public File transferToFile(MultipartFile multipartFile) {

        if (multipartFile.equals("") || multipartFile.getSize() <= 0) {
            multipartFile = null;
            return null;
        }
    //MultipartFile转换为File
        File file = null;
        InputStream ins = null;
        try {
            ins = multipartFile.getInputStream();
            file = new File(multipartFile.getOriginalFilename());
            OutputStream os = new FileOutputStream(file);
            int bytesRead = 0;
            byte[] buffer = new byte[8192];
            while ((bytesRead = ins.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            ins.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
//将本地生成的文件删除
//        File del = new File(file.toURI());
//        del.delete();
        return file;
    }

    @Override
    public Result<Object> downloadCourseVideo(String fileNumber, String tableName) {
        // 根据fileNumber、tableName获取信息表记录
        YcqYjsUploadVideo ycqYjsUploadVideo=ycqYjsUploadVideoDao.query(fileNumber, tableName);
        String uploadPath=ycqYjsUploadVideo.getSystemName()+"/"+ycqYjsUploadVideo.getUserName()+"/"+ycqYjsUploadVideo.getUploadDate()+"/originalVideo/"+ycqYjsUploadVideo.getFileName();
        Map<String, Object> returnMap = UploadFileUtil.downloadUrl(accessKey, accessSecret , endPoint,bucketName , uploadPath, 60 * 24 * 7 );
        return Result.ofSuccess(JSONUtils.toJSONString(returnMap));
    }

    @Override
    public Result<Object> updateCourseVideoStatus(String fileId) {
        ycqYjsUploadVideoDao.updateStatus(fileId);
        return Result.ofSuccess();
    }

    @Override
    public Result<Object> queryRealTimeVideoAddress(String fileNumber, String tableName) {
        // 根据fileNumber、tableName获取信息表记录
        YcqYjsUploadVideo ycqYjsUploadVideo=ycqYjsUploadVideoDao.query(fileNumber, tableName);
        if (ycqYjsUploadVideo == null) {
            return Result.ofSuccessMsg(YcqYjsCourseMgrConstants.FILE_STATUS_MGS_ONE);
        }
        // 获取视频转化状态
        Integer fileStatus = ycqYjsUploadVideo.getFileStatus();
        if (YcqYjsCourseMgrConstants.IS_FILE_STATUS_ONE== fileStatus) {
            return Result.ofSuccessMsg(YcqYjsCourseMgrConstants.FILE_STATUS_MGS_TWO);
        }
        // 获取实时播放地址
        Map requestMap=new HashMap();
        requestMap.put("systemName",systemName);
        requestMap.put("userName" ,ycqYjsUploadVideo.getUserName());
        requestMap.put("uploadDate" ,ycqYjsUploadVideo.getUploadDate());
        requestMap.put("fileID" ,ycqYjsUploadVideo.getFileId());
        requestMap.put("fileType" , "1");

        String curTime = String.valueOf(new Date().getTime());
        String nonce = HttpClientUtil.getStringRandom(22);
        String checkSum = HttpClientUtil.getCheckSum(nonce, appSecret, curTime);

        requestMap.put("appKey", appKey);
        requestMap.put("nonce", nonce);
        requestMap.put("curTime", curTime);
        requestMap.put("checkNum", checkSum);
        String returnStr = HttpClientUtil.doPost(url+UploadFileUtil.FILE_URL,requestMap);
        try {
            JSONObject jsonObject =JsonUtil.jsonToObject(returnStr, JSONObject.class);
            String fileUrl = jsonObject.getStr("fileUrl");
            fileUrl = fileUrl.replaceAll(url, "/");
            jsonObject.set("fileUrl",fileUrl);
            return Result.ofSuccess(jsonObject);
        } catch (Exception e) {
            log.info("获取实时视频失败:{}", e.getMessage());
        }
        return Result.ofSuccess(returnStr);
    }

    @Override
    public Result<Object> courseNameCheck(YcqYjsCourseMgr ycqYjsCourseMgr) {
        int i = ycqYjsCourseMgrJdbcDao.courseNameCheck(ycqYjsCourseMgr.getCourseName());
        if (i!=0) {
            return Result.ofSuccess(false);
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Object> generateUploadAddress(String userName,String systemName,String originalVideo,String fileName) {
        // 上传文件构造的地址

        java.util.Date day=new Date();
        SimpleDateFormat sdf= new SimpleDateFormat("yyyyMMdd");
        String uploadDate = sdf.format(day);
        String uploadPath=systemName+"/"+userName+"/"+uploadDate+"/"+originalVideo+"/"+fileName;
        // 获取预上传地址
        Map<String, Object> map = UploadFileUtil.signUrl(accessKey, accessSecret , endPoint,bucketName , uploadPath, 60 * 24 * 7 );
        String actionUrl= map.get("fileUrl").toString();
        System.out.println("上传地址为:"+actionUrl);
        return Result.ofSuccess(actionUrl);
    }

    @Override
    public Result<Object> transcodingVideo(String userName, String uploadDate, String fileName) {
        Map requestMap=new HashMap();
        requestMap.put("systemName",systemName);
        requestMap.put("userName" ,userName);
        requestMap.put("uploadDate" ,uploadDate);
        requestMap.put("fileName" ,fileName);
        String curTime = String.valueOf(new Date().getTime());
        String nonce = HttpClientUtil.getStringRandom(22);
        String checkSum = HttpClientUtil.getCheckSum(nonce, appSecret, curTime);
        requestMap.put("appKey", appKey);
        requestMap.put("nonce", nonce);
        requestMap.put("curTime", curTime);
        requestMap.put("checkNum", checkSum);
        if (StringUtils.isEmpty(notifyUrl)) {
            try{
                String ip = InetAddress.getLocalHost().getHostAddress();
                String port = environment.getProperty("server.port");
                String environmentName = environment.getProperty("spring.application.name") ;
                notifyUrl = "http://"+ip+":"+port+"/api/"+environmentName+"/streaming/updateCourseVideoStatus";

            }catch (Exception e) {
                log.error("获取环境信息失败:{}", e.getMessage());
            }
        }
        requestMap.put("notifyURL", notifyUrl);
        String returnStr = HttpClientUtil.doPost(url+UploadFileUtil.UPLOAD_FILENOTIFICATION,requestMap);
        System.out.println("流媒体响应结果:"+returnStr);
        return Result.ofSuccess(returnStr);
    }

}