package com.crunii.micro.service.studycenter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.DesensitizedUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.builder.ExcelWriterBuilder;
import com.alibaba.excel.write.builder.ExcelWriterSheetBuilder;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.studycenter.common.base.ExamplePredicate;
import com.crunii.micro.service.studycenter.common.base.FieldRestriction;
import com.crunii.micro.service.studycenter.common.base.TimestampRangePredicate;
import com.crunii.micro.service.studycenter.constants.StudyCenterConstants;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsNegativePerson;
import com.crunii.micro.service.studycenter.dto.vo.NegativePersonCondition;
import com.crunii.micro.service.studycenter.dto.vo.NegativePersonExcel;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsNegativePersonVo;
import com.crunii.micro.service.studycenter.enums.DelFlagEnum;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsNegativePersonDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsNegativePersonJdbcDao;
import com.crunii.micro.service.studycenter.service.YcqYjsNegativePersonService;
import com.crunii.micro.service.studycenter.util.BeanCopyUtil;
import com.crunii.micro.service.studycenter.util.CharacterCodeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 负面人员清单表;
 *
 * @author : Charler
 * @date : 2023-9-21
 */
@Slf4j
@Service
public class YcqYjsNegativePersonServiceImpl implements YcqYjsNegativePersonService {

    @Autowired
    private YcqYjsNegativePersonDao negativePersonDao;

    @Resource
    private YcqYjsNegativePersonJdbcDao negativePersonJdbcDao;


    private final static String CODE = "FMRY";

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createNegativePerson(YcqYjsNegativePersonVo createReq) {
        // 身份证号查询如果不为空  存在  不允许新增
        /*YcqYjsNegativePerson ycqYjsNegativePerson = negativePersonDao.getUniqueByProp("idNumber", createReq.getIdNumber());
        if (ycqYjsNegativePerson != null) {
            throw new BusinessException("此身份证号已存在，无法新增！");
        }*/
            if (negativePersonDao.countByIdNumberAndDelFlag(createReq.getIdNumber(), DelFlagEnum.UN_DELETE.getCode()) > 0 ){
            throw new BusinessException("此身份证号已存在，无法新增！");
        }
        // 1.新增负面人员基本信息
        YcqYjsNegativePerson negativePerson = new YcqYjsNegativePerson();
        BeanUtil.copyProperties(createReq, negativePerson);
        // 人员编号生成规则：FMRY开头
        String generateCode = CharacterCodeUtil.generateCharacterCode(CODE);
        negativePerson.setPersonNumber(generateCode);
        negativePerson.setDelFlag(StudyCenterConstants.COMMON_NO);
        if (createReq.getCreatedBy() == null && CurrWorkerUtil.getCurrWorker() != null) {
            negativePerson.setCreatedBy(new BigInteger(String.valueOf((CurrWorkerUtil.getCurrWorker().getLoginId()))));
        }
        negativePerson.setCreatedTime(new Date());
        negativePersonDao.saveAndFlush(negativePerson);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delNegativePerson(BigInteger id) {
        YcqYjsNegativePerson negativePerson = negativePersonDao.getById(id);
        if (null == negativePerson) {
            throw new BusinessException("负面人员信息不存在");
        }
        // 删除试卷
        negativePerson.setDelFlag(CommonConstants.LOG_FLAG_YES);
        negativePersonDao.saveAndFlush(negativePerson);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateNegativePerson(YcqYjsNegativePersonVo updateReq) {
        if (updateReq.getId() == null) {
            throw new BusinessException("id为空，不能修改");
        }
        YcqYjsNegativePerson negativePerson = negativePersonDao.getById(updateReq.getId());
        if (null == negativePerson) {
            throw new BusinessException("负面人员信息不存在");
        }
        // 判断脱敏后是否修改
        // 手机号
        if (updateReq.getPhoneNumber().equals(DesensitizedUtil.mobilePhone(negativePerson.getPhoneNumber()))) {
            updateReq.setPhoneNumber(negativePerson.getPhoneNumber());
        }
        // 身份证
        if (updateReq.getIdNumber().equals(DesensitizedUtil.mobilePhone(negativePerson.getIdNumber()))) {
            updateReq.setIdNumber(negativePerson.getIdNumber());
        }
        BeanUtils.copyProperties(updateReq, negativePerson,"id","personNumber","dataSource","delFlag","createdBy","createdTime");
        if (updateReq.getUpdatedBy() == null && CurrWorkerUtil.getCurrWorker() != null) {
            negativePerson.setUpdatedBy(new BigInteger(String.valueOf((CurrWorkerUtil.getCurrWorker().getLoginId()))));
        }
        negativePerson.setUpdatedTime(new Date());
        negativePersonDao.saveAndFlush(negativePerson);
    }

    @Override
    public YcqYjsNegativePerson getNegativePersonInfo(BigInteger id) {
        // 1.负面人员信息
        return negativePersonDao.getById(id);
    }

    /**
     *  分页查询所有负面人员
     * @param req
     * @return
     */
    @Override
    public PageRsp<YcqYjsNegativePerson> pageYcqYjsNegativePerson(PageReq<NegativePersonCondition> req) {
        return negativePersonJdbcDao.pageQuery(req);
    }



    @Override
    public PageRsp<YcqYjsNegativePersonVo> pageSearchNegativePerson(PageReq<NegativePersonCondition> pageReq) {
        // 条件参数
        NegativePersonCondition pageReqQuery = pageReq.getQuery();
        YcqYjsNegativePerson negativePerson = new YcqYjsNegativePerson();
        negativePerson.setIdentityType(StringUtils.isNotEmpty(pageReqQuery.getIdentityType()) ? pageReqQuery.getIdentityType() : null);
        negativePerson.setPersonNumber(StringUtils.isNotEmpty(pageReqQuery.getPersonNumber()) ? pageReqQuery.getPersonNumber() : null);
        negativePerson.setPersonName(StringUtils.isNotEmpty(pageReqQuery.getPersonName()) ? pageReqQuery.getPersonName() : null);
        negativePerson.setPhoneNumber(StringUtils.isNotEmpty(pageReqQuery.getPhoneNumber()) ? pageReqQuery.getPhoneNumber() : null);
        negativePerson.setNegativeReason(StringUtils.isNotEmpty(pageReqQuery.getNegativeReason()) ? pageReqQuery.getNegativeReason() : null);
        negativePerson.setIdNumber(StringUtils.isNotEmpty(pageReqQuery.getIdNumber()) ? pageReqQuery.getIdNumber() : null);
        negativePerson.setDelFlag(CommonConstants.LOG_FLAG_NO);

        // 条件参数匹配原则  exact精确查询  contains模糊查询
        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("identityType",ExampleMatcher.GenericPropertyMatchers.exact())
                .withMatcher("personNumber",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("personName",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("phoneNumber",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("negativeReason",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("delFlag",ExampleMatcher.GenericPropertyMatchers.exact())
                .withMatcher("idNumber",ExampleMatcher.GenericPropertyMatchers.contains())
                .withIgnoreNullValues();
        Example example = Example.of(negativePerson,exampleMatcher);

        // 时间范围参数
        List<FieldRestriction<YcqYjsNegativePerson>> fieldRestrictions = new ArrayList<>();
        Date startTime = StringUtils.isNoneBlank(pageReqQuery.getStartTime()) ? DateUtil.parse(pageReqQuery.getStartTime() + " 00:00:00", DateUtil.defaultDTPattern) : null;
        Date endTime = StringUtils.isNoneBlank(pageReqQuery.getEndTime()) ? DateUtil.parse(pageReqQuery.getEndTime() + " 23:59:59", DateUtil.defaultDTPattern) : null;
        if (null != startTime) {
            fieldRestrictions.add(new TimestampRangePredicate<YcqYjsNegativePerson>("createdTime", startTime.getTime(), endTime.getTime()));
        }
        ExamplePredicate examplePredicate = ExamplePredicate.of(example, fieldRestrictions);

        // 分页参数
        Sort sort = Sort.by(Sort.Direction.DESC, "createdTime");
        int pageNumber = pageReq.getPageNo() <= 1 ? 0 : pageReq.getPageNo() - 1;
        PageRequest pageRequest = PageRequest.of(pageNumber, pageReq.getPageSize(), sort);
        Page<YcqYjsNegativePerson> page = negativePersonDao.findAll(examplePredicate, pageRequest);

        PageRsp<YcqYjsNegativePersonVo> pageRsp = new PageRsp();
        pageRsp.setPageNo(pageReq.getPageNo());
        pageRsp.setPageSize(pageReq.getPageSize());
        pageRsp.setTotalNum(page.getTotalElements());

        if (CollectionUtils.isEmpty(page.getContent())){
            pageRsp.setResult(new ArrayList<>());
            return pageRsp;
        }

        pageRsp.setResult(page.getContent().parallelStream().map(x->{
            YcqYjsNegativePersonVo vo = new YcqYjsNegativePersonVo();
            BeanUtil.copyProperties(x, vo);
            return vo;
        }).collect(Collectors.toList()));

        return pageRsp;

    }


    /**
     * 负面人员信息excel导出
     *
     * @param response
     * @param req
     */
    @Override
    public void exportNegativePersonExcel(HttpServletResponse response, NegativePersonCondition req) {
        try {
            log.info("YcqYjsNegativePersonServiceImpl.exportNegativePersonExcel.param：{}", req.toString());
            List<YcqYjsNegativePerson> negativeList = negativePersonJdbcDao.queryByIds(req.getIds());
            String fileName = "负面人员信息";
            export(response, fileName, negativeList);
        } catch (BusinessException e) {
            log.error("YcqYjsNegativePersonServiceImpl.exportNegativePersonExcel.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsNegativePersonServiceImpl.exportNegativePersonExcel.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export(HttpServletResponse response, String fileName, List<YcqYjsNegativePerson> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=" + encodedFileName + ".xlsx");
            ExcelWriterBuilder write = EasyExcel.write(response.getOutputStream(), NegativePersonExcel.class);
            ExcelWriterSheetBuilder sheet1 = write.sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy());
            List<NegativePersonExcel> negativePersonExcelList = BeanCopyUtil.convert(dataList, NegativePersonExcel.class);
            sheet1.doWrite(negativePersonExcelList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("YcqYjsNegativePersonServiceImpl.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 自定义Excel导出策略，设置表头和数据行的字体和高度
     *
     * @return Excel导出策略
     */
    private HorizontalCellStyleStrategy getHeightAndFontStrategy() {
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 11);
        headWriteFont.setBold(true);
        headWriteCellStyle.setWriteFont(headWriteFont);
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
        WriteFont contentWriteFont = new WriteFont();
        contentWriteFont.setFontHeightInPoints((short) 11);
        contentWriteCellStyle.setWriteFont(contentWriteFont);
        contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
    }

}
