package com.crunii.micro.service.studycenter.service.impl;


import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.studycenter.dto.studycenter.*;
import com.crunii.micro.service.studycenter.dto.vo.TreeNodeVo;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsCouresSpecialVo;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsSpecialMgrVo;

import com.crunii.micro.service.studycenter.jdbc.YcqYjsCouresSpecialJdbcDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsCourseMgrJdbcDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsSpecialMgrDao;
import com.crunii.micro.service.studycenter.service.YcqYjsSpecialMgrService;

import com.crunii.micro.service.studycenter.util.BeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 一件事-专题管理;(YCQ_YJS_SPECIAL_MGR)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2023-9-21
 */
@Service
public class YcqYjsSpecialMgrServiceImpl implements YcqYjsSpecialMgrService {

    @Resource
    private YcqYjsSpecialMgrDao ycqYjsSpecialMgrDao;

    @Resource
    private YcqYjsCouresSpecialJdbcDao ycqYjsCouresSpecialJdbcDao;

    @Resource
    private YcqYjsCourseMgrJdbcDao ycqYjsCourseMgrJdbcDao;


    @Override
    public Result<YcqYjsSpecialMgrDto> queryById(BigInteger id) {
        YcqYjsSpecialMgrDto ycqYjsSpecialMgrDto = ycqYjsSpecialMgrDao.queryById(id);
//        YcqYjsSpecialMgrDto ycqYjsSpecialMgrDto = BeanUtils.copyProperties(ycqYjsSpecialMgr, YcqYjsSpecialMgrDto.class);
        List<YcqYjsCourseMgrRsp> list = ycqYjsSpecialMgrDao.queryYcqYjsCourseMgrBySpecialId(id);
        ycqYjsSpecialMgrDto.setCourseMgrList(list);
        Result<YcqYjsSpecialMgrDto> result = new Result<>();
        result.setData(ycqYjsSpecialMgrDto);
        return result;
    }

    @Override
    public Result<PageRsp<YcqYjsSpecialMgrDto>> paginQuery(PageReq<YcqYjsSpecialMgrVo> req) {
        PageRsp<YcqYjsSpecialMgrDto> rsp = ycqYjsSpecialMgrDao.paginQuery(req);
        return Result.ofSuccess(rsp);
    }

    @Override
    public Result<Object> insert(YcqYjsSpecialMgrVo ycqYjsSpecialMgrVo) {
        YcqYjsSpecialMgr ycqYjsSpecialMgr = BeanUtils.copyProperties(ycqYjsSpecialMgrVo, YcqYjsSpecialMgr.class);
        ycqYjsSpecialMgr = ycqYjsSpecialMgrDao.insert(ycqYjsSpecialMgr);

        // 获取专题主键id
        BigInteger specialId = ycqYjsSpecialMgr.getId();
        // 保存专题和课程关联关系
        List<BigInteger> courseIds = ycqYjsSpecialMgrVo.getCourseIds();
        // 批量新增专题关联的课程
        List<YcqYjsCouresSpecial> list = new ArrayList<>();
        YcqYjsCouresSpecial ycqYjsCouresSpecial = null;
        if (CollectionUtils.isNotEmpty(courseIds)) {
            for (BigInteger courseId : courseIds) {
                ycqYjsCouresSpecial = new YcqYjsCouresSpecial();
                ycqYjsCouresSpecial.setCourseId(courseId);
                ycqYjsCouresSpecial.setSpecialId(specialId);
                ycqYjsCouresSpecial.setCreateTime(new Date());
                list.add(ycqYjsCouresSpecial);
            }
            ycqYjsCouresSpecialJdbcDao.batchInsert(list);
        }
        return Result.ofSuccess(ycqYjsSpecialMgr);
    }

    @Override
    public Result<Object> update(YcqYjsSpecialMgrVo ycqYjsSpecialMgrVo) {
        YcqYjsSpecialMgr ycqYjsSpecialMgr = BeanUtils.copyProperties(ycqYjsSpecialMgrVo, YcqYjsSpecialMgr.class);
        ycqYjsSpecialMgrDao.update(ycqYjsSpecialMgr);
        // 更新专题和课程关联关系
        List<BigInteger> courseIds =ycqYjsSpecialMgrVo.getCourseIds();
        List<YcqYjsCouresSpecial> list = new ArrayList<>();
        YcqYjsCouresSpecial ycqYjsCouresSpecial = null;
        if (CollectionUtils.isNotEmpty(courseIds)) {
            // 先删除，后新增
            ycqYjsCouresSpecialJdbcDao.deleteSpecialId(ycqYjsSpecialMgr.getId());
            for (BigInteger courseId : courseIds) {
                ycqYjsCouresSpecial = new YcqYjsCouresSpecial();
                ycqYjsCouresSpecial.setCourseId(courseId);
                ycqYjsCouresSpecial.setSpecialId(ycqYjsSpecialMgr.getId());
                ycqYjsCouresSpecial.setCreateTime(new Date());
                list.add(ycqYjsCouresSpecial);
            }
            ycqYjsCouresSpecialJdbcDao.batchInsert(list);
        }
        return Result.ofSuccess(ycqYjsSpecialMgr);
    }

    @Override
    public Result<Object> deleteById(BigInteger id) {
        ycqYjsSpecialMgrDao.deleteById(id);
        return Result.ofSuccess(id);
    }

    @Override
    public Result<Object> createRelationship(YcqYjsCouresSpecialVo YcqYjsCouresSpecialvo) {
        // 课程id集合
        List<BigInteger> courseIds = YcqYjsCouresSpecialvo.getCourseIds();
        // 专题id
        BigInteger specialId = YcqYjsCouresSpecialvo.getSpecialId();
        // 先清除专题关联的所有课程
        ycqYjsCouresSpecialJdbcDao.deleteSpecialId(specialId);
        // 批量新增专题关联的课程
        List<YcqYjsCouresSpecial> list = new ArrayList<>();
        YcqYjsCouresSpecial ycqYjsCouresSpecial = null;
        for (BigInteger courseId : courseIds) {
            ycqYjsCouresSpecial = new YcqYjsCouresSpecial();
            ycqYjsCouresSpecial.setCourseId(courseId);
            ycqYjsCouresSpecial.setSpecialId(specialId);
            ycqYjsCouresSpecial.setCreateTime(new Date());
            list.add(ycqYjsCouresSpecial);
        }
        ycqYjsCouresSpecialJdbcDao.batchInsert(list);
        return Result.ofSuccess(YcqYjsCouresSpecialvo);
    }

    @Override
    public Result<Object> relatedCourses() {
        Map<Integer,List<YcqYjsCourseMgrResp>> returnMap = ycqYjsCourseMgrJdbcDao.queryCourseType();
        List<TreeNodeVo> treeNodeVoList = new ArrayList<>();
        returnMap.entrySet().stream().forEach(entry -> {
            TreeNodeVo treeNodeVo = new TreeNodeVo();
            Integer key = entry.getKey();
            List<Map<String, Object>> value = transform(entry.getValue());
            treeNodeVo.setChildren(value);
            treeNodeVo.setKey(key.toString());
            treeNodeVo.setValue(key.toString());
            // 课程分类名称
            String courseTypeName = entry.getValue().get(0).getCourseTypeName();
            treeNodeVo.setTitle(courseTypeName);
            treeNodeVoList.add(treeNodeVo);
        });
        return Result.ofSuccess(treeNodeVoList);
    }

    @Override
    public List<YcqYjsSpecialMgr> querySpecialInfoList() {
        List<YcqYjsSpecialMgr> list=ycqYjsSpecialMgrDao.querySpecialInfoList();
        return list;
    }

    @Override
    public Result<List<YcqYjsCourseMgrResp>> queryCourseInfoBySpecialType(YcqYjsSpecialMgr ycqYjsSpecialMgr) {
        List<YcqYjsCourseMgrResp> list =ycqYjsSpecialMgrDao.queryCourseInfoBySpecialType(ycqYjsSpecialMgr);
        return Result.ofSuccess(list);
    }

    @Override
    public Result<Object> queryAppVideoList(YcqYjsSpecialMgr ycqYjsSpecialMgr) {
        List<YcqYjsSpecialMgr> list = ycqYjsSpecialMgrDao.queryAppVideoList(ycqYjsSpecialMgr);
        return Result.ofSuccess(list);
    }

    private List<Map<String, Object>> transform(List<YcqYjsCourseMgrResp> list) {
        List<Map<String, Object>> treeList = new ArrayList<>();
        for (YcqYjsCourseMgrResp ycqYjsCourseMgrResp : list) {
            Map<String, Object> treeNodeVo = new HashMap<>();
            treeNodeVo.put("key", ycqYjsCourseMgrResp.getId().toString());
            treeNodeVo.put("title", ycqYjsCourseMgrResp.getCourseName());
            treeNodeVo.put("value", ycqYjsCourseMgrResp.getId().toString());
            treeList.add(treeNodeVo);
        }
        return treeList;
    }

}
