package com.crunii.micro.service.studycenter.service.impl;

import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.service.studycenter.common.base.ExamplePredicate;
import com.crunii.micro.service.studycenter.common.base.FieldRestriction;
import com.crunii.micro.service.studycenter.common.base.ValueInPredicate;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestPaper;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestPaperQuestionRel;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestionReq;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsTestPaperJdbcDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsTestPaperQuestionRelJdbcDao;
import com.crunii.micro.service.studycenter.service.YcqYjsTestPaperQuestionRelService;
import com.crunii.micro.service.studycenter.service.YcqYjsTestQuestionService;
import com.crunii.micro.service.studycenter.util.SecurityUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Author：dadm
 * Date：2023-09-20 18:40
 * Description：<描述>
 */
@Service
public class YcqYjsTestPaperQuestionRelServiceImpl implements YcqYjsTestPaperQuestionRelService {

    @Autowired private YcqYjsTestPaperQuestionRelJdbcDao testPaperQuestionRelJdbcDao;
    @Autowired private YcqYjsTestQuestionService testQuestionService;
    @Autowired private YcqYjsTestPaperJdbcDao ycqYjsTestPaperJdbcDao;


    @Override
    public void bindTestPaperQuestionRel(Long paperId, List<Long> questionIds) {
        // 1.解绑试卷下的题目
        unBindTestPaperQuestionRel(paperId);

        // 2.绑定试卷题目
        List<YcqYjsTestPaperQuestionRel> testPaperQuestionRels = questionIds.parallelStream().map(x->{
            YcqYjsTestPaperQuestionRel testPaperQuestionRel = new YcqYjsTestPaperQuestionRel();
            testPaperQuestionRel.setPaperId(paperId);
            testPaperQuestionRel.setQuestionId(x);
            testPaperQuestionRel.setDelFlag(CommonConstants.LOG_FLAG_NO);
            testPaperQuestionRel.setCreateId(SecurityUtils.getLoginId());
            testPaperQuestionRel.setCreateTime(new Date());
            return testPaperQuestionRel;
        }).collect(Collectors.toList());

        testPaperQuestionRelJdbcDao.saveAllAndFlush(testPaperQuestionRels);
    }

    @Override
    public void unBindTestPaperQuestionRel(Long paperId) {
        unBindTestPaperQuestionRel(paperId, new ArrayList<>());
    }

    @Override
    public void unBindTestPaperQuestionRel(Long paperId, Long questionId) {
        List<Long> questionIds = new ArrayList<>();
        questionIds.add(questionId);
        unBindTestPaperQuestionRel(paperId, questionIds);
    }

    @Override
    public void unBindTestPaperQuestionRel(Long paperId, List<Long> questionIds) {
        YcqYjsTestPaperQuestionRel testPaperQuestionRel = new YcqYjsTestPaperQuestionRel();
        testPaperQuestionRel.setPaperId(paperId);
        testPaperQuestionRel.setDelFlag(CommonConstants.LOG_FLAG_NO);

        List<FieldRestriction<YcqYjsTestPaperQuestionRel>> fieldRestrictions = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(questionIds)) {
            fieldRestrictions.add(new ValueInPredicate<YcqYjsTestPaperQuestionRel, Long>("question_id", questionIds));
        }

        ExamplePredicate examplePredicate = ExamplePredicate.of(Example.of(testPaperQuestionRel), fieldRestrictions);
        List<YcqYjsTestPaperQuestionRel> testPaperQuestionRels = testPaperQuestionRelJdbcDao.findAll(examplePredicate);

        if (!CollectionUtils.isEmpty(testPaperQuestionRels)){
            Date currentTime = new Date();
            Long loginId = SecurityUtils.getLoginId();
            testPaperQuestionRels.forEach(x->{
                x.setDelFlag(CommonConstants.LOG_FLAG_YES);
                x.setUpdateTime(currentTime);
                x.setUpdateId(loginId);
            });

            testPaperQuestionRelJdbcDao.saveAllAndFlush(testPaperQuestionRels);
        }
    }

    @Override
    public List<YcqYjsTestQuestionReq> getYcqYjsTestQuestionByPaperId(Long paperId) {
        // 1.查询试题关联的题目ids
        YcqYjsTestPaperQuestionRel testPaperQuestionRel = new YcqYjsTestPaperQuestionRel();
        testPaperQuestionRel.setPaperId(paperId);
        testPaperQuestionRel.setDelFlag(CommonConstants.LOG_FLAG_NO);
        List<YcqYjsTestPaperQuestionRel> testPaperQuestionRels = testPaperQuestionRelJdbcDao.findAll(Example.of(testPaperQuestionRel));

        if (CollectionUtils.isEmpty(testPaperQuestionRels)) {
            return new ArrayList<>();
        }else {
            // 2.根据题目ids查询题目
            List<YcqYjsTestQuestionReq> ycqYjsTestQuestionReqs =  testQuestionService.queryByIds(testPaperQuestionRels.stream().map(x-> BigInteger.valueOf(x.getQuestionId())).collect(Collectors.toList())).getData();
            // 3.排序：单选题 多选题 判断题
            ycqYjsTestQuestionReqs.sort(Comparator.comparing(x -> x.getTopicType()));
            return ycqYjsTestQuestionReqs;
        }
    }

    @Override
    public Map<Long, Long> getYcqYjsTestQuestionCountMap(List<Long> paperIds) {
        YcqYjsTestPaperQuestionRel testPaperQuestionRel = new YcqYjsTestPaperQuestionRel();
        testPaperQuestionRel.setDelFlag(CommonConstants.LOG_FLAG_NO);

        List<FieldRestriction<YcqYjsTestPaperQuestionRel>> fieldRestrictions = new ArrayList<>();
        fieldRestrictions.add(new ValueInPredicate<YcqYjsTestPaperQuestionRel, Long>("paperId", paperIds));

        ExamplePredicate examplePredicate = ExamplePredicate.of(Example.of(testPaperQuestionRel), fieldRestrictions);
        List<YcqYjsTestPaperQuestionRel> testPaperQuestionRels = testPaperQuestionRelJdbcDao.findAll(examplePredicate);

        return testPaperQuestionRels.stream().collect(Collectors.groupingBy(YcqYjsTestPaperQuestionRel::getPaperId, Collectors.counting()));
    }

    @Override
    public Map<BigInteger, Integer> getTestPaperCountMapByQuestionIds(List<BigInteger> questionIds) {
        YcqYjsTestPaperQuestionRel testPaperQuestionRel = new YcqYjsTestPaperQuestionRel();
        testPaperQuestionRel.setDelFlag(CommonConstants.LOG_FLAG_NO);

        List<FieldRestriction<YcqYjsTestPaperQuestionRel>> fieldRestrictions = new ArrayList<>();
        fieldRestrictions.add(new ValueInPredicate<YcqYjsTestPaperQuestionRel, Long>("questionId", questionIds.parallelStream().map(x->x.longValue()).collect(Collectors.toList())));

        ExamplePredicate examplePredicate = ExamplePredicate.of(Example.of(testPaperQuestionRel), fieldRestrictions);
        List<YcqYjsTestPaperQuestionRel> testPaperQuestionRels = testPaperQuestionRelJdbcDao.findAll(examplePredicate);
        if (CollectionUtils.isNotEmpty(testPaperQuestionRels)){
            Map<Long, Long> testPaperCountMap = testPaperQuestionRels.stream().collect(Collectors.groupingBy(YcqYjsTestPaperQuestionRel::getQuestionId, Collectors.counting()));
            Map<BigInteger, Integer> testPaperCountMapResp = new HashMap<>();
            testPaperCountMap.forEach((k,v)->{
                testPaperCountMapResp.put(BigInteger.valueOf(k), Integer.valueOf(v+""));
            });

            return testPaperCountMapResp;
        }else {
            return new HashMap<>();
        }
    }

    @Override
    public int getTestPaperCountByQuestionId(BigInteger questionId) {
        YcqYjsTestPaperQuestionRel testPaperQuestionRel = new YcqYjsTestPaperQuestionRel();
        testPaperQuestionRel.setDelFlag(CommonConstants.LOG_FLAG_NO);
        testPaperQuestionRel.setQuestionId(questionId.longValue());
        long count = testPaperQuestionRelJdbcDao.count(Example.of(testPaperQuestionRel));
        return Integer.valueOf(count+"");
    }

    @Override
    public Map<BigInteger, List<YcqYjsTestPaper>> getTestPaperYcqYjsTestPaperMapByQuestionIds(List<BigInteger> questionIds) {
        // 1.根据questionIds查询关联表
        YcqYjsTestPaperQuestionRel testPaperQuestionRel = new YcqYjsTestPaperQuestionRel();
        testPaperQuestionRel.setDelFlag(CommonConstants.LOG_FLAG_NO);
        List<FieldRestriction<YcqYjsTestPaperQuestionRel>> fieldRestrictions = new ArrayList<>();
        fieldRestrictions.add(new ValueInPredicate<YcqYjsTestPaperQuestionRel, Long>("questionId", questionIds.parallelStream().map(x->x.longValue()).collect(Collectors.toList())));
        ExamplePredicate examplePredicate = ExamplePredicate.of(Example.of(testPaperQuestionRel), fieldRestrictions);
        List<YcqYjsTestPaperQuestionRel> testPaperQuestionRels = testPaperQuestionRelJdbcDao.findAll(examplePredicate);
        if (CollectionUtils.isEmpty(testPaperQuestionRels)){
            return new HashMap<>();
        }

        // 2.根据关联表查询试卷表
        YcqYjsTestPaper testPaper = new YcqYjsTestPaper();
        testPaper.setDelFlag(CommonConstants.LOG_FLAG_NO);
        List<FieldRestriction<YcqYjsTestPaper>> ycqYjsTestPaperFieldRestrictions = new ArrayList<>();
        ycqYjsTestPaperFieldRestrictions.add(new ValueInPredicate<YcqYjsTestPaper, Long>("id", testPaperQuestionRels.parallelStream().map(x->x.getPaperId()).collect(Collectors.toList())));
        ExamplePredicate examplePredicateTestPaper = ExamplePredicate.of(Example.of(testPaper), ycqYjsTestPaperFieldRestrictions);
        List<YcqYjsTestPaper> ycqYjsTestPapers = ycqYjsTestPaperJdbcDao.findAll(examplePredicateTestPaper);

        // 3.组装返回数据
        Map<Long, List<YcqYjsTestPaperQuestionRel>> ycqYjsTestPaperQuestionRelMap = testPaperQuestionRels.stream().collect(Collectors.groupingBy(YcqYjsTestPaperQuestionRel::getQuestionId));
        Map<BigInteger, List<YcqYjsTestPaper>> respMap = new HashMap<>();
        ycqYjsTestPaperQuestionRelMap.forEach((key,value)->{
            List<Long> paperIds = value.stream().map(x->x.getPaperId()).collect(Collectors.toList());
            respMap.put(BigInteger.valueOf(key), ycqYjsTestPapers.stream().filter(x->paperIds.contains(x)).collect(Collectors.toList()));
        });

        return respMap;
    }

    @Override
    public List<YcqYjsTestPaper> getTestPaperYcqYjsTestPaperByQuestionId(BigInteger questionId) {
        YcqYjsTestPaperQuestionRel testPaperQuestionRel = new YcqYjsTestPaperQuestionRel();
        testPaperQuestionRel.setDelFlag(CommonConstants.LOG_FLAG_NO);
        testPaperQuestionRel.setQuestionId(questionId.longValue());
        List<YcqYjsTestPaperQuestionRel> ycqYjsTestPaperQuestionRels = testPaperQuestionRelJdbcDao.findAll(Example.of(testPaperQuestionRel));

        if (CollectionUtils.isNotEmpty(ycqYjsTestPaperQuestionRels)){
            YcqYjsTestPaper testPaper = new YcqYjsTestPaper();
            testPaper.setDelFlag(CommonConstants.LOG_FLAG_NO);
            List<FieldRestriction<YcqYjsTestPaper>> fieldRestrictions = new ArrayList<>();
            fieldRestrictions.add(new ValueInPredicate<YcqYjsTestPaper, Long>("id", ycqYjsTestPaperQuestionRels.parallelStream().map(x->x.getPaperId()).collect(Collectors.toList())));
            ExamplePredicate examplePredicate = ExamplePredicate.of(Example.of(testPaper), fieldRestrictions);
            return ycqYjsTestPaperJdbcDao.findAll(examplePredicate);
        } else {
            return new ArrayList<>();
        }
    }


}
