package com.crunii.micro.service.studycenter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.json.JSONObject;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.studycenter.common.base.ExamplePredicate;
import com.crunii.micro.service.studycenter.common.base.FieldRestriction;
import com.crunii.micro.service.studycenter.common.base.TimestampRangePredicate;
import com.crunii.micro.service.studycenter.constants.StudyCenterConstants;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseMgr;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsCourseTestRel;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestPaper;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestionReq;
import com.crunii.micro.service.studycenter.dto.vo.TestPaperCondition;
import com.crunii.micro.service.studycenter.dto.vo.TestPaperReq;
import com.crunii.micro.service.studycenter.dto.vo.TestPaperVO;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsCourseMgrJdbcDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsTestPaperJdbcDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsTestPaperJdbcRepo;
import com.crunii.micro.service.studycenter.service.YcqYjsCourseMgrService;
import com.crunii.micro.service.studycenter.service.YcqYjsTestPaperQuestionRelService;
import com.crunii.micro.service.studycenter.service.YcqYjsTestPaperService;
import com.crunii.micro.service.studycenter.service.YcqYjsTestQuestionService;
import com.crunii.micro.service.studycenter.util.CharacterCodeUtil;
import com.crunii.micro.service.studycenter.util.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Author：dadm
 * Date：2023-09-20 17:22
 * Description：<描述>
 */
@Service
@Slf4j
public class YcqYjsTestPaperServiceImpl implements YcqYjsTestPaperService {

    @Autowired private YcqYjsTestPaperJdbcDao testPaperJdbcDao;
    @Autowired private YcqYjsTestPaperJdbcRepo testPaperJdbcRepo;
    @Autowired private YcqYjsTestPaperQuestionRelService testPaperQuestionRelService;
    @Autowired private YcqYjsTestQuestionService testQuestionService;
    @Autowired private ObsService obsService;
    @Autowired private YcqYjsCourseMgrService ycqYjsCourseMgrService;
    @Resource private YcqYjsCourseMgrJdbcDao ycqYjsCourseMgrJdbcDao;




    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createTestPaper(TestPaperReq createReq) {
        // 1.新增试卷基本信息
        YcqYjsTestPaper testPaper = new YcqYjsTestPaper();
        BeanUtil.copyProperties(createReq, testPaper);
        testPaper.setStatus(StudyCenterConstants.COMMON_YES);
        testPaper.setAcceptCount(0);

        Date currentTime= new Date();
        testPaper.setCode(CharacterCodeUtil.generateCharacterCode(StudyCenterConstants.TEST_PAPER_CODE_PREFIX));
        testPaper.setDelFlag(StudyCenterConstants.COMMON_NO);
        testPaper.setCreateId(SecurityUtils.getLoginId());
        testPaper.setCreateTime(currentTime);
        testPaper.setPublishTime(currentTime);
        testPaper.setPublishUserId(SecurityUtils.getLoginId());
        testPaper.setPublishUserName(SecurityUtils.getWorkerName());

        testPaperJdbcDao.saveAndFlush(testPaper);

        // 2.关联试卷题目
        testPaperQuestionRelService.bindTestPaperQuestionRel(testPaper.getId(), createReq.getQuestionIds());

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delTestPaper(Long id) {
        YcqYjsTestPaper testPaper = testPaperJdbcDao.getById(id);

        // 1.试卷是否关联课程
        YcqYjsCourseTestRel ycqYjsCourseTestRel = new YcqYjsCourseTestRel();
        ycqYjsCourseTestRel.setTestType(CommonConstants.LOG_FLAG_YES);
        ycqYjsCourseTestRel.setTestId(BigInteger.valueOf(id));
        List<YcqYjsCourseMgr> ycqYjsCourseMgrs = ycqYjsCourseMgrService.queryYcqYjsCourseByTestId(ycqYjsCourseTestRel).getData();
        if (CollectionUtils.isNotEmpty(ycqYjsCourseMgrs)){
            throw new BusinessException("试卷已关联课程不能删除：" + ycqYjsCourseMgrs.stream().map(x->x.getCourseName()).collect(Collectors.joining(",")));
        }

        // 删除试卷
        testPaper.setDelFlag(CommonConstants.LOG_FLAG_YES);
        testPaper.setUpdateId(SecurityUtils.getLoginId());
        testPaper.setUpdateTime(new Date());
        testPaperJdbcDao.saveAndFlush(testPaper);

        // 解绑试卷下题目
        testPaperQuestionRelService.unBindTestPaperQuestionRel(id);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateTestPaper(TestPaperReq updateReq) {
        YcqYjsTestPaper testPaper = testPaperJdbcDao.getById(updateReq.getId());

        // 修改试卷
        BeanUtil.copyProperties(updateReq, testPaper, "id", "status", "publishUserId", "publishUserName", "publishTime", "acceptCount", "delFlag", "createTime", "createId", "code");
        testPaper.setUpdateId(SecurityUtils.getLoginId());
        testPaper.setUpdateTime(new Date());

        // 关联试卷题目
        testPaperQuestionRelService.bindTestPaperQuestionRel(testPaper.getId(), updateReq.getQuestionIds());

    }

    @Override
    public TestPaperVO getTestPaperInfo(Long id) {
        // 1.试卷
        YcqYjsTestPaper testPaper = testPaperJdbcDao.getById(id);

        // 2.关联的题目
        List<YcqYjsTestQuestionReq> testQuestions = testPaperQuestionRelService.getYcqYjsTestQuestionByPaperId(id);
        TestPaperVO testPaperVO = new TestPaperVO();
        BeanUtil.copyProperties(testPaper, testPaperVO);
        testPaperVO.setTestQuestionVOS(testQuestions);

        return testPaperVO;
    }

   @Override
   public PageRsp<TestPaperVO> pageSearchTestPaper(PageReq<TestPaperCondition> pageReq) {
       PageRsp<YcqYjsTestPaper> page = null;
       if (null != pageReq.getQuery().getPageType() && StudyCenterConstants.PAGE_TYPE_COURSE_TEST_PAPER == pageReq.getQuery().getPageType()){
           pageReq.getQuery().setCurrentTime(DateUtil.format(new Date(), DateUtil.defaultDTPattern));
           page = testPaperJdbcRepo.paginRelQuery(pageReq);
       }else {
           page = testPaperJdbcRepo.paginQuery(pageReq);
       }

       PageRsp<TestPaperVO> pageRsp = new PageRsp();
       pageRsp.setPageNo(pageReq.getPageNo());
       pageRsp.setPageSize(pageReq.getPageSize());
       pageRsp.setTotalNum(page.getTotalNum());
       if (CollectionUtils.isNotEmpty(page.getResult())){
           Map<Long, Long> questionCountMap = testPaperQuestionRelService.getYcqYjsTestQuestionCountMap(page.getResult().stream().map(x->x.getId()).collect(Collectors.toList()));
           pageRsp.setResult(page.getResult().parallelStream().map(x->{
               TestPaperVO vo = new TestPaperVO();
               BeanUtil.copyProperties(x, vo);
               vo.setQuestionCount(null != questionCountMap.get(x.getId()) ? Integer.valueOf(questionCountMap.get(x.getId())+"") : 0);
               return vo;
           }).collect(Collectors.toList()));
       }else {
           pageRsp.setResult(new ArrayList<>());
       }

       return pageRsp;

   }

    @Override
    public void startOrEndTestPaper(Long id, Integer status) {
        YcqYjsTestPaper testPaper = testPaperJdbcDao.getById(id);
        testPaper.setStatus(status);
        testPaper.setUpdateTime(new Date());
        testPaper.setUpdateId(SecurityUtils.getLoginId());
        testPaperJdbcDao.saveAndFlush(testPaper);
    }

    @Override
    public FileVo doStoreImage(MultipartFile file) {
        Result<FileVo> fileVoResult = obsService.feignFileUpload(file);
        if(!fileVoResult.isSuccess()){
            throw new BusinessException("调用上传失败！");
        }
        return fileVoResult.getData();
    }

    @Override
    public PageRsp<TestPaperVO> pageSearchTestPaperByQzh(PageReq<TestPaperCondition> pageReq) {
        PageRsp<YcqYjsTestPaper> page = null;
        if (null != pageReq.getQuery().getPageType() && StudyCenterConstants.PAGE_TYPE_COURSE_TEST_PAPER == pageReq.getQuery().getPageType()){
            pageReq.getQuery().setCurrentTime(DateUtil.format(new Date(), DateUtil.defaultDTPattern));
            page = testPaperJdbcRepo.paginRelQuery(pageReq);
        }else {
            page = testPaperJdbcRepo.paginQuery(pageReq);
        }

        PageRsp<TestPaperVO> pageRsp = new PageRsp();
        pageRsp.setPageNo(pageReq.getPageNo());
        pageRsp.setPageSize(pageReq.getPageSize());
        pageRsp.setTotalNum(page.getTotalNum());
        if (CollectionUtils.isNotEmpty(page.getResult())){
            Map<Long, Long> questionCountMap = testPaperQuestionRelService.getYcqYjsTestQuestionCountMap(page.getResult().stream().map(x->x.getId()).collect(Collectors.toList()));
            pageRsp.setResult(page.getResult().parallelStream().map(x->{
                TestPaperVO vo = new TestPaperVO();
                BeanUtil.copyProperties(x, vo);
                vo.setQuestionCount(null != questionCountMap.get(x.getId()) ? Integer.valueOf(questionCountMap.get(x.getId())+"") : 0);
                return vo;
            }).collect(Collectors.toList()));
        }else {
            pageRsp.setResult(new ArrayList<>());
        }

        return pageRsp;
    }

}
