package com.crunii.micro.service.studycenter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.EasyExcel;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.entity.main.YcqAppConfigUrl;
import com.crunii.micro.domain.repo.main.YcqAppConfigUrlRepo;
import com.crunii.micro.service.studycenter.constants.StudyCenterConstants;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestion;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestionReq;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTopicClassify;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTopicOption;
import com.crunii.micro.service.studycenter.dto.vo.ExcelImportTestQuestionModel;
import com.crunii.micro.service.studycenter.dto.vo.ImportVO;
import com.crunii.micro.service.studycenter.handle.ExcelImportRegionSiteListener;
import com.crunii.micro.service.studycenter.handle.YcqYjsTestPaperHandle;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsTestQuestionJdbcDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsTopicOptionJdbcDao;
import com.crunii.micro.service.studycenter.service.YcqYjsTestPaperQuestionRelService;
import com.crunii.micro.service.studycenter.service.YcqYjsTestQuestionService;
import com.crunii.micro.service.studycenter.service.YcqYjsTopicClassifyService;
import com.crunii.micro.service.studycenter.util.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.beans.PropertyDescriptor;
import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 试题表;
 *
 * @author : Charler
 * @date : 2023-9-21
 */
@Slf4j
@Service
public class YcqYjsTestQuestionServiceImpl implements YcqYjsTestQuestionService {

    @Resource
    private YcqYjsTestQuestionJdbcDao testQuestionJdbcDao;
    @Resource
    private YcqYjsTopicOptionJdbcDao topicOptionDao;
    @Autowired
    private YcqYjsTestPaperHandle ycqYjsTestPaperHandle;
    @Autowired
    private YcqYjsTestPaperQuestionRelService testPaperQuestionRelService;
    @Autowired
    private YcqYjsTopicClassifyService ycqYjsTopicClassifyService;
    @Resource
    private YcqAppConfigUrlRepo ycqAppConfigUrlRepo;


    @Override
    public List<YcqYjsTestQuestion> searchAll() {
        return testQuestionJdbcDao.searchAll();
    }


    /**
     * 保存试题信息表和试题选项信息表
     * @param createReq
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> createTestQuestionInfo(YcqYjsTestQuestionReq createReq) {
        log.info("输入对象为" + createReq.toString());

        YcqYjsTestQuestion testQuestion = new YcqYjsTestQuestion();
        BeanUtil.copyProperties(createReq, testQuestion);
        // 如果创建人id为空获取登陆者id
        if (createReq.getCreatedBy() == null) {
            createReq.setCreatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));
        }
        testQuestion.setCreatedTime(new Date());
        testQuestion.setDelFlag(StudyCenterConstants.COMMON_NO);
        // 保存试题表  并返回试题信息
        YcqYjsTestQuestion ycqYjsTestQuestion = testQuestionJdbcDao.insert(testQuestion);
        createReq.setId(ycqYjsTestQuestion.getId());
        // 保存试题选项信息表
        List<YcqYjsTopicOption> topicOptionList = new ArrayList<>();
        createReq.getTopicOptinonList().forEach(e -> {
            // 如果创建人id为空获取登陆者id
            if (e.getCreatedBy() == null) {
                e.setCreatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));
            }
            e.setCreatedTime(new Date());
            e.setDelFlag(StudyCenterConstants.COMMON_NO);
            e.setTestQuestionId(ycqYjsTestQuestion.getId());
            //log.info("试题ID为" + testQuestion.getId()); //去除log缩短响应时间

            topicOptionList.add(e);
        });
        // 批量保存试题选项表
        topicOptionDao.batchInsert(topicOptionList);

        return Result.ofSuccess(createReq);
    }

    /**
     *  按照分类  题目和类型分页查询所有试题
     * @param req
     * @return
     */
    @Override
    public PageRsp<YcqYjsTestQuestion> pageYcqYjsTestQuestion(PageReq<YcqYjsTestQuestionReq> req) {
        return testQuestionJdbcDao.pageQuery(req);
    }

    /**
     *  按照分类  题目和类型分页查询所有试题
     * @param req
     * @return
     */
    @Override
    public PageRsp<YcqYjsTestQuestion> pageYcqYjsTestQuestionByIds(PageReq<YcqYjsTestQuestionReq> req) {

        List<YcqYjsTestQuestion> testQuestionList = testQuestionJdbcDao.getTestQuestionsByDelIds(req.getQuery());

        PageRsp<YcqYjsTestQuestion> pageRsp = new PageRsp<>();
        pageRsp.setPageNo(req.getPageNo());
        pageRsp.setPageSize(req.getPageSize());
        pageRsp.setTotalNum(testQuestionList.size());
        pageRsp.setResult(testQuestionList.size() > 0 ? testQuestionList : new ArrayList<>(0));
        return pageRsp;
    }

    /**
     * 更新试题相关信息功能
     * @param updateReq
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> updateTestQuestionInfo(YcqYjsTestQuestionReq updateReq) {
        log.info("输入对象为" + updateReq.toString());
        // 更新id为空直接返回  无需修改
        if (null == updateReq.getId()) {
            return Result.ofSuccess("试题Id为空，无法修改");
        }
        YcqYjsTestQuestion testQuestion = testQuestionJdbcDao.queryById(updateReq.getId());
        if (null == testQuestion) {
            return Result.ofSuccess("未查询到试题");
        }
        copyPropertiesNonNull(updateReq, testQuestion);
        // 更新试题表
        testQuestionJdbcDao.update(testQuestion);
        //  更新关联试题选项表
        List<YcqYjsTopicOption> topicOptions = topicOptionDao.queryByTestId(updateReq.getId());
        if (CollectionUtils.isEmpty(topicOptions)) {
            return Result.ofSuccess(updateReq);
        }
        Map<String, YcqYjsTopicOption> collectMap = topicOptions.stream().collect(Collectors.toMap(YcqYjsTopicOption::getOption, Function.identity(), (key1, key2) -> key2));
        List<YcqYjsTopicOption> topicOptionList = new ArrayList<>();
        updateReq.getTopicOptinonList().forEach(e -> {
            YcqYjsTopicOption ycqYjsTopicOption = collectMap.get(e.getOption());
            if (ycqYjsTopicOption != null) {
                copyPropertiesNonNull(e, ycqYjsTopicOption);
                topicOptionDao.update(ycqYjsTopicOption);
            } else {
                // 不存在新增
                // 保存试题选项信息表
                e.setCreatedBy(topicOptions.get(0).getCreatedBy());
                e.setCreatedTime(new Date());
                e.setDelFlag(StudyCenterConstants.COMMON_NO);
                e.setTestQuestionId(updateReq.getId());
                log.info("试题ID为" + updateReq.getId());
                e.setUpdatedTime(new Date());
                e.setUpdatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));
                topicOptionList.add(e);
            }

        });
        // 批量保存试题选项表
        topicOptionDao.batchInsert(topicOptionList);
        // 匹配删除选项
        Map<String, YcqYjsTopicOption> collectDelMap = updateReq.getTopicOptinonList().stream().collect(Collectors.toMap(YcqYjsTopicOption::getOption, Function.identity(), (key1, key2) -> key2));
        topicOptions.forEach(e -> {
            YcqYjsTopicOption ycqYjsTopicOption = collectDelMap.get(e.getOption());
            // 不存在删除
            if (ycqYjsTopicOption == null) {
                topicOptionDao.deleteById(e.getId());
            }
        });

        return Result.ofSuccess(updateReq);
    }

    /**
     * 根据id删除试题
     * @param id
     * @return
     */
    @Override
    public Result<Object> deleteById(BigInteger id) {
        testQuestionJdbcDao.deleteById(id);
        return Result.ofSuccess(id);
    }

    /**
     * 根据ids删除试题
     * @param ids
     * @return
     */
    @Override
    public Result<Object> deleteByIds(List<BigInteger> ids) {
        testQuestionJdbcDao.deleteByIds(ids);
        return Result.ofSuccess(ids);
    }

    /**
     * 根据id查询试题相关信息
     * @param id
     * @return
     */
    @Override
    public Result<YcqYjsTestQuestionReq> queryById(BigInteger id) {
        YcqYjsTestQuestionReq testQuestionReq = new YcqYjsTestQuestionReq();
        // 通过id查询试题表信息
        YcqYjsTestQuestion testQuestion = testQuestionJdbcDao.queryById(id);
        BeanUtil.copyProperties(testQuestion, testQuestionReq);
        // 通过试题表id查询试题选项表集合
        List<YcqYjsTopicOption> topicOptions = topicOptionDao.queryByTestId(id);
        testQuestionReq.setTopicOptinonList(topicOptions);
        // 设置返回信息
        Result<YcqYjsTestQuestionReq> result = new Result<>();
        int countNum = testPaperQuestionRelService.getTestPaperCountByQuestionId(id);
        testQuestionReq.setCountNum(countNum);
        result.setData(testQuestionReq);
        return result;
    }

    /**
     * 根据试题集合ids查询试题和试题选项表相关信息
     * @param ids
     * @return
     */
    @Override
    public Result<List<YcqYjsTestQuestionReq>> queryByIds(List<BigInteger> ids) {
        List<YcqYjsTestQuestionReq> testQuestionReqList = new ArrayList<>();
        // 批量查询出所有试题信息集合
        List<YcqYjsTestQuestion> testQuestionList = testQuestionJdbcDao.getTestQuestions(ids);
        // 批量查询所有试题选项集合
        List<YcqYjsTopicOption> topicOptions = topicOptionDao.queryByTestIds(ids);
        // 按照试题集合遍历
        testQuestionList.forEach(r -> {
            YcqYjsTestQuestionReq ycqYjsTestQuestionReq = new YcqYjsTestQuestionReq();
            BeanUtil.copyProperties(r, ycqYjsTestQuestionReq);
            List<YcqYjsTopicOption> topicOptinonList = new ArrayList<>();
            // 循环试题选项表信息
            topicOptions.forEach(e -> {
                // 判断试题选项表中ids和试题表id是否相同
                if (e.getTestQuestionId().equals(r.getId())) {
                    topicOptinonList.add(e);
                }
            });
            ycqYjsTestQuestionReq.setTopicOptinonList(topicOptinonList);
            int countNum = testPaperQuestionRelService.getTestPaperCountByQuestionId(r.getId());
            ycqYjsTestQuestionReq.setCountNum(countNum);
            testQuestionReqList.add(ycqYjsTestQuestionReq);

        });
        // 设置返回信息
        Result<List<YcqYjsTestQuestionReq>> result = new Result<>();
        result.setData(testQuestionReqList);
        return result;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ImportVO createImportTestQuestionExcel(MultipartFile file, BigInteger organizeId, String organizeName) {
        // 0.Excel 格式判断
        ycqYjsTestPaperHandle.checkExcelFormat(file);

        List<ExcelImportTestQuestionModel> models = new ArrayList<>();
        ImportVO importVO = new ImportVO();
        File excelFile = null;
        try {
            // 1.查询AppConfig
            List<YcqAppConfigUrl> ycqAppConfigUrls = ycqAppConfigUrlRepo.findAll(Example.of(new YcqAppConfigUrl()));
            Map<String, YcqAppConfigUrl> ycqAppConfigUrlMap = ycqAppConfigUrls.stream().collect(Collectors.toMap(YcqAppConfigUrl::getAppName, Function.identity(), (oldObject, newObject) -> newObject));

            // 2.试题分类
            List<YcqYjsTopicClassify> ycqYjsTopicClassifies = ycqYjsTopicClassifyService.getTopicClassifyList(null);
            Map<String, YcqYjsTopicClassify> ycqYjsTopicClassifyMap = ycqYjsTopicClassifies.stream().collect(Collectors.toMap(x->x.getOrganizeId() + x.getTopicClassifyName(), Function.identity(), (oldObject, newObject) -> newObject));

            // 3.导入
            String excelFilePath = System.getProperty("user.dir");
            String readExcelFilePath = excelFilePath+ File.separator + UUID.randomUUID().toString()+ ".xlsx";
            excelFile = new File(readExcelFilePath);
            FileUtils.copyInputStreamToFile(file.getInputStream(), excelFile);

            EasyExcel.read(readExcelFilePath, ExcelImportTestQuestionModel.class,
                    new ExcelImportRegionSiteListener(
                            ycqYjsTestPaperHandle,
                            models,
                            ycqAppConfigUrlMap,
                            ycqYjsTopicClassifyMap,
                            organizeId,
                            importVO)).sheet().headRowNumber(0).doRead();

            // 4.如果存在格式错误，则直接返回
            return importVO.getCountFail() > 0 ? importVO : batchCreateTestQuestion(models, importVO);
        } catch (IOException e){
            // 5.删除excel文件
            excelFile.delete();
            if (null != excelFile && excelFile.exists()){
                excelFile.delete();
            }
            e.printStackTrace();
            throw new BusinessException("获取文件流失败");
        } finally {
        }
    }

    @Override
    public ImportVO batchCreateTestQuestion(List<ExcelImportTestQuestionModel> models, ImportVO importVO) {
        Map<String, String> topicClassifyNameMap = new HashMap<>();

        // 1.如果有错误的直接返回
        if (models.stream().filter(x->!x.getSuccess()).count() == 0) {
            // 2.组装数据入库
            models.forEach(x->{
                importVO.getQuestionIds().add(doCreateTestQuestion(x));
            });
        }else {
            log.info("导入数据格式存在错误：{}", Arrays.toString(models.toArray()));
        }

        return importVO;

    }

    private BigInteger doCreateTestQuestion(ExcelImportTestQuestionModel model){
        // 1.新增题目
        Date currentTime = new Date();
        YcqYjsTestQuestion testQuestion = new YcqYjsTestQuestion();
        testQuestion.setTopic(model.getTopic());
        testQuestion.setTopicType(StudyCenterConstants.QUESTION_TOPIC_TYPE.get(model.getTopicType()));
        testQuestion.setTopicClassifyId(model.getTopicClassifyId());
        testQuestion.setTopicClassifyName(model.getTopicClassifyName());
        testQuestion.setOrganizeId(model.getOrganizeId());
        testQuestion.setOrganizeName(model.getOrganizationName());
        testQuestion.setDelFlag(CommonConstants.LOG_FLAG_NO);
        testQuestion.setCreatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));
        testQuestion.setCreatedTime(currentTime);
        YcqYjsTestQuestion ycqYjsTestQuestion = testQuestionJdbcDao.insert(testQuestion);


        // 2.新增选项
        List<YcqYjsTopicOption> topicOptions = new ArrayList<>();
        initYcqYjsTopicOption(ycqYjsTestQuestion.getId(), StudyCenterConstants.QUESTION_TOPIC_OPTION_A, model.getOptionA(), model.getAnswerOption(), currentTime, topicOptions);  //  选项A
        initYcqYjsTopicOption(ycqYjsTestQuestion.getId(), StudyCenterConstants.QUESTION_TOPIC_OPTION_B, model.getOptionB(), model.getAnswerOption(), currentTime, topicOptions);  //  选项B
        initYcqYjsTopicOption(ycqYjsTestQuestion.getId(), StudyCenterConstants.QUESTION_TOPIC_OPTION_C, model.getOptionC(), model.getAnswerOption(), currentTime, topicOptions);   //  选项C
        initYcqYjsTopicOption(ycqYjsTestQuestion.getId(), StudyCenterConstants.QUESTION_TOPIC_OPTION_D, model.getOptionD(), model.getAnswerOption(), currentTime, topicOptions);  //  选项D
        initYcqYjsTopicOption(ycqYjsTestQuestion.getId(), StudyCenterConstants.QUESTION_TOPIC_OPTION_E, model.getOptionE(), model.getAnswerOption(), currentTime, topicOptions);  //  选项E
        initYcqYjsTopicOption(ycqYjsTestQuestion.getId(), StudyCenterConstants.QUESTION_TOPIC_OPTION_F, model.getOptionF(), model.getAnswerOption(), currentTime, topicOptions);  //  选项F

        topicOptionDao.batchInsert(topicOptions);
        return ycqYjsTestQuestion.getId();

    }

    private void initYcqYjsTopicOption(BigInteger questionId, String option, String optionContent, String answerOption, Date currentTime, List<YcqYjsTopicOption> topicOptions){
        if (StringUtils.isEmpty(optionContent)) {
            return;
        }
        YcqYjsTopicOption topicOption = new YcqYjsTopicOption();
        topicOption.setOption(option);
        topicOption.setOptionContent(optionContent);
        topicOption.setIsAnswer(answerOption.contains(option) ? CommonConstants.LOG_FLAG_YES : CommonConstants.LOG_FLAG_NO);
        topicOption.setTestQuestionId(questionId);
        topicOption.setDelFlag(CommonConstants.LOG_FLAG_NO);
        topicOption.setCreatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));
        topicOption.setCreatedTime(currentTime);

        topicOptions.add(topicOption);
    }

    private void copyPropertiesNonNull(Object source, Object target) {
        if (source != null) {
            final BeanWrapper src = new BeanWrapperImpl(source);
            PropertyDescriptor[] pds = src.getPropertyDescriptors();

            List<String> nullProperties = new ArrayList<>();
            for (PropertyDescriptor pd : pds) {
                String propertyName = pd.getName();
                if (src.getPropertyValue(propertyName) == null) {
                    nullProperties.add(propertyName);
                }
            }
            String[] nullPropertyNames = nullProperties.toArray(new String[0]);
            BeanUtil.copyProperties(source, target, nullPropertyNames);
        }
    }

}
