package com.crunii.micro.service.studycenter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.studycenter.common.base.ExamplePredicate;
import com.crunii.micro.service.studycenter.common.base.FieldRestriction;
import com.crunii.micro.service.studycenter.common.base.TimestampRangePredicate;
import com.crunii.micro.service.studycenter.constants.StudyCenterConstants;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTestQuestion;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsTopicClassify;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsTopicClassifyCondition;
import com.crunii.micro.service.studycenter.dto.vo.YcqYjsTopicClassifyVo;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsTestQuestionJdbcDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsTopicClassifyDao;
import com.crunii.micro.service.studycenter.service.YcqYjsTopicClassifyService;
import com.crunii.micro.service.studycenter.util.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.Valid;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 题目分类service实现类;
 *
 * @author : Charler
 * @date : 2023-9-21
 */
@Slf4j
@Service
public class YcqYjsTopicClassifyServiceImpl implements YcqYjsTopicClassifyService {

    @Autowired
    private YcqYjsTopicClassifyDao topicClassifyDao;

    @Autowired
    private YcqYjsTestQuestionJdbcDao testQuestionJdbcDao;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createTopicClassify(YcqYjsTopicClassifyVo createReq) {
//        YcqYjsTopicClassify topicClassify = new YcqYjsTopicClassify();
//
//        topicClassify.setTopicClassifyName(createReq.getTopicClassifyName());
//        topicClassify.setOrganizeId(createReq.getOrganizeId());
//        topicClassify.setDelFlag(CommonConstants.LOG_FLAG_NO);
//        List<YcqYjsTopicClassify> all = topicClassifyDao.findAll(Example.of(topicClassify));
        // 1.新增题目分类信息
        YcqYjsTopicClassify ycqYjsTopicClassify = new YcqYjsTopicClassify();
        BeanUtil.copyProperties(createReq, ycqYjsTopicClassify);
        ycqYjsTopicClassify.setDelFlag(StudyCenterConstants.COMMON_NO);
        if (createReq.getCreatedBy() == null) {
            ycqYjsTopicClassify.setCreatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));
        }
        ycqYjsTopicClassify.setCreatedTime(new Date());
        topicClassifyDao.saveAndFlush(ycqYjsTopicClassify);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delTopicClassify(BigInteger id) {
        YcqYjsTopicClassify topicClassify = topicClassifyDao.getById(id);
        if (null == topicClassify) {
            throw new BusinessException("题目分类不存在");
        }
        // 删除试卷
        topicClassify.setDelFlag(CommonConstants.LOG_FLAG_YES);
        topicClassifyDao.saveAndFlush(topicClassify);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateTopicClassify(YcqYjsTopicClassifyVo updateReq) {
        if (updateReq.getId() == null) {
            throw new BusinessException("分类id为空，不能修改");
        }
        YcqYjsTopicClassify topicClassify = topicClassifyDao.getById(updateReq.getId());
        if (null == topicClassify) {
            throw new BusinessException("题目分类不存在");
        }
        BeanUtils.copyProperties(updateReq, topicClassify,"id","delFlag","createdBy","createdTime");

        if (updateReq.getUpdatedBy() == null && CurrWorkerUtil.getCurrWorker() != null) {
            topicClassify.setUpdatedBy(new BigInteger(String.valueOf((CurrWorkerUtil.getCurrWorker().getLoginId()))));
        }
        topicClassify.setUpdatedTime(new Date());
        topicClassifyDao.saveAndFlush(topicClassify);
        // 题目分类更新成功更新关联试题表
        List<YcqYjsTestQuestion> testQuestions = testQuestionJdbcDao.queryByClassifyId(updateReq.getId());
        testQuestions.forEach(e -> {
            e.setTopicClassifyName(updateReq.getTopicClassifyName());
            e.setOrganizeId(updateReq.getOrganizeId());
            e.setOrganizeName(updateReq.getOrganizeName());
            // 更新试题表
            testQuestionJdbcDao.updateByClassify(e);
        });

    }

    @Override
    public YcqYjsTopicClassify getTopicClassify(BigInteger id) {
        return topicClassifyDao.getById(id);
    }

    /**
     *  分页查询所有题目分类
     * @param pageReq
     * @return
     */
    @Override
    public PageRsp<YcqYjsTopicClassifyVo> pageSearchTopicClassify(PageReq<YcqYjsTopicClassifyCondition> pageReq) {
        // 条件参数
        @Valid YcqYjsTopicClassifyCondition pageReqQuery = pageReq.getQuery();
        YcqYjsTopicClassify topicClassify = new YcqYjsTopicClassify();
        topicClassify.setTopicClassifyName(StringUtils.isNotEmpty(pageReqQuery.getTopicClassifyName()) ? pageReqQuery.getTopicClassifyName() : null);
        topicClassify.setOrganizeName(StringUtils.isNotEmpty(pageReqQuery.getOrganizeName()) ? pageReqQuery.getOrganizeName() : null);
        topicClassify.setDelFlag(CommonConstants.LOG_FLAG_NO);

        // 条件参数匹配原则  exact精确查询  contains模糊查询
        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("topicClassifyName",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("organizeName",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("delFlag",ExampleMatcher.GenericPropertyMatchers.exact());

        Example example = Example.of(topicClassify, exampleMatcher);
        // 时间范围参数
        List<FieldRestriction<YcqYjsTopicClassify>> fieldRestrictions = new ArrayList<>();
        Date startTime = StringUtils.isNoneBlank(pageReqQuery.getStartTime()) ? DateUtil.parse(pageReqQuery.getStartTime() + " 00:00:00", DateUtil.defaultDTPattern) : null;
        Date endTime = StringUtils.isNoneBlank(pageReqQuery.getEndTime()) ? DateUtil.parse(pageReqQuery.getEndTime() + " 23:59:59", DateUtil.defaultDTPattern) : null;
        if (null != startTime) {
            fieldRestrictions.add(new TimestampRangePredicate<YcqYjsTopicClassify>("createdTime", startTime.getTime(), endTime.getTime()));
        }
        ExamplePredicate examplePredicate = ExamplePredicate.of(example, fieldRestrictions);

        // 分页参数
        Sort sort = Sort.by(Sort.Direction.DESC, "createdTime");
        int pageNumber = pageReq.getPageNo() <= 1 ? 0 : pageReq.getPageNo() - 1;
        PageRequest pageRequest = PageRequest.of(pageNumber, pageReq.getPageSize(), sort);
        Page<YcqYjsTopicClassify> page = topicClassifyDao.findAll(examplePredicate, pageRequest);

        PageRsp<YcqYjsTopicClassifyVo> pageRsp = new PageRsp();
        pageRsp.setPageNo(pageReq.getPageNo());
        pageRsp.setPageSize(pageReq.getPageSize());
        pageRsp.setTotalNum(page.getTotalElements());

        if (CollectionUtils.isEmpty(page.getContent())){
            pageRsp.setResult(new ArrayList<>());
            return pageRsp;
        }

        pageRsp.setResult(page.getContent().parallelStream().map(x->{
            YcqYjsTopicClassifyVo vo = new YcqYjsTopicClassifyVo();
            BeanUtil.copyProperties(x, vo);
            //解决精度丢失问题
            vo.setOrganizeIdStr(String.valueOf(x.getOrganizeId()));
            return vo;
        }).collect(Collectors.toList()));

        return pageRsp;

    }

    @Override
    public List<YcqYjsTopicClassify> getTopicClassifyList(BigInteger organizeId) {
        YcqYjsTopicClassify topicClassify = new YcqYjsTopicClassify();
        topicClassify.setOrganizeId(null != organizeId ? organizeId : null);
        topicClassify.setDelFlag(CommonConstants.LOG_FLAG_NO);
        return topicClassifyDao.findAll(Example.of(topicClassify));
    }


}
