package com.crunii.micro.service.studycenter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.DesensitizedUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.builder.ExcelWriterBuilder;
import com.alibaba.excel.write.builder.ExcelWriterSheetBuilder;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.studycenter.common.base.ExamplePredicate;
import com.crunii.micro.service.studycenter.common.base.FieldRestriction;
import com.crunii.micro.service.studycenter.common.base.TimestampRangePredicate;
import com.crunii.micro.service.studycenter.constants.StudyCenterConstants;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsNegativePerson;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsUnmatchedPerson;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsYearServiceTime;
import com.crunii.micro.service.studycenter.dto.vo.*;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsUnmatchedPersonDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsUnmatchedPersonJdbcDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsYearServiceTimeJdbcDao;
import com.crunii.micro.service.studycenter.service.YcqYjsUnmatchedPersonService;
import com.crunii.micro.service.studycenter.util.BeanCopyUtil;
import com.crunii.micro.service.studycenter.util.CharacterCodeUtil;
import com.crunii.micro.service.studycenter.util.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 不匹配人员清单表;
 *
 * @author : Charler
 * @date : 2023-9-21
 */
@Slf4j
@Service
public class YcqYjsUnmatchedPersonServiceImpl implements YcqYjsUnmatchedPersonService {

    @Autowired
    private YcqYjsUnmatchedPersonDao unmatchedPersonDao;

    @Resource
    private YcqYjsUnmatchedPersonJdbcDao unmatchedPersonJdbcDao;

    private final static String CODE = "YJRY";

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createUnmatchedPerson(YcqYjsUnmatchedPersonVo createReq) {
        // 身份证号查询如果不为空  存在  不允许新增
        /*YcqYjsUnmatchedPerson ycqYjsUnmatchedPerson = unmatchedPersonDao.getUniqueByProp("idNumber", createReq.getIdNumber());
        if (ycqYjsUnmatchedPerson != null) {
            throw new BusinessException("此身份证号已存在，无法新增！");
        }*/
        Map<String, Object> props = new HashMap<>();
        props.put("idNumber", createReq.getIdNumber());
        props.put("delFlag", 0);
        List<YcqYjsUnmatchedPerson> ycqYjsUnmatchedPerson = unmatchedPersonDao.findByProps(props);
        if (CollectionUtils.isNotEmpty(ycqYjsUnmatchedPerson) ) {
            throw new BusinessException("此身份证号已存在，无法新增！");
        }
        // 1.新增不匹配人员基本信息
        YcqYjsUnmatchedPerson unmatchedPerson = new YcqYjsUnmatchedPerson();
        BeanUtil.copyProperties(createReq, unmatchedPerson);
        // 人员编号生成规则：YJRY开头
        String generateCode = CharacterCodeUtil.generateCharacterCode(CODE);
        unmatchedPerson.setPersonNumber(generateCode);
        unmatchedPerson.setDelFlag(StudyCenterConstants.COMMON_NO);
        // 新增默认0：匹配
        unmatchedPerson.setIsMate(StudyCenterConstants.YES);
        if (createReq.getCreatedBy() == null) {
            unmatchedPerson.setCreatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));
        }
        unmatchedPerson.setCreatedTime(new Date());
        unmatchedPersonDao.saveAndFlush(unmatchedPerson);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delUnmatchedPerson(BigInteger id) {
        YcqYjsUnmatchedPerson unmatchedPerson = unmatchedPersonDao.getById(id);
        if (null == unmatchedPerson) {
            throw new BusinessException("不匹配人员信息不存在");
        }
        // 删除不匹配人员
        unmatchedPerson.setDelFlag(CommonConstants.LOG_FLAG_YES);
        unmatchedPersonDao.saveAndFlush(unmatchedPerson);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateUnmatchedPerson(YcqYjsUnmatchedPersonVo updateReq) {
        if (updateReq.getId() == null) {
            throw new BusinessException("id为空，不能修改");
        }
        YcqYjsUnmatchedPerson unmatchedPerson = unmatchedPersonDao.getById(updateReq.getId());
        if (null == unmatchedPerson) {
            throw new BusinessException("不匹配人员信息不存在");
        }
        // 判断脱敏后是否修改
        // 手机号
        if (updateReq.getPhoneNumber().equals(DesensitizedUtil.mobilePhone(unmatchedPerson.getPhoneNumber()))) {
            updateReq.setPhoneNumber(unmatchedPerson.getPhoneNumber());
        }
        // 身份证
        if (updateReq.getIdNumber().equals(DesensitizedUtil.mobilePhone(unmatchedPerson.getIdNumber()))) {
            updateReq.setIdNumber(unmatchedPerson.getIdNumber());
        }
        // 身份证号查询如果不为空  存在  不允许新增
        if (unmatchedPersonDao.countByIdNumberAndDelFlagAndIdNot(updateReq.getIdNumber(),0,updateReq.getId()) >0 ) {
            throw new BusinessException("此身份证号已存在，无法新增！");
        }
        BeanUtils.copyProperties(updateReq, unmatchedPerson, "id","personNumber","isMate","delFlag","createdBy","createdTime");
        if (updateReq.getUpdatedBy() == null && CurrWorkerUtil.getCurrWorker() != null) {
            unmatchedPerson.setUpdatedBy(new BigInteger(String.valueOf((CurrWorkerUtil.getCurrWorker().getLoginId()))));
        }
        unmatchedPerson.setUpdatedTime(new Date());
        unmatchedPersonDao.saveAndFlush(unmatchedPerson);
    }

    @Override
    public YcqYjsUnmatchedPerson getUnmatchedPerson(BigInteger id) {
        // 1.不匹配人员信息
        return unmatchedPersonDao.getById(id);
    }

    /**
     *  分页查询所有不匹配人员
     * @param req
     * @return
     */
    @Override
    public PageRsp<YcqYjsUnmatchedPerson> pageYcqYjsUnmatchedPerson(PageReq<UnmatchedPersonCondition> req) {
        return unmatchedPersonJdbcDao.pageQuery(req);
    }

    @Override
    public PageRsp<YcqYjsUnmatchedPersonVo> pageSearchUnmatchedPerson(PageReq<UnmatchedPersonCondition> pageReq) {
        // 条件参数
        UnmatchedPersonCondition pageReqQuery = pageReq.getQuery();
        YcqYjsUnmatchedPerson nnmatchedPerson = new YcqYjsUnmatchedPerson();
        nnmatchedPerson.setIdentityType(StringUtils.isNotEmpty(pageReqQuery.getIdentityType()) ? pageReqQuery.getIdentityType() : null);
        nnmatchedPerson.setPersonNumber(StringUtils.isNotEmpty(pageReqQuery.getPersonNumber()) ? pageReqQuery.getPersonNumber() : null);
        nnmatchedPerson.setPersonName(StringUtils.isNotEmpty(pageReqQuery.getPersonName()) ? pageReqQuery.getPersonName() : null);
        nnmatchedPerson.setPhoneNumber(StringUtils.isNotEmpty(pageReqQuery.getPhoneNumber()) ? pageReqQuery.getPhoneNumber() : null);
        nnmatchedPerson.setUnmatchedReason(StringUtils.isNotEmpty(pageReqQuery.getUnmatchedReason()) ? pageReqQuery.getUnmatchedReason() : null);
        nnmatchedPerson.setIdNumber(StringUtils.isNotEmpty(pageReqQuery.getIdNumber()) ? pageReqQuery.getIdNumber() : null);
        nnmatchedPerson.setDelFlag(CommonConstants.LOG_FLAG_NO);

        // 条件参数匹配原则  exact精确查询  contains模糊查询
        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("identityType",ExampleMatcher.GenericPropertyMatchers.exact())     // title模糊查询
                .withMatcher("personNumber",ExampleMatcher.GenericPropertyMatchers.contains())       // 所属业务精确查询
                .withMatcher("personName",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("phoneNumber",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("unmatchedReason",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("idNumber",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("delFlag",ExampleMatcher.GenericPropertyMatchers.exact())
                .withIgnoreNullValues();
        Example example = Example.of(nnmatchedPerson,exampleMatcher);

        // 时间范围参数
        List<FieldRestriction<YcqYjsUnmatchedPerson>> fieldRestrictions = new ArrayList<>();
        Date startTime = StringUtils.isNoneBlank(pageReqQuery.getStartTime()) ? DateUtil.parse(pageReqQuery.getStartTime() + " 00:00:00", DateUtil.defaultDTPattern) : null;
        Date endTime = StringUtils.isNoneBlank(pageReqQuery.getEndTime()) ? DateUtil.parse(pageReqQuery.getEndTime() + " 23:59:59", DateUtil.defaultDTPattern) : null;
        if (null != startTime) {
            fieldRestrictions.add(new TimestampRangePredicate<YcqYjsUnmatchedPerson>("createdTime", startTime.getTime(), endTime.getTime()));
        }
        ExamplePredicate examplePredicate = ExamplePredicate.of(example, fieldRestrictions);

        // 分页参数
        Sort sort = Sort.by(Sort.Direction.DESC, "createdTime");
        int pageNumber = pageReq.getPageNo() <= 1 ? 0 : pageReq.getPageNo() - 1;
        PageRequest pageRequest = PageRequest.of(pageNumber, pageReq.getPageSize(), sort);
        Page<YcqYjsUnmatchedPerson> page = unmatchedPersonDao.findAll(examplePredicate, pageRequest);

        PageRsp<YcqYjsUnmatchedPersonVo> pageRsp = new PageRsp();
        pageRsp.setPageNo(pageReq.getPageNo());
        pageRsp.setPageSize(pageReq.getPageSize());
        pageRsp.setTotalNum(page.getTotalElements());

        if (CollectionUtils.isEmpty(page.getContent())){
            pageRsp.setResult(new ArrayList<>());
            return pageRsp;
        }

        pageRsp.setResult(page.getContent().parallelStream().map(x->{
            YcqYjsUnmatchedPersonVo vo = new YcqYjsUnmatchedPersonVo();
            BeanUtil.copyProperties(x, vo);
            return vo;
        }).collect(Collectors.toList()));

        return pageRsp;

    }

    /**
     * 不匹配人员信息excel导出
     *
     * @param response
     * @param req
     */
    @Override
    public void exportUnmatchedPersonExcel(HttpServletResponse response, UnmatchedPersonCondition req) {
        try {
            log.info("YcqYjsUnmatchedPersonServiceImpl.exportUnmatchedPersonExcel.param：{}", req.toString());
            List<YcqYjsUnmatchedPerson> unmatchedPersonList = unmatchedPersonJdbcDao.queryByIds(req.getIds());
            String fileName = "预警人员清单";
            export(response, fileName, unmatchedPersonList);
        } catch (BusinessException e) {
            log.error("YcqYjsUnmatchedPersonServiceImpl.exportUnmatchedPersonExcel.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsUnmatchedPersonServiceImpl.exportUnmatchedPersonExcel.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export(HttpServletResponse response, String fileName, List<YcqYjsUnmatchedPerson> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=" + encodedFileName + ".xlsx");
            ExcelWriterBuilder write = EasyExcel.write(response.getOutputStream(), UnmatchedPersonExcel.class);
            ExcelWriterSheetBuilder sheet1 = write.sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy());
            List<UnmatchedPersonExcel> UnmatchedPersonExcelList = BeanCopyUtil.convert(dataList, UnmatchedPersonExcel.class);
            sheet1.doWrite(UnmatchedPersonExcelList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("YcqYjsUnmatchedPersonServiceImpl.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 自定义Excel导出策略，设置表头和数据行的字体和高度
     *
     * @return Excel导出策略
     */
    private HorizontalCellStyleStrategy getHeightAndFontStrategy() {
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 11);
        headWriteFont.setBold(true);
        headWriteCellStyle.setWriteFont(headWriteFont);
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
        WriteFont contentWriteFont = new WriteFont();
        contentWriteFont.setFontHeightInPoints((short) 11);
        contentWriteCellStyle.setWriteFont(contentWriteFont);
        contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
    }

}
