package com.crunii.micro.service.studycenter.service.impl;

import com.alibaba.druid.support.json.JSONUtils;
import com.alibaba.nacos.shaded.com.google.gson.Gson;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsUploadUserVideo;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsUploadVideo;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsUploadUserVideoDao;
import com.crunii.micro.service.studycenter.service.YcqYjsUploadUserVideoService;
import com.crunii.micro.service.studycenter.util.UploadFileUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * 用户上传学习视频表;(YCQ_YJS_UPLOAD_USER_VIDEO)表服务实现类
 * @author : http://www.chiner.pro
 * @date : 2023-10-5
 */
@Service
public class YcqYjsUploadUserVideoServiceImpl implements YcqYjsUploadUserVideoService {

    @Value("${upload.url}")
    private String urlPrefix;

    @Value("${streaming.systemName:qqcqglxt}")
    private String systemName;

    @Value("${obs.s3.accessKey}")
    private String accessKey;

    @Value("${obs.s3.accessSecret}")
    private String accessSecret;

    @Value("${obs.s3.bucketName}")
    private String bucketName;

    @Value("${obs.s3.endPoint}")
    private String endPoint;

    @Resource
    private ObsService obsService;

    @Resource
    private YcqYjsUploadUserVideoDao ycqYjsUploadUserVideoDao;

    @Override
    public Result<YcqYjsUploadUserVideo> insert(MultipartFile file,String  fileNumber, String tableName) {
        YcqYjsUploadUserVideo ycqYjsUploadUserVideo = new YcqYjsUploadUserVideo();
        // 获取当前登录人id
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        int loginId = 1;
        String userName= "ceshi";
        if (currWorker!=null) {
            userName= currWorker.getWorkerName();
            loginId = currWorker.getLoginId();
        }
        java.util.Date day=new Date();
        SimpleDateFormat sdf= new SimpleDateFormat("yyyyMMdd");
        String uploadDate = sdf.format(day);
        String fileName = file.getOriginalFilename();
//        Result<FileVo> resultobs = obsService.feignFileUpload(file);
//        String code = resultobs.getCode();
//        if (!"0".equals(code)) {
//            return Result.ofFail("用户上传学习视频失败!");
//        }
//        FileVo fileVo = resultobs.getData();
//        String fileKey = fileVo.getFileKey();

        // 获取预上传地址
        String uploadPath=systemName+"/"+userName+"/"+uploadDate+"/app/"+fileName;
        Map<String, Object> map = UploadFileUtil.signUrl(accessKey, accessSecret , endPoint,bucketName , uploadPath, 60 * 24 * 7 );
        String actionUrl= map.get("fileUrl").toString();
        System.out.println("上传地址为:"+actionUrl);
        // 上传文件
        String tempLine = UploadFileUtil.uploadFile(actionUrl, file);
        System.out.println("上传响应:"+tempLine);


        ycqYjsUploadUserVideo.setFileName(fileName);
        ycqYjsUploadUserVideo.setFileNumber(fileNumber);
        ycqYjsUploadUserVideo.setTableName(tableName);
        String[] fileArr=fileName.split("\\.");
        ycqYjsUploadUserVideo.setFileType(null);
        if (fileArr.length==2) {
            ycqYjsUploadUserVideo.setFileType(fileArr[1]);
        }
        ycqYjsUploadUserVideo.setUploadDate(uploadDate);
        ycqYjsUploadUserVideo.setFileSize(BigInteger.valueOf(file.getSize()));
        ycqYjsUploadUserVideo.setDelFlag(0);
//        ycqYjsUploadUserVideo.setUrl(urlPrefix+fileKey);
        ycqYjsUploadUserVideo.setUserId(loginId+"");
        ycqYjsUploadUserVideo.setUserName(userName);
        ycqYjsUploadUserVideo = ycqYjsUploadUserVideoDao.insert(ycqYjsUploadUserVideo);
        return Result.ofSuccess(ycqYjsUploadUserVideo);
    }

    @Override
    public Result<Object> downloadCourseVideo(String fileNumber, String tableName) {
        // 根据fileNumber、tableName获取信息表记录
        YcqYjsUploadUserVideo ycqYjsUploadUserVideo=ycqYjsUploadUserVideoDao.query(fileNumber, tableName);
        // 获取当前登录人id
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        String userName= "ceshi";
        if (currWorker!=null) {
            userName= currWorker.getWorkerName();
        }
        String uploadPath=systemName+"/"+ycqYjsUploadUserVideo.getUserName()+"/"+ycqYjsUploadUserVideo.getUploadDate()+"/app/"+ycqYjsUploadUserVideo.getFileName();
        Map<String, Object> returnMap = UploadFileUtil.downloadUrl(accessKey, accessSecret , endPoint,bucketName , uploadPath, 60 * 24 * 7 );
        return Result.ofSuccess(JSONUtils.toJSONString(returnMap));
    }
}
