package com.crunii.micro.service.studycenter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.DesensitizedUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.builder.ExcelWriterBuilder;
import com.alibaba.excel.write.builder.ExcelWriterSheetBuilder;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.dto.OrderProp;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.DateUtil;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.studycenter.common.base.ExamplePredicate;
import com.crunii.micro.service.studycenter.common.base.FieldRestriction;
import com.crunii.micro.service.studycenter.common.base.TimestampRangePredicate;
import com.crunii.micro.service.studycenter.constants.StudyCenterConstants;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsNegativePerson;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsUnmatchedPerson;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsYearServiceTime;
import com.crunii.micro.service.studycenter.dto.vo.*;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsNegativePersonJdbcDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsYearServiceTimeDao;
import com.crunii.micro.service.studycenter.jdbc.YcqYjsYearServiceTimeJdbcDao;
import com.crunii.micro.service.studycenter.service.YcqYjsYearServiceTimeService;
import com.crunii.micro.service.studycenter.util.BeanCopyUtil;
import com.crunii.micro.service.studycenter.util.CharacterCodeUtil;
import com.crunii.micro.service.studycenter.util.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.JpaSort;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 不匹配人员清单表;
 *
 * @author : Charler
 * @date : 2023-9-21
 */
@Slf4j
@Service
public class YcqYjsYearServiceTimeServiceImpl implements YcqYjsYearServiceTimeService {

    @Autowired
    private YcqYjsYearServiceTimeDao yearServiceTimeDao;

    @Resource
    private YcqYjsYearServiceTimeJdbcDao yearServiceTimeJdbcDao;

    private final static String CODE = "YJRY";


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createYearServiceTime(YcqYjsYearServiceTimeVo createReq) {

        // 身份证号查询如果不为空  存在  不允许新增
        Map<String, Object> props = new HashMap<>();
        props.put("idNumber", createReq.getIdNumber());
        props.put("delFlag", 0);
        List<YcqYjsYearServiceTime> ycqYjsYearServiceTime = yearServiceTimeDao.findByProps(props);
        if (CollectionUtils.isNotEmpty(ycqYjsYearServiceTime) ) {
            throw new BusinessException("此身份证号已存在，无法新增！");
        }
        // 1.新增年度服务时长预警信息
        YcqYjsYearServiceTime yearServiceTime = new YcqYjsYearServiceTime();
        BeanUtil.copyProperties(createReq, yearServiceTime);
        // 人员编号生成规则：YJRY开头
        String generateCode = CharacterCodeUtil.generateCharacterCode(CODE);
        yearServiceTime.setPersonNumber(generateCode);
        yearServiceTime.setDelFlag(StudyCenterConstants.COMMON_NO);
        if (createReq.getCreatedBy() == null) {
            yearServiceTime.setCreatedBy(BigInteger.valueOf(SecurityUtils.getLoginId()));
        }
        yearServiceTime.setCreatedTime(new Date());
        yearServiceTimeDao.saveAndFlush(yearServiceTime);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delYearServiceTime(BigInteger id) {
        YcqYjsYearServiceTime yearServiceTime = yearServiceTimeDao.getById(id);
        if (null == yearServiceTime) {
            throw new BusinessException("年度服务时长预警信息不存在");
        }
        // 删除试卷
        yearServiceTime.setDelFlag(CommonConstants.LOG_FLAG_YES);
        yearServiceTimeDao.saveAndFlush(yearServiceTime);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateYearServiceTime(YcqYjsYearServiceTimeVo updateReq) {
        if (updateReq.getId() == null) {
            throw new BusinessException("id为空，不能修改");
        }
        YcqYjsYearServiceTime yearServiceTime = yearServiceTimeDao.getById(updateReq.getId());
        if (null == yearServiceTime) {
            throw new BusinessException("年度服务时长预警信息不存在");
        }
        // 判断脱敏后是否修改
        // 手机号
        if (updateReq.getPhoneNumber().equals(DesensitizedUtil.mobilePhone(yearServiceTime.getPhoneNumber()))) {
            updateReq.setPhoneNumber(yearServiceTime.getPhoneNumber());
        }
        // 身份证
        if (updateReq.getIdNumber().equals(DesensitizedUtil.mobilePhone(yearServiceTime.getIdNumber()))) {
            updateReq.setIdNumber(yearServiceTime.getIdNumber());
        }
        BeanUtils.copyProperties(updateReq, yearServiceTime,"id","personNumber","delFlag","createdBy","createdTime");
        if (updateReq.getUpdatedBy() == null && CurrWorkerUtil.getCurrWorker() != null) {
            yearServiceTime.setUpdatedBy(new BigInteger(String.valueOf((CurrWorkerUtil.getCurrWorker().getLoginId()))));
        }
        yearServiceTime.setUpdatedTime(new Date());
        yearServiceTimeDao.saveAndFlush(yearServiceTime);
    }

    @Override
    public YcqYjsYearServiceTime getYearServiceTime(BigInteger id) {
        // 1.负面人员信息
        return yearServiceTimeDao.getById(id);
    }

    /**
     *  分页查询所有年度服务时长人员
     * @param req
     * @return
     */
    @Override
    public PageRsp<YcqYjsYearServiceTime> pageYcqYjsYearServiceTime(PageReq<YearServiceTimeCondition> req) {
        return yearServiceTimeJdbcDao.pageQuery(req);
    }

    @Override
    public PageRsp<YcqYjsYearServiceTimeVo> pageSearchYearServiceTime(PageReq<YearServiceTimeCondition> pageReq) {
        // 条件参数
        @Valid YearServiceTimeCondition pageReqQuery = pageReq.getQuery();
        YcqYjsYearServiceTime yearServiceTime = new YcqYjsYearServiceTime();
        yearServiceTime.setIdentityType(StringUtils.isNotEmpty(pageReqQuery.getIdentityType()) ? pageReqQuery.getIdentityType() : null);
        yearServiceTime.setPersonNumber(StringUtils.isNotEmpty(pageReqQuery.getPersonNumber()) ? pageReqQuery.getPersonNumber() : null);
        yearServiceTime.setPersonName(StringUtils.isNotEmpty(pageReqQuery.getPersonName()) ? pageReqQuery.getPersonName() : null);
        yearServiceTime.setPhoneNumber(StringUtils.isNotEmpty(pageReqQuery.getPhoneNumber()) ? pageReqQuery.getPhoneNumber() : null);
        yearServiceTime.setYearServiceTime(StringUtils.isNotEmpty(pageReqQuery.getYearServiceTime()) ? pageReqQuery.getYearServiceTime() : null);
        yearServiceTime.setIdNumber(StringUtils.isNotEmpty(pageReqQuery.getIdNumber()) ? pageReqQuery.getIdNumber() : null);
        yearServiceTime.setDelFlag(CommonConstants.LOG_FLAG_NO);

        // 条件参数匹配原则  exact精确查询  contains模糊查询
        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("identityType",ExampleMatcher.GenericPropertyMatchers.exact())     // title模糊查询
                .withMatcher("personNumber",ExampleMatcher.GenericPropertyMatchers.contains())       // 所属业务精确查询
                .withMatcher("personName",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("phoneNumber",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("yearServiceTime",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("idNumber",ExampleMatcher.GenericPropertyMatchers.contains())
                .withMatcher("delFlag",ExampleMatcher.GenericPropertyMatchers.exact())
                .withIgnoreNullValues();
        Example example = Example.of(yearServiceTime,exampleMatcher);

        // 时间范围参数
        List<FieldRestriction<YcqYjsYearServiceTime>> fieldRestrictions = new ArrayList<>();
        Date startTime = StringUtils.isNoneBlank(pageReqQuery.getStartTime()) ? DateUtil.parse(pageReqQuery.getStartTime() + " 00:00:00", DateUtil.defaultDTPattern) : null;
        Date endTime = StringUtils.isNoneBlank(pageReqQuery.getEndTime()) ? DateUtil.parse(pageReqQuery.getEndTime() + " 23:59:59", DateUtil.defaultDTPattern) : null;
        if (null != startTime) {
            fieldRestrictions.add(new TimestampRangePredicate<YcqYjsYearServiceTime>("createdTime", startTime.getTime(), endTime.getTime()));
        }
        ExamplePredicate examplePredicate = ExamplePredicate.of(example, fieldRestrictions);

        // 分页参数
        // 判断前端是否传了排序条件
        Sort sort = null;
        if (CollectionUtils.isNotEmpty(pageReq.getOrderProps())) {
            // 服务时长字段为varchar，实际排序为int，固需要特殊处理
            if ("yearServiceTime".equals(pageReq.getOrderProps().get(0).getProp())) {
                sort = JpaSort.unsafe(pageReq.getOrderProps().get(0).isAscending() ? Sort.Direction.ASC:Sort.Direction.DESC, "yearServiceTimeInt");
            } else {
                sort = Sort.by(pageReq.getOrderProps().get(0).isAscending() ? Sort.Direction.ASC:Sort.Direction.DESC, pageReq.getOrderProps().get(0).getProp());
            }
        } else {
            sort = Sort.by(Sort.Direction.DESC, "createdTime");
        }
        int pageNumber = pageReq.getPageNo() <= 1 ? 0 : pageReq.getPageNo() - 1;
        PageRequest pageRequest = PageRequest.of(pageNumber, pageReq.getPageSize(), sort);
        Page<YcqYjsYearServiceTime> page = yearServiceTimeDao.findAll(examplePredicate, pageRequest);

        PageRsp<YcqYjsYearServiceTimeVo> pageRsp = new PageRsp();
        pageRsp.setPageNo(pageReq.getPageNo());
        pageRsp.setPageSize(pageReq.getPageSize());
        pageRsp.setTotalNum(page.getTotalElements());

        if (CollectionUtils.isEmpty(page.getContent())){
            pageRsp.setResult(new ArrayList<>());
            return pageRsp;
        }

        pageRsp.setResult(page.getContent().parallelStream().map(x->{
            YcqYjsYearServiceTimeVo vo = new YcqYjsYearServiceTimeVo();
            BeanUtil.copyProperties(x, vo);
            return vo;
        }).collect(Collectors.toList()));

        return pageRsp;

    }

    /**
     * 年度服务时长信息excel导出
     *
     * @param response
     * @param req
     */
    @Override
    public void exportYearServiceTimeExcel(HttpServletResponse response, YearServiceTimeCondition req) {
        try {
            log.info("YcqYjsYearServiceTimeServiceImpl.exportYearServiceTimeExcel.param：{}", req.toString());
            List<YcqYjsYearServiceTime> yearServiceTime = yearServiceTimeJdbcDao.queryByIds(req.getIds());
            String fileName = "年度服务时长信息";
            export(response, fileName, yearServiceTime);
        } catch (BusinessException e) {
            log.error("YcqYjsYearServiceTimeServiceImpl.exportYearServiceTimeExcel.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsYearServiceTimeServiceImpl.exportYearServiceTimeExcel.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export(HttpServletResponse response, String fileName, List<YcqYjsYearServiceTime> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()).replaceAll("\\+", "%20");
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename*=" + encodedFileName + ".xlsx");
            ExcelWriterBuilder write = EasyExcel.write(response.getOutputStream(), YearServiceTimeExcel.class);
            ExcelWriterSheetBuilder sheet1 = write.sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy());
            List<YearServiceTimeExcel> YearServiceTimeExcelList = BeanCopyUtil.convert(dataList, YearServiceTimeExcel.class);
            sheet1.doWrite(YearServiceTimeExcelList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("YcqYjsYearServiceTimeServiceImpl.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 自定义Excel导出策略，设置表头和数据行的字体和高度
     *
     * @return Excel导出策略
     */
    private HorizontalCellStyleStrategy getHeightAndFontStrategy() {
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 11);
        headWriteFont.setBold(true);
        headWriteCellStyle.setWriteFont(headWriteFont);
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
        WriteFont contentWriteFont = new WriteFont();
        contentWriteFont.setFontHeightInPoints((short) 11);
        contentWriteCellStyle.setWriteFont(contentWriteFont);
        contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
    }

}
