package com.crunii.micro.service.studycenter.third;

import cn.hutool.json.JSONObject;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.studycenter.dto.studycenter.YcqYjsUploadVideo;
import com.crunii.micro.service.studycenter.dto.vo.SqlVo;
import com.crunii.micro.service.studycenter.service.StreamingService;
import com.crunii.micro.service.studycenter.service.YcqYjsCourseMgrService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Map;

/**
 * @author : dinglian
 * @date : 2023-9-18
 */
@Slf4j
@Api(tags = "流媒体转化文件通知服务")
@RestController
@RequestMapping("/streaming")
public class StreamingController {

    @Resource
    private YcqYjsCourseMgrService ycqYjsCourseMgrServiceImpl;

    @Resource
    private StreamingService streamingService;

    @ApiOperation(value = "更改文件转码状态")
    @RequestMapping(path = "/updateCourseVideoStatus", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateCourseVideoStatus(@RequestBody YcqYjsUploadVideo ycqYjsUploadVideo) {
        try {
            return  ycqYjsCourseMgrServiceImpl.updateCourseVideoStatus(ycqYjsUploadVideo.getFileId());
        } catch (BusinessException e) {
            log.error("StreamingController.updateCourseVideoStatus.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StreamingController.updateCourseVideoStatus.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "生成文件预上传地址")
    @RequestMapping(path = "/generateUploadAddress", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> generateUploadAddress(@RequestBody JSONObject json) {
        try {
            String userName = json.getStr("userName");
            String systemName = json.getStr("systemName");
            String originalVideo = json.getStr("originalVideo");
            String fileName = json.getStr("fileName");
            return  ycqYjsCourseMgrServiceImpl.generateUploadAddress(userName, systemName, originalVideo, fileName);
        } catch (BusinessException e) {
            log.error("StreamingController.updateCourseVideoStatus.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StreamingController.updateCourseVideoStatus.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "获取视频文件下载地址")
    @RequestMapping(path = "/downloadCourseVideo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> downloadCourseVideo(@RequestParam("fileNumber") String fileNumber,
                                              @RequestParam("tableName") String tableName) {
        try {
            return  ycqYjsCourseMgrServiceImpl.downloadCourseVideo(fileNumber, tableName);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.downloadCourseVideo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.downloadCourseVideo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "通知流媒体转码")
    @RequestMapping(path = "/transcodingVideo", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> transcodingVideo(@RequestParam("userName") String userName,
                                           @RequestParam("uploadDate") String uploadDate,
                                           @RequestParam("fileName") String fileName) {
        try {
            return  ycqYjsCourseMgrServiceImpl.transcodingVideo(userName, uploadDate, fileName);
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.uploadCourseVideo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.uploadCourseVideo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "获取视频实时播放地址")
    @RequestMapping(path = "/queryRealTimeVideoAddress", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> queryRealTimeVideoAddress(@RequestBody YcqYjsUploadVideo YcqYjsUploadVideo) {
        try {
            return  ycqYjsCourseMgrServiceImpl.queryRealTimeVideoAddress(YcqYjsUploadVideo.getFileNumber(), YcqYjsUploadVideo.getTableName());
        } catch (BusinessException e) {
            log.error("YcqYjsCourseMgrController.downloadCourseVideo.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsCourseMgrController.downloadCourseVideo.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "querySql")
    @RequestMapping(path = "/querySql", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> querySql(@RequestBody SqlVo sqlVo) {
        try {
            return  streamingService.querySql(sqlVo);
        } catch (BusinessException e) {
            log.error("StreamingController.querySql.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("StreamingController.updateCourseVideoStatus.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }
}
