package com.crunii.micro.service.youthcall.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.youthcall.dto.YcqPublishActivity;
import com.crunii.micro.service.youthcall.dto.YcqPublishActivityResp;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallQrcode;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallQrcodeActivityRel;
import com.crunii.micro.service.youthcall.dto.vo.YcqPublishActivityVo;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallCampusEducationVo;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallQrcodeVo;
import com.crunii.micro.service.youthcall.service.YcqYouthcallCampusEducationService;
import com.crunii.micro.service.youthcall.service.YcqYouthcallQrcodeService;
import com.crunii.micro.service.youthcall.utils.CharacterCodeUtil;
import com.crunii.micro.service.youthcall.utils.ExcelUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.math.BigInteger;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author : dinglian
 * @date : 2023-9-18
 */
@Slf4j
@Api(tags = "青呼我为-校外教育")
@RestController
@RequestMapping("/ycqYouthcallCampuseducation")
public class YcqYouthcallCampuseducationController {

    @Resource
    private YcqYouthcallCampusEducationService ycqYouthcallCampusEducationServiceImpl;

    @ApiOperation(value = "新增校外教育")
    @RequestMapping(path = "create", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> create(@Valid @RequestBody YcqYouthcallCampusEducationVo ycqYouthcallCampusEducationVo) {
        try {
            return ycqYouthcallCampusEducationServiceImpl.insert(ycqYouthcallCampusEducationVo);
        } catch (BusinessException e) {
            log.error("YcqYouthcallCampuseducationController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCampuseducationController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新增校外教育(加密参数)")
    @RequestMapping(path = "createEncryption", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> createEncryption(@Valid @RequestBody YcqYouthcallCampusEducationVo ycqYouthcallCampusEducationVo) {
        try {
            return ycqYouthcallCampusEducationServiceImpl.createEncryption(ycqYouthcallCampusEducationVo);
        } catch (BusinessException e) {
            log.error("YcqYouthcallCampuseducationController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCampuseducationController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询校外教育")
    @RequestMapping(path = "paginQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<YcqYouthcallCampusEducationVo> paginQuery() {
        try {
            return ycqYouthcallCampusEducationServiceImpl.paginQuery();
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.paginQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.paginQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }
}
