package com.crunii.micro.service.youthcall.controller;

import com.alibaba.fastjson.JSON;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallCheckInApply;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallCheckInApplyQueryCondition;
import com.crunii.micro.service.youthcall.service.YcqYouthcallCheckInApplyService;
import com.crunii.micro.service.youthcall.thirdrequest.AesUtil;
import com.crunii.micro.service.youthcall.thirdrequest.QnyzDataRequestService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.math.BigInteger;
import java.util.*;

/**
 * @author PANJL
 * @date 2023-12-16 17:21
 */
@Slf4j
@Api(tags = "住房保障-驿站入住申请记录")
@RestController
@RequestMapping("/ycqYouthcallCheckInApply/auth")
public class YcqYouthcallCheckInApplyController {

    private final YcqYouthcallCheckInApplyService ycqYouthcallCheckInApplyService;

    public YcqYouthcallCheckInApplyController(YcqYouthcallCheckInApplyService ycqYouthcallCheckInApplyService) {
        this.ycqYouthcallCheckInApplyService = ycqYouthcallCheckInApplyService;
    }

    @ApiOperation(value = "入住申请")
    @RequestMapping(path = "/saveApply", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> saveApply(HttpServletRequest request) {
        try {
            if (!(request instanceof MultipartHttpServletRequest)) {
                throw new BusinessException("文件上传请求类型错误");
            }
            // 上传文件列表
            Map<String, List<MultipartFile>> fileMap = new HashMap<>();
            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
            multipartRequest.getFileMap().keySet().forEach(name -> fileMap.put(name, multipartRequest.getFiles(name)));
            // 申请数据
            Map<String, String> dataMap = new HashMap<>();
            Enumeration<String> parameterNames = request.getParameterNames();
            while (parameterNames.hasMoreElements()) {
                String parameterName = parameterNames.nextElement();
                String parameterValue = request.getParameter(parameterName);
                dataMap.put(parameterName, parameterValue);
            }
            YcqYouthcallCheckInApply ycqYouthcallCheckInApply = JSON.parseObject(JSON.toJSONString(dataMap), YcqYouthcallCheckInApply.class);
            ycqYouthcallCheckInApplyService.saveApply(ycqYouthcallCheckInApply, fileMap);
            return Result.ofSuccess();
        } catch (BusinessException e) {
            log.error("YcqYouthcallCheckInApplyController.saveApply.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCheckInApplyController.saveApply.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "取消申请")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "applyId", value = "申请id", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/cancelApply", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> cancelApply(@Valid @RequestParam BigInteger applyId) {
        try {
            ycqYouthcallCheckInApplyService.cancelApply(applyId);
            return Result.ofSuccess();
        } catch (BusinessException e) {
            log.error("YcqYouthcallCheckInApplyController.cancelApply.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCheckInApplyController.cancelApply.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据申请id查询申请记录详情")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "applyId", value = "申请id", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/queryDetail", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYouthcallCheckInApply> queryDetail(@Valid @RequestParam BigInteger applyId) {
        try {
            YcqYouthcallCheckInApply ycqYouthcallCheckInApply = ycqYouthcallCheckInApplyService.queryByApplyId(applyId);
            return Result.ofSuccess(ycqYouthcallCheckInApply);
        } catch (BusinessException e) {
            log.error("YcqYouthcallCheckInApplyController.queryDetail.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCheckInApplyController.queryDetail.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "是否允许用户申请入住，180天内不能有2次取消申请")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "applyUserId", value = "申请人id", required = true, dataType = "String")
    )
    @RequestMapping(path = "/allowApply", method = RequestMethod.GET)
    @ResponseBody
    public Result<Boolean> allowApply(@Valid @RequestParam String applyUserId) {
        try {
            return Result.ofSuccess(ycqYouthcallCheckInApplyService.allowApply(applyUserId));
        } catch (BusinessException e) {
            log.error("YcqYouthcallCheckInApplyController.allowApply.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCheckInApplyController.allowApply.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询用户入住申请分页列表")
    @RequestMapping(path = "/pageQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<YcqYouthcallCheckInApply>> pageQuery(@RequestBody PageReq<YcqYouthcallCheckInApplyQueryCondition> req) {
        try {
            return Result.ofSuccess(ycqYouthcallCheckInApplyService.pageQuery(req));
        } catch (BusinessException e) {
            log.error("YcqYouthcallCheckInApplyController.pageQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCheckInApplyController.pageQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @Resource
    private QnyzDataRequestService qnyzDataRequestService;

    @RequestMapping(path = "/encrypt", method = RequestMethod.POST)
    @ResponseBody
    public Result<String> encrypt(@RequestBody Map<String, String> req) {
        try {
            // token
            req.put("token", qnyzDataRequestService.getToken());
            String encrypt = AesUtil.encrypt(JSON.toJSONString(req), AesUtil.AESKEY);
            return Result.ofSuccess(encrypt);
        } catch (BusinessException e) {
            log.error("YcqYouthcallCheckInApplyController.allowApply.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCheckInApplyController.allowApply.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
