package com.crunii.micro.service.youthcall.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.youthcall.dto.vo.commonDemand.CommonDemandBaseVo;
import com.crunii.micro.service.youthcall.dto.vo.commonDemand.CommonDemandEditVo;
import com.crunii.micro.service.youthcall.dto.vo.commonDemand.CommonDemandVo;
import com.crunii.micro.service.youthcall.dto.vo.commonDemand.YcqYouthcallCommonDemandReq;
import com.crunii.micro.service.youthcall.service.YcqYouthcallCommonDemandService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigInteger;
import java.util.List;

/**
 * @author : yingtiansheng
 * @date : 2023-11-01
 */
@Slf4j
@Api(tags = "共性需求管理")
@RestController
@RequestMapping("/ycqYouthcallCommonDemand/auth")
public class YcqYouthcallCommonDemandController {

    @Resource
    private YcqYouthcallCommonDemandService ycqYouthcallCommonDemandService;

    @ApiOperation(value = "分页查询共性需求")
    @RequestMapping(path = "pageQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<CommonDemandVo>> pageQuery(@RequestBody PageReq<YcqYouthcallCommonDemandReq> req) {
        try {
            return ycqYouthcallCommonDemandService.pageQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYouthcallCommonDemandController.pageQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCommonDemandController.pageQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询共性需求列表")
    @RequestMapping(path = "listQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<CommonDemandBaseVo>> listQuery(@RequestBody YcqYouthcallCommonDemandReq req) {
        try {
            return ycqYouthcallCommonDemandService.listQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYouthcallCommonDemandController.listQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCommonDemandController.listQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("删除")
    @GetMapping("del")
    public Result<?> del(BigInteger id, @ApiIgnore @CurrWorker Worker worker){
        if (id == null) {
            log.error("YcqYouthcallCommonDemandController.del.error. id is null");
            return Result.ofFail("系统内部异常，请联系管理员！");
        } else {
            ycqYouthcallCommonDemandService.deleteCommonDemand(id,worker);
            return Result.ofSuccess(true);
        }
    }

    @ApiOperation("移除所有绑定个性化需求")
    @GetMapping("removeAllPersonalDemand")
    public Result<?> removeAllPersonalDemand(BigInteger id, @ApiIgnore @CurrWorker Worker worker){
        if (id == null) {
            log.error("YcqYouthcallCommonDemandController.removeAllPersonalDemand.error. id is null");
            return Result.ofFail("系统内部异常，请联系管理员！");
        } else {
            ycqYouthcallCommonDemandService.removeAllPersonalDemand(id,worker);
            return Result.ofSuccess(true);
        }
    }

    @ApiOperation("查询共性需求详情")
    @GetMapping("queryDetailById")
    public Result<?> queryDetailById(BigInteger id){
        if (id == null) {
            log.error("YcqYouthcallCommonDemandController.queryDetailById.error. id is null");
            return Result.ofFail("系统内部异常，请联系管理员！");
        } else {
            return ycqYouthcallCommonDemandService.queryDetailById(id);
        }
    }

    @ApiOperation(value = "更新共性需求")
    @RequestMapping(path = "update", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> update(@Valid @RequestBody CommonDemandEditVo commonDemandEditVo, @ApiIgnore @CurrWorker Worker worker) {
        try {
            return ycqYouthcallCommonDemandService.edit(commonDemandEditVo,worker);
        } catch (BusinessException e) {
            log.error("YcqYouthcallPersonalDemandController.update.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallPersonalDemandController.update.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "共性需求转工单")
    @RequestMapping(path = "changeToWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> changeToWorkOrder(@Valid @RequestBody CommonDemandEditVo commonDemandEditVo, @ApiIgnore @CurrWorker Worker worker) {
        try {
            return ycqYouthcallCommonDemandService.changeToWorkOrder(commonDemandEditVo.getId(),worker);
        } catch (BusinessException e) {
            log.error("YcqYouthcallPersonalDemandController.changeToWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallPersonalDemandController.changeToWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
