package com.crunii.micro.service.youthcall.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.youthcall.dto.vo.personalDemand.*;
import com.crunii.micro.service.youthcall.service.YcqYouthcallPersonalDemandService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigInteger;
import java.util.List;

/**
 * @author : yingtiansheng
 * @date : 2023-11-01
 */
@Slf4j
@Api(tags = "个性化需求管理")
@RestController
@RequestMapping("/ycqYouthcallPersonalDemand/auth")
public class YcqYouthcallPersonalDemandController {

    @Resource
    private YcqYouthcallPersonalDemandService ycqYouthcallPersonalDemandService;

    @ApiOperation(value = "分页查询个性化需求")
    @RequestMapping(path = "pageQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<PersonalDemandVo>> pageQuery(@RequestBody PageReq<YcqYouthcallPersonalDemandReq> req) {
        try {
            return ycqYouthcallPersonalDemandService.pageQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYouthcallPersonalDemandController.pageQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallPersonalDemandController.pageQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新增个性化需求")
    @RequestMapping(path = "create", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> create(@Valid @RequestBody YcqYouthcallPersonalDemandInsertVo ycqYouthcallPersonalDemandInsertVo, @ApiIgnore @CurrWorker Worker worker) {
        try {
            return ycqYouthcallPersonalDemandService.insertPersonalDemand(ycqYouthcallPersonalDemandInsertVo,worker);
        } catch (BusinessException e) {
            log.error("YcqYouthcallPersonalDemandController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallPersonalDemandController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "撤销个性化需求")
    @RequestMapping(path = "revokePersonalDemand", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> revokePersonalDemand(@Valid @RequestBody YcqYouthcallPersonalDemandRevokeVo ycqYouthcallWorkOrderDispatchUpdateVo, @ApiIgnore @CurrWorker Worker worker) {
        try {
            if (ycqYouthcallWorkOrderDispatchUpdateVo == null || ycqYouthcallWorkOrderDispatchUpdateVo.getId() == null) {
                return Result.ofFail("参数异常");
            } else {
                return ycqYouthcallPersonalDemandService.revokePersonalDemand(ycqYouthcallWorkOrderDispatchUpdateVo.getId(),worker);
            }
        } catch (BusinessException e) {
            log.error("YcqYouthcallPersonalDemandController.revokePersonalDemand.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallPersonalDemandController.revokePersonalDemand.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "批量转为共性需求")
    @RequestMapping(path = "changeCommon", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> changeCommon(@Valid @RequestBody YcqYouthcallPersonalToCommonVo ycqYouthcallPersonalToCommonVo, @ApiIgnore @CurrWorker Worker worker) {
        try {
            return ycqYouthcallPersonalDemandService.changeCommon(ycqYouthcallPersonalToCommonVo,worker);
        } catch (BusinessException e) {
            log.error("YcqYouthcallPersonalDemandController.changeCommon.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallPersonalDemandController.changeCommon.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据idList查询个性化列表数据")
    @RequestMapping(path = "selectByIds", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<PersonalDemandVo>> selectByIds(@Valid @RequestBody YcqYouthcallPersonalToCommonVo ycqYouthcallPersonalToCommonVo) {
        try {
            return ycqYouthcallPersonalDemandService.selectByIds(ycqYouthcallPersonalToCommonVo);
        } catch (BusinessException e) {
            log.error("YcqYouthcallPersonalDemandController.selectByIds.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallPersonalDemandController.selectByIds.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据idList查询个性化图片数据")
    @RequestMapping(path = "selectImgByIds", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<String>> selectImgByIds(@Valid @RequestBody YcqYouthcallPersonalToCommonVo ycqYouthcallPersonalToCommonVo) {
        try {
            return ycqYouthcallPersonalDemandService.selectImgByIds(ycqYouthcallPersonalToCommonVo);
        } catch (BusinessException e) {
            log.error("YcqYouthcallPersonalDemandController.selectImgByIds.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallPersonalDemandController.selectImgByIds.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("删除")
    @GetMapping("del")
    public Result<?> del(BigInteger id, @ApiIgnore @CurrWorker Worker worker){
        if (id == null) {
            log.error("YcqYouthcallPersonalDemandController.del.error. id is null");
            return Result.ofFail("系统内部异常，请联系管理员！");
        } else {
            ycqYouthcallPersonalDemandService.deletePersonalDemand(id,worker);
            return Result.ofSuccess(true);
        }
    }

}
