package com.crunii.micro.service.youthcall.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.youthcall.dto.vo.productAdviceVo.ProductAdviceVo;
import com.crunii.micro.service.youthcall.dto.vo.productAdviceVo.YcqYouthcallProductAdviceDealReq;
import com.crunii.micro.service.youthcall.dto.vo.productAdviceVo.YcqYouthcallProductAdviceInsertVo;
import com.crunii.micro.service.youthcall.dto.vo.productAdviceVo.YcqYouthcallProductAdviceReq;
import com.crunii.micro.service.youthcall.service.YcqYouthcallProductAdviceService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigInteger;

/**
 * @author : yingtiansheng
 * @date : 2023-11-03
 */
@Slf4j
@Api(tags = "产品建议管理")
@RestController
@RequestMapping("/ycqYouthcallProductAdvice/auth")
public class YcqYouthcallProductAdviceController {

    @Resource
    private YcqYouthcallProductAdviceService ycqYouthcallProductAdviceService;

    @ApiOperation(value = "分页查询产品建议")
    @RequestMapping(path = "pageQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<ProductAdviceVo>> pageQuery(@RequestBody PageReq<YcqYouthcallProductAdviceReq> req) {
        try {
            return ycqYouthcallProductAdviceService.pageQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYouthcallProductAdviceController.pageQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallProductAdviceController.pageQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新增产品建议")
    @RequestMapping(path = "create", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> create(@Valid @RequestBody YcqYouthcallProductAdviceInsertVo ycqYouthcallProductAdviceInsertVo, @ApiIgnore @CurrWorker Worker worker) {
        try {
            return ycqYouthcallProductAdviceService.insertProductAdvice(ycqYouthcallProductAdviceInsertVo,worker);
        } catch (BusinessException e) {
            log.error("YcqYouthcallProductAdviceController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallProductAdviceController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "产品建议处理")
    @RequestMapping(path = "dealProductAdvice", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> dealProductAdvice(@Valid @RequestBody YcqYouthcallProductAdviceDealReq ycqYouthcallProductAdviceDealReq) {
        try {
            return ycqYouthcallProductAdviceService.dealProductAdvice(ycqYouthcallProductAdviceDealReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.dealProductAdvice.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.dealProductAdvice.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("删除")
    @GetMapping("del")
    public Result<?> del(BigInteger id, @ApiIgnore @CurrWorker Worker worker){
        if (id == null) {
            log.error("YcqYouthcallProductAdviceController.del.error. id is null");
            return Result.ofFail("系统内部异常，请联系管理员！");
        } else {
            ycqYouthcallProductAdviceService.deleteProductAdvice(id,worker);
            return Result.ofSuccess(true);
        }
    }

}
