package com.crunii.micro.service.youthcall.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.youthcall.dto.YcqPublishActivity;
import com.crunii.micro.service.youthcall.dto.YcqPublishActivityResp;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallQrcode;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallQrcodeActivityRel;
import com.crunii.micro.service.youthcall.dto.vo.YcqPublishActivityVo;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallQrcodeVo;
import com.crunii.micro.service.youthcall.service.YcqYouthcallQrcodeService;
import com.crunii.micro.service.youthcall.utils.CharacterCodeUtil;
import com.crunii.micro.service.youthcall.utils.ExcelUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.math.BigInteger;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author : dinglian
 * @date : 2023-9-18
 */
@Slf4j
@Api(tags = "青呼码管理")
@RestController
@RequestMapping("/ycqYouthcallQrcodeMgr/auth")
public class YcqYouthcallQrcodeController {

    @Resource
    private YcqYouthcallQrcodeService ycqYouthcallQrcodeServiceImpl;

    @ApiOperation(value = "查询青呼码管理")
    @RequestMapping(path = "paginQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<YcqYouthcallQrcode>> paginQuery(@RequestBody PageReq<YcqYouthcallQrcodeVo> req) {
        try {
            return ycqYouthcallQrcodeServiceImpl.paginQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.paginQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.paginQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新增青呼码管理")
    @RequestMapping(path = "create", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> create(@Valid @RequestBody YcqYouthcallQrcode ycqYouthcallQrcode) {
        try {
            return ycqYouthcallQrcodeServiceImpl.insert(ycqYouthcallQrcode);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改青呼码管理")
    @RequestMapping(path = "update", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> update(@Valid @RequestBody YcqYouthcallQrcode ycqYouthcallQrcode) {
        try {
            return ycqYouthcallQrcodeServiceImpl.update(ycqYouthcallQrcode);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.update.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.update.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除青呼码管理")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/del", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> del(BigInteger id) {
        try {
            return ycqYouthcallQrcodeServiceImpl.deleteById(id);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.del.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.del.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据id更改青呼码管理服务状态")
    @RequestMapping(path = "updateStatus", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateStatus(@Valid @RequestBody YcqYouthcallQrcode ycqYouthcallQrcode) {
        try {
            return ycqYouthcallQrcodeServiceImpl.updateStatus(ycqYouthcallQrcode);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.del.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.del.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据id更改青呼码管理是否可见全部")
    @RequestMapping(path = "updateVisibleAll", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateVisibleAll(@Valid @RequestBody YcqYouthcallQrcode ycqYouthcallQrcode) {
        try {
            return ycqYouthcallQrcodeServiceImpl.updateVisibleAll(ycqYouthcallQrcode);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.del.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.del.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据id查询青呼码管理")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/queryById", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYouthcallQrcode> queryById(@RequestParam("id")BigInteger id) {
        try {
            return ycqYouthcallQrcodeServiceImpl.queryById(id);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.queryById.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.queryById.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据青呼码编号查询青呼码管理")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/queryByYouthcallCode", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYouthcallQrcode> queryByYouthcallCode(@RequestParam("youthcallCode")String youthcallCode) {
        try {
            return ycqYouthcallQrcodeServiceImpl.queryByYouthcallCode(youthcallCode);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.queryByYouthcallCode.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.queryByYouthcallCode.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "批量新增青呼码管理")
    @RequestMapping(path = "createBath", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> createBath(@RequestBody List<YcqYouthcallQrcode> ycqYouthcallQrcodes) {
        try {
            return ycqYouthcallQrcodeServiceImpl.createBath(ycqYouthcallQrcodes);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "生成编号")
    @RequestMapping(path = "generateNumber", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> generateNumber(String code) {
        try {
            String generateCode = CharacterCodeUtil.generateCharacterCode(code);
            return Result.ofSuccess(generateCode);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.generateNumber.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.generateNumber.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询青呼码关联的活动")
    @RequestMapping(path = "queryQrcodeActivity", method = RequestMethod.POST)
    @ResponseBody
    public Result<List<YcqPublishActivity>> queryQrcodeActivity(@RequestBody YcqPublishActivityVo ycqPublishActivityVo) {
        try {
            return ycqYouthcallQrcodeServiceImpl.queryQrcodeActivity(ycqPublishActivityVo);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.queryQrcodeActivity.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.queryQrcodeActivity.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询青呼码关联的活动带分页")
    @RequestMapping(path = "queryQrcodeActivityByPage", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<YcqPublishActivityResp>> queryQrcodeActivityByPage(@RequestBody PageReq<YcqPublishActivityVo> req) {
        try {
            return ycqYouthcallQrcodeServiceImpl.queryQrcodeActivityByPage(req);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.queryQrcodeActivity.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.queryQrcodeActivity.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改青呼码活动入口")
    @RequestMapping(path = "updateQrCodeInlet", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateQrCodeInlet(@Valid @RequestBody YcqYouthcallQrcodeActivityRel ycqYouthcallQrcodeActivityRel) {
        try {
            return ycqYouthcallQrcodeServiceImpl.updateQrCodeInlet(ycqYouthcallQrcodeActivityRel);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.del.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.del.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "青呼码关联活动")
    @RequestMapping(path = "bathSaveQrcodeActivityRel", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> bathSaveQrcodeActivityRel(@RequestBody List<YcqYouthcallQrcodeActivityRel> list) {
        try {
            return ycqYouthcallQrcodeServiceImpl.bathSaveQrcodeActivityRel(list);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.bathSaveQrcodeActivityRel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.bathSaveQrcodeActivityRel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 导出青呼码列表
     */
    @ApiOperation(value = "导出青呼码列表")
    @RequestMapping(path = "export", method = RequestMethod.POST)
    @ResponseBody
    public void export(HttpServletResponse response,@RequestBody YcqYouthcallQrcodeVo req) throws MalformedURLException {
        Result<List<YcqYouthcallQrcode>> result = ycqYouthcallQrcodeServiceImpl.export(req);
        List<YcqYouthcallQrcode> list = result.getData();
        // 表头数据
        List<Object> head = Arrays.asList(
                "青呼码编码","青呼码名称","青呼码图片","所属组织","创建人","创建时间");
        List<List<Object>> sheetDataList = new ArrayList<>();
        sheetDataList.add(head);
        SimpleDateFormat ymd=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        // 用户数据
        for (YcqYouthcallQrcode em:list){
            List<Object> user = new ArrayList<>();
            user.add(em.getYouthcallCode());
            user.add(em.getYouthcallName());
            //图片的话转成url放入链接就可以
            URL url = null;
            try{
                url = new URL(em.getQrCodeUrl());
            } catch (Exception e) {

            }
            user.add(url);
            user.add(em.getOrgName());
            user.add(em.getCreateUserName());
            user.add(ymd.format(em.getCreateTime()));
            sheetDataList.add(user);
        }
        // 导出数据
        ExcelUtils.export(response,"青呼码列表", sheetDataList);
    }
}
