package com.crunii.micro.service.youthcall.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrder;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallWorkOrderVo;
import com.crunii.micro.service.youthcall.dto.vo.workOrder.*;
import com.crunii.micro.service.youthcall.service.YcqYouthcallWorkOrderService;
import com.crunii.micro.service.youthcall.utils.CharacterCodeUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * @author : dinglian
 * @date : 2023-9-18
 */
@Slf4j
@Api(tags = "工单管理")
@RestController
@RequestMapping("/ycqYouthcallWorkOrder/auth")
public class YcqYouthcallWorkOrderController {

    @Resource
    private YcqYouthcallWorkOrderService ycqYouthcallWorkOrderServiceImpl;

    @ApiOperation(value = "查询工单管理")
    @RequestMapping(path = "paginQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<WorkOrderBaseVo>> paginQuery(@RequestBody PageReq<YcqYouthcallWorkOrderVo> req) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.paginQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.paginQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.paginQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询上报工单管理")
    @RequestMapping(path = "reportPaginQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<WorkOrderBaseVo>> reportPaginQuery(@RequestBody PageReq<YcqYouthcallWorkOrderVo> req) {
        try {
            // 业务需求：上报工单管理列表展示上报到综治系统、民呼我为的工单
            return ycqYouthcallWorkOrderServiceImpl.reportPaginQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.reportPaginQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.reportPaginQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "查询我的工单管理")
    @RequestMapping(path = "myPaginQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<WorkOrderMyVo>> myPaginQuery(@RequestBody PageReq<YcqYouthcallWorkOrderVo> req) {
        try {
            // 业务需求：上报工单管理列表展示上报到综治系统、民呼我为的工单
            return ycqYouthcallWorkOrderServiceImpl.myPaginQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.myPaginQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.myPaginQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新增工单管理")
    @RequestMapping(path = "create", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> create(@Valid @RequestBody YcqYouthcallWorkOrder ycqYouthcallWorkOrder) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.insert(ycqYouthcallWorkOrder);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改工单管理流程状态")
    @RequestMapping(path = "updateProcessStatus", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> updateProcessStatus(@Valid @RequestBody YcqYouthcallWorkOrder ycqYouthcallWorkOrder) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.updateProcessStatus(ycqYouthcallWorkOrder);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "删除工单管理")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/del", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> del(BigInteger id) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.deleteById(id);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.del.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.del.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据id查询工单管理")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/queryById", method = RequestMethod.GET)
    @ResponseBody
    public Result<YcqYouthcallWorkOrder> queryById(@RequestParam("id")BigInteger id) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.queryById(id);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.queryById.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.queryById.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据登录人id查询诉求总数、待处理诉求数、已处理诉求数")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/queryWorkOrderCount", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> queryWorkOrderCount(@RequestParam("proposerId")String proposerId) {
        try {
            BigInteger bigIntegerString = BigInteger.valueOf(Long.parseLong(proposerId));
            return ycqYouthcallWorkOrderServiceImpl.queryWorkOrderCount(bigIntegerString);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.queryWorkOrderCount.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.queryWorkOrderCount.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "生成工单编号")
    @RequestMapping(path = "generateNumber", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> generateNumber(String code) {
        try {
            String generateCode = CharacterCodeUtil.generateCharacterCode(code);
            return Result.ofSuccess(generateCode);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.generateNumber.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.generateNumber.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新增诉求反馈")
    @RequestMapping(path = "createAppeal", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> createAppeal(@Valid @RequestBody YcqYouthcallWorkOrderInsertReq ycqYouthcallWorkOrderInsertReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.insertAppeal(ycqYouthcallWorkOrderInsertReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.createAppeal.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.createAppeal.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "工单处理")
    @RequestMapping(path = "dealWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> dealWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderDealReq ycqYouthcallWorkOrderDealReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.dealWorkOrder(ycqYouthcallWorkOrderDealReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.dealWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.dealWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "工单上报")
    @RequestMapping(path = "escalationWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> escalationWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderEscalationReq ycqYouthcallWorkOrderEscalationReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.escalationWorkOrder(ycqYouthcallWorkOrderEscalationReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.escalationWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.escalationWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "工单撤销上报")
    @RequestMapping(path = "revokeEscalationWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> revokeEscalationWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderEscalationReq ycqYouthcallWorkOrderEscalationReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.revokeEscalationWorkOrder(ycqYouthcallWorkOrderEscalationReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.revokeEscalationWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.revokeEscalationWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "工单下派")
    @RequestMapping(path = "allocationWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> allocationWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderAllocationReq ycqYouthcallWorkOrderAllocationReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.allocationWorkOrder(ycqYouthcallWorkOrderAllocationReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.allocationWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.allocationWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "工单撤销下派")
    @RequestMapping(path = "revokeAllocationWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> revokeAllocationWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderAllocationReq ycqYouthcallWorkOrderAllocationReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.revokeAllocationWorkOrder(ycqYouthcallWorkOrderAllocationReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.revokeAllocationWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.revokeAllocationWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "同级转派")
    @RequestMapping(path = "peerTransferWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> peerTransferWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderPeerTransferReq ycqYouthcallWorkOrderPeerTransferReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.peerTransferWorkOrder(ycqYouthcallWorkOrderPeerTransferReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.peerTransferWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.peerTransferWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "工单撤销同级转派")
    @RequestMapping(path = "revokePeerTransferWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> revokePeerTransferWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderPeerTransferReq ycqYouthcallWorkOrderPeerTransferReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.revokePeerTransferWorkOrder(ycqYouthcallWorkOrderPeerTransferReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.revokePeerTransferWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.revokePeerTransferWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "工单回访")
    @RequestMapping(path = "followUpWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> followUpWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderFollowUpReq ycqYouthcallWorkOrderFollowUpReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.followUpWorkOrder(ycqYouthcallWorkOrderFollowUpReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.followUpWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.followUpWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "工单归档")
    @RequestMapping(path = "fileWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> fileWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderFileReq ycqYouthcallWorkOrderFileReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.fileWorkOrder(ycqYouthcallWorkOrderFileReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.fileWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.fileWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "工单评价")
    @RequestMapping(path = "evaluateWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> evaluateWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderEvaluateReq ycqYouthcallWorkOrderEvaluateReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.evaluateWorkOrder(ycqYouthcallWorkOrderEvaluateReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.evaluateWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.evaluateWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "工单回复")
    @RequestMapping(path = "replyWorkOrder", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> replyWorkOrder(@Valid @RequestBody YcqYouthcallWorkOrderReplyReq ycqYouthcallWorkOrderReplyReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.replyWorkOrder(ycqYouthcallWorkOrderReplyReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.replyWorkOrder.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.replyWorkOrder.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据工单id查询工单详情(APP专用)")
    @RequestMapping(path = "queryDetailAppById", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> queryDetailAppById(@RequestParam("id")BigInteger id) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.queryDetailAppById(id);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.queryDetailAppById.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.queryDetailAppById.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据工单id查询工单详情")
    @RequestMapping(path = "queryDetailById", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> queryDetailById(@RequestParam("id")BigInteger id) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.queryDetailById(id);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.queryDetailById.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.queryDetailById.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("归档工单excel导出")
    @RequestMapping(value = "/exportFileWorkOrderExcel", method = {RequestMethod.POST})
    @ResponseBody
    public Result<Boolean> exportFileWorkOrderExcel(HttpServletResponse response, @RequestBody YcqYouthcallWorkOrderExportReq req) {
        log.info("YcqYouthcallWorkOrderController.exportFileWorkOrderExcel.param：{}", req.toString());
        try {
            ycqYouthcallWorkOrderServiceImpl.exportFileWorkOrderExcel(response,req);
            return Result.ofSuccess(true);
        } catch (BusinessException e) {
            log.error("YcqYjsNegativePersonController.exportNegativePersonExcel.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYjsNegativePersonController.exportNegativePersonExcel.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
