package com.crunii.micro.service.youthcall.controller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.youthcall.dto.vo.orderDispatch.*;
import com.crunii.micro.service.youthcall.service.YcqYouthcallWorkOrderDispatchService;
import com.crunii.micro.web.mvc.CurrWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * @author : yingtiansheng
 * @date : 2023-10-31
 */
@Slf4j
@Api(tags = "工单调度管理")
@RestController
@RequestMapping("/ycqYouthcallWorkOrderDispatch/auth")
public class YcqYouthcallWorkOrderDispatchController {

    @Resource
    private YcqYouthcallWorkOrderDispatchService ycqYouthcallWorkOrderDispatchService;

    @ApiOperation(value = "分页查询工单调度")
    @RequestMapping(path = "pageQuery", method = RequestMethod.POST)
    @ResponseBody
    public Result<PageRsp<WorkOrderDispatchVo>> pageQuery(@RequestBody PageReq<YcqYouthcallWorkOrderDispatchReq> req) {
        try {
            return ycqYouthcallWorkOrderDispatchService.pageQuery(req);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderDispatchController.pageQuery.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderDispatchController.pageQuery.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "新增工单调度")
    @RequestMapping(path = "create", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> create(@Valid @RequestBody YcqYouthcallWorkOrderDispatchInsertVo ycqYouthcallWorkOrderDispatchInsertVo, @ApiIgnore @CurrWorker Worker worker) {
        try {
            return ycqYouthcallWorkOrderDispatchService.insertWorkOrderDispatch(ycqYouthcallWorkOrderDispatchInsertVo,worker);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderDispatchController.create.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderDispatchController.create.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "修改工单调度工单类型")
    @RequestMapping(path = "update", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> update(@Valid @RequestBody YcqYouthcallWorkOrderDispatchUpdateVo ycqYouthcallWorkOrderDispatchUpdateVo, @ApiIgnore @CurrWorker Worker worker) {
        try {
            return ycqYouthcallWorkOrderDispatchService.updateWorkOrderDispatch(ycqYouthcallWorkOrderDispatchUpdateVo,worker);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderDispatchController.update.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderDispatchController.update.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "切换工单调度状态")
    @RequestMapping(path = "switchStatus", method = RequestMethod.POST)
    @ResponseBody
    public Result<Boolean> switchStatus(@Valid @RequestBody YcqYouthcallWorkOrderDispatchStatusVo ycqYouthcallWorkOrderDispatchStatusVo, @ApiIgnore @CurrWorker Worker worker) {
        try {
            return ycqYouthcallWorkOrderDispatchService.switchStatus(ycqYouthcallWorkOrderDispatchStatusVo,worker);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderDispatchController.switchStatus.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderDispatchController.switchStatus.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation("删除")
    @GetMapping("del")
    public Result<?> del(BigInteger id){
        if (id == null) {
            log.error("YcqYouthcallWorkOrderDispatchController.del.error. id is null");
            return Result.ofFail("系统内部异常，请联系管理员！");
        } else {
            ycqYouthcallWorkOrderDispatchService.deleteWorkOrderDispatch(id);
            return Result.ofSuccess(true);
        }
    }

    @ApiOperation(value = "查询所有的已经有的组织")
    @RequestMapping(path = "getExitOrgByType", method = RequestMethod.GET)
    @ResponseBody
    public Result<AppOrgVo> getExitOrgByType(String workOrderType) {
        try {
            if (workOrderType == null) {
                log.error("YcqYouthcallWorkOrderDispatchController.getExitOrgByType.error. workOrderType is null");
                return Result.ofFail("系统内部异常，请联系管理员！");
            }
            List<String> list = new ArrayList<>();
            AppOrgVo org = ycqYouthcallWorkOrderDispatchService.findAppAndOrgByTypeOnAndOff(workOrderType);
            return Result.ofSuccess(org);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderDispatchController.getExitOrgByType.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderDispatchController.getExitOrgByType.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
