package com.crunii.micro.service.youthcall.dto;

import com.crunii.micro.common.dto.OrderProp;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallCheckInApplyQueryCondition;
import com.crunii.micro.service.youthcall.utils.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author PANJL
 * @date 2023-12-16 17:49
 */
@Slf4j
@Repository
public class YcqYouthcallCheckInApplyDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public static final String PAGE_SQL ="SELECT " +
            "a.apply_id, a.apply_user_name, a.id_card, a.apply_type, a.hotel_name, a.actual_audit_status_label, a.check_in_date, a.check_in_days, a.apply_time " +
            "FROM YCQ_YOUTHCALL_HS_CHECKIN_APPLY a WHERE a.del_flag = 0 ";

    public static final String PAGE_COUNT_SQL ="SELECT count(1) FROM YCQ_YOUTHCALL_HS_CHECKIN_APPLY a WHERE a.del_flag = 0 ";

    public PageRsp<YcqYouthcallCheckInApply> pageQuery(PageReq<YcqYouthcallCheckInApplyQueryCondition> req){
        log.info("YcqYouthcallCheckInApplyDao.pageQuery.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        // 申请人id
        String applyUserId = req.getQuery().getApplyUserId();
        if (StringUtils.isNotEmpty(applyUserId)) {
            String fragment = " and a.apply_user_id = ? ";
            sql.append(fragment);
            countSql.append(fragment);
            param.add(applyUserId);
        }
        // 申请人姓名
        String applyUserName = req.getQuery().getApplyUserName();
        if (StringUtils.isNotEmpty(applyUserName)) {
            String fragment = " and a.apply_user_name like '%?%' ";
            sql.append(fragment);
            countSql.append(fragment);
            param.add(applyUserName);
        }
        // 驿站id
        String hotelId = req.getQuery().getHotelId();
        if (StringUtils.isNotEmpty(hotelId)) {
            String fragment = " and a.hotel_id = ? ";
            sql.append(fragment);
            countSql.append(fragment);
            param.add(hotelId);
        }
        // 驿站名称
        String hotelName = req.getQuery().getHotelName();
        if (StringUtils.isNotEmpty(hotelName)) {
            String fragment = " and a.hotel_name like '%?%' ";
            sql.append(fragment);
            countSql.append(fragment);
            param.add(hotelName);
        }
        // 审核状态
        String auditStatus = req.getQuery().getAuditStatus();
        if (StringUtils.isNotEmpty(auditStatus)) {
            String fragment = " and a.actual_audit_status_label = ? ";
            sql.append(fragment);
            countSql.append(fragment);
            param.add(applyUserName);
        }
        // 身份证号
        String idCard = req.getQuery().getIdCard();
        if (StringUtils.isNotEmpty(idCard)) {
            String fragment = " and a.id_card like '%?%' ";
            sql.append(fragment);
            countSql.append(fragment);
            param.add(applyUserName);
        }

        List<OrderProp> orderProps = req.getOrderProps();
        // 默认按申请时间倒叙排
        if (CollectionUtils.isEmpty(orderProps)) {
            sql.append(" order by a.apply_time desc NULLS LAST ");
        } else {
            sql.append(" order by a.apply_time desc, ");
            for (int i = 0; i < orderProps.size(); i++) {
                OrderProp orderProp = orderProps.get(i);
                boolean ascending = orderProp.isAscending();
                // 根据属性名找到注解
                String prop = orderProp.getProp();
                String tableFiled = BeanUtil.gettAnnotations(YcqYouthcallPersonalDemand.class, prop);
                if (i== orderProps.size()-1) {
                    if (ascending) {
                        sql.append(tableFiled).append(" asc ");
                    } else {
                        sql.append(tableFiled).append(" desc ");
                    }
                } else {
                    if (ascending) {
                        sql.append(tableFiled).append(" asc, ");
                    } else {
                        sql.append(tableFiled).append(" desc, ");
                    }
                }
            }
            sql.append(" NULLS LAST ");
        }
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<YcqYouthcallCheckInApply> result;
        // 是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(YcqYouthcallCheckInApply.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<YcqYouthcallCheckInApply> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }

}
