package com.crunii.micro.service.youthcall.dto;

import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.youthcall.enums.DelFlagEnum;
import com.crunii.micro.service.youthcall.enums.WorkOrderStatusEnum;
import com.crunii.micro.service.youthcall.utils.CharacterCodeUtil;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

@Entity
@Table(name="YCQ_YOUTHCALL_WORK_ORDER")
@NamedQuery(name = "YcqYouthcallWorkOrder.findAll", query = "SELECT t FROM YcqYouthcallWorkOrder t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = {"id"})
@ApiModel(description = "青呼我为-工单信息表")
@Data
public class YcqYouthcallWorkOrder implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    @ApiModelProperty(name = "主键",notes = "")
    private BigInteger id;

    @Column(name = "work_order_no")
    @ApiModelProperty(name = "工单编号",notes = "")
    private String workOrderNo;

    @Column(name = "work_order_name")
    @ApiModelProperty(name = "工单标题",notes = "")
    private String workOrderName;

    @Column(name = "work_order_type")
    @ApiModelProperty(name = "工单类型",notes = "")
    private String workOrderType;

    @Column(name = "affiliated_business_id")
    @ApiModelProperty(name = "所属业务",notes = "")
    private BigInteger affiliatedBusinessId;

    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Column(name = "org_id")
    @ApiModelProperty(name = "所属组织id",notes = "")
    private BigInteger orgId;

    @Column(name = "org_name")
    @ApiModelProperty(name = "所属组织名称",notes = "")
    private String orgName;

    @Column(name = "process_status")
    @ApiModelProperty(name = "工单状态",notes = "")
    private String processStatus;

    @Column(name = "source")
    @ApiModelProperty(name = "工单来源",notes = "")
    private String source;

    @Column(name = "proposer_id")
    @ApiModelProperty(name = "提出人id",notes = "")
    private BigInteger proposerId;

    @Column(name = "proposer_name")
    @ApiModelProperty(name = "提出人名称",notes = "")
    private String proposerName;

    @Column(name = "proposed_time")
    @ApiModelProperty(name = "提出时间",notes = "")
    private Date proposedTime;

    @Column(name = "process_user_id")
    @ApiModelProperty(name = "处理人id",notes = "")
    private BigInteger processUserId;

    @Column(name = "process_user_name")
    @ApiModelProperty(name = "处理人名称",notes = "")
    private String processUserName;

    @Column(name = "process_time")
    @ApiModelProperty(name = "处理时间",notes = "")
    private Date processTime;

    @Column(name = "del_flag")
    @ApiModelProperty(name = "删除标识(1:删除，0:未删除)",notes = "")
    private Integer delFlag;

    @Column(name = "work_order_content")
    @ApiModelProperty(name = "工单内容",notes = "")
    private String workOrderContent;

    @Column(name = "proposer_mobile")
    @ApiModelProperty(name = "提出人联系方式",notes = "")
    private String proposerMobile;

    @Column(name = "app_name")
    @ApiModelProperty(name = "业务名称",notes = "")
    private String appName;

    @Column(name = "app_code")
    @ApiModelProperty(name = "业务code",notes = "")
    private String appCode;

    @Column(name = "work_order_img")
    @ApiModelProperty(name = "工单图片",notes = "")
    private String workOrderImg;

    @Column(name = "source_id")
    @ApiModelProperty(name = "工单来源唯一标识(青呼码qrcode、)",notes = "")
    private String sourceId;

    @Column(name = "process_mobile")
    @ApiModelProperty(name = "处理人联系方式",notes = "")
    private String processMobile;

    @Column(name = "flow_instance_id")
    @ApiModelProperty(name = "工作流id",notes = "")
    private BigInteger flowInstanceId;

    @Column(name = "phone")
    @ApiModelProperty(name = "提出诉求时用户留下的手机号",notes = "")
    private String phone;

    @Column(name = "zone_id")
    @ApiModelProperty(name = "所属区域id",notes = "")
    private Integer zoneId;

    public static YcqYouthcallWorkOrder commonDemandToWorkOrder(YcqYouthcallCommonDemand ycqYouthcallCommonDemand, Worker worker) {
        if (ycqYouthcallCommonDemand == null) {
            return null;
        } else {
            YcqYouthcallWorkOrder ycqYouthcallWorkOrder = new YcqYouthcallWorkOrder();
            ycqYouthcallWorkOrder.setWorkOrderNo(CharacterCodeUtil.generateCharacterCode("QHWW"));
            ycqYouthcallWorkOrder.setWorkOrderName(ycqYouthcallCommonDemand.getDemandTitle());
            ycqYouthcallWorkOrder.setWorkOrderType(ycqYouthcallCommonDemand.getDemandType());
            ycqYouthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.PENDING.getValue()); //待处理
            ycqYouthcallWorkOrder.setSource("01"); //一般建议
            ycqYouthcallWorkOrder.setProposerId(BigInteger.valueOf(worker.getLoginId()));
            ycqYouthcallWorkOrder.setProposerName(worker.getWorkerName());
            ycqYouthcallWorkOrder.setProposerMobile(worker.getMobile());
            ycqYouthcallWorkOrder.setProposedTime(new Date());
            ycqYouthcallWorkOrder.setWorkOrderContent(ycqYouthcallCommonDemand.getDemandContent());
            ycqYouthcallWorkOrder.setAppName(ycqYouthcallCommonDemand.getAppName());
            ycqYouthcallWorkOrder.setAppCode(ycqYouthcallCommonDemand.getAppCode());
            ycqYouthcallWorkOrder.setWorkOrderImg(ycqYouthcallCommonDemand.getDemandImg());
            ycqYouthcallWorkOrder.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
            return ycqYouthcallWorkOrder;
        }
    }
}
