package com.crunii.micro.service.youthcall.dto;

import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.youthcall.dto.vo.workOrder.*;
import com.crunii.micro.service.youthcall.enums.DelFlagEnum;
import com.crunii.micro.service.youthcall.enums.WorkOrderStatusEnum;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.util.CollectionUtils;

import javax.persistence.*;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

@Entity
@Table(name = "YCQ_YOUTHCALL_WORK_ORDER_PROCESS")
@NamedQuery(name = "YcqYouthcallWorkOrderProcess.findAll", query = "SELECT t FROM YcqYouthcallWorkOrderProcess t")
@DynamicInsert
@DynamicUpdate
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = {"id"})
@ApiModel(description = "青呼我为-工单流程表")
public class YcqYouthcallWorkOrderProcess implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @ApiModelProperty(name = "id", notes = "主键")
    @Column(name = "id")
    private BigInteger id;

    @Column(name = "work_order_id")
    @ApiModelProperty(name = "工单表主键",notes = "")
    private BigInteger workOrderId;

    @Column(name = "process_status")
    @ApiModelProperty(name = "工单状态",notes = "")
    private String processStatus;

    @Column(name = "deal_content")
    @ApiModelProperty(name = "处理内容、上报下派说明、评价内容、回访内容",notes = "")
    private String dealContent;

    @Column(name = "attachment")
    @ApiModelProperty(name = "附件",notes = "")
    private String attachment;

    @Column(name = "report_type")
    @ApiModelProperty(name = "上报对象(1:上级组织 2:综治系统 3:民呼我为)",notes = "")
    private String reportType;

    @Column(name = "priority")
    @ApiModelProperty(name = "工单优先级(上报、下派: 1:低 2:中 3:高)",notes = "")
    private String priority;

    @Column(name = "transfer_org_id")
    @ApiModelProperty(name = "流转组织id(上报、下派)",notes = "")
    private BigInteger transferOrgId;

    @Column(name = "transfer_org_name")
    @ApiModelProperty(name = "流转组织名称(上报、下派)",notes = "")
    private String transferOrgName;

    @Column(name = "evaluate_star")
    @ApiModelProperty(name = "评价星级",notes = "")
    private String evaluateStar;

    @Column(name = "follow_up_type")
    @ApiModelProperty(name = "回访方式",notes = "")
    private String followUpType;

    @ApiModelProperty(name = "createdTime", notes = "创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @Column(name = "created_time")
    private Date createdTime;

    @ApiModelProperty(name = "createdId", notes = "创建人ID")
    @Column(name = "created_id")
    private Integer createdId;

    @ApiModelProperty(name = "createdName", notes = "创建人名称")
    @Column(name = "created_name")
    private String createdName;

    @ApiModelProperty(name = "createdMobile", notes = "创建人联系方式")
    @Column(name = "created_mobile")
    private String createdMobile;

    @ApiModelProperty(name = "lastUpdatedTime", notes = "最后更新时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @Column(name = "last_updated_time")
    private Date lastUpdatedTime;

    @ApiModelProperty(name = "lastUpdatedId", notes = "最后更新人ID")
    @Column(name = "last_updated_id")
    private Integer lastUpdatedId;

    @ApiModelProperty(name = "delFlag", notes = "是否删除：0-未删除；1-已删除")
    @Column(name = "del_flag")
    private Integer delFlag;

    public static YcqYouthcallWorkOrderProcess dealProcess(BigInteger workOrderId, String dealContent, String attachment, Worker worker) {
        Date curr = new Date();
        YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = new YcqYouthcallWorkOrderProcess();
        ycqYouthcallWorkOrderProcess.setWorkOrderId(workOrderId);
        ycqYouthcallWorkOrderProcess.setProcessStatus(WorkOrderStatusEnum.DONE.getValue());
        ycqYouthcallWorkOrderProcess.setDealContent(dealContent);
        ycqYouthcallWorkOrderProcess.setAttachment(attachment);
        if (worker != null) {
            ycqYouthcallWorkOrderProcess.setCreatedId(worker.getLoginId());
            ycqYouthcallWorkOrderProcess.setCreatedName(worker.getWorkerName());
            ycqYouthcallWorkOrderProcess.setCreatedMobile(worker.getMobile());
            ycqYouthcallWorkOrderProcess.setLastUpdatedId(worker.getLoginId());
        }
        ycqYouthcallWorkOrderProcess.setCreatedTime(curr);
        ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
        ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        return ycqYouthcallWorkOrderProcess;
    }

    public static YcqYouthcallWorkOrderProcess escalationProcess(YcqYouthcallWorkOrderEscalationReq ycqYouthcallWorkOrderEscalationReq, Worker worker) {
        Date curr = new Date();
        YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = new YcqYouthcallWorkOrderProcess();
        ycqYouthcallWorkOrderProcess.setWorkOrderId(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId());
        ycqYouthcallWorkOrderProcess.setProcessStatus(WorkOrderStatusEnum.PENDING_ESCALATION.getValue());
        ycqYouthcallWorkOrderProcess.setDealContent(ycqYouthcallWorkOrderEscalationReq.getDealContent());
        ycqYouthcallWorkOrderProcess.setReportType(ycqYouthcallWorkOrderEscalationReq.getReportType());
        ycqYouthcallWorkOrderProcess.setPriority(ycqYouthcallWorkOrderEscalationReq.getPriority());
        ycqYouthcallWorkOrderProcess.setTransferOrgId(ycqYouthcallWorkOrderEscalationReq.getTransferOrgId());
        ycqYouthcallWorkOrderProcess.setTransferOrgName(ycqYouthcallWorkOrderEscalationReq.getTransferOrgName());
        if (worker != null) {
            ycqYouthcallWorkOrderProcess.setCreatedId(worker.getLoginId());
            ycqYouthcallWorkOrderProcess.setCreatedName(worker.getWorkerName());
            ycqYouthcallWorkOrderProcess.setCreatedMobile(worker.getMobile());
            ycqYouthcallWorkOrderProcess.setLastUpdatedId(worker.getLoginId());
        }
        ycqYouthcallWorkOrderProcess.setCreatedTime(curr);
        ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
        ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        return ycqYouthcallWorkOrderProcess;
    }

    public static YcqYouthcallWorkOrderProcess allocationProcess(YcqYouthcallWorkOrderAllocationReq ycqYouthcallWorkOrderAllocationReq, Worker worker) {
        Date curr = new Date();
        YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = new YcqYouthcallWorkOrderProcess();
        ycqYouthcallWorkOrderProcess.setWorkOrderId(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId());
        ycqYouthcallWorkOrderProcess.setProcessStatus(WorkOrderStatusEnum.PENDING_ALLOCATION.getValue());
        ycqYouthcallWorkOrderProcess.setDealContent(ycqYouthcallWorkOrderAllocationReq.getDealContent());
        ycqYouthcallWorkOrderProcess.setPriority(ycqYouthcallWorkOrderAllocationReq.getPriority());
        ycqYouthcallWorkOrderProcess.setTransferOrgId(ycqYouthcallWorkOrderAllocationReq.getTransferOrgId());
        ycqYouthcallWorkOrderProcess.setTransferOrgName(ycqYouthcallWorkOrderAllocationReq.getTransferOrgName());
        if (worker != null) {
            ycqYouthcallWorkOrderProcess.setCreatedId(worker.getLoginId());
            ycqYouthcallWorkOrderProcess.setCreatedName(worker.getWorkerName());
            ycqYouthcallWorkOrderProcess.setCreatedMobile(worker.getMobile());
            ycqYouthcallWorkOrderProcess.setLastUpdatedId(worker.getLoginId());
        }
        ycqYouthcallWorkOrderProcess.setCreatedTime(curr);
        ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
        ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        return ycqYouthcallWorkOrderProcess;
    }

    public static YcqYouthcallWorkOrderProcess peerTransferProcess(YcqYouthcallWorkOrderPeerTransferReq ycqYouthcallWorkOrderPeerTransferReq, Worker worker) {
        Date curr = new Date();
        YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = new YcqYouthcallWorkOrderProcess();
        ycqYouthcallWorkOrderProcess.setWorkOrderId(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId());
        ycqYouthcallWorkOrderProcess.setProcessStatus(WorkOrderStatusEnum.PENDING_PEER_TRANSFER.getValue());
        ycqYouthcallWorkOrderProcess.setDealContent(ycqYouthcallWorkOrderPeerTransferReq.getDealContent());
        ycqYouthcallWorkOrderProcess.setPriority(ycqYouthcallWorkOrderPeerTransferReq.getPriority());
        ycqYouthcallWorkOrderProcess.setTransferOrgId(ycqYouthcallWorkOrderPeerTransferReq.getTransferOrgId());
        ycqYouthcallWorkOrderProcess.setTransferOrgName(ycqYouthcallWorkOrderPeerTransferReq.getTransferOrgName());
        if (worker != null) {
            ycqYouthcallWorkOrderProcess.setCreatedId(worker.getLoginId());
            ycqYouthcallWorkOrderProcess.setCreatedName(worker.getWorkerName());
            ycqYouthcallWorkOrderProcess.setCreatedMobile(worker.getMobile());
            ycqYouthcallWorkOrderProcess.setLastUpdatedId(worker.getLoginId());
        }
        ycqYouthcallWorkOrderProcess.setCreatedTime(curr);
        ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
        ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        return ycqYouthcallWorkOrderProcess;
    }

    public static YcqYouthcallWorkOrderProcess followUpProcess(YcqYouthcallWorkOrderFollowUpReq ycqYouthcallWorkOrderFollowUpReq, Worker worker) {
        Date curr = new Date();
        YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = new YcqYouthcallWorkOrderProcess();
        ycqYouthcallWorkOrderProcess.setWorkOrderId(ycqYouthcallWorkOrderFollowUpReq.getWorkOrderId());
        ycqYouthcallWorkOrderProcess.setProcessStatus(WorkOrderStatusEnum.FOLLOW_UP.getValue());
        ycqYouthcallWorkOrderProcess.setDealContent(ycqYouthcallWorkOrderFollowUpReq.getDealContent());
        ycqYouthcallWorkOrderProcess.setFollowUpType(ycqYouthcallWorkOrderFollowUpReq.getFollowUpType());
        if (worker != null) {
            ycqYouthcallWorkOrderProcess.setCreatedId(worker.getLoginId());
            ycqYouthcallWorkOrderProcess.setCreatedName(worker.getWorkerName());
            ycqYouthcallWorkOrderProcess.setCreatedMobile(worker.getMobile());
            ycqYouthcallWorkOrderProcess.setLastUpdatedId(worker.getLoginId());
        }
        ycqYouthcallWorkOrderProcess.setCreatedTime(curr);
        ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
        ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        return ycqYouthcallWorkOrderProcess;
    }

    public static YcqYouthcallWorkOrderProcess fileProcess(YcqYouthcallWorkOrderFileReq ycqYouthcallWorkOrderFileReq, Worker worker) {
        Date curr = new Date();
        YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = new YcqYouthcallWorkOrderProcess();
        ycqYouthcallWorkOrderProcess.setWorkOrderId(ycqYouthcallWorkOrderFileReq.getWorkOrderId());
        ycqYouthcallWorkOrderProcess.setProcessStatus(WorkOrderStatusEnum.FILE.getValue());
        if (worker != null) {
            ycqYouthcallWorkOrderProcess.setCreatedId(worker.getLoginId());
            ycqYouthcallWorkOrderProcess.setCreatedName(worker.getWorkerName());
            ycqYouthcallWorkOrderProcess.setCreatedMobile(worker.getMobile());
            ycqYouthcallWorkOrderProcess.setLastUpdatedId(worker.getLoginId());
        }
        ycqYouthcallWorkOrderProcess.setCreatedTime(curr);
        ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
        ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        return ycqYouthcallWorkOrderProcess;
    }

    public static YcqYouthcallWorkOrderProcess evaluateProcess(YcqYouthcallWorkOrderEvaluateReq ycqYouthcallWorkOrderEvaluateReq, Worker worker) {
        Date curr = new Date();
        YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = new YcqYouthcallWorkOrderProcess();
        ycqYouthcallWorkOrderProcess.setWorkOrderId(ycqYouthcallWorkOrderEvaluateReq.getWorkOrderId());
        ycqYouthcallWorkOrderProcess.setProcessStatus(WorkOrderStatusEnum.EVALUATE.getValue());
        ycqYouthcallWorkOrderProcess.setDealContent(ycqYouthcallWorkOrderEvaluateReq.getDealContent());
        ycqYouthcallWorkOrderProcess.setEvaluateStar(ycqYouthcallWorkOrderEvaluateReq.getEvaluateStar());
        if (worker != null) {
            ycqYouthcallWorkOrderProcess.setCreatedId(worker.getLoginId());
            ycqYouthcallWorkOrderProcess.setCreatedName(worker.getWorkerName());
            ycqYouthcallWorkOrderProcess.setCreatedMobile(worker.getMobile());
            ycqYouthcallWorkOrderProcess.setLastUpdatedId(worker.getLoginId());
        }
        ycqYouthcallWorkOrderProcess.setCreatedTime(curr);
        ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
        ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        return ycqYouthcallWorkOrderProcess;
    }

    public static YcqYouthcallWorkOrderProcess replyProcess(YcqYouthcallWorkOrderReplyReq ycqYouthcallWorkOrderReplyReq, Worker worker) {
        Date curr = new Date();
        YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = new YcqYouthcallWorkOrderProcess();
        ycqYouthcallWorkOrderProcess.setWorkOrderId(ycqYouthcallWorkOrderReplyReq.getWorkOrderId());
        ycqYouthcallWorkOrderProcess.setProcessStatus(WorkOrderStatusEnum.REPLY.getValue());
        ycqYouthcallWorkOrderProcess.setDealContent(ycqYouthcallWorkOrderReplyReq.getDealContent());
        // 判断是否有附件
        if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderReplyReq.getAttachmentList())) {
            ycqYouthcallWorkOrderProcess.setAttachment(String.join(",", ycqYouthcallWorkOrderReplyReq.getAttachmentList()));
        }
        if (worker != null) {
            ycqYouthcallWorkOrderProcess.setCreatedId(worker.getLoginId());
            ycqYouthcallWorkOrderProcess.setCreatedName(worker.getWorkerName());
            ycqYouthcallWorkOrderProcess.setCreatedMobile(worker.getMobile());
            ycqYouthcallWorkOrderProcess.setLastUpdatedId(worker.getLoginId());
        }
        ycqYouthcallWorkOrderProcess.setCreatedTime(curr);
        ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
        ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        return ycqYouthcallWorkOrderProcess;
    }

    public static YcqYouthcallWorkOrderProcess pendingProcess(YcqYouthcallWorkOrder order) {
        YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = new YcqYouthcallWorkOrderProcess();
        ycqYouthcallWorkOrderProcess.setWorkOrderId(order.getId());
        ycqYouthcallWorkOrderProcess.setProcessStatus(WorkOrderStatusEnum.PENDING.getValue());
        ycqYouthcallWorkOrderProcess.setCreatedTime(order.getProposedTime());
        ycqYouthcallWorkOrderProcess.setCreatedName(order.getProposerName());
        ycqYouthcallWorkOrderProcess.setDealContent("提出了工单需求");
        ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        if (order.getProposerId()!=null){
            ycqYouthcallWorkOrderProcess.setCreatedId(order.getProposerId().intValue());
            ycqYouthcallWorkOrderProcess.setLastUpdatedId(order.getProposerId().intValue());
        }
        ycqYouthcallWorkOrderProcess.setLastUpdatedTime(order.getProposedTime());
        ycqYouthcallWorkOrderProcess.setCreatedMobile(order.getProposerMobile());
        return ycqYouthcallWorkOrderProcess;
    }
}
