package com.crunii.micro.service.youthcall.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.youthcall.dto.YcqPublishActivity;
import com.crunii.micro.service.youthcall.dto.YcqPublishActivityResp;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallQrcode;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrder;
import com.crunii.micro.service.youthcall.dto.vo.YcqPublishActivityVo;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallQrcodeVo;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

@Repository
public class YcqPublishActivityDao {
    // 根据青呼码id查询关联的活动和未关联的活动
    private static String QUERY_BY_QRCODEID="select a.id,a.activity_title,a.sponsor,a.start_time,a.end_time from YCQ_PUBLISH_ACTIVITY a where 1=1 and a.IS_DELETED = 0 ";

    private static String QUERY_ACTIVITY=" SELECT "+
            " a.*, (SELECT count(1) FROM YCQ_ACTIVITY_REGISTER b WHERE a.ID = b.ACTIVITY_ID) AS join_person,"+
            " b.QR_CODE_INLET FROM"+
            " YCQ_PUBLISH_ACTIVITY a,YCQ_YOUTHCALL_QRCODE_ACTIVITY_REL b  where 1=1 AND a.ID = b.activity_id and a.IS_DELETED = 0 ";

    private static String QUERY_ACTIVITY_COUNT_SQL=" SELECT count(1) FROM"+
            " YCQ_PUBLISH_ACTIVITY a,YCQ_YOUTHCALL_QRCODE_ACTIVITY_REL b  where 1=1 AND a.ID = b.activity_id and a.IS_DELETED = 0 ";

    public static final String PAGE_COUNT_SQL ="select count(1) from YCQ_PUBLISH_ACTIVITY a where 1 = 1 and a.IS_DELETED=0 ";


    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    public List<YcqPublishActivity> paginQuery(YcqPublishActivityVo ycqPublishActivityVo) {
        StringBuilder sb = new StringBuilder(QUERY_BY_QRCODEID);
        List<Object> params = new ArrayList<>();
        String activityTitle = ycqPublishActivityVo.getActivityTitle();
        // 活动名称
        if (StringUtils.hasText(activityTitle)) {
            sb.append(" and a.activity_title like concat('%',?,'%') ");
            params.add(activityTitle);
        }

        BigInteger qrcodeId = ycqPublishActivityVo.getQrcodeId();
        if (qrcodeId != null) {
            Boolean isActivity = ycqPublishActivityVo.getIsActivity();
            // 已关联活动
            if (isActivity) {
                sb.append(" AND EXISTS (SELECT 1 FROM YCQ_YOUTHCALL_QRCODE_ACTIVITY_REL b WHERE a.ID = b.ACTIVITY_ID  and b.QRCODE_ID= ?) ");
                params.add(qrcodeId);
            }
            // 未关联活动
            if (!isActivity) {
                sb.append(" AND not EXISTS (SELECT 1 FROM YCQ_YOUTHCALL_QRCODE_ACTIVITY_REL b WHERE a.ID = b.ACTIVITY_ID  and b.QRCODE_ID= ?) ");
                params.add(qrcodeId);
            }
        }

        sb.append(" order by a.create_time desc ");
        List<YcqPublishActivity> list = JT.queryForBeans(YcqPublishActivity.class, sb.toString(),params.toArray());
        return list;
    }

    public PageRsp<YcqPublishActivityResp> queryQrcodeActivityByPage(PageReq<YcqPublishActivityVo> req) {
        StringBuilder sb = new StringBuilder(QUERY_ACTIVITY);
        StringBuilder sbCount = new StringBuilder(QUERY_ACTIVITY_COUNT_SQL);
        List<Object> params = new ArrayList<>();
        String activityTitle = req.getQuery().getActivityTitle();
        Integer activityState = req.getQuery().getActivityState();
        // 活动名称
        if (StringUtils.hasText(activityTitle)) {
            sb.append(" and a.activity_title like concat('%',?,'%') ");
            sbCount.append(" and a.activity_title like concat('%',?,'%') ");
            params.add(activityTitle);
        }
        // 活动状态
        if (activityState!=null) {
            sb.append(" and a.activity_state = ? ");
            sbCount.append(" and a.activity_state = ?  ");
            params.add(activityState);
        }
        // 活动时间
        if (StringUtils.hasText(req.getQuery().getStartTimeBegin())) {
            sb.append(" and a.start_time >= ? ");
            sbCount.append(" and a.start_time  >= ? ");
            params.add(req.getQuery().getStartTimeBegin());
        }

        if (StringUtils.hasText(req.getQuery().getStartTimeEnd())) {
            sb.append(" and a.end_time <= ? ");
            sbCount.append(" and a.end_time  <= ? ");
            params.add(req.getQuery().getStartTimeEnd());
        }

        // 青呼码id
        if (req.getQuery().getQrcodeId()!=null) {
            sb.append(" and b.qrcode_id = ? ");
            sbCount.append(" and b.qrcode_id = ?  ");
            params.add(req.getQuery().getQrcodeId());
        }

        sb.append(" order by a.create_time desc ");
        int pageNo = req.getPageNo();
        int pageSize = req.getPageSize();
        int count = JT.queryForInt(sbCount.toString(), params.toArray());
        List list = new ArrayList();
        if (count>0) {
            list = queryForPage(sb.toString(), pageNo, pageSize, params);
        }

        PageRsp<YcqPublishActivityResp> rsp = new PageRsp<>();
        rsp.setResult(list);
        rsp.setTotalNum(count);
        rsp.setPageNo(pageNo);
        rsp.setPageSize(pageSize);

        return rsp;
    }

    public List queryForPage(String sql, int pageNo, int pageSize, List<Object> params) {
        int offset = (pageNo - 1) * pageSize;
        sql = sql + " limit " + pageSize + " offset " + offset;
        return JT.queryForBeans(YcqPublishActivityResp.class, sql, params.toArray());
    }
}
