package com.crunii.micro.service.youthcall.jdbc;

import com.crunii.micro.common.dto.OrderProp;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallCommonDemand;
import com.crunii.micro.service.youthcall.dto.vo.commonDemand.CommonDemandVo;
import com.crunii.micro.service.youthcall.dto.vo.commonDemand.YcqYouthcallCommonDemandReq;
import com.crunii.micro.service.youthcall.utils.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Repository
public class YcqYouthcallCommonDemandDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public static final String PAGE_SQL ="SELECT a.*,subquery.person_demand_content_str FROM YCQ_YOUTHCALL_COMMON_DEMAND a LEFT JOIN ( " +
            "SELECT b.common_demand_id,LISTAGG(c.demand_content , '|||') WITHIN GROUP (ORDER BY c.created_time desc) AS person_demand_content_str " +
            "FROM YCQ_YOUTHCALL_COMMON_PERSONAL b LEFT JOIN YCQ_YOUTHCALL_PERSONAL_DEMAND c ON b.personal_demand_id = c.id AND b.del_flag = 0 AND c.del_flag = 0 " +
            "WHERE b.del_flag = 0 GROUP BY b.common_demand_id) subquery ON a.id = subquery.common_demand_id WHERE a.del_flag = 0 ";

    public static final String PAGE_COUNT_SQL ="SELECT count(1) FROM YCQ_YOUTHCALL_COMMON_DEMAND WHERE del_flag = 0 ";

    public PageRsp<CommonDemandVo> pageSearch(PageReq<YcqYouthcallCommonDemandReq> req){
        log.info("YcqYouthcallCommonDemandDao.pageSearch.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        // 共性需求编号
        if (StringUtils.isNotBlank(req.getQuery().getCommonDemandNo())) {
            sql.append(" and a.common_demand_no = ? ");
            countSql.append(" and common_demand_no = ? ");
            param.add(req.getQuery().getCommonDemandNo());
        }
        // 需求标题
        if (StringUtils.isNotBlank(req.getQuery().getDemandTitle())) {
            sql.append(" and a.demand_title like ? ");
            countSql.append(" and demand_title like ? ");
            param.add("%" + req.getQuery().getDemandTitle() + "%");
        }
        // 需求内容
        if (StringUtils.isNotBlank(req.getQuery().getDemandContent())) {
            sql.append(" and a.demand_content like ? ");
            countSql.append(" and demand_content like ? ");
            param.add("%" + req.getQuery().getDemandContent() + "%");
        }
        // 需求类型
        if (StringUtils.isNotBlank(req.getQuery().getDemandType())) {
            sql.append(" and a.demand_type = ? ");
            countSql.append(" and demand_type = ? ");
            param.add(req.getQuery().getDemandType());
        }
        // 所属业务
        if (StringUtils.isNotBlank(req.getQuery().getAppCode())) {
            sql.append(" and FIND_IN_SET(?, a.app_code) > 0 ");
            countSql.append(" and FIND_IN_SET(?, app_code) > 0  ");
            param.add(req.getQuery().getAppCode());
        }
        // 需求状态(01:待处理 02:已转工单)
        if (req.getQuery().getCommonDemandStatus() != null ) {
            sql.append(" and a.common_demand_status = ? ");
            countSql.append(" and common_demand_status = ? ");
            param.add(req.getQuery().getCommonDemandStatus());
        }
        // 创建时间 开始时间
        if (req.getQuery().getStartCreatedTime() != null) {
            sql.append(" and CAST(a.created_time AS DATE) >=  ? ");
            countSql.append(" and CAST(created_time AS DATE) >= ? ");
            param.add(req.getQuery().getStartCreatedTime());
        }
        // 创建时间 结束时间
        if (req.getQuery().getEndCreatedTime() != null) {
            sql.append(" and CAST(a.created_time AS DATE) <= ?  ");
            countSql.append(" and CAST(created_time AS DATE) <= ?  ");
            param.add(req.getQuery().getEndCreatedTime());
        }
        // 处理时间 开始时间
        if (req.getQuery().getStartDealTime() != null) {
            sql.append(" and CAST(a.deal_time AS DATE) >=  ? ");
            countSql.append(" and CAST(deal_time AS DATE) >= ? ");
            param.add(req.getQuery().getStartDealTime());
        }
        // 处理时间 结束时间
        if (req.getQuery().getEndDealTime() != null) {
            sql.append(" and CAST(a.deal_time AS DATE) <= ?  ");
            countSql.append(" and CAST(deal_time AS DATE) <= ?  ");
            param.add(req.getQuery().getEndDealTime());
        }

        List<OrderProp> orderProps = req.getOrderProps();
        // 默认按时间倒叙排
        if (CollectionUtils.isEmpty(orderProps)) {
            sql.append(" order by a.created_time desc NULLS LAST  ");
        } else {
            sql.append(" order by ");
            for (int i = 0; i < orderProps.size(); i++) {
                OrderProp orderProp = orderProps.get(i);
                boolean ascending = orderProp.isAscending();
                // 根据属性名找到注解
                String prop = orderProp.getProp();
                String tableFiled = BeanUtil.gettAnnotations(YcqYouthcallCommonDemand.class, prop);
                if (i== orderProps.size()-1) {
                    if (ascending) {
                        sql.append( tableFiled + " asc " );
                    } else {
                        sql.append( tableFiled + " desc " );
                    }
                } else {
                    if (ascending) {
                        sql.append( tableFiled + " asc, " );
                    } else {
                        sql.append( tableFiled + " desc, " );
                    }
                }
            }
            sql.append(" NULLS LAST ");
        }
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<CommonDemandVo> result;
        // 是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(CommonDemandVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<CommonDemandVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }
}
