package com.crunii.micro.service.youthcall.jdbc;

import com.crunii.micro.common.dto.OrderProp;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallPersonalDemand;
import com.crunii.micro.service.youthcall.dto.vo.personalDemand.PersonalDemandVo;
import com.crunii.micro.service.youthcall.dto.vo.personalDemand.YcqYouthcallPersonalDemandReq;
import com.crunii.micro.service.youthcall.utils.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Repository
public class YcqYouthcallPersonalDemandDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public static final String PAGE_SQL ="SELECT * FROM YCQ_YOUTHCALL_PERSONAL_DEMAND WHERE del_flag = 0 ";

    public static final String PAGE_COUNT_SQL ="SELECT count(1) FROM YCQ_YOUTHCALL_PERSONAL_DEMAND WHERE del_flag = 0 ";

    public PageRsp<PersonalDemandVo> pageSearch(PageReq<YcqYouthcallPersonalDemandReq> req){
        log.info("YcqYouthcallPersonalDemandDao.pageSearch.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        // 个性化需求id列表
        if (!CollectionUtils.isEmpty(req.getQuery().getIdList())) {
            // 构建动态的IN子句
            String inClause = req.getQuery().getIdList().stream()
                    .map(String::valueOf) // 将整数转换为字符串
                    .collect(Collectors.joining(", ", "(", ")")); // 生成"(1, 2, 3)"格式的子句
            sql.append(" and id IN  ").append(inClause);
            countSql.append(" and id IN  ").append(inClause);
        }
        // 提出人id
        if (req.getQuery().getProposerId() != null) {
            sql.append(" and created_id = ? ");
            countSql.append(" and created_id = ? ");
            param.add(req.getQuery().getProposerId());
        }
        // 个性化需求编号
        if (StringUtils.isNotBlank(req.getQuery().getPersonalDemandNo())) {
            sql.append(" and personal_demand_no = ? ");
            countSql.append(" and personal_demand_no = ? ");
            param.add(req.getQuery().getPersonalDemandNo());
        }
        // 需求内容
        if (StringUtils.isNotBlank(req.getQuery().getDemandContent())) {
            sql.append(" and demand_content like ? ");
            countSql.append(" and demand_content like ? ");
            param.add("%" + req.getQuery().getDemandContent() + "%");
        }
        // 需求状态(01:未转化 02:已转化 03:已撤销)
        if (req.getQuery().getPersonalDemandStatus() != null ) {
            sql.append(" and personal_demand_status = ? ");
            countSql.append(" and personal_demand_status = ? ");
            param.add(req.getQuery().getPersonalDemandStatus());
        }
        // 创建时间 开始时间
        if (req.getQuery().getStartCreatedTime() != null) {
            sql.append(" and CAST(created_time AS DATE) >=  ? ");
            countSql.append(" and CAST(created_time AS DATE) >= ? ");
            param.add(req.getQuery().getStartCreatedTime());
        }
        // 创建时间 结束时间
        if (req.getQuery().getEndCreatedTime() != null) {
            sql.append(" and CAST(created_time AS DATE) <= ?  ");
            countSql.append(" and CAST(created_time AS DATE) <= ?  ");
            param.add(req.getQuery().getEndCreatedTime());
        }
        // 处理时间 开始时间
        if (req.getQuery().getStartDealTime() != null) {
            sql.append(" and CAST(deal_time AS DATE) >=  ? ");
            countSql.append(" and CAST(deal_time AS DATE) >= ? ");
            param.add(req.getQuery().getStartDealTime());
        }
        // 处理时间 结束时间
        if (req.getQuery().getEndDealTime() != null) {
            sql.append(" and CAST(deal_time AS DATE) <= ?  ");
            countSql.append(" and CAST(deal_time AS DATE) <= ?  ");
            param.add(req.getQuery().getEndDealTime());
        }

        List<OrderProp> orderProps = req.getOrderProps();
        // 默认按时间倒叙排
        if (CollectionUtils.isEmpty(orderProps)) {
            sql.append(" order by created_time desc NULLS LAST  ");
        } else {
            sql.append(" order by ");
            for (int i = 0; i < orderProps.size(); i++) {
                OrderProp orderProp = orderProps.get(i);
                boolean ascending = orderProp.isAscending();
                // 根据属性名找到注解
                String prop = orderProp.getProp();
                String tableFiled = BeanUtil.gettAnnotations(YcqYouthcallPersonalDemand.class, prop);
                if (i== orderProps.size()-1) {
                    if (ascending) {
                        sql.append( tableFiled + " asc " );
                    } else {
                        sql.append( tableFiled + " desc " );
                    }
                } else {
                    if (ascending) {
                        sql.append( tableFiled + " asc, " );
                    } else {
                        sql.append( tableFiled + " desc, " );
                    }
                }
            }
            sql.append(" NULLS LAST ");
        }
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<PersonalDemandVo> result;
        // 是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(PersonalDemandVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<PersonalDemandVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }
}
