package com.crunii.micro.service.youthcall.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.youthcall.dto.vo.productAdviceVo.ProductAdviceVo;
import com.crunii.micro.service.youthcall.dto.vo.productAdviceVo.YcqYouthcallProductAdviceReq;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Repository
public class YcqYouthcallProductAdviceDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public static final String PAGE_SQL ="SELECT * FROM YCQ_YOUTHCALL_PRODUCT_ADVICE WHERE del_flag = 0 ";

    public static final String PAGE_COUNT_SQL ="SELECT count(1) FROM YCQ_YOUTHCALL_PRODUCT_ADVICE WHERE del_flag = 0 ";

    public PageRsp<ProductAdviceVo> pageSearch(PageReq<YcqYouthcallProductAdviceReq> req){
        log.info("YcqYouthcallProductAdviceDao.pageSearch.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        // 提出人id
        if (req.getQuery().getProposerId() != null) {
            sql.append(" and created_id = ? ");
            countSql.append(" and created_id = ? ");
            param.add(req.getQuery().getProposerId());
        }
        // 产品建议编号
        if (StringUtils.isNotBlank(req.getQuery().getAdviceNo())) {
            sql.append(" and advice_no = ? ");
            countSql.append(" and advice_no = ? ");
            param.add(req.getQuery().getAdviceNo());
        }
        // 产品建议标题
        if (StringUtils.isNotBlank(req.getQuery().getAdviceTitle())) {
            sql.append(" and advice_title like ? ");
            countSql.append(" and advice_title like ? ");
            param.add("%" + req.getQuery().getAdviceTitle() + "%");
        }
        // 产品建议类型(1:功能问题 2:用户体验 3:产品建议)
        if (req.getQuery().getAdviceType() != null ) {
            sql.append(" and advice_type = ? ");
            countSql.append(" and advice_type = ? ");
            param.add(req.getQuery().getAdviceType());
        }
        // 产品建议状态(1:未处理 2:已处理)
        if (req.getQuery().getAdviceStatus() != null ) {
            sql.append(" and advice_status = ? ");
            countSql.append(" and advice_status = ? ");
            param.add(req.getQuery().getAdviceStatus());
        }
        // 所属业务
        if (req.getQuery().getAppCode() != null ) {
            sql.append(" and app_code = ? ");
            countSql.append(" and app_code = ? ");
            param.add(req.getQuery().getAppCode());
        }
        // 创建时间 开始时间
        if (req.getQuery().getStartCreatedTime() != null) {
            sql.append(" and CAST(created_time AS DATE) >=  ? ");
            countSql.append(" and CAST(created_time AS DATE) >= ? ");
            param.add(req.getQuery().getStartCreatedTime());
        }
        // 创建时间 结束时间
        if (req.getQuery().getEndCreatedTime() != null) {
            sql.append(" and CAST(created_time AS DATE) <= ?  ");
            countSql.append(" and CAST(created_time AS DATE) <= ?  ");
            param.add(req.getQuery().getEndCreatedTime());
        }
        sql.append(" order by created_time desc ");
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<ProductAdviceVo> result;
        // 是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(ProductAdviceVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<ProductAdviceVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }
}
