package com.crunii.micro.service.youthcall.jdbc;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallQrcode;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallQrcodeVo;
import com.crunii.micro.service.youthcall.utils.BaseJdbcTemplateUtil;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Repository
public class YcqYouthcallQrcodeDao {

    public static final String PAGE_SQL ="SELECT * FROM YCQ_YOUTHCALL_QRCODE a where 1 = 1 and a.del_flag=0 ";

    public static final String PAGE_COUNT_SQL ="select count(1) from YCQ_YOUTHCALL_QRCODE a where 1 = 1 and a.del_flag=0 ";

    public static final String QUERY_SQL ="SELECT * FROM YCQ_YOUTHCALL_QRCODE a where a.id = ? ";

    public static final String QUERY_YOUTHCALLCODE_SQL = "SELECT * FROM YCQ_YOUTHCALL_QRCODE a where a.youthcall_code = ? and a.del_flag=0";

    // 修改语句
    public static final String UPDATE_SQL ="update YCQ_YOUTHCALL_QRCODE a " +
            " set a.youthcall_name=?,a.org_id=?,a.org_name=?,a.status=?," +
            " a.qr_code_url=?, a.purpose=?,a.qr_code_config=?,a.IS_VISIBLE_ALL=?,a.update_time=? where a.id=?";

    // 删除语句
    public static final String DELETE_SQL ="update YCQ_YOUTHCALL_QRCODE a set a.del_flag=1  where id = ? ";

    // 根据id更改青呼码管理服务状态
    public static final String UPDATE_STATUS_SQL ="update YCQ_YOUTHCALL_QRCODE a set a.status=?  where id = ? ";

    // 根据id更改青呼码管理是否可见全部
    public static final String UPDATE_VISIBLE_ALL_SQL ="update YCQ_YOUTHCALL_QRCODE a set a.is_visible_all=?  where id = ? ";

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;

    public PageRsp<YcqYouthcallQrcode> paginQuery(PageReq<YcqYouthcallQrcodeVo> req) {
        StringBuilder sb = new StringBuilder(PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(PAGE_COUNT_SQL);

        List<Object> params = new ArrayList<>();
        if (req.getQuery()!=null) {
            // 组织机构id
            if (req.getQuery().getOrgId() != null) {
                sb.append(" and a.org_id like concat('%',?,'%') ");
                sbCount.append(" and a.org_id like concat('%',?,'%') ");
                params.add(req.getQuery().getOrgId());
            }
            // 青呼码编码
            if (StringUtils.hasText(req.getQuery().getYouthcallCode())) {
                sb.append(" and a.youthcall_code like concat('%',?,'%') ");
                sbCount.append(" and a.youthcall_code like concat('%',?,'%') ");
                params.add(req.getQuery().getYouthcallCode());
            }
            // 青呼码名称
            if (StringUtils.hasText(req.getQuery().getYouthcallName())) {
                sb.append(" and a.youthcall_name  like concat('%',?,'%') ");
                sbCount.append(" and a.youthcall_name  like concat('%',?,'%') ");
                params.add(req.getQuery().getYouthcallName());
            }

            // 创建时间
            if (StringUtils.hasText(req.getQuery().getCreateTimeBegin())) {
                sb.append(" and a.create_time >= ? ");
                sbCount.append(" and a.create_time  >= ? ");
                params.add(req.getQuery().getCreateTimeBegin());
            }

            if (StringUtils.hasText(req.getQuery().getCreateTimeEnd())) {
                sb.append(" and a.create_time <= ? ");
                sbCount.append(" and a.create_time  <= ? ");
                params.add(req.getQuery().getCreateTimeEnd());
            }

            // 状态
            if (StringUtils.hasText(req.getQuery().getStatus())) {
                sb.append(" and a.status = ? ");
                sbCount.append(" and a.status  = ? ");
                params.add(req.getQuery().getStatus());
            }

        }
        sb.append(" order by a.create_time desc ");
        int pageNo = req.getPageNo();
        int pageSize = req.getPageSize();
        int count = JT.queryForInt(sbCount.toString(), params.toArray());
        List<YcqYouthcallQrcode> list = new ArrayList();
        if (count>0) {
            list = queryForPage(sb.toString(), pageNo, pageSize, params);
        }

        PageRsp<YcqYouthcallQrcode> rsp = new PageRsp<>();
        rsp.setResult(list);
        rsp.setTotalNum(count);
        rsp.setPageNo(pageNo);
        rsp.setPageSize(pageSize);

        return rsp;
    }

    public List queryForPage(String sql, int pageNo, int pageSize, List<Object> params) {
        int offset = (pageNo - 1) * pageSize;
        sql = sql + " limit " + pageSize + " offset " + offset;
        return JT.queryForBeans(YcqYouthcallQrcode.class, sql, params.toArray());
    }

    public YcqYouthcallQrcode queryById(BigInteger id) {
        YcqYouthcallQrcode ycqYouthcallQrcode = JT.queryForBean(YcqYouthcallQrcode.class, QUERY_SQL, new Object[]{id});
        return ycqYouthcallQrcode;
    }

    public YcqYouthcallQrcode queryByYouthcallCode(String  youthcallCode) {
        YcqYouthcallQrcode ycqYouthcallQrcode = JT.queryForBean(YcqYouthcallQrcode.class, QUERY_YOUTHCALLCODE_SQL, new Object[]{youthcallCode});
        return ycqYouthcallQrcode;
    }

    public YcqYouthcallQrcode insert(YcqYouthcallQrcode ycqYouthcallQrcode) {
        ycqYouthcallQrcode.setCreateTime(new Date());
        ycqYouthcallQrcode.setDelFlag(0);
        String insertsql = BaseJdbcTemplateUtil.insert(YcqYouthcallQrcode.class);
        Object[] objs = BaseJdbcTemplateUtil.getValues(ycqYouthcallQrcode);
        PreparedStatementCreator create = new PreparedStatementCreator() {
            //利用传统的方法进行JDBC查询
            @Override
            public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
                //插入用户，返回主键
                PreparedStatement preparedStatement = connection.prepareStatement(insertsql,
                        PreparedStatement.RETURN_GENERATED_KEYS);
                for (int i = 0; i < objs.length; i++) {
                    preparedStatement.setObject(i+1, objs[i]);
                }
                return preparedStatement;
            }
        };
        //2.创建keyHoldle
        GeneratedKeyHolder keyHolder = new GeneratedKeyHolder();
        JT.update(create, keyHolder);
        //获取生成的主键key
        Long key = keyHolder.getKey().longValue();
        BigInteger id = new BigInteger(String.valueOf(key));
        ycqYouthcallQrcode.setId(id);
        return ycqYouthcallQrcode;
    }

    public void update(YcqYouthcallQrcode ycqYouthcallQrcode)  {
        ycqYouthcallQrcode.setUpdateTime(new Date());
        try {
            JT.update(UPDATE_SQL, ycqYouthcallQrcode.getYouthcallName(),ycqYouthcallQrcode.getOrgId(),ycqYouthcallQrcode.getOrgName(),
                    ycqYouthcallQrcode.getStatus(),ycqYouthcallQrcode.getQrCodeUrl(),ycqYouthcallQrcode.getPurpose(),
                    ycqYouthcallQrcode.getQrCodeConfig(),ycqYouthcallQrcode.getIsVisibleAll(),
                    ycqYouthcallQrcode.getUpdateTime(),
                    ycqYouthcallQrcode.getId());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void deleteById(BigInteger id)  {
        try {
            JT.update(DELETE_SQL, new Object[]{id});
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void updateStatus(YcqYouthcallQrcode ycqYouthcallQrcode) {
        try {
            JT.update(UPDATE_STATUS_SQL, ycqYouthcallQrcode.getStatus(), ycqYouthcallQrcode.getId());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void updateVisibleAll(YcqYouthcallQrcode ycqYouthcallQrcode) {
        try {
            JT.update(UPDATE_VISIBLE_ALL_SQL, ycqYouthcallQrcode.getIsVisibleAll(), ycqYouthcallQrcode.getId());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 批量新增
     * */
    public void batchInsert(List<YcqYouthcallQrcode> list) {
        YcqYouthcallQrcode ycqYouthcallQrcode = null;
        List<Object[]> listObj = new ArrayList<>();
        Object[] obj = null;
        for (int i = 0; i < list.size(); i++) {
            ycqYouthcallQrcode = list.get(i);
            ycqYouthcallQrcode.setCreateTime(new Date());
            ycqYouthcallQrcode.setDelFlag(0);
            obj = BaseJdbcTemplateUtil.getValues(ycqYouthcallQrcode);
            listObj.add(obj);
        }
        String insertsql = BaseJdbcTemplateUtil.insert(YcqYouthcallQrcode.class);
        JT.batchUpdate(insertsql, listObj);
    }

    public List<YcqYouthcallQrcode> paginQueryNoPage(YcqYouthcallQrcodeVo ycqYouthcallQrcodeVo){
        StringBuilder sb = new StringBuilder(PAGE_SQL);
        List<Object> params = new ArrayList<>();
        if (ycqYouthcallQrcodeVo!=null) {
            // 组织机构id
            if (ycqYouthcallQrcodeVo.getOrgId() != null) {
                sb.append(" and a.org_id like concat('%',?,'%') ");
                params.add(ycqYouthcallQrcodeVo.getOrgId());
            }
            // 青呼码编码
            if (StringUtils.hasText(ycqYouthcallQrcodeVo.getYouthcallCode())) {
                sb.append(" and a.youthcall_code like  concat('%',?,'%') ");
                params.add(ycqYouthcallQrcodeVo.getYouthcallCode());
            }
            // 青呼码名称
            if (StringUtils.hasText(ycqYouthcallQrcodeVo.getYouthcallName())) {
                sb.append(" and a.youthcall_name  like concat('%',?,'%') ");
                params.add(ycqYouthcallQrcodeVo.getYouthcallName());
            }

            // 创建时间
            if (StringUtils.hasText(ycqYouthcallQrcodeVo.getCreateTimeBegin())) {
                sb.append(" and a.create_time >= ? ");
                params.add(ycqYouthcallQrcodeVo.getCreateTimeBegin());
            }

            if (StringUtils.hasText(ycqYouthcallQrcodeVo.getCreateTimeEnd())) {
                sb.append(" and a.create_time <= ? ");
                params.add(ycqYouthcallQrcodeVo.getCreateTimeEnd());
            }

            // 状态
            if (StringUtils.hasText(ycqYouthcallQrcodeVo.getStatus())) {
                sb.append(" and a.status = ? ");
                params.add(ycqYouthcallQrcodeVo.getStatus());
            }

        }
        sb.append(" order by a.create_time desc ");
        List<YcqYouthcallQrcode> list = JT.queryForBeans(YcqYouthcallQrcode.class, sb.toString(), params.toArray());
        return list;
    }
}
