package com.crunii.micro.service.youthcall.jdbc;

import com.crunii.micro.common.dto.OrderProp;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrder;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrderResp;
import com.crunii.micro.service.youthcall.dto.WorkerRole;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallWorkOrderVo;
import com.crunii.micro.service.youthcall.dto.vo.workOrder.WorkOrderBaseVo;
import com.crunii.micro.service.youthcall.dto.vo.workOrder.WorkOrderExcelVo;
import com.crunii.micro.service.youthcall.dto.vo.workOrder.WorkOrderMyVo;
import com.crunii.micro.service.youthcall.enums.UseRoleEnum;
import com.crunii.micro.service.youthcall.enums.WorkOrderStatusEnum;
import com.crunii.micro.service.youthcall.utils.BaseJdbcTemplateUtil;
import com.crunii.micro.service.youthcall.utils.BeanUtil;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Repository
public class YcqYouthcallWorkOrderDao {
    public static final String WORKER_ORDER_ADMIN = "青呼我为工单管理员";



    // admin所在的组织处理过，就返回找到的showType，否则为1
    public static final String ADMIN_PAGE_SQL = "select a.*," +
            "COALESCE((SELECT wwoi.show_type FROM YCQ_YOUTHCALL_WORK_ORDER_ORG wwoi " +
            "WHERE wwoi.work_order_id = a.id AND wwoi.org_id = ? LIMIT 1 ),1) AS show_type,"+
            "(SELECT CASE WHEN count(1) > 0 THEN '1' ELSE '0' end FROM tsm_org o WHERE o.parent_org_id = a.org_id) as nextorg," +
            "(SELECT CASE WHEN IFNULL(t.parent_org_id, 0) = 0 THEN '0' ELSE '1' end " +
            "FROM tsm_org t " +
            "where t.org_id = a.org_id)  AS porg from YCQ_YOUTHCALL_WORK_ORDER a where 1 = 1 and a.del_flag=0 ";

    public static final String REPORT_PAGE_SQL ="SELECT a.* FROM YCQ_YOUTHCALL_WORK_ORDER a " +
            "LEFT JOIN YCQ_YOUTHCALL_WORK_ORDER_PROCESS b ON a.id = b.work_order_id AND b.process_status = '08' AND b.report_type IN ('2','3') AND b.del_flag = 0 " +
            "where 1 = 1 AND b.id IS NOT NULL and a.del_flag = 0 ";

    public static final String MY_PAGE_SQL ="SELECT a.*,b.created_name as evaluate_name,b.created_time as evaluate_time,b.evaluate_star,b.deal_content as evaluate_content FROM YCQ_YOUTHCALL_WORK_ORDER a " +
            "LEFT JOIN YCQ_YOUTHCALL_WORK_ORDER_PROCESS b ON a.id = b.work_order_id AND b.process_status = '06' AND b.del_flag = 0 " +
            "where 1 = 1 and a.del_flag = 0 ";

    public static final String ADMIN_PAGE_COUNT_SQL ="select count(1) from YCQ_YOUTHCALL_WORK_ORDER a where 1 = 1 and a.del_flag=0 ";

    public static final String REPORT_PAGE_COUNT_SQL ="SELECT count(1) FROM YCQ_YOUTHCALL_WORK_ORDER a " +
            "LEFT JOIN YCQ_YOUTHCALL_WORK_ORDER_PROCESS b ON a.id = b.work_order_id AND b.process_status = '08' AND b.report_type IN ('2','3') AND b.del_flag = 0 " +
            "where 1 = 1 AND b.id IS NOT NULL and a.del_flag = 0 ";



    public static final String GET_WORK_ORDER_ID = "(select b.work_order_id\n" +
            "               from YCQ_YOUTHCALL_WORK_ORDER_ORG b\n" +
            "               where 1 = 1\n" +
            "                 and b.org_id IN ( (SELECT org_id\n" +
            "                                    FROM (SELECT org_id, CONNECT_BY_ISLEAF AS is_leaf\n" +
            "                                          FROM TSM_ORG START WITH org_id = ?\n" +
            "                                          CONNECT BY PRIOR org_id = parent_org_id))))";

    public static final String HAS_AUTH="SELECT c.org_id FROM TSM_WORKER f\n" +
            "INNER JOIN TSM_WORKER_ROLE e ON f.login_id = e.login_id\n" +
            "INNER JOIN TSM_ROLE d ON d.role_id = e.role_id\n" +
            "INNER JOIN TSM_ROLE_ORG c ON c.role_id = d.role_id\n" +
            "WHERE d.state=1 and f.login_id = ? AND d.role_name = ? and c.org_id = ?";


    public static final String MY_PAGE_COUNT_SQL ="SELECT count(1) FROM YCQ_YOUTHCALL_WORK_ORDER a " +
            "LEFT JOIN YCQ_YOUTHCALL_WORK_ORDER_PROCESS b ON a.id = b.work_order_id AND b.process_status = '06' AND b.del_flag = 0 " +
            "where 1 = 1 and a.del_flag = 0 ";

    public static final String QUERY_SQL ="SELECT * FROM YCQ_YOUTHCALL_WORK_ORDER a where id = ? ";

    public static final String QUERYIDS_SQL ="SELECT * FROM YCQ_YOUTHCALL_WORK_ORDER where del_flag=0 and id in ";

    public static final String UPDATE_PROCESSSTATUS_SQL= "update YCQ_YOUTHCALL_WORK_ORDER a set a.process_status=? where a.id=? ";

    // 修改语句
//    public static final String UPDATE_SQL ="update YCQ_YOUTHCALL_WORK_ORDER a " +
//            " set a.youthcall_name=?,a.org_id=?,a.org_name=?,a.status=?," +
//            " a.purpose=? where a.id=?";

    // 删除语句
    public static final String DELETE_SQL ="update YCQ_YOUTHCALL_WORK_ORDER a set a.del_flag=1  where id = ? ";

    // 根据组织id查询当前组织的同级组织
    public static final String QUERY_SAMEORG_SQL = "SELECT COUNT(1) FROM tsm_org t WHERE t.parent_org_id = (SELECT o.parent_org_id FROM tsm_org o WHERE o.org_id  = ?) AND t.org_id !=? ";
    public static final String GET_ROLE_BY_LOGINID = "select *from TSM_WORKER_ROLE where login_id=?;";

    // APP端统计
    public static final String QUERY_PROCESS_SQL="select count(1) total, " +
            "sum(case when t.process_status='01' then 1 else 0 end) as pending_process, " +
            "sum(case when t.process_status='02' then 1 else 0 end) as processed " +
            " from YCQ_YOUTHCALL_WORK_ORDER t where t.proposer_id=?";
    @Resource(name = "mainJT")
    protected BaseJdbcTemplate JT;


    public boolean isSuperAdmin() {
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (currWorker==null){
            return false;
        }
        List<WorkerRole> roles = getRoleByLoginId(currWorker.getLoginId());
        if (roles != null) {
            List<String> collect = roles.stream().map(WorkerRole::getRoleId).map(String::valueOf).collect(Collectors.toList());
            if (collect.contains(UseRoleEnum.SUPER_ADMIN_ROLE.getCode().toString())) {
                return true;
            } else {
                return false;
            }
        }
        return false;
    }

    public boolean hasAuth(int loginId, String roleName, String orgId) {
        String s2 = JT.queryForRawObject(String.class, HAS_AUTH, new Object[]{loginId, roleName, orgId});
        return StringUtils.hasText(s2);
    }

    /**
     * 普通用户
     * @param req
     * @return
     */
    public PageRsp<WorkOrderBaseVo> paginQuery(PageReq<YcqYouthcallWorkOrderVo> req) {

        List<Object> params = new ArrayList<>();
        StringBuilder sb = new StringBuilder(ADMIN_PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(ADMIN_PAGE_COUNT_SQL);
        int index = sb.indexOf("?");
        if (StringUtils.isEmpty(req.getQuery().getOrgId())) {
            sb.replace(index, index + 1, "-1");
        } else {
            sb.replace(index, index + 1, req.getQuery().getOrgId().toString());
        }

        if (req.getQuery() != null) {
            // 工单id
            if (req.getQuery().getId() != null) {
                sb.append(" and a.id = ? ");
                sbCount.append(" and a.id = ? ");
                params.add(req.getQuery().getId());
            }
            // 工单编号
            if (StringUtils.hasText(req.getQuery().getWorkOrderNo())) {
                sb.append(" and a.work_order_no like concat('%',?,'%') ");
                sbCount.append(" and a.work_order_no like concat('%',?,'%') ");
                params.add(req.getQuery().getWorkOrderNo());
            }
            // 工单类型
            if (StringUtils.hasText(req.getQuery().getWorkOrderType())) {
                sb.append(" and a.work_order_type = ? ");
                sbCount.append(" and a.work_order_type = ? ");
                params.add(req.getQuery().getWorkOrderType());
            }
            // 工单类型多选
            if (!CollectionUtils.isEmpty(req.getQuery().getWorkOrderTypes())) {
                // 构建动态的IN子句
                String inClause = req.getQuery().getWorkOrderTypes().stream()
                        .map(String::valueOf) // 将整数转换为字符串
                        .collect(Collectors.joining("', '", "('", "')")); // 生成"(1, 2, 3)"格式的子句
                sb.append(" and a.work_order_type IN  ").append(inClause);
                sbCount.append(" and a.work_order_type IN ").append(inClause);
            }
            // 所属业务
            if (StringUtils.hasText(req.getQuery().getAppCode())) {
                sb.append(" and FIND_IN_SET(?, a.app_code) > 0 ");
                sbCount.append(" and FIND_IN_SET(?, a.app_code) > 0  ");
                params.add(req.getQuery().getAppCode());
            }

            // 组织机构id,查询该机构及以下的数据
            // 如果是普通用户，前端传递的orgId不使用，自己可以获得
            if (req.getQuery().getOrgId() != null) {
                sb.append(" and a.id IN "+GET_WORK_ORDER_ID);
                sbCount.append(" and a.id IN "+GET_WORK_ORDER_ID);
                params.add(req.getQuery().getOrgId());
            }

            // 工单状态
            if (StringUtils.hasText(req.getQuery().getProcessStatus())) {
                sb.append(" and a.process_status = ? ");
                sbCount.append(" and a.process_status  = ? ");
                params.add(req.getQuery().getProcessStatus());
            }

            // 工单来源
            if (StringUtils.hasText(req.getQuery().getSource())) {
                sb.append(" and a.source = ? ");
                sbCount.append(" and a.source  = ? ");
                params.add(req.getQuery().getSource());
            }

            // 提出时间
            if (StringUtils.hasText(req.getQuery().getProposedTimeBegin())) {
                sb.append(" and CAST(a.proposed_time AS DATE) >= ? ");
                sbCount.append(" and CAST(a.proposed_time AS DATE)  >= ? ");
                params.add(req.getQuery().getProposedTimeBegin());
            }

            if (StringUtils.hasText(req.getQuery().getProposedTimeEnd())) {
                sb.append(" and CAST(a.proposed_time AS DATE) <= ? ");
                sbCount.append(" and CAST(a.proposed_time AS DATE)  <= ? ");
                params.add(req.getQuery().getProposedTimeEnd());
            }
            // 处理时间
            if (StringUtils.hasText(req.getQuery().getProcessTimeBegin())) {
                sb.append(" and CAST(a.proposed_time AS DATE) >= ? ");
                sbCount.append(" and CAST(a.proposed_time AS DATE)  >= ? ");
                params.add(req.getQuery().getProcessTimeBegin());
            }

            if (StringUtils.hasText(req.getQuery().getProcessTimeEnd())) {
                sb.append(" and CAST(a.proposed_time AS DATE) <= ? ");
                sbCount.append(" and CAST(a.proposed_time AS DATE)  <= ? ");
                params.add(req.getQuery().getProcessTimeEnd());
            }

            // 根据当前登录人id过滤数据
            if (req.getQuery().getProposerId() != null) {
                sb.append(" and a.proposer_id = ? ");
                sbCount.append(" and a.proposer_id = ? ");
                params.add(req.getQuery().getProposerId());
            }

            // 工单来源id
            if (StringUtils.hasText(req.getQuery().getSourceId())) {
                sb.append(" and a.source_id = ? ");
                sbCount.append(" and a.source_id  = ? ");
                params.add(req.getQuery().getSourceId());
            }
        }

        List<OrderProp> orderProps = req.getOrderProps();
        StringBuilder lastQuerySql;
        StringBuilder lastCountSql;
        Object[] lastObjects;

        lastQuerySql = sb;
        lastCountSql = new StringBuilder(sbCount);
        lastObjects = params.toArray();




        // 默认按时间倒叙排
        if (CollectionUtils.isEmpty(orderProps)) {
            lastQuerySql.append(" order by proposed_time desc NULLS LAST ");
        } else {
            lastQuerySql.append(" order by ");
            for (int i = 0; i < orderProps.size(); i++) {
                OrderProp orderProp = orderProps.get(i);
                boolean ascending = orderProp.isAscending();
                // 根据属性名找到注解
                String prop = orderProp.getProp();
                String tableFiled = BeanUtil.gettAnnotations(YcqYouthcallWorkOrder.class, prop);
                if (i == orderProps.size() - 1) {
                    if (ascending) {
                        lastQuerySql.append(tableFiled + " asc ");
                    } else {
                        lastQuerySql.append(tableFiled + " desc ");
                    }
                } else {
                    if (ascending) {
                        lastQuerySql.append(tableFiled + " asc, ");
                    } else {
                        lastQuerySql.append(tableFiled + " desc, ");
                    }
                }
            }
            lastQuerySql.append(" NULLS LAST ");
        }
        int pageNo = req.getPageNo();
        int pageSize = req.getPageSize();
        int count = JT.queryForInt(lastCountSql.toString(), lastObjects);
        List list = new ArrayList();
        if (count > 0) {
            list = queryForPage(lastQuerySql.toString(), pageNo, pageSize, params);
        }

        PageRsp<WorkOrderBaseVo> rsp = new PageRsp<>();
        rsp.setResult(list);
        rsp.setTotalNum(count);
        rsp.setPageNo(pageNo);
        rsp.setPageSize(pageSize);
        rsp.next();

        return rsp;
    }

    public PageRsp<WorkOrderBaseVo> reportPaginQuery(PageReq<YcqYouthcallWorkOrderVo> req) {

        StringBuilder sb = new StringBuilder(REPORT_PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(REPORT_PAGE_COUNT_SQL);

        List<Object> params = new ArrayList<>();
        if (req.getQuery()!=null) {
            // 工单编号
            if (StringUtils.hasText(req.getQuery().getWorkOrderNo())) {
                sb.append(" and a.work_order_no like concat('%',?,'%') ");
                sbCount.append(" and a.work_order_no like concat('%',?,'%') ");
                params.add(req.getQuery().getWorkOrderNo());
            }
            // 工单类型
            if (StringUtils.hasText(req.getQuery().getWorkOrderType())) {
                sb.append(" and a.work_order_type = ? ");
                sbCount.append(" and a.work_order_type = ? ");
                params.add(req.getQuery().getWorkOrderType());
            }
            // 工单类型多选
            if (!CollectionUtils.isEmpty(req.getQuery().getWorkOrderTypes())) {
                // 构建动态的IN子句
                String inClause = req.getQuery().getWorkOrderTypes().stream()
                        .map(String::valueOf) // 将整数转换为字符串
                        .collect(Collectors.joining("', '", "('", "')")); // 生成"(1, 2, 3)"格式的子句
                sb.append(" and work_order_type IN  ").append(inClause);
                sbCount.append(" and work_order_type IN ").append(inClause);
            }
            // 所属业务
            if (StringUtils.hasText(req.getQuery().getAppCode())) {
                sb.append(" and FIND_IN_SET(?, app_code) > 0 ");
                sbCount.append(" and FIND_IN_SET(?, app_code) > 0  ");
                params.add(req.getQuery().getAppCode());
            }

            // 组织机构id
            if (req.getQuery().getOrgId() != null) {
                sb.append(" AND EXISTS (SELECT 1 FROM (SELECT org_id, parent_org_id FROM TSM_ORG  START WITH org_id = ? CONNECT BY PRIOR(org_id) = parent_org_id) as b WHERE a.org_id=b.org_id) ");
                sbCount.append(" AND EXISTS (SELECT 1 FROM (SELECT org_id, parent_org_id FROM TSM_ORG  START WITH org_id = ? CONNECT BY PRIOR(org_id) = parent_org_id) as b WHERE a.org_id=b.org_id) ");
                params.add(req.getQuery().getOrgId());
            }

            // 工单状态
            if (StringUtils.hasText(req.getQuery().getProcessStatus())) {
                sb.append(" and a.process_status = ? ");
                sbCount.append(" and a.process_status  = ? ");
                params.add(req.getQuery().getProcessStatus());
            }

            // 工单来源
            if (StringUtils.hasText(req.getQuery().getSource())) {
                sb.append(" and a.source = ? ");
                sbCount.append(" and a.source  = ? ");
                params.add(req.getQuery().getSource());
            }

            // 提出时间
            if (StringUtils.hasText(req.getQuery().getProposedTimeBegin())) {
                sb.append(" and CAST(a.proposed_time AS DATE) >= ? ");
                sbCount.append(" and CAST(a.proposed_time AS DATE)  >= ? ");
                params.add(req.getQuery().getProposedTimeBegin());
            }

            if (StringUtils.hasText(req.getQuery().getProposedTimeEnd())) {
                sb.append(" and CAST(a.proposed_time AS DATE) <= ? ");
                sbCount.append(" and CAST(a.proposed_time AS DATE)  <= ? ");
                params.add(req.getQuery().getProposedTimeEnd());
            }
            // 处理时间
            if (StringUtils.hasText(req.getQuery().getProcessTimeBegin())) {
                sb.append(" and CAST(a.proposed_time AS DATE) >= ? ");
                sbCount.append(" and CAST(a.proposed_time AS DATE)  >= ? ");
                params.add(req.getQuery().getProcessTimeBegin());
            }

            if (StringUtils.hasText(req.getQuery().getProcessTimeEnd())) {
                sb.append(" and CAST(a.proposed_time AS DATE) <= ? ");
                sbCount.append(" and CAST(a.proposed_time AS DATE)  <= ? ");
                params.add(req.getQuery().getProcessTimeEnd());
            }

            // 根据当前登录人id过滤数据
            if (req.getQuery().getProposerId() != null) {
                sb.append(" and a.proposer_id = ? ");
                sbCount.append(" and a.proposer_id = ? ");
                params.add(req.getQuery().getProposerId());
            }

            // 工单来源id
            if (StringUtils.hasText(req.getQuery().getSourceId())) {
                sb.append(" and a.source_id = ? ");
                sbCount.append(" and a.source_id  = ? ");
                params.add(req.getQuery().getSourceId());
            }
        }

        List<OrderProp> orderProps = req.getOrderProps();
        // 默认按时间倒叙排
        if (CollectionUtils.isEmpty(orderProps)) {
            sb.append(" order by a.proposed_time desc NULLS LAST  ");
        } else {
            sb.append(" order by ");
            for (int i = 0; i < orderProps.size(); i++) {
                OrderProp orderProp = orderProps.get(i);
                boolean ascending = orderProp.isAscending();
                // 根据属性名找到注解
                String prop = orderProp.getProp();
                String tableFiled = BeanUtil.gettAnnotations(YcqYouthcallWorkOrder.class, prop);
                if (i== orderProps.size()-1) {
                    if (ascending) {
                        sb.append( tableFiled + " asc " );
                    } else {
                        sb.append( tableFiled + " desc " );
                    }
                } else {
                    if (ascending) {
                        sb.append( tableFiled + " asc, " );
                    } else {
                        sb.append( tableFiled + " desc, " );
                    }
                }
            }
            sb.append(" NULLS LAST ");
        }

        int pageNo = req.getPageNo();
        int pageSize = req.getPageSize();
        int count = JT.queryForInt(sbCount.toString(), params.toArray());
        List list = new ArrayList();
        if (count>0) {
            list = queryForPage(sb.toString(), pageNo, pageSize, params);
        }

        PageRsp<WorkOrderBaseVo> rsp = new PageRsp<>();
        rsp.setResult(list);
        rsp.setTotalNum(count);
        rsp.setPageNo(pageNo);
        rsp.setPageSize(pageSize);
        rsp.next();

        return rsp;
    }

    public PageRsp<WorkOrderMyVo> myPaginQuery(PageReq<YcqYouthcallWorkOrderVo> req) {

        StringBuilder sb = new StringBuilder(MY_PAGE_SQL);
        StringBuilder sbCount = new StringBuilder(MY_PAGE_COUNT_SQL);

        List<Object> params = new ArrayList<>();
        if (req.getQuery()!=null) {
            // 工单状态
            if (StringUtils.hasText(req.getQuery().getProcessStatus())) {
                String processStatus = req.getQuery().getProcessStatus();
                // 待处理
                if (WorkOrderStatusEnum.PENDING.getValue().equals(processStatus)) {
                    sb.append(" and a.process_status IN ('01','08','09') ");
                    sbCount.append(" and a.process_status IN ('01','08','09')  ");
                } else if (WorkOrderStatusEnum.UN_EVALUATE.getValue().equals(processStatus)) {
                    // 待评价，查询已处理、回访、归档，且未评价过的数据
                    sb.append(" and a.process_status IN ('02','03','04') and b.id IS NULL ");
                    sbCount.append(" and a.process_status IN ('02','03','04') and b.id IS NULL ");
                } else if (WorkOrderStatusEnum.EVALUATE.getValue().equals(processStatus)) {
                    // 已评价，查询已处理、回访、归档，且已评价过的数据
                    sb.append(" and a.process_status IN ('02','03','04') and b.id IS NOT NULL ");
                    sbCount.append(" and a.process_status IN ('02','03','04') and b.id IS NOT NULL ");
                } else {
                    sb.append(" and a.process_status = ? ");
                    sbCount.append(" and a.process_status = ? ");
                    params.add(processStatus);
                }
            }

            // 工单标题
            if (StringUtils.hasText(req.getQuery().getWorkOrderName())) {
                sb.append(" and a.work_order_name like concat('%',?,'%') ");
                sbCount.append(" and a.work_order_name like concat('%',?,'%') ");
                params.add(req.getQuery().getWorkOrderName());
            }

            // 根据当前登录人id过滤数据
            if (req.getQuery().getProposerId() != null) {
                sb.append(" and a.proposer_id = ? ");
                sbCount.append(" and a.proposer_id = ? ");
                params.add(req.getQuery().getProposerId());
            }
        }

        List<OrderProp> orderProps = req.getOrderProps();
        // 默认按时间倒叙排
        if (CollectionUtils.isEmpty(orderProps)) {
            sb.append(" order by a.proposed_time desc NULLS LAST ");
        } else {
            sb.append(" order by ");
            for (int i = 0; i < orderProps.size(); i++) {
                OrderProp orderProp = orderProps.get(i);
                boolean ascending = orderProp.isAscending();
                // 根据属性名找到注解
                String prop = orderProp.getProp();
                String tableFiled = BeanUtil.gettAnnotations(YcqYouthcallWorkOrder.class, prop);
                if (i== orderProps.size()-1) {
                    if (ascending) {
                        sb.append( tableFiled + " asc " );
                    } else {
                        sb.append( tableFiled + " desc " );
                    }
                } else {
                    if (ascending) {
                        sb.append( tableFiled + " asc, " );
                    } else {
                        sb.append( tableFiled + " desc, " );
                    }
                }
            }
            sb.append(" NULLS LAST ");
        }

        int pageNo = req.getPageNo();
        int pageSize = req.getPageSize();
        int count = JT.queryForInt(sbCount.toString(), params.toArray());
        List list = new ArrayList();
        if (count>0) {
            list = queryMyForPage(sb.toString(), pageNo, pageSize, params);
        }

        PageRsp<WorkOrderMyVo> rsp = new PageRsp<>();
        rsp.setResult(list);
        rsp.setTotalNum(count);
        rsp.setPageNo(pageNo);
        rsp.setPageSize(pageSize);
        rsp.next();

        return rsp;
    }

    public List queryForPage(String sql, int pageNo, int pageSize, List<Object> params) {
        int offset = (pageNo - 1) * pageSize;
        sql = sql + " limit " + pageSize + " offset " + offset;
        return JT.queryForBeans(WorkOrderBaseVo.class, sql, params.toArray());
    }

    public List queryMyForPage(String sql, int pageNo, int pageSize, List<Object> params) {
        int offset = (pageNo - 1) * pageSize;
        sql = sql + " limit " + pageSize + " offset " + offset;
        return JT.queryForBeans(WorkOrderMyVo.class, sql, params.toArray());
    }

    public YcqYouthcallWorkOrder insert(YcqYouthcallWorkOrder ycqYouthcallWorkOrder) {
        ycqYouthcallWorkOrder.setProposedTime(new Date());
        ycqYouthcallWorkOrder.setDelFlag(0);
        String insertsql = BaseJdbcTemplateUtil.insert(YcqYouthcallWorkOrder.class);
        Object[] objs = BaseJdbcTemplateUtil.getValues(ycqYouthcallWorkOrder);
        PreparedStatementCreator create = new PreparedStatementCreator() {
            //利用传统的方法进行JDBC查询
            @Override
            public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
                //插入用户，返回主键
                PreparedStatement preparedStatement = connection.prepareStatement(insertsql,
                        PreparedStatement.RETURN_GENERATED_KEYS);
                for (int i = 0; i < objs.length; i++) {
                    preparedStatement.setObject(i+1, objs[i]);
                }
                return preparedStatement;
            }
        };
        //2.创建keyHoldle
        GeneratedKeyHolder keyHolder = new GeneratedKeyHolder();
        JT.update(create, keyHolder);
        //获取生成的主键key
        Long key = keyHolder.getKey().longValue();
        BigInteger id = new BigInteger(String.valueOf(key));
        ycqYouthcallWorkOrder.setId(id);
        return ycqYouthcallWorkOrder;
    }

    public YcqYouthcallWorkOrder queryById(BigInteger id) {
        YcqYouthcallWorkOrder ycqYouthcallWorkOrder = JT.queryForBean(YcqYouthcallWorkOrder.class, QUERY_SQL, new Object[]{id});
        return ycqYouthcallWorkOrder;
    }

    public void deleteById(BigInteger id) {
        JT.update(DELETE_SQL, new Object[]{id});
    }

    public void updateProcessStatus(YcqYouthcallWorkOrder ycqYouthcallWorkOrder) {
        JT.update(UPDATE_PROCESSSTATUS_SQL, new Object[]{ycqYouthcallWorkOrder.getProcessStatus(),ycqYouthcallWorkOrder.getId()});
    }

    public YcqYouthcallWorkOrderResp queryWorkOrderCount(BigInteger proposerId){
        YcqYouthcallWorkOrderResp ycqYouthcallWorkOrderResp = JT.queryForBean(YcqYouthcallWorkOrderResp.class, QUERY_PROCESS_SQL, new Object[]{proposerId});
        return ycqYouthcallWorkOrderResp;

    }

    public List<WorkOrderExcelVo> queryByIds(List<BigInteger> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptyList();
        } else {
            String inSql = ids.stream().map(id -> id.toString()).collect(Collectors.joining(",", "(", ")"));
            String sql = QUERYIDS_SQL + inSql +" ORDER BY CASE id " + generateOrderClause(ids) + "END";
            return JT.queryForBeans(WorkOrderExcelVo.class, sql);
        }
    }

    private String generateOrderClause(List<BigInteger> ids) {
        StringBuilder orderClause = new StringBuilder();
        for (int i = 0; i < ids.size(); i++) {
            orderClause.append(String.format("WHEN %d THEN %d ", ids.get(i), i + 1));
        }
        return orderClause.toString();
    }

    public int querySameSql(BigInteger orgId) {
        int i = JT.queryForInt(QUERY_SAMEORG_SQL,  new Object[]{orgId,orgId});
        return i;
    }

    public List<WorkerRole> getRoleByLoginId(int loginId ) {
        List<WorkerRole> hellos = JT.queryForBeans(WorkerRole.class, GET_ROLE_BY_LOGINID, new Object[]{loginId});
        return hellos;
    }
}
