package com.crunii.micro.service.youthcall.jdbc;

import com.crunii.micro.common.dto.OrderProp;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.utils.SqlUtil;
import com.crunii.micro.domain.common.jdbc.BaseJdbcTemplate;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallPersonalDemand;
import com.crunii.micro.service.youthcall.dto.vo.orderDispatch.WorkOrderDispatchVo;
import com.crunii.micro.service.youthcall.dto.vo.orderDispatch.YcqYouthcallWorkOrderDispatchReq;
import com.crunii.micro.service.youthcall.utils.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Repository
public class YcqYouthcallWorkOrderDispatchDao {

    @Resource(name = "mainJT")
    protected BaseJdbcTemplate jt;

    public static final String PAGE_SQL ="SELECT a.*,b.org_list,b.org_ids,b.worker_name_list,b.org_category_list,b.mobile_list,b.org_ids FROM YCQ_YOUTHCALL_WORK_ORDER_DISPATCH a " +
            "LEFT JOIN (SELECT dispatch_id, " +
            "LISTAGG(IFNULL (org_name,'-'), ',') WITHIN GROUP (ORDER BY created_time) AS org_list, " +
            "LISTAGG(org_id, ',') WITHIN GROUP (ORDER BY created_time) AS org_ids, " +
            "LISTAGG(IFNULL (worker_name,'-'), ',') WITHIN GROUP (ORDER BY created_time) AS worker_name_list, " +
            "LISTAGG(IFNULL (ocyl_org_category,'-'), ',') WITHIN GROUP (ORDER BY created_time) AS org_category_list, " +
            "LISTAGG(IFNULL (mobile,'-'), ',') WITHIN GROUP (ORDER BY created_time) AS mobile_list " +
            "FROM YCQ_YOUTHCALL_WORK_ORDER_DISPATCH_ORG WHERE del_flag = 0 GROUP BY dispatch_id) b " +
            "ON a.id = b.dispatch_id WHERE a.del_flag = 0 ";

    public static final String PAGE_COUNT_SQL ="SELECT count(1) FROM YCQ_YOUTHCALL_WORK_ORDER_DISPATCH a " +
            "LEFT JOIN (SELECT dispatch_id, " +
            "LISTAGG(IFNULL (org_name,'-'), ',') WITHIN GROUP (ORDER BY created_time) AS org_list, " +
            "LISTAGG(org_id, ',') WITHIN GROUP (ORDER BY created_time) AS org_ids " +
            "FROM YCQ_YOUTHCALL_WORK_ORDER_DISPATCH_ORG WHERE del_flag = 0 GROUP BY dispatch_id) b " +
            "ON a.id = b.dispatch_id WHERE a.del_flag = 0 ";

    public PageRsp<WorkOrderDispatchVo> pageSearch(PageReq<YcqYouthcallWorkOrderDispatchReq> req){
        log.info("YcqYouthcallWorkOrderDispatchDao.pageSearch.param：{}", req.toString());

        StringBuilder sql = new StringBuilder(PAGE_SQL);
        StringBuilder countSql = new StringBuilder(PAGE_COUNT_SQL);
        List<Object> param = new ArrayList<>();
        // 工单类型
        if (StringUtils.isNotBlank(req.getQuery().getWorkOrderType())) {
            sql.append(" and a.work_order_type = ? ");
            countSql.append(" and a.work_order_type = ? ");
            param.add(req.getQuery().getWorkOrderType());
        }
        // 责任组织
        if (StringUtils.isNotBlank(req.getQuery().getOrgId())) {
            sql.append(" and FIND_IN_SET(?, b.org_ids) > 0 ");
            countSql.append(" and FIND_IN_SET(?, b.org_ids) > 0  ");
            param.add(req.getQuery().getOrgId());
        }
        // 展示状态 0:展示 1:不展示
        if (req.getQuery().getDispatchStatus() != null ) {
            sql.append(" and a.dispatch_status = ? ");
            countSql.append(" and a.dispatch_status = ? ");
            param.add(req.getQuery().getDispatchStatus());
        }
        // 创建时间 开始时间
        if (req.getQuery().getStartCreatedTime() != null) {
            sql.append(" and CAST(a.created_time AS DATE) >=  ? ");
            countSql.append(" and CAST(a.created_time AS DATE) >= ? ");
            param.add(req.getQuery().getStartCreatedTime());
        }
        // 创建时间 结束时间
        if (req.getQuery().getEndCreatedTime() != null) {
            sql.append(" and CAST(a.created_time AS DATE) <= ?  ");
            countSql.append(" and CAST(a.created_time AS DATE) <= ?  ");
            param.add(req.getQuery().getEndCreatedTime());
        }

        List<OrderProp> orderProps = req.getOrderProps();
        // 默认按时间倒叙排
        if (CollectionUtils.isEmpty(orderProps)) {
            sql.append(" order by a.dispatch_status desc, a.created_time desc NULLS LAST  ");
        } else {
            sql.append(" order by a.dispatch_status desc, ");
            for (int i = 0; i < orderProps.size(); i++) {
                OrderProp orderProp = orderProps.get(i);
                boolean ascending = orderProp.isAscending();
                // 根据属性名找到注解
                String prop = orderProp.getProp();
                String tableFiled = BeanUtil.gettAnnotations(YcqYouthcallPersonalDemand.class, prop);
                if (i== orderProps.size()-1) {
                    if (ascending) {
                        sql.append( tableFiled + " asc " );
                    } else {
                        sql.append( tableFiled + " desc " );
                    }
                } else {
                    if (ascending) {
                        sql.append( tableFiled + " asc, " );
                    } else {
                        sql.append( tableFiled + " desc, " );
                    }
                }
            }
            sql.append(" NULLS LAST ");
        }
        int total = jt.queryForInt(countSql.toString(), param.toArray());
        String dbType = jt.getDbType();
        List<WorkOrderDispatchVo> result;
        // 是否存在数据
        if (total > 0) {
            SqlUtil.appendLimit(dbType, param, sql, req);
            result = jt.queryForBeans(WorkOrderDispatchVo.class, sql.toString(), param.toArray());
        } else {
            result = new ArrayList<>();
        }
        PageRsp<WorkOrderDispatchVo> rsp = new PageRsp<>();
        rsp.setResult(result);
        rsp.setTotalNum(total);
        rsp.setPageNo(req.getPageNo());
        rsp.setPageSize(req.getPageSize());
        return rsp;
    }
}
