package com.crunii.micro.service.youthcall.notauthcontroller;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.service.youthcall.dto.YcqPublishActivity;
import com.crunii.micro.service.youthcall.dto.YcqPublishActivityResp;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallQrcode;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallQrcodeActivityRel;
import com.crunii.micro.service.youthcall.dto.vo.YcqPublishActivityVo;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallQrcodeVo;
import com.crunii.micro.service.youthcall.service.YcqYouthcallQrcodeService;
import com.crunii.micro.service.youthcall.utils.CharacterCodeUtil;
import com.crunii.micro.service.youthcall.utils.ExcelUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.math.BigInteger;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author : dinglian
 * @date : 2023-9-18
 */
@Slf4j
@Api(tags = "青呼码管理")
@RestController
@RequestMapping("/ycqYouthcallQrcodeMgrNauth")
public class YcqYouthcallQrcodeNauthController {

    @Resource
    private YcqYouthcallQrcodeService ycqYouthcallQrcodeServiceImpl;

    @ApiOperation(value = "根据青呼码编号查询青呼码管理")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "id", value = "唯一主键", required = true, dataType = "BigInteger")
    )
    @RequestMapping(path = "/queryByYouthcallCode", method = RequestMethod.GET)
    @ResponseBody
    public     Result<YcqYouthcallQrcode> queryByYouthcallCode(@RequestParam("youthcallCode")String youthcallCode) {
        try {
            return ycqYouthcallQrcodeServiceImpl.queryByYouthcallCode(youthcallCode);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.queryByYouthcallCode.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.queryByYouthcallCode.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

}
