package com.crunii.micro.service.youthcall.notauthcontroller;

import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.feign.client.OrgService;
import com.crunii.micro.service.youthcall.dto.vo.workOrder.YcqYouthcallWorkOrderInsertReq;
import com.crunii.micro.service.youthcall.service.YcqYouthcallWorkOrderService;
import com.crunii.micro.service.youthcall.utils.CharacterCodeUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * @author : dinglian
 * @date : 2023-9-18
 */
@Slf4j
@Api(tags = "工单管理(无鉴权)")
@RestController
@RequestMapping("/ycqYouthcallWorkOrderNauthController")
public class YcqYouthcallWorkOrderNauthController {

    @Resource
    private YcqYouthcallWorkOrderService ycqYouthcallWorkOrderServiceImpl;

    @Resource
    OrgService orgService;

    @ApiOperation(value = "新增诉求反馈")
    @RequestMapping(path = "createAppeal", method = RequestMethod.POST)
    @ResponseBody
    public Result<Object> createAppeal(@Valid @RequestBody YcqYouthcallWorkOrderInsertReq ycqYouthcallWorkOrderInsertReq) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.insertAppeal(ycqYouthcallWorkOrderInsertReq);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderController.createAppeal.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderController.createAppeal.error: {}", e.getMessage());
            e.printStackTrace();
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "生成工单编号")
    @RequestMapping(path = "generateNumber", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> generateNumber(String code) {
        try {
            String generateCode = CharacterCodeUtil.generateCharacterCode(code);
            return Result.ofSuccess(generateCode);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.generateNumber.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.generateNumber.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "发送验证码")
    @RequestMapping(path = "sendValidateCode", method = RequestMethod.GET)
    @ResponseBody
    public Result<Object> sendValidateCode(String phone) {
        try {
            return ycqYouthcallWorkOrderServiceImpl.sendValidateCode(phone);
        } catch (BusinessException e) {
            log.error("YcqYouthcallQrcodeController.sendValidateCode.error:{}", e.getMessage());
            return Result.ofFail(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallQrcodeController.sendValidateCode.error: {}", e.getMessage());
            return Result.ofFail("系统内部异常，请联系管理员！");
        }
    }

    @ApiOperation(value = "根据区域id获取组织")
    @RequestMapping(path = "/getOrgByZoneId", method = RequestMethod.GET)
    @ResponseBody
    public Result<List<TsmOrg>> getOrgByZoneId(@RequestParam("zoneId") Integer zoneId) {
        return orgService.getOrgByZoneId(zoneId);
    }

    @ApiOperation(value = "根据组织名获取组织(精确查找)")
    @RequestMapping(path = "/getOrgByOrgName", method = RequestMethod.GET)
    @ResponseBody
    public Result<TsmOrg> getOrgByOrgName(@RequestParam("orgName") String orgName) {
        return orgService.getOrgByOrgName(orgName);
    }

    @GetMapping("/testSetKey")
    @ResponseBody
    public String testSetKey(){
        ycqYouthcallWorkOrderServiceImpl.testSetKey("18996106168");
        return "hello";
    }

    @GetMapping("/testGetKey")
    @ResponseBody
    public String testGetKey(){
        String s = ycqYouthcallWorkOrderServiceImpl.testGetKey("18996106168");
        log.info("ssss获取验证码： "+ s);
        return "hello1";
    }
}
