package com.crunii.micro.service.youthcall.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallCheckInApply;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallCheckInApplyQueryCondition;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigInteger;
import java.util.List;
import java.util.Map;

/**
 * @author PANJL
 * @date 2023-12-16 13:56
 */
public interface YcqYouthcallCheckInApplyService {

    /**
     * 入住申请
     *
     * @param ycqYouthcallCheckInApply 实例对象
     */
    void saveApply(YcqYouthcallCheckInApply ycqYouthcallCheckInApply, Map<String, List<MultipartFile>> fileMap);

    /**
     * 取消申请
     *
     * @param applyId 申请id
     */
    void cancelApply(BigInteger applyId);

    /**
     * 更新数据
     *
     * @param ycqYouthcallCheckInApply 实例对象
     */
    void update(YcqYouthcallCheckInApply ycqYouthcallCheckInApply);

    /**
     * 更新申请状态
     *
     * @param applyId      申请id
     * @param auditStatus  审核状态
     * @param auditComment 审核意见
     */
    void updateAuditStatus(BigInteger applyId, String auditStatus, String auditComment);

    /**
     * 通过申请ID删除单条数据
     *
     * @param applyId 申请id
     */
    void deleteByApplyId(BigInteger applyId);

    /**
     * 通过申请ID查询单条数据
     *
     * @param applyId 申请id
     * @return 实例对象
     */
    YcqYouthcallCheckInApply queryByApplyId(BigInteger applyId);

    /**
     * 分页查询
     *
     * @param req 筛选条件
     * @return 查询结果
     */
    PageRsp<YcqYouthcallCheckInApply> pageQuery(PageReq<YcqYouthcallCheckInApplyQueryCondition> req);

    /**
     * 获取所有申请记录
     */
    List<YcqYouthcallCheckInApply> getAll();

    /**
     * 是否允许申请
     *
     * @param applyUserId 申请用户id
     */
    boolean allowApply(String applyUserId);
}
