package com.crunii.micro.service.youthcall.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.youthcall.dto.*;
import com.crunii.micro.service.youthcall.dto.vo.YcqPublishActivityVo;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallQrcodeVo;

import java.math.BigInteger;
import java.util.List;

public interface YcqYouthcallQrcodeService {
    /**
     * 分页查询
     *
     * @param req 筛选条件
     * @return 查询结果
     */
    Result<PageRsp<YcqYouthcallQrcode>> paginQuery(PageReq<YcqYouthcallQrcodeVo> req);

    /**
     * 新增数据
     *
     * @param ycqYouthcallQrcode 实例对象
     * @return 实例对象
     */
    Result<Object> insert(YcqYouthcallQrcode ycqYouthcallQrcode);

    /**
     * 新增数据
     *
     * @param ycqYouthcallQrcode 实例对象
     * @return 实例对象
     */
    Result<Object> update(YcqYouthcallQrcode ycqYouthcallQrcode);

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    Result<YcqYouthcallQrcode> queryById(BigInteger id);

    /**
     * 通过青呼码查询单条数据
     *
     * @param youthcallCode 主键
     * @return 实例对象
     */
    Result<YcqYouthcallQrcode> queryByYouthcallCode(String youthcallCode);

    /**
     * 通过ID删除单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    Result<Object> deleteById(BigInteger id);

    /**
     * 根据id更改青呼码管理服务状态
     *
     * @param ycqYouthcallQrcode 实例对象
     * @return 实例对象
     */
    Result<Object> updateStatus(YcqYouthcallQrcode ycqYouthcallQrcode);

    /**
     * 根据id更改青呼码管理是否可见全部
     *
     * @param ycqYouthcallQrcode 实例对象
     * @return 实例对象
     */
    Result<Object> updateVisibleAll(YcqYouthcallQrcode ycqYouthcallQrcode);

    /**
     * 批量新增数据
     *
     * @param ycqYouthcallQrcodes 实例对象
     * @return 实例对象
     */
    Result<Object> createBath(List<YcqYouthcallQrcode> ycqYouthcallQrcodes);

    /**
     * 查询青呼码关联的活动
     *
     * @param ycqPublishActivityVo 实例对象
     * @return 实例对象
     */
    Result<List<YcqPublishActivity>> queryQrcodeActivity(YcqPublishActivityVo ycqPublishActivityVo);

    /**
     * 查询青呼码关联的活动带分页
     *
     * @param req 实例对象
     * @return 实例对象
     */
    Result<PageRsp<YcqPublishActivityResp>> queryQrcodeActivityByPage(PageReq<YcqPublishActivityVo> req);

    /**
     * 青呼码关联活动
     *
     * @param ycqYouthcallQrcodeActivityRels 实例对象
     * @return 实例对象
     */
    Result<Object> bathSaveQrcodeActivityRel(List<YcqYouthcallQrcodeActivityRel> ycqYouthcallQrcodeActivityRels);

    /**
     * 分页查询
     *
     * @param req 筛选条件
     * @return 查询结果
     */
    Result<List<YcqYouthcallQrcode>> export(YcqYouthcallQrcodeVo req);

    /**
     * 修改青呼码活动入口
     *
     * @param ycqYouthcallQrcodeActivityRel 实例对象
     * @return 实例对象
     */
    Result<Object> updateQrCodeInlet(YcqYouthcallQrcodeActivityRel ycqYouthcallQrcodeActivityRel);
}
