package com.crunii.micro.service.youthcall.service;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrder;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallWorkOrderVo;
import com.crunii.micro.service.youthcall.dto.vo.workOrder.*;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;

public interface YcqYouthcallWorkOrderService {
    /**
     * 分页查询
     *
     * @param req 筛选条件
     * @return 查询结果
     */
    Result<PageRsp<WorkOrderBaseVo>> paginQuery(PageReq<YcqYouthcallWorkOrderVo> req);

    /**
     * 上报工单分页查询(上报工单管理列表展示上报到综治系统、民呼我为的工单)
     *
     * @param req 筛选条件
     * @return 查询结果
     */
    Result<PageRsp<WorkOrderBaseVo>> reportPaginQuery(PageReq<YcqYouthcallWorkOrderVo> req);

    /**
     * 我的工单分页查询
     *
     * @param req 筛选条件
     * @return 查询结果
     */
    Result<PageRsp<WorkOrderMyVo>> myPaginQuery(PageReq<YcqYouthcallWorkOrderVo> req);

    /**
     * 新增数据
     *
     * @param ycqYouthcallWorkOrder 实例对象
     * @return 实例对象
     */
    Result<Object> insert(YcqYouthcallWorkOrder ycqYouthcallWorkOrder);

    /**
     * 修改数据
     *
     * @param ycqYouthcallWorkOrder 实例对象
     * @return 实例对象
     */
    Result<Object> update(YcqYouthcallWorkOrder ycqYouthcallWorkOrder);


    /**
     * 根据登录人id查询诉求总数、待处理诉求数、已处理诉求数
     * */
    Result<Object> queryWorkOrderCount(BigInteger proposerId);

    /**
     * 修改数据
     *
     * @param ycqYouthcallWorkOrder 实例对象
     * @return 实例对象
     */
    Result<Object> updateProcessStatus(YcqYouthcallWorkOrder ycqYouthcallWorkOrder);

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    Result<YcqYouthcallWorkOrder> queryById(BigInteger id);

    /**
     * 通过ID删除单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    Result<Object> deleteById(BigInteger id);

    /**
     * app端提交诉求反馈
     *
     * @param ycqYouthcallWorkOrderInsertReq 对象
     * @return 实例对象
     */
    Result<Object> insertAppeal(YcqYouthcallWorkOrderInsertReq ycqYouthcallWorkOrderInsertReq);

    /**
     * 处理工单
     *
     * @param ycqYouthcallWorkOrderDealReq 对象
     * @return 实例对象
     */
    Result<Object> dealWorkOrder(YcqYouthcallWorkOrderDealReq ycqYouthcallWorkOrderDealReq);

    /**
     * 上报工单
     *
     * @param ycqYouthcallWorkOrderEscalationReq 对象
     * @return 实例对象
     */
    Result<Object> escalationWorkOrder(YcqYouthcallWorkOrderEscalationReq ycqYouthcallWorkOrderEscalationReq);

    /**
     * 撤销上报工单
     *
     * @param ycqYouthcallWorkOrderEscalationReq 对象
     * @return 实例对象
     */
    Result<Object> revokeEscalationWorkOrder(YcqYouthcallWorkOrderEscalationReq ycqYouthcallWorkOrderEscalationReq);

    /**
     * 下派工单
     *
     * @param ycqYouthcallWorkOrderAllocationReq 对象
     * @return 实例对象
     */
    Result<Object> allocationWorkOrder(YcqYouthcallWorkOrderAllocationReq ycqYouthcallWorkOrderAllocationReq);

    /**
     * 撤销下派工单
     *
     * @param ycqYouthcallWorkOrderAllocationReq 对象
     * @return 实例对象
     */
    Result<Object> revokeAllocationWorkOrder(YcqYouthcallWorkOrderAllocationReq ycqYouthcallWorkOrderAllocationReq);

    /**
     * 统计转派
     *
     * @param ycqYouthcallWorkOrderPeerTransferReq 对象
     * @return 实例对象
     */

    Result<Object> peerTransferWorkOrder(YcqYouthcallWorkOrderPeerTransferReq ycqYouthcallWorkOrderPeerTransferReq);

    /**
     * 撤销转派
     *
     * @param ycqYouthcallWorkOrderPeerTransferReq 对象
     * @return 实例对象
     */

    Result<Object> revokePeerTransferWorkOrder(YcqYouthcallWorkOrderPeerTransferReq ycqYouthcallWorkOrderPeerTransferReq);

    /**
     * 工单回访
     *
     * @param ycqYouthcallWorkOrderFollowUpReq 对象
     * @return 实例对象
     */
    Result<Object> followUpWorkOrder(YcqYouthcallWorkOrderFollowUpReq ycqYouthcallWorkOrderFollowUpReq);

    /**
     * 工单归档
     *
     * @param ycqYouthcallWorkOrderFileReq 对象
     * @return 实例对象
     */
    Result<Object> fileWorkOrder(YcqYouthcallWorkOrderFileReq ycqYouthcallWorkOrderFileReq);

    /**
     * 工单评价
     *
     * @param ycqYouthcallWorkOrderEvaluateReq 对象
     * @return 实例对象
     */
    Result<Object> evaluateWorkOrder(YcqYouthcallWorkOrderEvaluateReq ycqYouthcallWorkOrderEvaluateReq);

    /**
     * 工单回复
     *
     * @param ycqYouthcallWorkOrderReplyReq 对象
     * @return 实例对象
     */
    Result<Object> replyWorkOrder(YcqYouthcallWorkOrderReplyReq ycqYouthcallWorkOrderReplyReq);

    /**
     * 根据工单id查询详情
     *
     * @param id 工单id
     * @return 实例对象
     */
    Result<Object> queryDetailById(BigInteger id);

    /**
     * 根据工单id导出
     */
    void exportFileWorkOrderExcel(HttpServletResponse response, YcqYouthcallWorkOrderExportReq req);

    /**
     * 发送验证码
     */
    Result<Object> sendValidateCode(String phone);

    /**
     * 根据工单id查询详情(APP专用)
     *
     * @param id 工单id
     * @return 实例对象
     */
    Result<Object> queryDetailAppById(BigInteger id);

    String testSetKey(String phone);

    String testGetKey(String phone);

}
