package com.crunii.micro.service.youthcall.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallCampusEducation;
import com.crunii.micro.service.youthcall.dto.vo.CarouselChart;
import com.crunii.micro.service.youthcall.dto.vo.ExcellentStudent;
import com.crunii.micro.service.youthcall.dto.vo.RecommendByTeachers;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallCampusEducationVo;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallCampusEducationDao;
import com.crunii.micro.service.youthcall.service.YcqYouthcallCampusEducationService;
import com.crunii.micro.service.youthcall.utils.JsonSerilizable;
import com.crunii.micro.service.youthcall.utils.Sm2Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicHeader;
import org.apache.http.util.EntityUtils;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.nio.charset.Charset;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

/**
 * 青呼我为-校外教育;(YCQ_YOUTHCALL_CAMPUS_EDUCATION)表服务实现类
 * @author : http://www.chiner.pro
 * @date : 2023-9-20
 */
@Slf4j
@Service
public class YcqYouthcallCampusEducationServiceImpl implements YcqYouthcallCampusEducationService {

    @Resource
    private YcqYouthcallCampusEducationDao ycqYouthcallCampusEducationDao;

    @Value("${qqcq.publicKey}")
    private String publicKey;

    @Value("${qqcq.privateKey}")
    private String privateKey;

    @Override
    public Result<YcqYouthcallCampusEducationVo> paginQuery() {
        YcqYouthcallCampusEducation ycqYouthcallCampusEducation = ycqYouthcallCampusEducationDao.query();
        YcqYouthcallCampusEducationVo ycqYouthcallCampusEducationVo = new YcqYouthcallCampusEducationVo();
        ycqYouthcallCampusEducationVo.setInterestCoursesUrl(ycqYouthcallCampusEducation.getInterestCoursesUrl());
        ycqYouthcallCampusEducationVo.setPracticalActivity(ycqYouthcallCampusEducation.getPracticalActivity());
        ycqYouthcallCampusEducationVo.setCarouselCharts(byteArrayToList(ycqYouthcallCampusEducation.getCarouselChart(), CarouselChart.class));
        ycqYouthcallCampusEducationVo.setExcellentStudents(byteArrayToList(ycqYouthcallCampusEducation.getExcellentStudent(), ExcellentStudent.class));
        ycqYouthcallCampusEducationVo.setRecommendByTeachers(byteArrayToList(ycqYouthcallCampusEducation.getRecommendByTeachers(), RecommendByTeachers.class));
        return Result.ofSuccess(ycqYouthcallCampusEducationVo);
    }

    @Override
    public Result<Object> insert(YcqYouthcallCampusEducationVo ycqYouthcallCampusEducationVo) {
        // 轮播图集合
        List<CarouselChart> carouselCharts = ycqYouthcallCampusEducationVo.getCarouselCharts();
        // 名师推荐
        List<RecommendByTeachers> recommendByTeachers = ycqYouthcallCampusEducationVo.getRecommendByTeachers();
        // 优秀学员
        List<ExcellentStudent> excellentStudents = ycqYouthcallCampusEducationVo.getExcellentStudents();
        YcqYouthcallCampusEducation ycqYouthcallCampusEducation = new YcqYouthcallCampusEducation();
        ycqYouthcallCampusEducation.setCarouselChart(tobyteAry(carouselCharts));
        ycqYouthcallCampusEducation.setRecommendByTeachers(tobyteAry(recommendByTeachers));
        ycqYouthcallCampusEducation.setExcellentStudent(tobyteAry(excellentStudents));
        ycqYouthcallCampusEducation = ycqYouthcallCampusEducationDao.insert(ycqYouthcallCampusEducation);

        return Result.ofSuccess(ycqYouthcallCampusEducation);
    }

    @Override
    public Result<Object> createEncryption(YcqYouthcallCampusEducationVo ycqYouthcallCampusEducationVo) {
        String sysCode = "YOUTHCALL";
        String url = "http://localhost:11011/pushController/offCampusEducation";
        // url ="http://121.229.175.82:63221/api/service-sso/pushController/offCampusEducation";
        String df = "yyyyMMddHHmmss";
        Random r = new Random();
        StringBuilder rs = new StringBuilder();
        for (int i = 0; i < 6; i++) {
            rs.append(r.nextInt(10));
        }
        HashMap<String, Object> pushMap = new HashMap<>();
        String encode = Sm2Util.encrypt(publicKey, JSONObject.toJSONString(ycqYouthcallCampusEducationVo));
        String request = Sm2Util.decrypt(privateKey, encode);
        log.info("解密字符串:{}", request);
        pushMap.put("encode", encode);
        pushMap.put("transactionId", sysCode + LocalDateTime.now().format(DateTimeFormatter.ofPattern(df)) + rs);
        pushMap.put("sysCode", sysCode);
        HttpClient httpClient = HttpClients.createDefault();
        HttpPost httpPost = new HttpPost(url);
        httpPost.addHeader(new BasicHeader("Content-Type", "application/json"));
        httpPost.setEntity(new StringEntity(JSONObject.toJSONString(pushMap), Charset.defaultCharset()));
        HttpResponse response = null;
        try {
            response = httpClient.execute(httpPost);
            String result = EntityUtils.toString(response.getEntity());
            JSONObject jsonObject = JSONObject.parseObject(result);
            log.info("jsonObject--->{}", jsonObject);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return null;
    }

    private byte[] tobyteAry(List list) {
        byte[] bytes = null;
        try {
            bytes = JsonSerilizable.serilizableForList(list).getBytes();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return bytes;
    }

    public List byteArrayToList(byte[] bytes, Class clazz) {
        List list = new ArrayList<>();
        try {
            list =JsonSerilizable.deserilizableForListFromFile(new String(bytes), clazz);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return list;

    }
}
