package com.crunii.micro.service.youthcall.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.ObsService;
import com.crunii.micro.feign.client.dto.FileVo;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallCheckInApply;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallCheckInApplyDao;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallCheckInCancelApply;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallCheckInApplyQueryCondition;
import com.crunii.micro.service.youthcall.enums.CheckInApplyStatusEnum;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallCheckInApplyRepo;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallCheckInCancelApplyRepo;
import com.crunii.micro.service.youthcall.service.YcqYouthcallCheckInApplyService;
import com.crunii.micro.service.youthcall.thirdrequest.QnyzDataRequestService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author PANJL
 * @date 2023-12-16 14:29
 */
@Slf4j
@Service
public class YcqYouthcallCheckInApplyServiceImpl implements YcqYouthcallCheckInApplyService {

    @Resource
    private YcqYouthcallCheckInApplyRepo ycqYouthcallCheckInApplyRepo;

    @Resource
    private YcqYouthcallCheckInApplyDao ycqYouthcallCheckInApplyDao;

    @Resource
    private YcqYouthcallCheckInCancelApplyRepo ycqYouthcallCheckInCancelApplyRepo;

    @Resource
    private QnyzDataRequestService qnyzDataRequestService;

    @Resource
    private ObsService obsService;

    @Override
    public void saveApply(YcqYouthcallCheckInApply ycqYouthcallCheckInApply, Map<String, List<MultipartFile>> fileMap) {
        // 面试通知最多不能超过5张
        List<MultipartFile> interviewPic = fileMap.getOrDefault("interviewPic", null);
        if (interviewPic != null && interviewPic.size() > 5) {
            throw new BusinessException("面试通知最多不能超过5张");
        }
        // 文件大小不能超过2m
        for (Map.Entry<String, List<MultipartFile>> entry : fileMap.entrySet()) {
            List<MultipartFile> multipartFiles = entry.getValue();
            for (MultipartFile multipartFile : multipartFiles) {
                double size = multipartFile.getSize() / (1024.0 * 1024.0);
                if (size > 2)
                    throw new BusinessException("附件 " + multipartFile.getOriginalFilename() + " 大小不能超过2M");
            }
        }
        /*
        // 属性校验和赋予默认值
        // 审核状态，默认待审核
        ycqYouthcallCheckInApply.setAuditStatus(CheckInApplyStatusEnum.YSSTATE_0.getAuditStatusCode());
        ycqYouthcallCheckInApply.setOriginalAuditStatusLabel(CheckInApplyStatusEnum.YSSTATE_0.getOriginalLabel());
        ycqYouthcallCheckInApply.setActualAuditStatusLabel(CheckInApplyStatusEnum.YSSTATE_0.getActualLabel());
        // 申请类型，默认入住申请
        ycqYouthcallCheckInApply.setApplyType("入住申请");
        // 驿站id
        String hotelId = ycqYouthcallCheckInApply.getHotelId();
        if (StringUtils.isEmpty(hotelId)) throw new BusinessException("驿站id不能为空");
        // 驿站名称
        String hotelName = ycqYouthcallCheckInApply.getHotelName();
        if (StringUtils.isEmpty(hotelName)) throw new BusinessException("驿站名称不能为空");
        // 申请人id
        String applyUserId = ycqYouthcallCheckInApply.getApplyUserId();
        if (StringUtils.isEmpty(applyUserId)) throw new BusinessException("申请人id不能为空");
        // 申请人姓名
        String applyUserName = ycqYouthcallCheckInApply.getApplyUserName();
        if (StringUtils.isEmpty(applyUserName)) throw new BusinessException("申请人姓名不能为空");
        // 判断用户是否累计2次取消申请，累计两次取消申请后在180天内不能再次申请
        boolean allowApply = this.allowApply(applyUserId);
        if (!allowApply) throw new BusinessException("180天内有2次取消申请，不允许发起申请");
        // 手机号
        String phone = ycqYouthcallCheckInApply.getPhone();
        if (StringUtils.isEmpty(phone)) throw new BusinessException("手机号不能为空");
        // 身份证号
        String idCard = ycqYouthcallCheckInApply.getIdCard();
        if (StringUtils.isEmpty(idCard)) throw new BusinessException("身份证号不能为空");
        // 入住时间
        Date checkInDate = ycqYouthcallCheckInApply.getCheckInDate();
        if (checkInDate == null) throw new BusinessException("入住时间不能为空");
        // 入住天数
        Integer checkInDays = ycqYouthcallCheckInApply.getCheckInDays();
        if (checkInDays == null) throw new BusinessException("入住天数不能为空");
        // 入住天数大于0的整数、小于8
        if (checkInDays <= 0) throw new BusinessException("入住天数必须是大于0的整数");
        if (checkInDays >= 8) throw new BusinessException("入住天数必须小于8天");
        // todo 计算离店时间
        ycqYouthcallCheckInApply.setCheckOutDate(null);
        // 是否毕业
        Integer grad = ycqYouthcallCheckInApply.getGrad();
        if (grad == null) throw new BusinessException("是否毕业不能为空");
        if (grad != 0 && grad != 1) throw new BusinessException("是否毕业的值只能为0或1");
        // 已毕业
        if (grad == 1) {
//            // 毕业证书
//            String gradCert = ycqYouthcallCheckInApply.getGradCert();
//            if (StringUtils.isEmpty(gradCert)) throw new BusinessException("毕业证书不能为空");
//            // 学位证书
//            String degreeCert = ycqYouthcallCheckInApply.getDegreeCert();
//            if (StringUtils.isEmpty(degreeCert)) throw new BusinessException("学位证书不能为空");
        }
        // 未毕业
        else {
//            // 学籍证明或学生证
//            String studentCard = ycqYouthcallCheckInApply.getStudentCard();
//            if (StringUtils.isEmpty(studentCard)) throw new BusinessException("学位证书不能为空");
        }
        // 申请时间
        Date currentDate = new Date();
        ycqYouthcallCheckInApply.setApplyTime(currentDate);
        // 创建时间
        ycqYouthcallCheckInApply.setCreateTime(currentDate);
        ycqYouthcallCheckInApply = ycqYouthcallCheckInApplyRepo.save(ycqYouthcallCheckInApply);
        // 当前登录人为创建人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (currWorker != null) {
            ycqYouthcallCheckInApply.setCreateUserId(currWorker.getLoginId());
            ycqYouthcallCheckInApply.setCreateUserName(currWorker.getWorkerName());
        }
        ycqYouthcallCheckInApply.setDelFlag(0);
        */
        // 调用入住申请接口，返回入住申请id
//        // 身份证正面
//        ycqYouthcallCheckInApply.setIdCardFront(this.uploadFile(fileMap, "fontCardPic"));
//        // 身份证反面
//        ycqYouthcallCheckInApply.setIdCardBack(this.uploadFile(fileMap, "backCardPic"));
//        // 面试通知
//        ycqYouthcallCheckInApply.setInterview(this.uploadFile(fileMap, "interviewPic"));
//        // 毕业证书
//        ycqYouthcallCheckInApply.setGradCert(this.uploadFile(fileMap, "diplomaPic"));
//        // 学位证书
//        ycqYouthcallCheckInApply.setDegreeCert(this.uploadFile(fileMap, "graduatePic"));
//        // 学籍证明或学生证
//        ycqYouthcallCheckInApply.setStudentCard(this.uploadFile(fileMap, "schoolPic"));
        String applyIdStr;
        try {
            applyIdStr = qnyzDataRequestService.saveApply(ycqYouthcallCheckInApply, fileMap);
        } catch (Exception e) {
            throw new BusinessException(e.getMessage());
        }
        // 入住申请id
        BigInteger applyId = BigInteger.valueOf(Long.parseLong(applyIdStr));
//        ycqYouthcallCheckInApply.setApplyId(applyId);
        System.out.println("applyId: " + applyId);
    }

    private String uploadFile(Map<String, List<MultipartFile>> fileMap, String fieldName) {
        List<String> fileUrls = new ArrayList<>();
        List<MultipartFile> multipartFileList = fileMap.getOrDefault(fieldName, null);
        if (CollectionUtils.isNotEmpty(multipartFileList)) {
            for (MultipartFile multipartFile : multipartFileList) {
                Result<FileVo> fileVoResult = obsService.feignFileUpload(multipartFile);
                if (fileVoResult.isSuccess()) {
                    fileUrls.add(fileVoResult.getData().getFileUrl());
                } else {
                    throw new BusinessException("上传学籍证明或学生证失败：{}", fileVoResult.getMsg());
                }
            }
        }
        return fileUrls.isEmpty() ? null : StringUtils.join(fileUrls, ",");
    }

    @Override
    public void cancelApply(BigInteger applyId) {
        // 调用取消申请接口
        try {
            qnyzDataRequestService.applyCancel(applyId.toString());
        } catch (Exception e) {
            throw new BusinessException(e.getMessage());
        }
        // 申请状态更新为已关闭
        this.updateAuditStatus(applyId, CheckInApplyStatusEnum.YSSTATE_2.getAuditStatusCode(), null);
        // 新增一条取消申请记录
        YcqYouthcallCheckInApply ycqYouthcallCheckInApply = this.queryByApplyId(applyId);
        if (ycqYouthcallCheckInApply != null) {
            YcqYouthcallCheckInCancelApply ycqYouthcallCheckInCancelApply = new YcqYouthcallCheckInCancelApply();
            ycqYouthcallCheckInCancelApply.setApplyId(applyId);
            ycqYouthcallCheckInCancelApply.setApplyUserId(ycqYouthcallCheckInApply.getApplyUserId());
            ycqYouthcallCheckInCancelApply.setCancelTime(new Date());
            ycqYouthcallCheckInCancelApplyRepo.save(ycqYouthcallCheckInCancelApply);
        }
    }

    @Override
    public void update(YcqYouthcallCheckInApply ycqYouthcallCheckInApply) {
        // 当前登录人为修改人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (currWorker != null) {
            ycqYouthcallCheckInApply.setUpdateUserId(currWorker.getLoginId());
            ycqYouthcallCheckInApply.setUpdateUserName(currWorker.getWorkerName());
        }
        ycqYouthcallCheckInApply.setUpdateTime(new Date());
        ycqYouthcallCheckInApplyRepo.save(ycqYouthcallCheckInApply);
    }

    @Override
    public void updateAuditStatus(BigInteger applyId, String auditStatus, String auditComment) {
        YcqYouthcallCheckInApply ycqYouthcallCheckInApply = this.queryByApplyId(applyId);
        if (ycqYouthcallCheckInApply == null) {
            throw new BusinessException("入住申请不存在");
        }
        // 更新审核状态
        ycqYouthcallCheckInApply.setAuditStatus(auditStatus);
        // 更新审核状态显示值
        CheckInApplyStatusEnum checkInApplyStatusEnum = CheckInApplyStatusEnum.fromValue(auditStatus);
        if (checkInApplyStatusEnum != null) {
            ycqYouthcallCheckInApply.setOriginalAuditStatusLabel(checkInApplyStatusEnum.getOriginalLabel());
            ycqYouthcallCheckInApply.setActualAuditStatusLabel(checkInApplyStatusEnum.getActualLabel());
        } else {
            ycqYouthcallCheckInApply.setOriginalAuditStatusLabel(auditStatus);
            ycqYouthcallCheckInApply.setActualAuditStatusLabel(auditStatus);
        }
        // 更新审核意见
        ycqYouthcallCheckInApply.setAuditComment(auditComment);
        // 更新数据
        this.update(ycqYouthcallCheckInApply);
    }

    @Override
    public void deleteByApplyId(BigInteger applyId) {
        YcqYouthcallCheckInApply ycqYouthcallCheckInApply = this.queryByApplyId(applyId);
        if (ycqYouthcallCheckInApply == null) {
            throw new BusinessException("入住申请不存在");
        }
        // 更新逻辑删除标识
        ycqYouthcallCheckInApply.setDelFlag(1);
        // 更新数据
        this.update(ycqYouthcallCheckInApply);
    }

    @Override
    public YcqYouthcallCheckInApply queryByApplyId(BigInteger applyId) {
        return ycqYouthcallCheckInApplyRepo.findById(applyId).orElse(null);
    }

    @Override
    public PageRsp<YcqYouthcallCheckInApply> pageQuery(PageReq<YcqYouthcallCheckInApplyQueryCondition> req) {
        return ycqYouthcallCheckInApplyDao.pageQuery(req);
    }

    @Override
    public List<YcqYouthcallCheckInApply> getAll() {
        return ycqYouthcallCheckInApplyRepo.findAll();
    }

    @Override
    public boolean allowApply(String applyUserId) {
        // 查询申请人的申请取消列表
        List<YcqYouthcallCheckInCancelApply> list = ycqYouthcallCheckInCancelApplyRepo.findListByApplyUserId(applyUserId);
        // 为空，允许申请
        if (list == null || list.isEmpty()) return true;
        long count = list.stream()
                .filter(ycqYouthcallCheckInCancelApply -> {
                    // 取消日期
                    Date cancelTime = ycqYouthcallCheckInCancelApply.getCancelTime();
                    LocalDate cancelDate = cancelTime.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
                    // 180天内
                    return cancelDate.isAfter(LocalDate.now().minusDays(180));
                }).count();
        return count < 2;
    }

}
