package com.crunii.micro.service.youthcall.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.youthcall.dto.*;
import com.crunii.micro.service.youthcall.dto.vo.commonDemand.*;
import com.crunii.micro.service.youthcall.dto.vo.personalDemand.PersonalDemandVo;
import com.crunii.micro.service.youthcall.enums.*;
import com.crunii.micro.service.youthcall.jdbc.*;
import com.crunii.micro.service.youthcall.service.YcqYouthcallCommonDemandService;
import com.crunii.micro.service.youthcall.utils.CharacterCodeUtil;
import com.crunii.micro.service.youthcall.utils.CopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 青呼我为-共性需求管理;(YCQ_YOUTHCALL_COMMON_DEMAND)表服务实现类
 * @author : yingtiansheng
 * @date : 2023-11-01
 */
@Slf4j
@Service
public class YcqYouthcallCommonDemandServiceImpl implements YcqYouthcallCommonDemandService {

    @Resource
    private YcqYouthcallCommonDemandDao ycqYouthcallCommonDemandDao;

    @Resource
    private YcqYouthcallPersonalDemandRepo ycqYouthcallPersonalDemandRepo;

    @Resource
    private YcqYouthcallCommonDemandRepo ycqYouthcallCommonDemandRepo;

    @Resource
    private YcqYouthcallCommonPersonalRepo ycqYouthcallCommonPersonalRepo;

    @Resource
    private YcqYouthcallDemandLogRepo ycqYouthcallDemandLogRepo;

    @Resource
    private YcqYouthcallWorkOrderDispatchRepo ycqYouthcallWorkOrderDispatchRepo;

    @Resource
    private YcqYouthcallWorkOrderDispatchOrgRepo ycqYouthcallWorkOrderDispatchOrgRepo;

    @Resource
    private YcqYouthcallWorkOrderRepo ycqYouthcallWorkOrderRepo;

    @Resource
    private YcqYouthcallWorkOrderOrgRepo ycqYouthcallWorkOrderOrgRepo;

    @Resource
    private YcqYouthcallWorkOrderProcessRepo ycqYouthcallWorkOrderProcessRepo;


    @Override
    public Result<PageRsp<CommonDemandVo>> pageQuery(PageReq<YcqYouthcallCommonDemandReq> req) {
        try {
            log.info("YcqYouthcallCommonDemandServiceImpl.pageQuery.param：{}", req.toString());
            PageRsp<CommonDemandVo> pageRsp = ycqYouthcallCommonDemandDao.pageSearch(req);
            pageRsp.getResult().forEach(commonDemandVo -> {
                commonDemandVo.setPersonDemandContentList(StringUtils.isNotEmpty(commonDemandVo.getPersonDemandContentStr()) ? Arrays.asList(commonDemandVo.getPersonDemandContentStr().split("\\|\\|\\|")) : new ArrayList<>());
                commonDemandVo.setDemandImgList(StringUtils.isNotEmpty(commonDemandVo.getDemandImg()) ? Arrays.asList(commonDemandVo.getDemandImg().split(",")) : new ArrayList<>());
            });
            return Result.ofSuccess(pageRsp);
        } catch (BusinessException e) {
            log.error("YcqYouthcallCommonDemandServiceImpl.pageQuery.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCommonDemandServiceImpl.pageQuery.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public Result<List<CommonDemandBaseVo>> listQuery(YcqYouthcallCommonDemandReq req) {
        try {
            List<YcqYouthcallCommonDemand> ycqYouthcallCommonDemandList = new ArrayList<>();
            List<CommonDemandBaseVo> commonDemandBaseVoList = new ArrayList<>();
            if (req == null || StringUtils.isEmpty(req.getDemandType())) {
                ycqYouthcallCommonDemandList = ycqYouthcallCommonDemandRepo.findByDelFlag(DelFlagEnum.UN_DELETE.getCode());
            } else {
                ycqYouthcallCommonDemandList = ycqYouthcallCommonDemandRepo.findByDemandTypeAndDelFlag(req.getDemandType(), DelFlagEnum.UN_DELETE.getCode());
            }
            ycqYouthcallCommonDemandList.forEach(ycqYouthcallCommonDemand -> {
                CommonDemandBaseVo commonDemandBaseVo = new CommonDemandBaseVo();
                CopyUtil.copyPropertiesNonNull(ycqYouthcallCommonDemand,commonDemandBaseVo);
                if (StringUtils.isNotBlank(ycqYouthcallCommonDemand.getDemandImg())) {
                    commonDemandBaseVo.setDemandImgList(Arrays.asList(ycqYouthcallCommonDemand.getDemandImg().split(",")));
                }
                commonDemandBaseVoList.add(commonDemandBaseVo);
            });
            return Result.ofSuccess(commonDemandBaseVoList);
        } catch (BusinessException e) {
            log.error("YcqYouthcallCommonDemandServiceImpl.listQuery.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallCommonDemandServiceImpl.listQuery.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Boolean> edit(CommonDemandEditVo commonDemandEditVo, Worker worker) {
        YcqYouthcallCommonDemand ycqYouthcallCommonDemand = ycqYouthcallCommonDemandRepo.getOne(commonDemandEditVo.getId());
        if (ycqYouthcallCommonDemand == null) {
            return Result.ofFail("未找到共性数据");
        } else {
            Date curr = new Date();
            // 更新共性表数据
            ycqYouthcallCommonDemand.setDemandTitle(commonDemandEditVo.getDemandTitle());
            ycqYouthcallCommonDemand.setDemandContent(commonDemandEditVo.getDemandContent());
            ycqYouthcallCommonDemand.setDemandType(commonDemandEditVo.getDemandType());
            // 更新类型后需根据工单调度情况更新业务类型(根据需求类型查询工单调度表信息，获取所属业务)
            List<YcqYouthcallWorkOrderDispatch> ycqYouthcallWorkOrderDispatchList = ycqYouthcallWorkOrderDispatchRepo.findByWorkOrderTypeAndDispatchStatusAndDelFlag(commonDemandEditVo.getDemandType(),DispatchStatusEnum.OPEN.getValue(),DelFlagEnum.UN_DELETE.getCode());
            if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderDispatchList)) {
                ycqYouthcallCommonDemand.setAppCode(String.join(",",ycqYouthcallWorkOrderDispatchList.stream()
                        .map(YcqYouthcallWorkOrderDispatch::getAppCode)
                        .distinct()
                        .collect(Collectors.toList())));
                ycqYouthcallCommonDemand.setAppName(String.join(",",ycqYouthcallWorkOrderDispatchList.stream()
                        .map(YcqYouthcallWorkOrderDispatch::getAppName)
                        .distinct()
                        .collect(Collectors.toList())));
            }
            if (!CollectionUtils.isEmpty(commonDemandEditVo.getDemandImgList())) {
                ycqYouthcallCommonDemand.setDemandImg(String.join(",", commonDemandEditVo.getDemandImgList()));
            } else {
                ycqYouthcallCommonDemand.setDemandImg("");
            }
            ycqYouthcallCommonDemand.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallCommonDemand.setLastUpdatedTime(curr);
            ycqYouthcallCommonDemandRepo.saveAndFlush(ycqYouthcallCommonDemand);

            List<YcqYouthcallCommonPersonal> commonPersonalList = new ArrayList<>();
            if (CollectionUtils.isEmpty(commonDemandEditVo.getPersonalDemandIdList())) {
                commonPersonalList = ycqYouthcallCommonPersonalRepo.findByCommonDemandIdAndDelFlag(commonDemandEditVo.getId(),DelFlagEnum.UN_DELETE.getCode());
            } else {
                commonPersonalList = ycqYouthcallCommonPersonalRepo.findByCommonDemandIdAndPersonalDemandIdNotInAndDelFlag(commonDemandEditVo.getId(),commonDemandEditVo.getPersonalDemandIdList(),DelFlagEnum.UN_DELETE.getCode());
            }
            List<BigInteger> personalDemandIdList = commonPersonalList.stream()
                    .map(YcqYouthcallCommonPersonal::getPersonalDemandId)
                    .distinct()
                    .collect(Collectors.toList());
            // 更新共性需求绑定个性化需求关系表
            commonPersonalList.forEach(ycqYouthcallCommonPersonal -> {
                ycqYouthcallCommonPersonal.setDelFlag(DelFlagEnum.DELETED.getCode());
                ycqYouthcallCommonPersonal.setLastUpdatedId(worker.getLoginId());
                ycqYouthcallCommonPersonal.setLastUpdatedTime(curr);
            });
            ycqYouthcallCommonPersonalRepo.saveAllAndFlush(commonPersonalList);

            // 更新个性化需求表，移除的个性化需求状态改为已撤销
            List<YcqYouthcallPersonalDemand> ycqYouthcallPersonalDemandList = ycqYouthcallPersonalDemandRepo.findByIdIn(personalDemandIdList);
            List<YcqYouthcallDemandLog> ycqYouthcallDemandLogList = new ArrayList<>();
            ycqYouthcallPersonalDemandList.forEach(ycqYouthcallPersonalDemand -> {
                ycqYouthcallPersonalDemand.setPersonalDemandStatus(PersonalDemandStatusEnum.REVOKE.getValue());
                ycqYouthcallPersonalDemand.setDealId(worker.getLoginId());
                ycqYouthcallPersonalDemand.setDealName(worker.getWorkerName());
                ycqYouthcallPersonalDemand.setDealTime(curr);
                ycqYouthcallPersonalDemand.setLastUpdatedId(worker.getLoginId());
                ycqYouthcallPersonalDemand.setLastUpdatedTime(curr);

                YcqYouthcallDemandLog ycqYouthcallDemandLog = new YcqYouthcallDemandLog();
                ycqYouthcallDemandLog.setCommonDemandId(commonDemandEditVo.getId());
                ycqYouthcallDemandLog.setPersonalDemandId(ycqYouthcallPersonalDemand.getId());
                ycqYouthcallDemandLog.setChangeType(RevokeTypeEnum.COMMON_ONE.getValue());
                ycqYouthcallDemandLog.setDemandContent(ycqYouthcallPersonalDemand.getDemandContent());
                ycqYouthcallDemandLog.setCreatedTime(curr);
                ycqYouthcallDemandLog.setCreatedId(worker.getLoginId());
                ycqYouthcallDemandLog.setLastUpdatedTime(curr);
                ycqYouthcallDemandLog.setLastUpdatedId(worker.getLoginId());
                ycqYouthcallDemandLog.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
                ycqYouthcallDemandLogList.add(ycqYouthcallDemandLog);
            });
            ycqYouthcallPersonalDemandRepo.saveAllAndFlush(ycqYouthcallPersonalDemandList);

            // 个性化需求状态变更日志表增加数据
            ycqYouthcallDemandLogRepo.saveAllAndFlush(ycqYouthcallDemandLogList);
        }
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteCommonDemand(BigInteger id, Worker worker) {
        Date curr = new Date();
        YcqYouthcallCommonDemand ycqYouthcallCommonDemand = ycqYouthcallCommonDemandRepo.getOne(id);
        if (ycqYouthcallCommonDemand == null) {
            throw new BusinessException("未找到该共性需求");
        } else {
            // 删除共性表数据
            ycqYouthcallCommonDemand.setDelFlag(DelFlagEnum.DELETED.getCode());
            ycqYouthcallCommonDemand.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallCommonDemand.setLastUpdatedTime(new Date());
            ycqYouthcallCommonDemandRepo.saveAndFlush(ycqYouthcallCommonDemand);

            // 去除该共性需求下所有个性需求绑定关系
            List<YcqYouthcallCommonPersonal> ycqYouthcallCommonPersonalList = ycqYouthcallCommonPersonalRepo.findByCommonDemandIdAndDelFlag(id,DelFlagEnum.UN_DELETE.getCode());
            if (!CollectionUtils.isEmpty(ycqYouthcallCommonPersonalList)) {
                List<BigInteger> personalDemandIdList = ycqYouthcallCommonPersonalList.stream()
                        .map(YcqYouthcallCommonPersonal::getPersonalDemandId)
                        .distinct()
                        .collect(Collectors.toList());
                ycqYouthcallCommonPersonalList.forEach(ycqYouthcallCommonPersonal -> {
                    ycqYouthcallCommonPersonal.setDelFlag(DelFlagEnum.DELETED.getCode());
                    ycqYouthcallCommonPersonal.setLastUpdatedId(worker.getLoginId());
                    ycqYouthcallCommonPersonal.setLastUpdatedTime(curr);
                });
                ycqYouthcallCommonPersonalRepo.saveAllAndFlush(ycqYouthcallCommonPersonalList);

                List<YcqYouthcallPersonalDemand> ycqYouthcallPersonalDemandList = ycqYouthcallPersonalDemandRepo.findByIdIn(personalDemandIdList);
                List<YcqYouthcallDemandLog> ycqYouthcallDemandLogList = new ArrayList<>();
                // 该共性需求下所有个性需求状态改为待转化
                ycqYouthcallPersonalDemandList.forEach(ycqYouthcallPersonalDemand -> {
                    ycqYouthcallPersonalDemand.setPersonalDemandStatus(PersonalDemandStatusEnum.UN_TRANSFORM.getValue());
                    ycqYouthcallPersonalDemand.setLastUpdatedId(worker.getLoginId());
                    ycqYouthcallPersonalDemand.setLastUpdatedTime(curr);

                    YcqYouthcallDemandLog ycqYouthcallDemandLog = new YcqYouthcallDemandLog();
                    ycqYouthcallDemandLog.setCommonDemandId(id);
                    ycqYouthcallDemandLog.setPersonalDemandId(ycqYouthcallPersonalDemand.getId());
                    ycqYouthcallDemandLog.setChangeType(RevokeTypeEnum.COMMON_DELETE.getValue());
                    ycqYouthcallDemandLog.setDemandContent(ycqYouthcallPersonalDemand.getDemandContent());
                    ycqYouthcallDemandLog.setCreatedId(worker.getLoginId());
                    ycqYouthcallDemandLog.setCreatedTime(curr);
                    ycqYouthcallDemandLog.setLastUpdatedId(worker.getLoginId());
                    ycqYouthcallDemandLog.setLastUpdatedTime(curr);
                    ycqYouthcallDemandLog.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
                    ycqYouthcallDemandLogList.add(ycqYouthcallDemandLog);
                });
                ycqYouthcallPersonalDemandRepo.saveAllAndFlush(ycqYouthcallPersonalDemandList);

                // 个性化需求状态日志表增加数据
                ycqYouthcallDemandLogRepo.saveAllAndFlush(ycqYouthcallDemandLogList);
            } else {
                log.info("无绑定个性需求");
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void removeAllPersonalDemand(BigInteger id, Worker worker) {
        List<YcqYouthcallCommonPersonal> ycqYouthcallCommonPersonalList = ycqYouthcallCommonPersonalRepo.findByCommonDemandIdAndDelFlag(id,DelFlagEnum.UN_DELETE.getCode());
        // 判断是否存在绑定关系
        if (!CollectionUtils.isEmpty(ycqYouthcallCommonPersonalList)) {
            Date curr = new Date();
            List<BigInteger> personalDemandIdList = ycqYouthcallCommonPersonalList.stream()
                    .map(YcqYouthcallCommonPersonal::getPersonalDemandId)
                    .distinct()
                    .collect(Collectors.toList());
            // 去除所有已绑定的个性化需求
            ycqYouthcallCommonPersonalList.forEach(ycqYouthcallCommonPersonal -> {
                ycqYouthcallCommonPersonal.setDelFlag(DelFlagEnum.DELETED.getCode());
                ycqYouthcallCommonPersonal.setLastUpdatedTime(curr);
                ycqYouthcallCommonPersonal.setLastUpdatedId(worker.getLoginId());
            });
            ycqYouthcallCommonPersonalRepo.saveAllAndFlush(ycqYouthcallCommonPersonalList);

            // 对应的个性化需求状态改为已撤销、增加个性化需求状态变更日志
            List<YcqYouthcallPersonalDemand> ycqYouthcallPersonalDemandList = ycqYouthcallPersonalDemandRepo.findByIdIn(personalDemandIdList);
            List<YcqYouthcallDemandLog> ycqYouthcallDemandLogList = new ArrayList<>();
            ycqYouthcallPersonalDemandList.forEach(ycqYouthcallPersonalDemand -> {
                ycqYouthcallPersonalDemand.setPersonalDemandStatus(PersonalDemandStatusEnum.REVOKE.getValue());
                ycqYouthcallPersonalDemand.setDealName(worker.getWorkerName());
                ycqYouthcallPersonalDemand.setDealId(worker.getLoginId());
                ycqYouthcallPersonalDemand.setDealTime(curr);
                ycqYouthcallPersonalDemand.setLastUpdatedId(worker.getLoginId());
                ycqYouthcallPersonalDemand.setLastUpdatedTime(curr);

                YcqYouthcallDemandLog ycqYouthcallDemandLog = new YcqYouthcallDemandLog();
                ycqYouthcallDemandLog.setCommonDemandId(id);
                ycqYouthcallDemandLog.setPersonalDemandId(ycqYouthcallPersonalDemand.getId());
                ycqYouthcallDemandLog.setChangeType(RevokeTypeEnum.COMMON_ALL.getValue());
                ycqYouthcallDemandLog.setDemandContent(ycqYouthcallPersonalDemand.getDemandContent());
                ycqYouthcallDemandLog.setCreatedId(worker.getLoginId());
                ycqYouthcallDemandLog.setCreatedTime(curr);
                ycqYouthcallDemandLog.setLastUpdatedId(worker.getLoginId());
                ycqYouthcallDemandLog.setLastUpdatedTime(curr);
                ycqYouthcallDemandLog.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
                ycqYouthcallDemandLogList.add(ycqYouthcallDemandLog);
            });
            ycqYouthcallPersonalDemandRepo.saveAllAndFlush(ycqYouthcallPersonalDemandList);
            ycqYouthcallDemandLogRepo.saveAllAndFlush(ycqYouthcallDemandLogList);
        } else {
            log.info("当前共性需求无绑定的个性化需求");
        }
    }

    @Override
    public Result<CommonDemandDetailVo> queryDetailById(BigInteger id) {
        CommonDemandDetailVo commonDemandDetailVo = new CommonDemandDetailVo();
        // 查询共性需求基础信息
        YcqYouthcallCommonDemand ycqYouthcallCommonDemand = ycqYouthcallCommonDemandRepo.getOne(id);
        if (ycqYouthcallCommonDemand == null) {
            return Result.ofFail("未找到当前共性需求");
        }
        CopyUtil.copyPropertiesNonNull(ycqYouthcallCommonDemand,commonDemandDetailVo);
        // 设置共性需求图片列表
        if (StringUtils.isNotEmpty(commonDemandDetailVo.getDemandImg())) {
            commonDemandDetailVo.setCommonDemandImgList(Arrays.asList(commonDemandDetailVo.getDemandImg().split(",")));
        } else {
            log.info("无共性需求图片");
        }
        // 查询工序需求绑定的个性化需求列表
        List<YcqYouthcallCommonPersonal> ycqYouthcallCommonPersonalList = ycqYouthcallCommonPersonalRepo.findByCommonDemandIdAndDelFlag(id,DelFlagEnum.UN_DELETE.getCode());
        // 判断是否存在绑定的个性化需求
        if (!CollectionUtils.isEmpty(ycqYouthcallCommonPersonalList)) {
            List<BigInteger> personalDemandIdList = ycqYouthcallCommonPersonalList.stream()
                    .map(YcqYouthcallCommonPersonal::getPersonalDemandId)
                    .distinct()
                    .collect(Collectors.toList());
            List<YcqYouthcallPersonalDemand> ycqYouthcallPersonalDemandList = ycqYouthcallPersonalDemandRepo.findByIdIn(personalDemandIdList);
            List<PersonalDemandVo> personDemandList = new ArrayList<>();
            List<String> allPersonalDemandImgList = new ArrayList<>();
            ycqYouthcallPersonalDemandList.forEach(ycqYouthcallPersonalDemand -> {
                PersonalDemandVo personalDemandVo = new PersonalDemandVo();
                CopyUtil.copyPropertiesNonNull(ycqYouthcallPersonalDemand,personalDemandVo);
                // 判断是否存在图片
                if (StringUtils.isNotEmpty(personalDemandVo.getDemandImg())) {
                    personalDemandVo.setDemandImgList(Arrays.asList(personalDemandVo.getDemandImg().split(",")));
                    allPersonalDemandImgList.addAll(Arrays.asList(personalDemandVo.getDemandImg().split(",")));
                }
                personDemandList.add(personalDemandVo);
            });
            commonDemandDetailVo.setPersonDemandList(personDemandList);
            commonDemandDetailVo.setAllPersonalDemandImgList(allPersonalDemandImgList);
        } else {
            commonDemandDetailVo.setPersonDemandList(new ArrayList<>());
            commonDemandDetailVo.setAllPersonalDemandImgList(new ArrayList<>());
        }

        return Result.ofSuccess(commonDemandDetailVo);
    }

    @Override
    public Result<Boolean> changeToWorkOrder(BigInteger id, Worker worker) {
        YcqYouthcallCommonDemand ycqYouthcallCommonDemand = ycqYouthcallCommonDemandRepo.getOne(id);
        if (ycqYouthcallCommonDemand == null) {
            return Result.ofFail("未找到共性需求");
        } else {
            if (CommonDemandStatusEnum.TRANSFERRED.getValue().equals(ycqYouthcallCommonDemand.getCommonDemandStatus())) {
                return Result.ofFail("该共性需求已转工单，请勿重复转化");
            } else {
                Date curr = new Date();
                // 更新共性需求状态
                ycqYouthcallCommonDemand.setCommonDemandStatus(CommonDemandStatusEnum.TRANSFERRED.getValue());
                ycqYouthcallCommonDemand.setDealId(worker.getLoginId());
                ycqYouthcallCommonDemand.setDealName(worker.getWorkerName());
                ycqYouthcallCommonDemand.setDealTime(curr);
                ycqYouthcallCommonDemand.setLastUpdatedId(worker.getLoginId());
                ycqYouthcallCommonDemand.setLastUpdatedTime(curr);
                ycqYouthcallCommonDemandRepo.saveAndFlush(ycqYouthcallCommonDemand);

                //工单表插入数据
                YcqYouthcallWorkOrder ycqYouthcallWorkOrder = YcqYouthcallWorkOrder.commonDemandToWorkOrder(ycqYouthcallCommonDemand,worker);
                BigInteger workOrderId = ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder).getId();

                //工单组织表插入数据 根据工单类型、所属业务确认组织信息
                List<YcqYouthcallWorkOrderDispatch> ycqYouthcallWorkOrderDispatchList = ycqYouthcallWorkOrderDispatchRepo.findByWorkOrderTypeAndAppCodeAndDispatchStatusAndDelFlag(ycqYouthcallCommonDemand.getDemandType(),ycqYouthcallCommonDemand.getAppCode(), DispatchStatusEnum.OPEN.getValue(), DelFlagEnum.UN_DELETE.getCode());
                if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderDispatchList)) {
                    List<BigInteger> idList = ycqYouthcallWorkOrderDispatchList.stream()
                            .map(YcqYouthcallWorkOrderDispatch::getId)
                            .collect(Collectors.toList());
                    List<YcqYouthcallWorkOrderDispatchOrg> workOrderDispatchOrgList = ycqYouthcallWorkOrderDispatchOrgRepo.findByDispatchIdInAndDelFlag(idList,DelFlagEnum.UN_DELETE.getCode());
                    List<YcqYouthcallWorkOrderOrg> ycqYouthcallWorkOrderOrgList = new ArrayList<>();
                    workOrderDispatchOrgList.forEach(workOrderDispatchOrg -> {
                        YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = new YcqYouthcallWorkOrderOrg();
                        ycqYouthcallWorkOrderOrg.setWorkOrderId(workOrderId);
                        ycqYouthcallWorkOrderOrg.setAppCode(ycqYouthcallCommonDemand.getAppCode());
                        ycqYouthcallWorkOrderOrg.setAppName(ycqYouthcallCommonDemand.getAppName());
                        ycqYouthcallWorkOrderOrg.setOrgId(workOrderDispatchOrg.getOrgId());
                        ycqYouthcallWorkOrderOrg.setOrgName(workOrderDispatchOrg.getOrgName());
                        ycqYouthcallWorkOrderOrg.setWorkOrderType(ycqYouthcallCommonDemand.getDemandType());
                        ycqYouthcallWorkOrderOrg.setOrgType(OrderOrgTypeEnum.ONE.getValue());
                        ycqYouthcallWorkOrderOrg.setCreatedId(worker.getLoginId());
                        ycqYouthcallWorkOrderOrg.setCreatedTime(curr);
                        ycqYouthcallWorkOrderOrg.setLastUpdatedId(worker.getLoginId());
                        ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
                        ycqYouthcallWorkOrderOrg.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
                        ycqYouthcallWorkOrderOrg.setShowType(OrderOrgShowTypeEnum.DEAL.getValue());
                        ycqYouthcallWorkOrderOrgList.add(ycqYouthcallWorkOrderOrg);
                    });
                    ycqYouthcallWorkOrderOrgRepo.saveAllAndFlush(ycqYouthcallWorkOrderOrgList.stream().collect(Collectors.collectingAndThen(
                            Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(YcqYouthcallWorkOrderOrg::getOrgId))),
                            ArrayList::new)));
                }
                YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = YcqYouthcallWorkOrderProcess.pendingProcess(ycqYouthcallWorkOrder);
                ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
            }
            return Result.ofSuccess(true);
        }
    }

}
