package com.crunii.micro.service.youthcall.service.impl;

import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.service.youthcall.dto.*;
import com.crunii.micro.service.youthcall.dto.vo.personalDemand.PersonalDemandVo;
import com.crunii.micro.service.youthcall.dto.vo.personalDemand.YcqYouthcallPersonalDemandInsertVo;
import com.crunii.micro.service.youthcall.dto.vo.personalDemand.YcqYouthcallPersonalDemandReq;
import com.crunii.micro.service.youthcall.dto.vo.personalDemand.YcqYouthcallPersonalToCommonVo;
import com.crunii.micro.service.youthcall.enums.*;
import com.crunii.micro.service.youthcall.jdbc.*;
import com.crunii.micro.service.youthcall.service.YcqYouthcallPersonalDemandService;
import com.crunii.micro.service.youthcall.utils.CharacterCodeUtil;
import com.crunii.micro.service.youthcall.utils.CopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 青呼我为-个性化需求管理;(YCQ_YOUTHCALL_PERSONAL_DEMAND)表服务实现类
 * @author : yingtiansheng
 * @date : 2023-11-01
 */
@Slf4j
@Service
public class YcqYouthcallPersonalDemandServiceImpl implements YcqYouthcallPersonalDemandService {

    @Resource
    private YcqYouthcallPersonalDemandDao ycqYouthcallPersonalDemandDao;

    @Resource
    private YcqYouthcallPersonalDemandRepo ycqYouthcallPersonalDemandRepo;

    @Resource
    private YcqYouthcallCommonDemandRepo ycqYouthcallCommonDemandRepo;

    @Resource
    private YcqYouthcallCommonPersonalRepo ycqYouthcallCommonPersonalRepo;

    @Resource
    private YcqYouthcallDemandLogRepo ycqYouthcallDemandLogRepo;

    @Resource
    private YcqYouthcallWorkOrderDispatchRepo ycqYouthcallWorkOrderDispatchRepo;

    @Resource
    private RedisTemplate<String,String> redisTemplate;


    @Override
    public Result<PageRsp<PersonalDemandVo>> pageQuery(PageReq<YcqYouthcallPersonalDemandReq> req) {
        try {
            log.info("YcqYouthcallPersonalDemandServiceImpl.pageQuery.param：{}", req.toString());
            PageRsp<PersonalDemandVo> personalDemandVoPageRsp = ycqYouthcallPersonalDemandDao.pageSearch(req);
            personalDemandVoPageRsp.getResult().forEach(personalDemandVo -> {
                // 图片转换成list
                if (StringUtils.isNotBlank(personalDemandVo.getDemandImg())) {
                    String[] array = personalDemandVo.getDemandImg().split(",");
                    personalDemandVo.setDemandImgList(Arrays.asList(array));
                } else {
                    log.info("暂无图片");
                }
            });
            return Result.ofSuccess(personalDemandVoPageRsp);
        } catch (BusinessException e) {
            log.error("YcqYouthcallPersonalDemandServiceImpl.pageQuery.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallPersonalDemandServiceImpl.pageQuery.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public Result<Boolean> insertPersonalDemand(YcqYouthcallPersonalDemandInsertVo ycqYouthcallWorkOrderDispatchInsertVo, Worker worker) {
//        String phone = ycqYouthcallWorkOrderDispatchInsertVo.getPhone();
//        // 当传了手机号，需要校验验证码
//        if (!StringUtils.isEmpty(phone)) {
//            String code = ycqYouthcallWorkOrderDispatchInsertVo.getCode();
//            //校验验证码
//            Object checkCode = redisTemplate.opsForValue().get(RedisKeys.VALIDATE_CODE_KEY + phone);
//            if (ObjectUtils.isEmpty(code) || !code.equals(checkCode)) {
//                throw new BusinessException("验证码错误或已过期，请重新获取");
//            }
//            //校验成功后删除
//            redisTemplate.delete(RedisKeys.VALIDATE_CODE_KEY + phone);
//        }
        Date curr = new Date();
        YcqYouthcallPersonalDemand ycqYouthcallPersonalDemand = new YcqYouthcallPersonalDemand();
        ycqYouthcallPersonalDemand.setPersonalDemandNo(CharacterCodeUtil.generateCharacterCode("GXHXQ"));
        ycqYouthcallPersonalDemand.setDemandContent(ycqYouthcallWorkOrderDispatchInsertVo.getDemandContent());
        if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderDispatchInsertVo.getImgList())) {
            ycqYouthcallPersonalDemand.setDemandImg(String.join(",",ycqYouthcallWorkOrderDispatchInsertVo.getImgList()));
        }
        ycqYouthcallPersonalDemand.setPersonalDemandStatus(PersonalDemandStatusEnum.UN_TRANSFORM.getValue());
        ycqYouthcallPersonalDemand.setCreatedId(worker.getLoginId());
        ycqYouthcallPersonalDemand.setCreatedTime(curr);
        ycqYouthcallPersonalDemand.setCreatedName(worker.getWorkerName());
        ycqYouthcallPersonalDemand.setLastUpdatedId(worker.getLoginId());
        ycqYouthcallPersonalDemand.setLastUpdatedTime(curr);
        ycqYouthcallPersonalDemand.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        ycqYouthcallPersonalDemandRepo.saveAndFlush(ycqYouthcallPersonalDemand);
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Boolean> revokePersonalDemand(BigInteger id, Worker worker) {
        YcqYouthcallPersonalDemand ycqYouthcallPersonalDemand = ycqYouthcallPersonalDemandRepo.getOne(id);
        if (ycqYouthcallPersonalDemand == null) {
            return Result.ofFail("未找到该个性化需求");
        } else {
            Date curr = new Date();
            // 更新个性化需求表
            ycqYouthcallPersonalDemand.setPersonalDemandStatus(PersonalDemandStatusEnum.REVOKE.getValue());
            ycqYouthcallPersonalDemand.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallPersonalDemand.setLastUpdatedTime(curr);
            ycqYouthcallPersonalDemand.setDealId(worker.getLoginId());
            ycqYouthcallPersonalDemand.setDealName(worker.getWorkerName());
            ycqYouthcallPersonalDemand.setDealTime(curr);
            ycqYouthcallPersonalDemandRepo.saveAndFlush(ycqYouthcallPersonalDemand);

            //移除共性个性化关联关系
            List<YcqYouthcallCommonPersonal> ycqYouthcallCommonPersonalList = ycqYouthcallCommonPersonalRepo.findByPersonalDemandIdAndDelFlag(id,DelFlagEnum.UN_DELETE.getCode());
            if (!CollectionUtils.isEmpty(ycqYouthcallCommonPersonalList)) {
                ycqYouthcallCommonPersonalList.forEach(ycqYouthcallCommonPersonal -> {
                    ycqYouthcallCommonPersonal.setDelFlag(DelFlagEnum.DELETED.getCode());
                    ycqYouthcallCommonPersonal.setLastUpdatedId(worker.getLoginId());
                    ycqYouthcallCommonPersonal.setLastUpdatedTime(curr);
                });
                ycqYouthcallCommonPersonalRepo.saveAllAndFlush(ycqYouthcallCommonPersonalList);
            } else {
                log.info("该个性化需求未关联共性需求");
            }

            //增加撤销日志
            YcqYouthcallDemandLog ycqYouthcallDemandLog = new YcqYouthcallDemandLog();
            ycqYouthcallDemandLog.setPersonalDemandId(id);
            ycqYouthcallDemandLog.setDemandContent(ycqYouthcallPersonalDemand.getDemandContent());
            ycqYouthcallDemandLog.setChangeType(RevokeTypeEnum.PERSONAL.getValue());
            ycqYouthcallDemandLog.setCreatedId(worker.getLoginId());
            ycqYouthcallDemandLog.setCreatedTime(curr);
            ycqYouthcallDemandLog.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallDemandLog.setLastUpdatedTime(curr);
            ycqYouthcallDemandLog.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
            ycqYouthcallDemandLogRepo.saveAndFlush(ycqYouthcallDemandLog);
        }
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Boolean> changeCommon(YcqYouthcallPersonalToCommonVo ycqYouthcallPersonalToCommonVo, Worker worker) {
        // 判断个性化需求是否已转共性
        long count = ycqYouthcallPersonalDemandRepo.countByIdInAndPersonalDemandStatusAndDelFlag(ycqYouthcallPersonalToCommonVo.getIdList(),PersonalDemandStatusEnum.TRANSFORM.getValue(),DelFlagEnum.UN_DELETE.getCode());
        if (count > 0) {
            return Result.ofFail("存在已转化的个性化工单，请勿重复转化");
        }
        Date curr = new Date();
        BigInteger commonDemandId = ycqYouthcallPersonalToCommonVo.getCommonDemandId();

        // 新创建共性需求时需增加共性需求表数据
        if (CreateCommonDemandTypeEnum.CREATE.getValue().equals(ycqYouthcallPersonalToCommonVo.getCreateCommonType())) {
            String demandType = ycqYouthcallPersonalToCommonVo.getDemandType();
            YcqYouthcallCommonDemand ycqYouthcallCommonDemand = new YcqYouthcallCommonDemand();
            ycqYouthcallCommonDemand.setCommonDemandNo(CharacterCodeUtil.generateCharacterCode("GXXQ"));
            ycqYouthcallCommonDemand.setDemandTitle(ycqYouthcallPersonalToCommonVo.getDemandTitle());
            ycqYouthcallCommonDemand.setDemandContent(ycqYouthcallPersonalToCommonVo.getDemandContent());
            ycqYouthcallCommonDemand.setDemandType(demandType);
            if (!CollectionUtils.isEmpty(ycqYouthcallPersonalToCommonVo.getDemandImgList())) {
                ycqYouthcallCommonDemand.setDemandImg(String.join(",",ycqYouthcallPersonalToCommonVo.getDemandImgList()));
            }
            // 根据需求类型查询工单调度表信息，获取所属业务
            List<YcqYouthcallWorkOrderDispatch> ycqYouthcallWorkOrderDispatchList = ycqYouthcallWorkOrderDispatchRepo.findByWorkOrderTypeAndDispatchStatusAndDelFlag(demandType,DispatchStatusEnum.OPEN.getValue(),DelFlagEnum.UN_DELETE.getCode());
            if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderDispatchList)) {
                ycqYouthcallCommonDemand.setAppCode(String.join(",",ycqYouthcallWorkOrderDispatchList.stream()
                        .map(YcqYouthcallWorkOrderDispatch::getAppCode)
                        .distinct()
                        .collect(Collectors.toList())));
                ycqYouthcallCommonDemand.setAppName(String.join(",",ycqYouthcallWorkOrderDispatchList.stream()
                        .map(YcqYouthcallWorkOrderDispatch::getAppName)
                        .distinct()
                        .collect(Collectors.toList())));
            }
            ycqYouthcallCommonDemand.setCommonDemandStatus(CommonDemandStatusEnum.PENDING.getValue());
            ycqYouthcallCommonDemand.setCreatedId(worker.getLoginId());
            ycqYouthcallCommonDemand.setCreatedName(worker.getWorkerName());
            ycqYouthcallCommonDemand.setCreatedTime(curr);
            ycqYouthcallCommonDemand.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallCommonDemand.setLastUpdatedTime(curr);
            ycqYouthcallCommonDemand.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
            commonDemandId = ycqYouthcallCommonDemandRepo.saveAndFlush(ycqYouthcallCommonDemand).getId();
        } else {
            // 更新共性需求内容/图片
            YcqYouthcallCommonDemand ycqYouthcallCommonDemand = ycqYouthcallCommonDemandRepo.getOne(commonDemandId);
            if (!CollectionUtils.isEmpty(ycqYouthcallPersonalToCommonVo.getDemandImgList())) {
                ycqYouthcallCommonDemand.setDemandImg(String.join(",",ycqYouthcallPersonalToCommonVo.getDemandImgList()));
            }
            ycqYouthcallCommonDemand.setDemandContent(ycqYouthcallPersonalToCommonVo.getDemandContent());
            ycqYouthcallCommonDemand.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallCommonDemand.setLastUpdatedTime(curr);
            ycqYouthcallCommonDemandRepo.saveAndFlush(ycqYouthcallCommonDemand);
        }
        BigInteger finalCommonDemandId = commonDemandId;

        //个性化需求表需求状态更新、个性化需求状态日志表增加数据
        List<YcqYouthcallPersonalDemand> ycqYouthcallPersonalDemandList = ycqYouthcallPersonalDemandRepo.findByIdIn(ycqYouthcallPersonalToCommonVo.getIdList());
        List<YcqYouthcallDemandLog> ycqYouthcallDemandLogList = new ArrayList<>();
        ycqYouthcallPersonalDemandList.forEach(ycqYouthcallPersonalDemand -> {
            ycqYouthcallPersonalDemand.setPersonalDemandStatus(PersonalDemandStatusEnum.TRANSFORM.getValue());
            ycqYouthcallPersonalDemand.setLastUpdatedTime(curr);
            ycqYouthcallPersonalDemand.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallPersonalDemand.setDealTime(curr);
            ycqYouthcallPersonalDemand.setDealId(worker.getLoginId());
            ycqYouthcallPersonalDemand.setDealName(worker.getWorkerName());

            YcqYouthcallDemandLog ycqYouthcallDemandLog = new YcqYouthcallDemandLog();
            ycqYouthcallDemandLog.setCommonDemandId(finalCommonDemandId);
            ycqYouthcallDemandLog.setPersonalDemandId(ycqYouthcallPersonalDemand.getId());
            ycqYouthcallDemandLog.setChangeType(RevokeTypeEnum.COMMON_TRANSFORM.getValue());
            ycqYouthcallDemandLog.setDemandContent(ycqYouthcallPersonalDemand.getDemandContent());
            ycqYouthcallDemandLog.setCreatedId(worker.getLoginId());
            ycqYouthcallDemandLog.setCreatedTime(curr);
            ycqYouthcallDemandLog.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallDemandLog.setLastUpdatedTime(curr);
            ycqYouthcallDemandLog.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
            ycqYouthcallDemandLogList.add(ycqYouthcallDemandLog);
        });
        ycqYouthcallPersonalDemandRepo.saveAllAndFlush(ycqYouthcallPersonalDemandList);
        ycqYouthcallDemandLogRepo.saveAllAndFlush(ycqYouthcallDemandLogList);

        //共性个性化关联表增加数据
        List<YcqYouthcallCommonPersonal> ycqYouthcallCommonPersonalList = new ArrayList<>();
        ycqYouthcallPersonalToCommonVo.getIdList().forEach(personalDemandId -> {
            YcqYouthcallCommonPersonal ycqYouthcallCommonPersonal = new YcqYouthcallCommonPersonal();
            ycqYouthcallCommonPersonal.setCommonDemandId(finalCommonDemandId);
            ycqYouthcallCommonPersonal.setPersonalDemandId(personalDemandId);
            ycqYouthcallCommonPersonal.setCreatedId(worker.getLoginId());
            ycqYouthcallCommonPersonal.setCreatedTime(curr);
            ycqYouthcallCommonPersonal.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallCommonPersonal.setLastUpdatedTime(curr);
            ycqYouthcallCommonPersonal.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
            ycqYouthcallCommonPersonalList.add(ycqYouthcallCommonPersonal);
        });
        ycqYouthcallCommonPersonalRepo.saveAllAndFlush(ycqYouthcallCommonPersonalList);

        return Result.ofSuccess(true);
    }

    @Override
    public Result<List<PersonalDemandVo>> selectByIds(YcqYouthcallPersonalToCommonVo ycqYouthcallPersonalToCommonVo) {
        if (ycqYouthcallPersonalToCommonVo == null || CollectionUtils.isEmpty(ycqYouthcallPersonalToCommonVo.getIdList())) {
            return Result.ofFail("id异常");
        } else {
            List<YcqYouthcallPersonalDemand> ycqYouthcallPersonalDemandList = ycqYouthcallPersonalDemandRepo.findByIdIn(ycqYouthcallPersonalToCommonVo.getIdList());
            List<PersonalDemandVo> personalDemandVoList = new ArrayList<>();
            ycqYouthcallPersonalDemandList.forEach(ycqYouthcallPersonalDemand -> {
                PersonalDemandVo personalDemandVo = new PersonalDemandVo();
                CopyUtil.copyPropertiesNonNull(ycqYouthcallPersonalDemand,personalDemandVo);
                // 图片转换成list
                if (StringUtils.isNotBlank(personalDemandVo.getDemandImg())) {
                    String[] array = personalDemandVo.getDemandImg().split(",");
                    personalDemandVo.setDemandImgList(Arrays.asList(array));
                } else {
                    log.info("暂无图片");
                }
                personalDemandVoList.add(personalDemandVo);
            });
            return Result.ofSuccess(personalDemandVoList);
        }
    }

    @Override
    public Result<List<String>> selectImgByIds(YcqYouthcallPersonalToCommonVo ycqYouthcallPersonalToCommonVo) {
        if (ycqYouthcallPersonalToCommonVo == null || CollectionUtils.isEmpty(ycqYouthcallPersonalToCommonVo.getIdList())) {
            return Result.ofFail("id异常");
        } else {
            List<String> imgList = new ArrayList<>();
            List<YcqYouthcallPersonalDemand> ycqYouthcallPersonalDemandList = ycqYouthcallPersonalDemandRepo.findByIdIn(ycqYouthcallPersonalToCommonVo.getIdList());
            ycqYouthcallPersonalDemandList.forEach(ycqYouthcallPersonalDemand -> {
                // 图片转换成list
                if (StringUtils.isNotBlank(ycqYouthcallPersonalDemand.getDemandImg())) {
                    imgList.addAll(Arrays.asList(ycqYouthcallPersonalDemand.getDemandImg().split(",")));
                } else {
                    log.info("暂无图片");
                }
            });
            return Result.ofSuccess(imgList);
        }
    }

    @Override
    public void deletePersonalDemand(BigInteger id, Worker worker) {
        YcqYouthcallPersonalDemand ycqYouthcallPersonalDemand = ycqYouthcallPersonalDemandRepo.getOne(id);
        if (ycqYouthcallPersonalDemand == null) {
            throw new BusinessException("未找到该个性化需求");
        } else {
            ycqYouthcallPersonalDemand.setDelFlag(DelFlagEnum.DELETED.getCode());
            ycqYouthcallPersonalDemand.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallPersonalDemand.setLastUpdatedTime(new Date());
            ycqYouthcallPersonalDemandRepo.saveAndFlush(ycqYouthcallPersonalDemand);
        }
    }
}
