package com.crunii.micro.service.youthcall.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallProductAdvice;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrder;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrderProcess;
import com.crunii.micro.service.youthcall.dto.vo.productAdviceVo.ProductAdviceVo;
import com.crunii.micro.service.youthcall.dto.vo.productAdviceVo.YcqYouthcallProductAdviceDealReq;
import com.crunii.micro.service.youthcall.dto.vo.productAdviceVo.YcqYouthcallProductAdviceInsertVo;
import com.crunii.micro.service.youthcall.dto.vo.productAdviceVo.YcqYouthcallProductAdviceReq;
import com.crunii.micro.service.youthcall.enums.AdviceStatusEnum;
import com.crunii.micro.service.youthcall.enums.DelFlagEnum;
import com.crunii.micro.service.youthcall.enums.WorkOrderStatusEnum;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallProductAdviceDao;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallProductAdviceRepo;
import com.crunii.micro.service.youthcall.service.YcqYouthcallProductAdviceService;
import com.crunii.micro.service.youthcall.utils.CharacterCodeUtil;
import com.crunii.micro.service.youthcall.utils.CopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.Arrays;
import java.util.Date;

/**
 * 青呼我为-产品建议管理;(YCQ_YOUTHCALL_PRODUCT_ADVICE)表服务实现类
 * @author : yingtiansheng
 * @date : 2023-11-01
 */
@Slf4j
@Service
public class YcqYouthcallProductAdviceServiceImpl implements YcqYouthcallProductAdviceService {

    @Resource
    private YcqYouthcallProductAdviceDao ycqYouthcallProductAdviceDao;

    @Resource
    private YcqYouthcallProductAdviceRepo ycqYouthcallProductAdviceRepo;

    @Override
    public Result<PageRsp<ProductAdviceVo>> pageQuery(PageReq<YcqYouthcallProductAdviceReq> req) {
        try {
            log.info("YcqYouthcallProductAdviceServiceImpl.pageQuery.param：{}", req.toString());
            PageRsp<ProductAdviceVo> productAdviceVoPageRsp = ycqYouthcallProductAdviceDao.pageSearch(req);
            productAdviceVoPageRsp.getResult().forEach(productAdviceVo -> {
                // 图片转换成list
                if (StringUtils.isNotBlank(productAdviceVo.getAdviceImg())) {
                    String[] array = productAdviceVo.getAdviceImg().split(",");
                    productAdviceVo.setAdviceImgList(Arrays.asList(array));
                } else {
                    log.info("暂无图片");
                }
            });
            return Result.ofSuccess(productAdviceVoPageRsp);
        } catch (BusinessException e) {
            log.error("YcqYouthcallProductAdviceServiceImpl.pageQuery.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallProductAdviceServiceImpl.pageQuery.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public Result<Boolean> insertProductAdvice(YcqYouthcallProductAdviceInsertVo ycqYouthcallProductAdviceInsertVo, Worker worker) {
        Date curr = new Date();
        YcqYouthcallProductAdvice ycqYouthcallProductAdvice = new YcqYouthcallProductAdvice();
        CopyUtil.copyPropertiesNonNull(ycqYouthcallProductAdviceInsertVo,ycqYouthcallProductAdvice);
        ycqYouthcallProductAdvice.setAdviceNo(CharacterCodeUtil.generateCharacterCode("CPJY"));
        if (!CollectionUtils.isEmpty(ycqYouthcallProductAdviceInsertVo.getAdviceImgList())) {
            ycqYouthcallProductAdvice.setAdviceImg(String.join(",",ycqYouthcallProductAdviceInsertVo.getAdviceImgList()));
        } else {
            log.info("暂无图片");
        }
        ycqYouthcallProductAdvice.setAdviceStatus(AdviceStatusEnum.UN_DEAL.getValue());
        ycqYouthcallProductAdvice.setCreatedId(worker.getLoginId());
        ycqYouthcallProductAdvice.setCreatedTime(curr);
        ycqYouthcallProductAdvice.setCreatedName(worker.getWorkerName());
        ycqYouthcallProductAdvice.setLastUpdatedId(worker.getLoginId());
        ycqYouthcallProductAdvice.setLastUpdatedTime(curr);
        ycqYouthcallProductAdvice.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        ycqYouthcallProductAdviceRepo.saveAndFlush(ycqYouthcallProductAdvice);
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Object> dealProductAdvice(YcqYouthcallProductAdviceDealReq ycqYouthcallProductAdviceDealReq) {
        // 验证是否已处理过
        if (ycqYouthcallProductAdviceDealReq.getProductAdviceId() == null) {
            return Result.ofFail("产品建议id异常");
        } else {
            if (ycqYouthcallProductAdviceRepo.countByIdAndAdviceStatusAndDelFlag(ycqYouthcallProductAdviceDealReq.getProductAdviceId(), AdviceStatusEnum.DEAL.getValue(),DelFlagEnum.UN_DELETE.getCode()) > 0) {
                return Result.ofFail("该产品建议已处理过");
            } else {
                Date curr = new Date();
                // 获取当前登陆人
                Worker currWorker = CurrWorkerUtil.getCurrWorker();
                // 更新工单表状态
                YcqYouthcallProductAdvice ycqYouthcallProductAdvice = ycqYouthcallProductAdviceRepo.getOne(ycqYouthcallProductAdviceDealReq.getProductAdviceId());
                ycqYouthcallProductAdvice.setAdviceStatus(AdviceStatusEnum.DEAL.getValue());
                ycqYouthcallProductAdvice.setDealContent(ycqYouthcallProductAdviceDealReq.getDealContent());
                ycqYouthcallProductAdvice.setAttachment(CollectionUtils.isEmpty(ycqYouthcallProductAdviceDealReq.getAttachmentList()) ? "":String.join(",", ycqYouthcallProductAdviceDealReq.getAttachmentList()));
                if (currWorker != null) {
                    ycqYouthcallProductAdvice.setDealId(currWorker.getLoginId());
                    ycqYouthcallProductAdvice.setDealTime(curr);
                    ycqYouthcallProductAdvice.setDealName(currWorker.getWorkerName());
                    ycqYouthcallProductAdvice.setLastUpdatedId(currWorker.getLoginId());
                    ycqYouthcallProductAdvice.setLastUpdatedTime(curr);
                }
                ycqYouthcallProductAdviceRepo.saveAndFlush(ycqYouthcallProductAdvice);
            }
        }
        return Result.ofSuccess(true);
    }


    @Override
    public void deleteProductAdvice(BigInteger id, Worker worker) {
        YcqYouthcallProductAdvice ycqYouthcallProductAdvice = ycqYouthcallProductAdviceRepo.getOne(id);
        if (ycqYouthcallProductAdvice == null) {
            throw new BusinessException("未找到该产品建议");
        } else {
            ycqYouthcallProductAdvice.setDelFlag(DelFlagEnum.DELETED.getCode());
            ycqYouthcallProductAdvice.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallProductAdvice.setLastUpdatedTime(new Date());
            ycqYouthcallProductAdviceRepo.saveAndFlush(ycqYouthcallProductAdvice);
        }
    }
}
