package com.crunii.micro.service.youthcall.service.impl;

import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.service.youthcall.dto.YcqPublishActivity;
import com.crunii.micro.service.youthcall.dto.YcqPublishActivityResp;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallQrcode;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallQrcodeActivityRel;
import com.crunii.micro.service.youthcall.dto.vo.YcqPublishActivityVo;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallQrcodeVo;
import com.crunii.micro.service.youthcall.jdbc.YcqPublishActivityDao;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallQrcodeActivityRelDao;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallQrcodeActivityRelRepo;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallQrcodeDao;
import com.crunii.micro.service.youthcall.service.YcqYouthcallQrcodeService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 青呼我为-青呼码管理;(YCQ_YOUTHCALL_QRCODE)表服务实现类
 * @author : http://www.chiner.pro
 * @date : 2023-9-20
 */
@Service
public class YcqYouthcallQrcodeServiceImpl implements YcqYouthcallQrcodeService {

    @Resource
    private YcqYouthcallQrcodeDao ycqYouthcallQrcodeDao;

    @Resource
    private YcqPublishActivityDao ycqPublishActivityDao;

    @Resource
    private YcqYouthcallQrcodeActivityRelDao ycqYouthcallQrcodeActivityRelDao;

    @Resource
    private YcqYouthcallQrcodeActivityRelRepo ycqYouthcallQrcodeActivityRelRepo;

    @Value("${app.qrCodeUrl}")
    private String qrCodeUrl;

    @Override
    public Result<PageRsp<YcqYouthcallQrcode>> paginQuery(PageReq<YcqYouthcallQrcodeVo> req) {

        return Result.ofSuccess(ycqYouthcallQrcodeDao.paginQuery(req));
    }

    @Override
    public Result<Object> insert(YcqYouthcallQrcode ycqYouthcallQrcode) {
        // 获取当前登陆人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (currWorker!=null) {
            ycqYouthcallQrcode.setCreateUserId(currWorker.getLoginId());
            ycqYouthcallQrcode.setCreateUserName(currWorker.getWorkerName());
        }
        ycqYouthcallQrcode = ycqYouthcallQrcodeDao.insert(ycqYouthcallQrcode);
        return Result.ofSuccess(ycqYouthcallQrcode);
    }

    @Override
    public Result<Object> update(YcqYouthcallQrcode ycqYouthcallQrcode) {
        ycqYouthcallQrcodeDao.update(ycqYouthcallQrcode);
        return Result.ofSuccess();
    }

    @Override
    public Result<YcqYouthcallQrcode> queryById(BigInteger id) {
        YcqYouthcallQrcode ycqYouthcallQrcode = ycqYouthcallQrcodeDao.queryById(id);
        return Result.ofSuccess(ycqYouthcallQrcode);
    }

    @Override
    public Result<YcqYouthcallQrcode> queryByYouthcallCode(String youthcallCode) {
        YcqYouthcallQrcode ycqYouthcallQrcode = ycqYouthcallQrcodeDao.queryByYouthcallCode(youthcallCode);
        return Result.ofSuccess(ycqYouthcallQrcode);
    }

    @Override
    public Result<Object> deleteById(BigInteger id) {
        ycqYouthcallQrcodeDao.deleteById(id);
        return Result.ofSuccess();
    }

    @Override
    public Result<Object> updateStatus(YcqYouthcallQrcode ycqYouthcallQrcode) {
        ycqYouthcallQrcodeDao.updateStatus(ycqYouthcallQrcode);
        return Result.ofSuccess();
    }

    @Override
    public Result<Object> updateVisibleAll(YcqYouthcallQrcode ycqYouthcallQrcode) {
        ycqYouthcallQrcodeDao.updateVisibleAll(ycqYouthcallQrcode);
        return Result.ofSuccess();
    }

    @Override
    public Result<Object> createBath(List<YcqYouthcallQrcode> ycqYouthcallQrcodes) {
        // 获取当前登陆人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (currWorker!=null) {
            for (YcqYouthcallQrcode ycqYouthcallQrcode : ycqYouthcallQrcodes) {
                ycqYouthcallQrcode.setCreateUserId(currWorker.getLoginId());
                ycqYouthcallQrcode.setCreateUserName(currWorker.getWorkerName());
            }
        }
        ycqYouthcallQrcodeDao.batchInsert(ycqYouthcallQrcodes);
        return Result.ofSuccess();
    }

    @Override
    public Result<List<YcqPublishActivity>> queryQrcodeActivity(YcqPublishActivityVo ycqPublishActivityVo) {
        List<YcqPublishActivity> list = ycqPublishActivityDao.paginQuery(ycqPublishActivityVo);
        return Result.ofSuccess(list);
    }

    @Override
    public Result<PageRsp<YcqPublishActivityResp>> queryQrcodeActivityByPage(PageReq<YcqPublishActivityVo> req) {
        PageRsp<YcqPublishActivityResp> pageRsp = ycqPublishActivityDao.queryQrcodeActivityByPage(req);
        return Result.ofSuccess(pageRsp);
    }

    @Override
    public Result<Object> bathSaveQrcodeActivityRel(List<YcqYouthcallQrcodeActivityRel> ycqYouthcallQrcodeActivityRels) {
        if (CollectionUtils.isEmpty(ycqYouthcallQrcodeActivityRels)) {
            return Result.ofSuccess("集合不能为空");
        }
        // 先清空该青呼码关联的活动
        BigInteger qrcodeId = ycqYouthcallQrcodeActivityRels.get(0).getQrcodeId();
        ycqYouthcallQrcodeActivityRelDao.deleteSqlBYQrcodeId(qrcodeId);
        // 获取当前登陆人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        int loginId = -1;
        String workerName = "";
        if (currWorker!=null) {
            loginId = currWorker.getLoginId();
            workerName = currWorker.getWorkerName();
        }
        for (YcqYouthcallQrcodeActivityRel ycqYouthcallQrcodeActivityRel : ycqYouthcallQrcodeActivityRels) {
            ycqYouthcallQrcodeActivityRel.setCreateUserId(loginId);
            ycqYouthcallQrcodeActivityRel.setCreateUserName(workerName);
        }
        ycqYouthcallQrcodeActivityRelRepo.saveAllAndFlush(ycqYouthcallQrcodeActivityRels);
        return Result.ofSuccess();
    }

    @Override
    public Result<List<YcqYouthcallQrcode>> export(YcqYouthcallQrcodeVo req) {
        List<YcqYouthcallQrcode> pageRsp = ycqYouthcallQrcodeDao.paginQueryNoPage(req);
        return Result.ofSuccess(pageRsp);
    }

    @Override
    public Result<Object> updateQrCodeInlet(YcqYouthcallQrcodeActivityRel ycqYouthcallQrcodeActivityRel) {
        ycqYouthcallQrcodeActivityRelDao.updateQrcodeinlet(ycqYouthcallQrcodeActivityRel);
        return Result.ofSuccess();
    }
}
