package com.crunii.micro.service.youthcall.service.impl;

import com.crunii.micro.common.constants.CommonConstants;
import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.common.utils.JsonUtil;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.TsmWorker;
import com.crunii.micro.domain.entity.main.YcqAppConfigUrl;
import com.crunii.micro.domain.repo.main.TsmOrgRepo;
import com.crunii.micro.domain.repo.main.TsmWorkerRepo;
import com.crunii.micro.domain.repo.main.YcqAppConfigUrlRepo;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrderDispatch;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrderDispatchOrg;
import com.crunii.micro.service.youthcall.dto.vo.WorkerInfo;
import com.crunii.micro.service.youthcall.dto.vo.orderDispatch.*;
import com.crunii.micro.service.youthcall.enums.DelFlagEnum;
import com.crunii.micro.service.youthcall.enums.DispatchStatusEnum;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallWorkOrderDispatchDao;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallWorkOrderDispatchOrgRepo;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallWorkOrderDispatchRepo;
import com.crunii.micro.service.youthcall.service.YcqYouthcallWorkOrderDispatchService;
import com.crunii.micro.service.youthcall.utils.CopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RMapCache;
import org.redisson.api.RedissonClient;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 青呼我为-工单管理;(YCQ_YOUTHCALL_WORK_ORDER_DISPATCH)表服务实现类
 * @author : yingtiansheng
 * @date : 2023-10-31
 */
@Slf4j
@Service
public class YcqYouthcallWorkOrderDispatchServiceImpl implements YcqYouthcallWorkOrderDispatchService {

    @Resource
    private YcqYouthcallWorkOrderDispatchDao ycqYouthcallWorkOrderDispatchDao;

    @Resource
    private YcqYouthcallWorkOrderDispatchRepo ycqYouthcallWorkOrderDispatchRepo;

    @Resource
    private YcqYouthcallWorkOrderDispatchOrgRepo ycqYouthcallWorkOrderDispatchOrgRepo;

    @Resource
    private YcqAppConfigUrlRepo ycqAppConfigUrlRepo;

    @Resource
    private TsmOrgRepo tsmOrgRepo;

    @Resource
    private TsmWorkerRepo tsmWorkerRepo;


    @Override
    public Result<PageRsp<WorkOrderDispatchVo>> pageQuery(PageReq<YcqYouthcallWorkOrderDispatchReq> req) {
        try {
            log.info("YcqYouthcallWorkOrderDispatchServiceImpl.pageQuery.param：{}", req.toString());
            PageRsp<WorkOrderDispatchVo> workOrderDispatchVoPageRsp = ycqYouthcallWorkOrderDispatchDao.pageSearch(req);
            workOrderDispatchVoPageRsp.getResult().forEach(workOrderDispatchVo -> {
                if (StringUtils.isNotEmpty(workOrderDispatchVo.getOrgIds())) {
                    workOrderDispatchVo.setOrgIdList(Arrays.asList(workOrderDispatchVo.getOrgIds().split(",")));
                } else {
                    workOrderDispatchVo.setOrgIdList(new ArrayList<>());
                }
            });
            return Result.ofSuccess(workOrderDispatchVoPageRsp);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderDispatchServiceImpl.pageQuery.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderDispatchServiceImpl.pageQuery.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    @Override
    public Result<Boolean> insertWorkOrderDispatch(YcqYouthcallWorkOrderDispatchInsertVo ycqYouthcallWorkOrderDispatchInsertVo, Worker worker) {
        YcqYouthcallWorkOrderDispatch ycqYouthcallWorkOrderDispatch = new YcqYouthcallWorkOrderDispatch();
        CopyUtil.copyPropertiesNonNull(ycqYouthcallWorkOrderDispatchInsertVo,ycqYouthcallWorkOrderDispatch);
        // 判断是否存在该业务类型数据
        /*if (ycqYouthcallWorkOrderDispatchRepo.countByWorkOrderTypeAndDelFlag(ycqYouthcallWorkOrderDispatchInsertVo.getWorkOrderType(),DelFlagEnum.UN_DELETE.getCode()) > 0) {
            return Result.ofFail("该工单类型已绑定业务组织");
        }*/
        //业务名称
        List<YcqAppConfigUrl> ycqAppConfigUrlList = ycqAppConfigUrlRepo.findByStateAndAppCode(CommonConstants.STATE_VALID, ycqYouthcallWorkOrderDispatchInsertVo.getAppCode());
        if (ycqAppConfigUrlList.size() < 1) {
            return Result.ofFail("不存在该业务编码");
        } else {
            // 新增工单调度表数据
            Date curr = new Date();
            ycqYouthcallWorkOrderDispatch.setAppName(ycqAppConfigUrlList.get(0).getAppName());
            // 调度信息状态默认为“关”
            ycqYouthcallWorkOrderDispatch.setDispatchStatus(DispatchStatusEnum.CLOSE.getValue());
            ycqYouthcallWorkOrderDispatch.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
            ycqYouthcallWorkOrderDispatch.setCreatedId(worker.getLoginId());
            ycqYouthcallWorkOrderDispatch.setCreatedName(worker.getWorkerName());
            ycqYouthcallWorkOrderDispatch.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallWorkOrderDispatch.setCreatedTime(curr);
            ycqYouthcallWorkOrderDispatch.setLastUpdatedTime(curr);
            BigInteger dispatchId = ycqYouthcallWorkOrderDispatchRepo.save(ycqYouthcallWorkOrderDispatch).getId();

            // 新增工单调度组织关联表数据
            List<YcqYouthcallWorkOrderDispatchOrg> ycqYouthcallWorkOrderDispatchOrgList = new ArrayList<>();
            List<BigInteger> orgIdList = ycqYouthcallWorkOrderDispatchInsertVo.getOrgIdList();
            orgIdList.forEach(orgId -> {
                TsmOrg tsmOrg = tsmOrgRepo.findOrgById(String.valueOf(orgId));
                YcqYouthcallWorkOrderDispatchOrg ycqYouthcallWorkOrderDispatchOrg = new YcqYouthcallWorkOrderDispatchOrg();
                ycqYouthcallWorkOrderDispatchOrg.setDispatchId(dispatchId);
                ycqYouthcallWorkOrderDispatchOrg.setOrgId(orgId);
                ycqYouthcallWorkOrderDispatchOrg.setOrgName(tsmOrg.getOrgName());
                if (tsmOrg.getManagePeople() != null) {
                    TsmWorker tsmWorker = tsmWorkerRepo.getOne(tsmOrg.getManagePeople());
                    ycqYouthcallWorkOrderDispatchOrg.setManagePeople(tsmOrg.getManagePeople());
                    ycqYouthcallWorkOrderDispatchOrg.setWorkerName(tsmWorker.getWorkerName());
                    ycqYouthcallWorkOrderDispatchOrg.setMobile(tsmWorker.getMobile());
                }
                ycqYouthcallWorkOrderDispatchOrg.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
                ycqYouthcallWorkOrderDispatchOrg.setCreatedId(worker.getLoginId());
                ycqYouthcallWorkOrderDispatchOrg.setLastUpdatedId(worker.getLoginId());
                ycqYouthcallWorkOrderDispatchOrg.setCreatedTime(curr);
                ycqYouthcallWorkOrderDispatchOrg.setLastUpdatedTime(curr);
                ycqYouthcallWorkOrderDispatchOrgList.add(ycqYouthcallWorkOrderDispatchOrg);
            });
            ycqYouthcallWorkOrderDispatchOrgRepo.saveAll(ycqYouthcallWorkOrderDispatchOrgList);
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Boolean> updateWorkOrderDispatch(YcqYouthcallWorkOrderDispatchUpdateVo updateReq, Worker worker) {
        YcqYouthcallWorkOrderDispatch ycqYouthcallWorkOrderDispatch = ycqYouthcallWorkOrderDispatchRepo.getOne(updateReq.getId());
        if (ycqYouthcallWorkOrderDispatch == null) {
            return Result.ofFail("未找到工单调度数据");
        } else {
            ycqYouthcallWorkOrderDispatch.setWorkOrderType(updateReq.getWorkOrderType());
            ycqYouthcallWorkOrderDispatch.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallWorkOrderDispatch.setLastUpdatedTime(new Date());
            ycqYouthcallWorkOrderDispatchRepo.saveAndFlush(ycqYouthcallWorkOrderDispatch);
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Boolean> switchStatus(YcqYouthcallWorkOrderDispatchStatusVo updateReq, Worker worker) {
        YcqYouthcallWorkOrderDispatch ycqYouthcallWorkOrderDispatch = ycqYouthcallWorkOrderDispatchRepo.getOne(updateReq.getId());
        if (ycqYouthcallWorkOrderDispatch == null) {
            return Result.ofFail("未找到工单调度数据");
        } else {
            ycqYouthcallWorkOrderDispatch.setDispatchStatus(updateReq.getDispatchStatus());
            ycqYouthcallWorkOrderDispatch.setLastUpdatedId(worker.getLoginId());
            ycqYouthcallWorkOrderDispatch.setLastUpdatedTime(new Date());
            ycqYouthcallWorkOrderDispatchRepo.saveAndFlush(ycqYouthcallWorkOrderDispatch);
        }
        return Result.ofSuccess(true);
    }

    @Override
    public void deleteWorkOrderDispatch(BigInteger id) {
        YcqYouthcallWorkOrderDispatch ycqYouthcallWorkOrderDispatch = ycqYouthcallWorkOrderDispatchRepo.getOne(id);
        // 判断是否存在数据
        if (ycqYouthcallWorkOrderDispatch == null) {
            throw new BusinessException("未找到工单调度数据");
        } else {
            ycqYouthcallWorkOrderDispatch.setDelFlag(DelFlagEnum.DELETED.getCode());
            ycqYouthcallWorkOrderDispatchRepo.save(ycqYouthcallWorkOrderDispatch);
            //同步删除调度组织关联表数据
            List<YcqYouthcallWorkOrderDispatchOrg> ycqYouthcallWorkOrderDispatchOrgList = ycqYouthcallWorkOrderDispatchOrgRepo.findByDispatchIdAndDelFlag(id, DelFlagEnum.UN_DELETE.getCode());
            if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderDispatchOrgList)) {
                ycqYouthcallWorkOrderDispatchOrgList.forEach(ycqYouthcallWorkOrderDispatchOrg -> {
                    ycqYouthcallWorkOrderDispatchOrg.setDelFlag(DelFlagEnum.DELETED.getCode());
                });
                ycqYouthcallWorkOrderDispatchOrgRepo.saveAllAndFlush(ycqYouthcallWorkOrderDispatchOrgList);
            }
        }
    }

    @Override
    public AppOrgVo findAppAndOrgByType(String workOrderType) {
        AppOrgVo appOrgVo = new AppOrgVo();
        // 查询业务信息
        List<YcqYouthcallWorkOrderDispatch> ycqYouthcallWorkOrderDispatchList = ycqYouthcallWorkOrderDispatchRepo.findByWorkOrderTypeAndDispatchStatusAndDelFlag(workOrderType,DispatchStatusEnum.OPEN.getValue(),DelFlagEnum.UN_DELETE.getCode());
        if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderDispatchList)) {
            appOrgVo.setAppCode(String.join(",",ycqYouthcallWorkOrderDispatchList.stream()
                    .map(YcqYouthcallWorkOrderDispatch::getAppCode)
                    .distinct()
                    .collect(Collectors.toList())));
            appOrgVo.setAppName(String.join(",",ycqYouthcallWorkOrderDispatchList.stream()
                    .map(YcqYouthcallWorkOrderDispatch::getAppName)
                    .distinct()
                    .collect(Collectors.toList())));
        }
        // 查询组织信息
        List<YcqYouthcallWorkOrderDispatchOrg> ycqYouthcallWorkOrderDispatchOrgList = ycqYouthcallWorkOrderDispatchOrgRepo.findByDispatchIdInAndDelFlag(ycqYouthcallWorkOrderDispatchList.stream().map(YcqYouthcallWorkOrderDispatch::getId).collect(Collectors.toList()),DelFlagEnum.UN_DELETE.getCode());
        if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderDispatchOrgList)) {
            appOrgVo.setOrgList(tsmOrgRepo.findByOrgIdIn(ycqYouthcallWorkOrderDispatchOrgList.stream().map(ycqYouthcallWorkOrderDispatchOrg -> Long.parseLong(String.valueOf(ycqYouthcallWorkOrderDispatchOrg.getOrgId()))).collect(Collectors.toList())));
        }
        return appOrgVo;
    }

    @Override
    public AppOrgVo findAppAndOrgByTypeOnAndOff(String workOrderType) {
        AppOrgVo appOrgVo = new AppOrgVo();
        // 查询业务信息
        List<YcqYouthcallWorkOrderDispatch> ycqYouthcallWorkOrderDispatchList = ycqYouthcallWorkOrderDispatchRepo.findByWorkOrderTypeAndDispatchStatusAndDelFlag(workOrderType,DispatchStatusEnum.OPEN.getValue(),DelFlagEnum.UN_DELETE.getCode());
        List<YcqYouthcallWorkOrderDispatch> close = ycqYouthcallWorkOrderDispatchRepo.findByWorkOrderTypeAndDispatchStatusAndDelFlag(workOrderType,DispatchStatusEnum.CLOSE.getValue(),DelFlagEnum.UN_DELETE.getCode());
        ycqYouthcallWorkOrderDispatchList.addAll(close);
        if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderDispatchList)) {
            appOrgVo.setAppCode(String.join(",",ycqYouthcallWorkOrderDispatchList.stream()
                    .map(YcqYouthcallWorkOrderDispatch::getAppCode)
                    .distinct()
                    .collect(Collectors.toList())));
            appOrgVo.setAppName(String.join(",",ycqYouthcallWorkOrderDispatchList.stream()
                    .map(YcqYouthcallWorkOrderDispatch::getAppName)
                    .distinct()
                    .collect(Collectors.toList())));
        }
        // 查询组织信息
        List<YcqYouthcallWorkOrderDispatchOrg> ycqYouthcallWorkOrderDispatchOrgList = ycqYouthcallWorkOrderDispatchOrgRepo.findByDispatchIdInAndDelFlag(ycqYouthcallWorkOrderDispatchList.stream().map(YcqYouthcallWorkOrderDispatch::getId).collect(Collectors.toList()),DelFlagEnum.UN_DELETE.getCode());
        if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderDispatchOrgList)) {
            appOrgVo.setOrgList(tsmOrgRepo.findByOrgIdIn(ycqYouthcallWorkOrderDispatchOrgList.stream().map(ycqYouthcallWorkOrderDispatchOrg -> Long.parseLong(String.valueOf(ycqYouthcallWorkOrderDispatchOrg.getOrgId()))).collect(Collectors.toList())));
        }
        return appOrgVo;
    }
}
