package com.crunii.micro.service.youthcall.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.builder.ExcelWriterBuilder;
import com.alibaba.excel.write.builder.ExcelWriterSheetBuilder;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.crunii.micro.common.constants.RedisKeys;
import com.crunii.micro.common.dto.PageReq;
import com.crunii.micro.common.dto.PageRsp;
import com.crunii.micro.common.dto.Result;
import com.crunii.micro.common.exception.BusinessException;
import com.crunii.micro.domain.dto.Worker;
import com.crunii.micro.domain.entity.main.TsmOrg;
import com.crunii.micro.domain.entity.main.TsmPropertyValue;
import com.crunii.micro.domain.repo.main.TsmPropertyValueRepo;
import com.crunii.micro.domain.util.CurrWorkerUtil;
import com.crunii.micro.feign.client.OrgService;
import com.crunii.micro.feign.client.SmsService;
import com.crunii.micro.feign.client.SysService;
import com.crunii.micro.feign.client.dto.WorkerC;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrder;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrderOrg;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrderProcess;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallWorkOrderResp;
import com.crunii.micro.service.youthcall.dto.vo.YcqYouthcallWorkOrderVo;
import com.crunii.micro.service.youthcall.dto.vo.orderDispatch.AppOrgVo;
import com.crunii.micro.service.youthcall.dto.vo.workOrder.*;
import com.crunii.micro.service.youthcall.enums.*;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallWorkOrderDao;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallWorkOrderOrgRepo;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallWorkOrderProcessRepo;
import com.crunii.micro.service.youthcall.jdbc.YcqYouthcallWorkOrderRepo;
import com.crunii.micro.service.youthcall.service.YcqYouthcallWorkOrderDispatchService;
import com.crunii.micro.service.youthcall.service.YcqYouthcallWorkOrderService;
import com.crunii.micro.service.youthcall.utils.BeanCopyUtil;
import com.crunii.micro.service.youthcall.utils.CharacterCodeUtil;
import com.crunii.micro.service.youthcall.utils.CopyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 青呼我为-工单管理;(YCQ_YOUTHCALL_WORK_ORDER)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2023-9-20
 */
@Slf4j
@Service
public class YcqYouthcallWorkOrderServiceImpl implements YcqYouthcallWorkOrderService {
    public static final String WORKER_ORDER_ADMIN = "青呼我为工单管理员";
    @Resource
    private YcqYouthcallWorkOrderDao ycqYouthcallWorkOrderDao;

    @Resource
    private YcqYouthcallWorkOrderDispatchService ycqYouthcallWorkOrderDispatchService;

    @Resource
    private YcqYouthcallWorkOrderRepo ycqYouthcallWorkOrderRepo;

    @Resource
    private YcqYouthcallWorkOrderProcessRepo ycqYouthcallWorkOrderProcessRepo;

    @Resource
    private YcqYouthcallWorkOrderOrgRepo ycqYouthcallWorkOrderOrgRepo;

    @Resource
    private TsmPropertyValueRepo tsmPropertyValueRepo;

    @Resource
    private SysService sysService;

    @Resource
    private RedisTemplate<String,Object> redisTemplate;

    @Resource
    SmsService smsService;

    @Resource
    OrgService orgService;


    @Override
    public Result<PageRsp<WorkOrderBaseVo>> paginQuery(PageReq<YcqYouthcallWorkOrderVo> req) {
        PageRsp<WorkOrderBaseVo> pageRsp = ycqYouthcallWorkOrderDao.paginQuery(req);
        if (pageRsp.getResult() != null) {
            pageRsp.getResult().forEach(workOrderBaseVo -> {
                workOrderBaseVo.setWorkOrderImgList(StringUtils.isNotEmpty(workOrderBaseVo.getWorkOrderImg()) ? Arrays.asList(workOrderBaseVo.getWorkOrderImg().split(",")) : new ArrayList<>());
                int i = ycqYouthcallWorkOrderDao.querySameSql(workOrderBaseVo.getOrgId());
                workOrderBaseVo.setSameOrg(i > 0 ? "1" : "0");
            });
        }
        return Result.ofSuccess(pageRsp);
    }

    @Override
    public Result<PageRsp<WorkOrderBaseVo>> reportPaginQuery(PageReq<YcqYouthcallWorkOrderVo> req) {
        PageRsp<WorkOrderBaseVo> pageRsp = ycqYouthcallWorkOrderDao.reportPaginQuery(req);
        pageRsp.getResult().forEach(workOrderBaseVo -> {
            workOrderBaseVo.setWorkOrderImgList(StringUtils.isNotEmpty(workOrderBaseVo.getWorkOrderImg()) ? Arrays.asList(workOrderBaseVo.getWorkOrderImg().split(",")) : new ArrayList<>());
        });
        return Result.ofSuccess(pageRsp);
    }

    @Override
    public Result<PageRsp<WorkOrderMyVo>> myPaginQuery(PageReq<YcqYouthcallWorkOrderVo> req) {
        // 获取当前登陆人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (currWorker == null) {
            return Result.ofFail("获取当前用户信息异常");
        } else {
            req.getQuery().setProposerId(BigInteger.valueOf(currWorker.getLoginId()));
            PageRsp<WorkOrderMyVo> pageRsp = ycqYouthcallWorkOrderDao.myPaginQuery(req);
            pageRsp.getResult().forEach(workOrderMyVo -> {
                // 设置状态标签
                if (WorkOrderStatusEnum.PENDING.getValue().equals(workOrderMyVo.getProcessStatus()) ||
                        WorkOrderStatusEnum.PENDING_ESCALATION.getValue().equals(workOrderMyVo.getProcessStatus()) ||
                        WorkOrderStatusEnum.PENDING_ALLOCATION.getValue().equals(workOrderMyVo.getProcessStatus()) ||
                        WorkOrderStatusEnum.PENDING_PEER_TRANSFER.getValue().equals(workOrderMyVo.getProcessStatus())
                ) {
                    workOrderMyVo.setStatusTag("待处理");
                } else {
                    if (StringUtils.isNotEmpty(workOrderMyVo.getEvaluateName())) {
                        workOrderMyVo.setStatusTag("已评价");
                    } else {
                        workOrderMyVo.setStatusTag("待评价");
                    }
                }
                workOrderMyVo.setWorkOrderImgList(StringUtils.isNotEmpty(workOrderMyVo.getWorkOrderImg()) ? Arrays.asList(workOrderMyVo.getWorkOrderImg().split(",")) : new ArrayList<>());
            });
            return Result.ofSuccess(pageRsp);
        }
    }

    @Override
    public Result<Object> insert(YcqYouthcallWorkOrder ycqYouthcallWorkOrder) {
        String userName = "admin";
        // 获取当前登陆人
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        if (currWorker != null) {
            userName = currWorker.getWorkerName();
            String mobile = currWorker.getMobile();
            ycqYouthcallWorkOrder.setProposerId(BigInteger.valueOf(currWorker.getLoginId()));
            ycqYouthcallWorkOrder.setProposerName(userName);
            ycqYouthcallWorkOrder.setProposerMobile(mobile);
        }
        ycqYouthcallWorkOrder.setProcessStatus("01");
        ycqYouthcallWorkOrder = ycqYouthcallWorkOrderDao.insert(ycqYouthcallWorkOrder);
        return Result.ofSuccess(ycqYouthcallWorkOrder);
    }

    @Override
    public Result<Object> update(YcqYouthcallWorkOrder ycqYouthcallWorkOrder) {
//        ycqYouthcallWorkOrderDao.update(ycqYouthcallWorkOrder);
        return Result.ofSuccess();
    }

    @Override
    public Result<Object> queryWorkOrderCount(BigInteger proposerId) {
        YcqYouthcallWorkOrderResp ycqYouthcallWorkOrderResp = ycqYouthcallWorkOrderDao.queryWorkOrderCount(proposerId);
        return Result.ofSuccess(ycqYouthcallWorkOrderResp);
    }

    @Override
    public Result<Object> updateProcessStatus(YcqYouthcallWorkOrder ycqYouthcallWorkOrder) {
        ycqYouthcallWorkOrderDao.updateProcessStatus(ycqYouthcallWorkOrder);
        return Result.ofSuccess();
    }

    @Override
    public Result<YcqYouthcallWorkOrder> queryById(BigInteger id) {
        YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderDao.queryById(id);
        return Result.ofSuccess(ycqYouthcallWorkOrder);
    }

    @Override
    public Result<Object> deleteById(BigInteger id) {
        ycqYouthcallWorkOrderDao.deleteById(id);
        return Result.ofSuccess();
    }

    @Override
    @Transactional
    public Result<Object> insertAppeal(YcqYouthcallWorkOrderInsertReq ycqYouthcallWorkOrderInsertReq) {
//        String phone = ycqYouthcallWorkOrderInsertReq.getPhone();
//        //当传了手机号，需要校验验证码
//        if (!StringUtils.isEmpty(phone)) {
//            String code = ycqYouthcallWorkOrderInsertReq.getCode();
//            //校验验证码
//            Object checkCode = redisTemplate.opsForValue().get(RedisKeys.VALIDATE_CODE_KEY + phone);
//            if (ObjectUtils.isEmpty(code) || !code.equals(checkCode)) {
////                throw new BusinessException("验证码错误或已过期，请重新获取");
//            }
//            //校验成功后删除
//            redisTemplate.delete(RedisKeys.VALIDATE_CODE_KEY + phone);
//        }

        // 获取当前登录人
//        CurrWorkerUtil.mockCurrWorker();
        Worker currWorker = CurrWorkerUtil.getCurrWorker();
        Date curr = new Date();
        YcqYouthcallWorkOrder youthcallWorkOrder = new YcqYouthcallWorkOrder();
        CopyUtil.copyPropertiesNonNull(ycqYouthcallWorkOrderInsertReq, youthcallWorkOrder);
        youthcallWorkOrder.setWorkOrderNo(CharacterCodeUtil.generateCharacterCode("QHWW"));
        youthcallWorkOrder.setZoneId(ycqYouthcallWorkOrderInsertReq.getZoneId()); //所属区域id
        if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderInsertReq.getWorkOrderImgList())) {
            youthcallWorkOrder.setWorkOrderImg(String.join(",", ycqYouthcallWorkOrderInsertReq.getWorkOrderImgList()));
        } else {
            log.info("暂无图片");
        }
        // 根据工单类型查询工单调度表，得到组织信息、业务信息
        AppOrgVo appOrgVo = ycqYouthcallWorkOrderDispatchService.findAppAndOrgByType(ycqYouthcallWorkOrderInsertReq.getWorkOrderType());
//        if (!CollectionUtils.isEmpty(appOrgVo.getOrgList())) {
//            // 此表orgName只有展示使用
//            List<String> orgNameList = appOrgVo.getOrgList().stream().map(TsmOrg::getOrgName).collect(Collectors.toList());
//            String nameJoin = StringUtils.join(orgNameList, ",");
//            youthcallWorkOrder.setOrgId(BigInteger.valueOf(appOrgVo.getOrgList().get(0).getOrgId()));
//            youthcallWorkOrder.setOrgName(nameJoin);
//        } else {
//            log.info("暂无关联工单调度组织");
//        }
        //根据区域id获取直属组织
        Result<List<TsmOrg>> tsmOrgListResult = orgService.getOrgByZoneId(ycqYouthcallWorkOrderInsertReq.getZoneId());
        List<TsmOrg> tsmOrgList = tsmOrgListResult.getData();
        TsmOrg tsmOrg;
        if (!ObjectUtils.isEmpty(tsmOrgList)) {
            //取第一个关联的直属组织（有多个，配置错误前台自己改）
            tsmOrg = tsmOrgList.get(0);
        } else {
            //没有找到所属区域的组织，使用默认团重庆市委
            Result<TsmOrg> orgResult = orgService.getOrgByOrgName("团重庆市委");
            if (ObjectUtils.isEmpty(orgResult)) {
                throw new BusinessException("默认的1级区域直属组织‘团重庆市委’没有找到");
            }
            tsmOrg = orgResult.getData();
        }

        if (ObjectUtils.isEmpty(tsmOrg)) {
            throw new BusinessException("暂无关联工单调度组织");
        }
        youthcallWorkOrder.setOrgId(BigInteger.valueOf(tsmOrg.getOrgId()));
        youthcallWorkOrder.setOrgName(tsmOrg.getOrgName());

        youthcallWorkOrder.setAppCode(appOrgVo.getAppCode());
        youthcallWorkOrder.setAppName(appOrgVo.getAppName());
        youthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.PENDING.getValue());
        // 判断来源
        if (StringUtils.isNotEmpty(ycqYouthcallWorkOrderInsertReq.getSource())) {
            youthcallWorkOrder.setSource(ycqYouthcallWorkOrderInsertReq.getSource());
        } else {
            youthcallWorkOrder.setSource("01"); //一般建议
        }
        if (currWorker != null) {
            youthcallWorkOrder.setProposerId(BigInteger.valueOf(currWorker.getLoginId()));
            youthcallWorkOrder.setProposerName(currWorker.getWorkerName());
            youthcallWorkOrder.setProposerMobile(currWorker.getMobile());
        } else {
            // 新增用户数据
//            WorkerC workerC = new WorkerC();
//            if (youthcallWorkOrder.getOrgId() == null) {
//                throw new BusinessException("当前工单类型没有组织处理");
//            }
//            workerC.setOrgId(youthcallWorkOrder.getOrgId().toString());
//            workerC.setMobile(youthcallWorkOrder.getPhone());
//            workerC.setWorkerName(youthcallWorkOrder.getPhone());
//            workerC.setLoginCode(youthcallWorkOrder.getPhone());
//            workerC.setState(IfFlagEnum.TRUE.code());
//            workerC.setRoleIds(Arrays.asList(UseRoleEnum.NORMAL.code()));
//            log.info("同步新增用户数据：{}", workerC);
//            Result<Integer> worker = sysService.createWorkerInfo(workerC);
//            if (worker == null || worker.getData() == null) {
//                throw new BusinessException("同步新增用户数据失败:" + worker.getMsg());
//            }
//            BigInteger loginId = new BigInteger(String.valueOf(worker.getData()));
//            youthcallWorkOrder.setProposerId(loginId);
            youthcallWorkOrder.setProposerName(youthcallWorkOrder.getPhone());
            youthcallWorkOrder.setProposerMobile(youthcallWorkOrder.getPhone());
        }
        youthcallWorkOrder.setProposedTime(curr);
        youthcallWorkOrder.setDelFlag(DelFlagEnum.UN_DELETE.getCode());

        //工单组织关联表增加数据
        BigInteger id = ycqYouthcallWorkOrderRepo.saveAndFlush(youthcallWorkOrder).getId();
        // 增加待处理流程
        YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = YcqYouthcallWorkOrderProcess.pendingProcess(youthcallWorkOrder);
        ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
//        if (!CollectionUtils.isEmpty(appOrgVo.getOrgList())) {
//            List<YcqYouthcallWorkOrderOrg> ycqYouthcallWorkOrderOrgList = new ArrayList<>();
//            appOrgVo.getOrgList().forEach(tsmOrg -> {
//                YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = new YcqYouthcallWorkOrderOrg();
//                ycqYouthcallWorkOrderOrg.setWorkOrderId(id);
//                ycqYouthcallWorkOrderOrg.setAppCode(appOrgVo.getAppCode());
//                ycqYouthcallWorkOrderOrg.setAppName(appOrgVo.getAppName());
//                ycqYouthcallWorkOrderOrg.setOrgId(BigInteger.valueOf(tsmOrg.getOrgId()));
//                ycqYouthcallWorkOrderOrg.setOrgName(tsmOrg.getOrgName());
//                ycqYouthcallWorkOrderOrg.setWorkOrderType(ycqYouthcallWorkOrderInsertReq.getWorkOrderType());
//                if (currWorker != null) {
//                    ycqYouthcallWorkOrderOrg.setCreatedId(currWorker.getLoginId());
//                    ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
//                }
//                ycqYouthcallWorkOrderOrg.setCreatedTime(curr);
//                ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
//                ycqYouthcallWorkOrderOrg.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
//                ycqYouthcallWorkOrderOrg.setOrgType(OrderOrgTypeEnum.ONE.getValue());
//                ycqYouthcallWorkOrderOrg.setShowType(OrderOrgShowTypeEnum.DEAL.getValue());
//                ycqYouthcallWorkOrderOrgList.add(ycqYouthcallWorkOrderOrg);
//            });
//            ycqYouthcallWorkOrderOrgRepo.saveAllAndFlush(ycqYouthcallWorkOrderOrgList);
//        } else {
//            log.info("暂无关联工单调度组织");
//        }
        YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = new YcqYouthcallWorkOrderOrg();
        ycqYouthcallWorkOrderOrg.setWorkOrderId(id);
        ycqYouthcallWorkOrderOrg.setAppCode(appOrgVo.getAppCode());
        ycqYouthcallWorkOrderOrg.setAppName(appOrgVo.getAppName());
        ycqYouthcallWorkOrderOrg.setOrgId(BigInteger.valueOf(tsmOrg.getOrgId()));
        ycqYouthcallWorkOrderOrg.setOrgName(tsmOrg.getOrgName());
        ycqYouthcallWorkOrderOrg.setWorkOrderType(ycqYouthcallWorkOrderInsertReq.getWorkOrderType());
        if (currWorker != null) {
            ycqYouthcallWorkOrderOrg.setCreatedId(currWorker.getLoginId());
            ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
        }
        ycqYouthcallWorkOrderOrg.setCreatedTime(curr);
        ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
        ycqYouthcallWorkOrderOrg.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
        ycqYouthcallWorkOrderOrg.setOrgType(OrderOrgTypeEnum.ONE.getValue());
        ycqYouthcallWorkOrderOrg.setShowType(OrderOrgShowTypeEnum.DEAL.getValue());
        ycqYouthcallWorkOrderOrgRepo.saveAndFlush(ycqYouthcallWorkOrderOrg);
//        int noCommit = 1 / 0;
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> dealWorkOrder(YcqYouthcallWorkOrderDealReq ycqYouthcallWorkOrderDealReq) {
        // 验证是否已处理过
        if (ycqYouthcallWorkOrderDealReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            if (ycqYouthcallWorkOrderProcessRepo.countByWorkOrderIdAndProcessStatusAndDelFlag(ycqYouthcallWorkOrderDealReq.getWorkOrderId(), WorkOrderStatusEnum.DONE.getValue(), DelFlagEnum.UN_DELETE.getCode()) > 0) {
                return Result.ofFail("该工单已处理过");
            } else {
                Date curr = new Date();
                // 获取当前登陆人
                Worker currWorker = CurrWorkerUtil.getCurrWorker();
                // 更新工单表状态
                YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderRepo.getOne(ycqYouthcallWorkOrderDealReq.getWorkOrderId());
                ycqYouthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.DONE.getValue());
                if (currWorker != null) {
                    ycqYouthcallWorkOrder.setProcessUserId(BigInteger.valueOf(currWorker.getLoginId()));
                    ycqYouthcallWorkOrder.setProcessTime(curr);
                    ycqYouthcallWorkOrder.setProcessUserName(currWorker.getWorkerName());
                    ycqYouthcallWorkOrder.setProcessMobile(currWorker.getMobile());
                }
                ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
                // 工单过程表增加处理数据
                String attachment = "";
                if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderDealReq.getAttachmentList())) {
                    attachment = String.join(",", ycqYouthcallWorkOrderDealReq.getAttachmentList());
                } else {
                    log.info("该工单回复无附件");
                }
                YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = YcqYouthcallWorkOrderProcess.dealProcess(ycqYouthcallWorkOrderDealReq.getWorkOrderId(), ycqYouthcallWorkOrderDealReq.getDealContent(), attachment, currWorker);
                ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
            }
        }
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> escalationWorkOrder(YcqYouthcallWorkOrderEscalationReq ycqYouthcallWorkOrderEscalationReq) {
        // 判断参数异常
        if (ycqYouthcallWorkOrderEscalationReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            // 验证是否已处理、已评价、已回访、已归档
            YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderRepo.getOne(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId());
            if (WorkOrderStatusEnum.DONE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.FOLLOW_UP.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.FILE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.EVALUATE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                return Result.ofFail("该工单已处理,无需上报");
            }
            // 上报对象为上级组织
            if (ReportTypeEnum.ORG.getValue().equals(ycqYouthcallWorkOrderEscalationReq.getReportType())) {
                // 验证是否已上报过此组织
                if (ycqYouthcallWorkOrderProcessRepo.countByWorkOrderIdAndTransferOrgIdAndProcessStatusAndDelFlag(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), ycqYouthcallWorkOrderEscalationReq.getTransferOrgId(), WorkOrderStatusEnum.PENDING_ESCALATION.getValue(), DelFlagEnum.UN_DELETE.getCode()) > 0) {
                    return Result.ofFail("该工单已上报过该组织,请勿重复上报");
                }
            } else {
                // 上报对象为 综治系统、民呼我为，验证是否已上报过
                if (ycqYouthcallWorkOrderProcessRepo.countByWorkOrderIdAndProcessStatusAndReportTypeAndDelFlag(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), WorkOrderStatusEnum.PENDING_ESCALATION.getValue(), ycqYouthcallWorkOrderEscalationReq.getReportType(), DelFlagEnum.UN_DELETE.getCode()) > 0) {
                    return Result.ofFail("该工单已上报,请勿重复上报");
                }
            }
            Date curr = new Date();
            // 获取当前登陆人
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            // 更新工单表状态
            ycqYouthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.PENDING_ESCALATION.getValue());
            ycqYouthcallWorkOrder.setOrgId(ycqYouthcallWorkOrderEscalationReq.getTransferOrgId());
            ycqYouthcallWorkOrder.setOrgName(ycqYouthcallWorkOrderEscalationReq.getTransferOrgName());
            ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
            // 工单过程表增加处理数据
            YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = YcqYouthcallWorkOrderProcess.escalationProcess(ycqYouthcallWorkOrderEscalationReq, currWorker);
            ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
            // 工单组织表增加关联组织(上报)
            if (ReportTypeEnum.ORG.getValue().equals(ycqYouthcallWorkOrderEscalationReq.getReportType())) {
                // a转给b a可撤回 b可处理
                // a转给b b转给c  a可查看 b可撤回 c可处理
                ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), OrderOrgShowTypeEnum.REVOKE.getValue(), OrderOrgShowTypeEnum.LOOK.getValue());
                if (ycqYouthcallWorkOrderDao.isSuperAdmin()) {
                    if (ycqYouthcallWorkOrderEscalationReq.getTransferOrgId().toString().equals(currWorker.getOrgId())) {
                        throw new BusinessException("工单不能上报给当前组织");
                    }
                    // 工单组织表存放的是当前有权限处理工单的组织，如果是管理员 需要在新加一条数据
                    List<YcqYouthcallWorkOrderOrg> byWorkOrderIdAndShowType = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndShowType(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue());
                    List<YcqYouthcallWorkOrderOrg> adminWorkOrder = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndOrgId(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), ycqYouthcallWorkOrderEscalationReq.getTransferOrgId());
                    if (CollectionUtils.isEmpty(byWorkOrderIdAndShowType)) {
                        throw new BusinessException("当前没有可处理的组织");
                    } else if (CollectionUtils.isEmpty(adminWorkOrder)) {
                        YcqYouthcallWorkOrderOrg old = byWorkOrderIdAndShowType.get(0);
                        YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = new YcqYouthcallWorkOrderOrg();
                        CopyUtil.copyPropertiesNonNull(old, ycqYouthcallWorkOrderOrg);
                        ycqYouthcallWorkOrderOrg.setId(null);
                        ycqYouthcallWorkOrderOrg.setOrgId(new BigInteger(currWorker.getOrgId()));
                        ycqYouthcallWorkOrderOrg.setOrgName(currWorker.getOrgName());
                        ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
                        ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
                        ycqYouthcallWorkOrderOrg.setShowType(OrderOrgShowTypeEnum.REVOKE.getValue());
                        ycqYouthcallWorkOrderOrgRepo.saveAndFlush(ycqYouthcallWorkOrderOrg);
                        ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgShowTypeEnum.LOOK.getValue());
                    }
                } else {
                    ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgShowTypeEnum.REVOKE.getValue());
                }
                YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = new YcqYouthcallWorkOrderOrg();
                // 已经存在记录了
                List<YcqYouthcallWorkOrderOrg> byWorkOrderIdAndOrgId = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndOrgId(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), ycqYouthcallWorkOrderEscalationReq.getTransferOrgId());
                if (!CollectionUtils.isEmpty(byWorkOrderIdAndOrgId)) {
                    ycqYouthcallWorkOrderOrgRepo.modifyShowTypeAndOrgTypeByOrgIdAndWorkOrderId(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), ycqYouthcallWorkOrderEscalationReq.getTransferOrgId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgTypeEnum.TWO.getValue());
                } else {
                    ycqYouthcallWorkOrderOrg.setWorkOrderId(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId());
                    ycqYouthcallWorkOrderOrg.setAppCode(ycqYouthcallWorkOrder.getAppCode());
                    ycqYouthcallWorkOrderOrg.setAppName(ycqYouthcallWorkOrder.getAppName());
                    ycqYouthcallWorkOrderOrg.setOrgId(ycqYouthcallWorkOrderEscalationReq.getTransferOrgId());
                    ycqYouthcallWorkOrderOrg.setOrgName(ycqYouthcallWorkOrderEscalationReq.getTransferOrgName());
                    ycqYouthcallWorkOrderOrg.setWorkOrderType(ycqYouthcallWorkOrder.getWorkOrderType());
                    ycqYouthcallWorkOrderOrg.setOrgType(OrderOrgTypeEnum.TWO.getValue());
                    if (currWorker != null) {
                        ycqYouthcallWorkOrderOrg.setCreatedId(currWorker.getLoginId());
                        ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
                    }
                    ycqYouthcallWorkOrderOrg.setCreatedTime(curr);
                    ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
                    ycqYouthcallWorkOrderOrg.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
                    ycqYouthcallWorkOrderOrg.setShowType(OrderOrgShowTypeEnum.DEAL.getValue());
                    ycqYouthcallWorkOrderOrgRepo.saveAndFlush(ycqYouthcallWorkOrderOrg);
                }
            }
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Object> revokeEscalationWorkOrder(YcqYouthcallWorkOrderEscalationReq ycqYouthcallWorkOrderEscalationReq) {
        // 判断参数异常
        if (ycqYouthcallWorkOrderEscalationReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            // 验证是否待处理-已上报
            YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderRepo.getOne(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId());
            if (WorkOrderStatusEnum.PENDING_ESCALATION.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                Date curr = new Date();
                // 获取当前登陆人
                Worker currWorker = CurrWorkerUtil.getCurrWorker();
                // 更新工单表状态
                ycqYouthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.PENDING.getValue());
                ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
                // 工单过程表删除数据
                List<YcqYouthcallWorkOrderProcess> ycqYouthcallWorkOrderProcessList = ycqYouthcallWorkOrderProcessRepo.findByWorkOrderIdAndProcessStatusAndDelFlag(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), WorkOrderStatusEnum.PENDING_ESCALATION.getValue(), DelFlagEnum.UN_DELETE.getCode());
                // 判断过程表是否存在上报数据
                if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderProcessList)) {
                    YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = ycqYouthcallWorkOrderProcessList.get(0);
                    ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.DELETED.getCode());
                    if (currWorker != null) {
                        ycqYouthcallWorkOrderProcess.setLastUpdatedId(currWorker.getLoginId());
                    }
                    ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
                    ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
                }

                // 工单组织表删除关联组织(上报)
                List<YcqYouthcallWorkOrderOrg> ycqYouthcallWorkOrderOrgList = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndOrgTypeAndShowTypeAndDelFlag(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), OrderOrgTypeEnum.TWO.getValue(), OrderOrgShowTypeEnum.DEAL.getValue(), DelFlagEnum.UN_DELETE.getCode());
                // 可撤回的加入工单表
                List<YcqYouthcallWorkOrderOrg> byWorkOrderIdAndShowTypeAndDelFlag = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndShowTypeAndDelFlag(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), OrderOrgShowTypeEnum.REVOKE.getValue(), DelFlagEnum.UN_DELETE.getCode());
                List<String> orgNameList = byWorkOrderIdAndShowTypeAndDelFlag.stream().map(YcqYouthcallWorkOrderOrg::getOrgName).collect(Collectors.toList());
                String nameJoin = StringUtils.join(orgNameList, ",");
                ycqYouthcallWorkOrder.setOrgId(byWorkOrderIdAndShowTypeAndDelFlag.get(0).getOrgId());
                ycqYouthcallWorkOrder.setOrgName(nameJoin);
                ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
                ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgShowTypeEnum.LOOK.getValue());
                ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderEscalationReq.getWorkOrderId(), OrderOrgShowTypeEnum.REVOKE.getValue(), OrderOrgShowTypeEnum.DEAL.getValue());
                // 判断工单组织表是否存在上报数据
                if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderOrgList)) {
                    YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = ycqYouthcallWorkOrderOrgList.get(0);
                    ycqYouthcallWorkOrderOrg.setDelFlag(DelFlagEnum.DELETED.getCode());
                    if (currWorker != null) {
                        ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
                    }
                    ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
                    ycqYouthcallWorkOrderOrgRepo.saveAndFlush(ycqYouthcallWorkOrderOrg);
                }
            } else {
                return Result.ofFail("当前状态不为已上报状态，无法撤销");
            }
        }
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> allocationWorkOrder(YcqYouthcallWorkOrderAllocationReq ycqYouthcallWorkOrderAllocationReq) {
        // 判断参数异常
        if (ycqYouthcallWorkOrderAllocationReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            // 验证是否已处理、已评价、已回访、已归档
            YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderRepo.getOne(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId());
            if (WorkOrderStatusEnum.DONE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.FOLLOW_UP.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.FILE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.EVALUATE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                return Result.ofFail("该工单已处理,无需下派");
            }
            // 验证是否已下派过此组织
            if (ycqYouthcallWorkOrderProcessRepo.countByWorkOrderIdAndTransferOrgIdAndProcessStatusAndDelFlag(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), ycqYouthcallWorkOrderAllocationReq.getTransferOrgId(), WorkOrderStatusEnum.PENDING_ALLOCATION.getValue(), DelFlagEnum.UN_DELETE.getCode()) > 0) {
                return Result.ofFail("该工单已下派过该组织,请勿重复下派");
            }
            Date curr = new Date();
            // 获取当前登陆人
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            // 更新工单表状态
            ycqYouthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.PENDING_ALLOCATION.getValue());
            ycqYouthcallWorkOrder.setOrgId(ycqYouthcallWorkOrderAllocationReq.getTransferOrgId());
            ycqYouthcallWorkOrder.setOrgName(ycqYouthcallWorkOrderAllocationReq.getTransferOrgName());
            ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
            // 工单过程表增加处理数据
            YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = YcqYouthcallWorkOrderProcess.allocationProcess(ycqYouthcallWorkOrderAllocationReq, currWorker);
            ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
            // a转给b a可撤回 b可处理
            // a转给b b转给c  a可查看 b可撤回 c可处理
            ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), OrderOrgShowTypeEnum.REVOKE.getValue(), OrderOrgShowTypeEnum.LOOK.getValue());
            if (ycqYouthcallWorkOrderDao.isSuperAdmin()) {
                // 工单组织表存放的是当前有权限处理工单的组织，如果是管理员 需要在新加一条数据
                List<YcqYouthcallWorkOrderOrg> byWorkOrderIdAndShowType = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndShowType(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue());
                List<YcqYouthcallWorkOrderOrg> adminWorkOrder = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndOrgId(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), ycqYouthcallWorkOrderAllocationReq.getTransferOrgId());
                if (CollectionUtils.isEmpty(byWorkOrderIdAndShowType)) {
                    throw new BusinessException("当前没有可处理的组织");
                } else if (CollectionUtils.isEmpty(adminWorkOrder)) {
                    YcqYouthcallWorkOrderOrg old = byWorkOrderIdAndShowType.get(0);
                    YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = new YcqYouthcallWorkOrderOrg();
                    CopyUtil.copyPropertiesNonNull(old, ycqYouthcallWorkOrderOrg);
                    ycqYouthcallWorkOrderOrg.setId(null);
                    ycqYouthcallWorkOrderOrg.setOrgId(new BigInteger(currWorker.getOrgId()));
                    ycqYouthcallWorkOrderOrg.setOrgName(currWorker.getOrgName());
                    ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
                    ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
                    ycqYouthcallWorkOrderOrg.setShowType(OrderOrgShowTypeEnum.REVOKE.getValue());
                    ycqYouthcallWorkOrderOrgRepo.saveAndFlush(ycqYouthcallWorkOrderOrg);
                    ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgShowTypeEnum.LOOK.getValue());
                }
            } else {
                ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgShowTypeEnum.REVOKE.getValue());
            }

            // 工单组织表增加关联组织(下派)
            YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = new YcqYouthcallWorkOrderOrg();
            // 已经存在记录了
            List<YcqYouthcallWorkOrderOrg> byWorkOrderIdAndOrgId = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndOrgId(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), ycqYouthcallWorkOrderAllocationReq.getTransferOrgId());
            if (!CollectionUtils.isEmpty(byWorkOrderIdAndOrgId)) {
                ycqYouthcallWorkOrderOrgRepo.modifyShowTypeAndOrgTypeByOrgIdAndWorkOrderId(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), ycqYouthcallWorkOrderAllocationReq.getTransferOrgId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgTypeEnum.THREE.getValue());
            } else {
                ycqYouthcallWorkOrderOrg.setWorkOrderId(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId());
                ycqYouthcallWorkOrderOrg.setAppCode(ycqYouthcallWorkOrder.getAppCode());
                ycqYouthcallWorkOrderOrg.setAppName(ycqYouthcallWorkOrder.getAppName());
                ycqYouthcallWorkOrderOrg.setOrgId(ycqYouthcallWorkOrderAllocationReq.getTransferOrgId());
                ycqYouthcallWorkOrderOrg.setOrgName(ycqYouthcallWorkOrderAllocationReq.getTransferOrgName());
                ycqYouthcallWorkOrderOrg.setWorkOrderType(ycqYouthcallWorkOrder.getWorkOrderType());
                ycqYouthcallWorkOrderOrg.setOrgType(OrderOrgTypeEnum.THREE.getValue());
                if (currWorker != null) {
                    ycqYouthcallWorkOrderOrg.setCreatedId(currWorker.getLoginId());
                    ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
                }
                ycqYouthcallWorkOrderOrg.setCreatedTime(curr);
                ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
                ycqYouthcallWorkOrderOrg.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
                ycqYouthcallWorkOrderOrg.setShowType(OrderOrgShowTypeEnum.DEAL.getValue());
                ycqYouthcallWorkOrderOrgRepo.saveAndFlush(ycqYouthcallWorkOrderOrg);
            }
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Object> revokeAllocationWorkOrder(YcqYouthcallWorkOrderAllocationReq ycqYouthcallWorkOrderAllocationReq) {
        // 判断参数异常
        if (ycqYouthcallWorkOrderAllocationReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            // 验证是否待处理-已下派
            YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderRepo.getOne(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId());
            if (WorkOrderStatusEnum.PENDING_ALLOCATION.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                Date curr = new Date();
                // 获取当前登陆人
                Worker currWorker = CurrWorkerUtil.getCurrWorker();
                // 更新工单表状态
                ycqYouthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.PENDING.getValue());
                ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
                // 工单过程表删除数据
                List<YcqYouthcallWorkOrderProcess> ycqYouthcallWorkOrderProcessList = ycqYouthcallWorkOrderProcessRepo.findByWorkOrderIdAndProcessStatusAndDelFlag(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), WorkOrderStatusEnum.PENDING_ALLOCATION.getValue(), DelFlagEnum.UN_DELETE.getCode());
                // 判断过程表是否存在下派数据
                if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderProcessList)) {
                    YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = ycqYouthcallWorkOrderProcessList.get(0);
                    ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.DELETED.getCode());
                    if (currWorker != null) {
                        ycqYouthcallWorkOrderProcess.setLastUpdatedId(currWorker.getLoginId());
                    }
                    ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
                    ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
                }

                // 工单组织表删除关联组织(下派)
                List<YcqYouthcallWorkOrderOrg> ycqYouthcallWorkOrderOrgList = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndOrgTypeAndShowTypeAndDelFlag(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), OrderOrgTypeEnum.THREE.getValue(), OrderOrgShowTypeEnum.DEAL.getValue(), DelFlagEnum.UN_DELETE.getCode());
                // 可撤回的加入工单表
                List<YcqYouthcallWorkOrderOrg> byWorkOrderIdAndShowTypeAndDelFlag = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndShowTypeAndDelFlag(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), OrderOrgShowTypeEnum.REVOKE.getValue(), DelFlagEnum.UN_DELETE.getCode());
                List<String> orgNameList = byWorkOrderIdAndShowTypeAndDelFlag.stream().map(YcqYouthcallWorkOrderOrg::getOrgName).collect(Collectors.toList());
                String nameJoin = StringUtils.join(orgNameList, ",");
                ycqYouthcallWorkOrder.setOrgId(byWorkOrderIdAndShowTypeAndDelFlag.get(0).getOrgId());
                ycqYouthcallWorkOrder.setOrgName(nameJoin);
                ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
                // 可撤回的变为可操作
                ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgShowTypeEnum.LOOK.getValue());
                ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderAllocationReq.getWorkOrderId(), OrderOrgShowTypeEnum.REVOKE.getValue(), OrderOrgShowTypeEnum.DEAL.getValue());

                // 判断工单组织表是否存在下派数据
                if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderOrgList)) {
                    YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = ycqYouthcallWorkOrderOrgList.get(0);
                    ycqYouthcallWorkOrderOrg.setDelFlag(DelFlagEnum.DELETED.getCode());
                    if (currWorker != null) {
                        ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
                    }
                    ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
                    ycqYouthcallWorkOrderOrgRepo.saveAndFlush(ycqYouthcallWorkOrderOrg);
                }
            } else {
                return Result.ofFail("当前状态不为已下派状态，无法撤销");
            }
        }
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> peerTransferWorkOrder(YcqYouthcallWorkOrderPeerTransferReq ycqYouthcallWorkOrderPeerTransferReq) {
        // 判断参数异常
        if (ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            // 验证是否已处理、已评价、已回访、已归档
            YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderRepo.getOne(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId());
            if (WorkOrderStatusEnum.DONE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.FOLLOW_UP.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.FILE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.EVALUATE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                return Result.ofFail("该工单已处理,无需同级转派");
            }
            // 验证是否已同级转派过此组织
            if (ycqYouthcallWorkOrderProcessRepo.countByWorkOrderIdAndTransferOrgIdAndProcessStatusAndDelFlag(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), ycqYouthcallWorkOrderPeerTransferReq.getTransferOrgId(), WorkOrderStatusEnum.PENDING_PEER_TRANSFER.getValue(), DelFlagEnum.UN_DELETE.getCode()) > 0) {
                return Result.ofFail("该工单已同级转派过该组织,请勿重复同级转派");
            }
            Date curr = new Date();
            // 获取当前登陆人
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            // 更新工单表状态
            ycqYouthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.PENDING_PEER_TRANSFER.getValue());
            ycqYouthcallWorkOrder.setOrgId(ycqYouthcallWorkOrderPeerTransferReq.getTransferOrgId());
            ycqYouthcallWorkOrder.setOrgName(ycqYouthcallWorkOrderPeerTransferReq.getTransferOrgName());
            ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
            // 工单过程表增加处理数据
            YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = YcqYouthcallWorkOrderProcess.peerTransferProcess(ycqYouthcallWorkOrderPeerTransferReq, currWorker);
            ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
            // a转给b a可撤回 b可处理
            // a转给b b转给c  a可查看 b可撤回 c可处理
            ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), OrderOrgShowTypeEnum.REVOKE.getValue(), OrderOrgShowTypeEnum.LOOK.getValue());
            if (ycqYouthcallWorkOrderDao.isSuperAdmin()) {
                // 工单组织表存放的是当前有权限处理工单的组织，如果是管理员 需要在新加一条数据
                List<YcqYouthcallWorkOrderOrg> byWorkOrderIdAndShowType = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndShowType(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue());
                List<YcqYouthcallWorkOrderOrg> adminWorkOrder = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndOrgId(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), ycqYouthcallWorkOrderPeerTransferReq.getTransferOrgId());
                if (CollectionUtils.isEmpty(byWorkOrderIdAndShowType)) {
                    throw new BusinessException("当前没有可处理的组织");
                } else if (CollectionUtils.isEmpty(adminWorkOrder)) {
                    YcqYouthcallWorkOrderOrg old = byWorkOrderIdAndShowType.get(0);
                    YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = new YcqYouthcallWorkOrderOrg();
                    CopyUtil.copyPropertiesNonNull(old, ycqYouthcallWorkOrderOrg);
                    ycqYouthcallWorkOrderOrg.setId(null);
                    ycqYouthcallWorkOrderOrg.setOrgId(new BigInteger(currWorker.getOrgId()));
                    ycqYouthcallWorkOrderOrg.setOrgName(currWorker.getOrgName());
                    ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
                    ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
                    ycqYouthcallWorkOrderOrg.setShowType(OrderOrgShowTypeEnum.REVOKE.getValue());
                    ycqYouthcallWorkOrderOrgRepo.saveAndFlush(ycqYouthcallWorkOrderOrg);
                    ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgShowTypeEnum.LOOK.getValue());
                }
            } else {
                ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgShowTypeEnum.REVOKE.getValue());
            }
            // 工单组织表增加关联组织(同级转派)
            YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = new YcqYouthcallWorkOrderOrg();
            // 已经存在记录了
            List<YcqYouthcallWorkOrderOrg> byWorkOrderIdAndOrgId = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndOrgId(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), ycqYouthcallWorkOrderPeerTransferReq.getTransferOrgId());
            if (!CollectionUtils.isEmpty(byWorkOrderIdAndOrgId)) {
                ycqYouthcallWorkOrderOrgRepo.modifyShowTypeAndOrgTypeByOrgIdAndWorkOrderId(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), ycqYouthcallWorkOrderPeerTransferReq.getTransferOrgId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgTypeEnum.FIVE.getValue());
            } else {
                ycqYouthcallWorkOrderOrg.setWorkOrderId(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId());
                ycqYouthcallWorkOrderOrg.setAppCode(ycqYouthcallWorkOrder.getAppCode());
                ycqYouthcallWorkOrderOrg.setAppName(ycqYouthcallWorkOrder.getAppName());
                ycqYouthcallWorkOrderOrg.setOrgId(ycqYouthcallWorkOrderPeerTransferReq.getTransferOrgId());
                ycqYouthcallWorkOrderOrg.setOrgName(ycqYouthcallWorkOrderPeerTransferReq.getTransferOrgName());
                ycqYouthcallWorkOrderOrg.setWorkOrderType(ycqYouthcallWorkOrder.getWorkOrderType());
                ycqYouthcallWorkOrderOrg.setOrgType(OrderOrgTypeEnum.FIVE.getValue());
                if (currWorker != null) {
                    ycqYouthcallWorkOrderOrg.setCreatedId(currWorker.getLoginId());
                    ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
                }
                ycqYouthcallWorkOrderOrg.setCreatedTime(curr);
                ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
                ycqYouthcallWorkOrderOrg.setDelFlag(DelFlagEnum.UN_DELETE.getCode());
                ycqYouthcallWorkOrderOrg.setShowType(OrderOrgShowTypeEnum.DEAL.getValue());
                ycqYouthcallWorkOrderOrgRepo.saveAndFlush(ycqYouthcallWorkOrderOrg);
            }
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Object> revokePeerTransferWorkOrder(YcqYouthcallWorkOrderPeerTransferReq ycqYouthcallWorkOrderPeerTransferReq) {
        // 判断参数异常
        if (ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            // 验证是否待处理-已同级转派
            YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderRepo.getOne(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId());
            if (WorkOrderStatusEnum.PENDING_PEER_TRANSFER.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                Date curr = new Date();
                // 获取当前登陆人
                Worker currWorker = CurrWorkerUtil.getCurrWorker();
                // 更新工单表状态
                ycqYouthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.PENDING.getValue());
                ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
                // 工单过程表删除数据
                List<YcqYouthcallWorkOrderProcess> ycqYouthcallWorkOrderProcessList = ycqYouthcallWorkOrderProcessRepo.findByWorkOrderIdAndProcessStatusAndDelFlag(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), WorkOrderStatusEnum.PENDING_PEER_TRANSFER.getValue(), DelFlagEnum.UN_DELETE.getCode());
                // 判断过程表是否存在同级转派数据
                if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderProcessList)) {
                    YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = ycqYouthcallWorkOrderProcessList.get(0);
                    ycqYouthcallWorkOrderProcess.setDelFlag(DelFlagEnum.DELETED.getCode());
                    if (currWorker != null) {
                        ycqYouthcallWorkOrderProcess.setLastUpdatedId(currWorker.getLoginId());
                    }
                    ycqYouthcallWorkOrderProcess.setLastUpdatedTime(curr);
                    ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
                }

                // 工单组织表删除关联组织(同级转派)
                List<YcqYouthcallWorkOrderOrg> ycqYouthcallWorkOrderOrgList = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndOrgTypeAndShowTypeAndDelFlag(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), OrderOrgTypeEnum.FIVE.getValue(), OrderOrgShowTypeEnum.DEAL.getValue(), DelFlagEnum.UN_DELETE.getCode());
                // 可撤回的加入工单表
                List<YcqYouthcallWorkOrderOrg> byWorkOrderIdAndShowTypeAndDelFlag = ycqYouthcallWorkOrderOrgRepo.findByWorkOrderIdAndShowTypeAndDelFlag(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), OrderOrgShowTypeEnum.REVOKE.getValue(), DelFlagEnum.UN_DELETE.getCode());
                List<String> orgNameList = byWorkOrderIdAndShowTypeAndDelFlag.stream().map(YcqYouthcallWorkOrderOrg::getOrgName).collect(Collectors.toList());
                String nameJoin = StringUtils.join(orgNameList, ",");
                ycqYouthcallWorkOrder.setOrgId(byWorkOrderIdAndShowTypeAndDelFlag.get(0).getOrgId());
                ycqYouthcallWorkOrder.setOrgName(nameJoin);
                ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
                ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), OrderOrgShowTypeEnum.DEAL.getValue(), OrderOrgShowTypeEnum.LOOK.getValue());
                ycqYouthcallWorkOrderOrgRepo.modifyShowType(ycqYouthcallWorkOrderPeerTransferReq.getWorkOrderId(), OrderOrgShowTypeEnum.REVOKE.getValue(), OrderOrgShowTypeEnum.DEAL.getValue());
                // 判断工单组织表是否存在同级转派数据
                if (!CollectionUtils.isEmpty(ycqYouthcallWorkOrderOrgList)) {
                    YcqYouthcallWorkOrderOrg ycqYouthcallWorkOrderOrg = ycqYouthcallWorkOrderOrgList.get(0);
                    ycqYouthcallWorkOrderOrg.setDelFlag(DelFlagEnum.DELETED.getCode());
                    if (currWorker != null) {
                        ycqYouthcallWorkOrderOrg.setLastUpdatedId(currWorker.getLoginId());
                    }
                    ycqYouthcallWorkOrderOrg.setLastUpdatedTime(curr);
                    ycqYouthcallWorkOrderOrgRepo.saveAndFlush(ycqYouthcallWorkOrderOrg);
                }
            } else {
                return Result.ofFail("当前状态不为已同级转派状态，无法撤销");
            }
        }
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> followUpWorkOrder(YcqYouthcallWorkOrderFollowUpReq ycqYouthcallWorkOrderFollowUpReq) {
        // 判断参数异常
        if (ycqYouthcallWorkOrderFollowUpReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderRepo.getOne(ycqYouthcallWorkOrderFollowUpReq.getWorkOrderId());
            // 验证是否已归档
            if (WorkOrderStatusEnum.FILE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                return Result.ofFail("该工单已归档,无需回访");
            } else if (WorkOrderStatusEnum.FOLLOW_UP.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                // 验证是否已回访
                return Result.ofFail("该工单已回访,请勿重复回访");
            } else if (WorkOrderStatusEnum.DONE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                Date curr = new Date();
                // 获取当前登陆人
                Worker currWorker = CurrWorkerUtil.getCurrWorker();
                // 更新工单表状态
                ycqYouthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.FOLLOW_UP.getValue());
                /*if (currWorker != null) {
                    ycqYouthcallWorkOrder.setProcessUserId(BigInteger.valueOf(currWorker.getLoginId()));
                    ycqYouthcallWorkOrder.setProcessTime(curr);
                    ycqYouthcallWorkOrder.setProcessUserName(currWorker.getWorkerName());
                    ycqYouthcallWorkOrder.setProcessMobile(currWorker.getMobile());
                }*/
                ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
                // 工单过程表增加处理数据
                YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = YcqYouthcallWorkOrderProcess.followUpProcess(ycqYouthcallWorkOrderFollowUpReq, currWorker);
                ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
            } else {
                // 验证是否未处理
                return Result.ofFail("该工单未处理，暂时无法回访");
            }

        }
        return Result.ofSuccess(true);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> fileWorkOrder(YcqYouthcallWorkOrderFileReq ycqYouthcallWorkOrderFileReq) {
        // 判断参数异常
        if (ycqYouthcallWorkOrderFileReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderRepo.getOne(ycqYouthcallWorkOrderFileReq.getWorkOrderId());
            // 验证是否已归档
            if (WorkOrderStatusEnum.FILE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                return Result.ofFail("该工单已归档,请勿重复归档");
            } else if (WorkOrderStatusEnum.FOLLOW_UP.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                Date curr = new Date();
                // 获取当前登陆人
                Worker currWorker = CurrWorkerUtil.getCurrWorker();
                // 更新工单表状态
                ycqYouthcallWorkOrder.setProcessStatus(WorkOrderStatusEnum.FILE.getValue());
                /*if (currWorker != null) {
                    ycqYouthcallWorkOrder.setProcessUserId(BigInteger.valueOf(currWorker.getLoginId()));
                    ycqYouthcallWorkOrder.setProcessTime(curr);
                    ycqYouthcallWorkOrder.setProcessUserName(currWorker.getWorkerName());
                    ycqYouthcallWorkOrder.setProcessMobile(currWorker.getMobile());
                }*/
                ycqYouthcallWorkOrderRepo.saveAndFlush(ycqYouthcallWorkOrder);
                // 工单过程表增加处理数据
                YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = YcqYouthcallWorkOrderProcess.fileProcess(ycqYouthcallWorkOrderFileReq, currWorker);
                ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
            } else {
                return Result.ofFail("该工单未回访,暂时无法归档");
            }
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Object> evaluateWorkOrder(YcqYouthcallWorkOrderEvaluateReq ycqYouthcallWorkOrderEvaluateReq) {
        // 判断参数异常
        if (ycqYouthcallWorkOrderEvaluateReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            YcqYouthcallWorkOrder ycqYouthcallWorkOrder = ycqYouthcallWorkOrderRepo.getOne(ycqYouthcallWorkOrderEvaluateReq.getWorkOrderId());
            // 验证是否已处理、已回访、已归档
            if (WorkOrderStatusEnum.DONE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.FOLLOW_UP.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus()) ||
                    WorkOrderStatusEnum.FILE.getValue().equals(ycqYouthcallWorkOrder.getProcessStatus())) {
                // 验证是否已评价
                if (ycqYouthcallWorkOrderProcessRepo.countByWorkOrderIdAndProcessStatusAndDelFlag(ycqYouthcallWorkOrderEvaluateReq.getWorkOrderId(), WorkOrderStatusEnum.EVALUATE.getValue(), DelFlagEnum.UN_DELETE.getCode()) > 0) {
                    return Result.ofFail("该工单已评价,请勿重复评价");
                } else {
                    Date curr = new Date();
                    // 获取当前登陆人
                    Worker currWorker = CurrWorkerUtil.getCurrWorker();
                    // 工单过程表增加处理数据
                    YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = YcqYouthcallWorkOrderProcess.evaluateProcess(ycqYouthcallWorkOrderEvaluateReq, currWorker);
                    ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
                }
            } else {
                return Result.ofFail("该工单未处理,无法评价");
            }
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Object> replyWorkOrder(YcqYouthcallWorkOrderReplyReq ycqYouthcallWorkOrderReplyReq) {
        // 判断参数异常
        if (ycqYouthcallWorkOrderReplyReq.getWorkOrderId() == null) {
            return Result.ofFail("工单id异常");
        } else {
            // 获取当前登陆人
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            // 工单过程表增加处理数据
            YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess = YcqYouthcallWorkOrderProcess.replyProcess(ycqYouthcallWorkOrderReplyReq, currWorker);
            ycqYouthcallWorkOrderProcessRepo.saveAndFlush(ycqYouthcallWorkOrderProcess);
        }
        return Result.ofSuccess(true);
    }

    @Override
    public Result<Object> queryDetailById(BigInteger id) {
        // 判断参数异常
        if (id == null) {
            return Result.ofFail("工单id异常");
        } else {
            // 查询基本信息
            PageReq<YcqYouthcallWorkOrderVo> ycqYouthcallWorkOrderPageReq = new PageReq<>();
            YcqYouthcallWorkOrderVo order = new YcqYouthcallWorkOrderVo();
            order.setId(id);
            order.setOrgId(new BigInteger(CurrWorkerUtil.getCurrWorker().getOrgId()));
            ycqYouthcallWorkOrderPageReq.setQuery(order);
            Worker currWorker = CurrWorkerUtil.getCurrWorker();
            Boolean isSuperAdmin = ycqYouthcallWorkOrderDao.isSuperAdmin();
            Boolean hasAuth = ycqYouthcallWorkOrderDao.hasAuth(currWorker.getLoginId(), WORKER_ORDER_ADMIN, currWorker.getOrgId());
            if (!isSuperAdmin && ! hasAuth){
                throw new BusinessException("没有青呼我为工单管理员权限");
            }
            PageRsp<WorkOrderBaseVo> rsp = ycqYouthcallWorkOrderDao.paginQuery(ycqYouthcallWorkOrderPageReq);
            if (CollectionUtils.isEmpty(rsp.getResult())) {
                return Result.ofFail("暂无工单信息");
            } else {
                WorkOrderDetailVo workOrderDetailVo = getWorkOrderDetailVo(id, rsp);
                return Result.ofSuccess(workOrderDetailVo);
            }
        }
    }
    @Override
    public Result<Object> queryDetailAppById(BigInteger id) {
        // 判断参数异常
        if (id == null) {
            return Result.ofFail("工单id异常");
        } else {
            // 查询基本信息
            PageReq<YcqYouthcallWorkOrderVo> ycqYouthcallWorkOrderPageReq = new PageReq<>();
            YcqYouthcallWorkOrderVo order = new YcqYouthcallWorkOrderVo();
            order.setId(id);
            ycqYouthcallWorkOrderPageReq.setQuery(order);
            PageRsp<WorkOrderBaseVo> rsp = ycqYouthcallWorkOrderDao.paginQuery(ycqYouthcallWorkOrderPageReq);
            if (CollectionUtils.isEmpty(rsp.getResult())) {
                return Result.ofFail("暂无工单信息");
            } else {
                WorkOrderDetailVo workOrderDetailVo = getWorkOrderDetailVo(id, rsp);
                return Result.ofSuccess(workOrderDetailVo);
            }
        }
    }

    private WorkOrderDetailVo getWorkOrderDetailVo(BigInteger id, PageRsp<WorkOrderBaseVo> rsp) {
        WorkOrderDetailVo workOrderDetailVo = new WorkOrderDetailVo();
        List<YcqYouthcallWorkOrderProcess> ycqYouthcallWorkOrderProcessList = ycqYouthcallWorkOrderProcessRepo.findByWorkOrderIdAndDelFlagOrderByCreatedTimeAsc(id, DelFlagEnum.UN_DELETE.getCode());
        List<WorkOrderDealVo> workOrderDealVoList = new ArrayList<>();
        List<WorkOrderEscalationVo> workOrderEscalationVoList = new ArrayList<>();
        List<WorkOrderAllocationVo> workOrderAllocationVoList = new ArrayList<>();
        List<WorkOrderEvaluateVo> workOrderEvaluateVoList = new ArrayList<>();
        List<WorkOrderFollowUpVo> workOrderFollowUpVoList = new ArrayList<>();
        List<WorkOrderFileVo> workOrderFileVoList = new ArrayList<>();
        List<WorkOrderReplyVo> workOrderReplyVoList = new ArrayList<>();
        List<WorkOrderProcessVo> workOrderProcessVoList = new ArrayList<>();
        for (YcqYouthcallWorkOrderProcess ycqYouthcallWorkOrderProcess : ycqYouthcallWorkOrderProcessList) {
            // 处理数据
            if (WorkOrderStatusEnum.DONE.getValue().equals(ycqYouthcallWorkOrderProcess.getProcessStatus())) {
                WorkOrderDealVo workOrderDealVo = new WorkOrderDealVo();
                workOrderDealVo.setDealWorkerName(ycqYouthcallWorkOrderProcess.getCreatedName());
                workOrderDealVo.setDealContent(ycqYouthcallWorkOrderProcess.getDealContent());
                workOrderDealVo.setDealTime(ycqYouthcallWorkOrderProcess.getCreatedTime());
                workOrderDealVoList.add(workOrderDealVo);
            }
            // 上报数据
            if (WorkOrderStatusEnum.PENDING_ESCALATION.getValue().equals(ycqYouthcallWorkOrderProcess.getProcessStatus())) {
                WorkOrderEscalationVo workOrderEscalationVo = new WorkOrderEscalationVo();
                workOrderEscalationVo.setEscalationType(ycqYouthcallWorkOrderProcess.getReportType());
                workOrderEscalationVo.setEscalationOrgId(ycqYouthcallWorkOrderProcess.getTransferOrgId());
                workOrderEscalationVo.setEscalationOrgName(ycqYouthcallWorkOrderProcess.getTransferOrgName());
                workOrderEscalationVo.setPriority(ycqYouthcallWorkOrderProcess.getPriority());
                workOrderEscalationVo.setDealContent(ycqYouthcallWorkOrderProcess.getDealContent());
                workOrderEscalationVo.setEscalationWorkerName(ycqYouthcallWorkOrderProcess.getCreatedName());
                workOrderEscalationVo.setEscalationWorkerMobile(ycqYouthcallWorkOrderProcess.getCreatedMobile());
                workOrderEscalationVoList.add(workOrderEscalationVo);
            }
            // 下派数据
            if (WorkOrderStatusEnum.PENDING_ALLOCATION.getValue().equals(ycqYouthcallWorkOrderProcess.getProcessStatus())) {
                WorkOrderAllocationVo workOrderAllocationVo = new WorkOrderAllocationVo();
                workOrderAllocationVo.setAllocationOrgId(ycqYouthcallWorkOrderProcess.getTransferOrgId());
                workOrderAllocationVo.setAllocationOrgName(ycqYouthcallWorkOrderProcess.getTransferOrgName());
                workOrderAllocationVo.setPriority(ycqYouthcallWorkOrderProcess.getPriority());
                workOrderAllocationVo.setDealContent(ycqYouthcallWorkOrderProcess.getDealContent());
                workOrderAllocationVo.setAllocationWorkerName(ycqYouthcallWorkOrderProcess.getCreatedName());
                workOrderAllocationVo.setAllocationWorkerMobile(ycqYouthcallWorkOrderProcess.getCreatedMobile());
                workOrderAllocationVoList.add(workOrderAllocationVo);
            }
            // 评价数据
            if (WorkOrderStatusEnum.EVALUATE.getValue().equals(ycqYouthcallWorkOrderProcess.getProcessStatus())) {
                WorkOrderEvaluateVo workOrderEvaluateVo = new WorkOrderEvaluateVo();
                workOrderEvaluateVo.setEvaluateName(ycqYouthcallWorkOrderProcess.getCreatedName());
                workOrderEvaluateVo.setEvaluateTime(ycqYouthcallWorkOrderProcess.getCreatedTime());
                workOrderEvaluateVo.setEvaluateStar(ycqYouthcallWorkOrderProcess.getEvaluateStar());
                workOrderEvaluateVo.setEvaluateContent(ycqYouthcallWorkOrderProcess.getDealContent());
                workOrderEvaluateVoList.add(workOrderEvaluateVo);
            }
            // 回访数据
            if (WorkOrderStatusEnum.FOLLOW_UP.getValue().equals(ycqYouthcallWorkOrderProcess.getProcessStatus())) {
                WorkOrderFollowUpVo workOrderFollowUpVo = new WorkOrderFollowUpVo();
                workOrderFollowUpVo.setFollowUpContent(ycqYouthcallWorkOrderProcess.getDealContent());
                workOrderFollowUpVo.setFollowUpName(ycqYouthcallWorkOrderProcess.getCreatedName());
                workOrderFollowUpVo.setFollowUpTime(ycqYouthcallWorkOrderProcess.getCreatedTime());
                workOrderFollowUpVo.setFollowUpType(ycqYouthcallWorkOrderProcess.getFollowUpType());
                workOrderFollowUpVoList.add(workOrderFollowUpVo);
            }
            // 归档数据
            if (WorkOrderStatusEnum.FILE.getValue().equals(ycqYouthcallWorkOrderProcess.getProcessStatus())) {
                WorkOrderFileVo workOrderFileVo = new WorkOrderFileVo();
                workOrderFileVo.setFileName(ycqYouthcallWorkOrderProcess.getCreatedName());
                workOrderFileVo.setFileTime(ycqYouthcallWorkOrderProcess.getCreatedTime());
                workOrderFileVoList.add(workOrderFileVo);
            }
            // 回复数据
            if (WorkOrderStatusEnum.REPLY.getValue().equals(ycqYouthcallWorkOrderProcess.getProcessStatus())) {
                WorkOrderReplyVo workOrderReplyVo = new WorkOrderReplyVo();
                workOrderReplyVo.setReplyName(ycqYouthcallWorkOrderProcess.getCreatedName());
                workOrderReplyVo.setReplyTime(ycqYouthcallWorkOrderProcess.getCreatedTime());
                workOrderReplyVo.setReplyContent(ycqYouthcallWorkOrderProcess.getDealContent());
                if (StringUtils.isNotEmpty(ycqYouthcallWorkOrderProcess.getAttachment())) {
                    workOrderReplyVo.setAttachmentList(Arrays.asList(ycqYouthcallWorkOrderProcess.getAttachment().split(",")));
                }
                workOrderReplyVoList.add(workOrderReplyVo);
            }
            //整体流程
            WorkOrderProcessVo workOrderProcessVo = new WorkOrderProcessVo();
            workOrderProcessVo.setDealTime(ycqYouthcallWorkOrderProcess.getCreatedTime());
            workOrderProcessVo.setDealName(ycqYouthcallWorkOrderProcess.getCreatedName());
            workOrderProcessVo.setProcessStatus(ycqYouthcallWorkOrderProcess.getProcessStatus());
            workOrderProcessVoList.add(workOrderProcessVo);
        }
        // 基本信息
        WorkOrderBaseVo workOrderBaseVo = rsp.getResult().get(0);
        int i = ycqYouthcallWorkOrderDao.querySameSql(workOrderBaseVo.getOrgId());
        workOrderBaseVo.setSameOrg(i > 0 ? "1" : "0");
        // 判断是否有工单图片
        if (StringUtils.isNotEmpty(workOrderBaseVo.getWorkOrderImg())) {
            workOrderBaseVo.setWorkOrderImgList(Arrays.asList(workOrderBaseVo.getWorkOrderImg().split(",")));
        }
        workOrderDetailVo.setWorkOrderBaseVo(workOrderBaseVo);

        // 处理信息
        workOrderDetailVo.setWorkOrderDealList(workOrderDealVoList);

        //上报信息
        workOrderDetailVo.setWorkOrderEscalationList(workOrderEscalationVoList);

        //下派信息
        workOrderDetailVo.setWorkOrderAllocationList(workOrderAllocationVoList);

        //评价信息
        workOrderDetailVo.setWorkOrderEvaluateList(workOrderEvaluateVoList);

        //回访信息
        workOrderDetailVo.setWorkOrderFollowUpList(workOrderFollowUpVoList);

        //归档信息
        workOrderDetailVo.setWorkOrderFileList(workOrderFileVoList);

        //回复信息
        workOrderDetailVo.setWorkOrderReplyList(workOrderReplyVoList);

        //整体流程信息
        workOrderDetailVo.setWorkOrderProcessList(workOrderProcessVoList);
        return workOrderDetailVo;
    }

    @Override
    public void exportFileWorkOrderExcel(HttpServletResponse response, YcqYouthcallWorkOrderExportReq req) {
        try {
            log.info("YcqYouthcallWorkOrderServiceImpl.exportFileWorkOrderExcel.param：{}", req.toString());
            List<WorkOrderExcelVo> workOrderList = ycqYouthcallWorkOrderDao.queryByIds(req.getIds());
            workOrderList.forEach(workOrderExcelVo -> {
                TsmPropertyValue tsmPropertyValue = tsmPropertyValueRepo.findTsmPropertyValueByValueCodeAndPropCode(workOrderExcelVo.getWorkOrderType(), "youthcall-workOrderType");
                TsmPropertyValue tsmPropertyValue2 = tsmPropertyValueRepo.findTsmPropertyValueByValueCodeAndPropCode(workOrderExcelVo.getSource(), "youthcall-source");
                if (tsmPropertyValue != null) {
                    workOrderExcelVo.setWorkOrderTypeName(tsmPropertyValue.getValueName());
                } else {
                    workOrderExcelVo.setWorkOrderTypeName(workOrderExcelVo.getWorkOrderType());
                }
                if (tsmPropertyValue2 != null) {
                    workOrderExcelVo.setSourceName(tsmPropertyValue2.getValueName());
                } else {
                    workOrderExcelVo.setSourceName(workOrderExcelVo.getSource());
                }
                workOrderExcelVo.setProcessStatusName(WorkOrderStatusEnum.fromValue(workOrderExcelVo.getProcessStatus()).getName());
            });
            String fileName = "归档工单信息";
            export(response, fileName, workOrderList);
        } catch (BusinessException e) {
            log.error("YcqYouthcallWorkOrderServiceImpl.exportFileWorkOrderExcel.error:{}", e.getMessage());
            throw new BusinessException(e.getMessage());
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderServiceImpl.exportFileWorkOrderExcel.error:{}", e.getMessage());
            e.printStackTrace();
            throw new BusinessException("系统内部异常，请联系管理员！");
        }
    }

    /**
     * 导出
     *
     * @param response
     * @param fileName
     * @param dataList
     * @throws BusinessException
     */
    private void export(HttpServletResponse response, String fileName, List<WorkOrderExcelVo> dataList) throws BusinessException {
        try {
            // 这里URLEncoder.encode可以防止中文乱码
            String encodedFileName = URLEncoder.encode(fileName + ".xlsx", StandardCharsets.UTF_8.name());
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=" + encodedFileName);
            ExcelWriterBuilder write = EasyExcel.write(response.getOutputStream(), YcqYouthcallWorkExcel.class);

            ExcelWriterSheetBuilder sheet1 = write.sheet("Sheet1")
                    // 设置单元格宽度自适应
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    // 设置单元格高度和字体
                    .registerWriteHandler(getHeightAndFontStrategy());
            List<YcqYouthcallWorkExcel> workExcelList = BeanCopyUtil.convert(dataList, YcqYouthcallWorkExcel.class);
            sheet1.doWrite(workExcelList);
            log.info("下载{}条记录到文件{}", dataList.size(), fileName);
        } catch (Exception e) {
            log.error("YcqYouthcallWorkOrderServiceImpl.export.error:" + e.getMessage());
            e.printStackTrace();
            throw new BusinessException(e.getMessage());
        }
    }

    /**
     * 自定义Excel导出策略，设置表头和数据行的字体和高度
     *
     * @return Excel导出策略
     */
    private HorizontalCellStyleStrategy getHeightAndFontStrategy() {
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 11);
        headWriteFont.setBold(true);
        headWriteCellStyle.setWriteFont(headWriteFont);
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
        WriteFont contentWriteFont = new WriteFont();
        contentWriteFont.setFontHeightInPoints((short) 11);
        contentWriteCellStyle.setWriteFont(contentWriteFont);
        contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
    }

    @Override
    public Result<Object> sendValidateCode(String phone) {
        String regex = "^1[3-9]\\d{9}$";
        boolean matches = phone.matches(regex);
        if (!matches) {
            throw new BusinessException("手机号格式异常");
        }

//        Worker currWorker = CurrWorkerUtil.getCurrWorker();
//        if (currWorker == null) {
//            return Result.ofFail("获取当前用户信息异常");
//        }
        try {
            //调用smsService的smsCode方法获取验证码
            Result<String> result = smsService.smsCode(phone);
            //判断响应状态码确认请求是否通过
            if (!"0".equals(result.getCode())) {
                return Result.ofFail("远程调用获取短信失败");
            }
            // 存入redis
            redisTemplate.opsForValue().set(RedisKeys.VALIDATE_CODE_KEY + phone, result.getData(), 300, TimeUnit.SECONDS);
//            System.out.println(redisTemplate.opsForValue().get(RedisKeys.VALIDATE_CODE_KEY + phone));
            log.info("发送验证码:" + phone + "*******" + RedisKeys.VALIDATE_CODE_KEY + phone + "*******" + redisTemplate.opsForValue().get(RedisKeys.VALIDATE_CODE_KEY + phone));
//            System.out.println(RedisKeys.VALIDATE_CODE_KEY + phone);
//            System.out.println(host + "::" + port + "::" + password);
            return Result.ofSuccess(result.getData());
//            return Result.ofSuccess();
        } catch (Exception e) {
            log.error("远程调用获取短信接口失败" + e);
            return Result.ofFail("获取当前用户信息异常");
        }
    }

    @Override
    public String testSetKey(String phone) {
        redisTemplate.opsForValue().set(RedisKeys.VALIDATE_CODE_KEY + phone, "877777", 300, TimeUnit.SECONDS);
        System.out.println(RedisKeys.VALIDATE_CODE_KEY + phone + ":" + "877777");
        log.info(RedisKeys.VALIDATE_CODE_KEY + phone + ":" + "877777");
        return "testSetKey";
    }

    @Override
    public String testGetKey(String phone) {
        String code = (String) redisTemplate.opsForValue().get(RedisKeys.VALIDATE_CODE_KEY + phone);
        System.out.println(RedisKeys.VALIDATE_CODE_KEY + phone + "::" + code);
        log.info(RedisKeys.VALIDATE_CODE_KEY + phone + "::" + code);
        return (String) redisTemplate.opsForValue().get(RedisKeys.VALIDATE_CODE_KEY + phone);
    }
}
