package com.crunii.micro.service.youthcall.thirdrequest;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.crunii.micro.service.youthcall.dto.YcqYouthcallCheckInApply;
import com.crunii.micro.service.youthcall.enums.CheckInApplyStatusEnum;
import com.crunii.micro.service.youthcall.service.YcqYouthcallCheckInApplyService;
import com.crunii.micro.service.youthcall.service.YcqYouthcallHousingSecurityService;
import okhttp3.*;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.util.*;

@Component
public class QnyzDataRequestService {

    private static Logger logger = LoggerFactory.getLogger(QnyzDataRequestService.class.getName());
    private String baseUrl = "https://ggfw.rlsbj.cq.gov.cn/cqsbmh/wisdom-ut/rsrc-qnyz_rcgy-web";

    /**
     * 渠道号
     */
    @Value("${qnyz.channelId:QCCQAPP}")
    private String channelId;

    /**
     * 通用参数
     */
    private static final String PARAM_CHANNEL_ID = "channelId";

    @Resource
    private YcqYouthcallHousingSecurityService ycqYouthcallHousingSecurityService;

    @Resource
    private YcqYouthcallCheckInApplyService ycqYouthcallCheckInApplyService;


    /**
     * 获取token
     */
    public String getToken() {
        Object token = this.get(QnyzUriEnum.GET_TOKEN.getUri(), null, true);
        return String.valueOf(token);
    }

    /**
     * 定时查询驿站列表信息、查询驿站详细信息，调用本地接口对象存储数据
     */
    @Scheduled(cron = "0/30 * * * * ?")
    public void queryHotelList() {
        List<Map<String, Object>> list = new ArrayList<>();
        // 每次读取500条
        int pageSize = 500;
        int pageIndex = 0;
        // 循环判断是否还有下一页
        boolean next = false;
        do {
            Map<String, Object> queryListParamMap = new HashMap<>();
            queryListParamMap.put("pageSize", pageSize);
            queryListParamMap.put("pageIndex", pageIndex);
            // 调用数据接口
            Object pageData = this.get(QnyzUriEnum.QUERY_HOTEL_LIST.getUri(), queryListParamMap, false);
            if (pageData == null) continue;
            TypeReference<Map<String, Object>> typeReference = new TypeReference<Map<String, Object>>() {};
            Map<String, Object> dataMap = JSON.parseObject(JSON.toJSONString(pageData), typeReference);
            // 计算是否还有下一页
            int pageCount = (int) dataMap.get("pageCount");
            // 没有读取到最后一页，继续读取
            next = pageIndex < pageCount - 1;
            pageIndex++;
            // 对数据进行处理
            Object data = dataMap.get("data");
            if (data == null) continue;
            TypeReference<List<Map<String, Object>>> listTypeReference = new TypeReference<List<Map<String, Object>>>() {};
            List<Map<String, Object>> listDataMap = JSON.parseObject(JSON.toJSONString(data), listTypeReference);
            for (Map<String, Object> map : listDataMap) {
                // 驿站数据对象
                int id = (int) map.get("id");
                // 调用驿站详情接口
                Map<String, Object> itemParamMap = new HashMap<>();
                itemParamMap.put("hotelId", id);
                Object detailData = this.get(QnyzUriEnum.GET_HOTEL_INFO_BY_ID.getUri(), itemParamMap, false);
                // 详情数据不能为空
                if (detailData == null) continue;
                TypeReference<Map<String, Object>> detailDataTypeReference = new TypeReference<Map<String, Object>>() {};
                Map<String, Object> detailDataMap = JSON.parseObject(JSON.toJSONString(detailData), detailDataTypeReference);
                // 驿站区域名称
                detailDataMap.put("organ", map.get("organ"));
                // 驿站封面图片
                detailDataMap.put("picurl", map.get("picurl"));
                list.add(detailDataMap);
            }
        } while (next);
        // 调用驿站服务，推送数据
        if (list.isEmpty()) return;
        ycqYouthcallHousingSecurityService.synchroData(list);
    }

    /**
     * 定时刷新入住申请状态和审批意见
     */
//    @Scheduled(cron = "0/30 * * * * ?")
    public void flushApplyStatus() {
        // 获取申请列表
        List<YcqYouthcallCheckInApply> list = ycqYouthcallCheckInApplyService.getAll();
        if (list == null || list.isEmpty()) return;
        list.stream()
                // 未删除的数据
                .filter(ycqYouthcallCheckInApply -> ycqYouthcallCheckInApply.getDelFlag() == 0)
                // 当前申请状态不是已关闭的都要查询最新审核状态
                .filter(ycqYouthcallCheckInApply -> !CheckInApplyStatusEnum.YSSTATE_2.getAuditStatusCode().equals(ycqYouthcallCheckInApply.getAuditStatus()))
                // 遍历查询申请的最新审核状态
                .forEach(ycqYouthcallCheckInApply -> {
                    // 申请id
                    BigInteger applyId = ycqYouthcallCheckInApply.getApplyId();
                    Map<String, Object> paramMap = new HashMap<>();
                    paramMap.put("applyId", applyId.toString());
                    Object data = this.get(QnyzUriEnum.GET_AUDIT_STATUS.getUri(), paramMap, false);
                    if (data != null) {
                        TypeReference<Map<String, Object>> typeReference = new TypeReference<Map<String, Object>>() {};
                        Map<String, Object> dataMap = JSON.parseObject(JSON.toJSONString(data), typeReference);
                        // 审核状态
                        String auditStatus = String.valueOf(dataMap.get("auditState"));
                        // 审核意见
                        String auditComment = String.valueOf(dataMap.get("auditContent"));
                        // 更新最新的申请状态和意见
                        ycqYouthcallCheckInApplyService.updateAuditStatus(applyId, auditStatus, auditComment);
                    }
                });
    }

    /**
     * 查询字典数据
     */
    public void queryDictionary() {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("aaa100", "ABD120");
        // 调用数据接口
        Object data = this.get(QnyzUriEnum.GET_AA_10.getUri(), paramMap, false);
        if (data == null) return;
        TypeReference<List<Map<String, Object>>> listTypeReference = new TypeReference<List<Map<String, Object>>>() {};
        List<Map<String, Object>> listDataMap = JSON.parseObject(JSON.toJSONString(data), listTypeReference);
        if (listDataMap == null || listDataMap.isEmpty()) return;
        System.out.println(listDataMap);
    }

    /**
     * 入住申请
     */
    public String saveApply(YcqYouthcallCheckInApply ycqYouthcallCheckInApply, Map<String, List<MultipartFile>> fileMap) {
        LinkedHashMap<String, String> paramMap = new LinkedHashMap<>();
        // token
        paramMap.put("token", this.getToken());
        // 渠道号
        paramMap.put(PARAM_CHANNEL_ID, channelId);
        // 申请人姓名
        paramMap.put("name", ycqYouthcallCheckInApply.getApplyUserName());
        // 身份证号
        paramMap.put("idcard", ycqYouthcallCheckInApply.getIdCard());
        // 手机号
        paramMap.put("telephone", ycqYouthcallCheckInApply.getPhone());
        // 驿站id
        paramMap.put("hotelId", ycqYouthcallCheckInApply.getHotelId());
        // 是否已毕业
        paramMap.put("isGraduation", ycqYouthcallCheckInApply.getGrad().toString());
        // 入住开始时间
        Date checkInDate = ycqYouthcallCheckInApply.getCheckInDate();
        paramMap.put("startTime", new SimpleDateFormat("yyyy-MM-dd").format(checkInDate));
        // 入住天数
        paramMap.put("onDays", ycqYouthcallCheckInApply.getCheckInDays().toString());
        Object applyId = this.post(QnyzUriEnum.SAVE_APPLY_INFO.getUri(), paramMap, fileMap);
        if (applyId == null) throw new IllegalArgumentException("入住申请失败，申请id为空");
        return String.valueOf(applyId);
    }

    /**
     * 取消申请
     */
    public void applyCancel(String applyId) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("applyId", applyId);
        paramMap.put("token", this.getToken());
        this.get(QnyzUriEnum.APPLY_CANCEL.getUri(), paramMap, true);
    }

    /**
     * 发送带参get请求
     */
    private Object get(String uri, Map<String, Object> paramMap, boolean throwError) {
        // 组装字符串参数
        if (paramMap == null) paramMap = new HashMap<>();
        paramMap.put(PARAM_CHANNEL_ID, channelId);
        List<String> paramsList = new ArrayList<>();
        paramMap.forEach((key, value) -> paramsList.add(key + "=" + (Objects.isNull(value) ? "" : value)));
        // 构建请求url
        StringBuilder url = new StringBuilder();
        url.append(baseUrl).append(uri).append("?").append(StringUtils.join(paramsList, "&"));
        try {
            // 组装http请求对象
            Request request = new Request.Builder()
                    .url(url.toString())
                    .get()
                    .build();
            OkHttpClient.Builder builder = new OkHttpClient().newBuilder()
                    .readTimeout(Duration.ofMinutes(2))
                    .connectTimeout(Duration.ofSeconds(70));
            // 执行请求，取到返回值
            Response response = builder.build().newCall(request).execute();
            assert response.body() != null;
            // 转为响应结果对象
            QnyzResponse responseData = JSON.parseObject(response.body().string(), QnyzResponse.class);
            // 校验状态码
            int statusCode = responseData.getStatusCode();
            if (statusCode != 200) {
                throw new IllegalArgumentException("状态码为" + statusCode + "，" + responseData.getMessage());
            }
            // 校验结果
            QnyzResultData resultData = responseData.getResult();
            int code = Integer.parseInt(resultData.getCode());
            if (code == 0) {
                throw new IllegalArgumentException("返回状态为" + code + "，" + resultData.getMsg());
            }
            return resultData.getData();
        } catch (Exception e) {
            logger.error("请求失败，" + paramMap + "，" + e.getMessage());
            if (throwError) throw new IllegalArgumentException("请求失败，" + e.getMessage());
        }
        return null;
    }

    /**
     * 发送post请求
     */
    private Object post(String uri, LinkedHashMap<String, String> bodyData, Map<String, List<MultipartFile>> fileMap) {
        try {
            // 设置请求体
            MediaType mediaType = MediaType.parse("multipart/form-data; boundary=----WebKitFormBoundaryQC7A2hAquBA4IaJA");
            MultipartBody.Builder multipartBodyBuilder = new MultipartBody.Builder().setType(Objects.requireNonNull(mediaType));
            // 文件上传
            fileMap.forEach((name, multipartFiles) -> {
                for (MultipartFile multipartFile : multipartFiles) {
                    String originalFilename = multipartFile.getOriginalFilename();
                    try {
                        InputStream inputStream = multipartFile.getInputStream();
                        // inputStream 转为 byte[]
                        byte[] bytes = IOUtils.toByteArray(inputStream);
                        RequestBody fileBody = RequestBody.create(MediaType.parse(Objects.requireNonNull(multipartFile.getContentType())), bytes);
                        multipartBodyBuilder.addFormDataPart(name, originalFilename, fileBody);
                    } catch (IOException e) {
                        logger.error("上传文件失败，" + e.getMessage());
                    }
                }
            });
            // 申请信息转为字符串后加密
            String applyJson = AesUtil.encrypt(JSON.toJSONString(bodyData), AesUtil.AESKEY);
            assert applyJson != null;
            multipartBodyBuilder.addFormDataPart("applyJson", applyJson);
            // 渠道号
            multipartBodyBuilder.addFormDataPart(PARAM_CHANNEL_ID, channelId);
            RequestBody requestBody = multipartBodyBuilder.build();
            // 组装http请求对象
            Request request = new Request.Builder()
                    .url(baseUrl + uri)
                    .post(requestBody)
                    .build();
            OkHttpClient.Builder builder = new OkHttpClient().newBuilder()
                    .readTimeout(Duration.ofMinutes(2))
                    .connectTimeout(Duration.ofSeconds(70));
            // 执行请求，取到返回值
            Response response = builder.build().newCall(request).execute();
            assert response.body() != null;
            // 转为响应结果对象
            QnyzResponse responseData = JSON.parseObject(response.body().string(), QnyzResponse.class);
            // 校验状态码
            int statusCode = responseData.getStatusCode();
            if (statusCode != 200) {
                throw new IllegalArgumentException(responseData.getMessage());
            }
            // 校验结果
            QnyzResultData resultData = responseData.getResult();
            int code = Integer.parseInt(resultData.getCode());
            if (code == 0) {
                throw new IllegalArgumentException(resultData.getMsg());
            }
            return resultData.getData();
        } catch (Exception e) {
            logger.error("请求失败，" + bodyData + "，" + e.getMessage());
            throw new IllegalArgumentException(e.getMessage());
        }
    }

}
